(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   1976457,      55494]*)
(*NotebookOutlinePosition[   1978292,      55550]*)
(*  CellTagsIndexPosition[   1978248,      55546]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Chapter 2. \nAn Introduction to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " "
}], "Title",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["A System for Doing Mathematics by Computer", "Subsubtitle",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["2.1 Systems, Kernels and Front Ends", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The first thing that you need to do is to learn how to communicate with \
the computer and the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " software system. It is important to realize that the software is in two \
pieces. The calculations are done by a computational \"kernel\", but you \
communicate with the kernel through a front end. The kernel and the front end \
may be running on the same computer, or on different computers linked by a \
network. This introduction is targeted at people using a windowing operating \
system, and the front end is a Notebook interface. This interface creates and \
uses ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Notebook files - you can exchange Notebooks between computer systems such \
as MacOS, Windows95/NT and UNIX. The Notebooks are just text files, and can \
be passed between systems using only text-translation utilities. \n\nIf you \
have not used ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " before, how to proceed depends on whether you have ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " already installed on your computer. If it is not installed at all, you \
may need some help if you are working in a multi-computer networked \
environment running under UNIX, for example - under these circumstances, it \
is almost certainly best to request help from your System Administrator. If \
you are running on a solitary system, or otherwise doing an installation \
local to your computer, it is usually sufficient to follow the instructions \
in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " installation instructions. These are contained in the booklet \"Getting \
started with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " on Macintosh/Windows (etc.) systems\", specific to your own computer. If \
you have never used ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " before, it will be helpful to run through the examples contained in that \
booklet. The discussion below is designed to complement that booklet, but is \
essentially self-contained.\n\nBegin by starting ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". You can do this on any system by navigating to the icon for ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "3.0, and double-clicking on it. Do not use the \"MathKernel\" program icon \
for this. On particular systems there will be other routes to access ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Under Windows 95, the program can be found under the \"Start\" task. On \
both Windows 95 and MacOS, it is convenient to make short-cuts/aliases and \
place them on your desktop - it is really up to you to customize your route \
to access the system once you have figured out what is most convenient. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will start up, giving the version number, your licence information, and \
warnings about software piracy."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Input and Output", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "When a new ",
  "Notebook",
  " is opened you can type straight into the blank window. Type \"3 + 3\" \
into the system (not the quotes, just the expression within the quotes). To \
enter this expression hold down the shift key and hit the \"return\" key.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " labels both your input and its output with numbers, in this case [1], if \
you are starting a fresh session.  You should see the following, with the \
input prefixed by In[1], the output prefixed by Out[1]:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["3+3", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(6\)], "Output"]
}, Open  ]],

Cell["\<\
Now type the command  \"Expand[(2 x + y)^3]\", and enter this by \
SHIFT-RETURN (ENTER, not to be confused with RETURN, and usually located at \
the bottom right of keyboards, may also work and is a useful short-cut on \
some computers).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Expand[(2 x + y)^3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(8\ x\^3 + 12\ x\^2\ y + 6\ x\ y\^2 + y\^3\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Throughout this chapter, whenever we present examples like this, \
expressions in bold type are what you should type, followed by SHIFT-RETURN \
or ENTER.  To see some of the capabilities of ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "try the following sequences of statements.  When you enter the commands in \
bold, they will acquire  \"In[n] := \" labels.  Also,",
  StyleBox[" if you make a mistake, just re-type the correct expression",
    FontSlant->"Italic"],
  ". We will come back to how to edit expressions shortly; the following are \
all short enough to make re-typing painless. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The first example uses the ",
  StyleBox["N",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function to ask for \[Pi], denoted in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s Input Form by ",
  StyleBox["Pi",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", to 100-digit precision."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["N[Pi, 100]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(3.1415926535897932384626433832795028841971693993751058209749445923078164\
06286208998628034825342117067982148086513`100\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The next example introduces the plotting capabilities of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by using ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  Given the function and the range over which to draw the graph, the ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function will produce the appropriate graph."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Plot[x^2 Sin[x], {x, -3, 3}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 0.309016 0.0745958 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.18254 .29652 -6 -9 ]
[.18254 .29652 6 0 ]
[.34127 .29652 -6 -9 ]
[.34127 .29652 6 0 ]
[.65873 .29652 -3 -9 ]
[.65873 .29652 3 0 ]
[.81746 .29652 -3 -9 ]
[.81746 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .01063 -12 -4.5 ]
[.4875 .01063 0 4.5 ]
[.4875 .15982 -12 -4.5 ]
[.4875 .15982 0 4.5 ]
[.4875 .45821 -6 -4.5 ]
[.4875 .45821 0 4.5 ]
[.4875 .6074 -6 -4.5 ]
[.4875 .6074 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-3)] .02381 .29652 0 1 Mshowa
.18254 .30902 m
.18254 .31527 L
s
[(-2)] .18254 .29652 0 1 Mshowa
.34127 .30902 m
.34127 .31527 L
s
[(-1)] .34127 .29652 0 1 Mshowa
.65873 .30902 m
.65873 .31527 L
s
[(1)] .65873 .29652 0 1 Mshowa
.81746 .30902 m
.81746 .31527 L
s
[(2)] .81746 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(3)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.05556 .30902 m
.05556 .31277 L
s
.0873 .30902 m
.0873 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.15079 .30902 m
.15079 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.24603 .30902 m
.24603 .31277 L
s
.27778 .30902 m
.27778 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.37302 .30902 m
.37302 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.43651 .30902 m
.43651 .31277 L
s
.46825 .30902 m
.46825 .31277 L
s
.53175 .30902 m
.53175 .31277 L
s
.56349 .30902 m
.56349 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.62698 .30902 m
.62698 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.72222 .30902 m
.72222 .31277 L
s
.75397 .30902 m
.75397 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.84921 .30902 m
.84921 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.9127 .30902 m
.9127 .31277 L
s
.94444 .30902 m
.94444 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .01063 m
.50625 .01063 L
s
[(-4)] .4875 .01063 1 0 Mshowa
.5 .15982 m
.50625 .15982 L
s
[(-2)] .4875 .15982 1 0 Mshowa
.5 .45821 m
.50625 .45821 L
s
[(2)] .4875 .45821 1 0 Mshowa
.5 .6074 m
.50625 .6074 L
s
[(4)] .4875 .6074 1 0 Mshowa
.125 Mabswid
.5 .04793 m
.50375 .04793 L
s
.5 .08523 m
.50375 .08523 L
s
.5 .12253 m
.50375 .12253 L
s
.5 .19712 m
.50375 .19712 L
s
.5 .23442 m
.50375 .23442 L
s
.5 .27172 m
.50375 .27172 L
s
.5 .34631 m
.50375 .34631 L
s
.5 .38361 m
.50375 .38361 L
s
.5 .42091 m
.50375 .42091 L
s
.5 .49551 m
.50375 .49551 L
s
.5 .5328 m
.50375 .5328 L
s
.5 .5701 m
.50375 .5701 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .21427 m
.04262 .14974 L
.06244 .09614 L
.08255 .05639 L
.09396 .04007 L
.09905 .03417 L
.10458 .02869 L
.11004 .02422 L
.11508 .02089 L
.11989 .0184 L
.12235 .01738 L
.125 .01648 L
.12738 .01582 L
.12956 .01536 L
.13061 .01518 L
.13175 .01502 L
.13283 .0149 L
.13384 .01482 L
.13511 .01474 L
.13631 .01472 L
.13738 .01472 L
.13855 .01476 L
.13982 .01484 L
.14046 .01489 L
.14117 .01496 L
.14357 .0153 L
.14608 .01579 L
.14881 .0165 L
.15378 .01821 L
.15861 .02038 L
.16308 .02281 L
.17322 .02975 L
.184 .03906 L
.22175 .08358 L
.26195 .14166 L
.30064 .19713 L
.34178 .24678 L
.3623 .2662 L
.3814 .28072 L
.39983 .29149 L
.41951 .2997 L
.42975 .30276 L
.44086 .30525 L
.44595 .30613 L
.45134 .3069 L
.45639 .30749 L
.46101 .30792 L
.46585 .30828 L
.47045 .30854 L
.4753 .30874 L
Mistroke
.47804 .30882 L
.48058 .30888 L
.48298 .30892 L
.48524 .30896 L
.48726 .30898 L
.48945 .30899 L
.49059 .309 L
.49185 .30901 L
.49315 .30901 L
.49437 .30901 L
.49552 .30901 L
.49656 .30902 L
.49776 .30902 L
.4989 .30902 L
.49998 .30902 L
.50111 .30902 L
.50236 .30902 L
.5035 .30902 L
.50475 .30902 L
.50546 .30902 L
.50612 .30902 L
.50731 .30902 L
.50858 .30903 L
.50991 .30903 L
.51135 .30904 L
.51262 .30905 L
.51396 .30907 L
.5165 .3091 L
.51884 .30914 L
.52104 .30919 L
.52339 .30925 L
.52763 .30941 L
.53267 .30966 L
.53728 .30997 L
.54207 .31039 L
.54717 .31095 L
.55634 .31228 L
.5662 .31427 L
.57709 .31723 L
.58717 .32076 L
.59641 .32472 L
.61726 .33643 L
.63616 .35054 L
.65685 .36984 L
.69738 .41824 L
.7364 .47392 L
.77786 .53392 L
.79864 .56043 L
.81781 .58066 L
.82745 .5888 L
Mistroke
.83647 .59495 L
.84139 .59765 L
.84588 .59969 L
.85088 .60144 L
.85363 .60217 L
.85489 .60245 L
.85624 .6027 L
.85759 .60291 L
.85884 .60307 L
.86002 .60318 L
.86128 .60327 L
.8624 .60331 L
.86345 .60332 L
.86461 .6033 L
.86584 .60324 L
.86716 .60313 L
.8679 .60305 L
.86859 .60297 L
.87119 .60253 L
.87355 .60197 L
.87606 .60122 L
.88058 .59942 L
.88481 .59721 L
.89446 .59019 L
.9043 .58008 L
.91465 .56604 L
.93316 .53181 L
.95292 .48164 L
.97129 .42186 L
.97619 .40376 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{191, 117.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004N0000/1000`40O003h00Oogoo7goo003oOolO
Ool00?mooamoo`00T7oo00<007ooOol0Rgoo0027Ool30006Ool00`00Oomoo`2;Ool008Qoo`03001o
ogoo00Eoo`03001oogoo08]oo`00Q7oo1P001Woo0P00S7oo000/Ool2001=Ool70002Ool01@00Oomo
ogoo00001goo00<007ooOol0Rgoo000XOol500000goo00000003001BOol01000Oomoo`001goo00<0
07ooOol0Rgoo000VOol2000;Ool00`00Oomoo`1@Ool00`00Ool00007Ool00`00Oomoo`2;Ool002Eo
o`03001oogoo00aoo`80051oo`03001oo`0000Moo`03001oogoo08]oo`0097oo00<007ooOol03goo
00<007ooOol0CWoo0P001goo00<007ooOol0Rgoo000SOol00`00Oomoo`0AOol2001?Ool00`00Oomo
o`05Ool00`00Oomoo`2;Ool0029oo`03001oogoo01Aoo`03001oogoo05Aoo`03001oogoo08]oo`00
8Goo00<007ooOol05Woo00<007ooOol0Dgoo00<007ooOol0Rgoo000QOol00`00Oomoo`0FOol00`00
Oomoo`1COol00`00Oomoo`2;Ool0021oo`03001oogoo01Qoo`03001oogoo059oo`03001oogoo08]o
o`007goo00<007ooOol06Woo00<007ooOol0DGoo0P00S7oo000OOol00`00Oomoo`0KOol00`00Oomo
o`1@Ool00`00Oomoo`2;Ool001ioo`03001oogoo01eoo`03001oogoo04moo`03001oogoo08]oo`00
7Woo00<007ooOol07Woo00<007ooOol0CWoo00<007ooOol0Rgoo000MOol00`00Oomoo`0OOol00`00
Oomoo`1>Ool00`00Oomoo`2;Ool001eoo`03001oogoo021oo`03001oogoo04eoo`03001oogoo08]o
o`0077oo00<007ooOol08Woo00<007ooOol0C7oo00<007ooOol0Rgoo000LOol00`00Oomoo`0SOol0
0`00Oomoo`1;Ool00`00Oomoo`2;Ool001]oo`03001oogoo02Eoo`03001oogoo04Yoo`03001oogoo
08]oo`006Woo00<007ooOol09goo00<007ooOol0BGoo00<007ooOol0Rgoo000JOol00`00Oomoo`0W
Ool00`00Oomoo`19Ool2002<Ool001Uoo`03001oogoo02Uoo`03001oogoo04Qoo`03001oogoo08]o
o`006Goo00<007ooOol0:Woo00<007ooOol0Agoo00<007ooOol0Rgoo000HOol00`00Oomoo`0[Ool0
0`00Oomoo`17Ool00`00Oomoo`2;Ool001Qoo`03001oogoo02aoo`03001oogoo04Ioo`03001oogoo
08]oo`005goo00<007ooOol0;Woo00<007ooOol0AGoo00<007ooOol0Rgoo000GOol00`00Oomoo`0^
Ool00`00Oomoo`15Ool00`00Oomoo`2;Ool001Moo`03001oogoo02moo`03001oogoo04Aoo`03001o
ogoo08]oo`005Woo00<007ooOol0<Goo00<007ooOol0@goo00<007ooOol0Rgoo000FOol00`00Oomo
o`0bOol00`00Oomoo`12Ool00`00Oomoo`2;Ool001Ioo`03001oogoo039oo`03001oogoo049oo`80
08aoo`005Goo00<007ooOol0=7oo00<007ooOol0@Goo00<007ooOol0Rgoo000EOol00`00Oomoo`0e
Ool00`00Oomoo`10Ool00`00Oomoo`2;Ool001Eoo`03001oogoo03Eoo`03001oogoo041oo`03001o
ogoo08]oo`0057oo00<007ooOol0=goo00<007ooOol0?goo00<007ooOol0Rgoo000DOol00`00Oomo
o`0hOol00`00Oomoo`0nOol00`00Oomoo`2;Ool001Aoo`03001oogoo03Qoo`03001oogoo03ioo`03
001oogoo08]oo`004goo00<007ooOol0>Woo00<007ooOol0<Goo1P001Woo00<007ooOol0Rgoo000C
Ool00`00Oomoo`0kOol00`00Oomoo`0aOol01@00Oomoogoo00001Woo00<007ooOol0Rgoo000COol0
0`00Oomoo`0kOol00`00Oomoo`0bOol00`00Oomoo`07Ool00`00Oomoo`2;Ool0019oo`03001oogoo
03eoo`03001oogoo02Ioo`L000Eoo`03001oogoo00Ioo`8008aoo`004Woo00<007ooOol0?Woo00<0
07ooOol0<Woo00<007ooOol01Goo00<007ooOol0Rgoo000BOol00`00Oomoo`0nOol00`00Oomoo`0c
Ool00`00Oomoo`04Ool00`00Oomoo`2;Ool0015oo`03001oogoo041oo`03001oogoo039oo`03001o
ogoo00Aoo`03001oogoo08]oo`004Goo00<007ooOol0@Goo00<007ooOol0;7oo00<007ooOol00Woo
00<007ooOol017oo00<007ooOol0Rgoo000AOol00`00Oomoo`11Ool00`00Oomoo`0]Ool40007Ool0
0`00Oomoo`2;Ool0015oo`03001oogoo049oo`03001oogoo03Moo`03001oogoo08]oo`0047oo00<0
07ooOol0A7oo00<007ooOol0=Woo00<007ooOol0Rgoo000@Ool00`00Oomoo`14Ool00`00Oomoo`0f
Ool00`00Oomoo`2;Ool0011oo`03001oogoo04Eoo`03001oogoo03Eoo`03001oogoo08]oo`0047oo
00<007ooOol0AWoo00<007ooOol0=7oo0P00S7oo000?Ool00`00Oomoo`17Ool00`00Oomoo`0dOol0
0`00Oomoo`2;Ool000moo`03001oogoo04Qoo`03001oogoo03=oo`03001oogoo08]oo`003goo00<0
07ooOol0BGoo00<007ooOol0<Woo00<007ooOol0Rgoo000>Ool00`00Oomoo`1;Ool00`00Oomoo`0a
Ool00`00Oomoo`2;Ool000ioo`03001oogoo04aoo`03001oogoo031oo`03001oogoo08]oo`00GGoo
00<007ooOol0<7oo00<007ooOol0Rgoo001NOol00`00Oomoo`0_Ool00`00Oomoo`2;Ool005moo`03
001oogoo02ioo`03001oogoo08]oo`00H7oo00<007ooOol0;Goo00<007ooOol0Rgoo001POol00`00
Oomoo`0]Ool2002<Ool0065oo`03001oogoo02aoo`03001oogoo08]oo`00HWoo00<007ooOol0:goo
00<007ooOol0Rgoo001SOol00`00Oomoo`0ZOol00`00Oomoo`2;Ool006Aoo`8002Yoo`03001oogoo
08]oo`00IWoo00<007ooOol09goo00<007ooOol0Rgoo001WOol00`00Oomoo`0VOol00`00Oomoo`2;
Ool0011oo`@002Ioo`H002Ioo`D002Eoo`03001oogoo02Eoo`D002Eoo`H002Moo`@000]oo`003goo
00<007ooOol00Woo00<007ooOol097oo00D007ooOomoo`0002Qoo`8002Ioo`03001oogoo02Moo`03
001oogoo02Ioo`05001oogooOol0000VOol00`00Oomoo`02Ool00`00Oomoo`08Ool001Aoo`03001o
ogoo02Eoo`03001oogoo02Uoo`04001oo`00000TOol00`00Oomoo`0WOol00`00Oomoo`0WOol00`00
Oomoo`0/Ool00`00Oomoo`08Ool000Ioo`L000Moo`03001oogoo01Yoo`L000Eoo`03001oogoo01ao
o`L000Eoo`05001oogooOol0000SOol2000XOol00`00Oomoo`0XOol00`00Oomoo`0[Ool00`00Oomo
o`08Ool0015oo`<002Yoo`03001oogoo02Moo`03001oogoo009oo`03001oogoo021oo`03001oogoo
02Moo`03001oogoo02Uoo`03001oogoo02Moo`<000]oo`0057oo00<007ooOol0:7oo00<007ooOol0
9Woo00<007ooOol00goo00<007ooOol07goo00<007ooOol09goo00<007ooOol0:Woo00<007ooOol0
:Goo00<007ooOol027oo000DOol00`00Oomoo`0XOol00`00Oomoo`0VOol00`00Oomoo`04Ool2000O
Ool00`00Oomoo`0WOol00`00Oomoo`0ZOol00`00Oomoo`0YOol00`00Oomoo`08Ool000moo`03001o
ogoo009oo`03001oogoo02=oo`03001oogoo009oo`03001oogoo02Aoo`<000Qoo`8001eoo`03001o
ogoo02Eoo`<002Moo`03001oogoo009oo`03001oogoo02Aoo`03001oogoo009oo`03001oogoo00Qo
o`0047oo10009goo1000:7oo0P002Woo00<007ooOol06Woo00<007ooOol09Woo0P00:7oo1000:7oo
10002goo001dOol3000IOol00`00Oomoo`2;Ool007Moo`<001Ioo`03001oogoo08]oo`00NWoo0P00
57oo00<007ooOol0Rgoo001lOol3000AOol00`00Oomoo`2;Ool007moo`/000Ioo`03001oogoo08]o
o`001gooo`004`001Goo000>Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00
Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00
Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00
Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool30005Ool00`00Oomoo`06
Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`0:Ool0091oo`03001oogoo015oo`<007Moo`00T7oo00<007oo
Ool057oo0`00M7oo002@Ool00`00Oomoo`0GOol2001bOol0091oo`03001oogoo01Uoo`80071oo`00
T7oo00<007ooOol06goo00<007ooOol0KGoo002@Ool00`00Oomoo`0LOol2001]Ool0091oo`03001o
ogoo01ioo`03001oogoo06Yoo`00T7oo00<007ooOol07goo0P00JWoo002@Ool2000ROol00`00Oomo
o`1WOol0091oo`03001oogoo029oo`03001oogoo06Ioo`00T7oo00<007ooOol08goo00<007ooOol0
IGoo002@Ool00`00Oomoo`0TOol00`00Oomoo`1TOol0091oo`03001oogoo02Eoo`03001oogoo06=o
o`00T7oo00<007ooOol09Woo00<007ooOol0HWoo002@Ool00`00Oomoo`0WOol00`00Oomoo`1QOol0
091oo`03001oogoo02Moo`03001oogoo065oo`00T7oo00<007ooOol0:7oo00<007ooOol0H7oo002@
Ool00`00Oomoo`0YOol00`00Oomoo`1OOol0091oo`8002]oo`03001oogoo05ioo`00T7oo00<007oo
Ool0:goo00<007ooOol0GGoo002@Ool00`00Oomoo`0/Ool00`00Oomoo`1LOol0091oo`03001oogoo
02eoo`03001oogoo05]oo`00T7oo00<007ooOol0;Goo00<007ooOol0Fgoo002@Ool00`00Oomoo`0^
Ool00`00Oomoo`1=Ool00`00Oomoo`0:Ool0091oo`03001oogoo02moo`03001oogoo04aoo`03001o
ogoo00Yoo`00T7oo00<007ooOol0<7oo00<007ooOol0Bgoo00<007ooOol02Woo002@Ool00`00Oomo
o`0aOol00`00Oomoo`19Ool00`00Oomoo`0;Ool0091oo`03001oogoo039oo`03001oogoo04Qoo`03
001oogoo00]oo`00T7oo0P00<goo00<007ooOol0B7oo00<007ooOol02goo002@Ool00`00Oomoo`0c
Ool00`00Oomoo`17Ool00`00Oomoo`0;Ool0091oo`03001oogoo03Aoo`03001oogoo04Ioo`03001o
ogoo00]oo`00T7oo00<007ooOol0=Goo00<007ooOol0A7oo00<007ooOol037oo002@Ool00`00Oomo
o`0eOol00`00Oomoo`14Ool00`00Oomoo`0<Ool0091oo`03001oogoo03Ioo`03001oogoo04=oo`03
001oogoo00aoo`00T7oo00<007ooOol0=goo00<007ooOol0@Goo00<007ooOol03Goo002@Ool00`00
Oomoo`0hOol00`00Oomoo`10Ool00`00Oomoo`0=Ool008Aoo`H000Ioo`03001oogoo03Qoo`03001o
ogoo041oo`03001oogoo00eoo`00QGoo00D007ooOomoo`0000Ioo`03001oogoo03Uoo`03001oogoo
03ioo`03001oogoo00ioo`00QWoo00<007ooOol01goo0P00>goo00<007ooOol0?Goo00<007ooOol0
3Woo0027Ool00`00Oomoo`06Ool00`00Oomoo`0jOol00`00Oomoo`0mOol00`00Oomoo`0>Ool008Qo
o`03001oogoo00Eoo`03001oogoo03]oo`03001oogoo03aoo`03001oogoo00ioo`00RGoo00<007oo
Ool017oo00<007ooOol0?7oo00<007ooOol0>Woo00<007ooOol03goo0029Ool00`00Oomoo`04Ool0
0`00Oomoo`0mOol00`00Oomoo`0iOol00`00Oomoo`0?Ool008Aoo`03001oogoo009oo`03001oogoo
00Aoo`03001oogoo03eoo`03001oogoo03Uoo`03001oogoo00moo`00QGoo10001goo00<007ooOol0
?Woo00<007ooOol0=goo00<007ooOol047oo002@Ool00`00Oomoo`0oOol00`00Oomoo`0fOol00`00
Oomoo`0@Ool0091oo`03001oogoo03moo`03001oogoo03Ioo`03001oogoo011oo`00T7oo00<007oo
Ool0@7oo00<007ooOol0=7oo00<007ooOol04Goo002@Ool20012Ool00`00Oomoo`0cOol00`00Oomo
o`0AOol0091oo`03001oogoo045oo`03001oogoo039oo`03001oogoo019oo`00T7oo00<007ooOol0
@Woo00<007ooOol0<Goo00<007ooOol04Woo002@Ool00`00Oomoo`13Ool00`00Oomoo`0_Ool00`00
Oomoo`0COol0091oo`03001oogoo04=oo`03001oogoo02moo`03001oogoo01=oo`00T7oo00<007oo
Ool0A7oo00<007ooOol0;Woo00<007ooOol04goo002@Ool00`00Oomoo`14Ool00`00Oomoo`0]Ool0
0`00Oomoo`0DOol0091oo`03001oogoo04Eoo`03001oogoo02aoo`03001oogoo01Aoo`00T7oo00<0
07ooOol0AWoo00<007ooOol0:Woo00<007ooOol05Goo002@Ool00`00Oomoo`16Ool00`00Oomoo`0Z
Ool00`00Oomoo`0EOol0091oo`8004Qoo`03001oogoo02Qoo`03001oogoo01Ioo`00T7oo00<007oo
Ool0B7oo00<007ooOol09goo00<007ooOol05Woo002@Ool00`00Oomoo`18Ool00`00Oomoo`0VOol0
0`00Oomoo`0GOol0091oo`03001oogoo04Uoo`03001oogoo02Eoo`03001oogoo01Moo`00T7oo00<0
07ooOol0BWoo00<007ooOol08goo00<007ooOol067oo002@Ool00`00Oomoo`1;Ool00`00Oomoo`0Q
Ool00`00Oomoo`0IOol0091oo`03001oogoo04aoo`03001oogoo021oo`03001oogoo01Uoo`00T7oo
00<007ooOol0CGoo00<007ooOol07Woo00<007ooOol06Woo002@Ool00`00Oomoo`1>Ool00`00Oomo
o`0MOol00`00Oomoo`0JOol0091oo`03001oogoo04moo`03001oogoo01]oo`03001oogoo01]oo`00
T7oo00<007ooOol0D7oo00<007ooOol06Woo00<007ooOol06goo002@Ool2001BOol00`00Oomoo`0H
Ool00`00Oomoo`0LOol0091oo`03001oogoo059oo`03001oogoo01Moo`03001oogoo01aoo`00T7oo
00<007ooOol0Dgoo00<007ooOol05Goo00<007ooOol07Goo002@Ool00`00Oomoo`1DOol00`00Oomo
o`0COol00`00Oomoo`0NOol0091oo`03001oogoo05Eoo`03001oogoo015oo`03001oogoo01moo`00
T7oo00<007ooOol0EWoo0P003goo0P008Woo002@Ool00`00Oomoo`1HOol00`00Oomoo`0;Ool00`00
Oomoo`0ROol008Moo`<000Ioo`03001oogoo05Uoo`8000Yoo`03001oogoo02=oo`00R7oo00<007oo
Ool01Goo00<007ooOol0Fgoo0`0017oo0`009Woo0024Ool60006Ool00`00Oomoo`1NOol4000YOol0
08Aoo`05001oogooOol00007Ool2002<Ool008Eoo`04001oogoo0007Ool00`00Oomoo`2;Ool008Io
o`03001oo`0000Moo`03001oogoo08]oo`00QWoo00<007oo00001goo00<007ooOol0Rgoo0027Ool2
002EOol008Qoo`03001oogoo09=oo`00ogoo7goo0000\
\>"],
  ImageRangeCache->{{{0, 285.5}, {175.875, 0}} -> {-3.33448, -4.33265, 
  0.0347316, 0.0739042}}]
}, Open  ]],

Cell[TextData[{
  "In this simple program we define the function ",
  StyleBox["f",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " that gives the fourth power of its argument and then apply ",
  StyleBox["f",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " twice to ",
  StyleBox["y",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  Use an ordinary RETURN to get the second line and then SHIFT-RETURN to \
execute the expression. Note the colon and semicolon which we shall examine \
later."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
f[x_] := x^4;  
f[f[y]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(y\^16\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The final example shows the incremental capabilities of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by using the ",
  StyleBox["Table",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function to produce a list of the cubes of integers up to 10."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["cubes = Table[ i^3, {i, 0, 10}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({0, 1, 8, 27, 64, 125, 216, 343, 512, 729, 1000}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We can plot given data like the list ",
  StyleBox["cubes",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " using the function ",
  StyleBox["ListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[".",
    FontFamily->"Courier"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["ListPlot[cubes];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0714286 0.0952381 0.0147151 0.000588604 [
[.30952 .00222 -3 -9 ]
[.30952 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.69048 .00222 -3 -9 ]
[.69048 .00222 3 0 ]
[.88095 .00222 -6 -9 ]
[.88095 .00222 6 0 ]
[.10655 .13244 -18 -4.5 ]
[.10655 .13244 0 4.5 ]
[.10655 .25016 -18 -4.5 ]
[.10655 .25016 0 4.5 ]
[.10655 .36788 -18 -4.5 ]
[.10655 .36788 0 4.5 ]
[.10655 .4856 -18 -4.5 ]
[.10655 .4856 0 4.5 ]
[.10655 .60332 -24 -4.5 ]
[.10655 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.30952 .01472 m
.30952 .02097 L
s
[(4)] .30952 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(6)] .5 .00222 0 1 Mshowa
.69048 .01472 m
.69048 .02097 L
s
[(8)] .69048 .00222 0 1 Mshowa
.88095 .01472 m
.88095 .02097 L
s
[(10)] .88095 .00222 0 1 Mshowa
.125 Mabswid
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.07143 .01472 m
.07143 .01847 L
s
.02381 .01472 m
.02381 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.97619 .01472 m
.97619 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.11905 .13244 m
.1253 .13244 L
s
[(200)] .10655 .13244 1 0 Mshowa
.11905 .25016 m
.1253 .25016 L
s
[(400)] .10655 .25016 1 0 Mshowa
.11905 .36788 m
.1253 .36788 L
s
[(600)] .10655 .36788 1 0 Mshowa
.11905 .4856 m
.1253 .4856 L
s
[(800)] .10655 .4856 1 0 Mshowa
.11905 .60332 m
.1253 .60332 L
s
[(1000)] .10655 .60332 1 0 Mshowa
.125 Mabswid
.11905 .04415 m
.1228 .04415 L
s
.11905 .07358 m
.1228 .07358 L
s
.11905 .10301 m
.1228 .10301 L
s
.11905 .16187 m
.1228 .16187 L
s
.11905 .1913 m
.1228 .1913 L
s
.11905 .22073 m
.1228 .22073 L
s
.11905 .27959 m
.1228 .27959 L
s
.11905 .30902 m
.1228 .30902 L
s
.11905 .33845 m
.1228 .33845 L
s
.11905 .39731 m
.1228 .39731 L
s
.11905 .42674 m
.1228 .42674 L
s
.11905 .45617 m
.1228 .45617 L
s
.11905 .51503 m
.1228 .51503 L
s
.11905 .54446 m
.1228 .54446 L
s
.11905 .57389 m
.1228 .57389 L
s
.25 Mabswid
.11905 0 m
.11905 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.008 w
.02381 .01472 Mdot
.11905 .0153 Mdot
.21429 .01942 Mdot
.30952 .03061 Mdot
.40476 .05239 Mdot
.5 .08829 Mdot
.59524 .14185 Mdot
.69048 .21661 Mdot
.78571 .31608 Mdot
.88095 .44381 Mdot
.97619 .60332 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{213, 131.562},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004o0000aA000`40O003h00Oogoo@7oo003oOom0
Ool00?mood1oo`00ogoo@7oo001YOol3000cOol4000cOol4000`Ool50005Ool2000YOol006Yoo`03
001oogoo035oo`03001oogoo009oo`03001oogoo02moo`03001oogoo009oo`03001oogoo02moo`03
001oogoo00Aoo`04001oogoo000XOol006Ioo`H0039oo`03001oogoo009oo`03001oogoo02moo`03
001oogoo009oo`03001oogoo02moo`03001oogoo00=oo`03001oogoo009oo`03001oogoo02Eoo`00
IWoo00D007ooOomoo`0003=oo`03001oogoo009oo`03001oogoo02moo`03001oogoo009oo`03001o
ogoo02moo`03001oogoo00=oo`03001oogoo009oo`03001oogoo02Eoo`00Igoo00@007ooOol003=o
o`8000=oo`03001oogoo031oo`@0039oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo02Eo
o`00J7oo00<007oo0000<goo00<007oo00000P00<Woo00<007ooOol00Woo00<007ooOol0;goo00<0
07ooOol00goo00<007ooOol00Woo00<007ooOol09Goo001XOol00`00Ool0000dOol00`00Oomoo`0c
Ool00`00Oomoo`02Ool00`00Oomoo`0_Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`0U
Ool006Uoo`8003Aoo`8003Aoo`03001oogoo009oo`03001oogoo02eoo`<000Ioo`04001oogoo000X
Ool006Yoo`03001oogoo03Aoo`<0039oo`@0035oo`8000Moo`8002Uoo`00<Woo00<007ooOol0ogoo
2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo
2goo000?Oooo000W000:Ool001Eoo`8000eoo`03001oogoo00Yoo`8000eoo`03001oogoo00Yoo`80
00aoo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo
00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo
00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo
00]oo`03001oogoo00moo`00<Woo00<007ooOol067oo0P006Woo00<007ooOol0=Goo00<007ooOol0
=Goo00<007ooOol0=Goo00<007ooOol0:goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0=7oo0P00e7oo000bOol00`00Oomoo`0dOol2003DOol0039oo`03
001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol2003oOol<Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0D7oo0P00^7oo000bOol00`00Oomoo`1@Ool2002hOol0039oo`03
001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03
001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol2003oOol<Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
06aoo`8009aoo`00<Woo00<007ooOol0K7oo0P00W7oo000bOol00`00Oomoo`3oOol;Ool0039oo`03
001oogoo0?moo`]oo`00<Woo0P00ogoo37oo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`0057oo1P001Goo0P001goo0P0027oo00<007ooOol0ogoo2goo000EOol01@00Oomoogoo
000017oo00@007ooOol000Eoo`04001oogoo0007Ool00`00Oomoo`3oOol;Ool001Ioo`03001oogoo
00Aoo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`04Ool00`00Oomoo`3oOol;Ool0
01Moo`03001oogoo00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`04Ool2003o
Ool<Ool001Qoo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`04
Ool00`00Oomoo`3oOol;Ool001Uoo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007oo
Ool017oo00<007ooOol0Qgoo0P00PGoo000IOol01@00Oomoogoo000017oo00D007ooOomoo`0000Ao
o`03001oogoo00Aoo`03001oogoo08Moo`80085oo`0057oo00<007ooOol00Woo00<007ooOol00Woo
00@007ooOol000Eoo`04001oogoo0007Ool00`00Oomoo`3oOol;Ool001Eoo`@000Ioo`8000Moo`80
00Qoo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0
039oo`800?moo`aoo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03
001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03
001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`80
0?moo`aoo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0Xgoo0P00IGoo000bOol00`00Oomoo`2SOol2001UOol0039oo`80
0?moo`aoo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo
0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool001Moo`<000Eoo`80
00Moo`8000Qoo`03001oogoo0?moo`]oo`0067oo00<007ooOol00goo00@007ooOol000Eoo`04001o
ogoo0007Ool00`00Oomoo`3oOol;Ool001Aoo`H000=oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`04Ool2003oOol<Ool001Aoo`05001oogooOol00004Ool00`00Oomoo`02Ool01@00
Oomoogoo000017oo00<007ooOol017oo00<007ooOol0ogoo2goo000EOol01000Oomoo`0017oo00<0
07ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo00Aoo`03001oogoo0?moo`]oo`005Woo00<0
07oo000017oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo00Aoo`03001oogoo0?mo
o`]oo`005Woo00<007oo000017oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo00Ao
o`03001oogoo0?moo`]oo`005goo0P001Goo00@007ooOol000Eoo`04001oogoo0007Ool00`00Oomo
o`3oOol;Ool001Qoo`03001oogoo00Aoo`8000Moo`8000Qoo`03001oogoo0?moo`]oo`00<Woo00<0
07ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`800?moo`aoo`00<Woo00<007ooOol0
ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0
ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0
ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`800?moo`aoo`00<Woo00<007ooOol0_goo0P00
BGoo000bOol00`00Oomoo`2oOol20019Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0
ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0
ogoo2goo000bOol2003oOol<Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo
000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo
000EOol40006Ool20007Ool20008Ool00`00Oomoo`3oOol;Ool001Aoo`03001oogoo009oo`03001o
ogoo009oo`04001oogoo0005Ool01000Oomoo`001goo00<007ooOol0ogoo2goo000DOol00`00Oomo
o`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001oogoo00Aoo`03001oogoo0?mo
o`]oo`0057oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomo
o`04Ool2003oOol<Ool001Aoo`8000=oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<0
07ooOol017oo00<007ooOol0ogoo2goo000DOol00`00Ool000020004Ool00`00Oomoo`02Ool01@00
Oomoogoo000017oo00<007ooOol017oo00<007ooOol0ogoo2goo000EOol00`00Oomoo`05Ool00`00
Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol017oo00<007ooOol0ogoo2goo000EOol20007
Ool01000Oomoo`001Goo00@007ooOol000Moo`03001oogoo0?moo`]oo`005goo0`001Goo0P001goo
0P0027oo00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]o
o`00<Woo0P00ogoo37oo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo
00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo
00<007ooOol0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo
0P00ogoo37oo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007oo
Ool0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0=]oo`8002eoo`00<Woo00<0
07ooOol0fgoo0P00;Goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo
0P00ogoo37oo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007oo
Ool0ogoo2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`005Goo10001Woo
0P001goo0P0027oo00<007ooOol0ogoo2goo000DOol00`00Oomoo`02Ool00`00Oomoo`02Ool01000
Oomoo`001Goo00@007ooOol000Moo`03001oogoo0?moo`]oo`0057oo00<007ooOol00Woo00D007oo
Oomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`04Ool2003oOol<Ool001Aoo`03001oogoo
009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol017oo00<007ooOol0ogoo
2goo000EOol40004Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol017oo00<007oo
Ool0ogoo2goo000DOol00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03
001oogoo00Aoo`03001oogoo0?moo`]oo`0057oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05
001oogooOol00004Ool00`00Oomoo`04Ool00`00Oomoo`3oOol;Ool001Aoo`03001oogoo009oo`03
001oogoo009oo`04001oogoo0005Ool01000Oomoo`001goo00<007ooOol0ogoo2goo000EOol40006
Ool20007Ool20008Ool00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007oo
Ool0ogoo2goo000bOol2003oOol<Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo
2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo
2goo000bOol00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo
2goo000bOol2003oOol<Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000b
Ool00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000b
Ool00`00Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo0P00ogoo37oo000bOol00`00
Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`00<Woo00<007ooOol0ogoo2goo000bOol00`00
Oomoo`3oOol;Ool0039oo`03001oogoo0?moo`]oo`0037oo1@001Goo0P001goo0P001goo0P0027oo
00<007ooOol0ogoo2goo000>Ool00`00Oomoo`04Ool01000Oomoo`001Goo00@007ooOol000Eoo`04
001oogoo0007Ool00`00Oomoo`3oOol;Ool000ioo`03001oogoo00=oo`03001oogoo009oo`05001o
ogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol017oo00<007ooOol0mgoo0P004Goo000>
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03
001oogoo00Aoo`800?Qoo`80015oo`003Woo00<007ooOol00goo00<007ooOol00Woo00D007ooOomo
o`0000Aoo`05001oogooOol00004Ool00`00Oomoo`04Ool00`00Oomoo`3oOol;Ool000ioo`03001o
ogoo00=oo`03001oogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol0
17oo00<007ooOol0ogoo2goo000>Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo0000
17oo00D007ooOomoo`0000Aoo`03001oogoo00Aoo`03001oogoo0?moo`]oo`0037oo0`001Woo00@0
07ooOol000Eoo`04001oogoo0005Ool01000Oomoo`001goo00<007ooOol0ogoo2goo000=Ool20007
Ool20007Ool20007Ool2003oOolFOol00?mood1oo`00\
\>"],
  ImageRangeCache->{{{0, 318.5}, {196.312, 0}} -> {0.205204, -99.5394, 
  0.0537731, 8.70066}}]
}, Open  ]],

Cell[TextData[{
  "Don't worry about the details of the positioning of the axes for now. But \
you should note that if asked to plot a simple one-dimensional list, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will use the position in the list as the \"",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "\" or horizontal coordinate, so that, e.g., the last element is plotted as \
the point (11, 1000)."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Remarks", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In our examples we have seen the four basic areas of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". These are:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Numerical Calculations", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
For example, we asked for the numerical value of \[Pi] to 100 \
digits.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Symbolic or Algebraic Calculations", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We asked for the expansion of ",
  Cell[BoxData[
      \(TraditionalForm\`\((2  x + y)\)\^3\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphics", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["We plotted a function, and a sequence of numbers.", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Programming", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
We defined a function, and asked for it to be applied to a specific \
argument.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We will explore each of these topics in more detail later. Right now it is \
necessary to do some groundwork on understanding how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " works. From here on, the expressions that we enter may be a little \
longer, and if we make a mistake we don't want to have to keep re-typing our \
input. So at this point it will be useful to learn a few more tricks for \
managing our input. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using the Cursor", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
If you move the cursor up and down the Notebook in which you have \
just done the previous calculations you will see how it changes form \
dependent on which type of cell it is lying in, or whether it is between \
cells. A horizontal line under a cell indicates that any text typed will be \
placed after that cell. To place a horizontal line after any cell you move \
the cursor so that it indicates that it is lying between cells, i.e. when it \
is horizontal, and then click. Practice will get you used to being able to \
place text anywhere in a Notebook.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Opening and Closing Cells", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In the Notebook environment all text is held in cells. These can \
have different styles associated with them, and can be set up to allow \
different styles of cell to be nested within one another. By double clicking \
on any cell bracket which is grouping other cells you will be able to hide \
the text contained in those sub-cells. This can be retrieved by \
double-clicking on the same cell bracket again. When a cell contains hidden \
sub-cells you will notice how the lower edge of the bracket changes to a \
filled triangle. You may find this way of hiding information in the Notebooks \
useful in organizing a presentation or teaching session. It may take some \
practice to get used to how these cells are nested.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Editing Text in Cells", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Changing the text in a cell is done in a similar way to that used \
in word-processing applications. The cursor can be placed at any particular \
point in the text by moving the mouse and clicking on that spot. Selecting \
text with the mouse allows that text to be removed or typed over. Notice how \
you can only edit text in certain cells. You will be able to edit your input \
cells but not in the cells containing the output. This has been set by the \
way in which the different cells are defined. Here are some handy tips:

(1) use the mouse and scroll bars to get up and down the Notebook;
(2) use an ordinary RETURN to enter the next line of a multi-line expression;
(3) use SHIFT-RETURN (or ENTER) to execute an expression;
(4) use the mouse COPY and PASTE to grab previous formulae.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Quitting and Panicking", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[TextData[{
  "Sometimes, say if you make a mistake, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may go into a long internal session and all you want it to do is stop. If \
you need to stop ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in the middle of a calculation, go to the menu and choose \"Kernel\". You \
then get some options, of which the ones to pick are \"Abort Evaluation\" or \
\"Interrupt Evaluation...\".  The menus also offer keyboard short-cuts \
specific to your operating system, which you should learn. The kernel can \
sometimes take time to abort a calculation. It is better to wait for it to \
finish sorting itself out than to keep thumping the keyboard! If you lose \
patience waiting, or otherwise wish to exit ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " completely, you can click on \"Quit\" having interrupted the session, or \
type ",
  StyleBox["Quit",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " at an \"In\" prompt. There are other options in the \"Kernel\" menu for \
quitting and re-starting. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Brackets Galore", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Our first examples show that we need to understand how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " reads expressions, and especially different types of brackets: [ ], { }, \
( ). Sometimes people use square brackets [], round brackets () and curly \
brackets {} more or less interchangeably. In order to ensure that the \
computer understands unambiguously what we mean, it is necessary to introduce \
some conventions. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Arguments of Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Arguments of functions are always enclosed in square brackets. This \
applies just as much to standard mathematical functions like ",
  StyleBox["Sin[x]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " as to built-in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " graphics functions like ",
  StyleBox["Plot3D[]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", which take another function and some ranges as a minimum set of \
arguments, and user-defined functions like ",
  StyleBox["f",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " above. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Grouping", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[TextData[{
  "When we wish to group together some expressions for applying a subsequent \
operation, round brackets are employed. For example ",
  StyleBox["(x + y)^3",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " means the cube of all of ",
  StyleBox["x + y",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  Note that this distinction gives the expressions ",
  StyleBox["f[x+y]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["f(x+y)",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " two quite different meanings, and at the same time we have acquired the \
freedom to use a space between adjacent quantities to denote multiplication. \
If you wish to you can explicitly type a * to denote multiplication. For \
example ",
  StyleBox["2 3",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is identical in meaning to ",
  StyleBox["2*3",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".\n\nNote that you don't always need to use round brackets for grouping, \
since the usual rules of precedence apply. For example ",
  StyleBox["x^3 + 4",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " means ",
  StyleBox["(x^3) + 4",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and not ",
  StyleBox["x^(3 + 4)",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". A good policy is to use grouping brackets in any situation where there \
is a potential for ambiguity. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["List Brackets", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Curly brackets ",
  StyleBox["{ }",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and double square brackets ",
  StyleBox["[[ ]]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " are also used in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". These are used with list structures to denote a list, and an indexed \
member of a list, respectively. We shall look specifically at these \
structures later on. For now it will suffice to appreciate that a list \
consists of a number of elements, separated by commas, with a curly bracket \
at each end."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Input/Output, Standard and Traditional Forms", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[TextData[{
  "Versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " prior to version 3.0 used styles where inputs and outputs were both based \
on pure text representations. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 adds new forms called \"Standard Form\" and \"Traditional Form\". \
These are much more like traditional mathematical notation - indeed, this is \
exactly the idea of \"Traditional Form\". \"Standard Form\" is a half-way \
form that goes as far as possible towards traditional mathematical notation \
without introducing any ambiguities. Throughout this chapter the default \
output format type is \"Standard\". You can ask for explicit conversion as \
follows:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(expression\  = \ Exp[\(-x^2\)]; \[IndentingNewLine]StandardForm[
      expression]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-x\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[expression]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(-x\^2\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(InputForm[expression]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["E^(-x^2)", "Output",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "These conversions can be done using the menus, and defaults can also be \
set. It is impossible to give a hard and fast rule that has been applied \
throughout this book, for how expressions are represented, as the choice is \
rather sensitive to the context, but we try to adhere to the \"WYSIWMAMA\"  \
(what you see is what ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " made) principle - note the partially double-struck notation. Generally, \
chapters involving more mathematical development will make more use of the \
traditional form for outputs and equations. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.2 Basic Graphics", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Before we launch into investigating each of the main areas of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in detail, it will be useful to look at some of the basic graphic \
commands used in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". While working in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " it is very often helpful to involve graphics in a calculation. This is \
never more true than when looking at new ideas and will be used extensively \
through this chapter and the rest of the book, to illustrate points more \
clearly.  For this reason, it will be useful if you feel reasonably \
comfortable with the most basic graphics commands at the outset. Do not be \
too concerned about the details of their operation at this point. We shall \
look more carefully at the range of graphics capabilities available in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " later."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Producing a Plot", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We have already seen the function ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " which will display the plot for a given function. This needs to have the \
range of values over which the plot is to be made specified. You supply ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " with two arguments - the first is the function, and the second is a list \
in the form ",
  StyleBox["{variable, start, end}",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Plot[x^3 - 8 x^2 + 19 x - 8, {x, 0, 5}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.190476 0.0488093 0.0462091 [
[.21429 .03631 -3 -9 ]
[.21429 .03631 3 0 ]
[.40476 .03631 -3 -9 ]
[.40476 .03631 3 0 ]
[.59524 .03631 -3 -9 ]
[.59524 .03631 3 0 ]
[.78571 .03631 -3 -9 ]
[.78571 .03631 3 0 ]
[.97619 .03631 -3 -9 ]
[.97619 .03631 3 0 ]
[.01131 .14123 -6 -4.5 ]
[.01131 .14123 0 4.5 ]
[.01131 .23365 -6 -4.5 ]
[.01131 .23365 0 4.5 ]
[.01131 .32606 -6 -4.5 ]
[.01131 .32606 0 4.5 ]
[.01131 .41848 -6 -4.5 ]
[.01131 .41848 0 4.5 ]
[.01131 .5109 -12 -4.5 ]
[.01131 .5109 0 4.5 ]
[.01131 .60332 -12 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .04881 m
.21429 .05506 L
s
[(1)] .21429 .03631 0 1 Mshowa
.40476 .04881 m
.40476 .05506 L
s
[(2)] .40476 .03631 0 1 Mshowa
.59524 .04881 m
.59524 .05506 L
s
[(3)] .59524 .03631 0 1 Mshowa
.78571 .04881 m
.78571 .05506 L
s
[(4)] .78571 .03631 0 1 Mshowa
.97619 .04881 m
.97619 .05506 L
s
[(5)] .97619 .03631 0 1 Mshowa
.125 Mabswid
.0619 .04881 m
.0619 .05256 L
s
.1 .04881 m
.1 .05256 L
s
.1381 .04881 m
.1381 .05256 L
s
.17619 .04881 m
.17619 .05256 L
s
.25238 .04881 m
.25238 .05256 L
s
.29048 .04881 m
.29048 .05256 L
s
.32857 .04881 m
.32857 .05256 L
s
.36667 .04881 m
.36667 .05256 L
s
.44286 .04881 m
.44286 .05256 L
s
.48095 .04881 m
.48095 .05256 L
s
.51905 .04881 m
.51905 .05256 L
s
.55714 .04881 m
.55714 .05256 L
s
.63333 .04881 m
.63333 .05256 L
s
.67143 .04881 m
.67143 .05256 L
s
.70952 .04881 m
.70952 .05256 L
s
.74762 .04881 m
.74762 .05256 L
s
.82381 .04881 m
.82381 .05256 L
s
.8619 .04881 m
.8619 .05256 L
s
.9 .04881 m
.9 .05256 L
s
.9381 .04881 m
.9381 .05256 L
s
.25 Mabswid
0 .04881 m
1 .04881 L
s
.02381 .14123 m
.03006 .14123 L
s
[(2)] .01131 .14123 1 0 Mshowa
.02381 .23365 m
.03006 .23365 L
s
[(4)] .01131 .23365 1 0 Mshowa
.02381 .32606 m
.03006 .32606 L
s
[(6)] .01131 .32606 1 0 Mshowa
.02381 .41848 m
.03006 .41848 L
s
[(8)] .01131 .41848 1 0 Mshowa
.02381 .5109 m
.03006 .5109 L
s
[(10)] .01131 .5109 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(12)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .07191 m
.02756 .07191 L
s
.02381 .09502 m
.02756 .09502 L
s
.02381 .11812 m
.02756 .11812 L
s
.02381 .16433 m
.02756 .16433 L
s
.02381 .18744 m
.02756 .18744 L
s
.02381 .21054 m
.02756 .21054 L
s
.02381 .25675 m
.02756 .25675 L
s
.02381 .27985 m
.02756 .27985 L
s
.02381 .30296 m
.02756 .30296 L
s
.02381 .34917 m
.02756 .34917 L
s
.02381 .37227 m
.02756 .37227 L
s
.02381 .39538 m
.02756 .39538 L
s
.02381 .44159 m
.02756 .44159 L
s
.02381 .46469 m
.02756 .46469 L
s
.02381 .4878 m
.02756 .4878 L
s
.02381 .53401 m
.02756 .53401 L
s
.02381 .55711 m
.02756 .55711 L
s
.02381 .58021 m
.02756 .58021 L
s
.02381 .0257 m
.02756 .0257 L
s
.02381 .0026 m
.02756 .0026 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.10874 0 m
.14415 .09793 L
.18221 .18018 L
.20342 .21707 L
.22272 .24547 L
.26171 .28907 L
.28086 .3043 L
.29075 .3107 L
.30164 .31666 L
.3112 .32099 L
.32148 .32474 L
.32715 .32642 L
.33233 .32773 L
.33726 .32878 L
.34251 .32968 L
.34767 .33037 L
.35235 .33082 L
.35461 .33098 L
.3558 .33105 L
.35706 .33111 L
.35816 .33116 L
.35937 .3312 L
.36047 .33123 L
.36149 .33125 L
.36278 .33126 L
.36351 .33127 L
.36418 .33127 L
.36541 .33126 L
.36672 .33124 L
.36789 .33121 L
.36899 .33118 L
.37149 .33107 L
.37418 .33091 L
.37665 .33072 L
.38221 .33016 L
.38695 .32954 L
.39202 .32873 L
.40114 .32691 L
.42184 .32121 L
.44318 .31336 L
.46334 .30446 L
.5018 .28465 L
.54272 .26167 L
.58212 .24021 L
.62001 .22251 L
.63946 .21526 L
.66035 .2093 L
.67048 .2072 L
.67583 .20632 L
.68153 .20556 L
Mistroke
.68435 .20525 L
.687 .20501 L
.68958 .20482 L
.69196 .20468 L
.69422 .20458 L
.69536 .20455 L
.69661 .20451 L
.69786 .20449 L
.69856 .20449 L
.69921 .20448 L
.70045 .20448 L
.70163 .20449 L
.70277 .20451 L
.704 .20454 L
.70517 .20458 L
.70623 .20463 L
.70871 .20476 L
.71131 .20494 L
.71622 .20543 L
.72158 .20616 L
.73113 .20801 L
.74001 .21039 L
.75056 .21407 L
.76051 .21843 L
.77911 .22906 L
.79916 .24437 L
.81763 .26228 L
.85708 .31396 L
.89502 .3828 L
.93541 .47919 L
.97429 .59689 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{171, 105.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool002Yo
o`<001]oo`@001]oo`<001aoo`<001]oo`<000Aoo`003Woo00<007ooOol02Woo00<007ooOol03Goo
00<007ooOol06goo00<007ooOol07Woo00<007ooOol06Woo00<007ooOol07Goo00<007ooOol00Goo
000>Ool00`00Oomoo`0:Ool00`00Oomoo`0=Ool00`00Oomoo`0LOol00`00Oomoo`0MOol00`00Oomo
o`0HOol4000NOol00`00Oomoo`01Ool000ioo`03001oogoo00]oo`03001oogoo00aoo`03001oogoo
01eoo`03001oogoo01Yoo`8001]oo`03001oo`0001aoo`<000Aoo`003Woo00<007ooOol02goo00<0
07ooOol037oo00<007ooOol06Woo00@007ooOol001ioo`03001oogoo01Qoo`03001oo`0001aoo`03
001oogoo00Aoo`003Woo00<007ooOol02goo00<007ooOol02goo0P007Goo0P0077oo0`0077oo0P00
77oo10000goo000>Ool00`00Oomoo`0<Ool00`00Oomoo`2;Ool000ioo`03001oogoo00aoo`03001o
ogoo08]oo`003Woo00<007ooOol037oo00<007ooOol0Rgoo000:OonQ000000ioo`03001oogoo00=o
o`03001oogoo00=oo`07001oogooOol007oo00001Goo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<0
07ooOol00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo000>
Ool00`00Oomoo`0>Ool00`00Oomoo`29Ool000ioo`03001oogoo00ioo`03001oogoo08Uoo`003Woo
00<007ooOol03Woo00<007ooOol0RGoo000>Ool00`00Oomoo`0?Ool00`00Oomoo`28Ool000ioo`03
001oogoo00moo`03001oogoo08Qoo`003Woo00<007ooOol03goo00<007ooOol0R7oo000>Ool00`00
Oomoo`0@Ool00`00Oomoo`27Ool000ioo`03001oogoo011oo`03001oogoo08Moo`003Woo00<007oo
Ool04Goo00<007ooOol0QWoo000>Ool00`00Oomoo`0AOol00`00Oomoo`26Ool000ioo`03001oogoo
019oo`03001oogoo08Eoo`001goo10000goo00<007ooOol04Woo00<007ooOol0QGoo0008Ool00`00
Oomoo`03Ool00`00Oomoo`0COol00`00Oomoo`24Ool000Uoo`03001oogoo009oo`03001oogoo01=o
o`03001oogoo08Aoo`002Woo00D007ooOomoo`0001Eoo`03001oogoo08Aoo`001goo00@007ooOol0
00=oo`03001oogoo01Aoo`03001oogoo08=oo`0027oo0P0017oo00<007ooOol057oo00<007ooOol0
Pgoo000>Ool00`00Oomoo`0EOol00`00Oomoo`22Ool000ioo`03001oogoo01Eoo`03001oogoo089o
o`003Woo00<007ooOol05Woo00<007ooOol0PGoo000>Ool00`00Oomoo`0FOol00`00Oomoo`21Ool0
00ioo`03001oogoo01Moo`03001oogoo081oo`003Woo00<007ooOol05goo00<007ooOol0P7oo000>
Ool00`00Oomoo`0HOol00`00Oomoo`1:Ool9000/Ool000ioo`03001oogoo01Qoo`03001oogoo04Io
o`@000Uoo`<002Uoo`003Woo00<007ooOol06Goo00<007ooOol0@Woo0`0047oo0`009Woo0008Ool3
0003Ool00`00Oomoo`0IOol00`00Oomoo`0oOol3000FOol00`00Oomoo`0SOol000Uoo`03001oogoo
009oo`03001oogoo01Yoo`03001oogoo03aoo`8001Yoo`8002=oo`001goo10000goo00<007ooOol0
6goo00<007ooOol0>Goo0P007Woo00<007ooOol087oo0007Ool00`00Ool00004Ool00`00Oomoo`0K
Ool00`00Oomoo`0gOol2000QOol00`00Oomoo`0OOol000Moo`03001oo`0000Aoo`03001oogoo01ao
o`03001oogoo03=oo`<002Aoo`03001oogoo01ioo`0027oo0P0017oo00<007ooOol07Goo00<007oo
Ool0<7oo0P00:7oo00<007ooOol07Goo000>Ool00`00Oomoo`0NOol00`00Oomoo`0]Ool2000[Ool0
0`00Oomoo`0LOol000ioo`03001oogoo01moo`03001oogoo02]oo`03001oogoo02]oo`03001oogoo
01aoo`003Woo00<007ooOol087oo00<007ooOol0:7oo0P00;goo00<007ooOol06goo000>Ool00`00
Oomoo`0QOol00`00Oomoo`0VOol00`00Oomoo`0`Ool00`00Oomoo`0JOol000ioo`03001oogoo029o
o`03001oogoo02=oo`8003Aoo`03001oogoo01Uoo`003Woo00<007ooOol08goo0P008Goo0P00=goo
00<007ooOol067oo000>Ool00`00Oomoo`0UOol00`00Oomoo`0LOol2000iOol00`00Oomoo`0HOol0
00ioo`03001oogoo02Ioo`03001oogoo01Uoo`8003aoo`03001oogoo01Moo`0027oo0P0017oo00<0
07ooOol09goo0P005Woo0`00?goo00<007ooOol05Woo0007Ool01000Oomoo`000goo00<007ooOol0
:Goo00<007ooOol04Goo0P00@goo00<007ooOol05Goo0007Ool01000Oomoo`000goo00<007ooOol0
:Woo0`003Woo0P00AGoo00<007ooOol05Goo0007Ool30004Ool00`00Oomoo`0]Ool>0018Ool00`00
Oomoo`0DOol000Moo`03001oogoo00Aoo`03001oogoo08=oo`03001oogoo01Aoo`0027oo0`000goo
00<007ooOol0Q7oo00<007ooOol04goo000>Ool00`00Oomoo`24Ool00`00Oomoo`0COol000ioo`03
001oogoo08Eoo`03001oogoo019oo`003Woo00<007ooOol0QWoo00<007ooOol04Goo000>Ool00`00
Oomoo`26Ool00`00Oomoo`0AOol000ioo`03001oogoo08Moo`03001oogoo011oo`003Woo00<007oo
Ool0Qgoo00<007ooOol047oo000>Ool00`00Oomoo`28Ool00`00Oomoo`0?Ool000ioo`03001oogoo
08Qoo`03001oogoo00moo`003Woo00<007ooOol0RGoo00<007ooOol03Woo0008Ool20004Ool00`00
Oomoo`29Ool00`00Oomoo`0>Ool000Moo`04001oogoo0003Ool00`00Oomoo`29Ool00`00Oomoo`0>
Ool000Moo`04001oogoo0003Ool00`00Oomoo`2:Ool00`00Oomoo`0=Ool000Qoo`8000Aoo`03001o
ogoo08Yoo`03001oogoo00eoo`001goo00@007ooOol000=oo`03001oogoo08]oo`03001oogoo00ao
o`0027oo0P0017oo00<007ooOol0Rgoo00<007ooOol037oo000>Ool00`00Oomoo`2;Ool00`00Oomo
o`0<Ool000ioo`03001oogoo08aoo`03001oogoo00]oo`003Woo00<007ooOol0S7oo00<007ooOol0
2goo000>Ool00`00Oomoo`2=Ool00`00Oomoo`0:Ool000ioo`03001oogoo08eoo`03001oogoo00Yo
o`003Woo00<007ooOol0SGoo00<007ooOol02Woo000>Ool00`00Oomoo`2>Ool00`00Oomoo`09Ool0
00ioo`03001oogoo08ioo`03001oogoo00Uoo`003Woo00<007ooOol0SWoo00<007ooOol02Goo0000
17oo0000000017oo0P0017oo00<007ooOol0Sgoo00<007ooOol027oo0002Ool00`00Oomoo`02Ool0
1000Oomoo`000goo00<007ooOol0Sgoo00<007ooOol027oo0002Ool00`00Oomoo`02Ool01000Oomo
o`000goo00<007ooOol0Sgoo00<007ooOol027oo0002Ool00`00Oomoo`02Ool01000Oomoo`000goo
00<007ooOol0T7oo00<007ooOol01goo0002Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007oo
Ool0T7oo00<007ooOol01goo00000goo00000005Ool20004Ool00`00Oomoo`2@Ool00`00Oomoo`07
Ool000ioo`03001oogoo095oo`03001oogoo00Ioo`003Woo00<007ooOol0TGoo00<007ooOol01Woo
000>Ool00`00Oomoo`2AOol00`00Oomoo`06Ool000ioo`03001oogoo099oo`03001oogoo00Eoo`00
3Woo00<007ooOol0TWoo00<007ooOol01Goo000>Ool00`00Oomoo`2BOol00`00Oomoo`05Ool000io
o`03001oogoo09=oo`03001oogoo00Aoo`003Woo00<007ooOol0Tgoo00<007ooOol017oo000>Ool0
0`00Oomoo`2COol00`00Oomoo`04Ool00004Ool000000003Ool40003Ool00`00Oomoo`2DOol00`00
Oomoo`03Ool0009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo09Aoo`03001oogoo00=o
o`000Woo00<007ooOol017oo00<007ooOol00Woo00<007ooOol0UGoo00<007ooOol00Woo0002Ool0
0`00Oomoo`05Ool01@00Oomoogoo0000W7oo0002Ool00`00Oomoo`02Ool01000Oomoo`000goo00<0
07ooOol0VWoo00000goo00000005Ool2002QOol00:]oo`00\
\>"],
  ImageRangeCache->{{{0, 170}, {104.5, 0}} -> {-0.459877, -1.51557, 
  0.0328524, 0.135419}}]
}, Open  ]],

Cell[TextData[{
  "Given a set of values expressed as a list ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", the function ",
  StyleBox["ListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will plot ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " assuming that ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is regularly spaced, and will use the position in the list as the \
horizontal coordinate.  The command which changes the ",
  StyleBox["PlotStyle",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " plots the points large so that we can see them clearly."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
data = {3,-0.5,2,0,1,0.5};
points = ListPlot[data,PlotStyle -> PointSize[0.03]];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.190476 0.0988014 0.168173 [
[.21429 .0863 -3 -9 ]
[.21429 .0863 3 0 ]
[.40476 .0863 -3 -9 ]
[.40476 .0863 3 0 ]
[.59524 .0863 -3 -9 ]
[.59524 .0863 3 0 ]
[.78571 .0863 -3 -9 ]
[.78571 .0863 3 0 ]
[.97619 .0863 -3 -9 ]
[.97619 .0863 3 0 ]
[.01131 .01472 -24 -4.5 ]
[.01131 .01472 0 4.5 ]
[.01131 .18289 -18 -4.5 ]
[.01131 .18289 0 4.5 ]
[.01131 .26697 -6 -4.5 ]
[.01131 .26697 0 4.5 ]
[.01131 .35106 -18 -4.5 ]
[.01131 .35106 0 4.5 ]
[.01131 .43515 -6 -4.5 ]
[.01131 .43515 0 4.5 ]
[.01131 .51923 -18 -4.5 ]
[.01131 .51923 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .0988 m
.21429 .10505 L
s
[(2)] .21429 .0863 0 1 Mshowa
.40476 .0988 m
.40476 .10505 L
s
[(3)] .40476 .0863 0 1 Mshowa
.59524 .0988 m
.59524 .10505 L
s
[(4)] .59524 .0863 0 1 Mshowa
.78571 .0988 m
.78571 .10505 L
s
[(5)] .78571 .0863 0 1 Mshowa
.97619 .0988 m
.97619 .10505 L
s
[(6)] .97619 .0863 0 1 Mshowa
.125 Mabswid
.0619 .0988 m
.0619 .10255 L
s
.1 .0988 m
.1 .10255 L
s
.1381 .0988 m
.1381 .10255 L
s
.17619 .0988 m
.17619 .10255 L
s
.25238 .0988 m
.25238 .10255 L
s
.29048 .0988 m
.29048 .10255 L
s
.32857 .0988 m
.32857 .10255 L
s
.36667 .0988 m
.36667 .10255 L
s
.44286 .0988 m
.44286 .10255 L
s
.48095 .0988 m
.48095 .10255 L
s
.51905 .0988 m
.51905 .10255 L
s
.55714 .0988 m
.55714 .10255 L
s
.63333 .0988 m
.63333 .10255 L
s
.67143 .0988 m
.67143 .10255 L
s
.70952 .0988 m
.70952 .10255 L
s
.74762 .0988 m
.74762 .10255 L
s
.82381 .0988 m
.82381 .10255 L
s
.8619 .0988 m
.8619 .10255 L
s
.9 .0988 m
.9 .10255 L
s
.9381 .0988 m
.9381 .10255 L
s
.25 Mabswid
0 .0988 m
1 .0988 L
s
.02381 .01472 m
.03006 .01472 L
s
[(-0.5)] .01131 .01472 1 0 Mshowa
.02381 .18289 m
.03006 .18289 L
s
[(0.5)] .01131 .18289 1 0 Mshowa
.02381 .26697 m
.03006 .26697 L
s
[(1)] .01131 .26697 1 0 Mshowa
.02381 .35106 m
.03006 .35106 L
s
[(1.5)] .01131 .35106 1 0 Mshowa
.02381 .43515 m
.03006 .43515 L
s
[(2)] .01131 .43515 1 0 Mshowa
.02381 .51923 m
.03006 .51923 L
s
[(2.5)] .01131 .51923 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03153 m
.02756 .03153 L
s
.02381 .04835 m
.02756 .04835 L
s
.02381 .06517 m
.02756 .06517 L
s
.02381 .08198 m
.02756 .08198 L
s
.02381 .11562 m
.02756 .11562 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .14925 m
.02756 .14925 L
s
.02381 .16607 m
.02756 .16607 L
s
.02381 .1997 m
.02756 .1997 L
s
.02381 .21652 m
.02756 .21652 L
s
.02381 .23334 m
.02756 .23334 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .28379 m
.02756 .28379 L
s
.02381 .30061 m
.02756 .30061 L
s
.02381 .31743 m
.02756 .31743 L
s
.02381 .33424 m
.02756 .33424 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .38469 m
.02756 .38469 L
s
.02381 .40151 m
.02756 .40151 L
s
.02381 .41833 m
.02756 .41833 L
s
.02381 .45196 m
.02756 .45196 L
s
.02381 .46878 m
.02756 .46878 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .50242 m
.02756 .50242 L
s
.02381 .53605 m
.02756 .53605 L
s
.02381 .55287 m
.02756 .55287 L
s
.02381 .56968 m
.02756 .56968 L
s
.02381 .5865 m
.02756 .5865 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .60332 Mdot
.21429 .01472 Mdot
.40476 .43515 Mdot
.59524 .0988 Mdot
.78571 .26697 Mdot
.97619 .18289 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{171, 105.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCacheValid->False,
  ImageRangeCache->{{{0, 176}, {108.25, 0}} -> {0.114773, -0.843436, 
  0.0341492, 0.0386781}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Displaying a Plot Again", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "All graphics can be displayed again using the function ",
  StyleBox["Show",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[".",
    FontFamily->"Courier"],
  " We have given our last plot the name ",
  StyleBox["points",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  We can now ask to see that particular plot again."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Show[points];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.190476 0.0988014 0.168173 [
[.21429 .0863 -3 -9 ]
[.21429 .0863 3 0 ]
[.40476 .0863 -3 -9 ]
[.40476 .0863 3 0 ]
[.59524 .0863 -3 -9 ]
[.59524 .0863 3 0 ]
[.78571 .0863 -3 -9 ]
[.78571 .0863 3 0 ]
[.97619 .0863 -3 -9 ]
[.97619 .0863 3 0 ]
[.01131 .01472 -24 -4.5 ]
[.01131 .01472 0 4.5 ]
[.01131 .18289 -18 -4.5 ]
[.01131 .18289 0 4.5 ]
[.01131 .26697 -6 -4.5 ]
[.01131 .26697 0 4.5 ]
[.01131 .35106 -18 -4.5 ]
[.01131 .35106 0 4.5 ]
[.01131 .43515 -6 -4.5 ]
[.01131 .43515 0 4.5 ]
[.01131 .51923 -18 -4.5 ]
[.01131 .51923 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .0988 m
.21429 .10505 L
s
[(2)] .21429 .0863 0 1 Mshowa
.40476 .0988 m
.40476 .10505 L
s
[(3)] .40476 .0863 0 1 Mshowa
.59524 .0988 m
.59524 .10505 L
s
[(4)] .59524 .0863 0 1 Mshowa
.78571 .0988 m
.78571 .10505 L
s
[(5)] .78571 .0863 0 1 Mshowa
.97619 .0988 m
.97619 .10505 L
s
[(6)] .97619 .0863 0 1 Mshowa
.125 Mabswid
.0619 .0988 m
.0619 .10255 L
s
.1 .0988 m
.1 .10255 L
s
.1381 .0988 m
.1381 .10255 L
s
.17619 .0988 m
.17619 .10255 L
s
.25238 .0988 m
.25238 .10255 L
s
.29048 .0988 m
.29048 .10255 L
s
.32857 .0988 m
.32857 .10255 L
s
.36667 .0988 m
.36667 .10255 L
s
.44286 .0988 m
.44286 .10255 L
s
.48095 .0988 m
.48095 .10255 L
s
.51905 .0988 m
.51905 .10255 L
s
.55714 .0988 m
.55714 .10255 L
s
.63333 .0988 m
.63333 .10255 L
s
.67143 .0988 m
.67143 .10255 L
s
.70952 .0988 m
.70952 .10255 L
s
.74762 .0988 m
.74762 .10255 L
s
.82381 .0988 m
.82381 .10255 L
s
.8619 .0988 m
.8619 .10255 L
s
.9 .0988 m
.9 .10255 L
s
.9381 .0988 m
.9381 .10255 L
s
.25 Mabswid
0 .0988 m
1 .0988 L
s
.02381 .01472 m
.03006 .01472 L
s
[(-0.5)] .01131 .01472 1 0 Mshowa
.02381 .18289 m
.03006 .18289 L
s
[(0.5)] .01131 .18289 1 0 Mshowa
.02381 .26697 m
.03006 .26697 L
s
[(1)] .01131 .26697 1 0 Mshowa
.02381 .35106 m
.03006 .35106 L
s
[(1.5)] .01131 .35106 1 0 Mshowa
.02381 .43515 m
.03006 .43515 L
s
[(2)] .01131 .43515 1 0 Mshowa
.02381 .51923 m
.03006 .51923 L
s
[(2.5)] .01131 .51923 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03153 m
.02756 .03153 L
s
.02381 .04835 m
.02756 .04835 L
s
.02381 .06517 m
.02756 .06517 L
s
.02381 .08198 m
.02756 .08198 L
s
.02381 .11562 m
.02756 .11562 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .14925 m
.02756 .14925 L
s
.02381 .16607 m
.02756 .16607 L
s
.02381 .1997 m
.02756 .1997 L
s
.02381 .21652 m
.02756 .21652 L
s
.02381 .23334 m
.02756 .23334 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .28379 m
.02756 .28379 L
s
.02381 .30061 m
.02756 .30061 L
s
.02381 .31743 m
.02756 .31743 L
s
.02381 .33424 m
.02756 .33424 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .38469 m
.02756 .38469 L
s
.02381 .40151 m
.02756 .40151 L
s
.02381 .41833 m
.02756 .41833 L
s
.02381 .45196 m
.02756 .45196 L
s
.02381 .46878 m
.02756 .46878 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .50242 m
.02756 .50242 L
s
.02381 .53605 m
.02756 .53605 L
s
.02381 .55287 m
.02756 .55287 L
s
.02381 .56968 m
.02756 .56968 L
s
.02381 .5865 m
.02756 .5865 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .60332 Mdot
.21429 .01472 Mdot
.40476 .43515 Mdot
.59524 .0988 Mdot
.78571 .26697 Mdot
.97619 .18289 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{171, 105.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool00:]o
o`00Zgoo002[Ool00:]oo`00Zgoo0008Ool20004Ool20003Ool30003Ool00`00Oomoo`0IOol2001d
Ool000Moo`04001oogoo0003Ool20006Ool01000Oomoo`006Woo1000Lgoo00000goo000000030000
1Goo001oogoo00002goo00<007ooOol00P006Goo1000Lgoo0007Ool01000Oomoo`0027oo0`000goo
00<007ooOol06Goo0P00M7oo0007Ool01000Oomoo`0027oo00<007ooOol00goo0P00T7oo0008Ool2
0009Ool40002Ool00`00Oomoo`0GOol4000IOol3000JOol3000HOol3000JOol20004Ool001Uoo`03
001oogoo01Qoo`03001oogoo01aoo`03001oogoo01Qoo`03001oogoo01Yoo`03001oogoo01Ioo`04
001oogoo0003Ool001Uoo`8001Yoo`03001oogoo01]oo`03001oogoo01Ioo`@001]oo`03001oogoo
01Ioo`04001oogoo0003Ool001Uoo`03001oogoo01Yoo`03001oogoo01Qoo`8001Uoo`03001oo`00
01Uoo`<001Uoo`<000Aoo`006Goo0P0067oo00@007ooOol001aoo`03001oogoo01Ioo`03001oo`00
01Uoo`03001oogoo01Uoo`03001oogoo00Aoo`006Goo00<007ooOol067oo0P006Woo0`006Woo0P00
6Goo10006Goo0`000goo000IOol00`00Oomoo`2?Ool001Uoo`80091oo`006Goo00<007ooOol0DGoo
0P00?7oo000FOonE000001Uoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001o
ogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001o
ogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo009oo`03001o
ogoo00=oo`04001oogooOol40003Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool001Uoo`03001oogoo055oo`8003aoo`006Goo0P00T7oo000I
Ool00`00Oomoo`2?Ool001Uoo`80091oo`006Goo00<007ooOol0Sgoo000IOol00`00Oomoo`2?Ool0
01Uoo`80091oo`006Goo00<007ooOol0Sgoo000IOol2002@Ool000Qoo`8000Aoo`8000=oo`<000=o
o`03001oogoo08Uoo`8000Aoo`001goo00@007ooOol000=oo`8000Ioo`04001oogoo002:Ool40003
Ool000Moo`04001oogoo000;Ool00`00Oomoo`020029Ool40003Ool000Moo`04001oogoo0008Ool3
0003Ool00`00Oomoo`29Ool20004Ool000Moo`04001oogoo0008Ool00`00Oomoo`03Ool2002@Ool0
00Qoo`8000Uoo`@0009oo`03001oogoo08moo`006Goo00<007ooOol0Sgoo000IOol2002@Ool001Uo
o`03001oogoo08moo`006Goo0P00T7oo000IOol00`00Oomoo`2?Ool001Uoo`03001oogoo08moo`00
4goo0`000goo0P00T7oo000DOol00`00Oomoo`02Ool00`00Oomoo`1]Ool2000POol001Aoo`03001o
ogoo009oo`8006eoo`@001moo`0057oo00<007ooOol00Woo00<007ooOol0K7oo10007goo000DOol0
0`00Oomoo`02Ool00`00Oomoo`1]Ool2000POol001=oo`8000Aoo`80091oo`006Goo00<007ooOol0
Sgoo000IOol2002@Ool001Uoo`03001oogoo08moo`006Goo0P00T7oo000IOol00`00Oomoo`2?Ool0
01Uoo`03001oogoo08moo`001goo0`0017oo0P000goo0`000goo0P00T7oo0008Ool00`00Oomoo`03
Ool20006Ool01000Oomoo`00TGoo0008Ool00`00Oomoo`0;Ool00`00Oomoo`02002@Ool000Qoo`03
001oogoo00Qoo`<000=oo`03001oogoo08moo`0027oo00<007ooOol027oo00<007ooOol00goo00<0
07ooOol0Sgoo0007Ool2000:Ool40002Ool2002@Ool001Uoo`03001oogoo08moo`006Goo0P00T7oo
000IOol00`00Oomoo`2?Ool001Uoo`03001oogoo08moo`006Goo0P00T7oo000IOol00`00Oomoo`2?
Ool001Uoo`80091oo`004goo10000Woo00<007ooOol0Sgoo000DOol00`00Oomoo`02Ool00`00Oomo
o`0eOol2001HOol001Eoo`04001oogooOol2000eOol4001GOol001Ioo`04001oogoo000fOol4001G
Ool001=oo`04001oogoo0002Ool2000fOol2001HOol001Aoo`8000=oo`03001oogoo08moo`006Goo
00<007ooOol0Sgoo000IOol2002@Ool001Uoo`03001oogoo08moo`006Goo0P00T7oo000IOol00`00
Oomoo`2?Ool001Uoo`03001oogoo08moo`001goo10000goo0P000goo0`000goo0P00T7oo0008Ool0
0`00Oomoo`03Ool20006Ool01000Oomoo`00TGoo0009Ool00`00Oomoo`0:Ool00`00Oomoo`02002@
Ool000Yoo`03001oogoo00Ioo`<000=oo`03001oogoo08moo`001goo00@007ooOol000Qoo`03001o
ogoo00=oo`03001oogoo08moo`0027oo0P002Goo10000Woo0P00T7oo000IOol00`00Oomoo`2?Ool0
01Uoo`80091oo`006Goo00<007ooOol0Sgoo000IOol00`00Oomoo`2?Ool001Uoo`80091oo`006Goo
00<007ooOol0Sgoo000IOol2002@Ool001=oo`<000=oo`80091oo`005Woo00<007oo00000`00Sgoo
000FOol00`00Ool00003002?Ool001Aoo`8000=oo`80091oo`005Woo00@007ooOol0095oo`004goo
0`00UGoo002[Ool00:]oo`00Zgoo002[Ool00:]oo`00\
\>"],
  ImageRangeCache->{{{0, 170}, {104.5, 0}} -> {0.0810918, -0.853966, 
  0.0355526, 0.0402676}}]
}, Open  ]],

Cell[TextData[{
  "All of the graphics displayed can be altered to suit particular needs.  In \
our plot for ",
  StyleBox["ListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " we specified a particular size for the points to be plotted. Similarly, \
we can ask for those points to be joined by a continuous line."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["pointsb = ListPlot[data, PlotJoined -> True];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.190476 0.0988014 0.168173 [
[.21429 .0863 -3 -9 ]
[.21429 .0863 3 0 ]
[.40476 .0863 -3 -9 ]
[.40476 .0863 3 0 ]
[.59524 .0863 -3 -9 ]
[.59524 .0863 3 0 ]
[.78571 .0863 -3 -9 ]
[.78571 .0863 3 0 ]
[.97619 .0863 -3 -9 ]
[.97619 .0863 3 0 ]
[.01131 .01472 -24 -4.5 ]
[.01131 .01472 0 4.5 ]
[.01131 .18289 -18 -4.5 ]
[.01131 .18289 0 4.5 ]
[.01131 .26697 -6 -4.5 ]
[.01131 .26697 0 4.5 ]
[.01131 .35106 -18 -4.5 ]
[.01131 .35106 0 4.5 ]
[.01131 .43515 -6 -4.5 ]
[.01131 .43515 0 4.5 ]
[.01131 .51923 -18 -4.5 ]
[.01131 .51923 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .0988 m
.21429 .10505 L
s
[(2)] .21429 .0863 0 1 Mshowa
.40476 .0988 m
.40476 .10505 L
s
[(3)] .40476 .0863 0 1 Mshowa
.59524 .0988 m
.59524 .10505 L
s
[(4)] .59524 .0863 0 1 Mshowa
.78571 .0988 m
.78571 .10505 L
s
[(5)] .78571 .0863 0 1 Mshowa
.97619 .0988 m
.97619 .10505 L
s
[(6)] .97619 .0863 0 1 Mshowa
.125 Mabswid
.0619 .0988 m
.0619 .10255 L
s
.1 .0988 m
.1 .10255 L
s
.1381 .0988 m
.1381 .10255 L
s
.17619 .0988 m
.17619 .10255 L
s
.25238 .0988 m
.25238 .10255 L
s
.29048 .0988 m
.29048 .10255 L
s
.32857 .0988 m
.32857 .10255 L
s
.36667 .0988 m
.36667 .10255 L
s
.44286 .0988 m
.44286 .10255 L
s
.48095 .0988 m
.48095 .10255 L
s
.51905 .0988 m
.51905 .10255 L
s
.55714 .0988 m
.55714 .10255 L
s
.63333 .0988 m
.63333 .10255 L
s
.67143 .0988 m
.67143 .10255 L
s
.70952 .0988 m
.70952 .10255 L
s
.74762 .0988 m
.74762 .10255 L
s
.82381 .0988 m
.82381 .10255 L
s
.8619 .0988 m
.8619 .10255 L
s
.9 .0988 m
.9 .10255 L
s
.9381 .0988 m
.9381 .10255 L
s
.25 Mabswid
0 .0988 m
1 .0988 L
s
.02381 .01472 m
.03006 .01472 L
s
[(-0.5)] .01131 .01472 1 0 Mshowa
.02381 .18289 m
.03006 .18289 L
s
[(0.5)] .01131 .18289 1 0 Mshowa
.02381 .26697 m
.03006 .26697 L
s
[(1)] .01131 .26697 1 0 Mshowa
.02381 .35106 m
.03006 .35106 L
s
[(1.5)] .01131 .35106 1 0 Mshowa
.02381 .43515 m
.03006 .43515 L
s
[(2)] .01131 .43515 1 0 Mshowa
.02381 .51923 m
.03006 .51923 L
s
[(2.5)] .01131 .51923 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03153 m
.02756 .03153 L
s
.02381 .04835 m
.02756 .04835 L
s
.02381 .06517 m
.02756 .06517 L
s
.02381 .08198 m
.02756 .08198 L
s
.02381 .11562 m
.02756 .11562 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .14925 m
.02756 .14925 L
s
.02381 .16607 m
.02756 .16607 L
s
.02381 .1997 m
.02756 .1997 L
s
.02381 .21652 m
.02756 .21652 L
s
.02381 .23334 m
.02756 .23334 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .28379 m
.02756 .28379 L
s
.02381 .30061 m
.02756 .30061 L
s
.02381 .31743 m
.02756 .31743 L
s
.02381 .33424 m
.02756 .33424 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .38469 m
.02756 .38469 L
s
.02381 .40151 m
.02756 .40151 L
s
.02381 .41833 m
.02756 .41833 L
s
.02381 .45196 m
.02756 .45196 L
s
.02381 .46878 m
.02756 .46878 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .50242 m
.02756 .50242 L
s
.02381 .53605 m
.02756 .53605 L
s
.02381 .55287 m
.02756 .55287 L
s
.02381 .56968 m
.02756 .56968 L
s
.02381 .5865 m
.02756 .5865 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.21429 .01472 L
.40476 .43515 L
.59524 .0988 L
.78571 .26697 L
.97619 .18289 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{171, 105.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool00:]o
o`00Zgoo002[Ool00:]oo`00Zgoo0008Ool20004Ool20003Ool30003Ool00`00Oomoo`2?Ool000Mo
o`04001oogoo0003Ool20006Ool01000Oomoo`00TGoo00000goo0000000300001Goo001oogoo0000
2goo00<007ooOol00P006Woo00<007ooOol0Lgoo0007Ool01000Oomoo`0027oo0`000goo00<007oo
Ool06Goo00<007ooOol0Lgoo0007Ool01000Oomoo`0027oo00<007ooOol00goo0P006Woo0P00M7oo
0008Ool20009Ool40002Ool00`00Oomoo`0GOol4000IOol3000JOol3000HOol3000JOol20004Ool0
01Uoo`03001oogoo01Qoo`04001oogoo000KOol00`00Oomoo`0HOol00`00Oomoo`0JOol00`00Oomo
o`0FOol01000Oomoo`000goo000IOol2000IOol200000goo001oo`0JOol00`00Oomoo`0FOol4000K
Ool00`00Oomoo`0FOol01000Oomoo`000goo000IOol00`00Oomoo`0GOol01P00Oomoo`00Ool001Qo
o`8001Uoo`03001oo`0001Uoo`<001Uoo`<000Aoo`006Goo0P0067oo00H007ooOol007oo000JOol0
0`00Oomoo`0FOol00`00Ool0000IOol00`00Oomoo`0IOol00`00Oomoo`04Ool001Uoo`03001oogoo
01Moo`<000=oo`03001oogoo01Aoo`<001Yoo`8001Uoo`@001Uoo`<000=oo`006Goo00<007ooOol0
5Woo00<007ooOol017oo00<007ooOol0Kgoo000IOol2000GOol00`00Oomoo`04Ool00`00Oomoo`1_
Ool001Uoo`03001oogoo01Ioo`03001oogoo00Eoo`03001oogoo06ioo`005WooU@00000IOol00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool20003Ool00`00Oomoo`03
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Ool00002
Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02
Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02
Ool001Uoo`03001oogoo01Eoo`03001oogoo00Moo`03001oogoo02moo`05001oogooOol0000iOol0
01Uoo`8001Eoo`03001oogoo00Uoo`03001oogoo02ioo`03001oogoo009oo`03001oogoo03Ioo`00
6Goo00<007ooOol057oo00<007ooOol02Goo00<007ooOol0;Goo00<007ooOol017oo00<007ooOol0
=Goo000IOol2000EOol00`00Oomoo`0:Ool00`00Oomoo`0[Ool00`00Oomoo`06Ool00`00Oomoo`0d
Ool001Uoo`03001oogoo01=oo`03001oogoo00]oo`03001oogoo02]oo`03001oogoo00Moo`03001o
ogoo03=oo`006Goo00<007ooOol04goo00<007ooOol037oo00<007ooOol0:Goo00<007ooOol02Goo
00<007ooOol0<Woo000IOol2000DOol00`00Oomoo`0<Ool00`00Oomoo`0YOol00`00Oomoo`0:Ool2
000bOol001Uoo`03001oogoo019oo`03001oogoo00ioo`03001oogoo02Moo`03001oogoo00eoo`03
001oogoo02moo`006Goo0P004goo00<007ooOol03Woo00<007ooOol09goo00<007ooOol03Woo00<0
07ooOol0;Woo0008Ool20004Ool20003Ool30003Ool00`00Oomoo`0BOol00`00Oomoo`0?Ool00`00
Oomoo`0UOol00`00Oomoo`0@Ool00`00Oomoo`0]Ool000Moo`04001oogoo0003Ool20006Ool01000
Oomoo`004goo00<007ooOol047oo00<007ooOol09Goo00<007ooOol04Goo00<007ooOol0;7oo0007
Ool01000Oomoo`002goo00<007ooOol00P004Woo00<007ooOol047oo00<007ooOol097oo00<007oo
Ool04goo00<007ooOol09Goo0P0017oo0007Ool01000Oomoo`0027oo0`000goo00<007ooOol04Goo
00<007ooOol04Goo00<007ooOol08Woo00<007ooOol05Goo00<007ooOol08Goo0`001Woo0007Ool0
1000Oomoo`0027oo00<007ooOol00goo0P004Goo00<007ooOol04Woo00<007ooOol08Woo00<007oo
Ool05Woo00<007ooOol07Woo0P002Goo0008Ool20009Ool40002Ool00`00Oomoo`0@Ool00`00Oomo
o`0COol00`00Oomoo`0POol00`00Oomoo`0HOol00`00Oomoo`0KOol2000;Ool001Uoo`03001oogoo
011oo`03001oogoo01=oo`03001oogoo021oo`03001oogoo01Uoo`8001Qoo`<000eoo`006Goo0P00
4Goo00<007ooOol057oo00<007ooOol07Woo00<007ooOol077oo00<007ooOol04goo0P0047oo000I
Ool00`00Oomoo`0?Ool00`00Oomoo`0EOol00`00Oomoo`0NOol00`00Oomoo`0MOol00`00Oomoo`0@
Ool2000BOol001Uoo`80011oo`03001oogoo01Ioo`03001oogoo01aoo`03001oogoo01moo`03001o
ogoo00aoo`<001Aoo`006Goo00<007ooOol03goo00<007ooOol05Woo00<007ooOol077oo00<007oo
Ool087oo00<007ooOol02Goo0P005goo000IOol00`00Oomoo`0>Ool00`00Oomoo`0HOol00`00Oomo
o`0JOol00`00Oomoo`0ROol00`00Oomoo`06Ool2000IOol001=oo`<000=oo`8000moo`03001oogoo
01Qoo`03001oogoo01Uoo`03001oogoo02Aoo`03001oogoo009oo`<001]oo`0057oo00<007ooOol0
0Woo00<007ooOol03Woo00<007ooOol06Goo00<007ooOol067oo00<007ooOol09Goo00@007oo0000
01ioo`0057oo00<007ooOol00Woo0P003Woo00<007ooOol06Woo00<007ooOol05goo00<007ooOol0
9goo00<007ooOol07Woo000DOol00`00Oomoo`02Ool00`00Oomoo`0=Ool00`00Oomoo`0KOol00`00
Oomoo`0FOol00`00Oomoo`18Ool001Aoo`03001oogoo009oo`03001oogoo00eoo`03001oogoo01]o
o`03001oogoo01Eoo`03001oogoo04Uoo`004goo0P0017oo0P003Goo00<007ooOol077oo00<007oo
Ool05Goo00<007ooOol0BGoo000IOol00`00Oomoo`0<Ool00`00Oomoo`0MOol00`00Oomoo`0COol0
0`00Oomoo`1:Ool001Uoo`8000eoo`03001oogoo01eoo`03001oogoo019oo`03001oogoo04]oo`00
6Goo00<007ooOol02goo00<007ooOol07goo00<007ooOol04Goo00<007ooOol0Bgoo000IOol2000<
Ool00`00Oomoo`0OOol00`00Oomoo`0@Ool00`00Oomoo`1<Ool001Uoo`03001oogoo00]oo`03001o
ogoo021oo`03001oogoo00moo`03001oogoo04aoo`006Goo00<007ooOol02Woo00<007ooOol08Goo
00<007ooOol03Woo00<007ooOol0CGoo0007Ool30004Ool20003Ool30003Ool2000;Ool00`00Oomo
o`0ROol00`00Oomoo`0=Ool00`00Oomoo`1=Ool000Qoo`03001oogoo00=oo`8000Ioo`04001oogoo
000<Ool00`00Oomoo`0ROol00`00Oomoo`0<Ool00`00Oomoo`1>Ool000Qoo`03001oogoo00]oo`03
001oogoo008000Yoo`03001oogoo02Aoo`03001oogoo00]oo`03001oogoo04ioo`0027oo00<007oo
Ool027oo0`000goo00<007ooOol02Goo00<007ooOol097oo00<007ooOol02Woo00<007ooOol0Cgoo
0008Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`09Ool00`00Oomoo`0UOol00`00Oomo
o`08Ool00`00Oomoo`1@Ool000Moo`8000Yoo`@0009oo`8000Uoo`03001oogoo02Ioo`03001oogoo
00Qoo`03001oogoo051oo`006Goo00<007ooOol027oo00<007ooOol09goo00<007ooOol01Woo00<0
07ooOol0DGoo000IOol20009Ool00`00Oomoo`0WOol00`00Oomoo`06Ool00`00Oomoo`1AOol001Uo
o`03001oogoo00Moo`03001oogoo02Qoo`03001oogoo00Eoo`03001oogoo059oo`006Goo00<007oo
Ool01goo00<007ooOol0:Goo00<007ooOol017oo00<007ooOol0DWoo000IOol20008Ool00`00Oomo
o`0YOol00`00Oomoo`03Ool00`00Oomoo`1COol001Uoo`03001oogoo00Ioo`03001oogoo02]oo`03
001oogoo009oo`03001oogoo05=oo`006Goo0P001goo00<007ooOol0:goo00D007ooOomoo`0005Io
o`004goo10000Woo00<007ooOol01Woo00<007ooOol0;7oo00<007oo0000Egoo000DOol00`00Oomo
o`02Ool00`00Oomoo`06Ool00`00Oomoo`0/Ool00`00Ool0001GOol001Eoo`04001oogooOol20006
Ool00`00Oomoo`0^Ool00`00Oomoo`1FOol001Ioo`04001oogoo0007Ool00`00Oomoo`27Ool001=o
o`04001oogoo0002Ool20006Ool00`00Oomoo`27Ool001Aoo`8000=oo`03001oogoo00Aoo`03001o
ogoo08Qoo`006Goo00<007ooOol017oo00<007ooOol0R7oo000IOol20005Ool00`00Oomoo`28Ool0
01Uoo`03001oogoo00=oo`03001oogoo08Uoo`006Goo0P0017oo00<007ooOol0RGoo000IOol00`00
Oomoo`03Ool00`00Oomoo`29Ool001Uoo`03001oogoo009oo`03001oogoo08Yoo`001goo10000goo
0P000goo0`000goo0P000goo00<007ooOol0RWoo0008Ool00`00Oomoo`03Ool20006Ool01000Oomo
o`0017oo00<007ooOol0RWoo0009Ool00`00Oomoo`0:Ool00`00Oomoo`020002Ool00`00Oomoo`2;
Ool000Yoo`03001oogoo00Ioo`<000=oo`05001oogooOol0002=Ool000Moo`04001oogoo0008Ool0
0`00Oomoo`03Ool01@00Oomoogoo0000SGoo0008Ool20009Ool40002Ool200000goo001oo`2=Ool0
01Uoo`04001oogoo002>Ool001Uoo`800003Ool007oo08eoo`006Goo00<007oo0000Sgoo000IOol0
0`00Ool0002?Ool001Uoo`<008moo`006Goo0P00T7oo000IOol2002@Ool001=oo`<000=oo`80091o
o`005Woo00@007ooOol0095oo`005Woo00<007ooOol00P00T7oo000DOol20003Ool00`00Oomoo`2?
Ool001Ioo`04001oogoo002AOol001=oo`<009Eoo`00Zgoo002[Ool00:]oo`00Zgoo002[Ool00001
\
\>"],
  ImageRangeCache->{{{0, 170}, {104.5, 0}} -> {0.0810918, -0.853966, 
  0.0355526, 0.0402676}}]
}, Open  ]],

Cell[TextData[{
  "If you have executed the previous few commands on a computer you may have \
received a spelling warning. Here and elsewhere in this text we have \
suppressed messages from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " warning us about possible spelling mistakes suggested by a new variable \
name being very much like one previously introduced. The ",
  StyleBox["Show",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " command can be used to combine plots:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[points, pointsb]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.166667 0.190476 0.0988014 0.168173 [
[.21429 .0863 -3 -9 ]
[.21429 .0863 3 0 ]
[.40476 .0863 -3 -9 ]
[.40476 .0863 3 0 ]
[.59524 .0863 -3 -9 ]
[.59524 .0863 3 0 ]
[.78571 .0863 -3 -9 ]
[.78571 .0863 3 0 ]
[.97619 .0863 -3 -9 ]
[.97619 .0863 3 0 ]
[.01131 .01472 -24 -4.5 ]
[.01131 .01472 0 4.5 ]
[.01131 .18289 -18 -4.5 ]
[.01131 .18289 0 4.5 ]
[.01131 .26697 -6 -4.5 ]
[.01131 .26697 0 4.5 ]
[.01131 .35106 -18 -4.5 ]
[.01131 .35106 0 4.5 ]
[.01131 .43515 -6 -4.5 ]
[.01131 .43515 0 4.5 ]
[.01131 .51923 -18 -4.5 ]
[.01131 .51923 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .0988 m
.21429 .10505 L
s
[(2)] .21429 .0863 0 1 Mshowa
.40476 .0988 m
.40476 .10505 L
s
[(3)] .40476 .0863 0 1 Mshowa
.59524 .0988 m
.59524 .10505 L
s
[(4)] .59524 .0863 0 1 Mshowa
.78571 .0988 m
.78571 .10505 L
s
[(5)] .78571 .0863 0 1 Mshowa
.97619 .0988 m
.97619 .10505 L
s
[(6)] .97619 .0863 0 1 Mshowa
.125 Mabswid
.0619 .0988 m
.0619 .10255 L
s
.1 .0988 m
.1 .10255 L
s
.1381 .0988 m
.1381 .10255 L
s
.17619 .0988 m
.17619 .10255 L
s
.25238 .0988 m
.25238 .10255 L
s
.29048 .0988 m
.29048 .10255 L
s
.32857 .0988 m
.32857 .10255 L
s
.36667 .0988 m
.36667 .10255 L
s
.44286 .0988 m
.44286 .10255 L
s
.48095 .0988 m
.48095 .10255 L
s
.51905 .0988 m
.51905 .10255 L
s
.55714 .0988 m
.55714 .10255 L
s
.63333 .0988 m
.63333 .10255 L
s
.67143 .0988 m
.67143 .10255 L
s
.70952 .0988 m
.70952 .10255 L
s
.74762 .0988 m
.74762 .10255 L
s
.82381 .0988 m
.82381 .10255 L
s
.8619 .0988 m
.8619 .10255 L
s
.9 .0988 m
.9 .10255 L
s
.9381 .0988 m
.9381 .10255 L
s
.25 Mabswid
0 .0988 m
1 .0988 L
s
.02381 .01472 m
.03006 .01472 L
s
[(-0.5)] .01131 .01472 1 0 Mshowa
.02381 .18289 m
.03006 .18289 L
s
[(0.5)] .01131 .18289 1 0 Mshowa
.02381 .26697 m
.03006 .26697 L
s
[(1)] .01131 .26697 1 0 Mshowa
.02381 .35106 m
.03006 .35106 L
s
[(1.5)] .01131 .35106 1 0 Mshowa
.02381 .43515 m
.03006 .43515 L
s
[(2)] .01131 .43515 1 0 Mshowa
.02381 .51923 m
.03006 .51923 L
s
[(2.5)] .01131 .51923 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(3)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03153 m
.02756 .03153 L
s
.02381 .04835 m
.02756 .04835 L
s
.02381 .06517 m
.02756 .06517 L
s
.02381 .08198 m
.02756 .08198 L
s
.02381 .11562 m
.02756 .11562 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .14925 m
.02756 .14925 L
s
.02381 .16607 m
.02756 .16607 L
s
.02381 .1997 m
.02756 .1997 L
s
.02381 .21652 m
.02756 .21652 L
s
.02381 .23334 m
.02756 .23334 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .28379 m
.02756 .28379 L
s
.02381 .30061 m
.02756 .30061 L
s
.02381 .31743 m
.02756 .31743 L
s
.02381 .33424 m
.02756 .33424 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .38469 m
.02756 .38469 L
s
.02381 .40151 m
.02756 .40151 L
s
.02381 .41833 m
.02756 .41833 L
s
.02381 .45196 m
.02756 .45196 L
s
.02381 .46878 m
.02756 .46878 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .50242 m
.02756 .50242 L
s
.02381 .53605 m
.02756 .53605 L
s
.02381 .55287 m
.02756 .55287 L
s
.02381 .56968 m
.02756 .56968 L
s
.02381 .5865 m
.02756 .5865 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .60332 Mdot
.21429 .01472 Mdot
.40476 .43515 Mdot
.59524 .0988 Mdot
.78571 .26697 Mdot
.97619 .18289 Mdot
.5 Mabswid
.02381 .60332 m
.21429 .01472 L
.40476 .43515 L
.59524 .0988 L
.78571 .26697 L
.97619 .18289 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{171, 105.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002[0000JA000`40O003h00OZgoo002[Ool00:]o
o`00Zgoo002[Ool00:]oo`00Zgoo0008Ool20004Ool20003Ool30003Ool00`00Oomoo`0IOol2001d
Ool000Moo`04001oogoo0003Ool20006Ool01000Oomoo`006Woo1000Lgoo00000goo000000030000
1Goo001oogoo00002goo00<007ooOol00P006Goo1000Lgoo0007Ool01000Oomoo`0027oo0`000goo
00<007ooOol06Goo0P00M7oo0007Ool01000Oomoo`0027oo00<007ooOol00goo0P006Woo0P00M7oo
0008Ool20009Ool40002Ool00`00Oomoo`0GOol4000IOol3000JOol3000HOol3000JOol20004Ool0
01Uoo`03001oogoo01Qoo`04001oogoo000KOol00`00Oomoo`0HOol00`00Oomoo`0JOol00`00Oomo
o`0FOol01000Oomoo`000goo000IOol2000IOol200000goo001oo`0JOol00`00Oomoo`0FOol4000K
Ool00`00Oomoo`0FOol01000Oomoo`000goo000IOol00`00Oomoo`0GOol01P00Oomoo`00Ool001Qo
o`8001Uoo`03001oo`0001Uoo`<001Uoo`<000Aoo`006Goo0P0067oo00H007ooOol007oo000JOol0
0`00Oomoo`0FOol00`00Ool0000IOol00`00Oomoo`0IOol00`00Oomoo`04Ool001Uoo`03001oogoo
01Moo`<000=oo`03001oogoo01Aoo`<001Yoo`8001Uoo`@001Uoo`<000=oo`006Goo00<007ooOol0
5Woo00<007ooOol017oo00<007ooOol0Kgoo000IOol2000GOol00`00Oomoo`04Ool00`00Oomoo`1_
Ool001Uoo`03001oogoo01Ioo`03001oogoo00Eoo`03001oogoo031oo`8003aoo`005WooU@00000I
Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool20003Ool00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00
Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool01000Oomoogoo1@000Woo
00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo
00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo
000IOol00`00Oomoo`0EOol00`00Oomoo`07Ool00`00Oomoo`0_Ool20002Ool00`00Oomoo`0gOol0
01Uoo`8001Eoo`03001oogoo00Uoo`03001oogoo02ioo`03001oogoo009oo`03001oogoo03Ioo`00
6Goo00<007ooOol057oo00<007ooOol02Goo00<007ooOol0;Goo00<007ooOol017oo00<007ooOol0
=Goo000IOol2000EOol00`00Oomoo`0:Ool00`00Oomoo`0[Ool00`00Oomoo`06Ool00`00Oomoo`0d
Ool001Uoo`03001oogoo01=oo`03001oogoo00]oo`03001oogoo02]oo`03001oogoo00Moo`03001o
ogoo03=oo`006Goo00<007ooOol04goo00<007ooOol037oo00<007ooOol0:Goo00<007ooOol02Goo
00<007ooOol0<Woo000IOol2000DOol00`00Oomoo`0<Ool00`00Oomoo`0YOol00`00Oomoo`0:Ool2
000bOol001Uoo`03001oogoo019oo`03001oogoo00ioo`03001oogoo02Moo`03001oogoo00eoo`03
001oogoo02moo`006Goo0P004goo00<007ooOol03Woo00<007ooOol09goo00<007ooOol03Woo00<0
07ooOol0;Woo0008Ool20004Ool20003Ool30003Ool00`00Oomoo`0BOol00`00Oomoo`0?Ool00`00
Oomoo`0UOol00`00Oomoo`0@Ool00`00Oomoo`0WOol20004Ool000Moo`04001oogoo0003Ool20006
Ool01000Oomoo`004goo00<007ooOol047oo00<007ooOol09Goo00<007ooOol04Goo00<007ooOol0
9Goo10000goo0007Ool01000Oomoo`002goo00<007ooOol00P004Woo00<007ooOol047oo00<007oo
Ool097oo00<007ooOol04goo00<007ooOol097oo10000goo0007Ool01000Oomoo`0027oo0`000goo
00<007ooOol04Goo00<007ooOol04Goo00<007ooOol08Woo00<007ooOol05Goo00<007ooOol08Goo
1@0017oo0007Ool01000Oomoo`0027oo00<007ooOol00goo0P004Goo00<007ooOol04Woo00<007oo
Ool08Woo00<007ooOol05Woo00<007ooOol07Woo0P002Goo0008Ool20009Ool40002Ool00`00Oomo
o`0@Ool00`00Oomoo`0COol00`00Oomoo`0POol00`00Oomoo`0HOol00`00Oomoo`0KOol2000;Ool0
01Uoo`03001oogoo011oo`03001oogoo01=oo`03001oogoo021oo`03001oogoo01Uoo`8001Qoo`<0
00eoo`006Goo0P004Goo00<007ooOol057oo00<007ooOol07Woo00<007ooOol077oo00<007ooOol0
4goo0P0047oo000IOol00`00Oomoo`0?Ool00`00Oomoo`0EOol00`00Oomoo`0NOol00`00Oomoo`0M
Ool00`00Oomoo`0@Ool2000BOol001Uoo`80011oo`03001oogoo01Ioo`03001oogoo01aoo`03001o
ogoo01moo`03001oogoo00aoo`<001Aoo`006Goo00<007ooOol03goo00<007ooOol05Woo00<007oo
Ool077oo00<007ooOol087oo00<007ooOol02Goo0P005goo000IOol00`00Oomoo`0>Ool00`00Oomo
o`0HOol00`00Oomoo`0JOol00`00Oomoo`0ROol00`00Oomoo`06Ool2000IOol001=oo`<000=oo`80
00moo`03001oogoo01Qoo`03001oogoo01Uoo`03001oogoo02Aoo`03001oogoo009oo`<001]oo`00
57oo00<007ooOol00Woo00<007ooOol03Woo00<007ooOol06Goo00<007ooOol067oo00<007ooOol0
9Goo10007Woo000DOol00`00Oomoo`02Ool2000>Ool00`00Oomoo`0JOol00`00Oomoo`0GOol00`00
Oomoo`0UOol4000OOol001Aoo`03001oogoo009oo`03001oogoo00eoo`03001oogoo01]oo`03001o
ogoo01Ioo`03001oogoo02Eoo`@001moo`0057oo00<007ooOol00Woo00<007ooOol03Goo00<007oo
Ool06goo00<007ooOol05Goo00<007ooOol09goo0P0087oo000COol20004Ool2000=Ool00`00Oomo
o`0LOol00`00Oomoo`0EOol00`00Oomoo`19Ool001Uoo`03001oogoo00aoo`03001oogoo01eoo`03
001oogoo01=oo`03001oogoo04Yoo`006Goo0P003Goo00<007ooOol07Goo00<007ooOol04Woo00<0
07ooOol0Bgoo000IOol00`00Oomoo`0;Ool00`00Oomoo`0OOol00`00Oomoo`0AOol00`00Oomoo`1;
Ool001Uoo`8000aoo`03001oogoo01moo`03001oogoo011oo`03001oogoo04aoo`006Goo00<007oo
Ool02goo00<007ooOol087oo00<007ooOol03goo00<007ooOol0C7oo000IOol00`00Oomoo`0:Ool0
0`00Oomoo`0QOol00`00Oomoo`0>Ool00`00Oomoo`1=Ool000Moo`<000Aoo`8000=oo`<000=oo`80
00]oo`03001oogoo029oo`03001oogoo00eoo`03001oogoo04eoo`0027oo00<007ooOol00goo0P00
1Woo00@007ooOol000aoo`03001oogoo029oo`03001oogoo00aoo`03001oogoo04ioo`0027oo00<0
07ooOol02goo00<007ooOol00P002Woo00<007ooOol097oo00<007ooOol02goo00<007ooOol0CWoo
0008Ool00`00Oomoo`08Ool30003Ool00`00Oomoo`09Ool00`00Oomoo`0TOol00`00Oomoo`0:Ool0
0`00Oomoo`1?Ool000Qoo`03001oogoo00Qoo`03001oogoo00=oo`03001oogoo00Uoo`03001oogoo
02Eoo`03001oogoo00Qoo`03001oogoo051oo`001goo0P002Woo10000Woo0P002Goo00<007ooOol0
9Woo00<007ooOol027oo00<007ooOol0D7oo000IOol00`00Oomoo`08Ool00`00Oomoo`0WOol00`00
Oomoo`06Ool00`00Oomoo`1AOol001Uoo`8000Uoo`03001oogoo02Moo`03001oogoo00Ioo`03001o
ogoo055oo`006Goo00<007ooOol01goo00<007ooOol0:7oo00<007ooOol01Goo00<007ooOol0DWoo
000IOol00`00Oomoo`07Ool00`00Oomoo`0YOol00`00Oomoo`04Ool00`00Oomoo`1BOol001Uoo`80
00Qoo`03001oogoo02Uoo`03001oogoo00=oo`03001oogoo05=oo`006Goo00<007ooOol01Woo00<0
07ooOol0:goo00<007ooOol00Woo00<007ooOol0Dgoo000IOol20007Ool00`00Oomoo`0[Ool01@00
Oomoogoo0000EWoo000COol40002Ool00`00Oomoo`06Ool00`00Oomoo`0/Ool00`00Ool0001GOol0
01Aoo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo02aoo`<005Moo`005Goo00@007ooOomo
o`8000Ioo`03001oogoo02aoo`@005Moo`005Woo00@007ooOol000Moo`03001oogoo02aoo`@005Mo
o`004goo00@007ooOol0009oo`8000Ioo`03001oogoo02eoo`8005Qoo`0057oo0P000goo00<007oo
Ool017oo00<007ooOol0R7oo000IOol00`00Oomoo`04Ool00`00Oomoo`28Ool001Uoo`8000Eoo`03
001oogoo08Qoo`006Goo00<007ooOol00goo00<007ooOol0RGoo000IOol20004Ool00`00Oomoo`29
Ool001Uoo`03001oogoo00=oo`03001oogoo08Uoo`006Goo00<007ooOol00Woo00<007ooOol0RWoo
0007Ool40003Ool20003Ool30003Ool20003Ool00`00Oomoo`2:Ool000Qoo`03001oogoo00=oo`80
00Ioo`04001oogoo0004Ool00`00Oomoo`2:Ool000Uoo`03001oogoo00Yoo`03001oogoo0080009o
o`03001oogoo08]oo`002Woo00<007ooOol01Woo0`000goo00D007ooOomoo`0008eoo`001goo00@0
07ooOol000Qoo`03001oogoo00=oo`05001oogooOol0002=Ool000Qoo`8000Uoo`@0009oo`800003
Ool007oo08eoo`006Goo00@007ooOol008ioo`006Goo0P0000=oo`00Ool0SGoo000IOol00`00Ool0
002?Ool001Uoo`03001oo`0008moo`006Goo0`00Sgoo000IOol2002@Ool001Uoo`80091oo`004goo
0`000goo0P00T7oo000FOol00`00Ool00003002?Ool001Ioo`03001oo`0000<008moo`0057oo0P00
0goo0P00T7oo000FOol01000Oomoo`00TGoo000COol3002EOol00:]oo`00Zgoo002[Ool00:]oo`00
Zgoo0000\
\>"],
  ImageRangeCache->{{{0, 170}, {104.5, 0}} -> {0.0810918, -0.853966, 
  0.0355526, 0.0402676}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.3 Numerical Calculations", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Exact Arithmetic", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Degrees of Precision", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "With most computer languages you are restricted to certain levels of \
precision in the values of numbers.  This is not the case when you are using \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Try the following series of inputs. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["30!", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(265252859812191058636308480000000\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{N[30!], N[30!, 23]}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({2.65252859812191044`*^32, 2.6525285981219105863630848`23*^32}\)], 
  "Output"]
}, Open  ]],

Cell["\<\
The first result is exact, the second is approximate and has used \
the degree of precision given by default. In the last expression we have \
specified a precision of 23 digits.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exact and Approximate Numbers", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "When we ask for the square root of a number, for example, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will return the most exact version. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Sqrt[51]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\@51\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Note that if you are using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 2.2 or earlier, the output here will differ - you will still see the \
letters \"Sqrt\" in the output. Only when we ask explicitly for that to be \
given in a numerical form will ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " return a number which approximates this square root. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["N[Sqrt[51]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(7.14142842854285042`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "In evaluating the last expression ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " displays the answer to six significant figures. It will have been \
calculated, in  its numerical form, to a higher precision, which will be \
dependent on the machine you are using.  We can find the degree of precision \
by asking for the value of ",
  StyleBox["Precision",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " on that particular output line,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Precision[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(16\)], "Output"]
}, Open  ]],

Cell["\<\
So in this case, the result will have been calculated to 16 SF, but \
only displayed to six figures.  If that result were to be used in any further \
calculations, the full 16 figures would be used.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "How ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Decides on Exact or Approximate"
}], "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "This following example highlights the different routes ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will take when dealing with numerical expressions. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["(3/4)^100", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(515377520732011331036461129765621272702107522001\/\
1606938044258990275541962092341162602522202993782792835301376\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "In this expression the answer has been returned in the most exact form \
possible.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will keep to that form unless explicitly asked otherwise, by the use of ",
  
  StyleBox["N[ ]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", for example.  An alternative way of presenting the above calculation is"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["0.75^100", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(3.20720218538150358`*^-13\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This second expression has numbers given in their floating point or \
approximate form, the expression is then kept in this approximate form and \
the answer given in that form. ",
  StyleBox["Any",
    FontSlant->"Italic"],
  " number with a decimal point in it will be taken to be approximate.  Of \
course, we can always ask for that approximation to be more precise: "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["N[%,10]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    StyleBox["3.20720218538150358`*^-13",
      StyleBoxAutoDelete->True,
      PrintPrecision->10]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Practical Example with Exact and Approximate Numbers ", 
  "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
For another illustration of these two paths down which numbers may \
be sent, we can look at solutions to equations. Giving the equation in exact \
form:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Solve[{x^2 + 3 x + 11/10 == 0}, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{x \[Rule] 1\/10\ \((\(-15\) - \@115)\)}, {x \[Rule] 
          1\/10\ \((\(-15\) + \@115)\)}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["N[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{x \[Rule] \(-2.57238052947636086`\)}, {x \[Rule] \
\(-0.427619470523639222`\)}}\)], "Output"]
}, Open  ]],

Cell["Alternatively, in an approximate form:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Solve[{x^2 + 3 x + 1.1 == 0}, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{x \[Rule] \(-2.57238052947636086`\)}, {x \[Rule] \
\(-0.427619470523639222`\)}}\)], "Output"]
}, Open  ]],

Cell["\<\
Again, we chose the way in which numbers would be treated by the \
way we presented the equation - the presence of a decimal point implied the \
numerical treatment. This example raises some other issues that it is a good \
idea to explore right away.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Note on \"Equals\"", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
You will have noticed that in the last section we used a double \
equals sign to denote that an expression was equal to zero.  The use of the \
single equals sign will be examined more carefully shortly, where its role in \
assignment will be illustrated. For the moment it will be sufficient to \
highlight that when defining a symbolic equation the double equals sign \
should be used. This does not involve assigning any particular values to any \
variables.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["x == 0", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(x == 0\)], "Output"]
}, Open  ]],

Cell["\<\
If actually setting a quantity equal to some value for future \
substitution the single equals sign is used.  This assigns that value to the \
variable.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["x = 0", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "(Having set this value for ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " we must clear it for future calculations)"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["Clear[x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Extracting Results from the Solve Function", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It can be frustrating for new users that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " returns results from ",
  StyleBox["Solve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " as some replacement rules indicated by the arrow:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Solve[{x^2 + 3 x + 11/10 == 0}, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{x \[Rule] 1\/10\ \((\(-15\) - \@115)\)}, {x \[Rule] 
          1\/10\ \((\(-15\) + \@115)\)}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "What we really want are a list of values of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " given these rules. In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " the term \"given\" is invoked by the use of \"/.\", as shown here:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(x /. \ %\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1\/10\ \((\(-15\) - \@115)\), 1\/10\ \((\(-15\) + \@115)\)}\)], 
  "Output"]
}, Open  ]],

Cell["You can do the whole thing at once:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(x /. \ Solve[{x^2\  + \ 3\ x\  + \ 11/10\  == \ 0}, \ x]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1\/10\ \((\(-15\) - \@115)\), 1\/10\ \((\(-15\) + \@115)\)}\)], 
  "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Some Useful Numerical Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The usual arithmetic functions are as you would expect for ",
  Cell[BoxData[
      \(TraditionalForm\`+\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`-\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`*\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`/\)]],
  " etc. Note that multiplication is denoted by a star or a space. A cross or \
\"times\" symbol is not used. The square root of a number is calculated by \
the ",
  StyleBox["Sqrt",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Sqrt[78543.]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(280.255240807375471`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similar functions exist for exponentials, logarithms and many other \
numerical functions. You should note that sometimes long words are used, and \
there may be capitals within the name. For example, The function ",
  StyleBox["FactorInteger",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " produces a list of the prime factors of a number and the power to which \
they are raised - useful in cryptography!"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["FactorInteger[15931157760]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{2, 8}, {3, 8}, {5, 1}, {7, 1}, {271, 1}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This is telling us that ",
  Cell[BoxData[
      \(TraditionalForm\`15931157760 = 
        2\^8\[Cross]\ 3\^8\[Cross]\ 5\[Cross]\ 7\[Cross]\ 271\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["From Data to Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Simple Function Fitting", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Suppose that we have a data set and wish to represent it in terms of \
functions by a least-squares fit. This can be accomplished in a very simple \
way in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Let's invent some data:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["fitstuff = {6, 3, 5, 8, 7}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({6, 3, 5, 8, 7}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The elements of ",
  StyleBox["fitstuff",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " are interpreted as being the values of a function at equally spaced value \
points 1, 2, 3, 4, 5.  Now we can plot those points.  Calling the plot ",
  StyleBox["dataplot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will allow us to use it again later."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["dataplot = ListPlot[fitstuff, PlotStyle-> PointSize[0.03]];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.238095 -0.338447 0.117721 [
[.2619 .00222 -3 -9 ]
[.2619 .00222 3 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .13244 -6 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -6 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -6 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -6 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(2)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(3)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(4)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(5)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(4)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(5)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(6)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(7)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(8)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .03826 m
.02756 .03826 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .08535 m
.02756 .08535 L
s
.02381 .10889 m
.02756 .10889 L
s
.02381 .15598 m
.02756 .15598 L
s
.02381 .17952 m
.02756 .17952 L
s
.02381 .20307 m
.02756 .20307 L
s
.02381 .22661 m
.02756 .22661 L
s
.02381 .2737 m
.02756 .2737 L
s
.02381 .29724 m
.02756 .29724 L
s
.02381 .32079 m
.02756 .32079 L
s
.02381 .34433 m
.02756 .34433 L
s
.02381 .39142 m
.02756 .39142 L
s
.02381 .41497 m
.02756 .41497 L
s
.02381 .43851 m
.02756 .43851 L
s
.02381 .46205 m
.02756 .46205 L
s
.02381 .50914 m
.02756 .50914 L
s
.02381 .53269 m
.02756 .53269 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57977 m
.02756 .57977 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .36788 Mdot
.2619 .01472 Mdot
.5 .25016 Mdot
.7381 .60332 Mdot
.97619 .4856 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol002mo
o`@001moo`<0025oo`<001moo`<000]oo`00<7oo00<007ooOol08Woo00<007ooOol07goo00<007oo
Ool08Goo00<007ooOol027oo000aOol00`00Oomoo`0QOol00`00Oomoo`0MOol4000ROol00`00Oomo
o`08Ool0039oo`03001oogoo01ioo`80021oo`03001oo`00021oo`<000]oo`00;goo00@007ooOol0
029oo`03001oogoo01eoo`03001oo`00021oo`03001oogoo00]oo`00<7oo0P0087oo0`008Goo0P00
87oo10002Woo002VOol000moo`03001oogoo09Aoo`003goo00<007ooOol07goo0P00Lgoo000;OonC
0008Ool000moo`03001oogoo01ioo`@0021oo`03001oogoo021oo`03001oogoo021oo`03001oogoo
00Uoo`003goo00<007ooOol07goo0P00Lgoo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Ao
o`003goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<0
07ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo
000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo0009Ool30003
Ool00`00Oomoo`2DOol000Yoo`03001oogoo009oo`03001oogoo09Aoo`0027oo10000goo00<007oo
Ool0U7oo0008Ool00`00Ool00004Ool00`00Oomoo`2DOol000Qoo`03001oo`0000Aoo`03001oogoo
09Aoo`002Goo0P0017oo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Ao
o`003goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<0
07ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo
000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`0027oo0`0017oo00<007ooOol0U7oo000;
Ool01@00Oomoogoo0000@goo0P00DGoo000;Ool01@00Oomoogoo0000@Woo1000D7oo0008Ool30004
Ool00`00Oomoo`10Ool4001@Ool000Qoo`03001oogoo00Aoo`03001oogoo045oo`80055oo`0027oo
10000goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<0
07ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo
000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo000?Ool00`00
Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo0009Ool20003Ool2002FOol0
00Qoo`05001oogoo001oo`04002EOol000Qoo`05001oogoo001oo`04002EOol000Qoo`<000=oo`80
09Ioo`0027oo00<007ooOol017oo00<007ooOol0U7oo0009Ool30003Ool00`00Oomoo`2DOol000mo
o`03001oogoo09Aoo`003goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo
09Aoo`003goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo
00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0
U7oo0009Ool00`00Oomoo`03Ool00`00Oomoo`27Ool2000;Ool000Uoo`03001oogoo00=oo`03001o
ogoo08Ioo`@000Yoo`002Woo00<007ooOol00Woo00<007ooOol0QWoo10002Woo000:Ool00`00Oomo
o`02Ool00`00Oomoo`27Ool2000;Ool000]oo`05001oogooOol0002FOol000Qoo`@000=oo`03001o
ogoo09Aoo`003goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`00
3goo00<007ooOol0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007oo
Ool0U7oo000?Ool00`00Oomoo`2DOol000moo`03001oogoo09Aoo`003goo00<007ooOol0U7oo000?
Ool00`00Oomoo`2DOol000Uoo`8000Aoo`03001oogoo09Aoo`0027oo00@007ooOol000=oo`03001o
ogoo06Aoo`8002ioo`0027oo00@007ooOol000=oo`03001oogoo06=oo`@002eoo`002Goo0P0017oo
00<007ooOol0Hgoo1000;Goo0008Ool01000Oomoo`000goo00<007ooOol0I7oo0P00;Woo0009Ool2
002KOol00:Ioo`00\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {0.569087, 2.37123, 
  0.0287096, 0.0580661}}]
}, Open  ]],

Cell[TextData[{
  "Now type the following, exactly as written. Do not be too concerned about \
the syntax used to define the function ",
  StyleBox["psi",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", we will investigate this later - you are most concerned about the \
function ",
  StyleBox["Fit",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["psi[x_] = Fit[fitstuff, {1, x, x^2, x^3, x^4}, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(17.0000000000008633`\[InvisibleSpace]\) - 15.7500000000002859`\ x + 
      5.04166666666680196`\ x\^2 - 0.250000000000027133`\ x\^3 - 
      0.04166666666666341`\ x\^4\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["psiplot = Plot[psi[x], {x, 1, 5}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.238095 -0.312511 0.1091 [
[.2619 .00229 -3 -9 ]
[.2619 .00229 3 0 ]
[.5 .00229 -3 -9 ]
[.5 .00229 3 0 ]
[.7381 .00229 -3 -9 ]
[.7381 .00229 3 0 ]
[.97619 .00229 -3 -9 ]
[.97619 .00229 3 0 ]
[.01131 .12389 -6 -4.5 ]
[.01131 .12389 0 4.5 ]
[.01131 .23299 -6 -4.5 ]
[.01131 .23299 0 4.5 ]
[.01131 .34209 -6 -4.5 ]
[.01131 .34209 0 4.5 ]
[.01131 .45119 -6 -4.5 ]
[.01131 .45119 0 4.5 ]
[.01131 .56029 -6 -4.5 ]
[.01131 .56029 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01479 m
.2619 .02104 L
s
[(2)] .2619 .00229 0 1 Mshowa
.5 .01479 m
.5 .02104 L
s
[(3)] .5 .00229 0 1 Mshowa
.7381 .01479 m
.7381 .02104 L
s
[(4)] .7381 .00229 0 1 Mshowa
.97619 .01479 m
.97619 .02104 L
s
[(5)] .97619 .00229 0 1 Mshowa
.125 Mabswid
.07143 .01479 m
.07143 .01854 L
s
.11905 .01479 m
.11905 .01854 L
s
.16667 .01479 m
.16667 .01854 L
s
.21429 .01479 m
.21429 .01854 L
s
.30952 .01479 m
.30952 .01854 L
s
.35714 .01479 m
.35714 .01854 L
s
.40476 .01479 m
.40476 .01854 L
s
.45238 .01479 m
.45238 .01854 L
s
.54762 .01479 m
.54762 .01854 L
s
.59524 .01479 m
.59524 .01854 L
s
.64286 .01479 m
.64286 .01854 L
s
.69048 .01479 m
.69048 .01854 L
s
.78571 .01479 m
.78571 .01854 L
s
.83333 .01479 m
.83333 .01854 L
s
.88095 .01479 m
.88095 .01854 L
s
.92857 .01479 m
.92857 .01854 L
s
.25 Mabswid
0 .01479 m
1 .01479 L
s
.02381 .12389 m
.03006 .12389 L
s
[(4)] .01131 .12389 1 0 Mshowa
.02381 .23299 m
.03006 .23299 L
s
[(5)] .01131 .23299 1 0 Mshowa
.02381 .34209 m
.03006 .34209 L
s
[(6)] .01131 .34209 1 0 Mshowa
.02381 .45119 m
.03006 .45119 L
s
[(7)] .01131 .45119 1 0 Mshowa
.02381 .56029 m
.03006 .56029 L
s
[(8)] .01131 .56029 1 0 Mshowa
.125 Mabswid
.02381 .03661 m
.02756 .03661 L
s
.02381 .05843 m
.02756 .05843 L
s
.02381 .08025 m
.02756 .08025 L
s
.02381 .10207 m
.02756 .10207 L
s
.02381 .14571 m
.02756 .14571 L
s
.02381 .16753 m
.02756 .16753 L
s
.02381 .18935 m
.02756 .18935 L
s
.02381 .21117 m
.02756 .21117 L
s
.02381 .25481 m
.02756 .25481 L
s
.02381 .27663 m
.02756 .27663 L
s
.02381 .29845 m
.02756 .29845 L
s
.02381 .32027 m
.02756 .32027 L
s
.02381 .36391 m
.02756 .36391 L
s
.02381 .38573 m
.02756 .38573 L
s
.02381 .40755 m
.02756 .40755 L
s
.02381 .42937 m
.02756 .42937 L
s
.02381 .47301 m
.02756 .47301 L
s
.02381 .49483 m
.02756 .49483 L
s
.02381 .51665 m
.02756 .51665 L
s
.02381 .53847 m
.02756 .53847 L
s
.02381 .58211 m
.02756 .58211 L
s
.02381 .60393 m
.02756 .60393 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .34209 m
.06244 .23695 L
.10458 .14735 L
.12507 .11274 L
.14415 .08554 L
.16371 .06253 L
.18221 .04514 L
.192 .0376 L
.20262 .03068 L
.21222 .02554 L
.2212 .02165 L
.23064 .01852 L
.23565 .01724 L
.24104 .01616 L
.24373 .01574 L
.24667 .01536 L
.24944 .01508 L
.25077 .01498 L
.25202 .01489 L
.25325 .01483 L
.25389 .0148 L
.25457 .01477 L
.25577 .01474 L
.25688 .01472 L
.25816 .01472 L
.25935 .01472 L
.26066 .01475 L
.2614 .01477 L
.26207 .0148 L
.26435 .01491 L
.26565 .015 L
.26687 .0151 L
.27198 .01567 L
.27639 .01636 L
.28115 .0173 L
.29198 .02023 L
.30191 .02384 L
.32084 .03305 L
.34154 .04641 L
.3821 .08156 L
.42114 .12495 L
.46264 .17912 L
.50262 .2369 L
.54109 .29552 L
.58201 .35872 L
.62141 .41801 L
.6593 .47125 L
.69964 .52136 L
.73846 .56061 L
.75964 .57738 L
Mistroke
.76937 .58384 L
.77974 .58979 L
.79015 .59474 L
.79958 .5983 L
.80414 .59969 L
.80906 .60095 L
.81337 .60184 L
.81798 .60256 L
.82057 .60286 L
.82292 .60307 L
.82405 .60315 L
.82528 .60322 L
.82644 .60327 L
.82751 .6033 L
.82876 .60332 L
.82993 .60332 L
.83121 .6033 L
.83192 .60328 L
.83258 .60326 L
.83384 .60321 L
.8352 .60312 L
.83796 .60289 L
.84073 .60256 L
.84325 .60218 L
.84889 .60104 L
.85411 .59963 L
.85904 .59797 L
.86879 .59374 L
.87909 .58787 L
.89756 .57354 L
.91717 .5527 L
.9355 .52765 L
.97437 .45525 L
.97619 .45119 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol002io
o`@0021oo`<0029oo`<0021oo`<000Uoo`00;goo00<007ooOol08goo00<007ooOol087oo00<007oo
Ool08Woo00<007ooOol01Woo000`Ool00`00Oomoo`0ROol00`00Oomoo`0NOol4000SOol00`00Oomo
o`06Ool0035oo`03001oogoo01moo`80025oo`03001oo`00025oo`<000Uoo`00;Woo00@007ooOol0
02=oo`03001oogoo01ioo`03001oo`00025oo`03001oogoo00Uoo`00;goo0P008Goo0`008Woo0P00
8Goo100027oo002VOol000eoo`03001oogoo09Ioo`003Goo00<007ooOol0UWoo0009OonG0006Ool0
00eoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`8000Eoo`03
001oogoo009oo`@000Ioo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo
00Aoo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo
00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo
00Moo`003Goo00<007ooOol05goo0`003Woo00<007ooOol0Jgoo000=Ool00`00Oomoo`0EOol2000B
Ool2001[Ool000eoo`03001oogoo01Aoo`03001oogoo01Aoo`03001oogoo06Qoo`003Goo00<007oo
Ool057oo00<007ooOol05Goo0P00J7oo000=Ool00`00Oomoo`0COol00`00Oomoo`0HOol00`00Oomo
o`1UOol000eoo`03001oogoo019oo`03001oogoo01Yoo`03001oogoo06Aoo`003Goo00<007ooOol0
4Woo00<007ooOol06goo00<007ooOol0Hgoo000=Ool00`00Oomoo`0AOol00`00Oomoo`0MOol00`00
Oomoo`1ROol000eoo`03001oogoo011oo`03001oogoo01moo`03001oogoo065oo`003Goo00<007oo
Ool03goo00<007ooOol08Goo00<007ooOol0H7oo000=Ool00`00Oomoo`0?Ool00`00Oomoo`0ROol0
0`00Oomoo`1OOol000eoo`03001oogoo00ioo`03001oogoo02Aoo`03001oogoo05ioo`001goo0`00
0goo00<007ooOol03Goo00<007ooOol09Woo00<007ooOol0GGoo0008Ool00`00Oomoo`02Ool00`00
Oomoo`0<Ool00`00Oomoo`0XOol00`00Oomoo`1LOol000Ioo`@000=oo`8000eoo`03001oogoo02Uo
o`03001oogoo05]oo`001Woo00<007oo000017oo00<007ooOol02goo00<007ooOol0:Woo00<007oo
Ool0Fgoo0006Ool00`00Ool00004Ool00`00Oomoo`0:Ool00`00Oomoo`0/Ool00`00Oomoo`1JOol0
00Moo`8000Aoo`03001oogoo00Yoo`03001oogoo02eoo`03001oogoo05Uoo`003Goo00<007ooOol0
2Goo00<007ooOol0;goo00<007ooOol0F7oo000=Ool00`00Oomoo`09Ool00`00Oomoo`0`Ool00`00
Oomoo`1GOol000eoo`03001oogoo00Qoo`03001oogoo035oo`03001oogoo05Moo`003Goo00<007oo
Ool027oo00<007ooOol0<Woo00<007ooOol0EWoo000=Ool00`00Oomoo`07Ool00`00Oomoo`0dOol0
0`00Oomoo`1EOol000eoo`03001oogoo00Moo`03001oogoo03Eoo`03001oogoo05Aoo`003Goo00<0
07ooOol01Woo00<007ooOol0=Woo00<007ooOol0E7oo000=Ool00`00Oomoo`05Ool00`00Oomoo`0h
Ool00`00Oomoo`1COol000eoo`03001oogoo00Eoo`03001oogoo03Uoo`03001oogoo059oo`003Goo
00<007ooOol017oo00<007ooOol0>goo00<007ooOol0DGoo000=Ool00`00Oomoo`04Ool00`00Oomo
o`0kOol00`00Oomoo`1AOol000Ioo`<000Aoo`03001oogoo00=oo`03001oogoo03eoo`03001oogoo
051oo`002Goo00D007ooOomoo`0000Eoo`03001oogoo03ioo`03001oogoo04moo`002Goo00@007oo
Oomoo`8000=oo`03001oogoo041oo`03001oogoo04ioo`001Woo0`0017oo00<007ooOol00Woo00<0
07ooOol0@7oo00<007ooOol0CWoo0006Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`11
Ool00`00Oomoo`1=Ool000Ioo`@000=oo`05001oogooOol00014Ool00`00Oomoo`1=Ool000eoo`05
001oogooOol00015Ool00`00Oomoo`1<Ool000eoo`05001oogooOol00016Ool00`00Oomoo`1;Ool0
00eoo`04001oogoo0017Ool00`00Oomoo`1;Ool000eoo`04001oogoo0018Ool00`00Oomoo`1:Ool0
00eoo`04001oogoo0018Ool00`00Oomoo`1:Ool000eoo`03001oo`0004Yoo`03001oogoo04Uoo`00
3Goo00<007oo0000BWoo00<007ooOol0BGoo000=Ool00`00Ool0001;Ool00`00Oomoo`18Ool000eo
o`8004eoo`03001oogoo04Moo`003Goo0P00CGoo00<007ooOol0Agoo0007Ool20004Ool2001>Ool0
0`00Oomoo`16Ool000Ioo`04001oogoo0003Ool00`00Oomoo`1=Ool00`00Oomoo`16Ool000Ioo`04
001oogoo0003Ool2001?Ool00`00Oomoo`15Ool000Ioo`<000Aoo`03001oogoo04moo`03001oogoo
04Aoo`001Woo00<007ooOol017oo00<007ooOol0Cgoo00<007ooOol0A7oo0007Ool30003Ool00`00
Oomoo`1@Ool00`00Oomoo`13Ool000eoo`03001oogoo051oo`03001oogoo04=oo`003Goo00<007oo
Ool0DGoo00<007ooOol0@Woo000=Ool00`00Oomoo`1BOol00`00Oomoo`11Ool000eoo`03001oogoo
05=oo`03001oogoo041oo`003Goo00<007ooOol0Dgoo00<007ooOol0@7oo000=Ool00`00Oomoo`1D
Ool00`00Oomoo`0oOol000eoo`03001oogoo05Eoo`03001oogoo03ioo`003Goo00<007ooOol0EWoo
00<007ooOol0?Goo000=Ool00`00Oomoo`1FOol00`00Oomoo`0mOol000eoo`03001oogoo05Moo`03
001oogoo03aoo`001goo00<007ooOol00goo00<007ooOol0F7oo00<007ooOol0>goo0007Ool00`00
Oomoo`03Ool00`00Oomoo`1IOol00`00Oomoo`0jOol000Qoo`03001oogoo009oo`8005Yoo`03001o
ogoo031oo`03001oogoo00Moo`0027oo00<007ooOol00Woo00<007ooOol0FWoo00<007ooOol0;Woo
00<007ooOol027oo0009Ool01@00Oomoogoo0000GGoo00<007ooOol0;Goo00<007ooOol027oo0006
Ool40003Ool00`00Oomoo`1LOol00`00Oomoo`0[Ool00`00Oomoo`09Ool000eoo`03001oogoo05ao
o`03001oogoo02]oo`03001oogoo00Uoo`003Goo00<007ooOol0GGoo00<007ooOol0:Goo00<007oo
Ool02Woo000=Ool00`00Oomoo`1NOol00`00Oomoo`0XOol00`00Oomoo`0:Ool000eoo`03001oogoo
05moo`03001oogoo02Ioo`03001oogoo00]oo`003Goo00<007ooOol0Ggoo00<007ooOol09Woo00<0
07ooOol02goo000=Ool00`00Oomoo`1POol00`00Oomoo`0UOol00`00Oomoo`0;Ool000eoo`03001o
ogoo065oo`03001oogoo02=oo`03001oogoo00aoo`003Goo00<007ooOol0HWoo00<007ooOol08Woo
00<007ooOol037oo000=Ool00`00Oomoo`1SOol00`00Oomoo`0POol00`00Oomoo`0=Ool000eoo`03
001oogoo06Aoo`03001oogoo01moo`03001oogoo00eoo`003Goo00<007ooOol0IGoo00<007ooOol0
7Goo00<007ooOol03Woo0007Ool20004Ool00`00Oomoo`1VOol00`00Oomoo`0KOol00`00Oomoo`0?
Ool000Ioo`04001oogoo0003Ool00`00Oomoo`1WOol00`00Oomoo`0IOol00`00Oomoo`0@Ool000Io
o`04001oogoo0003Ool2001YOol2000HOol00`00Oomoo`0AOol000Moo`8000Aoo`03001oogoo06Yo
o`03001oogoo01Aoo`03001oogoo019oo`001Woo00@007ooOol000=oo`03001oogoo06]oo`80019o
o`8001Eoo`001goo0P0017oo00<007ooOol0KGoo00<007ooOol03Woo00<007ooOol05Goo000=Ool0
0`00Oomoo`1^Ool2000=Ool00`00Oomoo`0FOol000eoo`03001oogoo071oo`03001oogoo00Moo`<0
01Uoo`003Goo00<007ooOol0LGoo2@0077oo000=Ool00`00Oomoo`2FOol000eoo`03001oogoo09Io
o`00\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {0.63171, 2.3359, 0.0279771, 
  0.0610561}}]
}, Open  ]],

Cell[TextData[{
  "Now we can show the original points of the data set as well as the fitting \
function, and see that the function passes through them. At the same time we \
can see how to override ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s conventions on the vertical plot range:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Show[psiplot, dataplot, PlotRange -> {0, 10}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.238095 0 0.0618034 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -3 -9 ]
[.7381 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .12361 -6 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -6 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -6 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -6 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(2)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(3)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(4)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(5)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(10)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .37082 m
.06244 .31126 L
.10458 .2605 L
.12507 .2409 L
.14415 .22549 L
.16371 .21246 L
.18221 .20261 L
.192 .19833 L
.20263 .19441 L
.21222 .1915 L
.2212 .1893 L
.23064 .18752 L
.23565 .1868 L
.24104 .18619 L
.24373 .18595 L
.24667 .18573 L
.24944 .18558 L
.25077 .18552 L
.25202 .18547 L
.25325 .18543 L
.25389 .18542 L
.25457 .1854 L
.25578 .18538 L
.25688 .18537 L
.25816 .18537 L
.25935 .18537 L
.26066 .18539 L
.2614 .1854 L
.26207 .18541 L
.26435 .18548 L
.26565 .18553 L
.26687 .18559 L
.27198 .18591 L
.27639 .1863 L
.28115 .18683 L
.29198 .18849 L
.30191 .19054 L
.32084 .19575 L
.34154 .20332 L
.3821 .22324 L
.42114 .24782 L
.46264 .2785 L
.50262 .31123 L
.54109 .34444 L
.58201 .38024 L
.62141 .41383 L
.6593 .44399 L
.69964 .47237 L
.73846 .49461 L
.75964 .50411 L
Mistroke
.76937 .50777 L
.77974 .51114 L
.79015 .51394 L
.79958 .51596 L
.80414 .51675 L
.80906 .51746 L
.81337 .51796 L
.81798 .51837 L
.82057 .51855 L
.82292 .51866 L
.82405 .51871 L
.82528 .51875 L
.82644 .51877 L
.82751 .51879 L
.82876 .5188 L
.82993 .5188 L
.83121 .51879 L
.83192 .51878 L
.83258 .51877 L
.83384 .51874 L
.8352 .51869 L
.83796 .51856 L
.84073 .51837 L
.84325 .51816 L
.84889 .51751 L
.85411 .51671 L
.85904 .51577 L
.86879 .51338 L
.87909 .51005 L
.89756 .50194 L
.91717 .49013 L
.9355 .47594 L
.97437 .43492 L
.97619 .43262 L
Mfstroke
.03 w
.02381 .37082 Mdot
.2619 .18541 Mdot
.5 .30902 Mdot
.7381 .49443 Mdot
.97619 .43262 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol003Ao
o`@001eoo`<001moo`<001ioo`<000]oo`00=Goo00<007ooOol087oo00<007ooOol07Goo00<007oo
Ool087oo00<007ooOol027oo000fOol00`00Oomoo`0OOol00`00Oomoo`0KOol4000QOol00`00Oomo
o`08Ool003Moo`03001oogoo01aoo`8001ioo`03001oo`0001moo`<000]oo`00=7oo00@007ooOol0
021oo`03001oogoo01]oo`03001oo`0001moo`03001oogoo00]oo`00=Goo0P007Woo0`007goo0P00
7goo10002Woo002VOol00:Ioo`00YWoo000AOon=0008Ool001Eoo`03001oogoo01ioo`03001oogoo
01ioo`03001oogoo01ioo`03001oogoo01moo`03001oogoo00Uoo`005Goo00<007ooOol0SWoo000E
Ool00`00Oomoo`2>Ool001Eoo`03001oogoo08ioo`005Goo00<007ooOol0SWoo000EOol00`00Oomo
o`2>Ool001Eoo`03001oogoo08ioo`005Goo00<007ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eo
o`03001oogoo08ioo`005Goo00<007ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eoo`03001oogoo
08ioo`005Goo00<007ooOol0SWoo000>Ool40003Ool00`00Oomoo`2>Ool000moo`03001oogoo00=o
o`03001oogoo08ioo`0047oo00<007ooOol00Woo0P00Sgoo000AOol01@00Oomoogoo0000T7oo000>
Ool01000Oomoo`000goo00<007ooOol0SWoo000?Ool20004Ool00`00Oomoo`2>Ool001Eoo`03001o
ogoo08ioo`005Goo00<007ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eoo`03001oogoo01eoo`80
06moo`005Goo00<007ooOol077oo1000KWoo000EOol00`00Oomoo`0HOol>001XOol001Eoo`03001o
ogoo01Aoo`@000Eoo`8000Moo`<006Eoo`005Goo00<007ooOol04Woo0P005Goo0P00Hgoo000EOol0
0`00Oomoo`0@Ool2000IOol2001QOol001Eoo`03001oogoo00moo`03001oogoo01]oo`8005moo`00
5Goo00<007ooOol03Goo0P0087oo0P00GGoo000?Ool30003Ool00`00Oomoo`0<Ool00`00Oomoo`0R
Ool2001KOol0011oo`03001oogoo009oo`03001oogoo00]oo`03001oogoo02Eoo`03001oogoo05Qo
o`003Woo10000goo0P002goo00<007ooOol09goo0P00F7oo000>Ool00`00Ool00004Ool00`00Oomo
o`09Ool00`00Oomoo`0ZOol00`00Oomoo`1EOol000ioo`03001oo`0000Aoo`03001oogoo00Qoo`03
001oogoo02aoo`03001oogoo05Aoo`003goo0P0017oo00<007ooOol027oo00<007ooOol0;Goo00<0
07ooOol0Dgoo000EOol00`00Oomoo`07Ool00`00Oomoo`0_Ool00`00Oomoo`1BOol001Eoo`03001o
ogoo00Ioo`03001oogoo035oo`80059oo`005Goo00<007ooOol01Goo00<007ooOol0=7oo00<007oo
Ool0Cgoo000EOol00`00Oomoo`04Ool00`00Oomoo`0fOol4001=Ool001Eoo`03001oogoo00=oo`03
001oogoo03Qoo`@004aoo`005Goo00<007ooOol00Woo00<007ooOol0>Goo1000C7oo000EOol00`00
Oomoo`02Ool00`00Oomoo`0jOol3001<Ool001Eoo`05001oogooOol00010Ool00`00Oomoo`19Ool0
01Eoo`05001oogooOol00011Ool00`00Oomoo`18Ool001Eoo`04001oogoo0013Ool00`00Oomoo`17
Ool001Eoo`03001oo`0004Eoo`8004Moo`005Goo00<007oo0000Agoo00<007ooOol0A7oo000?Ool2
0003Ool30019Ool00`00Oomoo`13Ool000ioo`05001oogoo001oo`04001:Ool00`00Oomoo`12Ool0
00ioo`05001oogoo001oo`04001;Ool00`00Oomoo`11Ool000ioo`<000=oo`8004eoo`03001oogoo
041oo`003Woo00<007ooOol017oo00<007ooOol0C7oo00<007ooOol0?goo000?Ool30003Ool00`00
Oomoo`1=Ool00`00Oomoo`0nOol001Eoo`03001oogoo04ioo`03001oogoo03eoo`005Goo00<007oo
Ool0Cgoo00<007ooOol0?7oo000EOol00`00Oomoo`1@Ool2000lOol001Eoo`03001oogoo059oo`03
001oogoo02aoo`8000]oo`005Goo00<007ooOol0Dgoo00<007ooOol0:Woo10002Woo000EOol00`00
Oomoo`1DOol00`00Oomoo`0YOol4000:Ool001Eoo`03001oogoo05Eoo`8002Uoo`<000]oo`005Goo
00<007ooOol0Egoo00<007ooOol09Goo00<007ooOol037oo000EOol00`00Oomoo`1HOol2000TOol0
0`00Oomoo`0=Ool001Eoo`03001oogoo05Yoo`03001oogoo021oo`03001oogoo00ioo`005Goo00<0
07ooOol0Fgoo0P007Woo0P004Goo000?Ool20004Ool00`00Oomoo`1MOol200000goo0000000HOol0
0`00Oomoo`0AOol000ioo`04001oogoo0003Ool00`00Oomoo`1OOol4000FOol00`00Oomoo`0BOol0
00ioo`04001oogoo0003Ool2001POol4000EOol00`00Oomoo`0COol000moo`8000Aoo`03001oogoo
061oo`800003Ool00000015oo`8001Ioo`003Woo00@007ooOol000=oo`03001oogoo06Eoo`@000Uo
o`@001Qoo`003goo0P0017oo00<007ooOol0JGoo2@0077oo000EOol00`00Oomoo`2>Ool001Eoo`03
001oogoo08ioo`005Goo00<007ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eoo`03001oogoo08io
o`005Goo00<007ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eoo`03001oogoo08ioo`005Goo00<0
07ooOol0SWoo000EOol00`00Oomoo`2>Ool001Eoo`03001oogoo08ioo`0027oo0`0017oo0P0017oo
00<007ooOol0SWoo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007ooOol0SWoo0009Ool0
0`00Oomoo`02Ool01000Oomoo`000goo0P00Sgoo0009Ool00`00Oomoo`02Ool01000Oomoo`00U7oo
0009Ool00`00Oomoo`02Ool01000Oomoo`00U7oo0008Ool20005Ool2002EOol00:Ioo`00\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {0.35856, -1.24646, 
  0.0301152, 0.116018}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fitting Functions to an Irregular Array", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "This approach can be applied to lists of pairs of ",
  Cell[BoxData[
      \(TraditionalForm\`\((x, \ y)\)\)]],
  " values also, and any set of functions can be used to construct the \
function, provided they appear with unknown coefficients in a linear \
combination. For example for the set of data ",
  StyleBox["tStuff",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", we can fit a trigonometric function to the data. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["tStuff={{0,2.8},{2,11.},{3,5.3},{5,-4.4},{8,11.5}};", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["mi[x_] = Fit[tStuff, {1, Sin[x], Cos[x]}, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\(3.47465274522383449`\[InvisibleSpace]\) - 
      0.685386602474810047`\ Cos[x] + 7.99653703476163091`\ Sin[x]\)], 
  "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["miPlot = Plot[mi[x],{x,0,8}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.181604 0.0366693 [
[.2619 .1691 -3 -9 ]
[.2619 .1691 3 0 ]
[.5 .1691 -3 -9 ]
[.5 .1691 3 0 ]
[.7381 .1691 -3 -9 ]
[.7381 .1691 3 0 ]
[.97619 .1691 -3 -9 ]
[.97619 .1691 3 0 ]
[.01131 .08993 -24 -4.5 ]
[.01131 .08993 0 4.5 ]
[.01131 .27328 -18 -4.5 ]
[.01131 .27328 0 4.5 ]
[.01131 .36495 -6 -4.5 ]
[.01131 .36495 0 4.5 ]
[.01131 .45662 -18 -4.5 ]
[.01131 .45662 0 4.5 ]
[.01131 .5483 -12 -4.5 ]
[.01131 .5483 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .1816 m
.2619 .18785 L
s
[(2)] .2619 .1691 0 1 Mshowa
.5 .1816 m
.5 .18785 L
s
[(4)] .5 .1691 0 1 Mshowa
.7381 .1816 m
.7381 .18785 L
s
[(6)] .7381 .1691 0 1 Mshowa
.97619 .1816 m
.97619 .18785 L
s
[(8)] .97619 .1691 0 1 Mshowa
.125 Mabswid
.08333 .1816 m
.08333 .18535 L
s
.14286 .1816 m
.14286 .18535 L
s
.20238 .1816 m
.20238 .18535 L
s
.32143 .1816 m
.32143 .18535 L
s
.38095 .1816 m
.38095 .18535 L
s
.44048 .1816 m
.44048 .18535 L
s
.55952 .1816 m
.55952 .18535 L
s
.61905 .1816 m
.61905 .18535 L
s
.67857 .1816 m
.67857 .18535 L
s
.79762 .1816 m
.79762 .18535 L
s
.85714 .1816 m
.85714 .18535 L
s
.91667 .1816 m
.91667 .18535 L
s
.25 Mabswid
0 .1816 m
1 .1816 L
s
.02381 .08993 m
.03006 .08993 L
s
[(-2.5)] .01131 .08993 1 0 Mshowa
.02381 .27328 m
.03006 .27328 L
s
[(2.5)] .01131 .27328 1 0 Mshowa
.02381 .36495 m
.03006 .36495 L
s
[(5)] .01131 .36495 1 0 Mshowa
.02381 .45662 m
.03006 .45662 L
s
[(7.5)] .01131 .45662 1 0 Mshowa
.02381 .5483 m
.03006 .5483 L
s
[(10)] .01131 .5483 1 0 Mshowa
.125 Mabswid
.02381 .01659 m
.02756 .01659 L
s
.02381 .03493 m
.02756 .03493 L
s
.02381 .05326 m
.02756 .05326 L
s
.02381 .0716 m
.02756 .0716 L
s
.02381 .10827 m
.02756 .10827 L
s
.02381 .1266 m
.02756 .1266 L
s
.02381 .14493 m
.02756 .14493 L
s
.02381 .16327 m
.02756 .16327 L
s
.02381 .19994 m
.02756 .19994 L
s
.02381 .21827 m
.02756 .21827 L
s
.02381 .23661 m
.02756 .23661 L
s
.02381 .25494 m
.02756 .25494 L
s
.02381 .29161 m
.02756 .29161 L
s
.02381 .30995 m
.02756 .30995 L
s
.02381 .32828 m
.02756 .32828 L
s
.02381 .34662 m
.02756 .34662 L
s
.02381 .38328 m
.02756 .38328 L
s
.02381 .40162 m
.02756 .40162 L
s
.02381 .41995 m
.02756 .41995 L
s
.02381 .43829 m
.02756 .43829 L
s
.02381 .47496 m
.02756 .47496 L
s
.02381 .49329 m
.02756 .49329 L
s
.02381 .51163 m
.02756 .51163 L
s
.02381 .52996 m
.02756 .52996 L
s
.02381 .56663 m
.02756 .56663 L
s
.02381 .58497 m
.02756 .58497 L
s
.02381 .6033 m
.02756 .6033 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .28388 m
.06244 .3787 L
.10458 .47348 L
.12507 .51284 L
.14415 .54412 L
.16254 .56856 L
.17277 .5795 L
.18221 .58784 L
.19242 .59489 L
.19781 .59776 L
.20351 .60015 L
.2059 .60096 L
.20846 .60169 L
.21068 .60222 L
.2131 .60267 L
.21456 .60289 L
.21592 .60305 L
.21722 .60317 L
.21795 .60322 L
.21861 .60326 L
.21986 .60331 L
.22056 .60332 L
.22121 .60332 L
.2224 .6033 L
.22365 .60325 L
.22487 .60316 L
.2262 .60304 L
.22861 .60272 L
.23096 .60229 L
.23313 .60179 L
.23806 .6003 L
.24329 .59817 L
.25201 .59339 L
.26163 .58634 L
.28097 .56674 L
.30183 .53803 L
.34297 .46188 L
.38259 .37132 L
.4207 .27765 L
.46126 .18167 L
.50031 .10299 L
.52022 .07087 L
.5307 .05659 L
.5418 .0436 L
.55171 .03395 L
.56261 .02553 L
.56738 .02259 L
.57252 .01994 L
.57701 .01806 L
.58179 .01652 L
Mistroke
.58455 .01585 L
.58704 .01537 L
.58825 .01519 L
.58956 .01502 L
.59079 .0149 L
.59192 .01481 L
.59256 .01478 L
.59324 .01475 L
.59448 .01472 L
.59524 .01472 L
.59595 .01472 L
.59659 .01474 L
.59729 .01477 L
.59797 .01481 L
.59872 .01486 L
.60024 .015 L
.60164 .01517 L
.60296 .01537 L
.60542 .01584 L
.60801 .01647 L
.61264 .01794 L
.61762 .02002 L
.62306 .02286 L
.63336 .02988 L
.64271 .03805 L
.66368 .06236 L
.6829 .09138 L
.70371 .12915 L
.74468 .2183 L
.78414 .31433 L
.82208 .40626 L
.86247 .49317 L
.88108 .52667 L
.90134 .55708 L
.91236 .57065 L
.92256 .58122 L
.93293 .58992 L
.94267 .59616 L
.95033 .59971 L
.95464 .60119 L
.9586 .6022 L
.96072 .60261 L
.96269 .60291 L
.9638 .60304 L
.96485 .60314 L
.96606 .60323 L
.96716 .60328 L
.96826 .60331 L
.96942 .60332 L
.9704 .6033 L
Mistroke
.97149 .60325 L
.97266 .60318 L
.97392 .60307 L
.97619 .60278 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol00:Io
o`00YWoo002VOol00:Ioo`00YWoo000IOol00`00Oomoo`2:Ool001Uoo`03001oogoo08Yoo`006Goo
0P00CGoo1P00>7oo000IOol00`00Oomoo`1:Ool30005Ool2000fOol001Uoo`03001oogoo04Yoo`03
001oogoo00Moo`8003Aoo`006Goo0P00BGoo0P0037oo00<007ooOol0<Goo000IOol00`00Oomoo`17
Ool00`00Oomoo`0<Ool00`00Oomoo`0aOol001Uoo`8004Qoo`03001oogoo00eoo`03001oogoo031o
o`006Goo00<007ooOol0AWoo00<007ooOol03goo00<007ooOol0;goo000IOol00`00Oomoo`15Ool0
0`00Oomoo`0AOol00`00Oomoo`0^Ool001Uoo`8004Eoo`03001oogoo019oo`03001oogoo02ioo`00
1goo10000goo0P000goo0`000goo00<007ooOol0A7oo00<007ooOol04goo00<007ooOol0;Goo0008
Ool00`00Oomoo`03Ool20006Ool01000Oomoo`00AGoo00<007ooOol057oo00<007ooOol0;Goo0000
0goo000000030003Ool00`00Oomoo`0:Ool00`00Oomoo`020013Ool00`00Oomoo`0FOol00`00Oomo
o`0/Ool000Yoo`03001oogoo00Ioo`<000=oo`03001oogoo045oo`03001oogoo01Moo`03001oogoo
02aoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`80049oo`03001oogoo01Qoo`03001oogoo
02]oo`0027oo0P002Goo10000Woo00<007ooOol0@7oo00<007ooOol06Woo00<007ooOol0:Woo000I
Ool00`00Oomoo`0MOol4000OOol3000JOol00`00Oomoo`02Ool2000POol20004Ool001Uoo`8001mo
o`03001oogoo01moo`8001aoo`04001oogoo0002Ool00`00Oomoo`0LOol01000Oomoo`000goo000I
Ool00`00Oomoo`0OOol00`00Oomoo`0MOol4000KOol01000Oomoo`000Woo00<007ooOol077oo00@0
07ooOol000=oo`006Goo0P008Goo00<007ooOol077oo00<007oo00007Goo00<007oo00000P0087oo
0P0017oo000IOol00`00Oomoo`0MOol01000Oomoo`007Goo0P0000=oo`00Ool077oo00<007oo0000
8Goo00@007ooOol000=oo`006Goo00<007ooOol07Woo0P007Woo00@007oo000001ioo`03001oo`00
008001moo`8000Aoo`006Goo0P00?Woo00<007ooOol087oo00<007ooOol09goo000IOol00`00Oomo
o`0mOol00`00Oomoo`0QOol00`00Oomoo`0VOol001Uoo`03001oogoo03eoo`03001oogoo025oo`03
001oogoo02Ioo`005WooT000000IOol00`00Oomoo`0lOol00`00Oomoo`0SOol00`00Oomoo`0UOol0
01Uoo`8003aoo`03001oogoo02Aoo`03001oogoo02Eoo`006Goo00<007ooOol0>goo00<007ooOol0
9Goo00<007ooOol097oo000IOol00`00Oomoo`0jOol00`00Oomoo`0VOol00`00Oomoo`0TOol001Uo
o`8003]oo`03001oogoo02Moo`03001oogoo02=oo`006Goo00<007ooOol0>Goo00<007ooOol0:7oo
00<007ooOol08goo000IOol2000jOol00`00Oomoo`0YOol00`00Oomoo`0ROol001Uoo`03001oogoo
03Uoo`03001oogoo02Uoo`03001oogoo029oo`006Goo00<007ooOol0>7oo00<007ooOol0:goo00<0
07ooOol08Goo000IOol2000iOol00`00Oomoo`0[Ool00`00Oomoo`0QOol000Moo`@000=oo`8000=o
o`<000=oo`03001oogoo03Moo`03001oogoo02aoo`03001oogoo025oo`0027oo00<007ooOol00goo
0P001Woo00@007ooOol003Uoo`03001oogoo02eoo`03001oogoo021oo`002Goo00<007ooOol02Woo
00<007ooOol00P00=goo00<007ooOol0;Woo00<007ooOol087oo000:Ool00`00Oomoo`06Ool30003
Ool00`00Oomoo`0fOol00`00Oomoo`0_Ool00`00Oomoo`0OOol000Moo`04001oogoo0008Ool00`00
Oomoo`03Ool2000fOol00`00Oomoo`0`Ool00`00Oomoo`0OOol000Qoo`8000Uoo`@0009oo`8003Io
o`03001oogoo035oo`03001oogoo01ioo`006Goo0P00=Goo00<007ooOol0<Woo00<007ooOol07Woo
000IOol3000dOol00`00Oomoo`0bOol00`00Oomoo`0NOol001Uoo`03001oo`0003Aoo`03001oogoo
03=oo`03001oogoo01eoo`006Goo00<007oo0000<goo00<007ooOol0=7oo00<007ooOol07Goo000I
Ool200000goo001oo`0aOol00`00Oomoo`0eOol00`00Oomoo`0LOol001Uoo`04001oogoo000aOol0
0`00Oomoo`0fOol00`00Oomoo`0LOol001Uoo`80009oo`03001oogoo02ioo`03001oogoo03Ioo`03
001oogoo01aoo`004goo0`000goo00D007ooOomoo`0002moo`03001oogoo03Qoo`03001oogoo01]o
o`005Woo00@007ooOol000Aoo`03001oogoo02aoo`03001oogoo03Qoo`03001oogoo01]oo`005Woo
00<007ooOol00P000goo00<007ooOol0;7oo00<007ooOol0>Goo00<007ooOol06Woo000COol30003
Ool00`00Oomoo`02Ool00`00Oomoo`0[Ool00`00Oomoo`0jOol00`00Oomoo`0JOol001=oo`03001o
ogoo00=oo`8000Aoo`03001oogoo02Yoo`03001oogoo03Yoo`03001oogoo01Yoo`004goo10000Woo
00<007ooOol00goo00<007ooOol0:Goo00<007ooOol0?7oo00<007ooOol06Goo000IOol00`00Oomo
o`04Ool00`00Oomoo`0XOol00`00Oomoo`0lOol00`00Oomoo`0IOol001Uoo`8000Eoo`03001oogoo
02Qoo`03001oogoo03aoo`03001oogoo01Uoo`006Goo00<007ooOol01Goo00<007ooOol09Woo00<0
07ooOol0?Woo00<007ooOol067oo000IOol00`00Oomoo`05Ool00`00Oomoo`0VOol00`00Oomoo`0n
Ool00`00Oomoo`0HOol001Uoo`8000Moo`03001oogoo02Aoo`03001oogoo041oo`03001oogoo01Mo
o`006Goo00<007ooOol01Woo00<007ooOol097oo00<007ooOol0@7oo00<007ooOol05goo000IOol2
0007Ool00`00Oomoo`0TOol00`00Oomoo`11Ool00`00Oomoo`0FOol000Qoo`03001oogoo00=oo`80
00=oo`<000=oo`03001oogoo00Moo`03001oogoo029oo`03001oogoo049oo`03001oogoo01Ioo`00
27oo00<007ooOol00goo0P001Woo00@007ooOol000Uoo`03001oogoo029oo`03001oogoo04=oo`03
001oogoo01Eoo`002Goo00<007ooOol02Woo00<007ooOol00P002Goo00<007ooOol08Goo00<007oo
Ool0@goo00<007ooOol05Goo0009Ool00`00Oomoo`07Ool30003Ool00`00Oomoo`08Ool00`00Oomo
o`0POol00`00Oomoo`15Ool00`00Oomoo`0DOol000Yoo`03001oogoo00Ioo`03001oogoo00=oo`80
00Yoo`03001oogoo01moo`03001oogoo04Eoo`03001oogoo01Aoo`001goo100027oo10000Woo00<0
07ooOol02Goo00<007ooOol07Woo00<007ooOol0Agoo00<007ooOol04goo000IOol00`00Oomoo`0:
Ool00`00Oomoo`0LOol00`00Oomoo`18Ool00`00Oomoo`0COol001Uoo`8000]oo`03001oogoo01ao
o`03001oogoo04Uoo`03001oogoo019oo`006Goo00<007ooOol02goo00<007ooOol06Woo00<007oo
Ool0BWoo00<007ooOol04Woo000IOol00`00Oomoo`0;Ool00`00Oomoo`0JOol00`00Oomoo`1;Ool0
0`00Oomoo`0AOol001Uoo`8000eoo`03001oogoo01Qoo`03001oogoo04eoo`03001oogoo011oo`00
6Goo00<007ooOol037oo00<007ooOol05goo00<007ooOol0CWoo00<007ooOol047oo000IOol2000>
Ool00`00Oomoo`0FOol00`00Oomoo`1?Ool00`00Oomoo`0?Ool000eoo`<000Aoo`8000=oo`03001o
ogoo00eoo`03001oogoo01Eoo`03001oogoo051oo`03001oogoo00moo`003Woo00<007ooOol00Woo
00@007ooOol0009oo`03001oogoo00ioo`03001oogoo01Aoo`03001oogoo055oo`03001oogoo00io
o`003Woo00<007ooOol00Woo00@007ooOol0009oo`8000moo`03001oogoo01=oo`03001oogoo059o
o`03001oogoo00ioo`003Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00moo`03001o
ogoo015oo`03001oogoo05Aoo`03001oogoo00eoo`003Woo00<007ooOol00Woo00@007ooOol0009o
o`03001oogoo011oo`03001oogoo011oo`03001oogoo05Eoo`03001oogoo00aoo`003Goo0P001Goo
0P000goo0P004Woo0P003Woo0P00FGoo00<007ooOol02goo000IOol00`00Oomoo`0COol00`00Oomo
o`0:Ool00`00Oomoo`1IOol00`00Oomoo`0;Ool001Uoo`8001Aoo`03001oogoo00Uoo`03001oogoo
05]oo`8000]oo`006Goo00<007ooOol057oo00<007ooOol01goo00<007ooOol0GWoo00<007ooOol0
27oo000IOol00`00Oomoo`0EOol30004Ool2001ROol30007Ool001Uoo`8001Uoo`@006Moo`<000Ao
o`006Goo00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool00:Ioo`00YWoo002VOol00:Ioo`00YWoo
002VOol00001\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {-1.51866, -6.22405, 
  0.0589011, 0.191224}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["tStuffPlot = ListPlot[tStuff,PlotStyle->PointSize[0.03]];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.177599 0.0370191 [
[.2619 .1651 -3 -9 ]
[.2619 .1651 3 0 ]
[.5 .1651 -3 -9 ]
[.5 .1651 3 0 ]
[.7381 .1651 -3 -9 ]
[.7381 .1651 3 0 ]
[.97619 .1651 -3 -9 ]
[.97619 .1651 3 0 ]
[.01131 .08505 -24 -4.5 ]
[.01131 .08505 0 4.5 ]
[.01131 .27015 -18 -4.5 ]
[.01131 .27015 0 4.5 ]
[.01131 .36269 -6 -4.5 ]
[.01131 .36269 0 4.5 ]
[.01131 .45524 -18 -4.5 ]
[.01131 .45524 0 4.5 ]
[.01131 .54779 -12 -4.5 ]
[.01131 .54779 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .1776 m
.2619 .18385 L
s
[(2)] .2619 .1651 0 1 Mshowa
.5 .1776 m
.5 .18385 L
s
[(4)] .5 .1651 0 1 Mshowa
.7381 .1776 m
.7381 .18385 L
s
[(6)] .7381 .1651 0 1 Mshowa
.97619 .1776 m
.97619 .18385 L
s
[(8)] .97619 .1651 0 1 Mshowa
.125 Mabswid
.08333 .1776 m
.08333 .18135 L
s
.14286 .1776 m
.14286 .18135 L
s
.20238 .1776 m
.20238 .18135 L
s
.32143 .1776 m
.32143 .18135 L
s
.38095 .1776 m
.38095 .18135 L
s
.44048 .1776 m
.44048 .18135 L
s
.55952 .1776 m
.55952 .18135 L
s
.61905 .1776 m
.61905 .18135 L
s
.67857 .1776 m
.67857 .18135 L
s
.79762 .1776 m
.79762 .18135 L
s
.85714 .1776 m
.85714 .18135 L
s
.91667 .1776 m
.91667 .18135 L
s
.25 Mabswid
0 .1776 m
1 .1776 L
s
.02381 .08505 m
.03006 .08505 L
s
[(-2.5)] .01131 .08505 1 0 Mshowa
.02381 .27015 m
.03006 .27015 L
s
[(2.5)] .01131 .27015 1 0 Mshowa
.02381 .36269 m
.03006 .36269 L
s
[(5)] .01131 .36269 1 0 Mshowa
.02381 .45524 m
.03006 .45524 L
s
[(7.5)] .01131 .45524 1 0 Mshowa
.02381 .54779 m
.03006 .54779 L
s
[(10)] .01131 .54779 1 0 Mshowa
.125 Mabswid
.02381 .10356 m
.02756 .10356 L
s
.02381 .12207 m
.02756 .12207 L
s
.02381 .14058 m
.02756 .14058 L
s
.02381 .15909 m
.02756 .15909 L
s
.02381 .19611 m
.02756 .19611 L
s
.02381 .21462 m
.02756 .21462 L
s
.02381 .23313 m
.02756 .23313 L
s
.02381 .25164 m
.02756 .25164 L
s
.02381 .28866 m
.02756 .28866 L
s
.02381 .30717 m
.02756 .30717 L
s
.02381 .32568 m
.02756 .32568 L
s
.02381 .34419 m
.02756 .34419 L
s
.02381 .3812 m
.02756 .3812 L
s
.02381 .39971 m
.02756 .39971 L
s
.02381 .41822 m
.02756 .41822 L
s
.02381 .43673 m
.02756 .43673 L
s
.02381 .47375 m
.02756 .47375 L
s
.02381 .49226 m
.02756 .49226 L
s
.02381 .51077 m
.02756 .51077 L
s
.02381 .52928 m
.02756 .52928 L
s
.02381 .06654 m
.02756 .06654 L
s
.02381 .04803 m
.02756 .04803 L
s
.02381 .02952 m
.02756 .02952 L
s
.02381 .01101 m
.02756 .01101 L
s
.02381 .5663 m
.02756 .5663 L
s
.02381 .58481 m
.02756 .58481 L
s
.02381 .60332 m
.02756 .60332 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .28125 Mdot
.2619 .58481 Mdot
.38095 .3738 Mdot
.61905 .01472 Mdot
.97619 .60332 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol00:Io
o`00YWoo002VOol00:Ioo`00YWoo000IOol00`00Oomoo`1BOol2000fOol001Uoo`80059oo`@003Eo
o`006Goo00<007ooOol0DGoo1000=Goo000IOol00`00Oomoo`1BOol2000fOol001Uoo`8008]oo`00
6Goo00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool001Uoo`8008]oo`006Goo00<007ooOol0RWoo
000IOol2002;Ool000Moo`@000=oo`8000=oo`<000=oo`03001oogoo08Yoo`0027oo00<007ooOol0
0goo0P001Woo00@007ooOol008aoo`0000=oo`0000000`000goo00<007ooOol02Woo00<007ooOol0
0P00Rgoo000:Ool00`00Oomoo`06Ool30003Ool00`00Oomoo`2:Ool000Moo`04001oogoo0008Ool0
0`00Oomoo`03Ool2002;Ool000Qoo`8000Uoo`@0009oo`03001oogoo08Yoo`006Goo00<007ooOol0
7Goo10007goo0`007goo0P0087oo0P0017oo000IOol2000OOol00`00Oomoo`0POol00`00Oomoo`0M
Ool01000Oomoo`007Woo00@007ooOol000=oo`006Goo00<007ooOol07goo00<007ooOol07Goo1000
7Woo00@007ooOol001ioo`04001oogoo0003Ool001Uoo`03001oogoo021oo`03001oogoo01aoo`03
001oo`0001moo`<0021oo`8000Aoo`006Goo0P007Woo00@007ooOol001ioo`03001oo`0001moo`03
001oogoo01moo`04001oogoo0003Ool001Uoo`03001oogoo01ioo`80021oo`80021oo`<001moo`80
00Aoo`006Goo0P00Rgoo000IOol00`00Oomoo`2:Ool001Uoo`03001oogoo08Yoo`005WooT000000I
Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06
Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06
Ool00`00Oomoo`05Ool00`00Oomoo`02Ool001Uoo`03001oogoo08Yoo`006Goo0P00Rgoo000IOol0
0`00Oomoo`2:Ool001Uoo`8008]oo`006Goo00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool001Uo
o`8008]oo`006Goo00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool000Moo`@000=oo`8000=oo`<0
00=oo`8008]oo`0027oo00<007ooOol00goo0P001Woo00@007ooOol008aoo`002Goo00<007ooOol0
2Woo00<007ooOol00P00Rgoo000:Ool00`00Oomoo`06Ool30002Ool4002:Ool000Moo`04001oogoo
0008Ool00`00Oomoo`02Ool4002:Ool000Qoo`8000Uoo`@0009oo`8008]oo`006Goo00<007ooOol0
RWoo000IOol00`00Oomoo`2:Ool001Uoo`8008]oo`006Goo00<007ooOol0RWoo000IOol2002;Ool0
01Uoo`03001oogoo08Yoo`006Goo00<007ooOol0RWoo000COol30003Ool2002;Ool001Ioo`04001o
ogoo002<Ool001Ioo`03001oogoo008008]oo`004goo0`000goo00<007ooOol0<7oo0P00F7oo000C
Ool00`00Oomoo`03Ool00`00Oomoo`0_Ool4001GOol001=oo`@0009oo`80031oo`@005Moo`006Goo
00<007ooOol0<7oo0P00F7oo000IOol00`00Oomoo`2:Ool001Uoo`8008]oo`006Goo00<007ooOol0
RWoo000IOol2002;Ool001Uoo`03001oogoo08Yoo`006Goo00<007ooOol0RWoo000IOol2002;Ool0
00Qoo`03001oogoo00=oo`8000=oo`<000=oo`03001oogoo08Yoo`0027oo00<007ooOol00goo0P00
1Woo00@007ooOol008aoo`002Goo00<007ooOol02Woo00<007ooOol00P00Rgoo0009Ool00`00Oomo
o`07Ool30003Ool00`00Oomoo`2:Ool000Yoo`03001oogoo00Ioo`03001oogoo00=oo`8008]oo`00
1goo100027oo10000Woo00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool001Uoo`8008]oo`006Goo
00<007ooOol0RWoo000IOol00`00Oomoo`2:Ool001Uoo`8008]oo`006Goo00<007ooOol0RWoo000I
Ool2002;Ool000eoo`<000Aoo`8000=oo`03001oogoo08Yoo`003Woo00<007ooOol00Woo00@007oo
Ool0009oo`03001oogoo08Yoo`003Woo00<007ooOol00Woo00@007ooOol0009oo`8008]oo`003Woo
00<007ooOol00Woo00@007ooOol0009oo`03001oogoo08Yoo`003Woo00<007ooOol00Woo00@007oo
Ool0009oo`8008]oo`003Goo0P001Goo0P000goo00<007ooOol0RWoo000IOol00`00Oomoo`0OOol2
001YOol001Uoo`8001moo`@006Qoo`006Goo00<007ooOol07Woo1000HWoo0P0017oo000IOol00`00
Oomoo`0OOol2001ROol40003Ool001Uoo`8008Aoo`@000=oo`006Goo00<007ooOol0Q7oo0P0017oo
000IOol00`00Oomoo`2:Ool00:Ioo`00YWoo002VOol00:Ioo`00YWoo002VOol00001\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {-1.51866, -6.05705, 
  0.0589011, 0.189417}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Show[miPlot,tStuffPlot];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0.181604 0.0366693 [
[.2619 .1691 -3 -9 ]
[.2619 .1691 3 0 ]
[.5 .1691 -3 -9 ]
[.5 .1691 3 0 ]
[.7381 .1691 -3 -9 ]
[.7381 .1691 3 0 ]
[.97619 .1691 -3 -9 ]
[.97619 .1691 3 0 ]
[.01131 .08993 -24 -4.5 ]
[.01131 .08993 0 4.5 ]
[.01131 .27328 -18 -4.5 ]
[.01131 .27328 0 4.5 ]
[.01131 .36495 -6 -4.5 ]
[.01131 .36495 0 4.5 ]
[.01131 .45662 -18 -4.5 ]
[.01131 .45662 0 4.5 ]
[.01131 .5483 -12 -4.5 ]
[.01131 .5483 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .1816 m
.2619 .18785 L
s
[(2)] .2619 .1691 0 1 Mshowa
.5 .1816 m
.5 .18785 L
s
[(4)] .5 .1691 0 1 Mshowa
.7381 .1816 m
.7381 .18785 L
s
[(6)] .7381 .1691 0 1 Mshowa
.97619 .1816 m
.97619 .18785 L
s
[(8)] .97619 .1691 0 1 Mshowa
.125 Mabswid
.08333 .1816 m
.08333 .18535 L
s
.14286 .1816 m
.14286 .18535 L
s
.20238 .1816 m
.20238 .18535 L
s
.32143 .1816 m
.32143 .18535 L
s
.38095 .1816 m
.38095 .18535 L
s
.44048 .1816 m
.44048 .18535 L
s
.55952 .1816 m
.55952 .18535 L
s
.61905 .1816 m
.61905 .18535 L
s
.67857 .1816 m
.67857 .18535 L
s
.79762 .1816 m
.79762 .18535 L
s
.85714 .1816 m
.85714 .18535 L
s
.91667 .1816 m
.91667 .18535 L
s
.25 Mabswid
0 .1816 m
1 .1816 L
s
.02381 .08993 m
.03006 .08993 L
s
[(-2.5)] .01131 .08993 1 0 Mshowa
.02381 .27328 m
.03006 .27328 L
s
[(2.5)] .01131 .27328 1 0 Mshowa
.02381 .36495 m
.03006 .36495 L
s
[(5)] .01131 .36495 1 0 Mshowa
.02381 .45662 m
.03006 .45662 L
s
[(7.5)] .01131 .45662 1 0 Mshowa
.02381 .5483 m
.03006 .5483 L
s
[(10)] .01131 .5483 1 0 Mshowa
.125 Mabswid
.02381 .01659 m
.02756 .01659 L
s
.02381 .03493 m
.02756 .03493 L
s
.02381 .05326 m
.02756 .05326 L
s
.02381 .0716 m
.02756 .0716 L
s
.02381 .10827 m
.02756 .10827 L
s
.02381 .1266 m
.02756 .1266 L
s
.02381 .14493 m
.02756 .14493 L
s
.02381 .16327 m
.02756 .16327 L
s
.02381 .19994 m
.02756 .19994 L
s
.02381 .21827 m
.02756 .21827 L
s
.02381 .23661 m
.02756 .23661 L
s
.02381 .25494 m
.02756 .25494 L
s
.02381 .29161 m
.02756 .29161 L
s
.02381 .30995 m
.02756 .30995 L
s
.02381 .32828 m
.02756 .32828 L
s
.02381 .34662 m
.02756 .34662 L
s
.02381 .38328 m
.02756 .38328 L
s
.02381 .40162 m
.02756 .40162 L
s
.02381 .41995 m
.02756 .41995 L
s
.02381 .43829 m
.02756 .43829 L
s
.02381 .47496 m
.02756 .47496 L
s
.02381 .49329 m
.02756 .49329 L
s
.02381 .51163 m
.02756 .51163 L
s
.02381 .52996 m
.02756 .52996 L
s
.02381 .56663 m
.02756 .56663 L
s
.02381 .58497 m
.02756 .58497 L
s
.02381 .6033 m
.02756 .6033 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .28388 m
.06244 .3787 L
.10458 .47348 L
.12507 .51284 L
.14415 .54412 L
.16254 .56856 L
.17277 .5795 L
.18221 .58784 L
.19242 .59489 L
.19781 .59776 L
.20351 .60015 L
.2059 .60096 L
.20846 .60169 L
.21068 .60222 L
.2131 .60267 L
.21456 .60289 L
.21592 .60305 L
.21722 .60317 L
.21795 .60322 L
.21861 .60326 L
.21986 .60331 L
.22056 .60332 L
.22121 .60332 L
.2224 .6033 L
.22365 .60325 L
.22487 .60316 L
.2262 .60304 L
.22861 .60272 L
.23096 .60229 L
.23313 .60179 L
.23806 .6003 L
.24329 .59817 L
.25201 .59339 L
.26163 .58634 L
.28097 .56674 L
.30183 .53803 L
.34297 .46188 L
.38259 .37132 L
.4207 .27765 L
.46126 .18167 L
.50031 .10299 L
.52022 .07087 L
.5307 .05659 L
.5418 .0436 L
.55171 .03395 L
.56261 .02553 L
.56738 .02259 L
.57252 .01994 L
.57701 .01806 L
.58179 .01652 L
Mistroke
.58455 .01585 L
.58704 .01537 L
.58825 .01519 L
.58956 .01502 L
.59079 .0149 L
.59192 .01481 L
.59256 .01478 L
.59324 .01475 L
.59448 .01472 L
.59524 .01472 L
.59595 .01472 L
.59659 .01474 L
.59729 .01477 L
.59797 .01481 L
.59872 .01486 L
.60024 .015 L
.60164 .01517 L
.60296 .01537 L
.60542 .01584 L
.60801 .01647 L
.61264 .01794 L
.61762 .02002 L
.62306 .02286 L
.63336 .02988 L
.64271 .03805 L
.66368 .06236 L
.6829 .09138 L
.70371 .12915 L
.74468 .2183 L
.78414 .31433 L
.82208 .40626 L
.86247 .49317 L
.88108 .52667 L
.90134 .55708 L
.91236 .57065 L
.92256 .58122 L
.93293 .58992 L
.94267 .59616 L
.95033 .59971 L
.95464 .60119 L
.9586 .6022 L
.96072 .60261 L
.96269 .60291 L
.9638 .60304 L
.96485 .60314 L
.96606 .60323 L
.96716 .60328 L
.96826 .60331 L
.96942 .60332 L
.9704 .6033 L
Mistroke
.97149 .60325 L
.97266 .60318 L
.97392 .60307 L
.97619 .60278 L
Mfstroke
.03 w
.02381 .28428 Mdot
.2619 .58497 Mdot
.38095 .37595 Mdot
.61905 .02026 Mdot
.97619 .6033 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{166, 102.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002V0000IQ000`40O003h00OYWoo002VOol00:Io
o`00YWoo002VOol00:Ioo`00YWoo000IOol00`00Oomoo`2:Ool001Uoo`03001oogoo059oo`8003Io
o`006Goo0P00CGoo2@00=Goo000IOol00`00Oomoo`1:Ool30004Ool4000eOol001Uoo`03001oogoo
04Yoo`03001oogoo00Eoo`@003Aoo`006Goo0P00BGoo0P0037oo00<007ooOol0<Goo000IOol00`00
Oomoo`17Ool00`00Oomoo`0<Ool00`00Oomoo`0aOol001Uoo`8004Qoo`03001oogoo00eoo`03001o
ogoo031oo`006Goo00<007ooOol0AWoo00<007ooOol03goo00<007ooOol0;goo000IOol00`00Oomo
o`15Ool00`00Oomoo`0AOol00`00Oomoo`0^Ool001Uoo`8004Eoo`03001oogoo019oo`03001oogoo
02ioo`001goo10000goo0P000goo0`000goo00<007ooOol0A7oo00<007ooOol04goo00<007ooOol0
;Goo0008Ool00`00Oomoo`03Ool20006Ool01000Oomoo`00AGoo00<007ooOol057oo00<007ooOol0
;Goo00000goo000000030003Ool00`00Oomoo`0:Ool00`00Oomoo`020013Ool00`00Oomoo`0FOol0
0`00Oomoo`0/Ool000Yoo`03001oogoo00Ioo`<000=oo`03001oogoo045oo`03001oogoo01Moo`03
001oogoo02aoo`001goo00@007ooOol000Qoo`03001oogoo00=oo`80049oo`03001oogoo01Qoo`03
001oogoo02]oo`0027oo0P002Goo10000Woo00<007ooOol0@7oo00<007ooOol06Woo00<007ooOol0
:Woo000IOol00`00Oomoo`0MOol4000OOol3000JOol00`00Oomoo`02Ool2000POol20004Ool001Uo
o`8001moo`03001oogoo01moo`8001aoo`04001oogoo0002Ool00`00Oomoo`0LOol01000Oomoo`00
0goo000IOol00`00Oomoo`0OOol00`00Oomoo`0MOol4000KOol01000Oomoo`000Woo00<007ooOol0
77oo00@007ooOol000=oo`006Goo0P008Goo00<007ooOol077oo00<007oo00007Goo00<007oo0000
0P0087oo0P0017oo000IOol00`00Oomoo`0MOol01000Oomoo`007Goo0P0000=oo`00Ool077oo00<0
07oo00008Goo00@007ooOol000=oo`006Goo00<007ooOol07Woo0P007Woo00@007oo000001ioo`03
001oo`00008001moo`8000Aoo`006Goo0P00?Woo00<007ooOol087oo00<007ooOol09goo000IOol0
0`00Oomoo`0mOol00`00Oomoo`0QOol00`00Oomoo`0VOol001Uoo`03001oogoo03eoo`03001oogoo
025oo`03001oogoo02Ioo`005WooT000000IOol00`00Oomoo`0lOol00`00Oomoo`0SOol00`00Oomo
o`0UOol001Uoo`8003aoo`03001oogoo02Aoo`03001oogoo02Eoo`006Goo00<007ooOol0>goo00<0
07ooOol09Goo00<007ooOol097oo000IOol00`00Oomoo`0jOol00`00Oomoo`0VOol00`00Oomoo`0T
Ool001Uoo`8003]oo`03001oogoo02Moo`03001oogoo02=oo`006Goo00<007ooOol0>Goo00<007oo
Ool0:7oo00<007ooOol08goo000IOol2000jOol00`00Oomoo`0YOol00`00Oomoo`0ROol001Uoo`03
001oogoo03Uoo`03001oogoo02Uoo`03001oogoo029oo`006Goo00<007ooOol0>7oo00<007ooOol0
:goo00<007ooOol08Goo000IOol2000iOol00`00Oomoo`0[Ool00`00Oomoo`0QOol000Moo`@000=o
o`8000=oo`<000=oo`03001oogoo03Moo`03001oogoo02aoo`03001oogoo025oo`0027oo00<007oo
Ool00goo0P001Woo00@007ooOol003Uoo`03001oogoo02eoo`03001oogoo021oo`002Goo00<007oo
Ool02Woo00<007ooOol00P00=goo00<007ooOol0;Woo00<007ooOol087oo000:Ool00`00Oomoo`06
Ool30002Ool4000fOol00`00Oomoo`0_Ool00`00Oomoo`0OOol000Moo`04001oogoo0008Ool00`00
Oomoo`02Ool4000eOol00`00Oomoo`0`Ool00`00Oomoo`0OOol000Qoo`8000Uoo`@0009oo`8003Io
o`03001oogoo035oo`03001oogoo01ioo`006Goo0P00=Goo00<007ooOol0<Woo00<007ooOol07Woo
000IOol3000dOol00`00Oomoo`0bOol00`00Oomoo`0NOol001Uoo`03001oo`0003Aoo`03001oogoo
03=oo`03001oogoo01eoo`006Goo00<007oo0000<goo00<007ooOol0=7oo00<007ooOol07Goo000I
Ool200000goo001oo`0aOol00`00Oomoo`0eOol00`00Oomoo`0LOol001Uoo`04001oogoo000aOol0
0`00Oomoo`0fOol00`00Oomoo`0LOol001Uoo`80009oo`03001oogoo02ioo`03001oogoo03Ioo`03
001oogoo01aoo`004goo0`000goo00D007ooOomoo`0002moo`03001oogoo03Qoo`03001oogoo01]o
o`005Woo00@007ooOol000Aoo`03001oogoo02aoo`03001oogoo03Qoo`03001oogoo01]oo`005Woo
00<007ooOol00P000goo00<007ooOol0:goo0P00>goo00<007ooOol06Woo000COol30003Ool00`00
Oomoo`02Ool00`00Oomoo`0ZOol4000jOol00`00Oomoo`0JOol001=oo`03001oogoo00=oo`8000Ao
o`03001oogoo02Uoo`@003Yoo`03001oogoo01Yoo`004goo10000Woo00<007ooOol00goo00<007oo
Ool0:Goo0`00?7oo00<007ooOol06Goo000IOol00`00Oomoo`04Ool00`00Oomoo`0XOol00`00Oomo
o`0lOol00`00Oomoo`0IOol001Uoo`8000Eoo`03001oogoo02Qoo`03001oogoo03aoo`03001oogoo
01Uoo`006Goo00<007ooOol01Goo00<007ooOol09Woo00<007ooOol0?Woo00<007ooOol067oo000I
Ool00`00Oomoo`05Ool00`00Oomoo`0VOol00`00Oomoo`0nOol00`00Oomoo`0HOol001Uoo`8000Mo
o`03001oogoo02Aoo`03001oogoo041oo`03001oogoo01Moo`006Goo00<007ooOol01Woo00<007oo
Ool097oo00<007ooOol0@7oo00<007ooOol05goo000IOol20007Ool00`00Oomoo`0TOol00`00Oomo
o`11Ool00`00Oomoo`0FOol000Qoo`03001oogoo00=oo`8000=oo`<000=oo`03001oogoo00Moo`03
001oogoo029oo`03001oogoo049oo`03001oogoo01Ioo`0027oo00<007ooOol00goo0P001Woo00@0
07ooOol000Uoo`03001oogoo029oo`03001oogoo04=oo`03001oogoo01Eoo`002Goo00<007ooOol0
2Woo00<007ooOol00P002Goo00<007ooOol08Goo00<007ooOol0@goo00<007ooOol05Goo0009Ool0
0`00Oomoo`07Ool30003Ool00`00Oomoo`08Ool00`00Oomoo`0POol00`00Oomoo`15Ool00`00Oomo
o`0DOol000Yoo`03001oogoo00Ioo`03001oogoo00=oo`8000Yoo`03001oogoo01moo`03001oogoo
04Eoo`03001oogoo01Aoo`001goo100027oo10000Woo00<007ooOol02Goo00<007ooOol07Woo00<0
07ooOol0Agoo00<007ooOol04goo000IOol00`00Oomoo`0:Ool00`00Oomoo`0LOol00`00Oomoo`18
Ool00`00Oomoo`0COol001Uoo`8000]oo`03001oogoo01aoo`03001oogoo04Uoo`03001oogoo019o
o`006Goo00<007ooOol02goo00<007ooOol06Woo00<007ooOol0BWoo00<007ooOol04Woo000IOol0
0`00Oomoo`0;Ool00`00Oomoo`0JOol00`00Oomoo`1;Ool00`00Oomoo`0AOol001Uoo`8000eoo`03
001oogoo01Qoo`03001oogoo04eoo`03001oogoo011oo`006Goo00<007ooOol037oo00<007ooOol0
5goo00<007ooOol0CWoo00<007ooOol047oo000IOol2000>Ool00`00Oomoo`0FOol00`00Oomoo`1?
Ool00`00Oomoo`0?Ool000eoo`<000Aoo`8000=oo`03001oogoo00eoo`03001oogoo01Eoo`03001o
ogoo051oo`03001oogoo00moo`003Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00io
o`03001oogoo01Aoo`03001oogoo055oo`03001oogoo00ioo`003Woo00<007ooOol00Woo00@007oo
Ool0009oo`8000moo`03001oogoo01=oo`03001oogoo059oo`03001oogoo00ioo`003Woo00<007oo
Ool00Woo00@007ooOol0009oo`03001oogoo00moo`03001oogoo015oo`03001oogoo05Aoo`03001o
ogoo00eoo`003Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo011oo`03001oogoo011o
o`03001oogoo05Eoo`03001oogoo00aoo`003Goo0P001Goo0P000goo0P004Woo0P003Woo0P00FGoo
00<007ooOol02goo000IOol00`00Oomoo`0COol00`00Oomoo`09Ool2001KOol00`00Oomoo`0;Ool0
01Uoo`8001Aoo`03001oogoo00Qoo`@005]oo`8000]oo`006Goo00<007ooOol057oo00<007ooOol0
1goo1000GGoo00<007ooOol00Woo0P0017oo000IOol00`00Oomoo`0EOol30004Ool200000goo0000
001OOol70003Ool001Uoo`8001Uoo`@006Moo`@000=oo`006Goo00<007ooOol0Q7oo0P0017oo000I
Ool00`00Oomoo`2:Ool00:Ioo`00YWoo002VOol00:Ioo`00YWoo002VOol00001\
\>"],
  ImageRangeCache->{{{0, 165}, {101.438, 0}} -> {-1.51866, -6.22405, 
  0.0589011, 0.191224}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other Numerical Operations", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Numerical Solution of Equations", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We have already seen how to solve a quadratic. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can cope with all polynomials up to degree 4 analytically. For degree 5 \
and above, usually a numerical approach is required. This involves the ",
  StyleBox["NSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["NSolve[5 x^5 - 15 x^3 + 10 x + 1 == 0, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{x \[Rule] \(-1.45681933381877648`\)}, {x \[Rule] \
\(-0.901735549576984318`\)}, {x \[Rule] \(-0.101566183851026359`\)}, {x \
\[Rule] 1.11145002653588087`}, {x \[Rule] 1.34867104071090637`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  StyleBox["NSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and its symbolic counterpart can be used to solve sets of equations \
also:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["NSolve[{y + 3 x == x^3, 4 y^2 - x == 4}, {x, y}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{y \[Rule] \(-1.17126791726758372`\), 
        x \[Rule] 
          1.48747413608137186`}, {y \[Rule] \(-1.04454502994749875`\), 
        x \[Rule] 
          0.364297278352082765`}, {y \[Rule] \(-0.734259851538123164`\), 
        x \[Rule] \(-1.84344988167691354`\)}, {y \[Rule] 
          0.777156655492660775`, 
        x \[Rule] \(-1.58411013129388216`\)}, {y \[Rule] 
          0.957685274011354081`, 
        x \[Rule] \(-0.331355663767191455`\)}, {y \[Rule] 
          1.21523086924918755`, x \[Rule] 1.90714426230453248`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "To obtain the results as a list of ",
  Cell[BoxData[
      \(TraditionalForm\`\((x, y)\)\)]],
  " pairs, you use /. for given, as before:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["{x, y} /. %", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{1.48747413608137186`, \(-1.17126791726758372`\)}, \
{0.364297278352082765`, \(-1.04454502994749875`\)}, \
{\(-1.84344988167691354`\), \(-0.734259851538123164`\)}, \
{\(-1.58411013129388216`\), 
        0.777156655492660775`}, {\(-0.331355663767191455`\), 
        0.957685274011354081`}, {1.90714426230453248`, 
        1.21523086924918755`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "For functions other than linear and polynomial systems, the ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function is better. You give it a starting point and it iterates from \
there to a root:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["FindRoot[Cos[x] == 0, {x, 1.5}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({x \[Rule] 1.57079632679434162`}\)], "Output"]
}, Open  ]],

Cell["\<\
The result is a numerical approximation to \[Pi]/2:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["N[Pi/2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1.57079632679489655`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "However, this function should be used with some care.  (The function ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " operates using Newton's method which, if you have experience with its \
use, you will know is most effective when supplied with an approximation very \
near to a root.)  For example if we supply a first approximation of 3.1, the \
root finally given is nowhere near 3.1."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["FindRoot[Cos[x] == 0, {x, 3.1}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({x \[Rule] \(-20.4203522483336374`\)}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The tangent to the curve ",
  Cell[BoxData[
      \(TraditionalForm\`y\  = \ cos(x)\)]],
  ", at ",
  Cell[BoxData[
      \(TraditionalForm\`\(x\  = \ 3.1, \)\)]],
  " is nearly horizontal, so that the first iteration takes us to a point far \
away from the starting value. The process then converges! In financial \
applications ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is particularly useful for solving non-linear equations, such as arise in \
calculating implied volatility, solving for an internal rate of return, or \
obtaining a critical price defined by a non-linear equation. We shall see its \
use many times."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.4 Lists", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "There are some recurrent patterns in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " of which you will need to be aware, no matter what the type of \
calculation you are doing. The use of list structures in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " occurs in many diverse areas. We have already used them without too much \
fuss - now is a good time to go into a little more detail."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["One-Dimensional Lists", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Lists are used when dealing with data, or tables of values.  A list \
is denoted by using curly brackets to enclose the elements of the list, which \
are separated from each other by commas. We have already used some lists in \
our examples.  An example of a simple list is\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["data = {0,1,4,9,16,25,36,49,64,81,100}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We can select individual elements of such a list, or index elements of the \
list using the number which shows their position in the list. This involves \
the fourth type of bracketing, that of double square brackets.  For example \
we can select the fourth element of the list ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["data[[4]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(9\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "There are built-in functions for re-arranging the elements of lists \
including functions such as ",
  StyleBox["Reverse",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", which re-orders the list in the opposite direction, and ",
  StyleBox["RotateRight",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", which takes the last element of the list and brings it to the front."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Reverse[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({100, 81, 64, 49, 36, 25, 16, 9, 4, 1, 0}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["RotateRight[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({100, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The functions ",
  StyleBox["Prepend",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Append",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " can be used to add elements to lists."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Prepend[data, 1001]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Append[data, 25000]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100, 25000}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Note that neither of these changes the meaning of ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(data\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The functions ",
  StyleBox["PrependTo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["AppendTo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " have the same effect as ",
  StyleBox["Prepend",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Append",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", but also update their argument:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["PrependTo[data, 1001]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(data\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We select particular parts using functions, ",
  StyleBox["First",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["Rest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and",
  StyleBox[" ",
    FontFamily->"Courier"],
  StyleBox["Last",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["First[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1001\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Rest[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Last[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(100\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Our lists may well be generated within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " by use of the ",
  StyleBox["Table",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " command (examined in more detail later), or they may be generated outside \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", imported and then manipulated, possibly to obtain  a graphic \
representation."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Arrays, Matrices and Lists of Lists", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Next up from a list is the concept of a list of lists. We use this \
idea in several ways. Here is an example of a list of lists:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["listb = {{1,2,1,2}, {4,3,4,3}, {7,8,9,6}}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{1, 2, 1, 2}, {4, 3, 4, 3}, {7, 8, 9, 6}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We have already seen how the function ",
  StyleBox["FactorInteger",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " supplies its answer in a list of lists form.  Also, lists of lists are \
how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " represents matrices, and lists of ",
  Cell[BoxData[
      \(TraditionalForm\`\((x, y)\)\)]],
  " pairs."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["MatrixForm[listb]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "2", "1", "2"},
            {"4", "3", "4", "3"},
            {"7", "8", "9", "6"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell["We can ask about the dimensions of this matrix.", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Dimensions[listb]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3, 4}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "A list of lists is the way we represent any two-dimensional array of \
numbers. It could be an array for plotting or a matrix, or a rank-2 tensor. \
Matrix multiplication is specified by a \"dot\" (.), to distinguish it from \
element by element multiplication of lists.  The simple square matrix ",
  StyleBox["SqMatrix",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will illustrate this.  "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["SqMatrix={{1,6},{4,2}}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{1, 6}, {4, 2}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SqMatrix.SqMatrix", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{25, 18}, {12, 28}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["SqMatrix SqMatrix", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{1, 36}, {16, 4}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Selecting out specific rows is done in a similar way to that in which we \
select individual elements from a list. The second row of ",
  StyleBox["SqMatrix",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is therefore denoted by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["SqMatrix[[2]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({4, 2}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similarly we can select an individual element in a matrix by specifying \
its position in the matrix by row and then column, just as when referring to \
an entry in the form ",
  Cell[BoxData[
      \(TraditionalForm\`A\_\(i\ j\)\)]],
  " :"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["SqMatrix[[1,2]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(6\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Various other standard matrix operations may be carried out using specific \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions. For example we can transpose a matrix,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["MatrixForm[Transpose[listb]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "4", "7"},
            {"2", "3", "8"},
            {"1", "4", "9"},
            {"2", "3", "6"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[TextData[{
  "or find the inverse of a square matrix. To demonstrate this we first add a \
row to ",
  StyleBox["listb",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to make a square matrix:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
invertible = Prepend[listb, {4, 2, 6, 1}];
inverse = Inverse[invertible];
MatrixForm[inverse]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {\(-1\), \(-\(5\/2\)\), "1", \(1\/2\)},
            {\(-1\), \(-\(11\/5\)\), \(-\(1\/5\)\), "1"},
            {"1", \(19\/10\), \(-\(3\/5\)\), \(-\(1\/2\)\)},
            {"1", "3", "0", \(-1\)}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(MatrixForm[inverse . invertible]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", GridBox[{
            {"1", "0", "0", "0"},
            {"0", "1", "0", "0"},
            {"0", "0", "1", "0"},
            {"0", "0", "0", "1"}
            }], ")"}],
      (MatrixForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Flatten and Partition", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "One of the manipulations which it may be necessary to perform on a list is \
the conversion from an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-dimensional list to a one-dimensional list and back again.  This is \
easily accomplished using ",
  StyleBox["Flatten",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Partition",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["listc = Flatten[listb]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1, 2, 1, 2, 4, 3, 4, 3, 7, 8, 9, 6}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Partition[listc, 4]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{1, 2, 1, 2}, {4, 3, 4, 3}, {7, 8, 9, 6}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Note that the second argument to ",
  StyleBox["Partition",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " represents the length of the sub-lists generated."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applying Functions to Lists", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are two different situations under which you may want to apply a \
function to a list.  Either the elements of the list are to be used as the \
arguments to a function, or a function is to be applied to each element of \
the list individually, the result being another list. These are two similar \
processes, but should not be confused. For functions which act on a sequence \
of arguments we use the ",
  StyleBox["Apply",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function to supply the argument set."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["data", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Apply[Plus,data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1386\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "When a function is to be applied to each element of the set we can use the \
function ",
  StyleBox["Map",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", or for certain numerical functions we can use the whole set as the \
argument."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sqrt[data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\@1001, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Map[Sqrt, data]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\@1001, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Incrementation and Nesting", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Incrementation", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "To perform a simple incremental function there are a few ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions which can be used."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Do[Print[{n, n^2, n^3}], {n, 1, 4}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1, 1, 1}\)], "Print"],

Cell[BoxData[
    \({2, 4, 8}\)], "Print"],

Cell[BoxData[
    \({3, 9, 27}\)], "Print"],

Cell[BoxData[
    \({4, 16, 64}\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "In this example the command ",
  StyleBox["Print[{n, n^2, n^3}]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  StyleBox[" ",
    FontFamily->"Courier"],
  "is executed for ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " taking values starting at 1 and being incremented by one each time until \
",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " equals 4. The function ",
  StyleBox["Table",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", used to produce lists, is of a similar form."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Table[i^2 +3 i,{i,1,15}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({4, 10, 18, 28, 40, 54, 70, 88, 108, 130, 154, 180, 208, 238, 270}\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "In this case the resulting list or table is made up of each of the results \
obtained when the quantity ",
  Cell[BoxData[
      \(TraditionalForm\`i\^2 + 3  i\)]],
  " is calculated for ",
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  " running from 1 to 15 in steps of 1."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Iterative Application of Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Iteration is at the heart of many modern areas of mathematics including \
dynamical systems, chaos and fractals. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has some simple tools for dealing with iterated systems, based on ",
  StyleBox["Nest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and related functions. ",
  StyleBox["Nest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " in particular is a handy way of simulating random walks. We shall give \
some examples shortly. Before giving some simpler examples, we make sure that \
we have removed any previous definitions of several handy symbols:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[x, y, f, g, h, z]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  StyleBox["Nest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function applies a given function (its first argument) to its second \
argument a certain number of times, specified by its last argument:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Nest[g, x, 3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(g[g[g[x]]]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "An interesting related function is ",
  StyleBox["NestList",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["NestList[Sin, x, 3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({x, Sin[x], Sin[Sin[x]], Sin[Sin[Sin[x]]]}\)], "Output"]
}, Open  ]],

Cell["\<\
There is also a neat pair of generalizations to functions of two \
variables:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Fold[h, x, {a, b, c}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(h[h[h[x, a], b], c]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["FoldList[h, x, {a, b, c}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({x, h[x, a], h[h[x, a], b], h[h[h[x, a], b], c]}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["An Example Using Nest", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For example, what does ",
  StyleBox["Sin[Sin[Sin ... (100 times)[x]...]]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " look like? Ask ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Plot[Evaluate[Nest[Sin, x, 100]], {x, -2 Pi, 2 Pi}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.0757881 0.309017 1.7346 [
[.04527 .29652 -6 -9 ]
[.04527 .29652 6 0 ]
[.19685 .29652 -6 -9 ]
[.19685 .29652 6 0 ]
[.34842 .29652 -6 -9 ]
[.34842 .29652 6 0 ]
[.65158 .29652 -3 -9 ]
[.65158 .29652 3 0 ]
[.80315 .29652 -3 -9 ]
[.80315 .29652 3 0 ]
[.95473 .29652 -3 -9 ]
[.95473 .29652 3 0 ]
[.4875 .04883 -30 -4.5 ]
[.4875 .04883 0 4.5 ]
[.4875 .13556 -24 -4.5 ]
[.4875 .13556 0 4.5 ]
[.4875 .22229 -30 -4.5 ]
[.4875 .22229 0 4.5 ]
[.4875 .39575 -24 -4.5 ]
[.4875 .39575 0 4.5 ]
[.4875 .48248 -18 -4.5 ]
[.4875 .48248 0 4.5 ]
[.4875 .56921 -24 -4.5 ]
[.4875 .56921 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.04527 .30902 m
.04527 .31527 L
s
[(-6)] .04527 .29652 0 1 Mshowa
.19685 .30902 m
.19685 .31527 L
s
[(-4)] .19685 .29652 0 1 Mshowa
.34842 .30902 m
.34842 .31527 L
s
[(-2)] .34842 .29652 0 1 Mshowa
.65158 .30902 m
.65158 .31527 L
s
[(2)] .65158 .29652 0 1 Mshowa
.80315 .30902 m
.80315 .31527 L
s
[(4)] .80315 .29652 0 1 Mshowa
.95473 .30902 m
.95473 .31527 L
s
[(6)] .95473 .29652 0 1 Mshowa
.125 Mabswid
.08317 .30902 m
.08317 .31277 L
s
.12106 .30902 m
.12106 .31277 L
s
.15895 .30902 m
.15895 .31277 L
s
.23474 .30902 m
.23474 .31277 L
s
.27264 .30902 m
.27264 .31277 L
s
.31053 .30902 m
.31053 .31277 L
s
.38632 .30902 m
.38632 .31277 L
s
.42421 .30902 m
.42421 .31277 L
s
.46211 .30902 m
.46211 .31277 L
s
.53789 .30902 m
.53789 .31277 L
s
.57579 .30902 m
.57579 .31277 L
s
.61368 .30902 m
.61368 .31277 L
s
.68947 .30902 m
.68947 .31277 L
s
.72736 .30902 m
.72736 .31277 L
s
.76526 .30902 m
.76526 .31277 L
s
.84105 .30902 m
.84105 .31277 L
s
.87894 .30902 m
.87894 .31277 L
s
.91683 .30902 m
.91683 .31277 L
s
.00738 .30902 m
.00738 .31277 L
s
.99262 .30902 m
.99262 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .04883 m
.50625 .04883 L
s
[(-0.15)] .4875 .04883 1 0 Mshowa
.5 .13556 m
.50625 .13556 L
s
[(-0.1)] .4875 .13556 1 0 Mshowa
.5 .22229 m
.50625 .22229 L
s
[(-0.05)] .4875 .22229 1 0 Mshowa
.5 .39575 m
.50625 .39575 L
s
[(0.05)] .4875 .39575 1 0 Mshowa
.5 .48248 m
.50625 .48248 L
s
[(0.1)] .4875 .48248 1 0 Mshowa
.5 .56921 m
.50625 .56921 L
s
[(0.15)] .4875 .56921 1 0 Mshowa
.125 Mabswid
.5 .06617 m
.50375 .06617 L
s
.5 .08352 m
.50375 .08352 L
s
.5 .10086 m
.50375 .10086 L
s
.5 .11821 m
.50375 .11821 L
s
.5 .1529 m
.50375 .1529 L
s
.5 .17025 m
.50375 .17025 L
s
.5 .18759 m
.50375 .18759 L
s
.5 .20494 m
.50375 .20494 L
s
.5 .23963 m
.50375 .23963 L
s
.5 .25698 m
.50375 .25698 L
s
.5 .27432 m
.50375 .27432 L
s
.5 .29167 m
.50375 .29167 L
s
.5 .32636 m
.50375 .32636 L
s
.5 .34371 m
.50375 .34371 L
s
.5 .36106 m
.50375 .36106 L
s
.5 .3784 m
.50375 .3784 L
s
.5 .41309 m
.50375 .41309 L
s
.5 .43044 m
.50375 .43044 L
s
.5 .44779 m
.50375 .44779 L
s
.5 .46513 m
.50375 .46513 L
s
.5 .49982 m
.50375 .49982 L
s
.5 .51717 m
.50375 .51717 L
s
.5 .53452 m
.50375 .53452 L
s
.5 .55186 m
.50375 .55186 L
s
.5 .03148 m
.50375 .03148 L
s
.5 .01413 m
.50375 .01413 L
s
.5 .58655 m
.50375 .58655 L
s
.5 .6039 m
.50375 .6039 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.02846 .40927 L
.03068 .44835 L
.03279 .47854 L
.03527 .50649 L
.03754 .5259 L
.03997 .54184 L
.04262 .55483 L
.045 .56371 L
.04762 .57124 L
.05036 .57728 L
.05293 .58169 L
.05751 .58748 L
.06009 .5899 L
.06244 .59174 L
.0678 .59491 L
.07277 .59699 L
.07823 .59864 L
.08405 .59992 L
.08959 .60081 L
.09482 .60145 L
.09951 .6019 L
.10458 .60228 L
.10917 .60256 L
.11408 .60279 L
.11948 .60298 L
.12452 .60312 L
.12723 .60318 L
.13018 .60323 L
.13296 .60326 L
.13554 .60329 L
.13665 .6033 L
.13785 .6033 L
.13917 .60331 L
.14041 .60332 L
.14172 .60332 L
.14238 .60332 L
.14311 .60332 L
.14439 .60332 L
.14558 .60331 L
.14686 .60331 L
.14821 .6033 L
.15063 .60328 L
.15324 .60326 L
.15611 .60322 L
.16124 .60312 L
.16604 .60299 L
.17072 .60283 L
.175 .60264 L
.18463 .60203 L
Mistroke
.18941 .60161 L
.1945 .60103 L
.1993 .60033 L
.20365 .59953 L
.20879 .59829 L
.21434 .59647 L
.21958 .59405 L
.22209 .59254 L
.22439 .59091 L
.2291 .58651 L
.2317 .58328 L
.23415 .57947 L
.23851 .57016 L
.24098 .56284 L
.24325 .55417 L
.24571 .542 L
.24802 .52705 L
.25056 .50536 L
.25328 .47388 L
.25599 .43238 L
.25884 .37731 L
.26393 .26323 L
.26629 .21374 L
.26885 .16846 L
.27111 .13656 L
.2735 .11028 L
.27629 .0874 L
.27885 .07193 L
.28117 .06133 L
.28371 .05237 L
.28611 .04583 L
.28829 .04107 L
.29065 .03693 L
.29321 .0333 L
.29777 .02849 L
.30209 .02526 L
.30693 .02264 L
.31147 .02084 L
.31646 .01936 L
.32184 .01817 L
.32717 .01729 L
.33279 .01659 L
.33759 .01613 L
.34284 .01574 L
.34811 .01543 L
.3531 .01521 L
.35757 .01505 L
.36243 .01492 L
.36771 .01482 L
.37038 .01478 L
Mistroke
.37192 .01476 L
.37332 .01475 L
.37451 .01474 L
.37582 .01473 L
.37653 .01473 L
.3772 .01472 L
.37851 .01472 L
.37974 .01472 L
.38086 .01472 L
.38215 .01472 L
.38337 .01472 L
.38453 .01472 L
.38578 .01473 L
.38696 .01474 L
.38803 .01474 L
.39065 .01477 L
.39314 .0148 L
.39809 .01489 L
.40088 .01495 L
.40346 .01502 L
.41242 .01537 L
.41695 .01561 L
.42194 .01594 L
.42686 .01636 L
.43204 .01693 L
.43646 .01755 L
.4413 .01841 L
.44653 .01964 L
.45211 .02144 L
.45738 .02383 L
.46216 .02688 L
.46447 .02879 L
.46702 .03132 L
.46949 .03435 L
.47215 .0384 L
.47464 .04319 L
.47691 .04868 L
.47907 .05518 L
.48136 .06393 L
.48401 .0772 L
.48647 .09362 L
.48939 .12028 L
.49204 .15321 L
.49455 .19373 L
.49726 .24752 L
.50216 .35788 L
.50455 .40739 L
.5071 .45195 L
.50931 .48273 L
.5117 .5088 L
Mistroke
.51452 .53156 L
.51718 .54727 L
.51976 .55867 L
.52216 .56672 L
.52467 .5733 L
.52702 .57817 L
.52983 .58274 L
.53239 .58604 L
.53761 .59098 L
.5405 .59298 L
.54324 .59453 L
.54826 .59673 L
.55282 .59821 L
.55808 .59949 L
.56304 .6004 L
.56763 .60106 L
.57187 .60154 L
.57686 .602 L
.58142 .60233 L
.58611 .6026 L
.59115 .60282 L
.59581 .60299 L
.60024 .60311 L
.60273 .60316 L
.60537 .60321 L
.60766 .60324 L
.61013 .60327 L
.61151 .60329 L
.61277 .6033 L
.61405 .6033 L
.61525 .60331 L
.61656 .60332 L
.61725 .60332 L
.618 .60332 L
.61878 .60332 L
.61952 .60332 L
.62018 .60332 L
.62089 .60332 L
.62213 .60331 L
.62325 .60331 L
.62455 .6033 L
.62578 .60329 L
.62797 .60327 L
.63035 .60325 L
.63297 .60321 L
.63575 .60315 L
.64076 .60303 L
.64645 .60284 L
.6518 .60261 L
.65661 .60233 L
Mistroke
.6618 .60195 L
.66673 .60149 L
.67133 .60095 L
.67637 .60019 L
.68177 .59911 L
.68744 .59756 L
.6928 .59552 L
.69761 .59297 L
.70033 .5911 L
.70284 .589 L
.70749 .58382 L
.7101 .57989 L
.71249 .57537 L
.7169 .56373 L
.71912 .55555 L
.72154 .54403 L
.72425 .52681 L
.72671 .50577 L
.72954 .47296 L
.73222 .43173 L
.73695 .33509 L
.74215 .22034 L
.74458 .17598 L
.74719 .13803 L
.74965 .11069 L
.75189 .09161 L
.75442 .07527 L
.75718 .06205 L
.75978 .05274 L
.76219 .04608 L
.76644 .03756 L
.76887 .03399 L
.77114 .03125 L
.77572 .02705 L
.77829 .02525 L
.78066 .02386 L
.78538 .02168 L
.79044 .01996 L
.79521 .01875 L
.79955 .0179 L
.80465 .01711 L
.81018 .01647 L
.81541 .016 L
.82024 .01566 L
.8249 .0154 L
.82991 .01518 L
.83464 .01502 L
.83897 .01491 L
.84402 .01482 L
.84689 .01478 L
Mistroke
.84825 .01476 L
.84954 .01475 L
.85077 .01474 L
.8521 .01473 L
.8533 .01472 L
.85441 .01472 L
.85561 .01472 L
.85693 .01472 L
.8576 .01472 L
.85831 .01472 L
.85959 .01472 L
.86081 .01472 L
.86151 .01473 L
.86216 .01473 L
.86488 .01475 L
.86724 .01477 L
.86977 .01481 L
.87259 .01486 L
.87555 .01492 L
.88084 .01506 L
.88582 .01524 L
.89045 .01546 L
.89553 .01576 L
.90096 .01617 L
.9063 .0167 L
.91121 .01733 L
.9166 .01824 L
.92232 .01953 L
.92727 .02106 L
.93191 .02299 L
.937 .02591 L
.93986 .02809 L
.94247 .03053 L
.94652 .03553 L
.94879 .03918 L
.95084 .04321 L
.95277 .04778 L
.95484 .05381 L
.9586 .06873 L
.96073 .08036 L
.96306 .0968 L
.9655 .11941 L
.96777 .14685 L
.97184 .21445 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{177, 109.125},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002a0000KA000`40O003h00OF7oo00<007ooOol0
EWoo001HOol00`00Oomoo`1FOol005Qoo`03001oogoo05Ioo`00>Goo50002goo00<007ooOol0<Woo
4`004Goo000fOol3000DOol30008Ool00`00Oomoo`0^Ool4000COol4000=Ool003Aoo`8001Yoo`80
00Ioo`03001oogoo02eoo`8001Yoo`03001oogoo00Yoo`00<goo00<007ooOol077oo00<007ooOol0
0goo00<007ooOol0;7oo00<007ooOol06goo00<007ooOol02Goo000cOol00`00Oomoo`09Ool20004
Ool20003Ool30003Ool40004Ool00`00Oomoo`0[Ool00`00Oomoo`0MOol00`00Oomoo`08Ool0039o
o`03001oogoo00Uoo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`02Ool00`00
Oomoo`0ZOol00`00Oomoo`0NOol00`00Oomoo`08Ool0039oo`03001oogoo00=oo`D00005Ool007oo
Ool00009Ool00`00Oomoo`05Ool00`00Oomoo`02Ool2000[Ool00`00Oomoo`0OOol00`00Oomoo`07
Ool0039oo`03001oogoo00Uoo`04001oogoo0009Ool00`00Oomoo`02Ool300000goo001oo`02Ool0
0`00Oomoo`0ZOol00`00Oomoo`0OOol00`00Oomoo`07Ool0035oo`03001oogoo00Yoo`04001oogoo
0009Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00<007ooOol0:Goo00<007ooOol087oo00<0
07ooOol01goo000aOol00`00Oomoo`0;Ool20009Ool20004Ool50003Ool00`00Oomoo`0YOol00`00
Oomoo`0QOol00`00Oomoo`06Ool0035oo`03001oogoo021oo`05001oogooOol0000[Ool00`00Oomo
o`0QOol00`00Oomoo`06Ool0035oo`03001oogoo025oo`04001oogoo000[Ool00`00Oomoo`0QOol0
0`00Oomoo`06Ool0031oo`03001oogoo029oo`04001oogoo000[Ool00`00Oomoo`0QOol00`00Oomo
o`06Ool0031oo`03001oogoo029oo`04001oogoo000[Ool00`00Oomoo`0ROol00`00Oomoo`05Ool0
031oo`03001oogoo029oo`04001oogoo000[Ool00`00Oomoo`0ROol00`00Oomoo`05Ool0031oo`03
001oogoo029oo`04001oogoo000[Ool00`00Oomoo`0ROol00`00Oomoo`05Ool0031oo`03001oogoo
029oo`04001oogoo000ZOol00`00Oomoo`0SOol00`00Oomoo`05Ool0031oo`03001oogoo02=oo`03
001oo`0002Yoo`03001oogoo02=oo`03001oogoo00Eoo`00<7oo00<007ooOol08goo00<007oo0000
:Woo00<007ooOol08goo00<007ooOol01Goo000`Ool00`00Oomoo`0BOol20004Ool20003Ool30003
Ool00`00Ool0000ZOol00`00Oomoo`0SOol00`00Oomoo`05Ool002moo`03001oogoo019oo`04001o
ogoo0003Ool20004Ool00`00Oomoo`02Ool00`00Ool0000ZOol00`00Oomoo`0SOol00`00Oomoo`05
Ool002moo`03001oogoo00aoo`D00005Ool007ooOol00009Ool00`00Oomoo`02Ool01000Ool00000
:Goo00<007ooOol08goo00<007ooOol01Goo000_Ool00`00Oomoo`0BOol01000Oomoo`002Goo00<0
07ooOol00Woo00<007oo0000:Woo00<007ooOol08goo00<007ooOol01Goo000_Ool00`00Oomoo`0B
Ool01000Oomoo`002Goo00<007ooOol00Woo00<007oo0000:Woo00<007ooOol08goo00<007ooOol0
1Goo000_Ool00`00Oomoo`0COol20009Ool20004Ool00`00Ool0000ZOol00`00Oomoo`0SOol00`00
Oomoo`05Ool002moo`03001oogoo02Aoo`03001oo`0002Yoo`03001oogoo02=oo`03001oogoo00Eo
o`00;goo00<007ooOol097oo00<007oo0000:Goo00<007ooOol097oo00<007ooOol01Goo000_Ool0
0`00Oomoo`0TOol00`00Ool0000YOol00`00Oomoo`0TOol00`00Oomoo`05Ool002moo`03001oogoo
02Eoo`8002Uoo`03001oogoo02Aoo`03001oogoo00Eoo`00;goo00<007ooOol09Goo0P00:Goo00<0
07ooOol097oo00<007ooOol01Goo000_Ool00`00Oomoo`0UOol2000YOol00`00Oomoo`0UOol00`00
Oomoo`04Ool002moo`03001oogoo02Eoo`8002Uoo`03001oogoo02Eoo`03001oogoo00Aoo`00;goo
00<007ooOol09Goo0P00:Goo00<007ooOol09Goo00<007ooOol017oo000_Ool00`00Oomoo`0UOol2
000YOol00`00Oomoo`0UOol00`00Oomoo`04Ool002moo`03001oogoo00eoo`8000Aoo`8000Aoo`80
00=oo`<000Aoo`8002Uoo`03001oogoo02Eoo`03001oogoo00Aoo`00;goo00<007ooOol037oo00@0
07ooOol000=oo`8000=oo`04001oogoo0005Ool01000Oomoogoo0P00:Goo00<007ooOol09Goo00<0
07ooOol017oo000_Ool00`00Oomoo`06Ool500001Goo001oogoo000027oo00@007ooOol000Eoo`04
001oogooOol3000XOol00`00Oomoo`0UOol00`00Oomoo`04Ool002moo`03001oogoo00aoo`04001o
ogoo0008Ool01000Oomoo`000Woo0`0017oo0P00:Goo00<007ooOol09Goo00<007ooOol017oo000_
Ool00`00Oomoo`0<Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00Aoo`8002Uoo`03
001oogoo02Eoo`03001oogoo00Aoo`00;goo00<007ooOol03Goo0P002Woo0P000goo10000goo0P00
:Goo00<007ooOol09Goo00<007ooOol017oo000^Ool00`00Oomoo`0VOol2000YOol00`00Oomoo`0U
Ool00`00Oomoo`04Ool002ioo`03001oogoo02Ioo`8002Uoo`03001oogoo02Eoo`03001oogoo00Ao
o`002Woo0P0067oo0`001goo00<007ooOol03Goo10005Goo0P005goo10003Woo00<007ooOol01goo
0`005goo0P000Woo00<007ooOol017oo0009Ool01000Oomoo`0067oo00<007ooOol01Woo00<007oo
Ool03Woo00<007ooOol05Goo0P0067oo00<007ooOol03Woo00<007ooOol027oo00<007ooOol05Goo
00H007ooOol007oo0006Ool000=oo`D00005Ool007ooOol0000@Ool500000goo000000020007Ool0
0`00Oomoo`07Ool50003Ool00`00Oomoo`0DOol2000IOol00`00Oomoo`0=Ool00`00Oomoo`06Ool4
000FOol01000Oomoo`000Woo00<007ooOol00goo0009Ool3000GOol00`00Ool00008Ool00`00Oomo
o`0@Ool00`00Oomoo`0COol2000JOol00`00Oomoo`0<Ool00`00Oomoo`06Ool00`00Ool0000GOol3
0003Ool00`00Oomoo`03Ool000Uoo`03001oogoo01Moo`03001oo`0000Qoo`03001oogoo00eoo`04
001oogoo000EOol2000GOol01000Oomoo`003Woo00<007ooOol01Woo00<007oo00005goo00<007oo
Ool00goo00<007ooOol00goo000:Ool3000GOol20008Ool00`00Oomoo`0>Ool2000FOol2000HOol2
000>Ool00`00Oomoo`08Ool2000HOol30002Ool00`00Oomoo`03Ool002ioo`03001oogoo02Ioo`80
02Qoo`03001oogoo02Moo`03001oogoo00=oo`00;Woo00<007ooOol09Woo0P00:7oo00<007ooOol0
9goo00<007ooOol00goo000^Ool00`00Oomoo`0VOol2000XOol00`00Oomoo`0WOol00`00Oomoo`03
Ool00;00005oo`0000Eoo`00Oomoo`0000=oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo
00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo009oo`03001oo`0000Eoo`03001oogoo
00Aoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo
00=oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo
00=oo`03001oo`0000Aoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo
00=oo`03001oogoo00Aoo`03001oogoo00Aoo`40005oo`5oo`0017oo00D007ooOomoo`0001Uoo`03
001oogoo00Uoo`03001oogoo00aoo`03001oogoo01Qoo`03001oogoo01Moo`03001oogoo00aoo`03
001oogoo00Uoo`03001oogoo01Moo`03001oogoo00Moo`0017oo00<007ooOol09Woo00<007ooOol0
:7oo00<007ooOol09Woo00<007ooOol0;Goo0004Ool00`00Oomoo`0VOol00`00Oomoo`0XOol00`00
Oomoo`0VOol00`00Oomoo`0]Ool000Aoo`03001oogoo02Ioo`03001oogoo02Qoo`03001oogoo02Io
o`03001oogoo02eoo`0017oo00<007ooOol09Woo00<007ooOol0:7oo00<007ooOol09Woo00<007oo
Ool0;Goo0004Ool00`00Oomoo`0VOol00`00Oomoo`0XOol00`00Oomoo`0VOol00`00Oomoo`0]Ool0
00Aoo`03001oogoo02Ioo`03001oogoo02Qoo`03001oogoo02Ioo`03001oogoo02eoo`0017oo00<0
07ooOol09Woo00<007ooOol0:7oo00<007ooOol09Woo00<007ooOol0;Goo0005Ool00`00Oomoo`0U
Ool00`00Oomoo`0XOol00`00Oomoo`0VOol00`00Oomoo`0]Ool000Eoo`03001oogoo02Eoo`03001o
ogoo02Qoo`03001oogoo02Ioo`03001oogoo02eoo`001Goo00<007ooOol09Goo00<007ooOol0:7oo
00<007ooOol09Woo00<007ooOol0;Goo0005Ool00`00Oomoo`0UOol00`00Oomoo`0XOol00`00Oomo
o`0VOol00`00Oomoo`0]Ool000Eoo`03001oogoo02Eoo`03001oogoo00moo`8000Aoo`8000Aoo`80
00=oo`<000Eoo`03001oogoo02Eoo`03001oogoo02ioo`001Goo00<007ooOol09Goo00<007ooOol0
3Woo00@007ooOol000=oo`8000=oo`04001oogoo0005Ool00`00Oomoo`02Ool00`00Oomoo`0UOol0
0`00Oomoo`0^Ool000Eoo`03001oogoo02Eoo`03001oogoo00ioo`04001oogoo0008Ool01000Oomo
o`001Goo00<007ooOol00Woo0P009Woo00<007ooOol0;Woo0005Ool00`00Oomoo`0UOol00`00Oomo
o`0>Ool01000Oomoo`0027oo00@007ooOol0009oo`<000Eoo`03001oogoo02Eoo`03001oogoo02io
o`001Goo00<007ooOol09Goo00<007ooOol03Woo00@007ooOol000Qoo`04001oogoo0002Ool00`00
Oomoo`05Ool00`00Oomoo`0UOol00`00Oomoo`0^Ool000Eoo`03001oogoo02Eoo`03001oogoo00mo
o`8000Yoo`8000=oo`@000Aoo`03001oogoo02Eoo`03001oogoo02ioo`001Goo00<007ooOol09Goo
00<007ooOol0:7oo00<007ooOol09Goo00<007ooOol0;Woo0005Ool00`00Oomoo`0UOol00`00Oomo
o`0XOol00`00Oomoo`0UOol00`00Oomoo`0^Ool000Eoo`03001oogoo02Eoo`03001oogoo02Qoo`80
02Ioo`03001oogoo02ioo`001Goo00<007ooOol09Goo00<007ooOol0:7oo0P009Woo00<007ooOol0
;Woo0005Ool00`00Oomoo`0UOol00`00Oomoo`0XOol2000VOol00`00Oomoo`0^Ool000Eoo`03001o
ogoo02Eoo`03001oogoo02Qoo`8002Ioo`03001oogoo02ioo`001Goo00<007ooOol097oo00<007oo
Ool0:Goo0P009Woo00<007ooOol0;Woo0005Ool00`00Oomoo`0TOol00`00Oomoo`0YOol2000VOol0
0`00Oomoo`0^Ool000Ioo`03001oogoo02=oo`03001oogoo02Uoo`8002Ioo`03001oogoo02ioo`00
1Woo00<007ooOol08goo00<007ooOol05Woo0P0017oo0P000goo0`001Goo0P009Woo00<007ooOol0
;Woo0006Ool00`00Oomoo`0SOol00`00Oomoo`0EOol01000Oomoo`000goo0P0017oo00<007ooOol0
17oo0P009Woo00<007ooOol0;Woo0006Ool00`00Oomoo`0SOol00`00Oomoo`0EOol01000Oomoo`00
2Goo00<007ooOol017oo0P009Woo00<007ooOol0;Woo0006Ool00`00Oomoo`0SOol00`00Oomoo`0E
Ool01000Oomoo`002Goo00<007ooOol017oo0P009Woo00<007ooOol0;Woo0006Ool00`00Oomoo`0S
Ool00`00Oomoo`0EOol01000Oomoo`002Goo00<007ooOol017oo0P009Goo00<007ooOol0;goo0006
Ool00`00Oomoo`0SOol00`00Oomoo`0FOol20009Ool20006Ool00`00Ool0000TOol00`00Oomoo`0_
Ool000Ioo`03001oogoo02=oo`03001oogoo02Uoo`03001oo`0002Aoo`03001oogoo02moo`001Woo
00<007ooOol08goo00<007ooOol0:Goo00<007oo000097oo00<007ooOol0;goo0007Ool00`00Oomo
o`0ROol00`00Oomoo`0YOol00`00Ool0000TOol00`00Oomoo`0_Ool000Moo`03001oogoo025oo`03
001oogoo02Yoo`03001oo`0002Aoo`03001oogoo02moo`001goo00<007ooOol08Goo00<007ooOol0
:Woo00<007oo000097oo00<007ooOol0;goo0007Ool00`00Oomoo`0QOol00`00Oomoo`0ZOol00`00
Ool0000TOol00`00Oomoo`0_Ool000Moo`03001oogoo025oo`03001oogoo02Yoo`04001oogoo000R
Ool00`00Oomoo`0`Ool000Moo`03001oogoo025oo`03001oogoo02Yoo`04001oogoo000ROol00`00
Oomoo`0`Ool000Moo`03001oogoo025oo`03001oogoo02Yoo`04001oogoo000ROol00`00Oomoo`0`
Ool000Moo`03001oogoo025oo`03001oogoo015oo`8000Aoo`8000=oo`<000=oo`<000Eoo`04001o
ogoo000ROol00`00Oomoo`0`Ool000Qoo`03001oogoo01moo`03001oogoo015oo`04001oogoo0003
Ool20004Ool00`00Oomoo`05Ool00`00Oomoo`02Ool01000Oomoo`008Goo00<007ooOol0<Goo0008
Ool00`00Oomoo`0OOol00`00Oomoo`0AOol01000Oomoo`002Goo00<007ooOol01Goo00<007ooOol0
0Woo0P0000=oo`00Ool087oo00<007ooOol0<Goo0009Ool00`00Oomoo`0NOol00`00Oomoo`0AOol0
1000Oomoo`002Goo00<007ooOol00Woo0`001Goo00D007ooOomoo`00021oo`03001oogoo035oo`00
2Goo00<007ooOol07Goo00<007ooOol04Woo00@007ooOol000Uoo`03001oogoo009oo`03001oogoo
00Eoo`04001oogooOol2000NOol00`00Oomoo`0bOol000Yoo`03001oogoo01]oo`03001oogoo01Ao
o`8000Uoo`8000Aoo`@000Aoo`03001oogoo009oo`03001oogoo01]oo`8003Aoo`002goo0P006Goo
0P00<7oo00<007ooOol00goo0P006Woo0P00=Goo000=Ool4000BOol4000aOol00`00Oomoo`05Ool4
000BOol4000gOol0015ooa8003Eoo`03001oogoo00Qooa<003]oo`00F7oo00<007ooOol0EWoo001H
Ool00`00Oomoo`1FOol00001\
\>"],
  ImageRangeCache->{{{0, 176}, {108.125, 0}} -> {-6.63696, -0.178151, 
  0.07542, 0.00329525}}]
}, Open  ]],

Cell[TextData[{
  "The ",
  StyleBox["Evaluate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function has been used to make the plot easy for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to draw. It prevents ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " repeating the ",
  StyleBox["Nest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " operation at every point at which the function is sampled. With ",
  StyleBox["Evaluate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " in place it will only do that once. The example shown, and why it looks \
like a square wave, are discussed in detail by Gray and Glynn (1990)."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Few Useful Observations", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Once or twice we have made use of a couple of features that deserve \
explicit comment.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Referencing Earlier Results", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "During a sequence of calculations you will probably want to refer to the \
results of previous calculations.  This is achieved easily using a per cent \
sign, ",
  StyleBox["%",
    FontFamily->"Courier"],
  ", immediately followed by the number of the output corresponding to the \
result. Recall the definition of data:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["data", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "A special shorthand for the last computed result is ",
  StyleBox["%",
    FontFamily->"Courier"],
  "; ",
  StyleBox["%%",
    FontFamily->"Courier"],
  " denotes the next-to-last result."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["%", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1001, 0, 1, 4, 9, 16, 25, 36, 49, 64, 81, 100}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["%%[[3]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Suppressing Data Output", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Our list ",
  StyleBox["data",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is short, so there is no disadvantage in having ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " output the full result.  If you are working with large data sets, \
however, you will almost certainly not want to see the results at each stage. \
 Consequently, it may sometimes be desirable to suppress large results of \
purely symbolic operations.  In the first of the following operations, the \
data set ",
  StyleBox["bigdata",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is created without being displayed at all. ",
  StyleBox[
  "The semicolon is the key ingredient for suppressing output completely. ",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["bigdata=Table[i^2,{i,0,10000}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The next example shows a way of producing a shortened form of \
output.  Remember this is only shortening what is displayed. The actual \
result is still a list of 10001 terms.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Short[bigdata]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[\({0, 1, 4, 9, 16, \[LeftSkeleton]9991\[RightSkeleton], 99920016, 
        99940009, 99960004, 99980001, 100000000}\),
      Short]], "Output"]
}, Open  ]],

Cell[TextData[{
  "You may get different results depending on the exact version of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " you are using. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.5 Algebra", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Simple Algebraic Manipulation", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We'll begin by doing some simple algebra. Enter the following statements \
executing each one as you go.  You will see that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has its own ideas about how to order the terms in a polynomial \[Dash] do \
not let this worry you."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["y^4 + 4 y^3 + 6 y^2", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(6\ y\^2 + 4\ y\^3 + y\^4\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now, add ",
  Cell[BoxData[
      \(TraditionalForm\`4  y\)]],
  " to this last result:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["% + 4 y", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(4\ y + 6\ y\^2 + 4\ y\^3 + y\^4\)], "Output"]
}, Open  ]],

Cell["Add 1 to this last result, and factorize this:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Factor[% + 1]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(\((1 + y)\)\^4\)], "Output"]
}, Open  ]],

Cell["Undo the factorization by expanding this last result:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Expand[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1 + 4\ y + 6\ y\^2 + 4\ y\^3 + y\^4\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Assign ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " to be this last quartic expression in ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["x = %", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1 + 4\ y + 6\ y\^2 + 4\ y\^3 + y\^4\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Find the solutions for the quartic expression to be zero, using ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " as a shorter version of the quartic expression:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Solve[x == 0, y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({{y \[Rule] \(-1\)}, {y \[Rule] \(-1\)}, {y \[Rule] \(-1\)}, {y \[Rule] \
\(-1\)}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Assign the value 2 to ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", and without displaying that result ask for the resulting value of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
y = 2;
x\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(81\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now cancel any values associated with ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", and ask for the value of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Clear[y];x", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1 + 4\ y + 6\ y\^2 + 4\ y\^3 + y\^4\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The three things you should appreciate from this little session are the \
use of ",
  StyleBox["Expand",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["Factor",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Clear",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  Also, we used ",
  StyleBox["Solve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " as encountered in the Numerical Calculations section, to give us \
solutions to an algebraic equation.  It is a good idea to get into the habit \
of using ",
  StyleBox["Clear",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " frequently, to make sure that you do not cause problems simply by trying \
to define quantities that conflict with earlier definitions.  You should \
clear the definitions of ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["y",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " now, if you are running this session on a computer."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[x,y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Simplifying Rational Expressions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It is essential that you should have cleared the values for ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["y",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " from the last sequence of calculations, if you are trying out this \
chapter on a computer. If you have not done this ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will go into an internal loop and will complain about exceeding its \
recursion depth.  If this happens you will want to abort the calculation, \
which as you will remember from the introductory section is done by selecting \
\"Interrupt Evaluation...\"  or \"Abort Evaluation\"  from the Kernel menu."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "First, we will input this simple rational expression involving ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["y = 1/(x^3 - 4) - 1/(x^2 - 2)", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(1\/\(\(-2\) + x\^2\)\)\) + 1\/\(\(-4\) + x\^3\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now, we can ask for all of the terms to be gathered together over a common \
denominator, and this expression to be assigned to be ",
  StyleBox["u",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["u = Together[y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(2 + x\^2 - x\^3\)\/\(\((\(-2\) + x\^2)\)\ \((\(-4\) + x\^3)\)\)\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "Using ",
  StyleBox["u",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", the shorthand for the expression, we can select out the denominator and \
numerator of the expression."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Denominator[u]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\((\(-2\) + x\^2)\)\ \((\(-4\) + x\^3)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numerator[u]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(2 + x\^2 - x\^3\)], "Output"]
}, Open  ]],

Cell["\<\
Expansion can be aimed only at the denominator of the expression.\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["ExpandDenominator[u]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(2 + x\^2 - x\^3\)\/\(8 - 4\ x\^2 - 2\ x\^3 + x\^5\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "With rational, rather than polynomial expressions, it may help to separate \
out the top and bottom for manipulation.  In most cases you will find ",
  StyleBox["Expand",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["Factor",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Simplify",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " sufficient.  You should take care in the use of ",
  StyleBox["Simplify",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". It can sometimes get into quite a lengthy process, and even have \
unpredictable results.  It is much easier to use more functions but keep \
control over what changes are taking place."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applying Simplification Rules Selectively", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The following example will give some idea of how to apply operations \
selectively.  We begin by defining the algebraic expression ",
  StyleBox["expr",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["expr = (x^3 - 216) (x - 3)^2/((x - 2)(x - 4)(x - 6))", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\((\(-3\) + x)\)\^2\ \((\(-216\) + x\^3)\)\)\/\(\((\(-6\) + x)\)\ \((\
\(-4\) + x)\)\ \((\(-2\) + x)\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Applying ",
  StyleBox["Expand",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to this expression will expand each term in the numerator. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Expand[expr]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(1944\/\(\((\(-6\) + x)\)\ \((\(-4\) + x)\)\ \((\(-2\) + 
                  x)\)\)\)\) + \(1296\ x\)\/\(\((\(-6\) + x)\)\ \((\(-4\) + \
x)\)\ \((\(-2\) + x)\)\) - \(216\ x\^2\)\/\(\((\(-6\) + x)\)\ \((\(-4\) + \
x)\)\ \((\(-2\) + x)\)\) + \(9\ x\^3\)\/\(\((\(-6\) + x)\)\ \((\(-4\) + x)\)\ \
\((\(-2\) + x)\)\) - \(6\ x\^4\)\/\(\((\(-6\) + x)\)\ \((\(-4\) + x)\)\ \
\((\(-2\) + x)\)\) + 
      x\^5\/\(\((\(-6\) + x)\)\ \((\(-4\) + x)\)\ \((\(-2\) + x)\)\)\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  StyleBox["ExpandAll",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", however, will take ",
  StyleBox["every",
    FontSlant->"Italic"],
  " part of the expression."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["ExpandAll[expr]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(1944\/\(\(-48\) + 44\ x - 12\ x\^2 + 
              x\^3\)\)\) + \(1296\ x\)\/\(\(-48\) + 44\ x - 12\ x\^2 + x\^3\) \
- \(216\ x\^2\)\/\(\(-48\) + 44\ x - 12\ x\^2 + x\^3\) + \(9\ x\^3\)\/\(\(-48\
\) + 44\ x - 12\ x\^2 + x\^3\) - \(6\ x\^4\)\/\(\(-48\) + 44\ x - 12\ x\^2 + \
x\^3\) + x\^5\/\(\(-48\) + 44\ x - 12\ x\^2 + x\^3\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "You have already used ",
  StyleBox["Together",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", which will gather all terms over a common denominator."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Together[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(324 - 162\ x + 9\ x\^2 + x\^4\)\/\(8 - 6\ x + x\^2\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Splitting this expression into partial fractions uses ",
  StyleBox["Apart",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Apart[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(37 + 38\/\(\(-4\) + x\) - 26\/\(\(-2\) + x\) + 6\ x + x\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The ",
  StyleBox["Factor",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function will implicitly gather the terms together and factorize the \
resulting expression:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Factor[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\((\(-3\) + x)\)\^2\ \((36 + 6\ x + x\^2)\)\)\/\(\((\(-4\) + x)\)\ \
\((\(-2\) + x)\)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Another route to this simplified version of ",
  StyleBox["expr",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " would have been using the function ",
  StyleBox["Cancel",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  This will make sure that any possible cancellations are carried out."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Cancel[expr]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\((\(-3\) + x)\)\^2\ \((36 + 6\ x + x\^2)\)\)\/\(\((\(-4\) + x)\)\ \
\((\(-2\) + x)\)\)\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Other Useful Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Another useful pair of operations are ",
  StyleBox["Collect",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Coefficient",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  We shall use another polynomial expression to illustrate their use."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Clear[x, y];
stuff = (2 x + 3)^2 (x + y)^2\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\((3 + 2\ x)\)\^2\ \((x + y)\)\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Expand[stuff]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9\ x\^2 + 12\ x\^3 + 4\ x\^4 + 18\ x\ y + 24\ x\^2\ y + 8\ x\^3\ y + 
      9\ y\^2 + 12\ x\ y\^2 + 4\ x\^2\ y\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Collect[%, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(4\ x\^4 + 9\ y\^2 + x\^3\ \((12 + 8\ y)\) + 
      x\^2\ \((9 + 24\ y + 4\ y\^2)\) + x\ \((18\ y + 12\ y\^2)\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "So ",
  StyleBox["Collect",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " literally collects together terms with the same power of ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  We can pick out the coefficients by using ",
  StyleBox["Coefficient",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  Here we ask for the coefficient of ",
  StyleBox["x^2",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " (all the terms, including those in ",
  StyleBox["y",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ") in the expression."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Coefficient[Expand[stuff], x^2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9 + 24\ y + 4\ y\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This type of operation can be very useful in doing perturbation theory, \
and can also be used with the ",
  StyleBox["Series",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function which we will come to shortly. An alternative syntax is as \
follows - this must be used to extract the \"constant\" term:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Coefficient[Expand[stuff], x, 2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9 + 24\ y + 4\ y\^2\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Coefficient[Expand[stuff], x, 0]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9\ y\^2\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Making Substitutions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Substitutions are a common requirement in manipulation. One way of \
doing this is to make a simple assignment  and then ask for the new value of \
an expression.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["x=3 y;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["stuff", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(16\ y\^2\ \((3 + 6\ y)\)\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This will mean however that from now on ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " will always be read as ",
  Cell[BoxData[
      \(TraditionalForm\`3  y\)]],
  " and stuff will always equal ",
  Cell[BoxData[
      \(TraditionalForm\`16 \(\( y\^2\)(3 + 6  y)\)\^2\)]],
  ". To prevent this we will have to explicitly remove the assignment of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ". "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The technique to use to make a substitution which will not make a \
permanent change is to use a replacement rule. These are the same as are \
given as solutions to ",
  StyleBox["NSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". To make the same substitution define the rule that ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " goes to ",
  Cell[BoxData[
      \(TraditionalForm\`3  y\)]],
  ", and apply that using /. to ",
  StyleBox["stuff",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["stuff /. x -> 3 y", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(16\ y\^2\ \((3 + 6\ y)\)\^2\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["stuff", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\((3 + 2\ x)\)\^2\ \((x + y)\)\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "You can see that this temporary substitution does not alter the stored \
definition of ",
  StyleBox["stuff",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". This technique can be used for any temporary substitutions whether they \
are numerical or algebraic."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.6 Calculus", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We now come to a topic that is fundamental to appreciating how useful ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can be in modelling financial derivatives. We are interested in finding \
the values of certain options, together with the derivatives of those values \
with respect to variables such as the underlying price, its volatility, \
interest rates, and so on. When there is some form of analytical prescription \
available, it is very convenient, not to say less error-prone, to perform the \
computation of such \"Greeks\" symbolically. Even when one is working with an \
essentially numerical scheme, we shall still want to test it against known \
analytic solutions for special cases, or we may wish to differentiate a \
function based on interpolated numerical values."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Differentiation", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Differentiation and integration work in a straightforward way.  \
There are various notations for differentiation, of which the D operator is \
perhaps the most flexible.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Clear[x];
D[x^n, x]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(n\ x\^\(\(-1\) + n\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrate[%, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(x\^n\)], "Output"]
}, Open  ]],

Cell["\<\
Partial differentiation can also be carried out by stipulating the \
variable which you wish to differentiate by.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["D[x^4 y^2 + x^2, y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(2\ x\^4\ y\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Mixed and multiple derivatives are also supported.  We can differentiate \
once with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " and once with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["D[x^3 y^2 + x^2, y, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(6\ x\^2\ y\)], "Output"]
}, Open  ]],

Cell["\<\
Higher derivatives are specified using the list structure to supply \
the variable and the number of times by which to differentiate with respect \
to that variable.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["D[x^3 y^2 + x^2, {x, 3}, {y, 2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(12\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integration - Symbolic and Numeric", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Integrating with respect to y will reverse differentiation up to an \
arbitrary additive function of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Integrate[%%, y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(3\ x\^2\ y\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similarly multiple integration can be carried out.  As in the examples for \
differentiation, to integrate with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Integrate[x^2 y,x,y]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(x\^3\ y\^2\)\/6\)], "Output"]
}, Open  ]],

Cell["\<\
Definite integration needs a list supplying the upper and lower \
limits of the integration variable.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Integrate[1/(1-x),{x,2,4}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-Log[3]\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "There is also the facility to calculate integrals numerically.  This is \
available for any functions, including those which ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " cannot do symbolically. As in the case of definite symbolic integration, \
we must specify the range of integration. This can frequently be used to \
compute expected values of complicated payoffs, if they do not work \
symbolically. This next example shows how symbolic integration is not \
possible, but numerical integration yields a result."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Integrate[Sin[Sin[z]],{z,0,2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(\[Integral]\_0\%2 Sin[Sin[z]] \[DifferentialD]z\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NIntegrate[Sin[Sin[z]],{z,0,2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1.24705605824400311`\)], "Output"]
}, Open  ]],

Cell["\<\
This is another function under which problems can occur, and you \
should take care.  For example:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Integrate[Sin[1000 x],{x,0,2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/1000 - Cos[2000]\/1000\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["N[%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.00136745954910090361`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NIntegrate[Sin[1000 x],{x,0,2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(NIntegrate::"ncvb" \( : \ \) 
      "NIntegrate failed to converge to prescribed accuracy after \!\(7\) \
recursive bisections in \!\(x\) near \!\(x\) = \!\(1.4140625`\)."\)], 
  "Message"],

Cell[BoxData[
    \(0.00184811425338333847`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This answer should not be believed because we have been warned about a \
problem. The ways to get around this problem are covered more fully when we \
have investigated how built-in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions can be manipulated in the next section."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Limits", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Of course, differentiation is defined by a limiting procedure. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can take limits explicitly:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Limit[((x + h)^3 - x^3)/h, h -> 0]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(3\ x\^2\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Series", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " also knows how to use Taylor's Theorem through the ",
  StyleBox["Series",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " command. The use of ",
  StyleBox["Series",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " generates a certain type of expression truncated by an \"O\" expression. \
These can be manipulated according to the usual rules of asymptotic series, \
or converted to a normal form without the \"O\". Note that the next power in \
the \"O\" term is not necessarily computed exactly:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Series[Sin[Sin[x]], {x, 0, 5}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"x", "-", \(x\^3\/3\), "+", \(x\^5\/10\), "+", 
        InterpretationBox[\(O[x]\^6\),
          SeriesData[ x, 0, {}, 1, 6, 1]]}],
      SeriesData[ x, 0, {1, 0, 
        Rational[ -1, 3], 0, 
        Rational[ 1, 10]}, 1, 6, 1]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["%^2", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
      \(x\^2\), "-", \(\(2\ x\^4\)\/3\), "+", \(\(14\ x\^6\)\/45\), "+", 
        InterpretationBox[\(O[x]\^7\),
          SeriesData[ x, 0, {}, 2, 7, 1]]}],
      SeriesData[ x, 0, {1, 0, 
        Rational[ -2, 3], 0, 
        Rational[ 14, 45]}, 2, 7, 1]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Normal[%%]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(x - x\^3\/3 + x\^5\/10\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple Ordinary Differential Equations", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A large part of this book is concerned with various methods for \
solving partial differential equations, so we shall explore that topic in \
some detail later. In our introduction we shall take a brief look at \
ODEs.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Symbolic Solution of Differential Equations", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is a useful tool for dealing with certain differential equations.  The \
function ",
  StyleBox["DSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " can deal with a large class of equations.  It is used in a similar way to \
",
  StyleBox["Solve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", except that you need to specify both the independent and dependent \
variables. Notice how the equation is specified by using the double = sign."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["DSolve[y''[x] + y[x] == 0, y[x], x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{y[x] \[Rule] C[2]\ Cos[x] - C[1]\ Sin[x]}}\)], "Output"]
}, Open  ]],

Cell["\<\
If you give appropriate initial conditions, the arbitrary constants \
are removed.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["DSolve[{y'[x] + x y[x] == 0, y[0] == 1}, y[x], x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \({{y[x] \[Rule] E\^\(-\(x\^2\/2\)\)}}\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[TextData[{
  "To check that this is indeed a solution of the given equation we use the \
technique of replacement. The solutions to these equations are given as \
replacement rules presented in a list. We therefore need to extract a \
particular element of our solution list and apply that rule to get the \
solution. Setting the solution to be represented by ",
  StyleBox["s",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " we select the first element of the list (we only have one in this \
example), and then apply that rule to",
  StyleBox[" y[x]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to see the expression which is a solution."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["s=y[x]/.%[[1]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(x\^2\/2\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
Now we can check if this is indeed a solution to our original \
equation.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["D[s,x] + x s", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Numerical Solutions to Differential Equations", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In a similar way to numerical integration there is a function ",
  StyleBox["NDSolve",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " which will find numerical solutions to differential equations and extends \
the scope of the equations to which we can find solutions. This function \
returns an interpolating function, rather than the usual table of values.  \
This allows you to ask for the values of the solution anywhere in the \
solution range, for example.  As usual, the use of \"/.\" allows for the \
extraction of numerical values.  "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Clear[x, y, t];
soln = NDSolve[ {x'[t] == y[t],y'[t] == -4.0*x[t] - 0.1*y[t],
x[0] == 1,y[0] == 1},{x, y}, {t, 10}]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"{", 
        RowBox[{
          RowBox[{"x", "\[Rule]", 
            TagBox[\(InterpolatingFunction[{{0.`, 10.`}}, "<>"]\),
              False,
              Editable->False]}], ",", 
          RowBox[{"y", "\[Rule]", 
            TagBox[\(InterpolatingFunction[{{0.`, 10.`}}, "<>"]\),
              False,
              Editable->False]}]}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Note that the answer is supplied as an ",
  StyleBox["InterpolatingFunction",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " - this allows a numerical solution computed in steps to be used just like \
an ordinary function. We can select out the specific solution for any ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  ".  At ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ 2\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["{x[2], y[2]} /. soln", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{\(-0.951524237300220576`\), 0.793320539050230255`}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We shall use the ",
  StyleBox["InterpolatingFunction ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "as a means of handling the output of finite-difference solvers for option \
equations. We shall explore numerical methods for solving PDEs in some detail \
later in this book, starting in Chapter 13."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complex Numbers", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Built-In Functions for Complex Manipulation", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The use of complex numbers and functions is relatively unusual in \
derivatives modelling, but recent research has introduced Laplace transform \
methods as a powerful tool for the analytical pricing of options previously \
thought to be unpriceable except through an essentially numerical scheme \
(e.g. arithmetically averaged Asian options). So we need to use complex \
numbers, and to deal with them in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". The symbol I (capital) is the square root of ",
  Cell[BoxData[
      \(TraditionalForm\`\(-1\)\)]],
  ", in Standard Form.  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " operations work on complex numbers in just the same way they work on real \
numbers. In addition, various special operations for dealing with complex \
variables are provided."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Let us find the solution set for the following equation. ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["qsoln = Solve[3 x^2 - 4 x + 6 == 0, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{x \[Rule] 1\/3\ \((2 - I\ \@14)\)}, {
        x \[Rule] 1\/3\ \((2 + I\ \@14)\)}}\)], "Output"]
}, Open  ]],

Cell["We put that into an approximate numerical form,", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["N[qsoln]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{x \[Rule] 
          \(0.666666666666666607`\[InvisibleSpace]\) - 
            1.24721912892464703`\ I}, {
        x \[Rule] 
          \(0.666666666666666607`\[InvisibleSpace]\) + 
            1.24721912892464703`\ I}}\)], "Output"]
}, Open  ]],

Cell["\<\
and then create the set containing the numerical solutions to the \
original equation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["solutions = x /. %", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(0.666666666666666607`\[InvisibleSpace]\) - 1.24721912892464703`\ I, 
      \(0.666666666666666607`\[InvisibleSpace]\) + 1.24721912892464703`\ I}
      \)], "Output"]
}, Open  ]],

Cell["\<\
As with the exponential function, a special typeface is used for I \
in Traditional Form notation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[qsoln]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`{{x \[Rule] 1\/3\ \((2 - \[ImaginaryI]\ \@14)\)}, {
        x \[Rule] 1\/3\ \((2 + \[ImaginaryI]\ \@14)\)}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The function ",
  StyleBox["Re",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will return the real part of any complex number.  We can apply this \
function to our set ",
  StyleBox["solutions",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to get the real parts of our set."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Re[solutions]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.666666666666666607`, 0.666666666666666607`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Similarly, the function ",
  StyleBox["Im",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will give the imaginary part of a complex number. But note that symbols \
are in general treated as complex - for example:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["z = x + I y;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Im[z]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Im[x] + Re[y]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The function ",
  StyleBox["Conjugate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will give the complex conjugate of a complex number:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Conjugate[solutions]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(0.666666666666666607`\[InvisibleSpace]\) + 1.24721912892464703`\ I, 
      \(0.666666666666666607`\[InvisibleSpace]\) - 1.24721912892464703`\ I}
      \)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The function ",
  StyleBox["Abs",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the absolute value of a complex number (i.e. ",
  Cell[BoxData[
      \(TraditionalForm\`\@\(x\^2 + y\^2\)\)]],
  "):"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Abs[solutions]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.41421356237309492`, 1.41421356237309492`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The function ",
  StyleBox["Arg",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will give the argument of the complex number (i.e. ",
  Cell[BoxData[
      \(TraditionalForm\`ArcTan(y/x)\)]],
  ")"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Arg[solutions]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-1.07991364850558513`\), 1.07991364850558513`}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "There is a useful ",
  StyleBox["ComplexExpand",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function, which carries out an expansion assuming that all variables are \
real:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["ComplexExpand[Cot[x + I y]]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(Sin[2\ x]\/\(Cos[2\ x] - Cosh[2\ y]\)\)\) + 
      \(I\ Sinh[2\ y]\)\/\(Cos[2\ x] - Cosh[2\ y]\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This function is very useful for getting ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to output what you expect to see when your mind assumes things are real, \
but the computer can't read your mind!"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Complex Contour Integration", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For integration in the complex plane the function ",
  StyleBox["NIntegrate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " can take complex numbers to specify the boundary around which the \
function is to be integrated."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["NIntegrate[1/z, {z, 2, I, -2, -I, 2}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(0.`\[InvisibleSpace]\) + 6.28318530717959156`\ I\)], "Output"]
}, Open  ]],

Cell["\<\
We shall use complex contour integration to invert certain types of \
Laplace transform for option values.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.7 More about Functions", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We have already seen several ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions in the course of our investigations so far.  These are only a \
small proportion of those available. Some of the others are obvious.  One \
example is trigonometric functions.  Since you already know we can use ",
  StyleBox["Sin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " you would expect ",
  StyleBox["Cos",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["Tan",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and all the usual inverse and hyperbolic versions. These can all be used \
in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " in the same way as ",
  StyleBox["Sin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", as can the inverses of these trigonometric functions, as illustrated \
below."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Plot[ArcSin[x], {x, -1, 1}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.476191 0.309017 0.187407 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -12 -9 ]
[.2619 .29652 12 0 ]
[.7381 .29652 -9 -9 ]
[.7381 .29652 9 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .02791 -24 -4.5 ]
[.4875 .02791 0 4.5 ]
[.4875 .12161 -12 -4.5 ]
[.4875 .12161 0 4.5 ]
[.4875 .21531 -24 -4.5 ]
[.4875 .21531 0 4.5 ]
[.4875 .40272 -18 -4.5 ]
[.4875 .40272 0 4.5 ]
[.4875 .49642 -6 -4.5 ]
[.4875 .49642 0 4.5 ]
[.4875 .59013 -18 -4.5 ]
[.4875 .59013 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-1)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-0.5)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(0.5)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(1)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.21429 .30902 m
.21429 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.40476 .30902 m
.40476 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.59524 .30902 m
.59524 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.78571 .30902 m
.78571 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .02791 m
.50625 .02791 L
s
[(-1.5)] .4875 .02791 1 0 Mshowa
.5 .12161 m
.50625 .12161 L
s
[(-1)] .4875 .12161 1 0 Mshowa
.5 .21531 m
.50625 .21531 L
s
[(-0.5)] .4875 .21531 1 0 Mshowa
.5 .40272 m
.50625 .40272 L
s
[(0.5)] .4875 .40272 1 0 Mshowa
.5 .49642 m
.50625 .49642 L
s
[(1)] .4875 .49642 1 0 Mshowa
.5 .59013 m
.50625 .59013 L
s
[(1.5)] .4875 .59013 1 0 Mshowa
.125 Mabswid
.5 .04665 m
.50375 .04665 L
s
.5 .06539 m
.50375 .06539 L
s
.5 .08413 m
.50375 .08413 L
s
.5 .10287 m
.50375 .10287 L
s
.5 .14035 m
.50375 .14035 L
s
.5 .15909 m
.50375 .15909 L
s
.5 .17783 m
.50375 .17783 L
s
.5 .19657 m
.50375 .19657 L
s
.5 .23405 m
.50375 .23405 L
s
.5 .25279 m
.50375 .25279 L
s
.5 .27154 m
.50375 .27154 L
s
.5 .29028 m
.50375 .29028 L
s
.5 .32776 m
.50375 .32776 L
s
.5 .3465 m
.50375 .3465 L
s
.5 .36524 m
.50375 .36524 L
s
.5 .38398 m
.50375 .38398 L
s
.5 .42146 m
.50375 .42146 L
s
.5 .4402 m
.50375 .4402 L
s
.5 .45894 m
.50375 .45894 L
s
.5 .47768 m
.50375 .47768 L
s
.5 .51516 m
.50375 .51516 L
s
.5 .53391 m
.50375 .53391 L
s
.5 .55265 m
.50375 .55265 L
s
.5 .57139 m
.50375 .57139 L
s
.5 .00917 m
.50375 .00917 L
s
.5 .60887 m
.50375 .60887 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .02781 L
.02605 .03284 L
.02846 .04084 L
.03279 .05108 L
.04262 .06749 L
.05281 .08038 L
.06244 .09065 L
.10458 .1254 L
.14299 .15017 L
.18386 .17296 L
.22321 .19277 L
.26105 .2105 L
.30133 .22836 L
.34011 .24484 L
.38133 .26182 L
.42104 .2778 L
.45923 .29295 L
.49988 .30897 L
.53901 .32439 L
.58059 .34089 L
.62066 .35702 L
.65921 .3729 L
.70021 .39034 L
.7397 .40787 L
.77767 .42569 L
.81809 .44611 L
.857 .46787 L
.89836 .49473 L
.91906 .51065 L
.93821 .52804 L
.95662 .54949 L
.96592 .56441 L
.96847 .56961 L
.97122 .57629 L
.97248 .57999 L
.97381 .58466 L
.97495 .58986 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{194, 119.813},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004S0000/a000`40O003h00OTWoo00<007ooOol0
SWoo002BOol00`00Oomoo`2>Ool0099oo`03001oogoo08ioo`00TWoo0P00Sgoo0009Ool00`00Oomo
o`26Ool00`00Oomoo`2>Ool000Uoo`03001oogoo08Ioo`03001oogoo08ioo`002Goo00<007ooOol0
JGoo1@001Goo0`001Goo10001goo00<007ooOol0SWoo0009Ool00`00Oomoo`1[Ool00`00Oomoo`05
Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`2>Ool000Uoo`03001oogoo06]o
o`03001oogoo00Eoo`<000Uoo`03001oogoo00Aoo`8008moo`002Goo00<007ooOol0Ggoo1`001Goo
00<007ooOol04Goo00<007ooOol017oo00<007ooOol0SWoo0009Ool00`00Oomoo`1[Ool00`00Oomo
o`0<Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`2>Ool000Uoo`03001oogoo06]oo`03
001oogoo00aoo`D000Moo`03001oogoo08ioo`002Woo00<007ooOol0JWoo00<007ooOol037oo00<0
07ooOol02Goo00<007ooOol0SWoo000:Ool00`00Oomoo`1XOol3000>Ool00`00Oomoo`09Ool00`00
Oomoo`2>Ool000]oo`03001oogoo06Qoo`8000ioo`D000Moo`8008moo`002goo00<007ooOol0Q7oo
00<007ooOol0SWoo000;Ool00`00Oomoo`24Ool00`00Oomoo`2>Ool000aoo`03001oogoo08=oo`03
001oogoo08ioo`003Goo00<007ooOol0PWoo00<007ooOol0SWoo000=Ool00`00Oomoo`22Ool2002?
Ool000ioo`03001oogoo085oo`03001oogoo08ioo`003goo00<007ooOol0P7oo00<007ooOol0SWoo
000@Ool00`00Oomoo`1oOol00`00Oomoo`2>Ool0015oo`03001oogoo07ioo`03001oogoo08ioo`00
4Woo00<007ooOol0OGoo0P00Sgoo000COol00`00Oomoo`1lOol00`00Oomoo`2>Ool001Aoo`8007ao
o`03001oogoo08ioo`005Woo00<007ooOol0NGoo00<007ooOol0SWoo000GOol00`00Oomoo`1hOol0
0`00Oomoo`2>Ool001Qoo`03001oogoo07Moo`03001oogoo08ioo`006Goo00<007ooOol0MWoo0P00
Sgoo000JOol2001fOol00`00Oomoo`2>Ool001aoo`03001oogoo07=oo`03001oogoo08ioo`007Goo
00<007ooOol0Igoo1@001Woo00<007ooOol0SWoo000NOol00`00Oomoo`1XOol00`00Oomoo`06Ool0
0`00Oomoo`2>Ool001moo`03001oogoo06Moo`03001oogoo00Ioo`8008moo`0087oo0P00Fgoo1`00
1Goo00<007ooOol01Woo00<007ooOol0SWoo000ROol00`00Oomoo`1TOol00`00Oomoo`06Ool00`00
Oomoo`2>Ool002=oo`8006Aoo`03001oogoo00Ioo`03001oogoo08ioo`009Goo0P00HWoo00<007oo
Ool01Woo00<007ooOol0SWoo000WOol00`00Oomoo`1MOol30008Ool00`00Oomoo`2>Ool002Qoo`80
05ioo`8000Qoo`8008moo`00:Woo00<007ooOol0IGoo00<007ooOol0SWoo000[Ool2001UOol00`00
Oomoo`2>Ool002eoo`8006=oo`03001oogoo08ioo`00;goo00<007ooOol0H7oo00<007ooOol0SWoo
000`Ool2001POol2002?Ool0039oo`8005ioo`03001oogoo08ioo`00=7oo00<007ooOol0Fgoo00<0
07ooOol0SWoo000eOol2001KOol00`00Oomoo`2>Ool003Moo`8005Uoo`03001oogoo08ioo`00>Goo
0P00Egoo0P00Sgoo000kOol2001EOol00`00Oomoo`2>Ool003eoo`03001oogoo059oo`03001oogoo
08ioo`00?Woo0P00DWoo00<007ooOol0SWoo0010Ool2001@Ool00`00Oomoo`2>Ool0049oo`8004io
o`03001oogoo08ioo`00A7oo0P00C7oo0P00Sgoo0016Ool30019Ool00`00Oomoo`2>Ool004Uoo`80
04Moo`03001oogoo08ioo`00Bgoo0P00:Goo0P001goo0`001Goo10001goo00<007ooOol0SWoo001=
Ool2000VOol01000Oomoo`001Woo0`0017oo00<007ooOol00Woo00<007ooOol017oo00<007ooOol0
SWoo001?Ool2000SOol00`00Oomoo`02Ool00`00Oomoo`03Ool30009Ool00`00Oomoo`04Ool2002?
Ool0055oo`<001Moo`L0009oo`03001oogoo009oo`03001oogoo00moo`03001oogoo00Aoo`03001o
ogoo08ioo`00E7oo0P007Woo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol00Woo00<007oo
Ool017oo00<007ooOol0SWoo001FOol2000LOol00`00Oomoo`02Ool00`00Oomoo`0:Ool50007Ool0
0`00Oomoo`2>Ool005Qoo`8001Yoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo00Uoo`03
001oogoo08ioo`00FWoo0`0067oo00@007ooOol000eoo`03001oogoo00Uoo`03001oogoo08ioo`00
GGoo0P005goo0P003Woo1@001goo0P00Sgoo001OOol2000aOol00`00Oomoo`2>Ool0065oo`<002io
o`03001oogoo08ioo`00I7oo0P00;7oo00<007ooOol0SWoo001VOol2000ZOol00`00Oomoo`2>Ool0
06Qoo`<002Moo`8008moo`00Jgoo0P009Goo00<007ooOol0SWoo001]Ool2000SOol00`00Oomoo`2>
Ool000]oo`D003Moo`8000Moo`<000Eoo`@001=oo`<0021oo`03001oogoo03Moo`8000Moo`<000Eo
o`@003Moo`D000Ioo`003Goo00<007ooOol0=Woo00@007ooOol000Ioo`<000Aoo`03001oogoo009o
o`03001oogoo01=oo`<001eoo`03001oogoo03Ioo`04001oogoo0006Ool30004Ool00`00Oomoo`02
Ool00`00Oomoo`0fOol00`00Oomoo`06Ool000eoo`03001oogoo03Eoo`03001oogoo009oo`03001o
ogoo00=oo`<000Uoo`03001oogoo01Ioo`<001Yoo`8003Ioo`03001oogoo009oo`03001oogoo00=o
o`<000Uoo`03001oogoo03Ioo`03001oogoo00Ioo`0000=oo`0000001@001Goo00<007ooOol0;7oo
1`000Woo00<007ooOol00Woo00<007ooOol03goo00<007ooOol06Goo0`005goo00<007ooOol0=Goo
00<007ooOol00Woo00<007ooOol03goo00<007ooOol0=Woo00<007ooOol01Woo000=Ool00`00Oomo
o`0eOol00`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`0LOol2000EOol0
0`00Oomoo`0eOol00`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`0fOol0
0`00Oomoo`06Ool000eoo`03001oogoo03Eoo`03001oogoo009oo`03001oogoo00Yoo`D0025oo`80
01=oo`03001oogoo03Eoo`03001oogoo009oo`03001oogoo00Yoo`D003Uoo`03001oogoo00Ioo`00
3Goo00<007ooOol0=Goo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol09Goo0`0047oo00<0
07ooOol0=Goo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol0>goo00<007ooOol01Woo000;
Ool3000hOol01000Oomoo`003Goo00<007ooOol0:7oo0P003Woo00<007ooOol0=Woo00@007ooOol0
00eoo`03001oogoo03Uoo`<000Qoo`0037oo0P00>Goo0P003Woo1@00:7oo0P0037oo0P00>7oo0P00
3Woo1@00>7oo0P0027oo0026Ool30009Ool00`00Oomoo`2>Ool008Uoo`<000Ioo`03001oogoo08io
o`00S7oo0`000goo00<007ooOol0SWoo002?Ool4002@Ool0009oool0028000002Goo00<007ooOol0
2Woo00<007ooOol02goo00<007ooOol02goo00<007ooOol02Woo00<007ooOol02goo00<007ooOol0
2goo00<007ooOol02Woo00<007ooOol02goo00<007ooOol02goo00<007ooOol02goo00@007oo0000
00Uoo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo
00]oo`03001oogoo00]oo`03001oogoo00Yoo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo
00Eoo`002Goo00<007ooOol0@Goo00<007ooOol0@Woo00@007ooOomoo`<003eoo`03001oogoo049o
o`03001oogoo00Eoo`00TWoo00<007ooOol017oo0P00R7oo002BOol00`00Oomoo`06Ool20026Ool0
099oo`03001oogoo00Qoo`<008=oo`00TWoo0P0037oo0`00P7oo002BOol00`00Oomoo`0>Ool3001m
Ool0099oo`03001oogoo015oo`<007Yoo`00TWoo00<007ooOol057oo0P00N7oo002BOol00`00Oomo
o`0FOol2001fOol0099oo`8001Uoo`<007=oo`00TWoo00<007ooOol06goo0P00LGoo002BOol00`00
Oomoo`0MOol2001_Ool0099oo`03001oogoo01moo`8006eoo`00TWoo00<007ooOol08Goo0P00Jgoo
002BOol2000TOol3001XOol0099oo`03001oogoo02Ioo`8006Ioo`00TWoo00<007ooOol0:7oo0P00
I7oo002BOol00`00Oomoo`0ZOol2001ROol0099oo`03001oogoo02aoo`<005moo`00TWoo00<007oo
Ool0;goo0P00GGoo002BOol2000bOol2001KOol0099oo`03001oogoo03=oo`<005Qoo`00TWoo00<0
07ooOol0=Woo0P00EWoo001fOol20007Ool30005Ool40007Ool00`00Oomoo`0hOol3001COol007Eo
o`04001oogoo0006Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`0kOol2001A
Ool007Aoo`03001oogoo009oo`03001oogoo00=oo`<000Uoo`03001oogoo00Aoo`8003ioo`8004mo
o`00M7oo00<007ooOol00Woo00<007ooOol03goo00<007ooOol017oo00<007ooOol0?goo0`00C7oo
001dOol00`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomo
o`12Ool2001:Ool007Aoo`03001oogoo009oo`03001oogoo00Yoo`D000Moo`03001oogoo04Aoo`80
04Qoo`00M7oo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol0AWoo0`00
AGoo001eOol01000Oomoo`003Goo00<007ooOol02Goo00<007ooOol0BGoo0P00@goo001fOol2000>
Ool50007Ool2001<Ool20011Ool0099oo`03001oogoo04eoo`8003moo`00TWoo00<007ooOol0Cgoo
0P00?Goo002BOol00`00Oomoo`1AOol2000kOol0099oo`03001oogoo05=oo`03001oogoo03Qoo`00
TWoo0P00EGoo0P00>7oo002BOol00`00Oomoo`1FOol2000fOol0099oo`03001oogoo05Qoo`8003Ao
o`00TWoo00<007ooOol0FWoo0P00<Woo002BOol00`00Oomoo`1LOol2000`Ool0099oo`8005moo`03
001oogoo02eoo`00TWoo00<007ooOol0Ggoo0P00;Goo002BOol00`00Oomoo`1QOol2000[Ool0099o
o`03001oogoo06=oo`8002Uoo`00TWoo00<007ooOol0IGoo00<007ooOol09Woo002BOol00`00Oomo
o`1VOol2000VOol0099oo`8006Uoo`03001oogoo02=oo`00TWoo00<007ooOol0JGoo0P008goo002B
Ool00`00Oomoo`1[Ool00`00Oomoo`0POol008Moo`D000Ioo`03001oogoo06aoo`80021oo`00RGoo
00<007ooOol01Woo00<007ooOol0KWoo00<007ooOol07Goo0029Ool00`00Oomoo`06Ool2001`Ool2
000MOol008Uoo`03001oogoo00Ioo`03001oogoo075oo`03001oogoo01Yoo`00RGoo00<007ooOol0
1Woo00<007ooOol0LWoo0P006Woo0029Ool00`00Oomoo`06Ool00`00Oomoo`1dOol00`00Oomoo`0G
Ool008Uoo`03001oogoo00Ioo`03001oogoo07Eoo`8001Moo`00Qgoo0`0027oo0P00N7oo00<007oo
Ool057oo0028Ool20008Ool00`00Oomoo`1hOol00`00Oomoo`0COol0099oo`03001oogoo07Uoo`03
001oogoo019oo`00TWoo00<007ooOol0NWoo00<007ooOol04Goo002BOol00`00Oomoo`1kOol00`00
Oomoo`0@Ool0099oo`03001oogoo07]oo`03001oogoo011oo`00TWoo0P00OGoo00<007ooOol03goo
002BOol00`00Oomoo`1mOol00`00Oomoo`0>Ool0099oo`03001oogoo07ioo`03001oogoo00eoo`00
TWoo00<007ooOol0Ogoo00<007ooOol037oo002BOol00`00Oomoo`20Ool00`00Oomoo`0;Ool0099o
o`80085oo`03001oogoo00]oo`00TWoo00<007ooOol0PGoo00<007ooOol02Woo002BOol00`00Oomo
o`22Ool00`00Oomoo`09Ool0099oo`03001oogoo089oo`03001oogoo00Uoo`00TWoo00<007ooOol0
Pgoo00<007ooOol027oo002BOol00`00Oomoo`23Ool00`00Oomoo`08Ool0099oo`8008Aoo`03001o
ogoo00Qoo`00TWoo00<007ooOol0Q7oo00<007ooOol01goo002BOol00`00Oomoo`25Ool00`00Oomo
o`06Ool007Eoo`D000Eoo`<000Eoo`@000Moo`03001oogoo08Eoo`03001oogoo00Ioo`00Mgoo00<0
07ooOol01Goo0`0017oo00<007ooOol00Woo00<007ooOol017oo00<007ooOol0QGoo00<007ooOol0
1Woo001gOol00`00Oomoo`05Ool30009Ool00`00Oomoo`04Ool20026Ool00`00Oomoo`06Ool007Mo
o`03001oogoo015oo`03001oogoo00Aoo`03001oogoo08Eoo`03001oogoo00Ioo`00Mgoo00<007oo
Ool037oo00<007ooOol00Woo00<007ooOol017oo00<007ooOol0QGoo00<007ooOol01Woo001gOol0
0`00Oomoo`0<Ool50007Ool00`00Oomoo`26Ool00`00Oomoo`05Ool007Moo`03001oogoo00aoo`03
001oogoo00Uoo`03001oogoo08Ioo`03001oogoo00Eoo`00MGoo0`003Woo00<007ooOol02Goo0P00
Sgoo001fOol2000>Ool50007Ool00`00Oomoo`2>Ool0099oo`03001oogoo08ioo`00TWoo00<007oo
Ool0SWoo0000\
\>"],
  ImageRangeCache->{{{0, 290}, {178.688, 0}} -> {-1.06567, -1.65620, 
  0.0109432, 0.027806}}]
}, Open  ]],

Cell[TextData[{
  "You will have already spotted that all ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s built-in functions begin with a capital letter. Names which can be \
thought of as composed of two or more pieces may have capitals in the middle. \
 This means that when naming your own functions you can use any name starting \
with lower-case letters and be sure that you are not duplicating any ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " built-in function. We'll come back to user-defined functions shortly. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Getting Information about Built-In Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "As you start to explore the capabilities of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " you will need to get information about the functions available.  There is \
the facility to get information about built-in functions from within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  To ask about a function just type a \"?\" before the function name."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["?Sqrt", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \("Sqrt[z] gives the square root of z."\)], "Print",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[TextData[{
  "The wild card \"*\" can also be used to obtain a list of functions, with a \
similar sequence in common.  So we can obtain a list of all built-in \
functions related to the trigonometric function ",
  StyleBox["Sin",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?*Sin*", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
ArcSin              SingleLetterItalics Sinh
ArcSinh             SingularityDepth    SinhIntegral
IncludeSingularTerm SingularValues      SinIntegral
Sin\
\>", "Print"]
}, Open  ]],

Cell[TextData[{
  "Many of the built-in functions that you will use have options associated \
with them.  These set default values for various variables associated with \
the execution of that function.  To see how the options are set for a \
function, ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " for example,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Options[Plot]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({AspectRatio \[Rule] 1\/GoldenRatio, Axes \[Rule] Automatic, 
      AxesLabel \[Rule] None, AxesOrigin \[Rule] Automatic, 
      AxesStyle \[Rule] Automatic, Background \[Rule] Automatic, 
      ColorOutput \[Rule] Automatic, Compiled \[Rule] True, 
      DefaultColor \[Rule] Automatic, Epilog \[Rule] {}, Frame \[Rule] False, 
      FrameLabel \[Rule] None, FrameStyle \[Rule] Automatic, 
      FrameTicks \[Rule] Automatic, GridLines \[Rule] None, 
      ImageSize \[Rule] Automatic, MaxBend \[Rule] 10.`, 
      PlotDivision \[Rule] 30.`, PlotLabel \[Rule] None, 
      PlotPoints \[Rule] 25, PlotRange \[Rule] Automatic, 
      PlotRegion \[Rule] Automatic, PlotStyle \[Rule] Automatic, 
      Prolog \[Rule] {}, RotateLabel \[Rule] True, Ticks \[Rule] Automatic, 
      DefaultFont \[RuleDelayed] $DefaultFont, 
      DisplayFunction \[RuleDelayed] $DisplayFunction, 
      FormatType \[RuleDelayed] $FormatType, 
      TextStyle \[RuleDelayed] $TextStyle}\)], "Output",
  FontSize->9]
}, Open  ]],

Cell[TextData[{
  "Changing any of these options can be done when executing the function by \
explicitly stating the desired option value as the final argument to the \
function.  For example, we can change the label points on the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-axis for a sine curve plot, and add suitable grid lines."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot[Sin[x],{x,0,4 Pi},
\t\tTicks -> {{0, Pi, 2Pi, 3Pi, 4Pi}, Automatic},
\t\tGridLines -> {Table[k*Pi, {k, 0, 4, 1/2}] ,None} ];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0757881 0.309012 0.294307 [
[.2619 .29651 -5 -12 ]
[.2619 .29651 5 0 ]
[.5 .29651 -9.3125 -12 ]
[.5 .29651 9.3125 0 ]
[.7381 .29651 -9.3125 -12 ]
[.7381 .29651 9.3125 0 ]
[.97619 .29651 -9.3125 -12 ]
[.97619 .29651 9.3125 0 ]
[.01131 .01471 -12 -4.5 ]
[.01131 .01471 0 4.5 ]
[.01131 .16186 -24 -4.5 ]
[.01131 .16186 0 4.5 ]
[.01131 .45617 -18 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 .5 r
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .61803 L
s
.14286 0 m
.14286 .61803 L
s
.2619 0 m
.2619 .61803 L
s
.38095 0 m
.38095 .61803 L
s
.5 0 m
.5 .61803 L
s
.61905 0 m
.61905 .61803 L
s
.7381 0 m
.7381 .61803 L
s
.85714 0 m
.85714 .61803 L
s
.97619 0 m
.97619 .61803 L
s
0 g
.2619 .30901 m
.2619 .31526 L
s
gsave
.2619 .29651 -66 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(p) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
grestore
.5 .30901 m
.5 .31526 L
s
gsave
.5 .29651 -70.3125 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(2) show
71.625000 13.000000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(p) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.7381 .30901 m
.7381 .31526 L
s
gsave
.7381 .29651 -70.3125 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(3) show
71.625000 13.000000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(p) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
.97619 .30901 m
.97619 .31526 L
s
gsave
.97619 .29651 -70.3125 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(4) show
71.625000 13.000000 moveto
%%IncludeResource: font Math1Mono
%%IncludeFont: Math1Mono
/Math1Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(p) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math1Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math1Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math1Mono
%%+ Courier
grestore
0 .30901 m
1 .30901 L
s
.02381 .01471 m
.03006 .01471 L
s
[(-1)] .01131 .01471 1 0 Mshowa
.02381 .16186 m
.03006 .16186 L
s
[(-0.5)] .01131 .16186 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(0.5)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(1)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04414 m
.02756 .04414 L
s
.02381 .07357 m
.02756 .07357 L
s
.02381 .103 m
.02756 .103 L
s
.02381 .13243 m
.02756 .13243 L
s
.02381 .19129 m
.02756 .19129 L
s
.02381 .22072 m
.02756 .22072 L
s
.02381 .25015 m
.02756 .25015 L
s
.02381 .27958 m
.02756 .27958 L
s
.02381 .33844 m
.02756 .33844 L
s
.02381 .36787 m
.02756 .36787 L
s
.02381 .3973 m
.02756 .3973 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30901 m
.06244 .45263 L
.08255 .51497 L
.0932 .54236 L
.10458 .56657 L
.11009 .57624 L
.11531 .58409 L
.12038 .59047 L
.12507 .59525 L
.12951 .59877 L
.13178 .60018 L
.13422 .60141 L
.13549 .60193 L
.13666 .60234 L
.13784 .60268 L
.13895 .60293 L
.14022 .60314 L
.1409 .60322 L
.14161 .60328 L
.1423 .60331 L
.14295 .60332 L
.14415 .60328 L
.14531 .60316 L
.14656 .60297 L
.14773 .60271 L
.14881 .60241 L
.15143 .60144 L
.15392 .60019 L
.15886 .59679 L
.16423 .59169 L
.17383 .57909 L
.1827 .56357 L
.20137 .51988 L
.22197 .45702 L
.26217 .30797 L
.30086 .16431 L
.32051 .10345 L
.3309 .07659 L
.342 .05273 L
.35275 .03484 L
.35784 .02829 L
.36253 .02336 L
.36724 .01951 L
.36972 .01793 L
.37235 .0166 L
.37463 .01573 L
.37573 .01541 L
.37677 .01515 L
.37799 .01493 L
.37911 .01479 L
Mistroke
.38031 .01472 L
.38162 .01472 L
.38227 .01475 L
.38296 .01481 L
.38419 .01497 L
.38551 .01524 L
.38695 .01563 L
.38955 .0166 L
.39197 .01781 L
.39649 .02087 L
.40134 .02529 L
.40698 .03189 L
.41232 .03955 L
.42232 .05747 L
.44227 .10589 L
.4607 .16316 L
.50002 .3091 L
.53782 .44987 L
.55884 .51523 L
.56803 .53913 L
.57808 .56136 L
.58722 .57774 L
.59696 .59091 L
.60236 .59621 L
.60726 .59976 L
.60962 .60104 L
.61086 .6016 L
.61218 .60211 L
.61333 .60248 L
.61459 .60281 L
.61574 .60304 L
.61682 .60319 L
.61808 .60329 L
.61871 .60332 L
.61941 .60332 L
.62065 .60325 L
.62181 .60312 L
.62248 .60302 L
.62318 .60288 L
.62443 .60258 L
.62578 .60216 L
.62725 .6016 L
.62973 .6004 L
.63238 .59878 L
.63715 .59497 L
.64214 .58976 L
.64686 .58372 L
.6557 .56958 L
.67568 .52489 L
.69413 .47033 L
Mistroke
.7335 .32685 L
.77136 .18396 L
.79241 .11569 L
.81166 .06613 L
.82118 .04723 L
.8262 .03889 L
.83155 .03132 L
.8367 .02535 L
.84136 .02107 L
.84361 .01939 L
.84604 .01786 L
.84835 .01668 L
.85045 .01585 L
.85159 .01549 L
.85283 .01518 L
.85414 .01494 L
.85538 .01479 L
.85653 .01472 L
.85776 .01472 L
.85881 .01478 L
.85997 .01491 L
.86063 .01502 L
.86132 .01515 L
.86256 .01546 L
.86535 .01643 L
.86777 .01759 L
.87041 .0192 L
.87569 .02347 L
.88133 .02956 L
.89148 .04439 L
.90173 .06419 L
.91284 .09067 L
.933 .15021 L
.97393 .30025 L
.97619 .30901 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{194, 119.813},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004S0000/a000`40O003h00Oogoo97oo003oOolT
Ool00?moobAoo`00ogoo97oo003oOolTOol00?moobAoo`00ogoo97oo003oOolTOol00?moobAoo`00
ogoo97oo003oOolTOol00?moobAoo`009goo00<007ooOol06Woo00<03gooOol06goo00<03gooOol0
6Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol0
6goo00<03gooOol037oo000MOol50005Ool00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0K
Ool00`0?Oomoo`0<Ool001moo`03001oogoo00Eoo`03001oogoo01Yoo`0300moogoo01]oo`0300mo
ogoo01Yoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo01]oo`0300mo
ogoo01]oo`0300moogoo00aoo`007goo00<007ooOol01Goo00<007ooOol06Woo00<03gooOol06goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol06Woo00<03gooOol06goo
00<03gooOol06goo00<03gooOol037oo000COol70005Ool00`00Oomoo`05Ool2000KOol00`0?Oomo
o`0KOol00`0?Oomoo`0HOol6000JOol00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`0IOol5
000KOol00`0?Oomoo`0<Ool001moo`03001oogoo00Eoo`03001oogoo01Yoo`0300moogoo01]oo`03
00moogoo01Moo`800004Ool03gooOol2000IOol00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomo
o`0HOol01000Oomoo`0?0Woo0P006Goo00<03gooOol037oo000OOol00`00Oomoo`05Ool00`00Oomo
o`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0FOol01@00Oomoogoo00l017oo00<007ooOol05Woo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol05goo00D007ooOomoo`0?00Aoo`03001oogoo01Io
o`0300moogoo00aoo`007goo00<007ooOol01Goo00<007ooOol06Woo00<03gooOol06goo00<03goo
Ool05Goo00<007ooOol00Woo00<03gooOol00goo00<007ooOol05Goo00<03gooOol06goo00<03goo
Ool06Woo00<03gooOol05Woo00<007ooOol00Woo00<03gooOol00Woo00<007ooOol05Woo00<03goo
Ool037oo000MOol30007Ool00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0DOol00`00Oomo
o`03Ool00`0?Oomoo`04Ool00`00Oomoo`0DOol00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomo
o`0EOol00`00Oomoo`03Ool00`0?Oomoo`03Ool00`00Oomoo`0EOol00`0?Oomoo`0<Ool001ioo`80
00Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo01=oo`03001oogoo00Aoo`0300moogoo
00Aoo`03001oogoo01Aoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo01Aoo`03001oogoo
00Aoo`0300moogoo00=oo`03001oogoo01Eoo`0300moogoo00aoo`009goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol04goo00<007ooOol017oo00<03gooOol01Goo00<007ooOol04goo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol04goo00<007ooOol01Goo00<03gooOol017oo00<0
07ooOol057oo00<03gooOol037oo000WOol2000KOol00`0?Oomoo`0KOol00`0?Oomoo`0BOol00`00
Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0COol00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?
Oomoo`0COol00`00Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0COol00`0?Oomoo`0<Ool002Mo
o`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo019oo`03001oogoo00Eoo`0300moogoo00Io
o`03001oogoo019oo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo019oo`03001oogoo00Io
o`0300moogoo00Eoo`03001oogoo01=oo`0300moogoo00aoo`009goo00<007ooOol06Woo00<03goo
Ool06goo00<03gooOol04Goo00<007ooOol01Woo00<03gooOol01goo00<007ooOol04Goo00<03goo
Ool06goo00<03gooOol06Woo00<03gooOol04Woo00<007ooOol01Woo00<03gooOol01Woo00<007oo
Ool04Woo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0AOol0
0`00Oomoo`06Ool00`0?Oomoo`07Ool00`00Oomoo`0AOol00`0?Oomoo`0KOol00`0?Oomoo`0JOol0
0`0?Oomoo`0AOol00`00Oomoo`07Ool00`0?Oomoo`07Ool00`00Oomoo`0AOol00`0?Oomoo`0<Ool0
02Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo011oo`03001oogoo00Moo`0300moogoo
00Qoo`03001oogoo011oo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo015oo`03001oogoo
00Moo`0300moogoo00Moo`03001oogoo015oo`0300moogoo00aoo`009goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol047oo00<007ooOol01goo00<03gooOol027oo00<007ooOol047oo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol04Goo00<007ooOol01goo00<03gooOol027oo00<0
07ooOol047oo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0?
Ool00`00Oomoo`08Ool00`0?Oomoo`09Ool00`00Oomoo`0?Ool00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`08Ool00`00Oomoo`0@Ool00`0?Oomoo`0<
Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00moo`03001oogoo00Qoo`0300mo
ogoo00Uoo`03001oogoo00moo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo011oo`03001o
ogoo00Qoo`0300moogoo00Uoo`03001oogoo00moo`0300moogoo00aoo`009goo0P006goo00<03goo
Ool06goo00<03gooOol03Woo00<007ooOol02Goo00<03gooOol02Goo00<007ooOol03goo00<03goo
Ool06goo00<03gooOol06Woo00<03gooOol03goo00<007ooOol02Goo00<03gooOol02Goo00<007oo
Ool03goo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0>Ool0
0`00Oomoo`09Ool00`0?Oomoo`0:Ool00`00Oomoo`0>Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol0
0`0?Oomoo`0?Ool00`00Oomoo`09Ool00`0?Oomoo`0:Ool00`00Oomoo`0>Ool00`0?Oomoo`0<Ool0
02Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00eoo`03001oogoo00Yoo`0300moogoo
00Yoo`03001oogoo00ioo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00moo`03001oogoo
00Uoo`0300moogoo00Yoo`03001oogoo00ioo`0300moogoo00aoo`009goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol03Goo00<007ooOol02Woo00<03gooOol02goo00<007ooOol03Goo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol03Woo00<007ooOol02Woo00<03gooOol02Woo00<0
07ooOol03Woo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0<
Ool00`00Oomoo`0;Ool00`0?Oomoo`0;Ool00`00Oomoo`0=Ool00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`0>Ool00`00Oomoo`0:Ool00`0?Oomoo`0;Ool00`00Oomoo`0=Ool00`0?Oomoo`0<
Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00aoo`03001oogoo00]oo`0300mo
ogoo00aoo`03001oogoo00aoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00eoo`03001o
ogoo00]oo`0300moogoo00]oo`03001oogoo00eoo`0300moogoo00aoo`009goo00<007ooOol06Woo
00<03gooOol06goo00<03gooOol037oo00<007ooOol02goo00<03gooOol037oo00<007ooOol037oo
00<03gooOol06goo00<03gooOol06Woo00<03gooOol03Goo00<007ooOol02goo00<03gooOol037oo
00<007ooOol037oo00<03gooOol037oo000WOol2000KOol00`0?Oomoo`0KOol00`0?Oomoo`0;Ool0
0`00Oomoo`0<Ool00`0?Oomoo`0<Ool00`00Oomoo`0<Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol0
0`0?Oomoo`0<Ool00`00Oomoo`0<Ool00`0?Oomoo`0<Ool00`00Oomoo`0<Ool00`0?Oomoo`0<Ool0
02Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00]oo`03001oogoo00aoo`0300moogoo
00eoo`03001oogoo00]oo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00aoo`03001oogoo
00aoo`0300moogoo00aoo`03001oogoo00aoo`0300moogoo00aoo`009goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol02goo00<007ooOol037oo00<03gooOol03Goo00<007ooOol02goo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol037oo00<007ooOol037oo00<03gooOol03Goo00<0
07ooOol02goo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0:
Ool00`00Oomoo`0=Ool00`0?Oomoo`0=Ool00`00Oomoo`0;Ool00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`0;Ool00`00Oomoo`0=Ool00`0?Oomoo`0=Ool00`00Oomoo`0;Ool00`0?Oomoo`0<
Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00Yoo`03001oogoo00eoo`0300mo
ogoo00ioo`03001oogoo00Yoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00]oo`03001o
ogoo00eoo`0300moogoo00eoo`03001oogoo00]oo`0300moogoo00aoo`009goo00<007ooOol06Woo
00<03gooOol06goo00<03gooOol02Woo00<007ooOol03Goo00<03gooOol03Woo00<007ooOol02Woo
00<03gooOol06goo00<03gooOol06Woo00<03gooOol02goo00<007ooOol03Goo00<03gooOol03Woo
00<007ooOol02Woo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomo
o`0:Ool00`00Oomoo`0=Ool00`0?Oomoo`0>Ool00`00Oomoo`0:Ool00`0?Oomoo`0KOol00`0?Oomo
o`0JOol00`0?Oomoo`0:Ool00`00Oomoo`0>Ool00`0?Oomoo`0>Ool00`00Oomoo`0:Ool00`0?Oomo
o`0<Ool002Moo`8001]oo`0300moogoo01]oo`0300moogoo00Uoo`03001oogoo00ioo`0300moogoo
00moo`03001oogoo00Uoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00Yoo`03001oogoo
00ioo`0300moogoo00ioo`03001oogoo00Yoo`0300moogoo00aoo`009goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol02Goo00<007ooOol03Woo00<03gooOol03goo00<007ooOol02Goo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol02Woo00<007ooOol03Woo00<03gooOol03goo00<0
07ooOol02Goo00<03gooOol037oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`09
Ool00`00Oomoo`0>Ool00`0?Oomoo`0?Ool00`00Oomoo`09Ool00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`09Ool00`00Oomoo`0?Ool00`0?Oomoo`0?Ool00`00Oomoo`09Ool00`0?Oomoo`0<
Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300moogoo00Qoo`03001oogoo00moo`0300mo
ogoo00moo`03001oogoo00Uoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo00Uoo`03001o
ogoo00moo`0300moogoo00moo`03001oogoo00Uoo`0300moogoo00aoo`009goo00<007ooOol06Woo
00<03gooOol06goo00<03gooOol027oo00<007ooOol03goo00<03gooOol047oo00<007ooOol027oo
00<03gooOol06goo00<03gooOol06Woo00<03gooOol02Goo00<007ooOol03goo00<03gooOol047oo
00<007ooOol027oo00<03gooOol037oo000<Ool20007Ool30005Ool40006Ool00`00Oomoo`0JOol0
0`0?Oomoo`0KOol00`0?Oomoo`08Ool00`00Oomoo`0?Ool00`0?Oomoo`0@Ool00`00Oomoo`08Ool0
0`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`08Ool00`00Oomoo`0@Ool00`0?Oomoo`0@Ool0
0`00Oomoo`08Ool00`0?Oomoo`0<Ool000]oo`04001oogoo0006Ool30004Ool00`00Oomoo`02Ool0
0`00Oomoo`03Ool00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`08Ool00`00Oomoo`0?Ool0
0`0?Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`08Ool0
0`00Oomoo`0@Ool00`0?Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`0<Ool000Yoo`03001oogoo
009oo`03001oogoo00=oo`<000Uoo`03001oogoo00=oo`03001oogoo01Yoo`0300moogoo01]oo`03
00moogoo00Moo`03001oogoo011oo`0300moogoo015oo`03001oogoo00Moo`0300moogoo01]oo`03
00moogoo01Yoo`0300moogoo00Qoo`03001oogoo011oo`0300moogoo011oo`03001oogoo00Qoo`03
00moogoo00aoo`0000=oo`0000001@000Woo00<007ooOol00Woo00<007ooOol03goo00<007ooOol0
0goo0P006goo00<03gooOol06goo00<03gooOol01goo00<007ooOol047oo00<03gooOol04Goo00<0
07ooOol01goo00<03gooOol06goo00<03gooOol06Woo00<03gooOol027oo00<007ooOol047oo00<0
3gooOol04Goo00<007ooOol01goo00<03gooOol037oo000:Ool00`00Oomoo`02Ool00`00Oomoo`0:
Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`07
Ool00`00Oomoo`0@Ool00`0?Oomoo`0AOol00`00Oomoo`07Ool00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`07Ool00`00Oomoo`0AOol00`0?Oomoo`0AOol00`00Oomoo`07Ool00`0?Oomoo`0<
Ool000Yoo`03001oogoo009oo`03001oogoo00Yoo`D000Ioo`03001oogoo01Yoo`0300moogoo01]o
o`0300moogoo00Moo`03001oogoo011oo`0300moogoo015oo`03001oogoo00Moo`0300moogoo01]o
o`0300moogoo01Yoo`0300moogoo00Moo`03001oogoo015oo`0300moogoo015oo`03001oogoo00Mo
o`0300moogoo00aoo`002Woo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol027oo00<007oo
Ool06Woo00<03gooOol06goo00<03gooOol01Woo00<007ooOol04Goo00<03gooOol04Woo00<007oo
Ool01Woo00<03gooOol06goo00<03gooOol06Woo00<03gooOol01goo00<007ooOol04Goo00<03goo
Ool04Goo00<007ooOol01goo00<03gooOol037oo000;Ool01000Oomoo`003Goo00<007ooOol027oo
00<007ooOol06Woo00<03gooOol06goo00<03gooOol01Woo00<007ooOol04Goo00<03gooOol04Woo
00<007ooOol01Woo00<03gooOol06goo00<03gooOol06Woo00<03gooOol01Woo00<007ooOol04Woo
00<03gooOol04Woo00<007ooOol01Woo00<03gooOol037oo000<Ool2000>Ool50006Ool00`00Oomo
o`0JOol00`0?Oomoo`0KOol00`0?Oomoo`06Ool00`00Oomoo`0AOol00`0?Oomoo`0BOol00`00Oomo
o`06Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`06Ool00`00Oomoo`0BOol00`0?Oomo
o`0BOol00`00Oomoo`06Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`03
00moogoo00Eoo`03001oogoo019oo`0300moogoo01=oo`03001oogoo00Eoo`0300moogoo01]oo`03
00moogoo01Yoo`0300moogoo00Ioo`03001oogoo019oo`0300moogoo019oo`03001oogoo00Ioo`03
00moogoo00aoo`009goo0P006goo00<03gooOol06goo00<03gooOol01Goo00<007ooOol04Woo00<0
3gooOol04goo00<007ooOol01Goo00<03gooOol06goo00<03gooOol06Woo00<03gooOol01Woo00<0
07ooOol04Woo00<03gooOol04goo00<007ooOol01Goo00<03gooOol037oo000WOol00`00Oomoo`0J
Ool00`0?Oomoo`0KOol00`0?Oomoo`05Ool00`00Oomoo`0BOol00`0?Oomoo`0COol00`00Oomoo`05
Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`05Ool00`00Oomoo`0COol00`0?Oomoo`0C
Ool00`00Oomoo`05Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300mo
ogoo00Eoo`03001oogoo019oo`0300moogoo01=oo`03001oogoo00Eoo`0300moogoo01]oo`0300mo
ogoo01Yoo`0300moogoo00Eoo`03001oogoo01=oo`0300moogoo01=oo`03001oogoo00Eoo`0300mo
ogoo00aoo`009goo00<007ooOol06Woo00<03gooOol06goo00<03gooOol017oo00<007ooOol04goo
00<03gooOol057oo00<007ooOol017oo00<03gooOol06goo00<03gooOol06Woo00<03gooOol01Goo
00<007ooOol04goo00<03gooOol04goo00<007ooOol01Goo00<03gooOol037oo000WOol00`00Oomo
o`0JOol00`0?Oomoo`0KOol00`0?Oomoo`04Ool00`00Oomoo`0COol00`0?Oomoo`0DOol00`00Oomo
o`04Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`04Ool00`00Oomoo`0DOol00`0?Oomo
o`0DOol00`00Oomoo`04Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`03
00moogoo00Aoo`03001oogoo01=oo`0300moogoo01Aoo`03001oogoo00Aoo`0300moogoo01]oo`03
00moogoo01Yoo`0300moogoo00Aoo`03001oogoo01Aoo`0300moogoo01Aoo`03001oogoo00Aoo`03
00moogoo00aoo`009goo00<007ooOol06Woo00<03gooOol06goo00<03gooOol017oo00<007ooOol0
4goo00<03gooOol057oo00<007ooOol017oo00<03gooOol06goo00<03gooOol06Woo00<03gooOol0
17oo00<007ooOol057oo00<03gooOol057oo00<007ooOol017oo00<03gooOol037oo000WOol2000K
Ool00`0?Oomoo`0KOol00`0?Oomoo`03Ool00`00Oomoo`0DOol00`0?Oomoo`0EOol00`00Oomoo`03
Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`04Ool00`00Oomoo`0DOol00`0?Oomoo`0D
Ool00`00Oomoo`04Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300mo
ogoo00=oo`03001oogoo01Aoo`0300moogoo01Eoo`03001oogoo00=oo`0300moogoo01]oo`0300mo
ogoo01Yoo`0300moogoo00=oo`03001oogoo01Eoo`0300moogoo01Eoo`03001oogoo00=oo`0300mo
ogoo00aoo`009goo00<007ooOol06Woo00<03gooOol06goo00<03gooOol00goo00<007ooOol057oo
00<03gooOol05Goo00<007ooOol00goo00<03gooOol06goo00<03gooOol06Woo00<03gooOol00goo
00<007ooOol05Goo00<03gooOol05Goo00<007ooOol00goo00<03gooOol037oo000WOol00`00Oomo
o`0JOol00`0?Oomoo`0KOol00`0?Oomoo`02Ool00`00Oomoo`0EOol00`0?Oomoo`0FOol00`00Oomo
o`02Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`03Ool00`00Oomoo`0EOol00`0?Oomo
o`0EOol00`00Oomoo`03Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`03
00moogoo009oo`03001oogoo01Eoo`0300moogoo01Ioo`03001oogoo009oo`0300moogoo01]oo`03
00moogoo01Yoo`0300moogoo00=oo`03001oogoo01Eoo`0300moogoo01Ioo`03001oogoo009oo`03
00moogoo00aoo`009goo00<007ooOol06Woo00<03gooOol06goo00<03gooOol00Woo00<007ooOol0
5Goo00<03gooOol05Woo00<007ooOol00Woo00<03gooOol06goo00<03gooOol06Woo00<03gooOol0
0Woo00<007ooOol05Woo00<03gooOol05Woo00<007ooOol00Woo00<03gooOol037oo000WOol00`00
Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`02Ool00`00Oomoo`0EOol00`0?Oomoo`0FOol00`00
Oomoo`02Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`02Ool00`00Oomoo`0FOol00`0?
Oomoo`0FOol00`00Oomoo`02Ool00`0?Oomoo`0<Ool002Moo`03001oogoo01Yoo`0300moogoo01Io
o`03001oogoo009oo`0500moo`00Ool0000HOol00`0?Oomoo`0AOol70003Ool00`0?Ool00006Ool0
0`00Oomoo`0BOol00`0?Oomoo`0AOol40005Ool00`0?Ool00002Ool01@00Oomoogoo000057oo00<0
3gooOol057oo0`0017oo00<03goo00001Woo00<007ooOol00goo000WOol2000KOol00`0?Oomoo`0G
Ool01P00Oomoogoo00l0009oo`03001oogoo01Ioo`0300moogoo019oo`04001oogooOol20003Ool0
100?Oomoo`0017oo00<007ooOol04goo00<03gooOol047oo00<007ooOol00Woo00<007ooOol00Woo
00<03gooOol00P000goo00<007ooOol04goo00<03gooOol05Goo00<007oo00000goo00@03gooOol0
00Aoo`03001oogoo00Aoo`009goo00<007ooOol06Woo00<03gooOol067oo00D007ooOol03`00009o
o`03001oogoo01Ioo`0300moogoo01=oo`05001oogooOol00003Ool01@0?Oomoogoo00000goo00<0
07ooOol04goo00<03gooOol05Goo00<007ooOol00Woo00D03gooOomoo`0000=oo`03001oogoo01=o
o`0300moogoo015oo`L000=oo`0500moogooOol00003Ool00`00Oomoo`04Ool002Moo`03001oogoo
01Yoo`0300moogoo01Qoo`07001oogoo00l007oo00006Goo00<03gooOol057oo00D007ooOomoo`00
009oo`0500moogooOol00003Ool00`00Oomoo`0COol00`0?Oomoo`0EOol00`00Oomoo`02Ool01@0?
Oomoogoo00000goo00<007ooOol04goo00<03gooOol04Goo00L007ooOomoo`00Ool00003Ool01@0?
Oomoogoo00000goo00<007ooOol017oo000WOol00`00Oomoo`0JOol00`0?Oomoo`0IOol01P00Ool0
3`00Ool001Uoo`0300moogoo01Eoo`04001oogoo0002Ool00`0?Oomoo`02Ool01000Oomoo`005Goo
00<03gooOol04Woo0`001Goo00@03gooOomoo`80009oo`03001oogoo01=oo`0300moogoo019oo`06
001oogoo001oo`000goo00<03gooOol00Woo00@007ooOol000Ioo`009goo00<007ooOol06Woo00<0
3gooOol06Goo00@007oo00moo`8001Uoo`0300moogoo01Ioo`03001oo`00009oo`0300moogoo009o
o`05001oogooOol0000DOol00`0?Oomoo`0EOol00`00Oomoo`02Ool01P0?Oomoo`00Ool000=oo`03
001oogoo019oo`0300moogoo01=oo`03001oo`00009oo`04001oogoo00l4Ool01@00Oomoogoo0000
1Goo000WOol00`00Oomoo`0JOol00`0?Oomoo`0GOol8000IOol00`0?Oomoo`0FOol00`00Ool00002
Ool00`0?Oomoo`08000COol00`0?Oomoo`0EOol00`00Oomoo`02Ool00`0?Oomoo`08000COol00`0?
Oomoo`0COol00`00Ool00002Ool01000Oomoo`0?0Woo200017oo000WOol00`00Oomoo`0JOol00`0?
Oomoo`0KOol00`0?Ool0000JOol00`0?Oomoo`0AOol00`00Oomoo`02Ool01P00Oomoo`00Ool03aeo
o`0300moogoo011oo`03001oogoo009oo`03001oogoo009oo`0400moogoo000JOol00`0?Oomoo`0D
Ool20002Ool01000Oomoo`0?3Woo000WOol2000KOol00`0?Oomoo`0KOol00`0?Ool0000JOol00`0?
Oomoo`0BOol40003Ool00`00Ool03`0MOol00`0?Oomoo`0AOol40005Ool00`0?Ool0000KOol00`0?
Oomoo`0EOol01`00Oomoogoo001oo`0?00ioo`009goo00<007ooOol06Woo00<03gooOol06goo00<0
3goo00006Woo00<03gooOol06Goo00<007oo00l07Goo00<03gooOol06Woo00<03goo00006goo00<0
3gooOol06Goo00<007oo00l03Woo000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?Ool0000J
Ool00`0?Oomoo`0IOol00`00Ool03`0MOol00`0?Oomoo`0JOol00`0?Ool0000KOol00`0?Oomoo`0I
Ool00`00Ool03`0>Ool002Moo`03001oogoo01Yoo`0300moogoo01]oo`0300l007oo01Yoo`0300mo
ogoo01Yoo`0300003goo01aoo`0300moogoo01Yoo`0300moo`0001]oo`0300moogoo01Uoo`03001o
o`0?00ioo`009goo00<007ooOol06Woo00<03gooOol06goo00<03`00Ool06Woo00<03gooOol06Woo
00<0000?Ool077oo00<03gooOol06Woo00<03`00Ool06goo00<03gooOol06Woo00<0000?Ool03Goo
000WOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`0?001oo`0JOol00`0?Oomoo`0JOol00`0000mo
o`0LOol00`0?Oomoo`0JOol00`0?001oo`0KOol00`0?Oomoo`0JOol00`0000moo`0=Ool002Moo`03
001oogoo01Yoo`0300moogoo01]oo`03001oogoo01Yoo`0300moogoo01]oo`03001oogoo01]oo`03
00moogoo01Yoo`0300l007oo01]oo`0300moogoo01Yoo`0300003goo00eoo`008Goon@002Goo000W
Ool00`00Oomoo`0JOol00`0?Oomoo`0KOol00`00Oomoo`0JOol00`0?Oomoo`0KOol00`00Oomoo`0K
Ool00`0?Oomoo`0JOol00`00Oomoo`0KOol00`0?Oomoo`0KOol00`00Oomoo`0<Ool002Moo`03001o
ogoo01Yoo`0300moogoo01]oo`03001oogoo01Yoo`0300moogoo01]oo`03001oogoo01]oo`0300mo
ogoo01Yoo`03001oogoo01]oo`0300moogoo01]oo`03001oogoo00aoo`009goo0P006goo00<03goo
Ool06Woo00<0000?Ool06goo00<03gooOol06goo00<03`00Ool06goo00<03gooOol06Woo00<007oo
Ool06goo00<03gooOol06goo00<03gooOol037oo000WOol2000KOol00`0?Oomoo`0JOol00`0000mo
o`0KOol00`0?Oomoo`0KOol00`0?001oo`0KOol00`0?Oomoo`0IOol00`0000moo`0LOol00`0?Oomo
o`0KOol00`0?Oomoo`0<Ool002Moo`8001]oo`0300moogoo01Yoo`0300003goo01]oo`0300moogoo
01]oo`0300l007oo01]oo`0300moogoo01Uoo`0300003goo01aoo`0300moogoo01]oo`0300moogoo
00aoo`009goo0P006goo00<03gooOol06Woo00<0000?Ool06goo00<03gooOol06goo00<03`00Ool0
6goo00<03gooOol06Goo00<0000?Ool077oo00<03gooOol06goo00<03gooOol037oo000WOol3000J
Ool00`0?Oomoo`0IOol00`00Ool03`0LOol00`0?Oomoo`0KOol00`0?Ool0000KOol00`0?Oomoo`0I
Ool00`0000moo`0LOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oo`0001Yoo`0300mo
ogoo01Uoo`03001oo`0?01aoo`0300moogoo01]oo`0300moo`0001]oo`0300moogoo01Qoo`03001o
o`0?01eoo`0300moogoo01]oo`0300moogoo00aoo`009goo00<007oo00006Woo00<03gooOol06Goo
00<007oo00l077oo00<03gooOol06goo00<03goo00006goo00<03gooOol067oo00<007oo00l07Goo
00<03gooOol06goo00<03gooOol037oo000WOol00`00Ool0000JOol00`0?Oomoo`0HOol01000Oomo
o`0?77oo00<03gooOol06goo00<03goo00006goo00<03gooOol067oo00<007oo00l07Goo00<03goo
Ool06goo00<03gooOol037oo000WOol01000Oomoo`006Goo00<03gooOol067oo00@007ooOol03aao
o`0300moogoo01]oo`0400moogoo000JOol00`0?Oomoo`0GOol01000Oomoo`0?7Goo00<03gooOol0
6goo00<03gooOol037oo000WOol01000Oomoo`006Goo00<03gooOol067oo00@007ooOol03aaoo`03
00moogoo01]oo`0400moogoo000JOol00`0?Oomoo`0GOol01000Oomoo`0?7Goo00<03gooOol06goo
00<03gooOol037oo000WOol01000Oomoo`006Goo00<03gooOol067oo00@007ooOol03aaoo`0300mo
ogoo01]oo`0400moogoo000JOol00`0?Oomoo`0GOol01000Oomoo`0?7Goo00<03gooOol06goo00<0
3gooOol037oo000WOol01000Oomoo`006Goo00<03gooOol05goo00D007ooOomoo`0?01aoo`0300mo
ogoo01]oo`0400moogoo000JOol00`0?Oomoo`0GOol01000Oomoo`0?7Goo00<03gooOol06goo00<0
3gooOol037oo000WOol20002Ool00`00Oomoo`0FOol00`0?Oomoo`0GOol01@00Oomoogoo00l077oo
00<03gooOol06goo00D03gooOomoo`0001Uoo`0300moogoo01Ioo`05001oogooOol03`0MOol00`0?
Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`05001oogooOol0000HOol00`0?Oomoo`0GOol01@00Oomo
ogoo00l077oo00<03gooOol06goo00D03gooOomoo`0001Uoo`0300moogoo01Ioo`05001oogooOol0
3`0MOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`05001oogooOol0000HOol00`0?Oomoo`0G
Ool01@00Oomoogoo00l077oo00<03gooOol06goo00D03gooOomoo`0001Uoo`0300moogoo01Ioo`05
001oogooOol03`0MOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`05001oogooOol0000HOol0
0`0?Oomoo`0FOol00`00Oomoo`02Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`02Ool0
0`00Oomoo`0FOol00`0?Oomoo`0EOol00`00Oomoo`02Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol0
0`0?Oomoo`0<Ool002Moo`03001oogoo009oo`03001oogoo01Eoo`0300moogoo01Ioo`03001oogoo
009oo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo009oo`03001oogoo01Ioo`0300moogoo
01Eoo`03001oogoo009oo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`009goo00<0
07ooOol00Woo00<007ooOol05Goo00<03gooOol05Woo00<007ooOol00Woo00<03gooOol06Woo00<0
3gooOol06goo00<03gooOol00Woo00<007ooOol05Woo00<03gooOol05Goo00<007ooOol00Woo00<0
3gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`02Ool00`00Oomoo`0E
Ool00`0?Oomoo`0EOol00`00Oomoo`03Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`02
Ool00`00Oomoo`0FOol00`0?Oomoo`0EOol00`00Oomoo`02Ool00`0?Oomoo`0KOol00`0?Oomoo`0K
Ool00`0?Oomoo`0<Ool002Moo`8000=oo`03001oogoo01Eoo`0300moogoo01Eoo`03001oogoo00=o
o`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00=oo`03001oogoo01Eoo`0300moogoo01Ao
o`03001oogoo00=oo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`009goo00<007oo
Ool00goo00<007ooOol057oo00<03gooOol05Goo00<007ooOol00goo00<03gooOol06Woo00<03goo
Ool06goo00<03gooOol00goo00<007ooOol05Goo00<03gooOol057oo00<007ooOol00goo00<03goo
Ool06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`03Ool00`00Oomoo`0DOol0
0`0?Oomoo`0EOol00`00Oomoo`03Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`03Ool0
0`00Oomoo`0EOol00`0?Oomoo`0DOol00`00Oomoo`03Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol0
0`0?Oomoo`0<Ool002Moo`03001oogoo00=oo`03001oogoo01Aoo`0300moogoo01Aoo`03001oogoo
00Aoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00=oo`03001oogoo01Eoo`0300moogoo
01=oo`03001oogoo00Aoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`009goo00<0
07ooOol00goo00<007ooOol057oo00<03gooOol057oo00<007ooOol017oo00<03gooOol06Woo00<0
3gooOol06goo00<03gooOol017oo00<007ooOol057oo00<03gooOol04goo00<007ooOol017oo00<0
3gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`04Ool00`00Oomoo`0C
Ool00`0?Oomoo`0DOol00`00Oomoo`04Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`04
Ool00`00Oomoo`0DOol00`0?Oomoo`0COol00`00Oomoo`04Ool00`0?Oomoo`0KOol00`0?Oomoo`0K
Ool00`0?Oomoo`0<Ool002Moo`03001oogoo00Aoo`03001oogoo01=oo`0300moogoo01Aoo`03001o
ogoo00Aoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00Aoo`03001oogoo01Aoo`0300mo
ogoo01=oo`03001oogoo00Aoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`009goo
0P001Goo00<007ooOol04goo00<03gooOol04goo00<007ooOol01Goo00<03gooOol06Woo00<03goo
Ool06goo00<03gooOol017oo00<007ooOol057oo00<03gooOol04Woo00<007ooOol01Goo00<03goo
Ool06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`04Ool00`00Oomoo`0COol0
0`0?Oomoo`0COol00`00Oomoo`05Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`05Ool0
0`00Oomoo`0COol00`0?Oomoo`0BOol00`00Oomoo`05Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol0
0`0?Oomoo`0<Ool002Moo`03001oogoo00Eoo`03001oogoo019oo`0300moogoo01=oo`03001oogoo
00Eoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00Eoo`03001oogoo01=oo`0300moogoo
019oo`03001oogoo00Eoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`009goo00<0
07ooOol01Goo00<007ooOol04Woo00<03gooOol04Woo00<007ooOol01Woo00<03gooOol06Woo00<0
3gooOol06goo00<03gooOol01Goo00<007ooOol04goo00<03gooOol04Goo00<007ooOol01Woo00<0
3gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`05Ool00`00Oomoo`0B
Ool00`0?Oomoo`0BOol00`00Oomoo`06Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`05
Ool00`00Oomoo`0COol00`0?Oomoo`0AOol00`00Oomoo`06Ool00`0?Oomoo`0KOol00`0?Oomoo`0K
Ool00`0?Oomoo`0<Ool000aoo`8000Moo`<000Eoo`@000Ioo`03001oogoo00Eoo`03001oogoo019o
o`0300moogoo019oo`03001oogoo00Ioo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00Io
o`03001oogoo019oo`0300moogoo015oo`03001oogoo00Ioo`0300moogoo01]oo`0300moogoo01]o
o`0300moogoo00aoo`002goo00@007ooOol000Ioo`<000Aoo`03001oogoo009oo`03001oogoo00=o
o`03001oogoo00Ioo`03001oogoo015oo`0300moogoo019oo`03001oogoo00Ioo`0300moogoo01Yo
o`0300moogoo01]oo`0300moogoo00Ioo`03001oogoo019oo`0300moogoo015oo`03001oogoo00Io
o`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`002Woo00<007ooOol00Woo00<007oo
Ool00goo0`002Goo00<007ooOol00goo00<007ooOol01Woo00<007ooOol04Goo00<03gooOol04Goo
00<007ooOol01goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol01Woo00<007ooOol04Woo
00<03gooOol047oo00<007ooOol01goo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo
000:Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool00`00Oomoo`03Ool20007Ool00`00Oomoo`0AOol0
0`0?Oomoo`0AOol00`00Oomoo`07Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`07Ool0
0`00Oomoo`0AOol00`0?Oomoo`0@Ool00`00Oomoo`07Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol0
0`0?Oomoo`0<Ool000Yoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo
00=oo`03001oogoo00Moo`03001oogoo011oo`0300moogoo015oo`03001oogoo00Moo`0300moogoo
01Yoo`0300moogoo01]oo`0300moogoo00Moo`03001oogoo015oo`0300moogoo011oo`03001oogoo
00Moo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`002Woo00<007ooOol00Woo00<0
07ooOol02Woo1@001Woo00<007ooOol01goo00<007ooOol047oo00<03gooOol04Goo00<007ooOol0
1goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol01goo00<007ooOol04Goo00<03gooOol0
3goo00<007ooOol027oo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000:Ool00`00
Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`0@Ool00`0?
Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`08Ool00`00
Oomoo`0@Ool00`0?Oomoo`0?Ool00`00Oomoo`08Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol00`0?
Oomoo`0<Ool000]oo`04001oogoo000=Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`0?
Ool00`0?Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`08
Ool00`00Oomoo`0@Ool00`0?Oomoo`0?Ool00`00Oomoo`08Ool00`0?Oomoo`0KOol00`0?Oomoo`0K
Ool00`0?Oomoo`0<Ool000aoo`8000ioo`D000Ioo`03001oogoo00Qoo`03001oogoo00moo`0300mo
ogoo011oo`03001oogoo00Qoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00Qoo`03001o
ogoo011oo`0300moogoo00moo`03001oogoo00Qoo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo00aoo`009goo00<007ooOol027oo00<007ooOol03goo00<03gooOol03goo00<007ooOol02Goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol02Goo00<007ooOol03goo00<03gooOol03Woo
00<007ooOol02Goo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol2000:Ool0
0`00Oomoo`0>Ool00`0?Oomoo`0?Ool00`00Oomoo`09Ool00`0?Oomoo`0JOol00`0?Oomoo`0KOol0
0`0?Oomoo`09Ool00`00Oomoo`0?Ool00`0?Oomoo`0>Ool00`00Oomoo`09Ool00`0?Oomoo`0KOol0
0`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo00Uoo`03001oogoo00ioo`0300moogoo
00moo`03001oogoo00Uoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00Yoo`03001oogoo
00ioo`0300moogoo00ioo`03001oogoo00Uoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo
00aoo`009goo00<007ooOol02Goo00<007ooOol03Woo00<03gooOol03Woo00<007ooOol02Woo00<0
3gooOol06Woo00<03gooOol06goo00<03gooOol02Woo00<007ooOol03Woo00<03gooOol03Goo00<0
07ooOol02Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`0:
Ool00`00Oomoo`0=Ool00`0?Oomoo`0>Ool00`00Oomoo`0:Ool00`0?Oomoo`0JOol00`0?Oomoo`0K
Ool00`0?Oomoo`0:Ool00`00Oomoo`0>Ool00`0?Oomoo`0=Ool00`00Oomoo`0:Ool00`0?Oomoo`0K
Ool00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo00Yoo`03001oogoo00eoo`0300mo
ogoo00ioo`03001oogoo00Yoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00]oo`03001o
ogoo00eoo`0300moogoo00eoo`03001oogoo00Yoo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo00aoo`009goo00<007ooOol02Woo00<007ooOol03Goo00<03gooOol03Goo00<007ooOol02goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol02goo00<007ooOol03Goo00<03gooOol03Goo
00<007ooOol02Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomo
o`0;Ool00`00Oomoo`0<Ool00`0?Oomoo`0=Ool00`00Oomoo`0;Ool00`0?Oomoo`0JOol00`0?Oomo
o`0KOol00`0?Oomoo`0;Ool00`00Oomoo`0=Ool00`0?Oomoo`0<Ool00`00Oomoo`0;Ool00`0?Oomo
o`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`8000aoo`03001oogoo00aoo`0300moogoo
00eoo`03001oogoo00]oo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00aoo`03001oogoo
00aoo`0300moogoo00aoo`03001oogoo00]oo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo
00aoo`009goo00<007ooOol02goo00<007ooOol037oo00<03gooOol037oo00<007ooOol037oo00<0
3gooOol06Woo00<03gooOol06goo00<03gooOol037oo00<007ooOol037oo00<03gooOol037oo00<0
07ooOol02goo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomoo`0<
Ool00`00Oomoo`0;Ool00`0?Oomoo`0<Ool00`00Oomoo`0<Ool00`0?Oomoo`0JOol00`0?Oomoo`0K
Ool00`0?Oomoo`0=Ool00`00Oomoo`0;Ool00`0?Oomoo`0;Ool00`00Oomoo`0<Ool00`0?Oomoo`0K
Ool00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo00aoo`03001oogoo00]oo`0300mo
ogoo00aoo`03001oogoo00aoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00eoo`03001o
ogoo00]oo`0300moogoo00]oo`03001oogoo00aoo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo00aoo`009goo00<007ooOol03Goo00<007ooOol02Woo00<03gooOol02goo00<007ooOol03Goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol03Goo00<007ooOol02goo00<03gooOol02goo
00<007ooOol037oo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomo
o`0=Ool00`00Oomoo`0:Ool00`0?Oomoo`0;Ool00`00Oomoo`0=Ool00`0?Oomoo`0JOol00`0?Oomo
o`0KOol00`0?Oomoo`0>Ool00`00Oomoo`0:Ool00`0?Oomoo`0:Ool00`00Oomoo`0=Ool00`0?Oomo
o`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo00eoo`03001oogoo00Yoo`03
00moogoo00Yoo`03001oogoo00ioo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00ioo`03
001oogoo00Yoo`0300moogoo00Yoo`03001oogoo00eoo`0300moogoo01]oo`0300moogoo01]oo`03
00moogoo00aoo`009goo00<007ooOol03Woo00<007ooOol02Goo00<03gooOol02Woo00<007ooOol0
3Woo00<03gooOol06Woo00<03gooOol06goo00<03gooOol03Woo00<007ooOol02Woo00<03gooOol0
2Goo00<007ooOol03Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol2000?
Ool00`00Oomoo`09Ool00`0?Oomoo`09Ool00`00Oomoo`0?Ool00`0?Oomoo`0JOol00`0?Oomoo`0K
Ool00`0?Oomoo`0?Ool00`00Oomoo`09Ool00`0?Oomoo`09Ool00`00Oomoo`0>Ool00`0?Oomoo`0K
Ool00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo00moo`03001oogoo00Qoo`0300mo
ogoo00Uoo`03001oogoo00moo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo00moo`03001o
ogoo00Uoo`0300moogoo00Qoo`03001oogoo00moo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo00aoo`009goo00<007ooOol03goo00<007ooOol027oo00<03gooOol027oo00<007ooOol047oo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol03goo00<007ooOol02Goo00<03gooOol027oo
00<007ooOol03goo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomo
o`0@Ool00`00Oomoo`07Ool00`0?Oomoo`08Ool00`00Oomoo`0@Ool00`0?Oomoo`0JOol00`0?Oomo
o`0KOol00`0?Oomoo`0@Ool00`00Oomoo`08Ool00`0?Oomoo`08Ool00`00Oomoo`0?Ool00`0?Oomo
o`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo011oo`03001oogoo00Moo`03
00moogoo00Moo`03001oogoo015oo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo011oo`03
001oogoo00Qoo`0300moogoo00Moo`03001oogoo011oo`0300moogoo01]oo`0300moogoo01]oo`03
00moogoo00aoo`009goo00<007ooOol04Goo00<007ooOol01Woo00<03gooOol01goo00<007ooOol0
4Goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol047oo00<007ooOol027oo00<03gooOol0
1goo00<007ooOol047oo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00
Oomoo`0AOol00`00Oomoo`06Ool00`0?Oomoo`06Ool00`00Oomoo`0BOol00`0?Oomoo`0JOol00`0?
Oomoo`0KOol00`0?Oomoo`0AOol00`00Oomoo`07Ool00`0?Oomoo`06Ool00`00Oomoo`0AOol00`0?
Oomoo`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`8001=oo`03001oogoo00Eoo`0300mo
ogoo00Ioo`03001oogoo019oo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo019oo`03001o
ogoo00Ioo`0300moogoo00Ioo`03001oogoo015oo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo00aoo`009goo00<007ooOol04Woo00<007ooOol01Goo00<03gooOol01Goo00<007ooOol04goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol04goo00<007ooOol01Goo00<03gooOol01Goo
00<007ooOol04Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000WOol00`00Oomo
o`0BOol00`00Oomoo`05Ool00`0?Oomoo`05Ool00`00Oomoo`0COol00`0?Oomoo`0JOol00`0?Oomo
o`0KOol00`0?Oomoo`0COol00`00Oomoo`05Ool00`0?Oomoo`04Ool00`00Oomoo`0COol00`0?Oomo
o`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool002Moo`03001oogoo01=oo`8000Eoo`0300moogoo
00Aoo`03001oogoo01Aoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo01Aoo`03001oogoo
00Aoo`0300moogoo00=oo`03001oogoo01Aoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo
00aoo`009goo00<007ooOol05Goo00<007ooOol00Woo00<03gooOol00goo00<007ooOol05Goo00<0
3gooOol06Woo00<03gooOol06goo00<03gooOol05Goo00<007ooOol00goo00<03gooOol00Woo00<0
07ooOol05Goo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000MOol50005Ool00`00
Oomoo`0FOol01@00Oomoogoo00l017oo00<007ooOol05Woo00<03gooOol06Woo00<03gooOol06goo
00<03gooOol05Woo00<007ooOol00Woo00D03gooOomoo`0001Qoo`0300moogoo01]oo`0300moogoo
01]oo`0300moogoo00aoo`007goo00<007ooOol01Goo00<007ooOol05goo00@007ooOol03`=oo`03
001oogoo01Moo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo01Moo`80009oo`0400moogoo
000IOol00`0?Oomoo`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool001moo`03001oogoo00Eoo`80
01Qoo`L001Yoo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo01Qoo`H001Yoo`0300moogoo
01]oo`0300moogoo01]oo`0300moogoo00aoo`007goo00<007ooOol01Goo00<007ooOol06Woo00<0
3gooOol06goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol06Woo00<0
3gooOol06goo00<03gooOol06goo00<03gooOol037oo000OOol00`00Oomoo`05Ool00`00Oomoo`0J
Ool00`0?Oomoo`0KOol00`0?Oomoo`0JOol00`0?Oomoo`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0J
Ool00`0?Oomoo`0KOol00`0?Oomoo`0KOol00`0?Oomoo`0<Ool001moo`03001oogoo00Eoo`03001o
ogoo01Yoo`0300moogoo01]oo`0300moogoo01Yoo`0300moogoo01]oo`0300moogoo01]oo`0300mo
ogoo01Yoo`0300moogoo01]oo`0300moogoo01]oo`0300moogoo00aoo`007goo00<007ooOol01Goo
00<007ooOol06Woo00<03gooOol06goo00<03gooOol06Woo00<03gooOol06goo00<03gooOol06goo
00<03gooOol06Woo00<03gooOol06goo00<03gooOol06goo00<03gooOol037oo000MOol3003oOol4
Ool001ioo`800?moo`Aoo`00ogoo97oo003oOolTOol00?moobAoo`00ogoo97oo003oOolTOol00?mo
obAoo`00ogoo97oo003oOolTOol00?moobAoo`00ogoo97oo0000\
\>"],
  ImageRangeCache->{{{0, 290}, {178.688, 0}} -> {-2.07445, -1.22031, 
  0.079561, 0.0204880}}]
}, Open  ]],

Cell[TextData[{
  "Note that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " allows a slight abuse of notation when it comes to pre-multiplying a \
symbol by a number. In the above,  ",
  StyleBox["2Pi ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "is interpreted as twice \[Pi], even in the absence of a star or space. \
Options may also be used to cure problems with functions such as ",
  StyleBox["NIntegrate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[NIntegrate]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"AccuracyGoal", "\[Rule]", 
          InterpretationBox["\[Infinity]",
            DirectedInfinity[ 1]]}], ",", \(Compiled \[Rule] True\), ",", 
        \(GaussPoints \[Rule] Automatic\), ",", 
        \(MaxPoints \[Rule] Automatic\), ",", \(MaxRecursion \[Rule] 6\), 
        ",", \(Method \[Rule] Automatic\), ",", \(MinRecursion \[Rule] 0\), 
        ",", \(PrecisionGoal \[Rule] Automatic\), ",", 
        \(SingularityDepth \[Rule] 4\), ",", 
        \(WorkingPrecision \[Rule] 16\)}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["NIntegrate[Sin[1000 x],{x,0,2}, MaxRecursion ->9]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.00136745954910141964`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Built-In Special Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell["Examples of Special Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " also knows a phenomenal number of special functions. If you do the kind \
of work that employs handbooks of functions and their values and properties, \
you may be surprised by the breadth of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s built-in capabilities. As well as a host of number-theoretic and \
combinatorial functions, there are many \"classical\" functions. Let us take \
a look at a few examples.  First of all, we can use Bessel functions:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot[{BesselJ[0, x], BesselI[1, x]}, {x, 0, 10},
PlotRange -> {-5, 5}, PlotStyle -> {Thickness[0.015], Thickness[0.01]}];\
\>",
   "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 0.309017 0.0618034 [
[.21429 .29652 -3 -9 ]
[.21429 .29652 3 0 ]
[.40476 .29652 -3 -9 ]
[.40476 .29652 3 0 ]
[.59524 .29652 -3 -9 ]
[.59524 .29652 3 0 ]
[.78571 .29652 -3 -9 ]
[.78571 .29652 3 0 ]
[.97619 .29652 -6 -9 ]
[.97619 .29652 6 0 ]
[.01131 .0618 -12 -4.5 ]
[.01131 .0618 0 4.5 ]
[.01131 .18541 -12 -4.5 ]
[.01131 .18541 0 4.5 ]
[.01131 .43262 -6 -4.5 ]
[.01131 .43262 0 4.5 ]
[.01131 .55623 -6 -4.5 ]
[.01131 .55623 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .30902 m
.21429 .31527 L
s
[(2)] .21429 .29652 0 1 Mshowa
.40476 .30902 m
.40476 .31527 L
s
[(4)] .40476 .29652 0 1 Mshowa
.59524 .30902 m
.59524 .31527 L
s
[(6)] .59524 .29652 0 1 Mshowa
.78571 .30902 m
.78571 .31527 L
s
[(8)] .78571 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(10)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.07143 .30902 m
.07143 .31277 L
s
.11905 .30902 m
.11905 .31277 L
s
.16667 .30902 m
.16667 .31277 L
s
.2619 .30902 m
.2619 .31277 L
s
.30952 .30902 m
.30952 .31277 L
s
.35714 .30902 m
.35714 .31277 L
s
.45238 .30902 m
.45238 .31277 L
s
.5 .30902 m
.5 .31277 L
s
.54762 .30902 m
.54762 .31277 L
s
.64286 .30902 m
.64286 .31277 L
s
.69048 .30902 m
.69048 .31277 L
s
.7381 .30902 m
.7381 .31277 L
s
.83333 .30902 m
.83333 .31277 L
s
.88095 .30902 m
.88095 .31277 L
s
.92857 .30902 m
.92857 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .0618 m
.03006 .0618 L
s
[(-4)] .01131 .0618 1 0 Mshowa
.02381 .18541 m
.03006 .18541 L
s
[(-2)] .01131 .18541 1 0 Mshowa
.02381 .43262 m
.03006 .43262 L
s
[(2)] .01131 .43262 1 0 Mshowa
.02381 .55623 m
.03006 .55623 L
s
[(4)] .01131 .55623 1 0 Mshowa
.125 Mabswid
.02381 .09271 m
.02756 .09271 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.015 w
.02381 .37082 m
.02499 .37082 L
.02605 .37081 L
.02729 .3708 L
.02846 .37078 L
.03053 .37074 L
.03279 .37068 L
.03527 .3706 L
.0379 .37048 L
.04262 .37022 L
.04749 .36987 L
.05205 .36947 L
.06244 .3683 L
.07305 .36676 L
.08274 .36504 L
.10458 .3602 L
.14429 .34846 L
.18248 .33482 L
.22313 .31957 L
.26226 .30591 L
.28223 .29985 L
.30384 .29424 L
.32471 .2899 L
.33464 .28825 L
.34391 .28695 L
.35375 .28584 L
.36273 .28507 L
.36785 .28473 L
.3727 .28448 L
.37518 .28438 L
.37782 .28429 L
.3792 .28425 L
.38072 .28421 L
.3821 .28419 L
.38339 .28416 L
.38453 .28415 L
.38574 .28414 L
.38643 .28413 L
.38709 .28413 L
.38831 .28413 L
.3896 .28413 L
.39079 .28413 L
.3921 .28414 L
.39276 .28415 L
.39349 .28416 L
.39579 .28419 L
.39827 .28425 L
.40274 .28439 L
.40833 .28464 L
.41364 .28495 L
Mistroke
.4236 .28573 L
.43321 .28669 L
.44344 .28794 L
.46184 .2907 L
.50348 .29878 L
.54359 .3077 L
.5822 .31588 L
.60178 .31947 L
.62325 .32277 L
.63313 .32403 L
.64371 .32518 L
.65363 .32605 L
.66279 .32669 L
.66764 .32696 L
.67299 .32719 L
.67837 .32738 L
.68131 .32745 L
.68407 .3275 L
.68654 .32754 L
.68778 .32755 L
.68849 .32755 L
.68913 .32756 L
.6903 .32756 L
.69154 .32756 L
.69268 .32756 L
.69377 .32756 L
.69504 .32756 L
.69567 .32755 L
.69638 .32755 L
.69878 .32752 L
.70013 .3275 L
.70158 .32747 L
.70421 .32741 L
.70948 .32725 L
.71436 .32706 L
.72542 .32645 L
.73517 .32573 L
.74553 .32478 L
.78325 .32 L
.82342 .31336 L
.86208 .30656 L
.90319 .30012 L
.92369 .29756 L
.94278 .29569 L
.95893 .29456 L
.96797 .29411 L
.97619 .29382 L
Mfstroke
.01 w
.02381 .30902 m
.06244 .32181 L
.10458 .33765 L
.14415 .35639 L
.18221 .38035 L
.20342 .39734 L
.22272 .41576 L
.26171 .46427 L
.28086 .49525 L
.30164 .53578 L
.32109 .5816 L
s
.32109 .5816 m
.33382 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{208, 128.5},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004h0000`1000`40O003h00Oogoo>Goo003oOoli
Ool00?moocUoo`00ogoo>Goo003oOoliOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0
ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0
ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0
ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`800?moob9oo`005Goo00<007ooOol0ogoo8Goo
000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0ogoo8Goo
000EOol00`00Oomoo`3oOolQOol000eoo`<000Eoo`03001oogoo0?moob5oo`003Woo00<007ooOol0
17oo00<007ooOol0ogoo8Goo000:Ool60005Ool00`00Oomoo`3oOolQOol00003Ool0000000D0009o
o`05001oogooOol00006Ool3003oOolQOol000]oo`04001oogoo0006Ool00`00Oomoo`3oOolQOol0
00aoo`03001oo`0000Ioo`03001oogoo0?moob5oo`0037oo00<007oo00001Woo00<007ooOol0ogoo
8Goo000=Ool20006Ool00`00Oomoo`3oOolQOol000ioo`03001oogoo00Aoo`03001oogoo0?moob5o
o`005Goo00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5o
o`005Goo0P00ogoo8Woo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo
00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo
00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo
0P00ogoo8Woo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007oo
Ool0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007oo
Ool0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo0P00ogoo
8Woo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0ogoo
8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007ooOol0ogoo
8Goo000:Ool60005Ool00`00Oomoo`3oOolQOol000]oo`05001oogooOol00005Ool00`00Oomoo`3o
OolQOol000aoo`03001oogoo00Ioo`<00?moob5oo`0000=oo`0000001@001Goo00<007ooOol01Goo
00<007ooOol0ogoo8Goo000>Ool00`00Oomoo`04Ool00`00Oomoo`3oOolQOol000moo`03001oogoo
00=oo`03001oogoo0?moob5oo`003goo00<007ooOol00goo00<007ooOol0ogoo8Goo000:Ool00`00
Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`3oOolQOol000]oo`@000Ioo`03001oogoo0?moob5o
o`005Goo00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`800?moob9oo`005Goo
00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo
00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo
00<007ooOol0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`800?moob9oo`005Goo00<007oo
Ool0ogoo8Goo000EOol00`00Oomoo`3oOolQOol001Eoo`03001oogoo0?moob5oo`005Goo00<007oo
Ool0ogoo8Goo000EOol00`00Oomoo`0bOol6000eOol3000cOol4000dOol4000`Ool50005Ool20005
Ool001Eoo`03001oogoo03=oo`05001oogooOol0000fOol00`00Oomoo`0aOol00`00Oomoo`02Ool0
0`00Oomoo`0`Ool00`00Oomoo`02Ool00`00Oomoo`0_Ool00`00Oomoo`04Ool01000Oomoo`0017oo
000EOol00`00Oomoo`0dOol00`00Oomoo`0cOol6000bOol00`00Oomoo`02Ool00`00Oomoo`0`Ool0
0`00Oomoo`02Ool00`00Oomoo`0_Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`01Ool0
01Eoo`03001oogoo03Eoo`03001oogoo039oo`05001oogooOol0000cOol00`00Oomoo`02Ool00`00
Oomoo`0`Ool00`00Oomoo`02Ool00`00Oomoo`0_Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00
Oomoo`01Ool001Eoo`03001oogoo03Ioo`03001oogoo039oo`04001oogoo000cOol20003Ool00`00
Oomoo`0aOol4000bOol00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`01Ool001Eoo`8003Qo
o`03001oogoo02Iooa40035oo`03001oo`00008003=oo`03001oogoo009oo`03001oogoo02moo`03
001oogoo00=oo`03001oogoo009oo`03001oogoo005oo`005Goo00<007ooOol0=goo00<007ooOol0
7Woo9P009Goo00<007ooOol0=7oo00<007ooOol00Woo00<007ooOol0;goo00<007ooOol00goo00<0
07ooOol00Woo00<007ooOol00Goo000EOol00`00Oomoo`0bOol00`00Oomoo`02Ool00`00Oomoo`0G
Oola000QOol2000eOol00`00Oomoo`02Ool00`00Oomoo`0]Ool30006Ool01000Oomoo`0017oo000E
Ool00`00Oomoo`0cOol4000EOolj000OOol3000cOol4000]Ool<00000goo00000005Ool001Eoo`03
001oogoo04UooaX000]ooa`007MooaL000Qoo`005Goo00<007ooOol0AWoo5@0087oo5000K7oo7@00
27oo000EOol00`00Oomoo`13OolA000[OolF001POolS0008Ool001Eoo`03001oogoo041oo`l003Ao
oaL005Eoob00015oo`005Goo00<007ooOol0?Goo3`00>goo5`00C7oo6P0077oo000>Oooo000Y0002
Ool001Aoo`P000Moo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oo`0000h0
00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo
00IooaD000aoo`03001oogoo00]oo`03001oogoo00]oo`03001oogoo00]ooa/00003Ool007oo00ao
o`03001oogoo00aoo`03001oogoo00Moo`005Goo00<007oo00002000;Goo3P00E7oo5P00:Goo7000
;Woo000EOol00`00Oomoo`03Ool8000WOol>001KOolM000GOolN000cOol001Eoo`03001oogoo00Mo
o`L0025oo`h0069oodP003Uoo`005Goo00<007ooOol02Woo1P0077oo3P00JWoo?@00?goo000EOol0
0`00Oomoo`0=Ool5000GOol?001bOol_0017Ool001Eoo`03001oogoo00moo`H0015oo`l0081ooad0
04ioo`005Goo00<007ooOol04Goo1P0037oo3`00kWoo000EOol2000EOol50007Ool?003aOol001Eo
o`03001oogoo01Ioo`D000=oo`h00?Aoo`005Goo00<007ooOol067oo4@00mgoo000EOol00`00Oomo
o`0HOol>003jOol001Eoo`03001oogoo01Eoo`h00?eoo`005Goo00<007ooOol047oo4`00oGoo000E
Ool00`00Oomoo`0<OolB0002Ool4003lOol001Eoo`03001oogoo00=ooaP000Moo`@00?Yoo`004goo
7@002goo1@00n7oo000COolH000BOol4003gOol001=ooa@001Qoo`@00?Eoo`004goo2`008Woo1000
m7oo000EOol00`00Oomoo`0ZOol4003bOol001Eoo`03001oogoo02]oo`@00?5oo`005Goo00<007oo
Ool0;Goo0`00l7oo000EOol00`00Oomoo`0^Ool3003_Ool001Eoo`03001oogoo02moo`<00>ioo`00
5Goo00<007ooOol0<7oo1000k7oo000EOol2000bOol4003[Ool001Eoo`03001oogoo03=oo`<00>Yo
o`005Goo00<007ooOol0=7oo0`00jGoo000EOol00`00Oomoo`0eOol3003XOol001Eoo`03001oogoo
03Ioo`<00>Moo`005Goo00<007ooOol0=goo0P00igoo000EOol00`00Oomoo`0gOol3003VOol000Yo
o`H000Eoo`03001oogoo03Qoo`<00>Eoo`002goo00D007ooOomoo`0000Eoo`03001oogoo03Uoo`<0
0>Aoo`0037oo00<007ooOol01Woo0`00>Woo0`00hgoo000=Ool00`00Oomoo`05Ool00`00Oomoo`0k
Ool2003SOol000ioo`03001oogoo00Aoo`03001oogoo03]oo`<00>9oo`003goo00<007ooOol00goo
00<007ooOol0?7oo0`00hGoo000?Ool00`00Oomoo`03Ool00`00Oomoo`0mOol3003POol000Yoo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo03ioo`800>1oo`002goo10001Woo00<007ooOol0
?Woo0`00ggoo000EOol00`00Oomoo`0oOol3003NOol001Eoo`03001oogoo041oo`<00=eoo`005Goo
0P00@Woo0`00g7oo000EOol00`00Oomoo`12Ool2003LOol001Eoo`03001oogoo049oo`<00=]oo`00
5Goo00<007ooOol0@goo0`00fWoo000EOol00`00Oomoo`14Ool2003JOol001Eoo`03001oogoo04Ao
o`<00=Uoo`005Goo00<007ooOol0AGoo0`00f7oo000EOol00`00Oomoo`16Ool2003HOol001Eoo`03
001oogoo04Ioo`<00=Moo`005Goo0P00B7oo0`00eWoo000EOol00`00Oomoo`18Ool2003FOol001Eo
o`03001oogoo04Qoo`<00=Eoo`005Goo00<007ooOol0BGoo0P00eGoo000EOol00`00Oomoo`19Ool3
003DOol001Eoo`03001oogoo04Yoo`800=Aoo`005Goo00<007ooOol0BWoo0`00dgoo000EOol00`00
Oomoo`1;Ool2003COol001Eoo`03001oogoo04]oo`<00=9oo`005Goo0P00CGoo0P00dWoo000EOol0
0`00Oomoo`1<Ool3003AOol001Eoo`03001oogoo04eoo`800=5oo`005Goo00<007ooOol0CGoo0`00
d7oo000EOol00`00Oomoo`1>Ool2003@Ool001Eoo`03001oogoo04ioo`<00<moo`005Goo00<007oo
Ool0Cgoo0P00cgoo000=Ool30005Ool00`00Oomoo`1?Ool3003>Ool000ioo`03001oogoo00Aoo`03
001oogoo051oo`800<ioo`002Woo1P001Goo0`00D7oo0P00cWoo000:Ool01@00Oomoogoo00001Woo
00<007ooOol0D7oo0`00cGoo000;Ool01000Oomoo`001Woo00<007ooOol0DGoo0P00cGoo000<Ool0
0`00Ool00006Ool00`00Oomoo`1AOol3003<Ool000aoo`03001oo`0000Ioo`03001oogoo059oo`80
0<aoo`003Goo0P001Woo00<007ooOol0DWoo0`00bgoo000>Ool00`00Oomoo`04Ool00`00Oomoo`1C
Ool2003;Ool001Eoo`03001oogoo05=oo`800<]oo`005Goo00<007ooOol0Dgoo0`00bWoo000EOol2
001EOol2003:Ool001Eoo`03001oogoo05Aoo`800<Yoo`005Goo00<007ooOol0E7oo0`00bGoo000E
Ool00`00Oomoo`1EOol20039Ool001Eoo`03001oogoo05Eoo`800<Uoo`005Goo00<007ooOol0EGoo
0`00b7oo000EOol00`00Oomoo`1FOol20038Ool001Eoo`03001oogoo05Ioo`800<Qoo`005Goo00<0
07ooOol0EWoo0P00b7oo000EOol00`00Oomoo`1FOol30037Ool001Eoo`03001oogoo05Moo`800<Mo
o`00Kgoo0P00agoo003oOoliOol00?moocUoo`00ogoo>Goo0000\
\>"],
  ImageRangeCache->{{{0, 311}, {191.75, 0}} -> {-0.773423, -5.27054, 
  0.0535106, 0.0824590}}]
}, Open  ]],

Cell["There is a selection of orthogonal polynomials:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["LegendreP[5, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(\(15\ x\)\/8 - \(35\ x\^3\)\/4 + \(63\ x\^5\)\/8\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["SphericalHarmonicY[1, 1, theta, phi]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(1\/2\)\)\ E\^\(I\ phi\)\ \@\(3\/\(2\ \[Pi]\)\)\ Sin[theta]\)], 
  "Output"]
}, Open  ]],

Cell["\<\
This is probably a good place to remark on how Greek letters are \
input - previously we have used \[Pi] without explaining how we wrote it, and \
our spherical harmonic function looks rather better written as:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
TraditionalForm[SphericalHarmonicY[1, 1, \[Theta], \[Phi]]]\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\(1\/2\)\)\ \[ExponentialE]\^\(\[ImaginaryI]\ \[Phi]\)\ 
      \@\(3\/\(2\ \[Pi]\)\)\ \(sin(\[Theta])\)\)], "Output"]
}, Open  ]],

Cell["\<\
The trick is to write \"ESC\\thetaESC\" where ESC stands for the \
escape key. Occurrences of \[Pi] are generated with \"ESC\\piESC\". \
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Naming of Special Functions", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This should give you a feel for the way in which these functions are \
named.  Many special functions can be classified under an overall name, like \
Legendre.  Within this there may be more than one type, like the ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  " Legendre functions.  And finally, these may be indexed by an integer or \
some other variable. Hence the Bessel function ",
  Cell[BoxData[
      \(TraditionalForm\`\(K\_n\)(x)\)]],
  " is denoted by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["BesselK[n, x]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(BesselK[n, x]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Most of these functions are in the core kernel.  However, the statistical \
distributions are accessed from one of the Packages. We shall look at \
Packages later. Note that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " knows some particular values of special functions, and also knows \
series:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Gamma[1/2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\@\[Pi]\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Series[BesselJ[0, x], {x, 0, 6}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    InterpretationBox[
      RowBox[{
      "1", "-", \(x\^2\/4\), "+", \(x\^4\/64\), "-", \(x\^6\/2304\), "+", 
        InterpretationBox[\(O[x]\^8\),
          SeriesData[ x, 0, {}, 0, 8, 1]]}],
      SeriesData[ x, 0, {1, 0, 
        Rational[ -1, 4], 0, 
        Rational[ 1, 64], 0, 
        Rational[ -1, 2304]}, 0, 8, 1]]], "Output",
  GroupPageBreakWithin->True]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Building Your Own Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Defining a Simple Function", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
When defining a function by using an explicit formula, it is \
important to specify the variables involved in that function by use of the \
underscore \"_\".  The delayed assignment sign \":=\" then allows you to \
define the formula of your function.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["h[x_, y_] := x^2 / y^3", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This has defined the function ",
  StyleBox["h",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".  We can now ask for that function to be evaluated for specific values of \
",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["h[1, 2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/8\)], "Output"]
}, Open  ]],

Cell["You can now ask about your function:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?h", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Global`h"\)], "Print"],

Cell[BoxData[
    InterpretationBox[
      StyleBox[\(h[x_, \ y_]\  := \ x^2/y^3\),
        ShowStringCharacters->True,
        NumberMarks->True],
      InputForm[ 
        Definition[ h]],
      Editable->True,
      AutoDelete->True]], "Print"]
}, Open  ]],

Cell[TextData[{
  "If you wish to clear that definition, you use ",
  StyleBox["Clear",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[h]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Your definition for ",
  StyleBox["h",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " no longer exists."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?h", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Global`h"\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "There are many types of function which can be defined, in the same way as \
the algebraic one we have just defined. Functions can employ any of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " built-in functions;"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["funcSin[x_,y_] := Sin[x] + Cos[y]^2", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["funcSq[x_, y_, z_] := Sqrt[x^2 + y^2 + z^2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Functions can also be defined which will perform a test on the argument(s) \
returning the result True or False.",
  StyleBox[" ",
    FontFamily->"Courier"],
  StyleBox["testDiv5",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will check whether the argument is divisible by 5:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["testDiv5[t_] := IntegerQ[t/5]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["testDiv5[8]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(False\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["testDiv5[15]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Defining Functions in Their Pure Form", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If you only intend to use the function once, or to use it with operations \
such as ",
  StyleBox["Map",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["Nest",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", there is a more economical way to define it. Instead of using a letter \
or name for the arguments in the function, the syntax for \"pure\" functions \
uses the \"#\" to indicate the arguments. So the expression which defines the \
action of a function to add 3 to a number is",
  StyleBox[" ",
    FontFamily->"Courier"],
  StyleBox["# + 3",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " instead of ",
  StyleBox["x + 3",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". To actually turn this into a function we use an \"&\" after the \
expression of the function. So"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["f[x_] := x + 3", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["can be written as", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["(# + 3) &", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(#1 + 3 &\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "without using the definition of ",
  StyleBox["f",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". So if we apply that function to 2,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["(# + 3) & [2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(5\)], "Output"]
}, Open  ]],

Cell["\<\
This technique can also be used instead of defining functions of \
more than one variable. Using the # notation, we now label each of the \
arguments with a number immediately (i.e. no space) after the #. So instead \
of\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["g[x_, y_] := x^2 + y", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["g[2,3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(7\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell["we use", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["(#1^2 + #2) & [2,3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(7\)], "Output"]
}, Open  ]],

Cell["\<\
Notice how the numbers used for labelling directly correspond to \
the order in which those arguments will be given. The advantage of this \
technique for defining functions which will only be used once is that the \
definitions do not need to be stored, they are just used at the appropriate \
moment.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Using Pure Functions in Simple Programs", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Programs can be shortened in length by using pure functions instead \
of separately defining a general global function which can be used in those \
functions which specifically ask for function definitions.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["NestList[#^2 &, x, 5]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({x, x\^2, x\^4, x\^8, x\^16, x\^32}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Fold[Sqrt[#1 + #2] &, t, {l, m, n}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\@\(n + \@\(m + \@\(l + t\)\)\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(Clear[p, q, r, s]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["NestList[Append[Rest[#],1]&, {p,q,r,s}, 3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{p, q, r, s}, {q, r, s, 1}, {r, s, 1, 1}, {s, 1, 1, 1}}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This form of writing functions can also be used for defining functions \
which will perform a test. As an example we can define a function which will \
return True if a number is divisible by 3, and use this as the criterion in \
the function ",
  StyleBox["Select",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Select[ {1, 2, 3, 4, 5, 6, 7, 8, 9}, IntegerQ[(#/3)] &]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({3, 6, 9}\)], "Output"]
}, Open  ]],

Cell["\<\
A function which will return True if the second argument is less \
than the first can be used as the test by which a list can be sorted.\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sort[ {8, 34, 9, 2, 7, 4, 75, 4}, (#2 < #1) &]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({75, 34, 9, 8, 7, 4, 4, 2}\)], "Output"]
}, Open  ]],

Cell["\<\
This technique can be particularly useful when manipulating lists. \
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Defining a Function Recursively", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
There may be functions which you wish to define recursively giving \
values for the end points. This can be done in the following way. \
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[f]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["f[1] = 1;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["f[2] = 1;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["f[n_] := f[n-1] + f[n-2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can now evaluate ",
  StyleBox["f[3]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["f[3]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(2\)], "Output"]
}, Open  ]],

Cell["\<\
There is a useful modification of this function which makes it much \
more efficient, that forces the function to remember previously computed \
values.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Clear[g]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["g[1] = 1;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["g[2] = 1;", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["g[n_] := g[n] =g[n-1] + g[n-2]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The following illustration makes it clear that the use of the \
second function for computing the Fibonacci series is strongly \
preferred:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[f[35]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \({467.999999999999971`\ Second, 9227465}\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[g[35]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.`\ Second, 9227465}\)], "Output"]
}, Open  ]],

Cell["\<\
Such a device will turn out to be useful in a simple implementation \
of the binomial tree model, where we recurse back through a tree from the \
payoff. Note that here and throughout this text, the timings reported are \
based on the computer used for editing and final testing of this manuscript, \
which runs a 266 MHz PowerPC 750 processor. You must make allowances for the \
clock speed and efficiency of your own system when making comparisons or \
assessments about what computations are feasible.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Difference between := and =", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Constructing functions is dependent on the use of \":=\".  The use of the \
colon with equals means that the evaluation of the right side of the equation \
is delayed, and it will only be evaluated when that function is actually \
called.  As an example let us define some functions which will be dependent \
on the value of a variable that we call ",
  StyleBox["M",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and to which we assign a value of 2 initially:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["M=2", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now if we define ",
  StyleBox["funcOne",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", using :=, the right-hand side will not be evaluated yet."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["funcOne[x_] := M x", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Defining the function ",
  StyleBox["funcTwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " using = will mean that the right-hand side is evaluated using the present \
value of ",
  StyleBox["M",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["funcTwo[x_] = M x", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(2\ x\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "With the present definitions both ",
  StyleBox["funcOne[2]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["funcTwo[2]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " will evaluate to 4. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["{funcOne[2],funcTwo[2]}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({4, 4}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "If we were to change the value of ",
  StyleBox["M",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " however, the definition of ",
  StyleBox["funcOne",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " would: allow the new value to be used, whereas the definition of ",
  StyleBox["funcTwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " has already been set by the original definition of ",
  StyleBox["M",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["M=100", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(100\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{funcOne[2],funcTwo[2]}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[BoxData[
    \({200, 4}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Assignment and Random Walks", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Another way to see the difference between = and := is to define two \
functions from which we can produce random walks and plot those resultant \
functions. Firstly let us define two functions which will add a random number \
between ",
  Cell[BoxData[
      \(TraditionalForm\`\(-0.5\)\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\(+0.5\)\)]],
  " to their argument:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["addrandONE[x_] := x + Random[Real,{-0.5,0.5}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["addrandTWO[x_] = x + Random[Real,{-0.5,0.5}]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-0.365924890929254731`\) + x\)], "Output"]
}, Open  ]],

Cell["\<\
So if we ask for each of these functions evaluated twice on the \
same value,\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["{addrandONE[0],addrandONE[0]}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-0.405744721277714237`\), \(-0.19209552972021644`\)}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["{addrandTWO[0],addrandTWO[0]}", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-0.365924890929254731`\), \(-0.365924890929254731`\)}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "The second function does not act as a random function. Now we can form \
random walks based on these functions using the function ",
  StyleBox["NestList",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["walkONE = NestList[addrandONE,0,1000];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["walkTWO = NestList[addrandTWO,0,1000];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and now we can display these lists using ",
  StyleBox["ListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and asking that the points should be joined."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["ListPlot[walkONE, PlotJoined->True];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00095143 0.384091 0.0414348 [
[.2141 .37159 -9 -9 ]
[.2141 .37159 9 0 ]
[.40438 .37159 -9 -9 ]
[.40438 .37159 9 0 ]
[.59467 .37159 -9 -9 ]
[.59467 .37159 9 0 ]
[.78495 .37159 -9 -9 ]
[.78495 .37159 9 0 ]
[.97524 .37159 -12 -9 ]
[.97524 .37159 12 0 ]
[.01131 .05261 -12 -4.5 ]
[.01131 .05261 0 4.5 ]
[.01131 .13548 -12 -4.5 ]
[.01131 .13548 0 4.5 ]
[.01131 .21835 -12 -4.5 ]
[.01131 .21835 0 4.5 ]
[.01131 .30122 -12 -4.5 ]
[.01131 .30122 0 4.5 ]
[.01131 .46696 -6 -4.5 ]
[.01131 .46696 0 4.5 ]
[.01131 .54983 -6 -4.5 ]
[.01131 .54983 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2141 .38409 m
.2141 .39034 L
s
[(200)] .2141 .37159 0 1 Mshowa
.40438 .38409 m
.40438 .39034 L
s
[(400)] .40438 .37159 0 1 Mshowa
.59467 .38409 m
.59467 .39034 L
s
[(600)] .59467 .37159 0 1 Mshowa
.78495 .38409 m
.78495 .39034 L
s
[(800)] .78495 .37159 0 1 Mshowa
.97524 .38409 m
.97524 .39034 L
s
[(1000)] .97524 .37159 0 1 Mshowa
.125 Mabswid
.07138 .38409 m
.07138 .38784 L
s
.11895 .38409 m
.11895 .38784 L
s
.16652 .38409 m
.16652 .38784 L
s
.26167 .38409 m
.26167 .38784 L
s
.30924 .38409 m
.30924 .38784 L
s
.35681 .38409 m
.35681 .38784 L
s
.45195 .38409 m
.45195 .38784 L
s
.49952 .38409 m
.49952 .38784 L
s
.5471 .38409 m
.5471 .38784 L
s
.64224 .38409 m
.64224 .38784 L
s
.68981 .38409 m
.68981 .38784 L
s
.73738 .38409 m
.73738 .38784 L
s
.83252 .38409 m
.83252 .38784 L
s
.8801 .38409 m
.8801 .38784 L
s
.92767 .38409 m
.92767 .38784 L
s
.25 Mabswid
0 .38409 m
1 .38409 L
s
.02381 .05261 m
.03006 .05261 L
s
[(-8)] .01131 .05261 1 0 Mshowa
.02381 .13548 m
.03006 .13548 L
s
[(-6)] .01131 .13548 1 0 Mshowa
.02381 .21835 m
.03006 .21835 L
s
[(-4)] .01131 .21835 1 0 Mshowa
.02381 .30122 m
.03006 .30122 L
s
[(-2)] .01131 .30122 1 0 Mshowa
.02381 .46696 m
.03006 .46696 L
s
[(2)] .01131 .46696 1 0 Mshowa
.02381 .54983 m
.03006 .54983 L
s
[(4)] .01131 .54983 1 0 Mshowa
.125 Mabswid
.02381 .07333 m
.02756 .07333 L
s
.02381 .09405 m
.02756 .09405 L
s
.02381 .11476 m
.02756 .11476 L
s
.02381 .1562 m
.02756 .1562 L
s
.02381 .17692 m
.02756 .17692 L
s
.02381 .19763 m
.02756 .19763 L
s
.02381 .23907 m
.02756 .23907 L
s
.02381 .25979 m
.02756 .25979 L
s
.02381 .2805 m
.02756 .2805 L
s
.02381 .32194 m
.02756 .32194 L
s
.02381 .34266 m
.02756 .34266 L
s
.02381 .36337 m
.02756 .36337 L
s
.02381 .40481 m
.02756 .40481 L
s
.02381 .42553 m
.02756 .42553 L
s
.02381 .44624 m
.02756 .44624 L
s
.02381 .48768 m
.02756 .48768 L
s
.02381 .5084 m
.02756 .5084 L
s
.02381 .52911 m
.02756 .52911 L
s
.02381 .0319 m
.02756 .0319 L
s
.02381 .01118 m
.02756 .01118 L
s
.02381 .57055 m
.02756 .57055 L
s
.02381 .59126 m
.02756 .59126 L
s
.02381 .61198 m
.02756 .61198 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02476 .38409 m
.02571 .38646 L
.02666 .3925 L
.02762 .38385 L
.02857 .3693 L
.02952 .36024 L
.03047 .3439 L
.03142 .354 L
.03237 .3343 L
.03332 .3282 L
.03428 .31294 L
.03523 .31555 L
.03618 .32886 L
.03713 .32071 L
.03808 .30437 L
.03903 .31071 L
.03998 .29924 L
.04094 .28217 L
.04189 .29714 L
.04284 .29021 L
.04379 .2979 L
.04474 .31162 L
.04569 .32104 L
.04664 .31021 L
.0476 .30514 L
.04855 .29577 L
.0495 .27843 L
.05045 .29697 L
.0514 .28574 L
.05235 .30614 L
.0533 .32586 L
.05426 .31358 L
.05521 .30133 L
.05616 .30712 L
.05711 .32138 L
.05806 .32721 L
.05901 .32237 L
.05996 .31559 L
.06092 .32547 L
.06187 .34568 L
.06282 .3316 L
.06377 .32116 L
.06472 .3368 L
.06567 .34321 L
.06662 .34215 L
.06758 .33872 L
.06853 .32422 L
.06948 .32075 L
.07043 .30404 L
.07138 .28926 L
Mistroke
.07233 .27138 L
.07328 .27008 L
.07424 .28532 L
.07519 .27086 L
.07614 .25397 L
.07709 .24424 L
.07804 .25102 L
.07899 .25148 L
.07994 .24105 L
.0809 .2462 L
.08185 .2371 L
.0828 .22363 L
.08375 .22404 L
.0847 .2297 L
.08565 .21397 L
.0866 .23165 L
.08756 .23714 L
.08851 .2571 L
.08946 .26315 L
.09041 .26354 L
.09136 .26281 L
.09231 .26553 L
.09326 .26757 L
.09422 .26202 L
.09517 .25846 L
.09612 .24176 L
.09707 .24927 L
.09802 .23747 L
.09897 .23008 L
.09992 .24382 L
.10088 .22385 L
.10183 .2323 L
.10278 .21462 L
.10373 .20249 L
.10468 .21233 L
.10563 .21354 L
.10658 .21617 L
.10754 .2191 L
.10849 .22395 L
.10944 .2282 L
.11039 .24606 L
.11134 .24974 L
.11229 .26926 L
.11324 .2524 L
.1142 .25027 L
.11515 .23051 L
.1161 .22728 L
.11705 .23669 L
.118 .2174 L
.11895 .23506 L
Mistroke
.1199 .24504 L
.12086 .24552 L
.12181 .25434 L
.12276 .23754 L
.12371 .24678 L
.12466 .25953 L
.12561 .26531 L
.12656 .28136 L
.12752 .30148 L
.12847 .29229 L
.12942 .27473 L
.13037 .26714 L
.13132 .26168 L
.13227 .26897 L
.13322 .25426 L
.13418 .26371 L
.13513 .25944 L
.13608 .26287 L
.13703 .27101 L
.13798 .2795 L
.13893 .29918 L
.13988 .31393 L
.14084 .32064 L
.14179 .33218 L
.14274 .32117 L
.14369 .31472 L
.14464 .33332 L
.14559 .34095 L
.14654 .34141 L
.1475 .34293 L
.14845 .33503 L
.1494 .34733 L
.15035 .34839 L
.1513 .33838 L
.15225 .32733 L
.1532 .3265 L
.15416 .3123 L
.15511 .31572 L
.15606 .29866 L
.15701 .3091 L
.15796 .31988 L
.15891 .34059 L
.15986 .3361 L
.16082 .31734 L
.16177 .32916 L
.16272 .31441 L
.16367 .32392 L
.16462 .31433 L
.16557 .31645 L
.16652 .32887 L
Mistroke
.16748 .3405 L
.16843 .344 L
.16938 .32494 L
.17033 .31511 L
.17128 .31393 L
.17223 .32585 L
.17318 .32644 L
.17414 .30591 L
.17509 .29506 L
.17604 .28709 L
.17699 .28116 L
.17794 .27793 L
.17889 .26343 L
.17984 .26574 L
.1808 .26974 L
.18175 .26652 L
.1827 .27722 L
.18365 .27757 L
.1846 .29047 L
.18555 .28128 L
.1865 .26176 L
.18746 .25098 L
.18841 .24104 L
.18936 .24016 L
.19031 .22972 L
.19126 .23616 L
.19221 .22456 L
.19316 .21279 L
.19412 .22426 L
.19507 .23948 L
.19602 .24802 L
.19697 .23605 L
.19792 .23765 L
.19887 .24013 L
.19982 .23388 L
.20078 .24586 L
.20173 .24125 L
.20268 .2207 L
.20363 .23116 L
.20458 .22565 L
.20553 .23105 L
.20648 .23087 L
.20744 .24915 L
.20839 .23211 L
.20934 .23632 L
.21029 .2262 L
.21124 .2337 L
.21219 .23826 L
.21314 .23219 L
.2141 .23635 L
Mistroke
.21505 .23473 L
.216 .23035 L
.21695 .23352 L
.2179 .24317 L
.21885 .25373 L
.2198 .24061 L
.22076 .22146 L
.22171 .20791 L
.22266 .20401 L
.22361 .19962 L
.22456 .2058 L
.22551 .19208 L
.22646 .19844 L
.22742 .17883 L
.22837 .1589 L
.22932 .16608 L
.23027 .17487 L
.23122 .19303 L
.23217 .18961 L
.23312 .1862 L
.23408 .16677 L
.23503 .15964 L
.23598 .14158 L
.23693 .15472 L
.23788 .15763 L
.23883 .1756 L
.23978 .17508 L
.24074 .15786 L
.24169 .17092 L
.24264 .1813 L
.24359 .17922 L
.24454 .19625 L
.24549 .1925 L
.24644 .18656 L
.2474 .19901 L
.24835 .20904 L
.2493 .21966 L
.25025 .2126 L
.2512 .22427 L
.25215 .20639 L
.2531 .1875 L
.25406 .18301 L
.25501 .17738 L
.25596 .18364 L
.25691 .16346 L
.25786 .14537 L
.25881 .13708 L
.25976 .15092 L
.26072 .14855 L
.26167 .13321 L
Mistroke
.26262 .14615 L
.26357 .1565 L
.26452 .16178 L
.26547 .15678 L
.26642 .15109 L
.26738 .16512 L
.26833 .15343 L
.26928 .13367 L
.27023 .13623 L
.27118 .11952 L
.27213 .11793 L
.27308 .12594 L
.27404 .13756 L
.27499 .118 L
.27594 .11458 L
.27689 .10637 L
.27784 .1029 L
.27879 .0978 L
.27974 .09385 L
.2807 .08301 L
.28165 .06711 L
.2826 .06889 L
.28355 .08802 L
.2845 .0718 L
.28545 .06367 L
.2864 .07582 L
.28736 .06896 L
.28831 .07845 L
.28926 .09674 L
.29021 .11557 L
.29116 .09968 L
.29211 .10823 L
.29306 .10323 L
.29402 .11806 L
.29497 .12448 L
.29592 .10429 L
.29687 .10839 L
.29782 .12206 L
.29877 .11118 L
.29972 .11992 L
.30068 .10677 L
.30163 .10483 L
.30258 .11862 L
.30353 .11748 L
.30448 .09952 L
.30543 .11651 L
.30638 .13189 L
.30734 .12625 L
.30829 .13713 L
.30924 .12125 L
Mistroke
.31019 .12278 L
.31114 .12836 L
.31209 .14168 L
.31304 .12768 L
.314 .12438 L
.31495 .14214 L
.3159 .13974 L
.31685 .13163 L
.3178 .14262 L
.31875 .15985 L
.3197 .17407 L
.32066 .17301 L
.32161 .17416 L
.32256 .16194 L
.32351 .16858 L
.32446 .14875 L
.32541 .15683 L
.32636 .16646 L
.32732 .17035 L
.32827 .15424 L
.32922 .16766 L
.33017 .16222 L
.33112 .17594 L
.33207 .19643 L
.33302 .1876 L
.33398 .1973 L
.33493 .17698 L
.33588 .19075 L
.33683 .20594 L
.33778 .2186 L
.33873 .2214 L
.33968 .22256 L
.34064 .20604 L
.34159 .22219 L
.34254 .23148 L
.34349 .21299 L
.34444 .21603 L
.34539 .22369 L
.34634 .20562 L
.3473 .18625 L
.34825 .20192 L
.3492 .22067 L
.35015 .21942 L
.3511 .23688 L
.35205 .21842 L
.353 .22189 L
.35396 .22764 L
.35491 .24532 L
.35586 .25641 L
.35681 .2709 L
Mistroke
.35776 .27624 L
.35871 .25944 L
.35966 .27605 L
.36062 .25717 L
.36157 .239 L
.36252 .24175 L
.36347 .25417 L
.36442 .23986 L
.36537 .23311 L
.36632 .23363 L
.36728 .22229 L
.36823 .22104 L
.36918 .21164 L
.37013 .21082 L
.37108 .20452 L
.37203 .20525 L
.37298 .21781 L
.37394 .22025 L
.37489 .21169 L
.37584 .22967 L
.37679 .21576 L
.37774 .22124 L
.37869 .22231 L
.37964 .20508 L
.3806 .20654 L
.38155 .2081 L
.3825 .21328 L
.38345 .19421 L
.3844 .19313 L
.38535 .21266 L
.3863 .22614 L
.38726 .24209 L
.38821 .22705 L
.38916 .22534 L
.39011 .22945 L
.39106 .22592 L
.39201 .241 L
.39296 .26083 L
.39392 .25052 L
.39487 .26698 L
.39582 .24878 L
.39677 .24546 L
.39772 .26442 L
.39867 .28362 L
.39962 .30005 L
.40058 .31196 L
.40153 .30913 L
.40248 .32485 L
.40343 .31909 L
.40438 .30873 L
Mistroke
.40533 .32144 L
.40628 .33551 L
.40724 .35155 L
.40819 .34238 L
.40914 .36232 L
.41009 .38116 L
.41104 .39152 L
.41199 .40478 L
.41294 .3999 L
.4139 .40155 L
.41485 .41755 L
.4158 .43169 L
.41675 .4164 L
.4177 .42231 L
.41865 .43579 L
.4196 .43254 L
.42056 .41901 L
.42151 .42643 L
.42246 .4442 L
.42341 .44975 L
.42436 .45978 L
.42531 .47219 L
.42626 .475 L
.42722 .4702 L
.42817 .48824 L
.42912 .50729 L
.43007 .51478 L
.43102 .49843 L
.43197 .51725 L
.43292 .49674 L
.43388 .51458 L
.43483 .5057 L
.43578 .50868 L
.43673 .50725 L
.43768 .48837 L
.43863 .48605 L
.43958 .48361 L
.44054 .49699 L
.44149 .48534 L
.44244 .46556 L
.44339 .45593 L
.44434 .44117 L
.44529 .43247 L
.44624 .42786 L
.4472 .42891 L
.44815 .42246 L
.4491 .43167 L
.45005 .41114 L
.451 .41487 L
.45195 .41008 L
Mistroke
.4529 .3911 L
.45386 .40763 L
.45481 .41326 L
.45576 .40825 L
.45671 .39215 L
.45766 .39685 L
.45861 .37877 L
.45956 .39592 L
.46052 .37798 L
.46147 .36428 L
.46242 .36936 L
.46337 .35242 L
.46432 .36684 L
.46527 .3522 L
.46622 .3462 L
.46718 .36473 L
.46813 .36713 L
.46908 .37782 L
.47003 .39148 L
.47098 .39576 L
.47193 .40965 L
.47288 .42016 L
.47384 .40937 L
.47479 .39772 L
.47574 .40988 L
.47669 .42458 L
.47764 .42888 L
.47859 .44295 L
.47954 .4505 L
.4805 .43978 L
.48145 .44144 L
.4824 .45908 L
.48335 .46385 L
.4843 .44611 L
.48525 .46341 L
.4862 .47727 L
.48716 .48834 L
.48811 .50596 L
.48906 .50854 L
.49001 .52459 L
.49096 .51254 L
.49191 .49875 L
.49286 .50838 L
.49382 .49944 L
.49477 .49422 L
.49572 .49063 L
.49667 .49034 L
.49762 .47233 L
.49857 .47567 L
.49952 .4781 L
Mistroke
.50048 .49422 L
.50143 .48286 L
.50238 .49937 L
.50333 .4918 L
.50428 .48555 L
.50523 .47726 L
.50618 .46829 L
.50714 .45774 L
.50809 .45491 L
.50904 .45347 L
.50999 .45415 L
.51094 .4467 L
.51189 .462 L
.51284 .46523 L
.5138 .45724 L
.51475 .44286 L
.5157 .42782 L
.51665 .41927 L
.5176 .43722 L
.51855 .44715 L
.5195 .45311 L
.52046 .44185 L
.52141 .43575 L
.52236 .42253 L
.52331 .43309 L
.52426 .41247 L
.52521 .41058 L
.52616 .42565 L
.52712 .42173 L
.52807 .43013 L
.52902 .41649 L
.52997 .42139 L
.53092 .44102 L
.53187 .43014 L
.53282 .43654 L
.53378 .42818 L
.53473 .41178 L
.53568 .41839 L
.53663 .41206 L
.53758 .39736 L
.53853 .41673 L
.53948 .41117 L
.54044 .40761 L
.54139 .40369 L
.54234 .39638 L
.54329 .38137 L
.54424 .36319 L
.54519 .35177 L
.54614 .35463 L
.5471 .33951 L
Mistroke
.54805 .34394 L
.549 .33817 L
.54995 .32422 L
.5509 .32143 L
.55185 .31879 L
.5528 .32882 L
.55376 .31596 L
.55471 .30333 L
.55566 .31501 L
.55661 .31269 L
.55756 .29551 L
.55851 .29699 L
.55946 .29428 L
.56042 .28594 L
.56137 .27011 L
.56232 .25643 L
.56327 .23656 L
.56422 .25286 L
.56517 .26506 L
.56612 .24568 L
.56708 .2647 L
.56803 .2717 L
.56898 .26033 L
.56993 .27678 L
.57088 .27065 L
.57183 .26271 L
.57278 .24457 L
.57374 .2431 L
.57469 .26032 L
.57564 .26306 L
.57659 .2785 L
.57754 .26895 L
.57849 .25377 L
.57944 .23812 L
.5804 .25001 L
.58135 .2597 L
.5823 .26795 L
.58325 .28136 L
.5842 .28836 L
.58515 .29101 L
.5861 .29842 L
.58706 .31624 L
.58801 .33176 L
.58896 .33307 L
.58991 .34218 L
.59086 .33229 L
.59181 .33845 L
.59276 .34403 L
.59372 .33856 L
.59467 .35732 L
Mistroke
.59562 .36091 L
.59657 .34724 L
.59752 .34526 L
.59847 .34057 L
.59942 .34944 L
.60038 .36603 L
.60133 .35852 L
.60228 .34876 L
.60323 .36645 L
.60418 .35264 L
.60513 .35759 L
.60608 .35514 L
.60704 .34512 L
.60799 .34937 L
.60894 .36764 L
.60989 .36807 L
.61084 .36325 L
.61179 .34548 L
.61274 .33392 L
.6137 .32353 L
.61465 .33326 L
.6156 .33062 L
.61655 .34525 L
.6175 .33682 L
.61845 .32224 L
.6194 .31256 L
.62036 .30845 L
.62131 .32542 L
.62226 .3227 L
.62321 .31714 L
.62416 .29983 L
.62511 .30584 L
.62606 .30614 L
.62702 .29368 L
.62797 .29213 L
.62892 .27988 L
.62987 .26949 L
.63082 .27348 L
.63177 .27439 L
.63272 .28242 L
.63368 .29756 L
.63463 .29862 L
.63558 .29037 L
.63653 .28806 L
.63748 .27276 L
.63843 .25574 L
.63938 .25357 L
.64034 .23899 L
.64129 .25898 L
.64224 .27235 L
Mistroke
.64319 .25358 L
.64414 .24275 L
.64509 .24475 L
.64604 .24295 L
.647 .26221 L
.64795 .26608 L
.6489 .24706 L
.64985 .23701 L
.6508 .2371 L
.65175 .23251 L
.6527 .2446 L
.65366 .25127 L
.65461 .27117 L
.65556 .27927 L
.65651 .29694 L
.65746 .28182 L
.65841 .28926 L
.65936 .27895 L
.66032 .2912 L
.66127 .27239 L
.66222 .26128 L
.66317 .24483 L
.66412 .2578 L
.66507 .24635 L
.66602 .23329 L
.66698 .24839 L
.66793 .23865 L
.66888 .24971 L
.66983 .2381 L
.67078 .22862 L
.67173 .21717 L
.67268 .21757 L
.67364 .22659 L
.67459 .24242 L
.67554 .2396 L
.67649 .25405 L
.67744 .26388 L
.67839 .25089 L
.67934 .25112 L
.6803 .25997 L
.68125 .24164 L
.6822 .25968 L
.68315 .26838 L
.6841 .27531 L
.68505 .28882 L
.686 .3026 L
.68696 .31904 L
.68791 .31671 L
.68886 .32256 L
.68981 .34195 L
Mistroke
.69076 .34742 L
.69171 .35474 L
.69266 .35149 L
.69362 .35964 L
.69457 .35583 L
.69552 .34204 L
.69647 .35049 L
.69742 .36353 L
.69837 .38326 L
.69932 .37573 L
.70028 .39508 L
.70123 .40038 L
.70218 .39917 L
.70313 .4035 L
.70408 .42046 L
.70503 .42844 L
.70598 .43924 L
.70694 .45737 L
.70789 .4401 L
.70884 .45502 L
.70979 .4701 L
.71074 .46984 L
.71169 .46743 L
.71264 .48368 L
.7136 .47257 L
.71455 .48571 L
.7155 .46583 L
.71645 .45322 L
.7174 .46663 L
.71835 .47286 L
.7193 .46524 L
.72026 .4603 L
.72121 .4747 L
.72216 .46774 L
.72311 .46149 L
.72406 .47197 L
.72501 .46686 L
.72596 .47628 L
.72692 .4738 L
.72787 .45557 L
.72882 .46038 L
.72977 .47239 L
.73072 .46646 L
.73167 .45403 L
.73262 .46448 L
.73358 .45604 L
.73453 .47323 L
.73548 .46526 L
.73643 .4661 L
.73738 .46524 L
Mistroke
.73833 .4816 L
.73928 .46552 L
.74024 .47367 L
.74119 .4873 L
.74214 .49056 L
.74309 .50013 L
.74404 .5146 L
.74499 .51448 L
.74594 .50327 L
.7469 .52308 L
.74785 .52195 L
.7488 .53311 L
.74975 .54511 L
.7507 .56243 L
.75165 .5772 L
.7526 .59707 L
.75356 .59428 L
.75451 .60332 L
.75546 .58692 L
.75641 .59452 L
.75736 .59524 L
.75831 .59154 L
.75926 .59502 L
.76022 .58276 L
.76117 .58784 L
.76212 .5795 L
.76307 .59555 L
.76402 .59038 L
.76497 .57148 L
.76592 .57428 L
.76688 .55514 L
.76783 .57082 L
.76878 .58384 L
.76973 .58755 L
.77068 .59026 L
.77163 .57406 L
.77258 .55437 L
.77354 .56147 L
.77449 .57013 L
.77544 .55477 L
.77639 .55859 L
.77734 .57737 L
.77829 .58171 L
.77924 .57947 L
.7802 .56185 L
.78115 .56362 L
.7821 .54377 L
.78305 .53306 L
.784 .53108 L
.78495 .52047 L
Mistroke
.7859 .50529 L
.78686 .52047 L
.78781 .51667 L
.78876 .52397 L
.78971 .50722 L
.79066 .52744 L
.79161 .53133 L
.79256 .51421 L
.79352 .51546 L
.79447 .53117 L
.79542 .53403 L
.79637 .53053 L
.79732 .54383 L
.79827 .55418 L
.79922 .57394 L
.80018 .57238 L
.80113 .56062 L
.80208 .5525 L
.80303 .56915 L
.80398 .58655 L
.80493 .57392 L
.80588 .55579 L
.80684 .55371 L
.80779 .561 L
.80874 .54283 L
.80969 .53023 L
.81064 .51123 L
.81159 .53195 L
.81254 .55125 L
.8135 .53915 L
.81445 .53697 L
.8154 .55409 L
.81635 .55143 L
.8173 .54433 L
.81825 .56001 L
.8192 .55991 L
.82016 .56467 L
.82111 .56793 L
.82206 .58458 L
.82301 .56532 L
.82396 .56111 L
.82491 .55179 L
.82586 .57249 L
.82682 .55655 L
.82777 .54426 L
.82872 .53235 L
.82967 .53442 L
.83062 .5319 L
.83157 .51706 L
.83252 .49704 L
Mistroke
.83348 .49738 L
.83443 .49487 L
.83538 .48144 L
.83633 .49424 L
.83728 .47604 L
.83823 .47713 L
.83918 .48708 L
.84014 .48626 L
.84109 .4731 L
.84204 .45358 L
.84299 .43805 L
.84394 .45468 L
.84489 .44559 L
.84584 .46605 L
.8468 .47544 L
.84775 .48068 L
.8487 .4716 L
.84965 .48728 L
.8506 .48825 L
.85155 .48468 L
.8525 .49425 L
.85346 .49173 L
.85441 .48683 L
.85536 .48256 L
.85631 .47107 L
.85726 .49178 L
.85821 .47959 L
.85916 .48323 L
.86012 .46922 L
.86107 .46813 L
.86202 .46671 L
.86297 .45045 L
.86392 .47032 L
.86487 .46803 L
.86582 .46143 L
.86678 .44925 L
.86773 .45749 L
.86868 .45547 L
.86963 .46019 L
.87058 .46349 L
.87153 .46008 L
.87248 .4631 L
.87344 .44614 L
.87439 .4323 L
.87534 .44003 L
.87629 .42485 L
.87724 .43351 L
.87819 .44466 L
.87914 .44316 L
.8801 .42799 L
Mistroke
.88105 .42812 L
.882 .41491 L
.88295 .40671 L
.8839 .41334 L
.88485 .39417 L
.8858 .37651 L
.88676 .36915 L
.88771 .35736 L
.88866 .36551 L
.88961 .38073 L
.89056 .38586 L
.89151 .39681 L
.89246 .37952 L
.89342 .37072 L
.89437 .35854 L
.89532 .34575 L
.89627 .36615 L
.89722 .35047 L
.89817 .35127 L
.89912 .33295 L
.90008 .32397 L
.90103 .31786 L
.90198 .29944 L
.90293 .317 L
.90388 .32862 L
.90483 .315 L
.90578 .3255 L
.90674 .31571 L
.90769 .32578 L
.90864 .3091 L
.90959 .30624 L
.91054 .28752 L
.91149 .26873 L
.91244 .25755 L
.9134 .27027 L
.91435 .26133 L
.9153 .28054 L
.91625 .29887 L
.9172 .30306 L
.91815 .28619 L
.9191 .30572 L
.92006 .31901 L
.92101 .30168 L
.92196 .28242 L
.92291 .29021 L
.92386 .28889 L
.92481 .26926 L
.92576 .25316 L
.92672 .27005 L
.92767 .26164 L
Mistroke
.92862 .25223 L
.92957 .26663 L
.93052 .25273 L
.93147 .24028 L
.93242 .25445 L
.93338 .26685 L
.93433 .25103 L
.93528 .27047 L
.93623 .2512 L
.93718 .25183 L
.93813 .23751 L
.93908 .21791 L
.94004 .21516 L
.94099 .21194 L
.94194 .19881 L
.94289 .18664 L
.94384 .18051 L
.94479 .17583 L
.94574 .17562 L
.9467 .18549 L
.94765 .17827 L
.9486 .16898 L
.94955 .1726 L
.9505 .17016 L
.95145 .15164 L
.9524 .14865 L
.95336 .14545 L
.95431 .13475 L
.95526 .12278 L
.95621 .12811 L
.95716 .12002 L
.95811 .11059 L
.95906 .09717 L
.96002 .08115 L
.96097 .06667 L
.96192 .05612 L
.96287 .06616 L
.96382 .07408 L
.96477 .09344 L
.96572 .07435 L
.96668 .06981 L
.96763 .06169 L
.96858 .06054 L
.96953 .0523 L
.97048 .03426 L
.97143 .01472 L
.97238 .03066 L
.97334 .04559 L
.97429 .02534 L
.97524 .0295 L
Mistroke
.97619 .02793 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{185, 114.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004E0000Za000`40O003h00Oogoo5Woo003oOolF
Ool00?mooaIoo`00ogoo5Woo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolFOol001Eoo`03001o
ogoo0?eoo`005Goo00<007ooOol0oGoo000EOol00`00Oomoo`3mOol001Eoo`800?ioo`005Goo00<0
07ooOol0jGoo00<007ooOol04Goo000EOol00`00Oomoo`3YOol00`00Oomoo`0AOol001Eoo`03001o
ogoo0>Uoo`03001oogoo015oo`005Goo00<007ooOol0jGoo0P004Woo000EOol2003ZOol2000BOol0
01Eoo`03001oogoo0>Uoo`80019oo`005Goo00<007ooOol0jGoo0P004Woo000;Ool40006Ool00`00
Oomoo`3YOol2000BOol000Yoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo0>Uoo`80019o
o`002Woo00<007ooOol00Woo00<007ooOol00goo0P00jWoo00<007ooOol04Goo00000goo00000005
0002Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`3WOol00`00Ool0000COol000]oo`@0
00Ioo`03001oogoo0>Moo`8001Aoo`002Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
?Woo00<007ooOol0YGoo0`0057oo000:Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0m
Ool2002WOol3000DOol000Yoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo03eoo`800:Mo
o`<001Aoo`002goo10001Woo0P00?Woo0`00YWoo0`0057oo000EOol00`00Oomoo`0mOol3002VOol3
000DOol001Eoo`03001oogoo03eoo`03001oo`000:Ioo`<001Aoo`005Goo00<007ooOol0?Goo00<0
07oo0000YWoo0P005Goo000EOol00`00Oomoo`0lOol01000Oomoo`00YWoo0P005Goo000EOol2000m
Ool01000Oomoo`00YWoo0P005Goo000EOol00`00Oomoo`0lOol01000Oomoo`000goo00<007ooOol0
X7oo00<007ooOol057oo000EOol00`00Oomoo`0lOol00`00Oomoo`05002ROol00`00Oomoo`0DOol0
01Eoo`03001oogoo03aoo`03001oogoo00D00:9oo`03001oogoo01Aoo`005Goo00<007ooOol0?7oo
00<007ooOol01@00XWoo00<007ooOol057oo000EOol2000mOol00`00Oomoo`05002ROol00`00Oomo
o`0DOol001Eoo`03001oogoo03Yoo`8000Aoo`D00:1oo`03001oogoo01Eoo`005Goo00<007ooOol0
>Woo0P0017oo0P0000Aoo`00000009moo`03001oogoo01Eoo`002goo10001Woo00<007ooOol0>Woo
0P001goo0`00Wgoo00<007ooOol05Goo000:Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomo
o`0hOol01000Ool000001goo1000WWoo00<007ooOol05Goo000:Ool00`00Oomoo`02Ool00`00Oomo
o`03Ool2000hOol200000goo00000008Ool3002NOol00`00Oomoo`0EOol00003Ool0000000D0009o
o`03001oogoo009oo`03001oogoo00=oo`03001oogoo03Moo`800003Ool0000000Uoo`8009ioo`03
001oogoo01Eoo`002Woo0P000goo00<007ooOol00goo00<007ooOol0<Woo00<007ooOol00Woo0P00
00=oo`00Ool02Goo0P00WWoo00<007ooOol05Goo000:Ool00`00Ool000020006Ool00`00Oomoo`0b
Ool00`00Oomoo`02Ool200000goo001oo`0:Ool00`00Ool0002KOol2000GOol000]oo`03001oogoo
00Moo`03001oogoo039oo`03001oogoo009oo`@000]oo`03001oo`0009]oo`03001oogoo01Ioo`00
2goo0P0027oo0P00<Woo0P0017oo10002goo00@007oo000009Yoo`03001oogoo01Ioo`003Goo0`00
1Goo00<007ooOol0<7oo10000goo0`0037oo1000VWoo00<007ooOol05Woo000EOol00`00Oomoo`0`
Ool40003Ool00`00Ool0000<Ool4002JOol00`00Oomoo`0FOol001Eoo`03001oogoo031oo`@000=o
o`03001oogoo00aoo`@009Uoo`8001Qoo`005Goo00<007ooOol0;goo1@000goo00<007ooOol037oo
1000VGoo0P0067oo000EOol2000`Ool50002Ool2000>Ool200000goo0000002GOol2000IOol001Eo
o`03001oogoo02moo`<00004Ool007ooOol2000AOol2002GOol2000IOol001Eoo`03001oogoo02mo
o`<00004Ool007ooOol2000AOol2002GOol2000IOol001Eoo`03001oogoo02moo`<00005Ool00000
Ool0000COol01000Oomoo`00U7oo00<007ooOol067oo000EOol00`00Oomoo`0_Ool30002Ool00`00
Ool0000COol01000Oomoo`002Goo00<007ooOol0R7oo00<007ooOol067oo000EOol00`00Oomoo`0_
Ool00`00Oomoo`02Ool00`00Ool0000COol01000Oomoo`0027oo0P00RWoo00<007ooOol067oo000E
Ool2000_Ool20004Ool00`00Ool0000COol00`00Oomoo`020007Ool2002:Ool00`00Oomoo`0HOol0
01Eoo`03001oogoo015oo`03001oogoo01Yoo`8000Aoo`03001oo`0001Aoo`04001oo`000004Ool5
002:Ool00`00Oomoo`0HOol000eoo`<000Eoo`03001oogoo015oo`03001oogoo01Yoo`8000Eoo`80
01Aoo`@000Aoo`D008Uoo`03001oogoo01Uoo`003Woo00<007ooOol017oo00<007ooOol047oo0P00
5Goo00<007ooOol017oo00<007ooOol017oo0P0057oo100017oo1@00RGoo00<007ooOol06Goo000:
Ool60005Ool00`00Oomoo`0<Ool01000Oomoogoo0P000Woo00<007ooOol047oo00<007ooOol017oo
00<007ooOol017oo0P0057oo100017oo0`0000=oo`00Ool0AGoo0P00@Goo00<007ooOol06Goo0000
0goo000000050002Ool01@00Oomoogoo00001Woo0P003Goo00T007ooOomoo`00Ool007oo00004Woo
00<007oo000017oo00<007ooOol017oo00<007ooOol04goo1@000goo0`0000=oo`00Ool0AGoo0P00
@Goo00<007ooOol06Goo000;Ool01000Oomoo`001Woo00<007ooOol02goo0P000goo00D007oo001o
o`00019oo`03001oo`0000Aoo`03001oogoo00Aoo`03001oogoo01=oo`D0009oo`05001oo`00001o
o`020015Ool20011Ool00`00Oomoo`0IOol000aoo`03001oo`0000Ioo`03001oogoo00]oo`8000=o
o`05001oo`00Ool0000BOol00`00Ool000020002Ool00`00Oomoo`0KOol01000Ool000000Woo00<0
07oo00000Woo0P00AGoo0P00@Goo00<007ooOol06Goo000<Ool00`00Ool00006Ool00`00Oomoo`0;
Ool20002Ool2000017oo000000004Goo0P0000=oo`00000010007Goo00@007oo0000009oo`03001o
o`00009oo`80041oo`04001oogooOol30011Ool00`00Oomoo`0IOol000eoo`8000Ioo`03001oogoo
00]oo`04001oo`00Ool200000goo00000002000@Ool9000OOol20002Ool00`00Oomoo`02Ool2000Z
Ool00`00Oomoo`0COol01000Oomoogoo0`00@Goo00<007ooOol06Goo000>Ool00`00Oomoo`04Ool2
000<Ool01000Ool007oo0P0000=oo`0000000P0047oo2@0087oo00@007oo000000Aoo`8002Yoo`05
001oogooOol0000>Ool00`00Oomoo`020002Ool3000oOol00`00Ool0000KOol001Eoo`03001oogoo
00Yoo`800003Ool007oo00800004Ool00000Ool2000?Ool50002Ool2000POol01000Ool0000017oo
0P0000=oo`00Ool09goo00<007ooOol00P003Woo1@0000=oo`0000000P0000=oo`00Ool0?7oo00<0
07oo00006goo000EOol00`00Oomoo`0:Ool2000017oo001oo`000Woo0P0000=oo`0000003Woo00<0
07ooOol00`000Woo0P0087oo00@007oo000000Eoo`<002Qoo`800003Ool0000000ioo`D00003Ool0
000000800003Ool007oo03aoo`03001oo`0001]oo`005Goo00<007ooOol02Woo00H007ooOol007oo
0002Ool200000goo0000000>Ool00`00Oomoo`05Ool00`00Oomoo`0OOol00`00Ool00007Ool2000X
Ool200000goo0000000>Ool500000goo00000004000lOol4000KOol001Eoo`03001oogoo00Yoo`03
001oogoo008000=oo`800005Ool00000Ool0000<Ool00`00Oomoo`05Ool00`00Oomoo`0OOol00`00
Ool00007Ool2000XOol5000>Ool5000017oo001oogoo0`00>7oo00@007ooOomoo`<001aoo`005Goo
0P002goo00<007ooOol00P000goo0P000Woo00<007oo000037oo00<007ooOol09goo0`001goo0P00
:7oo1P003Goo0`0000Aoo`00Ool0009oo`<003Qoo`80009oo`<001aoo`005Goo00<007ooOol02Woo
00<007ooOol00P0017oo00<007ooOol010002Goo00<007oo0000:Goo0`001goo0P00:7oo1P003Goo
0`0000Aoo`00Ool000=oo`8003Qoo`800003Ool00000008001aoo`005Goo00<007ooOol02Goo00<0
07ooOol00Woo00<007ooOol00Woo00<007ooOol010002Goo0`00:Woo00<007ooOol01Woo0P00:7oo
1P002goo00<007oo00000P0000Aoo`00Ool000Aoo`03001oogoo03Ioo`800003Ool00000008001ao
o`005Goo00<007ooOol02Goo00<007ooOol02Woo0P0000=oo`00Ool027oo0`00:Woo00<007ooOol0
1goo00<007ooOol09Woo1P002goo00@007oo0000009oo`03001oo`0000Aoo`03001oogoo03Ioo`80
0005Ool00000Ool0000LOol001Eoo`03001oogoo00Uoo`03001oogoo00Yoo`800003Ool007oo00Qo
o`<002Yoo`03001oogoo00Moo`03001oogoo02Ioo`04001oo`00Ool2000;Ool00`00Ool00003Ool0
0`00Ool00004Ool00`00Oomoo`0fOol200000goo001oo`0NOol001Eoo`8000Aoo`03001oogoo00=o
o`03001oogoo00Yoo`800003Ool007oo00Qoo`03001oo`0003Aoo`03001oogoo02Ioo`04001oogoo
Ool2000;Ool00`00Ool00003Ool30004Ool00`00Oomoo`0fOol200000goo001oo`0NOol001Eoo`07
001oogooOol007oo000017oo0P0037oo0P0000=oo`00Ool027oo00<007oo0000=7oo00<007ooOol0
9Woo00<007ooOol00Woo00<007ooOol02Goo0P0017oo0P001Woo00<007ooOol0=Goo0P0000=oo`00
Ool07Woo000EOol01P00Oomoogoo001oo`8000=oo`03001oogoo00]oo`800003Ool007oo00Qoo`03
001oo`0003Aoo`03001oogoo02Ioo`03001oogoo009oo`03001oogoo00Qoo`03001oo`0000Aoo`80
00Ioo`03001oogoo03Eoo`@001moo`002Woo1P001Goo00H007ooOomoo`00Ool20003Ool00`00Oomo
o`0;Ool200000goo001oo`08Ool00`00Ool0000dOol00`00Oomoo`0VOol00`00Oomoo`03Ool00`00
Oomoo`07Ool00`00Ool00004Ool20006Ool00`00Oomoo`0eOol4000OOol000]oo`05001oogooOol0
0005Ool01P00Oomoogoo001oo`8000=oo`03001oogoo00]oo`800003Ool007oo00Moo`03001oogoo
03Eoo`03001oogoo02Eoo`03001oogoo00Aoo`03001oogoo00Moo`03001oo`0000Aoo`03001oogoo
00Eoo`03001oogoo03Eoo`<0021oo`0037oo00<007ooOol01Woo0P000Woo10000goo00<007ooOol0
2goo00@007ooOol000Aoo`05001oogooOol0000gOol00`00Oomoo`0UOol00`00Oomoo`04Ool00`00
Oomoo`06Ool200000goo001oo`0;Ool00`00Oomoo`0cOol00`00Ool00002000POol00003Ool00000
00D000Eoo`03001oogoo00Eoo`03001oogoo00D000=oo`03001oogoo00ioo`03001oogoo009oo`05
001oogooOol0000gOol00`00Oomoo`0TOol20006Ool00`00Oomoo`06Ool2000>Ool00`00Oomoo`0c
Ool01@00Ool007oo000087oo000>Ool00`00Oomoo`04Ool01@00Oomoo`00Ool010000Woo00<007oo
Ool03Woo00<007ooOol00Woo00D007ooOomoo`0003Qoo`8002Aoo`8000Ioo`03001oogoo00Eoo`<0
00ioo`03001oogoo03=oo`<00003Ool007oo01moo`003goo00<007ooOol00goo00L007oo000007oo
001oo`020002Ool00`00Oomoo`0?Ool00`00Oomoo`020003Ool00`00Oomoo`0fOol2000TOol20006
Ool00`00Oomoo`05Ool2000?Ool00`00Oomoo`0bOol30002Ool00`00Oomoo`0NOol000moo`03001o
ogoo00=oo`07001oo`00001oo`00Ool00`0000=oo`00Ool047oo0P0000=oo`0000001000>7oo0P00
97oo00<007ooOol01Goo00<007ooOol01Goo0P003goo00<007ooOol0<Woo0`000Woo00<007ooOol0
7Woo000:Ool00`00Oomoo`02Ool00`00Oomoo`03Ool01`00Ool00000Ool007oo00D0015oo`800003
Ool0000000@003Qoo`8002=oo`8000Moo`03001oogoo00Eoo`8000moo`03001oogoo039oo`<0009o
o`03001oogoo01ioo`002goo10001Woo10000goo1@004Goo0P0000=oo`0000001000>7oo0P008goo
0P001goo00<007ooOol017oo0`0047oo00<007ooOol0<Goo0`008goo000EOol01000Ool000000goo
1@004Goo0P0000=oo`0000001000>Goo00<007ooOol08Goo0P001goo00<007ooOol017oo0P004Goo
00<007ooOol0<Goo0P0097oo000EOol00`00Ool00006Ool3000AOol200000goo000000070005Ool2
0007Ool2000LOol30005Ool30006Ool2000KOol40005Ool30004Ool200000goo0000000>Ool00`00
Oomoo`09Ool40006Ool20007Ool2000COol00`00Oomoo`050005Ool20007Ool20007Ool20005Ool0
01Eoo`03001oo`0000Ioo`<0019oo`800003Ool00000008000=oo`03001oogoo009oo`04001oogoo
0005Ool01000Oomoo`0077oo00<007ooOol00goo00@007ooOol000Eoo`04001oogoo000IOol20003
Ool00`00Oomoo`02Ool200000goo001oo`02Ool30002Ool00`00Oomoo`0;Ool00`00Oomoo`08Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool01000Oomoo`001Goo00@007ooOol0019oo`03001oogoo009o
o`03001oogoo00Aoo`04001oogoo0005Ool01000Oomoo`001Goo00@007ooOol000Aoo`005Goo00<0
07oo00001Woo0`004Woo0P0000=oo`00Ool00`001Woo00<007ooOol00Woo00D007ooOomoo`0000Ao
o`03001oogoo01Eoo`H000=oo`04001oogooOol20003Ool00`00Oomoo`02Ool00`00Oomoo`0FOol2
0003Ool01P00Oomoogoo001oo`@0009oo`8000Aoo`03001oogoo00Yoo`03001oogoo00Qoo`03001o
ogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol03goo00<007ooOol0
0Woo00<007ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool0
0`00Oomoo`01Ool001Eoo`<000Ioo`8001=oo`8000Aoo`03001oo`0000Eoo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`0EOol01@00Oomoogoo000017oo00@007ooOomoo`8000=oo`03
001oogoo009oo`03001oogoo01Ioo`03001oogoo009oo`06001oogooOol007oo1P0000=oo`00Ool0
0goo00<007ooOol02Woo0P002Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol0
0004Ool00`00Oomoo`0>Ool20004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo0000
17oo00D007ooOomoo`0000Aoo`03001oogoo005oo`005Goo0`0077oo00<007ooOol01Goo00<007oo
Ool00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo01Ioo`04001oogoo0004Ool0
1000Oomoogoo0P000goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol04Goo0P000goo00D0
07ooOomoo`00009oo`D00003Ool007oo00=oo`03001oogoo00Yoo`8000Yoo`@000Aoo`03001oogoo
009oo`05001oogooOol00004Ool00`00Oomoo`0>Ool20004Ool00`00Oomoo`03Ool00`00Oomoo`02
Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001oogoo005oo`005Goo0`009Goo00D0
07ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`0GOol00`00Ool00004Ool01000Oomo
ogoo0P000goo00<007ooOol00Woo00@007ooOomoo`8001=oo`03001oo`00008000Aoo`03001oogoo
00D00003Ool007oo00=oo`03001oogoo00Yoo`8000Uoo`03001oogoo009oo`05001oogooOol00004
Ool01@00Oomoogoo000017oo00<007ooOol03Woo0P0017oo00<007ooOol00goo00<007ooOol00Woo
00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`01Ool001Eoo`<002Eoo`05001o
ogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol05goo00<007oo000017oo00<007ooOol0
0Woo00D007ooOomoo`0000Aoo`04001oogooOol2000COol20007Ool00`00Oomoo`0500000goo001o
o`03Ool00`00Oomoo`0:Ool20009Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomo
o`0000Aoo`03001oogoo00aoo`04001oo`000004Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00
Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001oogoo005oo`005Goo0P008Goo00<007ooOol0
0Woo00<007ooOol00Woo00@007ooOol000Eoo`04001oogoo000KOol20005Ool00`00Oomoo`020004
Ool01000Oomoo`0017oo0P004Woo00@007oo000000Moo`03001oo`0000800005Ool007ooOol00002
Ool00`00Oomoo`0<Ool20008Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01000Oomoo`001Goo00@0
07ooOol000moo`04001oo`000002Ool30006Ool01000Oomoo`001Goo00@007ooOol000Eoo`04001o
ogoo0004Ool001Eoo`80029oo`@000Ioo`8000Moo`8001eoo`03001oogoo00Aoo`800003Ool007oo
00Aoo`8000Eoo`80019oo`04001oogooOol30005Ool400000goo001oo`02Ool2000@Ool00`00Oomo
o`07Ool40006Ool20007Ool2000@Ool00`00Ool00004Ool20007Ool20007Ool20007Ool20005Ool0
01Eoo`8005ioo`03001oogoo00Yoo`80019oo`03001oogoo00]oo`800003Ool007oo01Aoo`03001o
ogoo02aoo`03001oo`0002Ioo`005Goo0P00GWoo00<007ooOol02Woo0P004Woo00<007ooOol097oo
00<007ooOol0;7oo00<007oo00009Woo000EOol2001NOol00`00Oomoo`0:Ool00`00Ool0000AOol0
0`00Oomoo`0TOol00`00Oomoo`0/Ool3000VOol001Eoo`8005ioo`03001oogoo00Uoo`800003Ool0
07oo011oo`03001oogoo02Aoo`03001oogoo02aoo`8002Moo`003goonP0037oo000EOol20009Ool0
0`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`09Ool0
0`00Oomoo`09Ool00`00Oomoo`09Ool20009Ool01P00Ool00000Ool000Ioo`03001oogoo00Qoo`80
00]oo`03001oogoo00Uoo`03001oogoo00Uoo`03001oo`0000Uoo`03001oogoo00Uoo`03001oogoo
00Qoo`03001oogoo00Uoo`800003Ool007oo00Moo`03001oogoo00Uoo`03001oogoo011oo`005Goo
00<007ooOol0;7oo00<007ooOol0;Goo0P002Woo0P000Woo00<007ooOol03goo00<007ooOol02Woo
00<007ooOol05goo00<007ooOol04goo00<007ooOol05Goo00<007oo000057oo00<007ooOol047oo
000EOol00`00Oomoo`1MOol00`00Oomoo`08Ool20002Ool2000?Ool2000VOol00`00Oomoo`0[Ool0
0`00Ool0000WOol001Eoo`03001oogoo05eoo`8000Uoo`80009oo`8000moo`8002Moo`03001oogoo
02Yoo`03001oogoo02Moo`005Goo0P00GWoo0P0027oo0P000goo0P003goo0P009goo00<007ooOol0
:Woo00<007ooOol09goo000EOol00`00Oomoo`1NOol00`00Oomoo`06Ool20003Ool2000?Ool2000W
Ool00`00Oomoo`0ZOol00`00Oomoo`0WOol001Eoo`03001oogoo05ioo`03001oogoo00Ioo`8000=o
o`8000]oo`800003Ool0000002Qoo`03001oogoo02Yoo`03001oogoo02Moo`005Goo00<007ooOol0
GWoo0P001goo00<007ooOol00goo00<007ooOol02Goo1@00:7oo00<007ooOol0:Woo00<007ooOol0
9goo000EOol00`00Oomoo`1NOol30006Ool00`00Oomoo`03Ool20009Ool5000YOol00`00Oomoo`0Z
Ool00`00Oomoo`0WOol001Eoo`8005moo`<000Eoo`8000Ioo`03001oogoo00Moo`D002Uoo`03001o
ogoo02Qoo`03001oo`0002Uoo`005Goo00<007ooOol0GWoo0`001Goo0P001Woo00<007ooOol01Woo
1P00:Woo00<007ooOol09goo0`00:Goo000EOol00`00Oomoo`1NOol30005Ool20006Ool00`00Oomo
o`06Ool6000ZOol00`00Oomoo`0WOol2000ZOol001Eoo`03001oogoo05moo`8000Eoo`03001oogoo
00Eoo`03001oogoo00Ioo`<00003Ool0000002Yoo`03001oogoo02Moo`8002Yoo`005Goo00<007oo
Ool0H7oo00<007ooOol00goo00<007ooOol01Goo0P001goo0`0000=oo`00Ool0:Woo00<007ooOol0
7goo00<007ooOol01Goo0P00:Woo000EOol00`00Oomoo`1POol00`00Oomoo`03Ool00`00Oomoo`05
Ool20007Ool3000]Ool00`00Oomoo`0OOol00`00Oomoo`05Ool2000ZOol001Eoo`80065oo`03001o
ogoo00=oo`03001oogoo00Eoo`<000Eoo`<002ioo`03001oogoo01moo`03001oogoo00=oo`03001o
o`0002]oo`005Goo00<007ooOol0H7oo00<007ooOol00Woo00<007ooOol01Woo0`001Goo0`00;Woo
0`000goo00<007ooOol06Goo00<007ooOol00Woo0P0000=oo`00Ool0:Woo000:Ool60005Ool00`00
Oomoo`1QOol01@00Oomoogoo00002Goo0P0017oo0`00;goo0`000Woo0`006Woo0P000goo0P0000=o
o`00Ool0:Woo000;Ool01@00Oomoogoo00001Goo00<007ooOol0HGoo00D007ooOomoo`0000Uoo`80
00Aoo`<0031oo`L001Yoo`8000=oo`<002aoo`0037oo00<007ooOol01Woo00<007ooOol0HGoo00D0
07ooOomoo`0000Uoo`8000Aoo`03001oo`00031oo`P001Uoo`8000=oo`<002aoo`003Goo00<007oo
Ool01Goo0P00HWoo00D007ooOomoo`0000Yoo`03001oogoo009oo`03001oogoo031oo`P001Uoo`80
009oo`8002ioo`003Woo00<007ooOol017oo00<007ooOol0HGoo00D007ooOomoo`0000Yoo`06001o
ogoo001oo`00<Woo20006Goo0P0000Aoo`00000002ioo`003goo00<007ooOol00goo00<007ooOol0
HGoo00D007ooOomoo`0000Yoo`06001oogoo001oo`00<Woo0P0000=oo`0000000Woo00<007ooOol0
5Woo0`0000=oo`000000;goo000?Ool00`00Oomoo`03Ool00`00Oomoo`1QOol01@00Oomoogoo0000
2Woo00H007ooOol007oo000bOol20005Ool00`00Oomoo`0EOol400000goo0000000_Ool000Yoo`03
001oogoo009oo`03001oogoo00=oo`03001oogoo069oo`04001oo`00000:Ool00`00Oomoo`02000c
Ool20006Ool00`00Oomoo`0DOol20002Ool3000_Ool000]oo`@000Ioo`8006=oo`04001oo`00000:
Ool00`00Oomoo`02000kOol00`00Oomoo`0DOol20002Ool3000_Ool001Eoo`03001oogoo069oo`04
001oo`00000:Ool00`00Ool00002000kOol00`00Oomoo`0DOol01000Oomoogoo0`00;goo000EOol0
0`00Oomoo`1ROol4000:Ool01@00Ool007oo0000>goo00<007ooOol04goo0P000goo00<007ooOol0
;goo000EOol00`00Oomoo`1ROol3000;Ool300000goo001oo`0jOol00`00Oomoo`0COol00`00Oomo
o`0dOol001Eoo`03001oogoo069oo`<000]oo`<003eoo`8000Uoo`03001oogoo00Qoo`03001oogoo
03Aoo`005Goo0P00Hgoo0`0037oo0P00?Goo0P002Goo0P002Goo00<007ooOol0=7oo000EOol00`00
Oomoo`1ROol2000=Ool00`00Oomoo`0lOol20009Ool20004Ool00`00Oomoo`02Ool00`00Oomoo`0d
Ool001Eoo`03001oogoo069oo`8000eoo`03001oogoo03aoo`8000Uoo`<000=oo`03001oogoo009o
o`03001oogoo03Aoo`005Goo00<007ooOol0LGoo00<007ooOol0?Goo00<007ooOol01goo0`000goo
00<007ooOol00Woo00<007ooOol0=7oo000EOol00`00Oomoo`1aOol00`00Oomoo`0mOol00`00Oomo
o`07Ool30003Ool00`00Oomoo`02Ool00`00Oomoo`0dOol001Eoo`800;=oo`03001oogoo00Eoo`04
001oo`000003Ool00`00Oomoo`02Ool00`00Oomoo`0dOol001Eoo`03001oogoo0;9oo`03001oogoo
00Eoo`04001oo`000003Ool01000Oomoogoo0P00=Woo000EOol00`00Oomoo`2bOol00`00Oomoo`05
Ool01000Oomoo`000goo0P000Woo0P00=Woo000=Ool30005Ool00`00Oomoo`2bOol00`00Oomoo`05
Ool01P00Oomoogoo001oo`<0009oo`03001oogoo03Eoo`003Woo00<007ooOol017oo00<007ooOol0
/Woo00<007ooOol01Goo00H007ooOomoo`00Ool400000goo001oo`0fOol000Yoo`H000Eoo`800;=o
o`03001oogoo00Eoo`06001oogooOol007oo100000=oo`00Ool0=Woo000:Ool01@00Oomoogoo0000
1Woo00<007ooOol0/Woo00<007ooOol01Goo00@007ooOomoo`P003Moo`002goo00@007ooOol000Io
o`03001oogoo0;9oo`04001oogooOol300000goo001oo`02Ool30002Ool3000gOol000aoo`03001o
o`0000Ioo`03001oogoo0;9oo`04001oogooOol50003Ool30002Ool3000gOol000aoo`03001oo`00
00Ioo`03001oogoo0;9oo`04001oogooOol50003Ool30002Ool3000gOol000eoo`8000Ioo`03001o
ogoo0;9oo`04001oogooOol40004Ool30003Ool2000gOol000ioo`03001oogoo00Aoo`800;=oo`04
001oogooOol200000goo001oo`03Ool30003Ool2000gOol001Eoo`03001oogoo0;9oo`04001oogoo
Ool200000goo001oo`04Ool20003Ool00`00Oomoo`0fOol001Eoo`03001oogoo0;9oo`03001oogoo
00<00003Ool007oo00Aoo`8000=oo`03001oogoo03Ioo`005Goo00<007ooOol0/Woo00<007oo0000
0`001goo00<007ooOol00Woo00<007ooOol0=Woo000EOol00`00Oomoo`2bOol400000goo001oo`06
Ool00`00Oomoo`0kOol001Eoo`800;=oo`@004Moo`005Goo00<007ooOol0/Woo1000Agoo000EOol0
0`00Oomoo`2cOol00`00Oomoo`17Ool001Eoo`03001oogoo0;=oo`03001oogoo04Moo`005Goo00<0
07ooOol0oGoo000EOol2003nOol001Eoo`03001oogoo0?eoo`00ogoo5Woo003oOolFOol00?mooaIo
o`00ogoo5Woo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolFOol00001\
\>"],
  ImageRangeCache->{{{0, 276.5}, {170.375, 0}} -> {-88.9708, -10.0523, 
  6.31901, 0.145098}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["ListPlot[walkTWO, PlotJoined->True];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00095143 0.603319 0.00160854 [
[.2141 .59082 -9 -9 ]
[.2141 .59082 9 0 ]
[.40438 .59082 -9 -9 ]
[.40438 .59082 9 0 ]
[.59467 .59082 -9 -9 ]
[.59467 .59082 9 0 ]
[.78495 .59082 -9 -9 ]
[.78495 .59082 9 0 ]
[.97524 .59082 -12 -9 ]
[.97524 .59082 12 0 ]
[.01131 .04033 -24 -4.5 ]
[.01131 .04033 0 4.5 ]
[.01131 .12076 -24 -4.5 ]
[.01131 .12076 0 4.5 ]
[.01131 .20118 -24 -4.5 ]
[.01131 .20118 0 4.5 ]
[.01131 .28161 -24 -4.5 ]
[.01131 .28161 0 4.5 ]
[.01131 .36204 -24 -4.5 ]
[.01131 .36204 0 4.5 ]
[.01131 .44247 -24 -4.5 ]
[.01131 .44247 0 4.5 ]
[.01131 .52289 -18 -4.5 ]
[.01131 .52289 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2141 .60332 m
.2141 .60957 L
s
[(200)] .2141 .59082 0 1 Mshowa
.40438 .60332 m
.40438 .60957 L
s
[(400)] .40438 .59082 0 1 Mshowa
.59467 .60332 m
.59467 .60957 L
s
[(600)] .59467 .59082 0 1 Mshowa
.78495 .60332 m
.78495 .60957 L
s
[(800)] .78495 .59082 0 1 Mshowa
.97524 .60332 m
.97524 .60957 L
s
[(1000)] .97524 .59082 0 1 Mshowa
.125 Mabswid
.07138 .60332 m
.07138 .60707 L
s
.11895 .60332 m
.11895 .60707 L
s
.16652 .60332 m
.16652 .60707 L
s
.26167 .60332 m
.26167 .60707 L
s
.30924 .60332 m
.30924 .60707 L
s
.35681 .60332 m
.35681 .60707 L
s
.45195 .60332 m
.45195 .60707 L
s
.49952 .60332 m
.49952 .60707 L
s
.5471 .60332 m
.5471 .60707 L
s
.64224 .60332 m
.64224 .60707 L
s
.68981 .60332 m
.68981 .60707 L
s
.73738 .60332 m
.73738 .60707 L
s
.83252 .60332 m
.83252 .60707 L
s
.8801 .60332 m
.8801 .60707 L
s
.92767 .60332 m
.92767 .60707 L
s
.25 Mabswid
0 .60332 m
1 .60332 L
s
.02381 .04033 m
.03006 .04033 L
s
[(-350)] .01131 .04033 1 0 Mshowa
.02381 .12076 m
.03006 .12076 L
s
[(-300)] .01131 .12076 1 0 Mshowa
.02381 .20118 m
.03006 .20118 L
s
[(-250)] .01131 .20118 1 0 Mshowa
.02381 .28161 m
.03006 .28161 L
s
[(-200)] .01131 .28161 1 0 Mshowa
.02381 .36204 m
.03006 .36204 L
s
[(-150)] .01131 .36204 1 0 Mshowa
.02381 .44247 m
.03006 .44247 L
s
[(-100)] .01131 .44247 1 0 Mshowa
.02381 .52289 m
.03006 .52289 L
s
[(-50)] .01131 .52289 1 0 Mshowa
.125 Mabswid
.02381 .05642 m
.02756 .05642 L
s
.02381 .0725 m
.02756 .0725 L
s
.02381 .08859 m
.02756 .08859 L
s
.02381 .10467 m
.02756 .10467 L
s
.02381 .13684 m
.02756 .13684 L
s
.02381 .15293 m
.02756 .15293 L
s
.02381 .16901 m
.02756 .16901 L
s
.02381 .1851 m
.02756 .1851 L
s
.02381 .21727 m
.02756 .21727 L
s
.02381 .23336 m
.02756 .23336 L
s
.02381 .24944 m
.02756 .24944 L
s
.02381 .26553 m
.02756 .26553 L
s
.02381 .2977 m
.02756 .2977 L
s
.02381 .31378 m
.02756 .31378 L
s
.02381 .32987 m
.02756 .32987 L
s
.02381 .34595 m
.02756 .34595 L
s
.02381 .37812 m
.02756 .37812 L
s
.02381 .39421 m
.02756 .39421 L
s
.02381 .41029 m
.02756 .41029 L
s
.02381 .42638 m
.02756 .42638 L
s
.02381 .45855 m
.02756 .45855 L
s
.02381 .47464 m
.02756 .47464 L
s
.02381 .49072 m
.02756 .49072 L
s
.02381 .50681 m
.02756 .50681 L
s
.02381 .53898 m
.02756 .53898 L
s
.02381 .55506 m
.02756 .55506 L
s
.02381 .57115 m
.02756 .57115 L
s
.02381 .58723 m
.02756 .58723 L
s
.02381 .02425 m
.02756 .02425 L
s
.02381 .00816 m
.02756 .00816 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02476 .60332 m
.02571 .60273 L
.02666 .60214 L
.02762 .60155 L
.02857 .60096 L
.02952 .60038 L
.03047 .59979 L
.03142 .5992 L
.03237 .59861 L
.03332 .59802 L
.03428 .59743 L
.03523 .59684 L
.03618 .59626 L
.03713 .59567 L
.03808 .59508 L
.03903 .59449 L
.03998 .5939 L
.04094 .59331 L
.04189 .59272 L
.04284 .59214 L
.04379 .59155 L
.04474 .59096 L
.04569 .59037 L
.04664 .58978 L
.0476 .58919 L
.04855 .5886 L
.0495 .58802 L
.05045 .58743 L
.0514 .58684 L
.05235 .58625 L
.0533 .58566 L
.05426 .58507 L
.05521 .58448 L
.05616 .58389 L
.05711 .58331 L
.05806 .58272 L
.05901 .58213 L
.05996 .58154 L
.06092 .58095 L
.06187 .58036 L
.06282 .57977 L
.06377 .57919 L
.06472 .5786 L
.06567 .57801 L
.06662 .57742 L
.06758 .57683 L
.06853 .57624 L
.06948 .57565 L
.07043 .57507 L
.07138 .57448 L
Mistroke
.07233 .57389 L
.07328 .5733 L
.07424 .57271 L
.07519 .57212 L
.07614 .57153 L
.07709 .57095 L
.07804 .57036 L
.07899 .56977 L
.07994 .56918 L
.0809 .56859 L
.08185 .568 L
.0828 .56741 L
.08375 .56683 L
.0847 .56624 L
.08565 .56565 L
.0866 .56506 L
.08756 .56447 L
.08851 .56388 L
.08946 .56329 L
.09041 .56271 L
.09136 .56212 L
.09231 .56153 L
.09326 .56094 L
.09422 .56035 L
.09517 .55976 L
.09612 .55917 L
.09707 .55859 L
.09802 .558 L
.09897 .55741 L
.09992 .55682 L
.10088 .55623 L
.10183 .55564 L
.10278 .55505 L
.10373 .55446 L
.10468 .55388 L
.10563 .55329 L
.10658 .5527 L
.10754 .55211 L
.10849 .55152 L
.10944 .55093 L
.11039 .55034 L
.11134 .54976 L
.11229 .54917 L
.11324 .54858 L
.1142 .54799 L
.11515 .5474 L
.1161 .54681 L
.11705 .54622 L
.118 .54564 L
.11895 .54505 L
Mistroke
.1199 .54446 L
.12086 .54387 L
.12181 .54328 L
.12276 .54269 L
.12371 .5421 L
.12466 .54152 L
.12561 .54093 L
.12656 .54034 L
.12752 .53975 L
.12847 .53916 L
.12942 .53857 L
.13037 .53798 L
.13132 .5374 L
.13227 .53681 L
.13322 .53622 L
.13418 .53563 L
.13513 .53504 L
.13608 .53445 L
.13703 .53386 L
.13798 .53328 L
.13893 .53269 L
.13988 .5321 L
.14084 .53151 L
.14179 .53092 L
.14274 .53033 L
.14369 .52974 L
.14464 .52915 L
.14559 .52857 L
.14654 .52798 L
.1475 .52739 L
.14845 .5268 L
.1494 .52621 L
.15035 .52562 L
.1513 .52503 L
.15225 .52445 L
.1532 .52386 L
.15416 .52327 L
.15511 .52268 L
.15606 .52209 L
.15701 .5215 L
.15796 .52091 L
.15891 .52033 L
.15986 .51974 L
.16082 .51915 L
.16177 .51856 L
.16272 .51797 L
.16367 .51738 L
.16462 .51679 L
.16557 .51621 L
.16652 .51562 L
Mistroke
.16748 .51503 L
.16843 .51444 L
.16938 .51385 L
.17033 .51326 L
.17128 .51267 L
.17223 .51209 L
.17318 .5115 L
.17414 .51091 L
.17509 .51032 L
.17604 .50973 L
.17699 .50914 L
.17794 .50855 L
.17889 .50797 L
.17984 .50738 L
.1808 .50679 L
.18175 .5062 L
.1827 .50561 L
.18365 .50502 L
.1846 .50443 L
.18555 .50384 L
.1865 .50326 L
.18746 .50267 L
.18841 .50208 L
.18936 .50149 L
.19031 .5009 L
.19126 .50031 L
.19221 .49972 L
.19316 .49914 L
.19412 .49855 L
.19507 .49796 L
.19602 .49737 L
.19697 .49678 L
.19792 .49619 L
.19887 .4956 L
.19982 .49502 L
.20078 .49443 L
.20173 .49384 L
.20268 .49325 L
.20363 .49266 L
.20458 .49207 L
.20553 .49148 L
.20648 .4909 L
.20744 .49031 L
.20839 .48972 L
.20934 .48913 L
.21029 .48854 L
.21124 .48795 L
.21219 .48736 L
.21314 .48678 L
.2141 .48619 L
Mistroke
.21505 .4856 L
.216 .48501 L
.21695 .48442 L
.2179 .48383 L
.21885 .48324 L
.2198 .48266 L
.22076 .48207 L
.22171 .48148 L
.22266 .48089 L
.22361 .4803 L
.22456 .47971 L
.22551 .47912 L
.22646 .47853 L
.22742 .47795 L
.22837 .47736 L
.22932 .47677 L
.23027 .47618 L
.23122 .47559 L
.23217 .475 L
.23312 .47441 L
.23408 .47383 L
.23503 .47324 L
.23598 .47265 L
.23693 .47206 L
.23788 .47147 L
.23883 .47088 L
.23978 .47029 L
.24074 .46971 L
.24169 .46912 L
.24264 .46853 L
.24359 .46794 L
.24454 .46735 L
.24549 .46676 L
.24644 .46617 L
.2474 .46559 L
.24835 .465 L
.2493 .46441 L
.25025 .46382 L
.2512 .46323 L
.25215 .46264 L
.2531 .46205 L
.25406 .46147 L
.25501 .46088 L
.25596 .46029 L
.25691 .4597 L
.25786 .45911 L
.25881 .45852 L
.25976 .45793 L
.26072 .45735 L
.26167 .45676 L
Mistroke
.26262 .45617 L
.26357 .45558 L
.26452 .45499 L
.26547 .4544 L
.26642 .45381 L
.26738 .45322 L
.26833 .45264 L
.26928 .45205 L
.27023 .45146 L
.27118 .45087 L
.27213 .45028 L
.27308 .44969 L
.27404 .4491 L
.27499 .44852 L
.27594 .44793 L
.27689 .44734 L
.27784 .44675 L
.27879 .44616 L
.27974 .44557 L
.2807 .44498 L
.28165 .4444 L
.2826 .44381 L
.28355 .44322 L
.2845 .44263 L
.28545 .44204 L
.2864 .44145 L
.28736 .44086 L
.28831 .44028 L
.28926 .43969 L
.29021 .4391 L
.29116 .43851 L
.29211 .43792 L
.29306 .43733 L
.29402 .43674 L
.29497 .43616 L
.29592 .43557 L
.29687 .43498 L
.29782 .43439 L
.29877 .4338 L
.29972 .43321 L
.30068 .43262 L
.30163 .43204 L
.30258 .43145 L
.30353 .43086 L
.30448 .43027 L
.30543 .42968 L
.30638 .42909 L
.30734 .4285 L
.30829 .42791 L
.30924 .42733 L
Mistroke
.31019 .42674 L
.31114 .42615 L
.31209 .42556 L
.31304 .42497 L
.314 .42438 L
.31495 .42379 L
.3159 .42321 L
.31685 .42262 L
.3178 .42203 L
.31875 .42144 L
.3197 .42085 L
.32066 .42026 L
.32161 .41967 L
.32256 .41909 L
.32351 .4185 L
.32446 .41791 L
.32541 .41732 L
.32636 .41673 L
.32732 .41614 L
.32827 .41555 L
.32922 .41497 L
.33017 .41438 L
.33112 .41379 L
.33207 .4132 L
.33302 .41261 L
.33398 .41202 L
.33493 .41143 L
.33588 .41085 L
.33683 .41026 L
.33778 .40967 L
.33873 .40908 L
.33968 .40849 L
.34064 .4079 L
.34159 .40731 L
.34254 .40673 L
.34349 .40614 L
.34444 .40555 L
.34539 .40496 L
.34634 .40437 L
.3473 .40378 L
.34825 .40319 L
.3492 .4026 L
.35015 .40202 L
.3511 .40143 L
.35205 .40084 L
.353 .40025 L
.35396 .39966 L
.35491 .39907 L
.35586 .39848 L
.35681 .3979 L
Mistroke
.35776 .39731 L
.35871 .39672 L
.35966 .39613 L
.36062 .39554 L
.36157 .39495 L
.36252 .39436 L
.36347 .39378 L
.36442 .39319 L
.36537 .3926 L
.36632 .39201 L
.36728 .39142 L
.36823 .39083 L
.36918 .39024 L
.37013 .38966 L
.37108 .38907 L
.37203 .38848 L
.37298 .38789 L
.37394 .3873 L
.37489 .38671 L
.37584 .38612 L
.37679 .38554 L
.37774 .38495 L
.37869 .38436 L
.37964 .38377 L
.3806 .38318 L
.38155 .38259 L
.3825 .382 L
.38345 .38142 L
.3844 .38083 L
.38535 .38024 L
.3863 .37965 L
.38726 .37906 L
.38821 .37847 L
.38916 .37788 L
.39011 .3773 L
.39106 .37671 L
.39201 .37612 L
.39296 .37553 L
.39392 .37494 L
.39487 .37435 L
.39582 .37376 L
.39677 .37317 L
.39772 .37259 L
.39867 .372 L
.39962 .37141 L
.40058 .37082 L
.40153 .37023 L
.40248 .36964 L
.40343 .36905 L
.40438 .36847 L
Mistroke
.40533 .36788 L
.40628 .36729 L
.40724 .3667 L
.40819 .36611 L
.40914 .36552 L
.41009 .36493 L
.41104 .36435 L
.41199 .36376 L
.41294 .36317 L
.4139 .36258 L
.41485 .36199 L
.4158 .3614 L
.41675 .36081 L
.4177 .36023 L
.41865 .35964 L
.4196 .35905 L
.42056 .35846 L
.42151 .35787 L
.42246 .35728 L
.42341 .35669 L
.42436 .35611 L
.42531 .35552 L
.42626 .35493 L
.42722 .35434 L
.42817 .35375 L
.42912 .35316 L
.43007 .35257 L
.43102 .35199 L
.43197 .3514 L
.43292 .35081 L
.43388 .35022 L
.43483 .34963 L
.43578 .34904 L
.43673 .34845 L
.43768 .34786 L
.43863 .34728 L
.43958 .34669 L
.44054 .3461 L
.44149 .34551 L
.44244 .34492 L
.44339 .34433 L
.44434 .34374 L
.44529 .34316 L
.44624 .34257 L
.4472 .34198 L
.44815 .34139 L
.4491 .3408 L
.45005 .34021 L
.451 .33962 L
.45195 .33904 L
Mistroke
.4529 .33845 L
.45386 .33786 L
.45481 .33727 L
.45576 .33668 L
.45671 .33609 L
.45766 .3355 L
.45861 .33492 L
.45956 .33433 L
.46052 .33374 L
.46147 .33315 L
.46242 .33256 L
.46337 .33197 L
.46432 .33138 L
.46527 .3308 L
.46622 .33021 L
.46718 .32962 L
.46813 .32903 L
.46908 .32844 L
.47003 .32785 L
.47098 .32726 L
.47193 .32668 L
.47288 .32609 L
.47384 .3255 L
.47479 .32491 L
.47574 .32432 L
.47669 .32373 L
.47764 .32314 L
.47859 .32255 L
.47954 .32197 L
.4805 .32138 L
.48145 .32079 L
.4824 .3202 L
.48335 .31961 L
.4843 .31902 L
.48525 .31843 L
.4862 .31785 L
.48716 .31726 L
.48811 .31667 L
.48906 .31608 L
.49001 .31549 L
.49096 .3149 L
.49191 .31431 L
.49286 .31373 L
.49382 .31314 L
.49477 .31255 L
.49572 .31196 L
.49667 .31137 L
.49762 .31078 L
.49857 .31019 L
.49952 .30961 L
Mistroke
.50048 .30902 L
.50143 .30843 L
.50238 .30784 L
.50333 .30725 L
.50428 .30666 L
.50523 .30607 L
.50618 .30549 L
.50714 .3049 L
.50809 .30431 L
.50904 .30372 L
.50999 .30313 L
.51094 .30254 L
.51189 .30195 L
.51284 .30137 L
.5138 .30078 L
.51475 .30019 L
.5157 .2996 L
.51665 .29901 L
.5176 .29842 L
.51855 .29783 L
.5195 .29724 L
.52046 .29666 L
.52141 .29607 L
.52236 .29548 L
.52331 .29489 L
.52426 .2943 L
.52521 .29371 L
.52616 .29312 L
.52712 .29254 L
.52807 .29195 L
.52902 .29136 L
.52997 .29077 L
.53092 .29018 L
.53187 .28959 L
.53282 .289 L
.53378 .28842 L
.53473 .28783 L
.53568 .28724 L
.53663 .28665 L
.53758 .28606 L
.53853 .28547 L
.53948 .28488 L
.54044 .2843 L
.54139 .28371 L
.54234 .28312 L
.54329 .28253 L
.54424 .28194 L
.54519 .28135 L
.54614 .28076 L
.5471 .28018 L
Mistroke
.54805 .27959 L
.549 .279 L
.54995 .27841 L
.5509 .27782 L
.55185 .27723 L
.5528 .27664 L
.55376 .27606 L
.55471 .27547 L
.55566 .27488 L
.55661 .27429 L
.55756 .2737 L
.55851 .27311 L
.55946 .27252 L
.56042 .27193 L
.56137 .27135 L
.56232 .27076 L
.56327 .27017 L
.56422 .26958 L
.56517 .26899 L
.56612 .2684 L
.56708 .26781 L
.56803 .26723 L
.56898 .26664 L
.56993 .26605 L
.57088 .26546 L
.57183 .26487 L
.57278 .26428 L
.57374 .26369 L
.57469 .26311 L
.57564 .26252 L
.57659 .26193 L
.57754 .26134 L
.57849 .26075 L
.57944 .26016 L
.5804 .25957 L
.58135 .25899 L
.5823 .2584 L
.58325 .25781 L
.5842 .25722 L
.58515 .25663 L
.5861 .25604 L
.58706 .25545 L
.58801 .25487 L
.58896 .25428 L
.58991 .25369 L
.59086 .2531 L
.59181 .25251 L
.59276 .25192 L
.59372 .25133 L
.59467 .25075 L
Mistroke
.59562 .25016 L
.59657 .24957 L
.59752 .24898 L
.59847 .24839 L
.59942 .2478 L
.60038 .24721 L
.60133 .24662 L
.60228 .24604 L
.60323 .24545 L
.60418 .24486 L
.60513 .24427 L
.60608 .24368 L
.60704 .24309 L
.60799 .2425 L
.60894 .24192 L
.60989 .24133 L
.61084 .24074 L
.61179 .24015 L
.61274 .23956 L
.6137 .23897 L
.61465 .23838 L
.6156 .2378 L
.61655 .23721 L
.6175 .23662 L
.61845 .23603 L
.6194 .23544 L
.62036 .23485 L
.62131 .23426 L
.62226 .23368 L
.62321 .23309 L
.62416 .2325 L
.62511 .23191 L
.62606 .23132 L
.62702 .23073 L
.62797 .23014 L
.62892 .22956 L
.62987 .22897 L
.63082 .22838 L
.63177 .22779 L
.63272 .2272 L
.63368 .22661 L
.63463 .22602 L
.63558 .22544 L
.63653 .22485 L
.63748 .22426 L
.63843 .22367 L
.63938 .22308 L
.64034 .22249 L
.64129 .2219 L
.64224 .22132 L
Mistroke
.64319 .22073 L
.64414 .22014 L
.64509 .21955 L
.64604 .21896 L
.647 .21837 L
.64795 .21778 L
.6489 .21719 L
.64985 .21661 L
.6508 .21602 L
.65175 .21543 L
.6527 .21484 L
.65366 .21425 L
.65461 .21366 L
.65556 .21307 L
.65651 .21249 L
.65746 .2119 L
.65841 .21131 L
.65936 .21072 L
.66032 .21013 L
.66127 .20954 L
.66222 .20895 L
.66317 .20837 L
.66412 .20778 L
.66507 .20719 L
.66602 .2066 L
.66698 .20601 L
.66793 .20542 L
.66888 .20483 L
.66983 .20425 L
.67078 .20366 L
.67173 .20307 L
.67268 .20248 L
.67364 .20189 L
.67459 .2013 L
.67554 .20071 L
.67649 .20013 L
.67744 .19954 L
.67839 .19895 L
.67934 .19836 L
.6803 .19777 L
.68125 .19718 L
.6822 .19659 L
.68315 .19601 L
.6841 .19542 L
.68505 .19483 L
.686 .19424 L
.68696 .19365 L
.68791 .19306 L
.68886 .19247 L
.68981 .19188 L
Mistroke
.69076 .1913 L
.69171 .19071 L
.69266 .19012 L
.69362 .18953 L
.69457 .18894 L
.69552 .18835 L
.69647 .18776 L
.69742 .18718 L
.69837 .18659 L
.69932 .186 L
.70028 .18541 L
.70123 .18482 L
.70218 .18423 L
.70313 .18364 L
.70408 .18306 L
.70503 .18247 L
.70598 .18188 L
.70694 .18129 L
.70789 .1807 L
.70884 .18011 L
.70979 .17952 L
.71074 .17894 L
.71169 .17835 L
.71264 .17776 L
.7136 .17717 L
.71455 .17658 L
.7155 .17599 L
.71645 .1754 L
.7174 .17482 L
.71835 .17423 L
.7193 .17364 L
.72026 .17305 L
.72121 .17246 L
.72216 .17187 L
.72311 .17128 L
.72406 .1707 L
.72501 .17011 L
.72596 .16952 L
.72692 .16893 L
.72787 .16834 L
.72882 .16775 L
.72977 .16716 L
.73072 .16657 L
.73167 .16599 L
.73262 .1654 L
.73358 .16481 L
.73453 .16422 L
.73548 .16363 L
.73643 .16304 L
.73738 .16245 L
Mistroke
.73833 .16187 L
.73928 .16128 L
.74024 .16069 L
.74119 .1601 L
.74214 .15951 L
.74309 .15892 L
.74404 .15833 L
.74499 .15775 L
.74594 .15716 L
.7469 .15657 L
.74785 .15598 L
.7488 .15539 L
.74975 .1548 L
.7507 .15421 L
.75165 .15363 L
.7526 .15304 L
.75356 .15245 L
.75451 .15186 L
.75546 .15127 L
.75641 .15068 L
.75736 .15009 L
.75831 .14951 L
.75926 .14892 L
.76022 .14833 L
.76117 .14774 L
.76212 .14715 L
.76307 .14656 L
.76402 .14597 L
.76497 .14539 L
.76592 .1448 L
.76688 .14421 L
.76783 .14362 L
.76878 .14303 L
.76973 .14244 L
.77068 .14185 L
.77163 .14126 L
.77258 .14068 L
.77354 .14009 L
.77449 .1395 L
.77544 .13891 L
.77639 .13832 L
.77734 .13773 L
.77829 .13714 L
.77924 .13656 L
.7802 .13597 L
.78115 .13538 L
.7821 .13479 L
.78305 .1342 L
.784 .13361 L
.78495 .13302 L
Mistroke
.7859 .13244 L
.78686 .13185 L
.78781 .13126 L
.78876 .13067 L
.78971 .13008 L
.79066 .12949 L
.79161 .1289 L
.79256 .12832 L
.79352 .12773 L
.79447 .12714 L
.79542 .12655 L
.79637 .12596 L
.79732 .12537 L
.79827 .12478 L
.79922 .1242 L
.80018 .12361 L
.80113 .12302 L
.80208 .12243 L
.80303 .12184 L
.80398 .12125 L
.80493 .12066 L
.80588 .12008 L
.80684 .11949 L
.80779 .1189 L
.80874 .11831 L
.80969 .11772 L
.81064 .11713 L
.81159 .11654 L
.81254 .11595 L
.8135 .11537 L
.81445 .11478 L
.8154 .11419 L
.81635 .1136 L
.8173 .11301 L
.81825 .11242 L
.8192 .11183 L
.82016 .11125 L
.82111 .11066 L
.82206 .11007 L
.82301 .10948 L
.82396 .10889 L
.82491 .1083 L
.82586 .10771 L
.82682 .10713 L
.82777 .10654 L
.82872 .10595 L
.82967 .10536 L
.83062 .10477 L
.83157 .10418 L
.83252 .10359 L
Mistroke
.83348 .10301 L
.83443 .10242 L
.83538 .10183 L
.83633 .10124 L
.83728 .10065 L
.83823 .10006 L
.83918 .09947 L
.84014 .09889 L
.84109 .0983 L
.84204 .09771 L
.84299 .09712 L
.84394 .09653 L
.84489 .09594 L
.84584 .09535 L
.8468 .09477 L
.84775 .09418 L
.8487 .09359 L
.84965 .093 L
.8506 .09241 L
.85155 .09182 L
.8525 .09123 L
.85346 .09064 L
.85441 .09006 L
.85536 .08947 L
.85631 .08888 L
.85726 .08829 L
.85821 .0877 L
.85916 .08711 L
.86012 .08652 L
.86107 .08594 L
.86202 .08535 L
.86297 .08476 L
.86392 .08417 L
.86487 .08358 L
.86582 .08299 L
.86678 .0824 L
.86773 .08182 L
.86868 .08123 L
.86963 .08064 L
.87058 .08005 L
.87153 .07946 L
.87248 .07887 L
.87344 .07828 L
.87439 .0777 L
.87534 .07711 L
.87629 .07652 L
.87724 .07593 L
.87819 .07534 L
.87914 .07475 L
.8801 .07416 L
Mistroke
.88105 .07358 L
.882 .07299 L
.88295 .0724 L
.8839 .07181 L
.88485 .07122 L
.8858 .07063 L
.88676 .07004 L
.88771 .06946 L
.88866 .06887 L
.88961 .06828 L
.89056 .06769 L
.89151 .0671 L
.89246 .06651 L
.89342 .06592 L
.89437 .06534 L
.89532 .06475 L
.89627 .06416 L
.89722 .06357 L
.89817 .06298 L
.89912 .06239 L
.90008 .0618 L
.90103 .06121 L
.90198 .06063 L
.90293 .06004 L
.90388 .05945 L
.90483 .05886 L
.90578 .05827 L
.90674 .05768 L
.90769 .05709 L
.90864 .05651 L
.90959 .05592 L
.91054 .05533 L
.91149 .05474 L
.91244 .05415 L
.9134 .05356 L
.91435 .05297 L
.9153 .05239 L
.91625 .0518 L
.9172 .05121 L
.91815 .05062 L
.9191 .05003 L
.92006 .04944 L
.92101 .04885 L
.92196 .04827 L
.92291 .04768 L
.92386 .04709 L
.92481 .0465 L
.92576 .04591 L
.92672 .04532 L
.92767 .04473 L
Mistroke
.92862 .04415 L
.92957 .04356 L
.93052 .04297 L
.93147 .04238 L
.93242 .04179 L
.93338 .0412 L
.93433 .04061 L
.93528 .04003 L
.93623 .03944 L
.93718 .03885 L
.93813 .03826 L
.93908 .03767 L
.94004 .03708 L
.94099 .03649 L
.94194 .0359 L
.94289 .03532 L
.94384 .03473 L
.94479 .03414 L
.94574 .03355 L
.9467 .03296 L
.94765 .03237 L
.9486 .03178 L
.94955 .0312 L
.9505 .03061 L
.95145 .03002 L
.9524 .02943 L
.95336 .02884 L
.95431 .02825 L
.95526 .02766 L
.95621 .02708 L
.95716 .02649 L
.95811 .0259 L
.95906 .02531 L
.96002 .02472 L
.96097 .02413 L
.96192 .02354 L
.96287 .02296 L
.96382 .02237 L
.96477 .02178 L
.96572 .02119 L
.96668 .0206 L
.96763 .02001 L
.96858 .01942 L
.96953 .01884 L
.97048 .01825 L
.97143 .01766 L
.97238 .01707 L
.97334 .01648 L
.97429 .01589 L
.97524 .0153 L
Mistroke
.97619 .01472 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{185, 114.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004E0000Za000`40O003h00Oogoo5Woo003oOolF
Ool00?mooaIoo`00ogoo5Woo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolFOol00?mooaIoo`00
ogoo5Woo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolFOol002Ioo`03001oogoo0>aoo`009Woo
00<007ooOol0k7oo000VOol2003]Ool002Ioo`03001oogoo0=Uoo`03001oogoo011oo`009Woo00<0
07ooOol0egoo0`004Woo000VOol00`00Oomoo`3FOol2000DOol002Ioo`800=Eoo`<001Eoo`002goo
10001Goo10001Woo0P001Woo00<007ooOol0dgoo0P005goo000:Ool00`00Oomoo`02Ool01@00Oomo
ogoo000017oo00<007ooOol00Woo00@007ooOol000Eoo`03001oogoo0=5oo`8001Uoo`003goo00<0
07ooOol01Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`<00<moo`<001Yoo`0000=oo`000000
1@001goo00<007ooOol01Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo0<ioo`80
01aoo`0037oo0`0017oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001o
ogoo0<aoo`<001eoo`003goo00@007ooOomoo`D000Aoo`03001oogoo009oo`03001oogoo009oo`03
001oogoo0<]oo`8001moo`003goo00D007ooOomoo`0000Qoo`03001oogoo009oo`03001oogoo009o
o`800<Yoo`80025oo`002Woo00<007ooOol00Woo00D007ooOomoo`0000Uoo`04001oogoo0005Ool0
0`00Oomoo`37Ool3000ROol000]oo`@000Aoo`D000Ioo`8000Ioo`03001oogoo0<Ioo`8002Aoo`00
9Woo00<007ooOol0a7oo0P009Woo000VOol20034Ool2000WOol002Ioo`03001oogoo0<5oo`8002Uo
o`009Woo00<007ooOol0_goo0`00:Woo000VOol2002oOol2000/Ool002Ioo`03001oogoo0;aoo`80
02ioo`009Woo00<007ooOol0^Woo0`00;goo000VOol00`00Oomoo`2iOol2000aOol002Ioo`800;Qo
o`8003=oo`002goo10001Woo0P001goo0P001Woo00<007ooOol0]Woo0P00=7oo000:Ool00`00Oomo
o`02Ool00`00Oomoo`02Ool01000Oomoo`001Goo00@007ooOol000Eoo`03001oogoo0;Aoo`8003Io
o`003goo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`2b
Ool3000gOol00003Ool0000000D000Moo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<0
07ooOol00Woo0`00/Goo0P00>Goo000<Ool30004Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo
00<007ooOol00Woo00<007ooOol0[goo0`00>Woo000?Ool01@00Oomoogoo000017oo00D007ooOomo
o`0000Aoo`03001oogoo009oo`03001oogoo0:ioo`8003aoo`003goo00D007ooOomoo`0000Aoo`05
001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`2/Ool2000nOol000Yoo`03001oogoo009o
o`03001oogoo009oo`04001oogoo0005Ool01000Oomoo`001Goo0P00Zgoo0`00?goo000;Ool40006
Ool20007Ool20006Ool00`00Oomoo`2YOol20011Ool002Ioo`03001oogoo0:Moo`8004=oo`009Woo
0P00YWoo0`00A7oo000VOol00`00Oomoo`2TOol20016Ool002Ioo`03001oogoo0:9oo`<004Moo`00
9Woo00<007ooOol0XGoo0P00BGoo000VOol2002POol2001;Ool002Ioo`03001oogoo09eoo`<004ao
o`009Woo00<007ooOol0W7oo0P00CWoo000VOol00`00Oomoo`2JOol3001?Ool002Ioo`8009Uoo`<0
055oo`002Woo1P0017oo10001Woo0P001Woo00<007ooOol0Ugoo0P00Dgoo000;Ool01@00Oomoogoo
00000goo00<007ooOol00Woo00<007ooOol00Woo00@007ooOol000Eoo`03001oogoo09Eoo`<005Ao
o`0037oo00<007ooOol02Goo00D007ooOomoo`0000Aoo`03001oogoo009oo`<009Aoo`8005Ioo`00
00=oo`0000001@001Goo00<007ooOol027oo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001o
ogoo099oo`8005Qoo`003Woo00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03
001oogoo009oo`03001oogoo091oo`<005Uoo`003goo00@007ooOomoo`D000Aoo`03001oogoo009o
o`03001oogoo009oo`03001oogoo08moo`8005]oo`003goo00D007ooOomoo`0000Qoo`03001oogoo
009oo`03001oogoo009oo`8008ioo`<005aoo`002Woo00<007ooOol00Woo00D007ooOomoo`0000Uo
o`04001oogoo0005Ool00`00Oomoo`2;Ool3001NOol000]oo`@000Aoo`D000Ioo`8000Ioo`03001o
ogoo08Yoo`80061oo`009Woo00<007ooOol0R7oo0`00HGoo000VOol20028Ool2001SOol002Ioo`03
001oogoo08Eoo`8006Eoo`009Woo00<007ooOol0Q7oo0P00IWoo000VOol00`00Oomoo`22Ool2001X
Ool002Ioo`80085oo`<006Uoo`009Woo00<007ooOol0Ogoo0P00Jgoo000VOol00`00Oomoo`1mOol2
001]Ool002Ioo`8007aoo`<006ioo`002Woo1P001Goo0P001goo0P001Woo00<007ooOol0NWoo0P00
L7oo000;Ool01@00Oomoogoo000017oo00@007ooOol000Eoo`04001oogoo0005Ool00`00Oomoo`1h
Ool2001bOol000aoo`03001oogoo00Aoo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomo
o`02Ool00`00Oomoo`1gOol2001cOol00003Ool0000000D000Eoo`03001oogoo00=oo`03001oogoo
009oo`05001oogooOol00004Ool00`00Oomoo`02Ool3001eOol2001eOol000ioo`03001oogoo009o
o`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`1cOol3001fOol0
00moo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol0LWoo
0P00N7oo000?Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001o
ogoo071oo`<007Uoo`002Woo00<007ooOol00Woo00<007ooOol00Woo00@007ooOol000Eoo`04001o
ogoo0005Ool2001_Ool3001kOol000]oo`@000Ioo`8000Moo`8000Ioo`03001oogoo06eoo`8007eo
o`009Woo00<007ooOol0Jgoo0`00OWoo000VOol2001[Ool20020Ool002Ioo`03001oogoo06Qoo`80
089oo`009Woo00<007ooOol0IWoo0`00Pgoo000VOol00`00Oomoo`1UOol20025Ool002Ioo`8006Ao
o`<008Ioo`009Woo00<007ooOol0HWoo0P00R7oo000VOol00`00Oomoo`1POol2002:Ool002Ioo`03
001oogoo05ioo`<008]oo`009Woo0P00GWoo0P00SGoo000;Ool50004Ool40006Ool20006Ool00`00
Oomoo`1KOol2002?Ool000eoo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`04001o
ogoo0005Ool00`00Oomoo`1IOol3002@Ool000eoo`03001oogoo00Qoo`05001oogooOol00004Ool0
0`00Oomoo`02Ool00`00Oomoo`1HOol2002BOol00003Ool0000000D000Eoo`03001oogoo00Qoo`05
001oogooOol00004Ool00`00Oomoo`02Ool3001FOol3002COol000eoo`03001oogoo00=oo`03001o
ogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`1EOol2002EOol000eoo`03
001oogoo00=oo`D000Aoo`03001oogoo009oo`03001oogoo009oo`03001oogoo05=oo`8009Moo`00
3Goo00<007ooOol00goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol00Woo0P00DWoo0`00
V7oo000;Ool30005Ool00`00Oomoo`07Ool01000Oomoo`001Goo00<007ooOol0D7oo0P00VWoo000<
Ool20005Ool50006Ool20006Ool00`00Oomoo`1>Ool3002KOol002Ioo`03001oogoo04aoo`<009eo
o`009Woo0P00C7oo0P00Wgoo000VOol00`00Oomoo`19Ool3002POol002Ioo`03001oogoo04Qoo`80
0:9oo`009Woo00<007ooOol0AWoo0P00Y7oo000VOol20015Ool3002UOol002Ioo`03001oogoo04=o
o`800:Moo`009Woo00<007ooOol0@Goo0`00Z7oo000VOol20010Ool3002ZOol002Ioo`03001oogoo
03ioo`800:aoo`002goo1@001Goo0P001goo0P001Woo00<007ooOol0?7oo0`00[Goo000=Ool00`00
Oomoo`04Ool01000Oomoo`001Goo00@007ooOol000Eoo`03001oogoo03]oo`800:moo`003Goo00<0
07ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`<003Uoo`800;5o
o`0000=oo`0000001@001Goo00<007ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03
001oogoo009oo`03001oogoo03Qoo`800;9oo`003Goo00<007ooOol00goo00<007ooOol00Woo00D0
07ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo03Ioo`800;Aoo`003Goo00<007ooOol00goo
00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo03Aoo`<00;Eoo`00
3Goo00<007ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`8003Ao
o`800;Moo`002goo0`001Woo00@007ooOol000Eoo`04001oogoo0005Ool00`00Oomoo`0aOol3002h
Ool000aoo`8000Moo`8000Moo`8000Ioo`03001oogoo02moo`<00;Yoo`009Woo00<007ooOol0;Woo
0P00_7oo000VOol2000]Ool2002nOol002Ioo`03001oogoo02]oo`800;moo`009Woo00<007ooOol0
:Goo0P00`Goo000VOol2000XOol30032Ool002Ioo`03001oogoo02Ioo`800<Aoo`009Woo00<007oo
Ool097oo0`00aGoo000VOol00`00Oomoo`0SOol20037Ool002Ioo`80029oo`800<Uoo`0057oo1000
1Woo0P001Woo00<007ooOol07goo0`00bWoo000COol00`00Oomoo`02Ool00`00Oomoo`02Ool01000
Oomoo`001Goo00<007ooOol07Woo0P00c7oo000HOol01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol077oo0P00cWoo000:Ool70007Ool01@00Oomoogoo000017oo00<007ooOol00Woo0`00
6Woo0`00cgoo000COol00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007oo
Ool06Goo0P00dGoo000COol50004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0GOol3
003BOol001=oo`03001oogoo00Ioo`03001oogoo009oo`03001oogoo009oo`8001Moo`800=Aoo`00
4goo00<007ooOol01goo00@007ooOol000Eoo`03001oogoo01Aoo`800=Ioo`004goo1@001Woo0P00
1Woo00<007ooOol04Woo0`0027oo1P001Goo0P001goo0P006Goo0`001Goo0P001goo0P005goo1000
1Woo0P001goo0P005goo10001Woo0P001goo0P004Woo1@001Goo0P001goo0P001goo0P001Goo000V
Ool00`00Oomoo`0AOol2000;Ool01@00Oomoogoo000017oo00@007ooOol000Eoo`04001oogoo000I
Ool00`00Oomoo`03Ool01000Oomoo`001Goo00@007ooOol001Eoo`03001oogoo009oo`03001oogoo
009oo`04001oogoo0005Ool01000Oomoo`005Goo00<007ooOol00Woo00<007ooOol00Woo00@007oo
Ool000Eoo`04001oogoo000COol00`00Oomoo`04Ool01000Oomoo`001Goo00@007ooOol000Eoo`04
001oogoo0004Ool002Ioo`80011oo`8000ioo`03001oogoo00Aoo`03001oogoo009oo`05001oogoo
Ool00004Ool00`00Oomoo`0BOol60003Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007oo
Ool04Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`0B
Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001oogoo011oo`03
001oogoo00=oo`03001oogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007oo
Ool00Goo000VOol00`00Oomoo`0=Ool3000@Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomo
ogoo000017oo00<007ooOol04Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`0BOol00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Ao
o`03001oogoo019oo`03001oogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<0
07ooOol047oo00<007ooOol00goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol0
0004Ool00`00Oomoo`01Ool002Ioo`03001oogoo00aoo`8001=oo`03001oogoo009oo`03001oogoo
009oo`05001oogooOol00004Ool00`00Oomoo`0COol01000Oomoo`0017oo00<007ooOol00Woo00D0
07ooOomoo`0000Aoo`03001oogoo019oo`8000=oo`05001oogooOol00004Ool01@00Oomoogoo0000
17oo00<007ooOol04goo100017oo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo011o
o`03001oogoo00=oo`03001oogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<0
07ooOol00Goo000VOol00`00Oomoo`0:Ool3000EOol01@00Oomoogoo000017oo00D007ooOomoo`00
00Aoo`03001oogoo01Aoo`03001oo`0000Aoo`03001oogoo009oo`05001oogooOol00004Ool00`00
Oomoo`0BOol00`00Ool000020004Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol0
4Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`0@Ool0
0`00Oomoo`03Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`03001o
ogoo005oo`009Woo0P002Woo0P005goo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00
Oomoo`0DOol00`00Ool00004Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol04goo
00<007ooOol01Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo019oo`03001oogoo
009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00<007ooOol047oo00<007ooOol00goo
00<007ooOol00Woo00D007ooOomoo`0000Aoo`05001oogooOol00004Ool00`00Oomoo`01Ool002Io
o`03001oogoo00Moo`8001Aoo`03001oogoo009oo`03001oogoo009oo`04001oogoo0005Ool01000
Oomoo`0067oo0P001Goo00@007ooOol000Eoo`04001oogoo000FOol20007Ool01000Oomoo`001Goo
00@007ooOol001Eoo`03001oogoo009oo`03001oogoo009oo`04001oogoo0005Ool01000Oomoo`00
4Goo0`001Woo00@007ooOol000Eoo`04001oogoo0005Ool01000Oomoo`0017oo000VOol00`00Oomo
o`05Ool3000FOol40006Ool20007Ool2000JOol00`00Oomoo`04Ool20007Ool2000IOol30005Ool2
0007Ool2000GOol40006Ool20007Ool2000COol20007Ool20007Ool20007Ool20005Ool002Ioo`03
001oogoo00Aoo`800>Ioo`009Woo0P000goo0`00igoo000VOol00`00Oomoo`03003YOol002Ioo`04
001oo`00003[Ool0025oonP000aoo`009Woo00<007ooOol027oo00<007ooOol027oo00<007ooOol0
27oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol0
27oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol0
27oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol027oo00<007ooOol0
27oo00<007ooOol027oo00<007ooOol027oo00<007ooOol047oo000VOol00`00Oomoo`0YOol00`00
Oomoo`0YOol00`00Oomoo`0YOol00`00Oomoo`0YOol00`00Oomoo`0YOol00`00Oomoo`0@Ool002Io
o`03001oogoo0>aoo`009Woo00<007ooOol0k7oo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolF
Ool00?mooaIoo`00ogoo5Woo003oOolFOol00?mooaIoo`00ogoo5Woo003oOolFOol00?mooaIoo`00
ogoo5Woo003oOolFOol00001\
\>"],
  ImageRangeCache->{{{0, 276.5}, {170.375, 0}} -> {-177.068, -412.401, 
  6.83021, 4.03998}}]
}, Open  ]],

Cell[TextData[{
  "From this it is obvious what has happened in the function ",
  StyleBox["addrandTWO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". The function has added a constant number to the argument of ",
  StyleBox["addrandTWO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", that number being the first random number which was generated when the \
function was defined. The function ",
  StyleBox["addrandTWO",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " was displayed with the ",
  StyleBox["Random",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function replaced by a number when the function definition was entered \
and from that point on the function definition remained as such. \n\nNote the \
ease with which random walks may be constructed - we shall use this facility \
when carrying out Monte Carlo simulation of paths followed by stock prices."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Packages", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Although, as you have seen, there are many functions built-in to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", you may find that to cope with certain situations, you wish that there \
were functions available which were more specific to your task. Functions \
which extend the capability of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " are available in the Packages. The examples used here are based upon the \
Packages available with version 3.0 of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Earlier versions may not have the same capabilities.  The principles and \
techniques will remain the same for all versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ".  If you cannot duplicate these results exactly, investigate the Packages \
available with your version of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["A New Function for Fourier Transforms ", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The most efficient way of loading a Package is to use the ",
  StyleBox["Needs",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " command. There are other ways but this copes best with accidents, such as \
trying to load a package twice. Ordinarily, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can do numerical Fourier transforms using the ",
  StyleBox["Fourier",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function.  When we ask for information about that function we are told \
that this function operates on a list of numbers."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?Fourier", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Fourier[list] finds the discrete Fourier transform of a list of \
complex numbers."\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "There are no other built-in functions which will give us an analytical \
Fourier transform (unless we manage matters manually by using ",
  StyleBox["NIntegrate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ")."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?*Fourier*", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Fourier        InverseFourier", "Print"]
}, Open  ]],

Cell[TextData[{
  "However if we load the Package ",
  StyleBox["FourierTransform",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " which is in the set of Calculus Packages,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Needs[\"Calculus`FourierTransform`\"]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
we can now ask for a list of all the functions which that package \
has introduced.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?Calculus`FourierTransform`*", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
FourierCosSeriesCoefficient  FourierSinTransform          \
NFourierExpSeriesCoefficient
FourierCosTransform          FourierTransform             \
NFourierSinSeriesCoefficient
FourierExpSeries             FourierTrigSeries            NFourierTransform
FourierExpSeriesCoefficient  InverseFourierCosTransform   NFourierTrigSeries
FourierFrequencyConstant     InverseFourierSinTransform   \
NInverseFourierTransform
FourierOverallConstant       InverseFourierTransform      \
$FourierFrequencyConstant
FourierSample                NFourierCosSeriesCoefficient \
$FourierOverallConstant
FourierSinSeriesCoefficient  NFourierExpSeries\
\>", "Print",
  FontSize->7]
}, Open  ]],

Cell[TextData[{
  "And asking about the function ",
  StyleBox["FourierTransform",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", we find that this function will act on an algebraic expression."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["?FourierTransform", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("FourierTransform[expr, t, w] gives a function of w, which is the \
Fourier transform of expr, a function of t.  It is defined by \
FourierTransform[expr, t, w] = FourierOverallConstant * \
Integrate[Exp[FourierFrequencyConstant I w t] expr, {t, -Infinity, \
Infinity}]."\)], "Print"]
}, Open  ]],

Cell["\<\
We can now perform analytical Fourier transforms - here is a \
well-known example:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["FourierTransform[Exp[-t^2/s^2], t, w]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \(E\^\(\(-\(1\/4\)\)\ s\^2\ w\^2\)\ \@\[Pi]\ s\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["2.8 Further Graphics", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->True,
  AspectRatioFixed->True],

Cell["\<\
In the course of our investigations so far, we have produced a few \
plots of functions and data. Let's review this in a more systematic fashion. \
In this brief introduction we will only discuss standard built-in operations \
that are sufficient for simple graphics. \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Colours, Greys, Thickness and Dashing", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "As a useful preliminary we'll learn about the specification, and indeed \
the spelling of colours, and so on. This is a tricky section to write as this \
book is being written and published in Britain but refers to a software \
system that uses American English. When entering graphical programs, please \
get used to spelling colour as ",
  StyleBox["Color",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and grey as ",
  StyleBox["Gray",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "! In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " colours and greys are parametrized by intensities between 0 and 1. ",
  StyleBox["GrayLevel[0]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " denotes black, ",
  StyleBox["GrayLevel[1]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is white, and ",
  StyleBox["GrayLevel[0.5]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is an intermediate grey. Colours may be modelled in a variety of ways. \
The ",
  StyleBox["RGBColor",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function takes three arguments which denote the intensities of the red, \
green and blue components. So",
  StyleBox[" ",
    FontWeight->"Bold"],
  StyleBox["RGBColor[1, 0, 0]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is a pure bright red. Various other colour models can be used, such as",
  StyleBox[" CMYKColor",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", but we will focus on the RGB formulation. Useful graphical distinctions \
may also be made using dashing and thickness specifications. ",
  StyleBox["Thickness[x]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " specifies that lines which follow are to be drawn with a thickness ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", where ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is a fraction of the total plot width. ",
  StyleBox["Dashing[{u, v, w, x,...}]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " specifies that lines which follow have black and white segments of \
lengths ",
  StyleBox["u",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["v",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["w",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["x",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", . . .  etc., repeated cyclically. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plotting One or More Functions of One Variable", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The simplest plot command takes the form ",
  StyleBox["Plot[function, range]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". For example:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Plot[Sin[x^2], {x, -4, 4}];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.119048 0.309018 0.294304 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .01471 -12 -4.5 ]
[.4875 .01471 0 4.5 ]
[.4875 .16187 -24 -4.5 ]
[.4875 .16187 0 4.5 ]
[.4875 .45617 -18 -4.5 ]
[.4875 .45617 0 4.5 ]
[.4875 .60332 -6 -4.5 ]
[.4875 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-4)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-2)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(2)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(4)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.08333 .30902 m
.08333 .31277 L
s
.14286 .30902 m
.14286 .31277 L
s
.20238 .30902 m
.20238 .31277 L
s
.32143 .30902 m
.32143 .31277 L
s
.38095 .30902 m
.38095 .31277 L
s
.44048 .30902 m
.44048 .31277 L
s
.55952 .30902 m
.55952 .31277 L
s
.61905 .30902 m
.61905 .31277 L
s
.67857 .30902 m
.67857 .31277 L
s
.79762 .30902 m
.79762 .31277 L
s
.85714 .30902 m
.85714 .31277 L
s
.91667 .30902 m
.91667 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .01471 m
.50625 .01471 L
s
[(-1)] .4875 .01471 1 0 Mshowa
.5 .16187 m
.50625 .16187 L
s
[(-0.5)] .4875 .16187 1 0 Mshowa
.5 .45617 m
.50625 .45617 L
s
[(0.5)] .4875 .45617 1 0 Mshowa
.5 .60332 m
.50625 .60332 L
s
[(1)] .4875 .60332 1 0 Mshowa
.125 Mabswid
.5 .04415 m
.50375 .04415 L
s
.5 .07358 m
.50375 .07358 L
s
.5 .10301 m
.50375 .10301 L
s
.5 .13244 m
.50375 .13244 L
s
.5 .1913 m
.50375 .1913 L
s
.5 .22073 m
.50375 .22073 L
s
.5 .25016 m
.50375 .25016 L
s
.5 .27959 m
.50375 .27959 L
s
.5 .33845 m
.50375 .33845 L
s
.5 .36788 m
.50375 .36788 L
s
.5 .39731 m
.50375 .39731 L
s
.5 .42674 m
.50375 .42674 L
s
.5 .4856 m
.50375 .4856 L
s
.5 .51503 m
.50375 .51503 L
s
.5 .54446 m
.50375 .54446 L
s
.5 .57389 m
.50375 .57389 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .22429 m
.03279 .39753 L
.03793 .48549 L
.04262 .54788 L
.04517 .57275 L
.04634 .5818 L
.04758 .58969 L
.0489 .59615 L
.05012 .60029 L
.05143 .60279 L
.05218 .6033 L
.05288 .60318 L
.05412 .60158 L
.05529 .59843 L
.05634 .59428 L
.05749 .58835 L
.06006 .57001 L
.06244 .54715 L
.07382 .3816 L
.08454 .19783 L
.08922 .12851 L
.09417 .07046 L
.09675 .04771 L
.09812 .03797 L
.09959 .02939 L
.10094 .02322 L
.10217 .01905 L
.10341 .01626 L
.10458 .01492 L
.10574 .01483 L
.10682 .01585 L
.10802 .01819 L
.1093 .02212 L
.11057 .02743 L
.11192 .03448 L
.11431 .05068 L
.11862 .09018 L
.12332 .14608 L
.134 .30108 L
.14371 .4417 L
.14855 .50072 L
.1511 .52725 L
.1538 .5514 L
.15616 .56894 L
.15875 .58417 L
.15989 .58947 L
.16111 .59421 L
.16218 .59755 L
.16332 .60027 L
.16405 .60156 L
Mistroke
.16471 .60242 L
.16601 .60328 L
.16673 .60328 L
.16752 .60289 L
.16826 .60217 L
.16894 .60119 L
.17017 .59867 L
.17153 .59481 L
.17295 .5896 L
.17427 .58368 L
.1791 .55392 L
.18427 .50976 L
.20412 .27788 L
.21458 .15893 L
.22031 .1055 L
.22568 .06566 L
.23043 .03969 L
.2331 .02909 L
.23555 .02195 L
.23686 .01913 L
.23808 .01713 L
.23878 .01625 L
.23942 .01563 L
.24015 .01511 L
.24085 .01482 L
.24152 .01472 L
.24224 .0148 L
.24352 .01545 L
.24424 .01609 L
.2449 .01686 L
.24642 .01922 L
.24761 .02166 L
.2489 .02488 L
.25125 .03221 L
.25566 .05074 L
.26081 .07937 L
.26558 .11148 L
.28581 .28095 L
.30482 .43795 L
.31365 .49665 L
.32296 .54494 L
.32783 .56431 L
.33317 .58087 L
.33551 .58664 L
.33802 .59181 L
.34017 .59545 L
.34253 .59863 L
.34378 .59996 L
.34512 .60114 L
.34653 .6021 L
Mistroke
.34716 .60244 L
.34785 .60275 L
.34917 .60315 L
.35042 .60331 L
.35163 .60328 L
.35276 .60307 L
.35346 .60287 L
.35421 .60258 L
.35558 .60188 L
.35691 .60099 L
.35813 .59999 L
.36092 .59711 L
.36385 .59321 L
.37346 .57517 L
.384 .54832 L
.42444 .42439 L
.44309 .37568 L
.45293 .35484 L
.46337 .33684 L
.46862 .32945 L
.47442 .32261 L
.47974 .31754 L
.48463 .31393 L
.48963 .31125 L
.49225 .31027 L
.49357 .30988 L
.49501 .30954 L
.49629 .3093 L
.49745 .30915 L
.4981 .30909 L
.49879 .30905 L
.50006 .30902 L
.50125 .30905 L
.5025 .30915 L
.50357 .30928 L
.50475 .30949 L
.50605 .30978 L
.50725 .31011 L
.50997 .31108 L
.51237 .3122 L
.51493 .31364 L
.5198 .31716 L
.52421 .32119 L
.53499 .3344 L
.54511 .35114 L
.56525 .3961 L
.58374 .44875 L
.60421 .51311 L
.6142 .54321 L
.6233 .56755 L
Mistroke
.62782 .57799 L
.63274 .58768 L
.63738 .59496 L
.63945 .59753 L
.64167 .59981 L
.6429 .60084 L
.64402 .60163 L
.64516 .60228 L
.64621 .60275 L
.64744 .60312 L
.64809 .60324 L
.64878 .60331 L
.65006 .60327 L
.65122 .60305 L
.65252 .60259 L
.65372 .60195 L
.65504 .60102 L
.65643 .59976 L
.65877 .59699 L
.66134 .59298 L
.66658 .58155 L
.67154 .56652 L
.68084 .527 L
.69085 .46819 L
.70175 .38739 L
.72113 .21964 L
.73142 .1339 L
.73683 .09466 L
.74252 .06003 L
.74756 .03659 L
.75042 .0268 L
.75179 .02309 L
.75308 .02018 L
.75426 .01806 L
.75553 .01634 L
.75669 .0153 L
.75776 .0148 L
.75892 .01476 L
.76019 .01533 L
.76083 .01585 L
.76152 .01661 L
.76276 .01845 L
.76403 .02096 L
.7652 .02388 L
.76786 .03257 L
.7702 .04264 L
.7727 .05579 L
.78177 .12375 L
.79208 .23232 L
.80322 .36868 L
Mistroke
.81284 .48063 L
.81794 .53 L
.82074 .55274 L
.8234 .57082 L
.82599 .5849 L
.82735 .5908 L
.8288 .59586 L
.83002 .59913 L
.83071 .60058 L
.83136 .60166 L
.8325 .60291 L
.83373 .60332 L
.8349 .60279 L
.83618 .60119 L
.83683 .59996 L
.83752 .59835 L
.83876 .59465 L
.84117 .58454 L
.84337 .57207 L
.84857 .53043 L
.85421 .46831 L
.86444 .32468 L
.87473 .17223 L
.87976 .10819 L
.88435 .06162 L
.88683 .0424 L
.88918 .02859 L
.89047 .02296 L
.89167 .01902 L
.89298 .01615 L
.89364 .01528 L
.89438 .01478 L
.89507 .01476 L
.8958 .01521 L
.89711 .01725 L
.89784 .01907 L
.89852 .02121 L
.90006 .02763 L
.90141 .03503 L
.90264 .04318 L
.9054 .06626 L
.91023 .12129 L
.91536 .19626 L
.92455 .35347 L
.92932 .43404 L
.93444 .50988 L
.93738 .54527 L
.94007 .57107 L
.94136 .58097 L
.94277 .5898 L
Mistroke
.94341 .59313 L
.9441 .59617 L
.94534 .6003 L
.94626 .60225 L
.94727 .60325 L
.94833 .60302 L
.94931 .60162 L
.95024 .59927 L
.95108 .59625 L
.95205 .59173 L
.95297 .58643 L
.95502 .57114 L
.95692 .55286 L
.96123 .49813 L
.96848 .37375 L
.97619 .22429 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{195, 120.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004T0000]1000`40O003h00Oogoo9Goo003oOolU
Ool0099oo`03001oogoo08moo`00TWoo00<007ooOol0Sgoo0028Ool50005Ool00`00Oomoo`2?Ool0
08Yoo`03001oogoo00Eoo`03001oogoo08moo`00RWoo00<007ooOol01Goo00<007ooOol0Sgoo000R
Ool2000TOol3000cOol70005Ool00`00Oomoo`05Ool30016Ool2000TOol2000QOol0029oo`03001o
o`0002=oo`04001oogoo000nOol00`00Oomoo`05Ool00`00Oomoo`15Ool01000Oomoo`008goo00<0
07oo000087oo000ROol00`00Ool0000SOol00`00Oomoo`02000mOol00`00Oomoo`05Ool00`00Oomo
o`14Ool01@00Oomoogoo00008goo00<007oo000087oo000ROol00`00Ool0000SOol01@00Oomoogoo
0000?Goo00<007ooOol01Goo00<007ooOol0A7oo00D007ooOomoo`0002=oo`03001oo`00021oo`00
8Goo00@007ooOol0029oo`03001oogoo009oo`03001oogoo03Uoo`<000Moo`03001oogoo04Aoo`03
001oogoo009oo`03001oogoo01moo`04001oogoo000POol0025oo`05001oogooOol0000QOol00`00
Oomoo`02Ool00`00Oomoo`0jOol20007Ool00`00Oomoo`13Ool00`00Oomoo`03Ool00`00Oomoo`0O
Ool01@00Oomoogoo00007goo000QOol01@00Oomoogoo000087oo00<007ooOol00goo00<007ooOol0
@goo00<007ooOol0@goo00<007ooOol00goo00<007ooOol07goo00D007ooOomoo`0001moo`008Goo
00D007ooOomoo`00021oo`03001oogoo00=oo`03001oogoo04=oo`03001oogoo04=oo`03001oogoo
00Aoo`03001oogoo01ioo`05001oogooOol0000OOol0025oo`03001oogoo009oo`03001oogoo01eo
o`03001oogoo00Aoo`03001oogoo049oo`8004Aoo`03001oogoo00Aoo`03001oogoo01ioo`05001o
ogooOol0000OOol0025oo`03001oogoo009oo`03001oogoo01eoo`03001oogoo00Aoo`03001oogoo
049oo`03001oogoo04=oo`03001oogoo00Aoo`03001oogoo01ioo`05001oogooOol0000OOol0025o
o`03001oogoo009oo`03001oogoo01eoo`03001oogoo00Aoo`03001oogoo049oo`03001oogoo049o
o`03001oogoo00Eoo`03001oogoo01ioo`05001oogooOol0000OOol0025oo`03001oogoo009oo`03
001oogoo01aoo`03001oogoo00Eoo`03001oogoo049oo`03001oogoo049oo`03001oogoo00Eoo`03
001oogoo01eoo`03001oogoo009oo`03001oogoo01eoo`008Goo00<007ooOol00Woo00<007ooOol0
77oo00<007ooOol01Goo00<007ooOol0@Woo00<007ooOol0@Woo00<007ooOol01Goo00<007ooOol0
7Goo00<007ooOol00goo00<007ooOol077oo000POol00`00Oomoo`03Ool00`00Oomoo`0LOol00`00
Oomoo`06Ool00`00Oomoo`11Ool00`00Oomoo`12Ool00`00Oomoo`05Ool00`00Oomoo`0MOol00`00
Oomoo`03Ool00`00Oomoo`0LOol0021oo`03001oogoo00=oo`03001oogoo01aoo`03001oogoo00Io
o`03001oogoo045oo`03001oogoo049oo`03001oogoo00Eoo`03001oogoo01eoo`03001oogoo00=o
o`03001oogoo01aoo`0087oo00<007ooOol00goo00<007ooOol077oo00<007ooOol01Woo00<007oo
Ool0@Goo00<007ooOol0@Goo00<007ooOol01goo00<007ooOol077oo00<007ooOol00goo00<007oo
Ool077oo000POol00`00Oomoo`04Ool00`00Oomoo`0KOol00`00Oomoo`06Ool00`00Oomoo`11Ool2
0012Ool00`00Oomoo`07Ool00`00Oomoo`0LOol00`00Oomoo`03Ool00`00Oomoo`0LOol0021oo`03
001oogoo00Aoo`03001oogoo01]oo`03001oogoo00Moo`03001oogoo041oo`03001oogoo045oo`03
001oogoo00Moo`03001oogoo01aoo`03001oogoo00=oo`03001oogoo01aoo`0087oo00<007ooOol0
17oo00<007ooOol06Woo00<007ooOol027oo00<007ooOol0@7oo00<007ooOol0@Goo00<007ooOol0
1goo00<007ooOol077oo00<007ooOol00goo00<007ooOol077oo000POol00`00Oomoo`04Ool00`00
Oomoo`0JOol00`00Oomoo`08Ool00`00Oomoo`10Ool00`00Oomoo`11Ool00`00Oomoo`07Ool00`00
Oomoo`0LOol00`00Oomoo`03Ool00`00Oomoo`0LOol001moo`03001oogoo00Eoo`03001oogoo01Yo
o`03001oogoo00Qoo`03001oogoo041oo`03001oogoo041oo`03001oogoo00Qoo`03001oogoo01]o
o`03001oogoo00Aoo`03001oogoo01aoo`007goo00<007ooOol01Goo00<007ooOol06Woo00<007oo
Ool027oo00<007ooOol0@7oo00<007ooOol0@7oo00<007ooOol02Goo00<007ooOol06Woo00<007oo
Ool017oo00<007ooOol077oo000OOol00`00Oomoo`05Ool00`00Oomoo`0JOol00`00Oomoo`08Ool0
0`00Oomoo`10Ool00`00Oomoo`10Ool00`00Oomoo`09Ool00`00Oomoo`0JOol00`00Oomoo`04Ool0
0`00Oomoo`0LOol001moo`03001oogoo00Eoo`03001oogoo01Uoo`03001oogoo00Yoo`03001oogoo
03moo`03001oogoo041oo`03001oogoo00Uoo`03001oogoo01Yoo`03001oogoo00Eoo`03001oogoo
01]oo`007goo00<007ooOol01Goo00<007ooOol06Goo00<007ooOol02Woo00<007ooOol0?goo00<0
07ooOol0@7oo00<007ooOol02Goo00<007ooOol06Woo00<007ooOol01Goo00<007ooOol06goo000O
Ool00`00Oomoo`05Ool00`00Oomoo`0IOol00`00Oomoo`0:Ool00`00Oomoo`0oOol20011Ool00`00
Oomoo`09Ool00`00Oomoo`0JOol00`00Oomoo`05Ool00`00Oomoo`0KOol001moo`03001oogoo00Eo
o`03001oogoo01Uoo`03001oogoo00Yoo`03001oogoo03moo`03001oogoo041oo`03001oogoo00Uo
o`03001oogoo01Yoo`03001oogoo00Eoo`03001oogoo01]oo`007goo00<007ooOol01Goo00<007oo
Ool06Goo00<007ooOol02Woo00<007ooOol0?goo00<007ooOol0@7oo00<007ooOol02Goo00<007oo
Ool06Woo00<007ooOol01Goo00<007ooOol06goo000OOol00`00Oomoo`05Ool00`00Oomoo`0IOol0
0`00Oomoo`0:Ool00`00Oomoo`0oOol00`00Oomoo`0oOol00`00Oomoo`0;Ool00`00Oomoo`0IOol0
0`00Oomoo`05Ool00`00Oomoo`0KOol001ioo`03001oogoo00Ioo`03001oogoo01Uoo`03001oogoo
00Yoo`03001oogoo03moo`03001oogoo03moo`03001oogoo00]oo`03001oogoo01Uoo`03001oogoo
00Eoo`03001oogoo01]oo`007Woo00<007ooOol01goo00<007ooOol067oo00<007ooOol02Woo00<0
07ooOol0?goo00<007ooOol0?goo00<007ooOol02goo00<007ooOol06Goo00<007ooOol01Goo00<0
07ooOol06goo000NOol00`00Oomoo`07Ool00`00Oomoo`0HOol00`00Oomoo`0:Ool00`00Oomoo`0o
Ool00`00Oomoo`0oOol00`00Oomoo`0;Ool00`00Oomoo`0HOol00`00Oomoo`06Ool00`00Oomoo`0K
Ool001ioo`03001oogoo00Moo`03001oogoo01Qoo`03001oogoo00]oo`03001oogoo03ioo`03001o
ogoo03moo`03001oogoo00]oo`03001oogoo01Qoo`03001oogoo00Ioo`03001oogoo01]oo`007Woo
00<007ooOol01goo00<007ooOol05goo00<007ooOol037oo00<007ooOol0?Woo0P00@7oo00<007oo
Ool02goo00<007ooOol067oo00<007ooOol01Woo00<007ooOol06goo000NOol00`00Oomoo`07Ool0
0`00Oomoo`0GOol00`00Oomoo`0<Ool00`00Oomoo`0nOol00`00Oomoo`0oOol00`00Oomoo`0;Ool0
0`00Oomoo`0HOol00`00Oomoo`06Ool00`00Oomoo`0KOol001ioo`03001oogoo00Moo`03001oogoo
01Moo`03001oogoo00aoo`03001oogoo03ioo`03001oogoo03moo`03001oogoo00]oo`03001oogoo
01Qoo`03001oogoo00Ioo`03001oogoo01]oo`007Woo00<007ooOol01goo00<007ooOol05goo00<0
07ooOol037oo00<007ooOol0?Woo00<007ooOol0?goo00<007ooOol037oo00<007ooOol05goo00<0
07ooOol01goo00<007ooOol06Woo000NOol00`00Oomoo`07Ool00`00Oomoo`0GOol00`00Oomoo`0<
Ool00`00Oomoo`0nOol00`00Oomoo`0oOol00`00Oomoo`0<Ool00`00Oomoo`0GOol00`00Oomoo`07
Ool00`00Oomoo`0JOol001ioo`03001oogoo00Moo`03001oogoo01Moo`03001oogoo00aoo`03001o
ogoo03ioo`03001oogoo03ioo`03001oogoo00eoo`03001oogoo01Moo`03001oogoo00Moo`03001o
ogoo01Yoo`007Woo00<007ooOol01goo00<007ooOol05goo00<007ooOol037oo00<007ooOol08goo
0P001goo0`001Goo10001Woo00<007ooOol0?Woo00<007ooOol03Goo00<007ooOol05goo00<007oo
Ool01goo00<007ooOol06Woo000NOol00`00Oomoo`07Ool00`00Oomoo`0GOol00`00Oomoo`0=Ool0
0`00Oomoo`0QOol01000Oomoo`001Woo0`0017oo00<007ooOol00Woo00<007ooOol00goo00<007oo
Ool0?Woo00<007ooOol03Goo00<007ooOol05Woo00<007ooOol027oo00<007ooOol06Woo000NOol0
0`00Oomoo`07Ool00`00Oomoo`0GOol00`00Oomoo`0=Ool00`00Oomoo`0POol00`00Oomoo`02Ool0
0`00Oomoo`03Ool30009Ool00`00Oomoo`03Ool3000nOol00`00Oomoo`0=Ool00`00Oomoo`0FOol0
0`00Oomoo`08Ool00`00Oomoo`0JOol001eoo`03001oogoo00Qoo`03001oogoo01Moo`03001oogoo
00eoo`03001oogoo01Moo`L0009oo`03001oogoo009oo`03001oogoo00moo`03001oogoo00=oo`03
001oogoo03ioo`03001oogoo00eoo`03001oogoo01Ioo`03001oogoo00Qoo`03001oogoo01Yoo`00
7Goo00<007ooOol027oo00<007ooOol05goo00<007ooOol03Goo00<007ooOol087oo00<007ooOol0
0Woo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0?Woo00<007ooOol0
3Goo00<007ooOol05Woo00<007ooOol027oo00<007ooOol06Woo000MOol00`00Oomoo`08Ool00`00
Oomoo`0GOol00`00Oomoo`0=Ool00`00Oomoo`0POol00`00Oomoo`02Ool00`00Oomoo`0:Ool50006
Ool00`00Oomoo`0nOol00`00Oomoo`0=Ool00`00Oomoo`0FOol00`00Oomoo`08Ool00`00Oomoo`0J
Ool001eoo`03001oogoo00Uoo`03001oogoo01Ioo`03001oogoo00eoo`03001oogoo021oo`03001o
ogoo009oo`03001oogoo00Yoo`03001oogoo00Qoo`03001oogoo03ioo`03001oogoo00eoo`03001o
ogoo01Ioo`03001oogoo00Qoo`03001oogoo01Yoo`007Goo00<007ooOol02Goo00<007ooOol05Woo
00<007ooOol03Goo00<007ooOol08Goo00@007ooOol000eoo`03001oogoo00Qoo`03001oogoo03eo
o`03001oogoo00moo`03001oogoo01Eoo`03001oogoo00Qoo`03001oogoo01Yoo`007Goo00<007oo
Ool02Goo00<007ooOol05Goo00<007ooOol03Woo00<007ooOol08Woo0P003Woo1@001Woo00<007oo
Ool0?Goo00<007ooOol03goo00<007ooOol05Goo00<007ooOol02Goo00<007ooOol06Goo000MOol0
0`00Oomoo`09Ool00`00Oomoo`0EOol00`00Oomoo`0?Ool00`00Oomoo`0lOol00`00Oomoo`0mOol0
0`00Oomoo`0?Ool00`00Oomoo`0EOol00`00Oomoo`09Ool00`00Oomoo`0IOol001eoo`03001oogoo
00Uoo`03001oogoo01Eoo`03001oogoo00moo`03001oogoo03aoo`8003ioo`03001oogoo00moo`03
001oogoo01Eoo`03001oogoo00Uoo`03001oogoo01Uoo`007Goo00<007ooOol02Goo00<007ooOol0
5Goo00<007ooOol03goo00<007ooOol0?7oo00<007ooOol0?Goo00<007ooOol03goo00<007ooOol0
5Goo00<007ooOol02Goo00<007ooOol06Goo000MOol00`00Oomoo`09Ool00`00Oomoo`0EOol00`00
Oomoo`0?Ool00`00Oomoo`0lOol00`00Oomoo`0mOol00`00Oomoo`0?Ool00`00Oomoo`0EOol00`00
Oomoo`09Ool00`00Oomoo`0IOol001eoo`03001oogoo00Uoo`03001oogoo01Eoo`03001oogoo00mo
o`03001oogoo03aoo`03001oogoo03eoo`03001oogoo00moo`03001oogoo01Aoo`03001oogoo00Yo
o`03001oogoo01Uoo`007Goo00<007ooOol02Goo00<007ooOol05Goo00<007ooOol03goo00<007oo
Ool0?7oo00<007ooOol0?Goo00<007ooOol03goo00<007ooOol057oo00<007ooOol02Woo00<007oo
Ool06Goo000MOol00`00Oomoo`09Ool00`00Oomoo`0EOol00`00Oomoo`0?Ool00`00Oomoo`0lOol0
0`00Oomoo`0lOol00`00Oomoo`0@Ool00`00Oomoo`0DOol00`00Oomoo`0:Ool00`00Oomoo`0IOol0
01eoo`03001oogoo00Uoo`03001oogoo01Eoo`03001oogoo00moo`03001oogoo03aoo`03001oogoo
03aoo`03001oogoo011oo`03001oogoo01Aoo`03001oogoo00Yoo`03001oogoo01Uoo`007Goo00<0
07ooOol02Goo00<007ooOol05Goo00<007ooOol047oo00<007ooOol0>goo00<007ooOol0?7oo00<0
07ooOol047oo00<007ooOol057oo00<007ooOol02Woo00<007ooOol06Goo000MOol00`00Oomoo`09
Ool00`00Oomoo`0EOol00`00Oomoo`0@Ool00`00Oomoo`0kOol00`00Oomoo`0lOol00`00Oomoo`0A
Ool00`00Oomoo`0COol00`00Oomoo`0:Ool00`00Oomoo`0IOol001eoo`03001oogoo00Uoo`03001o
ogoo01Aoo`03001oogoo015oo`03001oogoo03]oo`8003eoo`03001oogoo015oo`03001oogoo01=o
o`03001oogoo00Yoo`03001oogoo01Uoo`0037oo00<007ooOol03Woo00<007ooOol02Woo00<007oo
Ool04goo00<007ooOol04Goo00<007ooOol0>goo00<007ooOol0?7oo00<007ooOol04Goo00<007oo
Ool04goo00<007ooOol02Woo00<007ooOol03Woo00<007ooOol027oo000<Ool00`00Oomoo`0>Ool0
0`00Oomoo`0:Ool00`00Oomoo`0COol00`00Oomoo`0AOol00`00Oomoo`0kOol00`00Oomoo`0lOol0
0`00Oomoo`0AOol00`00Oomoo`0COol00`00Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`08Ool0
00aoo`03001oogoo00eoo`03001oogoo00]oo`03001oogoo01=oo`03001oogoo015oo`03001oogoo
03]oo`03001oogoo03aoo`03001oogoo015oo`03001oogoo01=oo`03001oogoo00Yoo`03001oogoo
00ioo`03001oogoo00Qoo`0037oo00<007ooOol03Goo00<007ooOol02goo00<007ooOol04goo00<0
07ooOol04Goo00<007ooOol0>goo00<007ooOol0>goo00<007ooOol04Woo00<007ooOol04goo00<0
07ooOol02Woo00<007ooOol03Woo00<007ooOol027oo000<Ool00`00Oomoo`0=Ool00`00Oomoo`0;
Ool00`00Oomoo`0COol00`00Oomoo`0AOol00`00Oomoo`0kOol00`00Oomoo`0kOol00`00Oomoo`0B
Ool00`00Oomoo`0COol00`00Oomoo`0;Ool00`00Oomoo`0=Ool00`00Oomoo`08Ool000aoo`03001o
ogoo00eoo`03001oogoo00]oo`03001oogoo01=oo`03001oogoo019oo`03001oogoo03Yoo`03001o
ogoo03]oo`03001oogoo019oo`03001oogoo01=oo`03001oogoo00]oo`03001oogoo00eoo`03001o
ogoo00Qoo`0037oo00<007ooOol03Goo00<007ooOol02goo00<007ooOol04goo00<007ooOol04Woo
00<007ooOol0>Woo00<007ooOol0>goo00<007ooOol04Woo00<007ooOol04goo00<007ooOol02goo
00<007ooOol03Goo00<007ooOol027oo000<Ool00`00Oomoo`0=Ool00`00Oomoo`0;Ool00`00Oomo
o`0COol00`00Oomoo`0BOol00`00Oomoo`0jOol2000lOol00`00Oomoo`0BOol00`00Oomoo`0BOol0
0`00Oomoo`0<Ool00`00Oomoo`0=Ool00`00Oomoo`08Ool000aoo`03001oogoo00eoo`03001oogoo
00]oo`03001oogoo01=oo`03001oogoo019oo`03001oogoo03Yoo`03001oogoo03]oo`03001oogoo
019oo`03001oogoo019oo`03001oogoo00aoo`03001oogoo00eoo`03001oogoo00Qoo`003Goo00<0
07ooOol037oo00<007ooOol02goo00<007ooOol04goo00<007ooOol04Woo00<007ooOol0>Woo00<0
07ooOol0>goo00<007ooOol04goo00<007ooOol04Goo00<007ooOol037oo00<007ooOol03Goo00<0
07ooOol027oo000=Ool00`00Oomoo`030009Ool00`00Oomoo`0;Ool00`00Oomoo`0BOol00`00Oomo
o`0>Ool6000lOol00`00Oomoo`0kOol00`00Ool00005000>Ool00`00Oomoo`0AOol00`00Oomoo`0<
Ool00`00Oomoo`0<Ool30009Ool000eoo`05001oogooOol0000:Ool00`00Oomoo`0;Ool00`00Oomo
o`0BOol00`00Oomoo`0?Ool01@00Oomoogoo0000?7oo00<007ooOol0>Woo00D007ooOomoo`0000=o
o`03001oogoo00aoo`03001oogoo015oo`03001oogoo00aoo`03001oogoo00aoo`8000Yoo`003Goo
1P002Goo00<007ooOol02goo00<007ooOol04Woo00<007ooOol047oo00@007ooOol003aoo`03001o
ogoo03Yoo`03001oogoo009oo`03001oogoo00moo`03001oogoo015oo`03001oogoo00aoo`03001o
ogoo00Uoo`H000Uoo`0017oo1`000Woo00D007ooOomoo`0000Yoo`03001oogoo00]oo`03001oogoo
019oo`03001oogoo00Eoo`L000Eoo`04001oogoo000kOol00`00Oomoo`0jOol00`00Oomoo`03Ool0
0`00Oomoo`0>Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool00`00Oomoo`09Ool00`00Oomoo`02000:
Ool000eoo`80009oo`03001oogoo00Qoo`03001oogoo00]oo`03001oogoo019oo`03001oogoo019o
o`03001oo`0003]oo`03001oogoo03Yoo`03001oogoo00Aoo`03001oogoo00eoo`03001oogoo015o
o`03001oogoo00aoo`03001oogoo00Yoo`04001oo`00000:Ool000eoo`05001oo`00Ool0000:Ool0
0`00Oomoo`0<Ool00`00Oomoo`0AOol00`00Oomoo`0COol2000kOol2000kOol00`00Oomoo`05Ool0
0`00Oomoo`0<Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool00`00Oomoo`0;Ool3000:Ool000eoo`05
001oo`00Ool0000:Ool00`00Oomoo`0<Ool00`00Oomoo`0AOol00`00Oomoo`0COol2000kOol00`00
Oomoo`0jOol00`00Oomoo`05Ool00`00Oomoo`0<Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool00`00
Oomoo`0;Ool3000:Ool000eoo`03001oogoo008000Yoo`03001oogoo00aoo`03001oogoo015oo`03
001oogoo00ioo`03001oogoo009oo`8003]oo`03001oogoo03Yoo`04001oogoo0004Ool00`00Oomo
o`0<Ool00`00Oomoo`0AOol00`00Oomoo`0<Ool00`00Oomoo`0<Ool2000:Ool000eoo`05001oogoo
Ool0000:Ool00`00Oomoo`0<Ool00`00Oomoo`0AOol00`00Oomoo`0?Ool400000goo001oo`0jOol0
0`00Oomoo`0jOol01000Oomoogoo10003goo00<007ooOol04Goo00<007ooOol037oo00<007ooOol0
37oo0P002Woo000=Ool00`00Oomoo`0;Ool00`00Oomoo`0=Ool00`00Oomoo`0AOol00`00Oomoo`0D
Ool00`00Oomoo`0iOol00`00Oomoo`0iOol00`00Oomoo`0EOol00`00Oomoo`0AOol00`00Oomoo`0<
Ool00`00Oomoo`0<Ool00`00Oomoo`09Ool000eoo`03001oogoo00]oo`03001oogoo00eoo`03001o
ogoo015oo`03001oogoo01Aoo`03001oogoo03Uoo`03001oogoo03Uoo`03001oogoo01Eoo`03001o
ogoo015oo`03001oogoo00aoo`03001oogoo00aoo`03001oogoo00Uoo`003Goo00<007ooOol02goo
00<007ooOol03Goo00<007ooOol047oo00<007ooOol05Woo00<007ooOol0>7oo00<007ooOol0>Goo
00<007ooOol05Goo00<007ooOol04Goo00<007ooOol037oo00<007ooOol037oo00<007ooOol02Goo
000=Ool00`00Oomoo`0;Ool00`00Oomoo`0=Ool00`00Oomoo`0@Ool00`00Oomoo`0FOol00`00Oomo
o`0hOol00`00Oomoo`0iOol00`00Oomoo`0FOol00`00Oomoo`0?Ool00`00Oomoo`0=Ool00`00Oomo
o`0<Ool00`00Oomoo`09Ool000eoo`03001oogoo00]oo`03001oogoo00eoo`03001oogoo011oo`03
001oogoo01Ioo`03001oogoo03Qoo`03001oogoo03Uoo`03001oogoo01Ioo`03001oogoo00moo`03
001oogoo00eoo`03001oogoo00aoo`03001oogoo00Uoo`001Gooo`007@000goo000<Ool00`00Ool0
000<Ool00`00Ool0000=Ool01000Oomoo`003goo00<007ooOol03Woo00<007ooOol01Goo00<007oo
Ool01Woo00<007ooOol03Woo00<007ooOol03Woo00<007ooOol02Goo00D007ooOomoo`0000=oo`80
00]oo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00eoo`8000moo`03001oo`0000ioo`8000ioo`8000Yoo`003Woo00<007ooOol02Woo00<007ooOol0
3Goo00<007ooOol047oo00<007ooOol05Woo00<007ooOol0<goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol0<goo00<007ooOol05Woo00<007ooOol03goo00<007ooOol03Woo00<007ooOol0
2goo00<007ooOol02Goo000>Ool00`00Oomoo`0:Ool00`00Oomoo`0=Ool00`00Oomoo`0@Ool00`00
Oomoo`0FOol00`00Oomoo`0bOol00`00Oomoo`03Ool00`00Oomoo`04Ool00`00Oomoo`0aOol00`00
Oomoo`0GOol00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0;Ool00`00Oomoo`09Ool000io
o`03001oogoo00Yoo`03001oogoo00eoo`03001oogoo011oo`03001oogoo01Ioo`03001oogoo035o
o`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo035oo`03001oogoo01Moo`03001oogoo00mo
o`03001oogoo00ioo`03001oogoo00]oo`03001oogoo00Uoo`003Woo00<007ooOol02Woo00<007oo
Ool03Woo00<007ooOol03goo00<007ooOol05Woo00<007ooOol0<7oo00<007ooOol01Goo00<007oo
Ool01Goo00<007ooOol0<7oo00<007ooOol05goo00<007ooOol03goo00<007ooOol03Woo00<007oo
Ool02goo00<007ooOol02Goo000>Ool00`00Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`0?Ool0
0`00Oomoo`0GOol00`00Oomoo`0_Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`0_Ool0
0`00Oomoo`0GOol00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0;Ool00`00Oomoo`09Ool0
00ioo`03001oogoo00Yoo`03001oogoo00ioo`03001oogoo00moo`03001oogoo01Moo`03001oogoo
02ioo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo02ioo`03001oogoo01Moo`03001oogoo
00moo`03001oogoo00ioo`03001oogoo00]oo`03001oogoo00Uoo`003Woo00<007ooOol02Woo00<0
07ooOol03Woo00<007ooOol03Woo00<007ooOol067oo00<007ooOol0;Goo00<007ooOol01goo0P00
27oo00<007ooOol0;Woo00<007ooOol05goo00<007ooOol03goo00<007ooOol03Woo00<007ooOol0
2Woo00<007ooOol02Woo000>Ool00`00Oomoo`0:Ool00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00
Oomoo`0HOol00`00Oomoo`0]Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`0]Ool00`00
Oomoo`0GOol00`00Oomoo`0?Ool00`00Oomoo`0>Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool000io
o`03001oogoo00Yoo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo01Qoo`03001oogoo02ao
o`03001oogoo00Qoo`03001oogoo00Uoo`03001oogoo02aoo`03001oogoo01Moo`03001oogoo00mo
o`03001oogoo00ioo`03001oogoo00Yoo`03001oogoo00Yoo`003Woo00<007ooOol02Woo00<007oo
Ool03Woo00<007ooOol03Woo00<007ooOol067oo00<007ooOol0:goo00<007ooOol02Goo00<007oo
Ool02Goo00<007ooOol0:goo00<007ooOol06Goo00<007ooOol03Woo00<007ooOol03Woo00<007oo
Ool02Woo00<007ooOol02Woo000>Ool00`00Oomoo`09Ool00`00Oomoo`0?Ool00`00Oomoo`0>Ool0
0`00Oomoo`0HOol00`00Oomoo`0[Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00Oomoo`0ZOol0
0`00Oomoo`0IOol00`00Oomoo`0=Ool00`00Oomoo`0?Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool0
00ioo`03001oogoo00Uoo`03001oogoo00moo`03001oogoo00ioo`03001oogoo01Qoo`03001oogoo
02Yoo`03001oogoo00Yoo`03001oogoo00Yoo`03001oogoo02Yoo`03001oogoo01Uoo`03001oogoo
00eoo`03001oogoo00moo`03001oogoo00Yoo`03001oogoo00Yoo`003Woo00<007ooOol02Goo00<0
07ooOol03goo00<007ooOol03Woo00<007ooOol067oo00<007ooOol0:Woo00<007ooOol02Woo00<0
07ooOol02goo00<007ooOol0:Goo00<007ooOol06Goo00<007ooOol03Goo00<007ooOol03goo00<0
07ooOol02Woo00<007ooOol02Woo000>Ool00`00Oomoo`09Ool00`00Oomoo`0?Ool00`00Oomoo`0>
Ool00`00Oomoo`0IOol00`00Oomoo`0XOol00`00Oomoo`0;Ool00`00Oomoo`0;Ool00`00Oomoo`0Y
Ool00`00Oomoo`0IOol00`00Oomoo`0=Ool00`00Oomoo`0?Ool00`00Oomoo`0:Ool00`00Oomoo`0:
Ool000ioo`03001oogoo00Uoo`03001oogoo00moo`03001oogoo00ioo`03001oogoo01Uoo`03001o
ogoo02Qoo`03001oogoo00]oo`8000eoo`03001oogoo02Qoo`03001oogoo01Uoo`03001oogoo00eo
o`03001oogoo00moo`03001oogoo00Yoo`03001oogoo00Yoo`003Woo00<007ooOol02Goo00<007oo
Ool03goo00<007ooOol03Woo00<007ooOol06Goo00<007ooOol09goo00<007ooOol037oo00<007oo
Ool037oo00<007ooOol0:7oo00<007ooOol06Goo00<007ooOol03Goo00<007ooOol03goo00<007oo
Ool02Woo00<007ooOol02Woo000?Ool00`00Oomoo`08Ool00`00Oomoo`0@Ool00`00Oomoo`0=Ool0
0`00Oomoo`0IOol00`00Oomoo`0WOol00`00Oomoo`0<Ool00`00Oomoo`0=Ool00`00Oomoo`0WOol0
0`00Oomoo`0IOol00`00Oomoo`0=Ool00`00Oomoo`0?Ool00`00Oomoo`0:Ool00`00Oomoo`0:Ool0
00moo`03001oogoo00Qoo`03001oogoo011oo`03001oogoo00aoo`03001oogoo01Yoo`03001oogoo
02Ioo`03001oogoo00eoo`03001oogoo00eoo`03001oogoo02Ioo`03001oogoo01Yoo`03001oogoo
00eoo`03001oogoo011oo`03001oogoo00Uoo`03001oogoo00Yoo`003goo00<007ooOol027oo00<0
07ooOol047oo00<007ooOol037oo00<007ooOol06Woo00<007ooOol09Woo00<007ooOol03Goo00<0
07ooOol03Woo00<007ooOol09Goo00<007ooOol06Woo00<007ooOol03Goo00<007ooOol047oo00<0
07ooOol02Goo00<007ooOol02Woo000?Ool00`00Oomoo`08Ool00`00Oomoo`0@Ool00`00Oomoo`0<
Ool00`00Oomoo`0JOol00`00Oomoo`0UOol00`00Oomoo`0>Ool00`00Oomoo`0>Ool00`00Oomoo`0U
Ool00`00Oomoo`0JOol00`00Oomoo`0=Ool00`00Oomoo`0@Ool00`00Oomoo`09Ool00`00Oomoo`0:
Ool000moo`03001oogoo00Qoo`03001oogoo011oo`03001oogoo00aoo`03001oogoo01Yoo`03001o
ogoo02Eoo`03001oogoo00ioo`03001oogoo00moo`03001oogoo02Aoo`03001oogoo01Yoo`03001o
ogoo00eoo`03001oogoo011oo`03001oogoo00Uoo`03001oogoo00Yoo`003goo00<007ooOol027oo
00<007ooOol047oo00<007ooOol037oo00<007ooOol06Woo00<007ooOol09Goo00<007ooOol03Woo
00<007ooOol03goo00<007ooOol097oo00<007ooOol06goo00<007ooOol037oo00<007ooOol047oo
00<007ooOol02Goo00<007ooOol02Woo000?Ool00`00Oomoo`08Ool00`00Oomoo`0@Ool00`00Oomo
o`0<Ool00`00Oomoo`0KOol00`00Oomoo`0SOol00`00Oomoo`0?Ool00`00Oomoo`0@Ool00`00Oomo
o`0SOol00`00Oomoo`0KOol00`00Oomoo`0<Ool00`00Oomoo`0@Ool00`00Oomoo`09Ool00`00Oomo
o`0:Ool000moo`03001oogoo00Qoo`03001oogoo011oo`03001oogoo00aoo`03001oogoo01]oo`03
001oogoo02=oo`03001oogoo00moo`80015oo`03001oogoo02=oo`03001oogoo01]oo`03001oogoo
00]oo`03001oogoo015oo`03001oogoo00Uoo`03001oogoo00Yoo`003goo00<007ooOol027oo00<0
07ooOol047oo00<007ooOol037oo00<007ooOol06goo00<007ooOol08Woo00<007ooOol047oo00<0
07ooOol04Goo00<007ooOol08Woo00<007ooOol06goo00<007ooOol02goo00<007ooOol04Goo00<0
07ooOol02Goo00<007ooOol02Woo000?Ool00`00Oomoo`08Ool00`00Oomoo`0@Ool00`00Oomoo`0<
Ool00`00Oomoo`0KOol00`00Oomoo`0ROol00`00Oomoo`0@Ool00`00Oomoo`0AOol00`00Oomoo`0Q
Ool00`00Oomoo`0LOol00`00Oomoo`0;Ool00`00Oomoo`0AOol00`00Oomoo`09Ool00`00Oomoo`0:
Ool000moo`03001oogoo00Qoo`03001oogoo011oo`03001oogoo00aoo`03001oogoo01]oo`03001o
ogoo029oo`03001oogoo011oo`03001oogoo015oo`03001oogoo025oo`03001oogoo01aoo`03001o
ogoo00]oo`03001oogoo015oo`03001oogoo00Qoo`03001oogoo00]oo`003goo00<007ooOol01goo
00<007ooOol04Goo00<007ooOol037oo00<007ooOol06goo00<007ooOol08Goo00<007ooOol04Goo
00<007ooOol04Woo00<007ooOol087oo00<007ooOol077oo00<007ooOol02goo00<007ooOol04Goo
00<007ooOol027oo00<007ooOol02goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0AOol00`00Oomo
o`0;Ool00`00Oomoo`0LOol00`00Oomoo`0QOol00`00Oomoo`0AOol00`00Oomoo`0BOol00`00Oomo
o`0POol00`00Oomoo`0LOol00`00Oomoo`0;Ool00`00Oomoo`0BOol00`00Oomoo`07Ool00`00Oomo
o`0;Ool000moo`03001oogoo00Moo`03001oogoo019oo`03001oogoo00Yoo`03001oogoo01aoo`03
001oogoo025oo`03001oogoo015oo`03001oogoo019oo`03001oogoo021oo`03001oogoo01aoo`03
001oogoo00]oo`03001oogoo019oo`03001oogoo00Moo`03001oogoo00]oo`003goo00<007ooOol0
1goo00<007ooOol04Woo00<007ooOol02Woo00<007ooOol077oo00<007ooOol087oo00<007ooOol0
4Woo00<007ooOol04goo00<007ooOol07goo00<007ooOol077oo00<007ooOol02goo00<007ooOol0
4Woo00<007ooOol01goo00<007ooOol02goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0BOol00`00
Oomoo`0:Ool00`00Oomoo`0MOol00`00Oomoo`0OOol00`00Oomoo`0BOol2000DOol00`00Oomoo`0O
Ool00`00Oomoo`0MOol00`00Oomoo`0:Ool00`00Oomoo`0BOol00`00Oomoo`07Ool00`00Oomoo`0;
Ool000moo`03001oogoo00Moo`03001oogoo019oo`03001oogoo00Yoo`03001oogoo01eoo`03001o
ogoo01moo`03001oogoo019oo`03001oogoo01=oo`03001oogoo01ioo`03001oogoo01ioo`03001o
ogoo00Yoo`03001oogoo019oo`03001oogoo00Moo`03001oogoo00]oo`003goo00<007ooOol01goo
00<007ooOol04Woo00<007ooOol02Woo00<007ooOol07Goo00<007ooOol07Woo00<007ooOol04goo
00<007ooOol057oo00<007ooOol07Goo00<007ooOol07Woo00<007ooOol02Woo00<007ooOol04Woo
00<007ooOol01goo00<007ooOol02goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0BOol00`00Oomo
o`0:Ool00`00Oomoo`0MOol00`00Oomoo`0NOol00`00Oomoo`0COol00`00Oomoo`0DOol00`00Oomo
o`0MOol00`00Oomoo`0NOol00`00Oomoo`0:Ool00`00Oomoo`0BOol00`00Oomoo`07Ool00`00Oomo
o`0;Ool000moo`03001oogoo00Moo`03001oogoo019oo`03001oogoo00Yoo`03001oogoo01eoo`03
001oogoo01ioo`03001oogoo01=oo`03001oogoo01Aoo`03001oogoo01eoo`03001oogoo01ioo`03
001oogoo00Yoo`03001oogoo019oo`03001oogoo00Moo`03001oogoo00]oo`0047oo00<007ooOol0
1Woo00<007ooOol04Woo00<007ooOol02Woo00<007ooOol07Goo00<007ooOol07Goo00<007ooOol0
57oo00<007ooOol05Goo00<007ooOol077oo00<007ooOol07Woo00<007ooOol02Woo00<007ooOol0
4Woo00<007ooOol01goo00<007ooOol02goo000@Ool00`00Oomoo`06Ool00`00Oomoo`0BOol00`00
Oomoo`0:Ool00`00Oomoo`0MOol00`00Oomoo`0IOol20002Ool00`00Oomoo`02Ool30005Ool40006
Ool00`00Oomoo`0EOol00`00Oomoo`0LOol00`00Oomoo`0NOol00`00Oomoo`09Ool00`00Oomoo`0C
Ool00`00Oomoo`07Ool00`00Oomoo`0;Ool0011oo`03001oogoo00Ioo`03001oogoo019oo`03001o
ogoo00Yoo`03001oogoo01ioo`03001oogoo01Moo`06001oogoo001oo`0017oo0`0017oo00<007oo
Ool00Woo00<007ooOol00goo00<007ooOol05Goo00<007ooOol077oo00<007ooOol07Woo00<007oo
Ool02Goo00<007ooOol04goo00<007ooOol01goo00<007ooOol02goo000@Ool00`00Oomoo`06Ool0
0`00Oomoo`0BOol00`00Oomoo`09Ool00`00Oomoo`0OOol00`00Oomoo`0FOol00`00Oomoo`02Ool0
0`00Oomoo`03Ool30009Ool00`00Oomoo`03Ool3000FOol00`00Oomoo`0KOol00`00Oomoo`0NOol0
0`00Oomoo`09Ool00`00Oomoo`0COol00`00Oomoo`07Ool00`00Oomoo`0;Ool0011oo`03001oogoo
00Ioo`03001oogoo019oo`03001oogoo00Uoo`03001oogoo01moo`03001oogoo01Ioo`03001oogoo
009oo`03001oogoo00moo`03001oogoo00=oo`03001oogoo01Ioo`03001oogoo01Yoo`03001oogoo
01moo`03001oogoo00Uoo`03001oogoo01=oo`03001oogoo00Moo`03001oogoo00]oo`0047oo00<0
07ooOol01Woo00<007ooOol04Woo00<007ooOol02Goo00<007ooOol07goo00<007ooOol05Woo00<0
07ooOol00Woo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol05Woo00<0
07ooOol06Woo00<007ooOol07goo00<007ooOol02Goo00<007ooOol04goo00<007ooOol01goo00<0
07ooOol02goo000@Ool00`00Oomoo`05Ool00`00Oomoo`0COol00`00Oomoo`09Ool00`00Oomoo`0O
Ool00`00Oomoo`0FOol01000Oomoogoo0P0037oo1@001Woo00<007ooOol05goo00<007ooOol06Goo
00<007ooOol087oo00<007ooOol027oo00<007ooOol04goo00<007ooOol01goo00<007ooOol02goo
000@Ool00`00Oomoo`05Ool00`00Oomoo`0COol00`00Oomoo`09Ool00`00Oomoo`0POol00`00Oomo
o`0EOol01000Oomoogoo0P0037oo00<007ooOol027oo00<007ooOol05goo00<007ooOol06Goo00<0
07ooOol087oo00<007ooOol027oo00<007ooOol04goo00<007ooOol01Woo00<007ooOol037oo000@
Ool00`00Oomoo`05Ool00`00Oomoo`0DOol00`00Oomoo`08Ool00`00Oomoo`0POol00`00Oomoo`0F
Ool01000Oomoo`003Goo00<007ooOol027oo00<007ooOol05goo00<007ooOol06Goo00<007ooOol0
87oo00<007ooOol027oo00<007ooOol057oo00<007ooOol01Goo00<007ooOol037oo000@Ool00`00
Oomoo`05Ool00`00Oomoo`0DOol00`00Oomoo`08Ool00`00Oomoo`0POol00`00Oomoo`0GOol2000>
Ool50006Ool00`00Oomoo`0HOol00`00Oomoo`0HOol00`00Oomoo`0POol00`00Oomoo`08Ool00`00
Oomoo`0DOol00`00Oomoo`05Ool00`00Oomoo`0<Ool0011oo`03001oogoo00Eoo`03001oogoo01Ao
o`03001oogoo00Qoo`03001oogoo021oo`03001oogoo01Qoo`03001oogoo01Moo`03001oogoo01Qo
o`03001oogoo01Qoo`03001oogoo021oo`03001oogoo00Qoo`03001oogoo01Aoo`03001oogoo00Eo
o`03001oogoo00aoo`0047oo00<007ooOol01Goo00<007ooOol057oo00<007ooOol027oo00<007oo
Ool087oo00<007ooOol067oo00<007ooOol05goo00<007ooOol067oo00<007ooOol05goo00<007oo
Ool08Goo00<007ooOol027oo00<007ooOol057oo00<007ooOol01Goo00<007ooOol037oo000@Ool0
0`00Oomoo`05Ool00`00Oomoo`0DOol00`00Oomoo`08Ool00`00Oomoo`0POol00`00Oomoo`0GOol0
0`00Oomoo`0HOol2000JOol00`00Oomoo`0FOol00`00Oomoo`0QOol00`00Oomoo`08Ool00`00Oomo
o`0DOol00`00Oomoo`05Ool00`00Oomoo`0<Ool0011oo`03001oogoo00Eoo`03001oogoo01Aoo`03
001oogoo00Qoo`03001oogoo025oo`03001oogoo01Ioo`03001oogoo01Qoo`03001oogoo01Uoo`03
001oogoo01Ioo`03001oogoo025oo`03001oogoo00Qoo`03001oogoo01Aoo`03001oogoo00Eoo`03
001oogoo00aoo`0047oo00<007ooOol01Goo00<007ooOol057oo00<007ooOol01goo00<007ooOol0
8Woo00<007ooOol05Woo00<007ooOol067oo00<007ooOol06Goo00<007ooOol05Woo00<007ooOol0
8Goo00<007ooOol027oo00<007ooOol057oo00<007ooOol01Goo00<007ooOol037oo000@Ool00`00
Oomoo`05Ool00`00Oomoo`0DOol00`00Oomoo`07Ool00`00Oomoo`0ROol00`00Oomoo`0FOol00`00
Oomoo`0HOol00`00Oomoo`0JOol00`00Oomoo`0EOol00`00Oomoo`0QOol00`00Oomoo`08Ool00`00
Oomoo`0DOol00`00Oomoo`05Ool00`00Oomoo`0<Ool0011oo`03001oogoo00Eoo`03001oogoo01Ao
o`03001oogoo00Moo`03001oogoo029oo`03001oogoo01Eoo`03001oogoo01Uoo`03001oogoo01Yo
o`03001oogoo01Aoo`03001oogoo029oo`03001oogoo00Moo`03001oogoo01Eoo`03001oogoo00Eo
o`03001oogoo00aoo`0047oo00<007ooOol01Goo00<007ooOol057oo00<007ooOol01goo00<007oo
Ool08Woo00<007ooOol05Goo00<007ooOol06Goo00<007ooOol06Woo00<007ooOol057oo00<007oo
Ool08Woo00<007ooOol01goo00<007ooOol05Goo00<007ooOol01Goo00<007ooOol037oo000@Ool0
0`00Oomoo`05Ool00`00Oomoo`0DOol00`00Oomoo`07Ool00`00Oomoo`0ROol00`00Oomoo`0EOol0
0`00Oomoo`0IOol00`00Oomoo`0KOol00`00Oomoo`0COol00`00Oomoo`0SOol00`00Oomoo`06Ool0
0`00Oomoo`0EOol00`00Oomoo`05Ool00`00Oomoo`0<Ool0011oo`03001oogoo00Eoo`03001oogoo
01Aoo`03001oogoo00Moo`03001oogoo02=oo`03001oogoo01=oo`03001oogoo01Yoo`03001oogoo
01]oo`03001oogoo01=oo`03001oogoo02=oo`03001oogoo00Ioo`03001oogoo01Eoo`03001oogoo
00Eoo`03001oogoo00aoo`0047oo00<007ooOol01Goo00<007ooOol05Goo00<007ooOol01Woo00<0
07ooOol08goo00<007ooOol04goo00<007ooOol06Woo0P0077oo00<007ooOol04goo00<007ooOol0
8goo00<007ooOol01Woo00<007ooOol05Goo00<007ooOol017oo00<007ooOol03Goo000AOol00`00
Oomoo`04Ool00`00Oomoo`0EOol00`00Oomoo`06Ool00`00Oomoo`0SOol00`00Oomoo`0COol00`00
Oomoo`0JOol00`00Oomoo`0LOol00`00Oomoo`0AOol00`00Oomoo`0TOol00`00Oomoo`06Ool00`00
Oomoo`0EOol00`00Oomoo`04Ool00`00Oomoo`0=Ool0015oo`03001oogoo00=oo`03001oogoo01Io
o`03001oogoo00Ioo`03001oogoo02=oo`03001oogoo019oo`03001oogoo01]oo`03001oogoo01ao
o`03001oogoo015oo`03001oogoo02Aoo`03001oogoo00Ioo`03001oogoo01Eoo`03001oogoo00Ao
o`03001oogoo00eoo`004Goo00<007ooOol00goo00<007ooOol05Woo00<007ooOol01Woo00<007oo
Ool08goo00<007ooOol04Woo00<007ooOol06goo00<007ooOol077oo00<007ooOol04Goo00<007oo
Ool097oo00<007ooOol01Woo00<007ooOol05Goo00<007ooOol017oo00<007ooOol03Goo000AOol0
0`00Oomoo`03Ool00`00Oomoo`0FOol00`00Oomoo`06Ool00`00Oomoo`0SOol00`00Oomoo`0BOol0
0`00Oomoo`0KOol00`00Oomoo`0MOol00`00Oomoo`0@Ool00`00Oomoo`0TOol00`00Oomoo`06Ool0
0`00Oomoo`0FOol00`00Oomoo`03Ool00`00Oomoo`0=Ool0015oo`03001oogoo00=oo`03001oogoo
01Ioo`03001oogoo00Eoo`03001oogoo02Aoo`03001oogoo015oo`03001oogoo01aoo`03001oogoo
01eoo`03001oogoo011oo`03001oogoo02Aoo`03001oogoo00Ioo`03001oogoo01Ioo`03001oogoo
00=oo`03001oogoo00eoo`004Goo00<007ooOol00goo00<007ooOol05Woo00<007ooOol01Goo00<0
07ooOol09Goo00<007ooOol047oo00<007ooOol077oo00<007ooOol07Goo00<007ooOol047oo00<0
07ooOol097oo00<007ooOol01Woo00<007ooOol05Woo00<007ooOol00goo00<007ooOol03Goo000A
Ool00`00Oomoo`03Ool00`00Oomoo`0GOol00`00Oomoo`04Ool00`00Oomoo`0UOol00`00Oomoo`0@
Ool00`00Oomoo`0LOol00`00Oomoo`0NOol00`00Oomoo`0>Ool00`00Oomoo`0UOol00`00Oomoo`06
Ool00`00Oomoo`0FOol00`00Oomoo`03Ool00`00Oomoo`0=Ool0015oo`03001oogoo00=oo`03001o
ogoo01Moo`03001oogoo00Aoo`03001oogoo02Eoo`03001oogoo00moo`03001oogoo01eoo`8001mo
o`03001oogoo00ioo`03001oogoo02Ioo`03001oogoo00Aoo`03001oogoo01Moo`03001oogoo009o
o`03001oogoo00ioo`004Goo00<007ooOol00goo00<007ooOol05goo00<007ooOol017oo00<007oo
Ool09Goo00<007ooOol03goo00<007ooOol07Goo00<007ooOol07Woo00<007ooOol03Woo00<007oo
Ool09Woo00<007ooOol017oo00<007ooOol05goo00<007ooOol00Woo00<007ooOol03Woo000AOol0
0`00Oomoo`03Ool00`00Oomoo`0GOol00`00Oomoo`04Ool00`00Oomoo`0VOol00`00Oomoo`0>Ool0
0`00Oomoo`0MOol00`00Oomoo`0NOol00`00Oomoo`0>Ool00`00Oomoo`0VOol00`00Oomoo`04Ool0
0`00Oomoo`0GOol00`00Oomoo`02Ool00`00Oomoo`0>Ool0015oo`03001oogoo00=oo`03001oogoo
01Moo`03001oogoo00Aoo`03001oogoo02Ioo`03001oogoo00eoo`03001oogoo01ioo`03001oogoo
01moo`03001oogoo00eoo`03001oogoo02Ioo`03001oogoo00Aoo`03001oogoo01Moo`03001oogoo
009oo`03001oogoo00ioo`004Goo00<007ooOol00goo00<007ooOol05goo00<007ooOol017oo00<0
07ooOol09Woo00<007ooOol03Goo00<007ooOol07Woo00<007ooOol07goo00<007ooOol037oo00<0
07ooOol09goo00<007ooOol017oo00<007ooOol05goo00<007ooOol00Woo00<007ooOol03Woo000B
Ool01@00Oomoogoo00006Woo00<007ooOol017oo00<007ooOol09goo00<007ooOol037oo00<007oo
Ool07Woo00<007ooOol07goo00<007ooOol037oo00<007ooOol09goo00<007ooOol017oo00<007oo
Ool067oo00D007ooOomoo`00011oo`004Woo00D007ooOomoo`0001Yoo`03001oogoo00=oo`03001o
ogoo02Qoo`03001oogoo00]oo`03001oogoo01moo`03001oogoo021oo`03001oogoo00]oo`03001o
ogoo02Qoo`03001oogoo009oo`03001oogoo01Uoo`05001oogooOol0000@Ool0019oo`05001oogoo
Ool0000JOol00`00Oomoo`03Ool00`00Oomoo`0XOol00`00Oomoo`0;Ool00`00Oomoo`0OOol00`00
Oomoo`0POol00`00Oomoo`0;Ool00`00Oomoo`0XOol00`00Oomoo`02Ool00`00Oomoo`0IOol01@00
Oomoogoo000047oo000BOol01@00Oomoogoo00006Woo00<007ooOol00goo00<007ooOol0:7oo00<0
07ooOol02goo00<007ooOol07goo0P008Goo00<007ooOol02Woo00<007ooOol0:Goo00<007ooOol0
0Woo00<007ooOol06Goo00D007ooOomoo`00011oo`004Woo00D007ooOomoo`0001Yoo`03001oogoo
00=oo`03001oogoo02Uoo`03001oogoo00Uoo`03001oogoo021oo`03001oogoo025oo`03001oogoo
00Uoo`03001oogoo02Uoo`03001oogoo009oo`03001oogoo01Uoo`05001oogooOol0000@Ool0019o
o`05001oogooOol0000KOol01@00Oomoogoo0000;7oo00<007ooOol02Goo00<007ooOol087oo00<0
07ooOol08Goo00<007ooOol02Goo00<007ooOol0:Goo00<007ooOol00Woo00<007ooOol06Goo00D0
07ooOomoo`00011oo`004Woo00D007ooOomoo`0001]oo`05001oogooOol0000/Ool00`00Oomoo`08
Ool00`00Oomoo`0QOol00`00Oomoo`0ROol00`00Oomoo`07Ool00`00Oomoo`0ZOol00`00Oomoo`02
Ool00`00Oomoo`0IOol01000Oomoo`004Goo000BOol01@00Oomoogoo00006goo00D007ooOomoo`00
02eoo`03001oogoo00Ioo`03001oogoo029oo`03001oogoo02=oo`03001oogoo00Ioo`03001oogoo
02Yoo`03001oogoo009oo`03001oogoo01Uoo`04001oogoo000AOol001=oo`04001oogoo000KOol0
1@00Oomoogoo0000;Woo00<007ooOol01Goo00<007ooOol08Woo00<007ooOol08goo00<007ooOol0
1Goo00<007ooOol0;7oo00@007ooOol001aoo`04001oogoo000AOol001=oo`03001oo`0001eoo`04
001oogoo000^Ool00`00Oomoo`04Ool00`00Oomoo`0IOol50005Ool00`00Oomoo`0TOol00`00Oomo
o`04Ool00`00Oomoo`0/Ool01000Oomoo`007Goo00<007oo00004Goo000COol00`00Ool0000MOol0
0`00Ool0000_Ool20004Ool00`00Oomoo`0LOol00`00Oomoo`05Ool00`00Oomoo`0UOol00`00Oomo
o`02Ool00`00Oomoo`0]Ool01000Oomoo`007Goo00<007oo00004Goo000COol00`00Ool0000MOol3
000`Ool20002Ool00`00Oomoo`0MOol00`00Oomoo`05Ool00`00Oomoo`0UOol20002Ool2000`Ool2
000NOol2000BOol001Aoo`03001oogoo01eoo`80035oo`<0021oo`03001oogoo00Eoo`<002Moo`80
039oo`8001moo`03001oogoo011oo`00RWoo00<007ooOol01Goo00<007ooOol0Sgoo002:Ool00`00
Oomoo`05Ool00`00Oomoo`2?Ool008Yoo`03001oogoo00Eoo`03001oogoo08moo`00R7oo0`001goo
00<007ooOol0Sgoo0029Ool2002IOol00?moobEoo`00\
\>"],
  ImageRangeCache->{{{0, 291.5}, {179.625, 0}} -> {-4.37187, -1.08129, 
  0.0446443, 0.0180589}}]
}, Open  ]],

Cell[TextData[{
  "Now let's plot a family of four functions on the same graph, using colour \
and other attributes and the ",
  StyleBox["Legend",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " Package to label things properly. For further details of how to manage \
legends, see Shaw and Tigg (1993), but note that when used with a legend, the \
thickness and dashing attributes should be used in absolute form. Cloning of \
the following example is encouraged: "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Needs[\"Graphics`Legend`\"];
Plot[{Sin[x], Sin[x^2], Cos[x], Cos[x^2]},{x, -4, 4},
PlotStyle -> {{RGBColor[1, 0, 0]}, {GrayLevel[0.2]},
{RGBColor[0,0,1],AbsoluteThickness[2]},
{AbsoluteDashing[{2, 3}]}}, 
PlotLegend -> {Sin[x], Sin[x^2], Cos[x], Cos[x^2]},
LegendPosition -> {1.2, -0.5},
PlotLabel -> FontForm[\"Coloured Curves\", {\"Helvetica\", 12}]
];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .44061 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.363294 0.339485 0.220304 0.339485 [
[ 0 0 0 0 ]
[ 1 .44061 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
% Start of sub-graphic
p
0.0238095 0.0104907 0.702779 0.430117 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.119048 0.309017 0.294302 [
[.02381 .29652 -6 -9 ]
[.02381 .29652 6 0 ]
[.2619 .29652 -6 -9 ]
[.2619 .29652 6 0 ]
[.7381 .29652 -3 -9 ]
[.7381 .29652 3 0 ]
[.97619 .29652 -3 -9 ]
[.97619 .29652 3 0 ]
[.4875 .01471 -12 -4.5 ]
[.4875 .01471 0 4.5 ]
[.4875 .16187 -24 -4.5 ]
[.4875 .16187 0 4.5 ]
[.4875 .45617 -18 -4.5 ]
[.4875 .45617 0 4.5 ]
[.4875 .60332 -6 -4.5 ]
[.4875 .60332 0 4.5 ]
[.5 .63053 -46.7188 0 ]
[.5 .63053 46.7188 14 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .30902 m
.02381 .31527 L
s
[(-4)] .02381 .29652 0 1 Mshowa
.2619 .30902 m
.2619 .31527 L
s
[(-2)] .2619 .29652 0 1 Mshowa
.7381 .30902 m
.7381 .31527 L
s
[(2)] .7381 .29652 0 1 Mshowa
.97619 .30902 m
.97619 .31527 L
s
[(4)] .97619 .29652 0 1 Mshowa
.125 Mabswid
.08333 .30902 m
.08333 .31277 L
s
.14286 .30902 m
.14286 .31277 L
s
.20238 .30902 m
.20238 .31277 L
s
.32143 .30902 m
.32143 .31277 L
s
.38095 .30902 m
.38095 .31277 L
s
.44048 .30902 m
.44048 .31277 L
s
.55952 .30902 m
.55952 .31277 L
s
.61905 .30902 m
.61905 .31277 L
s
.67857 .30902 m
.67857 .31277 L
s
.79762 .30902 m
.79762 .31277 L
s
.85714 .30902 m
.85714 .31277 L
s
.91667 .30902 m
.91667 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .01471 m
.50625 .01471 L
s
[(-1)] .4875 .01471 1 0 Mshowa
.5 .16187 m
.50625 .16187 L
s
[(-0.5)] .4875 .16187 1 0 Mshowa
.5 .45617 m
.50625 .45617 L
s
[(0.5)] .4875 .45617 1 0 Mshowa
.5 .60332 m
.50625 .60332 L
s
[(1)] .4875 .60332 1 0 Mshowa
.125 Mabswid
.5 .04414 m
.50375 .04414 L
s
.5 .07358 m
.50375 .07358 L
s
.5 .10301 m
.50375 .10301 L
s
.5 .13244 m
.50375 .13244 L
s
.5 .1913 m
.50375 .1913 L
s
.5 .22073 m
.50375 .22073 L
s
.5 .25016 m
.50375 .25016 L
s
.5 .27959 m
.50375 .27959 L
s
.5 .33845 m
.50375 .33845 L
s
.5 .36788 m
.50375 .36788 L
s
.5 .39731 m
.50375 .39731 L
s
.5 .42674 m
.50375 .42674 L
s
.5 .4856 m
.50375 .4856 L
s
.5 .51503 m
.50375 .51503 L
s
.5 .54446 m
.50375 .54446 L
s
.5 .57389 m
.50375 .57389 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -107.719 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 18.000000 moveto
 541.000000 18.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 18.000000 lineto
closepath
clip newpath
63.000000 14.000000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 12.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 14.000000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 12.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Coloured) show
114.625000 14.000000 moveto
(Curves) show
152.437500 14.000000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 12.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Helvetica
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Helvetica
%%DocumentSuppliedFonts:
%%DocumentFonts: Helvetica
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
.02381 .53175 m
.06244 .45878 L
.10458 .36169 L
.14415 .26432 L
.18221 .17517 L
.20342 .13084 L
.22272 .09537 L
.24299 .06415 L
.26171 .04161 L
.271 .03284 L
.28086 .02538 L
.29075 .01984 L
.29637 .01758 L
.2991 .01672 L
.30164 .01605 L
.30404 .01555 L
.3066 .01514 L
.30807 .01497 L
.30878 .0149 L
.30944 .01485 L
.31068 .01477 L
.31133 .01474 L
.31202 .01472 L
.31319 .01472 L
.31445 .01474 L
.31564 .01479 L
.31672 .01486 L
.31799 .01497 L
.31937 .01514 L
.32189 .01553 L
.3241 .01599 L
.32655 .01662 L
.33151 .01826 L
.3361 .02024 L
.34041 .02248 L
.35024 .029 L
.36059 .03792 L
.3791 .05892 L
.39884 .08796 L
.41721 .12045 L
.45625 .20328 L
.49775 .30346 L
.53773 .40074 L
.5762 .48479 L
.59568 .5209 L
.61712 .55405 L
.62693 .56664 L
.63747 .57822 L
.64736 .58715 L
.65652 .59373 L
Mistroke
.66184 .59677 L
.66665 .59903 L
.672 .60099 L
.67493 .60181 L
.67769 .60242 L
.68015 .60283 L
.68138 .60299 L
.68273 .60313 L
.68389 .60322 L
.68513 .60328 L
.68629 .60331 L
.68736 .60332 L
.68801 .60331 L
.68869 .60329 L
.68993 .60323 L
.69061 .60318 L
.69126 .60313 L
.69272 .60298 L
.69517 .60263 L
.6978 .60211 L
.70277 .60074 L
.70739 .59901 L
.71788 .59348 L
.72809 .58596 L
.73899 .5757 L
.76038 .54916 L
.78054 .51705 L
.81906 .44007 L
.86003 .34348 L
.89948 .2465 L
.93742 .15954 L
.97619 .08629 L
Mfstroke
.2 g
.02381 .22429 m
.03279 .39753 L
.03793 .48549 L
.04262 .54787 L
.04517 .57275 L
.04634 .5818 L
.04758 .58969 L
.0489 .59615 L
.05012 .60029 L
.05143 .60278 L
.05218 .60329 L
.05288 .60318 L
.05412 .60157 L
.05529 .59843 L
.05634 .59427 L
.05749 .58834 L
.06006 .57001 L
.06244 .54714 L
.07382 .38159 L
.08454 .19783 L
.08922 .12851 L
.09417 .07046 L
.09675 .04771 L
.09812 .03797 L
.09959 .02939 L
.10094 .02322 L
.10217 .01905 L
.10341 .01626 L
.10458 .01492 L
.10574 .01483 L
.10682 .01585 L
.10802 .01819 L
.1093 .02212 L
.11057 .02743 L
.11192 .03448 L
.11431 .05068 L
.11862 .09018 L
.12332 .14608 L
.134 .30108 L
.14371 .44169 L
.14855 .50072 L
.1511 .52725 L
.1538 .5514 L
.15616 .56894 L
.15875 .58417 L
.15989 .58946 L
.16111 .59421 L
.16218 .59754 L
.16332 .60026 L
.16405 .60156 L
Mistroke
.16471 .60242 L
.16601 .60328 L
.16673 .60328 L
.16752 .60289 L
.16826 .60217 L
.16894 .60119 L
.17017 .59866 L
.17153 .59481 L
.17295 .58959 L
.17427 .58367 L
.1791 .55391 L
.18427 .50975 L
.20412 .27788 L
.21458 .15893 L
.22031 .1055 L
.22568 .06566 L
.23043 .03969 L
.2331 .02909 L
.23555 .02195 L
.23686 .01913 L
.23808 .01713 L
.23878 .01625 L
.23942 .01563 L
.24015 .01511 L
.24085 .01482 L
.24152 .01472 L
.24224 .0148 L
.24352 .01546 L
.24424 .01609 L
.2449 .01686 L
.24642 .01922 L
.24761 .02166 L
.2489 .02488 L
.25125 .03221 L
.25566 .05074 L
.26081 .07937 L
.26558 .11148 L
.28581 .28094 L
.30482 .43795 L
.31365 .49665 L
.32296 .54493 L
.32783 .56431 L
.33317 .58087 L
.33551 .58664 L
.33802 .59181 L
.34017 .59545 L
.34253 .59863 L
.34378 .59996 L
.34512 .60114 L
.34653 .6021 L
Mistroke
.34716 .60244 L
.34785 .60274 L
.34917 .60314 L
.35042 .60331 L
.35163 .60327 L
.35276 .60307 L
.35346 .60287 L
.35421 .60258 L
.35558 .60188 L
.35691 .60098 L
.35813 .59999 L
.36092 .59711 L
.36385 .5932 L
.37346 .57517 L
.384 .54832 L
.42444 .42439 L
.44309 .37568 L
.45293 .35484 L
.46337 .33684 L
.46862 .32945 L
.47442 .32261 L
.47974 .31754 L
.48463 .31392 L
.48963 .31125 L
.49225 .31026 L
.49357 .30988 L
.49501 .30953 L
.49629 .3093 L
.49745 .30915 L
.4981 .30909 L
.49879 .30905 L
.50006 .30902 L
.50125 .30905 L
.5025 .30915 L
.50357 .30928 L
.50475 .30949 L
.50605 .30978 L
.50725 .31011 L
.50997 .31108 L
.51237 .3122 L
.51493 .31364 L
.5198 .31716 L
.52421 .32119 L
.53499 .3344 L
.54511 .35113 L
.56525 .39609 L
.58374 .44875 L
.60421 .51311 L
.6142 .54321 L
.6233 .56755 L
Mistroke
.62782 .57799 L
.63274 .58768 L
.63738 .59495 L
.63945 .59753 L
.64167 .59981 L
.6429 .60084 L
.64402 .60163 L
.64516 .60228 L
.64621 .60275 L
.64744 .60312 L
.64809 .60324 L
.64878 .60331 L
.65006 .60327 L
.65122 .60305 L
.65252 .60259 L
.65372 .60195 L
.65504 .60101 L
.65643 .59975 L
.65877 .59698 L
.66134 .59298 L
.66658 .58155 L
.67154 .56652 L
.68084 .527 L
.69085 .46818 L
.70175 .38739 L
.72113 .21963 L
.73142 .1339 L
.73683 .09466 L
.74252 .06003 L
.74756 .03659 L
.75042 .0268 L
.75179 .02309 L
.75308 .02018 L
.75426 .01806 L
.75553 .01634 L
.75669 .0153 L
.75776 .0148 L
.75892 .01476 L
.76019 .01533 L
.76083 .01585 L
.76152 .01661 L
.76276 .01845 L
.76403 .02096 L
.7652 .02388 L
.76786 .03257 L
.7702 .04264 L
.7727 .05579 L
.78177 .12375 L
.79208 .23232 L
.80322 .36868 L
Mistroke
.81284 .48063 L
.81794 .53 L
.82074 .55274 L
.8234 .57081 L
.82599 .5849 L
.82735 .5908 L
.8288 .59586 L
.83002 .59913 L
.83071 .60057 L
.83136 .60166 L
.8325 .60291 L
.83373 .60332 L
.8349 .60279 L
.83618 .60118 L
.83683 .59996 L
.83752 .59835 L
.83876 .59465 L
.84117 .58454 L
.84337 .57207 L
.84857 .53042 L
.85421 .46831 L
.86444 .32468 L
.87473 .17223 L
.87976 .10819 L
.88435 .06162 L
.88683 .0424 L
.88918 .02859 L
.89047 .02296 L
.89167 .01902 L
.89298 .01615 L
.89364 .01528 L
.89438 .01478 L
.89507 .01476 L
.8958 .01521 L
.89711 .01725 L
.89784 .01907 L
.89852 .02121 L
.90006 .02763 L
.90141 .03503 L
.90264 .04318 L
.9054 .06626 L
.91023 .12129 L
.91536 .19625 L
.92455 .35346 L
.92932 .43404 L
.93444 .50987 L
.93738 .54526 L
.94007 .57107 L
.94136 .58097 L
.94277 .5898 L
Mistroke
.94341 .59312 L
.9441 .59616 L
.94534 .6003 L
.94626 .60225 L
.94727 .60325 L
.94833 .60302 L
.94931 .60162 L
.95024 .59926 L
.95108 .59624 L
.95205 .59172 L
.95297 .58643 L
.95502 .57113 L
.95692 .55286 L
.96123 .49812 L
.96848 .37375 L
.97619 .22429 L
Mfstroke
0 0 1 r
2 Mabswid
.02381 .11665 m
.04262 .084 L
.06244 .05567 L
.08255 .0341 L
.09396 .02531 L
.09905 .02222 L
.10458 .01947 L
.11004 .01736 L
.11243 .01662 L
.11508 .01595 L
.11756 .01545 L
.11989 .0151 L
.12118 .01496 L
.12235 .01485 L
.123 .01481 L
.12371 .01477 L
.125 .01473 L
.12626 .01472 L
.12744 .01474 L
.12851 .01478 L
.12966 .01485 L
.13092 .01497 L
.13225 .01512 L
.13463 .01549 L
.13683 .01593 L
.13889 .01644 L
.14357 .01791 L
.14858 .01999 L
.15402 .02283 L
.16392 .02952 L
.18248 .04722 L
.20226 .07305 L
.22326 .10759 L
.26101 .18455 L
.30122 .27993 L
.3399 .37496 L
.38104 .46822 L
.40157 .50833 L
.42067 .54036 L
.44027 .56705 L
.45878 .58585 L
.46859 .59314 L
.47376 .5962 L
.47923 .59885 L
.48384 .60061 L
.48628 .60137 L
.48891 .60204 L
.49131 .60254 L
.49351 .60288 L
.49457 .60301 L
Mistroke
.49572 .60313 L
.49681 .60321 L
.49782 .60327 L
.49911 .60331 L
.49976 .60332 L
.50048 .60332 L
.50174 .60329 L
.50292 .60323 L
.50363 .60318 L
.50431 .60313 L
.50557 .603 L
.50842 .60258 L
.5109 .60209 L
.51319 .60151 L
.51838 .59982 L
.52268 .59799 L
.52742 .59555 L
.537 .58922 L
.54712 .58056 L
.55639 .57092 L
.57726 .54349 L
.61694 .4724 L
.6551 .38694 L
.69571 .2875 L
.73481 .19398 L
.77636 .10828 L
.7972 .07386 L
.81639 .04851 L
.83511 .03028 L
.84455 .02368 L
.84987 .02074 L
.85491 .01849 L
.85979 .01681 L
.86255 .01608 L
.8651 .01554 L
.86724 .01519 L
.86847 .01503 L
.8696 .01492 L
.87079 .01482 L
.87207 .01475 L
.8732 .01472 L
.8744 .01472 L
.8757 .01474 L
.87689 .0148 L
.87755 .01485 L
.87827 .0149 L
.87956 .01504 L
.88207 .01539 L
.88438 .01583 L
.88968 .01726 L
Mistroke
.8953 .01941 L
.90472 .02446 L
.91485 .03187 L
.93316 .05031 L
.95431 .07918 L
.9744 .11332 L
.97619 .11665 L
Mfstroke
0 g
.5 Mabswid
[ 2 3 ] 0 Mabsdash
.02381 .02718 m
.02499 .02137 L
.02605 .01766 L
.02729 .01523 L
.02846 .01477 L
.02954 .01592 L
.03053 .01831 L
.03163 .02239 L
.03279 .02834 L
.0352 .04587 L
.03746 .0682 L
.04262 .13708 L
.05278 .31634 L
.06244 .48196 L
.06751 .54656 L
.07038 .57312 L
.07175 .58297 L
.07305 .59052 L
.07423 .59591 L
.07551 .6001 L
.07667 .60241 L
.07774 .60329 L
.0789 .60289 L
.08017 .60084 L
.08081 .59918 L
.0815 .59695 L
.08274 .59172 L
.0842 .58367 L
.08555 .57446 L
.08861 .54775 L
.09417 .48139 L
.10458 .3199 L
.11356 .17875 L
.11828 .11625 L
.12324 .06461 L
.12589 .04409 L
.12729 .03536 L
.12879 .02771 L
.13017 .02221 L
.13142 .01851 L
.13268 .01605 L
.13387 .01488 L
.13508 .01483 L
.13636 .01602 L
.13745 .01804 L
.13866 .02129 L
.13995 .02598 L
.14134 .03236 L
.14385 .04724 L
.1491 .09105 L
Mistroke
.15398 .14423 L
.16502 .2903 L
.17468 .41997 L
.17976 .47952 L
.18512 .53169 L
.18776 .55256 L
.19064 .57148 L
.19327 .58493 L
.19572 .59418 L
.19701 .59778 L
.19824 .60035 L
.19934 .60198 L
.20053 .60302 L
.20183 .6033 L
.20249 .60311 L
.20321 .60264 L
.20449 .60115 L
.20569 .59902 L
.20678 .59645 L
.20797 .593 L
.21037 .58403 L
.21465 .56176 L
.21966 .52681 L
.2243 .4873 L
.24308 .2929 L
.25296 .19276 L
.2634 .10572 L
.26871 .07175 L
.27152 .05688 L
.27454 .04341 L
.27731 .03333 L
.27984 .02606 L
.28105 .02321 L
.28238 .02059 L
.28354 .01869 L
.28479 .01707 L
.28603 .01587 L
.2872 .01514 L
.28788 .01488 L
.28859 .01474 L
.28986 .01481 L
.29051 .01501 L
.2912 .01534 L
.29244 .01624 L
.2932 .01697 L
.29391 .0178 L
.29526 .01967 L
.29768 .0241 L
.29991 .02934 L
.30495 .04495 L
Mistroke
.31463 .08718 L
.32369 .13741 L
.34432 .26817 L
.36287 .38012 L
.38311 .47684 L
.39408 .51584 L
.40426 .54389 L
.41399 .56413 L
.41939 .57293 L
.42436 .57966 L
.43465 .59006 L
.44007 .59392 L
.44579 .59701 L
.45076 .59902 L
.45541 .60043 L
.4605 .60154 L
.46336 .602 L
.46596 .60234 L
.46844 .60259 L
.47077 .60278 L
.4732 .60294 L
.47588 .60307 L
.47819 .60315 L
.4803 .60321 L
.48262 .60325 L
.48379 .60327 L
.48506 .60328 L
.48646 .60329 L
.48779 .6033 L
.48898 .60331 L
.49026 .60331 L
.49093 .60331 L
.49167 .60332 L
.49244 .60332 L
.49316 .60332 L
.49444 .60332 L
.49516 .60332 L
.49582 .60332 L
.4971 .60332 L
.49828 .60332 L
.49957 .60332 L
.5003 .60332 L
.50095 .60332 L
.50216 .60332 L
.50347 .60332 L
.5047 .60332 L
.50583 .60332 L
.5071 .60332 L
.5083 .60332 L
.50938 .60331 L
Mistroke
.51055 .60331 L
.51182 .6033 L
.51317 .6033 L
.51433 .60329 L
.5156 .60328 L
.51786 .60324 L
.51913 .60322 L
.5203 .60319 L
.52256 .60313 L
.5247 .60305 L
.52718 .60292 L
.52983 .60274 L
.53233 .60252 L
.53459 .60227 L
.53726 .60191 L
.54015 .60142 L
.54539 .60022 L
.5503 .59864 L
.55489 .5967 L
.55992 .59393 L
.56532 .59009 L
.57097 .58493 L
.57633 .5788 L
.58638 .56347 L
.59602 .5432 L
.60628 .5147 L
.62469 .44327 L
.64416 .33971 L
.66243 .22461 L
.6809 .11095 L
.68596 .08422 L
.69137 .05929 L
.69604 .04149 L
.70111 .02682 L
.70247 .02379 L
.70392 .02103 L
.70525 .01891 L
.7065 .0173 L
.70767 .01614 L
.70893 .01527 L
.70965 .01495 L
.71033 .01477 L
.7116 .01476 L
.71275 .01511 L
.71397 .01587 L
.71512 .01696 L
.71616 .01826 L
.71873 .02274 L
.72114 .02865 L
.72371 .03677 L
Mistroke
.72642 .0474 L
.73128 .07167 L
.7365 .10506 L
.74224 .14969 L
.76341 .3627 L
.77343 .46613 L
.7827 .54435 L
.78735 .57312 L
.7897 .58431 L
.79105 .58965 L
.79229 .5938 L
.79353 .59722 L
.79488 .60011 L
.79558 .60126 L
.79624 .6021 L
.79693 .60275 L
.79767 .60318 L
.79832 .60332 L
.79903 .60322 L
.80031 .60237 L
.80156 .6007 L
.80271 .59843 L
.8038 .59562 L
.80498 .59189 L
.80709 .58328 L
.80961 .56996 L
.81191 .55493 L
.81691 .51337 L
.82165 .46386 L
.83983 .2235 L
.84482 .15883 L
.8502 .09805 L
.85267 .07454 L
.85527 .05344 L
.85771 .03749 L
.85993 .02647 L
.86114 .02193 L
.86229 .01864 L
.8635 .01623 L
.86481 .01489 L
.86596 .01481 L
.86717 .01585 L
.86821 .01767 L
.86936 .02065 L
.8707 .02548 L
.87192 .03113 L
.87468 .0481 L
.87724 .06893 L
.87967 .093 L
.88971 .22792 L
Mistroke
.90067 .40491 L
.90599 .48345 L
.90859 .51689 L
.91099 .5437 L
.91314 .56396 L
.91549 .58171 L
.91663 .58851 L
.91785 .59442 L
.91914 .59906 L
.92035 .60188 L
.92103 .60281 L
.92175 .60328 L
.92305 .60276 L
.92377 .60171 L
.92444 .60022 L
.92518 .59804 L
.92598 .59505 L
.92734 .58845 L
.92877 .57945 L
.93132 .55829 L
.9361 .50262 L
.94132 .42212 L
.97619 .02718 L
Mfstroke
MathSubEnd
P
% End of sub-graphic
0 g
.78765 .03359 m
.78765 .30517 L
.97619 .30517 L
.97619 .03359 L
F
1 g
.77068 .05056 m
.77068 .32215 L
.95922 .32215 L
.95922 .05056 L
F
0 g
.001 w
[ ] 0 setdash
.77068 .05056 m
.95922 .05056 L
.95922 .32215 L
.77068 .32215 L
.77068 .05056 L
s
% Start of sub-graphic
p
0.770676 0.0505614 0.959216 0.322149 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0297619 0.297619 0.0297619 0.297619 [
[ 0 0 0 0 ]
[ 1 1.44048 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.44048 L
0 1.44048 L
closepath
clip
newpath
% Start of sub-graphic
p
0.0535714 0.0535714 0.35119 0.35119 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.309017 0.294302 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 g
.5 Mabswid
[ 2 3 ] 0 Mabsdash
.02381 .30902 m
.97619 .30902 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38988 .20238 -61 -10.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.4375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.437500 moveto
 541.000000 16.437500 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.437500 lineto
closepath
clip newpath
63.000000 13.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Cos) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
81.000000 13.437500 moveto
(@) show
87.000000 13.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(x) show
93.000000 9.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.125000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
98.250000 13.437500 moveto
(D) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math2Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math2Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math2Mono
%%+ Courier
grestore
% Start of sub-graphic
p
0.0535714 0.39881 0.35119 0.696429 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.309017 0.294302 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
2 Mabswid
[ ] 0 setdash
.02381 .30902 m
.97619 .30902 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38988 .54762 -61 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Cos) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
81.000000 13.000000 moveto
(@) show
87.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(x) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
93.000000 13.000000 moveto
(D) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math2Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math2Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math2Mono
%%+ Courier
grestore
% Start of sub-graphic
p
0.0535714 0.744048 0.35119 1.04167 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.309017 0.294302 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.2 g
.5 Mabswid
[ ] 0 setdash
.02381 .30902 m
.97619 .30902 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38988 .89286 -61 -10.2188 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20.4375 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.437500 moveto
 541.000000 16.437500 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.437500 lineto
closepath
clip newpath
63.000000 13.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Sin) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
81.000000 13.437500 moveto
(@) show
87.000000 13.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(x) show
93.000000 9.437500 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 7.125000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(2) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
98.250000 13.437500 moveto
(D) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math2Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math2Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math2Mono
%%+ Courier
grestore
% Start of sub-graphic
p
0.0535714 1.08929 0.35119 1.3869 MathSubStart
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.952381 0.309017 0.294302 [
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.5 Mabswid
[ ] 0 setdash
.02381 .30902 m
.97619 .30902 L
s
MathSubEnd
P
% End of sub-graphic
0 g
gsave
.38988 1.2381 -61 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Sin) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
81.000000 13.000000 moveto
(@) show
87.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(x) show
%%IncludeResource: font Math2Mono
%%IncludeFont: Math2Mono
/Math2Mono findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
93.000000 13.000000 moveto
(D) show
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Math2Mono
%%+ font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Math2Mono
%%+ Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Math2Mono
%%+ Courier
grestore
MathSubEnd
P
% End of sub-graphic
0 0 m
1 0 L
1 .44061 L
0 .44061 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{375, 164.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40008b0000ma000`40O003h00Oogooogoo=7oo003o
OoooOoldOol00?mooomoocAoo`00ogooogoo=7oo003oOoooOoldOol00?mooomoocAoo`00c7oo00<0
07ooOol0ogooI7oo003<Ool00`00Oomoo`3oOomTOol00<aoo`03001oogoo0?moofAoo`00`Woo0`00
1goo00<007ooOol0ogooI7oo0015Ool901meOol00`00Oomoo`06Ool00`00Oomoo`1lOol901oNOol0
02Ioo`8001Uooa407amoo`8HaQAoo`Ml02ioo`D0009oo`03001oogoo00Ioo`<004Ioo`8000moo`8H
aQiooa407m]oo`009Woo0P005goo5@0O77oo00<HaWoo6<H03Goo0P000Woo0g`01goo17`0<Goo00<0
07ooOol01Woo00<007ooOol0AGoo00@007ooOol000eoo`046<IooaS66<HKOolE01oIOol002Ioo`03
001oogoo01Eoo`T07`Aoo`0300007`0000P07aYoo`046<Ioogoo6<H<Ool01@00Ooml07`000003Goo
0W`0;goo00<007ooOol01Woo00<007ooOol0EGoo00DHaWooOomooaS601Yoo`P07`0300007`0000=o
o`T07mQoo`00?7oo1`0O00<HaWooOol017oo00@007ooOol0009oo`L07aQoo`056<IoogooOolHaP0;
Ool00`00O01l0003Ool00`00Oomoo`0<Ool2O00/Ool20008Ool00`00Oomoo`1EOol00aS6Oomoo`02
Ool00aS6Oomoo`0FOol601l2Ool01000Oomoo`001Woo00<HaP0O01l01@0OeWoo000kOol601l2Ool0
0aS6Oomoo`04Ool01@00Oomoogoo00000goo1P0O5goo00<HaWooOol00Woo00<HaWooOol01goo0W`0
1Goo00<007ooOol03Woo00=l07ooOol0<goo00<007ooOol0E7oo00<HaWooOol00goo00<HaWooOol0
5Goo1@0O17oo00D007ooOomoo`0000Eoo`036<Ioogoo00L07mAoo`00:7oo00<007ooOol03goo1P0O
00@HaWooOolHaPIoo`05001oogooOol00005Ool501lEOol00aS6Oomoo`03Ool00aS6Oomoo`06Ool0
0g`0Oomoo`06Ool00`00Oomoo`0>Ool2O00cOol00`00Oomoo`13Ool00`00Oomoo`04Ool00`00Oomo
o`07Ool00aS6Oomoo`04Ool00aS6Oomoo`0COol501l9Ool00`00Oomoo`03Ool011S6Oomoogoo1`0O
dgoo000XOol00`00Oomoo`0>Ool501l2Ool011S6OomooaS64Goo1@0O57oo00<HaWooOol017oo00<H
aWooOol017oo00=l07ooOol06Woo00=l07ooOol0<7oo00<007ooOol0@goo00<007ooOol017oo00<0
07ooOol01goo00<HaWooOol017oo00<HaWooOol04Woo1@0O47oo00HHaWooOomooaS6Ool501oCOol0
02Uoo`03001oogoo00aoo`D07`9oo`036<Ioogoo009oo`036<Ioogoo00moo`D07a=oo`036<Ioogoo
00Aoo`036<Ioogoo00=oo`04O01oogoo000KOol00g`0Oomoo`0_Ool00`00Oomoo`12Ool00`00Oomo
o`06Ool00`00Oomoo`05Ool00aS6Oomoo`05Ool00aS6Oomoo`0AOol501lAOol01AS6Oomoogoo6<H0
0goo100O4Woo00<007ooOol0_Goo000YOol00`00Oomoo`0;Ool501l3Ool00aS6Oomoo`02Ool00aS6
Oomoo`0@Ool501lAOol00aS6Oomoo`05Ool00aS6Oomoo`02Ool01G`0Oomoogoo000077oo00=l07oo
Ool0;Woo00<007ooOol0@Woo00<007ooOol01Woo00<007ooOol01Goo00<HaWooOol01Goo00<HaWoo
Ool047oo1@0O4Goo00<HaWooOol00Woo00@HaWooOomoo`D07a5oo`03001oogoo0;eoo`00=Woo1@0O
17oo00<HaWooOol00Woo00<HaWooOol00Woo00<007ooOol037oo1@0O47oo00<HaWooOol01Goo00DH
aWooOomoog`000=oo`03001oogoo00Moo`03001oogoo015oo`03O01oogoo02ioo`03001oogoo04]o
o`03001oogoo00Eoo`036<Ioogoo00Eoo`036<Ioogoo00moo`D07`Qoo`03001oogoo00Moo`036<Io
ogoo00=oo`046<IoogooOol501l@Ool00`00Oomoo`14OomZ000?Ool003Eoo`D07`Eoo`036<Ioogoo
009oo`036<Ioogoo009oo`03001oogoo00=oo`03001oogoo00Moo`D07`moo`036<Ioogoo00Ioo`03
6<Ioog`000Aoo`03001oogoo00Moo`03001oogoo019oo`03O01oogoo02eoo`8004eoo`03001oogoo
00Aoo`036<Ioogoo00Eoo`036<Ioogoo00ioo`D07`Uoo`03001oogoo00=oo`05001oogooOolHaP05
Ool00aS6Oomoo`02Ool501l?Ool00`00Oomoo`14OomZ000?Ool003Aoo`D07`Ioo`036<Ioogoo00=o
o`056<IoogooOol00005Ool00`00Oomoo`08Ool501l>Ool00aS6Oomoo`06Ool00aS6Ooml000?Ool0
0`00Oomoo`0BOol00g`0Oomoo`0/Ool00`00Oomoo`1COol00aS6Oomoo`05Ool00aS6Oomoo`0=Ool5
01l9Ool00`00Oomoo`04Ool01@00Oomoogoo6<H01Goo00<HaWooOol00goo1@0OEGooJP003goo000c
Ool501l7Ool00aS6Oomoo`03Ool01AS6Oomoogoo00001Goo00<007ooOol02Goo100O3Woo00<HaWoo
Ool01Woo00<HaW`0Ool09Goo00=l07ooOol0:goo00<007ooOol0@Goo00<007ooOol03goo00<HaWoo
Ool01Goo00<HaWooOol03Goo100O2Woo00<007ooOol017oo00D007ooOomooaS600Eoo`036<Ioogoo
00Aoo`D07eAoofX000moo`00:Goo00<007ooOol01Woo1@0O27oo00<HaWooOol00goo00<HaWooOol0
1goo00<007ooOol02Woo100O3Goo00<HaWooOol01Woo00<HaWooOol09Goo00=l07ooOol0:goo00<0
07ooOol0@Goo00<007ooOol03Woo00<HaWooOol01goo00<HaWooOol02goo100O4Woo00D007ooOomo
oaS600Eoo`036<Ioogoo00Eoo`@07eAoofX000moo`00:Goo00<007ooOol01Woo100O2Goo00<HaWoo
Ool00goo00<HaWooOol057oo1@0O2goo00<HaWooOol01Woo00=l01S6Ool09goo00=l07ooOol0:Woo
00<007ooOol0@7oo00<007ooOol03goo00<HaWooOol01goo00<HaWooOol02Woo1@0O5Goo00<HaWoo
Ool017oo00<HaWooOol01Woo100ODgooJP003goo000ZOol00`00Oomoo`04Ool401l:Ool00aS6Oomo
o`03Ool00aS6Oomoo`0EOol501l:Ool00aS6Oomoo`05Ool00g`0OolHaP05Ool00`00Oomoo`09Ool0
0`00Oomoo`0EOol00g`0Oomoo`0YOol00`00Oomoo`10Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool0
0aS6Oomoo`07Ool00aS6Oomoo`09Ool501lFOol00aS6Oomoo`05Ool00aS6Oomoo`05Ool501l;Ool0
0`00Oomoo`14OomZ000?Ool002Yoo`03001oogoo00=oo`D07`Yoo`036<Ioogoo00=oo`046<Ioogoo
000EOol501l9Ool00aS6Oomoo`04Ool01G`0Oomoogoo6<H017oo00<007ooOol02Goo00<007ooOol0
5Woo00=l07ooOol0:7oo00<007ooOol0CGoo00<007ooOol00Woo00<HaWooOol01goo00<HaWooOol0
27oo1@0O3Goo00<007ooOol01goo00<HaWooOol01Goo00<HaWooOol01Woo1@0O2Woo00<007ooOol0
A7ooJP003goo000`Ool401l:Ool00aS6Oomoo`04Ool011S6Oomoo`001goo00<007ooOol037oo100O
2Goo00<HaWooOol00goo00=l07ooOol00Woo00DHaWooOomoo`0000eoo`03001oogoo01Eoo`03O01o
ogoo02Qoo`03001oogoo04eoo`03001oogoo009oo`036<Ioogoo00Moo`036<Ioogoo00Qoo`@07`io
o`03001oogoo00Eoo`03001ooaS600Moo`036<Ioogoo00Moo`D07`Qoo`03001oogoo03aoog<000mo
o`00;goo100O2goo00<HaWooOol017oo00@HaWooOol000Moo`03001oogoo00eoo`@07`Qoo`036<Io
ogoo00=oo`03O01oogoo009oo`056<IoogooOol0000=Ool00`00Oomoo`0FOol00g`0Oomoo`0WOol0
0`00Oomoo`1BOol00aS6Oomoo`07Ool00aS6Oomoo`07Ool401l>Ool00`00Oomoo`06Ool00`00OolH
aP07Ool00aS6Oomoo`08Ool401l8Ool00`00Oomoo`0lOol00`00Oomoo`1VOol:000?Ool002ioo`D0
7`]oo`036<Ioogoo00Aoo`046<Ioogoo0007Ool00`00Oomoo`0=Ool501l7Ool00aS6Oomoo`02Ool0
0g`0Oomoo`03Ool00aS6Oomoo`0YOol00g`0Oomoo`0VOol00`00Oomoo`0oOol00`00Oomoo`0@Ool0
0aS6Oomoo`07Ool00aS6Oomoo`06Ool501l>Ool00`00Oomoo`06Ool00`00OolHaP07Ool00aS6Oomo
o`09Ool401m6Ool00`00Oomoo`1VOol:000?Ool002Yoo`03001oogoo00D07`aoo`036<Ioogoo00Ao
o`036<Ioogoo00Qoo`03001oogoo00ioo`D07`Ioo`056<IoogooOoml0006Ool00aS6Oomoo`0YOol0
0g`0Oomoo`0VOol00`00Oomoo`0oOol00`00Oomoo`0?Ool00aS6Oomoo`09Ool00aS6Oomoo`04Ool5
01lHOol00`00OolHaP07Ool00aS6Oomoo`09Ool501m5Ool00`00Oomoo`1VOol:000?Ool002Yoo`03
001oogoo00@07`eoo`036<Ioogoo00Eoo`036<Ioogoo01Uoo`@07`Ioo`056<IoogooOoml0007Ool0
0aS6Oomoo`0YOol00g`0Oomoo`0UOol2000oOol00`00Oomoo`0@Ool00aS6Oomoo`09Ool00aS6Oomo
o`04Ool401lKOol00aS6Oomoo`05Ool00aS6Oomoo`0:Ool501m4Ool00`00Oomoo`1VOol:000?Ool0
02Yoo`03001oo`0O00<07`ioo`036<Ioogoo00Eoo`036<Ioogoo01Yoo`@07`Aoo`056<IoogooOoml
0008Ool011S6Oomoo`003Woo00<007ooOol067oo00=l07ooOol097oo00<007ooOol0?Woo00<007oo
Ool03Goo00@007ooOolHaP]oo`036<Ioogoo00=oo`@07aaoo`036<Ioogoo00Eoo`036<Ioogoo00]o
o`D07`Aoo`03001oogoo03aoo`03001oogoo06Ioo`X000moo`00:Woo00<0000O01l00`0O3Woo00<H
aWooOol01Goo00<HaWooOol06Woo100O17oo00@HaWooOoml00Uoo`046<Ioogoo000>Ool00`00Oomo
o`0IOol00g`0Oomoo`0SOol00`00Oomoo`1>Ool01000OomooaS62goo00<HaWooOol00goo100O77oo
00<HaWooOol01Goo00<HaWooOol037oo100O17oo00<007ooOol0?7oo00<007ooOol0IWoo2P003goo
000[Ool401l?Ool00aS6Oomoo`05Ool00aS6001oo`08Ool00`00Oomoo`0@Ool401l3Ool011S6Oomo
og`02Goo00<HaWoo000047oo00<007ooOol067oo00=l07ooOol08goo00<007ooOol0CWoo00@007oo
OolHaP]oo`036<Ioogoo009oo`@07a9oo`03001oogoo00Ioo`03001ooaS600Moo`036<Ioogoo00eo
o`@07`=oo`03001oogoo03aoo`03001oogoo06Ioo`X000moo`00:Woo100O47oo00<HaWooOol01Goo
00<HaP00Ool027oo00<007ooOol047oo1@0O0Woo00<HaWooO0002Woo00<HaWoo000047oo00<007oo
Ool06Goo00=l07ooOol08Woo00<007ooOol0CWoo00@007ooOolHaP]oo`046<IoogooOol501lBOol0
0`00Oomoo`06Ool00`00OolHaP07Ool00aS6Oomoo`0=Ool401l3Ool00`00O01oo`0lOol00`00Oomo
o`1VOol:000?Ool002Yoo`@07`moo`036<Ioogoo00Ioo`036<H007oo00Qoo`03001oogoo015oo`@0
7`9oo`036<Ioog`000Yoo`036<Ioogoo02eoo`03O01oogoo025oo`03001oogoo051oo`036<Ioogoo
00Yoo`046<IoogooOol401lCOol00`00Oomoo`07Ool00`00Oomoo`06Ool00aS6Oomoo`0>Ool401l3
Ool00g`0Oomoo`0kOol00`00Oomoo`1VOol:000?Ool002Uoo`807`0300007goo00moo`036<Ioogoo
00Ioo`036<H007oo00Qoo`03001oogoo019oo`@07`03OolHaW`000aoo`036<Ioogoo02aoo`03O01o
ogoo025oo`03001oogoo03eoo`03001oogoo011oo`036<Ioogoo00]oo`036<Ioo`0O00<07aAoo`03
001oogoo00Moo`03001oogoo00Ioo`036<Ioogoo00ioo`D07`03Ooml07oo03eoo`03001oogoo06Io
o`X000moo`00:7oo0`0O00<0000OOol03goo00<HaWooOol01Woo00<HaWooOol07Goo100O00=ooaS6
Ool037oo00<HaWooOol0;Goo00=l07ooOol087oo00<007ooOol0?Goo00<007ooOol047oo00<HaWoo
Ool02goo00<HaWoo01l00`0O7Woo00<HaWooOol01goo00<HaWooOol03Woo100O00=oog`0Ool0?Goo
00<007ooOol0IWoo2P003goo000XOol301l00`00Oomoo`0?Ool00aS6Oomoo`06Ool00aS6Oomoo`0N
Ool401l00g`0Oomoo`0;Ool00`00Oomoo`0@Ool00`00Oomoo`0KOol00g`0Oomoo`0OOol00`00Oomo
o`0mOol00`00Oomoo`0?Ool00`006<Ioo`0<Ool00aS601l07`0201lOOol00aS6Oomoo`07Ool00aS6
Oomoo`0?Ool401loOol00`00Oomoo`1VOol:000?Ool002Moo`@07`03001oogoo00moo`036<Ioogoo
00Ioo`036<Ioogoo01ioo`D07`eoo`03001oogoo011oo`03001oogoo01]oo`03O01oogoo01moo`03
001oogoo03eoo`03001oogoo00moo`03000HaWoo00aoo`D07amoo`036<Ioogoo00Moo`036<Ioogoo
00moo`<07`03001oogoo03eoo`03001oogoo06Ioo`X000moo`009goo100O4Woo00<HaWooOol01Woo
00<HaP00Ool027oo00<007ooOol057oo100O3Goo00<007ooOol047oo00<007ooOol077oo00=l07oo
Ool07Woo0P00DGoo00<007ooOol02goo100O5Goo00<007ooOol027oo00<HaWooOol01goo00<HaWoo
Ool047oo0P0O00<0000OOol0?Goo00<007ooOol0IWoo2P003goo000VOol401lCOol00aS6Oomoo`06
Ool00aS6001oo`08Ool00`00Oomoo`0EOol401l<Ool00`006<Ioo`0@Ool00`00Oomoo`0MOol00g`0
Oomoo`0MOol00`00Oomoo`1?Ool00aS6001oo`0;Ool401lFOol00`00Oomoo`08Ool00`00Oomoo`07
Ool00aS6Oomoo`0@Ool201l00`0001l07`0mOol00`00Oomoo`1VOol:000?Ool002Eoo`D07a=oo`03
6<Ioogoo00Ioo`03000HaWoo00Uoo`03001oogoo01Aoo`@07`eoo`036<Ioogoo02moo`03O01oogoo
01eoo`03001oogoo04moo`036<Ioogoo00]oo`@07aIoo`03001oogoo00Qoo`03001oogoo00Moo`03
6<Ioogoo011oo`03O0007`0000807ceoo`03001oogoo06Ioo`X000moo`009Goo100O57oo00<HaWoo
Ool01Woo00<001S6Ool02Goo00<007ooOol04goo00=l07oo01l00`0O37oo00<HaWooOol0<7oo00=l
07ooOol077oo00<007ooOol0?7oo00<007ooOol047oo00<HaWooOol02Woo100O5goo00<007ooOol0
27oo00<007ooOol01goo00<HaWooOol047oo00=l07oo01l00`0O?7oo00<007ooOol0IWoo2P003goo
000TOol401l3Ool00`00Oomoo`0?Ool00aS6Oomoo`07Ool00aS6Oomoo`0MOol00g`0Oomoo`0401l<
Ool00aS6Oomoo`0`Ool00g`0Oomoo`0LOol00`00Oomoo`0lOol00`00Oomoo`0@Ool00aS6Oomoo`0:
Ool401lROol00`00Oomoo`07Ool00aS6Oomoo`0?Ool00g`0Oomoo`0401llOol00`00Oomoo`0mOol2
000=Ool2000HOol:000?Ool002Aoo`@07`=oo`03001oogoo00ioo`036<Ioogoo00Qoo`036<Ioogoo
01eoo`04O01oogoo6<H401l;Ool00aS6Oomoo`0aOol00g`0Oomoo`0KOol00`00Oomoo`0lOol00`00
Oomoo`0@Ool00aS6Oomoo`09Ool401l00aS6Oomoo`0POol00aS6Oomoo`07Ool00aS6Oomoo`0?Ool0
17`0Oomoogoo0`0O?7oo00<007ooOol0?Goo00<007ooOol03Goo00<007ooOol05Woo2P003goo000T
Ool301l4Ool00`00Oomoo`0>Ool00aS6Oomoo`08Ool00aS6Oomoo`0LOol01G`0Oomoogoo6<H0100O
2Goo00<007oo6<H04Woo00<007ooOol07goo00=l07ooOol06Woo00<007ooOol0?7oo00<007ooOol0
47oo00<HaP00Ool02Goo100O00<HaWooOol087oo00<HaWooOol01goo00<HaWooOol03Woo00El07oo
Oomoo`0000<07caoo`03001oogoo02Uoo`@000=oo`<0009oo`@000Aoo`03001oogoo00800003Ool0
000000Qoo`03001oogoo01Ioo`X000moo`00:goo00<007ooOol03Woo00<HaWooOol027oo00<HaWoo
Ool027oo00<007ooOol04Goo00Al07ooOolHaP9oo`@07`Qoo`03001ooaS6019oo`03001oogoo01mo
o`03O01oogoo01Yoo`03001oogoo04moo`036<H007oo00Qoo`@07`03OolHaWoo025oo`036<Ioogoo
00Moo`036<Ioogoo00ioo`05O01oogooOol0000oOol00`00Oomoo`0XOol00`00Oomoo`04Ool01@00
Oomoogoo000017oo00<007ooOol00Woo00L007ooOomoo`00Ool00009Ool00`00Oomoo`0FOol:000?
Ool003aoo`036<Ioogoo00Moo`03000HaWoo00Uoo`03001oogoo011oo`05O01oogooOolHaP02Ool4
01l8Ool01000OomooaS64Goo00<007ooOol087oo00=l07ooOol06Goo00<007ooOol0CWoo00@HaWoo
Ool000Qoo`@07`03OolHaWoo01Eoo`03001oogoo00Qoo`036<Ioo`0000Qoo`036<Ioogoo00eoo`05
O01oogooOol00010Ool00`00Oomoo`03Ool40003Ool40004Ool40003Ool4000;Ool00`00Oomoo`04
Ool01P00Oomoogoo001oo`@000Aoo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo01Ioo`X0
00moo`00?7oo00<HaWooOol01goo00<001S6Ool02Goo00<007ooOol047oo00El07ooOomooaS600=o
o`@07`Moo`04001oogoo6<HAOol00`00Oomoo`0POol00g`0Oomoo`0IOol00`00Oomoo`1>Ool011S6
Oomoo`001goo100O0Woo00<HaWooOol057oo00<007ooOol027oo00<HaWoo000027oo00<HaWooOol0
3Goo00El07ooOomoo`00041oo`03001oogoo02Qoo`03001oogoo00Aoo`07001oogooOol007oo0000
1goo00L007ooOomoo`00Ool00009Ool00`00Oomoo`0FOol:000?Ool003aoo`036<Ioogoo00Moo`03
000HaWoo00Uoo`03001oogoo00moo`03O01oogoo009oo`046<IoogooOol401l:Ool00aS6Oomoo`0c
Ool00g`0Oomoo`0HOol2000lOol00`00Oomoo`0@Ool00aS6Oomoo`08Ool401l2Ool00aS6Oomoo`0D
Ool00`00Oomoo`08Ool00aS6Ool00008Ool00aS6Oomoo`0<Ool00g`0Oomoo`13Ool00`00Oomoo`0X
Ool01@00Oomoogoo00000goo0`000Woo100017oo00<007ooOol00P0000=oo`0000000Woo0`000goo
00<007ooOol05Woo2P003goo000/Ool00`00Oomoo`0=Ool00aS6Oomoo`07Ool00`00OolHaP0KOol0
0g`0Oomoo`02Ool00aS6Oomoo`02Ool401l9Ool00aS6Oomoo`0cOol00g`0Oomoo`0HOol00`00Oomo
o`0kOol00`00Oomoo`0@Ool00aS6Oomoo`07Ool401l3Ool00aS6Oomoo`0DOol00`00Oomoo`08Ool0
0aS6Ool00008Ool00aS6Oomoo`0<Ool00g`0Oomoo`13Ool00`00Oomoo`0YOol4000@Ool20009Ool0
1000Oomoogoo0P0067oo2P003goo000/Ool00`00Oomoo`0=Ool00aS6Oomoo`09Ool00aS6Oomoo`0H
Ool00g`0Oomoo`03Ool00aS6Oomoo`02Ool401l9Ool00aS6Oomoo`0cOol00g`0Oomoo`0HOol00`00
Oomoo`0jOol00`00Oomoo`0AOol00aS6Oomoo`07Ool401l3Ool00aS6Oomoo`0OOol00aS6Oomoo`08
Ool00aS6Oomoo`0;Ool00g`0Oomoo`14Ool00`00Oomoo`19Ool00`00Oomoo`0JOol:000?Ool002ao
o`03001oogoo00eoo`036<Ioogoo00Uoo`036<Ioogoo01Qoo`03O01oogoo00=oo`036<Ioogoo00=o
o`@07`Aoo`05001oogooOolHaP0BOol00`00Oomoo`0QOol00g`0Oomoo`0GOol00`00Oomoo`0jOol0
0`00Oomoo`0AOol011S6Oomoo`001Goo100O1Goo00<HaWooOol07Woo00<HaWooOol027oo00<HaWoo
Ool02goo00=l07ooOol0A7oo00<007ooOol0Agoo0`0077oo2P003goo000/Ool00`00Oomoo`0=Ool0
0aS6Oomoo`09Ool00aS6Oomoo`0GOol00g`0Oomoo`04Ool00aS6Oomoo`03Ool401l4Ool01@00Oomo
ogoo6<H04Woo00<007ooOol08Goo00=l07ooOol05goo00<007ooOol0CWoo00@HaWooOol000Eoo`@0
7`Eoo`036<Ioogoo01ioo`036<Ioogoo00Uoo`036<Ioogoo00Uoo`03O01oogoo00Aoo`03001oogoo
03ioo`03001oogoo06Ioo`X000moo`00?7oo00<HaWooOol01Woo00@007ooOolHaPYoo`03001oogoo
00aoo`03O01oogoo00Aoo`036<Ioogoo00Aoo`@07`=oo`05001oogooOolHaP0COol00`00Oomoo`0Q
Ool00g`0Oomoo`0FOol00`00Oomoo`1>Ool01AS6Oomoogoo00000goo100O1Woo00<HaWooOol04goo
00<007ooOol027oo00<HaWooOol02Goo00<HaWooOol02Goo00=l07ooOol017oo00<007ooOol0?Woo
00<007ooOol0IWoo2P003goo000kOol00aS6Oomoo`07Ool01000OomooaS62Woo00<007ooOol02goo
00=l07ooOol017oo00<HaWooOol01Goo100O0goo00D007ooOomooaS601=oo`03001oogoo025oo`03
O01oogoo01Ioo`03001oogoo04eoo`036<Ioogoo009oo`04001oogooOol401l6Ool00aS6Oomoo`0C
Ool00`00Oomoo`08Ool00aS6Ool00009Ool00aS6Oomoo`08Ool00g`0Oomoo`05Ool00`00Oomoo`0n
Ool00`00Oomoo`1VOol:000?Ool003]oo`036<Ioogoo00Moo`04001oogoo6<H:Ool00`00Oomoo`0;
Ool00g`0Oomoo`04Ool00aS6Oomoo`06Ool301l8Ool00aS6Oomoo`0eOol00g`0Oomoo`0EOol00`00
Oomoo`1=Ool00aS6Oomoo`05Ool401l7Ool00aS6Oomoo`0COol00`00Oomoo`08Ool00aS6Ool00009
Ool00aS6Oomoo`08Ool00g`0Oomoo`05Ool00`00Oomoo`0nOol00`00Oomoo`1VOol:000?Ool003]o
o`036<Ioogoo00Moo`04001oogoo6<H:Ool00`00Oomoo`0:Ool00g`0Oomoo`05Ool00aS6Oomoo`06
Ool401l7Ool00aS6Oomoo`0eOol00g`0Oomoo`0EOol00`00Oomoo`0jOol00`00Oomoo`0@Ool00aS6
Oomoo`05Ool401l7Ool00aS6Oomoo`0COol00`00Oomoo`08Ool00aS6Ool00009Ool00aS6Oomoo`07
Ool00g`0Oomoo`17Ool00`00Oomoo`1VOol:000?Ool002aoo`03001oogoo00aoo`036<Ioogoo00Yo
o`036<Ioogoo01Eoo`03O01oogoo00Eoo`036<Ioogoo00Ioo`@07`Moo`036<Ioogoo03Ioo`04O01o
o`000004Ool20003Ool30007Ool00`00Oomoo`0jOol00`00Oomoo`0@Ool00aS6Oomoo`04Ool401l8
Ool00aS6Oomoo`0MOol011S6Oomoo`002Goo00<HaWooOol01goo00=l07ooOol0Agoo00<007ooOol0
IWoo2P003goo000/Ool00`00Oomoo`0<Ool00aS6Oomoo`0:Ool00aS6Oomoo`0EOol00g`0Oomoo`05
Ool00aS6Oomoo`07Ool401l6Ool00aS6Oomoo`0fOol01G`0001oogoo00000goo0P001Woo00<007oo
Ool017oo00<007ooOol0>Goo00<007ooOol04Goo00<HaWooOol017oo100O27oo00<HaWooOol07Goo
00<HaWooOol02Woo00<HaWooOol01Woo00=l07ooOol0B7oo00<007ooOol0IWoo2P003goo000/Ool0
0`00Oomoo`0<Ool00aS6Oomoo`0:Ool00aS6Oomoo`0DOol00g`0Oomoo`06Ool00aS6Oomoo`07Ool4
01l00goo001oo`03Ool00aS6Oomoo`0@Ool00`00Oomoo`0NOol500001GooO01oogoo00002goo00<0
07ooOol017oo0`00>Goo00<007ooOol04Goo00<HaWooOol00Woo00<0000O01l00P0O2Goo00<HaWoo
Ool07Goo00<HaWooOol02Woo00<HaWooOol01Woo00=l07ooOol01Woo00<007ooOol0?goo00<007oo
Ool0IWoo2P003goo000/Ool00`00Oomoo`0<Ool00aS6Oomoo`0:Ool00aS6Oomoo`08Ool00`00Oomo
o`09Ool00g`0Oomoo`06Ool00aS6Oomoo`08Ool401l00`00Oomoo`02Ool00aS6Oomoo`0@Ool00`00
Oomoo`0TOol017`0Oomoo`0027oo0`001goo00<007ooOol0CGoo00<HaWooOol00Woo00<0000O01l0
0P0O2Woo00<HaWooOol077oo00<HaWooOol02Woo00<HaWooOol01Goo00=l07ooOol01goo00<007oo
Ool0?goo00<007ooOol0IWoo2P003goo000kOol00aS6Oomoo`07Ool01@00Oomoogoo6<H02Goo00<0
07ooOol027oo00=l07ooOol01goo00<HaWooOol027oo0`0O00<007ooOol00goo00<HaWooOol04Goo
00<007ooOol08goo00Al07ooOol000Qoo`03001oogoo00Moo`03001oogoo04aoo`036<Ioogoo00=o
o`0400007`0O01l;Ool00aS6Oomoo`0AOol00`00Oomoo`08Ool01AS6Oomoogoo000027oo00<HaWoo
Ool01Goo00=l07ooOol01goo00<007ooOol0?goo00<007ooOol0IWoo2P003goo000kOol00aS6Oomo
o`07Ool01@00Oomoogoo6<H02Goo00<007ooOol027oo00=l07ooOol01goo00<HaWooOol02Goo0P0O
00<0000OOol00goo00<HaWooOol04Goo00<007ooOol097oo00=l0000Ool027oo10001Woo00<007oo
Ool0C7oo00<HaWooOol00goo00@0000O01l07`]oo`036<Ioogoo015oo`03001oogoo00Qoo`056<Io
ogooOol00008Ool00aS6Oomoo`04Ool00g`0Oomoo`08Ool00`00Oomoo`0oOol00`00Oomoo`1VOol:
000?Ool003]oo`036<Ioogoo00Moo`05001oogooOolHaP09Ool00`00Oomoo`07Ool00g`0Oomoo`08
Ool00aS6Oomoo`09Ool401l5Ool00aS6Oomoo`0gOol00g`0Oomoo`0BOol00`00Oomoo`0hOol00`00
Oomoo`0AOol00aS6Oomoo`02Ool401l<Ool00aS6Oomoo`0AOol00`00Oomoo`08Ool01AS6Oomoogoo
000027oo00<HaWooOol017oo00=l07ooOol0BWoo00<007ooOol0IWoo2P003goo000]Ool00`00Oomo
o`0;Ool00aS6Oomoo`07Ool01@00Oomoogoo6<H04goo00=l07ooOol01goo00<HaWooOol02goo100O
17oo00<HaWooOol0>7oo00=l07ooOol04Goo00<007ooOol0>7oo00<007ooOol04Goo00<HaWooOol0
0Woo100O37oo00<HaWooOol04Goo00<007ooOol027oo00DHaWooOomoo`0000Qoo`036<Ioogoo00Ao
o`03O01oogoo04Yoo`03001oogoo06Ioo`X000moo`00;Goo00<007ooOol02goo00<HaWooOol02goo
00<HaWooOol047oo00=l07ooOol027oo00<HaWooOol02goo100O17oo00<HaWooOol0>7oo00=l07oo
Ool04Goo00<007ooOol0>7oo00<007ooOol04Goo00@HaWooOomoo`@07`eoo`036<Ioogoo01aoo`03
6<Ioogoo00]oo`036<Ioogoo009oo`03O01oogoo04]oo`03001oogoo06Ioo`X000moo`00;Goo00<0
07ooOol02Woo00<HaWooOol037oo00<HaWooOol047oo00=l07ooOol027oo00<HaWooOol037oo00@0
000O01l07`=oo`036<Ioogoo015oo`03001oogoo02Eoo`03O01oogoo011oo`03001oogoo03Qoo`03
001oogoo015oo`046<IoogooOol301l00`00Oomoo`0;Ool00aS6Oomoo`0LOol00aS6Oomoo`0;Ool0
0aS6Oomoo`02Ool00g`0Oomoo`1;Ool00`00Oomoo`1VOol:000?Ool002eoo`03001oogoo00Yoo`03
6<Ioogoo00aoo`036<Ioogoo00moo`03O01oogoo00Uoo`036<Ioogoo00aoo`0400007`0O01l3Ool0
0aS6Oomoo`0AOol00`00Oomoo`0UOol00g`0Oomoo`0@Ool00`00Oomoo`1<Ool00aS6Oomoo`0401l0
0`00Oomoo`0;Ool00aS6Oomoo`0LOol00aS6Oomoo`0;Ool01AS6OomoogooO00037oo00<007ooOol0
?goo00<007ooOol0IWoo2P003goo000jOol00aS6Oomoo`07Ool00`00Oomoo`02Ool00aS6Oomoo`08
Ool00`00Oomoo`04Ool00g`0Oomoo`09Ool00aS6Oomoo`0;Ool00`00Ool07`0301l2Ool00aS6Oomo
o`0AOol00`00Oomoo`0VOol00g`0Oomoo`0?Ool00`00Oomoo`1<Ool00aS6Oomoo`0401l00`00Oomo
o`0;Ool00aS6Oomoo`0AOol00`00Oomoo`08Ool00aS6Oomoo`0;Ool01AS6OomoogooO00037oo00<0
07ooOol0?goo00<007ooOol0IWoo2P003goo000jOol00aS6Oomoo`07Ool00`00Oomoo`02Ool00aS6
Oomoo`08Ool00`00Oomoo`04Ool00g`0Oomoo`09Ool00aS6Oomoo`0;Ool00`00Ool07`0301l2Ool0
0aS6Oomoo`0AOol00`00Oomoo`0VOol00g`0Oomoo`0?Ool2001<Ool00aS6Oomoo`0401l00goo001o
o`0<Ool00aS6Oomoo`0AOol00`00Oomoo`08Ool01AS6Oomoogoo00002Goo00@HaWooOoml00aoo`03
001oogoo041oo`03001oogoo06Ioo`X000moo`00>Woo00<HaWooOol01goo00<007ooOol00Woo00<H
aWooOol027oo00<007ooOol00goo00=l07ooOol02Woo00<HaWooOol03Woo0`0O0Woo00<HaWooOol0
>Woo00=l07ooOol03goo00<007ooOol0Bgoo00<HaWooOol0100O47oo00<HaWooOol03goo00<007oo
Ool027oo00<HaWooOol00Woo00<007ooOol01goo00@HaWooOoml00aoo`03001oogoo041oo`03001o
ogoo06Ioo`X000moo`00>Woo00<HaWooOol01goo00<007ooOol00Woo00<HaWooOol027oo00<007oo
Ool00goo00=l07ooOol02Woo00<HaWooOol03Woo100O0Woo00<HaWooOol0>Woo00=l07ooOol03Woo
00<007ooOol0=goo00<007ooOol04Goo00<HaWoo01l00`0O4Goo00<HaWooOol03goo00<007ooOol0
27oo00<HaWooOol00Woo00<007ooOol01goo00@HaWooOoml04moo`03001oogoo06Ioo`X000moo`00
;Goo00<007ooOol02Woo00<HaWooOol037oo00<HaWooOol03Goo00=l07ooOol02goo00<HaWooOol0
3Woo100O0Woo00<HaWooOol0>Woo00=l07ooOol03Woo00<007ooOol0=goo00<007ooOol04Goo00<H
aWoo01l00`0O4Goo00<HaWooOol06Woo00<HaWooOol00Woo00<007ooOol01goo00<HaWooO000D7oo
00<007ooOol0IWoo2P003goo000]Ool00`00Oomoo`0:Ool00aS6Oomoo`0<Ool00aS6Oomoo`0=Ool0
0g`0Oomoo`0;Ool00aS6Oomoo`0?Ool401l00goo6<Ioo`0BOol00`00Oomoo`0WOol00g`0Oomoo`0=
Ool00`00Oomoo`0gOol00`00Oomoo`0AOol00aS6Ool07`0201lBOol00aS6Oomoo`0JOol00aS6Oomo
o`0<Ool00aS6Ooml001@Ool00`00Oomoo`1VOol:000?Ool002eoo`03001oogoo00Yoo`036<Ioogoo
00eoo`036<Ioogoo00]oo`03O01oogoo00]oo`036<Ioogoo00aoo`04001oogooOol401l00goo6<Io
o`0BOol00`00Oomoo`0WOol00g`0Oomoo`0=Ool00`00Oomoo`0gOol00`00Oomoo`0AOol00aS601l0
7`0201l3Ool00`00Oomoo`0<Ool00aS6Oomoo`0JOol00aS6Oomoo`0<Ool00aS6O01oo`0=Ool00`00
Oomoo`10Ool00`00Oomoo`1VOol:000?Ool002eoo`03001oogoo00Yoo`036<Ioogoo00eoo`036<Io
ogoo00Moo`05001oogooOoml000=Ool00aS6Oomoo`0<Ool00`00Oomoo`02Ool401l00aS6Oomoo`0A
Ool00`00Oomoo`0XOol00g`0Oomoo`0<Ool00`00Oomoo`1;Ool00aS601l07`0201l3Ool00`00Oomo
o`0<Ool00aS6Oomoo`0JOol00aS6Oomoo`0<Ool00aS6O01oo`0=Ool00`00Oomoo`10Ool00`00Oomo
o`1VOol:000?Ool003Yoo`036<Ioogoo00Moo`03001oogoo00=oo`036<Ioogoo00Moo`05001oogoo
Ooml000=Ool00aS6Oomoo`0<Ool00`00Oomoo`02Ool401l00aS6Oomoo`0AOol00`00Oomoo`0XOol0
0g`0Oomoo`0<Ool00`00Oomoo`1:Ool00aS601l07`0201l5Ool00`00Oomoo`0;Ool00aS6Oomoo`0?
Ool00`00Oomoo`08Ool00aS6Oomoo`03Ool00`00Oomoo`06Ool00aS6Oomoo`0=Ool00`00Oomoo`10
Ool00`00Oomoo`1VOol:000?Ool003Yoo`036<Ioogoo00Moo`03001oogoo00=oo`036<Ioogoo00Qo
o`03001oog`000ioo`036<Ioogoo00aoo`03001oogoo00=oo`<07`036<Ioogoo03eoo`03O01oogoo
00]oo`03001oogoo04Yoo`036<H07`0O00807`Eoo`03001oogoo00]oo`036<Ioogoo00moo`03001o
ogoo00Qoo`036<Ioogoo00=oo`03001oogoo00Ioo`036<Ioogoo00eoo`03001oogoo041oo`03001o
ogoo06Ioo`X000moo`00>Woo00<HaWooOol01goo00<007ooOol00goo00<HaWooOol027oo00<007oo
O0003Woo00<HaWooOol04Woo100O?goo00=l07ooOol02goo00<007ooOol0=Woo00<007ooOol04Goo
100O57oo00<HaWooOol03goo00<007ooOol027oo00<HaWooOol00goo00<007ooOol01Woo00<HaWoo
Ool0D7oo00<007ooOol0?Goo0P0027oo0P007Goo2P003goo000]Ool00`00Oomoo`0:Ool00aS6Oomo
o`07Ool00`00Oomoo`03Ool00aS6Oomoo`09Ool00g`0Oomoo`0=Ool00aS6Oomoo`0BOol401loOol0
0g`0Oomoo`0;Ool2000gOol00`00Oomoo`0AOol401lEOol00aS6Oomoo`0>Ool00`00Oomoo`08Ool0
0aS6Oomoo`03Ool00`00Oomoo`05Ool00g`06<Ioo`1AOol00`00Oomoo`0mOol00`00Oomoo`08Ool0
0`00Oomoo`0KOol:000?Ool002eoo`03001oogoo00Yoo`036<Ioogoo00eoo`036<Ioogoo00Uoo`03
O01oogoo00eoo`036<Ioogoo01=oo`@07cmoo`03O01oogoo00Yoo`03001oogoo03Ioo`03001oogoo
015oo`<07aIoo`036<Ioogoo01Uoo`036<Ioogoo00]oo`03O00HaWoo055oo`03001oogoo02Uoo`@0
00=oo`<0009oo`@000Aoo`03001oogoo00800003Ool0000000=oo`03001oogoo01]oo`X000moo`00
9Goo00<HaWooOol01Woo00<007ooOol02Goo00<HaWooOol03Goo00<HaWooOol027oo00=l07ooOol0
3Woo00<HaWooOol02goo00<007ooOol01Goo100O4goo00<007ooOol0:Goo00=l07ooOol02Woo00<0
07ooOol0=Woo00<007ooOol047oo100O1goo00<007ooOol037oo00<HaWooOol06Goo00<HaWooOol0
2Woo00Al07ooOolHaQ=oo`036<Ioogoo03]oo`03001oogoo009oob407`Eoo`03001oogoo00Aoo`05
001oogooOol00004Ool00`00Oomoo`02Ool01`00Oomoogoo001oo`0000Aoo`03001oogoo01]oo`X0
00moo`009Goo00<HaWooOol01Woo00<007ooOol02Goo00<HaWooOol03Goo00<HaWooOol027oo00=l
07ooOol03Woo00<HaWooOol02goo00<007ooOol01Woo100O4Woo00<007ooOol0:Woo00=l07ooOol0
2Goo00<007ooOol0BGoo100O1goo00<007ooOol037oo00<HaWooOol06Goo00<HaWooOol00goo00<0
07ooOol017oo00Al07ooOolHaPeoo`03001oogoo00=oo`036<Ioogoo03]oo`03001oogoo009oob40
7`Eoo`03001oogoo00Aoo`06001oogooOol007oo100017oo00<007ooOol00Woo00<007ooOol00goo
00<007ooOol06goo2P003goo000UOol00aS6Oomoo`0AOol00aS6Oomoo`07Ool00`00Oomoo`04Ool0
0aS6Oomoo`07Ool00g`0001oo`0?Ool00aS6Oomoo`0;Ool00`00Oomoo`06Ool401lBOol00`00Oomo
o`0ZOol00g`0Oomoo`09Ool00`00Oomoo`18Ool401l8Ool00`00Oomoo`0<Ool00aS6Oomoo`0=Ool0
0`00Oomoo`08Ool00aS6Oomoo`04Ool00`00Oomoo`03Ool01G`0Oomoogoo6<H03Goo00<007ooOol0
0goo00<HaWooOol0>goo00<007ooOol00Woo8@0O1Goo00<007ooOol017oo00L007ooOomoo`00Ool0
0007Ool01`00Oomoogoo001oo`0000Aoo`03001oogoo01]oo`X000moo`009Goo00<HaWooOol04Goo
00<HaWooOol01goo00<007ooOol017oo00<HaWooOol01goo00=l0000Ool03goo00<HaWooOol02goo
00<007ooOol01goo0`0O4Woo00<007ooOol0:goo00=l07ooOol027oo00<007ooOol0B7oo100O27oo
00<007ooOol037oo00<HaWooOol03Goo00<007ooOol027oo00<HaWooOol017oo00<007ooOol00goo
00El07ooOomooaS600eoo`03001oogoo00=oo`036<Ioogoo03]oo`03001oogoo02Qoo`05001oogoo
Ool00003Ool30002Ool40004Ool00`00Oomoo`0200000goo00000003Ool00`00Oomoo`0KOol:000?
Ool002Eoo`036<Ioogoo015oo`036<Ioogoo00Moo`03001oogoo00Aoo`036<Ioogoo00Moo`03O000
07oo00ioo`036<Ioogoo01Ioo`@07cmoo`03O01oogoo00Qoo`03001oogoo03Eoo`03001oogoo00mo
o`@07aQoo`036<Ioogoo00eoo`03001oogoo00Qoo`036<Ioogoo00Yoo`05O01oogooOolHaP0=Ool0
0`00Oomoo`03Ool00aS6Oomoo`0kOol00`00Oomoo`0YOol4000@Ool20008Ool2000MOol:000?Ool0
02Eoo`036<Ioogoo015oo`036<Ioogoo00Moo`03001oogoo00Aoo`036<Ioogoo00Ioo`03O01oo`00
00moo`036<Ioogoo01Ioo`@07cmoo`03O01oogoo00Qoo`03001oogoo03Eoo`03001oogoo00moo`@0
7aQoo`036<Ioogoo00eoo`03001oogoo00Qoo`036<Ioogoo00Uoo`03O01oogoo009oo`036<Ioogoo
015oo`036<Ioogoo03]oo`03001oogoo06Ioo`X000moo`009Goo00<HaWooOol01Woo00<007ooOol0
27oo00<HaWooOol03Woo00<HaWooOol01Woo00=l07ooOol03goo00<HaWooOol05goo100O?goo00=l
07ooOol01goo00<007ooOol0=7oo00<007ooOol047oo0`0O6Goo00<HaWooOol067oo00<HaWooOol0
2Goo00=l07ooOol00Woo00<HaWooOol04Goo00<HaWooOol0>goo00<007ooOol0IWoo2P003goo000U
Ool00aS6Oomoo`06Ool00`00Oomoo`08Ool00aS6Oomoo`0?Ool00aS6Oomoo`04Ool00g`0Oomoo`0@
Ool00aS6Oomoo`0GOol401lAOol00`00Oomoo`0[Ool00g`0Oomoo`07Ool00`00Oomoo`0dOol00`00
Oomoo`0?Ool401l;Ool00`00Oomoo`0;Ool00aS6Oomoo`0HOol00aS6Oomoo`08Ool00g`0Oomoo`03
Ool00aS6Oomoo`0AOol00aS6Oomoo`0kOol00`00Oomoo`1VOol:000?Ool002Eoo`036<Ioogoo00Io
o`03001oogoo00Qoo`036<Ioogoo00moo`036<Ioogoo00Aoo`03O01oogoo011oo`036<Ioogoo00]o
o`03001oogoo00Yoo`@07a1oo`03001oogoo02aoo`03O01oogoo00Ioo`03001oogoo04Ioo`@07`]o
o`03001oogoo00]oo`036<Ioogoo01Qoo`036<Ioogoo00Eoo`04001oogooO005Ool00aS6Oomoo`0;
Ool00`00Oomoo`03Ool00aS6Oomoo`0kOol00`00Oomoo`1VOol:000?Ool002Eoo`036<Ioogoo00Io
o`03001oogoo00Qoo`036<Ioogoo00moo`036<Ioogoo00=oo`05O01oogooOol0000?Ool00aS6Oomo
o`0;Ool00`00Oomoo`0:Ool401lAOol00`00Oomoo`0[Ool00g`0Oomoo`06Ool20016Ool401l00aS6
Oomoo`09Ool00`00Oomoo`0<Ool00aS6Oomoo`0GOol00aS6Oomoo`05Ool00`00Ooml0006Ool00aS6
Oomoo`0;Ool00`00Oomoo`03Ool00aS6Oomoo`0kOol00`00Oomoo`1VOol:000?Ool002Eoo`036<Io
ogoo015oo`036<Ioogoo00Moo`03001oogoo00Eoo`036<Ioogoo00=oo`05O01oogooOol0000?Ool0
0aS6Oomoo`0;Ool00`00Oomoo`0:Ool011S601l07`0O4Goo00<007ooOol0;7oo00=l07ooOol01Goo
00<007ooOol0AGoo100O00<HaWooOol02Goo00<007ooOol037oo00<HaWooOol037oo00<007ooOol0
27oo00<HaWooOol01Goo00<007ooO0001Woo00<HaWooOol02Woo00<007ooOol017oo00<HaWooOol0
>goo00<007ooOol0IWoo2P003goo000VOol00aS6Oomoo`0@Ool00aS6Oomoo`07Ool00`00Oomoo`05
Ool00aS6Oomoo`03Ool00g`0Oomoo`02Ool00`00Oomoo`0<Ool00aS6Oomoo`0;Ool00`00Oomoo`0:
Ool00aS601l07`0201loOol00g`0Oomoo`05Ool00`00Oomoo`14Ool401l00goo6<Ioo`0IOol00aS6
Oomoo`0<Ool00`00Oomoo`08Ool00aS6Oomoo`05Ool00`00Ooml0006Ool00aS6Oomoo`0:Ool00`00
Oomoo`04Ool00aS6Oomoo`0kOol00`00Oomoo`1VOol:000?Ool002Ioo`036<Ioogoo011oo`036<Io
ogoo00Moo`03001oogoo00Eoo`036<Ioogoo009oo`03O01oogoo00=oo`03001oogoo00aoo`036<Io
ogoo01Qoo`036<H07`0O00807cmoo`03O01oogoo00Eoo`03001oogoo03Aoo`03001oogoo00eoo`@0
7`036<Ioogoo01Uoo`036<Ioogoo00aoo`03001oogoo00Qoo`036<Ioogoo00Ioo`03O01oogoo00Eo
o`036<Ioogoo015oo`036<Ioogoo03]oo`03001oogoo06Ioo`X000moo`009Woo00<HaWooOol01Goo
00<007ooOol027oo00<HaWooOol01goo00<007ooOol01Goo00<HaWooOol00Woo00=l07ooOol04Woo
00<HaWooOol067oo00<HaWoo01l00`0O?goo00=l07ooOol017oo00<007ooOol0=7oo00<007ooOol0
3Goo0`0O00=ooaS6Ool06Woo00<HaWooOol037oo00<007ooOol027oo00<HaWooOol01Woo00=l07oo
Ool01Goo00<HaWooOol04Goo00<HaWooOol0>goo00<007ooOol0IWoo2P003goo000VOol00aS6Oomo
o`05Ool00`00Oomoo`08Ool00aS6Oomoo`0?Ool01AS6OomoogooO00057oo00<HaWooOol06Goo00<H
aWoo01l00`0O?goo00=l07ooOol017oo00<007ooOol0<goo00<007ooOol03Goo100O00=ooaS6Ool0
6Woo00<HaWooOol05goo00<HaWooOol01Goo00=l07ooOol01Woo00<HaWooOol047oo00<HaWooOol0
?7oo00<007ooOol0IWoo2P003goo000VOol00aS6Oomoo`05Ool00`00Oomoo`08Ool00aS6Oomoo`0?
Ool01AS6OomoogooO00057oo00<HaWooOol037oo00<007ooOol02Woo00<HaWooOol0100O3Woo00<0
07ooOol0;Woo00=l07ooOol00goo00<007ooOol0<goo00<007ooOol03Goo100O00=ooaS6Ool037oo
00<007ooOol02goo00<HaWooOol05goo00<HaWooOol01Goo00<007ooOol01Woo00<HaWooOol047oo
00<HaWooOol0?7oo00<007ooOol0IWoo2P003goo000VOol00aS6Oomoo`05Ool00`00Oomoo`08Ool0
0aS6Oomoo`0?Ool011S6Oomoog`05Goo00<HaWooOol037oo00<007ooOol02goo00<HaWoo01l00`0O
3Woo00<007ooOol0;Woo00=l07ooOol00goo00<007ooOol0@Woo100O0Woo00<HaWooOol02goo00<0
07ooOol02goo00<HaWooOol05Woo00<HaWooOol01Goo00=l0000Ool01goo00<HaWooOol02Woo00<0
07ooOol00goo00<HaWooOol0?7oo00<007ooOol0IWoo2P003goo000VOol00aS6Oomoo`0@Ool00aS6
Oomoo`06Ool00`00Oomoo`06Ool011S6Oomoog`01goo00<007ooOol02goo00<HaWooOol02goo00<0
07ooOol037oo00<HaWooOol00`0O3goo00<007ooOol0;Woo00=l07ooOol00Woo00<007ooOol0@Woo
100O0Woo00<HaWooOol02goo00<007ooOol02goo00<HaWooOol02goo00<007ooOol027oo00<HaWoo
Ool01Goo00=l0000Ool01goo00<HaWooOol02Woo00<007ooOol00goo00<HaWooOol0?7oo00<007oo
Ool0IWoo2P003goo000VOol011S6000000003goo00<HaWooOol01Woo00<007ooOol01Woo00@HaWoo
Ooml00Moo`03001oogoo00]oo`036<Ioogoo00]oo`03001oogoo00Aoo`@000Aoo`036<Ioogoo00@0
7`ioo`03001oogoo02ioo`03O01oogoo009oo`03001oogoo045oo`@07`=oo`036<Ioogoo00=oo`@0
00Aoo`03001oogoo00]oo`036<Ioogoo00]oo`03001oogoo00Qoo`036<Ioogoo00Eoo`03O00007oo
00Qoo`036<Ioogoo00Uoo`03001oogoo00=oo`036<H0000003aoo`03001oogoo06Ioo`X000moo`00
9Woo00<HaWoo000047oo00<HaWooOol01Woo00<007ooOol01Woo00<HaWooO00027oo00<007ooOol0
2goo00<HaWooOol04goo00<007ooOol017oo00<HaWooOol0100O?goo00=l07ooOol00Woo0P00<goo
00<007ooOol037oo100O0goo00<HaWooOol017oo00<007ooOol04Woo00<HaWooOol02goo00<007oo
Ool027oo00<HaWooOol017oo00=l07ooOol02Goo00<HaWooOol02Goo00<007ooOol00goo00<HaP00
Ool0?7oo00<007ooOol0IWoo2P003goo000POol500000goo6<H00002000?Ool00aS6Oomoo`06Ool0
0`00Oomoo`07Ool00aS6O01oo`07Ool00`00Oomoo`0;Ool00aS6Oomoo`0<Ool50003Ool00`00Oomo
o`03Ool011S6Oomoogoo100O?goo00El07ooOomoo`0003Aoo`03001oogoo00aoo`<07`=oo`036<Io
ogoo00Ioo`03001oogoo015oo`036<Ioogoo00]oo`03001oogoo00Qoo`036<Ioogoo00Aoo`03O01o
ogoo00Uoo`036<Ioogoo00ioo`04000HaP00000lOol00`00Oomoo`1VOol:000?Ool002Ioo`036<Io
o`0000Ioo`03001oogoo00Moo`036<Ioogoo011oo`036<Ioogoo01Eoo`036<Ioogoo01Eoo`03001o
ogoo009oo`046<IoogooOol401loOol01G`0Oomoogoo0000=7oo00<007ooOol02goo100O0goo00<H
aWooOol01goo00<007ooOol04Goo00<HaWooOol05Goo00<HaWooOol00goo00=l07ooOol02Woo00<H
aWooOol03Woo00<001S60000?Goo00<007ooOol0IWoo2P003goo000VOol00aS6Ool00006Ool00`00
Oomoo`07Ool00aS6Oomoo`0@Ool00aS6Oomoo`0EOol00aS6Oomoo`0BOol01000Oomoo`0017oo00<H
aWooOol00Woo0`0O3Goo00<007ooOol0<7oo00Al07ooOol003Aoo`03001oogoo00]oo`@07`=oo`03
6<Ioogoo00Aoo`04001oogoo0004Ool00`00Oomoo`0<Ool00aS6Oomoo`0EOol00aS6Oomoo`03Ool0
0g`0Oomoo`0:Ool00aS6Oomoo`0>Ool00`006<H0000mOol00`00Oomoo`1VOol:000?Ool002Ioo`03
6<H0000000Ioo`03001oogoo00Ioo`036<Ioogoo011oo`03O00HaWoo01Ioo`036<Ioogoo00]oo`03
001oogoo00Eoo`8000Eoo`036<Ioogoo009oo`@07`aoo`03001oogoo031oo`04O01oogoo0011Ool4
01l4Ool00aS6Oomoo`05Ool20005Ool00`00Oomoo`0<Ool00aS6Oomoo`0EOol00aS6Oomoo`03Ool0
17`0Oomoo`002Goo00<HaWooOol027oo00<007ooOol017oo00<HaP00Ool0?7oo00<007ooOol0IWoo
2P003goo000VOol00aS6Oomoo`06Ool00`00Oomoo`06Ool00aS6Oomoo`0@Ool00g`06<Ioo`09Ool0
0`00Oomoo`0:Ool00aS6Oomoo`0;Ool00`00Oomoo`0<Ool00aS6Oomoo`02Ool401l=Ool00`00Oomo
o`0`Ool00g`0Ool00011Ool401l4Ool00aS6Oomoo`0=Ool00`00Oomoo`0;Ool00aS6Oomoo`0EOol0
0aS6Oomoo`02Ool01G`0Oomoogoo00002Goo00<HaWooOol027oo00<007ooOol017oo00<HaWooOol0
?7oo00<007ooOol0IWoo2P003goo000VOol00aS6Oomoo`0?Ool00aS6Oomoo`07Ool00`00Oomoo`06
Ool00g`06<Ioo`09Ool00`00Oomoo`09Ool00aS6Oomoo`0<Ool00`00Oomoo`0=Ool00aS6Oomoo`02
Ool401l<Ool00`00Oomoo`0`Ool00g`0Ool00011Ool301l5Ool00aS6Oomoo`0=Ool00`00Oomoo`0;
Ool00aS6Oomoo`0:Ool00`00Oomoo`08Ool00aS6Oomoo`02Ool00g`0Oomoo`02Ool00`00Oomoo`06
Ool00aS6Oomoo`08Ool00`00Oomoo`04Ool00aS6Oomoo`0lOol00`00Oomoo`1VOol:000?Ool002Io
o`036<Ioogoo00moo`036<Ioogoo00Moo`03001oogoo00Eoo`03O01ooaS600Yoo`03001oogoo00Uo
o`036<Ioogoo00aoo`03001oogoo00eoo`036<Ioogoo009oo`@07cmoo`03O01oo`00041oo`@07`Eo
o`036<Ioogoo01]oo`036<Ioogoo00Yoo`03001oogoo00Qoo`056<IoogooOoml0005Ool00`00Oomo
o`06Ool00aS6Oomoo`08Ool00`00Oomoo`04Ool00aS6Oomoo`0lOol00`00Oomoo`1VOol:000?Ool0
02Ioo`036<Ioogoo00moo`036<Ioogoo00Moo`03001oogoo00Eoo`03O01ooaS600Yoo`03001oogoo
00Uoo`036<Ioogoo01aoo`036<Ioogoo00=oo`@07cmoo`03O00007oo039oo`03001oogoo00Yoo`@0
7`Eoo`036<Ioogoo01]oo`036<Ioogoo00Yoo`03001oogoo00Qoo`056<IoogooOoml000>Ool00aS6
Oomoo`0?Ool00aS6Oomoo`0lOol00`00Oomoo`1VOol:000?Ool002Ioo`036<Ioogoo00Ioo`03001o
ogoo00Ioo`036<Ioogoo00Moo`03001oogoo00Aoo`04O01oogoo6<HFOol00aS6Oomoo`0LOol00aS6
Oomoo`03Ool401loOol00g`0001oo`0bOol00`00Oomoo`09Ool401l5Ool00aS6Oomoo`0LOol00aS6
Oomoo`0:Ool00`00Oomoo`08Ool01AS6OomoogooO0003Woo00<HaWooOol03goo00<HaWooOol0?7oo
00<007ooOol0IWoo2P003goo000MOol900000aS60000000?00000aS60000000>000017`0000001S6
5P0000<HaP000000700000<HaP00000010000`0O?P001AS6?@00100O1@0000<HaP000000700000<H
aP000000500000DHaP00000007`000l000036<H0000000l000036<H0000000P003Aoo`03001oogoo
06Ioo`X000moo`009Goo00<001S6Ool01goo00<007ooOol01Woo00<HaWoo00003Woo00Al07ooOolH
aP9oo`03001oogoo015oo`036<H007oo00]oo`03001oogoo00Aoo`03001oogoo00Moo`036<Ioogoo
00Aoo`@07`03001oogoo00Qoo`03001oogoo00Moo`03001oogoo019oo`03001oogoo00eoo`<HaP9o
o`03O01ooaS6008HaQ1oo`03001oogoo015oo`03001oogoo00Qoo`03001oogoo00Moo`0300007`0O
00807`Ioo`036<Ioogoo00Moo`03001oogoo00Aoo`03001oogoo00]oo`036<Ioogoo019oo`03001o
oaS600=oo`03O01oogoo00Aoo`03001oogoo00Eoo`03000HaWoo00Uoo`03001oogoo00Aoo`036<H0
07oo03aoo`03001oogoo06Ioo`X000moo`009Goo00<007oo6<H01goo00<007ooOol01Woo00<HaWoo
Ool03Goo00El07ooOomooaS601Ioo`036<Ioogoo00]oo`03001oogoo00Aoo`03001oogoo00Moo`03
6<Ioogoo00Aoo`@07`]oo`03001oogoo02]oo`036<Ioogoo00=oo`03O01oogoo009oo`8HaSQoo`@0
7`Ioo`036<Ioogoo00Moo`03001oogoo00Aoo`03001oogoo00]oo`036<Ioogoo01Aoo`046<Ioogoo
O007Ool00`00Oomoo`06Ool00aS6Oomoo`08Ool00`00Oomoo`04Ool00aS6001oo`0lOol00`00Oomo
o`1VOol:000?Ool002Moo`036<Ioogoo00ioo`036<Ioogoo00Ioo`03001oogoo00Aoo`05O01oogoo
OolHaP0:Ool00`00Oomoo`09Ool00aS6Oomoo`0;Ool00`00Oomoo`0>Ool00aS6Oomoo`05Ool401l:
Ool00`00Oomoo`0ZOol00aS6Oomoo`04Ool00`00O01oo`04Ool00aS6Oomoo`0eOol301l7Ool00aS6
Oomoo`0>Ool00`00Oomoo`0;Ool00aS6Oomoo`09Ool00`00Oomoo`08Ool011S6Oomoog`01goo00<0
07ooOol01Woo00<HaWooOol01goo00<007ooOol01Goo00<HaWooOol0?7oo00<007ooOol0IWoo2P00
3goo000WOol00aS6Oomoo`0>Ool00aS6Oomoo`06Ool00`00Oomoo`03Ool00g`0Oomoo`02Ool00aS6
Oomoo`08Ool00`00Oomoo`09Ool00aS6Oomoo`0;Ool00`00Oomoo`0>Ool00aS6Oomoo`05Ool401lf
Ool00aS6Oomoo`05Ool00`00O01oo`05Ool00aS6Oomoo`0cOol401l7Ool00aS6Oomoo`0>Ool00`00
Oomoo`0<Ool00aS6Oomoo`08Ool00`00Oomoo`08Ool00aS6Ooml0008Ool00`00Oomoo`06Ool00aS6
Oomoo`07Ool00`00Oomoo`05Ool00aS6Oomoo`0lOol00`00Oomoo`0mOol2000=Ool2000HOol:000?
Ool002Moo`036<Ioogoo00ioo`036<Ioogoo00Ioo`03001oogoo00=oo`03O01oogoo009oo`036<Io
ogoo00Qoo`03001oogoo00Uoo`036<Ioogoo01eoo`036<Ioogoo00Eoo`@07cAoo`036<Ioogoo00Io
o`03001oog`000Eoo`036<Ioogoo02Qoo`03001oogoo00Qoo`@07`Moo`036<Ioogoo01eoo`036<Io
ogoo00Qoo`03001oogoo00Qoo`036<Ioog`0015oo`036<Ioogoo00moo`036<Ioogoo03aoo`03001o
ogoo03eoo`03001oogoo00eoo`03001oogoo01Ioo`X000moo`009goo00<HaWooOol03Woo00<HaWoo
Ool01Woo00<007ooOol00goo00=l07ooOol00goo00<HaWooOol01goo00<007ooOol02Goo00<HaWoo
Ool07Goo00<HaWooOol01Goo100O<goo00<HaWooOol01goo00<007ooO0001Woo00<HaWooOol09goo
00<007ooOol01goo100O27oo00<HaWooOol07Goo00<HaWooOol027oo00<007ooOol027oo00<HaWoo
O0004Goo00<HaWooOol03goo00<HaWooOol0?7oo00<007ooOol0:Goo0`000goo1@0000Aoo`00001o
o`8000=oo`03001oogoo00800003Ool0000000Qoo`03001oogoo01Ioo`X000moo`009goo00<HaWoo
Ool01Goo00<007ooOol01Woo00<HaWooOol02goo00=l07ooOol017oo00<HaWooOol04Woo00<HaWoo
Ool07Woo00<HaWooOol01Woo0`0O<Woo00<HaWooOol027oo00@007ooOoml00Ioo`036<Ioogoo02Io
o`03001oogoo00Moo`@07`Qoo`036<Ioogoo01eoo`036<Ioogoo01=oo`036<Il07oo015oo`036<Io
ogoo00moo`036<Ioogoo03aoo`03001oogoo02Uoo`04001oogoo0004Ool01@00Oomoogoo00000Woo
00<007ooOol00Woo00L007ooOomoo`00Ool00009Ool00`00Oomoo`0FOol:000?Ool002Moo`036<Io
ogoo00Eoo`03001oogoo00Ioo`036<Ioogoo00]oo`03O01oogoo00Aoo`036<Ioogoo019oo`036<Io
ogoo01ioo`036<Ioogoo00Ioo`@07`Uoo`03001oogoo02Eoo`036<Ioogoo00Qoo`04001oogooO007
Ool00aS6Oomoo`0UOol00`00Oomoo`07Ool301l8Ool00aS6Oomoo`0@Ool00`00Oomoo`0;Ool00aS6
Oomoo`0COol00aS6O01oo`0BOol00aS6Oomoo`0>Ool00aS6Oomoo`0lOol00`00Oomoo`0[Ool20004
Ool01@00Oomoogoo00000Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol027oo00<007oo
Ool05Woo2P003goo000WOol00aS6Oomoo`06Ool00`00Oomoo`05Ool00aS6Oomoo`0:Ool00g`0Oomo
o`05Ool00aS6Oomoo`0BOol00aS6Oomoo`0;Ool00`00Oomoo`0@Ool00aS6Oomoo`06Ool401l9Ool0
0`00Oomoo`0TOol00aS6Oomoo`09Ool01000Oomoog`027oo00<HaWooOol0;Goo100O27oo00<HaWoo
Ool047oo00<007ooOol02goo00<HaWooOol04goo00<HaWooOol027oo00<007ooOol01goo00<HaWoo
Ool01Woo00<007ooOol01Goo00<HaWooOol0?7oo00<007ooOol00goo7aS61goo0P001Woo00D007oo
Oomoo`00009oo`03001oogoo009oo`07001oogooOol007oo00002Goo00<007ooOol05Woo2P003goo
000WOol00aS6Oomoo`06Ool00`00Oomoo`05Ool00aS6Oomoo`0:Ool00g`0Oomoo`05Ool00aS6Oomo
o`08Ool00`00Oomoo`07Ool00aS6Oomoo`0;Ool00`00Oomoo`0@Ool00aS6Oomoo`07Ool401l8Ool0
0`00Oomoo`0SOol00aS6Oomoo`0:Ool20002Ool00g`0Oomoo`05Ool00aS6Oomoo`0]Ool401l8Ool0
0aS6Oomoo`0@Ool00`00Oomoo`0;Ool00aS6Oomoo`08Ool00`00Oomoo`08Ool00aS6Oomoo`08Ool0
0`00Oomoo`07Ool00aS6Oomoo`06Ool00`00Oomoo`05Ool00aS6Oomoo`0lOol00`00Oomoo`0YOol0
1000Oomoo`000goo0P000Woo1@0017oo00<007ooOol00P0000=oo`0000000Woo0`000goo00<007oo
Ool05Woo2P003goo000WOol00aS6Oomoo`0>Ool00aS6Oomoo`06Ool01000Oomoog`027oo00<HaWoo
Ool027oo00<007ooOol01goo00<HaWooOol02goo00<007ooOol047oo00<HaWooOol01goo100O27oo
00<007ooOol08goo00<HaWooOol02Woo00D007ooOomoog`000Qoo`036<Ioogoo02]oo`@07`Uoo`03
6<Ioogoo011oo`03001oogoo00]oo`036<Ioogoo00Qoo`03001oogoo00Qoo`036<Ioogoo00Uoo`03
001oogoo00Ioo`036<Ioogoo00Ioo`03001oogoo00Aoo`036<Ioogoo03eoo`03001oogoo02Yoo`<0
011oo`8000Uoo`04001oogooOol2000HOol:000?Ool002Moo`036<Ioogoo00ioo`036<Ioogoo00Io
o`04001oogooO008Ool00aS6Oomoo`08Ool00`00Oomoo`07Ool00aS6Oomoo`0NOol00aS6Oomoo`08
Ool401l/Ool00aS6Oomoo`0;Ool00`00Oomoo`02Ool00g`0Oomoo`06Ool00aS6Oomoo`0QOol00`00
Oomoo`06Ool401l9Ool00aS6Oomoo`0NOol00aS6Oomoo`07Ool00`00Oomoo`08Ool00g`06<Ioo`0:
Ool00`00Oomoo`06Ool00aS6Oomoo`06Ool00`00Oomoo`04Ool00aS6Oomoo`0mOol00`00Oomoo`0a
Ool00`00Oomoo`0EOol00`00Oomoo`0JOol:000?Ool002Moo`036<Ioogoo00ioo`036<Ioogoo00Io
o`04001oogooO008Ool00aS6Oomoo`08Ool00`00Oomoo`07Ool00aS6Oomoo`0NOol00aS6Oomoo`08
Ool401l[Ool00aS6Oomoo`0<Ool00`00Oomoo`02Ool00g`0Oomoo`06Ool00aS6Oomoo`0QOol00`00
Oomoo`06Ool301l:Ool00aS6Oomoo`0NOol00aS6Oomoo`07Ool00`00Oomoo`08Ool00g`06<Ioo`0C
Ool00aS6Oomoo`0=Ool00aS6Oomoo`0mOol00`00Oomoo`17Ool3000LOol:000?Ool002Moo`036<Io
ogoo00Ioo`03001oogoo00Eoo`036<Ioogoo00Ioo`03001oog`000Uoo`036<Ioogoo019oo`036<Io
ogoo01moo`036<Ioogoo00Qoo`<07b]oo`036<Ioogoo00aoo`03001oogoo009oo`03O01oogoo00Mo
o`036<Ioogoo01moo`03001oogoo00Ioo`@07`Yoo`036<Ioogoo01ioo`036<Ioogoo015oo`03O01o
oaS601Aoo`036<Ioogoo00eoo`036<Ioogoo03eoo`03001oogoo06Ioo`X000moo`009goo00<HaWoo
Ool01Woo00<007ooOol01Goo00<HaWooOol027oo00=l07ooOol01goo00<HaWooOol04Woo00<HaWoo
Ool07goo00<HaWooOol027oo100O1goo00<007ooOol07goo00<HaWooOol03Goo00<007ooOol00goo
00=l07ooOol01goo00<HaWooOol07Woo00<007ooOol01Woo100O2Woo00<HaWooOol07Woo00<HaWoo
Ool04Goo00=l01S6Ool057oo00<HaWooOol03Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo
000WOol00aS6Oomoo`06Ool00`00Oomoo`04Ool00aS6Oomoo`08Ool00g`0Oomoo`08Ool00aS6Oomo
o`0BOol00aS6Oomoo`0;Ool00`00Oomoo`0AOol00aS6Oomoo`08Ool401l7Ool00`00Oomoo`0NOol0
0aS6Oomoo`0>Ool00`00Oomoo`03Ool00g`0Oomoo`07Ool00aS6Oomoo`0VOol401l:Ool00aS6Oomo
o`0BOol00`00Oomoo`0;Ool00aS6Oomoo`0?Ool00g`0OolHaP0<Ool00`00Oomoo`06Ool00aS6Oomo
o`05Ool00`00Oomoo`05Ool00aS6Oomoo`0mOol00`00Oomoo`1VOol:000?Ool002Moo`036<Ioogoo
00Ioo`03001oogoo00Aoo`036<Ioogoo00Qoo`03O01oogoo00Qoo`036<Ioogoo019oo`036<Ioogoo
00]oo`03001oogoo015oo`036<Ioogoo00Uoo`@07`Moo`03001oogoo01eoo`036<Ioogoo00ioo`03
001oogoo00Aoo`03O01oogoo00Moo`036<Ioogoo02Eoo`@07`Yoo`036<Ioogoo019oo`03001oogoo
00]oo`036<Ioogoo00moo`03O01ooaS600aoo`03001oogoo00Ioo`036<Ioogoo00Eoo`03001oogoo
00Eoo`036<Ioogoo03eoo`03001oogoo06Ioo`X000moo`009goo00<HaWooOol03Goo00<HaWooOol0
1Woo00<007ooO0002Woo00<HaWooOol027oo00<007ooOol01goo00<HaWooOol02Woo00<007ooOol0
4Woo00<HaWooOol02Goo100O1goo00<007ooOol077oo00<HaWooOol03goo00<007ooOol017oo00=l
07ooOol01goo00<HaWooOol097oo100O2goo00<HaWooOol04Woo00<007ooOol02goo00<HaWooOol0
1Woo00<007ooOol01Woo00=l07oo6<H037oo00<007ooOol01Woo00<HaWooOol01Goo00<007ooOol0
1Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo000WOol00aS6Oomoo`0=Ool00aS6Oomoo`06
Ool00`00O01oo`0:Ool00aS6Oomoo`08Ool00`00Oomoo`06Ool00aS6Oomoo`0;Ool00`00Oomoo`0B
Ool00aS6Oomoo`0:Ool401lUOol00aS6Oomoo`0?Ool00`00Oomoo`04Ool00g`0Oomoo`08Ool00aS6
Oomoo`0SOol401l;Ool00aS6Oomoo`0BOol00`00Oomoo`0;Ool00aS6Oomoo`06Ool00`00Oomoo`05
Ool017`0OomooaS637oo00<007ooOol01Woo00<HaWooOol01Goo00<007ooOol01Goo00<HaWooOol0
?Goo00<007ooOol0IWoo2P003goo000WOol00aS6Oomoo`0=Ool00aS6Oomoo`06Ool00`00O01oo`0:
Ool00aS6Oomoo`09Ool00`00Oomoo`05Ool00aS6Oomoo`0POol00aS6Oomoo`0:Ool401lTOol00aS6
Oomoo`0@Ool00`00Oomoo`05Ool00g`0Oomoo`07Ool00aS6Oomoo`0LOol00`00Oomoo`04Ool301l<
Ool00aS6Oomoo`0POol00aS6Oomoo`06Ool00`00Oomoo`05Ool017`0OomooaS65Goo00<HaWooOol0
3Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo000WOol00aS6Oomoo`0=Ool00aS6Oomoo`06
Ool00`00Oomoo`0:Ool00aS6Oomoo`09Ool00`00Oomoo`05Ool00aS6Oomoo`0POol00aS6Oomoo`0;
Ool301lTOol00aS6Oomoo`0@Ool20006Ool00g`0Oomoo`07Ool00aS6Oomoo`0LOol00`00Oomoo`03
Ool401l<Ool00aS6Oomoo`0POol00aS6Oomoo`06Ool00`00Oomoo`04Ool01G`0Oomoogoo6<H05Goo
00<HaWooOol03Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo000XOol00aS6Oomoo`05Ool0
0`00Oomoo`04Ool00aS6Oomoo`06Ool00g`0Oomoo`0:Ool00aS6Oomoo`0AOol00aS6Oomoo`0POol0
0aS6Oomoo`0;Ool401lSOol00aS6Oomoo`0@Ool00`00Oomoo`06Ool00g`0Oomoo`07Ool00aS6Oomo
o`0JOol00`00Oomoo`04Ool401l<Ool00aS6Oomoo`0POol00aS6Oomoo`0=Ool01G`0Oomoogoo6<H0
5Goo00<HaWooOol03Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo000XOol00aS6Oomoo`05
Ool00`00Oomoo`04Ool00aS6Oomoo`06Ool00g`0Oomoo`0;Ool00aS6Oomoo`0@Ool00aS6Oomoo`0;
Ool00`00Oomoo`0COol00aS6Oomoo`0:Ool401l5Ool00`00Oomoo`0JOol00aS6Oomoo`0AOol00`00
Oomoo`06Ool00g`0Oomoo`07Ool00aS6Oomoo`0JOol00`00Oomoo`03Ool401l=Ool00aS6Oomoo`0B
Ool00`00Oomoo`0;Ool00aS6Oomoo`0<Ool00g`0Oomoo`02Ool00aS6Oomoo`0COol00aS6Oomoo`0=
Ool00aS6Oomoo`0mOol00`00Oomoo`1VOol:000?Ool002Qoo`036<Ioogoo00Ioo`03001oogoo00=o
o`036<Ioogoo00Eoo`03O01oogoo00aoo`036<Ioogoo011oo`036<Ioogoo00]oo`03001oogoo01=o
o`036<Ioogoo00]oo`@07`Aoo`03001oogoo01Yoo`036<Ioogoo015oo`03001oogoo00Moo`03O01o
ogoo00Moo`036<Ioogoo01moo`@07`eoo`036<Ioogoo019oo`03001oogoo00]oo`036<Ioogoo00ao
o`03O01oogoo009oo`036<Ioogoo00Yoo`03001oogoo00Moo`036<Ioogoo00Aoo`03001oogoo00Eo
o`036<Ioogoo03eoo`03001oogoo06Ioo`X000moo`00:7oo00<HaWooOol01Woo00<007ooOol00goo
00<HaWooOol01Goo00=l07ooOol037oo00<HaWooOol027oo00<007ooOol01Goo00<HaWooOol02goo
00<007ooOol04goo00<HaWooOol02goo100O1Goo00<007ooOol067oo00<HaWooOol04Woo00<007oo
Ool01goo00=l07ooOol01goo00<HaWooOol07goo0`0O3Goo00<HaWooOol057oo00<007ooOol02Woo
00<HaWooOol01Woo00<007ooOol00Woo00=l07ooOol00goo00<HaWooOol02Woo00<007ooOol01goo
00<HaWooOol017oo00<007ooOol01Goo00<HaWooOol0?Goo00<007ooOol0IWoo2P003goo000XOol0
0aS6Oomoo`0<Ool00aS6Oomoo`04Ool00g`0Ool0000=Ool00aS6Oomoo`08Ool00`00Oomoo`05Ool0
0aS6Oomoo`0;Ool00`00Oomoo`0COol00aS6Oomoo`0<Ool301l5Ool00`00Oomoo`0HOol00aS6Oomo
o`0BOol00`00Oomoo`07Ool00g`0Oomoo`08Ool00aS6Oomoo`0MOol401l=Ool00aS6Oomoo`0DOol0
0`00Oomoo`0:Ool00aS6Oomoo`06Ool00`00Oomoo`02Ool00g`0Oomoo`03Ool00aS6Oomoo`0;Ool0
0`00Oomoo`06Ool00aS6Oomoo`03Ool00`00Oomoo`06Ool00aS6Oomoo`0mOol00`00Oomoo`1VOol:
000?Ool002Qoo`036<Ioogoo00aoo`036<Ioogoo00Aoo`03O01oo`0000eoo`036<Ioogoo00Qoo`03
001oogoo00Eoo`036<Ioogoo025oo`036<Ioogoo00aoo`@07amoo`036<Ioogoo019oo`03001oogoo
00Qoo`03O01oogoo00Moo`036<Ioogoo01Moo`03001oogoo00=oo`@07`eoo`036<Ioogoo025oo`03
6<Ioogoo00Eoo`03001oogoo00=oo`03O01oogoo00=oo`036<Ioogoo00]oo`03001oogoo00Ioo`03
6<Ioogoo00=oo`03001oogoo00Ioo`036<Ioogoo03eoo`03001oogoo06Ioo`X000moo`00:7oo00<H
aWooOol037oo00<HaWooOol00goo00Al07ooOol000eoo`036<Ioogoo00Qoo`03001oogoo00Eoo`03
6<Ioogoo025oo`036<Ioogoo00aoo`@07aioo`036<Ioogoo01=oo`03001oogoo00Qoo`03O01oogoo
00Moo`036<Ioogoo01Moo`03001oogoo009oo`@07`ioo`036<Ioogoo029oo`036<Ioogoo00Aoo`03
001oogoo009oo`03O01oogoo00Aoo`036<Ioogoo01Aoo`036<Ioogoo00aoo`036<Ioogoo03eoo`03
001oogoo06Ioo`X000moo`00:7oo00<HaWooOol01Woo00<007ooOol00goo00<HaWooOol00goo00Al
07ooOol000eoo`036<Ioogoo011oo`036<Ioogoo025oo`036<Ioogoo00eoo`@07aeoo`036<Ioogoo
01=oo`03001oogoo00Uoo`03O01oogoo00Moo`036<Ioogoo01Ioo`03001oogoo009oo`@07`ioo`03
6<Ioogoo029oo`036<Ioogoo00Uoo`03O01oogoo00Aoo`036<Ioogoo01Aoo`036<Ioogoo00aoo`03
6<Ioogoo03eoo`03001oogoo06Ioo`X000moo`00:7oo00<HaWooOol01Woo00<007ooOol00goo00<H
aWooOol00goo00=l07ooOol03Woo00<HaWooOol03goo00<HaWooOol08Woo00<HaWooOol03Goo100O
17oo00<007ooOol05Goo00<HaWooOol057oo00<007ooOol02Goo00=l07ooOol01goo00<HaWooOol0
5Woo00@007ooOomoo`@07`moo`036<Ioogoo029oo`036<Ioogoo00Qoo`03O01oogoo00Eoo`036<Io
ogoo01Aoo`036<Ioogoo00aoo`036<Ioogoo03eoo`03001oogoo06Ioo`X000moo`00:7oo00<HaWoo
Ool01Woo00<007ooOol00goo00<HaWooOol00Woo00=l07ooOol03goo00<HaWooOol03goo00<HaWoo
Ool02goo00<007ooOol057oo00<HaWooOol03Woo100O0goo00<007ooOol05Goo00<HaWooOol057oo
0P002Woo00=l07ooOol027oo00<HaWooOol06Goo100O3goo00<HaWooOol057oo00<007ooOol02goo
00<HaWooOol027oo00=l07ooOol01Goo00<HaWooOol02goo00<007ooOol01Woo00<HaWooOol00goo
00<007ooOol01Goo00<HaWooOol0?Woo00<007ooOol0IWoo2P003goo000XOol00aS6Oomoo`06Ool0
0`00Oomoo`03Ool00aS6Oomoo`02Ool00g`0Oomoo`0?Ool00aS6Oomoo`0?Ool00aS6Oomoo`0;Ool0
0`00Oomoo`0EOol00aS6Oomoo`0=Ool401l3Ool00`00Oomoo`0EOol00aS6Oomoo`0DOol00`00Oomo
o`0:Ool00g`0Oomoo`07Ool00aS6Oomoo`0IOol301l@Ool00aS6Oomoo`0DOol00`00Oomoo`0;Ool0
0aS6Oomoo`07Ool00g`0Oomoo`05Ool00aS6Oomoo`0<Ool00`00Oomoo`06Ool00aS6Oomoo`03Ool0
0`00Oomoo`05Ool00aS6Oomoo`0nOol00`00Oomoo`1VOol:000?Ool002Qoo`036<Ioogoo00aoo`05
6<IoogooOoml0003Ool00`00Oomoo`0<Ool00aS6Oomoo`08Ool00`00Oomoo`04Ool00aS6Oomoo`0;
Ool00`00Oomoo`0EOol00aS6Oomoo`0>Ool301l3Ool00`00Oomoo`0DOol00aS6Oomoo`0EOol00`00
Oomoo`0:Ool00g`0Oomoo`08Ool00aS6Oomoo`0GOol401l@Ool00aS6Oomoo`0DOol00`00Oomoo`0;
Ool00aS6Oomoo`04Ool01000Oomoog`01goo00<HaWooOol037oo00<007ooOol01Woo00<HaWooOol0
0goo00<007ooOol01Goo00<HaWooOol0?Woo00<007ooOol0IWoo2P003goo000XOol00aS6Oomoo`0;
Ool00aS6Oomoo`02Ool01G`0Oomoogoo00003Woo00<HaWooOol027oo00<007ooOol017oo00<HaWoo
Ool02goo00<007ooOol05Goo00<HaWooOol03Woo100O6Goo00<HaWooOol05Goo00<007ooOol02goo
00=l07ooOol01goo00<HaWooOol05goo100O3goo00<HaWooOol05Goo00<007ooOol02goo00<HaWoo
Ool017oo00@007ooOoml00Moo`036<Ioogoo00aoo`03001oogoo00Ioo`036<Ioogoo00=oo`03001o
ogoo00Eoo`036<Ioogoo03ioo`03001oogoo06Ioo`X000moo`00:7oo00<HaWooOol02goo00<HaWoo
Ool00Woo00El07ooOomoo`0000ioo`036<Ioogoo00Uoo`03001oogoo00=oo`036<Ioogoo02=oo`03
6<Ioogoo00ioo`@07aUoo`036<Ioogoo01Eoo`03001oogoo00]oo`03O01oogoo00Moo`036<Ioogoo
01=oo`03001oogoo00@07a1oo`036<Ioogoo02=oo`036<Ioogoo00Aoo`03001oog`000Qoo`036<Io
ogoo01Eoo`036<Ioogoo00]oo`036<Ioogoo03ioo`03001oogoo06Ioo`X000moo`00:7oo00<HaWoo
Ool02goo00DHaWooOomoog`000Aoo`03001oogoo00aoo`036<Ioogoo00Uoo`03001oogoo00=oo`03
6<Ioogoo02=oo`036<Ioogoo00moo`@07aMoo`036<Ioogoo01Ioo`03001oogoo00aoo`03O01oogoo
00Moo`036<Ioogoo019oo`03001oogoo00@07a1oo`036<Ioogoo02=oo`036<Ioogoo00Aoo`03001o
og`000Qoo`036<Ioogoo01Eoo`036<Ioogoo00]oo`036<Ioogoo03ioo`03001oogoo06Ioo`X000mo
o`00:7oo00<HaWooOol01Woo00<007ooOol00Woo00DHaWooOomoog`001=oo`036<Ioogoo00moo`03
6<Ioogoo02=oo`036<Ioogoo00moo`@07`9oo`03001oogoo019oo`036<Ioogoo01Ioo`03001oogoo
00aoo`03O01oogoo00Moo`036<Ioogoo015oo`03001oogoo00@07a5oo`036<Ioogoo02=oo`036<Io
ogoo00Eoo`03O01oogoo00Moo`036<Ioogoo01Eoo`036<Ioogoo00]oo`036<Ioogoo03ioo`03001o
ogoo06Ioo`X000moo`00:7oo00<HaWooOol01Woo00<007ooOol00Woo00@HaWooOoml01Aoo`036<Io
ogoo00moo`036<Ioogoo00]oo`03001oogoo01Eoo`036<Ioogoo011oo`@07`03Ool007oo019oo`03
6<Ioogoo01Moo`03001oogoo00aoo`03O01oogoo00Moo`036<Ioogoo015oo`03001oogoo00@07a5o
o`036<Ioogoo01Ioo`03001oogoo00Yoo`036<Ioogoo00Eoo`03O01oogoo00Moo`036<Ioogoo00ao
o`03001oogoo00Moo`036<Ioogoo00Yoo`036<Ioogoo03ioo`03001oogoo06Ioo`X000moo`00:7oo
00<HaWooOol01Woo00<007ooOol00Woo00@HaWooOoml01Aoo`036<Ioogoo00moo`036<Ioogoo00]o
o`03001oogoo01Eoo`036<Ioogoo011oo`@07`9oo`03001oogoo011oo`036<Ioogoo01Moo`03001o
ogoo00eoo`03O01oogoo00Moo`036<Ioogoo01=oo`<07a9oo`036<Ioogoo01Ioo`03001oogoo00Yo
o`036<Ioogoo00Aoo`03O01oogoo00Qoo`036<Ioogoo00aoo`03001oogoo00Moo`056<IoogooOol0
0008Ool00aS6Oomoo`0nOol00`00Oomoo`1VOol:000?Ool002Uoo`036<Ioogoo00Eoo`03001oogoo
009oo`046<IoogooO00EOol00aS6Oomoo`08Ool00`00Oomoo`03Ool00aS6Oomoo`0:Ool00`00Oomo
o`0FOol00aS6Oomoo`0AOol301l2Ool00`00Oomoo`0@Ool00aS6Oomoo`0GOol00`00Oomoo`0=Ool0
0g`0Oomoo`07Ool00aS6Oomoo`0BOol401lBOol00aS6Oomoo`0FOol00`00Oomoo`0:Ool00aS6Oomo
o`03Ool00`00O01oo`09Ool00aS6Oomoo`0=Ool00`00Oomoo`06Ool01AS6Oomoogoo000027oo00<H
aWooOol0?Woo00<007ooOol0IWoo2P003goo000YOol00aS6Oomoo`0:Ool00aS6Ooml0006Ool00`00
Oomoo`0=Ool00aS6Oomoo`08Ool00`00Oomoo`03Ool00aS6Oomoo`0:Ool00`00Oomoo`0GOol00aS6
Oomoo`0@Ool401lCOol00aS6Oomoo`0HOol2000?Ool00g`0Oomoo`06Ool00aS6Oomoo`0BOol401lB
Ool00aS6Oomoo`0FOol00`00Oomoo`0;Ool00aS6Oomoo`02Ool00`00Oomoo`09Ool00aS6Oomoo`0=
Ool00`00Oomoo`06Ool01AS6Oomoogoo000027oo00<HaWooOol0?Woo00<007ooOol0?Goo0P0027oo
0P007Goo2P003goo000YOol00aS6Oomoo`0:Ool00aS6Ooml0006Ool00`00Oomoo`0=Ool00aS6Oomo
o`08Ool00`00Oomoo`02Ool00aS6Oomoo`0UOol00aS6Oomoo`0@Ool401lCOol00aS6Oomoo`0HOol0
0`00Oomoo`0>Ool00g`0Oomoo`07Ool00aS6Oomoo`0>Ool00`00Ool07`0301lCOol00aS6Oomoo`0T
Ool00aS6Oomoo`02Ool00`00Oomoo`09Ool00aS6Oomoo`0FOol01AS6Oomoogoo000027oo00<HaWoo
Ool0?Woo00<007ooOol0?Goo00<007ooOol027oo00<007ooOol06goo2P003goo000YOol00aS6Oomo
o`0:Ool00aS6O01oo`06Ool00`00Oomoo`0=Ool00aS6Oomoo`08Ool00`00Oomoo`02Ool00aS6Oomo
o`0UOol00aS6Oomoo`0AOol401lBOol00aS6Oomoo`0HOol00`00Oomoo`0?Ool00g`0Oomoo`06Ool0
0aS6Oomoo`0>Ool00`00Ool07`0301lBOol00aS6Oomoo`0UOol00aS6Oomoo`02Ool00`00Oomoo`09
Ool00aS6Oomoo`0FOol00aS6Oomoo`0:Ool00aS6Oomoo`0nOol00`00Oomoo`0YOol30003Ool50000
17oo000007oo0P000goo00<007ooOol00P0000=oo`0000000goo00<007ooOol06goo2P003goo000Y
Ool00aS6Oomoo`06Ool01P00Oomoogoo6<Il00Moo`03001oogoo00eoo`036<Ioogoo00eoo`036<Io
ogoo02Eoo`036<Ioogoo015oo`@07a5oo`036<Ioogoo01Uoo`03001oogoo00moo`03O01oogoo00Io
o`036<Ioogoo00ioo`0300007`0O00807a=oo`036<Ioogoo02Eoo`056<IoogooOoml000<Ool00aS6
Oomoo`0FOol00aS6Oomoo`0:Ool00aS6Oomoo`0nOol00`00Oomoo`0YOol01000Oomoo`0017oo00D0
07ooOomoo`00009oo`03001oogoo009oo`07001oogooOol007oo000017oo00<007ooOol06goo2P00
3goo000YOol00aS6Oomoo`06Ool01@00Oomoogoo6<H067oo00<HaWooOol03Goo00<HaWooOol09Goo
00<HaWooOol04Woo100O00<007ooOol03Goo00<HaWooOol06Goo00<007ooOol03goo00=l07ooOol0
1goo00<HaWooOol03Goo00<0000O01l00P0O4goo00<HaWooOol09Goo00DHaWooOomoog`000aoo`03
6<Ioogoo01Ioo`036<Ioogoo00Uoo`036<Ioogoo03moo`03001oogoo00=ooaml00Uoo`8000Aoo`05
001oogooOol00002Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0K
Ool:000?Ool002Uoo`036<Ioogoo00Ioo`05001oogooOolHaP0HOol00aS6Oomoo`0=Ool00aS6Oomo
o`0;Ool00`00Oomoo`0GOol00aS6Oomoo`0BOol401l00`00Oomoo`0=Ool00aS6Oomoo`0IOol00`00
Oomoo`0@Ool00g`0Oomoo`06Ool00aS6Oomoo`0>Ool301lDOol00aS6Oomoo`0HOol00`00Oomoo`0:
Ool011S6Oomoog`03Goo00<HaWooOol03Goo00<007ooOol01Woo00DHaWooOomoo`0000Moo`036<Io
ogoo03moo`03001oogoo02Uoo`8000Ioo`05001oogooOol00002Ool00`00Oomoo`02Ool01`00Oomo
ogoo001oo`0000Aoo`03001oogoo01]oo`X000moo`00:Goo00<HaWooOol01Woo00D007ooOomooaS6
01Qoo`036<Ioogoo00eoo`036<Ioogoo00]oo`03001oogoo01Moo`036<Ioogoo01=oo`<07`03001o
ogoo00aoo`036<Ioogoo01Yoo`03001oogoo011oo`03O01oogoo00Ioo`036<Ioogoo00eoo`@07aAo
o`036<Ioogoo01Qoo`03001oogoo00Yoo`046<IoogooO00=Ool00aS6Oomoo`0=Ool00`00Oomoo`06
Ool01AS6Oomoogoo00001goo00<HaWooOol0?goo00<007ooOol0:Goo00@007ooOol000=oo`80009o
o`D000Aoo`03001oogoo00800003Ool0000000=oo`03001oogoo01]oo`X000moo`00:Goo00<HaWoo
Ool02Goo00<HaWooOol01goo00<007ooOol03Goo00<HaWooOol02Goo00D007ooOomooaS600eoo`03
001oogoo01Moo`036<Ioogoo01=oo`<07`03001oogoo00aoo`036<Ioogoo01Yoo`03001oogoo015o
o`03O01oogoo00Ioo`036<Ioogoo00aoo`@07aAoo`036<Ioogoo01Qoo`03001oogoo00Yoo`036<Io
og`0009oo`03001oogoo00Uoo`036<Ioogoo00eoo`03001oogoo00Ioo`056<IoogooOol00007Ool0
0aS6Oomoo`0oOol00`00Oomoo`0ZOol3000@Ool20008Ool2000MOol:000?Ool002Uoo`036<Ioogoo
00Uoo`036<Ioogoo00Moo`03001oogoo00eoo`036<Ioogoo00Uoo`05001oogooOolHaP0=Ool00`00
Oomoo`0HOol00aS6Oomoo`0BOol401l>Ool00aS6Oomoo`0JOol00`00Oomoo`0AOol00g`0Oomoo`06
Ool00aS6Oomoo`0;Ool0100001l07`0O5Goo00<HaWooOol067oo00<007ooOol02Woo00<HaWooO000
0Woo00<007ooOol027oo00<HaWooOol03Woo00<007ooOol01Woo00DHaWooOomoo`0000Moo`036<Io
ogoo03moo`03001oogoo035oo`03001oogoo039oo`X000moo`00:Goo00<HaWooOol027oo00=l01S6
Ool01goo00<007ooOol03Woo00<HaWooOol02Goo00D007ooOomooaS602Qoo`036<Ioogoo01=oo`@0
7`aoo`036<Ioogoo00Ioo`8000Aoo`8000=oo`<000Moo`03001oogoo019oo`03O01oogoo00Eoo`03
6<Ioogoo00]oo`0400007`0O01lEOol00aS6Oomoo`0UOol00aS6O01oo`02Ool00`00Oomoo`08Ool0
0aS6Oomoo`0GOol00aS6Oomoo`09Ool00aS6Oomoo`0oOol00`00Oomoo`1VOol:000?Ool002Uoo`03
6<Ioogoo00Qoo`03O00HaWoo00Moo`03001oogoo00ioo`036<Ioogoo00Uoo`05001oogooOolHaP0X
Ool00aS6Oomoo`0COol401l<Ool00aS6Oomoo`05Ool01000Oomoo`000goo0P001Woo00<007ooOol0
17oo00<007ooOol04Woo00=l07ooOol01Goo00<HaWooOol02Woo00@0000O01l07aIoo`036<Ioogoo
02Eoo`036<Il07oo009oo`03001oogoo00Qoo`036<Ioogoo01Qoo`036<Ioogoo00Qoo`036<Ioogoo
03moo`03001oogoo06Ioo`X000moo`00:Goo00<HaWooOol01Woo00@007ooO00HaQUoo`036<Ioogoo
00eoo`036<Ioogoo02Ioo`036<Ioogoo01Aoo`807`0300007goo00Yoo`036<Ioogoo00Eoo`04001o
ogoo000;Ool00`00Oomoo`04Ool3000BOol00g`0Oomoo`06Ool00aS6Oomoo`09Ool0100001l07`0O
5Goo00<HaWooOol09Woo00<HaWooOol03Goo00<HaWooOol067oo00<HaWooOol027oo00<HaWooOol0
?goo00<007ooOol0IWoo2P003goo000YOol00aS6Oomoo`06Ool01000O01ooaS66Woo00<HaWooOol0
37oo00<HaWooOol02Woo00<007ooOol06Goo00<HaWooOol057oo0P0O00<0000OOol02Goo00<HaWoo
Ool01Woo00@007ooOol000Qoo`<000Moo`03001oogoo01=oo`03O01oogoo00Eoo`036<Ioogoo00Uo
o`<07aIoo`036<Ioogoo01Uoo`03001oogoo00Yoo`036<Ioogoo00eoo`036<Ioogoo00ioo`03001o
ogoo00Moo`036<Ioogoo00Qoo`036<Ioogoo03moo`03001oogoo06Ioo`X000moo`00:Goo00<HaWoo
Ool01Woo00@007`0OolHaQYoo`036<Ioogoo00]oo`036<Ioogoo00]oo`03001oogoo01Yoo`036<Io
ogoo01Aoo`807`03001oogoo00Qoo`036<Ioogoo00Ioo`04001oogoo0008Ool00`00Oomoo`07Ool0
0`00Oomoo`0COol00g`0Oomoo`05Ool00aS6Oomoo`08Ool401lFOol00aS6Oomoo`0IOol00`00Oomo
o`09Ool00g`0OolHaP0>Ool00aS6Oomoo`0>Ool00`00Oomoo`07Ool00aS6Ool00008Ool00aS6Oomo
o`0oOol00`00Oomoo`1VOol:000?Ool002Yoo`036<Ioogoo00Eoo`04001oogoo6<HJOol00aS6Oomo
o`08Ool01000OomooaS63Goo00<007ooOol06Woo00<HaWooOol057oo0P0O00<0000OOol027oo00<H
aWooOol01goo0P002Goo10001Woo00<007ooOol057oo00=l07ooOol01Goo00<HaWooOol01goo100O
5Woo00<HaWooOol06Goo00<007ooOol02Goo00=l07oo6<H00Woo00<007ooOol02Goo00<HaWooOol0
3goo00<007ooOol01Woo00<HaWoo000027oo00<HaWooOol0?goo00<007ooOol0IWoo2P003goo000Z
Ool00aS6Oomoo`05Ool017`0OomooaS627oo00<007ooOol03goo00<HaWooOol027oo00@007ooOolH
aPeoo`03001oogoo01Yoo`036<Ioogoo01Aoo`@07`Qoo`036<Ioogoo01eoo`03001oogoo01Aoo`03
O01oogoo00Eoo`036<Ioogoo00Ioo`@07aMoo`036<Ioogoo01Yoo`03001oogoo00Moo`04O01oogoo
6<H2Ool00`00Oomoo`09Ool00aS6Oomoo`0?Ool00`00Oomoo`06Ool00aS6Ool00008Ool00aS6Oomo
o`0oOol00`00Oomoo`1VOol:000?Ool002Yoo`036<Ioogoo00Aoo`05O01oogooOolHaP08Ool00`00
Oomoo`0?Ool00aS6Oomoo`09Ool00`00OolHaP0ZOol00aS6Oomoo`0EOol401l7Ool00aS6Oomoo`0M
Ool00`00Oomoo`0EOol00g`0Oomoo`04Ool00aS6Oomoo`06Ool0100O00007`0O5goo00<HaWooOol0
97oo00Al07ooOolHaP9oo`03001oogoo00Uoo`036<Ioogoo01Qoo`036<Ioo`0000Qoo`036<Ioogoo
03moo`03001oogoo06Ioo`X000moo`00:Woo00<HaWooOol00goo00=l07ooOol00Woo00<HaWooOol0
1Woo00<007ooOol03goo00<HaWooOol02Goo00<007oo6<H0:Woo00<HaWooOol05Goo100O1goo00<H
aWooOol07Goo00<007ooOol05Goo00=l07ooOol01Goo00<HaWooOol017oo0P0O00<0000OOol05goo
00<HaWooOol08goo00El07ooOomooaS6009oo`03001oogoo00Uoo`036<Ioogoo01Qoo`036<Ioogoo
00Qoo`036<Ioogoo03moo`03001oogoo06Ioo`03001oogoo01Ioo`00:Woo00<HaWooOol00goo00Il
07ooOol007oo6<H8Ool00`00Oomoo`0?Ool00aS6Oomoo`0;Ool00aS6Oomoo`0YOol00aS6Oomoo`0E
Ool0100O00007`0O1Goo00<HaWooOol07Woo00<007ooOol05Goo00=l07ooOol01Goo00<HaWooOol0
17oo00@07`0001l07aMoo`036<Ioogoo02Aoo`05O01oogooOolHaP0>Ool00aS6Oomoo`0HOol00aS6
Oomoo`08Ool00aS6Oomoo`0oOol00`00Oomoo`1VOol00`00Oomoo`0FOol002Yoo`036<Ioogoo009o
o`07O01oogooOol007oo6<H06Woo00<HaWooOol02goo00<HaWooOol0:Goo00<HaWooOol05Goo00@0
7`0001l07`Eoo`036<Ioogoo01ioo`03001oogoo01Ioo`03O01oogoo00Aoo`036<Ioogoo00=oo`@0
7aQoo`036<Ioogoo02=oo`03O01oogoo009oo`036<Ioogoo00aoo`036<Ioogoo01Qoo`036<Ioogoo
00Moo`036<Ioogoo041oo`03001oogoo06Ioo`03001oogoo01Ioo`00:Woo00<HaWooOol00Woo00Ml
07ooOomoo`00OolHaP0JOol00aS6Oomoo`0;Ool00aS6Oomoo`0:Ool00`00Oomoo`0LOol00aS6Oomo
o`0FOol0100O00007`0O17oo00<HaWooOol07Woo00<007ooOol05Woo00=l07ooOol01Goo00@HaWoo
Oomoo`D07aQoo`036<Ioogoo01]oo`03001oogoo00Eoo`03O01oogoo009oo`036<Ioogoo00aoo`03
6<Ioogoo00moo`03001oogoo00Ioo`036<Ioo`0000Moo`036<Ioogoo041oo`03001oogoo06Ioo`03
001oogoo01Ioo`00:Woo00DHaWooOomoog`000Aoo`03001ooaS601Yoo`036<Ioogoo00]oo`036<Io
ogoo00Yoo`03001oogoo01aoo`036<Ioogoo01Ioo`0401l0000O01l3Ool00aS6Oomoo`0OOol2000H
Ool00g`0Oomoo`04Ool011S6Oomoogoo100O6Goo00<HaWooOol06goo00<007ooOol01Goo00=l07oo
Ool00Woo00<HaWooOol02goo00<HaWooOol047oo00<007ooOol01Woo00<HaWoo00001goo00<HaWoo
Ool0@7oo00<007ooOol0IWoo00<007ooOol05Woo000ZOol01AS6OomoogooO0001Goo00<HaWooOol0
1Woo00<007ooOol04Goo00<HaWooOol027oo00<007oo6<H037oo00<007ooOol077oo00<HaWooOol0
5goo100O0Woo00<HaWooOol07goo00<007ooOol05goo00=l07ooOol017oo00<HaWooOol0100O6Woo
00<HaWooOol06goo00<007ooOol017oo00=l07ooOol00goo00@HaWooOol000Yoo`036<Ioogoo011o
o`03001oogoo00Ioo`036<Ioo`0000Moo`036<Ioogoo041oo`03001oogoo06Ioo`03001oogoo01Io
o`00:Woo00@HaWooOoml00Ioo`036<Ioogoo00Ioo`03001oogoo015oo`036<Ioogoo00Qoo`03001o
oaS600aoo`03001oogoo01eoo`036<Ioogoo01Ioo`@07`9oo`036<Ioogoo01moo`03001oogoo01Qo
o`03O01oogoo00=oo`036<Ioogoo00<07`03001oogoo01Moo`036<Ioogoo01aoo`03001oogoo00Ao
o`03O01oogoo00=oo`046<Ioogoo000:Ool00aS6Oomoo`0@Ool00`00Oomoo`07Ool00aS6001oo`06
Ool00aS6Oomoo`10Ool00`00Oomoo`1VOol00`00Oomoo`0FOol002Yoo`046<IoogooO006Ool00aS6
Oomoo`06Ool00`00Oomoo`0AOol00aS6Oomoo`08Ool00`006<Ioo`0/Ool00aS6Oomoo`0GOol401l0
0aS6Oomoo`0POol00`00Oomoo`0HOol00g`0Oomoo`04Ool00aS601l07`0201l00`00Oomoo`0GOol0
0aS6Oomoo`0ROol00g`0Oomoo`05Ool00aS6001oo`0:Ool00aS6Oomoo`0JOol00aS6Oomoo`06Ool0
0aS6Oomoo`10Ool00`00Oomoo`1VOol00`00Oomoo`0FOol002Yoo`036<Ioog`000Moo`036<Ioogoo
00Ioo`03001oogoo015oo`036<Ioogoo00Qoo`03000HaWoo02aoo`036<Ioogoo01Moo`@07`036<Io
ogoo021oo`03001oogoo01Uoo`03O01oogoo00=oo`056<H07`0O01l0000JOol00aS6Oomoo`0ROol0
0g`0Oomoo`05Ool00aS6001oo`0:Ool00aS6Oomoo`0JOol00aS6Oomoo`06Ool00aS6Oomoo`10OomZ
000HOol002Yoo`036<Il07oo00Ioo`03000HaWoo01]oo`036<Ioogoo00Uoo`036<Ioogoo02]oo`03
6<Ioogoo01Moo`0300007`0O00807b9oo`03001oogoo01Yoo`03O01oogoo009oo`@07`03001oogoo
01Qoo`036<Ioogoo025oo`03O01oogoo00Ioo`036<Ioogoo00Yoo`036<Ioogoo01Yoo`036<Ioogoo
00Ioo`036<Ioogoo0<9oo`00:goo00<HaWooOol01Goo00<001S6Ool06goo00<HaWooOol02Goo00<H
aWooOol02goo00<007ooOol07Goo00<HaWooOol05goo00<0000O01l00P0O8Woo00<007ooOol06Woo
00=l07ooOol00Woo100O6goo00<HaWooOol07Goo00D007ooOomoog`000Qoo`036<Ioogoo00Yoo`03
6<Ioogoo011oo`03001oogoo00Moo`036<Ioogoo00Ioo`036<Ioogoo0<9oo`00:Woo00=l01S6Ool0
1Woo00<001S6Ool06goo00<HaWooOol02Goo00<HaWooOol02goo00<007ooOol07Goo00<HaWooOol0
67oo00<0000O01l00P0O8Goo00<007ooOol06goo00=l07ooOol0100O6goo00<HaWooOol07Woo00@0
07ooOoml00Uoo`036<Ioogoo00Yoo`036<Ioogoo011oo`03001oogoo00Moo`03001oogoo00Ioo`03
6<Ioogoo0<9oo`00:Woo00=l01S6Ool01Woo00<001S6Ool01goo00<007ooOol04Goo00<HaWooOol0
2Goo00<007ooOol02Woo00<007ooOol07goo00<HaWooOol067oo100O8Goo00<007ooOol06goo00=l
07oo01l0100O6goo00<HaWooOol07Woo00@007ooOoml00Uoo`036<H007oo00Yoo`036<Ioogoo015o
o`03001oogoo00Ioo`03001oogoo00Ioo`036<Ioogoo0<9oo`00:Goo00=l07oo6<H027oo00<HaWoo
Ool01Woo00<007ooOol04Goo00<HaWooOol02Goo00<007ooOol02Woo00<007ooOol07goo00<HaWoo
Ool067oo00<HaP0O01l00P0O87oo00<007ooOol077oo00=l000O01l00P0O00<HaP00Ool06Goo00<H
aWooOol07Woo00<007ooO0002Woo00<HaP00Ool02Woo00<HaWooOol04Goo00<007ooOol01Woo00<0
07ooOol01Woo00<HaWooOol0`Woo000YOol00g`0OolHaP08Ool00aS6Oomoo`06Ool00`00Oomoo`0A
Ool00aS6Oomoo`09Ool00`00Oomoo`0/Ool00aS6Oomoo`0HOol00aS601l07`0301lOOol2000MOol4
01l00goo6<H0000JOol00aS6Oomoo`0POol00g`0Oomoo`08Ool00aS6001oo`0:Ool00aS6Oomoo`0J
Ool00`00Oomoo`06Ool00aS6Oomoo`32Ool002Qoo`04O01oogoo6<H8Ool00aS6Oomoo`06Ool00`00
Oomoo`0BOol00aS6Oomoo`08Ool00`00Oomoo`0/Ool00aS6Oomoo`0HOol00aS6Ool07`0301lOOol0
0`00Oomoo`0LOol401l00goo6<H0000JOol00aS6Oomoo`0OOol00g`0Oomoo`09Ool00`00Oomoo`09
Ool00aS6Oomoo`0KOol00aS6Oomoo`06Ool00aS6Oomoo`32Ool002Qoo`04O01oogoo6<H8Ool00`00
Oomoo`0KOol00aS6Oomoo`08Ool00aS6Oomoo`0/Ool00aS6Oomoo`0GOol011S6001oogoo100O7Woo
00<007ooOol06goo100O0Woo00<001S6Ool067oo00<HaWooOol087oo00=l07ooOol02Woo00<HaWoo
Ool027oo00<HaWooOol06goo00<HaWooOol01Woo00<HaWooOol0`Woo000WOol01G`0Oomoogoo6<H0
27oo00<007ooOol06goo00<HaWooOol01goo00<HaWooOol0;Woo00<HaWooOol05Woo00@HaP00Oomo
o`D07aeoo`03001oogoo01]oo`@07`=oo`036<Ioogoo01Moo`036<Ioogoo01moo`03O00007oo00]o
o`036<Ioogoo00Qoo`036<Ioogoo01]oo`036<Ioogoo00Ioo`036<Ioogoo0<9oo`009Woo00=l07oo
Ool00Woo00<HaWooOol01Woo00<007ooOol06goo00<HaWooOol01goo00<HaWooOol02goo00<007oo
Ool087oo00<HaWooOol05Woo00<HaWoo00000Woo100O7Goo00<007ooOol06Woo100O00Eoog`0Oomo
oaS601Uoo`036<Ioogoo01ioo`03O01oo`0000aoo`036<Ioogoo00Qoo`036<Ioogoo019oo`03001o
ogoo00Ioo`03001oogoo00Eoo`036<Ioogoo0<=oo`009Woo00=l07ooOol00Woo00<HaWooOol01Goo
00<HaP00Ool077oo00<HaWooOol01goo00<HaWooOol02goo00<007ooOol087oo00<HaWooOol05Goo
00@HaWooOol000=oo`@07aaoo`03001oogoo01Yoo`@07`9oo`04O01oogoo6<HHOol00aS6Oomoo`0N
Ool00g`0Ool0000<Ool00aS6Oomoo`08Ool00aS6Oomoo`0BOol00`00Oomoo`06Ool00`006<Ioo`05
Ool00aS6Oomoo`33Ool002Eoo`03O01oogoo00=oo`036<Ioogoo00Eoo`036<Ioogoo00Ioo`03001o
ogoo01=oo`036<Ioogoo00Moo`036<Ioo`0000Yoo`03001oogoo025oo`036<Ioogoo01Eoo`036<Io
ogoo00Aoo`@07aaoo`03001oogoo01Uoo`@07`=oo`04001oogoo6<HHOol00aS6Oomoo`0MOol017`0
Oomoo`002goo00<001S6Ool02Goo00<HaWooOol04Woo00<007ooOol01Goo00<007oo6<H01Woo00<H
aWooOol0`goo000/Ool00aS6Oomoo`04Ool00aS6Oomoo`06Ool00`00Oomoo`0COol00aS6Oomoo`07
Ool00aS6Ool0000:Ool00`00Oomoo`0QOol00aS6Oomoo`0EOol00aS6Oomoo`05Ool401lKOol00`00
Oomoo`0HOol501l3Ool01@00O01oogoo6<H05Woo00<HaWooOol07Goo00=l07ooOol03Goo00<001S6
Ool02Goo00<HaWooOol04Woo00<007ooOol01Goo00<007oo6<H01Woo00<HaWooOol0`goo000/Ool0
0aS6Oomoo`04Ool00aS6Oomoo`06Ool00`00Oomoo`0COol00aS6Oomoo`07Ool00aS6Ool0000^Ool0
0aS6Oomoo`0DOol00aS6Oomoo`06Ool501lJOol00`00Oomoo`0HOol401l3Ool01P00Oomoog`0OolH
aQIoo`036<Ioogoo01eoo`03O01oogoo00eoo`03000HaWoo00Uoo`036<Ioogoo01aoo`036<Ioogoo
00Aoo`036<Ioogoo0<=oo`00;7oo00<HaWooOol017oo00<HaWooOol01Woo00<007ooOol04goo00<H
aWooOol01goo00<HaWooOol0;goo00<HaWooOol04goo00<HaWooOol00goo00@007ooOomoo`@07aYo
o`03001oogoo01Moo`@07`Aoo`06001oogooO01ooaS65Woo00<HaWooOol077oo00=l07ooOol03Woo
00<001S6Ool02Goo00<HaWooOol077oo00<HaWooOol017oo00<HaWooOol0`goo000/Ool00aS6Oomo
o`04Ool00aS6001oo`0MOol00aS6Oomoo`06Ool00aS6Oomoo`0_Ool00aS6Oomoo`0COol00aS6Oomo
o`03Ool00`00Oomoo`02Ool401lIOol00`00Oomoo`0GOol401l8Ool00g`0OolHaP0EOol00aS6Oomo
o`0LOol00g`0Oomoo`0@Ool00aS6Oomoo`06Ool00aS6Oomoo`0MOol00aS6Oomoo`04Ool00aS6Oomo
o`33Ool002aoo`036<Ioogoo00Aoo`036<H007oo01eoo`036<Ioogoo00Eoo`036<Ioogoo00]oo`03
001oogoo029oo`036<Ioogoo019oo`036<Ioogoo00Aoo`03001oogoo009oo`D07aQoo`8001Moo`@0
7`Yoo`03O00HaWoo01Aoo`036<Ioogoo01]oo`03O01oogoo00Aoo`03001oogoo00Yoo`036<Ioogoo
00Ioo`036<Ioogoo01Aoo`03001oogoo00Ioo`036<Ioogoo00Aoo`036<Ioogoo0<=oo`00;7oo00<H
aWooOol017oo00<HaP00Ool07Goo00<HaWooOol01Goo00<HaWooOol02goo00<007ooOol08Woo00<H
aWooOol04Woo00<HaWooOol01Goo00<007ooOol00Woo1@0O5goo00<007ooOol05Goo1@0O2Woo00=l
07oo6<H04goo00<HaWooOol06goo00=l07ooOol01Goo00<007ooOol02Woo00<HaWooOol01Woo00<H
aWooOol057oo00<007ooOol017oo00<007oo6<H01Woo00<HaWooOol0`goo000/Ool00aS6Oomoo`04
Ool00aS6Ool00006Ool00`00Oomoo`0DOol00aS6Oomoo`05Ool011S6Oomoo`002Goo00<007ooOol0
8goo00<HaWooOol04Woo00<HaWooOol02goo1@0O5Woo00<007ooOol05Goo100O17oo00<007ooOol0
1Goo00=l01S6Ool04Woo00<HaWooOol06goo00=l07ooOol01Goo00<007ooOol01goo00@007ooOolH
aPQoo`036<Ioogoo01Aoo`03001oogoo00Aoo`03001ooaS600Ioo`036<Ioogoo0<=oo`00;7oo00<H
aWooOol017oo00<HaWooOol01Woo00<007ooOol057oo00<HaWooOol01Goo00@HaWooOol000Uoo`03
001oogoo02Aoo`036<Ioogoo011oo`036<Ioogoo00eoo`@07aIoo`03001oogoo01Aoo`@07`Eoo`03
001oogoo00Ioo`036<Ioogoo015oo`036<Ioogoo01Yoo`03O01oogoo00Ioo`03001oogoo00Moo`04
001oogoo6<H8Ool00aS6Oomoo`0DOol00`00Oomoo`04Ool00`00OolHaP06Ool00aS6Oomoo`33Ool0
02aoo`036<Ioogoo00Aoo`036<Ioogoo00Eoo`03001oogoo01Eoo`036<Ioogoo00Eoo`056<Ioogoo
Ool0000_Ool00aS6Oomoo`0@Ool00aS6Oomoo`0>Ool401lEOol00`00Oomoo`0COol501l4Ool00`00
Oomoo`07Ool00g`06<Ioo`0AOol00aS6Oomoo`0IOol00g`0Oomoo`0AOol01000OomooaS627oo00<H
aWooOol06goo00<007oo6<H01Woo00<HaWooOol0`goo000/Ool00aS6Oomoo`03Ool00aS6Oomoo`06
Ool00`00Oomoo`0EOol00aS6Oomoo`05Ool01AS6Oomoogoo0000;goo00<HaWooOol03goo00<HaWoo
Ool02Goo00<007ooOol00goo1@0O57oo00<007ooOol04Woo1@0O1Goo00<007ooOol027oo00<HaWoo
Ool047oo00<HaWooOol067oo00=l07ooOol04Woo00@007ooOolHaPQoo`036<Ioogoo01ioo`036<Io
ogoo00=oo`036<Ioogoo0<=oo`00;Goo00<HaWooOol00Woo00@HaWooOol001eoo`036<Ioogoo00Eo
o`036<Ioogoo039oo`036<Ioogoo00ioo`036<Ioogoo00Yoo`03001oogoo00=oo`D07a=oo`03001o
ogoo019oo`@07a5oo`036<Il07oo00moo`036<Ioogoo01Qoo`03O01oogoo01Moo`036<Ioogoo00Ao
o`036<Ioogoo01moo`036<Ioogoo009oo`036<Ioogoo0<Aoo`00;Goo00<HaWooOol00Woo00@HaWoo
Ool001eoo`036<Ioogoo00Eoo`036<Ioogoo00Yoo`03001oogoo02Eoo`036<Ioogoo00ioo`036<Io
ogoo00Yoo`03001oogoo00Aoo`@07a=oo`03001oogoo015oo`@07a=oo`036<Il07oo00ioo`036<Io
ogoo01Moo`03O01oogoo00]oo`03001oogoo00Yoo`036<Ioogoo00Aoo`036<Ioogoo01moo`036<Io
ogoo009oo`036<Ioogoo0<Aoo`00;Goo00<HaWooOol00Woo00@HaWooOol001eoo`036<Ioogoo00Eo
o`036<Ioogoo00Yoo`03001oogoo02Eoo`036<Ioogoo00eoo`036<Ioogoo01=oo`@07a9oo`03001o
ogoo00moo`H07a=oo`036<Ioog`000ioo`036<Ioogoo01Ioo`03O01oogoo00aoo`03001oogoo00Yo
o`036<Ioogoo00Aoo`036<Ioogoo01Ioo`03001oogoo009oo`05001oogooOolHaP04Ool00aS6Oomo
o`34Ool002eoo`036<Ioogoo009oo`046<Ioogoo000MOol00aS6Oomoo`04Ool00aS6Oomoo`03Ool0
0`00Oomoo`04Ool00`00Oomoo`0VOol00aS6Oomoo`0=Ool00aS6Oomoo`0COol501lAOol00`00Oomo
o`0>Ool601l5Ool00`00Oomoo`0<Ool011S6Oomoog`037oo00<HaWooOol05Woo00=l07ooOol03Goo
00<007ooOol02Woo00<HaWooOol017oo00<HaWooOol05Woo00<007ooOol00Woo00D007ooOomooaS6
00Aoo`036<Ioogoo0<Aoo`00;Goo00<HaWooOol00Woo00<HaWooOol01Woo00<007ooOol05Goo00<H
aWooOol017oo00<HaWooOol00goo00<007ooOol017oo00<007ooOol09goo00<HaWooOol02goo00<H
aWooOol05Goo1@0O47oo0P003Woo1P0O1Woo00<007ooOol03Goo00@HaWooOoml00]oo`036<Ioogoo
01Eoo`03O01oogoo00ioo`03001oogoo00Eoo`03001oogoo009oo`036<Ioogoo00Aoo`036<Ioogoo
01Ioo`03001oogoo009oo`05001oogooOolHaP04Ool00aS6Oomoo`34Ool002eoo`036<Ioogoo009o
o`036<Ioogoo00Ioo`03001oogoo01Ioo`036<Ioogoo00=oo`036<Ioogoo00=oo`03001oogoo02io
o`036<Ioogoo00]oo`036<Ioogoo00moo`03001oogoo00Aoo`D07`moo`03001oogoo00aoo`D07`Mo
o`03001oogoo00moo`046<IoogooO00:Ool00aS6Oomoo`0EOol00g`0Oomoo`0FOol00`00Oomoo`02
Ool00aS6Oomoo`04Ool00aS6Oomoo`0FOol00`00Oomoo`02Ool01@00Oomoogoo6<H017oo00<HaWoo
Ool0a7oo000]Ool00aS6Oomoo`02Ool00aS6Oomoo`05Ool00`00Oomoo`0GOol00aS6Oomoo`03Ool0
0aS6Oomoo`03Ool00`00Oomoo`0^Ool00aS6Oomoo`0:Ool00aS6Oomoo`0@Ool20006Ool501l>Ool0
0`00Oomoo`0;Ool501lJOol011S6Oomoog`02Goo00<HaWooOol05Goo00=l07ooOol05Woo00<007oo
Ool017oo00<HaWooOol00Woo00<HaWooOol087oo00DHaWooOomooaS60<Moo`00;Goo00DHaWooOomo
oaS600Qoo`03001oogoo01Moo`036<Ioogoo00=oo`036<Ioogoo03Aoo`036<Ioogoo00Yoo`036<Io
ogoo019oo`03001oogoo00Aoo`H07`aoo`03001oogoo00Yoo`D07a]oo`056<IoogooOoml0008Ool0
0aS6Oomoo`0DOol00g`0Oomoo`0GOol00`00Oomoo`04Ool00aS6Oomoo`02Ool00aS6Oomoo`0POol0
1AS6Oomoogoo6<H0agoo000]Ool01AS6Oomoogoo6<H017oo00<007ooOol06goo00<HaWooOol00goo
00<HaWooOol0=Goo00<HaWooOol027oo00<HaWooOol06goo1P0O2goo00<007ooOol027oo1P0O7Goo
00DHaWooOomoog`000Moo`036<Ioogoo019oo`9l025oo`036<Ioogoo009oo`036<Ioogoo025oo`04
6<Ioogoo6<K7Ool002eoo`056<IoogooOolHaP04Ool00`00Oomoo`0LOol01AS6Oomoogoo6<H037oo
00<007ooOol0:Goo00<HaWooOol027oo00<HaWooOol077oo1P0O2Woo00<007ooOol01Woo1`0O27oo
00<007ooOol04goo00<HaWooOol00Woo0W`017oo00<HaWooOol04Woo00=l07ooOol057oo0P002goo
00<HaWooOol00Woo00<HaWooOol067oo00<007ooOol01Woo00@HaWooOolHa/Moo`00;Goo00DHaWoo
OomooaS600Aoo`03001oogoo01aoo`056<IoogooOolHaP0<Ool00`00Oomoo`0ZOol00aS6Oomoo`07
Ool00aS6Oomoo`0FOol00`00Oomoo`05Ool601l8Ool00`00Oomoo`05Ool701l6Ool3000GOol00aS6
Oomoo`03Ool2O002Ool00aS6Oomoo`0AOol00g`0Oomoo`0FOol00`00Oomoo`09Ool01AS6Oomoogoo
6<H06goo00@007ooOol000Eoo`046<Ioogoo6<K7Ool002ioo`046<Ioogoo6<H5Ool00`00Ool0000K
Ool01AS6Oomoogoo6<H01goo00<007ooOol00Woo00<007ooOol0:goo00<HaWooOol01Goo00<HaWoo
Ool067oo0P001Woo1`0O1Woo00<007ooOol00goo1`0O8goo00<HaWooOol017oo00=l01S6Ool04Goo
0W`06Goo00D007ooOomoo`0000Moo`056<IoogooOolHaP0KOol01000Oomoo`001Goo00@HaWooOolH
a/Moo`00;Woo00<HaWoo6<H027oo00<007ooOol06Woo00@HaWooOolHaPQoo`04001oogoo000^Ool0
0aS6Oomoo`04Ool00aS6Oomoo`0KOol00`00Oomoo`04Ool901l3Ool01000Oomoogoo1`0O9Woo00<H
aWooOol00goo00<HaW`0O0003goo0W`06goo00D007ooOomoo`0000Qoo`046<Ioogoo6<HKOol01000
Oomoo`001Goo00@HaWooOolHa/Moo`00;Woo00<HaWoo6<H027oo00<007ooOol06Woo00<HaWoo6<H0
2Goo00<007ooOol0<7oo0QS60goo00<HaWooOol087oo00<007ooOol00Woo500O1Goo0`007goo0QS6
0goo0QS60Woo0g`02Goo0g`087oo00<007ooOol01goo11S69Goo0QS6b7oo000_Ool26<H7Ool00`00
Oomoo`0KOol36<HmOol46<HTOol30002Ool5000401l4000301l4000WOol56<H6Ool9O00SOol00`00
Oomoo`08Ool26<HVOol26<K8Ool00<=oo`04001oogooOol;01ooOomQOol00<=oo`03001oogoo00Io
o`03001oogoo0?moofAoo`00`Woo0P0027oo00<007ooOol0ogooI7oo003<Ool00`00Oomoo`3oOomT
Ool00<aoo`03001oogoo0?moofAoo`00ogooogoo=7oo003oOoooOoldOol00?mooomoocAoo`00ogoo
ogoo=7oo003oOoooOoldOol00?mooomoocAoo`00ogooogoo=7oo003oOoooOoldOol00?mooomoocAo
o`00XWoo10000goo100000=oo`00Ool00Woo10000goo0P0000Aoo`00Ool000Aoo`@000=oo`<00003
Ool007oo00Ioo`@000=oo`800004Ool007oo0006Ool01000Oomoogoo10000goo0`00ogoo?7oo002Q
Ool00`00Oomoo`02Ool00`00Ool00004Ool20002Ool00`00Oomoo`02Ool00`00Ool00002Ool20000
0goo001oo`02Ool00`00Oomoo`02Ool00`00Ool00003Ool20006Ool00`00Oomoo`02Ool00`00Ool0
0002Ool200000goo001oo`04Ool01@00Ool007oo000017oo00<007oo00000goo00<007ooOol0ogoo
>Goo002POol00`00Oomoo`04Ool20004Ool20002Ool00`00Oomoo`02Ool00`00Ool00003Ool00`00
Ool00003Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`04Ool20003
Ool00`00Ool00005Ool01@00Ool007oo00002Woo00<007ooOol0ogoo>Goo002POol00`00Oomoo`05
Ool00`00Oomoo`02Ool20002Ool00`00Oomoo`02Ool00`00Ool00003Ool00`00Ool00003Ool60000
0goo001oo`03Ool00`00Oomoo`03Ool00`00Oomoo`05Ool01`00Oomoogoo001oo`0000Eoo`04001o
o`00Ool60002Ool3003oOollOol00:1oo`03001oogoo00Eoo`03001oogoo009oo`80009oo`03001o
ogoo009oo`03001oo`0000=oo`03001oo`0000=oo`03001oogoo009oo`03001oo`0000Aoo`03001o
ogoo00=oo`03001oogoo00Eoo`07001oogooOol007oo00001Goo00D007oo001oo`0000Aoo`03001o
o`000?moocmoo`00X7oo00<007ooOol01Goo00<007ooOol00Woo0P000Woo00<007ooOol00Woo00<0
07oo00000goo00@007oo0000009oo`03001oogoo009oo`03001oo`0000=oo`8000Eoo`03001oogoo
00Eoo`06001oogooOol007oo0P000goo00@007ooOomoo`8000Aoo`03001oo`0000=oo`03001oogoo
0?moocUoo`00X7oo00<007ooOol017oo00<007oo00000`0000=oo`00Ool00Woo10000Woo00T007oo
Oomoo`00Ool007oo00000Woo10000goo0`0000=oo`00Ool017oo00<007ooOol017oo0P000goo00D0
07oo001oo`00009oo`06001oogooOol007oo10000goo0`00ogoo?7oo002QOol00`00Oomoo`02Ool0
0`00Oomoo`05Ool00`00Oomoo`0MOol00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`3oOomF
Ool00:9oo`@000Qoo`03001oogoo01eoo`03001oogoo00Eoo`@00?mooeUoo`00ogooogoo=7oo003o
OoooOoldOol00?mooomoocAoo`00ogooogoo=7oo003oOoooOoldOol00?mooomoocAoo`00\
\>"],
  ImageRangeCache->{{{0, 561.5}, {246.375, 0}} -> {-1.07629, -0.648942, 
  0.00790192, 0.00790192}, {{14.4375, 394.125}, {240.5, 
  5.8125}} -> {-4.90274, -1.10705, 0.0359999, 0.0145623}, {{432.063, 537.5}, {
  218.063, 66.1875}} -> {-13.8694, -1.00231, 0.0478031, 0.0478031}, {{
  437.688, 469.063}, {212.375, 181}} -> {-14.6729, -5.38115, 0.0501997, 
  0.16245}, {{437.688, 469.063}, {176, 144.625}} -> {-14.6729, -9.32055, 
  0.0501997, 0.16245}, {{437.688, 469.063}, {139.563, 
  108.188}} -> {-14.6729, -13.2667, 0.0501997, 0.16245}, {{437.688, 
  469.063}, {103.188, 71.8125}} -> {-14.6729, -17.2061, 0.0501997, 0.16245}}]
}, Open  ]],

Cell[TextData[{
  "Note that the different specifications can be used singly or together; by \
combining two specifications we obtained a thick blue line. The use of such \
specifications give ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " a capability to represent many data sets on the same graph. There isn't \
time to discuss legends in detail here, but it is important to note that \
there is a Package for treating legends. ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " also has several options. Let's explore a few of the relevant ones:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Plot[{x^2, x^3}, {x, -3, 3}, PlotRange -> {-3, 3},
PlotLabel -> \"Squares and Cubes\", Frame -> True];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.15873 0.309017 0.103006 [
[.02381 -0.0125 -6 -9 ]
[.02381 -0.0125 6 0 ]
[.18254 -0.0125 -6 -9 ]
[.18254 -0.0125 6 0 ]
[.34127 -0.0125 -6 -9 ]
[.34127 -0.0125 6 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.65873 -0.0125 -3 -9 ]
[.65873 -0.0125 3 0 ]
[.81746 -0.0125 -3 -9 ]
[.81746 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .10301 -12 -4.5 ]
[-0.0125 .10301 0 4.5 ]
[-0.0125 .20601 -12 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .30902 -6 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .41202 -6 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .51503 -6 -4.5 ]
[-0.0125 .51503 0 4.5 ]
[-0.0125 .61803 -6 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 .61803 .125 0 ]
[ 1 0 .125 0 ]
[.5 .63053 -53 0 ]
[.5 .63053 53 12 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(-3)] .02381 -0.0125 0 1 Mshowa
.18254 0 m
.18254 .00625 L
s
[(-2)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(-1)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(0)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(1)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(2)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(3)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .10301 m
.00625 .10301 L
s
[(-2)] -0.0125 .10301 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(-1)] -0.0125 .20601 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(0)] -0.0125 .30902 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(1)] -0.0125 .41202 1 0 Mshowa
0 .51503 m
.00625 .51503 L
s
[(2)] -0.0125 .51503 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(3)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
.02381 .61178 m
.02381 .61803 L
s
.18254 .61178 m
.18254 .61803 L
s
.34127 .61178 m
.34127 .61803 L
s
.5 .61178 m
.5 .61803 L
s
.65873 .61178 m
.65873 .61803 L
s
.81746 .61178 m
.81746 .61803 L
s
.97619 .61178 m
.97619 .61803 L
s
.125 Mabswid
.05556 .61428 m
.05556 .61803 L
s
.0873 .61428 m
.0873 .61803 L
s
.11905 .61428 m
.11905 .61803 L
s
.15079 .61428 m
.15079 .61803 L
s
.21429 .61428 m
.21429 .61803 L
s
.24603 .61428 m
.24603 .61803 L
s
.27778 .61428 m
.27778 .61803 L
s
.30952 .61428 m
.30952 .61803 L
s
.37302 .61428 m
.37302 .61803 L
s
.40476 .61428 m
.40476 .61803 L
s
.43651 .61428 m
.43651 .61803 L
s
.46825 .61428 m
.46825 .61803 L
s
.53175 .61428 m
.53175 .61803 L
s
.56349 .61428 m
.56349 .61803 L
s
.59524 .61428 m
.59524 .61803 L
s
.62698 .61428 m
.62698 .61803 L
s
.69048 .61428 m
.69048 .61803 L
s
.72222 .61428 m
.72222 .61803 L
s
.75397 .61428 m
.75397 .61803 L
s
.78571 .61428 m
.78571 .61803 L
s
.84921 .61428 m
.84921 .61803 L
s
.88095 .61428 m
.88095 .61803 L
s
.9127 .61428 m
.9127 .61803 L
s
.94444 .61428 m
.94444 .61803 L
s
.25 Mabswid
0 .61803 m
1 .61803 L
s
.99375 0 m
1 0 L
s
.99375 .10301 m
1 .10301 L
s
.99375 .20601 m
1 .20601 L
s
.99375 .30902 m
1 .30902 L
s
.99375 .41202 m
1 .41202 L
s
.99375 .51503 m
1 .51503 L
s
.125 Mabswid
.99625 .0206 m
1 .0206 L
s
.99625 .0412 m
1 .0412 L
s
.99625 .0618 m
1 .0618 L
s
.99625 .0824 m
1 .0824 L
s
.99625 .12361 m
1 .12361 L
s
.99625 .14421 m
1 .14421 L
s
.99625 .16481 m
1 .16481 L
s
.99625 .18541 m
1 .18541 L
s
.99625 .22661 m
1 .22661 L
s
.99625 .24721 m
1 .24721 L
s
.99625 .26781 m
1 .26781 L
s
.99625 .28842 m
1 .28842 L
s
.99625 .32962 m
1 .32962 L
s
.99625 .35022 m
1 .35022 L
s
.99625 .37082 m
1 .37082 L
s
.99625 .39142 m
1 .39142 L
s
.99625 .43262 m
1 .43262 L
s
.99625 .45322 m
1 .45322 L
s
.99625 .47383 m
1 .47383 L
s
.99625 .49443 m
1 .49443 L
s
.99625 .53563 m
1 .53563 L
s
.99625 .55623 m
1 .55623 L
s
.99625 .57683 m
1 .57683 L
s
.99625 .59743 m
1 .59743 L
s
.25 Mabswid
1 0 m
1 .61803 L
s
0 .30902 m
1 .30902 L
s
.5 0 m
.5 .61803 L
s
gsave
.5 .63053 -114 -4 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(Squares) show
111.000000 13.000000 moveto
(and) show
135.000000 13.000000 moveto
(Cubes) show
165.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.22524 .61803 m
.26171 .54116 L
.30316 .46743 L
.34309 .40968 L
.3815 .36643 L
.40095 .34913 L
.42237 .33366 L
.44268 .32245 L
.45178 .31852 L
.46172 .31501 L
.4671 .31344 L
.4721 .3122 L
.47727 .31113 L
.48196 .31035 L
.48658 .30975 L
.4887 .30954 L
.49093 .30935 L
.49332 .3092 L
.49438 .30915 L
.49552 .3091 L
.49675 .30906 L
.49789 .30904 L
.49859 .30903 L
.49925 .30902 L
.50049 .30902 L
.50163 .30903 L
.50286 .30905 L
.50401 .30908 L
.50508 .30912 L
.50754 .30925 L
.51014 .30944 L
.51268 .30967 L
.51504 .30994 L
.5204 .31072 L
.5293 .31253 L
.53882 .31518 L
.54906 .31886 L
.56016 .32382 L
.58032 .33539 L
.60019 .35006 L
.62123 .36911 L
.65912 .41253 L
.69946 .47167 L
.73829 .54115 L
s
.73829 .54115 m
.7746 .61803 L
s
.2715 0 m
.28158 .04064 L
.30316 .11257 L
.32216 .16414 L
.34309 .2095 L
.36292 .24268 L
.38395 .26876 L
.40336 .28577 L
.41426 .29278 L
.42424 .29782 L
.43408 .30164 L
.44322 .3043 L
.45325 .30639 L
.45886 .30722 L
.46395 .30781 L
.46882 .30824 L
.47149 .30842 L
.47397 .30856 L
.47837 .30876 L
.4807 .30883 L
.48319 .30889 L
.48587 .30894 L
.48739 .30897 L
.48879 .30898 L
.49001 .30899 L
.49136 .309 L
.49203 .309 L
.49276 .30901 L
.49408 .30901 L
.49529 .30901 L
.49661 .30902 L
.49727 .30902 L
.49798 .30902 L
.49926 .30902 L
.50054 .30902 L
.50175 .30902 L
.50284 .30902 L
.50401 .30902 L
.50525 .30902 L
.50637 .30902 L
.50767 .30903 L
.5089 .30904 L
.51004 .30904 L
.51109 .30905 L
.51348 .30908 L
.51484 .3091 L
.51613 .30912 L
.51852 .30918 L
.5211 .30926 L
.52389 .30937 L
Mistroke
.52923 .30966 L
.53191 .30985 L
.53486 .31011 L
.54007 .31067 L
.54493 .31135 L
.54996 .31223 L
.55541 .3134 L
.56533 .3162 L
.57498 .31987 L
.58389 .32423 L
.6026 .33683 L
.61324 .34641 L
.62321 .35719 L
.64343 .38502 L
.66195 .41842 L
.68252 .46562 L
.70163 .52014 L
Mfstroke
.70163 .52014 m
.72757 .61803 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{208, 128.5},
  ImageMargins->{{13, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004h0000`1000`40O003h00Oogoo>Goo003oOoli
Ool00?moocUoo`00<Goo10008Woo1P008goo1@007goo0P009Goo1@008Goo1P008goo10006Woo000`
Ool00`00Oomoo`02Ool00`00Oomoo`0POol01@00Oomoogoo00009Goo00<007ooOol07Woo00@007oo
Ool002Ioo`03001oogoo029oo`05001oogooOol0000ROol00`00Oomoo`02Ool00`00Oomoo`0GOol0
03Eoo`03001oogoo025oo`03001oogoo02Ioo`03001oogoo01eoo`03001oogoo009oo`03001oogoo
02=oo`03001oogoo02=oo`03001oogoo02Qoo`03001oogoo01Moo`009goo1`001goo00<007ooOol0
5Woo1`001Goo00<007ooOol06Goo1`001Goo00<007ooOol07Goo00<007ooOol00Woo00<007ooOol0
8goo00<007ooOol097oo00<007ooOol09goo00<007ooOol05goo000bOol3000VOol00`00Oomoo`0T
Ool00`00Oomoo`0MOol00`00Oomoo`02Ool00`00Oomoo`0SOol00`00Oomoo`0UOol00`00Oomoo`0S
Ool3000JOol003Eoo`03001oogoo02Aoo`03001oogoo02=oo`03001oogoo01eoo`03001oogoo009o
o`03001oogoo02=oo`03001oogoo02Ioo`03001oogoo02Eoo`03001oogoo01Moo`00=Goo00<007oo
Ool097oo00<007ooOol08goo00<007ooOol07Goo00<007ooOol00Woo00<007ooOol08goo00<007oo
Ool09Woo00<007ooOol09Goo00<007ooOol05goo000`Ool00`00Oomoo`02Ool00`00Oomoo`0OOol0
0`00Oomoo`02Ool00`00Oomoo`0QOol3000POol01000Oomoo`0097oo0`008goo00<007ooOol00Woo
00<007ooOol087oo00<007ooOol00Woo00<007ooOol05goo000aOol4000SOol4000UOol2000QOol2
000VOol2000TOol4000TOol4000JOol00?moocUoo`00ogoo>Goo003oOoliOol00?moocUoo`00:Goo
nP005Goo000YOol00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool0
0`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`04Ool2
0007Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomo
o`05Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomo
o`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomo
o`05Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomo
o`05Ool00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`0COol002Uoo`03001oogoo00=oo`03
001oogoo02Aoo`03001oogoo01Aoo`03001oogoo00ioo`03001oogoo02Aoo`03001oogoo02Eoo`03
001oogoo02Eoo`03001oogoo02Aoo`03001oogoo00=oo`03001oogoo01=oo`00:Goo00<007ooOol0
@Woo00<007ooOol0=7oo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomoo`12Ool00`00
Oomoo`0dOol00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`8004=oo`03001oogoo03Aoo`03001o
ogoo07Uoo`8001Eoo`00:Goo00<007ooOol0@Woo00<007ooOol0=7oo00<007ooOol0NWoo00<007oo
Ool04goo000YOol00`00Oomoo`12Ool00`00Oomoo`0dOol00`00Oomoo`1jOol00`00Oomoo`0COol0
02Uoo`03001oogoo04=oo`03001oogoo03=oo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo00<0
07ooOol0@goo00<007ooOol0<goo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomoo`13
Ool00`00Oomoo`0cOol00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`8004Aoo`03001oogoo03=o
o`03001oogoo07Uoo`8001Eoo`00:Goo00<007ooOol0@goo00<007ooOol0<goo00<007ooOol0NWoo
00<007ooOol04goo000YOol00`00Oomoo`14Ool00`00Oomoo`0bOol00`00Oomoo`1jOol00`00Oomo
o`0COol002Uoo`03001oogoo04Aoo`03001oogoo039oo`03001oogoo07Yoo`03001oogoo01=oo`00
:Goo00<007ooOol0A7oo00<007ooOol0<Woo00<007ooOol0NWoo00<007ooOol04goo000YOol20016
Ool00`00Oomoo`0aOol00`00Oomoo`1iOol2000EOol002Uoo`03001oogoo04Eoo`03001oogoo035o
o`03001oogoo07Yoo`03001oogoo01=oo`00:Goo00<007ooOol0AGoo00<007ooOol0<Goo00<007oo
Ool0NWoo00<007ooOol04goo000YOol00`00Oomoo`15Ool00`00Oomoo`0aOol00`00Oomoo`1jOol0
0`00Oomoo`0COol002Uoo`03001oogoo04Ioo`03001oogoo031oo`03001oogoo07Yoo`03001oogoo
01=oo`00:Goo0P00Agoo00<007ooOol0<7oo00<007ooOol0NGoo0P005Goo000YOol00`00Oomoo`16
Ool00`00Oomoo`0`Ool00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`03001oogoo04Ioo`03001o
ogoo031oo`03001oogoo07Yoo`03001oogoo01=oo`007Woo1P001Goo00<007ooOol0Agoo00<007oo
Ool0;goo00<007ooOol0NWoo00<007ooOol04goo000OOol01@00Oomoogoo00001Goo00<007ooOol0
Agoo00<007ooOol0;goo00<007ooOol0NWoo00<007ooOol04goo000POol00`00Oomoo`06Ool20018
Ool00`00Oomoo`0_Ool00`00Oomoo`1iOol2000EOol001Eoo`L000Eoo`03001oogoo00Eoo`03001o
ogoo04Qoo`03001oogoo02ioo`03001oogoo07Yoo`03001oogoo01=oo`008Woo00<007ooOol017oo
00<007ooOol0B7oo00<007ooOol0;Woo00<007ooOol0NWoo00<007ooOol04goo000SOol00`00Oomo
o`03Ool00`00Oomoo`18Ool00`00Oomoo`0^Ool00`00Oomoo`1jOol00`00Oomoo`0COol002=oo`03
001oogoo00=oo`03001oogoo04Uoo`03001oogoo02eoo`03001oogoo07Yoo`03001oogoo01=oo`00
7Woo00<007ooOol00Woo00<007ooOol00goo0P00BWoo00<007ooOol0;Goo00<007ooOol0NGoo0P00
5Goo000OOol40006Ool00`00Oomoo`19Ool00`00Oomoo`0]Ool00`00Oomoo`1jOol00`00Oomoo`0C
Ool002Uoo`03001oogoo04Yoo`03001oogoo02aoo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo
00<007ooOol0BWoo00<007ooOol0;7oo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomo
o`1;Ool00`00Oomoo`0[Ool00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`8004aoo`03001oogoo
02]oo`03001oogoo07Uoo`8001Eoo`00:Goo00<007ooOol0Bgoo00<007ooOol0:goo00<007ooOol0
NWoo00<007ooOol04goo000YOol00`00Oomoo`1<Ool00`00Oomoo`0ZOol00`00Oomoo`1jOol00`00
Oomoo`0COol002Uoo`03001oogoo04aoo`03001oogoo02Yoo`03001oogoo07Yoo`03001oogoo01=o
o`00:Goo00<007ooOol0C7oo00<007ooOol0:Woo00<007ooOol0NWoo00<007ooOol04goo000YOol2
001>Ool00`00Oomoo`0YOol00`00Oomoo`1iOol2000EOol002Uoo`03001oogoo04eoo`03001oogoo
02Uoo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo00<007ooOol0CWoo00<007ooOol0:7oo00<0
07ooOol0NWoo00<007ooOol04goo000YOol00`00Oomoo`1>Ool00`00Oomoo`0XOol00`00Oomoo`1j
Ool00`00Oomoo`0COol002Uoo`03001oogoo04ioo`03001oogoo02Qoo`03001oogoo07Yoo`03001o
ogoo01=oo`00:Goo00<007ooOol0Cgoo00<007ooOol09goo00<007ooOol0NWoo00<007ooOol04goo
000YOol2001@Ool00`00Oomoo`0WOol00`00Oomoo`1iOol2000EOol002Uoo`03001oogoo051oo`03
001oogoo02Ioo`03001oogoo07Yoo`03001oogoo01=oo`007goo1@001Goo00<007ooOol0D7oo00<0
07ooOol09Woo00<007ooOol0NWoo00<007ooOol04goo000QOol00`00Oomoo`05Ool00`00Oomoo`1A
Ool00`00Oomoo`0UOol00`00Oomoo`1jOol00`00Oomoo`0COol0025oo`03001oogoo00Eoo`03001o
ogoo055oo`03001oogoo02Eoo`03001oogoo07Yoo`03001oogoo01=oo`005Goo1`001Goo00<007oo
Ool01Goo0P00DWoo00<007ooOol09Goo00<007ooOol0NGoo0P005Goo000QOol00`00Oomoo`05Ool0
0`00Oomoo`1BOol00`00Oomoo`0TOol00`00Oomoo`1jOol00`00Oomoo`0COol0025oo`03001oogoo
00Eoo`03001oogoo059oo`03001oogoo02Aoo`03001oogoo07Yoo`03001oogoo01=oo`008Goo00<0
07ooOol01Goo00<007ooOol0Dgoo00<007ooOol08goo00<007ooOol0NWoo00<007ooOol04goo000O
Ool30007Ool00`00Oomoo`1DOol00`00Oomoo`0ROol00`00Oomoo`1jOol00`00Oomoo`0COol0021o
o`8000Moo`8005Eoo`03001oogoo029oo`03001oogoo07Uoo`8001Eoo`00:Goo00<007ooOol0EGoo
00<007ooOol08Goo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomoo`1EOol00`00Oomo
o`0QOol00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`03001oogoo05Ioo`03001oogoo021oo`03
001oogoo07Yoo`03001oogoo01=oo`00:Goo00<007ooOol0Egoo00<007ooOol07goo00<007ooOol0
NWoo00<007ooOol04goo000YOol2001IOol00`00Oomoo`0NOol00`00Oomoo`1iOol2000EOol002Uo
o`03001oogoo05Uoo`03001oogoo01eoo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo00<007oo
Ool0FWoo00<007ooOol077oo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomoo`1KOol0
0`00Oomoo`0KOol00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`03001oogoo05aoo`03001oogoo
01Yoo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo0P00GWoo00<007ooOol06Goo00<007ooOol0
NGoo0P005Goo000YOol00`00Oomoo`1MOol00`00Oomoo`0IOol00`00Oomoo`1jOol00`00Oomoo`0C
Ool002Uoo`03001oogoo05ioo`03001oogoo01Qoo`03001oogoo07Yoo`03001oogoo01=oo`00:Goo
00<007ooOol0Ggoo00<007ooOol05goo00<007ooOol0NWoo00<007ooOol04goo000YOol00`00Oomo
o`1POol00`00Oomoo`0FOol00`00Oomoo`1jOol00`00Oomoo`0COol002Uoo`80069oo`<001Eoo`03
001oogoo07Uoo`8001Eoo`00:Goo00<007ooOol0I7oo0P004goo00<007ooOol0NWoo00<007ooOol0
4goo000YOol00`00Oomoo`1VOol00`00Oomoo`0@Ool00`00Oomoo`1jOol00`00Oomoo`0COol0021o
o`8000Moo`03001oogoo06Moo`80011oo`03001oogoo07Yoo`03001oogoo01=oo`007goo00@007oo
Ool000Ioo`03001oogoo06Uoo`D000]oo`03001oogoo07Yoo`03001oogoo01=oo`007Woo00<007oo
Ool00Woo00<007ooOol00goonP005Goo000NOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomo
o`1aOol700000goo001oo`02Ool=001/Ool00`00Oomoo`0COol001ioo`03001oogoo009oo`03001o
ogoo00=oo`03001oogoo06moo`8000Qoo`03001oogoo00Moo`<000Aoo`8006Yoo`03001oogoo01=o
o`007Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0K7oo0`002Woo00<007ooOol02Woo
0P0017oo0P00J7oo00<007ooOol04goo000NOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomo
o`1ZOol2000=Ool00`00Oomoo`0<Ool20004Ool2001VOol00`00Oomoo`0COol001moo`04001oogoo
0006Ool00`00Oomoo`1XOol2000?Ool00`00Oomoo`0>Ool20004Ool2001TOol00`00Oomoo`0COol0
021oo`8000Moo`8006Qoo`03001oogoo00moo`03001oogoo011oo`03001oogoo00=oo`03001oogoo
061oo`8001Eoo`00:Goo00<007ooOol0IGoo0P004Woo00<007ooOol04Goo0P0017oo00<007ooOol0
H7oo00<007ooOol04goo000YOol00`00Oomoo`1SOol2000DOol00`00Oomoo`0COol00`00Oomoo`02
Ool00`00Oomoo`1OOol00`00Oomoo`0COol002Uoo`03001oogoo069oo`03001oogoo01Aoo`03001o
ogoo01Aoo`03001oogoo009oo`03001oogoo05ioo`03001oogoo01=oo`00:Goo00<007ooOol0HGoo
00<007ooOol05Goo00<007ooOol05Goo00<007ooOol00Woo00<007ooOol0GGoo00<007ooOol04goo
000YOol2001QOol00`00Oomoo`0FOol00`00Oomoo`0FOol20003Ool00`00Oomoo`1KOol2000EOol0
02Uoo`03001oogoo05moo`03001oogoo01Moo`03001oogoo01Qoo`05001oogooOol0001MOol00`00
Oomoo`0COol002Uoo`03001oogoo05ioo`03001oogoo01Qoo`03001oogoo01Uoo`800003Ool007oo
05aoo`03001oogoo01=oo`00:Goo00<007ooOol0GGoo00<007ooOol06Goo00<007ooOol06goo00<0
07oo0000G7oo00<007ooOol04goo000YOol00`00Oomoo`1LOol00`00Oomoo`0JOol00`00Oomoo`0L
Ool00`00Ool0001KOol00`00Oomoo`0COol002Uoo`8005aoo`03001oogoo01]oo`03001oogoo01ao
o`04001oogoo001IOol2000EOol002Uoo`03001oogoo05Yoo`03001oogoo01aoo`03001oogoo01eo
o`03001oo`0005Yoo`03001oogoo01=oo`00:Goo00<007ooOol0FGoo00<007ooOol07Goo00<007oo
Ool07Woo00<007oo0000FGoo00<007ooOol04goo000YOol00`00Oomoo`1HOol00`00Oomoo`0NOol0
0`00Oomoo`0OOol00`00Ool0001HOol00`00Oomoo`0COol002Uoo`03001oogoo05Moo`03001oogoo
01moo`03001oogoo021oo`8005Qoo`03001oogoo01=oo`00:Goo0P00Egoo00<007ooOol087oo00<0
07ooOol08Goo0P00EWoo0P005Goo000YOol00`00Oomoo`1EOol00`00Oomoo`0QOol00`00Oomoo`0Q
Ool2001GOol00`00Oomoo`0COol002Uoo`03001oogoo05Aoo`03001oogoo029oo`03001oogoo029o
o`8005Ioo`03001oogoo01=oo`007goo1@001Goo00<007ooOol0Dgoo00<007ooOol08goo00<007oo
Ool08goo0P00EGoo00<007ooOol04goo000QOol00`00Oomoo`05Ool00`00Oomoo`1BOol00`00Oomo
o`0TOol00`00Oomoo`0TOol00`00Oomoo`1COol00`00Oomoo`0COol0025oo`03001oogoo00Eoo`80
05=oo`03001oogoo02Aoo`03001oogoo02Eoo`03001oogoo055oo`8001Eoo`008Goo00<007ooOol0
1Goo00<007ooOol0DGoo00<007ooOol09Goo00<007ooOol09Goo00<007ooOol0DWoo00<007ooOol0
4goo000QOol00`00Oomoo`05Ool00`00Oomoo`1@Ool00`00Oomoo`0VOol00`00Oomoo`0VOol00`00
Oomoo`1AOol00`00Oomoo`0COol0025oo`03001oogoo00Eoo`03001oogoo051oo`03001oogoo02Io
o`03001oogoo02Ioo`80059oo`03001oogoo01=oo`008Goo00<007ooOol01Goo00<007ooOol0Cgoo
00<007ooOol09goo00<007ooOol09goo00<007ooOol0D7oo00<007ooOol04goo000OOol30007Ool2
001?Ool00`00Oomoo`0XOol00`00Oomoo`0WOol2001@Ool2000EOol0021oo`8000Moo`03001oogoo
04ioo`03001oogoo02Qoo`03001oogoo02Qoo`80051oo`03001oogoo01=oo`00:Goo00<007ooOol0
CGoo00<007ooOol0:Goo00<007ooOol0:7oo0P00D7oo00<007ooOol04goo000YOol00`00Oomoo`1<
Ool00`00Oomoo`0ZOol00`00Oomoo`0YOol2001?Ool00`00Oomoo`0COol002Uoo`03001oogoo04ao
o`03001oogoo02Yoo`03001oogoo02Uoo`03001oo`0004ioo`03001oogoo01=oo`00:Goo0P00C7oo
00<007ooOol0:goo00<007ooOol0:Goo00<007oo0000CGoo0P005Goo000YOol00`00Oomoo`1:Ool0
0`00Oomoo`0/Ool00`00Oomoo`0ZOol00`00Ool0001=Ool00`00Oomoo`0COol002Uoo`03001oogoo
04Yoo`03001oogoo02aoo`03001oogoo02Yoo`04001oogoo001<Ool00`00Oomoo`0COol002Uoo`03
001oogoo04Uoo`03001oogoo02eoo`03001oogoo02]oo`03001oo`0004aoo`03001oogoo01=oo`00
:Goo00<007ooOol0B7oo00<007ooOol0;Woo00<007ooOol0:goo00@007ooOol004]oo`03001oogoo
01=oo`00:Goo0P00BGoo00<007ooOol0;Woo00<007ooOol0;7oo00@007ooOol004Uoo`8001Eoo`00
:Goo00<007ooOol0Agoo00<007ooOol0;goo00<007ooOol0;7oo00@007ooOol004Yoo`03001oogoo
01=oo`00:Goo00<007ooOol0Agoo00<007ooOol0;goo00<007ooOol0;7oo00D007ooOomoo`0004Uo
o`03001oogoo01=oo`00:Goo00<007ooOol0AWoo00<007ooOol0<7oo00<007ooOol0;Goo00@007oo
Ool004Uoo`03001oogoo01=oo`00:Goo00<007ooOol0AGoo00<007ooOol0<Goo00<007ooOol0;Goo
00D007ooOomoo`0004Qoo`03001oogoo01=oo`00:Goo00<007ooOol0AGoo00<007ooOol0<Goo00<0
07ooOol0;Goo00<007ooOol00Woo00<007ooOol0AGoo00<007ooOol04goo000YOol20015Ool00`00
Oomoo`0bOol00`00Oomoo`0^Ool01@00Oomoogoo0000AWoo0P005Goo000YOol00`00Oomoo`14Ool0
0`00Oomoo`0bOol00`00Oomoo`0^Ool00`00Oomoo`02Ool00`00Oomoo`14Ool00`00Oomoo`0COol0
01ioo`H000Eoo`03001oogoo04=oo`03001oogoo03=oo`03001oogoo02ioo`03001oogoo009oo`03
001oogoo04Aoo`03001oogoo01=oo`007goo00D007ooOomoo`0000Eoo`03001oogoo04=oo`03001o
ogoo03=oo`03001oogoo02moo`03001oogoo009oo`03001oogoo04=oo`03001oogoo01=oo`0087oo
00<007ooOol01Woo00<007ooOol0@Woo00<007ooOol0=7oo00<007ooOol0;goo00<007ooOol00goo
00<007ooOol0@Woo00<007ooOol04goo000QOol00`00Oomoo`05Ool20013Ool00`00Oomoo`0dOol0
0`00Oomoo`0_Ool00`00Oomoo`03Ool00`00Oomoo`11Ool2000EOol0029oo`03001oogoo00Aoo`03
001oogoo045oo`03001oogoo03Eoo`03001oogoo031oo`03001oogoo00=oo`03001oogoo045oo`03
001oogoo01=oo`008goo00<007ooOol00goo00<007ooOol0@7oo00<007ooOol0=Woo00<007ooOol0
<7oo00<007ooOol00goo00<007ooOol0@Goo00<007ooOol04goo000SOol00`00Oomoo`03Ool00`00
Oomoo`10Ool00`00Oomoo`0fOol00`00Oomoo`0`Ool00`00Oomoo`04Ool00`00Oomoo`10Ool00`00
Oomoo`0COol001ioo`03001oogoo009oo`03001oogoo00=oo`03001oogoo03moo`03001oogoo03Mo
o`03001oogoo035oo`03001oogoo00Aoo`03001oogoo03moo`03001oogoo01=oo`007goo10001Woo
0P00@7oo00<007ooOol0=goo00<007ooOol0<Goo00<007ooOol017oo00<007ooOol0?Woo0P005Goo
000YOol00`00Oomoo`0nOol00`00Oomoo`0hOol00`00Oomoo`0aOol00`00Oomoo`05Ool00`00Oomo
o`0nOol00`00Oomoo`0COol002Uoo`03001oogoo03ioo`03001oogoo03Qoo`03001oogoo035oo`03
001oogoo00Eoo`03001oogoo03ioo`03001oogoo01=oo`00:Goo00<007ooOol0?Goo00<007ooOol0
>Goo00<007ooOol0<Woo00<007ooOol01Goo00<007ooOol0?Goo00<007ooOol04goo000YOol00`00
Oomoo`0mOol00`00Oomoo`0iOol00`00Oomoo`0bOol00`00Oomoo`05Ool00`00Oomoo`0mOol00`00
Oomoo`0COol002Uoo`8003eoo`03001oogoo03Yoo`03001oogoo039oo`03001oogoo00Ioo`03001o
ogoo03]oo`8001Eoo`00:Goo00<007ooOol0?7oo00<007ooOol0>Woo00<007ooOol0<Woo00<007oo
Ool01Woo00<007ooOol0?7oo00<007ooOol04goo000YOol00`00Oomoo`0kOol00`00Oomoo`0kOol0
0`00Oomoo`0cOol00`00Oomoo`06Ool00`00Oomoo`0kOol00`00Oomoo`0COol002Uoo`03001oogoo
03]oo`03001oogoo03]oo`03001oogoo03=oo`03001oogoo00Ioo`03001oogoo03]oo`03001oogoo
01=oo`00:Goo00<007ooOol0>Woo00<007ooOol0?7oo00<007ooOol0<goo00<007ooOol01goo00<0
07ooOol0>Woo00<007ooOol04goo000YOol2000kOol00`00Oomoo`0lOol00`00Oomoo`0cOol00`00
Oomoo`07Ool00`00Oomoo`0iOol2000EOol002Uoo`03001oogoo03Uoo`03001oogoo03eoo`03001o
ogoo03Aoo`03001oogoo00Moo`03001oogoo03Uoo`03001oogoo01=oo`00:Goo00<007ooOol0>Goo
00<007ooOol0?Goo00<007ooOol0=7oo00<007ooOol01goo00<007ooOol0>Goo00<007ooOol04goo
000YOol00`00Oomoo`0iOol00`00Oomoo`0mOol00`00Oomoo`0dOol00`00Oomoo`07Ool00`00Oomo
o`0iOol00`00Oomoo`0COol002Uoo`03001oogoo03Qoo`03001oogoo03ioo`03001oogoo03Aoo`03
001oogoo00Qoo`03001oogoo03Qoo`03001oogoo01=oo`00:Goo0P00>Goo00<007ooOol0?Woo00<0
07ooOol0=Goo00<007ooOol01goo00<007ooOol0=goo0P005Goo000YOol00`00Oomoo`0gOol00`00
Oomoo`0oOol00`00Oomoo`0eOol00`00Oomoo`08Ool00`00Oomoo`0gOol00`00Oomoo`0COol002Uo
o`03001oogoo03Moo`03001oogoo03moo`03001oogoo03Eoo`03001oogoo00Qoo`03001oogoo03Mo
o`03001oogoo01=oo`007goo10001Woo00<007ooOol0=Woo00<007ooOol0@7oo00<007ooOol0=Goo
00<007ooOol02Goo00<007ooOol0=Woo00<007ooOol04goo000NOol00`00Oomoo`02Ool00`00Oomo
o`03Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomo
o`04Ool00`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol01Goo00<0
07ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<0
07ooOol01Goo00<007ooOol017oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<0
07ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo0P001Woo00D007ooOomo
o`0000=oo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00Eoo`03001o
ogoo00Eoo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo01=oo`008goo00<007ooOol00goo
nP005Goo000SOol00`00Oomoo`3oOolCOol0021oo`<00?mooaIoo`008goo00<007ooOol0ogoo4goo
000SOol00`00Oomoo`3oOolCOol001ioo`03001oogoo009oo`03001oogoo0?mooa=oo`007goo1000
A7oo1000cGoo001YOol00`00Oomoo`3<Ool006Uoo`03001oogoo0<aoo`00FWoo1P001Goo0`0000=o
o`00Ool00goo0`0000=oo`0000000Woo0`0000=oo`0000000Woo1P001Goo10000goo1@003Woo0`00
00=oo`0000000Woo0`000Woo0`000goo0`0000=oo`00000037oo10001Goo0`0000Aoo`00001oo`80
0004Ool000000005Ool40003Ool50019Ool005Yoo`03001oogoo00=oo`05001oogooOol00003Ool2
0003Ool01000Oomoogoo0P000Woo00@007ooOomoo`8000Eoo`03001oogoo00Eoo`03001oogoo009o
o`04001oogoo0004Ool00`00Oomoo`0:Ool01000Oomoogoo0P0017oo00<007ooOol00Woo00D007oo
Oomoo`0000=oo`8000aoo`03001oogoo009oo`05001oogooOol00003Ool20003Ool20003Ool01@00
Oomoogoo000017oo00@007ooOol000Aoo`03001oogoo04Ioo`00FWoo00<007ooOol00goo00@007oo
Ool000Eoo`05001oogooOol00004Ool01000Oomoo`0017oo00<007ooOol00goo00<007ooOol017oo
00<007ooOol02goo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo
00@007ooOol000Eoo`03001oogoo00Uoo`03001oogoo00Moo`03001oogoo009oo`05001oogooOol0
0005Ool00`00Ool0000=Ool00`00Oomoo`16Ool0061oo`04001oogoo0005Ool01@00Oomoogoo0000
17oo00@007ooOomoo`D000Eoo`03001oogoo00Aoo`L000=oo`@000ioo`D000Aoo`03001oogoo009o
o`04001oogoo0005Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00Oomoo`02Ool01@00Oomoogoo
00001Goo00<007oo00001P000goo1000BGoo001KOol50003Ool00`00Oomoo`03Ool01@00Oomoogoo
000017oo00<007ooOol01Goo00<007ooOol00goo00<007ooOol017oo00<007ooOol00goo00@007oo
Ool001Ioo`03001oogoo009oo`03001oogoo009oo`04001oogoo0005Ool00`00Oomoo`09Ool00`00
Oomoo`07Ool00`00Oomoo`02Ool01@00Oomoogoo00001Goo00<007oo00001Goo00@007ooOol004eo
o`00FWoo00<007ooOol01goo00@007ooOomoo`8000=oo`03001oogoo009oo`03001oogoo00Eoo`03
001oogoo00=oo`8000Ioo`05001oogooOol00003Ool00`00Oomoo`02Ool00`00Oomoo`0?Ool00`00
Oomoo`02Ool20003Ool01@00Oomoogoo00000goo0P002goo00<007ooOol01goo00<007ooOol00Woo
00@007ooOomoo`8000=oo`05001oogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol0AWoo
001JOol00`00Oomoo`03Ool00`00Oomoo`02Ool3000017oo000007oo0P000Woo0`000goo100017oo
0`0000=oo`0000000P000goo0`001Goo1@003Goo100017oo0P0000Aoo`00000000Eoo`<00003Ool0
07oo00Yoo`03001oogoo00=oo`03001oogoo0080009oo`<000=oo`03001oo`00008000Eoo`<000Eo
o`D004Qoo`00FWoo00<007ooOol00goo00<007ooOol0Egoo00<007ooOol02Woo00<007ooOol00Woo
00<007ooOol02Woo00<007ooOol0G7oo001KOol6001IOol00`00Oomoo`0;Ool5000<Ool00`00Oomo
o`1LOol00;Uoo`8001eoo`8005ioo`00ogoo>Goo003oOoliOol00?moocUoo`00\
\>"],
  ImageRangeCache->{{{0, 311}, {191.75, 0}} -> {-4.19558, -3.64728, 
  0.0379201, 0.0584341}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot[{Sqrt[Abs[x]], 0.5x^2}, {x, -2, 2}, PlotRange -> All,
AspectRatio -> 1, Axes -> None, PlotPoints -> 150];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  PageBreakWithin->True,
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.0238095 0.47619 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.02381 .69724 m
.03003 .69283 L
.03682 .68798 L
.04319 .6834 L
.04932 .67896 L
.05585 .6742 L
.06213 .66958 L
.0688 .66464 L
.07524 .65984 L
.08142 .65519 L
.08801 .65021 L
.09434 .64537 L
.10044 .64068 L
.10693 .63565 L
.11317 .63077 L
.11981 .62554 L
.12621 .62046 L
.13236 .61553 L
.13891 .61024 L
.14521 .6051 L
.15191 .59958 L
.15836 .59422 L
.16457 .58901 L
.17118 .58342 L
.17754 .57798 L
.18365 .5727 L
.19016 .56702 L
.19643 .5615 L
.20309 .55557 L
.20951 .54979 L
.21569 .54417 L
.22225 .53812 L
.22858 .53223 L
.23466 .52651 L
.24114 .52033 L
.24737 .51432 L
.25399 .50785 L
.26038 .50153 L
.26651 .49537 L
.27305 .48872 L
.27934 .48224 L
.28602 .47524 L
.29246 .46839 L
.29866 .46171 L
.30525 .45448 L
.3116 .44741 L
.3177 .44049 L
.3242 .433 L
.33045 .42565 L
.3371 .41769 L
Mistroke
.3435 .40987 L
.34966 .4022 L
.35622 .39386 L
.36253 .38565 L
.36924 .37671 L
.3757 .36788 L
.38192 .35916 L
.38853 .34964 L
.3949 .34019 L
.40102 .33084 L
.40754 .32055 L
.41382 .3103 L
.42049 .29899 L
.42691 .28764 L
.4331 .27623 L
.43967 .2635 L
.44601 .25057 L
.4521 .23741 L
.45858 .22242 L
.46482 .20685 L
.47145 .18869 L
.47785 .16907 L
.48399 .14728 L
.48742 .13326 L
.49053 .11876 L
.49381 .10057 L
.49683 .07874 L
.49834 .06361 L
.49909 .05317 L
.4993 .04958 L
.49953 .04507 L
.49974 .03967 L
.49993 .0321 L
.50014 .03515 L
.50024 .03892 L
.50036 .04221 L
.50075 .05056 L
.50121 .05781 L
.50165 .06343 L
.50328 .0797 L
.50653 .10265 L
.50951 .11896 L
.51588 .1468 L
.52202 .16861 L
.52854 .18869 L
.53483 .20594 L
.54151 .22263 L
.54794 .23749 L
.55413 .25087 L
.56072 .26428 L
Mistroke
.56706 .27653 L
.57316 .28777 L
.57965 .29923 L
.5859 .30983 L
.59254 .32069 L
.59894 .33078 L
.6051 .34018 L
.61165 .34989 L
.61795 .35897 L
.62465 .36836 L
.63111 .37717 L
.63732 .38545 L
.64393 .39405 L
.65029 .40214 L
.65641 .40977 L
.66293 .41773 L
.6692 .42523 L
.67586 .43306 L
.68229 .44047 L
.68846 .44747 L
.69503 .45479 L
.70136 .46173 L
.70745 .4683 L
.71393 .47518 L
.72016 .48171 L
.72679 .48856 L
.73318 .49505 L
.73932 .50122 L
.74585 .5077 L
.75215 .51385 L
.75883 .52031 L
.76528 .52645 L
.77148 .53229 L
.77807 .53842 L
.78442 .54427 L
.79053 .54982 L
.79703 .55568 L
.80328 .56125 L
.80994 .56711 L
.81634 .5727 L
.82251 .57802 L
.82906 .58363 L
.83538 .58897 L
.84209 .5946 L
.84855 .59997 L
.85478 .60508 L
.86139 .61048 L
.86776 .61563 L
.87389 .62054 L
.88041 .62572 L
Mistroke
.88669 .63067 L
.89337 .63588 L
.8998 .64087 L
.90598 .64562 L
.91256 .65064 L
.9189 .65544 L
.92499 .66001 L
.93148 .66485 L
.93772 .66947 L
.94436 .67435 L
.95075 .67901 L
.9569 .68347 L
.96345 .68817 L
.96975 .69267 L
.97619 .69724 L
Mfstroke
.02381 .97619 m
.03003 .95146 L
.03682 .92486 L
.04319 .90023 L
.04932 .87687 L
.05585 .85235 L
.06213 .82908 L
.0688 .80471 L
.07524 .78159 L
.08142 .75967 L
.08801 .73671 L
.09434 .71495 L
.10044 .69433 L
.10693 .67273 L
.11317 .65228 L
.11981 .63088 L
.12621 .61063 L
.13236 .59147 L
.13891 .57144 L
.14521 .55248 L
.15191 .53271 L
.15836 .51402 L
.16457 .49636 L
.17118 .47793 L
.17754 .46054 L
.18365 .44413 L
.19016 .427 L
.19643 .41085 L
.20309 .39405 L
.20951 .37822 L
.21569 .36331 L
.22225 .34781 L
.22858 .33322 L
.23466 .31951 L
.24114 .30525 L
.24737 .29187 L
.25399 .27799 L
.26038 .26497 L
.26651 .25277 L
.27305 .24014 L
.27934 .22832 L
.28602 .21611 L
.29246 .20471 L
.29866 .19407 L
.30525 .18311 L
.3116 .17289 L
.3177 .16339 L
.3242 .15362 L
.33045 .14455 L
.3371 .13527 L
Mistroke
.3435 .12667 L
.34966 .11874 L
.35622 .11064 L
.36253 .10318 L
.36924 .09563 L
.3757 .0887 L
.38192 .08237 L
.38853 .076 L
.3949 .07021 L
.40102 .06496 L
.40754 .05971 L
.41382 .05501 L
.42049 .05036 L
.42691 .04624 L
.4331 .04261 L
.43967 .03909 L
.44601 .03605 L
.4521 .03345 L
.45858 .03102 L
.46482 .02901 L
.47145 .02723 L
.47785 .02587 L
.48399 .02489 L
.48716 .0245 L
.49053 .02419 L
.49219 .02407 L
.49398 .02396 L
.4948 .02392 L
.49567 .02389 L
.49648 .02386 L
.49723 .02384 L
.4976 .02383 L
.49801 .02383 L
.49836 .02382 L
.49875 .02382 L
.49895 .02381 L
.49916 .02381 L
.49928 .02381 L
.49939 .02381 L
.4996 .02381 L
.49978 .02381 L
.49998 .02381 L
.50019 .02381 L
.50039 .02381 L
.50057 .02381 L
.50077 .02381 L
.50113 .02381 L
.50149 .02382 L
.50181 .02382 L
.50255 .02384 L
Mistroke
.50334 .02386 L
.50485 .02391 L
.50649 .02399 L
.50943 .02418 L
.51284 .0245 L
.51608 .0249 L
.52223 .02589 L
.52878 .02729 L
.53509 .02898 L
.54179 .03114 L
.54825 .03359 L
.55446 .03627 L
.56107 .03947 L
.56743 .04291 L
.57355 .04653 L
.58007 .05073 L
.58634 .05512 L
.593 .06014 L
.59942 .06533 L
.6056 .07064 L
.61217 .07666 L
.6185 .08279 L
.62458 .089 L
.63106 .09595 L
.6373 .10298 L
.64393 .11081 L
.65031 .1187 L
.65645 .12662 L
.66299 .13539 L
.66928 .14417 L
.67597 .15386 L
.68241 .16356 L
.68861 .17322 L
.69521 .18385 L
.70156 .19444 L
.70766 .20493 L
.71416 .21645 L
.72042 .22787 L
.72707 .24037 L
.73348 .25276 L
.73964 .26501 L
.7462 .27839 L
.75252 .29162 L
.75923 .30604 L
.76569 .32029 L
.77191 .33434 L
.77853 .34964 L
.7849 .36472 L
.79103 .37954 L
.79755 .39566 L
Mistroke
.80383 .41152 L
.8105 .42874 L
.81693 .44569 L
.82312 .46231 L
.8297 .48036 L
.83604 .49807 L
.84213 .51543 L
.84862 .53425 L
.85486 .55269 L
.8615 .57267 L
.86789 .59225 L
.87404 .61142 L
.88059 .63216 L
.88689 .65247 L
.89358 .67442 L
.90003 .69592 L
.90624 .71694 L
.91284 .73966 L
.9192 .76188 L
.92532 .78357 L
.93183 .807 L
.93809 .82989 L
.94475 .85459 L
.95117 .87873 L
.95734 .90228 L
.96391 .92769 L
.97023 .95249 L
.97619 .97619 L
Mfstroke
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{129.438, 129.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000320000`Q000`40O003h00O`Woo0032Ool00<9o
o`00`Woo0032Ool005aoo`X005aoo`00F7oo10002Woo0`00FGoo001EOol30008Ool00`00Oomoo`06
Ool3001FOol005Aoo`03001oogoo00Uoo`03001oogoo00Uoo`03001oogoo05=oo`00DWoo0P0037oo
00<007ooOol02Woo0`00DWoo001@Ool2000>Ool00`00Oomoo`0=Ool00`00Oomoo`1?Ool004moo`03
001oogoo00ioo`03001oogoo00ioo`03001oogoo04ioo`00CWoo00<007ooOol03goo00<007ooOol0
3goo00<007ooOol0CGoo001=Ool00`00Oomoo`0@Ool00`00Oomoo`0@Ool00`00Oomoo`1<Ool004ao
o`03001oogoo015oo`03001oogoo015oo`8004aoo`00BWoo0P004goo00<007oo000057oo00<007oo
Ool0BGoo0019Ool00`00Oomoo`0COol00`00Ool0000EOol00`00Oomoo`18Ool004Uoo`03001oogoo
01=oo`03001oo`0001Eoo`03001oogoo04Qoo`00B7oo00<007ooOol057oo00<007oo00005Woo00<0
07ooOol0Agoo0017Ool00`00Oomoo`0EOol00`00Ool0000GOol20017Ool004Ioo`03001oogoo01Io
o`03001oo`0001Uoo`03001oogoo04Aoo`00AGoo00<007ooOol05goo00<007oo00006Goo00<007oo
Ool0A7oo0014Ool00`00Oomoo`0GOol01@00Oomoogoo00006Goo00<007ooOol0@goo0013Ool00`00
Oomoo`0HOol01@00Oomoogoo00006Woo00<007ooOol0@Woo0013Ool00`00Oomoo`0HOol01@00Oomo
ogoo00006Woo00<007ooOol0@Woo0012Ool00`00Oomoo`0IOol01@00Oomoogoo00006goo00<007oo
Ool0@Goo0011Ool00`00Oomoo`0JOol01@00Oomoogoo000077oo00<007ooOol0@7oo0010Ool00`00
Oomoo`0KOol00`00Oomoo`02Ool00`00Oomoo`0JOol00`00Oomoo`0oOol003moo`03001oogoo01]o
o`03001oogoo00=oo`03001oogoo01]oo`03001oogoo03ioo`00?goo00<007ooOol06goo00<007oo
Ool00goo00<007ooOol06goo00<007ooOol0?Woo000nOol00`00Oomoo`0LOol00`00Oomoo`03Ool0
0`00Oomoo`0LOol00`00Oomoo`0mOol003ioo`03001oogoo01aoo`03001oogoo00=oo`03001oogoo
01aoo`03001oogoo03eoo`00?Goo00<007ooOol077oo00<007ooOol01Goo00<007ooOol077oo00<0
07ooOol0?7oo000mOol00`00Oomoo`0LOol00`00Oomoo`05Ool00`00Oomoo`0LOol00`00Oomoo`0l
Ool003aoo`03001oogoo01eoo`03001oogoo00Eoo`03001oogoo01eoo`03001oogoo03]oo`00>goo
00<007ooOol07Woo00<007ooOol01Woo00<007ooOol07Goo00<007ooOol0>Woo000jOol00`00Oomo
o`0NOol00`00Oomoo`07Ool00`00Oomoo`0NOol00`00Oomoo`0iOol003Yoo`03001oogoo01ioo`03
001oogoo00Qoo`03001oogoo01eoo`03001oogoo03Uoo`00>Goo00<007ooOol07goo00<007ooOol0
27oo00<007ooOol07Woo00<007ooOol0>7oo000iOol00`00Oomoo`0NOol00`00Oomoo`0:Ool00`00
Oomoo`0MOol00`00Oomoo`0hOol003Qoo`03001oogoo01ioo`03001oogoo00]oo`03001oogoo01io
o`03001oogoo03Moo`00>7oo00<007ooOol07Woo00<007ooOol02goo00<007ooOol07Woo00<007oo
Ool0=goo000gOol00`00Oomoo`0NOol00`00Oomoo`0=Ool00`00Oomoo`0NOol00`00Oomoo`0fOol0
03Ioo`03001oogoo01moo`03001oogoo00eoo`03001oogoo01moo`03001oogoo03Eoo`00=Goo00<0
07ooOol087oo00<007ooOol03Goo00<007ooOol087oo00<007ooOol0=7oo000eOol00`00Oomoo`0O
Ool00`00Oomoo`0?Ool00`00Oomoo`0OOol00`00Oomoo`0dOol003Aoo`03001oogoo021oo`03001o
ogoo00moo`03001oogoo021oo`03001oogoo03=oo`00=7oo00<007ooOol087oo00<007ooOol03goo
00<007ooOol087oo00<007ooOol0<goo000dOol00`00Oomoo`0OOol00`00Oomoo`0AOol00`00Oomo
o`0OOol00`00Oomoo`0cOol003=oo`03001oogoo021oo`03001oogoo019oo`03001oogoo01moo`03
001oogoo039oo`00<goo00<007ooOol07goo00<007ooOol04goo00<007ooOol07goo00<007ooOol0
<Woo000bOol00`00Oomoo`0OOol00`00Oomoo`0EOol00`00Oomoo`0OOol00`00Oomoo`0aOol0039o
o`03001oogoo01moo`03001oogoo01Eoo`03001oogoo01moo`03001oogoo035oo`00<Woo00<007oo
Ool07Woo00<007ooOol05goo00<007ooOol07goo00<007ooOol0<7oo000aOol00`00Oomoo`0OOol0
0`00Oomoo`0GOol00`00Oomoo`0POol00`00Oomoo`0_Ool0035oo`03001oogoo01moo`03001oogoo
01Moo`03001oogoo021oo`03001oogoo02moo`00<7oo00<007ooOol07goo00<007ooOol06Goo00<0
07ooOol087oo00<007ooOol0;Woo000_Ool00`00Oomoo`0POol00`00Oomoo`0IOol00`00Oomoo`0P
Ool00`00Oomoo`0^Ool002moo`03001oogoo01moo`03001oogoo01]oo`03001oogoo01moo`03001o
ogoo02ioo`00;Woo00<007ooOol07goo00<007ooOol07Goo00<007ooOol07goo00<007ooOol0;Goo
000^Ool00`00Oomoo`0NOol00`00Oomoo`0OOol00`00Oomoo`0NOol00`00Oomoo`0]Ool002eoo`03
001oogoo01moo`03001oogoo01moo`03001oogoo01moo`03001oogoo02aoo`00;Goo00<007ooOol0
7Woo00<007ooOol08Goo00<007ooOol07Woo00<007ooOol0;7oo000]Ool00`00Oomoo`0NOol00`00
Oomoo`0QOol00`00Oomoo`0NOol00`00Oomoo`0/Ool002aoo`03001oogoo01ioo`03001oogoo02=o
o`03001oogoo01ioo`03001oogoo02]oo`00;7oo00<007ooOol07Woo00<007ooOol08goo00<007oo
Ool07goo00<007ooOol0:Woo000/Ool00`00Oomoo`0MOol00`00Oomoo`0UOol00`00Oomoo`0NOol0
0`00Oomoo`0ZOol002]oo`03001oogoo01ioo`03001oogoo02Ioo`03001oogoo01ioo`03001oogoo
02Uoo`00:Woo00<007ooOol07Woo00<007ooOol0:7oo00<007ooOol07Goo00<007ooOol0:Goo000Z
Ool00`00Oomoo`0MOol00`00Oomoo`0YOol00`00Oomoo`0MOol00`00Oomoo`0YOol002Uoo`03001o
ogoo01eoo`03001oogoo02]oo`03001oogoo01eoo`03001oogoo02Qoo`00:Goo00<007ooOol077oo
00<007ooOol0;7oo00<007ooOol07Goo00<007ooOol0:7oo000YOol00`00Oomoo`0LOol00`00Oomo
o`0]Ool00`00Oomoo`0LOol00`00Oomoo`0XOol002Qoo`03001oogoo01aoo`03001oogoo02moo`03
001oogoo01aoo`03001oogoo02Moo`00:7oo00<007ooOol077oo00<007ooOol0<7oo00<007ooOol0
6goo00<007ooOol09goo000WOol00`00Oomoo`0KOol2000dOol00`00Oomoo`0JOol00`00Oomoo`0W
Ool002Moo`03001oogoo01Yoo`03001oogoo03Eoo`03001oogoo01Yoo`03001oogoo02Ioo`009goo
00<007ooOol06Woo00<007ooOol0=Goo00<007ooOol06goo00<007ooOol09Goo000VOol00`00Oomo
o`0JOol00`00Oomoo`0gOol00`00Oomoo`0JOol00`00Oomoo`0UOol002Ioo`03001oogoo01Yoo`03
001oogoo03Moo`03001oogoo01]oo`03001oogoo02Aoo`009Goo00<007ooOol06Woo00<007ooOol0
>Goo0P006goo00<007ooOol097oo000UOol00`00Oomoo`0IOol00`00Oomoo`0lOol00`00Oomoo`0H
Ool00`00Oomoo`0TOol002Aoo`03001oogoo01Uoo`03001oogoo03ioo`03001oogoo01Qoo`03001o
ogoo02=oo`0097oo00<007ooOol067oo00<007ooOol0?goo00<007ooOol067oo00<007ooOol08goo
000TOol00`00Oomoo`0GOol00`00Oomoo`11Ool00`00Oomoo`0GOol00`00Oomoo`0SOol002=oo`03
001oogoo01Moo`03001oogoo04=oo`03001oogoo01Moo`03001oogoo029oo`008goo00<007ooOol0
5goo00<007ooOol0@goo00<007ooOol05goo00<007ooOol08Woo000SOol00`00Oomoo`0EOol20017
Ool2000GOol00`00Oomoo`0ROol0029oo`03001oogoo01Eoo`03001oogoo04Uoo`03001oogoo01Ao
o`03001oogoo029oo`008Woo00<007ooOol05Goo00<007ooOol0BGoo00<007ooOol05Goo00<007oo
Ool08Goo000ROol00`00Oomoo`0DOol00`00Oomoo`1;Ool00`00Oomoo`0DOol00`00Oomoo`0QOol0
029oo`03001oogoo01=oo`03001oogoo04eoo`03001oogoo01=oo`03001oogoo025oo`008Goo00<0
07ooOol04goo00<007ooOol0Cgoo0P0057oo00<007ooOol087oo000QOol00`00Oomoo`0BOol00`00
Oomoo`1BOol00`00Oomoo`0BOol00`00Oomoo`0OOol0025oo`03001oogoo015oo`03001oogoo05=o
o`03001oogoo019oo`03001oogoo01moo`0087oo00<007ooOol04Goo00<007ooOol0EGoo00<007oo
Ool04Goo00<007ooOol07goo000OOol00`00Oomoo`0AOol00`00Oomoo`1GOol00`00Oomoo`0AOol0
0`00Oomoo`0NOol001moo`03001oogoo011oo`03001oogoo05Uoo`80015oo`03001oogoo01ioo`00
7goo00<007ooOol03goo00<007ooOol0G7oo00<007ooOol03Woo00<007ooOol07Woo000NOol00`00
Oomoo`0?Ool00`00Oomoo`1MOol00`00Oomoo`0?Ool00`00Oomoo`0MOol001ioo`03001oogoo00io
o`03001oogoo05moo`03001oogoo00ioo`03001oogoo01eoo`007Woo00<007ooOol03Goo00<007oo
Ool0HGoo00<007ooOol03Goo00<007ooOol07Goo000MOol00`00Oomoo`0=Ool00`00Oomoo`1SOol0
0`00Oomoo`0<Ool00`00Oomoo`0MOol001eoo`03001oogoo00]oo`8006Moo`8000eoo`03001oogoo
01aoo`007Goo00<007ooOol02Woo00<007ooOol0JGoo00<007ooOol02Woo00<007ooOol077oo000M
Ool00`00Oomoo`09Ool00`00Oomoo`1[Ool00`00Oomoo`09Ool00`00Oomoo`0LOol001aoo`03001o
ogoo00Yoo`03001oogoo06]oo`03001oogoo00Yoo`03001oogoo01]oo`0077oo00<007ooOol02Goo
00<007ooOol0KGoo00<007ooOol02Woo00<007ooOol06Woo000LOol00`00Oomoo`07Ool2001aOol2
000:Ool00`00Oomoo`0JOol001]oo`03001oogoo00Moo`03001oogoo07=oo`03001oogoo00Moo`03
001oogoo01Yoo`006goo00<007ooOol01Woo00<007ooOol0MGoo00<007ooOol01goo00<007ooOol0
6Goo000JOol00`00Oomoo`06Ool00`00Oomoo`1gOol00`00Oomoo`06Ool00`00Oomoo`0IOol001Yo
o`03001oogoo00Eoo`03001oogoo07Uoo`8000Ioo`03001oogoo01Uoo`006Woo00<007ooOol00goo
0P00OWoo00<007ooOol00goo00<007ooOol06Goo000IOol00`00Oomoo`03Ool00`00Oomoo`1oOol0
0`00Oomoo`03Ool00`00Oomoo`0HOol001Uoo`03001oogoo009oo`03001oogoo085oo`03001oogoo
009oo`03001oogoo01Qoo`006Goo00D007ooOomoo`0008Eoo`80009oo`03001oogoo01Qoo`0067oo
00<007ooOol00P00R7oo00@007ooOol001Uoo`0067oo00<007oo0000Rgoo00<007oo00006Goo000H
Ool2002=Ool2000IOol001Qoo`03001oogoo08eoo`8001Qoo`005Woo0P00TGoo0P005goo000EOol0
0`00Ool0002BOol2000FOol001Aoo`04001oogoo002BOol00`00Ool0000EOol001=oo`04001oogoo
002COol01000Oomoo`0057oo000AOol20003Ool00`00Oomoo`2BOol00`00Oomoo`02000BOol0011o
o`03001oogoo009oo`03001oogoo09=oo`03001oogoo009oo`03001oogoo00moo`003goo00<007oo
Ool00goo00<007ooOol0Tgoo00<007ooOol00goo00<007ooOol03Woo000>Ool00`00Oomoo`04Ool0
0`00Oomoo`2COol00`00Oomoo`04Ool00`00Oomoo`0=Ool000eoo`03001oogoo00Aoo`03001oogoo
09Eoo`03001oogoo00Aoo`8000eoo`002goo0P001goo00<007ooOol0UGoo00<007ooOol01Woo00<0
07ooOol02Woo0009Ool20009Ool00`00Oomoo`2EOol00`00Oomoo`07Ool00`00Oomoo`09Ool000Qo
o`03001oogoo00Uoo`03001oogoo09Eoo`03001oogoo00Qoo`<000Qoo`001Woo0P002goo00<007oo
Ool0UWoo00<007ooOol02goo00<007ooOol01Goo0005Ool00`00Oomoo`0;Ool00`00Oomoo`2GOol0
0`00Oomoo`0;Ool00`00Oomoo`04Ool000Aoo`03001oogoo00aoo`03001oogoo09Moo`03001oogoo
00aoo`03001oogoo00=oo`004goo00<007ooOol0Ugoo00<007ooOol04Woo000BOol00`00Oomoo`2I
Ool00`00Oomoo`0AOol0019oo`03001oogoo09Uoo`03001oogoo015oo`004Woo00<007ooOol0VWoo
00<007ooOol047oo000AOol00`00Oomoo`2KOol00`00Oomoo`0@Ool0015oo`03001oogoo09]oo`03
001oogoo011oo`004Goo00<007ooOol0W7oo00<007ooOol03goo000@Ool00`00Oomoo`2MOol00`00
Oomoo`0?Ool0011oo`03001oogoo09eoo`03001oogoo00moo`0047oo00<007ooOol0WGoo00<007oo
Ool03goo000?Ool00`00Oomoo`2OOol00`00Oomoo`0>Ool000moo`03001oogoo09moo`03001oogoo
00ioo`003goo00<007ooOol0Wgoo00<007ooOol03Woo000?Ool00`00Oomoo`2OOol00`00Oomoo`0>
Ool000ioo`03001oogoo0:1oo`03001oogoo00ioo`003Woo00<007ooOol0XGoo00<007ooOol03Goo
000>Ool00`00Oomoo`2QOol00`00Oomoo`0=Ool000ioo`03001oogoo0:5oo`03001oogoo00eoo`00
3Goo00<007ooOol0XWoo00<007ooOol03Goo000=Ool00`00Oomoo`2SOol00`00Oomoo`0<Ool000eo
o`03001oogoo0:=oo`03001oogoo00aoo`003Goo00<007ooOol0Xgoo00<007ooOol037oo000=Ool0
0`00Oomoo`2TOol00`00Oomoo`0;Ool000aoo`03001oogoo0:Eoo`03001oogoo00]oo`0037oo00<0
07ooOol0YGoo00<007ooOol02goo000<Ool00`00Oomoo`2VOol00`00Oomoo`0:Ool000]oo`03001o
ogoo0:Moo`03001oogoo00Yoo`002goo00<007ooOol0Ygoo00<007ooOol02Woo000;Ool00`00Oomo
o`2XOol00`00Oomoo`09Ool000Yoo`03001oogoo0:Uoo`03001oogoo00Uoo`002Woo00<007ooOol0
ZGoo00<007ooOol02Goo000:Ool00`00Oomoo`2YOol00`00Oomoo`09Ool000Uoo`03001oogoo0:]o
o`03001oogoo00Qoo`002Goo00<007ooOol0Zgoo00<007ooOol027oo0009Ool00`00Oomoo`2[Ool0
0`00Oomoo`08Ool000Uoo`03001oogoo0:]oo`03001oogoo00Qoo`002Goo00<007ooOol0Zgoo00<0
07ooOol027oo0008Ool00`00Oomoo`2]Ool00`00Oomoo`07Ool000Qoo`03001oogoo0:eoo`03001o
ogoo00Moo`0027oo00<007ooOol0[Goo00<007ooOol01goo0008Ool00`00Oomoo`2]Ool00`00Oomo
o`07Ool000Moo`03001oogoo0:moo`03001oogoo00Ioo`001goo00<007ooOol0[goo00<007ooOol0
1Woo0007Ool00`00Oomoo`2_Ool00`00Oomoo`06Ool000Moo`03001oogoo0;1oo`03001oogoo00Eo
o`001Woo00<007ooOol0/Goo00<007ooOol01Goo0006Ool00`00Oomoo`2aOol00`00Oomoo`05Ool0
00Ioo`03001oogoo0;9oo`03001oogoo00Aoo`001Goo00<007ooOol0/goo00<007ooOol017oo0005
Ool00`00Oomoo`2cOol00`00Oomoo`04Ool000Eoo`03001oogoo0;=oo`03001oogoo00Aoo`0017oo
00<007ooOol0]Goo00<007ooOol00goo0004Ool00`00Oomoo`2eOol00`00Oomoo`03Ool000Aoo`03
001oogoo0;Eoo`03001oogoo00=oo`00`Woo0032Ool00<9oo`00`Woo0000\
\>"],
  ImageRangeCache->{{{0, 193.125}, {193.125, 0}} -> {-2.10002, -0.0500105, 
  0.0326217, 0.0163109}}]
}, Open  ]],

Cell["\<\
Let's look the idea of a parametric plot, where x and y are given \
as a function of a third variable, say t:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
ParametricPlot[{Sin[10 t], Sin[11 t]}, {t, 0, 2 Pi},
AspectRatio -> Automatic];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.500001 0.476191 0.500001 0.476191 [
[.02381 .4875 -6 -9 ]
[.02381 .4875 6 0 ]
[.2619 .4875 -12 -9 ]
[.2619 .4875 12 0 ]
[.7381 .4875 -9 -9 ]
[.7381 .4875 9 0 ]
[.97619 .4875 -3 -9 ]
[.97619 .4875 3 0 ]
[.4875 .02381 -12 -4.5 ]
[.4875 .02381 0 4.5 ]
[.4875 .2619 -24 -4.5 ]
[.4875 .2619 0 4.5 ]
[.4875 .7381 -18 -4.5 ]
[.4875 .7381 0 4.5 ]
[.4875 .97619 -6 -4.5 ]
[.4875 .97619 0 4.5 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .5 m
.02381 .50625 L
s
[(-1)] .02381 .4875 0 1 Mshowa
.2619 .5 m
.2619 .50625 L
s
[(-0.5)] .2619 .4875 0 1 Mshowa
.7381 .5 m
.7381 .50625 L
s
[(0.5)] .7381 .4875 0 1 Mshowa
.97619 .5 m
.97619 .50625 L
s
[(1)] .97619 .4875 0 1 Mshowa
.125 Mabswid
.07143 .5 m
.07143 .50375 L
s
.11905 .5 m
.11905 .50375 L
s
.16667 .5 m
.16667 .50375 L
s
.21429 .5 m
.21429 .50375 L
s
.30952 .5 m
.30952 .50375 L
s
.35714 .5 m
.35714 .50375 L
s
.40476 .5 m
.40476 .50375 L
s
.45238 .5 m
.45238 .50375 L
s
.54762 .5 m
.54762 .50375 L
s
.59524 .5 m
.59524 .50375 L
s
.64286 .5 m
.64286 .50375 L
s
.69048 .5 m
.69048 .50375 L
s
.78572 .5 m
.78572 .50375 L
s
.83333 .5 m
.83333 .50375 L
s
.88095 .5 m
.88095 .50375 L
s
.92857 .5 m
.92857 .50375 L
s
.25 Mabswid
0 .5 m
1 .5 L
s
.5 .02381 m
.50625 .02381 L
s
[(-1)] .4875 .02381 1 0 Mshowa
.5 .2619 m
.50625 .2619 L
s
[(-0.5)] .4875 .2619 1 0 Mshowa
.5 .7381 m
.50625 .7381 L
s
[(0.5)] .4875 .7381 1 0 Mshowa
.5 .97619 m
.50625 .97619 L
s
[(1)] .4875 .97619 1 0 Mshowa
.125 Mabswid
.5 .07143 m
.50375 .07143 L
s
.5 .11905 m
.50375 .11905 L
s
.5 .16667 m
.50375 .16667 L
s
.5 .21429 m
.50375 .21429 L
s
.5 .30952 m
.50375 .30952 L
s
.5 .35714 m
.50375 .35714 L
s
.5 .40476 m
.50375 .40476 L
s
.5 .45238 m
.50375 .45238 L
s
.5 .54762 m
.50375 .54762 L
s
.5 .59524 m
.50375 .59524 L
s
.5 .64286 m
.50375 .64286 L
s
.5 .69048 m
.50375 .69048 L
s
.5 .78572 m
.50375 .78572 L
s
.5 .83333 m
.50375 .83333 L
s
.5 .88095 m
.50375 .88095 L
s
.5 .92857 m
.50375 .92857 L
s
.25 Mabswid
.5 0 m
.5 1 L
s
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.5 Mabswid
.5 .5 m
.7658 .78872 L
.88224 .90701 L
.91998 .94172 L
.9505 .96613 L
.95664 .97017 L
.96226 .97333 L
.96997 .97609 L
.97443 .97527 L
.97619 .97056 L
.9745 .96141 L
.97198 .95442 L
.96867 .94686 L
.94777 .90867 L
.766 .65782 L
.45783 .30945 L
.29345 .15587 L
.21324 .09298 L
.17778 .06898 L
.14783 .05123 L
.12207 .0384 L
.10951 .03321 L
.0969 .0289 L
.08517 .02588 L
.07508 .02427 L
.06459 .02385 L
.05603 .02479 L
.04339 .02949 L
.03761 .03385 L
.03315 .03897 L
.02721 .05102 L
.02412 .06751 L
.02421 .0859 L
.02747 .10905 L
.03352 .13389 L
.04159 .15906 L
.06933 .22539 L
.11142 .30512 L
.34804 .6275 L
.51317 .79149 L
.59194 .85524 L
.66236 .90352 L
.72545 .93861 L
.7559 .95226 L
.78705 .96357 L
.81641 .97133 L
.84206 .97531 L
.85678 .97616 L
.87008 .97587 L
.89313 .97243 L
Mistroke
.91115 .96635 L
.92806 .95677 L
.9418 .94487 L
.95231 .93173 L
.96231 .91345 L
.97012 .89045 L
.97458 .86581 L
.97616 .84082 L
.97494 .80882 L
.97073 .77614 L
.96443 .74501 L
.95494 .7095 L
.92392 .6248 L
.88398 .54245 L
.7869 .39088 L
.65769 .24255 L
.5835 .1759 L
.50125 .11557 L
.42354 .07141 L
.38964 .05622 L
.35412 .04313 L
.31556 .03245 L
.28096 .02631 L
.26046 .02437 L
.24166 .02381 L
.20803 .02608 L
.19157 .02892 L
.17483 .03316 L
.14666 .04388 L
.13207 .05154 L
.11917 .05979 L
.09259 .08233 L
.07357 .1049 L
.05656 .1325 L
.04476 .15881 L
.03473 .19059 L
.03053 .20946 L
.02756 .22744 L
.02529 .24788 L
.02402 .27012 L
.02449 .30909 L
.02621 .33191 L
.02875 .35356 L
.03626 .39648 L
.04642 .43786 L
.07323 .51637 L
.11283 .60034 L
.16718 .68778 L
.28652 .82294 L
.35517 .87791 L
Mistroke
.39636 .90459 L
.43475 .92555 L
.46786 .94071 L
.50415 .95427 L
.54246 .96513 L
.57861 .97207 L
.61188 .97552 L
.6467 .97597 L
.67878 .97336 L
.70706 .96844 L
.74043 .95919 L
.77021 .94739 L
.80116 .93104 L
.83212 .90969 L
.86187 .88327 L
.88726 .85468 L
.90783 .82591 L
.92691 .79269 L
.94099 .76204 L
.95386 .72632 L
.96309 .69232 L
.96954 .65931 L
.97436 .61943 L
.97614 .58208 L
.97517 .54047 L
.97086 .49634 L
.963 .45128 L
.95287 .41073 L
.93774 .36475 L
.92019 .32259 L
.88148 .25162 L
.82967 .18257 L
.76226 .1185 L
.72486 .09181 L
.6816 .06723 L
.64322 .05044 L
.62161 .04289 L
.60086 .03687 L
.56008 .02841 L
.52243 .02445 L
.48312 .02417 L
.46103 .02574 L
.44081 .02828 L
.4039 .03566 L
.36561 .0472 L
.33199 .06078 L
.30149 .07609 L
.23599 .11989 L
.17246 .18016 L
.12289 .24487 L
Mistroke
.0993 .28401 L
.07704 .32868 L
.05952 .37223 L
.04617 .41412 L
.03717 .45052 L
.03004 .49 L
.02526 .5339 L
.02381 .57383 L
.02527 .61469 L
.0303 .65829 L
.0383 .69815 L
.04852 .73373 L
.06136 .76763 L
.07578 .79786 L
.10853 .84926 L
.13002 .87482 L
.15165 .89609 L
.17691 .91649 L
.20645 .93548 L
.23862 .95126 L
.27077 .96274 L
.30344 .97056 L
.33471 .97483 L
.36934 .97618 L
.40845 .97375 L
.42837 .97098 L
.44999 .96685 L
.48936 .95637 L
.52398 .94404 L
.56171 .9273 L
.63632 .88379 L
.70234 .83282 L
.75981 .77739 L
.8262 .69702 L
.87951 .61419 L
.91835 .53644 L
.94975 .45107 L
.96137 .40722 L
.96656 .38234 L
.97034 .35991 L
.97331 .33662 L
.97535 .31208 L
.97615 .28986 L
.97596 .26926 L
.97303 .23188 L
.96734 .19924 L
.95786 .16596 L
.9511 .14862 L
.94409 .13375 L
.92902 .10855 L
Mistroke
.91252 .08775 L
.89181 .06808 L
.86675 .05086 L
.85309 .04376 L
.83747 .03724 L
.80715 .02866 L
.77543 .02441 L
.74436 .02411 L
.71411 .02697 L
.68471 .03239 L
.65148 .04134 L
.61476 .05443 L
.53792 .09172 L
.4666 .13749 L
.31885 .2662 L
.17892 .43898 L
.12213 .5313 L
.07452 .62845 L
.05675 .6739 L
.0434 .71515 L
.0343 .75055 L
.02761 .78682 L
.02429 .81972 L
.02402 .84752 L
.02699 .87669 L
.03302 .90126 L
.04134 .92105 L
.05135 .93683 L
.06483 .95119 L
.08195 .96298 L
.10176 .97105 L
.11407 .97396 L
.12607 .97557 L
.14988 .97592 L
.17721 .97259 L
.19259 .96928 L
.20968 .96458 L
.242 .95311 L
.32164 .91321 L
.40325 .85881 L
.49524 .78412 L
.66034 .61807 L
.80129 .44052 L
.90199 .28128 L
.93637 .21314 L
.9595 .15595 L
.96807 .12815 L
.97137 .11462 L
.97395 .10114 L
.97612 .07983 L
Mistroke
.97541 .06295 L
.97364 .05388 L
.97111 .04656 L
.96734 .03967 L
.96225 .03372 L
.95114 .02684 L
.94345 .02469 L
.93537 .02385 L
.91802 .02526 L
.89973 .02995 L
.87583 .03934 L
.85058 .0521 L
.72082 .14396 L
.57719 .27453 L
.24703 .64159 L
.0648 .88844 L
.03927 .9312 L
.03043 .94881 L
.02551 .96137 L
.02386 .96912 L
.02471 .97428 L
.02881 .97619 L
.03539 .97443 L
.0404 .97192 L
.04666 .96806 L
.06031 .95812 L
.10212 .92177 L
.21233 .81166 L
.50452 .49502 L
.78143 .19486 L
.89428 .08159 L
.93358 .04677 L
.94829 .03542 L
.959 .02843 L
.96725 .02457 L
.97258 .02392 L
.97564 .02638 L
.97594 .03264 L
.97299 .04299 L
.97025 .04967 L
.9663 .05809 L
.94535 .09533 L
.78133 .32289 L
.48528 .66247 L
.33364 .80921 L
.20879 .91018 L
.14985 .94766 L
.12376 .96085 L
.11021 .96652 L
.09815 .97072 L
Mistroke
.08616 .97391 L
.07604 .97562 L
.0661 .97619 L
.05639 .97528 L
.04895 .9732 L
.04205 .96966 L
.03653 .96509 L
.03238 .95991 L
.02645 .94643 L
.02399 .9306 L
.02462 .90957 L
.02625 .8976 L
.02907 .88341 L
.03617 .85723 L
.04707 .8261 L
.07502 .76284 L
.17379 .59545 L
.30045 .42758 L
.45925 .2576 L
.54685 .18007 L
.63948 .11119 L
.71577 .06621 L
.74728 .05137 L
.78014 .03869 L
.8095 .03021 L
.83513 .02548 L
.86024 .02381 L
.8847 .02583 L
.89719 .02865 L
.90796 .03232 L
.91895 .03751 L
.92959 .04434 L
.94512 .05881 L
.95259 .06868 L
.95867 .07897 L
.96761 .10072 L
.97304 .12344 L
.976 .15285 L
.97551 .18369 L
.97181 .21714 L
.96832 .23702 L
.96422 .2559 L
.94119 .33181 L
.91031 .40545 L
.8682 .4858 L
.75381 .65165 L
.67552 .7396 L
.5989 .81124 L
.53045 .86461 L
.45536 .91205 L
Mistroke
.41449 .9329 L
.37727 .94868 L
.34242 .96051 L
.30644 .9695 L
.27683 .97419 L
.24613 .97617 L
.21844 .97511 L
.19429 .97163 L
.16898 .96501 L
.14326 .95448 L
.11873 .9399 L
.0982 .92317 L
.07809 .9011 L
.06008 .874 L
.04748 .848 L
.03666 .81661 L
.032 .79776 L
.02865 .77986 L
.02587 .75842 L
.02431 .73736 L
.02415 .69845 L
.02552 .67568 L
.02775 .65417 L
.03474 .61093 L
.04445 .56936 L
.07048 .49054 L
.10943 .40601 L
.16299 .31819 L
.2165 .24901 L
.28224 .18096 L
.35698 .12083 L
.39437 .0966 L
.43037 .07665 L
.46407 .06089 L
.50086 .04683 L
.53429 .03688 L
.56501 .03016 L
.59789 .02559 L
.63355 .02381 L
.66994 .02562 L
.70303 .0307 L
.73769 .0399 L
.75638 .04669 L
.77287 .05384 L
.80395 .07066 L
.83074 .08924 L
.85968 .11456 L
.88783 .14604 L
.90972 .17705 L
.92968 .21283 L
Mistroke
.94427 .24619 L
.95695 .28399 L
.96604 .32133 L
.97183 .35654 L
.97556 .39893 L
.97619 .42328 L
.97582 .44601 L
.97278 .48798 L
.9662 .53267 L
.95759 .57181 L
.94743 .60727 L
.9168 .68464 L
.88144 .74845 L
.8365 .80952 L
.77905 .86765 L
.71927 .91173 L
.68266 .93224 L
.64824 .94761 L
.62879 .95475 L
.60743 .96135 L
.56811 .97027 L
.52964 .97507 L
.49431 .97615 L
.45358 .97345 L
.41523 .96699 L
.3804 .95774 L
.34869 .94639 L
.31475 .93093 L
.27979 .91115 L
.24303 .88562 L
.20563 .85387 L
.14504 .78649 L
.09644 .71074 L
.06201 .63458 L
.0484 .59365 L
.03707 .54902 L
.02953 .50648 L
.02537 .46769 L
.02382 .42244 L
.02453 .39718 L
.02621 .3741 L
.03183 .3326 L
.0359 .31216 L
.04134 .2901 L
.05512 .24779 L
.07107 .21132 L
.09212 .17417 L
.11805 .13878 L
.14192 .11299 L
.16989 .08875 L
Mistroke
.20332 .06631 L
.2367 .04955 L
.26994 .03751 L
.28952 .03233 L
.30823 .02859 L
.32834 .0258 L
.34701 .0243 L
.3689 .02382 L
.38993 .0246 L
.42764 .0289 L
.46299 .03619 L
.50201 .0478 L
.54349 .06418 L
.62751 .11033 L
.70277 .16756 L
.77863 .24339 L
.84973 .33701 L
.89707 .4185 L
.93706 .51055 L
.95214 .55707 L
.95891 .58243 L
.9643 .60618 L
.96842 .62807 L
.97192 .65148 L
.97554 .69141 L
.97605 .72738 L
.9734 .76511 L
.97079 .78326 L
.96704 .80209 L
.95782 .83415 L
.94558 .86329 L
.93119 .88826 L
.91604 .90827 L
.89721 .92734 L
.88564 .93674 L
.87265 .94561 L
.84752 .95875 L
.81795 .96884 L
.80008 .97267 L
.78275 .97499 L
.74984 .97609 L
.71832 .9736 L
.70033 .9708 L
.68038 .96661 L
.64316 .95598 L
.60921 .94331 L
.57107 .92594 L
.49267 .88045 L
.3538 .76857 L
.20967 .60449 L
Mistroke
.1426 .50414 L
.09275 .4119 L
.05937 .33332 L
.04501 .29035 L
.03512 .25307 L
.02797 .21568 L
.02553 .1962 L
.0242 .17857 L
.02417 .14913 L
.02753 .12026 L
.03393 .0961 L
.04247 .07685 L
.05546 .05815 L
.07081 .04404 L
.08748 .03427 L
.10499 .02805 L
.12641 .0244 L
.15159 .02418 L
.16708 .02577 L
.18195 .02833 L
.19894 .03234 L
.21758 .03793 L
.2505 .05041 L
.28856 .06843 L
.43656 .1667 L
.60092 .31745 L
.76298 .50727 L
.88335 .68587 L
.92167 .75617 L
.95263 .82531 L
.96291 .85433 L
.97074 .88255 L
.97499 .90612 L
.97619 .9249 L
.97446 .94269 L
.97204 .95114 L
.96886 .95791 L
.96075 .96759 L
.94888 .97396 L
.93501 .97617 L
.91699 .97455 L
.8972 .96921 L
.87672 .96106 L
.83234 .93731 L
.77836 .90094 L
.48618 .63262 L
.17663 .26879 L
.08534 .14278 L
.05455 .09516 L
.03528 .06121 L
Mistroke
.02944 .04895 L
.02582 .0396 L
.02399 .03212 L
.02427 .02661 L
.02702 .02401 L
.0317 .02426 L
.0392 .02742 L
.04972 .03402 L
.11745 .09269 L
.21271 .18874 L
.5 .5 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{129.438, 129.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000320000`Q000`40O003h00O`Woo0032Ool00<9o
o`00HWoo00<007ooOol0GGoo001ROol00`00Oomoo`1MOol005Uoo`D000Aoo`03001oogoo05eoo`00
Fgoo00<007ooOol017oo00<007ooOol0GGoo0009Ool20003Ool60006Ool<0009Ool90008OolC0006
OolC0006OolE0007Ool<0006Ool:0006Ool70003Ool20006Ool000Uoo`03001oo`00008000Ioo`H0
00aoo`T000Uoo`P000moo`X0009oo`03001oogoo00Aoo`03001oogoo00Moo`H001Eoo`L000aoo`H0
00Yoo`H000Moo`<0009oo`03001oogoo00=oo`002Goo00@007oo001oo`8000Eoo`<0009oo`8000Uo
o`<0009oo`@000aoo`@00003Ool0000000<000aoo`D000Eoo`@000Eoo`03001oogoo00Ioo`<000=o
o`<000ioo`H000Aoo`<000Yoo`<000=oo`<000Qoo`<0009oo`8000Eoo`800005Ool007ooOol00005
Ool000Uoo`8000Aoo`03001oogoo008000Moo`8000Aoo`<000Uoo`<000Ioo`<000Yoo`<000Ioo`<0
00aoo`03001oo`000080009oo`04001oogooOol50009Ool40006Ool4000=Ool30004Ool30009Ool3
0003Ool20007Ool20002Ool00`00Oomoo`02Ool00`00Ool00005Ool000Uoo`8000Eoo`8000]oo`@0
00moo`H0011oo`H000moo`03001oogoo009oo`H0019oo`H001Aoo`@000moo`<000]oo`8000Ioo`80
00Eoo`002Goo0P001Goo0P002Woo0`0000=oo`0000003Woo1@004goo1@0037oo0`000goo0P0000Ao
o`00000001Aoo`800004Ool00000000AOol30002Ool00`00Oomoo`0;Ool20002Ool2000:Ool20006
Ool00`00Oomoo`04Ool000Uoo`8000Aoo`04001oogoo0008Ool00`00Oomoo`04Ool00`00Oomoo`08
Ool30005Ool2000>Ool30005Ool3000:Ool200000goo00000003Ool00`00Oomoo`03000?Ool20006
Ool3000<Ool20006Ool2000:Ool00`00Oomoo`04Ool20007Ool01000Oomoo`001Goo0P001Goo0009
Ool00`00Ool00002Ool00`00Oomoo`02Ool20004Ool20008Ool20006Ool2000:Ool00`00Oomoo`08
Ool3000;Ool20009Ool20005Ool00`00Oomoo`03Ool3000:Ool2000;Ool30007Ool2000:Ool20007
Ool00`00Oomoo`07Ool00`00Oomoo`03Ool00`00Oomoo`02Ool01`00Oomoogoo001oo`0000Eoo`00
2Goo00<007oo00000Woo00<007ooOol017oo00@007ooOol000aoo`04001oogooOol2000=Ool20006
Ool2000@Ool20004Ool30007Ool00`00Oomoo`06Ool20006Ool2000@Ool20003Ool2000>Ool20003
Ool2000;Ool01@00Oomoogoo00001Woo00H007ooOol007oo0005Ool000Uoo`03001oogoo008000Qo
o`8000ioo`<0015oo`80009oo`8001Aoo`@000Yoo`8000Uoo`800004Ool00000000DOol00`00Ool0
000BOol3000>Ool00`00Ool00008Ool20002Ool00`00Oomoo`03Ool000Uoo`04001oogoo0009Ool2
000>Ool3000COol2000FOol4000:Ool00`00Oomoo`09Ool3000FOol3000COol3000>Ool00`00Oomo
o`07Ool20002Ool00`00Oomoo`03Ool000Uoo`03001oogoo008000Moo`04001oogoo000<Ool01000
Oomoogoo0P0047oo00@007oo000001=oo`8000Aoo`8000Qoo`03001oogoo00Moo`8000=oo`8001=o
o`04001oogooOol2000?Ool20003Ool00`00Oomoo`09Ool200000goo001oo`07Ool20002Ool00`00
Oomoo`03Ool000Uoo`05001oo`00Ool00007Ool01@00Oomoogoo00002Woo00<007ooOol017oo00<0
07ooOol02goo0P0017oo00<007ooOol03goo00<007ooOol01Woo0P001Woo00<007ooOol01Goo0P00
1goo00<007ooOol03Woo0P001Woo00<007ooOol02goo00<007ooOol017oo00<007ooOol01goo00<0
07ooOol00Woo00<007ooOol017oo00<007oo00000Woo00<007ooOol00goo0009Ool01@00Ool007oo
00001Woo00<007ooOol00goo00<007ooOol01Woo00<007ooOol01Woo0P002Woo00<007ooOol01Goo
0P003Goo0P002goo00<007ooOol00goo00<007ooOol00goo0P002Woo0P0037oo0P002Goo0P002Woo
00<007ooOol01Woo00<007ooOol01Goo00<007ooOol00goo00<007ooOol017oo00H007ooOol007oo
0005Ool000Uoo`8000=oo`03001oogoo009oo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo
00Uoo`03001oogoo00Eoo`8000Yoo`8000Uoo`8000ioo`8000=oo`03001oogoo009oo`03001oogoo
00aoo`8000Uoo`03001oogoo00]oo`03001oogoo00Eoo`8000Yoo`03001oogoo00=oo`03001oogoo
00Eoo`03001oogoo009oo`07001oogooOol007oo00001Goo0009Ool20003Ool01@00Oomoogoo0000
2Goo00<007ooOol00Woo00<007ooOol02goo00<007ooOol00goo00<007ooOol037oo00<007ooOol0
17oo0P004Woo0P0000Aoo`00Oomoo`80015oo`8000Eoo`8000moo`8000Aoo`03001oogoo00]oo`05
001oogooOol00009Ool01@00Oomoogoo000017oo0P001Goo0009Ool20004Ool01000Oomoo`002Woo
00@007ooOol000moo`03001oogoo0080011oo`8000=oo`03001oogoo01Aoo`800003Ool007oo01Ao
o`80009oo`03001oogoo015oo`04001oogoo000?Ool00`00Ool0000;Ool00`00Ool00005Ool20005
Ool000Uoo`03001oogoo00Aoo`8000aoo`80015oo`8001Aoo`<001Qoo`<001Moo`8001Eoo`80015o
o`03001oogoo00Yoo`8000Ioo`03001oogoo00Aoo`002Goo00<007ooOol017oo0P0037oo0P004Goo
00<007oo00004Woo10005goo0`0000=oo`00Ool057oo00<007oo000057oo00<007oo000047oo0P00
37oo0P001Woo0P001Goo0009Ool20005Ool2000;Ool01000Oomoo`003goo00D007ooOomoo`00011o
o`03001oogoo009oo`03001oogoo01=oo`03001oo`0000=oo`03001oogoo015oo`04001oogooOol2
000AOol01@00Oomoogoo00003Woo00@007ooOol000]oo`03001oo`0000Eoo`8000Eoo`002Goo0P00
1Goo00<007oo00002Goo00<007ooOol00Woo00<007ooOol02goo00<007ooOol00goo00<007ooOol0
37oo00<007ooOol017oo0P004Goo0P000Woo00<007ooOol00Woo00<007ooOol03Woo0P001Woo00<0
07ooOol037oo0P001Goo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol01goo00@007ooOol0
00Eoo`8000Eoo`002Goo0P0017oo00@007ooOol000Qoo`03001oogoo00Aoo`03001oogoo00Uoo`03
001oogoo00Eoo`8000]oo`03001oogoo00Moo`03001oogoo00eoo`03001oogoo009oo`03001oogoo
00=oo`8000eoo`03001oogoo00Moo`8000]oo`03001oogoo00Ioo`03001oogoo00Qoo`03001oogoo
00Aoo`03001oogoo00Eoo`03001oogoo009oo`03001oogoo009oo`8000Eoo`002Goo0P0017oo00D0
07ooOomoo`0000Ioo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Qoo`03001oogoo00Mo
o`03001oogoo00Uoo`8000]oo`8000Eoo`03001oogoo00Eoo`03001oogoo00Qoo`8000aoo`03001o
ogoo00Moo`03001oogoo00Qoo`03001oogoo00Ioo`03001oogoo00Ioo`03001oogoo00Aoo`03001o
ogoo009oo`07001oogooOol007oo00001Goo0009Ool00`00Ool00002Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`07Ool00`00Oomoo`05Ool00`00Oomoo`0:Ool00`00Oomoo`05Ool00`00
Oomoo`0<Ool20008Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`0=
Ool00`00Oomoo`04Ool2000<Ool00`00Oomoo`04Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00
Oomoo`04Ool01P00Oomoo`00Ool000Eoo`002Goo00D007oo001oo`0000Ioo`03001oogoo009oo`03
001oogoo00Uoo`03001oogoo00=oo`03001oogoo00aoo`03001oogoo009oo`80015oo`03001oogoo
00Aoo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo00moo`8000=oo`03
001oogoo00eoo`03001oogoo009oo`03001oogoo00Uoo`05001oogooOol00007Ool01P00Oomoo`00
Ool000Eoo`002Goo00D007oo001oo`0000Moo`04001oogoo000=Ool01@00Oomoogoo000047oo0P00
00=oo`00Ool04goo0P000goo00<007ooOol01goo00<007ooOol027oo0P000Woo0P0057oo00<007oo
00004Goo00@007ooOol000eoo`03001oo`0000Qoo`03001oo`00009oo`03001oogoo00=oo`002Goo
00@007ooOol000Uoo`8000moo`03001oo`0001=oo`03001oogoo01Eoo`03001oo`0000Yoo`03001o
ogoo00Yoo`8001Moo`03001oogoo015oo`8000moo`8000Uoo`80009oo`03001oogoo00=oo`002Goo
00@007ooOol000Uoo`80011oo`03001oogoo015oo`03001oo`0001Moo`8000Yoo`8000Yoo`<001Io
o`03001oo`00019oo`8000moo`8000Uoo`80009oo`03001oogoo00=oo`002Goo00<007ooOol00P00
1goo00<007oo00003goo00<007oo00004Goo00D007ooOomoo`0001Aoo`80009oo`03001oogoo00Mo
o`03001oogoo00Qoo`05001oogooOol0000DOol01000Oomoo`004Goo00@007ooOol000eoo`04001o
ogoo0008Ool20002Ool00`00Oomoo`03Ool000Uoo`03001oogoo008000Moo`04001oogoo000=Ool0
1@00Oomoogoo00003goo00<007ooOol00goo00<007ooOol047oo00<007ooOol00goo00<007ooOol0
1Woo00<007ooOol01goo00<007ooOol00goo0P004Goo00<007ooOol00Woo00<007ooOol03Goo00<0
07ooOol00Woo00<007ooOol02Woo00D007ooOomoo`0000Moo`80009oo`03001oogoo00=oo`002Goo
00D007oo001oo`0000Ioo`03001oogoo009oo`03001oogoo00Uoo`03001oogoo00=oo`03001oogoo
00]oo`03001oogoo00Eoo`03001oogoo00ioo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo
00Eoo`8000Qoo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo00]oo`03001oogoo00Aoo`03
001oogoo00Qoo`03001oogoo009oo`03001oogoo00Aoo`03001oo`00009oo`03001oogoo00=oo`00
2Goo00<007oo00000Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol027oo00<007ooOol0
17oo00<007ooOol02Woo00<007ooOol01Woo00<007ooOol037oo00<007ooOol01goo00<007ooOol0
17oo00<007ooOol017oo00<007ooOol02Goo00<007ooOol02Woo0P0027oo00<007ooOol02Goo00<0
07ooOol01Goo00<007ooOol01goo00<007ooOol017oo00<007ooOol00goo00H007ooOol007oo0005
Ool000Uoo`03001oo`00009oo`03001oogoo009oo`03001oogoo00Aoo`03001oogoo00Moo`03001o
ogoo00Eoo`03001oogoo00Uoo`03001oogoo00Qoo`03001oogoo00Yoo`03001oogoo00Uoo`03001o
ogoo00=oo`03001oogoo00=oo`03001oogoo00]oo`03001oogoo00Qoo`03001oogoo00Uoo`03001o
ogoo00Moo`03001oogoo00Moo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00=oo`06001o
ogoo001oo`001Goo0009Ool00`00Ool00003Ool01000Oomoo`0027oo00<007ooOol01Goo00<007oo
Ool01goo00<007ooOol01goo00<007ooOol02Woo00<007ooOol027oo00<007ooOol02goo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol03Goo00<007ooOol01Woo00<007ooOol02goo00<007oo
Ool01Woo00<007ooOol027oo00<007ooOol017oo00<007ooOol01Woo00D007ooOomoo`0000=oo`03
001oo`0000Eoo`002Goo0P0017oo00@007ooOol000Uoo`03001oogoo00=oo`03001oogoo00Uoo`03
001oogoo00Eoo`03001oogoo00aoo`03001oogoo00Eoo`8000moo`05001oogooOol00003Ool00`00
Oomoo`0?Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool00`00Oomoo`04Ool00`00Oomoo`0:Ool00`00
Oomoo`02Ool00`00Oomoo`07Ool01@00Oomoogoo00000goo0P001Woo0009Ool20005Ool2000;Ool0
1@00Oomoogoo00003Goo00<007ooOol00goo00<007ooOol03Woo00<007ooOol00goo00<007ooOol0
47oo00@007ooOol0009oo`03001oogoo015oo`8000=oo`03001oogoo00moo`03001oogoo009oo`03
001oogoo00]oo`05001oogooOol0000:Ool01@00Oomoogoo000017oo00<007ooOol017oo0009Ool2
0005Ool2000;Ool01000Oomoo`003goo00D007ooOomoo`00019oo`05001oogooOol0000DOol00`00
Ool00002000FOol00`00Ool0000COol01000Oomoo`003goo00<007oo000037oo00<007oo00001Goo
00<007ooOol017oo0009Ool20005Ool00`00Oomoo`0;Ool00`00Ool0000@Ool00`00Ool0000DOol0
0`00Ool0000FOol3000HOol00`00Oomoo`0COol2000AOol2000<Ool00`00Ool00005Ool00`00Oomo
o`04Ool000Uoo`03001oogoo00Aoo`8000eoo`03001oogoo011oo`03001oogoo01Aoo`03001oogoo
01Eoo`8001Qoo`03001oo`0001Aoo`80015oo`8000eoo`03001oogoo00Aoo`8000Eoo`002Goo0P00
17oo00<007oo000037oo0P004Goo00<007oo000057oo00@007oo000001Aoo`04001oo`00000EOol2
0003Ool00`00Oomoo`0@Ool01000Oomoo`003goo00@007ooOol000aoo`03001oogoo00Aoo`8000Eo
o`002Goo0P0017oo00@007ooOol000Yoo`04001oogoo000?Ool01@00Oomoogoo00004Goo0P0017oo
00<007ooOol047oo00H007ooOol007oo000COol00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomo
o`02Ool00`00Oomoo`0<Ool01000Oomoo`002goo00<007oo00001Goo0P001Goo0009Ool20004Ool0
1000Oomoo`002Goo00<007ooOol00Woo00<007ooOol02goo00<007ooOol00goo00<007ooOol03Goo
00<007ooOol01Goo00<007ooOol03Woo00D007ooOomoo`00009oo`80011oo`03001oogoo00Ioo`03
001oogoo00aoo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo00Qoo`03
001oo`0000Eoo`8000Eoo`002Goo00<007oo00000Woo00<007ooOol00Woo00<007ooOol01Goo00<0
07ooOol017oo00<007ooOol02Goo00<007ooOol01Goo00<007ooOol02goo00<007ooOol01goo00<0
07ooOol037oo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol037oo00<007ooOol01goo00<0
07ooOol02goo00<007ooOol01Woo00<007ooOol027oo00<007ooOol017oo00<007ooOol01Woo00D0
07ooOomoo`0000Aoo`8000Eoo`002Goo00<007oo00000Woo00<007ooOol00Woo00<007ooOol01Goo
00<007ooOol017oo00<007ooOol027oo00<007ooOol01goo00<007ooOol02Goo00<007ooOol00Woo
0P001Goo00<007oo00000P001Goo10001Goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol0
2Goo00<007ooOol02Goo00<007ooOol01goo00<007ooOol01goo00<007ooOol01Woo00<007ooOol0
17oo00<007ooOol00Woo00L007ooOomoo`00Ool00005Ool000Uoo`03001oo`00009oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Moo`03001oogoo
00Qoo`03001oogoo009oo`04001oogoo0005Ool40004Ool01P00Oomoo`00Ool000Aoo`03001oogoo
00Aoo`03001oogoo00Qoo`03001oogoo00]oo`03001oogoo00Qoo`03001oogoo00Qoo`03001oogoo
00Ioo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo009oo`07001oogooOol007oo00001Goo
0009Ool01@00Ool007oo00001Woo00<007ooOol00goo00<007ooOol01goo00<007ooOol01Goo00<0
07ooOol02Goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo0`00
1goo00<007oo000017oo0P001Woo00<007ooOol01Woo00<007ooOol03Goo00<007ooOol01Woo00<0
07ooOol02Woo00<007ooOol017oo00<007ooOol027oo00<007ooOol00Woo00<007ooOol017oo00H0
07ooOol007oo0005Ool000Uoo`03001oogoo008000Moo`05001oogooOol0000;Ool00`00Oomoo`03
Ool00`00Oomoo`0;Ool01000Oomoogoo1`000Woo00<007ooOol00Woo00<007ooOol017oo00<007oo
Ool01Goo00@007ooOol000Aoo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo00moo`03001o
ogoo00Aoo`03001oogoo00aoo`03001oogoo009oo`03001oogoo00Yoo`05001oogooOol00006Ool0
1P00Oomoo`00Ool000Eoo`002Goo00<007ooOol00P0027oo00<007oo000037oo00<007ooOol00Woo
00<007ooOol03Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol01Goo
00<007ooOol00Woo0P000goo00<007ooOol00Woo00<007ooOol01goo00<007ooOol00Woo00<007oo
Ool04Goo00<007ooOol00Woo00<007ooOol03Goo00<007ooOol00Woo00<007ooOol02Woo00@007oo
Ool000Moo`03001oo`00009oo`03001oogoo00=oo`002Goo00<007ooOol00P0027oo00<007oo0000
3Goo00D007ooOomoo`00011oo`05001oogooOol00006Ool00`00Oomoo`02Ool00`00Oomoo`06Ool0
1000Oomoogoo1@001Goo00<007ooOol027oo00D007ooOomoo`0001Aoo`04001oogoo000AOol01000
Oomoo`003Woo00<007oo000027oo0P000Woo00<007ooOol00goo0009Ool00`00Oomoo`020009Ool0
0`00Oomoo`0=Ool00`00Ool0000BOol00`00Ool00007Ool00`00Oomoo`02Ool00`00Oomoo`07Ool0
1000Ool000002Goo00<007ooOol02Goo00<007oo00005Goo00<007oo00004goo0P003goo0P002Goo
0P000Woo00<007ooOol00goo0009Ool00`00Oomoo`020009Ool00`00Oomoo`0>Ool00`00Oomoo`0B
Ool00`00Oomoo`07Ool01000Oomoo`002goo00<007oo00002Goo00<007ooOol02Woo00<007ooOol0
5Goo00<007ooOol04Woo0P0047oo00<007ooOol01goo00D007ooOomoo`0000Eoo`002Goo00<007oo
Ool00P0027oo00<007oo00003Woo00<007oo00004Woo00<007oo00002Goo0P002goo00<007oo0000
1@001Goo00<007ooOol02Goo00<007oo00005Goo00<007oo00004goo00<007oo00003Woo00<007oo
000027oo00D007ooOomoo`0000Eoo`002Goo00<007ooOol00P0027oo00<007oo00003Goo00D007oo
Oomoo`00015oo`04001oogoo000DOol01@00Oomoogoo00002Goo00<007ooOol027oo00D007ooOomo
o`0001=oo`05001oogooOol0000AOol01000Oomoo`003Goo00@007ooOol000Qoo`80009oo`03001o
ogoo00=oo`002Goo00<007oo00000Woo00<007ooOol017oo00D007ooOomoo`0000aoo`03001oogoo
009oo`03001oogoo00eoo`03001oogoo009oo`03001oogoo011oo`03001oogoo00=oo`03001oogoo
00Ioo`03001oogoo00Moo`03001oogoo00=oo`03001oogoo00moo`03001oogoo00=oo`03001oogoo
00eoo`03001oogoo009oo`03001oogoo00Yoo`05001oogooOol00007Ool20002Ool00`00Oomoo`03
Ool000Uoo`03001oo`00009oo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`03Ool0
0`00Oomoo`0<Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`05Ool00`00Oomoo`05Ool2
0007Ool00`00Oomoo`05Ool00`00Oomoo`0=Ool00`00Oomoo`05Ool00`00Oomoo`0<Ool00`00Oomo
o`03Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Ool00002Ool00`00Oomo
o`03Ool000Uoo`03001oo`00009oo`03001oogoo00Aoo`03001oogoo009oo`03001oogoo00Moo`03
001oogoo00Eoo`03001oogoo00Yoo`03001oogoo00Ioo`03001oogoo00aoo`03001oogoo00Ioo`03
001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Moo`03001oogoo00]oo`03001oogoo00Moo`03
001oogoo00Yoo`03001oogoo00Eoo`03001oogoo00Moo`03001oogoo00=oo`03001oogoo00=oo`06
001oogoo001oo`001Goo0009Ool00`00Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`04Ool00`00
Oomoo`05Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`0;Ool00`00
Oomoo`08Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`09Ool00`00Oomoo`0:Ool00`00
Oomoo`07Ool00`00Oomoo`09Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00
Oomoo`02Ool01P00Oomoo`00Ool000Eoo`002Goo0P0017oo00<007ooOol00Woo00<007ooOol01Goo
00<007ooOol017oo00<007ooOol027oo00<007ooOol01Woo00<007ooOol02Goo00<007ooOol02Goo
00<007ooOol02Woo00<007ooOol00goo00<007ooOol017oo00<007ooOol02Goo00<007ooOol02Goo
00<007ooOol02Goo00<007ooOol01goo00<007ooOol027oo00<007ooOol017oo00<007ooOol01Woo
00D007ooOomoo`0000=oo`03001oo`0000Eoo`002Goo0P0017oo00D007ooOomoo`0000Qoo`03001o
ogoo00=oo`03001oogoo00Yoo`03001oogoo00Eoo`03001oogoo00Yoo`03001oogoo00Moo`03001o
ogoo00aoo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00]oo`03001oogoo00Moo`03001o
ogoo00]oo`03001oogoo00Ioo`03001oogoo00Uoo`03001oogoo00=oo`03001oogoo00Moo`04001o
ogoo0003Ool00`00Ool00005Ool000Uoo`8000Aoo`05001oogooOol00009Ool01@00Oomoogoo0000
3Woo00<007ooOol00goo00<007ooOol037oo00<007ooOol01Goo00<007ooOol03Woo00D007ooOomo
o`0000Aoo`03001oogoo00eoo`03001oogoo00Eoo`03001oogoo00eoo`03001oogoo00Aoo`03001o
ogoo00]oo`05001oogooOol0000:Ool01000Oomoo`000goo00<007oo00001Goo0009Ool20005Ool0
0`00Ool0000;Ool01000Oomoo`003goo00D007ooOomoo`00011oo`03001oogoo00=oo`03001oogoo
011oo`04001oogoo0003Ool00`00Oomoo`0?Ool00`00Oomoo`03Ool00`00Oomoo`0?Ool00`00Oomo
o`02Ool00`00Oomoo`0<Ool01000Oomoo`002goo00<007oo00001Goo0P001Goo0009Ool20005Ool0
0`00Ool0000<Ool2000@Ool01000Oomoo`004Woo00<007ooOol00Woo00<007ooOol04Goo00<007oo
00000Woo00<007ooOol04Goo00D007ooOomoo`0001=oo`04001oogoo000@Ool00`00Ool0000<Ool2
0005Ool20005Ool000Uoo`8000Ioo`03001oogoo00]oo`80015oo`8001Aoo`04001oogoo000EOol2
00000goo001oo`0DOol00`00Ool0000EOol2000AOol2000=Ool00`00Oomoo`04Ool00`00Oomoo`04
Ool000Uoo`03001oogoo00Eoo`03001oogoo00]oo`80019oo`03001oogoo01=oo`8001Ioo`<001Mo
o`03001oogoo01Aoo`03001oogoo011oo`8000eoo`03001oogoo00Aoo`03001oogoo00Aoo`002Goo
00<007ooOol017oo00<007oo00002goo00@007ooOol0011oo`03001oo`0001Aoo`8001Moo`03001o
ogoo01Eoo`03001oo`0001Aoo`03001oo`00015oo`03001oo`0000aoo`8000Eoo`03001oogoo00Ao
o`002Goo0P001Goo00<007oo00002goo00@007ooOol000moo`05001oogooOol0000BOol01000Oomo
o`005Goo0`005Goo00D007ooOomoo`0001=oo`04001oogoo000?Ool01000Oomoo`002goo00<007oo
00001Goo00<007ooOol017oo0009Ool20004Ool01@00Oomoogoo00002Goo00<007ooOol00Woo00<0
07ooOol02goo00<007ooOol00goo00<007ooOol03Woo00<007ooOol00Woo00<007ooOol04Woo0P00
00=oo`00Ool04Woo00<007ooOol00Woo00<007ooOol047oo00<007ooOol00Woo00<007ooOol037oo
00D007ooOomoo`0000Yoo`04001oogoo0004Ool00`00Oomoo`04Ool000Uoo`8000Aoo`05001oogoo
Ool00008Ool00`00Oomoo`04Ool00`00Oomoo`09Ool00`00Oomoo`05Ool00`00Oomoo`0<Ool00`00
Oomoo`03Ool00`00Oomoo`0AOol00`00Ool00002Ool00`00Oomoo`0@Ool00`00Oomoo`03Ool00`00
Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`07Ool01@00
Oomoogoo00000goo0P001Woo0009Ool20004Ool01@00Oomoogoo000027oo00<007ooOol01Goo00<0
07ooOol027oo00<007ooOol01Goo00<007ooOol02goo00<007ooOol01Goo00<007ooOol03goo00@0
07ooOol000=oo`03001oogoo00ioo`03001oogoo00Eoo`03001oogoo00aoo`03001oogoo00Ioo`03
001oogoo00Qoo`03001oogoo00Aoo`03001oogoo00Ioo`05001oogooOol00003Ool00`00Ool00005
Ool000Uoo`03001oo`00009oo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Ioo`03001o
ogoo00Moo`03001oogoo00Moo`03001oogoo00Uoo`03001oogoo00Moo`03001oogoo00eoo`05001o
ogooOol00004Ool00`00Oomoo`0<Ool00`00Oomoo`07Ool00`00Oomoo`0:Ool00`00Oomoo`08Ool0
0`00Oomoo`07Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`03Ool01P00Oomoo`00Ool0
00Eoo`002Goo00<007oo00000Woo00<007ooOol00goo00<007ooOol017oo00<007ooOol01goo00<0
07ooOol01Goo00<007ooOol02Goo00<007ooOol01goo00<007ooOol02Goo00<007ooOol02goo00<0
07ooOol00Woo00<007ooOol00goo00<007ooOol02Woo00<007ooOol02Goo00<007ooOol027oo00<0
07ooOol02Goo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol00goo00H0
07ooOol007oo0005Ool000Uoo`03001oo`00009oo`03001oogoo00Aoo`03001oogoo009oo`03001o
ogoo00Uoo`03001oogoo00=oo`03001oogoo00]oo`03001oogoo00Eoo`03001oogoo00]oo`03001o
ogoo00Uoo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo00]oo`03001o
ogoo00Ioo`03001oogoo00]oo`03001oogoo00Aoo`03001oogoo00Moo`03001oogoo00Aoo`03001o
ogoo00=oo`03001oo`00009oo`03001oogoo00=oo`002Goo00<007oo00000Woo00<007ooOol017oo
00D007ooOomoo`0000eoo`05001oogooOol0000?Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool00`00
Oomoo`07Ool00`00Oomoo`04Ool20005Ool00`00Oomoo`07Ool00`00Oomoo`0=Ool00`00Oomoo`05
Ool00`00Oomoo`0<Ool00`00Oomoo`03Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`05
Ool20002Ool00`00Oomoo`03Ool000Uoo`03001oogoo008000Qoo`04001oogoo000=Ool01000Oomo
o`004Goo00D007ooOomoo`00015oo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03
001oogoo00Eoo`03001oogoo00moo`03001oogoo00=oo`03001oogoo00eoo`03001oogoo009oo`03
001oogoo00Yoo`05001oogooOol00007Ool20002Ool00`00Oomoo`03Ool000Uoo`03001oogoo0080
00Qoo`03001oo`0000moo`03001oo`00019oo`03001oo`0001=oo`03001oogoo00Aoo`03001oogoo
00Eoo`03001oogoo00Ioo`03001oogoo00=oo`03001oogoo015oo`05001oogooOol0000AOol01000
Oomoo`003Goo00@007ooOol000Qoo`80009oo`03001oogoo00=oo`002Goo00<007ooOol00P002Goo
0P0047oo00<007ooOol04Goo0P005Goo00<007ooOol00Woo00<007ooOol01Woo00<007ooOol01goo
00D007ooOomoo`0001Aoo`04001oogoo000COol00`00Ool0000>Ool00`00Ool00008Ool01@00Oomo
ogoo00001Goo0009Ool00`00Oomoo`020009Ool00`00Oomoo`0>Ool00`00Ool0000BOol2000FOol0
1000Oomoo`002Goo00<007ooOol027oo00<007oo00005Woo0P0057oo0P003goo0P002Goo0P000Woo
00<007ooOol00goo0009Ool00`00Oomoo`020009Ool2000>Ool01000Oomoo`004Woo00<007oo0000
5Woo0P002Woo00<007ooOol02Goo00<007ooOol05Goo0P005Goo00<007ooOol03Woo00<007ooOol0
1goo0P000Woo00<007ooOol00goo0009Ool00`00Ool000040006Ool00`00Ool0000=Ool00`00Oomo
o`02Ool01000Oomoogoo0P001goo0`0000=oo`00Ool00Woo10004Woo0P002Woo00<007ooOol027oo
00<007oo00005Goo0P0000=oo`00Ool017oo0`001Goo10000Woo00<007oo00003Woo0P002Goo1`00
0goo0009Ool00`00Oomoo`020008Ool01000Oomoo`0037oo00<007ooOol00goo00<007oo00000Woo
00<007ooOol017oo10000goo00<007oo00000Woo00<007ooOol03Woo00<007oo00002Woo00<007oo
Ool027oo00@007ooOol001=oo`800004Ool007oo0004Ool30004Ool00`00Oomoo`02Ool20002Ool0
0`00Oomoo`0<Ool00`00Ool00008Ool20002Ool00`00Oomoo`03Ool000Uoo`04001oo`00Ool20006
Ool01@00Oomoogoo00002goo00<007ooOol01Goo00<007ooOol00Woo00<007ooOol00goo0`001goo
00<007oo00003goo00D007ooOomoo`0000Uoo`03001oogoo00Moo`05001oogooOol0000BOol20003
Ool00`00Ool00003Ool30009Ool20003Ool00`00Oomoo`0:Ool01000Oomoo`001goo00H007ooOol0
07oo0005Ool00003Ool0000000D00005Ool007oo001oo`020006Ool00`00Oomoo`02Ool00`00Oomo
o`07Ool70002Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`06Ool2000>Ool00`00Oomo
o`03Ool00`00Oomoo`06Ool20007Ool00`00Oomoo`03Ool00`00Oomoo`0?Ool00`00Oomoo`02Ool0
1000Oomoo`003Woo00<007ooOol00goo00<007ooOol027oo00<007ooOol00Woo00<007ooOol017oo
00H007ooOol007oo0005Ool000Uoo`04001oo`00Ool20005Ool00`00Oomoo`03Ool00`00Oomoo`07
Ool00`00Oomoo`06Ool20003Ool00`00Oomoo`04Ool00`00Oomoo`03Ool01000Oomoogoo0P003Goo
00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol03Goo
0P0017oo00D007ooOomoo`0000Qoo`04001oogooOol20005Ool00`00Oomoo`08Ool00`00Oomoo`02
Ool00`00Oomoo`04Ool01P00Oomoo`00Ool000Eoo`002Goo00L007oo001oo`00Ool00004Ool00`00
Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`07Ool00`00Ool00002Ool00`00Oomoo`03Ool00`00
Oomoo`04Ool6000=Ool00`00Oomoo`06Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`07
Ool00`00Oomoo`0;Ool00`00Ool00004Ool01@00Oomoogoo000027oo1@001goo00<007ooOol01Woo
00<007ooOol017oo00<007ooOol00Woo00L007ooOomoo`00Ool00005Ool000Uoo`07001oo`00Ool0
07oo00000goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01goo00H007ooOol007oo0004
Ool00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`03
Ool00`00Oomoo`03Ool00`00Oomoo`09Ool00`00Oomoo`09Ool01000Oomoo`0017oo00<007ooOol0
0Woo00<007ooOol01Goo00<007oo00002Woo00<007ooOol01Goo00<007ooOol017oo00<007ooOol0
0Woo00L007ooOomoo`00Ool00005Ool000Uoo`D00003Ool007oo009oo`03001oogoo00Ioo`03001o
ogoo00=oo`03001oogoo00Uoo`04001oo`000005Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00
Oomoo`07Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00
Oomoo`07Ool00`00Oomoo`02Ool01000Oomoo`001Woo00<007ooOol017oo00<007oo00002Woo00<0
07ooOol017oo00<007ooOol01Goo00<007ooOol00Woo00@007ooOomoo`<000Eoo`002Goo0P0000=o
o`0000000Woo00<007oo00002Goo00<007ooOol00Woo00<007ooOol02goo0`0017oo00<007ooOol0
1Woo1@000goo00<007ooOol01Goo00<007ooOol037oo00D007ooOomoo`0000=oo`03001oogoo00eo
o`03001oogoo00Ioo`03001oogoo00=oo`8000Qoo`03001oogoo00=oo`D000Uoo`03001oogoo00=o
o`03001oogoo00Ioo`04001oogoo0005Ool20005Ool000Uoo`8000Eoo`03001oo`0000Yoo`05001o
ogooOol0000@Ool01000Oomoo`004Woo00<007ooOol00goo00<007ooOol03Woo00@007ooOol0009o
o`03001oogoo00moo`03001oogoo00Aoo`03001oogoo00moo`03001oogoo009oo`03001oogoo00]o
o`03001oogoo009oo`03001oogoo00Moo`04001oogoo0005Ool00`00Oomoo`04Ool000Yoo`03001o
ogoo00=oo`8000]oo`04001oogoo000AOol00`00Ool0000COol00`00Oomoo`02Ool00`00Oomoo`0@
Ool01@00Ool007oo00004Woo00<007ooOol00goo00<007ooOol04Goo00@007ooOol000moo`05001o
ogooOol0000:Ool00`00Ool00005Ool00`00Oomoo`04Ool000Yoo`03001oogoo00Aoo`03001oogoo
00Yoo`03001oo`00019oo`8001Aoo`04001oogoo000DOol3000DOol01@00Oomoogoo00005Goo00<0
07oo000047oo00<007oo00002goo0P001Woo00<007ooOol017oo0004Oonn000000Uoo`8000Eoo`80
00Qoo`04001oogooOol20003Ool00`00Oomoo`06Ool00`00Oomoo`03Ool01000Oomoo`0027oo00<0
07ooOol01Woo00<007ooOol00P0017oo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol01Woo
00<007ooOol01Woo00<007ooOol00goo00@007ooOol000Qoo`03001oogoo00Ioo`06001oogoo001o
o`000goo00<007ooOol01Woo00<007ooOol00Woo00D007ooOomoo`0000Qoo`8000Ioo`8000Eoo`00
2Woo00<007ooOol00goo00<007oo00002goo0P004Goo00D007ooOomoo`0001Aoo`03001oo`0001Ao
o`<001Eoo`03001oo`0001Eoo`04001oogoo000@Ool00`00Ool0000;Ool00`00Ool00005Ool00`00
Oomoo`04Ool000Uoo`8000Aoo`04001oogoo000:Ool01000Oomoo`0047oo00<007ooOol00Woo00<0
07ooOol047oo00D007ooOomoo`00019oo`05001oo`00Ool0000COol01@00Oomoogoo00004goo00<0
07ooOol00Woo00<007ooOol037oo00@007ooOol000]oo`03001oo`0000Aoo`8000Ioo`002Goo0P00
17oo00@007ooOol000Yoo`05001oogooOol0000>Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00
Oomoo`03Ool00`00Oomoo`0>Ool01000Oomoo`000Woo00<007ooOol047oo00<007ooOol00Woo00<0
07ooOol047oo00<007ooOol00goo00<007ooOol02goo00D007ooOomoo`0000Uoo`05001oogooOol0
0003Ool00`00Ool00005Ool000Uoo`8000Aoo`05001oogooOol00008Ool00`00Oomoo`02Ool00`00
Oomoo`0;Ool00`00Oomoo`06Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool00`00Oomoo`0=Ool01@00
Oomoogoo00000goo00<007ooOol03Woo00<007ooOol017oo00<007ooOol03Woo00<007ooOol017oo
00<007ooOol02Woo00<007ooOol00goo00<007ooOol01Woo00D007ooOomoo`0000=oo`03001oo`00
00Eoo`002Goo00<007oo00000Woo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol017oo00<0
07ooOol02Goo00<007ooOol01goo00<007ooOol02goo00<007ooOol01Woo00<007ooOol02goo00<0
07ooOol00Woo00<007ooOol00Woo00<007ooOol037oo00<007ooOol01Goo00<007ooOol03Goo00<0
07ooOol01Woo00<007ooOol027oo00<007ooOol017oo00<007ooOol01Woo00D007ooOomoo`0000=o
o`03001oo`0000Eoo`002Goo00<007oo00000Woo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool01Goo00<007ooOol027oo00<007ooOol027oo00<007ooOol02Goo00<007ooOol027oo00<007oo
Ool02Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol01goo00<007oo
Ool02goo00<007ooOol027oo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol017oo00<007oo
Ool00goo00H007ooOol007oo0005Ool000Uoo`03001oo`00009oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00Yoo`03001oogoo00Moo`03001oogoo
00Yoo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo
00Uoo`03001oogoo00Uoo`03001oogoo00Uoo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo
00=oo`03001oogoo00=oo`06001oogoo001oo`001Goo0009Ool01@00Ool007oo00001Woo00<007oo
Ool00goo00<007ooOol01goo00<007ooOol01Goo00<007ooOol037oo00<007ooOol01Goo00<007oo
Ool037oo00<007ooOol01Goo00<007ooOol01Goo0P001Woo00<007ooOol01Woo00<007ooOol02goo
00<007ooOol027oo00<007ooOol02Woo00<007ooOol017oo00<007ooOol027oo00<007ooOol00Woo
00<007ooOol017oo00H007ooOol007oo0005Ool000Uoo`05001oo`00Ool00007Ool01@00Oomoogoo
00002goo00<007ooOol00goo00<007ooOol03Goo00<007ooOol017oo00<007ooOol03Woo00<007oo
Ool00goo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol017oo00<007ooOol03Goo00<007oo
Ool01Woo00<007ooOol037oo00<007ooOol00Woo00<007ooOol02Woo00D007ooOomoo`0000Moo`80
009oo`03001oogoo00=oo`002Goo00D007oo001oo`0000Moo`05001oogooOol0000<Ool00`00Oomo
o`02Ool00`00Oomoo`0>Ool00`00Oomoo`03Ool00`00Oomoo`0?Ool01@00Oomoogoo00002Goo00<0
07ooOol01goo00<007ooOol00goo00<007ooOol03Woo00<007ooOol017oo00<007ooOol03Goo00<0
07ooOol00Woo00<007ooOol02Woo00@007ooOol000Qoo`80009oo`03001oogoo00=oo`002Goo00@0
07ooOol000Uoo`03001oo`0000eoo`05001oogooOol0000BOol01@00Oomoogoo00004Woo00D007oo
Oomoo`0000Uoo`03001oogoo00Qoo`05001oogooOol0000BOol00`00Oomoo`02Ool00`00Oomoo`0?
Ool01000Oomoo`003Woo00<007oo000027oo0P000Woo00<007ooOol00goo0009Ool01000Oomoo`00
2Goo00<007oo00003Woo00<007oo000057oo00<007oo000057oo00<007oo00002Woo00<007ooOol0
2Goo00<007oo000057oo00@007ooOol001=oo`8000moo`8000Uoo`80009oo`03001oogoo00=oo`00
2Goo00@007ooOol000Yoo`03001oogoo00ioo`03001oogoo01=oo`8001Ioo`03001oogoo00Uoo`03
001oogoo00Yoo`03001oogoo01Aoo`03001oo`0001=oo`80011oo`03001oogoo00Qoo`04001oogoo
0005Ool000Uoo`04001oogoo000:Ool00`00Oomoo`0>Ool00`00Oomoo`0COol2000EOol00`00Ool0
000:Ool00`00Oomoo`09Ool2000FOol2000DOol2000?Ool2000:Ool01000Oomoo`001Goo0009Ool0
1000Oomoo`002Goo0P003goo00<007oo00004goo00@007ooOol001=oo`05001oogooOol00009Ool0
0`00Oomoo`08Ool01000Oomoo`005Goo0P004goo00@007ooOol000ioo`03001oo`0000Qoo`80009o
o`03001oogoo00=oo`002Goo00<007ooOol00P0027oo00<007oo00003Goo00D007ooOomoo`00015o
o`05001oogooOol0000BOol00`00Oomoo`03Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`02Ool00`00Oomoo`0AOol01000Oomoo`004Woo00@007ooOol000eoo`04001oogoo0008Ool20002
Ool00`00Oomoo`03Ool000Uoo`05001oo`00Ool00007Ool01000Oomoo`0037oo00<007ooOol00Woo
00<007ooOol03Woo00<007ooOol00goo00<007ooOol03Woo00<007ooOol01Goo00<007ooOol01Goo
0P0027oo00<007ooOol00goo00<007ooOol047oo00D007ooOomoo`00011oo`03001oogoo009oo`03
001oogoo00Yoo`04001oogoo0008Ool01@00Ool007oo00001Goo0009Ool01@00Ool007oo00001goo
00D007ooOomoo`0000]oo`03001oogoo00=oo`03001oogoo00eoo`03001oogoo00Aoo`03001oogoo
00aoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00ioo`03001oogoo00=oo`03001oogoo00aoo`03001oogoo00Aoo`03001oogoo00Qoo`03001oogoo
009oo`03001oogoo00Aoo`06001oogoo001oo`001Goo0009Ool00`00Ool00002Ool00`00Oomoo`03
Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00Oomoo`0;Ool00`00Oomoo`06
Ool00`00Oomoo`0:Ool00`00Oomoo`08Ool00`00Oomoo`04Ool00`00Oomoo`05Ool00`00Oomoo`07
Ool00`00Oomoo`0<Ool00`00Oomoo`05Ool00`00Oomoo`0;Ool00`00Oomoo`05Ool00`00Oomoo`07
Ool00`00Oomoo`02Ool00`00Oomoo`04Ool01P00Oomoo`00Ool000Eoo`002Goo00<007oo00000Woo
00<007ooOol00goo00<007ooOol00goo00<007ooOol01goo00<007ooOol01Goo00<007ooOol02Woo
00<007ooOol01goo00<007ooOol02Goo00<007ooOol02Woo00<007ooOol00goo00<007ooOol017oo
00<007ooOol02Goo00<007ooOol02Woo00<007ooOol01goo00<007ooOol02Goo00<007ooOol01Woo
00<007ooOol01Woo00<007ooOol017oo00<007ooOol00goo00H007ooOol007oo0005Ool000Uoo`03
001oo`0000=oo`05001oogooOol00007Ool00`00Oomoo`05Ool00`00Oomoo`07Ool00`00Oomoo`08
Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00Oomoo`0<Ool00`00Oomoo`02Ool00`00Oomoo`03
Ool00`00Oomoo`0;Ool00`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00Oomoo`08
Ool00`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01`00Oomoogoo001oo`0000Eoo`00
2Goo00<007oo00000goo00D007ooOomoo`0000Moo`03001oogoo00Aoo`03001oogoo00Uoo`03001o
ogoo00Ioo`03001oogoo00]oo`03001oogoo00Eoo`03001oogoo00ioo`05001oogooOol00004Ool0
0`00Oomoo`0=Ool00`00Oomoo`06Ool00`00Oomoo`0:Ool00`00Oomoo`06Ool00`00Oomoo`0:Ool0
0`00Oomoo`03Ool00`00Oomoo`06Ool01@00Oomoogoo000017oo0P001Goo0009Ool20004Ool01000
Oomoo`002Goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol017oo00<007ooOol03Goo00<0
07ooOol017oo00<007ooOol03goo00@007ooOol000=oo`03001oogoo00moo`03001oogoo00Aoo`03
001oogoo00aoo`03001oogoo00Eoo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo00Moo`05
001oogooOol00004Ool20005Ool000Uoo`8000Eoo`03001oo`0000Yoo`05001oogooOol0000>Ool0
0`00Oomoo`02Ool00`00Oomoo`0?Ool00`00Oomoo`02Ool00`00Oomoo`0AOol30003Ool00`00Oomo
o`0@Ool00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool01@00Oomo
ogoo00002Woo00<007oo00001Goo0P001Goo0009Ool20005Ool2000;Ool01@00Oomoogoo00003goo
00@007ooOol001=oo`04001oogoo000FOol01000Oomoo`0057oo00@007ooOol0019oo`05001oogoo
Ool0000@Ool00`00Ool0000;Ool00`00Ool00005Ool20005Ool000Yoo`03001oogoo00Aoo`03001o
ogoo00Yoo`03001oo`00015oo`8001Eoo`8001Moo`<001Ioo`8001Aoo`04001oogoo000@Ool00`00
Ool0000;Ool20006Ool00`00Oomoo`04Ool000Uoo`8000Ioo`03001oogoo00]oo`03001oogoo015o
o`03001oogoo01=oo`8001Moo`<001Ioo`03001oogoo01Aoo`80019oo`03001oogoo00]oo`03001o
ogoo00Aoo`03001oogoo00Aoo`002Goo0P001Goo0P003Goo0P004Goo0P0057oo00@007ooOol001Io
o`04001oogoo000DOol00`00Ool0000EOol2000AOol00`00Ool0000<Ool00`00Oomoo`04Ool00`00
Oomoo`04Ool000Uoo`8000Eoo`03001oo`0000]oo`03001oo`00011oo`04001oogoo000BOol00`00
Oomoo`02Ool00`00Oomoo`0BOol20003Ool00`00Oomoo`0@Ool01@00Oomoogoo000057oo00<007oo
000047oo00<007oo00002goo0P001Woo00<007ooOol017oo0009Ool20005Ool00`00Ool0000;Ool0
1000Oomoo`003Woo00<007ooOol00Woo00<007ooOol03Woo00<007ooOol017oo00<007ooOol03goo
0P0000=oo`00Ool00goo00<007ooOol03Woo00<007ooOol00goo00<007ooOol047oo00@007ooOol0
00moo`05001oogooOol0000:Ool00`00Ool00005Ool00`00Oomoo`04Ool000Uoo`8000Aoo`05001o
ogooOol00009Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool0
0`00Oomoo`06Ool00`00Oomoo`0=Ool01@00Oomoogoo00001Goo00<007ooOol037oo00<007ooOol0
1Goo00<007ooOol03Woo00<007ooOol00Woo00<007ooOol02goo00<007ooOol00goo00<007ooOol0
1Woo00@007ooOol000Eoo`03001oogoo00Aoo`002Goo0P0017oo00D007ooOomoo`0000Qoo`03001o
ogoo00=oo`03001oogoo00Yoo`03001oogoo00Eoo`03001oogoo00Yoo`03001oogoo00Qoo`03001o
ogoo00]oo`03001oogoo009oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo00Moo`03001o
ogoo00aoo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo00=oo`03001oogoo00Ioo`05001o
ogooOol00004Ool20005Ool000Uoo`03001oo`0000=oo`05001oogooOol00008Ool00`00Oomoo`04
Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`0:Ool00`00Oomoo`09
Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`0:
Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo
00000goo00<007oo00001Goo0009Ool00`00Ool00002Ool00`00Oomoo`03Ool00`00Oomoo`04Ool0
0`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`09Ool00`00Oomoo`06Ool00`00Oomoo`0<Ool0
0`00Oomoo`07Ool00`00Oomoo`04Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`0;Ool0
0`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`06Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool01`00Oomoogoo001oo`0000Eoo`002Goo00<007oo00000Woo00<007ooOol00goo
00<007ooOol017oo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol02Woo00<007ooOol01Goo
00<007ooOol03Goo00<007ooOol01Goo00<007ooOol01Goo0P0027oo00<007ooOol017oo00<007oo
Ool03Goo00<007ooOol01Woo00<007ooOol02Goo00<007ooOol01Woo00<007ooOol01goo00<007oo
Ool00goo00<007ooOol00goo00H007ooOol007oo0005Ool000Uoo`03001oo`00009oo`03001oogoo
00Aoo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo00Aoo`03001oogoo00]oo`03001oogoo
00Aoo`03001oogoo00moo`03001oogoo00=oo`03001oogoo00Ioo`03001oogoo00Qoo`03001oogoo
009oo`03001oogoo00moo`03001oogoo00Aoo`03001oogoo00]oo`03001oogoo00Aoo`03001oogoo
00Uoo`05001oogooOol00006Ool01P00Oomoo`00Ool000Eoo`002Goo00D007oo001oo`0000Moo`05
001oogooOol0000<Ool00`00Oomoo`02Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`0A
Ool01@00Oomoogoo00002Goo00<007ooOol02Goo0P0000=oo`00Ool04Woo00<007ooOol00Woo00<0
07ooOol03Goo00<007ooOol00goo00<007ooOol02Goo00D007ooOomoo`0000Ioo`06001oogoo001o
o`001Goo0009Ool01@00Ool007oo00001goo00D007ooOomoo`0000eoo`04001oogoo000AOol01000
Oomoo`005Goo00<007oo00002Woo00<007ooOol02goo00<007ooOol04goo00@007ooOol0011oo`03
001oogoo009oo`03001oogoo00]oo`03001oo`0000Qoo`80009oo`03001oogoo00=oo`002Goo00<0
07ooOol00P0027oo00<007oo00003goo0P004goo0P005goo00<007ooOol02Goo00<007ooOol02Woo
00<007oo00005Goo0P004Woo00@007ooOol000ioo`8000Uoo`80009oo`03001oogoo00=oo`002Goo
00@007ooOol000Uoo`03001oo`00011oo`03001oogoo015oo`8001Ioo`03001oo`0000Yoo`03001o
ogoo00Uoo`05001oogooOol0000DOol2000COol2000@Ool00`00Oomoo`07Ool20002Ool00`00Oomo
o`03Ool000Uoo`04001oogoo000:Ool00`00Oomoo`0>Ool00`00Ool0000AOol01000Oomoo`0027oo
0P001goo10000goo1@001Goo00<007ooOol027oo00<007ooOol00goo00<007ooOol047oo00<007oo
00004goo0P003goo00<007oo000027oo0P000Woo00<007ooOol00goo0009Ool01@00Ool007oo0000
27oo00<007oo00003Woo00@007ooOol0011oo`03001oogoo009oo`03001oogoo00Aoo`04001oogoo
0006Ool30004Ool20003Ool00`00Oomoo`02Ool00`00Oomoo`07Ool00`00Oomoo`05Ool00`00Oomo
o`0>Ool01@00Oomoogoo00004Goo00<007oo00003goo00<007oo000027oo0P000Woo00<007ooOol0
0goo0009Ool01@00Ool007oo000027oo00<007oo00003Goo00<007ooOol00Woo00<007ooOol037oo
00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo0`001Woo00@007oo
Ool000Aoo`03001oogoo00Ioo`03001oogoo00Moo`03001oogoo00aoo`03001oogoo00=oo`03001o
ogoo00eoo`05001oogooOol0000=Ool01@00Oomoogoo00001goo0P000Woo00<007ooOol00goo0009
Ool01@00Ool007oo00001goo00D007ooOomoo`0000aoo`03001oogoo00=oo`03001oogoo00Yoo`03
001oogoo00Eoo`03001oogoo009oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00Qoo`03
001oo`0000Aoo`8000Ioo`03001oogoo00Uoo`03001oogoo00Yoo`03001oogoo00Eoo`03001oogoo
00]oo`03001oogoo00=oo`03001oogoo00Uoo`03001oogoo009oo`03001oogoo00Aoo`06001oogoo
001oo`001Goo0009Ool20003Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`08Ool00`00
Oomoo`05Ool00`00Oomoo`08Ool00`00Oomoo`07Ool01@00Oomoogoo000017oo00D007ooOomoo`00
00Qoo`04001oogooOol20004Ool00`00Oomoo`04Ool00`00Oomoo`0;Ool00`00Oomoo`08Ool00`00
Oomoo`07Ool00`00Oomoo`0:Ool00`00Oomoo`04Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool01P00Oomoo`00Ool000Eoo`002Goo0P000goo00<007ooOol00goo00<007ooOol00goo
00<007ooOol01goo00<007ooOol01goo00<007ooOol01Woo00<007ooOol02Goo00@007ooOol000Ao
o`04001oogoo0009Ool60004Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool00`00Oomoo`06Ool00`00
Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00Oomoo`07Ool00`00Oomoo`04Ool00`00
Oomoo`02Ool01`00Oomoogoo001oo`0000Eoo`002Goo0P0017oo00D007ooOomoo`0000Moo`03001o
ogoo00Eoo`03001oogoo00Uoo`03001oogoo00Eoo`03001oogoo00Yoo`03001oo`0000Aoo`03001o
o`0000Yoo`03001oogoo00=oo`05001oogooOol00004Ool00`00Oomoo`0?Ool00`00Oomoo`04Ool0
0`00Oomoo`0;Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`05Ool0
1@00Oomoogoo00000goo00<007oo00001Goo0009Ool20004Ool01@00Oomoogoo00001goo00<007oo
Ool01Goo00<007ooOol02Goo00<007ooOol017oo00<007ooOol037oo00<007oo00000Woo00<007oo
00002goo00<007ooOol017oo00@007ooOol000=oo`03001oogoo015oo`03001oogoo009oo`03001o
ogoo00eoo`03001oogoo00Aoo`03001oogoo00Uoo`03001oogoo00Aoo`03001oogoo00Ioo`04001o
ogoo0004Ool00`00Ool00005Ool000Uoo`8000Eoo`03001oo`0000Uoo`03001oogoo00=oo`03001o
ogoo00]oo`03001oogoo009oo`03001oogoo00ioo`06001oo`00001oo`0037oo1@000goo00<007oo
00000Woo00<007ooOol04goo0P0000=oo`00Ool047oo00<007ooOol00Woo00<007ooOol02goo00<0
07ooOol00Woo00<007ooOol027oo00<007oo00001Goo0P001Goo0009Ool00`00Oomoo`04Ool00`00
Ool0000:Ool01@00Oomoogoo00003goo00@007ooOol0019oo`800003Ool007oo01Eoo`800003Ool0
07oo01Moo`03001oogoo015oo`04001oogoo000>Ool01@00Oomoogoo00002goo0P001Woo0P001Goo
0009Ool00`00Oomoo`05Ool00`00Oomoo`09Ool01000Oomoo`004Goo0P005Goo00<007ooOol05Woo
0P0067oo00<007oo00004goo0P0047oo00@007ooOol000aoo`03001oogoo00Aoo`03001oogoo00Ao
o`002Goo00<007ooOol01Goo00<007ooOol02Woo00<007oo00004Goo0P0057oo00<007oo00005goo
0P005goo00D007ooOomoo`00019oo`80015oo`8000eoo`03001oogoo00Aoo`03001oogoo00Aoo`00
2Goo00<007ooOol017oo0P003Goo00<007ooOol03goo00@007ooOol0019oo`05001oogooOol0000D
Ool5000EOol00`00Oomoo`03Ool00`00Oomoo`0>Ool01000Oomoo`0047oo0P0037oo00<007oo0000
1Goo00<007ooOol017oo0009Ool20005Ool00`00Ool0000;Ool2000@Ool00`00Oomoo`02Ool00`00
Oomoo`0>Ool00`00Oomoo`03Ool2000AOol01000Oomoo`000Woo0P004Woo00<007ooOol01Goo00<0
07ooOol037oo00D007ooOomoo`00011oo`8000aoo`03001oo`0000Eoo`03001oogoo00Aoo`002Goo
0P0017oo00@007ooOol000Yoo`04001oogoo000>Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool00`00
Oomoo`06Ool00`00Oomoo`0=Ool01@00Oomoogoo000017oo00<007ooOol03Woo00<007ooOol01goo
00<007ooOol02Woo00<007ooOol00goo00<007ooOol037oo00@007ooOol000]oo`03001oo`0000Ao
o`03001oo`0000Eoo`002Goo0P0017oo00@007ooOol000Yoo`05001oogooOol0000<Ool00`00Oomo
o`05Ool00`00Oomoo`0;Ool00`00Oomoo`08Ool00`00Oomoo`0;Ool00`00Oomoo`02Ool00`00Oomo
o`03Ool00`00Oomoo`0<Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`05Ool00`00Oomo
o`0:Ool00`00Oomoo`02Ool00`00Oomoo`07Ool01@00Oomoogoo00000goo00<007oo00001Goo0009
Ool20004Ool01@00Oomoogoo000027oo00<007ooOol00goo00<007ooOol027oo00<007ooOol01goo
00<007ooOol027oo0P0037oo00<007ooOol02Goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool02Woo00<007ooOol02goo00<007ooOol01Woo00<007ooOol01goo00<007ooOol027oo00<007oo
Ool00goo00<007ooOol01Woo00<007ooOol00Woo00L007ooOomoo`00Ool00005Ool000Uoo`03001o
o`00009oo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo00Eoo`03001oogoo00Eoo`8000]o
o`03001oogoo00Ioo`03001oogoo00eoo`8000Moo`8000Ioo`03001oogoo00Eoo`8000Qoo`8000mo
o`03001oogoo00Aoo`03001oogoo00Uoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Eo
o`03001oogoo00=oo`06001oogoo001oo`001Goo0009Ool00`00Ool00002Ool00`00Oomoo`03Ool0
0`00Oomoo`03Ool00`00Oomoo`07Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool0
0`00Oomoo`0@Ool00`00Oomoo`03Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`04Ool0
0`00Oomoo`0@Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00`00Oomoo`04Ool00`00Oomoo`07Ool0
0`00Oomoo`03Ool00`00Oomoo`04Ool01P00Oomoo`00Ool000Eoo`002Goo00D007oo001oo`0000Mo
o`03001oogoo009oo`03001oogoo00Qoo`05001oogooOol0000@Ool20003Ool00`00Oomoo`0BOol0
1@00Oomoogoo00002Goo00<007ooOol027oo00<007ooOol00Woo00<007ooOol04Woo00@007ooOol0
00moo`03001oogoo009oo`03001oogoo00Uoo`05001oogooOol00008Ool20002Ool00`00Oomoo`03
Ool000Uoo`03001oogoo008000Moo`05001oogooOol0000<Ool00`00Ool0000COol3000FOol00`00
Ool0000:Ool00`00Oomoo`09Ool01000Ool000005Woo0P004Goo00@007ooOol000aoo`05001oogoo
Ool00008Ool20002Ool00`00Oomoo`03Ool000Uoo`04001oogoo0009Ool00`00Ool0000>Ool00`00
Oomoo`0BOol2000GOol2000;Ool2000;Ool2000FOol4000AOol2000>Ool00`00Ool0000:Ool01000
Oomoo`001Goo0009Ool01000Oomoo`002Woo00<007ooOol037oo00<007oo00004Woo00<007ooOol0
0P0057oo00<007ooOol00P002Goo00<007ooOol02Goo00@007ooOol001Aoo`03001oogoo009oo`03
001oogoo00ioo`8000moo`03001oogoo00Uoo`04001oogoo0005Ool000Uoo`03001oogoo008000Qo
o`8000eoo`05001oogooOol0000@Ool00`00Oomoo`03Ool00`00Oomoo`0@Ool00`00Oomoo`03Ool0
0`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`02Ool00`00Oomoo`0@Ool00`00Oomoo`04Ool0
0`00Oomoo`0<Ool01000Oomoo`003Woo0P002Goo0P000Woo00<007ooOol00goo0009Ool01@00Ool0
07oo000027oo00<007oo00002goo00<007ooOol00goo00<007ooOol02goo0P001goo00<007ooOol0
3Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo0P001Woo00<007ooOol03Goo0P00
27oo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol02Woo00<007oo000027oo00H007ooOol0
07oo0005Ool000Uoo`05001oo`00Ool00007Ool01@00Oomoogoo00002Goo00<007ooOol01Goo0P00
2Woo00<007ooOol027oo00<007ooOol037oo00<007ooOol01goo00<007ooOol017oo00<007ooOol0
1Goo00<007ooOol01goo0P0037oo00<007ooOol02Goo00<007ooOol027oo00<007ooOol017oo00<0
07ooOol027oo00D007ooOomoo`0000Moo`06001oogoo001oo`001Goo0009Ool00`00Ool00002Ool0
0`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00Oomoo`08Ool00`00Oomoo`06Ool0
0`00Oomoo`0:Ool2000:Ool2000;Ool20004Ool00`00Oomoo`04Ool00`00Oomoo`0:Ool00`00Oomo
o`08Ool00`00Oomoo`0;Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool01`00Oomoogoo001oo`0000Eoo`002Goo0P000goo00<007ooOol00Woo
00<007ooOol01Goo00<007ooOol00goo00<007ooOol02Woo00<007ooOol00goo0P003goo00<007oo
Ool01Woo00<007ooOol03Goo00D007ooOomoo`0000Aoo`8000ioo`8000Ioo`8000moo`03001oogoo
00=oo`8000Yoo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo009oo`07001oogooOol007oo
00001Goo0009Ool20004Ool01@00Oomoogoo000027oo00D007ooOomoo`0000ioo`05001oogooOol0
000BOol00`00Oomoo`03Ool2000AOol01000Oomoo`000goo00<007ooOol047oo00<007ooOol00Woo
00<007ooOol047oo00D007ooOomoo`0000eoo`03001oogoo00=oo`03001oogoo00Eoo`05001oogoo
Ool00005Ool20005Ool000Uoo`8000Aoo`04001oogoo000:Ool00`00Ool0000@Ool00`00Ool0000D
Ool20002Ool00`00Oomoo`0BOol01000Ool007oo0P0057oo0P0000=oo`00Ool04goo00<007oo0000
3goo00D007ooOomoo`0000Uoo`04001oogoo0005Ool20005Ool000Uoo`03001oogoo00Aoo`8000ao
o`03001oogoo011oo`03001oogoo01Eoo`8001Ioo`<001Moo`8001Ioo`03001oogoo00moo`<000]o
o`03001oo`0000Eoo`03001oogoo00Aoo`002Goo00<007ooOol017oo0P002goo00<007oo00003goo
0P0000=oo`00000057oo00@007oo000001Eoo`@001Eoo`04001oogoo000COol200000goo001oo`0?
Ool2000=Ool00`00Oomoo`04Ool00`00Oomoo`04Ool000Uoo`8000Eoo`8000Yoo`05001oogooOol0
000=Ool00`00Oomoo`03Ool2000@Ool20004Ool2000AOol2000017oo001oogoo0P004Goo0P0017oo
0P0047oo00<007ooOol00Woo00<007ooOol037oo00@007ooOol000aoo`03001oogoo00Aoo`03001o
ogoo00Aoo`002Goo0P0017oo00@007ooOol000Qoo`03001oogoo00=oo`8000Yoo`03001oogoo00Io
o`03001oogoo00aoo`03001oogoo00Ioo`03001oogoo00eoo`05001oogooOol00004Ool2000>Ool0
0`00Oomoo`06Ool00`00Oomoo`0<Ool00`00Oomoo`04Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool0
0`00Oomoo`08Ool00`00Ool00005Ool20005Ool000Uoo`8000=oo`03001oogoo009oo`03001oogoo
00Aoo`03001oogoo00Ioo`03001oogoo00Ioo`03001oogoo00Qoo`8000Yoo`8000Yoo`8000]oo`80
00Aoo`03001oogoo00Aoo`03001oogoo00Uoo`8000Yoo`8000]oo`03001oogoo00Ioo`8000Uoo`03
001oogoo00Aoo`03001oogoo00Ioo`05001oogooOol00004Ool20005Ool000Uoo`8000=oo`03001o
ogoo00=oo`03001oogoo009oo`03001oogoo00Qoo`03001oogoo00=oo`8000eoo`8000Moo`03001o
ogoo00aoo`8000Qoo`03001oogoo00Aoo`03001oogoo00Eoo`8000Moo`8000ioo`8000Qoo`03001o
ogoo00Uoo`8000Eoo`8000Qoo`8000Eoo`03001oogoo00=oo`06001oogoo001oo`001Goo0009Ool0
1@00Ool007oo00001goo00@007ooOol000aoo`80009oo`03001oogoo00moo`03001oogoo009oo`80
015oo`03001oogoo00=oo`8000Moo`03001oogoo00Moo`8000Aoo`03001oogoo011oo`03001oogoo
00=oo`8000ioo`05001oogooOol0000<Ool01@00Oomoogoo00001Woo00H007ooOol007oo0005Ool0
00Uoo`03001oogoo008000Qoo`8000moo`8001=oo`@001Aoo`800003Ool0000000Uoo`03001oogoo
00Uoo`@001Aoo`D0015oo`<000ioo`03001oo`0000Qoo`05001oo`00Ool00005Ool000Uoo`04001o
ogoo0008Ool00`00Ool0000=Ool200000goo0000000BOol3000FOol3000:Ool00`00Oomoo`0:Ool3
000DOol4000BOol01000Ool000003Woo0P0027oo0P000Woo00<007ooOol00goo0009Ool01@00Ool0
07oo00001Woo00D007ooOomoo`0000]oo`03001oogoo00=oo`03001oogoo00aoo`<000=oo`80019o
o`8000=oo`8000Qoo`8000Uoo`8000=oo`80011oo`8000Aoo`03001oogoo00eoo`8000Aoo`03001o
ogoo00Uoo`80009oo`03001oogoo00Eoo`80009oo`03001oogoo00=oo`002Goo00<007oo00000Woo
00<007ooOol00Woo00<007ooOol00goo00<007ooOol01Woo0P001goo0P002Woo0P0027oo0`003Goo
0P001goo0P001Woo00<007ooOol01Goo0`001goo00<007ooOol02Woo0`001goo0P002goo0P001goo
0P0027oo00<007ooOol00goo00<007ooOol017oo00D007oo001oo`0000Eoo`002Goo00<007oo0000
0Woo00D007ooOomoo`0000Moo`8000Eoo`03001oogoo00Uoo`8000Ioo`8000eoo`8000Qoo`<000]o
o`8000Aoo`03001oogoo00=oo`8000]oo`8000Qoo`8000aoo`<000Ioo`8000]oo`03001oogoo00Ao
o`03001oogoo00Eoo`03001oogoo009oo`06001oogoo001oo`001Goo0009Ool20004Ool00`00Ool0
000:Ool200000goo0000000>Ool20002Ool2000AOol30003Ool2000@Ool3000017oo001oogoo0`00
3goo0`000goo0P004Goo0`0000=oo`0000003Woo0P000Woo0P002Goo00@007oo000000Aoo`8000Eo
o`002Goo0P001Goo0P002goo0P004Woo0`005Goo0`005Goo10005Goo0`005Goo0`0047oo0`0037oo
00<007ooOol017oo00<007ooOol017oo000:Ool00`00Oomoo`02Ool01000Oomoo`002Goo00<007oo
Ool00P003Woo0P000goo0`0047oo0`000Woo0`003goo20004goo0`0000Aoo`000000015oo`04001o
ogooOol3000;Ool20003Ool00`00Oomoo`07Ool200000goo001oo`04Ool00`00Oomoo`04Ool000Uo
o`8000=oo`03001oogoo009oo`8000Eoo`8000Eoo`<000Uoo`8000Qoo`<000Qoo`D000Qoo`<000Mo
o`D000Aoo`04001oogooOol3000<Ool40007Ool3000:Ool40007Ool30005Ool30006Ool30005Ool0
0`00Oomoo`02Ool01`00Oomoogoo001oo`0000Eoo`002Goo00D007oo001oo`0000Moo`800003Ool0
000000Yoo`T000eoo`P0011oo`X000Ioo`03001oogoo00Aoo`H0009oo`@000ioo`X000ioo`D000ao
o`D000Ioo`<0009oo`03001oogoo00=oo`002Goo1P0017oo2@001goo100000=oo`0000000P002Goo
1P000Woo2`001Woo4@000goo00<007ooOol01goo2@0037oo3P002goo0`0000=oo`0000000P0027oo
10000Woo10000Woo1@0000=oo`00Ool017oo0009Ool00`00Oomoo`03Ool40009Ool70009Ool9000C
Ool6000=Ool01000Oomoogoo3@002Goo30003Woo2`0027oo20002Woo0P001Goo0P001Goo001KOol0
0`00Oomoo`04Ool00`00Oomoo`1MOol005]oo`03001oogoo00Aoo`03001oogoo05eoo`00Fgoo00<0
07ooOol017oo00<007ooOol0GGoo001IOol30006Ool00`00Oomoo`1MOol005Yoo`8006Ioo`00`Woo
0000\
\>"],
  ImageRangeCache->{{{0, 193.125}, {193.125, 0}} -> {-1.10022, -1.07517, 
  0.0167016, 0.0167016}}]
}, Open  ]],

Cell[TextData[{
  "Please do not use this function for doing log plots and similar \
constructions. The facility for doing these is not part of the core kernel \
but is in a Package called ",
  StyleBox["Graphics`Graphics`",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ",  along with other useful plots. Here is the command to load the Package \
and an obvious simple application:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Needs[\"Graphics`Graphics`\"];
LogPlot[Exp[-x], {x, 1, 5}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.238095 0.75047 0.338828 [
[.02381 -0.0125 -3 -9 ]
[.02381 -0.0125 3 0 ]
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -3 -9 ]
[.7381 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[-0.0125 .07281 -28 -6 ]
[-0.0125 .07281 0 6 ]
[-0.0125 .17481 -28 -6 ]
[-0.0125 .17481 0 6 ]
[-0.0125 .30964 -28 -6 ]
[-0.0125 .30964 0 6 ]
[-0.0125 .41164 -22 -6 ]
[-0.0125 .41164 0 6 ]
[-0.0125 .51364 -22 -6 ]
[-0.0125 .51364 0 6 ]
[ -0.0005 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 0 m
.02381 .00625 L
s
[(1)] .02381 -0.0125 0 1 Mshowa
.2619 0 m
.2619 .00625 L
s
[(2)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(3)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(4)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(5)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.07143 0 m
.07143 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.16667 0 m
.16667 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.35714 0 m
.35714 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45238 0 m
.45238 .00375 L
s
.54762 0 m
.54762 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.64286 0 m
.64286 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.83333 0 m
.83333 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.92857 0 m
.92857 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07281 m
.00625 .07281 L
s
gsave
-0.0125 .07281 -89 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.01) show
87.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .17481 m
.00625 .17481 L
s
gsave
-0.0125 .17481 -89 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.02) show
87.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .30964 m
.00625 .30964 L
s
gsave
-0.0125 .30964 -89 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.05) show
87.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .41164 m
.00625 .41164 L
s
gsave
-0.0125 .41164 -83 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.1) show
81.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
0 .51364 m
.00625 .51364 L
s
gsave
-0.0125 .51364 -83 -10 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000000 0.000000 0.000000 setrgbcolor
1.000000 setlinewidth
gsave
newpath
 61.000000 16.000000 moveto
 541.000000 16.000000 lineto
 541.000000 4.000000 lineto
 61.000000 4.000000 lineto
 61.000000 16.000000 lineto
closepath
clip newpath
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(0.2) show
81.000000 13.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.001 w
0 .23448 m
.00375 .23448 L
s
0 .27681 m
.00375 .27681 L
s
0 .33647 m
.00375 .33647 L
s
0 .35916 m
.00375 .35916 L
s
0 .37881 m
.00375 .37881 L
s
0 .39614 m
.00375 .39614 L
s
0 .5733 m
.00375 .5733 L
s
0 .61564 m
.00375 .61564 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .57944 L
.10458 .5534 L
.14415 .52894 L
.18221 .50542 L
.22272 .48039 L
.26171 .45629 L
.30316 .43067 L
.34309 .406 L
.3815 .38225 L
.42237 .357 L
.46172 .33268 L
.49955 .30929 L
.53984 .2844 L
.57861 .26043 L
.61984 .23495 L
.65954 .21041 L
.69774 .18681 L
.73838 .16169 L
.77751 .13751 L
.81909 .11181 L
.85916 .08704 L
.89771 .06322 L
.93871 .03788 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.125, 111.938},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004?0000Ya000`40O003h00Oogoo47oo003oOol@
Ool00?mooa1oo`00ogoo47oo003oOol@Ool00?mooa1oo`00ogoo47oo003oOol@Ool00?mooa1oo`00
;goo1@00<7oo1P00<Goo1000=7oo0`00<7oo10001Goo000aOol00`00Oomoo`0aOol01@00Oomoogoo
0000<7oo00<007ooOol00Woo00<007ooOol0<Woo00<007ooOol0;Woo00<007ooOol00Woo00<007oo
Ool00Woo000aOol00`00Oomoo`0bOol00`00Oomoo`0fOol00`00Oomoo`0^Ool6000dOol00`00Oomo
o`02Ool0035oo`03001oogoo03=oo`03001oogoo03Eoo`03001oogoo02ioo`05001oogooOol0000e
Ool00`00Oomoo`02Ool0035oo`03001oogoo03Aoo`03001oogoo035oo`<0039oo`04001oogoo000`
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool0035oo`03001oogoo03Eoo`03001oogoo03=oo`03001o
ogoo031oo`03001oo`00031oo`D000Eoo`00<Goo00<007ooOol0=Goo00<007ooOol0<goo00<007oo
Ool0<7oo00<007oo0000<7oo00<007ooOol01goo000_Ool3000bOol00`00Oomoo`02Ool00`00Oomo
o`0^Ool00`00Oomoo`02Ool00`00Oomoo`0aOol2000`Ool00`00Oomoo`07Ool0031oo`8003=oo`@0
039oo`@003Eoo`03001oogoo02ioo`D000Eoo`00ogoo47oo003oOol@Ool00?mooa1oo`00;7ooh`00
000/Ool00`00Oomoo`03Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomo
o`08Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomo
o`08Ool00`00Oomoo`07Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomo
o`07Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`07Ool00`00Oomo
o`08Ool00`00Oomoo`08Ool00`00Oomoo`03Ool002aoo`03001oogoo00=oo`03001oogoo039oo`03
001oogoo03=oo`03001oogoo03=oo`03001oogoo03=oo`03001oogoo00=oo`00;7oo00<007ooOol0
h7oo000/Ool00`00Oomoo`3IOol20005Ool002aoo`03001oogoo0=Moo`8000Moo`00;7oo00<007oo
Ool0eGoo0P002Goo000/Ool00`00Oomoo`3DOol00`00Oomoo`09Ool002aoo`03001oogoo0=9oo`80
00aoo`00;7oo00<007ooOol0d7oo0P003Woo000/Ool00`00Oomoo`3?Ool00`00Oomoo`0>Ool002ao
o`03001oogoo0<eoo`80015oo`00;7oo00<007ooOol0c7oo00<007ooOol04Goo0006Ool20007Ool3
0006Ool20006Ool50007Ool00`00Oomoo`3:Ool2000DOol000Eoo`04001oogoo0006Ool30005Ool0
1000Oomoo`001goo00<007ooOol01goo00<007ooOol0bGoo00<007ooOol057oo0004Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`07Ool0
0`00Oomoo`37Ool2000GOol000Aoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo009oo`03
001oogoo00Aoo`03001oogoo00Moo`03001oogoo0<Eoo`8001Uoo`0017oo00<007ooOol00Woo00<0
07ooOol02Woo00<007ooOol00Woo00<007ooOol017oo00<007ooOol01goo0`00`goo0P006goo0004
Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`04Ool00`00Oomoo`07
Ool00`00Oomoo`32Ool00`00Oomoo`0KOol000Aoo`03001oogoo009oo`03001oogoo00Yoo`03001o
ogoo009oo`03001oogoo00Aoo`03001oogoo00Moo`03001oogoo0<1oo`8001ioo`001Goo00@007oo
Ool000ioo`04001oogoo0005Ool30009Ool00`00Oomoo`2nOol2000POol000Ioo`80011oo`8000Mo
o`8000Uoo`03001oogoo0;eoo`03001oogoo021oo`00;7oo00<007ooOol0^goo0P008goo000/Ool0
0`00Oomoo`2jOol00`00Oomoo`0SOol002aoo`03001oogoo0;Qoo`8002Ioo`00;7oo00<007ooOol0
]goo00<007ooOol09Woo000/Ool00`00Oomoo`2eOol2000YOol002aoo`03001oogoo0;=oo`8002]o
o`00;7oo00<007ooOol0/Goo0P00;Goo000/Ool00`00Oomoo`2`Ool00`00Oomoo`0]Ool002aoo`03
001oogoo0:ioo`80031oo`00;7oo00<007ooOol0[7oo0P00<Woo000/Ool00`00Oomoo`2[Ool00`00
Oomoo`0bOol002aoo`03001oogoo0:Uoo`8003Eoo`00;7oo00<007ooOol0Z7oo00<007ooOol0=Goo
000/Ool00`00Oomoo`2VOol2000hOol000Ioo`8000Moo`<000Ioo`8000Eoo`H000Moo`03001oogoo
0:Eoo`03001oogoo03Qoo`001Goo00@007ooOol000Ioo`<000Eoo`04001oogoo0005Ool01@00Oomo
ogoo00001goo00<007ooOol0Xgoo0P00>goo0004Ool00`00Oomoo`02Ool00`00Oomoo`03Ool30004
Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`08Ool00`00Oomoo`2ROol00`00Oomoo`0k
Ool000Aoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo00Aoo`03001o
ogoo00Moo`03001oogoo0:1oo`8003ioo`0017oo00<007ooOol00Woo00<007ooOol02Woo00<007oo
Ool00Woo00<007ooOol01Goo00<007ooOol01Woo0`00Wgoo00<007ooOol0?Woo0004Ool00`00Oomo
o`02Ool00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomo
o`2MOol20011Ool000Aoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo009oo`03001oogoo
00Ioo`03001oogoo00Eoo`03001oogoo09aoo`03001oogoo045oo`001Goo00@007ooOol000ioo`04
001oogoo0004Ool00`00Oomoo`02Ool00`00Oomoo`05Ool00`00Oomoo`2JOol20014Ool000Ioo`80
011oo`8000Ioo`@000Qoo`03001oogoo09Qoo`8004Ioo`00;7oo00<007ooOol0UWoo0P00B7oo000/
Ool00`00Oomoo`2EOol00`00Oomoo`18Ool002aoo`03001oogoo09=oo`8004]oo`00;7oo00<007oo
Ool0TGoo0P00CGoo000/Ool00`00Oomoo`2?Ool2001?Ool002aoo`03001oogoo08eoo`80055oo`00
;7oo00<007ooOol0S7oo00<007ooOol0DGoo000/Ool00`00Oomoo`2:Ool2001DOol002aoo`8008Uo
o`8005Ioo`00;7oo00<007ooOol0Qgoo00<007ooOol0EWoo000/Ool00`00Oomoo`25Ool2001IOol0
02aoo`03001oogoo08Aoo`03001oogoo05Uoo`00;7oo00<007ooOol0PWoo0P00G7oo000/Ool00`00
Oomoo`21Ool00`00Oomoo`1LOol002aoo`03001oogoo07moo`8005moo`00;7oo00<007ooOol0OWoo
00<007ooOol0Ggoo000/Ool00`00Oomoo`1lOol2001ROol002aoo`03001oogoo07]oo`03001oogoo
069oo`00;7oo0P00NWoo0P00IGoo000/Ool00`00Oomoo`1hOol00`00Oomoo`1UOol002aoo`03001o
ogoo07Ioo`8006Qoo`00;7oo00<007ooOol0M7oo0P00JWoo0006Ool20007Ool30006Ool20006Ool4
0008Ool00`00Oomoo`1bOol2001/Ool000Eoo`04001oogoo0006Ool30005Ool01000Oomoo`0017oo
00<007ooOol00Woo00<007ooOol01Goo00<007ooOol0LGoo00<007ooOol0K7oo0004Ool00`00Oomo
o`02Ool00`00Oomoo`03Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`06Ool00`00Oomoo`05Ool0
0`00Oomoo`1_Ool2001_Ool000Aoo`03001oogoo009oo`03001oogoo00Yoo`03001oogoo009oo`03
001oogoo00Ioo`03001oogoo00Eoo`<006eoo`80075oo`0017oo00<007ooOol00Woo00<007ooOol0
2Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo00Eoo`03001oogoo06aoo`03001o
ogoo075oo`0017oo00<007ooOol00Woo00<007ooOol02Woo00<007ooOol00Woo00@007ooOomoo`D0
00Qoo`03001oogoo06]oo`03001oogoo079oo`0017oo00<007ooOol00Woo00<007ooOol02Woo00<0
07ooOol00Woo00D007ooOomoo`0000aoo`03001oogoo06Uoo`8007Eoo`001Goo00@007ooOol000io
o`04001oogoo0004Ool00`00Oomoo`0:Ool00`00Oomoo`1XOol00`00Oomoo`1eOol000Ioo`80011o
o`8000Eoo`D000Qoo`03001oogoo06Moo`03001oogoo07Ioo`00;7oo0P00IWoo0P00NGoo000/Ool0
0`00Oomoo`1SOol2001kOol002aoo`03001oogoo065oo`8007eoo`00;7oo00<007ooOol0H7oo00<0
07ooOol0OGoo000/Ool00`00Oomoo`1NOol20020Ool002aoo`03001oogoo05aoo`80089oo`00;7oo
0P00Fgoo0P00Q7oo000/Ool00`00Oomoo`1IOol00`00Oomoo`24Ool002aoo`03001oogoo05Moo`80
08Moo`00;7oo00<007ooOol0EGoo0P00RGoo000/Ool2001EOol00`00Oomoo`29Ool002aoo`03001o
ogoo059oo`8008aoo`00;7oo00<007ooOol0DGoo00<007ooOol0S7oo000/Ool00`00Oomoo`1?Ool2
002?Ool000moo`8000Moo`<000Eoo`D000Moo`8004ioo`80095oo`003Woo00@007ooOol000Ioo`<0
00Moo`03001oogoo00Moo`03001oogoo04aoo`03001oogoo095oo`003Goo00<007ooOol00Woo00<0
07ooOol00goo0`001goo00<007ooOol01goo00<007ooOol0BWoo0P00U7oo000=Ool00`00Oomoo`02
Ool00`00Oomoo`0=Ool00`00Oomoo`07Ool30019Ool00`00Oomoo`2DOol000eoo`03001oogoo009o
o`03001oogoo00eoo`03001oogoo00Moo`03001oogoo04Moo`8009Moo`003Goo00<007ooOol00Woo
00<007ooOol03Goo00<007ooOol01goo00<007ooOol0AWoo00<007ooOol0Ugoo000=Ool00`00Oomo
o`02Ool00`00Oomoo`0=Ool00`00Oomoo`07Ool00`00Oomoo`14Ool2002JOol000ioo`04001oogoo
000>Ool30009Ool00`00Oomoo`13Ool00`00Oomoo`2JOol000moo`80011oo`8000Uoo`03001oogoo
045oo`8009eoo`00;7oo00<007ooOol0?goo0P00Wgoo000/Ool00`00Oomoo`0mOol2002QOol002ao
o`03001oogoo03]oo`800:=oo`00;7oo00<007ooOol0>Goo0P00YGoo000/Ool00`00Oomoo`0gOol2
002WOol002aoo`03001oogoo03Ioo`03001oogoo0:Moo`00;7oo00<007ooOol0=Goo00<007ooOol0
Z7oo000/Ool00`00Oomoo`0cOol2002[Ool002aoo`03001oogoo039oo`03001oogoo0:]oo`00;7oo
00<007ooOol0<Goo00<007ooOol0[7oo000/Ool00`00Oomoo`0_Ool2002_Ool002aoo`03001oogoo
02eoo`800;5oo`00;7oo00<007ooOol0;7oo00<007ooOol0/Goo000/Ool00`00Oomoo`0ZOol2002d
Ool000moo`8000Moo`<000Aoo`H000Moo`03001oogoo02Qoo`800;Ioo`003Woo00@007ooOol000Io
o`<000Eoo`05001oogooOol00007Ool00`00Oomoo`0WOol00`00Oomoo`2fOol000eoo`03001oogoo
009oo`03001oogoo00=oo`<000Ioo`03001oogoo00Qoo`03001oogoo02Eoo`800;Uoo`003Goo00<0
07ooOol00Woo00<007ooOol03Goo00<007ooOol01goo0`0097oo00<007ooOol0^Goo000=Ool00`00
Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`0ROol2002lOol000eoo`03001o
ogoo009oo`03001oogoo00moo`03001oogoo00Eoo`03001oogoo021oo`800;ioo`003Goo00<007oo
Ool00Woo00<007ooOol03goo00<007ooOol01Goo00<007ooOol07goo00<007ooOol0_Woo000>Ool0
1000Oomoo`003Goo00<007ooOol00Woo00<007ooOol01Goo00<007ooOol07Goo0P00`Goo000?Ool2
000?Ool40008Ool00`00Oomoo`0KOol20033Ool002aoo`03001oogoo01Uoo`800<Eoo`00;7oo00<0
07ooOol067oo00<007ooOol0aGoo000/Ool00`00Oomoo`0FOol20038Ool002aoo`03001oogoo01Ao
o`800<Yoo`00;7oo00<007ooOol04Woo0P00c7oo000/Ool00`00Oomoo`0AOol00`00Oomoo`3<Ool0
02aoo`03001oogoo00moo`800<moo`00;7oo00<007ooOol03Goo0P00dGoo000/Ool2000=Ool00`00
Oomoo`3AOol002aoo`03001oogoo00Yoo`800=Aoo`00;7oo00<007ooOol02Goo00<007ooOol0e7oo
000/Ool00`00Oomoo`08Ool00`00Oomoo`3EOol002aoo`03001oogoo00Ioo`800=Qoo`00;7oo00<0
07ooOol01Goo00<007ooOol0f7oo000/Ool00`00Oomoo`04Ool00`00Oomoo`3IOol002aoo`03001o
ogoo00=oo`03001oogoo0=Yoo`00;7oo00<007ooOol0h7oo000/Ool2003QOol002aoo`03001oogoo
0>1oo`00ogoo47oo003oOol@Ool00?mooa1oo`00ogoo47oo003oOol@Ool00001\
\>"],
  ImageRangeCache->{{{0, 270.688}, {166.875, 0}} -> {0.0664719, -2.49982, 
  0.0278932, 0.0196006}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Plotting One or More Data Sets with One Independent Variable", 
  "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  StyleBox["ListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " function takes data represented as a list and produces scatter and line \
plots. Let's make up some data (remember the semicolon stops ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " from producing the lists on the screen):"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
polydata = Table[10*Sqrt[i], {i, 3, 10}];
polydatab = Table[{i, i^(3/2)}, {i, 3, 10}];
redblobs = ListPlot[polydata, 
PlotStyle -> {RGBColor[1, 0, 0], PointSize[0.05]}, 
PlotRange -> {0, 50}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 0.136054 0 0.0123607 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .12361 -12 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -12 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -12 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -12 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(2)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(3)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(4)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(5)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(6)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(7)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(8)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(10)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(20)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(30)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(40)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .02472 m
.02756 .02472 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .07416 m
.02756 .07416 L
s
.02381 .09889 m
.02756 .09889 L
s
.02381 .14833 m
.02756 .14833 L
s
.02381 .17305 m
.02756 .17305 L
s
.02381 .19777 m
.02756 .19777 L
s
.02381 .22249 m
.02756 .22249 L
s
.02381 .27193 m
.02756 .27193 L
s
.02381 .29666 m
.02756 .29666 L
s
.02381 .32138 m
.02756 .32138 L
s
.02381 .3461 m
.02756 .3461 L
s
.02381 .39554 m
.02756 .39554 L
s
.02381 .42026 m
.02756 .42026 L
s
.02381 .44498 m
.02756 .44498 L
s
.02381 .46971 m
.02756 .46971 L
s
.02381 .51915 m
.02756 .51915 L
s
.02381 .54387 m
.02756 .54387 L
s
.02381 .56859 m
.02756 .56859 L
s
.02381 .59331 m
.02756 .59331 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.05 w
.02381 .21409 Mdot
.15986 .24721 Mdot
.29592 .27639 Mdot
.43197 .30277 Mdot
.56803 .32703 Mdot
.70408 .34961 Mdot
.84014 .37082 Mdot
.97619 .39088 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{167, 103.125},
  ImageMargins->{{34, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003j0000VQ000`40O003h00OnWoo003jOol00?Yo
o`00>Goo1P005goo10006goo0`0067oo10006Goo10006Woo00<007ooOol067oo100047oo000jOol0
1@00Oomoogoo00005Woo00<007ooOol00Woo00<007ooOol06Goo00<007ooOol05Woo00<007ooOol0
0Woo00<007ooOol05Goo00<007ooOol00Woo00<007ooOol067oo00<007ooOol05Woo00<007ooOol0
0Woo00<007ooOol03Goo000kOol00`00Oomoo`0LOol00`00Oomoo`0EOol6000LOol00`00Oomoo`0E
Ool00`00Oomoo`02Ool00`00Oomoo`0HOol00`00Oomoo`0FOol00`00Oomoo`02Ool00`00Oomoo`0=
Ool003aoo`03001oogoo01]oo`03001oogoo01Eoo`05001oogooOol0000MOol00`00Oomoo`0EOol0
0`00Oomoo`02Ool00`00Oomoo`0HOol00`00Oomoo`0FOol00`00Oomoo`02Ool00`00Oomoo`0=Ool0
03eoo`03001oogoo01Moo`<001Uoo`04001oogoo000HOol00`00Oomoo`02Ool00`00Oomoo`0EOol2
0003Ool00`00Oomoo`0IOol00`00Oomoo`0FOol4000@Ool003ioo`03001oogoo01Uoo`03001oogoo
01Moo`03001oo`0001Qoo`D001Qoo`03001oo`00008001aoo`03001oogoo01Eoo`03001oogoo009o
o`03001oogoo00eoo`00?Woo00<007ooOol06Goo00<007ooOol05goo00<007oo000067oo00<007oo
Ool06goo00<007ooOol07Goo00<007ooOol05Goo00<007ooOol00Woo00<007ooOol03Goo000iOol0
0`00Oomoo`02Ool00`00Oomoo`0DOol00`00Oomoo`02Ool00`00Oomoo`0HOol2000HOol00`00Oomo
o`0KOol2000JOol00`00Oomoo`02Ool00`00Oomoo`0DOol00`00Oomoo`02Ool00`00Oomoo`0=Ool0
03Yoo`@001Qoo`@001aoo`03001oogoo01Ioo`D001]oo`<001Moo`H001Moo`@0011oo`00nWoo003j
Ool00?Yoo`00nWoo000JOooE000;Ool001moo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
009oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00ioo`007goo00<007ooOol0f7oo000O
Ool00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo0P00fGoo000OOol00`00Oomoo`3HOol0
01moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`800=Uo
o`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo00<0
07ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`800=Uoo`007goo00<007ooOol0f7oo000OOol0
0`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000OOol2003IOol001mo
o`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000=Ool50005Ool20006Ool00`00Oomoo`3HOol0
00moo`03001oogoo00Aoo`04001oogoo0005Ool00`00Oomoo`3HOol000moo`03001oogoo00=oo`03
001oogoo009oo`03001oogoo009oo`<00=Qoo`003goo00<007ooOol00goo00<007ooOol00Woo00<0
07ooOol00Woo00<007ooOol0f7oo000?Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`3HOol000moo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`03001o
ogoo0=Qoo`003goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0f7oo
000=Ool30006Ool01000Oomoo`001Goo0P00fGoo000>Ool20007Ool20006Ool00`00Oomoo`3HOol0
01moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`03001o
ogoo0=Qoo`007goo0P00fGoo000OOol00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo00<0
07ooOol0f7oo000NOol4O03HOol001aoo`Ql0=Ioo`0077oo27`0eWoo000KOol:O03EOol001]oo`Yl
0=Eoo`006goo2W`0eGoo000KOol:O03EOol001aoo`Ql01Ioo`Al0;aoo`0077oo27`057oo27`0^Woo
000<Ool60005Ool20005Ool4O00FOol8O02jOol000eoo`05001oogooOol00004Ool01000Oomoo`00
1Goo00<007ooOol05Goo2W`0^Goo000>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool3000EOol:O02iOol000moo`03001oogoo00=oo`03001oogoo009oo`03001oogoo009oo`03001o
ogoo01Eoo`Yl0;Uoo`0047oo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool05Goo2W`05Woo17`0Wgoo000AOol01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol0
5Woo27`05Goo27`0WGoo000AOol01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol05Woo
27`05Goo27`0WGoo000<Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01000Oomoo`001Goo00<007oo
Ool067oo17`05Woo2W`0W7oo000=Ool40006Ool20006Ool2000cOol:O02LOol001moo`03001oogoo
039oo`Yl09aoo`007goo00<007ooOol0<Woo2W`05Woo17`0PWoo000OOol00`00Oomoo`0cOol8O00E
Ool8O020Ool001moo`03001oogoo03=oo`Ql01Eoo`Ql081oo`007goo0P00=Woo17`05Woo2W`0Ogoo
000OOol00`00Oomoo`1?Ool:O01oOol001moo`03001oogoo04moo`Yl01Ioo`Al06Eoo`007goo00<0
07ooOol0Cgoo2W`057oo27`0Hgoo000OOol00`00Oomoo`1@Ool8O00EOol8O01SOol001moo`80055o
o`Ql01Aoo`Yl069oo`007goo00<007ooOol0DWoo17`05Woo2W`0HWoo000OOol00`00Oomoo`1/Ool:
O00EOol4O019Ool001moo`03001oogoo06aoo`Yl01=oo`Ql04Moo`007goo00<007ooOol0KGoo27`0
57oo27`0Agoo000OOol2001^Ool8O00COol:O016Ool001moo`03001oogoo06moo`Al01Eoo`Yl01Io
o`Al02aoo`007goo00<007ooOol0R7oo2W`057oo27`0:Woo000=Ool40006Ool20006Ool00`00Oomo
o`28Ool:O00DOol8O00ZOol000aoo`03001oogoo009oo`03001oogoo009oo`04001oogoo0005Ool0
0`00Oomoo`29Ool8O00DOol:O00YOol0015oo`05001oogooOol00004Ool00`00Oomoo`02Ool30029
Ool8O00DOol:O00FOol4O00?Ool0015oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomo
o`2;Ool4O00FOol:O00DOol8O00=Ool000ioo`<000Aoo`03001oogoo009oo`03001oogoo009oo`03
001oogoo0:Eoo`Yl01Aoo`Ql00eoo`004Goo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001o
ogoo0:Ioo`Ql01Aoo`Yl00aoo`004Goo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
0:Ioo`Ql01Aoo`Yl00aoo`0037oo00<007ooOol00Woo00<007ooOol00Woo00@007ooOol000Eoo`03
001oogoo0:Qoo`Al01Ioo`Yl00aoo`003Goo10001Woo0P001Woo0P00`goo2W`037oo000OOol00`00
Oomoo`33Ool8O00=Ool001moo`03001oogoo0<=oo`Ql00eoo`007goo00<007ooOol0aGoo17`03goo
000OOol00`00Oomoo`3HOol001moo`800=Uoo`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3H
Ool001moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000OOol2003IOol001moo`03001oogoo
0=Qoo`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo
0P00fGoo000OOol00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`003goo0`001Goo0P001Woo00<0
07ooOol0f7oo000@Ool00`00Oomoo`03Ool01000Oomoo`001Goo00<007ooOol0f7oo000<Ool60003
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`3HOol000aoo`05001oogooOol00004Ool0
0`00Oomoo`02Ool00`00Oomoo`02Ool3003HOol000eoo`04001oogoo0004Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`3HOol000ioo`03001oo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo0=Qoo`003Woo00<007oo000017oo00<007ooOol00Woo00<007ooOol00Woo00<0
07ooOol0f7oo000?Ool20005Ool01000Oomoo`001Goo00<007ooOol0f7oo000@Ool00`00Oomoo`04
Ool20006Ool2003IOol001moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo000OOol00`00Oomo
o`3HOol001moo`03001oogoo0=Qoo`007goo0P00fGoo000OOol00`00Oomoo`3HOol001moo`03001o
ogoo0=Qoo`007goo00<007ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`800=Uoo`007goo00<0
07ooOol0f7oo000OOol00`00Oomoo`3HOol001moo`03001oogoo0=Qoo`007goo00<007ooOol0f7oo
000OOol00`00Oomoo`3HOol001moo`800=Uoo`007goo00<007ooOol0f7oo000=Ool40006Ool20006
Ool00`00Oomoo`3HOol000aoo`03001oogoo009oo`03001oogoo009oo`04001oogoo0005Ool00`00
Oomoo`3HOol0015oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`3HOol0015oo`05
001oogooOol00004Ool00`00Oomoo`02Ool3003HOol000aoo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`3MOol000aoo`D000Aoo`03001oogoo009oo`03001oogoo0=eoo`0037oo00<0
07ooOol01Woo00<007ooOol00Woo00<007ooOol0gGoo000<Ool00`00Oomoo`07Ool01000Oomoo`00
h7oo000<Ool50006Ool2003QOol00?Yoo`00\
\>"],
  ImageRangeCache->{{{0, 249.5}, {153.688, 0}} -> {-0.103731, -6.17239, 
  0.0520969, 0.57343}}]
}, Open  ]],

Cell[TextData[{
  "Note the use of ",
  StyleBox["PlotStyle",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", and the fact that the first \"",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "-value\" is 1."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
ListPlot[polydatab, PlotRange -> {0, 40},
PlotJoined -> True];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.384354 0.136054 0 0.0154508 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -6 -9 ]
[.97619 -0.0125 6 0 ]
[.01131 .07725 -6 -4.5 ]
[.01131 .07725 0 4.5 ]
[.01131 .15451 -12 -4.5 ]
[.01131 .15451 0 4.5 ]
[.01131 .23176 -12 -4.5 ]
[.01131 .23176 0 4.5 ]
[.01131 .30902 -12 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .38627 -12 -4.5 ]
[.01131 .38627 0 4.5 ]
[.01131 .46353 -12 -4.5 ]
[.01131 .46353 0 4.5 ]
[.01131 .54078 -12 -4.5 ]
[.01131 .54078 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(4)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(5)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(6)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(7)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(8)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(9)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(10)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .07725 m
.03006 .07725 L
s
[(5)] .01131 .07725 1 0 Mshowa
.02381 .15451 m
.03006 .15451 L
s
[(10)] .01131 .15451 1 0 Mshowa
.02381 .23176 m
.03006 .23176 L
s
[(15)] .01131 .23176 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(20)] .01131 .30902 1 0 Mshowa
.02381 .38627 m
.03006 .38627 L
s
[(25)] .01131 .38627 1 0 Mshowa
.02381 .46353 m
.03006 .46353 L
s
[(30)] .01131 .46353 1 0 Mshowa
.02381 .54078 m
.03006 .54078 L
s
[(35)] .01131 .54078 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(40)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .01545 m
.02756 .01545 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .04635 m
.02756 .04635 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .10816 m
.02756 .10816 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .13906 m
.02756 .13906 L
s
.02381 .16996 m
.02756 .16996 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20086 m
.02756 .20086 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26266 m
.02756 .26266 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .29357 m
.02756 .29357 L
s
.02381 .32447 m
.02756 .32447 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .35537 m
.02756 .35537 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .41717 m
.02756 .41717 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .44807 m
.02756 .44807 L
s
.02381 .47898 m
.02756 .47898 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .50988 m
.02756 .50988 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57168 m
.02756 .57168 L
s
.02381 .58713 m
.02756 .58713 L
s
.02381 .60258 m
.02756 .60258 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .08028 m
.15986 .12361 L
.29592 .17275 L
.43197 .22708 L
.56803 .28615 L
.70408 .34961 L
.84014 .41717 L
.97619 .4886 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{167, 103.125},
  ImageMargins->{{34, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003j0000VQ000`40O003h00OnWoo003jOol00?Yo
o`00>Woo0`005goo10006Goo10006Woo00<007ooOol06Goo100067oo0`005Woo1@001Goo0P003Woo
000kOol00`00Oomoo`0EOol00`00Oomoo`02Ool00`00Oomoo`0EOol00`00Oomoo`02Ool00`00Oomo
o`0HOol00`00Oomoo`0GOol00`00Oomoo`02Ool00`00Oomoo`0HOol2000FOol00`00Oomoo`04Ool0
1000Oomoo`003Goo000gOol6000KOol00`00Oomoo`0EOol00`00Oomoo`02Ool00`00Oomoo`0HOol0
0`00Oomoo`0GOol00`00Oomoo`02Ool00`00Oomoo`0IOol00`00Oomoo`0DOol00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`0:Ool003Moo`05001oogooOol0000LOol00`00Oomoo`0EOol00`00
Oomoo`02Ool00`00Oomoo`0HOol00`00Oomoo`0GOol00`00Oomoo`02Ool00`00Oomoo`0FOol30000
0goo001oo`0DOol00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`0:Ool003Qoo`04001oogoo
000GOol00`00Oomoo`02Ool00`00Oomoo`0EOol20003Ool00`00Oomoo`0IOol00`00Oomoo`0GOol4
000HOol01000Oomoogoo0P005Goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol02Woo000i
Ool00`00Ool0000GOol5000HOol00`00Ool00002000LOol00`00Oomoo`0FOol00`00Oomoo`02Ool0
0`00Oomoo`0EOol00`00Oomoo`02Ool00`00Oomoo`0COol00`00Oomoo`03Ool00`00Oomoo`02Ool0
0`00Oomoo`0:Ool003Uoo`03001oo`0001Moo`03001oogoo01]oo`03001oogoo01eoo`03001oogoo
01Ioo`03001oogoo009oo`03001oogoo01Eoo`03001oogoo009oo`03001oogoo01=oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo00Yoo`00>Woo0P005goo00<007ooOol06goo0P006Woo00<0
07ooOol00Woo00<007ooOol05Goo00<007ooOol00Woo00<007ooOol05Goo00<007ooOol00Woo00<0
07ooOol04Goo0`001Woo00@007ooOol000eoo`00>goo00<007ooOol05Goo1@006goo0`005goo1P00
67oo10006Goo10005Goo0P001goo0P003Woo003jOol00?Yoo`00nWoo003jOol001QoomD000eoo`00
7Goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0goo00<007ooOol047oo000MOol00`00Oomoo`3JOol001eoo`800=]oo`007Goo00<007ooOol0fWoo
000MOol00`00Oomoo`3JOol001eoo`800=]oo`007Goo00<007ooOol0fWoo000MOol00`00Oomoo`3J
Ool001eoo`03001oogoo0=Yoo`007Goo0P00fgoo000MOol00`00Oomoo`3JOol001eoo`03001oogoo
0=Yoo`007Goo0P00fgoo000DOol40005Ool00`00Oomoo`3JOol001=oo`03001oogoo009oo`03001o
ogoo009oo`03001oogoo0=Yoo`0067oo00<007ooOol00Woo0`00fWoo000HOol00`00Oomoo`02Ool3
003JOol001=oo`03001oogoo009oo`03001oogoo009oo`03001oogoo00<00=Moo`004goo1@001Goo
00<007ooOol00goo0`00e7oo000COol00`00Oomoo`07Ool20007Ool3003AOol001=oo`03001oogoo
00Moo`03001oogoo00Uoo`@00<eoo`004goo1@001Goo00<007ooOol03Goo0`00bWoo000MOol2000A
Ool30037Ool001eoo`03001oogoo01=oo`<00<Aoo`007Goo00<007ooOol05Woo0`00`Goo000MOol2
000JOol3002nOol001eoo`03001oogoo01aoo`<00;]oo`007Goo00<007ooOol07goo0`00^7oo000M
Ool2000SOol2002fOol000]oo`D000Eoo`8000Ioo`03001oogoo02Aoo`<00;=oo`003Goo00<007oo
Ool017oo00@007ooOol000Eoo`03001oogoo02Moo`<00;1oo`003Goo00<007ooOol00goo00<007oo
Ool00Woo00<007ooOol00Woo00<007ooOol0:Woo0P00[Woo000=Ool00`00Oomoo`03Ool00`00Oomo
o`02Ool00`00Oomoo`02Ool3000/Ool3002[Ool000eoo`03001oogoo00=oo`03001oogoo009oo`03
001oogoo009oo`03001oogoo02moo`800:Uoo`003Goo00<007ooOol00goo00<007ooOol00Woo00<0
07ooOol00Woo00<007ooOol0<Goo0`00YWoo000=Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool2000eOol3002SOol000]oo`<000Ioo`04001oogoo0005Ool00`00Oomoo`0gOol2002Q
Ool000aoo`8000Moo`8000Ioo`03001oogoo03Uoo`<009ioo`007Goo0P00?Goo0`00Vgoo000MOol0
0`00Oomoo`0oOol2002IOol001eoo`03001oogoo045oo`<009Ioo`007Goo0P00AGoo0`00Tgoo000M
Ool00`00Oomoo`17Ool2002AOol001eoo`03001oogoo04Uoo`<008ioo`007Goo00<007ooOol0C7oo
0P00S7oo000MOol2001?Ool30029Ool000]oo`D000Aoo`@000Eoo`03001oogoo055oo`<008Ioo`00
3Goo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0E7oo0P00Q7oo000=
Ool00`00Oomoo`08Ool00`00Oomoo`02Ool3001FOol20022Ool000eoo`03001oogoo00Qoo`03001o
ogoo009oo`03001oogoo05Qoo`80081oo`003Goo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol0FWoo0`00OGoo000=Ool00`00Oomoo`03Ool50005Ool2001NOol2001kOol0
00eoo`03001oogoo00=oo`03001oogoo00Moo`03001oogoo05moo`8007Uoo`002goo0`001Goo00<0
07ooOol01goo00<007ooOol0HGoo0P00Mgoo000<Ool20005Ool50005Ool00`00Oomoo`1SOol2001e
Ool001eoo`8006Ioo`8007=oo`007Goo00<007ooOol0Igoo0P00LGoo000MOol00`00Oomoo`1YOol3
001^Ool001eoo`8006eoo`8006aoo`007Goo00<007ooOol0KWoo0P00JWoo000MOol00`00Oomoo`1`
Ool2001XOol001eoo`8007=oo`8006Ioo`002Woo1P001Goo0P001Woo00<007ooOol0M7oo0`00Hgoo
000;Ool01@00Oomoogoo000017oo00@007ooOol000Eoo`03001oogoo07Moo`80065oo`0037oo00<0
07ooOol017oo00<007ooOol00Woo00<007ooOol00Woo0`00NGoo0P00Ggoo000=Ool00`00Oomoo`03
Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`1kOol2001MOol000ioo`03001oogoo009o
o`03001oogoo009oo`03001oogoo009oo`03001oogoo07eoo`<005Yoo`003goo00D007ooOomoo`00
00Aoo`03001oogoo009oo`03001oogoo081oo`8005Qoo`003goo00D007ooOomoo`0000Aoo`03001o
ogoo009oo`8008=oo`8005Ioo`002Woo00<007ooOol00Woo00<007ooOol00Woo00@007ooOol000Eo
o`03001oogoo08Aoo`8005Aoo`002goo10001Woo0P001Woo00<007ooOol0QWoo0P00DWoo000MOol2
0029Ool3001?Ool001eoo`03001oogoo08]oo`8004eoo`007Goo00<007ooOol0SGoo0P00Bgoo000M
Ool2002@Ool20019Ool001eoo`03001oogoo095oo`8004Moo`007Goo00<007ooOol0Tgoo0P00AGoo
000MOol2002FOol20013Ool000Yoo`H000Aoo`@000Eoo`03001oogoo09Moo`80045oo`002goo00D0
07ooOomoo`0000=oo`03001oogoo009oo`03001oogoo009oo`03001oogoo09Uoo`8003moo`0037oo
00<007ooOol02Goo00<007ooOol00Woo00<007ooOol0Vgoo0`00?7oo000=Ool00`00Oomoo`08Ool0
0`00Oomoo`02Ool3002NOol2000jOol000ioo`03001oogoo009oo`03001oogoo009oo`03001oogoo
009oo`03001oogoo0:1oo`8003Qoo`003goo00@007ooOomoo`D000Eoo`03001oogoo0:9oo`8003Io
o`003goo00D007ooOomoo`0000Uoo`800:Eoo`8003Aoo`002Woo00<007ooOol00Woo00D007ooOomo
o`0000Uoo`03001oogoo0:Ioo`80039oo`002goo100017oo1@001Goo00<007ooOol0Z7oo0P00<7oo
000MOol2002[Ool2000^Ool001eoo`03001oogoo0:aoo`8002aoo`007Goo00<007ooOol0[Woo0P00
:Woo000MOol00`00Oomoo`2`Ool2000XOol001eoo`800;=oo`8002Ioo`007Goo00<007ooOol0]7oo
0P0097oo000MOol00`00Oomoo`2fOol2000ROol001eoo`800;Uoo`80021oo`002goo10001Woo0P00
1Woo00<007ooOol0^Woo00<007ooOol07Goo000:Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01000
Oomoo`001Goo00<007ooOol0^goo0P007Goo000?Ool01@00Oomoogoo000017oo00<007ooOol00Woo
0`00_Goo0P006goo000?Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol0_goo0P00
6Goo000<Ool30004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`31Ool2000GOol000mo
o`05001oogooOol00004Ool00`00Oomoo`02Ool20034Ool2000EOol000moo`05001oogooOol00004
Ool00`00Oomoo`02Ool00`00Oomoo`35Ool2000COol000Yoo`03001oogoo009oo`03001oogoo009o
o`04001oogoo0005Ool00`00Oomoo`37Ool00`00Oomoo`0@Ool000]oo`@000Ioo`8000Ioo`03001o
ogoo0=Yoo`007Goo0P00fgoo000MOol00`00Oomoo`3JOol001eoo`03001oogoo0=Yoo`007Goo0P00
fgoo000MOol00`00Oomoo`3JOol001eoo`03001oogoo0=Yoo`007Goo0P00fgoo000;Ool40005Ool4
0005Ool00`00Oomoo`3JOol000Yoo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02
Ool00`00Oomoo`3JOol000moo`03001oogoo00Ioo`03001oogoo009oo`<00=Yoo`003goo00<007oo
Ool01Woo00<007ooOol00Woo00<007ooOol0fWoo000<Ool30004Ool00`00Oomoo`02Ool00`00Oomo
o`02Ool00`00Oomoo`3JOol000moo`04001oogooOol50005Ool00`00Oomoo`3JOol000moo`05001o
ogooOol00009Ool2003KOol000Yoo`03001oogoo009oo`05001oogooOol00009Ool00`00Oomoo`3J
Ool000]oo`@000Aoo`D000Eoo`03001oogoo0=Yoo`007Goo0P00fgoo000MOol00`00Oomoo`3JOol0
01eoo`03001oogoo0=Yoo`007Goo0P00fgoo000MOol00`00Oomoo`3JOol001eoo`03001oogoo0=Yo
o`007Goo0P00fgoo000=Ool30005Ool20006Ool00`00Oomoo`3JOol000ioo`03001oogoo00=oo`04
001oogoo0005Ool00`00Oomoo`3JOol000Yoo`H000=oo`03001oogoo009oo`03001oogoo009oo`03
001oogoo0=Yoo`002Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo009oo`<00=Yo
o`002goo00@007ooOol000Aoo`03001oogoo009oo`03001oogoo0=moo`0037oo00<007oo000017oo
00<007ooOol00Woo00<007ooOol0ggoo000<Ool00`00Ool00004Ool00`00Oomoo`02Ool00`00Oomo
o`3OOol000eoo`8000Eoo`04001oogoo003ROol000ioo`03001oogoo00Aoo`800>=oo`00nWoo0000
\
\>"],
  ImageRangeCache->{{{0, 249.5}, {153.688, 0}} -> {1.96507, -4.93794, 
  0.0520969, 0.458746}}]
}, Open  ]],

Cell[TextData[{
  "Be careful about whether you are dealing with just values or pairs of \
values. Similar log plots are available in the graphics Package. Perhaps one \
of the more irritating features of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is the absence of a core kernel function for plotting several sets of \
data simultaneously on one graph. There are two ways around this. First, we \
can always use ",
  StyleBox["Show",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to combine previously created plots:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
polydatac = Table[i, {i, 3, 10}];

blueblobs = ListPlot[polydatac, 
PlotStyle -> {RGBColor[0, 0, 1], PointSize[0.02]}, 
PlotRange -> {0, 40}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.112245 0.136054 0 0.0154508 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .07725 -6 -4.5 ]
[.01131 .07725 0 4.5 ]
[.01131 .15451 -12 -4.5 ]
[.01131 .15451 0 4.5 ]
[.01131 .23176 -12 -4.5 ]
[.01131 .23176 0 4.5 ]
[.01131 .30902 -12 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .38627 -12 -4.5 ]
[.01131 .38627 0 4.5 ]
[.01131 .46353 -12 -4.5 ]
[.01131 .46353 0 4.5 ]
[.01131 .54078 -12 -4.5 ]
[.01131 .54078 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(2)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(3)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(4)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(5)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(6)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(7)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(8)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .07725 m
.03006 .07725 L
s
[(5)] .01131 .07725 1 0 Mshowa
.02381 .15451 m
.03006 .15451 L
s
[(10)] .01131 .15451 1 0 Mshowa
.02381 .23176 m
.03006 .23176 L
s
[(15)] .01131 .23176 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(20)] .01131 .30902 1 0 Mshowa
.02381 .38627 m
.03006 .38627 L
s
[(25)] .01131 .38627 1 0 Mshowa
.02381 .46353 m
.03006 .46353 L
s
[(30)] .01131 .46353 1 0 Mshowa
.02381 .54078 m
.03006 .54078 L
s
[(35)] .01131 .54078 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(40)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .01545 m
.02756 .01545 L
s
.02381 .0309 m
.02756 .0309 L
s
.02381 .04635 m
.02756 .04635 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .10816 m
.02756 .10816 L
s
.02381 .12361 m
.02756 .12361 L
s
.02381 .13906 m
.02756 .13906 L
s
.02381 .16996 m
.02756 .16996 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .20086 m
.02756 .20086 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .24721 m
.02756 .24721 L
s
.02381 .26266 m
.02756 .26266 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .29357 m
.02756 .29357 L
s
.02381 .32447 m
.02756 .32447 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .35537 m
.02756 .35537 L
s
.02381 .37082 m
.02756 .37082 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .41717 m
.02756 .41717 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .44807 m
.02756 .44807 L
s
.02381 .47898 m
.02756 .47898 L
s
.02381 .49443 m
.02756 .49443 L
s
.02381 .50988 m
.02756 .50988 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .57168 m
.02756 .57168 L
s
.02381 .58713 m
.02756 .58713 L
s
.02381 .60258 m
.02756 .60258 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
0 0 1 r
.02 w
.02381 .04635 Mdot
.15986 .0618 Mdot
.29592 .07725 Mdot
.43197 .09271 Mdot
.56803 .10816 Mdot
.70408 .12361 Mdot
.84014 .13906 Mdot
.97619 .15451 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{167, 103.125},
  ImageMargins->{{34, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002W0000Ia000`40O003h00OYgoo002WOol002Io
o`@000moo`<0015oo`<000moo`<0015oo`80019oo`03001oogoo011oo`8000]oo`009goo00<007oo
Ool04Woo00<007ooOol03goo00<007ooOol04Goo00<007ooOol03Goo00@007ooOol0015oo`03001o
ogoo00moo`04001oogoo000:Ool002Qoo`03001oogoo015oo`03001oogoo00eoo`@0019oo`03001o
ogoo00eoo`04001oogoo000BOol00`00Oomoo`0>Ool01000Oomoo`002Woo000YOol00`00Oomoo`0>
Ool2000@Ool00`00Ool0000@Ool3000@Ool3000COol00`00Oomoo`0?Ool2000;Ool002Ioo`04001o
ogoo000BOol00`00Oomoo`0=Ool00`00Ool0000@Ool00`00Oomoo`0@Ool00`00Oomoo`0DOol00`00
Oomoo`0=Ool01000Oomoo`002Woo000WOol2000@Ool3000AOol2000@Ool4000@Ool3000@Ool4000@
Ool2000;Ool00:Moo`00Ygoo002WOol0015oohh000Qoo`005Goo00D007ooOomoo`00009oo`05001o
ogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`00009oo`05001oogooOol00003Ool0
1@00Oomoogoo00000goo00@007ooOol000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00@007ooOol000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo00@007ooOol000=oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00@007ooOol000=oo`05001oogooOol00003Ool0
1@00Oomoogoo00002goo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007oo
Ool0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`0057oo0P0OTGoo000DOol201nA
Ool000ioo`<000Aoo`03001oogoo00moo`807gioo`004Goo00D007ooOomoo`00015oo`807gioo`00
4Goo00@007ooOomoo`8002=oo`807f]oo`003Woo0`0017oo00<007ooOol08Woo0P0OJgoo000>Ool0
0`00Oomoo`04Ool00`00Oomoo`0eOol201mHOol000ioo`@000=oo`03001oogoo03Eoo`807eQoo`00
5Goo00<007ooOol0B7oo0P0OAGoo000EOol00`00Oomoo`18Ool201m5Ool001Eoo`03001oogoo05ao
o`807c5oo`005Goo00<007ooOol0G7oo0P0O<Goo000EOol00`00Oomoo`2?Ool000Qoo`<000Aoo`80
00Aoo`03001oogoo06moo`807aioo`002Goo00<007ooOol00Woo00@007ooOol000=oo`03001oogoo
06moo`807aioo`002Goo00<007ooOol00Woo00@007ooOol000=oo`8008=oo`807`]oo`002Goo00<0
07ooOol00Woo00@007ooOol000=oo`03001oogoo089oo`807`]oo`002Goo00<007ooOol00Woo00@0
07ooOol000=oo`03001oogoo08moo`0027oo0P001Goo0P0017oo00<007ooOol0Sgoo000EOol00`00
Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool0
01Eoo`03001oogoo08moo`0027oo0`000goo0`0017oo00<007ooOol0Sgoo0009Ool00`00Oomoo`05
Ool01@00Oomoogoo0000TGoo0009Ool00`00Oomoo`05Ool01000Oomoogoo0P00T7oo0009Ool00`00
Oomoo`02Ool30004Ool00`00Oomoo`2?Ool000Uoo`03001oogoo009oo`03001oogoo00Aoo`03001o
ogoo08moo`0027oo0P0017oo10000goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03
001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08mo
o`0027oo10000goo0P0017oo00<007ooOol0Sgoo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo
00<007ooOol0Sgoo000:Ool01@00Oomoogoo00000Woo00@007ooOomoo`80091oo`002goo00@007oo
Ool0009oo`05001oogooOol0002AOol000Qoo`04001oogoo0002Ool01000Oomoo`000goo00<007oo
Ool0Sgoo0009Ool20004Ool20004Ool00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<0
07ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`0027oo10000Woo0`0017oo
00<007ooOol0Sgoo0009Ool00`00Oomoo`05Ool01@00Oomoogoo0000TGoo000:Ool00`00Oomoo`04
Ool01000Oomoogoo0P00T7oo000;Ool00`00Oomoo`030004Ool00`00Oomoo`2?Ool000Qoo`04001o
ogoo0002Ool00`00Oomoo`04Ool00`00Oomoo`2?Ool000Uoo`8000=oo`@000=oo`03001oogoo08mo
o`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<0
07ooOol0Sgoo000EOol00`00Oomoo`2?Ool000Qoo`<000Aoo`8000Aoo`03001oogoo08moo`002goo
00@007ooOol0009oo`05001oogooOol0002AOol000]oo`04001oogoo0002Ool01000Oomoogoo0P00
T7oo0009Ool20003Ool01000Oomoo`000goo00<007ooOol0Sgoo000;Ool01000Oomoo`000Woo00D0
07ooOomoo`00095oo`0027oo0`0017oo0P0017oo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool0
01Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001o
ogoo08moo`0027oo0`000goo0`0017oo00<007ooOol0Sgoo000;Ool00`00Oomoo`03Ool01@00Oomo
ogoo0000TGoo000;Ool00`00Oomoo`03Ool01000Oomoogoo0P00T7oo0009Ool20003Ool30004Ool0
0`00Oomoo`2?Ool000]oo`04001oogoo0006Ool00`00Oomoo`2?Ool000Qoo`<000=oo`@000=oo`03
001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08mo
o`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool000Uoo`<000=oo`8000Aoo`03001oogoo
08moo`002Woo00D007ooOomoo`00009oo`05001oogooOol0002AOol000Qoo`@0009oo`04001oogoo
0003Ool2002@Ool000Qoo`03001oo`0000=oo`04001oogoo002EOol000Qoo`03001oo`0000=oo`04
001oogoo002EOol000Uoo`8000Aoo`8009Ioo`00Ygoo0000\
\>"],
  ImageRangeCache->{{{0, 166}, {102.125, 0}} -> {-0.112487, -4.95347, 
  0.0522929, 0.460472}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Show[redblobs, blueblobs];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.119048 0 0.0123607 [
[.2619 -0.0125 -3 -9 ]
[.2619 -0.0125 3 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.7381 -0.0125 -3 -9 ]
[.7381 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .12361 -12 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -12 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -12 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -12 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 0 m
.2619 .00625 L
s
[(2)] .2619 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(4)] .5 -0.0125 0 1 Mshowa
.7381 0 m
.7381 .00625 L
s
[(6)] .7381 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(8)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.08333 0 m
.08333 .00375 L
s
.14286 0 m
.14286 .00375 L
s
.20238 0 m
.20238 .00375 L
s
.32143 0 m
.32143 .00375 L
s
.38095 0 m
.38095 .00375 L
s
.44048 0 m
.44048 .00375 L
s
.55952 0 m
.55952 .00375 L
s
.61905 0 m
.61905 .00375 L
s
.67857 0 m
.67857 .00375 L
s
.79762 0 m
.79762 .00375 L
s
.85714 0 m
.85714 .00375 L
s
.91667 0 m
.91667 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(10)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(20)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(30)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(40)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(50)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .02472 m
.02756 .02472 L
s
.02381 .04944 m
.02756 .04944 L
s
.02381 .07416 m
.02756 .07416 L
s
.02381 .09889 m
.02756 .09889 L
s
.02381 .14833 m
.02756 .14833 L
s
.02381 .17305 m
.02756 .17305 L
s
.02381 .19777 m
.02756 .19777 L
s
.02381 .22249 m
.02756 .22249 L
s
.02381 .27193 m
.02756 .27193 L
s
.02381 .29666 m
.02756 .29666 L
s
.02381 .32138 m
.02756 .32138 L
s
.02381 .3461 m
.02756 .3461 L
s
.02381 .39554 m
.02756 .39554 L
s
.02381 .42026 m
.02756 .42026 L
s
.02381 .44498 m
.02756 .44498 L
s
.02381 .46971 m
.02756 .46971 L
s
.02381 .51915 m
.02756 .51915 L
s
.02381 .54387 m
.02756 .54387 L
s
.02381 .56859 m
.02756 .56859 L
s
.02381 .59331 m
.02756 .59331 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.05 w
.14286 .21409 Mdot
.2619 .24721 Mdot
.38095 .27639 Mdot
.5 .30277 Mdot
.61905 .32703 Mdot
.7381 .34961 Mdot
.85714 .37082 Mdot
.97619 .39088 Mdot
0 0 1 r
.02 w
.14286 .03708 Mdot
.2619 .04944 Mdot
.38095 .0618 Mdot
.5 .07416 Mdot
.61905 .08652 Mdot
.7381 .09889 Mdot
.85714 .11125 Mdot
.97619 .12361 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{167, 103.125},
  ImageMargins->{{34, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002W0000Ia000`40O003h00OYgoo002WOol003Ao
o`@001moo`<001ioo`80021oo`8000]oo`00=Goo00<007ooOol087oo00<007ooOol077oo00@007oo
Ool001ioo`04001oogoo000:Ool003Ioo`03001oogoo01eoo`@001eoo`04001oogoo000NOol01000
Oomoo`002Woo000gOol00`00Oomoo`0LOol00`00Ool0000NOol3000POol2000;Ool003Aoo`04001o
ogoo000NOol00`00Ool0000NOol00`00Oomoo`0OOol01000Oomoo`002Woo000eOol2000POol2000O
Ool3000OOol2000;Ool00:Moo`00Ygoo002WOol0015oohh000Qoo`005Goo00<007ooOol01Goo00<0
07ooOol01Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol01Goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<0
07ooOol02Goo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo
000EOol00`00Oomoo`0=Ool201n0Ool001Eoo`03001oogoo00eoo`807h1oo`005Goo00<007ooOol0
7Goo0P0OL7oo000EOol00`00Oomoo`0MOol201m`Ool001Eoo`03001oogoo02ioo`807emoo`005Goo
00<007ooOol0;Woo0P0O3goo0P0OCWoo000EOol00`00Oomoo`0oOol201m>Ool001Eoo`03001oogoo
051oo`807ceoo`005Goo00<007ooOol0D7oo0P0O?Goo000EOol00`00Oomoo`1POol201l]Ool001Eo
o`03001oogoo061oo`807beoo`0027oo0`0017oo0P0017oo00<007ooOol0LGoo0P0O77oo0009Ool0
0`00Oomoo`02Ool01000Oomoo`000goo00<007ooOol0LGoo0P0O3goo0P0O2goo0009Ool00`00Oomo
o`02Ool01000Oomoo`000goo0P00Pgoo0P0O2goo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo
00<007ooOol0Sgoo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007ooOol0Sgoo0008Ool2
0005Ool20004Ool00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000E
Ool00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomo
o`0=Ool3O01oOol001Eoo`03001oogoo00aoo`El07ioo`005Goo00<007ooOol02goo1g`0OGoo000E
Ool00`00Oomoo`0;Ool7O01mOol001Eoo`03001oogoo00]oo`Ml07eoo`005Goo00<007ooOol037oo
1G`037oo0g`0Kgoo0008Ool40003Ool20004Ool00`00Oomoo`0=Ool3O00<Ool5O01^Ool000Uoo`03
001oogoo009oo`04001oogoo0003Ool00`00Oomoo`0KOol7O01]Ool000Yoo`05001oogooOol00002
Ool01000Oomoogoo0P0077oo1g`0KGoo000;Ool01000Oomoo`000Woo00D007ooOomoo`0001eoo`Ml
00aoo`=l05ioo`0027oo00@007ooOol0009oo`04001oogoo0003Ool00`00Oomoo`0LOol5O00<Ool5
O01MOol000Uoo`8000Aoo`8000Aoo`03001oogoo01eoo`=l00aoo`Ml05aoo`005Goo00<007ooOol0
;7oo1g`0G7oo000EOol00`00Oomoo`0/Ool7O00<Ool3O01=Ool001Eoo`03001oogoo02eoo`El00ao
o`El04aoo`005Goo00<007ooOol0;Woo0g`037oo1g`0Bgoo000EOol00`00Oomoo`0mOol7O00;Ool3
O00mOol001Eoo`03001oogoo03eoo`Ml00Yoo`El03aoo`005Goo00<007ooOol0?Woo1G`02Woo1g`0
>goo000EOol00`00Oomoo`0oOol3O00;Ool7O00<Ool3O00/Ool001Eoo`03001oogoo04eoo`Ml00]o
o`El02]oo`005Goo00<007ooOol0CWoo1G`02goo1g`0:Woo000EOol00`00Oomoo`1?Ool3O00<Ool7
O00<Ool3O00KOol000Qoo`<000Aoo`8000Aoo`03001oogoo05ioo`Ml00]oo`El01Yoo`002goo00@0
07ooOol0009oo`05001oogooOol0001QOol5O00;Ool7O00IOol000]oo`04001oogoo0002Ool01000
Oomoogoo0P00HGoo0g`037oo1g`037oo0g`02Woo0009Ool20003Ool01000Oomoo`000goo00<007oo
Ool0Kgoo1g`02goo1G`02Goo000;Ool01000Oomoo`000Woo00D007ooOomoo`00079oo`El00]oo`Ml
00Qoo`0027oo0`0017oo0P0017oo00<007ooOol0LGoo0g`037oo1g`027oo000EOol00`00Oomoo`20
Ool7O008Ool001Eoo`03001oogoo085oo`El00Uoo`005Goo00<007ooOol0PWoo0g`02Woo000EOol0
0`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?
Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03
001oogoo08moo`005Goo00<007ooOol0Sgoo0009Ool30003Ool20004Ool00`00Oomoo`2?Ool000Yo
o`05001oogooOol00002Ool01@00Oomoogoo0000TGoo0008Ool40002Ool01000Oomoo`000goo0P00
T7oo0008Ool00`00Ool00003Ool01000Oomoo`000goo00<007ooOol0Sgoo0008Ool00`00Ool00003
Ool01000Oomoo`000goo00<007ooOol0Sgoo0009Ool20004Ool20004Ool00`00Oomoo`2?Ool001Eo
o`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo
08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo
00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0
Sgoo0008Ool30004Ool20004Ool00`00Oomoo`2?Ool000]oo`04001oogoo0002Ool01@00Oomoogoo
0000TGoo000;Ool01000Oomoo`000Woo00@007ooOomoo`80091oo`0027oo0`000goo00@007ooOol0
09Eoo`0027oo00<007ooOol00goo00@007ooOol009Eoo`0027oo10000goo0P00UWoo002WOol00001
\
\>"],
  ImageRangeCache->{{{0, 166}, {102.125, 0}} -> {-1.27141, -6.19181, 
  0.0597629, 0.575587}}]
}, Open  ]],

Cell[TextData[{
  "Secondly we can use the function ",
  StyleBox["MultipleListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " which is only available in the ",
  StyleBox["MultipleListPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " Package."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Needs[\"Graphics`MultipleListPlot`\"];
lista = {5, 7, 4, 3}; listb = {9, 8, 7, 6};\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell["\<\
MultipleListPlot[lista, listb, listc,
PlotJoined -> True, PlotRange -> {0, 10},
LineStyles -> {{RGBColor[1, 0, 0]}, {RGBColor[0, 1, 0]},
{RGBColor[0, 0, 1]}}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.293651 0.31746 0 0.0618034 [
[.18254 -0.0125 -9 -9 ]
[.18254 -0.0125 9 0 ]
[.34127 -0.0125 -3 -9 ]
[.34127 -0.0125 3 0 ]
[.5 -0.0125 -9 -9 ]
[.5 -0.0125 9 0 ]
[.65873 -0.0125 -3 -9 ]
[.65873 -0.0125 3 0 ]
[.81746 -0.0125 -9 -9 ]
[.81746 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .12361 -6 -4.5 ]
[.01131 .12361 0 4.5 ]
[.01131 .24721 -6 -4.5 ]
[.01131 .24721 0 4.5 ]
[.01131 .37082 -6 -4.5 ]
[.01131 .37082 0 4.5 ]
[.01131 .49443 -6 -4.5 ]
[.01131 .49443 0 4.5 ]
[.01131 .61803 -12 -4.5 ]
[.01131 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 0 m
.18254 .00625 L
s
[(1.5)] .18254 -0.0125 0 1 Mshowa
.34127 0 m
.34127 .00625 L
s
[(2)] .34127 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(2.5)] .5 -0.0125 0 1 Mshowa
.65873 0 m
.65873 .00625 L
s
[(3)] .65873 -0.0125 0 1 Mshowa
.81746 0 m
.81746 .00625 L
s
[(3.5)] .81746 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(4)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05556 0 m
.05556 .00375 L
s
.0873 0 m
.0873 .00375 L
s
.11905 0 m
.11905 .00375 L
s
.15079 0 m
.15079 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.24603 0 m
.24603 .00375 L
s
.27778 0 m
.27778 .00375 L
s
.30952 0 m
.30952 .00375 L
s
.37302 0 m
.37302 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.43651 0 m
.43651 .00375 L
s
.46825 0 m
.46825 .00375 L
s
.53175 0 m
.53175 .00375 L
s
.56349 0 m
.56349 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62698 0 m
.62698 .00375 L
s
.69048 0 m
.69048 .00375 L
s
.72222 0 m
.72222 .00375 L
s
.75397 0 m
.75397 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.84921 0 m
.84921 .00375 L
s
.88095 0 m
.88095 .00375 L
s
.9127 0 m
.9127 .00375 L
s
.94444 0 m
.94444 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .12361 m
.03006 .12361 L
s
[(2)] .01131 .12361 1 0 Mshowa
.02381 .24721 m
.03006 .24721 L
s
[(4)] .01131 .24721 1 0 Mshowa
.02381 .37082 m
.03006 .37082 L
s
[(6)] .01131 .37082 1 0 Mshowa
.02381 .49443 m
.03006 .49443 L
s
[(8)] .01131 .49443 1 0 Mshowa
.02381 .61803 m
.03006 .61803 L
s
[(10)] .01131 .61803 1 0 Mshowa
.125 Mabswid
.02381 .0309 m
.02756 .0309 L
s
.02381 .0618 m
.02756 .0618 L
s
.02381 .09271 m
.02756 .09271 L
s
.02381 .15451 m
.02756 .15451 L
s
.02381 .18541 m
.02756 .18541 L
s
.02381 .21631 m
.02756 .21631 L
s
.02381 .27812 m
.02756 .27812 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33992 m
.02756 .33992 L
s
.02381 .40172 m
.02756 .40172 L
s
.02381 .43262 m
.02756 .43262 L
s
.02381 .46353 m
.02756 .46353 L
s
.02381 .52533 m
.02756 .52533 L
s
.02381 .55623 m
.02756 .55623 L
s
.02381 .58713 m
.02756 .58713 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
1 0 0 r
.001 w
.02381 .30902 m
.34127 .43262 L
.65873 .24721 L
.97619 .18541 L
s
0 g
.0001 w
.02381 .30902 -1.875 0 Mabsadd m
.02381 .30902 0 2.5 Mabsadd L
.02381 .30902 1.875 0 Mabsadd L
.02381 .30902 0 -2.5 Mabsadd L
.02381 .30902 -1.875 0 Mabsadd L closepath F
.34127 .43262 -1.875 0 Mabsadd m
.34127 .43262 0 2.5 Mabsadd L
.34127 .43262 1.875 0 Mabsadd L
.34127 .43262 0 -2.5 Mabsadd L
.34127 .43262 -1.875 0 Mabsadd L closepath F
.65873 .24721 -1.875 0 Mabsadd m
.65873 .24721 0 2.5 Mabsadd L
.65873 .24721 1.875 0 Mabsadd L
.65873 .24721 0 -2.5 Mabsadd L
.65873 .24721 -1.875 0 Mabsadd L closepath F
.97619 .18541 -1.875 0 Mabsadd m
.97619 .18541 0 2.5 Mabsadd L
.97619 .18541 1.875 0 Mabsadd L
.97619 .18541 0 -2.5 Mabsadd L
.97619 .18541 -1.875 0 Mabsadd L closepath F
0 1 0 r
.001 w
.02381 .55623 m
.34127 .49443 L
.65873 .43262 L
.97619 .37082 L
s
0 g
.0001 w
.02381 .55623 0 2.5 Mabsadd m
.02381 .55623 -0.56518 .7779 Mabsadd L
.02381 .55623 -2.37764 .77254 Mabsadd L
.02381 .55623 -0.91448 -0.29713 Mabsadd L
.02381 .55623 -1.46946 -2.02254 Mabsadd L
.02381 .55623 0 -0.96154 Mabsadd L
.02381 .55623 1.46946 -2.02254 Mabsadd L
.02381 .55623 .91448 -0.29713 Mabsadd L
.02381 .55623 2.37764 .77254 Mabsadd L
.02381 .55623 .56518 .7779 Mabsadd L
.02381 .55623 0 2.5 Mabsadd L closepath F
.34127 .49443 0 2.5 Mabsadd m
.34127 .49443 -0.56518 .7779 Mabsadd L
.34127 .49443 -2.37764 .77254 Mabsadd L
.34127 .49443 -0.91448 -0.29713 Mabsadd L
.34127 .49443 -1.46946 -2.02254 Mabsadd L
.34127 .49443 0 -0.96154 Mabsadd L
.34127 .49443 1.46946 -2.02254 Mabsadd L
.34127 .49443 .91448 -0.29713 Mabsadd L
.34127 .49443 2.37764 .77254 Mabsadd L
.34127 .49443 .56518 .7779 Mabsadd L
.34127 .49443 0 2.5 Mabsadd L closepath F
.65873 .43262 0 2.5 Mabsadd m
.65873 .43262 -0.56518 .7779 Mabsadd L
.65873 .43262 -2.37764 .77254 Mabsadd L
.65873 .43262 -0.91448 -0.29713 Mabsadd L
.65873 .43262 -1.46946 -2.02254 Mabsadd L
.65873 .43262 0 -0.96154 Mabsadd L
.65873 .43262 1.46946 -2.02254 Mabsadd L
.65873 .43262 .91448 -0.29713 Mabsadd L
.65873 .43262 2.37764 .77254 Mabsadd L
.65873 .43262 .56518 .7779 Mabsadd L
.65873 .43262 0 2.5 Mabsadd L closepath F
.97619 .37082 0 2.5 Mabsadd m
.97619 .37082 -0.56518 .7779 Mabsadd L
.97619 .37082 -2.37764 .77254 Mabsadd L
.97619 .37082 -0.91448 -0.29713 Mabsadd L
.97619 .37082 -1.46946 -2.02254 Mabsadd L
.97619 .37082 0 -0.96154 Mabsadd L
.97619 .37082 1.46946 -2.02254 Mabsadd L
.97619 .37082 .91448 -0.29713 Mabsadd L
.97619 .37082 2.37764 .77254 Mabsadd L
.97619 .37082 .56518 .7779 Mabsadd L
.97619 .37082 0 2.5 Mabsadd L closepath F
0 0 1 r
.001 w
.02381 .12361 m
.34127 .12361 L
.65873 .12361 L
.97619 .12361 L
s
0 g
.0001 w
.02381 .12361 -2.5 2.5 Mabsadd m
.02381 .12361 2.5 2.5 Mabsadd L
.02381 .12361 2.5 -2.5 Mabsadd L
.02381 .12361 -2.5 -2.5 Mabsadd L
.02381 .12361 -2.5 2.5 Mabsadd L closepath F
.34127 .12361 -2.5 2.5 Mabsadd m
.34127 .12361 2.5 2.5 Mabsadd L
.34127 .12361 2.5 -2.5 Mabsadd L
.34127 .12361 -2.5 -2.5 Mabsadd L
.34127 .12361 -2.5 2.5 Mabsadd L closepath F
.65873 .12361 -2.5 2.5 Mabsadd m
.65873 .12361 2.5 2.5 Mabsadd L
.65873 .12361 2.5 -2.5 Mabsadd L
.65873 .12361 -2.5 -2.5 Mabsadd L
.65873 .12361 -2.5 2.5 Mabsadd L closepath F
.97619 .12361 -2.5 2.5 Mabsadd m
.97619 .12361 2.5 2.5 Mabsadd L
.97619 .12361 2.5 -2.5 Mabsadd L
.97619 .12361 -2.5 -2.5 Mabsadd L
.97619 .12361 -2.5 2.5 Mabsadd L closepath F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{167, 103.125},
  ImageMargins->{{34, 0}, {0, 1}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002W0000Ia000`40O003h00OYgoo002WOol002=o
o`<000Aoo`8000=oo`<000eoo`@000eoo`@000=oo`8000=oo`<000eoo`<000eoo`<000Aoo`8000=o
o`<000moo`<000Yoo`0097oo00<007ooOol00goo0P001Woo00<007ooOol02goo00<007ooOol03Woo
00<007ooOol00goo0P001Woo00<007ooOol03Goo00<007ooOol03Goo00@007ooOomoo`8000Ioo`03
001oogoo00eoo`03001oogoo00Uoo`0097oo00<007ooOol02goo00<007ooOol037oo00<007ooOol0
3Woo00<007ooOol02Woo00<007ooOol03Goo00<007ooOol03Goo00<007ooOol02Goo00<007ooOol0
2goo10002Woo000TOol00`00Oomoo`08Ool3000@Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool3000>
Ool2000>Ool20009Ool3000>Ool00`00Ool0000;Ool002Aoo`03001oogoo00Qoo`03001oogoo00eo
o`04001oogoo000=Ool01000Oomoo`0027oo00<007ooOol047oo00<007ooOol03Goo00<007ooOol0
1Woo00<007ooOol03Woo00<007oo00002goo000SOol2000:Ool4000=Ool2000?Ool20009Ool4000<
Ool3000=Ool30009Ool4000>Ool2000;Ool00:Moo`00Ygoo002WOol0015oohh000Qoo`005Goo00D0
07ooOomoo`0000Aoo`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000=o
o`03001oogoo009oo`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Ao
o`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000=oo`03001oogoo009o
o`05001oogooOol00004Ool01@00Oomoogoo000017oo00D007ooOomoo`0000Aoo`05001oogooOol0
0004Ool00`00Oomoo`09Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00
Oomoo`2?Ool001Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool0
01Eoo`03001oogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001o
ogoo08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`00
5Goo00<007ooOol0Sgoo000>Ool9000XOol5000XOol5000WOol5000:Ool000moo`03001oogoo00D0
02Qoo`D002Qoo`D002Moo`D000Yoo`0047oo00<007oo00001000:00O1@00:00O1@009`0O1@002Woo
000AOol6000XOol5000XOol5000WOol5000:Ool000ioo`03001oogoo00H002Qoo`D002Qoo`D002Mo
o`D000Yoo`003goo0P0017oo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo
08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`23Ool00`00Oomoo`09Ool001Eoo`03001o
ogoo085oo`04O0000000000:Ool001Eoo`03001oogoo07aoo`El00@000Yoo`005Goo00<007ooOol0
Mgoo1G`01Woo0`002Woo000EOol00`00Oomoo`1bOol5O00<Ool00`00Oomoo`09Ool001Eoo`03001o
ogoo06eoo`El01eoo`005Goo00<007ooOol0J7oo1G`08Woo000EOol00`00Oomoo`1SOol5O00WOol0
00moo`<000=oo`03001oogoo05ioo`El02aoo`0047oo00<007ooOol00Woo00<007ooOol0EWoo00<0
07ooOol01G`0<Goo000>Ool40003Ool2001FOol300000g`0Oomoo`0dOol000ioo`03001oo`0000Ao
o`03001oogoo05=oo`9l00<003Moo`003Woo00<007oo000017oo00<007ooOol0DWoo00El07ooOomo
o`0003Qoo`003goo0P0017oo00<007ooOol0D7oo0W`0?Goo000EOol00`00Oomoo`1>Ool2O00oOol0
01Eoo`03001oogoo04aoo`9l045oo`005Goo00<007ooOol0Bgoo00=l07ooOol0@Goo000EOol00`00
Oomoo`19Ool2O014Ool001Eoo`03001oogoo04Moo`9l04Ioo`0057oo0`00Agoo00=l07ooOol0AWoo
000DOol30003O012Ool2O019Ool001Eoo`03001oogoo009oo`9l03ioo`9l04]oo`005Goo00<007oo
Ool017oo0g`0>Goo0W`0CGoo000EOol00`00Oomoo`07Ool3O00eOol00g`0Oomoo`1=Ool001Eoo`03
001oogoo00Yoo`9l035oo`9l051oo`003goo0P0017oo00<007ooOol037oo0g`0;7oo0W`0DWoo000>
Ool01000Oomoo`000goo00<007ooOol03goo0W`0:7oo0W`0AWoo0P0037oo000>Ool01000Oomoo`00
0goo0P004Woo0g`097oo00=l07ooOol0AWoo00<3h00000002goo000>Ool30004Ool00`00Oomoo`0D
Ool3O00OOol2O014Ool50n04000:Ool000ioo`03001oogoo00Aoo`03001oogoo01Moo`9l01]oo`9l
045oo`D3h0Ioo`03001oogoo00Yoo`003goo0`000goo00<007ooOol06Goo0g`05goo00=l07ooOol0
?7oo1@?P67oo000EOol00`00Oomoo`0LOol2O00COol2O00jOol50n0MOol001Eoo`03001oogoo01io
o`=l00ioo`9l03Moo`D3h29oo`005Goo00<007ooOol08Goo0g`02Goo0W`0=7oo1@?P9goo000EOol0
0`00Oomoo`0TOol2O006Ool00g`0Oomoo`0_Ool50n0/Ool001Eoo`03001oogoo02Ioo`=l0003001l
07`002Uoo`04001oogooOol50n0aOol001Eoo`03001oogoo02Qoo`<002Uoo`030n0000000083h3Io
o`005Goo00<007ooOol0:7oo0`0097oo1@?P00Aoo`00000003Moo`005Goo00<007ooOol0:Goo00<0
07ooOol07Woo1@?P1goo00<007ooOol0=Woo000EOol00`00Oomoo`15Ool50n15Ool001Eoo`03001o
ogoo041oo`D3h4Yoo`005Goo00<007ooOol0>goo1@?PCgoo000EOol00`00Oomoo`0fOol50n1DOol0
00moo`8000Aoo`03001oogoo035oo`D3h5Uoo`003Woo00@007ooOol000=oo`03001oogoo02Qoo`04
001oo`00Ool50n1NOol000ioo`04001oogoo0003Ool2000WOol20n04001SOol000moo`8000Aoo`03
001oogoo025oo`D3h0=oo`03001oogoo06=oo`003Woo00@007ooOol000=oo`03001oogoo01]oo`H3
h0Qoo`03001oogoo06=oo`003goo0P0017oo00<007ooOol05Woo1@?PM7oo000EOol00`00Oomoo`0@
Ool60n1iOol001Eoo`03001oogoo00]oo`D3h7moo`005Goo00<007ooOol01Goo1P?PQ7oo000COol3
0002Ool50n2:Ool001Aoo`800083h8moo`004goo1000T7oo000DOol2002AOol001Eoo`03001oogoo
08moo`005Goo00<007ooOol0Sgoo000EOol00`00Oomoo`2?Ool001Eoo`03001oogoo08moo`0027oo
0`0017oo0P0017oo00<007ooOol0Sgoo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo00<007oo
Ool0Sgoo0009Ool00`00Oomoo`02Ool01000Oomoo`000goo0P00T7oo0009Ool00`00Oomoo`02Ool0
1000Oomoo`00UGoo0009Ool00`00Oomoo`02Ool01000Oomoo`00UGoo0008Ool20005Ool2002FOol0
0:Moo`00\
\>"],
  ImageRangeCache->{{{0, 166}, {102.125, 0}} -> {0.523221, -1.23836, 
  0.0224112, 0.115118}}]
}, Open  ]],

Cell["\<\
The management of legends in conjunction with this Package is \
discussed in detail by Shaw and Tigg (1993).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions of Two or More Variables", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "The basic functions to know about are ",
  StyleBox["Plot3D",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["ContourPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["DensityPlot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and their ",
  StyleBox["List",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " analogues. The forms of the command are very similar to those for ",
  StyleBox["Plot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " etc. A few examples will suffice. Let's make a function to use in our \
examples."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(func[x_, \ y_]\  := \ Sin[x^2]\ Cos[y]; \n
    Plot3D[func[x, \ y], \ {x, \ \(-2\), \ 2}, \ {y, \ \(-2\), \ 2}, \ 
      PlotPoints\  -> \ 40]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.50253 .05935 -5.27396 -9 ]
[.50253 .05935 .72604 0 ]
[.68156 -0.01098 -4.96123 -9 ]
[.68156 -0.01098 1.03877 0 ]
[.70096 -0.00478 0 -6.26206 ]
[.70096 -0.00478 12 2.73794 ]
[.78313 .12104 0 -6.13858 ]
[.78313 .12104 12 2.86142 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9224 .33431 0 -5.93715 ]
[.9224 .33431 6 3.06285 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.01922 .29575 -24 -2.76821 ]
[.01922 .29575 0 6.23179 ]
[.00938 .35745 -6 -2.82049 ]
[.00938 .35745 0 6.17951 ]
[-0.00086 .42164 -18 -2.87511 ]
[-0.00086 .42164 0 6.12489 ]
[-0.01153 .48849 -6 -2.93222 ]
[-0.01153 .48849 0 6.06778 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(-2)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(-1)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(1)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(2)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.68874 0 m
.68263 .00239 L
s
[(-2)] .70096 -0.00478 -1 .39157 Mshowa
.7708 .12553 m
.76464 .12778 L
s
[(-1)] .78313 .12104 -1 .36413 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.9099 .3383 m
.90365 .34029 L
s
[(1)] .9224 .33431 -1 .31937 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(2)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70593 .0263 m
.70226 .02771 L
s
.72272 .05198 m
.71904 .05338 L
s
.73912 .07706 m
.73543 .07845 L
s
.75514 .10158 m
.75145 .10294 L
s
.78611 .14895 m
.78241 .15028 L
s
.80108 .17185 m
.79737 .17317 L
s
.81573 .19425 m
.81201 .19555 L
s
.83006 .21617 m
.82633 .21745 L
s
.8578 .2586 m
.85407 .25986 L
s
.87123 .27915 m
.8675 .28039 L
s
.88439 .29928 m
.88065 .3005 L
s
.89727 .31899 m
.89353 .3202 L
s
.92227 .35722 m
.91851 .3584 L
s
.93439 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .3951 L
s
.95792 .41176 m
.95416 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03147 .29104 m
.03759 .28868 L
s
[(-0.5)] .01922 .29575 1 -0.38484 Mshowa
.02167 .35286 m
.02782 .35057 L
s
[(0)] .00938 .35745 1 -0.37322 Mshowa
.01148 .41719 m
.01765 .41496 L
s
[(0.5)] -0.00086 .42164 1 -0.36109 Mshowa
.00086 .48417 m
.00706 .48201 L
s
[(1)] -0.01153 .48849 1 -0.3484 Mshowa
.125 Mabswid
.02954 .30321 m
.03322 .3018 L
s
.0276 .31548 m
.03128 .31408 L
s
.02564 .32784 m
.02932 .32645 L
s
.02366 .3403 m
.02735 .33892 L
s
.01967 .36552 m
.02336 .36415 L
s
.01765 .37828 m
.02134 .37692 L
s
.01561 .39115 m
.0193 .38979 L
s
.01355 .40411 m
.01725 .40277 L
s
.00939 .43037 m
.0131 .42904 L
s
.00729 .44365 m
.01099 .44233 L
s
.00516 .45705 m
.00887 .45574 L
s
.00302 .47056 m
.00674 .46925 L
s
.03338 .27896 m
.03706 .27754 L
s
.03528 .26698 m
.03895 .26555 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.765 .652 .737 r
.39309 .70904 .4009 .72445 .41497 .70546 .40708 .6935 Metetra
.8 .694 .748 r
.40708 .6935 .41497 .70546 .42905 .6842 .42111 .67618 Metetra
.831 .751 .775 r
.42111 .67618 .42905 .6842 .44306 .66347 .4351 .65923 Metetra
.854 .829 .822 r
.4351 .65923 .44306 .66347 .45695 .64527 .44901 .64421 Metetra
.845 .93 .884 r
.44901 .64421 .45695 .64527 .47074 .63073 .46285 .63202 Metetra
.71 .982 .898 r
.46285 .63202 .47074 .63073 .48446 .62024 .47665 .62295 Metetra
.367 .825 .734 r
.47665 .62295 .48446 .62024 .49818 .61358 .49045 .61686 Metetra
.49045 .61686 .49818 .61358 .51193 .61018 .50428 .61329 Metetra
.136 0 0 r
.50428 .61329 .51193 .61018 .52575 .60928 .51819 .61167 Metetra
.196 0 0 r
.51819 .61167 .52575 .60928 .53968 .6101 .53219 .61138 Metetra
.171 0 0 r
.53219 .61138 .53968 .6101 .55373 .61189 .5463 .61183 Metetra
.069 0 0 r
.5463 .61183 .55373 .61189 .5679 .61404 .56052 .61255 Metetra
.096 .641 .84 r
.56052 .61255 .5679 .61404 .58219 .61604 .57486 .61315 Metetra
.273 .727 .956 r
.57486 .61315 .58219 .61604 .5966 .61755 .5893 .61336 Metetra
.411 .761 .995 r
.5893 .61336 .5966 .61755 .61111 .61831 .60383 .61298 Metetra
.503 .765 .99 r
.60383 .61298 .61111 .61831 .62571 .61818 .61846 .61191 Metetra
.563 .759 .97 r
.61846 .61191 .62571 .61818 .64039 .61707 .63317 .61007 Metetra
.604 .752 .949 r
.63317 .61007 .64039 .61707 .65513 .61494 .64795 .60743 Metetra
.633 .749 .932 r
.64795 .60743 .65513 .61494 .66992 .61176 .66278 .60399 Metetra
.657 .751 .919 r
.66278 .60399 .66992 .61176 .68475 .60755 .67767 .59973 Metetra
.677 .757 .911 r
.67767 .59973 .68475 .60755 .69961 .60227 .69259 .59465 Metetra
.697 .769 .908 r
.69259 .59465 .69961 .60227 .71448 .59595 .70755 .58875 Metetra
.715 .787 .908 r
.70755 .58875 .71448 .59595 .72935 .58858 .72252 .58203 Metetra
.734 .81 .912 r
.72252 .58203 .72935 .58858 .74421 .5802 .73751 .57453 Metetra
.751 .841 .919 r
.73751 .57453 .74421 .5802 .75905 .57089 .7525 .5663 Metetra
.765 .878 .929 r
.7525 .5663 .75905 .57089 .77387 .5608 .7675 .55746 Metetra
.77 .919 .937 r
.7675 .55746 .77387 .5608 .78867 .55016 .7825 .54818 Metetra
.754 .958 .938 r
.7825 .54818 .78867 .55016 .80349 .53935 .79753 .53876 Metetra
.697 .975 .915 r
.79753 .53876 .80349 .53935 .81836 .52886 .81262 .52958 Metetra
.571 .94 .847 r
.81262 .52958 .81836 .52886 .83335 .51934 .82782 .52114 Metetra
.372 .827 .724 r
.82782 .52114 .83335 .51934 .84854 .51155 .8432 .51405 Metetra
.8432 .51405 .84854 .51155 .86405 .50634 .85884 .50895 Metetra
.065 0 0 r
.85884 .50895 .86405 .50634 .88 .5045 .87483 .50647 Metetra
.217 0 0 r
.87483 .50647 .88 .5045 .8965 .50666 .89128 .50709 Metetra
.296 0 0 r
.89128 .50709 .8965 .50666 .91366 .51305 .90825 .51099 Metetra
.265 0 0 r
.90825 .51099 .91366 .51305 .93149 .52335 .92575 .51789 Metetra
0 .387 .821 r
.92575 .51789 .93149 .52335 .94991 .5364 .94372 .52688 Metetra
.17 .451 .879 r
.94372 .52688 .94991 .5364 .96868 .5501 .96199 .53635 Metetra
.374 .494 .852 r
.96199 .53635 .96868 .5501 .98743 .56153 .98025 .54402 Metetra
.737 .638 .75 r
.38525 .69327 .39309 .70904 .40708 .6935 .39915 .68129 Metetra
.77 .685 .769 r
.39915 .68129 .40708 .6935 .42111 .67618 .41311 .66806 Metetra
.801 .751 .807 r
.41311 .66806 .42111 .67618 .4351 .65923 .42705 .65506 Metetra
.822 .844 .865 r
.42705 .65506 .4351 .65923 .44901 .64421 .44096 .64337 Metetra
.791 .949 .924 r
.44096 .64337 .44901 .64421 .46285 .63202 .45484 .63364 Metetra
.598 .951 .877 r
.45484 .63364 .46285 .63202 .47665 .62295 .4687 .62607 Metetra
.263 .752 .661 r
.4687 .62607 .47665 .62295 .49045 .61686 .48259 .62058 Metetra
.48259 .62058 .49045 .61686 .50428 .61329 .49651 .61684 Metetra
.098 0 0 r
.49651 .61684 .50428 .61329 .51819 .61167 .51051 .61446 Metetra
.12 0 0 r
.51051 .61446 .51819 .61167 .53219 .61138 .52459 .61299 Metetra
.065 0 0 r
.52459 .61299 .53219 .61138 .5463 .61183 .53877 .61204 Metetra
.06 .625 .731 r
.53877 .61204 .5463 .61183 .56052 .61255 .55305 .61127 Metetra
.225 .733 .891 r
.55305 .61127 .56052 .61255 .57486 .61315 .56743 .6104 Metetra
.377 .79 .98 r
.56743 .6104 .57486 .61315 .5893 .61336 .58191 .60925 Metetra
.484 .798 .999 r
.58191 .60925 .5893 .61336 .60383 .61298 .59648 .60768 Metetra
.551 .786 .986 r
.59648 .60768 .60383 .61298 .61846 .61191 .61114 .60562 Metetra
.592 .771 .965 r
.61114 .60562 .61846 .61191 .63317 .61007 .62588 .60302 Metetra
.62 .759 .945 r
.62588 .60302 .63317 .61007 .64795 .60743 .64069 .59986 Metetra
.641 .752 .929 r
.64069 .59986 .64795 .60743 .66278 .60399 .65557 .59613 Metetra
.657 .751 .919 r
.65557 .59613 .66278 .60399 .67767 .59973 .67051 .59183 Metetra
.671 .755 .914 r
.67051 .59183 .67767 .59973 .69259 .59465 .68551 .58695 Metetra
.685 .766 .913 r
.68551 .58695 .69259 .59465 .70755 .58875 .70055 .5815 Metetra
.698 .783 .917 r
.70055 .5815 .70755 .58875 .72252 .58203 .71563 .57547 Metetra
.711 .807 .924 r
.71563 .57547 .72252 .58203 .73751 .57453 .73074 .56888 Metetra
.723 .838 .936 r
.73074 .56888 .73751 .57453 .7525 .5663 .74588 .56178 Metetra
.73 .876 .949 r
.74588 .56178 .7525 .5663 .7675 .55746 .76105 .55425 Metetra
.725 .918 .959 r
.76105 .55425 .7675 .55746 .7825 .54818 .77626 .5464 Metetra
.693 .952 .954 r
.77626 .5464 .7825 .54818 .79753 .53876 .79151 .53844 Metetra
.613 .953 .911 r
.79151 .53844 .79753 .53876 .81262 .52958 .80683 .53064 Metetra
.467 .889 .81 r
.80683 .53064 .81262 .52958 .82782 .52114 .82224 .52335 Metetra
.276 .76 .664 r
.82224 .52335 .82782 .52114 .8432 .51405 .83781 .51698 Metetra
.83781 .51698 .8432 .51405 .85884 .50895 .85358 .512 Metetra
.054 0 0 r
.85358 .512 .85884 .50895 .87483 .50647 .86963 .50885 Metetra
.143 0 0 r
.86963 .50885 .87483 .50647 .89128 .50709 .88601 .50785 Metetra
.16 0 0 r
.88601 .50785 .89128 .50709 .90825 .51099 .90278 .50912 Metetra
0 .51 .777 r
.90278 .50912 .90825 .51099 .92575 .51789 .91995 .51247 Metetra
.158 .571 .932 r
.91995 .51247 .92575 .51789 .94372 .52688 .93747 .51725 Metetra
.353 .567 .919 r
.93747 .51725 .94372 .52688 .96199 .53635 .95521 .52232 Metetra
.477 .552 .854 r
.95521 .52232 .96199 .53635 .98025 .54402 .97299 .52611 Metetra
.701 .621 .765 r
.37738 .67722 .38525 .69327 .39915 .68129 .39118 .66888 Metetra
.728 .671 .795 r
.39118 .66888 .39915 .68129 .41311 .66806 .40503 .65984 Metetra
.754 .747 .846 r
.40503 .65984 .41311 .66806 .42705 .65506 .41891 .6509 Metetra
.765 .854 .917 r
.41891 .6509 .42705 .65506 .44096 .64337 .4328 .64267 Metetra
.694 .949 .956 r
.4328 .64267 .44096 .64337 .45484 .63364 .4467 .63549 Metetra
.449 .882 .826 r
.4467 .63549 .45484 .63364 .4687 .62607 .46063 .62949 Metetra
.46063 .62949 .4687 .62607 .48259 .62058 .4746 .62462 Metetra
.4746 .62462 .48259 .62058 .49651 .61684 .48862 .62071 Metetra
.049 0 0 r
.48862 .62071 .49651 .61684 .51051 .61446 .50271 .61753 Metetra
.029 0 0 r
.50271 .61753 .51051 .61446 .52459 .61299 .51688 .61485 Metetra
.51688 .61485 .52459 .61299 .53877 .61204 .53113 .61244 Metetra
.202 .731 .797 r
.53113 .61244 .53877 .61204 .55305 .61127 .54548 .61012 Metetra
.357 .815 .932 r
.54548 .61012 .55305 .61127 .56743 .6104 .55992 .60773 Metetra
.477 .841 .992 r
.55992 .60773 .56743 .6104 .58191 .60925 .57444 .60517 Metetra
.552 .828 .997 r
.57444 .60517 .58191 .60925 .59648 .60768 .58905 .60238 Metetra
.595 .804 .98 r
.58905 .60238 .59648 .60768 .61114 .60562 .60374 .5993 Metetra
.62 .781 .958 r
.60374 .5993 .61114 .60562 .62588 .60302 .61852 .59592 Metetra
.636 .765 .94 r
.61852 .59592 .62588 .60302 .64069 .59986 .63337 .59222 Metetra
.648 .754 .927 r
.63337 .59222 .64069 .59986 .65557 .59613 .64829 .5882 Metetra
.657 .751 .919 r
.64829 .5882 .65557 .59613 .67051 .59183 .66329 .58386 Metetra
.665 .753 .916 r
.66329 .58386 .67051 .59183 .68551 .58695 .67835 .57919 Metetra
.672 .762 .918 r
.67835 .57919 .68551 .58695 .70055 .5815 .69347 .57419 Metetra
.679 .778 .925 r
.69347 .57419 .70055 .5815 .71563 .57547 .70866 .56887 Metetra
.686 .801 .936 r
.70866 .56887 .71563 .57547 .73074 .56888 .7239 .56323 Metetra
.689 .832 .952 r
.7239 .56323 .73074 .56888 .74588 .56178 .73919 .5573 Metetra
.686 .87 .968 r
.73919 .5573 .74588 .56178 .76105 .55425 .75454 .55112 Metetra
.667 .909 .978 r
.75454 .55112 .76105 .55425 .77626 .5464 .76994 .54475 Metetra
.615 .933 .963 r
.76994 .54475 .77626 .5464 .79151 .53844 .78541 .53831 Metetra
.511 .912 .896 r
.78541 .53831 .79151 .53844 .80683 .53064 .80095 .53194 Metetra
.356 .825 .766 r
.80095 .53194 .80683 .53064 .82224 .52335 .81659 .52584 Metetra
.81659 .52584 .82224 .52335 .83781 .51698 .83235 .52024 Metetra
.83235 .52024 .83781 .51698 .85358 .512 .84826 .51539 Metetra
.031 0 0 r
.84826 .51539 .85358 .512 .86963 .50885 .86436 .51153 Metetra
.054 0 0 r
.86436 .51153 .86963 .50885 .88601 .50785 .88068 .50884 Metetra
.88068 .50884 .88601 .50785 .90278 .50912 .89726 .50739 Metetra
.193 .707 .888 r
.89726 .50739 .90278 .50912 .91995 .51247 .91409 .50706 Metetra
.401 .728 .99 r
.91409 .50706 .91995 .51247 .93747 .51725 .93115 .50749 Metetra
.513 .657 .925 r
.93115 .50749 .93747 .51725 .95521 .52232 .94838 .50806 Metetra
.566 .6 .846 r
.94838 .50806 .95521 .52232 .97299 .52611 .96566 .5079 Metetra
.653 .599 .781 r
.36949 .66099 .37738 .67722 .39118 .66888 .38315 .65633 Metetra
.666 .649 .823 r
.38315 .65633 .39118 .66888 .40503 .65984 .39688 .65154 Metetra
.677 .732 .891 r
.39688 .65154 .40503 .65984 .41891 .6509 .41067 .64675 Metetra
.659 .846 .972 r
.41067 .64675 .41891 .6509 .4328 .64267 .42453 .64205 Metetra
.527 .902 .957 r
.42453 .64205 .4328 .64267 .4467 .63549 .43845 .63749 Metetra
.274 .774 .744 r
.43845 .63749 .4467 .63549 .46063 .62949 .45243 .63311 Metetra
.45243 .63311 .46063 .62949 .4746 .62462 .46647 .62888 Metetra
.46647 .62888 .4746 .62462 .48862 .62071 .48059 .62479 Metetra
.48059 .62479 .48862 .62071 .50271 .61753 .49478 .62079 Metetra
.49478 .62079 .50271 .61753 .51688 .61485 .50905 .61686 Metetra
.199 .72 .698 r
.50905 .61686 .51688 .61485 .53113 .61244 .52339 .61295 Metetra
.349 .825 .854 r
.52339 .61295 .53113 .61244 .54548 .61012 .53781 .60903 Metetra
.48 .879 .957 r
.53781 .60903 .54548 .61012 .55992 .60773 .55231 .60509 Metetra
.566 .877 .993 r
.55231 .60509 .55992 .60773 .57444 .60517 .56688 .6011 Metetra
.612 .849 .989 r
.56688 .6011 .57444 .60517 .58905 .60238 .58154 .59705 Metetra
.635 .817 .97 r
.58154 .59705 .58905 .60238 .60374 .5993 .59627 .59294 Metetra
.646 .79 .951 r
.59627 .59294 .60374 .5993 .61852 .59592 .61108 .58876 Metetra
.652 .77 .935 r
.61108 .58876 .61852 .59592 .63337 .59222 .62597 .58451 Metetra
.655 .757 .925 r
.62597 .58451 .63337 .59222 .64829 .5882 .64094 .5802 Metetra
.657 .751 .919 r
.64094 .5802 .64829 .5882 .66329 .58386 .65599 .57581 Metetra
.658 .751 .919 r
.65599 .57581 .66329 .58386 .67835 .57919 .67112 .57135 Metetra
.659 .758 .923 r
.67112 .57135 .67835 .57919 .69347 .57419 .68632 .56682 Metetra
.659 .772 .933 r
.68632 .56682 .69347 .57419 .70866 .56887 .70161 .56222 Metetra
.656 .793 .947 r
.70161 .56222 .70866 .56887 .7239 .56323 .71697 .55755 Metetra
.65 .822 .966 r
.71697 .55755 .7239 .56323 .73919 .5573 .73242 .55281 Metetra
.633 .856 .984 r
.73242 .55281 .73919 .5573 .75454 .55112 .74794 .54801 Metetra
.595 .888 .991 r
.74794 .54801 .75454 .55112 .76994 .54475 .76354 .54316 Metetra
.52 .897 .962 r
.76354 .54316 .76994 .54475 .78541 .53831 .77923 .53828 Metetra
.397 .854 .869 r
.77923 .53828 .78541 .53831 .80095 .53194 .795 .53339 Metetra
.246 .753 .719 r
.795 .53339 .80095 .53194 .81659 .52584 .81086 .52851 Metetra
.81086 .52851 .81659 .52584 .83235 .52024 .82681 .5237 Metetra
.82681 .5237 .83235 .52024 .84826 .51539 .84286 .51898 Metetra
.84286 .51898 .84826 .51539 .86436 .51153 .85901 .5144 Metetra
.85901 .5144 .86436 .51153 .88068 .50884 .87528 .50998 Metetra
.204 .725 .714 r
.87528 .50998 .88068 .50884 .89726 .50739 .89166 .50573 Metetra
.444 .864 .947 r
.89166 .50573 .89726 .50739 .91409 .50706 .90816 .50163 Metetra
.601 .828 .986 r
.90816 .50163 .91409 .50706 .93115 .50749 .92477 .49763 Metetra
.638 .718 .905 r
.92477 .49763 .93115 .50749 .94838 .50806 .94149 .49363 Metetra
.64 .637 .832 r
.94149 .49363 .94838 .50806 .96566 .5079 .95829 .48948 Metetra
.59 .568 .797 r
.36156 .64467 .36949 .66099 .38315 .65633 .37507 .64369 Metetra
.573 .609 .851 r
.37507 .64369 .38315 .65633 .39688 .65154 .38866 .64316 Metetra
.544 .689 .934 r
.38866 .64316 .39688 .65154 .41067 .64675 .40235 .64255 Metetra
.462 .785 .999 r
.40235 .64255 .41067 .64675 .42453 .64205 .41615 .64143 Metetra
.275 .772 .886 r
.41615 .64143 .42453 .64205 .43845 .63749 .43006 .63954 Metetra
.43006 .63954 .43845 .63749 .45243 .63311 .44409 .6368 Metetra
.44409 .6368 .45243 .63311 .46647 .62888 .45822 .63324 Metetra
.45822 .63324 .46647 .62888 .48059 .62479 .47243 .62896 Metetra
.47243 .62896 .48059 .62479 .49478 .62079 .48673 .62413 Metetra
.48673 .62413 .49478 .62079 .50905 .61686 .50109 .61892 Metetra
.345 .818 .765 r
.50109 .61892 .50905 .61686 .52339 .61295 .51553 .61348 Metetra
.486 .9 .895 r
.51553 .61348 .52339 .61295 .53781 .60903 .53003 .60795 Metetra
.587 .922 .966 r
.53003 .60795 .53781 .60903 .55231 .60509 .5446 .60242 Metetra
.641 .9 .984 r
.5446 .60242 .55231 .60509 .56688 .6011 .55923 .59698 Metetra
.663 .863 .975 r
.55923 .59698 .56688 .6011 .58154 .59705 .57394 .59166 Metetra
.67 .827 .959 r
.57394 .59166 .58154 .59705 .59627 .59294 .58871 .58651 Metetra
.67 .796 .942 r
.58871 .58651 .59627 .59294 .61108 .58876 .60357 .58153 Metetra
.667 .774 .93 r
.60357 .58153 .61108 .58876 .62597 .58451 .6185 .57673 Metetra
.662 .759 .922 r
.6185 .57673 .62597 .58451 .64094 .5802 .63352 .57211 Metetra
.657 .751 .919 r
.63352 .57211 .64094 .5802 .65599 .57581 .64862 .56768 Metetra
.651 .749 .921 r
.64862 .56768 .65599 .57581 .67112 .57135 .66381 .56343 Metetra
.645 .753 .928 r
.66381 .56343 .67112 .57135 .68632 .56682 .6791 .55937 Metetra
.636 .764 .94 r
.6791 .55937 .68632 .56682 .70161 .56222 .69449 .5555 Metetra
.624 .782 .957 r
.69449 .5555 .70161 .56222 .71697 .55755 .70997 .5518 Metetra
.604 .806 .978 r
.70997 .5518 .71697 .55755 .73242 .55281 .72556 .54827 Metetra
.57 .834 .995 r
.72556 .54827 .73242 .55281 .74794 .54801 .74126 .54487 Metetra
.51 .854 .994 r
.74126 .54487 .74794 .54801 .76354 .54316 .75705 .54156 Metetra
.412 .844 .948 r
.75705 .54156 .76354 .54316 .77923 .53828 .77295 .53826 Metetra
.277 .781 .833 r
.77295 .53826 .77923 .53828 .795 .53339 .78895 .53487 Metetra
.78895 .53487 .795 .53339 .81086 .52851 .80503 .53126 Metetra
.80503 .53126 .81086 .52851 .82681 .5237 .82117 .52725 Metetra
.82117 .52725 .82681 .5237 .84286 .51898 .83736 .52266 Metetra
.83736 .52266 .84286 .51898 .85901 .5144 .85358 .51734 Metetra
.85358 .51734 .85901 .5144 .87528 .50998 .86979 .51115 Metetra
.4 .849 .771 r
.86979 .51115 .87528 .50998 .89166 .50573 .88598 .50405 Metetra
.645 .952 .945 r
.88598 .50405 .89166 .50573 .90816 .50163 .90215 .49614 Metetra
.74 .875 .943 r
.90215 .49614 .90816 .50163 .92477 .49763 .91833 .48767 Metetra
.731 .756 .872 r
.91833 .48767 .92477 .49763 .94149 .49363 .93454 .47908 Metetra
.7 .666 .813 r
.93454 .47908 .94149 .49363 .95829 .48948 .95088 .47096 Metetra
.504 .524 .81 r
.3536 .62837 .36156 .64467 .37507 .64369 .36694 .631 Metetra
.426 .539 .868 r
.36694 .631 .37507 .64369 .38866 .64316 .38036 .63472 Metetra
.313 .582 .938 r
.38036 .63472 .38866 .64316 .40235 .64255 .39392 .63828 Metetra
.14 .611 .923 r
.39392 .63828 .40235 .64255 .41615 .64143 .40765 .64074 Metetra
0 .558 .729 r
.40765 .64074 .41615 .64143 .43006 .63954 .42155 .64154 Metetra
.073 0 0 r
.42155 .64154 .43006 .63954 .44409 .6368 .43562 .64046 Metetra
.04 0 0 r
.43562 .64046 .44409 .6368 .45822 .63324 .44982 .63756 Metetra
.44982 .63756 .45822 .63324 .47243 .62896 .46414 .6331 Metetra
.46414 .6331 .47243 .62896 .48673 .62413 .47854 .62742 Metetra
.328 .795 .687 r
.47854 .62742 .48673 .62413 .50109 .61892 .49302 .62092 Metetra
.484 .898 .821 r
.49302 .62092 .50109 .61892 .51553 .61348 .50756 .61395 Metetra
.604 .948 .917 r
.50756 .61395 .51553 .61348 .53003 .60795 .52215 .60679 Metetra
.673 .945 .961 r
.52215 .60679 .53003 .60795 .5446 .60242 .53679 .59968 Metetra
.701 .911 .968 r
.53679 .59968 .5446 .60242 .55923 .59698 .55149 .59278 Metetra
.706 .87 .959 r
.55149 .59278 .55923 .59698 .57394 .59166 .56625 .58619 Metetra
.701 .832 .945 r
.56625 .58619 .57394 .59166 .58871 .58651 .58107 .57999 Metetra
.691 .801 .933 r
.58107 .57999 .58871 .58651 .60357 .58153 .59597 .57421 Metetra
.68 .778 .924 r
.59597 .57421 .60357 .58153 .6185 .57673 .61095 .56886 Metetra
.669 .761 .919 r
.61095 .56886 .6185 .57673 .63352 .57211 .62602 .56394 Metetra
.657 .751 .919 r
.62602 .56394 .63352 .57211 .64862 .56768 .64118 .55946 Metetra
.644 .746 .923 r
.64118 .55946 .64862 .56768 .66381 .56343 .65644 .55543 Metetra
.63 .748 .933 r
.65644 .55543 .66381 .56343 .6791 .55937 .6718 .55184 Metetra
.612 .755 .947 r
.6718 .55184 .6791 .55937 .69449 .5555 .68729 .54869 Metetra
.588 .768 .965 r
.68729 .54869 .69449 .5555 .70997 .5518 .70289 .54596 Metetra
.553 .786 .986 r
.70289 .54596 .70997 .5518 .72556 .54827 .71862 .54364 Metetra
.499 .803 .999 r
.71862 .54364 .72556 .54827 .74126 .54487 .73448 .54164 Metetra
.415 .806 .987 r
.73448 .54164 .74126 .54487 .75705 .54156 .75047 .53988 Metetra
.296 .775 .921 r
.75047 .53988 .75705 .54156 .77295 .53826 .76658 .53818 Metetra
.16 .701 .792 r
.76658 .53818 .77295 .53826 .78895 .53487 .78279 .5363 Metetra
.78279 .5363 .78895 .53487 .80503 .53126 .79909 .53395 Metetra
.003 0 0 r
.79909 .53395 .80503 .53126 .82117 .52725 .81543 .53075 Metetra
.81543 .53075 .82117 .52725 .83736 .52266 .83176 .52631 Metetra
.83176 .52631 .83736 .52266 .85358 .51734 .84804 .52023 Metetra
.84804 .52023 .85358 .51734 .86979 .51115 .8642 .51226 Metetra
.573 .936 .803 r
.8642 .51226 .86979 .51115 .88598 .50405 .88021 .50229 Metetra
.78 .982 .906 r
.88021 .50229 .88598 .50405 .90215 .49614 .89607 .49055 Metetra
.828 .889 .886 r
.89607 .49055 .90215 .49614 .91833 .48767 .91182 .47762 Metetra
.798 .777 .834 r
.91182 .47762 .91833 .48767 .93454 .47908 .92756 .46448 Metetra
.749 .688 .793 r
.92756 .46448 .93454 .47908 .95088 .47096 .94344 .45243 Metetra
.384 .459 .815 r
.34559 .61215 .3536 .62837 .36694 .631 .35875 .61834 Metetra
.202 .415 .846 r
.35875 .61834 .36694 .631 .38036 .63472 .37199 .62622 Metetra
0 .373 .834 r
.37199 .62622 .38036 .63472 .39392 .63828 .3854 .63389 Metetra
0 .332 .705 r
.3854 .63389 .39392 .63828 .40765 .64074 .39904 .63991 Metetra
.282 0 0 r
.39904 .63991 .40765 .64074 .42155 .64154 .41292 .64339 Metetra
.214 0 0 r
.41292 .64339 .42155 .64154 .43562 .64046 .42702 .64396 Metetra
.085 0 0 r
.42702 .64396 .43562 .64046 .44982 .63756 .44129 .64172 Metetra
.44129 .64172 .44982 .63756 .46414 .6331 .45571 .63707 Metetra
.45571 .63707 .46414 .6331 .47854 .62742 .47023 .63056 Metetra
.456 .876 .754 r
.47023 .63056 .47854 .62742 .49302 .62092 .48482 .62277 Metetra
.605 .954 .86 r
.48482 .62277 .49302 .62092 .50756 .61395 .49946 .61426 Metetra
.697 .973 .922 r
.49946 .61426 .50756 .61395 .52215 .60679 .51415 .60548 Metetra
.738 .952 .946 r
.51415 .60548 .52215 .60679 .53679 .59968 .52888 .5968 Metetra
.748 .914 .948 r
.52888 .5968 .53679 .59968 .55149 .59278 .54365 .58845 Metetra
.741 .873 .941 r
.54365 .58845 .55149 .59278 .56625 .58619 .55847 .58061 Metetra
.727 .836 .932 r
.55847 .58061 .56625 .58619 .58107 .57999 .57335 .57338 Metetra
.71 .805 .924 r
.57335 .57338 .58107 .57999 .59597 .57421 .5883 .5668 Metetra
.693 .781 .918 r
.5883 .5668 .59597 .57421 .61095 .56886 .60333 .5609 Metetra
.675 .763 .917 r
.60333 .5609 .61095 .56886 .62602 .56394 .61844 .55569 Metetra
.657 .751 .919 r
.61844 .55569 .62602 .56394 .64118 .55946 .63366 .55117 Metetra
.637 .744 .926 r
.63366 .55117 .64118 .55946 .65644 .55543 .64898 .54734 Metetra
.614 .742 .937 r
.64898 .54734 .65644 .55543 .6718 .55184 .66443 .54421 Metetra
.586 .745 .952 r
.66443 .54421 .6718 .55184 .68729 .54869 .68 .54177 Metetra
.549 .752 .971 r
.68 .54177 .68729 .54869 .70289 .54596 .69572 .54001 Metetra
.496 .76 .989 r
.69572 .54001 .70289 .54596 .71862 .54364 .71158 .53887 Metetra
.42 .763 .996 r
.71158 .53887 .71862 .54364 .73448 .54164 .7276 .53827 Metetra
.313 .747 .969 r
.7276 .53827 .73448 .54164 .75047 .53988 .74378 .53804 Metetra
.18 .697 .886 r
.74378 .53804 .75047 .53988 .76658 .53818 .76009 .53792 Metetra
.053 .619 .75 r
.76009 .53792 .76658 .53818 .78279 .5363 .77652 .53756 Metetra
.029 0 0 r
.77652 .53756 .78279 .5363 .79909 .53395 .79303 .53647 Metetra
.043 0 0 r
.79303 .53647 .79909 .53395 .81543 .53075 .80956 .53408 Metetra
.80956 .53408 .81543 .53075 .83176 .52631 .82604 .52977 Metetra
.82604 .52977 .83176 .52631 .84804 .52023 .84238 .52295 Metetra
.429 .843 .671 r
.84238 .52295 .84804 .52023 .8642 .51226 .8585 .51319 Metetra
.709 .983 .81 r
.8585 .51319 .8642 .51226 .88021 .50229 .87435 .50037 Metetra
.863 .978 .855 r
.87435 .50037 .88021 .50229 .89607 .49055 .88991 .48482 Metetra
.882 .886 .83 r
.88991 .48482 .89607 .49055 .91182 .47762 .90525 .46749 Metetra
.845 .788 .796 r
.90525 .46749 .91182 .47762 .92756 .46448 .92053 .44988 Metetra
.789 .706 .772 r
.92053 .44988 .92756 .46448 .94344 .45243 .93599 .43399 Metetra
.219 .365 .798 r
.33753 .59613 .34559 .61215 .35875 .61834 .3505 .60575 Metetra
0 .224 .746 r
.3505 .60575 .35875 .61834 .37199 .62622 .36354 .61766 Metetra
0 .1 .604 r
.36354 .61766 .37199 .62622 .3854 .63389 .37679 .62936 Metetra
.509 0 0 r
.37679 .62936 .3854 .63389 .39904 .63991 .39033 .63887 Metetra
.469 0 0 r
.39033 .63887 .39904 .63991 .41292 .64339 .40417 .64497 Metetra
.327 0 0 r
.40417 .64497 .41292 .64339 .42702 .64396 .41828 .64718 Metetra
.119 0 0 r
.41828 .64718 .42702 .64396 .44129 .64172 .43263 .64559 Metetra
.43263 .64559 .44129 .64172 .45571 .63707 .44715 .64076 Metetra
.376 .822 .695 r
.44715 .64076 .45571 .63707 .47023 .63056 .46178 .63341 Metetra
.575 .938 .809 r
.46178 .63341 .47023 .63056 .48482 .62277 .4765 .62435 Metetra
.702 .983 .88 r
.4765 .62435 .48482 .62277 .49946 .61426 .49126 .61433 Metetra
.765 .978 .915 r
.49126 .61433 .49946 .61426 .51415 .60548 .50605 .60396 Metetra
.786 .948 .927 r
.50605 .60396 .51415 .60548 .52888 .5968 .52087 .59373 Metetra
.784 .91 .927 r
.52087 .59373 .52888 .5968 .54365 .58845 .53572 .58396 Metetra
.769 .871 .922 r
.53572 .58396 .54365 .58845 .55847 .58061 .55061 .57489 Metetra
.749 .836 .918 r
.55061 .57489 .55847 .58061 .57335 .57338 .56555 .56665 Metetra
.727 .807 .914 r
.56555 .56665 .57335 .57338 .5883 .5668 .58055 .55929 Metetra
.705 .783 .912 r
.58055 .55929 .5883 .5668 .60333 .5609 .59563 .55286 Metetra
.681 .764 .914 r
.59563 .55286 .60333 .5609 .61844 .55569 .61079 .54735 Metetra
.657 .751 .919 r
.61079 .54735 .61844 .55569 .63366 .55117 .62607 .54279 Metetra
.63 .741 .928 r
.62607 .54279 .63366 .55117 .64898 .54734 .64145 .53916 Metetra
.599 .736 .941 r
.64145 .53916 .64898 .54734 .66443 .54421 .65697 .53648 Metetra
.56 .734 .957 r
.65697 .53648 .66443 .54421 .68 .54177 .67264 .53473 Metetra
.508 .733 .975 r
.67264 .53473 .68 .54177 .69572 .54001 .68846 .53391 Metetra
.437 .729 .988 r
.68846 .53391 .69572 .54001 .71158 .53887 .70446 .53394 Metetra
.338 .715 .984 r
.70446 .53394 .71158 .53887 .7276 .53827 .72063 .5347 Metetra
.211 .679 .941 r
.72063 .5347 .7276 .53827 .74378 .53804 .73698 .53597 Metetra
.072 .616 .845 r
.73698 .53597 .74378 .53804 .76009 .53792 .75349 .53741 Metetra
0 .542 .713 r
.75349 .53741 .76009 .53792 .77652 .53756 .77012 .53853 Metetra
.095 0 0 r
.77012 .53853 .77652 .53756 .79303 .53647 .78684 .53868 Metetra
.072 0 0 r
.78684 .53868 .79303 .53647 .80956 .53408 .80356 .5371 Metetra
.80356 .5371 .80956 .53408 .82604 .52977 .82018 .53292 Metetra
.82018 .53292 .82604 .52977 .84238 .52295 .83659 .52536 Metetra
.554 .912 .72 r
.83659 .52536 .84238 .52295 .8585 .51319 .85267 .51384 Metetra
.806 .999 .801 r
.85267 .51384 .8585 .51319 .87435 .50037 .86837 .49819 Metetra
.911 .96 .806 r
.86837 .49819 .87435 .50037 .88991 .48482 .88366 .47892 Metetra
.916 .875 .782 r
.88366 .47892 .88991 .48482 .90525 .46749 .89862 .45729 Metetra
.878 .792 .761 r
.89862 .45729 .90525 .46749 .92053 .44988 .91348 .43534 Metetra
.821 .719 .752 r
.91348 .43534 .92053 .44988 .93599 .43399 .92854 .41573 Metetra
.008 .234 .742 r
.32942 .58038 .33753 .59613 .3505 .60575 .34218 .59327 Metetra
0 0 .56 r
.34218 .59327 .3505 .60575 .36354 .61766 .35501 .60907 Metetra
.629 .129 0 r
.35501 .60907 .36354 .61766 .37679 .62936 .36809 .62464 Metetra
.671 .126 0 r
.36809 .62464 .37679 .62936 .39033 .63887 .3815 .63753 Metetra
.593 .022 0 r
.3815 .63753 .39033 .63887 .40417 .64497 .39529 .64619 Metetra
.416 0 0 r
.39529 .64619 .40417 .64497 .41828 .64718 .40942 .64998 Metetra
.142 0 0 r
.40942 .64998 .41828 .64718 .43263 .64559 .42384 .64904 Metetra
.42384 .64904 .43263 .64559 .44715 .64076 .43846 .64402 Metetra
.482 .89 .765 r
.43846 .64402 .44715 .64076 .46178 .63341 .45321 .63586 Metetra
.674 .977 .848 r
.45321 .63586 .46178 .63341 .4765 .62435 .46805 .62556 Metetra
.773 .991 .886 r
.46805 .62556 .4765 .62435 .49126 .61433 .48293 .61406 Metetra
.813 .971 .901 r
.48293 .61406 .49126 .61433 .50605 .60396 .49784 .60215 Metetra
.821 .938 .905 r
.49784 .60215 .50605 .60396 .52087 .59373 .51275 .59041 Metetra
.81 .901 .905 r
.51275 .59041 .52087 .59373 .53572 .58396 .52769 .57927 Metetra
.791 .866 .904 r
.52769 .57927 .53572 .58396 .55061 .57489 .54265 .56901 Metetra
.768 .835 .904 r
.54265 .56901 .55061 .57489 .56555 .56665 .55765 .55978 Metetra
.742 .808 .904 r
.55765 .55978 .56555 .56665 .58055 .55929 .57271 .55167 Metetra
.715 .784 .907 r
.57271 .55167 .58055 .55929 .59563 .55286 .58785 .54472 Metetra
.687 .766 .911 r
.58785 .54472 .59563 .55286 .61079 .54735 .60307 .53893 Metetra
.657 .751 .919 r
.60307 .53893 .61079 .54735 .62607 .54279 .61839 .53432 Metetra
.623 .739 .93 r
.61839 .53432 .62607 .54279 .64145 .53916 .63384 .53088 Metetra
.583 .73 .944 r
.63384 .53088 .64145 .53916 .65697 .53648 .64944 .52863 Metetra
.532 .721 .96 r
.64944 .52863 .65697 .53648 .67264 .53473 .66519 .52755 Metetra
.466 .711 .975 r
.66519 .52755 .67264 .53473 .68846 .53391 .68111 .52763 Metetra
.376 .694 .982 r
.68111 .52763 .68846 .53391 .70446 .53394 .69723 .52879 Metetra
.257 .662 .965 r
.69723 .52879 .70446 .53394 .72063 .5347 .71355 .53086 Metetra
.115 .608 .908 r
.71355 .53086 .72063 .5347 .73698 .53597 .73006 .53359 Metetra
0 .537 .804 r
.73006 .53359 .73698 .53597 .75349 .53741 .74676 .53654 Metetra
.12 0 0 r
.74676 .53654 .75349 .53741 .77012 .53853 .76359 .53911 Metetra
.147 0 0 r
.76359 .53911 .77012 .53853 .78684 .53868 .78051 .54048 Metetra
.09 0 0 r
.78051 .54048 .78684 .53868 .80356 .5371 .79741 .53967 Metetra
.79741 .53967 .80356 .5371 .82018 .53292 .81417 .53563 Metetra
.346 .798 .662 r
.81417 .53563 .82018 .53292 .83659 .52536 .83065 .52734 Metetra
.663 .962 .758 r
.83065 .52734 .83659 .52536 .85267 .51384 .84671 .51409 Metetra
.87 .995 .785 r
.84671 .51409 .85267 .51384 .86837 .49819 .86227 .4957 Metetra
.94 .938 .765 r
.86227 .4957 .86837 .49819 .88366 .47892 .87731 .47281 Metetra
.937 .863 .742 r
.87731 .47281 .88366 .47892 .89862 .45729 .89193 .44702 Metetra
.902 .793 .731 r
.89193 .44702 .89862 .45729 .91348 .43534 .90639 .4209 Metetra
.848 .73 .733 r
.90639 .4209 .91348 .43534 .92854 .41573 .9211 .39774 Metetra
0 .075 .637 r
.32124 .56498 .32942 .58038 .34218 .59327 .3338 .58097 Metetra
.628 .192 0 r
.3338 .58097 .34218 .59327 .35501 .60907 .34641 .60045 Metetra
.763 .279 0 r
.34641 .60045 .35501 .60907 .36809 .62464 .35929 .61971 Metetra
.762 .249 0 r
.35929 .61971 .36809 .62464 .3815 .63753 .37258 .63583 Metetra
.675 .129 0 r
.37258 .63583 .3815 .63753 .39529 .64619 .3863 .64694 Metetra
.485 0 0 r
.3863 .64694 .39529 .64619 .40942 .64998 .40044 .65226 Metetra
.155 0 0 r
.40044 .65226 .40942 .64998 .42384 .64904 .41492 .65194 Metetra
.26 .757 .694 r
.41492 .65194 .42384 .64904 .43846 .64402 .42964 .64674 Metetra
.581 .942 .825 r
.42964 .64674 .43846 .64402 .45321 .63586 .44452 .63778 Metetra
.75 .993 .87 r
.44452 .63778 .45321 .63586 .46805 .62556 .45948 .62629 Metetra
.822 .984 .881 r
.45948 .62629 .46805 .62556 .48293 .61406 .47449 .61337 Metetra
.846 .956 .883 r
.47449 .61337 .48293 .61406 .49784 .60215 .48951 .59997 Metetra
.844 .924 .884 r
.48951 .59997 .49784 .60215 .51275 .59041 .50454 .58678 Metetra
.83 .891 .885 r
.50454 .58678 .51275 .59041 .52769 .57927 .51956 .57433 Metetra
.808 .86 .887 r
.51956 .57433 .52769 .57927 .54265 .56901 .5346 .56292 Metetra
.782 .832 .891 r
.5346 .56292 .54265 .56901 .55765 .55978 .54968 .55276 Metetra
.754 .807 .895 r
.54968 .55276 .55765 .55978 .57271 .55167 .5648 .54393 Metetra
.724 .786 .901 r
.5648 .54393 .57271 .55167 .58785 .54472 .57998 .53648 Metetra
.692 .767 .909 r
.57998 .53648 .58785 .54472 .60307 .53893 .59526 .53043 Metetra
.657 .751 .919 r
.59526 .53043 .60307 .53893 .61839 .53432 .61064 .52577 Metetra
.616 .736 .932 r
.61064 .52577 .61839 .53432 .63384 .53088 .62616 .52251 Metetra
.567 .723 .947 r
.62616 .52251 .63384 .53088 .64944 .52863 .64182 .52066 Metetra
.505 .708 .962 r
.64182 .52066 .64944 .52863 .66519 .52755 .65765 .52022 Metetra
.424 .688 .973 r
.65765 .52022 .66519 .52755 .68111 .52763 .67367 .52115 Metetra
.316 .657 .972 r
.67367 .52115 .68111 .52763 .69723 .52879 .68991 .52338 Metetra
.18 .608 .942 r
.68991 .52338 .69723 .52879 .71355 .53086 .70636 .52671 Metetra
.03 .539 .872 r
.70636 .52671 .71355 .53086 .73006 .53359 .72303 .53083 Metetra
0 .465 .768 r
.72303 .53083 .73006 .53359 .74676 .53654 .7399 .53524 Metetra
.182 0 0 r
.7399 .53524 .74676 .53654 .76359 .53911 .75692 .53919 Metetra
.187 0 0 r
.75692 .53919 .76359 .53911 .78051 .54048 .77403 .54172 Metetra
.097 0 0 r
.77403 .54172 .78051 .54048 .79741 .53967 .7911 .54167 Metetra
.7911 .54167 .79741 .53967 .81417 .53563 .808 .53774 Metetra
.442 .864 .732 r
.808 .53774 .81417 .53563 .83065 .52734 .82455 .52875 Metetra
.753 .99 .784 r
.82455 .52875 .83065 .52734 .84671 .51409 .84061 .51384 Metetra
.912 .98 .766 r
.84061 .51384 .84671 .51409 .86227 .4957 .85605 .49281 Metetra
.956 .917 .731 r
.85605 .49281 .86227 .4957 .87731 .47281 .87086 .46645 Metetra
.95 .851 .709 r
.87086 .46645 .87731 .47281 .89193 .44702 .88518 .4367 Metetra
.919 .793 .705 r
.88518 .4367 .89193 .44702 .90639 .4209 .89929 .40662 Metetra
.869 .738 .716 r
.89929 .40662 .90639 .4209 .9211 .39774 .91367 .38011 Metetra
0 0 .49 r
.31298 .55002 .32124 .56498 .3338 .58097 .32533 .56888 Metetra
.764 .324 0 r
.32533 .56888 .3338 .58097 .34641 .60045 .33771 .5918 Metetra
.833 .372 0 r
.33771 .5918 .34641 .60045 .35929 .61971 .3504 .61453 Metetra
.814 .329 0 r
.3504 .61453 .35929 .61971 .37258 .63583 .36355 .63369 Metetra
.73 .205 0 r
.36355 .63369 .37258 .63583 .3863 .64694 .3772 .64712 Metetra
.539 0 0 r
.3772 .64712 .3863 .64694 .40044 .65226 .39134 .65389 Metetra
.158 0 0 r
.39134 .65389 .40044 .65226 .41492 .65194 .40587 .65414 Metetra
.339 .816 .773 r
.40587 .65414 .41492 .65194 .42964 .64674 .42069 .64877 Metetra
.666 .974 .87 r
.42069 .64877 .42964 .64674 .44452 .63778 .4357 .63907 Metetra
.804 .991 .879 r
.4357 .63907 .44452 .63778 .45948 .62629 .4508 .62644 Metetra
.854 .968 .871 r
.4508 .62644 .45948 .62629 .47449 .61337 .46594 .61217 Metetra
.866 .938 .865 r
.46594 .61217 .47449 .61337 .48951 .59997 .48108 .59735 Metetra
.86 .908 .864 r
.48108 .59735 .48951 .59997 .50454 .58678 .49621 .5828 Metetra
.844 .879 .867 r
.49621 .5828 .50454 .58678 .51956 .57433 .51134 .5691 Metetra
.821 .853 .872 r
.51134 .5691 .51956 .57433 .5346 .56292 .52646 .55662 Metetra
.794 .829 .878 r
.52646 .55662 .5346 .56292 .54968 .55276 .54161 .54557 Metetra
.765 .806 .886 r
.54161 .54557 .54968 .55276 .5648 .54393 .55679 .53606 Metetra
.732 .786 .896 r
.55679 .53606 .5648 .54393 .57998 .53648 .57204 .52814 Metetra
.697 .768 .906 r
.57204 .52814 .57998 .53648 .59526 .53043 .58737 .52183 Metetra
.657 .751 .919 r
.58737 .52183 .59526 .53043 .61064 .52577 .60281 .51712 Metetra
.61 .734 .933 r
.60281 .51712 .61064 .52577 .62616 .52251 .61839 .51403 Metetra
.552 .716 .949 r
.61839 .51403 .62616 .52251 .64182 .52066 .63412 .51256 Metetra
.479 .694 .963 r
.63412 .51256 .64182 .52066 .65765 .52022 .65002 .51271 Metetra
.383 .664 .969 r
.65002 .51271 .65765 .52022 .67367 .52115 .66614 .51444 Metetra
.259 .619 .958 r
.66614 .51444 .67367 .52115 .68991 .52338 .68248 .51767 Metetra
.111 .555 .916 r
.68248 .51767 .68991 .52338 .70636 .52671 .69905 .52219 Metetra
0 .476 .837 r
.69905 .52219 .70636 .52671 .72303 .53083 .71587 .52762 Metetra
0 .404 .737 r
.71587 .52762 .72303 .53083 .7399 .53524 .7329 .53339 Metetra
.229 0 0 r
.7329 .53339 .7399 .53524 .75692 .53919 .7501 .53865 Metetra
.214 0 0 r
.7501 .53865 .75692 .53919 .77403 .54172 .76738 .54229 Metetra
.093 0 0 r
.76738 .54229 .77403 .54172 .7911 .54167 .78463 .54295 Metetra
.169 .702 .707 r
.78463 .54295 .7911 .54167 .808 .53774 .80166 .53914 Metetra
.537 .92 .798 r
.80166 .53914 .808 .53774 .82455 .52875 .81829 .52947 Metetra
.82 1 .8 r
.81829 .52947 .82455 .52875 .84061 .51384 .83434 .51297 Metetra
.938 .961 .748 r
.83434 .51297 .84061 .51384 .85605 .49281 .84969 .48944 Metetra
.966 .896 .704 r
.84969 .48944 .85605 .49281 .87086 .46645 .86431 .45981 Metetra
.958 .839 .683 r
.86431 .45981 .87086 .46645 .88518 .4367 .87836 .42633 Metetra
.932 .791 .682 r
.87836 .42633 .88518 .4367 .89929 .40662 .89216 .39255 Metetra
.887 .745 .7 r
.89216 .39255 .89929 .40662 .91367 .38011 .90626 .36291 Metetra
.63 .228 0 r
.30465 .53555 .31298 .55002 .32533 .56888 .31678 .55705 Metetra
.839 .408 0 r
.31678 .55705 .32533 .56888 .33771 .5918 .32894 .58313 Metetra
.871 .431 .053 r
.32894 .58313 .33771 .5918 .3504 .61453 .34142 .60906 Metetra
.846 .383 .06 r
.34142 .60906 .3504 .61453 .36355 .63369 .35442 .63105 Metetra
.768 .262 0 r
.35442 .63105 .36355 .63369 .3772 .64712 .368 .64663 Metetra
.58 .01 0 r
.368 .64663 .3772 .64712 .39134 .65389 .38213 .65475 Metetra
.148 0 0 r
.38213 .65475 .39134 .65389 .40587 .65414 .39672 .65554 Metetra
.422 .868 .848 r
.39672 .65554 .40587 .65414 .42069 .64877 .41164 .65 Metetra
.731 .985 .899 r
.41164 .65 .42069 .64877 .4357 .63907 .42676 .6396 Metetra
.84 .977 .877 r
.42676 .6396 .4357 .63907 .4508 .62644 .442 .6259 Metetra
.874 .947 .858 r
.442 .6259 .4508 .62644 .46594 .61217 .45727 .61037 Metetra
.879 .918 .848 r
.45727 .61037 .46594 .61217 .48108 .59735 .47254 .59424 Metetra
.87 .891 .847 r
.47254 .59424 .48108 .59735 .49621 .5828 .48779 .57842 Metetra
.853 .867 .85 r
.48779 .57842 .49621 .5828 .51134 .5691 .50302 .56355 Metetra
.831 .845 .858 r
.50302 .56355 .51134 .5691 .52646 .55662 .51823 .55007 Metetra
.804 .824 .867 r
.51823 .55007 .52646 .55662 .54161 .54557 .53345 .53819 Metetra
.773 .805 .878 r
.53345 .53819 .54161 .54557 .55679 .53606 .54871 .52806 Metetra
.739 .786 .89 r
.54871 .52806 .55679 .53606 .57204 .52814 .56401 .5197 Metetra
.701 .768 .904 r
.56401 .5197 .57204 .52814 .58737 .52183 .57941 .51314 Metetra
.657 .751 .919 r
.57941 .51314 .58737 .52183 .60281 .51712 .5949 .50838 Metetra
.604 .732 .935 r
.5949 .50838 .60281 .51712 .61839 .51403 .61054 .50544 Metetra
.538 .71 .95 r
.61054 .50544 .61839 .51403 .63412 .51256 .62633 .50431 Metetra
.454 .681 .962 r
.62633 .50431 .63412 .51256 .65002 .51271 .64231 .505 Metetra
.345 .64 .963 r
.64231 .505 .65002 .51271 .66614 .51444 .65851 .50748 Metetra
.208 .582 .943 r
.65851 .50748 .66614 .51444 .68248 .51767 .67494 .51164 Metetra
.052 .505 .891 r
.67494 .51164 .68248 .51767 .69905 .52219 .69164 .51724 Metetra
0 .421 .808 r
.69164 .51724 .69905 .52219 .71587 .52762 .70858 .52389 Metetra
0 .353 .715 r
.70858 .52389 .71587 .52762 .7329 .53339 .72576 .53093 Metetra
.262 0 0 r
.72576 .53093 .7329 .53339 .7501 .53865 .74312 .53741 Metetra
.228 0 0 r
.74312 .53741 .7501 .53865 .76738 .54229 .76058 .54207 Metetra
.077 0 0 r
.76058 .54207 .76738 .54229 .78463 .54295 .77797 .54338 Metetra
.234 .752 .787 r
.77797 .54338 .78463 .54295 .80166 .53914 .79514 .53967 Metetra
.623 .96 .853 r
.79514 .53967 .80166 .53914 .81829 .52947 .81185 .52938 Metetra
.867 .995 .806 r
.81185 .52938 .81829 .52947 .83434 .51297 .82792 .5114 Metetra
.953 .94 .733 r
.82792 .5114 .83434 .51297 .84969 .48944 .84319 .48553 Metetra
.971 .878 .684 r
.84319 .48553 .84969 .48944 .86431 .45981 .85766 .45285 Metetra
.964 .829 .662 r
.85766 .45285 .86431 .45981 .87836 .42633 .87148 .41593 Metetra
.941 .789 .663 r
.87148 .41593 .87836 .42633 .89216 .39255 .88502 .37873 Metetra
.901 .751 .685 r
.88502 .37873 .89216 .39255 .90626 .36291 .89888 .34622 Metetra
.747 .33 0 r
.29622 .52166 .30465 .53555 .31678 .55705 .30815 .54552 Metetra
.88 .461 .043 r
.30815 .54552 .31678 .55705 .32894 .58313 .32008 .57445 Metetra
.893 .47 .115 r
.32008 .57445 .32894 .58313 .34142 .60906 .33235 .60328 Metetra
.867 .421 .106 r
.33235 .60328 .34142 .60906 .35442 .63105 .3452 .62784 Metetra
.795 .303 .02 r
.3452 .62784 .35442 .63105 .368 .64663 .3587 .64539 Metetra
.61 .044 0 r
.3587 .64539 .368 .64663 .38213 .65475 .37282 .65473 Metetra
.126 0 0 r
.37282 .65473 .38213 .65475 .39672 .65554 .38746 .65601 Metetra
.5 .905 .91 r
.38746 .65601 .39672 .65554 .41164 .65 .40247 .65032 Metetra
.776 .979 .911 r
.40247 .65032 .41164 .65 .42676 .6396 .41772 .63926 Metetra
.861 .955 .87 r
.41772 .63926 .42676 .6396 .442 .6259 .43309 .62458 Metetra
.884 .924 .844 r
.43309 .62458 .442 .6259 .45727 .61037 .4485 .6079 Metetra
.886 .897 .833 r
.4485 .6079 .45727 .61037 .47254 .59424 .4639 .59056 Metetra
.876 .875 .831 r
.4639 .59056 .47254 .59424 .48779 .57842 .47927 .57357 Metetra
.859 .855 .836 r
.47927 .57357 .48779 .57842 .50302 .56355 .4946 .55765 Metetra
.837 .837 .845 r
.4946 .55765 .50302 .56355 .51823 .55007 .5099 .54325 Metetra
.811 .82 .857 r
.5099 .54325 .51823 .55007 .53345 .53819 .52521 .53062 Metetra
.78 .803 .871 r
.52521 .53062 .53345 .53819 .54871 .52806 .54053 .5199 Metetra
.745 .786 .886 r
.54053 .5199 .54871 .52806 .56401 .5197 .5559 .51114 Metetra
.705 .769 .902 r
.5559 .51114 .56401 .5197 .57941 .51314 .57135 .50436 Metetra
.657 .75 .919 r
.57135 .50436 .57941 .51314 .5949 .50838 .58691 .49955 Metetra
.599 .73 .936 r
.58691 .49955 .5949 .50838 .61054 .50544 .6026 .49674 Metetra
.525 .703 .952 r
.6026 .49674 .61054 .50544 .62633 .50431 .61846 .49592 Metetra
.431 .668 .961 r
.61846 .49592 .62633 .50431 .64231 .505 .63451 .49709 Metetra
.31 .617 .957 r
.63451 .49709 .64231 .505 .65851 .50748 .65078 .50024 Metetra
.163 .548 .928 r
.65078 .50024 .65851 .50748 .67494 .51164 .66731 .50523 Metetra
.004 .462 .867 r
.66731 .50523 .67494 .51164 .69164 .51724 .6841 .51183 Metetra
0 .376 .784 r
.6841 .51183 .69164 .51724 .70858 .52389 .70117 .51957 Metetra
0 .313 .701 r
.70117 .51957 .70858 .52389 .72576 .53093 .71848 .52776 Metetra
0 .301 .65 r
.71848 .52776 .72576 .53093 .74312 .53741 .73599 .53535 Metetra
.229 0 0 r
.73599 .53535 .74312 .53741 .76058 .54207 .75359 .54095 Metetra
0 .534 .737 r
.75359 .54095 .76058 .54207 .77797 .54338 .77114 .54286 Metetra
.306 .798 .864 r
.77114 .54286 .77797 .54338 .79514 .53967 .78843 .53923 Metetra
.694 .98 .893 r
.78843 .53923 .79514 .53967 .81185 .52938 .80523 .52835 Metetra
.897 .98 .806 r
.80523 .52835 .81185 .52938 .82792 .5114 .82132 .50901 Metetra
.961 .919 .72 r
.82132 .50901 .82792 .5114 .84319 .48553 .83655 .481 Metetra
.974 .863 .668 r
.83655 .481 .84319 .48553 .85766 .45285 .8509 .44554 Metetra
.967 .82 .646 r
.8509 .44554 .85766 .45285 .87148 .41593 .86453 .40549 Metetra
.948 .787 .648 r
.86453 .40549 .87148 .41593 .88502 .37873 .87787 .3652 Metetra
.913 .756 .673 r
.87787 .3652 .88502 .37873 .89888 .34622 .89153 .33011 Metetra
.82 .399 0 r
.28769 .50838 .29622 .52166 .30815 .54552 .29942 .53433 Metetra
.903 .495 .108 r
.29942 .53433 .30815 .54552 .32008 .57445 .31112 .56577 Metetra
.906 .496 .158 r
.31112 .56577 .32008 .57445 .33235 .60328 .3232 .59717 Metetra
.881 .448 .138 r
.3232 .59717 .33235 .60328 .3452 .62784 .33589 .624 Metetra
.815 .334 .041 r
.33589 .624 .3452 .62784 .3587 .64539 .34931 .64329 Metetra
.631 .068 0 r
.34931 .64329 .3587 .64539 .37282 .65473 .36342 .65373 Metetra
.089 0 0 r
.36342 .65373 .37282 .65473 .38746 .65601 .3781 .65544 Metetra
.567 .922 .954 r
.3781 .65544 .38746 .65601 .40247 .65032 .3932 .6496 Metetra
.805 .959 .912 r
.3932 .6496 .40247 .65032 .41772 .63926 .40857 .63796 Metetra
.871 .929 .86 r
.40857 .63796 .41772 .63926 .43309 .62458 .42408 .6224 Metetra
.889 .9 .831 r
.42408 .6224 .43309 .62458 .4485 .6079 .43963 .60468 Metetra
.888 .877 .819 r
.43963 .60468 .4485 .6079 .4639 .59056 .45516 .58626 Metetra
.879 .859 .818 r
.45516 .58626 .4639 .59056 .47927 .57357 .47064 .56823 Metetra
.863 .843 .824 r
.47064 .56823 .47927 .57357 .4946 .55765 .48608 .55136 Metetra
.842 .829 .834 r
.48608 .55136 .4946 .55765 .5099 .54325 .50148 .53613 Metetra
.816 .814 .848 r
.50148 .53613 .5099 .54325 .52521 .53062 .51687 .52284 Metetra
.786 .8 .864 r
.51687 .52284 .52521 .53062 .54053 .5199 .53227 .5116 Metetra
.75 .785 .881 r
.53227 .5116 .54053 .5199 .5559 .51114 .54771 .50248 Metetra
.708 .769 .9 r
.54771 .50248 .5559 .51114 .57135 .50436 .56322 .49549 Metetra
.657 .75 .919 r
.56322 .49549 .57135 .50436 .58691 .49955 .57883 .49063 Metetra
.594 .728 .937 r
.57883 .49063 .58691 .49955 .6026 .49674 .59458 .48792 Metetra
.513 .697 .952 r
.59458 .48792 .6026 .49674 .61846 .49592 .6105 .48736 Metetra
.41 .655 .959 r
.6105 .48736 .61846 .49592 .63451 .49709 .62661 .48897 Metetra
.28 .596 .949 r
.62661 .48897 .63451 .49709 .65078 .50024 .64296 .49269 Metetra
.127 .517 .913 r
.64296 .49269 .65078 .50024 .66731 .50523 .65956 .49842 Metetra
0 .426 .848 r
.65956 .49842 .66731 .50523 .6841 .51183 .67645 .50589 Metetra
0 .34 .767 r
.67645 .50589 .6841 .51183 .70117 .51957 .69362 .51461 Metetra
0 .284 .696 r
.69362 .51461 .70117 .51957 .71848 .52776 .71105 .52382 Metetra
0 .283 .662 r
.71105 .52382 .71848 .52776 .73599 .53535 .72869 .53239 Metetra
0 .363 .691 r
.72869 .53239 .73599 .53535 .75359 .54095 .74644 .53881 Metetra
0 .556 .8 r
.74644 .53881 .75359 .54095 .77114 .54286 .76412 .54125 Metetra
.378 .83 .928 r
.76412 .54125 .77114 .54286 .78843 .53923 .78153 .5377 Metetra
.746 .98 .916 r
.78153 .5377 .78843 .53923 .80523 .52835 .79841 .52628 Metetra
.914 .959 .803 r
.79841 .52628 .80523 .52835 .82132 .50901 .81455 .50572 Metetra
.964 .899 .709 r
.81455 .50572 .82132 .50901 .83655 .481 .82976 .47579 Metetra
.974 .849 .655 r
.82976 .47579 .83655 .481 .8509 .44554 .84402 .43786 Metetra
.969 .812 .633 r
.84402 .43786 .8509 .44554 .86453 .40549 .85753 .39502 Metetra
.954 .785 .635 r
.85753 .39502 .86453 .40549 .87787 .3652 .8707 .35199 Metetra
.924 .761 .662 r
.8707 .35199 .87787 .3652 .89153 .33011 .88421 .31462 Metetra
.863 .445 0 r
.27905 .49578 .28769 .50838 .29942 .53433 .29059 .5235 Metetra
.916 .517 .156 r
.29059 .5235 .29942 .53433 .31112 .56577 .30208 .55709 Metetra
.914 .514 .19 r
.30208 .55709 .31112 .56577 .3232 .59717 .31396 .59068 Metetra
.891 .468 .16 r
.31396 .59068 .3232 .59717 .33589 .624 .32651 .61947 Metetra
.83 .357 .053 r
.32651 .61947 .33589 .624 .34931 .64329 .33984 .64027 Metetra
.643 .083 0 r
.33984 .64027 .34931 .64329 .36342 .65373 .35393 .65164 Metetra
0 .535 .77 r
.35393 .65164 .36342 .65373 .3781 .65544 .36865 .65372 Metetra
.619 .919 .977 r
.36865 .65372 .3781 .65544 .3932 .6496 .38384 .64775 Metetra
.82 .933 .905 r
.38384 .64775 .3932 .6496 .40857 .63796 .39933 .63559 Metetra
.874 .903 .848 r
.39933 .63559 .40857 .63796 .42408 .6224 .41497 .61926 Metetra
.889 .878 .818 r
.41497 .61926 .42408 .6224 .43963 .60468 .43065 .60064 Metetra
.888 .859 .807 r
.43065 .60064 .43963 .60468 .45516 .58626 .44631 .58128 Metetra
.879 .844 .806 r
.44631 .58128 .45516 .58626 .47064 .56823 .46192 .56234 Metetra
.864 .832 .813 r
.46192 .56234 .47064 .56823 .48608 .55136 .47747 .54465 Metetra
.845 .82 .825 r
.47747 .54465 .48608 .55136 .50148 .53613 .49297 .52871 Metetra
.82 .809 .84 r
.49297 .52871 .50148 .53613 .51687 .52284 .50844 .51482 Metetra
.79 .797 .858 r
.50844 .51482 .51687 .52284 .53227 .5116 .52392 .50313 Metetra
.754 .784 .878 r
.52392 .50313 .53227 .5116 .54771 .50248 .53943 .49369 Metetra
.711 .769 .898 r
.53943 .49369 .54771 .50248 .56322 .49549 .555 .48652 Metetra
.657 .75 .919 r
.555 .48652 .56322 .49549 .57883 .49063 .57067 .48161 Metetra
.589 .726 .938 r
.57067 .48161 .57883 .49063 .59458 .48792 .58648 .47898 Metetra
.503 .692 .953 r
.58648 .47898 .59458 .48792 .6105 .48736 .60245 .47865 Metetra
.393 .644 .957 r
.60245 .47865 .6105 .48736 .62661 .48897 .61862 .48061 Metetra
.256 .577 .942 r
.61862 .48061 .62661 .48897 .64296 .49269 .63503 .48483 Metetra
.099 .492 .901 r
.63503 .48483 .64296 .49269 .65956 .49842 .65171 .49118 Metetra
0 .397 .833 r
.65171 .49118 .65956 .49842 .67645 .50589 .66868 .49939 Metetra
0 .314 .757 r
.66868 .49939 .67645 .50589 .69362 .51461 .68594 .50895 Metetra
0 .265 .697 r
.68594 .50895 .69362 .51461 .71105 .52382 .70348 .51903 Metetra
0 .274 .68 r
.70348 .51903 .71105 .52382 .72869 .53239 .72124 .52844 Metetra
0 .367 .73 r
.72124 .52844 .72869 .53239 .74644 .53881 .7391 .53557 Metetra
.043 .577 .857 r
.7391 .53557 .74644 .53881 .76412 .54125 .7569 .53846 Metetra
.443 .845 .973 r
.7569 .53846 .76412 .54125 .78153 .5377 .77443 .53496 Metetra
.78 .964 .924 r
.77443 .53496 .78153 .5377 .79841 .52628 .7914 .52307 Metetra
.922 .935 .797 r
.7914 .52307 .79841 .52628 .81455 .50572 .80759 .50144 Metetra
.964 .88 .701 r
.80759 .50144 .81455 .50572 .82976 .47579 .82281 .46985 Metetra
.973 .836 .647 r
.82281 .46985 .82976 .47579 .84402 .43786 .83703 .42977 Metetra
.97 .805 .623 r
.83703 .42977 .84402 .43786 .85753 .39502 .85045 .38452 Metetra
.958 .783 .625 r
.85045 .38452 .85753 .39502 .8707 .35199 .86353 .33915 Metetra
.932 .765 .652 r
.86353 .33915 .8707 .35199 .88421 .31462 .87694 .29983 Metetra
.889 .475 .063 r
.27028 .4839 .27905 .49578 .29059 .5235 .28165 .51306 Metetra
.923 .532 .19 r
.28165 .51306 .29059 .5235 .30208 .55709 .29294 .54842 Metetra
.919 .526 .212 r
.29294 .54842 .30208 .55709 .31396 .59068 .30464 .58382 Metetra
.898 .482 .175 r
.30464 .58382 .31396 .59068 .32651 .61947 .31704 .61422 Metetra
.84 .373 .057 r
.31704 .61422 .32651 .61947 .33984 .64027 .33029 .63625 Metetra
.645 .088 0 r
.33029 .63625 .33984 .64027 .35393 .65164 .34437 .6484 Metetra
.024 .567 .841 r
.34437 .6484 .35393 .65164 .36865 .65372 .35913 .65077 Metetra
.654 .901 .982 r
.35913 .65077 .36865 .65372 .38384 .64775 .3744 .64468 Metetra
.826 .903 .894 r
.3744 .64468 .38384 .64775 .39933 .63559 .39 .63209 Metetra
.873 .876 .836 r
.39 .63209 .39933 .63559 .41497 .61926 .40577 .61511 Metetra
.885 .856 .807 r
.40577 .61511 .41497 .61926 .43065 .60064 .42159 .59573 Metetra
.885 .841 .796 r
.42159 .59573 .43065 .60064 .44631 .58128 .43738 .57558 Metetra
.877 .83 .796 r
.43738 .57558 .44631 .58128 .46192 .56234 .45311 .55588 Metetra
.864 .821 .804 r
.45311 .55588 .46192 .56234 .47747 .54465 .46877 .53749 Metetra
.846 .812 .817 r
.46877 .53749 .47747 .54465 .49297 .52871 .48436 .52096 Metetra
.822 .804 .834 r
.48436 .52096 .49297 .52871 .50844 .51482 .49993 .50658 Metetra
.793 .794 .853 r
.49993 .50658 .50844 .51482 .52392 .50313 .51548 .4945 Metetra
.757 .783 .874 r
.51548 .4945 .52392 .50313 .53943 .49369 .53106 .48479 Metetra
.713 .769 .897 r
.53106 .48479 .53943 .49369 .555 .48652 .5467 .47745 Metetra
.657 .75 .919 r
.5467 .47745 .555 .48652 .57067 .48161 .56243 .47249 Metetra
.586 .724 .939 r
.56243 .47249 .57067 .48161 .58648 .47898 .57829 .46992 Metetra
.495 .687 .953 r
.57829 .46992 .58648 .47898 .60245 .47865 .59431 .46976 Metetra
.379 .635 .955 r
.59431 .46976 .60245 .47865 .61862 .48061 .61054 .472 Metetra
.238 .562 .936 r
.61054 .472 .61862 .48061 .63503 .48483 .62701 .47662 Metetra
.08 .472 .891 r
.62701 .47662 .63503 .48483 .65171 .49118 .64375 .48348 Metetra
0 .377 .824 r
.64375 .48348 .65171 .49118 .66868 .49939 .66079 .4923 Metetra
0 .297 .754 r
.66079 .4923 .66868 .49939 .68594 .50895 .67813 .50253 Metetra
0 .255 .705 r
.67813 .50253 .68594 .50895 .70348 .51903 .69576 .51333 Metetra
0 .272 .703 r
.69576 .51333 .70348 .51903 .72124 .52844 .71361 .52342 Metetra
0 .377 .769 r
.71361 .52342 .72124 .52844 .7391 .53557 .73158 .53113 Metetra
.1 .593 .903 r
.73158 .53113 .7391 .53557 .7569 .53846 .7495 .53438 Metetra
.496 .843 .996 r
.7495 .53438 .7569 .53846 .77443 .53496 .76713 .53092 Metetra
.799 .938 .92 r
.76713 .53092 .77443 .53496 .7914 .52307 .7842 .51863 Metetra
.923 .91 .79 r
.7842 .51863 .7914 .52307 .80759 .50144 .80045 .4961 Metetra
.962 .863 .695 r
.80045 .4961 .80759 .50144 .82281 .46985 .8157 .46312 Metetra
.971 .825 .64 r
.8157 .46312 .82281 .46985 .83703 .42977 .82992 .42125 Metetra
.97 .8 .616 r
.82992 .42125 .83703 .42977 .85045 .38452 .84331 .37401 Metetra
.961 .782 .616 r
.84331 .37401 .85045 .38452 .86353 .33915 .85633 .3267 Metetra
.939 .77 .645 r
.85633 .3267 .86353 .33915 .87694 .29983 .8697 .28576 Metetra
.903 .493 .111 r
.26138 .47277 .27028 .4839 .28165 .51306 .27259 .50304 Metetra
.927 .541 .216 r
.27259 .50304 .28165 .51306 .29294 .54842 .28371 .53975 Metetra
.923 .534 .228 r
.28371 .53975 .29294 .54842 .30464 .58382 .29524 .57654 Metetra
.903 .491 .183 r
.29524 .57654 .30464 .58382 .31704 .61422 .30751 .60819 Metetra
.846 .383 .053 r
.30751 .60819 .31704 .61422 .33029 .63625 .32069 .63118 Metetra
.637 .083 0 r
.32069 .63118 .33029 .63625 .34437 .6484 .33474 .64391 Metetra
.095 .593 .899 r
.33474 .64391 .34437 .6484 .35913 .65077 .34954 .64651 Metetra
.676 .872 .973 r
.34954 .64651 .35913 .65077 .3744 .64468 .36489 .64032 Metetra
.825 .872 .88 r
.36489 .64032 .3744 .64468 .39 .63209 .3806 .62737 Metetra
.867 .851 .824 r
.3806 .62737 .39 .63209 .40577 .61511 .39649 .60988 Metetra
.88 .835 .798 r
.39649 .60988 .40577 .61511 .42159 .59573 .41243 .58989 Metetra
.881 .824 .787 r
.41243 .58989 .42159 .59573 .43738 .57558 .42835 .56911 Metetra
.874 .816 .788 r
.42835 .56911 .43738 .57558 .45311 .55588 .4442 .54881 Metetra
.863 .81 .797 r
.4442 .54881 .45311 .55588 .46877 .53749 .45997 .52987 Metetra
.846 .804 .81 r
.45997 .52987 .46877 .53749 .48436 .52096 .47567 .51286 Metetra
.823 .798 .828 r
.47567 .51286 .48436 .52096 .49993 .50658 .49132 .49808 Metetra
.795 .791 .849 r
.49132 .49808 .49993 .50658 .51548 .4945 .50695 .4857 Metetra
.759 .782 .872 r
.50695 .4857 .51548 .4945 .53106 .48479 .5226 .47576 Metetra
.714 .769 .895 r
.5226 .47576 .53106 .48479 .5467 .47745 .5383 .46828 Metetra
.657 .75 .919 r
.5383 .46828 .5467 .47745 .56243 .47249 .55409 .46327 Metetra
.583 .723 .939 r
.55409 .46327 .56243 .47249 .57829 .46992 .57001 .46074 Metetra
.489 .684 .953 r
.57001 .46074 .57829 .46992 .59431 .46976 .58609 .46069 Metetra
.369 .627 .953 r
.58609 .46069 .59431 .46976 .61054 .472 .60237 .46315 Metetra
.226 .55 .931 r
.60237 .46315 .61054 .472 .62701 .47662 .61889 .46806 Metetra
.07 .458 .884 r
.61889 .46806 .62701 .47662 .64375 .48348 .63568 .47531 Metetra
0 .364 .819 r
.63568 .47531 .64375 .48348 .66079 .4923 .65278 .48458 Metetra
0 .288 .756 r
.65278 .48458 .66079 .4923 .67813 .50253 .67019 .49532 Metetra
0 .253 .718 r
.67019 .49532 .67813 .50253 .69576 .51333 .68789 .50666 Metetra
0 .278 .729 r
.68789 .50666 .69576 .51333 .71361 .52342 .70583 .51728 Metetra
0 .39 .805 r
.70583 .51728 .71361 .52342 .73158 .53113 .72389 .52543 Metetra
.16 .603 .935 r
.72389 .52543 .73158 .53113 .7495 .53438 .7419 .52895 Metetra
.536 .826 .999 r
.7419 .52895 .7495 .53438 .76713 .53092 .75963 .5255 Metetra
.806 .906 .91 r
.75963 .5255 .76713 .53092 .7842 .51863 .77679 .51289 Metetra
.919 .885 .783 r
.77679 .51289 .7842 .51863 .80045 .4961 .79313 .48964 Metetra
.957 .847 .691 r
.79313 .48964 .80045 .4961 .8157 .46312 .80844 .45556 Metetra
.969 .816 .637 r
.80844 .45556 .8157 .46312 .82992 .42125 .8227 .41229 Metetra
.97 .795 .611 r
.8227 .41229 .82992 .42125 .84331 .37401 .83611 .36349 Metetra
.963 .782 .61 r
.83611 .36349 .84331 .37401 .85633 .3267 .84913 .31465 Metetra
.945 .774 .639 r
.84913 .31465 .85633 .3267 .8697 .28576 .86249 .27245 Metetra
.91 .504 .147 r
.25233 .46242 .26138 .47277 .27259 .50304 .26342 .49346 Metetra
.928 .546 .234 r
.26342 .49346 .27259 .50304 .28371 .53975 .27438 .5311 Metetra
.924 .539 .238 r
.27438 .5311 .28371 .53975 .29524 .57654 .28576 .56885 Metetra
.906 .498 .186 r
.28576 .56885 .29524 .57654 .30751 .60819 .29792 .60135 Metetra
.849 .388 .043 r
.29792 .60135 .30751 .60819 .32069 .63118 .31103 .625 Metetra
.616 .068 0 r
.31103 .625 .32069 .63118 .33474 .64391 .32507 .63814 Metetra
.167 .609 .937 r
.32507 .63814 .33474 .64391 .34954 .64651 .3399 .64088 Metetra
.687 .84 .956 r
.3399 .64088 .34954 .64651 .36489 .64032 .35532 .6346 Metetra
.82 .842 .865 r
.35532 .6346 .36489 .64032 .3806 .62737 .37113 .62138 Metetra
.86 .827 .814 r
.37113 .62138 .3806 .62737 .39649 .60988 .38713 .60351 Metetra
.873 .816 .789 r
.38713 .60351 .39649 .60988 .41243 .58989 .4032 .58309 Metetra
.875 .808 .78 r
.4032 .58309 .41243 .58989 .42835 .56911 .41924 .56185 Metetra
.87 .804 .782 r
.41924 .56185 .42835 .56911 .4442 .54881 .4352 .54111 Metetra
.86 .8 .791 r
.4352 .54111 .4442 .54881 .45997 .52987 .45108 .52177 Metetra
.844 .797 .805 r
.45108 .52177 .45997 .52987 .47567 .51286 .46688 .50441 Metetra
.823 .793 .824 r
.46688 .50441 .47567 .51286 .49132 .49808 .48262 .48934 Metetra
.796 .788 .845 r
.48262 .48934 .49132 .49808 .50695 .4857 .49833 .47672 Metetra
.76 .78 .869 r
.49833 .47672 .50695 .4857 .5226 .47576 .51405 .46661 Metetra
.715 .768 .894 r
.51405 .46661 .5226 .47576 .5383 .46828 .52982 .45902 Metetra
.657 .75 .919 r
.52982 .45902 .5383 .46828 .55409 .46327 .54567 .45395 Metetra
.582 .722 .94 r
.54567 .45395 .55409 .46327 .57001 .46074 .56164 .45143 Metetra
.485 .681 .953 r
.56164 .45143 .57001 .46074 .58609 .46069 .57777 .45145 Metetra
.363 .622 .951 r
.57777 .45145 .58609 .46069 .60237 .46315 .5941 .45403 Metetra
.22 .543 .927 r
.5941 .45403 .60237 .46315 .61889 .46806 .61067 .45914 Metetra
.069 .45 .881 r
.61067 .45914 .61889 .46806 .63568 .47531 .62751 .46663 Metetra
0 .358 .82 r
.62751 .46663 .63568 .47531 .65278 .48458 .64466 .4762 Metetra
0 .288 .764 r
.64466 .4762 .65278 .48458 .67019 .49532 .66212 .48729 Metetra
0 .259 .736 r
.66212 .48729 .67019 .49532 .68789 .50666 .67988 .49899 Metetra
0 .29 .756 r
.67988 .49899 .68789 .50666 .70583 .51728 .69788 .50996 Metetra
0 .405 .836 r
.69788 .50996 .70583 .51728 .72389 .52543 .71602 .5184 Metetra
.217 .605 .95 r
.71602 .5184 .72389 .52543 .7419 .52895 .73412 .52209 Metetra
.565 .801 .988 r
.73412 .52209 .7419 .52895 .75963 .5255 .75193 .51863 Metetra
.806 .872 .896 r
.75193 .51863 .75963 .5255 .77679 .51289 .76919 .50577 Metetra
.912 .86 .777 r
.76919 .50577 .77679 .51289 .79313 .48964 .78562 .482 Metetra
.952 .831 .689 r
.78562 .482 .79313 .48964 .80844 .45556 .80102 .44713 Metetra
.965 .807 .635 r
.80102 .44713 .80844 .45556 .8227 .41229 .81536 .40285 Metetra
.969 .79 .608 r
.81536 .40285 .8227 .41229 .83611 .36349 .82883 .35295 Metetra
.965 .782 .606 r
.82883 .35295 .83611 .36349 .84913 .31465 .84191 .30304 Metetra
.951 .779 .634 r
.84191 .30304 .84913 .31465 .86249 .27245 .85531 .25994 Metetra
.913 .509 .174 r
.24313 .45288 .25233 .46242 .26342 .49346 .25411 .48432 Metetra
.928 .548 .247 r
.25411 .48432 .26342 .49346 .27438 .5311 .26494 .52245 Metetra
.925 .542 .244 r
.26494 .52245 .27438 .5311 .28576 .56885 .2762 .56072 Metetra
.908 .5 .184 r
.2762 .56072 .28576 .56885 .29792 .60135 .28827 .59368 Metetra
.848 .387 .024 r
.28827 .59368 .29792 .60135 .31103 .625 .30132 .61768 Metetra
.581 .042 0 r
.30132 .61768 .31103 .625 .32507 .63814 .31535 .63102 Metetra
.235 .615 .955 r
.31535 .63102 .32507 .63814 .3399 .64088 .33021 .63382 Metetra
.692 .806 .936 r
.33021 .63382 .3399 .64088 .35532 .6346 .3457 .62747 Metetra
.812 .813 .851 r
.3457 .62747 .35532 .6346 .37113 .62138 .3616 .61409 Metetra
.851 .804 .804 r
.3616 .61409 .37113 .62138 .38713 .60351 .37771 .59598 Metetra
.865 .798 .782 r
.37771 .59598 .38713 .60351 .4032 .58309 .3939 .57528 Metetra
.869 .794 .774 r
.3939 .57528 .4032 .58309 .41924 .56185 .41005 .55377 Metetra
.865 .792 .777 r
.41005 .55377 .41924 .56185 .4352 .54111 .42612 .53276 Metetra
.856 .79 .786 r
.42612 .53276 .4352 .54111 .45108 .52177 .4421 .51317 Metetra
.842 .79 .801 r
.4421 .51317 .45108 .52177 .46688 .50441 .458 .49559 Metetra
.822 .788 .82 r
.458 .49559 .46688 .50441 .48262 .48934 .47383 .48033 Metetra
.795 .785 .843 r
.47383 .48033 .48262 .48934 .49833 .47672 .48962 .46756 Metetra
.761 .779 .868 r
.48962 .46756 .49833 .47672 .51405 .46661 .50542 .45733 Metetra
.716 .768 .894 r
.50542 .45733 .51405 .46661 .52982 .45902 .52125 .44965 Metetra
.657 .75 .919 r
.52125 .44965 .52982 .45902 .54567 .45395 .53716 .44453 Metetra
.581 .722 .94 r
.53716 .44453 .54567 .45395 .56164 .45143 .55318 .44198 Metetra
.483 .68 .952 r
.55318 .44198 .56164 .45143 .57777 .45145 .56936 .44202 Metetra
.361 .619 .95 r
.56936 .44202 .57777 .45145 .5941 .45403 .58574 .44465 Metetra
.221 .539 .925 r
.58574 .44465 .5941 .45403 .61067 .45914 .60235 .44983 Metetra
.076 .447 .88 r
.60235 .44983 .61067 .45914 .62751 .46663 .61923 .45744 Metetra
0 .359 .824 r
.61923 .45744 .62751 .46663 .64466 .4762 .63642 .46715 Metetra
0 .294 .776 r
.63642 .46715 .64466 .4762 .66212 .48729 .65392 .47841 Metetra
0 .271 .756 r
.65392 .47841 .66212 .48729 .67988 .49899 .67173 .49029 Metetra
0 .307 .782 r
.67173 .49029 .67988 .49899 .69788 .50996 .68978 .50143 Metetra
.005 .42 .859 r
.68978 .50143 .69788 .50996 .71602 .5184 .70798 .51 Metetra
.27 .603 .953 r
.70798 .51 .71602 .5184 .73412 .52209 .72614 .51376 Metetra
.584 .771 .969 r
.72614 .51376 .73412 .52209 .75193 .51863 .74404 .51027 Metetra
.8 .839 .88 r
.74404 .51027 .75193 .51863 .76919 .50577 .76139 .49724 Metetra
.903 .837 .771 r
.76139 .49724 .76919 .50577 .78562 .482 .77793 .47315 Metetra
.945 .817 .688 r
.77793 .47315 .78562 .482 .80102 .44713 .79344 .43781 Metetra
.961 .799 .635 r
.79344 .43781 .80102 .44713 .81536 .40285 .80789 .39295 Metetra
.967 .787 .607 r
.80789 .39295 .81536 .40285 .82883 .35295 .82148 .3424 Metetra
.966 .782 .604 r
.82148 .3424 .82883 .35295 .84191 .30304 .83467 .29186 Metetra
.955 .784 .631 r
.83467 .29186 .84191 .30304 .85531 .25994 .84817 .24825 Metetra
.912 .51 .193 r
.23376 .44416 .24313 .45288 .25411 .48432 .24467 .47563 Metetra
.927 .547 .255 r
.24467 .47563 .25411 .48432 .26494 .52245 .25541 .51382 Metetra
.925 .542 .245 r
.25541 .51382 .26494 .52245 .2762 .56072 .26657 .55215 Metetra
.908 .5 .177 r
.26657 .55215 .2762 .56072 .28827 .59368 .27857 .58517 Metetra
.842 .381 0 r
.27857 .58517 .28827 .59368 .30132 .61768 .29158 .6092 Metetra
.528 .005 0 r
.29158 .6092 .30132 .61768 .31535 .63102 .3056 .62254 Metetra
.295 .614 .956 r
.3056 .62254 .31535 .63102 .33021 .63382 .3205 .62531 Metetra
.691 .773 .914 r
.3205 .62531 .33021 .63382 .3457 .62747 .33605 .61891 Metetra
.802 .786 .838 r
.33605 .61891 .3457 .62747 .3616 .61409 .35203 .60545 Metetra
.841 .783 .796 r
.35203 .60545 .3616 .61409 .37771 .59598 .36824 .58725 Metetra
.857 .781 .776 r
.36824 .58725 .37771 .59598 .3939 .57528 .38452 .56646 Metetra
.861 .78 .77 r
.38452 .56646 .3939 .57528 .41005 .55377 .40078 .54485 Metetra
.859 .78 .773 r
.40078 .54485 .41005 .55377 .42612 .53276 .41696 .52374 Metetra
.851 .781 .783 r
.41696 .52374 .42612 .53276 .4421 .51317 .43304 .50407 Metetra
.839 .782 .798 r
.43304 .50407 .4421 .51317 .458 .49559 .44903 .4864 Metetra
.82 .783 .818 r
.44903 .4864 .458 .49559 .47383 .48033 .46494 .47106 Metetra
.794 .781 .841 r
.46494 .47106 .47383 .48033 .48962 .46756 .48082 .45821 Metetra
.76 .777 .867 r
.48082 .45821 .48962 .46756 .50542 .45733 .49668 .44792 Metetra
.715 .767 .893 r
.49668 .44792 .50542 .45733 .52125 .44965 .51258 .44018 Metetra
.657 .75 .919 r
.51258 .44018 .52125 .44965 .53716 .44453 .52855 .43501 Metetra
.581 .722 .94 r
.52855 .43501 .53716 .44453 .55318 .44198 .54463 .43241 Metetra
.483 .679 .952 r
.54463 .43241 .55318 .44198 .56936 .44202 .56086 .4324 Metetra
.364 .618 .948 r
.56086 .4324 .56936 .44202 .58574 .44465 .57728 .43499 Metetra
.228 .539 .925 r
.57728 .43499 .58574 .44465 .60235 .44983 .59393 .44014 Metetra
.091 .45 .882 r
.59393 .44014 .60235 .44983 .61923 .45744 .61085 .44772 Metetra
0 .367 .832 r
.61085 .44772 .61923 .45744 .63642 .46715 .62807 .45742 Metetra
0 .307 .791 r
.62807 .45742 .63642 .46715 .65392 .47841 .6456 .46866 Metetra
0 .289 .777 r
.6456 .46866 .65392 .47841 .67173 .49029 .66344 .48053 Metetra
0 .327 .805 r
.66344 .48053 .67173 .49029 .68978 .50143 .68153 .49165 Metetra
.066 .435 .874 r
.68153 .49165 .68978 .50143 .70798 .51 .69977 .5002 Metetra
.317 .597 .946 r
.69977 .5002 .70798 .51 .72614 .51376 .71799 .50393 Metetra
.596 .742 .946 r
.71799 .50393 .72614 .51376 .74404 .51027 .73596 .50038 Metetra
.792 .807 .864 r
.73596 .50038 .74404 .51027 .76139 .49724 .75341 .48727 Metetra
.891 .815 .766 r
.75341 .48727 .76139 .49724 .77793 .47315 .77006 .46306 Metetra
.937 .804 .688 r
.77006 .46306 .77793 .47315 .79344 .43781 .78571 .42757 Metetra
.957 .792 .636 r
.78571 .42757 .79344 .43781 .80789 .39295 .80031 .38255 Metetra
.965 .784 .608 r
.80031 .38255 .80789 .39295 .82148 .3424 .81406 .33183 Metetra
.966 .783 .604 r
.81406 .33183 .82148 .3424 .83467 .29186 .8274 .28114 Metetra
.958 .789 .63 r
.8274 .28114 .83467 .29186 .84817 .24825 .84105 .23738 Metetra
.91 .507 .205 r
.22422 .43626 .23376 .44416 .24467 .47563 .23508 .4674 Metetra
.924 .544 .259 r
.23508 .4674 .24467 .47563 .25541 .51382 .24576 .50521 Metetra
.924 .539 .243 r
.24576 .50521 .25541 .51382 .26657 .55215 .25687 .54314 Metetra
.906 .497 .165 r
.25687 .54314 .26657 .55215 .27857 .58517 .26882 .5758 Metetra
.83 .368 0 r
.26882 .5758 .27857 .58517 .29158 .6092 .28181 .59954 Metetra
0 .043 .543 r
.28181 .59954 .29158 .6092 .3056 .62254 .29584 .61268 Metetra
.346 .607 .946 r
.29584 .61268 .3056 .62254 .3205 .62531 .31076 .61534 Metetra
.688 .743 .893 r
.31076 .61534 .3205 .62531 .33605 .61891 .32637 .60889 Metetra
.791 .761 .826 r
.32637 .60889 .33605 .61891 .35203 .60545 .34242 .59545 Metetra
.83 .764 .789 r
.34242 .59545 .35203 .60545 .36824 .58725 .35871 .57732 Metetra
.847 .765 .771 r
.35871 .57732 .36824 .58725 .38452 .56646 .37509 .55661 Metetra
.853 .767 .766 r
.37509 .55661 .38452 .56646 .40078 .54485 .39145 .53508 Metetra
.852 .769 .77 r
.39145 .53508 .40078 .54485 .41696 .52374 .40772 .51406 Metetra
.846 .773 .78 r
.40772 .51406 .41696 .52374 .43304 .50407 .42389 .49445 Metetra
.834 .776 .796 r
.42389 .49445 .43304 .50407 .44903 .4864 .43997 .47683 Metetra
.817 .778 .816 r
.43997 .47683 .44903 .4864 .46494 .47106 .45597 .46152 Metetra
.792 .778 .84 r
.45597 .46152 .46494 .47106 .48082 .45821 .47192 .44868 Metetra
.759 .775 .866 r
.47192 .44868 .48082 .45821 .49668 .44792 .48786 .43837 Metetra
.715 .766 .893 r
.48786 .43837 .49668 .44792 .51258 .44018 .50383 .4306 Metetra
.657 .75 .918 r
.50383 .4306 .51258 .44018 .52855 .43501 .51986 .42537 Metetra
.582 .722 .939 r
.51986 .42537 .52855 .43501 .54463 .43241 .53599 .42269 Metetra
.486 .68 .951 r
.53599 .42269 .54463 .43241 .56086 .4324 .55227 .42259 Metetra
.37 .619 .948 r
.55227 .42259 .56086 .4324 .57728 .43499 .56873 .42506 Metetra
.241 .542 .926 r
.56873 .42506 .57728 .43499 .59393 .44014 .58541 .43007 Metetra
.113 .457 .887 r
.58541 .43007 .59393 .44014 .61085 .44772 .60236 .43748 Metetra
.005 .38 .842 r
.60236 .43748 .61085 .44772 .62807 .45742 .6196 .44699 Metetra
0 .326 .808 r
.6196 .44699 .62807 .45742 .6456 .46866 .63716 .45804 Metetra
0 .312 .799 r
.63716 .45804 .6456 .46866 .66344 .48053 .65501 .4697 Metetra
0 .35 .825 r
.65501 .4697 .66344 .48053 .68153 .49165 .67313 .48062 Metetra
.128 .449 .882 r
.67313 .48062 .68153 .49165 .69977 .5002 .6914 .48899 Metetra
.358 .589 .932 r
.6914 .48899 .69977 .5002 .71799 .50393 .70966 .49257 Metetra
.603 .714 .922 r
.70966 .49257 .71799 .50393 .73596 .50038 .7277 .48894 Metetra
.781 .777 .849 r
.7277 .48894 .73596 .50038 .75341 .48727 .74524 .47583 Metetra
.879 .793 .762 r
.74524 .47583 .75341 .48727 .77006 .46306 .76201 .45172 Metetra
.928 .791 .689 r
.76201 .45172 .77006 .46306 .78571 .42757 .77782 .41642 Metetra
.951 .785 .639 r
.77782 .41642 .78571 .42757 .80031 .38255 .79261 .37166 Metetra
.963 .782 .611 r
.79261 .37166 .80031 .38255 .81406 .33183 .80657 .32125 Metetra
.966 .784 .606 r
.80657 .32125 .81406 .33183 .8274 .28114 .82011 .27086 Metetra
.961 .795 .631 r
.82011 .27086 .8274 .28114 .84105 .23738 .83394 .22734 Metetra
.905 .501 .212 r
.21448 .42917 .22422 .43626 .23508 .4674 .22534 .45962 Metetra
.921 .538 .258 r
.22534 .45962 .23508 .4674 .24576 .50521 .23601 .4966 Metetra
.922 .534 .236 r
.23601 .4966 .24576 .50521 .25687 .54314 .2471 .53369 Metetra
.903 .489 .146 r
.2471 .53369 .25687 .54314 .26882 .5758 .25904 .56558 Metetra
.811 .346 0 r
.25904 .56558 .26882 .5758 .28181 .59954 .27203 .58871 Metetra
0 .1 .618 r
.27203 .58871 .28181 .59954 .29584 .61268 .28607 .60145 Metetra
.389 .598 .929 r
.28607 .60145 .29584 .61268 .31076 .61534 .30102 .6039 Metetra
.684 .716 .873 r
.30102 .6039 .31076 .61534 .32637 .60889 .31667 .59744 Metetra
.78 .738 .815 r
.31667 .59744 .32637 .60889 .34242 .59545 .33278 .58411 Metetra
.819 .745 .782 r
.33278 .58411 .34242 .59545 .35871 .57732 .34915 .56619 Metetra
.837 .75 .767 r
.34915 .56619 .35871 .57732 .37509 .55661 .36561 .54573 Metetra
.844 .754 .764 r
.36561 .54573 .37509 .55661 .39145 .53508 .38204 .52447 Metetra
.845 .759 .768 r
.38204 .52447 .39145 .53508 .40772 .51406 .3984 .5037 Metetra
.839 .764 .779 r
.3984 .5037 .40772 .51406 .42389 .49445 .41466 .48432 Metetra
.829 .769 .795 r
.41466 .48432 .42389 .49445 .43997 .47683 .43082 .46688 Metetra
.812 .773 .816 r
.43082 .46688 .43997 .47683 .45597 .46152 .4469 .45171 Metetra
.789 .774 .84 r
.4469 .45171 .45597 .46152 .47192 .44868 .46293 .43897 Metetra
.757 .773 .866 r
.46293 .43897 .47192 .44868 .48786 .43837 .47894 .4287 Metetra
.714 .765 .893 r
.47894 .4287 .48786 .43837 .50383 .4306 .49497 .42092 Metetra
.657 .75 .918 r
.49497 .42092 .50383 .4306 .51986 .42537 .51107 .41563 Metetra
.584 .723 .939 r
.51107 .41563 .51986 .42537 .53599 .42269 .52726 .41285 Metetra
.491 .681 .95 r
.52726 .41285 .53599 .42269 .55227 .42259 .54358 .41259 Metetra
.381 .622 .947 r
.54358 .41259 .55227 .42259 .56873 .42506 .56008 .41485 Metetra
.26 .549 .927 r
.56008 .41485 .56873 .42506 .58541 .43007 .5768 .4196 Metetra
.141 .469 .893 r
.5768 .4196 .58541 .43007 .60236 .43748 .59377 .42671 Metetra
.044 .397 .854 r
.59377 .42671 .60236 .43748 .6196 .44699 .61103 .43588 Metetra
0 .349 .825 r
.61103 .43588 .6196 .44699 .63716 .45804 .6286 .44654 Metetra
0 .337 .818 r
.6286 .44654 .63716 .45804 .65501 .4697 .64647 .4578 Metetra
.045 .374 .841 r
.64647 .4578 .65501 .4697 .67313 .48062 .66459 .46834 Metetra
.186 .461 .884 r
.66459 .46834 .67313 .48062 .6914 .48899 .68288 .47636 Metetra
.393 .581 .916 r
.68288 .47636 .6914 .48899 .70966 .49257 .70117 .4797 Metetra
.608 .688 .899 r
.70117 .4797 .70966 .49257 .7277 .48894 .71927 .47597 Metetra
.769 .75 .835 r
.71927 .47597 .7277 .48894 .74524 .47583 .7369 .46294 Metetra
.866 .773 .758 r
.7369 .46294 .74524 .47583 .76201 .45172 .7538 .43913 Metetra
.918 .778 .692 r
.7538 .43913 .76201 .45172 .77782 .41642 .76979 .40435 Metetra
.945 .778 .644 r
.76979 .40435 .77782 .41642 .79261 .37166 .7848 .36029 Metetra
.96 .78 .616 r
.7848 .36029 .79261 .37166 .80657 .32125 .79899 .31065 Metetra
.966 .786 .609 r
.79899 .31065 .80657 .32125 .82011 .27086 .81279 .26102 Metetra
.963 .802 .633 r
.81279 .26102 .82011 .27086 .83394 .22734 .82685 .21812 Metetra
.899 .491 .214 r
.20456 .4229 .21448 .42917 .22534 .45962 .21545 .45229 Metetra
.917 .53 .253 r
.21545 .45229 .22534 .45962 .23601 .4966 .22615 .488 Metetra
.918 .526 .224 r
.22615 .488 .23601 .4966 .2471 .53369 .23726 .52379 Metetra
.896 .477 .12 r
.23726 .52379 .2471 .53369 .25904 .56558 .24922 .55451 Metetra
.779 .313 0 r
.24922 .55451 .25904 .56558 .27203 .58871 .26223 .57672 Metetra
0 .164 .688 r
.26223 .57672 .27203 .58871 .28607 .60145 .27629 .58885 Metetra
.424 .588 .91 r
.27629 .58885 .28607 .60145 .30102 .6039 .29128 .59102 Metetra
.678 .692 .855 r
.29128 .59102 .30102 .6039 .31667 .59744 .30697 .58456 Metetra
.768 .717 .805 r
.30697 .58456 .31667 .59744 .33278 .58411 .32313 .57145 Metetra
.807 .728 .777 r
.32313 .57145 .33278 .58411 .34915 .56619 .33955 .55387 Metetra
.826 .735 .764 r
.33955 .55387 .34915 .56619 .36561 .54573 .35607 .53384 Metetra
.834 .742 .762 r
.35607 .53384 .36561 .54573 .38204 .52447 .37258 .51303 Metetra
.836 .749 .767 r
.37258 .51303 .38204 .52447 .3984 .5037 .38902 .49269 Metetra
.832 .756 .779 r
.38902 .49269 .3984 .5037 .41466 .48432 .40535 .47368 Metetra
.823 .762 .795 r
.40535 .47368 .41466 .48432 .43082 .46688 .42159 .45656 Metetra
.807 .768 .816 r
.42159 .45656 .43082 .46688 .4469 .45171 .43774 .44164 Metetra
.785 .771 .84 r
.43774 .44164 .4469 .45171 .46293 .43897 .45384 .42906 Metetra
.754 .77 .867 r
.45384 .42906 .46293 .43897 .47894 .4287 .46993 .41888 Metetra
.712 .764 .893 r
.46993 .41888 .47894 .4287 .49497 .42092 .48603 .41112 Metetra
.657 .75 .918 r
.48603 .41112 .49497 .42092 .51107 .41563 .50218 .40578 Metetra
.586 .724 .938 r
.50218 .40578 .51107 .41563 .52726 .41285 .51842 .40286 Metetra
.498 .684 .949 r
.51842 .40286 .52726 .41285 .54358 .41259 .5348 .40239 Metetra
.395 .628 .947 r
.5348 .40239 .54358 .41259 .56008 .41485 .55134 .40436 Metetra
.283 .559 .929 r
.55134 .40436 .56008 .41485 .5768 .4196 .56809 .40875 Metetra
.176 .485 .899 r
.56809 .40875 .5768 .4196 .59377 .42671 .58509 .41542 Metetra
.09 .419 .866 r
.58509 .41542 .59377 .42671 .61103 .43588 .60236 .42408 Metetra
.042 .375 .841 r
.60236 .42408 .61103 .43588 .6286 .44654 .61993 .43418 Metetra
.044 .365 .835 r
.61993 .43418 .6286 .44654 .64647 .4578 .6378 .44486 Metetra
.109 .398 .852 r
.6378 .44486 .64647 .4578 .66459 .46834 .65592 .45482 Metetra
.241 .473 .881 r
.65592 .45482 .66459 .46834 .68288 .47636 .67421 .46235 Metetra
.424 .574 .898 r
.67421 .46235 .68288 .47636 .70117 .4797 .69253 .46534 Metetra
.611 .665 .878 r
.69253 .46534 .70117 .4797 .71927 .47597 .71067 .46148 Metetra
.757 .725 .823 r
.71067 .46148 .71927 .47597 .7369 .46294 .72839 .44862 Metetra
.851 .754 .755 r
.72839 .44862 .7369 .46294 .7538 .43913 .74543 .42531 Metetra
.906 .766 .695 r
.74543 .42531 .7538 .43913 .76979 .40435 .76161 .39138 Metetra
.938 .772 .65 r
.76161 .39138 .76979 .40435 .7848 .36029 .77687 .34842 Metetra
.956 .778 .622 r
.77687 .34842 .7848 .36029 .79899 .31065 .79134 .30004 Metetra
.965 .789 .615 r
.79134 .30004 .79899 .31065 .81279 .26102 .80543 .25162 Metetra
.965 .809 .638 r
.80543 .25162 .81279 .26102 .82685 .21812 .81976 .20972 Metetra
.89 .477 .21 r
.19442 .41741 .20456 .4229 .21545 .45229 .2054 .4454 Metetra
.911 .518 .244 r
.2054 .4454 .21545 .45229 .22615 .488 .21617 .47941 Metetra
.913 .514 .206 r
.21617 .47941 .22615 .488 .23726 .52379 .22735 .51346 Metetra
.886 .458 .084 r
.22735 .51346 .23726 .52379 .24922 .55451 .23937 .54262 Metetra
.732 .265 0 r
.23937 .54262 .24922 .55451 .26223 .57672 .25242 .56361 Metetra
0 .229 .746 r
.25242 .56361 .26223 .57672 .27629 .58885 .26653 .57495 Metetra
.454 .579 .89 r
.26653 .57495 .27629 .58885 .29128 .59102 .28155 .57675 Metetra
.673 .67 .839 r
.28155 .57675 .29128 .59102 .30697 .58456 .29727 .57029 Metetra
.756 .698 .797 r
.29727 .57029 .30697 .58456 .32313 .57145 .31346 .55748 Metetra
.795 .712 .773 r
.31346 .55748 .32313 .57145 .33955 .55387 .32993 .5404 Metetra
.815 .721 .762 r
.32993 .5404 .33955 .55387 .35607 .53384 .3465 .52096 Metetra
.824 .73 .761 r
.3465 .52096 .35607 .53384 .37258 .51303 .36306 .50077 Metetra
.827 .739 .767 r
.36306 .50077 .37258 .51303 .38902 .49269 .37956 .48102 Metetra
.824 .748 .78 r
.37956 .48102 .38902 .49269 .40535 .47368 .39596 .46255 Metetra
.816 .756 .797 r
.39596 .46255 .40535 .47368 .42159 .45656 .41226 .44588 Metetra
.801 .763 .818 r
.41226 .44588 .42159 .45656 .43774 .44164 .42849 .4313 Metetra
.78 .767 .842 r
.42849 .4313 .43774 .44164 .45384 .42906 .44466 .41897 Metetra
.75 .768 .868 r
.44466 .41897 .45384 .42906 .46993 .41888 .46081 .40894 Metetra
.71 .763 .894 r
.46081 .40894 .46993 .41888 .48603 .41112 .47698 .40122 Metetra
.657 .75 .918 r
.47698 .40122 .48603 .41112 .50218 .40578 .49319 .39582 Metetra
.59 .725 .938 r
.49319 .39582 .50218 .40578 .51842 .40286 .50949 .39274 Metetra
.507 .687 .948 r
.50949 .39274 .51842 .40286 .5348 .40239 .52592 .392 Metetra
.412 .635 .947 r
.52592 .392 .5348 .40239 .55134 .40436 .5425 .39361 Metetra
.31 .571 .932 r
.5425 .39361 .55134 .40436 .56809 .40875 .55928 .39752 Metetra
.215 .503 .906 r
.55928 .39752 .56809 .40875 .58509 .41542 .5763 .40362 Metetra
.14 .443 .878 r
.5763 .40362 .58509 .41542 .60236 .42408 .59359 .41161 Metetra
.101 .403 .856 r
.59359 .41161 .60236 .42408 .61993 .43418 .61116 .42097 Metetra
.109 .394 .849 r
.61116 .42097 .61993 .43418 .6378 .44486 .62901 .43088 Metetra
.173 .422 .858 r
.62901 .43088 .6378 .44486 .65592 .45482 .64713 .44011 Metetra
.292 .484 .875 r
.64713 .44011 .65592 .45482 .67421 .46235 .66541 .44699 Metetra
.45 .567 .881 r
.66541 .44699 .67421 .46235 .69253 .46534 .68374 .44953 Metetra
.612 .646 .859 r
.68374 .44953 .69253 .46534 .71067 .46148 .70192 .44552 Metetra
.745 .703 .811 r
.70192 .44552 .71067 .46148 .72839 .44862 .71972 .4329 Metetra
.836 .736 .753 r
.71972 .4329 .72839 .44862 .74543 .42531 .73691 .4103 Metetra
.894 .754 .699 r
.73691 .4103 .74543 .42531 .76161 .39138 .7533 .37752 Metetra
.93 .766 .657 r
.7533 .37752 .76161 .39138 .77687 .34842 .76882 .33607 Metetra
.951 .777 .63 r
.76882 .33607 .77687 .34842 .79134 .30004 .78361 .2894 Metetra
.963 .792 .623 r
.78361 .2894 .79134 .30004 .80543 .25162 .79802 .24265 Metetra
.966 .817 .646 r
.79802 .24265 .80543 .25162 .81976 .20972 .81265 .2021 Metetra
.879 .458 .199 r
.18408 .41269 .19442 .41741 .2054 .4454 .19518 .43892 Metetra
.904 .502 .228 r
.19518 .43892 .2054 .4454 .21617 .47941 .20608 .47082 Metetra
.906 .497 .181 r
.20608 .47082 .21617 .47941 .22735 .51346 .21738 .5027 Metetra
.868 .431 .034 r
.21738 .5027 .22735 .51346 .23937 .54262 .22949 .52993 Metetra
.66 .2 0 r
.22949 .52993 .23937 .54262 .25242 .56361 .24262 .54941 Metetra
0 .291 .79 r
.24262 .54941 .25242 .56361 .26653 .57495 .25678 .55977 Metetra
.479 .572 .871 r
.25678 .55977 .26653 .57495 .28155 .57675 .27183 .56113 Metetra
.667 .651 .825 r
.27183 .56113 .28155 .57675 .29727 .57029 .28758 .5547 Metetra
.744 .68 .789 r
.28758 .5547 .29727 .57029 .31346 .55748 .30379 .54228 Metetra
.782 .696 .769 r
.30379 .54228 .31346 .55748 .32993 .5404 .32028 .52582 Metetra
.802 .708 .761 r
.32028 .52582 .32993 .5404 .3465 .52096 .33689 .50713 Metetra
.813 .719 .761 r
.33689 .50713 .3465 .52096 .36306 .50077 .35349 .48773 Metetra
.817 .73 .769 r
.35349 .48773 .36306 .50077 .37956 .48102 .37004 .46873 Metetra
.815 .74 .781 r
.37004 .46873 .37956 .48102 .39596 .46255 .38649 .45093 Metetra
.808 .75 .799 r
.38649 .45093 .39596 .46255 .41226 .44588 .40286 .43483 Metetra
.794 .758 .82 r
.40286 .43483 .41226 .44588 .42849 .4313 .41915 .4207 Metetra
.774 .764 .844 r
.41915 .4207 .42849 .4313 .44466 .41897 .43538 .40869 Metetra
.745 .766 .869 r
.43538 .40869 .44466 .41897 .46081 .40894 .4516 .39885 Metetra
.707 .762 .895 r
.4516 .39885 .46081 .40894 .47698 .40122 .46783 .3912 Metetra
.657 .75 .918 r
.46783 .3912 .47698 .40122 .49319 .39582 .48411 .38574 Metetra
.594 .727 .937 r
.48411 .38574 .49319 .39582 .50949 .39274 .50047 .38248 Metetra
.518 .691 .947 r
.50047 .38248 .50949 .39274 .52592 .392 .51694 .38142 Metetra
.431 .643 .946 r
.51694 .38142 .52592 .392 .5425 .39361 .53357 .38258 Metetra
.341 .585 .934 r
.53357 .38258 .5425 .39361 .55928 .39752 .55038 .38592 Metetra
.257 .523 .912 r
.55038 .38592 .55928 .39752 .5763 .40362 .56742 .39131 Metetra
.194 .469 .888 r
.56742 .39131 .5763 .40362 .59359 .41161 .58471 .39848 Metetra
.163 .433 .868 r
.58471 .39848 .59359 .41161 .61116 .42097 .60228 .40695 Metetra
.174 .423 .859 r
.60228 .40695 .61116 .42097 .62901 .43088 .62013 .41592 Metetra
.233 .444 .861 r
.62013 .41592 .62901 .43088 .64713 .44011 .63822 .42423 Metetra
.338 .494 .867 r
.63822 .42423 .64713 .44011 .66541 .44699 .65649 .43033 Metetra
.474 .562 .864 r
.65649 .43033 .66541 .44699 .68374 .44953 .67482 .43233 Metetra
.613 .629 .842 r
.67482 .43233 .68374 .44953 .70192 .44552 .69303 .42814 Metetra
.733 .682 .802 r
.69303 .42814 .70192 .44552 .71972 .4329 .71092 .41585 Metetra
.821 .719 .752 r
.71092 .41585 .71972 .4329 .73691 .4103 .72825 .39413 Metetra
.881 .743 .705 r
.72825 .39413 .73691 .4103 .7533 .37752 .74485 .3628 Metetra
.92 .76 .666 r
.74485 .3628 .7533 .37752 .76882 .33607 .76067 .32326 Metetra
.945 .776 .64 r
.76067 .32326 .76882 .33607 .78361 .2894 .77579 .27872 Metetra
.96 .795 .633 r
.77579 .27872 .78361 .2894 .79802 .24265 .79057 .23407 Metetra
.965 .826 .656 r
.79057 .23407 .79802 .24265 .81265 .2021 .80554 .19525 Metetra
.865 .433 .182 r
.17351 .40869 .18408 .41269 .19518 .43892 .18479 .43283 Metetra
.894 .481 .206 r
.18479 .43283 .19518 .43892 .20608 .47082 .19587 .46222 Metetra
.895 .473 .145 r
.19587 .46222 .20608 .47082 .21738 .5027 .20734 .49154 Metetra
.84 .39 0 r
.20734 .49154 .21738 .5027 .22949 .52993 .21959 .51648 Metetra
0 0 .431 r
.21959 .51648 .22949 .52993 .24262 .54941 .23282 .53418 Metetra
.09 .347 .818 r
.23282 .53418 .24262 .54941 .25678 .55977 .24704 .54339 Metetra
.501 .566 .854 r
.24704 .54339 .25678 .55977 .27183 .56113 .26213 .54424 Metetra
.661 .635 .813 r
.26213 .54424 .27183 .56113 .28758 .5547 .2779 .53785 Metetra
.732 .664 .783 r
.2779 .53785 .28758 .5547 .30379 .54228 .29413 .52589 Metetra
.769 .682 .766 r
.29413 .52589 .30379 .54228 .32028 .52582 .31063 .51018 Metetra
.79 .695 .76 r
.31063 .51018 .32028 .52582 .33689 .50713 .32724 .4924 Metetra
.801 .708 .762 r
.32724 .4924 .33689 .50713 .35349 .48773 .34387 .47393 Metetra
.806 .72 .77 r
.34387 .47393 .35349 .48773 .37004 .46873 .36045 .45584 Metetra
.805 .732 .784 r
.36045 .45584 .37004 .46873 .38649 .45093 .37695 .43885 Metetra
.798 .743 .802 r
.37695 .43885 .38649 .45093 .40286 .43483 .39336 .42343 Metetra
.786 .753 .823 r
.39336 .42343 .40286 .43483 .41915 .4207 .40971 .40984 Metetra
.767 .76 .847 r
.40971 .40984 .41915 .4207 .43538 .40869 .42601 .39823 Metetra
.74 .763 .872 r
.42601 .39823 .43538 .40869 .4516 .39885 .44229 .38863 Metetra
.704 .76 .896 r
.44229 .38863 .4516 .39885 .46783 .3912 .45859 .38107 Metetra
.657 .75 .918 r
.45859 .38107 .46783 .3912 .48411 .38574 .47493 .37555 Metetra
.599 .729 .935 r
.47493 .37555 .48411 .38574 .50047 .38248 .49134 .37207 Metetra
.53 .696 .945 r
.49134 .37207 .50047 .38248 .51694 .38142 .50787 .37065 Metetra
.453 .652 .945 r
.50787 .37065 .51694 .38142 .53357 .38258 .52454 .37129 Metetra
.374 .6 .935 r
.52454 .37129 .53357 .38258 .55038 .38592 .54139 .37395 Metetra
.302 .544 .917 r
.54139 .37395 .55038 .38592 .56742 .39131 .55845 .37852 Metetra
.249 .496 .895 r
.55845 .37852 .56742 .39131 .58471 .39848 .57575 .38474 Metetra
.225 .462 .877 r
.57575 .38474 .58471 .39848 .60228 .40695 .59331 .39215 Metetra
.238 .451 .865 r
.59331 .39215 .60228 .40695 .62013 .41592 .61114 .40002 Metetra
.291 .466 .861 r
.61114 .40002 .62013 .41592 .63822 .42423 .62921 .40726 Metetra
.38 .504 .858 r
.62921 .40726 .63822 .42423 .65649 .43033 .64746 .41244 Metetra
.495 .558 .849 r
.64746 .41244 .65649 .43033 .67482 .43233 .66578 .4138 Metetra
.614 .614 .827 r
.66578 .4138 .67482 .43233 .69303 .42814 .68402 .40943 Metetra
.721 .664 .793 r
.68402 .40943 .69303 .42814 .71092 .41585 .70198 .39753 Metetra
.805 .702 .752 r
.70198 .39753 .71092 .41585 .72825 .39413 .71945 .37687 Metetra
.866 .731 .711 r
.71945 .37687 .72825 .39413 .74485 .3628 .73628 .34726 Metetra
.909 .753 .676 r
.73628 .34726 .74485 .3628 .76067 .32326 .7524 .30999 Metetra
.938 .775 .652 r
.7524 .30999 .76067 .32326 .77579 .27872 .76789 .26801 Metetra
.957 .8 .646 r
.76789 .26801 .77579 .27872 .79057 .23407 .78305 .22587 Metetra
.964 .836 .669 r
.78305 .22587 .79057 .23407 .80554 .19525 .79839 .18912 Metetra
.846 .4 .155 r
.16272 .40537 .17351 .40869 .18479 .43283 .17422 .4271 Metetra
.88 .452 .173 r
.17422 .4271 .18479 .43283 .19587 .46222 .18554 .4536 Metetra
.877 .438 .094 r
.18554 .4536 .19587 .46222 .20734 .49154 .19723 .47999 Metetra
.792 .33 0 r
.19723 .47999 .20734 .49154 .21959 .51648 .20966 .50232 Metetra
0 0 .553 r
.20966 .50232 .21959 .51648 .23282 .53418 .22303 .51799 Metetra
.193 .396 .833 r
.22303 .51799 .23282 .53418 .24704 .54339 .23733 .5259 Metetra
.52 .561 .838 r
.23733 .5259 .24704 .54339 .26213 .54424 .25246 .52616 Metetra
.655 .62 .802 r
.25246 .52616 .26213 .54424 .2779 .53785 .26824 .51983 Metetra
.72 .649 .777 r
.26824 .51983 .2779 .53785 .29413 .52589 .28446 .50841 Metetra
.756 .668 .764 r
.28446 .50841 .29413 .52589 .31063 .51018 .30095 .49356 Metetra
.776 .683 .761 r
.30095 .49356 .31063 .51018 .32724 .4924 .31757 .47681 Metetra
.788 .697 .764 r
.31757 .47681 .32724 .4924 .34387 .47393 .3342 .45943 Metetra
.794 .711 .773 r
.3342 .45943 .34387 .47393 .36045 .45584 .3508 .44237 Metetra
.794 .724 .787 r
.3508 .44237 .36045 .45584 .37695 .43885 .36732 .42632 Metetra
.788 .737 .805 r
.36732 .42632 .37695 .43885 .39336 .42343 .38378 .41169 Metetra
.777 .748 .827 r
.38378 .41169 .39336 .42343 .40971 .40984 .40018 .39874 Metetra
.759 .756 .85 r
.40018 .39874 .40971 .40984 .42601 .39823 .41653 .38759 Metetra
.734 .76 .874 r
.41653 .38759 .42601 .39823 .44229 .38863 .43288 .37828 Metetra
.7 .759 .897 r
.43288 .37828 .44229 .38863 .45859 .38107 .44924 .37083 Metetra
.657 .749 .918 r
.44924 .37083 .45859 .38107 .47493 .37555 .46564 .36524 Metetra
.604 .731 .934 r
.46564 .36524 .47493 .37555 .49134 .37207 .48211 .36153 Metetra
.543 .701 .943 r
.48211 .36153 .49134 .37207 .50787 .37065 .4987 .3597 Metetra
.476 .662 .943 r
.4987 .3597 .50787 .37065 .52454 .37129 .51541 .35974 Metetra
.408 .615 .935 r
.51541 .35974 .52454 .37129 .54139 .37395 .5323 .36164 Metetra
.348 .566 .919 r
.5323 .36164 .54139 .37395 .55845 .37852 .54938 .36527 Metetra
.305 .522 .9 r
.54938 .36527 .55845 .37852 .57575 .38474 .56669 .3704 Metetra
.286 .491 .882 r
.56669 .3704 .57575 .38474 .59331 .39215 .58425 .37662 Metetra
.299 .478 .868 r
.58425 .37662 .59331 .39215 .61114 .40002 .60206 .38323 Metetra
.344 .486 .858 r
.60206 .38323 .61114 .40002 .62921 .40726 .6201 .38927 Metetra
.419 .513 .848 r
.6201 .38927 .62921 .40726 .64746 .41244 .63832 .39341 Metetra
.514 .555 .835 r
.63832 .39341 .64746 .41244 .66578 .4138 .65663 .39406 Metetra
.614 .602 .815 r
.65663 .39406 .66578 .4138 .68402 .40943 .67488 .38947 Metetra
.709 .647 .786 r
.67488 .38947 .68402 .40943 .70198 .39753 .69292 .37802 Metetra
.788 .686 .752 r
.69292 .37802 .70198 .39753 .71945 .37687 .71054 .35858 Metetra
.85 .718 .718 r
.71054 .35858 .71945 .37687 .73628 .34726 .7276 .33095 Metetra
.896 .746 .688 r
.7276 .33095 .73628 .34726 .7524 .30999 .74403 .29628 Metetra
.929 .773 .667 r
.74403 .29628 .7524 .30999 .76789 .26801 .7599 .25726 Metetra
.951 .805 .662 r
.7599 .25726 .76789 .26801 .78305 .22587 .77547 .21801 Metetra
.96 .848 .686 r
.77547 .21801 .78305 .22587 .79839 .18912 .7912 .18366 Metetra
.819 .355 .116 r
.15169 .40268 .16272 .40537 .17422 .4271 .16347 .4217 Metetra
.858 .411 .125 r
.16347 .4217 .17422 .4271 .18554 .4536 .17509 .44497 Metetra
.847 .387 .02 r
.17509 .44497 .18554 .4536 .19723 .47999 .18706 .46808 Metetra
.712 .24 0 r
.18706 .46808 .19723 .47999 .20966 .50232 .19972 .48749 Metetra
0 .124 .669 r
.19972 .48749 .20966 .50232 .22303 .51799 .21324 .50092 Metetra
.283 .437 .839 r
.21324 .50092 .22303 .51799 .23733 .5259 .22764 .50737 Metetra
.537 .558 .824 r
.22764 .50737 .23733 .5259 .25246 .52616 .24282 .507 Metetra
.65 .608 .793 r
.24282 .507 .25246 .52616 .26824 .51983 .2586 .50073 Metetra
.708 .635 .773 r
.2586 .50073 .26824 .51983 .28446 .50841 .27481 .48991 Metetra
.742 .654 .763 r
.27481 .48991 .28446 .50841 .30095 .49356 .29127 .47603 Metetra
.762 .671 .762 r
.29127 .47603 .30095 .49356 .31757 .47681 .30786 .46044 Metetra
.774 .686 .767 r
.30786 .46044 .31757 .47681 .3342 .45943 .32449 .44426 Metetra
.78 .702 .777 r
.32449 .44426 .3342 .45943 .3508 .44237 .34108 .42837 Metetra
.781 .716 .792 r
.34108 .42837 .3508 .44237 .36732 .42632 .35763 .41337 Metetra
.777 .73 .81 r
.35763 .41337 .36732 .42632 .38378 .41169 .37411 .39964 Metetra
.766 .743 .831 r
.37411 .39964 .38378 .41169 .40018 .39874 .39055 .3874 Metetra
.75 .752 .854 r
.39055 .3874 .40018 .39874 .41653 .38759 .40696 .37677 Metetra
.727 .758 .877 r
.40696 .37677 .41653 .38759 .43288 .37828 .42336 .36778 Metetra
.696 .757 .899 r
.42336 .36778 .43288 .37828 .44924 .37083 .43978 .36047 Metetra
.657 .749 .918 r
.43978 .36047 .44924 .37083 .46564 .36524 .45625 .35482 Metetra
.61 .733 .932 r
.45625 .35482 .46564 .36524 .48211 .36153 .47279 .35085 Metetra
.557 .707 .941 r
.47279 .35085 .48211 .36153 .4987 .3597 .48942 .34856 Metetra
.5 .672 .941 r
.48942 .34856 .4987 .3597 .51541 .35974 .50619 .34795 Metetra
.444 .631 .934 r
.50619 .34795 .51541 .35974 .5323 .36164 .52311 .34899 Metetra
.394 .587 .92 r
.52311 .34899 .5323 .36164 .54938 .36527 .54022 .35159 Metetra
.36 .548 .903 r
.54022 .35159 .54938 .36527 .56669 .3704 .55755 .35551 Metetra
.345 .518 .884 r
.55755 .35551 .56669 .3704 .58425 .37662 .5751 .3604 Metetra
.356 .503 .867 r
.5751 .3604 .58425 .37662 .60206 .38323 .59289 .36562 Metetra
.393 .505 .853 r
.59289 .36562 .60206 .38323 .6201 .38927 .6109 .37032 Metetra
.454 .523 .839 r
.6109 .37032 .6201 .38927 .63832 .39341 .62909 .37331 Metetra
.531 .553 .823 r
.62909 .37331 .63832 .39341 .65663 .39406 .64738 .37318 Metetra
.615 .592 .804 r
.64738 .37318 .65663 .39406 .67488 .38947 .66565 .36837 Metetra
.697 .632 .78 r
.66565 .36837 .67488 .38947 .69292 .37802 .68375 .35743 Metetra
.77 .67 .752 r
.68375 .35743 .69292 .37802 .71054 .35858 .70151 .33934 Metetra
.832 .706 .725 r
.70151 .33934 .71054 .35858 .7276 .33095 .7188 .31392 Metetra
.881 .739 .701 r
.7188 .31392 .7276 .33095 .74403 .29628 .73555 .28217 Metetra
.918 .772 .684 r
.73555 .28217 .74403 .29628 .7599 .25726 .75181 .24645 Metetra
.944 .81 .682 r
.75181 .24645 .7599 .25726 .77547 .21801 .76782 .21046 Metetra
.955 .86 .708 r
.76782 .21046 .77547 .21801 .7912 .18366 .78396 .17883 Metetra
.781 .292 .058 r
.14043 .40056 .15169 .40268 .16347 .4217 .15255 .41659 Metetra
.825 .35 .054 r
.15255 .41659 .16347 .4217 .17509 .44497 .16452 .43631 Metetra
.795 .307 0 r
.16452 .43631 .17509 .44497 .18706 .46808 .17681 .45582 Metetra
.578 .106 0 r
.17681 .45582 .18706 .46808 .19972 .48749 .18974 .47205 Metetra
0 .251 .763 r
.18974 .47205 .19972 .48749 .21324 .50092 .20346 .48303 Metetra
.36 .471 .837 r
.20346 .48303 .21324 .50092 .22764 .50737 .21798 .48792 Metetra
.553 .557 .812 r
.21798 .48792 .22764 .50737 .24282 .507 .2332 .48686 Metetra
.645 .597 .786 r
.2332 .48686 .24282 .507 .2586 .50073 .24898 .48066 Metetra
.696 .622 .77 r
.24898 .48066 .2586 .50073 .27481 .48991 .26516 .47051 Metetra
.727 .641 .763 r
.26516 .47051 .27481 .48991 .29127 .47603 .28159 .45768 Metetra
.747 .659 .763 r
.28159 .45768 .29127 .47603 .30786 .46044 .29814 .44334 Metetra
.759 .676 .77 r
.29814 .44334 .30786 .46044 .32449 .44426 .31473 .4285 Metetra
.766 .692 .781 r
.31473 .4285 .32449 .44426 .34108 .42837 .33131 .41388 Metetra
.767 .708 .797 r
.33131 .41388 .34108 .42837 .35763 .41337 .34786 .40003 Metetra
.763 .723 .815 r
.34786 .40003 .35763 .41337 .37411 .39964 .36436 .38728 Metetra
.755 .737 .836 r
.36436 .38728 .37411 .39964 .39055 .3874 .38083 .37583 Metetra
.74 .748 .859 r
.38083 .37583 .39055 .3874 .40696 .37677 .39728 .36577 Metetra
.719 .755 .881 r
.39728 .36577 .40696 .37677 .42336 .36778 .41374 .35715 Metetra
.691 .756 .901 r
.41374 .35715 .42336 .36778 .43978 .36047 .43022 .34999 Metetra
.657 .749 .918 r
.43022 .34999 .43978 .36047 .45625 .35482 .44675 .34428 Metetra
.617 .735 .931 r
.44675 .34428 .45625 .35482 .47279 .35085 .46335 .34002 Metetra
.572 .713 .938 r
.46335 .34002 .47279 .35085 .48942 .34856 .48005 .33724 Metetra
.525 .682 .938 r
.48005 .33724 .48942 .34856 .50619 .34795 .49687 .33592 Metetra
.479 .646 .931 r
.49687 .33592 .50619 .34795 .52311 .34899 .51384 .33604 Metetra
.44 .608 .919 r
.51384 .33604 .52311 .34899 .54022 .35159 .53098 .3375 Metetra
.412 .572 .902 r
.53098 .3375 .54022 .35159 .55755 .35551 .54832 .34011 Metetra
.401 .543 .883 r
.54832 .34011 .55755 .35551 .5751 .3604 .56587 .34355 Metetra
.41 .526 .864 r
.56587 .34355 .5751 .3604 .59289 .36562 .58364 .34727 Metetra
.439 .522 .846 r
.58364 .34727 .59289 .36562 .6109 .37032 .60162 .35052 Metetra
.486 .532 .829 r
.60162 .35052 .6109 .37032 .62909 .37331 .61978 .35227 Metetra
.547 .553 .812 r
.61978 .35227 .62909 .37331 .64738 .37318 .63805 .35129 Metetra
.616 .583 .794 r
.63805 .35129 .64738 .37318 .66565 .36837 .65633 .34624 Metetra
.686 .618 .775 r
.65633 .34624 .66565 .36837 .68375 .35743 .67448 .33586 Metetra
.752 .655 .754 r
.67448 .33586 .68375 .35743 .70151 .33934 .69238 .31925 Metetra
.811 .692 .733 r
.69238 .31925 .70151 .33934 .7188 .31392 .7099 .29624 Metetra
.862 .73 .716 r
.7099 .29624 .7188 .31392 .73555 .28217 .72697 .26767 Metetra
.903 .77 .704 r
.72697 .26767 .73555 .28217 .75181 .24645 .74363 .23558 Metetra
.933 .815 .706 r
.74363 .23558 .75181 .24645 .76782 .21046 .76008 .20318 Metetra
.945 .874 .735 r
.76008 .20318 .76782 .21046 .78396 .17883 .77665 .17455 Metetra
.721 .201 0 r
.12894 .39894 .14043 .40056 .15255 .41659 .14143 .41174 Metetra
.766 .255 0 r
.14143 .41174 .15255 .41659 .16452 .43631 .15382 .4276 Metetra
.695 .178 0 r
.15382 .4276 .16452 .43631 .17681 .45582 .16649 .44325 Metetra
0 .078 .61 r
.16649 .44325 .17681 .45582 .18974 .47205 .17974 .45607 Metetra
.13 .364 .824 r
.17974 .45607 .18974 .47205 .20346 .48303 .19368 .46444 Metetra
.426 .499 .831 r
.19368 .46444 .20346 .48303 .21798 .48792 .20833 .46766 Metetra
.567 .556 .802 r
.20833 .46766 .21798 .48792 .2332 .48686 .22361 .46587 Metetra
.641 .588 .779 r
.22361 .46587 .2332 .48686 .24898 .48066 .23939 .45975 Metetra
.684 .61 .767 r
.23939 .45975 .24898 .48066 .26516 .47051 .25552 .4503 Metetra
.712 .629 .763 r
.25552 .4503 .26516 .47051 .28159 .45768 .27189 .4386 Metetra
.73 .647 .766 r
.27189 .4386 .28159 .45768 .29814 .44334 .28838 .42561 Metetra
.742 .665 .774 r
.28838 .42561 .29814 .44334 .31473 .4285 .30492 .41219 Metetra
.749 .682 .786 r
.30492 .41219 .31473 .4285 .33131 .41388 .32147 .39895 Metetra
.751 .7 .803 r
.32147 .39895 .33131 .41388 .34786 .40003 .33801 .38634 Metetra
.749 .716 .822 r
.33801 .38634 .34786 .40003 .36436 .38728 .35452 .37465 Metetra
.741 .731 .842 r
.35452 .37465 .36436 .38728 .38083 .37583 .37101 .36404 Metetra
.729 .744 .864 r
.37101 .36404 .38083 .37583 .39728 .36577 .38751 .35461 Metetra
.71 .751 .884 r
.38751 .35461 .39728 .36577 .41374 .35715 .40401 .34639 Metetra
.686 .754 .903 r
.40401 .34639 .41374 .35715 .43022 .34999 .42055 .33939 Metetra
.657 .749 .918 r
.42055 .33939 .43022 .34999 .44675 .34428 .43715 .33361 Metetra
.623 .738 .929 r
.43715 .33361 .44675 .34428 .46335 .34002 .45381 .32906 Metetra
.587 .718 .934 r
.45381 .32906 .46335 .34002 .48005 .33724 .47057 .32575 Metetra
.549 .692 .934 r
.47057 .32575 .48005 .33724 .49687 .33592 .48745 .32367 Metetra
.513 .661 .927 r
.48745 .32367 .49687 .33592 .51384 .33604 .50447 .32279 Metetra
.483 .627 .915 r
.50447 .32279 .51384 .33604 .53098 .3375 .52165 .32304 Metetra
.462 .594 .899 r
.52165 .32304 .53098 .3375 .54832 .34011 .539 .32425 Metetra
.453 .567 .879 r
.539 .32425 .54832 .34011 .56587 .34355 .55656 .32614 Metetra
.459 .547 .859 r
.55656 .32614 .56587 .34355 .58364 .34727 .57432 .32826 Metetra
.48 .538 .839 r
.57432 .32826 .58364 .34727 .60162 .35052 .59228 .32996 Metetra
.515 .541 .821 r
.59228 .32996 .60162 .35052 .61978 .35227 .6104 .33039 Metetra
.562 .554 .803 r
.6104 .33039 .61978 .35227 .63805 .35129 .62865 .32851 Metetra
.616 .577 .787 r
.62865 .32851 .63805 .35129 .65633 .34624 .64693 .32321 Metetra
.675 .606 .771 r
.64693 .32321 .65633 .34624 .67448 .33586 .66514 .31343 Metetra
.733 .64 .756 r
.66514 .31343 .67448 .33586 .69238 .31925 .68316 .2984 Metetra
.789 .678 .742 r
.68316 .2984 .69238 .31925 .7099 .29624 .7009 .27797 Metetra
.84 .72 .732 r
.7009 .27797 .7099 .29624 .72697 .26767 .71828 .25281 Metetra
.884 .766 .728 r
.71828 .25281 .72697 .26767 .74363 .23558 .73536 .22463 Metetra
.917 .821 .736 r
.73536 .22463 .74363 .23558 .76008 .20318 .75226 .19613 Metetra
.931 .889 .768 r
.75226 .19613 .76008 .20318 .77665 .17455 .76926 .17076 Metetra
.621 .063 0 r
.11721 .39776 .12894 .39894 .14143 .41174 .13014 .40708 Metetra
.655 .099 0 r
.13014 .40708 .14143 .41174 .15382 .4276 .14299 .41885 Metetra
.501 0 0 r
.14299 .41885 .15382 .4276 .16649 .44325 .1561 .43039 Metetra
0 .287 .785 r
.1561 .43039 .16649 .44325 .17974 .45607 .16971 .43962 Metetra
.289 .455 .852 r
.16971 .43962 .17974 .45607 .19368 .46444 .1839 .44523 Metetra
.482 .523 .823 r
.1839 .44523 .19368 .46444 .20833 .46766 .1987 .4467 Metetra
.58 .557 .793 r
.1987 .4467 .20833 .46766 .22361 .46587 .21404 .44414 Metetra
.636 .58 .774 r
.21404 .44414 .22361 .46587 .23939 .45975 .2298 .43812 Metetra
.672 .599 .765 r
.2298 .43812 .23939 .45975 .25552 .4503 .24588 .42941 Metetra
.696 .617 .764 r
.24588 .42941 .25552 .4503 .27189 .4386 .26218 .41889 Metetra
.713 .635 .769 r
.26218 .41889 .27189 .4386 .28838 .42561 .27859 .40733 Metetra
.724 .653 .779 r
.27859 .40733 .28838 .42561 .30492 .41219 .29507 .39541 Metetra
.731 .672 .792 r
.29507 .39541 .30492 .41219 .32147 .39895 .31157 .38362 Metetra
.734 .691 .809 r
.31157 .38362 .32147 .39895 .33801 .38634 .32808 .37232 Metetra
.733 .709 .829 r
.32808 .37232 .33801 .38634 .35452 .37465 .34459 .36175 Metetra
.727 .725 .849 r
.34459 .36175 .35452 .37465 .37101 .36404 .3611 .35205 Metetra
.716 .739 .869 r
.3611 .35205 .37101 .36404 .38751 .35461 .37762 .34329 Metetra
.7 .748 .888 r
.37762 .34329 .38751 .35461 .40401 .34639 .39418 .33549 Metetra
.681 .752 .905 r
.39418 .33549 .40401 .34639 .42055 .33939 .41077 .32866 Metetra
.657 .749 .918 r
.41077 .32866 .42055 .33939 .43715 .33361 .42743 .32282 Metetra
.63 .74 .927 r
.42743 .32282 .43715 .33361 .45381 .32906 .44417 .31796 Metetra
.602 .724 .931 r
.44417 .31796 .45381 .32906 .47057 .32575 .461 .31409 Metetra
.573 .702 .929 r
.461 .31409 .47057 .32575 .48745 .32367 .47794 .3112 Metetra
.546 .675 .922 r
.47794 .3112 .48745 .32367 .50447 .32279 .49501 .30927 Metetra
.524 .645 .91 r
.49501 .30927 .50447 .32279 .52165 .32304 .51223 .30824 Metetra
.508 .615 .893 r
.51223 .30824 .52165 .32304 .539 .32425 .52961 .30797 Metetra
.501 .588 .874 r
.52961 .30797 .539 .32425 .55656 .32614 .54718 .30823 Metetra
.504 .567 .853 r
.54718 .30823 .55656 .32614 .57432 .32826 .56493 .30866 Metetra
.518 .554 .832 r
.56493 .30866 .57432 .32826 .59228 .32996 .58286 .30874 Metetra
.543 .55 .812 r
.58286 .30874 .59228 .32996 .6104 .33039 .60096 .30779 Metetra
.576 .556 .795 r
.60096 .30779 .6104 .33039 .62865 .32851 .61918 .30498 Metetra
.617 .571 .78 r
.61918 .30498 .62865 .32851 .64693 .32321 .63746 .29942 Metetra
.664 .595 .768 r
.63746 .29942 .64693 .32321 .66514 .31343 .65572 .29026 Metetra
.713 .625 .758 r
.65572 .29026 .66514 .31343 .68316 .2984 .67386 .27689 Metetra
.763 .663 .752 r
.67386 .27689 .68316 .2984 .7009 .27797 .69181 .25917 Metetra
.813 .708 .749 r
.69181 .25917 .7009 .27797 .71828 .25281 .70951 .23762 Metetra
.858 .761 .754 r
.70951 .23762 .71828 .25281 .73536 .22463 .72698 .21359 Metetra
.895 .825 .77 r
.72698 .21359 .73536 .22463 .75226 .19613 .74433 .18924 Metetra
.908 .903 .808 r
.74433 .18924 .75226 .19613 .76926 .17076 .76178 .16738 Metetra
.447 0 0 r
.10524 .39693 .11721 .39776 .13014 .40708 .11866 .40258 Metetra
.431 0 0 r
.11866 .40258 .13014 .40708 .14299 .41885 .13203 .41004 Metetra
0 .323 .77 r
.13203 .41004 .14299 .41885 .1561 .43039 .14564 .41728 Metetra
.197 .467 .886 r
.14564 .41728 .1561 .43039 .16971 .43962 .15964 .42276 Metetra
.417 .521 .857 r
.15964 .42276 .16971 .43962 .1839 .44523 .17412 .42552 Metetra
.53 .544 .814 r
.17412 .42552 .1839 .44523 .1987 .4467 .18908 .42517 Metetra
.593 .559 .786 r
.18908 .42517 .1987 .4467 .21404 .44414 .20448 .42182 Metetra
.632 .574 .771 r
.20448 .42182 .21404 .44414 .2298 .43812 .22023 .41589 Metetra
.66 .589 .765 r
.22023 .41589 .2298 .43812 .24588 .42941 .23625 .40796 Metetra
.679 .605 .766 r
.23625 .40796 .24588 .42941 .26218 .41889 .25245 .39866 Metetra
.694 .623 .773 r
.25245 .39866 .26218 .41889 .27859 .40733 .26877 .38858 Metetra
.704 .642 .784 r
.26877 .38858 .27859 .40733 .29507 .39541 .28517 .37822 Metetra
.711 .662 .799 r
.28517 .37822 .29507 .39541 .31157 .38362 .30161 .36794 Metetra
.714 .682 .817 r
.30161 .36794 .31157 .38362 .32808 .37232 .31808 .35802 Metetra
.714 .701 .836 r
.31808 .35802 .32808 .37232 .34459 .36175 .33457 .34863 Metetra
.71 .719 .856 r
.33457 .34863 .34459 .36175 .3611 .35205 .35108 .33987 Metetra
.702 .734 .875 r
.35108 .33987 .3611 .35205 .37762 .34329 .36764 .3318 Metetra
.69 .744 .893 r
.36764 .3318 .37762 .34329 .39418 .33549 .38423 .32445 Metetra
.675 .75 .907 r
.38423 .32445 .39418 .33549 .41077 .32866 .40088 .31782 Metetra
.657 .749 .918 r
.40088 .31782 .41077 .32866 .42743 .32282 .41761 .31191 Metetra
.637 .743 .925 r
.41761 .31191 .42743 .32282 .44417 .31796 .43441 .30672 Metetra
.617 .73 .927 r
.43441 .30672 .44417 .31796 .461 .31409 .45131 .30227 Metetra
.597 .711 .924 r
.45131 .30227 .461 .31409 .47794 .3112 .46832 .29854 Metetra
.578 .687 .916 r
.46832 .29854 .47794 .3112 .49501 .30927 .48545 .29551 Metetra
.562 .661 .903 r
.48545 .29551 .49501 .30927 .51223 .30824 .50272 .29314 Metetra
.55 .633 .886 r
.50272 .29314 .51223 .30824 .52961 .30797 .52014 .29133 Metetra
.544 .607 .866 r
.52014 .29133 .52961 .30797 .54718 .30823 .53772 .2899 Metetra
.545 .585 .845 r
.53772 .2899 .54718 .30823 .56493 .30866 .55547 .28858 Metetra
.553 .568 .824 r
.55547 .28858 .56493 .30866 .58286 .30874 .57339 .28698 Metetra
.568 .56 .805 r
.57339 .28698 .58286 .30874 .60096 .30779 .59146 .2846 Metetra
.59 .559 .788 r
.59146 .2846 .60096 .30779 .61918 .30498 .60965 .28083 Metetra
.619 .568 .775 r
.60965 .28083 .61918 .30498 .63746 .29942 .62793 .275 Metetra
.653 .585 .766 r
.62793 .275 .63746 .29942 .65572 .29026 .64623 .26649 Metetra
.692 .611 .761 r
.64623 .26649 .65572 .29026 .67386 .27689 .66448 .25484 Metetra
.734 .646 .761 r
.66448 .25484 .67386 .27689 .69181 .25917 .68263 .23993 Metetra
.779 .692 .768 r
.68263 .23993 .69181 .25917 .70951 .23762 .70063 .22214 Metetra
.824 .751 .783 r
.70063 .22214 .70951 .23762 .72698 .21359 .71849 .20245 Metetra
.862 .826 .81 r
.71849 .20245 .72698 .21359 .74433 .18924 .7363 .18248 Metetra
.872 .914 .854 r
.7363 .18248 .74433 .18924 .76178 .16738 .7542 .16433 Metetra
.15 0 0 r
.09304 .39636 .10524 .39693 .11866 .40258 .10699 .39818 Metetra
0 .516 .817 r
.10699 .39818 .11866 .40258 .13203 .41004 .12095 .40115 Metetra
.228 .589 .947 r
.12095 .40115 .13203 .41004 .14564 .41728 .13509 .40395 Metetra
.418 .586 .91 r
.13509 .40395 .14564 .41728 .15964 .42276 .14953 .40557 Metetra
.516 .569 .848 r
.14953 .40557 .15964 .42276 .17412 .42552 .16431 .4054 Metetra
.57 .562 .805 r
.16431 .4054 .17412 .42552 .18908 .42517 .17946 .4032 Metetra
.605 .562 .78 r
.17946 .4032 .18908 .42517 .20448 .42182 .19492 .39905 Metetra
.629 .569 .768 r
.19492 .39905 .20448 .42182 .22023 .41589 .21066 .39321 Metetra
.647 .579 .765 r
.21066 .39321 .22023 .41589 .23625 .40796 .2266 .38606 Metetra
.661 .594 .768 r
.2266 .38606 .23625 .40796 .25245 .39866 .2427 .37802 Metetra
.673 .611 .777 r
.2427 .37802 .25245 .39866 .26877 .38858 .25892 .36946 Metetra
.682 .63 .79 r
.25892 .36946 .26877 .38858 .28517 .37822 .27522 .3607 Metetra
.688 .65 .807 r
.27522 .3607 .28517 .37822 .30161 .36794 .29158 .35197 Metetra
.692 .672 .825 r
.29158 .35197 .30161 .36794 .31808 .35802 .30799 .34347 Metetra
.693 .692 .845 r
.30799 .34347 .31808 .35802 .33457 .34863 .32445 .33529 Metetra
.692 .712 .864 r
.32445 .33529 .33457 .34863 .35108 .33987 .34097 .32752 Metetra
.687 .728 .882 r
.34097 .32752 .35108 .33987 .36764 .3318 .35754 .32017 Metetra
.679 .74 .897 r
.35754 .32017 .36764 .3318 .38423 .32445 .37417 .31328 Metetra
.669 .748 .91 r
.37417 .31328 .38423 .32445 .40088 .31782 .39088 .30684 Metetra
.657 .749 .918 r
.39088 .30684 .40088 .31782 .41761 .31191 .40767 .30086 Metetra
.644 .745 .922 r
.40767 .30086 .41761 .31191 .43441 .30672 .42454 .29535 Metetra
.631 .735 .922 r
.42454 .29535 .43441 .30672 .45131 .30227 .44152 .29029 Metetra
.619 .719 .918 r
.44152 .29029 .45131 .30227 .46832 .29854 .4586 .28569 Metetra
.607 .699 .909 r
.4586 .28569 .46832 .29854 .48545 .29551 .4758 .28153 Metetra
.597 .675 .895 r
.4758 .28153 .48545 .29551 .50272 .29314 .49313 .27779 Metetra
.589 .649 .878 r
.49313 .27779 .50272 .29314 .52014 .29133 .51059 .27439 Metetra
.584 .624 .858 r
.51059 .27439 .52014 .29133 .53772 .2899 .5282 .27121 Metetra
.582 .601 .837 r
.5282 .27121 .53772 .2899 .55547 .28858 .54596 .2681 Metetra
.584 .582 .816 r
.54596 .2681 .55547 .28858 .57339 .28698 .56386 .26478 Metetra
.591 .569 .798 r
.56386 .26478 .57339 .28698 .59146 .2846 .58191 .26094 Metetra
.603 .563 .783 r
.58191 .26094 .59146 .2846 .60965 .28083 .60008 .25619 Metetra
.62 .565 .771 r
.60008 .25619 .60965 .28083 .62793 .275 .61836 .25009 Metetra
.642 .576 .765 r
.61836 .25009 .62793 .275 .64623 .26649 .63669 .24224 Metetra
.669 .596 .765 r
.63669 .24224 .64623 .26649 .66448 .25484 .65505 .23235 Metetra
.702 .628 .771 r
.65505 .23235 .66448 .25484 .68263 .23993 .67338 .22032 Metetra
.739 .673 .787 r
.67338 .22032 .68263 .23993 .70063 .22214 .69166 .2064 Metetra
.779 .736 .814 r
.69166 .2064 .70063 .22214 .71849 .20245 .7099 .1912 Metetra
.814 .82 .855 r
.7099 .1912 .71849 .20245 .7363 .18248 .72815 .17578 Metetra
.816 .917 .906 r
.72815 .17578 .7363 .18248 .7542 .16433 .74649 .16152 Metetra
.244 .755 .86 r
.0806 .39596 .09304 .39636 .10699 .39818 .09514 .39383 Metetra
.4 .789 .989 r
.09514 .39383 .10699 .39818 .12095 .40115 .10973 .39218 Metetra
.518 .729 .969 r
.10973 .39218 .12095 .40115 .13509 .40395 .12445 .39043 Metetra
.568 .653 .893 r
.12445 .39043 .13509 .40395 .14953 .40557 .13936 .38813 Metetra
.591 .603 .833 r
.13936 .38813 .14953 .40557 .16431 .4054 .15449 .38499 Metetra
.605 .577 .796 r
.15449 .38499 .16431 .4054 .17946 .4032 .16983 .38092 Metetra
.616 .566 .775 r
.16983 .38092 .17946 .4032 .19492 .39905 .18536 .37595 Metetra
.626 .565 .766 r
.18536 .37595 .19492 .39905 .21066 .39321 .20108 .3702 Metetra
.634 .571 .766 r
.20108 .3702 .21066 .39321 .2266 .38606 .21694 .36385 Metetra
.642 .582 .772 r
.21694 .36385 .2266 .38606 .2427 .37802 .23293 .35708 Metetra
.65 .598 .783 r
.23293 .35708 .2427 .37802 .25892 .36946 .24902 .35005 Metetra
.657 .617 .797 r
.24902 .35005 .25892 .36946 .27522 .3607 .26521 .34292 Metetra
.663 .638 .815 r
.26521 .34292 .27522 .3607 .29158 .35197 .28147 .33578 Metetra
.668 .661 .834 r
.28147 .33578 .29158 .35197 .30799 .34347 .29782 .32871 Metetra
.67 .683 .853 r
.29782 .32871 .30799 .34347 .32445 .33529 .31425 .32178 Metetra
.671 .704 .872 r
.31425 .32178 .32445 .33529 .34097 .32752 .33075 .315 Metetra
.67 .722 .888 r
.33075 .315 .34097 .32752 .35754 .32017 .34733 .3084 Metetra
.667 .736 .902 r
.34733 .3084 .35754 .32017 .37417 .31328 .364 .30198 Metetra
.662 .745 .912 r
.364 .30198 .37417 .31328 .39088 .30684 .38076 .29574 Metetra
.657 .749 .918 r
.38076 .29574 .39088 .30684 .40767 .30086 .39761 .28969 Metetra
.651 .747 .92 r
.39761 .28969 .40767 .30086 .42454 .29535 .41456 .28383 Metetra
.646 .74 .918 r
.41456 .28383 .42454 .29535 .44152 .29029 .43162 .27816 Metetra
.64 .727 .911 r
.43162 .27816 .44152 .29029 .4586 .28569 .44878 .27267 Metetra
.634 .709 .901 r
.44878 .27267 .4586 .28569 .4758 .28153 .46606 .26736 Metetra
.629 .688 .886 r
.46606 .26736 .4758 .28153 .49313 .27779 .48345 .26221 Metetra
.624 .664 .869 r
.48345 .26221 .49313 .27779 .51059 .27439 .50096 .25719 Metetra
.619 .639 .849 r
.50096 .25719 .51059 .27439 .5282 .27121 .51861 .25226 Metetra
.615 .616 .828 r
.51861 .25226 .5282 .27121 .54596 .2681 .53638 .24732 Metetra
.613 .595 .809 r
.53638 .24732 .54596 .2681 .56386 .26478 .55428 .24227 Metetra
.613 .579 .792 r
.55428 .24227 .56386 .26478 .58191 .26094 .57232 .23696 Metetra
.616 .568 .778 r
.57232 .23696 .58191 .26094 .60008 .25619 .59047 .23121 Metetra
.622 .564 .769 r
.59047 .23121 .60008 .25619 .61836 .25009 .60874 .22484 Metetra
.632 .568 .765 r
.60874 .22484 .61836 .25009 .63669 .24224 .62711 .21766 Metetra
.646 .582 .769 r
.62711 .21766 .63669 .24224 .65505 .23235 .64555 .20953 Metetra
.664 .607 .781 r
.64555 .20953 .65505 .23235 .67338 .22032 .66405 .20043 Metetra
.689 .649 .805 r
.66405 .20043 .67338 .22032 .69166 .2064 .6826 .19043 Metetra
.717 .712 .845 r
.6826 .19043 .69166 .2064 .7099 .1912 .7012 .17983 Metetra
.741 .802 .901 r
.7012 .17983 .7099 .1912 .72815 .17578 .71987 .16909 Metetra
.73 .904 .956 r
.71987 .16909 .72815 .17578 .74649 .16152 .73866 .15884 Metetra
.57 .923 .954 r
.06795 .39565 .0806 .39596 .09514 .39383 .08312 .38946 Metetra
.664 .879 .979 r
.08312 .38946 .09514 .39383 .10973 .39218 .09838 .38312 Metetra
.68 .777 .924 r
.09838 .38312 .10973 .39218 .12445 .39043 .11373 .37677 Metetra
.666 .688 .861 r
.11373 .37677 .12445 .39043 .13936 .38813 .12915 .37051 Metetra
.649 .628 .816 r
.12915 .37051 .13936 .38813 .15449 .38499 .14462 .36439 Metetra
.636 .591 .787 r
.14462 .36439 .15449 .38499 .16983 .38092 .16017 .35845 Metetra
.627 .571 .772 r
.16017 .35845 .16983 .38092 .18536 .37595 .17578 .35266 Metetra
.623 .563 .766 r
.17578 .35266 .18536 .37595 .20108 .3702 .19147 .34701 Metetra
.621 .563 .768 r
.19147 .34701 .20108 .3702 .21694 .36385 .20725 .34145 Metetra
.622 .571 .776 r
.20725 .34145 .21694 .36385 .23293 .35708 .22311 .33595 Metetra
.625 .585 .788 r
.22311 .33595 .23293 .35708 .24902 .35005 .23907 .33046 Metetra
.629 .603 .805 r
.23907 .33046 .24902 .35005 .26521 .34292 .25513 .32495 Metetra
.634 .625 .823 r
.25513 .32495 .26521 .34292 .28147 .33578 .27129 .3194 Metetra
.64 .648 .843 r
.27129 .3194 .28147 .33578 .29782 .32871 .28756 .31379 Metetra
.644 .673 .862 r
.28756 .31379 .29782 .32871 .31425 .32178 .30394 .3081 Metetra
.648 .696 .88 r
.30394 .3081 .31425 .32178 .33075 .315 .32042 .30234 Metetra
.651 .716 .895 r
.32042 .30234 .33075 .315 .34733 .3084 .33701 .29648 Metetra
.654 .732 .907 r
.33701 .29648 .34733 .3084 .364 .30198 .35371 .29054 Metetra
.656 .743 .914 r
.35371 .29054 .364 .30198 .38076 .29574 .37052 .28451 Metetra
.657 .749 .918 r
.37052 .28451 .38076 .29574 .39761 .28969 .38744 .27839 Metetra
.658 .75 .918 r
.38744 .27839 .39761 .28969 .41456 .28383 .40447 .27219 Metetra
.659 .745 .913 r
.40447 .27219 .41456 .28383 .43162 .27816 .42161 .26589 Metetra
.66 .734 .905 r
.42161 .26589 .43162 .27816 .44878 .27267 .43886 .2595 Metetra
.659 .719 .892 r
.43886 .2595 .44878 .27267 .46606 .26736 .45621 .25302 Metetra
.658 .699 .877 r
.45621 .25302 .46606 .26736 .48345 .26221 .47368 .24645 Metetra
.655 .677 .859 r
.47368 .24645 .48345 .26221 .50096 .25719 .49126 .23981 Metetra
.651 .653 .84 r
.49126 .23981 .50096 .25719 .51861 .25226 .50895 .2331 Metetra
.645 .63 .82 r
.50895 .2331 .51861 .25226 .53638 .24732 .52675 .22633 Metetra
.64 .608 .802 r
.52675 .22633 .53638 .24732 .55428 .24227 .54466 .21955 Metetra
.634 .589 .786 r
.54466 .21955 .55428 .24227 .57232 .23696 .56268 .21277 Metetra
.628 .574 .774 r
.56268 .21277 .57232 .23696 .59047 .23121 .58083 .20603 Metetra
.624 .565 .767 r
.58083 .20603 .59047 .23121 .60874 .22484 .59909 .19939 Metetra
.621 .562 .766 r
.59909 .19939 .60874 .22484 .62711 .21766 .61748 .19287 Metetra
.62 .567 .773 r
.61748 .19287 .62711 .21766 .64555 .20953 .636 .18651 Metetra
.622 .584 .79 r
.636 .18651 .64555 .20953 .66405 .20043 .65465 .18032 Metetra
.627 .618 .822 r
.65465 .18032 .66405 .20043 .6826 .19043 .67345 .17428 Metetra
.633 .675 .872 r
.67345 .17428 .6826 .19043 .7012 .17983 .69239 .16832 Metetra
.634 .762 .94 r
.69239 .16832 .7012 .17983 .71987 .16909 .71147 .16234 Metetra
.601 .861 .993 r
.71147 .16234 .71987 .16909 .73866 .15884 .73068 .15619 Metetra
.753 .959 .937 r
.05508 .3953 .06795 .39565 .08312 .38946 .07092 .38501 Metetra
.788 .882 .915 r
.07092 .38501 .08312 .38946 .09838 .38312 .0869 .37395 Metetra
.767 .787 .87 r
.0869 .37395 .09838 .38312 .11373 .37677 .10291 .36299 Metetra
.729 .707 .829 r
.10291 .36299 .11373 .37677 .12915 .37051 .11886 .35278 Metetra
.693 .646 .799 r
.11886 .35278 .12915 .37051 .14462 .36439 .13471 .34372 Metetra
.662 .604 .779 r
.13471 .34372 .14462 .36439 .16017 .35845 .15048 .33592 Metetra
.638 .577 .769 r
.15048 .33592 .16017 .35845 .17578 .35266 .16617 .32933 Metetra
.62 .561 .767 r
.16617 .32933 .17578 .35266 .19147 .34701 .18184 .32377 Metetra
.608 .556 .771 r
.18184 .32377 .19147 .34701 .20725 .34145 .19752 .319 Metetra
.6 .56 .781 r
.19752 .319 .20725 .34145 .22311 .33595 .21325 .31474 Metetra
.597 .571 .795 r
.21325 .31474 .22311 .33595 .23907 .33046 .22907 .31077 Metetra
.598 .588 .812 r
.22907 .31077 .23907 .33046 .25513 .32495 .24499 .30688 Metetra
.602 .61 .832 r
.24499 .30688 .25513 .32495 .27129 .3194 .26103 .30291 Metetra
.608 .635 .852 r
.26103 .30291 .27129 .3194 .28756 .31379 .27721 .29874 Metetra
.615 .661 .871 r
.27721 .29874 .28756 .31379 .30394 .3081 .29353 .2943 Metetra
.623 .686 .888 r
.29353 .2943 .30394 .3081 .32042 .30234 .30999 .28954 Metetra
.632 .709 .902 r
.30999 .28954 .32042 .30234 .33701 .29648 .32658 .28444 Metetra
.64 .727 .911 r
.32658 .28444 .33701 .29648 .35371 .29054 .34331 .27898 Metetra
.649 .741 .917 r
.34331 .27898 .35371 .29054 .37052 .28451 .36017 .27315 Metetra
.657 .749 .918 r
.36017 .27315 .37052 .28451 .38744 .27839 .37715 .26696 Metetra
.665 .752 .915 r
.37715 .26696 .38744 .27839 .40447 .27219 .39426 .2604 Metetra
.672 .749 .908 r
.39426 .2604 .40447 .27219 .42161 .26589 .41148 .25348 Metetra
.678 .74 .898 r
.41148 .25348 .42161 .26589 .43886 .2595 .42882 .24619 Metetra
.682 .727 .884 r
.42882 .24619 .43886 .2595 .45621 .25302 .44627 .23854 Metetra
.684 .709 .867 r
.44627 .23854 .45621 .25302 .47368 .24645 .46382 .23056 Metetra
.683 .688 .849 r
.46382 .23056 .47368 .24645 .49126 .23981 .48147 .22229 Metetra
.679 .666 .83 r
.48147 .22229 .49126 .23981 .50895 .2331 .49922 .21382 Metetra
.673 .643 .812 r
.49922 .21382 .50895 .2331 .52675 .22633 .51705 .20524 Metetra
.664 .62 .795 r
.51705 .20524 .52675 .22633 .54466 .21955 .53498 .19674 Metetra
.653 .599 .782 r
.53498 .19674 .54466 .21955 .56268 .21277 .55301 .1885 Metetra
.641 .581 .772 r
.55301 .1885 .56268 .21277 .58083 .20603 .57115 .18079 Metetra
.627 .566 .767 r
.57115 .18079 .58083 .20603 .59909 .19939 .58941 .17387 Metetra
.611 .556 .769 r
.58941 .17387 .59909 .19939 .61748 .19287 .60781 .16801 Metetra
.594 .553 .778 r
.60781 .16801 .61748 .19287 .636 .18651 .6264 .16339 Metetra
.574 .558 .799 r
.6264 .16339 .636 .18651 .65465 .18032 .64519 .16008 Metetra
.552 .578 .834 r
.64519 .16008 .65465 .18032 .67345 .17428 .66421 .15797 Metetra
.524 .62 .889 r
.66421 .15797 .67345 .17428 .69239 .16832 .68346 .15665 Metetra
.485 .691 .958 r
.68346 .15665 .69239 .16832 .71147 .16234 .70292 .15546 Metetra
.424 .774 .996 r
.70292 .15546 .71147 .16234 .73068 .15619 .72254 .15348 Metetra
.841 .943 .889 r
.04201 .39483 .05508 .3953 .07092 .38501 .05855 .38043 Metetra
.847 .863 .854 r
.05855 .38043 .07092 .38501 .0869 .37395 .07528 .36465 Metetra
.816 .785 .823 r
.07528 .36465 .0869 .37395 .10291 .36299 .09198 .34913 Metetra
.773 .716 .8 r
.09198 .34913 .10291 .36299 .11886 .35278 .10849 .33504 Metetra
.728 .66 .783 r
.10849 .33504 .11886 .35278 .13471 .34372 .12474 .32307 Metetra
.686 .616 .772 r
.12474 .32307 .13471 .34372 .15048 .33592 .14073 .31346 Metetra
.649 .583 .768 r
.14073 .31346 .15048 .33592 .16617 .32933 .15651 .30608 Metetra
.618 .562 .769 r
.15651 .30608 .16617 .32933 .18184 .32377 .17216 .30061 Metetra
.594 .55 .775 r
.17216 .30061 .18184 .32377 .19752 .319 .18774 .2966 Metetra
.577 .549 .786 r
.18774 .2966 .19752 .319 .21325 .31474 .20333 .29357 Metetra
.567 .556 .802 r
.20333 .29357 .21325 .31474 .22907 .31077 .21899 .29108 Metetra
.563 .572 .82 r
.21899 .29108 .22907 .31077 .24499 .30688 .23477 .28878 Metetra
.565 .594 .841 r
.23477 .28878 .24499 .30688 .26103 .30291 .25069 .28635 Metetra
.572 .62 .861 r
.25069 .28635 .26103 .30291 .27721 .29874 .26677 .28361 Metetra
.583 .648 .88 r
.26677 .28361 .27721 .29874 .29353 .2943 .28302 .2804 Metetra
.596 .676 .896 r
.28302 .2804 .29353 .2943 .30999 .28954 .29944 .27664 Metetra
.611 .701 .908 r
.29944 .27664 .30999 .28954 .32658 .28444 .31603 .27227 Metetra
.626 .723 .916 r
.31603 .27227 .32658 .28444 .34331 .27898 .33278 .26728 Metetra
.642 .739 .919 r
.33278 .26728 .34331 .27898 .36017 .27315 .34969 .26166 Metetra
.657 .749 .918 r
.34969 .26166 .36017 .27315 .37715 .26696 .36674 .25539 Metetra
.671 .754 .913 r
.36674 .25539 .37715 .26696 .39426 .2604 .38393 .24849 Metetra
.684 .753 .903 r
.38393 .24849 .39426 .2604 .41148 .25348 .40125 .24094 Metetra
.695 .746 .891 r
.40125 .24094 .41148 .25348 .42882 .24619 .41868 .23275 Metetra
.702 .734 .875 r
.41868 .23275 .42882 .24619 .44627 .23854 .43623 .22395 Metetra
.707 .718 .858 r
.43623 .22395 .44627 .23854 .46382 .23056 .45387 .21457 Metetra
.707 .699 .84 r
.45387 .21457 .46382 .23056 .48147 .22229 .4716 .2047 Metetra
.704 .677 .821 r
.4716 .2047 .48147 .22229 .49922 .21382 .48942 .19449 Metetra
.697 .655 .804 r
.48942 .19449 .49922 .21382 .51705 .20524 .5073 .18414 Metetra
.686 .632 .789 r
.5073 .18414 .51705 .20524 .53498 .19674 .52526 .17394 Metetra
.671 .61 .778 r
.52526 .17394 .53498 .19674 .55301 .1885 .5433 .16429 Metetra
.653 .589 .77 r
.5433 .16429 .55301 .1885 .57115 .18079 .56143 .15562 Metetra
.629 .569 .768 r
.56143 .15562 .57115 .18079 .58941 .17387 .57969 .14844 Metetra
.601 .552 .772 r
.57969 .14844 .58941 .17387 .60781 .16801 .59812 .1432 Metetra
.565 .538 .783 r
.59812 .1432 .60781 .16801 .6264 .16339 .61675 .14028 Metetra
.52 .529 .805 r
.61675 .14028 .6264 .16339 .64519 .16008 .63566 .1398 Metetra
.462 .529 .84 r
.63566 .1398 .64519 .16008 .66421 .15797 .65488 .14154 Metetra
.388 .544 .889 r
.65488 .14154 .66421 .15797 .68346 .15665 .67441 .14482 Metetra
.296 .581 .94 r
.67441 .14482 .68346 .15665 .70292 .15546 .69423 .1484 Metetra
.214 .642 .953 r
.69423 .1484 .70292 .15546 .72254 .15348 .71424 .15058 Metetra
.884 .915 .842 r
.02874 .39413 .04201 .39483 .05855 .38043 .04601 .37565 Metetra
.877 .843 .807 r
.04601 .37565 .05855 .38043 .07528 .36465 .06353 .35522 Metetra
.845 .779 .787 r
.06353 .35522 .07528 .36465 .09198 .34913 .08095 .33522 Metetra
.803 .722 .775 r
.08095 .33522 .09198 .34913 .10849 .33504 .09803 .31734 Metetra
.756 .671 .769 r
.09803 .31734 .10849 .33504 .12474 .32307 .11469 .30256 Metetra
.707 .628 .766 r
.11469 .30256 .12474 .32307 .14073 .31346 .13091 .29119 Metetra
.659 .591 .767 r
.13091 .29119 .14073 .31346 .15651 .30608 .14679 .28305 Metetra
.616 .563 .772 r
.14679 .28305 .15651 .30608 .17216 .30061 .16241 .27766 Metetra
.579 .545 .78 r
.16241 .27766 .17216 .30061 .18774 .2966 .17789 .27438 Metetra
.551 .537 .793 r
.17789 .27438 .18774 .2966 .20333 .29357 .19334 .27253 Metetra
.532 .541 .809 r
.19334 .27253 .20333 .29357 .21899 .29108 .20884 .27148 Metetra
.524 .554 .828 r
.20884 .27148 .21899 .29108 .23477 .28878 .22446 .27071 Metetra
.524 .575 .849 r
.22446 .27071 .23477 .28878 .25069 .28635 .24024 .26979 Metetra
.532 .603 .87 r
.24024 .26979 .25069 .28635 .26677 .28361 .25622 .26843 Metetra
.547 .634 .889 r
.25622 .26843 .26677 .28361 .28302 .2804 .2724 .26642 Metetra
.566 .665 .904 r
.2724 .26642 .28302 .2804 .29944 .27664 .28878 .26363 Metetra
.588 .693 .915 r
.28878 .26363 .29944 .27664 .31603 .27227 .30536 .25999 Metetra
.611 .718 .921 r
.30536 .25999 .31603 .27227 .33278 .26728 .32213 .25546 Metetra
.635 .736 .922 r
.32213 .25546 .33278 .26728 .34969 .26166 .33909 .25003 Metetra
.657 .749 .918 r
.33909 .25003 .34969 .26166 .36674 .25539 .3562 .24369 Metetra
.677 .756 .91 r
.3562 .24369 .36674 .25539 .38393 .24849 .37348 .23644 Metetra
.695 .756 .899 r
.37348 .23644 .38393 .24849 .40125 .24094 .39089 .22828 Metetra
.71 .751 .884 r
.39089 .22828 .40125 .24094 .41868 .23275 .40843 .21921 Metetra
.721 .741 .867 r
.40843 .21921 .41868 .23275 .43623 .22395 .42608 .20927 Metetra
.727 .726 .849 r
.42608 .20927 .43623 .22395 .45387 .21457 .44382 .19853 Metetra
.729 .708 .83 r
.44382 .19853 .45387 .21457 .4716 .2047 .46165 .1871 Metetra
.727 .688 .813 r
.46165 .1871 .4716 .2047 .48942 .19449 .47954 .17519 Metetra
.719 .666 .797 r
.47954 .17519 .48942 .19449 .5073 .18414 .49749 .16311 Metetra
.707 .644 .784 r
.49749 .16311 .5073 .18414 .52526 .17394 .51549 .15128 Metetra
.689 .621 .775 r
.51549 .15128 .52526 .17394 .5433 .16429 .53355 .14024 Metetra
.665 .597 .77 r
.53355 .14024 .5433 .16429 .56143 .15562 .55169 .13065 Metetra
.633 .573 .77 r
.55169 .13065 .56143 .15562 .57969 .14844 .56995 .12321 Metetra
.59 .549 .776 r
.56995 .12321 .57969 .14844 .59812 .1432 .58838 .11858 Metetra
.534 .523 .789 r
.58838 .11858 .59812 .1432 .61675 .14028 .60707 .11729 Metetra
.459 .496 .81 r
.60707 .11729 .61675 .14028 .63566 .1398 .62607 .11954 Metetra
.358 .469 .837 r
.62607 .11954 .63566 .1398 .65488 .14154 .64545 .12504 Metetra
.228 .446 .864 r
.64545 .12504 .65488 .14154 .67441 .14482 .66524 .1328 Metetra
.087 .441 .878 r
.66524 .1328 .67441 .14482 .69423 .1484 .68538 .14108 Metetra
.001 .483 .866 r
.68538 .14108 .69423 .1484 .71424 .15058 .70575 .14739 Metetra
.905 .887 .804 r
.0153 .39308 .02874 .39413 .04601 .37565 .03332 .3706 Metetra
.893 .824 .772 r
.03332 .3706 .04601 .37565 .06353 .35522 .05165 .34565 Metetra
.864 .772 .758 r
.05165 .34565 .06353 .35522 .08095 .33522 .0698 .3213 Metetra
.825 .725 .754 r
.0698 .3213 .08095 .33522 .09803 .31734 .08747 .29977 Metetra
.778 .681 .756 r
.08747 .29977 .09803 .31734 .11469 .30256 .10454 .28228 Metetra
.725 .638 .761 r
.10454 .28228 .11469 .30256 .13091 .29119 .12101 .26922 Metetra
.67 .599 .768 r
.12101 .26922 .13091 .29119 .14679 .28305 .13697 .26035 Metetra
.615 .566 .776 r
.13697 .26035 .14679 .28305 .16241 .27766 .15257 .25503 Metetra
.565 .541 .787 r
.15257 .25503 .16241 .27766 .17789 .27438 .16795 .25245 Metetra
.523 .526 .8 r
.16795 .25245 .17789 .27438 .19334 .27253 .18326 .25172 Metetra
.494 .524 .816 r
.18326 .25172 .19334 .27253 .20884 .27148 .19859 .25205 Metetra
.479 .534 .836 r
.19859 .25205 .20884 .27148 .22446 .27071 .21405 .25275 Metetra
.477 .555 .856 r
.21405 .25275 .22446 .27071 .24024 .26979 .2297 .25328 Metetra
.487 .584 .877 r
.2297 .25328 .24024 .26979 .25622 .26843 .24556 .25325 Metetra
.508 .617 .896 r
.24556 .25325 .25622 .26843 .2724 .26642 .26166 .2524 Metetra
.534 .652 .911 r
.26166 .2524 .2724 .26642 .28878 .26363 .278 .25054 Metetra
.565 .685 .92 r
.278 .25054 .28878 .26363 .30536 .25999 .29457 .24759 Metetra
.597 .712 .925 r
.29457 .24759 .30536 .25999 .32213 .25546 .31136 .24351 Metetra
.628 .734 .924 r
.31136 .24351 .32213 .25546 .33909 .25003 .32836 .23826 Metetra
.657 .749 .918 r
.32836 .23826 .33909 .25003 .3562 .24369 .34554 .23185 Metetra
.683 .758 .908 r
.34554 .23185 .3562 .24369 .37348 .23644 .3629 .22426 Metetra
.706 .759 .894 r
.3629 .22426 .37348 .23644 .39089 .22828 .38041 .2155 Metetra
.724 .755 .877 r
.38041 .2155 .39089 .22828 .40843 .21921 .39806 .20558 Metetra
.737 .746 .859 r
.39806 .20558 .40843 .21921 .42608 .20927 .41582 .19454 Metetra
.746 .733 .84 r
.41582 .19454 .42608 .20927 .44382 .19853 .43368 .18247 Metetra
.749 .716 .822 r
.43368 .18247 .44382 .19853 .46165 .1871 .45161 .16952 Metetra
.747 .698 .805 r
.45161 .16952 .46165 .1871 .47954 .17519 .46959 .15598 Metetra
.739 .677 .79 r
.46959 .15598 .47954 .17519 .49749 .16311 .48761 .14224 Metetra
.726 .655 .779 r
.48761 .14224 .49749 .16311 .51549 .15128 .50566 .12884 Metetra
.705 .632 .772 r
.50566 .12884 .51549 .15128 .53355 .14024 .52375 .11648 Metetra
.676 .607 .77 r
.52375 .11648 .53355 .14024 .55169 .13065 .54191 .10601 Metetra
.636 .579 .773 r
.54191 .10601 .55169 .13065 .56995 .12321 .56017 .09831 Metetra
.58 .546 .782 r
.56017 .09831 .56995 .12321 .58838 .11858 .57862 .09425 Metetra
.501 .508 .795 r
.57862 .09425 .58838 .11858 .60707 .11729 .59733 .09451 Metetra
.391 .459 .811 r
.59733 .09451 .60707 .11729 .62607 .11954 .61641 .09938 Metetra
.241 .399 .821 r
.61641 .09938 .62607 .11954 .64545 .12504 .63594 .10848 Metetra
.058 .333 .813 r
.63594 .10848 .64545 .12504 .66524 .1328 .65594 .12059 Metetra
0 .29 .782 r
.65594 .12059 .66524 .1328 .68538 .14108 .67638 .13343 Metetra
0 .325 .758 r
.67638 .13343 .68538 .14108 .70575 .14739 .69708 .14379 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(-2)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(-1)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(1)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(2)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo001kOol4000fOol007aoo`03001oogoo03Ioo`00OGoo00<007oo
Ool0=Goo001nOol00`00Oomoo`0dOol007]oo`04001oogoo000:Ool4000XOol007aoo`8000aoo`03
001oogoo02Qoo`00Pgoo1@000goo00<007ooOol09goo002<Ool00`00Oomoo`0VOol007moo`8000Qo
o`04001oogoo000XOol007aoo`H000Qoo`8002Uoo`00NWoo0P0017oo0P00<goo001OOol3000EOol5
0004Ool00`00Ool0000bOol0061oo`03001oogoo019oo`8000Uoo`04001oo`00000aOol0061oo`03
001oogoo00moo`@000Yoo`04001oo`00000aOol0061oo`03001oogoo00aoo`<000ioo`05001oogoo
Ool0000`Ool0061oo`03001oogoo00Yoo`@000moo`05001oogooOol0000`Ool005moo`8000Uoo`<0
01=oo`03001oogoo009oo`03001oogoo02eoo`00J7oo0`005Goo00<007ooOol00Woo0P00;Woo001U
Ool30002Ool00`00Oomoo`0COol00`00Oomoo`03Ool00`00Oomoo`0/Ool006=oo`8001aoo`03001o
ogoo00=oo`03001oogoo02]oo`00H7oo0`000Woo00<007ooOol06Goo00<007ooOol017oo00<007oo
Ool0:Woo0015Ool2000GOol2000QOol00`00Oomoo`03Ool2000/Ool004Aoo`04001oogoo000COol3
00000goo001oo`0POol00`00Oomoo`05Ool00`00Oomoo`0YOol004Aoo`04001oogoo000@Ool3000C
Ool7000<Ool00`00Oomoo`06Ool00`00Oomoo`09Ool3000LOol004Aoo`04001oogoo000>Ool20002
Ool00`00Oomoo`0>Ool300001DXi00126D8I00000S7I0`002Goo00<007ooOol01Woo0P002goo00<0
07ooOol06goo0014Ool01000Oomoo`002goo0`004Woo0`0000EBF000BSU:>@0000926@04000afC7I
00027IX20007Ool00`00Oomoo`07Ool00`00Oomoo`02Ool50002Ool00`00Oomoo`0KOol004Eoo`80
00Yoo`80009oo`03001oogoo00moo`03001FN00000=BF004001:>DXi0002@QT02000<MTaf@007IX0
00EJ0006Ool00`00Oomoo`07Ool00`00Oomoo`09Ool00`00Oomoo`0KOol004ioo`<001Eoo`05001F
N5IhEWP00002DUP00`00BST0000:00000`EJ00000004Ool00`00Oomoo`08Ool00`00Oomoo`08Ool0
0`00Oomoo`0KOol004aoo`800003Ool0000001Aoo`8000=FN003001BF59H00800005BSP004HIAQT0
0002>OT00`00;OX]nP0300000`4i001oo`02Ool00`00Oomoo`08Ool20008Ool2000MOol004Uoo`<0
01Moo`80009JV003001FN5Ih00<00005DUP004XhBSP00002AQT01000>OTin@000Rgj00<001gK0000
0`0000=oo`00Ool02Woo00<007ooOol09Goo0017Ool200000goo0000000FOol00`00GYP00002FYP3
00001EIh001BF59H00000TXh00<004HIAQT02P0000<9g00000000P002goo00<007ooOol097oo000[
Ool3000FOol3000IOol20003GYP200001EYh001FN5Ih00000e9H00<004Xh00000P0000E:>@00@QU2
6@00008j6P03000b?38l00H000]oo`03001oogoo02=oo`00;7oo00<007ooOol04goo0P0000=oo`00
Ool067oo00<005ji00000UjH0P000UYh00H005IhEWQFN000DUP300001Dhh001:>DXi00000d8I00H0
03XJ>QX0038l00029Uh60008Ool00`00Oomoo`0SOol002Eoo`D0009oo`03001oogoo011oo`<001]o
o`8000=N^@800003FYT005Yh009JN003001FN00000800003DUP004hh009>>005001:>DXi00126@0<
0002=al30007Ool00`00Oomoo`0ROol002aoo`03001oogoo00eoo`<0009oo`03001oogoo01Qoo`03
001Nf@00009N^@8000=JV@03001JN5Yh00800003DUP0059H009BF005001>>4hh001:>@0300001DXi
0016FTIJ00000d9l0P0000<ngP0000000P0000=OWP0000000P001Goo00<007ooOol08Goo000/Ool0
0`00Oomoo`0;Ool2000NOol20003G]T200000eji001JV@02FYT01000FWP000000e9H00@0059HDUP0
009>>0800003CSP004Xi009:>@040016FTIJ0002@W`300001E<N000005nN00000V^l00@006nk0000
00=oo`80029oo`00:goo0P002Woo0`000Woo00<007ooOol06Woo00<005kJ00000ekI00D005jiG[UN
^@00009JV@80009FN004001BF59H0002DUP300001Dhh001>>4hh00000TXi00<004IJAUX00`0000EB
^`00000005ll00<00005Jk`006nk001gV@030002Ool00`00Oomoo`0POol003Eoo`80025oo`06001N
fUkJG]X005kI0P000eji0`0000EFV@00EWQFN000009BF0<00005DUP004hhCSP00002CSP00`00BSU:
>@0300000e9j001B^`0300000ell001[F`02Je/01P00Kk/007NI001gV0<0009oo`03001oogoo01mo
o`00<Woo0`000Woo00<007ooOol07Goo00<005kj00000UkJ0P000UkJ00D005ji001JV@0000=FV@03
001FN00000800005DUP0059HDUP00003CSP00`00CSP00002000014iI001BNU9j0P000U[K0P000f]K
00H006mI001gV@00MiP200001G]f000007oo00008Goo000`Ool3000ROol20002G_X3000015kJ001N
fUkJ0P000eZI00<005JI00000P0000EBN@00DUQBF00000=BF003001>>00000800003CUP004iI009>
F@800004FYT005[KF]/3000016]K001_FFmI0P0000=gV000NgL00P0000=kM@0000000P0087oo000D
Ool4000EOol30002Ool00`00Oomoo`0MOol300001Ukk001NnP00G_/0009NfP<00003F[X005ZI009J
V@800005EWT0059iDWT00003DUP00`00DUP0000200000dhh001>F002CUP01000CUT000000eZI00<0
05[K00000V;j00D006mIKeT007=H00<00007NgL00000001kE@00Ne@00P007goo000EOol00`00Oomo
o`0COol3000SOol00`00Ga`00002G_/20003G_/200000e[J001J^P02F[X01000FYT000000eIi00D0
059iDWT0059H00<00003CUP004hh009>>003001>F00000800003EWP005ZI009JV@<0000<H_X00000
KeT007=H001gE`00MeH007]f0`000W]D0P001Woo0P005goo000>Ool50003Ool00`00Oomoo`0?Ool3
000TOol20003Ga`20002G_/01000G_/000000e[J00<005ZjF[X00P000UJI00D005IiEWT0059i00<0
0003DUP004iH009>F003001>>4hh00800003DUP005Ih009FN004001JV@00G[T30002JaT01@00LeP0
07MG00000WMF0P0000IkM@00001kE000Ne@20004Ool01000Oomoo`005Woo000GOol00`00Oomoo`0;
Ool5000SOol20002Fa`01000Ga`000000ekk0P000U[K00<005[JF]X00`0000EF^P00EYUFV@00009F
N@800003DUT0059H009BF008001>F4iH001>>000DUP0009BF003001FN5Ih008000=N^@04001[6F/I
JaT20002MeL01000MeH007/e0P0000=kE@00Ne@00P0000=k=00000000goo00@007ooOol001Ioo`00
57oo00@007ooOol000]oo`8002Moo`03001G7@00009K7080009K7005001Nn`00F_/00002F]/30000
0eKK001F^P02E[X00`00EYT0000200001E9i001BFE9I00000U9H00<004iHCUP00P000U9H00P0059H
DUP005Ih001JV0000Uji0P000V/I00L006lH000007MG001gEP02000027/e000007]D001kE000Nc@2
0003Ool01000Oomoo`005Woo000EOol20009Ool5000UOol20002Ead3000015/L001K75/L0P000e[k
0P0000=Ff`00E]/00UKK00<005Jj00000P0000=BVP00DWT00U9i00<0059I00001@0000=>F000DUP0
0U9H00@0059H000000=JV0800006H]P00000001_60000W<G00H007MF001k=@00NcD200001W]D0000
07/d001k=080009oo`04001oogoo000FOol001ioo`8002Aoo`H00004E_d000000002Ead00`00Fa`0
00020002F_`01000F_/000000eKK00<005KK00000P0000=BVP00DYX00U:J00<0059i00000P0000E>
F@00CUQ>F00000=>F003001BF00000800004EWP005ZHFYP20002H]P20002KaP01@00LaMc5`00MeH0
0P0000=k=@00Nc@00P0000EkE000Nc@007/d00<0009oo`8001Moo`006goo0`0000=oo`00Ool08Goo
0P000TkM00<005;mD_d00P000UKm00<005LMEad00P000ULL00<005[lF_`00P000UKl00<005KKE]/0
0P0000=B^`00DYX00U:J00<005:J00000P0000E>N@00CUU>F@0000=>F003001>F00000800003DUP0
05Ih009FN0800003GYP006;H00@0000;KaP007<G001c5P00NcD007/e001k=00200000g]D001k=002
00000g/d0000000KOol001Yoo`8000mooaL00004B[d004kMC]d3000015;m001FoEKm0`0000AG7@00
EaaG70<00004E_`005KlE_`300001EKL001B^e:k00000U:J0P0000=>NP00CWT00Tii00D004iICUT0
04iH00<00003CUP0059H009BF005001FN5Ih001NV00300001f;H001Zef[G001c5`00009c5P04001k
=@00NcD200000g/d00000002000017/d000007/d0P006Woo000JOol3000<Ool200001diH001:>DXi
00126@00008n6P05000n>P00?S/00003?U`00`00@W`00002AYd00`00B[d000020002C]d00`00D_d0
0002000015Km001G7ELM0`0000AFo@00E_aFo0<00004E_`005KLE]`600001DjK001>NTij00000Tii
00<004iI00000P0000=>F@00CUP00TiH00D0059HDUP005Ih00<000=NV00:001Ze`00J]L007<G001c
5P00MaD3000027/d000007/c001k<`00Nc@2000017/d000000006Goo000JOol200000goo001oo`08
Ool200001EIh001>F4iH00000TXi00<0048I00000ShJ00D003hj000n>`00008nG0L0009:g@03001>
gDkM0080009Bo@03001Fo@000080009G7@03001Fo@000080009Fo003001Fo00000800005D]`004jk
C[/00003CY/00`00CWX0000400000dYI001>F@02CUT01@00CUQ>F000DUP00`000UYh00<005jHGYP0
0P000V[G00D006[g001c5P00009g5@04001g5@00Nc@200000g/c001k<`02000017/d000007/d0P00
6Goo000JOol01@00Oomoogoo00001goo0P000UIh00D004iHCUP004Xi00d00007AW/004JLAY`004Zl
00000T[M0`0000ABo@00D_eBo@80009Fo@03001G7ELM0080009Fo@03001Fo5Kl0080009Bg005001B
g5;L001>^`0600001DYj001:FTYJ00000TYI00D004iICUT004iH00@00007DUP005YhFWP005jH0000
0V:h0P0000MZm`00001c5P00MaD00002MaD200001W/d000007/c001k<`800003Nc@007/d00<001Qo
o`006Goo0P0017oo00<007ooOol00goo00@005jHGYP0009FN006001>F000001:>0000THi00<004Hi
00000T8j00@0049K@U/00096N`030016W00000@0009>g@03001Bo@0000800004D_d005KmE_d30000
15Km001FoEKm0`0000ABo@00D]aBg0D00005BY`004YkBW/00003BWX01`00BUY:FP00BUT004iI00@0
0005CUP0059HDUP00002FWP20003H[P01P00J_MZm`00K_H0009g5@06001g5@00Na@007/d0P0000=k
<`00Nc<00P0000Ak=000001k50<001Moo`006Goo0P001Goo00@007ooOomoo`80009NV003001FN000
00800005CUP004XhBSP00002AST00`00AST00002@SX00`00@U/0000400001DZL001>_4jl00000TkM
0P000U;m00<005;m00000P000UKm00<005Km00000P000U;m00<005;m00000P0000E>g@00C[a>_000
009:W005001:NdYk001:NP0900001DXi001>F4iH00000e9H0P0000AJN000H[QR^0800003J_L006kf
0080009g5@04001g5@00Na@200001G/c001k<`00Nc<00`0000=k=000Na@00`005goo000IOol00`00
Oomoo`05Ool01@00Ool006:h00000UjH0P0000=FF000CUP00TiH00@004Xh0016>@P00006BWX004Yk
BW/004ZL1P0000ABg000D_eBo@80009Bo@03001FoEKm0080009Bo@03001BoE;m0080009>o@05001>
gDkM001>_00800000dIK000000020002BST01000BSU:>@000diH00@0059H000000=JN080009V]`80
0003K_H0000000800005MaD007/D001k500200001G/c001k<`00Nc<00P0000Ak=000001k50<001Io
o`0027oo0P0017oo0P000goo0`000goo00<007ooOol01Woo00@006:hH[QR^0800006FYP005IHEUP0
04iH1@0000E:>@00BSU:>@00009:FP05001:NTYj001:N`0200000djk001>_002C[`00`00D]`00002
000015;m001BoE;m0`0000=Fo@000000100000ABo@00C_e>o@D00005A[`004JLAY`00002AW/40002
AUX01000BSU:>@000TXi00<004iH00000P0000AFN000FWQJN08000=V]`05001^mVkf001bmP020000
1GLE001k5000Na@00P0000=k<`00Nc<00P0000Ak<`00001k5080009k508001Eoo`001goo00@007oo
Ool000=oo`8000Ioo`03001oogoo008000Moo`05001R^6:hH[P00003FYP01P00EUP00000CSP0009>
>004001:>DXi0002BST00`00BUX0000400001DjK001>^djk00000Tjl0P000U;L00<005;m00000P00
0UKm0P000UKm00<005;m00001@0000E>g@00B[e:_@000096_0030016W00000800004AW/004IK0002
AUX00`00BST0000500000diH001FN002EWP200000ejH001V]`02I[L200000fkf001bmP0200000gLE
00000002Na@200000g/c001k<`02000017/c000007/D0P000W/D0P005Goo00000goo000000030000
1Goo001oogoo00002goo00@007ooOol000Moo`04001Zn000H[P20003FYP200001E9H001>>4hh0000
0Thh00@004Xi001:>@<00006CUT004ijCWX004jK1P0000ABg000D]aBg080009Fo0<00004E_d005Km
E_d20002D_d00`00C_e>o@0E00001DHi001:>DXi00000diH00@005IhEWP000=NV003001V]`00009V
e`03001bmW;f00<00005Na@007/D001k50020002Nc<3000027/c000007/D001k5000Ma@20005Ool3
000<Ool000Moo`04001oogoo0008Ool30002Ool20006Ool01@00J_QZn6[h00000V:h00@005ZH0000
009BF005001>>4hh001>>00500000diI001>F@02CUT01P00CWX00000DYX000=B^`03001Bg0000080
009Fo080009Fo003001Fo@0000800004E_d005;mD_d3000014km001:gD[M100000E2W@00@Wa2O000
0092F`030012F`000092>P040016>DHi0002BST01@00CUQ>F000EWP00P0015jH00L006KGI]MVe`00
L_H00002L_H01000Na@007/D0P0000Ek4`00000007/c00800004Nc<000000002Na@00`00Ma@00002
0005Ool00`00Oomoo`0;Ool000Moo`04001oogoo0008Ool00`00Oomoo`02Ool20005Ool01000J_QZ
n6[h0P000V:h0P000UIh00D0059HDUP004hh00800005CSP004hhCSP00002CUT00`00CUT000020000
1E9j001BVU:J00000U:k0P000UKK00D005Kl001Fo000009Fo080009Fo@03001Fo@0000800004D_d0
04kmC_dB0002@SX00`00AST00002BST00`00CUQ>F0020002EWP00`00GYQNV0020002I]L01@00J]L0
0000L_H00`000W/D0P0000=k4`00Na<00P0000Ak<`00001k<`800007Na@007LDMa@007oo00001Goo
00<007ooOol02goo0008Ool20009Ool400000goo00000005Ool2000016[h001Vn6Kh0P0000IN^@00
EWQFN000DUP300001Dhh001>>4hh00000Thh00H004iI0000059i0003DWX00`00DYX00002000015Jk
001FfeKK0P000UKl0`0000AJo000E_eFo@80009Fo@03001Bo@0000<00004B]d004KM0002@[d01P00
?YdnW@00>W`0008jG003000j>`0000X00007CUT005IhEWP005jH00000V:H0P000V[G00D007;fL_H0
07LE00<00003Na@00000009k4`800004Nc<00000Nc<2000027/D001g5000MaD007oo0004Ool00`00
Oomoo`0;Ool001Qoo`8000Aoo`04001_66lH0003I_P01P00G[UN^Eji001FN0800005DUP004hhCSP0
0002CSP00`00CSP0000200001DiI001BNE9i00000U9j0P000UJj00<005Jk00000P000U[K0P000U[k
00<005[l00000P000ULL1@0000A>o@00B]e:g@`0008jG080008n>P040012>T8j0002AST01P00CUU>
FDiI001FN08000=RV005001Zef[GJ]L00002L_H2000017LE000007/D0P0000=k4`00Nc<00P0000Ak
<`00001k<`80009g5004001g5@00Ool20003Ool00`00Oomoo`0;Ool001Qoo`03001oogoo009oo`07
001_66lH001_6@00I_P00P000eji0P0000EFN000DUQBF000009>>006001>>000001>F0000diI00<0
059i00000P0000AFVP00E[YF^P80009Jf`05001Jf`00F_/00002F_/20002F_`00`00Ea`000020000
0eKm0000000400001d[N0012_T:n000nW@00008jO@03000fG00000D0008n>P040012>T8j0002AST0
0`00CUT00002000015Ih001RV6:H0P000V[G00H006kF000007LE0002MaD200001G/C001k4`00Nc<0
0P0000Ek<`00000007/c00800004Ma@00000MaD30002Ool2000=Ool001Qoo`8000=oo`03001_6000
009_6@80009Rf@05001N^@00FYT00002EWP00`00DUP0000400001TiH001>F4iH001>F@<00003EWT0
05JJ009FVP800004F[X005[KF]/20002F_/300000e[k001Jo0030002Ea`20002E_d00`00C_d0000>
0000138l000f6cHK2P0000=>F@00EWP00UIh0P000V:h00<006[G00000VkF00@007LE001g5@<00006
Na<00000Nc<007/c0P0000]k<`00001k<`00Nc<007LE001g5@00009oo`03001oogoo00ioo`004Woo
0P000goo0P000goo00H007=H000006lI0002JcX01000H]T000000eZI00P005Ih0000059H001>F000
0diH00H004iH0000059i0003EWT01000EYX000000eZj0P0000MJfP00F_/005kk001Jn`040002Fa`2
000015/L001FoEKm0P000Tkm00<004KN00000Sjn00H003ZM>Yd0039M0002;S`300001cHK000j6cXK
000n>P000096>@05001>FDiICUT00002EWP00`00H[QR^002H[P01000K]I^eVkF0P000WLE00T007LE
000007/C001k<`00Nc<00`0000=k<`0000000W/c00L007LE000007<E001oo`02000?Ool0015oo`04
001oogoo0002Ool20002Ool01P00LeQcF000LeT000=[>P80009NfP03001JV@0000800004DUT0059H
0002CUP00`00CUP0000200001E9H001BNE9i00000UIi0P000UZI00@005Zj000000=JfP03001Nn`00
00<0009Nn`06001K7000Ga`005/L0`0000=Fo@0000003`0000<^700000000P000SXK00<003hj0000
0THi00D004iICUT005Ih008000=R^080009^eP05001^mP00MaD00002MaD200000g/C001k<`020000
17/c000007/c0P000W/c0P0000Ag5@00LaD0009oo`03001oogoo00eoo`004Goo00@007ooOol0009o
o`800005Ool00000LeP00002LeT2000016/j001S>`000UkJ0P0000EBVP00DUUBF@00009BF006001>
F000001BF0000e9H00<0059i00000P0000AFV@00FYUJV@80009J^P03001JfP0000800004G_/005kj
0002G_/20002Ga`20002Fa`20002E_d00`00C_d00002A]h6000012iM000V72HL0`0000<^6`000000
10000THi00<004iICUT00P000UZH00<006:h00000VJh0P000Vkf00H007LE001g5@00Na@20002Nc<2
00003W/c00000000Nc<007/c001k=000MaD007<E0003Ool00`00Oomoo`0<Ool0015oo`04001oogoo
0002Ool200000goo001cV@02000017=I001_Nfmk0P0000IS>`00000005KK0002DYX00`00DUUBF@04
00001E9H001BF59H00000U9H0P0000=FN000EYT00UJI0P0000AJ^@00F[YJ^P<00003G]X00000009N
nP<0009Nn`80009O70800004Fa`005KmE_d500001T;N000jWSZN000^O@<000049Q`002Gl9O`30000
138K000j6SXJ1@0000A>F@00FYQJV08000=V^006001^mVkf001bmP000WLE00<007/D00000`0000Ak
<`00001o<`<0009k<`08001k=000001c5@00L_H0009oo`8000eoo`004Goo00@007ooOol0009oo`06
001oo`00LiT007>j0`0000=_N`00Iid00P000ULM00L005KKE]/005:J001BF@0200001DiH001BF59H
00000U9H00@0059H000000=FN004001FV@000002F[T3000025kI001NfP00G]X005kj0002H_X02P00
G_/005lK001O7000Ga/005/L0`0000AFo@00C_e>o@T000069Sd00000000Uo0000RGk0`0000Dj6P00
@SY2>P0000=>F@800003FYT006Jh009V^003001^mP00009bmP03001g5GLE00800005Na@00000001k
<`0300000glc00000002Nc<200001W/d001c5@00L_H000=oo`03001oogoo00]oo`004Woo0P000goo
0P0000AcV@00LkX0009[g0@0009GW`03001G7@0000H00005CUT004iHCUP00002DUP01000DUP00000
0UIh00<005IhEWP00P0000=JV@0000000`000ekI00D005kJG]YNfP0000=RnP80009O6`80009O6`80
009K7003001Fo@0000800007C_d004;N@]h003JN00000RYM0P000Qgl0`0000<Un`00;O/010000T8j
00@004iI0000009JV@05001V^6Jh001Vf00200000g;f001c5P0200000g/D00000002Nc@2000017/c
000007lc100000Qk=000Nc@007<E001bmP0017oo00<007ooOol02Woo000GOol@000014;o001:W000
0TYj00L004iICUT004iH001BF00300001U9H001FN5Ih001FN0@00003FYT00000009N^@0E001Nf@00
H]T005kJ001RnP00H_X006;j001O6`00Ha/005lK001O6`00009K7080009Fo@/000058Qd00000000M
g0050002=Q/500000dij001JV@02FYT20002I]P01000L_IbmP000W<F00L007/D000007/d001k=002
000027lc000007lc001o<`00Nc@200001W/e001bmP00K_H000=oo`03001oogoo00Yoo`005Woo0P00
3Goo1@0000I:W000BWY:NP00CUT400000e9H001BF002DUP00`00EWP000020002FWP2000015jH001N
^Eji0P000V;I0P000V;j0P000V;j0`0000ES6`00000005lK00<00006Fa`00000001Fo@000T[n1P00
00<VG@0000001@0000HMf`00000003HK0002@SX01P00CWY>NTij001JV@8000=Vf004001bmW;f0002
LaH200001G/D001k=000Nc@00P0000Ao<`000000009o<`800008Nc@007/e001bmP00K_H000Aoo`03
001oogoo00=oo`@0009oo`005Woo0P004Goo1`0000A:>@00CSP0009BF003001BF00000800004EWP0
05YhFWP20003GYP2000035ji001Rf@00H]T006;j001Rn@00H_X000=RnP03001S6f<K0080009O6`80
0004Fa`005KmE_d3000014[n000n_P000S:N1P000QFk0P000RGK0`000T8j00<004ij00000P000Uji
00@006KH001Zn080009c5P06001c5P00001k=0000W/d0`0000Qo<`00001o<`00Oc@007/d0P0000Ag
=@00K_I^mP8000Aoo`03001oogoo00=oo`03001oogoo009oo`005Woo0P004Woo00D003IM0012F`00
009:>@03001>>4hh00@00003EUP005Ih009FN0800003FYP005jH009NV005001N^EjiG[T00003H]T0
1@00H_URnF;i00000f;j0P0000US6`00Ha/005lK001O6`00Fa`00`0000=Fo@0000002P0000<Eo000
00000P0000@E^`000000008b6`@00003CWX005ji009N^@8000=Zn003001c5P00009c5P800005Nc@0
07/d001k=00300001Glc001o<`00Oc@00P0000Ik=@00McD006kf0002K_L01@00Oomoogoo00001Woo
00<007ooOol00Goo0004Ool20004Ool20003Ool30004Ool00`00Oomoo`0BOol50002BST400001U9H
001FF5IH001FN08000=JV00D001NV000G[P005ji001Rf000H]T006;I001Rn@00I_T006;j001W6P80
009S6`80009O6`80009K7080009Fo@03001:oP0000D0008R?@8000032K/000fK008000057M/00000
<Q/00002?S/02000CWY>NP00G[T005ji0002J_P200002G<F001c5P00MaH007/d001k=002000017/d
000007lc0P0000Eo=000Ne@007/e00800004McH006kgK_L20004Ool00`00Oomoo`04Ool10001Ool1
Ool000=oo`04001oogoo0003Ool20006Ool01000Oomoogoo0P0057oo0P0000<f6`0000000P0000E:
>@00DUQBF000009FF0800003FWP005ZH009JV08000=N^005001Rf6;HH]P00003H]T00`00I_UVn@02
00001VLJ001W6P00Ha/0009S6P80009S6`800004Fa`005KmE_d20002B_h01000?[h002in0`0000<A
o00000000P0000<=V`007M/010000Shk00@004ij000000=N^@80009Zn007001c5P00MaH007Lf0000
0W/d0P0000Mk=000000007lc001o=002000027]E001g=@00McH006kg0002J_P00`00Oomoo`030002
Ool01000Oomoo`000Woo0003Ool01000Oomoo`002goo00D007ooOomoo`0001Ioo`800004=Q/0048j
0002BST00`00DUQBF00200002EIh001JN5Yh001JV000GYP00002G[P20002H]P20002H]T200001FKi
001VnFKi00000fLJ0P0000US6P00IaX006<K001S6`00Fa`00`0000=Fo@000000300000<5NP000000
0`0000<Yn`0000000`0000A>VP00G[UN^@80009Zn0800005LaH007LF001g=P03000017/d000007mD
0P0000=o<`00Oc@00P0000=kE@00McD00P0000=c=`00J_P00V[h0`0000=oo`0000000goo0P000goo
0003Ool01000Oomoo`0027oo0`000goo00<007ooOol04Woo0`0000=oo`0000000P0000E2>P00BST0
059H0080009FN003001JN5Yh008000=NV080009R^080009Rf080009Vn@04001VnFKiI_T200001FLJ
001W6VLJ00000VLJ0P000V<K0P000U/L0P0000AFo@00B_i:oPH000037Sd0000000D0008E^`<00003
?S/004jJ009>VP800004G]T006[h0002KaP00`00MaIg5P020002McH2000017/d000007mD100000Uk
E000001kE@00McH007<g00000V[h0P0017oo00<007ooOol01Woo0003Ool01000Oomoo`0027oo00<0
07ooOol00goo0P0047oo0`0017oo0P0000<Uf`000000100000=BF000EWP00UIh0P0000=JV000GYP0
0UjH00D006:hH[QR^00000=Rf004001VnFKiI_T200001FKi001VnFKi00000VLJ0P0000IW6P00IaX0
06<K0002Ha/200000e/L001Fo@0300001T[n000n_Sjn000^OP<000032M`0000000D0008Un`<00003
CYX0000000=Nf@04001_66lHKaP3000017Lf001g=WLf0P0000Mk=000000007mD001oM00200000g]E
001kE@0200000g<g001c>0020002I]P01@00Oomoogoo000027oo0004Ool20009Ool40002Ool00`00
Oomoo`0<Ool30008Ool30002?QX00`00BST00002DUP01000EWP000000eZH00l005jH001N^000H[P0
06:h001Rf000I]P006Ki00000VKi00<006KiI_T00P0000EW6P00IaYW6P0000=W6P800005Ha/006<K
001K70020002E_d<00001P5j0000005J000=V`<00005>U`00000CY/00003G]T01000KaQ_60000VlH
00H007Lf001g=P00NeH200001GmD00000000Og@00`0000UkE@00001kMP00LcL007<h00000fKH00D0
07ooOomoo`0000Qoo`005Goo0P002Woo0`0037oo0`000ShJ00@004Xi001BF0800003EWP005ZH009J
V08000=N^005001R^6:hH[P00002I]P20002I_T2000016Ki001[6F/I0P0000MW6P00IaX006LJ001W
6P020002Ha/200001U/L00000000E_d0009:oPH000037Sd0000000H000057O`00000>U`00002CY/0
1`00G]T006;j001_6000009_60800005McH007]F001kEP02000017mD000007md100000YkE@00001g
M`00LcP006li001Vf08000Aoo`03001oogoo00Eoo`005Goo00<007ooOol01Woo0`0047oo00<0015j
00001@0000UBF000EWQFN000FYP005jH00000Ujh0P000V:h0P0000IVf000I_P006Ki0002J_P20000
1F/I001[6F/I00000VLJ0P0000AW6P00IaYW6P800003Ha/00000009K7003001Fo@0000800004B_h0
03jn0002;Yh200022M`200001@4i000000007O`01`000V;j0P000VlH00P007<h001g=P00NeH007]F
0`0000=oE000Og@01@0000=kMP00MgL00P000Vli0P000UkI00D007ooOomoo`0000Moo`005Goo0P00
17oo0`0057oo0P0000@enP00BST0009BF003001FN5Ih008000=NV080009R^080009Vf005001Vn6Kh
I_P00003J_P00`00J_P00002JaT2000016LJ001[6V/J0P0000MW6P00001S6`00Ha/00002Fa`20000
0eKm0000000B0002=U`02P00C[a>_000H_X006;j001_60000W<h0P0000EkEP00NeH007]f00<00006
Og@00000OiD007^F0P0000UgM`00MgT006li001W6P00G]T00P000goo00<007ooOol01Goo000DOol0
1000Oomoogoo0`005goo0`0000@enP00BST0009BF0800004FWP005jHGYP20003H[P01@00I]QVf6KH
00000fKh0P0000=Zn000J_P00P0000A[6@00JaU[6@800005JaX006/jJcX00002IcX200001F<K0000
0000Fa`00P000UKm00<004[n00005P000V;j0P000W<h00@007=H001kEP800003NgH007]f00800007
Og@00000001kUP00NkL00P0000AgN@00LiX0009W6P800003F[X007oo009oo`03001oogoo00Eoo`00
3Woo0`000goo10006goo100000=:>@0000000P000eYh0P0000=NV000H[P00V:h0P0000IVf000I]P0
06Kh0002J_P00`00J_P00002JaP01`00JaU[6F/I001[6@00009[>P800003IcX006Lk0080009S>`80
009K70@00007B_h004;N@]h003:N00000R9N100000L16000000Ag000<Ud00002C]`03P00H_X006<K
001c>000LeP007=H001kEP00NgH200001G^F00000000OiD00P0000Ik]`00001gf@00LiX2000016LJ
001O6`000UZj00@007ooOol000Moo`003goo00<007ooOol00Woo0P007Woo00P002gj;OX004Hi001B
N@000eYh00L005jHGYQNV000H[P00002H[P01@00I]QVf6KH00000V[h00<006[h00000f/H0P0000=[
6@00JaT00P0000=[>P00JcX00P0000eW>`00Ic/006<k001S>`00Fa`005/L001Fo@0I0003Ha/20000
0g=H001cF00300001G]f00000000NiH0100000=k]`000000100000=[W00000000elK0`000Woo00<0
07ooOol01Goo000?Ool00`00Oomoo`04Ool00`00Oomoo`0KOol2000012gj0016>@000U9i00@005Yh
000000=NV080009R^080009Vf0800004J_P006[hJ_P200001f/H001[66/H001[6@00009[>@03001[
>P00009[>P05001W>fLkIc/00002Hc/20002Fa`200000eKm0000000900024Qd400001@Vk000^GBiM
00000TkM00@006<K001W>`800007LeP007Mh001gV@00NgH0100000=ke`0000000P0000Agf@00001S
o0800003Ggh0000000<00005DYX007ooOol00007Ool000moo`03001oogoo00Eoo`03001oogoo01]o
o`@00003AST0059i0080009JV007001NV000H[P006:h00000VKH0P000V[h00@006[hJ_P0009Zn006
001[66/H001_60000f/i0P000f/j0P0000=W>`00Ic/00P0000ES?000000005/L00800004E_d004km
C_dF00000fLk001WG00200001GMh001gV@00MkT00P0000=kUP0000000P0000=gn00000001`0000E?
G`00D]aBg000009BVP04001oogoo0006Ool000moo`03001oogoo00Ioo`03001oogoo01]oo`03000U
^P0000<00004DWT005ZHFYP20003H[P01@00I]QVf6KH00000f[h0P0000=Zn000J_P00P0000A[6000
KaQ_60800003JcT006/i00800003JcX006/j00800003Ic/006Lk0080009O?0800003Fa`005Km00<0
0004C_d004KNA]h300000bjN000000090002C_h200001FML001WO000MgP00P0000=g^@00MkT00`00
00=ke`0000004P000Woo00<007ooOol017oo000>Ool20009Ool2000LOol200000dHj001BN@02DWT2
000015jH001R^6:h0P000fKH0P0000MZn000J_QZn000J_P00002KaP2000026lH001_6FlI001[>@00
KcT200003V/j001[>V/j001W>`00Ic/005ll001O?000Fa`20002Ead500000dKN000ngP0>00000d/O
00000002Ig`300000gNi001g^@090004Ool00`00Oomoo`0700001DYk001oogoo00001Woo000KOol0
0`00Oomoo`0IOol01P006IX004Hj001BN@8000=NV080009R^003001Vf000009Vf006001Zn6[h001Z
n0000flH00<006lH00000VlI00D006li001_>@00009_>@800003JcX006/j00800003Ic/00000009O
?0800007Fa`00000001G7@00C_d04@0000HJOP00001;?`00Ig`20002Imd00`00MkT000070007Ool0
0`00Oomoo`02Ool20002;Ud300000goo001oo`05Ool001aoo`03001oogoo01Uoo`03000IVP000080
009BN@04001NV0000003H[P20002I]P2000016[h001Zn6[h0P0000M_6000KaQ_6000KaT00002KcT3
000026li001_>Vlj001[>P00Je/200001F<k00000000Gc`00P000U/L10000Tkm5`0000=Wg@000000
1`002Woo00<007ooOol00goo0`000Shk0P001Woo000MOol00`00Oomoo`0HOol30000148j001BNE9i
0P000Uji00L006:h001Vf000I]P00002J_P01000J_QZn0000VlH00D006lHKaP006lH00800006KcT0
06li001_>@000Vlj00D006mJ001[F`00009WF`03001S?6<l00800007Gc`00000001K7@00Ead00`00
00=>o@00B_h04@000SmO0`0000=Oo@0000000`003Goo00<007ooOol00goo1P001Woo000NOol2000I
Ool00`002GX00003000015JJ001N^Eji0P000fKH00@006[hJ_QZn0800003J_P006lH0080000:KaP0
06lHKaP006lh001_>@00KcT200001Flj001_FP00JeX00P0000IWF`00001S?000Hc`2000015ll001K
7E/M0P0000=G7@0000000`0000=>oP0000005P0047oo00<007ooOol01Goo00@002gk;O/000Ioo`00
87oo00<007ooOol05goo0P0000=2F`00EYX00UJJ0P0000=N^@00I]P00VKH0P0000MZn000J_QZn000
KaP00002KaP2000026lH001_>000KcT006li0002KeT00`00KeX00002JeX3000026MK000006<l001S
?000Gc`300000eLM00000002Ead01000D_d000000Tkm00<004[n0000100000=7?`000000100000=?
W`0000000`004goo00<007ooOol01Goo10001Goo000QOol00`00Oomoo`0GOol200000d9K001FVP02
0003G[T00`00I]P00002I]P01P00J_QZn000J_P000=_6008001_6000KcP006li001_F@800003KeT0
06mJ00800004JeX00000Jg/200001FML00000000Hc`00P000U/m0P0000=G7@0000000`0000=Bo@00
00000`0000=?7P0000000`0000=??`0000001`005Woo00<007ooOol01Woo0P001Goo000ROol00`00
Oomoo`0FOol500000eJJ001N^@02G[T20002I]P2000016[h001Zn6[h0P0000Q_6000KaP006lh001_
>@000VmI00H007=I001_FP00KeX300001F]k001[Nf]k00000VML0P0000EO?000000005/m00800004
Ead005LMEad200000e;m0000000300000e<N00000004000015]N000005nN0`0067oo00<007ooOol0
1Woo00<007ooOol00goo000SOol2000GOol200002ciK001FVUJJ001N^@00H]T006KH00000VKh00@0
06[hJ_P0009_6003001_6000009_>0<00008KeT007=I001cN@00KeX0009_NP<00003Jg/006Ml0080
0004He`00000Gcd300000e/M0000000300001ELM000005;m00000U<M0P0000EG7@00000005Lm0080
0005Ged00000Hid0000IOol00`00Oomoo`06Ool00`00Oomoo`03Ool002Eoo`03001oogoo01Aoo`D0
0003EYX00000009Rf@8000=Vn0800006J_P006lH001_60000Vlh00L006lh001cF@00LeT00002LgT3
000016mj000006nK0P0000=WO000Ig`00P0000=OG@0000000U/m0P0000AG7@00EaeG7@800003Ead0
000000<00004Ead00000Ead3000015/m000006=M0P006goo00<007ooOol01Goo00<007ooOol00goo
000VOol00`00Oomoo`0DOol300001SYL001F^P00H]T0009Rf@04001Vn000J_T2000016lH001_66lH
0P0000I_>000001cF@00LeT200000g=i001cNP04000026nK001[W6^L001WO000Hgd200001EmM0000
0000Fcd01@0000EG7@00001G7@00009G7@800004Ead00000Fcd300000elm0000000NOol00`00Oomo
o`04Ool00`00Oomoo`03Ool002Moo`8001Ioo`<00003E[X0000000=Rf@800005J_T006/IJaT00002
KaP01@00KcU_>@00KcP00P000W=I00@007=i001cNP80009_V`<00003Ji`0000000<00003Ggh00000
009KGP800003Ech0000000<00003Dad0000000<00004Ead00000Ead20002Fcd2000QOol00`00Oomo
o`03Ool00`00Oomoo`03Ool002Uoo`03001oogoo01Aoo`03000fG@00009Ff`03001Rf@00009RnP04
001[6F/IJaT200001FlH001_>@00KeT00`0000EcF@00LgT007>J00<00004Ki/00000Kk`300000fNm
001O_P0200001E]n00000000Eeh00P0000AC?P00DaeC7@800003Dad005LM00<00007Ead00000Fad0
05/m00008goo00<007ooOol017oo00<007ooOol00Woo000ZOol00`00Oomoo`0DOol300000eKK0000
0003H_X200000f/I001[>@0200003Fli001_F@00LeT007=i001cN@00LiX007>J00H00003Jm`00000
00<00003Fkh0000000@00003Del0000000<00003Dah0000000800007Ead00000001G7@00Fad00P00
9Goo00<007ooOol00goo00<007ooOol00Woo000[Ool00`00Oomoo`0COol20000239m001FfeKK001R
nP00IaX2000036/i001_>Fli001_F@00LeT007=i001cVP<00004LiX00000Km/200000f_l0000000C
00000dlN000000020002Ead4000XOol00`00Oomoo`02Ool00`00Oomoo`02Ool002aoo`8001Aoo`<0
009Ff`800005IaX006LjIcX00003KcT300000g=I00000002LiX200000g>k0000000K00000dlN0000
0003000/Ool00`00Oomoo`02Ool00`00Oomoo`02Ool002ioo`03001oogoo019oo`05000^OP00001J
o00300001FLj001WF`00KcT00P0000=_NP0000000P0000AcVP00001c^`<00003Jo`0000000@000]o
o`X0039oo`05001oogooOol00004Ool002moo`03001oogoo019oo`@0009K7@80009WF`<00004KgX0
06nJKiX=001;Ool01000Oomoo`0017oo000`Ool00`00Oomoo`0COol400001e/m001WF`00Je/006]l
00800004KiX00000Kk/200000f_l00000004001>Ool00`00Ool00004Ool0035oo`8001Aoo`<00003
Fcd0000000800004Jg`006^LJi`;001@Ool01000Oomoo`000goo000cOol00`00Oomoo`0COol40000
1EMo00000000Ji`00`0000=So@0000001@00DWoo00<007oo00000goo000dOol00`00Oomoo`0COol4
0002Dil9000016]k001_FP00Dgoo0P000goo000eOol00`00Oomoo`0EOol900001V;k00000000KeX0
05=oo`<000=oo`00=Woo0P005Woo00D0059hDWP005JJ00800006H_/006LI001[6000D7oo10001Woo
000hOol00`00Oomoo`0COol00`00DWQBN0020002G[T00`00IaT00002JaP00`00Oomoo`1:Ool4000:
Ool003Uoo`03001oogoo01=oo`800005FWP005ji001R^00200000f/H001oo`18Ool4000>Ool003Yo
o`03001oogoo019oo`05001JN5YhFWP00002H[P00`00I]L00016Ool4000BOol003]oo`8001=oo`06
001JN000001R^0000VKG00<007ooOol0@7oo10005Woo000mOol00`00Oomoo`0AOol01000GYMNU`00
0VKG00<007ooOol0?Woo0`006Woo000nOol00`00Oomoo`0@Ool01P00GYL00000I]L003eoo`@001eo
o`00?goo00<007ooOol047oo00@006:GHYL003Yoo`@0025oo`00@7oo0P0047oo00@006:GHYL003Io
o`@002Eoo`00@Woo00<007ooOol03Woo0P00<goo1000:Goo0013Ool00`00Oomoo`0=Ool00`00Oomo
o`0^Ool4000]Ool004Aoo`8000eoo`03001oogoo02Yoo`@0035oo`00AWoo00<007ooOol02Woo00<0
07ooOol09Woo1000=Goo0017Ool00`00Oomoo`09Ool00`00Oomoo`0ROol4000iOol004Qoo`03001o
ogoo00Qoo`03001oogoo01ioo`@003eoo`00BGoo0P0027oo00<007ooOol06Woo1000@Goo001;Ool0
0`00Oomoo`05Ool00`00Oomoo`0FOol40015Ool004aoo`03001oogoo00Aoo`03001oogoo01=oo`<0
04Uoo`00CGoo00<007ooOol00goo00<007ooOol03goo1000C7oo001>Ool20003Ool00`00Oomoo`0;
Ool4001@Ool0051oo`04001oogoo0009Ool4001DOol0055oo`03001oo`0000Eoo`@005Qoo`00DWoo
0P0000=oo`0000000P00G7oo001COol2001POol00;Eoo`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.127938, -0.0269615, 
  0.00609341, 0.00609341}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[func[x, \ y], \ {x, \ \(-2\), \ 2}, \ {y, \ \(-2\), \ 2}, 
      \ PlotPoints\  -> \ 40]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.240385 0.5 0.240385 [
[.01923 -0.0125 -6 -9 ]
[.01923 -0.0125 6 0 ]
[.25962 -0.0125 -6 -9 ]
[.25962 -0.0125 6 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.74038 -0.0125 -3 -9 ]
[.74038 -0.0125 3 0 ]
[.98077 -0.0125 -3 -9 ]
[.98077 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01923 -12 -4.5 ]
[-0.0125 .01923 0 4.5 ]
[-0.0125 .25962 -12 -4.5 ]
[-0.0125 .25962 0 4.5 ]
[-0.0125 .5 -6 -4.5 ]
[-0.0125 .5 0 4.5 ]
[-0.0125 .74038 -6 -4.5 ]
[-0.0125 .74038 0 4.5 ]
[-0.0125 .98077 -6 -4.5 ]
[-0.0125 .98077 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 1 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01923 0 m
.01923 .00625 L
s
[(-2)] .01923 -0.0125 0 1 Mshowa
.25962 0 m
.25962 .00625 L
s
[(-1)] .25962 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(0)] .5 -0.0125 0 1 Mshowa
.74038 0 m
.74038 .00625 L
s
[(1)] .74038 -0.0125 0 1 Mshowa
.98077 0 m
.98077 .00625 L
s
[(2)] .98077 -0.0125 0 1 Mshowa
.125 Mabswid
.06731 0 m
.06731 .00375 L
s
.11538 0 m
.11538 .00375 L
s
.16346 0 m
.16346 .00375 L
s
.21154 0 m
.21154 .00375 L
s
.30769 0 m
.30769 .00375 L
s
.35577 0 m
.35577 .00375 L
s
.40385 0 m
.40385 .00375 L
s
.45192 0 m
.45192 .00375 L
s
.54808 0 m
.54808 .00375 L
s
.59615 0 m
.59615 .00375 L
s
.64423 0 m
.64423 .00375 L
s
.69231 0 m
.69231 .00375 L
s
.78846 0 m
.78846 .00375 L
s
.83654 0 m
.83654 .00375 L
s
.88462 0 m
.88462 .00375 L
s
.93269 0 m
.93269 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01923 m
.00625 .01923 L
s
[(-2)] -0.0125 .01923 1 0 Mshowa
0 .25962 m
.00625 .25962 L
s
[(-1)] -0.0125 .25962 1 0 Mshowa
0 .5 m
.00625 .5 L
s
[(0)] -0.0125 .5 1 0 Mshowa
0 .74038 m
.00625 .74038 L
s
[(1)] -0.0125 .74038 1 0 Mshowa
0 .98077 m
.00625 .98077 L
s
[(2)] -0.0125 .98077 1 0 Mshowa
.125 Mabswid
0 .06731 m
.00375 .06731 L
s
0 .11538 m
.00375 .11538 L
s
0 .16346 m
.00375 .16346 L
s
0 .21154 m
.00375 .21154 L
s
0 .30769 m
.00375 .30769 L
s
0 .35577 m
.00375 .35577 L
s
0 .40385 m
.00375 .40385 L
s
0 .45192 m
.00375 .45192 L
s
0 .54808 m
.00375 .54808 L
s
0 .59615 m
.00375 .59615 L
s
0 .64423 m
.00375 .64423 L
s
0 .69231 m
.00375 .69231 L
s
0 .78846 m
.00375 .78846 L
s
0 .83654 m
.00375 .83654 L
s
0 .88462 m
.00375 .88462 L
s
0 .93269 m
.00375 .93269 L
s
.25 Mabswid
0 0 m
0 1 L
s
.01923 .99375 m
.01923 1 L
s
.25962 .99375 m
.25962 1 L
s
.5 .99375 m
.5 1 L
s
.74038 .99375 m
.74038 1 L
s
.98077 .99375 m
.98077 1 L
s
.125 Mabswid
.06731 .99625 m
.06731 1 L
s
.11538 .99625 m
.11538 1 L
s
.16346 .99625 m
.16346 1 L
s
.21154 .99625 m
.21154 1 L
s
.30769 .99625 m
.30769 1 L
s
.35577 .99625 m
.35577 1 L
s
.40385 .99625 m
.40385 1 L
s
.45192 .99625 m
.45192 1 L
s
.54808 .99625 m
.54808 1 L
s
.59615 .99625 m
.59615 1 L
s
.64423 .99625 m
.64423 1 L
s
.69231 .99625 m
.69231 1 L
s
.78846 .99625 m
.78846 1 L
s
.83654 .99625 m
.83654 1 L
s
.88462 .99625 m
.88462 1 L
s
.93269 .99625 m
.93269 1 L
s
.25 Mabswid
0 1 m
1 1 L
s
.99375 .01923 m
1 .01923 L
s
.99375 .25962 m
1 .25962 L
s
.99375 .5 m
1 .5 L
s
.99375 .74038 m
1 .74038 L
s
.99375 .98077 m
1 .98077 L
s
.125 Mabswid
.99625 .06731 m
1 .06731 L
s
.99625 .11538 m
1 .11538 L
s
.99625 .16346 m
1 .16346 L
s
.99625 .21154 m
1 .21154 L
s
.99625 .30769 m
1 .30769 L
s
.99625 .35577 m
1 .35577 L
s
.99625 .40385 m
1 .40385 L
s
.99625 .45192 m
1 .45192 L
s
.99625 .54808 m
1 .54808 L
s
.99625 .59615 m
1 .59615 L
s
.99625 .64423 m
1 .64423 L
s
.99625 .69231 m
1 .69231 L
s
.99625 .78846 m
1 .78846 L
s
.99625 .83654 m
1 .83654 L
s
.99625 .88462 m
1 .88462 L
s
.99625 .93269 m
1 .93269 L
s
.25 Mabswid
1 0 m
1 1 L
s
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
.4 g
.01923 .98077 m
.98077 .98077 L
.98077 .01923 L
.01923 .01923 L
F
0 g
.5 Mabswid
.5 g
.01923 .11081 m
.04389 .10256 L
.05375 .0932 L
.06293 .06854 L
.06629 .04389 L
.06802 .01923 L
.01923 .01923 L
F
0 g
.01923 .11081 m
.04389 .10256 L
.05375 .0932 L
.06293 .06854 L
.06629 .04389 L
.06802 .01923 L
s
.5 g
.01923 .88919 m
.04389 .89744 L
.05375 .9068 L
.06293 .93146 L
.06629 .95611 L
.06802 .98077 L
.01923 .98077 L
F
0 g
.01923 .88919 m
.04389 .89744 L
.05375 .9068 L
.06293 .93146 L
.06629 .95611 L
.06802 .98077 L
s
.3 g
.01923 .83587 m
.02356 .83284 L
.04271 .80819 L
.04389 .80555 L
.0509 .78353 L
.05547 .75888 L
.05836 .73422 L
.06034 .70957 L
.06175 .68491 L
.06279 .66026 L
.06357 .6356 L
.06415 .61095 L
.06457 .58629 L
.06487 .56164 L
.06506 .53698 L
.06515 .51233 L
.06515 .48767 L
.06506 .46302 L
.06487 .43836 L
.06457 .41371 L
.06415 .38905 L
.06357 .3644 L
.06279 .33974 L
.06175 .31509 L
.06034 .29043 L
.05836 .26578 L
.05547 .24112 L
.0509 .21647 L
.04389 .19445 L
.04271 .19181 L
.02356 .16716 L
.01923 .16413 L
F
0 g
.01923 .83587 m
.02356 .83284 L
.04271 .80819 L
.04389 .80555 L
.0509 .78353 L
.05547 .75888 L
.05836 .73422 L
.06034 .70957 L
.06175 .68491 L
.06279 .66026 L
.06357 .6356 L
.06415 .61095 L
.06457 .58629 L
.06487 .56164 L
.06506 .53698 L
.06515 .51233 L
.06515 .48767 L
.06506 .46302 L
.06487 .43836 L
.06457 .41371 L
.06415 .38905 L
.06357 .3644 L
.06279 .33974 L
.06175 .31509 L
.06034 .29043 L
.05836 .26578 L
.05547 .24112 L
.0509 .21647 L
.04389 .19445 L
.04271 .19181 L
.02356 .16716 L
.01923 .16413 L
s
.2 g
.01923 .78033 m
.0299 .75888 L
.03747 .73422 L
.04243 .70957 L
.04389 .70017 L
.04588 .68491 L
.04838 .66026 L
.05022 .6356 L
.05159 .61095 L
.05258 .58629 L
.05328 .56164 L
.05372 .53698 L
.05394 .51233 L
.05394 .48767 L
.05372 .46302 L
.05328 .43836 L
.05258 .41371 L
.05159 .38905 L
.05022 .3644 L
.04838 .33974 L
.04588 .31509 L
.04389 .29983 L
.04243 .29043 L
.03747 .26578 L
.0299 .24112 L
.01923 .21967 L
F
0 g
.01923 .78033 m
.0299 .75888 L
.03747 .73422 L
.04243 .70957 L
.04389 .70017 L
.04588 .68491 L
.04838 .66026 L
.05022 .6356 L
.05159 .61095 L
.05258 .58629 L
.05328 .56164 L
.05372 .53698 L
.05394 .51233 L
.05394 .48767 L
.05372 .46302 L
.05328 .43836 L
.05258 .41371 L
.05159 .38905 L
.05022 .3644 L
.04838 .33974 L
.04588 .31509 L
.04389 .29983 L
.04243 .29043 L
.03747 .26578 L
.0299 .24112 L
.01923 .21967 L
s
.1 g
.01923 .71849 m
.02215 .70957 L
.02839 .68491 L
.03278 .66026 L
.03596 .6356 L
.03828 .61095 L
.03996 .58629 L
.04113 .56164 L
.04187 .53698 L
.04223 .51233 L
.04223 .48767 L
.04187 .46302 L
.04113 .43836 L
.03996 .41371 L
.03828 .38905 L
.03596 .3644 L
.03278 .33974 L
.02839 .31509 L
.02215 .29043 L
.01923 .28151 L
F
0 g
.01923 .71849 m
.02215 .70957 L
.02839 .68491 L
.03278 .66026 L
.03596 .6356 L
.03828 .61095 L
.03996 .58629 L
.04113 .56164 L
.04187 .53698 L
.04223 .51233 L
.04223 .48767 L
.04187 .46302 L
.04113 .43836 L
.03996 .41371 L
.03828 .38905 L
.03596 .3644 L
.03278 .33974 L
.02839 .31509 L
.02215 .29043 L
.01923 .28151 L
s
.6 g
.01923 .05689 m
.02939 .04389 L
.0405 .01923 L
.01923 .01923 L
F
0 g
.01923 .05689 m
.02939 .04389 L
.0405 .01923 L
s
.6 g
.01923 .94311 m
.02939 .95611 L
.0405 .98077 L
.01923 .98077 L
F
0 g
.01923 .94311 m
.02939 .95611 L
.0405 .98077 L
s
.01923 .63982 m
.01997 .6356 L
.02355 .61095 L
.02611 .58629 L
.02788 .56164 L
.02899 .53698 L
.02953 .51233 L
.02953 .48767 L
.02899 .46302 L
.02788 .43836 L
.02611 .41371 L
.02355 .38905 L
.01997 .3644 L
.01923 .36018 L
F
.01923 .63982 m
.01997 .6356 L
.02355 .61095 L
.02611 .58629 L
.02788 .56164 L
.02899 .53698 L
.02953 .51233 L
.02953 .48767 L
.02899 .46302 L
.02788 .43836 L
.02611 .41371 L
.02355 .38905 L
.01997 .3644 L
.01923 .36018 L
s
.5 g
.11785 .13764 m
.1425 .13336 L
.16716 .13174 L
.19181 .1312 L
.21647 .13132 L
.24112 .13198 L
.26578 .1332 L
.29043 .13515 L
.31509 .13815 L
.33833 .1425 L
.33974 .14284 L
.3644 .15051 L
.38905 .16412 L
.39283 .16716 L
.41371 .19164 L
.41381 .19181 L
.42521 .21647 L
.43305 .24112 L
.43836 .26402 L
.43862 .26578 L
.44182 .29043 L
.4445 .31509 L
.44672 .33974 L
.44854 .3644 L
.44999 .38905 L
.45111 .41371 L
.45193 .43836 L
.45246 .46302 L
.45273 .48767 L
.45273 .51233 L
.45246 .53698 L
.45193 .56164 L
.45111 .58629 L
.44999 .61095 L
.44854 .6356 L
.44672 .66026 L
.4445 .68491 L
.44182 .70957 L
.43862 .73422 L
.43836 .73598 L
.43305 .75888 L
.42521 .78353 L
.41381 .80819 L
.41371 .80836 L
.39283 .83284 L
.38905 .83588 L
.3644 .84949 L
.33974 .85716 L
.33833 .8575 L
.31509 .86185 L
.29043 .86485 L
.26578 .8668 L
.24112 .86802 L
.21647 .86868 L
.19181 .8688 L
.16716 .86826 L
.1425 .86664 L
.11785 .86236 L
.10561 .8575 L
.0932 .84549 L
.08758 .83284 L
.08269 .80819 L
.08043 .78353 L
.07914 .75888 L
.07832 .73422 L
.07775 .70957 L
.07735 .68491 L
.07705 .66026 L
.07683 .6356 L
.07667 .61095 L
.07654 .58629 L
.07646 .56164 L
.0764 .53698 L
.07638 .51233 L
.07638 .48767 L
.0764 .46302 L
.07646 .43836 L
.07654 .41371 L
.07667 .38905 L
.07683 .3644 L
.07705 .33974 L
.07735 .31509 L
.07775 .29043 L
.07832 .26578 L
.07914 .24112 L
.08043 .21647 L
.08269 .19181 L
.08758 .16716 L
.0932 .15451 L
.10561 .1425 L
F
0 g
.11785 .13764 m
.1425 .13336 L
.16716 .13174 L
.19181 .1312 L
.21647 .13132 L
.24112 .13198 L
.26578 .1332 L
.29043 .13515 L
.31509 .13815 L
.33833 .1425 L
.33974 .14284 L
.3644 .15051 L
.38905 .16412 L
.39283 .16716 L
.41371 .19164 L
.41381 .19181 L
.42521 .21647 L
.43305 .24112 L
.43836 .26402 L
.43862 .26578 L
.44182 .29043 L
.4445 .31509 L
.44672 .33974 L
.44854 .3644 L
.44999 .38905 L
.45111 .41371 L
.45193 .43836 L
.45246 .46302 L
.45273 .48767 L
.45273 .51233 L
.45246 .53698 L
.45193 .56164 L
.45111 .58629 L
.44999 .61095 L
.44854 .6356 L
.44672 .66026 L
.4445 .68491 L
.44182 .70957 L
.43862 .73422 L
.43836 .73598 L
.43305 .75888 L
.42521 .78353 L
.41381 .80819 L
.41371 .80836 L
.39283 .83284 L
.38905 .83588 L
.3644 .84949 L
.33974 .85716 L
.33833 .8575 L
.31509 .86185 L
Mistroke
.29043 .86485 L
.26578 .8668 L
.24112 .86802 L
.21647 .86868 L
.19181 .8688 L
.16716 .86826 L
.1425 .86664 L
.11785 .86236 L
.10561 .8575 L
.0932 .84549 L
.08758 .83284 L
.08269 .80819 L
.08043 .78353 L
.07914 .75888 L
.07832 .73422 L
.07775 .70957 L
.07735 .68491 L
.07705 .66026 L
.07683 .6356 L
.07667 .61095 L
.07654 .58629 L
.07646 .56164 L
.0764 .53698 L
.07638 .51233 L
.07638 .48767 L
.0764 .46302 L
.07646 .43836 L
.07654 .41371 L
.07667 .38905 L
.07683 .3644 L
.07705 .33974 L
.07735 .31509 L
.07775 .29043 L
.07832 .26578 L
.07914 .24112 L
.08043 .21647 L
.08269 .19181 L
.08758 .16716 L
.0932 .15451 L
.10561 .1425 L
.11785 .13764 L
Mfstroke
.6 g
.1425 .18418 m
.16716 .17488 L
.19181 .17185 L
.21647 .17254 L
.24112 .17628 L
.26578 .18328 L
.28542 .19181 L
.29043 .19457 L
.31509 .21228 L
.31955 .21647 L
.33974 .24107 L
.33978 .24112 L
.35352 .26578 L
.36356 .29043 L
.3644 .29286 L
.37106 .31509 L
.37684 .33974 L
.38133 .3644 L
.3848 .38905 L
.3874 .41371 L
.38905 .43484 L
.38926 .43836 L
.39043 .46302 L
.39101 .48767 L
.39101 .51233 L
.39043 .53698 L
.38926 .56164 L
.38905 .56516 L
.3874 .58629 L
.3848 .61095 L
.38133 .6356 L
.37684 .66026 L
.37106 .68491 L
.3644 .70714 L
.36356 .70957 L
.35352 .73422 L
.33978 .75888 L
.33974 .75893 L
.31955 .78353 L
.31509 .78772 L
.29043 .80543 L
.28542 .80819 L
.26578 .81672 L
.24112 .82372 L
.21647 .82746 L
.19181 .82815 L
.16716 .82512 L
.1425 .81582 L
.13139 .80819 L
.11785 .79076 L
.11394 .78353 L
.10502 .75888 L
.09974 .73422 L
.09629 .70957 L
.09387 .68491 L
.0932 .67622 L
.09209 .66026 L
.09076 .6356 L
.08977 .61095 L
.08905 .58629 L
.08854 .56164 L
.08822 .53698 L
.08806 .51233 L
.08806 .48767 L
.08822 .46302 L
.08854 .43836 L
.08905 .41371 L
.08977 .38905 L
.09076 .3644 L
.09209 .33974 L
.0932 .32378 L
.09387 .31509 L
.09629 .29043 L
.09974 .26578 L
.10502 .24112 L
.11394 .21647 L
.11785 .20924 L
.13139 .19181 L
F
0 g
.1425 .18418 m
.16716 .17488 L
.19181 .17185 L
.21647 .17254 L
.24112 .17628 L
.26578 .18328 L
.28542 .19181 L
.29043 .19457 L
.31509 .21228 L
.31955 .21647 L
.33974 .24107 L
.33978 .24112 L
.35352 .26578 L
.36356 .29043 L
.3644 .29286 L
.37106 .31509 L
.37684 .33974 L
.38133 .3644 L
.3848 .38905 L
.3874 .41371 L
.38905 .43484 L
.38926 .43836 L
.39043 .46302 L
.39101 .48767 L
.39101 .51233 L
.39043 .53698 L
.38926 .56164 L
.38905 .56516 L
.3874 .58629 L
.3848 .61095 L
.38133 .6356 L
.37684 .66026 L
.37106 .68491 L
.3644 .70714 L
.36356 .70957 L
.35352 .73422 L
.33978 .75888 L
.33974 .75893 L
.31955 .78353 L
.31509 .78772 L
.29043 .80543 L
.28542 .80819 L
.26578 .81672 L
.24112 .82372 L
.21647 .82746 L
.19181 .82815 L
.16716 .82512 L
.1425 .81582 L
.13139 .80819 L
.11785 .79076 L
Mistroke
.11394 .78353 L
.10502 .75888 L
.09974 .73422 L
.09629 .70957 L
.09387 .68491 L
.0932 .67622 L
.09209 .66026 L
.09076 .6356 L
.08977 .61095 L
.08905 .58629 L
.08854 .56164 L
.08822 .53698 L
.08806 .51233 L
.08806 .48767 L
.08822 .46302 L
.08854 .43836 L
.08905 .41371 L
.08977 .38905 L
.09076 .3644 L
.09209 .33974 L
.0932 .32378 L
.09387 .31509 L
.09629 .29043 L
.09974 .26578 L
.10502 .24112 L
.11394 .21647 L
.11785 .20924 L
.13139 .19181 L
.1425 .18418 L
Mfstroke
.3 g
.36412 .01923 m
.34427 .04389 L
.33974 .04815 L
.31509 .06554 L
.30938 .06854 L
.29043 .07652 L
.26578 .08359 L
.24112 .08801 L
.21647 .09037 L
.19181 .09081 L
.16716 .08889 L
.1425 .08303 L
.11886 .06854 L
.11785 .06741 L
.10322 .04389 L
.0961 .01923 L
F
0 g
.36412 .01923 m
.34427 .04389 L
.33974 .04815 L
.31509 .06554 L
.30938 .06854 L
.29043 .07652 L
.26578 .08359 L
.24112 .08801 L
.21647 .09037 L
.19181 .09081 L
.16716 .08889 L
.1425 .08303 L
.11886 .06854 L
.11785 .06741 L
.10322 .04389 L
.0961 .01923 L
s
.3 g
.36412 .98077 m
.34427 .95611 L
.33974 .95185 L
.31509 .93446 L
.30938 .93146 L
.29043 .92348 L
.26578 .91641 L
.24112 .91199 L
.21647 .90963 L
.19181 .90919 L
.16716 .91111 L
.1425 .91697 L
.11886 .93146 L
.11785 .93259 L
.10322 .95611 L
.0961 .98077 L
F
0 g
.36412 .98077 m
.34427 .95611 L
.33974 .95185 L
.31509 .93446 L
.30938 .93146 L
.29043 .92348 L
.26578 .91641 L
.24112 .91199 L
.21647 .90963 L
.19181 .90919 L
.16716 .91111 L
.1425 .91697 L
.11886 .93146 L
.11785 .93259 L
.10322 .95611 L
.0961 .98077 L
s
.7 g
.19181 .21401 m
.21647 .21533 L
.22214 .21647 L
.24112 .22255 L
.26578 .23629 L
.2722 .24112 L
.29043 .25908 L
.29585 .26578 L
.31158 .29043 L
.31509 .29728 L
.32293 .31509 L
.33145 .33974 L
.33795 .3644 L
.33974 .37251 L
.34288 .38905 L
.34655 .41371 L
.34917 .43836 L
.35085 .46302 L
.35168 .48767 L
.35168 .51233 L
.35085 .53698 L
.34917 .56164 L
.34655 .58629 L
.34288 .61095 L
.33974 .62749 L
.33795 .6356 L
.33145 .66026 L
.32293 .68491 L
.31509 .70272 L
.31158 .70957 L
.29585 .73422 L
.29043 .74092 L
.2722 .75888 L
.26578 .76371 L
.24112 .77745 L
.22214 .78353 L
.21647 .78467 L
.19181 .78599 L
.17706 .78353 L
.16716 .78015 L
.1425 .76194 L
.13981 .75888 L
.12551 .73422 L
.11786 .70957 L
.11785 .70953 L
.11234 .68491 L
.10851 .66026 L
.10576 .6356 L
.10377 .61095 L
.10234 .58629 L
.10135 .56164 L
.10072 .53698 L
.10041 .51233 L
.10041 .48767 L
.10072 .46302 L
.10135 .43836 L
.10234 .41371 L
.10377 .38905 L
.10576 .3644 L
.10851 .33974 L
.11234 .31509 L
.11785 .29047 L
.11786 .29043 L
.12551 .26578 L
.13981 .24112 L
.1425 .23806 L
.16716 .21985 L
.17706 .21647 L
F
0 g
.19181 .21401 m
.21647 .21533 L
.22214 .21647 L
.24112 .22255 L
.26578 .23629 L
.2722 .24112 L
.29043 .25908 L
.29585 .26578 L
.31158 .29043 L
.31509 .29728 L
.32293 .31509 L
.33145 .33974 L
.33795 .3644 L
.33974 .37251 L
.34288 .38905 L
.34655 .41371 L
.34917 .43836 L
.35085 .46302 L
.35168 .48767 L
.35168 .51233 L
.35085 .53698 L
.34917 .56164 L
.34655 .58629 L
.34288 .61095 L
.33974 .62749 L
.33795 .6356 L
.33145 .66026 L
.32293 .68491 L
.31509 .70272 L
.31158 .70957 L
.29585 .73422 L
.29043 .74092 L
.2722 .75888 L
.26578 .76371 L
.24112 .77745 L
.22214 .78353 L
.21647 .78467 L
.19181 .78599 L
.17706 .78353 L
.16716 .78015 L
.1425 .76194 L
.13981 .75888 L
.12551 .73422 L
.11786 .70957 L
.11785 .70953 L
.11234 .68491 L
.10851 .66026 L
.10576 .6356 L
.10377 .61095 L
.10234 .58629 L
Mistroke
.10135 .56164 L
.10072 .53698 L
.10041 .51233 L
.10041 .48767 L
.10072 .46302 L
.10135 .43836 L
.10234 .41371 L
.10377 .38905 L
.10576 .3644 L
.10851 .33974 L
.11234 .31509 L
.11785 .29047 L
.11786 .29043 L
.12551 .26578 L
.13981 .24112 L
.1425 .23806 L
.16716 .21985 L
.17706 .21647 L
.19181 .21401 L
Mfstroke
.8 g
.19181 .25941 m
.21647 .26152 L
.22801 .26578 L
.24112 .27322 L
.2611 .29043 L
.26578 .29628 L
.27892 .31509 L
.29043 .33791 L
.29123 .33974 L
.30029 .3644 L
.30699 .38905 L
.3119 .41371 L
.31509 .43595 L
.31536 .43836 L
.31757 .46302 L
.31865 .48767 L
.31865 .51233 L
.31757 .53698 L
.31536 .56164 L
.31509 .56405 L
.3119 .58629 L
.30699 .61095 L
.30029 .6356 L
.29123 .66026 L
.29043 .66209 L
.27892 .68491 L
.26578 .70372 L
.2611 .70957 L
.24112 .72678 L
.22801 .73422 L
.21647 .73848 L
.19181 .74059 L
.17208 .73422 L
.16716 .73118 L
.14547 .70957 L
.1425 .70066 L
.13542 .68491 L
.12801 .66026 L
.12322 .6356 L
.11997 .61095 L
.11785 .58807 L
.1177 .58629 L
.11597 .56164 L
.11489 .53698 L
.11437 .51233 L
.11437 .48767 L
.11489 .46302 L
.11597 .43836 L
.1177 .41371 L
.11785 .41193 L
.11997 .38905 L
.12322 .3644 L
.12801 .33974 L
.13542 .31509 L
.1425 .29934 L
.14547 .29043 L
.16716 .26882 L
.17208 .26578 L
F
0 g
.19181 .25941 m
.21647 .26152 L
.22801 .26578 L
.24112 .27322 L
.2611 .29043 L
.26578 .29628 L
.27892 .31509 L
.29043 .33791 L
.29123 .33974 L
.30029 .3644 L
.30699 .38905 L
.3119 .41371 L
.31509 .43595 L
.31536 .43836 L
.31757 .46302 L
.31865 .48767 L
.31865 .51233 L
.31757 .53698 L
.31536 .56164 L
.31509 .56405 L
.3119 .58629 L
.30699 .61095 L
.30029 .6356 L
.29123 .66026 L
.29043 .66209 L
.27892 .68491 L
.26578 .70372 L
.2611 .70957 L
.24112 .72678 L
.22801 .73422 L
.21647 .73848 L
.19181 .74059 L
.17208 .73422 L
.16716 .73118 L
.14547 .70957 L
.1425 .70066 L
.13542 .68491 L
.12801 .66026 L
.12322 .6356 L
.11997 .61095 L
.11785 .58807 L
.1177 .58629 L
.11597 .56164 L
.11489 .53698 L
.11437 .51233 L
.11437 .48767 L
.11489 .46302 L
.11597 .43836 L
.1177 .41371 L
.11785 .41193 L
Mistroke
.11997 .38905 L
.12322 .3644 L
.12801 .33974 L
.13542 .31509 L
.1425 .29934 L
.14547 .29043 L
.16716 .26882 L
.17208 .26578 L
.19181 .25941 L
Mfstroke
.9 g
.19181 .31125 m
.21647 .31456 L
.21763 .31509 L
.24112 .33345 L
.24662 .33974 L
.26144 .3644 L
.26578 .37482 L
.2711 .38905 L
.27806 .41371 L
.28278 .43836 L
.28573 .46302 L
.28716 .48767 L
.28716 .51233 L
.28573 .53698 L
.28278 .56164 L
.27806 .58629 L
.2711 .61095 L
.26578 .62518 L
.26144 .6356 L
.24662 .66026 L
.24112 .66655 L
.21763 .68491 L
.21647 .68544 L
.19181 .68875 L
.181 .68491 L
.16716 .67378 L
.1552 .66026 L
.1453 .6356 L
.1425 .61902 L
.14055 .61095 L
.13597 .58629 L
.13299 .56164 L
.1312 .53698 L
.13035 .51233 L
.13035 .48767 L
.1312 .46302 L
.13299 .43836 L
.13597 .41371 L
.14055 .38905 L
.1425 .38098 L
.1453 .3644 L
.1552 .33974 L
.16716 .32622 L
.181 .31509 L
F
0 g
.19181 .31125 m
.21647 .31456 L
.21763 .31509 L
.24112 .33345 L
.24662 .33974 L
.26144 .3644 L
.26578 .37482 L
.2711 .38905 L
.27806 .41371 L
.28278 .43836 L
.28573 .46302 L
.28716 .48767 L
.28716 .51233 L
.28573 .53698 L
.28278 .56164 L
.27806 .58629 L
.2711 .61095 L
.26578 .62518 L
.26144 .6356 L
.24662 .66026 L
.24112 .66655 L
.21763 .68491 L
.21647 .68544 L
.19181 .68875 L
.181 .68491 L
.16716 .67378 L
.1552 .66026 L
.1453 .6356 L
.1425 .61902 L
.14055 .61095 L
.13597 .58629 L
.13299 .56164 L
.1312 .53698 L
.13035 .51233 L
.13035 .48767 L
.1312 .46302 L
.13299 .43836 L
.13597 .41371 L
.14055 .38905 L
.1425 .38098 L
.1453 .3644 L
.1552 .33974 L
.16716 .32622 L
.181 .31509 L
.19181 .31125 L
s
.2 g
.28533 .01923 m
.26578 .03221 L
.24112 .04283 L
.23772 .04389 L
.21647 .04845 L
.19181 .04948 L
.16716 .04493 L
.16329 .04389 L
.1425 .03085 L
.13144 .01923 L
F
0 g
.28533 .01923 m
.26578 .03221 L
.24112 .04283 L
.23772 .04389 L
.21647 .04845 L
.19181 .04948 L
.16716 .04493 L
.16329 .04389 L
.1425 .03085 L
.13144 .01923 L
s
.2 g
.28533 .98077 m
.26578 .96779 L
.24112 .95717 L
.23772 .95611 L
.21647 .95155 L
.19181 .95052 L
.16716 .95507 L
.16329 .95611 L
.1425 .96915 L
.13144 .98077 L
F
0 g
.28533 .98077 m
.26578 .96779 L
.24112 .95717 L
.23772 .95611 L
.21647 .95155 L
.19181 .95052 L
.16716 .95507 L
.16329 .95611 L
.1425 .96915 L
.13144 .98077 L
s
1 g
.19181 .37842 m
.21647 .38452 L
.22123 .38905 L
.23668 .41371 L
.24112 .42478 L
.24537 .43836 L
.25044 .46302 L
.25274 .48767 L
.25274 .51233 L
.25044 .53698 L
.24537 .56164 L
.24112 .57522 L
.23668 .58629 L
.22123 .61095 L
.21647 .61548 L
.19181 .62158 L
.17784 .61095 L
.16716 .59206 L
.16436 .58629 L
.15623 .56164 L
.15222 .53698 L
.15056 .51233 L
.15056 .48767 L
.15222 .46302 L
.15623 .43836 L
.16436 .41371 L
.16716 .40794 L
.17784 .38905 L
F
0 g
.19181 .37842 m
.21647 .38452 L
.22123 .38905 L
.23668 .41371 L
.24112 .42478 L
.24537 .43836 L
.25044 .46302 L
.25274 .48767 L
.25274 .51233 L
.25044 .53698 L
.24537 .56164 L
.24112 .57522 L
.23668 .58629 L
.22123 .61095 L
.21647 .61548 L
.19181 .62158 L
.17784 .61095 L
.16716 .59206 L
.16436 .58629 L
.15623 .56164 L
.15222 .53698 L
.15056 .51233 L
.15056 .48767 L
.15222 .46302 L
.15623 .43836 L
.16436 .41371 L
.16716 .40794 L
.17784 .38905 L
.19181 .37842 L
s
.5 g
.68491 .13815 m
.70957 .13515 L
.73422 .1332 L
.75888 .13198 L
.78353 .13132 L
.80819 .1312 L
.83284 .13174 L
.8575 .13336 L
.88215 .13764 L
.89439 .1425 L
.9068 .15451 L
.91242 .16716 L
.91731 .19181 L
.91957 .21647 L
.92086 .24112 L
.92168 .26578 L
.92225 .29043 L
.92265 .31509 L
.92295 .33974 L
.92317 .3644 L
.92333 .38905 L
.92346 .41371 L
.92354 .43836 L
.9236 .46302 L
.92362 .48767 L
.92362 .51233 L
.9236 .53698 L
.92354 .56164 L
.92346 .58629 L
.92333 .61095 L
.92317 .6356 L
.92295 .66026 L
.92265 .68491 L
.92225 .70957 L
.92168 .73422 L
.92086 .75888 L
.91957 .78353 L
.91731 .80819 L
.91242 .83284 L
.9068 .84549 L
.89439 .8575 L
.88215 .86236 L
.8575 .86664 L
.83284 .86826 L
.80819 .8688 L
.78353 .86868 L
.75888 .86802 L
.73422 .8668 L
.70957 .86485 L
.68491 .86185 L
.66167 .8575 L
.66026 .85716 L
.6356 .84949 L
.61095 .83588 L
.60717 .83284 L
.58629 .80836 L
.58619 .80819 L
.57479 .78353 L
.56695 .75888 L
.56164 .73598 L
.56138 .73422 L
.55818 .70957 L
.5555 .68491 L
.55328 .66026 L
.55146 .6356 L
.55001 .61095 L
.54889 .58629 L
.54807 .56164 L
.54754 .53698 L
.54727 .51233 L
.54727 .48767 L
.54754 .46302 L
.54807 .43836 L
.54889 .41371 L
.55001 .38905 L
.55146 .3644 L
.55328 .33974 L
.5555 .31509 L
.55818 .29043 L
.56138 .26578 L
.56164 .26402 L
.56695 .24112 L
.57479 .21647 L
.58619 .19181 L
.58629 .19164 L
.60717 .16716 L
.61095 .16412 L
.6356 .15051 L
.66026 .14284 L
.66167 .1425 L
F
0 g
.68491 .13815 m
.70957 .13515 L
.73422 .1332 L
.75888 .13198 L
.78353 .13132 L
.80819 .1312 L
.83284 .13174 L
.8575 .13336 L
.88215 .13764 L
.89439 .1425 L
.9068 .15451 L
.91242 .16716 L
.91731 .19181 L
.91957 .21647 L
.92086 .24112 L
.92168 .26578 L
.92225 .29043 L
.92265 .31509 L
.92295 .33974 L
.92317 .3644 L
.92333 .38905 L
.92346 .41371 L
.92354 .43836 L
.9236 .46302 L
.92362 .48767 L
.92362 .51233 L
.9236 .53698 L
.92354 .56164 L
.92346 .58629 L
.92333 .61095 L
.92317 .6356 L
.92295 .66026 L
.92265 .68491 L
.92225 .70957 L
.92168 .73422 L
.92086 .75888 L
.91957 .78353 L
.91731 .80819 L
.91242 .83284 L
.9068 .84549 L
.89439 .8575 L
.88215 .86236 L
.8575 .86664 L
.83284 .86826 L
.80819 .8688 L
.78353 .86868 L
.75888 .86802 L
.73422 .8668 L
.70957 .86485 L
.68491 .86185 L
Mistroke
.66167 .8575 L
.66026 .85716 L
.6356 .84949 L
.61095 .83588 L
.60717 .83284 L
.58629 .80836 L
.58619 .80819 L
.57479 .78353 L
.56695 .75888 L
.56164 .73598 L
.56138 .73422 L
.55818 .70957 L
.5555 .68491 L
.55328 .66026 L
.55146 .6356 L
.55001 .61095 L
.54889 .58629 L
.54807 .56164 L
.54754 .53698 L
.54727 .51233 L
.54727 .48767 L
.54754 .46302 L
.54807 .43836 L
.54889 .41371 L
.55001 .38905 L
.55146 .3644 L
.55328 .33974 L
.5555 .31509 L
.55818 .29043 L
.56138 .26578 L
.56164 .26402 L
.56695 .24112 L
.57479 .21647 L
.58619 .19181 L
.58629 .19164 L
.60717 .16716 L
.61095 .16412 L
.6356 .15051 L
.66026 .14284 L
.66167 .1425 L
.68491 .13815 L
Mfstroke
.6 g
.73422 .18328 m
.75888 .17628 L
.78353 .17254 L
.80819 .17185 L
.83284 .17488 L
.8575 .18418 L
.86861 .19181 L
.88215 .20924 L
.88606 .21647 L
.89498 .24112 L
.90026 .26578 L
.90371 .29043 L
.90613 .31509 L
.9068 .32378 L
.90791 .33974 L
.90924 .3644 L
.91023 .38905 L
.91095 .41371 L
.91146 .43836 L
.91178 .46302 L
.91194 .48767 L
.91194 .51233 L
.91178 .53698 L
.91146 .56164 L
.91095 .58629 L
.91023 .61095 L
.90924 .6356 L
.90791 .66026 L
.9068 .67622 L
.90613 .68491 L
.90371 .70957 L
.90026 .73422 L
.89498 .75888 L
.88606 .78353 L
.88215 .79076 L
.86861 .80819 L
.8575 .81582 L
.83284 .82512 L
.80819 .82815 L
.78353 .82746 L
.75888 .82372 L
.73422 .81672 L
.71458 .80819 L
.70957 .80543 L
.68491 .78772 L
.68045 .78353 L
.66026 .75893 L
.66022 .75888 L
.64648 .73422 L
.63644 .70957 L
.6356 .70714 L
.62894 .68491 L
.62316 .66026 L
.61867 .6356 L
.6152 .61095 L
.6126 .58629 L
.61095 .56516 L
.61074 .56164 L
.60957 .53698 L
.60899 .51233 L
.60899 .48767 L
.60957 .46302 L
.61074 .43836 L
.61095 .43484 L
.6126 .41371 L
.6152 .38905 L
.61867 .3644 L
.62316 .33974 L
.62894 .31509 L
.6356 .29286 L
.63644 .29043 L
.64648 .26578 L
.66022 .24112 L
.66026 .24107 L
.68045 .21647 L
.68491 .21228 L
.70957 .19457 L
.71458 .19181 L
F
0 g
.73422 .18328 m
.75888 .17628 L
.78353 .17254 L
.80819 .17185 L
.83284 .17488 L
.8575 .18418 L
.86861 .19181 L
.88215 .20924 L
.88606 .21647 L
.89498 .24112 L
.90026 .26578 L
.90371 .29043 L
.90613 .31509 L
.9068 .32378 L
.90791 .33974 L
.90924 .3644 L
.91023 .38905 L
.91095 .41371 L
.91146 .43836 L
.91178 .46302 L
.91194 .48767 L
.91194 .51233 L
.91178 .53698 L
.91146 .56164 L
.91095 .58629 L
.91023 .61095 L
.90924 .6356 L
.90791 .66026 L
.9068 .67622 L
.90613 .68491 L
.90371 .70957 L
.90026 .73422 L
.89498 .75888 L
.88606 .78353 L
.88215 .79076 L
.86861 .80819 L
.8575 .81582 L
.83284 .82512 L
.80819 .82815 L
.78353 .82746 L
.75888 .82372 L
.73422 .81672 L
.71458 .80819 L
.70957 .80543 L
.68491 .78772 L
.68045 .78353 L
.66026 .75893 L
.66022 .75888 L
.64648 .73422 L
.63644 .70957 L
Mistroke
.6356 .70714 L
.62894 .68491 L
.62316 .66026 L
.61867 .6356 L
.6152 .61095 L
.6126 .58629 L
.61095 .56516 L
.61074 .56164 L
.60957 .53698 L
.60899 .51233 L
.60899 .48767 L
.60957 .46302 L
.61074 .43836 L
.61095 .43484 L
.6126 .41371 L
.6152 .38905 L
.61867 .3644 L
.62316 .33974 L
.62894 .31509 L
.6356 .29286 L
.63644 .29043 L
.64648 .26578 L
.66022 .24112 L
.66026 .24107 L
.68045 .21647 L
.68491 .21228 L
.70957 .19457 L
.71458 .19181 L
.73422 .18328 L
Mfstroke
.3 g
.9039 .01923 m
.89678 .04389 L
.88215 .06741 L
.88114 .06854 L
.8575 .08303 L
.83284 .08889 L
.80819 .09081 L
.78353 .09037 L
.75888 .08801 L
.73422 .08359 L
.70957 .07652 L
.69062 .06854 L
.68491 .06554 L
.66026 .04815 L
.65573 .04389 L
.63588 .01923 L
F
0 g
.9039 .01923 m
.89678 .04389 L
.88215 .06741 L
.88114 .06854 L
.8575 .08303 L
.83284 .08889 L
.80819 .09081 L
.78353 .09037 L
.75888 .08801 L
.73422 .08359 L
.70957 .07652 L
.69062 .06854 L
.68491 .06554 L
.66026 .04815 L
.65573 .04389 L
.63588 .01923 L
s
.3 g
.9039 .98077 m
.89678 .95611 L
.88215 .93259 L
.88114 .93146 L
.8575 .91697 L
.83284 .91111 L
.80819 .90919 L
.78353 .90963 L
.75888 .91199 L
.73422 .91641 L
.70957 .92348 L
.69062 .93146 L
.68491 .93446 L
.66026 .95185 L
.65573 .95611 L
.63588 .98077 L
F
0 g
.9039 .98077 m
.89678 .95611 L
.88215 .93259 L
.88114 .93146 L
.8575 .91697 L
.83284 .91111 L
.80819 .90919 L
.78353 .90963 L
.75888 .91199 L
.73422 .91641 L
.70957 .92348 L
.69062 .93146 L
.68491 .93446 L
.66026 .95185 L
.65573 .95611 L
.63588 .98077 L
s
.7 g
.78353 .21533 m
.80819 .21401 L
.82294 .21647 L
.83284 .21985 L
.8575 .23806 L
.86019 .24112 L
.87449 .26578 L
.88214 .29043 L
.88215 .29047 L
.88766 .31509 L
.89149 .33974 L
.89424 .3644 L
.89623 .38905 L
.89766 .41371 L
.89865 .43836 L
.89928 .46302 L
.89959 .48767 L
.89959 .51233 L
.89928 .53698 L
.89865 .56164 L
.89766 .58629 L
.89623 .61095 L
.89424 .6356 L
.89149 .66026 L
.88766 .68491 L
.88215 .70953 L
.88214 .70957 L
.87449 .73422 L
.86019 .75888 L
.8575 .76194 L
.83284 .78015 L
.82294 .78353 L
.80819 .78599 L
.78353 .78467 L
.77786 .78353 L
.75888 .77745 L
.73422 .76371 L
.7278 .75888 L
.70957 .74092 L
.70415 .73422 L
.68842 .70957 L
.68491 .70272 L
.67707 .68491 L
.66855 .66026 L
.66205 .6356 L
.66026 .62749 L
.65712 .61095 L
.65345 .58629 L
.65083 .56164 L
.64915 .53698 L
.64832 .51233 L
.64832 .48767 L
.64915 .46302 L
.65083 .43836 L
.65345 .41371 L
.65712 .38905 L
.66026 .37251 L
.66205 .3644 L
.66855 .33974 L
.67707 .31509 L
.68491 .29728 L
.68842 .29043 L
.70415 .26578 L
.70957 .25908 L
.7278 .24112 L
.73422 .23629 L
.75888 .22255 L
.77786 .21647 L
F
0 g
.78353 .21533 m
.80819 .21401 L
.82294 .21647 L
.83284 .21985 L
.8575 .23806 L
.86019 .24112 L
.87449 .26578 L
.88214 .29043 L
.88215 .29047 L
.88766 .31509 L
.89149 .33974 L
.89424 .3644 L
.89623 .38905 L
.89766 .41371 L
.89865 .43836 L
.89928 .46302 L
.89959 .48767 L
.89959 .51233 L
.89928 .53698 L
.89865 .56164 L
.89766 .58629 L
.89623 .61095 L
.89424 .6356 L
.89149 .66026 L
.88766 .68491 L
.88215 .70953 L
.88214 .70957 L
.87449 .73422 L
.86019 .75888 L
.8575 .76194 L
.83284 .78015 L
.82294 .78353 L
.80819 .78599 L
.78353 .78467 L
.77786 .78353 L
.75888 .77745 L
.73422 .76371 L
.7278 .75888 L
.70957 .74092 L
.70415 .73422 L
.68842 .70957 L
.68491 .70272 L
.67707 .68491 L
.66855 .66026 L
.66205 .6356 L
.66026 .62749 L
.65712 .61095 L
.65345 .58629 L
.65083 .56164 L
.64915 .53698 L
Mistroke
.64832 .51233 L
.64832 .48767 L
.64915 .46302 L
.65083 .43836 L
.65345 .41371 L
.65712 .38905 L
.66026 .37251 L
.66205 .3644 L
.66855 .33974 L
.67707 .31509 L
.68491 .29728 L
.68842 .29043 L
.70415 .26578 L
.70957 .25908 L
.7278 .24112 L
.73422 .23629 L
.75888 .22255 L
.77786 .21647 L
.78353 .21533 L
Mfstroke
.8 g
.78353 .26152 m
.80819 .25941 L
.82792 .26578 L
.83284 .26882 L
.85453 .29043 L
.8575 .29934 L
.86458 .31509 L
.87199 .33974 L
.87678 .3644 L
.88003 .38905 L
.88215 .41193 L
.8823 .41371 L
.88403 .43836 L
.88511 .46302 L
.88563 .48767 L
.88563 .51233 L
.88511 .53698 L
.88403 .56164 L
.8823 .58629 L
.88215 .58807 L
.88003 .61095 L
.87678 .6356 L
.87199 .66026 L
.86458 .68491 L
.8575 .70066 L
.85453 .70957 L
.83284 .73118 L
.82792 .73422 L
.80819 .74059 L
.78353 .73848 L
.77199 .73422 L
.75888 .72678 L
.7389 .70957 L
.73422 .70372 L
.72108 .68491 L
.70957 .66209 L
.70877 .66026 L
.69971 .6356 L
.69301 .61095 L
.6881 .58629 L
.68491 .56405 L
.68464 .56164 L
.68243 .53698 L
.68135 .51233 L
.68135 .48767 L
.68243 .46302 L
.68464 .43836 L
.68491 .43595 L
.6881 .41371 L
.69301 .38905 L
.69971 .3644 L
.70877 .33974 L
.70957 .33791 L
.72108 .31509 L
.73422 .29628 L
.7389 .29043 L
.75888 .27322 L
.77199 .26578 L
F
0 g
.78353 .26152 m
.80819 .25941 L
.82792 .26578 L
.83284 .26882 L
.85453 .29043 L
.8575 .29934 L
.86458 .31509 L
.87199 .33974 L
.87678 .3644 L
.88003 .38905 L
.88215 .41193 L
.8823 .41371 L
.88403 .43836 L
.88511 .46302 L
.88563 .48767 L
.88563 .51233 L
.88511 .53698 L
.88403 .56164 L
.8823 .58629 L
.88215 .58807 L
.88003 .61095 L
.87678 .6356 L
.87199 .66026 L
.86458 .68491 L
.8575 .70066 L
.85453 .70957 L
.83284 .73118 L
.82792 .73422 L
.80819 .74059 L
.78353 .73848 L
.77199 .73422 L
.75888 .72678 L
.7389 .70957 L
.73422 .70372 L
.72108 .68491 L
.70957 .66209 L
.70877 .66026 L
.69971 .6356 L
.69301 .61095 L
.6881 .58629 L
.68491 .56405 L
.68464 .56164 L
.68243 .53698 L
.68135 .51233 L
.68135 .48767 L
.68243 .46302 L
.68464 .43836 L
.68491 .43595 L
.6881 .41371 L
.69301 .38905 L
Mistroke
.69971 .3644 L
.70877 .33974 L
.70957 .33791 L
.72108 .31509 L
.73422 .29628 L
.7389 .29043 L
.75888 .27322 L
.77199 .26578 L
.78353 .26152 L
Mfstroke
.9 g
.78353 .31456 m
.80819 .31125 L
.819 .31509 L
.83284 .32622 L
.8448 .33974 L
.8547 .3644 L
.8575 .38098 L
.85945 .38905 L
.86403 .41371 L
.86701 .43836 L
.8688 .46302 L
.86965 .48767 L
.86965 .51233 L
.8688 .53698 L
.86701 .56164 L
.86403 .58629 L
.85945 .61095 L
.8575 .61902 L
.8547 .6356 L
.8448 .66026 L
.83284 .67378 L
.819 .68491 L
.80819 .68875 L
.78353 .68544 L
.78237 .68491 L
.75888 .66655 L
.75338 .66026 L
.73856 .6356 L
.73422 .62518 L
.7289 .61095 L
.72194 .58629 L
.71722 .56164 L
.71427 .53698 L
.71284 .51233 L
.71284 .48767 L
.71427 .46302 L
.71722 .43836 L
.72194 .41371 L
.7289 .38905 L
.73422 .37482 L
.73856 .3644 L
.75338 .33974 L
.75888 .33345 L
.78237 .31509 L
F
0 g
.78353 .31456 m
.80819 .31125 L
.819 .31509 L
.83284 .32622 L
.8448 .33974 L
.8547 .3644 L
.8575 .38098 L
.85945 .38905 L
.86403 .41371 L
.86701 .43836 L
.8688 .46302 L
.86965 .48767 L
.86965 .51233 L
.8688 .53698 L
.86701 .56164 L
.86403 .58629 L
.85945 .61095 L
.8575 .61902 L
.8547 .6356 L
.8448 .66026 L
.83284 .67378 L
.819 .68491 L
.80819 .68875 L
.78353 .68544 L
.78237 .68491 L
.75888 .66655 L
.75338 .66026 L
.73856 .6356 L
.73422 .62518 L
.7289 .61095 L
.72194 .58629 L
.71722 .56164 L
.71427 .53698 L
.71284 .51233 L
.71284 .48767 L
.71427 .46302 L
.71722 .43836 L
.72194 .41371 L
.7289 .38905 L
.73422 .37482 L
.73856 .3644 L
.75338 .33974 L
.75888 .33345 L
.78237 .31509 L
.78353 .31456 L
s
.2 g
.86856 .01923 m
.8575 .03085 L
.83671 .04389 L
.83284 .04493 L
.80819 .04948 L
.78353 .04845 L
.76228 .04389 L
.75888 .04283 L
.73422 .03221 L
.71467 .01923 L
F
0 g
.86856 .01923 m
.8575 .03085 L
.83671 .04389 L
.83284 .04493 L
.80819 .04948 L
.78353 .04845 L
.76228 .04389 L
.75888 .04283 L
.73422 .03221 L
.71467 .01923 L
s
.2 g
.86856 .98077 m
.8575 .96915 L
.83671 .95611 L
.83284 .95507 L
.80819 .95052 L
.78353 .95155 L
.76228 .95611 L
.75888 .95717 L
.73422 .96779 L
.71467 .98077 L
F
0 g
.86856 .98077 m
.8575 .96915 L
.83671 .95611 L
.83284 .95507 L
.80819 .95052 L
.78353 .95155 L
.76228 .95611 L
.75888 .95717 L
.73422 .96779 L
.71467 .98077 L
s
1 g
.78353 .38452 m
.80819 .37842 L
.82216 .38905 L
.83284 .40794 L
.83564 .41371 L
.84377 .43836 L
.84778 .46302 L
.84944 .48767 L
.84944 .51233 L
.84778 .53698 L
.84377 .56164 L
.83564 .58629 L
.83284 .59206 L
.82216 .61095 L
.80819 .62158 L
.78353 .61548 L
.77877 .61095 L
.76332 .58629 L
.75888 .57522 L
.75463 .56164 L
.74956 .53698 L
.74726 .51233 L
.74726 .48767 L
.74956 .46302 L
.75463 .43836 L
.75888 .42478 L
.76332 .41371 L
.77877 .38905 L
F
0 g
.78353 .38452 m
.80819 .37842 L
.82216 .38905 L
.83284 .40794 L
.83564 .41371 L
.84377 .43836 L
.84778 .46302 L
.84944 .48767 L
.84944 .51233 L
.84778 .53698 L
.84377 .56164 L
.83564 .58629 L
.83284 .59206 L
.82216 .61095 L
.80819 .62158 L
.78353 .61548 L
.77877 .61095 L
.76332 .58629 L
.75888 .57522 L
.75463 .56164 L
.74956 .53698 L
.74726 .51233 L
.74726 .48767 L
.74956 .46302 L
.75463 .43836 L
.75888 .42478 L
.76332 .41371 L
.77877 .38905 L
.78353 .38452 L
s
.5 g
.98077 .11081 m
.95611 .10256 L
.94625 .0932 L
.93707 .06854 L
.93371 .04389 L
.93198 .01923 L
.98077 .01923 L
F
0 g
.98077 .11081 m
.95611 .10256 L
.94625 .0932 L
.93707 .06854 L
.93371 .04389 L
.93198 .01923 L
s
.5 g
.98077 .88919 m
.95611 .89744 L
.94625 .9068 L
.93707 .93146 L
.93371 .95611 L
.93198 .98077 L
.98077 .98077 L
F
0 g
.98077 .88919 m
.95611 .89744 L
.94625 .9068 L
.93707 .93146 L
.93371 .95611 L
.93198 .98077 L
s
.3 g
.98077 .83587 m
.97644 .83284 L
.95729 .80819 L
.95611 .80555 L
.9491 .78353 L
.94453 .75888 L
.94164 .73422 L
.93966 .70957 L
.93825 .68491 L
.93721 .66026 L
.93643 .6356 L
.93585 .61095 L
.93543 .58629 L
.93513 .56164 L
.93494 .53698 L
.93485 .51233 L
.93485 .48767 L
.93494 .46302 L
.93513 .43836 L
.93543 .41371 L
.93585 .38905 L
.93643 .3644 L
.93721 .33974 L
.93825 .31509 L
.93966 .29043 L
.94164 .26578 L
.94453 .24112 L
.9491 .21647 L
.95611 .19445 L
.95729 .19181 L
.97644 .16716 L
.98077 .16413 L
F
0 g
.98077 .83587 m
.97644 .83284 L
.95729 .80819 L
.95611 .80555 L
.9491 .78353 L
.94453 .75888 L
.94164 .73422 L
.93966 .70957 L
.93825 .68491 L
.93721 .66026 L
.93643 .6356 L
.93585 .61095 L
.93543 .58629 L
.93513 .56164 L
.93494 .53698 L
.93485 .51233 L
.93485 .48767 L
.93494 .46302 L
.93513 .43836 L
.93543 .41371 L
.93585 .38905 L
.93643 .3644 L
.93721 .33974 L
.93825 .31509 L
.93966 .29043 L
.94164 .26578 L
.94453 .24112 L
.9491 .21647 L
.95611 .19445 L
.95729 .19181 L
.97644 .16716 L
.98077 .16413 L
s
.2 g
.98077 .78033 m
.9701 .75888 L
.96253 .73422 L
.95757 .70957 L
.95611 .70017 L
.95412 .68491 L
.95162 .66026 L
.94978 .6356 L
.94841 .61095 L
.94742 .58629 L
.94672 .56164 L
.94628 .53698 L
.94606 .51233 L
.94606 .48767 L
.94628 .46302 L
.94672 .43836 L
.94742 .41371 L
.94841 .38905 L
.94978 .3644 L
.95162 .33974 L
.95412 .31509 L
.95611 .29983 L
.95757 .29043 L
.96253 .26578 L
.9701 .24112 L
.98077 .21967 L
F
0 g
.98077 .78033 m
.9701 .75888 L
.96253 .73422 L
.95757 .70957 L
.95611 .70017 L
.95412 .68491 L
.95162 .66026 L
.94978 .6356 L
.94841 .61095 L
.94742 .58629 L
.94672 .56164 L
.94628 .53698 L
.94606 .51233 L
.94606 .48767 L
.94628 .46302 L
.94672 .43836 L
.94742 .41371 L
.94841 .38905 L
.94978 .3644 L
.95162 .33974 L
.95412 .31509 L
.95611 .29983 L
.95757 .29043 L
.96253 .26578 L
.9701 .24112 L
.98077 .21967 L
s
.1 g
.98077 .71849 m
.97785 .70957 L
.97161 .68491 L
.96722 .66026 L
.96404 .6356 L
.96172 .61095 L
.96004 .58629 L
.95887 .56164 L
.95813 .53698 L
.95777 .51233 L
.95777 .48767 L
.95813 .46302 L
.95887 .43836 L
.96004 .41371 L
.96172 .38905 L
.96404 .3644 L
.96722 .33974 L
.97161 .31509 L
.97785 .29043 L
.98077 .28151 L
F
0 g
.98077 .71849 m
.97785 .70957 L
.97161 .68491 L
.96722 .66026 L
.96404 .6356 L
.96172 .61095 L
.96004 .58629 L
.95887 .56164 L
.95813 .53698 L
.95777 .51233 L
.95777 .48767 L
.95813 .46302 L
.95887 .43836 L
.96004 .41371 L
.96172 .38905 L
.96404 .3644 L
.96722 .33974 L
.97161 .31509 L
.97785 .29043 L
.98077 .28151 L
s
.6 g
.98077 .05689 m
.97061 .04389 L
.9595 .01923 L
.98077 .01923 L
F
0 g
.98077 .05689 m
.97061 .04389 L
.9595 .01923 L
s
.6 g
.98077 .94311 m
.97061 .95611 L
.9595 .98077 L
.98077 .98077 L
F
0 g
.98077 .94311 m
.97061 .95611 L
.9595 .98077 L
s
.98077 .63982 m
.98003 .6356 L
.97645 .61095 L
.97389 .58629 L
.97212 .56164 L
.97101 .53698 L
.97047 .51233 L
.97047 .48767 L
.97101 .46302 L
.97212 .43836 L
.97389 .41371 L
.97645 .38905 L
.98003 .3644 L
.98077 .36018 L
F
.98077 .63982 m
.98003 .6356 L
.97645 .61095 L
.97389 .58629 L
.97212 .56164 L
.97101 .53698 L
.97047 .51233 L
.97047 .48767 L
.97101 .46302 L
.97212 .43836 L
.97389 .41371 L
.97645 .38905 L
.98003 .3644 L
.98077 .36018 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{155.438, 155.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002K0000Va000`40O003h00OVgoo002KOol009]o
o`004Goo10007Woo0`0077oo0P007goo0`007goo10000Woo000BOol00`00Oomoo`0OOol00`00Oomo
o`0JOol01000Oomoo`007goo00<007ooOol07goo00<007ooOol00Woo000;Ool50003Ool00`00Oomo
o`0GOol50002Ool00`00Oomoo`0JOol01000Oomoo`007goo00<007ooOol087oo00<007ooOol00Goo
000DOol00`00Oomoo`0MOol00`00Oomoo`0JOol01000Oomoo`007goo00<007ooOol08Goo0@000Goo
0Goo000AOol01000Oomoo`007goo00<007ooOol06Woo00@007ooOol001moo`03001oogoo01ioo`04
001oogoo0002Ool0019oo`8001moo`8001eoo`8001moo`80025oo`8000=oo`00Vgoo002KOol009]o
o`001goo10000WooSP000008Ool00`00Oomoo`02Ool00`00Oomoo`2:Ool100000003Ool0000000<0
00=oo`04001oogooOol20004Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol04Woo
00<007ooOol027oo00<007ooOol097oo00<007ooOol027oo00<007ooOol04Woo00<007ooOol00Woo
00D007ooOomoo`0000=oo`03001oogoo009oo`8000002Woo00@007ooOol0009oo`=>L`05000mkcg_
?Nl00003<H`00`009BTU:@029BT00`006<HHaP0A6<H00`009BTU:@099BT00`00<H`aS00T<H`00`00
9BTU:@099BT00`006<HHaP0A6<H00`009BTU:@029BT01@00<H`aS36<00000cg_00<004icCW<00goo
0@000007Ool01000Oomoo`000Woo00<007ooOol00Tic00<003g_?Nl00Sg_00D0036<<H`aS00000DU
:@8000lHaP8000/U:@03000aS36<02HaS003000U:BDY00TU:@8000lHaP8000DU:@05000aS36<<H`0
0004?Nl00`00CW=oo`02Ool1000000Qoo`8000=oo`03001oogoo009>L`03000mkcg_008mk`03000a
S36<008aS003000U:BDY00@U:@03000HaQS600XHaP8000dU:@03000aS36<02HaS003000U:BDY00/U
:@8000`HaP03000U:BDY00@U:@03000aS36<008aS003000mkcg_008mk`03001>Lgoo009oo`400000
3Goo00D007ooOom>L`0000Dmk`03000aS36<008aS003000U:BDY00DU:@@0008HaPH000`U:@<002Xa
S08000dU:@H000<HaP<000LU:@03000aS36<008aS003000mkcg_00<mk`04001oogooOol1000000eo
o`05001oogooCW<00005?Nl00`00<H`aS002<H`00`009BTU:@099BT2000A9BT00`00<H`aS00]<H`0
0`009BTU:@0@9BT3000:9BT00`00<H`aS002<H`00`00?Nlmk`03?Nl01000Oomoogoo0@00000=Ool0
1000Oomoo`001Cg_00<0036<<H`0136<00<002DY9BT06RDY00<0036<<H`0;c6<00<002DY9BT06bDY
00<0036<<H`0136<00<003g_?Nl00cg_00<007ooOol00@00000=Ool200000goo?Nlmk`04?Nl00`00
<H`aS005<H`2000H9BT2000c<H`4000G9BT20007<H`00`00?Nlmk`03?Nl2Ool2000000eoo`03001o
ogoo00Hmk`03000aS36<00LaS003000U:BDY018U:@<003TaS0<001<U:@03000aS36<00LaS003000m
kcg_00<mk`=oo`4000003Goo00<007ooOol01Sg_00<0036<<H`0236<0`003BDY1000?c6<0`003BDY
0`002S6<00<003g_?Nl00cg_0goo0@00000=Ool00`00Oomoo`05?Nl00`00<H`aS00<<H`=0016<H`=
000><H`00`00?Nlmk`02?Nl3Ool1000000eoo`03001oogoo00@mk`8007`aS003000mkcg_008mk`=o
o`4000003Goo00<007ooOol00Sg_0P00Oc6<0`0000@mkgooOomoo`4000003Goo00<007ooOol00P00
Q36<0P000Woo0@00000=Ool200000goo<H`aS025<H`2Ool2000000eoo`03001oogoo08LaS0=oo`40
00003Goo00<007ooOol0436<5P00?C6<5P003S6<0goo0@00000=Ool00`00Oomoo`0=<H`3000F?Nl6
000`<H`7000F?Nl2000<<H`3Ool1000000eoo`03001oogoo00/aS08001lmk`@002PaS0@001lmk`80
00XaS0=oo`4000003Goo00<007ooOol02S6<00<003g_?Nl08cg_0P008c6<0`009Cg_00<0036<<H`0
1c6<0goo0@00000=Ool00`00Oomoo`0:<H`00`00?Nlmk`0U?Nl2000P<H`00`00?Nlmk`0V?Nl00`00
<H`aS007<H`3Ool1000000eoo`800003Ool0000000PaS003000mkcg_02Lmk`03000aS36<01/aS080
02Tmk`03000aS36<00HaS080005oo`40004000003Goo00D007ooOolU:@0000PaS003000mkcg_00Hm
k`l001<mk`03000aS36<01XaS003000mkcg_014mk`l000Pmk`03000aS36<00HaS004001oogooOol1
000000eoo`03001oogoo008U:@03000aS36<00@aS003000mkcg_00Dmk`8000m>L`<0014mk`03000a
S36<01PaS003000mkcg_00lmk`<000m>L`8000Lmk`03000aS36<00@aS003000U:Goo009oo`400000
3Goo00<007ooOol00bDY00<0036<<H`00c6<00<003g_?Nl013g_00<004icCW<04Tic00<003g_?Nl0
3cg_0P005c6<00<003g_?Nl03cg_00<004icCW<04Tic00<003g_?Nl013g_00<0036<<H`00c6<00<0
02DY9BT00goo0@00000=Ool00`00Oomoo`039BT00`00<H`aS003<H`00`00?Nlmk`04?Nl00`00CW=>
L`0CCW<2000??Nl00`00<H`aS00F<H`00`00?Nlmk`0=?Nl2000ECW<00`00?Nlmk`04?Nl00`00<H`a
S002<H`010009BTU:BDY0goo0@00000=Ool00`00Oomoo`039BT00`00<H`aS003<H`00`00?Nlmk`03
?Nl00`00CW=>L`0FCW<00`00?Nlmk`0=?Nl00`00<H`aS00D<H`00`00?Nlmk`0<?Nl2000HCW<00`00
?Nlmk`03?Nl00`00<H`aS002<H`010009BTU:BDY0goo0@00000=Ool200000goo9BTU:@029BT00`00
<H`aS002<H`00`00?Nlmk`02?Nl00`00CW=>L`0HCW<00`00?Nlmk`0<?Nl00`00<H`aS00D<H`00`00
?Nlmk`0;?Nl00`00CW=>L`0ICW<00`00?Nlmk`02?Nl01@00<H`aS36<000012DY0Woo0P00000=Ool0
1000Oomoo`000bDY00<0036<<H`00S6<00D003g_?Nlmk`0000I>L`/000]>L`03000mkcg_00`mk`03
000aS36<018aS003000mkcg_00`mk`03001>Ldic00U>L`/000I>L`05000mkcg_?Nl00003<H`00`00
9BTU:@029BT00`00Oomoo`01000000eoo`04001oogoo00039BT00`00<H`aS002<H`01@00?Nlmkcg_
00001Dic00<005[FF]H02E[F0P002Tic00<003g_?Nl02cg_00<0036<<H`04S6<00<003g_?Nl02cg_
00<004icCW<024ic0P002e[F00<004icCW<00dic00D003g_?Nlmk`0000<aS003000U:BDY008U:@03
001oogoo004000003Goo00D007ooOolHaP0000<U:@04000aS36<0003?Nl00`00CW=>L`03CW<00`00
F]IJeP0<F]H20009CW<00`00?Nlmk`0;?Nl00`00<H`aS00@<H`00`00?Nlmk`0;?Nl00`00CW=>L`07
CW<2000>F]H00`00CW=>L`03CW<01@00?Nlmkcg_00000S6<00D002DY9BTU:@0000=oo`4000003Goo
00D007ooOolHaP0000<U:@04000aS36<0003?Nl00`00CW=>L`02CW<00`00F]IJeP0?F]H00`00CW=>
L`07CW<00`00?Nlmk`0:?Nl00`00<H`aS00@<H`00`00?Nlmk`0:?Nl00`00CW=>L`06CW<2000AF]H0
0`00CW=>L`02CW<01@00?Nlmkcg_00000S6<00D002DY9BT001S600=oo`4000001goo0`000goo00D0
07ooOolHaP0000<U:@04000aS36<0003?Nl00`00CW=>L`02CW<00`00F]IJeP0@F]H00`00CW=>L`06
CW<00`00?Nlmk`0:?Nl00`00<H`aS00@<H`00`00?Nlmk`0:?Nl00`00CW=>L`06CW<00`00F]IJeP0@
F]H00`00CW=>L`02CW<01@00?Nlmkcg_00000S6<00D002DY9BT001S600=oo`40000027oo00<007oo
Ool00Woo00<007ooOol00QS600@002DY9BT0008aS005000mkcg_?Nl00003CW<00`00F]IJeP0BF]H0
0`00CW=>L`06CW<00`00?Nlmk`09?Nl00`00<H`aS00@<H`00`00?Nlmk`09?Nl00`00CW=>L`06CW<0
0`00F]IJeP0BF]H01@00CW=>Ldic00000cg_00@0036<<H`0008U:@03000HaWoo009oo`40000000=o
o`0000000`000Woo00<007ooOol00Woo0P0000AooaS66<H0008U:@04000aS36<0002?Nl00`00CW=>
L`02CW<00`00F]IJeP04F]H60009F]H00`00CW=>L`05CW<00`00?Nlmk`0:?Nl00`00<H`aS00><H`0
0`00?Nlmk`0:?Nl00`00CW=>L`05CW<00`00F]IJeP07F]H60006F]H00`00CW=>L`02CW<01P00?Nlm
k`00<H`0008U:@03000HaQS6009oo`80000027oo00<007ooOol00Woo00<007ooOol00aS600<002DY
00000S6<00@003g_?Nl000=>L`03001JeU[F00=JeP8000IW>@8000QJeP03001>Ldic00E>L`03000m
kcg_00Tmk`03000aS36<00haS003000mkcg_00Tmk`03001>Ldic00E>L`03001JeU[F00IJeP8000IW
>@8000EJeP05001>LdicCW<00002?Nl00`00<H`000029BT00`006<HHaP03Ool1000000Qoo`03001o
ogoo009oo`03001oogoo00<HaP03000U:@00008aS004000mkcg_0003CW<00`00F]IJeP02F]H00`00
IcUW>@08IcT20006F]H00`00CW=>L`05CW<00`00?Nlmk`09?Nl00`00<H`aS00><H`00`00?Nlmk`09
?Nl00`00CW=>L`05CW<00`00F]IJeP04F]H2000:IcT00`00F]IJeP02F]H01@00CW=>Ldic00000Sg_
00<0036<00000RDY00<001S66<H00goo0@000007Ool20004Ool01000Oomoo`000QS600H002DY9BT0
036<0002?Nl01000CW=>L`0015[F00<006LiIcT02fLi00<005[FF]H015[F00<004icCW<01Dic00<0
03g_?Nl023g_00<0036<<H`03S6<00<003g_?Nl023g_00<004icCW<01Dic00<005[FF]H015[F00<0
06LiIcT02fLi00<005[FF]H00U[F00@004icCW<0008mk`03000aS000008U:@04000HaQS60002Ool1
000000eoo`04001oogoo00026<H01P009BTU:@00<H`0008mk`04001>Ldic0003F]H00`00IcUW>@0=
IcT00`00F]IJeP04F]H00`00CW=>L`04CW<00`00?Nlmk`08?Nl00`00<H`aS00><H`00`00?Nlmk`08
?Nl00`00CW=>L`04CW<00`00F]IJeP04F]H00`00IcUW>@0=IcT01@00F]IJeU[F00000Tic00H003g_
?Nl0036<00029BT010006<HHaP000Woo0@00000=Ool01000Oomoo`000QS600H002DY9BT0036<0002
?Nl01000CW=>L`000e[F00<006LiIcT03VLi00<005[FF]H00e[F00<004icCW<014ic00<003g_?Nl0
23g_00<0036<<H`03S6<00<003g_?Nl01cg_00<004icCW<014ic00<005[FF]H015[F00<006LiIcT0
3VLi00D005[FF]IJeP00009>L`06000mkcg_000aS0000RDY00@001S66<H0009oo`4000003Goo00@0
07ooOol0008HaP06000U:BDY000aS0000Sg_00@004icCW<0009JeP03001W>FLi00mW>@03001JeU[F
00=JeP03001>Ldic00A>L`03000mkcg_00Pmk`03000aS36<00haS003000mkcg_00Lmk`03001>Ldic
00A>L`03001JeU[F00AJeP03001W>FLi00mW>@04001JeU[F0002CW<02000?Nlmk`00<H`002DY0003
6<H00`00Oomoo`01000000eoo`800005Ool<H`006<H000029BT01@00<H`003g_00000Tic00D005[F
F]IJeP0000IW>@<000UW>@03001JeU[F00=JeP03001>Ldic00A>L`03000mkcg_00Pmk`03000aS36<
00`aS003000mkcg_00Pmk`03001>Ldic00=>L`03001JeU[F00AJeP03001W>FLi00MW>@<000IW>@07
001JeU[FF]H004ic00000Sg_00D0036<000U:@00008HaP03001oogoo008000003Goo00D007ooOol<
H`00008HaP07000U:@00<H`003g_00000Tic00@005[FF]H000EW>@8000=cW0<000IW>@03001JeU[F
00AJeP03001>Ldic00=>L`03000mkcg_00Pmk`03000aS36<00`aS003000mkcg_00Pmk`03001>Ldic
00=>L`03001JeU[F00=JeP03001W>FLi00EW>@<000=cW08000EW>@06001JeU[F001>L`000Sg_00D0
036<000U:@00008HaP04001oogooOol1000000eoo`05001oogoo36<000026<H01`009BT0036<000m
k`00009>L`04001JeU[F0004IcT00`00LiacW006Li`00`00IcUW>@03IcT00`00F]IJeP04F]H00`00
CW=>L`03CW<00`00?Nlmk`08?Nl00`00<H`aS00<<H`00`00?Nlmk`08?Nl00`00CW=>L`03CW<00`00
F]IJeP03F]H00`00IcUW>@04IcT00`00LiacW006Li`00`00IcUW>@02IcT01000F]IJeP000Tic00L0
03g_000aS0009BT000026<H01000Oomoogoo0@00000=Ool00`00Oomoo`0236<02`006<H002DY000a
S000?Nl004ic00000U[F00<006LiIcT00VLi00<007>LLi`027>L00<006LiIcT00fLi00<005[FF]H0
15[F00<004icCW<00dic00<003g_?Nl01cg_00<0036<<H`0336<00<003g_?Nl01cg_00<004icCW<0
0dic00<005[FF]H00e[F00<006LiIcT016Li00<007>LLi`027>L00<006LiIcT00VLi00<005[F0000
0Tic00X003g_000aS0009BT001S6000<H`=oo`4000003Goo00<007ooOol00PaS00/001S6000U:@00
<H`003g_001>L`00009JeP05001W>FLiIcT0000<Li`00`00IcUW>@03IcT00`00F]IJeP03F]H00`00
CW=>L`03CW<00`00?Nlmk`07?Nl00`00<H`aS00<<H`00`00?Nlmk`07?Nl00`00CW=>L`03CW<00`00
F]IJeP03F]H00`00IcUW>@03IcT00`00LiacW00:Li`01`00IcUW>FLi001JeP00009>L`0:000mk`00
<H`002DY000HaP0036<3Ool1000000eoo`03001oogoo008<H`0;000HaP009BT0036<000mk`00CW<0
0002F]H01@00IcUW>FLi000037>L00<006LiIcT00fLi00<005[FF]H00e[F00<004icCW<00dic00<0
03g_?Nl01cg_00<0036<<H`0336<00<003g_?Nl01cg_00<004icCW<00dic00<005[FF]H00e[F00<0
06LiIcT00fLi00<007>LLi`02W>L00L006LiIcUW>@00F]H00002CW<02P00?Nl0036<000U:@006<H0
00aS0goo0@00000=Ool00`00Oomoo`0236<02`006<H002DY000aS000?Nl004ic00000U[F00D006Li
IcUW>@0000ecW003001W>FLi009W>@03001JeU[F00=JeP03001>Ldic00=>L`03000mkcg_00Lmk`03
000aS36<00`aS003000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F009JeP03001W>FLi00=W>@03
001cW7>L00]cW007001W>FLiIcT005[F00000Tic00X003g_000aS0009BT001S6000<H`=oo`400000
3Goo0P0000ioo`0036<001S6000U:@00<H`003g_001>L`000U[F00@006LiIcT000icW003001W>FLi
00=W>@03001JeU[F00=JeP03001>Ldic00=>L`03000mkcg_00Hmk`03000aS36<00`aS003000mkcg_
00Hmk`03001>Ldic00=>L`03001JeU[F00=JeP03001W>FLi00=W>@03001cW7>L00acW004001W>FLi
0002F]H03P00CW<003g_000aS0009BT001S6000<H`00Ool2000000eoo`0@001oogoo000<H`006<H0
02DY000aS000?Nl004ic0002F]H01000IcUW>@003g>L00<006LiIcT00VLi00<005[FF]H00e[F00<0
04icCW<00dic00<003g_?Nl01Sg_00<0036<<H`0336<00<003g_?Nl01Sg_00<004icCW<00dic00<0
05[FF]H00U[F00<006LiIcT00fLi00<007>LLi`03G>L00@006LiIcT0009JeP0=001>L`00?Nl0036<
000U:@006<H000aS00000Woo0@00000=Ool04000Oomoo`0036<001S6000U:@00<H`003g_001>L`00
0U[F00@006LiIcT0011cW005001W>FLiIcT00005F]H00`00CW=>L`03CW<00`00?Nlmk`06?Nl00`00
<H`aS00<<H`00`00?Nlmk`06?Nl00`00CW=>L`03CW<00`00F]IJeP02F]H00`00IcUW>@02IcT00`00
LiacW00>Li`01000IcUW>@000U[F00d004ic000mk`00<H`002DY000HaP0036<00002Ool1000000eo
o`03001oogoo0080000=36<000009BT0036<000mk`00CW<005[F00000VLi00<007>LLi`00g>L1000
27>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<<H`0
2c6<00<003g_?Nl01cg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00VLi00<007>LLi`0
1W>L10001G>L00/006LiIcT005[F001>L`00?Nl0036<008000046<H000aS0003Ool1000000eoo`03
001oogoo0080000=36<000009BT0036<000mk`00CW<005[F00000VLi00<007>LLi`00W>L00<007oo
Ool00Woo0P001W>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00dic00<003g_?Nl01Sg_
00<0036<<H`02c6<00<003g_?Nl01cg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00VLi
00<007>LLi`017>L0P0017oo00<007>LLi`00W>L00/006LiIcT005[F001>L`00?Nl0036<00800004
6<H000aS0003Ool1000000eoo`03001oogoo0080000=36<000009BT0036<000mk`00CW<005[F0000
0VLi00<007>LLi`00W>L00<007ooOol00goo00<007>LLi`017>L00<006LiIcT00VLi00<005[FF]H0
0U[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<<H`02c6<00<003g_?Nl01cg_00<004icCW<0
0dic00<005[FF]H00U[F00<006LiIcT00VLi00<007>LLi`017>L00<007ooOol00goo00<007>LLi`0
0W>L00/006LiIcT005[F001>L`00?Nl0036<008000046<H000aS0003Ool1000000eoo`800004Ool0
000036<200002RDY000aS000?Nl004ic001JeP000VLi00D007>LLiacW00000Moo`03001cW7>L00=c
W003001W>FLi009W>@03001JeU[F009JeP03001>Ldic00=>L`03000mkcg_00Hmk`03000aS36<00/a
S003000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F009JeP03001W>FLi009W>@03001cW7>L00=c
W003001oogoo00Eoo`05001cW7>LLi`00002IcT02000F]H004ic000mk`00<H`2000011S6000<H`00
0Woo0P00000=Ool00`00Oomoo`0200000`aS000HaP0200002S6<000mk`00CW<005[F001W>@0017>L
00<007ooOol01Goo00<007>LLi`017>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00dic
00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00<005[FF]H00U[F
00<006LiIcT00VLi00<007>LLi`00g>L00<007ooOol01Woo00<007>LLi`00W>L00X006Li001JeP00
CW<003g_000aS08000046<H000aS0003Ool1000000eoo`03001oogoo0080000336<001S60080000:
<H`003g_001>L`00F]H006Li0003Li`00`00Oomoo`07Ool00`00LiacW003Li`00`00IcUW>@02IcT0
0`00F]IJeP02F]H00`00CW=>L`03CW<00`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl0
0`00CW=>L`03CW<00`00F]IJeP02F]H00`00IcUW>@02IcT00`00LiacW002Li`00`00Oomoo`08Ool0
3P00LiacW7>L001W>@00F]H004ic000mk`00<H`2000011S6000<H`000goo0@00000=Ool00`00Oomo
o`0200000`aS000HaP0200002S6<000mk`00CW<005[F001W>@000g>L00<007ooOol027oo00<007>L
Li`00W>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<
<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00VLi00<007>L
Li`00W>L00<007ooOol027oo00h007>LLiacW000IcT005[F001>L`00?Nl0036<0P0000@HaP0036<0
00=oo`4000003Goo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P000U[F00<006Li0000
0W>L00<007ooOol02Goo00<007>LLi`00g>L00D006LiIcUW>@0000AJeP03001>Ldic00=>L`03000m
kcg_00Hmk`03000aS36<00XaS003000mkcg_00Hmk`03001>Ldic00=>L`05001JeU[FF]H00004IcT0
0`00LiacW003Li`00`00Oomoo`09Ool03@00LiacW000IcT005[F001>L`00?Nl0036<008000046<H0
00aS0003Ool1000000eoo`03001oogoo0080000336<001S600800003<H`003g_0080009JeP03001W
>@00009cW003001oogoo00Yoo`03001cW7>L009cW004001W>FLiIcT20003F]H00`00CW=>L`03CW<0
0`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl00`00CW=>L`03CW<01@00F]IJeU[F0000
16Li00<007>LLi`00W>L00<007ooOol02Woo00d007>LLi`006Li001JeP00CW<003g_000aS0020000
11S6000<H`000goo0@00000=Ool00`00Oomoo`0200000`aS000HaP0200000c6<000mk`020002F]H0
0`00IcT00002Li`00`00Oomoo`0:Ool00`00LiacW002Li`00`00IcUW>@02IcT01@00F]IJeU[F0000
1Dic00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00D005[FF]IJ
eP0000AW>@03001cW7>L009cW003001oogoo00Yoo`0=001cW7>L001W>@00F]H004ic000mk`00<H`0
0P0000@HaP0036<000=oo`4000003Goo0P0000Ioo`00000<H`006<H200000c6<000mk`020002F]H0
0`00IcT00002Li`00`00Oomoo`0:Ool00`00LiacW002Li`00`00IcUW>@02IcT01@00F]IJeU[F0000
1Dic00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00D005[FF]IJ
eP0000AW>@03001cW7>L009cW003001oogoo00Yoo`0=001cW7>L001W>@00F]H004ic000mk`00<H`0
0P0000@HaP0036<0009oo`8000003Goo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P00
00EJeP00IcUW>@00009cW003001oogoo00Yoo`03001cW7>L009cW003001W>FLi009W>@03001JeU[F
009JeP03001>Ldic009>L`03000mkcg_00Hmk`03000aS36<00XaS003000mkcg_00Hmk`03001>Ldic
009>L`03001JeU[F009JeP03001W>FLi009W>@05001cW7>LLi`0000=Ool01000LiacW0000VLi0P00
00E>L`00?Nl0036<008000046<H000aS0003Ool1000000eoo`03001oogoo0080000336<001S60080
0003<H`003g_00800005F]H006LiIcT00002Li`00`00Oomoo`0:Ool00`00LiacW002Li`00`00IcUW
>@02IcT00`00F]IJeP02F]H00`00CW=>L`02CW<00`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlm
k`06?Nl00`00CW=>L`02CW<00`00F]IJeP02F]H00`00IcUW>@02IcT01@00LiacW7>L00003Goo00@0
07>LLi`0009W>@800005CW<003g_000aS002000011S6000<H`000goo0@00000=Ool00`00Oomoo`02
00000`aS000HaP0200000c6<000mk`0200001E[F001W>FLi00000W>L00<007ooOol02Woo00<007>L
Li`00W>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00Tic00<003g_?Nl01Sg_00<0036<
<H`02S6<00<003g_?Nl01Sg_00<004icCW<00Tic00<005[FF]H00U[F00<006LiIcT00VLi00D007>L
LiacW00000eoo`04001cW7>L0002IcT200001Dic000mk`00<H`00P0000@HaP0036<000=oo`400000
3Goo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P0000MJeP00IcUW>@00Li`0000>Ool0
0`00LiacW002Li`01@00IcUW>FLi000015[F00<004icCW<00Tic00<003g_?Nl01Sg_00<0036<<H`0
2S6<00<003g_?Nl01Sg_00<004icCW<00Tic00<005[FF]H00U[F00D006LiIcUW>@0000AcW003001o
ogoo00aoo`03001cW000009W>@05001JeP00000mk`0200000bDY000HaP030003Ool1000000Qoo`80
00=oo`03001oogoo0080000336<001S600800003<H`003g_00800007F]H006LiIcT007>L00003Woo
00<007>LLi`00W>L00D006LiIcUW>@0000AJeP03001>Ldic009>L`03000mkcg_00Hmk`03000aS36<
00XaS003000mkcg_00Hmk`03001>Ldic009>L`03001JeU[F009JeP05001W>FLiIcT00004Li`00`00
Oomoo`0<Ool00`00Li`00002IcT01@00F]H00000?Nl00P0000<U:@006<H00`000goo0@000007Ool0
1000Oomoo`000Woo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P0000MJeP00IcUW>@00
Li`0000>Ool00`00LiacW002Li`01@00IcUW>FLi000015[F00<004icCW<00Tic00<003g_?Nl01Sg_
00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00Tic00<005[FF]H00U[F00D006LiIcUW>@00
00AcW003001oogoo00aoo`03001cW000009W>@05001JeP00000mk`0200000bDY000HaP030003Ool1
000000Moo`04001oogoo0002Ool200001Woo000000aS000HaP800003<H`003g_00800007F]H006Li
IcT007>L00003Woo00<007>LLi`00W>L00D006LiIcUW>@0000AJeP03001>Ldic009>L`03000mkcg_
00Hmk`03000aS36<00XaS003000mkcg_00Hmk`03001>Ldic009>L`03001JeU[F009JeP05001W>FLi
IcT00004Li`00`00Oomoo`0<Ool00`00Li`00002IcT01@00F]H00000?Nl00P0000<U:@006<H00`00
0Woo0P000007Ool01000Oomoo`000Woo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P00
00MJeP00IcUW>@00Li`0000>Ool00`00LiacW002Li`01@00IcUW>FLi000015[F00<004icCW<00Tic
00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00Tic00<005[FF]H00U[F
00D006LiIcUW>@0000AcW003001oogoo00aoo`03001cW000009W>@05001JeP00000mk`0200000bDY
000HaP030003Ool1000000Moo`04001oogoo0002Ool00`00Oomoo`0200000`aS000HaP0200000c6<
000mk`0200001e[F001W>FLi001cW00000ioo`03001cW7>L009cW005001W>FLiIcT00004F]H00`00
CW=>L`02CW<00`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl00`00CW=>L`02CW<00`00
F]IJeP02F]H01@00IcUW>FLi000017>L00<007ooOol037oo00<007>L00000VLi00D005[F000003g_
008000039BT001S600<000=oo`40000027oo0P000goo00<007ooOol00P0000<<H`006<H00P0000<a
S000?Nl00P0000MJeP00IcUW>@00Li`0000>Ool00`00LiacW002Li`01@00IcUW>FLi000015[F00<0
04icCW<00Tic00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00Tic00<0
05[FF]H00U[F00D006LiIcUW>@0000AcW003001oogoo00aoo`03001cW000009W>@05001JeP00000m
k`0200000bDY000HaP030003Ool1000000eoo`03001oogoo0080000336<001S600800003<H`003g_
00800005F]H006LiIcT00002Li`00`00Oomoo`0:Ool00`00LiacW002Li`00`00IcUW>@02IcT00`00
F]IJeP02F]H00`00CW=>L`02CW<00`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl00`00
CW=>L`02CW<00`00F]IJeP02F]H00`00IcUW>@02IcT01@00LiacW7>L00003Goo00@007>LLi`0009W
>@800005CW<003g_000aS002000011S6000<H`000goo0@00000=Ool00`00Oomoo`0200000`aS000H
aP0200000c6<000mk`0200001E[F001W>FLi00000W>L00<007ooOol02Woo00<007>LLi`00W>L00<0
06LiIcT00VLi00<005[FF]H00U[F00<004icCW<00Tic00<003g_?Nl01Sg_00<0036<<H`02S6<00<0
03g_?Nl01Sg_00<004icCW<00Tic00<005[FF]H00U[F00<006LiIcT00VLi00D007>LLiacW00000eo
o`04001cW7>L0002IcT200001Dic000mk`00<H`00P0000@HaP0036<000=oo`4000003Goo00<007oo
Ool00P0000<<H`006<H00P0000<aS000?Nl00P0000EJeP00IcUW>@00009cW003001oogoo00Yoo`03
001cW7>L009cW003001W>FLi009W>@03001JeU[F009JeP03001>Ldic009>L`03000mkcg_00Hmk`03
000aS36<00XaS003000mkcg_00Hmk`03001>Ldic009>L`03001JeU[F009JeP03001W>FLi009W>@05
001cW7>LLi`0000=Ool01000LiacW0000VLi0P0000E>L`00?Nl0036<008000046<H000aS0003Ool1
000000eoo`800006Ool0000036<001S60P0000<aS000?Nl00P000U[F00<006Li00000W>L00<007oo
Ool02Woo00<007>LLi`00W>L00<006LiIcT00VLi00D005[FF]IJeP0000E>L`03000mkcg_00Hmk`03
000aS36<00XaS003000mkcg_00Hmk`03001>Ldic00=>L`05001JeU[FF]H00004IcT00`00LiacW002
Li`00`00Oomoo`0:Ool03@00LiacW000IcT005[F001>L`00?Nl0036<008000046<H000aS0002Ool2
000000eoo`03001oogoo0080000336<001S600800003<H`003g_0080009JeP03001W>@00009cW003
001oogoo00Yoo`03001cW7>L009cW003001W>FLi009W>@05001JeU[FF]H00005CW<00`00?Nlmk`06
?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl00`00CW=>L`03CW<01@00F]IJeU[F000016Li00<007>L
Li`00W>L00<007ooOol02Woo00d007>LLi`006Li001JeP00CW<003g_000aS002000011S6000<H`00
0goo0@00000=Ool00`00Oomoo`0200000`aS000HaP0200000c6<000mk`020002F]H00`00IcT00002
Li`00`00Oomoo`0:Ool00`00LiacW002Li`01@00IcUW>FLi000015[F00<004icCW<00dic00<003g_
?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00D005[FF]IJeP0000AW>@03
001cW7>L009cW003001oogoo00Yoo`0=001cW7>L001W>@00F]H004ic000mk`00<H`00P0000@HaP00
36<000=oo`4000003Goo00<007ooOol00P0000<<H`006<H00P0000<aS000?Nl00P000U[F00<006Li
00000W>L00<007ooOol02Goo00<007>LLi`00g>L00D006LiIcUW>@0000AJeP03001>Ldic00=>L`03
000mkcg_00Hmk`03000aS36<00XaS003000mkcg_00Hmk`03001>Ldic00=>L`05001JeU[FF]H00004
IcT00`00LiacW003Li`00`00Oomoo`09Ool03@00LiacW000IcT005[F001>L`00?Nl0036<00800004
6<H000aS0003Ool1000000eoo`03001oogoo0080000336<001S60080000:<H`003g_001>L`00F]H0
06Li0003Li`00`00Oomoo`08Ool00`00LiacW002Li`00`00IcUW>@02IcT00`00F]IJeP02F]H00`00
CW=>L`03CW<00`00?Nlmk`06?Nl00`00<H`aS00:<H`00`00?Nlmk`06?Nl00`00CW=>L`03CW<00`00
F]IJeP02F]H00`00IcUW>@02IcT00`00LiacW002Li`00`00Oomoo`08Ool03P00LiacW7>L001W>@00
F]H004ic000mk`00<H`2000011S6000<H`000goo0@00000=Ool00`00Oomoo`0200000`aS000HaP02
00002S6<000mk`00CW<005[F001W>@000g>L00<007ooOol01goo00<007>LLi`00g>L00<006LiIcT0
0VLi00<005[FF]H00U[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<<H`02S6<00<003g_?Nl0
1Sg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00VLi00<007>LLi`00W>L00<007ooOol0
27oo00h007>LLiacW000IcT005[F001>L`00?Nl0036<0P0000@HaP0036<000=oo`4000003Goo0P00
00Ioo`00000<H`006<H200002S6<000mk`00CW<005[F001W>@0017>L00<007ooOol01Woo00<007>L
Li`00g>L00<006LiIcT00VLi00<005[FF]H00U[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<
<H`02S6<00<003g_?Nl01Sg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00VLi00<007>L
Li`00g>L00<007ooOol01Woo00<007>LLi`00W>L00X006Li001JeP00CW<003g_000aS08000046<H0
00aS0002Ool2000000eoo`03001oogoo0080000=36<000009BT0036<000mk`00CW<005[F00000VLi
00D007>LLiacW00000Moo`03001cW7>L00=cW003001W>FLi009W>@03001JeU[F009JeP03001>Ldic
00=>L`03000mkcg_00Hmk`03000aS36<00/aS003000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F
009JeP03001W>FLi009W>@03001cW7>L00=cW003001oogoo00Eoo`05001cW7>LLi`00002IcT02000
F]H004ic000mk`00<H`2000011S6000<H`000goo0@00000=Ool00`00Oomoo`0200003@aS000002DY
000aS000?Nl004ic001JeP00009W>@03001cW7>L009cW003001oogoo00=oo`03001cW7>L00AcW003
001W>FLi009W>@03001JeU[F009JeP03001>Ldic00=>L`03000mkcg_00Hmk`03000aS36<00/aS003
000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F009JeP03001W>FLi009W>@03001cW7>L00AcW003
001oogoo00=oo`03001cW7>L009cW00;001W>FLi001JeP00CW<003g_000aS002000011S6000<H`00
0goo0@00000=Ool00`00Oomoo`0200003@aS000002DY000aS000?Nl004ic001JeP00009W>@03001c
W7>L00=cW003001oogoo00<000IcW003001W>FLi009W>@03001JeU[F009JeP03001>Ldic00=>L`03
000mkcg_00Hmk`03000aS36<00/aS003000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F009JeP03
001W>FLi009W>@03001cW7>L00AcW0<0009oo`03001cW7>L00=cW00;001W>FLi001JeP00CW<003g_
000aS002000011S6000<H`000goo0@00000=Ool00`00Oomoo`0200003@aS000002DY000aS000?Nl0
04ic001JeP00009W>@03001cW7>L00AcW08000UcW003001W>FLi009W>@03001JeU[F009JeP03001>
Ldic00=>L`03000mkcg_00Hmk`03000aS36<00/aS003000mkcg_00Lmk`03001>Ldic00=>L`03001J
eU[F009JeP03001W>FLi009W>@03001cW7>L00McW08000IcW00;001W>FLi001JeP00CW<003g_000a
S002000011S6000<H`000goo0@00000=Ool04000Oomoo`0036<001S6000U:@00<H`003g_001>L`00
0U[F00@006LiIcT0011cW005001W>FLiIcT00005F]H00`00CW=>L`03CW<00`00?Nlmk`06?Nl00`00
<H`aS00<<H`00`00?Nlmk`06?Nl00`00CW=>L`03CW<00`00F]IJeP02F]H00`00IcUW>@02IcT00`00
LiacW00>Li`01000IcUW>@000U[F00d004ic000mk`00<H`002DY000HaP0036<00002Ool1000000eo
o`0@001oogoo000<H`006<H002DY000aS000?Nl004ic0002F]H01000IcUW>@003g>L00<006LiIcT0
0VLi00<005[FF]H00e[F00<004icCW<00dic00<003g_?Nl01Sg_00<0036<<H`0336<00<003g_?Nl0
1Sg_00<004icCW<00dic00<005[FF]H00U[F00<006LiIcT00fLi00<007>LLi`03G>L00@006LiIcT0
009JeP0=001>L`00?Nl0036<000U:@006<H000aS00000Woo0@00000=Ool200003Woo36<<H`006<H0
02DY000aS000?Nl004ic0002F]H01000IcUW>@003g>L00<006LiIcT00VLi00<005[FF]H00e[F00<0
04icCW<00dic00<003g_?Nl01Sg_00<0036<<H`0336<00<003g_?Nl01Sg_00<004icCW<00dic00<0
05[FF]H00U[F00<006LiIcT00fLi00<007>LLi`03G>L00@006LiIcT0009JeP0<001>L`00?Nl0036<
000U:@006<H000aS0Woo0P00000=Ool00`00Oomoo`0236<02`006<H002DY000aS000?Nl004ic0000
0U[F00D006LiIcUW>@0000ecW003001W>FLi009W>@03001JeU[F00=JeP03001>Ldic00=>L`03000m
kcg_00Lmk`03000aS36<00`aS003000mkcg_00Lmk`03001>Ldic00=>L`03001JeU[F009JeP03001W
>FLi00=W>@03001cW7>L00]cW007001W>FLiIcT005[F00000Tic00X003g_000aS0009BT001S6000<
H`=oo`4000003Goo00<007ooOol00PaS00/001S6000U:@00<H`003g_001>L`00009JeP05001W>FLi
IcT0000<Li`00`00IcUW>@03IcT00`00F]IJeP03F]H00`00CW=>L`03CW<00`00?Nlmk`07?Nl00`00
<H`aS00<<H`00`00?Nlmk`07?Nl00`00CW=>L`03CW<00`00F]IJeP03F]H00`00IcUW>@03IcT00`00
LiacW00:Li`01`00IcUW>FLi001JeP00009>L`0:000mk`00<H`002DY000HaP0036<3Ool1000000eo
o`03001oogoo008<H`0;000HaP009BT0036<000mk`00CW<00002F]H00`00IcUW>@02IcT00`00Liac
W008Li`00`00IcUW>@04IcT00`00F]IJeP03F]H00`00CW=>L`03CW<00`00?Nlmk`07?Nl00`00<H`a
S00<<H`00`00?Nlmk`07?Nl00`00CW=>L`03CW<00`00F]IJeP03F]H00`00IcUW>@04IcT00`00Liac
W008Li`00`00IcUW>@02IcT00`00F]H00002CW<02P00?Nl0036<000U:@006<H000aS0goo0@00000=
Ool00`00Oomoo`0236<02`006<H002DY000aS000?Nl004ic00000U[F00<006LiIcT00fLi00<007>L
Li`01W>L00<006LiIcT01FLi00<005[FF]H00e[F00<004icCW<00dic00<003g_?Nl01cg_00<0036<
<H`0336<00<003g_?Nl01cg_00<004icCW<00dic00<005[FF]H00e[F00<006LiIcT01FLi00<007>L
Li`01W>L00<006LiIcT00fLi00<005[F00000Tic00X003g_000aS0009BT001S6000<H`=oo`400000
3Goo00D007ooOol<H`00008HaP07000U:@00<H`003g_00000Tic00@005[FF]H000EW>@03001cW7>L
00AcW003001W>FLi00EW>@03001JeU[F00=JeP03001>Ldic00=>L`03000mkcg_00Pmk`03000aS36<
00`aS003000mkcg_00Pmk`03001>Ldic00=>L`03001JeU[F00=JeP03001W>FLi00EW>@03001cW7>L
00AcW003001W>FLi00=W>@04001JeU[F0002CW<01`00?Nl0036<000U:@00008HaP04001oogooOol1
000000eoo`05001oogoo36<000026<H01`009BT0036<000mk`00009>L`04001JeU[F0006IcT60007
IcT00`00F]IJeP04F]H00`00CW=>L`03CW<00`00?Nlmk`08?Nl00`00<H`aS00<<H`00`00?Nlmk`08
?Nl00`00CW=>L`03CW<00`00F]IJeP03F]H00`00IcUW>@06IcT60006IcT01P00F]IJeP00CW<0008m
k`05000aS0009BT000026<H01000Oomoogoo0@00000=Ool200000goo36<000026<H01`009BT0036<
000mk`00009>L`04001JeU[F000CIcT00`00F]IJeP03F]H00`00CW=>L`04CW<00`00?Nlmk`08?Nl0
0`00<H`aS00<<H`00`00?Nlmk`08?Nl00`00CW=>L`03CW<00`00F]IJeP04F]H00`00IcUW>@0AIcT0
1P00F]IJeP00CW<0008mk`05000aS0009BT000026<H00`00Oomoo`02000000eoo`04001oogoo0002
6<H01P009BTU:@00<H`0008mk`04001>Ldic0002F]H00`00IcUW>@0?IcT00`00F]IJeP03F]H00`00
CW=>L`04CW<00`00?Nlmk`08?Nl00`00<H`aS00><H`00`00?Nlmk`07?Nl00`00CW=>L`04CW<00`00
F]IJeP04F]H00`00IcUW>@0?IcT01000F]IJeP000Tic00P003g_?Nl0036<000U:@000aS600<007oo
Ool00@00000=Ool01000Oomoo`000QS600H002DY9BT0036<0002?Nl01000CW=>L`000e[F00<006Li
IcT03FLi00<005[FF]H015[F00<004icCW<014ic00<003g_?Nl023g_00<0036<<H`03S6<00<003g_
?Nl01cg_0P001Tic00<005[FF]H015[F00<006LiIcT03FLi00D005[FF]IJeP00009>L`06000mkcg_
000aS0000RDY00@001S66<H0009oo`4000003Goo00@007ooOol0008HaP06000U:BDY000aS0000Sg_
00@004icCW<000AJeP03001W>FLi00]W>@03001JeU[F00EJeP03001>Ldic00A>L`03000mkcg_00Pm
k`03000aS36<00haS003000mkcg_00Pmk`03001>Ldic00A>L`03001JeU[F00EJeP03001W>FLi00]W
>@03001JeU[F009JeP04001>Ldic0002?Nl00`00<H`000029BT010006<HHaP000Woo0@00000=Ool0
0`00Oomoo`036<H01P009BTU:@00<H`0008mk`04001>Ldic0005F]H00`00IcUW>@09IcT00`00F]IJ
eP05F]H00`00CW=>L`05CW<00`00?Nlmk`08?Nl00`00<H`aS00><H`00`00?Nlmk`08?Nl00`00CW=>
L`05CW<00`00F]IJeP05F]H00`00IcUW>@09IcT00`00F]IJeP03F]H01000CW=>L`000Sg_00<0036<
00000RDY00<001S66<H00goo0@000007Ool30003Ool00`00Oomoo`036<H00`009BT00002<H`01000
?Nlmk`000dic00<005[FF]H00e[F00<006LiIcT01VLi0P0025[F00<004icCW<014ic00<003g_?Nl0
2Cg_00<0036<<H`03S6<00<003g_?Nl02Cg_00<004icCW<014ic00<005[FF]H01U[F0P0026Li00<0
05[FF]H00e[F00D004icCW=>L`00008mk`03000aS000008U:@03000HaQS600=oo`40000027oo00<0
07ooOol00Woo00<007ooOol00aS600<002DY00000S6<00@003g_?Nl000=>L`03001JeU[F00=JeP<0
009W>@@000UJeP03001>Ldic00E>L`03000mkcg_00Tmk`03000aS36<00haS003000mkcg_00Tmk`03
001>Ldic00E>L`03001JeU[F00MJeP@0009W>@8000IJeP05001>LdicCW<00002?Nl00`00<H`00002
9BT00`006<HHaP03Ool1000000Qoo`03001oogoo009oo`800007OolHaQS66<H002DY00000S6<00@0
03g_?Nl000=>L`03001JeU[F00IJeP8000]JeP8000Q>L`03000mkcg_00Tmk`03000aS36<00haS003
000mkcg_00Tmk`03001>Ldic00I>L`03001JeU[F00YJeP8000QJeP05001>LdicCW<00002?Nl00`00
<H`000029BT00`006<HHaP02Ool2000000Qoo`03001oogoo009oo`03001oogoo008HaP04000U:BDY
0002<H`01@00?Nlmkcg_00000dic00<005[FF]H04E[F00<004icCW<01dic00<003g_?Nl02Cg_00<0
036<<H`0436<00<003g_?Nl02Cg_00<004icCW<01Tic00<005[FF]H04U[F00D004icCW=>L`0000<m
k`04000aS36<00029BT00`006<Ioo`02Ool1000000Qoo`03001oogoo009oo`05001oogoo6<H00003
9BT01000<H`aS0000cg_00<004icCW<00Tic00<005[FF]H03e[F00<004icCW<01dic00<003g_?Nl0
2Sg_00<0036<<H`0436<00<003g_?Nl02Sg_00<004icCW<01Tic00<005[FF]H045[F00<004icCW<0
0Tic00D003g_?Nlmk`00008aS005000U:BDY000HaP03Ool1000000Moo`8000Aoo`05001oogoo6<H0
00039BT01000<H`aS0000cg_00<004icCW<00Tic0P003e[F0P002Dic00<003g_?Nl02Sg_00<0036<
<H`0436<00<003g_?Nl02Sg_00<004icCW<01dic0P003e[F0P0014ic00D003g_?Nlmk`00008aS005
000U:BDY000HaP03Ool1000000eoo`04001oogoo00049BT01000<H`aS0000cg_00<004icCW<014ic
0P002e[F0P002Tic00<003g_?Nl02cg_00<0036<<H`0436<00<003g_?Nl02cg_00<004icCW<024ic
0P002e[F0P001Tic00D003g_?Nlmk`00008aS005000U:BDY9BT00003Ool1000000eoo`04001oogoo
00039BT00`00<H`aS002<H`01@00?Nlmkcg_00001dic00<005[FF]H01U[F0P002dic00<003g_?Nl0
2cg_00<0036<<H`04S6<00<003g_?Nl02cg_00<004icCW<02Dic0P0025[F00<004icCW<01Dic00D0
03g_?Nlmk`0000<aS003000U:BDY008U:@03001oogoo004000003Goo0P0000=oobDY9BT00RDY00<0
036<<H`00S6<00D003g_?Nlmk`0000Q>L`P000]>L`8000hmk`03000aS36<018aS003000mkcg_00`m
k`8000]>L`P000Q>L`05000mkcg_?Nl00003<H`00`009BTU:@029BT2Ool2000000eoo`03001oogoo
00@U:@03000aS36<008aS003000mkcg_008mk`03001>Ldic01M>L`03000mkcg_00hmk`03000aS36<
018aS003000mkcg_00hmk`03001>Ldic01M>L`03000mkcg_008mk`05000aS36<<H`000049BT3Ool1
000000eoo`03001oogoo00<U:@03000aS36<00<aS003000mkcg_00<mk`03001>Ldic01E>L`03000m
kcg_00hmk`03000aS36<01@aS003000mkcg_00hmk`03001>Ldic01E>L`03000mkcg_00<mk`03000a
S36<008aS004000U:BDY9BT3Ool1000000eoo`03001oogoo00<U:@03000aS36<00<aS003000mkcg_
00@mk`03001>Ldic015>L`<0010mk`8001LaS003000mkcg_00hmk`<001=>L`03000mkcg_00@mk`03
000aS36<008aS080008U:@=oo`4000003Goo00<007ooOol00bDY00<0036<<H`00c6<00<003g_?Nl0
1Cg_0`003Dic0`004Sg_00<0036<<H`05c6<00<003g_?Nl04Cg_0`003Dic0`001cg_00<0036<<H`0
0c6<00<002DY9BT00goo0@00000=Ool00`00Oomoo`029BT00`00<H`aS005<H`00`00?Nlmk`07?Nl4
0006CW<3000D?Nl00`00<H`aS00I<H`00`00?Nlmk`0C?Nl40006CW<30009?Nl00`00<H`aS005<H`0
0`009BUoo`02Ool1000000eoo`05001oogoo9BT00008<H`00`00?Nlmk`0;?Nl6000F?Nl00`00<H`a
S00K<H`00`00?Nlmk`0F?Nl6000<?Nl00`00<H`aS006<H`01000Oomoogoo0@00000=Ool200000goo
000aS008<H`00`00?Nlmk`0V?Nl2000N<H`2000X?Nl00`00<H`aS007<H`00`00Ool00001000000eo
o`03001oogoo00XaS003000mkcg_02@mk`80028aS003000mkcg_02Dmk`03000aS36<00LaS0=oo`40
00003Goo00<007ooOol02c6<00<003g_?Nl08Cg_0P009C6<0`008cg_00<0036<<H`0236<0goo0@00
000=Ool00`00Oomoo`0<<H`2000M?Nl4000Z<H`4000M?Nl2000;<H`3Ool1000000eoo`03001oogoo
00haS0X000Hmk`d0038aS0h000Hmk`T000daS0=oo`4000003Goo00<007ooOol0636<1P00CC6<1P00
5S6<0goo0@00000=Ool00`00Oomoo`27<H`3Ool1000000eoo`800003Ool0000008@aS080005oo`40
004000003Goo00<007ooOol00Sg_00<0036<<H`0Oc6<0P0000@mkgooOomoo`4000003Goo00<007oo
Ool00cg_0P00OS6<00@003g_?Nlmk`=oo`4000003Goo00<007ooOol01Cg_00<0036<<H`02c6<3000
Bc6<2`003C6<00<003g_?Nl00Sg_0goo0@00000=Ool00`00Oomoo`05?Nl00`00<H`aS008<H`3000<
9BT7000m<H`7000;9BT3000:<H`00`00?Nlmk`02?Nl3Ool1000000eoo`03001oogoo00Hmk`03000a
S36<00HaS003000U:BDY01@U:@<003LaS0<001DU:@03000aS36<00HaS003000mkcg_00<mk`=oo`40
00003Goo0P0000=oocg_?Nl013g_00<0036<<H`01C6<00<002DY9BT062DY00<0036<<H`0<S6<0P00
6BDY00<0036<<H`01C6<00<003g_?Nl00cg_0Woo0P00000=Ool00`00Oomoo`06?Nl00`00<H`aS005
<H`00`009BTU:@0H9BT2000a<H`2000K9BT00`00<H`aS005<H`00`00?Nlmk`03?Nl3Ool1000000eo
o`04001oogoo0005?Nl00`00<H`aS004<H`00`009BTU:@0K9BT2000^<H`00`009BTU:@0L9BT00`00
<H`aS004<H`00`00?Nlmk`03?Nl00`00Oomoo`01000000eoo`04001oogoo0006?Nl00`00<H`aS003
<H`00`009BTU:@079BT8000>9BT00`00<H`aS00Z<H`00`009BTU:@0=9BT800089BT00`00<H`aS003
<H`00`00?Nlmk`04?Nl00`00Oomoo`01000000eoo`05001oogooCW<00005?Nl00`00<H`aS002<H`0
0`009BTU:@049BT400086<H3000<9BT00`00<H`aS00X<H`00`009BTU:@0:9BT400086<H300069BT0
0`00<H`aS002<H`00`00?Nlmk`03?Nl01000Oomoogoo0@000007Ool40002Ool01@00Oomoodic0000
1Cg_00<0036<<H`00S6<00<002DY9BT00bDY00<001S66<H03AS60P002RDY00<0036<<H`0:36<00<0
02DY9BT022DY0P003aS600<002DY9BT00bDY00<0036<<H`00S6<00<003g_?Nl00cg_00@007ooOomo
o`40000027oo00<007ooOol00Woo00<007ooOol00Tic00<003g_?Nl00Sg_00D0036<<H`aS00000DU
:@03000HaQS6010HaP03000U:BDY00PU:@03000aS36<02HaS003000U:BDY00PU:@03000HaQS6010H
aP03000U:BDY00<U:@05000aS36<<H`00004?Nl00`00CW=oo`02Ool1000000Uoo`04001oogooOol2
00001GooCW=>Ldic00000cg_00D0036<<H`aS00000@U:@03000HaQS6018HaP03000U:BDY00PU:@03
000aS36<02@aS003000U:BDY00PU:@03000HaQS6018HaP03000U:BDY008U:@05000aS36<<H`00003
?Nl00`00CW=>L`02Ool2000000Yoo`04001oogoo002<Ool1000000Moo`04001oogoo0002Ool01000
Oomoo`001Woo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<0
07ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol00goo00<0
07ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol00goo00<007ooOol017oo00<0
07ooOol017oo00<007ooOol017oo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<0
07ooOol00@000008Ool20003Oon>000009]oo`00Vgoo0000\
\>"],
  ImageRangeCache->{{{0, 154.438}, {154.438, 0}} -> {-2.48762, -2.42095, 
  0.0296336, 0.0296336}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(DensityPlot[
      func[x, \ y], \ {x, \ \(-2\), \ 2}, \ {y, \ \(-2\), \ 2}, \ 
      PlotPoints\  -> \ 60, \ Mesh\  -> \ False]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% DensityGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.240385 0.5 0.240385 [
[.01923 -0.0125 -6 -9 ]
[.01923 -0.0125 6 0 ]
[.25962 -0.0125 -6 -9 ]
[.25962 -0.0125 6 0 ]
[.5 -0.0125 -3 -9 ]
[.5 -0.0125 3 0 ]
[.74038 -0.0125 -3 -9 ]
[.74038 -0.0125 3 0 ]
[.98077 -0.0125 -3 -9 ]
[.98077 -0.0125 3 0 ]
[ 0 0 -0.125 0 ]
[-0.0125 .01923 -12 -4.5 ]
[-0.0125 .01923 0 4.5 ]
[-0.0125 .25962 -12 -4.5 ]
[-0.0125 .25962 0 4.5 ]
[-0.0125 .5 -6 -4.5 ]
[-0.0125 .5 0 4.5 ]
[-0.0125 .74038 -6 -4.5 ]
[-0.0125 .74038 0 4.5 ]
[-0.0125 .98077 -6 -4.5 ]
[-0.0125 .98077 0 4.5 ]
[ 0 0 -0.125 0 ]
[ 0 1 .125 0 ]
[ 1 0 .125 0 ]
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.01923 0 m
.01923 .00625 L
s
[(-2)] .01923 -0.0125 0 1 Mshowa
.25962 0 m
.25962 .00625 L
s
[(-1)] .25962 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(0)] .5 -0.0125 0 1 Mshowa
.74038 0 m
.74038 .00625 L
s
[(1)] .74038 -0.0125 0 1 Mshowa
.98077 0 m
.98077 .00625 L
s
[(2)] .98077 -0.0125 0 1 Mshowa
.125 Mabswid
.06731 0 m
.06731 .00375 L
s
.11538 0 m
.11538 .00375 L
s
.16346 0 m
.16346 .00375 L
s
.21154 0 m
.21154 .00375 L
s
.30769 0 m
.30769 .00375 L
s
.35577 0 m
.35577 .00375 L
s
.40385 0 m
.40385 .00375 L
s
.45192 0 m
.45192 .00375 L
s
.54808 0 m
.54808 .00375 L
s
.59615 0 m
.59615 .00375 L
s
.64423 0 m
.64423 .00375 L
s
.69231 0 m
.69231 .00375 L
s
.78846 0 m
.78846 .00375 L
s
.83654 0 m
.83654 .00375 L
s
.88462 0 m
.88462 .00375 L
s
.93269 0 m
.93269 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .01923 m
.00625 .01923 L
s
[(-2)] -0.0125 .01923 1 0 Mshowa
0 .25962 m
.00625 .25962 L
s
[(-1)] -0.0125 .25962 1 0 Mshowa
0 .5 m
.00625 .5 L
s
[(0)] -0.0125 .5 1 0 Mshowa
0 .74038 m
.00625 .74038 L
s
[(1)] -0.0125 .74038 1 0 Mshowa
0 .98077 m
.00625 .98077 L
s
[(2)] -0.0125 .98077 1 0 Mshowa
.125 Mabswid
0 .06731 m
.00375 .06731 L
s
0 .11538 m
.00375 .11538 L
s
0 .16346 m
.00375 .16346 L
s
0 .21154 m
.00375 .21154 L
s
0 .30769 m
.00375 .30769 L
s
0 .35577 m
.00375 .35577 L
s
0 .40385 m
.00375 .40385 L
s
0 .45192 m
.00375 .45192 L
s
0 .54808 m
.00375 .54808 L
s
0 .59615 m
.00375 .59615 L
s
0 .64423 m
.00375 .64423 L
s
0 .69231 m
.00375 .69231 L
s
0 .78846 m
.00375 .78846 L
s
0 .83654 m
.00375 .83654 L
s
0 .88462 m
.00375 .88462 L
s
0 .93269 m
.00375 .93269 L
s
.25 Mabswid
0 0 m
0 1 L
s
.01923 .99375 m
.01923 1 L
s
.25962 .99375 m
.25962 1 L
s
.5 .99375 m
.5 1 L
s
.74038 .99375 m
.74038 1 L
s
.98077 .99375 m
.98077 1 L
s
.125 Mabswid
.06731 .99625 m
.06731 1 L
s
.11538 .99625 m
.11538 1 L
s
.16346 .99625 m
.16346 1 L
s
.21154 .99625 m
.21154 1 L
s
.30769 .99625 m
.30769 1 L
s
.35577 .99625 m
.35577 1 L
s
.40385 .99625 m
.40385 1 L
s
.45192 .99625 m
.45192 1 L
s
.54808 .99625 m
.54808 1 L
s
.59615 .99625 m
.59615 1 L
s
.64423 .99625 m
.64423 1 L
s
.69231 .99625 m
.69231 1 L
s
.78846 .99625 m
.78846 1 L
s
.83654 .99625 m
.83654 1 L
s
.88462 .99625 m
.88462 1 L
s
.93269 .99625 m
.93269 1 L
s
.25 Mabswid
0 1 m
1 1 L
s
.99375 .01923 m
1 .01923 L
s
.99375 .25962 m
1 .25962 L
s
.99375 .5 m
1 .5 L
s
.99375 .74038 m
1 .74038 L
s
.99375 .98077 m
1 .98077 L
s
.125 Mabswid
.99625 .06731 m
1 .06731 L
s
.99625 .11538 m
1 .11538 L
s
.99625 .16346 m
1 .16346 L
s
.99625 .21154 m
1 .21154 L
s
.99625 .30769 m
1 .30769 L
s
.99625 .35577 m
1 .35577 L
s
.99625 .40385 m
1 .40385 L
s
.99625 .45192 m
1 .45192 L
s
.99625 .54808 m
1 .54808 L
s
.99625 .59615 m
1 .59615 L
s
.99625 .64423 m
1 .64423 L
s
.99625 .69231 m
1 .69231 L
s
.99625 .78846 m
1 .78846 L
s
.99625 .83654 m
1 .83654 L
s
.99625 .88462 m
1 .88462 L
s
.99625 .93269 m
1 .93269 L
s
.25 Mabswid
1 0 m
1 1 L
s
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
% Start of gray image
p
.01923 .01923 translate
.96154 .96154 scale
60 string
60 60 8 [60 0 0 60 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  Mimage
9C90827365574B423A3532313234383C41464B50555A5E62
65686A6C6D6E6E6D6C6A6865625E5A55504B46413C383432
3132353A424B57657382909C
958B7F72665B5148423E3B3A3B3D4043474C5055595D6164
67696B6C6D6E6E6D6C6B696764615D5955504C4743403D3B
3A3B3E4248515B66727F8B95
8E857C71675E564F4A4644444446484B4E5256595D606366
686A6B6D6D6E6E6D6D6B6A686663605D5956524E4B484644
4444464A4F565E67717C858E
8680787169625B56524F4E4D4E4F515356585B5E61636567
696B6C6D6D6E6E6D6D6C6B69676563615E5B585653514F4E
4D4E4F52565B626971788086
7F7B75706A65615D5A5857575758595B5D5F616365666869
6B6C6C6D6E6E6E6E6D6C6C6B6968666563615F5D5B595857
5757585A5D61656A70757B7F
7875726F6C69666463616161616162636465666768696A6B
6C6D6D6D6E6E6E6E6D6D6D6C6B6A69686766656463626161
616161636466696C6F727578
70706F6E6D6D6C6B6B6B6B6B6B6B6B6B6B6C6C6C6C6D6D6D
6D6D6E6E6E6E6E6E6E6E6D6D6D6D6D6C6C6C6C6B6B6B6B6B
6B6B6B6B6B6C6D6D6E6F7070
696A6C6D6F70727373747474747474737372727170706F6F
6F6E6E6E6E6E6E6E6E6E6E6F6F6F70707172727373747474
747474737372706F6D6C6A69
6165686C7074777A7C7D7E7E7E7D7C7B7A79777674737271
706F6F6E6E6E6E6E6E6F6F70717273747677797A7B7C7D7E
7E7E7D7C7A7774706C686561
5A5F656C72787D818486888888878583817F7D7B78767473
71706F6F6E6E6E6E6F6F7071737476787B7D7F8183858788
88888684817D78726C655F5A
535A626B737B82888C8F919291908E8B8885827F7C797775
7371706F6E6E6E6E6F7071737577797C7F8285888B8E9091
92918F8C88827B736B625A53
4B545F6A757F888F94989A9B9B999693908C8884807D7977
7472706F6E6E6E6E6F70727477797D8084888C909396999B
9B9A98948F887F756A5F544B
444F5C6976828D969CA1A4A4A4A29F9B96928D8884807C78
7573716F6E6E6E6E6F717375787C8084888D92969B9FA2A4
A4A4A19C968D8276695C4F44
3E4A59687786929CA4A9ACADADAAA7A29D98928D87837E7A
777471706E6E6E6E707174777A7E83878D92989DA2A7AAAD
ADACA9A49C92867768594A3E
37455668798997A2ABB2B5B6B5B3AEA9A49D97918B85807C
787572706F6E6E6F707275787C80858B91979DA4A9AEB3B5
B6B5B2ABA297897968564537
314153677A8C9CA9B3B9BDBFBEBBB6B0AAA39C958E88827D
797572706F6E6E6F707275797D82888E959CA3AAB0B6BBBE
BFBDB9B3A99C8C7A67534131
2B3C51667B8FA0AEB9C1C5C7C5C2BDB7B0A8A199928B857F
7A7673706F6E6E6F7073767A7F858B9299A1A8B0B7BDC2C5
C7C5C1B9AEA08F7B66513C2B
25384E657C92A5B4C0C8CDCECDC9C4BDB5ADA59D958D8680
7B7773716F6E6E6F7173777B80868D959DA5ADB5BDC4C9CD
CECDC8C0B4A5927C654E3825
1F344C657E94A9B9C6CFD4D5D4D0CAC3BBB2A9A098908882
7C7774716F6E6E6F7174777C82889098A0A9B2BBC3CAD0D4
D5D4CFC6B9A9947E654C341F
1A304A647F97ACBECCD5DADCDBD7D0C9C0B6ADA39A928A83
7D7874716F6E6E6F7174787D838A929AA3ADB6C0C9D0D7DB
DCDAD5CCBEAC977F644A301A
162D48648099B0C3D1DBE1E2E1DCD6CEC4BBB0A69D948C84
7E7974716F6E6E6F7174797E848C949DA6B0BBC4CED6DCE1
E2E1DBD1C3B0998064482D16
112A4663809BB3C7D6E0E6E8E6E2DBD2C9BEB4A99F968D85
7F7975716F6E6E6F7175797F858D969FA9B4BEC9D2DBE2E6
E8E6E0D6C7B39B8063462A11
0D274463819DB6CADAE5EBEDEBE7E0D7CCC2B7ACA1978E86
7F7A75716F6E6E6F71757A7F868E97A1ACB7C2CCD7E0E7EB
EDEBE5DACAB69D816344270D
0A244262829FB9CEDEE9F0F2F0EBE4DAD0C5B9AEA3998F87
807A75726F6E6E6F72757A80878F99A3AEB9C5D0DAE4EBF0
F2F0E9DECEB99F826242240A
0722416282A0BBD1E1EDF3F6F4EFE7DED3C7BBB0A59A9088
817A75726F6E6E6F72757A8188909AA5B0BBC7D3DEE7EFF4
F6F3EDE1D1BBA08262412207
0520406283A2BDD3E4F0F7F9F7F2EAE0D5C9BDB1A69B9189
817B76726F6E6E6F72767B8189919BA6B1BDC9D5E0EAF2F7
F9F7F0E4D3BDA28362402005
031F3F6283A3BED5E6F2F9FBFAF4ECE2D7CBBFB3A79C9289
817B76726F6E6E6F72767B8189929CA7B3BFCBD7E2ECF4FA
FBF9F2E6D5BEA383623F1F03
011E3F6184A3BFD6E8F4FBFDFCF6EEE4D9CCC0B4A89D938A
827B76726F6E6E6F72767B828A939DA8B4C0CCD9E4EEF6FC
FDFBF4E8D6BFA384613F1E01
001D3E6184A4C0D7E9F5FDFFFDF7EFE5DACDC1B4A89D938A
827B76726F6E6E6F72767B828A939DA8B4C1CDDAE5EFF7FD
FFFDF5E9D7C0A484613E1D00
001C3E6184A4C0D8EAF6FDFFFEF8F0E6DACEC1B5A99D938A
827B76726F6E6E6F72767B828A939DA9B5C1CEDAE6F0F8FE
FFFDF6EAD8C0A484613E1C00
001C3E6184A4C0D8EAF6FDFFFEF8F0E6DACEC1B5A99D938A
827B76726F6E6E6F72767B828A939DA9B5C1CEDAE6F0F8FE
FFFDF6EAD8C0A484613E1C00
001D3E6184A4C0D7E9F5FDFFFDF7EFE5DACDC1B4A89D938A
827B76726F6E6E6F72767B828A939DA8B4C1CDDAE5EFF7FD
FFFDF5E9D7C0A484613E1D00
011E3F6184A3BFD6E8F4FBFDFCF6EEE4D9CCC0B4A89D938A
827B76726F6E6E6F72767B828A939DA8B4C0CCD9E4EEF6FC
FDFBF4E8D6BFA384613F1E01
031F3F6283A3BED5E6F2F9FBFAF4ECE2D7CBBFB3A79C9289
817B76726F6E6E6F72767B8189929CA7B3BFCBD7E2ECF4FA
FBF9F2E6D5BEA383623F1F03
0520406283A2BDD3E4F0F7F9F7F2EAE0D5C9BDB1A69B9189
817B76726F6E6E6F72767B8189919BA6B1BDC9D5E0EAF2F7
F9F7F0E4D3BDA28362402005
0722416282A0BBD1E1EDF3F6F4EFE7DED3C7BBB0A59A9088
817A75726F6E6E6F72757A8188909AA5B0BBC7D3DEE7EFF4
F6F3EDE1D1BBA08262412207
0A244262829FB9CEDEE9F0F2F0EBE4DAD0C5B9AEA3998F87
807A75726F6E6E6F72757A80878F99A3AEB9C5D0DAE4EBF0
F2F0E9DECEB99F826242240A
0D274463819DB6CADAE5EBEDEBE7E0D7CCC2B7ACA1978E86
7F7A75716F6E6E6F71757A7F868E97A1ACB7C2CCD7E0E7EB
EDEBE5DACAB69D816344270D
112A4663809BB3C7D6E0E6E8E6E2DBD2C9BEB4A99F968D85
7F7975716F6E6E6F7175797F858D969FA9B4BEC9D2DBE2E6
E8E6E0D6C7B39B8063462A11
162D48648099B0C3D1DBE1E2E1DCD6CEC4BBB0A69D948C84
7E7974716F6E6E6F7174797E848C949DA6B0BBC4CED6DCE1
E2E1DBD1C3B0998064482D16
1A304A647F97ACBECCD5DADCDBD7D0C9C0B6ADA39A928A83
7D7874716F6E6E6F7174787D838A929AA3ADB6C0C9D0D7DB
DCDAD5CCBEAC977F644A301A
1F344C657E94A9B9C6CFD4D5D4D0CAC3BBB2A9A098908882
7C7774716F6E6E6F7174777C82889098A0A9B2BBC3CAD0D4
D5D4CFC6B9A9947E654C341F
25384E657C92A5B4C0C8CDCECDC9C4BDB5ADA59D958D8680
7B7773716F6E6E6F7173777B80868D959DA5ADB5BDC4C9CD
CECDC8C0B4A5927C654E3825
2B3C51667B8FA0AEB9C1C5C7C5C2BDB7B0A8A199928B857F
7A7673706F6E6E6F7073767A7F858B9299A1A8B0B7BDC2C5
C7C5C1B9AEA08F7B66513C2B
314153677A8C9CA9B3B9BDBFBEBBB6B0AAA39C958E88827D
797572706F6E6E6F707275797D82888E959CA3AAB0B6BBBE
BFBDB9B3A99C8C7A67534131
37455668798997A2ABB2B5B6B5B3AEA9A49D97918B85807C
787572706F6E6E6F707275787C80858B91979DA4A9AEB3B5
B6B5B2ABA297897968564537
3E4A59687786929CA4A9ACADADAAA7A29D98928D87837E7A
777471706E6E6E6E707174777A7E83878D92989DA2A7AAAD
ADACA9A49C92867768594A3E
444F5C6976828D969CA1A4A4A4A29F9B96928D8884807C78
7573716F6E6E6E6E6F717375787C8084888D92969B9FA2A4
A4A4A19C968D8276695C4F44
4B545F6A757F888F94989A9B9B999693908C8884807D7977
7472706F6E6E6E6E6F70727477797D8084888C909396999B
9B9A98948F887F756A5F544B
535A626B737B82888C8F919291908E8B8885827F7C797775
7371706F6E6E6E6E6F7071737577797C7F8285888B8E9091
92918F8C88827B736B625A53
5A5F656C72787D818486888888878583817F7D7B78767473
71706F6F6E6E6E6E6F6F7071737476787B7D7F8183858788
88888684817D78726C655F5A
6165686C7074777A7C7D7E7E7E7D7C7B7A79777674737271
706F6F6E6E6E6E6E6E6F6F70717273747677797A7B7C7D7E
7E7E7D7C7A7774706C686561
696A6C6D6F70727373747474747474737372727170706F6F
6F6E6E6E6E6E6E6E6E6E6E6F6F6F70707172727373747474
747474737372706F6D6C6A69
70706F6E6D6D6C6B6B6B6B6B6B6B6B6B6B6C6C6C6C6D6D6D
6D6D6E6E6E6E6E6E6E6E6D6D6D6D6D6C6C6C6C6B6B6B6B6B
6B6B6B6B6B6C6D6D6E6F7070
7875726F6C69666463616161616162636465666768696A6B
6C6D6D6D6E6E6E6E6D6D6D6C6B6A69686766656463626161
616161636466696C6F727578
7F7B75706A65615D5A5857575758595B5D5F616365666869
6B6C6C6D6E6E6E6E6D6C6C6B6968666563615F5D5B595857
5757585A5D61656A70757B7F
8680787169625B56524F4E4D4E4F515356585B5E61636567
696B6C6D6D6E6E6D6D6C6B69676563615E5B585653514F4E
4D4E4F52565B626971788086
8E857C71675E564F4A4644444446484B4E5256595D606366
686A6B6D6D6E6E6D6D6B6A686663605D5956524E4B484644
4444464A4F565E67717C858E
958B7F72665B5148423E3B3A3B3D4043474C5055595D6164
67696B6C6D6E6E6D6C6B696764615D5955504C4743403D3B
3A3B3E4248515B66727F8B95
9C90827365574B423A3532313234383C41464B50555A5E62
65686A6C6D6E6E6D6C6A6865625E5A55504B46413C383432
3132353A424B57657382909C
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{156.438, 156.438},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002L0000W1000`40O003h00OW7oo002LOol009ao
o`004Goo10007Woo0`007Goo0P007goo0`007goo10000Woo000BOol00`00Oomoo`0OOol00`00Oomo
o`0KOol01000Oomoo`007goo00<007ooOol07goo00<007ooOol00Woo000;Ool50003Ool00`00Oomo
o`0GOol50002Ool00`00Oomoo`0KOol01000Oomoo`007goo00<007ooOol087oo00<007ooOol00Goo
000DOol00`00Oomoo`0MOol00`00Oomoo`0KOol01000Oomoo`007goo00<007ooOol08Goo0@000Goo
0Goo000AOol01000Oomoo`007goo00<007ooOol06goo00@007ooOol001moo`03001oogoo01ioo`04
001oogoo0002Ool0019oo`8001moo`8001ioo`8001moo`80025oo`8000=oo`00W7oo002LOol009ao
o`001goo10000WooS`000008Ool00`00Oomoo`02Ool00`00Oomoo`2;Ool100000003Ool0000000<0
00=oo`04001oogooOol2002;Ool2000000Yoo`04001oogoo0002Ool2CW<3BU82@Q02>Lh2<H`3:DX2
9BT28@P27>L<6<H47>L58@P29BT5:DX4;F/5<H`F=Jd5<H`4;F/5:DX29BT58@P47>L<6<H27>L28@P2
9BT3:DX2<H`2>Lh2@Q03BU82CW<3Ool1000000Moo`04001oogoo0002Ool00`00Oomoo`02CW<3BU82
@Q02>Lh2<H`3:DX29BT28@P27>L<6<H47>L58@P29BT5:DX4;F/5<H`F=Jd5<H`4;F/5:DX29BT58@P4
7>L<6<H27>L28@P29BT3:DX2<H`2>Lh2@Q03BU82CW<3Ool1000000Qoo`8000=oo`03001oogoo009:
DP=6<@8mk`8icP8aS0<]J`8YBP8U:@8Q20`Li`LQ208U:@@YBPD]J`LaS1He[@LaS0D]J`@YBP8U:@LQ
20`Li`8Q208U:@8YBP<]J`8aS08icP8mk`=6<@9:DP=oo`4000003Goo00<007ooOol00TYB0dHa0Sg_
0SW>0S6<0be[0RU:0RDY0R4831cW1b480RDY12U:1Be[1c6<5SF]1c6<1Be[12U:0RDY1b4831cW0R48
0RDY0RU:0be[0S6<0SW>0Sg_0dHa0TYB0goo0@00000=Ool00`00Oomoo`02BU83AS42?Nl2>Lh2<H`3
;F/2:DX29BT28@P<7>L78@P29BT4:DX5;F/7<H`F=Jd7<H`5;F/4:DX29BT78@P<7>L28@P29BT2:DX3
;F/2<H`2>Lh2?Nl3AS42BU83Ool1000000eoo`03001oogoo0096<@=2408mk`8icP8aS0<]J`8YBP@U
:@`Q20LU:@@YBPD]J`HaS1`e[@HaS0D]J`@YBPLU:@`Q20@U:@8YBP<]J`8aS08icP8mk`=24096<@=o
o`4000003Goo0P0000=oodHaAS400d8@0Sg_0SW>0S6<0be[0RU:12DY32481bDY12U:1Be[1S6<73F]
1S6<1Be[12U:1bDY324812DY0RU:0be[0S6<0SW>0Sg_0d8@0THa0Woo0P00000=Ool00`00Oomoo`05
@Q02?Nl2>Lh2=Jd3<H`2;F/4:DX<9BT7:DX6;F/9<H`L=Jd9<H`6;F/7:DX<9BT4:DX2;F/3<H`2=Jd2
>Lh2?Nl5@Q03Ool1000000eoo`03001oogoo00E2408mk`8icP8e[@<aS08]J`@YBP`U:@LYBPH]J`Ta
S1`e[@TaS0H]J`LYBP`U:@@YBP8]J`<aS08e[@8icP8mk`E240=oo`4000003Goo00<007ooOol01Cg_
13W>0SF]1C6<1be[1RU:32e[2C6<93F]2C6<32e[1RU:1be[1C6<0SF]13W>1Cg_0goo0@00000=Ool0
0`00Oomoo`05?Nl4>Lh2=Jd5<H`7;F/6:DX<;F/9<H`T=Jd9<H`<;F/6:DX7;F/5<H`2=Jd4>Lh5?Nl3
Ool1000000eoo`03001oogoo008mk`DicPLe[AlaS2he[AlaS0Le[@DicP8mk`=oo`4000003Goo00<0
07ooOol00Sg_1CW>1cF]7c6<;SF]7c6<1cF]1CW>0Sg_0goo0@00000=Ool200000goo?Nlmk`05>Lh7
=JdO<H`^=JdO<H`7=Jd5>Lh2?Nl2Ool2000000eoo`03001oogoo00DicWhe[@DicP=oo`4000003Goo
00<007ooOol01CW>OSF]1CW>0goo0@00000=Ool00`00Oomoo`0;=JdW>LhT=JdW>Lh;=Jd3Ool10000
00eoo`03001oogoo00/e[BLicR@e[BLicP/e[@=oo`4000003Goo00<007ooOol01C6<13F]1cW>6Cg_
43W>5SF]43W>6Cg_1cW>13F]1C6<0goo0@00000=Ool00`00Oomoo`05<H`4=Jd7>LhI?Nl@>LhF=Jd@
>LhI?Nl7>Lh4=Jd5<H`3Ool1000000eoo`800003Ool]Jbe[00<]J`8aS08e[@8icPDmk`M240I6<@Y2
40Tmk`/icQ8e[@/icPTmk`Y240I6<@M240Dmk`8icP8e[@8aS0D]J`9oo`8000003Goo00<007ooOol0
1Be[0S6<0SF]0SW>1Cg_1d8@1THa2T8@2Cg_2cW>4SF]2cW>2Cg_2T8@1THa1d8@1Cg_0SW>0SF]0S6<
1Be[0goo0@00000=Ool00`00Oomoo`05;F/2<H`2=Jd2>Lh5?Nl7@Q06AS4:@Q09?Nl;>LhB=Jd;>Lh9
?Nl:@Q06AS47@Q05?Nl2>Lh2=Jd2<H`5;F/3Ool1000000eoo`03001oogoo008YBP<]J`8aS08e[@8i
cP<mk`9240M6<@U:DPM6<@A240Lmk`/icPhe[@/icPLmk`A240M6<@U:DPM6<@9240<mk`8icP8e[@8a
S0<]J`8YBP=oo`4000003Goo00<007ooOol00RU:0be[0S6<0SF]0SW>0cg_0T8@1dHa2DYB1dHa148@
1cg_2cW>3SF]2cW>1cg_148@1dHa2DYB1dHa0T8@0cg_0SW>0SF]0S6<0be[0RU:0goo0@00000=Ool0
0`00Oomoo`029BT3:DX2;F/2=Jd2>Lh3?Nl4AS42BU8<CW<7BU84AS45@Q04?Nl9>Lh>=Jd9>Lh4?Nl5
@Q04AS47BU8<CW<2BU84AS43?Nl2>Lh2=Jd2;F/3:DX29BT3Ool1000000eoo`800003OolU:BDY00<Y
BP8]J`8e[@8icP<mk`A6<@9:DPa>L`M:DPA6<@E240@mk`TicPhe[@TicP@mk`E240A6<@M:DPa>L`9:
DPA6<@<mk`8icP8e[@8]J`<YBP8U:@9oo`8000003Goo00<007ooOol00R480bDY0Re[0SF]0SW>0d8@
0THa0TYB0Tic35:D14ic1DYB14Ha1D8@13g_1cW>3SF]1cW>13g_1D8@14Ha1DYB14ic35:D0Tic0TYB
0THa0d8@0SW>0SF]0Re[0bDY0R480goo0@00000=Ool00`00Oomoo`028@P39BT2;F/2=Jd2>Lh3@Q02
AS42BU82CW<<DY@4CW<5BU84AS45@Q04?Nl7>Lh>=Jd7>Lh4?Nl5@Q04AS45BU84CW<<DY@2CW<2BU82
AS43@Q02>Lh2=Jd2;F/39BT28@P3Ool1000000eoo`03001oogoo008Li`<U:@8]J`8e[@8icP=2409:
DP9>L`9BU0aF]@ABU0E>L`9:DP96<@E240@mk`TicPXe[@TicP@mk`E24096<@9:DPE>L`ABU0aF]@9B
U09>L`9:DP=2408icP8e[@8]J`<U:@8Li`=oo`4000003Goo00<007ooOol00QcW0bDY0Re[0SF]0SW>
0d8@0TYB0Tic0U:D35Je15:D1Dic0TYB0THa1D8@13g_2CW>2SF]2CW>13g_1D8@0THa0TYB1Dic15:D
35Je0U:D0Tic0TYB0d8@0SW>0SF]0Re[0bDY0QcW0goo0@000007Ool30003Ool00`00Oomoo`027>L3
9BT2;F/2=Jd2>Lh3@Q02BU82CW<2DY@<E[D4DY@5CW<2BU82AS45@Q04?Nl9>Lh:=Jd9>Lh4?Nl5@Q02
AS42BU85CW<4DY@<E[D2DY@2CW<2BU83@Q02>Lh2=Jd2;F/39BT27>L3Ool1000000Qoo`03001oogoo
009oo`03001oogoo008HaP<Q208YBP8e[@8mk`=6<@9:DP9BU09F]@aJePAF]@=BU09>L`A:DP=6<@A2
40Dmk`HicPXe[@HicPDmk`A240=6<@A:DP9>L`=BU0AF]@aJeP9F]@9BU09:DP=6<@8mk`8e[@8YBP<Q
208HaP=oo`40000000=oo`0000000`000Woo00<007ooOol00Woo0P0000=ooaS66<H00b480RU:0SF]
0Sg_0dHa0TYB0U:D0UJe35[F15Je0e:D0Tic14YB0dHa148@1Cg_1SW>2SF]1SW>1Cg_148@0dHa14YB
0Tic0e:D15Je35[F0UJe0U:D0TYB0dHa0Sg_0SF]0RU:0b480QS60Woo0P000008Ool00`00Oomoo`02
Ool00`00Oomoo`026<H38@P2:DX2<H`2?Nl3AS42CW<2E[D2F]H<G_L4F]H3E[D2DY@2CW<2BU85AS42
@Q05?Nl6>Lh:=Jd6>Lh5?Nl2@Q05AS42BU82CW<2DY@3E[D4F]H<G_L2F]H2E[D2CW<3AS42?Nl2<H`2
:DX38@P26<H3Ool1000000Qoo`03001oogoo009oo`03001oogoo008HaP<Q208YBP8aS08mk`=6<@9>
L`9F]@9JePaNm`AJeP=F]@9BU09>L`9:DPE6<@9240Dmk`HicPXe[@HicPDmk`9240E6<@9:DP9>L`9B
U0=F]@AJePaNm`9JeP9F]@9>L`=6<@8mk`8aS08YBP<Q208HaP=oo`4000001goo0P0017oo00<007oo
Ool00QBU0acW0RU:0S6<0Sg_0dHa0U:D0UJe0Ukg36<H0Ukg1E[F0UJe0U:D0Tic0dYB0THa0T8@1Cg_
1SW>2SF]1SW>1Cg_0T8@0THa0dYB0Tic0U:D0UJe1E[F0Ukg36<H0Ukg0UJe0U:D0dHa0Sg_0S6<0RU:
0acW0QBU0goo0@00000=Ool00`00Oomoo`025:D37>L2:DX2<H`2?Nl3AS42DY@2E[D2G_L<HaP2G_L5
F]H2E[D2DY@2CW<3BU82AS42@Q05?Nl6>Lh:=Jd6>Lh5?Nl2@Q02AS43BU82CW<2DY@2E[D5F]H2G_L<
HaP2G_L2E[D2DY@3AS42?Nl2<H`2:DX37>L25:D3Ool1000000eoo`03001oogoo008DY@<Li`8YBP8a
S08mk`=6<@9BU09F]@9Nm`aS609Nm`EJeP9F]@9BU09>L`=:DP96<@9240Dmk`HicPXe[@HicPDmk`92
4096<@=:DP9>L`9BU09F]@EJeP9Nm`aS609Nm`9F]@9BU0=6<@8mk`8aS08YBP<Li`8DY@=oo`400000
3Goo00<007ooOol00Q240acW0RDY0S6<0Sg_0dYB0U:D0U[F0V<H36Li0V<H0Ukg0e[F0UJe0U:D0Tic
0dYB0THa148@0cg_1SW>2SF]1SW>0cg_148@0THa0dYB0Tic0U:D0UJe0e[F0Ukg0V<H36Li0V<H0U[F
0U:D0dYB0Sg_0S6<0RDY0acW0Q240goo0@00000=Ool200000goo48@@Q0037>L29BT2<H`2?Nl3BU82
DY@2F]H2HaP<IcT2HaP2G_L3F]H2E[D2DY@2CW<3BU82AS44@Q03?Nl6>Lh:=Jd6>Lh3?Nl4@Q02AS43
BU82CW<2DY@2E[D3F]H2G_L2HaP<IcT2HaP2F]H2DY@3BU82?Nl2<H`29BT37>L248@2Ool2000000eo
o`03001oogoo008<H`<HaP8U:@8aS08mk`=:DP9F]@9Nm`9S60=W>@U[FP9W>@9S60=Nm`9JeP9F]@9B
U0=>L`9:DP96<@9240<mk`HicPXe[@HicP<mk`924096<@9:DP=>L`9BU09F]@9JeP=Nm`9S609W>@U[
FP=W>@9S609Nm`9F]@=:DP8mk`8aS08U:@<HaP8<H`=oo`4000003Goo00<007ooOol00PaS0aS60RDY
0S6<0Sg_0dYB0UJe0Ukg0V<H0fLi2F]J0VLi0V<H0ekg0U[F0UJe0U:D0dic0TYB0THa0T8@0cg_1SW>
2SF]1SW>0cg_0T8@0THa0TYB0dic0U:D0UJe0U[F0ekg0V<H0VLi2F]J0fLi0V<H0Ukg0UJe0dYB0Sg_
0S6<0RDY0aS60PaS0goo0@00000=Ool00`00Oomoo`0236<36<H29BT2<H`2?Nl3BU82E[D2G_L2IcT3
JeX6Kg/5JeX2IcT3HaP2F]H2E[D2DY@3CW<2BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5?Nl2@Q02AS42BU83
CW<2DY@2E[D2F]H3HaP2IcT5JeX6Kg/3JeX2IcT2G_L2E[D3BU82?Nl2<H`29BT36<H236<3Ool10000
00eoo`03001oogoo008<H`<HaP8U:@8aS08mk`=:DP9F]@9Nm`9W>@=[FPI_N`E[FP9W>@=S609JeP9F
]@9BU0=>L`9:DP96<@9240Dmk`@icPXe[@@icPDmk`924096<@9:DP=>L`9BU09F]@9JeP=S609W>@E[
FPI_N`=[FP9W>@9Nm`9F]@=:DP8mk`8aS08U:@<HaP8<H`=oo`4000003Goo00<007ooOol00PQ20aBU
0RDY0S6<0T8@0dic0U[F0V<H0V]J0fmk1W>L0fmk0V]J0VLi0f<H0Ukg0U[F0U:D0dic0TYB0THa0T8@
1Cg_13W>2SF]13W>1Cg_0T8@0THa0TYB0dic0U:D0U[F0Ukg0f<H0VLi0V]J0fmk1W>L0fmk0V]J0V<H
0U[F0dic0T8@0S6<0RDY0aBU0PQ20goo0@00000=Ool00`00Oomoo`0224835:D29BT2<H`2@Q03CW<2
F]H2HaP2JeX3Kg/6Li`3Kg/2JeX2IcT3HaP2G_L2F]H2DY@3CW<2BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5
?Nl2@Q02AS42BU83CW<2DY@2F]H2G_L3HaP2IcT2JeX3Kg/6Li`3Kg/2JeX2HaP2F]H3CW<2@Q02<H`2
9BT35:D22483Ool1000000eoo`800003Ool8@PQ200<DY@8U:@8aS09240=>L`9JeP9S609[FP=_N`Ic
W0=_N`9[FP9W>@=S609Nm`9JeP9BU0=>L`9:DP96<@9240Dmk`@icPXe[@@icPDmk`924096<@9:DP=>
L`9BU09JeP9Nm`=S609W>@9[FP=_N`IcW0=_N`9[FP9S609JeP=>L`92408aS08U:@<DY@88@P9oo`80
00003Goo00<007ooOol00PQ20aBU0R480S6<0T8@0dic0U[F0V<H0V]J1G>L0WNm1G>L0Vmk0V]J0fLi
0Ukg0U[F0UJe0dic0TYB0THa0T8@1Cg_13W>2SF]13W>1Cg_0T8@0THa0TYB0dic0UJe0U[F0Ukg0fLi
0V]J0Vmk1G>L0WNm1G>L0V]J0V<H0U[F0dic0T8@0S6<0R480aBU0PQ20goo0@00000=Ool00`00Oomo
o`0224835:D28@P2<H`2@Q03CW<2F]H2HaP2JeX5Li`2Mkd5Li`2Kg/2JeX3IcT2G_L2F]H2E[D3CW<2
BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5?Nl2@Q02AS42BU83CW<2E[D2F]H2G_L3IcT2JeX2Kg/5Li`2Mkd5
Li`2JeX2HaP2F]H3CW<2@Q02<H`28@P35:D22483Ool1000000eoo`03001oogoo00848@<@Q08Q208a
S09240=>L`9JeP9W>@9_N`=cW0Ig_@EcW09[FP=W>@9S609JeP9F]@=BU09:DP96<@9240Dmk`@icPXe
[@@icPDmk`924096<@9:DP=BU09F]@9JeP9S60=W>@9[FPEcW0Ig_@=cW09_N`9W>@9JeP=>L`92408a
S08Q20<@Q0848@=oo`4000003Goo00<007ooOol00P@Q0a240R480S6<0T8@0dic0U[F0VLi0Vmk0g>L
1WNm1G>L0V]J0fLi0V<H0U[F0UJe0e:D0TYB0THa0T8@1Cg_13W>2SF]13W>1Cg_0T8@0THa0TYB0e:D
0UJe0U[F0V<H0fLi0V]J1G>L1WNm0g>L0Vmk0VLi0U[F0dic0T8@0S6<0R480a240P@Q0goo0@00000=
Ool00`00Oomoo`02124348@28@P2<H`2@Q03CW<2G_L2IcT2Kg/3Mkd6Nmh3Mkd2Li`2Kg/3JeX2HaP2
G_L2E[D3DY@2CW<2AS45@Q02?Nl4>Lh:=Jd4>Lh2?Nl5@Q02AS42CW<3DY@2E[D2G_L2HaP3JeX2Kg/2
Li`3Mkd6Nmh3Mkd2Kg/2IcT2G_L3CW<2@Q02<H`28@P348@21243Ool1000000eoo`03001oogoo0084
8@<@Q08Q208aS09240=>L`9Nm`9W>@9_N`=g_@IkgP=g_@9cW09_N`=[FP9S609Nm`9F]@=BU09>L`96
<@E2408mk`@icPXe[@@icP8mk`E24096<@9>L`=BU09F]@9Nm`9S60=[FP9_N`9cW0=g_@IkgP=g_@9_
N`9W>@9Nm`=>L`92408aS08Q20<@Q0848@=oo`4000003Goo0P0000=oo`0000000a240R480S6<0T8@
0e:D0Ukg0V]J0W>L0gNm1W_N0gNm0W>L0Vmk0f]J0V<H0Ukg0U[F0e:D0Tic0TYB0THa0d8@0Sg_13W>
2SF]13W>0Sg_0d8@0THa0TYB0Tic0e:D0U[F0Ukg0V<H0f]J0Vmk0W>L0gNm1W_N0gNm0W>L0V]J0Ukg
0e:D0T8@0S6<0R480a240P000Woo0P00000=Ool00`00Oomoo`02000348@28@P2<H`2@Q03DY@2G_L2
JeX2Li`3Mkd6Nmh3Mkd2Li`2Kg/3JeX2HaP2G_L2F]H3DY@2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2
?Nl3@Q02AS42BU82CW<3DY@2F]H2G_L2HaP3JeX2Kg/2Li`3Mkd6Nmh3Mkd2Li`2JeX2G_L3DY@2@Q02
<H`28@P348@20003Ool1000000eoo`03001oogoo008000<@Q08Q208aS09240=BU09Nm`9[FP9cW0=g
_@IkgP=g_@9cW09_N`=[FP9S609Nm`9JeP=BU09>L`9:DP96<@=2408mk`@icPXe[@@icP8mk`=24096
<@9:DP9>L`=BU09JeP9Nm`9S60=[FP9_N`9cW0=g_@IkgP=g_@9cW09[FP9Nm`=BU092408aS08Q20<@
Q08000=oo`4000003Goo00<007ooOol00P000a240R480S6<0T8@0e:D0Ukg0V]J0W>L1G_N0Woo1G_N
0WNm0W>L0f]J0VLi0Ukg0U[F0e:D0Tic0TYB0THa0d8@0Sg_13W>2SF]13W>0Sg_0d8@0THa0TYB0Tic
0e:D0U[F0Ukg0VLi0f]J0W>L0WNm1G_N0Woo1G_N0W>L0V]J0Ukg0e:D0T8@0S6<0R480a240P000goo
0@00000=Ool00`00Oomoo`02000348@28@P2<H`2@Q03DY@2G_L2JeX2Li`5Nmh2Ool5Nmh2Mkd2Li`3
JeX2IcT2G_L2F]H3DY@2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3@Q02AS42BU82CW<3DY@2F]H2
G_L2IcT3JeX2Li`2Mkd5Nmh2Ool5Nmh2Li`2JeX2G_L3DY@2@Q02<H`28@P348@20003Ool1000000eo
o`03001oogoo008000<<H`8Li`8aS09240=BU09Nm`9[FP9cW0=kgPIoo`=kgP9g_@9cW0=[FP9W>@9N
m`9JeP=BU09>L`9:DP96<@=2408mk`@icPXe[@@icP8mk`=24096<@9:DP9>L`=BU09JeP9Nm`9W>@=[
FP9cW09g_@=kgPIoo`=kgP9cW09[FP9Nm`=BU092408aS08Li`<<H`8000=oo`4000003Goo0P0000=o
o`0000000`aS0QcW0S6<0T8@0e:D0Ukg0V]J0W>L0g_N1Woo0g_N0WNm0W>L0f]J0VLi0Ukg0U[F0e:D
0Tic0TYB0THa0d8@0Sg_13W>2SF]13W>0Sg_0d8@0THa0TYB0Tic0e:D0U[F0Ukg0VLi0f]J0W>L0WNm
0g_N1Woo0g_N0W>L0V]J0Ukg0e:D0T8@0S6<0QcW0`aS0P000Woo0P00000=Ool00`00Oomoo`020003
36<27>L2<H`2@Q03DY@2G_L2JeX2Mkd3Nmh6Ool3Nmh2Mkd2Li`3Kg/2IcT2HaP2F]H3E[D2CW<2BU82
AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3@Q02AS42BU82CW<3E[D2F]H2HaP2IcT3Kg/2Li`2Mkd3Nmh6Ool3
Nmh2Mkd2JeX2G_L3DY@2@Q02<H`27>L336<20003Ool1000000eoo`03001oogoo008000<<H`8Li`8a
S09240=BU09Nm`9[FP9g_@=kgPIoo`=kgP9g_@9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408m
k`@icPXe[@@icP8mk`=24096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09g_@=kgPIoo`=kgP9g_@9[
FP9Nm`=BU092408aS08Li`<<H`8000=oo`4000003Goo00<007ooOol00P000`aS0QcW0S6<0T8@0e:D
0V<H0V]J0WNm0g_N1Woo0g_N0WNm0W>L0fmk0VLi0V<H0U[F0eJe0Tic0TYB0THa0d8@0Sg_13W>2SF]
13W>0Sg_0d8@0THa0TYB0Tic0eJe0U[F0V<H0VLi0fmk0W>L0WNm0g_N1Woo0g_N0WNm0V]J0V<H0e:D
0T8@0S6<0QcW0`aS0P000goo0@00000=Ool00`00Oomoo`02000336<27>L2<H`2@Q03DY@2HaP2JeX2
Mkd3Nmh6Ool3Nmh2Mkd2Li`3Kg/2IcT2HaP2F]H3E[D2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3
@Q02AS42BU82CW<3E[D2F]H2HaP2IcT3Kg/2Li`2Mkd3Nmh6Ool3Nmh2Mkd2JeX2HaP3DY@2@Q02<H`2
7>L336<20003Ool1000000Qoo`8000=oo`03001oogoo008000<<H`8Li`8aS09240=BU09S609[FP9g
_@=kgPIoo`=kgP9g_@9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8mk`=2
4096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09g_@=kgPIoo`=kgP9g_@9[FP9S60=BU092408aS08L
i`<<H`8000=oo`4000001goo00@007ooOol0009oo`03001oogoo008000<<H`8Li`8aS09240=BU09S
609_N`9g_@=kgPUoo`9kgP9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8m
k`=24096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09kgPUoo`=kgP9g_@9_N`9S60=BU092408aS08L
i`<<H`8000=oo`4000001goo00@007ooOol0009oo`800003Ool0000000<<H`8Li`8aS09240=BU09S
609_N`9g_@=kgPUoo`9kgP9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8m
k`=24096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09kgPUoo`=kgP9g_@9_N`9S60=BU092408aS08L
i`<<H`80009oo`8000001goo00@007ooOol0009oo`03001oogoo008000<<H`8Li`8aS09240=BU09S
609_N`9g_@=kgPUoo`9kgP9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8m
k`=24096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09kgPUoo`=kgP9g_@9_N`9S60=BU092408aS08L
i`<<H`8000=oo`4000001goo00@007ooOol0009oo`03001oogoo008000<<H`8Li`8aS09240=BU09S
609_N`9g_@=kgPUoo`9kgP9cW0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8m
k`=24096<@9:DP9>L`=F]@9JeP9S609W>@=_N`9cW09kgPUoo`=kgP9g_@9_N`9S60=BU092408aS08L
i`<<H`8000=oo`40000027oo0P000goo00<007ooOol00P000`aS0QcW0S6<0T8@0e:D0V<H0V]J0WNm
0g_N1Woo0g_N0WNm0W>L0fmk0VLi0V<H0U[F0eJe0Tic0TYB0THa0d8@0Sg_13W>2SF]13W>0Sg_0d8@
0THa0TYB0Tic0eJe0U[F0V<H0VLi0fmk0W>L0WNm0g_N1Woo0g_N0WNm0V]J0V<H0e:D0T8@0S6<0QcW
0`aS0P000goo0@00000=Ool00`00Oomoo`02000336<27>L2<H`2@Q03DY@2HaP2JeX2Mkd3Nmh6Ool3
Nmh2Mkd2Li`3Kg/2IcT2HaP2F]H3E[D2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3@Q02AS42BU82
CW<3E[D2F]H2HaP2IcT3Kg/2Li`2Mkd3Nmh6Ool3Nmh2Mkd2JeX2HaP3DY@2@Q02<H`27>L336<20003
Ool1000000eoo`03001oogoo008000<<H`8Li`8aS09240=BU09S609[FP9g_@=kgPIoo`=kgP9g_@9c
W0=_N`9W>@9S609JeP=F]@9>L`9:DP96<@=2408mk`@icPXe[@@icP8mk`=24096<@9:DP9>L`=F]@9J
eP9S609W>@=_N`9cW09g_@=kgPIoo`=kgP9g_@9[FP9S60=BU092408aS08Li`<<H`8000=oo`400000
3Goo00<007ooOol00P000`aS0QcW0S6<0T8@0e:D0Ukg0V]J0WNm0g_N1Woo0g_N0WNm0W>L0fmk0VLi
0V<H0U[F0eJe0Tic0TYB0THa0d8@0Sg_13W>2SF]13W>0Sg_0d8@0THa0TYB0Tic0eJe0U[F0V<H0VLi
0fmk0W>L0WNm0g_N1Woo0g_N0WNm0V]J0Ukg0e:D0T8@0S6<0QcW0`aS0P000goo0@00000=Ool20000
0goo0000000336<27>L2<H`2@Q03DY@2G_L2JeX2Mkd3Nmh6Ool3Nmh2Mkd2Li`3Kg/2IcT2HaP2F]H3
E[D2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3@Q02AS42BU82CW<3E[D2F]H2HaP2IcT3Kg/2Li`2
Mkd3Nmh6Ool3Nmh2Mkd2JeX2G_L3DY@2@Q02<H`27>L336<20002Ool2000000eoo`03001oogoo0080
00<<H`8Li`8aS09240=BU09Nm`9[FP9cW0=kgPIoo`=kgP9g_@9cW0=[FP9W>@9Nm`9JeP=BU09>L`9:
DP96<@=2408mk`@icPXe[@@icP8mk`=24096<@9:DP9>L`=BU09JeP9Nm`9W>@=[FP9cW09g_@=kgPIo
o`=kgP9cW09[FP9Nm`=BU092408aS08Li`<<H`8000=oo`4000003Goo00<007ooOol00P000`aS0QcW
0S6<0T8@0e:D0Ukg0V]J0W>L0g_N1Woo0g_N0WNm0W>L0f]J0VLi0Ukg0U[F0e:D0Tic0TYB0THa0d8@
0Sg_13W>2SF]13W>0Sg_0d8@0THa0TYB0Tic0e:D0U[F0Ukg0VLi0f]J0W>L0WNm0g_N1Woo0g_N0W>L
0V]J0Ukg0e:D0T8@0S6<0QcW0`aS0P000goo0@00000=Ool00`00Oomoo`02000348@28@P2<H`2@Q03
DY@2G_L2JeX2Li`5Nmh2Ool5Nmh2Mkd2Li`3JeX2IcT2G_L2F]H3DY@2CW<2BU82AS43@Q02?Nl4>Lh:
=Jd4>Lh2?Nl3@Q02AS42BU82CW<3DY@2F]H2G_L2IcT3JeX2Li`2Mkd5Nmh2Ool5Nmh2Li`2JeX2G_L3
DY@2@Q02<H`28@P348@20003Ool1000000eoo`03001oogoo008000<@Q08Q208aS09240=BU09Nm`9[
FP9cW0EkgP9oo`EkgP9g_@9cW0=[FP9W>@9Nm`9JeP=BU09>L`9:DP96<@=2408mk`@icPXe[@@icP8m
k`=24096<@9:DP9>L`=BU09JeP9Nm`9W>@=[FP9cW09g_@EkgP9oo`EkgP9cW09[FP9Nm`=BU092408a
S08Q20<@Q08000=oo`4000003Goo00<007ooOol00P000a240R480S6<0T8@0e:D0Ukg0V]J0W>L0gNm
1W_N0gNm0W>L0Vmk0f]J0V<H0Ukg0U[F0e:D0Tic0TYB0THa0d8@0Sg_13W>2SF]13W>0Sg_0d8@0THa
0TYB0Tic0e:D0U[F0Ukg0V<H0f]J0Vmk0W>L0gNm1W_N0gNm0W>L0V]J0Ukg0e:D0T8@0S6<0R480a24
0P000goo0@00000=Ool00`00Oomoo`02000348@28@P2<H`2@Q03DY@2G_L2JeX2Li`3Mkd6Nmh3Mkd2
Li`2Kg/3JeX2HaP2G_L2F]H3DY@2CW<2BU82AS43@Q02?Nl4>Lh:=Jd4>Lh2?Nl3@Q02AS42BU82CW<3
DY@2F]H2G_L2HaP3JeX2Kg/2Li`3Mkd6Nmh3Mkd2Li`2JeX2G_L3DY@2@Q02<H`28@P348@20003Ool1
000000eoo`800003Ool0000000<@Q08Q208aS09240=BU09Nm`9[FP9cW0=g_@IkgP=g_@9cW09_N`=[
FP9S609Nm`9JeP=BU09>L`9:DP96<@=2408mk`@icPXe[@@icP8mk`=24096<@9:DP9>L`=BU09JeP9N
m`9S60=[FP9_N`9cW0=g_@IkgP=g_@9cW09[FP9Nm`=BU092408aS08Q20<@Q080009oo`8000003Goo
00<007ooOol00P@Q0a240R480S6<0T8@0dic0Ukg0VLi0Vmk0gNm1W_N0gNm0W>L0Vmk0f]J0V<H0Ukg
0UJe0e:D0Tic0THa1D8@0Sg_13W>2SF]13W>0Sg_1D8@0THa0Tic0e:D0UJe0Ukg0V<H0f]J0Vmk0W>L
0gNm1W_N0gNm0Vmk0VLi0Ukg0dic0T8@0S6<0R480a240P@Q0goo0@00000=Ool00`00Oomoo`021243
48@28@P2<H`2@Q03CW<2G_L2IcT2Kg/3Mkd6Nmh3Mkd2Li`2Kg/3JeX2HaP2G_L2E[D3DY@2CW<2AS45
@Q02?Nl4>Lh:=Jd4>Lh2?Nl5@Q02AS42CW<3DY@2E[D2G_L2HaP3JeX2Kg/2Li`3Mkd6Nmh3Mkd2Kg/2
IcT2G_L3CW<2@Q02<H`28@P348@21243Ool1000000eoo`03001oogoo00848@<@Q08Q208aS09240=>
L`9JeP9W>@9_N`=cW0Ig_@EcW09[FP=W>@9S609JeP9F]@=BU09:DP96<@9240Dmk`@icPXe[@@icPDm
k`924096<@9:DP=BU09F]@9JeP9S60=W>@9[FPEcW0Ig_@=cW09_N`9W>@9JeP=>L`92408aS08Q20<@
Q0848@=oo`4000003Goo00<007ooOol00P@Q0a240R480S6<0T8@0dic0U[F0VLi0Vmk0g>L1WNm1G>L
0V]J0fLi0V<H0U[F0UJe0e:D0TYB0THa0T8@1Cg_13W>2SF]13W>1Cg_0T8@0THa0TYB0e:D0UJe0U[F
0V<H0fLi0V]J1G>L1WNm0g>L0Vmk0VLi0U[F0dic0T8@0S6<0R480a240P@Q0goo0@00000=Ool00`00
Oomoo`0224835:D28@P2<H`2@Q03CW<2F]H2HaP2JeX5Li`2Mkd5Li`2Kg/2JeX3IcT2G_L2F]H2E[D3
CW<2BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5?Nl2@Q02AS42BU83CW<2E[D2F]H2G_L3IcT2JeX2Kg/5Li`2
Mkd5Li`2JeX2HaP2F]H3CW<2@Q02<H`28@P35:D22483Ool1000000eoo`03001oogoo0088@P<DY@8Q
208aS09240=>L`9JeP9S609[FPEcW09g_@EcW09_N`9[FP=W>@9Nm`9JeP9F]@=>L`9:DP96<@9240Dm
k`@icPXe[@@icPDmk`924096<@9:DP=>L`9F]@9JeP9Nm`=W>@9[FP9_N`EcW09g_@EcW09[FP9S609J
eP=>L`92408aS08Q20<DY@88@P=oo`4000003Goo0P0000=oo`Q224800aBU0RDY0S6<0T8@0dic0U[F
0V<H0V]J0fmk1W>L0fmk0V]J0VLi0f<H0Ukg0U[F0U:D0dic0TYB0THa0T8@1Cg_13W>2SF]13W>1Cg_
0T8@0THa0TYB0dic0U:D0U[F0Ukg0f<H0VLi0V]J0fmk1W>L0fmk0V]J0V<H0U[F0dic0T8@0S6<0RDY
0aBU0PQ20Woo0P00000=Ool00`00Oomoo`0224835:D29BT2<H`2@Q03CW<2F]H2HaP2JeX3Kg/6Li`3
Kg/2JeX2IcT3HaP2G_L2F]H2DY@3CW<2BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5?Nl2@Q02AS42BU83CW<2
DY@2F]H2G_L3HaP2IcT2JeX3Kg/6Li`3Kg/2JeX2HaP2F]H3CW<2@Q02<H`29BT35:D22483Ool10000
00eoo`03001oogoo0088@P<DY@8U:@8aS09240=>L`9JeP9S609[FP=_N`IcW0=_N`9[FP9W>@=S609N
m`9JeP9BU0=>L`9:DP96<@9240Dmk`@icPXe[@@icPDmk`924096<@9:DP=>L`9BU09JeP9Nm`=S609W
>@9[FP=_N`IcW0=_N`9[FP9S609JeP=>L`92408aS08U:@<DY@88@P=oo`4000003Goo00<007ooOol0
0PaS0aS60RDY0S6<0Sg_0dYB0UJe0Ukg0VLi0f]J1Vmk1F]J0VLi0f<H0U[F0UJe0U:D0dic0TYB0THa
0T8@1Cg_13W>2SF]13W>1Cg_0T8@0THa0TYB0dic0U:D0UJe0U[F0f<H0VLi1F]J1Vmk0f]J0VLi0Ukg
0UJe0dYB0Sg_0S6<0RDY0aS60PaS0goo0@00000=Ool00`00Oomoo`0236<36<H29BT2<H`2?Nl3BU82
E[D2G_L2IcT3JeX6Kg/5JeX2IcT3HaP2F]H2E[D2DY@3CW<2BU82AS42@Q05?Nl4>Lh:=Jd4>Lh5?Nl2
@Q02AS42BU83CW<2DY@2E[D2F]H3HaP2IcT5JeX6Kg/3JeX2IcT2G_L2E[D3BU82?Nl2<H`29BT36<H2
36<3Ool1000000eoo`03001oogoo008<H`<HaP8U:@8aS08mk`=:DP9F]@9Nm`9S60=W>@U[FP9W>@9S
60=Nm`9JeP9F]@9BU0=>L`9:DP96<@9240<mk`HicPXe[@HicP<mk`924096<@9:DP=>L`9BU09F]@9J
eP=Nm`9S609W>@U[FP=W>@9S609Nm`9F]@=:DP8mk`8aS08U:@<HaP8<H`=oo`4000003Goo0P0000=o
o`aS36<00aS60RDY0S6<0Sg_0dYB0UJe0Ukg0V<H0fLi2F]J0VLi0V<H0ekg0U[F0UJe0U:D0dic0TYB
0THa0T8@0cg_1SW>2SF]1SW>0cg_0T8@0THa0TYB0dic0U:D0UJe0U[F0ekg0V<H0VLi2F]J0fLi0V<H
0Ukg0UJe0dYB0Sg_0S6<0RDY0aS60PaS0Woo0P00000=Ool00`00Oomoo`0248@37>L29BT2<H`2?Nl3
BU82DY@2F]H2HaP<IcT2HaP2G_L3F]H2E[D2DY@2CW<3BU82AS44@Q03?Nl6>Lh:=Jd6>Lh3?Nl4@Q02
AS43BU82CW<2DY@2E[D3F]H2G_L2HaP<IcT2HaP2F]H2DY@3BU82?Nl2<H`29BT37>L248@3Ool10000
00eoo`03001oogoo008@Q0<Li`8U:@8aS08mk`=:DP9BU09JeP9S60aW>@9S609Nm`=JeP9F]@9BU09>
L`=:DP96<@A240<mk`HicPXe[@HicP<mk`A24096<@=:DP9>L`9BU09F]@=JeP9Nm`9S60aW>@9S609J
eP9BU0=:DP8mk`8aS08U:@<Li`8@Q0=oo`4000003Goo00<007ooOol00QBU0acW0RU:0S6<0Sg_0dHa
0U:D0UJe0Ukg36<H0Ukg1E[F0UJe0U:D0Tic0dYB0THa0T8@1Cg_1SW>2SF]1SW>1Cg_0T8@0THa0dYB
0Tic0U:D0UJe1E[F0Ukg36<H0Ukg0UJe0U:D0dHa0Sg_0S6<0RU:0acW0QBU0goo0@00000=Ool00`00
Oomoo`025:D37>L2:DX2<H`2?Nl3AS42DY@2E[D2G_L<HaP2G_L5F]H2E[D2DY@2CW<3BU82AS42@Q05
?Nl6>Lh:=Jd6>Lh5?Nl2@Q02AS43BU82CW<2DY@2E[D5F]H2G_L<HaP2G_L2E[D2DY@3AS42?Nl2<H`2
:DX37>L25:D3Ool1000000Moo`<000=oo`03001oogoo008DY@<Li`8YBP8aS08mk`=6<@9BU09F]@9N
m`aS609Nm`EJeP9F]@9BU09>L`=:DP96<@9240Dmk`HicPXe[@HicPDmk`924096<@=:DP9>L`9BU09F
]@EJeP9Nm`aS609Nm`9F]@9BU0=6<@8mk`8aS08YBP<Li`8DY@=oo`40000027oo00<007ooOol00Woo
00<007ooOol00QS60b480RU:0S6<0Sg_0dHa0Tic0UJe0U[F35kg15[F0eJe0U:D0Tic0TYB1DHa0T8@
1Cg_1SW>2SF]1SW>1Cg_0T8@1DHa0TYB0Tic0U:D0eJe15[F35kg0U[F0UJe0Tic0dHa0Sg_0S6<0RU:
0b480QS60goo0@000008Ool00`00Oomoo`02Ool200000goo6<HHaP038@P2:DX2<H`2?Nl3AS42CW<2
E[D2F]H<G_L4F]H3E[D2DY@2CW<2BU85AS42@Q05?Nl6>Lh:=Jd6>Lh5?Nl2@Q05AS42BU82CW<2DY@3
E[D4F]H<G_L2F]H2E[D2CW<3AS42?Nl2<H`2:DX38@P26<H2Ool2000000Qoo`03001oogoo009oo`03
001oogoo008HaP<Q208YBP8e[@8mk`=6<@9:DP9BU09F]@aJePAF]@=BU09>L`A:DP=6<@A240Dmk`Hi
cPXe[@HicPDmk`A240=6<@A:DP9>L`=BU0AF]@aJeP9F]@9BU09:DP=6<@8mk`8e[@8YBP<Q208HaP=o
o`40000027oo00<007ooOol00Woo00<007ooOol00QS60b480RU:0SF]0Sg_0dHa0TYB0U:D0UJe35[F
15Je0e:D0Tic14YB0dHa148@1Cg_1SW>2SF]1SW>1Cg_148@0dHa14YB0Tic0e:D15Je35[F0UJe0U:D
0TYB0dHa0Sg_0SF]0RU:0b480QS60goo0@000007Ool20004Ool00`00Oomoo`027>L39BT2;F/2=Jd2
>Lh3@Q02BU82CW<2DY@<E[D4DY@5CW<2BU82AS45@Q04?Nl9>Lh:=Jd9>Lh4?Nl5@Q02AS42BU85CW<4
DY@<E[D2DY@2CW<2BU83@Q02>Lh2=Jd2;F/39BT27>L3Ool1000000eoo`03001oogoo008Li`<U:@8]
J`8e[@8icP=2409:DP9>L`9BU0aF]@ABU0E>L`9:DP96<@E240@mk`TicPXe[@TicP@mk`E24096<@9:
DPE>L`ABU0aF]@9BU09>L`9:DP=2408icP8e[@8]J`<U:@8Li`=oo`4000003Goo00<007ooOol00QcW
0bDY0Re[0SF]0SW>0d8@0TYB0Tic0U:D35Je15:D1Dic0TYB0THa1D8@13g_2CW>2SF]2CW>13g_1D8@
0THa0TYB1Dic15:D35Je0U:D0Tic0TYB0d8@0SW>0SF]0Re[0bDY0QcW0goo0@00000=Ool00`00Oomo
o`028@P39BT2;F/2=Jd2>Lh3@Q02AS42BU82CW<<DY@4CW<5BU84AS45@Q04?Nl7>Lh>=Jd7>Lh4?Nl5
@Q04AS45BU84CW<<DY@2CW<2BU82AS43@Q02>Lh2=Jd2;F/39BT28@P3Ool1000000eoo`800003OolQ
224800<U:@8]J`8e[@8icP=24096<@9:DP9>L`aBU0A>L`E:DPA6<@E240@mk`LicPhe[@LicP@mk`E2
40A6<@E:DPA>L`aBU09>L`9:DP96<@=2408icP8e[@8]J`<U:@8Q209oo`8000003Goo00<007ooOol0
0RDY0bU:0Re[0SF]0SW>0cg_14Ha0TYB34ic1dYB14Ha1D8@13g_2CW>3SF]2CW>13g_1D8@14Ha1dYB
34ic0TYB14Ha0cg_0SW>0SF]0Re[0bU:0RDY0goo0@00000=Ool00`00Oomoo`029BT3:DX2;F/2=Jd2
>Lh3?Nl4AS42BU8<CW<7BU84AS45@Q04?Nl9>Lh>=Jd9>Lh4?Nl5@Q04AS47BU8<CW<2BU84AS43?Nl2
>Lh2=Jd2;F/3:DX29BT3Ool1000000eoo`03001oogoo008YBP<]J`8aS08e[@8icP<mk`9240M6<@U:
DPM6<@A240Lmk`/icPhe[@/icPLmk`A240M6<@U:DPM6<@9240<mk`8icP8e[@8aS0<]J`8YBP=oo`40
00003Goo00<007ooOol00RU:0be[0S6<0SF]0SW>0cg_0T8@1dHa2DYB1dHa148@1cg_2cW>3SF]2cW>
1cg_148@1dHa2DYB1dHa0T8@0cg_0SW>0SF]0S6<0be[0RU:0goo0@00000=Ool00`00Oomoo`05;F/2
<H`2=Jd2>Lh5?Nl7@Q06AS4:@Q09?Nl;>LhB=Jd;>Lh9?Nl:@Q06AS47@Q05?Nl2>Lh2=Jd2<H`5;F/3
Ool1000000eoo`03001oogoo00D]J`8aS08e[@8icPDmk`M240I6<@Y240Tmk`/icQ8e[@/icPTmk`Y2
40I6<@M240Dmk`8icP8e[@8aS0D]J`=oo`4000003Goo0P0000=oobe[;F/00be[0S6<0SF]0SW>1Cg_
1d8@1THa2T8@2Cg_2cW>4SF]2cW>2Cg_2T8@1THa1d8@1Cg_0SW>0SF]0S6<1Be[0Woo0P00000=Ool0
0`00Oomoo`05<H`4=Jd7>LhI?Nl@>LhF=Jd@>LhI?Nl7>Lh4=Jd5<H`3Ool1000000eoo`03001oogoo
00DaS0@e[@LicQTmka0icQHe[A0icQTmk`LicP@e[@DaS0=oo`4000003Goo00<007ooOol02cF]9cW>
93F]9cW>2cF]0goo0@00000=Ool00`00Oomoo`0;=JdW>LhT=JdW>Lh;=Jd3Ool1000000eoo`03001o
ogoo00DicWhe[@DicP=oo`4000003Goo00<007ooOol01CW>OSF]1CW>0goo0@00000=Ool200000goo
?Nlmk`05>Lh7=JdO<H`^=JdO<H`7=Jd5>Lh2?Nl2Ool2000000eoo`03001oogoo008mk`DicPLe[Ala
S2he[AlaS0Le[@DicP8mk`=oo`4000003Goo00<007ooOol00Sg_1CW>1cF]7c6<;SF]7c6<1cF]1CW>
0Sg_0goo0@00000=Ool00`00Oomoo`05?Nl4>Lh2=Jd5<H`7;F/6:DX<;F/9<H`T=Jd9<H`<;F/6:DX7
;F/5<H`2=Jd4>Lh5?Nl3Ool1000000eoo`03001oogoo00Dmk`@icP8e[@DaS0L]J`HYBP`]J`TaS2@e
[@TaS0`]J`HYBPL]J`DaS08e[@@icPDmk`=oo`4000003Goo00<007ooOol01D8@0Sg_0SW>0SF]0c6<
0Re[12U:32DY1bU:1Re[2C6<73F]2C6<1Re[1bU:32DY12U:0Re[0c6<0SF]0SW>0Sg_1D8@0goo0@00
000=Ool200000goo@Q124003@Q02?Nl2>Lh2=Jd3<H`2;F/4:DX<9BT7:DX6;F/9<H`L=Jd9<H`6;F/7
:DX<9BT4:DX2;F/3<H`2=Jd2>Lh2?Nl5@Q02Ool2000000eoo`03001oogoo0096<@=2408mk`8icP8a
S0<]J`8YBP@U:@`Q20LU:@@YBPD]J`HaS1`e[@HaS0D]J`@YBPLU:@`Q20@U:@8YBP<]J`8aS08icP8m
k`=24096<@=oo`4000003Goo00<007ooOol00THa0d8@0Sg_0SW>0S6<0be[0RU:12DY32481bDY12U:
1Be[1S6<73F]1S6<1Be[12U:1bDY324812DY0RU:0be[0S6<0SW>0Sg_0d8@0THa0goo0@00000=Ool0
0`00Oomoo`02BU83AS42?Nl2>Lh2<H`3;F/2:DX29BT28@P<7>L78@P29BT4:DX5;F/7<H`F=Jd7<H`5
;F/4:DX29BT78@P<7>L28@P29BT2:DX3;F/2<H`2>Lh2?Nl3AS42BU83Ool1000000eoo`03001oogoo
009:DP=6<@8mk`8icP8aS0<]J`8YBP8U:@8Q20`Li`LQ208U:@@YBPD]J`LaS1He[@LaS0D]J`@YBP8U
:@LQ20`Li`8Q208U:@8YBP<]J`8aS08icP8mk`=6<@9:DP=oo`4000001goo10000Woo00<007ooOol0
0TYB0dHa0Sg_0SW>0S6<0be[0RU:0RDY0R4831cW1b480RDY12U:1Be[1c6<5SF]1c6<1Be[12U:0RDY
1b4831cW0R480RDY0RU:0be[0S6<0SW>0Sg_0dHa0TYB0goo0@000008Ool00`00Oomoo`02Ool00`00
Oomoo`02CW<3BU82@Q02>Lh2<H`3:DX29BT28@P27>L<6<H47>L58@P29BT5:DX4;F/5<H`F=Jd5<H`4
;F/5:DX29BT58@P47>L<6<H27>L28@P29BT3:DX2<H`2>Lh2@Q03BU82CW<3Ool1000000Uoo`04001o
ogooOol200000gooCW=>L`03BU82@Q02>Lh2<H`3:DX29BT28@P27>L<6<H47>L58@P29BT5:DX4;F/5
<H`F=Jd5<H`4;F/5:DX29BT58@P47>L<6<H27>L28@P29BT3:DX2<H`2>Lh2@Q03BU82CW<2Ool20000
00Yoo`04001oogoo002=Ool1000000Moo`04001oogoo0002Ool01000Oomoo`001Woo00<007ooOol0
17oo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol0
17oo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol0
17oo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol017oo00<007ooOol0
17oo00<007ooOol00goo00<007ooOol017oo00<007ooOol017oo00<007ooOol00@000008Ool20003
Oon?000009aoo`00W7oo0000\
\>"],
  ImageRangeCache->{{{0, 155.438}, {155.438, 0}} -> {-2.48509, -2.41889, 
  0.0294226, 0.0294226}}]
}, Open  ]],

Cell["\<\
In the same way as for two-dimensional curves, the functions for \
three-dimensional surfaces also have options associated with them which allow \
the alteration of graphics to suit your needs.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[func[x, \ y], \ {x, \ \(-2\), \ 2}, \ {y, \ \(-2\), \ 2}, \n
      ViewPoint\  -> \ {0, \ 1, \ 0.3}, \nAxes\  -> \ None, \ 
      Boxed\  -> \ False, \ PlotPoints\  -> \ 20]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .38948 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 1 -0.226651 1 [
[ 0 0 0 0 ]
[ 1 .38948 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .38948 L
0 .38948 L
closepath
clip
newpath
.5 Mabswid
[ ] 0 setdash
.182 .341 .79 r
.66812 .33642 .67279 .3219 .6536 .31059 .64861 .31526 Metetra
0 .229 .749 r
.64861 .31526 .6536 .31059 .63469 .29993 .62963 .29552 Metetra
.418 0 0 r
.62963 .29552 .63469 .29993 .61636 .29436 .61168 .28528 Metetra
.403 0 0 r
.61168 .28528 .61636 .29436 .59844 .29406 .59447 .28474 Metetra
.263 0 0 r
.59447 .28474 .59844 .29406 .58068 .2971 .57754 .29032 Metetra
.116 0 0 r
.57754 .29032 .58068 .2971 .56291 .30133 .56059 .29809 Metetra
.56059 .29809 .56291 .30133 .54504 .30522 .54346 .30528 Metetra
.54346 .30528 .54504 .30522 .52707 .30801 .52615 .31046 Metetra
.245 .759 .822 r
.52615 .31046 .52707 .30801 .50903 .30944 .50873 .31311 Metetra
.218 .727 .891 r
.50873 .31311 .50903 .30944 .49097 .30944 .49127 .31311 Metetra
.846 .992 .738 r
.35139 .31526 .3464 .31059 .32721 .3219 .33188 .33642 Metetra
.547 .844 .521 r
.37037 .29552 .36531 .29993 .3464 .31059 .35139 .31526 Metetra
.38832 .28528 .38364 .29436 .36531 .29993 .37037 .29552 Metetra
.378 0 0 r
.40553 .28474 .40156 .29406 .38364 .29436 .38832 .28528 Metetra
.518 0 0 r
.42246 .29032 .41932 .2971 .40156 .29406 .40553 .28474 Metetra
.475 0 0 r
.43941 .29809 .43709 .30133 .41932 .2971 .42246 .29032 Metetra
.303 0 0 r
.45654 .30528 .45496 .30522 .43709 .30133 .43941 .29809 Metetra
0 .476 .789 r
.47385 .31046 .47293 .30801 .45496 .30522 .45654 .30528 Metetra
.105 .63 .878 r
.49127 .31311 .49097 .30944 .47293 .30801 .47385 .31046 Metetra
.355 .528 .889 r
.67279 .3219 .67767 .306 .65893 .30556 .6536 .31059 Metetra
.034 .445 .873 r
.6536 .31059 .65893 .30556 .6402 .30514 .63469 .29993 Metetra
.361 0 0 r
.63469 .29993 .6402 .30514 .62149 .30492 .61636 .29436 Metetra
.413 0 0 r
.61636 .29436 .62149 .30492 .6028 .30491 .59844 .29406 Metetra
.345 0 0 r
.59844 .29406 .6028 .30491 .58411 .30503 .58068 .2971 Metetra
.232 0 0 r
.58068 .2971 .58411 .30503 .56543 .3052 .56291 .30133 Metetra
.077 0 0 r
.56291 .30133 .56543 .3052 .54674 .30535 .54504 .30522 Metetra
.54504 .30522 .54674 .30535 .52805 .30546 .52707 .30801 Metetra
.194 .723 .829 r
.52707 .30801 .52805 .30546 .50935 .30552 .50903 .30944 Metetra
.217 .727 .89 r
.50903 .30944 .50935 .30552 .49065 .30552 .49097 .30944 Metetra
.703 .984 .867 r
.3464 .31059 .34107 .30556 .32233 .306 .32721 .3219 Metetra
.36531 .29993 .3598 .30514 .34107 .30556 .3464 .31059 Metetra
.203 0 0 r
.38364 .29436 .37851 .30492 .3598 .30514 .36531 .29993 Metetra
.403 0 0 r
.40156 .29406 .3972 .30491 .37851 .30492 .38364 .29436 Metetra
.445 0 0 r
.41932 .2971 .41589 .30503 .3972 .30491 .40156 .29406 Metetra
.36 0 0 r
.43709 .30133 .43457 .3052 .41589 .30503 .41932 .2971 Metetra
.176 0 0 r
.45496 .30522 .45326 .30535 .43457 .3052 .43709 .30133 Metetra
.022 .578 .818 r
.47293 .30801 .47195 .30546 .45326 .30535 .45496 .30522 Metetra
.159 .678 .886 r
.49097 .30944 .49065 .30552 .47195 .30546 .47293 .30801 Metetra
.593 .784 .971 r
.67767 .306 .68284 .28914 .66465 .30016 .65893 .30556 Metetra
.265 .746 .93 r
.65893 .30556 .66465 .30016 .64618 .31082 .6402 .30514 Metetra
.253 0 0 r
.6402 .30514 .64618 .31082 .6271 .3165 .62149 .30492 Metetra
.408 0 0 r
.62149 .30492 .6271 .3165 .60757 .3168 .6028 .30491 Metetra
.411 0 0 r
.6028 .30491 .60757 .3168 .58785 .31369 .58411 .30503 Metetra
.336 0 0 r
.58411 .30503 .58785 .31369 .56816 .30941 .56543 .3052 Metetra
.189 0 0 r
.56543 .3052 .56816 .30941 .54858 .3055 .54674 .30535 Metetra
.004 0 0 r
.54674 .30535 .54858 .3055 .5291 .30272 .52805 .30546 Metetra
.144 .683 .835 r
.52805 .30546 .5291 .30272 .50969 .30131 .50935 .30552 Metetra
.217 .727 .89 r
.50935 .30552 .50969 .30131 .49031 .30131 .49065 .30552 Metetra
.298 .773 .93 r
.34107 .30556 .33535 .30016 .31716 .28914 .32233 .306 Metetra
.23 0 0 r
.3598 .30514 .35382 .31082 .33535 .30016 .34107 .30556 Metetra
.4 0 0 r
.37851 .30492 .3729 .3165 .35382 .31082 .3598 .30514 Metetra
.413 0 0 r
.3972 .30491 .39243 .3168 .3729 .3165 .37851 .30492 Metetra
.344 0 0 r
.41589 .30503 .41215 .31369 .39243 .3168 .3972 .30491 Metetra
.2 0 0 r
.43457 .3052 .43184 .30941 .41215 .31369 .41589 .30503 Metetra
.014 0 0 r
.45326 .30535 .45142 .3055 .43184 .30941 .43457 .3052 Metetra
.138 .678 .835 r
.47195 .30546 .4709 .30272 .45142 .3055 .45326 .30535 Metetra
.215 .725 .89 r
.49065 .30552 .49031 .30131 .4709 .30272 .47195 .30546 Metetra
.797 .985 .895 r
.68284 .28914 .68837 .27181 .67079 .29441 .66465 .30016 Metetra
.451 .86 .694 r
.66465 .30016 .67079 .29441 .65265 .31655 .64618 .31082 Metetra
.117 0 0 r
.64618 .31082 .65265 .31655 .6332 .32847 .6271 .3165 Metetra
.388 0 0 r
.6271 .3165 .6332 .32847 .61275 .32911 .60757 .3168 Metetra
.462 0 0 r
.60757 .3168 .61275 .32911 .59191 .32257 .58785 .31369 Metetra
.42 0 0 r
.58785 .31369 .59191 .32257 .57112 .3136 .56816 .30941 Metetra
.282 0 0 r
.56816 .30941 .57112 .3136 .55056 .30547 .54858 .3055 Metetra
0 .502 .715 r
.54858 .3055 .55056 .30547 .53023 .29969 .5291 .30272 Metetra
.097 .642 .839 r
.5291 .30272 .53023 .29969 .51006 .29677 .50969 .30131 Metetra
.218 .727 .89 r
.50969 .30131 .51006 .29677 .48994 .29677 .49031 .30131 Metetra
0 .251 .679 r
.33535 .30016 .32921 .29441 .31163 .27181 .31716 .28914 Metetra
.599 .027 0 r
.35382 .31082 .34735 .31655 .32921 .29441 .33535 .30016 Metetra
.556 .003 0 r
.3729 .3165 .3668 .32847 .34735 .31655 .35382 .31082 Metetra
.41 0 0 r
.39243 .3168 .38725 .32911 .3668 .32847 .3729 .3165 Metetra
.217 0 0 r
.41215 .31369 .40809 .32257 .38725 .32911 .39243 .3168 Metetra
.011 0 0 r
.43184 .30941 .42888 .3136 .40809 .32257 .41215 .31369 Metetra
.159 .696 .713 r
.45142 .3055 .44944 .30547 .42888 .3136 .43184 .30941 Metetra
.254 .765 .835 r
.4709 .30272 .46977 .29969 .44944 .30547 .45142 .3055 Metetra
.271 .768 .889 r
.49031 .30131 .48994 .29677 .46977 .29969 .4709 .30272 Metetra
.804 .956 .625 r
.68837 .27181 .69437 .2545 .67743 .2883 .67079 .29441 Metetra
.67079 .29441 .67743 .2883 .65963 .32186 .65265 .31655 Metetra
.65265 .31655 .65963 .32186 .6398 .34012 .6332 .32847 Metetra
.353 0 0 r
.6332 .32847 .6398 .34012 .61836 .34111 .61275 .32911 Metetra
.493 0 0 r
.61275 .32911 .61836 .34111 .59629 .33107 .59191 .32257 Metetra
.474 0 0 r
.59191 .32257 .59629 .33107 .5743 .31738 .57112 .3136 Metetra
.347 0 0 r
.57112 .3136 .5743 .31738 .5527 .30501 .55056 .30547 Metetra
0 .431 .723 r
.55056 .30547 .5527 .30501 .53146 .29627 .53023 .29969 Metetra
.055 .601 .84 r
.53023 .29969 .53146 .29627 .51046 .29186 .51006 .29677 Metetra
.218 .727 .891 r
.51006 .29677 .51046 .29186 .48954 .29186 .48994 .29677 Metetra
.613 .14 0 r
.32921 .29441 .32257 .2883 .30563 .2545 .31163 .27181 Metetra
.767 .265 0 r
.34735 .31655 .34037 .32186 .32257 .2883 .32921 .29441 Metetra
.666 .12 0 r
.3668 .32847 .3602 .34012 .34037 .32186 .34735 .31655 Metetra
.392 0 0 r
.38725 .32911 .38164 .34111 .3602 .34012 .3668 .32847 Metetra
.067 0 0 r
.40809 .32257 .40371 .33107 .38164 .34111 .38725 .32911 Metetra
.42888 .3136 .4257 .31738 .40371 .33107 .40809 .32257 Metetra
.317 .793 .707 r
.44944 .30547 .4473 .30501 .4257 .31738 .42888 .3136 Metetra
.359 .832 .821 r
.46977 .29969 .46854 .29627 .4473 .30501 .44944 .30547 Metetra
.322 .805 .883 r
.48994 .29677 .48954 .29186 .46854 .29627 .46977 .29969 Metetra
.704 .822 .371 r
.69437 .2545 .70097 .23767 .68461 .28183 .67743 .2883 Metetra
.67743 .2883 .68461 .28183 .66715 .32624 .65963 .32186 Metetra
.65963 .32186 .66715 .32624 .64688 .35064 .6398 .34012 Metetra
.301 0 0 r
.6398 .34012 .64688 .35064 .62438 .35196 .61836 .34111 Metetra
.499 0 0 r
.61836 .34111 .62438 .35196 .60099 .33852 .59629 .33107 Metetra
.492 0 0 r
.59629 .33107 .60099 .33852 .57773 .32027 .5743 .31738 Metetra
.379 0 0 r
.5743 .31738 .57773 .32027 .555 .30387 .5527 .30501 Metetra
0 .369 .725 r
.5527 .30501 .555 .30387 .53278 .29233 .53146 .29627 Metetra
.02 .564 .84 r
.53146 .29627 .53278 .29233 .51089 .28651 .51046 .29186 Metetra
.219 .728 .892 r
.51046 .29186 .51089 .28651 .48911 .28651 .48954 .29186 Metetra
.743 .333 0 r
.32257 .2883 .31539 .28183 .29903 .23767 .30563 .2545 Metetra
.831 .39 0 r
.34037 .32186 .33285 .32624 .31539 .28183 .32257 .2883 Metetra
.733 .205 0 r
.3602 .34012 .35312 .35064 .33285 .32624 .34037 .32186 Metetra
.358 0 0 r
.38164 .34111 .37562 .35196 .35312 .35064 .3602 .34012 Metetra
.40371 .33107 .39901 .33852 .37562 .35196 .38164 .34111 Metetra
.4257 .31738 .42227 .32027 .39901 .33852 .40371 .33107 Metetra
.443 .855 .69 r
.4473 .30501 .445 .30387 .42227 .32027 .4257 .31738 Metetra
.445 .877 .799 r
.46854 .29627 .46722 .29233 .445 .30387 .4473 .30501 Metetra
.367 .835 .874 r
.48954 .29186 .48911 .28651 .46722 .29233 .46854 .29627 Metetra
.70097 .23767 .7083 .22177 .69242 .275 .68461 .28183 Metetra
.68461 .28183 .69242 .275 .67521 .32914 .66715 .32624 Metetra
.66715 .32624 .67521 .32914 .65445 .35913 .64688 .35064 Metetra
.229 0 0 r
.64688 .35064 .65445 .35913 .63081 .36077 .62438 .35196 Metetra
.475 0 0 r
.62438 .35196 .63081 .36077 .60603 .34421 .60099 .33852 Metetra
.478 0 0 r
.60099 .33852 .60603 .34421 .58142 .32182 .57773 .32027 Metetra
0 .152 .603 r
.57773 .32027 .58142 .32182 .55749 .30179 .555 .30387 Metetra
0 .32 .725 r
.555 .30387 .55749 .30179 .53421 .28774 .53278 .29233 Metetra
0 .533 .839 r
.53278 .29233 .53421 .28774 .51136 .28068 .51089 .28651 Metetra
.221 .729 .892 r
.51089 .28651 .51136 .28068 .48864 .28068 .48911 .28651 Metetra
.801 .43 0 r
.31539 .28183 .30758 .275 .2917 .22177 .29903 .23767 Metetra
.854 .457 0 r
.33285 .32624 .32479 .32914 .30758 .275 .31539 .28183 Metetra
.766 .261 0 r
.35312 .35064 .34555 .35913 .32479 .32914 .33285 .32624 Metetra
.304 0 0 r
.37562 .35196 .36919 .36077 .34555 .35913 .35312 .35064 Metetra
.39901 .33852 .39397 .34421 .36919 .36077 .37562 .35196 Metetra
.479 .842 .59 r
.42227 .32027 .41858 .32182 .39397 .34421 .39901 .33852 Metetra
.538 .893 .672 r
.445 .30387 .44251 .30179 .41858 .32182 .42227 .32027 Metetra
.511 .905 .776 r
.46722 .29233 .46579 .28774 .44251 .30179 .445 .30387 Metetra
.403 .857 .864 r
.48911 .28651 .48864 .28068 .46579 .28774 .46722 .29233 Metetra
.7083 .22177 .71652 .20722 .70095 .26778 .69242 .275 Metetra
.69242 .275 .70095 .26778 .68384 .32996 .67521 .32914 Metetra
.67521 .32914 .68384 .32996 .66249 .36467 .65445 .35913 Metetra
.132 0 0 r
.65445 .35913 .66249 .36467 .63764 .36657 .63081 .36077 Metetra
.421 0 0 r
.63081 .36077 .63764 .36657 .61141 .34738 .60603 .34421 Metetra
0 .064 .559 r
.60603 .34421 .61141 .34738 .58538 .32153 .58142 .32182 Metetra
0 .129 .62 r
.58142 .32182 .58538 .32153 .56018 .29848 .55749 .30179 Metetra
0 .288 .726 r
.55749 .30179 .56018 .29848 .53577 .28236 .53421 .28774 Metetra
0 .51 .839 r
.53421 .28774 .53577 .28236 .51187 .27428 .51136 .28068 Metetra
.222 .729 .893 r
.51136 .28068 .51187 .27428 .48813 .27428 .48864 .28068 Metetra
.831 .483 0 r
.30758 .275 .29905 .26778 .28348 .20722 .2917 .22177 Metetra
.86 .492 0 r
.32479 .32914 .31616 .32996 .29905 .26778 .30758 .275 Metetra
.774 .291 0 r
.34555 .35913 .33751 .36467 .31616 .32996 .32479 .32914 Metetra
.225 0 0 r
.36919 .36077 .36236 .36657 .33751 .36467 .34555 .35913 Metetra
.405 .81 .598 r
.39397 .34421 .38859 .34738 .36236 .36657 .36919 .36077 Metetra
.579 .887 .603 r
.41858 .32182 .41462 .32153 .38859 .34738 .39397 .34421 Metetra
.606 .916 .66 r
.44251 .30179 .43982 .29848 .41462 .32153 .41858 .32182 Metetra
.559 .922 .757 r
.46579 .28774 .46423 .28236 .43982 .29848 .44251 .30179 Metetra
.431 .873 .855 r
.48864 .28068 .48813 .27428 .46423 .28236 .46579 .28774 Metetra
.71652 .20722 .72583 .19436 .7103 .26015 .70095 .26778 Metetra
.70095 .26778 .7103 .26015 .69307 .32816 .68384 .32996 Metetra
.68384 .32996 .69307 .32816 .67099 .36633 .66249 .36467 Metetra
.012 0 0 r
.66249 .36467 .67099 .36633 .64485 .36843 .63764 .36657 Metetra
0 .176 .641 r
.63764 .36657 .64485 .36843 .61711 .3473 .61141 .34738 Metetra
0 .076 .6 r
.61141 .34738 .61711 .3473 .58962 .31891 .58538 .32153 Metetra
0 .124 .638 r
.58538 .32153 .58962 .31891 .56309 .29366 .56018 .29848 Metetra
0 .272 .731 r
.56018 .29848 .56309 .29366 .53746 .27605 .53577 .28236 Metetra
0 .496 .84 r
.53577 .28236 .53746 .27605 .51242 .26723 .51187 .27428 Metetra
.224 .73 .894 r
.51187 .27428 .51242 .26723 .48758 .26723 .48813 .27428 Metetra
.846 .512 0 r
.29905 .26778 .2897 .26015 .27417 .19436 .28348 .20722 Metetra
.856 .508 0 r
.31616 .32996 .30693 .32816 .2897 .26015 .29905 .26778 Metetra
.761 .3 0 r
.33751 .36467 .32901 .36633 .30693 .32816 .31616 .32996 Metetra
.12 0 0 r
.36236 .36657 .35515 .36843 .32901 .36633 .33751 .36467 Metetra
.529 .885 .658 r
.38859 .34738 .38289 .3473 .35515 .36843 .36236 .36657 Metetra
.654 .918 .618 r
.41462 .32153 .41038 .31891 .38289 .3473 .38859 .34738 Metetra
.655 .931 .655 r
.43982 .29848 .43691 .29366 .41038 .31891 .41462 .32153 Metetra
.592 .933 .745 r
.46423 .28236 .46254 .27605 .43691 .29366 .43982 .29848 Metetra
.45 .883 .848 r
.48813 .27428 .48758 .26723 .46254 .27605 .46423 .28236 Metetra
0 0 .002 r
.72583 .19436 .73643 .1835 .72059 .25205 .7103 .26015 Metetra
.7103 .26015 .72059 .25205 .70293 .32318 .69307 .32816 Metetra
.69307 .32816 .70293 .32318 .67992 .36323 .67099 .36633 Metetra
.121 .672 .765 r
.67099 .36633 .67992 .36323 .65242 .36543 .64485 .36843 Metetra
0 .222 .709 r
.64485 .36843 .65242 .36543 .62316 .34325 .61711 .3473 Metetra
0 .1 .637 r
.61711 .3473 .62316 .34325 .59417 .31349 .58962 .31891 Metetra
0 .134 .66 r
.58962 .31891 .59417 .31349 .56624 .28706 .56309 .29366 Metetra
0 .271 .742 r
.56309 .29366 .56624 .28706 .53932 .26865 .53746 .27605 Metetra
0 .492 .845 r
.53746 .27605 .53932 .26865 .51303 .25944 .51242 .26723 Metetra
.225 .731 .895 r
.51242 .26723 .51303 .25944 .48697 .25944 .48758 .26723 Metetra
.853 .525 .002 r
.2897 .26015 .27941 .25205 .26357 .1835 .27417 .19436 Metetra
.847 .512 0 r
.30693 .32816 .29707 .32318 .27941 .25205 .2897 .26015 Metetra
.731 .289 0 r
.32901 .36633 .32008 .36323 .29707 .32318 .30693 .32816 Metetra
.006 .578 .765 r
.35515 .36843 .34758 .36543 .32008 .36323 .32901 .36633 Metetra
.625 .937 .709 r
.38289 .3473 .37684 .34325 .34758 .36543 .35515 .36843 Metetra
.707 .941 .637 r
.41038 .31891 .40583 .31349 .37684 .34325 .38289 .3473 Metetra
.689 .943 .66 r
.43691 .29366 .43376 .28706 .40583 .31349 .41038 .31891 Metetra
.612 .94 .742 r
.46254 .27605 .46068 .26865 .43376 .28706 .43691 .29366 Metetra
.46 .888 .845 r
.48758 .26723 .48697 .25944 .46068 .26865 .46254 .27605 Metetra
0 0 .022 r
.73643 .1835 .7486 .17486 .73199 .2434 .72059 .25205 Metetra
.72059 .25205 .73199 .2434 .71346 .31455 .70293 .32318 Metetra
.70293 .32318 .71346 .31455 .68928 .35462 .67992 .36323 Metetra
.252 .759 .871 r
.67992 .36323 .68928 .35462 .66036 .35683 .65242 .36543 Metetra
0 .272 .765 r
.65242 .36543 .66036 .35683 .62956 .33463 .62316 .34325 Metetra
0 .136 .675 r
.62316 .34325 .62956 .33463 .59906 .30485 .59417 .31349 Metetra
0 .159 .687 r
.59417 .31349 .59906 .30485 .56967 .27842 .56624 .28706 Metetra
0 .286 .759 r
.56624 .28706 .56967 .27842 .54135 .26 .53932 .26865 Metetra
0 .498 .852 r
.53932 .26865 .54135 .26 .51371 .25079 .51303 .25944 Metetra
.226 .732 .896 r
.51303 .25944 .51371 .25079 .48629 .25079 .48697 .25944 Metetra
.854 .525 .003 r
.27941 .25205 .26801 .2434 .2514 .17486 .26357 .1835 Metetra
.833 .504 0 r
.29707 .32318 .28654 .31455 .26801 .2434 .27941 .25205 Metetra
.685 .258 0 r
.32008 .36323 .31072 .35462 .28654 .31455 .29707 .32318 Metetra
.137 .664 .873 r
.34758 .36543 .33964 .35683 .31072 .35462 .32008 .36323 Metetra
.694 .97 .755 r
.37684 .34325 .37044 .33463 .33964 .35683 .34758 .36543 Metetra
.745 .959 .663 r
.40583 .31349 .40094 .30485 .37044 .33463 .37684 .34325 Metetra
.71 .954 .674 r
.43376 .28706 .43033 .27842 .40094 .30485 .40583 .31349 Metetra
.622 .945 .748 r
.46068 .26865 .45865 .26 .43033 .27842 .43376 .28706 Metetra
.461 .888 .846 r
.48697 .25944 .48629 .25079 .45865 .26 .46068 .26865 Metetra
0 0 .028 r
.7486 .17486 .76263 .1686 .74468 .23409 .73199 .2434 Metetra
.73199 .2434 .74468 .23409 .72474 .30185 .71346 .31455 Metetra
.71346 .31455 .72474 .30185 .69908 .33991 .68928 .35462 Metetra
.362 .814 .941 r
.68928 .35462 .69908 .33991 .66866 .342 .66036 .35683 Metetra
0 .328 .81 r
.66036 .35683 .66866 .342 .63634 .32093 .62956 .33463 Metetra
0 .184 .715 r
.62956 .33463 .63634 .32093 .60432 .29263 .59906 .30485 Metetra
0 .199 .72 r
.59906 .30485 .60432 .29263 .57342 .26747 .56967 .27842 Metetra
0 .317 .783 r
.56967 .27842 .57342 .26747 .54359 .24993 .54135 .26 Metetra
.007 .514 .863 r
.54135 .26 .54359 .24993 .51445 .24114 .51371 .25079 Metetra
.228 .732 .897 r
.51371 .25079 .51445 .24114 .48555 .24114 .48629 .25079 Metetra
.85 .514 0 r
.26801 .2434 .25532 .23409 .23737 .1686 .2514 .17486 Metetra
.813 .484 0 r
.28654 .31455 .27526 .30185 .25532 .23409 .26801 .2434 Metetra
.62 .205 0 r
.31072 .35462 .30092 .33991 .27526 .30185 .28654 .31455 Metetra
.255 .725 .944 r
.33964 .35683 .33134 .342 .30092 .33991 .31072 .35462 Metetra
.738 .989 .797 r
.37044 .33463 .36366 .32093 .33134 .342 .33964 .35683 Metetra
.769 .974 .696 r
.40094 .30485 .39568 .29263 .36366 .32093 .37044 .33463 Metetra
.721 .964 .7 r
.43033 .27842 .42658 .26747 .39568 .29263 .40094 .30485 Metetra
.62 .948 .764 r
.45865 .26 .45641 .24993 .42658 .26747 .43033 .27842 Metetra
.453 .885 .851 r
.48629 .25079 .48555 .24114 .45641 .24993 .45865 .26 Metetra
0 0 .016 r
.76263 .1686 .7789 .16478 .75887 .22399 .74468 .23409 Metetra
.74468 .23409 .75887 .22399 .73687 .28478 .72474 .30185 Metetra
.72474 .30185 .73687 .28478 .70937 .31872 .69908 .33991 Metetra
.444 .84 .979 r
.69908 .33991 .70937 .31872 .67735 .32059 .66866 .342 Metetra
.044 .39 .85 r
.66866 .342 .67735 .32059 .64354 .30181 .63634 .32093 Metetra
0 .246 .76 r
.63634 .32093 .64354 .30181 .61 .27654 .60432 .29263 Metetra
0 .256 .762 r
.60432 .29263 .61 .27654 .57753 .25399 .57342 .26747 Metetra
0 .363 .814 r
.57342 .26747 .57753 .25399 .54608 .23823 .54359 .24993 Metetra
.036 .54 .876 r
.54359 .24993 .54608 .23823 .51529 .23033 .51445 .24114 Metetra
.229 .733 .898 r
.51445 .24114 .51529 .23033 .48471 .23033 .48555 .24114 Metetra
.838 .487 0 r
.25532 .23409 .24113 .22399 .2211 .16478 .23737 .1686 Metetra
.782 .447 0 r
.27526 .30185 .26313 .28478 .24113 .22399 .25532 .23409 Metetra
0 0 .443 r
.30092 .33991 .29063 .31872 .26313 .28478 .27526 .30185 Metetra
.351 .761 .981 r
.33134 .342 .32265 .32059 .29063 .31872 .30092 .33991 Metetra
.76 .996 .838 r
.36366 .32093 .35646 .30181 .32265 .32059 .33134 .342 Metetra
.779 .986 .738 r
.39568 .29263 .39 .27654 .35646 .30181 .36366 .32093 Metetra
.719 .973 .736 r
.42658 .26747 .42247 .25399 .39 .27654 .39568 .29263 Metetra
.606 .947 .789 r
.45641 .24993 .45392 .23823 .42247 .25399 .42658 .26747 Metetra
.437 .876 .859 r
.48555 .24114 .48471 .23033 .45392 .23823 .45641 .24993 Metetra
.7789 .16478 .79787 .16337 .77485 .21288 .75887 .22399 Metetra
.75887 .22399 .77485 .21288 .74998 .26312 .73687 .28478 Metetra
.618 .856 .491 r
.73687 .28478 .74998 .26312 .72023 .29091 .70937 .31872 Metetra
.498 .846 .995 r
.70937 .31872 .72023 .29091 .68651 .29243 .67735 .32059 Metetra
.143 .46 .887 r
.67735 .32059 .68651 .29243 .65124 .27709 .64354 .30181 Metetra
0 .325 .811 r
.64354 .30181 .65124 .27709 .61619 .25634 .61 .27654 Metetra
0 .332 .812 r
.61 .27654 .61619 .25634 .58207 .23776 .57753 .25399 Metetra
0 .426 .85 r
.57753 .25399 .58207 .23776 .54885 .22471 .54608 .23823 Metetra
.072 .575 .889 r
.54608 .23823 .54885 .22471 .51622 .21816 .51529 .23033 Metetra
.23 .734 .898 r
.51529 .23033 .51622 .21816 .48378 .21816 .48471 .23033 Metetra
.813 .437 0 r
.24113 .22399 .22515 .21288 .20213 .16337 .2211 .16478 Metetra
.734 .383 0 r
.26313 .28478 .25002 .26312 .22515 .21288 .24113 .22399 Metetra
0 0 .557 r
.29063 .31872 .27977 .29091 .25002 .26312 .26313 .28478 Metetra
.422 .78 .996 r
.32265 .32059 .31349 .29243 .27977 .29091 .29063 .31872 Metetra
.761 .992 .88 r
.35646 .30181 .34876 .27709 .31349 .29243 .32265 .32059 Metetra
.773 .994 .791 r
.39 .27654 .38381 .25634 .34876 .27709 .35646 .30181 Metetra
.702 .978 .783 r
.42247 .25399 .41793 .23776 .38381 .25634 .39 .27654 Metetra
.578 .941 .821 r
.45392 .23823 .45115 .22471 .41793 .23776 .42247 .25399 Metetra
.412 .862 .87 r
.48471 .23033 .48378 .21816 .45115 .22471 .45392 .23823 Metetra
.79787 .16337 .8201 .16424 .79298 .20053 .77485 .21288 Metetra
.77485 .21288 .79298 .20053 .76429 .23676 .74998 .26312 Metetra
.65 .92 .628 r
.74998 .26312 .76429 .23676 .73179 .25654 .72023 .29091 Metetra
.529 .841 .999 r
.72023 .29091 .73179 .25654 .69626 .25762 .68651 .29243 Metetra
.243 .538 .923 r
.68651 .29243 .69626 .25762 .65953 .24673 .65124 .27709 Metetra
.099 .425 .869 r
.65124 .27709 .65953 .24673 .62297 .2319 .61619 .25634 Metetra
.042 .43 .869 r
.61619 .25634 .62297 .2319 .58713 .21854 .58207 .23776 Metetra
.049 .505 .89 r
.58207 .23776 .58713 .21854 .55198 .20911 .54885 .22471 Metetra
.116 .616 .903 r
.54885 .22471 .55198 .20911 .51727 .20436 .51622 .21816 Metetra
.23 .734 .899 r
.51622 .21816 .51727 .20436 .48273 .20436 .48378 .21816 Metetra
.762 .345 0 r
.22515 .21288 .20702 .20053 .1799 .16424 .20213 .16337 Metetra
.648 .269 0 r
.25002 .26312 .23571 .23676 .20702 .20053 .22515 .21288 Metetra
0 .165 .696 r
.27977 .29091 .26821 .25654 .23571 .23676 .25002 .26312 Metetra
.473 .791 .999 r
.31349 .29243 .30374 .25762 .26821 .25654 .27977 .29091 Metetra
.738 .977 .921 r
.34876 .27709 .34047 .24673 .30374 .25762 .31349 .29243 Metetra
.744 .993 .852 r
.38381 .25634 .37703 .2319 .34047 .24673 .34876 .27709 Metetra
.663 .973 .84 r
.41793 .23776 .41287 .21854 .37703 .2319 .38381 .25634 Metetra
.531 .923 .857 r
.45115 .22471 .44802 .20911 .41287 .21854 .41793 .23776 Metetra
.377 .841 .881 r
.48378 .21816 .48273 .20436 .44802 .20911 .45115 .22471 Metetra
.8201 .16424 .84633 .16708 .81369 .18658 .79298 .20053 Metetra
.79298 .20053 .81369 .18658 .78007 .20564 .76429 .23676 Metetra
.648 .963 .792 r
.76429 .23676 .78007 .20564 .74426 .21589 .73179 .25654 Metetra
.544 .832 .998 r
.73179 .25654 .74426 .21589 .70675 .21645 .69626 .25762 Metetra
.345 .627 .957 r
.69626 .25762 .70675 .21645 .66858 .21082 .65953 .24673 Metetra
.222 .546 .929 r
.65953 .24673 .66858 .21082 .6305 .20311 .62297 .2319 Metetra
.156 .549 .927 r
.62297 .2319 .6305 .20311 .59281 .19611 .58713 .21854 Metetra
.138 .597 .926 r
.58713 .21854 .59281 .19611 .55551 .19113 .55198 .20911 Metetra
.164 .662 .914 r
.55198 .20911 .55551 .19113 .51848 .18861 .51727 .20436 Metetra
.231 .734 .899 r
.51727 .20436 .51848 .18861 .48152 .18861 .48273 .20436 Metetra
.645 .163 0 r
.20702 .20053 .18631 .18658 .15367 .16708 .1799 .16424 Metetra
0 0 .506 r
.23571 .23676 .21993 .20564 .18631 .18658 .20702 .20053 Metetra
.01 .384 .845 r
.26821 .25654 .25574 .21589 .21993 .20564 .23571 .23676 Metetra
.507 .798 .998 r
.30374 .25762 .29325 .21645 .25574 .21589 .26821 .25654 Metetra
.687 .945 .96 r
.34047 .24673 .33142 .21082 .29325 .21645 .30374 .25762 Metetra
.682 .972 .917 r
.37703 .2319 .3695 .20311 .33142 .21082 .34047 .24673 Metetra
.591 .947 .898 r
.41287 .21854 .40719 .19611 .3695 .20311 .37703 .2319 Metetra
.463 .888 .891 r
.44802 .20911 .44449 .19113 .40719 .19611 .41287 .21854 Metetra
.334 .812 .891 r
.48273 .20436 .48152 .18861 .44449 .19113 .44802 .20911 Metetra
.84633 .16708 .87749 .17144 .83757 .17059 .81369 .18658 Metetra
.379 .809 .637 r
.81369 .18658 .83757 .17059 .7977 .16978 .78007 .20564 Metetra
.579 .934 .936 r
.78007 .20564 .7977 .16978 .75794 .16935 .74426 .21589 Metetra
.545 .823 .997 r
.74426 .21589 .75794 .16935 .71825 .16932 .70675 .21645 Metetra
.447 .722 .984 r
.70675 .21645 .71825 .16932 .6786 .16956 .66858 .21082 Metetra
.359 .685 .98 r
.66858 .21082 .6786 .16956 .63894 .16988 .6305 .20311 Metetra
.287 .683 .973 r
.6305 .20311 .63894 .16988 .59926 .17018 .59281 .19611 Metetra
.236 .693 .951 r
.59281 .19611 .59926 .17018 .55956 .17039 .55551 .19113 Metetra
.216 .709 .92 r
.55551 .19113 .55956 .17039 .51986 .1705 .51848 .18861 Metetra
.231 .734 .899 r
.51848 .18861 .51986 .1705 .48014 .1705 .48152 .18861 Metetra
0 .209 .622 r
.18631 .18658 .16243 .17059 .12251 .17144 .15367 .16708 Metetra
0 .361 .813 r
.21993 .20564 .2023 .16978 .16243 .17059 .18631 .18658 Metetra
.289 .624 .961 r
.25574 .21589 .24206 .16935 .2023 .16978 .21993 .20564 Metetra
.529 .805 .997 r
.29325 .21645 .28175 .16932 .24206 .16935 .25574 .21589 Metetra
.605 .893 .989 r
.33142 .21082 .3214 .16956 .28175 .16932 .29325 .21645 Metetra
.572 .913 .97 r
.3695 .20311 .36106 .16988 .3214 .16956 .33142 .21082 Metetra
.478 .884 .943 r
.40719 .19611 .40074 .17018 .36106 .16988 .3695 .20311 Metetra
.371 .83 .915 r
.44449 .19113 .44044 .17039 .40074 .17018 .40719 .19611 Metetra
.284 .775 .898 r
.48152 .18861 .48014 .1705 .44044 .17039 .44449 .19113 Metetra
.056 0 0 r
.87749 .17144 .91478 .17662 .86538 .15196 .83757 .17059 Metetra
.194 .709 .885 r
.83757 .17059 .86538 .15196 .81771 .12912 .7977 .16978 Metetra
.436 .805 .994 r
.7977 .16978 .81771 .12912 .77325 .11733 .75794 .16935 Metetra
.536 .814 .997 r
.75794 .16935 .77325 .11733 .73112 .1167 .71825 .16932 Metetra
.544 .817 .997 r
.71825 .16932 .73112 .1167 .68989 .12312 .6786 .16956 Metetra
.499 .821 1 r
.6786 .16956 .68989 .12312 .64855 .13208 .63894 .16988 Metetra
.42 .811 .987 r
.63894 .16988 .64855 .13208 .60667 .1404 .59926 .17018 Metetra
.334 .783 .955 r
.59926 .17018 .60667 .1404 .56425 .14639 .55956 .17039 Metetra
.267 .753 .92 r
.55956 .17039 .56425 .14639 .52146 .14946 .51986 .1705 Metetra
.231 .734 .899 r
.51986 .1705 .52146 .14946 .47854 .14946 .48014 .1705 Metetra
.226 .736 .885 r
.16243 .17059 .13462 .15196 .08522 .17662 .12251 .17144 Metetra
.462 .825 .994 r
.2023 .16978 .18229 .12912 .13462 .15196 .16243 .17059 Metetra
.545 .82 .997 r
.24206 .16935 .22675 .11733 .18229 .12912 .2023 .16978 Metetra
.541 .814 .996 r
.28175 .16932 .26888 .1167 .22675 .11733 .24206 .16935 Metetra
.488 .813 1 r
.3214 .16956 .31011 .12312 .26888 .1167 .28175 .16932 Metetra
.408 .801 .987 r
.36106 .16988 .35145 .13208 .31011 .12312 .3214 .16956 Metetra
.324 .775 .955 r
.40074 .17018 .39333 .1404 .35145 .13208 .36106 .16988 Metetra
.26 .748 .92 r
.44044 .17039 .43575 .14639 .39333 .1404 .40074 .17018 Metetra
.229 .732 .899 r
.48014 .1705 .47854 .14946 .43575 .14639 .44044 .17039 Metetra
0 .33 .742 r
.91478 .17662 .95987 .1816 .89817 .12988 .86538 .15196 Metetra
0 .42 .851 r
.86538 .15196 .89817 .12988 .84082 .08355 .81771 .12912 Metetra
.268 .624 .96 r
.81771 .12912 .84082 .08355 .79078 .06019 .77325 .11733 Metetra
.517 .806 .998 r
.77325 .11733 .79078 .06019 .74584 .05895 .73112 .1167 Metetra
.624 .899 .986 r
.73112 .1167 .74584 .05895 .70287 .07162 .68989 .12312 Metetra
.614 .926 .97 r
.68989 .12312 .70287 .07162 .65967 .08948 .64855 .13208 Metetra
.532 .906 .956 r
.64855 .13208 .65967 .08948 .61532 .10624 .60667 .1404 Metetra
.422 .855 .936 r
.60667 .1404 .61532 .10624 .56975 .11846 .56425 .14639 Metetra
.315 .793 .914 r
.56425 .14639 .56975 .11846 .52334 .12475 .52146 .14946 Metetra
.231 .734 .899 r
.52146 .14946 .52334 .12475 .47666 .12475 .47854 .14946 Metetra
.661 .973 .845 r
.13462 .15196 .10183 .12988 .04013 .1816 .08522 .17662 Metetra
.782 .989 .891 r
.18229 .12912 .15918 .08355 .10183 .12988 .13462 .15196 Metetra
.721 .934 .958 r
.22675 .11733 .20922 .06019 .15918 .08355 .18229 .12912 Metetra
.543 .824 .998 r
.26888 .1167 .25416 .05895 .20922 .06019 .22675 .11733 Metetra
.344 .707 .984 r
.31011 .12312 .29713 .07162 .25416 .05895 .26888 .1167 Metetra
.208 .64 .951 r
.35145 .13208 .34033 .08948 .29713 .07162 .31011 .12312 Metetra
.148 .627 .923 r
.39333 .1404 .38468 .10624 .34033 .08948 .35145 .13208 Metetra
.142 .648 .904 r
.43575 .14639 .43025 .11846 .38468 .10624 .39333 .1404 Metetra
.172 .685 .895 r
.47854 .14946 .47666 .12475 .43025 .11846 .43575 .14639 Metetra
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{223.375, 87},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003O0000Ea000`40O003h00Oggoo003OOol00=mo
o`00ggoo003OOol00=moo`00ggoo003OOol00=moo`00ggoo003OOol00=moo`00ggoo003OOol002eo
o`l006Moo`h002ioo`00:goo0P0000=O_P00Ael024MO00@002kO;]l^g`D005eoo`D0009?W`030013
?d<o00M3?`04000ROP00000/Ool002Qoo`<000AO_P030017GdMO00M7G`03000^gbkO00D^g`<005Io
o`@000M?W`030013?d<o00I3?`03000ROR9n008ROP<002Uoo`009Woo0P001enn00<004MOAel01dMO
00<002kO;]l01RkO00@001ZN0000059oo`800003Ckl004nO00M?W`030013?d<o00M3?`03000ROR9n
00DROP8002Moo`0097oo0P002Enn00<004MOAel01dMO00<002kO;]l01RkO00@001ZN6YhJWP<004ao
o`<000=?_`03001?WdnO00I?W`030013?d<o00M3?`03000ROR9n00LROP8002Eoo`008Woo0P0035nn
00<004MOAel01dMO00<002kO;]l01RkO00<001ZN6Yh00aZN0P00B7oo0P001Tno00<004nOCil01TnO
00<004<o@cl01T<o00<0029n8Wh02R9n0P008goo000QOol00`00Io`0000<Gkh00`00Aem7G`07Ael0
0`00;]l^g`06;]l00`006YhJWP056Yh20014Ool20007Ckl00`00Cim?W`07Cil00`00@cm3?`06@cl0
0`008WhROP0:8Wh00`000K/0000ROol0021oo`05001Wo6OlIo`0000;Gkh00`00Aem7G`07Ael00`00
;]l^g`06;]l00`006YhJWP056Yh00`004Yd00002000nOol300000dNN001?_`06Ckl00`00Cim?W`06
Cil00`00@cm3?`07@cl00`008WhROP098Wh01@000K/1^`6k00008Goo000OOol00`00IoaWo003Io`0
0`00GkiO_P09Gkh00`00Aem7G`07Ael00`00;]l^g`05;]l00`006YhJWP056Yh00`004YdBW@024Yd2
000jOol20004Aih00`00Ckm?_`05Ckl00`00Cim?W`06Cil00`00@cm3?`06@cl00`008WhROP0:8Wh0
0`000K/1^`020K/2000OOol001ioo`03001Wo6Ol00AWo003001O_Unn00UO_P030017GdMO00M7G`03
000^gbkO00H^g`03000JWQZN00DJWP03000BWA:M00<BW@<003Aoo`<000E7WP03001?_dno00I?_`03
001?WdnO00I?W`030013?d<o00I3?`03000ROR9n00TROP030001^`6k00D1^`03001oogoo01aoo`00
77oo0P0026Ol00<005nnGkh025nn00<004MOAel01dMO00<002kO;]l01RkO00<001ZN6Yh01AZN00<0
01:M4Yd01Q:M0`00;Woo0`0024NN00<004noCkl01Tno00<004nOCil01TnO00<004<o@cl01T<o00<0
029n8Wh02B9n00<0006k0K/01P6k00<007ooOol06goo000KOol00`00IoaWo008Io`00`00GkiO_P09
Gkh00`00Aem7G`07Ael00`00;]l^g`05;]l00`006YhJWP056Yh00`004YdBW@064Yd00`004Y`BW003
000XOol30002=gd00`00Aii7WP06Aih00`00Ckm?_`06Ckl00`00Cim?W`05Cil00`00@cm3?`06@cl0
0`008WhROP098Wh00`000K/1^`080K/00`00Oomoo`0JOol001Yoo`03001Wo6Ol00YWo003001O_Unn
00MO_Ph000D^g`03000JWQZN00HJWP03000BWA:M00DBW@03000BW1:L00<BW0<0029oo`<000DgO@03
0017WTNN00E7WP03001?_dno00I?_`03001?WdnO00A?W`h000TROP030001^`6k00T1^`8001Yoo`00
6Goo00<006OlIo`036Ol00<005nnGkh00enn0`0000=7G`00Ael024MO00<003mO?el01@0000<^g`00
6Yh01QZN00<001:M4Yd01Q:M00<001:L4Y`01A:L20004goo1`001cMm00<004NNAih01TNN00<004no
Ckl01Tno00<004nO000010000TMO00<004MOAel01TMO00<003Lo=cl00`001B9n00<0006k0K/0306k
00<007ooOol05goo000HOol00`00IoaWo00=Io`00`00GkiO_P030005Ael00`00Aem7G`06Ael00`00
?eloG`05?el400056Yh00`004YdBW@064Yd00`004Y`BW0064Y`00`005[`F_0045[`C0005:cd00`00
=gdgO@06=gd00`00Aii7WP06Aih00`00Ckm?_`04Ckl40007Ael00`00Aem7G`06Ael00`00=clg?`03
=cl300028Wh00`000K/1^`0=0K/00`00Oomoo`0FOol001Ioo`<000mWo0<000Q7G`030017GdMO00I7
G`03000oGcmO00HoG`03000g?cLo00D000036Yh001:M00HBW@03000BW1:L00LBW003000F_1Jl00LF
_003000No1kl00LNo003000[?B/m00L[?@03000gOCMm00HgO@030017WTNN00E7WP03001?_dno00D0
008oG`030017GdMO00E7G`030017GdMO00I7G`03000g?cLo00Lg?`<000l1^`<001Ioo`005Goo00D0
05OkEo]Gn`0000aWo0800003>el004MO00Q7G`030017GdMO00M7G`03000oGcmO00DoG`03000g?cLo
00Dg?`L000<BW@03000BW1:L00HBW003000F_1Jl00LF_003000No1kl00LNo003000[?B/m00L[?@03
000gOCMm00DgO@030017WTNN0097WPL000LoG`030017GdMO00E7G`030017GdMO00I7G`03000g?cLo
00Pg?`04000Jg000000<0K/01@000EL1E`5G00005Goo000DOol00`00Eo]Gn`03Eo/20007Io`30004
>el00`00Aem7G`07Ael00`00Aem7G`06Ael00`00?eloG`06?el00`00=clg?`05=cl00`00:ah[7P03
:ah700054Y`00`005[`F_0075[`00`007_`No0067_`00`00:cd[?@07:cd00`00=gdgO@04=gd70004
=cl00`00?eloG`06?el00`00Aem7G`06Ael00`00Aem7G`06Ael00`00=clg?`07=cl00`006]`Jg002
6]`200080K/200050EL00`00Oomoo`0BOol001=oo`03001GneOk00IGn`03001Wo6Ol009Wo08000Lk
G`030017GdMO00M7G`030017GdMO00I7G`03000oGcmO00HoG`03000g?cLo00Dg?`03000[7R/N00H[
7P04000RoB;m8_d500000a:L000F_0075[`00`007_`No0067_`00`00:cd[?@07:cd00`00=gd00004
0002:ch00`00=clg?`06=cl00`00?eloG`06?el00`00Aem7G`06Ael00`00Aem7G`06Ael00`00=clg
?`07=cl00`006]`Jg0046]`200040K/200080EL00`00Oomoo`0AOol0019oo`03001GneOk00QGn`04
001Wo000000:>el00`00Aem7G`06Ael00`00Aem7G`06Ael00`00?eloG`06?el00`00=clg?`06=cl0
0`00:ah[7P05:ah00`008_dRo@068_dO0007:ch00`00=clg?`05=cl00`00?eloG`07?el00`00Aem7
G`05Ael00`00Aem7G`06Ael00`00=clg?`07=cl00`006]`Jg0076]`200000`6k0001E`0:0EL00`00
Oomoo`0@Ool0015oo`03001GneOk00UGn`<000/kG`030017GdMO00M7G`030017GdMO00E7G`03000o
GcmO00HoG`03000g?cLo00Hg?`03000[7R/N00H[7P03000RoB;m00HRo@03000No1kl00HNo003000N
o1kl00HNo003000S7B<M00LS7@03000[?R/n00H[?P03000g?cLo00Dg?`03000oGcmO00HoG`030017
GdMO00I7G`030017GdMO00I7G`03000g?cLo00Hg?`03000Jg1[L00TJg0<000`1E`03001oogoo00mo
o`003goo0P002UOk0P000akl00<003]O>el023]O00<004MOAel01dMO00<004MOAel01DMO00<003mO
?el01SmO00<003Lo=cl01SLo00<002/N:ah01R/N00<002;m8_d01R;m00<001kl7_`01Qkl00<001kl
7_`01Qkl00<002<M8ad01b<M00<002/n:ch01B/n00<003Lo=cl01SLo00<003mO?el01SmO00<004MO
Ael01TMO00<004MOAel01TMO00<003Lo=cl01SLo00<001[L6]`021[L00@000@01004008000/1E`03
001oogoo00ioo`003Woo00<005OkEo/01eOk0`001Qkl0P002C]O00<004MOAel01TMO00<004MOAel0
1TMO00<003mO?el01SmO00<003Lo=cl01CLo00<002/N:ah01R/N00<002;m8_d01b;m00<001kl7_`0
1Akl00<001kl7_`01Qkl00<002<M8ad01R<M00<002/n:ch01R/n00<003Lo=cl01SLo00<003mO?el0
1SmO00<004MOAel01TMO00<004MOAel01TMO00<003Lo=cl01CLo00<001[L6]`01a[L0P001P@00`00
2@5G00<007ooOol03Goo000=Ool00`00Eo]Gn`06Eo/2000;7_`00`00>elkG`06>el00`00Aem7G`07
Ael00`00Aem7G`05Ael00`00?eloG`06?el00`00=clg?`06=cl00`00:ah[7P06:ah00`008_dRo@06
8_d00`007_`No0057_`00`007_`No0067_`00`008adS7@068ad00`00:ch[?P06:ch00`00=clg?`05
=cl00`00?eloG`06?el00`00Aem7G`06Ael00`00Aem7G`06Ael00`00=clg?`06=cl00`006]`Jg006
6]`00`0010040009100200080EL00`00Oomoo`0<Ool000aoo`03001GneOk00EGn`8000037_`00000
0:h000031000000000L1E`03001oogoo00]oo`002goo00<005OkEo/00eOk1P003Woo0P001P5j00<0
02In9Wh01RIn00<004<o@cl01D<o00<004nOCil01TnO00<004^oBkl01T^o00<003nN?ih01SnN00<0
02mM;ed01RmM00<002LL9a`01BLL00<001kl7_`01Qkl00<001[M6]d01Q[M00<001kN7]h01QkN00<0
02Jo9[l01BJo00<002jo;[l01Rjo00<003[o>_l01S[o00<004MOAel01TMO00<004^mBkd01D^m00<0
03<d<c@013<d0P003Woo1`00105G00<007ooOol02Woo000:Ool00`00Eo/00004000FOol00`000GX1
NP040GX00`009WhVOP059Wh00`00@cm3?`05@cl00`00Cim?W`06Cil00`00Bkm;_`06Bkl00`00?iho
WP06?ih00`00;ed_G@06;ed00`009a`W70059a`00`007_`No0067_`00`006]dJg@066]d00`007]hN
gP057]h00`009[lV_`069[l00`00;[l^_`06;[l00`00>_ljo`06>_l00`00Aem7G`06Ael00`00Bke;
_@04Bkd00`00<c@c=004<c@00`00Oomoo`0EOol6000;Ool000Uoo`<001aoo`030001NP5j00<1NP03
000VORIn00HVOP030013?d<o00E3?`03001?WdnO00I?W`03001;_d^o00I;_`03000oWSnN00DoWP03
000_GBmM00H_G@03000W72LL00HW7003000No1kl00DNo003000JgA[M00DJg@03000NgQkN00HNgP03
000V_bJo00HV_`03000^_bjo00D^_`03000joc[o00Hjo`030017GdMO00I7G`03001;_D^m00E;_@03
000c=3<d00<c=003001oogoo01]oo`8000Yoo`00:Goo0P00105j00<002In9Wh01BIn00<004<o@cl0
1D<o00<004nOCil01TnO00<004^oBkl01T^o00<003nN?ih01SnN00<002mM;ed01BmM00<002LL9a`0
0RLL4@000a[M00<001kN7]h01QkN00<002Jo9[l01BJo00<002jo;[l01Rjo00<003[o>_l01S[o00<0
04MOAel01TMO00<004^mBkd01D^m00@003<d<c@c=08002Yoo`00:goo0P000P5j00<002In9Wh01BIn
00<004<o@cl01D<o00<004nOCil01TnO00<004^oBkl01T^o00<003nN?ih01SnN00@002mM;ed_G@T0
00@[?003000No1kl00DNo003000F_AJm008F_@P000<NgP03000V_bJo00HV_`03000^_bjo00H^_`03
000joc[o00Hjo`030017GdMO00I7G`03001;_D^m00A;_@03000c=3<d008002aoo`00;Goo0P0000<1
NP009Wh01RIn00<004<o@cl01D<o00<004nOCil01DnO00<004^oBkl01d^o00@003nN?ihoWPP000@k
W003000[?2/l00H[?003000No1kl00DNo003000F_AJm00DF_@03000BOA9m008BO@T000<V_`03000^
_bjo00H^_`03000joc[o00Djo`030017GdMO00I7G`03001;_D^m00E;_@03000c=00002ioo`00;Woo
0`001bIn00<004<o@cl01D<o00<004nOCil01DnO00<004^oBkl01D^o1P0014_L00<003^L>i`01C^L
00<002/l:c`01R/l00<001kl7_`01Akl00<001Jm5[d01AJm00<0019m4Wd01A9m00<0018m4Sd00a8m
1P001Rjo00<003[o>_l01C[o00<004MOAel01TMO00<004^mBkd01D^m0P00;goo000^Ool00`000100
400300049Wh00`00@cm3?`05@cl00`00Cim?W`06Cil00`00Bkl000040002Eod00`00Bma;g005Bm`0
0`00>i`kW006>i`01000:c`[?2/l4@000QJm00<0019m4Wd01Q9m00<0018m4Sd01Q8m00<001hm7Sd0
100000<^_`00>_l01c[o00<004MOAel01TMO00<004^mBkd00T^m1@00;goo000_Ool00`0001000002
0I/400000bIn0013?`06@cl01000Cim?WdnO1P001eOm00<004_LBm`01T_L00<003^L>i`00c^L1P00
13=L00<001kl7_`01Akl00<000il3W`00Pil1@001A9m00<0018m4Sd01a8m00<001hm7Sd011hm1P00
0c[o00<004MOAel01TMO00<004^m00000`000U?I0P00<7oo000`Ool00`00010000050I/=0004Emh0
0`00EoeGo@07Eod00`00Bma;g005Bm`00`00>i`000030002@k/00`00<e`cG005<e`00`007_`No004
7_`00`003W`>O0053W`00`001Q`6700400000a9m000B?@074Sd00`007SdN?@067Sd00`00;Yh^WP02
;Yh>0005DmT2000aOol0035oo`8000H1V`030013?d<o00E3?`03001GgUON00EGgP03001GoEOm00MG
o@03001;g4_L00A;g0<000M3^`04000cG3=L<e`?00023W`00`001Q`670051Q`300074Sd00`007SdN
?@067Sd00`00;Yh^WP05;Yh00`00Aem7G`06Ael00`00DmUCf@04DmT00`00Oomoo`0`Ool0035oo`03
0000400000H1V`030013?d<o00A3?`03001GgUON00IGgP03001GoEOm00IGo@04001;g4_LBm`30000
0eOj0013^`06@k/50003=g/00`007_`No0047_`00`002U`:G0022U`400071Q`00`001K/000020004
4Sd00`007SdN?@057Sd00`00;Yh^WP06;Yh00`00Aem7G`05Ael00`00DmUCf@04DmT2000bOol0039o
o`8000H1V`030013?d<o00E3?`03001GgUON00EGgP03001GoEOm00MGo@<000EGnP040013^d>k@k/3
00000d_J000gN`04=g/;00042U`00`000K/0000200041Q`00`001K/5^`021K/400087Sd00`00;Yh^
WP05;Yh00`00Aem7G`06Ael00`00DmUCf@03DmT2000cOol003=oo`030001V`6K00@1V`030013?d<o
00E3?`03001GgUON00EGgP03001GoEOm00IGo@8000MGnP040013^`000005BmX00`00=g/000020000
0cNK000No0057_`00`001S`6?00300000`YL0001^`030K/400081K/200077Sd00`00;Yh^WP05;Yh0
0`00Aem7G`06Ael00`00DmUCf@03DmT00`00Oomoo`0bOol003Aoo`8000D1V`030013?d<o00A3?`03
001GgUON00EGgP03001GoEOm00=Go@<00003Go/005Oj00IGnP8000I;fP<000@gV`03000No1kl00@N
o0030006?0Hl00<6?0<000H1^`8000L5^`03000=^`00008000@N?@03000^WRjN00D^WP030017GdMO
00E7G`03001CfE?I009Cf@8003Eoo`00=7oo00@0002f000000<1V`030013?d<o00A3?`03001GgUON
00IGgP03001Go@00008000AOn`03001GnUOj00=GnP800003FoT004_J00=;fP800003CmT003NK00<g
V`/000@6?0040001NP0000040K/010000ET0000010Fk00<000fk3K/00`fk100022jN00<004MOAel0
1DMO00<005?IDmT00P0000=C]000Ool0=7oo000eOol00`000;H0]P0200000`6K0013?`05@cl00`00
EmiGgP04Emh40008Go/00`00EoYGnP020003FoT01@00BmY;fT_J00000doI00<003NK00000P0000<k
V`007_`01Akl00<0008K0Q/01000105j00D0006k0K/1^`0000<1F@050005^`Fk1K/000083K/30006
;Yh00`00Aem7G`05Ael00`00DmT00002Dk@00`00Oomoo`0dOol003Eoo`030000]P2f0080]P<000I3
?`03001GgP0000<0009Oo@03001Oneok00MOn`<000IKn@<000=?f@<000@kV`X000<26`8000@1NP<0
00D1F@8000031K/000fk00L=^`03000N?Ahm00@00003;Yh004MO00I7G`<000=C]003001oogoo03Ao
o`00=Woo00<0002f0;H00`2f2@001Uom00<005okGo/01Uok0P001e_i0P000doI0P0000A?f000>i/k
V`<00003>i/001kl00@No0030001n`7k00@000030ET0000000@1NP8000H1F@8000P=^`03000N?Ahm
00@N?@X000AC]003001oogoo03Eoo`00=Woo00<0002f0;H00`2f00<003lo?cl013lo00<005omGod0
1Uom00<005okGo/015ok00<006?i00001U_i00<005og00000ToI00@004oHCmQ?f0<000<kV`T000@1
n`800081F@070001NP5j0GX0004H00001@5I00<0005I00001Pfk00<001hm7Sd01Qhm00<004=O@el0
1D=O00<005>dDk@00U>d00<007ooOol0=Goo000gOol200030;H00`00?clo?`05?cl00`00GoeOo@05
God00`00Go]On`02Go/20002HoT00`00FoUKn@02FoT20002GoL30002CmP200000doG000kV`030000
0c^K000No0047_`00`000O/000020000107k0001>0000P5I0`000`4H0P000`5I00<0005I0ET00P00
10fk00<001hm7Sd01Qhm00<004=O@el014=O00@005>dDkAC]08003Qoo`00=goo00<0000A00000`2f
00<003lo?cl013lo00<005omGod01Eom00<005okGo/00P001F?i00@005_i000000=Om`8000=?f003
001?edoG00<00003>i/0000000X00081n`800081>0800081F@8000@160030001F@0000D1F@80008=
^`03000N?Ahm00DN?@030013Gd=O00E3G`06001C]5>d001?K`00>7oo000hOol00`00014000020;H0
0`00?clo?`04?cl00`00GoeOo@06God20007HoT20004GoL01000GmH004oH0P000ToG1@000S^K2@00
00<1nP0000000`000`4h00D0005I0000e`0000@1608000L1F@8000PN?@030013Gd=O00E3G`05001C
]000Cfl0000iOol003Uoo`0500004@000;H00007?cl00`00GoeOo@04God20007HoT20004GoL01@00
GmIOeUoF00000ToG0`0000=;]`00>i/02`0000<26P0000000P0000@15`0000000081>0800080e`80
00<1608000L1F@8000LN?@030013Gd=O00A3G`05001C]000Cfl0000jOol003Yoo`0400004@000007
?cl01000GoeOoEom0`0000=So000HoT01F?i00<006?g00000eog00@005oFGmIOeP800004CmL004oG
0002BkL500000c=k000No0037_`00`000SX000020000108J0000004G0`0000@1>0000;D000<0e`06
0001604H0000n0001P5I00<0017L00000P0011hm00<004=O@el014=O0P0000=?K`00Ool0>Woo000j
Ool00`0001404@020006?cl01000God000001F?l00@006?iHoUSn@8000=Sm`03001Om`0000=OeP07
001KeE_E001?e`00BkL010000S=k2`0000@2>P000001=`8000030AL0000000<0]@040000e`3G0=L2
00030?P200030ET00`004M`Ag0034M`40007@el01000Cfm?K`00>goo000kOol00`0001404@090007
Ho`01000HoUSn@001F?g0P000eoF00D005_EFmD005_E01T000030CL0004G008000<0]@040000e`3G
0=L200040?P010000ET1F@00217L20000Tm_00<007ooOol0>Woo000lOol00`0001400007=al00`00
HoaSo005Ho`00`00HoT00005HoL00`00HoH00002GmH01@00FmEKe@00FmD00P0000E7^000@iP003^I
00d000031WX0000000@000050CL0000009D000030;D01P000=L0e`000;H000@0n0030001F@0000PA
g003000oOcmo00@oO`03001?K`0003eoo`00?Goo00<0000A00001cLO00<006?lHo`016?l0P001F?g
00@006?fHoISmP80009Ke@07001Ke@00001?]@00@iP06@0000@0T`0009D000<0]@8000<0]P030000
n03h0080n08000LAg003000oOcmo00@oO`03001?K`0003ioo`00?Goo00<0000A00001cLO00<006?l
Ho`00V?l0P0000=SnP00HoL00f?g00@006?fHoISmP80009Ke@05001Ke@00001?]@0M0000102C0000
U@000`2e00<0002f0;H00P2f00H0003h0?P0n0001I/200054M`00`00?gloO`04?gl00`00Cfl0000n
Ool003ioo`8000Lg7`05001So6?lHo`00004HoX01000HoMSm`000f?f00H005oE001Ke@00FmD20000
14ne000000001Goo3`001Goo0`0000P0T`0009D0002e0000U@00102f00@0003h0?P000<5V`8000<A
g003000oOcmo00@oO`8003moo`00?goo0P001cLO0`001F?j00<006?g00000f?f00<005oEGmD00P00
00EKe@00001?]@0000aoo`L000]oo`80000309<0002E00800080U@030000]P2f0080]P8000H5V`<0
00HoO`<003moo`00?goo2P001f?j0P000f?f00@005oEGmD0009Ke0800003CkD007oo01moo`800005
09<0000007@0000209D00`000;H0]P020;H200071I/90010Ool0041oo`03000_7blO00D_7`03001S
nV?j00ASnP03001On@00009SmP04001OeEoE0002Fm@4000SOol20000101C0000M0000P2E00L0002f
0;H0]P000ET000061I/00`00>elkG`04>el00`00Oomoo`0oOol0045oo`03000_7blO00@_7`03001S
nV?j009SnP8000=On@80009Oe@03001Ke5_D00<002Ioo`80000405<0001d000209D00`000;H00003
0ET200031I/00`00>elkG`04>el20011Ool0045oo`8000H_7`04001SnP000004GoT20002GmD00`00
FmAKe003000XOol20000101C0000M0000P2E0P001@5I00@000FK1I/000HkG`03001oogoo041oo`00
@Woo00<002lO;al012lO0P001Eoi00D005_h001Oe@00009Ke0<002Yoo`80000805<0001d0000U@00
0AP000D1F@030005V`0000HkG`03001oogoo041oo`00@goo1`0015oi0P000U_h0`0000AKe0000000
02aoo`80000305<0001d00800081608000@1F@P004=oo`00@goo00<002;n8_h00b;n00D005oiGoUO
n@0000=Kn003001?]Tnf00@002ioo`D000030?H0004H008160050001F@5I0ET00005;eh00`00Oomo
o`12Ool004Aoo`03000RoR;n00<RoP03001On@0000=Kn003001?]Tnf00<0039oo`8000040;@0003f
00030AP00`000ET00005;eh00`00Oomoo`13Ool004Eoo`L000=Kn003001?]Tnf008003Ioo`<00003
0?H0004H008160L004Ioo`00AWoo00<001:k4[/00a:k00<005_h00000Tnf0P00>Goo0P0000@0mP00
0AP000DS6`03001oogoo04Eoo`00Agoo200000=?]P00Ool0>goo0P0000<0mP0000001@00B7oo001=
Ool3000oOol3001=Ool00=moo`00ggoo003OOol00=moo`00\
\>"],
  ImageRangeCache->{{{0, 222.375}, {86, 0}} -> {-0.00355512, 0.226649, 
  0.00452888, 0.00452888}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ContourPlot[func[x, \ y], \ {x, \ \(-2\), \ 2}, \ {y, \ \(-2\), \ 2}, 
      \nContours\  -> \ 40, \ ContourLines\  -> \ False, \n
      ColorFunction\  -> \ Hue, \ Frame\  -> \ False]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% ContourGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.240385 0.5 0.240385 [
[ 0 0 0 0 ]
[ 1 1 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1 L
0 1 L
closepath
clip
newpath
0 1 .7 r
.01923 .98077 m
.98077 .98077 L
.98077 .01923 L
.01923 .01923 L
F
0 1 .55 r
.01923 .12433 m
.06956 .15659 L
.07162 .22527 L
.07201 .29396 L
.07216 .36264 L
.07223 .43132 L
.07225 .5 L
.07223 .56868 L
.07216 .63736 L
.07201 .70604 L
.07162 .77473 L
.06956 .84341 L
.08791 .88483 L
.15659 .87923 L
.22527 .8791 L
.29396 .87977 L
.36264 .88214 L
.43132 .89548 L
.45067 .91209 L
.47128 .98077 L
.01923 .98077 L
F
0 1 .7 r
.01923 .87567 m
.07583 .91209 L
.07377 .98077 L
.01923 .98077 L
F
0 1 .4 r
.01923 .86165 m
.04739 .84341 L
.0639 .77473 L
.0671 .70604 L
.06833 .63736 L
.06887 .56868 L
.06902 .5 L
.06887 .43132 L
.06833 .36264 L
.0671 .29396 L
.0639 .22527 L
.04739 .15659 L
.01923 .13835 L
F
0 1 .85 r
.01923 .11032 m
.05357 .08791 L
.06603 .01923 L
.01923 .01923 L
F
.01923 .88968 m
.05357 .91209 L
.06603 .98077 L
.01923 .98077 L
F
0 1 .25 r
.01923 .84758 m
.02567 .84341 L
.05632 .77473 L
.06224 .70604 L
.06454 .63736 L
.06553 .56868 L
.06582 .5 L
.06553 .43132 L
.06454 .36264 L
.06224 .29396 L
.05632 .22527 L
.02567 .15659 L
.01923 .15242 L
F
0 1 .1 r
.01923 .83345 m
.04882 .77473 L
.05744 .70604 L
.06078 .63736 L
.06223 .56868 L
.06265 .5 L
.06223 .43132 L
.06078 .36264 L
.05744 .29396 L
.04882 .22527 L
.01923 .16655 L
F
.05 1 0 r
.01923 .81917 m
.04139 .77473 L
.05267 .70604 L
.05704 .63736 L
.05894 .56868 L
.05949 .5 L
.05894 .43132 L
.05704 .36264 L
.05267 .29396 L
.04139 .22527 L
.01923 .18083 L
F
0 1 1 r
.01923 .09627 m
.03199 .08791 L
.05843 .01923 L
.01923 .01923 L
F
.01923 .90373 m
.03199 .91209 L
.05843 .98077 L
.01923 .98077 L
F
.2 1 0 r
.01923 .80463 m
.03398 .77473 L
.04793 .70604 L
.05333 .63736 L
.05568 .56868 L
.05636 .5 L
.05568 .43132 L
.05333 .36264 L
.04793 .29396 L
.03398 .22527 L
.01923 .19537 L
F
.35 1 0 r
.01923 .78976 m
.02656 .77473 L
.04322 .70604 L
.04964 .63736 L
.05243 .56868 L
.05324 .5 L
.05243 .43132 L
.04964 .36264 L
.04322 .29396 L
.02656 .22527 L
.01923 .21024 L
F
0 .85 1 r
.01923 .08212 m
.05094 .01923 L
.01923 .01923 L
F
.01923 .91788 m
.05094 .98077 L
.01923 .98077 L
F
.5 1 0 r
.01923 .77449 m
.03852 .70604 L
.04596 .63736 L
.0492 .56868 L
.05013 .5 L
.0492 .43132 L
.04596 .36264 L
.03852 .29396 L
.01923 .22551 L
F
.65 1 0 r
.01923 .75912 m
.03382 .70604 L
.0423 .63736 L
.04598 .56868 L
.04704 .5 L
.04598 .43132 L
.0423 .36264 L
.03382 .29396 L
.01923 .24088 L
F
.8 1 0 r
.01923 .74303 m
.02912 .70604 L
.03864 .63736 L
.04276 .56868 L
.04395 .5 L
.04276 .43132 L
.03864 .36264 L
.02912 .29396 L
.01923 .25697 L
F
0 .7 1 r
.01923 .06782 m
.04351 .01923 L
.01923 .01923 L
F
.01923 .93218 m
.04351 .98077 L
.01923 .98077 L
F
.95 1 0 r
.01923 .72601 m
.02441 .70604 L
.03498 .63736 L
.03955 .56868 L
.04087 .5 L
.03955 .43132 L
.03498 .36264 L
.02441 .29396 L
.01923 .27399 L
F
1 .9 0 r
.01923 .70786 m
.01969 .70604 L
.03133 .63736 L
.03635 .56868 L
.0378 .5 L
.03635 .43132 L
.03133 .36264 L
.01969 .29396 L
.01923 .29214 L
F
0 .55 1 r
.01923 .05331 m
.03611 .01923 L
.01923 .01923 L
F
.01923 .94669 m
.03611 .98077 L
.01923 .98077 L
F
1 .75 0 r
.01923 .68991 m
.02767 .63736 L
.03315 .56868 L
.03472 .5 L
.03315 .43132 L
.02767 .36264 L
.01923 .31009 L
F
1 .6 0 r
.01923 .66953 m
.024 .63736 L
.02994 .56868 L
.03165 .5 L
.02994 .43132 L
.024 .36264 L
.01923 .33047 L
F
0 .4 1 r
.01923 .03856 m
.02872 .01923 L
.01923 .01923 L
F
.01923 .96144 m
.02872 .98077 L
.01923 .98077 L
F
1 .45 0 r
.01923 .64542 m
.02033 .63736 L
.02673 .56868 L
.02857 .5 L
.02673 .43132 L
.02033 .36264 L
.01923 .35458 L
F
1 .3 0 r
.01923 .62898 m
.02352 .56868 L
.02549 .5 L
.02352 .43132 L
.01923 .37102 L
F
1 .15 0 r
.01923 .59101 m
.02029 .56868 L
.0224 .5 L
.02029 .43132 L
.01923 .40899 L
F
0 .25 1 r
.01923 .0235 m
.02131 .01923 L
.01923 .01923 L
F
.01923 .9765 m
.02131 .98077 L
.01923 .98077 L
F
1 0 0 r
.01923 .51226 m
.0193 .5 L
.01923 .48774 L
F
0 1 .55 r
.47128 .01923 m
.45067 .08791 L
.43132 .10452 L
.36264 .11786 L
.29396 .12023 L
.22527 .1209 L
.15659 .12077 L
.08791 .11517 L
.07583 .08791 L
.07377 .01923 L
F
0 1 .85 r
.15659 .13266 m
.22527 .13188 L
.29396 .13605 L
.36264 .15098 L
.36692 .15659 L
.4249 .22527 L
.43132 .23882 L
.44213 .29396 L
.44902 .36264 L
.45231 .43132 L
.45329 .5 L
.45231 .56868 L
.44902 .63736 L
.44213 .70604 L
.43132 .76118 L
.4249 .77473 L
.36692 .84341 L
.36264 .84902 L
.29396 .86395 L
.22527 .86812 L
.15659 .86734 L
.09319 .84341 L
.08791 .83199 L
.07951 .77473 L
.077 .70604 L
.07604 .63736 L
.07563 .56868 L
.07551 .5 L
.07563 .43132 L
.07604 .36264 L
.077 .29396 L
.07951 .22527 L
.08791 .16801 L
.09319 .15659 L
F
0 1 1 r
.15659 .14458 m
.22527 .14288 L
.29396 .15193 L
.31098 .15659 L
.36264 .18454 L
.37982 .22527 L
.40639 .29396 L
.4213 .36264 L
.42869 .43132 L
.43093 .5 L
.42869 .56868 L
.4213 .63736 L
.40639 .70604 L
.37982 .77473 L
.36264 .81546 L
.31098 .84341 L
.29396 .84807 L
.22527 .85712 L
.15659 .85542 L
.1201 .84341 L
.08791 .77664 L
.08764 .77473 L
.08207 .70604 L
.07997 .63736 L
.07906 .56868 L
.0788 .5 L
.07906 .43132 L
.07997 .36264 L
.08207 .29396 L
.08764 .22527 L
.08791 .22336 L
.1201 .15659 L
F
0 1 .4 r
.40882 .01923 m
.36264 .08466 L
.35588 .08791 L
.29396 .1044 L
.22527 .10991 L
.15659 .10888 L
.09988 .08791 L
.08791 .06188 L
.0817 .01923 L
F
.40882 .98077 m
.36264 .91534 L
.35588 .91209 L
.29396 .8956 L
.22527 .89009 L
.15659 .89112 L
.09988 .91209 L
.08791 .93812 L
.0817 .98077 L
F
0 .85 1 r
.15659 .15656 m
.22527 .15393 L
.24862 .15659 L
.29396 .16788 L
.36264 .2196 L
.36365 .22527 L
.38118 .29396 L
.39552 .36264 L
.4035 .43132 L
.40602 .5 L
.4035 .56868 L
.39552 .63736 L
.38118 .70604 L
.36365 .77473 L
.36264 .7804 L
.29396 .83212 L
.24862 .84341 L
.22527 .84607 L
.15659 .84344 L
.15644 .84341 L
.09607 .77473 L
.08791 .71495 L
.08724 .70604 L
.08395 .63736 L
.08254 .56868 L
.08213 .5 L
.08254 .43132 L
.08395 .36264 L
.08724 .29396 L
.08791 .28505 L
.09607 .22527 L
.15644 .15659 L
F
0 .7 1 r
.15659 .16855 m
.22527 .16501 L
.29396 .18405 L
.34325 .22527 L
.36264 .25657 L
.36792 .29396 L
.37864 .36264 L
.3857 .43132 L
.38806 .5 L
.3857 .56868 L
.37864 .63736 L
.36792 .70604 L
.36264 .74343 L
.34325 .77473 L
.29396 .81595 L
.22527 .83499 L
.15659 .83145 L
.10489 .77473 L
.09254 .70604 L
.088 .63736 L
.08791 .63689 L
.08606 .56868 L
.0855 .5 L
.08606 .43132 L
.08791 .36311 L
.088 .36264 L
.09254 .29396 L
.10489 .22527 L
F
0 .55 1 r
.15659 .18069 m
.22527 .17617 L
.29396 .20056 L
.32263 .22527 L
.36107 .29396 L
.36264 .29806 L
.36863 .36264 L
.37392 .43132 L
.37583 .5 L
.37392 .56868 L
.36863 .63736 L
.36264 .70194 L
.36107 .70604 L
.32263 .77473 L
.29396 .79944 L
.22527 .82383 L
.15659 .81931 L
.11425 .77473 L
.09797 .70604 L
.09212 .63736 L
.08964 .56868 L
.08893 .5 L
.08964 .43132 L
.09212 .36264 L
.09797 .29396 L
.11425 .22527 L
F
0 1 .25 r
.37333 .01923 m
.36264 .05067 L
.29609 .08791 L
.29396 .08849 L
.22527 .09889 L
.15659 .09695 L
.128 .08791 L
.08987 .01923 L
F
.37333 .98077 m
.36264 .94933 L
.29609 .91209 L
.29396 .91151 L
.22527 .90111 L
.15659 .90305 L
.128 .91209 L
.08987 .98077 L
F
0 .4 1 r
.15659 .19301 m
.22527 .18747 L
.29396 .21753 L
.30291 .22527 L
.34644 .29396 L
.36264 .34676 L
.36343 .36264 L
.36679 .43132 L
.36814 .5 L
.36679 .56868 L
.36343 .63736 L
.36264 .65324 L
.34644 .70604 L
.30291 .77473 L
.29396 .78247 L
.22527 .81253 L
.15659 .80699 L
.12434 .77473 L
.10357 .70604 L
.09632 .63736 L
.09327 .56868 L
.0924 .5 L
.09327 .43132 L
.09632 .36264 L
.10357 .29396 L
.12434 .22527 L
F
0 .25 1 r
.15659 .20556 m
.22527 .19896 L
.28299 .22527 L
.29396 .23481 L
.33297 .29396 L
.35391 .36264 L
.36264 .41229 L
.36294 .43132 L
.36376 .5 L
.36294 .56868 L
.36264 .58771 L
.35391 .63736 L
.33297 .70604 L
.29396 .76519 L
.28299 .77473 L
.22527 .80104 L
.15659 .79444 L
.13552 .77473 L
.10938 .70604 L
.10063 .63736 L
.09698 .56868 L
.09594 .5 L
.09698 .43132 L
.10063 .36264 L
.10938 .29396 L
.13552 .22527 L
F
0 1 .1 r
.35996 .01923 m
.29396 .07242 L
.22527 .08783 L
.15659 .08495 L
.09837 .01923 L
F
.35996 .98077 m
.29396 .92758 L
.22527 .91217 L
.15659 .91505 L
.09837 .98077 L
F
0 .1 1 r
.15659 .21838 m
.22527 .21065 L
.26154 .22527 L
.29396 .25269 L
.32021 .29396 L
.34289 .36264 L
.35356 .43132 L
.35677 .5 L
.35356 .56868 L
.34289 .63736 L
.32021 .70604 L
.29396 .74731 L
.26154 .77473 L
.22527 .78935 L
.15659 .78162 L
.1485 .77473 L
.11543 .70604 L
.10504 .63736 L
.10076 .56868 L
.09955 .5 L
.10076 .43132 L
.10504 .36264 L
.11543 .29396 L
.1485 .22527 L
F
.05 0 1 r
.22527 .22261 m
.23395 .22527 L
.29396 .27168 L
.30773 .29396 L
.33251 .36264 L
.34389 .43132 L
.3473 .5 L
.34389 .56868 L
.33251 .63736 L
.30773 .70604 L
.29396 .72832 L
.23395 .77473 L
.22527 .77739 L
.16857 .77473 L
.15659 .76864 L
.12179 .70604 L
.10958 .63736 L
.10463 .56868 L
.10323 .5 L
.10463 .43132 L
.10958 .36264 L
.12179 .29396 L
.15659 .23136 L
.16857 .22527 L
F
.2 0 1 r
.15659 .24458 m
.22527 .23459 L
.29396 .29208 L
.29509 .29396 L
.32256 .36264 L
.33472 .43132 L
.33833 .5 L
.33472 .56868 L
.32256 .63736 L
.29509 .70604 L
.29396 .70792 L
.22527 .76541 L
.15659 .75542 L
.12854 .70604 L
.11428 .63736 L
.10859 .56868 L
.10699 .5 L
.10859 .43132 L
.11428 .36264 L
.12854 .29396 L
F
.05 1 0 r
.33764 .01923 m
.29396 .05611 L
.22527 .07668 L
.15659 .07285 L
.1073 .01923 L
F
.33764 .98077 m
.29396 .94389 L
.22527 .92332 L
.15659 .92715 L
.1073 .98077 L
F
.35 0 1 r
.15659 .25835 m
.22527 .2469 L
.2825 .29396 L
.29396 .31248 L
.31282 .36264 L
.32591 .43132 L
.32975 .5 L
.32591 .56868 L
.31282 .63736 L
.29396 .68752 L
.2825 .70604 L
.22527 .7531 L
.15659 .74165 L
.13581 .70604 L
.11915 .63736 L
.11267 .56868 L
.11086 .5 L
.11267 .43132 L
.11915 .36264 L
.13581 .29396 L
F
.5 0 1 r
.15659 .2728 m
.22527 .25969 L
.26924 .29396 L
.29396 .33622 L
.3031 .36264 L
.31732 .43132 L
.32143 .5 L
.31732 .56868 L
.3031 .63736 L
.29396 .66378 L
.26924 .70604 L
.22527 .74031 L
.15659 .7272 L
.14378 .70604 L
.12424 .63736 L
.11687 .56868 L
.11483 .5 L
.11687 .43132 L
.12424 .36264 L
.14378 .29396 L
F
.2 1 0 r
.31747 .01923 m
.29396 .03949 L
.22527 .06543 L
.15659 .06062 L
.11679 .01923 L
F
.31747 .98077 m
.29396 .96051 L
.22527 .93457 L
.15659 .93938 L
.11679 .98077 L
F
.65 0 1 r
.15659 .28806 m
.22527 .27308 L
.2545 .29396 L
.29322 .36264 L
.29396 .36316 L
.30881 .43132 L
.31325 .5 L
.30881 .56868 L
.29396 .63684 L
.29322 .63736 L
.2545 .70604 L
.22527 .72692 L
.15659 .71194 L
.15283 .70604 L
.12958 .63736 L
.12123 .56868 L
.11893 .5 L
.12123 .43132 L
.12958 .36264 L
.15283 .29396 L
F
.8 0 1 r
.22527 .28716 m
.23649 .29396 L
.28345 .36264 L
.29396 .38557 L
.30025 .43132 L
.30508 .5 L
.30025 .56868 L
.29396 .61443 L
.28345 .63736 L
.23649 .70604 L
.22527 .71284 L
.16637 .70604 L
.15659 .69673 L
.13525 .63736 L
.12576 .56868 L
.12319 .5 L
.12576 .43132 L
.13525 .36264 L
.15659 .30327 L
.16637 .29396 L
F
.35 1 0 r
.29776 .01923 m
.29396 .02248 L
.22527 .05405 L
.15659 .04822 L
.1271 .01923 L
F
.29776 .98077 m
.29396 .97752 L
.22527 .94595 L
.15659 .95178 L
.1271 .98077 L
F
.95 0 1 r
.15659 .31945 m
.22527 .30124 L
.27327 .36264 L
.29161 .43132 L
.29396 .44931 L
.2968 .5 L
.29396 .55069 L
.29161 .56868 L
.27327 .63736 L
.22527 .69876 L
.15659 .68055 L
.14133 .63736 L
.1305 .56868 L
.12762 .5 L
.1305 .43132 L
.14133 .36264 L
F
1 0 .9 r
.15659 .3379 m
.22527 .31589 L
.26238 .36264 L
.28302 .43132 L
.28857 .5 L
.28302 .56868 L
.26238 .63736 L
.22527 .68411 L
.15659 .6621 L
.14798 .63736 L
.1355 .56868 L
.13225 .5 L
.1355 .43132 L
.14798 .36264 L
F
1 0 .75 r
.15659 .35931 m
.22527 .33235 L
.25025 .36264 L
.2741 .43132 L
.28024 .5 L
.2741 .56868 L
.25025 .63736 L
.22527 .66765 L
.15659 .64069 L
.15544 .63736 L
.14083 .56868 L
.13715 .5 L
.14083 .43132 L
.15544 .36264 L
F
.5 1 0 r
.27778 .01923 m
.22527 .04251 L
.15659 .03563 L
.13862 .01923 L
F
.27778 .98077 m
.22527 .95749 L
.15659 .96437 L
.13862 .98077 L
F
1 0 .6 r
.22527 .35117 m
.23566 .36264 L
.26467 .43132 L
.27156 .5 L
.26467 .56868 L
.23566 .63736 L
.22527 .64883 L
.16708 .63736 L
.15659 .62846 L
.14658 .56868 L
.14237 .5 L
.14658 .43132 L
.15659 .37154 L
.16708 .36264 L
F
1 0 .45 r
.15659 .4019 m
.22527 .36559 L
.2544 .43132 L
.26234 .5 L
.2544 .56868 L
.22527 .63441 L
.15659 .5981 L
.15289 .56868 L
.148 .5 L
.15289 .43132 L
F
.65 1 0 r
.25549 .01923 m
.22527 .0308 L
.15659 .02281 L
.15222 .01923 L
F
.25549 .98077 m
.22527 .9692 L
.15659 .97719 L
.15222 .98077 L
F
1 0 .3 r
.22527 .38283 m
.24268 .43132 L
.25226 .5 L
.24268 .56868 L
.22527 .61717 L
.16118 .56868 L
.15659 .5449 L
.15421 .5 L
.15659 .4551 L
.16118 .43132 L
F
1 0 .15 r
.22527 .42339 m
.22782 .43132 L
.24066 .5 L
.22782 .56868 L
.22527 .57661 L
.17406 .56868 L
.16284 .5 L
.17406 .43132 L
F
1 0 0 r
.22527 .48919 m
.22571 .5 L
.22527 .51081 L
.17601 .5 L
F
0 1 .55 r
.98077 .87567 m
.93044 .84341 L
.92838 .77473 L
.92799 .70604 L
.92784 .63736 L
.92777 .56868 L
.92775 .5 L
.92777 .43132 L
.92784 .36264 L
.92799 .29396 L
.92838 .22527 L
.93044 .15659 L
.91209 .11517 L
.84341 .12077 L
.77473 .1209 L
.70604 .12023 L
.63736 .11786 L
.56868 .10452 L
.54933 .08791 L
.52872 .01923 L
.98077 .01923 L
F
.92623 .98077 m
.92417 .91209 L
.91209 .88483 L
.84341 .87923 L
.77473 .8791 L
.70604 .87977 L
.63736 .88214 L
.56868 .89548 L
.54933 .91209 L
.52872 .98077 L
F
0 1 .85 r
.63736 .15098 m
.70604 .13605 L
.77473 .13188 L
.84341 .13266 L
.90681 .15659 L
.91209 .16801 L
.92049 .22527 L
.923 .29396 L
.92396 .36264 L
.92437 .43132 L
.92449 .5 L
.92437 .56868 L
.92396 .63736 L
.923 .70604 L
.92049 .77473 L
.91209 .83199 L
.90681 .84341 L
.84341 .86734 L
.77473 .86812 L
.70604 .86395 L
.63736 .84902 L
.63308 .84341 L
.5751 .77473 L
.56868 .76118 L
.55787 .70604 L
.55098 .63736 L
.54769 .56868 L
.54671 .5 L
.54769 .43132 L
.55098 .36264 L
.55787 .29396 L
.56868 .23882 L
.5751 .22527 L
.63308 .15659 L
F
0 1 1 r
.70604 .15193 m
.77473 .14288 L
.84341 .14458 L
.8799 .15659 L
.91209 .22336 L
.91236 .22527 L
.91793 .29396 L
.92003 .36264 L
.92094 .43132 L
.9212 .5 L
.92094 .56868 L
.92003 .63736 L
.91793 .70604 L
.91236 .77473 L
.91209 .77664 L
.8799 .84341 L
.84341 .85542 L
.77473 .85712 L
.70604 .84807 L
.68902 .84341 L
.63736 .81546 L
.62018 .77473 L
.59361 .70604 L
.5787 .63736 L
.57131 .56868 L
.56907 .5 L
.57131 .43132 L
.5787 .36264 L
.59361 .29396 L
.62018 .22527 L
.63736 .18454 L
.68902 .15659 L
F
0 1 .4 r
.9183 .01923 m
.91209 .06188 L
.90012 .08791 L
.84341 .10888 L
.77473 .10991 L
.70604 .1044 L
.64412 .08791 L
.63736 .08466 L
.59118 .01923 L
F
.9183 .98077 m
.91209 .93812 L
.90012 .91209 L
.84341 .89112 L
.77473 .89009 L
.70604 .8956 L
.64412 .91209 L
.63736 .91534 L
.59118 .98077 L
F
0 .85 1 r
.77473 .15393 m
.84341 .15656 L
.84356 .15659 L
.90393 .22527 L
.91209 .28505 L
.91276 .29396 L
.91605 .36264 L
.91746 .43132 L
.91787 .5 L
.91746 .56868 L
.91605 .63736 L
.91276 .70604 L
.91209 .71495 L
.90393 .77473 L
.84356 .84341 L
.84341 .84344 L
.77473 .84607 L
.75138 .84341 L
.70604 .83212 L
.63736 .7804 L
.63635 .77473 L
.61882 .70604 L
.60448 .63736 L
.5965 .56868 L
.59398 .5 L
.5965 .43132 L
.60448 .36264 L
.61882 .29396 L
.63635 .22527 L
.63736 .2196 L
.70604 .16788 L
.75138 .15659 L
F
0 .7 1 r
.70604 .18405 m
.77473 .16501 L
.84341 .16855 L
.89511 .22527 L
.90746 .29396 L
.912 .36264 L
.91209 .36311 L
.91394 .43132 L
.9145 .5 L
.91394 .56868 L
.91209 .63689 L
.912 .63736 L
.90746 .70604 L
.89511 .77473 L
.84341 .83145 L
.77473 .83499 L
.70604 .81595 L
.65675 .77473 L
.63736 .74343 L
.63208 .70604 L
.62136 .63736 L
.6143 .56868 L
.61194 .5 L
.6143 .43132 L
.62136 .36264 L
.63208 .29396 L
.63736 .25657 L
.65675 .22527 L
F
0 .55 1 r
.70604 .20056 m
.77473 .17617 L
.84341 .18069 L
.88575 .22527 L
.90203 .29396 L
.90788 .36264 L
.91036 .43132 L
.91107 .5 L
.91036 .56868 L
.90788 .63736 L
.90203 .70604 L
.88575 .77473 L
.84341 .81931 L
.77473 .82383 L
.70604 .79944 L
.67737 .77473 L
.63893 .70604 L
.63736 .70194 L
.63137 .63736 L
.62608 .56868 L
.62417 .5 L
.62608 .43132 L
.63137 .36264 L
.63736 .29806 L
.63893 .29396 L
.67737 .22527 L
F
0 1 .25 r
.91013 .01923 m
.872 .08791 L
.84341 .09695 L
.77473 .09889 L
.70604 .08849 L
.70391 .08791 L
.63736 .05067 L
.62667 .01923 L
F
.91013 .98077 m
.872 .91209 L
.84341 .90305 L
.77473 .90111 L
.70604 .91151 L
.70391 .91209 L
.63736 .94933 L
.62667 .98077 L
F
0 .4 1 r
.70604 .21753 m
.77473 .18747 L
.84341 .19301 L
.87566 .22527 L
.89643 .29396 L
.90368 .36264 L
.90673 .43132 L
.9076 .5 L
.90673 .56868 L
.90368 .63736 L
.89643 .70604 L
.87566 .77473 L
.84341 .80699 L
.77473 .81253 L
.70604 .78247 L
.69709 .77473 L
.65356 .70604 L
.63736 .65324 L
.63657 .63736 L
.63321 .56868 L
.63186 .5 L
.63321 .43132 L
.63657 .36264 L
.63736 .34676 L
.65356 .29396 L
.69709 .22527 L
F
0 .25 1 r
.77473 .19896 m
.84341 .20556 L
.86448 .22527 L
.89062 .29396 L
.89937 .36264 L
.90302 .43132 L
.90406 .5 L
.90302 .56868 L
.89937 .63736 L
.89062 .70604 L
.86448 .77473 L
.84341 .79444 L
.77473 .80104 L
.71701 .77473 L
.70604 .76519 L
.66703 .70604 L
.64609 .63736 L
.63736 .58771 L
.63706 .56868 L
.63624 .5 L
.63706 .43132 L
.63736 .41229 L
.64609 .36264 L
.66703 .29396 L
.70604 .23481 L
.71701 .22527 L
F
0 1 .1 r
.90163 .01923 m
.84341 .08495 L
.77473 .08783 L
.70604 .07242 L
.64004 .01923 L
F
.90163 .98077 m
.84341 .91505 L
.77473 .91217 L
.70604 .92758 L
.64004 .98077 L
F
0 .1 1 r
.77473 .21065 m
.84341 .21838 L
.8515 .22527 L
.88457 .29396 L
.89496 .36264 L
.89924 .43132 L
.90045 .5 L
.89924 .56868 L
.89496 .63736 L
.88457 .70604 L
.8515 .77473 L
.84341 .78162 L
.77473 .78935 L
.73846 .77473 L
.70604 .74731 L
.67979 .70604 L
.65711 .63736 L
.64644 .56868 L
.64323 .5 L
.64644 .43132 L
.65711 .36264 L
.67979 .29396 L
.70604 .25269 L
.73846 .22527 L
F
.05 0 1 r
.77473 .22261 m
.83143 .22527 L
.84341 .23136 L
.87821 .29396 L
.89042 .36264 L
.89537 .43132 L
.89677 .5 L
.89537 .56868 L
.89042 .63736 L
.87821 .70604 L
.84341 .76864 L
.83143 .77473 L
.77473 .77739 L
.76605 .77473 L
.70604 .72832 L
.69227 .70604 L
.66749 .63736 L
.65611 .56868 L
.6527 .5 L
.65611 .43132 L
.66749 .36264 L
.69227 .29396 L
.70604 .27168 L
.76605 .22527 L
F
.2 0 1 r
.70604 .29208 m
.77473 .23459 L
.84341 .24458 L
.87146 .29396 L
.88572 .36264 L
.89141 .43132 L
.89301 .5 L
.89141 .56868 L
.88572 .63736 L
.87146 .70604 L
.84341 .75542 L
.77473 .76541 L
.70604 .70792 L
.70491 .70604 L
.67744 .63736 L
.66528 .56868 L
.66167 .5 L
.66528 .43132 L
.67744 .36264 L
.70491 .29396 L
F
.05 1 0 r
.8927 .01923 m
.84341 .07285 L
.77473 .07668 L
.70604 .05611 L
.66236 .01923 L
F
.8927 .98077 m
.84341 .92715 L
.77473 .92332 L
.70604 .94389 L
.66236 .98077 L
F
.35 0 1 r
.77473 .2469 m
.84341 .25835 L
.86419 .29396 L
.88085 .36264 L
.88733 .43132 L
.88914 .5 L
.88733 .56868 L
.88085 .63736 L
.86419 .70604 L
.84341 .74165 L
.77473 .7531 L
.7175 .70604 L
.70604 .68752 L
.68718 .63736 L
.67409 .56868 L
.67025 .5 L
.67409 .43132 L
.68718 .36264 L
.70604 .31248 L
.7175 .29396 L
F
.5 0 1 r
.77473 .25969 m
.84341 .2728 L
.85622 .29396 L
.87576 .36264 L
.88313 .43132 L
.88517 .5 L
.88313 .56868 L
.87576 .63736 L
.85622 .70604 L
.84341 .7272 L
.77473 .74031 L
.73076 .70604 L
.70604 .66378 L
.6969 .63736 L
.68268 .56868 L
.67857 .5 L
.68268 .43132 L
.6969 .36264 L
.70604 .33622 L
.73076 .29396 L
F
.2 1 0 r
.88321 .01923 m
.84341 .06062 L
.77473 .06543 L
.70604 .03949 L
.68253 .01923 L
F
.88321 .98077 m
.84341 .93938 L
.77473 .93457 L
.70604 .96051 L
.68253 .98077 L
F
.65 0 1 r
.77473 .27308 m
.84341 .28806 L
.84717 .29396 L
.87042 .36264 L
.87877 .43132 L
.88107 .5 L
.87877 .56868 L
.87042 .63736 L
.84717 .70604 L
.84341 .71194 L
.77473 .72692 L
.7455 .70604 L
.70678 .63736 L
.70604 .63684 L
.69119 .56868 L
.68675 .5 L
.69119 .43132 L
.70604 .36316 L
.70678 .36264 L
.7455 .29396 L
F
.8 0 1 r
.77473 .28716 m
.83363 .29396 L
.84341 .30327 L
.86475 .36264 L
.87424 .43132 L
.87681 .5 L
.87424 .56868 L
.86475 .63736 L
.84341 .69673 L
.83363 .70604 L
.77473 .71284 L
.76351 .70604 L
.71655 .63736 L
.70604 .61443 L
.69975 .56868 L
.69492 .5 L
.69975 .43132 L
.70604 .38557 L
.71655 .36264 L
.76351 .29396 L
F
.35 1 0 r
.8729 .01923 m
.84341 .04822 L
.77473 .05405 L
.70604 .02248 L
.70224 .01923 L
F
.8729 .98077 m
.84341 .95178 L
.77473 .94595 L
.70604 .97752 L
.70224 .98077 L
F
.95 0 1 r
.77473 .30124 m
.84341 .31945 L
.85867 .36264 L
.8695 .43132 L
.87238 .5 L
.8695 .56868 L
.85867 .63736 L
.84341 .68055 L
.77473 .69876 L
.72673 .63736 L
.70839 .56868 L
.70604 .55069 L
.7032 .5 L
.70604 .44931 L
.70839 .43132 L
.72673 .36264 L
F
1 0 .9 r
.77473 .31589 m
.84341 .3379 L
.85202 .36264 L
.8645 .43132 L
.86775 .5 L
.8645 .56868 L
.85202 .63736 L
.84341 .6621 L
.77473 .68411 L
.73762 .63736 L
.71698 .56868 L
.71143 .5 L
.71698 .43132 L
.73762 .36264 L
F
1 0 .75 r
.77473 .33235 m
.84341 .35931 L
.84456 .36264 L
.85917 .43132 L
.86285 .5 L
.85917 .56868 L
.84456 .63736 L
.84341 .64069 L
.77473 .66765 L
.74975 .63736 L
.7259 .56868 L
.71976 .5 L
.7259 .43132 L
.74975 .36264 L
F
.5 1 0 r
.86138 .01923 m
.84341 .03563 L
.77473 .04251 L
.72222 .01923 L
F
.86138 .98077 m
.84341 .96437 L
.77473 .95749 L
.72222 .98077 L
F
1 0 .6 r
.77473 .35117 m
.83292 .36264 L
.84341 .37154 L
.85342 .43132 L
.85763 .5 L
.85342 .56868 L
.84341 .62846 L
.83292 .63736 L
.77473 .64883 L
.76434 .63736 L
.73533 .56868 L
.72844 .5 L
.73533 .43132 L
.76434 .36264 L
F
1 0 .45 r
.77473 .36559 m
.84341 .4019 L
.84711 .43132 L
.852 .5 L
.84711 .56868 L
.84341 .5981 L
.77473 .63441 L
.7456 .56868 L
.73766 .5 L
.7456 .43132 L
F
.65 1 0 r
.84778 .01923 m
.84341 .02281 L
.77473 .0308 L
.74451 .01923 L
F
.84778 .98077 m
.84341 .97719 L
.77473 .9692 L
.74451 .98077 L
F
1 0 .3 r
.77473 .38283 m
.83882 .43132 L
.84341 .4551 L
.84579 .5 L
.84341 .5449 L
.83882 .56868 L
.77473 .61717 L
.75732 .56868 L
.74774 .5 L
.75732 .43132 L
F
1 0 .15 r
.77473 .42339 m
.82594 .43132 L
.83716 .5 L
.82594 .56868 L
.77473 .57661 L
.77218 .56868 L
.75934 .5 L
.77218 .43132 L
F
1 0 0 r
.77473 .48919 m
.82399 .5 L
.77473 .51081 L
.77429 .5 L
F
0 1 .7 r
.98077 .12433 m
.92417 .08791 L
.92623 .01923 L
.98077 .01923 L
F
0 1 .4 r
.98077 .86165 m
.95261 .84341 L
.9361 .77473 L
.9329 .70604 L
.93167 .63736 L
.93113 .56868 L
.93098 .5 L
.93113 .43132 L
.93167 .36264 L
.9329 .29396 L
.9361 .22527 L
.95261 .15659 L
.98077 .13835 L
F
0 1 .85 r
.98077 .11032 m
.94643 .08791 L
.93397 .01923 L
.98077 .01923 L
F
.98077 .88968 m
.94643 .91209 L
.93397 .98077 L
.98077 .98077 L
F
0 1 .25 r
.98077 .84758 m
.97433 .84341 L
.94368 .77473 L
.93776 .70604 L
.93546 .63736 L
.93447 .56868 L
.93418 .5 L
.93447 .43132 L
.93546 .36264 L
.93776 .29396 L
.94368 .22527 L
.97433 .15659 L
.98077 .15242 L
F
0 1 .1 r
.98077 .83345 m
.95118 .77473 L
.94256 .70604 L
.93922 .63736 L
.93777 .56868 L
.93735 .5 L
.93777 .43132 L
.93922 .36264 L
.94256 .29396 L
.95118 .22527 L
.98077 .16655 L
F
.05 1 0 r
.98077 .81917 m
.95861 .77473 L
.94733 .70604 L
.94296 .63736 L
.94106 .56868 L
.94051 .5 L
.94106 .43132 L
.94296 .36264 L
.94733 .29396 L
.95861 .22527 L
.98077 .18083 L
F
0 1 1 r
.98077 .09627 m
.96801 .08791 L
.94157 .01923 L
.98077 .01923 L
F
.98077 .90373 m
.96801 .91209 L
.94157 .98077 L
.98077 .98077 L
F
.2 1 0 r
.98077 .80463 m
.96602 .77473 L
.95207 .70604 L
.94667 .63736 L
.94432 .56868 L
.94364 .5 L
.94432 .43132 L
.94667 .36264 L
.95207 .29396 L
.96602 .22527 L
.98077 .19537 L
F
.35 1 0 r
.98077 .78976 m
.97344 .77473 L
.95678 .70604 L
.95036 .63736 L
.94757 .56868 L
.94676 .5 L
.94757 .43132 L
.95036 .36264 L
.95678 .29396 L
.97344 .22527 L
.98077 .21024 L
F
0 .85 1 r
.98077 .08212 m
.94906 .01923 L
.98077 .01923 L
F
.98077 .91788 m
.94906 .98077 L
.98077 .98077 L
F
.5 1 0 r
.98077 .77449 m
.96148 .70604 L
.95404 .63736 L
.9508 .56868 L
.94987 .5 L
.9508 .43132 L
.95404 .36264 L
.96148 .29396 L
.98077 .22551 L
F
.65 1 0 r
.98077 .75912 m
.96618 .70604 L
.9577 .63736 L
.95402 .56868 L
.95296 .5 L
.95402 .43132 L
.9577 .36264 L
.96618 .29396 L
.98077 .24088 L
F
.8 1 0 r
.98077 .74303 m
.97088 .70604 L
.96136 .63736 L
.95724 .56868 L
.95605 .5 L
.95724 .43132 L
.96136 .36264 L
.97088 .29396 L
.98077 .25697 L
F
0 .7 1 r
.98077 .06782 m
.95649 .01923 L
.98077 .01923 L
F
.98077 .93218 m
.95649 .98077 L
.98077 .98077 L
F
.95 1 0 r
.98077 .72601 m
.97559 .70604 L
.96502 .63736 L
.96045 .56868 L
.95913 .5 L
.96045 .43132 L
.96502 .36264 L
.97559 .29396 L
.98077 .27399 L
F
1 .9 0 r
.98077 .70786 m
.98031 .70604 L
.96867 .63736 L
.96365 .56868 L
.9622 .5 L
.96365 .43132 L
.96867 .36264 L
.98031 .29396 L
.98077 .29214 L
F
0 .55 1 r
.98077 .05331 m
.96389 .01923 L
.98077 .01923 L
F
.98077 .94669 m
.96389 .98077 L
.98077 .98077 L
F
1 .75 0 r
.98077 .68991 m
.97233 .63736 L
.96685 .56868 L
.96528 .5 L
.96685 .43132 L
.97233 .36264 L
.98077 .31009 L
F
1 .6 0 r
.98077 .66953 m
.976 .63736 L
.97006 .56868 L
.96835 .5 L
.97006 .43132 L
.976 .36264 L
.98077 .33047 L
F
0 .4 1 r
.98077 .03856 m
.97128 .01923 L
.98077 .01923 L
F
.98077 .96144 m
.97128 .98077 L
.98077 .98077 L
F
1 .45 0 r
.98077 .64542 m
.97967 .63736 L
.97327 .56868 L
.97143 .5 L
.97327 .43132 L
.97967 .36264 L
.98077 .35458 L
F
1 .3 0 r
.98077 .62898 m
.97648 .56868 L
.97451 .5 L
.97648 .43132 L
.98077 .37102 L
F
1 .15 0 r
.98077 .59101 m
.97971 .56868 L
.9776 .5 L
.97971 .43132 L
.98077 .40899 L
F
0 .25 1 r
.98077 .0235 m
.97869 .01923 L
.98077 .01923 L
F
.98077 .9765 m
.97869 .98077 L
.98077 .98077 L
F
1 0 0 r
.98077 .51226 m
.9807 .5 L
.98077 .48774 L
F
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{149.438, 149.438},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002E0000UA000`40O003h00OUGoo002EOol009Eo
o`00UGoo0002Ool0203o0Il2?`;O0gl3o`?k0oH20o4010?/0nL3h`?S0POP0Q_P0RoP1CoP2E?P13oP
0RoP11_P0`OP0`?S0`?W1@?/2@?a2@?f2@?a1P?/10?W0`?S0POP11_P0boP0coP2e?P13oP0RoP0Q_P
00X7h0?S0nL3i`?/0o43mP?k0ol3O`82g`030Sl1Wgoo00=oo`000Woo0P6O00H2?`;O0gl3o`?k0oH2
0o420n`00`?W0n<3h`021n026n02;n09?n02Dn03?n03;n056n031n030n<40nL50n`90o490oH:0o45
0n`50nL30n<21n056n04;n03?n03Dn08?n02;n026n021n001P?S0nL3i`?/0o43mP83n`050ol3O`;O
0]l2?`04Ool0009oo`050Il2?`;O0gl3o`020o/00`?f0o43l@030n`00`?W0n<3h`021n026n07;n05
?n04;n066n021n040n<40nL50n`:0o4:0oH:0o450n`50nL30n<21n076n04;n05?n07;n026n021n00
0`?S0nL3i`020n`01P?a0oH3mP?k0ol3O`82g`030Smoogoo009oo`000Woo00H2?`;O0]l3O`?o0o/2
0oH30o420n`20nL20n<21n026n0=;n066n031n040n<50nL40n`:0o4;0oH:0o450n`60nL30n<31n07
6n0=;n026n021n000`?S0nL3i`030n`010?a0oH3mP?k0P?o00<3O`;O0Sl017oo0002Ool00`8o0]l3
O`020ol00`?k0oH3mP030o430n`20nL20n<21n076n05;n056n051n030n<70nL40n`:0o4;0oH;0o44
0n`70nL30n<51n066n05;n076n021n0010?S0nL3i`?W0P?/0P?a0P?f0P?k0P?o00<3O`;OOol00goo
0002Ool010;O0gl3o`?o0P?k0`?f0P?a0`?/0`?W0P?S0POP3Q_P10OP1@?S1P?W1@?/2`?a30?f2`?a
1@?/1P?W1@?S1@OP3Q_P0POP00@3h`?W0nL3i`<3k083l@83mP83n`83o`030gl2ggoo00=oo`000Woo
00@2g`=o0ol3o`83n`<3mP83l@@3k0<3i`83h`L7h0DKh0@7h0L3h`D3i`H3k0/3l@d3mP`3l@H3k0D3
i`P3h`@7h0DKh0L7h083h`<3i`<3k083l@83mP<3n`83o`030gmoogoo009oo`000Woo00<3O`?o0ol0
0`?k0`?f0P?a1@?/0`?W0P?S30OP2@?S10?W1`?/30?a3@?f30?a20?/10?W2P?S30OP0P?S0`?W10?/
0P?a0`?f0P?k0P?o00<3OgooOol00Woo0002Ool00`=o0ol3o`020o/40oH30o440n`40nLC0n<50nL9
0n`<0o4>0oH<0o490n`60nLC0n<30nL40n`30o430oH20o/30ol4Ool0009oo`<3o`83n`@3mP<3l@D3
k0@3i`d3h`P3i`X3k0`3l@l3mPd3l@X3k0T3i`d3h`@3i`@3k0<3l@<3mP<3n`83o`Aoo`000Woo00@3
o`?k0o/3n`D3mPH3l@D3k1<3i`X3k0l3lA03mQ03l@X3k1<3i`D3k0L3l@<3mP<3n`Aoo`000Woo0P?k
20?f20?a2@?/1`?W30?/4P?a4@?f4P?a30?/1`?W2@?/2`?a10?f00<3ngooOol00Woo0002Ool00`?k
0oH3mP070oH;0o4F0n`D0o4C0oHD0o4F0n`?0o440oH4Ool0009oo`/3mS03lA/3mSL3l@83mPAoo`00
0Woo4@?f9@?a9@?f9@?a1@?f2@?a00<3mWooOol00Woo0002Oon60oH90o44Ool0009oo`83lH@3mPT3
l@Aoo`000Woo10?a3`?f6P?k=`?f6P?k2@?f20?a17oo0002Ool20n`40o4:0oH:0o/90ol?0o/]0oH?
0o/90ol:0o/60oH70o400`?/Oomoo`02Ool0009oo`D3k0<3l@D3mPD3naH3o``3nbP3mPd3naH3o`D3
n`@3mP@3l@<3k0Aoo`000Woo00<3i`?/0n`00P?/0`?a0`?f10?k1P?o40=o2@?o1P?k9`?f20?k20?o
40=o1@?o10?k10?f0`?a0`?/00<3igooOol00Woo0002Ool20nL30n`30o430oH40o/60olC0gl80ol5
0o/V0oH60o/70olD0gl50ol40o/30oH30o430n`00`?WOomoo`02Ool0009oo`83i`@3k083l@<3mP<3
n`H3o`030gl2g`;O00`2g`P3O`H3o`D3nb@3mPD3n`H3o`T3O`d2g`83O`@3o`@3n`<3mP<3l@83k083
i`Aoo`000Woo00<3h`?W0nL00`?/0P?a0`?f0`?k1@?o00<3O`;O0]l010;O1`8o1`;O10=o1`?o10?k
8P?f10?k1`?o1@=o1`;O1`8o1@;O0P=o10?o0`?k0`?f0P?a0`?/0P?W17oo0002Ool20n<010?W0n`3
k0?/0P?a0`?f0`?k10?o00<3O`;O0Sl03`8o1`;O0P=o1P?o1@?k80?f10?k1`?o0P=o20;O3`8o00<2
g`=o0gl00`?o0`?k0`?f0P?a0`?/00<3i`?SOol00goo0002Ool010OP0n<3i`?W0P?/0P?a0P?f0`?k
10?o00<3O`;O0Sl01P8o1`6O1@8o1P;O0P=o1P?o1@?k7P?f1@?k1P?o0P=o1P;O1P8o1`6O1P8o00<2
g`=o0gl00`?o0P?k0`?f0P?a0P?/0P?W00<3hgooOol00Woo0002Ool21n0010?S0nL3k0?/0P?a0P?f
0`?k0`?o00@3O`;O0Sl2?a01W`D2?`@2g`<3O`@3o`H3nad3mP@3n`D3o`<3O`@2g`H2?`l1W`82?`03
0]l3O`=o0083o`<3n`030oH3l@?a00<3k0030nL3h`OP00Aoo`000Woo00<Kh0OP0n<00P?W0P?/00<3
l@?f0oH00P?k0`?o00@3O`;O0Sl2?`L1W`P0o`@1W`D2?`<2g`<3O`@3o`H3na/3mPD3n`@3o`<3O`<2
g`D2?`H1W`L0o`H1W`82?`030]l3O`=o0083o`83n`030oH3l@?a0083k083i`030n<7h7oo00=oo`00
0Woo0Q_P00<7h0?S0nL00P?/00<3l@?f0oH00P?k0P?o00@3O`;O0Sl2?`81W`H0o`L0O`@0o`@1W`<2
?`<2g`<3O`<3o`L3naT3mPH3n`<3o`<3O`<2g`<2?`H1W`<0o`L0O`H0o`040Il2?`8o0]l20gl010?o
0o/3n`?f0P?a0P?/00@3i`?S1n0Kh0Aoo`000Woo00T_h1_P6n07h0?S0nL3k0?a0oH00`?k00<3o`=o
0]l00P8o0P6O0P3o3`1o103o0P6O0`8o0`;O0`=o0`?o1`?k5`?f1P?k0`?o0`=o0`;O0`8o106O0`3o
3`1o0P3o00@1W`8o0Sl2g`83O`040ol3n`?f0o430n`010?W0n<7h1_P17oo0002Ool02BoP6n0Kh0OP
0n<3i`?/0o43mP020o/00`?o0gl2g`020Sl20Il20?l207l<11l307l30?l20Il30Sl30]l30gl30ol7
0o/F0oH70o/20ol30gl30]l30Sl30Il30?l307l=11l0101o0?l0o`6O0P8o00L2g`=o0gl3o`?k0oH3
l@020n`01@?W0n<7h1_P;n0017oo0002Ool02BoP6n0Kh0OP0n<3i`?/0o43mP020o/00`?o0gl2g`02
0Sl20Il20?l00`1o11l47`0<11l307l30?l20Il30Sl30]l20gl40ol70o/E0oH70o/20ol30gl30]l3
0Sl20Il30?l307l>11l207l20?l00`6O0Sl2g`020gl010?o0o/3mP?a0P?/00D3i`?S1n0Kh2oP00Ao
o`000Woo0RoP00LKh0OP0n<3i`?/0o43mP020o/00`?o0gl2g`020Sl00`6O0?l0o`0207l511l761l3
11l307l20?l30Il20Sl40]l20gl30ol70o/D0oH70o/30ol20gl30]l30Sl20Il30?l307l411l761l5
11l01P1o0?l0o`6O0Sl2g`83O`040ol3n`?f0o420n`01@?W0n<7h1_P;n0017oo0002Ool02CoP;n0K
h1_P0n<3i`?/0o43mP020o/010?o0gl2g`8o0P6O0P3o00<0O`@O61l031PO10@O0P1o0P3o0P6O0`8o
0`;O0P=o0`?o1`?k50?f1`?k0P?o0P=o10;O0`8o0P6O0P3o0P1o1@@O31PO00D47`1o07l0o`6O0082
?`060]l3O`?o0o/3mP?a0P?/00D3i`?S1n0Kh2oP00Aoo`000Woo00UCh2oP6n0Kh0?S0nL3k0?a0oH0
0P?k00@3o`;O0Sl2?`81W`040?l0O`1o11l561l7;1l261l411l207l20?l20Il20Sl40]l00`=o0ol3
o`020ol60o/D0oH70o/20ol20gl30]l30Sl20Il20?l207l511l261l7;1l561l01`@O07l0o`3o0Il2
?`;O0083O`030o/3mP?a0083k0050n<7h1_P;n0oh004Ool0009oo`09Dn0_h2oP6n07h0?S0nL3l@?f
0083n`050gl2g`8o0Sl1W`020?l00`1o11lH7`0=;1l261l411l00`1o0?l0o`030Il20Sl30]l20gl3
0ol60o/D0oH60o/30ol00`=o0]l2g`020]l30Sl20Il00`3o07l0O`0411l361l<;1l021PO11l0O`1o
0?l1W`8o0]l20gl00`?o0o/3l@020n`01@?S1n0Kh2oP?n0017oo0002Ool03VOP?n0_h1_P1n03h`?W
0o43mP?k0ol3O`;O0Sl20Il01@3o07l0O`@O61l022`O13`O0R`O0aPO0`@O00<0O`3o0?l00P6O0`8o
0P;O0P=o0`?o1P?k4`?f1`?k0P?o0P=o0`;O0`8o0P6O0P3o00<0O`@O11l00P@O0aPO0R`O1C`O1b`O
00LH7`@O07l0o`6O0Sl2g`020gl02P?o0o/3l@?/0nL3h`OP6n0_h5?P17oo0002Ool03VOPDn0_h1_P
1n03h`?W0o43mP?k0ol3O`;O0Sl20Il0103o07l47aPO12`O2S`O0R`O0aPO0P@O0P1o0P3o0P6O0P8o
0P;O0P=o10?o1P?k4P?f1`?k0P?o0P=o0P;O0`8o0P6O0P3o00@0O`@O11l47`<H7`</7`/l7`8/7`03
61l47`1o0080o`0>0Il2g`=o0gl3o`?k0o43k0?W0n<7h1_P?n1Ch0Aoo`000Woo00=Wh5?P;n000Q_P
00X3h`?W0o43mP?k0ol3O`;O0Sl1W`80o`0407l47aPO;1l8?1l4D1l2?1l2;1l361l211l00`1o0?l0
o`020Il30Sl20]l00`=o0ol3o`020ol60o/B0oH60o/30ol20gl20]l30Sl20Il00`3o07l0O`0211l3
61l2;1l3?1l5D1l6?1l2;1l04QPO11l0O`3o0Il2?`;O0gl3o`?k0o43k0?W0n<7h1_P?n1Ch0Aoo`00
0Woo00akh6OP?n0_h1_P0n<3i`?a0oH3n`?o0]l20Sl01@6O0?l0O`@O61l00R`O0c`O2U0O0S`O0R`O
0aPO00<47`1o07l00P3o0P6O0P8o0P;O0P=o0`?o1P?k4P?f1P?k0`?o00<3O`;O0]l00`8o0P6O0P3o
00<0O`@O11l00aPO0R`O0S`O350O0S`O01</7aPO11l0O`3o0Il2?`;O0gl3o`?k0o43k0?S1n0Kh2oP
?n1Wh004Ool0009oo`0=Nn1Wh5?P;n0Kh0?S0nL3l@?f0o/3o`;O0Sl00P6O00H0o`1o11lH7b`O?1l6
D1l6I1l2D1l2?1l2;1l261l211l207l20?l20Il20Sl00`;O0gl3O`040ol50o/A0oH70o/20ol20gl2
0]l30Sl20Il0103o07l0O`@O0aPO0R`O0S`O0e0O1V@O1E0O0S`O018/7aPO11l0O`3o0Il2g`=o0ol3
n`?a0n`3h`OP6n0oh5?PNn04Ool0009oo`0DNn1Wh5?P;n0Kh0?S0nL3l@?f0o/3O`;O0Sl1W`3o07l4
7aPO;1ll7`9@7`]T7`9@7`8l7`</7`0361l47`@O0080O`80o`81W`82?`030]l3O`=o00@3o`D3na43
mPH3n`<3o`83O`040]l2?`8o0Sl20Il00`3o07l0O`0211l261l2;1l2?1l3D1l<I1l2?1l04R`O61l4
7`1o0?l1W`;O0gl3o`?k0o43k0?S1n0Kh3oPDn1kh0Aoo`000Woo01Ekh6OPDn0_h1_P0n<3i`?a0oH3
n`=o0]l2?`6O0?l0O`@O61l/7c`OD1l026@O0WPO0f@O0U0O0S`O0b`O00@H7`@O11l0O`80o`81W`82
?`030]l3O`=o00@3o`D3na43mPH3n`<3o`83O`040]l2?`8o0Sl20Il00`3o07l0O`0211l011PO;1l/
7b`O0S`O0e0O0V@O17PO1V@O01A@7c`O;1lH7`@O07l0o`6O0]l3O`?o0o/3l@?/0n<7h1_P?n1Ch7_P
17oo0002Ool05Gn0Nn1Ch2oP6n03h`?W0o43mP?k0gl2g`8o0Il0o`1o11lH7b`O?1m@7`03I1l8N1l3
I1l2D1l2?1l2;1l00aPO11l47`0207l20?l20Il00`8o0]l2g`020gl40ol40o/A0oH60o/30ol20gl0
0`;O0Sl2?`020Il20?l207l00`@O61l/7`02;1l2?1l3D1l3I1l9N1l2I1l2?1l04B`O61l47`3o0Il2
g`=o0ol3n`?a0n`3h`OP6n0oh5?PNn0017oo0002Ool05Gn0Nn1Ch2oP6n03h`?W0o43mP?k0gl2g`8o
0?l0O`@O61l/7c`OD1mT7`08N1l2O1`2N1l2I1l2D1l2?1l3;1l211l207l20?l20Il00`8o0]l2g`02
0gl40ol40o/A0oH50o/30ol30gl00`;O0Sl2?`020Il00`3o07l0O`0211l00aPO;1l/7`02?1l3D1l3
I1l017POO1al77`L1gPO01AT7c`O?1l/7aPO11l0o`6O0]l3O`?o0o/3l@?/0n<7h1_P?n1Wh7n017oo
0002Ool05GkPNn1Ch2oP6n03h`?W0o43mP?k0gl2g`8o0?l0O`@O61l/7c`OD1mT7`05N1l6O1`2N1l2
I1l2D1l2?1l2;1l011PO11l47`1o0P3o0P6O00<2?`;O0]l00P=o10?o10?k4@?f1@?k0`?o0`=o00<2
g`8o0Sl00P6O00<0o`1o07l00P@O00<H7b`O;1l00S`O0e0O0V@O0WPO1W`L17PO01AT7e0O?1l/7aPO
11l0o`6O0]l3O`?o0o/3l@?/0n<7h1_P?n1Wh7n017oo0002Ool057kPNn1Wh5?P;n07h0?S0n`3mP?k
0gl2g`8o0?l0O`@O61l/7c`OD1l3N1l7O1`00g`GO1al7002N1l2I1l01U0O?1ll7c`O;1lH7`847`80
O`80o`040Il2?`8o0]l20gl40ol50o/@0oH50o/30ol30gl010;O0Sl1W`6O0P3o0P1o00@47aPO;1l/
7`8l7`=@7`9T7`9h7`03O1al5g`G00Ml709h7`0CI1ll7b`O61l47`1o0?l1W`;O0ol3n`?a0n`3h`OP
6n0oh6OPOh0017oo0002Ool05GkPNn1Wh5?P;n07h0?S0n`3mP?o0gl2?`6O0?l0OaPO;1ll7e0OI1mh
7`06O1`5O1L2O1`01GPOI1mT7f@OD1l00S`O0R`O00@H7`@O11l0O`80o`81W`040Sl2g`=o0gl40ol5
0o/?0oH50o/40ol20gl00`;O0Sl2?`020Il00`3o07l0O`0211l00aPO;1ll7`02?1l2D1l2I1l2N1l2
O1`5O1L5O1`057POI1m@7b`O61l47`1o0?l1W`;O0ol3n`?a0nL7h1_P?n1Ch7_POh04Ool0009oo`0C
OV1oP7_PDn0_h0OP0n<3k0?f0ol3O`8o0Il0o`1o61l/7c`OD1l00WPO17`L27`G00=l77PON1l00V@O
0U0O00@l7b`O;1lH7`847`0307l0o`3o0081W`030Sl2g`;O0083O`@3o`@3n`l3mPD3n`@3o`83O`04
0]l2?`8o0Il20?l207l010@O61lH7b`O0S`O0U0O0V@O0WPO0W`L27`G0g`L01Ah7f@OD1ll7b`O61l4
7`3o0Il2g`?o0o/3l@?W1n0Kh3oPDn1kh7kP17oo0002Ool057iPO^1kh5?P;n07h0?S0n`3mP?o0gl2
?`3o07l47aPO;1ll7e0ON1l2O1`7O1L2O1<3O1L00g`LN1mh7`02I1l01E0O?1ll7b`O61l00P@O0P1o
0P3o00@1W`8o0]l2g`83O`@3o`@3n`l3mPD3n`<3o`<3O`040]l2?`6O0Il20?l207l01@@O61lH7b`O
?1l00e0O00=T7gPON1l00W`L0W`G0g`C1g`G01Al77POI1ll7b`O61l47`3o0Il2g`?o0o/3l@?W1n0K
h3oPIn1oP7iP17oo0002Ool05Gg0O^1kh5?P;n07h0?S0n`3mP?o0gl2?`3o07lH7b`O?1m@7f@ON1ml
7003O1L8O1<3O1L00g`LN1mh7`02I1l01E0O?1l/7b`O61l00P@O00D0O`3o0?l1W`8o0082g`83O`@3
o`@3n`l3mP@3n`@3o`<3O`050]l2?`6O0Il0o`0207l211l011PO;1ll7c`O0U0O00=T7gPON1l00W`L
0W`G27`C0g`G01Al77POI1m@7b`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP17oo0002Ool0
4gg0O^1kh5?P;n07h0?S0n`3mP?o0gl2?`3o07lH7b`O?1m@7gPO009l7003O1Ml4g`C00Il4`03O0il
4g`G009l5`03O1ah7gPO009T7`05D1ll7b`O;1lH7`0211l01@1o0?l0o`6O0Sl00P;O0P=o10?o10?k
3`?f10?k10?o0`=o00<2g`8o0Il00P3o0P1o00D47aPO61l/7c`O009@7`03I1mh7gPO009l709l5`9l
4`9l3PQl4`0DO1ah7f@OD1l/7aPO11l0o`6O0]l3o`?k0o43i`OP6n0oh6OPOh1nH0Aoo`000Woo01=m
`7kPNn1Ch2oP1n03h`?/0oH3o`=o0Sl0o`1o61l/7c`OD1mh7`02O1`7O1<4O0h00g`CO1Ml5`02O1`0
1GPOI1mT7f@O?1l00R`O00@H7`@O11l0O`80o`040Il2?`;O0]l20gl40ol40o/?0oH40o/40ol30gl0
0`;O0Sl1W`020?l207l01@@O61lH7b`O?1l00U0O00=T7gPON1l00W`L0W`G0W`C17`>1W`C01Al77PO
I1m@7b`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP17oo0002Ool04gg0O^1kh5?P;n07h0?S
0n`3mP?o0gl2?`3o07lH7b`O?1m@7gPO009l70Il4`El3P05O1=l5g`GO1Ml7002N1l2I1l01E0O?1l/
7b`O61l00P@O00@0O`3o0Il2?`82g`83O`@3o`@3n`l3mP@3n`@3o`<3O`030]l2?`6O0080o`80O`06
11lH7aPO;1ll7e0O0V@O0WPO00=l77`GO1L00W`C1W`>1G`C01Al5g`LN1m@7c`O61l47`3o0Il2g`?o
0o/3l@?W1n0Kh3oPIn1oP7iP17oo0002Ool05Gg0O^1kh5?P;n07h0?S0n`3mP?o0gl2?`3o07lH7b`O
D1mT7gPOO1al5`04O1<5O0h017`9O0il4g`C0W`G00=l77PON1l00V@O00E@7c`O;1l/7aPO00847`04
07l0o`3o0Sl20]l20gl40ol40o/?0oH40o/40ol30gl010;O0Sl1W`3o0P1o00H47aPO61l/7c`OD1l2
I1l2N1l2O1`2O1L2O1<00g`>O0Ul2@05O0h3O1<057`GO1ah7e0O?1lH7`@O0?l1W`;O0ol3n`?a0nL7
h1_P?n1Wh7n0OV04Ool0009oo`0DOL1nh7_PDn0_h0OP0n<3k0?f0ol3O`8o0?l0OaPO;1m@7f@ON1ml
70=l4`Il3P=l2@04O0il4g`GO1L2O1`01GPOI1mT7e0O?1l00R`O00@H7`@O11l0O`80o`030Sl2g`;O
0083O`@3o`@3n`l3mP@3n`@3o`<3O`040]l2?`3o0?l207l01P@O61lH7b`O?1m@7`9T7`9h7`9l709l
5`03O1=l3W`>00=l2@Il3P0EO1=l5g`LN1m@7c`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP
00Aoo`000Woo01Am87iPOh1Wh3oP6n03h`?/0oH3o`=o0Sl0o`1o61l/7e0OI1mh7g`L0W`C1G`>1G`9
00=l3W`CO1<00W`G00El77PON1mT7e0O008l7`05;1lH7`@O11l0O`020?l0106O0Sl2g`;O0P=o10?o
10?k3P?f10?k10?o0`=o00@2g`8o0?l0o`80O`0611lH7aPO;1ll7e0O0V@O0WPO00=l77`GO1L00W`C
00=l3W`9O0T00g`91W`>01Al4g`LN1mT7e0O;1lH7`1o0?l2?`=o0o/3l@?W1n0Kh3oPIn1oP7iP17oo
0002Ool05GdPOV1oP6OP?n0Kh0?S0n`3mP?o0]l1W`1o11l/7c`OD1mh7g`LO1Ml4`05O0h7O0T017`>
O1=l5g`G0W`L00=h7f@OI1l00S`O00D/7aPO11l47`1o0080o`040Il2?`;O0]l20gl40ol40o/>0oH3
0o/40ol30gl00`;O0Sl1W`020?l207l01P@O61lH7b`O?1m@7`9T7`03N1ml77`L009l5`03O1=l3W`>
00Ml2@Al3P0DO1=l77POI1m@7b`O61l0O`3o0Sl3O`?k0o43i`OP;n1Ch7_POV1m80Aoo`000Woo01Em
87iPOh1Wh3oP6n03h`?/0oH3o`;O0Il0O`@O;1ll7e0ON1ml77`GO1<017`>27`900Al3W`CO1=l5`9l
7005N1mT7f@OD1ll7`02;1l01aPO11l47`1o0?l1W`8o0082g`83O`@3o`@3n`h3mP<3n`@3o`<3O`03
0]l2?`6O0080o`80O`0511lH7aPO;1ll7`03I1l017POO1al5g`G0W`C00=l3W`9O0T01g`90g`>01Al
4g`GO1ah7e0O?1lH7`1o0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7dP17oo0002Ool05Gb0OV1oP6OP?n0K
h0?S0n`3mP?o0]l1W`1o11l/7e0OI1mh7g`LO1Ml4`02O0h:O0T00g`>O1=l4`02O1L01G`LN1mh7f@O
D1l00S`O00P/7aPO11l47`1o0?l1W`8o0P;O0P=o10?o10?k3P?f0`?k10?o0`=o00@2g`8o0Il0o`80
O`0611lH7aPO;1ll7e0O0V@O0WPO00Al77`GO1Ml4`9l3PYl2@9l3P0DO1=l5g`LN1m@7c`O61l0O`3o
0Sl3O`?k0o43i`OP;n1Ch7_POV1m80Aoo`000Woo01AlP7iPOh1Wh3oP6n03h`?/0oH3o`;O0Il0O`@O
;1m@7f@ON1ml5g`C0W`>1G`917`40g`900Al3W`CO1=l5`9l7003N1mT7e0O008l7`08;1lH7`@O11l0
O`3o0Il2?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l01P@O61lH7b`O?1m@7`9T
7`03N1ml77`L009l5`03O1=l3W`>009l2@El10Al2@0EO0il4g`GO1ah7e0O?1lH7`1o0?l2?`=o0o/3
l@?W1n0_h5?PNn1nH7dP00Aoo`000Woo01ElP7iPOh1Wh3oP6n03h`?/0oH3o`;O0Il0O`@O;1m@7f@O
N1ml5g`CO0h00W`927`40g`900Al3W`CO1=l5`9l7003N1mT7e0O008l7`08;1lH7`@O11l0O`3o0Il2
?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l01P@O61lH7b`O?1m@7`9T7`05N1ml
77`GO1Ml4`02O0h2O0T8O0@2O0T05G`>O1=l5g`LN1m@7c`O61l0O`3o0Sl3O`?k0o43i`OP;n1Ch7_P
OV1m8004Ool0009oo`0EO81nH7n0In0oh1_P0n<3k0?f0ol2g`6O07l47b`OD1mT7gPOO1Ml4g`>009l
2@Ql10=l2@04O0il4g`CO1L2O1`00gPOI1m@7`02?1l022`O61l47`@O07l0o`6O0Sl20]l20gl40ol4
0o/>0oH30o/40ol30gl010;O0Sl1W`3o0P1o00H47aPO61l/7c`OD1l2I1l01GPOO1al5g`GO1<00W`>
0W`927`40W`901El3W`CO1Ml77POD1ll7aPO07l0o`8o0gl3n`?a0nL7h2oPDn1kh7iPOB0017oo0002
Ool05Gb0OV1oP6OP?n0Kh0?S0n`3mP?o0]l1W`1o11l/7e0OI1mh7g`GO1=l3P02O0T9O0@2O0T017`>
O1=l4g`G0W`L00=h7f@OD1l00S`O00P/7aPO11l47`1o0?l1W`8o0P;O0P=o10?o10?k3P?f0`?k10?o
0`=o00@2g`8o0Il0o`80O`0711lH7aPO;1ll7e0OI1l00WPO00Al77`GO1Ml4`9l3P9l2@Ul109l2@0D
O1=l5g`LN1m@7c`O61l0O`3o0Sl3O`?k0o43i`OP;n1Ch7_POV1m80Aoo`000Woo01ElP7iPOh1Wh3oP
6n03h`?/0oH3o`;O0Il0O`@O;1m@7f@ON1ml5g`CO0h00W`92G`40W`900Al3W`CO1=l5`9l7003N1mT
7e0O008l7`08;1lH7`@O11l0O`3o0Il2?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l2
07l01`@O61lH7b`O?1m@7f@O009h7`05O1al5g`GO1=l3P02O0T:O0@2O0T057`CO1Ml77POD1ll7aPO
07l0o`8o0gl3n`?a0nL7h2oPDn1kh7iPOB04Ool0009oo`0EO81nH7n0In0oh1_P0n<3k0?f0ol2g`6O
07l47b`OD1mT7gPOO1Ml4g`>009l2@Ul109l2@04O0il4g`CO1L2O1`00gPOI1m@7`02?1l022`O61l4
7`@O07l0o`6O0Sl20]l20gl40ol40o/>0oH30o/40ol30gl010;O0Sl1W`3o0P1o00L47aPO61l/7c`O
D1mT7`02N1l01G`LO1Ml5g`CO0h00W`92W`40W`901Al4g`GO1ah7e0O?1lH7`1o0?l2?`=o0o/3l@?W
1n0_h5?PNn1nH7dP17oo0002Ool05Gb0OL1nh7_P?n0Kh0?S0n`3mP?o0]l1W`1o11l/7e0OI1mh7g`G
O1=l3P02O0T9O0@3O0T01W`>O1=l4g`GO1ah7`9T7`03D1ll7b`O008H7`0511l0O`3o0Il2?`020]l3
0gl30ol40o/>0oH30o/40ol30gl010;O0Sl1W`3o0P1o00L47aPO61l/7c`OD1mT7`02N1l01G`LO1Ml
5g`CO0h00W`92W`40W`901Al3W`CO1Mh7f@O?1lH7`@O0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7dP17oo
0002Ool04gb0OL1nh7_P?n0Kh0?S0n`3mP?o0]l1W`1o61l/7e0ON1ml77`G009l3P03O0Ul17`400Ql
10=l2@05O0il4g`CO1Ml7002N1l016@OD1ll7b`O0QPO00D47`1o0?l1W`8o0082g`<3O`<3o`@3n`h3
mP<3n`@3o`83O`82g`040Sl1W`3o07l211l00aPO;1ll7`02D1l00f@ON1ml7002O1L00g`CO0il3P02
O0T:O0@2O0T057`>O1=l5gPOI1ll7aPO11l0o`8o0gl3n`?a0nL7h3oPIn1oP7iPOB04Ool0009oo`0C
O81m`7kPNn0oh1_P0n<3k0?f0ol2g`6O07lH7b`OD1mh7g`LO1L00W`>00=l2G`4O0@02G`40W`900El
3W`CO1=l5g`L009h7`04I1m@7c`O;1l261l01@@O07l0o`6O0Sl00P;O0`=o0`?o10?k3P?f0`?k10?o
0P=o0P;O00@2?`6O0?l0O`847`0361l/7c`O009@7`03I1mh7g`L009l5`03O1=l3W`>009l2@]l100E
O0Ul3W`CO1Mh7f@O?1lH7`@O0?l2?`=o0o/3l@?W1n0oh6OPOh1nH7dP00Aoo`000Woo01AlP7g0O^1k
h3oP6n03h`?/0oH3o`;O0Il0OaPO;1m@7gPOO1al5g`>0W`92g`40W`900El3W`CO1=l5g`L009h7`04
I1m@7c`O;1l261l01@@O07l0o`6O0Sl00P;O0`=o0`?o10?k3P?f0`?k10?o0P=o0P;O00@2?`6O0?l0
O`847`0361l/7c`O009@7`03I1mh7g`L009l5`04O1=l3W`9O0T<O0@05G`9O0il4g`GN1mT7c`O61l4
7`3o0Sl3O`?k0o43i`OP?n1Wh7n0OV1m8004Ool0009oo`0DO81m`7kPNn0oh1_P0n<3k0?f0ol2g`6O
07lH7b`OD1mh7g`LO1Ml3P9l2@Il10=l009l109l2@05O0il4g`CO1Ml7002N1l016@OD1ll7b`O0QPO
00D47`1o0?l1W`8o0082g`<3O`<3o`@3n`h3mP<3n`@3o`83O`82g`040Sl1W`3o07l211l00aPO;1ll
7`02D1l00f@ON1ml7002O1L017`CO0il2G`90g`41G`017`401El2G`>O1=l5gPOI1ll7aPO11l0o`8o
0gl3n`?a0nL7h3oPIn1oP7iPOB0017oo0002Ool057b0OL1nh7_P?n0Kh0?S0n`3mP?o0]l1W`1o61l/
7e0ON1ml77`GO0h2O0T5O0@4O002O0@2O0T01G`>O1=l4g`GO1`00WPO00AT7e0O?1l/7`8H7`0511l0
O`3o0Il2?`020]l30gl30ol40o/>0oH30o/40ol20gl20]l0108o0Il0o`1o0P@O00<H7b`O?1l00U0O
00=T7gPOO1`00W`G00Al4g`>O0Ul2@9l10Il00Al100EO0Ul3W`CO1Mh7f@O?1lH7`@O0?l2?`=o0o/3
l@?W1n0oh6OPOh1nH7dP00Aoo`000Woo01AlP7g0O^1kh3oP6n03h`?/0oH3o`;O0Il0OaPO;1m@7gPO
O1al5g`>0W`927`400=l07`4O0@00W`900El3W`CO1=l5g`L009h7`04I1m@7c`O;1l261l01@@O07l0
o`6O0Sl00P;O0`=o0`?o10?k3P?f0`?k10?o0P=o0P;O00@2?`6O0?l0O`847`0361l/7c`O009@7`03
I1mh7g`L009l5`04O1=l3W`9O0T3O0@2O007O0@05G`9O0il4g`GN1mT7c`O61l47`3o0Sl3O`?k0o43
i`OP?n1Wh7n0OV1m8004Ool0009oo`0DO81m`7kPNn0oh1_P0n<3k0?f0ol2g`6O07lH7b`OD1mh7g`L
O1Ml3P9l2@]l109l2@05O0il4g`CO1Ml7002N1l016@OD1ll7b`O0QPO00D47`1o0?l1W`8o0082g`<3
O`<3o`@3n`h3mP<3n`@3o`83O`82g`040Sl1W`3o07l211l00aPO;1ll7`02D1l00f@ON1ml7002O1L0
0g`CO0il3P02O0T;O0@05G`9O0il4g`GN1mT7c`O61l47`3o0Sl3O`?k0o43i`OP?n1Wh7n0OV1m8004
Ool0009oo`0CO81m`7kPNn0oh1_P0n<3k0?f0ol2g`6O07lH7b`OD1mh7g`LO1L00W`>00=l2G`4O0@0
27`40g`900El3W`CO1=l5g`L009h7`04I1m@7c`O;1l261l01@@O07l0o`6O0Sl00P;O0`=o0`?o10?k
3P?f0`?k10?o0P=o0P;O00@2?`6O0?l0O`847`0361l/7c`O009@7`03I1mh7g`L009l5`03O1=l3W`>
009l2@Yl109l2@0DO0il4g`GN1mT7c`O61l47`3o0Sl3O`?k0o43i`OP?n1Wh7n0OV1m80Aoo`000Woo
01=lP7g0O^1kh3oP6n03h`?/0oH3o`;O0Il0OaPO;1m@7gPOO1al5`02O0h00g`9O0Al1008O0@3O0T0
1G`>O1=l4g`GO1`00WPO00AT7e0O?1l/7`8H7`0511l0O`3o0Il2?`020]l30gl30ol40o/>0oH30o/4
0ol20gl20]l0108o0Il0o`1o0P@O00<H7b`O?1l00U0O00=T7gPOO1`00W`G00=l4g`>O0h00W`92W`4
0W`901Al3W`CO1Mh7f@O?1lH7`@O0?l2?`=o0o/3l@?W1n0oh6OPOh1nH7dP17oo0002Ool05Gb0OL1n
h7_P?n0Kh0?S0n`3mP?o0]l1W`1o11l/7e0OI1mh7g`GO1=l3P02O0T9O0@3O0T01W`>O1=l4g`GO1ah
7`9T7`03D1ll7b`O008H7`0511l0O`3o0Il2?`020]l30gl30ol40o/>0oH30o/40ol30gl010;O0Sl1
W`3o0P1o00L47aPO61l/7c`OD1mT7`02N1l01G`LO1Ml5g`CO0h00W`92W`40W`901Al3W`CO1Mh7f@O
?1lH7`@O0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7dP17oo0002Ool05Gb0OV1oP6OP?n0Kh0?S0n`3mP?o
0]l1W`1o11l/7e0OI1mh7g`GO1=l3P02O0T9O0@2O0T017`>O1=l4g`G0W`L00=h7f@OD1l00S`O00P/
7aPO11l47`1o0?l1W`8o0P;O0P=o10?o10?k3P?f0`?k10?o0`=o00@2g`8o0Il0o`80O`0711lH7aPO
;1ll7e0OI1l00WPO00El77`GO1Ml4g`>009l2@Yl109l2@0DO1=l5g`LN1m@7c`O61l0O`3o0Sl3O`?k
0o43i`OP;n1Ch7_POV1m80Aoo`000Woo01ElP7iPOh1Wh3oP6n03h`?/0oH3o`;O0Il0O`@O;1m@7f@O
N1ml5g`CO0h00W`92G`40W`900Al3W`CO1=l5`9l7003N1mT7e0O008l7`08;1lH7`@O11l0O`3o0Il2
?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l01`@O61lH7b`O?1m@7f@O009h7`04
O1al5g`GO1<2O0h2O0T9O0@2O0T057`CO1Ml77POD1ll7aPO07l0o`8o0gl3n`?a0nL7h2oPDn1kh7iP
OB04Ool0009oo`0EO81nH7n0In0oh1_P0n<3k0?f0ol2g`6O07l47b`OD1mT7gPOO1Ml4g`>009l2@Ql
10=l2@04O0il4g`CO1L2O1`00gPOI1m@7`02?1l022`O61l47`@O07l0o`6O0Sl20]l20gl40ol40o/>
0oH30o/40ol30gl010;O0Sl1W`3o0P1o00L47aPO61l/7c`OD1mT7`02N1l017`LO1Ml5g`C0W`>0W`9
27`40g`901Al4g`GO1ah7e0O?1lH7`1o0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7dP17oo0002Ool05Gb0
OV1oP6OP?n0Kh0?S0n`3mP?o0]l1W`1o11l/7e0OI1mh7g`GO1=l3P02O0T8O0@3O0T017`>O1=l4g`G
0W`L00=h7f@OD1l00S`O00P/7aPO11l47`1o0?l1W`8o0P;O0P=o10?o10?k3P?f0`?k10?o0`=o00@2
g`8o0Il0o`80O`0611lH7aPO;1ll7e0O0V@O00Eh7g`LO1Ml5g`C009l3P9l2@Ql109l2@0EO0il4g`G
O1ah7e0O?1lH7`1o0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7dP00Aoo`000Woo01AlP7iPOh1Wh3oP6n03
h`?/0oH3o`;O0Il0O`@O;1m@7f@ON1ml5g`C0g`>0g`91G`40g`900Al3W`CO1=l5`9l7003N1mT7e0O
008l7`08;1lH7`@O11l0O`3o0Il2?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l0
1P@O61lH7b`O?1m@7`9T7`05N1ml77`GO1Ml4`02O0h2O0T6O0@3O0T2O0h057`CO1Ml77POD1ll7aPO
07l0o`8o0gl3n`?a0nL7h2oPDn1kh7iPOB04Ool0009oo`0DO81nH7n0In0oh1_P0n<3k0?f0ol2g`6O
07l47b`OD1mT7gPOO1Ml4`Al3PIl2@05O0Al2G`9O0Ul3P02O1<01W`GO1al77POI1m@7`8l7`08;1lH
7`@O11l0O`3o0Il2?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l01P@O61lH7b`O
?1m@7`9T7`03N1ml77`L009l5`03O1=l3W`>009l2@9l10El2@=l3P0DO1=l5g`LN1m@7c`O61l0O`3o
0Sl3O`?k0o43i`OP;n1Ch7_POV1m80Aoo`000Woo01ElP7iPOh1Wh3oP6n03h`?/0oH3o`;O0Il0O`@O
;1m@7f@ON1ml77`GO1<017`>27`900=l3W`CO1<00W`G00El77PON1mT7e0O008l7`08;1lH7`@O11l0
O`3o0Il2?`82g`83O`@3o`@3n`h3mP<3n`@3o`<3O`040]l2?`6O0?l207l01P@O61lH7b`O?1m@7`9T
7`9h7`04O1al5g`GO1<2O0h8O0T4O0h057`CO1Ml77POD1ll7aPO07l0o`8o0gl3n`?a0nL7h2oPDn1k
h7iPOB04Ool0009oo`0EOB1nH7n0In0oh1_P0n<3k0?f0ol2g`6O07l47b`O?1m@7gPOO1al5g`C00El
3PMl2@04O0il4g`CO1L2O1`01GPOI1mT7e0O?1l00R`O00LH7`@O11l0O`3o0Il2?`020]l20gl40ol4
0o/>0oH30o/40ol30gl00`;O0Sl1W`020?l207l01P@O61lH7b`O?1m@7`9T7`04N1ml77`GO1L2O1<0
0g`>O0Ul2@05O0T5O0h057`CO1Ml77POD1ll7aPO07l0o`8o0gl3n`?a0nL7h2oPDn1kh7iPOB04Ool0
009oo`0EOB1nH7n0In0oh1_P0n<3k0?f0ol2g`6O07l47b`O?1m@7gPOO1al5g`C00Ml3PEl2@04O0il
4g`GO1L2O1`00gPOI1m@7`02?1l01B`O61l47`@O07l00P3o00@1W`8o0]l2g`83O`@3o`@3n`h3mP<3
n`@3o`<3O`030]l2?`6O0080o`80O`0611lH7aPO;1ll7e0O0V@O00=h7g`LO1`00W`G00=l4g`>O0h0
1G`91W`>01Al4g`LN1mT7e0O;1lH7`1o0?l2?`=o0o/3l@?W1n0_h5?PNn1nH7g017oo0002Ool057g0
OV1oP6OP?n0Kh0?S0n`3mP?o0gl2?`3o07lH7b`OD1mT7gPOO1`3O1<6O0h3O0T00g`>O1=l4`02O1L0
1G`LN1mh7f@OD1l00S`O00D/7aPO11l47`1o0080o`040Il2?`;O0]l20gl40ol40o/>0oH40o/40ol3
0gl010;O0Sl0o`3o0P1o00H47aPO61l/7c`OD1l2I1l2N1l00g`LO1Ml5`02O1<00g`>O0Ul2@02O0T6
O0h05G`CO1Ml77POI1m@7b`O61l0O`3o0Sl3O`?k0o43i`OP6n0oh6OPOh1nH004Ool0009oo`0DOL1n
h7_PDn0_h0OP0n<3k0?f0ol3O`8o0?l0OaPO;1m@7f@ON1ml70El4`El3P9l2@04O0il4g`GO1L2O1`0
1GPOI1mT7e0O?1l00R`O00@H7`@O11l0O`80o`030Sl2g`;O0083O`@3o`@3n`l3mP@3n`@3o`<3O`04
0]l2?`3o0?l207l01P@O61lH7b`O?1m@7`9T7`9h7`03O1al5g`G009l4`04O0il2G`9O0T5O0h3O1<0
57`GO1ah7e0O?1lH7`@O0?l1W`;O0ol3n`?a0nL7h1_P?n1Wh7n0OV04Ool0009oo`0EOL1nh7_PDn0_
h0OP0n<3k0?f0ol3O`8o0?l0OaPO;1m@7f@ON1ml77`G00Il4`Al3P03O0Ul4g`C009l5`9l7005N1mT
7f@OD1ll7`02;1l011PO11l47`1o0P3o00<2?`;O0]l00P=o10?o10?k3`?f10?k10?o0`=o00@2g`8o
0Il0o`80O`0611lH7aPO;1ll7e0O0V@O0WPO0W`L0W`G0W`C00=l3W`9O0h00g`>1G`C01Al5g`LN1m@
7c`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP17oo0002Ool05Gg0O^1kh5?P;n07h0?S0n`3
mP?o0gl2?`3o07lH7b`O?1m@7gPOO1al5`08O1<3O0h2O1<2O1L00g`LN1mh7`02I1l01E0O?1l/7b`O
61l00P@O00@0O`3o0Il2?`82g`83O`@3o`@3n`l3mP@3n`@3o`<3O`030]l2?`6O0080o`80O`0611lH
7aPO;1ll7e0O0V@O00=h7g`LO1`00W`G0W`C17`>1g`C01Al5g`LN1m@7c`O61l47`3o0Il2g`?o0o/3
l@?W1n0Kh3oPIn1oP7iP17oo0002Ool04gg0O^1kh5?P;n07h0?S0n`3mP?o0gl2?`3o07lH7b`O?1m@
7gPO009l70Yl4`03O0il4g`G009l5`03O1ah7gPO009T7`8l7`05;1lH7`@O11l0O`020?l0106O0Sl2
g`;O0P=o10?o10?k3`?f10?k10?o0`=o00<2g`8o0Il00P3o0P1o00H47aPO61l/7c`OD1l2I1l2N1l0
0g`LO1Ml5`03O1<00g`>O1=l4`07O1<057`LN1mT7e0O;1lH7`@O0?l1W`;O0ol3n`?a0nL7h1_P?n1W
h7n0OV04Ool0009oo`0COL1nh7_PDn0_h0OP0n<3k0?f0ol3O`8o0?l0OaPO;1ll7e0ON1l00W`L00=l
5g`CO1<02G`C0W`G0W`L00Eh7f@OI1m@7c`O008/7`0461l47`@O07l20?l0106O0Sl2g`;O0P=o10?o
10?k3`?f10?k10?o0`=o00<2g`8o0Il00P3o0P1o00D47aPO61l/7c`O009@7`03I1mh7gPO009l709l
5`]l4`0EO1Ml77POI1m@7b`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP00Aoo`000Woo01=m
`7kPNn1Ch2oP1n03h`?/0oH3o`=o0Sl0o`1o61l/7c`OD1mh7`04O1`5O1L4O1<3O1L00g`LN1mh7`02
I1l01E0O?1l/7b`O61l00P@O00D0O`3o0?l1W`8o0082g`83O`@3o`@3n`l3mP@3n`@3o`<3O`030]l2
?`6O0080o`80O`0511lH7aPO;1ll7`02D1l00f@ON1mh7`03O1`3O1L4O1<5O1L2O1`04gPOI1m@7b`O
61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPIn1oP7iP00Aoo`000Woo015nH7kPNn1Ch2oP1n03h`?/0oH3
o`=o0Sl0o`1o61l/7c`O009@7`03N1ml77`L00Al70Ql5`9l709h7`9T7`05D1ll7b`O;1lH7`0211l0
1@1o0?l0o`6O0Sl00P;O0P=o10?o10?k3`?f10?k10?o0`=o00D2g`8o0Il1W`3o0080O`847`0461l/
7c`O?1l2D1l00f@ON1mh7`03O1`8O1L5O1`04gPOI1m@7b`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oP
In1oP7iP00Aoo`000Woo01=nH7kPNn1Ch2oP1n03h`?/0oH3o`=o0Sl1W`3o07lH7b`O?1m@7`02N1l7
O1`5O1L2O1`2N1l2I1l01E0O?1ll7b`O61l00P@O0P1o0P3o00@1W`8o0]l2g`83O`@3o`@3n`l3mPD3
n`@3o`83O`040]l2?`6O0Il20?l207l010@O61lH7b`O0S`O0U0O00=T7gPON1l00g`L1G`G1W`L01Ah
7f@OD1ll7b`O61l47`3o0Il2g`?o0o/3l@?W1n0Kh3oPDn1kh7kP17oo0002Ool057kPNn1Wh5?P;n07
h0?S0n`3mP?o0gl2?`6O0?l0OaPO;1ll7e0OI1l3N1l8O1`00g`GO1al7002N1l2I1l00e0O?1ll7`02
;1l011PO11l47`1o0P3o0P6O00@2?`;O0gl3O`@3o`D3n`l3mPD3n`@3o`83O`040]l2?`8o0Il20?l2
07l010@O61lH7b`O0S`O0U0O0V@O0WPO0g`L00=l5g`LO1`01W`L0WPO01=T7e0O;1lH7`@O07l0o`6O
0]l3o`?k0o43i`OP6n0oh5?PNn1oP004Ool0009oo`0BO^1kh6OPDn0_h0OP0n<3k0?f0ol3O`8o0Il0
o`1o61l/7c`O0U0O1WPO1g`L0WPO0f@O00A@7c`O?1l/7`8H7`847`0307l0o`3o0081W`040Sl2g`=o
0gl40ol50o/?0oH50o/40ol20gl20]l0108o0Il1W`3o0P1o0P@O00<H7b`O?1l00S`O0U0O0V@O0WPO
27`L1GPO01=T7e0O;1lH7`@O07l0o`6O0]l3o`?k0o43i`OP6n0oh5?PNn1oP004Ool0009oo`0EO^1k
h6OPDn0_h0OP0n<3k0?f0o/3O`;O0Sl0o`1o11lH7b`O?1m@7f@O00Mh7`El7005N1mT7f@OI1m@7`03
?1l012`O61l47`@O0P1o0P3o00@1W`8o0]l2g`83O`@3o`D3na03mPD3n`<3o`<3O`040]l2?`6O0Il2
0?l207l010@O61lH7b`O0S`O0e0O0V@O0WPO1G`L1WPO01AT7e0O?1l/7aPO11l0O`3o0Il2g`?o0o/3
l@?/0n<7h1_P?n1Wh7n017oo0002Ool057n0Nn1Ch2oP6n03h`?W0o43mP?k0gl2g`8o0?l0O`@O61l/
7c`OD1l4I1l7N1l00g`LN1mh7`02I1l2D1l2?1l2;1l011PO11l47`1o0P3o0P6O00<2?`;O0]l00P=o
10?o10?k4@?f1@?k0`?o0`=o00<2g`8o0Sl00P6O00<0o`1o07l00P@O00<H7b`O;1l00S`O0e0O0V@O
0WPO00=l77PON1l01GPO0f@O01=@7c`O;1lH7`@O0?l1W`;O0gl3o`?k0o43k0?S1n0Kh3oPIn1oP004
Ool0009oo`0COh1kh5?P;n0Kh0?S0nL3l@?f0o/3O`;O0Sl1W`3o07lH7b`O?1l00U0O1V@O1WPO0V@O
0U0O0S`O0R`O00<H7`@O11l00P1o0P3o0P6O00<2?`;O0]l00P=o10?o10?k4@?f1@?k10?o0P=o00<2
g`8o0Sl00P6O00<0o`1o07l00P@O00<H7b`O;1l00c`O0U0O0f@O1WPO1V@O0S`O014/7aPO11l0o`6O
0]l3O`?o0o/3l@?/0n<7h1_P?n1Ch7_P00Aoo`000Woo01Ekh6OPDn0_h1_P0n<3i`?a0oH3n`=o0]l2
?`6O0?l0O`@O61l/7c`OD1l02F@O0WPO0f@O00A@7c`O?1ll7`8/7`0361l47`@O0080O`80o`81W`04
0Sl2g`=o0gl40ol50o/A0oH60o/30ol20gl00`;O0Sl2?`020Il20?l207l00`@O61lH7`02;1l2?1l3
D1l3I1l2N1l8I1l0550O?1l/7aPO11l0O`3o0Il2g`=o0ol3n`?a0n`3h`OP6n0oh5?PNn04Ool0009o
o`0DNn1Wh5?P;n0Kh0?S0nL3l@?f0o/3O`;O0Sl1W`3o07l47aPO;1ll7`9@7`aT7`9@7`8l7`8/7`8H
7`847`0307l0o`3o0081W`82?`030]l3O`=o00@3o`D3na43mPH3n`<3o`83O`040]l2?`8o0Sl20Il0
0`3o07l0O`0211l261l2;1l2?1l3D1l;I1l2D1l04c`O;1lH7`@O07l0o`6O0]l3O`?o0o/3l@?/0n<7
h1_P?n1Ch7_P00Aoo`000Woo01=kh6OPDn0_h1_P0n<3i`?a0oH3n`?o0]l2?`6O0?l0O`@O61l/7`02
?1l7D1l4I1l2D1l2?1l2;1l261l311l207l20?l20Il20Sl00`;O0gl3O`040ol50o/A0oH60o/30ol2
0gl010;O0Sl2?`8o0P6O00<0o`1o07l010@O0QPO0R`O0S`O0e0O16@O1U0O0S`O018/7aPO11l0O`3o
0Il2g`=o0ol3n`?a0n`3h`OP6n0oh5?PNn04Ool0009oo`0=Nn1Wh5?P;n0Kh0?S0nL3l@?f0o/3o`;O
0Sl00P6O00D0o`1o11lH7b`O00Dl7`Q@7`8l7`8/7`8H7`@47`80O`80o`81W`82?`030]l3O`=o00@3
o`D3na43mPL3n`83o`83O`82g`<2?`81W`030?l0O`1o00@47`8H7`8/7`<l7`U@7`@l7`0B;1lH7`@O
07l0o`6O0]l3O`?o0o/3l@?/0n<7h1_P?n1Ch6OP17oo0002Ool04VOPDn0oh2oP6n03h`?W0o43mP?k
0ol3O`;O0Sl1W`3o07l47`8H7`03;1ll7c`O00Ll7`9@7`8l7`8/7`8H7`@47`80O`80o`81W`82?`82
g`83O`<3o`H3na83mPH3n`<3o`83O`82g`<2?`030Il0o`3o0080O`@47`8H7`</7`8l7`=@7`Pl7`0C
;1lH7`@O07l0o`6O0Sl2g`=o0ol3n`?a0n`3i`?S1n0Kh3oPDn0017oo0002Ool03fOPDn0_h1_P1n03
h`?W0o43mP?k0ol3O`;O0Sl1W`020?l00`1o11lH7`05;1l8?1l2;1l261l311l307l20?l20Il20Sl2
0]l20gl40ol60o/B0oH60o/30ol20gl20]l30Sl20Il00`3o07l0O`0207l311l361l2;1l9?1l3;1l2
61l01@@O07l0o`6O0]l00P=o00X3o`?k0o43k0?W0n<7h1_P?n1Ch0Aoo`000Woo00iWh5?P;n0Kh0OP
0n<3i`?a0oH3n`?o0gl2g`8o0P6O00D0o`1o07l47aPO00T/7`8l7`8/7`8H7`<47`@0O`80o`81W`82
?`82g`83O`@3o`H3na83mPL3n`83o`83O`<2g`<2?`81W`030?l0O`1o0080O`@47`8H7`8/7`<l7`P/
7`0361l47`1o0080o`0>0Il2g`=o0gl3o`?k0o43k0?W0n<7h1_P?n1Ch0Aoo`000Woo00UCh3oP;n0K
h0OP0n<3i`?a0oH00P?k00D3o`;O0Sl2?`6O0080o`0307l47aPO00@H7`P/7`8H7`<47`@0O`80o`81
W`82?`<2g`83O`<3o`H3na<3mPL3n`<3o`040gl2g`;O0]l30Sl20Il20?l407l411l261l9;1l361l0
1`@O07l0O`3o0Il2?`;O0083O`030ol3n`?a0083k0050n<7h1_P;n1Ch004Ool0009oo`09Dn0_h1_P
6n03h`?W0n`3l@?f0083n`040ol2g`8o0Sl20Il0103o07l0O`@O2APO0R`O0QPO0P@O101o0`3o0P6O
0`8o0`;O00<3O`?o0ol00P?o1P?k50?f1`?k0P?o0P=o0`;O0`8o0P6O0`3o101o0`@O0QPO0b`O21PO
00`47`1o0?l0o`6O0Sl2g`=o0ol3n`?f0o420n`01@?S1n0Kh2oP?n0017oo0002Ool02E?P;n0Kh1_P
0n<3i`?/0o43mP020o/010?o0gl2g`8o0P6O0P3o00<0O`@O11l00`@O21PO0P@O101o103o00@1W`8o
0Sl2?`<2g`83O`<3o`L3na@3mPL3n`83o`<3O`<2g`<2?`030Il0o`3o0080o`@0O`<47`TH7`<47`80
O`0:0?l1W`8o0Sl2g`=o0ol3n`?f0o420n`01@?W0n<7h1_P?n0017oo0002Ool02CoP;n0Kh0OP0n<3
i`?/0o43mP020o/00`?o0gl2g`020Sl00`6O0?l0o`0207l911l261l211l407l40?l20Il20Sl30]l3
0gl30ol70o/D0oH70o/30ol20gl30]l30Sl20Il40?l407l311l361l811l01P1o0?l0o`6O0Sl2g`83
O`040ol3n`?f0o420n`01@?W0n<7h1_P;n0017oo0002Ool02BoP6n0Kh0OP0n<3i`?/0o43mP020o/0
0`?o0gl2g`020Sl20Il20?l307l:11l407l40?l20Il30Sl30]l20gl40ol70o/E0oH70o/20ol30gl3
0]l30Sl30Il30?l407l;11l307l20?l00`6O0Sl2g`020gl010?o0o/3mP?a0P?/00D3i`?S1n0Kh2oP
00Aoo`000Woo00T_h1_P6n07h0?S0nL3k0?a0oH00P?k00@3o`=o0gl2g`82?`81W`80o`h0O`@0o`<1
W`<2?`82g`@3O`<3o`L3naH3mPL3n`83o`D3O`82g`<2?`@1W`@0o`h0O`80o`040Il2?`8o0]l20gl0
10?o0o/3mP?a0P?/00D3i`?S1n0Kh2oP00Aoo`000Woo00T_h1_P6n07h0?S0nL3k0?a0oH00`?k00@3
o`=o0gl2g`82?`81W`L0o`D0O`D0o`<1W`@2?`82g`@3O`@3o`L3naL3mPL3n`<3o`@3O`82g`@2?`@1
W`D0o`D0O`L0o`81W`82?`070]l3O`=o0ol3n`?f0o400`?/00@3i`?S1n0Kh0Aoo`000Woo0Q_P00<7
h0?S0nL00P?/00<3l@?f0oH00P?k0P?o0P=o00<2g`8o0Sl00P6O3@3o106O1@8o0P;O10=o10?o1`?k
6@?f1P?k10?o10=o0P;O1@8o1@6O3@3o0P6O0P8o00@2g`=o0gl3o`83n`030oH3l@?a0083k0040nL3
h`OP6n04Ool0009oo`066n07h0OP0n<3i`?/0P?a0P?f0P?k0`?o0P=o00<2g`8o0Sl01`6O1@3o106O
1P8o0P;O10=o1@?o1P?k6`?f1@?k1@?o10=o0P;O1`8o106O1@3o1`6O0P8o00<2g`=o0gl00P?o0P?k
00<3mP?a0o400P?/0P?W00<3h`OPOol00goo0002Ool21n0010?S0nL3k0?/0P?a0P?f0`?k0`?o0P=o
00<2g`8o0Sl03@6O1@8o10;O10=o1@?o1P?k7@?f1@?k1@?o10=o10;O1P8o3@6O0P8o00<2g`=o0gl0
0P?o0`?k00<3mP?a0o400`?/00<3i`?S1n0017oo0002Ool010OP0n<3i`?W0P?/0P?a0P?f0`?k10?o
0P=o00<2g`8o0Sl01@8o1@6O108o1P;O0`=o1`?o1@?k7P?f1@?k1`?o0`=o1`;O108o1@6O1`8o00<2
g`=o0gl00`?o0P?k0`?f0P?a0P?/00<3i`?S1n0017oo0002Ool00`OP0n<3i`030n`20o420oH40o/4
0ol20gl00`;O0Sl2?`0:0Sl60]l50gl60ol60o/P0oH60o/60ol50gl70]l<0Sl00`;O0gl3O`030ol3
0o/30oH20o430n`00`?W0n=oo`03Ool0009oo`030n<3i`?W00<3k083l@<3mP<3n`D3o`83O`l2g`L3
O`D3o`L3nb83mPL3n`D3o`P3O`l2g`83O`@3o`<3n`<3mP83l@<3k0030nL3hgoo00=oo`000Woo00<3
h`?W0n`00P?/0`?a0`?f0`?k1P?o1`=o1P;O2@=o1@?o1`?k90?f20?k10?o2P=o1P;O1`=o10?o10?k
0`?f0`?a0P?/0P?W17oo0002Ool20nL30n`30o430oH40o/60olA0gl70ol80o/V0oH90o/70olA0gl5
0ol40o/30oH30o430n`00`?WOomoo`02Ool0009oo`040nL3k0?/0n`40o450oH50o/90ol60gl:0ol9
0o/X0oH:0o/:0ol60gl90ol40o/70oH30o420n`4Ool0009oo`83k0L3l@L3mPD3n`l3o`d3nbl3mPd3
n`l3o`D3n`X3mP<3l@030naoogoo009oo`000Woo00<3k0?a0o401P?a2@?f6P?k>@?f6@?k3`?f0P?a
17oo0002Ool:0o4;0oH?0o]90oH?0o/B0oH00`?aOomoo`02Ool0009oo`X3lHD3mPAoo`000Woo0P?f
2@?aQ0?f17oo0002Ool40oHc0o4S0oH[0o4:0oH4Ool0009oo`83n`@3mPh3lA43k1L3lAT3mQL3lA43
k0T3l@P3mP030o]oogoo009oo`000Woo10?k0`?f2P?a6`?/4`?a4P?f4`?a70?/1P?a1P?f0`?k17oo
0002Ool20ol30o/40oH50o450n`B0nL<0n`?0o4@0oH?0o4=0n`B0nL50n`40o440oH30o/00`?oOomo
o`02Ool0009oo`83o`<3n`@3mP<3l@D3k0T3i`P3h`T3i`P3k0d3l@l3mPh3l@P3k0X3i`P3h`P3i`@3
k0@3l@<3mP<3n`83o`Aoo`000Woo0`?o0P?k10?f0`?a1@?/10?W40?S20?W1P?/3@?a3`?f3@?a1`?/
20?W40?S1@?W10?/0`?a0`?f0`?k0P?o17oo0002Ool00`=o0ol3o`020o/40oH30o440n`50nL50n<8
1n060n<60nL60n`=0o4>0oH=0o460n`60nL70n<81n050n<40nL40n`30o430oH20o/30ol4Ool0009o
o`030gl3o`?o00<3n`<3mP83l@D3k0@3i`030n<7h0OP00h7h0H3h`D3i`D3k0`3l@d3mPd3l@D3k0D3
i`H3ha07h083h`@3i`@3k083l@<3mP83n`83o`030gmoogoo009oo`000Woo00@2g`=o0ol3o`83n`<3
mP83l@@3k0@3i`030n<7h1_P00/Kh0L7h0@3h`H3i`<3k0`3l@d3mP`3l@@3k0H3i`@3h`P7h0/Kh003
1n03h`?S00<3i`@3k083l@83mP<3n`83o`030gmoogoo009oo`000Woo00@2g`=o0ol3o`83n`<3mP83
l@@3k0<3i`030n<7h0OP00DKh0L_h0<Kh0L7h083h`H3i`<3k0`3l@`3mP`3l@<3k0H3i`<3h`L7h0<K
h0L_h0DKh0031n03h`?S00<3i`<3k083l@83mP83n`83o`030gl2ggoo00=oo`000Woo0P;O00@3O`?o
0o/3n`83mP<3l@<3k0<3i`040n<7h0OP6n0>;n036n061n020n<50nL40n`;0o4;0oH<0o430n`60nL3
0n<51n046n0=;n026n000`OP0n<3i`030nL20n`20o420oH20o/20ol00`=o0]moo`03Ool0009oo`03
0Sl2g`=o0083o`030o/3mP?f00<3l@83k0<3i`040n<7h0OP6n06;n07?n04;n046n041n030n<40nL4
0n`:0o4;0oH;0o440n`50nL30n<31n046n05;n07?n05;n026n000`OP0n<3i`020nL30n`00`?a0oH3
mP020o/010?o0gl2g`;O17oo0002Ool20Sl00`;O0gl3o`020o/00`?f0o43l@030n`20nL010?S1n07
h1_P0RoP3SoP12oP11_P0`OP0`?S0`?W10?/2P?a2`?f2P?a1@?/10?W0`?S0`OP0a_P1BoP3CoP0RoP
0Q_P00<7h0?S0nL00P?W0P?/00@3l@?f0oH3n`83o`030gl2g`8o00Aoo`000Woo00D1W`8o0]l3O`?o
0083n`030oH3l@?a0083k083i`040n<7h0OP6n02;n06?n07Dn04?n04;n036n031n030n<20nL50n`:
0o4:0oH:0o440n`40nL30n<31n026n05;n04?n07Dn05?n02;n026n0010OP0n<3i`?W0P?/00H3l@?f
0o/3n`?o0gl20]l00`8oOomoo`02Ool0009oo`080?l1W`8o0]l3O`?o0o/3mP83l@83k0050nL3h`OP
1n0Kh002;n02?n0>Dn04?n03;n036n031n030n<20nL50n`90o490oH:0o450n`30nL30n<31n026n04
;n03?n0>Dn02?n02;n026n001`OP0n<3i`?W0n`3l@?f0083n`050ol3O`;O0]l1W`04Ool009Eoo`00
UGoo0000\
\>"],
  ImageRangeCache->{{{0, 148.438}, {148.438, 0}} -> {-2.08002, -2.08002, 
  0.0280255, 0.0280255}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Animation", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[TextData[{
  "There is also the facility in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to be able to animate graphics which you have produced. This is done by \
creating all of the cells which will make up your movie and then asking to \
see them displayed one after the other. As an example we can see how a \
three-dimensional surface alters as we step ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " over slightly larger values. Only the first frame is shown in the book. \
Such techniques can be used to present plots of option values and their \
Greeks as functions of other variables, using the animation to represent the \
evolution through time, or perhaps to vary the angle of view, in order to \
bring out particular features."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Table[Plot3D[Sin[x+t] Cos[y+t], {x, -Pi, Pi}, {y, -Pi, Pi}],
\t{t,0,2*Pi-Pi/10,Pi/10}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02454 .26244 -12 -2.74006 ]
[.02454 .26244 0 6.25994 ]
[.0159 .31657 -24 -2.78582 ]
[.0159 .31657 0 6.21418 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00232 .43075 -18 -2.88287 ]
[-0.00232 .43075 0 6.11713 ]
[-0.01193 .49103 -6 -2.93439 ]
[-0.01193 .49103 0 6.06561 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03676 .25766 m
.04287 .25527 L
s
[(-1)] .02454 .26244 1 -0.3911 Mshowa
.02817 .3119 m
.0343 .30956 L
s
[(-0.5)] .0159 .31657 1 -0.38093 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01003 .42631 m
.01621 .4241 L
s
[(0.5)] -0.00232 .43075 1 -0.35936 Mshowa
.00046 .48671 m
.00666 .48456 L
s
[(1)] -0.01193 .49103 1 -0.34791 Mshowa
.125 Mabswid
.03507 .26836 m
.03873 .26693 L
s
.03336 .27913 m
.03703 .27771 L
s
.03164 .28997 m
.03531 .28856 L
s
.02991 .3009 m
.03358 .29949 L
s
.02641 .32298 m
.03009 .32158 L
s
.02464 .33413 m
.02833 .33274 L
s
.02286 .34537 m
.02655 .34399 L
s
.02107 .35668 m
.02476 .35531 L
s
.01744 .37956 m
.02114 .3782 L
s
.01561 .39112 m
.01931 .38977 L
s
.01377 .40277 m
.01746 .40142 L
s
.01191 .4145 m
.01561 .41316 L
s
.00815 .43822 m
.01185 .43689 L
s
.00625 .45021 m
.00996 .44889 L
s
.00433 .46229 m
.00805 .46098 L
s
.0024 .47445 m
.00612 .47315 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
0 .096 .575 r
.37901 .68206 .40169 .70185 .43838 .73689 .41587 .71254 Metetra
0 .368 .834 r
.41587 .71254 .43838 .73689 .47671 .76393 .45433 .73569 Metetra
.421 .591 .913 r
.45433 .73569 .47671 .76393 .51647 .77466 .49426 .74403 Metetra
.65 .676 .861 r
.49426 .74403 .51647 .77466 .55711 .76398 .53517 .73301 Metetra
.757 .71 .808 r
.53517 .73301 .55711 .76398 .59791 .73161 .57641 .70237 Metetra
.813 .734 .776 r
.57641 .70237 .59791 .73161 .63828 .6822 .61741 .65627 Metetra
.846 .757 .765 r
.61741 .65627 .63828 .6822 .67796 .62396 .6579 .60207 Metetra
.862 .786 .775 r
.6579 .60207 .67796 .62396 .71717 .56642 .69799 .54838 Metetra
.856 .821 .813 r
.69799 .54838 .71717 .56642 .75651 .51833 .73822 .50319 Metetra
.805 .855 .887 r
.73822 .50319 .75651 .51833 .79688 .48619 .77936 .47247 Metetra
.64 .84 .977 r
.77936 .47247 .79688 .48619 .83921 .47349 .82227 .45946 Metetra
.314 .683 .978 r
.82227 .45946 .83921 .47349 .88433 .48043 .86772 .46433 Metetra
.036 .47 .88 r
.86772 .46433 .88433 .48043 .93269 .5039 .91613 .48419 Metetra
0 .378 .836 r
.91613 .48419 .93269 .5039 .98416 .53763 .96743 .51328 Metetra
.009 .432 .863 r
.35573 .66174 .37901 .68206 .41587 .71254 .39313 .67928 Metetra
.386 .535 .882 r
.39313 .67928 .41587 .71254 .45433 .73569 .43186 .69152 Metetra
.552 .56 .816 r
.43186 .69152 .45433 .73569 .49426 .74403 .47186 .69342 Metetra
.637 .582 .776 r
.47186 .69342 .49426 .74403 .53517 .73301 .51282 .68196 Metetra
.695 .61 .758 r
.51282 .68196 .53517 .73301 .57641 .70237 .55429 .65697 Metetra
.742 .649 .757 r
.55429 .65697 .57641 .70237 .61741 .65627 .59585 .62119 Metetra
.786 .705 .774 r
.59585 .62119 .61741 .65627 .6579 .60207 .63727 .57956 Metetra
.826 .787 .814 r
.63727 .57956 .6579 .60207 .69799 .54838 .67861 .53802 Metetra
.839 .902 .882 r
.67861 .53802 .69799 .54838 .73822 .50319 .72019 .50222 Metetra
.693 .975 .914 r
.72019 .50222 .73822 .50319 .77936 .47247 .76255 .47648 Metetra
.245 .752 .721 r
.76255 .47648 .77936 .47247 .82227 .45946 .80632 .4632 Metetra
.104 0 0 r
.80632 .4632 .82227 .45946 .86772 .46433 .85203 .4625 Metetra
.183 0 0 r
.85203 .4625 .86772 .46433 .91613 .48419 .90001 .47224 Metetra
.003 .499 .865 r
.90001 .47224 .91613 .48419 .96743 .51328 .95021 .48822 Metetra
.515 .706 .958 r
.33183 .64088 .35573 .66174 .39313 .67928 .37019 .63963 Metetra
.587 .599 .831 r
.37019 .63963 .39313 .67928 .43186 .69152 .40942 .6363 Metetra
.611 .554 .766 r
.40942 .6363 .43186 .69152 .47186 .69342 .44952 .62915 Metetra
.628 .543 .739 r
.44952 .62915 .47186 .69342 .51282 .68196 .49037 .61719 Metetra
.647 .553 .735 r
.49037 .61719 .51282 .68196 .55429 .65697 .53183 .60035 Metetra
.671 .585 .751 r
.53183 .60035 .55429 .65697 .59585 .62119 .57376 .57953 Metetra
.702 .65 .795 r
.57376 .57953 .59585 .62119 .63727 .57956 .61607 .55642 Metetra
.739 .774 .882 r
.61607 .55642 .63727 .57956 .67861 .53802 .65876 .5331 Metetra
.68 .946 .96 r
.65876 .5331 .67861 .53802 .72019 .50222 .70195 .51163 Metetra
.70195 .51163 .72019 .50222 .76255 .47648 .74584 .49368 Metetra
.135 0 0 r
.74584 .49368 .76255 .47648 .80632 .4632 .79067 .48017 Metetra
.221 0 0 r
.79067 .48017 .80632 .4632 .85203 .4625 .83666 .47115 Metetra
.124 0 0 r
.83666 .47115 .85203 .4625 .90001 .47224 .88393 .46577 Metetra
.288 .761 .936 r
.88393 .46577 .90001 .47224 .95021 .48822 .93247 .4624 Metetra
.739 .766 .875 r
.30728 .61945 .33183 .64088 .37019 .63963 .34698 .59729 Metetra
.689 .635 .79 r
.34698 .59729 .37019 .63963 .40942 .6363 .38696 .57697 Metetra
.652 .566 .746 r
.38696 .57697 .40942 .6363 .44952 .62915 .42723 .56011 Metetra
.625 .532 .728 r
.42723 .56011 .44952 .62915 .49037 .61719 .46792 .54762 Metetra
.606 .52 .732 r
.46792 .54762 .49037 .61719 .53183 .60035 .50921 .53956 Metetra
.591 .533 .758 r
.50921 .53956 .53183 .60035 .57376 .57953 .55128 .53509 Metetra
.576 .582 .822 r
.55128 .53509 .57376 .57953 .61607 .55642 .59425 .53262 Metetra
.533 .704 .949 r
.59425 .53262 .61607 .55642 .65876 .5331 .63817 .53009 Metetra
.241 .749 .878 r
.63817 .53009 .65876 .5331 .70195 .51163 .68297 .52537 Metetra
.143 0 0 r
.68297 .52537 .70195 .51163 .74584 .49368 .72849 .51667 Metetra
.214 0 0 r
.72849 .51667 .74584 .49368 .79067 .48017 .77451 .50294 Metetra
.136 0 0 r
.77451 .50294 .79067 .48017 .83666 .47115 .82084 .48413 Metetra
.82084 .48413 .83666 .47115 .88393 .46577 .86739 .46117 Metetra
.668 .963 .933 r
.86739 .46117 .88393 .46577 .93247 .4624 .9142 .43581 Metetra
.817 .767 .806 r
.28205 .59743 .30728 .61945 .34698 .59729 .3233 .55633 Metetra
.751 .666 .767 r
.3233 .55633 .34698 .59729 .38696 .57697 .36419 .52092 Metetra
.688 .594 .745 r
.36419 .52092 .38696 .57697 .42723 .56011 .40478 .49553 Metetra
.628 .543 .739 r
.40478 .49553 .42723 .56011 .46792 .54762 .44536 .48254 Metetra
.566 .507 .748 r
.44536 .48254 .46792 .54762 .50921 .53956 .48642 .48208 Metetra
.491 .486 .779 r
.48642 .48208 .50921 .53956 .55128 .53509 .52844 .492 Metetra
.383 .488 .842 r
.52844 .492 .55128 .53509 .59425 .53262 .5718 .50812 Metetra
.172 .509 .912 r
.5718 .50812 .59425 .53262 .63817 .53009 .61665 .52479 Metetra
0 .429 .692 r
.61665 .52479 .63817 .53009 .68297 .52537 .66281 .53588 Metetra
.253 0 0 r
.66281 .53588 .68297 .52537 .72849 .51667 .70981 .53604 Metetra
.138 0 0 r
.70981 .53604 .72849 .51667 .77451 .50294 .75701 .52208 Metetra
.75701 .52208 .77451 .50294 .82084 .48413 .80382 .49382 Metetra
.589 .911 .664 r
.80382 .49382 .82084 .48413 .86739 .46117 .84991 .45417 Metetra
.867 .973 .855 r
.84991 .45417 .86739 .46117 .9142 .43581 .89536 .40839 Metetra
.846 .762 .77 r
.25611 .57479 .28205 .59743 .3233 .55633 .29883 .52034 Metetra
.793 .699 .761 r
.29883 .52034 .3233 .55633 .36419 .52092 .34069 .47446 Metetra
.726 .641 .763 r
.34069 .47446 .36419 .52092 .40478 .49553 .38172 .44317 Metetra
.639 .582 .774 r
.38172 .44317 .40478 .49553 .44536 .48254 .42234 .42971 Metetra
.52 .518 .794 r
.42234 .42971 .44536 .48254 .48642 .48208 .46325 .43427 Metetra
.35 .445 .817 r
.46325 .43427 .48642 .48208 .52844 .492 .50516 .45393 Metetra
.114 .36 .823 r
.50516 .45393 .52844 .492 .5718 .50812 .54869 .4829 Metetra
0 .285 .771 r
.54869 .4829 .5718 .50812 .61665 .52479 .5941 .51313 Metetra
0 .288 .664 r
.5941 .51313 .61665 .52479 .66281 .53588 .64119 .53556 Metetra
.162 0 0 r
.64119 .53556 .66281 .53588 .70981 .53604 .68932 .54211 Metetra
.68932 .54211 .70981 .53604 .75701 .52208 .73755 .52784 Metetra
.635 .957 .781 r
.73755 .52784 .75701 .52208 .80382 .49382 .78496 .49249 Metetra
.874 .992 .81 r
.78496 .49249 .80382 .49382 .84991 .45417 .83106 .44059 Metetra
.919 .912 .796 r
.83106 .44059 .84991 .45417 .89536 .40839 .87594 .38012 Metetra
.853 .76 .759 r
.22944 .55152 .25611 .57479 .29883 .52034 .27324 .49183 Metetra
.824 .739 .771 r
.27324 .49183 .29883 .52034 .34069 .47446 .31592 .44184 Metetra
.767 .714 .803 r
.31592 .44184 .34069 .47446 .38172 .44317 .35748 .40814 Metetra
.657 .669 .85 r
.35748 .40814 .38172 .44317 .42234 .42971 .39837 .39426 Metetra
.448 .575 .889 r
.39837 .39426 .42234 .42971 .46325 .43427 .43935 .40041 Metetra
.133 .408 .855 r
.43935 .40041 .46325 .43427 .50516 .45393 .48128 .42342 Metetra
0 .243 .753 r
.48128 .42342 .50516 .45393 .54869 .4829 .52489 .45693 Metetra
0 .199 .701 r
.52489 .45693 .54869 .4829 .5941 .51313 .57054 .49201 Metetra
0 .325 .778 r
.57054 .49201 .5941 .51313 .64119 .53556 .61809 .51851 Metetra
.192 .616 .946 r
.61809 .51851 .64119 .53556 .68932 .54211 .66687 .52725 Metetra
.608 .857 .991 r
.66687 .52725 .68932 .54211 .73755 .52784 .71585 .51258 Metetra
.823 .904 .896 r
.71585 .51258 .73755 .52784 .78496 .49249 .76397 .47419 Metetra
.894 .879 .813 r
.76397 .47419 .78496 .49249 .83106 .44059 .81064 .41729 Metetra
.908 .846 .771 r
.81064 .41729 .83106 .44059 .87594 .38012 .8559 .35095 Metetra
.846 .762 .77 r
.202 .52757 .22944 .55152 .27324 .49183 .24622 .47179 Metetra
.842 .792 .804 r
.24622 .47179 .27324 .49183 .31592 .44184 .2894 .42473 Metetra
.8 .827 .872 r
.2894 .42473 .31592 .44184 .35748 .40814 .33149 .39244 Metetra
.652 .831 .969 r
.33149 .39244 .35748 .40814 .39837 .39426 .3729 .3782 Metetra
.275 .671 .97 r
.3729 .3782 .39837 .39426 .43935 .40041 .41432 .38218 Metetra
0 .379 .796 r
.41432 .38218 .43935 .40041 .48128 .42342 .45657 .40145 Metetra
0 .238 .708 r
.45657 .40145 .48128 .42342 .52489 .45693 .50036 .43016 Metetra
0 .285 .771 r
.50036 .43016 .52489 .45693 .57054 .49201 .5461 .46015 Metetra
.138 .445 .879 r
.5461 .46015 .57054 .49201 .61809 .51851 .59373 .48224 Metetra
.443 .595 .907 r
.59373 .48224 .61809 .51851 .66687 .52725 .6427 .4882 Metetra
.646 .68 .868 r
.6427 .4882 .66687 .52725 .71585 .51258 .69208 .473 Metetra
.761 .725 .819 r
.69208 .473 .71585 .51258 .76397 .47419 .74093 .43637 Metetra
.827 .756 .784 r
.74093 .43637 .76397 .47419 .81064 .41729 .78865 .38294 Metetra
.865 .784 .77 r
.78865 .38294 .81064 .41729 .8559 .35095 .83521 .32084 Metetra
.817 .767 .806 r
.17377 .50293 .202 .52757 .24622 .47179 .21754 .45953 Metetra
.831 .861 .868 r
.21754 .45953 .24622 .47179 .2894 .42473 .26075 .422 Metetra
.757 .955 .938 r
.26075 .422 .2894 .42473 .33149 .39244 .30331 .39474 Metetra
.429 .871 .878 r
.30331 .39474 .33149 .39244 .3729 .3782 .34547 .38017 Metetra
.34547 .38017 .3729 .3782 .41432 .38218 .38776 .37843 Metetra
.183 0 0 r
.38776 .37843 .41432 .38218 .45657 .40145 .43078 .38733 Metetra
0 .432 .794 r
.43078 .38733 .45657 .40145 .50036 .43016 .47508 .40258 Metetra
.172 .509 .912 r
.47508 .40258 .50036 .43016 .5461 .46015 .52099 .41838 Metetra
.408 .541 .878 r
.52099 .41838 .5461 .46015 .59373 .48224 .56851 .42841 Metetra
.544 .559 .82 r
.56851 .42841 .59373 .48224 .6427 .4882 .61731 .42716 Metetra
.629 .582 .782 r
.61731 .42716 .6427 .4882 .69208 .473 .66677 .41132 Metetra
.693 .615 .764 r
.66677 .41132 .69208 .473 .74093 .43637 .71625 .38072 Metetra
.747 .66 .764 r
.71625 .38072 .74093 .43637 .78865 .38294 .7653 .33839 Metetra
.797 .723 .783 r
.7653 .33839 .78865 .38294 .83521 .32084 .81384 .28975 Metetra
.739 .766 .875 r
.1447 .47755 .17377 .50293 .21754 .45953 .18717 .45274 Metetra
.721 .922 .96 r
.18717 .45274 .21754 .45953 .26075 .422 .22986 .42981 Metetra
.447 .88 .813 r
.22986 .42981 .26075 .422 .30331 .39474 .27272 .41044 Metetra
.27272 .41044 .30331 .39474 .34547 .38017 .31584 .39559 Metetra
.031 0 0 r
.31584 .39559 .34547 .38017 .38776 .37843 .35944 .3853 Metetra
.35944 .3853 .38776 .37843 .43078 .38733 .40375 .37869 Metetra
.336 .781 .96 r
.40375 .37869 .43078 .38733 .47508 .40258 .449 .37412 Metetra
.533 .704 .949 r
.449 .37412 .47508 .40258 .52099 .41838 .49535 .36944 Metetra
.581 .599 .836 r
.49535 .36944 .52099 .41838 .56851 .42841 .54282 .36241 Metetra
.6 .551 .771 r
.54282 .36241 .56851 .42841 .61731 .42716 .59131 .35117 Metetra
.618 .538 .742 r
.59131 .35117 .61731 .42716 .66677 .41132 .64063 .33464 Metetra
.639 .55 .738 r
.64063 .33464 .66677 .41132 .71625 .38072 .69056 .31273 Metetra
.665 .586 .757 r
.69056 .31273 .71625 .38072 .7653 .33839 .74095 .28645 Metetra
.701 .657 .804 r
.74095 .28645 .7653 .33839 .81384 .28975 .79176 .25762 Metetra
.515 .706 .958 r
.11475 .45142 .1447 .47755 .18717 .45274 .15526 .44783 Metetra
.32 .789 .93 r
.15526 .44783 .18717 .45274 .22986 .42981 .19695 .44197 Metetra
.19695 .44197 .22986 .42981 .27272 .41044 .2399 .43201 Metetra
.2399 .43201 .27272 .41044 .31584 .39559 .28409 .41688 Metetra
.28409 .41688 .31584 .39559 .35944 .3853 .32934 .39651 Metetra
.447 .87 .747 r
.32934 .39651 .35944 .3853 .40375 .37869 .37541 .37188 Metetra
.729 .946 .952 r
.37541 .37188 .40375 .37869 .449 .37412 .42209 .34476 Metetra
.739 .774 .882 r
.42209 .34476 .449 .37412 .49535 .36944 .46925 .31736 Metetra
.687 .64 .798 r
.46925 .31736 .49535 .36944 .54282 .36241 .51689 .29186 Metetra
.644 .564 .75 r
.51689 .29186 .54282 .36241 .59131 .35117 .56515 .27 Metetra
.614 .525 .73 r
.56515 .27 .59131 .35117 .64063 .33464 .61425 .25274 Metetra
.593 .513 .733 r
.61425 .25274 .64063 .33464 .69056 .31273 .66445 .24012 Metetra
.579 .527 .761 r
.66445 .24012 .69056 .31273 .74095 .28645 .71598 .23124 Metetra
.569 .581 .826 r
.71598 .23124 .74095 .28645 .79176 .25762 .76894 .22439 Metetra
.009 .432 .863 r
.08389 .42449 .11475 .45142 .15526 .44783 .1222 .44049 Metetra
.195 0 0 r
.1222 .44049 .15526 .44783 .19695 .44197 .16262 .45076 Metetra
.138 0 0 r
.16262 .45076 .19695 .44197 .2399 .43201 .2055 .44981 Metetra
.2055 .44981 .2399 .43201 .28409 .41688 .25074 .43438 Metetra
.465 .864 .685 r
.25074 .43438 .28409 .41688 .32934 .39651 .29779 .40427 Metetra
.784 .998 .846 r
.29779 .40427 .32934 .39651 .37541 .37188 .34588 .3625 Metetra
.869 .919 .859 r
.34588 .3625 .37541 .37188 .42209 .34476 .39431 .31445 Metetra
.826 .787 .814 r
.39431 .31445 .42209 .34476 .46925 .31736 .44262 .26652 Metetra
.756 .677 .774 r
.44262 .26652 .46925 .31736 .51689 .29186 .49075 .22471 Metetra
.686 .596 .75 r
.49075 .22471 .51689 .29186 .56515 .27 .539 .19359 Metetra
.618 .538 .742 r
.539 .19359 .56515 .27 .61425 .25274 .58796 .17562 Metetra
.552 .499 .751 r
.58796 .17562 .61425 .25274 .66445 .24012 .6383 .17091 Metetra
.482 .482 .78 r
.6383 .17091 .66445 .24012 .71598 .23124 .69065 .17719 Metetra
.401 .495 .84 r
.69065 .17719 .71598 .23124 .76894 .22439 .74532 .19002 Metetra
0 .096 .575 r
.05208 .39672 .08389 .42449 .1222 .44049 .08852 .42652 Metetra
.414 0 0 r
.08852 .42652 .1222 .44049 .16262 .45076 .1278 .44836 Metetra
.13 0 0 r
.1278 .44836 .16262 .45076 .2055 .44981 .17056 .45391 Metetra
.394 .852 .852 r
.17056 .45391 .2055 .44981 .25074 .43438 .21669 .4381 Metetra
.773 .989 .892 r
.21669 .4381 .25074 .43438 .29779 .40427 .26538 .40065 Metetra
.889 .94 .841 r
.26538 .40065 .29779 .40427 .34588 .3625 .31541 .34626 Metetra
.896 .861 .798 r
.31541 .34626 .34588 .3625 .39431 .31445 .36562 .28314 Metetra
.862 .786 .775 r
.36562 .28314 .39431 .31445 .44262 .26652 .41526 .22074 Metetra
.805 .716 .767 r
.41526 .22074 .44262 .26652 .49075 .22471 .46415 .16763 Metetra
.728 .649 .77 r
.46415 .16763 .49075 .22471 .539 .19359 .51271 .13009 Metetra
.631 .582 .78 r
.51271 .13009 .539 .19359 .58796 .17562 .56174 .11144 Metetra
.511 .515 .796 r
.56174 .11144 .58796 .17562 .6383 .17091 .61223 .11182 Metetra
.369 .453 .816 r
.61223 .11182 .6383 .17091 .69065 .17719 .66509 .12818 Metetra
.214 .41 .839 r
.66509 .12818 .69065 .17719 .74532 .19002 .72088 .15445 Metetra
.537 .015 0 r
.01926 .36808 .05208 .39672 .08852 .42652 .0548 .40279 Metetra
0 .168 .606 r
.0548 .40279 .08852 .42652 .1278 .44836 .09353 .42872 Metetra
.116 .582 .916 r
.09353 .42872 .1278 .44836 .17056 .45391 .13631 .43646 Metetra
.627 .848 .984 r
.13631 .43646 .17056 .45391 .21669 .4381 .18308 .42016 Metetra
.819 .866 .881 r
.18308 .42016 .21669 .4381 .26538 .40065 .23291 .37953 Metetra
.874 .836 .806 r
.23291 .37953 .26538 .40065 .31541 .34626 .28435 .31996 Metetra
.883 .808 .77 r
.28435 .31996 .31541 .34626 .36562 .28314 .33597 .25079 Metetra
.871 .784 .763 r
.33597 .25079 .36562 .28314 .41526 .22074 .38683 .18264 Metetra
.837 .76 .778 r
.38683 .18264 .41526 .22074 .46415 .16763 .43665 .12502 Metetra
.772 .728 .812 r
.43665 .12502 .46415 .16763 .51271 .13009 .48583 .08476 Metetra
.653 .673 .856 r
.48583 .08476 .51271 .13009 .56174 .11144 .53526 .06546 Metetra
.465 .578 .883 r
.53526 .06546 .56174 .11144 .61223 .11182 .58604 .06725 Metetra
.239 .456 .869 r
.58604 .06725 .61223 .11182 .66509 .12818 .63924 .08684 Metetra
.062 .362 .832 r
.63924 .08684 .66509 .12818 .72088 .15445 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0000=oo`0000002P003goo00<007ooOol0
0goo00<007ooOol02Goo00<007ooOol077oo001SOol600000e:k000jG006>U`00`007M/Mf`03000<
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00Oomoo`0KOol004Moo`8001Qoo`D000EB
^`03000jG3YL00DjG003000MfagK008Mf`<000Uoo`03001oogoo00Aoo`03001oogoo00Yoo`03001o
ogoo01Yoo`00AWoo00@007ooOol001Aoo`D000QB^`03000jG3YL00DjG003000MfagK00DMf`8000Mo
o`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000LOol004Ioo`04001oogoo000BOol20002
Ool00`00H_T00009D[/00`00>U`jG005>U`00`007M/Mf`047M/010001GX000001Goo00<007ooOol0
17oo0P002Goo0P007Goo0016Ool01000Oomoo`003goo0`0000Aoo`00Ool000=Rn@03001B^e:k00MB
^`03000jG3YL00DjG003000MfagK00@Mf`030005NPEj008000=oo`03001oogoo00Ioo`03001oogoo
02Eoo`00AWoo00@007ooOol000eoo`8000Aoo`8000ERn@03001B^e:k00IB^`03000jG3YL00DjG003
000MfagK00@Mf`030005NPEj0085NP04001oogoo0009Ool00`00Oomoo`0TOol004Moo`8000]oo`<0
00Eoo`03001RnF;i00IRn@03001B^e:k00EB^``000@Mf`030005NPEj0085NP<000Uoo`03001oogoo
02Aoo`00DWoo0`001Woo00<006;iH_T01f;i00<005:kD[/00U:k0`0000=BF000@QT01T8I00<002gJ
;MX00`000QgK00<000Ej1GX00`Ej00<007ooOol027oo00<007ooOol08goo001?Ool30008Ool00`00
H_URn@09H_T01000D[/000001E9H00<0048I@QT01D8I00<002gJ;MX00RgJ0`001@Ej00<007ooOol0
2Goo00<007ooOol08Woo001<Ool3000;Ool00`00H_URn@09H_T20007DUP00`00@QU26@05@QT00`00
;MX]fP05;MX200031GX2000:Ool00`00Oomoo`0ROol004Yoo`<000aoo`03001[600000URn@03001N
V00000MBF00300126D8I00I26@03000]fRgJ00@]fP08000I^P000005NP001GX000Yoo`03001oogoo
025oo`00Agoo0`003Woo00D006/HJaQ[600000MRn@05001NV5jHGYP00007DUP00`00@QU26@05@QT0
0`00;MX]fP05;MX00`006KXI^P04000;Ool00`00Oomoo`0POol004Eoo`80011oo`03001[66/H00=[
6003001RnF;i009Rn@8000ANV003001BF59H00EBF00300126D8I00E26@03000]fRgJ00D]fP06000I
^QVj6KX001Vj0P002Woo00<007ooOol087oo0012Ool3000AOol00`00JaQ[6004JaP01@00H_URnF;i
00001UjH00<0059HDUP01U9H00<0048I@QT01D8I00<002gJ;MX01BgJ00@001Vj6KX000<I^P03001o
ogoo00Qoo`03001oogoo01moo`009Goo10005goo0P004goo00<006/HJaP01V/H00<006;i000025jH
00<0059HDUP01E9H00<0048I@QT01D8I00<002gJ;MX01RgJ00<001Vj00000aVj00<007ooOol02Goo
00<007ooOol07Woo000VOol00`00Oomoo`0DOol3000DOol00`00JaQ[6008JaP00`00GYQNV007GYP0
0`00DUQBF005DUP00`00@QU26@02@QT:0002;MX00`006KX000046KX00`00Oomoo`07Ool2000POol0
01moo`D000=oo`03001oogoo015oo`8001Ioo`03001[66/H00Q[608000YNV003001BF59H00ABF0D0
00A5n003000mn3gh00<mn0H000@I^P03001oogoo00Qoo`03001oogoo01eoo`00:7oo00<007ooOol0
3Goo0`005goo00<006/HJaP026/H00@006KHI]P000UNV004001BF59HDUP300000dhg0015n007AOP0
0`00?OPmn005?OP200000c7j000000040009Ool00`00Oomoo`0MOol002Eoo`04001oogoo000=Ool2
000JOol00`00JaQ[6007JaP01@00I]QVf6KH00002EjH00@0059H000000E>=`030015n4Gh00E5n003
000mn3gh00Hmn08000DanP8000Uoo`03001oogoo01aoo`009Woo0P002goo0`006goo00<006lh0000
1f/H00<006KHI]P00fKH00<005jHGYP01UjH0P001dhg00<004GhAOP01TGh00<003gh?OP01Cgh0P00
1S7j00<007ooOol027oo00<007ooOol06goo000aOol200000goo001oo`0JOol01@00KcQ_>6lh0000
1F/H00<006KHI]P016KH00<005jHGYP01EjH00<005Ig00001dhg00<004GhAOP01TGh00<003gh?OP0
1Sgh00<0037j<OX01C7j00<007ooOol01goo00<007ooOol06goo000^Ool3000NOol00`00KcQ_>003
KcP01@00JaQ[66/H00001fKH00<005jHGYP00ejH0P000eIg00<004hgCSL01Dhg00<004GhAOP01TGh
00<003gh?OP01Cgh00<0037j<OX01C7j00<007ooOol01goo0P0077oo000[Ool3000QOol00`00KcQ_
>004KcP00`00JaP00009I]P00`00GYQNV0020005EWL00`00CSM>=`05CSL00`00AOQ5n006AOP00`00
?OPmn005?OP00`00<OXanP06<OX00`00Oomoo`08Ool00`00Oomoo`03Ool2000DOol002Uoo`80009o
o`03001oogoo01ioo`03001_>6lh00I_>003001Vf6KH00QVf003001NV00000MFM`03001>=dhg00I>
=`030015n4Gh00E5n003000mn3gh00Dmn08000LanP03001oogoo00Qoo`03001oogoo009oo`04001o
ogoo000COol002Ioo`<002Aoo`03001_>6lh00M_>003001Vf6KH00QVf08000UFM`03001>=dhg00E>
=`030015n4Gh00I5n003000mn3gh00@mn08000@anPD000Yoo`05001oogooOol00002Ool00`00Oomo
o`0AOol002Aoo`8002Moo`03001_>6lh00I_>003001_>00000QVf003001R^00000UFM`03001>=dhg
00I>=`030015n4Gh00E5n003000mn3gh008mn0P000A:FP03001oogoo00Uoo`04001oogoo0002Ool0
0`00Oomoo`0AOol0025oo`<002Qoo`03001_>6lh00I_>004001_>6lh0007I]P01@00H[QR^6:h0000
25Ig00<004hgCSL01Thg00<004GhAOP014Gh1P000TXH00<004XH000024YJ00<007ooOol01goo0P00
0Woo00@007ooOol001=oo`007goo0P00:Woo00<006lhKcP01Vlh00<006lhKcP00Vlh00<006KHI]P0
0fKH00<006:hH[P00V:h00<005IgEWL01eIg00<004hgCSL01Dhg00<004GhAOP010000TXG00<004XH
BQP00TXH0P000TXH00<004YJBUX01TYJ00<007ooOol027oo00<007ooOol00P0057oo000LOol3000[
Ool00`00KcQ_>006KcP00`00KcQ_>003KcP00`00I]QVf002I]P00`00H[QR^003H[P00`00EWMFM`07
EWL00`00CSM>=`05CSL30007BQL400001DXH001:64XH00002DYJ00<007ooOol027oo00<007ooOol0
5Goo000KOol2000]Ool20007KcP00`00KcQ_>004KcP01000I]QVf0001f:h00<005IgEWL01UIg00<0
04hgCSL00Thg0`0000=>5`00BQL01DXG0`001DXH00@004XHBQP000Q:FP8000Uoo`03001oogoo01Eo
o`0057oo0`0017oo00<007oo0000:goo00@007<hLcP000E_>003001_>6lh00E_>003001Vf00000QR
^003001FMeIg00MFM`@000A>5`04001:5dXGBQL30002BQL00`00BQQ:6004BQP00`00BQP00007BUX0
0`00F[T0000:Ool00`00Oomoo`0DOol001Eoo`03001oogoo00=oo`04001oogoo000YOol00`00LcQc
>002LcP01@00KcQ_>6lh00002Flh00<006:hH[P01f:h00<005IgEWL01UIg0P0024hG0`001TXG00<0
04XHBQP014XH0P001DYJ0P000eZi00<007ooOol01goo0P005Goo000>Ool50002Ool00`00Oomoo`03
Ool01@00Oomoogoo00009goo00<007<hLcP017<h00<006lh00002Flh0P002V:h00<005IgEWL00eIg
0P0000=BE`00CQL014hG100024XG00<004XHBQP014XH00<004XH00000TYJ0P001UZi00<007ooOol0
1goo00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0TOol00`00
LcQc>005LcP00`00KcQ_>007KcP01000JcY[>P002F:h00@005IgEWMFM`8000=BE`03001>5dhG00<0
00=>5`03001:5dXG00I:5`03001:64XH00E:60800003BUX005Zi00MJ^@03001oogoo00Qoo`03001o
ogoo019oo`005Goo00<007ooOol00Woo0P001Goo00<007ooOol08Woo00<007<hLcP01W<h0P001flh
00<006/jJcX00V/j00<006:hH[P01V:h00@005Ig000000EBE`<000M>5`03001:5dXG00I:5`03001:
64XH00E:608000YJ^@03001oogoo00Moo`03001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo
00<007ooOol087oo00<007<hLcP01W<h00@007=iLgT000E_>003001[>V/j00=[>P03001R^6:h00IR
^08000EBE`800003DUL004hG00M>5`03001:5dXG00I:5`03001:64XH00=:60@000YJ^@03001oogoo
00Qoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol07goo00<007<hLcP01G<h00<0
07=iLgT00W=i00D006lhKcQ_>00000M[>P03001R^6:h00=R^0800004EYT0059GDUL30003DUL00`00
CQM>5`06CQL00`00BQM:5`06BQL01000BQQ:64XH0P000eIH0P0025Zi00<006Ki00002Woo00<007oo
Ool047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0MOol00`00LcQc>005LcP00`00LgUcN@04LgT0
0`00KcP00008JcX00`00H[QR^002H[P01000EYUFVEJI0`001e9G00<004hGCQL01ThG00<004XGBQL0
1TXG0`001UIH00<005ZiF[T01EZi00D006KiI_UVn@0000Qoo`80019oo`0027oo0P0017oo0P000goo
0`000goo0P002Woo00<007ooOol06goo00<007<hLcP01G<h00<007=iLgT01W=i00<006/jJcX026/j
00D006:h000005JI00<00003EYT0059G00MBE`03001>5dhG00I>5`03001:5dXG00E:5`8000UFF003
001J^EZi00=J^@03001VnFKi00=Vn@03001oogoo00Moo`03001oogoo00moo`001goo00@007ooOol0
00=oo`8000Ioo`03001oogoo008000]oo`03001oogoo01Uoo`8000Mc>003001cNG=i00EcN@8000Y[
>PD000AFV@03001BEe9G00IBE`03001>5dhG00I>5`03001:5dXG009:5`<00003DSL005IH00QFF003
001J^EZi009J^@03001VnFKi00EVn@03001oogoo00Moo`03001oogoo00ioo`0000=oo`0000000`00
00Eoo`00Oomoo`0000]oo`04001oogoo000=Ool00`00Oomoo`0HOol00`00KeT00005LcP00`00LgUc
N@05LgT01000Kk]_^`001f/j100025JI00<0059GDUL01U9G00<004hGCQL01ThG10001E8g00<005IH
EUP01eIH00@005ZiF[T000QVn@03001oogoo00Moo`03001oogoo00ioo`001goo00@007ooOol000Qo
o`<000=oo`03001oogoo00aoo`03001oogoo01Ioo`05001_FFmIKeT00003LcP00`00LgUcN@05LgT0
0`00Kk]_^`02Kk/00`00JcY[>P02JcX30003Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL00`00CQM>
5`05CQL20008DSL00`00EUQFF007EUP30008I_T00`00KcP00009Ool00`00Oomoo`0=Ool000Moo`04
001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool2000EOol00`00KeU_F@03KeT00`00LcP0
0007LgT00`00Kk]_^`04Kk/40006Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL00`00CQM>5`02CQL3
00000dhg001B=`07DSL00`00EUQFF007EUP20008I_T01@00KcQ_>6lh00002Goo00<007ooOol00Woo
10001Woo0008Ool20009Ool40002Ool00`00Oomoo`0?Ool00`00Oomoo`0AOol00`00KeU_F@05KeT2
0006LgT00`00Kk]_^`04Kk/30009Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL40004CSL00`00DSMB
=`07DSL00`00EUQFF005EUP01000G[P000001VKi00<006lhKcP00Vlh00<007ooOol01goo00<007oo
Ool00goo00<007ooOol01Woo000IOol00`00Oomoo`0@Ool00`00Oomoo`0?Ool00`00KeU_F@06KeT0
1000LmX000000g=i00<006nkKk/00Vnk100000=OgP00Ga`025lL00<005JIEYT01EJI00<0059GDUL0
159G0`0024hg00<0058gDSL01U8g00<005IHEUP015IH00@005jhG[QN^08000AVn@03001_>6lh00A_
>003001oogoo00Moo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P004goo00<007ooOol03Woo
00<006mIKeT01FmI00<007?JLmX00W?J00<007=i00000Vnk0`0000=_^`00Gmh00eoN00<005lLGa`0
1elL00<005JIEYT015JI00<0059GDUL010000Thh00<004hgCSL01Thg00<0058gDSL01U8g00<005IH
EUP00eIH00<005jhG[P00Ujh00<006Ki00000fKi00<006lhKcP016lh00<007ooOol027oo00<007oo
Ool00goo00<007ooOol017oo000HOol2000DOol00`00Oomoo`0<Ool00`00KeU_F@05KeT00`00LmYc
fP03LmX50002Kk/20006Gmh00`00GaaO7006Ga`00`00EYUFV@02EYT50007CSP00`00CSM>=`05CSL0
0`00DSMB=`07DSL01@00EUQFF5IH00001Ujh0P000VKi00<006lhKcP016lh00<007Mh000027oo0P00
0Woo00@007ooOol000Ioo`004Woo0P0017oo0P005Goo00<007ooOol02Woo00<006mIKeT01FmI00<0
07?JLmX00g?J0`000fOk10001eoN00<005lLGa`00UlL20000dYj00<004hhCSP01Thh00<004hgCSL0
1Thg00<0058gDSL01U8g00@005IHEUP000MN^004001Vn@000006KcP01@00MgQgN7Mh00002Goo00<0
07ooOol00P001goo000AOol01000Oomoo`000goo00<007oo00005Goo00<007ooOol027oo00<006mI
KeT01FmI00@007?JLmYcfP<000AWn`80008kM`h000A6gP03001:NTYj00E:NP03001>>4hh00E>>003
001>=dhg00I>=`03001B=e8g00IB=`03001FF00000QN^004001Vn@000006KcP01@00MgQgN7Mh0000
2Woo00<007ooOol027oo000AOol01000Oomoo`000goo00@007ooOol001Eoo`03001oogoo00Eoo`<0
00I_F@03001cfP0000@000=Wn`L0008R_`@000D[7P030016gTKN00A6gP03001:NTYj00A:NP03001>
>4hh00I>>003001>=dhg00E>=`03001B=e8g00IB=`8000YN^004001_>6lh0004KcP00`00MgQgN003
MgP00`00Oomoo`07Ool00`00Oomoo`08Ool0015oo`04001oogoo0002Ool20003Ool00`00Oomoo`0C
Ool00`00Oomoo`03Ool01000Jga[O6]l0P000fmI0`000W?J00@006OkIo]Wn`<000ICG`03000R_b:o
00<R_`<000<[7P030016gTKN00=6gP03001:NTYj00A:NP03001>>4hh00I>>003001>=dhg00I>=`03
001B=e8g00AB=`03001JN00000YN^004001_>6lh0003KcP00`00MgQgN004MgP00`00Oomoo`08Ool0
0`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00Oomoo`02Ool2000DOol01@00Oomoogoo0000
1V]l10000g?J00<006Ok000010001e=O1P0000<R_`0000001@001DKN00<004YjBWX014Yj00<004hh
CSP01Dhh00<004hgCSL01Thg00<0058gDSL00U8g0P000eYh00<005jhG[P01Ujh00<006/I00000Vlh
00@006lhKcP000EgN003001g^@0000Uoo`03001oogoo00Moo`004Woo0P000goo0P001Goo0P0057oo
0`001F]l3@000fMK0P000U=O0`0000=F^`00>U`00cYL100000<1V@0000000`000dKN00<004YjBWX0
14Yj00<004hhCSP01Thh00<004hgCSL01Dhg00<0058gDSL00P001EYh00<005jhG[P01Ejh00D006/I
JaU[6@00009_>08000EgN005001g^GNiMkT00009Ool00`00Oomoo`06Ool001Moo`03001oogoo00Eo
o`8001=oo`03001[O6]l009[O0<00003Jg`006?l00=So0<00003Hmh0000000EWF`<000EF^`03000j
G3YL00<jG003000A^`0000@000065Qd000000016gP001TYj00<004hhCSP01Dhh00<004hgCSL01Thg
0P0025Yh00<005jhG[P00ejh00<006/IJaT00V/I00<006lhKcP00P0017Mh00<007NiMkT00gNi00<0
07ooOol01goo00<007ooOol01Goo000GOol00`00Oomoo`06Ool3000@Ool00`00Jga[O0030003Jg`0
1000HoaSo6?l0`000V?N00<006MKIe/016MK00<006;I00001eJk00<003YL>U`00cYL00<0016k4K/0
10000aHM0P001TYj00<004hhCSP01Dhh00<004hgCSL014hg0`002EYh00<005jhG[P00Ujh00<006/I
JaT016/I00@006lh000000=gN003001g^GNi00=g^@8000Uoo`03001oogoo00Eoo`005goo00<007oo
Ool01goo0`003Goo1@0016]l0P000f?l0`000V?N00<006MKIe/00fMK0P000f;I00<005JkE[/015Jk
00<003YL>U`013YL00<0016k4K/00P0000<0mP005Qd00aHM1@0000=:NP00CSP01dhh00<004hg0000
0`000U9I00<005YhFWP01eYh00<005jhG[P00Ujh00<006/IJaT01F/I00@007MhMgP0009gN003001g
^GNi009g^@04001_nfok0009Ool00`00Oomoo`04Ool000Eoo`8000Aoo`8000=oo`<000Aoo`8000Uo
o`D000Moo`D00003Dgl0000000=[O003001So6?l009So004001[Nf]kJg/30004Ie/00`00H]URf@04
H]T00`00E[]F^`04E[/00`00>U`jG004>U`010004K/A^a6k0`000aHM00<003Hl=S`00SHl30001e9I
00<005YhFWP01eYh00@005jhG[P000Q[6@80009gN08000Ag^@03001_nfok009_n`03001oogoo00Eo
o`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000<Ool60002Ool40006Dgl0
1000Jg`006?l100016]k00<006/iJcT00P0000=WF`00H]T01V;I00<005JkE[/015Jk2P0000<A^`00
0?P00`000QHM00<003Hl=S`00cHl00<004HjASX01DHj00<0059IDUT01U9I00<005YhFWP01eYh00<0
05jh000026/I00H007>JLiX007Mh0004MkT00`00Ko]_n`03Ko/20003Ool30002Ool00`00Oomoo`03
Ool000Aoo`04001oogoo000;Ool00`00Oomoo`02000=Ool01@000K/000003Ud00`0025=o1`001F]k
00<006/iJcT00f/i00<006;IH]T01f;I00<005JkE[/00`0000=BF000@QT01D8I00<002gJ00000P00
00<0n0000CP00P0000<F7@00=S`013Hl00<004HjASX01DHj00<0059IDUT01U9I00<005YhFWP01UYh
0P0026/I00<007>JLiX00W>J0`000WNi00<006okKo/00fok00<005OM00000`001Goo00<007ooOol0
0goo0004Ool01000Oomoo`0027oo0`000goo00<007ooOol03Goo00<0006k00001`000U=o0`000S=k
0P0000=O_P00Jg/016]k00<006/iJcT016/i00<006;IH]T01f;I0`001E9H00<0048I@QT0148I00<0
02gJ;MX00P0000@0n0000CP1>08000Df?0030016>THj00E6>P03001BFE9I00IBF@03001JN5Yh00EJ
N003001RnP0000M[6@03001cVW>J00=cVP03001_n@00008000A_n`8000=Gg@03001oogoo00Eoo`03
001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`8000eoo`H00092gPX0009O_P03
001[Nf]k00=[N`03001[>F/i00A[>@03001Zn00000MRf@03001NV00000IBF00300126D8I00A26@03
000]fRgJ008]fP<00081>08000Df?0030016>THj00A6>P03001BFE9I00IBF@03001JN5Yh00=JN080
00=RnP03001[6F/I00=[6@03001cVW>J00=cVP04001_nFoiKoT20003Ko/00`00EmeGg@04Emd00`00
Oomoo`04Ool00`00Oomoo`03Ool000Eoo`8000Uoo`@0009oo`03001oogoo00Uoo`<000Aoo`030012
gT;N00A2gP8000EO6`<000A[N`03001[>F/i00A[>@05001Zn6[hJ_P00004H]T20003GYP00`00DUQB
F004DUP00`00@QU26@03@QT00`00;MX]fP02;MX00`003GX0000200020CP20003=S`00`00ASY6>P05
ASX00`00DUUBF@06DUT00`00FWQJN0020005H_X00`00JaU[6@02JaT00`00LiYcVP03LiX01P00KoU_
nFoi001;]@800003Ko/005OM00IGg@03001oogoo00Eoo`03001oogoo009oo`005Woo00<007ooOol0
1Woo0`0027oo00<004;N@]h00T;N00<005lKGa/01ElK00<006/IJaT00P0000=[N`00JcT01F/i00<0
06[hJ_P00V[h00D006;IH]URf@0000ENV003001BF59H00ABF00300126D8I00A26@03000]fRgJ008]
fP03000=NP00008000030CP0017L00800003=S`004Hj00I6>P03001BFE9I00IBF@03001JN00000QR
nP04001[6F/I0005LiX01@00KoU_nFoi00000d^e30001Woo00<007ooOol00Woo000FOol00`00Oomo
o`03Ool3000<Ool01000@]h000001elK00<006/IJaT00f/I0P001F/i00<006[hJ_P016[h00<006;I
00001UjH00<0059HDUP01E9H00<0048I@QT00d8I00<002gJ;MX00bgJ00<000ej3GX010000Q7L00<0
03Hl00001dHj00<0059IDUT01E9I0P002F;j00<006/I00001G>J00H006oiKoU_n@00BkD300000bZo
0000000500039ad00`00Oomoo`04Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000@Ool20007
Ga/00`00JaU[6@04JaT00`00KaP00003JcT00`00J_QZn006J_P00`00GYQNV006GYP00`00DUQBF004
DUP00`00@QU26@03@QT00`00;MX]fP03;MX01P003GX=NPej0000eP80008Ag08000M6>P03001BFE9I
009BF@<00003D[/006;j00QRnP<000EcVP03001_nFoi00<0009oo`D00085o0D000039ad007oo00Ao
o`03001oogoo009oo`005Goo10005Goo00<005lKGa/00elK00<006/IJaT016/I00L006lHKaQ_6000
JcT00008J_P20008GYP00`00DUQBF003DUP=00000bgJ000=NP023GX200000`3F000Ag0024M`50002
ASX00`00DUT000020005D[/00`00H_YRnP05H_X01@00Jia[W6^L00000g>J00<006oiKoT00P001goo
10000`Gl1@0017oo00<007ooOol00Woo000>Ool30004Ool2000HOol01@00Ga]O6elK00001V/I00<0
06lHKaP00flH00<006[hJ_P01V[h00@006KHI]P000MNV003001BF59H00<0009B=`03001:5dXG00=:
5`03000mn3gh008mn0@0008=NP8000050=H000004M`00004>Wd40008D[/00`00H_YRnP04H_X00`00
Jia[W003Ji`00`00LiX00002KoT00`00Oomoo`08Ool00`00Oomoo`0400001@6J0001n`7k00000goo
00<007ooOol00Woo000?Ool00`00Oomoo`05Ool00`00Oomoo`0FOol00`00Ga/00006JaT00`00KaQ_
6004KaP00`00J_QZn005J_P01@00I]QVf6KH00001ejH0`001E8g00<004XGBQL014XG00<003gh?OP0
0cgh00<0037j00001@0000@1F0004M`000HjO@03001B^e:k00MB^`03001RnV;j009RnP03001[W6^L
00A[W0D000aoo`03001oogoo009oo`8000040IX000000003Ool00`00Oomoo`02Ool000moo`03001o
ogoo00Ioo`03001oogoo01Ioo`03001[6F/I00A[6@03001_66lH00A_6003001_600000EZn003001V
f6KH00=Vf003001NV5jH00=NV003001JM`0000MB=`03001:5dXG00=:5`03000mn3gh00@mn004000a
nS7j<OX200000`4h0001F0030006>Wd00`00D[]B^`07D[/01000H_YRnP001V^L00@005?H000000io
o`03001oogoo00Aoo`8000030IX0000000=oo`03001oogoo005oo`003goo00<007ooOol01goo00<0
07ooOol05Woo00<006/IJaT00f/I00<006lHKaP00flH00D006lHKaQ_600000=Zn003001Vf6KH00EV
f003001NV5jH008000=JM`03001B=e8g00AB=`03001:5dXG00A:5`03000mn3gh00<mn003000anS7j
00<anP<00081F003000jOCYm00DjO@03001B^e:k00IB^`03001RnP0000E[W0@0015oo`03001oogoo
00Eoo`040001VP6J0002Ool00`00Oomoo`01Ool000moo`03001oogoo00Qoo`8001Moo`05001[6F/I
JaT00006KaP00`00KaQ_6002KaP00`00J_P00008I]P00`00GYP00005FWL00`00DSMB=`04DSL00`00
BQM:5`04BQL00`00?OPmn004?OP00`00<OXanP02<OX00`005Qd0000200000`5H000jO@05>Wd00`00
D[]B^`05D[/40003Ji`4000DOol00`00Oomoo`05Ool01P000IX1VP00Ool000=oo`003Woo0P0037oo
00<007ooOol05Goo00<006/I00001VlH00<006lHKaP016lH00<006KHI]P01fKH0P001UYg00<0058g
DSL01E8g00<004XGBQL00dXG00<003gh?OP01Cgh00<0037j<OX00S7j00@001HM5QdF7@L0008jO@03
001B^e:k00@000A?O`03001[W000008001Ioo`03001oogoo00Ioo`<00003Ool007oo009oo`007Goo
00<007ooOol05Goo00<006lHKaP016lH00<006lHKaP01FlH00<006KHI]P01VKH00<006:h00001eYg
00<0058gDSL0158g00<004XGBQL014XG00<003gh?OP013gh00<0037j<OX00c7j00<001HM5Qd00QHM
100000<JOP0000000`001dmo100000=_FP00Ool05Woo00<007ooOol01goo0`000goo000NOol00`00
Oomoo`0EOol00`00KaQ_6002KaP00`00KaQ_6005KaP00`00KaP00006I]P01@00H[QR^6:h00001UYg
00<0058gDSL0158g00<004XGBQL014XG00<003gh?OP00Sgh3@0014KN1@0000<JOP0000000Tmo0`00
16mJ00<007ooOol05goo00<007ooOol01goo0P000goo000OOol2000FOol01@00KaQ_66lH00001VlH
00D006lHKaQ_600000AVf003001R^6:h009R^003001JMeYg00EJM`03001B=e8g00AB=`04001:5dXG
BQL70003BSP00`00BUY:FP04BUX00`00A]i6gP04A]h20003Ga`60005KeX00`00Oomoo`0IOol00`00
Oomoo`07Ool00`00Oomoo`01Ool0025oo`03001oogoo01Aoo`03001_600000M_6005001_66lHKaP0
0003I]P00`00H[QR^004H[P00`00FWMJM`04FWL00`00DSMB=`03DSL50003CQL00`00BSQ:>005BSP0
0`00BUY:FP04BUX00`00A]i6gP02A]h00`00GaaO7002Ga`00`00JcY[>P02JcX00`00KeY_FP03KeX0
0`00Oomoo`0JOol00`00Oomoo`08Ool10001Ool1Ool0029oo`03001oogoo01Aoo`8000I_6003001_
66lH00=_6003001Vf00000MR^003001JMeYg00AJM`03001B=e8g00<00003CSL004hG00I>5`03001:
>4Xh00A:>003001:FTYJ00E:FP040016gP000004Ga`00`00JcY[>P03JcX20003KeX00`00Oomoo`0L
Ool00`00Oomoo`07Ool10001Ool1Ool002=oo`03001oogoo01Eoo`03001_66lH009_6003001_66lH
00E_6003001R^6:h00IR^003001JMeYg00AJM`<000E>=`03001>5dhG00A>5`03001:>4Xh00E:>003
001:FTYJ00A:FP8000EO7003001[>V/j00=[>P06001_>6lh001_FP0087oo00<007ooOol01Woo0@00
0Goo0Goo000TOol00`00Oomoo`0EOol01@00KaQ_66lH00001VlH0P002F:h00@005YgFWMJM`800003
DUL004hg00E>=`03001>5dhG00A>5`03001:>4Xh00E:>004001:FTYJBUX30002FYT01@00GaaO75lL
00001F/j00<006lhKcP00Vlh00<007ooOol087oo00<007ooOol01Goo0@000Goo0Goo000UOol2000F
Ool00`00KaP00006KaP01000JaU[6@0026:h00@005Yg000000=BE`03001>=dhg00A>=`03001>5dhG
00E>5`03001:>4Xh00A:>004001:FP000006FYT00`00Ga`00006JcX00`00KcQ_>002KcP00`00Oomo
o`0POol00`00Oomoo`05Ool10001Ool1Ool002Moo`03001oogoo01Aoo`8000E_6003001[6F/I009[
6@03001R^6:h00ER^08000IBE`03001>=dhg00A>=`03001>5dhG00A>5`03001:>4Xh00A:>08000UJ
V@03001[>V/j00A[>P03001_>6lh009_>003001oogoo029oo`03001oogoo00Aoo`40005oo`5oo`00
:7oo00<007ooOol05Goo00D006lHKaQ_600000E[6@03001R^6:h009R^0<00003FYT0059G00EBE`03
001>=dhg00A>=`03001>5dhG00E>5`03001:>4Xh009:>003001FF00000UJV@8000A[>P03001_>6lh
009_>003001oogoo02Aoo`03001oogoo00=oo`40005oo`5oo`00:Goo00<007ooOol05Goo00<006lH
00001f/I00@006:h000000EJV@03001BEe9G00=BE`03001>=dhg00A>=`03001>5dhG00E>5`03001:
>4Xh008000=FF003001JVEZI00EJV@04001Vf6KH0003JcX00`00KcQ_>002KcP00`00Oomoo`0TOol0
0`00Oomoo`04Ool10001Ool002Yoo`8001Ioo`03001[6F/I00I[6@8000MJV@03001BEe9G00ABE`03
001>=dhg00A>=`03001>5dhG00A>5`03001:>00000EFF003001JVEZI00AJV@03001Vf6KH009Vf003
001[>P0000A_>003001oogoo02Ioo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol057oo00<0
06/IJaT016/I00<005lK00001eZI00<0059GDUL0159G00<004hgCSL014hg00<004hGCQL00dhG0`00
1eIH00<005ZIFYT00UZI00<006KHI]P016KH00<006lhKcP00Vlh00<007ooOol0:7oo00<007ooOol0
0Woo0@000Goo000]Ool00`00Oomoo`0DOol01000JaU[6F/I0P000elK00<005ZIFYT01EZI00<0059G
DUL00e9G00<004hgCSL014hg00<004hGCQL00`000U8g00<005IHEUP01UIH00@005ZIFYT000MVf080
00=_>003001oogoo02Uoo`05001oogooOol00001Ool002ioo`03001oogoo01Aoo`03001[6@0000EO
6`03001JVEZI00EJV@03001BEe9G00=BE`03001>=dhg00=>=`@000IB=`03001FF5IH00EFF003001J
V@0000MVf006001_66lH001_>000;7oo00D007ooOomoo`00005oo`00;goo0P005Goo0P001UlK00<0
05ZIFYT015ZI00<0059GDUL0159G00<004hg00000P000U8g00<0058gDSL01E8g00<005IHEUP01EIH
0P001fKH00<006lHKaP00VlH00<007ooOol0;7oo00@007ooOol0005oo`00<Goo00<007ooOol057oo
00<005lKGa/015lK00<005ZIFYT015ZI00<0059GDUL01@001U8g00<0058gDSL0158g00<005IHEUP0
15IH00<005jH00001VKH00<006lHKaP00flH00<007ooOol0;Goo00<007oo00000Goo000bOol00`00
Oomoo`0DOol00`00Ga]O6`03Ga/01000FYUJVEZI1P000dhh00<0058gDSL01E8g00<0058gDSL01E8g
00<005IHEUP00UIH00D005jHGYQNV00000AVf003001_66lH00=_6003001oogoo02ioo`03001oogoo
00400000<goo00<007ooOol057oo2@000dYj00<004hhCSP014hh00<0058gDSL0158g00<0058gDSL0
1E8g00D005IHEUQFF00000ANV003001Vf6KH009Vf003001_66lH009_6003001oogoo031oo`40005o
o`400000=7oo0P005Woo00D004;N@]i2gP0000I:NP03001>>4hh00=>>003001B=e8g00EB=`03001B
=e8g00AB=`04001FF5IH0006GYP01000I]QVf0001FlH00<007ooOol0<Goo0P00000fOol00`00Oomo
o`0DOol01@00@]i2gT;N00001DYj00<004hhCSP00dhh00<0058gDSL01E8g00<0058gDSL01E8g0P00
1ejH00<006KH00001FlH00<007ooOol0<Goo0`00000gOol00`00Oomoo`0DOol200000d;N001:NP05
BWX00`00CSQ>>003CSP00`00DSMB=`04DSL00`00DSMB=`04DSL20009GYP00`00KaQ_6003KaP00`00
Oomoo`0^Ool40003Ool003Qoo`03001oogoo01=oo`030001^`00008000E:NP03001>>4hh00=>>003
001B=e8g00EB=`03001B=e8g009B=`03001JN00000UNV003001_66lH009_6003001oogoo02]oo`@0
00Moo`00>Goo00<007ooOol04goo00@0006k0K/000E:NP03001>>4hh00A>>003001B=e8g00AB=`03
001B=e8g008000=JN003001NV5jH00ENV003001Zn00000=_6003001oogoo02Qoo`<000]oo`00>Woo
0P0057oo00<0006k0K/00`000dYj00<004hhCSP00dhh00<0058gDSL0158g00<0058g00001EYh00<0
05jHGYP015jH00L006[hJ_QZn000KaP0000WOol4000>Ool003aoo`03001oogoo019oo`<0008b?0@0
00E>>003001B=e8g009B=`@000MJN003001NV5jH009NV003001Zn6[h00=Zn003001oogoo029oo`@0
019oo`00?Goo00<007ooOol04goo0P00138l2`000e9H00<005YhFWP01UYh00@005jHGYP000IZn003
001oogoo01ioo`@001Ioo`00?Woo00<007ooOol04goo0P00138l00<004HjASX00dHj00<0059HDUP0
1E9H00<005YhFWP01EYh00<005jH00001V[h00<007ooOol06goo10006Woo000oOol2000DOol20004
<S`00`00ASY6>P03ASX00`00DUQBF004DUP00`00FWQJN005FWP20006J_P00`00Oomoo`0HOol4000N
Ool0045oo`03001oogoo011oo`03001oogoo0080008b?0030016>THj00=6>P03001BF59H00ABF003
001JN5Yh00AJN003001Rf@0000EZn003001oogoo01Ioo`<0029oo`00@Woo00<007ooOol03goo00@0
07ooOomoo`80008b?0030016>THj00=6>P03001BF59H00ABF004001JN5YhFWP20003H]T00`00J_QZ
n002J_P00`00Oomoo`0BOol4000UOol004=oo`03001oogoo00ioo`03001oogoo009oo`@000E6>P03
001BF59H00ABF004001JN5Yh0006H]T01000J_QZn0004Goo1000:Goo0014Ool2000>Ool00`00Oomo
o`03Ool00`000GX00005ASX00`00DUQBF004DUP00`00FWP00007H]T00`00J_P0000>Ool4000]Ool0
04Ioo`03001oogoo00]oo`03001oogoo00Aoo`D000=6>P03001BF59H00=BF08000QRf@8000]oo`@0
035oo`00Agoo00<007ooOol02Woo00<007ooOol01Woo00<003IM=Ud010000U9H0`0000=B^`00H]T0
1V;I00<007ooOol01goo1000=Goo0018Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00=UdfG@02
=Ud30005D[/00`00H]URf@03H]T00`00Oomoo`04Ool4000iOol004Uoo`8000Uoo`03001oogoo00Qo
o`03000fGCIM008fG@03001B^e:k00EB^`05001RfF;IH]T00004Ool3000mOol004]oo`03001oogoo
00Ioo`03001oogoo00Uoo`03000fGCIM008fG@03001B^e:k00EB^`04001Rf@00Ool40010Ool004ao
o`03001oogoo00Eoo`03001oogoo00Yoo`@00003=Ud005:k00=B^`H004Aoo`00CGoo00<007ooOol0
17oo00<007ooOol03Woo2000B7oo001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004mo
o`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo
00001Goo1000Egoo001COol200000goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128621, 0.0056064, 
  0.00599011, 0.00599011}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
0 .095 .556 r
.37769 .7139 .40063 .73217 .43782 .76001 .41492 .74343 Metetra
0 .457 .861 r
.41492 .74343 .43782 .76001 .47657 .77466 .45387 .75912 Metetra
.533 .77 .984 r
.45387 .75912 .47657 .77466 .51648 .7703 .49417 .75482 Metetra
.773 .825 .893 r
.49417 .75482 .51648 .7703 .55694 .74527 .53516 .72875 Metetra
.849 .813 .815 r
.53516 .72875 .55694 .74527 .59731 .70256 .57615 .6841 Metetra
.872 .798 .774 r
.57615 .6841 .59731 .70256 .6372 .649 .61667 .62809 Metetra
.871 .785 .763 r
.61667 .62809 .6372 .649 .6766 .59333 .65665 .56993 Metetra
.851 .775 .777 r
.65665 .56993 .6766 .59333 .71591 .54416 .69647 .51865 Metetra
.803 .76 .812 r
.69647 .51865 .71591 .54416 .75582 .50837 .73682 .48141 Metetra
.707 .726 .865 r
.73682 .48141 .75582 .50837 .7972 .49017 .77859 .46259 Metetra
.53 .648 .909 r
.77859 .46259 .7972 .49017 .84086 .49071 .82263 .46338 Metetra
.287 .517 .901 r
.82263 .46338 .84086 .49071 .88733 .50796 .86953 .48168 Metetra
.078 .391 .85 r
.86953 .48168 .88733 .50796 .93673 .53684 .91945 .5122 Metetra
0 .346 .824 r
.91945 .5122 .93673 .53684 .98855 .5697 .9719 .54695 Metetra
0 .358 .828 r
.35442 .68872 .37769 .7139 .41492 .74343 .39191 .71167 Metetra
.39 .551 .893 r
.39191 .71167 .41492 .74343 .45387 .75912 .43103 .72284 Metetra
.607 .627 .845 r
.43103 .72284 .45387 .75912 .49417 .75482 .47151 .71713 Metetra
.714 .665 .8 r
.47151 .71713 .49417 .75482 .53516 .72875 .51279 .69307 Metetra
.777 .696 .774 r
.51279 .69307 .53516 .72875 .57615 .6841 .55426 .65329 Metetra
.818 .729 .766 r
.55426 .65329 .57615 .6841 .61667 .62809 .59549 .60378 Metetra
.846 .769 .777 r
.59549 .60378 .61667 .62809 .65665 .56993 .63637 .55226 Metetra
.857 .822 .813 r
.63637 .55226 .65665 .56993 .69647 .51865 .67716 .50643 Metetra
.827 .883 .884 r
.67716 .50643 .69647 .51865 .73682 .48141 .71844 .47248 Metetra
.678 .908 .975 r
.71844 .47248 .73682 .48141 .77859 .46259 .76095 .45422 Metetra
.322 .758 .968 r
.76095 .45422 .77859 .46259 .82263 .46338 .80542 .4527 Metetra
0 .512 .857 r
.80542 .4527 .82263 .46338 .86953 .48168 .85239 .46606 Metetra
0 .4 .827 r
.85239 .46606 .86953 .48168 .91945 .5122 .902 .4897 Metetra
.059 .423 .867 r
.902 .4897 .91945 .5122 .9719 .54695 .95387 .51673 Metetra
.403 .563 .899 r
.3309 .65753 .35442 .68872 .39191 .71167 .36897 .66695 Metetra
.546 .557 .817 r
.36897 .66695 .39191 .71167 .43103 .72284 .40833 .669 Metetra
.617 .563 .771 r
.40833 .669 .43103 .72284 .47151 .71713 .44883 .66067 Metetra
.666 .583 .752 r
.44883 .66067 .47151 .71713 .51279 .69307 .49013 .64112 Metetra
.709 .616 .752 r
.49013 .64112 .51279 .69307 .55426 .65329 .53186 .61186 Metetra
.75 .668 .77 r
.53186 .61186 .55426 .65329 .59549 .60378 .57372 .57642 Metetra
.794 .751 .813 r
.57372 .57642 .59549 .60378 .63637 .55226 .61561 .53947 Metetra
.821 .877 .886 r
.61561 .53947 .63637 .55226 .67716 .50643 .65767 .5058 Metetra
.703 .977 .914 r
.65767 .5058 .67716 .50643 .71844 .47248 .70024 .47939 Metetra
.254 .747 .662 r
.70024 .47939 .71844 .47248 .76095 .45422 .74378 .46276 Metetra
.095 0 0 r
.74378 .46276 .76095 .45422 .80542 .4527 .78878 .4566 Metetra
.193 0 0 r
.78878 .4566 .80542 .4527 .85239 .46606 .83557 .45966 Metetra
0 .529 .803 r
.83557 .45966 .85239 .46606 .902 .4897 .88427 .46893 Metetra
.294 .607 .954 r
.88427 .46893 .902 .4897 .95387 .51673 .93466 .47999 Metetra
.623 .641 .847 r
.30709 .62237 .3309 .65753 .36897 .66695 .34611 .61416 Metetra
.623 .569 .772 r
.34611 .61416 .36897 .66695 .40833 .669 .38584 .6045 Metetra
.625 .541 .739 r
.38584 .6045 .40833 .669 .44883 .66067 .42629 .5929 Metetra
.631 .539 .732 r
.42629 .5929 .44883 .66067 .49013 .64112 .46739 .57921 Metetra
.643 .559 .745 r
.46739 .57921 .49013 .64112 .53186 .61186 .5091 .56369 Metetra
.662 .609 .785 r
.5091 .56369 .53186 .61186 .57372 .57642 .55137 .54689 Metetra
.688 .714 .869 r
.55137 .54689 .57372 .57642 .61561 .53947 .59419 .52962 Metetra
.658 .894 .981 r
.59419 .52962 .61561 .53947 .65767 .5058 .63759 .51272 Metetra
.248 .746 .676 r
.63759 .51272 .65767 .5058 .70024 .47939 .68163 .49692 Metetra
.105 0 0 r
.68163 .49692 .70024 .47939 .74378 .46276 .72642 .4827 Metetra
.19 0 0 r
.72642 .4827 .74378 .46276 .78878 .4566 .77205 .47019 Metetra
.104 0 0 r
.77205 .47019 .78878 .4566 .83557 .45966 .81861 .45914 Metetra
.279 .782 .835 r
.81861 .45914 .83557 .45966 .88427 .46893 .86612 .44898 Metetra
.596 .786 .971 r
.86612 .44898 .88427 .46893 .93466 .47999 .91457 .43889 Metetra
.727 .675 .799 r
.28286 .58595 .30709 .62237 .34611 .61416 .3231 .55958 Metetra
.676 .591 .753 r
.3231 .55958 .34611 .61416 .38584 .6045 .36337 .53806 Metetra
.635 .543 .733 r
.36337 .53806 .38584 .6045 .42629 .5929 .40378 .52317 Metetra
.602 .518 .732 r
.40378 .52317 .42629 .5929 .46739 .57921 .44457 .51538 Metetra
.572 .515 .753 r
.44457 .51538 .46739 .57921 .5091 .56369 .48605 .51382 Metetra
.538 .541 .806 r
.48605 .51382 .5091 .56369 .55137 .54689 .52846 .51636 Metetra
.475 .623 .915 r
.52846 .51636 .55137 .54689 .59419 .52962 .57197 .52 Metetra
.242 .707 .947 r
.57197 .52 .59419 .52962 .63759 .51272 .61656 .52134 Metetra
.122 0 0 r
.61656 .52134 .63759 .51272 .68163 .49692 .66205 .5173 Metetra
.194 0 0 r
.66205 .5173 .68163 .49692 .72642 .4827 .70815 .50581 Metetra
.103 0 0 r
.70815 .50581 .72642 .4827 .77205 .47019 .75453 .48625 Metetra
.75453 .48625 .77205 .47019 .81861 .45914 .80095 .4597 Metetra
.671 .976 .861 r
.80095 .4597 .81861 .45914 .86612 .44898 .84737 .42861 Metetra
.79 .853 .897 r
.84737 .42861 .86612 .44898 .91457 .43889 .89397 .39629 Metetra
.784 .7 .771 r
.25799 .55109 .28286 .58595 .3231 .55958 .29956 .50952 Metetra
.719 .626 .753 r
.29956 .50952 .3231 .55958 .36337 .53806 .34048 .47821 Metetra
.652 .567 .747 r
.34048 .47821 .36337 .53806 .40378 .52317 .38095 .46056 Metetra
.576 .52 .755 r
.38095 .46056 .40378 .52317 .44457 .51538 .42147 .4575 Metetra
.484 .482 .779 r
.42147 .4575 .44457 .51538 .48605 .51382 .46262 .46736 Metetra
.353 .456 .826 r
.46262 .46736 .48605 .51382 .52846 .51636 .50497 .48608 Metetra
.139 .44 .876 r
.50497 .48608 .52846 .51636 .57197 .52 .54887 .50761 Metetra
0 .395 .76 r
.54887 .50761 .57197 .52 .61656 .52134 .59434 .52491 Metetra
.254 0 0 r
.59434 .52491 .61656 .52134 .66205 .5173 .64105 .53133 Metetra
.115 0 0 r
.64105 .53133 .66205 .5173 .70815 .50581 .68835 .52223 Metetra
.68835 .52223 .70815 .50581 .75453 .48625 .73554 .49628 Metetra
.621 .93 .691 r
.73554 .49628 .75453 .48625 .80095 .4597 .78209 .45587 Metetra
.876 .983 .832 r
.78209 .45587 .80095 .4597 .84737 .42861 .82786 .40647 Metetra
.872 .851 .82 r
.82786 .40647 .84737 .42861 .89397 .39629 .8732 .35515 Metetra
.817 .724 .762 r
.23218 .52018 .25799 .55109 .29956 .50952 .27497 .46909 Metetra
.76 .675 .769 r
.27497 .46909 .29956 .50952 .34048 .47821 .31657 .43167 Metetra
.676 .621 .787 r
.31657 .43167 .34048 .47821 .38095 .46056 .35727 .41221 Metetra
.549 .554 .812 r
.35727 .41221 .38095 .46056 .42147 .4575 .39767 .41185 Metetra
.358 .464 .832 r
.39767 .41185 .42147 .4575 .46262 .46736 .4386 .42853 Metetra
.095 .348 .818 r
.4386 .42853 .46262 .46736 .50497 .48608 .48083 .45709 Metetra
0 .248 .752 r
.48083 .45709 .50497 .48608 .54887 .50761 .52489 .48974 Metetra
0 .25 .689 r
.52489 .48974 .54887 .50761 .59434 .52491 .57088 .51713 Metetra
0 .427 .721 r
.57088 .51713 .59434 .52491 .64105 .53133 .61842 .53019 Metetra
.241 .755 .842 r
.61842 .53019 .64105 .53133 .68835 .52223 .66667 .5225 Metetra
.674 .975 .889 r
.66667 .5225 .68835 .52223 .73554 .49628 .71468 .49218 Metetra
.874 .978 .841 r
.71468 .49218 .73554 .49628 .78209 .45587 .76168 .44262 Metetra
.918 .909 .797 r
.76168 .44262 .78209 .45587 .82786 .40647 .80747 .38136 Metetra
.896 .834 .776 r
.80747 .38136 .82786 .40647 .8732 .35515 .85246 .31797 Metetra
.837 .754 .771 r
.20517 .4947 .23218 .52018 .27497 .46909 .24879 .44142 Metetra
.797 .746 .806 r
.24879 .44142 .27497 .46909 .31657 .43167 .29101 .40249 Metetra
.707 .725 .864 r
.29101 .40249 .31657 .43167 .35727 .41221 .3321 .38236 Metetra
.504 .649 .923 r
.3321 .38236 .35727 .41221 .39767 .41185 .37271 .3822 Metetra
.153 .463 .889 r
.37271 .3822 .39767 .41185 .4386 .42853 .41371 .39989 Metetra
0 .255 .755 r
.41371 .39989 .4386 .42853 .48083 .45709 .45596 .43008 Metetra
0 .185 .69 r
.45596 .43008 .48083 .45709 .52489 .48974 .5001 .46461 Metetra
0 .29 .767 r
.5001 .46461 .52489 .48974 .57088 .51713 .54632 .49369 Metetra
.167 .543 .926 r
.54632 .49369 .57088 .51713 .61842 .53019 .59427 .50775 Metetra
.554 .765 .976 r
.59427 .50775 .61842 .53019 .66667 .5225 .64312 .49996 Metetra
.773 .837 .901 r
.64312 .49996 .66667 .5225 .71468 .49218 .69184 .46836 Metetra
.861 .839 .822 r
.69184 .46836 .71468 .49218 .76168 .44262 .73961 .41653 Metetra
.891 .827 .778 r
.73961 .41653 .76168 .44262 .80747 .38136 .78614 .35248 Metetra
.892 .815 .766 r
.78614 .35248 .80747 .38136 .85246 .31797 .8318 .28627 Metetra
.842 .793 .804 r
.17672 .47501 .20517 .4947 .24879 .44142 .22056 .42725 Metetra
.82 .847 .868 r
.22056 .42725 .24879 .44142 .29101 .40249 .26322 .39163 Metetra
.703 .886 .965 r
.26322 .39163 .29101 .40249 .3321 .38236 .30487 .37202 Metetra
.345 .766 .976 r
.30487 .37202 .3321 .38236 .37271 .3822 .34607 .36946 Metetra
0 .455 .796 r
.34607 .36946 .37271 .3822 .41371 .39989 .38761 .38207 Metetra
0 .281 .704 r
.38761 .38207 .41371 .39989 .45596 .43008 .43026 .40516 Metetra
0 .304 .774 r
.43026 .40516 .45596 .43008 .5001 .46461 .47463 .43174 Metetra
.127 .436 .875 r
.47463 .43174 .5001 .46461 .54632 .49369 .52094 .45349 Metetra
.413 .557 .889 r
.52094 .45349 .54632 .49369 .59427 .50775 .56897 .46242 Metetra
.601 .628 .85 r
.56897 .46242 .59427 .50775 .64312 .49996 .61804 .45293 Metetra
.714 .674 .81 r
.61804 .45293 .64312 .49996 .69184 .46836 .6673 .42339 Metetra
.786 .713 .783 r
.6673 .42339 .69184 .46836 .73961 .41653 .71602 .37674 Metetra
.834 .752 .773 r
.71602 .37674 .73961 .41653 .78614 .35248 .76387 .3196 Metetra
.863 .796 .783 r
.76387 .3196 .78614 .35248 .8318 .28627 .81112 .2604 Metetra
.815 .843 .87 r
.14669 .46022 .17672 .47501 .22056 .42725 .19005 .42484 Metetra
.763 .954 .936 r
.19005 .42484 .22056 .42725 .26322 .39163 .23285 .39686 Metetra
.471 .894 .857 r
.23285 .39686 .26322 .39163 .30487 .37202 .27518 .37886 Metetra
.27518 .37886 .30487 .37202 .34607 .36946 .31741 .37151 Metetra
.112 0 0 r
.31741 .37151 .34607 .36946 .38761 .38207 .36004 .37356 Metetra
0 .509 .781 r
.36004 .37356 .38761 .38207 .43026 .40516 .40362 .38192 Metetra
.208 .579 .942 r
.40362 .38192 .43026 .40516 .47463 .43174 .44857 .3921 Metetra
.425 .569 .894 r
.44857 .3921 .47463 .43174 .52094 .45349 .49507 .39898 Metetra
.538 .555 .82 r
.49507 .39898 .52094 .45349 .56897 .46242 .54301 .3978 Metetra
.608 .561 .776 r
.54301 .3978 .56897 .46242 .61804 .45293 .59202 .38528 Metetra
.661 .583 .757 r
.59202 .38528 .61804 .45293 .6673 .42339 .64158 .36051 Metetra
.709 .622 .758 r
.64158 .36051 .6673 .42339 .71602 .37674 .69122 .32516 Metetra
.756 .681 .779 r
.69122 .32516 .71602 .37674 .76387 .3196 .74072 .2831 Metetra
.802 .77 .823 r
.74072 .2831 .76387 .3196 .81112 .2604 .79019 .2394 Metetra
.681 .876 .972 r
.11513 .44835 .14669 .46022 .19005 .42484 .15727 .43006 Metetra
.457 .887 .861 r
.15727 .43006 .19005 .42484 .23285 .39686 .19987 .41287 Metetra
.19987 .41287 .23285 .39686 .27518 .37886 .24295 .39727 Metetra
.24295 .39727 .27518 .37886 .31741 .37151 .28658 .38339 Metetra
.28658 .38339 .31741 .37151 .36004 .37356 .33089 .37097 Metetra
.406 .849 .925 r
.33089 .37097 .36004 .37356 .40362 .38192 .376 .35942 Metetra
.598 .785 .97 r
.376 .35942 .40362 .38192 .44857 .3921 .42199 .34791 Metetra
.618 .643 .853 r
.42199 .34791 .44857 .3921 .49507 .39898 .46893 .33554 Metetra
.614 .567 .778 r
.46893 .33554 .49507 .39898 .54301 .3978 .51682 .32151 Metetra
.614 .536 .743 r
.51682 .32151 .54301 .3978 .59202 .38528 .56561 .30528 Metetra
.62 .533 .734 r
.56561 .30528 .59202 .38528 .64158 .36051 .61523 .28671 Metetra
.634 .556 .749 r
.61523 .28671 .64158 .36051 .69122 .32516 .66563 .26605 Metetra
.656 .611 .792 r
.66563 .26605 .69122 .32516 .74072 .2831 .71677 .24392 Metetra
.685 .72 .875 r
.71677 .24392 .74072 .2831 .79019 .2394 .76868 .22118 Metetra
.244 .707 .948 r
.08224 .43664 .11513 .44835 .15727 .43006 .12264 .43695 Metetra
.12264 .43695 .15727 .43006 .19987 .41287 .16472 .43174 Metetra
.16472 .43174 .19987 .41287 .24295 .39727 .20852 .41886 Metetra
.20852 .41886 .24295 .39727 .28658 .38339 .25383 .39771 Metetra
.493 .889 .736 r
.25383 .39771 .28658 .38339 .33089 .37097 .30024 .36937 Metetra
.765 .974 .92 r
.30024 .36937 .33089 .37097 .376 .35942 .34735 .33637 Metetra
.788 .828 .883 r
.34735 .33637 .376 .35942 .42199 .34791 .39483 .30209 Metetra
.729 .685 .808 r
.39483 .30209 .42199 .34791 .46893 .33554 .44253 .27001 Metetra
.671 .593 .759 r
.44253 .27001 .46893 .33554 .51682 .32151 .49055 .24308 Metetra
.625 .538 .736 r
.49055 .24308 .51682 .32151 .56561 .30528 .53914 .22316 Metetra
.589 .51 .734 r
.53914 .22316 .56561 .30528 .61523 .28671 .5887 .21074 Metetra
.558 .508 .756 r
.5887 .21074 .61523 .28671 .66563 .26605 .63959 .20487 Metetra
.53 .539 .809 r
.63959 .20487 .66563 .26605 .71677 .24392 .69209 .2033 Metetra
.492 .624 .909 r
.69209 .2033 .71677 .24392 .76868 .22118 .74626 .20278 Metetra
0 .302 .651 r
.04837 .42199 .08224 .43664 .12264 .43695 .0869 .43859 Metetra
.202 0 0 r
.0869 .43859 .12264 .43695 .16472 .43174 .12832 .44403 Metetra
.12832 .44403 .16472 .43174 .20852 .41886 .17276 .43355 Metetra
.494 .893 .752 r
.17276 .43355 .20852 .41886 .25383 .39771 .21975 .40577 Metetra
.796 .998 .854 r
.21975 .40577 .25383 .39771 .30024 .36937 .2684 .36317 Metetra
.882 .928 .848 r
.2684 .36317 .30024 .36937 .34735 .33637 .31772 .31139 Metetra
.855 .817 .811 r
.31772 .31139 .34735 .33637 .39483 .30209 .36693 .25766 Metetra
.793 .716 .779 r
.36693 .25766 .39483 .30209 .44253 .27001 .41567 .20916 Metetra
.721 .633 .759 r
.41567 .20916 .44253 .27001 .49055 .24308 .46407 .17163 Metetra
.644 .566 .752 r
.46407 .17163 .49055 .24308 .53914 .22316 .51263 .14861 Metetra
.563 .513 .758 r
.51263 .14861 .53914 .22316 .5887 .21074 .56211 .14102 Metetra
.474 .477 .781 r
.56211 .14102 .5887 .21074 .63959 .20487 .61328 .1471 Metetra
.372 .464 .825 r
.61328 .1471 .63959 .20487 .69209 .2033 .66672 .16254 Metetra
.25 .487 .889 r
.66672 .16254 .69209 .2033 .74626 .20278 .72262 .18095 Metetra
.476 0 0 r
.01394 .40164 .04837 .42199 .0869 .43859 .05105 .4285 Metetra
.212 0 0 r
.05105 .4285 .0869 .43859 .12832 .44403 .09194 .4407 Metetra
.348 .814 .918 r
.09194 .4407 .12832 .44403 .17276 .43355 .13689 .43159 Metetra
.755 .966 .932 r
.13689 .43159 .17276 .43355 .21975 .40577 .18528 .39926 Metetra
.877 .923 .852 r
.18528 .39926 .21975 .40577 .2684 .36317 .23585 .34722 Metetra
.894 .858 .798 r
.23585 .34722 .2684 .36317 .31772 .31139 .28716 .28324 Metetra
.874 .799 .773 r
.28716 .28324 .31772 .31139 .36693 .25766 .33804 .21714 Metetra
.831 .744 .768 r
.33804 .21714 .36693 .25766 .41567 .20916 .38794 .15835 Metetra
.766 .688 .777 r
.38794 .15835 .41567 .20916 .46407 .17163 .43697 .1142 Metetra
.672 .625 .794 r
.43697 .1142 .46407 .17163 .51263 .14861 .48577 .08908 Metetra
.542 .552 .815 r
.48577 .08908 .51263 .14861 .56211 .14102 .53532 .08411 Metetra
.377 .472 .83 r
.53532 .08411 .56211 .14102 .61328 .1471 .58667 .09713 Metetra
.198 .4 .835 r
.58667 .09713 .61328 .1471 .66672 .16254 .64065 .12275 Metetra
.055 .371 .839 r
.64065 .12275 .66672 .16254 .72262 .18095 .6976 .15276 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`l001Moo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol200001Goo000005Ii00001dHj00<0037j<OX010004goo00<007oo
Ool017oo0P002Goo0P007Goo0016Ool01000Oomoo`003goo0`0000=oo`00000015Ii00<004HjASX0
1DHj00<0037j<OX00c7j0`0047oo00<007ooOol01Woo00<007ooOol09Goo0016Ool01000Oomoo`00
3Goo0P000Woo0P001UIi00<004HjASX01DHj00<0037j<OX0137j00@001VJ000000ioo`03001oogoo
00Moo`03001oogoo02Aoo`00Agoo0P002goo0`000goo00<005IiEWT01eIi00<004HjASX01DHj00<0
037j<OX0137j00<001VJ6IX00`002goo00<007ooOol01goo00<007ooOol097oo001BOol30004Ool2
0009EWT00`00ASY6>P05ASX00`00<OXanP05<OX00`006IXIVP026IX20009Ool00`00Oomoo`08Ool0
0`00Oomoo`0SOol004moo`<000Ioo`04001Rf6;H0009EWT00`00ASY6>P04ASX00`00<OXanP05<OX0
0`006IXIVP046IX20007Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<000Qoo`05001Rf6;H
H]P00009EWT00`00ASY6>P05ASX00`00<OXanP05<OX00`006IXIVP036IX010001GX000001Goo00<0
07ooOol02Goo00<007ooOol08Woo001:Ool30009Ool00`00H]QRf003H]P00`00EWUFN@07EWT00`00
ASY6>P04ASX00`00<OXanP05<OX00`006IXIVP046IX00`001GX5NP030002Ool00`00Oomoo`0:Ool0
0`00Oomoo`0QOol004Moo`<0009oo`03001oogoo00Ioo`03001Rf6;H00ERf003001FNEIi00IFN@03
0016>THj0096>PT0008anP03000IVQVJ00@IVP030005NPEj0085NP<000eoo`03001oogoo021oo`00
AGoo0P003Goo00<006;HH]P01V;H00<005IiEWT01UIi1@000dXH00<003gh?OP00cgh1P000aVJ00<0
00Ej1GX010Ej00<007ooOol02goo00<007ooOol087oo0012Ool3000>Ool00`00H]QRf008H]P00`00
EWUFN@02EWT300000e9H001:6006BQP00`00?OPmn005?OP00`00;MX]fP0500051GX00`00Oomoo`0<
Ool00`00Oomoo`0OOol002Eoo`@001Moo`80015oo`03001Rf6;H00QRf003001FN@00008000EBF003
001:64XH00A:6003000mn3gh00Hmn003000]fRgJ00<]fP@00085NP8000ioo`03001oogoo01ioo`00
9Woo00<007ooOol057oo0`004Woo00<006[h00002F;H0P00259H00<004XHBQP01DXH00<003gh?OP0
1Cgh00<002gJ;MX012gJ00<001gl7O`00`0000<5NP00Ool02goo0P0087oo000OOol50003Ool00`00
Oomoo`0AOol2000DOol01000J_QZn00026;H00<005jH0000259H00<004XHBQP01DXH00<003gh?OP0
1Sgh00<002gJ;MX012gJ00@001gl7O`Mo0@000eoo`03001oogoo01eoo`00:7oo00<007ooOol03Goo
0`005Goo00<006[hJ_P00V[h00<006;HH]P016;H00D005jHGYQNV00000QBF003001:64XH00E:6003
000mn3gh00Dmn003000]fRgJ00@]fP05000Mo1gl7O`000037O`2000;Ool00`00Oomoo`0MOol002Eo
o`04001oogoo000=Ool2000GOol00`00J_QZn003J_P00`00H]QRf002H]P20004GYP00`00DUQBF006
DUP00`00BQQ:6005BQP00`00?OPmn005?OP00`00;MX]fP05;MX01@007O`Mo1gl000011gl00<007oo
Ool02Goo00<007ooOol077oo000VOol2000;Ool3000IOol00`00J_QZn004J_P01000H]QRf0001UjH
00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP01Sgh00<002gJ;MX01BgJ00@001gl7O`000DM
o003001oogoo00Uoo`03001oogoo01]oo`00<Goo0P0000=oo`00Ool067oo00<006[hJ_P01F[h00<0
06;H000025jH00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP01Cgh00<002gJ;MX01BgJ00@0
01gl7O`0008Mo0D000Yoo`03001oogoo01]oo`00;Woo0`0077oo00<006[hJ_P01f[h00<005jHGYP0
1ejH00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP00Sgh4@001Cim00<007ooOol01goo0P00
77oo000[Ool3000NOol00`00J_QZn007J_P20009GYP00`00DUQBF006DUP00`00BQQ:6004BQP60004
AQP00`00@SU2>@05@ST00`00?Wd00008?Wd00`00Oomoo`08Ool00`00Oomoo`03Ool2000DOol002Uo
o`80009oo`03001oogoo01]oo`03001Zn6[h00MZn004001Vf6KH0009GYP00`00DUQBF006DUP00`00
BQP000030002BQL00`00AQQ66006AQP00`00@SU2>@05@ST20008?Wd30008Ool00`00Oomoo`02Ool0
1000Oomoo`004goo000VOol3000ROol00`00J_QZn006J_P01@00I]QVf6KH00002EjH00<0059HDUP0
1E9H0`001dXG00<004HHAQP01DHH00<0048i@ST01T8i00<003im?Wd013im0P000eKk00<007ooOol0
1Woo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo0P0097oo00<006lh00001V[h00<006KH
I]P00VKH00<005jHGYP01ejH00<0059HDUP00U9H0`0000=>=`00BQL01dXG00<004HHAQP01THH00<0
048i@ST01D8i00<003im?Wd00Sim0P001UKk00<007ooOol01Woo00@007ooOol0009oo`03001oogoo
015oo`008Goo0`009Goo00D006lhKcQ_>00000AZn003001Vf6KH00AVf003001NV5jH00MNV0@000A>
=`03001:5dXG00I:5`03001664HH00I660030012>D8i00E2>@800003?Wd0000000QFn`8000Ioo`80
009oo`04001oogoo000COol001moo`8002Moo`03001_>6lh009_>005001Zn6[hJ_P00007I]P00`00
GYQNV006GYP20008CSL00`00BQM:5`06BQL00`00AQQ66005AQP00`00@SU2>@04@ST40009E_/00`00
IaX00008Ool00`00Oomoo`02000DOol001aoo`<002Qoo`03001_>6lh00A_>003001Zn00000UVf003
001NV5jH00=NV0800003EUP004hg00M>=`03001:5dXG00I:5`03001664HH00I660030012>D8i00D0
0003DWT005Kk00MFn`05001W6VLJIaX00008Ool00`00Oomoo`0EOol001]oo`8002Yoo`03001_>6lh
00E_>003001Vf6KH00QVf004001NV5jHGYP20004EUP00`00CSM>=`05CSL00`00BQM:5`06BQL00`00
AQQ66006AQP50003DWT00`00DWT00006E_/00`00IaYW6P03IaX00`00Oomoo`05Ool00`00Oomoo`0E
Ool001Aoo`<000Aoo`03001oo`0002Qoo`03001_>6lh00I_>003001Vf6KH00QVf004001NV0000006
EUP00`00CSM>=`06CSL00`00BQM:5`06BQL00`00AQQ66002AQP50007DWT00`00DWT00004E_/00`00
IaYW6P05IaX00`00Oomoo`05Ool00`00Oomoo`0DOol001Eoo`03001oogoo00=oo`04001oogoo000V
Ool00`00KcQ_>006KcP00`00KeT00009I]P20008EUP00`00CSM>=`06CSL00`00BQM:5`06BQL00`00
AQP0000200001DHH0000058g0000259i0P000eKk00<006LJIaX01VLJ00<007ooOol01Goo0P005Goo
000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo000097oo00<006lhKcP01Vlh00D006mIKeU_
F@0000MVf003001N^@0000UFF003001>=dhg00E>=`03001:5dXG00E:5`<00096608000AB=`03001B
NE9i00EBN@05001BN@00E_/00008IaX00`00KcT00007Ool00`00Oomoo`0COol001Eoo`03001oogoo
009oo`8000Aoo`03001oogoo021oo`03001_>6lh00I_>003001_FFmI00=_F@03001Vf6KH009Vf080
00=N^@03001FF5IH00IFF003001>=dhg00I>=`04001:5dXGBQL30002BQL30006DSL00`00DWUBN@06
DWT30008IaX01@00KcU_>Fli00001goo00<007ooOol04Woo000EOol00`00Oomoo`02Ool20005Ool0
0`00Oomoo`0OOol20006KcP00`00KeU_F@04KeT01@00I]QVf6KH00001Eji00<005IHEUP01UIH00<0
04hgCSL01Thg100014XG0P00258g00<0059iDWT01U9i0`001fLJ00<006liKcT00fli00<007ooOol0
17oo00<007ooOol04Woo000DOol20004Ool00`00Oomoo`05Ool00`00Oomoo`0MOol01000LgUcN@00
1Flh00<006mIKeT01FmI00<006KH00001eji00<005IHEUP01UIH00<004hgCSL014hg0P0014XG0`00
00=>=`00DSL0258g00<0059iDWT0159i00D006:h001R^00000EW6P03001_>Fli00A_>@03001oogoo
00Eoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol06goo00<007=iLgT00W=i00D0
06lhKcQ_>00000Q_F@8000QN^@03001FF5IH00IFF004001>=dhgCSL30002CSL40005CSL00`00DSMB
=`06DSL00`00DWUBN@03DWT01000H[QR^6:h0P0016LJ00<006liKcT016li00<007=H00001goo00<0
07ooOol047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0IOol00`00LgUcN@04LgT00`00KcP00008
KeT00`00Ie`00009G[T00`00EUQFF005EUP40003CSL30008CSL00`00DSMB=`07DSL01@00DWUBNE9i
00001F:h0P000VLJ00<006liKcT016li00D007=HLeQcF00000Eoo`80019oo`006Goo0P002Woo00<0
07ooOol05goo00<007=iLgT01W=i00<006mIKeT01VmI00D006MLIeaWG00000QN^@03001FF5IH00=F
F0@00003CSL000000080009>=`03001>=dhg00I>=`03001B=e8g00MB=`04001BNE9i0007H[P00`00
IaX00007KcT00`00LeQcF002LeP00`00Oomoo`04Ool00`00Oomoo`0?Ool000Qoo`8000Aoo`8000=o
o`<000=oo`8000]oo`03001oogoo01Eoo`03001cNG=i00IcN@03001c^`0000E_F@8000AWG003001N
^Eji00IN^@03001FF5IH00<0009FF0<000M>=`03001>=dhg00I>=`03001B=e8g00IB=`03001BN@00
00QR^08000M_>@03001cF7=H00AcF003001oogoo00Aoo`03001oogoo00ioo`001goo00@007ooOol0
00=oo`8000Ioo`04001oogoo000=Ool00`00Oomoo`0COol00`00LgUcN@06LgT01@00Lk]c^g>k0000
0fmI00<006MLIe`01FML00<005jiG[T01Eji0`000UIH0`0000=>F000CSL01dhg00<004hgCSL01Thg
00<0058gDSL01U8g0P002V:h00<006liKcT016li00<007=HLeP01G=H00<007ooOol017oo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo015o
o`03001cNG=i00IcN@03001c^g>k00=c^`03001_F@0000QWG003001N^Eji009N^@<00003G[T005IH
008000E>F003001>=dhg00E>=`03001>=dhg00I>=`03001B=e8g00EB=`03001JN00000UR^003001[
600000A_>@03001cF7=H00EcF003001cF00000Ioo`03001oogoo00eoo`001goo00@007ooOol000Qo
o`<000=oo`03001oogoo00eoo`80011oo`03001cNG=i00IcN@03001c^g>k00Ac^`<000UWG0@000=N
^@<000M>F003001>=dhg00I>=`03001>=dhg00I>=`03001B=e8g00=B=`05001JN5YhFWP00007H[P0
0`00JaQ[60020003KcT00`00LeQcF004LeP01000LeQcF0001goo00<007ooOol00Woo10001Woo0007
Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol03goo00<007ooOol037oo0P001g=i00<0
07>kLk/017>k00@006?mHoeSo@8000IWG080009N^@@0009>V`03001>F4iH00E>F003001>=dhg00I>
=`03001>=dhg00I>=`04001B=e8gDSL20004FWP00`00H[QR^004H[P00`00JaQ[6002JaP200000fli
001cF005LeP00`00LeQcF002LeP00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`06Ool000Qo
o`8000Uoo`@0009oo`03001oogoo011oo`03001oogoo00]oo`04001c^`000004LgT00`00Lk]c^`03
Lk/20006Hod00`00IeaWG0070007CY/00`00CUQ>F005CUP00`00CSM>=`05CSL00`00CSM>=`06CSL0
1000DSMB=`001eYh00<006:hH[P00V:h00<006/HJaP016/H0P001W=H00<007=HLeP00g=H00<007oo
Ool01Goo00<007ooOol00goo00<007ooOol01Goo000HOol2000COol00`00Oomoo`09Ool00`00Lk]c
^`02Lk/200000g=i001c^`04Lk/00`00HoeSo@05Hod60003Ccl00`00CY]>V`06CY/00`00CUQ>F005
CUP00`00CSM>=`06CSL00`00CSM>=`06CSL20008FWP00`00H[QR^002H[P00`00JaQ[6005JaP00`00
LeQcF004LeP00`00LeQcF004LeP00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qo
o`8001Aoo`03001oogoo00Moo`03001c^g>k00Ac^`D0009c^`03001SoF?m00=So@@0008cO@03001?
?dlo00A??`03001>VdjK00E>V`03001>F4iH00E>F003001>=dhg00I>=`03001>=dhg00E>=`8000YJ
N004001R^6:h0007JaP30004LeP00`00LeQcF005LeP00`00Oomoo`04Ool20002Ool01000Oomoo`00
1Woo000BOol20004Ool00`00Oomoo`0DOol00`00Oomoo`05Ool00`00Lk]c^`04Lk/00`00Io]Wn`03
Io/90006<gd00`00Ccm??`03Ccl00`00CY]>V`06CY/00`00CUQ>F005CUP00`00CSM>=`05CSL00`00
CSM>=`02CSL300000eIH001JN009FWP00`00H[P00007JaP00`00LeQcF0020002LeP00`00LeQcF005
LeP00`00KgX00007Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`0EOol0
0`00Oomoo`03Ool00`00Lk]c^`04Lk/00`00Io]Wn`02Io/200000cnG000000030002;al700000c=m
001??`04Ccl00`00CY]>V`05CY/00`00CUQ>F005CUP00`00CSM>=`06CSL40005EUP00`00FWQJN008
FWP00`00JaQ[6006JaP01`00LeQcF7=H001cF00000McF004001_NVmj0008Ool00`00Oomoo`08Ool0
015oo`04001oogoo0003Ool00`00Oomoo`0FOol01@00Oomoogoo00001W>k00@006OkIo]Wn`L0009K
WPl000=??`03001>VdjK00E>V`03001>F4iH00A>F003001>=dhg00A>=`<000QFF003001JN5Yh00MJ
N08000M[6003001cF7=H00=cF003001cF7=H00EcF003001_NVmj009_NP03001oogoo00Eoo`03001o
ogoo00Qoo`004Goo00@007ooOol0009oo`8001Uoo`03001oo`0000Ic^`03001WnfOk00H0009KWP<0
0003F_/004:M00A2W@03000Ag00000@000036Uh00000008000M>V`03001>F4iH00E>F003001>=`00
00<0009>>003001FF5IH00IFF003001JN5Yh00IJN004001VfFKI0005JaP00`00LeQcF004LeP20005
LeP00`00KgY_NP03KgX00`00Oomoo`06Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00
Oomoo`0HOol00`00Lk]c^`04Lk/00`00Io/0000600000e^N00000005F_/00`00@Ye2W@03@Yd01000
4M`Ag17L0`000aYN1P000TjK00<004iHCUP00TiH1@001dhh00<005IHEUP01UIH00<005YhFWP015Yh
00<006KII]T00VKI00D006/HJaQ[600000IcF004001g^@000003LeP00`00KgY_NP04KgX20007Ool0
0`00Oomoo`07Ool0019oo`8000=oo`8001Qoo`D000=c^`H000A[N`03001Jne[k00IJn`030012WD:M
00=2W@03000Ag17L008Ag08000<JGP03000fG3IL008fG0L000=6>P03001>>4hh00I>>003001FF5IH
00IFF003001JN5Yh00=JN003001VfFKI00=Vf@05001[66/HJaP00005LeP01000MkUg^GNi0P0000=c
F000KgX01Fmj00<006Ml000027oo00<007ooOol01Woo000GOol01000Ool000005Goo00<006?MHmd0
0f?M1P001V]k0P002E[k00<004:M@Yd00d:M00<0017L4M`00Q7L0`000QYN00<003IL=U`013IL00<0
04HjASX01DHj00<004hhCSP01Dhh00<005IHEUP01eIH00D005YhFWQJN00000MVf@03001[600000Ic
F007001g^GNiMkT007Ni00001Fmj0P000fMl00<007ooOol01Woo00<007ooOol01Goo000GOol00`00
Oomoo`03000AOol20005Hmd400000f?M001[N`05Jg/01000I_UVn@0025[k2P000Q7L100000<JGP00
=U`01CIL00<004HjASX014Hj00<004hhCSP01Thh00<005IHEUP01UIH00@005YhFWP000UVf@03001c
F7=H00AcF003001g^GNi00=g^@03001cnP0000=_NP03001WO6Ml00=WO003001oogoo00Ioo`03001o
ogoo00Eoo`005goo00<007ooOol00Woo0`003Woo00<006?MHmd00f?M10000V?M00<006]kJg/016]k
00<006KiI_T00VKi00<005[kF_/00U[k0`0000=FN@00AST014Hi00<002gJ;MX01@0000<1>0000000
1SIL00<004HjASX014Hj00<004hhCSP01Thh00<005IHEUP01UIH00<005Yh00002VKI00<007=HLeP0
0g=H00<007NiMkT00gNi00L007?j001cnP00KgX00007Ig`00`00Oomoo`06Ool00`00Oomoo`04Ool0
00Eoo`8000Aoo`8000=oo`<000Aoo`8000Aoo`@000]oo`04001SgF?MHmd:0005Jg/00`00I_UVn@04
I_T01000F_/000001EIi00<004HiAST00dHi00<002gJ;MX00RgJ0`0000<1>00000001SIL00<004Hj
ASX014Hj00<004hhCSP01Dhh00<005IHEUP01UIH0P002VKI00<006mJ00000g=H00<007NiMkT00gNi
00@007?jLoYcnP<000QWO003001oogoo00Eoo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05
001oogooOol00008Ool30005Ool:0006Je/00`00JcT0000200000f]k001Vn@07I_T00`00EWUFN@05
EWT00`00ASU6>@03AST00`00;MX]fP03;MX01@003GX000000CP00P0013IL00<004HjASX014Hj00<0
04hhCSP01Thh00<005IHEUP00eIH0P0000=J^@00I]T026KI00D006mJKeY_FP00009cF003001g^GNi
009g^@03001cnW?j009cnP03001Gn`0000QWO003001oogoo009oo`<0009oo`03001oogoo00=oo`00
17oo00@007ooOol000]oo`03001oogoo008000Yoo`H000<_G@<00003Eil0000000I[F`03001[>F/i
00=[>@03001VnFKi00IVn@8000QFN@030016>DHi00=6>@03000]fRgJ00<]fP05000=NP000001>004
0002=U`00`00ASY6>P03ASX00`00CSQ>>006CSP01000EUQFF5IH0P0015Zi00<006KII]T01FKI00<0
06mJKeX00VmJ00<007=H000017Ni00@007?jLoYcnP800004Eo/005Ok0006Ig`60005Ool00`00Oomo
o`03Ool000Aoo`04001oogoo0008Ool30003Ool2000>Ool50003Eil00`00Je][F`05Je/00`00JcU[
>@04JcT00`00I_UVn@05I_T01000H[QR^00025Ii00<004HiAST00dHi00<002gJ;MX00RgJ00<000ej
3GX010000Q6k0`001THj00<004hhCSP01Dhh00@005IH000000IJ^@03001VfFKI00AVf@03001_FVmJ
00A_FP8000=g^@03001cnW?j008000AGn`03001Gn`00009WO0<0009??`03001oogoo00Ioo`03001o
ogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo00aoo`<00006Ool00000
001GW`001f]K00<006/iJcT016/i00<006/H00001FKi00D006:hH[QR^00000QFN@030016>DHi00=6
>@03000]fRgJ00<]fP05000=NPej0000]P0200044K/30003ASX00`00CSQ>>004CSP30009F[T00`00
I]UVf@02I]T00`00KeY_FP04KeX01P00KoY_nP00MkT0009cnP03001GneOk00=Gn`H000I??`03001o
ogoo00Eoo`03001oogoo00=oo`001Goo0P002Goo10000Woo00<007ooOol02Goo0`001goo0P001V]K
00<006/iJcT016/i00D006/HJaQ[600000=Vn@03001R^6:h00=R^003001FNEIi00EFN@X000<]fP03
000=NPej008000030;H0016k008A^`03000f?3Hl00@0009>>0@0009>V`03001J^EZi00QJ^@04001V
fFKI0006KeX01000KoY_nVoj3P000R<j00<004loCcl014lo00<007ooOol01Woo00<007ooOol00Woo
000FOol00`00Oomoo`06Ool3000<Ool20003Je/00`00JcU[>@04JcT00`00JaQ[6003JaP00`00I_T0
0007H[P01000EWUFNEIi0`0000=BE`00BQP014XH00<003gh?OP010000`ej0P0000<0]P0000000Q6k
00<003Hl=S`00SHl0`001djK00<005ZiF[T01eZi00<006KI00001VmJ00D006ojKoY_nP00009?]PP0
00826`H000I??`03001oogoo00Eoo`03001oogoo009oo`005Goo0P001Goo0`004Goo00<006]K0000
1V/i00<006/HJaP01F/H00<006:hH[P01V:h00@005Ii000000ABE`03001:64XH00A:6003000mn3gh
008mn0@000083GX000000001>0004K/000Df?003001>VdjK00I>V`03001J^EZi00MJ^@<000E_FP06
001_nVojKoX004nf0P001T:M00<002HL00000`0000<1VP000000100014lo00<007ooOol017oo00<0
07ooOol00Woo000EOol01000Oomoogoo0`005Goo0P001V/i00<006/HJaP01F/H00<006:hH[P01V:h
0P001e9G00<004XHBQP00dXH00<003gh?OP00cgh00<002gJ;MX00`0000<0n0000CP00`001CHl00<0
04jKCY/01TjK00<005ZiF[T01EZi00D006=LHeaSG00000=_FP07001_nVojKoX004nf00000U[k00<0
04:M@Yd014:M00@002HL9Q`V70<000030IX00000008VOPD000Ioo`03001oogoo009oo`005Goo1000
6Woo0P000f/i00<006/HJaP01F/H00<006[h00001V:h00<005jH00001e9G00<004XHBQP014XH00<0
03gh?OP00cgh00<002gJ;MX00RgJ0`000P4h0P0013Hl00<004jKCY/01TjK00<005ZiF[T00eZi0P00
1F=L00<006mJ00000foj00<004nf000015[k00<004:M@Yd014:M00<002HL9Q`00RHL0P0000<1VP00
00001BIn0P001Goo00<007ooOol00Woo000>Ool30004Ool2000NOol00`00JcT00007JaP01@00J_QZ
n6[h000016:h00D005jHGYQNV00000IBE`03001:64XH00A:6003000mn3gh00<mn003000]fRgJ008]
fP03000Ag000008000030CP003Hl00@f?003001>VdjK00I>V`03001J^EZi008000MSG0@00004KoX0
00000005F_/:00029Q`00`002I/000020IX200049Wh00`00Oomoo`03Ool00`00Oomoo`02Ool000mo
o`03001oogoo00Eoo`03001oogoo01aoo`03001[66/H00E[6003001Zn6[h009Zn005001R^6:hH[P0
0004GYP00`00DUMBE`05DUL00`00BQQ:6003BQP00`00?OPmn004?OP00`00;MX]fP02;MX00`004M`A
g0060003=S`00`00CY]>V`05CY/00`00F[T00008He`01000EoaGo5Ol0`0015[k0`002Woo100000D9
V`00000000Fk008000<VOP03001oogoo009oo`03001oogoo009oo`003goo00<007ooOol01Woo00<0
07ooOol077oo00<006/HJaP00f/H00<006[hJ_P016[h00<006:h00001EjH00<0059GDUL01E9G00<0
04XHBQP014XH00<003gh?OP00cgh00<002gJ;MX00bgJ00<0017L4M`02@0014jK1P001V=L0P000UOl
0P000fLI00@005[k0000011oo`<000032I/000000085^`<000039Wh007oo00=oo`03001oogoo005o
o`003goo00<007ooOol01goo00<007ooOol077oo00D006/HJaQ[600000QZn003001NV5jH00ENV003
001BEe9G00ABE`03001:64XH00A:6003000mn3gh00<mn003000]fRgJ00@]fP04000Ag17L4M`40002
5Sd50006Aal01000HeaSG6=L0P000UOl0P001VLI00<007ooOol04Woo0P000PVK00<000Fk1K/00PFk
0P0017oo00<007ooOol00Goo000?Ool00`00Oomoo`08Ool2000MOol00`00JaP00008J_P20007GYP0
0`00DUMBE`05DUL01000BQQ:64XH6P000QHm00<004LOAal01TLO00<006=L00000UOl0`001VLI00<0
07ooOol05Goo00<000VK2I/00P0010Fk00D007ooOomoo`0000=oo`003Woo0P0037oo00<007ooOol0
6goo00<006[hJ_P01V[h00@006KHI]P000MNV003001BEe9G00=BE`D000=>5`03001:64XH00=:6003
0016>DHi00A6>@03000nOCim008nO@T000A77`P00003KeX006LI00EW6@03001oogoo01Ioo`@00003
0GX000000085^`05001oogooOol00003Ool001eoo`03001oogoo01Yoo`03001Zn6[h00EZn005001V
f6KHI]P00007GYP01000DUMBEe9G0P0000=B=`00CQL01DhG00<004XHBQP014XH00<004HiAST014Hi
00<003im?Wd01Cim2@000QlJ0`0016mJ00<006LIIaT00fLI00<007ooOol067oo00@007oo00000081
NP8000051K/007ooOol00003Ool001ioo`03001oogoo01Yoo`03001Zn6[h00=Zn003001Vf6KH009V
f003001NV5jH00ENV004001BE`000004DSL00`00CQM>5`04CQL00`00BQQ:6003BQP00`00ASU6>@05
AST00`00?WdnO@02?Wd20003F]/200000eNO000000040005KeX00`00KaP00003IaT00`00Oomoo`0J
Ool01000Oomoo`000`5j00D000Fk001oo`0000=oo`007goo0P006goo00D006[hJ_QZn00000IVf003
001NV5jH00ANV08000IB=`03001>5dhG00A>5`03001:64XH00A:60030016>DHi00E6>@04000nO@00
0006F]/30003Ji`00`00KeY_FP03KeX01`00KaQ_66lH001W6@0001ioo`03001oogoo00800081NP80
0003Ool007oo009oo`008Goo00<007ooOol06Goo00<006[h00001fKH00@005jHGYQNV0<00003EUP0
058g00EB=`03001>5dhG00A>5`03001:64XH00A:60030016>DHi00E6>@8000MJf`07001W6VLJIaX0
06^L00001FmJ00<006lHKaP00flH00<007ooOol07Goo00<007ooOol00Woo00L0005j0GX1NP00Ool0
0002Ool0029oo`03001oogoo01Uoo`03001Vf6KH00IVf004001NV0000004EUP00`00DSMB=`05DSL0
0`00CQM>5`04CQL00`00BQQ:6004BQP01000ASU6>DHi0`0000=FN@00F]/01E[K00<006LJIaX00fLJ
00<006mJKeX00fmJ00<006lHKaP016lH00<007ooOol07Woo00<007ooOol00Woo0P0000@1NP00Ool0
009oo`008goo00<007ooOol067oo00<006KHI]P01VKH0P001eIH00<0058gDSL0158g00<004hGCQL0
14hG00<004XHBQP014XH00@004Hi000000EFN@03001Jfe[K009Jf`03001W6VLJ00AW6P8000=_FP03
001_66lH00A_6003001oogoo021oo`03001oogoo00=oo`@0009oo`0097oo00<007ooOol067oo00<0
06KHI]P016KH00<005ji00001eIH00<0058gDSL0158g00<004hGCQL014hG00<004XHBQP00dXH0`00
25Ii00@005[KF]/000IW6P06001_66lH001_FP001VlH00<007ooOol08Woo00<007ooOol017oo0P00
0Woo000UOol2000IOol01000I]QVf6KH0P000eji00<005IHEUP015IH00<0058gDSL01E8g00<004hG
CQL014hG00<004XH00000P000U8g00<005IiEWT01UIi00<005[K00001VLJ00<006lHKaP00VlH00<0
06lHKaP016lH00<007ooOol08goo00<007ooOol01Goo0@000Goo0Goo000WOol00`00Oomoo`0GOol0
0`00I]P00005G[T00`00EUQFF005EUP00`00DSMB=`04DSL00`00CQM>5`03CQL30006DSL00`00EWUF
N@05EWT20006IaX00`00KaQ_6003KaP20005KaP00`00Oomoo`0TOol00`00Oomoo`04Ool10001Ool1
Ool002Qoo`03001oogoo01Moo`03001N^Eji00EN^@03001FF5IH00AFF003001B=e8g00AB=`03001>
5dhG00<00003DSL0058g00IB=`03001FNEIi00AFN@04001N^5jh0005IaX00`00KaQ_6002KaP01000
KcQ_>0000flH00<007ooOol09Woo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0FOol0
0`00G[UN^@05G[T00`00EUQFF004EUP00`00DSMB=`03DSL40005DSL00`00DSMB=`05DSL00`00EWUF
N@02EWT00`00G[QN^002G[P01@00IaYW6VLJ000016lH00<006lhKcP00Vlh00<006lH0000:Goo00<0
07ooOol017oo0@000Goo000ZOol2000GOol00`00G[UN^@04G[T00`00EUQFF005EUP00`00DSL00002
0002CSL00`00DSMB=`04DSL00`00DSMB=`06DSL01000EWUFN@001Ujh00<006LJ00001FlH00<006lh
KcP00flh00<007ooOol0:Goo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0EOol00`00G[UN
^@04G[T00`00EUQFF003EUP30005CSL00`00DSMB=`04DSL00`00DSMB=`06DSL00`00EWT00008G[P0
0`00KaQ_6003KaP00`00KcQ_>004KcP00`00Oomoo`0ZOol00`00Oomoo`02Ool10001Ool002eoo`03
001oogoo01Aoo`03001N^Eji00AN^@03001FF5IH00<00003CUP004hg00I>=`03001B=e8g00AB=`03
001B=e8g00EB=`8000UN^003001_66lH009_6003001_>6lh00A_>003001oogoo02aoo`05001oogoo
Ool00001Ool002ioo`03001oogoo01Aoo`04001N^EjiG[T50005CUP00`00CSM>=`04CSL00`00DSMB
=`04DSL00`00DSMB=`04DSL00`00FWP00008G[P00`00J_P00003KaP00`00KcQ_>003KcP00`00Oomo
o`0]Ool01@00Oomoogoo00000Goo000_Ool2000EOol30003CY/00`00CUQ>F004CUP00`00CSM>=`04
CSL00`00DSMB=`05DSL01000DSMB=e8g0P000eYh00<005jhG[P015jh00L006[hJ_QZn000KaP00005
KcP00`00Oomoo`0_Ool01000Oomoo`000Goo000aOol00`00Oomoo`0BOol00`00CY]>V`04CY/00`00
CUQ>F003CUP00`00CSM>=`05CSL00`00DSMB=`04DSL01000DSMB=`001EYh00<005jhG[P00ejh00<0
06[hJ_P00f[h00<006lhKcP00flh00<007ooOol0<Goo00<007oo00000Goo000bOol00`00Oomoo`0B
Ool00`00CY]>V`03CY/00`00CUQ>F004CUP00`00CSM>=`04CSL00`00DSMB=`04DSL00`00DSL00007
FWP01@00G[QN^5jh00001V[h0P000flh00<007ooOol0<Woo00<007ooOol00@00000cOol00`00Oomo
o`0BOol00`00CY]>V`03CY/00`00CUQ>F003CUP00`00CSM>=`04CSL00`00DSMB=`04DSL20008FWP0
1000G[QN^0001V[h00H006lhKcP006lh000fOol10001Ool1000003Aoo`80019oo`03001>VdjK00=>
V`03001>F4iH00=>F003001>=dhg00A>=`04001B=e8gDSL300000eIH001JN007FWP00`00G[P00006
J_P00`00KcQ_>002KcP00`00Oomoo`0fOol2000003Ioo`03001oogoo011oo`03001>VdjK00=>V`03
001>F4iH00=>F003001>=dhg00A>=`<000EFF003001JN5Yh00IJN003001Zn6[h00EZn003001_>6lh
009_>003001oogoo03Eoo`<00000=goo00<007ooOol047oo100000=>V`00CUP014iH00<004hgCSL0
0dhg0P001eIH00<005YhFWP01EYh0P001V[h00<006lhKcP00Vlh00<007ooOol0<Woo10000goo000h
Ool00`00Oomoo`0?Ool01000<U`bG39L1P0000A>F000CSM>=`<00003CUP005IH00MFF003001JN5Yh
00=JN004001Rf6;H0004J_P00`00KcQ_>002KcP00`00Oomoo`0_Ool40007Ool003Uoo`03001oogoo
00moo`03000bG39L008bG0040016>THjASX40005CUP00`00EUQFF005EUP00`00FWQJN002FWP00`00
H]QRf002H]P01@00J_QZn6[h000016lh00<007ooOol0;7oo0`002goo000jOol2000@Ool00`00<U`b
G002<U`00`00ASY6>P02ASX00`00CUQ>F004CUP00`00EUQFF006EUP01000FWQJN0001V;H00<006[h
000016lh00<007ooOol0:Goo10003Woo000lOol00`00Oomoo`0=Ool00`00<U`bG002<U`00`00ASY6
>P03ASX00`00CUQ>F004CUP00`00EUQFF005EUP00`00FWP00008H]P00`00KcQ_>002KcP00`00Oomo
o`0VOol4000BOol003eoo`03001oogoo00eoo`8000<bG0030016>THj0096>P03001>F4iH00A>F003
001FF5IH00EFF08000QRf003001_FP0000=_>003001oogoo029oo`@001Ioo`00?Woo00<007ooOol0
3Goo0`0000<bG000ASX014Hj00<004iHCUP00diH00<005IHEUP00eIH0P0000=J^@00H]P01V;H00L0
06mJKeY_FP00KcP0000QOol4000JOol003moo`8000ioo`040001NP000005ASX00`00CUQ>F004CUP0
1@00EUQFF5IH000015Zi00<006;HH]P00f;H00<006mJKeX00fmJ00<007ooOol077oo10007Woo0011
Ool00`00Oomoo`0<Ool00`000GX1NP030003ASX00`00CUQ>F003CUP01000EUP000001EZi00<006;H
H]P00V;H00<006mJKeX016mJ00<007ooOol06Goo0`008Woo0012Ool00`00Oomoo`0<Ool30002<S`4
0003CUP40008F[T01000H]QRf0001VmJ00<007ooOol05Woo10009Goo0013Ool00`00Oomoo`0=Ool2
0004<S`40003CY/00`00F[UJ^@06F[T00`00H]P00006KeX00`00Oomoo`0COol4000YOol004Aoo`80
00ioo`8000@b?003001>VdjK00A>V`03001J^EZi00EJ^@<000E_FP03001oogoo011oo`@002eoo`00
AWoo00<007ooOol02goo0`00138l00<004jKCY/014jK00<005ZiF[T00eZi00D006=LHeaSG00000=_
FP03001oogoo00eoo`@0035oo`00Agoo00<007ooOol02Woo00<007ooOol00P000S8l00<004jKCY/0
14jK00@005ZiF[UJ^@8000ESG003001_FP0000aoo`@003Eoo`00B7oo00<007ooOol02Goo00@007oo
Oomoo`<00003<S`004jK00E>V`03001J^@0000MSG08000Uoo`@003Uoo`00BGoo0P002Goo00<007oo
Ool00goo0`0014jK10001V=L0P0027oo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool6
0004Aal00`00HeaSG002He`00`00Oomoo`04Ool40010Ool004aoo`03001oogoo00Eoo`03001oogoo
00Moo`<000E77`04001SG0000003Ool40014Ool004eoo`03001oogoo00Aoo`03001oogoo00Yoo`L0
0003Ool00000008004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo001?Ool20003
Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03001oo`0000Eo
o`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.458 0 0 r
.37648 .74293 .39996 .75138 .43767 .76622 .4143 .7635 Metetra
.016 .586 .768 r
.4143 .7635 .43767 .76622 .47668 .7661 .45374 .76564 Metetra
.644 .95 .948 r
.45374 .76564 .47668 .7661 .51653 .74849 .49424 .74615 Metetra
.863 .939 .869 r
.49424 .74615 .51653 .74849 .55667 .71474 .5351 .70676 Metetra
.898 .869 .801 r
.5351 .70676 .55667 .71474 .59664 .66972 .5757 .65361 Metetra
.884 .81 .771 r
.5757 .65361 .59664 .66972 .63628 .62045 .6158 .59549 Metetra
.849 .76 .765 r
.6158 .59549 .63628 .62045 .67576 .57447 .65555 .54164 Metetra
.794 .713 .774 r
.65555 .54164 .67576 .57447 .71557 .53828 .6955 .49987 Metetra
.715 .661 .795 r
.6955 .49987 .71557 .53828 .75637 .51631 .73644 .47545 Metetra
.603 .599 .82 r
.73644 .47545 .75637 .51631 .79887 .5104 .77921 .47052 Metetra
.449 .522 .841 r
.77921 .47052 .79887 .5104 .84361 .5196 .8245 .48398 Metetra
.258 .435 .846 r
.8245 .48398 .84361 .5196 .89084 .54022 .87264 .51148 Metetra
.065 .362 .832 r
.87264 .51148 .89084 .54022 .94033 .56622 .92342 .5458 Metetra
0 .35 .818 r
.92342 .5458 .94033 .56622 .99133 .59004 .97598 .57772 Metetra
0 .39 .836 r
.35282 .72181 .37648 .74293 .4143 .7635 .39078 .74166 Metetra
.473 .681 .957 r
.39078 .74166 .4143 .7635 .45374 .76564 .43049 .74332 Metetra
.719 .759 .884 r
.43049 .74332 .45374 .76564 .49424 .74615 .47141 .72364 Metetra
.812 .77 .814 r
.47141 .72364 .49424 .74615 .5351 .70676 .51282 .68431 Metetra
.85 .772 .776 r
.51282 .68431 .5351 .70676 .5757 .65361 .55408 .63139 Metetra
.864 .776 .763 r
.55408 .63139 .5757 .65361 .6158 .59549 .59487 .57356 Metetra
.859 .783 .775 r
.59487 .57356 .6158 .59549 .65555 .54164 .6353 .51992 Metetra
.831 .79 .813 r
.6353 .51992 .65555 .54164 .6955 .49987 .67587 .4782 Metetra
.753 .784 .878 r
.67587 .4782 .6955 .49987 .73644 .47545 .71733 .45359 Metetra
.576 .727 .945 r
.71733 .45359 .73644 .47545 .77921 .47052 .76052 .44821 Metetra
.289 .575 .938 r
.76052 .44821 .77921 .47052 .8245 .48398 .80613 .46096 Metetra
.041 .405 .858 r
.80613 .46096 .8245 .48398 .87264 .51148 .85453 .48756 Metetra
0 .337 .816 r
.85453 .48756 .87264 .51148 .92342 .5458 .90555 .52087 Metetra
.045 .391 .851 r
.90555 .52087 .92342 .5458 .97598 .57772 .95838 .55178 Metetra
.366 .526 .883 r
.3292 .68798 .35282 .72181 .39078 .74166 .36746 .70068 Metetra
.572 .589 .831 r
.36746 .70068 .39078 .74166 .43049 .74332 .40733 .69913 Metetra
.676 .625 .79 r
.40733 .69913 .43049 .74332 .47141 .72364 .44839 .68093 Metetra
.74 .658 .768 r
.44839 .68093 .47141 .72364 .51282 .68431 .49007 .64737 Metetra
.787 .696 .764 r
.49007 .64737 .51282 .68431 .55408 .63139 .53184 .60305 Metetra
.824 .745 .777 r
.53184 .60305 .55408 .63139 .59487 .57356 .5734 .55464 Metetra
.849 .811 .812 r
.5734 .55464 .59487 .57356 .6353 .51992 .61479 .50932 Metetra
.839 .896 .879 r
.61479 .50932 .6353 .51992 .67587 .4782 .65637 .47327 Metetra
.703 .95 .955 r
.65637 .47327 .67587 .4782 .71733 .45359 .69871 .45074 Metetra
.317 .798 .903 r
.69871 .45074 .71733 .45359 .76052 .44821 .7425 .44347 Metetra
0 .529 .771 r
.7425 .44347 .76052 .44821 .80613 .46096 .78829 .45054 Metetra
0 .413 .779 r
.78829 .45054 .80613 .46096 .85453 .48756 .8364 .46844 Metetra
.022 .44 .869 r
.8364 .46844 .85453 .48756 .90555 .52087 .88673 .4914 Metetra
.263 .504 .898 r
.88673 .4914 .90555 .52087 .95838 .55178 .93865 .51221 Metetra
.552 .565 .821 r
.30572 .64405 .3292 .68798 .36746 .70068 .34452 .64474 Metetra
.606 .552 .767 r
.34452 .64474 .36746 .70068 .40733 .69913 .38449 .63793 Metetra
.644 .56 .745 r
.38449 .63793 .40733 .69913 .44839 .68093 .42543 .62247 Metetra
.678 .586 .745 r
.42543 .62247 .44839 .68093 .49007 .64737 .46706 .59897 Metetra
.715 .631 .763 r
.46706 .59897 .49007 .64737 .53184 .60305 .50907 .56969 Metetra
.756 .709 .807 r
.50907 .56969 .53184 .60305 .5734 .55464 .5513 .53801 Metetra
.793 .839 .886 r
.5513 .53801 .5734 .55464 .61479 .50932 .59374 .50769 Metetra
.708 .971 .93 r
.59374 .50769 .61479 .50932 .65637 .47327 .63655 .4821 Metetra
.63655 .4821 .65637 .47327 .69871 .45074 .68006 .46359 Metetra
.086 0 0 r
.68006 .46359 .69871 .45074 .7425 .44347 .72462 .45318 Metetra
.19 0 0 r
.72462 .45318 .7425 .44347 .78829 .45054 .77056 .45033 Metetra
.065 0 0 r
.77056 .45033 .78829 .45054 .8364 .46844 .81806 .45304 Metetra
.297 .677 .975 r
.81806 .45304 .8364 .46844 .88673 .4914 .86708 .45813 Metetra
.496 .618 .902 r
.86708 .45813 .88673 .4914 .93865 .51221 .9173 .4618 Metetra
.646 .592 .78 r
.28224 .59462 .30572 .64405 .34452 .64474 .32182 .58103 Metetra
.63 .546 .741 r
.32182 .58103 .34452 .64474 .38449 .63793 .36193 .56807 Metetra
.622 .53 .729 r
.36193 .56807 .38449 .63793 .42543 .62247 .4026 .55588 Metetra
.62 .536 .738 r
.4026 .55588 .42543 .62247 .46706 .59897 .44388 .54438 Metetra
.625 .571 .773 r
.44388 .54438 .46706 .59897 .50907 .56969 .48582 .53333 Metetra
.636 .654 .851 r
.48582 .53333 .50907 .56969 .5513 .53801 .52847 .52235 Metetra
.611 .82 .98 r
.52847 .52235 .5513 .53801 .59374 .50769 .57184 .51101 Metetra
.273 .776 .778 r
.57184 .51101 .59374 .50769 .63655 .4821 .61594 .4989 Metetra
.089 0 0 r
.61594 .4989 .63655 .4821 .68006 .46359 .66075 .48572 Metetra
.17 0 0 r
.66075 .48572 .68006 .46359 .72462 .45318 .70625 .47133 Metetra
.087 0 0 r
.70625 .47133 .72462 .45318 .77056 .45033 .75241 .45581 Metetra
.261 .759 .698 r
.75241 .45581 .77056 .45033 .81806 .45304 .79924 .43939 Metetra
.645 .875 .984 r
.79924 .43939 .81806 .45304 .86708 .45813 .84676 .42245 Metetra
.668 .693 .864 r
.84676 .42245 .86708 .45813 .9173 .4618 .89502 .40543 Metetra
.706 .621 .76 r
.25845 .54517 .28224 .59462 .32182 .58103 .29899 .51791 Metetra
.653 .561 .738 r
.29899 .51791 .32182 .58103 .36193 .56807 .33931 .49912 Metetra
.606 .522 .734 r
.33931 .49912 .36193 .56807 .4026 .55588 .37966 .48999 Metetra
.559 .503 .749 r
.37966 .48999 .4026 .55588 .44388 .54438 .42045 .48988 Metetra
.505 .508 .793 r
.42045 .48988 .44388 .54438 .48582 .53333 .46207 .49645 Metetra
.419 .551 .882 r
.46207 .49645 .48582 .53333 .52847 .52235 .50485 .50597 Metetra
.202 .619 .949 r
.50485 .50597 .52847 .52235 .57184 .51101 .54891 .51389 Metetra
.131 0 0 r
.54891 .51389 .57184 .51101 .61594 .4989 .59415 .51576 Metetra
.193 0 0 r
.59415 .51576 .61594 .4989 .66075 .48572 .64024 .50817 Metetra
.082 0 0 r
.64024 .50817 .66075 .48572 .70625 .47133 .68676 .48961 Metetra
.68676 .48961 .70625 .47133 .75241 .45581 .7333 .46089 Metetra
.668 .969 .792 r
.7333 .46089 .75241 .45581 .79924 .43939 .77966 .42493 Metetra
.835 .91 .889 r
.77966 .42493 .79924 .43939 .84676 .42245 .82594 .38603 Metetra
.774 .736 .818 r
.82594 .38603 .84676 .42245 .89502 .40543 .87252 .34892 Metetra
.752 .658 .757 r
.23387 .5009 .25845 .54517 .29899 .51791 .27542 .46308 Metetra
.681 .597 .755 r
.27542 .46308 .29899 .51791 .33931 .49912 .31605 .43978 Metetra
.596 .54 .763 r
.31605 .43978 .33931 .49912 .37966 .48999 .3562 .43286 Metetra
.485 .487 .784 r
.3562 .43286 .37966 .48999 .42045 .48988 .39651 .44134 Metetra
.328 .433 .817 r
.39651 .44134 .42045 .48988 .46207 .49645 .43773 .46149 Metetra
.093 .378 .84 r
.43773 .46149 .46207 .49645 .50485 .50597 .48043 .48715 Metetra
0 .324 .759 r
.48043 .48715 .50485 .50597 .54891 .51389 .52489 .51058 Metetra
.279 0 0 r
.52489 .51058 .54891 .51389 .59415 .51576 .57097 .52389 Metetra
.118 0 0 r
.57097 .52389 .59415 .51576 .64024 .50817 .61816 .52087 Metetra
.61816 .52087 .64024 .50817 .68676 .48961 .66566 .49874 Metetra
.655 .955 .739 r
.66566 .49874 .68676 .48961 .7333 .46089 .71278 .459 Metetra
.882 .985 .82 r
.71278 .459 .7333 .46089 .77966 .42493 .7591 .40703 Metetra
.896 .879 .81 r
.7591 .40703 .77966 .42493 .82594 .38603 .80475 .35056 Metetra
.835 .764 .783 r
.80475 .35056 .82594 .38603 .87252 .34892 .85034 .29771 Metetra
.79 .705 .771 r
.20796 .46578 .23387 .5009 .27542 .46308 .25041 .42225 Metetra
.716 .662 .796 r
.25041 .42225 .27542 .46308 .31605 .43978 .29145 .39641 Metetra
.589 .599 .83 r
.29145 .39641 .31605 .43978 .3562 .43286 .33161 .39045 Metetra
.376 .496 .853 r
.33161 .39045 .3562 .43286 .39651 .44134 .37171 .40322 Metetra
.074 .345 .819 r
.37171 .40322 .39651 .44134 .43773 .46149 .41266 .43034 Metetra
0 .209 .725 r
.41266 .43034 .43773 .46149 .48043 .48715 .45523 .46444 Metetra
0 .195 .672 r
.45523 .46444 .48043 .48715 .52489 .51058 .49986 .49612 Metetra
0 .369 .75 r
.49986 .49612 .52489 .51058 .57097 .52389 .54646 .51554 Metetra
.229 .717 .925 r
.54646 .51554 .57097 .52389 .61816 .52087 .59441 .51485 Metetra
.672 .948 .958 r
.59441 .51485 .61816 .52087 .66566 .49874 .64278 .49051 Metetra
.863 .951 .868 r
.64278 .49051 .66566 .49874 .71278 .459 .69063 .44444 Metetra
.911 .897 .801 r
.69063 .44444 .71278 .459 .7591 .40703 .73742 .3835 Metetra
.904 .84 .772 r
.73742 .3835 .7591 .40703 .80475 .35056 .78325 .31733 Metetra
.868 .787 .769 r
.78325 .31733 .80475 .35056 .85034 .29771 .8288 .25592 Metetra
.82 .769 .804 r
.18021 .44195 .20796 .46578 .25041 .42225 .22326 .39843 Metetra
.753 .773 .87 r
.22326 .39843 .25041 .42225 .29145 .39641 .26479 .37236 Metetra
.568 .729 .95 r
.26479 .37236 .29145 .39641 .33161 .39045 .30529 .36589 Metetra
.184 .537 .925 r
.30529 .36589 .33161 .39045 .37171 .40322 .3456 .37789 Metetra
0 .276 .754 r
.3456 .37789 .37171 .40322 .41266 .43034 .38666 .40403 Metetra
0 .172 .669 r
.38666 .40403 .41266 .43034 .45523 .46444 .42932 .43704 Metetra
0 .254 .744 r
.42932 .43704 .45523 .46444 .49986 .49612 .47407 .46762 Metetra
.126 .476 .897 r
.47407 .46762 .49986 .49612 .54646 .51554 .5209 .4861 Metetra
.495 .681 .949 r
.5209 .4861 .54646 .51554 .59441 .51485 .56927 .48473 Metetra
.718 .767 .891 r
.56927 .48473 .59441 .51485 .64278 .49051 .61824 .46002 Metetra
.821 .792 .824 r
.61824 .46002 .64278 .49051 .69063 .44444 .66688 .41388 Metetra
.867 .799 .781 r
.66688 .41388 .69063 .44444 .73742 .3835 .7146 .35305 Metetra
.884 .805 .766 r
.7146 .35305 .73742 .3835 .78325 .31733 .7614 .28706 Metetra
.878 .811 .779 r
.7614 .28706 .78325 .31733 .8288 .25592 .8079 .22574 Metetra
.831 .855 .865 r
.15024 .4294 .18021 .44195 .22326 .39843 .19347 .39162 Metetra
.739 .922 .953 r
.19347 .39162 .22326 .39843 .26479 .37236 .2355 .36761 Metetra
.407 .836 .955 r
.2355 .36761 .26479 .37236 .30529 .36589 .27672 .35916 Metetra
0 .527 .769 r
.27672 .35916 .30529 .36589 .3456 .37789 .31781 .36533 Metetra
0 .33 .682 r
.31781 .36533 .3456 .37789 .38666 .40403 .35958 .38253 Metetra
0 .332 .771 r
.35958 .38253 .38666 .40403 .42932 .43704 .40275 .40489 Metetra
.115 .442 .878 r
.40275 .40489 .42932 .43704 .47407 .46762 .44779 .42507 Metetra
.39 .534 .88 r
.44779 .42507 .47407 .46762 .5209 .4861 .49473 .43556 Metetra
.565 .589 .836 r
.49473 .43556 .5209 .4861 .56927 .48473 .5432 .4305 Metetra
.672 .629 .798 r
.5432 .4305 .56927 .48473 .61824 .46002 .59248 .40727 Metetra
.745 .669 .777 r
.59248 .40727 .61824 .46002 .66688 .41388 .64183 .36728 Metetra
.799 .714 .771 r
.64183 .36728 .66688 .41388 .7146 .35305 .69072 .31565 Metetra
.839 .768 .784 r
.69072 .31565 .7146 .35305 .7614 .28706 .73909 .25968 Metetra
.861 .835 .82 r
.73909 .25968 .7614 .28706 .8079 .22574 .78739 .20708 Metetra
.758 .944 .942 r
.11794 .42594 .15024 .4294 .19347 .39162 .16081 .39877 Metetra
.497 .907 .848 r
.16081 .39877 .19347 .39162 .2355 .36761 .20329 .37882 Metetra
.20329 .37882 .2355 .36761 .27672 .35916 .24559 .36711 Metetra
.045 0 0 r
.24559 .36711 .27672 .35916 .31781 .36533 .28809 .3631 Metetra
0 .575 .738 r
.28809 .3631 .31781 .36533 .35958 .38253 .33127 .36473 Metetra
.256 .664 .965 r
.33127 .36473 .35958 .38253 .40275 .40489 .37556 .36877 Metetra
.456 .614 .917 r
.37556 .36877 .40275 .40489 .44779 .42507 .42123 .37133 Metetra
.544 .564 .825 r
.42123 .37133 .44779 .42507 .49473 .43556 .46834 .36857 Metetra
.595 .548 .772 r
.46834 .36857 .49473 .43556 .5432 .4305 .51672 .35762 Metetra
.635 .557 .75 r
.51672 .35762 .5432 .4305 .59248 .40727 .56602 .33721 Metetra
.674 .587 .75 r
.56602 .33721 .59248 .40727 .64183 .36728 .61585 .30801 Metetra
.715 .639 .771 r
.61585 .30801 .64183 .36728 .69072 .31565 .66593 .27251 Metetra
.761 .723 .818 r
.66593 .27251 .69072 .31565 .73909 .25968 .71617 .23438 Metetra
.794 .85 .893 r
.71617 .23438 .73909 .25968 .78739 .20708 .7668 .19766 Metetra
.432 .87 .904 r
.08349 .42744 .11794 .42594 .16081 .39877 .12547 .414 Metetra
.12547 .414 .16081 .39877 .20329 .37882 .1683 .39943 Metetra
.1683 .39943 .20329 .37882 .24559 .36711 .21197 .3836 Metetra
.21197 .3836 .24559 .36711 .28809 .3631 .25645 .36658 Metetra
.462 .889 .872 r
.25645 .36658 .28809 .3631 .33127 .36473 .30171 .3486 Metetra
.665 .862 .974 r
.30171 .3486 .33127 .36473 .37556 .36877 .34771 .33007 Metetra
.666 .696 .868 r
.34771 .33007 .37556 .36877 .42123 .37133 .39444 .31143 Metetra
.639 .593 .786 r
.39444 .31143 .42123 .37133 .46834 .36857 .44192 .29307 Metetra
.619 .541 .745 r
.44192 .29307 .46834 .36857 .51672 .35762 .49019 .27531 Metetra
.61 .523 .731 r
.49019 .27531 .51672 .35762 .56602 .33721 .53931 .25827 Metetra
.609 .531 .741 r
.53931 .25827 .56602 .33721 .61585 .30801 .58935 .24186 Metetra
.616 .57 .779 r
.58935 .24186 .61585 .30801 .66593 .27251 .64038 .22581 Metetra
.631 .657 .857 r
.64038 .22581 .66593 .27251 .71617 .23438 .69245 .20972 Metetra
.626 .812 .972 r
.69245 .20972 .71617 .23438 .7668 .19766 .74555 .19312 Metetra
.048 0 0 r
.04742 .42842 .08349 .42744 .12547 .414 .08816 .42922 Metetra
.005 0 0 r
.08816 .42922 .12547 .414 .1683 .39943 .1312 .42031 Metetra
.1312 .42031 .1683 .39943 .21197 .3836 .17638 .40014 Metetra
.525 .904 .738 r
.17638 .40014 .21197 .3836 .25645 .36658 .22317 .36956 Metetra
.79 .988 .891 r
.22317 .36956 .25645 .36658 .30171 .3486 .27095 .33154 Metetra
.829 .869 .875 r
.27095 .33154 .30171 .3486 .34771 .33007 .3191 .29052 Metetra
.772 .729 .813 r
.3191 .29052 .34771 .33007 .39444 .31143 .36725 .2513 Metetra
.705 .627 .767 r
.36725 .2513 .39444 .31143 .44192 .29307 .41535 .21814 Metetra
.646 .558 .742 r
.41535 .21814 .44192 .29307 .49019 .27531 .46362 .19397 Metetra
.593 .515 .736 r
.46362 .19397 .49019 .27531 .53931 .25827 .51251 .18002 Metetra
.544 .495 .751 r
.51251 .18002 .53931 .25827 .58935 .24186 .56255 .17559 Metetra
.496 .504 .795 r
.56255 .17559 .58935 .24186 .64038 .22581 .61421 .17821 Metetra
.437 .556 .878 r
.61421 .17821 .64038 .22581 .69245 .20972 .66774 .18387 Metetra
.336 .662 .973 r
.66774 .18387 .69245 .20972 .74555 .19312 .72307 .18771 Metetra
.306 0 0 r
.01057 .42296 .04742 .42842 .08816 .42922 .05004 .43542 Metetra
.05004 .43542 .08816 .42922 .1312 .42031 .09321 .43114 Metetra
.494 .903 .814 r
.09321 .43114 .1312 .42031 .17638 .40014 .13979 .40724 Metetra
.802 .994 .87 r
.13979 .40724 .17638 .40014 .22317 .36956 .18887 .36529 Metetra
.888 .93 .841 r
.18887 .36529 .22317 .36956 .27095 .33154 .23917 .31084 Metetra
.875 .836 .804 r
.23917 .31084 .27095 .33154 .3191 .29052 .28956 .2518 Metetra
.825 .748 .778 r
.28956 .2518 .3191 .29052 .36725 .2513 .33931 .1965 Metetra
.758 .669 .764 r
.33931 .1965 .36725 .2513 .41535 .21814 .38824 .15195 Metetra
.677 .599 .761 r
.38824 .15195 .41535 .21814 .46362 .19397 .43671 .12282 Metetra
.584 .536 .767 r
.43671 .12282 .46362 .19397 .51251 .18002 .48548 .11101 Metetra
.475 .482 .785 r
.48548 .11101 .51251 .18002 .56255 .17559 .53544 .11547 Metetra
.349 .443 .816 r
.53544 .11547 .56255 .17559 .61421 .17821 .58741 .13228 Metetra
.206 .432 .859 r
.58741 .13228 .61421 .17821 .66774 .18387 .64192 .15492 Metetra
.074 .471 .889 r
.64192 .15492 .66774 .18387 .72307 .18771 .6989 .17516 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<000Uoo`L001eoo`03
001oogoo00Moo`03001oogoo02Aoo`00DWoo0`001goo10000TXh00<003gi?OT00Sgi1P005goo00<0
07ooOol027oo00<007ooOol08goo001?Ool30007Ool30007BSP00`00?OTmn@04?OT00`00;MX]fP04
000COol00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<000Qoo`800003EWP004Xh00M:>003000m
nCgi00Dmn@03000]fRgJ00<]fP<0011oo`03001oogoo00Uoo`03001oogoo029oo`00BWoo0`002Goo
00<005IhEWP00UIh00<004XhBSP01DXh00<003gi?OT01Cgi00<002gJ;MX012gJ00@001Vk000000io
o`03001oogoo00Yoo`03001oogoo025oo`00Agoo0`002Woo0P001EIh00<004XhBSP01TXh00<003gi
?OT01Cgi00<002gJ;MX012gJ00<001Vk6K/00`002goo00<007ooOol02goo00<007ooOol087oo0015
Ool2000;Ool20008EWP00`00BSQ:>005BSP00`00?OTmn@05?OT00`00;MX]fP05;MX00`006K/I^`02
6K/20009Ool00`00Oomoo`0;Ool00`00Oomoo`0POol0049oo`<000aoo`03001R^00000QFN003001:
>4Xh00E:>003000mnCgi00Dmn@03000]fRgJ00D]fP03000I^aVk00@I^`<000Ioo`03001oogoo00ao
o`03001oogoo01moo`009Goo10005goo0P003Woo00D006:hH[QR^00000QFN003001:>4Xh00E:>003
000mnCgi00Dmn@03000]fRgJ00D]fP03000I^aVk00@I^`030009o000008000=oo`03001oogoo00eo
o`03001oogoo01ioo`009Woo00<007ooOol057oo0`003goo00<006:hH[P00V:h00<005IhEWP01UIh
00<004XhBSP01DXh00<003gi?OT01Cgi00<002gJ;MX01BgJ00<001Vk6K/01AVk00@000Wl2O`9o0@0
00ioo`80021oo`007goo1@000goo00<007ooOol04Goo0P004Goo00<006:hH[P00f:h00<005IhEWP0
1UIh00<004XhBSP01DXh00<003gi?OT00Sgi2P000RgJ00<001Vk6K/01AVk00<000Wl2O`00`Wl0P00
3goo00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000BOol00`00H[QR^005H[P00`00EWQFN006
EWP00`00BSQ:>003BSP60004AOP00`00?QTn6@03?QTA00000`Wl0000000=Ool00`00Oomoo`0MOol0
02Eoo`04001oogoo000=Ool2000DOol00`00H[QR^006H[P00`00EWQFN006EWP00`00BSQ:>0030002
BQL00`00AOQ5n006AOP00`00?QTn6@05?QT00`00=S`f?005=S`00`00:[lZ_`03:[l6000;Ool00`00
Oomoo`0LOol002Ioo`8000]oo`<001Eoo`03001R^6:h00MR^003001FN5Ih00IFN0<000I:5`030015
n4Gh00E5n003000n6ChI00Hn6@03000f?3Hl00Df?003000Z_bZo008Z_`03000Z_bZo00<Z_`@000Qo
o`03001oogoo01]oo`00<Goo0P0000=oo`00Ool05Goo00<006:hH[P026:h00<005IhEWP00eIh0P00
00=B=`00BQL01TXG00<004GhAOP01TGh00<003hI?QT01ChI00<003Hl=S`01SHl00D002Zo:[lZ_`00
008Z_`<000=C?`@000Eoo`03001oogoo01]oo`00;Woo0`006Goo00<006[h00002F:h00@005IhEWQF
N08000=B=`03001:5dXG00E:5`030015n4Gh00I5n003000n6ChI00Hn6@03000f?3Hl00Hf?003000Z
_bZo00<000EC?`03001WO6Ml009WO003001oogoo009oo`8001aoo`00:goo0`006goo00@006[hJ_P0
00UR^004001FN0000006DSL00`00BQM:5`05BQL00`00AOQ5n006AOP00`00?QTn6@06?QT00`00=S`f
?004=S`50005Dcl20005Ig`20005Ool00`00Oomoo`03Ool2000DOol002Uoo`80009oo`03001oogoo
01Qoo`03001Zn6[h009Zn003001R^6:h00IR^08000QB=`03001:5dXG00E:5`030015n4Gh00I5n003
000n6ChI00Hn6@03000f?3Hl00@000AB^`04001C?e<oDcl20006Ig`01000KeY_FP0017oo00<007oo
Ool00Woo00@007ooOol001=oo`009Woo0`007Woo00<006[hJ_P00f[h00<006:hH[P01F:h00<005ZH
0000258g00<004XGBQL01TXG00<004GhAOP01DGh00<003hI?QT01ShI0`0025:k0P0000=C?`00Ig`0
1VMl00<006mJKeX00VmJ00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo
0P0087oo00<006[hJ_P016[h00<006:hH[P00f:h0P000eZH00<0058gDSL01U8g00<004XGBQL01DXG
00<004GhAOP01TGh00<003hI?QT00ShI0`0000=>F000D[/025:k0`001fMl00<006mJKeX00fmJ00<0
07ooOol00goo00@007ooOol0009oo`03001oogoo015oo`008Goo0`008Goo00<006[hJ_P01V[h00<0
06:hH[P00P001EZH00<0058gDSL01U8g00<004XGBQL01TXG00<004GhAOP01DGh00<003hI00000P00
1DiH00<005:kD[/01U:k00@006;jH_X000EWO003001_FVmJ00=_FP03001c>00000=oo`80009oo`04
001oogoo000COol001moo`8002=oo`03001Zn6[h00MZn003001R^00000MJV003001B=e8g00IB=`03
001:5dXG00I:5`030015n4Gh00A5n0<000Q>F003001B^e:k00EB^`8000=RnP05001WO6MlIg`00005
KeX01@00LcQc>7<h000017oo00<007ooOol00P0057oo000LOol3000TOol00`00J_QZn008J_P20009
FYP00`00DSMB=`06DSL00`00BQM:5`05BQL00`00AOQ5n0040002CQL00`00CUQ>F007CUP01000D[]B
^e:k0`0000=RnP00H_X00f;j00<006Ml00001VmJ00<007<hLcP00W<h00<007ooOol00Woo00<007oo
Ool05Goo000KOol2000VOol00`00J_QZn007J_P00`00H_X00009FYP00`00DSMB=`06DSL00`00BQM:
5`05BQL30007CQL00`00CUQ>F006CUP400000e:k001RnP02H_X00`00H_YRnP02H_X00`00KeY_FP04
KeX00`00LcQc>004LcP01@00Oomoogoo00005goo000DOol30004Ool00`00Ool0000TOol00`00KeT0
0007J_P01@00H_YRnV;j000025ZH00<0058gDSL01U8g00<004XGBQL00TXG0`0000=>5`00CQL01dhG
00<004iHCUP014iH10000U:k00<006;jH_X00f;j00D006;jH_YRnP0000E_FP03001c>7<h00Ec>003
001oogoo009oo`03001oogoo01Aoo`005Goo00<007ooOol00goo00@007ooOol0029oo`05001_FFmI
KeT00005J_P00`00H_YRnP02H_X00`00FYQJV007FYP00`00DSMB=`06DSL40004CQL00`00CQM>5`06
CQL01000CUQ>F4iH0`000diH00<005:k00001f;j00D006;j001[6@0000=_FP03001c>7<h00Ec>003
001_>00000=oo`8001Eoo`003Woo1@000Woo00<007ooOol00goo00D007ooOomoo`00021oo`03001_
FFmI00=_F@05001Zn6[hJ_P00006H_X00`00FYQJV006FYP00`00DSMB=`05DSL20008CQL00`00CQM>
5`06CQL30006CUP20008H_X20003JaT01000KeY_FP001W<h00@006lhKcP000Aoo`03001oogoo01=o
o`005Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol077oo00<006mIKeT01FmI00<006[h
00001f;j00<005ZHFYP01UZH00<0058gDSL00U8g0`0000=>=`00CQL01dhG00<004hGCQL00dhG1000
1diH00<005ZH000026;j0P000f/I00<006mJ00001g<h00D006lhKcQ_>00000Aoo`03001oogoo019o
o`005Goo00<007ooOol00Woo0P001Goo00<007ooOol06goo00<006mIKeT01FmI0P002F;j00<005ZH
FYP01UZH100014hg00<004hGCQL01ThG00<004hGCQL00`0014hG00<004iHCUP00TiH0P000eZH00<0
06;jH_X016;j00@006/IJaT000=[6@03001c>7<h00Ec>003001_>6lh009_>003001oogoo009oo`03
001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo00<007ooOol06Goo00<006mIKeT01FmI00<0
06]k00002F;j00<005ZHFYP01EZH0P0024hg00<004hGCQL01ThG0P001dhG00<004iHCUP00P001UZH
00<006;jH_X00V;j00D006/IJaU[6@0000=[6@03001c>7<h00Ac>003001_>6lh00=_>003001oogoo
00=oo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol05goo00<006mIKeT01FmI00D0
06]kJg][N`0000URnP04001JV5ZHFYP300000e9I001>=`07CSL00`00CQM>5`03CQL40008CQL00`00
CUP00008FYP01@00H_YRnV;j00001F/I00D006/I001c>00000Ac>003001_>6lh00E_>003001oogoo
00=oo`03001oogoo011oo`006Woo00<007ooOol01goo00<007ooOol05Goo00<006mIKeT01FmI00<0
06]kJg/00V]k00<006;jH_X01f;j00@005ZH000000EBF@03001>=dhg00E>=`03001>5dhG00<000=>
5`03001>5dhG00I>5`8000YJV003001RnP0000M[6@05001c>7<hLcP00003LcP00`00KcQ_>005KcP0
0`00Oomoo`02Ool2000BOol001Uoo`8000Yoo`03001oogoo01=oo`8000I_F@03001[Nf]k00A[N`03
001RnV;j00IRnP8000MBF@03001>=dhg00E>=`<000M>5`03001>5dhG00A>5`03001FE`0000]JV003
001[6F/I00I[6@8000=c>003001c>00000M_>003001[6@0000Eoo`03001oogoo00moo`0027oo0P00
17oo0P000goo0`000goo0P002goo00<007ooOol04Goo00<007>jLkX00P000fmI00<006]kJg/01V]k
00<006;jH_X00V;j0`0000=Ff`00DUT01U9I00<004hgCSL00Thg0`0000=>=`00CQL01dhG00<004hG
CQL00ThG0P000eIG00<005ZHFYP025ZH00<006/IJaT01F/I00<007<h000017<h00<006lhKcP01Flh
00@006/IJaT000Ioo`03001oogoo00ioo`001goo00@007ooOol000=oo`8000Ioo`04001oogoo000=
Ool00`00Oomoo`0?Ool00`00LkYc^P03LkX00`00KeT00009Jg/00`00H_X000020005E]/00`00DUUB
F@05DUT40004CSL00`00CQM>5`06CQL00`00CQM>5`020005EUL00`00FYQJV007FYP00`00I]P00005
JaT01@00LcQc>7<h00000g<h00<006lhKcP016lh00<006/IJaT00V/I00<007ooOol00goo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo00eo
o`03001c^W>j00Ac^P<000U[N`8000QFf`03001BFE9I00=BF@<000Q>=`03001>5dhG00I>5`8000MF
E`03001JV5ZH00IJV005001Vf6KHI]P00003JaT00`00LcQc>003LcP01@00LcP007=H000016lh00<0
06/IJaT00f/I00<007ooOol017oo00<007ooOol03Goo0007Ool01000Oomoo`0027oo0`000goo00<0
07ooOol03Goo0P0037oo00<007>jLkX017>j00@006OlIoaWo08000A[N`<00003Egl005KK00QFf`03
001BF@000080009BF@03001>=dhg00I>=`03001>5dhG00E>5`8000UFE`03001JV5ZH00AJV003001V
f6KH009Vf005001[6F/IJaT00005LcP20003LeP01000KcQ_>0001f/I00<007ooOol017oo00<007oo
Ool00Woo10001Woo0007Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol03goo00<007oo
Ool027oo00<007>jLkX017>j00<006OlIo`016Ol00@006]k000000EGO`03001FfeKK00IFf`8000IB
F@03001>=dhg00E>=`03001>5dhG009>5`<00003DSL005IG00QFE`03001JV5ZH00=JV003001Vf6KH
00AVf003001[6@0000Ic>08000=cF003001_>00000Q[6@03001oogoo00Aoo`03001oogoo00=oo`03
001oogoo00Ioo`0027oo0P002Goo10000Woo00<007ooOol047oo00<007ooOol01Woo00<007>jLkX0
0g>j0P001VOl10001eMo00<005KKE]/00UKK10001e9I00<004hgCSL01Thg10001E8g00<005IGEUL0
1eIG00D005ZHFYQJV00000QVf003001c>7<h00Ac>004001cF7=H0003LeP00`00JaU[6@07JaT20006
Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qoo`8001=oo`03001oogoo00Aoo`03001c^W>j00=c
^P03001Wo6Ol00=Wo0<000@kV`<000EGO`@000AFf`03001BFE9I00ABF@03001>=dhg00A>=`<000QB
=`03001FEeIG00MFE`04001JV5ZH0009I]P00`00LcQc>004LcP01@00LeQcF7=H00000W=H00<006/I
JaT01V/I00<006;j00001goo00<007ooOol00goo00<007ooOol017oo000HOol2000DOol01000Oomo
ogoo1@000W>j00<006Ol00005@001UKK00<0059IDUT01E9I00<004hg00000`000Thh00<0058gDSL0
1U8g00<005IGEUL01eIG00<005ZH00002FKH00<006lh000017<h00<007=HLeP00W=H00D007=H001c
V@0000I[6@05001RnV;jH_X00005Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomo
o`0DOol00`00Ool00005Io/<0004B_h00`005Sd0000300028[hB0007CSP00`00DSMB=`06DSL00`00
EUMFE`06EUL20009I]P01@00KcQ_>6lh00000W<h00<007=HLeP017=H00@007>ILiT000A[6@8000AR
nP03001oogoo00Eoo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo01Eoo`03
001WnfOk00=Wn`030013U`000080009O_P04001S6f<K0006B_h300025Sd200038[h00`00>WdjO@04
>Wd00`00AUY6FP04AUX00`00CSQ>>006CSP00`00DSMB=`06DSL00`00EUMFE`05EUL00`00G[P00008
I]P00`00KcQ_>002KcP00`00LcP00006LeP00`00LiT00002LiT01000JaU[6@001V;j00<007ooOol0
1Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol057oo00<006OkIo/00VOk1000
0enn0P0016<K00<004[nB_h00`000T[n00@001Hm5SdF?@<0008R_P03000jOCYm00@jO@030016FTIJ
00A6FP03001>>4hh00E>>003001B=e8g00IB=`03001FEeIG00AFE`05001N^5jhG[P00006I]P00`00
KcQ_>004KcP00`00LeQcF004LeP02@00LiUcVG>I001cV@00JaT00008H_X00`00Oomoo`05Ool00`00
Oomoo`08Ool0015oo`04001oogoo0002Ool2000EOol00`00Io]Wn`02Io/30004Gkh00`00Ha]S6`05
Ha/20003B_h600000aHm0000000200000b:n000jO@04>Wd00`00AUY6FP04AUX00`00CSQ>>006CSP0
0`00DSMB=`06DSL01000EUMFEeIG0P0015jh00<006KHI]P016KH00<006lhKcP016lh0P001G=H00H0
07>ILiUcV@00LiT20009H_X00`00Oomoo`06Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool0
0`00Oomoo`0BOol00`00Io]Wn`02Io/20002Jg/200000enn001S6`04Ha/80003BWX00`00<O/an`03
0000104H000000001SYm00<004IJAUX014IJ00<004hhCSP01Dhh00<0058gDSL01U8g00@005IGEUL0
00MN^003001Vf6KH009Vf003001_>6lh00A_>004001gVGNI0003LeP00`00LiUcV@03LiT00`00Jk`0
0009H_X00`00Oomoo`06Ool00`00Oomoo`07Ool0019oo`8000=oo`8001=oo`03001Wn`0000<000I[
N`80009S6`<0009S6`03001J^@0000M:NP03000anc7k008an`D000DjO@030016FTIJ00A6FP03001>
>4hh00E>>003001B=e8g00IB=`03001FE`0000UN^004001Vf6KH0006KcP00`00MiUgV@02MiT00`00
LeP00005LiT01@00Jk`006^l00002F;j00<007ooOol01Woo00<007ooOol01Woo000GOol00`00Oomo
o`0@Ool70006Jg/00`00JaT000020003Ha/20003F[T00`00BWY:NP04BWX00`00<O/an`02<O/00`00
2GX000040003>Wd00`00AUY6FP05AUX00`00CSQ>>005CSP00`00DSMB=`05DSL2000:G[P00`00I]P0
0007KcP00`00MiUgV@03MiT00`00LiUcV@03LiT01000Jka[_0000V^l00<006;jH_X016;j0`002Goo
00<007ooOol01Goo000GOol00`00Oomoo`0=Ool300000cnJ00000002Hmh00`00Jg][N`04Jg/30003
JaT00`00Ha/00005F[T00`00BWY:NP05BWX00`00<O/an`02<O/00`002GX0000200001PgL00000000
>Wd000I6FP03001>>4hh00E>>003001B=e8g00=B=`800003EYT005jh00UN^08000M_>003001gVGNI
00=gV@03001cnP0000=cV@03001[_6^l009[_004001[_6^l0002H_X30002E]/00`00Oomoo`07Ool0
0`00Oomoo`05Ool001Moo`@000Uoo`@00003?iX0000000=SgP03001[Nf]k009[N`<000M[6@03001J
^EZi00EJ^@03001:NTYj00E:NP03000anc7k008an`030009NPUj008000<=g08000M6FP03001>>4hh
00A>>004001B=e8gDSL20004EYT00`00G[QN^006G[P00`00JcX00006KcP00`00MiUgV@03MiT01`00
LoYcnW?j001cV@0000I[_003001[_000008000IFf`03001oogoo00Moo`03001oogoo00Aoo`001Goo
0P0017oo0P000goo0`0017oo0P0000=oo`0000004`0000I[N`00000006]k0007JaT20008F[T00`00
BWY:NP04BWX00`00<O/an`02<O/010002GX9NPUj0P0010gL1@000TIJ00<004hhCSP01Dhh0`001UJI
00<005jhG[P01Ejh00D006/jJcY[>P0000A_>003001gVGNI00=gV@03001cnW?j009cnP<000E[_0<0
0003Dgl005KK00IFf`03001oogoo00Ioo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001o
ogooOol00005Ool90007Ool500000f]k001[6@06JaT01000I]QVf0001eZi00<004YjBWX00TYj1`00
00<an`002GX00PUj0P000`gL00<0038l<S`00S8l2`002EJI00<005jhG[P00ejh00<006/jJcX00f/j
00D006lhKcQ_>00000AgV@03001cnW?j009cnP06001GnEOiEoT006^l0`000U=o00@005=oDgl000MF
f`03001oogoo009oo`<0009oo`03001oogoo00=oo`0017oo00@007ooOol000]oo`03001oogoo0080
01=oo`8000Eoo`03001[6F/I00E[6@05001Vf6KHI]P00007F[T50003CSP01000?OTmnCgi100000<9
NP000=L00P000PgL00<0038l<S`0138l00<004YJBUX014YJ00<005JIEYT01eJI00<005jhG[P00Ujh
00<006/jJcX01F/j00<006lh000017NI00@007?jLoYcnP8000AGn@<000ECO`03001CO`0000MFf`03
001oo`00008000Eoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`<000=oo`80011oo`<000Qo
o`03001[6F/I00=[6@03001Vf6KH00=Vf003001J^EZi00=J^@03001FN00000M>>003000mnCgi008m
n@03000Y^P00008000050=L000003M`00007<S`00`00BUY:FP04BUX00`00EYUFV@07EYT01000G[QN
^00026/j0`000gNI00D007?jLoYcnP0000=Gn@<000<S5P03001COe=o00=CO`03001CO`0000IFf`80
00Qoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo00aoo`<0
00aoo`05001[6F/IJaT00007I]P00`00F[UJ^@020003EWP00`00CSQ>>004CSP00`00?OTmn@03?OT0
0`00:KXY^P0200000`3G000000020006<S`00`00BUY:FP05BUX00`00EYUFV@06EYT00`00G[P00008
JcX01`00Km]_ffoK001gV@0000=cnP03001GnEOi00h00003Dgl0000000MFf`03001oogoo00Ioo`03
001oogoo00=oo`001Goo0P002Goo10000Woo00<007ooOol02Goo0`0047oo00<006/I000026KH00<0
05Zi00001EIh00<004hhCSP014hh00<003gi?OT00cgi00@002Vj:KXY^P8000030=L0004G008000Hb
?003001:FTYJ00A:FP03001FVEJI00IFV@8000Q[>P03001_ffoK009_f`/000@VGP<00003200002Jo
008V_`P0009Ff`03001oogoo00Moo`03001oogoo009oo`005Woo00<007ooOol01Woo0`0057oo00<0
06KHI]P01fKH0P001UIh00<004hhCSP01Dhh00<003gi?OT00cgi00<002Vj:KX00RVj1@001S8l00<0
04YJBUX01DYJ00<005JIEYT00eJI0P0000=K7000JcX01V/j00<006oKKm/00VoK00@005?GDmMCe`<0
00E:oP03000VGRIN008VGP030005V`00008000@V_`8000@nO0@000Qoo`03001oogoo009oo`005Goo
0P001Goo0`005goo00<006KHI]P01VKH00<006:h00001eIh00<004hhCSP014hh00<003gi?OT00cgi
00<002Vj:KX00RVj00<000VJ00001@00138l00<004YJBUX014YJ00<005JIEYT00UJI00<005/LFa`0
0U/L00<006/jJcX00f/j00<006oKKm/00VoK00<005?GDmL00P0000=S?000B_h00d[n1P000RIN00<0
00FK00000`000bJo0P001Sil00<007ooOol01Woo00<007ooOol00Woo000EOol01000Oomoogoo0`00
6goo00<006KHI]P016KH00D006:hH[QR^00000IFN003001>>4hh00A>>003000mnCgi00@mn@03000Y
^RVj008Y^P030009VP0000<000034O`0000000<000M:FP04001FV@000006Fa`01@00JcY[>V/j0000
16oK00@005?G000000=S?0D000=>NP03000j6SXJ00@00085V`H000039[l003il00DnO003001oogoo
00Eoo`03001oogoo009oo`005Goo10007goo00<006KHI]P00VKH00<006:hH[P00V:h00<005IhEWP0
1EIh00<004hhCSP014hh00<003gi?OT00cgi00<002Vj:KX00bVj00H000VJ2IX0003E00044O`:0009
Fa`00`00JcX00004Km/00`00DmL00004Hc`00`00F[T00007CWX00`00>QXj6P02>QX300021I/00`00
0M/1f`030006?W`00`00Oomoo`05Ool00`00Oomoo`02Ool000ioo`<000Aoo`80025oo`05001Vf6KH
I]P00006H[P00`00EWQFN004EWP00`00CSQ>>003CSP<00000bVj0009VP022IX300044O`00`00?[hn
_P05?[h00`00FaaK7007Fa`40002Km/30003Hc`20003F[T00`00CWY>NP04CWX00`00>QXj6P03>QX0
0`008K/000021I/200020M/30004?W`00`00Oomoo`05Ool00`00Oomoo`02Ool000moo`03001oogoo
00Eoo`03001oogoo01moo`03001Vf00000MR^003001FN5Ih00AFN003001>>4hh00<00003CQL004HG
00E65`0400126D8I@QT500022IX300034O`00`00?[hn_P05?[h00`00FaaK7004Fa`20004Emh30002
JcX00`00Hc`00005F[T00`00CWY>NP05CWX00`00>QXj6P03>QX00`008K/0000200001P5J0001f`7K
000R708000<nO003001oogoo00Aoo`03001oogoo009oo`003goo00<007ooOol01Woo00<007ooOol0
7goo00<006:hH[P01V:h00<005IhEWP015Ih0`001DhG00<004HGAQL014HG00<0048I@QT00T8I00<0
03Hl=S`0100000H1F0000000017l0007?[h00`00FaaK7002Fa`00`00EmiGgP02Emh20005JcX20007
F[T00`00CWY>NP04CWX00`00>QXj6P03>QX010008K/Q^b6k0P0000<1FP0000000P000R8L0P0000<n
O000Ool01Woo00<007ooOol00Goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0NOol00`00H[QR^007
H[P01000EWQFN5Ih0P0000=B=`00CQL01DhG00<004HGAQL014HG00<0048I@QT00d8I00<003Hl=S`0
0SHl20001cjn00@005/L000000=GgP<000E[>P03001Vf00000MJ^@03001>NTij009>NPH000<j6P03
000Q^b6k008Q^`030001FP5J00<000<R708000Moo`03001oogoo005oo`003goo00<007ooOol027oo
0P007goo00<006:hH[P01V:h00@005Ih000000AB=`03001>5dhG00=>5`0300165dHG00E65`030012
6D8I00=26@03000f?3Hl00@f?08000036Wh0000000/00005Emh00000Ji`00005JcX01@00I]QVf6KH
00001UZi1@001Woo10001B6k0P000P5J00<0028L8Q`00b8L00<007ooOol017oo00<007ooOol00Goo
000>Ool2000<Ool00`00Oomoo`0LOol00`00H[QR^006H[P20006DSL00`00CQM>5`04CQL00`00AQM6
5`04AQL00`00@QU26@03@QT00`00=S`f?004=S`40004Cel90002KcT00`00JcY[>P03JcX00`00I]QV
f002I]P00`00F[UJ^@03F[T00`00Oomoo`0<Ool300038K/010002GX0005J0`000b8L00<007ooOol0
17oo00<007ooOol00Goo000MOol00`00Oomoo`0LOol00`00H[QR^003H[P200000eYh001B=`05DSL0
0`00CQM>5`04CQL00`00AQM65`04AQL00`00@QU26@04@QT00`00=S`f?0030004DY/01@00Cem?GdmO
000016ML00<006liKcT00fli0P000f/j00<006KHI]P016KH00<005ZiF[T00P004Woo0`0000<Q^`00
2GX00`0000<5V`008Q`00R8L00<007ooOol00goo00<007ooOol00Goo000NOol00`00Oomoo`0LOol0
1@00H[QR^6:h00000eYh00<0058gDSL01E8g00<004hGCQL00dhG00<004HGAQL01DHG00<0048I@QT0
0d8I0`0025:K00<004mO000016ML00<006liKcT00fli00H006lhKcP006/j0007I]P00`00F[T0000G
Ool300022GX200001@FK0000028L00001Goo00<007ooOol00Goo000OOol2000LOol01000H[P00000
1EYh00<0058gDSL0158g00<004hGCQL014hG00<004HGAQL014HG00<0048I@QT00`0000=>F000DY/0
25:K0`000fML00<006liKcT00fli00<006lhKcP00Vlh00<006KHI]P01VKH0P006goo0P000PUj00<0
00FK1I/00`0017oo00<007ooOol00Goo000QOol00`00Oomoo`0JOol00`00FWQJN005FWP00`00DSMB
=`04DSL00`00CQM>5`04CQL00`00AQM65`03AQL40005CUP00`00DY]BV`05DY/01`00H]URfF;I001W
G00000E_>@03001_>6lh00=_>003001Vf6KH00EVf003001oogoo01eoo`D00085V`03001oogoo00=o
o`40005oo`5oo`008Woo00<007ooOol06Goo00<005YhFWP01EYh00<0058gDSL01E8g00<004hGCQL0
0dhG00<004HGAQL00`000Thg00<004iHCUP01DiH00<005:KDY/00U:K0P001F;I0P001Fli00<006lh
KcP00Vlh00<006lH00001FKH00<007ooOol087oo10000PFK00<007ooOol00Woo0@000Goo0Goo000S
Ool00`00Oomoo`0IOol00`00FWQJN005FWP00`00DSMB=`04DSL00`00CQM>5`03CQL30006CSL00`00
CUQ>F004CUP01@00DY]BVe:K00001f;I00<006[h00000fli00<006lhKcP00Vlh00@006lHKaP000AV
f003001oogoo02=oo`D000Aoo`40005oo`5oo`0097oo00<007ooOol067oo00<005YhFWP01EYh00<0
058gDSL0158g00<004hGCQL00`0000=>5`00CSL01Thg00<004iHCUP01DiH00<005:K00001f;I00L0
06[hJ_QZn000KcT00005KcP01@00KaQ_66lH00000VKH00<007ooOol09Goo1@000goo0@000Goo0Goo
000UOol2000IOol00`00FWQJN004FWP00`00DSMB=`03DSL40005CQL00`00CSM>=`04CSL00`00CUQ>
F005CUP20007H]T00`00J_QZn003J_P00`00KcQ_>003KcP00`00KaQ_6002KaP00`00I]P0000XOol0
0`00Oomoo`030002Ool10001Ool1Ool002Moo`03001oogoo01Ioo`03001JN5Yh00EJN003001B=`00
0080009B=`03001>5dhG00A>5`03001>=dhg00E>=`03001>F4iH009>F0800003FYP006;I00ERf@03
001Zn6[h00AZn003001_>6lh009_>003001_66lH00A_6003001oogoo02Qoo`04001oogooOol40002
Ool002Qoo`03001oogoo01Ioo`03001JN5Yh00=JN0<000EB=`03001>5dhG00A>5`03001>=dhg00E>
=`05001>F4iHCUP00004FYP00`00H]URf@03H]T00`00J_QZn003J_P00`00KaP00003KcP00`00KaQ_
6004KaP00`00Oomoo`0YOol00`00Oomoo`02Ool20002Ool002Uoo`03001oogoo01Eoo`03001JN5Yh
00<00003DUP0058g00IB=`03001>5dhG00A>5`03001>=dhg00A>=`04001>F0000006FYP01@00H]UR
fF;I00001F[h00L006lHKaQ_6000KcP00006KaP00`00Oomoo`0ZOol00`00Oomoo`04Ool10001Ool0
02Yoo`8001Ioo`8000EBF003001B=e8g00AB=`03001>5dhG00A>5`03001>=dhg00A>=`8000UJV003
001Rf@0000IZn003001_66lH009_6003001_66lH00A_6003001oogoo02aoo`03001oogoo00=oo`40
005oo`00;7oo00<007ooOol04goo00<0059HDUP0159H00<0058gDSL0158g00<004hGCQL01DhG00<0
04hgCSL00Thg00<005IG00002UZH00<006[hJ_P016[h00<006lHKaP00flH00<006lHKaP00flH00<0
07ooOol0;Woo00<007ooOol00Woo0@000Goo000]Ool00`00Oomoo`0COol00`00DUQBF003DUP00`00
DSMB=`05DSL00`00CQM>5`04CQL00`00CSM>=`020003EUL00`00FYQJV006FYP20005J_P00`00KaQ_
6003KaP00`00LcP00003KaP00`00Oomoo`0`Ool01@00Oomoogoo00000Goo000^Ool00`00Oomoo`0B
Ool00`00DUQBF004DUP00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CSL00005EUL00`00FYQJV005
FYP01000I]QVf0000f[h00<006lHKaP00flH00L007<hLcQc>000KaP0000cOol01@00Oomoogoo0000
0Goo000_Ool2000COol00`00DUQBF003DUP00`00DSMB=`04DSL00`00CQM>5`04CQL20007EUL00`00
FYQJV003FYP00`00I]QVf002I]P01000J_QZn0001FlH00<007<hLcP00W<h00<007ooOol0<goo00@0
07ooOol0005oo`00<Goo00<007ooOol047oo00<0059HDUP0159H00<0058gDSL00e8g00@004hGCQM>
5`<00003DSL005IG00IFE`03001JV5ZH009JV003001Vf6KH00=Vf003001Zn00000E_6003001c>7<h
00=c>003001oogoo03Aoo`03001oo`00005oo`00<Woo00<007ooOol047oo00<0059HDUP00e9H00<0
058gDSL0158g0`001E8g00<005IGEUL01EIG00@005ZHFYP000MVf08000A_6003001c>7<h00=c>003
001oogoo03Eoo`03001oogoo00400000<goo00<007ooOol03goo00<0059HDUP00e9H00@0058gDSMB
=`@000MB=`03001FEeIG00EFE`03001JV00000QVf003001_600000=_6003001c>7<h009c>003001o
ogoo03Moo`40005oo`400000=7oo0P0047oo2@000dhh00<0058gDSL01E8g00<005IGEUL01EIG0P00
26KH00L006lHKaQ_6000KaP00004LcP00`00Oomoo`0iOol2000003Ioo`03001oogoo00eoo`030016
FTIJ00=6FP03001>>4hh00A>>003001B=e8g00EB=`03001FEeIG00=FE`03001N^00000MVf003001_
66lH00=_6003001c>7<h00=c>003001oogoo03Qoo`<00000=goo00<007ooOol03Goo00<004IJAUX0
0dIJ00<004hhCSP00dhh00<0058gDSL01E8g00@005IGEUMFE`8000=N^003001Vf6KH00=Vf003001_
66lH00A_608000=c>003001oogoo03Eoo`@000=oo`00>7oo00<007ooOol037oo00<004IJAUX00dIJ
00<004hhCSP014hh00<0058gDSL0158g00@005IGEUL000IN^003001Vf6KH009Vf003001_66lH00=_
6006001cNG=i001c>000=7oo10001goo000iOol00`00Oomoo`0<Ool00`00AUY6FP02AUX00`00CSQ>
>004CSP00`00DSMB=`05DSL20007G[P01@00I]QVf6KH00001FlH00<007=iLgT00W=i00<007ooOol0
<7oo0`002goo000jOol2000<Ool00`00AUY6FP03AUX00`00CSQ>>003CSP00`00DSMB=`04DSL20009
G[P00`00I]P00005KaP00`00LgUcN@03LgT00`00Oomoo`0/Ool4000>Ool003aoo`03001oogoo00Uo
o`030016FTIJ00=6FP03001>>4hh00A>>004001B=e8gDSL200000eIi001N^008G[P20005KaP00`00
LgUcN@03LgT00`00Oomoo`0YOol4000BOol003eoo`03001oogoo00Uoo`<00096FP03001>>4hh00A>
>004001B=`000004EWT00`00G[QN^005G[P00`00IaX00005KaP00`00LgUcN@02LgT00`00Oomoo`0V
Ool4000FOol003ioo`03001oogoo00Qoo`03000^72hL00@000=>>0@000IFN@03001N^5jh00AN^005
001W6VLJIaX00003KaP00`00LgUcN@02LgT00`00Oomoo`0SOol4000JOol003moo`8000Uoo`03000^
72hL008^70@000=:FP03001FNEIi00EFN@03001N^5jh009N^003001W6VLJ00=W6P03001_600000Ac
N@03001oogoo021oo`@001ioo`00@Goo00<007ooOol01goo00D002hL;Q`^700000M:FP03001FNEIi
00EFN@04001N^5jh0007IaX30002LgT00`00Oomoo`0NOol3000ROol0049oo`03001oogoo00Moo`05
000^72hL;Q`00006BUX00`00EWUFN@05EWT00`00G[P00007IaX01000Km]_ffoK0P007Goo10009Goo
0013Ool00`00Oomoo`07Ool20002;Q`00`00BUY:FP04BUX00`00EWUFN@04EWT20007IaX01@00Km]_
ffoK00006goo1000:Goo0014Ool20009Ool30006BUX01000EWUFNEIi0`000UlL00<006LJIaX00fLJ
00<006oKKm/00P0067oo1000;Goo0016Ool00`00Oomoo`07Ool30005BUX01000EWT000001UlL00D0
06LJIaYW6P00009_f`03001oogoo01Aoo`@0035oo`00Agoo00<007ooOol01goo2@002ElL00<006LJ
00000VoK00<007ooOol04Goo1000=Goo0018Ool00`00Oomoo`08Ool20005?[h00`00GaaO7006Ga`5
000@Ool4000iOol004Uoo`8000Uoo`8000Dn_P03001O75lL009O70<00003Dmh0000000moo`<003eo
o`00Bgoo00<007ooOol01Woo00<007oo00001Cjn00@005lL0000009CgP8000eoo`@0041oo`00C7oo
00<007ooOol01Goo00<007ooOol02`002goo1000A7oo001=Ool00`00Oomoo`04Ool00`00Oomoo`0B
Ool40018Ool004ioo`03001oogoo00=oo`03001oogoo00ioo`@004aoo`00Cgoo0P000goo00<007oo
Ool02Woo1000D7oo001AOol01000Oomoo`002Goo0`00E7oo001BOol00`00Ool00005Ool4001GOol0
05=oo`800003Ool00000008005]oo`00E7oo0P00Ggoo002eOol00;Eoo`00]Goo002eOol00;Eoo`00
]Goo002eOol00001\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.376 0 0 r
.37588 .75735 .39996 .75138 .43802 .75186 .4143 .7635 Metetra
.4143 .7635 .43802 .75186 .47703 .74004 .45403 .75085 Metetra
.65 .961 .777 r
.45403 .75085 .47703 .74004 .51661 .71604 .49447 .71961 Metetra
.888 .971 .831 r
.49447 .71961 .51661 .71604 .55638 .68258 .535 .67411 Metetra
.896 .865 .801 r
.535 .67411 .55638 .68258 .59612 .64426 .57523 .62159 Metetra
.848 .77 .775 r
.57523 .62159 .59612 .64426 .63583 .60653 .61513 .57035 Metetra
.785 .693 .762 r
.61513 .57035 .63583 .60653 .67576 .57447 .65502 .52793 Metetra
.714 .629 .76 r
.65502 .52793 .67576 .57447 .71635 .55194 .6955 .49987 Metetra
.634 .572 .768 r
.6955 .49987 .71635 .55194 .75808 .54096 .73731 .48903 Metetra
.537 .52 .785 r
.73731 .48903 .75808 .54096 .80142 .54143 .78112 .49528 Metetra
.413 .473 .812 r
.78112 .49528 .80142 .54143 .84662 .55115 .82735 .51552 Metetra
.245 .43 .846 r
.82735 .51552 .84662 .55115 .89365 .56608 .87602 .54392 Metetra
.017 .393 .85 r
.87602 .54392 .89365 .56608 .94213 .58091 .9266 .57262 Metetra
0 .369 .728 r
.9266 .57262 .94213 .58091 .99133 .59004 .97801 .59301 Metetra
0 .531 .831 r
.35154 .74806 .37588 .75735 .4143 .7635 .39022 .7565 Metetra
.612 .888 .989 r
.39022 .7565 .4143 .7635 .45403 .75085 .43049 .74332 Metetra
.833 .895 .884 r
.43049 .74332 .45403 .75085 .49447 .71961 .47163 .70876 Metetra
.881 .847 .806 r
.47163 .70876 .49447 .71961 .535 .67411 .51291 .6579 Metetra
.882 .807 .77 r
.51291 .6579 .535 .67411 .57523 .62159 .5538 .59916 Metetra
.862 .774 .763 r
.5538 .59916 .57523 .62159 .61513 .57035 .59422 .54205 Metetra
.822 .742 .776 r
.59422 .54205 .61513 .57035 .65502 .52793 .63448 .49512 Metetra
.755 .704 .804 r
.63448 .49512 .65502 .52793 .6955 .49987 .67525 .46458 Metetra
.644 .649 .841 r
.67525 .46458 .6955 .49987 .73731 .48903 .71733 .45359 Metetra
.471 .564 .868 r
.71733 .45359 .73731 .48903 .78112 .49528 .76151 .462 Metetra
.248 .451 .862 r
.76151 .462 .78112 .49528 .82735 .51552 .80831 .4864 Metetra
.042 .35 .826 r
.80831 .4864 .82735 .51552 .87602 .54392 .85778 .52024 Metetra
0 .324 .806 r
.85778 .52024 .87602 .54392 .9266 .57262 .90939 .55462 Metetra
0 .422 .854 r
.90939 .55462 .9266 .57262 .97801 .59301 .96195 .57963 Metetra
.418 .608 .927 r
.32737 .72078 .35154 .74806 .39022 .7565 .36627 .72741 Metetra
.666 .697 .869 r
.36627 .72741 .39022 .7565 .43049 .74332 .40683 .71406 Metetra
.77 .725 .81 r
.40683 .71406 .43049 .74332 .47163 .70876 .44839 .68093 Metetra
.821 .741 .776 r
.44839 .68093 .47163 .70876 .51291 .6579 .49021 .63268 Metetra
.848 .759 .764 r
.49021 .63268 .51291 .6579 .5538 .59916 .53179 .57705 Metetra
.858 .78 .774 r
.53179 .57705 .5538 .59916 .59422 .54205 .57295 .52284 Metetra
.847 .807 .811 r
.57295 .52284 .59422 .54205 .63448 .49512 .61395 .47801 Metetra
.792 .832 .882 r
.61395 .47801 .63448 .49512 .67525 .46458 .65538 .44837 Metetra
.628 .81 .97 r
.65538 .44837 .67525 .46458 .71733 .45359 .69798 .4369 Metetra
.301 .653 .97 r
.69798 .4369 .71733 .45359 .76151 .462 .7425 .44347 Metetra
.005 .436 .863 r
.7425 .44347 .76151 .462 .80831 .4864 .78944 .46483 Metetra
0 .34 .81 r
.78944 .46483 .80831 .4864 .85778 .52024 .83891 .49488 Metetra
.025 .381 .844 r
.83891 .49488 .85778 .52024 .90939 .55462 .89043 .52531 Metetra
.248 .498 .897 r
.89043 .52531 .90939 .55462 .96195 .57963 .94297 .54699 Metetra
.549 .565 .823 r
.30366 .67668 .32737 .72078 .36627 .72741 .3428 .67773 Metetra
.644 .592 .781 r
.3428 .67773 .36627 .72741 .40683 .71406 .38341 .66453 Metetra
.706 .622 .762 r
.38341 .66453 .40683 .71406 .44839 .68093 .42501 .63721 Metetra
.754 .662 .76 r
.42501 .63721 .44839 .68093 .49021 .63268 .46706 .59897 Metetra
.795 .714 .775 r
.46706 .59897 .49021 .63268 .53179 .57705 .50914 .55519 Metetra
.831 .789 .812 r
.50914 .55519 .53179 .57705 .57295 .52284 .5511 .51217 Metetra
.838 .892 .878 r
.5511 .51217 .57295 .52284 .61395 .47801 .59307 .47575 Metetra
.718 .969 .934 r
.59307 .47575 .61395 .47801 .65538 .44837 .63548 .45027 Metetra
.317 .806 .816 r
.63548 .45027 .65538 .44837 .69798 .4369 .67885 .438 Metetra
.059 0 0 r
.67885 .438 .69798 .4369 .7425 .44347 .72375 .43885 Metetra
0 .424 .687 r
.72375 .43885 .7425 .44347 .78944 .46483 .77056 .45033 Metetra
0 .472 .859 r
.77056 .45033 .78944 .46483 .83891 .49488 .81939 .46786 Metetra
.259 .53 .916 r
.81939 .46786 .83891 .49488 .89043 .52531 .86991 .48535 Metetra
.451 .551 .867 r
.86991 .48535 .89043 .52531 .94297 .54699 .92143 .49632 Metetra
.605 .551 .768 r
.28041 .62051 .30366 .67668 .3428 .67773 .31989 .61347 Metetra
.629 .545 .741 r
.31989 .61347 .3428 .67773 .38341 .66453 .36036 .60064 Metetra
.652 .56 .738 r
.36036 .60064 .38341 .66453 .42501 .63721 .40165 .58205 Metetra
.68 .597 .756 r
.40165 .58205 .42501 .63721 .46706 .59897 .44353 .55892 Metetra
.716 .665 .799 r
.44353 .55892 .46706 .59897 .50914 .55519 .48582 .53333 Metetra
.753 .79 .882 r
.48582 .53333 .50914 .55519 .5511 .51217 .52845 .50779 Metetra
.703 .95 .955 r
.52845 .50779 .5511 .51217 .59307 .47575 .57147 .48473 Metetra
.288 .766 .66 r
.57147 .48473 .59307 .47575 .63548 .45027 .61504 .46604 Metetra
.064 0 0 r
.61504 .46604 .63548 .45027 .67885 .438 .6594 .45272 Metetra
.167 0 0 r
.6594 .45272 .67885 .438 .72375 .43885 .70478 .44473 Metetra
.067 0 0 r
.70478 .44473 .72375 .43885 .77056 .45033 .75138 .44094 Metetra
.307 .751 .963 r
.75138 .44094 .77056 .45033 .81939 .46786 .79924 .43939 Metetra
.53 .678 .932 r
.79924 .43939 .81939 .46786 .86991 .48535 .84825 .43752 Metetra
.584 .59 .824 r
.84825 .43752 .86991 .48535 .92143 .49632 .89816 .43276 Metetra
.644 .56 .746 r
.25733 .55936 .28041 .62051 .31989 .61347 .29727 .54345 Metetra
.621 .529 .729 r
.29727 .54345 .31989 .61347 .36036 .60064 .33753 .53104 Metetra
.605 .522 .734 r
.33753 .53104 .36036 .60064 .40165 .58205 .37826 .5221 Metetra
.594 .54 .763 r
.37826 .5221 .40165 .58205 .44353 .55892 .41963 .51588 Metetra
.586 .599 .833 r
.41963 .51588 .44353 .55892 .48582 .53333 .46179 .51107 Metetra
.55 .735 .961 r
.46179 .51107 .48582 .53333 .52845 .50779 .50485 .50597 Metetra
.28 .778 .877 r
.50485 .50597 .52845 .50779 .57147 .48473 .54881 .4989 Metetra
.075 0 0 r
.54881 .4989 .57147 .48473 .61504 .46604 .59359 .48848 Metetra
.149 0 0 r
.59359 .48848 .61504 .46604 .6594 .45272 .63906 .47396 Metetra
.06 0 0 r
.63906 .47396 .6594 .45272 .70478 .44473 .68508 .45536 Metetra
.68508 .45536 .70478 .44473 .75138 .44094 .73156 .4335 Metetra
.685 .944 .961 r
.73156 .4335 .75138 .44094 .79924 .43939 .77848 .40981 Metetra
.72 .758 .882 r
.77848 .40981 .79924 .43939 .84825 .43752 .82594 .38603 Metetra
.675 .626 .792 r
.82594 .38603 .84825 .43752 .89816 .43276 .87416 .36386 Metetra
.678 .586 .744 r
.23387 .5009 .25733 .55936 .29727 .54345 .27437 .47704 Metetra
.619 .536 .738 r
.27437 .47704 .29727 .54345 .33753 .53104 .31444 .46495 Metetra
.556 .502 .75 r
.31444 .46495 .33753 .53104 .37826 .5221 .35455 .46455 Metetra
.481 .485 .784 r
.35455 .46455 .37826 .5221 .41963 .51588 .39525 .47359 Metetra
.368 .493 .854 r
.39525 .47359 .41963 .51588 .46179 .51107 .43703 .48794 Metetra
.149 .523 .918 r
.43703 .48794 .46179 .51107 .50485 .50597 .48023 .50219 Metetra
.152 0 0 r
.48023 .50219 .50485 .50597 .54881 .4989 .52489 .51058 Metetra
.202 0 0 r
.52489 .51058 .54881 .4989 .59359 .48848 .57076 .50827 Metetra
.064 0 0 r
.57076 .50827 .59359 .48848 .63906 .47396 .61737 .49252 Metetra
.61737 .49252 .63906 .47396 .68508 .45536 .66417 .46348 Metetra
.674 .965 .757 r
.66417 .46348 .68508 .45536 .73156 .4335 .71078 .42413 Metetra
.863 .947 .869 r
.71078 .42413 .73156 .4335 .77848 .40981 .75711 .37953 Metetra
.819 .786 .822 r
.75711 .37953 .77848 .40981 .82594 .38603 .80343 .33557 Metetra
.743 .665 .773 r
.80343 .33557 .82594 .38603 .87416 .36386 .85034 .29771 Metetra
.715 .63 .762 r
.20931 .45178 .23387 .5009 .27437 .47704 .25041 .42225 Metetra
.625 .571 .773 r
.25041 .42225 .27437 .47704 .31444 .46495 .29045 .41027 Metetra
.5 .505 .794 r
.29045 .41027 .31444 .46495 .35455 .46455 .33009 .41572 Metetra
.315 .427 .817 r
.33009 .41572 .35455 .46455 .39525 .47359 .37018 .43544 Metetra
.054 .334 .814 r
.37018 .43544 .39525 .47359 .43703 .48794 .41151 .46351 Metetra
0 .256 .735 r
.41151 .46351 .43703 .48794 .48023 .50219 .45465 .4919 Metetra
.307 0 0 r
.45465 .4919 .48023 .50219 .52489 .51058 .49974 .51179 Metetra
.133 0 0 r
.49974 .51179 .52489 .51058 .57076 .50827 .54646 .51554 Metetra
.27 .765 .707 r
.54646 .51554 .57076 .50827 .61737 .49252 .59408 .49874 Metetra
.69 .979 .812 r
.59408 .49874 .61737 .49252 .66417 .46348 .64175 .46163 Metetra
.885 .98 .825 r
.64175 .46163 .66417 .46348 .71078 .42413 .68883 .40903 Metetra
.909 .892 .801 r
.68883 .40903 .71078 .42413 .75711 .37953 .73513 .34893 Metetra
.865 .796 .781 r
.73513 .34893 .75711 .37953 .80343 .33557 .78102 .29032 Metetra
.796 .71 .77 r
.78102 .29032 .80343 .33557 .85034 .29771 .82735 .24125 Metetra
.754 .701 .801 r
.18288 .41663 .20931 .45178 .25041 .42225 .22456 .38453 Metetra
.638 .653 .848 r
.22456 .38453 .25041 .42225 .29045 .41027 .26479 .37236 Metetra
.414 .55 .883 r
.26479 .37236 .29045 .41027 .33009 .41572 .30432 .37996 Metetra
.068 .365 .834 r
.30432 .37996 .33009 .41572 .37018 .43544 .34415 .40387 Metetra
0 .188 .706 r
.34415 .40387 .37018 .43544 .41151 .46351 .38523 .43744 Metetra
0 .151 .648 r
.38523 .43744 .41151 .46351 .45465 .4919 .42831 .47159 Metetra
0 .305 .743 r
.42831 .47159 .45465 .4919 .49974 .51179 .47363 .49617 Metetra
.191 .639 .944 r
.47363 .49617 .49974 .51179 .54646 .51554 .52088 .50227 Metetra
.634 .881 .987 r
.52088 .50227 .54646 .51554 .59408 .49874 .56927 .48473 Metetra
.834 .908 .888 r
.56927 .48473 .59408 .49874 .64175 .46163 .61778 .44383 Metetra
.894 .873 .809 r
.61778 .44383 .64175 .46163 .68883 .40903 .66563 .38524 Metetra
.901 .836 .772 r
.66563 .38524 .68883 .40903 .73513 .34893 .71252 .31826 Metetra
.882 .802 .766 r
.71252 .31826 .73513 .34893 .78102 .29032 .75882 .25323 Metetra
.837 .765 .783 r
.75882 .25323 .78102 .29032 .82735 .24125 .80544 .19923 Metetra
.788 .809 .869 r
.15393 .39742 .18288 .41663 .22456 .38453 .19609 .3662 Metetra
.635 .805 .965 r
.19609 .3662 .22456 .38453 .26479 .37236 .23678 .3535 Metetra
.242 .634 .96 r
.23678 .3535 .26479 .37236 .30432 .37996 .27672 .35916 Metetra
0 .326 .761 r
.27672 .35916 .30432 .37996 .34415 .40387 .31686 .37992 Metetra
0 .181 .656 r
.31686 .37992 .34415 .40387 .38523 .43744 .35819 .40955 Metetra
0 .237 .728 r
.35819 .40955 .38523 .43744 .42831 .47159 .40144 .4396 Metetra
.092 .431 .873 r
.40144 .4396 .42831 .47159 .47363 .49617 .44694 .46068 Metetra
.442 .612 .921 r
.44694 .46068 .47363 .49617 .52088 .50227 .49447 .46464 Metetra
.662 .702 .876 r
.49447 .46464 .52088 .50227 .56927 .48473 .54329 .44675 Metetra
.776 .741 .821 r
.54329 .44675 .56927 .48473 .61778 .44383 .59248 .40727 Metetra
.835 .764 .784 r
.59248 .40727 .61778 .44383 .66563 .38524 .64124 .35138 Metetra
.866 .785 .769 r
.64124 .35138 .66563 .38524 .71252 .31826 .68924 .28763 Metetra
.876 .808 .779 r
.68924 .28763 .71252 .31826 .75882 .25323 .73672 .22555 Metetra
.859 .831 .818 r
.73672 .22555 .75882 .25323 .80544 .19923 .78449 .17363 Metetra
.762 .939 .941 r
.12203 .3933 .15393 .39742 .19609 .3662 .16448 .36626 Metetra
.465 .883 .924 r
.16448 .36626 .19609 .3662 .23678 .3535 .20591 .3527 Metetra
.026 .598 .736 r
.20591 .3527 .23678 .3535 .27672 .35916 .24687 .35248 Metetra
.197 0 0 r
.24687 .35248 .27672 .35916 .31686 .37992 .28809 .3631 Metetra
0 .389 .773 r
.28809 .3631 .31686 .37992 .35819 .40955 .33035 .37987 Metetra
.122 .475 .896 r
.33035 .37987 .35819 .40955 .40144 .4396 .37426 .39661 Metetra
.383 .531 .881 r
.37426 .39661 .40144 .4396 .44694 .46068 .42008 .40665 Metetra
.54 .563 .827 r
.42008 .40665 .44694 .46068 .49447 .46464 .46771 .40433 Metetra
.637 .593 .788 r
.46771 .40433 .49447 .46464 .54329 .44675 .51665 .38644 Metetra
.705 .629 .769 r
.51665 .38644 .54329 .44675 .59248 .40727 .56623 .35316 Metetra
.76 .674 .767 r
.56623 .35316 .59248 .40727 .64124 .35138 .61585 .30801 Metetra
.807 .734 .783 r
.61585 .30801 .64124 .35138 .68924 .28763 .66522 .25691 Metetra
.841 .812 .821 r
.66522 .25691 .68924 .28763 .73672 .22555 .71444 .20667 Metetra
.84 .904 .882 r
.71444 .20667 .73672 .22555 .78449 .17363 .76406 .16349 Metetra
.514 .915 .861 r
.08714 .40057 .12203 .3933 .16448 .36626 .12962 .38041 Metetra
.12962 .38041 .16448 .36626 .20591 .3527 .17203 .36571 Metetra
.17203 .36571 .20591 .3527 .24687 .35248 .21462 .35641 Metetra
.21462 .35641 .24687 .35248 .28809 .3631 .25773 .35139 Metetra
.329 .759 .972 r
.25773 .35139 .28809 .3631 .33035 .37987 .30171 .3486 Metetra
.507 .68 .944 r
.30171 .3486 .33035 .37987 .37426 .39661 .34684 .34547 Metetra
.565 .589 .836 r
.34684 .34547 .37426 .39661 .42008 .40665 .39328 .33935 Metetra
.593 .548 .773 r
.39328 .33935 .42008 .40665 .46771 .40433 .44099 .32806 Metetra
.618 .541 .745 r
.44099 .32806 .46771 .40433 .51665 .38644 .48979 .31041 Metetra
.645 .558 .742 r
.48979 .31041 .51665 .38644 .56623 .35316 .53944 .28645 Metetra
.676 .599 .762 r
.53944 .28645 .56623 .35316 .61585 .30801 .58968 .2575 Metetra
.716 .675 .809 r
.58968 .2575 .61585 .30801 .66522 .25691 .64038 .22581 Metetra
.754 .799 .889 r
.64038 .22581 .66522 .25691 .71444 .20667 .69158 .1941 Metetra
.719 .941 .957 r
.69158 .1941 .71444 .20667 .76406 .16349 .74349 .16496 Metetra
.04967 .41308 .08714 .40057 .12962 .38041 .09191 .40131 Metetra
.09191 .40131 .12962 .38041 .17203 .36571 .13545 .38531 Metetra
.13545 .38531 .17203 .36571 .21462 .35641 .18016 .36511 Metetra
.514 .914 .831 r
.18016 .36511 .21462 .35641 .25773 .35139 .22582 .34155 Metetra
.724 .92 .959 r
.22582 .34155 .25773 .35139 .30171 .3486 .27219 .31609 Metetra
.72 .753 .878 r
.27219 .31609 .30171 .3486 .34684 .34547 .3191 .29052 Metetra
.673 .628 .796 r
.3191 .29052 .34684 .34547 .39328 .33935 .36647 .26655 Metetra
.635 .557 .75 r
.36647 .26655 .39328 .33935 .44099 .32806 .41436 .24552 Metetra
.608 .522 .731 r
.41436 .24552 .44099 .32806 .48979 .31041 .46292 .22816 Metetra
.591 .514 .736 r
.46292 .22816 .48979 .31041 .53944 .28645 .51236 .2144 Metetra
.582 .535 .768 r
.51236 .2144 .53944 .28645 .58968 .2575 .5629 .20345 Metetra
.579 .599 .837 r
.5629 .20345 .58968 .2575 .64038 .22581 .61467 .19387 Metetra
.566 .73 .952 r
.61467 .19387 .64038 .22581 .69158 .1941 .66774 .18387 Metetra
.439 .847 .97 r
.66774 .18387 .69158 .1941 .74349 .16496 .72201 .17161 Metetra
.06 0 0 r
.01057 .42296 .04967 .41308 .09191 .40131 .05237 .41943 Metetra
.05237 .41943 .09191 .40131 .13545 .38531 .09706 .40211 Metetra
.553 .922 .763 r
.09706 .40211 .13545 .38531 .18016 .36511 .1441 .37115 Metetra
.808 .992 .874 r
.1441 .37115 .18016 .36511 .22582 .34155 .19262 .32963 Metetra
.857 .895 .862 r
.19262 .32963 .22582 .34155 .27219 .31609 .24173 .28275 Metetra
.811 .768 .813 r
.24173 .28275 .27219 .31609 .3191 .29052 .29073 .23652 Metetra
.743 .665 .773 r
.29073 .23652 .3191 .29052 .36647 .26655 .33931 .1965 Metetra
.674 .587 .749 r
.33931 .1965 .36647 .26655 .41436 .24552 .38755 .16683 Metetra
.607 .53 .742 r
.38755 .16683 .41436 .24552 .46292 .22816 .43591 .14964 Metetra
.541 .493 .752 r
.43591 .14964 .46292 .22816 .51236 .2144 .48503 .14478 Metetra
.471 .48 .786 r
.48503 .14478 .51236 .2144 .5629 .20345 .53556 .14989 Metetra
.389 .501 .852 r
.53556 .14989 .5629 .20345 .61467 .19387 .58801 .16058 Metetra
.278 .571 .937 r
.58801 .16058 .61467 .19387 .66774 .18387 .64255 .17107 Metetra
.138 .643 .904 r
.64255 .17107 .66774 .18387 .72201 .17161 .6989 .17516 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<002eoo`03001oogoo
00Moo`03001oogoo02Aoo`00DWoo0`00;goo00<007ooOol027oo00<007ooOol08goo001?Ool3000b
Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<003Eoo`03001oogoo00Uoo`03001oogoo029o
o`00BWoo0`00=goo00<007ooOol02Woo00<007ooOol08Goo0017Ool3000DOol9000MOol00`00Oomo
o`0;Ool00`00Oomoo`0POol004Eoo`80019oo`D000A5n003000mnCgi008mn@H001Moo`03001oogoo
00]oo`03001oogoo021oo`00@Woo0`004Goo0`0000=>5`00AOP01dGh00<003gi?OT013gi00<0038K
<Q/010004goo00<007ooOol037oo00<007ooOol07goo000UOol4000GOol2000BOol20005CQL00`00
AOQ5n005AOP00`00?OTmn@05?OT00`00<Q/b6`03<Q/7000<Ool00`00Oomoo`03Ool60004Ool00`00
Oomoo`0NOol002Ioo`03001oogoo01Aoo`<0019oo`8000M>5`030015n4Gh00E5n003000mnCgi00Dm
n@03000b6c8K00Db6`03000RGB9M008RG@T000=oo`03001oo`0000<000=OgP03001[W6^L00800003
Ool00000021oo`007goo1@000goo00<007ooOol04Goo0P004goo0P0000=FE`00CQL01ThG00<004Gh
AOP01TGh00<003gi?OT01Cgi00<0038K<Q/01C8K00<0029M8Ud01R9M0P000Q:L1@0000<kO`000000
0eoN00<006^LJi`00f^L00@007ooOol001moo`00:7oo00<007ooOol03Goo0`004goo0P0015IG00<0
04hGCQL01DhG00<004GhAOP01DGh00<003gi?OT01Cgi00<0038K<Q/01S8K00<0029M8Ud01B9M1000
0c]o0`0015oN00<006^LJi`00f^L0P000Woo00<007ooOol07Goo000UOol01000Oomoo`003Goo0P00
57oo0P001UIG00<004hGCQL01DhG00<004GhAOP01DGh00<003gi?OT01Cgi00<0038K<Q/01c8K00<0
029M8Ud0100014[n0P0000<kO`00000015oN00<006^LJi`00f^L00@006mJKeX0009oo`03001oogoo
01aoo`009Woo0P002goo0`0057oo0P0025IG00<004hGCQL01ThG00<004GhAOP01DGh00<003gi?OT0
1Cgi00<0038K<Q/01C8K10002D[n10000eoN00<006^LJi`00f^L00<006mJKeX00VmJ00@007ooOol0
01eoo`00<Goo0P0000=oo`00Ool04goo00<005jh00002EIG00<004hGCQL01DhG00<004GhAOP01DGh
00<003gi?OT01Cgi00<0038K<Q/01@000TYj00<004[nB_h01d[n00@006<lHcaS?0800003Gmh006^L
00A[W003001_FVmJ00A_FP03001oo`0001eoo`00;Woo0`005goo00D005jhG[QN^00000QFE`03001>
5dhG00I>5`030015n4Gh00A5n003000mnCgi008mn@H000Q:NP03001:oT[n00A:oP8000AS?003001S
?00000E[W003001_FVmJ00E_FP04001oo`00000LOol002]oo`<001Uoo`03001N^5jh009N^003001F
EeIG00IFE`03001>5dhG00I>5`030015n4Gh00=5n0H000A:>003001:NTYj00Q:NP04001:oT[nB_h2
0006Hc`200000f/j001[W002Ji`00`00KeY_FP05KeX00`00JaT00002Ool00`00Oomoo`03Ool2000D
Ool002Uoo`80009oo`03001oogoo01Ioo`03001N^5jh00=N^003001FEeIG00MFE`03001>5dhG00E>
5`030015n4Gh00<0009:5`03001:>4Xh00I:>003001:NTYj00Q:NP03001:oP0000QS?007001[>V/j
JcX006^L00001fmJ00@006/IJaT0009oo`03001oogoo009oo`04001oogoo000COol002Ioo`<001ao
o`03001N^5jh00EN^003001FEeIG00IFE`03001>5dhG00E>5`<000I:5`03001:>4Xh00I:>003001:
NTYj00M:NP8000QS?08000A[>P03001_FVmJ00E_FP03001[6F/I009[6@04001oogoo0003Ool01000
Oomoo`004goo000TOol2000NOol00`00G[QN^006G[P00`00EUMFE`06EUL00`00CQM>5`02CQL30000
0dhG001:5`06BQL00`00BSQ:>006BSP00`00BWY:NP06BWX00`00F[T00007Hc`00`00JcX00004JcX2
0005KeX00`00JaU[6@04JaT01000Oomoo`000Woo00@007ooOol001=oo`008Goo0`007Woo0P002Ejh
00<005IGEUL01eIG100014hG00<004XGBQL01DXG00<004XhBSP01dXh00<004YjBWX00dYj0P000eZi
00<006<lHc`00f<l00@006/jJcX000=[>P04001c>7<h0003KeX00`00JaU[6@05JaT01000Ool00000
0Woo00@007ooOol001=oo`007goo0P0087oo00<006LJ00002Ujh00<005IGEUL01EIG0P0024hG00<0
04XGBQL01DXG00<004XhBSP01TXh00@004YjBWY:NP8000IJ^@05001S?6<lHc`00002JcX20002JcX0
0`00LcQc>002LcP00`00KeX00008JaT20002Ool00`00Oomoo`02000DOol001aoo`<0025oo`05001W
6VLJIaX00009G[P00`00EUMFE`03EUL200000e8g001>5`07CQL00`00BQM:5`05BQL00`00BSQ:>007
BSP00`00BWX00009F[T00`00Hc`0000300001F/j001[>V/j00001G<h00<006/IJaT01V/I00<006KH
00000goo00<007ooOol05Goo000KOol2000ROol00`00IaYW6P03IaX00`00G[QN^006G[P01000EUMF
EeIG0P000e8g00<004hGCQL01ThG00<004XGBQL01TXG00<004XhBSP01TXh0P002UZi0`001F/j0P00
1W<h00<006/IJaT01F/I00D006KHI]QVf000009oo`03001oogoo01Eoo`0057oo0`0017oo00<007oo
000087oo00<006LJIaX016LJ00<005jhG[P01ejh0`001U8g00<004hGCQL01ThG00<004XGBQL01DXG
00<004XhBSP01DXh00<005Ih00001eZi1@001V/j0P001G<h00<007<h00001F/I00<006KHI]P00VKH
00D007ooOomoo`0001Ioo`005Goo00<007ooOol00goo00@007ooOol001ioo`03001W6VLJ00IW6P03
001N^5jh00EN^08000QB=`03001>5dhG00I>5`03001:5dXG00I:5`03001:>4Xh009:>08000=FN004
001J^EZiF[T30002F[T01000I_UVn@0016/j00@007<hLcP000=c>004001c>7<h0004JaT00`00I]QV
f003I]P01000Oomoogoo0P005Goo000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo000077oo
00<006LJIaX01fLJ00<005jhG[P00Ujh0`0000=FV@00DSL01e8g00<004hGCQL01ThG00<004XGBQL0
1TXG00<004XhBSP00P001EIh100015Zi00<006KiI_T00VKi00D006/jJcY[>P00009c>004001c>7<h
0004LcP01000JaU[6@001fKH00D007ooOomoo`0001Eoo`005Goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol067oo00<006LJIaX02FLJ00@005jh000000EFV@03001B=e8g00IB=`03001>5dhG
00E>5`03001:5dXG00I:5`03001:>00000EFN0@000EJ^@03001VnFKi00AVn@03001[>P0000Ac>003
001c>00000Ac>003001[6@0000QVf003001oogoo009oo`03001oogoo019oo`005Goo00<007ooOol0
0Woo0P001Goo00<007ooOol05goo0P002VLJ0P001eJI00<0058gDSL01U8g00<004hGCQL01ThG00<0
04XGBQL01DXG0P000eIh0`000eIh00<005ZiF[T00UZi00<006KiI_T01VKi00<007<hLcP017<h00<0
07<hLcP017<h00<006KHI]P01fKH00<007ooOol00Woo00<007ooOol04Woo000DOol20004Ool00`00
Oomoo`05Ool00`00Oomoo`0EOol01000Ki]_V`001fLJ0P0000=O7000EYT01eJI00<0058gDSL01E8g
00<004hGCQL01ThG00<004XGBQL00dXG0P0000=B=`00EWP00P001eIh00@005ZiF[T000UVn@03001c
>7<h00=c>08000Ic>003001Vf6KH00QVf003001oogoo009oo`03001oogoo015oo`006Woo00<007oo
Ool01Woo00<007ooOol04goo00<006nKKi/00VnK00<006LJIaX00VLJ0P0015lL00<005JIEYT01EJI
00<0058gDSL01U8g00<004hGCQL01DhG00@004XGBQM:5`80009B=`<000UFN003001J^@0000UVn@03
001_>00000Ac>003001c>00000Ac>003001_>00000QVf08000Eoo`03001oogoo011oo`006Woo00<0
07ooOol01goo00<007ooOol04Goo00<006nKKi/016nK00@006LJ000000IO7003001FVEJI00IFV@03
001B=e8g00EB=`03001>5dhG00I>5`<00004DSL000000002DSL00`00EWQFN007EWP20009I_T01@00
KcQ_>6lh00000W<h00D007<hLcQc>000009c>004001_>6lh0007I]P00`00G[P00004Ool2000BOol0
01Uoo`8000Yoo`03001oogoo00moo`03001_VfnK00E_V`8000UO7003001FVEJI00EFV@03001B=e8g
00EB=`03001>5dhG00A>5`H000EB=`03001FN5Ih00IFN003001R^00000QVn@03001_>6lh009_>003
001c>00000Ec>003001c>00000=_>003001Vf6KH00=Vf005001N^5jhG[P00005Ool00`00Oomoo`0?
Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000]oo`03001oogoo00eoo`03001_VfnK00A_V`800003
Gkh005lL00QO7003001FVEJI00EFV@03001B=e8g00IB=`03001>5`0000H000UB=`03001FN5Ih00AF
N005001R^6:hH[P00006I_T00`00KcQ_>004KcP00`00LcQc>004LcP20004KcP00`00I]QVf002I]P0
0`00G[QN^002G[P00`00Oomoo`04Ool00`00Oomoo`0>Ool000Moo`04001oogoo0003Ool20006Ool0
1000Oomoo`003Goo00<007ooOol02goo00<006nKKi/016nK00<005nnGkh00Unn00<005lLGa`01UlL
00<005JIEYT01UJI00<0058gDSL00e8g100000A>=`00000000=>=`03001B=e8g00IB=`03001FN5Ih
00=FN003001R^6:h009R^003001VnFKi00AVn@03001_>6lh00A_>003001c>7<h00Ac>08000E_>004
001Vf6KH0005G[P00`00Oomoo`04Ool00`00Oomoo`0>Ool00003Ool0000000<00005Ool007ooOol0
000;Ool01000Oomoo`003Woo00<007ooOol027oo1@000VnK0P001Unn00<005lLGa`01UlL00<005JI
EYT01EJI00<0058gDSL00`000TXh0`001Thg00<0058gDSL01e8g00D005IhEWQFN00000IR^003001V
nFKi009Vn@03001_>6lh00A_>003001cF00000Ac>004001_>6lh0004KcP00`00I]P00007G[P00`00
Oomoo`04Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`0=Ool20007
Ool00`00Io]Wn`03Io/30008Gkh00`00GaaO7005Ga`00`00EYUFV@02EYT60002BSP300000dXh001>
=`06CSL00`00DSMB=`07DSL01000EWQFN00026:h00@006KiI_T000I_>005001cF7=HLeP00003LcP0
1000KcQ_>00016lh0P0025jh00<007ooOol01Goo00<007ooOol00Woo10001Woo0007Ool01000Oomo
o`0027oo00<007ooOol00goo00<007ooOol03goo00<007ooOol00goo00<006OkIo/00VOk0P000d>j
0`001Unn00<005lLGa`00UlL1`0014YJ00@004Xh000000A:>003001>=dhg00I>=`03001B=e8g00IB
=`03001FN00000UR^003001Vn@0000M_>003001cF7=H009cF003001c>00000A_>006001_>6lh001[
>P002Ejh00<007ooOol017oo00<007ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool0
0`00Oomoo`0@Ool00`00Oomoo`020004Io/00`00@kY3^P03@kX>0003@[h00`00BUY:FP04BUX30006
BSP00`00CSM>=`06CSL00`00DSMB=`06DSL2000;H[P00`00KcQ_>005KcP00`00LeQcF004LeP00`00
KcQ_>004KcP20003JcX00`00G[QN^006G[P00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`05
Ool001Qoo`8001=oo`8000AWn`P0009C?P03000NWP0000<000<[7`030012_T:n00A2_P03001:FTYJ
00<0009:FP03001:>4Xh00E:>003001>=dhg00E>=`03001B=e8g00EB=`03001JV00000]R^003001_
>6lh00A_>003001cF7=H00EcF003001_>6lh00A_>08000=[>P03001N^5jh00IN^003001oogoo00Io
o`03001oogoo00=oo`03001oogoo00Aoo`0067oo0P004goo00<006Ok00001P0025<n00@001jN7YhN
WP<0008[7`030012_T:n00=2_P<000E:FP03001:>4Xh00E:>003001>=dhg00I>=`03001B=e8g00=B
=`05001JV5ZHFYP00009H[P00`00JaT00004KcP00`00LeQcF005LeP00`00MiT00004KcP00`00JcX0
0003JcX00`00G[QN^006G[P20006Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomo
o`0@Ool600000f?N001W>`02Ic/20006Dch400000ajN0000000400001T:n00000000@[h000M:FP03
001:>4Xh00A:>003001>=dhg00I>=`04001B=e8gDSL20004FYP00`00H[QR^006H[P01@00JaU[6F/I
00000flh00<007=HLeP017=H00D007NIMiUgV@00009_>005001[>V/jJcX00003JcX00`00G[QN^003
G[P200000eJI001oo`07Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`0?
Ool01000AkP000000V?N0P001VLk00<005<nDch00`0000ABV`00=S`f?0<000030EP00000008000A2
_P03001:FTYJ00E:FP03001:>4Xh00E:>003001>=dhg00E>=`04001B=e8g0007FYP00`00H[QR^004
H[P00`00JaU[6@03JaT00`00KcP00006LeP00`00MiUgV@02MiT00`00KcP00005JcX01000JcY[>P00
0ejh0P0015JI00<007ooOol01Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol0
3Goo10000f?N00<006LkIc/01VLk0`001E:K00@003Hl=S`f?0D000040IP000000002@[h00`00BUY:
FP04BUX00`00BSQ:>005BSP00`00CSM>=`06CSL20008FYP00`00H[QR^003H[P00`00JaU[6@05JaT0
0`00LeQcF004LeP00`00MiUgV@04MiT00`00JcY[>P05JcX01000JcX005jh0P001UJI00<007ooOol0
1Woo00<007ooOol027oo000AOol01000Oomoo`000Woo0P003Goo100016?N00<006LkIc/01VLk00<0
06;I00001e:K0`0000<f?0002GX01@0000@=o000000000M:FP03001:>4Xh00A:>003001>=dhg00A>
=`<000UJV003001R^6:h009R^003001[6F/I00I[6@8000EcF003001gVGNI00=gV@03001_f`0000I[
>P04001[>P000008EYT00`00Oomoo`07Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00
Oomoo`09Ool;0006Ic/20003H]T00`00DY]BV`02DY/30004=S`010002GX9NPUj0`000`gl0`0014YJ
00<004XhBSP01DXh00<004hg00000`000U9I00<005ZHFYP025ZH00@006:hH[P000Q[6@04001cNG=i
0003LeP00`00MiUgV@03MiT01@00Km]_ffoK000016/j0`0000=O7000EYT025JI00<007ooOol01Woo
00<007ooOol01goo000BOol20003Ool20009Ool40009Ool20003Ic/00`00H]URf@04H]T40003DY/0
0`00=S`f?002=S`00`002GX9NP022GX200023O`00`00<Q`b70050004BSP50007DUT00`00FYQJV007
FYP00`00H[P00008JaT00`00LgUcN@02LgT00`00LeP00005MiT00`00Km]_f`03Km/01000JcX00000
0elL00<005lL000025JI00<007ooOol01goo00<007ooOol01Woo000GOol00`00Oomoo`05Ool4000>
Ool00`00Ic/00005H]T30006DY/500000cHl0009NP022GX400000`gl000b7004<Q`50003AUX00`00
DUUBF@06DUT00`00FYQJV007FYP20008JaT00`00LgUcN@04LgT00`00MiUgV@03MiT00`00Km]_f`04
Km/20005Ga`00`00Ga`00008EYT00`00Oomoo`08Ool00`00Oomoo`05Ool001Moo`03001oogoo00H0
019oo`04001RfF;IH]T30003H]T00`00DY]BV`0300000diH000n6@02?QT400022GX20000103g000=
o0001C8L00<004IJAUX01DIJ00<0059IDUT01U9I00<005ZHFYP01EZH00<006;j00001f/I00<007=i
LgT01G=i0P000gNI00<006oKKm/00foK0P0000=GgP00Ga`01ElL00<005lL000025JI00<007ooOol0
1goo00<007ooOol01Goo000GOol7000DOol40006H]T30005CUP00`00?QTn6@02?QT00`00:KX00004
0000103g000000001C8L00<004IJAUX014IJ00<0059IDUT01U9I00<005ZHFYP015ZH00D006;jH_YR
nP0000I[6@03001cNG=i00AcN@06001cnW?j001gV@0016oK0P0015ON00<005lLGa`015lL0P0025JI
00<007ooOol027oo00<007ooOol017oo0005Ool20004Ool20003Ool30004Ool2000GOol40007H]T0
0`00FYP00006CUP00`00?QTn6@03?QT01000:KXY^RVj0`000P3g00<0038L<Q`00c8L00<004IJAUX0
1DIJ00<0059IDUT01E9I00<005ZHFYP00UZH0P001F;j00<006/IJaT00V/I00<007=iLgT017=i00@0
07?jLoYcnP@00003Km/0000000MGgP03001O75lL00AO708000MFV@03001oogoo00Moo`<000Eoo`00
17oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000EOol30004Ool00`00H]URf@02H]T20003
FYP00`00CUQ>F004CUP00`00?QTn6@02?QT00`00:KXY^P03:KX200020?L00`00<Q`b7003<Q`00`00
AUY6FP04AUX00`00DUUBF@06DUT01000FYQJV0001f;j00D006/IJaU[6@0000IcN@05001cnW?jLoX0
0003EmP40007Emh00`00GaaO7004Ga`20004EYT50005Ool30002Ool00`00Oomoo`03Ool000Aoo`04
001oogoo000;Ool00`00Oomoo`02000COol20007Ool01@00H]URfF;I00001EZH00<004iHCUP014iH
00<003hI?QT00chI00<002Vj:KX00RVj00D000EJ0000003g008000<b70030016FTIJ00E6FP03001B
FE9I00EBF@03001JV00000URnP03001[6@0000IcN@05001cnW?jLoX00002EmPJ0004BUX00`00Oomo
o`030005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool2000@Ool3000:Ool00`00
H]T00007FYP00`00CUQ>F003CUP00`00?QTn6@03?QT00`00:KXY^P03:KX00`001EX0000200001PVk
00000000<Q`000I6FP03001BFE9I00EBF@8000YRnP<000EcN@05001cnW?jLoX00002EmP;00049ah0
0`00@[e2_@05@[d00`00BUY:FP06BUX20008Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool0
0`00Oomoo`03Ool00`00Oomoo`0<Ool3000>Ool00`00FYQJV006FYP00`00CUQ>F004CUP00`00?QTn
6@03?QT00`00:KXY^P03:KX00`001EX0000200032K/20007AUX01000DUUBFE9I0`0000=Fg000H_X0
26;j00D006^lJka[_00000=cN@06001cnW?jLoX005OH1`000RJO1P000bLN00<004:m@[d01D:m00<0
04YJBUX01DYJ00<007ooOol01goo00<007ooOol00goo0005Ool20009Ool40002Ool00`00Oomoo`09
Ool3000AOol00`00FYQJV006FYP00`00CUQ>F004CUP00`00?QTn6@03?QT00`00:KXY^P03:KX01P00
1EX5FP0009@000@9^`D00096FP04001BF@000005E]`00`00H_YRnP05H_X00`00Jka[_003Jk`00`00
LgT00003LoX50002DYX00`00?U/nF`0400020K/500000bLN0012_@05@[d00`00BUY:FP05BUX00`00
Oomoo`08Ool00`00Oomoo`02Ool001Ioo`03001oogoo00Ioo`<001Eoo`03001JV5ZH00IJV003001>
F4iH00=>F0`00004:KX000EJ1EX300032K/00`00>WdjO@02>Wd40008E]`00`00H_YRnP03H_X00`00
Jka[_004Jk`80006DYX00`00?U/nF`02?U/300030K/40007@[d00`00BUY:FP05BUX00`00Oomoo`07
Ool00`00Oomoo`02Ool001Eoo`8000Eoo`<001Qoo`03001JV5ZH00IJV003001>F4iH00<00003CSL0
04HG00A65`03000mnCgi008mn@@000031EX0000000800089^`03000jOCYm00@jO@03001Fg5KL00MF
g005001RnV;jH_X00006Jk`01000FoUKnE_i0P0000=Rf@00DYX01U:J00<003iK?U/00ciK00T001gK
0000006k0001F@000O/00`0014:m0P001TYJ00<007ooOol01goo00<007ooOol00Woo000EOol01000
Oomoogoo0`0077oo00<005ZHFYP01EZH0`001Dhg00<004HGAQL00dHG00<003gi?OT00cgi00<002gk
00000P0000H15`00000000Vk0006>Wd00`00E]aFg007E]`00`00H_X00005Jk`20002FoT30003H]T0
0`00DYYBVP05DYX00`00?U/nF`03?U/00`007M/Mf`0300000`5I000000028Qd60007BUX00`00Oomo
o`06Ool00`00Oomoo`02Ool001Eoo`@001moo`03001JV5ZH00=JV0800003EUL004hg00E>=`030016
5dHG00=65`03000mnCgi00@mn@03000]nbgk008000030AL00000008000LjO@03001Fg5KL00IFg0<0
00=[_080009Kn@800003Ie`006;I00ARf@03001BVU:J00ABVP03000nFciK00@nF`04000MfagK7M/2
00020ET00`008QdR7@038Qd40005BUX00`00Oomoo`06Ool00`00Oomoo`02Ool000ioo`<000Aoo`80
029oo`05001JV5ZHFYP00003EUL00`00CSM>=`04CSL00`00AQM65`04AQL00`00?OTmn@03?OT00`00
;O/]n`02;O/60006>Wd00`00E]aFg003E]`30003Dil00`00Jk`00003FoT200000fML001Rf@05H]T0
0`00DYYBVP04DYX600000ciK000Mf`047M/200000`5I000R7@048Qd00`00>S/000060008Ool00`00
Oomoo`02Ool000moo`03001oogoo00Eoo`03001oogoo01moo`04001JV0000005EUL00`00CSM>=`04
CSL00`00AQM65`03AQL00`00?OTmn@04?OT00`00;O/]n`03;O/90002>Wd00`00E]`000020006Dil6
0002KcT00`00H]URf@06H]T00`00DYYBVP0300000e9H00166@03AQT400037M/010001GX0005I0P00
128M00<003Xk000013Xk0P001goo00<007ooOol00Woo000?Ool00`00Oomoo`06Ool00`00Oomoo`0O
Ool00`00EUMFE`05EUL00`00CSM>=`04CSL00`00AQM65`03AQL00`00?OTmn@05?OT00`00;O/]n`03
;O/00`004QdB7@0300026Yh50005Dil60004KcT20008H]T30005DUP00`00AQU66@03AQT00`00=OT0
000200000agK0005NP0200000`6K000000028Qd00`00>S/00005>S/00`00Oomoo`06Ool00`00Oomo
o`01Ool000moo`03001oogoo00Moo`03001oogoo01ioo`03001FEeIG00EFE`03001>=dhg00A>=`03
00165dHG00A65`03000mnCgi00@mn@04000]nbgk;O/900066Yh200000e>O000000040005KcT01000
J_QZn0001F;I0P0000=JV000DUP01E9H00<004HIAQT014HI00@003Gi=OTen@<00085NP030001V`6K
008000048Qd003Xk0005>S/00`00Oomoo`05Ool00`00Oomoo`01Ool000moo`03001oogoo00Qoo`80
01moo`03001FEeIG00EFE`03001>=dhg00A>=`0300165dHG00=65`03000mnCgi008mn@H000A:NP03
0016oTKn0096oP`00003Ji`006li00A_>@03001Zn6[h009Zn005001RfF;IH]T00004FYP00`00DUQB
F004DUP00`00AQU66@03AQT00`00=OTen@04=OT200000`Ej0001V`020I/01@008Qd003Xk000013Xk
00<007ooOol01Goo00<007ooOol00Goo000>Ool2000<Ool00`00Oomoo`0LOol00`00EUMFE`05EUL0
0`00CSM>=`04CSL00`00AQM65`03AQL50003CSP00`00BWY:NP05BWX00`00A_i6oP02A_h00`00HcaS
?003Hc`00`00JcU[>@02JcT00`00KcU_>@03KcT00`00J_QZn003J_P01000H]T000001EZH00<0059H
DUP0159H00<004HIAQT014HI00<003Gi=OT00cGi00@001fk0005NP800081V`800003>S/003Xk00<j
>`03001oogoo00Eoo`03001oogoo005oo`007Goo00<007ooOol077oo00<005IGEUL015IG00<004hg
CSL014hg00<004HGAQL00`0000=>5`00CSP01Thh00<004YjBWX01DYj00@004Kn000000ES?003001[
>F/i00=[>@8000=_>@03001Zn6[h00EZn003001JV5ZH00EJV003001BF59H00EBF00300166DHI00=6
6@03000enCGi00@en@05000M^`000001F@0200020I/30004>S/00`00Oomoo`04Ool00`00Oomoo`01
Ool001ioo`03001oogoo01]oo`03001FEeIG00AFE`03001>=dhg00=>=`@000E>5`03001>>4hh00A>
>003001:NTYj00E:NP<000ES?003001[>F/i00=[>@06001_66lH001_>@0026[h00<005ZHFYP01UZH
00<0059HDUP0159H00<004HIAQT00dHI00<003Gi=OT013Gi00D001fk7K/M^`000081F@<000047O`0
00000002>S/00`00Oomoo`04Ool00`00Oomoo`01Ool001moo`8001]oo`03001FEeIG00EFE`03001>
=`000080009>5`03001>5dhG00A>5`03001>>4hh00E>>003001:NTYj00=:NP05001J^EZiF[T00003
Hc`00`00JcU[>@03JcT00`00KaQ_6002KaP00`00J_QZn006J_P00`00I]P00007FYP00`00DUQBF003
DUP;0004=OT01@007K/M^afk00000`5I0P000Qgl10001Woo00<007ooOol00Goo000QOol00`00Oomo
o`0IOol00`00EUMFE`03EUL30005CQL00`00CQM>5`04CQL00`00CSQ>>006CSP00`00BWY:NP020005
F[T00`00Hc`00006JcT00`00KaQ_6002KaP00`00J_QZn005J_P01000I]QVf0001eZH00<0059HDUP0
0`002goo1@0011fk0P000P5I0P0000<Mo0007O`00Qgl00<007ooOol017oo0@000Goo0Goo000ROol0
0`00Oomoo`0HOol00`00EUMFE`0300000e8g001>5`06CQL00`00CQM>5`04CQL00`00CSQ>>005CSP0
0`00BWX00008F[T00`00JcU[>@04JcT00`00KaQ_6002KaP00`00KaP00005J_P01@00I]QVf6KH0000
1eZH0`004Woo10000Qfk0`000P5I0`000Qgl00<007ooOol017oo0@000Goo0Goo000SOol00`00Oomo
o`0HOol20005DSL00`00CQM>5`04CQL00`00CQM>5`04CQL00`00CSQ>>005CSP20008F[T00`00I]P0
0004JcT00`00KaQ_6002KaP01@00KaQ_66lH00000f[h00<006KHI]P00fKH00<005ZHFYP00UZH0P00
6Goo100000D1V`000000006k0080008Mo003001oogoo00=oo`40005oo`5oo`0097oo00<007ooOol0
5goo00<0058gDSL0158g00<004hGCQL014hG00<004hGCQL01DhG00<004hhCSP00Thh0P0000=FN000
F[T01UZi00D006KHI]QVf00000=[>@03001_66lH009_6003001_66lH009_6003001Zn00000IVf005
001JV5ZHFYP0000NOol300000`6K000000020K/200000agl001oo`03Ool10001Ool1Ool002Eoo`80
01Qoo`03001B=e8g00=B=`03001>5dhG00E>5`03001>5dhG00A>5`05001>>4hhCSP00004EWP00`00
F[UJ^@03F[T00`00I]QVf003I]P00`00JcT00004KaP00`00KaQ_6004KaP00`00I]QVf005I]P01000
FYP000008Woo1P0000<1^`00000017oo0@000Goo0Goo000WOol00`00Oomoo`0EOol00`00DSMB=`04
DSL00`00CQM>5`04CQL00`00CQM>5`04CQL01000CSP000001EIh00<005ZiF[T00UZi00<006KHI]P0
1FKH00<006lHKaP00VlH00<006lHKaP01FlH00<006KHI]P01FKH0P009goo20000Woo0@000Goo0Goo
000XOol00`00Oomoo`0DOol00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CQM>5`04CQL20008EWP0
1000F[UJ^@0026KH0P000flH00<006lHKaP016lH00<006lH00001FKH00<007ooOol0:Woo1P0000=o
o`00Ool00Goo000YOol00`00Oomoo`0DOol00`00DSMB=`03DSL00`00CQM>5`04CQL01000CQM>5dhG
0`0000=B=`00EWP01eIh00<005Zi000026KH00H006lHKaP006lH0006KaP01000KaQ_600016KH00<0
07ooOol0;7oo00@007ooOomoo`@0005oo`00:Woo0P0057oo00<0058gDSL0158g00<004hGCQL014hG
0`001E8g00<005IhEWP01UIh00<006KHI]P01VKH00<006lHKaP00VlH00<006lHKaP016lH00<006lH
KaP00VlH00@006KHI]P0031oo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol04Woo00<0058g
DSL00e8g00<004hGCQL00dhG0P001e8g00<005IhEWP01UIh00<006KHI]P01FKH00<006lHKaP00flH
00<006lHKaP00flH00<006lHKaP00flH00<006KH0000<Woo00<007ooOol00Woo0@000Goo000]Ool0
0`00Oomoo`0AOol00`00DSMB=`03DSL00`00CQM>5`0300000e8g001B=`07DSL00`00EWQFN004EWP0
0`00H[P00005I]P00`00KaQ_6003KaP00`00LcP00003KaP00`00KaQ_6005KaP00`00Oomoo`0bOol0
1@00Oomoogoo00000Goo000^Ool00`00Oomoo`0@Ool01000DSMB=e8g1@001E8g00<0058gDSL01E8g
00<005IhEWP00eIh00D006:hH[QR^00000AVf003001_66lH009_6007001c>7<hLcP006lH00001flH
00<007ooOol0<goo00D007ooOomoo`00005oo`00;goo0P004Goo0`000dhh00<0058gDSL0158g00<0
058gDSL01E8g00<005IhEWP00UIh00<006:hH[P00V:h00D006KHI]QVf00000E_6003001c>7<h009c
>003001_66lH00E_6003001oogoo03Eoo`04001oogoo0001Ool0035oo`03001oogoo00ioo`03001>
>4hh00=>>003001B=e8g00AB=`03001B=e8g00IB=`04001FN5Ih0006H[P00`00I]P00005KaP00`00
LcQc>003LcP20005KaP00`00Oomoo`0gOol00`00Ool00001Ool0039oo`03001oogoo00ioo`03001>
>4hh00=>>003001B=e8g00AB=`03001B=e8g00EB=`03001FN00000QR^003001_66lH00A_6003001c
>7<h009c>004001cNG=i0003KaP00`00Oomoo`0hOol00`00Oomoo`01000003=oo`03001oogoo00eo
o`03001>>4hh00=>>003001B=e8g00AB=`03001B=e8g00EB=`8000QR^003001Zn00000A_6003001c
>7<h009c>003001cNG=i009cN@03001_600003aoo`40005oo`400000=7oo0P003Goo00<004hhCSP0
0dhh00<0058gDSL01E8g00<0058gDSL00U8g0P0000=JN000H[P01V:h00D006[hJ_QZn000009_6003
001c>7<h009c>003001cNG=i00=cN@8003ioo`800000=Woo00<007ooOol02goo00<004hhCSP00dhh
00<0058gDSL0158g00D0058gDSMB=`0000AJN003001R^6:h00=R^003001Zn6[h00=Zn003001_6000
00Ac>003001cNG=i009cN@03001oogoo03eoo`<00000=goo00<007ooOol02Woo00<004hhCSP00dhh
00<0058gDSL0158g00@0058g000000EJN003001R^6:h009R^003001Zn6[h00EZn08000=c>003001c
NG=i009cN@03001oogoo03Yoo`@000=oo`00>7oo00<007ooOol02Goo00<004hhCSP00dhh00<0058g
DSL00e8g0`0025Yh00@006:hH[P000QZn005001cN@00LcP00004LgT00`00Oomoo`0gOol40007Ool0
03Uoo`03001oogoo00Uoo`03001>>4hh00=>>003001B=`000080009BF003001JN5Yh00IJN003001R
^00000QZn004001cNG=iLgT20003LgT00`00Oomoo`0eOol3000;Ool003Yoo`8000Uoo`P000IBF003
001JN5Yh00IJN003001Zn6[h00IZn003001cNG=i009cN@05001cnP00LgT0000dOol4000>Ool003ao
o`03001oogoo00Ioo`030016FTIJ00=6FP03001BF59H00EBF003001JN5Yh00EJN08000MZn003001c
NG=i009cN@03001cnW?j0080035oo`@0019oo`00?Goo00<007ooOol01Woo00<004IJAUX00dIJ00<0
059HDUP01E9H00<005YhFWP00eYh00@006;iH_T000EZn003001cNG=i009cN@04001cnW?j000_Ool4
000FOol003ioo`03001oogoo00Eoo`030016FTIJ00=6FP03001BF59H00EBF003001JN5Yh009JN003
001RnF;i009Rn@05001Zn6[hJ_P00004LgT01000LoYcnP00;7oo10006Woo000oOol20006Ool00`00
AUY6FP03AUX00`00DUQBF005DUP01000FWQJN0001V;i00<006[h00001G=i00<007?j0000:Goo1000
7Woo0011Ool00`00Oomoo`04Ool00`00AUY6FP02AUX00`00DUQBF005DUP00`00FWP00007H_T30004
LgT00`00LoX0000WOol3000ROol0049oo`03001oogoo00=oo`030016FTIJ00=6FP03001BF59H00=B
F0<000MRn@04001[W6^LJi`2000017=i0000000097oo10009Goo0013Ool00`00Oomoo`03Ool40000
14IJ001BF59H0`000UKK00<006;iH_T00f;i0P001F^L0`008Woo1000:Goo0014Ool20004Ool00`00
=WdfO@040006E]/01@00H_URnF;i00001V^L0`007goo1000;Goo0016Ool00`00Oomoo`02Ool00`00
=WdfO@02=Wd00`00E]]Ff`05E]/00`00H_T00005Ji`3000MOol4000aOol004Moo`03001oogoo009o
o`03000fOCIm008fO@03001FfeKK00AFf`<000A[W0<001Yoo`@003Eoo`00B7oo00<007ooOol00Woo
100000@fO@00E]]Ff`<000=?W`800003Ji`0000001Qoo`@003Uoo`00BGoo0P0017oo1P001dnO0P00
5goo0`00?Goo001;Ool00`00Oomoo`03Ool60002Cil3000EOol40010Ool004aoo`03001oogoo00Eo
o`D001Aoo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo001>Ool00`00Oomo
o`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo00@0
07ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol200000goo00000002001K
Ool005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.258 0 0 r
.37614 .75111 .40063 .73217 .43872 .72292 .41492 .74343 Metetra
.41492 .74343 .43872 .72292 .47747 .70718 .45463 .72097 Metetra
.652 .949 .721 r
.45463 .72097 .47747 .70718 .51668 .68591 .49476 .68612 Metetra
.88 .953 .851 r
.49476 .68612 .51668 .68591 .55618 .66118 .53491 .64384 Metetra
.844 .804 .812 r
.53491 .64384 .55618 .66118 .59594 .63573 .57492 .60043 Metetra
.774 .692 .772 r
.57492 .60043 .59594 .63573 .63602 .61234 .61491 .56209 Metetra
.709 .615 .75 r
.61491 .56209 .63602 .61234 .6766 .59333 .65525 .53383 Metetra
.647 .56 .743 r
.65525 .53383 .6766 .59333 .71795 .58015 .69647 .51865 Metetra
.585 .521 .75 r
.69647 .51865 .71795 .58015 .76031 .57309 .73912 .51718 Metetra
.515 .497 .774 r
.73912 .51718 .76031 .57309 .80387 .57129 .78361 .52761 Metetra
.42 .489 .825 r
.78361 .52761 .80387 .57129 .84868 .57279 .8301 .54585 Metetra
.248 .503 .9 r
.8301 .54585 .84868 .57279 .89462 .57492 .87833 .56606 Metetra
0 .457 .798 r
.87833 .56606 .89462 .57492 .94137 .57475 .92766 .58161 Metetra
.294 0 0 r
.92766 .58161 .94137 .57475 .98855 .5697 .97713 .58639 Metetra
.35112 .75668 .37614 .75111 .41492 .74343 .39047 .74996 Metetra
.679 .978 .867 r
.39047 .74996 .41492 .74343 .45463 .72097 .43103 .72284 Metetra
.883 .957 .845 r
.43103 .72284 .45463 .72097 .49476 .68612 .47209 .67874 Metetra
.899 .867 .798 r
.47209 .67874 .49476 .68612 .53491 .64384 .51302 .62469 Metetra
.866 .789 .773 r
.51302 .62469 .53491 .64384 .57492 .60043 .55355 .56944 Metetra
.815 .724 .765 r
.55355 .56944 .57492 .60043 .61491 .56209 .59379 .52139 Metetra
.749 .665 .768 r
.59379 .52139 .61491 .56209 .65525 .53383 .63422 .48714 Metetra
.665 .607 .78 r
.63422 .48714 .65525 .53383 .69647 .51865 .67552 .47056 Metetra
.557 .547 .799 r
.67552 .47056 .69647 .51865 .73912 .51718 .71844 .47248 Metetra
.414 .482 .821 r
.71844 .47248 .73912 .51718 .78361 .52761 .76357 .49057 Metetra
.229 .415 .838 r
.76357 .49057 .78361 .52761 .8301 .54585 .81114 .51951 Metetra
.015 .359 .832 r
.81114 .51951 .8301 .54585 .87833 .56606 .8609 .55151 Metetra
0 .353 .781 r
.8609 .55151 .87833 .56606 .92766 .58161 .91199 .57747 Metetra
0 .454 .715 r
.91199 .57747 .92766 .58161 .97713 .58639 .96313 .58879 Metetra
.554 .801 .99 r
.32615 .74283 .35112 .75668 .39047 .74996 .36588 .73615 Metetra
.789 .839 .89 r
.36588 .73615 .39047 .74996 .43103 .72284 .40705 .70752 Metetra
.855 .816 .811 r
.40705 .70752 .43103 .72284 .47209 .67874 .44883 .66067 Metetra
.871 .794 .772 r
.44883 .66067 .47209 .67874 .51302 .62469 .49051 .60321 Metetra
.865 .777 .762 r
.49051 .60321 .51302 .62469 .55355 .56944 .53172 .54455 Metetra
.842 .763 .775 r
.53172 .54455 .55355 .56944 .59379 .52139 .57253 .49368 Metetra
.791 .744 .809 r
.57253 .49368 .59379 .52139 .63422 .48714 .6134 .4576 Metetra
.691 .706 .86 r
.6134 .4576 .63422 .48714 .67552 .47056 .65506 .44039 Metetra
.507 .625 .902 r
.65506 .44039 .67552 .47056 .71844 .47248 .6983 .44293 Metetra
.247 .486 .889 r
.6983 .44293 .71844 .47248 .76357 .49057 .74378 .46276 Metetra
.017 .351 .827 r
.74378 .46276 .76357 .49057 .81114 .51951 .79181 .49425 Metetra
0 .306 .796 r
.79181 .49425 .81114 .51951 .8609 .55151 .84215 .52912 Metetra
0 .389 .845 r
.84215 .52912 .8609 .55151 .91199 .57747 .89395 .5576 Metetra
.244 .592 .948 r
.89395 .5576 .91199 .57747 .96313 .58879 .94587 .57041 Metetra
.617 .643 .854 r
.3017 .70764 .32615 .74283 .36588 .73615 .34164 .69997 Metetra
.728 .68 .803 r
.34164 .69997 .36588 .73615 .40705 .70752 .38305 .67335 Metetra
.788 .706 .775 r
.38305 .67335 .40705 .70752 .44883 .66067 .42519 .63093 Metetra
.824 .734 .765 r
.42519 .63093 .44883 .66067 .49051 .60321 .46739 .57921 Metetra
.847 .768 .775 r
.46739 .57921 .49051 .60321 .53172 .54455 .50927 .52628 Metetra
.852 .812 .81 r
.50927 .52628 .53172 .54455 .57253 .49368 .55084 .47999 Metetra
.819 .864 .88 r
.55084 .47999 .57253 .49368 .6134 .4576 .59247 .44651 Metetra
.675 .882 .975 r
.59247 .44651 .6134 .4576 .65506 .44039 .63477 .42949 Metetra
.323 .736 .978 r
.63477 .42949 .65506 .44039 .6983 .44293 .67846 .42969 Metetra
0 .48 .851 r
.67846 .42969 .6983 .44293 .74378 .46276 .72412 .44493 Metetra
0 .357 .8 r
.72412 .44493 .74378 .46276 .79181 .49425 .77205 .47019 Metetra
.003 .385 .845 r
.77205 .47019 .79181 .49425 .84215 .52912 .8221 .4982 Metetra
.236 .484 .89 r
.8221 .4982 .84215 .52912 .89395 .5576 .87358 .5205 Metetra
.46 .576 .884 r
.87358 .5205 .89395 .5576 .94587 .57041 .92535 .52916 Metetra
.62 .568 .774 r
.27803 .65393 .3017 .70764 .34164 .69997 .31804 .6444 Metetra
.675 .592 .755 r
.31804 .6444 .34164 .69997 .38305 .67335 .3593 .62276 Metetra
.72 .628 .754 r
.3593 .62276 .38305 .67335 .42519 .63093 .40133 .59093 Metetra
.762 .68 .772 r
.40133 .59093 .42519 .63093 .46739 .57921 .44367 .5529 Metetra
.803 .758 .812 r
.44367 .5529 .46739 .57921 .50927 .52628 .48605 .51382 Metetra
.826 .874 .88 r
.48605 .51382 .50927 .52628 .55084 .47999 .52842 .47879 Metetra
.727 .974 .927 r
.52842 .47879 .55084 .47999 .59247 .44651 .57101 .45197 Metetra
.328 .807 .753 r
.57101 .45197 .59247 .44651 .63477 .42949 .61421 .43587 Metetra
.046 0 0 r
.61421 .43587 .63477 .42949 .67846 .42969 .65851 .43102 Metetra
.171 0 0 r
.65851 .43102 .67846 .42969 .72412 .44493 .7043 .4359 Metetra
0 .512 .819 r
.7043 .4359 .72412 .44493 .77205 .47019 .75181 .44707 Metetra
.264 .579 .942 r
.75181 .44707 .77205 .47019 .8221 .4982 .80095 .4597 Metetra
.457 .567 .878 r
.80095 .4597 .8221 .4982 .87358 .5205 .85131 .46836 Metetra
.557 .559 .812 r
.85131 .46836 .87358 .5205 .92535 .52916 .90221 .46812 Metetra
.622 .539 .74 r
.25503 .58849 .27803 .65393 .31804 .6444 .29503 .5766 Metetra
.633 .542 .734 r
.29503 .5766 .31804 .6444 .3593 .62276 .33582 .56166 Metetra
.65 .567 .748 r
.33582 .56166 .3593 .62276 .40133 .59093 .37729 .54409 Metetra
.674 .623 .79 r
.37729 .54409 .40133 .59093 .44367 .5529 .41934 .52482 Metetra
.705 .734 .873 r
.41934 .52482 .44367 .5529 .48605 .51382 .46191 .50506 Metetra
.679 .907 .975 r
.46191 .50506 .48605 .51382 .52842 .47879 .50497 .48608 Metetra
.317 .796 .724 r
.50497 .48608 .52842 .47879 .57101 .45197 .54859 .46895 Metetra
.036 0 0 r
.54859 .46895 .57101 .45197 .61421 .43587 .59288 .45433 Metetra
.134 0 0 r
.59288 .45433 .61421 .43587 .65851 .43102 .63797 .44238 Metetra
.044 0 0 r
.63797 .44238 .65851 .43102 .7043 .4359 .68397 .43267 Metetra
.321 .801 .901 r
.68397 .43267 .7043 .4359 .75181 .44707 .73097 .42428 Metetra
.578 .757 .963 r
.73097 .42428 .75181 .44707 .80095 .4597 .77896 .41599 Metetra
.612 .626 .84 r
.77896 .41599 .80095 .4597 .85131 .46836 .82786 .40647 Metetra
.617 .563 .771 r
.82786 .40647 .85131 .46836 .90221 .46812 .87752 .39457 Metetra
.628 .537 .732 r
.23218 .52018 .25503 .58849 .29503 .5766 .27218 .50587 Metetra
.597 .515 .733 r
.27218 .50587 .29503 .5766 .33582 .56166 .31234 .4978 Metetra
.57 .517 .756 r
.31234 .4978 .33582 .56166 .37729 .54409 .35297 .49508 Metetra
.541 .552 .815 r
.35297 .49508 .37729 .54409 .41934 .52482 .39438 .49572 Metetra
.484 .649 .931 r
.39438 .49572 .41934 .52482 .46191 .50506 .43679 .49699 Metetra
.258 .733 .939 r
.43679 .49699 .46191 .50506 .50497 .48608 .48031 .49588 Metetra
.07 0 0 r
.48031 .49588 .50497 .48608 .54859 .46895 .52489 .48974 Metetra
.132 0 0 r
.52489 .48974 .54859 .46895 .59288 .45433 .57033 .47687 Metetra
.028 0 0 r
.57033 .47687 .59288 .45433 .63797 .44238 .61637 .45688 Metetra
.61637 .45688 .63797 .44238 .68397 .43267 .66279 .43084 Metetra
.708 .979 .91 r
.66279 .43084 .68397 .43267 .73097 .42428 .70946 .40101 Metetra
.773 .824 .892 r
.70946 .40101 .73097 .42428 .77896 .41599 .75644 .37033 Metetra
.714 .669 .804 r
.75644 .37033 .77896 .41599 .82786 .40647 .80398 .34182 Metetra
.664 .583 .754 r
.80398 .34182 .82786 .40647 .87752 .39457 .85246 .31797 Metetra
.641 .558 .746 r
.20873 .45781 .23218 .52018 .27218 .50587 .24879 .44142 Metetra
.564 .511 .754 r
.24879 .44142 .27218 .50587 .31234 .4978 .28836 .43901 Metetra
.468 .474 .782 r
.28836 .43901 .31234 .4978 .35297 .49508 .3281 .44874 Metetra
.327 .45 .833 r
.3281 .44874 .35297 .49508 .39438 .49572 .36871 .46645 Metetra
.094 .436 .876 r
.36871 .46645 .39438 .49572 .43679 .49699 .41073 .48617 Metetra
0 .391 .721 r
.41073 .48617 .43679 .49699 .48031 .49588 .45446 .50111 Metetra
.224 0 0 r
.45446 .50111 .48031 .49588 .52489 .48974 .4998 .50501 Metetra
.051 0 0 r
.4998 .50501 .52489 .48974 .57033 .47687 .54632 .49369 Metetra
.54632 .49369 .57033 .47687 .61637 .45688 .5934 .46624 Metetra
.693 .971 .76 r
.5934 .46624 .61637 .45688 .66279 .43084 .64046 .42529 Metetra
.879 .964 .848 r
.64046 .42529 .66279 .43084 .70946 .40101 .68717 .3763 Metetra
.856 .828 .819 r
.68717 .3763 .70946 .40101 .75644 .37033 .73362 .32613 Metetra
.783 .707 .78 r
.73362 .32613 .75644 .37033 .80398 .34182 .78029 .28146 Metetra
.709 .619 .755 r
.78029 .28146 .80398 .34182 .85246 .31797 .82797 .24757 Metetra
.662 .609 .785 r
.18374 .40848 .20873 .45781 .24879 .44142 .22399 .39063 Metetra
.528 .538 .81 r
.22399 .39063 .24879 .44142 .28836 .43901 .26322 .39163 Metetra
.317 .439 .827 r
.26322 .39163 .28836 .43901 .3281 .44874 .30231 .40913 Metetra
.022 .308 .799 r
.30231 .40913 .3281 .44874 .36871 .46645 .34225 .43771 Metetra
0 .201 .705 r
.34225 .43771 .36871 .46645 .41073 .48617 .38386 .46944 Metetra
0 .221 .634 r
.38386 .46944 .41073 .48617 .45446 .50111 .42762 .49499 Metetra
.16 0 0 r
.42762 .49499 .45446 .50111 .4998 .50501 .47349 .50556 Metetra
.283 .785 .838 r
.47349 .50556 .4998 .50501 .54632 .49369 .52089 .49514 Metetra
.71 .984 .889 r
.52089 .49514 .54632 .49369 .5934 .46624 .56897 .46242 Metetra
.882 .967 .842 r
.56897 .46242 .5934 .46624 .64046 .42529 .61686 .41123 Metetra
.911 .893 .799 r
.61686 .41123 .64046 .42529 .68717 .3763 .66405 .34935 Metetra
.884 .817 .777 r
.66405 .34935 .68717 .3763 .73362 .32613 .71061 .2863 Metetra
.829 .745 .771 r
.71061 .2863 .73362 .32613 .78029 .28146 .75714 .23111 Metetra
.754 .677 .776 r
.75714 .23111 .78029 .28146 .82797 .24757 .80465 .19071 Metetra
.69 .707 .861 r
.15633 .3766 .18374 .40848 .22399 .39063 .19693 .35807 Metetra
.472 .623 .917 r
.19693 .35807 .22399 .39063 .26322 .39163 .23622 .35965 Metetra
.084 .414 .864 r
.23622 .35965 .26322 .39163 .30231 .40913 .27518 .37886 Metetra
0 .189 .699 r
.27518 .37886 .30231 .40913 .34225 .43771 .31491 .40998 Metetra
0 .123 .626 r
.31491 .40998 .34225 .43771 .38386 .46944 .35638 .44459 Metetra
0 .251 .723 r
.35638 .44459 .38386 .46944 .42762 .49499 .40019 .47267 Metetra
.143 .553 .925 r
.40019 .47267 .42762 .49499 .47349 .50556 .44637 .48469 Metetra
.575 .795 .983 r
.44637 .48469 .47349 .50556 .52089 .49514 .49438 .47416 Metetra
.79 .851 .896 r
.49438 .47416 .52089 .49514 .56897 .46242 .54325 .43963 Metetra
.867 .841 .817 r
.54325 .43963 .56897 .46242 .61686 .41123 .59202 .38528 Metetra
.889 .822 .775 r
.59202 .38528 .61686 .41123 .66405 .34935 .64008 .31955 Metetra
.885 .805 .765 r
.64008 .31955 .66405 .34935 .71061 .2863 .68739 .25271 Metetra
.858 .788 .781 r
.68739 .25271 .71061 .2863 .75714 .23111 .73456 .19438 Metetra
.798 .761 .819 r
.73456 .19438 .75714 .23111 .80465 .19071 .7826 .15189 Metetra
.692 .864 .963 r
.12577 .36346 .15633 .3766 .19693 .35807 .16688 .34506 Metetra
.324 .737 .978 r
.16688 .34506 .19693 .35807 .23622 .35965 .20677 .34421 Metetra
0 .405 .771 r
.20677 .34421 .23622 .35965 .27518 .37886 .24633 .35868 Metetra
0 .218 .652 r
.24633 .35868 .27518 .37886 .31491 .40998 .28658 .38339 Metetra
0 .244 .723 r
.28658 .38339 .31491 .40998 .35638 .44459 .32847 .41091 Metetra
.068 .408 .86 r
.32847 .41091 .35638 .44459 .40019 .47267 .37257 .43258 Metetra
.4 .563 .899 r
.37257 .43258 .40019 .47267 .44637 .48469 .41899 .44028 Metetra
.612 .646 .86 r
.41899 .44028 .44637 .48469 .49438 .47416 .46728 .42849 Metetra
.729 .691 .813 r
.46728 .42849 .49438 .47416 .54325 .43963 .51662 .39602 Metetra
.798 .724 .783 r
.51662 .39602 .54325 .43963 .59202 .38528 .56614 .34636 Metetra
.84 .757 .771 r
.56614 .34636 .59202 .38528 .64008 .31955 .61523 .28671 Metetra
.864 .794 .781 r
.61523 .28671 .64008 .31955 .68739 .25271 .6638 .22588 Metetra
.864 .836 .817 r
.6638 .22588 .68739 .25271 .73456 .19438 .71229 .17226 Metetra
.821 .875 .885 r
.71229 .17226 .73456 .19438 .7826 .15189 .76156 .13233 Metetra
.514 .913 .902 r
.09168 .36711 .12577 .36346 .16688 .34506 .13342 .3496 Metetra
.112 .663 .711 r
.13342 .3496 .16688 .34506 .20677 .34421 .17448 .34355 Metetra
.114 0 0 r
.17448 .34355 .20677 .34421 .24633 .35868 .2155 .34739 Metetra
0 .473 .775 r
.2155 .34739 .24633 .35868 .28658 .38339 .2572 .35764 Metetra
.157 .539 .924 r
.2572 .35764 .28658 .38339 .32847 .41091 .30024 .36937 Metetra
.394 .552 .893 r
.30024 .36937 .32847 .41091 .37257 .43258 .34507 .37701 Metetra
.528 .553 .824 r
.34507 .37701 .37257 .43258 .41899 .44028 .39177 .3755 Metetra
.61 .566 .78 r
.39177 .3755 .41899 .44028 .46728 .42849 .4401 .36152 Metetra
.67 .593 .761 r
.4401 .36152 .46728 .42849 .51662 .39602 .48952 .33432 Metetra
.721 .635 .761 r
.48952 .33432 .51662 .39602 .56614 .34636 .53948 .29602 Metetra
.769 .694 .78 r
.53948 .29602 .56614 .34636 .61523 .28671 .58954 .25102 Metetra
.812 .778 .822 r
.58954 .25102 .61523 .28671 .6638 .22588 .63959 .20487 Metetra
.828 .886 .885 r
.63959 .20487 .6638 .22588 .71229 .17226 .68987 .16306 Metetra
.745 .967 .933 r
.68987 .16306 .71229 .17226 .76156 .13233 .74093 .12997 Metetra
.05417 .38246 .09168 .36711 .13342 .3496 .0966 .3664 Metetra
.0966 .3664 .13342 .3496 .17448 .34355 .13937 .35304 Metetra
.13937 .35304 .17448 .34355 .2155 .34739 .18267 .34193 Metetra
.414 .841 .955 r
.18267 .34193 .2155 .34739 .2572 .35764 .22671 .33214 Metetra
.575 .758 .965 r
.22671 .33214 .2572 .35764 .30024 .36937 .27168 .32241 Metetra
.601 .628 .851 r
.27168 .32241 .30024 .36937 .34507 .37701 .31772 .31139 Metetra
.604 .56 .778 r
.31772 .31139 .34507 .37701 .39177 .3755 .36486 .2979 Metetra
.61 .534 .744 r
.36486 .2979 .39177 .3755 .4401 .36152 .41308 .28118 Metetra
.622 .536 .737 r
.41308 .28118 .4401 .36152 .48952 .33432 .46225 .26107 Metetra
.642 .565 .753 r
.46225 .26107 .48952 .33432 .53948 .29602 .51227 .23803 Metetra
.67 .627 .798 r
.51227 .23803 .53948 .29602 .58954 .25102 .56302 .21305 Metetra
.703 .74 .88 r
.56302 .21305 .58954 .25102 .63959 .20487 .61448 .18743 Metetra
.693 .897 .971 r
.61448 .18743 .63959 .20487 .68987 .16306 .66672 .16254 Metetra
.485 .901 .868 r
.66672 .16254 .68987 .16306 .74093 .12997 .7199 .13951 Metetra
.01394 .40164 .05417 .38246 .0966 .3664 .05706 .38729 Metetra
.05706 .38729 .0966 .3664 .13937 .35304 .1019 .36564 Metetra
.561 .933 .811 r
.1019 .36564 .13937 .35304 .18267 .34193 .14808 .33777 Metetra
.768 .955 .934 r
.14808 .33777 .18267 .34193 .22671 .33214 .19511 .30601 Metetra
.771 .805 .88 r
.19511 .30601 .22671 .33214 .27168 .32241 .24259 .27336 Metetra
.714 .671 .806 r
.24259 .27336 .27168 .32241 .31772 .31139 .29024 .24289 Metetra
.66 .583 .758 r
.29024 .24289 .31772 .31139 .36486 .2979 .33804 .21714 Metetra
.617 .531 .735 r
.33804 .21714 .36486 .2979 .41308 .28118 .38614 .19766 Metetra
.583 .507 .735 r
.38614 .19766 .41308 .28118 .46225 .26107 .43486 .18474 Metetra
.555 .51 .76 r
.43486 .18474 .46225 .26107 .51227 .23803 .48459 .17743 Metetra
.532 .55 .819 r
.48459 .17743 .51227 .23803 .56302 .21305 .53565 .17359 Metetra
.502 .649 .924 r
.53565 .17359 .56302 .21305 .61448 .18743 .58822 .17028 Metetra
.404 .781 .992 r
.58822 .17028 .61448 .18743 .66672 .16254 .64228 .16429 Metetra
.206 .732 .768 r
.64228 .16429 .66672 .16254 .7199 .13951 .6976 .15276 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<002eoo`03001oogoo
00Moo`03001oogoo02Aoo`00DWoo0`00;goo00<007ooOol027oo00<007ooOol08goo001?Ool3000b
Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<003Eoo`03001oogoo009oo`H00003Ool007oo
02=oo`00BWoo0`00=goo00<007oo00000`000UoM00H006]lJg`007oo000SOol004Moo`<0009oo`03
001oogoo03Eoo`800003?i/00000009Og@03001[O6]l009[O003001oo`00029oo`00AGoo0P00>goo
1@000eoM00<006]lJg`016]l0P008Woo0012Ool3000hOol50002?i/00`00GmeOg@02Gmd00`00Jga[
O004Jg`200000goo001oo`0POol002Eoo`@001Moo`8003Eoo`h000=Og@03001[O6]l00A[O006001W
6VLJ001oo`0087oo000VOol00`00Oomoo`0DOol3000_Ool80005<al20003Fil00`00Jia[W0020000
0eoM001[O005Jg`01000IaYW6VLJ0`0087oo000OOol50003Ool00`00Oomoo`0AOol2000SOol?0006
@Yd00`00<alc7`02<al20004Fil01P00Jia[W6^L001[W08000E[O003001W6VLJ00=W6P03001oo`00
01moo`00:7oo00<007ooOol03Goo0`008Goo10000THH00<004HjASX01DHj00<004:M@Yd02D:M00@0
03<O000000EKW`03001[W6^L009[W005001[W000KeX00003Jg`00`00IaYW6P05IaX2000OOol002Eo
o`04001oogoo000=Ool2000POol40007AQP00`00ASY6>P05ASX00`00@Ye2W@08@Yd30005Fil00`00
Jia[W004Ji`01`00KeY_FVmJ001[O00000QW6P03001oo`0001ioo`009Woo0P002goo0`007Woo1000
0TXG00<004HHAQP01THH00<004HjASX01THj00<004:M@Yd01D:M0P000e[l00D005^OFimKW`0000I[
W08000A_FP03001W6VLJ00MW6P800003Ool007oo01aoo`00<Goo0P0000=oo`00Ool06goo0`001dXG
00<004HHAQP01DHH00<004HjASX01dHj00<004:M@Yd00T:M0P001U[l00<005^O00001V^L00<006mJ
000016mJ00<006LJIaX01VLJ00D006:h001oo`0001eoo`00;Woo0`007Goo0`0000=>5`00BQL01dXG
00<004HHAQP01THH00<004HjASX01dHj00@004:M000000QJo0<000E[W004001_FVmJ0003KeX00`00
KcP00006IaX01000H[QR^6:h0`0077oo000[Ool3000NOol20005CQL00`00BQM:5`05BQL00`00AQQ6
6006AQP00`00ASY6>P07ASX20009F_`01@00IaYW6VLJ00000f^L00D006mJKeY_FP00009_FP04001_
>6lh0004IaX20004H[P01000Oomoo`001Goo0P0057oo000YOol20002Ool00`00Oomoo`0JOol20007
CQL00`00BQM:5`06BQL00`00AQQ66006AQP00`00ASY6>P04ASX200000eJI001Jo007F_`00`00IaYW
6P03IaX00`00Ji`00004KeX00`00KeX00004KcP01000IaYW6P001f:h00<007oo000017oo00@007oo
Ool001=oo`009Woo0`007goo0P0000=FF000CQL01ThG00<004XGBQL01TXG00<004HHAQP01THH00@0
04HjASY6>P<000AFV@03001Jo5[l00AJo003001W6VLJ00EW6P03001_FVmJ00=_FP8000E_>003001W
6P0000QR^004001oogoo0003Ool01000Oomoo`004goo000TOol2000POol20003EUP00`00CQM>5`06
CQL00`00BQM:5`05BQL00`00AQQ66006AQP01000ASX0000025JI00<005[lF_`00U[l00<006LJIaX0
1VLJ00<006mJKeX00fmJ00<006lhKcP01Flh00<006:hH[P01f:h00D007ooOomoo`00009oo`04001o
ogoo000COol0025oo`<0021oo`8000IFF003001>5dhG00E>5`03001:5dXG00I:5`03001664HH00E6
608000YFV@05001Jo5[lF_`00008IaX00`00KcP00003KeX20007KcP00`00H[QR^008H[P01000Ool0
00000Woo00@007ooOol001=oo`007goo0P008Goo0P0025IH00<004hGCQL01ThG00<004XGBQL01DXG
00<004HHAQP00dHH0P0000=BF000EYT02UJI00<005[l000026LJ00H006lhKcQ_>000KeX30006KcP0
0`00J_P00009H[P01@00Oomoogoo00000Woo0P0057oo000LOol3000ROol00`00F[T00008EUP00`00
CQM>5`06CQL00`00BQM:5`05BQL01000AQQ664HH0P00159H00<005JIEYT025JI0P0026LJ00<006lh
KcP00Vlh0`0000=_>000KcP016lh00@006[hJ_P000UR^08000=oo`03001oogoo01Eoo`006goo0P00
8Woo0P000eZi00<005IHEUP01UIH00<004hGCQL01DhG00<004XGBQL01TXG0`001U9H00<005JIEYT0
1eJI00<006;H00001fLJ00<006lhKcP00`0000=_>000KcP00Vlh00D006lhKcQ_>00000AZn003001R
^6:h00ER^003001JN00000=oo`03001oogoo01Eoo`0057oo0`0017oo00<007oo000087oo00<005Zi
F[T00eZi00<005IHEUP01UIH00<004hGCQL01ThG00<004XGBQL014XG0P002E9H00<005JIEYT01EJI
00D006;HH]QRf00000IW6P<000=_>003001c>00000=_>003001_>00000EZn003001R^6:h00=R^080
00=JN005001oogooOol0000FOol001Eoo`03001oogoo00=oo`04001oogoo000NOol00`00F[UJ^@05
F[T00`00EUQFF005EUP00`00CQM>5`06CQL01000BQM:5dXG0`0000=>=`00DUP02E9H00<005JIEYT0
0eJI00<006;HH]P00f;H00<006LJIaX00`001Flh00D007<hLcQc>00000=_>08000IZn003001R^6:h
008000EJN004001oogooOol2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0000K
Ool20008F[T00`00EUQFF006EUP00`00CQM>5`05CQL01000BQL000001Dhg00<0059HDUP01e9H00<0
05JIEYT00UJI00<006;HH]P016;H0`0000=W6P00KcP01Flh00D007<hLcQc>00000=_>003001Zn6[h
00EZn003001R^00000MJN003001oogoo009oo`03001oogoo01=oo`005Goo00<007ooOol00Woo0P00
17oo00<007ooOol05goo00<006<l00002EZi00<005IHEUP01EIH00<004hGCQL01DhG0P001dhg00<0
059HDUP0259H00@005JIEYT000ARf0<00004H]P006LJ0006KcP00`00LcQc>003LcP01@00KcP006[h
00001f[h00<005YhFWP01eYh00<007ooOol00Woo00<007ooOol04Woo000EOol00`00Oomoo`02Ool2
0005Ool00`00Oomoo`0DOol20003Hc`00`00F[UJ^@06F[T00`00EUQFF005EUP00`00CQM>5`02CQL3
00000dhg001>=`07CSL00`00DUQBF007DUP00`00EYT00002H]P30005H]P00`00KcQ_>004KcP00`00
LcQc>005LcP01@00J_QZn6[h00001F[h0P002EYh00<007ooOol00Woo00<007ooOol04Woo000DOol2
0004Ool00`00Oomoo`05Ool00`00Oomoo`0AOol20005Hc`00`00F[UJ^@06F[T00`00EUQFF006EUP4
0004CSL00`00CSM>=`06CSL00`00DUQBF007DUP200000f;H00000008H]P00`00KcQ_>004KcP00`00
LcQc>005LcP01@00J_QZn6[h000016[h00@006KHI]P000QJN003001oogoo00=oo`03001oogoo015o
o`006Woo00<007ooOol01Woo00<007ooOol03goo00<006<lHc`01V<l00<005ZiF[T01UZi00<005IH
EUP00eIH0`0024hg00<004hgCSL01Thg00<0059HDUP01E9H10002F;H00<006/H000016lh00<007<h
LcP01G<h00<007=H00000f[h00@006[hJ_P000=Vf003001JN5Yh00MJN003001oogoo00=oo`03001o
ogoo011oo`006Woo00<007ooOol01goo00<007ooOol037oo0`002F<l00<005ZiF[T01EZi00<005IH
EUP00`000Thh00<004hgCSL01Thg00<004hgCSL01Thg00<0059HDUP00e9H0`000UjH00<006;HH]P0
1V;H00D006/HJaQ[600000=_>003001c>7<h00Ac>005001cF7=HLeP00003J_P20004I]P00`00FWQJ
N007FWP00`00Oomoo`02Ool2000BOol001Uoo`8000Yoo`03001oogoo00Yoo`05001SgF?MHmd00008
Hc`00`00F[UJ^@02F[T60006CSP00`00CSM>=`05CSL00`00CSM>=`06CSL00`00DUQBF0040004GYP0
0`00H]QRf005H]P00`00JaQ[6003JaP00`00KcP00007LcP01@00LeQcF7=H00000f[h0P001FKH00<0
05YhFWP01UYh00<007ooOol017oo00<007ooOol03goo0008Ool20004Ool20003Ool30003Ool2000;
Ool00`00Oomoo`07Ool20005Hmd00`00HcaS?004Hc`60004CY/00`00CSQ>>005CSP00`00CSM>=`06
CSL00`00CSM>=`05CSL300000e9H001NV006GYP00`00H]QRf003H]P00`00JaQ[6005JaP00`00LcQc
>005LcP00`00LeQcF003LeP01@00J_P006KH00001FKH00<005YhFWP01UYh0P001Woo00<007ooOol0
3Woo0007Ool01000Oomoo`000goo0P001Woo00@007ooOol000eoo`03001oogoo00Eoo`03001SgF?M
00ISg@03001S?6<l00<0009;7P03001>VdjK00I>V`03001>>4hh00A>>003001>=dhg00I>=`03001>
=dhg009>=`<00004CSL0059H0008GYP00`00H]QRf002H]P00`00JaQ[6006JaP00`00LcQc>004LcP0
0`00LeQcF005LeP01@00I]QVf6KH00001FKH00<005YhFWP00eYh0P0000=FF000Ool01Goo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo00=o
o`03001SgF?M00ISg@@000I;7P03001>VdjK00E>V`03001>>4hh00E>>003001>=dhg00E>=`03001>
=`00008000A>=`8000YNV004001Rf6;H0008JaP00`00LeP00004LcP00`00LeQcF005LeP20004I]P0
0`00I]QVf002I]P01000FWQJN5Yh0P0015IH00<007ooOol017oo00<007ooOol03Goo0007Ool01000
Oomoo`0027oo0`000goo00<007ooOol03Goo0P0000=oo`0000000`000f?M0`0013MN00<004/NBah0
14/N00<004jKCY/014jK00<004hhCSP01Dhh00<004hgCSL014hg0`001Thg00<005IH00002UjH00<0
06;H000026/H00D007=HLeQcF00000=c>003001cF7=H00AcF004001_FVmJ0003I]P00`00I]QVf002
I]P01000FWP000001UIH00<007ooOol01Goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`00
27oo00<007ooOol00goo00<007ooOol03Woo0P000dNi2`000SMN00<004/NBah00d/N00<004jKCY/0
1DjK00<004hhCSP014hh00<004hgCSL00Thg0P000Thg00<004hgCSL00Thg0P000eIH00<005jHGYP0
25jH00<006/HJaP01V/H00<007=HLeP00g=H00<007<h00001W=H00D006mJKeY_FP0000AVf004001V
f6KHI]P20008EUP00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`06Ool000Qoo`8000Uoo`@0
009oo`03001oogoo00aoo`L000=KOP8000@Zo`H000E;7P03001>VdjK00A>V`03001>>4hh00E>>0@0
00A>=`03001>=dhg008000EFF003001NV5jH00MNV08000M[6003001cF7=H00EcF003001cF7=H00Ac
F003001_FVmJ00=_FP03001Vf6KH009Vf0<00003F[T005IH00QFF003001oogoo00Eoo`03001oogoo
00=oo`03001oogoo00Eoo`0067oo0P0037oo1P001e]n3P000T/N00<004jKCY/01DjK00<004hhCSP0
0Thh0`0024hg0P001eIH00<005jHGYP01UjH00@006KiI_T000I[6003001cF7=H00EcF003001cF7=H
00=cF003001_FVmJ00A_FP04001Vf6KHI]P30003F[T00`00EUQFF006EUP00`00Oomoo`06Ool00`00
Oomoo`03Ool00`00Oomoo`04Ool001Qoo`8000Uoo`P000IKOP80009Jf`03000nOCim00<nO@030009
^`0000L000I>V`03001>>0000080009>>003001>=dhg00A>=`<000UFF003001NV5jH00ANV003001V
nFKi009Vn@03001[66/H009[6003001cF7=H00EcF003001gV@0000=cF003001_FVmJ00E_FP04001V
f0000002F[T01@00F[UJ^EZi000025IH00<007ooOol01Goo0P000Woo00@007ooOol000Ioo`004Woo
0P0017oo00<007ooOol01Woo1P0017oo0`0000=KOP0000001E[K00<003im?Wd00cim00<000Vk2K/0
0`000QHm2@001Thh00<004hg00000`000TiH00<005IHEUP01eIH00<005jHGYP00ejH00<006KiI_T0
0fKi00D006/HJaQ[600000McF007001gVGNIMiT007=H000026mJ0P001EZi00@005ZiF[T000UFF003
001oogoo00Ioo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo00Aoo`@000]o
o`03001Jfe[K00IJf`03000nOCim00<nO@040009^`Vk2K/200025Sd00`00<S`0000=0007CUP00`00
EUQFF007EUP01@00GYQNV5jH00001fKi00<006/H00001g=H00<007NIMiT00gNI00<006mJKeX01VmJ
00<006=L00001UZi00@005ZiF[T000QFF003001oogoo00Moo`03001oogoo00Qoo`004Goo00@007oo
Ool000=oo`03001oogoo009oo`@000eoo`03001Jfe[K00MJf`03000nOCim00<nO@050009^`Vk0000
eP040005<S`00`00@SY2>P04@SX00`00CUQ>F006CUP00`00EUQFF007EUP01000GYQNV0002FKi00<0
07=HLeP01G=H00<007NIMiT017NI0P001FmJ0P000f=L00<005ZiF[T015Zi00<005Zi000025IH00<0
07ooOol01goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P0000=oo`0000000`0047oo00<0
05[KF]/01U[K2@0000<9^`0000000`001S8l00<0048j@SX0148j00<004iHCUP01DiH00<005IHEUP0
1eIH00<005jH00002VKi00<007=HLeP01G=H00<007NIMiT00gNI00@007?KLm/000=_FP03001SG6=L
00=SG003001J^EZi00AJ^@04001J^EZi0008EUP00`00Oomoo`07Ool00`00Oomoo`07Ool0015oo`04
001oogoo0003Ool4000DOol00`00F]]Jf`02F]/300000eIi0012>@04@ST50000103F000000001C8l
00<0048j@SX0148j00<004iHCUP01TiH00<005IHEUP01UIH0P002VKi00<006mJ00001G=H00<007NI
MiT00gNI00<007?KLm/00W?K00<006mJ00001f=L00<005ZiF[T015Zi00<005Zi00001UIH0`002Goo
00<007ooOol01goo000BOol20003Ool2000HOol01000F]/000001EIi00<0048i00000P0000A2>@00
:MXYfP8000040=H000000005<S`00`00@SY2>P04@SX00`00CUQ>F005CUP00`00EUQFF005EUP00`00
G]X00009I_T01@00KeY_FVmJ00000g=H00<007NIMiT00gNI00<007?KLm/00g?K0P002F=L00<005Zi
F[T015Zi0P000UIH10000TiH00<007ooOol027oo00<007ooOol01Woo000GOol00`00Oomoo`0HOol0
0`00EWUFN@04EWT30004@ST00`00:MXYfP02:MX40005<S`00`00@SY2>P04@SX00`00CUQ>F006CUP0
0`00EUQFF002EUP20003G]X00`00I_UVn@05I_T00`00KeY_FP03KeX00`00LeP00005MiT00`00Lm]c
f`03Lm/00`00Fod00009He`00`00F[UJ^@02F[T60006CUP00`00Oomoo`09Ool00`00Oomoo`05Ool0
01Moo`03001oogoo01Qoo`04001FNEIiEWT30002EWT00`00@SU2>@03@ST02000:MXYfRWJ0001>@00
07@20003<S`00`00@SY2>P05@SX00`00CUQ>F005CUP00`00EUQFF0020005G]X00`00I_UVn@04I_T0
0`00KeY_FP05KeX20004MiT00`00Lm]cf`02Lm/20003Fod00`00HeaSG005He`60003CYX20008CUP0
0`00Oomoo`08Ool00`00Oomoo`05Ool001Moo`03001oogoo01Uoo`<000EFN@030012>D8i00=2>@03
000YfRWJ008YfP040001>@0007@400000c8l0012>P05@SX00`00CUQ>F005CUP00`00EUP00008G]X0
0`00I_UVn@02I_T00`00KeY_FP05KeX00`00LmYcfP0200000gNI001cf`02Lm/20006Fod00`00HeaS
G0040002Bah00`00CYY>VP05CYX00`00CUQ>F006CUP00`00Oomoo`09Ool00`00Oomoo`04Ool000Eo
o`8000Aoo`8000=oo`<000Aoo`8001Moo`@000QFN@030012>D8i00=2>@05000YfRWJ:MX000020CT0
0`0007@000022K/20007@SX00`00CUQ>F003CUP3000:G]X01000I_UVn@001fmJ00<007?JLmX00W?J
100000=cf`00Fod01E_m1@001d/N00<004jJCYX01DjJ00<004iHCUP01DiH00<007ooOol027oo0`00
1Goo0004Ool01000Oomoo`000goo0P001Woo00D007ooOomoo`0001Eoo`<000Aoo`03001FNEIi00EF
N@030012>D8i00=2>@03000YfRWJ008YfP060001>@4i0000M0000`Vk1@000T8j00<004iHCUP00`00
0TjK00<005kJG]X025kJ00<006Ki00001fmJ00<007?JLmX00W?J00@005_hFoQKn0T000D[?003001;
7T/N00A;7P03001>VTjJ00E>VP03001>F4iH00E>F003001oogoo00Eoo`<0009oo`03001oogoo00=o
o`0017oo00@007ooOol000]oo`03001oogoo008001=oo`8000Moo`03001FNEIi00EFN@X0008YfP03
0001>@4i00<00089^`03000bG39L008bG0D000I>V`03001NfUkJ00MNfP8000M_FP03001cfW?J009c
fP03001Kn5_h00L00003:_l0000000D0008[?003001;7T/N00A;7P03001>VTjJ00A>VP8000M>F004
001oogooOol30005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool2000@Ool3000:
Ool01000EWUFNEIi0`0000=B=`00BQP014XH00<003Wi>OT00`00104i0`0000<9^`00<U`01C9L00<0
04jKCY/01TjK00<005kJG]X01EkJ00@006MlIg`000E_FP04001cfW?JLmX20002FoP200000e[K0012
O003@W`400000`7k000000070003Bah00`00CYY>VP05CYX20006CUP40008Ool00`00Oomoo`03Ool0
00Aoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool3000=Ool01000EWT00000158g
00<004XHBQP014XH00<003Wi>OT00SWi0`000P4i1@001S9L00<004jKCY/01DjK00<005kJG]X015kJ
00<006MlIg`00VMl00D006mJKeY_FP0000=cfP03001Kn5_h008000AJf`030012O49l00=2O003000M
o00000D000040QX0029N8Uh40007CYX00`00CUP00005CUP00`00Oomoo`09Ool00`00Oomoo`03Ool0
00Eoo`8000Uoo`@0009oo`03001oogoo00Uoo`<0015oo`03001B=e8g00EB=`03001:64XH00=:6003
000inCWi008in@03000YfRWJ00D000030AL0000000DbG003001>VdjK00I>V`03001NfUkJ009NfP03
001WO6Ml00AWO003001_FP0000=cfP@000MJf`030012O49l00=2O004000Mo1gl7O`20000105i0000
0000129N100014jJ00<004iH00001DiH00<007ooOol02Woo00<007ooOol00Woo000FOol00`00Oomo
o`06Ool3000DOol00`00DSMB=`05DSL00`00BQQ:6004BQP00`00>OTin@02>OT00`00:MXYfP02:MX3
00000`4G00000005<U`00`00CY]>V`05CY/01@00G]YNfUkJ00001fMl200000=Vn@00F]/01U[K00<0
049l@W`0149l00<001gl7O`00Qgl0P0000<1N@000000129N00<003YL>U`0100000A>VP00CUP000E>
F003001oogoo00Uoo`03001oogoo009oo`005Goo0P001Goo0`0067oo00<0058gDSL0158g00<004XH
BQP014XH00<003Wi>OT00SWi00<002WJ:MX00bWJ00D000fl0000004G00<000<bG003001>VdjK00E>
V`03001NfP0000MWO004001Ko5_lFo`200000f]l001Vn@02I_T00`00F]]Jf`04F]/:00027O`00`00
0GX000020GT200038Uh00`00>U`jG003>U`:000;Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3
000KOol00`00DSMB=`05DSL00`00BQQ:6003BQP00`00>OTin@03>OT00`00:MXYfP03:MX00`003K`=
_00300000a8m000000020004CY/60005Ig`20002Fo`20002Jg`00`00I_UVn@03I_T01000F]]Jfe[K
0P0000=FN000AST01DHi00<003Gj00000`0000D1NP000000006K0080008RGP03000jG3YL00<jG004
0016>DHi0004AST00`00Oomoo`09Ool00`00Oomoo`02Ool001Eoo`@001ioo`03001B=e8g00EB=`03
001:64XH00A:6003000inCWi00<in@03000YfRWJ00<YfP04000=_0fl3K`200034Sd50006Bcl01@00
IgaWO6Ml00000e_l00D006]lJga[O00000IVn@04001Jf`000004EWP00`00ASU6>@04AST01000=OXe
nSGj0P000P5j0P000P6K0`001SYL00@004HiAST000A6>@03001oogoo00Qoo`03001oogoo009oo`00
3Woo0`0017oo0P008Goo00<0058gDSL0158g00<004XHBQP014XH00<003Wi>OT00cWi00<002WJ:MX0
12WJ00@000fl3K`=_08000@B?@03001;?d/o00M;?`03001WO00000=Ko005001_>@00Jg`00007I_T2
0006EWP00`00ASU6>@04AST00`00=OXenP03=OX200020GX010000I/1V`6K0P001CYL00D004HiASU6
>@0000=6>@03001oogoo00Qoo`03001oogoo009oo`003goo00<007ooOol01Goo00<007ooOol07Woo
00<0058gDSL0158g00<004XHBQP00dXH5`000Q8m00<004/oBcl01D/o1`000fli00<006KiI_T01FKi
00<005jh00001eIh00<004HiAST014Hi00<003Gj=OX00SGj00<001fj00000P5j0P000`6K0P0013YL
00D004HiASU6>@0000=6>@03001oogoo00Moo`03001oogoo009oo`003goo00<007ooOol01Woo00<0
07ooOol07Woo00<0058gDSL0158g00<004XH00000P000ThG00<004XHBQP00dXH00<004HjASX014Hj
00<003jM?Yd01CjM10000T/o1P000V]l00<006liKcT00fli00<006KiI_T00fKi0P000ejh00<005Ih
EWP015Ih00<004HiAST01DHi00<003Gj=OX00SGj00<001fj00000P0000H1>@000I/1V`007O`20002
>U`01@00ASU6>DHi00000dHi00<007ooOol027oo00<007ooOol00Goo000?Ool00`00Oomoo`07Ool0
0`00Oomoo`0MOol00`00DSMB=`03DSL30005CQL00`00BQQ:6004BQP00`00ASY6>P04ASX00`00?Ydn
W@04?Yd00`00F_]Jn`0300000eNo001[O004Jg`00`00KcU_>@03KcT00`00JaP00003I_T00`00G[QN
^003G[P00`00EWQFN005EWP00`00ASU6>@04AST00`00=OXenP03=OX00`007KXM^P0200001@4i0000
006K00000Qgl0P0000<jG000AST00THi00@004HiAST000Yoo`03001oogoo005oo`003goo00<007oo
Ool027oo0P007Goo00<0058gDSL00`0000=B=`00CQL01ThG00<004XHBQP00dXH00<004HjASX01DHj
00@003jM?YdnW@8000IJn`8000A[O003001_>Fli00=_>@07001[66/HJaP006Ki00001ejh00<005Ih
EWP015Ih00<004HiAST00THi1P000SGj00<001fj7KX00Qfj00<0004i0CT00`000agl0P0014Hi00@0
04HiAST000Uoo`03001oogoo005oo`003Woo0P0037oo00<007ooOol06goo0P001E8g00<004hGCQL0
14hG00<004XHBQP014XH00<004HjASX01DHj00<003jM00001e[k00<006LIIaT00P0000=[O000KcT0
16li00<006/HJaP00f/H00<005jhG[P01Ujh00<005IhEWP00eIh1P000dXG00<0047h@OP0100011fj
0P000P4i00<001gl7O`00agl1@000THi00<007ooOol01goo00<007ooOol00Goo000MOol00`00Oomo
o`0JOol00`00DSMB=`04DSL00`00CQM>5`04CQL00`00BQQ:6004BQP00`00ASY6>P05ASX20007F_/0
0`00IaUW6@03IaT20005KcT00`00JaQ[6003JaP00`00G[QN^006G[P00`00EWQFN0030002DSL00`00
BQM:5`05BQL00`00@OQ1n002@OP300027KX00`000GX000020CT200037O`00`00>U`jG002>U`40008
Ool00`00Oomoo`01Ool001ioo`03001oogoo01Yoo`03001B=e8g00=B=`03001>5dhG00E>5`03001:
64XH00A:60040016>THjASX30002EWT00`00F_]Jn`03F_/00`00IaUW6@04IaT00`00KaP00003KcT0
0`00JaQ[6003JaP00`00J_P00007G[P30006DSL00`00BQM:5`04BQL00`00@OQ1n003@OP00`00=OX0
000300001P5j000000000I/000<Mo003000jG3YL008jG003000jG00000Qoo`03001oogoo005oo`00
7goo0P006Woo00<0058gDSL0158g00<004hGCQL014hG00<004XHBQP014XH00@004Hj000000IFN@05
001Jne[kF_/00006IaT01`00KaQ_66lH001_>@0000E[6004001Zn6[h0006G[P00`00FWL00007DSL0
0`00BQM:5`04BQL00`00@OQ1n004@OP01000=OXenSGj0P000P5j0P0000D1V`00000Mo00000DjG003
000jG00000Moo`03001oogoo005oo`008Goo00<007ooOol05goo00<0058gDSL0158g00<004hGCQL0
14hG00<004XHBQP00dXH0`002EIi00<005[k00001VLI00<006lHKaP00flH00<006/HJaP00f/H00<0
06[hJ_P00V[h00<005jhG[P00Ujh00D005YgFWMJM`0000IB=`03001:5dXG00E:5`030011n47h00=1
n003000enSGj00<enP800081NP030001V`6K00<000@jG003000jG00000Qoo`40005oo`5oo`008Woo
00<007ooOol05goo00<0058gDSL00e8g00<004hGCQL01DhG00<004XH00000P000U9G00<005IiEWT0
1eIi0P001VLI00<006lHKaP016lH00<006/HJaP00f/H00<006[hJ_P00f[h00@005jhG[P000AJM`03
001B=e8g00EB=`03001:5dXG00A:5`030011n47h00A1n003000enSGj008enP03000N700000@00081
V`03000jG3YL00<jG003000jG00000Moo`40005oo`5oo`008goo00<007ooOol05Woo00<0058gDSL0
158g00<004hGCQL00dhG0`001U9G00<005IiEWT01EIi00<006:h00001FLI00<006lHKaP016lH00<0
06lH00000f/H00<006[hJ_P016[h00<005jh00001EYg00<0058gDSL01E8g00<004XGBQL014XG00<0
047h@OP0147h00<003Gj=OX00cGj00@001hL7Q`N70@000040I/000000003>U`20007Ool10001Ool1
Ool002Aoo`03001oogoo01Eoo`03001B=e8g00AB=`04001>5dhGCQL200000e8g001BE`06DUL00`00
EWUFN@04EWT01@00H[QR^6:h00000fLI00<006lHKaP016lH00L006lHKaQ_6000JaP00007J_P20007
FWL00`00DSMB=`04DSL00`00BQM:5`05BQL00`00@OQ1n003@OP00`00=OXenP04=OX00`007Q`N7003
7Q`400000aiN000000030007Ool10001Ool1Ool002Eoo`8001Ioo`03001B=e8g00=B=`04001>5`00
0004DSL00`00DUMBE`05DUL00`00EWUFN@02EWT00`00H[QR^003H[P00`00IaT00006KaP00`00KaQ_
6003KaP00`00J_QZn005J_P00`00H]P00007FWL00`00DSMB=`05DSL01000BQM:5dXG5P0000<N7000
00000P0011iN0P000goo10000Woo000WOol00`00Oomoo`0COol00`00DSMB=`03DSL20006DSL00`00
DUMBE`06DUL01000EWUFN@001f:h00<006lHKaP016lH00<006lHKaP016lH00<006[hJ_P016[h00D0
06;HH]QRf00000MJM`03001B=e8g00=B=`D001Ioo`P0008NGP@0008T0080009oo`00:7oo00<007oo
Ool04Woo00<0058gDSL00`0000=>=`00DSL01E8g00<0059GDUL01U9G00<005Ii000026:h0P001FlH
00<006lHKaP00flH00<006lh000016[h00<006;HH]P00V;H00<005YgFWL01EYg00@0058gDSMB=`80
029oo`T00003Ool007oo005oo`00:Goo00<007ooOol04Woo0P001Dhg00<0058gDSL0158g00<0059G
DUL01E9G0P0026:h00@006[hJ_P000=_6003001_66lH00=_6005001_>6lhKcP00002J_P00`00H]QR
f003H]P00`00FWMJM`05FWL01000DSL00000:7oo0`0017oo0@000Goo000ZOol2000BOol00`00CSM>
=`04CSL00`00DSMB=`04DSL00`00DUMBE`04DUL00`00GYP00007H[P00`00J_QZn002J_P00`00KaP0
0006KaP01`00KcQ_>6lh001Zn00000MRf003001JMeYg00AJM`8002]oo`03001oogoo00=oo`40005o
o`00;7oo00<007ooOol03goo00<004hgCSL014hg00<0058gDSL01E8g00@0059GDUMBE`8000=NV003
001R^6:h00AR^003001Zn6[h00=Zn003001_66lH00A_6003001_>6lh00=_>003001Rf6;H00IRf004
001JMeYgFWL3000^Ool00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00moo`03001>=dhg00A>
=`03001B=e8g00AB=`04001BEe9G0005GYP00`00H[QR^003H[P00`00J_QZn004J_P20004KaP00`00
KcQ_>004KcP00`00H]QRf006H]P01000FWL00000<Woo00D007ooOomoo`00005oo`00;Woo00<007oo
Ool03Woo00<004hgCSL014hg00<0058gDSL0158g00<0059G00001ejH00D006:hH[QR^00000IZn004
001_>6lh0003KaP00`00KcQ_>003KcP00`00KcT00007H]P2000dOol01@00Oomoogoo00000Goo000_
Ool2000>Ool00`00CSM>=`04CSL00`00DSMB=`04DSL20009GYP01000H[QR^0001F[h00<006lhKcP0
0Vlh00<006lH00001Flh00D006liKcU_>@0000ARf08003Moo`04001oogoo0001Ool0035oo`03001o
ogoo00aoo`03001>=dhg00A>=`03001B=e8g009B=`03001FF00000UNV003001R^00000EZn003001_
>6lh00A_>003001_>6lh00=_>003001_>Fli00=_>@04001Rf000000jOol00`00Ool00001Ool0039o
o`03001oogoo00]oo`03001>=dhg00A>=`03001B=e8g008000=FF003001NV5jH00INV08000EZn003
001_>6lh00E_>08000=_>003001_>Fli00A_>@8003aoo`03001oogoo00400000<goo00<007ooOol0
2Woo00<004hgCSL014hg00<0058g00001EIH00<005jHGYP01EjH00<006KH00001F[h00<006lhKcP0
16lh00H007=iLgT006lh0005KcT2000oOol10001Ool1000003Aoo`8000]oo`04001>=dhgCSL40007
EUP00`00GYQNV003GYP01@00I]QVf6KH00000f[h00<006lhKcP016lh00<007=iLgT00W=i0P0016li
00<007ooOol0@7oo0P00000fOol00`00Oomoo`08Ool40003CUP00`00EUQFF005EUP00`00GYQNV002
GYP00`00I]QVf003I]P00`00J_P00006KcP00`00LgUcN@02LgT01@00Lm]cf`00KcT00P00@Woo0`00
000gOol00`00Oomoo`07Ool00`00CUQ>F005CUP00`00EUQFF005EUP01000GYQNV0001fKH0P001Flh
00<007=iLgT00W=i00@007?KLm]cf`80041oo`@000=oo`00>7oo00<007ooOol01goo00<004iHCUP0
14iH00<005IHEUP01EIH00<005jH000026KH00<006mI00000flh00<007=iLgT00W=i00D007?KLm]c
f`0003ioo`@000Moo`00>Goo00<007ooOol01Woo00<004iHCUP014iH00<005IHEUP01EIH0P0026KH
00L006mIKeU_F@00KcP00004LgT00`00Lm]cf`02000lOol3000;Ool003Yoo`8000Moo`03001>F4iH
00A>F003001FF5IH009FF0800003G[T006KH00IVf003001_FFmI00=_F@<0009cN@04001cfg?K000j
Ool4000>Ool003aoo`03001oogoo00Aoo`03001>F4iH00A>F005001FF5IHEUP00004G[T00`00I]QV
f003I]P00`00KeU_F@04KeT00`00Lm]cf`05000gOol4000BOol003eoo`03001oogoo00=oo`03001>
F4iH00A>F004001FF0000006G[T00`00I]QVf002I]P00`00KeU_F@03KeT01000Lm]cf`000U?G00<0
07ooOol0<goo10005Woo000nOol00`00Oomoo`03Ool00`00CUQ>F003CUP20008G[T01@00I]QVf6KH
00001FmI00D007?KLm/005?G0080039oo`@001Yoo`00?goo0P000goo00<004iHCUP00`0000=>V`00
G[T025ji00<006KH00001FmI00H007?KLm/005?G000`Ool4000NOol0045oo`04001oogooOol20005
CY/00`00G[UN^@06G[T30004KeT00`00Lm]cf`03000^Ool3000ROol0049oo`04001oogoo0007CY/0
0`00G[UN^@03G[T20003Ie`2000016mI001cfg?K0P00;7oo10009Goo0013Ool01000Oomoo`001TjK
00<005jiG[T00Uji00<006MLIe`016ML1@00:Woo1000:Goo0014Ool200000goo001>V`05CY/01000
G[T000001FML0P0000=Gn`000000:7oo1000;Goo0016Ool00`00Ool00003CY/50005Ie`01000Eo/0
00009Woo1000<Goo0017Ool00`00Ool000020005Acl200000fML00000002000TOol4000eOol004Qo
o`04001oogoo0006Acl4000ROol4000iOol004Uoo`800004Ool004LoAcl400000goo001oo`0OOol3
000mOol004]oo`04001oo`000005Ool00`00Oomoo`0JOol40010Ool004aoo`03001oogoo00Eoo`03
001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo001>Ool00`00
Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo
00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol200000goo00000002
001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02454 .26244 -12 -2.74006 ]
[.02454 .26244 0 6.25994 ]
[.0159 .31657 -24 -2.78582 ]
[.0159 .31657 0 6.21418 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00232 .43075 -18 -2.88287 ]
[-0.00232 .43075 0 6.11713 ]
[-0.01193 .49103 -6 -2.93439 ]
[-0.01193 .49103 0 6.06561 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03676 .25766 m
.04287 .25527 L
s
[(-1)] .02454 .26244 1 -0.3911 Mshowa
.02817 .3119 m
.0343 .30956 L
s
[(-0.5)] .0159 .31657 1 -0.38093 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01003 .42631 m
.01621 .4241 L
s
[(0.5)] -0.00232 .43075 1 -0.35936 Mshowa
.00046 .48671 m
.00666 .48456 L
s
[(1)] -0.01193 .49103 1 -0.34791 Mshowa
.125 Mabswid
.03507 .26836 m
.03873 .26693 L
s
.03336 .27913 m
.03703 .27771 L
s
.03164 .28997 m
.03531 .28856 L
s
.02991 .3009 m
.03358 .29949 L
s
.02641 .32298 m
.03009 .32158 L
s
.02464 .33413 m
.02833 .33274 L
s
.02286 .34537 m
.02655 .34399 L
s
.02107 .35668 m
.02476 .35531 L
s
.01744 .37956 m
.02114 .3782 L
s
.01561 .39112 m
.01931 .38977 L
s
.01377 .40277 m
.01746 .40142 L
s
.01191 .4145 m
.01561 .41316 L
s
.00815 .43822 m
.01185 .43689 L
s
.00625 .45021 m
.00996 .44889 L
s
.00433 .46229 m
.00805 .46098 L
s
.0024 .47445 m
.00612 .47315 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.201 0 0 r
.3771 .72791 .40169 .70185 .43949 .69119 .41587 .71254 Metetra
.41587 .71254 .43949 .69119 .47783 .68039 .45527 .6887 Metetra
.686 .979 .825 r
.45527 .6887 .47783 .68039 .51672 .66942 .49499 .65899 Metetra
.82 .869 .883 r
.49499 .65899 .51672 .66942 .55616 .6583 .53487 .62714 Metetra
.755 .704 .803 r
.53487 .62714 .55616 .6583 .59617 .64702 .57488 .59721 Metetra
.696 .609 .756 r
.57488 .59721 .59617 .64702 .63677 .63557 .61519 .57274 Metetra
.65 .555 .734 r
.61519 .57274 .63677 .63557 .67796 .62396 .65611 .55616 Metetra
.612 .524 .73 r
.65611 .55616 .67796 .62396 .71976 .61218 .69799 .54838 Metetra
.577 .513 .746 r
.69799 .54838 .71976 .61218 .76219 .60021 .74114 .54867 Metetra
.537 .525 .789 r
.74114 .54867 .76219 .60021 .80525 .58807 .78571 .55473 Metetra
.469 .579 .882 r
.78571 .55473 .80525 .58807 .84896 .57575 .83166 .56305 Metetra
.241 .659 .961 r
.83166 .56305 .84896 .57575 .89334 .56324 .87869 .56947 Metetra
.204 0 0 r
.87869 .56947 .89334 .56324 .9384 .55053 .92629 .57003 Metetra
.299 0 0 r
.92629 .57003 .9384 .55053 .98416 .53763 .97387 .56179 Metetra
.35165 .74588 .3771 .72791 .41587 .71254 .39135 .72644 Metetra
.675 .963 .747 r
.39135 .72644 .41587 .71254 .45527 .6887 .43186 .69152 Metetra
.89 .959 .837 r
.43186 .69152 .45527 .6887 .49499 .65899 .47258 .64615 Metetra
.871 .832 .805 r
.47258 .64615 .49499 .65899 .53487 .62714 .51312 .59735 Metetra
.808 .726 .773 r
.51312 .59735 .53487 .62714 .57488 .59721 .55341 .55246 Metetra
.741 .647 .755 r
.55341 .55246 .57488 .59721 .61519 .57274 .59372 .51768 Metetra
.672 .585 .749 r
.59372 .51768 .61519 .57274 .65611 .55616 .63455 .49716 Metetra
.599 .536 .755 r
.63455 .49716 .65611 .55616 .69799 .54838 .67652 .4924 Metetra
.513 .496 .776 r
.67652 .4924 .69799 .54838 .74114 .54867 .72019 .50222 Metetra
.399 .467 .814 r
.72019 .50222 .74114 .54867 .78571 .55473 .76589 .52272 Metetra
.22 .448 .867 r
.76589 .52272 .78571 .55473 .83166 .56305 .81356 .54778 Metetra
0 .412 .829 r
.81356 .54778 .83166 .56305 .87869 .56947 .86273 .5699 Metetra
.28 0 0 r
.86273 .5699 .87869 .56947 .92629 .57003 .91246 .58163 Metetra
.219 0 0 r
.91246 .58163 .92629 .57003 .97387 .56179 .96166 .57732 Metetra
.675 .963 .937 r
.3259 .74724 .35165 .74588 .39135 .72644 .36637 .72528 Metetra
.871 .933 .861 r
.36637 .72528 .39135 .72644 .43186 .69152 .40784 .68373 Metetra
.895 .859 .798 r
.40784 .68373 .43186 .69152 .47258 .64615 .44952 .62915 Metetra
.877 .798 .769 r
.44952 .62915 .47258 .64615 .51312 .59735 .49084 .57053 Metetra
.839 .747 .763 r
.49084 .57053 .51312 .59735 .55341 .55246 .53166 .51708 Metetra
.783 .699 .771 r
.53166 .51708 .55341 .55246 .59372 .51768 .57228 .47644 Metetra
.702 .646 .791 r
.57228 .47644 .59372 .51768 .63455 .49716 .61329 .45358 Metetra
.586 .583 .816 r
.61329 .45358 .63455 .49716 .67652 .4924 .65545 .45033 Metetra
.421 .502 .837 r
.65545 .45033 .67652 .4924 .72019 .50222 .69949 .46525 Metetra
.207 .406 .838 r
.69949 .46525 .72019 .50222 .76589 .52272 .74584 .49368 Metetra
0 .323 .81 r
.74584 .49368 .76589 .52272 .81356 .54778 .79453 .52811 Metetra
0 .31 .778 r
.79453 .52811 .81356 .54778 .86273 .5699 .84499 .55915 Metetra
0 .434 .796 r
.84499 .55915 .86273 .5699 .91246 .58163 .8961 .57726 Metetra
.19 .71 .873 r
.8961 .57726 .91246 .58163 .96166 .57732 .94646 .5751 Metetra
.738 .778 .885 r
.3005 .72663 .3259 .74724 .36637 .72528 .34142 .70427 Metetra
.823 .779 .811 r
.34142 .70427 .36637 .72528 .40784 .68373 .3835 .66216 Metetra
.853 .774 .773 r
.3835 .66216 .40784 .68373 .44952 .62915 .42588 .60692 Metetra
.862 .772 .761 r
.42588 .60692 .44952 .62915 .49084 .57053 .46792 .54762 Metetra
.853 .773 .773 r
.46792 .54762 .49084 .57053 .53166 .51708 .50943 .49355 Metetra
.821 .774 .809 r
.50943 .49355 .53166 .51708 .57228 .47644 .55063 .45237 Metetra
.739 .763 .872 r
.55063 .45237 .57228 .47644 .61329 .45358 .59211 .42908 Metetra
.554 .7 .937 r
.59211 .42908 .61329 .45358 .65545 .45033 .63464 .42553 Metetra
.248 .538 .923 r
.63464 .42553 .65545 .45033 .69949 .46525 .67895 .44025 Metetra
0 .356 .828 r
.67895 .44025 .69949 .46525 .74584 .49368 .72556 .46857 Metetra
0 .286 .779 r
.72556 .46857 .74584 .49368 .79453 .52811 .77451 .50294 Metetra
0 .354 .826 r
.77451 .50294 .79453 .52811 .84499 .55915 .8253 .53391 Metetra
.223 .529 .92 r
.8253 .53391 .84499 .55915 .8961 .57726 .87685 .55184 Metetra
.525 .705 .953 r
.87685 .55184 .8961 .57726 .94646 .5751 .92776 .54934 Metetra
.688 .638 .794 r
.27593 .68351 .3005 .72663 .34142 .70427 .31694 .66293 Metetra
.753 .669 .769 r
.31694 .66293 .34142 .70427 .3835 .66216 .35912 .62648 Metetra
.797 .704 .763 r
.35912 .62648 .3835 .66216 .42588 .60692 .40175 .57933 Metetra
.829 .748 .774 r
.40175 .57933 .42588 .60692 .46792 .54762 .44425 .5287 Metetra
.849 .806 .808 r
.44425 .5287 .46792 .54762 .50943 .49355 .48642 .48208 Metetra
.835 .88 .875 r
.48642 .48208 .50943 .49355 .55063 .45237 .52839 .44579 Metetra
.709 .932 .962 r
.52839 .44579 .55063 .45237 .59211 .42908 .57062 .42402 Metetra
.336 .797 .939 r
.57062 .42402 .59211 .42908 .63464 .42553 .61373 .41831 Metetra
0 .51 .789 r
.61373 .41831 .63464 .42553 .67895 .44025 .65836 .42743 Metetra
0 .37 .759 r
.65836 .42743 .67895 .44025 .72556 .46857 .70493 .44745 Metetra
0 .394 .84 r
.70493 .44745 .72556 .46857 .77451 .50294 .75355 .47213 Metetra
.219 .48 .891 r
.75355 .47213 .77451 .50294 .8253 .53391 .80382 .49382 Metetra
.439 .55 .871 r
.80382 .49382 .8253 .53391 .87685 .55184 .85493 .50486 Metetra
.587 .599 .832 r
.85493 .50486 .87685 .55184 .92776 .54934 .90581 .49947 Metetra
.649 .565 .747 r
.25234 .62239 .27593 .68351 .31694 .66293 .29309 .60526 Metetra
.687 .595 .747 r
.29309 .60526 .31694 .66293 .35912 .62648 .33483 .5793 Metetra
.727 .643 .765 r
.33483 .5793 .35912 .62648 .40175 .57933 .37715 .54723 Metetra
.769 .721 .808 r
.37715 .54723 .40175 .57933 .44425 .5287 .41972 .51294 Metetra
.804 .844 .881 r
.41972 .51294 .44425 .5287 .48642 .48208 .46237 .48062 Metetra
.731 .97 .932 r
.46237 .48062 .48642 .48208 .52839 .44579 .50516 .45393 Metetra
.336 .804 .712 r
.50516 .45393 .52839 .44579 .57062 .42402 .54835 .43537 Metetra
.034 0 0 r
.54835 .43537 .57062 .42402 .61373 .41831 .59229 .42589 Metetra
.163 0 0 r
.59229 .42589 .61373 .41831 .65836 .42743 .63735 .42471 Metetra
0 .528 .723 r
.63735 .42471 .65836 .42743 .70493 .44745 .68382 .42948 Metetra
.271 .644 .966 r
.68382 .42948 .70493 .44745 .75355 .47213 .73175 .43655 Metetra
.473 .6 .898 r
.73175 .43655 .75355 .47213 .80382 .49382 .78097 .44164 Metetra
.557 .561 .814 r
.78097 .44164 .80382 .49382 .85493 .50486 .83106 .44059 Metetra
.607 .553 .768 r
.83106 .44059 .85493 .50486 .90581 .49947 .88145 .43039 Metetra
.62 .529 .729 r
.22944 .55152 .25234 .62239 .29309 .60526 .26969 .53863 Metetra
.623 .541 .74 r
.26969 .53863 .29309 .60526 .33483 .5793 .31056 .52554 Metetra
.634 .582 .778 r
.31056 .52554 .33483 .5793 .37715 .54723 .35208 .51225 Metetra
.652 .674 .859 r
.35208 .51225 .37715 .54723 .41972 .51294 .39426 .49875 Metetra
.634 .844 .981 r
.39426 .49875 .41972 .51294 .46237 .48062 .43711 .48503 Metetra
.323 .809 .793 r
.43711 .48503 .46237 .48062 .50516 .45393 .48064 .47109 Metetra
.026 0 0 r
.48064 .47109 .50516 .45393 .54835 .43537 .52489 .45693 Metetra
.114 0 0 r
.52489 .45693 .54835 .43537 .59229 .42589 .56985 .44253 Metetra
.026 0 0 r
.56985 .44253 .59229 .42589 .63735 .42471 .61556 .4279 Metetra
.323 .809 .793 r
.61556 .4279 .63735 .42471 .68382 .42948 .66203 .41302 Metetra
.634 .844 .981 r
.66203 .41302 .68382 .42948 .73175 .43655 .70928 .39789 Metetra
.652 .674 .859 r
.70928 .39789 .73175 .43655 .78097 .44164 .75733 .38251 Metetra
.634 .582 .778 r
.75733 .38251 .78097 .44164 .83106 .44059 .80619 .36687 Metetra
.623 .541 .74 r
.80619 .36687 .83106 .44059 .88145 .43039 .8559 .35095 Metetra
.598 .516 .733 r
.20653 .48062 .22944 .55152 .26969 .53863 .24622 .47179 Metetra
.553 .5 .75 r
.24622 .47179 .26969 .53863 .31056 .52554 .28603 .47119 Metetra
.5 .511 .799 r
.28603 .47119 .31056 .52554 .35208 .51225 .32644 .47647 Metetra
.414 .567 .897 r
.32644 .47647 .35208 .51225 .39426 .49875 .36787 .48405 Metetra
.193 .64 .945 r
.36787 .48405 .39426 .49875 .43711 .48503 .41061 .48966 Metetra
.102 0 0 r
.41061 .48966 .43711 .48503 .48064 .47109 .45471 .48925 Metetra
.14 0 0 r
.45471 .48925 .48064 .47109 .52489 .45693 .49999 .47981 Metetra
.011 0 0 r
.49999 .47981 .52489 .45693 .56985 .44253 .5461 .46015 Metetra
.5461 .46015 .56985 .44253 .61556 .4279 .59267 .43124 Metetra
.719 .988 .852 r
.59267 .43124 .61556 .4279 .66203 .41302 .63941 .39593 Metetra
.821 .881 .888 r
.63941 .39593 .66203 .41302 .70928 .39789 .68626 .35829 Metetra
.759 .717 .813 r
.68626 .35829 .70928 .39789 .75733 .38251 .73339 .32268 Metetra
.694 .613 .762 r
.73339 .32268 .75733 .38251 .80619 .36687 .78123 .29289 Metetra
.643 .551 .737 r
.78123 .29289 .80619 .36687 .8559 .35095 .83034 .27149 Metetra
.581 .529 .761 r
.18266 .41871 .20653 .48062 .24622 .47179 .2219 .41293 Metetra
.464 .473 .783 r
.2219 .41293 .24622 .47179 .28603 .47119 .26075 .422 Metetra
.29 .416 .817 r
.26075 .422 .28603 .47119 .32644 .47647 .30005 .44198 Metetra
.027 .354 .829 r
.30005 .44198 .32644 .47647 .36787 .48405 .34063 .46662 Metetra
0 .299 .71 r
.34063 .46662 .36787 .48405 .41061 .48966 .38306 .48827 Metetra
.28 0 0 r
.38306 .48827 .41061 .48966 .45471 .48925 .4275 .49926 Metetra
.072 0 0 r
.4275 .49926 .45471 .48925 .49999 .47981 .47367 .49379 Metetra
.47367 .49379 .49999 .47981 .5461 .46015 .52092 .46954 Metetra
.713 .981 .783 r
.52092 .46954 .5461 .46015 .59267 .43124 .56851 .42841 Metetra
.888 .97 .833 r
.56851 .42841 .59267 .43124 .63941 .39593 .61585 .37596 Metetra
.884 .857 .81 r
.61585 .37596 .63941 .39593 .68626 .35829 .66276 .31988 Metetra
.821 .746 .781 r
.66276 .31988 .68626 .35829 .73339 .32268 .70952 .26808 Metetra
.746 .656 .762 r
.70952 .26808 .73339 .32268 .78123 .29289 .75684 .22715 Metetra
.668 .586 .754 r
.75684 .22715 .78123 .29289 .83034 .27149 .80564 .2014 Metetra
.568 .581 .827 r
.1568 .3725 .18266 .41871 .2219 .41293 .19589 .3682 Metetra
.331 .466 .846 r
.19589 .3682 .2219 .41293 .26075 .422 .23416 .38243 Metetra
0 .292 .79 r
.23416 .38243 .26075 .422 .30005 .44198 .27272 .41044 Metetra
0 .144 .663 r
.27272 .41044 .30005 .44198 .34063 .46662 .31266 .44461 Metetra
0 .144 .602 r
.31266 .44461 .34063 .46662 .38306 .48827 .35479 .47534 Metetra
0 .352 .703 r
.35479 .47534 .38306 .48827 .4275 .49926 .39943 .49282 Metetra
.247 .741 .911 r
.39943 .49282 .4275 .49926 .47367 .49379 .44626 .4895 Metetra
.7 .959 .946 r
.44626 .4895 .47367 .49379 .52092 .46954 .49449 .46231 Metetra
.871 .945 .861 r
.49449 .46231 .52092 .46954 .56851 .42841 .5431 .41378 Metetra
.908 .885 .799 r
.5431 .41378 .56851 .42841 .61585 .37596 .59131 .35117 Metetra
.895 .826 .771 r
.59131 .35117 .61585 .37596 .66276 .31988 .63879 .28437 Metetra
.856 .772 .768 r
.63879 .28437 .66276 .31988 .70952 .26808 .68584 .22329 Metetra
.792 .716 .779 r
.68584 .22329 .70952 .26808 .75684 .22715 .73328 .17599 Metetra
.701 .653 .799 r
.73328 .17599 .75684 .22715 .80564 .2014 .78223 .14761 Metetra
.54 .703 .945 r
.128 .34568 .1568 .3725 .19589 .3682 .16733 .34102 Metetra
.114 .484 .899 r
.16733 .34102 .19589 .3682 .23416 .38243 .20568 .35499 Metetra
0 .199 .689 r
.20568 .35499 .23416 .38243 .27272 .41044 .24421 .38283 Metetra
0 .096 .592 r
.24421 .38283 .27272 .41044 .31266 .44461 .28409 .41688 Metetra
0 .196 .685 r
.28409 .41688 .31266 .44461 .35479 .47534 .32625 .44746 Metetra
.079 .468 .889 r
.32625 .44746 .35479 .47534 .39943 .49282 .37107 .4647 Metetra
.505 .707 .961 r
.37107 .4647 .39943 .49282 .44626 .4895 .41832 .46097 Metetra
.738 .787 .893 r
.41832 .46097 .44626 .4895 .49449 .46231 .4672 .43318 Metetra
.833 .8 .82 r
.4672 .43318 .49449 .46231 .5431 .41378 .51666 .38387 Metetra
.871 .799 .777 r
.51666 .38387 .5431 .41378 .59131 .35117 .5658 .32041 Metetra
.881 .799 .764 r
.5658 .32041 .59131 .35117 .63879 .28437 .61425 .25274 Metetra
.87 .799 .777 r
.61425 .25274 .63879 .28437 .68584 .22329 .6622 .19084 Metetra
.831 .795 .818 r
.6622 .19084 .68584 .22329 .73328 .17599 .71044 .14281 Metetra
.739 .771 .879 r
.71044 .14281 .73328 .17599 .78223 .14761 .76008 .1138 Metetra
.401 .82 .97 r
.09556 .33859 .128 .34568 .16733 .34102 .13563 .33169 Metetra
0 .487 .761 r
.13563 .33169 .16733 .34102 .20568 .35499 .17489 .33989 Metetra
0 .265 .636 r
.17489 .33989 .20568 .35499 .24421 .38283 .21435 .3592 Metetra
0 .262 .714 r
.21435 .3592 .24421 .38283 .28409 .41688 .25504 .38327 Metetra
.044 .4 .855 r
.25504 .38327 .28409 .41688 .32625 .44746 .29779 .40427 Metetra
.366 .528 .885 r
.29779 .40427 .32625 .44746 .37107 .4647 .34297 .41437 Metetra
.569 .599 .844 r
.34297 .41437 .37107 .4647 .41832 .46097 .39044 .40761 Metetra
.685 .644 .803 r
.39044 .40761 .41832 .46097 .4672 .43318 .43956 .38161 Metetra
.759 .682 .778 r
.43956 .38161 .4672 .43318 .51666 .38387 .48948 .33834 Metetra
.809 .723 .77 r
.48948 .33834 .51666 .38387 .5658 .32041 .53942 .28352 Metetra
.844 .771 .781 r
.53942 .28352 .5658 .32041 .61425 .25274 .589 .22502 Metetra
.861 .829 .815 r
.589 .22502 .61425 .25274 .6622 .19084 .6383 .17091 Metetra
.838 .892 .879 r
.6383 .17091 .6622 .19084 .71044 .14281 .68792 .12787 Metetra
.722 .922 .96 r
.68792 .12787 .71044 .14281 .76008 .1138 .73875 .10022 Metetra
.05921 .34817 .09556 .33859 .13563 .33169 .1005 .33735 Metetra
.033 0 0 r
.1005 .33735 .13563 .33169 .17489 .33989 .14156 .33499 Metetra
0 .553 .754 r
.14156 .33499 .17489 .33989 .21435 .3592 .18302 .33867 Metetra
.208 .622 .951 r
.18302 .33867 .21435 .3592 .25504 .38327 .22552 .34467 Metetra
.422 .591 .913 r
.22552 .34467 .25504 .38327 .29779 .40427 .26957 .34862 Metetra
.529 .555 .826 r
.26957 .34862 .29779 .40427 .34297 .41437 .31541 .34626 Metetra
.592 .548 .774 r
.31541 .34626 .34297 .41437 .39044 .40761 .36299 .3345 Metetra
.64 .563 .752 r
.36299 .3345 .39044 .40761 .43956 .38161 .41196 .31211 Metetra
.684 .597 .752 r
.41196 .31211 .43956 .38161 .48948 .33834 .46187 .28008 Metetra
.729 .652 .773 r
.46187 .28008 .48948 .33834 .53942 .28352 .51225 .2414 Metetra
.775 .737 .818 r
.51225 .2414 .53942 .28352 .589 .22502 .56286 .20029 Metetra
.805 .855 .887 r
.56286 .20029 .589 .22502 .6383 .17091 .6137 .16126 Metetra
.747 .962 .937 r
.6137 .16126 .6383 .17091 .68792 .12787 .66509 .12818 Metetra
.502 .909 .857 r
.66509 .12818 .68792 .12787 .73875 .10022 .71754 .10368 Metetra
.01926 .36808 .05921 .34817 .1005 .33735 .06217 .35219 Metetra
.06217 .35219 .1005 .33735 .14156 .33499 .10583 .33602 Metetra
.486 .896 .917 r
.10583 .33602 .14156 .33499 .18302 .33867 .15025 .31957 Metetra
.647 .835 .973 r
.15025 .31957 .18302 .33867 .22552 .34467 .19545 .30283 Metetra
.648 .678 .865 r
.19545 .30283 .22552 .34467 .26957 .34862 .24146 .28579 Metetra
.626 .582 .785 r
.24146 .28579 .26957 .34862 .31541 .34626 .28829 .26844 Metetra
.612 .535 .744 r
.28829 .26844 .31541 .34626 .36299 .3345 .33597 .25079 Metetra
.607 .521 .732 r
.33597 .25079 .36299 .3345 .41196 .31211 .38452 .2328 Metetra
.612 .535 .744 r
.38452 .2328 .41196 .31211 .46187 .28008 .43397 .21449 Metetra
.626 .582 .785 r
.43397 .21449 .46187 .28008 .51225 .2414 .48434 .19583 Metetra
.648 .678 .865 r
.48434 .19583 .51225 .2414 .56286 .20029 .53566 .17683 Metetra
.647 .835 .973 r
.53566 .17683 .56286 .20029 .6137 .16126 .58795 .15746 Metetra
.486 .896 .917 r
.58795 .15746 .6137 .16126 .66509 .12818 .64124 .13773 Metetra
.64124 .13773 .66509 .12818 .71754 .10368 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`02Ool300000goo001oo`0VOol004Ioo`04001o
ogoo000=Ool2000[Ool00`00Ool000020003Gkh3000VOol004Moo`8000]oo`<002eoo`@000EO_P80
02Ioo`00DWoo0`00;Goo10001Unn00@005lL000002Eoo`00Cgoo0`00;Woo1@001Enn0P000elL0P00
97oo001<Ool3000^Ool80003Gkh20005Ga`2000TOol004Yoo`<002eoo`@0009Og@04001[W6^LJi`2
00000enn001O7007Ga`2000SOol004Moo`<002eoo`<00003?id00000009Og@03001[W6^L009[W003
001[W00000UO7003001oo`00029oo`00AGoo0P00;Goo0`000SnM0P000UoM0P001F^L0`002ElL0`00
8Woo0012Ool3000/Ool50002?id01@00GmeOgEoM00001f^L00@006/jJcX000IO70800004FYT007oo
000QOol002Eoo`@001Moo`8002aoo`<000ECG`@00003Gmd006^L00I[W08000=[>P03001O75lL009O
7003001JVEZI009JV@03001oo`00021oo`009Woo00<007ooOol057oo0`00:goo0`001e=O00<006Ml
Ig`00VMl00<006^LJi`01F^L00<006/j000016/j00@005lL000000EJV@<0021oo`007goo1@000goo
00<007ooOol04Goo0P00;7oo0P0025=O0P001FMl0P001F^L00@006/jJcX000E[>P03001JVEZI00IJ
V@03001oo`0001moo`00:7oo00<007ooOol03Goo0`00:goo0`000U:k00<005=ODel015=O00<006Ml
Ig`016Ml00@006mJKeX000=[W005001[>V/jJcX00004JcX20008FYT00`00Ool0000OOol002Eoo`04
001oogoo000=Ool2000[Ool30006D[/01000DemCGe=O0P001VMl00<006mJKeX00VmJ00<006^L0000
1F/j00@006/jJcX0009Vf003001JVEZI00IJV@03001oo`0001ioo`009Woo0P002goo0`00:goo0P00
2U:k00<005=O00001fMl00<006mJKeX016mJ00<006/jJcX016/j00<006/j00000fKH00<005ZIFYT0
1UZI00@007ooOol001eoo`00<Goo0P0000=oo`00Ool0:7oo0`0000=BF@00D[/02E:k0`001VMl00<0
06mJKeX01FmJ00<006/jJcX016/j0P0016KH00<005ZIFYT01UZI0P0000=oo`00Ool077oo000^Ool3
000ZOol30005DUT00`00D[]B^`06D[/01@00H_YRnV;j00001FMl00<006mJKeX016mJ00<006lh0000
1F/j00<006KHI]P016KH00<005ZIFYT00eZI0P0000=FF000Ool00P0077oo000[Ool3000[Ool20008
DUT00`00D[]B^`05D[/00`00H_YRnP03H_X01@00IgaWO6Ml00001VmJ00D006lhKcQ_>00000=[>P80
00IVf004001JVEZIFYT20004EUP01000Oomoo`001Goo0P0057oo000YOol20002Ool00`00Oomoo`0V
Ool300000dhg001BF@08DUT00`00D[]B^`03D[/00`00H_YRnP05H_X00`00Ig`00006KeX00`00KcQ_
>003KcP01@00JcX006KH00001VKH00@005ZI000000IFF004001oogoo0004Ool01000Oomoo`004goo
000VOol3000ZOol30005CSL00`00DUUBF@06DUT00`00D[]B^`02D[/00`00H_YRnP07H_X00`00KeY_
FP04KeX00`00KcQ_>005KcP01000I]QVf0001fKH00<005IHEUP01UIH00D007ooOomoo`0000=oo`04
001oogoo000COol002Aoo`8002]oo`8000Q>=`03001BFE9I00MBF@04001B^e:k000:H_X20005KeX0
0`00KcQ_>005KcP01000I]QVf0001VKH0P002EIH00D007ooOomoo`00009oo`04001oogoo000COol0
025oo`<002Yoo`<00003CQL004hg00M>=`03001BFE9I00MBF@03001B^`0000YRnP04001_66lH0003
KeX00`00KcQ_>005KcP01@00KaP006KH00001FKH00@005jHGYP000QFF003001oogoo0080009oo`04
001oogoo000COol001moo`8002Yoo`<000A>5`03001>=dhg00M>=`03001BFE9I00IBF@8000YRnP03
001_66lH009_6003001_FP0000M_>003001_66lH00<000AVf005001NV5jHGYP00008EUP00`00Oomo
o`02Ool00`00Oomoo`02000DOol001aoo`<002Yoo`8000Q>5`03001>=dhg00I>=`03001BFE9I00EB
F@03001NV00000URnP03001_66lH00A_6003001_>6lh00E_>003001_600000@000=Vf003001NV5jH
009NV003001FF5IH00MFF003001oogoo009oo`03001oogoo01Eoo`006goo0P00:Goo0`0000=>=`00
CQL01dhG00<004hgCSL01Thg00<0059IDUT0159I00D005jHGYQNV00000MRnP03001_66lH00E_6003
001_>6lh00A_>0<000A_60800003I]P005jH00ENV003001FF5IH00IFF003001oogoo009oo`03001o
ogoo01Eoo`0057oo0`0017oo00<007oo00009Woo0P001Dhg00<004hGCQL01DhG00<004hgCSL01dhg
00@0059IDUUBF@8000ENV003001RnV;j00=RnP03001_66lH00E_6003001c>000009_>0@000I_6003
001Vf00000MNV003001FF5IH00IFF003001oogoo00=oo`03001oogoo01Aoo`005Goo00<007ooOol0
0goo00@007ooOol002=oo`8000M>=`03001>5dhG00I>5`03001>=dhg00I>=`04001BFE9I0008GYP0
1@00H_YRnV;j00001flH00<007<hLcP00`000Vlh00<006lHKaP01VlH00<005jH00001UjH00<005IH
EUP01UIH0P0017oo0P005Goo000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo00007goo0`00
00=BF@00CSL01Thg00<004hGCQL01ThG00<004hgCSL01Thg00<0059I00002UjH00<006;j00001flH
10000W<h00<006lh000026lH0P0000=NV000GYP01UjH00<005IHEUP00eIH0P0000=B=`00Ool017oo
00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0IOol30005DUT0
0`00CSM>=`05CSL00`00CQM>5`05CQL00`00CSM>=`06CSL2000<GYP00`00KaQ_6004KaP30006LcP0
0`00KaQ_6006KaP01P00J_QZn000GYP000INV004001FF5IHEUP20003DSL00`00Oomoo`04Ool00`00
Oomoo`0BOol001Eoo`03001oogoo009oo`8000Eoo`03001oogoo01Eoo`<000QBF@03001>=dhg00E>
=`03001>5dhG00I>5`03001>=dhg00A>=`03001FN00000aNV004001_66lHKaP300000flH001c>006
LcP00`00KaQ_6005KaP01@00J_QZn6[h00000UjH00<005jHGYP00ejH00@005IH000000IB=`03001o
ogoo00=oo`03001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo00<007ooOol04Goo0`000U:k
00<0059IDUT01U9I00<004hgCSL01Thg00<004hGCQL01DhG00<004hgCSL00Thg0P000eIh00<005jH
GYP025jH00<006Kh00000P000flH00<007<hLcP01G<h00<007=H00001FlH00<006[hJ_P00f[h00<0
05jH00001EjH0P00258g00<007ooOol017oo00<007ooOol04Goo000JOol00`00Oomoo`06Ool00`00
Oomoo`0=Ool30006D[/00`00DUUBF@06DUT00`00CSM>=`05CSL00`00CQM>5`05CQL00`00CSM>=`02
0005EWP00`00GYQNV007GYP50004KaP00`00LcQc>004LcP01000LeQcF00016lH00<006[hJ_P016[h
00@005jHGYP0009NV0800003FWP0058g00MB=`03001oogoo00Eoo`03001oogoo011oo`006Woo00<0
07ooOol01goo00<007ooOol02Goo0`0000=CG`00D[/01U:k00<0059IDUT01U9I00<004hgCSL01Dhg
00<004hGCQL01ThG0P0025Ih00<005jHGYP015jH0`000fKh00D006lHKaQ_600000Ic>003001cF7=H
009cF004001_66lH0008J_P00`00GYQNV0020004FWP00`00DSMB=`06DSL00`00Oomoo`03Ool2000B
Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000Yoo`03001oogoo00Ioo`8000ECG`03001B^e:k00EB
^`03001BFE9I00EBF@03001>=dhg00I>=`03001>5dhG00A>5`8000YFN003001NV5jH00@000IVn003
001_600000Mc>003001cF7=H009cF003001_600000UZn003001NV000008000AJN003001B=e8g00IB
=`03001oogoo00Eoo`03001oogoo00moo`001goo00@007ooOol000=oo`8000Ioo`03001oogoo0080
00]oo`03001oogoo00=oo`<000MCG`03001B^e:k00AB^`03001BFE9I00IBF@03001>=dhg00E>=`04
001>5dhGCQL300000e9H001FN009EWP30002GYP00`00I_QVn006I_P00`00LcQc>005LcP00`00LeQc
F004LeP00`00J_QZn008J_P20002FWP00`00FWQJN002FWP00`00DSMB=`06DSL00`00Oomoo`06Ool0
0`00Oomoo`0>Ool00003Ool0000000<00005Ool007ooOol0000;Ool01000Oomoo`003Goo00<007oo
00000P000cnM0P001U=O00<005:kD[/015:k00<0059IDUT01E9I00<004hgCSL01Dhg00@004hG0000
00EBF003001FN5Ih00AFN0<00005EWP005jHGYP00009I_P00`00LcQc>004LcP00`00LeQcF005LeP0
0`00J_QZn007J_P00`00H]X00003FWP00`00FWQJN002FWP00`00DSMB=`06DSL00`00Oomoo`05Ool0
0`00Oomoo`0>Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`0;Ool900000cnM001CG`04
Del00`00D[]B^`04D[/00`00DUUBF@05DUT00`00CSM>=`04CSL30007DUP01000EWQFN5Ih0`0015Ih
00<005jH00002FKh00<006lh00001G<h00<007=HLeP017=H00<007=i00001f[h00@006;JH]X000AJ
N005001JN5YhFWP00008DSL00`00Oomoo`06Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool0
0`00Oomoo`03Ool00`00Oomoo`07Ool60004<el:00000e=O001B^`05D[/00`00DUUBF@05DUT00`00
CSL000030002BSP00`00DUQBF006DUP30007EWP20009I_P01@00KcQ_>6lh00000g<h00<007=HLeP0
17=H00D007=iLgUcN@0000AZn08000ARfP05001JN5YhFWP00003FWP00`00DSMB=`06DSL00`00Oomo
o`07Ool00`00Oomoo`02Ool40006Ool000Qoo`8000Uoo`@0009oo`03001oogoo00Aoo`@000Aoo`D0
00=6gP03000=o00000<000036Wh0000000D00003D[/0059I00=BF@D000M:>003001BF59H00=BF0<0
00QFN003001R^00000QVn003001_>6lh00=_>003001c>00000IcF003001cNG=i009cN@05001Zn6[h
J_P00006H]X00`00FWQJN002FWP01@00FWQJN5Yh0000258g00<007ooOol01Woo00<007ooOol00goo
00<007ooOol01Woo000IOol01000Oomoogoo10002Woo00<004KNA]h00dKN00<000gl3O`00`000QYn
00<003IM=Ud00SIM1P000d8j00<004XhBSP01TXh00<0059HDUP00`000e9H00<005IhEWP015Ih00D0
06:hH[QR^00000IVn003001_>6lh00E_>003001cF7=H00AcF003001cNG=i00AcN@03001Zn00000MR
fP03001JN5Yh00=JN004001JN5Yh0007DSL20009Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qo
o`800004Ool00000000>Ool00`00A]i6gP03A]h010003O`=o0gl0`0000<JOP00=Ud013IM00<0048j
@SX01D8j00<004XhBSP01DXh0`001U9H00<005IhEWP00eIh00<006:hH[P00V:h00<006KhI_P016Kh
00<006lhKcP01Flh0P001G=H00<007=iLgT017=i0P002F;J00<005YhFWP00eYh00<005Yh0000158g
0`0000=>=`00Ool02Goo00<007ooOol00goo00<007ooOol017oo000HOol3000BOol00`00A]i6gP03
A]h010003O`=o0gl1@001CIM00<0048j@SX0148j00<004XhBSP00TXh0`0000=:>000DUP01U9H00<0
05IhEWP00UIh00<006:hH[P016:h00<006KhI_P00VKh00<006lhKcP01Flh00@007NIMiT000=cF003
001cNG=i00AcN@03001[W00000URfP03001JN5Yh00=JN003001JN5Yh00@000A>=`03001oogoo00Mo
o`80009oo`04001oogoo0006Ool0019oo`8000Aoo`03001oogoo01=oo`T000033O`0000000<000@f
G@030012>T8j00E2>P@000A:>003001BF59H00IBF004001FN5Ih0008H[P01000I_QVn0001flh00<0
07NIMiT00WNI00<007=H00001W=i00D006^LJia[W00000QRfP03001JN5Yh00AJN0<000Q>=`03001o
ogoo00Qoo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo01=oo`03001:FTYJ
00=:FP03000YfbWK00<000040=H000000004=Ud00`00@SY2>P02@SX30007BSP00`00DUQBF006DUP0
0`00EWP00009H[P00`00I_P00007KcP00`00MiUgV@04MiT20005LgT00`00Jia[W003Ji`00`00H]YR
fP06H]X00`00FWP0000300000e9H00000008CSL00`00Oomoo`09Ool00`00Oomoo`08Ool0015oo`04
001oogoo0003Ool00`00Oomoo`0DOol00`00BUY:FP03BUX00`00:M/Yf`02:M/60002=Ud00`00@SX0
00020003@SX00`00BSQ:>005BSP00`00DUQBF005DUP2000;H[P00`00KcQ_>005KcP00`00MiUgV@04
MiT01000LoYcnP000g=i00<006^LJi`016^L00<006;JH]X016;J10001U9H0P001dhg00<007ooOol0
2Goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P005Woo00<004YJBUX00dYJ00<002WK:M/0
0RWK00<0004i00000P0000<5V`0000000P001T8j00<004XhBSP01DXh00<0059HDUP00e9H0P0000=F
V@00H[P02F:h0P001flh00<007NIMiT00gNI00<007?jLoX00W?j00<007=i000026^L00<006;JH]X0
0`000U:k00<0059HDUP01E9H0P0024hg00<007ooOol02Goo00<007ooOol01goo000AOol01000Oomo
o`000goo00<007ooOol05Goo00<004YJBUX00dYJ00<002WK:M/00RWK00<0004i00001P001T8j00<0
04XhBSP01DXh00@0059HDUQBF08000AFV@03001R^6:h00IR^004001[>V/j0005KcP00`00MiUgV@03
MiT00`00LoYcnP03LoX20009Ji`30006D[/00`00DUQBF005DUP00`00CSM>=`06CSL00`00Oomoo`09
Ool00`00Oomoo`07Ool0019oo`8000=oo`8001Moo`03001:FTYJ00A:FP03000YfbWK008Yf`D000<5
V`<000A2>P03001:>4Xh00E:>0<000IFV@03001R^6:h00ER^003001[>V/j009[>P05001_>6lhKcP0
0005MiT00`00LoYcnP03LoX01000Go]On`001F^L0`0000=CO`00D[/01U:k00<0059HDUP01U9H00<0
04hgCSL01Dhg00<007ooOol02Woo00<007ooOol01Woo000GOol00`00Oomoo`0GOol00`00BUY:FP03
BUX00`00:M/0000300030CT200021I/00`00;Q`^70050004BSP40009EYT00`00H[QR^003H[P00`00
JcY[>P04JcX00`00KcP00005MiT00`00LoYcnP03LoX00`00Go]On`02Go/00`00Ji`000020005Dgl0
0`00D[]B^`05D[/00`00DUQBF005DUP00`00CSM>=`06CSL00`00Oomoo`0:Ool00`00Oomoo`05Ool0
01Moo`03001oogoo01Moo`04001:FTYJBUX700000bWK0001>@020CT300000`FK000^7004;Q`50003
BW/00`00EYUFV@07EYT00`00H[QR^002H[P00`00JcY[>P06JcX20004MiT00`00LoYcnP02LoX20004
Go/40007Dgl00`00D[]B^`05D[/00`00DUQBF005DUP20007CSL00`00Oomoo`0:Ool00`00Oomoo`05
Ool001Moo`03001oogoo01Qoo`@0009:6004000inCWi>OT400020CT300000`FK000^7004;Q`00`00
BW]:N`05BW/00`00EYUFV@07EYT01000H[QR^0002F/j00H006oK000007NI0004LoX01000Go]Oneok
0`0012/i0P001U=o00<005:kD[/015:k00<0059HDUP01U9H0P001Thg00<007ooOol02goo00<007oo
Ool017oo0005Ool20004Ool20003Ool30004Ool2000GOol30006BQP00`00>OTin@02>OT00`009KX0
00070005;Q`00`00BW]:N`04BW/00`00EYUFV@07EYT00`00H[P00009JcX00`00Km]_f`02Km/30002
LoXA0003Dgl00`00D[]B^`05D[/00`00DUQBF005DUP00`00CSL00006CSL00`00Oomoo`09Ool30005
Ool000Aoo`04001oogoo0003Ool20006Ool01@00Oomoogoo00005Goo0`000goo00<004XHBQP00dXH
00<003Wi>OT00cWi00@002Fj9KXU^P<000030=D002hL00@^7003001:NdYk00E:N`03001FVEJI00IF
V@8000U[>P03001_ffoK009_f`04001KnE_iFoT40004A]d00`007SdN?@0700000b:N000000020007
D[/00`00DUQBF005DUP00`00CSL00002CSL50008Ool30002Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo000;Ool00`00Oomoo`02000COol20006Ool00`00BQQ:6004BQP00`00>OTin@02>OT00`009KXU
^P039KX200000`3E000^7004;Q`00`00BW]:N`04BW/00`00EYUFV@04EYT20002Gc`00`00JcY[>P05
JcX00`00Km]_f`02Km/00`00FoUKn@030002Ga/00`00A]e6g@03A]d00`007SdN?@027Sd500038YhE
0004CSP00`00Oomoo`03Ool30005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool0
0`00Oomoo`0?Ool30008Ool00`00BQQ:6004BQP00`00>OTin@03>OT00`009KXU^P029KX00`000GX0
00030003;Q`00`00BW]:N`05BW/01000EYUFVEJI0P001Ell00<006/jJcX00f/j00<006oKKm/00VoK
00<005_iFoT00P001ElK00<004KMA]d00dKM00<001hm7Sd00Qhm00@0005j0001N080008RWP03000n
O3il00DnO0030016>THj00E6>P04001>>4hh0005CSP50008Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo0008Ool00`00Oomoo`03Ool2000=Ool3000<Ool00`00BQQ:6003BQP00`00>OTin@03>OT00`00
9KXU^P039KX01@000GX000002M`00`0000<^7000BW/01DYk00@005JI000000QO?005001[>V/jJcX0
0004Km/01000FoT000001elK2P000Qhm00D0005j0000006J008000038Yh003il00HnO0030016>THj
00E6>P04001>>4hh0004CSP00`00Oomoo`0:Ool00`00Oomoo`03Ool000Eoo`8000Uoo`@0009oo`03
001oogoo00Uoo`<000moo`03001:64XH00A:6003000inCWi00<in@03000U^RFj00<U^P040001NP5j
00032M`20004BW/4000;Gc`00`00JcX00004Km/40006Ga/200000eZI001:FP04BUX00`00=QXf6P04
00020GX00`000IX1VP030006?W`00`00ASY6>P05ASX01000CSQ>>00014hh00<007ooOol02goo00<0
07ooOol00Woo000FOol00`00Oomoo`06Ool3000COol00`00BQQ:6003BQP00`00>OTin@03>OT00`00
9KXU^P039KX01@000GX1NP5j000010WL100014;N00<005llGc`025ll10000VoK0`000V/I00<005lK
Ga/00UlK00<005ZIFYT00UZI00<004YJBUX00dYJ00<003HJ=QX00SHJ0`0000<1NP000IX00P6J0P00
1Sil00<004HjASX014Hj00D004hhCSQ>>00000A>>003001oogoo00Yoo`03001oogoo009oo`005Woo
00<007ooOol00goo0`005Woo00<004XHBQP00dXH300000<U^P000GX00P5j00D000WL2M`9g00000Q2
gP03001O?5ll00EO?08000AKgP800003Ji/006/I00=[6@04001O6`000005FYT00`00BUY:FP04BUX0
0`00=QXf6P02=QX010006IX0005j0P000`6J0P0013il00<004HjASX01DHj00D004hhCSQ>>00000=>
>003001oogoo00Yoo`03001oogoo009oo`005Goo00@007ooOomoo`<001Uoo`03001:64XH00<00003
CQL004Gg00A5m`03000n6ChI008n6@P00089g0030012gT;N00I2gP03001O?5ll009O?08000AKgP80
009[V`03001[6F/I00A[6@03001JVEZI00IJV@03001:FTYJ00=:FP03000f6SHJ00<f6P03000IVP00
00<00081VP04000Mo0000002?W`00`00ASY6>P05ASX01@00CSQ>>4hh00000dhh00<007ooOol02Woo
00<007ooOol00Woo000EOol4000MOol20005CQL00`00AOM5m`03AOL00`00?QTn6@03?QT01000=U`f
G3IL1P0024;N00@005ll000000AKgP8000=[V`03001[6F/I00A[6@8000QJV@03001:FTYJ00A:FP03
000f6SHJ00<f6P03000IVQVJ00@000040IX001gl7O`200000cil0016>P06ASX01@00CSQ>>4hh0000
0dhh00<007ooOol02Goo00<007ooOol00Woo000>Ool30004Ool2000OOol00`00CQM>5`04CQL00`00
AOM5m`03AOL00`00?QTn6@04?QT00`00=U`fG004=U`30004@]h70002Fmh200001Fli000006^K0000
1V/I00@006KHI]P000MJV@03001:FTYJ009:FPH0008f6P03000IVQVJ008IVP030001>000008000<M
o08000M6>P03001>>4hh009>>004001>>4hh000;Ool00`00Oomoo`02Ool000moo`03001oogoo00Eo
o`03001oogoo01aoo`03001>5dhG00A>5`030015mdGg00A5m`03000n6ChI00@n6@03000fG3IL00Df
G0D000DNo@D000A_>@8000E[6@05001Vf6KHI]P00007FYT50003CSP00`00@OQ1n00400046IX20000
0`4h000Mo0047O`50002ASX00`00CSQ>>003CSP00`00CSP0000;Ool00`00Oomoo`02Ool000moo`03
001oogoo00Ioo`03001oogoo01aoo`03001>5dhG00A>5`030015mdGg00=5m`03000n6ChI00@n6@03
000fG3IL008fG0<000=B^`03001CO`0000@000=WO003001_>Fli00=_>@03001_600000=[6@03001V
f6KH00=Vf003001JVEZI009JV@800003EUL004hh00I>>0030011n47h0091n0<0008IVP040001F@00
0CP200037O`00`00>S/j>`02>S/<000;Ool00`00Oomoo`01Ool000moo`03001oogoo00Moo`03001o
ogoo01]oo`03001>5dhG00A>5`030015mdGg00=5m`03000n6ChI00Dn6@04000fG0000007D[/00`00
Dgl00005Ig`00`00KcU_>@03KcT01`00KaQ_66lH001[6@0000IVf005001JVEZIFYT00003EUL00`00
CSQ>>005CSP00`00@OQ1n003@OP00`00<MX0000300001@5I0000005j00000agl00<003Xk>S/013Xk
00<004YjBWX00dYj00<004Yj00002goo00<007ooOol00Goo000?Ool00`00Oomoo`08Ool2000KOol0
0`00CQM>5`04CQL00`00AOM5m`04AOL00`00?QTn6@03?QT30009D[/30004Ig`00`00KcU_>@03KcT0
0`00KaQ_6003KaP00`00I]QVf005I]P01000FYT000001EIG00<004hhCSP014hh00<0047h@OP0147h
00@0037J<MXafP800081F@060001NP00000Mo0001cXk00<004YjBWX00dYj00<004Yj00002Woo00<0
07ooOol00Goo000>Ool2000<Ool00`00Oomoo`0IOol00`00CQM>5`04CQL00`00AOM5m`03AOL00`00
?QTn6@030002DUP00`00D[]B^`06D[/01000H_URnF;i0P0000=WO000KcT016li00<006lHKaP016lH
00<006KHI]P01FKH0P001eIG00<004hhCSP01Dhh00<0047h@OP00d7h00<0037J<MX00c7J0P0000@1
F@000GX1NP<000Hj>`03001:NTYj00=:NP03001:NP0000Yoo`03001oogoo005oo`007Goo00<007oo
Ool067oo00<004hGCQL014hG00<004GgAOL00dGg0`001U9H00<005:kD[/00e:k0P001V;i00<006li
KcT00fli00<006lHKaP016lH00<006[h00001FKH00<005jH00001eIG00<004hhCSP01Dhh00<0047h
@OP0147h00<0037J<MX00S7J00<001gK00000`000P5j00<003Xk>S/013Xk00<004YjBWX014Yj00<0
04Yj00002Goo00<007ooOol00Goo000NOol00`00Oomoo`0GOol00`00CQM>5`04CQL00`00AOM5m`03
00000dhg001BF007DUP01@00D[]B^e:k000026;i0P000fli00<006lHKaP016lH00@006[hJ_P000AV
f005001NV5jHGYP00007EUL00`00CSQ>>004CSP00`00@OQ1n004@OP00`00<MXafP03<MX00`007M/M
f`0300020GX20005>S/00`00BWY:NP04BWX20009Ool00`00Oomoo`01Ool001moo`8001Moo`03001>
5dhG00=>5`@000E>=`03001BF59H00IBF003001B^`0000QRn@06001Zn6[h001_>@001VlH00<006[h
J_P00V[h00@006KHI]P000ANV003001FEeIG00EFE`03001>>4hh00E>>0030011n47h00=1n003000a
fS7J00@afP03000MfagK00D000047Qd000000002>S/00`00BWY:NP05BWX20006Ool30003Ool0025o
o`03001oogoo01Eoo`03001>5`000080009>5`03001>=dhg00A>=`03001BF59H00IBF08000QRn@03
001Zn6[h009Zn003001_66lH00A_6003001Zn6[h00=Zn003001Vf00000INV003001FEeIG00AFE`03
001>>4hh00=>>0h00003<MX001gK00<Mf`@000<N7@<000M:NP8000=oo`<0008T0080009oo`008Woo
00<007ooOol057oo0P001ThG00<004hgCSL014hg00<0059HDUP0159H00<005jH00001f;i00<006[h
J_P00f[h0P001FlH00<006[hJ_P016[h00<005jHGYP01EjH00<005IGEUL01EIG00<004hh00000P00
0ThG00<004XGBQL014XG00<004HIAQT00THI1@000QgK10001AhM30001B@00P000Woo000SOol00`00
Oomoo`0COol00`00CQM>5`05CQL00`00CSM>=`04CSL00`00DUQBF002DUP20003GYP00`00H_URn@03
H_T00`00J_QZn003J_P01000KaQ_60000flH00<006[hJ_P01F[h00<005jHGYP01UjH00<005IGEUL0
0eIG0`001DhG00<004XGBQL01DXG00<004HIAQT00dHI00<003iL?U`0100000@1^P00000000<N7@03
0012gT;N00I2gP8000DT0003001oo`00009oo`0097oo00<007ooOol04goo00<004hGCQL014hG00<0
04hgCSL01Dhg00@0059HDUP000INV005001RnF;iH_T00005J_P00`00KaQ_6002KaP00`00KaP00007
J_P00`00I_P00007GYP00`00EUMFE`0300000e8g001>5`06CQL00`00BQM:5`04BQL00`00AQU66@04
AQT00`00?U`nG002?U`800000ahM0012gP07@]h00`009000000400000goo001oo`01Ool002Eoo`80
01=oo`03001>5dhG00E>5`03001>=dhg00A>=`03001BF00000QNV003001Rn@0000IZn003001_66lH
00=_6003001Zn6[h00EZn004001Vn6Kh0007GYP30005DSL00`00CQM>5`04CQL00`00BQM:5`04BQL0
0`00AQU66@04AQT00`00?U`nG005?U`00`007[hN_P050007@]h2000370001000Oomoo`000Woo000W
Ool00`00Oomoo`0@Ool00`00CQM>5`05CQL00`00CSM>=`04CSL2000:GYP00`00J_QZn004J_P00`00
KaQ_6004KaP20005J_P00`00I_QVn002I_P00`00GYQNV002GYP200000eYh001B=`05DSL00`00CQM>
5`04CQL00`00BQM:5`05BQL00`00AQU66@04AQT00`00?U`nG003?U`40003Ool>0003Ool10001Ool1
Ool002Qoo`03001oogoo011oo`03001>5dhG00E>5`03001>=dhg009>=`03001FM`0000YNV003001Z
n6[h00AZn003001_66lH00=_6004001c>7<h0003J_P00`00I_QVn003I_P01@00GYQNV5jH00000eYh
00<0058gDSL0158g00<004hGCQL01DhG00<004XGBQL014XG00<004HIAQT014HI00<003iL?U`00`00
2goo1@000Woo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0?Ool00`00CQM>5`05CQL0
0`00CSM>=`020003EWL00`00GYQNV006GYP00`00I]P00004J_P00`00KaQ_6003KaP00`00LcQc>002
LcP00`00J_P00006I_P01000GYP000001EYh00<0058gDSL0158g00<004hGCQL014hG00<004XGBQL0
14XG00<004HIAQT00dHI10005Goo00<007ooOol017oo0@000Goo000ZOol2000?Ool00`00CQM>5`05
CQL00`00CSL00005EWL00`00GYQNV005GYP01@00I]QVf6KH00000V[h00<006lHKaP00flH00<007<h
LcP017<h00<006KhI_P01FKh0P001eYh00<0058gDSL0158g00<004hGCQL014hG00<004XGBQL01DXG
00<004HI00000P006Woo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0=Ool00`00CQM>5`04
CQL20007EWL00`00GYQNV003GYP00`00I]QVf003I]P00`00J_P00004KaP00`00LcQc>005LcP00`00
I_QVn004I_P00`00H]T00007FWP00`00DSMB=`05DSL00`00CQM>5`03CQL00`00BQM:5`03BQL4000N
Ool00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00aoo`04001>5dhGCQL300000e9G001FM`06
EWL00`00GYQNV002GYP00`00I]QVf005I]P00`00KaQ_6003KaP00`00LcQc>004LcP00`00KeT00004
I_P01@00H]URfF;I00001eYh00<0058gDSL0158g00<004hGCQL014hG00<004XG00000P008goo00D0
07ooOomoo`00005oo`00;Woo00<007ooOol02goo00@004hG000000ABE`03001FMeIg00IFM`04001N
V5jH0008I]P20003KaP00`00LcQc>004LcP01@00KeU_FFmI00000VKh00<006;IH]T00V;I00<005Yh
FWP01EYh00<0058gDSL0158g00@004hGCQM>5`D002Ioo`05001oogooOol00001Ool002moo`8000ao
o`03001BEe9G00EBE`03001FMeIg00EFM`03001NV00000QVf006001_66lH001_60001W<h00<006mI
KeT00VmI00<006Kh00001V;I00<005YhFWP015Yh00<0058gDSL00e8g1@00;7oo00@007ooOol0005o
o`00<Goo00<007ooOol02Goo00<0059GDUL01E9G00<005IgEWL01EIg0P0026KH00<006lHKaP00VlH
0P001G<h00<006mIKeT00fmI0P001f;I00<005YhFWP01EYh00<0058g00000P00<Woo00<007oo0000
0Goo000bOol00`00Oomoo`09Ool00`00DUMBE`04DUL00`00EWMFM`04EWL00`00H[P00007I]P00`00
KaQ_6003KaP00`00LgT00003LcP00`00KeU_F@03KeT00`00Jg`00008H]T00`00FWQJN003FWP3000e
Ool00`00Oomoo`01000003=oo`03001oogoo00Qoo`03001BEe9G00EBE`03001FMeIg009FM`05001R
^6:hH[P00005I]P00`00KaQ_6003KaP01`00LgUcNG=i001c>00000E_F@05001[O6]lJg`00007H]T0
0`00FWQJN003000iOol10001Ool1000003Aoo`8000Qoo`03001BEe9G00EBE`05001FMeIgEWL00005
H[P01@00I]QVf6KH00001FlH00<007=iLgT00g=i0P0016mI00<006]lJg`00f]l00<006;IH]T00f;I
1000?Goo0P00000fOol00`00Oomoo`06Ool00`00DUMBE`04DUL01000EWMFM`001f:h00<006KH0000
1VlH00<007=iLgT00g=i00H007?J000006mI0006Jg`00`00H]URf@030010Ool3000003Moo`03001o
ogoo00Eoo`03001BEe9G00EBE`8000UR^003001_66lH00A_6003001cNG=i00=cN@04001cfW?JLmX3
0006Jg`3000oOol40003Ool003Qoo`03001oogoo00Eoo`03001BEe9G00=BE`8000UR^003001[6@00
00A_6003001cNG=i00=cN@05001cfW?JLmX00003EoX00`00Jga[O003000nOol40007Ool003Uoo`03
001oogoo00Aoo`04001BEe9GDUL200000eZI001R^007H[P01@00JaU[6F/I00000flH00<007=iLgT0
0W=i00<007?JLmX00P000UOj1@00?Woo0`002goo000jOol20004Ool01000DUL0000015ZI00<006:h
H[P016:h00<006/IJaT00f/I00<006lH000017=i00<007?JLmX01@00?goo10003Woo000lOol00`00
Oomoo`02Ool00`00FYUJV@05FYT00`00H[QR^002H[P00`00JaU[6@05JaT30002LgT00`00LmYcfP02
000oOol4000BOol003eoo`05001oogooOol00007FYT01@00H[QR^6:h00001f/I00@006nkKk]_^`D0
03eoo`@001Ioo`00?Woo00D007ooOomoo`0000MJV@03001R^00000M[6@06001_^fnkKk/005OG0P00
>goo10006Woo000oOol200000goo001JV@06FYT20007JaT01000Kk]_^fnk0`00>Goo10007Woo0011
Ool00`00Ool00005FYT01000GaaO70001F/I00@006nkKk]_^`8003Qoo`<0029oo`00@Woo00<007oo
00000UZI0P0015lL00D006/IJaU[6@0000=_^`03001oogoo03Aoo`@002Eoo`00@goo0P0000=JV@00
Ga`01UlL00<006/I00000fnk00<007ooOol0<Goo1000:Goo0014Ool30007Ga`7000_Ool4000]Ool0
04Ioo`8000AO7080009Gg@80009oo`03001oogoo02Uoo`@0035oo`00Agoo0P0000=O700000000UOM
0P0017oo00<007ooOol09Goo1000=Goo0018Ool60006Ool00`00Oomoo`0QOol4000iOol004Uoo`80
00Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007ooOol01Woo00<007ooOol06Woo1000@7oo001<
Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool004eoo`03001oogoo00Aoo`03001oogoo019o
o`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo001?Ool20003Ool00`00Oomo
o`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03001oo`0000Eoo`@005Moo`00
Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo002e
Ool00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128621, 0.0056064, 
  0.00599011, 0.00599011}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.183 0 0 r
.37846 .69526 .40273 .67205 .44005 .66841 .41684 .68118 Metetra
.41684 .68118 .44005 .66841 .47798 .66923 .45574 .66493 Metetra
.669 .926 .973 r
.45574 .66493 .47798 .66923 .51671 .67229 .49509 .64751 Metetra
.703 .729 .87 r
.49509 .64751 .51671 .67229 .55631 .67485 .53487 .63015 Metetra
.668 .611 .783 r
.53487 .63015 .55631 .67485 .59673 .67414 .57512 .61402 Metetra
.643 .556 .742 r
.57512 .61402 .59673 .67414 .63782 .66795 .61592 .60002 Metetra
.627 .533 .728 r
.61592 .60002 .63782 .66795 .67935 .65515 .65737 .5886 Metetra
.617 .531 .734 r
.65737 .5886 .67935 .65515 .72111 .63592 .6996 .57967 Metetra
.613 .553 .763 r
.6996 .57967 .72111 .63592 .76299 .61176 .74267 .57263 Metetra
.61 .616 .832 r
.74267 .57263 .76299 .61176 .80501 .58513 .78662 .56646 Metetra
.574 .763 .968 r
.78662 .56646 .80501 .58513 .84736 .5589 .83138 .55992 Metetra
.192 .723 .772 r
.83138 .55992 .84736 .5589 .89036 .5358 .87684 .55177 Metetra
.201 0 0 r
.87684 .55177 .89036 .5358 .9344 .51788 .92286 .54104 Metetra
.272 0 0 r
.92286 .54104 .9344 .51788 .97986 .50616 .96928 .52724 Metetra
.35305 .71698 .37846 .69526 .41684 .68118 .39263 .6925 Metetra
.7 .979 .793 r
.39263 .6925 .41684 .68118 .45574 .66493 .43271 .65948 Metetra
.852 .907 .872 r
.43271 .65948 .45574 .66493 .49509 .64751 .47294 .62237 Metetra
.795 .747 .809 r
.47294 .62237 .49509 .64751 .53487 .63015 .51315 .58628 Metetra
.728 .643 .764 r
.51315 .58628 .53487 .63015 .57512 .61402 .55344 .55592 Metetra
.671 .577 .741 r
.55344 .55592 .57512 .61402 .61592 .60002 .59407 .53478 Metetra
.621 .535 .735 r
.59407 .53478 .61592 .60002 .65737 .5886 .63546 .52456 Metetra
.573 .51 .747 r
.63546 .52456 .65737 .5886 .6996 .57967 .67801 .52498 Metetra
.517 .505 .781 r
.67801 .52498 .6996 .57967 .74267 .57263 .72204 .53377 Metetra
.434 .528 .854 r
.72204 .53377 .74267 .57263 .78662 .56646 .76763 .54696 Metetra
.239 .582 .945 r
.76763 .54696 .78662 .56646 .83138 .55992 .81457 .55949 Metetra
.156 0 0 r
.81457 .55949 .83138 .55992 .87684 .55177 .86235 .56612 Metetra
.281 0 0 r
.86235 .56612 .87684 .55177 .92286 .54104 .9103 .56259 Metetra
.207 0 0 r
.9103 .56259 .92286 .54104 .96928 .52724 .95772 .54667 Metetra
.706 .983 .811 r
.32695 .7284 .35305 .71698 .39263 .6925 .36769 .69573 Metetra
.891 .955 .837 r
.36769 .69573 .39263 .6925 .43271 .65948 .40898 .64972 Metetra
.885 .847 .801 r
.40898 .64972 .43271 .65948 .47294 .62237 .4502 .59763 Metetra
.836 .755 .774 r
.4502 .59763 .47294 .62237 .51315 .58628 .49107 .54756 Metetra
.774 .681 .761 r
.49107 .54756 .51315 .58628 .55344 .55592 .53164 .50674 Metetra
.703 .618 .759 r
.53164 .50674 .55344 .55592 .59407 .53478 .57234 .48035 Metetra
.622 .562 .766 r
.57234 .48035 .59407 .53478 .63546 .52456 .61376 .4709 Metetra
.521 .51 .784 r
.61376 .4709 .63546 .52456 .67801 .52498 .65656 .47797 Metetra
.389 .461 .814 r
.65656 .47797 .67801 .52498 .72204 .53377 .70124 .49825 Metetra
.202 .415 .847 r
.70124 .49825 .72204 .53377 .76763 .54696 .74798 .52578 Metetra
0 .373 .83 r
.74798 .52578 .76763 .54696 .81457 .55949 .79651 .55273 Metetra
0 .35 .671 r
.79651 .55273 .81457 .55949 .86235 .56612 .84609 .57073 Metetra
.178 0 0 r
.84609 .57073 .86235 .56612 .9103 .56259 .89561 .57277 Metetra
.89561 .57277 .9103 .56259 .95772 .54667 .94397 .55507 Metetra
.842 .895 .876 r
.30078 .72219 .32695 .7284 .36769 .69573 .3424 .68544 Metetra
.88 .841 .803 r
.3424 .68544 .36769 .69573 .40898 .64972 .38468 .63312 Metetra
.876 .798 .77 r
.38468 .63312 .40898 .64972 .4502 .59763 .42684 .57387 Metetra
.853 .763 .763 r
.42684 .57387 .4502 .59763 .49107 .54756 .46844 .51717 Metetra
.811 .729 .774 r
.46844 .51717 .49107 .54756 .53164 .50674 .50953 .47139 Metetra
.742 .689 .801 r
.50953 .47139 .53164 .50674 .57234 .48035 .55055 .44243 Metetra
.627 .633 .837 r
.55055 .44243 .57234 .48035 .61376 .4709 .59219 .43313 Metetra
.445 .544 .863 r
.59219 .43313 .61376 .4709 .65656 .47797 .63523 .443 Metetra
.202 .422 .852 r
.63523 .443 .65656 .47797 .70124 .49825 .68028 .46831 Metetra
0 .312 .803 r
.68028 .46831 .70124 .49825 .74798 .52578 .72761 .50225 Metetra
0 .287 .774 r
.72761 .50225 .74798 .52578 .79651 .55273 .77697 .53575 Metetra
0 .405 .828 r
.77697 .53575 .79651 .55273 .84609 .57073 .82754 .55892 Metetra
.227 .677 .952 r
.82754 .55892 .84609 .57073 .89561 .57277 .87805 .56339 Metetra
.622 .914 .98 r
.87805 .56339 .89561 .57277 .94397 .55507 .9272 .54462 Metetra
.783 .737 .81 r
.27515 .69458 .30078 .72219 .3424 .68544 .31718 .65888 Metetra
.827 .747 .775 r
.31718 .65888 .3424 .68544 .38468 .63312 .35999 .60842 Metetra
.848 .758 .763 r
.35999 .60842 .38468 .63312 .42684 .57387 .40276 .55134 Metetra
.854 .774 .773 r
.40276 .55134 .42684 .57387 .46844 .51717 .44503 .49661 Metetra
.838 .794 .809 r
.44503 .49661 .46844 .51717 .50953 .47139 .48677 .45218 Metetra
.779 .811 .878 r
.48677 .45218 .50953 .47139 .55055 .44243 .52836 .42367 Metetra
.612 .782 .963 r
.52836 .42367 .55055 .44243 .59219 .43313 .57047 .4138 Metetra
.275 .619 .959 r
.57047 .4138 .59219 .43313 .63523 .443 .61384 .42212 Metetra
0 .393 .838 r
.61384 .42212 .63523 .443 .68028 .46831 .65908 .44502 Metetra
0 .294 .776 r
.65908 .44502 .68028 .46831 .72761 .50225 .70651 .47604 Metetra
0 .346 .821 r
.70651 .47604 .72761 .50225 .77697 .53575 .75593 .50655 Metetra
.212 .492 .9 r
.75593 .50655 .77697 .53575 .82754 .55892 .80663 .52723 Metetra
.486 .635 .92 r
.80663 .52723 .82754 .55892 .87805 .56339 .85745 .53021 Metetra
.681 .722 .881 r
.85745 .53021 .87805 .56339 .9272 .54462 .90715 .51122 Metetra
.717 .635 .765 r
.25044 .64651 .27515 .69458 .31718 .65888 .29231 .61671 Metetra
.764 .673 .761 r
.29231 .61671 .31718 .65888 .35999 .60842 .33502 .5759 Metetra
.803 .722 .775 r
.33502 .5759 .35999 .60842 .40276 .55134 .37791 .52995 Metetra
.833 .789 .81 r
.37791 .52995 .40276 .55134 .44503 .49661 .42058 .48548 Metetra
.837 .88 .874 r
.42058 .48548 .44503 .49661 .48677 .45218 .46298 .44852 Metetra
.732 .956 .945 r
.46298 .44852 .48677 .45218 .52836 .42367 .50535 .42341 Metetra
.367 .834 .883 r
.50535 .42341 .52836 .42367 .57047 .4138 .54818 .41229 Metetra
0 .549 .715 r
.54818 .41229 .57047 .4138 .61384 .42212 .59206 .41479 Metetra
0 .41 .709 r
.59206 .41479 .61384 .42212 .65908 .44502 .63747 .42809 Metetra
0 .436 .841 r
.63747 .42809 .65908 .44502 .70651 .47604 .68468 .44719 Metetra
.221 .505 .906 r
.68468 .44719 .70651 .47604 .75593 .50655 .7336 .46561 Metetra
.433 .545 .87 r
.7336 .46561 .75593 .50655 .80663 .52723 .78371 .47654 Metetra
.564 .573 .821 r
.78371 .47654 .80663 .52723 .85745 .53021 .83422 .47431 Metetra
.651 .603 .788 r
.83422 .47431 .85745 .53021 .90715 .51122 .88425 .45593 Metetra
.659 .568 .742 r
.22664 .58345 .25044 .64651 .29231 .61671 .26784 .56294 Metetra
.691 .609 .759 r
.26784 .56294 .29231 .61671 .33502 .5759 .30981 .53736 Metetra
.729 .68 .803 r
.30981 .53736 .33502 .5759 .37791 .52995 .35224 .50924 Metetra
.767 .802 .881 r
.35224 .50924 .37791 .52995 .42058 .48548 .39493 .48152 Metetra
.724 .951 .951 r
.39493 .48152 .42058 .48548 .46298 .44852 .43785 .45698 Metetra
.368 .826 .734 r
.43785 .45698 .46298 .44852 .50535 .42341 .48109 .43771 Metetra
.48109 .43771 .50535 .42341 .54818 .41229 .52489 .42476 Metetra
.116 0 0 r
.52489 .42476 .54818 .41229 .59206 .41479 .56952 .41793 Metetra
.025 0 0 r
.56952 .41793 .59206 .41479 .63747 .42809 .61523 .41585 Metetra
.306 .729 .973 r
.61523 .41585 .63747 .42809 .68468 .44719 .66216 .41612 Metetra
.509 .656 .926 r
.66216 .41612 .68468 .44719 .7336 .46561 .7103 .41578 Metetra
.571 .581 .824 r
.7103 .41578 .7336 .46561 .78371 .47654 .75945 .41182 Metetra
.603 .55 .768 r
.75945 .41182 .78371 .47654 .83422 .47431 .80928 .40186 Metetra
.629 .548 .744 r
.80928 .40186 .83422 .47431 .88425 .45593 .85941 .38464 Metetra
.605 .524 .736 r
.20333 .51379 .22664 .58345 .26784 .56294 .24351 .50379 Metetra
.599 .548 .769 r
.24351 .50379 .26784 .56294 .30981 .53736 .28425 .4957 Metetra
.598 .618 .843 r
.28425 .4957 .30981 .53736 .35224 .50924 .32571 .48848 Metetra
.571 .764 .969 r
.32571 .48848 .35224 .50924 .39493 .48152 .36802 .48085 Metetra
.326 .809 .882 r
.36802 .48085 .39493 .48152 .43785 .45698 .41123 .47155 Metetra
.002 0 0 r
.41123 .47155 .43785 .45698 .48109 .43771 .45534 .45958 Metetra
.079 0 0 r
.45534 .45958 .48109 .43771 .52489 .42476 .50025 .44444 Metetra
.50025 .44444 .52489 .42476 .56952 .41793 .54588 .4262 Metetra
.35 .813 .715 r
.54588 .4262 .56952 .41793 .61523 .41585 .59214 .40554 Metetra
.686 .918 .972 r
.59214 .40554 .61523 .41585 .66216 .41612 .63897 .38354 Metetra
.702 .735 .877 r
.63897 .38354 .66216 .41612 .7103 .41578 .68642 .36153 Metetra
.663 .614 .79 r
.68642 .36153 .7103 .41578 .75945 .41182 .73459 .34077 Metetra
.634 .553 .746 r
.73459 .34077 .75945 .41182 .80928 .40186 .78365 .32221 Metetra
.617 .527 .73 r
.78365 .32221 .80928 .40186 .85941 .38464 .83378 .30634 Metetra
.547 .497 .751 r
.1797 .44669 .20333 .51379 .24351 .50379 .21879 .44621 Metetra
.471 .484 .79 r
.21879 .44621 .24351 .50379 .28425 .4957 .25812 .45425 Metetra
.355 .502 .867 r
.25812 .45425 .28425 .4957 .32571 .48848 .29834 .46677 Metetra
.132 .54 .92 r
.29834 .46677 .32571 .48848 .36802 .48085 .33996 .47861 Metetra
.129 0 0 r
.33996 .47861 .36802 .48085 .41123 .47155 .38322 .48439 Metetra
.144 0 0 r
.38322 .48439 .41123 .47155 .45534 .45958 .42807 .47975 Metetra
.42807 .47975 .45534 .45958 .50025 .44444 .47415 .46239 Metetra
.47415 .46239 .50025 .44444 .54588 .4262 .52097 .4327 Metetra
.733 .99 .822 r
.52097 .4327 .54588 .4262 .59214 .40554 .56805 .39371 Metetra
.854 .92 .874 r
.56805 .39371 .59214 .40554 .63897 .38354 .61512 .35029 Metetra
.802 .765 .819 r
.61512 .35029 .63897 .38354 .68642 .36153 .66224 .30797 Metetra
.73 .652 .771 r
.66224 .30797 .68642 .36153 .73459 .34077 .70975 .27179 Metetra
.666 .577 .745 r
.70975 .27179 .73459 .34077 .78365 .32221 .75823 .24544 Metetra
.611 .529 .738 r
.75823 .24544 .78365 .32221 .83378 .30634 .80836 .2307 Metetra
.477 .491 .793 r
.15476 .39017 .1797 .44669 .21879 .44621 .19298 .39641 Metetra
.274 .406 .815 r
.19298 .39641 .21879 .44621 .25812 .45425 .2311 .41613 Metetra
0 .303 .798 r
.2311 .41613 .25812 .45425 .29834 .46677 .27016 .44326 Metetra
0 .228 .691 r
.27016 .44326 .29834 .46677 .33996 .47861 .31102 .46981 Metetra
.32 0 0 r
.31102 .46981 .33996 .47861 .38322 .48439 .35418 .4872 Metetra
.092 0 0 r
.35418 .4872 .38322 .48439 .42807 .47975 .3996 .48821 Metetra
.344 .814 .741 r
.3996 .48821 .42807 .47975 .47415 .46239 .44675 .46897 Metetra
.736 .992 .838 r
.44675 .46897 .47415 .46239 .52097 .4327 .49478 .43018 Metetra
.89 .965 .834 r
.49478 .43018 .52097 .4327 .56805 .39371 .5429 .37692 Metetra
.898 .872 .804 r
.5429 .37692 .56805 .39371 .61512 .35029 .5906 .31718 Metetra
.851 .779 .781 r
.5906 .31718 .61512 .35029 .66224 .30797 .63788 .25972 Metetra
.783 .696 .768 r
.63788 .25972 .66224 .30797 .70975 .27179 .68525 .21223 Metetra
.702 .623 .765 r
.68525 .21223 .70975 .27179 .75823 .24544 .73357 .18015 Metetra
.613 .56 .771 r
.73357 .18015 .75823 .24544 .80836 .2307 .78383 .16605 Metetra
.373 .522 .877 r
.12748 .3498 .15476 .39017 .19298 .39641 .16532 .35885 Metetra
0 .314 .804 r
.16532 .35885 .19298 .39641 .2311 .41613 .20281 .38363 Metetra
0 .125 .646 r
.20281 .38363 .2311 .41613 .27016 .44326 .2412 .41726 Metetra
0 .101 .584 r
.2412 .41726 .27016 .44326 .31102 .46981 .2816 .45045 Metetra
0 .286 .704 r
.2816 .45045 .31102 .46981 .35418 .4872 .32469 .47309 Metetra
.201 .666 .941 r
.32469 .47309 .35418 .4872 .3996 .48821 .37052 .47654 Metetra
.661 .901 .98 r
.37052 .47654 .3996 .48821 .44675 .46897 .41848 .45612 Metetra
.844 .908 .88 r
.41848 .45612 .44675 .46897 .49478 .43018 .46756 .4127 Metetra
.893 .866 .806 r
.46756 .4127 .49478 .43018 .5429 .37692 .51674 .35242 Metetra
.894 .826 .772 r
.51674 .35242 .5429 .37692 .5906 .31718 .56534 .28479 Metetra
.871 .789 .767 r
.56534 .28479 .5906 .31718 .63788 .25972 .6133 .2201 Metetra
.824 .75 .782 r
.6133 .2201 .63788 .25972 .68525 .21223 .66113 .1672 Metetra
.744 .701 .81 r
.66113 .1672 .68525 .21223 .73357 .18015 .70978 .13225 Metetra
.623 .634 .841 r
.70978 .13225 .73357 .18015 .78383 .16605 .76042 .11809 Metetra
.194 .591 .943 r
.09697 .32817 .12748 .3498 .16532 .35885 .13515 .33557 Metetra
0 .26 .71 r
.13515 .33557 .16532 .35885 .20281 .38363 .1729 .35784 Metetra
0 .114 .59 r
.1729 .35784 .20281 .38363 .2412 .41726 .2115 .38841 Metetra
0 .182 .674 r
.2115 .38841 .2412 .41726 .2816 .45045 .25208 .41848 Metetra
.042 .415 .862 r
.25208 .41848 .2816 .45045 .32469 .47309 .29538 .43849 Metetra
.444 .632 .934 r
.29538 .43849 .32469 .47309 .37052 .47654 .34151 .44034 Metetra
.68 .723 .882 r
.34151 .44034 .37052 .47654 .41848 .45612 .38993 .41964 Metetra
.79 .754 .821 r
.38993 .41964 .41848 .45612 .46756 .4127 .43968 .37722 Metetra
.842 .77 .782 r
.43968 .37722 .46756 .4127 .51674 .35242 .4897 .31881 Metetra
.866 .784 .767 r
.4897 .31881 .51674 .35242 .56534 .28479 .53928 .25338 Metetra
.87 .8 .778 r
.53928 .25338 .56534 .28479 .6133 .2201 .58827 .19065 Metetra
.849 .816 .817 r
.58827 .19065 .6133 .2201 .66113 .1672 .6371 .13902 Metetra
.78 .82 .884 r
.6371 .13902 .66113 .1672 .70978 .13225 .68662 .10435 Metetra
.623 .775 .954 r
.68662 .10435 .70978 .13225 .76042 .11809 .73796 .08937 Metetra
.024 .593 .769 r
.06267 .32461 .09697 .32817 .13515 .33557 .10202 .32602 Metetra
.231 0 0 r
.10202 .32602 .13515 .33557 .1729 .35784 .14114 .33844 Metetra
0 .334 .735 r
.14114 .33844 .1729 .35784 .2115 .38841 .18107 .35676 Metetra
.061 .434 .873 r
.18107 .35676 .2115 .38841 .25208 .41848 .22272 .37438 Metetra
.355 .521 .883 r
.22272 .37438 .25208 .41848 .29538 .43849 .2667 .38432 Metetra
.538 .569 .834 r
.2667 .38432 .29538 .43849 .34151 .44034 .31313 .38076 Metetra
.646 .605 .794 r
.31313 .38076 .34151 .44034 .38993 .41964 .36165 .36063 Metetra
.718 .642 .772 r
.36165 .36063 .38993 .41964 .43968 .37722 .41152 .32449 Metetra
.772 .687 .769 r
.41152 .32449 .43968 .37722 .4897 .31881 .46194 .27642 Metetra
.815 .742 .783 r
.46194 .27642 .4897 .31881 .53928 .25338 .51233 .22283 Metetra
.844 .811 .818 r
.51233 .22283 .53928 .25338 .58827 .19065 .56249 .17089 Metetra
.839 .892 .878 r
.56249 .17089 .58827 .19065 .6371 .13902 .61268 .12699 Metetra
.745 .947 .946 r
.61268 .12699 .6371 .13902 .68662 .10435 .66355 .09568 Metetra
.507 .888 .965 r
.66355 .09568 .68662 .10435 .73796 .08937 .71592 .07912 Metetra
.02447 .33522 .06267 .32461 .10202 .32602 .06584 .32706 Metetra
.095 .652 .753 r
.06584 .32706 .10202 .32602 .14114 .33844 .10746 .32371 Metetra
.305 .728 .973 r
.10746 .32371 .14114 .33844 .18107 .35676 .14987 .32274 Metetra
.478 .656 .939 r
.14987 .32274 .18107 .35676 .22272 .37438 .19352 .32109 Metetra
.548 .578 .837 r
.19352 .32109 .22272 .37438 .2667 .38432 .23873 .31572 Metetra
.587 .545 .775 r
.23873 .31572 .2667 .38432 .31313 .38076 .28555 .30418 Metetra
.619 .544 .748 r
.28555 .30418 .31313 .38076 .36165 .36063 .33385 .28518 Metetra
.652 .567 .746 r
.33385 .28518 .36165 .36063 .41152 .32449 .38332 .25896 Metetra
.689 .613 .766 r
.38332 .25896 .41152 .32449 .46194 .27642 .43359 .22719 Metetra
.731 .691 .813 r
.43359 .22719 .46194 .27642 .51233 .22283 .48438 .1926 Metetra
.768 .811 .887 r
.48438 .1926 .51233 .22283 .56249 .17089 .53559 .15837 Metetra
.737 .941 .951 r
.53559 .15837 .56249 .17089 .61268 .12699 .58731 .12745 Metetra
.525 .92 .867 r
.58731 .12745 .61268 .12699 .66355 .09568 .63984 .10206 Metetra
.27 .766 .711 r
.63984 .10206 .66355 .09568 .71592 .07912 .69357 .08327 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`8001moo`P00003Ool007oo00]oo`03001oogoo01Yoo`00AWoo00@0
07ooOol001Aoo`<001ioo`H000A3WP800003Ool007oo00Moo`04001oogoo000LOol004Ioo`04001o
ogoo000BOol2000NOol50003@ih00`00@ih00002000014lN000000002Goo0P007Goo0016Ool01000
Oomoo`003goo0`0000=oo`00Ool06goo10000eoN00<004>N00000P001DlN00<007oo00009goo0016
Ool01000Oomoo`003Goo0P007goo10001EoN0P0000=?7P00Cah01TlN00<007oo00009Woo0017Ool2
000;Ool3000NOol300000d>k001OgP05Gmh00`00HeaSG0020008Cah2000VOol0059oo`<001ioo`80
0093^`8000EOgP8000ASG003001?7TlN00E?7P@002Eoo`00Cgoo0`007goo1`0015oN0P001V=L0P00
0TlN10000TjJ00<007oo000097oo001<Ool3000QOol01@00GmiOgUoN00000f^L0P0000=OgP00He`0
1f=L10001djJ0P0097oo001:Ool3000QOol20003Gmh00`00Jia[W004Ji`00`00HeaSG007He`00`00
G]T00008CYX00`00Ool0000SOol004Moo`<0009oo`03001oogoo01eoo`8000AOgP03001[W6^L00E[
W003001SG6=L00ISG080009Nf@03001>VTjJ00I>VP03001oo`00029oo`00AGoo0P0097oo00<005oN
Gmh00eoN00<006^LJi`01F^L00<006mJ00001F=L0P0000ENf@00G]UNf@0000Q>VP03001oo`00029o
o`00@Woo0`0097oo0`0015oN00<006^LJi`01F^L00D006mJKeY_FP0000=SG005001NfEkIG]T00003
G]T00`00CYY>VP06CYX00`00Ool0000QOol002Eoo`@001Moo`8002Ioo`04001S?6<lHc`200000eoN
001[W006Ji`00`00KeY_FP03KeX00`00He`00004G]T01@00G]UNfEkI000024jJ00@007ooOol0021o
o`009Woo00<007ooOol057oo0`009goo00<006<lHc`016<l0P001V^L00<006mJKeX01FmJ00<005kI
G]T00ekI00<005kIG]T00UkI00<004jJCYX014jJ1@0087oo000OOol50003Ool00`00Oomoo`0AOol2
000XOol20007Hc`00`00KcX00005Ji`00`00KeY_FP05KeX00`00G]UNf@03G]T00`00G]UNf@02G]T0
0`00CYY>VP040002CSP01000Oomoo`007goo000XOol00`00Oomoo`0=Ool3000YOol00`00HcaS?006
Hc`01@00KcY_>Vlj00000f^L00<006mJKeX01FmJ00<006[i000015kI00<005kIG]T00UkI0`001dhh
00<007oo00007goo000UOol01000Oomoo`003Goo0P00:goo00<006<lHc`01V<l00<006ljKcX00flj
00<006^L00001fmJ00@006[iJ_T000ENf@06001NfEkI001JN00024hh00@007ooOol001ioo`009Woo
0P002goo0`00:goo0P0026<l00<006ljKcX01Flj00<006mJKeX01FmJ00<006[iJ_T00V[i00<005kI
G]T00UkI0`000eYh00<004hhCSP01Dhh00D007ooOomoo`0001eoo`00<Goo0P0000=oo`00Ool0:Woo
00<005kJ00001f<l00<006ljKcX01Vlj0P001FmJ00<006[iJ_T016[i00<005kIG]T00P001EYh00<0
04hhCSP01Thh00@007ooOol001eoo`00;Woo0`00;Goo0P000ekJ00<006<lHc`00f<l00<006ljKcX0
1Vlj00@006lhKcP000=_FP03001ZnF[i00EZn@05001Nf@00FWP00005FWP00`00CSQ>>006CSP00`00
Oomoo`02000LOol002]oo`<002ioo`8000INfP05001S?6<lHc`00008KcX00`00KcQ_>002KcP00`00
KeX00008J_T20002FWP00`00FWQJN004FWP00`00CSQ>>006CSP01@00Oomoogoo00001Goo0P0057oo
000YOol20002Ool00`00Oomoo`0[Ool00`00G]YNfP07G]X00`00Hc`00008KcX00`00KcQ_>004KcP0
0`00J_UZn@06J_T00`00I]P00002FWP00`00FWQJN004FWP00`00CSQ>>006CSP01@00Oomoogoo0000
17oo00@007ooOol001=oo`009Woo0`00<7oo0P002ekJ0P001flj00<006lhKcP01Flh00<006[iJ_T0
1F[i00L006KHI]QVf000FWP00007FWP00`00CSQ>>005CSP00`00Oomoo`02Ool01@00Oomoogoo0000
0Woo00<007ooOol04Goo000TOol2000bOol00`00FWP0000:G]X01000J_UZn@001Flj00<006lhKcP0
1Flh00<006lh00001F[i00<006KHI]P00VKH00<005Yh00001eYh00<004hhCSP014hh0`0017oo00@0
07ooOol0009oo`03001oogoo015oo`008Goo0`00<Woo0P000eYh00<005kJG]X01UkJ00<006[iJ_T0
0V[i00D006ljKcY_>P0000M_>004001_>6lh0004J_T00`00I]QVf004I]P20007FWP00`00CSQ>>004
0002CQL01000Oomoogoo0P000Woo00@007ooOol001=oo`007goo0P00<goo0P001EYh00<005kJG]X0
1EkJ00<006[iJ_T016[i00<006lj00001flh00<006lhKcP00Vlh00@006[iJ_T000MVf08000MJN0<0
00M>5`03001oogoo009oo`03001oogoo008001Aoo`0077oo0`00=7oo00<005YhFWP01UYh00<005kJ
G]X00ekJ00<006[iJ_T01V[i00<006lhKcP01Flh00<006lhKcP00flh00<006[i00001FKH1@001EYh
0P0000=FE`00CQL01dhG00<007ooOol00goo00<007ooOol05Goo000KOol2000dOol20009FWP00`00
G]YNfP02G]X00`00J_UZn@07J_T00`00KcQ_>005KcP00`00KcQ_>004KcP01000I]QVf6KH0`0016KH
0P000UYh0P0015IG00<004hGCQL01DhG00<007ooOol00goo00<007ooOol05Goo000DOol30004Ool0
0`00Ool0000bOol00`00DUL0000:FWP01000G]YNfP002F[i00<006lh00001Flh00<006lhKcP01Flh
10001fKH00@005Yh000000IFE`03001>5dhG00I>5`03001oogoo00=oo`03001oogoo01Aoo`005Goo
00<007ooOol00goo00@007ooOol002moo`8000=BE`03001JN5Yh00MJN003001NfP0000UZn@05001_
>6lhKcP00003KcP00`00KcQ_>004KcP40009I]P200000eIG001FE`05EUL00`00CQM>5`06CQL00`00
Oomoo`03Ool2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0000/Ool20005DUL0
0`00FWQJN008FWP00`00J_UZn@07J_T00`00KcQ_>003KcP00`00KcP00004KcP40002KaP00`00I]QV
f006I]P01@00GYP005IG00001eIG00<004hGCQL01DhG00<007ooOol017oo00<007ooOol04goo000E
Ool00`00Oomoo`02Ool20004Ool00`00Oomoo`0XOol00`00DUMBE`05DUL00`00FWQJN007FWP20009
J_T00`00KcQ_>004KcP00`00KcQ_>0030003KcP01@00KaQ_66lH00001VKH00L005jHGYQNV000EUL0
0006EUL00`00CQM>5`06CQL00`00Oomoo`04Ool00`00Oomoo`0BOol001Eoo`03001oogoo009oo`80
00Eoo`03001oogoo02Eoo`8000UBE`03001JN5Yh00EJN004001R^6:h0007J_T00`00KcQ_>005KcP3
0005KcP00`00KaQ_6002KaP00`00I]QVf002I]P20004GYP01000EUMFE`001EIG00<004hGCQL01ThG
00<007ooOol017oo00<007ooOol04Woo000DOol20004Ool00`00Oomoo`05Ool00`00Oomoo`0QOol3
00000dhg001BE`08DUL00`00FWQJN004FWP00`00H[QR^002H[P00`00J_UZn@03J_T00`00KcQ_>004
KcP200000g=H001_>004KcP00`00KaQ_6004KaP01000I]QVf0001UjH00@005IGEUL000IFE`03001>
5dhG00E>5`03001oogoo00Eoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol07Goo
0`001Dhg00<0059GDUL01U9G00<005YhFWP00eYh00<006:hH[P016:h00<006[iJ_T00V[i00@006lh
KcQ_>0@000=cF005001_>6lhKcP00007KaP00`00I]P00008GYP01000EUMFE`001EIG00<004hGCQL0
1ThG00<007ooOol01Goo00<007ooOol047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0HOol40008
CSL00`00DUMBE`07DUL01@00FWQJN5Yh00001f:h00H006[iJ_UZn@00KcP30002KcP00`00LeQcF002
LeP01@00KcQ_>6lh000026lH00<005jHGYP01ejH00D005IGEUMFE`0000AFE`03001>5dhG00A>5`<0
00Ioo`80019oo`006Goo0P002Woo00<007ooOol047oo1`000dXh00<004hgCSL01Thg00<0059GDUL0
1e9G00@005YhFWP000UR^003001Zn@00008000E_>003001cF7=H00=cF003001_>00000Q_608000UN
V003001FEeIG009FE`03001FEeIG009FE`03001>5`0000<0009>5`03001oogoo00Ioo`03001oogoo
00moo`0027oo0P0017oo0P000goo0`000goo0P002goo00<007ooOol027oo1`000dIJ00<004XhBSP0
1DXh00<004hgCSL01Thg00<0059GDUL01U9G00<005Yh00002F:h0`001flh00<007=HLeP01G=H00<0
06lHKaP01VlH00@006LJIaX000UNV005001FEeIGEUL00003EUL30007CQL00`00Oomoo`06Ool00`00
Oomoo`0>Ool000Moo`04001oogoo0003Ool20006Ool01000Oomoo`000Woo5@000cjN00<004IJAUX0
1DIJ00<004XhBSP014Xh00<004hgCSL01Thg00<0059GDUL01U9G0P001f:h0`000V:h00<006lhKcP0
16lh00<007=HLeP01W=H00<006lHKaP01FlH00D006LJIaYW6P0000UNV003001FEeIG009FE`<00003
DSL004hG00M>5`03001oogoo00Ioo`03001oogoo00ioo`0000=oo`0000000`0000Eoo`00Oomoo`00
00]oo`03001oogoo00<000Yoo`D000<Vo`03000nWSjN00@nWP030016FTIJ00A6FP03001:>4Xh00A:
>003001>=dhg00I>=`03001BEe9G00EBE`03001JV00000AR^0<000AR^003001[6@0000E_>003001c
F7=H00EcF003001cN@0000E_6003001W6VLJ00=W6P03001NV5jH00INV003001FE`0000800005DSL0
058gDSL00008CQL00`00Oomoo`07Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool0
0`00Oomoo`0;Ool00`006UhJGP0300029_l00`00?YhnWP03?Yh00`00AUY6FP04AUX00`00BSQ:>005
BSP00`00CSM>=`06CSL00`00DUMBE`03DUL01000FYQJV5ZH10001V:h00D006/IJaU[6@0000=_>003
001cF7=H00EcF005001cNG=iLgT00003KaP00`00IaYW6P04IaX00`00GYQNV006GYP20004DSL01@00
DSMB=e8g000024hG00<007ooOol01goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`0027oo
00<007ooOol00goo00<007ooOol037oo00@001YN6UhJGPD000DnWP030016FTIJ00A6FP03001:>4Xh
00A:>003001>=dhg00I>=`04001BEe9GDUL20003FYP30007H[P00`00JaU[6@03JaT00`00KcP00007
LeP00`00LgUcN@03LgT00`00KaP00007IaX00`00GYQNV004GYP200000eIi001B=`04DSL01000DSMB
=`0024hG00<007ooOol01goo00<007ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool0
0`00Oomoo`0=Ool400000aYN000000020004?Yh00`00AUY6FP04AUX00`00BSQ:>005BSP00`00CSM>
=`05CSL01000DUMBE`000UZH0`000UZH00<006:hH[P016:h00<006/IJaT01F/I00<007=HLeP01G=H
00<007=iLgT017=i0P002FLJ00@005jHGYQNV08000AFN@03001B=e8g00=B=`03001B=`0000Q>5`03
001oogoo00Qoo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P003goo00@002hL;Q`^70L0008n
WP030016FTIJ00A6FP03001:>4Xh00E:>003001>=dhg00I>=`D000IJV003001R^6:h009R^003001[
6F/I00I[6@8000EcF003001cNG=i00AcN@03001_^`0000UW6P04001NV0000006EWT00`00DSMB=`04
DSL00`00DSL00008CQL00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qoo`80011o
o`03000^72hL008^70030001F@0000@00003?Yh004IJ00E6FP03001:>4Xh00E:>003001>=dhg00A>
=`<000UJV005001R^6:hH[P00008JaT01000LgUcN@000g=H00<007=iLgT017=i00D006nkKk]_^`00
00QW6P8000QFN@03001B=e8g00AB=`03001B=`0000Q>5`03001oogoo00Moo`80009oo`04001oogoo
0006Ool0019oo`8000Aoo`03001oogoo011oo`03000^72hL008^70030001F@00008000031K/00000
00I6FP03001:>4Xh00E:>004001>=dhgCSL300000e9i001JV00:FYP00`00H[P00008JaT00`00LgUc
N@02LgT00`00LeP00006LgT00`00Kk]_^`03Kk/00`00IaYW6P02IaX300000e[l001FN@08EWT00`00
DSMB=`04DSL20008CQL00`00Oomoo`09Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool0
0`00Oomoo`0AOol00`00;Q`^7002;Q`00`000ET1F@0200021K/500000dIJ001:>006BSP40005DWT0
0`00FYQJV008FYP20008JaT00`00LgUcN@04LgT20005LgT00`00Kk]_^`05Kk/01000IaX000001E[l
00<005IiEWT01UIi00<0058gDSL01E8g0P0024hG00<007ooOol02Goo00<007ooOol027oo000AOol0
1000Oomoo`000goo00<007ooOol04Woo00<002hL;Q`00RhL00<0005I0ET00P0000<5^`00;Q`00bhL
2`001e9i00<005ZHFYP01eZH00<006LJ00001f/I00<007=iLgT01G=i00<007?J00000g=i00<006nk
Kk/01Vnk0P001e[l00<005IiEWT01UIi00<0058gDSL01U8g00<004hGCQL01DhG0P002goo00<007oo
Ool027oo000AOol01000Oomoo`000Woo0P005Goo100000@^70000ET1F@@000D^70030016FTIJ0080
00=6FP03001BNE9i00IBN@03001JV5ZH00EJV005001W6VLJIaX00006JaT00`00LgUcN@04LgT01`00
LmYcfW?J001cN@0000M_^`800003Ekl005[l00MJo003001FNEIi00IFN@03001B=e8g00EB=`03001>
5dhG009>5`<00003DSL007oo00]oo`03001oogoo00Moo`004Goo00@007ooOol000=oo`03001oogoo
01=oo`04000mnCgi?OT400000`5I0000]@020004;Q`30006AUX00`00DWUBN@06DWT00`00FYQJV003
FYP00`00IaYW6P03IaX00`00JaU[6@02JaT00`00LgUcN@04LgT00`00LmYcfP02LmX30005Kk/20004
Ekl00`00F_aJo005F_`00`00EWUFN@06EWT00`00DSMB=`06DSL40005DSL00`00Oomoo`09Ool00`00
Oomoo`07Ool0019oo`8000=oo`8001Ioo`03000mnCgi008mn@03000QVP00008000030;D0000000<0
0003;Q`004IJ00I6FP03001BNE9i00IBN@03001JV5ZH009JV003001W6VLJ00AW6P05001[6F/IJaT0
0006LgT00`00LmYcfP02LmX01000GoYOnUoj0P0000=_^`0000001eNo00<005[lF_`01E[l00<005Ii
EWT01EIi00<0058gDSL0158g10001e8g00<007ooOol02Woo00<007ooOol01Woo000GOol00`00Oomo
o`0EOol00`00?OTmn@03?OT00`008IXQVP040005;Q`00`00AUY6FP05AUX00`00DWUBN@06DWT01000
FYQJV00026LJ00<006/I00001W=i00<007?JLmX00W?J00<005ojGoX00eoj10001eNo00<005[lF_`0
1E[l00<005IiEWT01UIi00<0058g00000`000Thh0P001e8g00<007ooOol02goo00<007ooOol01Goo
000GOol00`00Oomoo`0FOol00`00?OTmn@02?OT40000126J0000]@001BhL00<004IJAUX014IJ00<0
059iDWT01U9i00<005ZH00002VLJ0P001W=i00D007?JLmYcfP0000=OnP<0008_EQ`000M>>08000MB
=`03001oogoo00Yoo`03001oogoo00Eoo`005goo00<007ooOol05Woo00<003gi?OT010000b6J1@00
0bhL00<004IJAUX014IJ00<0059iDWT01U9i0P002VLJ00<006nlKk`00P000g=i00D007?JLmYcfP00
009OnPL000038Wh0000000<000@Vo`030012WD:M00E2W@03001:FTYJ00E:FP03001>>4hh00I>>003
001B=`0000IB=`03001oogoo00]oo`03001oogoo00Aoo`001Goo0P0017oo0P000goo0`0017oo0P00
5goo0`000cgi00<0026J8IX00R6J00<0004i0000100000<^7000AUX01DIJ00<0059iDWT00U9i0`00
00=Fo000IaX026LJ00<006nlKk`00fnl00<007=i00000g?J1@0014ln00<0029n8Wh00R9n00<0006J
00000P000bKo00<004:M@Yd01D:M00<004YJBUX01DYJ00<004hhCSP01Dhh00@0058gDSL000EB=`03
001oogoo00Yoo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000EOol30000
0goo000mn@04?OT00`008IXQVP028IX010000CT000000PFk0P001TIJ00<0059i00000P001EKl00<0
06LJIaX01FLJ00<006nlKk`016nl2@001Tln10000R9n00<0006J0IX0100000<Vo`00@Yd01D:M00<0
04YJBUX01DYJ00<004hhCSP01Thh00<0058g00001U8g00<007ooOol01Woo0`000Woo00<007ooOol0
0goo0004Ool01000Oomoo`002goo00<007ooOol00P004goo0P0017oo00<003gi?OT00cgi00<0026J
8IX00R6J00<0004i0CT00P000`Fk20002EKl00<006LJIaX00fLJ00<006nlKk`016nl00@005ojGoYO
nP8000AS?003001??Tln00<00004DYX003Xk>S/400020IX20000106j000000001T:M00<004YJBUX0
14YJ00<004hhCSP01Thh00@0058gDSL000EB=`03001oogoo00=oo`<000Eoo`03001oogoo00=oo`00
17oo00@007ooOol000Qoo`<000=oo`80011oo`<000Moo`X000048IX0004i0CT200031K/00`00>Ydj
W@03>Yd00`00E_aFo008E_`01@00IaYW6VLJ00001Vnl00<005ojGoX00P001V<l0`001E:J00<003Xk
>S/00SXk0`000P6J0P000P6j0P0014:M00<004YJBUX01DYJ00<004hhCSP01Dhh00D0058gDSMB=`00
00AB=`03001oo`0000<000Qoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=o
o`03001oogoo00aoo`<000Yoo`030015n4Gh00=5n004000mnCgi?OT500000`4i000000021K/00`00
>YdjW@03>Yd00`00E_aFo008E_`00`00IaX00005Kk`200000eoj00000006Hc`200000ekI001BVP06
DYX00`00>S/j>`02>S/00`006K/0000200001@4h0001^P007Q`00`000T:M00<004YJBUX014YJ00<0
04hhCSP01Thh00D0058gDSMB=`0000AB=`03001oogoo00Yoo`03001oogoo00=oo`001Goo0P002Goo
10000Woo00<007ooOol02Goo0`003Goo00<004GhAOP014Gh00<003gi?OT00Sgi00<002hK;Q/01`00
1CZM00<005KlE_`01eKl0`0016nl00<005ojGoX00P0000=[>@00Hc`00f<l00<005kIG]T00UkI00<0
05:JDYX015:J00<003Xk>S/00cXk00<001Vk6K/00P0000@1>000000000<N70<000M:FP03001>>4hh
00E>>005001B=e8gDSL00003DSL2000=Ool00`00Oomoo`02Ool001Ioo`03001oogoo00Ioo`<0015o
o`030015n4Gh00=5n003000mnCgi00<mn@03000^6bhK008^6`8000030CH0000000DjW@03001Fo5Kl
00=Fo0<000=GW`800003Kk`0000000<000=[>@04001S?0000005G]T00`00DYYBVP05DYX00`00>S/j
>`03>S/00`006K/I^`0200020CP200057Q`30004BUX00`00CSQ>>005CSP00`00DSMB=`02DSL30000
0e9i001oo`0<Ool00`00Oomoo`02Ool001Eoo`8000Eoo`<001Aoo`030015n4Gh00A5n003000mnCgi
008mn@03000^6bhK00@^6`8000030CH0000000<00003>Yd005Kl00<000MGW`<0009[V`03001[>F/i
00=[>@03001NfEkI00INf@03001BVU:J00=BVPH00003>S/001Vk00<I^`8000030CP001hL00@N7003
000f>cHk00D000Q>>0@00003DWT0059i009BN@03001oogoo00Yoo`03001oogoo009oo`005Goo00@0
07ooOomoo`<001Moo`030015n4Gh00A5n003000mnCgi00<mn@03000^6bhK00@^6`D0008J_P@000IG
W`<000A[V`03001[>F/i00=[>@8000QNf@03001BVU:J00<0009>>00400126D8I@QT400026K/01000
0CT0004h0P0011hL00<003Hk=S/00cHk2P00159i00D0059iDWUBN@0000aoo`03001oogoo009oo`00
5Goo10006goo00<004GhAOP00dGh00<003gi?OT013gi00<002hK;Q/012hK00<0018m4Sd00P001AZn
0`0000=GW`0000000P0016^K00<006/iJcT00f/i00@006KhI_P000MNf@<000I>>00300126D8I0092
6@03000afP0000<000030CT00000008000<N7003000f>cHk00@f>`03001:FTYJ00E:FP03001BNE9i
00=BN@04001BNE9i000<Ool00`00Oomoo`02Ool000ioo`<000Aoo`8001eoo`030015n4Gh00A5n003
000mnCgi00<mn@03000^6bhK008^6a000004Hc`006/iJcT200000f^K001[>@04JcT00`00I_QVn002
I_P00`00G]UNf@03G]T00`00FWP00007CSP00`00@QU26@03@QT01000<MXafS7J0P000P4i10000QhL
00<003Hk=S/013Hk00<004YJBUX01DYJ00<0059iDWT00e9i00@0059iDWT000]oo`03001oogoo009o
o`003goo00<007ooOol01Goo00<007ooOol06Woo00<004GhAOP014Gh00@003gi?OTmn@L000E>NP03
001;7d/O009;7`03001S?6<l00=S?003001[>F/i00=[>@03001[>F/i00=[>@03001Vn6Kh00=Vn004
001NfEkIG]T20003FWP00`00CSQ>>004CSP00`00@QU26@03@QT00`00<MXafP03<MX200020CT00`00
0GX000020006=S/00`00BUY:FP05BUX00`00DWUBN@04DWT00`00DWT0000;Ool00`00Oomoo`02Ool0
00moo`03001oogoo00Ioo`03001oogoo01Uoo`030015n4Gh00=5n0D000=>>003001>NTij00M>NP04
001;7`000005Hc`00`00JcU[>@04JcT20003JcT00`00I_QVn005I_P00`00G]T00005FWP00`00CSQ>
>005CSP00`00@QU26@03@QT00`00<MXafP02<MX00`006K/000020CT200000`5j000f>`05=S/00`00
BUY:FP05BUX00`00DWUBN@04DWT00`00DWT0000<Ool00`00Oomoo`01Ool000moo`03001oogoo00Mo
o`03001oogoo01Uoo`030015n0000080009>5`03001>>4hh00A>>003001>NTij00I>NP<000ES?003
001[>F/i00A[>@06001_66lH001[>@0026Kh0P001UYh00<004hhCSP01Dhh00<0048I@QT00d8I00<0
037J<MX00c7J00<001Vk00000P0000<1>0000GX00P001CHk00<004YJBUX01DYJ00<0059iDWT0159i
0`002Goo0`000goo000?Ool00`00Oomoo`08Ool2000IOol20006CQL00`00CSQ>>003CSP00`00CWY>
NP05CWX01@00G[UN^Eji00000f<l00<006/iJcT016/i00<006lHKaP00VlH00<006KhI_P01VKh00<0
06:h00001eYh00<004hhCSP014hh00<0048I@QT0148I00<0037J<MX00c7J00<001Vk6K/00P0000<1
>000000010000SHk00<004YJBUX01DYJ00<0059iDWT00U9i1@001Woo0`000R0000<007ooOol00Goo
000>Ool2000<Ool00`00Oomoo`0FOol00`00CQM>5`05CQL00`00CSQ>>004CSP00`00CWY>NP02CWX2
0005G[T00`00Hc`00006JcT00`00KaQ_6003KaP00`00I_QVn005I_P01@00H[QR^6:h00001UYh00<0
04hhCSP01Dhh00<0048I@QT00d8I00<0037J<MX00c7J00<001Vk6K/00QVk00<0004h0CP00P000QWl
0`001dYJ00<0059i00000P000eKl00@005Kl001oo`@000@P008000=oo`007Goo00<007ooOol05Woo
00<004hGCQL014hG00<004hhCSP01Dhh00@004ijCWX000QN^@03001[>F/i00A[>@03001_66lH00=_
6003001_600000EVn003001R^6:h009R^003001JN5Yh00AJN003001>>4hh009>>0l00003<MX001Vk
00<I^`800081>003000Io1Wl008Io0T000MFo0<0008H0003000P020000<00003Ool007oo009oo`00
7Woo00<007ooOol05Goo00<004hGCQL01DhG00<004hhCSP014hh00<004ij00002Eji0P001F/i00<0
06lHKaP00VlH00D006lHKaQ_600000=Vn003001R^6:h00=R^003001JN5Yh00AJN0D000=>=`03001:
5dXG00A:5`030012648H009260D0008I^`<000030CP00000008Io003000nWCjM00DnW@03001Fo5Kl
00IFo0800003600000000080008H0003001oo`0000=oo`007goo0P005Goo00<004hGCQL01DhG00<0
04hhCSP014hh0P002Eji00@006KhI_P000=[>@03001_66lH00=_6003001_66lH009_6003001Vn000
00MR^003001JN5Yh009JN003001FE`0000M>=`03001:5dXG00A:5`030012648H00=26003000f6cHK
00@000030GX0000000<000036O`003jM00HnW@03001Fo5Kl00IFo0<000@H0004001oogoo0003Ool0
025oo`03001oogoo01=oo`03001>5dhG00E>5`03001>>4hh009>>003001JN00000QN^@03001Vn6Kh
009Vn003001[>@0000E_6003001_66lH00A_6003001R^6:h00IR^003001JN5Yh008000=FE`03001>
=dhg00A>=`03001:5dXG00A:5`030012648H00A26003000f6cHK008f6`H000040IX000000007?Yd0
0`00E_aFo005E_`400000aP000000004Ool10001Ool1Ool0029oo`03001oogoo019oo`03001>5dhG
00E>5`03001>>4hh008000=JN003001N^Eji00AN^@03001Vn6Kh00AVn08000A_6003001_66lH00E_
6003001R^6:h00IR^003001JN00000EFE`03001>=dhg00A>=`03001:5dXG00A:5`030012648H00A2
6003000f6cHK00@f6`03000NGP0000<000030IX0000000HnW@03001Fo5Kl009Fo0<00003Ckl004no
008000Ioo`40005oo`5oo`008goo00<007ooOol04Goo00<004hGCQL01DhG00<004hh00001UYh00<0
05jiG[T00Uji00<006KhI_P01FKh00<006lH00000flH00<006lHKaP016lH00<006/H00001f:h0P00
1UIG00<004hgCSL01Dhg00<004XGBQL014XG00<0048H@QP0148H00<003HK=Q/013HK00<001iN7Uh0
0QiN20000SjM00@005Kl000000=?_`<000Qoo`40005oo`5oo`0097oo00<007ooOol04Goo00<004hG
CQL014hG0P001eYh00D005jiG[UN^@0000MVn007001_66lHKaP006lH00001VlH00D006/HJaQ[6000
00ER^003001NV00000MFE`03001>=dhg00A>=`03001:5dXG00A:5`030012648H00A26003000f6cHK
008f6`L000=;7P<0008N_PD00004Ckl000000003Ool00`00Oomoo`05Ool10001Ool1Ool002Eoo`80
015oo`04001>5dhGCQL300000eIG001JN007FWP00`00G[T00007I_P00`00KaQ_6003KaP00`00KaQ_
6004KaP00`00JaQ[6002JaP01000H[QR^6:h0P000ejH00<005IGEUL015IG00<004hgCSL01Dhg00<0
04XGBQL00dXG00<0048H@QP00T8H1`0014ij00<004/NBah00T/N00@007ooOomoo`T000Eoo`03001o
ogoo00Eoo`40005oo`5oo`009goo00<007ooOol03Woo00@004hG000000EFE`03001JN5Yh00EJN080
00MVn003001_66lH00A_608000E_6003001[66/H00=[6003001R^00000ENV003001FEeIG00AFE`03
001>=dhg00E>=`03001:5dXG00=:5`D000=>>003001>NTij00I>NP04001;7P00000COol00`00Oomo
o`04Ool10001Ool1Ool002Qoo`03001oogoo00ioo`03001FEeIG00EFE`03001JN5Yh00AJN003001R
^00000IVn003001_66lH00A_6004001c>7<h0003KaP00`00JaQ[6004JaP20006GYP00`00EUMFE`05
EUL00`00CSM>=`04CSL00`00BQM:5`0300000dhG001>>006CSP00`00CWY>NP05CWX2000FOol00`00
Oomoo`03Ool10001Ool1Ool002Uoo`03001oogoo00eoo`03001FEeIG00EFE`03001JN5Yh00=JN005
001R^6:hH[P00005I_P00`00KaQ_6003KaP00`00LcQc>002LcP00`00KaP00006JaP00`00I_T00007
GYP00`00EUMFE`04EUL00`00CSM>=`03CSL40005CQL00`00CSQ>>004CSP00`00CWY>NP03CWX2000H
Ool00`00Oomoo`04Ool10001Ool002Yoo`8000ioo`03001FEeIG00EFE`05001JN5YhFWP00005H[P0
1@00I_QVn6Kh00001VlH00<007<hLcP00g<h00<006/HJaP016/H00D006KiI_UVn@0000INV003001F
EeIG00AFE`03001>=dhg00<0009B=`03001>5dhG00A>5`03001>>4hh00E>>005001>NTijCWX0000K
Ool00`00Oomoo`03Ool10001Ool002aoo`03001oogoo00]oo`03001FEeIG00EFE`04001JN5Yh0007
H[P00`00I_P00006KaP00`00LcQc>004LcP00`00JaQ[6003JaP00`00I_UVn@02I_T00`00GYQNV004
GYP00`00EUMFE`03EUL40005DSL00`00CQM>5`05CQL00`00CSQ>>004CSP01000CWX000007Goo00<0
07ooOol00Woo0@000Goo000]Ool00`00Oomoo`0:Ool00`00EUMFE`05EUL00`00FWP00009H[P00`00
KaQ_6004KaP00`00LcQc>004LcP00`00LgT00003JaP00`00I_UVn@04I_T00`00GYQNV004GYP00`00
EUL000020002DSL00`00DSMB=`05DSL00`00CQM>5`04CQL00`00CSQ>>004CSP2000POol01@00Oomo
ogoo00000Goo000^Ool00`00Oomoo`0:Ool00`00EUMFE`04EUL2000:H[P20004KaP00`00LcQc>004
LcP01`00LgUcNG=i001[600000MVn@03001NV5jH00=NV0<000EB=`03001B=e8g00EB=`03001>5dhG
00A>5`03001>>4hh00=>>003001oogoo021oo`05001oogooOol00001Ool002moo`8000Yoo`03001F
EeIG00=FE`03001JV00000UR^004001Zn6[h0003KaP00`00LcQc>003LcP00`00LgUcN@03LgT00`00
I_UVn@06I_T00`00GYQNV00300000eIi001B=`06DSL00`00DSMB=`04DSL00`00CQM>5`05CQL00`00
CSQ>>002000TOol01000Oomoo`000Goo000aOol00`00Oomoo`08Ool01@00EUMFEeIG00000eZH00<0
06:hH[P01F:h00<006[hJ_P00V[h00<006lH00001G<h00<007=iLgT00g=i00<006nk00001fKi0`00
1EIi00<0058gDSL0158g00<0058gDSL01E8g00<004hGCQL014hG00<004hh00009goo00<007oo0000
0Goo000bOol00`00Oomoo`07Ool01000EUMFE`0015ZH00<006:hH[P016:h00<006[hJ_P016[h0P00
1G<h00<007=iLgT00W=i00D006nkKk]_^`0000AVn@800003F_/005Ii00EFN@03001B=e8g00AB=`03
001B=e8g00EB=`03001>5dhG00A>5`8002Qoo`03001oogoo00400000<goo00<007ooOol01Woo00<0
05IG00001UZH00<006:hH[P00V:h00<006[hJ_P01F[h00<007=I00000g<h00<007=iLgT00W=i00<0
06nkKk/00fnk00@006Ki000000AJn`03001FNEIi00AFN@03001B=e8g00AB=`03001B=e8g00AB=`04
001>5dhGCQL3000[Ool10001Ool1000003Aoo`8000Moo`03001JV5ZH00IJV004001R^6:h0007J_P0
1`00LeUcFG=I001c>00000AcN@03001_^fnk00A_^`8000IJn`03001FNEIi00AFN@03001B=e8g00AB
=`03001B=e8g00EB=`<002moo`800000=Woo00<007ooOol017oo00<005ZHFYP01UZH00<006:h0000
1f[h00<007=ILeT00g=I0`000W=i00<006nkKk/00Vnk0`0000=G_`00F_/01U[k00<005IiEWT00eIi
00<0058gDSL0158g00<0058gDSL00e8g0`00<7oo0`00000gOol00`00Oomoo`04Ool00`00FYQJV006
FYP00`00J_QZn005J_P00`00LeUcF@03LeT01000LmYcfW?J100000=_^`0000001ENo00<005[kF_/0
15[k00<005IiEWT015Ii00<0058gDSL0158g00<0058g00000P00;goo10000goo000hOol00`00Oomo
o`03Ool00`00FYQJV005FYP00`00I_T00005J_P00`00LeUcF@03LeT01@00LmYcfW?J00000e_i1000
1UNo00<005[kF_/00e[k00<005IiEWT015Ii00@0058gDSMB=`D002ioo`@000Moo`00>Goo00<007oo
Ool00goo00<005ZHFYP00eZH00D006KiI_UVn@0000=Zn003001cFG=I00=cF@03001cfW?J0080009K
n@8000=ooaP0031oo`<000]oo`00>Woo0P000goo00<005ZHFYP00UZH00<006KiI_T00fKi00<006[h
00001G=I00@007?JLmX0009Kn@8004Uoo`@000ioo`00?7oo00D007ooOomoo`00009JV003001VnFKi
00EVn@8000AcF@03001cfW?J00@004Moo`@0019oo`00?Goo00H007ooOol005ZH0007I_T00`00Jia[
W00200000g=I000000040015Ool4000FOol003ioo`04001oogoo0007I_T00`00Jia[W002Ji`50002
Ool00`00Oomoo`0oOol4000JOol003moo`800003Ool006Ki00AVn@03001[W6^L009[W004001Kn@00
0004Ool00`00Oomoo`0kOol4000NOol0045oo`03001oo`0000=Vn@03001[W6^L009[W003001Kn@00
00Eoo`03001oogoo03Uoo`<0029oo`00@Woo00D007oo001Vn@0000A[W0<000Ioo`03001oogoo03Eo
o`@002Eoo`00@goo00<007oo00000P000V^L0P0027oo00<007ooOol0<Goo1000:Goo0014Ool20002
Ool2000:Ool00`00Oomoo`0]Ool4000]Ool004Ioo`03001oogoo00]oo`03001oogoo02Uoo`@0035o
o`00Agoo00<007ooOol02Woo00<007ooOol09Goo1000=Goo0018Ool00`00Oomoo`09Ool00`00Oomo
o`0QOol4000iOol004Uoo`8000Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007ooOol01Woo00<0
07ooOol06Woo1000@7oo001<Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool004eoo`03001o
ogoo00Aoo`03001oogoo019oo`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo
001?Ool20003Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03
001oo`0000Eoo`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo
002eOol00;Eoo`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.234 0 0 r
.37963 .66718 .40338 .65368 .44019 .66271 .41742 .66235 Metetra
.145 .689 .802 r
.41742 .66235 .44019 .66271 .47787 .67766 .45585 .65919 Metetra
.541 .734 .964 r
.45585 .65919 .47787 .67766 .51666 .6936 .49502 .65613 Metetra
.602 .612 .833 r
.49502 .65613 .51666 .6936 .55658 .70511 .53494 .65154 Metetra
.618 .559 .766 r
.53494 .65154 .55658 .70511 .59741 .70741 .57556 .64402 Metetra
.629 .543 .738 r
.57556 .64402 .59741 .70741 .63877 .69749 .61679 .63271 Metetra
.643 .549 .733 r
.61679 .63271 .63877 .69749 .68023 .67492 .6585 .6175 Metetra
.662 .576 .748 r
.6585 .6175 .68023 .67492 .72145 .64196 .70059 .59901 Metetra
.689 .635 .791 r
.70059 .59901 .72145 .64196 .76238 .60302 .74305 .57852 Metetra
.722 .756 .88 r
.74305 .57852 .76238 .60302 .80324 .56357 .78593 .55764 Metetra
.659 .936 .967 r
.78593 .55764 .80324 .56357 .8445 .52892 .82939 .53803 Metetra
.82939 .53803 .8445 .52892 .88682 .5033 .87364 .52106 Metetra
.188 0 0 r
.87364 .52106 .88682 .5033 .93088 .48915 .9189 .50754 Metetra
.261 0 0 r
.9189 .50754 .93088 .48915 .97722 .48681 .96535 .49757 Metetra
.35464 .68432 .37963 .66718 .41742 .66235 .39371 .66375 Metetra
.705 .971 .929 r
.39371 .66375 .41742 .66235 .45585 .65919 .43322 .64037 Metetra
.755 .789 .88 r
.43322 .64037 .45585 .65919 .49502 .65613 .47302 .61656 Metetra
.702 .648 .793 r
.47302 .61656 .49502 .65613 .53494 .65154 .51312 .59474 Metetra
.66 .574 .747 r
.51312 .59474 .53494 .65154 .57556 .64402 .55361 .57687 Metetra
.631 .537 .729 r
.55361 .57687 .57556 .64402 .61679 .63271 .59467 .56412 Metetra
.608 .522 .731 r
.59467 .56412 .61679 .63271 .6585 .6175 .63652 .55666 Metetra
.59 .529 .755 r
.63652 .55666 .6585 .6175 .70059 .59901 .67932 .5536 Metetra
.57 .569 .813 r
.67932 .5536 .70059 .59901 .74305 .57852 .72318 .55317 Metetra
.522 .675 .934 r
.72318 .55317 .74305 .57852 .78593 .55764 .76807 .55299 Metetra
.23 .733 .9 r
.76807 .55299 .78593 .55764 .82939 .53803 .8138 .55053 Metetra
.185 0 0 r
.8138 .55053 .82939 .53803 .87364 .52106 .86011 .54359 Metetra
.26 0 0 r
.86011 .54359 .87364 .52106 .9189 .50754 .90668 .53084 Metetra
.193 0 0 r
.90668 .53084 .9189 .50754 .96535 .49757 .95328 .51208 Metetra
.716 .981 .783 r
.32866 .69768 .35464 .68432 .39371 .66375 .36917 .66252 Metetra
.875 .93 .856 r
.36917 .66252 .39371 .66375 .43322 .64037 .40994 .62076 Metetra
.83 .785 .809 r
.40994 .62076 .43322 .64037 .47302 .61656 .45062 .5785 Metetra
.762 .678 .769 r
.45062 .5785 .47302 .61656 .51312 .59474 .49113 .54166 Metetra
.698 .604 .748 r
.49113 .54166 .51312 .59474 .55361 .57687 .53166 .51488 Metetra
.638 .552 .741 r
.53166 .51488 .55361 .57687 .59467 .56412 .57263 .50085 Metetra
.576 .514 .749 r
.57263 .50085 .59467 .56412 .63652 .55666 .61454 .49994 Metetra
.504 .492 .777 r
.61454 .49994 .63652 .55666 .67932 .5536 .65784 .51017 Metetra
.402 .488 .833 r
.65784 .51017 .67932 .5536 .72318 .55317 .70279 .52739 Metetra
.211 .505 .908 r
.70279 .52739 .72318 .55317 .76807 .55299 .74932 .54586 Metetra
0 .442 .746 r
.74932 .54586 .76807 .55299 .8138 .55053 .79703 .55917 Metetra
.278 0 0 r
.79703 .55917 .8138 .55053 .86011 .54359 .84522 .56161 Metetra
.189 0 0 r
.84522 .56161 .86011 .54359 .90668 .53084 .89308 .54955 Metetra
.89308 .54955 .90668 .53084 .95328 .51208 .93993 .52248 Metetra
.887 .943 .844 r
.30219 .69996 .32866 .69768 .36917 .66252 .34401 .65449 Metetra
.891 .852 .797 r
.34401 .65449 .36917 .66252 .40994 .62076 .38603 .59997 Metetra
.856 .775 .772 r
.38603 .59997 .40994 .62076 .45062 .5785 .42767 .54512 Metetra
.804 .711 .763 r
.42767 .54512 .45062 .5785 .49113 .54166 .46876 .49816 Metetra
.737 .653 .765 r
.46876 .49816 .49113 .54166 .53166 .51488 .50956 .46541 Metetra
.652 .595 .777 r
.50956 .46541 .53166 .51488 .57263 .50085 .55061 .45041 Metetra
.539 .533 .797 r
.55061 .45041 .57263 .50085 .61454 .49994 .59262 .45367 Metetra
.385 .465 .82 r
.59262 .45367 .61454 .49994 .65784 .51017 .63623 .47256 Metetra
.177 .39 .833 r
.63623 .47256 .65784 .51017 .70279 .52739 .68185 .50153 Metetra
0 .325 .807 r
.68185 .50153 .70279 .52739 .74932 .54586 .72947 .53266 Metetra
0 .321 .727 r
.72947 .53266 .74932 .54586 .79703 .55917 .77856 .55686 Metetra
.154 0 0 r
.77856 .55686 .79703 .55917 .84522 .56161 .82815 .5658 Metetra
.82815 .5658 .84522 .56161 .89308 .54955 .87709 .55411 Metetra
.585 .934 .762 r
.87709 .55411 .89308 .54955 .93993 .52248 .92438 .52101 Metetra
.859 .816 .806 r
.27577 .68574 .30219 .69996 .34401 .65449 .31851 .63664 Metetra
.869 .789 .77 r
.31851 .63664 .34401 .65449 .38603 .59997 .36147 .5777 Metetra
.859 .769 .761 r
.36147 .5777 .38603 .59997 .42767 .54512 .40395 .51853 Metetra
.832 .751 .774 r
.40395 .51853 .42767 .54512 .46876 .49816 .44571 .46809 Metetra
.779 .728 .806 r
.44571 .46809 .46876 .49816 .50956 .46541 .48699 .43319 Metetra
.674 .688 .855 r
.48699 .43319 .50956 .46541 .55061 .45041 .52836 .41762 Metetra
.481 .601 .895 r
.52836 .41762 .55061 .45041 .59262 .45367 .57059 .42194 Metetra
.197 .451 .875 r
.57059 .42194 .59262 .45367 .63623 .47256 .61442 .44334 Metetra
0 .303 .796 r
.61442 .44334 .63623 .47256 .68185 .50153 .66035 .47583 Metetra
0 .257 .756 r
.66035 .47583 .68185 .50153 .72947 .53266 .70842 .51081 Metetra
0 .36 .817 r
.70842 .51081 .72947 .53266 .77856 .55686 .75814 .53832 Metetra
.217 .603 .95 r
.75814 .53832 .77856 .55686 .82815 .5658 .80847 .54914 Metetra
.588 .827 .99 r
.80847 .54914 .82815 .5658 .87709 .55411 .85815 .53726 Metetra
.807 .894 .905 r
.85815 .53726 .87709 .55411 .92438 .52101 .90608 .50184 Metetra
.797 .715 .774 r
.24992 .65299 .27577 .68574 .31851 .63664 .29291 .60782 Metetra
.829 .738 .763 r
.29291 .60782 .31851 .63664 .36147 .5777 .33625 .55382 Metetra
.847 .766 .773 r
.33625 .55382 .36147 .5777 .40395 .51853 .37925 .49948 Metetra
.847 .802 .807 r
.37925 .49948 .40395 .51853 .44571 .46809 .42162 .45277 Metetra
.81 .845 .876 r
.42162 .45277 .44571 .46809 .48699 .43319 .46352 .41984 Metetra
.666 .856 .973 r
.46352 .41984 .48699 .43319 .52836 .41762 .50546 .40416 Metetra
.307 .707 .977 r
.50546 .40416 .52836 .41762 .57059 .42194 .54814 .40619 Metetra
0 .436 .831 r
.54814 .40619 .57059 .42194 .61442 .44334 .59224 .42339 Metetra
0 .304 .761 r
.59224 .42339 .61442 .44334 .66035 .47583 .63824 .45035 Metetra
0 .34 .813 r
.63824 .45035 .66035 .47583 .70842 .51081 .68626 .47934 Metetra
.191 .466 .886 r
.68626 .47934 .70842 .51081 .75814 .53832 .73588 .50157 Metetra
.451 .584 .895 r
.73588 .50157 .75814 .53832 .80847 .54914 .78625 .50895 Metetra
.632 .66 .859 r
.78625 .50895 .80847 .54914 .85815 .53726 .83629 .49627 Metetra
.745 .708 .817 r
.83629 .49627 .85815 .53726 .90608 .50184 .885 .4628 Metetra
.731 .639 .756 r
.22487 .6037 .24992 .65299 .29291 .60782 .26737 .56912 Metetra
.773 .69 .772 r
.26737 .56912 .29291 .60782 .33625 .55382 .31038 .52842 Metetra
.811 .764 .809 r
.31038 .52842 .33625 .55382 .37925 .49948 .35338 .48719 Metetra
.829 .868 .874 r
.35338 .48719 .37925 .49948 .42162 .45277 .39614 .45087 Metetra
.745 .966 .934 r
.39614 .45087 .42162 .45277 .46352 .41984 .43873 .42377 Metetra
.387 .848 .825 r
.43873 .42377 .46352 .41984 .50546 .40416 .48149 .40835 Metetra
.01 0 0 r
.48149 .40835 .50546 .40416 .54814 .40619 .52489 .40498 Metetra
.167 0 0 r
.52489 .40498 .54814 .40619 .59224 .42339 .56943 .41179 Metetra
0 .479 .816 r
.56943 .41179 .59224 .42339 .63824 .45035 .61548 .42495 Metetra
.224 .546 .929 r
.61548 .42495 .63824 .45035 .68626 .47934 .66314 .43916 Metetra
.434 .555 .878 r
.66314 .43916 .68626 .47934 .73588 .50157 .71218 .44855 Metetra
.55 .558 .816 r
.71218 .44855 .73588 .50157 .78625 .50895 .76207 .44789 Metetra
.625 .574 .778 r
.76207 .44789 .78625 .50895 .83629 .49627 .8121 .43388 Metetra
.682 .603 .761 r
.8121 .43388 .83629 .49627 .885 .4628 .86164 .40605 Metetra
.658 .576 .752 r
.20048 .54337 .22487 .6037 .26737 .56912 .24184 .52358 Metetra
.687 .637 .794 r
.24184 .52358 .26737 .56912 .31038 .52842 .28381 .50172 Metetra
.722 .751 .875 r
.28381 .50172 .31038 .52842 .35338 .48719 .32626 .47945 Metetra
.701 .917 .968 r
.32626 .47945 .35338 .48719 .39614 .45087 .36909 .45846 Metetra
.384 .841 .77 r
.36909 .45846 .39614 .45087 .43873 .42377 .41232 .44014 Metetra
.41232 .44014 .43873 .42377 .48149 .40835 .45606 .42531 Metetra
.08 0 0 r
.45606 .42531 .48149 .40835 .52489 .40498 .50048 .4141 Metetra
.50048 .4141 .52489 .40498 .56943 .41179 .54575 .40586 Metetra
.339 .798 .94 r
.54575 .40586 .56943 .41179 .61548 .42495 .59201 .39934 Metetra
.557 .73 .955 r
.59201 .39934 .61548 .42495 .66314 .43916 .6393 .39285 Metetra
.597 .612 .838 r
.6393 .39285 .66314 .43916 .71218 .44855 .68759 .38462 Metetra
.608 .557 .771 r
.68759 .38462 .71218 .44855 .76207 .44789 .73674 .37313 Metetra
.62 .538 .741 r
.73674 .37313 .76207 .44789 .8121 .43388 .78655 .35747 Metetra
.635 .545 .736 r
.78655 .35747 .8121 .43388 .86164 .40605 .83685 .33749 Metetra
.57 .52 .76 r
.17624 .4795 .20048 .54337 .24184 .52358 .21605 .47546 Metetra
.546 .565 .825 r
.21605 .47546 .24184 .52358 .28381 .50172 .25651 .47409 Metetra
.497 .677 .946 r
.25651 .47409 .28381 .50172 .32626 .47945 .29792 .47295 Metetra
.284 .76 .933 r
.29792 .47295 .32626 .47945 .36909 .45846 .34047 .46944 Metetra
.011 0 0 r
.34047 .46944 .36909 .45846 .41232 .44014 .38421 .46133 Metetra
.065 0 0 r
.38421 .46133 .41232 .44014 .45606 .42531 .42902 .44724 Metetra
.42902 .44724 .45606 .42531 .50048 .4141 .4747 .42698 Metetra
.372 .811 .655 r
.4747 .42698 .50048 .4141 .54575 .40586 .52101 .40157 Metetra
.725 .966 .938 r
.52101 .40157 .54575 .40586 .59201 .39934 .56777 .37306 Metetra
.756 .798 .887 r
.56777 .37306 .59201 .39934 .6393 .39285 .61494 .34403 Metetra
.7 .654 .801 r
.61494 .34403 .6393 .39285 .68759 .38462 .66264 .31707 Metetra
.654 .574 .752 r
.66264 .31707 .68759 .38462 .73674 .37313 .71109 .29428 Metetra
.621 .531 .732 r
.71109 .29428 .73674 .37313 .78655 .35747 .76062 .27691 Metetra
.597 .515 .733 r
.76062 .27691 .78655 .35747 .83685 .33749 .81156 .26512 Metetra
.45 .466 .784 r
.15135 .41982 .17624 .4795 .21605 .47546 .18958 .4293 Metetra
.299 .444 .839 r
.18958 .4293 .21605 .47546 .25651 .47409 .2284 .44593 Metetra
.05 .434 .872 r
.2284 .44593 .25651 .47409 .29792 .47295 .26856 .46381 Metetra
0 .393 .686 r
.26856 .46381 .29792 .47295 .34047 .46944 .31059 .47641 Metetra
.186 0 0 r
.31059 .47641 .34047 .46944 .38421 .46133 .35466 .47785 Metetra
.35466 .47785 .38421 .46133 .42902 .44724 .4005 .46442 Metetra
.377 .8 .611 r
.4005 .46442 .42902 .44724 .4747 .42698 .44754 .43558 Metetra
.747 .992 .811 r
.44754 .43558 .4747 .42698 .52101 .40157 .49512 .39418 Metetra
.876 .941 .856 r
.49512 .39418 .52101 .40157 .56777 .37306 .54272 .34564 Metetra
.841 .806 .817 r
.54272 .34564 .56777 .37306 .61494 .34403 .59016 .2966 Metetra
.769 .692 .777 r
.59016 .2966 .61494 .34403 .66264 .31707 .63765 .25339 Metetra
.697 .608 .753 r
.63765 .25339 .66264 .31707 .71109 .29428 .68571 .22094 Metetra
.629 .548 .744 r
.68571 .22094 .71109 .29428 .76062 .27691 .73509 .20203 Metetra
.563 .508 .751 r
.73509 .20203 .76062 .27691 .81156 .26512 .78652 .19705 Metetra
.267 .41 .821 r
.12485 .37076 .15135 .41982 .18958 .4293 .16191 .38903 Metetra
0 .26 .77 r
.16191 .38903 .18958 .4293 .2284 .44593 .19939 .41759 Metetra
0 .15 .646 r
.19939 .41759 .2284 .44593 .26856 .46381 .23847 .44837 Metetra
.381 0 0 r
.23847 .44837 .26856 .46381 .31059 .47641 .27999 .47208 Metetra
.152 0 0 r
.27999 .47208 .31059 .47641 .35466 .47785 .32426 .48014 Metetra
.335 .818 .842 r
.32426 .48014 .35466 .47785 .4005 .46442 .37096 .46702 Metetra
.743 .988 .891 r
.37096 .46702 .4005 .46442 .44754 .43558 .41926 .43193 Metetra
.887 .954 .842 r
.41926 .43193 .44754 .43558 .49512 .39418 .46816 .37913 Metetra
.904 .877 .799 r
.46816 .37913 .49512 .39418 .54272 .34564 .51683 .31662 Metetra
.873 .801 .776 r
.51683 .31662 .54272 .34564 .59016 .2966 .56494 .25391 Metetra
.817 .73 .769 r
.56494 .25391 .59016 .2966 .63765 .25339 .61271 .19978 Metetra
.741 .663 .773 r
.61271 .19978 .63765 .25339 .68571 .22094 .66084 .16083 Metetra
.646 .596 .784 r
.66084 .16083 .68571 .22094 .73509 .20203 .71031 .14073 Metetra
.532 .531 .799 r
.71031 .14073 .73509 .20203 .78652 .19705 .76217 .13994 Metetra
0 .354 .828 r
.09584 .33647 .12485 .37076 .16191 .38903 .13248 .35723 Metetra
0 .112 .626 r
.13248 .35723 .16191 .38903 .19939 .41759 .16942 .38932 Metetra
0 .056 .547 r
.16942 .38932 .19939 .41759 .23847 .44837 .20802 .42398 Metetra
0 .21 .668 r
.20802 .42398 .23847 .44837 .27999 .47208 .24934 .45102 Metetra
.125 .565 .922 r
.24934 .45102 .27999 .47208 .32426 .48014 .2938 .46094 Metetra
.599 .823 .985 r
.2938 .46094 .32426 .48014 .37096 .46702 .3411 .44757 Metetra
.806 .86 .89 r
.3411 .44757 .37096 .46702 .41926 .43193 .39033 .41004 Metetra
.871 .84 .812 r
.39033 .41004 .41926 .43193 .46816 .37913 .44032 .35311 Metetra
.887 .816 .773 r
.44032 .35311 .46816 .37913 .51683 .31662 .49007 .28567 Metetra
.878 .795 .764 r
.49007 .28567 .51683 .31662 .56494 .25391 .53912 .21818 Metetra
.847 .774 .78 r
.53912 .21818 .56494 .25391 .61271 .19978 .58763 .1602 Metetra
.785 .744 .816 r
.58763 .1602 .61271 .19978 .66084 .16083 .63633 .11882 Metetra
.672 .692 .861 r
.63633 .11882 .66084 .16083 .71031 .14073 .68627 .09793 Metetra
.495 .602 .889 r
.68627 .09793 .71031 .14073 .76217 .13994 .73859 .09802 Metetra
0 .338 .726 r
.06358 .31839 .09584 .33647 .13248 .35723 .10084 .33481 Metetra
0 .143 .582 r
.10084 .33481 .13248 .35723 .16942 .38932 .13838 .36117 Metetra
0 .175 .657 r
.13838 .36117 .16942 .38932 .20802 .42398 .17751 .38964 Metetra
.001 .375 .839 r
.17751 .38964 .20802 .42398 .24934 .45102 .21924 .41118 Metetra
.388 .571 .91 r
.21924 .41118 .24934 .45102 .2938 .46094 .26402 .41751 Metetra
.625 .664 .868 r
.26402 .41751 .2938 .46094 .3411 .44757 .31165 .40325 Metetra
.745 .706 .815 r
.31165 .40325 .3411 .44757 .39033 .41004 .36129 .36766 Metetra
.809 .734 .782 r
.36129 .36766 .39033 .41004 .44032 .35311 .41186 .31487 Metetra
.845 .761 .769 r
.41186 .31487 .44032 .35311 .49007 .28567 .46243 .25264 Metetra
.863 .79 .778 r
.46243 .25264 .49007 .28567 .53912 .21818 .51246 .19017 Metetra
.859 .825 .814 r
.51246 .19017 .53912 .21818 .58763 .1602 .56205 .13598 Metetra
.812 .856 .881 r
.56205 .13598 .58763 .1602 .63633 .11882 .61177 .09649 Metetra
.676 .846 .965 r
.61177 .09649 .63633 .11882 .68627 .09793 .66257 .07523 Metetra
.421 .731 .99 r
.66257 .07523 .68627 .09793 .73859 .09802 .71549 .07264 Metetra
.138 0 0 r
.02767 .31503 .06358 .31839 .10084 .33481 .06675 .32079 Metetra
0 .42 .748 r
.06675 .32079 .10084 .33481 .13838 .36117 .10623 .33301 Metetra
.091 .491 .899 r
.10623 .33301 .13838 .36117 .17751 .38964 .14706 .34628 Metetra
.359 .533 .891 r
.14706 .34628 .17751 .38964 .21924 .41118 .18999 .3546 Metetra
.519 .551 .828 r
.18999 .3546 .21924 .41118 .26402 .41751 .23536 .3526 Metetra
.614 .573 .785 r
.23536 .3526 .26402 .41751 .31165 .40325 .28305 .33691 Metetra
.68 .605 .764 r
.28305 .33691 .31165 .40325 .36129 .36766 .33251 .30706 Metetra
.734 .648 .763 r
.33251 .30706 .36129 .36766 .41186 .31487 .38301 .26562 Metetra
.781 .706 .78 r
.38301 .26562 .41186 .31487 .46243 .25264 .43388 .21758 Metetra
.82 .784 .818 r
.43388 .21758 .46243 .25264 .51246 .19017 .4847 .16897 Metetra
.831 .88 .878 r
.4847 .16897 .51246 .19017 .56205 .13598 .53547 .12563 Metetra
.758 .957 .937 r
.53547 .12563 .56205 .13598 .61177 .09649 .58656 .09206 Metetra
.532 .916 .928 r
.58656 .09206 .61177 .09649 .66257 .07523 .63861 .07082 Metetra
.267 .769 .875 r
.63861 .07082 .66257 .07523 .71549 .07264 .69235 .06223 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Aoo`H0
00Aoo`03001oogoo00Uoo`@001aoo`00IWoo0P004Woo1@001B<K0`000goo00<007ooOol02Goo00<0
07ooOol077oo001SOol3000BOol20003Akd;0002Ool00`00Oomoo`02Ool50003Ool00`00Oomoo`0K
Ool004Moo`8001Qoo`80019oo`<000=7_@80009GOP03000focKo008fo`04000focKo0003Ool00`00
Oomoo`0:Ool00`00Oomoo`0JOol004Ioo`04001oogoo000DOol3000BOol20004Akd20005Egh01@00
=_lfocKo00000cKo00D007ooOomoo`0000Qoo`04001oogoo000LOol004Ioo`04001oogoo000BOol2
000COol40002Akd20008Egh01000=_lfo`0013Ko00@007oo000000Uoo`8001eoo`00AWoo00@007oo
Ool000moo`<00003Ool007oo015oo`03001SgF?M009Sg@<000UGOPX0009oo`03001oogoo02Eoo`00
AWoo00@007ooOol000eoo`8001Eoo`8000ASg@05001WO6MlIg`00005Egh3000015KK000nO0001Cil
00D007ooOomoo`0002Ioo`00Agoo0P002goo0`005Goo0P001F?M00<006MlIg`00fMl00<005Mn0000
0P001EKK0P001Sil00@007ooOol002Ioo`00DWoo0`005Woo00<006?MHmd00f?M0P001VMl0P0025KK
0P001cil00@007ooOol002Eoo`00Cgoo0`005goo0`0016?M00<006MlIg`01FMl00<006Kj00002EKK
00<003il?W`01Cil00D007ooOomoo`0002Aoo`00C7oo0`006Goo00@006^LJia[W0800003Hmd006Ml
00IWO005001VnVKjI_X00008E]/20007?W`01000Oomoo`0097oo001:Ool3000JOol00`00Jia[W004
Ji`00`00IgaWO005Ig`00`00I_YVnP02I_X00`00E]]Ff`06E]/20004?W`50002Ool00`00Oomoo`0Q
Ool004Moo`<001aoo`03001[W6^L00A[W003001_FP0000EWO003001VnVKj00AVnP03001FfeKK00EF
f`H000A66@05001oogooOol0000ROol004Eoo`8001ioo`03001[W6^L00A[W005001_FVmJKeX00003
Ig`00`00I_YVnP06I_X00`00E]]Ff`02E]/300000e9i00166@07AQT01000Oomoo`008Woo0012Ool3
000OOol00`00Jia[W004Ji`00`00KeY_FP03KeX00`00Ig`00009I_X00`00E]/000020002DWT01000
DWUBN@001dHI00D007ooOomoo`00025oo`009Goo10005goo0P008Goo00<006^LJi`016^L00<006mJ
KeX01FmJ00<006KjI_X026Kj0P001E9i00@0059iDWT000M66@03001oogoo009oo`03001oogoo01io
o`009Woo00<007ooOol057oo0`008Woo0P001F^L00<006mJKeX01VmJ00<006KjI_X01fKj00<005jh
00001E9i00D0059iDWUBN@0000M66@03001oogoo0080021oo`007goo1@000goo00<007ooOol04Goo
0P0097oo00@006/jJcX000=[W003001_FVmJ00I_FP03001_600000MVnP05001N^5jhG[P00004DWT0
1@00DWUBNE9i00001dHI00<007ooOol00Woo00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000U
Ool00`00JcY[>P02JcX00`00Ji`00008KeX01@00KaQ_66lH000016Kj0P0015jh00<0059iDWT00U9i
00D0059iDWUBN@0000Q66@05001oogooOol0000OOol002Eoo`04001oogoo000=Ool2000WOol00`00
JcY[>P04JcX00`00KeY_FP06KeX00`00KaQ_6002KaP01@00I_YVnVKj00001ejh00<0059iDWT00U9i
00D0059iDWUBN@0000M66@03001oogoo009oo`03001oogoo01aoo`009Woo0P002goo0`00:7oo00<0
06/jJcX01F/j0P001VmJ00<006lHKaP016lH00<006Kj000025jh00<0059iDWT00U9i00D0059iDWUB
N@0000M66@03001oogoo00=oo`03001oogoo01]oo`00<Goo0P0000=oo`00Ool09goo00<006/jJcX0
1F/j00@006lhKcP000E_FP03001_66lH00E_6003001N^5jh00MN^003001BNE9i009BN@04001BNE9i
DWT:0004Ool00`00Oomoo`0KOol002ioo`<002]oo`03001[>V/j00E[>P03001_>6lh009_>005001_
FVmJKeX00008KaP00`00G[QN^008G[P01000DWUBNE9i100000=B=`00BQP01TXH00<007ooOol00Woo
0P0077oo000[Ool3000]Ool00`00JcY[>P06JcX00`00KcQ_>003KcP00`00KeX00008KaP00`00J_P0
0009G[P00`00DWT000020005DSL00`00BQQ:6005BQP00`00Oomoo`03Ool00`00Oomoo`03Ool2000D
Ool002Uoo`80009oo`03001oogoo02Uoo`8000Q[>P03001_>6lh00E_>003001_66lH00I_6004001Z
n6[h0009G[P20002DSL00`00DSMB=`03DSL00`00BQQ:6005BQP00`00Oomoo`03Ool00`00Oomoo`02
Ool01000Oomoo`004goo000VOol3000_Ool01000H]QRf0001V/j00<006lhKcP01Vlh00<006lHKaP0
1FlH00<006[hJ_P00V[h00<005jhG[P01Ejh00<005Yh00000U8g00<0058gDSL00e8g00<004XHBQP0
1DXH00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo0P00<Goo00<006;H
H]P00V;H00<006/jJcX00V/j00<006lhKcP01Vlh00<007<h00001FlH00<006[hJ_P00f[h00<005jh
G[P00ejh0P000eYh00<0058g00001U8g00<004XHBQP01DXH00<007ooOol017oo00@007ooOol0009o
o`03001oogoo015oo`008Goo0`00<Woo00<006;HH]P016;H00D006/jJcY[>P0000M_>005001c>7<h
LcP00004KaP00`00J_QZn004J_P00`00G[QN^0020005FWP00`00DSL00006DSL00`00BQQ:6005BQP0
0`00Oomoo`03Ool20002Ool01000Oomoo`004goo000OOol2000dOol00`00H]QRf006H]P00`00JcX0
0007KcP00`00LcQc>002LcP01@00KaQ_66lH00001f[h00<005jh00001eYh0`001U8g00<004XHBQP0
1TXH00<007ooOol017oo00<007ooOol00P0057oo000LOol3000eOol00`00H]QRf008H]P00`00KcQ_
>006KcP00`00LcQc>003LcP00`00KaP00009J_P00`00FWQJN004FWP50007DSL00`00BQQ:6005BQP0
0`00Oomoo`05Ool00`00Oomoo`0EOol001]oo`8003Ioo`03001Rf6;H00URf003001_>6lh00E_>003
001c>7<h00Ec>003001Zn6[h00MZn08000=JN0<000=JN003001B=`0000IB=`03001:64XH00E:6003
001oogoo00Eoo`03001oogoo01Eoo`0057oo0`0017oo00<007oo0000<goo0P002f;H00<006lH0000
1Flh00<007<hLcP01W<h00<006[hJ_P01V[h00<006;HH]P00`001UYh00@0058gDSL000EB=`03001:
64XH00A:60<000Moo`03001oogoo01Aoo`005Goo00<007ooOol00goo00@007ooOol0035oo`03001N
V00000YRf005001_66lHKaP00004KcP00`00LcQc>005LcP00`00KcP00006J_P50009FWP01000DSMB
=`001E8g00<004XH00000`000ThG00<007ooOol01Goo0P005Goo000>Ool50002Ool00`00Oomoo`03
Ool01@00Oomoogoo0000;goo00D005jHGYQNV00000QRf003001_66lH009_6005001_>6lhKcP00007
LcP01@00KcQ_>6lh00000f[h0`000f;H00<005YhFWP01eYh00@0058gDSL000AB=`<000M>5`03001o
ogoo00Eoo`03001oogoo01=oo`005Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0:goo
00<005jHGYP00UjH00<006;HH]P01F;H00<006lHKaP016lH00<006lh00001g<h00<006lhKcP00Vlh
1@001V;H00<005YhFWP01UYh00@0058gDSMB=`@00003CQL004hG00M>5`03001oogoo00Ioo`03001o
ogoo019oo`005Goo00<007ooOol00Woo0P001Goo00<007ooOol0:Goo00<005jHGYP015jH00<006;H
H]P00f;H00<006lHKaP01VlH00<007<hLcP01G<h00@006lhKcQ_>0@00003J_P006;H00IRf003001J
N5Yh00MJN0D000=>5`03001>5dhG00I>5`03001oogoo00Ioo`03001oogoo019oo`0057oo0P0017oo
00<007ooOol01Goo00<007ooOol09goo00<005jHGYP01EjH00<006;HH]P00V;H00<006lHKaP01flH
00<007<hLcP01G<h00@006lh000000A_>003001Rf6;H00IRf003001JN5Yh00IJN08000=>5`03001>
5dhG009>5`03001>5dhG00I>5`03001oogoo00Ioo`03001oogoo015oo`006Woo00<007ooOol01Woo
00<007ooOol09Goo00<005jHGYP01ejH00@006;HH]P000U_6003001cF00000Ec>0<000E_>08000UR
f003001JN5Yh00=JN0800003DUP004hG00=>5`05001>5dhGCQL00008CQL00`00Oomoo`07Ool00`00
Oomoo`0@Ool001Yoo`03001oogoo00Moo`03001oogoo029oo`8000YNV003001Rf00000U_6006001c
F7=HLeP007<h0`001flh00@006/jJcX000QRf004001JN5YhFWP20003DUP00`00CQM>5`03CQL01000
CQM>5`0024hG00<007ooOol01Woo0P004Woo000IOol50007Ool00`00Oomoo`0POol00`00EWP0000;
GYP00`00KaQ_6007KaP01000LeQcF7=H0`0000=c>000KcP01Vlh00D006/jJcY[>P0000QRf004001J
N0000006DUP00`00CQM>5`03CQL01000CQM>5`0024hG00<007ooOol01goo00<007ooOol03goo0008
Ool20004Ool20003Ool30003Ool20002Ool60003Ool00`00Oomoo`0MOol20003EWP00`00GYQNV008
GYP00`00KaQ_6006KaP00`00LeP000020003LeP00`00KcQ_>005KcP00`00JcY[>P03JcX00`00H]QR
f005H]P20008DUP00`00CQM>5`03CQL01000CQM>5`0024hG00<007ooOol027oo00<007ooOol03Woo
0007Ool01000Oomoo`000goo0P001Woo00@007ooOol000Moo`D00003Ool007oo01]oo`8000EFN003
001NV5jH00MNV003001Vn@0000I_60<000IcF08000E_>003001[>V/j00A[>P03001Rf6;H00=Rf080
0003FYT0059H00MBF003001>5dhG00A>5`03001>5`0000Q>5`03001oogoo00Qoo`03001oogoo00io
o`0000=oo`0000000`0000Eoo`00Oomoo`0000]oo`04001oogoo0009Ool00`000EL00003000JOol0
0`00EWQFN005EWP00`00GYQNV006GYP01@00I_UVnFKi00000VlH10001g=H00@007>ILiT000=_>003
001[>V/j00I[>P05001Rf6;HH]P00004FYT00`00DUQBF006DUP00`00CQM>5`04CQL00`00CQL00008
CQL00`00Oomoo`08Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`08
Ool200000`5G0009o003000DOol30009EWP00`00GYQNV004GYP00`00I_UVn@02I_T30002KaP00`00
LeQcF005LeP00`00LiUcV@02LiT00`00KcP00009JcX01000H]P000001EZI00<0059HDUP01U9H00<0
04hGCQL014hG00<004hG00001dhG0P002goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`00
27oo00<007ooOol00goo00<007ooOol02Goo10000`Wl0`003Goo10000TiI00<005IhEWP01eIh00<0
05jHGYP00ejH00@006KiI_UVn@80009Vn@03001_600000McF003001cVG>I00AcV@03001[>V/j00Q[
>P8000MJV@03001BF59H00IBF003001>5dhG00E>5`8000A>5`<00003DSL007oo00Yoo`03001oogoo
00=oo`03001oogoo00Ioo`0027oo0P002Goo10000Woo00<007ooOol02Woo00@0005j00000089o003
000^?2hl00<000Eoo`D000M>F@03001FN5Ih00MFN006001NV5jHGYP006Ki0`001FKi00<007=HLeP0
1G=H00<007>ILiT01G>I00<006/jJcX01f/j00<006<l000025ZI00<0059HDUP01U9H00<004hGCQL0
1DhG1@001E8g00<007ooOol02Goo00<007ooOol00goo00<007ooOol01Goo000HOol2000=Ool01000
0GX1NP000PWl00<002hl;S`00Rhl1@000d8j00<004iICUT01TiI00<005IhEWP01eIh00<005jHGYP0
0`0026Ki0P001G=H00<007>ILiT01G>I00<007?K00001V/j0P000f<l00<005ZIFYT01EZI00<0059H
DUP01U9H00<004hGCQL01DhG0P00258g00<007ooOol02Woo00<007ooOol00goo00<007ooOol017oo
000HOol2000>Ool00`000GX1NP0200000`Wl000^?003;S`00`00@SY2>P05@SX00`00CUU>F@06CUT0
0`00EWQFN006EWP3000:I_T01000KeU_F@000g=H00<007>ILiT01G>I00D007?KLm]cf`0000=[>P80
00ES?003001JVEZI00IJV@03001BF59H00EBF003001>5dhG009>5`<00003CSL0058g00MB=`03001o
ogoo00Uoo`80009oo`04001oogoo0006Ool0019oo`8000Aoo`03001oogoo00ioo`8000050GX00000
2O`00004;S`00`00@SY2>P04@SX00`00CUU>F@06CUT00`00EWQFN003EWP5000:I_T00`00KeU_F@02
KeT00`00LeP00007LiT00`00Lm]cf`03Lm/00`00JcX00008Hc`00`00FYUJV@05FYT00`00DUQBF006
DUP40005CSL00`00DSMB=`06DSL00`00Oomoo`0:Ool00`00Oomoo`020007Ool0015oo`04001oogoo
0003Ool00`00Oomoo`0>Ool00`008KX000040004;S`00`00@SY2>P05@SX00`00CUU>F@06CUT00`00
EWP000020002EWP00`00G]X00009I_T00`00KeU_F@04KeT20006LiT00`00Lm]cf`04Lm/20009Hc`0
0`00FYUJV@05FYT00`00DUQBF002DUP50008CSL20007DSL00`00Oomoo`0;Ool00`00Oomoo`08Ool0
015oo`04001oogoo0003Ool00`00Oomoo`0?Ool00`008KXQ^P040004;S`00`00@SY2>P04@SX00`00
CUU>F@05CUT30003EWP20003G]X00`00I_UVn@05I_T00`00KeU_F@05KeT01000LmX000000g>I00<0
07?KLm/00g?K0P000UoN00<006<lHc`01f<l00<005ZIFYT00eZI1P0014hh00<004hgCSL01Thg00<0
058g00001U8g00<007ooOol02goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P004Goo00<0
026j8KX00R6j10000bhl00<0048j@SX0148j00<004iICUT00TiI0P000TiI00@005IhEWP000INfP03
001VnFKi00=Vn@03001_FFmI00E_F@03001cfW?J009cfP03001cV@0000Ecf`03001OgUoN00=OgP03
001S?6<l00IS?003001JVEZI00<0009>NP03001>>4hh00I>>003001>=dhg00I>=`8000MB=`03001o
ogoo00]oo`03001oogoo00Moo`004Goo00@007ooOol000=oo`03001oogoo011oo`08000Q^R6j8KX0
004H0001NP800003;S`0048j00E2>P03001>F@00008000A>F@03001FN00000QNfP05001VnFKiI_T0
0007KeT00`00LmYcfP03LmX40002Lm/20007Gmh00`00HcaS?002Hc`60006CWX00`00CSQ>>005CSP0
0`00CSM>=`06CSL00`00DSL00006DSL00`00Oomoo`0;Ool00`00Oomoo`07Ool0019oo`8000=oo`80
019oo`03000Q^R6j008Q^P03000160000081NP03000^?00000A2>P<000M>F@8000YNfP03001Vn@00
00M_F@03001cfW?J00=cfP03001OnEoi009On@@000EOgPL000A6oP03001>NTij00E>NP03001>>4hh
00E>>003001>=dhg00I>=`04001B=e8g0005DSL00`00Oomoo`0<Ool00`00Oomoo`06Ool001Moo`03
001oogoo019oo`03000Q^R6j008Q^P03000160000081NP800004@SX000000002@SX00`00CUU>F@02
CUT300000djk001NfP09G]X30006KeT00`00LmYcfP03LmX01000GoUOnEoi0P000Rld1`0012/n00<0
04KnA_h01DKn00<004ijCWX014ij00<004hhCSP01Thh00<004hgCSL01Thg00<0058g00001E8g0P00
3Woo00<007ooOol01Goo000GOol00`00Oomoo`0BOol00`008KXQ^P028KX01@000AP160000GX01`00
00=2>P00CUT00`001Djk00<005kJG]X01UkJ00D006MlIgaWO00000E_F@04001cfW?JLmX20002GoT6
00049]l60002:ch00`00A_i6oP04A_h00`00CWY>NP05CWX00`00CSQ>>005CSP00`00CSM>=`06CSL0
1000DSMB=`000U8g0P0000=FN000Ool03Goo00<007ooOol01Goo000GOol00`00Oomoo`0COol40000
0b6j0000000200020GX00`00<UdbG@02<Ud30008C[/00`00G]YNfP05G]X00`00IgaWO003Ig`01@00
KeU_FFmI00000g?J00<005oiGoT00`000eMo40001TKn00<004ijCWX014ij00<004hhCSP01Dhh00<0
04hgCSL01Thg00@0058gDSMB=`8000AFN003001oogoo00aoo`03001oogoo00Aoo`001Goo0P0017oo
0P000goo0`0017oo0P0057oo00<003WI>MT01@0000D160000001NP0000DbG@03001>^djk00M>^`03
001NfUkJ009NfP8000MWO003001_F@0000=cfPD0009GO`<0009Ff`03000nO3il00@nO003000If`00
008000030OX00000008000=6oP03001>NTij00A>NP03001>>4hh00I>>003001>=dhg00I>=`<00003
EWP005Ih00=FN003001oogoo00]oo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogoo
Ool0000EOol30003>MT00`009MX000050005<Ud00`00C[]>^`07C[/01000G]YNfP002FMl20000fMl
0P001UKK00<003il?W`013il00<001WK6M/010000Qhm10001dij00<004hhCSP01Dhh00<004hgCSL0
14hg0`000eIh00<005IhEWP00eIh00<007ooOol01goo0`000Woo00<007ooOol00goo0004Ool01000
Oomoo`002goo00<007ooOol00P004goo0P0000=oo`00>MT013WI00<002GJ9MX00P0000<0U0000000
1C9M00<004jkC[/01Tjk00<005kJ00002FMl00@005olGoaOo08000EWO08000QFf`03000nO3il00@n
O003000IfaWK00@000@N?@@000A>NP03001>>4hh00E>>003001>=dhg00@0009>F003001FN5Ih009F
N003001FN5Ih009FN003001oogoo00Aoo`<000Eoo`03001oogoo00=oo`0017oo00@007ooOol000Qo
o`<000=oo`80011oo`<000Aoo`03000ifCWI008if@03000UfRGJ008UfP@000DbG@03001>^djk00A>
^`@000MWO080009Oo08000IWO004001RnF;i0008E]/00`00?W`nO003?W`00`006M/If`026M/30004
7Sd00`00=S`f?0050004CSP60007CUP00`00EWQFN002EWP01@00EWQFN5Ih00000Woo100027oo00<0
07ooOol00goo0004Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol037oo0`001goo00<0
03WI>MT00cWI00<002GJ9MX00RGJ200000@bG@00C[]>^`<000A?G`8000=WO08000=Oo0<000EWO003
001RnF;i009Rn@03001FfeKK00AFf`X0008If`030001>@00008000<N?@03000f?3Hl00<f?0D000A6
>P03001>F4iH00I>F003001FN5Ih00=FN004001FN5IhEWP2000<Ool00`00Oomoo`03Ool000Eoo`80
00Uoo`@0009oo`03001oogoo00Uoo`<000Yoo`03000ifCWI00<if@03000UfRGJ00<UfP030005^`Fk
00<000033Ud00000008000U?G`03001WO00000AOo005001[N`00KcT00003Ig`00`00H_URn@04H_T0
0`00E]]Ff`030002DWP00`00ASU6>@03AST00`00<MX0000300000`4i0000000200027Sd00`00=S`f
?003=S`00`00ASY6>P06ASX00`00CUQ>F005CUP00`00EWQFN003EWP01@00EWQFN5Ih00003Woo00<0
07ooOol00Woo000FOol00`00Oomoo`06Ool3000>Ool00`00>MTif@03>MT00`009MXUfP039MX01000
1K/5^`Fk0P0010iM00<004mOCel01TmO1`0000=[N`00KcT00Vli00<006Ml00001f;i0`001U9h00<0
04HiAST00dHi00@0037J<MXafP800081>@030001FP00008000Hf?0030016>THj00E6>P03001>F4iH
00I>F003001FN5Ih00=FN003001FN5Ih008000eoo`03001oogoo009oo`005Goo0P001Goo0`004Goo
00<003WI>MT00cWI00<002GJ9MX012GJ00@000Fk1K/5^`@0008>G@03001?GdmO009?G`L000=[N`03
001_>Fli00=_>@03001RnF;i00ERn@03001NV00000MBN0030016>DHi00=6>@03000afS7J00<afP80
00040CT0005J0EX20005=S`00`00ASY6>P05ASX00`00CUQ>F006CUP00`00EWQFN004EWP200000ekJ
001oo`0<Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000EOol@0004?[h400000dmO00000004
0005Jg/00`00KcU_>@04KcT20005H_T01@00GYQNV5jH00001U9h00<004HiAST014Hi00<0037J<MX0
0S7J00@001FJ0001>@800081FP8000@f?0030016>THj00E6>P03001>F4iH00I>F003001FN5Ih009F
N08000=NfP03001oogoo00Yoo`03001oogoo009oo`005Goo100067oo00<004XHBQP00dXH00<004IJ
AUX01DIJ00<003jn?[h013jn1`0000=K_P00Jg/016]k00<006liKcT016li00@006/HJaP000=Rn@03
001NV5jH009NV003001BN59h00EBN0030016>DHi00=6>@03000afS7J00<afP09000EVP0000016000
0EX001WL0080008f?0030016>THj00E6>P03001>F4iH00I>F003001FN5Ih0080009NfP04001NfUkJ
000<Ool00`00Oomoo`02Ool000ioo`<000Aoo`8001]oo`03001:64XH00=:60030016FTIJ00E6FP03
000n_Sjn008n_P03001O6elK00AO6`<000A[N`03001_>Fli00A_>@03001[66/H009[6003001Rn@00
00INV003001BN59h00ABN0030016>DHi00A6>@03000afS7J00<afP05000EVQFJ0001600300026M`2
00000cHl0016>P06ASX00`00CUQ>F005CUP00`00EWP00005G]X01000G]YNfP000Woo2P0017oo000?
Ool00`00Oomoo`05Ool00`00Oomoo`0HOol00`00BQQ:6003BQP00`00AUY6FP06AUX01000?[h00000
1UlK00@006LIIaUW6@800003Jg/006li00E_>@03001[66/H00A[6003001NV5jH00ENV003001BN59h
00ABN0030016>DHi00A6>@03000afS7J00<afP05000EVQFJ5IX000020AP200036M`20007ASX00`00
CUQ>F004CUP30007G]X00`00G]X00002000260000`00800P000300000goo001oo`03Ool000moo`03
001oogoo00Ioo`03001oogoo01Moo`03001:64XH00A:60030016FTIJ00E6FP8000MO6`03001W6FLI
00AW6@8000E_>@03001[66/H00E[6003001NV5jH00INV003001BN59h00=BN0d00003<MX001FJ008E
VP8000816003000Ig1WL008Ig0D00096>P03001>F4iH00@0009B^`03001NfUkJ00ENfP04001NfP00
00026004000260001000Oomoo`000goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0GOol00`00BQQ:
6004BQP01000AUY6FTIJ0`000UJI00<005lKGa/00elK00<006LIIaT01FLI00<006lH00000fli00<0
06/HJaP01F/H00<006KH00001ejH00<0059hDWP00`0000=B=`00BQL01DXG00<0047h@OP00T7h1000
0QFJ0P000P4H0P000QWL00<003YL>U`00SYL1@001e:k00<005kJG]X01EkJ00<006NLIi`00`0011P0
00D007ooOomoo`0000=oo`003goo00<007ooOol027oo0P005goo00<004XHBQP014XH00@004IJ0000
00IFV@05001O6elKGa/00007IaT01`00KaQ_66lH001_>@0000M[6004001Vf6KH0007GYP30005DSL0
0`00BQM:5`04BQL00`00@OQ1n003@OP00`00<OX0000600001@5j000Ig1WL00001SYL00<005:kD[/0
1U:k00<005kJG]X015kJ00<006NL00000VNL00<001P060000QP00P000goo00<007ooOol00Goo000>
Ool2000<Ool00`00Oomoo`0DOol00`00BQQ:6004BQP20009EYT00`00Ga/00007IaT00`00KaQ_6003
KaP00`00JaQ[6005JaP00`00I]QVf002I]P00`00GYQNV002GYP200000eYg001B=`05DSL00`00BQM:
5`04BQL00`00@OQ1n004@OP01000<OXanS7j0P0000@1F@000001NP8000036M`003YL00DjG003001B
^e:k00IB^`04001NfUkJG]X20002Ii`00`00IiaWW0050005Ool00`00Oomoo`01Ool001eoo`03001o
ogoo01Aoo`03001:64XH009:6003001FF00000YFV@03001W6FLI00EW6@03001_66lH00A_608000E[
6003001Vf6KH00=Vf005001NV5jHGYP00003FWL00`00DSMB=`04DSL00`00BQM:5`04BQL00`00@OQ1
n004@OP00`00<OXanP03<OX20000105I0001NP5j0P001SYL00<005:kD[/01U:k00@005kJG]X000EW
W0800003BkP007oo00Moo`03001oogoo005oo`007Woo00<007ooOol04goo00<004XHBQP00P000eIH
00<005JIEYT01eJI0P001FLI00<006lHKaP016lH00@006lHKaP000=[6003001Vf6KH00AVf004001N
V0000005FWL00`00DSMB=`04DSL00`00BQM:5`04BQL00`00@OQ1n004@OP00`00<OXanP02<OX00`00
6Qd0000300020GX00`00>U`jG004>U`00`00D[]B^`05D[/00`00G]X00005Ii`40009Ool00`00Oomo
o`01Ool001moo`8001=oo`03001:600000IFF003001FVEJI00EFV@04001Rf6;H0003IaT00`00KaQ_
6004KaP00`00KaQ_6002KaP00`00JaP00008I]P00`00FWMJM`05FWL00`00DSMB=`04DSL00`00BQM:
5`04BQL00`00@OQ1n004@OP00`00<OXanP03<OX010006QdJ7AXM0`0000@1NP00000000@jG003001B
^e:k00=B^`@000=WW0800004BkP007oo0009Ool00`00Oomoo`01Ool0025oo`03001oogoo015oo`03
001FF5IH00EFF003001FVEJI00AFV@03001Rf6;H009Rf003001W6@0000I_6003001_66lH00A_6003
001Vf6KH00IVf08000MJM`03001B=e8g00EB=`03001:5dXG00=:5`030011n47h00A1n003000anS7j
00@anP03000J7AXM008J7@@000036Wh0000000<0009B^`<000A;G`03001WW000009;^004001oogoo
000:Ool10001Ool1Ool0029oo`03001oogoo011oo`03001FF5IH00IFF003001FVEJI009FV@03001R
f6;H00ARf003001_66lH00A_6003001_66lH00E_6003001Vf6KH00EVf004001R^6:h0007FWL00`00
DSMB=`04DSL00`00BQM:5`04BQL01000@OQ1n47h4`0011Yn0`001T]O1P0017oo00<007ooOol01goo
0@000Goo0Goo000SOol00`00Oomoo`0@Ool00`00EUQFF005EUP01@00EYUFVEJI00001f;H0P001FlH
00<006lHKaP016lH00<006lH00001FKH00D006:hH[QR^00000MJM`03001B=e8g00AB=`04001:5dXG
BQL70003BQP00`00BUY:FP04BUX00`00@[e2_@06@[d500026Wh00`00Bem;G`04000:Ool00`00Oomo
o`06Ool10001Ool1Ool002Aoo`03001oogoo00moo`03001FF5IH00IFF003001FV@0000MRf004001Z
n6[h0003KaP00`00KaQ_6004KaP01@00KaQ_66lH00000fKH00<006:hH[P00f:h00<005YgFWL015Yg
00<0058gDSL00e8g1@000dhG00<004XHBQP01DXH00<004YJBUX014YJ00<004:m@[d00T:m0`000elL
1`003goo00<007ooOol01Goo0@000Goo0Goo000UOol2000@Ool00`00EUQFF005EUP20007H]P00`00
J_QZn002J_P00`00KaP00006KaP00`00KaQ_6003KaP00`00I]P00006H[P00`00FWMJM`05FWL00`00
DSL000020002DSL00`00CQM>5`04CQL00`00BQQ:6004BQP00`00BUY:FP05BUX01000@[d000001ElL
00<007ooOol057oo00<007ooOol01Goo0@000Goo0Goo000WOol00`00Oomoo`0=Ool00`00EUQFF004
EUP00`00GYP00007H]P00`00J_QZn003J_P00`00KaQ_6004KaP00`00KaQ_6005KaP00`00H[QR^005
H[P00`00FWMJM`04FWL30005DSL00`00CQM>5`04CQL00`00BQQ:6005BQP00`00BUY:FP04BUX20006
Ga`00`00Oomoo`0FOol00`00Oomoo`04Ool10001Ool1Ool002Qoo`03001oogoo00aoo`03001FF5IH
009FF08000=NV003001Rf6;H00=Rf003001Zn6[h00AZn08000E_6003001_66lH00A_608000QR^004
001JMeYgFWL200000eIG001B=`06DSL00`00CQM>5`04CQL00`00BQQ:6004BQP01000BUY:FTYJ0`00
00=JV@00Ga`015lL00<007ooOol067oo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0<
Ool01000EUQFF0001UjH00D006;HH]QRf00000IZn004001_>6lh0003KaP00`00KaQ_6004KaP01000
JcU[>@001f:h00@005Yg000000AFE`03001B=e8g00AB=`03001>5dhG00A>5`03001:64XH00A:6004
001:FP000005FYT01@00GaaO75lL00006goo00<007ooOol017oo0@000Goo000ZOol2000<Ool00`00
EUP00007GYP01@00H]QRf6;H00001F[h00<006lhKcP00Vlh00<006lH00001VlH00D006/iJcU[>@00
00MR^08000IFE`03001B=e8g00AB=`03001>5dhG00A>5`03001:64XH00A:608000QJV@03001O7000
01eoo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol02Woo00<005jHGYP01ejH00<006;H0000
1V[h00<006lhKcP00flh0P001FlH00<006/iJcT00f/i00@006:hH[QR^0<00003FYT005IG00EFE`03
001B=e8g00EB=`03001>5dhG00A>5`04001:64XHBQP200000eIG001JV@08FYT00`00Oomoo`0MOol0
0`00Oomoo`02Ool10001Ool002eoo`03001oogoo00Uoo`03001NV5jH00QNV003001Zn6[h00AZn003
001_>6lh00A_>003001cN@0000=_6003001[>F/i00A[>@04001R^0000005FYT00`00EUMFE`04EUL0
0`00DSMB=`04DSL00`00CQM>5`04CQL01000BQP0000015IG00<005ZIFYT01EZI00<007ooOol07goo
00D007ooOomoo`00005oo`00;Woo00<007ooOol02Goo00<005jHGYP01UjH00<006KH000016[h00<0
06lhKcP016lh00L007=iLgUcN@00KaP00007JcT20007FYT00`00EUMFE`04EUL00`00DSMB=`04DSL0
0`00CQM>5`04CQL20006EUL00`00FYUJV@04FYT00`00Oomoo`0POol01@00Oomoogoo00000Goo000_
Ool20009Ool00`00GYQNV005GYP01@00I]QVf6KH00000f[h00<006lhKcP00flh00<007=iLgT00g=i
0P001V/i00<006<l00001eZI00<005IGEUL015IG00<0058gDSL0158g00@004hGCQM>5`<00003DSL0
05IG00IFE`03001JVEZI009JV@03001oogoo029oo`04001oogoo0001Ool0035oo`03001oogoo00Mo
o`03001NV5jH00=NV003001Vf6KH00=Vf003001Zn00000E_>003001cNG=i00AcN@03001_^`0000=[
>@8000=S?003001JVEZI00EJV@03001FEeIG00AFE`03001B=e8g00AB=`<000AB=`03001FEeIG00IF
E`04001JVEZI000VOol00`00Ool00001Ool0039oo`03001oogoo00Ioo`03001NV5jH009NV003001V
f6KH00EVf08000E_>003001cNG=i00=cN@07001_^fnkKk/006/i00001F<l00<005ZIFYT01EZI00<0
05IGEUL015IG00<0058gDSL00e8g0P001e8g00<005IGEUL01EIG00<005ZI00009goo00<007ooOol0
0@00000cOol00`00Oomoo`06Ool01000GYQNV00026KH00<006mI00000flh00<007=iLgT00g=i00<0
06nkKk/00Vnk0P001f<l00<005ZIFYT015ZI00<005IGEUL015IG00<0058gDSL00`0000=B=`00DSL0
1U8g00<005IGEUL01EIG0P00:Goo0@000Goo0@00000dOol20006Ool00`00GYP00008I]P01`00KeU_
FFmI001_>00000EcN@03001_^fnk009_^`04001KoE_m0007Hc`00`00FYUJV@04FYT01000EUMFEeIG
1@001E8g00<0058gDSL0158g00<005IGEUL015IG00<007ooOol0:Woo0P00000fOol00`00Oomoo`04
Ool00`00I]QVf006I]P00`00KeU_F@03KeT20004LgT01000Kk]_^fnk0P0015_m00<006<lHc`016<l
00@005ZIFYUJV@L000=>>003001B=e8g00AB=`03001B=e8g00EB=`03001FEeIG009FE`03001oogoo
02Yoo`<00000=goo00<007ooOol00goo00<006KHI]P01FKH00<006mIKeT00fmI00<007?KLm/00P00
00=cN@00Kk/00Vnk00<005_mFod00U_m3@000dij00<004hhCSP014hh00<0058gDSL0158g00<0058g
DSL01E8g00D005IGEUMFE`0002Uoo`@000=oo`00>7oo00<007ooOol00goo00<006KHI]P00fKH00<0
06mIKeT00fmI00<007?KLm/00W?K3`0000<Bf@00A_h00dKn00<004ijCWX014ij00<004hhCSP014hh
00<0058gDSL0158g00<0058gDSL0158g00@005IGEUL002Ioo`@000Moo`00>Goo00<007ooOol00goo
00D006KHI]QVf00000E_F@03001cfg?K009cf`03001Kn@000080009oo`03001oogoo00Ioo`<000A6
oP03001>NTij00=>NP03001>>4hh00A>>003001B=e8g00AB=`03001B=e8g00EB=`8002Aoo`<000]o
o`00>Woo0P0017oo00<006KH00001FmI00@007?KLm]cf`@000Eoo`03001oogoo00Uoo`<00096oP03
001>NTij009>NP03001>>4hh00A>>003001B=e8g00EB=`03001B=e8g00=B=`80025oo`@000ioo`00
?7oo00<007ooOol00Woo0P0016mI00@007?KLm]cf`<000Moo`03001oogoo00aoo`<000E>NP03001>
>4hh00A>>003001B=e8g00AB=`04001B=e8gDSL2000OOol4000BOol003eoo`03001oogoo00=oo`80
0003KeT007?K009cf`8000Uoo`03001oogoo00moo`<0009>NP03001>>4hh00A>>003001B=e8g00AB
=`04001B=`00000MOol4000FOol003ioo`03001oogoo00Aoo`D000]oo`03001oogoo019oo`<000I>
>003001B=e8g00=B=`<001]oo`@001Yoo`00?goo0P0057oo00<007ooOol05Goo1@000Thh00<0058g
00000P006Woo10007Woo0011Ool00`00Oomoo`0@Ool00`00Oomoo`0KOol4000JOol3000ROol0049o
o`03001oogoo00moo`03001oogoo03Eoo`@002Eoo`00@goo00<007ooOol03Woo00<007ooOol0<Goo
1000:Goo0014Ool2000>Ool00`00Oomoo`0]Ool4000]Ool004Ioo`03001oogoo00]oo`03001oogoo
02Uoo`@0035oo`00Agoo00<007ooOol02Woo00<007ooOol09Goo1000=Goo0018Ool00`00Oomoo`09
Ool00`00Oomoo`0QOol4000iOol004Uoo`8000Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007oo
Ool01Woo00<007ooOol06Woo1000@7oo001<Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool0
04eoo`03001oogoo00Aoo`03001oogoo019oo`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol0
3Woo1000C7oo001?Ool20003Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001D
Ool0059oo`03001oo`0000Eoo`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol0
0;Eoo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.33 0 0 r
.3802 .65348 .40338 .65368 .43985 .67636 .41742 .66235 Metetra
.08 .544 .903 r
.41742 .66235 .43985 .67636 .47753 .70281 .45557 .67322 Metetra
.431 .58 .901 r
.45557 .67322 .47753 .70281 .51658 .72564 .4948 .68172 Metetra
.555 .56 .813 r
.4948 .68172 .51658 .72564 .55687 .7377 .53503 .68376 Metetra
.616 .559 .767 r
.53503 .68376 .55687 .7377 .59796 .73381 .57604 .67645 Metetra
.66 .574 .748 r
.57604 .67645 .59796 .73381 .63925 .71216 .61748 .65879 Metetra
.698 .605 .749 r
.61748 .65879 .63925 .71216 .68023 .67492 .65906 .63195 Metetra
.739 .656 .768 r
.65906 .63195 .68023 .67492 .72064 .62756 .70059 .59901 Metetra
.783 .74 .813 r
.70059 .59901 .72064 .62756 .76061 .57746 .74213 .5642 Metetra
.815 .877 .891 r
.74213 .5642 .76061 .57746 .80065 .53213 .78395 .53195 Metetra
.67 .973 .896 r
.78395 .53195 .80065 .53213 .84153 .49778 .8265 .50607 Metetra
.8265 .50607 .84153 .49778 .88411 .47839 .8703 .48906 Metetra
.153 0 0 r
.8703 .48906 .88411 .47839 .92917 .47524 .91584 .48173 Metetra
.229 0 0 r
.91584 .48173 .92917 .47524 .97722 .48681 .96343 .48311 Metetra
.182 .71 .705 r
.35587 .65899 .3802 .65348 .41742 .66235 .39424 .64967 Metetra
.602 .822 .984 r
.39424 .64967 .41742 .66235 .45557 .67322 .43322 .64037 Metetra
.64 .655 .85 r
.43322 .64037 .45557 .67322 .4948 .68172 .47281 .63067 Metetra
.632 .575 .772 r
.47281 .63067 .4948 .68172 .53503 .68376 .51304 .62019 Metetra
.628 .543 .738 r
.51304 .62019 .53503 .68376 .57604 .67645 .55388 .60868 Metetra
.63 .536 .73 r
.55388 .60868 .57604 .67645 .61748 .65879 .59533 .59605 Metetra
.637 .551 .741 r
.59533 .59605 .61748 .65879 .65906 .63195 .63737 .58239 Metetra
.651 .595 .779 r
.63737 .58239 .65906 .63195 .70059 .59901 .67998 .56797 Metetra
.671 .693 .862 r
.67998 .56797 .70059 .59901 .74213 .5642 .72318 .55317 Metetra
.637 .876 .986 r
.72318 .55317 .74213 .5642 .78395 .53195 .76702 .53843 Metetra
.76702 .53843 .78395 .53195 .8265 .50607 .81154 .5241 Metetra
.17 0 0 r
.81154 .5241 .8265 .50607 .8703 .48906 .85681 .51045 Metetra
.244 0 0 r
.85681 .51045 .8703 .48906 .91584 .48173 .90289 .49756 Metetra
.167 0 0 r
.90289 .49756 .91584 .48173 .96343 .48311 .94984 .48532 Metetra
.728 .988 .881 r
.33047 .6653 .35587 .65899 .39424 .64967 .37032 .63675 Metetra
.803 .842 .88 r
.37032 .63675 .39424 .64967 .43322 .64037 .41041 .60661 Metetra
.741 .689 .801 r
.41041 .60661 .43322 .64037 .47281 .63067 .45062 .5785 Metetra
.686 .6 .755 r
.45062 .5785 .47281 .63067 .51304 .62019 .49099 .55559 Metetra
.643 .549 .733 r
.49099 .55559 .51304 .62019 .55388 .60868 .53171 .53993 Metetra
.607 .521 .731 r
.53171 .53993 .55388 .60868 .59533 .59605 .57308 .53224 Metetra
.574 .513 .749 r
.57308 .53224 .59533 .59605 .63737 .58239 .6154 .53167 Metetra
.537 .532 .798 r
.6154 .53167 .63737 .58239 .67998 .56797 .65887 .53603 Metetra
.471 .6 .899 r
.65887 .53603 .67998 .56797 .72318 .55317 .70356 .54201 Metetra
.237 .683 .955 r
.70356 .54201 .72318 .55317 .76702 .53843 .74932 .54586 Metetra
.167 0 0 r
.74932 .54586 .76702 .53843 .81154 .5241 .79581 .54405 Metetra
.249 0 0 r
.79581 .54405 .81154 .5241 .85681 .51045 .84262 .5341 Metetra
.172 0 0 r
.84262 .5341 .85681 .51045 .90289 .49756 .88932 .51515 Metetra
.88932 .51515 .90289 .49756 .94984 .48532 .93568 .48821 Metetra
.886 .939 .844 r
.30427 .6669 .33047 .6653 .37032 .63675 .3457 .62193 Metetra
.857 .813 .805 r
.3457 .62193 .37032 .63675 .41041 .60661 .38707 .57434 Metetra
.796 .712 .773 r
.38707 .57434 .41041 .60661 .45062 .5785 .42807 .53115 Metetra
.73 .636 .754 r
.42807 .53115 .45062 .5785 .49099 .55559 .46876 .49816 Metetra
.662 .576 .748 r
.46876 .49816 .49099 .55559 .53171 .53993 .50949 .47914 Metetra
.588 .528 .755 r
.50949 .47914 .53171 .53993 .57308 .53224 .55081 .4753 Metetra
.499 .489 .777 r
.55081 .4753 .57308 .53224 .6154 .53167 .59327 .48519 Metetra
.377 .461 .82 r
.59327 .48519 .6154 .53167 .65887 .53603 .63732 .50483 Metetra
.181 .444 .873 r
.63732 .50483 .65887 .53603 .70356 .54201 .6831 .52814 Metetra
0 .403 .797 r
.6831 .52814 .70356 .54201 .74932 .54586 .73039 .54783 Metetra
.273 0 0 r
.73039 .54783 .74932 .54586 .79581 .54405 .77856 .55686 Metetra
.171 0 0 r
.77856 .55686 .79581 .54405 .84262 .5341 .82675 .55009 Metetra
.82675 .55009 .84262 .5341 .88932 .51515 .87413 .52575 Metetra
.53 .868 .604 r
.87413 .52575 .88932 .51515 .93568 .48821 .92019 .48597 Metetra
.889 .847 .797 r
.27768 .65887 .30427 .6669 .3457 .62193 .32047 .60375 Metetra
.867 .787 .77 r
.32047 .60375 .3457 .62193 .38707 .57434 .36301 .54557 Metetra
.828 .736 .763 r
.36301 .54557 .38707 .57434 .42807 .53115 .40486 .49332 Metetra
.771 .687 .77 r
.40486 .49332 .42807 .53115 .46876 .49816 .44604 .45432 Metetra
.689 .634 .789 r
.44604 .45432 .46876 .49816 .50949 .47914 .48699 .43319 Metetra
.569 .569 .814 r
.48699 .43319 .50949 .47914 .55081 .4753 .52837 .43141 Metetra
.392 .484 .835 r
.52837 .43141 .55081 .4753 .59327 .48519 .57094 .44723 Metetra
.156 .38 .83 r
.57094 .44723 .59327 .48519 .63732 .50483 .61531 .47575 Metetra
0 .288 .786 r
.61531 .47575 .63732 .50483 .6831 .52814 .66172 .5093 Metetra
0 .281 .738 r
.66172 .5093 .6831 .52814 .73039 .54783 .70995 .53851 Metetra
0 .429 .758 r
.70995 .53851 .73039 .54783 .77856 .55686 .75923 .55409 Metetra
.209 .731 .854 r
.75923 .55409 .77856 .55686 .82675 .55009 .80847 .54914 Metetra
.639 .964 .891 r
.80847 .54914 .82675 .55009 .87413 .52575 .85656 .52126 Metetra
.864 .987 .839 r
.85656 .52126 .87413 .52575 .92019 .48597 .90281 .47335 Metetra
.854 .773 .771 r
.25111 .638 .27768 .65887 .32047 .60375 .29471 .5813 Metetra
.858 .767 .761 r
.29471 .5813 .32047 .60375 .36301 .54557 .33806 .52148 Metetra
.845 .764 .772 r
.33806 .52148 .36301 .54557 .40486 .49332 .38064 .4678 Metetra
.809 .76 .808 r
.38064 .4678 .40486 .49332 .44604 .45432 .4224 .42774 Metetra
.723 .743 .868 r
.4224 .42774 .44604 .45432 .48699 .43319 .46378 .40602 Metetra
.531 .674 .929 r
.46378 .40602 .48699 .43319 .52837 .43141 .50546 .40416 Metetra
.206 .503 .907 r
.50546 .40416 .52837 .43141 .57094 .44723 .54824 .42036 Metetra
0 .311 .797 r
.54824 .42036 .57094 .44723 .61531 .47575 .59278 .4496 Metetra
0 .239 .74 r
.59278 .4496 .61531 .47575 .66172 .5093 .63941 .48406 Metetra
0 .325 .801 r
.63941 .48406 .66172 .5093 .70995 .53851 .68795 .5141 Metetra
.195 .535 .925 r
.68795 .5141 .70995 .53851 .75923 .55409 .73769 .53012 Metetra
.538 .735 .966 r
.73769 .53012 .75923 .55409 .80847 .54914 .78752 .52501 Metetra
.755 .819 .902 r
.78752 .52501 .80847 .54914 .85656 .52126 .83629 .49627 Metetra
.854 .834 .827 r
.83629 .49627 .85656 .52126 .90281 .47335 .88326 .44692 Metetra
.804 .712 .763 r
.22487 .6037 .25111 .638 .29471 .5813 .26849 .55438 Metetra
.832 .75 .773 r
.26849 .55438 .29471 .5813 .33806 .52148 .31205 .50227 Metetra
.846 .8 .806 r
.31205 .50227 .33806 .52148 .38064 .4678 .35505 .45506 Metetra
.828 .863 .872 r
.35505 .45506 .38064 .4678 .4224 .42774 .39739 .41906 Metetra
.709 .91 .965 r
.39739 .41906 .4224 .42774 .46378 .40602 .43941 .39833 Metetra
.35 .787 .965 r
.43941 .39833 .46378 .40602 .50546 .40416 .48168 .39415 Metetra
0 .491 .803 r
.48168 .39415 .50546 .40416 .54824 .42036 .52489 .40498 Metetra
0 .332 .74 r
.52489 .40498 .54824 .42036 .59278 .4496 .56963 .42652 Metetra
0 .353 .812 r
.56963 .42652 .59278 .4496 .63941 .48406 .61624 .45215 Metetra
.177 .46 .884 r
.61624 .45215 .63941 .48406 .68795 .5141 .66461 .47389 Metetra
.426 .552 .88 r
.66461 .47389 .68795 .5141 .73769 .53012 .71421 .48394 Metetra
.593 .613 .841 r
.71421 .48394 .73769 .53012 .78752 .52501 .76415 .47658 Metetra
.7 .658 .805 r
.76415 .47658 .78752 .52501 .83629 .49627 .81351 .44981 Metetra
.773 .699 .781 r
.81351 .44981 .83629 .49627 .88326 .44692 .86164 .40605 Metetra
.739 .656 .768 r
.19906 .55817 .22487 .6037 .26849 .55438 .24184 .52358 Metetra
.78 .731 .808 r
.24184 .52358 .26849 .55438 .31205 .50227 .28488 .48707 Metetra
.81 .845 .876 r
.28488 .48707 .31205 .50227 .35505 .45506 .32784 .45318 Metetra
.749 .963 .935 r
.32784 .45318 .35505 .45506 .39739 .41906 .37064 .42579 Metetra
.412 .859 .794 r
.37064 .42579 .39739 .41906 .43941 .39833 .41345 .40743 Metetra
.41345 .40743 .43941 .39833 .48168 .39415 .45662 .39895 Metetra
.129 0 0 r
.45662 .39895 .48168 .39415 .52489 .40498 .50059 .39933 Metetra
0 .529 .764 r
.50059 .39933 .52489 .40498 .56963 .42652 .54575 .40586 Metetra
.246 .613 .956 r
.54575 .40586 .56963 .42652 .61624 .45215 .59232 .4145 Metetra
.45 .584 .895 r
.59232 .4145 .61624 .45215 .66461 .47389 .64029 .42052 Metetra
.548 .558 .817 r
.64029 .42052 .66461 .47389 .71421 .48394 .68936 .41949 Metetra
.607 .556 .772 r
.68936 .41949 .71421 .48394 .76415 .47658 .73906 .40823 Metetra
.653 .573 .753 r
.73906 .40823 .76415 .47658 .81351 .44981 .78887 .38567 Metetra
.697 .609 .754 r
.78887 .38567 .81351 .44981 .86164 .40605 .83839 .3531 Metetra
.646 .596 .784 r
.17346 .50582 .19906 .55817 .24184 .52358 .21467 .49016 Metetra
.669 .695 .865 r
.21467 .49016 .24184 .52358 .28488 .48707 .25651 .47409 Metetra
.658 .863 .977 r
.25651 .47409 .28488 .48707 .32784 .45318 .29895 .45804 Metetra
.392 .851 .828 r
.29895 .45804 .32784 .45318 .37064 .42579 .34199 .4424 Metetra
.34199 .4424 .37064 .42579 .41345 .40743 .38566 .42743 Metetra
.041 0 0 r
.38566 .42743 .41345 .40743 .45662 .39895 .43002 .4132 Metetra
.43002 .4132 .45662 .39895 .50059 .39933 .47512 .39961 Metetra
.377 .841 .874 r
.47512 .39961 .50059 .39933 .54575 .40586 .52102 .38637 Metetra
.618 .814 .975 r
.52102 .38637 .54575 .40586 .59232 .4145 .56777 .37306 Metetra
.635 .658 .855 r
.56777 .37306 .59232 .4145 .64029 .42052 .61538 .35924 Metetra
.624 .574 .778 r
.61538 .35924 .64029 .42052 .68936 .41949 .66384 .34449 Metetra
.618 .538 .741 r
.66384 .34449 .68936 .41949 .73906 .40823 .71314 .32853 Metetra
.62 .531 .732 r
.71314 .32853 .73906 .40823 .78887 .38567 .76324 .31127 Metetra
.628 .548 .745 r
.76324 .31127 .78887 .38567 .83839 .3531 .81412 .29279 Metetra
.499 .519 .809 r
.14761 .45225 .17346 .50582 .21467 .49016 .18686 .45574 Metetra
.419 .591 .914 r
.18686 .45574 .21467 .49016 .25651 .47409 .22704 .46088 Metetra
.213 .673 .946 r
.22704 .46088 .25651 .47409 .29895 .45804 .26856 .46381 Metetra
.04 0 0 r
.26856 .46381 .29895 .45804 .34199 .4424 .3116 .46093 Metetra
.063 0 0 r
.3116 .46093 .34199 .4424 .38566 .42743 .35612 .44968 Metetra
.35612 .44968 .38566 .42743 .43002 .4132 .40183 .42919 Metetra
.404 .822 .638 r
.40183 .42919 .43002 .4132 .47512 .39961 .44837 .40051 Metetra
.751 .988 .896 r
.44837 .40051 .47512 .39961 .52102 .38637 .49538 .36633 Metetra
.805 .853 .886 r
.49538 .36633 .52102 .38637 .56777 .37306 .54264 .33039 Metetra
.743 .7 .811 r
.54264 .33039 .56777 .37306 .61538 .35924 .59016 .2966 Metetra
.683 .603 .76 r
.59016 .2966 .61538 .35924 .66384 .34449 .6382 .26832 Metetra
.634 .545 .736 r
.6382 .26832 .66384 .34449 .71314 .32853 .68713 .24776 Metetra
.595 .514 .733 r
.68713 .24776 .71314 .32853 .76324 .31127 .73742 .23563 Metetra
.561 .507 .752 r
.73742 .23563 .76324 .31127 .81412 .29279 .78946 .23103 Metetra
.254 .403 .82 r
.12082 .40294 .14761 .45225 .18686 .45574 .15818 .422 Metetra
0 .341 .82 r
.15818 .422 .18686 .45574 .22704 .46088 .19666 .4448 Metetra
0 .297 .678 r
.19666 .4448 .22704 .46088 .26856 .46381 .23711 .46391 Metetra
.247 0 0 r
.23711 .46391 .26856 .46381 .3116 .46093 .27999 .47208 Metetra
.27999 .47208 .3116 .46093 .35612 .44968 .32524 .46404 Metetra
.406 .836 .688 r
.32524 .46404 .35612 .44968 .40183 .42919 .37232 .43796 Metetra
.762 .996 .825 r
.37232 .43796 .40183 .42919 .44837 .40051 .42043 .39606 Metetra
.887 .949 .843 r
.42043 .39606 .44837 .40051 .49538 .36633 .46878 .34393 Metetra
.869 .836 .811 r
.46878 .34393 .49538 .36633 .54264 .33039 .5169 .28904 Metetra
.807 .73 .78 r
.5169 .28904 .54264 .33039 .59016 .2966 .56474 .23894 Metetra
.733 .644 .761 r
.56474 .23894 .59016 .2966 .6382 .26832 .61271 .19978 Metetra
.656 .576 .753 r
.61271 .19978 .6382 .26832 .68713 .24776 .6615 .17547 Metetra
.576 .523 .758 r
.6615 .17547 .68713 .24776 .73742 .23563 .71197 .16724 Metetra
.49 .485 .779 r
.71197 .16724 .73742 .23563 .78946 .23103 .76487 .17357 Metetra
0 .241 .757 r
.09234 .36228 .12082 .40294 .15818 .422 .12839 .39033 Metetra
0 .091 .603 r
.12839 .39033 .15818 .422 .19666 .4448 .16563 .42355 Metetra
.436 0 0 r
.16563 .42355 .19666 .4448 .23711 .46391 .20526 .45234 Metetra
0 .363 .68 r
.20526 .45234 .23711 .46391 .27999 .47208 .24798 .46717 Metetra
.293 .777 .913 r
.24798 .46717 .27999 .47208 .32524 .46404 .2938 .46094 Metetra
.729 .964 .939 r
.2938 .46094 .32524 .46404 .37232 .43796 .34203 .43116 Metetra
.875 .935 .856 r
.34203 .43116 .37232 .43796 .42043 .39606 .39155 .38086 Metetra
.901 .871 .799 r
.39155 .38086 .42043 .39606 .46878 .34393 .44127 .31757 Metetra
.885 .813 .773 r
.44127 .31757 .46878 .34393 .5169 .28904 .49046 .25113 Metetra
.843 .758 .768 r
.49046 .25113 .5169 .28904 .56474 .23894 .53899 .19122 Metetra
.779 .702 .778 r
.53899 .19122 .56474 .23894 .61271 .19978 .58732 .14553 Metetra
.687 .639 .797 r
.58732 .14553 .61271 .19978 .6615 .17547 .63633 .11882 Metetra
.562 .568 .817 r
.63633 .11882 .6615 .17547 .71197 .16724 .68708 .11257 Metetra
.409 .491 .832 r
.68708 .11257 .71197 .16724 .76487 .17357 .74056 .12493 Metetra
0 .129 .623 r
.06146 .33284 .09234 .36228 .12839 .39033 .09725 .36157 Metetra
0 .033 .52 r
.09725 .36157 .12839 .39033 .16563 .42355 .1342 .39565 Metetra
0 .153 .633 r
.1342 .39565 .16563 .42355 .20526 .45234 .17366 .42522 Metetra
.051 .472 .885 r
.17366 .42522 .20526 .45234 .24798 .46717 .21647 .44044 Metetra
.525 .737 .971 r
.21647 .44044 .24798 .46717 .2938 .46094 .2627 .43397 Metetra
.756 .804 .891 r
.2627 .43397 .2938 .46094 .34203 .43116 .31165 .40325 Metetra
.84 .805 .817 r
.31165 .40325 .34203 .43116 .39155 .38086 .36212 .35141 Metetra
.871 .798 .776 r
.36212 .35141 .39155 .38086 .44127 .31757 .4129 .28624 Metetra
.876 .792 .764 r
.4129 .28624 .44127 .31757 .49046 .25113 .46313 .21791 Metetra
.861 .788 .777 r
.46313 .21791 .49046 .25113 .53899 .19122 .5126 .15632 Metetra
.818 .779 .816 r
.5126 .15632 .53899 .19122 .58732 .14553 .56171 .10935 Metetra
.722 .75 .874 r
.56171 .10935 .58732 .14553 .63633 .11882 .61134 .08181 Metetra
.545 .673 .921 r
.61134 .08181 .63633 .11882 .68708 .11257 .66257 .07523 Metetra
.308 .54 .912 r
.66257 .07523 .68708 .11257 .74056 .12493 .71648 .08769 Metetra
.366 0 0 r
.02767 .31503 .06146 .33284 .09725 .36157 .06456 .33583 Metetra
0 .201 .658 r
.06456 .33583 .09725 .36157 .1342 .39565 .10254 .3608 Metetra
0 .363 .831 r
.10254 .3608 .1342 .39565 .17366 .42522 .14283 .38181 Metetra
.346 .529 .893 r
.14283 .38181 .17366 .42522 .21647 .44044 .18617 .39082 Metetra
.576 .614 .853 r
.18617 .39082 .21647 .44044 .2627 .43397 .23269 .38197 Metetra
.699 .66 .808 r
.23269 .38197 .2627 .43397 .31165 .40325 .2818 .35322 Metetra
.771 .695 .779 r
.2818 .35322 .31165 .40325 .36212 .35141 .33251 .30706 Metetra
.817 .731 .77 r
.33251 .30706 .36212 .35141 .4129 .28624 .38374 .24974 Metetra
.847 .773 .779 r
.38374 .24974 .4129 .28624 .46313 .21791 .43472 .18952 Metetra
.857 .822 .813 r
.43472 .18952 .46313 .21791 .5126 .15632 .48516 .13465 Metetra
.831 .875 .876 r
.48516 .13465 .5126 .15632 .56171 .10935 .53535 .09177 Metetra
.719 .898 .96 r
.53535 .09177 .56171 .10935 .61134 .08181 .58599 .06503 Metetra
.471 .814 .999 r
.58599 .06503 .61134 .08181 .66257 .07523 .63802 .05574 Metetra
.199 .632 .948 r
.63802 .05574 .66257 .07523 .71648 .08769 .69235 .06223 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo00Qoo`P000=oo`04001oogooOol3000[Ool006Qoo`<0
00Uoo`@000@oG`04000JWQZN6Yh40005Ool00`00Oomoo`09Ool4000LOol006Ioo`8000Uoo`<000To
G`03000JWQZN00@JWP03001oogoo00=oo`03001oogoo00Uoo`03001oogoo01aoo`00Hgoo0`002Goo
0P000enN0P0013mO1`000aZN0P0017oo00<007ooOol00Woo1@000goo00<007ooOol06goo0017Ool2
000HOol2000:Ool20006Gih50004A[d00`009SdV?@060004Ool00`00Oomoo`0:Ool00`00Oomoo`0J
Ool004Ioo`04001oogoo000DOol3000:Ool20007Gih00`00G_/00008A[d00`009SdV?@029Sd01000
9Sd0000017oo00<007ooOol01Woo00@007ooOol001aoo`00AWoo00@007ooOol0019oo`8000]oo`<0
00IOWP8000=Nn`030016_DJm00I6_@05000V?BHm9Sd000029Sd01000Oomoogoo0P002Goo0P007Goo
0016Ool01000Oomoo`003goo0`0000=oo`00Ool02Goo00D006]lJga[O00000=OWP8000INn`030016
_DJm00E6_@05000V?BHm9Sd000039Sd00`00Oomoo`02Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000>Ool00`00Jga[O003Jg`00`00Gih00008G_/00`00A[e6_@06A[d010009SdV?@0012Hm
00<007ooOol00Woo00<007ooOol097oo0017Ool2000;Ool3000?Ool00`00Jga[O004Jg`2000:G_/0
0`00A[e6_@02A[d700049Sd00`00Oomoo`02Ool00`00Oomoo`0TOol0059oo`<0011oo`03001[O6]l
00A[O003001[6P0000YNn`D000A6FP03000enP0000H000Aoo`03001oogoo02=oo`00Cgoo0`004Woo
00<006]lJg`016]l00D006/JJaY[6P0000MNn`800003EYT004IJ00M6FP8000DenP8000Aoo`03001o
ogoo029oo`00C7oo0`0057oo00<006]lJg`016]l00<006/JJaX00f/J00<005kkG_/00Ukk0P0015JI
00<004IJAUX01DIJ0P001SGj00<007ooOol00Woo00<007ooOol08Woo001:Ool3000EOol20005Jg`0
0`00JaY[6P05JaX01000G_/000001UJI00<004IJAUX01TIJ00<003Gj=OX01CGj00<007ooOol00Woo
00<007ooOol08Goo0017Ool3000GOol01000KeY_FP000f]l00<006/JJaX01f/J00<005JIEYT01eJI
00<004IJAUX01DIJ00<003Gj=OX01CGj00<007ooOol00goo00<007ooOol087oo0015Ool2000IOol0
0`00KeY_FP02KeX00`00Jg`0000:JaX00`00EYUFV@07EYT00`00AUY6FP05AUX00`00=OXenP06=OX0
0`00Oomoo`02Ool00`00Oomoo`0POol0049oo`<001Yoo`03001_FVmJ00A_FP03001[6V/J00Q[6P03
001Rf00000QFV@030016FTIJ00E6FP8000LenP03001oogoo00=oo`03001oogoo01moo`009Goo1000
5goo0P0077oo00<006mJKeX01FmJ00<006/JJaX01f/J00@006;HH]P000UFV@030016FTIJ00A6FP80
00PenP03001oogoo00=oo`03001oogoo01ioo`009Woo00<007ooOol057oo0`007Goo00<006mJKeX0
1FmJ00<006lh00001f/J00<006;HH]P00V;H00<005JIEYT01UJI00<004IJAUX00TIJ2P000SGj00<0
07ooOol00Woo0P0087oo000OOol50003Ool00`00Oomoo`0AOol2000OOol00`00KeY_FP05KeX01@00
KcQ_>6lh00001F/J00<006;HH]P00f;H00<005JIEYT01UJI1@000TXH00<004XH00001Cgh10001Goo
00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000POol00`00KeY_FP06KeX00`00KcQ_>002KcP0
1@00JaY[6V/J00001V;H00<005JIEYT015JI0P0000=BF000BQP014XH00@004XHBQP000Lmn003001o
ogoo00=oo`03001oogoo01eoo`009Goo00@007ooOol000eoo`80029oo`03001_FVmJ00I_FP03001_
>6lh00A_>003001[6P0000QRf004001FVEJIEYT20004DUP00`00BQQ:6003BQP00`00BQP00008?OP0
0`00Oomoo`03Ool00`00Oomoo`0LOol002Ioo`8000]oo`<002Aoo`8000I_FP03001_>6lh00I_>003
001Rf6;H00MRf004001FV@000006DUP00`00BQQ:6003BQP00`00BQP00008?OP00`00Oomoo`04Ool0
0`00Oomoo`0KOol0035oo`800003Ool007oo02=oo`04001_66lH0004KeX00`00KcQ_>007KcP00`00
H]QRf007H]P20008DUP00`00BQQ:6003BQP01000BQQ:600023gh00<007ooOol00goo00<007ooOol0
6goo000^Ool3000WOol00`00KaQ_6002KaP01@00KeY_FVmJ000026lh00<006/H00001f;H00<005jH
00002E9H00<004XHBQP00TXH00@004XHBQP000Pmn003001oogoo00=oo`8001aoo`00:goo0`00:Goo
00<006lHKaP016lH00<006mJ000026lh00@006/HJaP000IRf005001NV5jHGYP00008DUP00`00BQQ:
6002BQP01000BQQ:600023gh00<007ooOol01Goo00<007ooOol00goo0P0057oo000YOol20002Ool0
0`00Oomoo`0VOol00`00KaQ_6006KaP00`00KcQ_>006KcP00`00JaQ[6002JaP00`00H]QRf002H]P0
0`00GYQNV002GYP00`00DUQBF006DUP00`00BQQ:6002BQP01@00BQQ:64XH000023gh00<007ooOol0
17oo00<007ooOol00Woo00@007ooOol001=oo`009Woo0`00;Goo00<006lHKaP01VlH00<006lhKcP0
1Flh00<006/HJaP016/H00@006;HH]P000ENV003001BF59H00MBF005001:64XHBQP00003BQP00`00
?OPmn006?OP00`00Oomoo`05Ool01@00Oomoogoo00000Woo00<007ooOol04Goo000TOol2000_Ool0
0`00KaQ_6006KaP00`00LcP00005KcP00`00JaQ[6005JaP00`00H]P00007GYP00`00DUQBF006DUP0
1@00BQQ:64XH00000dXH00<003gh?OP00cgh1@001goo00@007ooOol0009oo`03001oogoo015oo`00
8Goo0`00<7oo00<006lHKaP01VlH00D007<hLcQc>00000=_>003001[66/H00M[6003001NV5jH00IN
V003001BF59H00IBF005001:64XHBQP00002BQP70004AQP00`00Oomoo`04Ool20002Ool01000Oomo
o`004goo000OOol2000bOol00`00KaQ_6006KaP00`00LcQc>003LcP00`00KcP00009JaP20008GYP0
0`00DUQBF007DUP00`00BQP000030002CQL00`00AQQ66006AQP00`00Oomoo`06Ool00`00Oomoo`02
000DOol001aoo`<003=oo`03001_66lH00M_6003001c>7<h00Ac>003001[66/H00M[6004001Vn6Kh
0008GYP00`00DUQBF005DUP300000dhG001>5`04CQL00`00AQQ66006AQP00`00Oomoo`06Ool00`00
Oomoo`0EOol001]oo`8003Eoo`03001_66lH00I_6003001c>7<h00Ec>003001[66/H00I[6005001V
n6KhI_P00008GYP00`00DUQBF002DUP300000e8g001>5`02CQL00`00CQM>5`02CQL00`00AQQ66006
AQP00`00Oomoo`06Ool00`00Oomoo`0EOol001Aoo`<000Aoo`03001oo`0003=oo`03001Zn00000I_
6003001c>7<h00Ec>003001cF00000I[6003001Vn6Kh00=Vn003001NV5jH00ENV003001BF0000080
00EB=`05001>5dhGCQL00003CQL00`00AQQ66006AQP00`00Oomoo`07Ool00`00Oomoo`0DOol001Eo
o`03001oogoo00=oo`04001oogoo000aOol01@00J_QZn6[h00001FlH00<007<hLcP017<h00D007=H
LeQcF00000A[6003001Vn6Kh00AVn003001NV5jH00ANV0<000QB=`03001>5dhG009>5`05001>5dhG
CQL00008AQP00`00Oomoo`06Ool2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0
000_Ool00`00J_QZn003J_P01@00KaQ_66lH00001W<h00<007=HLeP00W=H00D006/HJaQ[600000MV
n004001NV5jHGYP400000eIh001B=`07DSL00`00CQM>5`02CQL01@00CQM>5dhG000024HH00<007oo
Ool01goo00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0[Ool0
0`00J_QZn005J_P00`00KaP00007LcP00`00LeQcF003LeP00`00JaP00009I_P300000ejH001FN003
EWP00`00DSMB=`06DSL00`00CQM>5`02CQL01000CQM>5`0024HH00<007ooOol027oo00<007ooOol0
4Woo000EOol00`00Oomoo`02Ool20005Ool00`00Oomoo`0YOol00`00J_QZn007J_P00`00LcQc>005
LcP00`00LeQcF005LeP00`00I_QVn005I_P400000ejH00000005EWP00`00DSMB=`06DSL00`00CQM>
5`03CQL01000CQM>5`0024HH00<007ooOol01goo00<007ooOol04Woo000DOol20004Ool00`00Oomo
o`05Ool00`00Oomoo`0WOol00`00J_QZn008J_P00`00LcQc>004LcP00`00LeQcF006LeP00`00I_QV
n003I_P20003I_P20007EWP00`00DSMB=`06DSL00`00CQM>5`03CQL01000CQM>5`001dHH0P002Woo
00<007ooOol04Goo000JOol00`00Oomoo`06Ool00`00Oomoo`0UOol00`00J_QZn008J_P00`00KcP0
0005LcP00`00LeQcF005LeP01000KeT006Kh0`0016Kh00<005kI000025Ih00<0058gDSL01U8g00<0
04hGCQL00dhG00<004hG000014HH0`0000=B=`00Ool02Woo00<007ooOol047oo000JOol00`00Oomo
o`07Ool00`00Oomoo`0ROol2000:J_P01@00KcQ_>6lh00000g<h00<007=HLeP01G=H00<006mI0000
0P001VKh00D005kIG]UNf@0000MFN003001B=e8g00IB=`03001>5dhG00A>5`03001>5`0000<000EB
=`03001oogoo00Moo`80019oo`006Goo0P002Woo00<007ooOol087oo00<006;H00002F[h00<006lh
KcP00flh00<007<h00001g=H0`000VmI00<006KhI_P00VKh0P0015kI00<005IhEWP01UIh00<0058g
DSL01E8g00<004hGCQL014hG0`00258g00<007ooOol02Goo00<007ooOol03goo0008Ool20004Ool2
0003Ool30003Ool40009Ool00`00Oomoo`0NOol01@00H]QRf6;H00001f[h00<006lhKcP01Flh0P00
17=H0`001VmI00@006KhI_P000INf@03001FN5Ih00IFN003001B=e8g00IB=`03001>5`0000<00003
CQL0000000UB=`03001oogoo00Uoo`03001oogoo00ioo`001goo00@007ooOol000=oo`8000Ioo`04
001oogoo0002Ool30008Ool00`00Oomoo`0LOol00`00H]QRf003H]P00`00J_QZn004J_P00`00KcQ_
>005KcP00`00LgT000030002LeP00`00KeU_F@04KeT00`00I_P00008G]T00`00EWQFN006EWP00`00
DSMB=`04DSL30006CQL20008DSL00`00Oomoo`09Ool00`00Oomoo`0>Ool00003Ool0000000<00005
Ool007ooOol0000;Ool01000Oomoo`0017oo0P0027oo00<007ooOol06Woo00<006;HH]P01F;H00<0
06[hJ_P00V[h00<006lhKcP01Flh1@000g=H00<006mIKeT01FmI0P002EkI00<005IhEWP01UIh00@0
058gDSMB=`<00003CSL004hG00M>5`03001B=e8g00IB=`03001oogoo00Yoo`03001oogoo00eoo`00
1goo00@007ooOol000Qoo`<000=oo`03001oogoo00=oo`8000Qoo`8001Uoo`03001Rf6;H00IRf005
001Zn6[hJ_P00006KcP30004LgT00`00LeP00007KeT00`00Ig`0000:G]T00`00EWQFN005EWP01000
DSL000001Dhg00<004hGCQL01DhG00<0058gDSL01e8g00<007ooOol02Woo00<007ooOol00Woo1000
1Woo0007Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol01Goo00<007ooOol01goo00<0
07ooOol05Goo00<006;HH]P026;H00<006[h00001Flh0`001g=i0P001FmI0P000fMl00<005kIG]T0
1ekI00<005IhEWP015Ih0`001dhg00<004hGCQL01ThG00<0058gDSL01U8g00<007ooOol02Woo00<0
07ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool00`00Oomoo`06Ool20008Ool00`00
Oomoo`0BOol2000<H]P01000KcQ_>6lh0`0000=_>000LgT01W=i00@007?JLmX000=_F@03001WO6Ml
00=WO003001NfEkI00MNf@03001FN5Ih00@0009>F003001>=dhg00E>=`03001>5dhG00I>5`8000MB
=`8000aoo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P002Goo0P0027oo00<007ooOol047oo
00<005Zi00002f;H1@0016lh00<007=iLgT017=i00<007?JLmX00W?J00<006mI00001fMl00<005kI
G]T01UkI0`001diH00<004hgCSL01Dhg00<004hGCQL01DhG00<0058g00001E8g00<005Yh00003Goo
00<007ooOol00goo00<007ooOol017oo000HOol2000:Ool30007Ool00`00Oomoo`0=Ool20003F[T0
0`00H]QRf007H]P40005KcP00`00LgUcN@04LgT00`00LmYcfP03LmX20009Ig`00`00G]UNf@02G]T3
00000e:k001>F007CUP00`00CSM>=`05CSL00`00CQM>5`06CQL20004DSL01@00FWQJN5Yh00002goo
0P000Woo00@007ooOol000Ioo`004Woo0P0017oo00<007ooOol02Goo00<0003h00000`001Goo00<0
07ooOol02Woo0P001UZi00<006;HH]P00f;H0`0016/j00D006lhKcQ_>00000IcN@03001cfW?J00=c
fP04001So6?l0008Ig`00`00G]T000020005D[/00`00CUQ>F005CUP00`00CSM>=`06CSL00`00CQM>
5`05CQL01000DSL0058g0P0015Yh00<007ooOol02goo00<007ooOol00P001goo000AOol01000Oomo
o`000goo00<007ooOol02Woo00<0003h00000P5j0P0017oo00<007ooOol027oo00<005ZiF[T01UZi
00<006;HH]P00`0000=Rf000JcX01F/j00<006lh00001W=i00<007?JLmX00g?J00<006?lHo`00V?l
00<006MlIg`016Ml0`0025:k00<004iHCUP01TiH00<004hgCSL01Dhg00<004hGCQL01DhG00@0058g
DSL000MJN003001oogoo00]oo`03001oogoo00Qoo`004Goo00@007ooOol000=oo`03001oogoo00]o
o`030000n00000<1NPH000Eoo`@000UJ^@<000=Rf003001[>V/j00I[>P8000EcN@03001cfW?J009c
fP8000ISo003001WO6Ml00<0009?G`03001B^e:k00IB^`03001>F4iH00I>F003001>=dhg00E>=`03
001>5dhG00I>5`800003FWP005Yh00EJN003001oogoo00]oo`03001oogoo00Qoo`004Goo00@007oo
Ool0009oo`8000eoo`040000n03h00020GX00`00;Q`^7002;Q`60003BW/00`00F[UJ^@05F[T20000
0eZi001Rf002H]P00`00JcY[>P06JcX00`00Kk]_^`0300000g=i001cfP03LmX00`00HoaSo005Ho`4
0006Cel00`00D[]B^`06D[/00`00CUQ>F005CUP00`00CSM>=`06CSL00`00CQM>5`04CQL20002FWP0
0`00FWQJN004FWP00`00Oomoo`0<Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00Oomo
o`0<Ool00`000?P0n00200000`5j000^7004;Q`00`00BW]:N`05BW/01000F[UJ^EZi0`0015Zi00<0
06;H000026/j00<006nkKk/00fnk10000W?J00<006?lHo`00f?l0`0013=K00<004mOCel014mO00<0
05:kD[/01E:k00<004iHCUP01TiH00<004hgCSL01Dhg00@004hGCQM>5`<00003DUP005Yh009JN003
001JN5Yh00AJN003001oogoo00]oo`03001oogoo00Moo`004Woo0P000goo0P003goo0P0000D0n000
0001NP0000D^7003001:NdYk00E:N`<000MJ^@8000Q[>P03001_^fnk00=_^`03001SnV?j009SnPl0
0003<e/004mO00A?G`03001B^e:k00IB^`03001>F4iH00E>F003001>=dhg00E>=`04001>5`000005
DUP01@00FWQJN5Yh00001EYh00<007ooOol037oo00<007ooOol01Woo000GOol00`00Oomoo`0>Ool0
0`008IX000050004;Q`00`00BW]:N`02BW/40008F[T00`00Hc`00007JcX00`00Kk]_^`03Kk/01000
HoYSnV?j2@000d:m00<001XM00002@0000=?G`00D[/01U:k00<004iHCUP01DiH00<004hgCSL014hg
0`001e9H00D005YhFWQJN00000EJN08000ioo`03001oogoo00Eoo`005goo00<007ooOol03goo00<0
026J8IX00P0000<0U000;Q`012hL100014Yk00<005ZiF[T00eZi0P000f<l00<006/jJcX00f/j00<0
06nkKk/00fnk00@006?jHoYSnPH00003G_/004:m00E2_@04000J7AXM6Qd30000108H000NOQin1`00
0U:k00<004iHCUP01TiH00<004hg00000`000Thh00<0059HDUP01E9H00<005YhFWP00UYh00L005Yh
FWQJN000H]P0000>Ool00`00Oomoo`05Ool001Moo`03001oogoo00moo`03000QVR6J008QVP030000
U000008^70<000M:N`04001J^EZiF[T20006Hc`01@00JcY[>V/j00001Fnk00<006?jHoX00P0000=K
_P0000001Ekk00<004:m@[d014:m00<001XM6Qd00QXM10000ain00<003YL>U`00SYL3@001Thh00<0
059HDUP01U9H00<005YhFWP00UYh00<005Yh00000f;H00<007ooOol037oo00<007ooOol017oo0005
Ool20004Ool20003Ool30004Ool2000AOol010008IXQVR6J1P000RhL00<004YkBW/01DYk00<005Zi
00002F<l00<006/j00001Fnk00D006?j000006]k008000QNn`030012_D:m00A2_@03000J7AXM008J
7@8000030EL00000008NOP03000jG3YL00@jG0030016>THj00E6>P03001>>4hh00I>>003001BF59H
00EBF003001JN5Yh00=JN003001Rf6;H009Rf003001oogoo00]oo`<000Eoo`0017oo00@007ooOol0
00=oo`8000Ioo`05001oogooOol0000BOol00`008IXQVP0300000`5J000000040004BW/60009Hc`5
0002Kk/30004Jg/20007G_/;00026Qd300001@5i000001in00001SYL00<004HjASX01DHj00<004hh
CSP01Dhh00<0059HDUP01U9H00@005YhFWQJN08000ERf003001oogoo00Moo`<0009oo`03001oogoo
00=oo`0017oo00@007ooOol000]oo`03001oogoo008001=oo`8000<QVP030001FP5J00800085o0@0
00I2o`03001S?6<l00AS?08000EOgP<000E[N`04001W6FLI0004G_/200000eZI001:FP05BUX00`00
<OXanP040000104i0001N@5i0P001SYL00<004HjASX01DHj00<004hhCSP01Dhh00<0059HDUP01U9H
00@005YhFWP0009Rf003001Rf6;H009Rf003001oogoo00Aoo`<000Eoo`03001oogoo00=oo`0017oo
00@007ooOol000Qoo`<000=oo`80011oo`@000DQVP030001FP5J008000<5o0030012od;o00I2o`03
001S?6<l009S?003001OgUoN00=OgP@000E[N`03001W6FLI009W6@04001Nn`000004FYT00`00BUY:
FP04BUX00`00<OXanP02<OX400020GT20006>U`00`00ASY6>P04ASX00`00CSQ>>006CSP00`00DUQB
F005DUP00`00FWP00004H]P00`00H]QRf002H]P00`00Ool000030008Ool00`00Oomoo`03Ool000Ao
o`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool30004Ool500000b6J0001FP020EX3
00000`Gl0012o`07@_l01000Hc`000001EoN0`001V]k00<006LIIaT016LI00<005ZIFYT015ZI00<0
04YJBUX014YJ00<0037j<OX00c7j00<0016J00000P000P5i0P0013YL00<004HjASX01DHj00<004hh
CSP01Dhh00<0059HDUP01E9H0P001V;H00@006;HH]QRf08000aoo`03001oogoo00=oo`001Goo0P00
2Goo10000Woo00<007ooOol02Goo0`001goo00<003hI?QT00ShI2`0014;o20000UoN1@001F]k00<0
06LIIaT016LI0P001eZI00<004YJBUX014YJ00<0037j<OX00c7j00<0016J00000P0000<1N@005M`0
0P000SYL00<004HjASX01DHj00<004hhCSP01Dhh00<0059HDUP00e9H0P0000=J^@00H]P01F;H00@0
06;HH]P0009_FP03001oogoo00Yoo`03001oogoo009oo`005Woo00<007ooOol01Woo0`002goo00<0
03hI?QT00chI00<003IM=Ud01CIM1@001RLM10000fMl00<006li00000P0000=[N`00IaT01FLI00@0
06:hH[P000MJV@03001:FTYJ00A:FP03000anS7j008anP04000AVQ6J4IX400025M`200000cYL0016
>P06ASX00`00CSQ>>004CSP01000DUQBF59H0P0015Zi00<006;HH]P016;H0P0016mJ00<007ooOol0
2Goo00<007ooOol00Woo000EOol20005Ool3000>Ool00`00?QTn6@04?QT00`00=UdfG@06=Ud00`00
6[h000060006Ig`00`00KcU_>@03KcT00`00IaUW6@04IaT01@00H[QR^6:h00001eZI00<004YJBUX0
14YJ00<0037j<OX00c7j00H0016J4IXAVP000?L200035M`20007ASX00`00CSQ>>005CSP30006F[T0
0`00H]QRf004H]P20005KeX00`00Oomoo`08Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000B
Ool00`00?QTn6@04?QT00`00=UdfG@04=Ud40004Egl00`00IgaWO004Ig`00`00KcU_>@04KcT00`00
IaUW6@03IaT00`00H[QR^003H[P00`00FYUJV@04FYT<00000c7j000AVP034IX200000`3g000Eg003
5M`50002ASX00`00CSQ>>002CSP40009F[T00`00H]QRf002H]P00`00KeX00004KeX00`00KoX00005
0004Ool00`00Oomoo`02Ool001Eoo`@001Ioo`03000n6ChI00@n6@03000fGCIM00<000AFf`800003
Egl006Ml00EWO003001_>Fli00A_>@03001_600000=W6@03001R^6:h00ER^004001JVEZIFYT20000
0eIG001:6005BQP01000?OPmn3gh10000Q6J00@0004i0000m`80008Eg003000f?3Hl008f?0L000=:
NP03001J^EZi00QJ^@04001Rf6;H0003KeX01000KeY_FP000foj2P0017oo000>Ool30004Ool2000H
Ool00`00?QTn6@04?QT30008E]/30005Ig`00`00KcU_>@04KcT01`00KaQ_66lH001W6@0000QR^004
001JV@000004EUL00`00BQQ:6004BQP00`00?OPmn002?OP00`00<MX0000300001@4i0000005I0000
0QGL00<003Hl=S`013Hl00<004YjBWX01DYj00<005ZiF[T01eZi00<006;H00001FmJ0P000foj00<0
04=c@g<00P0017oo0P0017oo000?Ool00`00Oomoo`05Ool00`00Oomoo`0FOol01000?QTn6ChI0P00
00=BN@00E]/01eKK00@006;iH_URn@<00003Ig`006li00E_>@03001_66lH00=_6003001R^6:h00MR
^08000IFE`03001:64XH00A:6003000mn3gh00<mn004000afS7J<MX200020CT00`000ET000020007
=S`00`00BWY:NP04BWX00`00F[UJ^@07F[T20006KeX20002KoX01000@g<000001goo00<007ooOol0
0Woo000?Ool00`00Oomoo`06Ool00`00Oomoo`0EOol01000?QT00000159i00<005KKE]/00eKK0P00
1f;i0P001Fli00<006lHKaP016lH00<006:hH[P01V:h00<005jH00001UIG00<004XHBQP014XH00<0
03gh?OP00cgh00<0037J<MX00c7J0P0000@1>@000ET1F@8000Hf?003001:NTYj00E:NP03001J^EZi
00EJ^@03001SG00000E_FP04001_nP00KoX3000:Ool00`00Oomoo`01Ool000moo`03001oogoo00Mo
o`03001oogoo01Eoo`03001BNE9i00EBN@05001FfeKKE]/00009H_T00`00J_P00003KcT00`00KaQ_
6004KaP00`00J_P00006H[P01000GYQNV0001eIG00<004XHBQP014XH00<003gh?OP00cgh00<0037J
<MX00S7J00<001GK00000`0000<1F@00=S`01SHl00<004YjBWX014Yj00<005ZiF[T00eZi0P000f=L
00D006mJKeY_FP0000=_nP8000aoo`03001oogoo005oo`003goo00<007ooOol027oo0P005Goo00<0
059iDWT01U9i00<005KK00002F;i00L006[hJ_QZn000KcT00006KaP01@00J_QZn6[h000016:h00<0
05jHGYP00UjH00<005IGEUL015IG00<004XHBQP014XH00<003gh?OP00cgh00<0037J<MX00c7J00<0
01GK5M/00`0000@1F@00000000@f?003001:NTYj00E:NP03001J^EZi008000ISG003001_FP0000=_
nP03001oo`0000aoo`03001oogoo005oo`003Woo0P0037oo00<007ooOol04goo00<0059iDWT01U9i
00<006;iH_T01f;i00<006[hJ_P00f[h00<006lHKaP016lH00<006[hJ_P00V[h00D006:hH[QR^000
00ENV003001FEeIG00EFE`03001:64XH00=:6003000mn3gh00@mn003000afS7J00<afP04000EfaGK
5M/400000aXm000000030007BWX00`00F[T00008He`400001Foj001oogoo000037oo00<007ooOol0
0Goo000MOol00`00Oomoo`0BOol00`00DWUBN@05DWT00`00GYP00007H_T00`00J_QZn004J_P20005
KaP00`00J_QZn003J_P00`00H[P00006GYP00`00EUMFE`05EUL00`00BQQ:6004BQP00`00?OPmn003
?OP00`00<MXafP04<MX00`005M/Ef`025M/300046Sd:0008He`01000DmaCg5?L0P0017oo00<007oo
Ool02Goo00<007ooOol00Goo000NOol00`00Oomoo`0BOol00`00DWUBN@03DWT01@00GYQNV5jH0000
1F;i00<006[hJ_P016[h00@006lHKaP000=_6003001Zn6[h00EZn003001NV5jH00INV003001FEeIG
00AFE`04001:64XHBQPF00000aGK000000056Sd00`00A_i6oP05A_h00`00HeaSG003He`20002Dm`2
0007Ool00`00Oomoo`08Ool00`00Oomoo`01Ool001moo`80019oo`03001BNE9i009BN@03001NV5jH
00=NV005001RnF;iH_T00006J_P00`00KaQ_6002KaP00`00KaP00008J_P00`00GYQNV006GYP00`00
EUMFE`04EUL40003CQL00`00BQM:5`04BQL00`00ASU6>@04AST00`00?W`nO002?W`600036Sd00`00
A_i6oP05A_h01@00HeaSG6=L00000U?L0P002Woo00<007ooOol01goo00<007ooOol00Goo000QOol0
0`00Oomoo`0@Ool01000DWUBN@001ejH00<006;i00001f[h00<006lHKaP00flH00<006[hJ_P01V[h
00<006KH00001ejH00@005IGEUMFE`<00003DSL004hG00E>5`03001:5dXG00A:5`030016>DHi00A6
>@03000nO3il00DnO003000N_P0000@000M6oP04001SG000Dm`2000<Ool00`00Oomoo`08Ool10001
Ool1Ool0029oo`03001oogoo00moo`03001BN@0000UNV003001Zn6[h00EZn003001_66lH00A_6003
001Zn6[h00EZn004001Vf6KH0008GYP30004DSL00`00CQM>5`04CQL00`00BQM:5`04BQL00`00ASU6
>@04AST00`00?W`nO005?W`30002Dil=000?Ool00`00Oomoo`07Ool10001Ool1Ool002=oo`03001o
ogoo00moo`03001NV5jH00QNV003001Zn6[h00AZn003001_66lH00A_6003001_>00000EZn003001V
f6KH009Vf003001NV5jH00ANV08000MB=`03001>5dhG00=>5`03001:5dXG00A:5`030016>DHi00E6
>@04000nO3il?W`30003E]/00`00Dil00003Ji`5000DOol00`00Oomoo`06Ool10001Ool1Ool002Ao
o`03001oogoo00ioo`03001NV5jH00MNV003001Vf00000AZn003001_66lH00A_6005001_>6lhKcP0
0003J_P00`00I]QVf004I]P01000GYQNV5jH0P0000=FN000DSL01U8g00<004hGCQL014hG00<004XG
BQL014XG00<004HiAST014Hi00@003il000000IFf`<000=[W003001oogoo01Moo`03001oogoo00Eo
o`40005oo`5oo`009Goo0P003goo00<005jHGYP01EjH00D006KHI]QVf00000=Zn003001_66lH00=_
6003001_>6lh00=_>003001Zn00000MVf004001NV0000004EWP00`00DSMB=`04DSL00`00CQM>5`04
CQL00`00BQM:5`04BQL00`00ASU6>@03AST30007E]/01`00I_YVnVKj001[W00001Yoo`03001oogoo
00Eoo`40005oo`5oo`009goo00<007ooOol037oo00<005jHGYP015jH00<006KHI]P00fKH00<006[h
00001FlH00<006lhKcP01Flh00<006KHI]P01VKH0P001UIh00<0058gDSL01E8g00<004hGCQL014hG
00<004XGBQL00dXG00<004HiAST00`000U9h00<005KKE]/00eKK0P001FKj00<007ooOol06Woo00<0
07ooOol017oo0@000Goo0Goo000XOol00`00Oomoo`0<Ool00`00GYQNV002GYP00`00I]QVf005I]P2
0005KaP00`00KcQ_>004KcP20007I]P00`00G]T00006EWP00`00DSMB=`05DSL00`00CQM>5`04CQL0
0`00BQM:5`03BQL30006DWP01@00E]]FfeKK00001fKj00<007ooOol06goo00<007ooOol00goo0@00
0Goo0Goo000YOol00`00Oomoo`0<Ool01000GYQNV00026KH00<006lH00000flH00<006lhKcP016lh
00@006mIKeT000AVf08000=Nf@03001FN5Ih00AFN003001B=e8g00AB=`03001>5dhG00A>5`03001:
5dXG00<00003DSL0059h00MBN003001Ff`0000MVnP03001oogoo01aoo`03001oogoo00Aoo`40005o
o`00:Woo0P0037oo00<005jH000026KH00L006lHKaQ_6000KaP00006KcP00`00KeU_F@02KeT01000
I]QVf0001EkI00<005IhEWP015Ih00<0058gDSL0158g00<004hGCQL00dhG10001E8g00<0059hDWP0
1U9h00<006KjI_X01FKj00<007ooOol07Woo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0:
Ool00`00I]QVf006I]P00`00KaQ_6003KaP20005KcP00`00KeU_F@03KeT00`00I]P00006G]T00`00
EWQFN005EWP00`00DSMB=`04DSL00`00CQL000020002DSL00`00DSMB=`04DSL00`00DWQBN005DWP2
0006I_X00`00Oomoo`0POol00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00Uoo`03001Vf6KH
00EVf003001_66lH00A_6003001cN@0000=_>003001_FFmI00A_F@8000QNf@03001FN5Ih00AFN003
001B=e8g00=B=`<000EB=`03001B=e8g00EB=`03001BN59h00=BN004001NV5jH0005I_X00`00Oomo
o`0QOol01@00Oomoogoo00000Goo000^Ool00`00Oomoo`09Ool00`00I]QVf003I]P00`00KaQ_6004
KaP01`00LgUcNG=i001_>00000I_F@03001WG00000QNf@03001FN5Ih00AFN003001B=e8g00<00003
DSL0058g00IB=`03001B=e8g00AB=`03001BN59h009BN003001NV5jH009NV005001VnVKjI_X0000T
Ool01@00Oomoogoo00000Goo000_Ool2000:Ool00`00I]QVf002I]P00`00KaQ_6003KaP00`00LgUc
N@03LgT20004KeT20003Ie`00`00G]UNf@05G]T00`00EWQFN002EWP50005DSL00`00DSMB=`04DSL0
0`00DSMB=`05DSL01000DWQBN0001UjH00<006Kj00009Woo00@007ooOol0005oo`00<Goo00<007oo
Ool01goo00D006KHI]QVf00000E_6003001cNG=i00=cN@03001cfg?K00800003KeT006ML00EWG003
001NfEkI00ANf@D000=BF003001B=e8g00AB=`03001B=e8g00AB=`03001B=e8g00EB=`03001BN000
00QNV003001oogoo02Ioo`03001oo`00005oo`00<Woo00<007ooOol01goo00<006KH00001FlH00<0
07=iLgT00g=i00<007?KLm/00W?K0`001fML00<005kIG]T00`0000=BV`00DUP01E9H00<0058gDSL0
158g00<0058gDSL01E8g00<0058gDSL0158g0P002EjH00<007ooOol09Woo00<007ooOol00@00000c
Ool00`00Oomoo`07Ool20005KaP00`00LgUcN@02LgT00`00Lm]cf`02Lm/01000GoaOo5ol0P000fML
1@001E:K00<0059HDUP0159H00<0058gDSL0158g00<0058gDSL0158g00<0058gDSL00U8g0P0000=J
M`00GYP01ejH00<007ooOol0:7oo0@000Goo0@00000dOol20009Ool01@00KaQ_66lH000017=i00<0
07?KLm/00W?K00<005olGo`00eol10000dmO00<005:KDY/015:K00<0059HDUP0159H00<0058gDSL0
158g00<0058gDSL0158g00D0058gDSMB=`0000AJM`03001NV5jH00ANV003001oogoo02Yoo`800000
=Woo00<007ooOol01goo00<006lH000017=i00<007?KLm/00W?K00@005olGoaOo0@000035]H004mO
00A?G`03001BVe:K00ABV`03001BF59H00=BF003001B=e8g00AB=`03001B=e8g00EB=`<000EJM`03
001NV5jH00=NV003001oogoo02Yoo`<00000=goo00<007ooOol01goo0`000W=i00<007?KLm/00W?K
1@0017oo0`001DmO00<005:KDY/00e:K00<0059HDUP0159H00<0058gDSL0158g00<0058gDSL00e8g
0P0025Yg00D005jHGYQNV00002Uoo`@000=oo`00>7oo00<007ooOol02Goo20002Goo00<007oo0000
0`0000=?G`00DY/015:K00<0059HDUP0159H00<0058gDSL0158g00@0058gDSMB=`800003EUL005Yg
00MJM`04001NV5jH000VOol40007Ool003Uoo`03001oogoo01Uoo`04001oogooOol70003DY/00`00
DUQBF003DUP00`00DSMB=`04DSL01000DSL0000015IG00<005YgFWL01EYg00<005jH000097oo0`00
2goo000jOol2000IOol00`00Oomoo`03Ool30002=U`40006DUP01000DSMB=e8g10001UIG00<005Yg
FWL01EYg0P008Goo10003Woo000lOol00`00Oomoo`0FOol00`00Oomoo`05Ool20004=U`;0003CSP0
0`00EUMFE`05EUL00`00FWMJM`03FWL00`00Oomoo`0MOol4000BOol003eoo`03001oogoo01Eoo`03
001oogoo00Ioo`8000@fG0030016>THj00=6>P03001>>4hh00E>>003001FEeIG00AFE`03001JMeYg
009JM`03001oogoo01Yoo`@001Ioo`00?Woo00<007ooOol057oo00<007ooOol027oo00<003IL=U`0
0SIL00<004HjASX00dHj00<004hhCSP014hh00<005IGEUL01EIG00@005YgFWL001Uoo`@001Yoo`00
?goo0P0057oo00<007ooOol02Goo0P000SIL00<004HjASX00dHj00<004hhCSP014hh00<005IGEUL0
1EIG00<005Yg00005Woo10007Woo0011Ool00`00Oomoo`0@Ool00`00Oomoo`0<Ool01000=U`fG000
1DHj00<004hhCSP014hh00<005IGEUL015IG0P0057oo0`008Woo0012Ool00`00Oomoo`0?Ool00`00
Oomoo`0=Ool30005ASX00`00CSQ>>004CSP00`00EUMFE`02EUL2000BOol4000UOol004=oo`03001o
ogoo00ioo`03001oogoo00moo`030016>THj00A6>P03001>>4hh00=>>005001FEeIGEUL0000@Ool4
000YOol004Aoo`8000ioo`03001oogoo011oo`D00003ASX004hh00E>>0<000eoo`@002eoo`00AWoo
00<007ooOol02goo00<007ooOol05Goo2@002goo1000<Goo0017Ool00`00Oomoo`0:Ool00`00Oomo
o`0UOol4000eOol004Qoo`03001oogoo00Uoo`03001oogoo025oo`@003Uoo`00BGoo0P002Goo00<0
07ooOol07Woo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`0JOol40010Ool004aoo`03001o
ogoo00Eoo`03001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo
001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0
051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol20000
0goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo
0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.385 0 0 r
.37996 .65919 .40273 .67205 .43917 .70435 .41684 .68118 Metetra
.026 .407 .857 r
.41684 .68118 .43917 .70435 .47709 .73525 .45501 .70187 Metetra
.392 .533 .878 r
.45501 .70187 .47709 .73525 .51651 .75603 .49451 .71469 Metetra
.571 .581 .824 r
.49451 .71469 .51651 .75603 .55707 .7596 .53512 .71439 Metetra
.666 .613 .786 r
.53512 .71439 .55707 .7596 .59813 .74248 .57636 .69838 Metetra
.728 .646 .766 r
.57636 .69838 .59813 .74248 .63904 .70582 .61771 .66752 Metetra
.777 .686 .764 r
.61771 .66752 .63904 .70582 .67935 .65515 .65882 .62589 Metetra
.818 .74 .778 r
.65882 .62589 .67935 .65515 .719 .59875 .6996 .57967 Metetra
.848 .814 .816 r
.6996 .57967 .719 .59875 .7584 .54557 .7403 .53567 Metetra
.841 .911 .883 r
.7403 .53567 .7584 .54557 .79829 .50335 .78148 .49995 Metetra
.686 .963 .94 r
.78148 .49995 .79829 .50335 .83959 .47748 .82384 .47678 Metetra
.277 .78 .844 r
.82384 .47678 .83959 .47748 .88324 .47036 .86811 .46809 Metetra
0 .527 .728 r
.86811 .46809 .88324 .47036 .92991 .48123 .91484 .47326 Metetra
0 .442 .78 r
.91484 .47326 .92991 .48123 .97986 .50616 .96423 .48915 Metetra
.111 .622 .894 r
.35628 .65051 .37996 .65919 .41684 .68118 .39403 .65543 Metetra
.465 .634 .928 r
.39403 .65543 .41684 .68118 .45501 .70187 .43271 .65948 Metetra
.565 .573 .82 r
.43271 .65948 .45501 .70187 .49451 .71469 .47238 .65953 Metetra
.609 .552 .765 r
.47238 .65953 .49451 .71469 .53512 .71439 .51292 .65312 Metetra
.64 .556 .743 r
.51292 .65312 .53512 .71439 .57636 .69838 .55414 .6391 Metetra
.67 .577 .742 r
.55414 .6391 .57636 .69838 .61771 .66752 .59578 .61783 Metetra
.703 .619 .761 r
.59578 .61783 .61771 .66752 .65882 .62589 .63766 .59119 Metetra
.743 .695 .806 r
.63766 .59119 .65882 .62589 .6996 .57967 .67971 .56206 Metetra
.781 .831 .89 r
.67971 .56206 .6996 .57967 .7403 .53567 .72204 .53377 Metetra
.682 .968 .927 r
.72204 .53377 .7403 .53567 .78148 .49995 .76492 .50937 Metetra
.76492 .50937 .78148 .49995 .82384 .47678 .80871 .49108 Metetra
.142 0 0 r
.80871 .49108 .82384 .47678 .86811 .46809 .85377 .47993 Metetra
.227 0 0 r
.85377 .47993 .86811 .46809 .91484 .47326 .90039 .47558 Metetra
.094 0 0 r
.90039 .47558 .91484 .47326 .96423 .48915 .94869 .47636 Metetra
.664 .899 .979 r
.33167 .64378 .35628 .65051 .39403 .65543 .37071 .62808 Metetra
.687 .709 .865 r
.37071 .62808 .39403 .65543 .43271 .65948 .41022 .61244 Metetra
.657 .601 .781 r
.41022 .61244 .43271 .65948 .47238 .65953 .4502 .59763 Metetra
.636 .551 .742 r
.4502 .59763 .47238 .65953 .51292 .65312 .4907 .58424 Metetra
.624 .531 .729 r
.4907 .58424 .51292 .65312 .55414 .6391 .53178 .57253 Metetra
.618 .533 .736 r
.53178 .57253 .55414 .6391 .59578 .61783 .57352 .56242 Metetra
.618 .561 .768 r
.57352 .56242 .59578 .61783 .63766 .59119 .61599 .55345 Metetra
.622 .635 .842 r
.61599 .55345 .63766 .59119 .67971 .56206 .65923 .54491 Metetra
.592 .793 .976 r
.65923 .54491 .67971 .56206 .72204 .53377 .70324 .53596 Metetra
.228 .747 .77 r
.70324 .53596 .72204 .53377 .76492 .50937 .74798 .52578 Metetra
.147 0 0 r
.74798 .52578 .76492 .50937 .80871 .49108 .79338 .51374 Metetra
.223 0 0 r
.79338 .51374 .80871 .49108 .85377 .47993 .83935 .49955 Metetra
.149 0 0 r
.83935 .49955 .85377 .47993 .90039 .47558 .88584 .48331 Metetra
.88584 .48331 .90039 .47558 .94869 .47636 .93285 .46546 Metetra
.84 .882 .872 r
.30621 .63622 .33167 .64378 .37071 .62808 .34683 .60033 Metetra
.78 .731 .807 r
.34683 .60033 .37071 .62808 .41022 .61244 .38741 .56583 Metetra
.717 .632 .762 r
.38741 .56583 .41022 .61244 .4502 .59763 .4279 .53707 Metetra
.662 .569 .74 r
.4279 .53707 .4502 .59763 .4907 .58424 .46844 .51717 Metetra
.614 .529 .735 r
.46844 .51717 .4907 .58424 .53178 .57253 .50936 .50757 Metetra
.566 .507 .748 r
.50936 .50757 .53178 .57253 .57352 .56242 .55106 .5078 Metetra
.511 .506 .787 r
.55106 .5078 .57352 .56242 .61599 .55345 .5939 .51555 Metetra
.426 .539 .868 r
.5939 .51555 .61599 .55345 .65923 .54491 .63807 .52693 Metetra
.218 .6 .949 r
.63807 .52693 .65923 .54491 .70324 .53596 .68353 .53716 Metetra
.15 0 0 r
.68353 .53716 .70324 .53596 .74798 .52578 .73 .54137 Metetra
.24 0 0 r
.73 .54137 .74798 .52578 .79338 .51374 .77697 .53575 Metetra
.148 0 0 r
.77697 .53575 .79338 .51374 .83935 .49955 .82391 .51843 Metetra
.82391 .51843 .83935 .49955 .88584 .48331 .8704 .49004 Metetra
.591 .923 .705 r
.8704 .49004 .88584 .48331 .93285 .46546 .91632 .45352 Metetra
.874 .83 .801 r
.28009 .62501 .30621 .63622 .34683 .60033 .32228 .57336 Metetra
.824 .741 .773 r
.32228 .57336 .34683 .60033 .38741 .56583 .36403 .52442 Metetra
.762 .669 .759 r
.36403 .52442 .38741 .56583 .4279 .53707 .40516 .48513 Metetra
.692 .607 .757 r
.40516 .48513 .4279 .53707 .46844 .51717 .4459 .46032 Metetra
.609 .552 .765 r
.4459 .46032 .46844 .51717 .50936 .50757 .48677 .45218 Metetra
.504 .499 .784 r
.48677 .45218 .50936 .50757 .55106 .5078 .5284 .46 Metetra
.361 .448 .816 r
.5284 .46 .55106 .5078 .5939 .51555 .57141 .48024 Metetra
.151 .399 .846 r
.57141 .48024 .5939 .51555 .63807 .52693 .61616 .50686 Metetra
0 .35 .8 r
.61616 .50686 .63807 .52693 .68353 .53716 .66266 .53206 Metetra
.264 0 0 r
.66266 .53206 .68353 .53716 .73 .54137 .71045 .5477 Metetra
.153 0 0 r
.71045 .5477 .73 .54137 .77697 .53575 .75875 .54718 Metetra
.75875 .54718 .77697 .53575 .82391 .51843 .80663 .52723 Metetra
.586 .916 .686 r
.80663 .52723 .82391 .51843 .8704 .49004 .85336 .48896 Metetra
.867 .995 .799 r
.85336 .48896 .8704 .49004 .91632 .45352 .8987 .43752 Metetra
.87 .789 .769 r
.2535 .60783 .28009 .62501 .32228 .57336 .29696 .54806 Metetra
.844 .752 .762 r
.29696 .54806 .32228 .57336 .36403 .52442 .33972 .49176 Metetra
.8 .717 .773 r
.33972 .49176 .36403 .52442 .40516 .48513 .38154 .44711 Metetra
.729 .676 .798 r
.38154 .44711 .40516 .48513 .4459 .46032 .42266 .41972 Metetra
.609 .616 .833 r
.42266 .41972 .4459 .46032 .48677 .45218 .46367 .41208 Metetra
.414 .522 .859 r
.46367 .41208 .48677 .45218 .5284 .46 .50535 .42341 Metetra
.145 .388 .839 r
.50535 .42341 .5284 .46 .57141 .48024 .54845 .44962 Metetra
0 .265 .771 r
.54845 .44962 .57141 .48024 .61616 .50686 .59349 .48365 Metetra
0 .243 .729 r
.59349 .48365 .61616 .50686 .66266 .53206 .64052 .51625 Metetra
0 .387 .792 r
.64052 .51625 .66266 .53206 .71045 .5477 .6891 .53756 Metetra
.223 .696 .939 r
.6891 .53756 .71045 .5477 .75875 .54718 .73828 .53947 Metetra
.645 .933 .969 r
.73828 .53947 .75875 .54718 .80663 .52723 .78696 .51793 Metetra
.853 .955 .877 r
.78696 .51793 .80663 .52723 .85336 .48896 .83422 .47431 Metetra
.913 .908 .803 r
.83422 .47431 .85336 .48896 .8987 .43752 .87976 .415 Metetra
.848 .757 .762 r
.22664 .58345 .2535 .60783 .29696 .54806 .27073 .52489 Metetra
.849 .767 .772 r
.27073 .52489 .29696 .54806 .33972 .49176 .31414 .46966 Metetra
.829 .781 .807 r
.31414 .46966 .33972 .49176 .38154 .44711 .35657 .42566 Metetra
.766 .791 .874 r
.35657 .42566 .38154 .44711 .42266 .41972 .39821 .39834 Metetra
.592 .755 .956 r
.39821 .39834 .42266 .41972 .46367 .41208 .43962 .39014 Metetra
.236 .582 .944 r
.43962 .39014 .46367 .41208 .50535 .42341 .48159 .40026 Metetra
0 .341 .803 r
.48159 .40026 .50535 .42341 .54845 .44962 .52489 .42476 Metetra
0 .238 .73 r
.52489 .42476 .54845 .44962 .59349 .48365 .57005 .45676 Metetra
0 .304 .788 r
.57005 .45676 .59349 .48365 .64052 .51625 .61722 .48732 Metetra
.171 .485 .9 r
.61722 .48732 .64052 .51625 .6891 .53756 .66601 .50694 Metetra
.488 .657 .935 r
.66601 .50694 .6891 .53756 .73828 .53947 .71557 .5078 Metetra
.699 .745 .887 r
.71557 .5078 .73828 .53947 .78696 .51793 .76483 .48602 Metetra
.809 .78 .827 r
.76483 .48602 .78696 .51793 .83422 .47431 .8129 .44292 Metetra
.862 .797 .785 r
.8129 .44292 .83422 .47431 .87976 .415 .85941 .38464 Metetra
.81 .728 .774 r
.19965 .55196 .22664 .58345 .27073 .52489 .24351 .50379 Metetra
.835 .788 .807 r
.24351 .50379 .27073 .52489 .31414 .46966 .28699 .45791 Metetra
.834 .868 .869 r
.28699 .45791 .31414 .46966 .35657 .42566 .3298 .42049 Metetra
.738 .94 .951 r
.3298 .42049 .35657 .42566 .39821 .39834 .37206 .39587 Metetra
.396 .842 .927 r
.37206 .39587 .39821 .39834 .43962 .39014 .41419 .38602 Metetra
0 .548 .754 r
.41419 .38602 .43962 .39014 .48159 .40026 .4568 .39029 Metetra
0 .378 .706 r
.4568 .39029 .48159 .40026 .52489 .42476 .50055 .40549 Metetra
0 .39 .813 r
.50055 .40549 .52489 .42476 .57005 .45676 .54588 .4262 Metetra
.173 .476 .895 r
.54588 .4262 .57005 .45676 .61722 .48732 .59297 .44552 Metetra
.413 .539 .875 r
.59297 .44552 .61722 .48732 .66601 .50694 .64156 .45628 Metetra
.564 .58 .828 r
.64156 .45628 .66601 .50694 .71557 .5078 .69105 .45273 Metetra
.661 .616 .793 r
.69105 .45273 .71557 .5078 .76483 .48602 .74064 .43203 Metetra
.731 .655 .774 r
.74064 .43203 .76483 .48602 .8129 .44292 .78965 .39514 Metetra
.787 .703 .771 r
.78965 .39514 .8129 .44292 .85941 .38464 .83775 .34655 Metetra
.743 .694 .804 r
.17251 .51482 .19965 .55196 .24351 .50379 .21524 .48411 Metetra
.779 .81 .877 r
.21524 .48411 .24351 .50379 .28699 .45791 .25812 .45425 Metetra
.743 .949 .946 r
.25812 .45425 .28699 .45791 .3298 .42049 .30099 .42834 Metetra
.438 .873 .797 r
.30099 .42834 .3298 .42049 .37206 .39587 .34388 .40865 Metetra
.34388 .40865 .37206 .39587 .41419 .38602 .387 .39623 Metetra
.074 0 0 r
.387 .39623 .41419 .38602 .4568 .39029 .43068 .39073 Metetra
.003 0 0 r
.43068 .39073 .4568 .39029 .50055 .40549 .47526 .39045 Metetra
.288 .699 .973 r
.47526 .39045 .50055 .40549 .54588 .4262 .52102 .39259 Metetra
.484 .635 .921 r
.52102 .39259 .54588 .4262 .59297 .44552 .56805 .39371 Metetra
.558 .572 .824 r
.56805 .39371 .59297 .44552 .64156 .45628 .61626 .39043 Metetra
.599 .549 .77 r
.61626 .39043 .64156 .45628 .69105 .45273 .6654 .38007 Metetra
.632 .552 .747 r
.6654 .38007 .69105 .45273 .74064 .43203 .7151 .36138 Metetra
.665 .577 .746 r
.7151 .36138 .74064 .43203 .78965 .39514 .76502 .33477 Metetra
.702 .625 .767 r
.76502 .33477 .78965 .39514 .83775 .34655 .815 .30227 Metetra
.613 .638 .852 r
.14505 .47451 .17251 .51482 .21524 .48411 .18592 .46482 Metetra
.595 .791 .974 r
.18592 .46482 .21524 .48411 .25812 .45425 .2276 .45468 Metetra
.378 .84 .891 r
.2276 .45468 .25812 .45425 .30099 .42834 .27016 .44326 Metetra
.27016 .44326 .30099 .42834 .34388 .40865 .31361 .42991 Metetra
.005 0 0 r
.31361 .42991 .34388 .40865 .387 .39623 .35794 .41434 Metetra
.35794 .41434 .387 .39623 .43068 .39073 .40307 .39662 Metetra
.418 .863 .807 r
.40307 .39662 .43068 .39073 .47526 .39045 .44892 .37725 Metetra
.679 .889 .975 r
.44892 .37725 .47526 .39045 .52102 .39259 .49546 .35697 Metetra
.684 .715 .872 r
.49546 .35697 .52102 .39259 .56805 .39371 .54267 .33667 Metetra
.651 .603 .788 r
.54267 .33667 .56805 .39371 .61626 .39043 .5906 .31718 Metetra
.627 .547 .745 r
.5906 .31718 .61626 .39043 .6654 .38007 .63933 .29912 Metetra
.613 .525 .731 r
.63933 .29912 .6654 .38007 .7151 .36138 .68898 .28277 Metetra
.607 .527 .739 r
.68898 .28277 .7151 .36138 .76502 .33477 .73967 .26803 Metetra
.609 .559 .773 r
.73967 .26803 .76502 .33477 .815 .30227 .79149 .25442 Metetra
.344 .512 .88 r
.11694 .43396 .14505 .47451 .18592 .46482 .15563 .4446 Metetra
.123 .561 .921 r
.15563 .4446 .18592 .46482 .2276 .45468 .19573 .45403 Metetra
.094 0 0 r
.19573 .45403 .2276 .45468 .27016 .44326 .23769 .45729 Metetra
.078 0 0 r
.23769 .45729 .27016 .44326 .31361 .42991 .2816 .45045 Metetra
.2816 .45045 .31361 .42991 .35794 .41434 .32721 .43161 Metetra
.442 .845 .656 r
.32721 .43161 .35794 .41434 .40307 .39662 .37403 .4014 Metetra
.769 .995 .865 r
.37403 .4014 .40307 .39662 .44892 .37725 .4215 .36286 Metetra
.842 .893 .876 r
.4215 .36286 .44892 .37725 .49546 .35697 .46919 .32064 Metetra
.787 .746 .816 r
.46919 .32064 .49546 .35697 .54267 .33667 .51692 .27989 Metetra
.718 .639 .769 r
.51692 .27989 .54267 .33667 .5906 .31718 .56483 .24528 Metetra
.656 .568 .744 r
.56483 .24528 .5906 .31718 .63933 .29912 .6133 .2201 Metetra
.602 .522 .737 r
.6133 .2201 .63933 .29912 .68898 .28277 .66289 .20584 Metetra
.552 .499 .75 r
.66289 .20584 .68898 .28277 .73967 .26803 .71415 .20199 Metetra
.502 .502 .789 r
.71415 .20199 .73967 .26803 .79149 .25442 .76747 .20607 Metetra
0 .268 .774 r
.08776 .39599 .11694 .43396 .15563 .4446 .12447 .42214 Metetra
0 .2 .642 r
.12447 .42214 .15563 .4446 .19573 .45403 .16305 .44684 Metetra
.319 0 0 r
.16305 .44684 .19573 .45403 .23769 .45729 .20434 .46176 Metetra
.039 0 0 r
.20434 .46176 .23769 .45729 .2816 .45045 .24858 .4601 Metetra
.42 .861 .778 r
.24858 .4601 .2816 .45045 .32721 .43161 .29538 .43849 Metetra
.773 .997 .857 r
.29538 .43849 .32721 .43161 .37403 .4014 .34388 .39808 Metetra
.891 .948 .839 r
.34388 .39808 .37403 .4014 .4215 .36286 .39307 .34421 Metetra
.886 .853 .805 r
.39307 .34421 .4215 .36286 .46919 .32064 .44214 .28482 Metetra
.838 .763 .78 r
.44214 .28482 .46919 .32064 .51692 .27989 .49071 .22846 Metetra
.77 .682 .766 r
.49071 .22846 .51692 .27989 .56483 .24528 .53895 .18248 Metetra
.69 .611 .763 r
.53895 .18248 .56483 .24528 .6133 .2201 .58746 .15192 Metetra
.599 .548 .769 r
.58746 .15192 .6133 .2201 .66289 .20584 .6371 .13902 Metetra
.496 .495 .786 r
.6371 .13902 .66289 .20584 .71415 .20199 .68876 .143 Metetra
.378 .456 .815 r
.68876 .143 .71415 .20199 .76747 .20607 .74314 .16016 Metetra
0 .055 .571 r
.05707 .36271 .08776 .39599 .12447 .42214 .09257 .39637 Metetra
.482 0 0 r
.09257 .39637 .12447 .42214 .16305 .44684 .1302 .42859 Metetra
0 .271 .662 r
.1302 .42859 .16305 .44684 .20434 .46176 .17106 .44926 Metetra
.221 .696 .939 r
.17106 .44926 .20434 .46176 .24858 .4601 .21557 .45002 Metetra
.689 .916 .971 r
.21557 .45002 .24858 .4601 .29538 .43849 .26328 .42671 Metetra
.852 .905 .871 r
.26328 .42671 .29538 .43849 .34388 .39808 .31313 .38076 Metetra
.89 .858 .803 r
.31313 .38076 .34388 .39808 .39307 .34421 .36379 .31877 Metetra
.887 .815 .771 r
.36379 .31877 .39307 .34421 .44214 .28482 .41418 .25047 Metetra
.861 .776 .766 r
.41418 .25047 .44214 .28482 .49071 .22846 .46378 .1861 Metetra
.812 .735 .78 r
.46378 .1861 .49071 .22846 .53895 .18248 .51269 .13429 Metetra
.73 .685 .807 r
.51269 .13429 .53895 .18248 .58746 .15192 .56161 .10083 Metetra
.604 .617 .838 r
.56161 .10083 .58746 .15192 .6371 .13902 .61153 .08825 Metetra
.431 .527 .855 r
.61153 .08825 .6371 .13902 .68876 .143 .66355 .09568 Metetra
.24 .433 .85 r
.66355 .09568 .68876 .143 .74314 .16016 .71854 .11887 Metetra
.488 0 0 r
.02447 .33522 .05707 .36271 .09257 .39637 .06005 .36673 Metetra
0 .12 .606 r
.06005 .36673 .09257 .39637 .1302 .42859 .09777 .39678 Metetra
0 .398 .848 r
.09777 .39678 .1302 .42859 .17106 .44926 .13879 .41564 Metetra
.451 .655 .947 r
.13879 .41564 .17106 .44926 .21557 .45002 .1836 .41527 Metetra
.699 .744 .886 r
.1836 .41527 .21557 .45002 .26328 .42671 .2318 .39164 Metetra
.802 .765 .819 r
.2318 .39164 .26328 .42671 .31313 .38076 .28234 .34617 Metetra
.847 .773 .779 r
.28234 .34617 .31313 .38076 .36379 .31877 .33385 .28518 Metetra
.866 .781 .766 r
.33385 .28518 .36379 .31877 .41418 .25047 .3852 .21805 Metetra
.865 .791 .776 r
.3852 .21805 .41418 .25047 .46378 .1861 .43576 .15467 Metetra
.84 .802 .814 r
.43576 .15467 .46378 .1861 .51269 .13429 .48558 .10337 Metetra
.767 .8 .88 r
.48558 .10337 .51269 .13429 .56161 .10083 .53527 .06978 Metetra
.604 .75 .947 r
.53527 .06978 .56161 .10083 .61153 .08825 .58581 .05635 Metetra
.345 .614 .95 r
.58581 .05635 .61153 .08825 .66355 .09568 .63827 .06222 Metetra
.119 .458 .887 r
.63827 .06222 .66355 .09568 .71854 .11887 .69357 .08327 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo00L000aoo`04001oogooOol3000[Ool006Qoo`<00003
Ool000000080009>oP03000^ORin008^OPH000Ioo`03001oogoo00=oo`03001oogoo00Uoo`@001ao
o`00IWoo0P0000Aoo`00000000M>oP03000^ORin00D^OP03000=g000008000Aoo`03001oogoo00=o
o`03001oogoo00Uoo`03001oogoo01aoo`00Hgoo0`000Woo00<006<l00002Dkn00<002in;Wh01Bin
00@000gL3M`=g0<00003Ool007oo00Aoo`03001oogoo009oo`D000=oo`03001oogoo01]oo`00Agoo
0P0067oo0P0017oo00D006<lHcaS?00000U>oP03000^ORin00D^OP03000=g0gL00<=g08000Ioo`03
001oogoo00Yoo`03001oogoo01Yoo`00AWoo00@007ooOol001Aoo`<000Aoo`8000ES?003001>oTkn
00E>oPP0008^OP03000=g0gL008=g08000Moo`03001oogoo00Ioo`04001oogoo000LOol004Ioo`04
001oogoo000BOol20006Ool00`00HcaS?006Hc`00`00C_i>oP040002CWX00`00=Q/f6`03=Q/50003
3M`00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001oogoo000?Ool300000goo001oo`04Ool0
0`00HcaS?007Hc`30007CWX00`00=Q/f6`05=Q/01@007K/000003M`00P001goo00<007ooOol09Goo
0016Ool01000Oomoo`003Goo0P002Goo00<006<lHc`01f<l00<005ji000024ij00<003HK=Q/01SHK
00<001fk7K/00`0027oo00<007ooOol097oo0017Ool2000;Ool3000:Ool00`00JcX00007Hc`01@00
G[UN^Eji000024ij00<003HK=Q/01CHK00@001fk7K/M^`<000Moo`03001oogoo02Aoo`00DWoo0`00
2goo00D006/jJcY[>P0000ES?003001N^Eji009N^@03001>NTij00I>NP03000f6cHK00Hf6`04000M
^afk00027K/00`00Oomoo`05Ool00`00Oomoo`0SOol004moo`<000eoo`03001[>V/j00=[>P05001S
?6<lHc`00006G[T00`00CWY>NP06CWX00`00=Q/f6`05=Q/010007K/M^`000Qfk00<007ooOol01Woo
00<007ooOol08Woo001<Ool3000?Ool00`00JcY[>P05JcX00`00Hc`00007G[T00`00CWY>NP06CWX0
0`00=Q/f6`06=Q/00`007K/000037K/00`00Oomoo`05Ool00`00Oomoo`0ROol004Yoo`<0011oo`03
001[>V/j00M[>P03001N^Eji00MN^@03001>NTij00A>NPP0008f6`03000M^`0000<M^`03001oogoo
00Ioo`03001oogoo025oo`00Agoo0`000Woo00<007ooOol03Goo00<006/jJcX01f/j0P002Eji00<0
04ijCWX010000Thh00<003gi?OT00cgi1@0011fk00<007ooOol01Woo00<007ooOol087oo0015Ool2
000DOol00`00JcY[>P07JcX01000I_QVn00025ji0`001dhh00<003gi?OT01Cgi1@000Qfk00<007oo
Ool01Goo00<007ooOol087oo0012Ool3000EOol00`00JcY[>P07JcX01@00I_QVn6Kh00001eji00<0
05Yh000024hh00<003gi?OT01Sgi00@0037J<MXafP<000Qoo`03001oogoo01moo`009Goo10005goo
0P0067oo0P001f/j00<006KhI_P00fKh00<005jiG[T00Uji0P000eYh00<004hhCSP01Dhh00<003gi
?OT01Sgi00<0037J<MX0137J00<007ooOol01Woo00<007ooOol07Woo000VOol00`00Oomoo`0DOol3
000IOol01000KcQ_>0001F/j00<006KhI_P01FKh00@005jiG[T000EJN003001>>4hh00I>>003000m
nCgi00Dmn@8000DafP03001oogoo00Eoo`80021oo`007goo1@000goo00<007ooOol04Goo0P006goo
00<006lhKcP00Vlh00D006/jJcY[>P0000QVn003001N^@0000MJN003001>>4hh00E>>003000mnCgi
00Dmn@8000HafP03001oogoo00Ioo`03001oogoo01eoo`00:7oo00<007ooOol03Goo0`0077oo00<0
06lhKcP016lh00<006/j00002VKh00<005YhFWP01UYh00<004hhCSP01Dhh00<003gi?OT01Cgi0P00
1S7J00<007ooOol01Woo00<007ooOol07Goo000UOol01000Oomoo`003Goo0P007Woo00<006lhKcP0
1Vlh00<006KhI_P026Kh0P002EYh00<004hhCSP01Dhh00<003gi?OT01Cgi0P001S7J00<007ooOol0
1Woo00<007ooOol077oo000VOol2000;Ool3000POol00`00KcQ_>006KcP00`00I_QVn007I_P01000
H[QR^00025Yh00<004hhCSP01Dhh00<003gi?OT01Cgi0P001c7J00<007ooOol01Woo00<007ooOol0
6goo000aOol200000goo001oo`0OOol00`00KcQ_>006KcP00`00KaP00007I_P01@00H[QR^6:h0000
25Yh00<004hhCSP01Dhh00<003gi?OT00cgi2@000S7J00<007ooOol01Woo00<007ooOol06goo000^
Ool3000SOol00`00KcQ_>006KcP01000KaQ_60001VKh00<006:hH[P00V:h00<005YhFWP01eYh00<0
04hhCSP00dhh1`000TGg00<004Gg0000148I10001goo0P0077oo000[Ool3000UOol00`00KcQ_>006
KcP00`00KaQ_6002KaP00`00I_QVn002I_P00`00H[QR^004H[P00`00FWQJN006FWP00`00CSQ>>003
0002CQL00`00AOM5m`04AOL01000AOM5m`001T8I00<007ooOol01goo00<007ooOol00goo0P0057oo
000YOol20002Ool00`00Oomoo`0ROol00`00KcQ_>006KcP00`00KaQ_6004KaP01000I_QVn0001f:h
00<005YhFWP01UYh0`001ThG00<004GgAOL00dGg00@004GgAOL000M26@03001oogoo00Ioo`03001o
ogoo009oo`04001oogoo000COol002Ioo`<002Uoo`8000M_>003001_66lH00A_6003001Vn00000UR
^003001JN5Yh00=JN0800003DUL004hG00I>5`030015mdGg00=5m`040015mdGg0007@QT00`00Oomo
o`07Ool01@00Oomoogoo00000Woo00<007ooOol04Goo000TOol2000[Ool01000KaQ_60001Flh00<0
06lHKaP01VlH00<006:hH[P026:h00@005YhFWQJN08000=BE`03001>5dhG00E>5`030015mdGg00=5
m`050015mdGgAOL00007@QT00`00Oomoo`07Ool01000Oomoo`000Woo00<007ooOol04Goo000QOol3
000/Ool00`00KaQ_6002KaP01@00KcQ_>6lh000026lH0P002V:h00@005Yh000000IBE`03001>5dhG
00E>5`030015mdGg0095m`050015mdGgAOL00007@QT00`00Oomoo`06Ool20002Ool01000Oomoo`00
4goo000OOol2000^Ool00`00KaQ_6004KaP00`00KcP00008KaP01000JaQ[60002F:h0P00259G00<0
04hGCQL01DhG00@004GgAOM5m`8000=5m`0300126D8I00I26@03001oogoo00Moo`03001oogoo0080
01Aoo`0077oo0`00<7oo00<006lHKaP01FlH00<006lHKaP01VlH00D006/HJaQ[600000QR^003001J
V00000QBE`03001>5dhG00I>5`<00003AOL004Gg00=5m`0300126D8I00E26@03001oogoo00Qoo`03
001oogoo01Eoo`006goo0P00<Goo00<006lHKaP01VlH00<006lHKaP01FlH00<006/HJaP00f/H00<0
06:hH[P016:h00D005ZHFYQJV00000QBE`03001>5dhG009>5`@000A5m`050015mdGgAOL00006@QT3
0009Ool00`00Oomoo`0EOol001Aoo`<000Aoo`03001oo`0002moo`03001_66lH00I_6003001cF000
00E_6003001[66/H00A[6003001R^6:h009R^08000AJV003001BEe9G00IBE`03001>5`00008000A>
5`030015mdGg0095m`040015mdGg0002@QT40002CSP00`00Oomoo`08Ool00`00Oomoo`0DOol001Eo
o`03001oogoo00=oo`04001oogoo000^Ool00`00KaQ_6005KaP01@00LeQcF7=H00000flH00<006/H
JaP01V/H00@006:hH[P000IJV003001BEe9G00EBE`<000M>5`030015mdGg00=5m`D000M>>003001o
ogoo00Moo`8001Eoo`003Woo1@000Woo00<007ooOol00goo00D007ooOomoo`0002aoo`03001_66lH
00E_6003001cF7=H00=cF003001_600000U[6003001R^00000QJV004001BEe9GDUL30002DUL00`00
CQM>5`05CQL00`00AOM5m`040002CQL00`00CSQ>>006CSP00`00Oomoo`08Ool00`00Oomoo`0COol0
01Eoo`03001oogoo009oo`8000Aoo`03001oogoo02Qoo`03001_66lH00I_6003001cF7=H00AcF003
001[66/H00Q[608000UJV0@000EBE`03001>5dhG00E>5`<000A>5`04001>5dhG0008CSP00`00Oomo
o`08Ool00`00Oomoo`0BOol001Eoo`03001oogoo009oo`8000Eoo`03001oogoo02Ioo`03001_66lH
00I_6003001cF7=H00EcF003001[66/H00M[6003001VnP0000MJV0<000QBE`03001>5dhG009>5`<0
0003CQL004hG00E>5`03001>5`0000U>>003001oogoo00Moo`03001oogoo019oo`0057oo0P0017oo
00<007ooOol01Goo00<007ooOol09Goo0P001VlH00<007=HLeP01G=H00<007=i00001f/H00D006Kj
I_YVnP0000AJV08000=JV003001BEe9G00IBE`@000A>5`03001>5dhG00A>5`03001>5`0000U>>003
001oogoo00Qoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol08goo00@006lHKaP0
00E_6003001cF7=H00AcF005001cNG=iLgT00005JaP00`00I_YVnP02I_X00`00FYP000020005FYP0
0`00DUMBE`05DUL20008CQL00`00CQM>5`04CQL00`00CQL00008CSP2000:Ool00`00Oomoo`0@Ool0
01Yoo`03001oogoo00Moo`03001oogoo025oo`03001_66lH009_6005001_66lHKaP00006LeP00`00
LgUcN@03LgT01@00JaQ[66/H000016Kj0`0025ZH00<0059GDUL00U9G0`0000=B=`00CQL01dhG00<0
04hGCQL01DhG0P001dhh00<005Yh00002Goo0P004Woo000IOol2000:Ool00`00Oomoo`0OOol00`00
KaQ_6004KaP00`00KaP00006LeP00`00LgUcN@05LgT00`00JaP00002I_X30002I_X00`00FYQJV007
FYP40004DSL00`00CQM>5`06CQL00`00CQM>5`06CQL20004CSP20003FWP00`00Oomoo`08Ool00`00
Oomoo`0?Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000]oo`03001oogoo01eoo`03001_66lH00I_
6003001cF7=H00EcF003001cNG=i00EcN@D000EVnP03001JV5ZH00IJV08000QB=`03001>5dhG00I>
5`03001>5dhG00E>5`80009>>08000EJN003001oogoo00Uoo`03001oogoo00ioo`001goo00@007oo
Ool000=oo`8000Ioo`04001oogoo000=Ool00`00Oomoo`0KOol00`00KaQ_6007KaP20005LeP00`00
LgUcN@05LgT30009I_X00`00FYQJV002FYP300000e9i001B=`07DSL00`00CQM>5`06CQL00`00CQM>
5`06CQL00`00CSP00008FWP00`00Oomoo`08Ool00`00Oomoo`0>Ool00003Ool0000000<00005Ool0
07ooOol0000;Ool01000Oomoo`003Woo00<007ooOol06Goo00<006lHKaP01flH00@007=iLgT000=c
F003001cNG=i00=cN@<000=[W003001VnVKj00IVnP03001JV000008000EBN@03001B=e8g00EB=`03
001>5dhG00I>5`03001>5dhG00I>5`8000UJN003001oogoo00Uoo`03001oogoo00eoo`001goo00@0
07ooOol000Qoo`<000=oo`8000ioo`8001Qoo`03001_66lH00M_6003001cNG=i009cN@03001cF000
00=cN@<00003LgT006^L00A[W003001VnVKj00EVnP8000QBN@03001B=e8g00IB=`03001>5dhG00I>
5`03001>5dhG00A>5`03001FE`0000UJN003001oogoo00Yoo`03001oogoo009oo`@000Ioo`001goo
00@007ooOol000Qoo`03001oogoo00=oo`03001oo`0000moo`03001oogoo01=oo`8000U_6003001c
NG=i00AcN@800003LgT0000000=cN@03001[W6^L00A[W003001VnVKj009VnP<00003E]/0059i00MB
N@03001B=e8g00IB=`03001>5dhG00I>5`03001>5dhG009>5`8000=FE`03001JN5Yh00MJN003001o
ogoo00Uoo`03001oogoo00=oo`03001oogoo00Ioo`0027oo0P002Goo10000Woo00@007ooOol000mo
o`03001oogoo015oo`03001W6P0000Q_6003001cNG=i00EcN@@000=cN@03001[W6^L00I[W004001V
nP000005E]/00`00DWUBN@06DWT00`00DSMB=`05DSL00`00CQM>5`06CQL00`00CQM>5`020005EUL2
0008FWP00`00Oomoo`0:Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qoo`8000=oo`03001oogoo
00eoo`03001oogoo00moo`05001W6VLJIaX00007KaP00`00LgUcN@02LgT30004LmX00`00LgT00009
Ji`20007E]/00`00DWUBN@06DWT00`00DSMB=`06DSL00`00CQM>5`06CQL20007EUL00`00FWP00006
FWP00`00I]P0000<Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qoo`8000Aoo`8000ioo`03001o
ogoo00eoo`03001W6VLJ00=W6P03001_66lH00=_6003001cNG=i00<00003LgT007?J00AcfP<000I[
W0<00003Eil005KK00MFf`03001BNE9i00EBN@03001B=e8g00IB=`03001>5dhG00E>5`8000UFE`80
00EJN005001Vf6KHI]P0000:Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomoo`04
Ool2000>Ool00`00Oomoo`0;Ool00`00IaYW6P05IaX01000KaQ_66lH10000g=i00<007?JLmX00g?J
00@006?kHo]Sn`800004Ji`000000005Eil00`00E]]Ff`05E]/00`00DWUBN@06DWT00`00DSMB=`05
DSL00`00CQM>5`02CQL300000e8g001FE`08EUL00`00FWP00003FWP00`00I]QVf002I]P00`00Oomo
o`0:Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`05Ool2000>Ool00`00
Oomoo`09Ool00`00IaYW6P07IaX40005LgT00`00LmYcfP03LmX00`00Ho]Sn`03Ho/30008Eil00`00
E]]Ff`04E]/00`00DWUBN@06DWT00`00DSMB=`06DSL40005DSL00`00EUMFE`07EUL01@00FWP005Yh
00001VKH00<007ooOol02Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol01Woo
0`003Goo00<007ooOol01goo00<006LJIaX01VLJ1@001G=i00<007?JLmX00W?J0P000f?k0`000cMi
0`001ENO00<005KKE]/01EKK00<0059iDWT01E9i00<0058gDSL00U8g1@00258g00<005IGEUL01eIG
00@005YhFWP000QVf003001oogoo00Uoo`03001oogoo00Qoo`004Goo00@007ooOol0009oo`8000Uo
o`<000eoo`03001oogoo00Aoo`8000MW6P800003IaX006nK009_V`<00003LgT007?J00=cfP04001S
nf?kHo/@00000eNO001Ff`05E]/00`00DWUBN@02DWT80004CSP00`00DSMB=`06DSL00`00EUMFE`07
EUL01000FWP0000026KH00<007ooOol02Woo00<007ooOol01goo000AOol01000Oomoo`000goo00<0
07ooOol027oo100037oo00@007ooOomoo`800003F_`006LJ00=W6P<0009W6P03001_VfnK00A_V`@0
009cfP03001Sn`0000D000=;7P03000NGP0000D0008Vg`d000A6FP03001>>4hh00I>>003001B=e8g
00IB=`03001FEeIG00IFE`80009Vf003001Vf6KH00AVf003001_>@0000]oo`03001oogoo00Moo`00
4Woo0P000goo0P002Woo00<0004H00000`002goo0`0015[l100016LJ00<006nKKi/016nK00<006?k
Ho/00V?k1`001d/N00@001iN7UhNGP@000<Vg`03000nWCjM00@nW@030016FTIJ00I6FP03001>>4hh
00E>>003001B=e8g00IB=`03001FEeIG00EFE`03001NV00000=Vf003001Vf6KH009Vf005001_>Fli
KcT0000;Ool00`00Oomoo`06Ool001Moo`03001oogoo00Yoo`05000160000001V`020008Ool20004
F_`40005IaX00`00Ki]_V`04Ki/00`00Ho]Sn`02Ho/300000eoN001S>`02Hc/00`00Bai;7P05Bah0
1P007UhNGQiN0001F@<0008Vg`03000nWCjM00@nW@030016FTIJ00E6FP03001>>4hh00E>>003001B
=e8g00IB=`03001FEeIG00AFE`05001NV5jHGYP00002I]P01@00I]QVf6KH000016li00<007ooOol0
2Woo00<007ooOol01Goo000GOol00`00Oomoo`0;Ool010000AP160000P6K20000e[l0`0015[l00D0
06LJIaYW6P0000I_V`04001Snf?kHo/300000eoN00000005Hc/00`00Bai;7P02Bah600000aiN0001
F@060005?Yd00`00AUY6FP05AUX00`00CSQ>>006CSP00`00DSMB=`06DSL01000EUMFEeIG0P0015jH
00L006KHI]QVf000I]P00006KcT00`00Oomoo`09Ool00`00Oomoo`05Ool001Moo`03001oogoo00ao
o`060001604H0001V`001SZN100025[l00<006LJ00001VnK00@006?kHo]Sn`80009OgP03001S>f<k
00ES>`D000=>NP03000f6cHK00<00081F@030001VP6J008000DnW@030016FTIJ00E6FP03001>>4hh
00E>>003001B=e8g00IB=`04001FEeIG0006GYP00`00I]QVf002I]P20006KcT00`00Oomoo`0:Ool0
0`00Oomoo`04Ool000Eoo`8000Aoo`8000=oo`<000Aoo`8000ioo`030001604H00<000<jWP@000YJ
o0<000E_V`05001Snf?kHo/00002Jg/30006Hc/00`00G[T00007CWX00`00=Q/f6`02=Q/200000`5I
000000020IX20003?Yd00`00AUY6FP05AUX00`00CSQ>>005CSP00`00DSMB=`06DSL00`00EUL00008
GYP01000I]QVf6KH0P001Fli00<007Ni00002Woo0`001Goo0004Ool01000Oomoo`000goo0P001Woo
00D007ooOomoo`00011oo`<000030AP00000008000@jWP03001Jo5[l00EJo08000=K_`<00003Ki/0
06?k00<000=[N`04001[6F/I0003Hc/20003G[T00`00CWY>NP04CWX00`00=Q/f6`02=Q/00`004IX0
000300000`6J000Eo00300000cjM0016FP05AUX00`00CSQ>>006CSP00`00DSMB=`05DSL20009GYP0
1P00I]QVf000KcT000A_>@05001g^GNiMkT00006Ool30002Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo000;Ool00`00Oomoo`02000@Ool00`00;Q`^70040006>Yh00`00F_aJo002F_`20007Fkl50004
Jg/00`00JaU[6@02JaT00`00Hc/00006G[T00`00CWY>NP04CWX00`00=Q/f6`02=Q/00`004IXAVP05
00035O`20007AUX00`00CSQ>>005CSP00`00DSMB=`02DSL300000eIi001NV008GYP00`00I]P00003
KcT01@00KcU_>Fli000017Ni0P0000Aoo`00000000Eoo`03001oogoo00=oo`0017oo00@007ooOol0
00Qoo`<000=oo`80011oo`<000<^708000HjWP04001Jo0000006Fkl60005Jg/00`00JaU[6@04JaT0
0`00G[UN^@05G[T00`00CWY>NP04CWX00`00=Q/f6`03=Q/00`004IXAVP0200000`3g000000045O`3
0004AUX00`00CSQ>>005CSP00`00DSL000020005EWT00`00GYQNV006GYP20005KcT00`00KcT00005
MkT00`00KoT00008Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00
Oomoo`0<Ool300000goo000^7005;Q`;0003Fkl500000eoN001[N`04Jg/00`00JaU[6@04JaT20008
G[T01000CWY>NTij200000<f6`004IX00Q6J0P0000<0m`005O`00QGl00<003Hk=S/01@0014hh1@00
25Ii00<005jHGYP01EjH00<006LJ00001Vli00<007NiMkT00gNi00<006oiKoT00P001goo00<007oo
Ool00goo0005Ool20009Ool40002Ool00`00Oomoo`09Ool30005Ool00`00;Q`^7005;Q`300047]h7
0002Hc`30004Jg/00`00JaU[6@04JaT01000I]QVf0001eji10000dhh00<0047i@OT00T7i100000DA
VP000AP0003g0080008Eo003000f>cHk00<f>`D000A:FP03001FNEIi00MFN@03001NV5jH00=NV005
001W6VLJIaX00004KcT20004MkT01000KoU_n@000T^f00<007ooOol01Goo00<007ooOol00Woo000F
Ool00`00Oomoo`06Ool30009Ool00`00;Q`^7003;Q`800000dlo00000006Hc`00`00JcU[>@020000
0f]k001[6@05JaT01@00I]QVf6KH00001Eji0P0000=JN000CSP01Dhh00<0047i@OT00d7i00<002gJ
00000P0000L160000001>@005O`00006=S/00`00BUY:FP06BUX00`00EWUFN@06EWT00`00GYQNV002
GYP00`00IaYW6P03IaX01`00KcU_>Fli001g^@00009g^@04001_nFoi0003BkH30005Ool00`00Oomo
o`02Ool001Eoo`8000Eoo`<000aoo`03000^72hL00<000A>V`05001??dloCcl00007Hc`00`00JcU[
>@03JcT20005JaT00`00I]QVf003I]P01@00G[UN^Eji000015Yh00<004hhCSP00dhh00<0047i@OT0
147i00<002gJ;MX00P000P4H00<0004i00000P001SHk00<004YJBUX01DYJ00<005IiEWT01eIi00@0
05jHGYP000MW6P03001_>@0000=g^@800004KoT00000BkH70004Ool00`00Oomoo`02Ool001Eoo`04
001oogooOol3000@Ool20008CY/00`00Ccl00007Hc`00`00JcU[>@04JcT00`00KaP00003JaT00`00
I]QVf004I]P01000G[T000001EYh00<004hhCSP014hh00<0047i@OT00d7i00@002gJ;MX]fP800081
60030001>@4i008000Df>`03001:FTYJ00I:FP03001FNEIi00IFN@03001NV00000QW6P<000=g^@04
001_n@00BkH:0003Ool00`00Oomoo`02Ool001Eoo`@001=oo`03001>VdjK00M>V`<000IS?003001[
>F/i00A[>@07001_66lHKaP006/I000026KH00<005YhFWP01EYh00<004hhCSP014hh00<0047i@OT0
147i00<002gJ;MX00RgJ0P0000D160000001>@0000Hf>`03001:FTYJ00E:FP03001FNEIi00IFN@80
00QW6P08001_g6oLKm`007Ni001_n@<000Eoo`H0009oo`03001oogoo009oo`003Woo0`0017oo0P00
5Woo00<004jKCY/01DjK00@005kIG]UNf@8000=S?003001[>F/i00A[>@03001_66lH00=_6003001V
f6KH00MVf003001JN5Yh00IJN003001>>4hh00=>>0030011nD7i00A1n@03000]fRgJ008]fP03000A
V`0000<000030CT0000000@f>`03001:FTYJ00E:FP03001FNEIi00AFN@800003F_`006LJ00IW6P03
001_g6oL009_g0H000Uoo`8000040MP007oo0004Ool000moo`03001oogoo00Eoo`03001oogoo01Ao
o`03001>VdjK009>V`8000INf@03001S?00000I[>@03001_66lH00A_6003001Vf6KH00IVf003001R
^00000MJN003001>>4hh00A>>0030011nD7i00A1n@03000]fRgJ008]fP03000AVa6K00P0008f>`03
001:FTYJ00E:FP04001FNEIiEWT20004F_`00`00IaYW6P03IaX00`00Kma_g002Km`00`00BkD00004
Ool00`00Oomoo`08Ool40004Ool000moo`03001oogoo00Ioo`03001oogoo01=oo`05001>VdjKCY/0
0009G]T00`00JcU[>@04JcT00`00KaQ_6004KaP00`00KaP00006I]P01000H[QR^0001eYh00<004hh
CSP014hh00<0047i@OT0147i00<002gJ;MX00bgJ00<0016K4I/00P0000<0m`0000000QGl0`001dYJ
00@005Ii000000IJo003001W6VLJ009W6P03001_g6oL009_g0<000Eoo`03001oogoo00Yoo`<000=o
o`003goo00<007ooOol01goo00<007ooOol04goo00<004jK00002UkI0P001F/i00<006lHKaP00flH
00D006lHKaQ_600000AVf003001R^6:h009R^003001JN5Yh00EJN004001>>4hhCSP>00000bgJ000A
V`034I/200000`3g000Eo0035O`90009F_`01000IaYW6P000foL0`0027oo00<007ooOol02goo00<0
07ooOol00Goo000?Ool00`00Oomoo`08Ool2000DOol00`00G]UNf@08G]T01000I_QVn0000f/i00<0
06lHKaP00flH00<006lHKaP00VlH00D006KHI]QVf00000ER^003001JN5Yh00AJN0D000=>5`03001:
5dXG00A:5`0400126D8I@QT500024I/500025O`00`00?[dn_@05?[d00`00F_aJo007F_`00`00IaX0
0003Km`00`00BkD0000:Ool00`00Oomoo`0:Ool00`00Oomoo`01Ool000ioo`8000aoo`03001oogoo
015oo`03001NfEkI00MNf@03001Vn6Kh009Vn003001[>@0000E_6003001_66lH00A_6003001Vf000
00IR^003001JN5Yh00=JN003001FE`0000M>5`03001:5dXG00A:5`0300126D8I00926@03000f>cHk
00@000030GT00000008000035O`003jm00Hn_@03001Jo5[l00IJo0@00004Km`00000000;Ool00`00
Oomoo`0:Ool00`00Oomoo`01Ool001eoo`03001oogoo015oo`03001NfEkI00ENf@03001Vn6Kh00AV
n08000E_6003001_66lH00E_6003001R^6:h00IR^003001JN5Yh008000=FE`03001>5dhG00A>5`03
001:5dXG00A:5`0300126D8I00=26@03000f>cHk008f>`D000040IT000000007?[d00`00F_aJo002
F_`30003Dkl3000>Ool00`00Oomoo`09Ool00`00Oomoo`01Ool001ioo`03001oogoo015oo`03001N
fEkI00=Nf@03001Vn6Kh00EVn003001_600000=_6003001_66lH00E_608000QR^003001JN00000EF
E`03001>5dhG00A>5`03001:5dXG00A:5`0300126D8I00A26@03000f>cHk00@f>`L000Hn_@03001J
o000008000=C_`<0019oo`03001oogoo00Qoo`03001oogoo005oo`007goo0P004Woo00D005kIG]UN
f@0000MVn007001_66lHKaP006lH00001flH00@006[hJ_P000MR^08000IFE`03001>5dhG00E>5`03
001:5dXG00A:5`0300126D8I00=26@03000f>cHk00Df>`03000JOQYn00d0009C_`8001Ioo`03001o
ogoo00Moo`03001oogoo005oo`008Goo00<007ooOol047oo00<005kI00001fKh00<006lHKaP00flH
00<006lHKaP01FlH00<006[hJ_P00V[h00<006:hH[P00f:h00<005ZH00001eIG00<004hGCQL014hG
00<004XGBQL014XG00<0048I@QT0148I00<003Hk=S/00SHk1@0014/o2`0067oo00<007ooOol027oo
0@000Goo0Goo000ROol00`00Oomoo`0@Ool00`00I_QVn005I_P00`00KaQ_6004KaP20005KaP00`00
J_QZn003J_P01000H[QR^6:h0P000eZH00<005IGEUL015IG00<004hGCQL01DhG00<004XGBQL00dXG
00<0048I@QT00d8I1P0014jJ00D004/oBcm;?`0000ASG003001_FVmJ00=_FP03001oogoo01Moo`03
001oogoo00Moo`40005oo`5oo`008goo00<007ooOol03goo00<006KhI_P016Kh00<006lHKaP016lH
00@006lhKcP000=_6003001Zn6[h00EZn003001R^00000EJV003001FEeIG00AFE`03001>5dhG00E>
5`03001:5dXG00A:5`0300126@000080009>>003001>VTjJ00I>VP03001;?`0000ASG003001_FVmJ
00=_FP03001oogoo01Uoo`03001oogoo00Ioo`40005oo`5oo`0097oo00<007ooOol03goo00<006Kh
I_P00fKh00<006lHKaP00flH00<006lhKcP00Vlh00<006lH000026[h0P001eZH00<005IGEUL015IG
00<004hGCQL014hG00@004XGBQM:5`D000I>>003001>VTjJ00E>VP<000=SG003001_FVmJ00=_FP03
001oogoo01]oo`03001oogoo00Eoo`40005oo`5oo`009Goo0P0047oo00D006KhI_QVn00000I_6003
001_>6lh00=_>003001Zn6[h00IZn003001Rn@0000MJV003001FEeIG00AFE`03001>5dhG00=>5`D0
00=>=`03001>>4hh00E>>003001>VTjJ00=>VP8000=Nf@03001SG00000E_FP03001oogoo01aoo`03
001oogoo00Eoo`40005oo`5oo`009goo00<007ooOol03Woo00<006Kh00001VlH00<006lhKcP016lh
00<006[hJ_P01F[h00D006;iH_URn@0000IJV003001FEeIG00AFE`03001>5dhG00<0009>5`03001>
=dhg00A>=`03001>>4hh00E>>005001>VTjJCYX00006G]T00`00KeY_FP03KeX00`00Oomoo`0NOol0
0`00Oomoo`04Ool10001Ool1Ool002Qoo`03001oogoo00ioo`03001_66lH00A_6003001_>6lh00A_
>003001_F@0000EZn003001RnF;i009Rn@03001JV5ZH00EJV003001FEeIG00=FE`<000E>5`03001>
=dhg00A>=`03001>>4hh00E>>004001>VP000007G]T20003KeX00`00Oomoo`0POol00`00Oomoo`03
Ool10001Ool1Ool002Uoo`03001oogoo00ioo`03001_66lH009_6003001_>6lh00A_>005001_FFmI
KeT00003J_P00`00H_URn@03H_T00`00FYQJV005FYP00`00EUMFE`0300000e8g001>5`06CQL00`00
CSM>=`04CSL00`00CSQ>>004CSP20008G]T01P00J_QZn000KeX002=oo`03001oogoo00Aoo`40005o
o`00:Woo0P003goo00D006lHKaQ_600000I_>003001_FFmI009_F@03001Zn00000MRn@03001JV5ZH
00AJV0<000EB=`03001>5dhG00A>5`03001>=dhg00A>=`03001>>4hh00=>>003001JN00000MNf@03
001Zn6[h009Zn003001oogoo02=oo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol03Goo00<0
06lH00001Vlh00<006mIKeT00fmI0P0026;i00@005ZHFYQJV0<00003EWP0058g00EB=`03001>5dhG
00A>5`03001>=dhg00E>=`03001>>4hh008000=JN003001NfEkI00=Nf@03001Zn6[h00=Zn003001o
ogoo02Aoo`03001oogoo009oo`40005oo`00;Goo00<007ooOol03Goo0P001Flh00<006mIKeT00fmI
00@006^KJi/000MRn@04001JV0000004EWP00`00DSMB=`04DSL00`00CQM>5`05CQL00`00CSM>=`04
CSL00`00CSP00006FWP00`00G]UNf@02G]T00`00J_QZn002J_P00`00Oomoo`0VOol01@00Oomoogoo
00000Goo000^Ool00`00Oomoo`0>Ool20003KcP00`00KeU_F@02KeT00`00Ji][V`02Ji/00`00H_UR
n@04H_T20007EWP00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CSM>=`04CSL20007FWP01@00G]UN
fEkI000016[h00<007ooOol09goo00D007ooOomoo`00005oo`00;goo0P0047oo00<006lh000016mI
00<006^KJi/016^K00<006;iH_T00`0000=Ff`00EWP01UIh00<0058gDSL0158g00<004hGCQL014hG
00<004hgCSL00Thg0P0000=FE`00FWP01eYh00<005kI00001F[h00<007ooOol0:7oo00@007ooOol0
005oo`00<Goo00<007ooOol03Woo0`000VmI00<006^KJi/01F^K0`001EKK00<005IhEWP015Ih00<0
058gDSL0158g00<004hGCQL014hG00D004hgCSM>=`0000AFE`03001JN5Yh00EJN08000EZn003001o
ogoo02Yoo`03001oo`00005oo`00<Woo00<007ooOol047oo0`001F^K0P000UNO00<005KKE]/00eKK
00<005IhEWP01EIh00<0058gDSL0158g00<004hGCQL014hG0`001EIG00<005YhFWP015Yh00<006:h
000016[h00<007ooOol0:goo00<007ooOol00@00000cOol00`00Oomoo`0BOol200000f^K00000005
Eil00`00E]]Ff`03E]/00`00EWQFN004EWP00`00DSMB=`04DSL00`00CQM>5`03CQL20007EUL00`00
FWQJN003FWP01@00H[QR^6:h00000f[h00<007ooOol0;7oo0@000Goo0@00000dOol2000DOol30006
Eil00`00E]]Ff`03E]/00`00EWQFN003EWP00`00DSMB=`04DSL00`00CQM>5`0300000e8g001FE`07
EUL01@00FWQJN5Yh00001F:h00<006[h0000<7oo0P00000fOol00`00Oomoo`0DOol;00000eKK001F
N005EWP01000DSMB=e8g1@001E8g00<005IGEUL01EIG00@005YhFWP000MR^003001oogoo02ioo`<0
0000=goo00<007ooOol05goo0P0000<>O000>Yd00SZM30000dhh00<0058gDSL0158g00<005IGEUL0
1EIG00<005Yh000026:h00<007ooOol0:Woo10000goo000hOol00`00Oomoo`0HOol00`003W`00004
>Yd00`00BUY:FP03BUX00`00CSQ>>004CSP00`00DSMB=`05DSL00`00EUMFE`04EUL20008H[P00`00
Oomoo`0WOol40007Ool003Uoo`03001oogoo01Qoo`<000@jW@03001:FTYJ00=:FP03001>>4hh00A>
>003001B=e8g00AB=`03001FEeIG00=FE`03001NV00000MR^003001oogoo02Eoo`<000]oo`00>Woo
0P006Goo0P0013ZM00<004YJBUX00dYJ00<004hhCSP014hh00<0058gDSL0158g00<005IGEUL00UIG
00D005jHGYQNV00000ER^003001oogoo029oo`@000ioo`00?7oo00<007ooOol05Woo00@007oo0000
00<jW@03001:FTYJ00=:FP03001>>4hh00=>>003001B=e8g00EB=`04001FEeIG0005GYP00`00H[QR
^002H[P00`00Oomoo`0NOol4000BOol003eoo`03001oogoo01Eoo`04001oogooOol200000cZM001:
FP04BUX00`00CSQ>>004CSP00`00DSMB=`04DSL00`00EUL00006GYP01@00H[QR^6:h00007Goo1000
5Woo000nOol00`00Oomoo`0DOol00`00Oomoo`02Ool30005BUX00`00CSQ>>004CSP00`00DSMB=`04
DSL20008GYP00`00H[P0000JOol4000JOol003moo`8001Aoo`03001oogoo00=oo`D000=:FP03001>
>4hh00=>>004001B=e8gDSL300000eIi001NV007GYP2000GOol4000NOol0045oo`03001oogoo011o
o`03001oogoo00Ioo`03000b?38l00@000I>>0<000EFN@03001NV5jH00ANV003001oogoo01Aoo`<0
029oo`00@Woo00<007ooOol03goo00<007ooOol01goo00<0038l<S`00S8l20001eIi00<005jHGYP0
0ejH00<007ooOol04Goo10009Goo0013Ool00`00Oomoo`0>Ool00`00Oomoo`08Ool00`00<S`b?002
<S`00`00BUY:FP03BUX00`00EWUFN@06EWT01@00GYQNV5jH000047oo1000:Goo0014Ool2000>Ool0
0`00Oomoo`09Ool00`00<S`b?002<S`00`00BUY:FP03BUX00`00EWUFN@05EWT01000GYQNV0003Goo
1000;Goo0016Ool00`00Oomoo`0;Ool00`00Oomoo`0:Ool20002<S`00`00BUY:FP03BUX00`00EWUF
N@05EWT00`00GYP0000:Ool4000aOol004Moo`03001oogoo00Yoo`03001oogoo00aoo`800003<S`0
04YJ00A:FP03001FNEIi00AFN@8000Moo`@003Eoo`00B7oo00<007ooOol02Goo00<007ooOol03Woo
0P001TYJ00@005IiEWUFN@8000Eoo`@003Uoo`00BGoo0P002Goo00<007ooOol047oo1@0000=:FP00
EWT00P0017oo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`0EOol30002Ool40010Ool004ao
o`03001oogoo00Eoo`03001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo
1000B7oo001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo
00Yoo`@0051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001C
Ool200000goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eo
o`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02454 .26244 -12 -2.74006 ]
[.02454 .26244 0 6.25994 ]
[.0159 .31657 -24 -2.78582 ]
[.0159 .31657 0 6.21418 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00232 .43075 -18 -2.88287 ]
[-0.00232 .43075 0 6.11713 ]
[-0.01193 .49103 -6 -2.93439 ]
[-0.01193 .49103 0 6.06561 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03676 .25766 m
.04287 .25527 L
s
[(-1)] .02454 .26244 1 -0.3911 Mshowa
.02817 .3119 m
.0343 .30956 L
s
[(-0.5)] .0159 .31657 1 -0.38093 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01003 .42631 m
.01621 .4241 L
s
[(0.5)] -0.00232 .43075 1 -0.35936 Mshowa
.00046 .48671 m
.00666 .48456 L
s
[(1)] -0.01193 .49103 1 -0.34791 Mshowa
.125 Mabswid
.03507 .26836 m
.03873 .26693 L
s
.03336 .27913 m
.03703 .27771 L
s
.03164 .28997 m
.03531 .28856 L
s
.02991 .3009 m
.03358 .29949 L
s
.02641 .32298 m
.03009 .32158 L
s
.02464 .33413 m
.02833 .33274 L
s
.02286 .34537 m
.02655 .34399 L
s
.02107 .35668 m
.02476 .35531 L
s
.01744 .37956 m
.02114 .3782 L
s
.01561 .39112 m
.01931 .38977 L
s
.01377 .40277 m
.01746 .40142 L
s
.01191 .4145 m
.01561 .41316 L
s
.00815 .43822 m
.01185 .43689 L
s
.00625 .45021 m
.00996 .44889 L
s
.00433 .46229 m
.00805 .46098 L
s
.0024 .47445 m
.00612 .47315 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
0 .096 .575 r
.37901 .68206 .40169 .70185 .43838 .73689 .41587 .71254 Metetra
0 .368 .834 r
.41587 .71254 .43838 .73689 .47671 .76393 .45433 .73569 Metetra
.421 .591 .913 r
.45433 .73569 .47671 .76393 .51647 .77466 .49426 .74403 Metetra
.65 .676 .861 r
.49426 .74403 .51647 .77466 .55711 .76398 .53517 .73301 Metetra
.757 .71 .808 r
.53517 .73301 .55711 .76398 .59791 .73161 .57641 .70237 Metetra
.813 .734 .776 r
.57641 .70237 .59791 .73161 .63828 .6822 .61741 .65627 Metetra
.846 .757 .765 r
.61741 .65627 .63828 .6822 .67796 .62396 .6579 .60207 Metetra
.862 .786 .775 r
.6579 .60207 .67796 .62396 .71717 .56642 .69799 .54838 Metetra
.856 .821 .813 r
.69799 .54838 .71717 .56642 .75651 .51833 .73822 .50319 Metetra
.805 .855 .887 r
.73822 .50319 .75651 .51833 .79688 .48619 .77936 .47247 Metetra
.64 .84 .977 r
.77936 .47247 .79688 .48619 .83921 .47349 .82227 .45946 Metetra
.314 .683 .978 r
.82227 .45946 .83921 .47349 .88433 .48043 .86772 .46433 Metetra
.036 .47 .88 r
.86772 .46433 .88433 .48043 .93269 .5039 .91613 .48419 Metetra
0 .378 .836 r
.91613 .48419 .93269 .5039 .98416 .53763 .96743 .51328 Metetra
.009 .432 .863 r
.35573 .66174 .37901 .68206 .41587 .71254 .39313 .67928 Metetra
.386 .535 .882 r
.39313 .67928 .41587 .71254 .45433 .73569 .43186 .69152 Metetra
.552 .56 .816 r
.43186 .69152 .45433 .73569 .49426 .74403 .47186 .69342 Metetra
.637 .582 .776 r
.47186 .69342 .49426 .74403 .53517 .73301 .51282 .68196 Metetra
.695 .61 .758 r
.51282 .68196 .53517 .73301 .57641 .70237 .55429 .65697 Metetra
.742 .649 .757 r
.55429 .65697 .57641 .70237 .61741 .65627 .59585 .62119 Metetra
.786 .705 .774 r
.59585 .62119 .61741 .65627 .6579 .60207 .63727 .57956 Metetra
.826 .787 .814 r
.63727 .57956 .6579 .60207 .69799 .54838 .67861 .53802 Metetra
.839 .902 .882 r
.67861 .53802 .69799 .54838 .73822 .50319 .72019 .50222 Metetra
.693 .975 .914 r
.72019 .50222 .73822 .50319 .77936 .47247 .76255 .47648 Metetra
.245 .752 .721 r
.76255 .47648 .77936 .47247 .82227 .45946 .80632 .4632 Metetra
.104 0 0 r
.80632 .4632 .82227 .45946 .86772 .46433 .85203 .4625 Metetra
.183 0 0 r
.85203 .4625 .86772 .46433 .91613 .48419 .90001 .47224 Metetra
.003 .499 .865 r
.90001 .47224 .91613 .48419 .96743 .51328 .95021 .48822 Metetra
.515 .706 .958 r
.33183 .64088 .35573 .66174 .39313 .67928 .37019 .63963 Metetra
.587 .599 .831 r
.37019 .63963 .39313 .67928 .43186 .69152 .40942 .6363 Metetra
.611 .554 .766 r
.40942 .6363 .43186 .69152 .47186 .69342 .44952 .62915 Metetra
.628 .543 .739 r
.44952 .62915 .47186 .69342 .51282 .68196 .49037 .61719 Metetra
.647 .553 .735 r
.49037 .61719 .51282 .68196 .55429 .65697 .53183 .60035 Metetra
.671 .585 .751 r
.53183 .60035 .55429 .65697 .59585 .62119 .57376 .57953 Metetra
.702 .65 .795 r
.57376 .57953 .59585 .62119 .63727 .57956 .61607 .55642 Metetra
.739 .774 .882 r
.61607 .55642 .63727 .57956 .67861 .53802 .65876 .5331 Metetra
.68 .946 .96 r
.65876 .5331 .67861 .53802 .72019 .50222 .70195 .51163 Metetra
.70195 .51163 .72019 .50222 .76255 .47648 .74584 .49368 Metetra
.135 0 0 r
.74584 .49368 .76255 .47648 .80632 .4632 .79067 .48017 Metetra
.221 0 0 r
.79067 .48017 .80632 .4632 .85203 .4625 .83666 .47115 Metetra
.124 0 0 r
.83666 .47115 .85203 .4625 .90001 .47224 .88393 .46577 Metetra
.288 .761 .936 r
.88393 .46577 .90001 .47224 .95021 .48822 .93247 .4624 Metetra
.739 .766 .875 r
.30728 .61945 .33183 .64088 .37019 .63963 .34698 .59729 Metetra
.689 .635 .79 r
.34698 .59729 .37019 .63963 .40942 .6363 .38696 .57697 Metetra
.652 .566 .746 r
.38696 .57697 .40942 .6363 .44952 .62915 .42723 .56011 Metetra
.625 .532 .728 r
.42723 .56011 .44952 .62915 .49037 .61719 .46792 .54762 Metetra
.606 .52 .732 r
.46792 .54762 .49037 .61719 .53183 .60035 .50921 .53956 Metetra
.591 .533 .758 r
.50921 .53956 .53183 .60035 .57376 .57953 .55128 .53509 Metetra
.576 .582 .822 r
.55128 .53509 .57376 .57953 .61607 .55642 .59425 .53262 Metetra
.533 .704 .949 r
.59425 .53262 .61607 .55642 .65876 .5331 .63817 .53009 Metetra
.241 .749 .878 r
.63817 .53009 .65876 .5331 .70195 .51163 .68297 .52537 Metetra
.143 0 0 r
.68297 .52537 .70195 .51163 .74584 .49368 .72849 .51667 Metetra
.214 0 0 r
.72849 .51667 .74584 .49368 .79067 .48017 .77451 .50294 Metetra
.136 0 0 r
.77451 .50294 .79067 .48017 .83666 .47115 .82084 .48413 Metetra
.82084 .48413 .83666 .47115 .88393 .46577 .86739 .46117 Metetra
.668 .963 .933 r
.86739 .46117 .88393 .46577 .93247 .4624 .9142 .43581 Metetra
.817 .767 .806 r
.28205 .59743 .30728 .61945 .34698 .59729 .3233 .55633 Metetra
.751 .666 .767 r
.3233 .55633 .34698 .59729 .38696 .57697 .36419 .52092 Metetra
.688 .594 .745 r
.36419 .52092 .38696 .57697 .42723 .56011 .40478 .49553 Metetra
.628 .543 .739 r
.40478 .49553 .42723 .56011 .46792 .54762 .44536 .48254 Metetra
.566 .507 .748 r
.44536 .48254 .46792 .54762 .50921 .53956 .48642 .48208 Metetra
.491 .486 .779 r
.48642 .48208 .50921 .53956 .55128 .53509 .52844 .492 Metetra
.383 .488 .842 r
.52844 .492 .55128 .53509 .59425 .53262 .5718 .50812 Metetra
.172 .509 .912 r
.5718 .50812 .59425 .53262 .63817 .53009 .61665 .52479 Metetra
0 .429 .692 r
.61665 .52479 .63817 .53009 .68297 .52537 .66281 .53588 Metetra
.253 0 0 r
.66281 .53588 .68297 .52537 .72849 .51667 .70981 .53604 Metetra
.138 0 0 r
.70981 .53604 .72849 .51667 .77451 .50294 .75701 .52208 Metetra
.75701 .52208 .77451 .50294 .82084 .48413 .80382 .49382 Metetra
.589 .911 .664 r
.80382 .49382 .82084 .48413 .86739 .46117 .84991 .45417 Metetra
.867 .973 .855 r
.84991 .45417 .86739 .46117 .9142 .43581 .89536 .40839 Metetra
.846 .762 .77 r
.25611 .57479 .28205 .59743 .3233 .55633 .29883 .52034 Metetra
.793 .699 .761 r
.29883 .52034 .3233 .55633 .36419 .52092 .34069 .47446 Metetra
.726 .641 .763 r
.34069 .47446 .36419 .52092 .40478 .49553 .38172 .44317 Metetra
.639 .582 .774 r
.38172 .44317 .40478 .49553 .44536 .48254 .42234 .42971 Metetra
.52 .518 .794 r
.42234 .42971 .44536 .48254 .48642 .48208 .46325 .43427 Metetra
.35 .445 .817 r
.46325 .43427 .48642 .48208 .52844 .492 .50516 .45393 Metetra
.114 .36 .823 r
.50516 .45393 .52844 .492 .5718 .50812 .54869 .4829 Metetra
0 .285 .771 r
.54869 .4829 .5718 .50812 .61665 .52479 .5941 .51313 Metetra
0 .288 .664 r
.5941 .51313 .61665 .52479 .66281 .53588 .64119 .53556 Metetra
.162 0 0 r
.64119 .53556 .66281 .53588 .70981 .53604 .68932 .54211 Metetra
.68932 .54211 .70981 .53604 .75701 .52208 .73755 .52784 Metetra
.635 .957 .781 r
.73755 .52784 .75701 .52208 .80382 .49382 .78496 .49249 Metetra
.874 .992 .81 r
.78496 .49249 .80382 .49382 .84991 .45417 .83106 .44059 Metetra
.919 .912 .796 r
.83106 .44059 .84991 .45417 .89536 .40839 .87594 .38012 Metetra
.853 .76 .759 r
.22944 .55152 .25611 .57479 .29883 .52034 .27324 .49183 Metetra
.824 .739 .771 r
.27324 .49183 .29883 .52034 .34069 .47446 .31592 .44184 Metetra
.767 .714 .803 r
.31592 .44184 .34069 .47446 .38172 .44317 .35748 .40814 Metetra
.657 .669 .85 r
.35748 .40814 .38172 .44317 .42234 .42971 .39837 .39426 Metetra
.448 .575 .889 r
.39837 .39426 .42234 .42971 .46325 .43427 .43935 .40041 Metetra
.133 .408 .855 r
.43935 .40041 .46325 .43427 .50516 .45393 .48128 .42342 Metetra
0 .243 .753 r
.48128 .42342 .50516 .45393 .54869 .4829 .52489 .45693 Metetra
0 .199 .701 r
.52489 .45693 .54869 .4829 .5941 .51313 .57054 .49201 Metetra
0 .325 .778 r
.57054 .49201 .5941 .51313 .64119 .53556 .61809 .51851 Metetra
.192 .616 .946 r
.61809 .51851 .64119 .53556 .68932 .54211 .66687 .52725 Metetra
.608 .857 .991 r
.66687 .52725 .68932 .54211 .73755 .52784 .71585 .51258 Metetra
.823 .904 .896 r
.71585 .51258 .73755 .52784 .78496 .49249 .76397 .47419 Metetra
.894 .879 .813 r
.76397 .47419 .78496 .49249 .83106 .44059 .81064 .41729 Metetra
.908 .846 .771 r
.81064 .41729 .83106 .44059 .87594 .38012 .8559 .35095 Metetra
.846 .762 .77 r
.202 .52757 .22944 .55152 .27324 .49183 .24622 .47179 Metetra
.842 .792 .804 r
.24622 .47179 .27324 .49183 .31592 .44184 .2894 .42473 Metetra
.8 .827 .872 r
.2894 .42473 .31592 .44184 .35748 .40814 .33149 .39244 Metetra
.652 .831 .969 r
.33149 .39244 .35748 .40814 .39837 .39426 .3729 .3782 Metetra
.275 .671 .97 r
.3729 .3782 .39837 .39426 .43935 .40041 .41432 .38218 Metetra
0 .379 .796 r
.41432 .38218 .43935 .40041 .48128 .42342 .45657 .40145 Metetra
0 .238 .708 r
.45657 .40145 .48128 .42342 .52489 .45693 .50036 .43016 Metetra
0 .285 .771 r
.50036 .43016 .52489 .45693 .57054 .49201 .5461 .46015 Metetra
.138 .445 .879 r
.5461 .46015 .57054 .49201 .61809 .51851 .59373 .48224 Metetra
.443 .595 .907 r
.59373 .48224 .61809 .51851 .66687 .52725 .6427 .4882 Metetra
.646 .68 .868 r
.6427 .4882 .66687 .52725 .71585 .51258 .69208 .473 Metetra
.761 .725 .819 r
.69208 .473 .71585 .51258 .76397 .47419 .74093 .43637 Metetra
.827 .756 .784 r
.74093 .43637 .76397 .47419 .81064 .41729 .78865 .38294 Metetra
.865 .784 .77 r
.78865 .38294 .81064 .41729 .8559 .35095 .83521 .32084 Metetra
.817 .767 .806 r
.17377 .50293 .202 .52757 .24622 .47179 .21754 .45953 Metetra
.831 .861 .868 r
.21754 .45953 .24622 .47179 .2894 .42473 .26075 .422 Metetra
.757 .955 .938 r
.26075 .422 .2894 .42473 .33149 .39244 .30331 .39474 Metetra
.429 .871 .878 r
.30331 .39474 .33149 .39244 .3729 .3782 .34547 .38017 Metetra
.34547 .38017 .3729 .3782 .41432 .38218 .38776 .37843 Metetra
.183 0 0 r
.38776 .37843 .41432 .38218 .45657 .40145 .43078 .38733 Metetra
0 .432 .794 r
.43078 .38733 .45657 .40145 .50036 .43016 .47508 .40258 Metetra
.172 .509 .912 r
.47508 .40258 .50036 .43016 .5461 .46015 .52099 .41838 Metetra
.408 .541 .878 r
.52099 .41838 .5461 .46015 .59373 .48224 .56851 .42841 Metetra
.544 .559 .82 r
.56851 .42841 .59373 .48224 .6427 .4882 .61731 .42716 Metetra
.629 .582 .782 r
.61731 .42716 .6427 .4882 .69208 .473 .66677 .41132 Metetra
.693 .615 .764 r
.66677 .41132 .69208 .473 .74093 .43637 .71625 .38072 Metetra
.747 .66 .764 r
.71625 .38072 .74093 .43637 .78865 .38294 .7653 .33839 Metetra
.797 .723 .783 r
.7653 .33839 .78865 .38294 .83521 .32084 .81384 .28975 Metetra
.739 .766 .875 r
.1447 .47755 .17377 .50293 .21754 .45953 .18717 .45274 Metetra
.721 .922 .96 r
.18717 .45274 .21754 .45953 .26075 .422 .22986 .42981 Metetra
.447 .88 .813 r
.22986 .42981 .26075 .422 .30331 .39474 .27272 .41044 Metetra
.27272 .41044 .30331 .39474 .34547 .38017 .31584 .39559 Metetra
.031 0 0 r
.31584 .39559 .34547 .38017 .38776 .37843 .35944 .3853 Metetra
.35944 .3853 .38776 .37843 .43078 .38733 .40375 .37869 Metetra
.336 .781 .96 r
.40375 .37869 .43078 .38733 .47508 .40258 .449 .37412 Metetra
.533 .704 .949 r
.449 .37412 .47508 .40258 .52099 .41838 .49535 .36944 Metetra
.581 .599 .836 r
.49535 .36944 .52099 .41838 .56851 .42841 .54282 .36241 Metetra
.6 .551 .771 r
.54282 .36241 .56851 .42841 .61731 .42716 .59131 .35117 Metetra
.618 .538 .742 r
.59131 .35117 .61731 .42716 .66677 .41132 .64063 .33464 Metetra
.639 .55 .738 r
.64063 .33464 .66677 .41132 .71625 .38072 .69056 .31273 Metetra
.665 .586 .757 r
.69056 .31273 .71625 .38072 .7653 .33839 .74095 .28645 Metetra
.701 .657 .804 r
.74095 .28645 .7653 .33839 .81384 .28975 .79176 .25762 Metetra
.515 .706 .958 r
.11475 .45142 .1447 .47755 .18717 .45274 .15526 .44783 Metetra
.32 .789 .93 r
.15526 .44783 .18717 .45274 .22986 .42981 .19695 .44197 Metetra
.19695 .44197 .22986 .42981 .27272 .41044 .2399 .43201 Metetra
.2399 .43201 .27272 .41044 .31584 .39559 .28409 .41688 Metetra
.28409 .41688 .31584 .39559 .35944 .3853 .32934 .39651 Metetra
.447 .87 .747 r
.32934 .39651 .35944 .3853 .40375 .37869 .37541 .37188 Metetra
.729 .946 .952 r
.37541 .37188 .40375 .37869 .449 .37412 .42209 .34476 Metetra
.739 .774 .882 r
.42209 .34476 .449 .37412 .49535 .36944 .46925 .31736 Metetra
.687 .64 .798 r
.46925 .31736 .49535 .36944 .54282 .36241 .51689 .29186 Metetra
.644 .564 .75 r
.51689 .29186 .54282 .36241 .59131 .35117 .56515 .27 Metetra
.614 .525 .73 r
.56515 .27 .59131 .35117 .64063 .33464 .61425 .25274 Metetra
.593 .513 .733 r
.61425 .25274 .64063 .33464 .69056 .31273 .66445 .24012 Metetra
.579 .527 .761 r
.66445 .24012 .69056 .31273 .74095 .28645 .71598 .23124 Metetra
.569 .581 .826 r
.71598 .23124 .74095 .28645 .79176 .25762 .76894 .22439 Metetra
.009 .432 .863 r
.08389 .42449 .11475 .45142 .15526 .44783 .1222 .44049 Metetra
.195 0 0 r
.1222 .44049 .15526 .44783 .19695 .44197 .16262 .45076 Metetra
.138 0 0 r
.16262 .45076 .19695 .44197 .2399 .43201 .2055 .44981 Metetra
.2055 .44981 .2399 .43201 .28409 .41688 .25074 .43438 Metetra
.465 .864 .685 r
.25074 .43438 .28409 .41688 .32934 .39651 .29779 .40427 Metetra
.784 .998 .846 r
.29779 .40427 .32934 .39651 .37541 .37188 .34588 .3625 Metetra
.869 .919 .859 r
.34588 .3625 .37541 .37188 .42209 .34476 .39431 .31445 Metetra
.826 .787 .814 r
.39431 .31445 .42209 .34476 .46925 .31736 .44262 .26652 Metetra
.756 .677 .774 r
.44262 .26652 .46925 .31736 .51689 .29186 .49075 .22471 Metetra
.686 .596 .75 r
.49075 .22471 .51689 .29186 .56515 .27 .539 .19359 Metetra
.618 .538 .742 r
.539 .19359 .56515 .27 .61425 .25274 .58796 .17562 Metetra
.552 .499 .751 r
.58796 .17562 .61425 .25274 .66445 .24012 .6383 .17091 Metetra
.482 .482 .78 r
.6383 .17091 .66445 .24012 .71598 .23124 .69065 .17719 Metetra
.401 .495 .84 r
.69065 .17719 .71598 .23124 .76894 .22439 .74532 .19002 Metetra
0 .096 .575 r
.05208 .39672 .08389 .42449 .1222 .44049 .08852 .42652 Metetra
.414 0 0 r
.08852 .42652 .1222 .44049 .16262 .45076 .1278 .44836 Metetra
.13 0 0 r
.1278 .44836 .16262 .45076 .2055 .44981 .17056 .45391 Metetra
.394 .852 .852 r
.17056 .45391 .2055 .44981 .25074 .43438 .21669 .4381 Metetra
.773 .989 .892 r
.21669 .4381 .25074 .43438 .29779 .40427 .26538 .40065 Metetra
.889 .94 .841 r
.26538 .40065 .29779 .40427 .34588 .3625 .31541 .34626 Metetra
.896 .861 .798 r
.31541 .34626 .34588 .3625 .39431 .31445 .36562 .28314 Metetra
.862 .786 .775 r
.36562 .28314 .39431 .31445 .44262 .26652 .41526 .22074 Metetra
.805 .716 .767 r
.41526 .22074 .44262 .26652 .49075 .22471 .46415 .16763 Metetra
.728 .649 .77 r
.46415 .16763 .49075 .22471 .539 .19359 .51271 .13009 Metetra
.631 .582 .78 r
.51271 .13009 .539 .19359 .58796 .17562 .56174 .11144 Metetra
.511 .515 .796 r
.56174 .11144 .58796 .17562 .6383 .17091 .61223 .11182 Metetra
.369 .453 .816 r
.61223 .11182 .6383 .17091 .69065 .17719 .66509 .12818 Metetra
.214 .41 .839 r
.66509 .12818 .69065 .17719 .74532 .19002 .72088 .15445 Metetra
.537 .015 0 r
.01926 .36808 .05208 .39672 .08852 .42652 .0548 .40279 Metetra
0 .168 .606 r
.0548 .40279 .08852 .42652 .1278 .44836 .09353 .42872 Metetra
.116 .582 .916 r
.09353 .42872 .1278 .44836 .17056 .45391 .13631 .43646 Metetra
.627 .848 .984 r
.13631 .43646 .17056 .45391 .21669 .4381 .18308 .42016 Metetra
.819 .866 .881 r
.18308 .42016 .21669 .4381 .26538 .40065 .23291 .37953 Metetra
.874 .836 .806 r
.23291 .37953 .26538 .40065 .31541 .34626 .28435 .31996 Metetra
.883 .808 .77 r
.28435 .31996 .31541 .34626 .36562 .28314 .33597 .25079 Metetra
.871 .784 .763 r
.33597 .25079 .36562 .28314 .41526 .22074 .38683 .18264 Metetra
.837 .76 .778 r
.38683 .18264 .41526 .22074 .46415 .16763 .43665 .12502 Metetra
.772 .728 .812 r
.43665 .12502 .46415 .16763 .51271 .13009 .48583 .08476 Metetra
.653 .673 .856 r
.48583 .08476 .51271 .13009 .56174 .11144 .53526 .06546 Metetra
.465 .578 .883 r
.53526 .06546 .56174 .11144 .61223 .11182 .58604 .06725 Metetra
.239 .456 .869 r
.58604 .06725 .61223 .11182 .66509 .12818 .63924 .08684 Metetra
.062 .362 .832 r
.63924 .08684 .66509 .12818 .72088 .15445 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0000=oo`0000002P003goo00<007ooOol0
0goo00<007ooOol02Goo00<007ooOol077oo001SOol600000e:k000jG006>U`00`007M/Mf`03000<
Ool00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00Oomoo`0KOol004Moo`8001Qoo`D000EB
^`03000jG3YL00DjG003000MfagK008Mf`<000Uoo`03001oogoo00Aoo`03001oogoo00Yoo`03001o
ogoo01Yoo`00AWoo00@007ooOol001Aoo`D000QB^`03000jG3YL00DjG003000MfagK00DMf`8000Mo
o`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000LOol004Ioo`04001oogoo000BOol20002
Ool00`00H_T00009D[/00`00>U`jG005>U`00`007M/Mf`047M/010001GX000001Goo00<007ooOol0
17oo0P002Goo0P007Goo0016Ool01000Oomoo`003goo0`0000Aoo`00Ool000=Rn@03001B^e:k00MB
^`03000jG3YL00DjG003000MfagK00@Mf`030005NPEj008000=oo`03001oogoo00Ioo`03001oogoo
02Eoo`00AWoo00@007ooOol000eoo`8000Aoo`8000ERn@03001B^e:k00IB^`03000jG3YL00DjG003
000MfagK00@Mf`030005NPEj0085NP04001oogoo0009Ool00`00Oomoo`0TOol004Moo`8000]oo`<0
00Eoo`03001RnF;i00IRn@03001B^e:k00EB^``000@Mf`030005NPEj0085NP<000Uoo`03001oogoo
02Aoo`00DWoo0`001Woo00<006;iH_T01f;i00<005:kD[/00U:k0`0000=BF000@QT01T8I00<002gJ
;MX00`000QgK00<000Ej1GX00`Ej00<007ooOol027oo00<007ooOol08goo001?Ool30008Ool00`00
H_URn@09H_T01000D[/000001E9H00<0048I@QT01D8I00<002gJ;MX00RgJ0`001@Ej00<007ooOol0
2Goo00<007ooOol08Woo001<Ool3000;Ool00`00H_URn@09H_T20007DUP00`00@QU26@05@QT00`00
;MX]fP05;MX200031GX2000:Ool00`00Oomoo`0ROol004Yoo`<000aoo`03001[600000URn@03001N
V00000MBF00300126D8I00I26@03000]fRgJ00@]fP08000I^P000005NP001GX000Yoo`03001oogoo
025oo`00Agoo0`003Woo00D006/HJaQ[600000MRn@05001NV5jHGYP00007DUP00`00@QU26@05@QT0
0`00;MX]fP05;MX00`006KXI^P04000;Ool00`00Oomoo`0POol004Eoo`80011oo`03001[66/H00=[
6003001RnF;i009Rn@8000ANV003001BF59H00EBF00300126D8I00E26@03000]fRgJ00D]fP06000I
^QVj6KX001Vj0P002Woo00<007ooOol087oo0012Ool3000AOol00`00JaQ[6004JaP01@00H_URnF;i
00001UjH00<0059HDUP01U9H00<0048I@QT01D8I00<002gJ;MX01BgJ00@001Vj6KX000<I^P03001o
ogoo00Qoo`03001oogoo01moo`009Goo10005goo0P004goo00<006/HJaP01V/H00<006;i000025jH
00<0059HDUP01E9H00<0048I@QT01D8I00<002gJ;MX01RgJ00<001Vj00000aVj00<007ooOol02Goo
00<007ooOol07Woo000VOol00`00Oomoo`0DOol3000DOol00`00JaQ[6008JaP00`00GYQNV007GYP0
0`00DUQBF005DUP00`00@QU26@02@QT:0002;MX00`006KX000046KX00`00Oomoo`07Ool2000POol0
01moo`D000=oo`03001oogoo015oo`8001Ioo`03001[66/H00Q[608000YNV003001BF59H00ABF0D0
00A5n003000mn3gh00<mn0H000@I^P03001oogoo00Qoo`03001oogoo01eoo`00:7oo00<007ooOol0
3Goo0`005goo00<006/HJaP026/H00@006KHI]P000UNV004001BF59HDUP300000dhg0015n007AOP0
0`00?OPmn005?OP200000c7j000000040009Ool00`00Oomoo`0MOol002Eoo`04001oogoo000=Ool2
000JOol00`00JaQ[6007JaP01@00I]QVf6KH00002EjH00@0059H000000E>=`030015n4Gh00E5n003
000mn3gh00Hmn08000DanP8000Uoo`03001oogoo01aoo`009Woo0P002goo0`006goo00<006lh0000
1f/H00<006KHI]P00fKH00<005jHGYP01UjH0P001dhg00<004GhAOP01TGh00<003gh?OP01Cgh0P00
1S7j00<007ooOol027oo00<007ooOol06goo000aOol200000goo001oo`0JOol01@00KcQ_>6lh0000
1F/H00<006KHI]P016KH00<005jHGYP01EjH00<005Ig00001dhg00<004GhAOP01TGh00<003gh?OP0
1Sgh00<0037j<OX01C7j00<007ooOol01goo00<007ooOol06goo000^Ool3000NOol00`00KcQ_>003
KcP01@00JaQ[66/H00001fKH00<005jHGYP00ejH0P000eIg00<004hgCSL01Dhg00<004GhAOP01TGh
00<003gh?OP01Cgh00<0037j<OX01C7j00<007ooOol01goo0P0077oo000[Ool3000QOol00`00KcQ_
>004KcP00`00JaP00009I]P00`00GYQNV0020005EWL00`00CSM>=`05CSL00`00AOQ5n006AOP00`00
?OPmn005?OP00`00<OXanP06<OX00`00Oomoo`08Ool00`00Oomoo`03Ool2000DOol002Uoo`80009o
o`03001oogoo01ioo`03001_>6lh00I_>003001Vf6KH00QVf003001NV00000MFM`03001>=dhg00I>
=`030015n4Gh00E5n003000mn3gh00Dmn08000LanP03001oogoo00Qoo`03001oogoo009oo`04001o
ogoo000COol002Ioo`<002Aoo`03001_>6lh00M_>003001Vf6KH00QVf08000UFM`03001>=dhg00E>
=`030015n4Gh00I5n003000mn3gh00@mn08000@anPD000Yoo`05001oogooOol00002Ool00`00Oomo
o`0AOol002Aoo`8002Moo`03001_>6lh00I_>003001_>00000QVf003001R^00000UFM`03001>=dhg
00I>=`030015n4Gh00E5n003000mn3gh008mn0P000A:FP03001oogoo00Uoo`04001oogoo0002Ool0
0`00Oomoo`0AOol0025oo`<002Qoo`03001_>6lh00I_>004001_>6lh0007I]P01@00H[QR^6:h0000
25Ig00<004hgCSL01Thg00<004GhAOP014Gh1P000TXH00<004XH000024YJ00<007ooOol01goo0P00
0Woo00@007ooOol001=oo`007goo0P00:Woo00<006lhKcP01Vlh00<006lhKcP00Vlh00<006KHI]P0
0fKH00<006:hH[P00V:h00<005IgEWL01eIg00<004hgCSL01Dhg00<004GhAOP010000TXG00<004XH
BQP00TXH0P000TXH00<004YJBUX01TYJ00<007ooOol027oo00<007ooOol00P0057oo000LOol3000[
Ool00`00KcQ_>006KcP00`00KcQ_>003KcP00`00I]QVf002I]P00`00H[QR^003H[P00`00EWMFM`07
EWL00`00CSM>=`05CSL30007BQL400001DXH001:64XH00002DYJ00<007ooOol027oo00<007ooOol0
5Goo000KOol2000]Ool20007KcP00`00KcQ_>004KcP01000I]QVf0001f:h00<005IgEWL01UIg00<0
04hgCSL00Thg0`0000=>5`00BQL01DXG0`001DXH00@004XHBQP000Q:FP8000Uoo`03001oogoo01Eo
o`0057oo0`0017oo00<007oo0000:goo00@007<hLcP000E_>003001_>6lh00E_>003001Vf00000QR
^003001FMeIg00MFM`@000A>5`04001:5dXGBQL30002BQL00`00BQQ:6004BQP00`00BQP00007BUX0
0`00F[T0000:Ool00`00Oomoo`0DOol001Eoo`03001oogoo00=oo`04001oogoo000YOol00`00LcQc
>002LcP01@00KcQ_>6lh00002Flh00<006:hH[P01f:h00<005IgEWL01UIg0P0024hG0`001TXG00<0
04XHBQP014XH0P001DYJ0P000eZi00<007ooOol01goo0P005Goo000>Ool50002Ool00`00Oomoo`03
Ool01@00Oomoogoo00009goo00<007<hLcP017<h00<006lh00002Flh0P002V:h00<005IgEWL00eIg
0P0000=BE`00CQL014hG100024XG00<004XHBQP014XH00<004XH00000TYJ0P001UZi00<007ooOol0
1goo00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0TOol00`00
LcQc>005LcP00`00KcQ_>007KcP01000JcY[>P002F:h00@005IgEWMFM`8000=BE`03001>5dhG00<0
00=>5`03001:5dXG00I:5`03001:64XH00E:60800003BUX005Zi00MJ^@03001oogoo00Qoo`03001o
ogoo019oo`005Goo00<007ooOol00Woo0P001Goo00<007ooOol08Woo00<007<hLcP01W<h0P001flh
00<006/jJcX00V/j00<006:hH[P01V:h00@005Ig000000EBE`<000M>5`03001:5dXG00I:5`03001:
64XH00E:608000YJ^@03001oogoo00Moo`03001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo
00<007ooOol087oo00<007<hLcP01W<h00@007=iLgT000E_>003001[>V/j00=[>P03001R^6:h00IR
^08000EBE`800003DUL004hG00M>5`03001:5dXG00I:5`03001:64XH00=:60@000YJ^@03001oogoo
00Qoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol07goo00<007<hLcP01G<h00<0
07=iLgT00W=i00D006lhKcQ_>00000M[>P03001R^6:h00=R^0800004EYT0059GDUL30003DUL00`00
CQM>5`06CQL00`00BQM:5`06BQL01000BQQ:64XH0P000eIH0P0025Zi00<006Ki00002Woo00<007oo
Ool047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0MOol00`00LcQc>005LcP00`00LgUcN@04LgT0
0`00KcP00008JcX00`00H[QR^002H[P01000EYUFVEJI0`001e9G00<004hGCQL01ThG00<004XGBQL0
1TXG0`001UIH00<005ZiF[T01EZi00D006KiI_UVn@0000Qoo`80019oo`0027oo0P0017oo0P000goo
0`000goo0P002Woo00<007ooOol06goo00<007<hLcP01G<h00<007=iLgT01W=i00<006/jJcX026/j
00D006:h000005JI00<00003EYT0059G00MBE`03001>5dhG00I>5`03001:5dXG00E:5`8000UFF003
001J^EZi00=J^@03001VnFKi00=Vn@03001oogoo00Moo`03001oogoo00moo`001goo00@007ooOol0
00=oo`8000Ioo`03001oogoo008000]oo`03001oogoo01Uoo`8000Mc>003001cNG=i00EcN@8000Y[
>PD000AFV@03001BEe9G00IBE`03001>5dhG00I>5`03001:5dXG009:5`<00003DSL005IH00QFF003
001J^EZi009J^@03001VnFKi00EVn@03001oogoo00Moo`03001oogoo00ioo`0000=oo`0000000`00
00Eoo`00Oomoo`0000]oo`04001oogoo000=Ool00`00Oomoo`0HOol00`00KeT00005LcP00`00LgUc
N@05LgT01000Kk]_^`001f/j100025JI00<0059GDUL01U9G00<004hGCQL01ThG10001E8g00<005IH
EUP01eIH00@005ZiF[T000QVn@03001oogoo00Moo`03001oogoo00ioo`001goo00@007ooOol000Qo
o`<000=oo`03001oogoo00aoo`03001oogoo01Ioo`05001_FFmIKeT00003LcP00`00LgUcN@05LgT0
0`00Kk]_^`02Kk/00`00JcY[>P02JcX30003Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL00`00CQM>
5`05CQL20008DSL00`00EUQFF007EUP30008I_T00`00KcP00009Ool00`00Oomoo`0=Ool000Moo`04
001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0=Ool2000EOol00`00KeU_F@03KeT00`00LcP0
0007LgT00`00Kk]_^`04Kk/40006Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL00`00CQM>5`02CQL3
00000dhg001B=`07DSL00`00EUQFF007EUP20008I_T01@00KcQ_>6lh00002Goo00<007ooOol00Woo
10001Woo0008Ool20009Ool40002Ool00`00Oomoo`0?Ool00`00Oomoo`0AOol00`00KeU_F@05KeT2
0006LgT00`00Kk]_^`04Kk/30009Ga`00`00EYUFV@05EYT00`00DUMBE`06DUL40004CSL00`00DSMB
=`07DSL00`00EUQFF005EUP01000G[P000001VKi00<006lhKcP00Vlh00<007ooOol01goo00<007oo
Ool00goo00<007ooOol01Woo000IOol00`00Oomoo`0@Ool00`00Oomoo`0?Ool00`00KeU_F@06KeT0
1000LmX000000g=i00<006nkKk/00Vnk100000=OgP00Ga`025lL00<005JIEYT01EJI00<0059GDUL0
159G0`0024hg00<0058gDSL01U8g00<005IHEUP015IH00@005jhG[QN^08000AVn@03001_>6lh00A_
>003001oogoo00Moo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P004goo00<007ooOol03Woo
00<006mIKeT01FmI00<007?JLmX00W?J00<007=i00000Vnk0`0000=_^`00Gmh00eoN00<005lLGa`0
1elL00<005JIEYT015JI00<0059GDUL010000Thh00<004hgCSL01Thg00<0058gDSL01U8g00<005IH
EUP00eIH00<005jhG[P00Ujh00<006Ki00000fKi00<006lhKcP016lh00<007ooOol027oo00<007oo
Ool00goo00<007ooOol017oo000HOol2000DOol00`00Oomoo`0<Ool00`00KeU_F@05KeT00`00LmYc
fP03LmX50002Kk/20006Gmh00`00GaaO7006Ga`00`00EYUFV@02EYT50007CSP00`00CSM>=`05CSL0
0`00DSMB=`07DSL01@00EUQFF5IH00001Ujh0P000VKi00<006lhKcP016lh00<007Mh000027oo0P00
0Woo00@007ooOol000Ioo`004Woo0P0017oo0P005Goo00<007ooOol02Woo00<006mIKeT01FmI00<0
07?JLmX00g?J0`000fOk10001eoN00<005lLGa`00UlL20000dYj00<004hhCSP01Thh00<004hgCSL0
1Thg00<0058gDSL01U8g00@005IHEUP000MN^004001Vn@000006KcP01@00MgQgN7Mh00002Goo00<0
07ooOol00P001goo000AOol01000Oomoo`000goo00<007oo00005Goo00<007ooOol027oo00<006mI
KeT01FmI00@007?JLmYcfP<000AWn`80008kM`h000A6gP03001:NTYj00E:NP03001>>4hh00E>>003
001>=dhg00I>=`03001B=e8g00IB=`03001FF00000QN^004001Vn@000006KcP01@00MgQgN7Mh0000
2Woo00<007ooOol027oo000AOol01000Oomoo`000goo00@007ooOol001Eoo`03001oogoo00Eoo`<0
00I_F@03001cfP0000@000=Wn`L0008R_`@000D[7P030016gTKN00A6gP03001:NTYj00A:NP03001>
>4hh00I>>003001>=dhg00E>=`03001B=e8g00IB=`8000YN^004001_>6lh0004KcP00`00MgQgN003
MgP00`00Oomoo`07Ool00`00Oomoo`08Ool0015oo`04001oogoo0002Ool20003Ool00`00Oomoo`0C
Ool00`00Oomoo`03Ool01000Jga[O6]l0P000fmI0`000W?J00@006OkIo]Wn`<000ICG`03000R_b:o
00<R_`<000<[7P030016gTKN00=6gP03001:NTYj00A:NP03001>>4hh00I>>003001>=dhg00I>=`03
001B=e8g00AB=`03001JN00000YN^004001_>6lh0003KcP00`00MgQgN004MgP00`00Oomoo`08Ool0
0`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00Oomoo`02Ool2000DOol01@00Oomoogoo0000
1V]l10000g?J00<006Ok000010001e=O1P0000<R_`0000001@001DKN00<004YjBWX014Yj00<004hh
CSP01Dhh00<004hgCSL01Thg00<0058gDSL00U8g0P000eYh00<005jhG[P01Ujh00<006/I00000Vlh
00@006lhKcP000EgN003001g^@0000Uoo`03001oogoo00Moo`004Woo0P000goo0P001Goo0P0057oo
0`001F]l3@000fMK0P000U=O0`0000=F^`00>U`00cYL100000<1V@0000000`000dKN00<004YjBWX0
14Yj00<004hhCSP01Thh00<004hgCSL01Dhg00<0058gDSL00P001EYh00<005jhG[P01Ejh00D006/I
JaU[6@00009_>08000EgN005001g^GNiMkT00009Ool00`00Oomoo`06Ool001Moo`03001oogoo00Eo
o`8001=oo`03001[O6]l009[O0<00003Jg`006?l00=So0<00003Hmh0000000EWF`<000EF^`03000j
G3YL00<jG003000A^`0000@000065Qd000000016gP001TYj00<004hhCSP01Dhh00<004hgCSL01Thg
0P0025Yh00<005jhG[P00ejh00<006/IJaT00V/I00<006lhKcP00P0017Mh00<007NiMkT00gNi00<0
07ooOol01goo00<007ooOol01Goo000GOol00`00Oomoo`06Ool3000@Ool00`00Jga[O0030003Jg`0
1000HoaSo6?l0`000V?N00<006MKIe/016MK00<006;I00001eJk00<003YL>U`00cYL00<0016k4K/0
10000aHM0P001TYj00<004hhCSP01Dhh00<004hgCSL014hg0`002EYh00<005jhG[P00Ujh00<006/I
JaT016/I00@006lh000000=gN003001g^GNi00=g^@8000Uoo`03001oogoo00Eoo`005goo00<007oo
Ool01goo0`003Goo1@0016]l0P000f?l0`000V?N00<006MKIe/00fMK0P000f;I00<005JkE[/015Jk
00<003YL>U`013YL00<0016k4K/00P0000<0mP005Qd00aHM1@0000=:NP00CSP01dhh00<004hg0000
0`000U9I00<005YhFWP01eYh00<005jhG[P00Ujh00<006/IJaT01F/I00@007MhMgP0009gN003001g
^GNi009g^@04001_nfok0009Ool00`00Oomoo`04Ool000Eoo`8000Aoo`8000=oo`<000Aoo`8000Uo
o`D000Moo`D00003Dgl0000000=[O003001So6?l009So004001[Nf]kJg/30004Ie/00`00H]URf@04
H]T00`00E[]F^`04E[/00`00>U`jG004>U`010004K/A^a6k0`000aHM00<003Hl=S`00SHl30001e9I
00<005YhFWP01eYh00@005jhG[P000Q[6@80009gN08000Ag^@03001_nfok009_n`03001oogoo00Eo
o`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000<Ool60002Ool40006Dgl0
1000Jg`006?l100016]k00<006/iJcT00P0000=WF`00H]T01V;I00<005JkE[/015Jk2P0000<A^`00
0?P00`000QHM00<003Hl=S`00cHl00<004HjASX01DHj00<0059IDUT01U9I00<005YhFWP01eYh00<0
05jh000026/I00H007>JLiX007Mh0004MkT00`00Ko]_n`03Ko/20003Ool30002Ool00`00Oomoo`03
Ool000Aoo`04001oogoo000;Ool00`00Oomoo`02000=Ool01@000K/000003Ud00`0025=o1`001F]k
00<006/iJcT00f/i00<006;IH]T01f;I00<005JkE[/00`0000=BF000@QT01D8I00<002gJ00000P00
00<0n0000CP00P0000<F7@00=S`013Hl00<004HjASX01DHj00<0059IDUT01U9I00<005YhFWP01UYh
0P0026/I00<007>JLiX00W>J0`000WNi00<006okKo/00fok00<005OM00000`001Goo00<007ooOol0
0goo0004Ool01000Oomoo`0027oo0`000goo00<007ooOol03Goo00<0006k00001`000U=o0`000S=k
0P0000=O_P00Jg/016]k00<006/iJcT016/i00<006;IH]T01f;I0`001E9H00<0048I@QT0148I00<0
02gJ;MX00P0000@0n0000CP1>08000Df?0030016>THj00E6>P03001BFE9I00IBF@03001JN5Yh00EJ
N003001RnP0000M[6@03001cVW>J00=cVP03001_n@00008000A_n`8000=Gg@03001oogoo00Eoo`03
001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`8000eoo`H00092gPX0009O_P03
001[Nf]k00=[N`03001[>F/i00A[>@03001Zn00000MRf@03001NV00000IBF00300126D8I00A26@03
000]fRgJ008]fP<00081>08000Df?0030016>THj00A6>P03001BFE9I00IBF@03001JN5Yh00=JN080
00=RnP03001[6F/I00=[6@03001cVW>J00=cVP04001_nFoiKoT20003Ko/00`00EmeGg@04Emd00`00
Oomoo`04Ool00`00Oomoo`03Ool000Eoo`8000Uoo`@0009oo`03001oogoo00Uoo`<000Aoo`030012
gT;N00A2gP8000EO6`<000A[N`03001[>F/i00A[>@05001Zn6[hJ_P00004H]T20003GYP00`00DUQB
F004DUP00`00@QU26@03@QT00`00;MX]fP02;MX00`003GX0000200020CP20003=S`00`00ASY6>P05
ASX00`00DUUBF@06DUT00`00FWQJN0020005H_X00`00JaU[6@02JaT00`00LiYcVP03LiX01P00KoU_
nFoi001;]@800003Ko/005OM00IGg@03001oogoo00Eoo`03001oogoo009oo`005Woo00<007ooOol0
1Woo0`0027oo00<004;N@]h00T;N00<005lKGa/01ElK00<006/IJaT00P0000=[N`00JcT01F/i00<0
06[hJ_P00V[h00D006;IH]URf@0000ENV003001BF59H00ABF00300126D8I00A26@03000]fRgJ008]
fP03000=NP00008000030CP0017L00800003=S`004Hj00I6>P03001BFE9I00IBF@03001JN00000QR
nP04001[6F/I0005LiX01@00KoU_nFoi00000d^e30001Woo00<007ooOol00Woo000FOol00`00Oomo
o`03Ool3000<Ool01000@]h000001elK00<006/IJaT00f/I0P001F/i00<006[hJ_P016[h00<006;I
00001UjH00<0059HDUP01E9H00<0048I@QT00d8I00<002gJ;MX00bgJ00<000ej3GX010000Q7L00<0
03Hl00001dHj00<0059IDUT01E9I0P002F;j00<006/I00001G>J00H006oiKoU_n@00BkD300000bZo
0000000500039ad00`00Oomoo`04Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000@Ool20007
Ga/00`00JaU[6@04JaT00`00KaP00003JcT00`00J_QZn006J_P00`00GYQNV006GYP00`00DUQBF004
DUP00`00@QU26@03@QT00`00;MX]fP03;MX01P003GX=NPej0000eP80008Ag08000M6>P03001BFE9I
009BF@<00003D[/006;j00QRnP<000EcVP03001_nFoi00<0009oo`D00085o0D000039ad007oo00Ao
o`03001oogoo009oo`005Goo10005Goo00<005lKGa/00elK00<006/IJaT016/I00L006lHKaQ_6000
JcT00008J_P20008GYP00`00DUQBF003DUP=00000bgJ000=NP023GX200000`3F000Ag0024M`50002
ASX00`00DUT000020005D[/00`00H_YRnP05H_X01@00Jia[W6^L00000g>J00<006oiKoT00P001goo
10000`Gl1@0017oo00<007ooOol00Woo000>Ool30004Ool2000HOol01@00Ga]O6elK00001V/I00<0
06lHKaP00flH00<006[hJ_P01V[h00@006KHI]P000MNV003001BF59H00<0009B=`03001:5dXG00=:
5`03000mn3gh008mn0@0008=NP8000050=H000004M`00004>Wd40008D[/00`00H_YRnP04H_X00`00
Jia[W003Ji`00`00LiX00002KoT00`00Oomoo`08Ool00`00Oomoo`0400001@6J0001n`7k00000goo
00<007ooOol00Woo000?Ool00`00Oomoo`05Ool00`00Oomoo`0FOol00`00Ga/00006JaT00`00KaQ_
6004KaP00`00J_QZn005J_P01@00I]QVf6KH00001ejH0`001E8g00<004XGBQL014XG00<003gh?OP0
0cgh00<0037j00001@0000@1F0004M`000HjO@03001B^e:k00MB^`03001RnV;j009RnP03001[W6^L
00A[W0D000aoo`03001oogoo009oo`8000040IX000000003Ool00`00Oomoo`02Ool000moo`03001o
ogoo00Ioo`03001oogoo01Ioo`03001[6F/I00A[6@03001_66lH00A_6003001_600000EZn003001V
f6KH00=Vf003001NV5jH00=NV003001JM`0000MB=`03001:5dXG00=:5`03000mn3gh00@mn004000a
nS7j<OX200000`4h0001F0030006>Wd00`00D[]B^`07D[/01000H_YRnP001V^L00@005?H000000io
o`03001oogoo00Aoo`8000030IX0000000=oo`03001oogoo005oo`003goo00<007ooOol01goo00<0
07ooOol05Woo00<006/IJaT00f/I00<006lHKaP00flH00D006lHKaQ_600000=Zn003001Vf6KH00EV
f003001NV5jH008000=JM`03001B=e8g00AB=`03001:5dXG00A:5`03000mn3gh00<mn003000anS7j
00<anP<00081F003000jOCYm00DjO@03001B^e:k00IB^`03001RnP0000E[W0@0015oo`03001oogoo
00Eoo`040001VP6J0002Ool00`00Oomoo`01Ool000moo`03001oogoo00Qoo`8001Moo`05001[6F/I
JaT00006KaP00`00KaQ_6002KaP00`00J_P00008I]P00`00GYP00005FWL00`00DSMB=`04DSL00`00
BQM:5`04BQL00`00?OPmn004?OP00`00<OXanP02<OX00`005Qd0000200000`5H000jO@05>Wd00`00
D[]B^`05D[/40003Ji`4000DOol00`00Oomoo`05Ool01P000IX1VP00Ool000=oo`003Woo0P0037oo
00<007ooOol05Goo00<006/I00001VlH00<006lHKaP016lH00<006KHI]P01fKH0P001UYg00<0058g
DSL01E8g00<004XGBQL00dXG00<003gh?OP01Cgh00<0037j<OX00S7j00@001HM5QdF7@L0008jO@03
001B^e:k00@000A?O`03001[W000008001Ioo`03001oogoo00Ioo`<00003Ool007oo009oo`007Goo
00<007ooOol05Goo00<006lHKaP016lH00<006lHKaP01FlH00<006KHI]P01VKH00<006:h00001eYg
00<0058gDSL0158g00<004XGBQL014XG00<003gh?OP013gh00<0037j<OX00c7j00<001HM5Qd00QHM
100000<JOP0000000`001dmo100000=_FP00Ool05Woo00<007ooOol01goo0`000goo000NOol00`00
Oomoo`0EOol00`00KaQ_6002KaP00`00KaQ_6005KaP00`00KaP00006I]P01@00H[QR^6:h00001UYg
00<0058gDSL0158g00<004XGBQL014XG00<003gh?OP00Sgh3@0014KN1@0000<JOP0000000Tmo0`00
16mJ00<007ooOol05goo00<007ooOol01goo0P000goo000OOol2000FOol01@00KaQ_66lH00001VlH
00D006lHKaQ_600000AVf003001R^6:h009R^003001JMeYg00EJM`03001B=e8g00AB=`04001:5dXG
BQL70003BSP00`00BUY:FP04BUX00`00A]i6gP04A]h20003Ga`60005KeX00`00Oomoo`0IOol00`00
Oomoo`07Ool00`00Oomoo`01Ool0025oo`03001oogoo01Aoo`03001_600000M_6005001_66lHKaP0
0003I]P00`00H[QR^004H[P00`00FWMJM`04FWL00`00DSMB=`03DSL50003CQL00`00BSQ:>005BSP0
0`00BUY:FP04BUX00`00A]i6gP02A]h00`00GaaO7002Ga`00`00JcY[>P02JcX00`00KeY_FP03KeX0
0`00Oomoo`0JOol00`00Oomoo`08Ool10001Ool1Ool0029oo`03001oogoo01Aoo`8000I_6003001_
66lH00=_6003001Vf00000MR^003001JMeYg00AJM`03001B=e8g00<00003CSL004hG00I>5`03001:
>4Xh00A:>003001:FTYJ00E:FP040016gP000004Ga`00`00JcY[>P03JcX20003KeX00`00Oomoo`0L
Ool00`00Oomoo`07Ool10001Ool1Ool002=oo`03001oogoo01Eoo`03001_66lH009_6003001_66lH
00E_6003001R^6:h00IR^003001JMeYg00AJM`<000E>=`03001>5dhG00A>5`03001:>4Xh00E:>003
001:FTYJ00A:FP8000EO7003001[>V/j00=[>P06001_>6lh001_FP0087oo00<007ooOol01Woo0@00
0Goo0Goo000TOol00`00Oomoo`0EOol01@00KaQ_66lH00001VlH0P002F:h00@005YgFWMJM`800003
DUL004hg00E>=`03001>5dhG00A>5`03001:>4Xh00E:>004001:FTYJBUX30002FYT01@00GaaO75lL
00001F/j00<006lhKcP00Vlh00<007ooOol087oo00<007ooOol01Goo0@000Goo0Goo000UOol2000F
Ool00`00KaP00006KaP01000JaU[6@0026:h00@005Yg000000=BE`03001>=dhg00A>=`03001>5dhG
00E>5`03001:>4Xh00A:>004001:FP000006FYT00`00Ga`00006JcX00`00KcQ_>002KcP00`00Oomo
o`0POol00`00Oomoo`05Ool10001Ool1Ool002Moo`03001oogoo01Aoo`8000E_6003001[6F/I009[
6@03001R^6:h00ER^08000IBE`03001>=dhg00A>=`03001>5dhG00A>5`03001:>4Xh00A:>08000UJ
V@03001[>V/j00A[>P03001_>6lh009_>003001oogoo029oo`03001oogoo00Aoo`40005oo`5oo`00
:7oo00<007ooOol05Goo00D006lHKaQ_600000E[6@03001R^6:h009R^0<00003FYT0059G00EBE`03
001>=dhg00A>=`03001>5dhG00E>5`03001:>4Xh009:>003001FF00000UJV@8000A[>P03001_>6lh
009_>003001oogoo02Aoo`03001oogoo00=oo`40005oo`5oo`00:Goo00<007ooOol05Goo00<006lH
00001f/I00@006:h000000EJV@03001BEe9G00=BE`03001>=dhg00A>=`03001>5dhG00E>5`03001:
>4Xh008000=FF003001JVEZI00EJV@04001Vf6KH0003JcX00`00KcQ_>002KcP00`00Oomoo`0TOol0
0`00Oomoo`04Ool10001Ool002Yoo`8001Ioo`03001[6F/I00I[6@8000MJV@03001BEe9G00ABE`03
001>=dhg00A>=`03001>5dhG00A>5`03001:>00000EFF003001JVEZI00AJV@03001Vf6KH009Vf003
001[>P0000A_>003001oogoo02Ioo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol057oo00<0
06/IJaT016/I00<005lK00001eZI00<0059GDUL0159G00<004hgCSL014hg00<004hGCQL00dhG0`00
1eIH00<005ZIFYT00UZI00<006KHI]P016KH00<006lhKcP00Vlh00<007ooOol0:7oo00<007ooOol0
0Woo0@000Goo000]Ool00`00Oomoo`0DOol01000JaU[6F/I0P000elK00<005ZIFYT01EZI00<0059G
DUL00e9G00<004hgCSL014hg00<004hGCQL00`000U8g00<005IHEUP01UIH00@005ZIFYT000MVf080
00=_>003001oogoo02Uoo`05001oogooOol00001Ool002ioo`03001oogoo01Aoo`03001[6@0000EO
6`03001JVEZI00EJV@03001BEe9G00=BE`03001>=dhg00=>=`@000IB=`03001FF5IH00EFF003001J
V@0000MVf006001_66lH001_>000;7oo00D007ooOomoo`00005oo`00;goo0P005Goo0P001UlK00<0
05ZIFYT015ZI00<0059GDUL0159G00<004hg00000P000U8g00<0058gDSL01E8g00<005IHEUP01EIH
0P001fKH00<006lHKaP00VlH00<007ooOol0;7oo00@007ooOol0005oo`00<Goo00<007ooOol057oo
00<005lKGa/015lK00<005ZIFYT015ZI00<0059GDUL01@001U8g00<0058gDSL0158g00<005IHEUP0
15IH00<005jH00001VKH00<006lHKaP00flH00<007ooOol0;Goo00<007oo00000Goo000bOol00`00
Oomoo`0DOol00`00Ga]O6`03Ga/01000FYUJVEZI1P000dhh00<0058gDSL01E8g00<0058gDSL01E8g
00<005IHEUP00UIH00D005jHGYQNV00000AVf003001_66lH00=_6003001oogoo02ioo`03001oogoo
00400000<goo00<007ooOol057oo2@000dYj00<004hhCSP014hh00<0058gDSL0158g00<0058gDSL0
1E8g00D005IHEUQFF00000ANV003001Vf6KH009Vf003001_66lH009_6003001oogoo031oo`40005o
o`400000=7oo0P005Woo00D004;N@]i2gP0000I:NP03001>>4hh00=>>003001B=e8g00EB=`03001B
=e8g00AB=`04001FF5IH0006GYP01000I]QVf0001FlH00<007ooOol0<Goo0P00000fOol00`00Oomo
o`0DOol01@00@]i2gT;N00001DYj00<004hhCSP00dhh00<0058gDSL01E8g00<0058gDSL01E8g0P00
1ejH00<006KH00001FlH00<007ooOol0<Goo0`00000gOol00`00Oomoo`0DOol200000d;N001:NP05
BWX00`00CSQ>>003CSP00`00DSMB=`04DSL00`00DSMB=`04DSL20009GYP00`00KaQ_6003KaP00`00
Oomoo`0^Ool40003Ool003Qoo`03001oogoo01=oo`030001^`00008000E:NP03001>>4hh00=>>003
001B=e8g00EB=`03001B=e8g009B=`03001JN00000UNV003001_66lH009_6003001oogoo02]oo`@0
00Moo`00>Goo00<007ooOol04goo00@0006k0K/000E:NP03001>>4hh00A>>003001B=e8g00AB=`03
001B=e8g008000=JN003001NV5jH00ENV003001Zn00000=_6003001oogoo02Qoo`<000]oo`00>Woo
0P0057oo00<0006k0K/00`000dYj00<004hhCSP00dhh00<0058gDSL0158g00<0058g00001EYh00<0
05jHGYP015jH00L006[hJ_QZn000KaP0000WOol4000>Ool003aoo`03001oogoo019oo`<0008b?0@0
00E>>003001B=e8g009B=`@000MJN003001NV5jH009NV003001Zn6[h00=Zn003001oogoo029oo`@0
019oo`00?Goo00<007ooOol04goo0P00138l2`000e9H00<005YhFWP01UYh00@005jHGYP000IZn003
001oogoo01ioo`@001Ioo`00?Woo00<007ooOol04goo0P00138l00<004HjASX00dHj00<0059HDUP0
1E9H00<005YhFWP01EYh00<005jH00001V[h00<007ooOol06goo10006Woo000oOol2000DOol20004
<S`00`00ASY6>P03ASX00`00DUQBF004DUP00`00FWQJN005FWP20006J_P00`00Oomoo`0HOol4000N
Ool0045oo`03001oogoo011oo`03001oogoo0080008b?0030016>THj00=6>P03001BF59H00ABF003
001JN5Yh00AJN003001Rf@0000EZn003001oogoo01Ioo`<0029oo`00@Woo00<007ooOol03goo00@0
07ooOomoo`80008b?0030016>THj00=6>P03001BF59H00ABF004001JN5YhFWP20003H]T00`00J_QZ
n002J_P00`00Oomoo`0BOol4000UOol004=oo`03001oogoo00ioo`03001oogoo009oo`@000E6>P03
001BF59H00ABF004001JN5Yh0006H]T01000J_QZn0004Goo1000:Goo0014Ool2000>Ool00`00Oomo
o`03Ool00`000GX00005ASX00`00DUQBF004DUP00`00FWP00007H]T00`00J_P0000>Ool4000]Ool0
04Ioo`03001oogoo00]oo`03001oogoo00Aoo`D000=6>P03001BF59H00=BF08000QRf@8000]oo`@0
035oo`00Agoo00<007ooOol02Woo00<007ooOol01Woo00<003IM=Ud010000U9H0`0000=B^`00H]T0
1V;I00<007ooOol01goo1000=Goo0018Ool00`00Oomoo`09Ool00`00Oomoo`07Ool00`00=UdfG@02
=Ud30005D[/00`00H]URf@03H]T00`00Oomoo`04Ool4000iOol004Uoo`8000Uoo`03001oogoo00Qo
o`03000fGCIM008fG@03001B^e:k00EB^`05001RfF;IH]T00004Ool3000mOol004]oo`03001oogoo
00Ioo`03001oogoo00Uoo`03000fGCIM008fG@03001B^e:k00EB^`04001Rf@00Ool40010Ool004ao
o`03001oogoo00Eoo`03001oogoo00Yoo`@00003=Ud005:k00=B^`H004Aoo`00CGoo00<007ooOol0
17oo00<007ooOol03Woo2000B7oo001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004mo
o`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo
00001Goo1000Egoo001COol200000goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128621, 0.0056064, 
  0.00599011, 0.00599011}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
0 .095 .556 r
.37769 .7139 .40063 .73217 .43782 .76001 .41492 .74343 Metetra
0 .457 .861 r
.41492 .74343 .43782 .76001 .47657 .77466 .45387 .75912 Metetra
.533 .77 .984 r
.45387 .75912 .47657 .77466 .51648 .7703 .49417 .75482 Metetra
.773 .825 .893 r
.49417 .75482 .51648 .7703 .55694 .74527 .53516 .72875 Metetra
.849 .813 .815 r
.53516 .72875 .55694 .74527 .59731 .70256 .57615 .6841 Metetra
.872 .798 .774 r
.57615 .6841 .59731 .70256 .6372 .649 .61667 .62809 Metetra
.871 .785 .763 r
.61667 .62809 .6372 .649 .6766 .59333 .65665 .56993 Metetra
.851 .775 .777 r
.65665 .56993 .6766 .59333 .71591 .54416 .69647 .51865 Metetra
.803 .76 .812 r
.69647 .51865 .71591 .54416 .75582 .50837 .73682 .48141 Metetra
.707 .726 .865 r
.73682 .48141 .75582 .50837 .7972 .49017 .77859 .46259 Metetra
.53 .648 .909 r
.77859 .46259 .7972 .49017 .84086 .49071 .82263 .46338 Metetra
.287 .517 .901 r
.82263 .46338 .84086 .49071 .88733 .50796 .86953 .48168 Metetra
.078 .391 .85 r
.86953 .48168 .88733 .50796 .93673 .53684 .91945 .5122 Metetra
0 .346 .824 r
.91945 .5122 .93673 .53684 .98855 .5697 .9719 .54695 Metetra
0 .358 .828 r
.35442 .68872 .37769 .7139 .41492 .74343 .39191 .71167 Metetra
.39 .551 .893 r
.39191 .71167 .41492 .74343 .45387 .75912 .43103 .72284 Metetra
.607 .627 .845 r
.43103 .72284 .45387 .75912 .49417 .75482 .47151 .71713 Metetra
.714 .665 .8 r
.47151 .71713 .49417 .75482 .53516 .72875 .51279 .69307 Metetra
.777 .696 .774 r
.51279 .69307 .53516 .72875 .57615 .6841 .55426 .65329 Metetra
.818 .729 .766 r
.55426 .65329 .57615 .6841 .61667 .62809 .59549 .60378 Metetra
.846 .769 .777 r
.59549 .60378 .61667 .62809 .65665 .56993 .63637 .55226 Metetra
.857 .822 .813 r
.63637 .55226 .65665 .56993 .69647 .51865 .67716 .50643 Metetra
.827 .883 .884 r
.67716 .50643 .69647 .51865 .73682 .48141 .71844 .47248 Metetra
.678 .908 .975 r
.71844 .47248 .73682 .48141 .77859 .46259 .76095 .45422 Metetra
.322 .758 .968 r
.76095 .45422 .77859 .46259 .82263 .46338 .80542 .4527 Metetra
0 .512 .857 r
.80542 .4527 .82263 .46338 .86953 .48168 .85239 .46606 Metetra
0 .4 .827 r
.85239 .46606 .86953 .48168 .91945 .5122 .902 .4897 Metetra
.059 .423 .867 r
.902 .4897 .91945 .5122 .9719 .54695 .95387 .51673 Metetra
.403 .563 .899 r
.3309 .65753 .35442 .68872 .39191 .71167 .36897 .66695 Metetra
.546 .557 .817 r
.36897 .66695 .39191 .71167 .43103 .72284 .40833 .669 Metetra
.617 .563 .771 r
.40833 .669 .43103 .72284 .47151 .71713 .44883 .66067 Metetra
.666 .583 .752 r
.44883 .66067 .47151 .71713 .51279 .69307 .49013 .64112 Metetra
.709 .616 .752 r
.49013 .64112 .51279 .69307 .55426 .65329 .53186 .61186 Metetra
.75 .668 .77 r
.53186 .61186 .55426 .65329 .59549 .60378 .57372 .57642 Metetra
.794 .751 .813 r
.57372 .57642 .59549 .60378 .63637 .55226 .61561 .53947 Metetra
.821 .877 .886 r
.61561 .53947 .63637 .55226 .67716 .50643 .65767 .5058 Metetra
.703 .977 .914 r
.65767 .5058 .67716 .50643 .71844 .47248 .70024 .47939 Metetra
.254 .747 .662 r
.70024 .47939 .71844 .47248 .76095 .45422 .74378 .46276 Metetra
.095 0 0 r
.74378 .46276 .76095 .45422 .80542 .4527 .78878 .4566 Metetra
.193 0 0 r
.78878 .4566 .80542 .4527 .85239 .46606 .83557 .45966 Metetra
0 .529 .803 r
.83557 .45966 .85239 .46606 .902 .4897 .88427 .46893 Metetra
.294 .607 .954 r
.88427 .46893 .902 .4897 .95387 .51673 .93466 .47999 Metetra
.623 .641 .847 r
.30709 .62237 .3309 .65753 .36897 .66695 .34611 .61416 Metetra
.623 .569 .772 r
.34611 .61416 .36897 .66695 .40833 .669 .38584 .6045 Metetra
.625 .541 .739 r
.38584 .6045 .40833 .669 .44883 .66067 .42629 .5929 Metetra
.631 .539 .732 r
.42629 .5929 .44883 .66067 .49013 .64112 .46739 .57921 Metetra
.643 .559 .745 r
.46739 .57921 .49013 .64112 .53186 .61186 .5091 .56369 Metetra
.662 .609 .785 r
.5091 .56369 .53186 .61186 .57372 .57642 .55137 .54689 Metetra
.688 .714 .869 r
.55137 .54689 .57372 .57642 .61561 .53947 .59419 .52962 Metetra
.658 .894 .981 r
.59419 .52962 .61561 .53947 .65767 .5058 .63759 .51272 Metetra
.248 .746 .676 r
.63759 .51272 .65767 .5058 .70024 .47939 .68163 .49692 Metetra
.105 0 0 r
.68163 .49692 .70024 .47939 .74378 .46276 .72642 .4827 Metetra
.19 0 0 r
.72642 .4827 .74378 .46276 .78878 .4566 .77205 .47019 Metetra
.104 0 0 r
.77205 .47019 .78878 .4566 .83557 .45966 .81861 .45914 Metetra
.279 .782 .835 r
.81861 .45914 .83557 .45966 .88427 .46893 .86612 .44898 Metetra
.596 .786 .971 r
.86612 .44898 .88427 .46893 .93466 .47999 .91457 .43889 Metetra
.727 .675 .799 r
.28286 .58595 .30709 .62237 .34611 .61416 .3231 .55958 Metetra
.676 .591 .753 r
.3231 .55958 .34611 .61416 .38584 .6045 .36337 .53806 Metetra
.635 .543 .733 r
.36337 .53806 .38584 .6045 .42629 .5929 .40378 .52317 Metetra
.602 .518 .732 r
.40378 .52317 .42629 .5929 .46739 .57921 .44457 .51538 Metetra
.572 .515 .753 r
.44457 .51538 .46739 .57921 .5091 .56369 .48605 .51382 Metetra
.538 .541 .806 r
.48605 .51382 .5091 .56369 .55137 .54689 .52846 .51636 Metetra
.475 .623 .915 r
.52846 .51636 .55137 .54689 .59419 .52962 .57197 .52 Metetra
.242 .707 .947 r
.57197 .52 .59419 .52962 .63759 .51272 .61656 .52134 Metetra
.122 0 0 r
.61656 .52134 .63759 .51272 .68163 .49692 .66205 .5173 Metetra
.194 0 0 r
.66205 .5173 .68163 .49692 .72642 .4827 .70815 .50581 Metetra
.103 0 0 r
.70815 .50581 .72642 .4827 .77205 .47019 .75453 .48625 Metetra
.75453 .48625 .77205 .47019 .81861 .45914 .80095 .4597 Metetra
.671 .976 .861 r
.80095 .4597 .81861 .45914 .86612 .44898 .84737 .42861 Metetra
.79 .853 .897 r
.84737 .42861 .86612 .44898 .91457 .43889 .89397 .39629 Metetra
.784 .7 .771 r
.25799 .55109 .28286 .58595 .3231 .55958 .29956 .50952 Metetra
.719 .626 .753 r
.29956 .50952 .3231 .55958 .36337 .53806 .34048 .47821 Metetra
.652 .567 .747 r
.34048 .47821 .36337 .53806 .40378 .52317 .38095 .46056 Metetra
.576 .52 .755 r
.38095 .46056 .40378 .52317 .44457 .51538 .42147 .4575 Metetra
.484 .482 .779 r
.42147 .4575 .44457 .51538 .48605 .51382 .46262 .46736 Metetra
.353 .456 .826 r
.46262 .46736 .48605 .51382 .52846 .51636 .50497 .48608 Metetra
.139 .44 .876 r
.50497 .48608 .52846 .51636 .57197 .52 .54887 .50761 Metetra
0 .395 .76 r
.54887 .50761 .57197 .52 .61656 .52134 .59434 .52491 Metetra
.254 0 0 r
.59434 .52491 .61656 .52134 .66205 .5173 .64105 .53133 Metetra
.115 0 0 r
.64105 .53133 .66205 .5173 .70815 .50581 .68835 .52223 Metetra
.68835 .52223 .70815 .50581 .75453 .48625 .73554 .49628 Metetra
.621 .93 .691 r
.73554 .49628 .75453 .48625 .80095 .4597 .78209 .45587 Metetra
.876 .983 .832 r
.78209 .45587 .80095 .4597 .84737 .42861 .82786 .40647 Metetra
.872 .851 .82 r
.82786 .40647 .84737 .42861 .89397 .39629 .8732 .35515 Metetra
.817 .724 .762 r
.23218 .52018 .25799 .55109 .29956 .50952 .27497 .46909 Metetra
.76 .675 .769 r
.27497 .46909 .29956 .50952 .34048 .47821 .31657 .43167 Metetra
.676 .621 .787 r
.31657 .43167 .34048 .47821 .38095 .46056 .35727 .41221 Metetra
.549 .554 .812 r
.35727 .41221 .38095 .46056 .42147 .4575 .39767 .41185 Metetra
.358 .464 .832 r
.39767 .41185 .42147 .4575 .46262 .46736 .4386 .42853 Metetra
.095 .348 .818 r
.4386 .42853 .46262 .46736 .50497 .48608 .48083 .45709 Metetra
0 .248 .752 r
.48083 .45709 .50497 .48608 .54887 .50761 .52489 .48974 Metetra
0 .25 .689 r
.52489 .48974 .54887 .50761 .59434 .52491 .57088 .51713 Metetra
0 .427 .721 r
.57088 .51713 .59434 .52491 .64105 .53133 .61842 .53019 Metetra
.241 .755 .842 r
.61842 .53019 .64105 .53133 .68835 .52223 .66667 .5225 Metetra
.674 .975 .889 r
.66667 .5225 .68835 .52223 .73554 .49628 .71468 .49218 Metetra
.874 .978 .841 r
.71468 .49218 .73554 .49628 .78209 .45587 .76168 .44262 Metetra
.918 .909 .797 r
.76168 .44262 .78209 .45587 .82786 .40647 .80747 .38136 Metetra
.896 .834 .776 r
.80747 .38136 .82786 .40647 .8732 .35515 .85246 .31797 Metetra
.837 .754 .771 r
.20517 .4947 .23218 .52018 .27497 .46909 .24879 .44142 Metetra
.797 .746 .806 r
.24879 .44142 .27497 .46909 .31657 .43167 .29101 .40249 Metetra
.707 .725 .864 r
.29101 .40249 .31657 .43167 .35727 .41221 .3321 .38236 Metetra
.504 .649 .923 r
.3321 .38236 .35727 .41221 .39767 .41185 .37271 .3822 Metetra
.153 .463 .889 r
.37271 .3822 .39767 .41185 .4386 .42853 .41371 .39989 Metetra
0 .255 .755 r
.41371 .39989 .4386 .42853 .48083 .45709 .45596 .43008 Metetra
0 .185 .69 r
.45596 .43008 .48083 .45709 .52489 .48974 .5001 .46461 Metetra
0 .29 .767 r
.5001 .46461 .52489 .48974 .57088 .51713 .54632 .49369 Metetra
.167 .543 .926 r
.54632 .49369 .57088 .51713 .61842 .53019 .59427 .50775 Metetra
.554 .765 .976 r
.59427 .50775 .61842 .53019 .66667 .5225 .64312 .49996 Metetra
.773 .837 .901 r
.64312 .49996 .66667 .5225 .71468 .49218 .69184 .46836 Metetra
.861 .839 .822 r
.69184 .46836 .71468 .49218 .76168 .44262 .73961 .41653 Metetra
.891 .827 .778 r
.73961 .41653 .76168 .44262 .80747 .38136 .78614 .35248 Metetra
.892 .815 .766 r
.78614 .35248 .80747 .38136 .85246 .31797 .8318 .28627 Metetra
.842 .793 .804 r
.17672 .47501 .20517 .4947 .24879 .44142 .22056 .42725 Metetra
.82 .847 .868 r
.22056 .42725 .24879 .44142 .29101 .40249 .26322 .39163 Metetra
.703 .886 .965 r
.26322 .39163 .29101 .40249 .3321 .38236 .30487 .37202 Metetra
.345 .766 .976 r
.30487 .37202 .3321 .38236 .37271 .3822 .34607 .36946 Metetra
0 .455 .796 r
.34607 .36946 .37271 .3822 .41371 .39989 .38761 .38207 Metetra
0 .281 .704 r
.38761 .38207 .41371 .39989 .45596 .43008 .43026 .40516 Metetra
0 .304 .774 r
.43026 .40516 .45596 .43008 .5001 .46461 .47463 .43174 Metetra
.127 .436 .875 r
.47463 .43174 .5001 .46461 .54632 .49369 .52094 .45349 Metetra
.413 .557 .889 r
.52094 .45349 .54632 .49369 .59427 .50775 .56897 .46242 Metetra
.601 .628 .85 r
.56897 .46242 .59427 .50775 .64312 .49996 .61804 .45293 Metetra
.714 .674 .81 r
.61804 .45293 .64312 .49996 .69184 .46836 .6673 .42339 Metetra
.786 .713 .783 r
.6673 .42339 .69184 .46836 .73961 .41653 .71602 .37674 Metetra
.834 .752 .773 r
.71602 .37674 .73961 .41653 .78614 .35248 .76387 .3196 Metetra
.863 .796 .783 r
.76387 .3196 .78614 .35248 .8318 .28627 .81112 .2604 Metetra
.815 .843 .87 r
.14669 .46022 .17672 .47501 .22056 .42725 .19005 .42484 Metetra
.763 .954 .936 r
.19005 .42484 .22056 .42725 .26322 .39163 .23285 .39686 Metetra
.471 .894 .857 r
.23285 .39686 .26322 .39163 .30487 .37202 .27518 .37886 Metetra
.27518 .37886 .30487 .37202 .34607 .36946 .31741 .37151 Metetra
.112 0 0 r
.31741 .37151 .34607 .36946 .38761 .38207 .36004 .37356 Metetra
0 .509 .781 r
.36004 .37356 .38761 .38207 .43026 .40516 .40362 .38192 Metetra
.208 .579 .942 r
.40362 .38192 .43026 .40516 .47463 .43174 .44857 .3921 Metetra
.425 .569 .894 r
.44857 .3921 .47463 .43174 .52094 .45349 .49507 .39898 Metetra
.538 .555 .82 r
.49507 .39898 .52094 .45349 .56897 .46242 .54301 .3978 Metetra
.608 .561 .776 r
.54301 .3978 .56897 .46242 .61804 .45293 .59202 .38528 Metetra
.661 .583 .757 r
.59202 .38528 .61804 .45293 .6673 .42339 .64158 .36051 Metetra
.709 .622 .758 r
.64158 .36051 .6673 .42339 .71602 .37674 .69122 .32516 Metetra
.756 .681 .779 r
.69122 .32516 .71602 .37674 .76387 .3196 .74072 .2831 Metetra
.802 .77 .823 r
.74072 .2831 .76387 .3196 .81112 .2604 .79019 .2394 Metetra
.681 .876 .972 r
.11513 .44835 .14669 .46022 .19005 .42484 .15727 .43006 Metetra
.457 .887 .861 r
.15727 .43006 .19005 .42484 .23285 .39686 .19987 .41287 Metetra
.19987 .41287 .23285 .39686 .27518 .37886 .24295 .39727 Metetra
.24295 .39727 .27518 .37886 .31741 .37151 .28658 .38339 Metetra
.28658 .38339 .31741 .37151 .36004 .37356 .33089 .37097 Metetra
.406 .849 .925 r
.33089 .37097 .36004 .37356 .40362 .38192 .376 .35942 Metetra
.598 .785 .97 r
.376 .35942 .40362 .38192 .44857 .3921 .42199 .34791 Metetra
.618 .643 .853 r
.42199 .34791 .44857 .3921 .49507 .39898 .46893 .33554 Metetra
.614 .567 .778 r
.46893 .33554 .49507 .39898 .54301 .3978 .51682 .32151 Metetra
.614 .536 .743 r
.51682 .32151 .54301 .3978 .59202 .38528 .56561 .30528 Metetra
.62 .533 .734 r
.56561 .30528 .59202 .38528 .64158 .36051 .61523 .28671 Metetra
.634 .556 .749 r
.61523 .28671 .64158 .36051 .69122 .32516 .66563 .26605 Metetra
.656 .611 .792 r
.66563 .26605 .69122 .32516 .74072 .2831 .71677 .24392 Metetra
.685 .72 .875 r
.71677 .24392 .74072 .2831 .79019 .2394 .76868 .22118 Metetra
.244 .707 .948 r
.08224 .43664 .11513 .44835 .15727 .43006 .12264 .43695 Metetra
.12264 .43695 .15727 .43006 .19987 .41287 .16472 .43174 Metetra
.16472 .43174 .19987 .41287 .24295 .39727 .20852 .41886 Metetra
.20852 .41886 .24295 .39727 .28658 .38339 .25383 .39771 Metetra
.493 .889 .736 r
.25383 .39771 .28658 .38339 .33089 .37097 .30024 .36937 Metetra
.765 .974 .92 r
.30024 .36937 .33089 .37097 .376 .35942 .34735 .33637 Metetra
.788 .828 .883 r
.34735 .33637 .376 .35942 .42199 .34791 .39483 .30209 Metetra
.729 .685 .808 r
.39483 .30209 .42199 .34791 .46893 .33554 .44253 .27001 Metetra
.671 .593 .759 r
.44253 .27001 .46893 .33554 .51682 .32151 .49055 .24308 Metetra
.625 .538 .736 r
.49055 .24308 .51682 .32151 .56561 .30528 .53914 .22316 Metetra
.589 .51 .734 r
.53914 .22316 .56561 .30528 .61523 .28671 .5887 .21074 Metetra
.558 .508 .756 r
.5887 .21074 .61523 .28671 .66563 .26605 .63959 .20487 Metetra
.53 .539 .809 r
.63959 .20487 .66563 .26605 .71677 .24392 .69209 .2033 Metetra
.492 .624 .909 r
.69209 .2033 .71677 .24392 .76868 .22118 .74626 .20278 Metetra
0 .302 .651 r
.04837 .42199 .08224 .43664 .12264 .43695 .0869 .43859 Metetra
.202 0 0 r
.0869 .43859 .12264 .43695 .16472 .43174 .12832 .44403 Metetra
.12832 .44403 .16472 .43174 .20852 .41886 .17276 .43355 Metetra
.494 .893 .752 r
.17276 .43355 .20852 .41886 .25383 .39771 .21975 .40577 Metetra
.796 .998 .854 r
.21975 .40577 .25383 .39771 .30024 .36937 .2684 .36317 Metetra
.882 .928 .848 r
.2684 .36317 .30024 .36937 .34735 .33637 .31772 .31139 Metetra
.855 .817 .811 r
.31772 .31139 .34735 .33637 .39483 .30209 .36693 .25766 Metetra
.793 .716 .779 r
.36693 .25766 .39483 .30209 .44253 .27001 .41567 .20916 Metetra
.721 .633 .759 r
.41567 .20916 .44253 .27001 .49055 .24308 .46407 .17163 Metetra
.644 .566 .752 r
.46407 .17163 .49055 .24308 .53914 .22316 .51263 .14861 Metetra
.563 .513 .758 r
.51263 .14861 .53914 .22316 .5887 .21074 .56211 .14102 Metetra
.474 .477 .781 r
.56211 .14102 .5887 .21074 .63959 .20487 .61328 .1471 Metetra
.372 .464 .825 r
.61328 .1471 .63959 .20487 .69209 .2033 .66672 .16254 Metetra
.25 .487 .889 r
.66672 .16254 .69209 .2033 .74626 .20278 .72262 .18095 Metetra
.476 0 0 r
.01394 .40164 .04837 .42199 .0869 .43859 .05105 .4285 Metetra
.212 0 0 r
.05105 .4285 .0869 .43859 .12832 .44403 .09194 .4407 Metetra
.348 .814 .918 r
.09194 .4407 .12832 .44403 .17276 .43355 .13689 .43159 Metetra
.755 .966 .932 r
.13689 .43159 .17276 .43355 .21975 .40577 .18528 .39926 Metetra
.877 .923 .852 r
.18528 .39926 .21975 .40577 .2684 .36317 .23585 .34722 Metetra
.894 .858 .798 r
.23585 .34722 .2684 .36317 .31772 .31139 .28716 .28324 Metetra
.874 .799 .773 r
.28716 .28324 .31772 .31139 .36693 .25766 .33804 .21714 Metetra
.831 .744 .768 r
.33804 .21714 .36693 .25766 .41567 .20916 .38794 .15835 Metetra
.766 .688 .777 r
.38794 .15835 .41567 .20916 .46407 .17163 .43697 .1142 Metetra
.672 .625 .794 r
.43697 .1142 .46407 .17163 .51263 .14861 .48577 .08908 Metetra
.542 .552 .815 r
.48577 .08908 .51263 .14861 .56211 .14102 .53532 .08411 Metetra
.377 .472 .83 r
.53532 .08411 .56211 .14102 .61328 .1471 .58667 .09713 Metetra
.198 .4 .835 r
.58667 .09713 .61328 .1471 .66672 .16254 .64065 .12275 Metetra
.055 .371 .839 r
.64065 .12275 .66672 .16254 .72262 .18095 .6976 .15276 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`l001Moo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol200001Goo000005Ii00001dHj00<0037j<OX010004goo00<007oo
Ool017oo0P002Goo0P007Goo0016Ool01000Oomoo`003goo0`0000=oo`00000015Ii00<004HjASX0
1DHj00<0037j<OX00c7j0`0047oo00<007ooOol01Woo00<007ooOol09Goo0016Ool01000Oomoo`00
3Goo0P000Woo0P001UIi00<004HjASX01DHj00<0037j<OX0137j00@001VJ000000ioo`03001oogoo
00Moo`03001oogoo02Aoo`00Agoo0P002goo0`000goo00<005IiEWT01eIi00<004HjASX01DHj00<0
037j<OX0137j00<001VJ6IX00`002goo00<007ooOol01goo00<007ooOol097oo001BOol30004Ool2
0009EWT00`00ASY6>P05ASX00`00<OXanP05<OX00`006IXIVP026IX20009Ool00`00Oomoo`08Ool0
0`00Oomoo`0SOol004moo`<000Ioo`04001Rf6;H0009EWT00`00ASY6>P04ASX00`00<OXanP05<OX0
0`006IXIVP046IX20007Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<000Qoo`05001Rf6;H
H]P00009EWT00`00ASY6>P05ASX00`00<OXanP05<OX00`006IXIVP036IX010001GX000001Goo00<0
07ooOol02Goo00<007ooOol08Woo001:Ool30009Ool00`00H]QRf003H]P00`00EWUFN@07EWT00`00
ASY6>P04ASX00`00<OXanP05<OX00`006IXIVP046IX00`001GX5NP030002Ool00`00Oomoo`0:Ool0
0`00Oomoo`0QOol004Moo`<0009oo`03001oogoo00Ioo`03001Rf6;H00ERf003001FNEIi00IFN@03
0016>THj0096>PT0008anP03000IVQVJ00@IVP030005NPEj0085NP<000eoo`03001oogoo021oo`00
AGoo0P003Goo00<006;HH]P01V;H00<005IiEWT01UIi1@000dXH00<003gh?OP00cgh1P000aVJ00<0
00Ej1GX010Ej00<007ooOol02goo00<007ooOol087oo0012Ool3000>Ool00`00H]QRf008H]P00`00
EWUFN@02EWT300000e9H001:6006BQP00`00?OPmn005?OP00`00;MX]fP0500051GX00`00Oomoo`0<
Ool00`00Oomoo`0OOol002Eoo`@001Moo`80015oo`03001Rf6;H00QRf003001FN@00008000EBF003
001:64XH00A:6003000mn3gh00Hmn003000]fRgJ00<]fP@00085NP8000ioo`03001oogoo01ioo`00
9Woo00<007ooOol057oo0`004Woo00<006[h00002F;H0P00259H00<004XHBQP01DXH00<003gh?OP0
1Cgh00<002gJ;MX012gJ00<001gl7O`00`0000<5NP00Ool02goo0P0087oo000OOol50003Ool00`00
Oomoo`0AOol2000DOol01000J_QZn00026;H00<005jH0000259H00<004XHBQP01DXH00<003gh?OP0
1Sgh00<002gJ;MX012gJ00@001gl7O`Mo0@000eoo`03001oogoo01eoo`00:7oo00<007ooOol03Goo
0`005Goo00<006[hJ_P00V[h00<006;HH]P016;H00D005jHGYQNV00000QBF003001:64XH00E:6003
000mn3gh00Dmn003000]fRgJ00@]fP05000Mo1gl7O`000037O`2000;Ool00`00Oomoo`0MOol002Eo
o`04001oogoo000=Ool2000GOol00`00J_QZn003J_P00`00H]QRf002H]P20004GYP00`00DUQBF006
DUP00`00BQQ:6005BQP00`00?OPmn005?OP00`00;MX]fP05;MX01@007O`Mo1gl000011gl00<007oo
Ool02Goo00<007ooOol077oo000VOol2000;Ool3000IOol00`00J_QZn004J_P01000H]QRf0001UjH
00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP01Sgh00<002gJ;MX01BgJ00@001gl7O`000DM
o003001oogoo00Uoo`03001oogoo01]oo`00<Goo0P0000=oo`00Ool067oo00<006[hJ_P01F[h00<0
06;H000025jH00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP01Cgh00<002gJ;MX01BgJ00@0
01gl7O`0008Mo0D000Yoo`03001oogoo01]oo`00;Woo0`0077oo00<006[hJ_P01f[h00<005jHGYP0
1ejH00<0059HDUP01U9H00<004XHBQP01DXH00<003gh?OP00Sgh4@001Cim00<007ooOol01goo0P00
77oo000[Ool3000NOol00`00J_QZn007J_P20009GYP00`00DUQBF006DUP00`00BQQ:6004BQP60004
AQP00`00@SU2>@05@ST00`00?Wd00008?Wd00`00Oomoo`08Ool00`00Oomoo`03Ool2000DOol002Uo
o`80009oo`03001oogoo01]oo`03001Zn6[h00MZn004001Vf6KH0009GYP00`00DUQBF006DUP00`00
BQP000030002BQL00`00AQQ66006AQP00`00@SU2>@05@ST20008?Wd30008Ool00`00Oomoo`02Ool0
1000Oomoo`004goo000VOol3000ROol00`00J_QZn006J_P01@00I]QVf6KH00002EjH00<0059HDUP0
1E9H0`001dXG00<004HHAQP01DHH00<0048i@ST01T8i00<003im?Wd013im0P000eKk00<007ooOol0
1Woo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo0P0097oo00<006lh00001V[h00<006KH
I]P00VKH00<005jHGYP01ejH00<0059HDUP00U9H0`0000=>=`00BQL01dXG00<004HHAQP01THH00<0
048i@ST01D8i00<003im?Wd00Sim0P001UKk00<007ooOol01Woo00@007ooOol0009oo`03001oogoo
015oo`008Goo0`009Goo00D006lhKcQ_>00000AZn003001Vf6KH00AVf003001NV5jH00MNV0@000A>
=`03001:5dXG00I:5`03001664HH00I660030012>D8i00E2>@800003?Wd0000000QFn`8000Ioo`80
009oo`04001oogoo000COol001moo`8002Moo`03001_>6lh009_>005001Zn6[hJ_P00007I]P00`00
GYQNV006GYP20008CSL00`00BQM:5`06BQL00`00AQQ66005AQP00`00@SU2>@04@ST40009E_/00`00
IaX00008Ool00`00Oomoo`02000DOol001aoo`<002Qoo`03001_>6lh00A_>003001Zn00000UVf003
001NV5jH00=NV0800003EUP004hg00M>=`03001:5dXG00I:5`03001664HH00I660030012>D8i00D0
0003DWT005Kk00MFn`05001W6VLJIaX00008Ool00`00Oomoo`0EOol001]oo`8002Yoo`03001_>6lh
00E_>003001Vf6KH00QVf004001NV5jHGYP20004EUP00`00CSM>=`05CSL00`00BQM:5`06BQL00`00
AQQ66006AQP50003DWT00`00DWT00006E_/00`00IaYW6P03IaX00`00Oomoo`05Ool00`00Oomoo`0E
Ool001Aoo`<000Aoo`03001oo`0002Qoo`03001_>6lh00I_>003001Vf6KH00QVf004001NV0000006
EUP00`00CSM>=`06CSL00`00BQM:5`06BQL00`00AQQ66002AQP50007DWT00`00DWT00004E_/00`00
IaYW6P05IaX00`00Oomoo`05Ool00`00Oomoo`0DOol001Eoo`03001oogoo00=oo`04001oogoo000V
Ool00`00KcQ_>006KcP00`00KeT00009I]P20008EUP00`00CSM>=`06CSL00`00BQM:5`06BQL00`00
AQP0000200001DHH0000058g0000259i0P000eKk00<006LJIaX01VLJ00<007ooOol01Goo0P005Goo
000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo000097oo00<006lhKcP01Vlh00D006mIKeU_
F@0000MVf003001N^@0000UFF003001>=dhg00E>=`03001:5dXG00E:5`<00096608000AB=`03001B
NE9i00EBN@05001BN@00E_/00008IaX00`00KcT00007Ool00`00Oomoo`0COol001Eoo`03001oogoo
009oo`8000Aoo`03001oogoo021oo`03001_>6lh00I_>003001_FFmI00=_F@03001Vf6KH009Vf080
00=N^@03001FF5IH00IFF003001>=dhg00I>=`04001:5dXGBQL30002BQL30006DSL00`00DWUBN@06
DWT30008IaX01@00KcU_>Fli00001goo00<007ooOol04Woo000EOol00`00Oomoo`02Ool20005Ool0
0`00Oomoo`0OOol20006KcP00`00KeU_F@04KeT01@00I]QVf6KH00001Eji00<005IHEUP01UIH00<0
04hgCSL01Thg100014XG0P00258g00<0059iDWT01U9i0`001fLJ00<006liKcT00fli00<007ooOol0
17oo00<007ooOol04Woo000DOol20004Ool00`00Oomoo`05Ool00`00Oomoo`0MOol01000LgUcN@00
1Flh00<006mIKeT01FmI00<006KH00001eji00<005IHEUP01UIH00<004hgCSL014hg0P0014XG0`00
00=>=`00DSL0258g00<0059iDWT0159i00D006:h001R^00000EW6P03001_>Fli00A_>@03001oogoo
00Eoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol06goo00<007=iLgT00W=i00D0
06lhKcQ_>00000Q_F@8000QN^@03001FF5IH00IFF004001>=dhgCSL30002CSL40005CSL00`00DSMB
=`06DSL00`00DWUBN@03DWT01000H[QR^6:h0P0016LJ00<006liKcT016li00<007=H00001goo00<0
07ooOol047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0IOol00`00LgUcN@04LgT00`00KcP00008
KeT00`00Ie`00009G[T00`00EUQFF005EUP40003CSL30008CSL00`00DSMB=`07DSL01@00DWUBNE9i
00001F:h0P000VLJ00<006liKcT016li00D007=HLeQcF00000Eoo`80019oo`006Goo0P002Woo00<0
07ooOol05goo00<007=iLgT01W=i00<006mIKeT01VmI00D006MLIeaWG00000QN^@03001FF5IH00=F
F0@00003CSL000000080009>=`03001>=dhg00I>=`03001B=e8g00MB=`04001BNE9i0007H[P00`00
IaX00007KcT00`00LeQcF002LeP00`00Oomoo`04Ool00`00Oomoo`0?Ool000Qoo`8000Aoo`8000=o
o`<000=oo`8000]oo`03001oogoo01Eoo`03001cNG=i00IcN@03001c^`0000E_F@8000AWG003001N
^Eji00IN^@03001FF5IH00<0009FF0<000M>=`03001>=dhg00I>=`03001B=e8g00IB=`03001BN@00
00QR^08000M_>@03001cF7=H00AcF003001oogoo00Aoo`03001oogoo00ioo`001goo00@007ooOol0
00=oo`8000Ioo`04001oogoo000=Ool00`00Oomoo`0COol00`00LgUcN@06LgT01@00Lk]c^g>k0000
0fmI00<006MLIe`01FML00<005jiG[T01Eji0`000UIH0`0000=>F000CSL01dhg00<004hgCSL01Thg
00<0058gDSL01U8g0P002V:h00<006liKcT016li00<007=HLeP01G=H00<007ooOol017oo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo015o
o`03001cNG=i00IcN@03001c^g>k00=c^`03001_F@0000QWG003001N^Eji009N^@<00003G[T005IH
008000E>F003001>=dhg00E>=`03001>=dhg00I>=`03001B=e8g00EB=`03001JN00000UR^003001[
600000A_>@03001cF7=H00EcF003001cF00000Ioo`03001oogoo00eoo`001goo00@007ooOol000Qo
o`<000=oo`03001oogoo00eoo`80011oo`03001cNG=i00IcN@03001c^g>k00Ac^`<000UWG0@000=N
^@<000M>F003001>=dhg00I>=`03001>=dhg00I>=`03001B=e8g00=B=`05001JN5YhFWP00007H[P0
0`00JaQ[60020003KcT00`00LeQcF004LeP01000LeQcF0001goo00<007ooOol00Woo10001Woo0007
Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol03goo00<007ooOol037oo0P001g=i00<0
07>kLk/017>k00@006?mHoeSo@8000IWG080009N^@@0009>V`03001>F4iH00E>F003001>=dhg00I>
=`03001>=dhg00I>=`04001B=e8gDSL20004FWP00`00H[QR^004H[P00`00JaQ[6002JaP200000fli
001cF005LeP00`00LeQcF002LeP00`00Oomoo`04Ool00`00Oomoo`03Ool00`00Oomoo`06Ool000Qo
o`8000Uoo`@0009oo`03001oogoo011oo`03001oogoo00]oo`04001c^`000004LgT00`00Lk]c^`03
Lk/20006Hod00`00IeaWG0070007CY/00`00CUQ>F005CUP00`00CSM>=`05CSL00`00CSM>=`06CSL0
1000DSMB=`001eYh00<006:hH[P00V:h00<006/HJaP016/H0P001W=H00<007=HLeP00g=H00<007oo
Ool01Goo00<007ooOol00goo00<007ooOol01Goo000HOol2000COol00`00Oomoo`09Ool00`00Lk]c
^`02Lk/200000g=i001c^`04Lk/00`00HoeSo@05Hod60003Ccl00`00CY]>V`06CY/00`00CUQ>F005
CUP00`00CSM>=`06CSL00`00CSM>=`06CSL20008FWP00`00H[QR^002H[P00`00JaQ[6005JaP00`00
LeQcF004LeP00`00LeQcF004LeP00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qo
o`8001Aoo`03001oogoo00Moo`03001c^g>k00Ac^`D0009c^`03001SoF?m00=So@@0008cO@03001?
?dlo00A??`03001>VdjK00E>V`03001>F4iH00E>F003001>=dhg00I>=`03001>=dhg00E>=`8000YJ
N004001R^6:h0007JaP30004LeP00`00LeQcF005LeP00`00Oomoo`04Ool20002Ool01000Oomoo`00
1Woo000BOol20004Ool00`00Oomoo`0DOol00`00Oomoo`05Ool00`00Lk]c^`04Lk/00`00Io]Wn`03
Io/90006<gd00`00Ccm??`03Ccl00`00CY]>V`06CY/00`00CUQ>F005CUP00`00CSM>=`05CSL00`00
CSM>=`02CSL300000eIH001JN009FWP00`00H[P00007JaP00`00LeQcF0020002LeP00`00LeQcF005
LeP00`00KgX00007Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`0EOol0
0`00Oomoo`03Ool00`00Lk]c^`04Lk/00`00Io]Wn`02Io/200000cnG000000030002;al700000c=m
001??`04Ccl00`00CY]>V`05CY/00`00CUQ>F005CUP00`00CSM>=`06CSL40005EUP00`00FWQJN008
FWP00`00JaQ[6006JaP01`00LeQcF7=H001cF00000McF004001_NVmj0008Ool00`00Oomoo`08Ool0
015oo`04001oogoo0003Ool00`00Oomoo`0FOol01@00Oomoogoo00001W>k00@006OkIo]Wn`L0009K
WPl000=??`03001>VdjK00E>V`03001>F4iH00A>F003001>=dhg00A>=`<000QFF003001JN5Yh00MJ
N08000M[6003001cF7=H00=cF003001cF7=H00EcF003001_NVmj009_NP03001oogoo00Eoo`03001o
ogoo00Qoo`004Goo00@007ooOol0009oo`8001Uoo`03001oo`0000Ic^`03001WnfOk00H0009KWP<0
0003F_/004:M00A2W@03000Ag00000@000036Uh00000008000M>V`03001>F4iH00E>F003001>=`00
00<0009>>003001FF5IH00IFF003001JN5Yh00IJN004001VfFKI0005JaP00`00LeQcF004LeP20005
LeP00`00KgY_NP03KgX00`00Oomoo`06Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00
Oomoo`0HOol00`00Lk]c^`04Lk/00`00Io/0000600000e^N00000005F_/00`00@Ye2W@03@Yd01000
4M`Ag17L0`000aYN1P000TjK00<004iHCUP00TiH1@001dhh00<005IHEUP01UIH00<005YhFWP015Yh
00<006KII]T00VKI00D006/HJaQ[600000IcF004001g^@000003LeP00`00KgY_NP04KgX20007Ool0
0`00Oomoo`07Ool0019oo`8000=oo`8001Qoo`D000=c^`H000A[N`03001Jne[k00IJn`030012WD:M
00=2W@03000Ag17L008Ag08000<JGP03000fG3IL008fG0L000=6>P03001>>4hh00I>>003001FF5IH
00IFF003001JN5Yh00=JN003001VfFKI00=Vf@05001[66/HJaP00005LeP01000MkUg^GNi0P0000=c
F000KgX01Fmj00<006Ml000027oo00<007ooOol01Woo000GOol01000Ool000005Goo00<006?MHmd0
0f?M1P001V]k0P002E[k00<004:M@Yd00d:M00<0017L4M`00Q7L0`000QYN00<003IL=U`013IL00<0
04HjASX01DHj00<004hhCSP01Dhh00<005IHEUP01eIH00D005YhFWQJN00000MVf@03001[600000Ic
F007001g^GNiMkT007Ni00001Fmj0P000fMl00<007ooOol01Woo00<007ooOol01Goo000GOol00`00
Oomoo`03000AOol20005Hmd400000f?M001[N`05Jg/01000I_UVn@0025[k2P000Q7L100000<JGP00
=U`01CIL00<004HjASX014Hj00<004hhCSP01Thh00<005IHEUP01UIH00@005YhFWP000UVf@03001c
F7=H00AcF003001g^GNi00=g^@03001cnP0000=_NP03001WO6Ml00=WO003001oogoo00Ioo`03001o
ogoo00Eoo`005goo00<007ooOol00Woo0`003Woo00<006?MHmd00f?M10000V?M00<006]kJg/016]k
00<006KiI_T00VKi00<005[kF_/00U[k0`0000=FN@00AST014Hi00<002gJ;MX01@0000<1>0000000
1SIL00<004HjASX014Hj00<004hhCSP01Thh00<005IHEUP01UIH00<005Yh00002VKI00<007=HLeP0
0g=H00<007NiMkT00gNi00L007?j001cnP00KgX00007Ig`00`00Oomoo`06Ool00`00Oomoo`04Ool0
00Eoo`8000Aoo`8000=oo`<000Aoo`8000Aoo`@000]oo`04001SgF?MHmd:0005Jg/00`00I_UVn@04
I_T01000F_/000001EIi00<004HiAST00dHi00<002gJ;MX00RgJ0`0000<1>00000001SIL00<004Hj
ASX014Hj00<004hhCSP01Dhh00<005IHEUP01UIH0P002VKI00<006mJ00000g=H00<007NiMkT00gNi
00@007?jLoYcnP<000QWO003001oogoo00Eoo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05
001oogooOol00008Ool30005Ool:0006Je/00`00JcT0000200000f]k001Vn@07I_T00`00EWUFN@05
EWT00`00ASU6>@03AST00`00;MX]fP03;MX01@003GX000000CP00P0013IL00<004HjASX014Hj00<0
04hhCSP01Thh00<005IHEUP00eIH0P0000=J^@00I]T026KI00D006mJKeY_FP00009cF003001g^GNi
009g^@03001cnW?j009cnP03001Gn`0000QWO003001oogoo009oo`<0009oo`03001oogoo00=oo`00
17oo00@007ooOol000]oo`03001oogoo008000Yoo`H000<_G@<00003Eil0000000I[F`03001[>F/i
00=[>@03001VnFKi00IVn@8000QFN@030016>DHi00=6>@03000]fRgJ00<]fP05000=NP000001>004
0002=U`00`00ASY6>P03ASX00`00CSQ>>006CSP01000EUQFF5IH0P0015Zi00<006KII]T01FKI00<0
06mJKeX00VmJ00<007=H000017Ni00@007?jLoYcnP800004Eo/005Ok0006Ig`60005Ool00`00Oomo
o`03Ool000Aoo`04001oogoo0008Ool30003Ool2000>Ool50003Eil00`00Je][F`05Je/00`00JcU[
>@04JcT00`00I_UVn@05I_T01000H[QR^00025Ii00<004HiAST00dHi00<002gJ;MX00RgJ00<000ej
3GX010000Q6k0`001THj00<004hhCSP01Dhh00@005IH000000IJ^@03001VfFKI00AVf@03001_FVmJ
00A_FP8000=g^@03001cnW?j008000AGn`03001Gn`00009WO0<0009??`03001oogoo00Ioo`03001o
ogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo00aoo`<00006Ool00000
001GW`001f]K00<006/iJcT016/i00<006/H00001FKi00D006:hH[QR^00000QFN@030016>DHi00=6
>@03000]fRgJ00<]fP05000=NPej0000]P0200044K/30003ASX00`00CSQ>>004CSP30009F[T00`00
I]UVf@02I]T00`00KeY_FP04KeX01P00KoY_nP00MkT0009cnP03001GneOk00=Gn`H000I??`03001o
ogoo00Eoo`03001oogoo00=oo`001Goo0P002Goo10000Woo00<007ooOol02Goo0`001goo0P001V]K
00<006/iJcT016/i00D006/HJaQ[600000=Vn@03001R^6:h00=R^003001FNEIi00EFN@X000<]fP03
000=NPej008000030;H0016k008A^`03000f?3Hl00@0009>>0@0009>V`03001J^EZi00QJ^@04001V
fFKI0006KeX01000KoY_nVoj3P000R<j00<004loCcl014lo00<007ooOol01Woo00<007ooOol00Woo
000FOol00`00Oomoo`06Ool3000<Ool20003Je/00`00JcU[>@04JcT00`00JaQ[6003JaP00`00I_T0
0007H[P01000EWUFNEIi0`0000=BE`00BQP014XH00<003gh?OP010000`ej0P0000<0]P0000000Q6k
00<003Hl=S`00SHl0`001djK00<005ZiF[T01eZi00<006KI00001VmJ00D006ojKoY_nP00009?]PP0
00826`H000I??`03001oogoo00Eoo`03001oogoo009oo`005Goo0P001Goo0`004Goo00<006]K0000
1V/i00<006/HJaP01F/H00<006:hH[P01V:h00@005Ii000000ABE`03001:64XH00A:6003000mn3gh
008mn0@000083GX000000001>0004K/000Df?003001>VdjK00I>V`03001J^EZi00MJ^@<000E_FP06
001_nVojKoX004nf0P001T:M00<002HL00000`0000<1VP000000100014lo00<007ooOol017oo00<0
07ooOol00Woo000EOol01000Oomoogoo0`005Goo0P001V/i00<006/HJaP01F/H00<006:hH[P01V:h
0P001e9G00<004XHBQP00dXH00<003gh?OP00cgh00<002gJ;MX00`0000<0n0000CP00`001CHl00<0
04jKCY/01TjK00<005ZiF[T01EZi00D006=LHeaSG00000=_FP07001_nVojKoX004nf00000U[k00<0
04:M@Yd014:M00@002HL9Q`V70<000030IX00000008VOPD000Ioo`03001oogoo009oo`005Goo1000
6Woo0P000f/i00<006/HJaP01F/H00<006[h00001V:h00<005jH00001e9G00<004XHBQP014XH00<0
03gh?OP00cgh00<002gJ;MX00RgJ0`000P4h0P0013Hl00<004jKCY/01TjK00<005ZiF[T00eZi0P00
1F=L00<006mJ00000foj00<004nf000015[k00<004:M@Yd014:M00<002HL9Q`00RHL0P0000<1VP00
00001BIn0P001Goo00<007ooOol00Woo000>Ool30004Ool2000NOol00`00JcT00007JaP01@00J_QZ
n6[h000016:h00D005jHGYQNV00000IBE`03001:64XH00A:6003000mn3gh00<mn003000]fRgJ008]
fP03000Ag000008000030CP003Hl00@f?003001>VdjK00I>V`03001J^EZi008000MSG0@00004KoX0
00000005F_/:00029Q`00`002I/000020IX200049Wh00`00Oomoo`03Ool00`00Oomoo`02Ool000mo
o`03001oogoo00Eoo`03001oogoo01aoo`03001[66/H00E[6003001Zn6[h009Zn005001R^6:hH[P0
0004GYP00`00DUMBE`05DUL00`00BQQ:6003BQP00`00?OPmn004?OP00`00;MX]fP02;MX00`004M`A
g0060003=S`00`00CY]>V`05CY/00`00F[T00008He`01000EoaGo5Ol0`0015[k0`002Woo100000D9
V`00000000Fk008000<VOP03001oogoo009oo`03001oogoo009oo`003goo00<007ooOol01Woo00<0
07ooOol077oo00<006/HJaP00f/H00<006[hJ_P016[h00<006:h00001EjH00<0059GDUL01E9G00<0
04XHBQP014XH00<003gh?OP00cgh00<002gJ;MX00bgJ00<0017L4M`02@0014jK1P001V=L0P000UOl
0P000fLI00@005[k0000011oo`<000032I/000000085^`<000039Wh007oo00=oo`03001oogoo005o
o`003goo00<007ooOol01goo00<007ooOol077oo00D006/HJaQ[600000QZn003001NV5jH00ENV003
001BEe9G00ABE`03001:64XH00A:6003000mn3gh00<mn003000]fRgJ00@]fP04000Ag17L4M`40002
5Sd50006Aal01000HeaSG6=L0P000UOl0P001VLI00<007ooOol04Woo0P000PVK00<000Fk1K/00PFk
0P0017oo00<007ooOol00Goo000?Ool00`00Oomoo`08Ool2000MOol00`00JaP00008J_P20007GYP0
0`00DUMBE`05DUL01000BQQ:64XH6P000QHm00<004LOAal01TLO00<006=L00000UOl0`001VLI00<0
07ooOol05Goo00<000VK2I/00P0010Fk00D007ooOomoo`0000=oo`003Woo0P0037oo00<007ooOol0
6goo00<006[hJ_P01V[h00@006KHI]P000MNV003001BEe9G00=BE`D000=>5`03001:64XH00=:6003
0016>DHi00A6>@03000nOCim008nO@T000A77`P00003KeX006LI00EW6@03001oogoo01Ioo`@00003
0GX000000085^`05001oogooOol00003Ool001eoo`03001oogoo01Yoo`03001Zn6[h00EZn005001V
f6KHI]P00007GYP01000DUMBEe9G0P0000=B=`00CQL01DhG00<004XHBQP014XH00<004HiAST014Hi
00<003im?Wd01Cim2@000QlJ0`0016mJ00<006LIIaT00fLI00<007ooOol067oo00@007oo00000081
NP8000051K/007ooOol00003Ool001ioo`03001oogoo01Yoo`03001Zn6[h00=Zn003001Vf6KH009V
f003001NV5jH00ENV004001BE`000004DSL00`00CQM>5`04CQL00`00BQQ:6003BQP00`00ASU6>@05
AST00`00?WdnO@02?Wd20003F]/200000eNO000000040005KeX00`00KaP00003IaT00`00Oomoo`0J
Ool01000Oomoo`000`5j00D000Fk001oo`0000=oo`007goo0P006goo00D006[hJ_QZn00000IVf003
001NV5jH00ANV08000IB=`03001>5dhG00A>5`03001:64XH00A:60030016>DHi00E6>@04000nO@00
0006F]/30003Ji`00`00KeY_FP03KeX01`00KaQ_66lH001W6@0001ioo`03001oogoo00800081NP80
0003Ool007oo009oo`008Goo00<007ooOol06Goo00<006[h00001fKH00@005jHGYQNV0<00003EUP0
058g00EB=`03001>5dhG00A>5`03001:64XH00A:60030016>DHi00E6>@8000MJf`07001W6VLJIaX0
06^L00001FmJ00<006lHKaP00flH00<007ooOol07Goo00<007ooOol00Woo00L0005j0GX1NP00Ool0
0002Ool0029oo`03001oogoo01Uoo`03001Vf6KH00IVf004001NV0000004EUP00`00DSMB=`05DSL0
0`00CQM>5`04CQL00`00BQQ:6004BQP01000ASU6>DHi0`0000=FN@00F]/01E[K00<006LJIaX00fLJ
00<006mJKeX00fmJ00<006lHKaP016lH00<007ooOol07Woo00<007ooOol00Woo0P0000@1NP00Ool0
009oo`008goo00<007ooOol067oo00<006KHI]P01VKH0P001eIH00<0058gDSL0158g00<004hGCQL0
14hG00<004XHBQP014XH00@004Hi000000EFN@03001Jfe[K009Jf`03001W6VLJ00AW6P8000=_FP03
001_66lH00A_6003001oogoo021oo`03001oogoo00=oo`@0009oo`0097oo00<007ooOol067oo00<0
06KHI]P016KH00<005ji00001eIH00<0058gDSL0158g00<004hGCQL014hG00<004XHBQP00dXH0`00
25Ii00@005[KF]/000IW6P06001_66lH001_FP001VlH00<007ooOol08Woo00<007ooOol017oo0P00
0Woo000UOol2000IOol01000I]QVf6KH0P000eji00<005IHEUP015IH00<0058gDSL01E8g00<004hG
CQL014hG00<004XH00000P000U8g00<005IiEWT01UIi00<005[K00001VLJ00<006lHKaP00VlH00<0
06lHKaP016lH00<007ooOol08goo00<007ooOol01Goo0@000Goo0Goo000WOol00`00Oomoo`0GOol0
0`00I]P00005G[T00`00EUQFF005EUP00`00DSMB=`04DSL00`00CQM>5`03CQL30006DSL00`00EWUF
N@05EWT20006IaX00`00KaQ_6003KaP20005KaP00`00Oomoo`0TOol00`00Oomoo`04Ool10001Ool1
Ool002Qoo`03001oogoo01Moo`03001N^Eji00EN^@03001FF5IH00AFF003001B=e8g00AB=`03001>
5dhG00<00003DSL0058g00IB=`03001FNEIi00AFN@04001N^5jh0005IaX00`00KaQ_6002KaP01000
KcQ_>0000flH00<007ooOol09Woo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0FOol0
0`00G[UN^@05G[T00`00EUQFF004EUP00`00DSMB=`03DSL40005DSL00`00DSMB=`05DSL00`00EWUF
N@02EWT00`00G[QN^002G[P01@00IaYW6VLJ000016lH00<006lhKcP00Vlh00<006lH0000:Goo00<0
07ooOol017oo0@000Goo000ZOol2000GOol00`00G[UN^@04G[T00`00EUQFF005EUP00`00DSL00002
0002CSL00`00DSMB=`04DSL00`00DSMB=`06DSL01000EWUFN@001Ujh00<006LJ00001FlH00<006lh
KcP00flh00<007ooOol0:Goo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0EOol00`00G[UN
^@04G[T00`00EUQFF003EUP30005CSL00`00DSMB=`04DSL00`00DSMB=`06DSL00`00EWT00008G[P0
0`00KaQ_6003KaP00`00KcQ_>004KcP00`00Oomoo`0ZOol00`00Oomoo`02Ool10001Ool002eoo`03
001oogoo01Aoo`03001N^Eji00AN^@03001FF5IH00<00003CUP004hg00I>=`03001B=e8g00AB=`03
001B=e8g00EB=`8000UN^003001_66lH009_6003001_>6lh00A_>003001oogoo02aoo`05001oogoo
Ool00001Ool002ioo`03001oogoo01Aoo`04001N^EjiG[T50005CUP00`00CSM>=`04CSL00`00DSMB
=`04DSL00`00DSMB=`04DSL00`00FWP00008G[P00`00J_P00003KaP00`00KcQ_>003KcP00`00Oomo
o`0]Ool01@00Oomoogoo00000Goo000_Ool2000EOol30003CY/00`00CUQ>F004CUP00`00CSM>=`04
CSL00`00DSMB=`05DSL01000DSMB=e8g0P000eYh00<005jhG[P015jh00L006[hJ_QZn000KaP00005
KcP00`00Oomoo`0_Ool01000Oomoo`000Goo000aOol00`00Oomoo`0BOol00`00CY]>V`04CY/00`00
CUQ>F003CUP00`00CSM>=`05CSL00`00DSMB=`04DSL01000DSMB=`001EYh00<005jhG[P00ejh00<0
06[hJ_P00f[h00<006lhKcP00flh00<007ooOol0<Goo00<007oo00000Goo000bOol00`00Oomoo`0B
Ool00`00CY]>V`03CY/00`00CUQ>F004CUP00`00CSM>=`04CSL00`00DSMB=`04DSL00`00DSL00007
FWP01@00G[QN^5jh00001V[h0P000flh00<007ooOol0<Woo00<007ooOol00@00000cOol00`00Oomo
o`0BOol00`00CY]>V`03CY/00`00CUQ>F003CUP00`00CSM>=`04CSL00`00DSMB=`04DSL20008FWP0
1000G[QN^0001V[h00H006lhKcP006lh000fOol10001Ool1000003Aoo`80019oo`03001>VdjK00=>
V`03001>F4iH00=>F003001>=dhg00A>=`04001B=e8gDSL300000eIH001JN007FWP00`00G[P00006
J_P00`00KcQ_>002KcP00`00Oomoo`0fOol2000003Ioo`03001oogoo011oo`03001>VdjK00=>V`03
001>F4iH00=>F003001>=dhg00A>=`<000EFF003001JN5Yh00IJN003001Zn6[h00EZn003001_>6lh
009_>003001oogoo03Eoo`<00000=goo00<007ooOol047oo100000=>V`00CUP014iH00<004hgCSL0
0dhg0P001eIH00<005YhFWP01EYh0P001V[h00<006lhKcP00Vlh00<007ooOol0<Woo10000goo000h
Ool00`00Oomoo`0?Ool01000<U`bG39L1P0000A>F000CSM>=`<00003CUP005IH00MFF003001JN5Yh
00=JN004001Rf6;H0004J_P00`00KcQ_>002KcP00`00Oomoo`0_Ool40007Ool003Uoo`03001oogoo
00moo`03000bG39L008bG0040016>THjASX40005CUP00`00EUQFF005EUP00`00FWQJN002FWP00`00
H]QRf002H]P01@00J_QZn6[h000016lh00<007ooOol0;7oo0`002goo000jOol2000@Ool00`00<U`b
G002<U`00`00ASY6>P02ASX00`00CUQ>F004CUP00`00EUQFF006EUP01000FWQJN0001V;H00<006[h
000016lh00<007ooOol0:Goo10003Woo000lOol00`00Oomoo`0=Ool00`00<U`bG002<U`00`00ASY6
>P03ASX00`00CUQ>F004CUP00`00EUQFF005EUP00`00FWP00008H]P00`00KcQ_>002KcP00`00Oomo
o`0VOol4000BOol003eoo`03001oogoo00eoo`8000<bG0030016>THj0096>P03001>F4iH00A>F003
001FF5IH00EFF08000QRf003001_FP0000=_>003001oogoo029oo`@001Ioo`00?Woo00<007ooOol0
3Goo0`0000<bG000ASX014Hj00<004iHCUP00diH00<005IHEUP00eIH0P0000=J^@00H]P01V;H00L0
06mJKeY_FP00KcP0000QOol4000JOol003moo`8000ioo`040001NP000005ASX00`00CUQ>F004CUP0
1@00EUQFF5IH000015Zi00<006;HH]P00f;H00<006mJKeX00fmJ00<007ooOol077oo10007Woo0011
Ool00`00Oomoo`0<Ool00`000GX1NP030003ASX00`00CUQ>F003CUP01000EUP000001EZi00<006;H
H]P00V;H00<006mJKeX016mJ00<007ooOol06Goo0`008Woo0012Ool00`00Oomoo`0<Ool30002<S`4
0003CUP40008F[T01000H]QRf0001VmJ00<007ooOol05Woo10009Goo0013Ool00`00Oomoo`0=Ool2
0004<S`40003CY/00`00F[UJ^@06F[T00`00H]P00006KeX00`00Oomoo`0COol4000YOol004Aoo`80
00ioo`8000@b?003001>VdjK00A>V`03001J^EZi00EJ^@<000E_FP03001oogoo011oo`@002eoo`00
AWoo00<007ooOol02goo0`00138l00<004jKCY/014jK00<005ZiF[T00eZi00D006=LHeaSG00000=_
FP03001oogoo00eoo`@0035oo`00Agoo00<007ooOol02Woo00<007ooOol00P000S8l00<004jKCY/0
14jK00@005ZiF[UJ^@8000ESG003001_FP0000aoo`@003Eoo`00B7oo00<007ooOol02Goo00@007oo
Oomoo`<00003<S`004jK00E>V`03001J^@0000MSG08000Uoo`@003Uoo`00BGoo0P002Goo00<007oo
Ool00goo0`0014jK10001V=L0P0027oo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool6
0004Aal00`00HeaSG002He`00`00Oomoo`04Ool40010Ool004aoo`03001oogoo00Eoo`03001oogoo
00Moo`<000E77`04001SG0000003Ool40014Ool004eoo`03001oogoo00Aoo`03001oogoo00Yoo`L0
0003Ool00000008004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo001?Ool20003
Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03001oo`0000Eo
o`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.458 0 0 r
.37648 .74293 .39996 .75138 .43767 .76622 .4143 .7635 Metetra
.016 .586 .768 r
.4143 .7635 .43767 .76622 .47668 .7661 .45374 .76564 Metetra
.644 .95 .948 r
.45374 .76564 .47668 .7661 .51653 .74849 .49424 .74615 Metetra
.863 .939 .869 r
.49424 .74615 .51653 .74849 .55667 .71474 .5351 .70676 Metetra
.898 .869 .801 r
.5351 .70676 .55667 .71474 .59664 .66972 .5757 .65361 Metetra
.884 .81 .771 r
.5757 .65361 .59664 .66972 .63628 .62045 .6158 .59549 Metetra
.849 .76 .765 r
.6158 .59549 .63628 .62045 .67576 .57447 .65555 .54164 Metetra
.794 .713 .774 r
.65555 .54164 .67576 .57447 .71557 .53828 .6955 .49987 Metetra
.715 .661 .795 r
.6955 .49987 .71557 .53828 .75637 .51631 .73644 .47545 Metetra
.603 .599 .82 r
.73644 .47545 .75637 .51631 .79887 .5104 .77921 .47052 Metetra
.449 .522 .841 r
.77921 .47052 .79887 .5104 .84361 .5196 .8245 .48398 Metetra
.258 .435 .846 r
.8245 .48398 .84361 .5196 .89084 .54022 .87264 .51148 Metetra
.065 .362 .832 r
.87264 .51148 .89084 .54022 .94033 .56622 .92342 .5458 Metetra
0 .35 .818 r
.92342 .5458 .94033 .56622 .99133 .59004 .97598 .57772 Metetra
0 .39 .836 r
.35282 .72181 .37648 .74293 .4143 .7635 .39078 .74166 Metetra
.473 .681 .957 r
.39078 .74166 .4143 .7635 .45374 .76564 .43049 .74332 Metetra
.719 .759 .884 r
.43049 .74332 .45374 .76564 .49424 .74615 .47141 .72364 Metetra
.812 .77 .814 r
.47141 .72364 .49424 .74615 .5351 .70676 .51282 .68431 Metetra
.85 .772 .776 r
.51282 .68431 .5351 .70676 .5757 .65361 .55408 .63139 Metetra
.864 .776 .763 r
.55408 .63139 .5757 .65361 .6158 .59549 .59487 .57356 Metetra
.859 .783 .775 r
.59487 .57356 .6158 .59549 .65555 .54164 .6353 .51992 Metetra
.831 .79 .813 r
.6353 .51992 .65555 .54164 .6955 .49987 .67587 .4782 Metetra
.753 .784 .878 r
.67587 .4782 .6955 .49987 .73644 .47545 .71733 .45359 Metetra
.576 .727 .945 r
.71733 .45359 .73644 .47545 .77921 .47052 .76052 .44821 Metetra
.289 .575 .938 r
.76052 .44821 .77921 .47052 .8245 .48398 .80613 .46096 Metetra
.041 .405 .858 r
.80613 .46096 .8245 .48398 .87264 .51148 .85453 .48756 Metetra
0 .337 .816 r
.85453 .48756 .87264 .51148 .92342 .5458 .90555 .52087 Metetra
.045 .391 .851 r
.90555 .52087 .92342 .5458 .97598 .57772 .95838 .55178 Metetra
.366 .526 .883 r
.3292 .68798 .35282 .72181 .39078 .74166 .36746 .70068 Metetra
.572 .589 .831 r
.36746 .70068 .39078 .74166 .43049 .74332 .40733 .69913 Metetra
.676 .625 .79 r
.40733 .69913 .43049 .74332 .47141 .72364 .44839 .68093 Metetra
.74 .658 .768 r
.44839 .68093 .47141 .72364 .51282 .68431 .49007 .64737 Metetra
.787 .696 .764 r
.49007 .64737 .51282 .68431 .55408 .63139 .53184 .60305 Metetra
.824 .745 .777 r
.53184 .60305 .55408 .63139 .59487 .57356 .5734 .55464 Metetra
.849 .811 .812 r
.5734 .55464 .59487 .57356 .6353 .51992 .61479 .50932 Metetra
.839 .896 .879 r
.61479 .50932 .6353 .51992 .67587 .4782 .65637 .47327 Metetra
.703 .95 .955 r
.65637 .47327 .67587 .4782 .71733 .45359 .69871 .45074 Metetra
.317 .798 .903 r
.69871 .45074 .71733 .45359 .76052 .44821 .7425 .44347 Metetra
0 .529 .771 r
.7425 .44347 .76052 .44821 .80613 .46096 .78829 .45054 Metetra
0 .413 .779 r
.78829 .45054 .80613 .46096 .85453 .48756 .8364 .46844 Metetra
.022 .44 .869 r
.8364 .46844 .85453 .48756 .90555 .52087 .88673 .4914 Metetra
.263 .504 .898 r
.88673 .4914 .90555 .52087 .95838 .55178 .93865 .51221 Metetra
.552 .565 .821 r
.30572 .64405 .3292 .68798 .36746 .70068 .34452 .64474 Metetra
.606 .552 .767 r
.34452 .64474 .36746 .70068 .40733 .69913 .38449 .63793 Metetra
.644 .56 .745 r
.38449 .63793 .40733 .69913 .44839 .68093 .42543 .62247 Metetra
.678 .586 .745 r
.42543 .62247 .44839 .68093 .49007 .64737 .46706 .59897 Metetra
.715 .631 .763 r
.46706 .59897 .49007 .64737 .53184 .60305 .50907 .56969 Metetra
.756 .709 .807 r
.50907 .56969 .53184 .60305 .5734 .55464 .5513 .53801 Metetra
.793 .839 .886 r
.5513 .53801 .5734 .55464 .61479 .50932 .59374 .50769 Metetra
.708 .971 .93 r
.59374 .50769 .61479 .50932 .65637 .47327 .63655 .4821 Metetra
.63655 .4821 .65637 .47327 .69871 .45074 .68006 .46359 Metetra
.086 0 0 r
.68006 .46359 .69871 .45074 .7425 .44347 .72462 .45318 Metetra
.19 0 0 r
.72462 .45318 .7425 .44347 .78829 .45054 .77056 .45033 Metetra
.065 0 0 r
.77056 .45033 .78829 .45054 .8364 .46844 .81806 .45304 Metetra
.297 .677 .975 r
.81806 .45304 .8364 .46844 .88673 .4914 .86708 .45813 Metetra
.496 .618 .902 r
.86708 .45813 .88673 .4914 .93865 .51221 .9173 .4618 Metetra
.646 .592 .78 r
.28224 .59462 .30572 .64405 .34452 .64474 .32182 .58103 Metetra
.63 .546 .741 r
.32182 .58103 .34452 .64474 .38449 .63793 .36193 .56807 Metetra
.622 .53 .729 r
.36193 .56807 .38449 .63793 .42543 .62247 .4026 .55588 Metetra
.62 .536 .738 r
.4026 .55588 .42543 .62247 .46706 .59897 .44388 .54438 Metetra
.625 .571 .773 r
.44388 .54438 .46706 .59897 .50907 .56969 .48582 .53333 Metetra
.636 .654 .851 r
.48582 .53333 .50907 .56969 .5513 .53801 .52847 .52235 Metetra
.611 .82 .98 r
.52847 .52235 .5513 .53801 .59374 .50769 .57184 .51101 Metetra
.273 .776 .778 r
.57184 .51101 .59374 .50769 .63655 .4821 .61594 .4989 Metetra
.089 0 0 r
.61594 .4989 .63655 .4821 .68006 .46359 .66075 .48572 Metetra
.17 0 0 r
.66075 .48572 .68006 .46359 .72462 .45318 .70625 .47133 Metetra
.087 0 0 r
.70625 .47133 .72462 .45318 .77056 .45033 .75241 .45581 Metetra
.261 .759 .698 r
.75241 .45581 .77056 .45033 .81806 .45304 .79924 .43939 Metetra
.645 .875 .984 r
.79924 .43939 .81806 .45304 .86708 .45813 .84676 .42245 Metetra
.668 .693 .864 r
.84676 .42245 .86708 .45813 .9173 .4618 .89502 .40543 Metetra
.706 .621 .76 r
.25845 .54517 .28224 .59462 .32182 .58103 .29899 .51791 Metetra
.653 .561 .738 r
.29899 .51791 .32182 .58103 .36193 .56807 .33931 .49912 Metetra
.606 .522 .734 r
.33931 .49912 .36193 .56807 .4026 .55588 .37966 .48999 Metetra
.559 .503 .749 r
.37966 .48999 .4026 .55588 .44388 .54438 .42045 .48988 Metetra
.505 .508 .793 r
.42045 .48988 .44388 .54438 .48582 .53333 .46207 .49645 Metetra
.419 .551 .882 r
.46207 .49645 .48582 .53333 .52847 .52235 .50485 .50597 Metetra
.202 .619 .949 r
.50485 .50597 .52847 .52235 .57184 .51101 .54891 .51389 Metetra
.131 0 0 r
.54891 .51389 .57184 .51101 .61594 .4989 .59415 .51576 Metetra
.193 0 0 r
.59415 .51576 .61594 .4989 .66075 .48572 .64024 .50817 Metetra
.082 0 0 r
.64024 .50817 .66075 .48572 .70625 .47133 .68676 .48961 Metetra
.68676 .48961 .70625 .47133 .75241 .45581 .7333 .46089 Metetra
.668 .969 .792 r
.7333 .46089 .75241 .45581 .79924 .43939 .77966 .42493 Metetra
.835 .91 .889 r
.77966 .42493 .79924 .43939 .84676 .42245 .82594 .38603 Metetra
.774 .736 .818 r
.82594 .38603 .84676 .42245 .89502 .40543 .87252 .34892 Metetra
.752 .658 .757 r
.23387 .5009 .25845 .54517 .29899 .51791 .27542 .46308 Metetra
.681 .597 .755 r
.27542 .46308 .29899 .51791 .33931 .49912 .31605 .43978 Metetra
.596 .54 .763 r
.31605 .43978 .33931 .49912 .37966 .48999 .3562 .43286 Metetra
.485 .487 .784 r
.3562 .43286 .37966 .48999 .42045 .48988 .39651 .44134 Metetra
.328 .433 .817 r
.39651 .44134 .42045 .48988 .46207 .49645 .43773 .46149 Metetra
.093 .378 .84 r
.43773 .46149 .46207 .49645 .50485 .50597 .48043 .48715 Metetra
0 .324 .759 r
.48043 .48715 .50485 .50597 .54891 .51389 .52489 .51058 Metetra
.279 0 0 r
.52489 .51058 .54891 .51389 .59415 .51576 .57097 .52389 Metetra
.118 0 0 r
.57097 .52389 .59415 .51576 .64024 .50817 .61816 .52087 Metetra
.61816 .52087 .64024 .50817 .68676 .48961 .66566 .49874 Metetra
.655 .955 .739 r
.66566 .49874 .68676 .48961 .7333 .46089 .71278 .459 Metetra
.882 .985 .82 r
.71278 .459 .7333 .46089 .77966 .42493 .7591 .40703 Metetra
.896 .879 .81 r
.7591 .40703 .77966 .42493 .82594 .38603 .80475 .35056 Metetra
.835 .764 .783 r
.80475 .35056 .82594 .38603 .87252 .34892 .85034 .29771 Metetra
.79 .705 .771 r
.20796 .46578 .23387 .5009 .27542 .46308 .25041 .42225 Metetra
.716 .662 .796 r
.25041 .42225 .27542 .46308 .31605 .43978 .29145 .39641 Metetra
.589 .599 .83 r
.29145 .39641 .31605 .43978 .3562 .43286 .33161 .39045 Metetra
.376 .496 .853 r
.33161 .39045 .3562 .43286 .39651 .44134 .37171 .40322 Metetra
.074 .345 .819 r
.37171 .40322 .39651 .44134 .43773 .46149 .41266 .43034 Metetra
0 .209 .725 r
.41266 .43034 .43773 .46149 .48043 .48715 .45523 .46444 Metetra
0 .195 .672 r
.45523 .46444 .48043 .48715 .52489 .51058 .49986 .49612 Metetra
0 .369 .75 r
.49986 .49612 .52489 .51058 .57097 .52389 .54646 .51554 Metetra
.229 .717 .925 r
.54646 .51554 .57097 .52389 .61816 .52087 .59441 .51485 Metetra
.672 .948 .958 r
.59441 .51485 .61816 .52087 .66566 .49874 .64278 .49051 Metetra
.863 .951 .868 r
.64278 .49051 .66566 .49874 .71278 .459 .69063 .44444 Metetra
.911 .897 .801 r
.69063 .44444 .71278 .459 .7591 .40703 .73742 .3835 Metetra
.904 .84 .772 r
.73742 .3835 .7591 .40703 .80475 .35056 .78325 .31733 Metetra
.868 .787 .769 r
.78325 .31733 .80475 .35056 .85034 .29771 .8288 .25592 Metetra
.82 .769 .804 r
.18021 .44195 .20796 .46578 .25041 .42225 .22326 .39843 Metetra
.753 .773 .87 r
.22326 .39843 .25041 .42225 .29145 .39641 .26479 .37236 Metetra
.568 .729 .95 r
.26479 .37236 .29145 .39641 .33161 .39045 .30529 .36589 Metetra
.184 .537 .925 r
.30529 .36589 .33161 .39045 .37171 .40322 .3456 .37789 Metetra
0 .276 .754 r
.3456 .37789 .37171 .40322 .41266 .43034 .38666 .40403 Metetra
0 .172 .669 r
.38666 .40403 .41266 .43034 .45523 .46444 .42932 .43704 Metetra
0 .254 .744 r
.42932 .43704 .45523 .46444 .49986 .49612 .47407 .46762 Metetra
.126 .476 .897 r
.47407 .46762 .49986 .49612 .54646 .51554 .5209 .4861 Metetra
.495 .681 .949 r
.5209 .4861 .54646 .51554 .59441 .51485 .56927 .48473 Metetra
.718 .767 .891 r
.56927 .48473 .59441 .51485 .64278 .49051 .61824 .46002 Metetra
.821 .792 .824 r
.61824 .46002 .64278 .49051 .69063 .44444 .66688 .41388 Metetra
.867 .799 .781 r
.66688 .41388 .69063 .44444 .73742 .3835 .7146 .35305 Metetra
.884 .805 .766 r
.7146 .35305 .73742 .3835 .78325 .31733 .7614 .28706 Metetra
.878 .811 .779 r
.7614 .28706 .78325 .31733 .8288 .25592 .8079 .22574 Metetra
.831 .855 .865 r
.15024 .4294 .18021 .44195 .22326 .39843 .19347 .39162 Metetra
.739 .922 .953 r
.19347 .39162 .22326 .39843 .26479 .37236 .2355 .36761 Metetra
.407 .836 .955 r
.2355 .36761 .26479 .37236 .30529 .36589 .27672 .35916 Metetra
0 .527 .769 r
.27672 .35916 .30529 .36589 .3456 .37789 .31781 .36533 Metetra
0 .33 .682 r
.31781 .36533 .3456 .37789 .38666 .40403 .35958 .38253 Metetra
0 .332 .771 r
.35958 .38253 .38666 .40403 .42932 .43704 .40275 .40489 Metetra
.115 .442 .878 r
.40275 .40489 .42932 .43704 .47407 .46762 .44779 .42507 Metetra
.39 .534 .88 r
.44779 .42507 .47407 .46762 .5209 .4861 .49473 .43556 Metetra
.565 .589 .836 r
.49473 .43556 .5209 .4861 .56927 .48473 .5432 .4305 Metetra
.672 .629 .798 r
.5432 .4305 .56927 .48473 .61824 .46002 .59248 .40727 Metetra
.745 .669 .777 r
.59248 .40727 .61824 .46002 .66688 .41388 .64183 .36728 Metetra
.799 .714 .771 r
.64183 .36728 .66688 .41388 .7146 .35305 .69072 .31565 Metetra
.839 .768 .784 r
.69072 .31565 .7146 .35305 .7614 .28706 .73909 .25968 Metetra
.861 .835 .82 r
.73909 .25968 .7614 .28706 .8079 .22574 .78739 .20708 Metetra
.758 .944 .942 r
.11794 .42594 .15024 .4294 .19347 .39162 .16081 .39877 Metetra
.497 .907 .848 r
.16081 .39877 .19347 .39162 .2355 .36761 .20329 .37882 Metetra
.20329 .37882 .2355 .36761 .27672 .35916 .24559 .36711 Metetra
.045 0 0 r
.24559 .36711 .27672 .35916 .31781 .36533 .28809 .3631 Metetra
0 .575 .738 r
.28809 .3631 .31781 .36533 .35958 .38253 .33127 .36473 Metetra
.256 .664 .965 r
.33127 .36473 .35958 .38253 .40275 .40489 .37556 .36877 Metetra
.456 .614 .917 r
.37556 .36877 .40275 .40489 .44779 .42507 .42123 .37133 Metetra
.544 .564 .825 r
.42123 .37133 .44779 .42507 .49473 .43556 .46834 .36857 Metetra
.595 .548 .772 r
.46834 .36857 .49473 .43556 .5432 .4305 .51672 .35762 Metetra
.635 .557 .75 r
.51672 .35762 .5432 .4305 .59248 .40727 .56602 .33721 Metetra
.674 .587 .75 r
.56602 .33721 .59248 .40727 .64183 .36728 .61585 .30801 Metetra
.715 .639 .771 r
.61585 .30801 .64183 .36728 .69072 .31565 .66593 .27251 Metetra
.761 .723 .818 r
.66593 .27251 .69072 .31565 .73909 .25968 .71617 .23438 Metetra
.794 .85 .893 r
.71617 .23438 .73909 .25968 .78739 .20708 .7668 .19766 Metetra
.432 .87 .904 r
.08349 .42744 .11794 .42594 .16081 .39877 .12547 .414 Metetra
.12547 .414 .16081 .39877 .20329 .37882 .1683 .39943 Metetra
.1683 .39943 .20329 .37882 .24559 .36711 .21197 .3836 Metetra
.21197 .3836 .24559 .36711 .28809 .3631 .25645 .36658 Metetra
.462 .889 .872 r
.25645 .36658 .28809 .3631 .33127 .36473 .30171 .3486 Metetra
.665 .862 .974 r
.30171 .3486 .33127 .36473 .37556 .36877 .34771 .33007 Metetra
.666 .696 .868 r
.34771 .33007 .37556 .36877 .42123 .37133 .39444 .31143 Metetra
.639 .593 .786 r
.39444 .31143 .42123 .37133 .46834 .36857 .44192 .29307 Metetra
.619 .541 .745 r
.44192 .29307 .46834 .36857 .51672 .35762 .49019 .27531 Metetra
.61 .523 .731 r
.49019 .27531 .51672 .35762 .56602 .33721 .53931 .25827 Metetra
.609 .531 .741 r
.53931 .25827 .56602 .33721 .61585 .30801 .58935 .24186 Metetra
.616 .57 .779 r
.58935 .24186 .61585 .30801 .66593 .27251 .64038 .22581 Metetra
.631 .657 .857 r
.64038 .22581 .66593 .27251 .71617 .23438 .69245 .20972 Metetra
.626 .812 .972 r
.69245 .20972 .71617 .23438 .7668 .19766 .74555 .19312 Metetra
.048 0 0 r
.04742 .42842 .08349 .42744 .12547 .414 .08816 .42922 Metetra
.005 0 0 r
.08816 .42922 .12547 .414 .1683 .39943 .1312 .42031 Metetra
.1312 .42031 .1683 .39943 .21197 .3836 .17638 .40014 Metetra
.525 .904 .738 r
.17638 .40014 .21197 .3836 .25645 .36658 .22317 .36956 Metetra
.79 .988 .891 r
.22317 .36956 .25645 .36658 .30171 .3486 .27095 .33154 Metetra
.829 .869 .875 r
.27095 .33154 .30171 .3486 .34771 .33007 .3191 .29052 Metetra
.772 .729 .813 r
.3191 .29052 .34771 .33007 .39444 .31143 .36725 .2513 Metetra
.705 .627 .767 r
.36725 .2513 .39444 .31143 .44192 .29307 .41535 .21814 Metetra
.646 .558 .742 r
.41535 .21814 .44192 .29307 .49019 .27531 .46362 .19397 Metetra
.593 .515 .736 r
.46362 .19397 .49019 .27531 .53931 .25827 .51251 .18002 Metetra
.544 .495 .751 r
.51251 .18002 .53931 .25827 .58935 .24186 .56255 .17559 Metetra
.496 .504 .795 r
.56255 .17559 .58935 .24186 .64038 .22581 .61421 .17821 Metetra
.437 .556 .878 r
.61421 .17821 .64038 .22581 .69245 .20972 .66774 .18387 Metetra
.336 .662 .973 r
.66774 .18387 .69245 .20972 .74555 .19312 .72307 .18771 Metetra
.306 0 0 r
.01057 .42296 .04742 .42842 .08816 .42922 .05004 .43542 Metetra
.05004 .43542 .08816 .42922 .1312 .42031 .09321 .43114 Metetra
.494 .903 .814 r
.09321 .43114 .1312 .42031 .17638 .40014 .13979 .40724 Metetra
.802 .994 .87 r
.13979 .40724 .17638 .40014 .22317 .36956 .18887 .36529 Metetra
.888 .93 .841 r
.18887 .36529 .22317 .36956 .27095 .33154 .23917 .31084 Metetra
.875 .836 .804 r
.23917 .31084 .27095 .33154 .3191 .29052 .28956 .2518 Metetra
.825 .748 .778 r
.28956 .2518 .3191 .29052 .36725 .2513 .33931 .1965 Metetra
.758 .669 .764 r
.33931 .1965 .36725 .2513 .41535 .21814 .38824 .15195 Metetra
.677 .599 .761 r
.38824 .15195 .41535 .21814 .46362 .19397 .43671 .12282 Metetra
.584 .536 .767 r
.43671 .12282 .46362 .19397 .51251 .18002 .48548 .11101 Metetra
.475 .482 .785 r
.48548 .11101 .51251 .18002 .56255 .17559 .53544 .11547 Metetra
.349 .443 .816 r
.53544 .11547 .56255 .17559 .61421 .17821 .58741 .13228 Metetra
.206 .432 .859 r
.58741 .13228 .61421 .17821 .66774 .18387 .64192 .15492 Metetra
.074 .471 .889 r
.64192 .15492 .66774 .18387 .72307 .18771 .6989 .17516 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<000Uoo`L001eoo`03
001oogoo00Moo`03001oogoo02Aoo`00DWoo0`001goo10000TXh00<003gi?OT00Sgi1P005goo00<0
07ooOol027oo00<007ooOol08goo001?Ool30007Ool30007BSP00`00?OTmn@04?OT00`00;MX]fP04
000COol00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<000Qoo`800003EWP004Xh00M:>003000m
nCgi00Dmn@03000]fRgJ00<]fP<0011oo`03001oogoo00Uoo`03001oogoo029oo`00BWoo0`002Goo
00<005IhEWP00UIh00<004XhBSP01DXh00<003gi?OT01Cgi00<002gJ;MX012gJ00@001Vk000000io
o`03001oogoo00Yoo`03001oogoo025oo`00Agoo0`002Woo0P001EIh00<004XhBSP01TXh00<003gi
?OT01Cgi00<002gJ;MX012gJ00<001Vk6K/00`002goo00<007ooOol02goo00<007ooOol087oo0015
Ool2000;Ool20008EWP00`00BSQ:>005BSP00`00?OTmn@05?OT00`00;MX]fP05;MX00`006K/I^`02
6K/20009Ool00`00Oomoo`0;Ool00`00Oomoo`0POol0049oo`<000aoo`03001R^00000QFN003001:
>4Xh00E:>003000mnCgi00Dmn@03000]fRgJ00D]fP03000I^aVk00@I^`<000Ioo`03001oogoo00ao
o`03001oogoo01moo`009Goo10005goo0P003Woo00D006:hH[QR^00000QFN003001:>4Xh00E:>003
000mnCgi00Dmn@03000]fRgJ00D]fP03000I^aVk00@I^`030009o000008000=oo`03001oogoo00eo
o`03001oogoo01ioo`009Woo00<007ooOol057oo0`003goo00<006:hH[P00V:h00<005IhEWP01UIh
00<004XhBSP01DXh00<003gi?OT01Cgi00<002gJ;MX01BgJ00<001Vk6K/01AVk00@000Wl2O`9o0@0
00ioo`80021oo`007goo1@000goo00<007ooOol04Goo0P004Goo00<006:hH[P00f:h00<005IhEWP0
1UIh00<004XhBSP01DXh00<003gi?OT00Sgi2P000RgJ00<001Vk6K/01AVk00<000Wl2O`00`Wl0P00
3goo00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000BOol00`00H[QR^005H[P00`00EWQFN006
EWP00`00BSQ:>003BSP60004AOP00`00?QTn6@03?QTA00000`Wl0000000=Ool00`00Oomoo`0MOol0
02Eoo`04001oogoo000=Ool2000DOol00`00H[QR^006H[P00`00EWQFN006EWP00`00BSQ:>0030002
BQL00`00AOQ5n006AOP00`00?QTn6@05?QT00`00=S`f?005=S`00`00:[lZ_`03:[l6000;Ool00`00
Oomoo`0LOol002Ioo`8000]oo`<001Eoo`03001R^6:h00MR^003001FN5Ih00IFN0<000I:5`030015
n4Gh00E5n003000n6ChI00Hn6@03000f?3Hl00Df?003000Z_bZo008Z_`03000Z_bZo00<Z_`@000Qo
o`03001oogoo01]oo`00<Goo0P0000=oo`00Ool05Goo00<006:hH[P026:h00<005IhEWP00eIh0P00
00=B=`00BQL01TXG00<004GhAOP01TGh00<003hI?QT01ChI00<003Hl=S`01SHl00D002Zo:[lZ_`00
008Z_`<000=C?`@000Eoo`03001oogoo01]oo`00;Woo0`006Goo00<006[h00002F:h00@005IhEWQF
N08000=B=`03001:5dXG00E:5`030015n4Gh00I5n003000n6ChI00Hn6@03000f?3Hl00Hf?003000Z
_bZo00<000EC?`03001WO6Ml009WO003001oogoo009oo`8001aoo`00:goo0`006goo00@006[hJ_P0
00UR^004001FN0000006DSL00`00BQM:5`05BQL00`00AOQ5n006AOP00`00?QTn6@06?QT00`00=S`f
?004=S`50005Dcl20005Ig`20005Ool00`00Oomoo`03Ool2000DOol002Uoo`80009oo`03001oogoo
01Qoo`03001Zn6[h009Zn003001R^6:h00IR^08000QB=`03001:5dXG00E:5`030015n4Gh00I5n003
000n6ChI00Hn6@03000f?3Hl00@000AB^`04001C?e<oDcl20006Ig`01000KeY_FP0017oo00<007oo
Ool00Woo00@007ooOol001=oo`009Woo0`007Woo00<006[hJ_P00f[h00<006:hH[P01F:h00<005ZH
0000258g00<004XGBQL01TXG00<004GhAOP01DGh00<003hI?QT01ShI0`0025:k0P0000=C?`00Ig`0
1VMl00<006mJKeX00VmJ00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo
0P0087oo00<006[hJ_P016[h00<006:hH[P00f:h0P000eZH00<0058gDSL01U8g00<004XGBQL01DXG
00<004GhAOP01TGh00<003hI?QT00ShI0`0000=>F000D[/025:k0`001fMl00<006mJKeX00fmJ00<0
07ooOol00goo00@007ooOol0009oo`03001oogoo015oo`008Goo0`008Goo00<006[hJ_P01V[h00<0
06:hH[P00P001EZH00<0058gDSL01U8g00<004XGBQL01TXG00<004GhAOP01DGh00<003hI00000P00
1DiH00<005:kD[/01U:k00@006;jH_X000EWO003001_FVmJ00=_FP03001c>00000=oo`80009oo`04
001oogoo000COol001moo`8002=oo`03001Zn6[h00MZn003001R^00000MJV003001B=e8g00IB=`03
001:5dXG00I:5`030015n4Gh00A5n0<000Q>F003001B^e:k00EB^`8000=RnP05001WO6MlIg`00005
KeX01@00LcQc>7<h000017oo00<007ooOol00P0057oo000LOol3000TOol00`00J_QZn008J_P20009
FYP00`00DSMB=`06DSL00`00BQM:5`05BQL00`00AOQ5n0040002CQL00`00CUQ>F007CUP01000D[]B
^e:k0`0000=RnP00H_X00f;j00<006Ml00001VmJ00<007<hLcP00W<h00<007ooOol00Woo00<007oo
Ool05Goo000KOol2000VOol00`00J_QZn007J_P00`00H_X00009FYP00`00DSMB=`06DSL00`00BQM:
5`05BQL30007CQL00`00CUQ>F006CUP400000e:k001RnP02H_X00`00H_YRnP02H_X00`00KeY_FP04
KeX00`00LcQc>004LcP01@00Oomoogoo00005goo000DOol30004Ool00`00Ool0000TOol00`00KeT0
0007J_P01@00H_YRnV;j000025ZH00<0058gDSL01U8g00<004XGBQL00TXG0`0000=>5`00CQL01dhG
00<004iHCUP014iH10000U:k00<006;jH_X00f;j00D006;jH_YRnP0000E_FP03001c>7<h00Ec>003
001oogoo009oo`03001oogoo01Aoo`005Goo00<007ooOol00goo00@007ooOol0029oo`05001_FFmI
KeT00005J_P00`00H_YRnP02H_X00`00FYQJV007FYP00`00DSMB=`06DSL40004CQL00`00CQM>5`06
CQL01000CUQ>F4iH0`000diH00<005:k00001f;j00D006;j001[6@0000=_FP03001c>7<h00Ec>003
001_>00000=oo`8001Eoo`003Woo1@000Woo00<007ooOol00goo00D007ooOomoo`00021oo`03001_
FFmI00=_F@05001Zn6[hJ_P00006H_X00`00FYQJV006FYP00`00DSMB=`05DSL20008CQL00`00CQM>
5`06CQL30006CUP20008H_X20003JaT01000KeY_FP001W<h00@006lhKcP000Aoo`03001oogoo01=o
o`005Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol077oo00<006mIKeT01FmI00<006[h
00001f;j00<005ZHFYP01UZH00<0058gDSL00U8g0`0000=>=`00CQL01dhG00<004hGCQL00dhG1000
1diH00<005ZH000026;j0P000f/I00<006mJ00001g<h00D006lhKcQ_>00000Aoo`03001oogoo019o
o`005Goo00<007ooOol00Woo0P001Goo00<007ooOol06goo00<006mIKeT01FmI0P002F;j00<005ZH
FYP01UZH100014hg00<004hGCQL01ThG00<004hGCQL00`0014hG00<004iHCUP00TiH0P000eZH00<0
06;jH_X016;j00@006/IJaT000=[6@03001c>7<h00Ec>003001_>6lh009_>003001oogoo009oo`03
001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo00<007ooOol06Goo00<006mIKeT01FmI00<0
06]k00002F;j00<005ZHFYP01EZH0P0024hg00<004hGCQL01ThG0P001dhG00<004iHCUP00P001UZH
00<006;jH_X00V;j00D006/IJaU[6@0000=[6@03001c>7<h00Ac>003001_>6lh00=_>003001oogoo
00=oo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol05goo00<006mIKeT01FmI00D0
06]kJg][N`0000URnP04001JV5ZHFYP300000e9I001>=`07CSL00`00CQM>5`03CQL40008CQL00`00
CUP00008FYP01@00H_YRnV;j00001F/I00D006/I001c>00000Ac>003001_>6lh00E_>003001oogoo
00=oo`03001oogoo011oo`006Woo00<007ooOol01goo00<007ooOol05Goo00<006mIKeT01FmI00<0
06]kJg/00V]k00<006;jH_X01f;j00@005ZH000000EBF@03001>=dhg00E>=`03001>5dhG00<000=>
5`03001>5dhG00I>5`8000YJV003001RnP0000M[6@05001c>7<hLcP00003LcP00`00KcQ_>005KcP0
0`00Oomoo`02Ool2000BOol001Uoo`8000Yoo`03001oogoo01=oo`8000I_F@03001[Nf]k00A[N`03
001RnV;j00IRnP8000MBF@03001>=dhg00E>=`<000M>5`03001>5dhG00A>5`03001FE`0000]JV003
001[6F/I00I[6@8000=c>003001c>00000M_>003001[6@0000Eoo`03001oogoo00moo`0027oo0P00
17oo0P000goo0`000goo0P002goo00<007ooOol04Goo00<007>jLkX00P000fmI00<006]kJg/01V]k
00<006;jH_X00V;j0`0000=Ff`00DUT01U9I00<004hgCSL00Thg0`0000=>=`00CQL01dhG00<004hG
CQL00ThG0P000eIG00<005ZHFYP025ZH00<006/IJaT01F/I00<007<h000017<h00<006lhKcP01Flh
00@006/IJaT000Ioo`03001oogoo00ioo`001goo00@007ooOol000=oo`8000Ioo`04001oogoo000=
Ool00`00Oomoo`0?Ool00`00LkYc^P03LkX00`00KeT00009Jg/00`00H_X000020005E]/00`00DUUB
F@05DUT40004CSL00`00CQM>5`06CQL00`00CQM>5`020005EUL00`00FYQJV007FYP00`00I]P00005
JaT01@00LcQc>7<h00000g<h00<006lhKcP016lh00<006/IJaT00V/I00<007ooOol00goo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo00eo
o`03001c^W>j00Ac^P<000U[N`8000QFf`03001BFE9I00=BF@<000Q>=`03001>5dhG00I>5`8000MF
E`03001JV5ZH00IJV005001Vf6KHI]P00003JaT00`00LcQc>003LcP01@00LcP007=H000016lh00<0
06/IJaT00f/I00<007ooOol017oo00<007ooOol03Goo0007Ool01000Oomoo`0027oo0`000goo00<0
07ooOol03Goo0P0037oo00<007>jLkX017>j00@006OlIoaWo08000A[N`<00003Egl005KK00QFf`03
001BF@000080009BF@03001>=dhg00I>=`03001>5dhG00E>5`8000UFE`03001JV5ZH00AJV003001V
f6KH009Vf005001[6F/IJaT00005LcP20003LeP01000KcQ_>0001f/I00<007ooOol017oo00<007oo
Ool00Woo10001Woo0007Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol03goo00<007oo
Ool027oo00<007>jLkX017>j00<006OlIo`016Ol00@006]k000000EGO`03001FfeKK00IFf`8000IB
F@03001>=dhg00E>=`03001>5dhG009>5`<00003DSL005IG00QFE`03001JV5ZH00=JV003001Vf6KH
00AVf003001[6@0000Ic>08000=cF003001_>00000Q[6@03001oogoo00Aoo`03001oogoo00=oo`03
001oogoo00Ioo`0027oo0P002Goo10000Woo00<007ooOol047oo00<007ooOol01Woo00<007>jLkX0
0g>j0P001VOl10001eMo00<005KKE]/00UKK10001e9I00<004hgCSL01Thg10001E8g00<005IGEUL0
1eIG00D005ZHFYQJV00000QVf003001c>7<h00Ac>004001cF7=H0003LeP00`00JaU[6@07JaT20006
Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qoo`8001=oo`03001oogoo00Aoo`03001c^W>j00=c
^P03001Wo6Ol00=Wo0<000@kV`<000EGO`@000AFf`03001BFE9I00ABF@03001>=dhg00A>=`<000QB
=`03001FEeIG00MFE`04001JV5ZH0009I]P00`00LcQc>004LcP01@00LeQcF7=H00000W=H00<006/I
JaT01V/I00<006;j00001goo00<007ooOol00goo00<007ooOol017oo000HOol2000DOol01000Oomo
ogoo1@000W>j00<006Ol00005@001UKK00<0059IDUT01E9I00<004hg00000`000Thh00<0058gDSL0
1U8g00<005IGEUL01eIG00<005ZH00002FKH00<006lh000017<h00<007=HLeP00W=H00D007=H001c
V@0000I[6@05001RnV;jH_X00005Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomo
o`0DOol00`00Ool00005Io/<0004B_h00`005Sd0000300028[hB0007CSP00`00DSMB=`06DSL00`00
EUMFE`06EUL20009I]P01@00KcQ_>6lh00000W<h00<007=HLeP017=H00@007>ILiT000A[6@8000AR
nP03001oogoo00Eoo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo01Eoo`03
001WnfOk00=Wn`030013U`000080009O_P04001S6f<K0006B_h300025Sd200038[h00`00>WdjO@04
>Wd00`00AUY6FP04AUX00`00CSQ>>006CSP00`00DSMB=`06DSL00`00EUMFE`05EUL00`00G[P00008
I]P00`00KcQ_>002KcP00`00LcP00006LeP00`00LiT00002LiT01000JaU[6@001V;j00<007ooOol0
1Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol057oo00<006OkIo/00VOk1000
0enn0P0016<K00<004[nB_h00`000T[n00@001Hm5SdF?@<0008R_P03000jOCYm00@jO@030016FTIJ
00A6FP03001>>4hh00E>>003001B=e8g00IB=`03001FEeIG00AFE`05001N^5jhG[P00006I]P00`00
KcQ_>004KcP00`00LeQcF004LeP02@00LiUcVG>I001cV@00JaT00008H_X00`00Oomoo`05Ool00`00
Oomoo`08Ool0015oo`04001oogoo0002Ool2000EOol00`00Io]Wn`02Io/30004Gkh00`00Ha]S6`05
Ha/20003B_h600000aHm0000000200000b:n000jO@04>Wd00`00AUY6FP04AUX00`00CSQ>>006CSP0
0`00DSMB=`06DSL01000EUMFEeIG0P0015jh00<006KHI]P016KH00<006lhKcP016lh0P001G=H00H0
07>ILiUcV@00LiT20009H_X00`00Oomoo`06Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool0
0`00Oomoo`0BOol00`00Io]Wn`02Io/20002Jg/200000enn001S6`04Ha/80003BWX00`00<O/an`03
0000104H000000001SYm00<004IJAUX014IJ00<004hhCSP01Dhh00<0058gDSL01U8g00@005IGEUL0
00MN^003001Vf6KH009Vf003001_>6lh00A_>004001gVGNI0003LeP00`00LiUcV@03LiT00`00Jk`0
0009H_X00`00Oomoo`06Ool00`00Oomoo`07Ool0019oo`8000=oo`8001=oo`03001Wn`0000<000I[
N`80009S6`<0009S6`03001J^@0000M:NP03000anc7k008an`D000DjO@030016FTIJ00A6FP03001>
>4hh00E>>003001B=e8g00IB=`03001FE`0000UN^004001Vf6KH0006KcP00`00MiUgV@02MiT00`00
LeP00005LiT01@00Jk`006^l00002F;j00<007ooOol01Woo00<007ooOol01Woo000GOol00`00Oomo
o`0@Ool70006Jg/00`00JaT000020003Ha/20003F[T00`00BWY:NP04BWX00`00<O/an`02<O/00`00
2GX000040003>Wd00`00AUY6FP05AUX00`00CSQ>>005CSP00`00DSMB=`05DSL2000:G[P00`00I]P0
0007KcP00`00MiUgV@03MiT00`00LiUcV@03LiT01000Jka[_0000V^l00<006;jH_X016;j0`002Goo
00<007ooOol01Goo000GOol00`00Oomoo`0=Ool300000cnJ00000002Hmh00`00Jg][N`04Jg/30003
JaT00`00Ha/00005F[T00`00BWY:NP05BWX00`00<O/an`02<O/00`002GX0000200001PgL00000000
>Wd000I6FP03001>>4hh00E>>003001B=e8g00=B=`800003EYT005jh00UN^08000M_>003001gVGNI
00=gV@03001cnP0000=cV@03001[_6^l009[_004001[_6^l0002H_X30002E]/00`00Oomoo`07Ool0
0`00Oomoo`05Ool001Moo`@000Uoo`@00003?iX0000000=SgP03001[Nf]k009[N`<000M[6@03001J
^EZi00EJ^@03001:NTYj00E:NP03000anc7k008an`030009NPUj008000<=g08000M6FP03001>>4hh
00A>>004001B=e8gDSL20004EYT00`00G[QN^006G[P00`00JcX00006KcP00`00MiUgV@03MiT01`00
LoYcnW?j001cV@0000I[_003001[_000008000IFf`03001oogoo00Moo`03001oogoo00Aoo`001Goo
0P0017oo0P000goo0`0017oo0P0000=oo`0000004`0000I[N`00000006]k0007JaT20008F[T00`00
BWY:NP04BWX00`00<O/an`02<O/010002GX9NPUj0P0010gL1@000TIJ00<004hhCSP01Dhh0`001UJI
00<005jhG[P01Ejh00D006/jJcY[>P0000A_>003001gVGNI00=gV@03001cnW?j009cnP<000E[_0<0
0003Dgl005KK00IFf`03001oogoo00Ioo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001o
ogooOol00005Ool90007Ool500000f]k001[6@06JaT01000I]QVf0001eZi00<004YjBWX00TYj1`00
00<an`002GX00PUj0P000`gL00<0038l<S`00S8l2`002EJI00<005jhG[P00ejh00<006/jJcX00f/j
00D006lhKcQ_>00000AgV@03001cnW?j009cnP06001GnEOiEoT006^l0`000U=o00@005=oDgl000MF
f`03001oogoo009oo`<0009oo`03001oogoo00=oo`0017oo00@007ooOol000]oo`03001oogoo0080
01=oo`8000Eoo`03001[6F/I00E[6@05001Vf6KHI]P00007F[T50003CSP01000?OTmnCgi100000<9
NP000=L00P000PgL00<0038l<S`0138l00<004YJBUX014YJ00<005JIEYT01eJI00<005jhG[P00Ujh
00<006/jJcX01F/j00<006lh000017NI00@007?jLoYcnP8000AGn@<000ECO`03001CO`0000MFf`03
001oo`00008000Eoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`<000=oo`80011oo`<000Qo
o`03001[6F/I00=[6@03001Vf6KH00=Vf003001J^EZi00=J^@03001FN00000M>>003000mnCgi008m
n@03000Y^P00008000050=L000003M`00007<S`00`00BUY:FP04BUX00`00EYUFV@07EYT01000G[QN
^00026/j0`000gNI00D007?jLoYcnP0000=Gn@<000<S5P03001COe=o00=CO`03001CO`0000IFf`80
00Qoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=oo`03001oogoo00aoo`<0
00aoo`05001[6F/IJaT00007I]P00`00F[UJ^@020003EWP00`00CSQ>>004CSP00`00?OTmn@03?OT0
0`00:KXY^P0200000`3G000000020006<S`00`00BUY:FP05BUX00`00EYUFV@06EYT00`00G[P00008
JcX01`00Km]_ffoK001gV@0000=cnP03001GnEOi00h00003Dgl0000000MFf`03001oogoo00Ioo`03
001oogoo00=oo`001Goo0P002Goo10000Woo00<007ooOol02Goo0`0047oo00<006/I000026KH00<0
05Zi00001EIh00<004hhCSP014hh00<003gi?OT00cgi00@002Vj:KXY^P8000030=L0004G008000Hb
?003001:FTYJ00A:FP03001FVEJI00IFV@8000Q[>P03001_ffoK009_f`/000@VGP<00003200002Jo
008V_`P0009Ff`03001oogoo00Moo`03001oogoo009oo`005Woo00<007ooOol01Woo0`0057oo00<0
06KHI]P01fKH0P001UIh00<004hhCSP01Dhh00<003gi?OT00cgi00<002Vj:KX00RVj1@001S8l00<0
04YJBUX01DYJ00<005JIEYT00eJI0P0000=K7000JcX01V/j00<006oKKm/00VoK00@005?GDmMCe`<0
00E:oP03000VGRIN008VGP030005V`00008000@V_`8000@nO0@000Qoo`03001oogoo009oo`005Goo
0P001Goo0`005goo00<006KHI]P01VKH00<006:h00001eIh00<004hhCSP014hh00<003gi?OT00cgi
00<002Vj:KX00RVj00<000VJ00001@00138l00<004YJBUX014YJ00<005JIEYT00UJI00<005/LFa`0
0U/L00<006/jJcX00f/j00<006oKKm/00VoK00<005?GDmL00P0000=S?000B_h00d[n1P000RIN00<0
00FK00000`000bJo0P001Sil00<007ooOol01Woo00<007ooOol00Woo000EOol01000Oomoogoo0`00
6goo00<006KHI]P016KH00D006:hH[QR^00000IFN003001>>4hh00A>>003000mnCgi00@mn@03000Y
^RVj008Y^P030009VP0000<000034O`0000000<000M:FP04001FV@000006Fa`01@00JcY[>V/j0000
16oK00@005?G000000=S?0D000=>NP03000j6SXJ00@00085V`H000039[l003il00DnO003001oogoo
00Eoo`03001oogoo009oo`005Goo10007goo00<006KHI]P00VKH00<006:hH[P00V:h00<005IhEWP0
1EIh00<004hhCSP014hh00<003gi?OT00cgi00<002Vj:KX00bVj00H000VJ2IX0003E00044O`:0009
Fa`00`00JcX00004Km/00`00DmL00004Hc`00`00F[T00007CWX00`00>QXj6P02>QX300021I/00`00
0M/1f`030006?W`00`00Oomoo`05Ool00`00Oomoo`02Ool000ioo`<000Aoo`80025oo`05001Vf6KH
I]P00006H[P00`00EWQFN004EWP00`00CSQ>>003CSP<00000bVj0009VP022IX300044O`00`00?[hn
_P05?[h00`00FaaK7007Fa`40002Km/30003Hc`20003F[T00`00CWY>NP04CWX00`00>QXj6P03>QX0
0`008K/000021I/200020M/30004?W`00`00Oomoo`05Ool00`00Oomoo`02Ool000moo`03001oogoo
00Eoo`03001oogoo01moo`03001Vf00000MR^003001FN5Ih00AFN003001>>4hh00<00003CQL004HG
00E65`0400126D8I@QT500022IX300034O`00`00?[hn_P05?[h00`00FaaK7004Fa`20004Emh30002
JcX00`00Hc`00005F[T00`00CWY>NP05CWX00`00>QXj6P03>QX00`008K/0000200001P5J0001f`7K
000R708000<nO003001oogoo00Aoo`03001oogoo009oo`003goo00<007ooOol01Woo00<007ooOol0
7goo00<006:hH[P01V:h00<005IhEWP015Ih0`001DhG00<004HGAQL014HG00<0048I@QT00T8I00<0
03Hl=S`0100000H1F0000000017l0007?[h00`00FaaK7002Fa`00`00EmiGgP02Emh20005JcX20007
F[T00`00CWY>NP04CWX00`00>QXj6P03>QX010008K/Q^b6k0P0000<1FP0000000P000R8L0P0000<n
O000Ool01Woo00<007ooOol00Goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0NOol00`00H[QR^007
H[P01000EWQFN5Ih0P0000=B=`00CQL01DhG00<004HGAQL014HG00<0048I@QT00d8I00<003Hl=S`0
0SHl20001cjn00@005/L000000=GgP<000E[>P03001Vf00000MJ^@03001>NTij009>NPH000<j6P03
000Q^b6k008Q^`030001FP5J00<000<R708000Moo`03001oogoo005oo`003goo00<007ooOol027oo
0P007goo00<006:hH[P01V:h00@005Ih000000AB=`03001>5dhG00=>5`0300165dHG00E65`030012
6D8I00=26@03000f?3Hl00@f?08000036Wh0000000/00005Emh00000Ji`00005JcX01@00I]QVf6KH
00001UZi1@001Woo10001B6k0P000P5J00<0028L8Q`00b8L00<007ooOol017oo00<007ooOol00Goo
000>Ool2000<Ool00`00Oomoo`0LOol00`00H[QR^006H[P20006DSL00`00CQM>5`04CQL00`00AQM6
5`04AQL00`00@QU26@03@QT00`00=S`f?004=S`40004Cel90002KcT00`00JcY[>P03JcX00`00I]QV
f002I]P00`00F[UJ^@03F[T00`00Oomoo`0<Ool300038K/010002GX0005J0`000b8L00<007ooOol0
17oo00<007ooOol00Goo000MOol00`00Oomoo`0LOol00`00H[QR^003H[P200000eYh001B=`05DSL0
0`00CQM>5`04CQL00`00AQM65`04AQL00`00@QU26@04@QT00`00=S`f?0030004DY/01@00Cem?GdmO
000016ML00<006liKcT00fli0P000f/j00<006KHI]P016KH00<005ZiF[T00P004Woo0`0000<Q^`00
2GX00`0000<5V`008Q`00R8L00<007ooOol00goo00<007ooOol00Goo000NOol00`00Oomoo`0LOol0
1@00H[QR^6:h00000eYh00<0058gDSL01E8g00<004hGCQL00dhG00<004HGAQL01DHG00<0048I@QT0
0d8I0`0025:K00<004mO000016ML00<006liKcT00fli00H006lhKcP006/j0007I]P00`00F[T0000G
Ool300022GX200001@FK0000028L00001Goo00<007ooOol00Goo000OOol2000LOol01000H[P00000
1EYh00<0058gDSL0158g00<004hGCQL014hG00<004HGAQL014HG00<0048I@QT00`0000=>F000DY/0
25:K0`000fML00<006liKcT00fli00<006lhKcP00Vlh00<006KHI]P01VKH0P006goo0P000PUj00<0
00FK1I/00`0017oo00<007ooOol00Goo000QOol00`00Oomoo`0JOol00`00FWQJN005FWP00`00DSMB
=`04DSL00`00CQM>5`04CQL00`00AQM65`03AQL40005CUP00`00DY]BV`05DY/01`00H]URfF;I001W
G00000E_>@03001_>6lh00=_>003001Vf6KH00EVf003001oogoo01eoo`D00085V`03001oogoo00=o
o`40005oo`5oo`008Woo00<007ooOol06Goo00<005YhFWP01EYh00<0058gDSL01E8g00<004hGCQL0
0dhG00<004HGAQL00`000Thg00<004iHCUP01DiH00<005:KDY/00U:K0P001F;I0P001Fli00<006lh
KcP00Vlh00<006lH00001FKH00<007ooOol087oo10000PFK00<007ooOol00Woo0@000Goo0Goo000S
Ool00`00Oomoo`0IOol00`00FWQJN005FWP00`00DSMB=`04DSL00`00CQM>5`03CQL30006CSL00`00
CUQ>F004CUP01@00DY]BVe:K00001f;I00<006[h00000fli00<006lhKcP00Vlh00@006lHKaP000AV
f003001oogoo02=oo`D000Aoo`40005oo`5oo`0097oo00<007ooOol067oo00<005YhFWP01EYh00<0
058gDSL0158g00<004hGCQL00`0000=>5`00CSL01Thg00<004iHCUP01DiH00<005:K00001f;I00L0
06[hJ_QZn000KcT00005KcP01@00KaQ_66lH00000VKH00<007ooOol09Goo1@000goo0@000Goo0Goo
000UOol2000IOol00`00FWQJN004FWP00`00DSMB=`03DSL40005CQL00`00CSM>=`04CSL00`00CUQ>
F005CUP20007H]T00`00J_QZn003J_P00`00KcQ_>003KcP00`00KaQ_6002KaP00`00I]P0000XOol0
0`00Oomoo`030002Ool10001Ool1Ool002Moo`03001oogoo01Ioo`03001JN5Yh00EJN003001B=`00
0080009B=`03001>5dhG00A>5`03001>=dhg00E>=`03001>F4iH009>F0800003FYP006;I00ERf@03
001Zn6[h00AZn003001_>6lh009_>003001_66lH00A_6003001oogoo02Qoo`04001oogooOol40002
Ool002Qoo`03001oogoo01Ioo`03001JN5Yh00=JN0<000EB=`03001>5dhG00A>5`03001>=dhg00E>
=`05001>F4iHCUP00004FYP00`00H]URf@03H]T00`00J_QZn003J_P00`00KaP00003KcP00`00KaQ_
6004KaP00`00Oomoo`0YOol00`00Oomoo`02Ool20002Ool002Uoo`03001oogoo01Eoo`03001JN5Yh
00<00003DUP0058g00IB=`03001>5dhG00A>5`03001>=dhg00A>=`04001>F0000006FYP01@00H]UR
fF;I00001F[h00L006lHKaQ_6000KcP00006KaP00`00Oomoo`0ZOol00`00Oomoo`04Ool10001Ool0
02Yoo`8001Ioo`8000EBF003001B=e8g00AB=`03001>5dhG00A>5`03001>=dhg00A>=`8000UJV003
001Rf@0000IZn003001_66lH009_6003001_66lH00A_6003001oogoo02aoo`03001oogoo00=oo`40
005oo`00;7oo00<007ooOol04goo00<0059HDUP0159H00<0058gDSL0158g00<004hGCQL01DhG00<0
04hgCSL00Thg00<005IG00002UZH00<006[hJ_P016[h00<006lHKaP00flH00<006lHKaP00flH00<0
07ooOol0;Woo00<007ooOol00Woo0@000Goo000]Ool00`00Oomoo`0COol00`00DUQBF003DUP00`00
DSMB=`05DSL00`00CQM>5`04CQL00`00CSM>=`020003EUL00`00FYQJV006FYP20005J_P00`00KaQ_
6003KaP00`00LcP00003KaP00`00Oomoo`0`Ool01@00Oomoogoo00000Goo000^Ool00`00Oomoo`0B
Ool00`00DUQBF004DUP00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CSL00005EUL00`00FYQJV005
FYP01000I]QVf0000f[h00<006lHKaP00flH00L007<hLcQc>000KaP0000cOol01@00Oomoogoo0000
0Goo000_Ool2000COol00`00DUQBF003DUP00`00DSMB=`04DSL00`00CQM>5`04CQL20007EUL00`00
FYQJV003FYP00`00I]QVf002I]P01000J_QZn0001FlH00<007<hLcP00W<h00<007ooOol0<goo00@0
07ooOol0005oo`00<Goo00<007ooOol047oo00<0059HDUP0159H00<0058gDSL00e8g00@004hGCQM>
5`<00003DSL005IG00IFE`03001JV5ZH009JV003001Vf6KH00=Vf003001Zn00000E_6003001c>7<h
00=c>003001oogoo03Aoo`03001oo`00005oo`00<Woo00<007ooOol047oo00<0059HDUP00e9H00<0
058gDSL0158g0`001E8g00<005IGEUL01EIG00@005ZHFYP000MVf08000A_6003001c>7<h00=c>003
001oogoo03Eoo`03001oogoo00400000<goo00<007ooOol03goo00<0059HDUP00e9H00@0058gDSMB
=`@000MB=`03001FEeIG00EFE`03001JV00000QVf003001_600000=_6003001c>7<h009c>003001o
ogoo03Moo`40005oo`400000=7oo0P0047oo2@000dhh00<0058gDSL01E8g00<005IGEUL01EIG0P00
26KH00L006lHKaQ_6000KaP00004LcP00`00Oomoo`0iOol2000003Ioo`03001oogoo00eoo`030016
FTIJ00=6FP03001>>4hh00A>>003001B=e8g00EB=`03001FEeIG00=FE`03001N^00000MVf003001_
66lH00=_6003001c>7<h00=c>003001oogoo03Qoo`<00000=goo00<007ooOol03Goo00<004IJAUX0
0dIJ00<004hhCSP00dhh00<0058gDSL01E8g00@005IGEUMFE`8000=N^003001Vf6KH00=Vf003001_
66lH00A_608000=c>003001oogoo03Eoo`@000=oo`00>7oo00<007ooOol037oo00<004IJAUX00dIJ
00<004hhCSP014hh00<0058gDSL0158g00@005IGEUL000IN^003001Vf6KH009Vf003001_66lH00=_
6006001cNG=i001c>000=7oo10001goo000iOol00`00Oomoo`0<Ool00`00AUY6FP02AUX00`00CSQ>
>004CSP00`00DSMB=`05DSL20007G[P01@00I]QVf6KH00001FlH00<007=iLgT00W=i00<007ooOol0
<7oo0`002goo000jOol2000<Ool00`00AUY6FP03AUX00`00CSQ>>003CSP00`00DSMB=`04DSL20009
G[P00`00I]P00005KaP00`00LgUcN@03LgT00`00Oomoo`0/Ool4000>Ool003aoo`03001oogoo00Uo
o`030016FTIJ00=6FP03001>>4hh00A>>004001B=e8gDSL200000eIi001N^008G[P20005KaP00`00
LgUcN@03LgT00`00Oomoo`0YOol4000BOol003eoo`03001oogoo00Uoo`<00096FP03001>>4hh00A>
>004001B=`000004EWT00`00G[QN^005G[P00`00IaX00005KaP00`00LgUcN@02LgT00`00Oomoo`0V
Ool4000FOol003ioo`03001oogoo00Qoo`03000^72hL00@000=>>0@000IFN@03001N^5jh00AN^005
001W6VLJIaX00003KaP00`00LgUcN@02LgT00`00Oomoo`0SOol4000JOol003moo`8000Uoo`03000^
72hL008^70@000=:FP03001FNEIi00EFN@03001N^5jh009N^003001W6VLJ00=W6P03001_600000Ac
N@03001oogoo021oo`@001ioo`00@Goo00<007ooOol01goo00D002hL;Q`^700000M:FP03001FNEIi
00EFN@04001N^5jh0007IaX30002LgT00`00Oomoo`0NOol3000ROol0049oo`03001oogoo00Moo`05
000^72hL;Q`00006BUX00`00EWUFN@05EWT00`00G[P00007IaX01000Km]_ffoK0P007Goo10009Goo
0013Ool00`00Oomoo`07Ool20002;Q`00`00BUY:FP04BUX00`00EWUFN@04EWT20007IaX01@00Km]_
ffoK00006goo1000:Goo0014Ool20009Ool30006BUX01000EWUFNEIi0`000UlL00<006LJIaX00fLJ
00<006oKKm/00P0067oo1000;Goo0016Ool00`00Oomoo`07Ool30005BUX01000EWT000001UlL00D0
06LJIaYW6P00009_f`03001oogoo01Aoo`@0035oo`00Agoo00<007ooOol01goo2@002ElL00<006LJ
00000VoK00<007ooOol04Goo1000=Goo0018Ool00`00Oomoo`08Ool20005?[h00`00GaaO7006Ga`5
000@Ool4000iOol004Uoo`8000Uoo`8000Dn_P03001O75lL009O70<00003Dmh0000000moo`<003eo
o`00Bgoo00<007ooOol01Woo00<007oo00001Cjn00@005lL0000009CgP8000eoo`@0041oo`00C7oo
00<007ooOol01Goo00<007ooOol02`002goo1000A7oo001=Ool00`00Oomoo`04Ool00`00Oomoo`0B
Ool40018Ool004ioo`03001oogoo00=oo`03001oogoo00ioo`@004aoo`00Cgoo0P000goo00<007oo
Ool02Woo1000D7oo001AOol01000Oomoo`002Goo0`00E7oo001BOol00`00Ool00005Ool4001GOol0
05=oo`800003Ool00000008005]oo`00E7oo0P00Ggoo002eOol00;Eoo`00]Goo002eOol00;Eoo`00
]Goo002eOol00001\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.376 0 0 r
.37588 .75735 .39996 .75138 .43802 .75186 .4143 .7635 Metetra
.4143 .7635 .43802 .75186 .47703 .74004 .45403 .75085 Metetra
.65 .961 .777 r
.45403 .75085 .47703 .74004 .51661 .71604 .49447 .71961 Metetra
.888 .971 .831 r
.49447 .71961 .51661 .71604 .55638 .68258 .535 .67411 Metetra
.896 .865 .801 r
.535 .67411 .55638 .68258 .59612 .64426 .57523 .62159 Metetra
.848 .77 .775 r
.57523 .62159 .59612 .64426 .63583 .60653 .61513 .57035 Metetra
.785 .693 .762 r
.61513 .57035 .63583 .60653 .67576 .57447 .65502 .52793 Metetra
.714 .629 .76 r
.65502 .52793 .67576 .57447 .71635 .55194 .6955 .49987 Metetra
.634 .572 .768 r
.6955 .49987 .71635 .55194 .75808 .54096 .73731 .48903 Metetra
.537 .52 .785 r
.73731 .48903 .75808 .54096 .80142 .54143 .78112 .49528 Metetra
.413 .473 .812 r
.78112 .49528 .80142 .54143 .84662 .55115 .82735 .51552 Metetra
.245 .43 .846 r
.82735 .51552 .84662 .55115 .89365 .56608 .87602 .54392 Metetra
.017 .393 .85 r
.87602 .54392 .89365 .56608 .94213 .58091 .9266 .57262 Metetra
0 .369 .728 r
.9266 .57262 .94213 .58091 .99133 .59004 .97801 .59301 Metetra
0 .531 .831 r
.35154 .74806 .37588 .75735 .4143 .7635 .39022 .7565 Metetra
.612 .888 .989 r
.39022 .7565 .4143 .7635 .45403 .75085 .43049 .74332 Metetra
.833 .895 .884 r
.43049 .74332 .45403 .75085 .49447 .71961 .47163 .70876 Metetra
.881 .847 .806 r
.47163 .70876 .49447 .71961 .535 .67411 .51291 .6579 Metetra
.882 .807 .77 r
.51291 .6579 .535 .67411 .57523 .62159 .5538 .59916 Metetra
.862 .774 .763 r
.5538 .59916 .57523 .62159 .61513 .57035 .59422 .54205 Metetra
.822 .742 .776 r
.59422 .54205 .61513 .57035 .65502 .52793 .63448 .49512 Metetra
.755 .704 .804 r
.63448 .49512 .65502 .52793 .6955 .49987 .67525 .46458 Metetra
.644 .649 .841 r
.67525 .46458 .6955 .49987 .73731 .48903 .71733 .45359 Metetra
.471 .564 .868 r
.71733 .45359 .73731 .48903 .78112 .49528 .76151 .462 Metetra
.248 .451 .862 r
.76151 .462 .78112 .49528 .82735 .51552 .80831 .4864 Metetra
.042 .35 .826 r
.80831 .4864 .82735 .51552 .87602 .54392 .85778 .52024 Metetra
0 .324 .806 r
.85778 .52024 .87602 .54392 .9266 .57262 .90939 .55462 Metetra
0 .422 .854 r
.90939 .55462 .9266 .57262 .97801 .59301 .96195 .57963 Metetra
.418 .608 .927 r
.32737 .72078 .35154 .74806 .39022 .7565 .36627 .72741 Metetra
.666 .697 .869 r
.36627 .72741 .39022 .7565 .43049 .74332 .40683 .71406 Metetra
.77 .725 .81 r
.40683 .71406 .43049 .74332 .47163 .70876 .44839 .68093 Metetra
.821 .741 .776 r
.44839 .68093 .47163 .70876 .51291 .6579 .49021 .63268 Metetra
.848 .759 .764 r
.49021 .63268 .51291 .6579 .5538 .59916 .53179 .57705 Metetra
.858 .78 .774 r
.53179 .57705 .5538 .59916 .59422 .54205 .57295 .52284 Metetra
.847 .807 .811 r
.57295 .52284 .59422 .54205 .63448 .49512 .61395 .47801 Metetra
.792 .832 .882 r
.61395 .47801 .63448 .49512 .67525 .46458 .65538 .44837 Metetra
.628 .81 .97 r
.65538 .44837 .67525 .46458 .71733 .45359 .69798 .4369 Metetra
.301 .653 .97 r
.69798 .4369 .71733 .45359 .76151 .462 .7425 .44347 Metetra
.005 .436 .863 r
.7425 .44347 .76151 .462 .80831 .4864 .78944 .46483 Metetra
0 .34 .81 r
.78944 .46483 .80831 .4864 .85778 .52024 .83891 .49488 Metetra
.025 .381 .844 r
.83891 .49488 .85778 .52024 .90939 .55462 .89043 .52531 Metetra
.248 .498 .897 r
.89043 .52531 .90939 .55462 .96195 .57963 .94297 .54699 Metetra
.549 .565 .823 r
.30366 .67668 .32737 .72078 .36627 .72741 .3428 .67773 Metetra
.644 .592 .781 r
.3428 .67773 .36627 .72741 .40683 .71406 .38341 .66453 Metetra
.706 .622 .762 r
.38341 .66453 .40683 .71406 .44839 .68093 .42501 .63721 Metetra
.754 .662 .76 r
.42501 .63721 .44839 .68093 .49021 .63268 .46706 .59897 Metetra
.795 .714 .775 r
.46706 .59897 .49021 .63268 .53179 .57705 .50914 .55519 Metetra
.831 .789 .812 r
.50914 .55519 .53179 .57705 .57295 .52284 .5511 .51217 Metetra
.838 .892 .878 r
.5511 .51217 .57295 .52284 .61395 .47801 .59307 .47575 Metetra
.718 .969 .934 r
.59307 .47575 .61395 .47801 .65538 .44837 .63548 .45027 Metetra
.317 .806 .816 r
.63548 .45027 .65538 .44837 .69798 .4369 .67885 .438 Metetra
.059 0 0 r
.67885 .438 .69798 .4369 .7425 .44347 .72375 .43885 Metetra
0 .424 .687 r
.72375 .43885 .7425 .44347 .78944 .46483 .77056 .45033 Metetra
0 .472 .859 r
.77056 .45033 .78944 .46483 .83891 .49488 .81939 .46786 Metetra
.259 .53 .916 r
.81939 .46786 .83891 .49488 .89043 .52531 .86991 .48535 Metetra
.451 .551 .867 r
.86991 .48535 .89043 .52531 .94297 .54699 .92143 .49632 Metetra
.605 .551 .768 r
.28041 .62051 .30366 .67668 .3428 .67773 .31989 .61347 Metetra
.629 .545 .741 r
.31989 .61347 .3428 .67773 .38341 .66453 .36036 .60064 Metetra
.652 .56 .738 r
.36036 .60064 .38341 .66453 .42501 .63721 .40165 .58205 Metetra
.68 .597 .756 r
.40165 .58205 .42501 .63721 .46706 .59897 .44353 .55892 Metetra
.716 .665 .799 r
.44353 .55892 .46706 .59897 .50914 .55519 .48582 .53333 Metetra
.753 .79 .882 r
.48582 .53333 .50914 .55519 .5511 .51217 .52845 .50779 Metetra
.703 .95 .955 r
.52845 .50779 .5511 .51217 .59307 .47575 .57147 .48473 Metetra
.288 .766 .66 r
.57147 .48473 .59307 .47575 .63548 .45027 .61504 .46604 Metetra
.064 0 0 r
.61504 .46604 .63548 .45027 .67885 .438 .6594 .45272 Metetra
.167 0 0 r
.6594 .45272 .67885 .438 .72375 .43885 .70478 .44473 Metetra
.067 0 0 r
.70478 .44473 .72375 .43885 .77056 .45033 .75138 .44094 Metetra
.307 .751 .963 r
.75138 .44094 .77056 .45033 .81939 .46786 .79924 .43939 Metetra
.53 .678 .932 r
.79924 .43939 .81939 .46786 .86991 .48535 .84825 .43752 Metetra
.584 .59 .824 r
.84825 .43752 .86991 .48535 .92143 .49632 .89816 .43276 Metetra
.644 .56 .746 r
.25733 .55936 .28041 .62051 .31989 .61347 .29727 .54345 Metetra
.621 .529 .729 r
.29727 .54345 .31989 .61347 .36036 .60064 .33753 .53104 Metetra
.605 .522 .734 r
.33753 .53104 .36036 .60064 .40165 .58205 .37826 .5221 Metetra
.594 .54 .763 r
.37826 .5221 .40165 .58205 .44353 .55892 .41963 .51588 Metetra
.586 .599 .833 r
.41963 .51588 .44353 .55892 .48582 .53333 .46179 .51107 Metetra
.55 .735 .961 r
.46179 .51107 .48582 .53333 .52845 .50779 .50485 .50597 Metetra
.28 .778 .877 r
.50485 .50597 .52845 .50779 .57147 .48473 .54881 .4989 Metetra
.075 0 0 r
.54881 .4989 .57147 .48473 .61504 .46604 .59359 .48848 Metetra
.149 0 0 r
.59359 .48848 .61504 .46604 .6594 .45272 .63906 .47396 Metetra
.06 0 0 r
.63906 .47396 .6594 .45272 .70478 .44473 .68508 .45536 Metetra
.68508 .45536 .70478 .44473 .75138 .44094 .73156 .4335 Metetra
.685 .944 .961 r
.73156 .4335 .75138 .44094 .79924 .43939 .77848 .40981 Metetra
.72 .758 .882 r
.77848 .40981 .79924 .43939 .84825 .43752 .82594 .38603 Metetra
.675 .626 .792 r
.82594 .38603 .84825 .43752 .89816 .43276 .87416 .36386 Metetra
.678 .586 .744 r
.23387 .5009 .25733 .55936 .29727 .54345 .27437 .47704 Metetra
.619 .536 .738 r
.27437 .47704 .29727 .54345 .33753 .53104 .31444 .46495 Metetra
.556 .502 .75 r
.31444 .46495 .33753 .53104 .37826 .5221 .35455 .46455 Metetra
.481 .485 .784 r
.35455 .46455 .37826 .5221 .41963 .51588 .39525 .47359 Metetra
.368 .493 .854 r
.39525 .47359 .41963 .51588 .46179 .51107 .43703 .48794 Metetra
.149 .523 .918 r
.43703 .48794 .46179 .51107 .50485 .50597 .48023 .50219 Metetra
.152 0 0 r
.48023 .50219 .50485 .50597 .54881 .4989 .52489 .51058 Metetra
.202 0 0 r
.52489 .51058 .54881 .4989 .59359 .48848 .57076 .50827 Metetra
.064 0 0 r
.57076 .50827 .59359 .48848 .63906 .47396 .61737 .49252 Metetra
.61737 .49252 .63906 .47396 .68508 .45536 .66417 .46348 Metetra
.674 .965 .757 r
.66417 .46348 .68508 .45536 .73156 .4335 .71078 .42413 Metetra
.863 .947 .869 r
.71078 .42413 .73156 .4335 .77848 .40981 .75711 .37953 Metetra
.819 .786 .822 r
.75711 .37953 .77848 .40981 .82594 .38603 .80343 .33557 Metetra
.743 .665 .773 r
.80343 .33557 .82594 .38603 .87416 .36386 .85034 .29771 Metetra
.715 .63 .762 r
.20931 .45178 .23387 .5009 .27437 .47704 .25041 .42225 Metetra
.625 .571 .773 r
.25041 .42225 .27437 .47704 .31444 .46495 .29045 .41027 Metetra
.5 .505 .794 r
.29045 .41027 .31444 .46495 .35455 .46455 .33009 .41572 Metetra
.315 .427 .817 r
.33009 .41572 .35455 .46455 .39525 .47359 .37018 .43544 Metetra
.054 .334 .814 r
.37018 .43544 .39525 .47359 .43703 .48794 .41151 .46351 Metetra
0 .256 .735 r
.41151 .46351 .43703 .48794 .48023 .50219 .45465 .4919 Metetra
.307 0 0 r
.45465 .4919 .48023 .50219 .52489 .51058 .49974 .51179 Metetra
.133 0 0 r
.49974 .51179 .52489 .51058 .57076 .50827 .54646 .51554 Metetra
.27 .765 .707 r
.54646 .51554 .57076 .50827 .61737 .49252 .59408 .49874 Metetra
.69 .979 .812 r
.59408 .49874 .61737 .49252 .66417 .46348 .64175 .46163 Metetra
.885 .98 .825 r
.64175 .46163 .66417 .46348 .71078 .42413 .68883 .40903 Metetra
.909 .892 .801 r
.68883 .40903 .71078 .42413 .75711 .37953 .73513 .34893 Metetra
.865 .796 .781 r
.73513 .34893 .75711 .37953 .80343 .33557 .78102 .29032 Metetra
.796 .71 .77 r
.78102 .29032 .80343 .33557 .85034 .29771 .82735 .24125 Metetra
.754 .701 .801 r
.18288 .41663 .20931 .45178 .25041 .42225 .22456 .38453 Metetra
.638 .653 .848 r
.22456 .38453 .25041 .42225 .29045 .41027 .26479 .37236 Metetra
.414 .55 .883 r
.26479 .37236 .29045 .41027 .33009 .41572 .30432 .37996 Metetra
.068 .365 .834 r
.30432 .37996 .33009 .41572 .37018 .43544 .34415 .40387 Metetra
0 .188 .706 r
.34415 .40387 .37018 .43544 .41151 .46351 .38523 .43744 Metetra
0 .151 .648 r
.38523 .43744 .41151 .46351 .45465 .4919 .42831 .47159 Metetra
0 .305 .743 r
.42831 .47159 .45465 .4919 .49974 .51179 .47363 .49617 Metetra
.191 .639 .944 r
.47363 .49617 .49974 .51179 .54646 .51554 .52088 .50227 Metetra
.634 .881 .987 r
.52088 .50227 .54646 .51554 .59408 .49874 .56927 .48473 Metetra
.834 .908 .888 r
.56927 .48473 .59408 .49874 .64175 .46163 .61778 .44383 Metetra
.894 .873 .809 r
.61778 .44383 .64175 .46163 .68883 .40903 .66563 .38524 Metetra
.901 .836 .772 r
.66563 .38524 .68883 .40903 .73513 .34893 .71252 .31826 Metetra
.882 .802 .766 r
.71252 .31826 .73513 .34893 .78102 .29032 .75882 .25323 Metetra
.837 .765 .783 r
.75882 .25323 .78102 .29032 .82735 .24125 .80544 .19923 Metetra
.788 .809 .869 r
.15393 .39742 .18288 .41663 .22456 .38453 .19609 .3662 Metetra
.635 .805 .965 r
.19609 .3662 .22456 .38453 .26479 .37236 .23678 .3535 Metetra
.242 .634 .96 r
.23678 .3535 .26479 .37236 .30432 .37996 .27672 .35916 Metetra
0 .326 .761 r
.27672 .35916 .30432 .37996 .34415 .40387 .31686 .37992 Metetra
0 .181 .656 r
.31686 .37992 .34415 .40387 .38523 .43744 .35819 .40955 Metetra
0 .237 .728 r
.35819 .40955 .38523 .43744 .42831 .47159 .40144 .4396 Metetra
.092 .431 .873 r
.40144 .4396 .42831 .47159 .47363 .49617 .44694 .46068 Metetra
.442 .612 .921 r
.44694 .46068 .47363 .49617 .52088 .50227 .49447 .46464 Metetra
.662 .702 .876 r
.49447 .46464 .52088 .50227 .56927 .48473 .54329 .44675 Metetra
.776 .741 .821 r
.54329 .44675 .56927 .48473 .61778 .44383 .59248 .40727 Metetra
.835 .764 .784 r
.59248 .40727 .61778 .44383 .66563 .38524 .64124 .35138 Metetra
.866 .785 .769 r
.64124 .35138 .66563 .38524 .71252 .31826 .68924 .28763 Metetra
.876 .808 .779 r
.68924 .28763 .71252 .31826 .75882 .25323 .73672 .22555 Metetra
.859 .831 .818 r
.73672 .22555 .75882 .25323 .80544 .19923 .78449 .17363 Metetra
.762 .939 .941 r
.12203 .3933 .15393 .39742 .19609 .3662 .16448 .36626 Metetra
.465 .883 .924 r
.16448 .36626 .19609 .3662 .23678 .3535 .20591 .3527 Metetra
.026 .598 .736 r
.20591 .3527 .23678 .3535 .27672 .35916 .24687 .35248 Metetra
.197 0 0 r
.24687 .35248 .27672 .35916 .31686 .37992 .28809 .3631 Metetra
0 .389 .773 r
.28809 .3631 .31686 .37992 .35819 .40955 .33035 .37987 Metetra
.122 .475 .896 r
.33035 .37987 .35819 .40955 .40144 .4396 .37426 .39661 Metetra
.383 .531 .881 r
.37426 .39661 .40144 .4396 .44694 .46068 .42008 .40665 Metetra
.54 .563 .827 r
.42008 .40665 .44694 .46068 .49447 .46464 .46771 .40433 Metetra
.637 .593 .788 r
.46771 .40433 .49447 .46464 .54329 .44675 .51665 .38644 Metetra
.705 .629 .769 r
.51665 .38644 .54329 .44675 .59248 .40727 .56623 .35316 Metetra
.76 .674 .767 r
.56623 .35316 .59248 .40727 .64124 .35138 .61585 .30801 Metetra
.807 .734 .783 r
.61585 .30801 .64124 .35138 .68924 .28763 .66522 .25691 Metetra
.841 .812 .821 r
.66522 .25691 .68924 .28763 .73672 .22555 .71444 .20667 Metetra
.84 .904 .882 r
.71444 .20667 .73672 .22555 .78449 .17363 .76406 .16349 Metetra
.514 .915 .861 r
.08714 .40057 .12203 .3933 .16448 .36626 .12962 .38041 Metetra
.12962 .38041 .16448 .36626 .20591 .3527 .17203 .36571 Metetra
.17203 .36571 .20591 .3527 .24687 .35248 .21462 .35641 Metetra
.21462 .35641 .24687 .35248 .28809 .3631 .25773 .35139 Metetra
.329 .759 .972 r
.25773 .35139 .28809 .3631 .33035 .37987 .30171 .3486 Metetra
.507 .68 .944 r
.30171 .3486 .33035 .37987 .37426 .39661 .34684 .34547 Metetra
.565 .589 .836 r
.34684 .34547 .37426 .39661 .42008 .40665 .39328 .33935 Metetra
.593 .548 .773 r
.39328 .33935 .42008 .40665 .46771 .40433 .44099 .32806 Metetra
.618 .541 .745 r
.44099 .32806 .46771 .40433 .51665 .38644 .48979 .31041 Metetra
.645 .558 .742 r
.48979 .31041 .51665 .38644 .56623 .35316 .53944 .28645 Metetra
.676 .599 .762 r
.53944 .28645 .56623 .35316 .61585 .30801 .58968 .2575 Metetra
.716 .675 .809 r
.58968 .2575 .61585 .30801 .66522 .25691 .64038 .22581 Metetra
.754 .799 .889 r
.64038 .22581 .66522 .25691 .71444 .20667 .69158 .1941 Metetra
.719 .941 .957 r
.69158 .1941 .71444 .20667 .76406 .16349 .74349 .16496 Metetra
.04967 .41308 .08714 .40057 .12962 .38041 .09191 .40131 Metetra
.09191 .40131 .12962 .38041 .17203 .36571 .13545 .38531 Metetra
.13545 .38531 .17203 .36571 .21462 .35641 .18016 .36511 Metetra
.514 .914 .831 r
.18016 .36511 .21462 .35641 .25773 .35139 .22582 .34155 Metetra
.724 .92 .959 r
.22582 .34155 .25773 .35139 .30171 .3486 .27219 .31609 Metetra
.72 .753 .878 r
.27219 .31609 .30171 .3486 .34684 .34547 .3191 .29052 Metetra
.673 .628 .796 r
.3191 .29052 .34684 .34547 .39328 .33935 .36647 .26655 Metetra
.635 .557 .75 r
.36647 .26655 .39328 .33935 .44099 .32806 .41436 .24552 Metetra
.608 .522 .731 r
.41436 .24552 .44099 .32806 .48979 .31041 .46292 .22816 Metetra
.591 .514 .736 r
.46292 .22816 .48979 .31041 .53944 .28645 .51236 .2144 Metetra
.582 .535 .768 r
.51236 .2144 .53944 .28645 .58968 .2575 .5629 .20345 Metetra
.579 .599 .837 r
.5629 .20345 .58968 .2575 .64038 .22581 .61467 .19387 Metetra
.566 .73 .952 r
.61467 .19387 .64038 .22581 .69158 .1941 .66774 .18387 Metetra
.439 .847 .97 r
.66774 .18387 .69158 .1941 .74349 .16496 .72201 .17161 Metetra
.06 0 0 r
.01057 .42296 .04967 .41308 .09191 .40131 .05237 .41943 Metetra
.05237 .41943 .09191 .40131 .13545 .38531 .09706 .40211 Metetra
.553 .922 .763 r
.09706 .40211 .13545 .38531 .18016 .36511 .1441 .37115 Metetra
.808 .992 .874 r
.1441 .37115 .18016 .36511 .22582 .34155 .19262 .32963 Metetra
.857 .895 .862 r
.19262 .32963 .22582 .34155 .27219 .31609 .24173 .28275 Metetra
.811 .768 .813 r
.24173 .28275 .27219 .31609 .3191 .29052 .29073 .23652 Metetra
.743 .665 .773 r
.29073 .23652 .3191 .29052 .36647 .26655 .33931 .1965 Metetra
.674 .587 .749 r
.33931 .1965 .36647 .26655 .41436 .24552 .38755 .16683 Metetra
.607 .53 .742 r
.38755 .16683 .41436 .24552 .46292 .22816 .43591 .14964 Metetra
.541 .493 .752 r
.43591 .14964 .46292 .22816 .51236 .2144 .48503 .14478 Metetra
.471 .48 .786 r
.48503 .14478 .51236 .2144 .5629 .20345 .53556 .14989 Metetra
.389 .501 .852 r
.53556 .14989 .5629 .20345 .61467 .19387 .58801 .16058 Metetra
.278 .571 .937 r
.58801 .16058 .61467 .19387 .66774 .18387 .64255 .17107 Metetra
.138 .643 .904 r
.64255 .17107 .66774 .18387 .72201 .17161 .6989 .17516 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<002eoo`03001oogoo
00Moo`03001oogoo02Aoo`00DWoo0`00;goo00<007ooOol027oo00<007ooOol08goo001?Ool3000b
Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<003Eoo`03001oogoo00Uoo`03001oogoo029o
o`00BWoo0`00=goo00<007ooOol02Woo00<007ooOol08Goo0017Ool3000DOol9000MOol00`00Oomo
o`0;Ool00`00Oomoo`0POol004Eoo`80019oo`D000A5n003000mnCgi008mn@H001Moo`03001oogoo
00]oo`03001oogoo021oo`00@Woo0`004Goo0`0000=>5`00AOP01dGh00<003gi?OT013gi00<0038K
<Q/010004goo00<007ooOol037oo00<007ooOol07goo000UOol4000GOol2000BOol20005CQL00`00
AOQ5n005AOP00`00?OTmn@05?OT00`00<Q/b6`03<Q/7000<Ool00`00Oomoo`03Ool60004Ool00`00
Oomoo`0NOol002Ioo`03001oogoo01Aoo`<0019oo`8000M>5`030015n4Gh00E5n003000mnCgi00Dm
n@03000b6c8K00Db6`03000RGB9M008RG@T000=oo`03001oo`0000<000=OgP03001[W6^L00800003
Ool00000021oo`007goo1@000goo00<007ooOol04Goo0P004goo0P0000=FE`00CQL01ThG00<004Gh
AOP01TGh00<003gi?OT01Cgi00<0038K<Q/01C8K00<0029M8Ud01R9M0P000Q:L1@0000<kO`000000
0eoN00<006^LJi`00f^L00@007ooOol001moo`00:7oo00<007ooOol03Goo0`004goo0P0015IG00<0
04hGCQL01DhG00<004GhAOP01DGh00<003gi?OT01Cgi00<0038K<Q/01S8K00<0029M8Ud01B9M1000
0c]o0`0015oN00<006^LJi`00f^L0P000Woo00<007ooOol07Goo000UOol01000Oomoo`003Goo0P00
57oo0P001UIG00<004hGCQL01DhG00<004GhAOP01DGh00<003gi?OT01Cgi00<0038K<Q/01c8K00<0
029M8Ud0100014[n0P0000<kO`00000015oN00<006^LJi`00f^L00@006mJKeX0009oo`03001oogoo
01aoo`009Woo0P002goo0`0057oo0P0025IG00<004hGCQL01ThG00<004GhAOP01DGh00<003gi?OT0
1Cgi00<0038K<Q/01C8K10002D[n10000eoN00<006^LJi`00f^L00<006mJKeX00VmJ00@007ooOol0
01eoo`00<Goo0P0000=oo`00Ool04goo00<005jh00002EIG00<004hGCQL01DhG00<004GhAOP01DGh
00<003gi?OT01Cgi00<0038K<Q/01@000TYj00<004[nB_h01d[n00@006<lHcaS?0800003Gmh006^L
00A[W003001_FVmJ00A_FP03001oo`0001eoo`00;Woo0`005goo00D005jhG[QN^00000QFE`03001>
5dhG00I>5`030015n4Gh00A5n003000mnCgi008mn@H000Q:NP03001:oT[n00A:oP8000AS?003001S
?00000E[W003001_FVmJ00E_FP04001oo`00000LOol002]oo`<001Uoo`03001N^5jh009N^003001F
EeIG00IFE`03001>5dhG00I>5`030015n4Gh00=5n0H000A:>003001:NTYj00Q:NP04001:oT[nB_h2
0006Hc`200000f/j001[W002Ji`00`00KeY_FP05KeX00`00JaT00002Ool00`00Oomoo`03Ool2000D
Ool002Uoo`80009oo`03001oogoo01Ioo`03001N^5jh00=N^003001FEeIG00MFE`03001>5dhG00E>
5`030015n4Gh00<0009:5`03001:>4Xh00I:>003001:NTYj00Q:NP03001:oP0000QS?007001[>V/j
JcX006^L00001fmJ00@006/IJaT0009oo`03001oogoo009oo`04001oogoo000COol002Ioo`<001ao
o`03001N^5jh00EN^003001FEeIG00IFE`03001>5dhG00E>5`<000I:5`03001:>4Xh00I:>003001:
NTYj00M:NP8000QS?08000A[>P03001_FVmJ00E_FP03001[6F/I009[6@04001oogoo0003Ool01000
Oomoo`004goo000TOol2000NOol00`00G[QN^006G[P00`00EUMFE`06EUL00`00CQM>5`02CQL30000
0dhG001:5`06BQL00`00BSQ:>006BSP00`00BWY:NP06BWX00`00F[T00007Hc`00`00JcX00004JcX2
0005KeX00`00JaU[6@04JaT01000Oomoo`000Woo00@007ooOol001=oo`008Goo0`007Woo0P002Ejh
00<005IGEUL01eIG100014hG00<004XGBQL01DXG00<004XhBSP01dXh00<004YjBWX00dYj0P000eZi
00<006<lHc`00f<l00@006/jJcX000=[>P04001c>7<h0003KeX00`00JaU[6@05JaT01000Ool00000
0Woo00@007ooOol001=oo`007goo0P0087oo00<006LJ00002Ujh00<005IGEUL01EIG0P0024hG00<0
04XGBQL01DXG00<004XhBSP01TXh00@004YjBWY:NP8000IJ^@05001S?6<lHc`00002JcX20002JcX0
0`00LcQc>002LcP00`00KeX00008JaT20002Ool00`00Oomoo`02000DOol001aoo`<0025oo`05001W
6VLJIaX00009G[P00`00EUMFE`03EUL200000e8g001>5`07CQL00`00BQM:5`05BQL00`00BSQ:>007
BSP00`00BWX00009F[T00`00Hc`0000300001F/j001[>V/j00001G<h00<006/IJaT01V/I00<006KH
00000goo00<007ooOol05Goo000KOol2000ROol00`00IaYW6P03IaX00`00G[QN^006G[P01000EUMF
EeIG0P000e8g00<004hGCQL01ThG00<004XGBQL01TXG00<004XhBSP01TXh0P002UZi0`001F/j0P00
1W<h00<006/IJaT01F/I00D006KHI]QVf000009oo`03001oogoo01Eoo`0057oo0`0017oo00<007oo
000087oo00<006LJIaX016LJ00<005jhG[P01ejh0`001U8g00<004hGCQL01ThG00<004XGBQL01DXG
00<004XhBSP01DXh00<005Ih00001eZi1@001V/j0P001G<h00<007<h00001F/I00<006KHI]P00VKH
00D007ooOomoo`0001Ioo`005Goo00<007ooOol00goo00@007ooOol001ioo`03001W6VLJ00IW6P03
001N^5jh00EN^08000QB=`03001>5dhG00I>5`03001:5dXG00I:5`03001:>4Xh009:>08000=FN004
001J^EZiF[T30002F[T01000I_UVn@0016/j00@007<hLcP000=c>004001c>7<h0004JaT00`00I]QV
f003I]P01000Oomoogoo0P005Goo000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo000077oo
00<006LJIaX01fLJ00<005jhG[P00Ujh0`0000=FV@00DSL01e8g00<004hGCQL01ThG00<004XGBQL0
1TXG00<004XhBSP00P001EIh100015Zi00<006KiI_T00VKi00D006/jJcY[>P00009c>004001c>7<h
0004LcP01000JaU[6@001fKH00D007ooOomoo`0001Eoo`005Goo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol067oo00<006LJIaX02FLJ00@005jh000000EFV@03001B=e8g00IB=`03001>5dhG
00E>5`03001:5dXG00I:5`03001:>00000EFN0@000EJ^@03001VnFKi00AVn@03001[>P0000Ac>003
001c>00000Ac>003001[6@0000QVf003001oogoo009oo`03001oogoo019oo`005Goo00<007ooOol0
0Woo0P001Goo00<007ooOol05goo0P002VLJ0P001eJI00<0058gDSL01U8g00<004hGCQL01ThG00<0
04XGBQL01DXG0P000eIh0`000eIh00<005ZiF[T00UZi00<006KiI_T01VKi00<007<hLcP017<h00<0
07<hLcP017<h00<006KHI]P01fKH00<007ooOol00Woo00<007ooOol04Woo000DOol20004Ool00`00
Oomoo`05Ool00`00Oomoo`0EOol01000Ki]_V`001fLJ0P0000=O7000EYT01eJI00<0058gDSL01E8g
00<004hGCQL01ThG00<004XGBQL00dXG0P0000=B=`00EWP00P001eIh00@005ZiF[T000UVn@03001c
>7<h00=c>08000Ic>003001Vf6KH00QVf003001oogoo009oo`03001oogoo015oo`006Woo00<007oo
Ool01Woo00<007ooOol04goo00<006nKKi/00VnK00<006LJIaX00VLJ0P0015lL00<005JIEYT01EJI
00<0058gDSL01U8g00<004hGCQL01DhG00@004XGBQM:5`80009B=`<000UFN003001J^@0000UVn@03
001_>00000Ac>003001c>00000Ac>003001_>00000QVf08000Eoo`03001oogoo011oo`006Woo00<0
07ooOol01goo00<007ooOol04Goo00<006nKKi/016nK00@006LJ000000IO7003001FVEJI00IFV@03
001B=e8g00EB=`03001>5dhG00I>5`<00004DSL000000002DSL00`00EWQFN007EWP20009I_T01@00
KcQ_>6lh00000W<h00D007<hLcQc>000009c>004001_>6lh0007I]P00`00G[P00004Ool2000BOol0
01Uoo`8000Yoo`03001oogoo00moo`03001_VfnK00E_V`8000UO7003001FVEJI00EFV@03001B=e8g
00EB=`03001>5dhG00A>5`H000EB=`03001FN5Ih00IFN003001R^00000QVn@03001_>6lh009_>003
001c>00000Ec>003001c>00000=_>003001Vf6KH00=Vf005001N^5jhG[P00005Ool00`00Oomoo`0?
Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000]oo`03001oogoo00eoo`03001_VfnK00A_V`800003
Gkh005lL00QO7003001FVEJI00EFV@03001B=e8g00IB=`03001>5`0000H000UB=`03001FN5Ih00AF
N005001R^6:hH[P00006I_T00`00KcQ_>004KcP00`00LcQc>004LcP20004KcP00`00I]QVf002I]P0
0`00G[QN^002G[P00`00Oomoo`04Ool00`00Oomoo`0>Ool000Moo`04001oogoo0003Ool20006Ool0
1000Oomoo`003Goo00<007ooOol02goo00<006nKKi/016nK00<005nnGkh00Unn00<005lLGa`01UlL
00<005JIEYT01UJI00<0058gDSL00e8g100000A>=`00000000=>=`03001B=e8g00IB=`03001FN5Ih
00=FN003001R^6:h009R^003001VnFKi00AVn@03001_>6lh00A_>003001c>7<h00Ac>08000E_>004
001Vf6KH0005G[P00`00Oomoo`04Ool00`00Oomoo`0>Ool00003Ool0000000<00005Ool007ooOol0
000;Ool01000Oomoo`003Woo00<007ooOol027oo1@000VnK0P001Unn00<005lLGa`01UlL00<005JI
EYT01EJI00<0058gDSL00`000TXh0`001Thg00<0058gDSL01e8g00D005IhEWQFN00000IR^003001V
nFKi009Vn@03001_>6lh00A_>003001cF00000Ac>004001_>6lh0004KcP00`00I]P00007G[P00`00
Oomoo`04Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`0=Ool20007
Ool00`00Io]Wn`03Io/30008Gkh00`00GaaO7005Ga`00`00EYUFV@02EYT60002BSP300000dXh001>
=`06CSL00`00DSMB=`07DSL01000EWQFN00026:h00@006KiI_T000I_>005001cF7=HLeP00003LcP0
1000KcQ_>00016lh0P0025jh00<007ooOol01Goo00<007ooOol00Woo10001Woo0007Ool01000Oomo
o`0027oo00<007ooOol00goo00<007ooOol03goo00<007ooOol00goo00<006OkIo/00VOk0P000d>j
0`001Unn00<005lLGa`00UlL1`0014YJ00@004Xh000000A:>003001>=dhg00I>=`03001B=e8g00IB
=`03001FN00000UR^003001Vn@0000M_>003001cF7=H009cF003001c>00000A_>006001_>6lh001[
>P002Ejh00<007ooOol017oo00<007ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool0
0`00Oomoo`0@Ool00`00Oomoo`020004Io/00`00@kY3^P03@kX>0003@[h00`00BUY:FP04BUX30006
BSP00`00CSM>=`06CSL00`00DSMB=`06DSL2000;H[P00`00KcQ_>005KcP00`00LeQcF004LeP00`00
KcQ_>004KcP20003JcX00`00G[QN^006G[P00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`05
Ool001Qoo`8001=oo`8000AWn`P0009C?P03000NWP0000<000<[7`030012_T:n00A2_P03001:FTYJ
00<0009:FP03001:>4Xh00E:>003001>=dhg00E>=`03001B=e8g00EB=`03001JV00000]R^003001_
>6lh00A_>003001cF7=H00EcF003001_>6lh00A_>08000=[>P03001N^5jh00IN^003001oogoo00Io
o`03001oogoo00=oo`03001oogoo00Aoo`0067oo0P004goo00<006Ok00001P0025<n00@001jN7YhN
WP<0008[7`030012_T:n00=2_P<000E:FP03001:>4Xh00E:>003001>=dhg00I>=`03001B=e8g00=B
=`05001JV5ZHFYP00009H[P00`00JaT00004KcP00`00LeQcF005LeP00`00MiT00004KcP00`00JcX0
0003JcX00`00G[QN^006G[P20006Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomo
o`0@Ool600000f?N001W>`02Ic/20006Dch400000ajN0000000400001T:n00000000@[h000M:FP03
001:>4Xh00A:>003001>=dhg00I>=`04001B=e8gDSL20004FYP00`00H[QR^006H[P01@00JaU[6F/I
00000flh00<007=HLeP017=H00D007NIMiUgV@00009_>005001[>V/jJcX00003JcX00`00G[QN^003
G[P200000eJI001oo`07Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`0?
Ool01000AkP000000V?N0P001VLk00<005<nDch00`0000ABV`00=S`f?0<000030EP00000008000A2
_P03001:FTYJ00E:FP03001:>4Xh00E:>003001>=dhg00E>=`04001B=e8g0007FYP00`00H[QR^004
H[P00`00JaU[6@03JaT00`00KcP00006LeP00`00MiUgV@02MiT00`00KcP00005JcX01000JcY[>P00
0ejh0P0015JI00<007ooOol01Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol0
3Goo10000f?N00<006LkIc/01VLk0`001E:K00@003Hl=S`f?0D000040IP000000002@[h00`00BUY:
FP04BUX00`00BSQ:>005BSP00`00CSM>=`06CSL20008FYP00`00H[QR^003H[P00`00JaU[6@05JaT0
0`00LeQcF004LeP00`00MiUgV@04MiT00`00JcY[>P05JcX01000JcX005jh0P001UJI00<007ooOol0
1Woo00<007ooOol027oo000AOol01000Oomoo`000Woo0P003Goo100016?N00<006LkIc/01VLk00<0
06;I00001e:K0`0000<f?0002GX01@0000@=o000000000M:FP03001:>4Xh00A:>003001>=dhg00A>
=`<000UJV003001R^6:h009R^003001[6F/I00I[6@8000EcF003001gVGNI00=gV@03001_f`0000I[
>P04001[>P000008EYT00`00Oomoo`07Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00
Oomoo`09Ool;0006Ic/20003H]T00`00DY]BV`02DY/30004=S`010002GX9NPUj0`000`gl0`0014YJ
00<004XhBSP01DXh00<004hg00000`000U9I00<005ZHFYP025ZH00@006:hH[P000Q[6@04001cNG=i
0003LeP00`00MiUgV@03MiT01@00Km]_ffoK000016/j0`0000=O7000EYT025JI00<007ooOol01Woo
00<007ooOol01goo000BOol20003Ool20009Ool40009Ool20003Ic/00`00H]URf@04H]T40003DY/0
0`00=S`f?002=S`00`002GX9NP022GX200023O`00`00<Q`b70050004BSP50007DUT00`00FYQJV007
FYP00`00H[P00008JaT00`00LgUcN@02LgT00`00LeP00005MiT00`00Km]_f`03Km/01000JcX00000
0elL00<005lL000025JI00<007ooOol01goo00<007ooOol01Woo000GOol00`00Oomoo`05Ool4000>
Ool00`00Ic/00005H]T30006DY/500000cHl0009NP022GX400000`gl000b7004<Q`50003AUX00`00
DUUBF@06DUT00`00FYQJV007FYP20008JaT00`00LgUcN@04LgT00`00MiUgV@03MiT00`00Km]_f`04
Km/20005Ga`00`00Ga`00008EYT00`00Oomoo`08Ool00`00Oomoo`05Ool001Moo`03001oogoo00H0
019oo`04001RfF;IH]T30003H]T00`00DY]BV`0300000diH000n6@02?QT400022GX20000103g000=
o0001C8L00<004IJAUX01DIJ00<0059IDUT01U9I00<005ZHFYP01EZH00<006;j00001f/I00<007=i
LgT01G=i0P000gNI00<006oKKm/00foK0P0000=GgP00Ga`01ElL00<005lL000025JI00<007ooOol0
1goo00<007ooOol01Goo000GOol7000DOol40006H]T30005CUP00`00?QTn6@02?QT00`00:KX00004
0000103g000000001C8L00<004IJAUX014IJ00<0059IDUT01U9I00<005ZHFYP015ZH00D006;jH_YR
nP0000I[6@03001cNG=i00AcN@06001cnW?j001gV@0016oK0P0015ON00<005lLGa`015lL0P0025JI
00<007ooOol027oo00<007ooOol017oo0005Ool20004Ool20003Ool30004Ool2000GOol40007H]T0
0`00FYP00006CUP00`00?QTn6@03?QT01000:KXY^RVj0`000P3g00<0038L<Q`00c8L00<004IJAUX0
1DIJ00<0059IDUT01E9I00<005ZHFYP00UZH0P001F;j00<006/IJaT00V/I00<007=iLgT017=i00@0
07?jLoYcnP@00003Km/0000000MGgP03001O75lL00AO708000MFV@03001oogoo00Moo`<000Eoo`00
17oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000EOol30004Ool00`00H]URf@02H]T20003
FYP00`00CUQ>F004CUP00`00?QTn6@02?QT00`00:KXY^P03:KX200020?L00`00<Q`b7003<Q`00`00
AUY6FP04AUX00`00DUUBF@06DUT01000FYQJV0001f;j00D006/IJaU[6@0000IcN@05001cnW?jLoX0
0003EmP40007Emh00`00GaaO7004Ga`20004EYT50005Ool30002Ool00`00Oomoo`03Ool000Aoo`04
001oogoo000;Ool00`00Oomoo`02000COol20007Ool01@00H]URfF;I00001EZH00<004iHCUP014iH
00<003hI?QT00chI00<002Vj:KX00RVj00D000EJ0000003g008000<b70030016FTIJ00E6FP03001B
FE9I00EBF@03001JV00000URnP03001[6@0000IcN@05001cnW?jLoX00002EmPJ0004BUX00`00Oomo
o`030005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool2000@Ool3000:Ool00`00
H]T00007FYP00`00CUQ>F003CUP00`00?QTn6@03?QT00`00:KXY^P03:KX00`001EX0000200001PVk
00000000<Q`000I6FP03001BFE9I00EBF@8000YRnP<000EcN@05001cnW?jLoX00002EmP;00049ah0
0`00@[e2_@05@[d00`00BUY:FP06BUX20008Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool0
0`00Oomoo`03Ool00`00Oomoo`0<Ool3000>Ool00`00FYQJV006FYP00`00CUQ>F004CUP00`00?QTn
6@03?QT00`00:KXY^P03:KX00`001EX0000200032K/20007AUX01000DUUBFE9I0`0000=Fg000H_X0
26;j00D006^lJka[_00000=cN@06001cnW?jLoX005OH1`000RJO1P000bLN00<004:m@[d01D:m00<0
04YJBUX01DYJ00<007ooOol01goo00<007ooOol00goo0005Ool20009Ool40002Ool00`00Oomoo`09
Ool3000AOol00`00FYQJV006FYP00`00CUQ>F004CUP00`00?QTn6@03?QT00`00:KXY^P03:KX01P00
1EX5FP0009@000@9^`D00096FP04001BF@000005E]`00`00H_YRnP05H_X00`00Jka[_003Jk`00`00
LgT00003LoX50002DYX00`00?U/nF`0400020K/500000bLN0012_@05@[d00`00BUY:FP05BUX00`00
Oomoo`08Ool00`00Oomoo`02Ool001Ioo`03001oogoo00Ioo`<001Eoo`03001JV5ZH00IJV003001>
F4iH00=>F0`00004:KX000EJ1EX300032K/00`00>WdjO@02>Wd40008E]`00`00H_YRnP03H_X00`00
Jka[_004Jk`80006DYX00`00?U/nF`02?U/300030K/40007@[d00`00BUY:FP05BUX00`00Oomoo`07
Ool00`00Oomoo`02Ool001Eoo`8000Eoo`<001Qoo`03001JV5ZH00IJV003001>F4iH00<00003CSL0
04HG00A65`03000mnCgi008mn@@000031EX0000000800089^`03000jOCYm00@jO@03001Fg5KL00MF
g005001RnV;jH_X00006Jk`01000FoUKnE_i0P0000=Rf@00DYX01U:J00<003iK?U/00ciK00T001gK
0000006k0001F@000O/00`0014:m0P001TYJ00<007ooOol01goo00<007ooOol00Woo000EOol01000
Oomoogoo0`0077oo00<005ZHFYP01EZH0`001Dhg00<004HGAQL00dHG00<003gi?OT00cgi00<002gk
00000P0000H15`00000000Vk0006>Wd00`00E]aFg007E]`00`00H_X00005Jk`20002FoT30003H]T0
0`00DYYBVP05DYX00`00?U/nF`03?U/00`007M/Mf`0300000`5I000000028Qd60007BUX00`00Oomo
o`06Ool00`00Oomoo`02Ool001Eoo`@001moo`03001JV5ZH00=JV0800003EUL004hg00E>=`030016
5dHG00=65`03000mnCgi00@mn@03000]nbgk008000030AL00000008000LjO@03001Fg5KL00IFg0<0
00=[_080009Kn@800003Ie`006;I00ARf@03001BVU:J00ABVP03000nFciK00@nF`04000MfagK7M/2
00020ET00`008QdR7@038Qd40005BUX00`00Oomoo`06Ool00`00Oomoo`02Ool000ioo`<000Aoo`80
029oo`05001JV5ZHFYP00003EUL00`00CSM>=`04CSL00`00AQM65`04AQL00`00?OTmn@03?OT00`00
;O/]n`02;O/60006>Wd00`00E]aFg003E]`30003Dil00`00Jk`00003FoT200000fML001Rf@05H]T0
0`00DYYBVP04DYX600000ciK000Mf`047M/200000`5I000R7@048Qd00`00>S/000060008Ool00`00
Oomoo`02Ool000moo`03001oogoo00Eoo`03001oogoo01moo`04001JV0000005EUL00`00CSM>=`04
CSL00`00AQM65`03AQL00`00?OTmn@04?OT00`00;O/]n`03;O/90002>Wd00`00E]`000020006Dil6
0002KcT00`00H]URf@06H]T00`00DYYBVP0300000e9H00166@03AQT400037M/010001GX0005I0P00
128M00<003Xk000013Xk0P001goo00<007ooOol00Woo000?Ool00`00Oomoo`06Ool00`00Oomoo`0O
Ool00`00EUMFE`05EUL00`00CSM>=`04CSL00`00AQM65`03AQL00`00?OTmn@05?OT00`00;O/]n`03
;O/00`004QdB7@0300026Yh50005Dil60004KcT20008H]T30005DUP00`00AQU66@03AQT00`00=OT0
000200000agK0005NP0200000`6K000000028Qd00`00>S/00005>S/00`00Oomoo`06Ool00`00Oomo
o`01Ool000moo`03001oogoo00Moo`03001oogoo01ioo`03001FEeIG00EFE`03001>=dhg00A>=`03
00165dHG00A65`03000mnCgi00@mn@04000]nbgk;O/900066Yh200000e>O000000040005KcT01000
J_QZn0001F;I0P0000=JV000DUP01E9H00<004HIAQT014HI00@003Gi=OTen@<00085NP030001V`6K
008000048Qd003Xk0005>S/00`00Oomoo`05Ool00`00Oomoo`01Ool000moo`03001oogoo00Qoo`80
01moo`03001FEeIG00EFE`03001>=dhg00A>=`0300165dHG00=65`03000mnCgi008mn@H000A:NP03
0016oTKn0096oP`00003Ji`006li00A_>@03001Zn6[h009Zn005001RfF;IH]T00004FYP00`00DUQB
F004DUP00`00AQU66@03AQT00`00=OTen@04=OT200000`Ej0001V`020I/01@008Qd003Xk000013Xk
00<007ooOol01Goo00<007ooOol00Goo000>Ool2000<Ool00`00Oomoo`0LOol00`00EUMFE`05EUL0
0`00CSM>=`04CSL00`00AQM65`03AQL50003CSP00`00BWY:NP05BWX00`00A_i6oP02A_h00`00HcaS
?003Hc`00`00JcU[>@02JcT00`00KcU_>@03KcT00`00J_QZn003J_P01000H]T000001EZH00<0059H
DUP0159H00<004HIAQT014HI00<003Gi=OT00cGi00@001fk0005NP800081V`800003>S/003Xk00<j
>`03001oogoo00Eoo`03001oogoo005oo`007Goo00<007ooOol077oo00<005IGEUL015IG00<004hg
CSL014hg00<004HGAQL00`0000=>5`00CSP01Thh00<004YjBWX01DYj00@004Kn000000ES?003001[
>F/i00=[>@8000=_>@03001Zn6[h00EZn003001JV5ZH00EJV003001BF59H00EBF00300166DHI00=6
6@03000enCGi00@en@05000M^`000001F@0200020I/30004>S/00`00Oomoo`04Ool00`00Oomoo`01
Ool001ioo`03001oogoo01]oo`03001FEeIG00AFE`03001>=dhg00=>=`@000E>5`03001>>4hh00A>
>003001:NTYj00E:NP<000ES?003001[>F/i00=[>@06001_66lH001_>@0026[h00<005ZHFYP01UZH
00<0059HDUP0159H00<004HIAQT00dHI00<003Gi=OT013Gi00D001fk7K/M^`000081F@<000047O`0
00000002>S/00`00Oomoo`04Ool00`00Oomoo`01Ool001moo`8001]oo`03001FEeIG00EFE`03001>
=`000080009>5`03001>5dhG00A>5`03001>>4hh00E>>003001:NTYj00=:NP05001J^EZiF[T00003
Hc`00`00JcU[>@03JcT00`00KaQ_6002KaP00`00J_QZn006J_P00`00I]P00007FYP00`00DUQBF003
DUP;0004=OT01@007K/M^afk00000`5I0P000Qgl10001Woo00<007ooOol00Goo000QOol00`00Oomo
o`0IOol00`00EUMFE`03EUL30005CQL00`00CQM>5`04CQL00`00CSQ>>006CSP00`00BWY:NP020005
F[T00`00Hc`00006JcT00`00KaQ_6002KaP00`00J_QZn005J_P01000I]QVf0001eZH00<0059HDUP0
0`002goo1@0011fk0P000P5I0P0000<Mo0007O`00Qgl00<007ooOol017oo0@000Goo0Goo000ROol0
0`00Oomoo`0HOol00`00EUMFE`0300000e8g001>5`06CQL00`00CQM>5`04CQL00`00CSQ>>005CSP0
0`00BWX00008F[T00`00JcU[>@04JcT00`00KaQ_6002KaP00`00KaP00005J_P01@00I]QVf6KH0000
1eZH0`004Woo10000Qfk0`000P5I0`000Qgl00<007ooOol017oo0@000Goo0Goo000SOol00`00Oomo
o`0HOol20005DSL00`00CQM>5`04CQL00`00CQM>5`04CQL00`00CSQ>>005CSP20008F[T00`00I]P0
0004JcT00`00KaQ_6002KaP01@00KaQ_66lH00000f[h00<006KHI]P00fKH00<005ZHFYP00UZH0P00
6Goo100000D1V`000000006k0080008Mo003001oogoo00=oo`40005oo`5oo`0097oo00<007ooOol0
5goo00<0058gDSL0158g00<004hGCQL014hG00<004hGCQL01DhG00<004hhCSP00Thh0P0000=FN000
F[T01UZi00D006KHI]QVf00000=[>@03001_66lH009_6003001_66lH009_6003001Zn00000IVf005
001JV5ZHFYP0000NOol300000`6K000000020K/200000agl001oo`03Ool10001Ool1Ool002Eoo`80
01Qoo`03001B=e8g00=B=`03001>5dhG00E>5`03001>5dhG00A>5`05001>>4hhCSP00004EWP00`00
F[UJ^@03F[T00`00I]QVf003I]P00`00JcT00004KaP00`00KaQ_6004KaP00`00I]QVf005I]P01000
FYP000008Woo1P0000<1^`00000017oo0@000Goo0Goo000WOol00`00Oomoo`0EOol00`00DSMB=`04
DSL00`00CQM>5`04CQL00`00CQM>5`04CQL01000CSP000001EIh00<005ZiF[T00UZi00<006KHI]P0
1FKH00<006lHKaP00VlH00<006lHKaP01FlH00<006KHI]P01FKH0P009goo20000Woo0@000Goo0Goo
000XOol00`00Oomoo`0DOol00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CQM>5`04CQL20008EWP0
1000F[UJ^@0026KH0P000flH00<006lHKaP016lH00<006lH00001FKH00<007ooOol0:Woo1P0000=o
o`00Ool00Goo000YOol00`00Oomoo`0DOol00`00DSMB=`03DSL00`00CQM>5`04CQL01000CQM>5dhG
0`0000=B=`00EWP01eIh00<005Zi000026KH00H006lHKaP006lH0006KaP01000KaQ_600016KH00<0
07ooOol0;7oo00@007ooOomoo`@0005oo`00:Woo0P0057oo00<0058gDSL0158g00<004hGCQL014hG
0`001E8g00<005IhEWP01UIh00<006KHI]P01VKH00<006lHKaP00VlH00<006lHKaP016lH00<006lH
KaP00VlH00@006KHI]P0031oo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol04Woo00<0058g
DSL00e8g00<004hGCQL00dhG0P001e8g00<005IhEWP01UIh00<006KHI]P01FKH00<006lHKaP00flH
00<006lHKaP00flH00<006lHKaP00flH00<006KH0000<Woo00<007ooOol00Woo0@000Goo000]Ool0
0`00Oomoo`0AOol00`00DSMB=`03DSL00`00CQM>5`0300000e8g001B=`07DSL00`00EWQFN004EWP0
0`00H[P00005I]P00`00KaQ_6003KaP00`00LcP00003KaP00`00KaQ_6005KaP00`00Oomoo`0bOol0
1@00Oomoogoo00000Goo000^Ool00`00Oomoo`0@Ool01000DSMB=e8g1@001E8g00<0058gDSL01E8g
00<005IhEWP00eIh00D006:hH[QR^00000AVf003001_66lH009_6007001c>7<hLcP006lH00001flH
00<007ooOol0<goo00D007ooOomoo`00005oo`00;goo0P004Goo0`000dhh00<0058gDSL0158g00<0
058gDSL01E8g00<005IhEWP00UIh00<006:hH[P00V:h00D006KHI]QVf00000E_6003001c>7<h009c
>003001_66lH00E_6003001oogoo03Eoo`04001oogoo0001Ool0035oo`03001oogoo00ioo`03001>
>4hh00=>>003001B=e8g00AB=`03001B=e8g00IB=`04001FN5Ih0006H[P00`00I]P00005KaP00`00
LcQc>003LcP20005KaP00`00Oomoo`0gOol00`00Ool00001Ool0039oo`03001oogoo00ioo`03001>
>4hh00=>>003001B=e8g00AB=`03001B=e8g00EB=`03001FN00000QR^003001_66lH00A_6003001c
>7<h009c>004001cNG=i0003KaP00`00Oomoo`0hOol00`00Oomoo`01000003=oo`03001oogoo00eo
o`03001>>4hh00=>>003001B=e8g00AB=`03001B=e8g00EB=`8000QR^003001Zn00000A_6003001c
>7<h009c>003001cNG=i009cN@03001_600003aoo`40005oo`400000=7oo0P003Goo00<004hhCSP0
0dhh00<0058gDSL01E8g00<0058gDSL00U8g0P0000=JN000H[P01V:h00D006[hJ_QZn000009_6003
001c>7<h009c>003001cNG=i00=cN@8003ioo`800000=Woo00<007ooOol02goo00<004hhCSP00dhh
00<0058gDSL0158g00D0058gDSMB=`0000AJN003001R^6:h00=R^003001Zn6[h00=Zn003001_6000
00Ac>003001cNG=i009cN@03001oogoo03eoo`<00000=goo00<007ooOol02Woo00<004hhCSP00dhh
00<0058gDSL0158g00@0058g000000EJN003001R^6:h009R^003001Zn6[h00EZn08000=c>003001c
NG=i009cN@03001oogoo03Yoo`@000=oo`00>7oo00<007ooOol02Goo00<004hhCSP00dhh00<0058g
DSL00e8g0`0025Yh00@006:hH[P000QZn005001cN@00LcP00004LgT00`00Oomoo`0gOol40007Ool0
03Uoo`03001oogoo00Uoo`03001>>4hh00=>>003001B=`000080009BF003001JN5Yh00IJN003001R
^00000QZn004001cNG=iLgT20003LgT00`00Oomoo`0eOol3000;Ool003Yoo`8000Uoo`P000IBF003
001JN5Yh00IJN003001Zn6[h00IZn003001cNG=i009cN@05001cnP00LgT0000dOol4000>Ool003ao
o`03001oogoo00Ioo`030016FTIJ00=6FP03001BF59H00EBF003001JN5Yh00EJN08000MZn003001c
NG=i009cN@03001cnW?j0080035oo`@0019oo`00?Goo00<007ooOol01Woo00<004IJAUX00dIJ00<0
059HDUP01E9H00<005YhFWP00eYh00@006;iH_T000EZn003001cNG=i009cN@04001cnW?j000_Ool4
000FOol003ioo`03001oogoo00Eoo`030016FTIJ00=6FP03001BF59H00EBF003001JN5Yh009JN003
001RnF;i009Rn@05001Zn6[hJ_P00004LgT01000LoYcnP00;7oo10006Woo000oOol20006Ool00`00
AUY6FP03AUX00`00DUQBF005DUP01000FWQJN0001V;i00<006[h00001G=i00<007?j0000:Goo1000
7Woo0011Ool00`00Oomoo`04Ool00`00AUY6FP02AUX00`00DUQBF005DUP00`00FWP00007H_T30004
LgT00`00LoX0000WOol3000ROol0049oo`03001oogoo00=oo`030016FTIJ00=6FP03001BF59H00=B
F0<000MRn@04001[W6^LJi`2000017=i0000000097oo10009Goo0013Ool00`00Oomoo`03Ool40000
14IJ001BF59H0`000UKK00<006;iH_T00f;i0P001F^L0`008Woo1000:Goo0014Ool20004Ool00`00
=WdfO@040006E]/01@00H_URnF;i00001V^L0`007goo1000;Goo0016Ool00`00Oomoo`02Ool00`00
=WdfO@02=Wd00`00E]]Ff`05E]/00`00H_T00005Ji`3000MOol4000aOol004Moo`03001oogoo009o
o`03000fOCIm008fO@03001FfeKK00AFf`<000A[W0<001Yoo`@003Eoo`00B7oo00<007ooOol00Woo
100000@fO@00E]]Ff`<000=?W`800003Ji`0000001Qoo`@003Uoo`00BGoo0P0017oo1P001dnO0P00
5goo0`00?Goo001;Ool00`00Oomoo`03Ool60002Cil3000EOol40010Ool004aoo`03001oogoo00Eo
o`D001Aoo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo001>Ool00`00Oomo
o`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo00@0
07ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol200000goo00000002001K
Ool005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.258 0 0 r
.37614 .75111 .40063 .73217 .43872 .72292 .41492 .74343 Metetra
.41492 .74343 .43872 .72292 .47747 .70718 .45463 .72097 Metetra
.652 .949 .721 r
.45463 .72097 .47747 .70718 .51668 .68591 .49476 .68612 Metetra
.88 .953 .851 r
.49476 .68612 .51668 .68591 .55618 .66118 .53491 .64384 Metetra
.844 .804 .812 r
.53491 .64384 .55618 .66118 .59594 .63573 .57492 .60043 Metetra
.774 .692 .772 r
.57492 .60043 .59594 .63573 .63602 .61234 .61491 .56209 Metetra
.709 .615 .75 r
.61491 .56209 .63602 .61234 .6766 .59333 .65525 .53383 Metetra
.647 .56 .743 r
.65525 .53383 .6766 .59333 .71795 .58015 .69647 .51865 Metetra
.585 .521 .75 r
.69647 .51865 .71795 .58015 .76031 .57309 .73912 .51718 Metetra
.515 .497 .774 r
.73912 .51718 .76031 .57309 .80387 .57129 .78361 .52761 Metetra
.42 .489 .825 r
.78361 .52761 .80387 .57129 .84868 .57279 .8301 .54585 Metetra
.248 .503 .9 r
.8301 .54585 .84868 .57279 .89462 .57492 .87833 .56606 Metetra
0 .457 .798 r
.87833 .56606 .89462 .57492 .94137 .57475 .92766 .58161 Metetra
.294 0 0 r
.92766 .58161 .94137 .57475 .98855 .5697 .97713 .58639 Metetra
.35112 .75668 .37614 .75111 .41492 .74343 .39047 .74996 Metetra
.679 .978 .867 r
.39047 .74996 .41492 .74343 .45463 .72097 .43103 .72284 Metetra
.883 .957 .845 r
.43103 .72284 .45463 .72097 .49476 .68612 .47209 .67874 Metetra
.899 .867 .798 r
.47209 .67874 .49476 .68612 .53491 .64384 .51302 .62469 Metetra
.866 .789 .773 r
.51302 .62469 .53491 .64384 .57492 .60043 .55355 .56944 Metetra
.815 .724 .765 r
.55355 .56944 .57492 .60043 .61491 .56209 .59379 .52139 Metetra
.749 .665 .768 r
.59379 .52139 .61491 .56209 .65525 .53383 .63422 .48714 Metetra
.665 .607 .78 r
.63422 .48714 .65525 .53383 .69647 .51865 .67552 .47056 Metetra
.557 .547 .799 r
.67552 .47056 .69647 .51865 .73912 .51718 .71844 .47248 Metetra
.414 .482 .821 r
.71844 .47248 .73912 .51718 .78361 .52761 .76357 .49057 Metetra
.229 .415 .838 r
.76357 .49057 .78361 .52761 .8301 .54585 .81114 .51951 Metetra
.015 .359 .832 r
.81114 .51951 .8301 .54585 .87833 .56606 .8609 .55151 Metetra
0 .353 .781 r
.8609 .55151 .87833 .56606 .92766 .58161 .91199 .57747 Metetra
0 .454 .715 r
.91199 .57747 .92766 .58161 .97713 .58639 .96313 .58879 Metetra
.554 .801 .99 r
.32615 .74283 .35112 .75668 .39047 .74996 .36588 .73615 Metetra
.789 .839 .89 r
.36588 .73615 .39047 .74996 .43103 .72284 .40705 .70752 Metetra
.855 .816 .811 r
.40705 .70752 .43103 .72284 .47209 .67874 .44883 .66067 Metetra
.871 .794 .772 r
.44883 .66067 .47209 .67874 .51302 .62469 .49051 .60321 Metetra
.865 .777 .762 r
.49051 .60321 .51302 .62469 .55355 .56944 .53172 .54455 Metetra
.842 .763 .775 r
.53172 .54455 .55355 .56944 .59379 .52139 .57253 .49368 Metetra
.791 .744 .809 r
.57253 .49368 .59379 .52139 .63422 .48714 .6134 .4576 Metetra
.691 .706 .86 r
.6134 .4576 .63422 .48714 .67552 .47056 .65506 .44039 Metetra
.507 .625 .902 r
.65506 .44039 .67552 .47056 .71844 .47248 .6983 .44293 Metetra
.247 .486 .889 r
.6983 .44293 .71844 .47248 .76357 .49057 .74378 .46276 Metetra
.017 .351 .827 r
.74378 .46276 .76357 .49057 .81114 .51951 .79181 .49425 Metetra
0 .306 .796 r
.79181 .49425 .81114 .51951 .8609 .55151 .84215 .52912 Metetra
0 .389 .845 r
.84215 .52912 .8609 .55151 .91199 .57747 .89395 .5576 Metetra
.244 .592 .948 r
.89395 .5576 .91199 .57747 .96313 .58879 .94587 .57041 Metetra
.617 .643 .854 r
.3017 .70764 .32615 .74283 .36588 .73615 .34164 .69997 Metetra
.728 .68 .803 r
.34164 .69997 .36588 .73615 .40705 .70752 .38305 .67335 Metetra
.788 .706 .775 r
.38305 .67335 .40705 .70752 .44883 .66067 .42519 .63093 Metetra
.824 .734 .765 r
.42519 .63093 .44883 .66067 .49051 .60321 .46739 .57921 Metetra
.847 .768 .775 r
.46739 .57921 .49051 .60321 .53172 .54455 .50927 .52628 Metetra
.852 .812 .81 r
.50927 .52628 .53172 .54455 .57253 .49368 .55084 .47999 Metetra
.819 .864 .88 r
.55084 .47999 .57253 .49368 .6134 .4576 .59247 .44651 Metetra
.675 .882 .975 r
.59247 .44651 .6134 .4576 .65506 .44039 .63477 .42949 Metetra
.323 .736 .978 r
.63477 .42949 .65506 .44039 .6983 .44293 .67846 .42969 Metetra
0 .48 .851 r
.67846 .42969 .6983 .44293 .74378 .46276 .72412 .44493 Metetra
0 .357 .8 r
.72412 .44493 .74378 .46276 .79181 .49425 .77205 .47019 Metetra
.003 .385 .845 r
.77205 .47019 .79181 .49425 .84215 .52912 .8221 .4982 Metetra
.236 .484 .89 r
.8221 .4982 .84215 .52912 .89395 .5576 .87358 .5205 Metetra
.46 .576 .884 r
.87358 .5205 .89395 .5576 .94587 .57041 .92535 .52916 Metetra
.62 .568 .774 r
.27803 .65393 .3017 .70764 .34164 .69997 .31804 .6444 Metetra
.675 .592 .755 r
.31804 .6444 .34164 .69997 .38305 .67335 .3593 .62276 Metetra
.72 .628 .754 r
.3593 .62276 .38305 .67335 .42519 .63093 .40133 .59093 Metetra
.762 .68 .772 r
.40133 .59093 .42519 .63093 .46739 .57921 .44367 .5529 Metetra
.803 .758 .812 r
.44367 .5529 .46739 .57921 .50927 .52628 .48605 .51382 Metetra
.826 .874 .88 r
.48605 .51382 .50927 .52628 .55084 .47999 .52842 .47879 Metetra
.727 .974 .927 r
.52842 .47879 .55084 .47999 .59247 .44651 .57101 .45197 Metetra
.328 .807 .753 r
.57101 .45197 .59247 .44651 .63477 .42949 .61421 .43587 Metetra
.046 0 0 r
.61421 .43587 .63477 .42949 .67846 .42969 .65851 .43102 Metetra
.171 0 0 r
.65851 .43102 .67846 .42969 .72412 .44493 .7043 .4359 Metetra
0 .512 .819 r
.7043 .4359 .72412 .44493 .77205 .47019 .75181 .44707 Metetra
.264 .579 .942 r
.75181 .44707 .77205 .47019 .8221 .4982 .80095 .4597 Metetra
.457 .567 .878 r
.80095 .4597 .8221 .4982 .87358 .5205 .85131 .46836 Metetra
.557 .559 .812 r
.85131 .46836 .87358 .5205 .92535 .52916 .90221 .46812 Metetra
.622 .539 .74 r
.25503 .58849 .27803 .65393 .31804 .6444 .29503 .5766 Metetra
.633 .542 .734 r
.29503 .5766 .31804 .6444 .3593 .62276 .33582 .56166 Metetra
.65 .567 .748 r
.33582 .56166 .3593 .62276 .40133 .59093 .37729 .54409 Metetra
.674 .623 .79 r
.37729 .54409 .40133 .59093 .44367 .5529 .41934 .52482 Metetra
.705 .734 .873 r
.41934 .52482 .44367 .5529 .48605 .51382 .46191 .50506 Metetra
.679 .907 .975 r
.46191 .50506 .48605 .51382 .52842 .47879 .50497 .48608 Metetra
.317 .796 .724 r
.50497 .48608 .52842 .47879 .57101 .45197 .54859 .46895 Metetra
.036 0 0 r
.54859 .46895 .57101 .45197 .61421 .43587 .59288 .45433 Metetra
.134 0 0 r
.59288 .45433 .61421 .43587 .65851 .43102 .63797 .44238 Metetra
.044 0 0 r
.63797 .44238 .65851 .43102 .7043 .4359 .68397 .43267 Metetra
.321 .801 .901 r
.68397 .43267 .7043 .4359 .75181 .44707 .73097 .42428 Metetra
.578 .757 .963 r
.73097 .42428 .75181 .44707 .80095 .4597 .77896 .41599 Metetra
.612 .626 .84 r
.77896 .41599 .80095 .4597 .85131 .46836 .82786 .40647 Metetra
.617 .563 .771 r
.82786 .40647 .85131 .46836 .90221 .46812 .87752 .39457 Metetra
.628 .537 .732 r
.23218 .52018 .25503 .58849 .29503 .5766 .27218 .50587 Metetra
.597 .515 .733 r
.27218 .50587 .29503 .5766 .33582 .56166 .31234 .4978 Metetra
.57 .517 .756 r
.31234 .4978 .33582 .56166 .37729 .54409 .35297 .49508 Metetra
.541 .552 .815 r
.35297 .49508 .37729 .54409 .41934 .52482 .39438 .49572 Metetra
.484 .649 .931 r
.39438 .49572 .41934 .52482 .46191 .50506 .43679 .49699 Metetra
.258 .733 .939 r
.43679 .49699 .46191 .50506 .50497 .48608 .48031 .49588 Metetra
.07 0 0 r
.48031 .49588 .50497 .48608 .54859 .46895 .52489 .48974 Metetra
.132 0 0 r
.52489 .48974 .54859 .46895 .59288 .45433 .57033 .47687 Metetra
.028 0 0 r
.57033 .47687 .59288 .45433 .63797 .44238 .61637 .45688 Metetra
.61637 .45688 .63797 .44238 .68397 .43267 .66279 .43084 Metetra
.708 .979 .91 r
.66279 .43084 .68397 .43267 .73097 .42428 .70946 .40101 Metetra
.773 .824 .892 r
.70946 .40101 .73097 .42428 .77896 .41599 .75644 .37033 Metetra
.714 .669 .804 r
.75644 .37033 .77896 .41599 .82786 .40647 .80398 .34182 Metetra
.664 .583 .754 r
.80398 .34182 .82786 .40647 .87752 .39457 .85246 .31797 Metetra
.641 .558 .746 r
.20873 .45781 .23218 .52018 .27218 .50587 .24879 .44142 Metetra
.564 .511 .754 r
.24879 .44142 .27218 .50587 .31234 .4978 .28836 .43901 Metetra
.468 .474 .782 r
.28836 .43901 .31234 .4978 .35297 .49508 .3281 .44874 Metetra
.327 .45 .833 r
.3281 .44874 .35297 .49508 .39438 .49572 .36871 .46645 Metetra
.094 .436 .876 r
.36871 .46645 .39438 .49572 .43679 .49699 .41073 .48617 Metetra
0 .391 .721 r
.41073 .48617 .43679 .49699 .48031 .49588 .45446 .50111 Metetra
.224 0 0 r
.45446 .50111 .48031 .49588 .52489 .48974 .4998 .50501 Metetra
.051 0 0 r
.4998 .50501 .52489 .48974 .57033 .47687 .54632 .49369 Metetra
.54632 .49369 .57033 .47687 .61637 .45688 .5934 .46624 Metetra
.693 .971 .76 r
.5934 .46624 .61637 .45688 .66279 .43084 .64046 .42529 Metetra
.879 .964 .848 r
.64046 .42529 .66279 .43084 .70946 .40101 .68717 .3763 Metetra
.856 .828 .819 r
.68717 .3763 .70946 .40101 .75644 .37033 .73362 .32613 Metetra
.783 .707 .78 r
.73362 .32613 .75644 .37033 .80398 .34182 .78029 .28146 Metetra
.709 .619 .755 r
.78029 .28146 .80398 .34182 .85246 .31797 .82797 .24757 Metetra
.662 .609 .785 r
.18374 .40848 .20873 .45781 .24879 .44142 .22399 .39063 Metetra
.528 .538 .81 r
.22399 .39063 .24879 .44142 .28836 .43901 .26322 .39163 Metetra
.317 .439 .827 r
.26322 .39163 .28836 .43901 .3281 .44874 .30231 .40913 Metetra
.022 .308 .799 r
.30231 .40913 .3281 .44874 .36871 .46645 .34225 .43771 Metetra
0 .201 .705 r
.34225 .43771 .36871 .46645 .41073 .48617 .38386 .46944 Metetra
0 .221 .634 r
.38386 .46944 .41073 .48617 .45446 .50111 .42762 .49499 Metetra
.16 0 0 r
.42762 .49499 .45446 .50111 .4998 .50501 .47349 .50556 Metetra
.283 .785 .838 r
.47349 .50556 .4998 .50501 .54632 .49369 .52089 .49514 Metetra
.71 .984 .889 r
.52089 .49514 .54632 .49369 .5934 .46624 .56897 .46242 Metetra
.882 .967 .842 r
.56897 .46242 .5934 .46624 .64046 .42529 .61686 .41123 Metetra
.911 .893 .799 r
.61686 .41123 .64046 .42529 .68717 .3763 .66405 .34935 Metetra
.884 .817 .777 r
.66405 .34935 .68717 .3763 .73362 .32613 .71061 .2863 Metetra
.829 .745 .771 r
.71061 .2863 .73362 .32613 .78029 .28146 .75714 .23111 Metetra
.754 .677 .776 r
.75714 .23111 .78029 .28146 .82797 .24757 .80465 .19071 Metetra
.69 .707 .861 r
.15633 .3766 .18374 .40848 .22399 .39063 .19693 .35807 Metetra
.472 .623 .917 r
.19693 .35807 .22399 .39063 .26322 .39163 .23622 .35965 Metetra
.084 .414 .864 r
.23622 .35965 .26322 .39163 .30231 .40913 .27518 .37886 Metetra
0 .189 .699 r
.27518 .37886 .30231 .40913 .34225 .43771 .31491 .40998 Metetra
0 .123 .626 r
.31491 .40998 .34225 .43771 .38386 .46944 .35638 .44459 Metetra
0 .251 .723 r
.35638 .44459 .38386 .46944 .42762 .49499 .40019 .47267 Metetra
.143 .553 .925 r
.40019 .47267 .42762 .49499 .47349 .50556 .44637 .48469 Metetra
.575 .795 .983 r
.44637 .48469 .47349 .50556 .52089 .49514 .49438 .47416 Metetra
.79 .851 .896 r
.49438 .47416 .52089 .49514 .56897 .46242 .54325 .43963 Metetra
.867 .841 .817 r
.54325 .43963 .56897 .46242 .61686 .41123 .59202 .38528 Metetra
.889 .822 .775 r
.59202 .38528 .61686 .41123 .66405 .34935 .64008 .31955 Metetra
.885 .805 .765 r
.64008 .31955 .66405 .34935 .71061 .2863 .68739 .25271 Metetra
.858 .788 .781 r
.68739 .25271 .71061 .2863 .75714 .23111 .73456 .19438 Metetra
.798 .761 .819 r
.73456 .19438 .75714 .23111 .80465 .19071 .7826 .15189 Metetra
.692 .864 .963 r
.12577 .36346 .15633 .3766 .19693 .35807 .16688 .34506 Metetra
.324 .737 .978 r
.16688 .34506 .19693 .35807 .23622 .35965 .20677 .34421 Metetra
0 .405 .771 r
.20677 .34421 .23622 .35965 .27518 .37886 .24633 .35868 Metetra
0 .218 .652 r
.24633 .35868 .27518 .37886 .31491 .40998 .28658 .38339 Metetra
0 .244 .723 r
.28658 .38339 .31491 .40998 .35638 .44459 .32847 .41091 Metetra
.068 .408 .86 r
.32847 .41091 .35638 .44459 .40019 .47267 .37257 .43258 Metetra
.4 .563 .899 r
.37257 .43258 .40019 .47267 .44637 .48469 .41899 .44028 Metetra
.612 .646 .86 r
.41899 .44028 .44637 .48469 .49438 .47416 .46728 .42849 Metetra
.729 .691 .813 r
.46728 .42849 .49438 .47416 .54325 .43963 .51662 .39602 Metetra
.798 .724 .783 r
.51662 .39602 .54325 .43963 .59202 .38528 .56614 .34636 Metetra
.84 .757 .771 r
.56614 .34636 .59202 .38528 .64008 .31955 .61523 .28671 Metetra
.864 .794 .781 r
.61523 .28671 .64008 .31955 .68739 .25271 .6638 .22588 Metetra
.864 .836 .817 r
.6638 .22588 .68739 .25271 .73456 .19438 .71229 .17226 Metetra
.821 .875 .885 r
.71229 .17226 .73456 .19438 .7826 .15189 .76156 .13233 Metetra
.514 .913 .902 r
.09168 .36711 .12577 .36346 .16688 .34506 .13342 .3496 Metetra
.112 .663 .711 r
.13342 .3496 .16688 .34506 .20677 .34421 .17448 .34355 Metetra
.114 0 0 r
.17448 .34355 .20677 .34421 .24633 .35868 .2155 .34739 Metetra
0 .473 .775 r
.2155 .34739 .24633 .35868 .28658 .38339 .2572 .35764 Metetra
.157 .539 .924 r
.2572 .35764 .28658 .38339 .32847 .41091 .30024 .36937 Metetra
.394 .552 .893 r
.30024 .36937 .32847 .41091 .37257 .43258 .34507 .37701 Metetra
.528 .553 .824 r
.34507 .37701 .37257 .43258 .41899 .44028 .39177 .3755 Metetra
.61 .566 .78 r
.39177 .3755 .41899 .44028 .46728 .42849 .4401 .36152 Metetra
.67 .593 .761 r
.4401 .36152 .46728 .42849 .51662 .39602 .48952 .33432 Metetra
.721 .635 .761 r
.48952 .33432 .51662 .39602 .56614 .34636 .53948 .29602 Metetra
.769 .694 .78 r
.53948 .29602 .56614 .34636 .61523 .28671 .58954 .25102 Metetra
.812 .778 .822 r
.58954 .25102 .61523 .28671 .6638 .22588 .63959 .20487 Metetra
.828 .886 .885 r
.63959 .20487 .6638 .22588 .71229 .17226 .68987 .16306 Metetra
.745 .967 .933 r
.68987 .16306 .71229 .17226 .76156 .13233 .74093 .12997 Metetra
.05417 .38246 .09168 .36711 .13342 .3496 .0966 .3664 Metetra
.0966 .3664 .13342 .3496 .17448 .34355 .13937 .35304 Metetra
.13937 .35304 .17448 .34355 .2155 .34739 .18267 .34193 Metetra
.414 .841 .955 r
.18267 .34193 .2155 .34739 .2572 .35764 .22671 .33214 Metetra
.575 .758 .965 r
.22671 .33214 .2572 .35764 .30024 .36937 .27168 .32241 Metetra
.601 .628 .851 r
.27168 .32241 .30024 .36937 .34507 .37701 .31772 .31139 Metetra
.604 .56 .778 r
.31772 .31139 .34507 .37701 .39177 .3755 .36486 .2979 Metetra
.61 .534 .744 r
.36486 .2979 .39177 .3755 .4401 .36152 .41308 .28118 Metetra
.622 .536 .737 r
.41308 .28118 .4401 .36152 .48952 .33432 .46225 .26107 Metetra
.642 .565 .753 r
.46225 .26107 .48952 .33432 .53948 .29602 .51227 .23803 Metetra
.67 .627 .798 r
.51227 .23803 .53948 .29602 .58954 .25102 .56302 .21305 Metetra
.703 .74 .88 r
.56302 .21305 .58954 .25102 .63959 .20487 .61448 .18743 Metetra
.693 .897 .971 r
.61448 .18743 .63959 .20487 .68987 .16306 .66672 .16254 Metetra
.485 .901 .868 r
.66672 .16254 .68987 .16306 .74093 .12997 .7199 .13951 Metetra
.01394 .40164 .05417 .38246 .0966 .3664 .05706 .38729 Metetra
.05706 .38729 .0966 .3664 .13937 .35304 .1019 .36564 Metetra
.561 .933 .811 r
.1019 .36564 .13937 .35304 .18267 .34193 .14808 .33777 Metetra
.768 .955 .934 r
.14808 .33777 .18267 .34193 .22671 .33214 .19511 .30601 Metetra
.771 .805 .88 r
.19511 .30601 .22671 .33214 .27168 .32241 .24259 .27336 Metetra
.714 .671 .806 r
.24259 .27336 .27168 .32241 .31772 .31139 .29024 .24289 Metetra
.66 .583 .758 r
.29024 .24289 .31772 .31139 .36486 .2979 .33804 .21714 Metetra
.617 .531 .735 r
.33804 .21714 .36486 .2979 .41308 .28118 .38614 .19766 Metetra
.583 .507 .735 r
.38614 .19766 .41308 .28118 .46225 .26107 .43486 .18474 Metetra
.555 .51 .76 r
.43486 .18474 .46225 .26107 .51227 .23803 .48459 .17743 Metetra
.532 .55 .819 r
.48459 .17743 .51227 .23803 .56302 .21305 .53565 .17359 Metetra
.502 .649 .924 r
.53565 .17359 .56302 .21305 .61448 .18743 .58822 .17028 Metetra
.404 .781 .992 r
.58822 .17028 .61448 .18743 .66672 .16254 .64228 .16429 Metetra
.206 .732 .768 r
.64228 .16429 .66672 .16254 .7199 .13951 .6976 .15276 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`06Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000[Ool00`00Oomoo`07Ool00`00Oomoo`0TOol004Moo`8000]oo`<002eoo`03001oogoo
00Moo`03001oogoo02Aoo`00DWoo0`00;goo00<007ooOol027oo00<007ooOol08goo001?Ool3000b
Ool00`00Oomoo`09Ool00`00Oomoo`0ROol004aoo`<003Eoo`03001oogoo009oo`H00003Ool007oo
02=oo`00BWoo0`00=goo00<007oo00000`000UoM00H006]lJg`007oo000SOol004Moo`<0009oo`03
001oogoo03Eoo`800003?i/00000009Og@03001[O6]l009[O003001oo`00029oo`00AGoo0P00>goo
1@000eoM00<006]lJg`016]l0P008Woo0012Ool3000hOol50002?i/00`00GmeOg@02Gmd00`00Jga[
O004Jg`200000goo001oo`0POol002Eoo`@001Moo`8003Eoo`h000=Og@03001[O6]l00A[O006001W
6VLJ001oo`0087oo000VOol00`00Oomoo`0DOol3000_Ool80005<al20003Fil00`00Jia[W0020000
0eoM001[O005Jg`01000IaYW6VLJ0`0087oo000OOol50003Ool00`00Oomoo`0AOol2000SOol?0006
@Yd00`00<alc7`02<al20004Fil01P00Jia[W6^L001[W08000E[O003001W6VLJ00=W6P03001oo`00
01moo`00:7oo00<007ooOol03Goo0`008Goo10000THH00<004HjASX01DHj00<004:M@Yd02D:M00@0
03<O000000EKW`03001[W6^L009[W005001[W000KeX00003Jg`00`00IaYW6P05IaX2000OOol002Eo
o`04001oogoo000=Ool2000POol40007AQP00`00ASY6>P05ASX00`00@Ye2W@08@Yd30005Fil00`00
Jia[W004Ji`01`00KeY_FVmJ001[O00000QW6P03001oo`0001ioo`009Woo0P002goo0`007Woo1000
0TXG00<004HHAQP01THH00<004HjASX01THj00<004:M@Yd01D:M0P000e[l00D005^OFimKW`0000I[
W08000A_FP03001W6VLJ00MW6P800003Ool007oo01aoo`00<Goo0P0000=oo`00Ool06goo0`001dXG
00<004HHAQP01DHH00<004HjASX01dHj00<004:M@Yd00T:M0P001U[l00<005^O00001V^L00<006mJ
000016mJ00<006LJIaX01VLJ00D006:h001oo`0001eoo`00;Woo0`007Goo0`0000=>5`00BQL01dXG
00<004HHAQP01THH00<004HjASX01dHj00@004:M000000QJo0<000E[W004001_FVmJ0003KeX00`00
KcP00006IaX01000H[QR^6:h0`0077oo000[Ool3000NOol20005CQL00`00BQM:5`05BQL00`00AQQ6
6006AQP00`00ASY6>P07ASX20009F_`01@00IaYW6VLJ00000f^L00D006mJKeY_FP00009_FP04001_
>6lh0004IaX20004H[P01000Oomoo`001Goo0P0057oo000YOol20002Ool00`00Oomoo`0JOol20007
CQL00`00BQM:5`06BQL00`00AQQ66006AQP00`00ASY6>P04ASX200000eJI001Jo007F_`00`00IaYW
6P03IaX00`00Ji`00004KeX00`00KeX00004KcP01000IaYW6P001f:h00<007oo000017oo00@007oo
Ool001=oo`009Woo0`007goo0P0000=FF000CQL01ThG00<004XGBQL01TXG00<004HHAQP01THH00@0
04HjASY6>P<000AFV@03001Jo5[l00AJo003001W6VLJ00EW6P03001_FVmJ00=_FP8000E_>003001W
6P0000QR^004001oogoo0003Ool01000Oomoo`004goo000TOol2000POol20003EUP00`00CQM>5`06
CQL00`00BQM:5`05BQL00`00AQQ66006AQP01000ASX0000025JI00<005[lF_`00U[l00<006LJIaX0
1VLJ00<006mJKeX00fmJ00<006lhKcP01Flh00<006:hH[P01f:h00D007ooOomoo`00009oo`04001o
ogoo000COol0025oo`<0021oo`8000IFF003001>5dhG00E>5`03001:5dXG00I:5`03001664HH00E6
608000YFV@05001Jo5[lF_`00008IaX00`00KcP00003KeX20007KcP00`00H[QR^008H[P01000Ool0
00000Woo00@007ooOol001=oo`007goo0P008Goo0P0025IH00<004hGCQL01ThG00<004XGBQL01DXG
00<004HHAQP00dHH0P0000=BF000EYT02UJI00<005[l000026LJ00H006lhKcQ_>000KeX30006KcP0
0`00J_P00009H[P01@00Oomoogoo00000Woo0P0057oo000LOol3000ROol00`00F[T00008EUP00`00
CQM>5`06CQL00`00BQM:5`05BQL01000AQQ664HH0P00159H00<005JIEYT025JI0P0026LJ00<006lh
KcP00Vlh0`0000=_>000KcP016lh00@006[hJ_P000UR^08000=oo`03001oogoo01Eoo`006goo0P00
8Woo0P000eZi00<005IHEUP01UIH00<004hGCQL01DhG00<004XGBQL01TXG0`001U9H00<005JIEYT0
1eJI00<006;H00001fLJ00<006lhKcP00`0000=_>000KcP00Vlh00D006lhKcQ_>00000AZn003001R
^6:h00ER^003001JN00000=oo`03001oogoo01Eoo`0057oo0`0017oo00<007oo000087oo00<005Zi
F[T00eZi00<005IHEUP01UIH00<004hGCQL01ThG00<004XGBQL014XG0P002E9H00<005JIEYT01EJI
00D006;HH]QRf00000IW6P<000=_>003001c>00000=_>003001_>00000EZn003001R^6:h00=R^080
00=JN005001oogooOol0000FOol001Eoo`03001oogoo00=oo`04001oogoo000NOol00`00F[UJ^@05
F[T00`00EUQFF005EUP00`00CQM>5`06CQL01000BQM:5dXG0`0000=>=`00DUP02E9H00<005JIEYT0
0eJI00<006;HH]P00f;H00<006LJIaX00`001Flh00D007<hLcQc>00000=_>08000IZn003001R^6:h
008000EJN004001oogooOol2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0000K
Ool20008F[T00`00EUQFF006EUP00`00CQM>5`05CQL01000BQL000001Dhg00<0059HDUP01e9H00<0
05JIEYT00UJI00<006;HH]P016;H0`0000=W6P00KcP01Flh00D007<hLcQc>00000=_>003001Zn6[h
00EZn003001R^00000MJN003001oogoo009oo`03001oogoo01=oo`005Goo00<007ooOol00Woo0P00
17oo00<007ooOol05goo00<006<l00002EZi00<005IHEUP01EIH00<004hGCQL01DhG0P001dhg00<0
059HDUP0259H00@005JIEYT000ARf0<00004H]P006LJ0006KcP00`00LcQc>003LcP01@00KcP006[h
00001f[h00<005YhFWP01eYh00<007ooOol00Woo00<007ooOol04Woo000EOol00`00Oomoo`02Ool2
0005Ool00`00Oomoo`0DOol20003Hc`00`00F[UJ^@06F[T00`00EUQFF005EUP00`00CQM>5`02CQL3
00000dhg001>=`07CSL00`00DUQBF007DUP00`00EYT00002H]P30005H]P00`00KcQ_>004KcP00`00
LcQc>005LcP01@00J_QZn6[h00001F[h0P002EYh00<007ooOol00Woo00<007ooOol04Woo000DOol2
0004Ool00`00Oomoo`05Ool00`00Oomoo`0AOol20005Hc`00`00F[UJ^@06F[T00`00EUQFF006EUP4
0004CSL00`00CSM>=`06CSL00`00DUQBF007DUP200000f;H00000008H]P00`00KcQ_>004KcP00`00
LcQc>005LcP01@00J_QZn6[h000016[h00@006KHI]P000QJN003001oogoo00=oo`03001oogoo015o
o`006Woo00<007ooOol01Woo00<007ooOol03goo00<006<lHc`01V<l00<005ZiF[T01UZi00<005IH
EUP00eIH0`0024hg00<004hgCSL01Thg00<0059HDUP01E9H10002F;H00<006/H000016lh00<007<h
LcP01G<h00<007=H00000f[h00@006[hJ_P000=Vf003001JN5Yh00MJN003001oogoo00=oo`03001o
ogoo011oo`006Woo00<007ooOol01goo00<007ooOol037oo0`002F<l00<005ZiF[T01EZi00<005IH
EUP00`000Thh00<004hgCSL01Thg00<004hgCSL01Thg00<0059HDUP00e9H0`000UjH00<006;HH]P0
1V;H00D006/HJaQ[600000=_>003001c>7<h00Ac>005001cF7=HLeP00003J_P20004I]P00`00FWQJ
N007FWP00`00Oomoo`02Ool2000BOol001Uoo`8000Yoo`03001oogoo00Yoo`05001SgF?MHmd00008
Hc`00`00F[UJ^@02F[T60006CSP00`00CSM>=`05CSL00`00CSM>=`06CSL00`00DUQBF0040004GYP0
0`00H]QRf005H]P00`00JaQ[6003JaP00`00KcP00007LcP01@00LeQcF7=H00000f[h0P001FKH00<0
05YhFWP01UYh00<007ooOol017oo00<007ooOol03goo0008Ool20004Ool20003Ool30003Ool2000;
Ool00`00Oomoo`07Ool20005Hmd00`00HcaS?004Hc`60004CY/00`00CSQ>>005CSP00`00CSM>=`06
CSL00`00CSM>=`05CSL300000e9H001NV006GYP00`00H]QRf003H]P00`00JaQ[6005JaP00`00LcQc
>005LcP00`00LeQcF003LeP01@00J_P006KH00001FKH00<005YhFWP01UYh0P001Woo00<007ooOol0
3Woo0007Ool01000Oomoo`000goo0P001Woo00@007ooOol000eoo`03001oogoo00Eoo`03001SgF?M
00ISg@03001S?6<l00<0009;7P03001>VdjK00I>V`03001>>4hh00A>>003001>=dhg00I>=`03001>
=dhg009>=`<00004CSL0059H0008GYP00`00H]QRf002H]P00`00JaQ[6006JaP00`00LcQc>004LcP0
0`00LeQcF005LeP01@00I]QVf6KH00001FKH00<005YhFWP00eYh0P0000=FF000Ool01Goo00<007oo
Ool03Woo00000goo0000000300001Goo001oogoo00002goo00@007ooOol000ioo`03001oogoo00=o
o`03001SgF?M00ISg@@000I;7P03001>VdjK00E>V`03001>>4hh00E>>003001>=dhg00E>=`03001>
=`00008000A>=`8000YNV004001Rf6;H0008JaP00`00LeP00004LcP00`00LeQcF005LeP20004I]P0
0`00I]QVf002I]P01000FWQJN5Yh0P0015IH00<007ooOol017oo00<007ooOol03Goo0007Ool01000
Oomoo`0027oo0`000goo00<007ooOol03Goo0P0000=oo`0000000`000f?M0`0013MN00<004/NBah0
14/N00<004jKCY/014jK00<004hhCSP01Dhh00<004hgCSL014hg0`001Thg00<005IH00002UjH00<0
06;H000026/H00D007=HLeQcF00000=c>003001cF7=H00AcF004001_FVmJ0003I]P00`00I]QVf002
I]P01000FWP000001UIH00<007ooOol01Goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`00
27oo00<007ooOol00goo00<007ooOol03Woo0P000dNi2`000SMN00<004/NBah00d/N00<004jKCY/0
1DjK00<004hhCSP014hh00<004hgCSL00Thg0P000Thg00<004hgCSL00Thg0P000eIH00<005jHGYP0
25jH00<006/HJaP01V/H00<007=HLeP00g=H00<007<h00001W=H00D006mJKeY_FP0000AVf004001V
f6KHI]P20008EUP00`00Oomoo`05Ool00`00Oomoo`03Ool00`00Oomoo`06Ool000Qoo`8000Uoo`@0
009oo`03001oogoo00aoo`L000=KOP8000@Zo`H000E;7P03001>VdjK00A>V`03001>>4hh00E>>0@0
00A>=`03001>=dhg008000EFF003001NV5jH00MNV08000M[6003001cF7=H00EcF003001cF7=H00Ac
F003001_FVmJ00=_FP03001Vf6KH009Vf0<00003F[T005IH00QFF003001oogoo00Eoo`03001oogoo
00=oo`03001oogoo00Eoo`0067oo0P0037oo1P001e]n3P000T/N00<004jKCY/01DjK00<004hhCSP0
0Thh0`0024hg0P001eIH00<005jHGYP01UjH00@006KiI_T000I[6003001cF7=H00EcF003001cF7=H
00=cF003001_FVmJ00A_FP04001Vf6KHI]P30003F[T00`00EUQFF006EUP00`00Oomoo`06Ool00`00
Oomoo`03Ool00`00Oomoo`04Ool001Qoo`8000Uoo`P000IKOP80009Jf`03000nOCim00<nO@030009
^`0000L000I>V`03001>>0000080009>>003001>=dhg00A>=`<000UFF003001NV5jH00ANV003001V
nFKi009Vn@03001[66/H009[6003001cF7=H00EcF003001gV@0000=cF003001_FVmJ00E_FP04001V
f0000002F[T01@00F[UJ^EZi000025IH00<007ooOol01Goo0P000Woo00@007ooOol000Ioo`004Woo
0P0017oo00<007ooOol01Woo1P0017oo0`0000=KOP0000001E[K00<003im?Wd00cim00<000Vk2K/0
0`000QHm2@001Thh00<004hg00000`000TiH00<005IHEUP01eIH00<005jHGYP00ejH00<006KiI_T0
0fKi00D006/HJaQ[600000McF007001gVGNIMiT007=H000026mJ0P001EZi00@005ZiF[T000UFF003
001oogoo00Ioo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo00Aoo`@000]o
o`03001Jfe[K00IJf`03000nOCim00<nO@040009^`Vk2K/200025Sd00`00<S`0000=0007CUP00`00
EUQFF007EUP01@00GYQNV5jH00001fKi00<006/H00001g=H00<007NIMiT00gNI00<006mJKeX01VmJ
00<006=L00001UZi00@005ZiF[T000QFF003001oogoo00Moo`03001oogoo00Qoo`004Goo00@007oo
Ool000=oo`03001oogoo009oo`@000eoo`03001Jfe[K00MJf`03000nOCim00<nO@050009^`Vk0000
eP040005<S`00`00@SY2>P04@SX00`00CUQ>F006CUP00`00EUQFF007EUP01000GYQNV0002FKi00<0
07=HLeP01G=H00<007NIMiT017NI0P001FmJ0P000f=L00<005ZiF[T015Zi00<005Zi000025IH00<0
07ooOol01goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P0000=oo`0000000`0047oo00<0
05[KF]/01U[K2@0000<9^`0000000`001S8l00<0048j@SX0148j00<004iHCUP01DiH00<005IHEUP0
1eIH00<005jH00002VKi00<007=HLeP01G=H00<007NIMiT00gNI00@007?KLm/000=_FP03001SG6=L
00=SG003001J^EZi00AJ^@04001J^EZi0008EUP00`00Oomoo`07Ool00`00Oomoo`07Ool0015oo`04
001oogoo0003Ool4000DOol00`00F]]Jf`02F]/300000eIi0012>@04@ST50000103F000000001C8l
00<0048j@SX0148j00<004iHCUP01TiH00<005IHEUP01UIH0P002VKi00<006mJ00001G=H00<007NI
MiT00gNI00<007?KLm/00W?K00<006mJ00001f=L00<005ZiF[T015Zi00<005Zi00001UIH0`002Goo
00<007ooOol01goo000BOol20003Ool2000HOol01000F]/000001EIi00<0048i00000P0000A2>@00
:MXYfP8000040=H000000005<S`00`00@SY2>P04@SX00`00CUQ>F005CUP00`00EUQFF005EUP00`00
G]X00009I_T01@00KeY_FVmJ00000g=H00<007NIMiT00gNI00<007?KLm/00g?K0P002F=L00<005Zi
F[T015Zi0P000UIH10000TiH00<007ooOol027oo00<007ooOol01Woo000GOol00`00Oomoo`0HOol0
0`00EWUFN@04EWT30004@ST00`00:MXYfP02:MX40005<S`00`00@SY2>P04@SX00`00CUQ>F006CUP0
0`00EUQFF002EUP20003G]X00`00I_UVn@05I_T00`00KeY_FP03KeX00`00LeP00005MiT00`00Lm]c
f`03Lm/00`00Fod00009He`00`00F[UJ^@02F[T60006CUP00`00Oomoo`09Ool00`00Oomoo`05Ool0
01Moo`03001oogoo01Qoo`04001FNEIiEWT30002EWT00`00@SU2>@03@ST02000:MXYfRWJ0001>@00
07@20003<S`00`00@SY2>P05@SX00`00CUQ>F005CUP00`00EUQFF0020005G]X00`00I_UVn@04I_T0
0`00KeY_FP05KeX20004MiT00`00Lm]cf`02Lm/20003Fod00`00HeaSG005He`60003CYX20008CUP0
0`00Oomoo`08Ool00`00Oomoo`05Ool001Moo`03001oogoo01Uoo`<000EFN@030012>D8i00=2>@03
000YfRWJ008YfP040001>@0007@400000c8l0012>P05@SX00`00CUQ>F005CUP00`00EUP00008G]X0
0`00I_UVn@02I_T00`00KeY_FP05KeX00`00LmYcfP0200000gNI001cf`02Lm/20006Fod00`00HeaS
G0040002Bah00`00CYY>VP05CYX00`00CUQ>F006CUP00`00Oomoo`09Ool00`00Oomoo`04Ool000Eo
o`8000Aoo`8000=oo`<000Aoo`8001Moo`@000QFN@030012>D8i00=2>@05000YfRWJ:MX000020CT0
0`0007@000022K/20007@SX00`00CUQ>F003CUP3000:G]X01000I_UVn@001fmJ00<007?JLmX00W?J
100000=cf`00Fod01E_m1@001d/N00<004jJCYX01DjJ00<004iHCUP01DiH00<007ooOol027oo0`00
1Goo0004Ool01000Oomoo`000goo0P001Woo00D007ooOomoo`0001Eoo`<000Aoo`03001FNEIi00EF
N@030012>D8i00=2>@03000YfRWJ008YfP060001>@4i0000M0000`Vk1@000T8j00<004iHCUP00`00
0TjK00<005kJG]X025kJ00<006Ki00001fmJ00<007?JLmX00W?J00@005_hFoQKn0T000D[?003001;
7T/N00A;7P03001>VTjJ00E>VP03001>F4iH00E>F003001oogoo00Eoo`<0009oo`03001oogoo00=o
o`0017oo00@007ooOol000]oo`03001oogoo008001=oo`8000Moo`03001FNEIi00EFN@X0008YfP03
0001>@4i00<00089^`03000bG39L008bG0D000I>V`03001NfUkJ00MNfP8000M_FP03001cfW?J009c
fP03001Kn5_h00L00003:_l0000000D0008[?003001;7T/N00A;7P03001>VTjJ00A>VP8000M>F004
001oogooOol30005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool2000@Ool3000:
Ool01000EWUFNEIi0`0000=B=`00BQP014XH00<003Wi>OT00`00104i0`0000<9^`00<U`01C9L00<0
04jKCY/01TjK00<005kJG]X01EkJ00@006MlIg`000E_FP04001cfW?JLmX20002FoP200000e[K0012
O003@W`400000`7k000000070003Bah00`00CYY>VP05CYX20006CUP40008Ool00`00Oomoo`03Ool0
00Aoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool3000=Ool01000EWT00000158g
00<004XHBQP014XH00<003Wi>OT00SWi0`000P4i1@001S9L00<004jKCY/01DjK00<005kJG]X015kJ
00<006MlIg`00VMl00D006mJKeY_FP0000=cfP03001Kn5_h008000AJf`030012O49l00=2O003000M
o00000D000040QX0029N8Uh40007CYX00`00CUP00005CUP00`00Oomoo`09Ool00`00Oomoo`03Ool0
00Eoo`8000Uoo`@0009oo`03001oogoo00Uoo`<0015oo`03001B=e8g00EB=`03001:64XH00=:6003
000inCWi008in@03000YfRWJ00D000030AL0000000DbG003001>VdjK00I>V`03001NfUkJ009NfP03
001WO6Ml00AWO003001_FP0000=cfP@000MJf`030012O49l00=2O004000Mo1gl7O`20000105i0000
0000129N100014jJ00<004iH00001DiH00<007ooOol02Woo00<007ooOol00Woo000FOol00`00Oomo
o`06Ool3000DOol00`00DSMB=`05DSL00`00BQQ:6004BQP00`00>OTin@02>OT00`00:MXYfP02:MX3
00000`4G00000005<U`00`00CY]>V`05CY/01@00G]YNfUkJ00001fMl200000=Vn@00F]/01U[K00<0
049l@W`0149l00<001gl7O`00Qgl0P0000<1N@000000129N00<003YL>U`0100000A>VP00CUP000E>
F003001oogoo00Uoo`03001oogoo009oo`005Goo0P001Goo0`0067oo00<0058gDSL0158g00<004XH
BQP014XH00<003Wi>OT00SWi00<002WJ:MX00bWJ00D000fl0000004G00<000<bG003001>VdjK00E>
V`03001NfP0000MWO004001Ko5_lFo`200000f]l001Vn@02I_T00`00F]]Jf`04F]/:00027O`00`00
0GX000020GT200038Uh00`00>U`jG003>U`:000;Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3
000KOol00`00DSMB=`05DSL00`00BQQ:6003BQP00`00>OTin@03>OT00`00:MXYfP03:MX00`003K`=
_00300000a8m000000020004CY/60005Ig`20002Fo`20002Jg`00`00I_UVn@03I_T01000F]]Jfe[K
0P0000=FN000AST01DHi00<003Gj00000`0000D1NP000000006K0080008RGP03000jG3YL00<jG004
0016>DHi0004AST00`00Oomoo`09Ool00`00Oomoo`02Ool001Eoo`@001ioo`03001B=e8g00EB=`03
001:64XH00A:6003000inCWi00<in@03000YfRWJ00<YfP04000=_0fl3K`200034Sd50006Bcl01@00
IgaWO6Ml00000e_l00D006]lJga[O00000IVn@04001Jf`000004EWP00`00ASU6>@04AST01000=OXe
nSGj0P000P5j0P000P6K0`001SYL00@004HiAST000A6>@03001oogoo00Qoo`03001oogoo009oo`00
3Woo0`0017oo0P008Goo00<0058gDSL0158g00<004XHBQP014XH00<003Wi>OT00cWi00<002WJ:MX0
12WJ00@000fl3K`=_08000@B?@03001;?d/o00M;?`03001WO00000=Ko005001_>@00Jg`00007I_T2
0006EWP00`00ASU6>@04AST00`00=OXenP03=OX200020GX010000I/1V`6K0P001CYL00D004HiASU6
>@0000=6>@03001oogoo00Qoo`03001oogoo009oo`003goo00<007ooOol01Goo00<007ooOol07Woo
00<0058gDSL0158g00<004XHBQP00dXH5`000Q8m00<004/oBcl01D/o1`000fli00<006KiI_T01FKi
00<005jh00001eIh00<004HiAST014Hi00<003Gj=OX00SGj00<001fj00000P5j0P000`6K0P0013YL
00D004HiASU6>@0000=6>@03001oogoo00Moo`03001oogoo009oo`003goo00<007ooOol01Woo00<0
07ooOol07Woo00<0058gDSL0158g00<004XH00000P000ThG00<004XHBQP00dXH00<004HjASX014Hj
00<003jM?Yd01CjM10000T/o1P000V]l00<006liKcT00fli00<006KiI_T00fKi0P000ejh00<005Ih
EWP015Ih00<004HiAST01DHi00<003Gj=OX00SGj00<001fj00000P0000H1>@000I/1V`007O`20002
>U`01@00ASU6>DHi00000dHi00<007ooOol027oo00<007ooOol00Goo000?Ool00`00Oomoo`07Ool0
0`00Oomoo`0MOol00`00DSMB=`03DSL30005CQL00`00BQQ:6004BQP00`00ASY6>P04ASX00`00?Ydn
W@04?Yd00`00F_]Jn`0300000eNo001[O004Jg`00`00KcU_>@03KcT00`00JaP00003I_T00`00G[QN
^003G[P00`00EWQFN005EWP00`00ASU6>@04AST00`00=OXenP03=OX00`007KXM^P0200001@4i0000
006K00000Qgl0P0000<jG000AST00THi00@004HiAST000Yoo`03001oogoo005oo`003goo00<007oo
Ool027oo0P007Goo00<0058gDSL00`0000=B=`00CQL01ThG00<004XHBQP00dXH00<004HjASX01DHj
00@003jM?YdnW@8000IJn`8000A[O003001_>Fli00=_>@07001[66/HJaP006Ki00001ejh00<005Ih
EWP015Ih00<004HiAST00THi1P000SGj00<001fj7KX00Qfj00<0004i0CT00`000agl0P0014Hi00@0
04HiAST000Uoo`03001oogoo005oo`003Woo0P0037oo00<007ooOol06goo0P001E8g00<004hGCQL0
14hG00<004XHBQP014XH00<004HjASX01DHj00<003jM00001e[k00<006LIIaT00P0000=[O000KcT0
16li00<006/HJaP00f/H00<005jhG[P01Ujh00<005IhEWP00eIh1P000dXG00<0047h@OP0100011fj
0P000P4i00<001gl7O`00agl1@000THi00<007ooOol01goo00<007ooOol00Goo000MOol00`00Oomo
o`0JOol00`00DSMB=`04DSL00`00CQM>5`04CQL00`00BQQ:6004BQP00`00ASY6>P05ASX20007F_/0
0`00IaUW6@03IaT20005KcT00`00JaQ[6003JaP00`00G[QN^006G[P00`00EWQFN0030002DSL00`00
BQM:5`05BQL00`00@OQ1n002@OP300027KX00`000GX000020CT200037O`00`00>U`jG002>U`40008
Ool00`00Oomoo`01Ool001ioo`03001oogoo01Yoo`03001B=e8g00=B=`03001>5dhG00E>5`03001:
64XH00A:60040016>THjASX30002EWT00`00F_]Jn`03F_/00`00IaUW6@04IaT00`00KaP00003KcT0
0`00JaQ[6003JaP00`00J_P00007G[P30006DSL00`00BQM:5`04BQL00`00@OQ1n003@OP00`00=OX0
000300001P5j000000000I/000<Mo003000jG3YL008jG003000jG00000Qoo`03001oogoo005oo`00
7goo0P006Woo00<0058gDSL0158g00<004hGCQL014hG00<004XHBQP014XH00@004Hj000000IFN@05
001Jne[kF_/00006IaT01`00KaQ_66lH001_>@0000E[6004001Zn6[h0006G[P00`00FWL00007DSL0
0`00BQM:5`04BQL00`00@OQ1n004@OP01000=OXenSGj0P000P5j0P0000D1V`00000Mo00000DjG003
000jG00000Moo`03001oogoo005oo`008Goo00<007ooOol05goo00<0058gDSL0158g00<004hGCQL0
14hG00<004XHBQP00dXH0`002EIi00<005[k00001VLI00<006lHKaP00flH00<006/HJaP00f/H00<0
06[hJ_P00V[h00<005jhG[P00Ujh00D005YgFWMJM`0000IB=`03001:5dXG00E:5`030011n47h00=1
n003000enSGj00<enP800081NP030001V`6K00<000@jG003000jG00000Qoo`40005oo`5oo`008Woo
00<007ooOol05goo00<0058gDSL00e8g00<004hGCQL01DhG00<004XH00000P000U9G00<005IiEWT0
1eIi0P001VLI00<006lHKaP016lH00<006/HJaP00f/H00<006[hJ_P00f[h00@005jhG[P000AJM`03
001B=e8g00EB=`03001:5dXG00A:5`030011n47h00A1n003000enSGj008enP03000N700000@00081
V`03000jG3YL00<jG003000jG00000Moo`40005oo`5oo`008goo00<007ooOol05Woo00<0058gDSL0
158g00<004hGCQL00dhG0`001U9G00<005IiEWT01EIi00<006:h00001FLI00<006lHKaP016lH00<0
06lH00000f/H00<006[hJ_P016[h00<005jh00001EYg00<0058gDSL01E8g00<004XGBQL014XG00<0
047h@OP0147h00<003Gj=OX00cGj00@001hL7Q`N70@000040I/000000003>U`20007Ool10001Ool1
Ool002Aoo`03001oogoo01Eoo`03001B=e8g00AB=`04001>5dhGCQL200000e8g001BE`06DUL00`00
EWUFN@04EWT01@00H[QR^6:h00000fLI00<006lHKaP016lH00L006lHKaQ_6000JaP00007J_P20007
FWL00`00DSMB=`04DSL00`00BQM:5`05BQL00`00@OQ1n003@OP00`00=OXenP04=OX00`007Q`N7003
7Q`400000aiN000000030007Ool10001Ool1Ool002Eoo`8001Ioo`03001B=e8g00=B=`04001>5`00
0004DSL00`00DUMBE`05DUL00`00EWUFN@02EWT00`00H[QR^003H[P00`00IaT00006KaP00`00KaQ_
6003KaP00`00J_QZn005J_P00`00H]P00007FWL00`00DSMB=`05DSL01000BQM:5dXG5P0000<N7000
00000P0011iN0P000goo10000Woo000WOol00`00Oomoo`0COol00`00DSMB=`03DSL20006DSL00`00
DUMBE`06DUL01000EWUFN@001f:h00<006lHKaP016lH00<006lHKaP016lH00<006[hJ_P016[h00D0
06;HH]QRf00000MJM`03001B=e8g00=B=`D001Ioo`P0008NGP@0008T0080009oo`00:7oo00<007oo
Ool04Woo00<0058gDSL00`0000=>=`00DSL01E8g00<0059GDUL01U9G00<005Ii000026:h0P001FlH
00<006lHKaP00flH00<006lh000016[h00<006;HH]P00V;H00<005YgFWL01EYg00@0058gDSMB=`80
029oo`T00003Ool007oo005oo`00:Goo00<007ooOol04Woo0P001Dhg00<0058gDSL0158g00<0059G
DUL01E9G0P0026:h00@006[hJ_P000=_6003001_66lH00=_6005001_>6lhKcP00002J_P00`00H]QR
f003H]P00`00FWMJM`05FWL01000DSL00000:7oo0`0017oo0@000Goo000ZOol2000BOol00`00CSM>
=`04CSL00`00DSMB=`04DSL00`00DUMBE`04DUL00`00GYP00007H[P00`00J_QZn002J_P00`00KaP0
0006KaP01`00KcQ_>6lh001Zn00000MRf003001JMeYg00AJM`8002]oo`03001oogoo00=oo`40005o
o`00;7oo00<007ooOol03goo00<004hgCSL014hg00<0058gDSL01E8g00@0059GDUMBE`8000=NV003
001R^6:h00AR^003001Zn6[h00=Zn003001_66lH00A_6003001_>6lh00=_>003001Rf6;H00IRf004
001JMeYgFWL3000^Ool00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00moo`03001>=dhg00A>
=`03001B=e8g00AB=`04001BEe9G0005GYP00`00H[QR^003H[P00`00J_QZn004J_P20004KaP00`00
KcQ_>004KcP00`00H]QRf006H]P01000FWL00000<Woo00D007ooOomoo`00005oo`00;Woo00<007oo
Ool03Woo00<004hgCSL014hg00<0058gDSL0158g00<0059G00001ejH00D006:hH[QR^00000IZn004
001_>6lh0003KaP00`00KcQ_>003KcP00`00KcT00007H]P2000dOol01@00Oomoogoo00000Goo000_
Ool2000>Ool00`00CSM>=`04CSL00`00DSMB=`04DSL20009GYP01000H[QR^0001F[h00<006lhKcP0
0Vlh00<006lH00001Flh00D006liKcU_>@0000ARf08003Moo`04001oogoo0001Ool0035oo`03001o
ogoo00aoo`03001>=dhg00A>=`03001B=e8g009B=`03001FF00000UNV003001R^00000EZn003001_
>6lh00A_>003001_>6lh00=_>003001_>Fli00=_>@04001Rf000000jOol00`00Ool00001Ool0039o
o`03001oogoo00]oo`03001>=dhg00A>=`03001B=e8g008000=FF003001NV5jH00INV08000EZn003
001_>6lh00E_>08000=_>003001_>Fli00A_>@8003aoo`03001oogoo00400000<goo00<007ooOol0
2Woo00<004hgCSL014hg00<0058g00001EIH00<005jHGYP01EjH00<006KH00001F[h00<006lhKcP0
16lh00H007=iLgT006lh0005KcT2000oOol10001Ool1000003Aoo`8000]oo`04001>=dhgCSL40007
EUP00`00GYQNV003GYP01@00I]QVf6KH00000f[h00<006lhKcP016lh00<007=iLgT00W=i0P0016li
00<007ooOol0@7oo0P00000fOol00`00Oomoo`08Ool40003CUP00`00EUQFF005EUP00`00GYQNV002
GYP00`00I]QVf003I]P00`00J_P00006KcP00`00LgUcN@02LgT01@00Lm]cf`00KcT00P00@Woo0`00
000gOol00`00Oomoo`07Ool00`00CUQ>F005CUP00`00EUQFF005EUP01000GYQNV0001fKH0P001Flh
00<007=iLgT00W=i00@007?KLm]cf`80041oo`@000=oo`00>7oo00<007ooOol01goo00<004iHCUP0
14iH00<005IHEUP01EIH00<005jH000026KH00<006mI00000flh00<007=iLgT00W=i00D007?KLm]c
f`0003ioo`@000Moo`00>Goo00<007ooOol01Woo00<004iHCUP014iH00<005IHEUP01EIH0P0026KH
00L006mIKeU_F@00KcP00004LgT00`00Lm]cf`02000lOol3000;Ool003Yoo`8000Moo`03001>F4iH
00A>F003001FF5IH009FF0800003G[T006KH00IVf003001_FFmI00=_F@<0009cN@04001cfg?K000j
Ool4000>Ool003aoo`03001oogoo00Aoo`03001>F4iH00A>F005001FF5IHEUP00004G[T00`00I]QV
f003I]P00`00KeU_F@04KeT00`00Lm]cf`05000gOol4000BOol003eoo`03001oogoo00=oo`03001>
F4iH00A>F004001FF0000006G[T00`00I]QVf002I]P00`00KeU_F@03KeT01000Lm]cf`000U?G00<0
07ooOol0<goo10005Woo000nOol00`00Oomoo`03Ool00`00CUQ>F003CUP20008G[T01@00I]QVf6KH
00001FmI00D007?KLm/005?G0080039oo`@001Yoo`00?goo0P000goo00<004iHCUP00`0000=>V`00
G[T025ji00<006KH00001FmI00H007?KLm/005?G000`Ool4000NOol0045oo`04001oogooOol20005
CY/00`00G[UN^@06G[T30004KeT00`00Lm]cf`03000^Ool3000ROol0049oo`04001oogoo0007CY/0
0`00G[UN^@03G[T20003Ie`2000016mI001cfg?K0P00;7oo10009Goo0013Ool01000Oomoo`001TjK
00<005jiG[T00Uji00<006MLIe`016ML1@00:Woo1000:Goo0014Ool200000goo001>V`05CY/01000
G[T000001FML0P0000=Gn`000000:7oo1000;Goo0016Ool00`00Ool00003CY/50005Ie`01000Eo/0
00009Woo1000<Goo0017Ool00`00Ool000020005Acl200000fML00000002000TOol4000eOol004Qo
o`04001oogoo0006Acl4000ROol4000iOol004Uoo`800004Ool004LoAcl400000goo001oo`0OOol3
000mOol004]oo`04001oo`000005Ool00`00Oomoo`0JOol40010Ool004aoo`03001oogoo00Eoo`03
001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo001>Ool00`00
Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0051oo`00DGoo
00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol200000goo00000002
001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02454 .26244 -12 -2.74006 ]
[.02454 .26244 0 6.25994 ]
[.0159 .31657 -24 -2.78582 ]
[.0159 .31657 0 6.21418 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00232 .43075 -18 -2.88287 ]
[-0.00232 .43075 0 6.11713 ]
[-0.01193 .49103 -6 -2.93439 ]
[-0.01193 .49103 0 6.06561 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03676 .25766 m
.04287 .25527 L
s
[(-1)] .02454 .26244 1 -0.3911 Mshowa
.02817 .3119 m
.0343 .30956 L
s
[(-0.5)] .0159 .31657 1 -0.38093 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01003 .42631 m
.01621 .4241 L
s
[(0.5)] -0.00232 .43075 1 -0.35936 Mshowa
.00046 .48671 m
.00666 .48456 L
s
[(1)] -0.01193 .49103 1 -0.34791 Mshowa
.125 Mabswid
.03507 .26836 m
.03873 .26693 L
s
.03336 .27913 m
.03703 .27771 L
s
.03164 .28997 m
.03531 .28856 L
s
.02991 .3009 m
.03358 .29949 L
s
.02641 .32298 m
.03009 .32158 L
s
.02464 .33413 m
.02833 .33274 L
s
.02286 .34537 m
.02655 .34399 L
s
.02107 .35668 m
.02476 .35531 L
s
.01744 .37956 m
.02114 .3782 L
s
.01561 .39112 m
.01931 .38977 L
s
.01377 .40277 m
.01746 .40142 L
s
.01191 .4145 m
.01561 .41316 L
s
.00815 .43822 m
.01185 .43689 L
s
.00625 .45021 m
.00996 .44889 L
s
.00433 .46229 m
.00805 .46098 L
s
.0024 .47445 m
.00612 .47315 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.201 0 0 r
.3771 .72791 .40169 .70185 .43949 .69119 .41587 .71254 Metetra
.41587 .71254 .43949 .69119 .47783 .68039 .45527 .6887 Metetra
.686 .979 .825 r
.45527 .6887 .47783 .68039 .51672 .66942 .49499 .65899 Metetra
.82 .869 .883 r
.49499 .65899 .51672 .66942 .55616 .6583 .53487 .62714 Metetra
.755 .704 .803 r
.53487 .62714 .55616 .6583 .59617 .64702 .57488 .59721 Metetra
.696 .609 .756 r
.57488 .59721 .59617 .64702 .63677 .63557 .61519 .57274 Metetra
.65 .555 .734 r
.61519 .57274 .63677 .63557 .67796 .62396 .65611 .55616 Metetra
.612 .524 .73 r
.65611 .55616 .67796 .62396 .71976 .61218 .69799 .54838 Metetra
.577 .513 .746 r
.69799 .54838 .71976 .61218 .76219 .60021 .74114 .54867 Metetra
.537 .525 .789 r
.74114 .54867 .76219 .60021 .80525 .58807 .78571 .55473 Metetra
.469 .579 .882 r
.78571 .55473 .80525 .58807 .84896 .57575 .83166 .56305 Metetra
.241 .659 .961 r
.83166 .56305 .84896 .57575 .89334 .56324 .87869 .56947 Metetra
.204 0 0 r
.87869 .56947 .89334 .56324 .9384 .55053 .92629 .57003 Metetra
.299 0 0 r
.92629 .57003 .9384 .55053 .98416 .53763 .97387 .56179 Metetra
.35165 .74588 .3771 .72791 .41587 .71254 .39135 .72644 Metetra
.675 .963 .747 r
.39135 .72644 .41587 .71254 .45527 .6887 .43186 .69152 Metetra
.89 .959 .837 r
.43186 .69152 .45527 .6887 .49499 .65899 .47258 .64615 Metetra
.871 .832 .805 r
.47258 .64615 .49499 .65899 .53487 .62714 .51312 .59735 Metetra
.808 .726 .773 r
.51312 .59735 .53487 .62714 .57488 .59721 .55341 .55246 Metetra
.741 .647 .755 r
.55341 .55246 .57488 .59721 .61519 .57274 .59372 .51768 Metetra
.672 .585 .749 r
.59372 .51768 .61519 .57274 .65611 .55616 .63455 .49716 Metetra
.599 .536 .755 r
.63455 .49716 .65611 .55616 .69799 .54838 .67652 .4924 Metetra
.513 .496 .776 r
.67652 .4924 .69799 .54838 .74114 .54867 .72019 .50222 Metetra
.399 .467 .814 r
.72019 .50222 .74114 .54867 .78571 .55473 .76589 .52272 Metetra
.22 .448 .867 r
.76589 .52272 .78571 .55473 .83166 .56305 .81356 .54778 Metetra
0 .412 .829 r
.81356 .54778 .83166 .56305 .87869 .56947 .86273 .5699 Metetra
.28 0 0 r
.86273 .5699 .87869 .56947 .92629 .57003 .91246 .58163 Metetra
.219 0 0 r
.91246 .58163 .92629 .57003 .97387 .56179 .96166 .57732 Metetra
.675 .963 .937 r
.3259 .74724 .35165 .74588 .39135 .72644 .36637 .72528 Metetra
.871 .933 .861 r
.36637 .72528 .39135 .72644 .43186 .69152 .40784 .68373 Metetra
.895 .859 .798 r
.40784 .68373 .43186 .69152 .47258 .64615 .44952 .62915 Metetra
.877 .798 .769 r
.44952 .62915 .47258 .64615 .51312 .59735 .49084 .57053 Metetra
.839 .747 .763 r
.49084 .57053 .51312 .59735 .55341 .55246 .53166 .51708 Metetra
.783 .699 .771 r
.53166 .51708 .55341 .55246 .59372 .51768 .57228 .47644 Metetra
.702 .646 .791 r
.57228 .47644 .59372 .51768 .63455 .49716 .61329 .45358 Metetra
.586 .583 .816 r
.61329 .45358 .63455 .49716 .67652 .4924 .65545 .45033 Metetra
.421 .502 .837 r
.65545 .45033 .67652 .4924 .72019 .50222 .69949 .46525 Metetra
.207 .406 .838 r
.69949 .46525 .72019 .50222 .76589 .52272 .74584 .49368 Metetra
0 .323 .81 r
.74584 .49368 .76589 .52272 .81356 .54778 .79453 .52811 Metetra
0 .31 .778 r
.79453 .52811 .81356 .54778 .86273 .5699 .84499 .55915 Metetra
0 .434 .796 r
.84499 .55915 .86273 .5699 .91246 .58163 .8961 .57726 Metetra
.19 .71 .873 r
.8961 .57726 .91246 .58163 .96166 .57732 .94646 .5751 Metetra
.738 .778 .885 r
.3005 .72663 .3259 .74724 .36637 .72528 .34142 .70427 Metetra
.823 .779 .811 r
.34142 .70427 .36637 .72528 .40784 .68373 .3835 .66216 Metetra
.853 .774 .773 r
.3835 .66216 .40784 .68373 .44952 .62915 .42588 .60692 Metetra
.862 .772 .761 r
.42588 .60692 .44952 .62915 .49084 .57053 .46792 .54762 Metetra
.853 .773 .773 r
.46792 .54762 .49084 .57053 .53166 .51708 .50943 .49355 Metetra
.821 .774 .809 r
.50943 .49355 .53166 .51708 .57228 .47644 .55063 .45237 Metetra
.739 .763 .872 r
.55063 .45237 .57228 .47644 .61329 .45358 .59211 .42908 Metetra
.554 .7 .937 r
.59211 .42908 .61329 .45358 .65545 .45033 .63464 .42553 Metetra
.248 .538 .923 r
.63464 .42553 .65545 .45033 .69949 .46525 .67895 .44025 Metetra
0 .356 .828 r
.67895 .44025 .69949 .46525 .74584 .49368 .72556 .46857 Metetra
0 .286 .779 r
.72556 .46857 .74584 .49368 .79453 .52811 .77451 .50294 Metetra
0 .354 .826 r
.77451 .50294 .79453 .52811 .84499 .55915 .8253 .53391 Metetra
.223 .529 .92 r
.8253 .53391 .84499 .55915 .8961 .57726 .87685 .55184 Metetra
.525 .705 .953 r
.87685 .55184 .8961 .57726 .94646 .5751 .92776 .54934 Metetra
.688 .638 .794 r
.27593 .68351 .3005 .72663 .34142 .70427 .31694 .66293 Metetra
.753 .669 .769 r
.31694 .66293 .34142 .70427 .3835 .66216 .35912 .62648 Metetra
.797 .704 .763 r
.35912 .62648 .3835 .66216 .42588 .60692 .40175 .57933 Metetra
.829 .748 .774 r
.40175 .57933 .42588 .60692 .46792 .54762 .44425 .5287 Metetra
.849 .806 .808 r
.44425 .5287 .46792 .54762 .50943 .49355 .48642 .48208 Metetra
.835 .88 .875 r
.48642 .48208 .50943 .49355 .55063 .45237 .52839 .44579 Metetra
.709 .932 .962 r
.52839 .44579 .55063 .45237 .59211 .42908 .57062 .42402 Metetra
.336 .797 .939 r
.57062 .42402 .59211 .42908 .63464 .42553 .61373 .41831 Metetra
0 .51 .789 r
.61373 .41831 .63464 .42553 .67895 .44025 .65836 .42743 Metetra
0 .37 .759 r
.65836 .42743 .67895 .44025 .72556 .46857 .70493 .44745 Metetra
0 .394 .84 r
.70493 .44745 .72556 .46857 .77451 .50294 .75355 .47213 Metetra
.219 .48 .891 r
.75355 .47213 .77451 .50294 .8253 .53391 .80382 .49382 Metetra
.439 .55 .871 r
.80382 .49382 .8253 .53391 .87685 .55184 .85493 .50486 Metetra
.587 .599 .832 r
.85493 .50486 .87685 .55184 .92776 .54934 .90581 .49947 Metetra
.649 .565 .747 r
.25234 .62239 .27593 .68351 .31694 .66293 .29309 .60526 Metetra
.687 .595 .747 r
.29309 .60526 .31694 .66293 .35912 .62648 .33483 .5793 Metetra
.727 .643 .765 r
.33483 .5793 .35912 .62648 .40175 .57933 .37715 .54723 Metetra
.769 .721 .808 r
.37715 .54723 .40175 .57933 .44425 .5287 .41972 .51294 Metetra
.804 .844 .881 r
.41972 .51294 .44425 .5287 .48642 .48208 .46237 .48062 Metetra
.731 .97 .932 r
.46237 .48062 .48642 .48208 .52839 .44579 .50516 .45393 Metetra
.336 .804 .712 r
.50516 .45393 .52839 .44579 .57062 .42402 .54835 .43537 Metetra
.034 0 0 r
.54835 .43537 .57062 .42402 .61373 .41831 .59229 .42589 Metetra
.163 0 0 r
.59229 .42589 .61373 .41831 .65836 .42743 .63735 .42471 Metetra
0 .528 .723 r
.63735 .42471 .65836 .42743 .70493 .44745 .68382 .42948 Metetra
.271 .644 .966 r
.68382 .42948 .70493 .44745 .75355 .47213 .73175 .43655 Metetra
.473 .6 .898 r
.73175 .43655 .75355 .47213 .80382 .49382 .78097 .44164 Metetra
.557 .561 .814 r
.78097 .44164 .80382 .49382 .85493 .50486 .83106 .44059 Metetra
.607 .553 .768 r
.83106 .44059 .85493 .50486 .90581 .49947 .88145 .43039 Metetra
.62 .529 .729 r
.22944 .55152 .25234 .62239 .29309 .60526 .26969 .53863 Metetra
.623 .541 .74 r
.26969 .53863 .29309 .60526 .33483 .5793 .31056 .52554 Metetra
.634 .582 .778 r
.31056 .52554 .33483 .5793 .37715 .54723 .35208 .51225 Metetra
.652 .674 .859 r
.35208 .51225 .37715 .54723 .41972 .51294 .39426 .49875 Metetra
.634 .844 .981 r
.39426 .49875 .41972 .51294 .46237 .48062 .43711 .48503 Metetra
.323 .809 .793 r
.43711 .48503 .46237 .48062 .50516 .45393 .48064 .47109 Metetra
.026 0 0 r
.48064 .47109 .50516 .45393 .54835 .43537 .52489 .45693 Metetra
.114 0 0 r
.52489 .45693 .54835 .43537 .59229 .42589 .56985 .44253 Metetra
.026 0 0 r
.56985 .44253 .59229 .42589 .63735 .42471 .61556 .4279 Metetra
.323 .809 .793 r
.61556 .4279 .63735 .42471 .68382 .42948 .66203 .41302 Metetra
.634 .844 .981 r
.66203 .41302 .68382 .42948 .73175 .43655 .70928 .39789 Metetra
.652 .674 .859 r
.70928 .39789 .73175 .43655 .78097 .44164 .75733 .38251 Metetra
.634 .582 .778 r
.75733 .38251 .78097 .44164 .83106 .44059 .80619 .36687 Metetra
.623 .541 .74 r
.80619 .36687 .83106 .44059 .88145 .43039 .8559 .35095 Metetra
.598 .516 .733 r
.20653 .48062 .22944 .55152 .26969 .53863 .24622 .47179 Metetra
.553 .5 .75 r
.24622 .47179 .26969 .53863 .31056 .52554 .28603 .47119 Metetra
.5 .511 .799 r
.28603 .47119 .31056 .52554 .35208 .51225 .32644 .47647 Metetra
.414 .567 .897 r
.32644 .47647 .35208 .51225 .39426 .49875 .36787 .48405 Metetra
.193 .64 .945 r
.36787 .48405 .39426 .49875 .43711 .48503 .41061 .48966 Metetra
.102 0 0 r
.41061 .48966 .43711 .48503 .48064 .47109 .45471 .48925 Metetra
.14 0 0 r
.45471 .48925 .48064 .47109 .52489 .45693 .49999 .47981 Metetra
.011 0 0 r
.49999 .47981 .52489 .45693 .56985 .44253 .5461 .46015 Metetra
.5461 .46015 .56985 .44253 .61556 .4279 .59267 .43124 Metetra
.719 .988 .852 r
.59267 .43124 .61556 .4279 .66203 .41302 .63941 .39593 Metetra
.821 .881 .888 r
.63941 .39593 .66203 .41302 .70928 .39789 .68626 .35829 Metetra
.759 .717 .813 r
.68626 .35829 .70928 .39789 .75733 .38251 .73339 .32268 Metetra
.694 .613 .762 r
.73339 .32268 .75733 .38251 .80619 .36687 .78123 .29289 Metetra
.643 .551 .737 r
.78123 .29289 .80619 .36687 .8559 .35095 .83034 .27149 Metetra
.581 .529 .761 r
.18266 .41871 .20653 .48062 .24622 .47179 .2219 .41293 Metetra
.464 .473 .783 r
.2219 .41293 .24622 .47179 .28603 .47119 .26075 .422 Metetra
.29 .416 .817 r
.26075 .422 .28603 .47119 .32644 .47647 .30005 .44198 Metetra
.027 .354 .829 r
.30005 .44198 .32644 .47647 .36787 .48405 .34063 .46662 Metetra
0 .299 .71 r
.34063 .46662 .36787 .48405 .41061 .48966 .38306 .48827 Metetra
.28 0 0 r
.38306 .48827 .41061 .48966 .45471 .48925 .4275 .49926 Metetra
.072 0 0 r
.4275 .49926 .45471 .48925 .49999 .47981 .47367 .49379 Metetra
.47367 .49379 .49999 .47981 .5461 .46015 .52092 .46954 Metetra
.713 .981 .783 r
.52092 .46954 .5461 .46015 .59267 .43124 .56851 .42841 Metetra
.888 .97 .833 r
.56851 .42841 .59267 .43124 .63941 .39593 .61585 .37596 Metetra
.884 .857 .81 r
.61585 .37596 .63941 .39593 .68626 .35829 .66276 .31988 Metetra
.821 .746 .781 r
.66276 .31988 .68626 .35829 .73339 .32268 .70952 .26808 Metetra
.746 .656 .762 r
.70952 .26808 .73339 .32268 .78123 .29289 .75684 .22715 Metetra
.668 .586 .754 r
.75684 .22715 .78123 .29289 .83034 .27149 .80564 .2014 Metetra
.568 .581 .827 r
.1568 .3725 .18266 .41871 .2219 .41293 .19589 .3682 Metetra
.331 .466 .846 r
.19589 .3682 .2219 .41293 .26075 .422 .23416 .38243 Metetra
0 .292 .79 r
.23416 .38243 .26075 .422 .30005 .44198 .27272 .41044 Metetra
0 .144 .663 r
.27272 .41044 .30005 .44198 .34063 .46662 .31266 .44461 Metetra
0 .144 .602 r
.31266 .44461 .34063 .46662 .38306 .48827 .35479 .47534 Metetra
0 .352 .703 r
.35479 .47534 .38306 .48827 .4275 .49926 .39943 .49282 Metetra
.247 .741 .911 r
.39943 .49282 .4275 .49926 .47367 .49379 .44626 .4895 Metetra
.7 .959 .946 r
.44626 .4895 .47367 .49379 .52092 .46954 .49449 .46231 Metetra
.871 .945 .861 r
.49449 .46231 .52092 .46954 .56851 .42841 .5431 .41378 Metetra
.908 .885 .799 r
.5431 .41378 .56851 .42841 .61585 .37596 .59131 .35117 Metetra
.895 .826 .771 r
.59131 .35117 .61585 .37596 .66276 .31988 .63879 .28437 Metetra
.856 .772 .768 r
.63879 .28437 .66276 .31988 .70952 .26808 .68584 .22329 Metetra
.792 .716 .779 r
.68584 .22329 .70952 .26808 .75684 .22715 .73328 .17599 Metetra
.701 .653 .799 r
.73328 .17599 .75684 .22715 .80564 .2014 .78223 .14761 Metetra
.54 .703 .945 r
.128 .34568 .1568 .3725 .19589 .3682 .16733 .34102 Metetra
.114 .484 .899 r
.16733 .34102 .19589 .3682 .23416 .38243 .20568 .35499 Metetra
0 .199 .689 r
.20568 .35499 .23416 .38243 .27272 .41044 .24421 .38283 Metetra
0 .096 .592 r
.24421 .38283 .27272 .41044 .31266 .44461 .28409 .41688 Metetra
0 .196 .685 r
.28409 .41688 .31266 .44461 .35479 .47534 .32625 .44746 Metetra
.079 .468 .889 r
.32625 .44746 .35479 .47534 .39943 .49282 .37107 .4647 Metetra
.505 .707 .961 r
.37107 .4647 .39943 .49282 .44626 .4895 .41832 .46097 Metetra
.738 .787 .893 r
.41832 .46097 .44626 .4895 .49449 .46231 .4672 .43318 Metetra
.833 .8 .82 r
.4672 .43318 .49449 .46231 .5431 .41378 .51666 .38387 Metetra
.871 .799 .777 r
.51666 .38387 .5431 .41378 .59131 .35117 .5658 .32041 Metetra
.881 .799 .764 r
.5658 .32041 .59131 .35117 .63879 .28437 .61425 .25274 Metetra
.87 .799 .777 r
.61425 .25274 .63879 .28437 .68584 .22329 .6622 .19084 Metetra
.831 .795 .818 r
.6622 .19084 .68584 .22329 .73328 .17599 .71044 .14281 Metetra
.739 .771 .879 r
.71044 .14281 .73328 .17599 .78223 .14761 .76008 .1138 Metetra
.401 .82 .97 r
.09556 .33859 .128 .34568 .16733 .34102 .13563 .33169 Metetra
0 .487 .761 r
.13563 .33169 .16733 .34102 .20568 .35499 .17489 .33989 Metetra
0 .265 .636 r
.17489 .33989 .20568 .35499 .24421 .38283 .21435 .3592 Metetra
0 .262 .714 r
.21435 .3592 .24421 .38283 .28409 .41688 .25504 .38327 Metetra
.044 .4 .855 r
.25504 .38327 .28409 .41688 .32625 .44746 .29779 .40427 Metetra
.366 .528 .885 r
.29779 .40427 .32625 .44746 .37107 .4647 .34297 .41437 Metetra
.569 .599 .844 r
.34297 .41437 .37107 .4647 .41832 .46097 .39044 .40761 Metetra
.685 .644 .803 r
.39044 .40761 .41832 .46097 .4672 .43318 .43956 .38161 Metetra
.759 .682 .778 r
.43956 .38161 .4672 .43318 .51666 .38387 .48948 .33834 Metetra
.809 .723 .77 r
.48948 .33834 .51666 .38387 .5658 .32041 .53942 .28352 Metetra
.844 .771 .781 r
.53942 .28352 .5658 .32041 .61425 .25274 .589 .22502 Metetra
.861 .829 .815 r
.589 .22502 .61425 .25274 .6622 .19084 .6383 .17091 Metetra
.838 .892 .879 r
.6383 .17091 .6622 .19084 .71044 .14281 .68792 .12787 Metetra
.722 .922 .96 r
.68792 .12787 .71044 .14281 .76008 .1138 .73875 .10022 Metetra
.05921 .34817 .09556 .33859 .13563 .33169 .1005 .33735 Metetra
.033 0 0 r
.1005 .33735 .13563 .33169 .17489 .33989 .14156 .33499 Metetra
0 .553 .754 r
.14156 .33499 .17489 .33989 .21435 .3592 .18302 .33867 Metetra
.208 .622 .951 r
.18302 .33867 .21435 .3592 .25504 .38327 .22552 .34467 Metetra
.422 .591 .913 r
.22552 .34467 .25504 .38327 .29779 .40427 .26957 .34862 Metetra
.529 .555 .826 r
.26957 .34862 .29779 .40427 .34297 .41437 .31541 .34626 Metetra
.592 .548 .774 r
.31541 .34626 .34297 .41437 .39044 .40761 .36299 .3345 Metetra
.64 .563 .752 r
.36299 .3345 .39044 .40761 .43956 .38161 .41196 .31211 Metetra
.684 .597 .752 r
.41196 .31211 .43956 .38161 .48948 .33834 .46187 .28008 Metetra
.729 .652 .773 r
.46187 .28008 .48948 .33834 .53942 .28352 .51225 .2414 Metetra
.775 .737 .818 r
.51225 .2414 .53942 .28352 .589 .22502 .56286 .20029 Metetra
.805 .855 .887 r
.56286 .20029 .589 .22502 .6383 .17091 .6137 .16126 Metetra
.747 .962 .937 r
.6137 .16126 .6383 .17091 .68792 .12787 .66509 .12818 Metetra
.502 .909 .857 r
.66509 .12818 .68792 .12787 .73875 .10022 .71754 .10368 Metetra
.01926 .36808 .05921 .34817 .1005 .33735 .06217 .35219 Metetra
.06217 .35219 .1005 .33735 .14156 .33499 .10583 .33602 Metetra
.486 .896 .917 r
.10583 .33602 .14156 .33499 .18302 .33867 .15025 .31957 Metetra
.647 .835 .973 r
.15025 .31957 .18302 .33867 .22552 .34467 .19545 .30283 Metetra
.648 .678 .865 r
.19545 .30283 .22552 .34467 .26957 .34862 .24146 .28579 Metetra
.626 .582 .785 r
.24146 .28579 .26957 .34862 .31541 .34626 .28829 .26844 Metetra
.612 .535 .744 r
.28829 .26844 .31541 .34626 .36299 .3345 .33597 .25079 Metetra
.607 .521 .732 r
.33597 .25079 .36299 .3345 .41196 .31211 .38452 .2328 Metetra
.612 .535 .744 r
.38452 .2328 .41196 .31211 .46187 .28008 .43397 .21449 Metetra
.626 .582 .785 r
.43397 .21449 .46187 .28008 .51225 .2414 .48434 .19583 Metetra
.648 .678 .865 r
.48434 .19583 .51225 .2414 .56286 .20029 .53566 .17683 Metetra
.647 .835 .973 r
.53566 .17683 .56286 .20029 .6137 .16126 .58795 .15746 Metetra
.486 .896 .917 r
.58795 .15746 .6137 .16126 .66509 .12818 .64124 .13773 Metetra
.64124 .13773 .66509 .12818 .71754 .10368 .69556 .11761 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`80025oo`03001oogoo00Aoo`03001oogoo00Yoo`03001oogoo01Yo
o`00AWoo00@007ooOol001Aoo`<002=oo`03001oogoo00Eoo`03001oogoo00Ioo`04001oogoo000L
Ool004Ioo`04001oogoo000BOol2000VOol00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001o
ogoo000?Ool300000goo001oo`0UOol00`00Oomoo`02Ool300000goo001oo`0VOol004Ioo`04001o
ogoo000=Ool2000[Ool00`00Ool000020003Gkh3000VOol004Moo`8000]oo`<002eoo`@000EO_P80
02Ioo`00DWoo0`00;Goo10001Unn00@005lL000002Eoo`00Cgoo0`00;Woo1@001Enn0P000elL0P00
97oo001<Ool3000^Ool80003Gkh20005Ga`2000TOol004Yoo`<002eoo`@0009Og@04001[W6^LJi`2
00000enn001O7007Ga`2000SOol004Moo`<002eoo`<00003?id00000009Og@03001[W6^L009[W003
001[W00000UO7003001oo`00029oo`00AGoo0P00;Goo0`000SnM0P000UoM0P001F^L0`002ElL0`00
8Woo0012Ool3000/Ool50002?id01@00GmeOgEoM00001f^L00@006/jJcX000IO70800004FYT007oo
000QOol002Eoo`@001Moo`8002aoo`<000ECG`@00003Gmd006^L00I[W08000=[>P03001O75lL009O
7003001JVEZI009JV@03001oo`00021oo`009Woo00<007ooOol057oo0`00:goo0`001e=O00<006Ml
Ig`00VMl00<006^LJi`01F^L00<006/j000016/j00@005lL000000EJV@<0021oo`007goo1@000goo
00<007ooOol04Goo0P00;7oo0P0025=O0P001FMl0P001F^L00@006/jJcX000E[>P03001JVEZI00IJ
V@03001oo`0001moo`00:7oo00<007ooOol03Goo0`00:goo0`000U:k00<005=ODel015=O00<006Ml
Ig`016Ml00@006mJKeX000=[W005001[>V/jJcX00004JcX20008FYT00`00Ool0000OOol002Eoo`04
001oogoo000=Ool2000[Ool30006D[/01000DemCGe=O0P001VMl00<006mJKeX00VmJ00<006^L0000
1F/j00@006/jJcX0009Vf003001JVEZI00IJV@03001oo`0001ioo`009Woo0P002goo0`00:goo0P00
2U:k00<005=O00001fMl00<006mJKeX016mJ00<006/jJcX016/j00<006/j00000fKH00<005ZIFYT0
1UZI00@007ooOol001eoo`00<Goo0P0000=oo`00Ool0:7oo0`0000=BF@00D[/02E:k0`001VMl00<0
06mJKeX01FmJ00<006/jJcX016/j0P0016KH00<005ZIFYT01UZI0P0000=oo`00Ool077oo000^Ool3
000ZOol30005DUT00`00D[]B^`06D[/01@00H_YRnV;j00001FMl00<006mJKeX016mJ00<006lh0000
1F/j00<006KHI]P016KH00<005ZIFYT00eZI0P0000=FF000Ool00P0077oo000[Ool3000[Ool20008
DUT00`00D[]B^`05D[/00`00H_YRnP03H_X01@00IgaWO6Ml00001VmJ00D006lhKcQ_>00000=[>P80
00IVf004001JVEZIFYT20004EUP01000Oomoo`001Goo0P0057oo000YOol20002Ool00`00Oomoo`0V
Ool300000dhg001BF@08DUT00`00D[]B^`03D[/00`00H_YRnP05H_X00`00Ig`00006KeX00`00KcQ_
>003KcP01@00JcX006KH00001VKH00@005ZI000000IFF004001oogoo0004Ool01000Oomoo`004goo
000VOol3000ZOol30005CSL00`00DUUBF@06DUT00`00D[]B^`02D[/00`00H_YRnP07H_X00`00KeY_
FP04KeX00`00KcQ_>005KcP01000I]QVf0001fKH00<005IHEUP01UIH00D007ooOomoo`0000=oo`04
001oogoo000COol002Aoo`8002]oo`8000Q>=`03001BFE9I00MBF@04001B^e:k000:H_X20005KeX0
0`00KcQ_>005KcP01000I]QVf0001VKH0P002EIH00D007ooOomoo`00009oo`04001oogoo000COol0
025oo`<002Yoo`<00003CQL004hg00M>=`03001BFE9I00MBF@03001B^`0000YRnP04001_66lH0003
KeX00`00KcQ_>005KcP01@00KaP006KH00001FKH00@005jHGYP000QFF003001oogoo0080009oo`04
001oogoo000COol001moo`8002Yoo`<000A>5`03001>=dhg00M>=`03001BFE9I00IBF@8000YRnP03
001_66lH009_6003001_FP0000M_>003001_66lH00<000AVf005001NV5jHGYP00008EUP00`00Oomo
o`02Ool00`00Oomoo`02000DOol001aoo`<002Yoo`8000Q>5`03001>=dhg00I>=`03001BFE9I00EB
F@03001NV00000URnP03001_66lH00A_6003001_>6lh00E_>003001_600000@000=Vf003001NV5jH
009NV003001FF5IH00MFF003001oogoo009oo`03001oogoo01Eoo`006goo0P00:Goo0`0000=>=`00
CQL01dhG00<004hgCSL01Thg00<0059IDUT0159I00D005jHGYQNV00000MRnP03001_66lH00E_6003
001_>6lh00A_>0<000A_60800003I]P005jH00ENV003001FF5IH00IFF003001oogoo009oo`03001o
ogoo01Eoo`0057oo0`0017oo00<007oo00009Woo0P001Dhg00<004hGCQL01DhG00<004hgCSL01dhg
00@0059IDUUBF@8000ENV003001RnV;j00=RnP03001_66lH00E_6003001c>000009_>0@000I_6003
001Vf00000MNV003001FF5IH00IFF003001oogoo00=oo`03001oogoo01Aoo`005Goo00<007ooOol0
0goo00@007ooOol002=oo`8000M>=`03001>5dhG00I>5`03001>=dhg00I>=`04001BFE9I0008GYP0
1@00H_YRnV;j00001flH00<007<hLcP00`000Vlh00<006lHKaP01VlH00<005jH00001UjH00<005IH
EUP01UIH0P0017oo0P005Goo000>Ool50002Ool00`00Oomoo`03Ool01@00Oomoogoo00007goo0`00
00=BF@00CSL01Thg00<004hGCQL01ThG00<004hgCSL01Thg00<0059I00002UjH00<006;j00001flH
10000W<h00<006lh000026lH0P0000=NV000GYP01UjH00<005IHEUP00eIH0P0000=B=`00Ool017oo
00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0IOol30005DUT0
0`00CSM>=`05CSL00`00CQM>5`05CQL00`00CSM>=`06CSL2000<GYP00`00KaQ_6004KaP30006LcP0
0`00KaQ_6006KaP01P00J_QZn000GYP000INV004001FF5IHEUP20003DSL00`00Oomoo`04Ool00`00
Oomoo`0BOol001Eoo`03001oogoo009oo`8000Eoo`03001oogoo01Eoo`<000QBF@03001>=dhg00E>
=`03001>5dhG00I>5`03001>=dhg00A>=`03001FN00000aNV004001_66lHKaP300000flH001c>006
LcP00`00KaQ_6005KaP01@00J_QZn6[h00000UjH00<005jHGYP00ejH00@005IH000000IB=`03001o
ogoo00=oo`03001oogoo019oo`0057oo0P0017oo00<007ooOol01Goo00<007ooOol04Goo0`000U:k
00<0059IDUT01U9I00<004hgCSL01Thg00<004hGCQL01DhG00<004hgCSL00Thg0P000eIh00<005jH
GYP025jH00<006Kh00000P000flH00<007<hLcP01G<h00<007=H00001FlH00<006[hJ_P00f[h00<0
05jH00001EjH0P00258g00<007ooOol017oo00<007ooOol04Goo000JOol00`00Oomoo`06Ool00`00
Oomoo`0=Ool30006D[/00`00DUUBF@06DUT00`00CSM>=`05CSL00`00CQM>5`05CQL00`00CSM>=`02
0005EWP00`00GYQNV007GYP50004KaP00`00LcQc>004LcP01000LeQcF00016lH00<006[hJ_P016[h
00@005jHGYP0009NV0800003FWP0058g00MB=`03001oogoo00Eoo`03001oogoo011oo`006Woo00<0
07ooOol01goo00<007ooOol02Goo0`0000=CG`00D[/01U:k00<0059IDUT01U9I00<004hgCSL01Dhg
00<004hGCQL01ThG0P0025Ih00<005jHGYP015jH0`000fKh00D006lHKaQ_600000Ic>003001cF7=H
009cF004001_66lH0008J_P00`00GYQNV0020004FWP00`00DSMB=`06DSL00`00Oomoo`03Ool2000B
Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000Yoo`03001oogoo00Ioo`8000ECG`03001B^e:k00EB
^`03001BFE9I00EBF@03001>=dhg00I>=`03001>5dhG00A>5`8000YFN003001NV5jH00@000IVn003
001_600000Mc>003001cF7=H009cF003001_600000UZn003001NV000008000AJN003001B=e8g00IB
=`03001oogoo00Eoo`03001oogoo00moo`001goo00@007ooOol000=oo`8000Ioo`03001oogoo0080
00]oo`03001oogoo00=oo`<000MCG`03001B^e:k00AB^`03001BFE9I00IBF@03001>=dhg00E>=`04
001>5dhGCQL300000e9H001FN009EWP30002GYP00`00I_QVn006I_P00`00LcQc>005LcP00`00LeQc
F004LeP00`00J_QZn008J_P20002FWP00`00FWQJN002FWP00`00DSMB=`06DSL00`00Oomoo`06Ool0
0`00Oomoo`0>Ool00003Ool0000000<00005Ool007ooOol0000;Ool01000Oomoo`003Goo00<007oo
00000P000cnM0P001U=O00<005:kD[/015:k00<0059IDUT01E9I00<004hgCSL01Dhg00@004hG0000
00EBF003001FN5Ih00AFN0<00005EWP005jHGYP00009I_P00`00LcQc>004LcP00`00LeQcF005LeP0
0`00J_QZn007J_P00`00H]X00003FWP00`00FWQJN002FWP00`00DSMB=`06DSL00`00Oomoo`05Ool0
0`00Oomoo`0>Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`0;Ool900000cnM001CG`04
Del00`00D[]B^`04D[/00`00DUUBF@05DUT00`00CSM>=`04CSL30007DUP01000EWQFN5Ih0`0015Ih
00<005jH00002FKh00<006lh00001G<h00<007=HLeP017=H00<007=i00001f[h00@006;JH]X000AJ
N005001JN5YhFWP00008DSL00`00Oomoo`06Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool0
0`00Oomoo`03Ool00`00Oomoo`07Ool60004<el:00000e=O001B^`05D[/00`00DUUBF@05DUT00`00
CSL000030002BSP00`00DUQBF006DUP30007EWP20009I_P01@00KcQ_>6lh00000g<h00<007=HLeP0
17=H00D007=iLgUcN@0000AZn08000ARfP05001JN5YhFWP00003FWP00`00DSMB=`06DSL00`00Oomo
o`07Ool00`00Oomoo`02Ool40006Ool000Qoo`8000Uoo`@0009oo`03001oogoo00Aoo`@000Aoo`D0
00=6gP03000=o00000<000036Wh0000000D00003D[/0059I00=BF@D000M:>003001BF59H00=BF0<0
00QFN003001R^00000QVn003001_>6lh00=_>003001c>00000IcF003001cNG=i009cN@05001Zn6[h
J_P00006H]X00`00FWQJN002FWP01@00FWQJN5Yh0000258g00<007ooOol01Woo00<007ooOol00goo
00<007ooOol01Woo000IOol01000Oomoogoo10002Woo00<004KNA]h00dKN00<000gl3O`00`000QYn
00<003IM=Ud00SIM1P000d8j00<004XhBSP01TXh00<0059HDUP00`000e9H00<005IhEWP015Ih00D0
06:hH[QR^00000IVn003001_>6lh00E_>003001cF7=H00AcF003001cNG=i00AcN@03001Zn00000MR
fP03001JN5Yh00=JN004001JN5Yh0007DSL20009Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qo
o`800004Ool00000000>Ool00`00A]i6gP03A]h010003O`=o0gl0`0000<JOP00=Ud013IM00<0048j
@SX01D8j00<004XhBSP01DXh0`001U9H00<005IhEWP00eIh00<006:hH[P00V:h00<006KhI_P016Kh
00<006lhKcP01Flh0P001G=H00<007=iLgT017=i0P002F;J00<005YhFWP00eYh00<005Yh0000158g
0`0000=>=`00Ool02Goo00<007ooOol00goo00<007ooOol017oo000HOol3000BOol00`00A]i6gP03
A]h010003O`=o0gl1@001CIM00<0048j@SX0148j00<004XhBSP00TXh0`0000=:>000DUP01U9H00<0
05IhEWP00UIh00<006:hH[P016:h00<006KhI_P00VKh00<006lhKcP01Flh00@007NIMiT000=cF003
001cNG=i00AcN@03001[W00000URfP03001JN5Yh00=JN003001JN5Yh00@000A>=`03001oogoo00Mo
o`80009oo`04001oogoo0006Ool0019oo`8000Aoo`03001oogoo01=oo`T000033O`0000000<000@f
G@030012>T8j00E2>P@000A:>003001BF59H00IBF004001FN5Ih0008H[P01000I_QVn0001flh00<0
07NIMiT00WNI00<007=H00001W=i00D006^LJia[W00000QRfP03001JN5Yh00AJN0<000Q>=`03001o
ogoo00Qoo`03001oogoo008000Moo`004Goo00@007ooOol000=oo`03001oogoo01=oo`03001:FTYJ
00=:FP03000YfbWK00<000040=H000000004=Ud00`00@SY2>P02@SX30007BSP00`00DUQBF006DUP0
0`00EWP00009H[P00`00I_P00007KcP00`00MiUgV@04MiT20005LgT00`00Jia[W003Ji`00`00H]YR
fP06H]X00`00FWP0000300000e9H00000008CSL00`00Oomoo`09Ool00`00Oomoo`08Ool0015oo`04
001oogoo0003Ool00`00Oomoo`0DOol00`00BUY:FP03BUX00`00:M/Yf`02:M/60002=Ud00`00@SX0
00020003@SX00`00BSQ:>005BSP00`00DUQBF005DUP2000;H[P00`00KcQ_>005KcP00`00MiUgV@04
MiT01000LoYcnP000g=i00<006^LJi`016^L00<006;JH]X016;J10001U9H0P001dhg00<007ooOol0
2Goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P005Woo00<004YJBUX00dYJ00<002WK:M/0
0RWK00<0004i00000P0000<5V`0000000P001T8j00<004XhBSP01DXh00<0059HDUP00e9H0P0000=F
V@00H[P02F:h0P001flh00<007NIMiT00gNI00<007?jLoX00W?j00<007=i000026^L00<006;JH]X0
0`000U:k00<0059HDUP01E9H0P0024hg00<007ooOol02Goo00<007ooOol01goo000AOol01000Oomo
o`000goo00<007ooOol05Goo00<004YJBUX00dYJ00<002WK:M/00RWK00<0004i00001P001T8j00<0
04XhBSP01DXh00@0059HDUQBF08000AFV@03001R^6:h00IR^004001[>V/j0005KcP00`00MiUgV@03
MiT00`00LoYcnP03LoX20009Ji`30006D[/00`00DUQBF005DUP00`00CSM>=`06CSL00`00Oomoo`09
Ool00`00Oomoo`07Ool0019oo`8000=oo`8001Moo`03001:FTYJ00A:FP03000YfbWK008Yf`D000<5
V`<000A2>P03001:>4Xh00E:>0<000IFV@03001R^6:h00ER^003001[>V/j009[>P05001_>6lhKcP0
0005MiT00`00LoYcnP03LoX01000Go]On`001F^L0`0000=CO`00D[/01U:k00<0059HDUP01U9H00<0
04hgCSL01Dhg00<007ooOol02Woo00<007ooOol01Woo000GOol00`00Oomoo`0GOol00`00BUY:FP03
BUX00`00:M/0000300030CT200021I/00`00;Q`^70050004BSP40009EYT00`00H[QR^003H[P00`00
JcY[>P04JcX00`00KcP00005MiT00`00LoYcnP03LoX00`00Go]On`02Go/00`00Ji`000020005Dgl0
0`00D[]B^`05D[/00`00DUQBF005DUP00`00CSM>=`06CSL00`00Oomoo`0:Ool00`00Oomoo`05Ool0
01Moo`03001oogoo01Moo`04001:FTYJBUX700000bWK0001>@020CT300000`FK000^7004;Q`50003
BW/00`00EYUFV@07EYT00`00H[QR^002H[P00`00JcY[>P06JcX20004MiT00`00LoYcnP02LoX20004
Go/40007Dgl00`00D[]B^`05D[/00`00DUQBF005DUP20007CSL00`00Oomoo`0:Ool00`00Oomoo`05
Ool001Moo`03001oogoo01Qoo`@0009:6004000inCWi>OT400020CT300000`FK000^7004;Q`00`00
BW]:N`05BW/00`00EYUFV@07EYT01000H[QR^0002F/j00H006oK000007NI0004LoX01000Go]Oneok
0`0012/i0P001U=o00<005:kD[/015:k00<0059HDUP01U9H0P001Thg00<007ooOol02goo00<007oo
Ool017oo0005Ool20004Ool20003Ool30004Ool2000GOol30006BQP00`00>OTin@02>OT00`009KX0
00070005;Q`00`00BW]:N`04BW/00`00EYUFV@07EYT00`00H[P00009JcX00`00Km]_f`02Km/30002
LoXA0003Dgl00`00D[]B^`05D[/00`00DUQBF005DUP00`00CSL00006CSL00`00Oomoo`09Ool30005
Ool000Aoo`04001oogoo0003Ool20006Ool01@00Oomoogoo00005Goo0`000goo00<004XHBQP00dXH
00<003Wi>OT00cWi00@002Fj9KXU^P<000030=D002hL00@^7003001:NdYk00E:N`03001FVEJI00IF
V@8000U[>P03001_ffoK009_f`04001KnE_iFoT40004A]d00`007SdN?@0700000b:N000000020007
D[/00`00DUQBF005DUP00`00CSL00002CSL50008Ool30002Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo000;Ool00`00Oomoo`02000COol20006Ool00`00BQQ:6004BQP00`00>OTin@02>OT00`009KXU
^P039KX200000`3E000^7004;Q`00`00BW]:N`04BW/00`00EYUFV@04EYT20002Gc`00`00JcY[>P05
JcX00`00Km]_f`02Km/00`00FoUKn@030002Ga/00`00A]e6g@03A]d00`007SdN?@027Sd500038YhE
0004CSP00`00Oomoo`03Ool30005Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool30003Ool0
0`00Oomoo`0?Ool30008Ool00`00BQQ:6004BQP00`00>OTin@03>OT00`009KXU^P029KX00`000GX0
00030003;Q`00`00BW]:N`05BW/01000EYUFVEJI0P001Ell00<006/jJcX00f/j00<006oKKm/00VoK
00<005_iFoT00P001ElK00<004KMA]d00dKM00<001hm7Sd00Qhm00@0005j0001N080008RWP03000n
O3il00DnO0030016>THj00E6>P04001>>4hh0005CSP50008Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo0008Ool00`00Oomoo`03Ool2000=Ool3000<Ool00`00BQQ:6003BQP00`00>OTin@03>OT00`00
9KXU^P039KX01@000GX000002M`00`0000<^7000BW/01DYk00@005JI000000QO?005001[>V/jJcX0
0004Km/01000FoT000001elK2P000Qhm00D0005j0000006J008000038Yh003il00HnO0030016>THj
00E6>P04001>>4hh0004CSP00`00Oomoo`0:Ool00`00Oomoo`03Ool000Eoo`8000Uoo`@0009oo`03
001oogoo00Uoo`<000moo`03001:64XH00A:6003000inCWi00<in@03000U^RFj00<U^P040001NP5j
00032M`20004BW/4000;Gc`00`00JcX00004Km/40006Ga/200000eZI001:FP04BUX00`00=QXf6P04
00020GX00`000IX1VP030006?W`00`00ASY6>P05ASX01000CSQ>>00014hh00<007ooOol02goo00<0
07ooOol00Woo000FOol00`00Oomoo`06Ool3000COol00`00BQQ:6003BQP00`00>OTin@03>OT00`00
9KXU^P039KX01@000GX1NP5j000010WL100014;N00<005llGc`025ll10000VoK0`000V/I00<005lK
Ga/00UlK00<005ZIFYT00UZI00<004YJBUX00dYJ00<003HJ=QX00SHJ0`0000<1NP000IX00P6J0P00
1Sil00<004HjASX014Hj00D004hhCSQ>>00000A>>003001oogoo00Yoo`03001oogoo009oo`005Woo
00<007ooOol00goo0`005Woo00<004XHBQP00dXH300000<U^P000GX00P5j00D000WL2M`9g00000Q2
gP03001O?5ll00EO?08000AKgP800003Ji/006/I00=[6@04001O6`000005FYT00`00BUY:FP04BUX0
0`00=QXf6P02=QX010006IX0005j0P000`6J0P0013il00<004HjASX01DHj00D004hhCSQ>>00000=>
>003001oogoo00Yoo`03001oogoo009oo`005Goo00@007ooOomoo`<001Uoo`03001:64XH00<00003
CQL004Gg00A5m`03000n6ChI008n6@P00089g0030012gT;N00I2gP03001O?5ll009O?08000AKgP80
009[V`03001[6F/I00A[6@03001JVEZI00IJV@03001:FTYJ00=:FP03000f6SHJ00<f6P03000IVP00
00<00081VP04000Mo0000002?W`00`00ASY6>P05ASX01@00CSQ>>4hh00000dhh00<007ooOol02Woo
00<007ooOol00Woo000EOol4000MOol20005CQL00`00AOM5m`03AOL00`00?QTn6@03?QT01000=U`f
G3IL1P0024;N00@005ll000000AKgP8000=[V`03001[6F/I00A[6@8000QJV@03001:FTYJ00A:FP03
000f6SHJ00<f6P03000IVQVJ00@000040IX001gl7O`200000cil0016>P06ASX01@00CSQ>>4hh0000
0dhh00<007ooOol02Goo00<007ooOol00Woo000>Ool30004Ool2000OOol00`00CQM>5`04CQL00`00
AOM5m`03AOL00`00?QTn6@04?QT00`00=U`fG004=U`30004@]h70002Fmh200001Fli000006^K0000
1V/I00@006KHI]P000MJV@03001:FTYJ009:FPH0008f6P03000IVQVJ008IVP030001>000008000<M
o08000M6>P03001>>4hh009>>004001>>4hh000;Ool00`00Oomoo`02Ool000moo`03001oogoo00Eo
o`03001oogoo01aoo`03001>5dhG00A>5`030015mdGg00A5m`03000n6ChI00@n6@03000fG3IL00Df
G0D000DNo@D000A_>@8000E[6@05001Vf6KHI]P00007FYT50003CSP00`00@OQ1n00400046IX20000
0`4h000Mo0047O`50002ASX00`00CSQ>>003CSP00`00CSP0000;Ool00`00Oomoo`02Ool000moo`03
001oogoo00Ioo`03001oogoo01aoo`03001>5dhG00A>5`030015mdGg00=5m`03000n6ChI00@n6@03
000fG3IL008fG0<000=B^`03001CO`0000@000=WO003001_>Fli00=_>@03001_600000=[6@03001V
f6KH00=Vf003001JVEZI009JV@800003EUL004hh00I>>0030011n47h0091n0<0008IVP040001F@00
0CP200037O`00`00>S/j>`02>S/<000;Ool00`00Oomoo`01Ool000moo`03001oogoo00Moo`03001o
ogoo01]oo`03001>5dhG00A>5`030015mdGg00=5m`03000n6ChI00Dn6@04000fG0000007D[/00`00
Dgl00005Ig`00`00KcU_>@03KcT01`00KaQ_66lH001[6@0000IVf005001JVEZIFYT00003EUL00`00
CSQ>>005CSP00`00@OQ1n003@OP00`00<MX0000300001@5I0000005j00000agl00<003Xk>S/013Xk
00<004YjBWX00dYj00<004Yj00002goo00<007ooOol00Goo000?Ool00`00Oomoo`08Ool2000KOol0
0`00CQM>5`04CQL00`00AOM5m`04AOL00`00?QTn6@03?QT30009D[/30004Ig`00`00KcU_>@03KcT0
0`00KaQ_6003KaP00`00I]QVf005I]P01000FYT000001EIG00<004hhCSP014hh00<0047h@OP0147h
00@0037J<MXafP800081F@060001NP00000Mo0001cXk00<004YjBWX00dYj00<004Yj00002Woo00<0
07ooOol00Goo000>Ool2000<Ool00`00Oomoo`0IOol00`00CQM>5`04CQL00`00AOM5m`03AOL00`00
?QTn6@030002DUP00`00D[]B^`06D[/01000H_URnF;i0P0000=WO000KcT016li00<006lHKaP016lH
00<006KHI]P01FKH0P001eIG00<004hhCSP01Dhh00<0047h@OP00d7h00<0037J<MX00c7J0P0000@1
F@000GX1NP<000Hj>`03001:NTYj00=:NP03001:NP0000Yoo`03001oogoo005oo`007Goo00<007oo
Ool067oo00<004hGCQL014hG00<004GgAOL00dGg0`001U9H00<005:kD[/00e:k0P001V;i00<006li
KcT00fli00<006lHKaP016lH00<006[h00001FKH00<005jH00001eIG00<004hhCSP01Dhh00<0047h
@OP0147h00<0037J<MX00S7J00<001gK00000`000P5j00<003Xk>S/013Xk00<004YjBWX014Yj00<0
04Yj00002Goo00<007ooOol00Goo000NOol00`00Oomoo`0GOol00`00CQM>5`04CQL00`00AOM5m`03
00000dhg001BF007DUP01@00D[]B^e:k000026;i0P000fli00<006lHKaP016lH00@006[hJ_P000AV
f005001NV5jHGYP00007EUL00`00CSQ>>004CSP00`00@OQ1n004@OP00`00<MXafP03<MX00`007M/M
f`0300020GX20005>S/00`00BWY:NP04BWX20009Ool00`00Oomoo`01Ool001moo`8001Moo`03001>
5dhG00=>5`@000E>=`03001BF59H00IBF003001B^`0000QRn@06001Zn6[h001_>@001VlH00<006[h
J_P00V[h00@006KHI]P000ANV003001FEeIG00EFE`03001>>4hh00E>>0030011n47h00=1n003000a
fS7J00@afP03000MfagK00D000047Qd000000002>S/00`00BWY:NP05BWX20006Ool30003Ool0025o
o`03001oogoo01Eoo`03001>5`000080009>5`03001>=dhg00A>=`03001BF59H00IBF08000QRn@03
001Zn6[h009Zn003001_66lH00A_6003001Zn6[h00=Zn003001Vf00000INV003001FEeIG00AFE`03
001>>4hh00=>>0h00003<MX001gK00<Mf`@000<N7@<000M:NP8000=oo`<0008T0080009oo`008Woo
00<007ooOol057oo0P001ThG00<004hgCSL014hg00<0059HDUP0159H00<005jH00001f;i00<006[h
J_P00f[h0P001FlH00<006[hJ_P016[h00<005jHGYP01EjH00<005IGEUL01EIG00<004hh00000P00
0ThG00<004XGBQL014XG00<004HIAQT00THI1@000QgK10001AhM30001B@00P000Woo000SOol00`00
Oomoo`0COol00`00CQM>5`05CQL00`00CSM>=`04CSL00`00DUQBF002DUP20003GYP00`00H_URn@03
H_T00`00J_QZn003J_P01000KaQ_60000flH00<006[hJ_P01F[h00<005jHGYP01UjH00<005IGEUL0
0eIG0`001DhG00<004XGBQL01DXG00<004HIAQT00dHI00<003iL?U`0100000@1^P00000000<N7@03
0012gT;N00I2gP8000DT0003001oo`00009oo`0097oo00<007ooOol04goo00<004hGCQL014hG00<0
04hgCSL01Dhg00@0059HDUP000INV005001RnF;iH_T00005J_P00`00KaQ_6002KaP00`00KaP00007
J_P00`00I_P00007GYP00`00EUMFE`0300000e8g001>5`06CQL00`00BQM:5`04BQL00`00AQU66@04
AQT00`00?U`nG002?U`800000ahM0012gP07@]h00`009000000400000goo001oo`01Ool002Eoo`80
01=oo`03001>5dhG00E>5`03001>=dhg00A>=`03001BF00000QNV003001Rn@0000IZn003001_66lH
00=_6003001Zn6[h00EZn004001Vn6Kh0007GYP30005DSL00`00CQM>5`04CQL00`00BQM:5`04BQL0
0`00AQU66@04AQT00`00?U`nG005?U`00`007[hN_P050007@]h2000370001000Oomoo`000Woo000W
Ool00`00Oomoo`0@Ool00`00CQM>5`05CQL00`00CSM>=`04CSL2000:GYP00`00J_QZn004J_P00`00
KaQ_6004KaP20005J_P00`00I_QVn002I_P00`00GYQNV002GYP200000eYh001B=`05DSL00`00CQM>
5`04CQL00`00BQM:5`05BQL00`00AQU66@04AQT00`00?U`nG003?U`40003Ool>0003Ool10001Ool1
Ool002Qoo`03001oogoo011oo`03001>5dhG00E>5`03001>=dhg009>=`03001FM`0000YNV003001Z
n6[h00AZn003001_66lH00=_6004001c>7<h0003J_P00`00I_QVn003I_P01@00GYQNV5jH00000eYh
00<0058gDSL0158g00<004hGCQL01DhG00<004XGBQL014XG00<004HIAQT014HI00<003iL?U`00`00
2goo1@000Woo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0?Ool00`00CQM>5`05CQL0
0`00CSM>=`020003EWL00`00GYQNV006GYP00`00I]P00004J_P00`00KaQ_6003KaP00`00LcQc>002
LcP00`00J_P00006I_P01000GYP000001EYh00<0058gDSL0158g00<004hGCQL014hG00<004XGBQL0
14XG00<004HIAQT00dHI10005Goo00<007ooOol017oo0@000Goo000ZOol2000?Ool00`00CQM>5`05
CQL00`00CSL00005EWL00`00GYQNV005GYP01@00I]QVf6KH00000V[h00<006lHKaP00flH00<007<h
LcP017<h00<006KhI_P01FKh0P001eYh00<0058gDSL0158g00<004hGCQL014hG00<004XGBQL01DXG
00<004HI00000P006Woo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0=Ool00`00CQM>5`04
CQL20007EWL00`00GYQNV003GYP00`00I]QVf003I]P00`00J_P00004KaP00`00LcQc>005LcP00`00
I_QVn004I_P00`00H]T00007FWP00`00DSMB=`05DSL00`00CQM>5`03CQL00`00BQM:5`03BQL4000N
Ool00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00aoo`04001>5dhGCQL300000e9G001FM`06
EWL00`00GYQNV002GYP00`00I]QVf005I]P00`00KaQ_6003KaP00`00LcQc>004LcP00`00KeT00004
I_P01@00H]URfF;I00001eYh00<0058gDSL0158g00<004hGCQL014hG00<004XG00000P008goo00D0
07ooOomoo`00005oo`00;Woo00<007ooOol02goo00@004hG000000ABE`03001FMeIg00IFM`04001N
V5jH0008I]P20003KaP00`00LcQc>004LcP01@00KeU_FFmI00000VKh00<006;IH]T00V;I00<005Yh
FWP01EYh00<0058gDSL0158g00@004hGCQM>5`D002Ioo`05001oogooOol00001Ool002moo`8000ao
o`03001BEe9G00EBE`03001FMeIg00EFM`03001NV00000QVf006001_66lH001_60001W<h00<006mI
KeT00VmI00<006Kh00001V;I00<005YhFWP015Yh00<0058gDSL00e8g1@00;7oo00@007ooOol0005o
o`00<Goo00<007ooOol02Goo00<0059GDUL01E9G00<005IgEWL01EIg0P0026KH00<006lHKaP00VlH
0P001G<h00<006mIKeT00fmI0P001f;I00<005YhFWP01EYh00<0058g00000P00<Woo00<007oo0000
0Goo000bOol00`00Oomoo`09Ool00`00DUMBE`04DUL00`00EWMFM`04EWL00`00H[P00007I]P00`00
KaQ_6003KaP00`00LgT00003LcP00`00KeU_F@03KeT00`00Jg`00008H]T00`00FWQJN003FWP3000e
Ool00`00Oomoo`01000003=oo`03001oogoo00Qoo`03001BEe9G00EBE`03001FMeIg009FM`05001R
^6:hH[P00005I]P00`00KaQ_6003KaP01`00LgUcNG=i001c>00000E_F@05001[O6]lJg`00007H]T0
0`00FWQJN003000iOol10001Ool1000003Aoo`8000Qoo`03001BEe9G00EBE`05001FMeIgEWL00005
H[P01@00I]QVf6KH00001FlH00<007=iLgT00g=i0P0016mI00<006]lJg`00f]l00<006;IH]T00f;I
1000?Goo0P00000fOol00`00Oomoo`06Ool00`00DUMBE`04DUL01000EWMFM`001f:h00<006KH0000
1VlH00<007=iLgT00g=i00H007?J000006mI0006Jg`00`00H]URf@030010Ool3000003Moo`03001o
ogoo00Eoo`03001BEe9G00EBE`8000UR^003001_66lH00A_6003001cNG=i00=cN@04001cfW?JLmX3
0006Jg`3000oOol40003Ool003Qoo`03001oogoo00Eoo`03001BEe9G00=BE`8000UR^003001[6@00
00A_6003001cNG=i00=cN@05001cfW?JLmX00003EoX00`00Jga[O003000nOol40007Ool003Uoo`03
001oogoo00Aoo`04001BEe9GDUL200000eZI001R^007H[P01@00JaU[6F/I00000flH00<007=iLgT0
0W=i00<007?JLmX00P000UOj1@00?Woo0`002goo000jOol20004Ool01000DUL0000015ZI00<006:h
H[P016:h00<006/IJaT00f/I00<006lH000017=i00<007?JLmX01@00?goo10003Woo000lOol00`00
Oomoo`02Ool00`00FYUJV@05FYT00`00H[QR^002H[P00`00JaU[6@05JaT30002LgT00`00LmYcfP02
000oOol4000BOol003eoo`05001oogooOol00007FYT01@00H[QR^6:h00001f/I00@006nkKk]_^`D0
03eoo`@001Ioo`00?Woo00D007ooOomoo`0000MJV@03001R^00000M[6@06001_^fnkKk/005OG0P00
>goo10006Woo000oOol200000goo001JV@06FYT20007JaT01000Kk]_^fnk0`00>Goo10007Woo0011
Ool00`00Ool00005FYT01000GaaO70001F/I00@006nkKk]_^`8003Qoo`<0029oo`00@Woo00<007oo
00000UZI0P0015lL00D006/IJaU[6@0000=_^`03001oogoo03Aoo`@002Eoo`00@goo0P0000=JV@00
Ga`01UlL00<006/I00000fnk00<007ooOol0<Goo1000:Goo0014Ool30007Ga`7000_Ool4000]Ool0
04Ioo`8000AO7080009Gg@80009oo`03001oogoo02Uoo`@0035oo`00Agoo0P0000=O700000000UOM
0P0017oo00<007ooOol09Goo1000=Goo0018Ool60006Ool00`00Oomoo`0QOol4000iOol004Uoo`80
00Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007ooOol01Woo00<007ooOol06Woo1000@7oo001<
Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool004eoo`03001oogoo00Aoo`03001oogoo019o
o`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo001?Ool20003Ool00`00Oomo
o`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03001oo`0000Eoo`@005Moo`00
Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo002e
Ool00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128621, 0.0056064, 
  0.00599011, 0.00599011}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.183 0 0 r
.37846 .69526 .40273 .67205 .44005 .66841 .41684 .68118 Metetra
.41684 .68118 .44005 .66841 .47798 .66923 .45574 .66493 Metetra
.669 .926 .973 r
.45574 .66493 .47798 .66923 .51671 .67229 .49509 .64751 Metetra
.703 .729 .87 r
.49509 .64751 .51671 .67229 .55631 .67485 .53487 .63015 Metetra
.668 .611 .783 r
.53487 .63015 .55631 .67485 .59673 .67414 .57512 .61402 Metetra
.643 .556 .742 r
.57512 .61402 .59673 .67414 .63782 .66795 .61592 .60002 Metetra
.627 .533 .728 r
.61592 .60002 .63782 .66795 .67935 .65515 .65737 .5886 Metetra
.617 .531 .734 r
.65737 .5886 .67935 .65515 .72111 .63592 .6996 .57967 Metetra
.613 .553 .763 r
.6996 .57967 .72111 .63592 .76299 .61176 .74267 .57263 Metetra
.61 .616 .832 r
.74267 .57263 .76299 .61176 .80501 .58513 .78662 .56646 Metetra
.574 .763 .968 r
.78662 .56646 .80501 .58513 .84736 .5589 .83138 .55992 Metetra
.192 .723 .772 r
.83138 .55992 .84736 .5589 .89036 .5358 .87684 .55177 Metetra
.201 0 0 r
.87684 .55177 .89036 .5358 .9344 .51788 .92286 .54104 Metetra
.272 0 0 r
.92286 .54104 .9344 .51788 .97986 .50616 .96928 .52724 Metetra
.35305 .71698 .37846 .69526 .41684 .68118 .39263 .6925 Metetra
.7 .979 .793 r
.39263 .6925 .41684 .68118 .45574 .66493 .43271 .65948 Metetra
.852 .907 .872 r
.43271 .65948 .45574 .66493 .49509 .64751 .47294 .62237 Metetra
.795 .747 .809 r
.47294 .62237 .49509 .64751 .53487 .63015 .51315 .58628 Metetra
.728 .643 .764 r
.51315 .58628 .53487 .63015 .57512 .61402 .55344 .55592 Metetra
.671 .577 .741 r
.55344 .55592 .57512 .61402 .61592 .60002 .59407 .53478 Metetra
.621 .535 .735 r
.59407 .53478 .61592 .60002 .65737 .5886 .63546 .52456 Metetra
.573 .51 .747 r
.63546 .52456 .65737 .5886 .6996 .57967 .67801 .52498 Metetra
.517 .505 .781 r
.67801 .52498 .6996 .57967 .74267 .57263 .72204 .53377 Metetra
.434 .528 .854 r
.72204 .53377 .74267 .57263 .78662 .56646 .76763 .54696 Metetra
.239 .582 .945 r
.76763 .54696 .78662 .56646 .83138 .55992 .81457 .55949 Metetra
.156 0 0 r
.81457 .55949 .83138 .55992 .87684 .55177 .86235 .56612 Metetra
.281 0 0 r
.86235 .56612 .87684 .55177 .92286 .54104 .9103 .56259 Metetra
.207 0 0 r
.9103 .56259 .92286 .54104 .96928 .52724 .95772 .54667 Metetra
.706 .983 .811 r
.32695 .7284 .35305 .71698 .39263 .6925 .36769 .69573 Metetra
.891 .955 .837 r
.36769 .69573 .39263 .6925 .43271 .65948 .40898 .64972 Metetra
.885 .847 .801 r
.40898 .64972 .43271 .65948 .47294 .62237 .4502 .59763 Metetra
.836 .755 .774 r
.4502 .59763 .47294 .62237 .51315 .58628 .49107 .54756 Metetra
.774 .681 .761 r
.49107 .54756 .51315 .58628 .55344 .55592 .53164 .50674 Metetra
.703 .618 .759 r
.53164 .50674 .55344 .55592 .59407 .53478 .57234 .48035 Metetra
.622 .562 .766 r
.57234 .48035 .59407 .53478 .63546 .52456 .61376 .4709 Metetra
.521 .51 .784 r
.61376 .4709 .63546 .52456 .67801 .52498 .65656 .47797 Metetra
.389 .461 .814 r
.65656 .47797 .67801 .52498 .72204 .53377 .70124 .49825 Metetra
.202 .415 .847 r
.70124 .49825 .72204 .53377 .76763 .54696 .74798 .52578 Metetra
0 .373 .83 r
.74798 .52578 .76763 .54696 .81457 .55949 .79651 .55273 Metetra
0 .35 .671 r
.79651 .55273 .81457 .55949 .86235 .56612 .84609 .57073 Metetra
.178 0 0 r
.84609 .57073 .86235 .56612 .9103 .56259 .89561 .57277 Metetra
.89561 .57277 .9103 .56259 .95772 .54667 .94397 .55507 Metetra
.842 .895 .876 r
.30078 .72219 .32695 .7284 .36769 .69573 .3424 .68544 Metetra
.88 .841 .803 r
.3424 .68544 .36769 .69573 .40898 .64972 .38468 .63312 Metetra
.876 .798 .77 r
.38468 .63312 .40898 .64972 .4502 .59763 .42684 .57387 Metetra
.853 .763 .763 r
.42684 .57387 .4502 .59763 .49107 .54756 .46844 .51717 Metetra
.811 .729 .774 r
.46844 .51717 .49107 .54756 .53164 .50674 .50953 .47139 Metetra
.742 .689 .801 r
.50953 .47139 .53164 .50674 .57234 .48035 .55055 .44243 Metetra
.627 .633 .837 r
.55055 .44243 .57234 .48035 .61376 .4709 .59219 .43313 Metetra
.445 .544 .863 r
.59219 .43313 .61376 .4709 .65656 .47797 .63523 .443 Metetra
.202 .422 .852 r
.63523 .443 .65656 .47797 .70124 .49825 .68028 .46831 Metetra
0 .312 .803 r
.68028 .46831 .70124 .49825 .74798 .52578 .72761 .50225 Metetra
0 .287 .774 r
.72761 .50225 .74798 .52578 .79651 .55273 .77697 .53575 Metetra
0 .405 .828 r
.77697 .53575 .79651 .55273 .84609 .57073 .82754 .55892 Metetra
.227 .677 .952 r
.82754 .55892 .84609 .57073 .89561 .57277 .87805 .56339 Metetra
.622 .914 .98 r
.87805 .56339 .89561 .57277 .94397 .55507 .9272 .54462 Metetra
.783 .737 .81 r
.27515 .69458 .30078 .72219 .3424 .68544 .31718 .65888 Metetra
.827 .747 .775 r
.31718 .65888 .3424 .68544 .38468 .63312 .35999 .60842 Metetra
.848 .758 .763 r
.35999 .60842 .38468 .63312 .42684 .57387 .40276 .55134 Metetra
.854 .774 .773 r
.40276 .55134 .42684 .57387 .46844 .51717 .44503 .49661 Metetra
.838 .794 .809 r
.44503 .49661 .46844 .51717 .50953 .47139 .48677 .45218 Metetra
.779 .811 .878 r
.48677 .45218 .50953 .47139 .55055 .44243 .52836 .42367 Metetra
.612 .782 .963 r
.52836 .42367 .55055 .44243 .59219 .43313 .57047 .4138 Metetra
.275 .619 .959 r
.57047 .4138 .59219 .43313 .63523 .443 .61384 .42212 Metetra
0 .393 .838 r
.61384 .42212 .63523 .443 .68028 .46831 .65908 .44502 Metetra
0 .294 .776 r
.65908 .44502 .68028 .46831 .72761 .50225 .70651 .47604 Metetra
0 .346 .821 r
.70651 .47604 .72761 .50225 .77697 .53575 .75593 .50655 Metetra
.212 .492 .9 r
.75593 .50655 .77697 .53575 .82754 .55892 .80663 .52723 Metetra
.486 .635 .92 r
.80663 .52723 .82754 .55892 .87805 .56339 .85745 .53021 Metetra
.681 .722 .881 r
.85745 .53021 .87805 .56339 .9272 .54462 .90715 .51122 Metetra
.717 .635 .765 r
.25044 .64651 .27515 .69458 .31718 .65888 .29231 .61671 Metetra
.764 .673 .761 r
.29231 .61671 .31718 .65888 .35999 .60842 .33502 .5759 Metetra
.803 .722 .775 r
.33502 .5759 .35999 .60842 .40276 .55134 .37791 .52995 Metetra
.833 .789 .81 r
.37791 .52995 .40276 .55134 .44503 .49661 .42058 .48548 Metetra
.837 .88 .874 r
.42058 .48548 .44503 .49661 .48677 .45218 .46298 .44852 Metetra
.732 .956 .945 r
.46298 .44852 .48677 .45218 .52836 .42367 .50535 .42341 Metetra
.367 .834 .883 r
.50535 .42341 .52836 .42367 .57047 .4138 .54818 .41229 Metetra
0 .549 .715 r
.54818 .41229 .57047 .4138 .61384 .42212 .59206 .41479 Metetra
0 .41 .709 r
.59206 .41479 .61384 .42212 .65908 .44502 .63747 .42809 Metetra
0 .436 .841 r
.63747 .42809 .65908 .44502 .70651 .47604 .68468 .44719 Metetra
.221 .505 .906 r
.68468 .44719 .70651 .47604 .75593 .50655 .7336 .46561 Metetra
.433 .545 .87 r
.7336 .46561 .75593 .50655 .80663 .52723 .78371 .47654 Metetra
.564 .573 .821 r
.78371 .47654 .80663 .52723 .85745 .53021 .83422 .47431 Metetra
.651 .603 .788 r
.83422 .47431 .85745 .53021 .90715 .51122 .88425 .45593 Metetra
.659 .568 .742 r
.22664 .58345 .25044 .64651 .29231 .61671 .26784 .56294 Metetra
.691 .609 .759 r
.26784 .56294 .29231 .61671 .33502 .5759 .30981 .53736 Metetra
.729 .68 .803 r
.30981 .53736 .33502 .5759 .37791 .52995 .35224 .50924 Metetra
.767 .802 .881 r
.35224 .50924 .37791 .52995 .42058 .48548 .39493 .48152 Metetra
.724 .951 .951 r
.39493 .48152 .42058 .48548 .46298 .44852 .43785 .45698 Metetra
.368 .826 .734 r
.43785 .45698 .46298 .44852 .50535 .42341 .48109 .43771 Metetra
.48109 .43771 .50535 .42341 .54818 .41229 .52489 .42476 Metetra
.116 0 0 r
.52489 .42476 .54818 .41229 .59206 .41479 .56952 .41793 Metetra
.025 0 0 r
.56952 .41793 .59206 .41479 .63747 .42809 .61523 .41585 Metetra
.306 .729 .973 r
.61523 .41585 .63747 .42809 .68468 .44719 .66216 .41612 Metetra
.509 .656 .926 r
.66216 .41612 .68468 .44719 .7336 .46561 .7103 .41578 Metetra
.571 .581 .824 r
.7103 .41578 .7336 .46561 .78371 .47654 .75945 .41182 Metetra
.603 .55 .768 r
.75945 .41182 .78371 .47654 .83422 .47431 .80928 .40186 Metetra
.629 .548 .744 r
.80928 .40186 .83422 .47431 .88425 .45593 .85941 .38464 Metetra
.605 .524 .736 r
.20333 .51379 .22664 .58345 .26784 .56294 .24351 .50379 Metetra
.599 .548 .769 r
.24351 .50379 .26784 .56294 .30981 .53736 .28425 .4957 Metetra
.598 .618 .843 r
.28425 .4957 .30981 .53736 .35224 .50924 .32571 .48848 Metetra
.571 .764 .969 r
.32571 .48848 .35224 .50924 .39493 .48152 .36802 .48085 Metetra
.326 .809 .882 r
.36802 .48085 .39493 .48152 .43785 .45698 .41123 .47155 Metetra
.002 0 0 r
.41123 .47155 .43785 .45698 .48109 .43771 .45534 .45958 Metetra
.079 0 0 r
.45534 .45958 .48109 .43771 .52489 .42476 .50025 .44444 Metetra
.50025 .44444 .52489 .42476 .56952 .41793 .54588 .4262 Metetra
.35 .813 .715 r
.54588 .4262 .56952 .41793 .61523 .41585 .59214 .40554 Metetra
.686 .918 .972 r
.59214 .40554 .61523 .41585 .66216 .41612 .63897 .38354 Metetra
.702 .735 .877 r
.63897 .38354 .66216 .41612 .7103 .41578 .68642 .36153 Metetra
.663 .614 .79 r
.68642 .36153 .7103 .41578 .75945 .41182 .73459 .34077 Metetra
.634 .553 .746 r
.73459 .34077 .75945 .41182 .80928 .40186 .78365 .32221 Metetra
.617 .527 .73 r
.78365 .32221 .80928 .40186 .85941 .38464 .83378 .30634 Metetra
.547 .497 .751 r
.1797 .44669 .20333 .51379 .24351 .50379 .21879 .44621 Metetra
.471 .484 .79 r
.21879 .44621 .24351 .50379 .28425 .4957 .25812 .45425 Metetra
.355 .502 .867 r
.25812 .45425 .28425 .4957 .32571 .48848 .29834 .46677 Metetra
.132 .54 .92 r
.29834 .46677 .32571 .48848 .36802 .48085 .33996 .47861 Metetra
.129 0 0 r
.33996 .47861 .36802 .48085 .41123 .47155 .38322 .48439 Metetra
.144 0 0 r
.38322 .48439 .41123 .47155 .45534 .45958 .42807 .47975 Metetra
.42807 .47975 .45534 .45958 .50025 .44444 .47415 .46239 Metetra
.47415 .46239 .50025 .44444 .54588 .4262 .52097 .4327 Metetra
.733 .99 .822 r
.52097 .4327 .54588 .4262 .59214 .40554 .56805 .39371 Metetra
.854 .92 .874 r
.56805 .39371 .59214 .40554 .63897 .38354 .61512 .35029 Metetra
.802 .765 .819 r
.61512 .35029 .63897 .38354 .68642 .36153 .66224 .30797 Metetra
.73 .652 .771 r
.66224 .30797 .68642 .36153 .73459 .34077 .70975 .27179 Metetra
.666 .577 .745 r
.70975 .27179 .73459 .34077 .78365 .32221 .75823 .24544 Metetra
.611 .529 .738 r
.75823 .24544 .78365 .32221 .83378 .30634 .80836 .2307 Metetra
.477 .491 .793 r
.15476 .39017 .1797 .44669 .21879 .44621 .19298 .39641 Metetra
.274 .406 .815 r
.19298 .39641 .21879 .44621 .25812 .45425 .2311 .41613 Metetra
0 .303 .798 r
.2311 .41613 .25812 .45425 .29834 .46677 .27016 .44326 Metetra
0 .228 .691 r
.27016 .44326 .29834 .46677 .33996 .47861 .31102 .46981 Metetra
.32 0 0 r
.31102 .46981 .33996 .47861 .38322 .48439 .35418 .4872 Metetra
.092 0 0 r
.35418 .4872 .38322 .48439 .42807 .47975 .3996 .48821 Metetra
.344 .814 .741 r
.3996 .48821 .42807 .47975 .47415 .46239 .44675 .46897 Metetra
.736 .992 .838 r
.44675 .46897 .47415 .46239 .52097 .4327 .49478 .43018 Metetra
.89 .965 .834 r
.49478 .43018 .52097 .4327 .56805 .39371 .5429 .37692 Metetra
.898 .872 .804 r
.5429 .37692 .56805 .39371 .61512 .35029 .5906 .31718 Metetra
.851 .779 .781 r
.5906 .31718 .61512 .35029 .66224 .30797 .63788 .25972 Metetra
.783 .696 .768 r
.63788 .25972 .66224 .30797 .70975 .27179 .68525 .21223 Metetra
.702 .623 .765 r
.68525 .21223 .70975 .27179 .75823 .24544 .73357 .18015 Metetra
.613 .56 .771 r
.73357 .18015 .75823 .24544 .80836 .2307 .78383 .16605 Metetra
.373 .522 .877 r
.12748 .3498 .15476 .39017 .19298 .39641 .16532 .35885 Metetra
0 .314 .804 r
.16532 .35885 .19298 .39641 .2311 .41613 .20281 .38363 Metetra
0 .125 .646 r
.20281 .38363 .2311 .41613 .27016 .44326 .2412 .41726 Metetra
0 .101 .584 r
.2412 .41726 .27016 .44326 .31102 .46981 .2816 .45045 Metetra
0 .286 .704 r
.2816 .45045 .31102 .46981 .35418 .4872 .32469 .47309 Metetra
.201 .666 .941 r
.32469 .47309 .35418 .4872 .3996 .48821 .37052 .47654 Metetra
.661 .901 .98 r
.37052 .47654 .3996 .48821 .44675 .46897 .41848 .45612 Metetra
.844 .908 .88 r
.41848 .45612 .44675 .46897 .49478 .43018 .46756 .4127 Metetra
.893 .866 .806 r
.46756 .4127 .49478 .43018 .5429 .37692 .51674 .35242 Metetra
.894 .826 .772 r
.51674 .35242 .5429 .37692 .5906 .31718 .56534 .28479 Metetra
.871 .789 .767 r
.56534 .28479 .5906 .31718 .63788 .25972 .6133 .2201 Metetra
.824 .75 .782 r
.6133 .2201 .63788 .25972 .68525 .21223 .66113 .1672 Metetra
.744 .701 .81 r
.66113 .1672 .68525 .21223 .73357 .18015 .70978 .13225 Metetra
.623 .634 .841 r
.70978 .13225 .73357 .18015 .78383 .16605 .76042 .11809 Metetra
.194 .591 .943 r
.09697 .32817 .12748 .3498 .16532 .35885 .13515 .33557 Metetra
0 .26 .71 r
.13515 .33557 .16532 .35885 .20281 .38363 .1729 .35784 Metetra
0 .114 .59 r
.1729 .35784 .20281 .38363 .2412 .41726 .2115 .38841 Metetra
0 .182 .674 r
.2115 .38841 .2412 .41726 .2816 .45045 .25208 .41848 Metetra
.042 .415 .862 r
.25208 .41848 .2816 .45045 .32469 .47309 .29538 .43849 Metetra
.444 .632 .934 r
.29538 .43849 .32469 .47309 .37052 .47654 .34151 .44034 Metetra
.68 .723 .882 r
.34151 .44034 .37052 .47654 .41848 .45612 .38993 .41964 Metetra
.79 .754 .821 r
.38993 .41964 .41848 .45612 .46756 .4127 .43968 .37722 Metetra
.842 .77 .782 r
.43968 .37722 .46756 .4127 .51674 .35242 .4897 .31881 Metetra
.866 .784 .767 r
.4897 .31881 .51674 .35242 .56534 .28479 .53928 .25338 Metetra
.87 .8 .778 r
.53928 .25338 .56534 .28479 .6133 .2201 .58827 .19065 Metetra
.849 .816 .817 r
.58827 .19065 .6133 .2201 .66113 .1672 .6371 .13902 Metetra
.78 .82 .884 r
.6371 .13902 .66113 .1672 .70978 .13225 .68662 .10435 Metetra
.623 .775 .954 r
.68662 .10435 .70978 .13225 .76042 .11809 .73796 .08937 Metetra
.024 .593 .769 r
.06267 .32461 .09697 .32817 .13515 .33557 .10202 .32602 Metetra
.231 0 0 r
.10202 .32602 .13515 .33557 .1729 .35784 .14114 .33844 Metetra
0 .334 .735 r
.14114 .33844 .1729 .35784 .2115 .38841 .18107 .35676 Metetra
.061 .434 .873 r
.18107 .35676 .2115 .38841 .25208 .41848 .22272 .37438 Metetra
.355 .521 .883 r
.22272 .37438 .25208 .41848 .29538 .43849 .2667 .38432 Metetra
.538 .569 .834 r
.2667 .38432 .29538 .43849 .34151 .44034 .31313 .38076 Metetra
.646 .605 .794 r
.31313 .38076 .34151 .44034 .38993 .41964 .36165 .36063 Metetra
.718 .642 .772 r
.36165 .36063 .38993 .41964 .43968 .37722 .41152 .32449 Metetra
.772 .687 .769 r
.41152 .32449 .43968 .37722 .4897 .31881 .46194 .27642 Metetra
.815 .742 .783 r
.46194 .27642 .4897 .31881 .53928 .25338 .51233 .22283 Metetra
.844 .811 .818 r
.51233 .22283 .53928 .25338 .58827 .19065 .56249 .17089 Metetra
.839 .892 .878 r
.56249 .17089 .58827 .19065 .6371 .13902 .61268 .12699 Metetra
.745 .947 .946 r
.61268 .12699 .6371 .13902 .68662 .10435 .66355 .09568 Metetra
.507 .888 .965 r
.66355 .09568 .68662 .10435 .73796 .08937 .71592 .07912 Metetra
.02447 .33522 .06267 .32461 .10202 .32602 .06584 .32706 Metetra
.095 .652 .753 r
.06584 .32706 .10202 .32602 .14114 .33844 .10746 .32371 Metetra
.305 .728 .973 r
.10746 .32371 .14114 .33844 .18107 .35676 .14987 .32274 Metetra
.478 .656 .939 r
.14987 .32274 .18107 .35676 .22272 .37438 .19352 .32109 Metetra
.548 .578 .837 r
.19352 .32109 .22272 .37438 .2667 .38432 .23873 .31572 Metetra
.587 .545 .775 r
.23873 .31572 .2667 .38432 .31313 .38076 .28555 .30418 Metetra
.619 .544 .748 r
.28555 .30418 .31313 .38076 .36165 .36063 .33385 .28518 Metetra
.652 .567 .746 r
.33385 .28518 .36165 .36063 .41152 .32449 .38332 .25896 Metetra
.689 .613 .766 r
.38332 .25896 .41152 .32449 .46194 .27642 .43359 .22719 Metetra
.731 .691 .813 r
.43359 .22719 .46194 .27642 .51233 .22283 .48438 .1926 Metetra
.768 .811 .887 r
.48438 .1926 .51233 .22283 .56249 .17089 .53559 .15837 Metetra
.737 .941 .951 r
.53559 .15837 .56249 .17089 .61268 .12699 .58731 .12745 Metetra
.525 .92 .867 r
.58731 .12745 .61268 .12699 .66355 .09568 .63984 .10206 Metetra
.27 .766 .711 r
.63984 .10206 .66355 .09568 .71592 .07912 .69357 .08327 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Qoo`03
001oogoo00=oo`03001oogoo00Uoo`@001aoo`00IWoo0P0077oo00<007ooOol00goo00<007ooOol0
2Goo00<007ooOol077oo001SOol3000NOol00`00Oomoo`03Ool00`00Oomoo`02Ool50003Ool00`00
Oomoo`0KOol004Moo`8001Qoo`8001moo`P00003Ool007oo00]oo`03001oogoo01Yoo`00AWoo00@0
07ooOol001Aoo`<001ioo`H000A3WP800003Ool007oo00Moo`04001oogoo000LOol004Ioo`04001o
ogoo000BOol2000NOol50003@ih00`00@ih00002000014lN000000002Goo0P007Goo0016Ool01000
Oomoo`003goo0`0000=oo`00Ool06goo10000eoN00<004>N00000P001DlN00<007oo00009goo0016
Ool01000Oomoo`003Goo0P007goo10001EoN0P0000=?7P00Cah01TlN00<007oo00009Woo0017Ool2
000;Ool3000NOol300000d>k001OgP05Gmh00`00HeaSG0020008Cah2000VOol0059oo`<001ioo`80
0093^`8000EOgP8000ASG003001?7TlN00E?7P@002Eoo`00Cgoo0`007goo1`0015oN0P001V=L0P00
0TlN10000TjJ00<007oo000097oo001<Ool3000QOol01@00GmiOgUoN00000f^L0P0000=OgP00He`0
1f=L10001djJ0P0097oo001:Ool3000QOol20003Gmh00`00Jia[W004Ji`00`00HeaSG007He`00`00
G]T00008CYX00`00Ool0000SOol004Moo`<0009oo`03001oogoo01eoo`8000AOgP03001[W6^L00E[
W003001SG6=L00ISG080009Nf@03001>VTjJ00I>VP03001oo`00029oo`00AGoo0P0097oo00<005oN
Gmh00eoN00<006^LJi`01F^L00<006mJ00001F=L0P0000ENf@00G]UNf@0000Q>VP03001oo`00029o
o`00@Woo0`0097oo0`0015oN00<006^LJi`01F^L00D006mJKeY_FP0000=SG005001NfEkIG]T00003
G]T00`00CYY>VP06CYX00`00Ool0000QOol002Eoo`@001Moo`8002Ioo`04001S?6<lHc`200000eoN
001[W006Ji`00`00KeY_FP03KeX00`00He`00004G]T01@00G]UNfEkI000024jJ00@007ooOol0021o
o`009Woo00<007ooOol057oo0`009goo00<006<lHc`016<l0P001V^L00<006mJKeX01FmJ00<005kI
G]T00ekI00<005kIG]T00UkI00<004jJCYX014jJ1@0087oo000OOol50003Ool00`00Oomoo`0AOol2
000XOol20007Hc`00`00KcX00005Ji`00`00KeY_FP05KeX00`00G]UNf@03G]T00`00G]UNf@02G]T0
0`00CYY>VP040002CSP01000Oomoo`007goo000XOol00`00Oomoo`0=Ool3000YOol00`00HcaS?006
Hc`01@00KcY_>Vlj00000f^L00<006mJKeX01FmJ00<006[i000015kI00<005kIG]T00UkI0`001dhh
00<007oo00007goo000UOol01000Oomoo`003Goo0P00:goo00<006<lHc`01V<l00<006ljKcX00flj
00<006^L00001fmJ00@006[iJ_T000ENf@06001NfEkI001JN00024hh00@007ooOol001ioo`009Woo
0P002goo0`00:goo0P0026<l00<006ljKcX01Flj00<006mJKeX01FmJ00<006[iJ_T00V[i00<005kI
G]T00UkI0`000eYh00<004hhCSP01Dhh00D007ooOomoo`0001eoo`00<Goo0P0000=oo`00Ool0:Woo
00<005kJ00001f<l00<006ljKcX01Vlj0P001FmJ00<006[iJ_T016[i00<005kIG]T00P001EYh00<0
04hhCSP01Thh00@007ooOol001eoo`00;Woo0`00;Goo0P000ekJ00<006<lHc`00f<l00<006ljKcX0
1Vlj00@006lhKcP000=_FP03001ZnF[i00EZn@05001Nf@00FWP00005FWP00`00CSQ>>006CSP00`00
Oomoo`02000LOol002]oo`<002ioo`8000INfP05001S?6<lHc`00008KcX00`00KcQ_>002KcP00`00
KeX00008J_T20002FWP00`00FWQJN004FWP00`00CSQ>>006CSP01@00Oomoogoo00001Goo0P0057oo
000YOol20002Ool00`00Oomoo`0[Ool00`00G]YNfP07G]X00`00Hc`00008KcX00`00KcQ_>004KcP0
0`00J_UZn@06J_T00`00I]P00002FWP00`00FWQJN004FWP00`00CSQ>>006CSP01@00Oomoogoo0000
17oo00@007ooOol001=oo`009Woo0`00<7oo0P002ekJ0P001flj00<006lhKcP01Flh00<006[iJ_T0
1F[i00L006KHI]QVf000FWP00007FWP00`00CSQ>>005CSP00`00Oomoo`02Ool01@00Oomoogoo0000
0Woo00<007ooOol04Goo000TOol2000bOol00`00FWP0000:G]X01000J_UZn@001Flj00<006lhKcP0
1Flh00<006lh00001F[i00<006KHI]P00VKH00<005Yh00001eYh00<004hhCSP014hh0`0017oo00@0
07ooOol0009oo`03001oogoo015oo`008Goo0`00<Woo0P000eYh00<005kJG]X01UkJ00<006[iJ_T0
0V[i00D006ljKcY_>P0000M_>004001_>6lh0004J_T00`00I]QVf004I]P20007FWP00`00CSQ>>004
0002CQL01000Oomoogoo0P000Woo00@007ooOol001=oo`007goo0P00<goo0P001EYh00<005kJG]X0
1EkJ00<006[iJ_T016[i00<006lj00001flh00<006lhKcP00Vlh00@006[iJ_T000MVf08000MJN0<0
00M>5`03001oogoo009oo`03001oogoo008001Aoo`0077oo0`00=7oo00<005YhFWP01UYh00<005kJ
G]X00ekJ00<006[iJ_T01V[i00<006lhKcP01Flh00<006lhKcP00flh00<006[i00001FKH1@001EYh
0P0000=FE`00CQL01dhG00<007ooOol00goo00<007ooOol05Goo000KOol2000dOol20009FWP00`00
G]YNfP02G]X00`00J_UZn@07J_T00`00KcQ_>005KcP00`00KcQ_>004KcP01000I]QVf6KH0`0016KH
0P000UYh0P0015IG00<004hGCQL01DhG00<007ooOol00goo00<007ooOol05Goo000DOol30004Ool0
0`00Ool0000bOol00`00DUL0000:FWP01000G]YNfP002F[i00<006lh00001Flh00<006lhKcP01Flh
10001fKH00@005Yh000000IFE`03001>5dhG00I>5`03001oogoo00=oo`03001oogoo01Aoo`005Goo
00<007ooOol00goo00@007ooOol002moo`8000=BE`03001JN5Yh00MJN003001NfP0000UZn@05001_
>6lhKcP00003KcP00`00KcQ_>004KcP40009I]P200000eIG001FE`05EUL00`00CQM>5`06CQL00`00
Oomoo`03Ool2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0000/Ool20005DUL0
0`00FWQJN008FWP00`00J_UZn@07J_T00`00KcQ_>003KcP00`00KcP00004KcP40002KaP00`00I]QV
f006I]P01@00GYP005IG00001eIG00<004hGCQL01DhG00<007ooOol017oo00<007ooOol04goo000E
Ool00`00Oomoo`02Ool20004Ool00`00Oomoo`0XOol00`00DUMBE`05DUL00`00FWQJN007FWP20009
J_T00`00KcQ_>004KcP00`00KcQ_>0030003KcP01@00KaQ_66lH00001VKH00L005jHGYQNV000EUL0
0006EUL00`00CQM>5`06CQL00`00Oomoo`04Ool00`00Oomoo`0BOol001Eoo`03001oogoo009oo`80
00Eoo`03001oogoo02Eoo`8000UBE`03001JN5Yh00EJN004001R^6:h0007J_T00`00KcQ_>005KcP3
0005KcP00`00KaQ_6002KaP00`00I]QVf002I]P20004GYP01000EUMFE`001EIG00<004hGCQL01ThG
00<007ooOol017oo00<007ooOol04Woo000DOol20004Ool00`00Oomoo`05Ool00`00Oomoo`0QOol3
00000dhg001BE`08DUL00`00FWQJN004FWP00`00H[QR^002H[P00`00J_UZn@03J_T00`00KcQ_>004
KcP200000g=H001_>004KcP00`00KaQ_6004KaP01000I]QVf0001UjH00@005IGEUL000IFE`03001>
5dhG00E>5`03001oogoo00Eoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol07Goo
0`001Dhg00<0059GDUL01U9G00<005YhFWP00eYh00<006:hH[P016:h00<006[iJ_T00V[i00@006lh
KcQ_>0@000=cF005001_>6lhKcP00007KaP00`00I]P00008GYP01000EUMFE`001EIG00<004hGCQL0
1ThG00<007ooOol01Goo00<007ooOol047oo000JOol00`00Oomoo`07Ool00`00Oomoo`0HOol40008
CSL00`00DUMBE`07DUL01@00FWQJN5Yh00001f:h00H006[iJ_UZn@00KcP30002KcP00`00LeQcF002
LeP01@00KcQ_>6lh000026lH00<005jHGYP01ejH00D005IGEUMFE`0000AFE`03001>5dhG00A>5`<0
00Ioo`80019oo`006Goo0P002Woo00<007ooOol047oo1`000dXh00<004hgCSL01Thg00<0059GDUL0
1e9G00@005YhFWP000UR^003001Zn@00008000E_>003001cF7=H00=cF003001_>00000Q_608000UN
V003001FEeIG009FE`03001FEeIG009FE`03001>5`0000<0009>5`03001oogoo00Ioo`03001oogoo
00moo`0027oo0P0017oo0P000goo0`000goo0P002goo00<007ooOol027oo1`000dIJ00<004XhBSP0
1DXh00<004hgCSL01Thg00<0059GDUL01U9G00<005Yh00002F:h0`001flh00<007=HLeP01G=H00<0
06lHKaP01VlH00@006LJIaX000UNV005001FEeIGEUL00003EUL30007CQL00`00Oomoo`06Ool00`00
Oomoo`0>Ool000Moo`04001oogoo0003Ool20006Ool01000Oomoo`000Woo5@000cjN00<004IJAUX0
1DIJ00<004XhBSP014Xh00<004hgCSL01Thg00<0059GDUL01U9G0P001f:h0`000V:h00<006lhKcP0
16lh00<007=HLeP01W=H00<006lHKaP01FlH00D006LJIaYW6P0000UNV003001FEeIG009FE`<00003
DSL004hG00M>5`03001oogoo00Ioo`03001oogoo00ioo`0000=oo`0000000`0000Eoo`00Oomoo`00
00]oo`03001oogoo00<000Yoo`D000<Vo`03000nWSjN00@nWP030016FTIJ00A6FP03001:>4Xh00A:
>003001>=dhg00I>=`03001BEe9G00EBE`03001JV00000AR^0<000AR^003001[6@0000E_>003001c
F7=H00EcF003001cN@0000E_6003001W6VLJ00=W6P03001NV5jH00INV003001FE`0000800005DSL0
058gDSL00008CQL00`00Oomoo`07Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool0
0`00Oomoo`0;Ool00`006UhJGP0300029_l00`00?YhnWP03?Yh00`00AUY6FP04AUX00`00BSQ:>005
BSP00`00CSM>=`06CSL00`00DUMBE`03DUL01000FYQJV5ZH10001V:h00D006/IJaU[6@0000=_>003
001cF7=H00EcF005001cNG=iLgT00003KaP00`00IaYW6P04IaX00`00GYQNV006GYP20004DSL01@00
DSMB=e8g000024hG00<007ooOol01goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`0027oo
00<007ooOol00goo00<007ooOol037oo00@001YN6UhJGPD000DnWP030016FTIJ00A6FP03001:>4Xh
00A:>003001>=dhg00I>=`04001BEe9GDUL20003FYP30007H[P00`00JaU[6@03JaT00`00KcP00007
LeP00`00LgUcN@03LgT00`00KaP00007IaX00`00GYQNV004GYP200000eIi001B=`04DSL01000DSMB
=`0024hG00<007ooOol01goo00<007ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool0
0`00Oomoo`0=Ool400000aYN000000020004?Yh00`00AUY6FP04AUX00`00BSQ:>005BSP00`00CSM>
=`05CSL01000DUMBE`000UZH0`000UZH00<006:hH[P016:h00<006/IJaT01F/I00<007=HLeP01G=H
00<007=iLgT017=i0P002FLJ00@005jHGYQNV08000AFN@03001B=e8g00=B=`03001B=`0000Q>5`03
001oogoo00Qoo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P003goo00@002hL;Q`^70L0008n
WP030016FTIJ00A6FP03001:>4Xh00E:>003001>=dhg00I>=`D000IJV003001R^6:h009R^003001[
6F/I00I[6@8000EcF003001cNG=i00AcN@03001_^`0000UW6P04001NV0000006EWT00`00DSMB=`04
DSL00`00DSL00008CQL00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qoo`80011o
o`03000^72hL008^70030001F@0000@00003?Yh004IJ00E6FP03001:>4Xh00E:>003001>=dhg00A>
=`<000UJV005001R^6:hH[P00008JaT01000LgUcN@000g=H00<007=iLgT017=i00D006nkKk]_^`00
00QW6P8000QFN@03001B=e8g00AB=`03001B=`0000Q>5`03001oogoo00Moo`80009oo`04001oogoo
0006Ool0019oo`8000Aoo`03001oogoo011oo`03000^72hL008^70030001F@00008000031K/00000
00I6FP03001:>4Xh00E:>004001>=dhgCSL300000e9i001JV00:FYP00`00H[P00008JaT00`00LgUc
N@02LgT00`00LeP00006LgT00`00Kk]_^`03Kk/00`00IaYW6P02IaX300000e[l001FN@08EWT00`00
DSMB=`04DSL20008CQL00`00Oomoo`09Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool0
0`00Oomoo`0AOol00`00;Q`^7002;Q`00`000ET1F@0200021K/500000dIJ001:>006BSP40005DWT0
0`00FYQJV008FYP20008JaT00`00LgUcN@04LgT20005LgT00`00Kk]_^`05Kk/01000IaX000001E[l
00<005IiEWT01UIi00<0058gDSL01E8g0P0024hG00<007ooOol02Goo00<007ooOol027oo000AOol0
1000Oomoo`000goo00<007ooOol04Woo00<002hL;Q`00RhL00<0005I0ET00P0000<5^`00;Q`00bhL
2`001e9i00<005ZHFYP01eZH00<006LJ00001f/I00<007=iLgT01G=i00<007?J00000g=i00<006nk
Kk/01Vnk0P001e[l00<005IiEWT01UIi00<0058gDSL01U8g00<004hGCQL01DhG0P002goo00<007oo
Ool027oo000AOol01000Oomoo`000Woo0P005Goo100000@^70000ET1F@@000D^70030016FTIJ0080
00=6FP03001BNE9i00IBN@03001JV5ZH00EJV005001W6VLJIaX00006JaT00`00LgUcN@04LgT01`00
LmYcfW?J001cN@0000M_^`800003Ekl005[l00MJo003001FNEIi00IFN@03001B=e8g00EB=`03001>
5dhG009>5`<00003DSL007oo00]oo`03001oogoo00Moo`004Goo00@007ooOol000=oo`03001oogoo
01=oo`04000mnCgi?OT400000`5I0000]@020004;Q`30006AUX00`00DWUBN@06DWT00`00FYQJV003
FYP00`00IaYW6P03IaX00`00JaU[6@02JaT00`00LgUcN@04LgT00`00LmYcfP02LmX30005Kk/20004
Ekl00`00F_aJo005F_`00`00EWUFN@06EWT00`00DSMB=`06DSL40005DSL00`00Oomoo`09Ool00`00
Oomoo`07Ool0019oo`8000=oo`8001Ioo`03000mnCgi008mn@03000QVP00008000030;D0000000<0
0003;Q`004IJ00I6FP03001BNE9i00IBN@03001JV5ZH009JV003001W6VLJ00AW6P05001[6F/IJaT0
0006LgT00`00LmYcfP02LmX01000GoYOnUoj0P0000=_^`0000001eNo00<005[lF_`01E[l00<005Ii
EWT01EIi00<0058gDSL0158g10001e8g00<007ooOol02Woo00<007ooOol01Woo000GOol00`00Oomo
o`0EOol00`00?OTmn@03?OT00`008IXQVP040005;Q`00`00AUY6FP05AUX00`00DWUBN@06DWT01000
FYQJV00026LJ00<006/I00001W=i00<007?JLmX00W?J00<005ojGoX00eoj10001eNo00<005[lF_`0
1E[l00<005IiEWT01UIi00<0058g00000`000Thh0P001e8g00<007ooOol02goo00<007ooOol01Goo
000GOol00`00Oomoo`0FOol00`00?OTmn@02?OT40000126J0000]@001BhL00<004IJAUX014IJ00<0
059iDWT01U9i00<005ZH00002VLJ0P001W=i00D007?JLmYcfP0000=OnP<0008_EQ`000M>>08000MB
=`03001oogoo00Yoo`03001oogoo00Eoo`005goo00<007ooOol05Woo00<003gi?OT010000b6J1@00
0bhL00<004IJAUX014IJ00<0059iDWT01U9i0P002VLJ00<006nlKk`00P000g=i00D007?JLmYcfP00
009OnPL000038Wh0000000<000@Vo`030012WD:M00E2W@03001:FTYJ00E:FP03001>>4hh00I>>003
001B=`0000IB=`03001oogoo00]oo`03001oogoo00Aoo`001Goo0P0017oo0P000goo0`0017oo0P00
5goo0`000cgi00<0026J8IX00R6J00<0004i0000100000<^7000AUX01DIJ00<0059iDWT00U9i0`00
00=Fo000IaX026LJ00<006nlKk`00fnl00<007=i00000g?J1@0014ln00<0029n8Wh00R9n00<0006J
00000P000bKo00<004:M@Yd01D:M00<004YJBUX01DYJ00<004hhCSP01Dhh00@0058gDSL000EB=`03
001oogoo00Yoo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogooOol0000EOol30000
0goo000mn@04?OT00`008IXQVP028IX010000CT000000PFk0P001TIJ00<0059i00000P001EKl00<0
06LJIaX01FLJ00<006nlKk`016nl2@001Tln10000R9n00<0006J0IX0100000<Vo`00@Yd01D:M00<0
04YJBUX01DYJ00<004hhCSP01Thh00<0058g00001U8g00<007ooOol01Woo0`000Woo00<007ooOol0
0goo0004Ool01000Oomoo`002goo00<007ooOol00P004goo0P0017oo00<003gi?OT00cgi00<0026J
8IX00R6J00<0004i0CT00P000`Fk20002EKl00<006LJIaX00fLJ00<006nlKk`016nl00@005ojGoYO
nP8000AS?003001??Tln00<00004DYX003Xk>S/400020IX20000106j000000001T:M00<004YJBUX0
14YJ00<004hhCSP01Thh00@0058gDSL000EB=`03001oogoo00=oo`<000Eoo`03001oogoo00=oo`00
17oo00@007ooOol000Qoo`<000=oo`80011oo`<000Moo`X000048IX0004i0CT200031K/00`00>Ydj
W@03>Yd00`00E_aFo008E_`01@00IaYW6VLJ00001Vnl00<005ojGoX00P001V<l0`001E:J00<003Xk
>S/00SXk0`000P6J0P000P6j0P0014:M00<004YJBUX01DYJ00<004hhCSP01Dhh00D0058gDSMB=`00
00AB=`03001oo`0000<000Qoo`03001oogoo00=oo`0017oo00@007ooOol000Qoo`03001oogoo00=o
o`03001oogoo00aoo`<000Yoo`030015n4Gh00=5n004000mnCgi?OT500000`4i000000021K/00`00
>YdjW@03>Yd00`00E_aFo008E_`00`00IaX00005Kk`200000eoj00000006Hc`200000ekI001BVP06
DYX00`00>S/j>`02>S/00`006K/0000200001@4h0001^P007Q`00`000T:M00<004YJBUX014YJ00<0
04hhCSP01Thh00D0058gDSMB=`0000AB=`03001oogoo00Yoo`03001oogoo00=oo`001Goo0P002Goo
10000Woo00<007ooOol02Goo0`003Goo00<004GhAOP014Gh00<003gi?OT00Sgi00<002hK;Q/01`00
1CZM00<005KlE_`01eKl0`0016nl00<005ojGoX00P0000=[>@00Hc`00f<l00<005kIG]T00UkI00<0
05:JDYX015:J00<003Xk>S/00cXk00<001Vk6K/00P0000@1>000000000<N70<000M:FP03001>>4hh
00E>>005001B=e8gDSL00003DSL2000=Ool00`00Oomoo`02Ool001Ioo`03001oogoo00Ioo`<0015o
o`030015n4Gh00=5n003000mnCgi00<mn@03000^6bhK008^6`8000030CH0000000DjW@03001Fo5Kl
00=Fo0<000=GW`800003Kk`0000000<000=[>@04001S?0000005G]T00`00DYYBVP05DYX00`00>S/j
>`03>S/00`006K/I^`0200020CP200057Q`30004BUX00`00CSQ>>005CSP00`00DSMB=`02DSL30000
0e9i001oo`0<Ool00`00Oomoo`02Ool001Eoo`8000Eoo`<001Aoo`030015n4Gh00A5n003000mnCgi
008mn@03000^6bhK00@^6`8000030CH0000000<00003>Yd005Kl00<000MGW`<0009[V`03001[>F/i
00=[>@03001NfEkI00INf@03001BVU:J00=BVPH00003>S/001Vk00<I^`8000030CP001hL00@N7003
000f>cHk00D000Q>>0@00003DWT0059i009BN@03001oogoo00Yoo`03001oogoo009oo`005Goo00@0
07ooOomoo`<001Moo`030015n4Gh00A5n003000mnCgi00<mn@03000^6bhK00@^6`D0008J_P@000IG
W`<000A[V`03001[>F/i00=[>@8000QNf@03001BVU:J00<0009>>00400126D8I@QT400026K/01000
0CT0004h0P0011hL00<003Hk=S/00cHk2P00159i00D0059iDWUBN@0000aoo`03001oogoo009oo`00
5Goo10006goo00<004GhAOP00dGh00<003gi?OT013gi00<002hK;Q/012hK00<0018m4Sd00P001AZn
0`0000=GW`0000000P0016^K00<006/iJcT00f/i00@006KhI_P000MNf@<000I>>00300126D8I0092
6@03000afP0000<000030CT00000008000<N7003000f>cHk00@f>`03001:FTYJ00E:FP03001BNE9i
00=BN@04001BNE9i000<Ool00`00Oomoo`02Ool000ioo`<000Aoo`8001eoo`030015n4Gh00A5n003
000mnCgi00<mn@03000^6bhK008^6a000004Hc`006/iJcT200000f^K001[>@04JcT00`00I_QVn002
I_P00`00G]UNf@03G]T00`00FWP00007CSP00`00@QU26@03@QT01000<MXafS7J0P000P4i10000QhL
00<003Hk=S/013Hk00<004YJBUX01DYJ00<0059iDWT00e9i00@0059iDWT000]oo`03001oogoo009o
o`003goo00<007ooOol01Goo00<007ooOol06Woo00<004GhAOP014Gh00@003gi?OTmn@L000E>NP03
001;7d/O009;7`03001S?6<l00=S?003001[>F/i00=[>@03001[>F/i00=[>@03001Vn6Kh00=Vn004
001NfEkIG]T20003FWP00`00CSQ>>004CSP00`00@QU26@03@QT00`00<MXafP03<MX200020CT00`00
0GX000020006=S/00`00BUY:FP05BUX00`00DWUBN@04DWT00`00DWT0000;Ool00`00Oomoo`02Ool0
00moo`03001oogoo00Ioo`03001oogoo01Uoo`030015n4Gh00=5n0D000=>>003001>NTij00M>NP04
001;7`000005Hc`00`00JcU[>@04JcT20003JcT00`00I_QVn005I_P00`00G]T00005FWP00`00CSQ>
>005CSP00`00@QU26@03@QT00`00<MXafP02<MX00`006K/000020CT200000`5j000f>`05=S/00`00
BUY:FP05BUX00`00DWUBN@04DWT00`00DWT0000<Ool00`00Oomoo`01Ool000moo`03001oogoo00Mo
o`03001oogoo01Uoo`030015n0000080009>5`03001>>4hh00A>>003001>NTij00I>NP<000ES?003
001[>F/i00A[>@06001_66lH001[>@0026Kh0P001UYh00<004hhCSP01Dhh00<0048I@QT00d8I00<0
037J<MX00c7J00<001Vk00000P0000<1>0000GX00P001CHk00<004YJBUX01DYJ00<0059iDWT0159i
0`002Goo0`000goo000?Ool00`00Oomoo`08Ool2000IOol20006CQL00`00CSQ>>003CSP00`00CWY>
NP05CWX01@00G[UN^Eji00000f<l00<006/iJcT016/i00<006lHKaP00VlH00<006KhI_P01VKh00<0
06:h00001eYh00<004hhCSP014hh00<0048I@QT0148I00<0037J<MX00c7J00<001Vk6K/00P0000<1
>000000010000SHk00<004YJBUX01DYJ00<0059iDWT00U9i1@001Woo0`000R0000<007ooOol00Goo
000>Ool2000<Ool00`00Oomoo`0FOol00`00CQM>5`05CQL00`00CSQ>>004CSP00`00CWY>NP02CWX2
0005G[T00`00Hc`00006JcT00`00KaQ_6003KaP00`00I_QVn005I_P01@00H[QR^6:h00001UYh00<0
04hhCSP01Dhh00<0048I@QT00d8I00<0037J<MX00c7J00<001Vk6K/00QVk00<0004h0CP00P000QWl
0`001dYJ00<0059i00000P000eKl00@005Kl001oo`@000@P008000=oo`007Goo00<007ooOol05Woo
00<004hGCQL014hG00<004hhCSP01Dhh00@004ijCWX000QN^@03001[>F/i00A[>@03001_66lH00=_
6003001_600000EVn003001R^6:h009R^003001JN5Yh00AJN003001>>4hh009>>0l00003<MX001Vk
00<I^`800081>003000Io1Wl008Io0T000MFo0<0008H0003000P020000<00003Ool007oo009oo`00
7Woo00<007ooOol05Goo00<004hGCQL01DhG00<004hhCSP014hh00<004ij00002Eji0P001F/i00<0
06lHKaP00VlH00D006lHKaQ_600000=Vn003001R^6:h00=R^003001JN5Yh00AJN0D000=>=`03001:
5dXG00A:5`030012648H009260D0008I^`<000030CP00000008Io003000nWCjM00DnW@03001Fo5Kl
00IFo0800003600000000080008H0003001oo`0000=oo`007goo0P005Goo00<004hGCQL01DhG00<0
04hhCSP014hh0P002Eji00@006KhI_P000=[>@03001_66lH00=_6003001_66lH009_6003001Vn000
00MR^003001JN5Yh009JN003001FE`0000M>=`03001:5dXG00A:5`030012648H00=26003000f6cHK
00@000030GX0000000<000036O`003jM00HnW@03001Fo5Kl00IFo0<000@H0004001oogoo0003Ool0
025oo`03001oogoo01=oo`03001>5dhG00E>5`03001>>4hh009>>003001JN00000QN^@03001Vn6Kh
009Vn003001[>@0000E_6003001_66lH00A_6003001R^6:h00IR^003001JN5Yh008000=FE`03001>
=dhg00A>=`03001:5dXG00A:5`030012648H00A26003000f6cHK008f6`H000040IX000000007?Yd0
0`00E_aFo005E_`400000aP000000004Ool10001Ool1Ool0029oo`03001oogoo019oo`03001>5dhG
00E>5`03001>>4hh008000=JN003001N^Eji00AN^@03001Vn6Kh00AVn08000A_6003001_66lH00E_
6003001R^6:h00IR^003001JN00000EFE`03001>=dhg00A>=`03001:5dXG00A:5`030012648H00A2
6003000f6cHK00@f6`03000NGP0000<000030IX0000000HnW@03001Fo5Kl009Fo0<00003Ckl004no
008000Ioo`40005oo`5oo`008goo00<007ooOol04Goo00<004hGCQL01DhG00<004hh00001UYh00<0
05jiG[T00Uji00<006KhI_P01FKh00<006lH00000flH00<006lHKaP016lH00<006/H00001f:h0P00
1UIG00<004hgCSL01Dhg00<004XGBQL014XG00<0048H@QP0148H00<003HK=Q/013HK00<001iN7Uh0
0QiN20000SjM00@005Kl000000=?_`<000Qoo`40005oo`5oo`0097oo00<007ooOol04Goo00<004hG
CQL014hG0P001eYh00D005jiG[UN^@0000MVn007001_66lHKaP006lH00001VlH00D006/HJaQ[6000
00ER^003001NV00000MFE`03001>=dhg00A>=`03001:5dXG00A:5`030012648H00A26003000f6cHK
008f6`L000=;7P<0008N_PD00004Ckl000000003Ool00`00Oomoo`05Ool10001Ool1Ool002Eoo`80
015oo`04001>5dhGCQL300000eIG001JN007FWP00`00G[T00007I_P00`00KaQ_6003KaP00`00KaQ_
6004KaP00`00JaQ[6002JaP01000H[QR^6:h0P000ejH00<005IGEUL015IG00<004hgCSL01Dhg00<0
04XGBQL00dXG00<0048H@QP00T8H1`0014ij00<004/NBah00T/N00@007ooOomoo`T000Eoo`03001o
ogoo00Eoo`40005oo`5oo`009goo00<007ooOol03Woo00@004hG000000EFE`03001JN5Yh00EJN080
00MVn003001_66lH00A_608000E_6003001[66/H00=[6003001R^00000ENV003001FEeIG00AFE`03
001>=dhg00E>=`03001:5dXG00=:5`D000=>>003001>NTij00I>NP04001;7P00000COol00`00Oomo
o`04Ool10001Ool1Ool002Qoo`03001oogoo00ioo`03001FEeIG00EFE`03001JN5Yh00AJN003001R
^00000IVn003001_66lH00A_6004001c>7<h0003KaP00`00JaQ[6004JaP20006GYP00`00EUMFE`05
EUL00`00CSM>=`04CSL00`00BQM:5`0300000dhG001>>006CSP00`00CWY>NP05CWX2000FOol00`00
Oomoo`03Ool10001Ool1Ool002Uoo`03001oogoo00eoo`03001FEeIG00EFE`03001JN5Yh00=JN005
001R^6:hH[P00005I_P00`00KaQ_6003KaP00`00LcQc>002LcP00`00KaP00006JaP00`00I_T00007
GYP00`00EUMFE`04EUL00`00CSM>=`03CSL40005CQL00`00CSQ>>004CSP00`00CWY>NP03CWX2000H
Ool00`00Oomoo`04Ool10001Ool002Yoo`8000ioo`03001FEeIG00EFE`05001JN5YhFWP00005H[P0
1@00I_QVn6Kh00001VlH00<007<hLcP00g<h00<006/HJaP016/H00D006KiI_UVn@0000INV003001F
EeIG00AFE`03001>=dhg00<0009B=`03001>5dhG00A>5`03001>>4hh00E>>005001>NTijCWX0000K
Ool00`00Oomoo`03Ool10001Ool002aoo`03001oogoo00]oo`03001FEeIG00EFE`04001JN5Yh0007
H[P00`00I_P00006KaP00`00LcQc>004LcP00`00JaQ[6003JaP00`00I_UVn@02I_T00`00GYQNV004
GYP00`00EUMFE`03EUL40005DSL00`00CQM>5`05CQL00`00CSQ>>004CSP01000CWX000007Goo00<0
07ooOol00Woo0@000Goo000]Ool00`00Oomoo`0:Ool00`00EUMFE`05EUL00`00FWP00009H[P00`00
KaQ_6004KaP00`00LcQc>004LcP00`00LgT00003JaP00`00I_UVn@04I_T00`00GYQNV004GYP00`00
EUL000020002DSL00`00DSMB=`05DSL00`00CQM>5`04CQL00`00CSQ>>004CSP2000POol01@00Oomo
ogoo00000Goo000^Ool00`00Oomoo`0:Ool00`00EUMFE`04EUL2000:H[P20004KaP00`00LcQc>004
LcP01`00LgUcNG=i001[600000MVn@03001NV5jH00=NV0<000EB=`03001B=e8g00EB=`03001>5dhG
00A>5`03001>>4hh00=>>003001oogoo021oo`05001oogooOol00001Ool002moo`8000Yoo`03001F
EeIG00=FE`03001JV00000UR^004001Zn6[h0003KaP00`00LcQc>003LcP00`00LgUcN@03LgT00`00
I_UVn@06I_T00`00GYQNV00300000eIi001B=`06DSL00`00DSMB=`04DSL00`00CQM>5`05CQL00`00
CSQ>>002000TOol01000Oomoo`000Goo000aOol00`00Oomoo`08Ool01@00EUMFEeIG00000eZH00<0
06:hH[P01F:h00<006[hJ_P00V[h00<006lH00001G<h00<007=iLgT00g=i00<006nk00001fKi0`00
1EIi00<0058gDSL0158g00<0058gDSL01E8g00<004hGCQL014hG00<004hh00009goo00<007oo0000
0Goo000bOol00`00Oomoo`07Ool01000EUMFE`0015ZH00<006:hH[P016:h00<006[hJ_P016[h0P00
1G<h00<007=iLgT00W=i00D006nkKk]_^`0000AVn@800003F_/005Ii00EFN@03001B=e8g00AB=`03
001B=e8g00EB=`03001>5dhG00A>5`8002Qoo`03001oogoo00400000<goo00<007ooOol01Woo00<0
05IG00001UZH00<006:hH[P00V:h00<006[hJ_P01F[h00<007=I00000g<h00<007=iLgT00W=i00<0
06nkKk/00fnk00@006Ki000000AJn`03001FNEIi00AFN@03001B=e8g00AB=`03001B=e8g00AB=`04
001>5dhGCQL3000[Ool10001Ool1000003Aoo`8000Moo`03001JV5ZH00IJV004001R^6:h0007J_P0
1`00LeUcFG=I001c>00000AcN@03001_^fnk00A_^`8000IJn`03001FNEIi00AFN@03001B=e8g00AB
=`03001B=e8g00EB=`<002moo`800000=Woo00<007ooOol017oo00<005ZHFYP01UZH00<006:h0000
1f[h00<007=ILeT00g=I0`000W=i00<006nkKk/00Vnk0`0000=G_`00F_/01U[k00<005IiEWT00eIi
00<0058gDSL0158g00<0058gDSL00e8g0`00<7oo0`00000gOol00`00Oomoo`04Ool00`00FYQJV006
FYP00`00J_QZn005J_P00`00LeUcF@03LeT01000LmYcfW?J100000=_^`0000001ENo00<005[kF_/0
15[k00<005IiEWT015Ii00<0058gDSL0158g00<0058g00000P00;goo10000goo000hOol00`00Oomo
o`03Ool00`00FYQJV005FYP00`00I_T00005J_P00`00LeUcF@03LeT01@00LmYcfW?J00000e_i1000
1UNo00<005[kF_/00e[k00<005IiEWT015Ii00@0058gDSMB=`D002ioo`@000Moo`00>Goo00<007oo
Ool00goo00<005ZHFYP00eZH00D006KiI_UVn@0000=Zn003001cFG=I00=cF@03001cfW?J0080009K
n@8000=ooaP0031oo`<000]oo`00>Woo0P000goo00<005ZHFYP00UZH00<006KiI_T00fKi00<006[h
00001G=I00@007?JLmX0009Kn@8004Uoo`@000ioo`00?7oo00D007ooOomoo`00009JV003001VnFKi
00EVn@8000AcF@03001cfW?J00@004Moo`@0019oo`00?Goo00H007ooOol005ZH0007I_T00`00Jia[
W00200000g=I000000040015Ool4000FOol003ioo`04001oogoo0007I_T00`00Jia[W002Ji`50002
Ool00`00Oomoo`0oOol4000JOol003moo`800003Ool006Ki00AVn@03001[W6^L009[W004001Kn@00
0004Ool00`00Oomoo`0kOol4000NOol0045oo`03001oo`0000=Vn@03001[W6^L009[W003001Kn@00
00Eoo`03001oogoo03Uoo`<0029oo`00@Woo00D007oo001Vn@0000A[W0<000Ioo`03001oogoo03Eo
o`@002Eoo`00@goo00<007oo00000P000V^L0P0027oo00<007ooOol0<Goo1000:Goo0014Ool20002
Ool2000:Ool00`00Oomoo`0]Ool4000]Ool004Ioo`03001oogoo00]oo`03001oogoo02Uoo`@0035o
o`00Agoo00<007ooOol02Woo00<007ooOol09Goo1000=Goo0018Ool00`00Oomoo`09Ool00`00Oomo
o`0QOol4000iOol004Uoo`8000Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007ooOol01Woo00<0
07ooOol06Woo1000@7oo001<Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool004eoo`03001o
ogoo00Aoo`03001oogoo019oo`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol03Woo1000C7oo
001?Ool20003Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001DOol0059oo`03
001oo`0000Eoo`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol00;Eoo`00]Goo
002eOol00;Eoo`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.234 0 0 r
.37963 .66718 .40338 .65368 .44019 .66271 .41742 .66235 Metetra
.145 .689 .802 r
.41742 .66235 .44019 .66271 .47787 .67766 .45585 .65919 Metetra
.541 .734 .964 r
.45585 .65919 .47787 .67766 .51666 .6936 .49502 .65613 Metetra
.602 .612 .833 r
.49502 .65613 .51666 .6936 .55658 .70511 .53494 .65154 Metetra
.618 .559 .766 r
.53494 .65154 .55658 .70511 .59741 .70741 .57556 .64402 Metetra
.629 .543 .738 r
.57556 .64402 .59741 .70741 .63877 .69749 .61679 .63271 Metetra
.643 .549 .733 r
.61679 .63271 .63877 .69749 .68023 .67492 .6585 .6175 Metetra
.662 .576 .748 r
.6585 .6175 .68023 .67492 .72145 .64196 .70059 .59901 Metetra
.689 .635 .791 r
.70059 .59901 .72145 .64196 .76238 .60302 .74305 .57852 Metetra
.722 .756 .88 r
.74305 .57852 .76238 .60302 .80324 .56357 .78593 .55764 Metetra
.659 .936 .967 r
.78593 .55764 .80324 .56357 .8445 .52892 .82939 .53803 Metetra
.82939 .53803 .8445 .52892 .88682 .5033 .87364 .52106 Metetra
.188 0 0 r
.87364 .52106 .88682 .5033 .93088 .48915 .9189 .50754 Metetra
.261 0 0 r
.9189 .50754 .93088 .48915 .97722 .48681 .96535 .49757 Metetra
.35464 .68432 .37963 .66718 .41742 .66235 .39371 .66375 Metetra
.705 .971 .929 r
.39371 .66375 .41742 .66235 .45585 .65919 .43322 .64037 Metetra
.755 .789 .88 r
.43322 .64037 .45585 .65919 .49502 .65613 .47302 .61656 Metetra
.702 .648 .793 r
.47302 .61656 .49502 .65613 .53494 .65154 .51312 .59474 Metetra
.66 .574 .747 r
.51312 .59474 .53494 .65154 .57556 .64402 .55361 .57687 Metetra
.631 .537 .729 r
.55361 .57687 .57556 .64402 .61679 .63271 .59467 .56412 Metetra
.608 .522 .731 r
.59467 .56412 .61679 .63271 .6585 .6175 .63652 .55666 Metetra
.59 .529 .755 r
.63652 .55666 .6585 .6175 .70059 .59901 .67932 .5536 Metetra
.57 .569 .813 r
.67932 .5536 .70059 .59901 .74305 .57852 .72318 .55317 Metetra
.522 .675 .934 r
.72318 .55317 .74305 .57852 .78593 .55764 .76807 .55299 Metetra
.23 .733 .9 r
.76807 .55299 .78593 .55764 .82939 .53803 .8138 .55053 Metetra
.185 0 0 r
.8138 .55053 .82939 .53803 .87364 .52106 .86011 .54359 Metetra
.26 0 0 r
.86011 .54359 .87364 .52106 .9189 .50754 .90668 .53084 Metetra
.193 0 0 r
.90668 .53084 .9189 .50754 .96535 .49757 .95328 .51208 Metetra
.716 .981 .783 r
.32866 .69768 .35464 .68432 .39371 .66375 .36917 .66252 Metetra
.875 .93 .856 r
.36917 .66252 .39371 .66375 .43322 .64037 .40994 .62076 Metetra
.83 .785 .809 r
.40994 .62076 .43322 .64037 .47302 .61656 .45062 .5785 Metetra
.762 .678 .769 r
.45062 .5785 .47302 .61656 .51312 .59474 .49113 .54166 Metetra
.698 .604 .748 r
.49113 .54166 .51312 .59474 .55361 .57687 .53166 .51488 Metetra
.638 .552 .741 r
.53166 .51488 .55361 .57687 .59467 .56412 .57263 .50085 Metetra
.576 .514 .749 r
.57263 .50085 .59467 .56412 .63652 .55666 .61454 .49994 Metetra
.504 .492 .777 r
.61454 .49994 .63652 .55666 .67932 .5536 .65784 .51017 Metetra
.402 .488 .833 r
.65784 .51017 .67932 .5536 .72318 .55317 .70279 .52739 Metetra
.211 .505 .908 r
.70279 .52739 .72318 .55317 .76807 .55299 .74932 .54586 Metetra
0 .442 .746 r
.74932 .54586 .76807 .55299 .8138 .55053 .79703 .55917 Metetra
.278 0 0 r
.79703 .55917 .8138 .55053 .86011 .54359 .84522 .56161 Metetra
.189 0 0 r
.84522 .56161 .86011 .54359 .90668 .53084 .89308 .54955 Metetra
.89308 .54955 .90668 .53084 .95328 .51208 .93993 .52248 Metetra
.887 .943 .844 r
.30219 .69996 .32866 .69768 .36917 .66252 .34401 .65449 Metetra
.891 .852 .797 r
.34401 .65449 .36917 .66252 .40994 .62076 .38603 .59997 Metetra
.856 .775 .772 r
.38603 .59997 .40994 .62076 .45062 .5785 .42767 .54512 Metetra
.804 .711 .763 r
.42767 .54512 .45062 .5785 .49113 .54166 .46876 .49816 Metetra
.737 .653 .765 r
.46876 .49816 .49113 .54166 .53166 .51488 .50956 .46541 Metetra
.652 .595 .777 r
.50956 .46541 .53166 .51488 .57263 .50085 .55061 .45041 Metetra
.539 .533 .797 r
.55061 .45041 .57263 .50085 .61454 .49994 .59262 .45367 Metetra
.385 .465 .82 r
.59262 .45367 .61454 .49994 .65784 .51017 .63623 .47256 Metetra
.177 .39 .833 r
.63623 .47256 .65784 .51017 .70279 .52739 .68185 .50153 Metetra
0 .325 .807 r
.68185 .50153 .70279 .52739 .74932 .54586 .72947 .53266 Metetra
0 .321 .727 r
.72947 .53266 .74932 .54586 .79703 .55917 .77856 .55686 Metetra
.154 0 0 r
.77856 .55686 .79703 .55917 .84522 .56161 .82815 .5658 Metetra
.82815 .5658 .84522 .56161 .89308 .54955 .87709 .55411 Metetra
.585 .934 .762 r
.87709 .55411 .89308 .54955 .93993 .52248 .92438 .52101 Metetra
.859 .816 .806 r
.27577 .68574 .30219 .69996 .34401 .65449 .31851 .63664 Metetra
.869 .789 .77 r
.31851 .63664 .34401 .65449 .38603 .59997 .36147 .5777 Metetra
.859 .769 .761 r
.36147 .5777 .38603 .59997 .42767 .54512 .40395 .51853 Metetra
.832 .751 .774 r
.40395 .51853 .42767 .54512 .46876 .49816 .44571 .46809 Metetra
.779 .728 .806 r
.44571 .46809 .46876 .49816 .50956 .46541 .48699 .43319 Metetra
.674 .688 .855 r
.48699 .43319 .50956 .46541 .55061 .45041 .52836 .41762 Metetra
.481 .601 .895 r
.52836 .41762 .55061 .45041 .59262 .45367 .57059 .42194 Metetra
.197 .451 .875 r
.57059 .42194 .59262 .45367 .63623 .47256 .61442 .44334 Metetra
0 .303 .796 r
.61442 .44334 .63623 .47256 .68185 .50153 .66035 .47583 Metetra
0 .257 .756 r
.66035 .47583 .68185 .50153 .72947 .53266 .70842 .51081 Metetra
0 .36 .817 r
.70842 .51081 .72947 .53266 .77856 .55686 .75814 .53832 Metetra
.217 .603 .95 r
.75814 .53832 .77856 .55686 .82815 .5658 .80847 .54914 Metetra
.588 .827 .99 r
.80847 .54914 .82815 .5658 .87709 .55411 .85815 .53726 Metetra
.807 .894 .905 r
.85815 .53726 .87709 .55411 .92438 .52101 .90608 .50184 Metetra
.797 .715 .774 r
.24992 .65299 .27577 .68574 .31851 .63664 .29291 .60782 Metetra
.829 .738 .763 r
.29291 .60782 .31851 .63664 .36147 .5777 .33625 .55382 Metetra
.847 .766 .773 r
.33625 .55382 .36147 .5777 .40395 .51853 .37925 .49948 Metetra
.847 .802 .807 r
.37925 .49948 .40395 .51853 .44571 .46809 .42162 .45277 Metetra
.81 .845 .876 r
.42162 .45277 .44571 .46809 .48699 .43319 .46352 .41984 Metetra
.666 .856 .973 r
.46352 .41984 .48699 .43319 .52836 .41762 .50546 .40416 Metetra
.307 .707 .977 r
.50546 .40416 .52836 .41762 .57059 .42194 .54814 .40619 Metetra
0 .436 .831 r
.54814 .40619 .57059 .42194 .61442 .44334 .59224 .42339 Metetra
0 .304 .761 r
.59224 .42339 .61442 .44334 .66035 .47583 .63824 .45035 Metetra
0 .34 .813 r
.63824 .45035 .66035 .47583 .70842 .51081 .68626 .47934 Metetra
.191 .466 .886 r
.68626 .47934 .70842 .51081 .75814 .53832 .73588 .50157 Metetra
.451 .584 .895 r
.73588 .50157 .75814 .53832 .80847 .54914 .78625 .50895 Metetra
.632 .66 .859 r
.78625 .50895 .80847 .54914 .85815 .53726 .83629 .49627 Metetra
.745 .708 .817 r
.83629 .49627 .85815 .53726 .90608 .50184 .885 .4628 Metetra
.731 .639 .756 r
.22487 .6037 .24992 .65299 .29291 .60782 .26737 .56912 Metetra
.773 .69 .772 r
.26737 .56912 .29291 .60782 .33625 .55382 .31038 .52842 Metetra
.811 .764 .809 r
.31038 .52842 .33625 .55382 .37925 .49948 .35338 .48719 Metetra
.829 .868 .874 r
.35338 .48719 .37925 .49948 .42162 .45277 .39614 .45087 Metetra
.745 .966 .934 r
.39614 .45087 .42162 .45277 .46352 .41984 .43873 .42377 Metetra
.387 .848 .825 r
.43873 .42377 .46352 .41984 .50546 .40416 .48149 .40835 Metetra
.01 0 0 r
.48149 .40835 .50546 .40416 .54814 .40619 .52489 .40498 Metetra
.167 0 0 r
.52489 .40498 .54814 .40619 .59224 .42339 .56943 .41179 Metetra
0 .479 .816 r
.56943 .41179 .59224 .42339 .63824 .45035 .61548 .42495 Metetra
.224 .546 .929 r
.61548 .42495 .63824 .45035 .68626 .47934 .66314 .43916 Metetra
.434 .555 .878 r
.66314 .43916 .68626 .47934 .73588 .50157 .71218 .44855 Metetra
.55 .558 .816 r
.71218 .44855 .73588 .50157 .78625 .50895 .76207 .44789 Metetra
.625 .574 .778 r
.76207 .44789 .78625 .50895 .83629 .49627 .8121 .43388 Metetra
.682 .603 .761 r
.8121 .43388 .83629 .49627 .885 .4628 .86164 .40605 Metetra
.658 .576 .752 r
.20048 .54337 .22487 .6037 .26737 .56912 .24184 .52358 Metetra
.687 .637 .794 r
.24184 .52358 .26737 .56912 .31038 .52842 .28381 .50172 Metetra
.722 .751 .875 r
.28381 .50172 .31038 .52842 .35338 .48719 .32626 .47945 Metetra
.701 .917 .968 r
.32626 .47945 .35338 .48719 .39614 .45087 .36909 .45846 Metetra
.384 .841 .77 r
.36909 .45846 .39614 .45087 .43873 .42377 .41232 .44014 Metetra
.41232 .44014 .43873 .42377 .48149 .40835 .45606 .42531 Metetra
.08 0 0 r
.45606 .42531 .48149 .40835 .52489 .40498 .50048 .4141 Metetra
.50048 .4141 .52489 .40498 .56943 .41179 .54575 .40586 Metetra
.339 .798 .94 r
.54575 .40586 .56943 .41179 .61548 .42495 .59201 .39934 Metetra
.557 .73 .955 r
.59201 .39934 .61548 .42495 .66314 .43916 .6393 .39285 Metetra
.597 .612 .838 r
.6393 .39285 .66314 .43916 .71218 .44855 .68759 .38462 Metetra
.608 .557 .771 r
.68759 .38462 .71218 .44855 .76207 .44789 .73674 .37313 Metetra
.62 .538 .741 r
.73674 .37313 .76207 .44789 .8121 .43388 .78655 .35747 Metetra
.635 .545 .736 r
.78655 .35747 .8121 .43388 .86164 .40605 .83685 .33749 Metetra
.57 .52 .76 r
.17624 .4795 .20048 .54337 .24184 .52358 .21605 .47546 Metetra
.546 .565 .825 r
.21605 .47546 .24184 .52358 .28381 .50172 .25651 .47409 Metetra
.497 .677 .946 r
.25651 .47409 .28381 .50172 .32626 .47945 .29792 .47295 Metetra
.284 .76 .933 r
.29792 .47295 .32626 .47945 .36909 .45846 .34047 .46944 Metetra
.011 0 0 r
.34047 .46944 .36909 .45846 .41232 .44014 .38421 .46133 Metetra
.065 0 0 r
.38421 .46133 .41232 .44014 .45606 .42531 .42902 .44724 Metetra
.42902 .44724 .45606 .42531 .50048 .4141 .4747 .42698 Metetra
.372 .811 .655 r
.4747 .42698 .50048 .4141 .54575 .40586 .52101 .40157 Metetra
.725 .966 .938 r
.52101 .40157 .54575 .40586 .59201 .39934 .56777 .37306 Metetra
.756 .798 .887 r
.56777 .37306 .59201 .39934 .6393 .39285 .61494 .34403 Metetra
.7 .654 .801 r
.61494 .34403 .6393 .39285 .68759 .38462 .66264 .31707 Metetra
.654 .574 .752 r
.66264 .31707 .68759 .38462 .73674 .37313 .71109 .29428 Metetra
.621 .531 .732 r
.71109 .29428 .73674 .37313 .78655 .35747 .76062 .27691 Metetra
.597 .515 .733 r
.76062 .27691 .78655 .35747 .83685 .33749 .81156 .26512 Metetra
.45 .466 .784 r
.15135 .41982 .17624 .4795 .21605 .47546 .18958 .4293 Metetra
.299 .444 .839 r
.18958 .4293 .21605 .47546 .25651 .47409 .2284 .44593 Metetra
.05 .434 .872 r
.2284 .44593 .25651 .47409 .29792 .47295 .26856 .46381 Metetra
0 .393 .686 r
.26856 .46381 .29792 .47295 .34047 .46944 .31059 .47641 Metetra
.186 0 0 r
.31059 .47641 .34047 .46944 .38421 .46133 .35466 .47785 Metetra
.35466 .47785 .38421 .46133 .42902 .44724 .4005 .46442 Metetra
.377 .8 .611 r
.4005 .46442 .42902 .44724 .4747 .42698 .44754 .43558 Metetra
.747 .992 .811 r
.44754 .43558 .4747 .42698 .52101 .40157 .49512 .39418 Metetra
.876 .941 .856 r
.49512 .39418 .52101 .40157 .56777 .37306 .54272 .34564 Metetra
.841 .806 .817 r
.54272 .34564 .56777 .37306 .61494 .34403 .59016 .2966 Metetra
.769 .692 .777 r
.59016 .2966 .61494 .34403 .66264 .31707 .63765 .25339 Metetra
.697 .608 .753 r
.63765 .25339 .66264 .31707 .71109 .29428 .68571 .22094 Metetra
.629 .548 .744 r
.68571 .22094 .71109 .29428 .76062 .27691 .73509 .20203 Metetra
.563 .508 .751 r
.73509 .20203 .76062 .27691 .81156 .26512 .78652 .19705 Metetra
.267 .41 .821 r
.12485 .37076 .15135 .41982 .18958 .4293 .16191 .38903 Metetra
0 .26 .77 r
.16191 .38903 .18958 .4293 .2284 .44593 .19939 .41759 Metetra
0 .15 .646 r
.19939 .41759 .2284 .44593 .26856 .46381 .23847 .44837 Metetra
.381 0 0 r
.23847 .44837 .26856 .46381 .31059 .47641 .27999 .47208 Metetra
.152 0 0 r
.27999 .47208 .31059 .47641 .35466 .47785 .32426 .48014 Metetra
.335 .818 .842 r
.32426 .48014 .35466 .47785 .4005 .46442 .37096 .46702 Metetra
.743 .988 .891 r
.37096 .46702 .4005 .46442 .44754 .43558 .41926 .43193 Metetra
.887 .954 .842 r
.41926 .43193 .44754 .43558 .49512 .39418 .46816 .37913 Metetra
.904 .877 .799 r
.46816 .37913 .49512 .39418 .54272 .34564 .51683 .31662 Metetra
.873 .801 .776 r
.51683 .31662 .54272 .34564 .59016 .2966 .56494 .25391 Metetra
.817 .73 .769 r
.56494 .25391 .59016 .2966 .63765 .25339 .61271 .19978 Metetra
.741 .663 .773 r
.61271 .19978 .63765 .25339 .68571 .22094 .66084 .16083 Metetra
.646 .596 .784 r
.66084 .16083 .68571 .22094 .73509 .20203 .71031 .14073 Metetra
.532 .531 .799 r
.71031 .14073 .73509 .20203 .78652 .19705 .76217 .13994 Metetra
0 .354 .828 r
.09584 .33647 .12485 .37076 .16191 .38903 .13248 .35723 Metetra
0 .112 .626 r
.13248 .35723 .16191 .38903 .19939 .41759 .16942 .38932 Metetra
0 .056 .547 r
.16942 .38932 .19939 .41759 .23847 .44837 .20802 .42398 Metetra
0 .21 .668 r
.20802 .42398 .23847 .44837 .27999 .47208 .24934 .45102 Metetra
.125 .565 .922 r
.24934 .45102 .27999 .47208 .32426 .48014 .2938 .46094 Metetra
.599 .823 .985 r
.2938 .46094 .32426 .48014 .37096 .46702 .3411 .44757 Metetra
.806 .86 .89 r
.3411 .44757 .37096 .46702 .41926 .43193 .39033 .41004 Metetra
.871 .84 .812 r
.39033 .41004 .41926 .43193 .46816 .37913 .44032 .35311 Metetra
.887 .816 .773 r
.44032 .35311 .46816 .37913 .51683 .31662 .49007 .28567 Metetra
.878 .795 .764 r
.49007 .28567 .51683 .31662 .56494 .25391 .53912 .21818 Metetra
.847 .774 .78 r
.53912 .21818 .56494 .25391 .61271 .19978 .58763 .1602 Metetra
.785 .744 .816 r
.58763 .1602 .61271 .19978 .66084 .16083 .63633 .11882 Metetra
.672 .692 .861 r
.63633 .11882 .66084 .16083 .71031 .14073 .68627 .09793 Metetra
.495 .602 .889 r
.68627 .09793 .71031 .14073 .76217 .13994 .73859 .09802 Metetra
0 .338 .726 r
.06358 .31839 .09584 .33647 .13248 .35723 .10084 .33481 Metetra
0 .143 .582 r
.10084 .33481 .13248 .35723 .16942 .38932 .13838 .36117 Metetra
0 .175 .657 r
.13838 .36117 .16942 .38932 .20802 .42398 .17751 .38964 Metetra
.001 .375 .839 r
.17751 .38964 .20802 .42398 .24934 .45102 .21924 .41118 Metetra
.388 .571 .91 r
.21924 .41118 .24934 .45102 .2938 .46094 .26402 .41751 Metetra
.625 .664 .868 r
.26402 .41751 .2938 .46094 .3411 .44757 .31165 .40325 Metetra
.745 .706 .815 r
.31165 .40325 .3411 .44757 .39033 .41004 .36129 .36766 Metetra
.809 .734 .782 r
.36129 .36766 .39033 .41004 .44032 .35311 .41186 .31487 Metetra
.845 .761 .769 r
.41186 .31487 .44032 .35311 .49007 .28567 .46243 .25264 Metetra
.863 .79 .778 r
.46243 .25264 .49007 .28567 .53912 .21818 .51246 .19017 Metetra
.859 .825 .814 r
.51246 .19017 .53912 .21818 .58763 .1602 .56205 .13598 Metetra
.812 .856 .881 r
.56205 .13598 .58763 .1602 .63633 .11882 .61177 .09649 Metetra
.676 .846 .965 r
.61177 .09649 .63633 .11882 .68627 .09793 .66257 .07523 Metetra
.421 .731 .99 r
.66257 .07523 .68627 .09793 .73859 .09802 .71549 .07264 Metetra
.138 0 0 r
.02767 .31503 .06358 .31839 .10084 .33481 .06675 .32079 Metetra
0 .42 .748 r
.06675 .32079 .10084 .33481 .13838 .36117 .10623 .33301 Metetra
.091 .491 .899 r
.10623 .33301 .13838 .36117 .17751 .38964 .14706 .34628 Metetra
.359 .533 .891 r
.14706 .34628 .17751 .38964 .21924 .41118 .18999 .3546 Metetra
.519 .551 .828 r
.18999 .3546 .21924 .41118 .26402 .41751 .23536 .3526 Metetra
.614 .573 .785 r
.23536 .3526 .26402 .41751 .31165 .40325 .28305 .33691 Metetra
.68 .605 .764 r
.28305 .33691 .31165 .40325 .36129 .36766 .33251 .30706 Metetra
.734 .648 .763 r
.33251 .30706 .36129 .36766 .41186 .31487 .38301 .26562 Metetra
.781 .706 .78 r
.38301 .26562 .41186 .31487 .46243 .25264 .43388 .21758 Metetra
.82 .784 .818 r
.43388 .21758 .46243 .25264 .51246 .19017 .4847 .16897 Metetra
.831 .88 .878 r
.4847 .16897 .51246 .19017 .56205 .13598 .53547 .12563 Metetra
.758 .957 .937 r
.53547 .12563 .56205 .13598 .61177 .09649 .58656 .09206 Metetra
.532 .916 .928 r
.58656 .09206 .61177 .09649 .66257 .07523 .63861 .07082 Metetra
.267 .769 .875 r
.63861 .07082 .66257 .07523 .71549 .07264 .69235 .06223 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo01=oo`04001oogooOol3000[Ool006Qoo`<001Aoo`H0
00Aoo`03001oogoo00Uoo`@001aoo`00IWoo0P004Woo1@001B<K0`000goo00<007ooOol02Goo00<0
07ooOol077oo001SOol3000BOol20003Akd;0002Ool00`00Oomoo`02Ool50003Ool00`00Oomoo`0K
Ool004Moo`8001Qoo`80019oo`<000=7_@80009GOP03000focKo008fo`04000focKo0003Ool00`00
Oomoo`0:Ool00`00Oomoo`0JOol004Ioo`04001oogoo000DOol3000BOol20004Akd20005Egh01@00
=_lfocKo00000cKo00D007ooOomoo`0000Qoo`04001oogoo000LOol004Ioo`04001oogoo000BOol2
000COol40002Akd20008Egh01000=_lfo`0013Ko00@007oo000000Uoo`8001eoo`00AWoo00@007oo
Ool000moo`<00003Ool007oo015oo`03001SgF?M009Sg@<000UGOPX0009oo`03001oogoo02Eoo`00
AWoo00@007ooOol000eoo`8001Eoo`8000ASg@05001WO6MlIg`00005Egh3000015KK000nO0001Cil
00D007ooOomoo`0002Ioo`00Agoo0P002goo0`005Goo0P001F?M00<006MlIg`00fMl00<005Mn0000
0P001EKK0P001Sil00@007ooOol002Ioo`00DWoo0`005Woo00<006?MHmd00f?M0P001VMl0P0025KK
0P001cil00@007ooOol002Eoo`00Cgoo0`005goo0`0016?M00<006MlIg`01FMl00<006Kj00002EKK
00<003il?W`01Cil00D007ooOomoo`0002Aoo`00C7oo0`006Goo00@006^LJia[W0800003Hmd006Ml
00IWO005001VnVKjI_X00008E]/20007?W`01000Oomoo`0097oo001:Ool3000JOol00`00Jia[W004
Ji`00`00IgaWO005Ig`00`00I_YVnP02I_X00`00E]]Ff`06E]/20004?W`50002Ool00`00Oomoo`0Q
Ool004Moo`<001aoo`03001[W6^L00A[W003001_FP0000EWO003001VnVKj00AVnP03001FfeKK00EF
f`H000A66@05001oogooOol0000ROol004Eoo`8001ioo`03001[W6^L00A[W005001_FVmJKeX00003
Ig`00`00I_YVnP06I_X00`00E]]Ff`02E]/300000e9i00166@07AQT01000Oomoo`008Woo0012Ool3
000OOol00`00Jia[W004Ji`00`00KeY_FP03KeX00`00Ig`00009I_X00`00E]/000020002DWT01000
DWUBN@001dHI00D007ooOomoo`00025oo`009Goo10005goo0P008Goo00<006^LJi`016^L00<006mJ
KeX01FmJ00<006KjI_X026Kj0P001E9i00@0059iDWT000M66@03001oogoo009oo`03001oogoo01io
o`009Woo00<007ooOol057oo0`008Woo0P001F^L00<006mJKeX01VmJ00<006KjI_X01fKj00<005jh
00001E9i00D0059iDWUBN@0000M66@03001oogoo0080021oo`007goo1@000goo00<007ooOol04Goo
0P0097oo00@006/jJcX000=[W003001_FVmJ00I_FP03001_600000MVnP05001N^5jhG[P00004DWT0
1@00DWUBNE9i00001dHI00<007ooOol00Woo00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000U
Ool00`00JcY[>P02JcX00`00Ji`00008KeX01@00KaQ_66lH000016Kj0P0015jh00<0059iDWT00U9i
00D0059iDWUBN@0000Q66@05001oogooOol0000OOol002Eoo`04001oogoo000=Ool2000WOol00`00
JcY[>P04JcX00`00KeY_FP06KeX00`00KaQ_6002KaP01@00I_YVnVKj00001ejh00<0059iDWT00U9i
00D0059iDWUBN@0000M66@03001oogoo009oo`03001oogoo01aoo`009Woo0P002goo0`00:7oo00<0
06/jJcX01F/j0P001VmJ00<006lHKaP016lH00<006Kj000025jh00<0059iDWT00U9i00D0059iDWUB
N@0000M66@03001oogoo00=oo`03001oogoo01]oo`00<Goo0P0000=oo`00Ool09goo00<006/jJcX0
1F/j00@006lhKcP000E_FP03001_66lH00E_6003001N^5jh00MN^003001BNE9i009BN@04001BNE9i
DWT:0004Ool00`00Oomoo`0KOol002ioo`<002]oo`03001[>V/j00E[>P03001_>6lh009_>005001_
FVmJKeX00008KaP00`00G[QN^008G[P01000DWUBNE9i100000=B=`00BQP01TXH00<007ooOol00Woo
0P0077oo000[Ool3000]Ool00`00JcY[>P06JcX00`00KcQ_>003KcP00`00KeX00008KaP00`00J_P0
0009G[P00`00DWT000020005DSL00`00BQQ:6005BQP00`00Oomoo`03Ool00`00Oomoo`03Ool2000D
Ool002Uoo`80009oo`03001oogoo02Uoo`8000Q[>P03001_>6lh00E_>003001_66lH00I_6004001Z
n6[h0009G[P20002DSL00`00DSMB=`03DSL00`00BQQ:6005BQP00`00Oomoo`03Ool00`00Oomoo`02
Ool01000Oomoo`004goo000VOol3000_Ool01000H]QRf0001V/j00<006lhKcP01Vlh00<006lHKaP0
1FlH00<006[hJ_P00V[h00<005jhG[P01Ejh00<005Yh00000U8g00<0058gDSL00e8g00<004XHBQP0
1DXH00<007ooOol017oo00D007ooOomoo`00009oo`03001oogoo015oo`0097oo0P00<Goo00<006;H
H]P00V;H00<006/jJcX00V/j00<006lhKcP01Vlh00<007<h00001FlH00<006[hJ_P00f[h00<005jh
G[P00ejh0P000eYh00<0058g00001U8g00<004XHBQP01DXH00<007ooOol017oo00@007ooOol0009o
o`03001oogoo015oo`008Goo0`00<Woo00<006;HH]P016;H00D006/jJcY[>P0000M_>005001c>7<h
LcP00004KaP00`00J_QZn004J_P00`00G[QN^0020005FWP00`00DSL00006DSL00`00BQQ:6005BQP0
0`00Oomoo`03Ool20002Ool01000Oomoo`004goo000OOol2000dOol00`00H]QRf006H]P00`00JcX0
0007KcP00`00LcQc>002LcP01@00KaQ_66lH00001f[h00<005jh00001eYh0`001U8g00<004XHBQP0
1TXH00<007ooOol017oo00<007ooOol00P0057oo000LOol3000eOol00`00H]QRf008H]P00`00KcQ_
>006KcP00`00LcQc>003LcP00`00KaP00009J_P00`00FWQJN004FWP50007DSL00`00BQQ:6005BQP0
0`00Oomoo`05Ool00`00Oomoo`0EOol001]oo`8003Ioo`03001Rf6;H00URf003001_>6lh00E_>003
001c>7<h00Ec>003001Zn6[h00MZn08000=JN0<000=JN003001B=`0000IB=`03001:64XH00E:6003
001oogoo00Eoo`03001oogoo01Eoo`0057oo0`0017oo00<007oo0000<goo0P002f;H00<006lH0000
1Flh00<007<hLcP01W<h00<006[hJ_P01V[h00<006;HH]P00`001UYh00@0058gDSL000EB=`03001:
64XH00A:60<000Moo`03001oogoo01Aoo`005Goo00<007ooOol00goo00@007ooOol0035oo`03001N
V00000YRf005001_66lHKaP00004KcP00`00LcQc>005LcP00`00KcP00006J_P50009FWP01000DSMB
=`001E8g00<004XH00000`000ThG00<007ooOol01Goo0P005Goo000>Ool50002Ool00`00Oomoo`03
Ool01@00Oomoogoo0000;goo00D005jHGYQNV00000QRf003001_66lH009_6005001_>6lhKcP00007
LcP01@00KcQ_>6lh00000f[h0`000f;H00<005YhFWP01eYh00@0058gDSL000AB=`<000M>5`03001o
ogoo00Eoo`03001oogoo01=oo`005Goo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0:goo
00<005jHGYP00UjH00<006;HH]P01F;H00<006lHKaP016lH00<006lh00001g<h00<006lhKcP00Vlh
1@001V;H00<005YhFWP01UYh00@0058gDSMB=`@00003CQL004hG00M>5`03001oogoo00Ioo`03001o
ogoo019oo`005Goo00<007ooOol00Woo0P001Goo00<007ooOol0:Goo00<005jHGYP015jH00<006;H
H]P00f;H00<006lHKaP01VlH00<007<hLcP01G<h00@006lhKcQ_>0@00003J_P006;H00IRf003001J
N5Yh00MJN0D000=>5`03001>5dhG00I>5`03001oogoo00Ioo`03001oogoo019oo`0057oo0P0017oo
00<007ooOol01Goo00<007ooOol09goo00<005jHGYP01EjH00<006;HH]P00V;H00<006lHKaP01flH
00<007<hLcP01G<h00@006lh000000A_>003001Rf6;H00IRf003001JN5Yh00IJN08000=>5`03001>
5dhG009>5`03001>5dhG00I>5`03001oogoo00Ioo`03001oogoo015oo`006Woo00<007ooOol01Woo
00<007ooOol09Goo00<005jHGYP01ejH00@006;HH]P000U_6003001cF00000Ec>0<000E_>08000UR
f003001JN5Yh00=JN0800003DUP004hG00=>5`05001>5dhGCQL00008CQL00`00Oomoo`07Ool00`00
Oomoo`0@Ool001Yoo`03001oogoo00Moo`03001oogoo029oo`8000YNV003001Rf00000U_6006001c
F7=HLeP007<h0`001flh00@006/jJcX000QRf004001JN5YhFWP20003DUP00`00CQM>5`03CQL01000
CQM>5`0024hG00<007ooOol01Woo0P004Woo000IOol50007Ool00`00Oomoo`0POol00`00EWP0000;
GYP00`00KaQ_6007KaP01000LeQcF7=H0`0000=c>000KcP01Vlh00D006/jJcY[>P0000QRf004001J
N0000006DUP00`00CQM>5`03CQL01000CQM>5`0024hG00<007ooOol01goo00<007ooOol03goo0008
Ool20004Ool20003Ool30003Ool20002Ool60003Ool00`00Oomoo`0MOol20003EWP00`00GYQNV008
GYP00`00KaQ_6006KaP00`00LeP000020003LeP00`00KcQ_>005KcP00`00JcY[>P03JcX00`00H]QR
f005H]P20008DUP00`00CQM>5`03CQL01000CQM>5`0024hG00<007ooOol027oo00<007ooOol03Woo
0007Ool01000Oomoo`000goo0P001Woo00@007ooOol000Moo`D00003Ool007oo01]oo`8000EFN003
001NV5jH00MNV003001Vn@0000I_60<000IcF08000E_>003001[>V/j00A[>P03001Rf6;H00=Rf080
0003FYT0059H00MBF003001>5dhG00A>5`03001>5`0000Q>5`03001oogoo00Qoo`03001oogoo00io
o`0000=oo`0000000`0000Eoo`00Oomoo`0000]oo`04001oogoo0009Ool00`000EL00003000JOol0
0`00EWQFN005EWP00`00GYQNV006GYP01@00I_UVnFKi00000VlH10001g=H00@007>ILiT000=_>003
001[>V/j00I[>P05001Rf6;HH]P00004FYT00`00DUQBF006DUP00`00CQM>5`04CQL00`00CQL00008
CQL00`00Oomoo`08Ool00`00Oomoo`0=Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`08
Ool200000`5G0009o003000DOol30009EWP00`00GYQNV004GYP00`00I_UVn@02I_T30002KaP00`00
LeQcF005LeP00`00LiUcV@02LiT00`00KcP00009JcX01000H]P000001EZI00<0059HDUP01U9H00<0
04hGCQL014hG00<004hG00001dhG0P002goo00<007ooOol00Woo10001Woo0007Ool01000Oomoo`00
27oo00<007ooOol00goo00<007ooOol02Goo10000`Wl0`003Goo10000TiI00<005IhEWP01eIh00<0
05jHGYP00ejH00@006KiI_UVn@80009Vn@03001_600000McF003001cVG>I00AcV@03001[>V/j00Q[
>P8000MJV@03001BF59H00IBF003001>5dhG00E>5`8000A>5`<00003DSL007oo00Yoo`03001oogoo
00=oo`03001oogoo00Ioo`0027oo0P002Goo10000Woo00<007ooOol02Woo00@0005j00000089o003
000^?2hl00<000Eoo`D000M>F@03001FN5Ih00MFN006001NV5jHGYP006Ki0`001FKi00<007=HLeP0
1G=H00<007>ILiT01G>I00<006/jJcX01f/j00<006<l000025ZI00<0059HDUP01U9H00<004hGCQL0
1DhG1@001E8g00<007ooOol02Goo00<007ooOol00goo00<007ooOol01Goo000HOol2000=Ool01000
0GX1NP000PWl00<002hl;S`00Rhl1@000d8j00<004iICUT01TiI00<005IhEWP01eIh00<005jHGYP0
0`0026Ki0P001G=H00<007>ILiT01G>I00<007?K00001V/j0P000f<l00<005ZIFYT01EZI00<0059H
DUP01U9H00<004hGCQL01DhG0P00258g00<007ooOol02Woo00<007ooOol00goo00<007ooOol017oo
000HOol2000>Ool00`000GX1NP0200000`Wl000^?003;S`00`00@SY2>P05@SX00`00CUU>F@06CUT0
0`00EWQFN006EWP3000:I_T01000KeU_F@000g=H00<007>ILiT01G>I00D007?KLm]cf`0000=[>P80
00ES?003001JVEZI00IJV@03001BF59H00EBF003001>5dhG009>5`<00003CSL0058g00MB=`03001o
ogoo00Uoo`80009oo`04001oogoo0006Ool0019oo`8000Aoo`03001oogoo00ioo`8000050GX00000
2O`00004;S`00`00@SY2>P04@SX00`00CUU>F@06CUT00`00EWQFN003EWP5000:I_T00`00KeU_F@02
KeT00`00LeP00007LiT00`00Lm]cf`03Lm/00`00JcX00008Hc`00`00FYUJV@05FYT00`00DUQBF006
DUP40005CSL00`00DSMB=`06DSL00`00Oomoo`0:Ool00`00Oomoo`020007Ool0015oo`04001oogoo
0003Ool00`00Oomoo`0>Ool00`008KX000040004;S`00`00@SY2>P05@SX00`00CUU>F@06CUT00`00
EWP000020002EWP00`00G]X00009I_T00`00KeU_F@04KeT20006LiT00`00Lm]cf`04Lm/20009Hc`0
0`00FYUJV@05FYT00`00DUQBF002DUP50008CSL20007DSL00`00Oomoo`0;Ool00`00Oomoo`08Ool0
015oo`04001oogoo0003Ool00`00Oomoo`0?Ool00`008KXQ^P040004;S`00`00@SY2>P04@SX00`00
CUU>F@05CUT30003EWP20003G]X00`00I_UVn@05I_T00`00KeU_F@05KeT01000LmX000000g>I00<0
07?KLm/00g?K0P000UoN00<006<lHc`01f<l00<005ZIFYT00eZI1P0014hh00<004hgCSL01Thg00<0
058g00001U8g00<007ooOol02goo00<007ooOol027oo000AOol01000Oomoo`000Woo0P004Goo00<0
026j8KX00R6j10000bhl00<0048j@SX0148j00<004iICUT00TiI0P000TiI00@005IhEWP000INfP03
001VnFKi00=Vn@03001_FFmI00E_F@03001cfW?J009cfP03001cV@0000Ecf`03001OgUoN00=OgP03
001S?6<l00IS?003001JVEZI00<0009>NP03001>>4hh00I>>003001>=dhg00I>=`8000MB=`03001o
ogoo00]oo`03001oogoo00Moo`004Goo00@007ooOol000=oo`03001oogoo011oo`08000Q^R6j8KX0
004H0001NP800003;S`0048j00E2>P03001>F@00008000A>F@03001FN00000QNfP05001VnFKiI_T0
0007KeT00`00LmYcfP03LmX40002Lm/20007Gmh00`00HcaS?002Hc`60006CWX00`00CSQ>>005CSP0
0`00CSM>=`06CSL00`00DSL00006DSL00`00Oomoo`0;Ool00`00Oomoo`07Ool0019oo`8000=oo`80
019oo`03000Q^R6j008Q^P03000160000081NP03000^?00000A2>P<000M>F@8000YNfP03001Vn@00
00M_F@03001cfW?J00=cfP03001OnEoi009On@@000EOgPL000A6oP03001>NTij00E>NP03001>>4hh
00E>>003001>=dhg00I>=`04001B=e8g0005DSL00`00Oomoo`0<Ool00`00Oomoo`06Ool001Moo`03
001oogoo019oo`03000Q^R6j008Q^P03000160000081NP800004@SX000000002@SX00`00CUU>F@02
CUT300000djk001NfP09G]X30006KeT00`00LmYcfP03LmX01000GoUOnEoi0P000Rld1`0012/n00<0
04KnA_h01DKn00<004ijCWX014ij00<004hhCSP01Thh00<004hgCSL01Thg00<0058g00001E8g0P00
3Woo00<007ooOol01Goo000GOol00`00Oomoo`0BOol00`008KXQ^P028KX01@000AP160000GX01`00
00=2>P00CUT00`001Djk00<005kJG]X01UkJ00D006MlIgaWO00000E_F@04001cfW?JLmX20002GoT6
00049]l60002:ch00`00A_i6oP04A_h00`00CWY>NP05CWX00`00CSQ>>005CSP00`00CSM>=`06CSL0
1000DSMB=`000U8g0P0000=FN000Ool03Goo00<007ooOol01Goo000GOol00`00Oomoo`0COol40000
0b6j0000000200020GX00`00<UdbG@02<Ud30008C[/00`00G]YNfP05G]X00`00IgaWO003Ig`01@00
KeU_FFmI00000g?J00<005oiGoT00`000eMo40001TKn00<004ijCWX014ij00<004hhCSP01Dhh00<0
04hgCSL01Thg00@0058gDSMB=`8000AFN003001oogoo00aoo`03001oogoo00Aoo`001Goo0P0017oo
0P000goo0`0017oo0P0057oo00<003WI>MT01@0000D160000001NP0000DbG@03001>^djk00M>^`03
001NfUkJ009NfP8000MWO003001_F@0000=cfPD0009GO`<0009Ff`03000nO3il00@nO003000If`00
008000030OX00000008000=6oP03001>NTij00A>NP03001>>4hh00I>>003001>=dhg00I>=`<00003
EWP005Ih00=FN003001oogoo00]oo`<000Eoo`0017oo00@007ooOol000=oo`8000Ioo`05001oogoo
Ool0000EOol30003>MT00`009MX000050005<Ud00`00C[]>^`07C[/01000G]YNfP002FMl20000fMl
0P001UKK00<003il?W`013il00<001WK6M/010000Qhm10001dij00<004hhCSP01Dhh00<004hgCSL0
14hg0`000eIh00<005IhEWP00eIh00<007ooOol01goo0`000Woo00<007ooOol00goo0004Ool01000
Oomoo`002goo00<007ooOol00P004goo0P0000=oo`00>MT013WI00<002GJ9MX00P0000<0U0000000
1C9M00<004jkC[/01Tjk00<005kJ00002FMl00@005olGoaOo08000EWO08000QFf`03000nO3il00@n
O003000IfaWK00@000@N?@@000A>NP03001>>4hh00E>>003001>=dhg00@0009>F003001FN5Ih009F
N003001FN5Ih009FN003001oogoo00Aoo`<000Eoo`03001oogoo00=oo`0017oo00@007ooOol000Qo
o`<000=oo`80011oo`<000Aoo`03000ifCWI008if@03000UfRGJ008UfP@000DbG@03001>^djk00A>
^`@000MWO080009Oo08000IWO004001RnF;i0008E]/00`00?W`nO003?W`00`006M/If`026M/30004
7Sd00`00=S`f?0050004CSP60007CUP00`00EWQFN002EWP01@00EWQFN5Ih00000Woo100027oo00<0
07ooOol00goo0004Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol037oo0`001goo00<0
03WI>MT00cWI00<002GJ9MX00RGJ200000@bG@00C[]>^`<000A?G`8000=WO08000=Oo0<000EWO003
001RnF;i009Rn@03001FfeKK00AFf`X0008If`030001>@00008000<N?@03000f?3Hl00<f?0D000A6
>P03001>F4iH00I>F003001FN5Ih00=FN004001FN5IhEWP2000<Ool00`00Oomoo`03Ool000Eoo`80
00Uoo`@0009oo`03001oogoo00Uoo`<000Yoo`03000ifCWI00<if@03000UfRGJ00<UfP030005^`Fk
00<000033Ud00000008000U?G`03001WO00000AOo005001[N`00KcT00003Ig`00`00H_URn@04H_T0
0`00E]]Ff`030002DWP00`00ASU6>@03AST00`00<MX0000300000`4i0000000200027Sd00`00=S`f
?003=S`00`00ASY6>P06ASX00`00CUQ>F005CUP00`00EWQFN003EWP01@00EWQFN5Ih00003Woo00<0
07ooOol00Woo000FOol00`00Oomoo`06Ool3000>Ool00`00>MTif@03>MT00`009MXUfP039MX01000
1K/5^`Fk0P0010iM00<004mOCel01TmO1`0000=[N`00KcT00Vli00<006Ml00001f;i0`001U9h00<0
04HiAST00dHi00@0037J<MXafP800081>@030001FP00008000Hf?0030016>THj00E6>P03001>F4iH
00I>F003001FN5Ih00=FN003001FN5Ih008000eoo`03001oogoo009oo`005Goo0P001Goo0`004Goo
00<003WI>MT00cWI00<002GJ9MX012GJ00@000Fk1K/5^`@0008>G@03001?GdmO009?G`L000=[N`03
001_>Fli00=_>@03001RnF;i00ERn@03001NV00000MBN0030016>DHi00=6>@03000afS7J00<afP80
00040CT0005J0EX20005=S`00`00ASY6>P05ASX00`00CUQ>F006CUP00`00EWQFN004EWP200000ekJ
001oo`0<Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000EOol@0004?[h400000dmO00000004
0005Jg/00`00KcU_>@04KcT20005H_T01@00GYQNV5jH00001U9h00<004HiAST014Hi00<0037J<MX0
0S7J00@001FJ0001>@800081FP8000@f?0030016>THj00E6>P03001>F4iH00I>F003001FN5Ih009F
N08000=NfP03001oogoo00Yoo`03001oogoo009oo`005Goo100067oo00<004XHBQP00dXH00<004IJ
AUX01DIJ00<003jn?[h013jn1`0000=K_P00Jg/016]k00<006liKcT016li00@006/HJaP000=Rn@03
001NV5jH009NV003001BN59h00EBN0030016>DHi00=6>@03000afS7J00<afP09000EVP0000016000
0EX001WL0080008f?0030016>THj00E6>P03001>F4iH00I>F003001FN5Ih0080009NfP04001NfUkJ
000<Ool00`00Oomoo`02Ool000ioo`<000Aoo`8001]oo`03001:64XH00=:60030016FTIJ00E6FP03
000n_Sjn008n_P03001O6elK00AO6`<000A[N`03001_>Fli00A_>@03001[66/H009[6003001Rn@00
00INV003001BN59h00ABN0030016>DHi00A6>@03000afS7J00<afP05000EVQFJ0001600300026M`2
00000cHl0016>P06ASX00`00CUQ>F005CUP00`00EWP00005G]X01000G]YNfP000Woo2P0017oo000?
Ool00`00Oomoo`05Ool00`00Oomoo`0HOol00`00BQQ:6003BQP00`00AUY6FP06AUX01000?[h00000
1UlK00@006LIIaUW6@800003Jg/006li00E_>@03001[66/H00A[6003001NV5jH00ENV003001BN59h
00ABN0030016>DHi00A6>@03000afS7J00<afP05000EVQFJ5IX000020AP200036M`20007ASX00`00
CUQ>F004CUP30007G]X00`00G]X00002000260000`00800P000300000goo001oo`03Ool000moo`03
001oogoo00Ioo`03001oogoo01Moo`03001:64XH00A:60030016FTIJ00E6FP8000MO6`03001W6FLI
00AW6@8000E_>@03001[66/H00E[6003001NV5jH00INV003001BN59h00=BN0d00003<MX001FJ008E
VP8000816003000Ig1WL008Ig0D00096>P03001>F4iH00@0009B^`03001NfUkJ00ENfP04001NfP00
00026004000260001000Oomoo`000goo000?Ool00`00Oomoo`07Ool00`00Oomoo`0GOol00`00BQQ:
6004BQP01000AUY6FTIJ0`000UJI00<005lKGa/00elK00<006LIIaT01FLI00<006lH00000fli00<0
06/HJaP01F/H00<006KH00001ejH00<0059hDWP00`0000=B=`00BQL01DXG00<0047h@OP00T7h1000
0QFJ0P000P4H0P000QWL00<003YL>U`00SYL1@001e:k00<005kJG]X01EkJ00<006NLIi`00`0011P0
00D007ooOomoo`0000=oo`003goo00<007ooOol027oo0P005goo00<004XHBQP014XH00@004IJ0000
00IFV@05001O6elKGa/00007IaT01`00KaQ_66lH001_>@0000M[6004001Vf6KH0007GYP30005DSL0
0`00BQM:5`04BQL00`00@OQ1n003@OP00`00<OX0000600001@5j000Ig1WL00001SYL00<005:kD[/0
1U:k00<005kJG]X015kJ00<006NL00000VNL00<001P060000QP00P000goo00<007ooOol00Goo000>
Ool2000<Ool00`00Oomoo`0DOol00`00BQQ:6004BQP20009EYT00`00Ga/00007IaT00`00KaQ_6003
KaP00`00JaQ[6005JaP00`00I]QVf002I]P00`00GYQNV002GYP200000eYg001B=`05DSL00`00BQM:
5`04BQL00`00@OQ1n004@OP01000<OXanS7j0P0000@1F@000001NP8000036M`003YL00DjG003001B
^e:k00IB^`04001NfUkJG]X20002Ii`00`00IiaWW0050005Ool00`00Oomoo`01Ool001eoo`03001o
ogoo01Aoo`03001:64XH009:6003001FF00000YFV@03001W6FLI00EW6@03001_66lH00A_608000E[
6003001Vf6KH00=Vf005001NV5jHGYP00003FWL00`00DSMB=`04DSL00`00BQM:5`04BQL00`00@OQ1
n004@OP00`00<OXanP03<OX20000105I0001NP5j0P001SYL00<005:kD[/01U:k00@005kJG]X000EW
W0800003BkP007oo00Moo`03001oogoo005oo`007Woo00<007ooOol04goo00<004XHBQP00P000eIH
00<005JIEYT01eJI0P001FLI00<006lHKaP016lH00@006lHKaP000=[6003001Vf6KH00AVf004001N
V0000005FWL00`00DSMB=`04DSL00`00BQM:5`04BQL00`00@OQ1n004@OP00`00<OXanP02<OX00`00
6Qd0000300020GX00`00>U`jG004>U`00`00D[]B^`05D[/00`00G]X00005Ii`40009Ool00`00Oomo
o`01Ool001moo`8001=oo`03001:600000IFF003001FVEJI00EFV@04001Rf6;H0003IaT00`00KaQ_
6004KaP00`00KaQ_6002KaP00`00JaP00008I]P00`00FWMJM`05FWL00`00DSMB=`04DSL00`00BQM:
5`04BQL00`00@OQ1n004@OP00`00<OXanP03<OX010006QdJ7AXM0`0000@1NP00000000@jG003001B
^e:k00=B^`@000=WW0800004BkP007oo0009Ool00`00Oomoo`01Ool0025oo`03001oogoo015oo`03
001FF5IH00EFF003001FVEJI00AFV@03001Rf6;H009Rf003001W6@0000I_6003001_66lH00A_6003
001Vf6KH00IVf08000MJM`03001B=e8g00EB=`03001:5dXG00=:5`030011n47h00A1n003000anS7j
00@anP03000J7AXM008J7@@000036Wh0000000<0009B^`<000A;G`03001WW000009;^004001oogoo
000:Ool10001Ool1Ool0029oo`03001oogoo011oo`03001FF5IH00IFF003001FVEJI009FV@03001R
f6;H00ARf003001_66lH00A_6003001_66lH00E_6003001Vf6KH00EVf004001R^6:h0007FWL00`00
DSMB=`04DSL00`00BQM:5`04BQL01000@OQ1n47h4`0011Yn0`001T]O1P0017oo00<007ooOol01goo
0@000Goo0Goo000SOol00`00Oomoo`0@Ool00`00EUQFF005EUP01@00EYUFVEJI00001f;H0P001FlH
00<006lHKaP016lH00<006lH00001FKH00D006:hH[QR^00000MJM`03001B=e8g00AB=`04001:5dXG
BQL70003BQP00`00BUY:FP04BUX00`00@[e2_@06@[d500026Wh00`00Bem;G`04000:Ool00`00Oomo
o`06Ool10001Ool1Ool002Aoo`03001oogoo00moo`03001FF5IH00IFF003001FV@0000MRf004001Z
n6[h0003KaP00`00KaQ_6004KaP01@00KaQ_66lH00000fKH00<006:hH[P00f:h00<005YgFWL015Yg
00<0058gDSL00e8g1@000dhG00<004XHBQP01DXH00<004YJBUX014YJ00<004:m@[d00T:m0`000elL
1`003goo00<007ooOol01Goo0@000Goo0Goo000UOol2000@Ool00`00EUQFF005EUP20007H]P00`00
J_QZn002J_P00`00KaP00006KaP00`00KaQ_6003KaP00`00I]P00006H[P00`00FWMJM`05FWL00`00
DSL000020002DSL00`00CQM>5`04CQL00`00BQQ:6004BQP00`00BUY:FP05BUX01000@[d000001ElL
00<007ooOol057oo00<007ooOol01Goo0@000Goo0Goo000WOol00`00Oomoo`0=Ool00`00EUQFF004
EUP00`00GYP00007H]P00`00J_QZn003J_P00`00KaQ_6004KaP00`00KaQ_6005KaP00`00H[QR^005
H[P00`00FWMJM`04FWL30005DSL00`00CQM>5`04CQL00`00BQQ:6005BQP00`00BUY:FP04BUX20006
Ga`00`00Oomoo`0FOol00`00Oomoo`04Ool10001Ool1Ool002Qoo`03001oogoo00aoo`03001FF5IH
009FF08000=NV003001Rf6;H00=Rf003001Zn6[h00AZn08000E_6003001_66lH00A_608000QR^004
001JMeYgFWL200000eIG001B=`06DSL00`00CQM>5`04CQL00`00BQQ:6004BQP01000BUY:FTYJ0`00
00=JV@00Ga`015lL00<007ooOol067oo00<007ooOol00goo0@000Goo0Goo000YOol00`00Oomoo`0<
Ool01000EUQFF0001UjH00D006;HH]QRf00000IZn004001_>6lh0003KaP00`00KaQ_6004KaP01000
JcU[>@001f:h00@005Yg000000AFE`03001B=e8g00AB=`03001>5dhG00A>5`03001:64XH00A:6004
001:FP000005FYT01@00GaaO75lL00006goo00<007ooOol017oo0@000Goo000ZOol2000<Ool00`00
EUP00007GYP01@00H]QRf6;H00001F[h00<006lhKcP00Vlh00<006lH00001VlH00D006/iJcU[>@00
00MR^08000IFE`03001B=e8g00AB=`03001>5dhG00A>5`03001:64XH00A:608000QJV@03001O7000
01eoo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol02Woo00<005jHGYP01ejH00<006;H0000
1V[h00<006lhKcP00flh0P001FlH00<006/iJcT00f/i00@006:hH[QR^0<00003FYT005IG00EFE`03
001B=e8g00EB=`03001>5dhG00A>5`04001:64XHBQP200000eIG001JV@08FYT00`00Oomoo`0MOol0
0`00Oomoo`02Ool10001Ool002eoo`03001oogoo00Uoo`03001NV5jH00QNV003001Zn6[h00AZn003
001_>6lh00A_>003001cN@0000=_6003001[>F/i00A[>@04001R^0000005FYT00`00EUMFE`04EUL0
0`00DSMB=`04DSL00`00CQM>5`04CQL01000BQP0000015IG00<005ZIFYT01EZI00<007ooOol07goo
00D007ooOomoo`00005oo`00;Woo00<007ooOol02Goo00<005jHGYP01UjH00<006KH000016[h00<0
06lhKcP016lh00L007=iLgUcN@00KaP00007JcT20007FYT00`00EUMFE`04EUL00`00DSMB=`04DSL0
0`00CQM>5`04CQL20006EUL00`00FYUJV@04FYT00`00Oomoo`0POol01@00Oomoogoo00000Goo000_
Ool20009Ool00`00GYQNV005GYP01@00I]QVf6KH00000f[h00<006lhKcP00flh00<007=iLgT00g=i
0P001V/i00<006<l00001eZI00<005IGEUL015IG00<0058gDSL0158g00@004hGCQM>5`<00003DSL0
05IG00IFE`03001JVEZI009JV@03001oogoo029oo`04001oogoo0001Ool0035oo`03001oogoo00Mo
o`03001NV5jH00=NV003001Vf6KH00=Vf003001Zn00000E_>003001cNG=i00AcN@03001_^`0000=[
>@8000=S?003001JVEZI00EJV@03001FEeIG00AFE`03001B=e8g00AB=`<000AB=`03001FEeIG00IF
E`04001JVEZI000VOol00`00Ool00001Ool0039oo`03001oogoo00Ioo`03001NV5jH009NV003001V
f6KH00EVf08000E_>003001cNG=i00=cN@07001_^fnkKk/006/i00001F<l00<005ZIFYT01EZI00<0
05IGEUL015IG00<0058gDSL00e8g0P001e8g00<005IGEUL01EIG00<005ZI00009goo00<007ooOol0
0@00000cOol00`00Oomoo`06Ool01000GYQNV00026KH00<006mI00000flh00<007=iLgT00g=i00<0
06nkKk/00Vnk0P001f<l00<005ZIFYT015ZI00<005IGEUL015IG00<0058gDSL00`0000=B=`00DSL0
1U8g00<005IGEUL01EIG0P00:Goo0@000Goo0@00000dOol20006Ool00`00GYP00008I]P01`00KeU_
FFmI001_>00000EcN@03001_^fnk009_^`04001KoE_m0007Hc`00`00FYUJV@04FYT01000EUMFEeIG
1@001E8g00<0058gDSL0158g00<005IGEUL015IG00<007ooOol0:Woo0P00000fOol00`00Oomoo`04
Ool00`00I]QVf006I]P00`00KeU_F@03KeT20004LgT01000Kk]_^fnk0P0015_m00<006<lHc`016<l
00@005ZIFYUJV@L000=>>003001B=e8g00AB=`03001B=e8g00EB=`03001FEeIG009FE`03001oogoo
02Yoo`<00000=goo00<007ooOol00goo00<006KHI]P01FKH00<006mIKeT00fmI00<007?KLm/00P00
00=cN@00Kk/00Vnk00<005_mFod00U_m3@000dij00<004hhCSP014hh00<0058gDSL0158g00<0058g
DSL01E8g00D005IGEUMFE`0002Uoo`@000=oo`00>7oo00<007ooOol00goo00<006KHI]P00fKH00<0
06mIKeT00fmI00<007?KLm/00W?K3`0000<Bf@00A_h00dKn00<004ijCWX014ij00<004hhCSP014hh
00<0058gDSL0158g00<0058gDSL0158g00@005IGEUL002Ioo`@000Moo`00>Goo00<007ooOol00goo
00D006KHI]QVf00000E_F@03001cfg?K009cf`03001Kn@000080009oo`03001oogoo00Ioo`<000A6
oP03001>NTij00=>NP03001>>4hh00A>>003001B=e8g00AB=`03001B=e8g00EB=`8002Aoo`<000]o
o`00>Woo0P0017oo00<006KH00001FmI00@007?KLm]cf`@000Eoo`03001oogoo00Uoo`<00096oP03
001>NTij009>NP03001>>4hh00A>>003001B=e8g00EB=`03001B=e8g00=B=`80025oo`@000ioo`00
?7oo00<007ooOol00Woo0P0016mI00@007?KLm]cf`<000Moo`03001oogoo00aoo`<000E>NP03001>
>4hh00A>>003001B=e8g00AB=`04001B=e8gDSL2000OOol4000BOol003eoo`03001oogoo00=oo`80
0003KeT007?K009cf`8000Uoo`03001oogoo00moo`<0009>NP03001>>4hh00A>>003001B=e8g00AB
=`04001B=`00000MOol4000FOol003ioo`03001oogoo00Aoo`D000]oo`03001oogoo019oo`<000I>
>003001B=e8g00=B=`<001]oo`@001Yoo`00?goo0P0057oo00<007ooOol05Goo1@000Thh00<0058g
00000P006Woo10007Woo0011Ool00`00Oomoo`0@Ool00`00Oomoo`0KOol4000JOol3000ROol0049o
o`03001oogoo00moo`03001oogoo03Eoo`@002Eoo`00@goo00<007ooOol03Woo00<007ooOol0<Goo
1000:Goo0014Ool2000>Ool00`00Oomoo`0]Ool4000]Ool004Ioo`03001oogoo00]oo`03001oogoo
02Uoo`@0035oo`00Agoo00<007ooOol02Woo00<007ooOol09Goo1000=Goo0018Ool00`00Oomoo`09
Ool00`00Oomoo`0QOol4000iOol004Uoo`8000Uoo`03001oogoo01ioo`<003eoo`00Bgoo00<007oo
Ool01Woo00<007ooOol06Woo1000@7oo001<Ool00`00Oomoo`05Ool00`00Oomoo`0FOol40014Ool0
04eoo`03001oogoo00Aoo`03001oogoo019oo`@004Qoo`00CWoo00<007ooOol00goo00<007ooOol0
3Woo1000C7oo001?Ool20003Ool00`00Oomoo`0:Ool4001@Ool0055oo`04001oogoo0009Ool3001D
Ool0059oo`03001oo`0000Eoo`@005Moo`00Dgoo0P0000=oo`0000000P00Fgoo001DOol2001OOol0
0;Eoo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.0244 .26331 -12 -2.7408 ]
[.0244 .26331 0 6.2592 ]
[.01583 .31702 -24 -2.7862 ]
[.01583 .31702 0 6.2138 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00224 .43027 -18 -2.88246 ]
[-0.00224 .43027 0 6.11754 ]
[-0.01177 .49002 -6 -2.93353 ]
[-0.01177 .49002 0 6.06647 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03662 .25853 m
.04273 .25614 L
s
[(-1)] .0244 .26331 1 -0.39093 Mshowa
.02809 .31235 m
.03423 .31001 L
s
[(-0.5)] .01583 .31702 1 -0.38084 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01011 .42583 m
.01628 .42361 L
s
[(0.5)] -0.00224 .43027 1 -0.35945 Mshowa
.00062 .48571 m
.00682 .48355 L
s
[(1)] -0.01177 .49002 1 -0.3481 Mshowa
.125 Mabswid
.03494 .26914 m
.03861 .26772 L
s
.03325 .27983 m
.03692 .27841 L
s
.03154 .2906 m
.03521 .28918 L
s
.02982 .30143 m
.0335 .30003 L
s
.02635 .32334 m
.03003 .32194 L
s
.0246 .33441 m
.02828 .33302 L
s
.02283 .34555 m
.02652 .34417 L
s
.02105 .35678 m
.02474 .3554 L
s
.01746 .37947 m
.02115 .37811 L
s
.01564 .39093 m
.01934 .38958 L
s
.01381 .40248 m
.01751 .40114 L
s
.01197 .41411 m
.01567 .41278 L
s
.00824 .43763 m
.01195 .43631 L
s
.00636 .44952 m
.01007 .4482 L
s
.00446 .4615 m
.00817 .46019 L
s
.00255 .47356 m
.00626 .47226 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.33 0 0 r
.3802 .65348 .40338 .65368 .43985 .67636 .41742 .66235 Metetra
.08 .544 .903 r
.41742 .66235 .43985 .67636 .47753 .70281 .45557 .67322 Metetra
.431 .58 .901 r
.45557 .67322 .47753 .70281 .51658 .72564 .4948 .68172 Metetra
.555 .56 .813 r
.4948 .68172 .51658 .72564 .55687 .7377 .53503 .68376 Metetra
.616 .559 .767 r
.53503 .68376 .55687 .7377 .59796 .73381 .57604 .67645 Metetra
.66 .574 .748 r
.57604 .67645 .59796 .73381 .63925 .71216 .61748 .65879 Metetra
.698 .605 .749 r
.61748 .65879 .63925 .71216 .68023 .67492 .65906 .63195 Metetra
.739 .656 .768 r
.65906 .63195 .68023 .67492 .72064 .62756 .70059 .59901 Metetra
.783 .74 .813 r
.70059 .59901 .72064 .62756 .76061 .57746 .74213 .5642 Metetra
.815 .877 .891 r
.74213 .5642 .76061 .57746 .80065 .53213 .78395 .53195 Metetra
.67 .973 .896 r
.78395 .53195 .80065 .53213 .84153 .49778 .8265 .50607 Metetra
.8265 .50607 .84153 .49778 .88411 .47839 .8703 .48906 Metetra
.153 0 0 r
.8703 .48906 .88411 .47839 .92917 .47524 .91584 .48173 Metetra
.229 0 0 r
.91584 .48173 .92917 .47524 .97722 .48681 .96343 .48311 Metetra
.182 .71 .705 r
.35587 .65899 .3802 .65348 .41742 .66235 .39424 .64967 Metetra
.602 .822 .984 r
.39424 .64967 .41742 .66235 .45557 .67322 .43322 .64037 Metetra
.64 .655 .85 r
.43322 .64037 .45557 .67322 .4948 .68172 .47281 .63067 Metetra
.632 .575 .772 r
.47281 .63067 .4948 .68172 .53503 .68376 .51304 .62019 Metetra
.628 .543 .738 r
.51304 .62019 .53503 .68376 .57604 .67645 .55388 .60868 Metetra
.63 .536 .73 r
.55388 .60868 .57604 .67645 .61748 .65879 .59533 .59605 Metetra
.637 .551 .741 r
.59533 .59605 .61748 .65879 .65906 .63195 .63737 .58239 Metetra
.651 .595 .779 r
.63737 .58239 .65906 .63195 .70059 .59901 .67998 .56797 Metetra
.671 .693 .862 r
.67998 .56797 .70059 .59901 .74213 .5642 .72318 .55317 Metetra
.637 .876 .986 r
.72318 .55317 .74213 .5642 .78395 .53195 .76702 .53843 Metetra
.76702 .53843 .78395 .53195 .8265 .50607 .81154 .5241 Metetra
.17 0 0 r
.81154 .5241 .8265 .50607 .8703 .48906 .85681 .51045 Metetra
.244 0 0 r
.85681 .51045 .8703 .48906 .91584 .48173 .90289 .49756 Metetra
.167 0 0 r
.90289 .49756 .91584 .48173 .96343 .48311 .94984 .48532 Metetra
.728 .988 .881 r
.33047 .6653 .35587 .65899 .39424 .64967 .37032 .63675 Metetra
.803 .842 .88 r
.37032 .63675 .39424 .64967 .43322 .64037 .41041 .60661 Metetra
.741 .689 .801 r
.41041 .60661 .43322 .64037 .47281 .63067 .45062 .5785 Metetra
.686 .6 .755 r
.45062 .5785 .47281 .63067 .51304 .62019 .49099 .55559 Metetra
.643 .549 .733 r
.49099 .55559 .51304 .62019 .55388 .60868 .53171 .53993 Metetra
.607 .521 .731 r
.53171 .53993 .55388 .60868 .59533 .59605 .57308 .53224 Metetra
.574 .513 .749 r
.57308 .53224 .59533 .59605 .63737 .58239 .6154 .53167 Metetra
.537 .532 .798 r
.6154 .53167 .63737 .58239 .67998 .56797 .65887 .53603 Metetra
.471 .6 .899 r
.65887 .53603 .67998 .56797 .72318 .55317 .70356 .54201 Metetra
.237 .683 .955 r
.70356 .54201 .72318 .55317 .76702 .53843 .74932 .54586 Metetra
.167 0 0 r
.74932 .54586 .76702 .53843 .81154 .5241 .79581 .54405 Metetra
.249 0 0 r
.79581 .54405 .81154 .5241 .85681 .51045 .84262 .5341 Metetra
.172 0 0 r
.84262 .5341 .85681 .51045 .90289 .49756 .88932 .51515 Metetra
.88932 .51515 .90289 .49756 .94984 .48532 .93568 .48821 Metetra
.886 .939 .844 r
.30427 .6669 .33047 .6653 .37032 .63675 .3457 .62193 Metetra
.857 .813 .805 r
.3457 .62193 .37032 .63675 .41041 .60661 .38707 .57434 Metetra
.796 .712 .773 r
.38707 .57434 .41041 .60661 .45062 .5785 .42807 .53115 Metetra
.73 .636 .754 r
.42807 .53115 .45062 .5785 .49099 .55559 .46876 .49816 Metetra
.662 .576 .748 r
.46876 .49816 .49099 .55559 .53171 .53993 .50949 .47914 Metetra
.588 .528 .755 r
.50949 .47914 .53171 .53993 .57308 .53224 .55081 .4753 Metetra
.499 .489 .777 r
.55081 .4753 .57308 .53224 .6154 .53167 .59327 .48519 Metetra
.377 .461 .82 r
.59327 .48519 .6154 .53167 .65887 .53603 .63732 .50483 Metetra
.181 .444 .873 r
.63732 .50483 .65887 .53603 .70356 .54201 .6831 .52814 Metetra
0 .403 .797 r
.6831 .52814 .70356 .54201 .74932 .54586 .73039 .54783 Metetra
.273 0 0 r
.73039 .54783 .74932 .54586 .79581 .54405 .77856 .55686 Metetra
.171 0 0 r
.77856 .55686 .79581 .54405 .84262 .5341 .82675 .55009 Metetra
.82675 .55009 .84262 .5341 .88932 .51515 .87413 .52575 Metetra
.53 .868 .604 r
.87413 .52575 .88932 .51515 .93568 .48821 .92019 .48597 Metetra
.889 .847 .797 r
.27768 .65887 .30427 .6669 .3457 .62193 .32047 .60375 Metetra
.867 .787 .77 r
.32047 .60375 .3457 .62193 .38707 .57434 .36301 .54557 Metetra
.828 .736 .763 r
.36301 .54557 .38707 .57434 .42807 .53115 .40486 .49332 Metetra
.771 .687 .77 r
.40486 .49332 .42807 .53115 .46876 .49816 .44604 .45432 Metetra
.689 .634 .789 r
.44604 .45432 .46876 .49816 .50949 .47914 .48699 .43319 Metetra
.569 .569 .814 r
.48699 .43319 .50949 .47914 .55081 .4753 .52837 .43141 Metetra
.392 .484 .835 r
.52837 .43141 .55081 .4753 .59327 .48519 .57094 .44723 Metetra
.156 .38 .83 r
.57094 .44723 .59327 .48519 .63732 .50483 .61531 .47575 Metetra
0 .288 .786 r
.61531 .47575 .63732 .50483 .6831 .52814 .66172 .5093 Metetra
0 .281 .738 r
.66172 .5093 .6831 .52814 .73039 .54783 .70995 .53851 Metetra
0 .429 .758 r
.70995 .53851 .73039 .54783 .77856 .55686 .75923 .55409 Metetra
.209 .731 .854 r
.75923 .55409 .77856 .55686 .82675 .55009 .80847 .54914 Metetra
.639 .964 .891 r
.80847 .54914 .82675 .55009 .87413 .52575 .85656 .52126 Metetra
.864 .987 .839 r
.85656 .52126 .87413 .52575 .92019 .48597 .90281 .47335 Metetra
.854 .773 .771 r
.25111 .638 .27768 .65887 .32047 .60375 .29471 .5813 Metetra
.858 .767 .761 r
.29471 .5813 .32047 .60375 .36301 .54557 .33806 .52148 Metetra
.845 .764 .772 r
.33806 .52148 .36301 .54557 .40486 .49332 .38064 .4678 Metetra
.809 .76 .808 r
.38064 .4678 .40486 .49332 .44604 .45432 .4224 .42774 Metetra
.723 .743 .868 r
.4224 .42774 .44604 .45432 .48699 .43319 .46378 .40602 Metetra
.531 .674 .929 r
.46378 .40602 .48699 .43319 .52837 .43141 .50546 .40416 Metetra
.206 .503 .907 r
.50546 .40416 .52837 .43141 .57094 .44723 .54824 .42036 Metetra
0 .311 .797 r
.54824 .42036 .57094 .44723 .61531 .47575 .59278 .4496 Metetra
0 .239 .74 r
.59278 .4496 .61531 .47575 .66172 .5093 .63941 .48406 Metetra
0 .325 .801 r
.63941 .48406 .66172 .5093 .70995 .53851 .68795 .5141 Metetra
.195 .535 .925 r
.68795 .5141 .70995 .53851 .75923 .55409 .73769 .53012 Metetra
.538 .735 .966 r
.73769 .53012 .75923 .55409 .80847 .54914 .78752 .52501 Metetra
.755 .819 .902 r
.78752 .52501 .80847 .54914 .85656 .52126 .83629 .49627 Metetra
.854 .834 .827 r
.83629 .49627 .85656 .52126 .90281 .47335 .88326 .44692 Metetra
.804 .712 .763 r
.22487 .6037 .25111 .638 .29471 .5813 .26849 .55438 Metetra
.832 .75 .773 r
.26849 .55438 .29471 .5813 .33806 .52148 .31205 .50227 Metetra
.846 .8 .806 r
.31205 .50227 .33806 .52148 .38064 .4678 .35505 .45506 Metetra
.828 .863 .872 r
.35505 .45506 .38064 .4678 .4224 .42774 .39739 .41906 Metetra
.709 .91 .965 r
.39739 .41906 .4224 .42774 .46378 .40602 .43941 .39833 Metetra
.35 .787 .965 r
.43941 .39833 .46378 .40602 .50546 .40416 .48168 .39415 Metetra
0 .491 .803 r
.48168 .39415 .50546 .40416 .54824 .42036 .52489 .40498 Metetra
0 .332 .74 r
.52489 .40498 .54824 .42036 .59278 .4496 .56963 .42652 Metetra
0 .353 .812 r
.56963 .42652 .59278 .4496 .63941 .48406 .61624 .45215 Metetra
.177 .46 .884 r
.61624 .45215 .63941 .48406 .68795 .5141 .66461 .47389 Metetra
.426 .552 .88 r
.66461 .47389 .68795 .5141 .73769 .53012 .71421 .48394 Metetra
.593 .613 .841 r
.71421 .48394 .73769 .53012 .78752 .52501 .76415 .47658 Metetra
.7 .658 .805 r
.76415 .47658 .78752 .52501 .83629 .49627 .81351 .44981 Metetra
.773 .699 .781 r
.81351 .44981 .83629 .49627 .88326 .44692 .86164 .40605 Metetra
.739 .656 .768 r
.19906 .55817 .22487 .6037 .26849 .55438 .24184 .52358 Metetra
.78 .731 .808 r
.24184 .52358 .26849 .55438 .31205 .50227 .28488 .48707 Metetra
.81 .845 .876 r
.28488 .48707 .31205 .50227 .35505 .45506 .32784 .45318 Metetra
.749 .963 .935 r
.32784 .45318 .35505 .45506 .39739 .41906 .37064 .42579 Metetra
.412 .859 .794 r
.37064 .42579 .39739 .41906 .43941 .39833 .41345 .40743 Metetra
.41345 .40743 .43941 .39833 .48168 .39415 .45662 .39895 Metetra
.129 0 0 r
.45662 .39895 .48168 .39415 .52489 .40498 .50059 .39933 Metetra
0 .529 .764 r
.50059 .39933 .52489 .40498 .56963 .42652 .54575 .40586 Metetra
.246 .613 .956 r
.54575 .40586 .56963 .42652 .61624 .45215 .59232 .4145 Metetra
.45 .584 .895 r
.59232 .4145 .61624 .45215 .66461 .47389 .64029 .42052 Metetra
.548 .558 .817 r
.64029 .42052 .66461 .47389 .71421 .48394 .68936 .41949 Metetra
.607 .556 .772 r
.68936 .41949 .71421 .48394 .76415 .47658 .73906 .40823 Metetra
.653 .573 .753 r
.73906 .40823 .76415 .47658 .81351 .44981 .78887 .38567 Metetra
.697 .609 .754 r
.78887 .38567 .81351 .44981 .86164 .40605 .83839 .3531 Metetra
.646 .596 .784 r
.17346 .50582 .19906 .55817 .24184 .52358 .21467 .49016 Metetra
.669 .695 .865 r
.21467 .49016 .24184 .52358 .28488 .48707 .25651 .47409 Metetra
.658 .863 .977 r
.25651 .47409 .28488 .48707 .32784 .45318 .29895 .45804 Metetra
.392 .851 .828 r
.29895 .45804 .32784 .45318 .37064 .42579 .34199 .4424 Metetra
.34199 .4424 .37064 .42579 .41345 .40743 .38566 .42743 Metetra
.041 0 0 r
.38566 .42743 .41345 .40743 .45662 .39895 .43002 .4132 Metetra
.43002 .4132 .45662 .39895 .50059 .39933 .47512 .39961 Metetra
.377 .841 .874 r
.47512 .39961 .50059 .39933 .54575 .40586 .52102 .38637 Metetra
.618 .814 .975 r
.52102 .38637 .54575 .40586 .59232 .4145 .56777 .37306 Metetra
.635 .658 .855 r
.56777 .37306 .59232 .4145 .64029 .42052 .61538 .35924 Metetra
.624 .574 .778 r
.61538 .35924 .64029 .42052 .68936 .41949 .66384 .34449 Metetra
.618 .538 .741 r
.66384 .34449 .68936 .41949 .73906 .40823 .71314 .32853 Metetra
.62 .531 .732 r
.71314 .32853 .73906 .40823 .78887 .38567 .76324 .31127 Metetra
.628 .548 .745 r
.76324 .31127 .78887 .38567 .83839 .3531 .81412 .29279 Metetra
.499 .519 .809 r
.14761 .45225 .17346 .50582 .21467 .49016 .18686 .45574 Metetra
.419 .591 .914 r
.18686 .45574 .21467 .49016 .25651 .47409 .22704 .46088 Metetra
.213 .673 .946 r
.22704 .46088 .25651 .47409 .29895 .45804 .26856 .46381 Metetra
.04 0 0 r
.26856 .46381 .29895 .45804 .34199 .4424 .3116 .46093 Metetra
.063 0 0 r
.3116 .46093 .34199 .4424 .38566 .42743 .35612 .44968 Metetra
.35612 .44968 .38566 .42743 .43002 .4132 .40183 .42919 Metetra
.404 .822 .638 r
.40183 .42919 .43002 .4132 .47512 .39961 .44837 .40051 Metetra
.751 .988 .896 r
.44837 .40051 .47512 .39961 .52102 .38637 .49538 .36633 Metetra
.805 .853 .886 r
.49538 .36633 .52102 .38637 .56777 .37306 .54264 .33039 Metetra
.743 .7 .811 r
.54264 .33039 .56777 .37306 .61538 .35924 .59016 .2966 Metetra
.683 .603 .76 r
.59016 .2966 .61538 .35924 .66384 .34449 .6382 .26832 Metetra
.634 .545 .736 r
.6382 .26832 .66384 .34449 .71314 .32853 .68713 .24776 Metetra
.595 .514 .733 r
.68713 .24776 .71314 .32853 .76324 .31127 .73742 .23563 Metetra
.561 .507 .752 r
.73742 .23563 .76324 .31127 .81412 .29279 .78946 .23103 Metetra
.254 .403 .82 r
.12082 .40294 .14761 .45225 .18686 .45574 .15818 .422 Metetra
0 .341 .82 r
.15818 .422 .18686 .45574 .22704 .46088 .19666 .4448 Metetra
0 .297 .678 r
.19666 .4448 .22704 .46088 .26856 .46381 .23711 .46391 Metetra
.247 0 0 r
.23711 .46391 .26856 .46381 .3116 .46093 .27999 .47208 Metetra
.27999 .47208 .3116 .46093 .35612 .44968 .32524 .46404 Metetra
.406 .836 .688 r
.32524 .46404 .35612 .44968 .40183 .42919 .37232 .43796 Metetra
.762 .996 .825 r
.37232 .43796 .40183 .42919 .44837 .40051 .42043 .39606 Metetra
.887 .949 .843 r
.42043 .39606 .44837 .40051 .49538 .36633 .46878 .34393 Metetra
.869 .836 .811 r
.46878 .34393 .49538 .36633 .54264 .33039 .5169 .28904 Metetra
.807 .73 .78 r
.5169 .28904 .54264 .33039 .59016 .2966 .56474 .23894 Metetra
.733 .644 .761 r
.56474 .23894 .59016 .2966 .6382 .26832 .61271 .19978 Metetra
.656 .576 .753 r
.61271 .19978 .6382 .26832 .68713 .24776 .6615 .17547 Metetra
.576 .523 .758 r
.6615 .17547 .68713 .24776 .73742 .23563 .71197 .16724 Metetra
.49 .485 .779 r
.71197 .16724 .73742 .23563 .78946 .23103 .76487 .17357 Metetra
0 .241 .757 r
.09234 .36228 .12082 .40294 .15818 .422 .12839 .39033 Metetra
0 .091 .603 r
.12839 .39033 .15818 .422 .19666 .4448 .16563 .42355 Metetra
.436 0 0 r
.16563 .42355 .19666 .4448 .23711 .46391 .20526 .45234 Metetra
0 .363 .68 r
.20526 .45234 .23711 .46391 .27999 .47208 .24798 .46717 Metetra
.293 .777 .913 r
.24798 .46717 .27999 .47208 .32524 .46404 .2938 .46094 Metetra
.729 .964 .939 r
.2938 .46094 .32524 .46404 .37232 .43796 .34203 .43116 Metetra
.875 .935 .856 r
.34203 .43116 .37232 .43796 .42043 .39606 .39155 .38086 Metetra
.901 .871 .799 r
.39155 .38086 .42043 .39606 .46878 .34393 .44127 .31757 Metetra
.885 .813 .773 r
.44127 .31757 .46878 .34393 .5169 .28904 .49046 .25113 Metetra
.843 .758 .768 r
.49046 .25113 .5169 .28904 .56474 .23894 .53899 .19122 Metetra
.779 .702 .778 r
.53899 .19122 .56474 .23894 .61271 .19978 .58732 .14553 Metetra
.687 .639 .797 r
.58732 .14553 .61271 .19978 .6615 .17547 .63633 .11882 Metetra
.562 .568 .817 r
.63633 .11882 .6615 .17547 .71197 .16724 .68708 .11257 Metetra
.409 .491 .832 r
.68708 .11257 .71197 .16724 .76487 .17357 .74056 .12493 Metetra
0 .129 .623 r
.06146 .33284 .09234 .36228 .12839 .39033 .09725 .36157 Metetra
0 .033 .52 r
.09725 .36157 .12839 .39033 .16563 .42355 .1342 .39565 Metetra
0 .153 .633 r
.1342 .39565 .16563 .42355 .20526 .45234 .17366 .42522 Metetra
.051 .472 .885 r
.17366 .42522 .20526 .45234 .24798 .46717 .21647 .44044 Metetra
.525 .737 .971 r
.21647 .44044 .24798 .46717 .2938 .46094 .2627 .43397 Metetra
.756 .804 .891 r
.2627 .43397 .2938 .46094 .34203 .43116 .31165 .40325 Metetra
.84 .805 .817 r
.31165 .40325 .34203 .43116 .39155 .38086 .36212 .35141 Metetra
.871 .798 .776 r
.36212 .35141 .39155 .38086 .44127 .31757 .4129 .28624 Metetra
.876 .792 .764 r
.4129 .28624 .44127 .31757 .49046 .25113 .46313 .21791 Metetra
.861 .788 .777 r
.46313 .21791 .49046 .25113 .53899 .19122 .5126 .15632 Metetra
.818 .779 .816 r
.5126 .15632 .53899 .19122 .58732 .14553 .56171 .10935 Metetra
.722 .75 .874 r
.56171 .10935 .58732 .14553 .63633 .11882 .61134 .08181 Metetra
.545 .673 .921 r
.61134 .08181 .63633 .11882 .68708 .11257 .66257 .07523 Metetra
.308 .54 .912 r
.66257 .07523 .68708 .11257 .74056 .12493 .71648 .08769 Metetra
.366 0 0 r
.02767 .31503 .06146 .33284 .09725 .36157 .06456 .33583 Metetra
0 .201 .658 r
.06456 .33583 .09725 .36157 .1342 .39565 .10254 .3608 Metetra
0 .363 .831 r
.10254 .3608 .1342 .39565 .17366 .42522 .14283 .38181 Metetra
.346 .529 .893 r
.14283 .38181 .17366 .42522 .21647 .44044 .18617 .39082 Metetra
.576 .614 .853 r
.18617 .39082 .21647 .44044 .2627 .43397 .23269 .38197 Metetra
.699 .66 .808 r
.23269 .38197 .2627 .43397 .31165 .40325 .2818 .35322 Metetra
.771 .695 .779 r
.2818 .35322 .31165 .40325 .36212 .35141 .33251 .30706 Metetra
.817 .731 .77 r
.33251 .30706 .36212 .35141 .4129 .28624 .38374 .24974 Metetra
.847 .773 .779 r
.38374 .24974 .4129 .28624 .46313 .21791 .43472 .18952 Metetra
.857 .822 .813 r
.43472 .18952 .46313 .21791 .5126 .15632 .48516 .13465 Metetra
.831 .875 .876 r
.48516 .13465 .5126 .15632 .56171 .10935 .53535 .09177 Metetra
.719 .898 .96 r
.53535 .09177 .56171 .10935 .61134 .08181 .58599 .06503 Metetra
.471 .814 .999 r
.58599 .06503 .61134 .08181 .66257 .07523 .63802 .05574 Metetra
.199 .632 .948 r
.63802 .05574 .66257 .07523 .71648 .08769 .69235 .06223 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo00Qoo`P000=oo`04001oogooOol3000[Ool006Qoo`<0
00Uoo`@000@oG`04000JWQZN6Yh40005Ool00`00Oomoo`09Ool4000LOol006Ioo`8000Uoo`<000To
G`03000JWQZN00@JWP03001oogoo00=oo`03001oogoo00Uoo`03001oogoo01aoo`00Hgoo0`002Goo
0P000enN0P0013mO1`000aZN0P0017oo00<007ooOol00Woo1@000goo00<007ooOol06goo0017Ool2
000HOol2000:Ool20006Gih50004A[d00`009SdV?@060004Ool00`00Oomoo`0:Ool00`00Oomoo`0J
Ool004Ioo`04001oogoo000DOol3000:Ool20007Gih00`00G_/00008A[d00`009SdV?@029Sd01000
9Sd0000017oo00<007ooOol01Woo00@007ooOol001aoo`00AWoo00@007ooOol0019oo`8000]oo`<0
00IOWP8000=Nn`030016_DJm00I6_@05000V?BHm9Sd000029Sd01000Oomoogoo0P002Goo0P007Goo
0016Ool01000Oomoo`003goo0`0000=oo`00Ool02Goo00D006]lJga[O00000=OWP8000INn`030016
_DJm00E6_@05000V?BHm9Sd000039Sd00`00Oomoo`02Ool00`00Oomoo`0UOol004Ioo`04001oogoo
000=Ool2000>Ool00`00Jga[O003Jg`00`00Gih00008G_/00`00A[e6_@06A[d010009SdV?@0012Hm
00<007ooOol00Woo00<007ooOol097oo0017Ool2000;Ool3000?Ool00`00Jga[O004Jg`2000:G_/0
0`00A[e6_@02A[d700049Sd00`00Oomoo`02Ool00`00Oomoo`0TOol0059oo`<0011oo`03001[O6]l
00A[O003001[6P0000YNn`D000A6FP03000enP0000H000Aoo`03001oogoo02=oo`00Cgoo0`004Woo
00<006]lJg`016]l00D006/JJaY[6P0000MNn`800003EYT004IJ00M6FP8000DenP8000Aoo`03001o
ogoo029oo`00C7oo0`0057oo00<006]lJg`016]l00<006/JJaX00f/J00<005kkG_/00Ukk0P0015JI
00<004IJAUX01DIJ0P001SGj00<007ooOol00Woo00<007ooOol08Woo001:Ool3000EOol20005Jg`0
0`00JaY[6P05JaX01000G_/000001UJI00<004IJAUX01TIJ00<003Gj=OX01CGj00<007ooOol00Woo
00<007ooOol08Goo0017Ool3000GOol01000KeY_FP000f]l00<006/JJaX01f/J00<005JIEYT01eJI
00<004IJAUX01DIJ00<003Gj=OX01CGj00<007ooOol00goo00<007ooOol087oo0015Ool2000IOol0
0`00KeY_FP02KeX00`00Jg`0000:JaX00`00EYUFV@07EYT00`00AUY6FP05AUX00`00=OXenP06=OX0
0`00Oomoo`02Ool00`00Oomoo`0POol0049oo`<001Yoo`03001_FVmJ00A_FP03001[6V/J00Q[6P03
001Rf00000QFV@030016FTIJ00E6FP8000LenP03001oogoo00=oo`03001oogoo01moo`009Goo1000
5goo0P0077oo00<006mJKeX01FmJ00<006/JJaX01f/J00@006;HH]P000UFV@030016FTIJ00A6FP80
00PenP03001oogoo00=oo`03001oogoo01ioo`009Woo00<007ooOol057oo0`007Goo00<006mJKeX0
1FmJ00<006lh00001f/J00<006;HH]P00V;H00<005JIEYT01UJI00<004IJAUX00TIJ2P000SGj00<0
07ooOol00Woo0P0087oo000OOol50003Ool00`00Oomoo`0AOol2000OOol00`00KeY_FP05KeX01@00
KcQ_>6lh00001F/J00<006;HH]P00f;H00<005JIEYT01UJI1@000TXH00<004XH00001Cgh10001Goo
00<007ooOol07Goo000XOol00`00Oomoo`0=Ool3000POol00`00KeY_FP06KeX00`00KcQ_>002KcP0
1@00JaY[6V/J00001V;H00<005JIEYT015JI0P0000=BF000BQP014XH00@004XHBQP000Lmn003001o
ogoo00=oo`03001oogoo01eoo`009Goo00@007ooOol000eoo`80029oo`03001_FVmJ00I_FP03001_
>6lh00A_>003001[6P0000QRf004001FVEJIEYT20004DUP00`00BQQ:6003BQP00`00BQP00008?OP0
0`00Oomoo`03Ool00`00Oomoo`0LOol002Ioo`8000]oo`<002Aoo`8000I_FP03001_>6lh00I_>003
001Rf6;H00MRf004001FV@000006DUP00`00BQQ:6003BQP00`00BQP00008?OP00`00Oomoo`04Ool0
0`00Oomoo`0KOol0035oo`800003Ool007oo02=oo`04001_66lH0004KeX00`00KcQ_>007KcP00`00
H]QRf007H]P20008DUP00`00BQQ:6003BQP01000BQQ:600023gh00<007ooOol00goo00<007ooOol0
6goo000^Ool3000WOol00`00KaQ_6002KaP01@00KeY_FVmJ000026lh00<006/H00001f;H00<005jH
00002E9H00<004XHBQP00TXH00@004XHBQP000Pmn003001oogoo00=oo`8001aoo`00:goo0`00:Goo
00<006lHKaP016lH00<006mJ000026lh00@006/HJaP000IRf005001NV5jHGYP00008DUP00`00BQQ:
6002BQP01000BQQ:600023gh00<007ooOol01Goo00<007ooOol00goo0P0057oo000YOol20002Ool0
0`00Oomoo`0VOol00`00KaQ_6006KaP00`00KcQ_>006KcP00`00JaQ[6002JaP00`00H]QRf002H]P0
0`00GYQNV002GYP00`00DUQBF006DUP00`00BQQ:6002BQP01@00BQQ:64XH000023gh00<007ooOol0
17oo00<007ooOol00Woo00@007ooOol001=oo`009Woo0`00;Goo00<006lHKaP01VlH00<006lhKcP0
1Flh00<006/HJaP016/H00@006;HH]P000ENV003001BF59H00MBF005001:64XHBQP00003BQP00`00
?OPmn006?OP00`00Oomoo`05Ool01@00Oomoogoo00000Woo00<007ooOol04Goo000TOol2000_Ool0
0`00KaQ_6006KaP00`00LcP00005KcP00`00JaQ[6005JaP00`00H]P00007GYP00`00DUQBF006DUP0
1@00BQQ:64XH00000dXH00<003gh?OP00cgh1@001goo00@007ooOol0009oo`03001oogoo015oo`00
8Goo0`00<7oo00<006lHKaP01VlH00D007<hLcQc>00000=_>003001[66/H00M[6003001NV5jH00IN
V003001BF59H00IBF005001:64XHBQP00002BQP70004AQP00`00Oomoo`04Ool20002Ool01000Oomo
o`004goo000OOol2000bOol00`00KaQ_6006KaP00`00LcQc>003LcP00`00KcP00009JaP20008GYP0
0`00DUQBF007DUP00`00BQP000030002CQL00`00AQQ66006AQP00`00Oomoo`06Ool00`00Oomoo`02
000DOol001aoo`<003=oo`03001_66lH00M_6003001c>7<h00Ac>003001[66/H00M[6004001Vn6Kh
0008GYP00`00DUQBF005DUP300000dhG001>5`04CQL00`00AQQ66006AQP00`00Oomoo`06Ool00`00
Oomoo`0EOol001]oo`8003Eoo`03001_66lH00I_6003001c>7<h00Ec>003001[66/H00I[6005001V
n6KhI_P00008GYP00`00DUQBF002DUP300000e8g001>5`02CQL00`00CQM>5`02CQL00`00AQQ66006
AQP00`00Oomoo`06Ool00`00Oomoo`0EOol001Aoo`<000Aoo`03001oo`0003=oo`03001Zn00000I_
6003001c>7<h00Ec>003001cF00000I[6003001Vn6Kh00=Vn003001NV5jH00ENV003001BF0000080
00EB=`05001>5dhGCQL00003CQL00`00AQQ66006AQP00`00Oomoo`07Ool00`00Oomoo`0DOol001Eo
o`03001oogoo00=oo`04001oogoo000aOol01@00J_QZn6[h00001FlH00<007<hLcP017<h00D007=H
LeQcF00000A[6003001Vn6Kh00AVn003001NV5jH00ANV0<000QB=`03001>5dhG009>5`05001>5dhG
CQL00008AQP00`00Oomoo`06Ool2000EOol000ioo`D0009oo`03001oogoo00=oo`05001oogooOol0
000_Ool00`00J_QZn003J_P01@00KaQ_66lH00001W<h00<007=HLeP00W=H00D006/HJaQ[600000MV
n004001NV5jHGYP400000eIh001B=`07DSL00`00CQM>5`02CQL01@00CQM>5dhG000024HH00<007oo
Ool01goo00<007ooOol04goo000EOol00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`0[Ool0
0`00J_QZn005J_P00`00KaP00007LcP00`00LeQcF003LeP00`00JaP00009I_P300000ejH001FN003
EWP00`00DSMB=`06DSL00`00CQM>5`02CQL01000CQM>5`0024HH00<007ooOol027oo00<007ooOol0
4Woo000EOol00`00Oomoo`02Ool20005Ool00`00Oomoo`0YOol00`00J_QZn007J_P00`00LcQc>005
LcP00`00LeQcF005LeP00`00I_QVn005I_P400000ejH00000005EWP00`00DSMB=`06DSL00`00CQM>
5`03CQL01000CQM>5`0024HH00<007ooOol01goo00<007ooOol04Woo000DOol20004Ool00`00Oomo
o`05Ool00`00Oomoo`0WOol00`00J_QZn008J_P00`00LcQc>004LcP00`00LeQcF006LeP00`00I_QV
n003I_P20003I_P20007EWP00`00DSMB=`06DSL00`00CQM>5`03CQL01000CQM>5`001dHH0P002Woo
00<007ooOol04Goo000JOol00`00Oomoo`06Ool00`00Oomoo`0UOol00`00J_QZn008J_P00`00KcP0
0005LcP00`00LeQcF005LeP01000KeT006Kh0`0016Kh00<005kI000025Ih00<0058gDSL01U8g00<0
04hGCQL00dhG00<004hG000014HH0`0000=B=`00Ool02Woo00<007ooOol047oo000JOol00`00Oomo
o`07Ool00`00Oomoo`0ROol2000:J_P01@00KcQ_>6lh00000g<h00<007=HLeP01G=H00<006mI0000
0P001VKh00D005kIG]UNf@0000MFN003001B=e8g00IB=`03001>5dhG00A>5`03001>5`0000<000EB
=`03001oogoo00Moo`80019oo`006Goo0P002Woo00<007ooOol087oo00<006;H00002F[h00<006lh
KcP00flh00<007<h00001g=H0`000VmI00<006KhI_P00VKh0P0015kI00<005IhEWP01UIh00<0058g
DSL01E8g00<004hGCQL014hG0`00258g00<007ooOol02Goo00<007ooOol03goo0008Ool20004Ool2
0003Ool30003Ool40009Ool00`00Oomoo`0NOol01@00H]QRf6;H00001f[h00<006lhKcP01Flh0P00
17=H0`001VmI00@006KhI_P000INf@03001FN5Ih00IFN003001B=e8g00IB=`03001>5`0000<00003
CQL0000000UB=`03001oogoo00Uoo`03001oogoo00ioo`001goo00@007ooOol000=oo`8000Ioo`04
001oogoo0002Ool30008Ool00`00Oomoo`0LOol00`00H]QRf003H]P00`00J_QZn004J_P00`00KcQ_
>005KcP00`00LgT000030002LeP00`00KeU_F@04KeT00`00I_P00008G]T00`00EWQFN006EWP00`00
DSMB=`04DSL30006CQL20008DSL00`00Oomoo`09Ool00`00Oomoo`0>Ool00003Ool0000000<00005
Ool007ooOol0000;Ool01000Oomoo`0017oo0P0027oo00<007ooOol06Woo00<006;HH]P01F;H00<0
06[hJ_P00V[h00<006lhKcP01Flh1@000g=H00<006mIKeT01FmI0P002EkI00<005IhEWP01UIh00@0
058gDSMB=`<00003CSL004hG00M>5`03001B=e8g00IB=`03001oogoo00Yoo`03001oogoo00eoo`00
1goo00@007ooOol000Qoo`<000=oo`03001oogoo00=oo`8000Qoo`8001Uoo`03001Rf6;H00IRf005
001Zn6[hJ_P00006KcP30004LgT00`00LeP00007KeT00`00Ig`0000:G]T00`00EWQFN005EWP01000
DSL000001Dhg00<004hGCQL01DhG00<0058gDSL01e8g00<007ooOol02Woo00<007ooOol00Woo1000
1Woo0007Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol01Goo00<007ooOol01goo00<0
07ooOol05Goo00<006;HH]P026;H00<006[h00001Flh0`001g=i0P001FmI0P000fMl00<005kIG]T0
1ekI00<005IhEWP015Ih0`001dhg00<004hGCQL01ThG00<0058gDSL01U8g00<007ooOol02Woo00<0
07ooOol00goo00<007ooOol01Woo0008Ool20009Ool40002Ool00`00Oomoo`06Ool20008Ool00`00
Oomoo`0BOol2000<H]P01000KcQ_>6lh0`0000=_>000LgT01W=i00@007?JLmX000=_F@03001WO6Ml
00=WO003001NfEkI00MNf@03001FN5Ih00@0009>F003001>=dhg00E>=`03001>5dhG00I>5`8000MB
=`8000aoo`03001oogoo00=oo`03001oogoo00Eoo`0067oo0P002Goo0P0027oo00<007ooOol047oo
00<005Zi00002f;H1@0016lh00<007=iLgT017=i00<007?JLmX00W?J00<006mI00001fMl00<005kI
G]T01UkI0`001diH00<004hgCSL01Dhg00<004hGCQL01DhG00<0058g00001E8g00<005Yh00003Goo
00<007ooOol00goo00<007ooOol017oo000HOol2000:Ool30007Ool00`00Oomoo`0=Ool20003F[T0
0`00H]QRf007H]P40005KcP00`00LgUcN@04LgT00`00LmYcfP03LmX20009Ig`00`00G]UNf@02G]T3
00000e:k001>F007CUP00`00CSM>=`05CSL00`00CQM>5`06CQL20004DSL01@00FWQJN5Yh00002goo
0P000Woo00@007ooOol000Ioo`004Woo0P0017oo00<007ooOol02Goo00<0003h00000`001Goo00<0
07ooOol02Woo0P001UZi00<006;HH]P00f;H0`0016/j00D006lhKcQ_>00000IcN@03001cfW?J00=c
fP04001So6?l0008Ig`00`00G]T000020005D[/00`00CUQ>F005CUP00`00CSM>=`06CSL00`00CQM>
5`05CQL01000DSL0058g0P0015Yh00<007ooOol02goo00<007ooOol00P001goo000AOol01000Oomo
o`000goo00<007ooOol02Woo00<0003h00000P5j0P0017oo00<007ooOol027oo00<005ZiF[T01UZi
00<006;HH]P00`0000=Rf000JcX01F/j00<006lh00001W=i00<007?JLmX00g?J00<006?lHo`00V?l
00<006MlIg`016Ml0`0025:k00<004iHCUP01TiH00<004hgCSL01Dhg00<004hGCQL01DhG00@0058g
DSL000MJN003001oogoo00]oo`03001oogoo00Qoo`004Goo00@007ooOol000=oo`03001oogoo00]o
o`030000n00000<1NPH000Eoo`@000UJ^@<000=Rf003001[>V/j00I[>P8000EcN@03001cfW?J009c
fP8000ISo003001WO6Ml00<0009?G`03001B^e:k00IB^`03001>F4iH00I>F003001>=dhg00E>=`03
001>5dhG00I>5`800003FWP005Yh00EJN003001oogoo00]oo`03001oogoo00Qoo`004Goo00@007oo
Ool0009oo`8000eoo`040000n03h00020GX00`00;Q`^7002;Q`60003BW/00`00F[UJ^@05F[T20000
0eZi001Rf002H]P00`00JcY[>P06JcX00`00Kk]_^`0300000g=i001cfP03LmX00`00HoaSo005Ho`4
0006Cel00`00D[]B^`06D[/00`00CUQ>F005CUP00`00CSM>=`06CSL00`00CQM>5`04CQL20002FWP0
0`00FWQJN004FWP00`00Oomoo`0<Ool00`00Oomoo`07Ool0015oo`04001oogoo0003Ool00`00Oomo
o`0<Ool00`000?P0n00200000`5j000^7004;Q`00`00BW]:N`05BW/01000F[UJ^EZi0`0015Zi00<0
06;H000026/j00<006nkKk/00fnk10000W?J00<006?lHo`00f?l0`0013=K00<004mOCel014mO00<0
05:kD[/01E:k00<004iHCUP01TiH00<004hgCSL01Dhg00@004hGCQM>5`<00003DUP005Yh009JN003
001JN5Yh00AJN003001oogoo00]oo`03001oogoo00Moo`004Woo0P000goo0P003goo0P0000D0n000
0001NP0000D^7003001:NdYk00E:N`<000MJ^@8000Q[>P03001_^fnk00=_^`03001SnV?j009SnPl0
0003<e/004mO00A?G`03001B^e:k00IB^`03001>F4iH00E>F003001>=dhg00E>=`04001>5`000005
DUP01@00FWQJN5Yh00001EYh00<007ooOol037oo00<007ooOol01Woo000GOol00`00Oomoo`0>Ool0
0`008IX000050004;Q`00`00BW]:N`02BW/40008F[T00`00Hc`00007JcX00`00Kk]_^`03Kk/01000
HoYSnV?j2@000d:m00<001XM00002@0000=?G`00D[/01U:k00<004iHCUP01DiH00<004hgCSL014hg
0`001e9H00D005YhFWQJN00000EJN08000ioo`03001oogoo00Eoo`005goo00<007ooOol03goo00<0
026J8IX00P0000<0U000;Q`012hL100014Yk00<005ZiF[T00eZi0P000f<l00<006/jJcX00f/j00<0
06nkKk/00fnk00@006?jHoYSnPH00003G_/004:m00E2_@04000J7AXM6Qd30000108H000NOQin1`00
0U:k00<004iHCUP01TiH00<004hg00000`000Thh00<0059HDUP01E9H00<005YhFWP00UYh00L005Yh
FWQJN000H]P0000>Ool00`00Oomoo`05Ool001Moo`03001oogoo00moo`03000QVR6J008QVP030000
U000008^70<000M:N`04001J^EZiF[T20006Hc`01@00JcY[>V/j00001Fnk00<006?jHoX00P0000=K
_P0000001Ekk00<004:m@[d014:m00<001XM6Qd00QXM10000ain00<003YL>U`00SYL3@001Thh00<0
059HDUP01U9H00<005YhFWP00UYh00<005Yh00000f;H00<007ooOol037oo00<007ooOol017oo0005
Ool20004Ool20003Ool30004Ool2000AOol010008IXQVR6J1P000RhL00<004YkBW/01DYk00<005Zi
00002F<l00<006/j00001Fnk00D006?j000006]k008000QNn`030012_D:m00A2_@03000J7AXM008J
7@8000030EL00000008NOP03000jG3YL00@jG0030016>THj00E6>P03001>>4hh00I>>003001BF59H
00EBF003001JN5Yh00=JN003001Rf6;H009Rf003001oogoo00]oo`<000Eoo`0017oo00@007ooOol0
00=oo`8000Ioo`05001oogooOol0000BOol00`008IXQVP0300000`5J000000040004BW/60009Hc`5
0002Kk/30004Jg/20007G_/;00026Qd300001@5i000001in00001SYL00<004HjASX01DHj00<004hh
CSP01Dhh00<0059HDUP01U9H00@005YhFWQJN08000ERf003001oogoo00Moo`<0009oo`03001oogoo
00=oo`0017oo00@007ooOol000]oo`03001oogoo008001=oo`8000<QVP030001FP5J00800085o0@0
00I2o`03001S?6<l00AS?08000EOgP<000E[N`04001W6FLI0004G_/200000eZI001:FP05BUX00`00
<OXanP040000104i0001N@5i0P001SYL00<004HjASX01DHj00<004hhCSP01Dhh00<0059HDUP01U9H
00@005YhFWP0009Rf003001Rf6;H009Rf003001oogoo00Aoo`<000Eoo`03001oogoo00=oo`0017oo
00@007ooOol000Qoo`<000=oo`80011oo`@000DQVP030001FP5J008000<5o0030012od;o00I2o`03
001S?6<l009S?003001OgUoN00=OgP@000E[N`03001W6FLI009W6@04001Nn`000004FYT00`00BUY:
FP04BUX00`00<OXanP02<OX400020GT20006>U`00`00ASY6>P04ASX00`00CSQ>>006CSP00`00DUQB
F005DUP00`00FWP00004H]P00`00H]QRf002H]P00`00Ool000030008Ool00`00Oomoo`03Ool000Ao
o`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool30004Ool500000b6J0001FP020EX3
00000`Gl0012o`07@_l01000Hc`000001EoN0`001V]k00<006LIIaT016LI00<005ZIFYT015ZI00<0
04YJBUX014YJ00<0037j<OX00c7j00<0016J00000P000P5i0P0013YL00<004HjASX01DHj00<004hh
CSP01Dhh00<0059HDUP01E9H0P001V;H00@006;HH]QRf08000aoo`03001oogoo00=oo`001Goo0P00
2Goo10000Woo00<007ooOol02Goo0`001goo00<003hI?QT00ShI2`0014;o20000UoN1@001F]k00<0
06LIIaT016LI0P001eZI00<004YJBUX014YJ00<0037j<OX00c7j00<0016J00000P0000<1N@005M`0
0P000SYL00<004HjASX01DHj00<004hhCSP01Dhh00<0059HDUP00e9H0P0000=J^@00H]P01F;H00@0
06;HH]P0009_FP03001oogoo00Yoo`03001oogoo009oo`005Woo00<007ooOol01Woo0`002goo00<0
03hI?QT00chI00<003IM=Ud01CIM1@001RLM10000fMl00<006li00000P0000=[N`00IaT01FLI00@0
06:hH[P000MJV@03001:FTYJ00A:FP03000anS7j008anP04000AVQ6J4IX400025M`200000cYL0016
>P06ASX00`00CSQ>>004CSP01000DUQBF59H0P0015Zi00<006;HH]P016;H0P0016mJ00<007ooOol0
2Goo00<007ooOol00Woo000EOol20005Ool3000>Ool00`00?QTn6@04?QT00`00=UdfG@06=Ud00`00
6[h000060006Ig`00`00KcU_>@03KcT00`00IaUW6@04IaT01@00H[QR^6:h00001eZI00<004YJBUX0
14YJ00<0037j<OX00c7j00H0016J4IXAVP000?L200035M`20007ASX00`00CSQ>>005CSP30006F[T0
0`00H]QRf004H]P20005KeX00`00Oomoo`08Ool00`00Oomoo`02Ool001Eoo`04001oogooOol3000B
Ool00`00?QTn6@04?QT00`00=UdfG@04=Ud40004Egl00`00IgaWO004Ig`00`00KcU_>@04KcT00`00
IaUW6@03IaT00`00H[QR^003H[P00`00FYUJV@04FYT<00000c7j000AVP034IX200000`3g000Eg003
5M`50002ASX00`00CSQ>>002CSP40009F[T00`00H]QRf002H]P00`00KeX00004KeX00`00KoX00005
0004Ool00`00Oomoo`02Ool001Eoo`@001Ioo`03000n6ChI00@n6@03000fGCIM00<000AFf`800003
Egl006Ml00EWO003001_>Fli00A_>@03001_600000=W6@03001R^6:h00ER^004001JVEZIFYT20000
0eIG001:6005BQP01000?OPmn3gh10000Q6J00@0004i0000m`80008Eg003000f?3Hl008f?0L000=:
NP03001J^EZi00QJ^@04001Rf6;H0003KeX01000KeY_FP000foj2P0017oo000>Ool30004Ool2000H
Ool00`00?QTn6@04?QT30008E]/30005Ig`00`00KcU_>@04KcT01`00KaQ_66lH001W6@0000QR^004
001JV@000004EUL00`00BQQ:6004BQP00`00?OPmn002?OP00`00<MX0000300001@4i0000005I0000
0QGL00<003Hl=S`013Hl00<004YjBWX01DYj00<005ZiF[T01eZi00<006;H00001FmJ0P000foj00<0
04=c@g<00P0017oo0P0017oo000?Ool00`00Oomoo`05Ool00`00Oomoo`0FOol01000?QTn6ChI0P00
00=BN@00E]/01eKK00@006;iH_URn@<00003Ig`006li00E_>@03001_66lH00=_6003001R^6:h00MR
^08000IFE`03001:64XH00A:6003000mn3gh00<mn004000afS7J<MX200020CT00`000ET000020007
=S`00`00BWY:NP04BWX00`00F[UJ^@07F[T20006KeX20002KoX01000@g<000001goo00<007ooOol0
0Woo000?Ool00`00Oomoo`06Ool00`00Oomoo`0EOol01000?QT00000159i00<005KKE]/00eKK0P00
1f;i0P001Fli00<006lHKaP016lH00<006:hH[P01V:h00<005jH00001UIG00<004XHBQP014XH00<0
03gh?OP00cgh00<0037J<MX00c7J0P0000@1>@000ET1F@8000Hf?003001:NTYj00E:NP03001J^EZi
00EJ^@03001SG00000E_FP04001_nP00KoX3000:Ool00`00Oomoo`01Ool000moo`03001oogoo00Mo
o`03001oogoo01Eoo`03001BNE9i00EBN@05001FfeKKE]/00009H_T00`00J_P00003KcT00`00KaQ_
6004KaP00`00J_P00006H[P01000GYQNV0001eIG00<004XHBQP014XH00<003gh?OP00cgh00<0037J
<MX00S7J00<001GK00000`0000<1F@00=S`01SHl00<004YjBWX014Yj00<005ZiF[T00eZi0P000f=L
00D006mJKeY_FP0000=_nP8000aoo`03001oogoo005oo`003goo00<007ooOol027oo0P005Goo00<0
059iDWT01U9i00<005KK00002F;i00L006[hJ_QZn000KcT00006KaP01@00J_QZn6[h000016:h00<0
05jHGYP00UjH00<005IGEUL015IG00<004XHBQP014XH00<003gh?OP00cgh00<0037J<MX00c7J00<0
01GK5M/00`0000@1F@00000000@f?003001:NTYj00E:NP03001J^EZi008000ISG003001_FP0000=_
nP03001oo`0000aoo`03001oogoo005oo`003Woo0P0037oo00<007ooOol04goo00<0059iDWT01U9i
00<006;iH_T01f;i00<006[hJ_P00f[h00<006lHKaP016lH00<006[hJ_P00V[h00D006:hH[QR^000
00ENV003001FEeIG00EFE`03001:64XH00=:6003000mn3gh00@mn003000afS7J00<afP04000EfaGK
5M/400000aXm000000030007BWX00`00F[T00008He`400001Foj001oogoo000037oo00<007ooOol0
0Goo000MOol00`00Oomoo`0BOol00`00DWUBN@05DWT00`00GYP00007H_T00`00J_QZn004J_P20005
KaP00`00J_QZn003J_P00`00H[P00006GYP00`00EUMFE`05EUL00`00BQQ:6004BQP00`00?OPmn003
?OP00`00<MXafP04<MX00`005M/Ef`025M/300046Sd:0008He`01000DmaCg5?L0P0017oo00<007oo
Ool02Goo00<007ooOol00Goo000NOol00`00Oomoo`0BOol00`00DWUBN@03DWT01@00GYQNV5jH0000
1F;i00<006[hJ_P016[h00@006lHKaP000=_6003001Zn6[h00EZn003001NV5jH00INV003001FEeIG
00AFE`04001:64XHBQPF00000aGK000000056Sd00`00A_i6oP05A_h00`00HeaSG003He`20002Dm`2
0007Ool00`00Oomoo`08Ool00`00Oomoo`01Ool001moo`80019oo`03001BNE9i009BN@03001NV5jH
00=NV005001RnF;iH_T00006J_P00`00KaQ_6002KaP00`00KaP00008J_P00`00GYQNV006GYP00`00
EUMFE`04EUL40003CQL00`00BQM:5`04BQL00`00ASU6>@04AST00`00?W`nO002?W`600036Sd00`00
A_i6oP05A_h01@00HeaSG6=L00000U?L0P002Woo00<007ooOol01goo00<007ooOol00Goo000QOol0
0`00Oomoo`0@Ool01000DWUBN@001ejH00<006;i00001f[h00<006lHKaP00flH00<006[hJ_P01V[h
00<006KH00001ejH00@005IGEUMFE`<00003DSL004hG00E>5`03001:5dXG00A:5`030016>DHi00A6
>@03000nO3il00DnO003000N_P0000@000M6oP04001SG000Dm`2000<Ool00`00Oomoo`08Ool10001
Ool1Ool0029oo`03001oogoo00moo`03001BN@0000UNV003001Zn6[h00EZn003001_66lH00A_6003
001Zn6[h00EZn004001Vf6KH0008GYP30004DSL00`00CQM>5`04CQL00`00BQM:5`04BQL00`00ASU6
>@04AST00`00?W`nO005?W`30002Dil=000?Ool00`00Oomoo`07Ool10001Ool1Ool002=oo`03001o
ogoo00moo`03001NV5jH00QNV003001Zn6[h00AZn003001_66lH00A_6003001_>00000EZn003001V
f6KH009Vf003001NV5jH00ANV08000MB=`03001>5dhG00=>5`03001:5dXG00A:5`030016>DHi00E6
>@04000nO3il?W`30003E]/00`00Dil00003Ji`5000DOol00`00Oomoo`06Ool10001Ool1Ool002Ao
o`03001oogoo00ioo`03001NV5jH00MNV003001Vf00000AZn003001_66lH00A_6005001_>6lhKcP0
0003J_P00`00I]QVf004I]P01000GYQNV5jH0P0000=FN000DSL01U8g00<004hGCQL014hG00<004XG
BQL014XG00<004HiAST014Hi00@003il000000IFf`<000=[W003001oogoo01Moo`03001oogoo00Eo
o`40005oo`5oo`009Goo0P003goo00<005jHGYP01EjH00D006KHI]QVf00000=Zn003001_66lH00=_
6003001_>6lh00=_>003001Zn00000MVf004001NV0000004EWP00`00DSMB=`04DSL00`00CQM>5`04
CQL00`00BQM:5`04BQL00`00ASU6>@03AST30007E]/01`00I_YVnVKj001[W00001Yoo`03001oogoo
00Eoo`40005oo`5oo`009goo00<007ooOol037oo00<005jHGYP015jH00<006KHI]P00fKH00<006[h
00001FlH00<006lhKcP01Flh00<006KHI]P01VKH0P001UIh00<0058gDSL01E8g00<004hGCQL014hG
00<004XGBQL00dXG00<004HiAST00`000U9h00<005KKE]/00eKK0P001FKj00<007ooOol06Woo00<0
07ooOol017oo0@000Goo0Goo000XOol00`00Oomoo`0<Ool00`00GYQNV002GYP00`00I]QVf005I]P2
0005KaP00`00KcQ_>004KcP20007I]P00`00G]T00006EWP00`00DSMB=`05DSL00`00CQM>5`04CQL0
0`00BQM:5`03BQL30006DWP01@00E]]FfeKK00001fKj00<007ooOol06goo00<007ooOol00goo0@00
0Goo0Goo000YOol00`00Oomoo`0<Ool01000GYQNV00026KH00<006lH00000flH00<006lhKcP016lh
00@006mIKeT000AVf08000=Nf@03001FN5Ih00AFN003001B=e8g00AB=`03001>5dhG00A>5`03001:
5dXG00<00003DSL0059h00MBN003001Ff`0000MVnP03001oogoo01aoo`03001oogoo00Aoo`40005o
o`00:Woo0P0037oo00<005jH000026KH00L006lHKaQ_6000KaP00006KcP00`00KeU_F@02KeT01000
I]QVf0001EkI00<005IhEWP015Ih00<0058gDSL0158g00<004hGCQL00dhG10001E8g00<0059hDWP0
1U9h00<006KjI_X01FKj00<007ooOol07Woo00<007ooOol00goo0@000Goo000/Ool00`00Oomoo`0:
Ool00`00I]QVf006I]P00`00KaQ_6003KaP20005KcP00`00KeU_F@03KeT00`00I]P00006G]T00`00
EWQFN005EWP00`00DSMB=`04DSL00`00CQL000020002DSL00`00DSMB=`04DSL00`00DWQBN005DWP2
0006I_X00`00Oomoo`0POol00`00Oomoo`02Ool10001Ool002eoo`03001oogoo00Uoo`03001Vf6KH
00EVf003001_66lH00A_6003001cN@0000=_>003001_FFmI00A_F@8000QNf@03001FN5Ih00AFN003
001B=e8g00=B=`<000EB=`03001B=e8g00EB=`03001BN59h00=BN004001NV5jH0005I_X00`00Oomo
o`0QOol01@00Oomoogoo00000Goo000^Ool00`00Oomoo`09Ool00`00I]QVf003I]P00`00KaQ_6004
KaP01`00LgUcNG=i001_>00000I_F@03001WG00000QNf@03001FN5Ih00AFN003001B=e8g00<00003
DSL0058g00IB=`03001B=e8g00AB=`03001BN59h009BN003001NV5jH009NV005001VnVKjI_X0000T
Ool01@00Oomoogoo00000Goo000_Ool2000:Ool00`00I]QVf002I]P00`00KaQ_6003KaP00`00LgUc
N@03LgT20004KeT20003Ie`00`00G]UNf@05G]T00`00EWQFN002EWP50005DSL00`00DSMB=`04DSL0
0`00DSMB=`05DSL01000DWQBN0001UjH00<006Kj00009Woo00@007ooOol0005oo`00<Goo00<007oo
Ool01goo00D006KHI]QVf00000E_6003001cNG=i00=cN@03001cfg?K00800003KeT006ML00EWG003
001NfEkI00ANf@D000=BF003001B=e8g00AB=`03001B=e8g00AB=`03001B=e8g00EB=`03001BN000
00QNV003001oogoo02Ioo`03001oo`00005oo`00<Woo00<007ooOol01goo00<006KH00001FlH00<0
07=iLgT00g=i00<007?KLm/00W?K0`001fML00<005kIG]T00`0000=BV`00DUP01E9H00<0058gDSL0
158g00<0058gDSL01E8g00<0058gDSL0158g0P002EjH00<007ooOol09Woo00<007ooOol00@00000c
Ool00`00Oomoo`07Ool20005KaP00`00LgUcN@02LgT00`00Lm]cf`02Lm/01000GoaOo5ol0P000fML
1@001E:K00<0059HDUP0159H00<0058gDSL0158g00<0058gDSL0158g00<0058gDSL00U8g0P0000=J
M`00GYP01ejH00<007ooOol0:7oo0@000Goo0@00000dOol20009Ool01@00KaQ_66lH000017=i00<0
07?KLm/00W?K00<005olGo`00eol10000dmO00<005:KDY/015:K00<0059HDUP0159H00<0058gDSL0
158g00<0058gDSL0158g00D0058gDSMB=`0000AJM`03001NV5jH00ANV003001oogoo02Yoo`800000
=Woo00<007ooOol01goo00<006lH000017=i00<007?KLm/00W?K00@005olGoaOo0@000035]H004mO
00A?G`03001BVe:K00ABV`03001BF59H00=BF003001B=e8g00AB=`03001B=e8g00EB=`<000EJM`03
001NV5jH00=NV003001oogoo02Yoo`<00000=goo00<007ooOol01goo0`000W=i00<007?KLm/00W?K
1@0017oo0`001DmO00<005:KDY/00e:K00<0059HDUP0159H00<0058gDSL0158g00<0058gDSL00e8g
0P0025Yg00D005jHGYQNV00002Uoo`@000=oo`00>7oo00<007ooOol02Goo20002Goo00<007oo0000
0`0000=?G`00DY/015:K00<0059HDUP0159H00<0058gDSL0158g00@0058gDSMB=`800003EUL005Yg
00MJM`04001NV5jH000VOol40007Ool003Uoo`03001oogoo01Uoo`04001oogooOol70003DY/00`00
DUQBF003DUP00`00DSMB=`04DSL01000DSL0000015IG00<005YgFWL01EYg00<005jH000097oo0`00
2goo000jOol2000IOol00`00Oomoo`03Ool30002=U`40006DUP01000DSMB=e8g10001UIG00<005Yg
FWL01EYg0P008Goo10003Woo000lOol00`00Oomoo`0FOol00`00Oomoo`05Ool20004=U`;0003CSP0
0`00EUMFE`05EUL00`00FWMJM`03FWL00`00Oomoo`0MOol4000BOol003eoo`03001oogoo01Eoo`03
001oogoo00Ioo`8000@fG0030016>THj00=6>P03001>>4hh00E>>003001FEeIG00AFE`03001JMeYg
009JM`03001oogoo01Yoo`@001Ioo`00?Woo00<007ooOol057oo00<007ooOol027oo00<003IL=U`0
0SIL00<004HjASX00dHj00<004hhCSP014hh00<005IGEUL01EIG00@005YgFWL001Uoo`@001Yoo`00
?goo0P0057oo00<007ooOol02Goo0P000SIL00<004HjASX00dHj00<004hhCSP014hh00<005IGEUL0
1EIG00<005Yg00005Woo10007Woo0011Ool00`00Oomoo`0@Ool00`00Oomoo`0<Ool01000=U`fG000
1DHj00<004hhCSP014hh00<005IGEUL015IG0P0057oo0`008Woo0012Ool00`00Oomoo`0?Ool00`00
Oomoo`0=Ool30005ASX00`00CSQ>>004CSP00`00EUMFE`02EUL2000BOol4000UOol004=oo`03001o
ogoo00ioo`03001oogoo00moo`030016>THj00A6>P03001>>4hh00=>>005001FEeIGEUL0000@Ool4
000YOol004Aoo`8000ioo`03001oogoo011oo`D00003ASX004hh00E>>0<000eoo`@002eoo`00AWoo
00<007ooOol02goo00<007ooOol05Goo2@002goo1000<Goo0017Ool00`00Oomoo`0:Ool00`00Oomo
o`0UOol4000eOol004Qoo`03001oogoo00Uoo`03001oogoo025oo`@003Uoo`00BGoo0P002Goo00<0
07ooOol07Woo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`0JOol40010Ool004aoo`03001o
ogoo00Eoo`03001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo1000B7oo
001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo00Yoo`@0
051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001COol20000
0goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo
0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128698, 0.00557334, 
  0.00599054, 0.00599054}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.11022e-16 1.04977 -0.0679587 1.04977 [
[.13491 .20394 -11.9698 -9 ]
[.13491 .20394 .03022 0 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.55031 .04057 -5.18851 -9 ]
[.55031 .04057 .81149 0 ]
[.76162 .0881 0 -6.17057 ]
[.76162 .0881 12 2.82943 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.93925 .36012 0 -5.91342 ]
[.93925 .36012 6 3.08658 ]
[.02433 .26374 -12 -2.74116 ]
[.02433 .26374 0 6.25884 ]
[.0158 .31724 -24 -2.78639 ]
[.0158 .31724 0 6.21361 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.0022 .43003 -18 -2.88226 ]
[-0.0022 .43003 0 6.11774 ]
[-0.01169 .48953 -6 -2.93311 ]
[-0.01169 .48953 0 6.06689 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74932 .09266 m
.74316 .09495 L
s
[(-2)] .76162 .0881 -1 .37124 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.92673 .36405 m
.92047 .36602 L
s
[(2)] .93925 .36012 -1 .31409 Mshowa
.125 Mabswid
.77428 .13086 m
.77058 .1322 L
s
.79837 .1677 m
.79466 .16902 L
s
.82162 .20326 m
.8179 .20455 L
s
.86578 .27081 m
.86204 .27206 L
s
.88677 .30292 m
.88302 .30414 L
s
.90707 .33398 m
.90332 .33518 L
s
.72342 .05305 m
.71974 .05445 L
s
.69654 .01193 m
.69287 .01336 L
s
.94577 .39318 m
.94201 .39434 L
s
.96423 .4214 m
.96046 .42254 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03655 .25896 m
.04267 .25657 L
s
[(-1)] .02433 .26374 1 -0.39085 Mshowa
.02806 .31257 m
.03419 .31024 L
s
[(-0.5)] .0158 .31724 1 -0.3808 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0)] .00696 .37264 1 -0.37036 Mshowa
.01015 .42559 m
.01632 .42337 L
s
[(0.5)] -0.0022 .43003 1 -0.3595 Mshowa
.0007 .48521 m
.0069 .48306 L
s
[(1)] -0.01169 .48953 1 -0.3482 Mshowa
.125 Mabswid
.03488 .26953 m
.03855 .26811 L
s
.03319 .28018 m
.03686 .27876 L
s
.03149 .2909 m
.03517 .28949 L
s
.02978 .3017 m
.03346 .30029 L
s
.02632 .32352 m
.03001 .32212 L
s
.02458 .33454 m
.02826 .33315 L
s
.02282 .34564 m
.0265 .34426 L
s
.02105 .35682 m
.02474 .35545 L
s
.01747 .37942 m
.02116 .37806 L
s
.01566 .39084 m
.01935 .38949 L
s
.01383 .40234 m
.01753 .40099 L
s
.012 .41392 m
.0157 .41258 L
s
.00829 .43734 m
.01199 .43602 L
s
.00641 .44918 m
.01012 .44786 L
s
.00452 .4611 m
.00823 .45979 L
s
.00262 .47311 m
.00633 .47181 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.385 0 0 r
.37996 .65919 .40273 .67205 .43917 .70435 .41684 .68118 Metetra
.026 .407 .857 r
.41684 .68118 .43917 .70435 .47709 .73525 .45501 .70187 Metetra
.392 .533 .878 r
.45501 .70187 .47709 .73525 .51651 .75603 .49451 .71469 Metetra
.571 .581 .824 r
.49451 .71469 .51651 .75603 .55707 .7596 .53512 .71439 Metetra
.666 .613 .786 r
.53512 .71439 .55707 .7596 .59813 .74248 .57636 .69838 Metetra
.728 .646 .766 r
.57636 .69838 .59813 .74248 .63904 .70582 .61771 .66752 Metetra
.777 .686 .764 r
.61771 .66752 .63904 .70582 .67935 .65515 .65882 .62589 Metetra
.818 .74 .778 r
.65882 .62589 .67935 .65515 .719 .59875 .6996 .57967 Metetra
.848 .814 .816 r
.6996 .57967 .719 .59875 .7584 .54557 .7403 .53567 Metetra
.841 .911 .883 r
.7403 .53567 .7584 .54557 .79829 .50335 .78148 .49995 Metetra
.686 .963 .94 r
.78148 .49995 .79829 .50335 .83959 .47748 .82384 .47678 Metetra
.277 .78 .844 r
.82384 .47678 .83959 .47748 .88324 .47036 .86811 .46809 Metetra
0 .527 .728 r
.86811 .46809 .88324 .47036 .92991 .48123 .91484 .47326 Metetra
0 .442 .78 r
.91484 .47326 .92991 .48123 .97986 .50616 .96423 .48915 Metetra
.111 .622 .894 r
.35628 .65051 .37996 .65919 .41684 .68118 .39403 .65543 Metetra
.465 .634 .928 r
.39403 .65543 .41684 .68118 .45501 .70187 .43271 .65948 Metetra
.565 .573 .82 r
.43271 .65948 .45501 .70187 .49451 .71469 .47238 .65953 Metetra
.609 .552 .765 r
.47238 .65953 .49451 .71469 .53512 .71439 .51292 .65312 Metetra
.64 .556 .743 r
.51292 .65312 .53512 .71439 .57636 .69838 .55414 .6391 Metetra
.67 .577 .742 r
.55414 .6391 .57636 .69838 .61771 .66752 .59578 .61783 Metetra
.703 .619 .761 r
.59578 .61783 .61771 .66752 .65882 .62589 .63766 .59119 Metetra
.743 .695 .806 r
.63766 .59119 .65882 .62589 .6996 .57967 .67971 .56206 Metetra
.781 .831 .89 r
.67971 .56206 .6996 .57967 .7403 .53567 .72204 .53377 Metetra
.682 .968 .927 r
.72204 .53377 .7403 .53567 .78148 .49995 .76492 .50937 Metetra
.76492 .50937 .78148 .49995 .82384 .47678 .80871 .49108 Metetra
.142 0 0 r
.80871 .49108 .82384 .47678 .86811 .46809 .85377 .47993 Metetra
.227 0 0 r
.85377 .47993 .86811 .46809 .91484 .47326 .90039 .47558 Metetra
.094 0 0 r
.90039 .47558 .91484 .47326 .96423 .48915 .94869 .47636 Metetra
.664 .899 .979 r
.33167 .64378 .35628 .65051 .39403 .65543 .37071 .62808 Metetra
.687 .709 .865 r
.37071 .62808 .39403 .65543 .43271 .65948 .41022 .61244 Metetra
.657 .601 .781 r
.41022 .61244 .43271 .65948 .47238 .65953 .4502 .59763 Metetra
.636 .551 .742 r
.4502 .59763 .47238 .65953 .51292 .65312 .4907 .58424 Metetra
.624 .531 .729 r
.4907 .58424 .51292 .65312 .55414 .6391 .53178 .57253 Metetra
.618 .533 .736 r
.53178 .57253 .55414 .6391 .59578 .61783 .57352 .56242 Metetra
.618 .561 .768 r
.57352 .56242 .59578 .61783 .63766 .59119 .61599 .55345 Metetra
.622 .635 .842 r
.61599 .55345 .63766 .59119 .67971 .56206 .65923 .54491 Metetra
.592 .793 .976 r
.65923 .54491 .67971 .56206 .72204 .53377 .70324 .53596 Metetra
.228 .747 .77 r
.70324 .53596 .72204 .53377 .76492 .50937 .74798 .52578 Metetra
.147 0 0 r
.74798 .52578 .76492 .50937 .80871 .49108 .79338 .51374 Metetra
.223 0 0 r
.79338 .51374 .80871 .49108 .85377 .47993 .83935 .49955 Metetra
.149 0 0 r
.83935 .49955 .85377 .47993 .90039 .47558 .88584 .48331 Metetra
.88584 .48331 .90039 .47558 .94869 .47636 .93285 .46546 Metetra
.84 .882 .872 r
.30621 .63622 .33167 .64378 .37071 .62808 .34683 .60033 Metetra
.78 .731 .807 r
.34683 .60033 .37071 .62808 .41022 .61244 .38741 .56583 Metetra
.717 .632 .762 r
.38741 .56583 .41022 .61244 .4502 .59763 .4279 .53707 Metetra
.662 .569 .74 r
.4279 .53707 .4502 .59763 .4907 .58424 .46844 .51717 Metetra
.614 .529 .735 r
.46844 .51717 .4907 .58424 .53178 .57253 .50936 .50757 Metetra
.566 .507 .748 r
.50936 .50757 .53178 .57253 .57352 .56242 .55106 .5078 Metetra
.511 .506 .787 r
.55106 .5078 .57352 .56242 .61599 .55345 .5939 .51555 Metetra
.426 .539 .868 r
.5939 .51555 .61599 .55345 .65923 .54491 .63807 .52693 Metetra
.218 .6 .949 r
.63807 .52693 .65923 .54491 .70324 .53596 .68353 .53716 Metetra
.15 0 0 r
.68353 .53716 .70324 .53596 .74798 .52578 .73 .54137 Metetra
.24 0 0 r
.73 .54137 .74798 .52578 .79338 .51374 .77697 .53575 Metetra
.148 0 0 r
.77697 .53575 .79338 .51374 .83935 .49955 .82391 .51843 Metetra
.82391 .51843 .83935 .49955 .88584 .48331 .8704 .49004 Metetra
.591 .923 .705 r
.8704 .49004 .88584 .48331 .93285 .46546 .91632 .45352 Metetra
.874 .83 .801 r
.28009 .62501 .30621 .63622 .34683 .60033 .32228 .57336 Metetra
.824 .741 .773 r
.32228 .57336 .34683 .60033 .38741 .56583 .36403 .52442 Metetra
.762 .669 .759 r
.36403 .52442 .38741 .56583 .4279 .53707 .40516 .48513 Metetra
.692 .607 .757 r
.40516 .48513 .4279 .53707 .46844 .51717 .4459 .46032 Metetra
.609 .552 .765 r
.4459 .46032 .46844 .51717 .50936 .50757 .48677 .45218 Metetra
.504 .499 .784 r
.48677 .45218 .50936 .50757 .55106 .5078 .5284 .46 Metetra
.361 .448 .816 r
.5284 .46 .55106 .5078 .5939 .51555 .57141 .48024 Metetra
.151 .399 .846 r
.57141 .48024 .5939 .51555 .63807 .52693 .61616 .50686 Metetra
0 .35 .8 r
.61616 .50686 .63807 .52693 .68353 .53716 .66266 .53206 Metetra
.264 0 0 r
.66266 .53206 .68353 .53716 .73 .54137 .71045 .5477 Metetra
.153 0 0 r
.71045 .5477 .73 .54137 .77697 .53575 .75875 .54718 Metetra
.75875 .54718 .77697 .53575 .82391 .51843 .80663 .52723 Metetra
.586 .916 .686 r
.80663 .52723 .82391 .51843 .8704 .49004 .85336 .48896 Metetra
.867 .995 .799 r
.85336 .48896 .8704 .49004 .91632 .45352 .8987 .43752 Metetra
.87 .789 .769 r
.2535 .60783 .28009 .62501 .32228 .57336 .29696 .54806 Metetra
.844 .752 .762 r
.29696 .54806 .32228 .57336 .36403 .52442 .33972 .49176 Metetra
.8 .717 .773 r
.33972 .49176 .36403 .52442 .40516 .48513 .38154 .44711 Metetra
.729 .676 .798 r
.38154 .44711 .40516 .48513 .4459 .46032 .42266 .41972 Metetra
.609 .616 .833 r
.42266 .41972 .4459 .46032 .48677 .45218 .46367 .41208 Metetra
.414 .522 .859 r
.46367 .41208 .48677 .45218 .5284 .46 .50535 .42341 Metetra
.145 .388 .839 r
.50535 .42341 .5284 .46 .57141 .48024 .54845 .44962 Metetra
0 .265 .771 r
.54845 .44962 .57141 .48024 .61616 .50686 .59349 .48365 Metetra
0 .243 .729 r
.59349 .48365 .61616 .50686 .66266 .53206 .64052 .51625 Metetra
0 .387 .792 r
.64052 .51625 .66266 .53206 .71045 .5477 .6891 .53756 Metetra
.223 .696 .939 r
.6891 .53756 .71045 .5477 .75875 .54718 .73828 .53947 Metetra
.645 .933 .969 r
.73828 .53947 .75875 .54718 .80663 .52723 .78696 .51793 Metetra
.853 .955 .877 r
.78696 .51793 .80663 .52723 .85336 .48896 .83422 .47431 Metetra
.913 .908 .803 r
.83422 .47431 .85336 .48896 .8987 .43752 .87976 .415 Metetra
.848 .757 .762 r
.22664 .58345 .2535 .60783 .29696 .54806 .27073 .52489 Metetra
.849 .767 .772 r
.27073 .52489 .29696 .54806 .33972 .49176 .31414 .46966 Metetra
.829 .781 .807 r
.31414 .46966 .33972 .49176 .38154 .44711 .35657 .42566 Metetra
.766 .791 .874 r
.35657 .42566 .38154 .44711 .42266 .41972 .39821 .39834 Metetra
.592 .755 .956 r
.39821 .39834 .42266 .41972 .46367 .41208 .43962 .39014 Metetra
.236 .582 .944 r
.43962 .39014 .46367 .41208 .50535 .42341 .48159 .40026 Metetra
0 .341 .803 r
.48159 .40026 .50535 .42341 .54845 .44962 .52489 .42476 Metetra
0 .238 .73 r
.52489 .42476 .54845 .44962 .59349 .48365 .57005 .45676 Metetra
0 .304 .788 r
.57005 .45676 .59349 .48365 .64052 .51625 .61722 .48732 Metetra
.171 .485 .9 r
.61722 .48732 .64052 .51625 .6891 .53756 .66601 .50694 Metetra
.488 .657 .935 r
.66601 .50694 .6891 .53756 .73828 .53947 .71557 .5078 Metetra
.699 .745 .887 r
.71557 .5078 .73828 .53947 .78696 .51793 .76483 .48602 Metetra
.809 .78 .827 r
.76483 .48602 .78696 .51793 .83422 .47431 .8129 .44292 Metetra
.862 .797 .785 r
.8129 .44292 .83422 .47431 .87976 .415 .85941 .38464 Metetra
.81 .728 .774 r
.19965 .55196 .22664 .58345 .27073 .52489 .24351 .50379 Metetra
.835 .788 .807 r
.24351 .50379 .27073 .52489 .31414 .46966 .28699 .45791 Metetra
.834 .868 .869 r
.28699 .45791 .31414 .46966 .35657 .42566 .3298 .42049 Metetra
.738 .94 .951 r
.3298 .42049 .35657 .42566 .39821 .39834 .37206 .39587 Metetra
.396 .842 .927 r
.37206 .39587 .39821 .39834 .43962 .39014 .41419 .38602 Metetra
0 .548 .754 r
.41419 .38602 .43962 .39014 .48159 .40026 .4568 .39029 Metetra
0 .378 .706 r
.4568 .39029 .48159 .40026 .52489 .42476 .50055 .40549 Metetra
0 .39 .813 r
.50055 .40549 .52489 .42476 .57005 .45676 .54588 .4262 Metetra
.173 .476 .895 r
.54588 .4262 .57005 .45676 .61722 .48732 .59297 .44552 Metetra
.413 .539 .875 r
.59297 .44552 .61722 .48732 .66601 .50694 .64156 .45628 Metetra
.564 .58 .828 r
.64156 .45628 .66601 .50694 .71557 .5078 .69105 .45273 Metetra
.661 .616 .793 r
.69105 .45273 .71557 .5078 .76483 .48602 .74064 .43203 Metetra
.731 .655 .774 r
.74064 .43203 .76483 .48602 .8129 .44292 .78965 .39514 Metetra
.787 .703 .771 r
.78965 .39514 .8129 .44292 .85941 .38464 .83775 .34655 Metetra
.743 .694 .804 r
.17251 .51482 .19965 .55196 .24351 .50379 .21524 .48411 Metetra
.779 .81 .877 r
.21524 .48411 .24351 .50379 .28699 .45791 .25812 .45425 Metetra
.743 .949 .946 r
.25812 .45425 .28699 .45791 .3298 .42049 .30099 .42834 Metetra
.438 .873 .797 r
.30099 .42834 .3298 .42049 .37206 .39587 .34388 .40865 Metetra
.34388 .40865 .37206 .39587 .41419 .38602 .387 .39623 Metetra
.074 0 0 r
.387 .39623 .41419 .38602 .4568 .39029 .43068 .39073 Metetra
.003 0 0 r
.43068 .39073 .4568 .39029 .50055 .40549 .47526 .39045 Metetra
.288 .699 .973 r
.47526 .39045 .50055 .40549 .54588 .4262 .52102 .39259 Metetra
.484 .635 .921 r
.52102 .39259 .54588 .4262 .59297 .44552 .56805 .39371 Metetra
.558 .572 .824 r
.56805 .39371 .59297 .44552 .64156 .45628 .61626 .39043 Metetra
.599 .549 .77 r
.61626 .39043 .64156 .45628 .69105 .45273 .6654 .38007 Metetra
.632 .552 .747 r
.6654 .38007 .69105 .45273 .74064 .43203 .7151 .36138 Metetra
.665 .577 .746 r
.7151 .36138 .74064 .43203 .78965 .39514 .76502 .33477 Metetra
.702 .625 .767 r
.76502 .33477 .78965 .39514 .83775 .34655 .815 .30227 Metetra
.613 .638 .852 r
.14505 .47451 .17251 .51482 .21524 .48411 .18592 .46482 Metetra
.595 .791 .974 r
.18592 .46482 .21524 .48411 .25812 .45425 .2276 .45468 Metetra
.378 .84 .891 r
.2276 .45468 .25812 .45425 .30099 .42834 .27016 .44326 Metetra
.27016 .44326 .30099 .42834 .34388 .40865 .31361 .42991 Metetra
.005 0 0 r
.31361 .42991 .34388 .40865 .387 .39623 .35794 .41434 Metetra
.35794 .41434 .387 .39623 .43068 .39073 .40307 .39662 Metetra
.418 .863 .807 r
.40307 .39662 .43068 .39073 .47526 .39045 .44892 .37725 Metetra
.679 .889 .975 r
.44892 .37725 .47526 .39045 .52102 .39259 .49546 .35697 Metetra
.684 .715 .872 r
.49546 .35697 .52102 .39259 .56805 .39371 .54267 .33667 Metetra
.651 .603 .788 r
.54267 .33667 .56805 .39371 .61626 .39043 .5906 .31718 Metetra
.627 .547 .745 r
.5906 .31718 .61626 .39043 .6654 .38007 .63933 .29912 Metetra
.613 .525 .731 r
.63933 .29912 .6654 .38007 .7151 .36138 .68898 .28277 Metetra
.607 .527 .739 r
.68898 .28277 .7151 .36138 .76502 .33477 .73967 .26803 Metetra
.609 .559 .773 r
.73967 .26803 .76502 .33477 .815 .30227 .79149 .25442 Metetra
.344 .512 .88 r
.11694 .43396 .14505 .47451 .18592 .46482 .15563 .4446 Metetra
.123 .561 .921 r
.15563 .4446 .18592 .46482 .2276 .45468 .19573 .45403 Metetra
.094 0 0 r
.19573 .45403 .2276 .45468 .27016 .44326 .23769 .45729 Metetra
.078 0 0 r
.23769 .45729 .27016 .44326 .31361 .42991 .2816 .45045 Metetra
.2816 .45045 .31361 .42991 .35794 .41434 .32721 .43161 Metetra
.442 .845 .656 r
.32721 .43161 .35794 .41434 .40307 .39662 .37403 .4014 Metetra
.769 .995 .865 r
.37403 .4014 .40307 .39662 .44892 .37725 .4215 .36286 Metetra
.842 .893 .876 r
.4215 .36286 .44892 .37725 .49546 .35697 .46919 .32064 Metetra
.787 .746 .816 r
.46919 .32064 .49546 .35697 .54267 .33667 .51692 .27989 Metetra
.718 .639 .769 r
.51692 .27989 .54267 .33667 .5906 .31718 .56483 .24528 Metetra
.656 .568 .744 r
.56483 .24528 .5906 .31718 .63933 .29912 .6133 .2201 Metetra
.602 .522 .737 r
.6133 .2201 .63933 .29912 .68898 .28277 .66289 .20584 Metetra
.552 .499 .75 r
.66289 .20584 .68898 .28277 .73967 .26803 .71415 .20199 Metetra
.502 .502 .789 r
.71415 .20199 .73967 .26803 .79149 .25442 .76747 .20607 Metetra
0 .268 .774 r
.08776 .39599 .11694 .43396 .15563 .4446 .12447 .42214 Metetra
0 .2 .642 r
.12447 .42214 .15563 .4446 .19573 .45403 .16305 .44684 Metetra
.319 0 0 r
.16305 .44684 .19573 .45403 .23769 .45729 .20434 .46176 Metetra
.039 0 0 r
.20434 .46176 .23769 .45729 .2816 .45045 .24858 .4601 Metetra
.42 .861 .778 r
.24858 .4601 .2816 .45045 .32721 .43161 .29538 .43849 Metetra
.773 .997 .857 r
.29538 .43849 .32721 .43161 .37403 .4014 .34388 .39808 Metetra
.891 .948 .839 r
.34388 .39808 .37403 .4014 .4215 .36286 .39307 .34421 Metetra
.886 .853 .805 r
.39307 .34421 .4215 .36286 .46919 .32064 .44214 .28482 Metetra
.838 .763 .78 r
.44214 .28482 .46919 .32064 .51692 .27989 .49071 .22846 Metetra
.77 .682 .766 r
.49071 .22846 .51692 .27989 .56483 .24528 .53895 .18248 Metetra
.69 .611 .763 r
.53895 .18248 .56483 .24528 .6133 .2201 .58746 .15192 Metetra
.599 .548 .769 r
.58746 .15192 .6133 .2201 .66289 .20584 .6371 .13902 Metetra
.496 .495 .786 r
.6371 .13902 .66289 .20584 .71415 .20199 .68876 .143 Metetra
.378 .456 .815 r
.68876 .143 .71415 .20199 .76747 .20607 .74314 .16016 Metetra
0 .055 .571 r
.05707 .36271 .08776 .39599 .12447 .42214 .09257 .39637 Metetra
.482 0 0 r
.09257 .39637 .12447 .42214 .16305 .44684 .1302 .42859 Metetra
0 .271 .662 r
.1302 .42859 .16305 .44684 .20434 .46176 .17106 .44926 Metetra
.221 .696 .939 r
.17106 .44926 .20434 .46176 .24858 .4601 .21557 .45002 Metetra
.689 .916 .971 r
.21557 .45002 .24858 .4601 .29538 .43849 .26328 .42671 Metetra
.852 .905 .871 r
.26328 .42671 .29538 .43849 .34388 .39808 .31313 .38076 Metetra
.89 .858 .803 r
.31313 .38076 .34388 .39808 .39307 .34421 .36379 .31877 Metetra
.887 .815 .771 r
.36379 .31877 .39307 .34421 .44214 .28482 .41418 .25047 Metetra
.861 .776 .766 r
.41418 .25047 .44214 .28482 .49071 .22846 .46378 .1861 Metetra
.812 .735 .78 r
.46378 .1861 .49071 .22846 .53895 .18248 .51269 .13429 Metetra
.73 .685 .807 r
.51269 .13429 .53895 .18248 .58746 .15192 .56161 .10083 Metetra
.604 .617 .838 r
.56161 .10083 .58746 .15192 .6371 .13902 .61153 .08825 Metetra
.431 .527 .855 r
.61153 .08825 .6371 .13902 .68876 .143 .66355 .09568 Metetra
.24 .433 .85 r
.66355 .09568 .68876 .143 .74314 .16016 .71854 .11887 Metetra
.488 0 0 r
.02447 .33522 .05707 .36271 .09257 .39637 .06005 .36673 Metetra
0 .12 .606 r
.06005 .36673 .09257 .39637 .1302 .42859 .09777 .39678 Metetra
0 .398 .848 r
.09777 .39678 .1302 .42859 .17106 .44926 .13879 .41564 Metetra
.451 .655 .947 r
.13879 .41564 .17106 .44926 .21557 .45002 .1836 .41527 Metetra
.699 .744 .886 r
.1836 .41527 .21557 .45002 .26328 .42671 .2318 .39164 Metetra
.802 .765 .819 r
.2318 .39164 .26328 .42671 .31313 .38076 .28234 .34617 Metetra
.847 .773 .779 r
.28234 .34617 .31313 .38076 .36379 .31877 .33385 .28518 Metetra
.866 .781 .766 r
.33385 .28518 .36379 .31877 .41418 .25047 .3852 .21805 Metetra
.865 .791 .776 r
.3852 .21805 .41418 .25047 .46378 .1861 .43576 .15467 Metetra
.84 .802 .814 r
.43576 .15467 .46378 .1861 .51269 .13429 .48558 .10337 Metetra
.767 .8 .88 r
.48558 .10337 .51269 .13429 .56161 .10083 .53527 .06978 Metetra
.604 .75 .947 r
.53527 .06978 .56161 .10083 .61153 .08825 .58581 .05635 Metetra
.345 .614 .95 r
.58581 .05635 .61153 .08825 .66355 .09568 .63827 .06222 Metetra
.119 .458 .887 r
.63827 .06222 .66355 .09568 .71854 .11887 .69357 .08327 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.14416 .21324 m
.14879 .21789 L
s
[(-2)] .13491 .20394 .99496 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.55805 .05117 m
.56192 .05647 L
s
[(2)] .55031 .04057 .7295 1 Mshowa
.125 Mabswid
.19248 .19432 m
.19521 .19716 L
s
.24172 .17504 m
.2444 .17792 L
s
.2919 .15539 m
.29453 .15832 L
s
.39523 .11493 m
.39774 .11796 L
s
.44843 .0941 m
.45088 .09718 L
s
.50269 .07285 m
.50508 .07598 L
s
.09674 .23181 m
.09956 .23455 L
s
.05019 .25004 m
.05305 .25274 L
s
.61454 .02905 m
.61679 .03228 L
s
.67219 .00648 m
.67437 .00976 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.5, 147.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.0563277,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002e0000Ta000`40O003h00O]Goo002eOol00;Eo
o`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eoo`00]Goo001XOol4000FOol2000aOol006Uoo`03
001oogoo01=oo`<00003Ool007oo031oo`00JWoo00<007ooOol047oo0P0000Aoo`00Oomoo`80031o
o`00Jgoo00<007ooOol037oo0`001Woo00<007oo0000;goo001XOol01000Oomoo`0037oo0P002Goo
00<007oo0000;goo001YOol2000:Ool3000;Ool01000Oomoo`00;Woo001cOol2000>Ool01@00Oomo
ogoo0000;Goo001`Ool3000@Ool01@00Oomoogoo0000;Goo001]Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`0ZOol006]oo`800003Ool007oo00L000aoo`04001oogooOol3000[Ool006Qoo`<00003
Ool000000080009>oP03000^ORin008^OPH000Ioo`03001oogoo00=oo`03001oogoo00Uoo`@001ao
o`00IWoo0P0000Aoo`00000000M>oP03000^ORin00D^OP03000=g000008000Aoo`03001oogoo00=o
o`03001oogoo00Uoo`03001oogoo01aoo`00Hgoo0`000Woo00<006<l00002Dkn00<002in;Wh01Bin
00@000gL3M`=g0<00003Ool007oo00Aoo`03001oogoo009oo`D000=oo`03001oogoo01]oo`00Agoo
0P0067oo0P0017oo00D006<lHcaS?00000U>oP03000^ORin00D^OP03000=g0gL00<=g08000Ioo`03
001oogoo00Yoo`03001oogoo01Yoo`00AWoo00@007ooOol001Aoo`<000Aoo`8000ES?003001>oTkn
00E>oPP0008^OP03000=g0gL008=g08000Moo`03001oogoo00Ioo`04001oogoo000LOol004Ioo`04
001oogoo000BOol20006Ool00`00HcaS?006Hc`00`00C_i>oP040002CWX00`00=Q/f6`03=Q/50003
3M`00`00Oomoo`04Ool20009Ool2000MOol004Ioo`04001oogoo000?Ool300000goo001oo`04Ool0
0`00HcaS?007Hc`30007CWX00`00=Q/f6`05=Q/01@007K/000003M`00P001goo00<007ooOol09Goo
0016Ool01000Oomoo`003Goo0P002Goo00<006<lHc`01f<l00<005ji000024ij00<003HK=Q/01SHK
00<001fk7K/00`0027oo00<007ooOol097oo0017Ool2000;Ool3000:Ool00`00JcX00007Hc`01@00
G[UN^Eji000024ij00<003HK=Q/01CHK00@001fk7K/M^`<000Moo`03001oogoo02Aoo`00DWoo0`00
2goo00D006/jJcY[>P0000ES?003001N^Eji009N^@03001>NTij00I>NP03000f6cHK00Hf6`04000M
^afk00027K/00`00Oomoo`05Ool00`00Oomoo`0SOol004moo`<000eoo`03001[>V/j00=[>P05001S
?6<lHc`00006G[T00`00CWY>NP06CWX00`00=Q/f6`05=Q/010007K/M^`000Qfk00<007ooOol01Woo
00<007ooOol08Woo001<Ool3000?Ool00`00JcY[>P05JcX00`00Hc`00007G[T00`00CWY>NP06CWX0
0`00=Q/f6`06=Q/00`007K/000037K/00`00Oomoo`05Ool00`00Oomoo`0ROol004Yoo`<0011oo`03
001[>V/j00M[>P03001N^Eji00MN^@03001>NTij00A>NPP0008f6`03000M^`0000<M^`03001oogoo
00Ioo`03001oogoo025oo`00Agoo0`000Woo00<007ooOol03Goo00<006/jJcX01f/j0P002Eji00<0
04ijCWX010000Thh00<003gi?OT00cgi1@0011fk00<007ooOol01Woo00<007ooOol087oo0015Ool2
000DOol00`00JcY[>P07JcX01000I_QVn00025ji0`001dhh00<003gi?OT01Cgi1@000Qfk00<007oo
Ool01Goo00<007ooOol087oo0012Ool3000EOol00`00JcY[>P07JcX01@00I_QVn6Kh00001eji00<0
05Yh000024hh00<003gi?OT01Sgi00@0037J<MXafP<000Qoo`03001oogoo01moo`009Goo10005goo
0P0067oo0P001f/j00<006KhI_P00fKh00<005jiG[T00Uji0P000eYh00<004hhCSP01Dhh00<003gi
?OT01Sgi00<0037J<MX0137J00<007ooOol01Woo00<007ooOol07Woo000VOol00`00Oomoo`0DOol3
000IOol01000KcQ_>0001F/j00<006KhI_P01FKh00@005jiG[T000EJN003001>>4hh00I>>003000m
nCgi00Dmn@8000DafP03001oogoo00Eoo`80021oo`007goo1@000goo00<007ooOol04Goo0P006goo
00<006lhKcP00Vlh00D006/jJcY[>P0000QVn003001N^@0000MJN003001>>4hh00E>>003000mnCgi
00Dmn@8000HafP03001oogoo00Ioo`03001oogoo01eoo`00:7oo00<007ooOol03Goo0`0077oo00<0
06lhKcP016lh00<006/j00002VKh00<005YhFWP01UYh00<004hhCSP01Dhh00<003gi?OT01Cgi0P00
1S7J00<007ooOol01Woo00<007ooOol07Goo000UOol01000Oomoo`003Goo0P007Woo00<006lhKcP0
1Vlh00<006KhI_P026Kh0P002EYh00<004hhCSP01Dhh00<003gi?OT01Cgi0P001S7J00<007ooOol0
1Woo00<007ooOol077oo000VOol2000;Ool3000POol00`00KcQ_>006KcP00`00I_QVn007I_P01000
H[QR^00025Yh00<004hhCSP01Dhh00<003gi?OT01Cgi0P001c7J00<007ooOol01Woo00<007ooOol0
6goo000aOol200000goo001oo`0OOol00`00KcQ_>006KcP00`00KaP00007I_P01@00H[QR^6:h0000
25Yh00<004hhCSP01Dhh00<003gi?OT00cgi2@000S7J00<007ooOol01Woo00<007ooOol06goo000^
Ool3000SOol00`00KcQ_>006KcP01000KaQ_60001VKh00<006:hH[P00V:h00<005YhFWP01eYh00<0
04hhCSP00dhh1`000TGg00<004Gg0000148I10001goo0P0077oo000[Ool3000UOol00`00KcQ_>006
KcP00`00KaQ_6002KaP00`00I_QVn002I_P00`00H[QR^004H[P00`00FWQJN006FWP00`00CSQ>>003
0002CQL00`00AOM5m`04AOL01000AOM5m`001T8I00<007ooOol01goo00<007ooOol00goo0P0057oo
000YOol20002Ool00`00Oomoo`0ROol00`00KcQ_>006KcP00`00KaQ_6004KaP01000I_QVn0001f:h
00<005YhFWP01UYh0`001ThG00<004GgAOL00dGg00@004GgAOL000M26@03001oogoo00Ioo`03001o
ogoo009oo`04001oogoo000COol002Ioo`<002Uoo`8000M_>003001_66lH00A_6003001Vn00000UR
^003001JN5Yh00=JN0800003DUL004hG00I>5`030015mdGg00=5m`040015mdGg0007@QT00`00Oomo
o`07Ool01@00Oomoogoo00000Woo00<007ooOol04Goo000TOol2000[Ool01000KaQ_60001Flh00<0
06lHKaP01VlH00<006:hH[P026:h00@005YhFWQJN08000=BE`03001>5dhG00E>5`030015mdGg00=5
m`050015mdGgAOL00007@QT00`00Oomoo`07Ool01000Oomoo`000Woo00<007ooOol04Goo000QOol3
000/Ool00`00KaQ_6002KaP01@00KcQ_>6lh000026lH0P002V:h00@005Yh000000IBE`03001>5dhG
00E>5`030015mdGg0095m`050015mdGgAOL00007@QT00`00Oomoo`06Ool20002Ool01000Oomoo`00
4goo000OOol2000^Ool00`00KaQ_6004KaP00`00KcP00008KaP01000JaQ[60002F:h0P00259G00<0
04hGCQL01DhG00@004GgAOM5m`8000=5m`0300126D8I00I26@03001oogoo00Moo`03001oogoo0080
01Aoo`0077oo0`00<7oo00<006lHKaP01FlH00<006lHKaP01VlH00D006/HJaQ[600000QR^003001J
V00000QBE`03001>5dhG00I>5`<00003AOL004Gg00=5m`0300126D8I00E26@03001oogoo00Qoo`03
001oogoo01Eoo`006goo0P00<Goo00<006lHKaP01VlH00<006lHKaP01FlH00<006/HJaP00f/H00<0
06:hH[P016:h00D005ZHFYQJV00000QBE`03001>5dhG009>5`@000A5m`050015mdGgAOL00006@QT3
0009Ool00`00Oomoo`0EOol001Aoo`<000Aoo`03001oo`0002moo`03001_66lH00I_6003001cF000
00E_6003001[66/H00A[6003001R^6:h009R^08000AJV003001BEe9G00IBE`03001>5`00008000A>
5`030015mdGg0095m`040015mdGg0002@QT40002CSP00`00Oomoo`08Ool00`00Oomoo`0DOol001Eo
o`03001oogoo00=oo`04001oogoo000^Ool00`00KaQ_6005KaP01@00LeQcF7=H00000flH00<006/H
JaP01V/H00@006:hH[P000IJV003001BEe9G00EBE`<000M>5`030015mdGg00=5m`D000M>>003001o
ogoo00Moo`8001Eoo`003Woo1@000Woo00<007ooOol00goo00D007ooOomoo`0002aoo`03001_66lH
00E_6003001cF7=H00=cF003001_600000U[6003001R^00000QJV004001BEe9GDUL30002DUL00`00
CQM>5`05CQL00`00AOM5m`040002CQL00`00CSQ>>006CSP00`00Oomoo`08Ool00`00Oomoo`0COol0
01Eoo`03001oogoo009oo`8000Aoo`03001oogoo02Qoo`03001_66lH00I_6003001cF7=H00AcF003
001[66/H00Q[608000UJV0@000EBE`03001>5dhG00E>5`<000A>5`04001>5dhG0008CSP00`00Oomo
o`08Ool00`00Oomoo`0BOol001Eoo`03001oogoo009oo`8000Eoo`03001oogoo02Ioo`03001_66lH
00I_6003001cF7=H00EcF003001[66/H00M[6003001VnP0000MJV0<000QBE`03001>5dhG009>5`<0
0003CQL004hG00E>5`03001>5`0000U>>003001oogoo00Moo`03001oogoo019oo`0057oo0P0017oo
00<007ooOol01Goo00<007ooOol09Goo0P001VlH00<007=HLeP01G=H00<007=i00001f/H00D006Kj
I_YVnP0000AJV08000=JV003001BEe9G00IBE`@000A>5`03001>5dhG00A>5`03001>5`0000U>>003
001oogoo00Qoo`03001oogoo015oo`006Woo00<007ooOol01Woo00<007ooOol08goo00@006lHKaP0
00E_6003001cF7=H00AcF005001cNG=iLgT00005JaP00`00I_YVnP02I_X00`00FYP000020005FYP0
0`00DUMBE`05DUL20008CQL00`00CQM>5`04CQL00`00CQL00008CSP2000:Ool00`00Oomoo`0@Ool0
01Yoo`03001oogoo00Moo`03001oogoo025oo`03001_66lH009_6005001_66lHKaP00006LeP00`00
LgUcN@03LgT01@00JaQ[66/H000016Kj0`0025ZH00<0059GDUL00U9G0`0000=B=`00CQL01dhG00<0
04hGCQL01DhG0P001dhh00<005Yh00002Goo0P004Woo000IOol2000:Ool00`00Oomoo`0OOol00`00
KaQ_6004KaP00`00KaP00006LeP00`00LgUcN@05LgT00`00JaP00002I_X30002I_X00`00FYQJV007
FYP40004DSL00`00CQM>5`06CQL00`00CQM>5`06CQL20004CSP20003FWP00`00Oomoo`08Ool00`00
Oomoo`0?Ool000Qoo`8000Aoo`8000=oo`<000=oo`8000]oo`03001oogoo01eoo`03001_66lH00I_
6003001cF7=H00EcF003001cNG=i00EcN@D000EVnP03001JV5ZH00IJV08000QB=`03001>5dhG00I>
5`03001>5dhG00E>5`80009>>08000EJN003001oogoo00Uoo`03001oogoo00ioo`001goo00@007oo
Ool000=oo`8000Ioo`04001oogoo000=Ool00`00Oomoo`0KOol00`00KaQ_6007KaP20005LeP00`00
LgUcN@05LgT30009I_X00`00FYQJV002FYP300000e9i001B=`07DSL00`00CQM>5`06CQL00`00CQM>
5`06CQL00`00CSP00008FWP00`00Oomoo`08Ool00`00Oomoo`0>Ool00003Ool0000000<00005Ool0
07ooOol0000;Ool01000Oomoo`003Woo00<007ooOol06Goo00<006lHKaP01flH00@007=iLgT000=c
F003001cNG=i00=cN@<000=[W003001VnVKj00IVnP03001JV000008000EBN@03001B=e8g00EB=`03
001>5dhG00I>5`03001>5dhG00I>5`8000UJN003001oogoo00Uoo`03001oogoo00eoo`001goo00@0
07ooOol000Qoo`<000=oo`8000ioo`8001Qoo`03001_66lH00M_6003001cNG=i009cN@03001cF000
00=cN@<00003LgT006^L00A[W003001VnVKj00EVnP8000QBN@03001B=e8g00IB=`03001>5dhG00I>
5`03001>5dhG00A>5`03001FE`0000UJN003001oogoo00Yoo`03001oogoo009oo`@000Ioo`001goo
00@007ooOol000Qoo`03001oogoo00=oo`03001oo`0000moo`03001oogoo01=oo`8000U_6003001c
NG=i00AcN@800003LgT0000000=cN@03001[W6^L00A[W003001VnVKj009VnP<00003E]/0059i00MB
N@03001B=e8g00IB=`03001>5dhG00I>5`03001>5dhG009>5`8000=FE`03001JN5Yh00MJN003001o
ogoo00Uoo`03001oogoo00=oo`03001oogoo00Ioo`0027oo0P002Goo10000Woo00@007ooOol000mo
o`03001oogoo015oo`03001W6P0000Q_6003001cNG=i00EcN@@000=cN@03001[W6^L00I[W004001V
nP000005E]/00`00DWUBN@06DWT00`00DSMB=`05DSL00`00CQM>5`06CQL00`00CQM>5`020005EUL2
0008FWP00`00Oomoo`0:Ool00`00Oomoo`03Ool00`00Oomoo`05Ool001Qoo`8000=oo`03001oogoo
00eoo`03001oogoo00moo`05001W6VLJIaX00007KaP00`00LgUcN@02LgT30004LmX00`00LgT00009
Ji`20007E]/00`00DWUBN@06DWT00`00DSMB=`06DSL00`00CQM>5`06CQL20007EUL00`00FWP00006
FWP00`00I]P0000<Ool00`00Oomoo`03Ool00`00Oomoo`04Ool001Qoo`8000Aoo`8000ioo`03001o
ogoo00eoo`03001W6VLJ00=W6P03001_66lH00=_6003001cNG=i00<00003LgT007?J00AcfP<000I[
W0<00003Eil005KK00MFf`03001BNE9i00EBN@03001B=e8g00IB=`03001>5dhG00E>5`8000UFE`80
00EJN005001Vf6KHI]P0000:Ool20002Ool01000Oomoo`001Woo000BOol20004Ool00`00Oomoo`04
Ool2000>Ool00`00Oomoo`0;Ool00`00IaYW6P05IaX01000KaQ_66lH10000g=i00<007?JLmX00g?J
00@006?kHo]Sn`800004Ji`000000005Eil00`00E]]Ff`05E]/00`00DWUBN@06DWT00`00DSMB=`05
DSL00`00CQM>5`02CQL300000e8g001FE`08EUL00`00FWP00003FWP00`00I]QVf002I]P00`00Oomo
o`0:Ool00`00Oomoo`020007Ool0015oo`04001oogoo0003Ool00`00Oomoo`05Ool2000>Ool00`00
Oomoo`09Ool00`00IaYW6P07IaX40005LgT00`00LmYcfP03LmX00`00Ho]Sn`03Ho/30008Eil00`00
E]]Ff`04E]/00`00DWUBN@06DWT00`00DSMB=`06DSL40005DSL00`00EUMFE`07EUL01@00FWP005Yh
00001VKH00<007ooOol02Woo00<007ooOol027oo000AOol01000Oomoo`000goo00<007ooOol01Woo
0`003Goo00<007ooOol01goo00<006LJIaX01VLJ1@001G=i00<007?JLmX00W?J0P000f?k0`000cMi
0`001ENO00<005KKE]/01EKK00<0059iDWT01E9i00<0058gDSL00U8g1@00258g00<005IGEUL01eIG
00@005YhFWP000QVf003001oogoo00Uoo`03001oogoo00Qoo`004Goo00@007ooOol0009oo`8000Uo
o`<000eoo`03001oogoo00Aoo`8000MW6P800003IaX006nK009_V`<00003LgT007?J00=cfP04001S
nf?kHo/@00000eNO001Ff`05E]/00`00DWUBN@02DWT80004CSP00`00DSMB=`06DSL00`00EUMFE`07
EUL01000FWP0000026KH00<007ooOol02Woo00<007ooOol01goo000AOol01000Oomoo`000goo00<0
07ooOol027oo100037oo00@007ooOomoo`800003F_`006LJ00=W6P<0009W6P03001_VfnK00A_V`@0
009cfP03001Sn`0000D000=;7P03000NGP0000D0008Vg`d000A6FP03001>>4hh00I>>003001B=e8g
00IB=`03001FEeIG00IFE`80009Vf003001Vf6KH00AVf003001_>@0000]oo`03001oogoo00Moo`00
4Woo0P000goo0P002Woo00<0004H00000`002goo0`0015[l100016LJ00<006nKKi/016nK00<006?k
Ho/00V?k1`001d/N00@001iN7UhNGP@000<Vg`03000nWCjM00@nW@030016FTIJ00I6FP03001>>4hh
00E>>003001B=e8g00IB=`03001FEeIG00EFE`03001NV00000=Vf003001Vf6KH009Vf005001_>Fli
KcT0000;Ool00`00Oomoo`06Ool001Moo`03001oogoo00Yoo`05000160000001V`020008Ool20004
F_`40005IaX00`00Ki]_V`04Ki/00`00Ho]Sn`02Ho/300000eoN001S>`02Hc/00`00Bai;7P05Bah0
1P007UhNGQiN0001F@<0008Vg`03000nWCjM00@nW@030016FTIJ00E6FP03001>>4hh00E>>003001B
=e8g00IB=`03001FEeIG00AFE`05001NV5jHGYP00002I]P01@00I]QVf6KH000016li00<007ooOol0
2Woo00<007ooOol01Goo000GOol00`00Oomoo`0;Ool010000AP160000P6K20000e[l0`0015[l00D0
06LJIaYW6P0000I_V`04001Snf?kHo/300000eoN00000005Hc/00`00Bai;7P02Bah600000aiN0001
F@060005?Yd00`00AUY6FP05AUX00`00CSQ>>006CSP00`00DSMB=`06DSL01000EUMFEeIG0P0015jH
00L006KHI]QVf000I]P00006KcT00`00Oomoo`09Ool00`00Oomoo`05Ool001Moo`03001oogoo00ao
o`060001604H0001V`001SZN100025[l00<006LJ00001VnK00@006?kHo]Sn`80009OgP03001S>f<k
00ES>`D000=>NP03000f6cHK00<00081F@030001VP6J008000DnW@030016FTIJ00E6FP03001>>4hh
00E>>003001B=e8g00IB=`04001FEeIG0006GYP00`00I]QVf002I]P20006KcT00`00Oomoo`0:Ool0
0`00Oomoo`04Ool000Eoo`8000Aoo`8000=oo`<000Aoo`8000ioo`030001604H00<000<jWP@000YJ
o0<000E_V`05001Snf?kHo/00002Jg/30006Hc/00`00G[T00007CWX00`00=Q/f6`02=Q/200000`5I
000000020IX20003?Yd00`00AUY6FP05AUX00`00CSQ>>005CSP00`00DSMB=`06DSL00`00EUL00008
GYP01000I]QVf6KH0P001Fli00<007Ni00002Woo0`001Goo0004Ool01000Oomoo`000goo0P001Woo
00D007ooOomoo`00011oo`<000030AP00000008000@jWP03001Jo5[l00EJo08000=K_`<00003Ki/0
06?k00<000=[N`04001[6F/I0003Hc/20003G[T00`00CWY>NP04CWX00`00=Q/f6`02=Q/00`004IX0
000300000`6J000Eo00300000cjM0016FP05AUX00`00CSQ>>006CSP00`00DSMB=`05DSL20009GYP0
1P00I]QVf000KcT000A_>@05001g^GNiMkT00006Ool30002Ool00`00Oomoo`03Ool000Aoo`04001o
ogoo000;Ool00`00Oomoo`02000@Ool00`00;Q`^70040006>Yh00`00F_aJo002F_`20007Fkl50004
Jg/00`00JaU[6@02JaT00`00Hc/00006G[T00`00CWY>NP04CWX00`00=Q/f6`02=Q/00`004IXAVP05
00035O`20007AUX00`00CSQ>>005CSP00`00DSMB=`02DSL300000eIi001NV008GYP00`00I]P00003
KcT01@00KcU_>Fli000017Ni0P0000Aoo`00000000Eoo`03001oogoo00=oo`0017oo00@007ooOol0
00Qoo`<000=oo`80011oo`<000<^708000HjWP04001Jo0000006Fkl60005Jg/00`00JaU[6@04JaT0
0`00G[UN^@05G[T00`00CWY>NP04CWX00`00=Q/f6`03=Q/00`004IXAVP0200000`3g000000045O`3
0004AUX00`00CSQ>>005CSP00`00DSL000020005EWT00`00GYQNV006GYP20005KcT00`00KcT00005
MkT00`00KoT00008Ool00`00Oomoo`03Ool000Aoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00
Oomoo`0<Ool300000goo000^7005;Q`;0003Fkl500000eoN001[N`04Jg/00`00JaU[6@04JaT20008
G[T01000CWY>NTij200000<f6`004IX00Q6J0P0000<0m`005O`00QGl00<003Hk=S/01@0014hh1@00
25Ii00<005jHGYP01EjH00<006LJ00001Vli00<007NiMkT00gNi00<006oiKoT00P001goo00<007oo
Ool00goo0005Ool20009Ool40002Ool00`00Oomoo`09Ool30005Ool00`00;Q`^7005;Q`300047]h7
0002Hc`30004Jg/00`00JaU[6@04JaT01000I]QVf0001eji10000dhh00<0047i@OT00T7i100000DA
VP000AP0003g0080008Eo003000f>cHk00<f>`D000A:FP03001FNEIi00MFN@03001NV5jH00=NV005
001W6VLJIaX00004KcT20004MkT01000KoU_n@000T^f00<007ooOol01Goo00<007ooOol00Woo000F
Ool00`00Oomoo`06Ool30009Ool00`00;Q`^7003;Q`800000dlo00000006Hc`00`00JcU[>@020000
0f]k001[6@05JaT01@00I]QVf6KH00001Eji0P0000=JN000CSP01Dhh00<0047i@OT00d7i00<002gJ
00000P0000L160000001>@005O`00006=S/00`00BUY:FP06BUX00`00EWUFN@06EWT00`00GYQNV002
GYP00`00IaYW6P03IaX01`00KcU_>Fli001g^@00009g^@04001_nFoi0003BkH30005Ool00`00Oomo
o`02Ool001Eoo`8000Eoo`<000aoo`03000^72hL00<000A>V`05001??dloCcl00007Hc`00`00JcU[
>@03JcT20005JaT00`00I]QVf003I]P01@00G[UN^Eji000015Yh00<004hhCSP00dhh00<0047i@OT0
147i00<002gJ;MX00P000P4H00<0004i00000P001SHk00<004YJBUX01DYJ00<005IiEWT01eIi00@0
05jHGYP000MW6P03001_>@0000=g^@800004KoT00000BkH70004Ool00`00Oomoo`02Ool001Eoo`04
001oogooOol3000@Ool20008CY/00`00Ccl00007Hc`00`00JcU[>@04JcT00`00KaP00003JaT00`00
I]QVf004I]P01000G[T000001EYh00<004hhCSP014hh00<0047i@OT00d7i00@002gJ;MX]fP800081
60030001>@4i008000Df>`03001:FTYJ00I:FP03001FNEIi00IFN@03001NV00000QW6P<000=g^@04
001_n@00BkH:0003Ool00`00Oomoo`02Ool001Eoo`@001=oo`03001>VdjK00M>V`<000IS?003001[
>F/i00A[>@07001_66lHKaP006/I000026KH00<005YhFWP01EYh00<004hhCSP014hh00<0047i@OT0
147i00<002gJ;MX00RgJ0P0000D160000001>@0000Hf>`03001:FTYJ00E:FP03001FNEIi00IFN@80
00QW6P08001_g6oLKm`007Ni001_n@<000Eoo`H0009oo`03001oogoo009oo`003Woo0`0017oo0P00
5Woo00<004jKCY/01DjK00@005kIG]UNf@8000=S?003001[>F/i00A[>@03001_66lH00=_6003001V
f6KH00MVf003001JN5Yh00IJN003001>>4hh00=>>0030011nD7i00A1n@03000]fRgJ008]fP03000A
V`0000<000030CT0000000@f>`03001:FTYJ00E:FP03001FNEIi00AFN@800003F_`006LJ00IW6P03
001_g6oL009_g0H000Uoo`8000040MP007oo0004Ool000moo`03001oogoo00Eoo`03001oogoo01Ao
o`03001>VdjK009>V`8000INf@03001S?00000I[>@03001_66lH00A_6003001Vf6KH00IVf003001R
^00000MJN003001>>4hh00A>>0030011nD7i00A1n@03000]fRgJ008]fP03000AVa6K00P0008f>`03
001:FTYJ00E:FP04001FNEIiEWT20004F_`00`00IaYW6P03IaX00`00Kma_g002Km`00`00BkD00004
Ool00`00Oomoo`08Ool40004Ool000moo`03001oogoo00Ioo`03001oogoo01=oo`05001>VdjKCY/0
0009G]T00`00JcU[>@04JcT00`00KaQ_6004KaP00`00KaP00006I]P01000H[QR^0001eYh00<004hh
CSP014hh00<0047i@OT0147i00<002gJ;MX00bgJ00<0016K4I/00P0000<0m`0000000QGl0`001dYJ
00@005Ii000000IJo003001W6VLJ009W6P03001_g6oL009_g0<000Eoo`03001oogoo00Yoo`<000=o
o`003goo00<007ooOol01goo00<007ooOol04goo00<004jK00002UkI0P001F/i00<006lHKaP00flH
00D006lHKaQ_600000AVf003001R^6:h009R^003001JN5Yh00EJN004001>>4hhCSP>00000bgJ000A
V`034I/200000`3g000Eo0035O`90009F_`01000IaYW6P000foL0`0027oo00<007ooOol02goo00<0
07ooOol00Goo000?Ool00`00Oomoo`08Ool2000DOol00`00G]UNf@08G]T01000I_QVn0000f/i00<0
06lHKaP00flH00<006lHKaP00VlH00D006KHI]QVf00000ER^003001JN5Yh00AJN0D000=>5`03001:
5dXG00A:5`0400126D8I@QT500024I/500025O`00`00?[dn_@05?[d00`00F_aJo007F_`00`00IaX0
0003Km`00`00BkD0000:Ool00`00Oomoo`0:Ool00`00Oomoo`01Ool000ioo`8000aoo`03001oogoo
015oo`03001NfEkI00MNf@03001Vn6Kh009Vn003001[>@0000E_6003001_66lH00A_6003001Vf000
00IR^003001JN5Yh00=JN003001FE`0000M>5`03001:5dXG00A:5`0300126D8I00926@03000f>cHk
00@000030GT00000008000035O`003jm00Hn_@03001Jo5[l00IJo0@00004Km`00000000;Ool00`00
Oomoo`0:Ool00`00Oomoo`01Ool001eoo`03001oogoo015oo`03001NfEkI00ENf@03001Vn6Kh00AV
n08000E_6003001_66lH00E_6003001R^6:h00IR^003001JN5Yh008000=FE`03001>5dhG00A>5`03
001:5dXG00A:5`0300126D8I00=26@03000f>cHk008f>`D000040IT000000007?[d00`00F_aJo002
F_`30003Dkl3000>Ool00`00Oomoo`09Ool00`00Oomoo`01Ool001ioo`03001oogoo015oo`03001N
fEkI00=Nf@03001Vn6Kh00EVn003001_600000=_6003001_66lH00E_608000QR^003001JN00000EF
E`03001>5dhG00A>5`03001:5dXG00A:5`0300126D8I00A26@03000f>cHk00@f>`L000Hn_@03001J
o000008000=C_`<0019oo`03001oogoo00Qoo`03001oogoo005oo`007goo0P004Woo00D005kIG]UN
f@0000MVn007001_66lHKaP006lH00001flH00@006[hJ_P000MR^08000IFE`03001>5dhG00E>5`03
001:5dXG00A:5`0300126D8I00=26@03000f>cHk00Df>`03000JOQYn00d0009C_`8001Ioo`03001o
ogoo00Moo`03001oogoo005oo`008Goo00<007ooOol047oo00<005kI00001fKh00<006lHKaP00flH
00<006lHKaP01FlH00<006[hJ_P00V[h00<006:hH[P00f:h00<005ZH00001eIG00<004hGCQL014hG
00<004XGBQL014XG00<0048I@QT0148I00<003Hk=S/00SHk1@0014/o2`0067oo00<007ooOol027oo
0@000Goo0Goo000ROol00`00Oomoo`0@Ool00`00I_QVn005I_P00`00KaQ_6004KaP20005KaP00`00
J_QZn003J_P01000H[QR^6:h0P000eZH00<005IGEUL015IG00<004hGCQL01DhG00<004XGBQL00dXG
00<0048I@QT00d8I1P0014jJ00D004/oBcm;?`0000ASG003001_FVmJ00=_FP03001oogoo01Moo`03
001oogoo00Moo`40005oo`5oo`008goo00<007ooOol03goo00<006KhI_P016Kh00<006lHKaP016lH
00@006lhKcP000=_6003001Zn6[h00EZn003001R^00000EJV003001FEeIG00AFE`03001>5dhG00E>
5`03001:5dXG00A:5`0300126@000080009>>003001>VTjJ00I>VP03001;?`0000ASG003001_FVmJ
00=_FP03001oogoo01Uoo`03001oogoo00Ioo`40005oo`5oo`0097oo00<007ooOol03goo00<006Kh
I_P00fKh00<006lHKaP00flH00<006lhKcP00Vlh00<006lH000026[h0P001eZH00<005IGEUL015IG
00<004hGCQL014hG00@004XGBQM:5`D000I>>003001>VTjJ00E>VP<000=SG003001_FVmJ00=_FP03
001oogoo01]oo`03001oogoo00Eoo`40005oo`5oo`009Goo0P0047oo00D006KhI_QVn00000I_6003
001_>6lh00=_>003001Zn6[h00IZn003001Rn@0000MJV003001FEeIG00AFE`03001>5dhG00=>5`D0
00=>=`03001>>4hh00E>>003001>VTjJ00=>VP8000=Nf@03001SG00000E_FP03001oogoo01aoo`03
001oogoo00Eoo`40005oo`5oo`009goo00<007ooOol03Woo00<006Kh00001VlH00<006lhKcP016lh
00<006[hJ_P01F[h00D006;iH_URn@0000IJV003001FEeIG00AFE`03001>5dhG00<0009>5`03001>
=dhg00A>=`03001>>4hh00E>>005001>VTjJCYX00006G]T00`00KeY_FP03KeX00`00Oomoo`0NOol0
0`00Oomoo`04Ool10001Ool1Ool002Qoo`03001oogoo00ioo`03001_66lH00A_6003001_>6lh00A_
>003001_F@0000EZn003001RnF;i009Rn@03001JV5ZH00EJV003001FEeIG00=FE`<000E>5`03001>
=dhg00A>=`03001>>4hh00E>>004001>VP000007G]T20003KeX00`00Oomoo`0POol00`00Oomoo`03
Ool10001Ool1Ool002Uoo`03001oogoo00ioo`03001_66lH009_6003001_>6lh00A_>005001_FFmI
KeT00003J_P00`00H_URn@03H_T00`00FYQJV005FYP00`00EUMFE`0300000e8g001>5`06CQL00`00
CSM>=`04CSL00`00CSQ>>004CSP20008G]T01P00J_QZn000KeX002=oo`03001oogoo00Aoo`40005o
o`00:Woo0P003goo00D006lHKaQ_600000I_>003001_FFmI009_F@03001Zn00000MRn@03001JV5ZH
00AJV0<000EB=`03001>5dhG00A>5`03001>=dhg00A>=`03001>>4hh00=>>003001JN00000MNf@03
001Zn6[h009Zn003001oogoo02=oo`03001oogoo00=oo`40005oo`00;7oo00<007ooOol03Goo00<0
06lH00001Vlh00<006mIKeT00fmI0P0026;i00@005ZHFYQJV0<00003EWP0058g00EB=`03001>5dhG
00A>5`03001>=dhg00E>=`03001>>4hh008000=JN003001NfEkI00=Nf@03001Zn6[h00=Zn003001o
ogoo02Aoo`03001oogoo009oo`40005oo`00;Goo00<007ooOol03Goo0P001Flh00<006mIKeT00fmI
00@006^KJi/000MRn@04001JV0000004EWP00`00DSMB=`04DSL00`00CQM>5`05CQL00`00CSM>=`04
CSL00`00CSP00006FWP00`00G]UNf@02G]T00`00J_QZn002J_P00`00Oomoo`0VOol01@00Oomoogoo
00000Goo000^Ool00`00Oomoo`0>Ool20003KcP00`00KeU_F@02KeT00`00Ji][V`02Ji/00`00H_UR
n@04H_T20007EWP00`00DSMB=`04DSL00`00CQM>5`04CQL00`00CSM>=`04CSL20007FWP01@00G]UN
fEkI000016[h00<007ooOol09goo00D007ooOomoo`00005oo`00;goo0P0047oo00<006lh000016mI
00<006^KJi/016^K00<006;iH_T00`0000=Ff`00EWP01UIh00<0058gDSL0158g00<004hGCQL014hG
00<004hgCSL00Thg0P0000=FE`00FWP01eYh00<005kI00001F[h00<007ooOol0:7oo00@007ooOol0
005oo`00<Goo00<007ooOol03Woo0`000VmI00<006^KJi/01F^K0`001EKK00<005IhEWP015Ih00<0
058gDSL0158g00<004hGCQL014hG00D004hgCSM>=`0000AFE`03001JN5Yh00EJN08000EZn003001o
ogoo02Yoo`03001oo`00005oo`00<Woo00<007ooOol047oo0`001F^K0P000UNO00<005KKE]/00eKK
00<005IhEWP01EIh00<0058gDSL0158g00<004hGCQL014hG0`001EIG00<005YhFWP015Yh00<006:h
000016[h00<007ooOol0:goo00<007ooOol00@00000cOol00`00Oomoo`0BOol200000f^K00000005
Eil00`00E]]Ff`03E]/00`00EWQFN004EWP00`00DSMB=`04DSL00`00CQM>5`03CQL20007EUL00`00
FWQJN003FWP01@00H[QR^6:h00000f[h00<007ooOol0;7oo0@000Goo0@00000dOol2000DOol30006
Eil00`00E]]Ff`03E]/00`00EWQFN003EWP00`00DSMB=`04DSL00`00CQM>5`0300000e8g001FE`07
EUL01@00FWQJN5Yh00001F:h00<006[h0000<7oo0P00000fOol00`00Oomoo`0DOol;00000eKK001F
N005EWP01000DSMB=e8g1@001E8g00<005IGEUL01EIG00@005YhFWP000MR^003001oogoo02ioo`<0
0000=goo00<007ooOol05goo0P0000<>O000>Yd00SZM30000dhh00<0058gDSL0158g00<005IGEUL0
1EIG00<005Yh000026:h00<007ooOol0:Woo10000goo000hOol00`00Oomoo`0HOol00`003W`00004
>Yd00`00BUY:FP03BUX00`00CSQ>>004CSP00`00DSMB=`05DSL00`00EUMFE`04EUL20008H[P00`00
Oomoo`0WOol40007Ool003Uoo`03001oogoo01Qoo`<000@jW@03001:FTYJ00=:FP03001>>4hh00A>
>003001B=e8g00AB=`03001FEeIG00=FE`03001NV00000MR^003001oogoo02Eoo`<000]oo`00>Woo
0P006Goo0P0013ZM00<004YJBUX00dYJ00<004hhCSP014hh00<0058gDSL0158g00<005IGEUL00UIG
00D005jHGYQNV00000ER^003001oogoo029oo`@000ioo`00?7oo00<007ooOol05Woo00@007oo0000
00<jW@03001:FTYJ00=:FP03001>>4hh00=>>003001B=e8g00EB=`04001FEeIG0005GYP00`00H[QR
^002H[P00`00Oomoo`0NOol4000BOol003eoo`03001oogoo01Eoo`04001oogooOol200000cZM001:
FP04BUX00`00CSQ>>004CSP00`00DSMB=`04DSL00`00EUL00006GYP01@00H[QR^6:h00007Goo1000
5Woo000nOol00`00Oomoo`0DOol00`00Oomoo`02Ool30005BUX00`00CSQ>>004CSP00`00DSMB=`04
DSL20008GYP00`00H[P0000JOol4000JOol003moo`8001Aoo`03001oogoo00=oo`D000=:FP03001>
>4hh00=>>004001B=e8gDSL300000eIi001NV007GYP2000GOol4000NOol0045oo`03001oogoo011o
o`03001oogoo00Ioo`03000b?38l00@000I>>0<000EFN@03001NV5jH00ANV003001oogoo01Aoo`<0
029oo`00@Woo00<007ooOol03goo00<007ooOol01goo00<0038l<S`00S8l20001eIi00<005jHGYP0
0ejH00<007ooOol04Goo10009Goo0013Ool00`00Oomoo`0>Ool00`00Oomoo`08Ool00`00<S`b?002
<S`00`00BUY:FP03BUX00`00EWUFN@06EWT01@00GYQNV5jH000047oo1000:Goo0014Ool2000>Ool0
0`00Oomoo`09Ool00`00<S`b?002<S`00`00BUY:FP03BUX00`00EWUFN@05EWT01000GYQNV0003Goo
1000;Goo0016Ool00`00Oomoo`0;Ool00`00Oomoo`0:Ool20002<S`00`00BUY:FP03BUX00`00EWUF
N@05EWT00`00GYP0000:Ool4000aOol004Moo`03001oogoo00Yoo`03001oogoo00aoo`800003<S`0
04YJ00A:FP03001FNEIi00AFN@8000Moo`@003Eoo`00B7oo00<007ooOol02Goo00<007ooOol03Woo
0P001TYJ00@005IiEWUFN@8000Eoo`@003Uoo`00BGoo0P002Goo00<007ooOol047oo1@0000=:FP00
EWT00P0017oo0`00?Goo001;Ool00`00Oomoo`06Ool00`00Oomoo`0EOol30002Ool40010Ool004ao
o`03001oogoo00Eoo`03001oogoo01Ioo`@004Aoo`00CGoo00<007ooOol017oo00<007ooOol04Woo
1000B7oo001>Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool4001<Ool004moo`8000=oo`03001oogoo
00Yoo`@0051oo`00DGoo00@007ooOol000Uoo`<005Aoo`00DWoo00<007oo00001Goo1000Egoo001C
Ool200000goo00000002001KOol005Aoo`8005moo`00]Goo002eOol00;Eoo`00]Goo002eOol00;Eo
o`00]Goo0000\
\>"],
  ImageRangeCache->{{{0, 180.5}, {146.188, 0}} -> {-0.128731, 0.00555917, 
  0.00599072, 0.00599072}}]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 2 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The following are particularly useful for the topics \
indicated:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The definitive reference:\nWolfram, S., 1996,",
  StyleBox[" The Mathematica Book",
    FontSlant->"Italic"],
  ", Wolfram Media-Cambridge University Press.\n\nData analysis and general \
scientific work:\nShaw, W.T. and Tigg, J., 1993, ",
  StyleBox["Applied Mathematica",
    FontSlant->"Italic"],
  ", Addison-Wesley.\n\nGraphics galore:\nWickham-Jones, T., 1994, ",
  StyleBox["Mathematica Graphics, Techniques and Applications",
    FontSlant->"Italic"],
  ", Springer-TELOS.\n\nExploring general finance and economics:\nVarian, \
H.R. (editor), 1993, ",
  StyleBox["Economic and Financial Modeling with Mathematica",
    FontSlant->"Italic"],
  ", Springer-TELOS.\n\nExploring mathematics:\nGray, T.W. and Glynn, J., \
1990, ",
  StyleBox["Exploring Mathematics with Mathematica",
    FontSlant->"Italic"],
  ", Addison-Wesley."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{596, 674},
WindowMargins->{{14, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingStartingPageNumber->12,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"2. Introduction to Mathematica"}]]}, {Cell[ 
        TextData[ {"2. Introduction to Mathematica"}]], Inherited, 
      Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003?T97OnPokT=E@UX0b85N`?P0080004/0B`000006G<AP001
0000I00000400@410?l00BL?00400@S:dL0000000000074/01T1T00000008000
00000000007oo@S:^k3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 276, 10, 136, "Title"],
Cell[2010, 63, 122, 1, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2157, 68, 159, 3, 50, "Section"],
Cell[2319, 73, 3106, 63, 440, "Text"],

Cell[CellGroupData[{
Cell[5450, 140, 168, 4, 44, "Subsection",
  PageBreakAbove->False],
Cell[5621, 146, 688, 15, 78, "Text",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[6334, 165, 103, 2, 18, "Input"],
Cell[6440, 169, 35, 1, 20, "Output"]
}, Open  ]],
Cell[6490, 173, 366, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[6881, 185, 119, 2, 18, "Input"],
Cell[7003, 189, 75, 1, 20, "Output"]
}, Open  ]],
Cell[7093, 193, 744, 15, 62, "Text"],
Cell[7840, 210, 452, 16, 24, "Text"],

Cell[CellGroupData[{
Cell[8317, 230, 110, 2, 18, "Input"],
Cell[8430, 234, 153, 2, 30, "Output"]
}, Open  ]],
Cell[8598, 239, 531, 16, 36, "Text"],

Cell[CellGroupData[{
Cell[9154, 259, 129, 2, 18, "Input"],
Cell[9286, 263, 15227, 480, 124, 4999, 349, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[24528, 746, 635, 19, 49, "Text"],

Cell[CellGroupData[{
Cell[25188, 769, 131, 5, 28, "Input"],
Cell[25322, 776, 39, 1, 20, "Output"]
}, Open  ]],
Cell[25376, 780, 405, 12, 36, "Text"],

Cell[CellGroupData[{
Cell[25806, 796, 131, 2, 18, "Input"],
Cell[25940, 800, 82, 1, 20, "Output"]
}, Open  ]],
Cell[26037, 804, 391, 14, 24, "Text"],

Cell[CellGroupData[{
Cell[26453, 822, 116, 2, 18, "Input"],
Cell[26572, 826, 12167, 325, 138, 2913, 206, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[38754, 1154, 488, 11, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[39279, 1170, 134, 3, 40, "Subsection"],
Cell[39416, 1175, 260, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[39701, 1187, 152, 3, 32, "Subsubsection"],
Cell[39856, 1192, 200, 6, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40093, 1203, 164, 3, 32, "Subsubsection"],
Cell[40260, 1208, 238, 8, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40535, 1221, 138, 3, 32, "Subsubsection"],
Cell[40676, 1226, 170, 3, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[40883, 1234, 141, 3, 32, "Subsubsection"],
Cell[41027, 1239, 208, 6, 24, "Text"],
Cell[41238, 1247, 569, 12, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41844, 1264, 98, 1, 32, "Subsubsection"],
Cell[41945, 1267, 640, 10, 62, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[42622, 1282, 107, 1, 32, "Subsubsection"],
Cell[42732, 1285, 803, 12, 88, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43572, 1302, 103, 1, 32, "Subsubsection"],
Cell[43678, 1305, 874, 14, 126, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[44589, 1324, 133, 2, 32, "Subsubsection",
  PageBreakAbove->Automatic],
Cell[44725, 1328, 1156, 25, 100, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[45930, 1359, 94, 1, 40, "Subsection"],
Cell[46027, 1362, 524, 10, 49, "Text"],

Cell[CellGroupData[{
Cell[46576, 1376, 104, 1, 32, "Subsubsection"],
Cell[46683, 1379, 672, 20, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[47392, 1404, 119, 2, 32, "Subsubsection",
  PageBreakAbove->Automatic],
Cell[47514, 1408, 1530, 45, 113, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49081, 1458, 95, 1, 32, "Subsubsection"],
Cell[49179, 1461, 668, 18, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[49884, 1484, 155, 2, 32, "Subsubsection",
  PageBreakAbove->Automatic],
Cell[50042, 1488, 815, 17, 76, "Text",
  PageBreakWithin->True],

Cell[CellGroupData[{
Cell[50882, 1509, 172, 3, 30, "Input"],
Cell[51057, 1514, 46, 1, 22, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51140, 1520, 117, 2, 20, "Input"],
Cell[51260, 1524, 77, 1, 22, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[51374, 1530, 111, 2, 20, "Input"],
Cell[51488, 1534, 83, 1, 18, "Output"]
}, Open  ]],
Cell[51586, 1538, 670, 12, 62, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[52317, 1557, 142, 3, 44, "Section"],
Cell[52462, 1562, 1065, 24, 88, "Text"],

Cell[CellGroupData[{
Cell[53552, 1590, 143, 3, 40, "Subsection"],
Cell[53698, 1595, 657, 19, 49, "Text"],

Cell[CellGroupData[{
Cell[54380, 1618, 140, 2, 18, "Input"],
Cell[54523, 1622, 10647, 398, 112, 4552, 318, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[65185, 2023, 793, 26, 49, "Text"],

Cell[CellGroupData[{
Cell[66003, 2053, 188, 5, 28, "Input"],
Cell[66194, 2060, 3978, 277, 112, 3704, 270, "GraphicsData", 
"PostScript", "Graphics",
  ImageCacheValid->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[70221, 2343, 150, 3, 40, "Subsection"],
Cell[70374, 2348, 482, 15, 36, "Text"],

Cell[CellGroupData[{
Cell[70881, 2367, 113, 2, 18, "Input"],
Cell[70997, 2371, 7691, 324, 112, 3704, 270, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[78703, 2698, 445, 11, 36, "Text"],

Cell[CellGroupData[{
Cell[79173, 2713, 145, 2, 18, "Input"],
Cell[79321, 2717, 10714, 363, 112, 3693, 271, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[90050, 3083, 566, 13, 49, "Text"],

Cell[CellGroupData[{
Cell[90641, 3100, 117, 2, 20, "Input"],
Cell[90761, 3104, 10841, 370, 112, 3812, 278, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[101663, 3481, 150, 3, 44, "Section"],

Cell[CellGroupData[{
Cell[101838, 3488, 143, 3, 40, "Subsection"],

Cell[CellGroupData[{
Cell[102006, 3495, 150, 3, 32, "Subsubsection"],
Cell[102159, 3500, 383, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[102567, 3514, 103, 2, 18, "Input"],
Cell[102673, 3518, 67, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[102777, 3524, 120, 2, 18, "Input"],
Cell[102900, 3528, 99, 2, 20, "Output"]
}, Open  ]],
Cell[103014, 3533, 307, 7, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[103358, 3545, 159, 3, 32, "Subsubsection"],
Cell[103520, 3550, 290, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[103835, 3562, 108, 2, 18, "Input"],
Cell[103946, 3566, 38, 1, 21, "Output"]
}, Open  ]],
Cell[103999, 3570, 514, 13, 49, "Text"],

Cell[CellGroupData[{
Cell[104538, 3587, 111, 2, 18, "Input"],
Cell[104652, 3591, 54, 1, 20, "Output"]
}, Open  ]],
Cell[104721, 3595, 593, 15, 49, "Text"],

Cell[CellGroupData[{
Cell[105339, 3614, 112, 2, 18, "Input"],
Cell[105454, 3618, 36, 1, 20, "Output"]
}, Open  ]],
Cell[105505, 3622, 327, 7, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105869, 3634, 240, 8, 32, "Subsubsection"],
Cell[106112, 3644, 302, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[106439, 3656, 109, 2, 18, "Input"],
Cell[106551, 3660, 147, 2, 28, "Output"]
}, Open  ]],
Cell[106713, 3665, 505, 14, 36, "Text"],

Cell[CellGroupData[{
Cell[107243, 3683, 108, 2, 18, "Input"],
Cell[107354, 3687, 59, 1, 20, "Output"]
}, Open  ]],
Cell[107428, 3691, 508, 11, 49, "Text"],

Cell[CellGroupData[{
Cell[107961, 3706, 107, 2, 18, "Input"],
Cell[108071, 3710, 125, 3, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[108245, 3719, 188, 4, 32, "Subsubsection"],
Cell[108436, 3725, 282, 7, 36, "Text"],

Cell[CellGroupData[{
Cell[108743, 3736, 134, 2, 18, "Input"],
Cell[108880, 3740, 129, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[109046, 3747, 104, 2, 18, "Input"],
Cell[109153, 3751, 115, 2, 20, "Output"]
}, Open  ]],
Cell[109283, 3756, 159, 3, 24, "Text"],

Cell[CellGroupData[{
Cell[109467, 3763, 132, 2, 18, "Input"],
Cell[109602, 3767, 115, 2, 20, "Output"]
}, Open  ]],
Cell[109732, 3772, 380, 8, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[110149, 3785, 150, 3, 32, "Subsubsection"],
Cell[110302, 3790, 590, 11, 62, "Text"],

Cell[CellGroupData[{
Cell[110917, 3805, 106, 2, 18, "Input"],
Cell[111026, 3809, 40, 1, 20, "Output"]
}, Open  ]],
Cell[111081, 3813, 281, 7, 36, "Text"],

Cell[CellGroupData[{
Cell[111387, 3824, 105, 2, 18, "Input"],
Cell[111495, 3828, 35, 1, 20, "Output"]
}, Open  ]],
Cell[111545, 3832, 280, 9, 24, "Text"],
Cell[111828, 3843, 108, 2, 18, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[111973, 3850, 124, 1, 32, "Subsubsection"],
Cell[112100, 3853, 338, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[112463, 3867, 134, 2, 18, "Input"],
Cell[112600, 3871, 129, 2, 28, "Output"]
}, Open  ]],
Cell[112744, 3876, 335, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[113104, 3889, 98, 2, 20, "Input"],
Cell[113205, 3893, 97, 2, 28, "Output"]
}, Open  ]],
Cell[113317, 3898, 108, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[113450, 3903, 146, 2, 20, "Input"],
Cell[113599, 3907, 97, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[113745, 3915, 161, 3, 32, "Subsubsection"],
Cell[113909, 3920, 663, 23, 36, "Text"],

Cell[CellGroupData[{
Cell[114597, 3947, 112, 2, 18, "Input"],
Cell[114712, 3951, 54, 1, 20, "Output"]
}, Open  ]],
Cell[114781, 3955, 553, 12, 49, "Text"],

Cell[CellGroupData[{
Cell[115359, 3971, 126, 2, 18, "Input"],
Cell[115488, 3975, 76, 1, 20, "Output"]
}, Open  ]],
Cell[115579, 3979, 291, 9, 24, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[115919, 3994, 149, 3, 40, "Subsection"],

Cell[CellGroupData[{
Cell[116093, 4001, 153, 3, 32, "Subsubsection"],
Cell[116249, 4006, 376, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[116650, 4020, 126, 2, 18, "Input"],
Cell[116779, 4024, 49, 1, 20, "Output"]
}, Open  ]],
Cell[116843, 4028, 500, 14, 36, "Text"],

Cell[CellGroupData[{
Cell[117368, 4046, 159, 2, 18, "Input"],
Cell[117530, 4050, 6268, 261, 109, 2956, 215, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[123813, 4314, 484, 15, 36, "Text"],

Cell[CellGroupData[{
Cell[124322, 4333, 149, 2, 18, "Input"],
Cell[124474, 4337, 203, 3, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[124714, 4345, 134, 2, 18, "Input"],
Cell[124851, 4349, 10018, 378, 109, 4290, 302, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[134884, 4730, 417, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[135326, 4744, 146, 2, 18, "Input"],
Cell[135475, 4748, 8301, 344, 109, 4043, 287, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[143825, 5098, 169, 3, 32, "Subsubsection"],
Cell[143997, 5103, 568, 14, 49, "Text"],
Cell[144568, 5119, 151, 2, 18, "Input"],

Cell[CellGroupData[{
Cell[144744, 5125, 144, 2, 18, "Input"],
Cell[144891, 5129, 148, 3, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[145076, 5137, 129, 2, 18, "Input"],
Cell[145208, 5141, 10929, 409, 109, 4624, 326, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[156174, 5555, 157, 2, 18, "Input"],
Cell[156334, 5559, 6657, 273, 109, 3050, 224, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[163028, 5837, 124, 2, 18, "Input"],
Cell[163155, 5841, 10999, 414, 109, 4723, 332, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[174215, 6262, 153, 3, 40, "Subsection"],

Cell[CellGroupData[{
Cell[174393, 6269, 161, 3, 32, "Subsubsection"],
Cell[174557, 6274, 477, 13, 36, "Text"],

Cell[CellGroupData[{
Cell[175059, 6291, 141, 2, 18, "Input"],
Cell[175203, 6295, 225, 3, 20, "Output"]
}, Open  ]],
Cell[175443, 6301, 286, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[175754, 6314, 148, 2, 18, "Input"],
Cell[175905, 6318, 565, 11, 30, "Output"]
}, Open  ]],
Cell[176485, 6332, 275, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[176785, 6344, 111, 2, 18, "Input"],
Cell[176899, 6348, 378, 7, 30, "Output"]
}, Open  ]],
Cell[177292, 6358, 367, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[177684, 6372, 131, 2, 18, "Input"],
Cell[177818, 6376, 66, 1, 20, "Output"]
}, Open  ]],
Cell[177899, 6380, 180, 5, 24, "Text"],

Cell[CellGroupData[{
Cell[178104, 6389, 107, 2, 18, "Input"],
Cell[178214, 6393, 54, 1, 20, "Output"]
}, Open  ]],
Cell[178283, 6397, 558, 12, 49, "Text"],

Cell[CellGroupData[{
Cell[178866, 6413, 131, 2, 18, "Input"],
Cell[179000, 6417, 71, 1, 20, "Output"]
}, Open  ]],
Cell[179086, 6421, 797, 20, 62, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[179944, 6448, 133, 3, 44, "Section"],
Cell[180080, 6453, 547, 13, 49, "Text"],

Cell[CellGroupData[{
Cell[180652, 6470, 151, 3, 32, "Subsubsection"],
Cell[180806, 6475, 402, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[181233, 6487, 138, 2, 18, "Input"],
Cell[181374, 6491, 75, 1, 20, "Output"]
}, Open  ]],
Cell[181464, 6495, 493, 12, 49, "Text"],

Cell[CellGroupData[{
Cell[181982, 6511, 109, 2, 18, "Input"],
Cell[182094, 6515, 35, 1, 20, "Output"]
}, Open  ]],
Cell[182144, 6519, 479, 12, 36, "Text"],

Cell[CellGroupData[{
Cell[182648, 6535, 87, 1, 18, "Input"],
Cell[182738, 6538, 75, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[182850, 6544, 91, 1, 18, "Input"],
Cell[182944, 6547, 75, 1, 20, "Output"]
}, Open  ]],
Cell[183034, 6551, 305, 11, 24, "Text"],

Cell[CellGroupData[{
Cell[183364, 6566, 93, 1, 18, "Input"],
Cell[183460, 6569, 81, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[183578, 6575, 93, 1, 18, "Input"],
Cell[183674, 6578, 82, 1, 20, "Output"]
}, Open  ]],
Cell[183771, 6582, 217, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[184013, 6593, 94, 2, 20, "Input"],
Cell[184110, 6597, 75, 1, 20, "Output"]
}, Open  ]],
Cell[184200, 6601, 493, 19, 24, "Text"],

Cell[CellGroupData[{
Cell[184718, 6624, 95, 1, 18, "Input"],
Cell[184816, 6627, 81, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[184934, 6633, 94, 2, 20, "Input"],
Cell[185031, 6637, 81, 1, 20, "Output"]
}, Open  ]],
Cell[185127, 6641, 416, 17, 24, "Text"],

Cell[CellGroupData[{
Cell[185568, 6662, 85, 1, 18, "Input"],
Cell[185656, 6665, 38, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[185731, 6671, 84, 1, 18, "Input"],
Cell[185818, 6674, 75, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[185930, 6680, 84, 1, 18, "Input"],
Cell[186017, 6683, 37, 1, 20, "Output"]
}, Open  ]],
Cell[186069, 6687, 494, 15, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[186600, 6707, 117, 1, 32, "Subsubsection"],
Cell[186720, 6710, 210, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[186955, 6718, 115, 1, 18, "Input"],
Cell[187073, 6721, 76, 1, 20, "Output"]
}, Open  ]],
Cell[187164, 6725, 454, 14, 36, "Text"],

Cell[CellGroupData[{
Cell[187643, 6743, 91, 1, 18, "Input"],
Cell[187737, 6746, 215, 7, 42, "Output"]
}, Open  ]],
Cell[187967, 6756, 120, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[188112, 6761, 91, 1, 18, "Input"],
Cell[188206, 6764, 40, 1, 20, "Output"]
}, Open  ]],
Cell[188261, 6768, 491, 10, 49, "Text"],

Cell[CellGroupData[{
Cell[188777, 6782, 96, 1, 18, "Input"],
Cell[188876, 6785, 50, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[188963, 6791, 91, 1, 18, "Input"],
Cell[189057, 6794, 54, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[189148, 6800, 91, 1, 18, "Input"],
Cell[189242, 6803, 52, 1, 20, "Output"]
}, Open  ]],
Cell[189309, 6807, 327, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[189661, 6819, 87, 1, 18, "Input"],
Cell[189751, 6822, 40, 1, 20, "Output"]
}, Open  ]],
Cell[189806, 6826, 329, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[190160, 6838, 89, 1, 18, "Input"],
Cell[190252, 6841, 35, 1, 20, "Output"]
}, Open  ]],
Cell[190302, 6845, 274, 7, 36, "Text"],

Cell[CellGroupData[{
Cell[190601, 6856, 102, 1, 18, "Input"],
Cell[190706, 6859, 229, 8, 54, "Output"]
}, Open  ]],
Cell[190950, 6870, 276, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[191251, 6882, 175, 5, 37, "Input"],
Cell[191429, 6889, 321, 8, 64, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[191787, 6902, 122, 2, 20, "Input"],
Cell[191912, 6906, 249, 8, 54, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[192210, 6920, 103, 1, 32, "Subsubsection"],
Cell[192316, 6923, 508, 16, 36, "Text"],

Cell[CellGroupData[{
Cell[192849, 6943, 96, 1, 18, "Input"],
Cell[192948, 6946, 70, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[193055, 6952, 93, 1, 18, "Input"],
Cell[193151, 6955, 76, 1, 20, "Output"]
}, Open  ]],
Cell[193242, 6959, 254, 7, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[193533, 6971, 109, 1, 32, "Subsubsection"],
Cell[193645, 6974, 612, 12, 62, "Text"],

Cell[CellGroupData[{
Cell[194282, 6990, 78, 1, 18, "Input"],
Cell[194363, 6993, 81, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[194481, 6999, 90, 1, 18, "Input"],
Cell[194574, 7002, 38, 1, 20, "Output"]
}, Open  ]],
Cell[194627, 7006, 331, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[194983, 7019, 84, 1, 18, "Input"],
Cell[195070, 7022, 76, 1, 22, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[195183, 7028, 89, 1, 18, "Input"],
Cell[195275, 7031, 76, 1, 22, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[195400, 7038, 105, 1, 40, "Subsection"],

Cell[CellGroupData[{
Cell[195530, 7043, 96, 1, 32, "Subsubsection"],
Cell[195629, 7046, 233, 6, 24, "Text"],

Cell[CellGroupData[{
Cell[195887, 7056, 109, 1, 18, "Input"],
Cell[195999, 7059, 42, 1, 16, "Print"],
Cell[196044, 7062, 42, 1, 16, "Print"],
Cell[196089, 7065, 43, 1, 16, "Print"],
Cell[196135, 7068, 44, 1, 16, "Print"]
}, Open  ]],
Cell[196194, 7072, 602, 20, 36, "Text"],

Cell[CellGroupData[{
Cell[196821, 7096, 98, 1, 18, "Input"],
Cell[196922, 7099, 103, 2, 20, "Output"]
}, Open  ]],
Cell[197040, 7104, 366, 10, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[197443, 7119, 116, 1, 32, "Subsubsection"],
Cell[197562, 7122, 725, 17, 62, "Text"],
Cell[198290, 7141, 97, 1, 18, "Input"],
Cell[198390, 7144, 309, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[198724, 7156, 87, 1, 18, "Input"],
Cell[198814, 7159, 44, 1, 20, "Output"]
}, Open  ]],
Cell[198873, 7163, 206, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[199104, 7174, 93, 1, 18, "Input"],
Cell[199200, 7177, 76, 1, 20, "Output"]
}, Open  ]],
Cell[199291, 7181, 158, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[199474, 7189, 95, 1, 18, "Input"],
Cell[199572, 7192, 53, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[199662, 7198, 99, 1, 18, "Input"],
Cell[199764, 7201, 82, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[199895, 7208, 103, 1, 32, "Subsubsection"],
Cell[200001, 7211, 295, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[200321, 7225, 126, 1, 18, "Input"],
Cell[200450, 7228, 18095, 692, 116, 8213, 565, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[218560, 7923, 744, 22, 49, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[219353, 7951, 104, 1, 40, "Subsection"],
Cell[219460, 7954, 168, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[219653, 7962, 109, 1, 32, "Subsubsection"],
Cell[219765, 7965, 410, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[220200, 7978, 78, 1, 18, "Input"],
Cell[220281, 7981, 81, 1, 20, "Output"]
}, Open  ]],
Cell[220377, 7985, 277, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[220679, 7998, 75, 1, 18, "Input"],
Cell[220757, 8001, 81, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[220875, 8007, 81, 1, 18, "Input"],
Cell[220959, 8010, 35, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[221043, 8017, 105, 1, 32, "Subsubsection"],
Cell[221151, 8020, 832, 21, 62, "Text"],
Cell[221986, 8043, 105, 1, 18, "Input"],
Cell[222094, 8046, 257, 5, 36, "Text"],

Cell[CellGroupData[{
Cell[222376, 8055, 104, 2, 20, "Input"],
Cell[222483, 8059, 168, 3, 20, "Output"]
}, Open  ]],
Cell[222666, 8065, 227, 6, 24, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[222954, 8078, 135, 3, 44, "Section"],

Cell[CellGroupData[{
Cell[223114, 8085, 156, 3, 40, "Subsection"],
Cell[223273, 8090, 419, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[223717, 8104, 119, 2, 18, "Input"],
Cell[223839, 8108, 58, 1, 20, "Output"]
}, Open  ]],
Cell[223912, 8112, 224, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[224161, 8124, 107, 2, 18, "Input"],
Cell[224271, 8128, 65, 1, 20, "Output"]
}, Open  ]],
Cell[224351, 8132, 167, 3, 24, "Text"],

Cell[CellGroupData[{
Cell[224543, 8139, 113, 2, 18, "Input"],
Cell[224659, 8143, 48, 1, 20, "Output"]
}, Open  ]],
Cell[224722, 8147, 174, 3, 24, "Text"],

Cell[CellGroupData[{
Cell[224921, 8154, 109, 2, 18, "Input"],
Cell[225033, 8158, 69, 1, 20, "Output"]
}, Open  ]],
Cell[225117, 8162, 292, 11, 24, "Text"],

Cell[CellGroupData[{
Cell[225434, 8177, 105, 2, 18, "Input"],
Cell[225542, 8181, 69, 1, 20, "Output"]
}, Open  ]],
Cell[225626, 8185, 303, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[225954, 8197, 116, 2, 18, "Input"],
Cell[226073, 8201, 116, 2, 20, "Output"]
}, Open  ]],
Cell[226204, 8206, 288, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[226517, 8219, 90, 4, 28, "Input"],
Cell[226610, 8225, 36, 1, 20, "Output"]
}, Open  ]],
Cell[226661, 8229, 263, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[226949, 8242, 84, 1, 18, "Input"],
Cell[227036, 8245, 69, 1, 20, "Output"]
}, Open  ]],
Cell[227120, 8249, 1123, 36, 62, "Text"],
Cell[228246, 8287, 84, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[228355, 8292, 114, 1, 32, "Subsubsection"],
Cell[228472, 8295, 781, 18, 62, "Text"],
Cell[229256, 8315, 266, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[229547, 8328, 103, 1, 18, "Input"],
Cell[229653, 8331, 86, 1, 28, "Output"]
}, Open  ]],
Cell[229754, 8335, 295, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[230074, 8347, 89, 1, 18, "Input"],
Cell[230166, 8350, 102, 2, 30, "Output"]
}, Open  ]],
Cell[230283, 8355, 273, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[230581, 8367, 88, 1, 18, "Input"],
Cell[230672, 8370, 74, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[230783, 8376, 86, 1, 18, "Input"],
Cell[230872, 8379, 49, 1, 20, "Output"]
}, Open  ]],
Cell[230936, 8383, 148, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[231109, 8391, 94, 1, 18, "Input"],
Cell[231206, 8394, 87, 1, 28, "Output"]
}, Open  ]],
Cell[231308, 8398, 828, 23, 62, "Text",
  PageBreakAbove->Automatic]
}, Open  ]],

Cell[CellGroupData[{
Cell[232173, 8426, 123, 1, 32, "Subsubsection"],
Cell[232299, 8429, 298, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[232622, 8441, 126, 1, 18, "Input"],
Cell[232751, 8444, 140, 2, 30, "Output"]
}, Open  ]],
Cell[232906, 8449, 237, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[233168, 8460, 86, 1, 18, "Input"],
Cell[233257, 8463, 497, 8, 54, "Output"]
}, Open  ]],
Cell[233769, 8474, 262, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[234056, 8487, 89, 1, 18, "Input"],
Cell[234148, 8490, 367, 5, 52, "Output"]
}, Open  ]],
Cell[234530, 8498, 248, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[234803, 8509, 85, 1, 18, "Input"],
Cell[234891, 8512, 88, 1, 28, "Output"]
}, Open  ]],
Cell[234994, 8516, 222, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[235241, 8527, 82, 1, 18, "Input"],
Cell[235326, 8530, 92, 1, 28, "Output"]
}, Open  ]],
Cell[235433, 8534, 265, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[235723, 8546, 83, 1, 18, "Input"],
Cell[235809, 8549, 123, 2, 30, "Output"]
}, Open  ]],
Cell[235947, 8554, 396, 11, 36, "Text"],

Cell[CellGroupData[{
Cell[236368, 8569, 86, 1, 18, "Input"],
Cell[236457, 8572, 123, 2, 30, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[236629, 8580, 104, 1, 32, "Subsubsection"],
Cell[236736, 8583, 366, 11, 36, "Text"],

Cell[CellGroupData[{
Cell[237127, 8598, 124, 4, 28, "Input"],
Cell[237254, 8604, 67, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[237358, 8610, 87, 1, 18, "Input"],
Cell[237448, 8613, 147, 2, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[237632, 8620, 87, 1, 18, "Input"],
Cell[237722, 8623, 144, 2, 20, "Output"]
}, Open  ]],
Cell[237881, 8628, 677, 23, 36, "Text"],

Cell[CellGroupData[{
Cell[238583, 8655, 105, 1, 18, "Input"],
Cell[238691, 8658, 53, 1, 20, "Output"]
}, Open  ]],
Cell[238759, 8662, 404, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[239188, 8675, 106, 1, 18, "Input"],
Cell[239297, 8678, 53, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[239387, 8684, 106, 1, 18, "Input"],
Cell[239496, 8687, 41, 1, 20, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[239598, 8695, 99, 1, 40, "Subsection"],
Cell[239700, 8698, 243, 5, 36, "Text"],
Cell[239946, 8705, 80, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[240051, 8710, 79, 1, 18, "Input"],
Cell[240133, 8713, 61, 1, 20, "Output"]
}, Open  ]],
Cell[240209, 8717, 501, 15, 37, "Text"],
Cell[240713, 8734, 82, 1, 18, "Input"],
Cell[240798, 8737, 701, 23, 49, "Text"],

Cell[CellGroupData[{
Cell[241524, 8764, 121, 2, 18, "Input"],
Cell[241648, 8768, 91, 2, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[241776, 8775, 79, 1, 18, "Input"],
Cell[241858, 8778, 67, 1, 20, "Output"]
}, Open  ]],
Cell[241940, 8782, 357, 9, 36, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[242346, 8797, 136, 3, 44, "Section"],
Cell[242485, 8802, 884, 14, 88, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[243406, 8821, 139, 3, 44, "Section"],
Cell[243548, 8826, 299, 7, 36, "Text"],

Cell[CellGroupData[{
Cell[243872, 8837, 127, 5, 28, "Input"],
Cell[244002, 8844, 54, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[244093, 8850, 115, 2, 18, "Input"],
Cell[244211, 8854, 38, 1, 20, "Output"]
}, Open  ]],
Cell[244264, 8858, 242, 6, 24, "Text"],

Cell[CellGroupData[{
Cell[244531, 8868, 119, 2, 18, "Input"],
Cell[244653, 8872, 44, 1, 20, "Output"]
}, Open  ]],
Cell[244712, 8876, 368, 12, 24, "Text"],

Cell[CellGroupData[{
Cell[245105, 8892, 122, 2, 18, "Input"],
Cell[245230, 8896, 44, 1, 20, "Output"]
}, Open  ]],
Cell[245289, 8900, 294, 7, 36, "Text"],

Cell[CellGroupData[{
Cell[245608, 8911, 132, 2, 18, "Input"],
Cell[245743, 8915, 36, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[245816, 8921, 164, 3, 32, "Subsubsection"],
Cell[245983, 8926, 293, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[246301, 8939, 116, 2, 18, "Input"],
Cell[246420, 8943, 47, 1, 20, "Output"]
}, Open  ]],
Cell[246482, 8947, 326, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[246833, 8961, 94, 1, 18, "Input"],
Cell[246930, 8964, 51, 1, 28, "Output"]
}, Open  ]],
Cell[246996, 8968, 182, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[247203, 8976, 100, 1, 18, "Input"],
Cell[247306, 8979, 45, 1, 20, "Output"]
}, Open  ]],
Cell[247366, 8983, 617, 11, 62, "Text"],

Cell[CellGroupData[{
Cell[248008, 8998, 134, 2, 18, "Input"],
Cell[248145, 9002, 81, 1, 32, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[248263, 9008, 105, 1, 18, "Input"],
Cell[248371, 9011, 54, 1, 20, "Output"]
}, Open  ]],
Cell[248440, 9015, 179, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[248644, 9023, 104, 1, 18, "Input"],
Cell[248751, 9026, 59, 1, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[248847, 9032, 78, 1, 18, "Input"],
Cell[248928, 9035, 57, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[249022, 9041, 105, 1, 18, "Input"],
Cell[249130, 9044, 209, 4, 28, "Message"],
Cell[249342, 9050, 57, 1, 20, "Output"]
}, Open  ]],
Cell[249414, 9054, 379, 8, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[249830, 9067, 88, 1, 32, "Subsubsection"],
Cell[249921, 9070, 238, 6, 24, "Text"],

Cell[CellGroupData[{
Cell[250184, 9080, 108, 1, 18, "Input"],
Cell[250295, 9083, 41, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[250385, 9090, 112, 2, 32, "Subsubsection",
  PageBreakAbove->True],
Cell[250500, 9094, 656, 16, 49, "Text"],

Cell[CellGroupData[{
Cell[251181, 9114, 104, 1, 18, "Input"],
Cell[251288, 9117, 293, 7, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[251618, 9129, 77, 1, 18, "Input"],
Cell[251698, 9132, 321, 8, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[252056, 9145, 84, 1, 18, "Input"],
Cell[252143, 9148, 56, 1, 28, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[252248, 9155, 117, 1, 40, "Subsection"],
Cell[252368, 9158, 299, 6, 36, "Text"],

Cell[CellGroupData[{
Cell[252692, 9168, 125, 1, 32, "Subsubsection"],
Cell[252820, 9171, 604, 16, 49, "Text"],

Cell[CellGroupData[{
Cell[253449, 9191, 109, 1, 18, "Input"],
Cell[253561, 9194, 78, 1, 20, "Output"]
}, Open  ]],
Cell[253654, 9198, 163, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[253842, 9206, 153, 2, 18, "Input"],
Cell[253998, 9210, 100, 2, 25, "Output"]
}, Open  ]],
Cell[254113, 9215, 736, 16, 62, "Text"],

Cell[CellGroupData[{
Cell[254874, 9235, 88, 1, 18, "Input"],
Cell[254965, 9238, 53, 1, 24, "Output"]
}, Open  ]],
Cell[255033, 9242, 154, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[255212, 9250, 86, 1, 18, "Input"],
Cell[255301, 9253, 35, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[255385, 9260, 127, 1, 32, "Subsubsection"],
Cell[255515, 9263, 640, 12, 62, "Text"],

Cell[CellGroupData[{
Cell[256180, 9279, 197, 5, 37, "Input"],
Cell[256380, 9286, 415, 11, 20, "Output"]
}, Open  ]],
Cell[256810, 9300, 492, 14, 36, "Text"],

Cell[CellGroupData[{
Cell[257327, 9318, 94, 1, 18, "Input"],
Cell[257424, 9321, 87, 1, 20, "Output"]
}, Open  ]],
Cell[257526, 9325, 400, 9, 36, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[257975, 9340, 94, 1, 40, "Subsection"],

Cell[CellGroupData[{
Cell[258094, 9345, 125, 1, 32, "Subsubsection"],
Cell[258222, 9348, 915, 19, 88, "Text"],
Cell[259140, 9369, 130, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[259295, 9374, 112, 1, 18, "Input"],
Cell[259410, 9377, 117, 2, 28, "Output"]
}, Open  ]],
Cell[259542, 9382, 120, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[259687, 9387, 82, 1, 18, "Input"],
Cell[259772, 9390, 255, 6, 20, "Output"]
}, Open  ]],
Cell[260042, 9399, 167, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[260234, 9407, 92, 1, 18, "Input"],
Cell[260329, 9410, 188, 3, 20, "Output"]
}, Open  ]],
Cell[260532, 9416, 179, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[260736, 9424, 112, 2, 20, "Input"],
Cell[260851, 9428, 163, 3, 28, "Output"]
}, Open  ]],
Cell[261029, 9434, 384, 12, 36, "Text"],

Cell[CellGroupData[{
Cell[261438, 9450, 87, 1, 18, "Input"],
Cell[261528, 9453, 80, 1, 20, "Output"]
}, Open  ]],
Cell[261623, 9457, 311, 8, 36, "Text"],
Cell[261937, 9467, 86, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[262048, 9472, 79, 1, 18, "Input"],
Cell[262130, 9475, 47, 1, 20, "Output"]
}, Open  ]],
Cell[262192, 9479, 237, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[262454, 9490, 94, 1, 18, "Input"],
Cell[262551, 9493, 188, 3, 20, "Output"]
}, Open  ]],
Cell[262754, 9499, 302, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[263081, 9513, 88, 1, 18, "Input"],
Cell[263172, 9516, 78, 1, 20, "Output"]
}, Open  ]],
Cell[263265, 9520, 295, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[263585, 9534, 88, 1, 18, "Input"],
Cell[263676, 9537, 83, 1, 20, "Output"]
}, Open  ]],
Cell[263774, 9541, 274, 8, 24, "Text"],

Cell[CellGroupData[{
Cell[264073, 9553, 101, 1, 18, "Input"],
Cell[264177, 9556, 135, 2, 28, "Output"]
}, Open  ]],
Cell[264327, 9561, 302, 7, 36, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[264666, 9573, 109, 1, 32, "Subsubsection"],
Cell[264778, 9576, 320, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[265123, 9588, 111, 1, 18, "Input"],
Cell[265237, 9591, 84, 1, 20, "Output"]
}, Open  ]],
Cell[265336, 9595, 187, 4, 24, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[265584, 9606, 148, 3, 44, "Section"],
Cell[265735, 9611, 1006, 31, 62, "Text"],

Cell[CellGroupData[{
Cell[266766, 9646, 128, 2, 18, "Input"],
Cell[266897, 9650, 12918, 393, 126, 3697, 274, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[279830, 10046, 664, 14, 62, "Text"],

Cell[CellGroupData[{
Cell[280519, 10064, 174, 3, 32, "Subsubsection"],
Cell[280696, 10069, 513, 12, 49, "Text"],

Cell[CellGroupData[{
Cell[281234, 10085, 109, 2, 18, "Input"],
Cell[281346, 10089, 100, 2, 16, "Print"]
}, Open  ]],
Cell[281461, 10094, 362, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[281848, 10107, 109, 2, 18, "Input",
  PageBreakAbove->Automatic],
Cell[281960, 10111, 178, 5, 42, "Print"]
}, Open  ]],
Cell[282153, 10119, 413, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[282591, 10133, 87, 1, 18, "Input"],
Cell[282681, 10136, 1011, 17, 96, "Output"]
}, Open  ]],
Cell[283707, 10156, 412, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[284144, 10168, 211, 5, 37, "Input"],
Cell[284358, 10175, 41247, 879, 126, 8172, 466, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[325620, 11057, 563, 16, 49, "Text"],

Cell[CellGroupData[{
Cell[326208, 11077, 109, 2, 20, "Input"],
Cell[326320, 11081, 572, 11, 41, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[326929, 11097, 123, 1, 18, "Input"],
Cell[327055, 11100, 57, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[327161, 11107, 134, 2, 40, "Subsection",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[327320, 11113, 111, 1, 32, "Subsubsection"],
Cell[327434, 11116, 610, 12, 62, "Text"],

Cell[CellGroupData[{
Cell[328069, 11132, 206, 5, 28, "Input"],
Cell[328278, 11139, 12058, 403, 136, 4418, 304, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[340351, 11545, 120, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[340496, 11550, 119, 2, 18, "Input"],
Cell[340618, 11554, 112, 2, 28, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[340767, 11561, 110, 1, 18, "Input"],
Cell[340880, 11564, 100, 2, 36, "Output"]
}, Open  ]],
Cell[340995, 11569, 291, 5, 36, "Text"],

Cell[CellGroupData[{
Cell[341311, 11578, 144, 4, 18, "Input"],
Cell[341458, 11584, 162, 3, 36, "Output"]
}, Open  ]],
Cell[341635, 11590, 219, 5, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[341891, 11600, 109, 1, 32, "Subsubsection"],
Cell[342003, 11603, 617, 15, 49, "Text"],

Cell[CellGroupData[{
Cell[342645, 11622, 87, 1, 18, "Input"],
Cell[342735, 11625, 47, 1, 20, "Output"]
}, Open  ]],
Cell[342797, 11629, 397, 9, 36, "Text"],

Cell[CellGroupData[{
Cell[343219, 11642, 84, 1, 18, "Input"],
Cell[343306, 11645, 41, 1, 21, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[343384, 11651, 136, 2, 18, "Input"],
Cell[343523, 11655, 384, 10, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[343968, 11672, 106, 1, 40, "Subsection"],

Cell[CellGroupData[{
Cell[344099, 11677, 108, 1, 32, "Subsubsection"],
Cell[344210, 11680, 332, 6, 36, "Text"],
Cell[344545, 11688, 96, 1, 18, "Input"],
Cell[344644, 11691, 384, 14, 24, "Text"],

Cell[CellGroupData[{
Cell[345053, 11709, 81, 1, 18, "Input"],
Cell[345137, 11712, 38, 1, 28, "Output"]
}, Open  ]],
Cell[345190, 11716, 109, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[345324, 11721, 76, 1, 18, "Input"],
Cell[345403, 11724, 43, 1, 16, "Print"],
Cell[345449, 11727, 247, 8, 16, "Print"]
}, Open  ]],
Cell[345711, 11738, 214, 7, 24, "Text"],
Cell[345928, 11747, 82, 1, 18, "Input"],
Cell[346013, 11750, 201, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[346239, 11761, 76, 1, 18, "Input"],
Cell[346318, 11764, 43, 1, 16, "Print"]
}, Open  ]],
Cell[346376, 11768, 315, 7, 36, "Text"],
Cell[346694, 11777, 109, 1, 18, "Input"],
Cell[346806, 11780, 117, 1, 18, "Input"],
Cell[346926, 11783, 377, 10, 36, "Text"],
Cell[347306, 11795, 103, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[347434, 11800, 85, 1, 18, "Input"],
Cell[347522, 11803, 39, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[347598, 11809, 86, 1, 18, "Input"],
Cell[347687, 11812, 38, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[347774, 11819, 119, 1, 32, "Subsubsection"],
Cell[347896, 11822, 919, 26, 62, "Text"],
Cell[348818, 11850, 88, 1, 18, "Input"],
Cell[348909, 11853, 90, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[349024, 11858, 83, 1, 18, "Input"],
Cell[349110, 11861, 42, 1, 20, "Output"]
}, Open  ]],
Cell[349167, 11865, 231, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[349423, 11876, 87, 1, 18, "Input"],
Cell[349513, 11879, 35, 1, 20, "Output"]
}, Open  ]],
Cell[349563, 11883, 301, 6, 36, "Text"],
Cell[349867, 11891, 94, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[349986, 11896, 110, 2, 18, "Input"],
Cell[350099, 11900, 65, 2, 20, "Output"]
}, Open  ]],
Cell[350179, 11905, 79, 1, 24, "Text"],

Cell[CellGroupData[{
Cell[350283, 11910, 93, 1, 18, "Input"],
Cell[350379, 11913, 35, 1, 20, "Output"]
}, Open  ]],
Cell[350429, 11917, 383, 7, 49, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[350849, 11929, 121, 1, 32, "Subsubsection"],
Cell[350973, 11932, 287, 5, 36, "Text"],

Cell[CellGroupData[{
Cell[351285, 11941, 95, 1, 18, "Input"],
Cell[351383, 11944, 69, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[351489, 11950, 109, 1, 18, "Input"],
Cell[351601, 11953, 65, 1, 36, "Output"]
}, Open  ]],
Cell[351681, 11957, 107, 2, 20, "Input"],

Cell[CellGroupData[{
Cell[351813, 11963, 116, 1, 18, "Input"],
Cell[351932, 11966, 90, 1, 20, "Output"]
}, Open  ]],
Cell[352037, 11970, 412, 10, 36, "Text"],

Cell[CellGroupData[{
Cell[352474, 11984, 129, 1, 18, "Input"],
Cell[352606, 11987, 43, 1, 20, "Output"]
}, Open  ]],
Cell[352664, 11991, 220, 5, 24, "Text"],

Cell[CellGroupData[{
Cell[352909, 12000, 120, 1, 18, "Input"],
Cell[353032, 12003, 60, 1, 20, "Output"]
}, Open  ]],
Cell[353107, 12007, 150, 4, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[353294, 12016, 142, 2, 32, "Subsubsection",
  PageBreakAbove->Automatic],
Cell[353439, 12020, 219, 5, 24, "Text"],
Cell[353661, 12027, 82, 1, 18, "Input"],
Cell[353746, 12030, 83, 1, 18, "Input"],
Cell[353832, 12033, 83, 1, 18, "Input"],
Cell[353918, 12036, 98, 1, 18, "Input"],
Cell[354019, 12039, 187, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[354231, 12050, 78, 1, 18, "Input"],
Cell[354312, 12053, 35, 1, 20, "Output"]
}, Open  ]],
Cell[354362, 12057, 233, 5, 36, "Text"],
Cell[354598, 12064, 82, 1, 18, "Input"],
Cell[354683, 12067, 83, 1, 18, "Input"],
Cell[354769, 12070, 83, 1, 18, "Input"],
Cell[354855, 12073, 104, 1, 18, "Input"],
Cell[354962, 12076, 220, 5, 24, "Text"],

Cell[CellGroupData[{
Cell[355207, 12085, 133, 3, 20, "Input"],
Cell[355343, 12090, 103, 2, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[355483, 12097, 103, 2, 20, "Input"],
Cell[355589, 12101, 56, 1, 20, "Output"]
}, Open  ]],
Cell[355660, 12105, 585, 9, 62, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[356282, 12119, 113, 1, 32, "Subsubsection"],
Cell[356398, 12122, 559, 11, 49, "Text"],

Cell[CellGroupData[{
Cell[356982, 12137, 77, 1, 18, "Input"],
Cell[357062, 12140, 35, 1, 20, "Output"]
}, Open  ]],
Cell[357112, 12144, 244, 7, 24, "Text"],
Cell[357359, 12153, 92, 1, 18, "Input"],
Cell[357454, 12156, 352, 12, 24, "Text"],

Cell[CellGroupData[{
Cell[357831, 12172, 91, 1, 18, "Input"],
Cell[357925, 12175, 38, 1, 20, "Output"]
}, Open  ]],
Cell[357978, 12179, 315, 11, 24, "Text"],

Cell[CellGroupData[{
Cell[358318, 12194, 97, 1, 18, "Input"],
Cell[358418, 12197, 40, 1, 20, "Output"]
}, Open  ]],
Cell[358473, 12201, 578, 19, 36, "Text"],

Cell[CellGroupData[{
Cell[359076, 12224, 79, 1, 18, "Input"],
Cell[359158, 12227, 37, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[359232, 12233, 126, 2, 18, "Input",
  PageBreakAbove->Automatic],
Cell[359361, 12237, 42, 1, 20, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[359452, 12244, 109, 1, 32, "Subsubsection"],
Cell[359564, 12247, 469, 12, 49, "Text"],
Cell[360036, 12261, 119, 1, 18, "Input"],

Cell[CellGroupData[{
Cell[360180, 12266, 118, 1, 18, "Input"],
Cell[360301, 12269, 64, 1, 20, "Output"]
}, Open  ]],
Cell[360380, 12273, 158, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[360563, 12281, 103, 1, 18, "Input"],
Cell[360669, 12284, 89, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[360795, 12290, 103, 1, 18, "Input"],
Cell[360901, 12293, 90, 1, 20, "Output"]
}, Open  ]],
Cell[361006, 12297, 301, 8, 36, "Text"],
Cell[361310, 12307, 112, 1, 18, "Input"],
Cell[361425, 12310, 112, 1, 18, "Input"],
Cell[361540, 12313, 256, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[361821, 12324, 140, 2, 18, "Input"],
Cell[361964, 12328, 37275, 1478, 120, 19178, 1250, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[399276, 13811, 110, 1, 18, "Input"],
Cell[399389, 13814, 30728, 1420, 120, 19553, 1277, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[430132, 15237, 979, 23, 100, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[431160, 15266, 87, 1, 40, "Subsection"],
Cell[431250, 15269, 979, 23, 76, "Text"],

Cell[CellGroupData[{
Cell[432254, 15296, 120, 1, 32, "Subsubsection"],
Cell[432377, 15299, 654, 16, 49, "Text"],

Cell[CellGroupData[{
Cell[433056, 15319, 82, 1, 18, "Input"],
Cell[433141, 15322, 117, 2, 16, "Print"]
}, Open  ]],
Cell[433273, 15327, 309, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[433607, 15339, 84, 1, 18, "Input"],
Cell[433694, 15342, 46, 0, 16, "Print"]
}, Open  ]],
Cell[433755, 15345, 251, 7, 24, "Text"],
Cell[434009, 15354, 111, 1, 18, "Input"],
Cell[434123, 15357, 164, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[434312, 15365, 102, 1, 18, "Input"],
Cell[434417, 15368, 672, 15, 72, "Print"]
}, Open  ]],
Cell[435104, 15386, 273, 7, 24, "Text"],

Cell[CellGroupData[{
Cell[435402, 15397, 91, 1, 18, "Input"],
Cell[435496, 15400, 304, 5, 36, "Print"]
}, Open  ]],
Cell[435815, 15408, 163, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[436003, 15416, 141, 2, 18, "Input"],
Cell[436147, 15420, 108, 2, 22, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[436328, 15430, 168, 4, 44, "Section",
  PageBreakAbove->True],
Cell[436499, 15436, 397, 8, 36, "Text"],

Cell[CellGroupData[{
Cell[436921, 15448, 164, 3, 40, "Subsection"],
Cell[437088, 15453, 2609, 82, 140, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[439734, 15540, 173, 3, 40, "Subsection"],
Cell[439910, 15545, 286, 9, 24, "Text"],

Cell[CellGroupData[{
Cell[440221, 15558, 127, 2, 18, "Input"],
Cell[440351, 15562, 38689, 852, 128, 6672, 452, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[479055, 16417, 543, 11, 49, "Text"],

Cell[CellGroupData[{
Cell[479623, 16432, 436, 11, 96, "Input"],
Cell[480062, 16445, 94835, 2294, 172, 24270, 1418, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[574912, 18742, 657, 14, 62, "Text"],

Cell[CellGroupData[{
Cell[575594, 18760, 184, 4, 28, "Input"],
Cell[575781, 18766, 26618, 872, 136, 8680, 646, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[602436, 19643, 276, 7, 28, "Input",
  PageBreakAbove->Automatic,
  PageBreakWithin->True],
Cell[602715, 19652, 17045, 529, 136, 6034, 388, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[619775, 20184, 190, 4, 24, "Text"],

Cell[CellGroupData[{
Cell[619990, 20192, 161, 4, 28, "Input"],
Cell[620154, 20198, 57056, 1334, 136, 11384, 765, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[677225, 21535, 460, 10, 49, "Text"],

Cell[CellGroupData[{
Cell[677710, 21549, 170, 5, 28, "Input",
  PageBreakAbove->Automatic],
Cell[677883, 21556, 17445, 550, 118, 8567, 436, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[695377, 22112, 142, 2, 40, "Subsection"],
Cell[695522, 22116, 409, 11, 36, "Text"],

Cell[CellGroupData[{
Cell[695956, 22131, 274, 7, 56, "Input"],
Cell[696233, 22140, 10857, 367, 112, 3635, 273, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[707105, 22510, 287, 10, 24, "Text"],

Cell[CellGroupData[{
Cell[707417, 22524, 144, 4, 28, "Input"],
Cell[707564, 22530, 12516, 433, 112, 4359, 327, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[720095, 22966, 613, 14, 49, "Text"],

Cell[CellGroupData[{
Cell[720733, 22984, 254, 8, 56, "Input"],
Cell[720990, 22994, 8892, 388, 112, 4383, 327, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[729919, 23387, 100, 1, 18, "Input"],
Cell[730022, 23390, 7326, 276, 112, 2977, 217, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[737363, 23669, 341, 11, 24, "Text"],
Cell[737707, 23682, 193, 5, 28, "Input",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[737925, 23691, 270, 7, 48, "Input",
  PageBreakAbove->Automatic],
Cell[738198, 23700, 11354, 407, 112, 6430, 341, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[749567, 24110, 189, 4, 24, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[749793, 24119, 137, 2, 40, "Subsection",
  PageBreakAbove->True],
Cell[749933, 24123, 692, 24, 36, "Text"],

Cell[CellGroupData[{
Cell[750650, 24151, 234, 4, 30, "Input"],
Cell[750887, 24157, 152513, 3755, 153, 125649, 3419, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[903437, 27917, 225, 4, 20, "Input",
  PageBreakAbove->Automatic],
Cell[903665, 27923, 70775, 3036, 162, 38343, 2631, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[974477, 30964, 254, 5, 20, "Input"],
Cell[974734, 30971, 36420, 850, 164, 11515, 537, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[1011169, 31824, 303, 6, 36, "Text",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[1011497, 31834, 276, 4, 41, "Input"],
Cell[1011776, 31840, 42713, 906, 93, 28106, 721, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1054526, 32751, 280, 4, 41, "Input"],
Cell[1054809, 32757, 57083, 2214, 156, 25663, 1821, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1111941, 34977, 117, 2, 40, "Subsection",
  PageBreakAbove->Automatic],
Cell[1112061, 34981, 893, 19, 76, "Text"],

Cell[CellGroupData[{
Cell[1112979, 35004, 169, 4, 28, "Input"],

Cell[CellGroupData[{
Cell[1113173, 35012, 43299, 1021, 153, 20001, 728, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1156475, 36035, 43218, 1021, 156, 20008, 729, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1199696, 37058, 43216, 1022, 156, 20014, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1242915, 38082, 43058, 1018, 156, 20047, 729, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1285976, 39102, 42752, 1015, 156, 20061, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1328731, 40119, 42525, 1014, 156, 20057, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1371259, 41135, 42954, 1019, 156, 20056, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1414216, 42156, 43243, 1022, 156, 20090, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1457462, 43180, 43450, 1025, 156, 20017, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1500915, 44207, 43262, 1024, 156, 19983, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1544180, 45233, 43299, 1021, 156, 20001, 728, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1587482, 46256, 43218, 1021, 156, 20008, 729, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1630703, 47279, 43216, 1022, 156, 20014, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1673922, 48303, 43058, 1018, 156, 20047, 729, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1716983, 49323, 42752, 1015, 156, 20061, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1759738, 50340, 42525, 1014, 156, 20057, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1802266, 51356, 42954, 1019, 156, 20056, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1845223, 52377, 43243, 1022, 156, 20090, 731, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1888469, 53401, 43450, 1025, 156, 20017, 730, "GraphicsData", 
"PostScript", "Graphics"],
Cell[1931922, 54428, 43262, 1024, 156, 19983, 731, "GraphicsData", 
"PostScript", "Graphics"]
}, Closed]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1975257, 55460, 98, 1, 24, "Section"],
Cell[1975358, 55463, 146, 4, 24, "Text"],
Cell[1975507, 55469, 922, 21, 190, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

