(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    239531,       7742]*)
(*NotebookOutlinePosition[    241381,       7799]*)
(*  CellTagsIndexPosition[    241337,       7795]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 4. 
Mathematical Preliminaries\
\>", "Title",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell[TextData[StyleBox["Solving the Partial Differential Equations",
  FontSlant->"Italic"]], "Subsubtitle",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["4.1 Introduction", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nIn Chapter 3 we saw how to derive partial differential equations for \
various types of financial instrument. One view of basic option pricing is \
that one now just needs to solve these equations with various interesting \
boundary conditions, and we shall motivate our basic solutions via this \
approach. It has the advantages of simplicity, and calls on rather basic \
techniques in applied mathematics. A rather gentle approach will be adopted, \
and we shall gradually build in contributions from ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to help solve the relevant equations.\n\nMuch of the discussion presented \
here parallels closely the treatment of the heat equation to be found in many \
elementary calculus texts. Indeed, we shall see along the way how to convert \
some of the differential equations directly into the heat equation. The \
discussion will begin by looking at time-independent and \
asset-price-independent solutions, and gradually increase the complexity."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.2 The One-Variable Black-Scholes Equation", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We begin by writing down the standard form of our partial \
differential equation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "+", 
          RowBox[{"S", " ", \((r - q)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "+", 
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
One way to simplify matters is to seek solutions that are functions \
of one variable only. We obtain, for solutions that are functions of time \
only,\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For functions of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " only, we get something a little more interesting."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", " ", 
          RowBox[{"S", " ", \((r - q)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We shall suppose, until stated otherwise, that all the parameters \
in these equations are constants. The first equation has a simple solution of \
the form\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V = Z\ \[ExponentialE]\^\(r\ \((t - T)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where we have a boundary condition that ",
  Cell[BoxData[
      \(TraditionalForm\`V\  = \ Z\)]],
  " when ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ T\)]],
  ". This represents a trivial form of bond, representing a cash payment of ",
  
  Cell[BoxData[
      \(TraditionalForm\`Z\)]],
  " at a time ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " in the future. The value of this instrument is just the discounted, or \
present, value of this cash payment.\n\nIn the case of the time-independent \
equation, there are in fact two ways of proceeding. The first is to note that \
each time we differentiate, we also put back a power of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " in the equation. So if we were to try a power of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " for the solution, there is a good chance that the equation could be \
solved quite quickly. Matters are much simpler, and just as revealing for \
future analysis, if we consider the case ",
  Cell[BoxData[
      \(TraditionalForm\`q\  = \ 0\)]],
  ". So we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", " ", 
          RowBox[{"r", " ", "S", " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If we try ",
  Cell[BoxData[
      \(TraditionalForm\`V\  = \ C\ S\^\[Alpha]\)]],
  ", we obtain the condition"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`1\/2\ \(\[Alpha](\[Alpha] - 1)\)\ \[Sigma]\^2 - r + r\ \[Alpha] = 0\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This has solutions ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha] = 1\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\  = \ \(-2\)\ r/\[Sigma]\^2\)]],
  ". The first case gives us ",
  Cell[BoxData[
      \(TraditionalForm\`C\ S\)]],
  ", which is a multiple of the underlying asset. The second case is a little \
more interesting, as it corresponds nicely to a situation that is commonplace \
in the case of the heat equation, i.e., a steady-state solution to which a \
more general time-dependent solution settles down at very late times. The \
meaning of this solution will only become clear in Chapter 11 - for now we \
just give a hint of what is to come. We let"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B\_\[Infinity] = K\/\(\[Sigma]\^2\/\(2\ r\) + 1\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["so that ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(\[Sigma]\^2\ B\_\[Infinity]\)\/\(2\ r\) = 
      \(K\/\(\(2\ r\)\/\[Sigma]\^2 + 1\) = K - B\_\[Infinity]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and define", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`P(S) = \((K - B\_\[Infinity])\)\ 
        \((B\_\[Infinity]\/S)\)\^\(\(2\ r\)\/\[Sigma]\^2\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is clearly a special case of our second solution. It is easy to check \
that when ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(S\  = \)\ \)\)]],
  Cell[BoxData[
      \(TraditionalForm\`B\_\[Infinity]\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(P(S)\  = \ K\  - B\_\[Infinity]\ \)\)]],
  ". Furthermore, at that same point, we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\(P \((S)\)\)\), \(\[PartialD]S\),
          MultilineFunction->None], "=", \(-1\)}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and clearly ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  " tends to zero as ",
  Cell[BoxData[
      \(TraditionalForm\`\(S\ \)\)]],
  "becomes large. This is in fact the steady-state solution to the problem of \
an \"American Put\" option, whose time-dependent form we shall explore in \
Chapter 11. The quantity ",
  Cell[BoxData[
      \(TraditionalForm\`B\_t\)]],
  " is the critical price separating the regions where early exercise is \
desirable and where it is not, and for very large times the solution settles \
down to the one we have given. The solution of the Black-Scholes equation \
joins smoothly onto the Put option payoff function ",
  Cell[BoxData[
      \(TraditionalForm\`K - S\)]],
  ", and this smooth join is characterized by the conditions on our \
function.We shall explore this more thoroughly in Chapter 11."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " View of Solution"
}], "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  "-independent problem can be solved in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " as follows:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(V[t]\  /. \(DSolve[D[V[t], \ t]\  - r\ V[t] == 0, \ V[t], \ 
          t]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(r\ t\)\ C[1]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Setting ",
  Cell[BoxData[
      \(TraditionalForm\`q\  = \ 0\)]],
  ", as before, we can get the time-independent solutions just as easily."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(V[S]\  /. \(DSolve[
          1/2\ S^2\ \[Sigma]^2\ D[V[S], \ {S, 2}]\  + r\ S\ \ D[V[S], \ S] - 
              r\ V[S] == 0, \ V[S], \ S]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(2\ r\)\/\[Sigma]\^2\)\)\ C[1] + S\ C[2]\)], "Output"]
}, Open  ]],

Cell["\<\
We note that the quadratic equation we derived can also be solved \
directly:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Alpha]\  /. 
      Solve[1\/2\ \[Alpha] \((\[Alpha] - 1)\)\ \[Sigma]\^2 - r + 
            r\ \[Alpha] == 0, \ \[Alpha]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1, \(-\(\(2\ r\)\/\[Sigma]\^2\)\)}\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Constant-Coefficient Picture", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The solution of first and second order equations with constant \
coefficients is familiar to many. Equations of the type\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "+", 
          RowBox[{\((\ r - q)\), " ", "S", " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "can be reduced to constant coefficient equations by an illuminating change \
of variable that we shall find exceedingly useful in more general situations. \
Let ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  " be any suitable base value for ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". It could be the strike of an option, for example. Set"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S = K\ \[ExponentialE]\^x\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and observe that for any function ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{"S", " ", 
          FractionBox[\(\[PartialD]\(f \((S)\)\)\), \(\[PartialD]S\),
            MultilineFunction->None]}], "=", 
        FractionBox[\(\[PartialD]\(f \((S)\)\)\), \(\[PartialD]x\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["So the time-independent Black-Scholes equation becomes", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", 
            RowBox[{"(", 
              RowBox[{
                FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
                  MultilineFunction->None], "-", 
                FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
                  MultilineFunction->None]}], ")"}]}], "+", 
          RowBox[{\((\ r - q)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["or ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\(1\/2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
              MultilineFunction->None], " ", \(\[Sigma]\^2\)}], "-", 
          \(r\ V\), "+", 
          RowBox[{\((r - q - \[Sigma]\^2\/2)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}]}], "=", "0"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which is a standard constant-coefficient form that can be solved by \
standard methods, where we look for solutions of the form ",
  Cell[BoxData[
      \(TraditionalForm\`A\ \[ExponentialE]\^\(\[Lambda]\ x\)\)]],
  ". This leads to exactly the same type of power solutions as before, once \
the re-substitution "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`x = log(S\/K)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "has been made. This can be done with \"pen and paper\", or directly with ",
  
  StyleBox["Mathematica. ",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(soln\  = \ 
      V[x]\  /. \(DSolve[
            1/2\ \[Sigma]^2\ D[
                    V[x], \ {x, 2}]\  + \((r - q - \[Sigma]^2/2)\)\ D[V[x], \ 
                    x] - r\ V[x] == 0, \ V[x], \ x]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(\(x\ \((2\ q - 2\ r + \[Sigma]\^2 - \@\(8\ r\ \[Sigma]\^2 + \
\((\(-2\)\ q + 2\ r - \[Sigma]\^2)\)\^2\))\)\)\/\(2\ \[Sigma]\^2\)\)\ C[1] + 
      E\^\(\(x\ \((2\ q - 2\ r + \[Sigma]\^2 + \@\(8\ r\ \[Sigma]\^2 + \
\((\(-2\)\ q + 2\ r - \[Sigma]\^2)\)\^2\))\)\)\/\(2\ \[Sigma]\^2\)\)\ C[
          2]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Let's change variables back to ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ", putting ",
  Cell[BoxData[
      \(TraditionalForm\`K = 1\)]],
  " for illustration:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(subsoln\  = \ Simplify[soln\  /. \ x\  -> \ Log[S]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(\(-2\)\ q + 2\ r + \@\(8\ r\ \[Sigma]\^2 + \((2\ q - 2\ r + \
\[Sigma]\^2)\)\^2\)\)\/\(2\ \[Sigma]\^2\)\)\)\ \((\@S\ C[1] + 
          S\^\(1\/2 + \@\(8\ r\ \[Sigma]\^2 + \((2\ q - 2\ r + \[Sigma]\^2)\)\
\^2\)\/\[Sigma]\^2\)\ C[2])\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "For general ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " this is rather complicated, so let's re-check the ",
  Cell[BoxData[
      \(TraditionalForm\`q\  = \ 0\)]],
  " solution, giving ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " a kick to do the necessary simplifications:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[ExpandAll[subsoln\  /. \ q\  -> \ 0]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(2\ r + \@\((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \
\[Sigma]\^2\)\)\)\ \((\@S\ C[1] + 
          S\^\(1\/2 + \@\((2\ r + \[Sigma]\^2)\)\^2\/\[Sigma]\^2\)\ C[
              2])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[PowerExpand[%]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(2\ r\)\/\[Sigma]\^2\)\)\ C[1] + S\ C[2]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We recover once again the combination of an asset and a steady-state \
American Put.\n\nAlthough in this particular case we have gained nothing new, \
the fact that we can eliminate the powers of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " from the equation by changing variables to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is a useful observation. In the next section we shall proceed to analyse \
the general case using this observation. First we wish to take a more \
detailed look at the steady-state solutions for non-zero ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  ". "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Steady-State Solutions for Non-zero q", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's take a closer look at the two steady-state solutions when ",
  Cell[BoxData[
      \(TraditionalForm\`q\  > \ 0\)]],
  ". This is of importance for FX options, when ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " is the interest rate associated with the foreign currency."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[ExpandAll[subsoln]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(\(-2\)\ q + 2\ r + \@\(4\ q\^2 + 4\ q\ \((\(-2\)\ r + \
\[Sigma]\^2)\) + \((2\ r + \[Sigma]\^2)\)\^2\)\)\/\(2\ \[Sigma]\^2\)\)\)\ \((\
\@S\ C[1] + 
          S\^\(1\/2 + \@\(4\ q\^2 + 4\ q\ \((\(-2\)\ r + \[Sigma]\^2)\) + \
\((2\ r + \[Sigma]\^2)\)\^2\)\/\[Sigma]\^2\)\ C[2])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Expand[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(1\/2 + q\/\[Sigma]\^2 - r\/\[Sigma]\^2 - \@\(4\ q\^2 + 4\ q\ \
\((\(-2\)\ r + \[Sigma]\^2)\) + \((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \
\[Sigma]\^2\)\)\ C[1] + 
      S\^\(1\/2 + q\/\[Sigma]\^2 - r\/\[Sigma]\^2 + \@\(4\ q\^2 + 4\ q\ \
\((\(-2\)\ r + \[Sigma]\^2)\) + \((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \
\[Sigma]\^2\)\)\ C[2]\)], "Output"]
}, Open  ]],

Cell["We separate them out into two cases:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(asoln\  = \ %\  /. \ {C[1]\  -> \ 1, \ C[2]\  -> \ 0}\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(1\/2 + q\/\[Sigma]\^2 - r\/\[Sigma]\^2 - \@\(4\ q\^2 + 4\ q\ \
\((\(-2\)\ r + \[Sigma]\^2)\) + \((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \
\[Sigma]\^2\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(bsoln\  = \ %%\  /. \ {C[2]\  -> \ 1, \ C[1]\  -> \ 0}\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(1\/2 + q\/\[Sigma]\^2 - r\/\[Sigma]\^2 + \@\(4\ q\^2 + 4\ q\ \
\((\(-2\)\ r + \[Sigma]\^2)\) + \((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \
\[Sigma]\^2\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
Let's remind ourselves which is which by looking once more at the \
zero-yield limit:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[Simplify[asoln\  /. \ {q\  -> \ 0}]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\^\(-\(\(2\ r\)\/\[Sigma]\^2\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[Simplify[bsoln\  /. \ {q\  -> \ 0}]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(S\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "So the solution ",
  StyleBox["bsoln",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is the one corresponding to the asset, while ",
  StyleBox["asoln",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " corresponds to the steady state, or \"perpetual\" Put. When ",
  Cell[BoxData[
      \(TraditionalForm\`q > 0\)]],
  " ",
  StyleBox["bsoln",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " acquires an interesting new interpretation. To simplify the analysis, \
let"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Lambda]\  = \ PowerExpand[Log[bsoln\  /. \ S\  -> \ Exp[1]]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2 + q\/\[Sigma]\^2 - 
      r\/\[Sigma]\^2 + \@\(4\ q\^2 + 4\ q\ \((\(-2\)\ r + \[Sigma]\^2)\) + \
\((2\ r + \[Sigma]\^2)\)\^2\)\/\(2\ \[Sigma]\^2\)\)], "Output"]
}, Open  ]],

Cell["We define the functions", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`b(K, \[Lambda]) = K\/\(1 - 1\/\[Lambda]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`PCall(S, K, \[Lambda]) := 
      \((b(K, \[Lambda]) - K)\)\ \((S\/\(b(K, \[Lambda])\))\)\^\[Lambda]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["We define them symbolically thus:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(b[K_, \ \[Lambda]_]\ \  := \ K/\((1\  - \ 1/\[Lambda])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(PCall[S_, \ 
        K_, \ \[Lambda]_]\  := \ \((b[K, \[Lambda]]\  - \ 
            K)\)*\((S/b[K, \[Lambda]])\)^\[Lambda]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The function ",
  StyleBox["PCall",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is the steady-state solution for an American Call, provided ",
  Cell[BoxData[
      \(TraditionalForm\`q\  > \ 0\)]],
  ". When ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ b\)]],
  ", we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PCall[b[K, \[Lambda]], \ 
        K, \ \[Lambda]] == \((b[K, \[Lambda]] - K)\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(True\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      D[PCall[S, K, \[Lambda]], \ S]\  /. \ S\  -> \ b[K, \[Lambda]]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Thus the function ",
  StyleBox["PCall",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " joins smoothly onto the surface given by ",
  Cell[BoxData[
      \(TraditionalForm\`S - K\)]],
  " at the point ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ b\)]],
  ". This is the \"perpetual Call\" solution of the Black-Scholes equation. \
Note that as ",
  Cell[BoxData[
      \(TraditionalForm\`q\  \[RightArrow] \ 0\)]],
  ", the value of ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " becomes infinite.  Both the perpetual Call and Put solutions are well \
known - see the discussion and bibliography of Chapter 11. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakBelow->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.3 The Black-Scholes Equation and the Diffusion Equation", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We return to the standard and more general form of our partial \
differential equation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "+", 
          RowBox[{"S", " ", \((r - q)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "+", 
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We have already done the work to establish that by setting, for some base \
variable ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S = K\ \[ExponentialE]\^x\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["the time-dependent Black-Scholes equation becomes", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "+", 
          RowBox[{\(1\/2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
              MultilineFunction->None], " ", \(\[Sigma]\^2\)}], "-", 
          \(r\ V\), "+", 
          RowBox[{\((r - q - \[Sigma]\^2\/2)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}]}], "=", "0"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Making one further re-arrangement, this becomes, following the notation of \
Wilmott ",
  StyleBox["et al",
    FontSlant->"Italic"],
  " (1993), but generalizing to allow for dividends expressed though ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
            MultilineFunction->None], "-", \(k\_1\ V\), "+", 
          RowBox[{\((k\_2 - 1)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}]}], "=", 
        RowBox[{\(-\(2\/\(\ \[Sigma]\^2\)\)\), 
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_1 = \(2\ r\)\/\[Sigma]\^2\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_2 = \(2\ \((r - q)\)\)\/\[Sigma]\^2\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The next step is to re-scale the time variable. Generally, we are \
interested in an instrument with an expiry or maturity at a time ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " in the future. Bearing this in mind, we set, assuming that the volatility \
is constant, "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Tau] = 1\/2\ \[Sigma]\^2\ \((T - t)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and now we have arrived at", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
            MultilineFunction->None], "-", \(k\_1\ V\), "+", 
          RowBox[{\((k\_2 - 1)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}]}], "=", 
        FractionBox[\(\[PartialD]V\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "How one proceeds next depends on whether we can regard ",
  Cell[BoxData[
      \(TraditionalForm\`k\_i\)]],
  " as constant. If we can, matters are rather trivial, for writing"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = 
      C\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \((k\_2 - 1)\)\ x - 
            \((1\/4\ \((k\_2 - 1)\)\^2 + k\_1)\)\ \[Tau]\)\ 
        \(u(x, \[Tau])\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["eliminates several of the remaining terms, so that", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\^2 u\), \(\[PartialD]x\^2\),
          MultilineFunction->None], "=", 
        FractionBox[\(\[PartialD]u\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Various other devices can be used when some or all of the \
parameters are time-dependent, leading to the same simple diffusion equation. \
There is a vast amount of standard applied mathematics we can bring to bear \
on the solution of this equation. In Section 4.4 we shall look at how to \
solve this equation. First it is convenient to explore some special cases \
that do not require the power of our general approach.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Special Solutions I - Logs", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A few interesting solutions of our PDE can be derived by special \
methods. Let's first go back to the point where we had this equation:\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]x\^2\),
            MultilineFunction->None], "-", \(k\_1\ V\), "+", 
          RowBox[{\((k\_2 - 1)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}]}], "=", 
        RowBox[{\(-\(2\/\(\ \[Sigma]\^2\)\)\), 
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Matters are simplified if there is no second derivative term:\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          RowBox[{\((k\_2 - 1)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]x\),
              MultilineFunction->None]}], "=", 
          RowBox[{
            RowBox[{
              RowBox[{\(-\(2\/\(\ \[Sigma]\^2\)\)\), 
                FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
                  MultilineFunction->None]}], "+", \(k\_1\ V\)}], " ", "=", 
            " ", 
            RowBox[{\(-\(2\/\(\ \[Sigma]\^2\)\)\), 
              RowBox[{"(", 
                RowBox[{
                  FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
                    MultilineFunction->None], "-", \(r\ V\)}], ")"}]}]}]}], 
        " "}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We introduce an integrating factor for the right side of this \
equation, and let \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(w(x, t)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{\((k\_2 - 1)\), " ", 
          FractionBox[\(\[PartialD]w\), \(\[PartialD]x\),
            MultilineFunction->None]}], "=", " ", 
        RowBox[{\(-\(2\/\(\ \[Sigma]\^2\)\)\), 
          FractionBox[\(\[PartialD]w\), \(\[PartialD]t\),
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Solutions of this can easily be obtained by assuming that both sides are a \
constant, and looking for a solution of the form ",
  Cell[BoxData[
      \(TraditionalForm\`w\  = \ A(x\  - \ \[Alpha]\ t)\)]],
  ". We find that"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Alpha] = r - q - \[Sigma]\^2\/2\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Re-assembling it all, we find that, by a suitable re-organization \
of constants,\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = A\ \((\((\(-\(\[Sigma]\^2\/2\)\) - q + r)\)\ \((T - t)\) + log(S/K))
          \)\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which has the property that when ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ T\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V\  = \ A\ \(log(S/K)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This is what is known as the \"log contract\" - it represents an \
instrument that pays the log of the asset price at maturity.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Special Solutions II - Separation of Variables and Power \
Solutions\
\>", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["We have our general solution in the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = C\ \[ExponentialE]\^\(\(-\(1\/2\)\)\ \((k\_2 - 1)\)\ x - 
            \((1\/4\ \((k\_2 - 1)\)\^2 + k\_1)\)\ \[Tau]\)\ 
        \(u(x, \[Tau])\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "with the diffusion condition on ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\^2 u\), \(\[PartialD]x\^2\),
          MultilineFunction->None], "=", 
        FractionBox[\(\[PartialD]u\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None]}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A standard technique is to suppose that ",
  Cell[BoxData[
      \(TraditionalForm\`u\  = \ \(f(x)\)\ \(g(\[Tau])\)\)]],
  ". When we substitute into the diffusion equation, we find that, for some \
constant ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " (not necessarily real), "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\(g \((\[Tau])\)\)\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None], "=", \(\[Mu]\^2\ g\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\^2\( f \((x)\)\)\), \(\[PartialD]x\^2\),
          MultilineFunction->None], "=", \(\[Mu]\^2\ f\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[TextData[{
  " Case 1: ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\^2 > 0\)]]
}], "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This yields a family of solutions of the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u = Z\ \[ExponentialE]\^\(\[Mu]\ x\  + \ \(\[Mu]\^2\) \[Tau]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "By writing ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\  = \ 1/2 \((k\_2 - 1)\)\  + \ \[Lambda]\)]],
  ", and doing some reorganization, we obtain the general \"power contract\""
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = Z\ \((S\/K)\)\^\[Lambda]\ 
        \[ExponentialE]\^\(\((
              \((\(-\(\[Sigma]\^2\/2\)\) - q + r)\)\ \[Lambda] + 
                \(\[Lambda]\^2\ \[Sigma]\^2\)\/2 - r)\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This contains as special cases some examples we have seen already. \
When \[Lambda] = 0 we obtain the solution\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V = Z\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "representing a cash payment at maturity. When ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = \ 1\)]],
  " we obtain a slight generalization (yield-adjusted) of the pure asset, in \
the form"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = Z\ S\/K\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "with maturity value ",
  Cell[BoxData[
      \(TraditionalForm\`Z\ S/K\)]],
  ". The steady-state American Put solution is obtained when ",
  Cell[BoxData[
      \(TraditionalForm\`q\  = \ 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = \ \(-2\) r/\[Sigma]\^2\)]],
  ". More generally, when ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ T\)]],
  " we have the general power payoff"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V = Z\ \((S\/K)\)\^\[Lambda]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " Case 2: ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\^2 = 0\)]]
}], "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In this case we get ",
  Cell[BoxData[
      \(TraditionalForm\`u\  = \ a\ x\  + \ b\)]],
  ". Some re-organization leads to the special log-power solution:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = 
      \((A\ \(log(S\/K)\) + B)\)\ 
        \((S\/K)\)\^\(1\/2 - \(r - q\)\/\[Sigma]\^2\)\ 
        \[ExponentialE]\^\(\(-\((T - t)\)\)\ 
            \((1\/2\ \((\[Sigma]\/2 - \(r - q\)\/\[Sigma])\)\^2 + r)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["with payoff", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, T) = 
      \((A\ \(log(S\/K)\) + B)\)\ 
        \((S\/K)\)\^\(1\/2 - \(r - q\)\/\[Sigma]\^2\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  " Case 3: ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\^2 < 0\)]]
}], "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Writing ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu] = \[ImaginaryI]\ \[Lambda]\)]],
  " gets us to"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u = \((A\ \(sin(\[Lambda]\ x)\) + B\ \(cos(\[Lambda]\ x)\))\)\ 
        \[ExponentialE]\^\(\(-\[Lambda]\^2\)\ \[Tau]\)\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
These Fourier-mode solutions are more important for their role in \
constructing general solutions of the diffusion equation given an initial \
(payoff) condition - we shall not consider them in isolation.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Special Solutions III - Similarity Methods", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We have already seen how similarity solutions can be used to solve the \
coupled asset-FX problem. They are exceedingly useful here also. The \
diffusion equation is invariant under the re-scalings ",
  Cell[BoxData[
      \(TraditionalForm\`x\  \[RightArrow] \ \[Alpha]\ x\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\(\ \[Tau] \[RightArrow] \[Alpha]\^2\ \[Tau]\)\)]],
  ". To capture this invariance, we change variables to"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`z = x\/\@\[Tau]\ , \ \ \ \ \ y = \@\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["with inverse", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`x = y\ z\ , \ \ \ \ \ \[Tau] = y\^2\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This change of variables, and the chain rule, lead to the new PDE\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          
          FractionBox[\(\[PartialD]\^2\( u \((z, y)\)\)\), 
            \(\[PartialD]z\^2\),
            MultilineFunction->None], "+", 
          RowBox[{\(z\/2\), " ", 
            FractionBox[\(\[PartialD]\(u \((z, y)\)\)\), \(\[PartialD]z\),
              MultilineFunction->None]}]}], "=", 
        RowBox[{\(1\/2\), " ", "y", " ", 
          FractionBox[\(\[PartialD]\(u \((z, y)\)\)\), \(\[PartialD]y\),
            MultilineFunction->None]}]}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If we consider solutions to this PDE of the form ",
  Cell[BoxData[
      \(TraditionalForm\`u\  = \ y\^n\ \(h(z)\)\)]],
  ", we find that ",
  Cell[BoxData[
      \(TraditionalForm\`h\)]],
  " satisfies the ordinary differential equation"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\^2 h\), \(\[PartialD]z\^2\),
            MultilineFunction->None], "+", 
          RowBox[{\(1\/2\), " ", "z", " ", 
            FractionBox[\(\[PartialD]h\), \(\[PartialD]z\),
              MultilineFunction->None]}], "-", \(1\/2\ n\ h\)}], " ", "=", 
        "0"}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For general values of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " this is a slightly awkward equation - here is ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s view of the solutions:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(h[z]\  /. \ \(DSolve[
          D[h[z], \ {z, \ 2}]\  + \ z/2\ D[h[z], \ z]\  - \ n/2\ h[z]\  == 
            0, \ h[z], \ z]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\@z\^2\ C[2]\ Hypergeometric1F1[1\/2 - n\/2, 3\/2, \(-\(z\^2\/4\)\)] + 
      C[1]\ Hypergeometric1F1[\(-\(n\/2\)\), 1\/2, \(-\(z\^2\/4\)\)]\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "The solutions are much more straightforward if ",
  Cell[BoxData[
      \(TraditionalForm\`n = 0\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(h[z]\  /. \ \(DSolve[
          D[h[z], \ {z, \ 2}]\  + \ z/2\ D[h[z], \ z]\  == 0, \ h[z], \ 
          z]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(C[2] + \@\[Pi]\ C[1]\ Erf[z\/2]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "or if ",
  Cell[BoxData[
      \(TraditionalForm\`n = \(-1\)\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(h[z]\  /. \ \(DSolve[
          D[h[z], \ {z, \ 2}]\  + \ z/2\ D[h[z], \ z]\  + 1/2\ h[z]\  == 0, \ 
          h[z], \ z]\)[\([1]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(z\^2\/4\)\)\ C[1] + 
      E\^\(-\(z\^2\/4\)\)\ C[2]\ Erfi[\@z\^2\/2]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "This last case is worth a special look. If ",
  Cell[BoxData[
      \(TraditionalForm\`n = \(-1\)\)]],
  " the equation for ",
  Cell[BoxData[
      \(TraditionalForm\`h\)]],
  " can be written as"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\^2 h\), \(\[PartialD]z\^2\),
            MultilineFunction->None], "+", 
          RowBox[{\(1\/2\), 
            FractionBox[\(\[PartialD]\((z\ h)\)\), \(\[PartialD]z\),
              MultilineFunction->None]}]}], " ", "=", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["which integrates once to", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]\(h \((z)\)\)\), \(\[PartialD]z\),
            MultilineFunction->None], "+", \(\(1\/2\) z\ h\)}], "=", "C"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Introducing a suitable integrating factor, we obtain", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`D[\[ExponentialE]\^\(z\^2\/4\)\ \(h(z)\)] = 
      C\ \[ExponentialE]\^\(z\^2\/4\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If we now set ",
  Cell[BoxData[
      \(TraditionalForm\`C\  = \ 0\)]],
  " (which will in fact remove the ",
  StyleBox["Erfi",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " solution) we end up with "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`h(z) = A\ \[ExponentialE]\^\(-\(z\^2\/4\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This generates the following solution of the diffusion \
equation:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`G = \(A\ \[ExponentialE]\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\)\/\@\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It is convenient to normalize this so that the integral of ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " is unity. We can get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to do the work to find ",
  Cell[BoxData[
      \(TraditionalForm\`A\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      A/Sqrt[\[Tau]]\ Exp[\(-x^2\)/\((4\ \[Tau])\)], \ {x, \ \(-Infinity\), \ 
        Infinity}]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"If", "[", 
      RowBox[{\(Re[\[Tau]] > 0\), ",", \(2\ A\ \@\[Pi]\), ",", 
        RowBox[{
          SubsuperscriptBox["\[Integral]", 
            InterpretationBox[\(-\[Infinity]\),
              DirectedInfinity[ -1]], 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 
              1]]], \(\(\(A\ E\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\)\/\@\[Tau]\) \
\[DifferentialD]x\)}]}], "]"}]], "Output"]
}, Open  ]],

Cell["So the normalized solution is", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`G(x, \[Tau]) = 
      \[ExponentialE]\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\/\@\(4\ \[Pi]\ 
            \[Tau]\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let us take a look at this function for various values of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(G[x_, \ \[Tau]_]\  := \ 
      1/Sqrt[4\ Pi\ \[Tau]]\ Exp[\(-x^2\)/\((4\ \[Tau])\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
      Evaluate[Table[
          G[x, \ \[Tau]], \ {\[Tau], \ 0.01, \ 0.5, \ 
            0.05}]], \ {x, \ \(-2\), \ 2}, \ PlotRange\  -> \ All]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.5 0.238095 0.0147151 0.208655 [
[.02381 .00222 -6 -9 ]
[.02381 .00222 6 0 ]
[.2619 .00222 -6 -9 ]
[.2619 .00222 6 0 ]
[.7381 .00222 -3 -9 ]
[.7381 .00222 3 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.4875 .11904 -18 -4.5 ]
[.4875 .11904 0 4.5 ]
[.4875 .22337 -6 -4.5 ]
[.4875 .22337 0 4.5 ]
[.4875 .3277 -18 -4.5 ]
[.4875 .3277 0 4.5 ]
[.4875 .43203 -6 -4.5 ]
[.4875 .43203 0 4.5 ]
[.4875 .53635 -18 -4.5 ]
[.4875 .53635 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(-2)] .02381 .00222 0 1 Mshowa
.2619 .01472 m
.2619 .02097 L
s
[(-1)] .2619 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(1)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.5 .11904 m
.50625 .11904 L
s
[(0.5)] .4875 .11904 1 0 Mshowa
.5 .22337 m
.50625 .22337 L
s
[(1)] .4875 .22337 1 0 Mshowa
.5 .3277 m
.50625 .3277 L
s
[(1.5)] .4875 .3277 1 0 Mshowa
.5 .43203 m
.50625 .43203 L
s
[(2)] .4875 .43203 1 0 Mshowa
.5 .53635 m
.50625 .53635 L
s
[(2.5)] .4875 .53635 1 0 Mshowa
.125 Mabswid
.5 .03558 m
.50375 .03558 L
s
.5 .05645 m
.50375 .05645 L
s
.5 .07731 m
.50375 .07731 L
s
.5 .09818 m
.50375 .09818 L
s
.5 .13991 m
.50375 .13991 L
s
.5 .16077 m
.50375 .16077 L
s
.5 .18164 m
.50375 .18164 L
s
.5 .2025 m
.50375 .2025 L
s
.5 .24424 m
.50375 .24424 L
s
.5 .2651 m
.50375 .2651 L
s
.5 .28597 m
.50375 .28597 L
s
.5 .30683 m
.50375 .30683 L
s
.5 .34856 m
.50375 .34856 L
s
.5 .36943 m
.50375 .36943 L
s
.5 .39029 m
.50375 .39029 L
s
.5 .41116 m
.50375 .41116 L
s
.5 .45289 m
.50375 .45289 L
s
.5 .47376 m
.50375 .47376 L
s
.5 .49462 m
.50375 .49462 L
s
.5 .51549 m
.50375 .51549 L
s
.5 .55722 m
.50375 .55722 L
s
.5 .57808 m
.50375 .57808 L
s
.5 .59895 m
.50375 .59895 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01472 L
.02729 .01472 L
.02846 .01472 L
.02954 .01472 L
.03053 .01472 L
.03163 .01472 L
.03279 .01472 L
.03395 .01472 L
.0352 .01472 L
.03638 .01472 L
.03746 .01472 L
.03873 .01472 L
.03944 .01472 L
.04011 .01472 L
.04132 .01472 L
.04262 .01472 L
.04327 .01472 L
.04396 .01472 L
.04519 .01472 L
.04588 .01472 L
.04652 .01472 L
.04722 .01472 L
.04797 .01472 L
.04862 .01472 L
.04933 .01472 L
.05061 .01472 L
.05186 .01472 L
.05302 .01472 L
.05414 .01472 L
.05519 .01472 L
.05634 .01472 L
.05759 .01472 L
.05882 .01472 L
.06012 .01472 L
.06123 .01472 L
.06244 .01472 L
.06315 .01472 L
.06381 .01472 L
.06453 .01472 L
.06529 .01472 L
.06657 .01472 L
.06729 .01472 L
.06797 .01472 L
.06924 .01472 L
.0704 .01472 L
.07103 .01472 L
.07173 .01472 L
.073 .01472 L
Mistroke
.07376 .01472 L
.07448 .01472 L
.07513 .01472 L
.07583 .01472 L
.0766 .01472 L
.0774 .01472 L
.07809 .01472 L
.07885 .01472 L
.07956 .01472 L
.0802 .01472 L
.08095 .01472 L
.08166 .01472 L
.08291 .01472 L
.08362 .01472 L
.08427 .01472 L
.08544 .01472 L
.08669 .01472 L
.08787 .01472 L
.08896 .01472 L
.09016 .01472 L
.09148 .01472 L
.09214 .01472 L
.09285 .01472 L
.09413 .01472 L
.09483 .01472 L
.09548 .01472 L
.09619 .01472 L
.09695 .01472 L
.09821 .01472 L
.09893 .01472 L
.09961 .01472 L
.10086 .01472 L
.10201 .01472 L
.10333 .01472 L
.10458 .01472 L
.10572 .01472 L
.10694 .01472 L
.1081 .01472 L
.10916 .01472 L
.11033 .01472 L
.11162 .01472 L
.11227 .01472 L
.11297 .01472 L
.11422 .01472 L
.11491 .01472 L
.11554 .01472 L
.11624 .01472 L
.11698 .01472 L
.11822 .01472 L
.11892 .01472 L
Mistroke
.11958 .01472 L
.12081 .01472 L
.12194 .01472 L
.12323 .01472 L
.12446 .01472 L
.12562 .01472 L
.12668 .01472 L
.12789 .01472 L
.12903 .01472 L
.13009 .01472 L
.13107 .01472 L
.13214 .01472 L
.13327 .01472 L
.13441 .01472 L
.13564 .01472 L
.1368 .01472 L
.13785 .01472 L
.13909 .01472 L
.13979 .01472 L
.14043 .01472 L
.14162 .01472 L
.14288 .01472 L
.14402 .01472 L
.14526 .01472 L
.14656 .01472 L
.14777 .01472 L
.14891 .01472 L
.14994 .01472 L
.15113 .01472 L
.15226 .01472 L
.15348 .01472 L
.15478 .01472 L
.15589 .01472 L
.15711 .01472 L
.15842 .01472 L
.15915 .01472 L
.15982 .01472 L
.16113 .01472 L
.16236 .01472 L
.16352 .01472 L
.16456 .01472 L
.16577 .01472 L
.1669 .01472 L
.16796 .01472 L
.16893 .01472 L
.16999 .01472 L
.17111 .01472 L
.17225 .01472 L
.17347 .01472 L
.17462 .01472 L
Mistroke
.17566 .01472 L
.17689 .01472 L
.17758 .01472 L
.17823 .01472 L
.1794 .01472 L
.18065 .01472 L
.18185 .01472 L
.18316 .01472 L
.18441 .01472 L
.18556 .01472 L
.1867 .01472 L
.18773 .01472 L
.18893 .01472 L
.19005 .01472 L
.19128 .01472 L
.19258 .01472 L
.19369 .01472 L
.19491 .01472 L
.19623 .01472 L
.19696 .01472 L
.19763 .01472 L
.19894 .01472 L
.20017 .01472 L
.20122 .01472 L
.20238 .01472 L
.20359 .01472 L
.20472 .01472 L
.20578 .01472 L
.20675 .01472 L
.20781 .01472 L
.20894 .01472 L
.21008 .01472 L
.21129 .01472 L
.21244 .01472 L
.21349 .01472 L
.21472 .01472 L
.21605 .01472 L
.21731 .01472 L
.21847 .01472 L
.21972 .01472 L
.22091 .01472 L
.22198 .01472 L
.22314 .01472 L
.2244 .01472 L
.22574 .01472 L
.22701 .01472 L
.22816 .01472 L
.22933 .01472 L
.2304 .01472 L
.23158 .01472 L
Mistroke
.23283 .01472 L
.23391 .01472 L
.23493 .01472 L
.23604 .01472 L
.23722 .01472 L
.23861 .01472 L
.23988 .01472 L
.24129 .01472 L
.24277 .01472 L
.24406 .01472 L
.24527 .01472 L
.2466 .01472 L
.24802 .01472 L
.24922 .01472 L
.25053 .01472 L
.25189 .01472 L
.25316 .01472 L
.25435 .01472 L
.25543 .01472 L
.25668 .01472 L
.25786 .01472 L
.25902 .01472 L
.26026 .01472 L
.26143 .01472 L
.2625 .01472 L
.26376 .01472 L
.26512 .01472 L
.26641 .01472 L
.2676 .01472 L
.26882 .01472 L
.26997 .01472 L
.27123 .01472 L
.2726 .01472 L
.27394 .01472 L
.27536 .01472 L
.27658 .01472 L
.27791 .01472 L
.27915 .01472 L
.28032 .01472 L
.28252 .01472 L
.28376 .01472 L
.28489 .01472 L
.28622 .01472 L
.28746 .01472 L
.28952 .01472 L
.29061 .01472 L
.29179 .01472 L
.29301 .01472 L
.29418 .01472 L
.29637 .01472 L
Mistroke
.29761 .01472 L
.29896 .01472 L
.30014 .01472 L
.3014 .01472 L
.30253 .01472 L
.30377 .01472 L
.30596 .01472 L
.30729 .01472 L
.30851 .01472 L
.30982 .01472 L
.31124 .01472 L
.31256 .01472 L
.31378 .01472 L
.31493 .01472 L
.31618 .01472 L
.31837 .01472 L
.32042 .01472 L
.32161 .01472 L
.32286 .01472 L
.3251 .01472 L
.32744 .01472 L
.32962 .01472 L
.33201 .01472 L
.33335 .01472 L
.33457 .01472 L
.33688 .01472 L
.33942 .01472 L
.3407 .01472 L
.34207 .01472 L
.34455 .01473 L
.34702 .01473 L
.34936 .01474 L
.35147 .01475 L
.35376 .01476 L
.3565 .01478 L
.35899 .01481 L
.36185 .01485 L
.36455 .0149 L
.36706 .01496 L
.36936 .01503 L
.37189 .01514 L
.37456 .01529 L
.37711 .01547 L
.37953 .01569 L
.38406 .01628 L
.3866 .01674 L
.38894 .01727 L
.39168 .01805 L
.39424 .01896 L
.39848 .02096 L
Mistroke
.40072 .02234 L
.40314 .02412 L
.40807 .02889 L
.41259 .03497 L
.41735 .04365 L
.42242 .05613 L
.42721 .07161 L
.43156 .0893 L
.44222 .14974 L
.45227 .23027 L
.47063 .41711 L
.47566 .46802 L
.48106 .51719 L
.48572 .55272 L
.48818 .56817 L
.4908 .58175 L
.49212 .58741 L
.49358 .5927 L
.49492 .59665 L
.49615 .59948 L
.4974 .60157 L
.49806 .60235 L
.49876 .60292 L
.49993 .60332 L
.50122 .60294 L
.50246 .60175 L
.50359 .59998 L
.5049 .59711 L
.50614 .59362 L
.50834 .58556 L
.51072 .57424 L
.51604 .5402 L
.52167 .4932 L
.53178 .39178 L
.54192 .28589 L
.55156 .19699 L
.56021 .13373 L
.56964 .08405 L
.57452 .06556 L
.57972 .05041 L
.58462 .03973 L
.58907 .03251 L
.59424 .02643 L
.59715 .02388 L
.59983 .02198 L
.60475 .01937 L
.60723 .01841 L
.60996 .01756 L
.61247 .01694 L
.6152 .01641 L
Mistroke
.61779 .01601 L
.62017 .01572 L
.62469 .01533 L
.62697 .0152 L
.62949 .01508 L
.6322 .01498 L
.63468 .01491 L
.63741 .01486 L
.64028 .01482 L
.64279 .01479 L
.64512 .01477 L
.64768 .01475 L
.65041 .01474 L
.653 .01473 L
.65575 .01473 L
.65809 .01472 L
.66066 .01472 L
.66211 .01472 L
.66343 .01472 L
.66486 .01472 L
.66641 .01472 L
.66784 .01472 L
.66916 .01472 L
.67177 .01472 L
.67305 .01472 L
.67442 .01472 L
.67563 .01472 L
.6769 .01472 L
.67806 .01472 L
.67932 .01472 L
.68154 .01472 L
.6829 .01472 L
.68414 .01472 L
.68547 .01472 L
.68692 .01472 L
.68826 .01472 L
.6895 .01472 L
.69075 .01472 L
.69193 .01472 L
.6931 .01472 L
.69436 .01472 L
.69659 .01472 L
.69785 .01472 L
.69919 .01472 L
.7016 .01472 L
.70277 .01472 L
.70404 .01472 L
.70516 .01472 L
.70635 .01472 L
.70742 .01472 L
Mistroke
.70859 .01472 L
.71065 .01472 L
.71204 .01472 L
.71335 .01472 L
.71454 .01472 L
.7158 .01472 L
.71717 .01472 L
.71844 .01472 L
.71981 .01472 L
.72132 .01472 L
.72261 .01472 L
.72379 .01472 L
.72509 .01472 L
.72646 .01472 L
.72765 .01472 L
.72876 .01472 L
.72996 .01472 L
.73128 .01472 L
.73255 .01472 L
.7339 .01472 L
.73516 .01472 L
.73633 .01472 L
.7377 .01472 L
.73896 .01472 L
.74042 .01472 L
.7418 .01472 L
.74299 .01472 L
.74426 .01472 L
.74546 .01472 L
.74655 .01472 L
.74783 .01472 L
.74922 .01472 L
.75053 .01472 L
.75173 .01472 L
.75281 .01472 L
.75396 .01472 L
.75496 .01472 L
.75605 .01472 L
.75732 .01472 L
.75852 .01472 L
.75969 .01472 L
.76078 .01472 L
.76191 .01472 L
.76298 .01472 L
.76415 .01472 L
.76541 .01472 L
.76666 .01472 L
.76797 .01472 L
.76909 .01472 L
.77032 .01472 L
Mistroke
.77164 .01472 L
.77286 .01472 L
.7742 .01472 L
.77562 .01472 L
.77685 .01472 L
.778 .01472 L
.77926 .01472 L
.78062 .01472 L
.78182 .01472 L
.78313 .01472 L
.78436 .01472 L
.78551 .01472 L
.78653 .01472 L
.78767 .01472 L
.78885 .01472 L
.78996 .01472 L
.79119 .01472 L
.79252 .01472 L
.79392 .01472 L
.79522 .01472 L
.79644 .01472 L
.79755 .01472 L
.79884 .01472 L
.80005 .01472 L
.80137 .01472 L
.80203 .01472 L
.80276 .01472 L
.80405 .01472 L
.80527 .01472 L
.806 .01472 L
.80669 .01472 L
.80799 .01472 L
.80872 .01472 L
.8094 .01472 L
.81013 .01472 L
.81093 .01472 L
.81221 .01472 L
.81292 .01472 L
.81357 .01472 L
.81475 .01472 L
.81601 .01472 L
.81672 .01472 L
.81749 .01472 L
.81816 .01472 L
.81888 .01472 L
.81953 .01472 L
.82024 .01472 L
.8215 .01472 L
.82279 .01472 L
.82398 .01472 L
Mistroke
.82527 .01472 L
.82593 .01472 L
.82664 .01472 L
.82785 .01472 L
.82914 .01472 L
.83036 .01472 L
.83146 .01472 L
.83262 .01472 L
.83385 .01472 L
.8349 .01472 L
.83606 .01472 L
.8373 .01472 L
.83799 .01472 L
.83863 .01472 L
.83986 .01472 L
.84104 .01472 L
.84212 .01472 L
.84328 .01472 L
.84437 .01472 L
.84536 .01472 L
.84653 .01472 L
.84779 .01472 L
.84898 .01472 L
.85007 .01472 L
.85134 .01472 L
.85254 .01472 L
.85362 .01472 L
.85479 .01472 L
.85607 .01472 L
.85671 .01472 L
.85742 .01472 L
.85867 .01472 L
.85984 .01472 L
.86102 .01472 L
.86213 .01472 L
.86313 .01472 L
.86423 .01472 L
.86541 .01472 L
.86667 .01472 L
.86787 .01472 L
.86895 .01472 L
.87024 .01472 L
.87089 .01472 L
.87161 .01472 L
.87288 .01472 L
.87406 .01472 L
.87478 .01472 L
.87546 .01472 L
.87673 .01472 L
.87744 .01472 L
Mistroke
.8781 .01472 L
.87882 .01472 L
.87959 .01472 L
.88079 .01472 L
.8819 .01472 L
.88255 .01472 L
.88324 .01472 L
.88446 .01472 L
.88574 .01472 L
.88693 .01472 L
.88823 .01472 L
.88896 .01472 L
.88963 .01472 L
.89082 .01472 L
.89193 .01472 L
.89316 .01472 L
.89447 .01472 L
.89513 .01472 L
.89585 .01472 L
.89715 .01472 L
.89842 .01472 L
.8996 .01472 L
.9008 .01472 L
.90194 .01472 L
.90296 .01472 L
.90406 .01472 L
.90526 .01472 L
.90653 .01472 L
.90761 .01472 L
.9088 .01472 L
.90947 .01472 L
.9102 .01472 L
.91152 .01472 L
.9128 .01472 L
.91399 .01472 L
.91528 .01472 L
.91601 .01472 L
.91669 .01472 L
.91741 .01472 L
.91807 .01472 L
.9188 .01472 L
.91956 .01472 L
.92076 .01472 L
.92204 .01472 L
.92325 .01472 L
.92435 .01472 L
.92565 .01472 L
.92638 .01472 L
.92705 .01472 L
.92829 .01472 L
Mistroke
.92959 .01472 L
.93081 .01472 L
.93213 .01472 L
.93338 .01472 L
.93453 .01472 L
.93557 .01472 L
.93672 .01472 L
.93792 .01472 L
.93905 .01472 L
.94028 .01472 L
.94159 .01472 L
.94271 .01472 L
.94393 .01472 L
.94524 .01472 L
.94598 .01472 L
.94665 .01472 L
.94796 .01472 L
.94919 .01472 L
.95025 .01472 L
.95141 .01472 L
.95262 .01472 L
.95376 .01472 L
.95482 .01472 L
.95579 .01472 L
.95691 .01472 L
.95798 .01472 L
.95912 .01472 L
.96034 .01472 L
.96149 .01472 L
.96254 .01472 L
.96377 .01472 L
.96446 .01472 L
.9651 .01472 L
.96627 .01472 L
.96751 .01472 L
.96865 .01472 L
.96968 .01472 L
.97087 .01472 L
.972 .01472 L
.97304 .01472 L
.97401 .01472 L
.97507 .01472 L
.97619 .01472 L
Mfstroke
.02381 .01472 m
.02846 .01472 L
.03279 .01472 L
.03754 .01472 L
.04262 .01472 L
.04795 .01472 L
.053 .01472 L
.05753 .01472 L
.06244 .01472 L
.06751 .01472 L
.07308 .01472 L
.0789 .01472 L
.08435 .01472 L
.08944 .01472 L
.0941 .01472 L
.09947 .01472 L
.10458 .01472 L
.10961 .01472 L
.11417 .01472 L
.11943 .01472 L
.12439 .01472 L
.12899 .01472 L
.13322 .01473 L
.13786 .01473 L
.14277 .01474 L
.14773 .01474 L
.15225 .01475 L
.15747 .01476 L
.16241 .01477 L
.17119 .0148 L
.17615 .01482 L
.18074 .01485 L
.18538 .01488 L
.19037 .01492 L
.19941 .01503 L
.20464 .01511 L
.21016 .01522 L
.21532 .01534 L
.22001 .01547 L
.22956 .01583 L
.2348 .01608 L
.23975 .01637 L
.24857 .01702 L
.25812 .01797 L
.26758 .01925 L
.278 .02113 L
.28887 .02379 L
.29904 .02706 L
.30945 .03138 L
.31933 .03653 L
Mistroke
.33786 .04952 L
.35771 .06897 L
.37611 .09248 L
.41529 .15652 L
.4353 .19137 L
.44582 .20838 L
.45692 .22438 L
.4669 .23642 L
.47218 .24172 L
.47783 .24649 L
.48303 .24998 L
.48777 .25239 L
.49006 .25327 L
.49254 .25403 L
.49367 .25431 L
.49488 .25455 L
.496 .25473 L
.49704 .25486 L
.49818 .25495 L
.49943 .25501 L
.50065 .255 L
.50195 .25495 L
.50316 .25484 L
.5043 .25469 L
.50533 .25451 L
.50644 .25428 L
.509 .25359 L
.5117 .2526 L
.51655 .25022 L
.52106 .24731 L
.52527 .24399 L
.53484 .2345 L
.55517 .20684 L
.57372 .17588 L
.61353 .10789 L
.63173 .08183 L
.65183 .05886 L
.67158 .04232 L
.68241 .03554 L
.69258 .03045 L
.7024 .02655 L
.71274 .02335 L
.72157 .02123 L
.73124 .01944 L
.74149 .01802 L
.75082 .01707 L
.75599 .01666 L
.76159 .01629 L
.76685 .016 L
.77177 .01577 L
Mistroke
.78121 .01543 L
.78623 .0153 L
.79162 .01518 L
.79725 .01508 L
.8026 .015 L
.80788 .01494 L
.81265 .0149 L
.82225 .01483 L
.82751 .01481 L
.83247 .01479 L
.84131 .01476 L
.84629 .01475 L
.85086 .01474 L
.85557 .01474 L
.86061 .01473 L
.86497 .01473 L
.8697 .01473 L
.87526 .01472 L
.88052 .01472 L
.88564 .01472 L
.89037 .01472 L
.89501 .01472 L
.90001 .01472 L
.90473 .01472 L
.90905 .01472 L
.91385 .01472 L
.91909 .01472 L
.92457 .01472 L
.92966 .01472 L
.93471 .01472 L
.94019 .01472 L
.94536 .01472 L
.95013 .01472 L
.95443 .01472 L
.95916 .01472 L
.9641 .01472 L
.96872 .01472 L
.97263 .01472 L
.97619 .01472 L
Mfstroke
.02381 .01474 m
.03279 .01474 L
.04262 .01476 L
.05288 .01477 L
.06244 .0148 L
.0731 .01483 L
.0828 .01488 L
.094 .01495 L
.10458 .01505 L
.11428 .01517 L
.12455 .01534 L
.13333 .01552 L
.14299 .01579 L
.15313 .01614 L
.1624 .01655 L
.17311 .01716 L
.18328 .0179 L
.19287 .01876 L
.20309 .01989 L
.22147 .02263 L
.23176 .02463 L
.24289 .02725 L
.26306 .03341 L
.283 .04158 L
.30406 .05279 L
.34203 .07998 L
.38246 .11671 L
.42136 .15322 L
.44118 .1692 L
.45162 .17629 L
.46273 .18257 L
.47256 .18691 L
.47816 .18883 L
.48342 .19024 L
.48576 .19075 L
.48823 .1912 L
.49036 .19153 L
.49264 .1918 L
.49392 .19192 L
.49509 .19202 L
.49627 .19209 L
.49738 .19214 L
.49868 .19217 L
.49987 .19219 L
.50118 .19218 L
.50184 .19216 L
.50257 .19214 L
.50386 .19208 L
.50506 .192 L
.50638 .1919 L
Mistroke
.50778 .19176 L
.51038 .19142 L
.51273 .19104 L
.51752 .19002 L
.52202 .18877 L
.53206 .18502 L
.54291 .17956 L
.56289 .16616 L
.58397 .14849 L
.62199 .11244 L
.66247 .07631 L
.70143 .04961 L
.72271 .03901 L
.74284 .0314 L
.76279 .02585 L
.78122 .02216 L
.79087 .02069 L
.79992 .01953 L
.80997 .01848 L
.82054 .0176 L
.83021 .01696 L
.84076 .0164 L
.85036 .01601 L
.85928 .01572 L
.86983 .01545 L
.87978 .01526 L
.88871 .01513 L
.89838 .01502 L
.90787 .01494 L
.91829 .01487 L
.92918 .01483 L
.93935 .01479 L
.94924 .01477 L
.95861 .01475 L
.96703 .01474 L
.97619 .01474 L
Mfstroke
.02381 .015 m
.03279 .01507 L
.04262 .01518 L
.06244 .01547 L
.07298 .01568 L
.08426 .01597 L
.10458 .01669 L
.1148 .01718 L
.12589 .01782 L
.14603 .01937 L
.16586 .0215 L
.18689 .02458 L
.20643 .0284 L
.22473 .03294 L
.2635 .04621 L
.30472 .06616 L
.34443 .09024 L
.38262 .11537 L
.40394 .12882 L
.42327 .13982 L
.44365 .14954 L
.4624 .15624 L
.47176 .15867 L
.47653 .15965 L
.48165 .16051 L
.48433 .16087 L
.48678 .16116 L
.48927 .1614 L
.49157 .16158 L
.49425 .16173 L
.49566 .16179 L
.49639 .16181 L
.49717 .16183 L
.49847 .16186 L
.49969 .16187 L
.50039 .16187 L
.50114 .16186 L
.50177 .16185 L
.50247 .16184 L
.50379 .16181 L
.505 .16176 L
.50634 .1617 L
.50775 .16162 L
.51013 .16145 L
.51274 .16121 L
.51756 .16062 L
.52208 .1599 L
.53218 .15773 L
.54303 .15455 L
.5642 .14606 L
Mistroke
.5843 .13569 L
.62255 .11199 L
.66325 .08531 L
.70243 .06228 L
.72244 .05234 L
.74406 .04321 L
.7632 .03652 L
.78418 .0306 L
.80416 .02621 L
.82524 .02269 L
.84479 .02027 L
.86571 .0184 L
.87591 .01771 L
.88699 .01709 L
.89657 .01664 L
.90713 .01624 L
.92694 .01568 L
.93779 .01546 L
.94796 .0153 L
.9627 .01512 L
.97619 .015 L
Mfstroke
.02381 .01581 m
.04262 .0163 L
.06244 .01702 L
.08255 .01802 L
.10458 .01953 L
.12415 .02133 L
.14509 .02383 L
.16639 .02712 L
.18653 .03103 L
.22495 .04094 L
.26581 .05532 L
.30517 .07259 L
.343 .09126 L
.38329 .11121 L
.42206 .12778 L
.44319 .13474 L
.45292 .13732 L
.46328 .13957 L
.46863 .14053 L
.47367 .1413 L
.47858 .14193 L
.48309 .14239 L
.48739 .14273 L
.48955 .14286 L
.49192 .14298 L
.49431 .14307 L
.49564 .14311 L
.49686 .14313 L
.498 .14315 L
.49923 .14316 L
.5004 .14316 L
.50148 .14315 L
.50265 .14314 L
.50389 .14312 L
.50495 .14309 L
.50611 .14306 L
.5087 .14296 L
.5111 .14283 L
.51547 .14252 L
.52014 .14207 L
.52569 .14139 L
.53094 .1406 L
.54074 .13876 L
.56076 .13358 L
.57921 .1273 L
.61861 .1103 L
.6565 .09151 L
.69684 .07165 L
.73566 .05473 L
.77694 .04037 L
Mistroke
.79757 .03472 L
.8167 .03035 L
.85495 .02383 L
.87551 .02136 L
.89413 .01964 L
.91341 .01826 L
.93425 .01716 L
.95422 .0164 L
.9753 .01583 L
.97619 .01581 L
Mfstroke
.02381 .01718 m
.04262 .01804 L
.06244 .0192 L
.08255 .02072 L
.10458 .02285 L
.12579 .02545 L
.14509 .02834 L
.18408 .03595 L
.22401 .04643 L
.26243 .05903 L
.30329 .0746 L
.34264 .09056 L
.38048 .10531 L
.40152 .11264 L
.42077 .11849 L
.44087 .1235 L
.45954 .12699 L
.46873 .12825 L
.47852 .12925 L
.48412 .12966 L
.48687 .12981 L
.48939 .12993 L
.49169 .13002 L
.49413 .13008 L
.49539 .13011 L
.49657 .13013 L
.49786 .13014 L
.49858 .13015 L
.49925 .13015 L
.50042 .13015 L
.50153 .13015 L
.50274 .13014 L
.50405 .13012 L
.50535 .13009 L
.50672 .13006 L
.50916 .12999 L
.51143 .12989 L
.51354 .12979 L
.51833 .12949 L
.52359 .12907 L
.52837 .12859 L
.53909 .1272 L
.55828 .12369 L
.57907 .11853 L
.61999 .10514 L
.6594 .08974 L
.69728 .07437 L
.73762 .05902 L
.77645 .04629 L
.81772 .03555 L
Mistroke
.83835 .03127 L
.85748 .02793 L
.89573 .02282 L
.91629 .02082 L
.93491 .01938 L
.9542 .0182 L
.97503 .01723 L
.97619 .01718 L
Mfstroke
.02381 .01891 m
.04262 .02011 L
.06244 .02165 L
.10458 .02615 L
.14264 .0319 L
.18314 .04006 L
.22214 .04996 L
.26358 .06245 L
.30351 .07576 L
.34193 .08881 L
.38279 .10167 L
.40311 .10722 L
.42214 .1117 L
.44154 .11542 L
.45998 .11805 L
.46966 .11906 L
.47477 .11948 L
.48023 .11985 L
.48526 .12011 L
.48983 .12028 L
.49204 .12034 L
.4932 .12036 L
.49443 .12038 L
.4955 .1204 L
.49668 .12042 L
.49775 .12042 L
.49875 .12043 L
.50001 .12043 L
.50065 .12043 L
.50135 .12043 L
.50259 .12042 L
.50376 .12041 L
.50512 .12039 L
.50638 .12037 L
.5092 .1203 L
.51151 .12023 L
.51403 .12014 L
.5191 .11988 L
.52375 .11959 L
.52872 .1192 L
.53766 .11832 L
.5479 .11704 L
.55901 .11532 L
.57916 .11142 L
.62008 .10083 L
.65948 .08834 L
.69737 .07546 L
.73771 .06204 L
.77653 .05034 L
.81781 .03984 L
Mistroke
.85757 .03186 L
.89581 .0261 L
.91717 .02361 L
.93651 .02174 L
.97569 .01894 L
.97619 .01891 L
Mfstroke
.02381 .02081 m
.06244 .02411 L
.10458 .02916 L
.14415 .03551 L
.18221 .04318 L
.22272 .05297 L
.26171 .06365 L
.30316 .07574 L
.34309 .08727 L
.3815 .09731 L
.40095 .10171 L
.42237 .10583 L
.44268 .10895 L
.45178 .11006 L
.46172 .11107 L
.4671 .11152 L
.4721 .11188 L
.47727 .1122 L
.48196 .11243 L
.48658 .1126 L
.4887 .11266 L
.49093 .11272 L
.49332 .11276 L
.49438 .11278 L
.49552 .11279 L
.49675 .1128 L
.49789 .11281 L
.49859 .11281 L
.49925 .11282 L
.50049 .11282 L
.50163 .11281 L
.50286 .11281 L
.50401 .1128 L
.50508 .11278 L
.50754 .11275 L
.51014 .11269 L
.51268 .11262 L
.51504 .11254 L
.5204 .11232 L
.5293 .11179 L
.53882 .11102 L
.54906 .10997 L
.56016 .10856 L
.58032 .10536 L
.62123 .09665 L
.66064 .08623 L
.69852 .07525 L
.73886 .06348 L
.77769 .05286 L
.81897 .04293 L
Mistroke
.85873 .03499 L
.89697 .02895 L
.93767 .0241 L
.97619 .02081 L
Mfstroke
.02381 .02274 m
.06244 .02644 L
.10458 .03182 L
.14415 .03826 L
.18221 .04574 L
.22272 .05492 L
.26171 .06462 L
.30316 .07531 L
.34309 .08525 L
.3815 .09375 L
.40095 .09743 L
.42237 .10087 L
.44268 .10345 L
.45178 .10437 L
.46172 .1052 L
.4671 .10558 L
.4721 .10587 L
.47727 .10613 L
.48196 .10632 L
.48658 .10646 L
.4887 .10651 L
.49093 .10656 L
.49332 .1066 L
.49438 .10661 L
.49552 .10662 L
.49675 .10663 L
.49789 .10664 L
.49859 .10664 L
.49925 .10664 L
.50049 .10664 L
.50163 .10664 L
.50286 .10663 L
.50401 .10662 L
.50508 .10661 L
.50754 .10658 L
.51014 .10654 L
.51268 .10648 L
.51504 .10642 L
.5204 .10623 L
.5293 .10579 L
.53882 .10516 L
.54906 .10429 L
.56016 .10313 L
.58032 .10048 L
.62123 .0932 L
.66064 .08436 L
.69852 .07488 L
.73886 .06448 L
.77769 .05482 L
.81897 .04549 L
Mistroke
.85873 .03775 L
.89697 .03159 L
.93767 .02643 L
.97619 .02274 L
Mfstroke
.02381 .02459 m
.06244 .02856 L
.10458 .0341 L
.14415 .04049 L
.18221 .04767 L
.22272 .05624 L
.26171 .06507 L
.30316 .07457 L
.34309 .08325 L
.3815 .09057 L
.40095 .09371 L
.42237 .09663 L
.44268 .09881 L
.45178 .09959 L
.46172 .10029 L
.4671 .1006 L
.4721 .10085 L
.47727 .10107 L
.48196 .10123 L
.48658 .10135 L
.4887 .10139 L
.49093 .10143 L
.49332 .10146 L
.49438 .10147 L
.49552 .10148 L
.49675 .10149 L
.49789 .1015 L
.49859 .1015 L
.49925 .1015 L
.50049 .1015 L
.50163 .1015 L
.50286 .10149 L
.50401 .10149 L
.50508 .10148 L
.50754 .10145 L
.51014 .10141 L
.51268 .10137 L
.51504 .10131 L
.5204 .10115 L
.5293 .10079 L
.53882 .10026 L
.54906 .09952 L
.56016 .09854 L
.58032 .0963 L
.62123 .09009 L
.66064 .08248 L
.69852 .07419 L
.73886 .06494 L
.77769 .05615 L
.81897 .04744 L
Mistroke
.85873 .03999 L
.89697 .03387 L
.93767 .02855 L
.97619 .02459 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{231, 142.688},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/Q000`40O003h00Oogoo8Goo003oOolQ
Ool0015oo`D003aoo`D007Moo`D003Yoo`D000ioo`004Woo00@007ooOol003ioo`03001oogoo07Uo
o`03001oogoo03]oo`04001oogoo000>Ool000Uoo`H000Aoo`03001oogoo03=oo`H000Eoo`03001o
ogoo07Uoo`03001oogoo03aoo`03001oogoo00ioo`0057oo00<007ooOol0?Goo00<007ooOol0NGoo
00<007ooOol0?Goo00<007ooOol03Goo000EOol00`00Oomoo`0lOol00`00Oomoo`1iOol00`00Oomo
o`0nOol00`00Oomoo`0<Ool001Eoo`03001oogoo03aoo`03001oogoo07Uoo`03001oogoo03ioo`03
001oogoo00aoo`004Goo00D007ooOomoo`0003ioo`03001oogoo07Uoo`03001oogoo03Yoo`05001o
ogooOol0000>Ool0019oo`<003eoo`<007Uoo`<003eoo`<000moo`00ogoo8Goo002@Ool00`00Oomo
o`2=Ool0091oo`03001oogoo08eoo`00T7oo00<007ooOol0SGoo002@Ool00`00Oomoo`2=Ool000Uo
ool0010000Qoo`0047oo6P0000=oo`0000001`000Woo00<007ooOol00Woo2@002goo1@002Goo00<0
07ooOol02Goo00<007ooOol01goo100037oo00<007ooOol02Woo00<007ooOol02Woo00<007ooOol0
2Woo00<007oo00000P001goo00<007ooOol02Woo00<007ooOol01goo1P002Woo2`000Woo9`003goo
000@OolA0002OolL0005Ool70009Ool6000KOol2000IOol00`00Oomoo`0GOol2000LOol5000=Ool3
0008Ool60002OolT000?Ool001Aooal00003Ool0000000@000Eoo`@000Qoo`8000eoo`<001Uoo`03
001oogoo01Moo`03001oogoo01Ioo`03001oogoo01Uoo`<000aoo`H000Ioo`D000=oo`D00003Ool0
000001h0019oo`006Goo7P000goo10001Goo0`001goo0`003Goo00<007ooOol05goo00<007ooOol0
5Woo00<007ooOol05Woo00<007ooOol05goo0P002goo100027oo10000goo1@0000=oo`0000007P00
5goo000QOolE000017oo0000000017oo100017oo0`001goo0P0037oo00<007ooOol05goo00<007oo
Ool05Goo0P005Woo00<007ooOol05Goo0`002Woo0`002Goo0`000goo100000=oo`0000000`000Woo
5P007goo000YOol@00000goo000000030003Ool30004Ool30006Ool3000:Ool2000GOol00`00Oomo
o`0EOol00`00Oomoo`0EOol00`00Oomoo`0DOol00`00Oomoo`09Ool20008Ool40003Ool300000goo
000000020004OolA000WOol002iooa000003Ool0000000<00004Ool000000004Ool30006Ool2000:
Ool2000FOol00`00Oomoo`0DOol00`00Oomoo`0DOol00`00Oomoo`0DOol00`00Oomoo`08Ool20006
Ool40005Ool60003OolA000/Ool003Aoo`l00005Ool00000001oo`030004Ool30005Ool2000:Ool0
0`00Oomoo`0DOol00`00Oomoo`0COol00`00Oomoo`0COol00`00Oomoo`0DOol00`00Oomoo`07Ool2
0006Ool20006Ool500000goo0000000?000bOol003Uooa400004Ool000000004Ool20005Ool20009
Ool2000DOol00`00Oomoo`0COol00`00Oomoo`0BOol00`00Oomoo`0DOol00`00Oomoo`06Ool20006
Ool20005OolE000gOol003ioo`l00004Ool000000003Ool20005Ool20009Ool00`00Oomoo`0BOol0
0`00Oomoo`0BOol00`00Oomoo`0BOol00`00Oomoo`0COol00`00Oomoo`06Ool00`00Oomoo`04Ool2
0004OolB000mOol004=oo`d00003Ool0000000=oo`03001oogoo00Aoo`03001oogoo00Moo`03001o
ogoo015oo`03001oogoo019oo`8001=oo`03001oogoo019oo`03001oogoo00Eoo`8000Eoo`8000=o
oa00049oo`00BGoo2@0000Eoo`00000007oo008000Eoo`8000Qoo`03001oogoo011oo`03001oogoo
019oo`03001oogoo015oo`03001oogoo019oo`03001oogoo00Eoo`03001oogoo00Aoo`03001oogoo
00<00003Ool0000000L004Qoo`00CWoo1`0000Aoo`00001oo`8000Eoo`03001oogoo00Ioo`03001o
ogoo00moo`03001oogoo019oo`03001oogoo015oo`03001oogoo015oo`03001oogoo00Eoo`03001o
ogoo00=oo`800004Ool00000Ool7001=Ool005=oo`X000Aoo`03001oogoo00Ioo`03001oogoo00mo
o`03001oogoo015oo`03001oogoo015oo`03001oogoo011oo`03001oogoo00Aoo`8000Aoo`X0059o
o`00Egoo20000goo0P001Woo00<007ooOol03goo00<007ooOol04Goo00<007ooOol04Goo00<007oo
Ool03goo00<007ooOol017oo00<007ooOol00Woo2000EWoo001JOol70003Ool00`00Oomoo`04Ool0
0`00Oomoo`0>Ool00`00Oomoo`0AOol00`00Oomoo`0AOol00`00Oomoo`0>Ool00`00Oomoo`03Ool2
0004Ool6001IOol005eoo`L00003Ool0000000Eoo`03001oogoo00eoo`03001oogoo015oo`80015o
o`03001oogoo00ioo`03001oogoo00=oo`04001oogooOol6001LOol0061oo`X000=oo`03001oogoo
00eoo`03001oogoo011oo`03001oogoo011oo`03001oogoo00eoo`03001oogoo00=oo`P005moo`00
HWoo3@003Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol03Goo3000HGoo001UOol30000
0goo000000090009Ool00`00Oomoo`0@Ool00`00Oomoo`0@Ool00`00Oomoo`08Ool?001SOol006Mo
oa8000Aoo`03001oogoo011oo`03001oogoo011oo`03001oogoo00=ooa8006Eoo`00JGoo6P003Goo
0P003Goo6P00Igoo001[Ool800000goo000000090004OolL0003OolE001YOol006aoo`<00004Ool0
0000Ool30003OolB0005Ool00`00Oomoo`02OolD00000goo0000000200000goo00000002Ool2001[
Ool006eoo`<00003Ool0000000800003Ool00000008000Aoo`D000Aoo`h000Moo`L000Aoo`<0009o
o`@00003Ool0000006eoo`00Kgoo10000Woo0`000Woo1@0000Aoo`00001ooaT000Eoo`03001oo`00
00<0009oo`<000=oo`<006ioo`00L7oo1@0000=oo`00Ool00`000goo1P001goo00<007ooOol00Woo
00<007ooOol02Goo1`000goo0`000goo1@00Kgoo001bOol2000017oo000000000Woo1@001Goo6000
1Goo00<007oo00000P000goo0`0000Aoo`00Ool0071oo`00Lgoo0P0000=oo`00Ool00`0017oo1@00
0goo10001Goo00<007ooOol02Woo1P000goo0`000Woo1000LGoo001dOol30003Ool400000goo001o
o`03Ool60006Ool00`00Oomoo`06Ool40004Ool00`00Oomoo`020003Ool200000goo0000001bOol0
07Eoo`<0009oo`04001oogooOol30006Ool00`00Oomoo`0?0007Ool40004Ool01@00Oomoogoo0000
Lgoo001eOol50005Ool01000Ool0000017oo10001Goo00<007ooOol02goo0`001Goo0P000goo00<0
07ooOol0LWoo001fOol20002Ool00`00Oomoo`02Ool01000Oomoogoo10002Goo00<007ooOol01goo
10000Woo00@007ooOomoo`8000Aoo`03001oogoo07=oo`00MWoo00<007oo00000Woo0`000Woo00<0
07ooOol017oo4`001Woo00@007oo000000Eoo`8007Eoo`00Mgoo00<007oo000017oo0`003goo00<0
07ooOol03Goo0P001Woo0P00MWoo001hOol00`00Ool00005Ool3000=Ool00`00Oomoo`0;Ool30005
Ool200000goo001oo`1eOol007Qoo`03001oogoo008000=oo`03001oogoo00<000Yoo`03001oogoo
00Yoo`03001oo`0000Eoo`04001oogoo001gOol007Uoo`05001oogooOol00002Ool00`00Oomoo`03
Ool20008Ool00`00Oomoo`05Ool50002Ool00`00Oomoo`02Ool01000Oomoo`00N7oo001iOol00`00
Oomoo`02Ool00`00Ool00007Ool@0007Ool01@00Oomoogoo00000goo00<007ooOol0MWoo001jOol0
0`00Oomoo`02Ool2000?Ool00`00Oomoo`0<Ool01000Oomoo`000goo00<007ooOol0Mgoo001jOol0
0`00Oomoo`03Ool2000>Ool00`00Oomoo`0<Ool30004Ool00`00Oomoo`1gOol007]oo`03001oogoo
00=oo`8000eoo`03001oogoo00]oo`8000Eoo`03001oogoo07Qoo`00O7oo00<007ooOol00Woo00@0
07oo000000]oo`03001oogoo00Uoo`800003Ool007oo00=oo`03001oogoo07Uoo`00O7oo00<007oo
Ool00Woo00D007ooOomoo`0000Yoo`03001oogoo00Qoo`05001oogooOol00004Ool00`00Oomoo`1i
Ool007eoo`05001oogooOol00004Ool30007Ool20008Ool00`00Oomoo`02Ool01@00Oomoogoo0000
O7oo001mOol01@00Oomoogoo00001goo00<007ooOol017oo00<007ooOol01Goo0P0017oo00<007oo
Ool00Woo00<007ooOol0NWoo001nOol01000Oomoo`0027oo10000Woo00<007ooOol01@001Woo00D0
07ooOomoo`0007eoo`00OWoo00D007ooOomoo`0000]oo`D000]oo`04001oogoo001nOol007moo`04
001oogoo000=Ool00`00Oomoo`0;Ool01000Oomoo`00OWoo0020Ool00`00Ool0000=Ool2000<Ool0
0`00Ool0001oOol0081oo`03001oo`0000eoo`03001oogoo00]oo`03001oo`0007moo`00PGoo0P00
3Goo00<007ooOol02goo0P00P7oo0021Ool2000=Ool00`00Oomoo`0;Ool20020Ool0089oo`03001o
ogoo009oo`D000Aoo`03001oogoo00Yoo`80085oo`00Pgoo00<007ooOol00goo00<007ooOol017oo
00<007ooOol02Woo00<007ooOol0P7oo0023Ool20004Ool00`00Oomoo`04Ool30009Ool20022Ool0
08=oo`8000Aoo`03001oogoo00Aoo`03001oogoo00Qoo`03001oo`00089oo`00Pgoo00<007oo0000
0goo00<007ooOol017oo00<007ooOol01goo00@007ooOol0089oo`00Pgoo00@007ooOol0009oo`03
001oogoo00Aoo`03001oogoo00Ioo`05001oogooOol00022Ool008=oo`07001oogooOol007oo0000
1Woo00<007ooOol01Woo00D007ooOomoo`00089oo`00Pgoo00@007ooOomoo`<000Ioo`03001oogoo
00Eoo`03001oogoo009oo`03001oogoo081oo`00Pgoo00<007ooOol017oo00<007ooOol00goo0P00
1Goo00<007ooOol00Woo00<007ooOol0PGoo0023Ool00`00Oomoo`05Ool20003Ool01000Oomoogoo
0`001Goo00<007ooOol0PGoo0024Ool00`00Oomoo`06Ool70008Ool00`00Oomoo`21Ool008Aoo`03
001oogoo00Qoo`<000Yoo`03001oogoo085oo`00Q7oo00<007ooOol02Goo00<007ooOol02Goo00<0
07ooOol0PGoo0024Ool00`00Oomoo`09Ool2000:Ool00`00Oomoo`21Ool008Aoo`03001oogoo00Uo
o`03001oogoo00Uoo`03001oogoo085oo`00Q7oo00<007ooOol02Goo00<007ooOol02Goo00<007oo
Ool0PGoo0024Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`22Ool008Aoo`03001oogoo
00Uoo`03001oogoo00Qoo`03001oogoo089oo`00Q7oo00<007ooOol02Goo00<007ooOol027oo00<0
07ooOol0PWoo0024Ool00`00Oomoo`09Ool20009Ool00`00Oomoo`22Ool008Eoo`03001oogoo00Qo
o`03001oogoo00Qoo`03001oogoo089oo`00QGoo00<007ooOol027oo00<007ooOol027oo00<007oo
Ool0PWoo0025Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`22Ool008Eoo`03001oogoo
00Qoo`03001oogoo00Qoo`03001oogoo089oo`00QGoo00<007ooOol027oo00<007ooOol027oo00<0
07ooOol0PWoo0025Ool00`00Oomoo`08Ool20008Ool00`00Oomoo`23Ool008Eoo`03001oogoo00Qo
o`03001oogoo00Moo`03001oogoo08=oo`00QGoo00<007ooOol027oo00<007ooOol01goo00<007oo
Ool0Pgoo001gOol50004Ool20003Ool00`00Ool000030005Ool00`00Oomoo`07Ool00`00Oomoo`23
Ool007Uoo`03001oogoo00Aoo`8000=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00Mo
o`03001oogoo08=oo`00NGoo00<007ooOol02Woo00<007ooOol00Woo00<007ooOol00Woo0`001goo
00<007ooOol0Pgoo001iOol00`00Oomoo`0:Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomo
o`07Ool00`00Oomoo`23Ool007Uoo`03001oogoo00Yoo`D000Eoo`03001oogoo00Moo`03001oogoo
08=oo`00NGoo00<007ooOol02Woo0P0027oo00<007ooOol01goo00<007ooOol0Pgoo001iOol00`00
Oomoo`0:Ool20008Ool00`00Oomoo`06Ool00`00Oomoo`24Ool007Moo`<000aoo`D000Eoo`03001o
ogoo00Ioo`03001oogoo08Aoo`00QWoo00<007ooOol01goo0P001goo00<007ooOol0Q7oo0026Ool0
0`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`24Ool008Ioo`03001oogoo00Moo`03001oogoo
00Ioo`03001oogoo08Aoo`00QWoo00<007ooOol01goo00<007ooOol01Woo00<007ooOol0Q7oo0027
Ool00`00Oomoo`06Ool00`00Oomoo`06Ool00`00Oomoo`24Ool008Moo`03001oogoo00Ioo`8000Mo
o`03001oogoo08Aoo`00Qgoo00<007ooOol01Woo00<007ooOol01Woo00<007ooOol0Q7oo0027Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`25Ool008Moo`03001oogoo00Ioo`03001oogoo
00Eoo`03001oogoo08Eoo`00Qgoo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0QGoo0027
Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`25Ool008Moo`03001oogoo00Ioo`8000Io
o`03001oogoo08Eoo`00Qgoo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0QGoo0027Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`25Ool008Qoo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo08Eoo`00R7oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0QGoo0028
Ool00`00Oomoo`05Ool00`00Oomoo`05Ool00`00Oomoo`25Ool008Qoo`03001oogoo00Eoo`8000Io
o`03001oogoo08Eoo`00R7oo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0QGoo0028Ool0
0`00Oomoo`05Ool00`00Oomoo`04Ool00`00Oomoo`26Ool008Ioo`D000Eoo`03001oogoo00Aoo`03
001oogoo08Ioo`00Qgoo0P0000=oo`00Ool017oo00<007ooOol017oo00<007ooOol0QWoo0028Ool0
0`00Oomoo`05Ool30004Ool00`00Oomoo`26Ool008Qoo`8000Ioo`03001oogoo00Aoo`03001oogoo
08Ioo`00R7oo00<007oo00001Goo00<007ooOol017oo00<007ooOol0QWoo0028Ool00`00Ool00005
Ool00`00Oomoo`04Ool00`00Oomoo`26Ool008Ioo`03001oogoo008000Eoo`03001oogoo00Aoo`03
001oogoo08Ioo`00Qgoo0`001Woo00<007ooOol017oo00<007ooOol0QWoo0029Ool00`00Oomoo`04
Ool20005Ool00`00Oomoo`26Ool008Uoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo08Io
o`00RGoo00<007ooOol017oo00<007ooOol017oo00<007ooOol0QWoo0029Ool00`00Oomoo`04Ool0
0`00Oomoo`04Ool00`00Oomoo`26Ool008Uoo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo
08Ioo`00RGoo00<007ooOol017oo00<007ooOol00goo00<007ooOol0Qgoo0029Ool00`00Oomoo`04
Ool20004Ool00`00Oomoo`27Ool008Uoo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo08Mo
o`00RWoo00<007ooOol00goo00<007ooOol00goo00<007ooOol0Qgoo002:Ool00`00Oomoo`03Ool0
0`00Oomoo`03Ool00`00Oomoo`27Ool008Yoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
08Moo`00RWoo00<007ooOol00goo0P0017oo00<007ooOol0Qgoo002:Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`27Ool008Yoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo08Mo
o`00RWoo00<007ooOol00goo00<007ooOol00goo00<007ooOol0Qgoo002;Ool00`00Oomoo`02Ool0
0`00Oomoo`02Ool00`00Oomoo`28Ool008]oo`03001oogoo009oo`03001oogoo009oo`03001oogoo
08Qoo`00Rgoo00<007ooOol00Woo0P000goo00<007ooOol0R7oo002;Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`28Ool008]oo`03001oogoo009oo`03001oogoo009oo`03001oogoo08Qo
o`00MWoo1@001Goo0P001Goo1@0017oo00<007ooOol00Woo00<007ooOol0R7oo001gOol01000Oomo
o`001Goo0P002Goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0R7oo001hOol00`00Oomo
o`0@Ool00`00Oomoo`02Ool300000goo001oo`2:Ool007Uoo`03001oogoo00moo`8000=oo`05001o
ogooOol0002;Ool007Yoo`03001oogoo00Yoo`@00003Ool007oo009oo`05001oogooOol0002;Ool0
07Yoo`03001oogoo00Yoo`03001oogoo009oo`05001oogooOol00003Ool00`00Oomoo`29Ool007Io
o`05001oogooOol0000<Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00<007ooOol0RGoo001g
Ool3000=Ool400000goo001oo`02Ool01@00Oomoogoo0000Rgoo002<Ool01000Oomoogoo0P000Woo
00<007ooOol0RGoo002<Ool01@00Oomoogoo00000Woo00<007ooOol0RWoo002=Ool01000Oomoo`00
0Woo00<007ooOol0RWoo002=Ool01000Oomoo`000Woo00<007ooOol0RWoo002=Ool01000Oomoo`00
0Woo00<007ooOol0RWoo002=Ool01000Oomoo`000Woo00<007ooOol0RWoo002=Ool00`00Oomoo`02
00000goo001oo`2;Ool008eoo`06001oogoo001oo`00SGoo002>Ool01@00Ool007oo0000SGoo002>
Ool01@00Ool007oo0000SGoo002>Ool01@00Ool007oo0000SGoo002?Ool3002>Ool008moo`<008io
o`00T7oo00<007ooOol0SGoo002@Ool00`00Oomoo`2=Ool0091oo`03001oogoo08eoo`00T7oo00<0
07ooOol0SGoo0000\
\>"],
  ImageRangeCache->{{{0, 287.75}, {177.312, 0}} -> {-2.24926, -0.25984, 
  0.0194688, 0.0222157}}]
}, Open  ]],

Cell[TextData[{
  "For small ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  " it is strongly concentrated at the origin, and gradually spreads out. \
This solution is very special, as we shall see in the next section. It is \
sometimes called the \"Green's function\" for the diffusion equation, or the \
fundamental solution. \n\nWe have now generated a wide variety of particular \
solutions to our PDE by special methods. However, we do not have control over \
the form of these solutions at ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  ". What we really want to do is to be able to specify the solution at ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  " and evolve the solution from it. The means by which one achieves this \
makes a subtle combination of the Fourier-mode solutions and the Green's \
function we have just derived, and will be considered in the next section. \
Right now we develop a technical result about the Laplace transform of the \
Green's function. Note that if you are unfamiliar with transform calculus, \
the more basic results that follow later do not require the transform \
approach."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Useful Transforms of the Green's Function", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Later in our analysis we shall need some technical results about the \
time-Laplace and spatial-Fourier transforms of the Green's function. This is \
a good place to get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to help us, as some of the integrals are rather awkward to manage \
directly. We load a Package to help us:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Calculus`LaplaceTransform`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?LaplaceTransform\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("LaplaceTransform[expr, t, s] gives a function of s, which is the \
Laplace transform of expr, a function of t, t >= 0. It is defined by  \
LaplaceTransform[expr, t, s] = Integrate[Exp[-s t] expr, {t, 0, \
Infinity}]."\)], "Print"]
}, Open  ]],

Cell["\<\
Let's go for the transform of our fundamental solution, with \
respect to time:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(LaplaceTransform[Exp[\(-x^2\)/\((4*t)\)]/Sqrt[4\ Pi\ t], \ t, \ s]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(\(-\@s\)\ \@x\^2\)\/\(2\ \@s\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[PowerExpand[%]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\(-\@s\)\ x\)\/\(2\ \@s\)\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "What we want to note for future reference is that if we differentiate the \
fundamental solution with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ", we can get rid of the ",
  Cell[BoxData[
      \(TraditionalForm\`\@s\)]],
  " in the denominator. As a check on this idea, let's ask directly for the \
inverse transform of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(\(-x\)\ \@s\)\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(InverseLaplaceTransform[Exp[\(-x\)\ Sqrt[s]], \ s, \ t]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(-\(x\^2\/\(4\ t\)\)\)\ x\)\/\(2\ \@\[Pi]\ t\^\(3/2\)\)\)], 
  "Output"]
}, Open  ]],

Cell[TextData[{
  "We shall use these results later. We shall also record a Fourier transform \
w.r.t. ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Calculus`FourierTransform`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?FourierTransform\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("FourierTransform[expr, t, w] gives a function of w, which is the \
Fourier transform of expr, a function of t.  It is defined by \
FourierTransform[expr, t, w] = FourierOverallConstant * \
Integrate[Exp[FourierFrequencyConstant I w t] expr, {t, -Infinity, \
Infinity}]."\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "Let's go for the transform of our fundamental solution, with respect to ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FourierTransform[Exp[\(-x^2\)/\((4*t)\)]/Sqrt[4\ Pi\ t], \ x, \ w]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(\(-t\)\ w\^2\)\)], "Output"]
}, Open  ]],

Cell["Let's go back, just to check:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(InverseFourierTransform[Exp[\(-t\)\ w^2], \ w, \ x]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(x\^2\/\(4\ t\)\)\)\/\(2\ \@\[Pi]\ \@t\)\)], "Output"]
}, Open  ]],

Cell["\<\
We shall also use these results, and give a mathematical \
justification of the easier ones, in our subsequent analysis.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
4.4 The Diffusion Equation - Solution from Initial Conditions\
\>", 
  "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We now come to an important observation. Suppose the diffusion equation \
holds for all ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(-\[Infinity]\) < x < \[Infinity]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and for ",
  Cell[BoxData[
      \(TraditionalForm\`t > 0\)]],
  ". Suppose further that we are given that"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u(x, 0) = f(x)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is the initial condition for the diffusion equation. In financial \
terms it represents the option payoff when it is defined for all values of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". Then the solution to the diffusion equation with this initial condition \
is"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \[Integral]\_\(-\[Infinity]\)\%\[Infinity]\(\( G(x - y, \[Tau])\)\ 
            \(f(y)\)\) \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In one sense, this result is now almost obvious. Since ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " is a solution to the diffusion equation, so is any linear combination of \
",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  "s - here such a combination is given by integration. Since ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " integrates to unity, we can write"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau])\  - \ f(x) = 
      \[Integral]\_\(-\[Infinity]\)\%\[Infinity]\( G(x - y, \[Tau])\)\ [
            f(y) - f(x)] \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now as ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau] \[Rule] 0\)]],
  ", the integrand, which has two terms multiplied, vanishes - ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " concentrates itself around ",
  Cell[BoxData[
      \(TraditionalForm\`y\  = \ x\)]],
  ", where the other terms vanishes. Hence "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`u(x, \[Tau]) - f(x) \[Rule] 0\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "as ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau] \[Rule] 0\)]],
  ". So the initial conditions are satisfied."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Fourier Transform View", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The solution we have written down and justified can also be derived \
by the use of Fourier transform methods. If you are not familiar with \
transform methods you may wish to skip this section. We write the spatial \
Fourier transform as\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`v(\[Omega], \[Tau]) = 
      \[Integral]\_\(-\[Infinity]\)\%\[Infinity] 
            \[ExponentialE]\^\(\[ImaginaryI]\ \[Omega]\ x\)\ \(u(x, \[Tau])\) 
          \[DifferentialD]x\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \(1\/\(2\ \[Pi]\)\) 
        \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity] 
              \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \[Omega]\ x\)\ 
            \(v(\[Omega], \[Tau])\) \[DifferentialD]\[Omega]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In ",
  StyleBox["Mathematica,",
    FontSlant->"Italic"],
  " ",
  StyleBox["FourierOverallConstant",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " = 1 gives these Fourier conventions. Now as ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " satisfies the diffusion equation, ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " satisfies the equation"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          
          FractionBox[\(\[PartialD]\(v \((\[Omega], \[Tau])\)\)\), 
            \(\[PartialD]\[Tau]\),
            MultilineFunction->None], "+", " ", 
          \(\(\[Omega]\^2\) \(v \((\[Omega], \[Tau])\)\)\)}], " ", "=", 
        "0"}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This has the simple solution", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`v(\[Omega], \[Tau]) = 
      \(v(\[Omega], 0)\)\ \[ExponentialE]\^\(\(-\[Omega]\^2\)\ \[Tau]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "So using our formula for ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " at general times:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \(1\/\(2\ \[Pi]\)\) 
        \(\[Integral]\_\(-
              \[Infinity]\)\%\[Infinity]\( 
                \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \[Omega]\ x\)\) 
            \(v(\[Omega], 0)\)\ 
            \(\[ExponentialE]\^\(\(-\[Omega]\^2\)\ \[Tau]\)\) 
            \[DifferentialD]\[Omega]\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Substituting for the value of ",
  Cell[BoxData[
      \(TraditionalForm\`v\)]],
  " at ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \(1\/\(2\ \[Pi]\)\) 
        \(\[Integral]\_\(-
              \[Infinity]\)\%\[Infinity]\( 
                \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \[Omega]\ x\)\) 
            \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity] 
                  \[ExponentialE]\^\(\[ImaginaryI]\ \[Omega]\ y\)\ 
                \(u(y, 0)\) \[DifferentialD]y\ \ 
                \(\[ExponentialE]\^\(\(-\[Omega]\^2\)\ \[Tau]\)\) 
                \[DifferentialD]\[Omega]\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Re-organizing gives", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(u(x, \[Tau]) = 
      \[Integral]\_\(-\[Infinity]\)\%\[Infinity]\ 
        \(f(y)\) \(H(x - y, \ \[Tau])\)\ \[DifferentialD]y\ \ \)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["where ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`H(z, \[Tau]) = 
      \(1\/\(2\ \[Pi]\)\) 
        \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity] 
              \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \[Omega]\ z - 
                \[Omega]\^2\ \[Tau]\)\ \[DifferentialD]\[Omega]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "but we have already established that the quantity ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(\(-\[Omega]\^2\)\ \[Tau]\)\)]],
  "is the Fourier transform of our fundamental solution - hence the result. \
(If you wish to prove this transform result yourself, two steps are needed: \
(a) complete the square in the exponential; (b) use complex contour \
integration methods to evaluate the remaining integral. We shall not pursue \
this here.)"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.5 Method of Images: Boundaries and Boundary Conditions", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The previous section gave the solution when all values of ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " are relevant. Sometimes one wishes to consider a situation where the \
range is limited to domains of one of the following forms:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`x \[GreaterEqual] a\), 
    \(TraditionalForm\`x \[LessEqual] b\), 
    \(TraditionalForm\`a \[LessEqual] x \[LessEqual] b\)}], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "When this is the case, a condition must be given for the function on the \
given boundary. An interesting class of options, known as Barrier options, \
will require an understanding of this type of solution. We consider in detail \
the simplest case, with ",
  Cell[BoxData[
      \(TraditionalForm\`x \[GreaterEqual] a\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`u(a, \ \[Tau])\  = \ g(\[Tau])\)]],
  ". We consider the case ",
  Cell[BoxData[
      \(TraditionalForm\`g\  = \ 0\)]],
  " first. The solution to this problem can be obtained by a cunning \
modification to the fundamental solution, based on the \"method of images\". \
Consider"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(G\_1\)(x, y, \[Tau]) = 
      G(x - y, \[Tau]) - G(x - \((2\ a - y)\), \[Tau])\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "which consists of a fundamental solution centred on ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", with a second fundamental solution centred on the image point ",
  Cell[BoxData[
      \(TraditionalForm\`2  a\  - \ y\)]],
  ", which is the location of a point at ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " reflected in the boundary. Clearly, ",
  Cell[BoxData[
      \(TraditionalForm\`G\_1\)]],
  " solves the diffusion equation. Furthermore"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(G\_1\)(a, y, \[Tau]) = 
      \(G(a - y, \[Tau]) - G(a - \((2\ a - y)\), \[Tau]) = 
        \(G(a - y, \[Tau]) - G(y - a, \[Tau]) = 0\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "since ",
  Cell[BoxData[
      \(TraditionalForm\`G\)]],
  " is an even function of its spatial argument. So ",
  Cell[BoxData[
      \(TraditionalForm\`G\_1\)]],
  " vanishes on ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  " for all values of ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ". So any linear combination over different ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " does. Now consider the modified integral"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \[Integral]\_a\%\[Infinity]\(\( G\_1\)(x - y, \[Tau])\)\ \(f(y)\) 
          \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Clearly ",
  Cell[BoxData[
      \(TraditionalForm\`u(a, \ \[Tau])\  = \ 0\)]],
  ". As we approach ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  " the contributions to the integrand are focused on the points ",
  Cell[BoxData[
      \(TraditionalForm\`y\_1\  = \ x\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`y\_2\  = \ 2\ a\  - \ x\)]],
  ". But for ",
  Cell[BoxData[
      \(TraditionalForm\`x\  > a, \ then\ y\_2 < a\)]],
  ", which is outside the region of integration, and there is no contribution \
from ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  ". Hence, for ",
  Cell[BoxData[
      \(TraditionalForm\`x\  > a\)]],
  ",  "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`lim\+\(\[Tau] \[Rule] 0\)\[ThinSpace]u(x, \[Tau]) = f(x)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "as before. Hence we have solved the problem when the boundary condition is \
that ",
  Cell[BoxData[
      \(TraditionalForm\`u = \ 0\)]],
  " on ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  ". By identical arguments, it follows that, if we have ",
  Cell[BoxData[
      \(TraditionalForm\`\(x\ \)\)]],
  "bounded above by ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  ", where it vanishes,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \[Integral]\_\(-\[Infinity]\)\%b\(\( G\_2\)(x, \ y, \[Tau])\)\ \(f(y)\) 
          \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(G\_2\)(x, y, \[Tau]) = 
      G(x - y, \[Tau]) - G(x - \((2\ b - y)\), \[Tau])\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now suppose that ",
  Cell[BoxData[
      \(TraditionalForm\`a \[LessEqual] x \[LessEqual] b\)]],
  " and that ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " vanishes at both ends. The solution method is somewhat analogous to the \
effect you get when standing between two parallel mirrors - an infinite \
number of images are generated. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`G\_\[Infinity] = \ 
      G(x - y, \[Tau]) - G(x - \((2\ a - y)\), \[Tau]) - 
          G(x - \((2\ b - y)\), \[Tau])\n\t\ \ \ \ \ \ \ \ \ \ \  + 
          G(x - \((2\ a - \((2\ b - y)\))\), \[Tau]) + \ 
          \(+\(G(x - \((2\ b - \((2\ a - y)\))\), \[Tau])\)\) -  ... \)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This can be re-organized as", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(G\_\[Infinity]\)(x, y, \[Tau])\  = \ 
      \[Sum]\+\(k = \(-\[Infinity]\)\)\%\[Infinity] G(
            x - y\  + \ 2\ k\ \ \((b - a)\), \ \[Tau]\ ) - 
        G(x + y - 2\ a + 2\ k\ \((b - a)\)\ , \ \[Tau])\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The solution is then", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \[Integral]\_a\%b\(\( G\_\[Infinity]\)(x, y, \[Tau])\) \(f(y)\) 
          \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Addition of a Boundary Term", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Sometimes we wish to consider a solution with a given initial \
condition and a specified function along the boundary. The analysis of this \
is a little harder - if you do not wish to understand where Barrier option \
rebate formulae come from, you may skip this section! We can obtain such a \
solution by adding, to a solution where the boundary is zero, a solution \
where the initial value is zero, and the given boundary function specified. \
So, as an example, suppose that\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`x \[GreaterEqual] a\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`u(a, \[Tau]) = g\ \((\[Tau])\)\), 
    \(TraditionalForm\`u(x, 0) = 0\ for\ x \[GreaterEqual] a\)}], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`lim\+\(x \[Rule] \[Infinity]\)\[ThinSpace]u(x, \[Tau]) = 0\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This is where a Laplace transform picture becomes useful. We \
let\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`v(x, p) = 
      \[Integral]\_0
          \%\[Infinity]\(\( u(x, \[Tau])\)\ 
            \[ExponentialE]\^\(\(-p\)\ \[Tau]\)\) \[DifferentialD]\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Then the Laplace transform of the diffusion equation gives us\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]\^2\( v \((x, p)\)\)\), \(\[PartialD]x\^2\),
          MultilineFunction->None], "=", \(p\ \(v \((x, p)\)\)\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The solution of this vanishing at infinity is", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`v(x, p) = 
      \(\(v(a, p)\)\ \[ExponentialE]\^\(\(-\@p\)\ \((x - a)\)\)\  = \ 
        \(\(g\&~\)(p)\)\ \[ExponentialE]\^\(\(-\@p\)\ \((x - a)\)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["where the Laplace transform of the boundary condition is", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(g\&~\)(p) = 
      \[Integral]\_0\%\[Infinity]\( g(\[Tau])\)\ 
          \(\[ExponentialE]\^\(\(-p\)\ \[Tau]\)\) \[DifferentialD]\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It follows that the solution is the Laplace convolution of ",
  Cell[BoxData[
      \(TraditionalForm\`\(g(\[Tau])\ \)\)]],
  " with the inverse Laplace transform of ",
  Cell[BoxData[
      \(TraditionalForm\`\[ExponentialE]\^\(\(-\@p\)\ \((x - a)\)\)\)]],
  ", which we have already found with the help of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". The solution is just"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \(1\/\(2\ \@\[Pi]\)\) 
        \(\[Integral]\_0
            \%\[Tau]\(\( 
                    \[ExponentialE]\^\(-
                      \(\((x - a)\)\^2\/\(4\ \((\[Tau] - t)\)\)\)\)\ 
                  \((x - a)\)\ \(g(t)\)\)\/\((\[Tau] - t)\)\^\(3/2\)\) 
            \[DifferentialD]t\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This may be usefully re-organized by a change of variables:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \(1\/\(2\ \@\[Pi]\)\) 
        \(\[Integral]\_\(1/
              \[Tau]\)\%\[Infinity]\(\( 
                    \[ExponentialE]\^\(-\(\(s(x - a)\)\^2\/4\)\)\ 
                  \((x - a)\)\ \(g(\[Tau] - 1/s)\)\)\/s\^\(1/2\)\) 
            \[DifferentialD]s\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's do a quick check on this rather formidable integral, by setting ",
  Cell[BoxData[
      \(TraditionalForm\`g\  = \ 1. \)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
Integrate[(Exp[-(1/4)*s(x - a)^2]*(x - a))/Sqrt[s], {s, 1/\[Tau], \
Infinity}]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"If", "[", 
      RowBox[{\(\[Tau] > 0 && Re[\((a - x)\)\^2] > 0 && 
          Re[a\^2\/4 - \(a\ x\)\/2 + x\^2\/4] > 0\), 
        ",", \(\(2\ \@\[Pi]\ \((a - x)\)\ \((\(-1\) + 
                  Erf[\@\((a - x)\)\^2\/\(2\ \@\[Tau]\)])\)\)\/\@\((a - \
x)\)\^2\), ",", 
        RowBox[{
          SubsuperscriptBox["\[Integral]", \(1\/\[Tau]\), 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 
              1]]], \(\(\(E\^\(\(-\(1\/4\)\)\ s\ \((\(-a\) + x)\)\^2\)\ \
\((\(-a\) + x)\)\)\/\@s\) \[DifferentialD]s\)}]}], "]"}]], "Output"]
}, Open  ]],

Cell["Picking out the relevant piece, and normalizing, we get:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(%[\([2]\)]/\((2*Sqrt[Pi])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\((a - x)\)\ \((\(-1\) + Erf[\@\((a - x)\)\^2\/\(2\ \
\@\[Tau]\)])\)\)\/\@\((a - x)\)\^2\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "We are interested in the values of this for ",
  Cell[BoxData[
      \(TraditionalForm\`x\  > a\)]],
  ", so we consider"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[\(-%\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1 - Erf[\(a - x\)\/\(2\ \@\[Tau]\)]\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Note that ",
  Cell[BoxData[
      \(TraditionalForm\`Erf(0)\  = \ 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`Erf(\[Infinity]) = 1\)]],
  ", so this solution is unity on the boundary ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  " and vanishes when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\  = \ 0\)]],
  ". This not only is correct, but coincides with our ",
  Cell[BoxData[
      \(TraditionalForm\`n = 0\)]],
  " similarity solution if the arbitrary constants are set suitably.  \
However, the integral for a general ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  ", in the form we have given, can be extremely awkward to manage, and a \
different representation of the solution is frequently useful. We go back to \
the Laplace inversion integral in the form"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`q(x, t)\  = \ 
      \(1\/\(2\ \[Pi]\ \[ImaginaryI]\)\) 
        \(\[Integral]\_\(c - \[ImaginaryI]\ \[Infinity]\)\%\(c + 
              \[ImaginaryI]\ 
                \[Infinity]\)\(\[ExponentialE]\^\(\(-\@p\)\ \((x - a)\)\)\ 
              \[ExponentialE]\^\(p\ t\)\) \[DifferentialD]p\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
and deform the contour to wrap around the negative real axis. \
Taking care of the branch points gives\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`q(x, t) = 
      \(1\/\[Pi]\) 
        \(\[Integral]\_0
            \%\[Infinity]\( \[ExponentialE]\^\(\(-p\)\ t\)\ 
              \(sin(\@p\ \((x - a)\))\)\) \[DifferentialD]p\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The solution with the given boundary condition can then be \
represented as\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u = \[Integral]\_0\%t\(\( g(\[Tau])\)\ \(q(\[Tau] - t)\)\) 
          \[DifferentialD]\[Tau]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If you are not familiar with contour integration, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can reassure you that this makes sense!"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\ Integrate[
      Exp[\(-p\)\ t]\ Sin[Sqrt[p] x], \ {p, \ 0, \ Infinity}]\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"If", "[", 
      RowBox[{\(Im[x] == 0 && Re[t] > 0\), 
        ",", \(\(E\^\(-\(x\^2\/\(4\ t\)\)\)\ \@\[Pi]\ \((x\^2\/t)\)\^\(3/2\)\ \
Sign[x]\)\/\(2\ x\^2\)\), ",", 
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 
              1]]], \(\(E\^\(\(-p\)\ t\)\ Sin[\@p\ x]\) \
\[DifferentialD]p\)}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "A minor manipulation reduces this to the result we have used, for ",
  Cell[BoxData[
      \(TraditionalForm\`x > 0\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerExpand[%[\([2]\)]/Pi]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(-\(x\^2\/\(4\ t\)\)\)\ x\ Sign[x]\)\/\(2\ \@\[Pi]\ \
t\^\(3/2\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
Similar formulae may be derived for an upper boundary condition, \
and a more complicated analysis can be done with boundary conditions supplied \
on two barriers. This set of basic solutions to the heat equation will serve \
our main purpose, which is to illustrate how the values of several important \
contracts may be found.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Neumann and Impedance Boundary Conditions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are other types of boundary condition that merit consideration, both \
on general mathematical grounds and on financial grounds. Suppose we consider \
the region ",
  Cell[BoxData[
      \(TraditionalForm\`x > 0\)]],
  ", for simplicity, and consider the general impedance boundary condition"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]u\), \(\[PartialD]x\),
          MultilineFunction->None], "=", \(\[Alpha]\ u\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "at ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ 0\)]],
  ". When ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\  = \ 0\)]],
  " this is a \"Neumann\" boundary condition. This case can easily be solved \
by the method of images by adding the image Green's function rather than \
subtracting it. In physics it corresponds to an insulation boundary condition \
for the heat equation. When ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  " is non-zero matters are a little more complicated. These problems were \
first studied many years ago as a model of heat conduction when the rate of \
heat loss at a boundary is proportional to the temperature there. Our purpose \
here is to show how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can be used to help in the construction of the solution. This section is \
also quite hard and is in fact not necessary for the more basic \
option-pricing problems - you may come back to it later if you wish to. We \
shall give the details of corresponding financial interpretation later - it \
is related to path-dependent options.\n\nThere are various ways of \
approaching the solution to such a problem. Guided by the cases where simple \
images work - \[Alpha] infinite is the one we have already treated, \[Alpha] \
= 0 being done by swapping some signs, it is natural to seek a similar \
solution for general ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  ". We can do so by using Fourier methods. We know, for example, that for \
all values of ",
  Cell[BoxData[
      \(TraditionalForm\`\[Omega], \ a, \ b\)]],
  ", the functions"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\((a\ \[ExponentialE]\^\(\(-\[ImaginaryI]\)\ \[Omega]\ x\) + 
          b\ \[ExponentialE]\^\(\[ImaginaryI]\ \[Omega]\ x\))\)\ 
      \[ExponentialE]\^\(\(-\[Omega]\^2\)\ t\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "are solutions of the diffusion equation. As we wish to build a solution \
that is the fundamental solution plus a correction to satisfy boundary \
conditions, we set ",
  Cell[BoxData[
      \(TraditionalForm\`a\  = \ 1\)]],
  ". Then if we apply the boundary condition, it follows that"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`b = \(\(\[Omega] - \[ImaginaryI]\ \[Alpha]\ 
          \)\/\(\[Omega] + \[ImaginaryI]\ \[Alpha]\ \)\  = \ 
        1\  - \(2  \[ImaginaryI]\ \[Alpha]\ 
            \)\/\(\[Omega] + \[ImaginaryI]\ \[Alpha]\ \)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "To find out what the new term is, we first confirm the inverse Fourier \
transform of the second contribution to ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " - note that ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " is replaced by ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(-x\)\ \)\)]],
  " if we use standard Fourier conventions:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FourierTransform[\(-2\)\ \[Alpha]\ UnitStep[\(-x\)]\ Exp[\[Alpha]\ x]\ \
, \ x, \ \[Omega], \ FourierFrequencyConstant\  -> \ \(-1\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(2\ \[Alpha]\)\/\(\[Alpha] - I\ \[Omega]\)\)\)\)], "Output"]
}, Open  ]],

Cell["The following may be more intelligible!", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Integrate[\(-2\)\ \[Alpha]\ Exp[
            x\ \((\[Alpha]\  - \ I\ \[Omega])\)], \ {x, \ \(-Infinity\), \ 
          0}]\)[\([2]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(2\ \[Alpha]\)\/\(\[Alpha] - I\ \[Omega]\)\)\)\)], "Output"]
}, Open  ]],

Cell["To get the desired result we take the convolution of ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[Theta]\ \((\(-x\))\)\ \[ExponentialE]\^\(\[Alpha]\ x\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "with the fundamental solution, by ordinary integration with ",
  StyleBox["Mathematica:",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Clear[\[Alpha]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[\(-2\)\ \[Alpha]\ Exp[\[Alpha]\ p]\ 1/Sqrt[4\ Pi\ \[Tau]] 
        Exp[\(-\((p - x)\)^2\)/\((4\ \[Tau])\)], \ {p, \ \(-Infinity\), \ 
        0}]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"If", "[", 
      RowBox[{\(Re[\[Alpha] + x\/\(2\ \[Tau]\)] > 0 && Re[\[Tau]] > 0\), 
        ",", \(E\^\(\[Alpha]\ \((x + \[Alpha]\ \[Tau])\)\)\ \[Alpha]\ \((\(-1\
\) + Erf[\(x + 2\ \[Alpha]\ \[Tau]\)\/\(2\ \@\[Tau]\)])\)\), ",", 
        RowBox[{
          SubsuperscriptBox["\[Integral]", 
            InterpretationBox[\(-\[Infinity]\),
              DirectedInfinity[ -1]], 
            "0"], \(\(-\(\(E\^\(p\ \[Alpha] - \((p - x)\)\^2\/\(4\ \[Tau]\)\)\
\ \[Alpha]\)\/\(\@\[Pi]\ \@\[Tau]\)\)\) \[DifferentialD]p\)}]}], "]"}]], 
  "Output"]
}, Open  ]],

Cell["We want the middle piece of this:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(GG[x_, \ \[Tau]_, \ \[Alpha]_]\  = \ %[\([2]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(\[Alpha]\ \((x + \[Alpha]\ \[Tau])\)\)\ \[Alpha]\ \((\(-1\) + 
          Erf[\(x + 2\ \[Alpha]\ \[Tau]\)\/\(2\ \@\[Tau]\)])\)\)], "Output"]
}, Open  ]],

Cell["\<\
We now make some checks on this rather odd-looking object. Let's \
look at it for large \[Alpha], where we know what the answer should be, by \
first doing an asymptotic expansion of the Error function:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[Erf[q], \ {q, \ Infinity, \ 2}] - 1\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{\(E\^\(-q\^2\)\), " ", 
      RowBox[{"(", 
        InterpretationBox[
          RowBox[{\(-\(1\/\(\@\[Pi]\ q\)\)\), "+", 
            InterpretationBox[\(O[1\/q]\^3\),
              SeriesData[ q, 
                DirectedInfinity[ 1], {}, 1, 3, 1]]}],
          SeriesData[ q, 
            DirectedInfinity[ 1], {
            Times[ -1, 
              Power[ Pi, 
                Rational[ -1, 2]]]}, 1, 3, 1]], ")"}]}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Normal[%]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(E\^\(-q\^2\)\/\(\@\[Pi]\ q\)\)\)\)], "Output"]
}, Open  ]],

Cell["\<\
Let's substitute the argument as we found it, and multiply by the \
factor we have found:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(%\  /. \ 
      q\  -> \ \((\ x + 2\ \[Alpha]\ \[Tau])\)/\((2\ Sqrt[\[Tau]])\)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(2\ E\^\(-\(\((x + 2\ \[Alpha]\ \[Tau])\)\^2\/\(4\ \[Tau]\)\)\)\ \
\@\[Tau]\)\/\(\@\[Pi]\ \((x + 2\ \[Alpha]\ \[Tau])\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%*\[Alpha]*\ Exp[\[Alpha]\ \((x\  + \ \[Alpha]\ \[Tau])\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(2\ E\^\(\[Alpha]\ \((x + \[Alpha]\ \[Tau])\) - \((x + 2\ \
\[Alpha]\ \[Tau])\)\^2\/\(4\ \[Tau]\)\)\ \[Alpha]\ \@\[Tau]\)\/\(\@\[Pi]\ \
\((x + 2\ \[Alpha]\ \[Tau])\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Cancel[ExpandAll[%]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(2\ E\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\ \[Alpha]\ \@\[Tau]\)\/\(\@\
\[Pi]\ \((x + 2\ \[Alpha]\ \[Tau])\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Series[%, \ {\[Alpha], \ Infinity, \ 1}]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{\(-\(E\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\/\(\@\[Pi]\ \@\[Tau]\)\)\),
         "+", 
        InterpretationBox[\(O[1\/\[Alpha]]\^1\),
          SeriesData[ \[Alpha], 
            DirectedInfinity[ 1], {}, 0, 1, 1]]}],
      SeriesData[ \[Alpha], 
        DirectedInfinity[ 1], {
        Times[ -1, 
          Power[ E, 
            Times[ 
              Rational[ -1, 4], 
              Power[ x, 2], 
              Power[ \[Tau], -1]]], 
          Power[ Pi, 
            Rational[ -1, 2]], 
          Power[ \[Tau], 
            Rational[ -1, 2]]]}, 0, 1, 1]]], "Output"]
}, Open  ]],

Cell[TextData[{
  "Thus as \[Alpha] becomes infinite this correction gives us ",
  Cell[BoxData[
      \(TraditionalForm\`\(-2\)\)]],
  " times the fundamental solution, as we require. Now note that"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ExpandAll[
      D[GG[x, \ \[Tau], \ \[Alpha]], \ 
          x]\ \  - \ \[Alpha]\ GG[x, \ \[Tau], \ \[Alpha]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(-\(x\^2\/\(4\ \[Tau]\)\)\)\ \[Alpha]\)\/\(\@\[Pi]\ \@\[Tau]\)\)],
   "Output"]
}, Open  ]],

Cell[TextData[{
  "So we write down our full solution, based on a source at ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", in the form"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(K[x_, \ y_, \ \[Tau]_, \ \[Alpha]_]\  = \ 
      Exp[\(-\((x - y)\)^2\)/\((4\ \[Tau])\)]/
          Sqrt[4\ Pi\ \[Tau]]\  + \ \ Exp[\(-\((x + y)\)^2\)/\((4\ \[Tau])\)]/
          Sqrt[4\ Pi\ \[Tau]] + GG[x + y, \ \[Tau], \ \[Alpha]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(\((x - y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ \@\[Tau]\) + 
      E\^\(-\(\((x + y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ \@\[Tau]\) + 
      E\^\(\[Alpha]\ \((x + y + \[Alpha]\ \[Tau])\)\)\ \[Alpha]\ \((\(-1\) + 
            Erf[\(x + y + 2\ \[Alpha]\ \[Tau]\)\/\(2\ \@\[Tau]\)])\)\)], 
  "Output"]
}, Open  ]],

Cell["\<\
This is supposed to be our Green's function. Let's first check that \
we have satisfied the boundary condition. First we work it out for general \
position:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      D[K[x, \ y, \ \[Tau], \ \[Alpha]], \ 
          x]\ \  - \ \[Alpha]\ K[x, \ y, \ \[Tau], \ \[Alpha]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(\(E\^\(-\(\((x + y)\)\^2\/\(4\ \[Tau]\)\)\)\ \((\((1 + 
                      E\^\(\(x\ y\)\/\[Tau]\))\)\ x - \((\(-1\) + 
                      E\^\(\(x\ y\)\/\[Tau]\))\)\ \((y - 
                      2\ \[Alpha]\ \[Tau])\))\)\)\/\(4\ \@\[Pi]\ \
\[Tau]\^\(3/2\)\)\)\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%\  /. \ x -> \ 0\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
We may as well check that the diffusion equation is satisfied:\
\>",
   "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      D[K[x, \ y, \ \[Tau], \ \[Alpha]], \ {x, \ 2}]\  - \ 
        D[K[x, \ y, \ \[Tau], \ \[Alpha]], \ \[Tau]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["\<\
Here is the answer written out carefully in traditional \
mathematical form (the output of the following has been converted to the \
style of a numbered equation here):\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm[K[x, y, \[Tau], \ \[Alpha]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[ExponentialE]\^\(\[Alpha]\ \((x + y + \[Alpha]\ \[Tau])\)\)\ 
        \[Alpha]\ 
        \((erf(\(x + y + 2\ \[Alpha]\ \[Tau]\)\/\(2\ \@\[Tau]\)) - 1)\) + 
      \[ExponentialE]\^\(-\(\((x - y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ 
          \@\[Tau]\) + 
      \[ExponentialE]\^\(-\(\((x + y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ 
          \@\[Tau]\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is to be integrated against the initial condition for ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  ", expressed as ",
  Cell[BoxData[
      \(TraditionalForm\`f(y)\)]],
  ", over the relevant range, here ",
  Cell[BoxData[
      \(TraditionalForm\`y\  > \ 0\)]],
  ". If the range of interest is negative, we replace ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`\(-x\)\)]],
  " everywhere, and replace \[Alpha] by ",
  Cell[BoxData[
      \(TraditionalForm\`\(-\[Alpha]\)\)]],
  " to preserve the boundary condition. This leads to the solution"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(M[x_, \ y_, \ \[Tau]_, \ \[Alpha]_]\  = \ 
      Exp[\(-\((x - y)\)^2\)/\((4\ \[Tau])\)]/
          Sqrt[4\ Pi\ \[Tau]]\  + \ \ Exp[\(-\((x + y)\)^2\)/\((4\ \[Tau])\)]/
          Sqrt[4\ Pi\ \[Tau]] + 
        GG[\(-x\) - y, \ \[Tau], \ \(-\[Alpha]\)]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm[M[x, y, \[Tau], \ \[Alpha]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\[ExponentialE]\^\(\(-\[Alpha]\)\ 
              \((\(-x\) - y - \[Alpha]\ \[Tau])\)\)\)\ \[Alpha]\ 
        \((erf(\(\(-x\) - y - 2\ \[Alpha]\ \[Tau]\)\/\(2\ \@\[Tau]\)) - 1)
          \) + \[ExponentialE]\^\(-\(\((x - y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ 
          \@\[Pi]\ \@\[Tau]\) + 
      \[ExponentialE]\^\(-\(\((x + y)\)\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ 
          \@\[Tau]\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The reader may check that ",
  StyleBox["M",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " satisfies the diffusion and boundary conditions."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.6 Derivation of Basic Option Prices", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We have carried out a substantial investigation of the diffusion \
equation with various types of initial and boundary condition. We shall now \
map various option-pricing problems of interest into one of the standard \
forms we have developed. We shall begin by considering options where there \
are no \"barriers\". \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["A General Formula", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Suppose that the asset price ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " may attain any positive value,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`0 < S < \[Infinity]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and that the option payoff function ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V(S\_T, T) = P(S\_T)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We make the change of variables discussed in Section 4.3, including\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S = K\ \[ExponentialE]\^x, \ 
    S\_T = K\ \[ExponentialE]\^y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where the subscript ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " labels the asset value at expiry of the option. Thus we obtain the \
initial condition for ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`f(y) = \(u(y, 0) = 
        \(P(K\ \[ExponentialE]\^y)\)\ 
          \[ExponentialE]\^\(1\/2\ \((k\_2 - 1)\)\ y\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["We now write down the solution in the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = \ 
      \[ExponentialE]\^\(\(-\(1\/2\)\)\ \((k\_2 - 1)\)\ x - 
            \((1\/4\ \((k\_2 - 1)\)\^2 + k\_1)\)\ \[Tau]\)\ 
        \(u(x, \[Tau])\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(x, \[Tau]) = 
      \[Integral]\_\(-
            \[Infinity]\)\%\[Infinity]\(\(\( P(K\ \[ExponentialE]\^y)\)\ 
                \[ExponentialE]\^\(1\/2\ \((k\_2 - 1)\)\ y\)\ 
                \[ExponentialE]\^\(-
                    \(\((x - y)\)\^2\/\(4\ \[Tau]\)\)\)\)\/\@\(4\ \[Pi]\ 
                  \[Tau]\)\) \[DifferentialD]y\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Now we note that this pair can be simplified by completing the \
square in the exponential:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = \ 
      \(\[ExponentialE]\^\(\(-k\_1\) \[Tau]\)\) 
        \(\[Integral]\_\(-
              \[Infinity]\)\%\[Infinity]\(\(\( P(K\ \[ExponentialE]\^y)\)\ \ 
                  \[ExponentialE]\^\(-
                      \(\((x - y + \[Tau](k\_2 - 1))\)\^2\/\(4\ 
                            \[Tau]\)\)\)\)\/\@\(4\ \[Pi]\ \[Tau]\)\) 
            \[DifferentialD]y\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If we change variables in the integration to ",
  Cell[BoxData[
      \(TraditionalForm\`S\_T\)]],
  ", we obtain"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = \ 
      \(\[ExponentialE]\^\(\(-t\)\ \((T - t)\)\)\) 
        \(\[Integral]\_\(-\[Infinity]\)\%\[Infinity]\( P(S\_T)\)\ \ 
            \(\[CapitalPsi](S\_T, \ S, \ r, \ T, \ r, \ q, \ \[Sigma])\) 
            \[DifferentialD]y\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPsi]\)]],
  " is given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[CapitalPsi] = 
      \[ExponentialE]\^\(-
            \(\((log(S\_T\/S) - \((r - q - \[Sigma]\^2\/2)\)\ \((T - t)\))
                    \)\^2\/\(2\ \[Sigma]\^2\ \((T - t)\)\)\)\)\/\(\((
            \[Sigma]\ \@\(2\ \[Pi]\ \((T - t)\)\))\)\ S\_T\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now we can view this expression for ",
  Cell[BoxData[
      \(TraditionalForm\`V\)]],
  " as merely an integral, but it is more informative to note that ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPsi]\)]],
  " is the probability density function for a log-normal distribution of ",
  Cell[BoxData[
      \(TraditionalForm\`S\_T\)]],
  ", where the log of the expiry price has a normal distribution with mean"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`log(S) - \((r - q - \[Sigma]\^2\/2)\)\ \((T - t)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and variance", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Sigma]\^2\ \((T - t)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Our option value can now be seen to be the discounted value of the \
expected mean payoff, where the expectation is taken over the distribution \
given by ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPsi]\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ E\ [P(S\_T)]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In what follows, the following notation will be useful. The variable ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " is normalized with respect to a base value ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  " of the underlying asset, usually a strike or some similar variable."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`y = log(S\/K)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Then ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  " is normally distributed with mean"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`m = log(S\/K) + \((r - q - \[Sigma]\^2\/2)\)\ \((T - t)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and standard deviation", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[CapitalSigma] = \[Sigma]\ \@\(T - t\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["So we can write", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[CapitalPsi]\ d\ S\_T = 
      \(\(\[ExponentialE]\^\(-
                \(\((y - m)\)\^2\/\(2\ \[CapitalSigma]\^2\)\)\)\ d\ y
            \)\/\(\@\(2\ \[Pi]\)\ \[CapitalSigma]\) = 
        \(n(y, m, \[CapitalSigma])\)\ d\ y\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`n(y, m, \[CapitalSigma])\)]],
  " is the probability density function for a standard normal distribution \
with mean and standard deviation ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalSigma]\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cut-Off Power Payoffs", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Several cases of interest can be lumped together and based on some \
standard payoff functions. We consider a power of the stock price times a \
step function. Two cases are important. The cut-off power Call and the \
cut-off power Put, with payoffs, respectively,\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B\ S\^\[Lambda]\ \(\[Theta](S - K)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B\ S\^\[Lambda]\ \(\[Theta](K - S)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can input these to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and plot them:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
PowerCall[\[Lambda]_, S_, K_, B_] := B*S^\[Lambda]*UnitStep[S - \
K]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
PowerPut[\[Lambda]_, S_, K_, B_] := B*S^\[Lambda]*UnitStep[K - S]\
\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[PowerCall[0, S, 1, 1], \ {S, \ 0, \ 2}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.588604 [
[.2619 .00222 -9 -9 ]
[.2619 .00222 9 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -9 -9 ]
[.7381 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .13244 -18 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -18 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(0.5)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(1.5)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(0.2)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.4)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(0.6)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(0.8)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(1)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .01472 L
.10458 .01472 L
.14415 .01472 L
.18221 .01472 L
.22272 .01472 L
.26171 .01472 L
.30316 .01472 L
.34309 .01472 L
.3815 .01472 L
.42237 .01472 L
.46172 .01472 L
.48147 .01472 L
.49012 .01472 L
.49468 .01472 L
.49719 .01472 L
.49842 .01472 L
.49955 .01472 L
.50085 .60332 L
.50154 .60332 L
.50226 .60332 L
.50471 .60332 L
.5095 .60332 L
.51896 .60332 L
.53984 .60332 L
.57781 .60332 L
.61824 .60332 L
.65714 .60332 L
.6985 .60332 L
.73835 .60332 L
.77668 .60332 L
.81746 .60332 L
.85673 .60332 L
.89448 .60332 L
.93468 .60332 L
.97337 .60332 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{180.375, 111.375},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003Q0000Ra000`40O003h00OhGoo003QOol00>5o
o`00@7oo0`001Woo0P001Goo100097oo1@008goo1@0017oo0P001Goo10008goo1@001Goo000oOol0
1@00Oomoogoo00001Goo0P002Goo00<007ooOol08goo00<007ooOol09Goo00<007ooOol017oo0P00
2Goo00<007ooOol08Goo00@007ooOol000Eoo`00?goo00D007ooOomoo`00011oo`03001oogoo02=o
o`03001oogoo02Eoo`03001oogoo00moo`03001oogoo029oo`03001oogoo00Eoo`00?goo00D007oo
Oomoo`00011oo`03001oogoo02=oo`03001oogoo02Eoo`03001oogoo00moo`03001oogoo02=oo`03
001oogoo00Aoo`00?goo00D007ooOomoo`0000aoo`@002Ioo`03001oogoo02Eoo`03001oogoo00]o
o`@002Moo`03001oogoo00=oo`00?goo00D007ooOomoo`0000aoo`03001oogoo02Moo`03001oogoo
02Eoo`03001oogoo00]oo`03001oogoo02Qoo`03001oogoo00=oo`00?goo00D007ooOomoo`0000ao
o`03001oogoo02Moo`03001oogoo02Eoo`03001oogoo00]oo`03001oogoo02Aoo`05001oogooOol0
0005Ool0041oo`<000eoo`@002Aoo`<002Eoo`<000eoo`@002Aoo`<000Ioo`006Woo00<007ooOol0
a7oo000JOol00`00Oomoo`34Ool001Yoo`03001oogoo0<Aoo`005Goob`000Goo000JOol00`00Oomo
o`06Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`06Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`06Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`06Ool00`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomo
o`04Ool001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0
I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Ao
o`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol0
01Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000J
Ool00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo
00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03
001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00
Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007oo
Ool0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo
05eoo`03001oogoo06Aoo`000Woo0`001Woo0P0017oo1@0017oo00<007ooOol0GGoo00<007ooOol0
I7oo00000goo001oo`02Ool00`00Oomoo`03Ool20005Ool01000Oomoo`0017oo00<007ooOol0GGoo
00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0;Ool00`00Oomoo`04Ool00`00Oomoo`1M
Ool00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00aoo`03001oogoo00=oo`8005ioo`03
001oogoo06Aoo`0000=oo`00Ool00Woo00<007ooOol03Goo00<007ooOol00Woo00<007ooOol0GGoo
00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`1M
Ool00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Uoo`05001oogooOol00004Ool00`00
Oomoo`1MOol00`00Oomoo`1TOol0009oo`<000eoo`<000Eoo`03001oogoo05eoo`03001oogoo06Ao
o`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol0
01Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000J
Ool00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo
00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03
001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00
Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007oo
Ool0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo
05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo0002Ool30006Ool20006
Ool30004Ool00`00Oomoo`1MOol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00=oo`80
00Moo`03001oogoo00=oo`03001oogoo05eoo`03001oogoo06Aoo`0000=oo`00Ool00Woo00<007oo
Ool02Goo1@0017oo00<007ooOol0GGoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09
Ool01000Oomoo`001Goo0P00GWoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0:Ool0
0`00Ool00005Ool00`00Oomoo`1MOol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Yo
o`03001oo`0000Eoo`03001oogoo05eoo`03001oogoo06Aoo`0000=oo`00Ool00Woo00<007ooOol0
2goo0P001Goo00<007ooOol0GGoo00<007ooOol0I7oo0002Ool3000?Ool00`00Oomoo`03Ool00`00
Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007oo
Ool0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo
05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1M
Ool00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo
00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03
001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00
Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007oo
Ool0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo
06Aoo`000Woo0`001Woo0P001Goo0`001Goo00<007ooOol0GGoo00<007ooOol0I7oo00000goo001o
o`02Ool00`00Oomoo`03Ool20004Ool01@00Oomoogoo000017oo00<007ooOol0GGoo00<007ooOol0
I7oo00000goo001oo`02Ool00`00Oomoo`09Ool01@00Oomoogoo000017oo0P00GWoo00<007ooOol0
I7oo00000goo001oo`02Ool00`00Oomoo`09Ool01@00Oomoogoo000017oo00<007ooOol0GGoo00<0
07ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool40005Ool00`00Oomoo`1MOol00`00Oomo
o`1TOol00003Ool007oo009oo`03001oogoo00Uoo`03001oogoo00Ioo`03001oogoo05eoo`03001o
ogoo06Aoo`0000=oo`00Ool00Woo00<007ooOol02Woo00<007ooOol01Goo00<007ooOol0GGoo00<0
07ooOol0I7oo0002Ool3000>Ool30004Ool00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001o
ogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomo
o`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0
GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eo
o`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol0
0`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<0
07ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001o
ogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomo
o`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`000Woo0`001Woo0P001Goo0`001Goo00<0
07ooOol0GGoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`03Ool20004Ool01@00Oomo
ogoo000017oo00<007ooOol0GGoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool0
1@00Oomoogoo000017oo0P00GWoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool0
1@00Oomoogoo000017oo00<007ooOol0GGoo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomo
o`0:Ool30005Ool00`00Oomoo`1MOol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Uo
o`05001oogooOol00004Ool00`00Oomoo`1MOol00`00Oomoo`1TOol00003Ool007oo009oo`03001o
ogoo00Uoo`05001oogooOol00004Ool00`00Oomoo`1MOol00`00Oomoo`1TOol0009oo`<000eoo`<0
00Eoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000J
Ool00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo
00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03
001oogoo05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00
Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`006Woo00<007oo
Ool0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1MOol00`00Oomoo`1TOol001Yoo`03001oogoo
05eoo`03001oogoo06Aoo`006Woo00<007ooOol0GGoo00<007ooOol0I7oo000JOol00`00Oomoo`1M
Ool00`00Oomoo`1TOol001Yoo`03001oogoo05eoo`03001oogoo06Aoo`004Goo1@0017oo00<007oo
Ool0GGoo00<007ooOol0I7oo000COol00`00Oomoo`04Ool00`00Oomoo`1MOol00`00Oomoo`1TOol0
01=oo`03001oogoo00Aoo`03001oogoo05eoo`03001oogoo06Aoo`004goo00<007ooOol017oo0P00
GWooH@001Woo000COol00`00Oomoo`04Ool00`00Oomoo`34Ool001=oo`03001oogoo00Aoo`03001o
ogoo0<Aoo`004goo00<007ooOol017oo00<007ooOol0a7oo000AOol3003=Ool00>5oo`00\
\>"],
  ImageRangeCache->{{{0, 224.438}, {138.188, 0}} -> {-0.272884, -0.115939, 
  0.0130069, 0.0105228}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[PowerPut[2, S, 1, 1], \ {S, \ 0, \ 2}]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.47619 0.0147151 0.588687 [
[.2619 .00222 -9 -9 ]
[.2619 .00222 9 0 ]
[.5 .00222 -3 -9 ]
[.5 .00222 3 0 ]
[.7381 .00222 -9 -9 ]
[.7381 .00222 9 0 ]
[.97619 .00222 -3 -9 ]
[.97619 .00222 3 0 ]
[.01131 .13245 -18 -4.5 ]
[.01131 .13245 0 4.5 ]
[.01131 .25019 -18 -4.5 ]
[.01131 .25019 0 4.5 ]
[.01131 .36793 -18 -4.5 ]
[.01131 .36793 0 4.5 ]
[.01131 .48566 -18 -4.5 ]
[.01131 .48566 0 4.5 ]
[.01131 .6034 -6 -4.5 ]
[.01131 .6034 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(0.5)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(1)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(1.5)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(2)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13245 m
.03006 .13245 L
s
[(0.2)] .01131 .13245 1 0 Mshowa
.02381 .25019 m
.03006 .25019 L
s
[(0.4)] .01131 .25019 1 0 Mshowa
.02381 .36793 m
.03006 .36793 L
s
[(0.6)] .01131 .36793 1 0 Mshowa
.02381 .48566 m
.03006 .48566 L
s
[(0.8)] .01131 .48566 1 0 Mshowa
.02381 .6034 m
.03006 .6034 L
s
[(1)] .01131 .6034 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10302 m
.02756 .10302 L
s
.02381 .16189 m
.02756 .16189 L
s
.02381 .19132 m
.02756 .19132 L
s
.02381 .22076 m
.02756 .22076 L
s
.02381 .27962 m
.02756 .27962 L
s
.02381 .30906 m
.02756 .30906 L
s
.02381 .33849 m
.02756 .33849 L
s
.02381 .39736 m
.02756 .39736 L
s
.02381 .4268 m
.02756 .4268 L
s
.02381 .45623 m
.02756 .45623 L
s
.02381 .5151 m
.02756 .5151 L
s
.02381 .54453 m
.02756 .54453 L
s
.02381 .57397 m
.02756 .57397 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01473 L
.02729 .01475 L
.02846 .01477 L
.03053 .01483 L
.03279 .01492 L
.03527 .01506 L
.0379 .01523 L
.04262 .01563 L
.04749 .01617 L
.05205 .01679 L
.06244 .01859 L
.07305 .02101 L
.08274 .02373 L
.10458 .03165 L
.12357 .04055 L
.14429 .0524 L
.18493 .08211 L
.22406 .11882 L
.26565 .16655 L
.30571 .22103 L
.34426 .28131 L
.38527 .3539 L
.42475 .43206 L
.46273 .51485 L
.48224 .5603 L
.48785 .57375 L
.49318 .58665 L
.49549 .59231 L
.49795 .59835 L
.49924 .60151 L
.49997 .60332 L
.50065 .01472 L
.50185 .01472 L
.50315 .01472 L
.54429 .01472 L
.58391 .01472 L
.62202 .01472 L
.66258 .01472 L
.70163 .01472 L
.74313 .01472 L
.78311 .01472 L
.82157 .01472 L
.86249 .01472 L
.9019 .01472 L
.93978 .01472 L
.97619 .01472 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{180.375, 111.375},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003Q0000Ra000`40O003h00OhGoo003QOol00>5o
o`00@7oo0`001Woo0P001Goo100097oo1@008goo1@0017oo0P001Goo10008goo1@001Goo000oOol0
1@00Oomoogoo00001Goo0P002Goo00<007ooOol08goo00<007ooOol09Goo00<007ooOol017oo0P00
2Goo00<007ooOol08Goo00@007ooOol000Eoo`00?goo00D007ooOomoo`00011oo`03001oogoo02=o
o`03001oogoo02Eoo`03001oogoo00moo`03001oogoo029oo`03001oogoo00Eoo`00?goo00D007oo
Oomoo`00011oo`03001oogoo02=oo`03001oogoo02Eoo`03001oogoo00moo`03001oogoo02=oo`03
001oogoo00Aoo`00?goo00D007ooOomoo`0000aoo`@002Ioo`03001oogoo02Eoo`03001oogoo00]o
o`@002Moo`03001oogoo00=oo`00?goo00D007ooOomoo`0000aoo`03001oogoo02Moo`03001oogoo
02Eoo`03001oogoo00]oo`03001oogoo02Qoo`03001oogoo00=oo`00?goo00D007ooOomoo`0000ao
o`03001oogoo02Moo`03001oogoo02Eoo`03001oogoo00]oo`03001oogoo02Aoo`05001oogooOol0
0005Ool0041oo`<000eoo`@002Aoo`<002Eoo`<000eoo`@002Aoo`<000Ioo`006Woo00<007ooOol0
a7oo000JOol00`00Oomoo`34Ool001Yoo`03001oogoo0<Aoo`005Goob`000Goo000JOol00`00Oomo
o`02Ool70007Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool0
0`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool0
0`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`06Ool0
0`00Oomoo`07Ool00`00Oomoo`07Ool00`00Oomoo`06Ool00`00Oomoo`07Ool00`00Oomoo`04Ool0
01Yoo`03001oogoo00Uoo`<0055oo`03001oogoo06Aoo`006Woo00<007ooOol037oo0`00CWoo00<0
07ooOol0I7oo000JOol00`00Oomoo`0?Ool2001<Ool00`00Oomoo`1TOol001Yoo`03001oogoo015o
o`8004Yoo`03001oogoo06Aoo`006Woo00<007ooOol04goo00<007ooOol0Agoo00<007ooOol0I7oo
000JOol00`00Oomoo`0DOol00`00Oomoo`16Ool00`00Oomoo`1TOol001Yoo`03001oogoo01Eoo`80
04Ioo`03001oogoo06Aoo`006Woo00<007ooOol05goo00<007ooOol0@goo00<007ooOol0I7oo000J
Ool00`00Oomoo`0HOol00`00Oomoo`12Ool00`00Oomoo`1TOol001Yoo`03001oogoo01Uoo`80049o
o`03001oogoo06Aoo`006Woo00<007ooOol06goo00<007ooOol0?goo00<007ooOol0I7oo000JOol0
0`00Oomoo`0LOol00`00Oomoo`0nOol00`00Oomoo`1TOol001Yoo`03001oogoo01eoo`8003ioo`03
001oogoo06Aoo`006Woo00<007ooOol07goo00<007ooOol0>goo00<007ooOol0I7oo000JOol00`00
Oomoo`0POol00`00Oomoo`0jOol00`00Oomoo`1TOol001Yoo`03001oogoo025oo`03001oogoo03Uo
o`03001oogoo06Aoo`006Woo00<007ooOol08Woo00<007ooOol0>7oo00<007ooOol0I7oo000JOol0
0`00Oomoo`0SOol00`00Oomoo`0gOol00`00Oomoo`1TOol001Yoo`03001oogoo02Aoo`03001oogoo
03Ioo`03001oogoo06Aoo`000Woo0`001Woo0P0017oo1@0017oo00<007ooOol09Goo00<007ooOol0
=Goo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`03Ool20005Ool01000Oomoo`0017oo
00<007ooOol09Goo00<007ooOol0=Goo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0;
Ool00`00Oomoo`04Ool00`00Oomoo`0VOol00`00Oomoo`0dOol00`00Oomoo`1TOol00003Ool007oo
009oo`03001oogoo00aoo`03001oogoo00=oo`8002Qoo`03001oogoo03=oo`03001oogoo06Aoo`00
00=oo`00Ool00Woo00<007ooOol03Goo00<007ooOol00Woo00<007ooOol0:7oo00<007ooOol0<Woo
00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0=Ool00`00Oomoo`02Ool00`00Oomoo`0Y
Ool00`00Oomoo`0aOol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Uoo`05001oogoo
Ool00004Ool00`00Oomoo`0ZOol00`00Oomoo`0`Ool00`00Oomoo`1TOol0009oo`<000eoo`<000Eo
o`03001oogoo02]oo`03001oogoo02moo`03001oogoo06Aoo`006Woo00<007ooOol0;7oo00<007oo
Ool0;Woo00<007ooOol0I7oo000JOol00`00Oomoo`0]Ool00`00Oomoo`0]Ool00`00Oomoo`1TOol0
01Yoo`03001oogoo02ioo`03001oogoo02aoo`03001oogoo06Aoo`006Woo00<007ooOol0;Woo00<0
07ooOol0;7oo00<007ooOol0I7oo000JOol00`00Oomoo`0_Ool00`00Oomoo`0[Ool00`00Oomoo`1T
Ool001Yoo`03001oogoo031oo`03001oogoo02Yoo`03001oogoo06Aoo`006Woo00<007ooOol0<Goo
00<007ooOol0:Goo00<007ooOol0I7oo000JOol00`00Oomoo`0aOol00`00Oomoo`0YOol00`00Oomo
o`1TOol001Yoo`03001oogoo039oo`03001oogoo02Qoo`03001oogoo06Aoo`006Woo00<007ooOol0
<goo00<007ooOol09goo00<007ooOol0I7oo000JOol00`00Oomoo`0cOol00`00Oomoo`0WOol00`00
Oomoo`1TOol001Yoo`03001oogoo03Aoo`03001oogoo02Ioo`03001oogoo06Aoo`006Woo00<007oo
Ool0=Goo00<007ooOol09Goo00<007ooOol0I7oo000JOol00`00Oomoo`0fOol00`00Oomoo`0TOol0
0`00Oomoo`1TOol001Yoo`03001oogoo03Ioo`03001oogoo02Aoo`03001oogoo06Aoo`006Woo00<0
07ooOol0=goo00<007ooOol08goo00<007ooOol0I7oo0002Ool30006Ool20006Ool30004Ool00`00
Oomoo`0gOol00`00Oomoo`0SOol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00=oo`80
00Moo`03001oogoo00=oo`03001oogoo03Qoo`03001oogoo029oo`03001oogoo06Aoo`0000=oo`00
Ool00Woo00<007ooOol02Goo1@0017oo00<007ooOol0>Goo00<007ooOol08Goo00<007ooOol0I7oo
00000goo001oo`02Ool00`00Oomoo`09Ool01000Oomoo`001Goo0P00>Woo00<007ooOol08Goo00<0
07ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`0:Ool00`00Ool00005Ool00`00Oomoo`0jOol0
0`00Oomoo`0POol00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Yoo`03001oo`0000Eo
o`03001oogoo03Yoo`03001oogoo021oo`03001oogoo06Aoo`0000=oo`00Ool00Woo00<007ooOol0
2goo0P001Goo00<007ooOol0>goo00<007ooOol07goo00<007ooOol0I7oo0002Ool3000?Ool00`00
Oomoo`03Ool00`00Oomoo`0lOol00`00Oomoo`0NOol00`00Oomoo`1TOol001Yoo`03001oogoo03ao
o`03001oogoo01ioo`03001oogoo06Aoo`006Woo00<007ooOol0?Goo00<007ooOol07Goo00<007oo
Ool0I7oo000JOol00`00Oomoo`0mOol00`00Oomoo`0MOol00`00Oomoo`1TOol001Yoo`03001oogoo
03ioo`03001oogoo01aoo`03001oogoo06Aoo`006Woo00<007ooOol0?goo00<007ooOol06goo00<0
07ooOol0I7oo000JOol00`00Oomoo`0oOol00`00Oomoo`0KOol00`00Oomoo`1TOol001Yoo`03001o
ogoo041oo`03001oogoo01Yoo`03001oogoo06Aoo`006Woo00<007ooOol0@7oo00<007ooOol06Woo
00<007ooOol0I7oo000JOol00`00Oomoo`11Ool00`00Oomoo`0IOol00`00Oomoo`1TOol001Yoo`03
001oogoo049oo`03001oogoo01Qoo`03001oogoo06Aoo`006Woo00<007ooOol0@Woo00<007ooOol0
67oo00<007ooOol0I7oo000JOol00`00Oomoo`13Ool00`00Oomoo`0GOol00`00Oomoo`1TOol001Yo
o`03001oogoo04=oo`03001oogoo01Moo`03001oogoo06Aoo`006Woo00<007ooOol0A7oo00<007oo
Ool05Woo00<007ooOol0I7oo000JOol00`00Oomoo`15Ool00`00Oomoo`0EOol00`00Oomoo`1TOol0
01Yoo`03001oogoo04Eoo`03001oogoo01Eoo`03001oogoo06Aoo`000Woo0`001Woo0P001Goo0`00
1Goo00<007ooOol0AWoo00<007ooOol057oo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomo
o`03Ool20004Ool01@00Oomoogoo000017oo00<007ooOol0AWoo00<007ooOol057oo00<007ooOol0
I7oo00000goo001oo`02Ool00`00Oomoo`09Ool01@00Oomoogoo000017oo0P00B7oo00<007ooOol0
4goo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool01@00Oomoogoo000017oo00<0
07ooOol0Agoo00<007ooOol04goo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool4
0005Ool00`00Oomoo`18Ool00`00Oomoo`0BOol00`00Oomoo`1TOol00003Ool007oo009oo`03001o
ogoo00Uoo`03001oogoo00Ioo`03001oogoo04Qoo`03001oogoo019oo`03001oogoo06Aoo`0000=o
o`00Ool00Woo00<007ooOol02Woo00<007ooOol01Goo00<007ooOol0BGoo00<007ooOol04Goo00<0
07ooOol0I7oo0002Ool3000>Ool30004Ool00`00Oomoo`19Ool00`00Oomoo`0AOol00`00Oomoo`1T
Ool001Yoo`03001oogoo04Yoo`03001oogoo011oo`03001oogoo06Aoo`006Woo00<007ooOol0Bgoo
00<007ooOol03goo00<007ooOol0I7oo000JOol00`00Oomoo`1;Ool00`00Oomoo`0?Ool00`00Oomo
o`1TOol001Yoo`03001oogoo04aoo`03001oogoo00ioo`03001oogoo06Aoo`006Woo00<007ooOol0
C7oo00<007ooOol03Woo00<007ooOol0I7oo000JOol00`00Oomoo`1=Ool00`00Oomoo`0=Ool00`00
Oomoo`1TOol001Yoo`03001oogoo04eoo`03001oogoo00eoo`03001oogoo06Aoo`006Woo00<007oo
Ool0CWoo00<007ooOol037oo00<007ooOol0I7oo000JOol00`00Oomoo`1>Ool00`00Oomoo`0<Ool0
0`00Oomoo`1TOol001Yoo`03001oogoo04moo`03001oogoo00]oo`03001oogoo06Aoo`006Woo00<0
07ooOol0Cgoo00<007ooOol02goo00<007ooOol0I7oo000JOol00`00Oomoo`1?Ool00`00Oomoo`0;
Ool00`00Oomoo`1TOol001Yoo`03001oogoo051oo`03001oogoo00Yoo`03001oogoo06Aoo`006Woo
00<007ooOol0D7oo00<007ooOol02Woo00<007ooOol0I7oo000JOol00`00Oomoo`1AOol00`00Oomo
o`09Ool00`00Oomoo`1TOol001Yoo`03001oogoo055oo`03001oogoo00Uoo`03001oogoo06Aoo`00
0Woo0`001Woo0P001Goo0`001Goo00<007ooOol0DWoo00<007ooOol027oo00<007ooOol0I7oo0000
0goo001oo`02Ool00`00Oomoo`03Ool20004Ool01@00Oomoogoo000017oo00<007ooOol0DWoo00<0
07ooOol027oo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool01@00Oomoogoo0000
17oo0P00Dgoo00<007ooOol027oo00<007ooOol0I7oo00000goo001oo`02Ool00`00Oomoo`09Ool0
1@00Oomoogoo000017oo00<007ooOol0Dgoo00<007ooOol01goo00<007ooOol0I7oo00000goo001o
o`02Ool00`00Oomoo`0:Ool30005Ool00`00Oomoo`1COol00`00Oomoo`07Ool00`00Oomoo`1TOol0
0003Ool007oo009oo`03001oogoo00Uoo`05001oogooOol00004Ool00`00Oomoo`1DOol00`00Oomo
o`06Ool00`00Oomoo`1TOol00003Ool007oo009oo`03001oogoo00Uoo`05001oogooOol00004Ool0
0`00Oomoo`1DOol00`00Oomoo`06Ool00`00Oomoo`1TOol0009oo`<000eoo`<000Eoo`03001oogoo
05Aoo`03001oogoo00Ioo`03001oogoo06Aoo`006Woo00<007ooOol0EGoo00<007ooOol01Goo00<0
07ooOol0I7oo000JOol00`00Oomoo`1EOol00`00Oomoo`05Ool00`00Oomoo`1TOol001Yoo`03001o
ogoo05Ioo`03001oogoo00Aoo`03001oogoo06Aoo`006Woo00<007ooOol0EWoo00<007ooOol017oo
00<007ooOol0I7oo000JOol00`00Oomoo`1GOol00`00Oomoo`03Ool00`00Oomoo`1TOol001Yoo`03
001oogoo05Moo`03001oogoo00=oo`03001oogoo06Aoo`006Woo00<007ooOol0Egoo00<007ooOol0
0goo00<007ooOol0I7oo000JOol00`00Oomoo`1HOol00`00Oomoo`02Ool00`00Oomoo`1TOol001Yo
o`03001oogoo05Qoo`03001oogoo009oo`03001oogoo06Aoo`006Woo00<007ooOol0FGoo00D007oo
Oomoo`0006Ioo`006Woo00<007ooOol0FGoo00D007ooOomoo`0006Ioo`006Woo00<007ooOol0FWoo
00@007ooOol006Ioo`006Woo00<007ooOol0FWoo00@007ooOol006Ioo`006Woo00<007ooOol0FWoo
00@007ooOol006Ioo`006Woo00<007ooOol0Fgoo00<007oo0000IWoo000AOol50004Ool00`00Oomo
o`1KOol00`00Ool0001VOol001=oo`03001oogoo00Aoo`03001oogoo05aoo`8006Ioo`004goo00<0
07ooOol017oo00<007ooOol0G7oo0P00IWoo000COol00`00Oomoo`04Ool2001NOol00`00Oomoo`1T
Ool001=oo`03001oogoo00Aoo`03001oogoo0<Aoo`004goo00<007ooOol017oo00<007ooOol0a7oo
000COol00`00Oomoo`04Ool00`00Oomoo`34Ool0015oo`<00<eoo`00hGoo0000\
\>"],
  ImageRangeCache->{{{0, 224.438}, {138.188, 0}} -> {-0.273055, -0.115935, 
  0.0130087, 0.0105227}}]
}, Open  ]],

Cell["\<\
Let's look at the value of a \"Call\" of this type. We just have to \
note that\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`S\^\[Lambda] = K\^\[Lambda]\ \[ExponentialE]\^\(\[Lambda]\ y\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
and insert the result into our formula. Using the step function to \
limit the integration range, we get\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ B\ K\^\[Lambda]\ 
      \(\[Integral]\_0
          \%\[Infinity]\( 
                \[ExponentialE]\^\(\[Lambda]\ y - 
                  \((y - m)\)\^2\/\(2\ 
                      \[CapitalSigma]\^2\)\)\/\(\[CapitalSigma]\ 
                \@\(2\ \[Pi]\)\)\) \[DifferentialD]y\)\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to do the work of evaluating this. If you want to do it yourself, \
complete the square in the exponential and use the definition of the \
cumulative normal distribution function to present the answer. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Clear[r, \ y, \ K, \ B, \ m\ , \ \[CapitalSigma], \ \[Lambda]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(value\  = \ 
      Exp[\(-r\)\ \((T - t)\)]\ B\ K^\[Lambda]\ \(Integrate[
            1/\((\[CapitalSigma]\ Sqrt[
                      2\ Pi])\)\ Exp[\[Lambda]\ y\  - \ \((y\  - \ 
                          m)\)^2/\((2\ \[CapitalSigma]^2)\)], \ {y, \ 0, \ 
              Infinity}]\)[\([2]\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(B\ E\^\(\(-r\)\ \((\(-t\) + T)\) + m\ \[Lambda] + \(\[Lambda]\^2\ \
\[CapitalSigma]\^2\)\/2\)\ K\^\[Lambda]\ \@\[CapitalSigma]\^2\ \((1 + Erf[\(m \
+ \[Lambda]\ \[CapitalSigma]\^2\)\/\(\@2\ \@\[CapitalSigma]\^2\)])\)\)\/\(2\ \
\[CapitalSigma]\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(simpvalue\  = \ PowerExpand[value]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2\ B\ E\^\(\(-r\)\ \((\(-t\) + T)\) + m\ \[Lambda] + \(\[Lambda]\^2\ \
\[CapitalSigma]\^2\)\/2\)\ K\^\[Lambda]\ \((1 + 
          Erf[\(m + \[Lambda]\ \[CapitalSigma]\^2\)\/\(\@2\ \
\[CapitalSigma]\)])\)\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now we load a Package where the cumulative normal distribution function is \
defined. See Chapter 7 for a discussion of this Package. For now we just want \
to relate the Error function ",
  StyleBox["Erf",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " to the more familiar \"N\" function used in derivatives modelling. In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " \"N\" is used for numerical evaluation, we shall use the term ",
  StyleBox["Norm",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " for the cumulative normal distribution. This is defined in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " within a Package defined in Chapter 7 - we just look at the definition \
for now:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]; \nNorm[z]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2\ \((1 + Erf[z\/\@2])\)\)], "Output"]
}, Open  ]],

Cell["The value of the cut-off power Call is then just", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B\ \(K\^\[Lambda]\) 
      \[ExponentialE]\^\(\(\[CapitalSigma]\^2\ \[Lambda]\^2\)\/2 + 
          m\ \[Lambda] - r\ \((T - t)\)\)\ 
      \(Norm(m\/\[CapitalSigma] + \[Lambda]\ \[CapitalSigma])\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Similarly the value of the cut-off power Put is given by", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B\ \(K\^\[Lambda]\) 
      \[ExponentialE]\^\(\(\[CapitalSigma]\^2\ \[Lambda]\^2\)\/2 + 
          m\ \[Lambda] - r\ \((T - t)\)\)\ 
      \(Norm(\(-\(m\/\[CapitalSigma]\)\) - \[Lambda]\ \[CapitalSigma])\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can do a consistency check by adding together the Call and the Put - it \
is left as an exercise for you to see that the solutions add to the one we \
have given earlier for an option with payoff ",
  Cell[BoxData[
      \(TraditionalForm\`S\^\[Lambda]\)]],
  "."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["The Irritating Standard Notation!", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It has become conventional to use a rather silly notation for these \
expressions when ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = 0, 1\)]],
  ". When ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = 0\)]],
  " we define"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`d\_2 = \(m\/\[CapitalSigma] = 
        \(log(S\/K) + 
            \((r - q - \[Sigma]\^2\/2)\)\ \((T - t)\)\)\/\(\[Sigma]\ 
            \@\(T - t\)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda] = 1\)]],
  " definition is"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`d\_1 = \(m\/\[CapitalSigma] + \[CapitalSigma] = 
        \(log(S\/K) + 
            \((r - q + \[Sigma]\^2\/2)\)\ \((T - t)\)\)\/\(\[Sigma]\ 
            \@\(T - t\)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Binary Options", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "These are given by the cut-off power payoffs with ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = \ 0\)]],
  ". The Binary Call therefore has value"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(Norm(d\_2)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The Binary Put therefore has value", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(Norm(\(-d\_2\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Put-Call parity for binaries takes the form that the sum of a \
Binary Call and a Binary Put with the same strike is just the value of a cash \
payout at expiry:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Vanilla Calls and Puts", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
These are simple linear combinations of the cut-off power options. \
For a vanilla Call we have the payoff at expiry in the form\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`Max[S\_T - K, 0] = 
      S\_T\ \(\[Theta](S\_T - K)\) - K\ \(\[Theta](S\_T - K)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["whereas for a Put we have", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`Max[K - S\_T, 0] = 
      K\ \(\[Theta](K - S\_T)\) - S\_T\ \(\[Theta](K - S\_T)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The value of a Call is therefore", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`K\ \[ExponentialE]\^\(\[CapitalSigma]\^2\/2 + \ m\  - r\ \((T - t)\)\)\ 
        \(Norm(m\/\[CapitalSigma] + \[CapitalSigma])\) - \ 
      K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ 
        \(Norm(m\/\[CapitalSigma])\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Simplification of the exponentials reduces this to ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`C\  = \ S\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\)\ \(Norm(d\_1)\) - \ 
        K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(Norm(d\_2)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["For a Put, we obtain, similarly, ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`P\  = \ \ 
      K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(Norm(\(-d\_2\))\)\  - \ 
        S\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\)\ \(Norm(\(-d\_1\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This time Call-Put parity takes the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`C\  - P\  = \ 
      S\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\) - \ 
        K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that although we shall only consider Binaries and vanilla Calls/Puts \
in this particular form, ",
  StyleBox[
  "the methods we have developed allow the valuation of other exotics based \
on any linear combination of cut-off power options. ",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Simple Barrier Options", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now suppose that there is a special value of ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " on which the option value is set to a given value. If we set this value \
to zero, we obtain a standard \"knock-out\" option. If we prescribe a \
\"rebate\" function if the asset price reaches this point, we need to impose \
a boundary condition along the barrier. Both of these cases fit into our \
model of boundary conditions at a finite point. Let's consider the case of \
\"down-and-out\" options, where the rebate at the lower barrier, ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ H\)]],
  ", is zero and there is a given payoff function defined for ",
  Cell[BoxData[
      \(TraditionalForm\`S\_T\  > H\)]],
  ". We set "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`H = K\ \[ExponentialE]\^a\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`a = log(H\/K)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Then completing the square in the exponential functions gives \
us\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, t) = \ 
      \(\[ExponentialE]\^\(\(-k\_1\) \[Tau]\)\) 
          \(\[Integral]\_a
              \%\[Infinity]\(\(\( P(K\ \[ExponentialE]\^y)\)\ \ 
                    \[ExponentialE]\^\(-
                        \(\((x - y + \[Tau](k\_2 - 1))\)\^2\/\(4\ 
                              \[Tau]\)\)\)\)\/\@\(4\ \[Pi]\ \[Tau]\)\) 
              \[DifferentialD]y\)\ \n\ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \ \  - 
        \ \ \(\[ExponentialE]\^\(\(-k\_1\) \[Tau] + 
                \((a - x)\) \((k\_2 - 1)\)\)\) 
          \(\[Integral]\_a
              \%\[Infinity]\(\(\( P(K\ \[ExponentialE]\^y)\)\ \ 
                    \[ExponentialE]\^\(-
                        \(\((2  a - x - y + \[Tau](k\_2 - 1))\)\^2\/\(4\ 
                              \[Tau]\)\)\)\)\/\@\(4\ \[Pi]\ \[Tau]\)\) 
              \[DifferentialD]y\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If the option is such that the barrier is below the strike, so that ",
  Cell[BoxData[
      \(TraditionalForm\`a\  < \ 0\)]],
  ", then the first of these integrals is the solution in the absence of the \
barrier. The second term is then given by the first term with the corrections\
\n\n(a) replace ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " by ",
  Cell[BoxData[
      \(TraditionalForm\`2  a\  - \ x\)]],
  " within the solution,\n(b) then multiply the solution by "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\((a - x)\)\ \((k\_2 - 1)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In terms of standard financial variables, this just says that we \
make the replacement\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`log(S\/K) \[Rule] 2\ \(log(H\/K)\) - log(S\/K) = 
      log(H\^2\/\(K\ S\))\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["equivalent to ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S \[Rule] H\^2\/S\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and multiply by ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[ExponentialE]\^\(\((log(H\/K) - log(S\/K))\)\ \((k\_2 - 1)\)\)\  = \ 
      \(\((H\/S)\)\^\(k\_2 - 1\) = 
        \((H\/S)\)\^\(\(2\ \((r - q)\)\)\/\[Sigma]\^2 - 1\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For example, for a down-and-out call, with ",
  Cell[BoxData[
      \(TraditionalForm\`H < K\)]],
  ", we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`C\_do\  = \ 
      S\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\)\ \(Norm(d\_1)\) - \ 
        K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \(Norm(d\_2)\)\ \n
        \ \ \ \ \ \ \ \ \ \ \ \ \ \  - \ 
        \((H\/S)\)\^\(\(2\ \((r - q)\)\)\/\[Sigma]\^2 - 1\)\ 
          \((\(H\^2\ \[ExponentialE]\^\(\(-q\)\ \((T - t)\)\)\ 
                  \(Norm(y)\)\)\/S - 
              K\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ 
                \(Norm(y\  - \ \[Sigma] \@\( T - t\))\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`y = \(log(H\^2\/\(K\ S\)) + 
          \((r - q + \[Sigma]\^2\/2)\)\ \((T - t)\)\)\/\(\[Sigma]\ 
          \@\(T - t\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A host of variants of this case can be considered:\n\n(a) \"in\" variants, \
defined by in+out = Vanilla;\n(b) Put variants;\n(c) cases where ",
  Cell[BoxData[
      \(TraditionalForm\`H > K\)]],
  ";\n(d) the payment of a rebate when the barrier is reached.\n\nWe shall \
not go into a detailed analysis of the cases (a), (b), (c), each of which can \
be obtained by a minor modification of the arguments given. All the resulting \
formulae are spelt out in Chapter 8, in any case. However, we do need to \
explore case (d) - the payment of a rebate on knock-out."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Rebate Management", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The introduction of a rebate requires some careful integral analysis. \
Suppose that we specify a boundary condition ",
  Cell[BoxData[
      \(TraditionalForm\`u\  = \ g(\[Tau])\)]],
  " along the line ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  ". Then in addition to the solution we have given, there is an extra \
contribution ",
  Cell[BoxData[
      \(TraditionalForm\`u\_R\)]],
  " given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u\_R[x, \[Tau]] = 
      \[Integral]\_0\%\[Tau]\(\( g(s)\)\ \(q(\[Tau] - s)\)\) 
          \[DifferentialD]s\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`q(t) = \(1\/\[Pi]\) 
        \(\[Integral]\_0
            \%\[Infinity]\( \[ExponentialE]\^\(\(-p\)\ t\)\ 
              \(sin(\@p\ \((x - a)\))\)\) \[DifferentialD]p\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Suppose that we consider rebate functions of the general form\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`option\ value\  = \ 
      \(R(\[Tau]) = R\_0\ \[ExponentialE]\^\(\(-\[Mu]\)\ \[Tau]\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "along the line ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ a\)]],
  ". Then ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  " has the form"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`g(\[Tau]) = A\ \[ExponentialE]\^\(B\ \[Tau]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["where", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`A = \(R\_0\ \[ExponentialE]\^\(1\/2\ \((k\_2 - 1)\)\ a\) = 
        R\_0\ \((H\/K)\)\^\(1\/2\ \((k\_2 - 1)\)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`B = 1\/4\ \((k\_2 - 1)\)\^2 + k\_1 - \[Mu]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and we assume that ",
  Cell[BoxData[
      \(TraditionalForm\`B > 0\)]],
  ". Then the rebate contribution to ",
  Cell[BoxData[
      \(TraditionalForm\`u\)]],
  " is given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u\_R = A\/\[Pi]\ 
        \(\[Integral]\_0
            \%\[Infinity]\(\[Integral]\_0\%\[Tau]\( sin(\@p\ \((x - a)\))\)\ 
              \(\[ExponentialE]\^\(\((B + p)\)\ s - p\ \[Tau]\)\) 
              \[DifferentialD]s \[DifferentialD]p\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The ",
  Cell[BoxData[
      \(TraditionalForm\`s\)]],
  " integral can be done, leaving"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u\_R = A\/\[Pi]\ 
        \(\[Integral]\_0
            \%\[Infinity]\(\(\( sin(\@p\ \((x - a)\))\)\ 
                  \((\[ExponentialE]\^\(B\ \[Tau]\) - 
                      \[ExponentialE]\^\(\(-p\)\ \[Tau]\))\)\)\/\(B + p\)\) 
            \[DifferentialD]p\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "There are two terms in this integral. The first is given, for ",
  Cell[BoxData[
      \(TraditionalForm\`x > a\)]],
  ", by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(\(A\ \)\/\[Pi]\) \(\[ExponentialE]\^\(B\ \[Tau]\)\) 
        \(\[Integral]\_0
            \%\[Infinity]\(\( sin(\@p\ \((x - a)\))\)\/\(B + p\)\) 
            \[DifferentialD]p\)\  = \ 
      A\ \[ExponentialE]\^\(B\ \[Tau]\)\ 
        \[ExponentialE]\^\(\(-\@B\)\ \((x - a)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This can be verified with ",
  StyleBox["Mathematica:",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[
      Sin[Sqrt[p]\ \((x - a)\)]/\((B + p)\), \ {p, \ 0, \ Infinity}]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"If", "[", 
      RowBox[{\(Im[a - x] == 0 && Arg[B] \[NotEqual] \[Pi]\), 
        ",", \(\(-E\^\(-\@\(B\ \((a - x)\)\^2\)\)\)\ \[Pi]\ Sign[a - x]\), 
        ",", 
        RowBox[{
          SubsuperscriptBox["\[Integral]", "0", 
            InterpretationBox["\[Infinity]",
              DirectedInfinity[ 
              1]]], \(\(Sin[\@p\ \((\(-a\) + x)\)]\/\(B + 
                  p\)\) \[DifferentialD]p\)}]}], "]"}]], "Output"]
}, Open  ]],

Cell["\<\
The second term, with a change of variables, can be written as\
\>",
   "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\(\(2  A\ \)\/\[Pi]\)\) 
        \(\[Integral]\_0
            \%\[Infinity]\(\( p\ \(sin(p\ \((x - a)\))\)\ 
                  \[ExponentialE]\^\(\(-p\^2\)\ \[Tau]\)\)\/\(p\^2 + B\)\) 
            \[DifferentialD]p\)\ \n = 
      \(\(-\(\(A\ \)\/\[Pi]\)\) 
          \(\[Integral]\_\(-
                \[Infinity]\)\%\[Infinity]\(\( p\ \(sin(p\ \((x - a)\))\)\ 
                    \[ExponentialE]\^\(\(-p\^2\)\ \[Tau]\)\)\/\(p\^2 + B\)\) 
              \[DifferentialD]p\)\  = \ \(-\(\(A\ \)\/\[Pi]\)\)\ Im[D]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`D\)]],
  " is the Fourier integral"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[Integral]\_\(-
          \[Infinity]\)\%\[Infinity]\(\( 
                \[ExponentialE]\^\(\[ImaginaryI]\ p\ \((x - a)\) - 
                  p\^2\ \[Tau]\)\ p\)\/\(p\^2 + B\)\) \[DifferentialD]p\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This can now be evaluated as a convolution integral. The two terms \
in the convolution product are given by\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(InverseFourierTransform[Exp[\(-p^2\)\ \[Tau]], \ p, \ y, \ 
      FourierFrequencyConstant\  -> \ \(-1\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(-\(y\^2\/\(4\ \[Tau]\)\)\)\/\(2\ \@\[Pi]\ \@\[Tau]\)\)], "Output"]
}, Open  ]],

Cell[" and", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(InverseFourierTransform[p/\((p^2\  + \ B)\), \ p, \ y, \ 
      FourierFrequencyConstant\  -> \ \(-1\)]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-\(1\/2\)\)\ I\ E\^\(\(-\@B\)\ Abs[y]\)\ Sign[y]\)], "Output"]
}, Open  ]],

Cell["\<\
The form of this last term requires that we divide the convolution \
integration region into two (intermediate results are not shown):\
\>", "Text",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(piecea\  = \ \(Integrate[
          Exp[\(-\((x - a - y)\)^2\)/\((4\ \[Tau])\)\  - \ 
              Sqrt[B]\ y], \ {y, \ 0, \ Infinity}]\)[\([2]\)]; \)\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(pieceb\  = \ \(-\(Integrate[
            Exp[\(-\((x - a - y)\)^2\)/\((4\ \[Tau])\)\  + \ 
                Sqrt[B]\ y], \ {y, \ \(-Infinity\), \ 
              0}]\)[\([2]\)]\); \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The imaginary part of the convolution integral is therefore given \
by\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(ImD\  = \ 
      Simplify[2\ Pi/\((4\ Sqrt[Pi\ \[Tau]])\) \((piecea\  + \ 
              pieceb)\)]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Adding up all the pieces, the total contribution from the rebate to \
the solution of the heat equation is therefore\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(u\_R = 
      Simplify[\(-A\)/Pi\ ImD + \ 
          A*Exp[B*\[Tau]]*Exp[\(-Sqrt[B]\)*\((x\  - \ a)\)]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(1\/2\ A\ E\^\(\(-\@B\)\ \((a + x - \@B\ \[Tau])\)\)\ \((E\^\(2\ a\ \
\@B\) + E\^\(2\ \@B\ x\) + 
          E\^\(2\ \@B\ x\)\ Erf[\(a - x - 2\ \@B\ \[Tau]\)\/\(2\ \@\[Tau]\)] \
+ E\^\(2\ a\ \@B\)\ Erf[\(a - x + 2\ \@B\ \[Tau]\)\/\(2\ \@\[Tau]\)])\)\)], 
  "Output"]
}, Open  ]],

Cell["\<\
This rebate function looks a little complicated, so we had better \
check it - first the boundary condition:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[u\_R /. \ x\  -> \ a]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(A\ E\^\(B\ \[Tau]\)\)], "Output"]
}, Open  ]],

Cell["We also check that the diffusion equation is satisfied:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[D[u\_R, \ {x, \ 2}] - \ D[u\_R, \ \[Tau]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Finally we need to transform back to financial variables, and straighten \
out our notation. We do this in two steps. First we convert from ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " variables to ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " variables and replace the Error functions by the cumulative normal \
distribution. The rebate contribution to the option value is then:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`R\_0\ \[ExponentialE]\^\(\(-\[Mu]\)\ \[Tau]\)\ 
      \((H\/S)\)\^\(1\/2\ \((k\_2 - 1)\)\)\ 
      \((\((S\/H)\)\^\(\@B\)\ 
            \(Norm(\(log(H\/S) - 2\ \@B\ \[Tau]\)\/\@\(2\ \[Tau]\))\) + 
          \((H\/S)\)\^\(\@B\)\ 
            \(Norm(\(log(H\/S) + 2\ \@B\ \[Tau]\)\/\@\(2\ \[Tau]\))\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Next, we re-parametrize the rebate by setting ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\  = \ 2  \[Alpha]/\[Sigma]\^2\)]],
  ", so that the rebate is written as"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`R\_0\ \[ExponentialE]\^\(\(-\[Alpha]\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Then", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B = 1\/4\ \((k\_2 - 1)\)\^2 + k\_1 - \ 2  \[Alpha]/\[Sigma]\^2\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For general ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\)]],
  ", and hence general ",
  Cell[BoxData[
      \(TraditionalForm\`B > 0\)]],
  ", the rebate is given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`R\_0\ \[ExponentialE]\^\(\(-\[Alpha]\)\ \((T - t)\)\)\ \
\((H\/S)\)\^\(1\/2\ \((\(2\ \((r - q)\)\)\/\[Sigma]\^2 - 1)\)\)\ \
\((\((S\/H)\)\^\(\@B\)\ \(Norm(\(log(H\/S) - \ \(\@B\) \[Sigma]\^2\ \((T - t)\
\)\)\/\(\[Sigma]\ \@\(T - t\)\))\)\n\t\t + \((H\/S)\)\^\(\@B\)\ \
\(Norm(\(log(H\/S) + \ \@B\ \[Sigma]\^2\ \((T - t)\)\)\/\(\[Sigma]\ \@\(T - t\
\)\))\))\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "If the rebate takes the form of a cash payment ",
  Cell[BoxData[
      \(TraditionalForm\`R\_0\)]],
  " at maturity, the boundary condition is the discounted value of this, so \
that ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\  = \ r\)]],
  ". Then this expressions simplifies considerably, since"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(TraditionalForm\`\@B = \(1\/2\ \((k\_2 - 1)\) = \ 
        1\/2\ \((\(2\ \((r - q)\)\)\/\[Sigma]\^2 - 1)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["The rebate is then", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[BoxData[
    \(TraditionalForm\`R\_0\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\ \((\ 
        Norm(\(log(H\/S) - \ \ \((r - q - \ \[Sigma]\^2/2\ )\)\ \((T - t)\)\)\
\/\(\[Sigma]\ \@\(T - t\)\))\n\t + \ \((H\/S)\)\^\((\(2\ \((r - q)\)\)\/\
\[Sigma]\^2 - 1)\)\ \(Norm(\(log(H\/S) + \ \((r - q - \ \[Sigma]\^2/2\ )\) \
\((T - t)\)\)\/\(\[Sigma]\ \@\(T - t\)\))\))\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "Another important case is when the cash payment is paid when the barrier \
is reached - the \"cash-at-hit\" case. Then ",
  Cell[BoxData[
      \(TraditionalForm\`\[Alpha]\  = \ 0\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " is more complicated, but our model still applies. This is the case \
considered in Chapter 8, where references to other approaches to the \
derivation are given. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{

Cell["Remarks on the Double Barrier", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
We can analyse the double barrier by similar methods, using the \
infinite sum of Green's functions we have derived. This leads quickly to one \
of the results presented by Kunitomo and Ikeda (1992), who derived their \
result within a probabilistic framework. It is an interesting exercise, which \
we leave to the reader, to spell out the result based on an infinite family \
of image points. Our detailed analysis of the double barrier, given in \
Chapter 12, will use, instead, the closed-form solution given by Geman and \
Yor (1996) based on Laplace transform methods.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Financial Analogue of an Impedance Boundary Condition?", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[TextData[{
  "It should be by now clear that we can go on combining various techniques \
in partial differential equation theory to construct solutions for various \
initial (\"payoff\") and boundary (\"barrier\") conditions. When we run out \
of analytical steam, we can go to numerical methods. At this point, we have \
constructed financial interpretations of most of the classical solutions of \
the diffusion equation. People more interested in finance might object that \
this process is the wrong way around - we should really define problems of \
interest in finance and then develop the mathematics to solve them. However, \
it is also important to point out that most of the ",
  StyleBox["basic",
    FontSlant->"Italic"],
  " option-pricing problems were solved mathematically by people studying the \
heat equation about a hundred years ago. It is then a matter of applying a \
rather trivial change of variables to deduce many solutions to the \
Black-Scholes PDE. Some of these solutions have only been \"discovered\" very \
recently, using relatively complicated arguments based on probability and the \
risk-neutral expectation model. Of course, the translation of problems in \
finance to the state in which they can be solved by such elementary methods \
is an art in itself. \n\nWe have almost used up our family of classical \
solutions to the diffusion or heat equation. We have one left for which we \
have not yet constructed a financial interpretation - the solution to the \
problem with an impedance boundary condition (IBC). Problems similar to the \
one we have described were discussed as long ago as 1912, by Riemann, as \
discussed by Jeffreys and Jeffreys (1946), which contains a discussion of \
many methods of solving the diffusion problem.\n\nThe mapping of an important \
option-pricing problem into the diffusion equation with IBC was, so far as \
the author is aware, first given by Wilmott ",
  StyleBox["et al ",
    FontSlant->"Italic"],
  "(1993), and is based on an analysis of certain types of Lookback option. \
As outlined in Chapter 3, a path-dependency variable is added in the form"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True],

Cell[BoxData[
    \(TraditionalForm
    \`J\_n = \((
          \[Integral]\_0\%t\(\( f(S(\[Tau]), \[Tau])\)\^n\) 
              \[DifferentialD]\[Tau])\)\^\(1/n\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "for various choices about ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  ". The case of interest here is given by considering ",
  Cell[BoxData[
      \(TraditionalForm\`n \[Rule] \[Infinity]\)]],
  ", with ",
  Cell[BoxData[
      \(TraditionalForm\`f\  = \ S\)]],
  ", so that ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  " represents the maximum attained along the path. Recall that suitable \
boundary conditions involving ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  " must be added, and the payoff needs to be written in terms of it also. \
When the limit is taken the PDE contains no reference to ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]V\), \(\[PartialD]t\),
            MultilineFunction->None], "+", 
          RowBox[{\(S \((r - q)\)\), " ", 
            FractionBox[\(\[PartialD]V\), \(\[PartialD]S\),
              MultilineFunction->None]}], "+", 
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(S\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 V\), \(\[PartialD]S\^2\),
              MultilineFunction->None]}], "-", \(r\ V\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The quantity ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  " is the maximum attained on the path, and appears in the boundary \
conditions and the payoff function. Let's take a closer look at the case of a \
Lookback Put. The analysis given here borrows heavily from the presentation \
by Wilmott ",
  StyleBox["et al ",
    FontSlant->"Italic"],
  "(1993), but also shows explicitly how to use the Green's function for the \
impedance boundary condition to solve the problem. This is one approach to \
exercise 1 in Chapter 12 of Wilmott ",
  StyleBox["et al ",
    FontSlant->"Italic"],
  "(1993)! The solution can be arrived at by some cunning guesswork, but we \
shall proceed by formally mapping the financial problem into the problem for \
the heat equation that we have already solved, and directly deriving the \
solution."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The initial condition for a Lookback Put is", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`P(S, J, T) = Max[J - S, 0]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The boundary conditions are, on ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ 0\)]],
  ","
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`P(0, J, t) = J\ \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "On ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ J\)]],
  ", we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]P\), \(\[PartialD]J\),
          MultilineFunction->None], "=", "0"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Now we construct a similarity solution of the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`z = S\/J\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`P(S, J, t) = J\ \(W(z, \ t)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The Black-Scholes equation becomes", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        RowBox[{
          FractionBox[\(\[PartialD]W\), \(\[PartialD]t\),
            MultilineFunction->None], "+", 
          RowBox[{\(z \((r - q)\)\), " ", 
            FractionBox[\(\[PartialD]W\), \(\[PartialD]z\),
              MultilineFunction->None]}], "+", 
          RowBox[{\(1\/2\), " ", \(\[Sigma]\^2\), " ", \(z\^2\), " ", 
            FractionBox[\(\[PartialD]\^2 W\), \(\[PartialD]z\^2\),
              MultilineFunction->None]}], "-", \(r\ W\)}], "=", " ", "0"}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "on the region ",
  Cell[BoxData[
      \(TraditionalForm\`0 < z < 1\)]],
  ". The boundary and initial conditions are transformed according to"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`W(z, T) = Max[1 - z, 0]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`W(0, t) = \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now, on ",
  Cell[BoxData[
      \(TraditionalForm\`z\  = \ 1\)]],
  ", we have"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]W\), \(\[PartialD]z\),
          MultilineFunction->None], "=", "W"}], TraditionalForm]], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
So we see the IBC emerging. We now transform to the standard \
coordinates for the heat equation.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`W(z, t) = \ 
      \[ExponentialE]\^\(\(-\(1\/2\)\)\ \((k\_2 - 1)\)\ x - 
            \((1\/4\ \((k\_2 - 1)\)\^2 + k\_1)\)\ \[Tau]\)\ 
        \(u(x, \[Tau])\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  " is as usual, but "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`x = log(z)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["We only have to consider the range", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(-\[Infinity]\) < x < 0\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "since the \"option\" ",
  StyleBox["will",
    FontSlant->"Italic"],
  " be exercised, and in that range the initial condition is"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u = \[ExponentialE]\^\(1\/2\ \((k\_2 - 1)\)\ x\)\ 
        \((1 - \[ExponentialE]\^x)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The impedance boundary condition at ",
  Cell[BoxData[
      \(TraditionalForm\`x\  = \ 0\)]],
  " is now given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]u\), \(\[PartialD]x\),
          MultilineFunction->None], "=", \(1\/2\ \((k\_2 + 1)\)\ u\)}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Thus we have mapped the Lookback Put problem into our IBC model \
with \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Alpha] = 1\/2\ \((k\_2 + 1)\)\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to do the work for us. First the payoff in log variables:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["f[y_] := Exp[1/2*(Subscript[k, 2] - 1)*y]*(1 - Exp[y])", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We are to integrate this against the sum of these four functions ",
  StyleBox["mone",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " etc.:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Clear[\[Alpha]];
aux[x_, y_, \[Tau]_, \[Alpha]_] := Exp[\[Alpha] (x+y+\[Alpha]*\[Tau])];
nasty[x_, y_, \[Tau]_, \[Alpha]_] := Erf[(-x - y - \
2*\[Alpha]*\[Tau])/(2*Sqrt[\[Tau]])];
mone[x_, y_, \[Tau]_, \[Alpha]_] := 1/(E^((x - \
y)^2/(4*\[Tau]))*2*Sqrt[Pi]*Sqrt[\[Tau]]); 
mtwo[x_, y_, \[Tau]_, \[Alpha]_] := 1/(E^((x + \
y)^2/(4*\[Tau]))*2*Sqrt[Pi]*Sqrt[\[Tau]]); 
mthree[x_, y_, \[Tau]_, \[Alpha]_] := \[Alpha]*aux[x,y,\[Tau],\[Alpha]]; 
mfour[x_, y_, \[Tau]_, \[Alpha]_] := \
-aux[x,y,\[Tau],\[Alpha]]*\[Alpha]*nasty[x, y, \[Tau], \[Alpha]]; \
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Here is a reminder of the full fundamental solution for the \
diffusion equation with an IBC:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(M[x_, \ y_, \ \[Tau]_, \ \[Alpha]_]\  := \ 
      mone[x, y, \[Tau], \ \[Alpha]] + mtwo[x, y, \[Tau], \ \[Alpha]] + 
        mthree[x, y, \[Tau], \ \[Alpha]] + 
        mfour[x, y, \[Tau], \ \[Alpha]]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To isolate any problems, we split the integral into eight pieces:\
\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(fone[t_]\  := \ Exp[1/2*\((Subscript[k, \ 2]\  - \ 1)\)*y]; \n
    ftwo[t_]\  := \ \(-Exp[1/2*\((Subscript[k, \ 2]\  + 1)\)*y]\); \)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Six of them evaluate immediately into simple functions of types we \
have already encountered:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(\[Alpha]\  = \ 1/2 \((k\_2 + 1)\); \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(int[
        1]\  = \ \n\(Integrate[
          fone[y]*mone[x, y, \[Tau], \[Alpha]], {y, \(-Infinity\), 
            0}]\)[\([2]\)]; \n
    int[2]\  = \ \(Integrate[
          ftwo[y]*mone[x, y, \[Tau], \[Alpha]], \ {y, \(-Infinity\), \ 
            0}]\)[\([2]\)]; \n
    int[3]\  = \ \(Integrate[
          fone[y]*mtwo[x, y, \[Tau], \[Alpha]], \ {y, \(-Infinity\), \ 
            0}]\)[\([2]\)]; \n
    int[4]\  = \ \(Integrate[
          ftwo[y]*mtwo[x, y, \[Tau], \[Alpha]], \ {y, \(-Infinity\), \ 
            0}]\)[\([2]\)]; \n
    int[5]\  = \ \(Integrate[
          fone[y]*mthree[x, y, \[Tau], \ \[Alpha]], \ {y, \ \(-Infinity\), \ 
            0}]\)[\([2]\)]; \n
    int[6]\  = \ \(Integrate[
          ftwo[y]*mthree[x, y, \[Tau], \ \[Alpha]], \ {y, \ \(-Infinity\), \ 
            0}]\)[\([2]\)]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The Error function fools the symbolic integrator at first, e.g. (similarly \
for ",
  StyleBox["int[8]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ")"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(int[7]\  = \ 
      Integrate[
        fone[y]*mfour[x, y, \[Tau], \ \[Alpha]], \ {y, \ \(-Infinity\), \ 
          0}]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{\(-\[Alpha]\), " ", 
      RowBox[{
        SubsuperscriptBox["\[Integral]", 
          InterpretationBox[\(-\[Infinity]\),
            DirectedInfinity[ -1]], 
          "0"], \(\(E\^\(\[Alpha]\ \((x + y + \[Alpha]\ \[Tau])\) + 1\/2\ y\ \
\((\(-1\) + k\_2)\)\)\ Erf[\(\(-x\) - y - 2\ \[Alpha]\ \[Tau]\)\/\(2\ \
\@\[Tau]\)]\) \[DifferentialD]y\)}]}]], "Output"]
}, Open  ]],

Cell["\<\
So we integrate by parts semi-manually (you can check that there is \
no contribution from the lower limit at minus infinity):\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(gone[y_]\  = \ Integrate[fone[y]*aux[x, y, \[Tau], \ \[Alpha]], \ y]; \n
    gtwo[y_]\  = \ Integrate[ftwo[y]*aux[x, y, \[Tau], \ \[Alpha]], \ y]; \n
    dmfour[x_, \ y_, \ \[Tau]_, \ \[Alpha]_]\  = \ 
      D[\(-\[Alpha]\)*nasty[x, y, \[Tau], \ \[Alpha]], \ y]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(int[
        7]\  = \(-\[Alpha]\)*gone[0]*
          nasty[x, 0, \[Tau], \ \[Alpha]]\  - \ \(Integrate[
            gone[y]*dmfour[x, 
                y, \[Tau], \ \[Alpha]], \ {y, \ \(-Infinity\), \ 
              0}]\)[\([2]\)]; \n
    int[8]\  = \(-\[Alpha]\)*gtwo[0]*
          nasty[x, 0, \[Tau], \ \[Alpha]]\ \  - \ \(Integrate[
            gtwo[y]*dmfour[x, 
                y, \[Tau], \ \[Alpha]], \ {y, \ \(-Infinity\), \ 
              0}]\)[\([2]\)]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To get the result we add them all up and normalize for the \
valuation. On the way we check that the diffusion equation and IBC remain \
satisfied:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(v\  = \ Simplify[Sum[int[aa], \ {aa, \ 1, \ 8}]]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[D[v, \ {x, \ 2}]\  - \ D[v, \ \[Tau]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Simplify[D[v, \ x]\  - \ \[Alpha]\ v]\  /. \ {x\  -> \ 0}]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell["Normalizing:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(w\  = \ 
      Simplify[Exp[\(-\((1/2)\)\)*\((k\_2\  - \ 1)\)*
                x\  - \ \((1/4*\((k\_2\  - \ 1)\)^2\  + \ k\_1)\)*\[Tau]]*
          v]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can simplify the presentation of the result somewhat if we set the \
yield to zero, so that ",
  Cell[BoxData[
      \(TraditionalForm\`k\_2 = \ k\_1\)]],
  ". Finally, we multiply back up by ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  ":"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(simpw\  = \ 
      Simplify[ExpandAll[PowerExpand[w\  /. \ k\_2 -> \ k\_1]]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(1\/\(2\ k\_1\)\) \((E\^\(x - \((x + \[Tau])\)\ k\_1\)\ \((\(-1\) + 
                E\^\(\((x + \[Tau])\)\ k\_1\) - 
                Erf[\(x + \[Tau] - \[Tau]\ k\_1\)\/\(2\ \@\[Tau]\)] + 
                E\^\(\((x + \[Tau])\)\ k\_1\)\ Erf[\(x + \[Tau] + \[Tau]\ \
k\_1\)\/\(2\ \@\[Tau]\)])\) + \((\(-E\^x\) + E\^\(\(-\[Tau]\)\ k\_1\) - 
                E\^\(\(-\[Tau]\)\ k\_1\)\ Erf[\(x - \[Tau] + \[Tau]\ k\_1\)\/\
\(2\ \@\[Tau]\)] + 
                E\^x\ Erf[\(x + \[Tau] + \[Tau]\ k\_1\)\/\(2\ \@\[Tau]\)])\)\ \
k\_1)\)\)], "Output",
  CellMargins->{{30, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "Finally, we multiply back up by ",
  Cell[BoxData[
      \(TraditionalForm\`J\)]],
  ", and we package in terms of the Norm function, obtaining"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-J\)\ \[ExponentialE]\^x\ 
        \(Norm(\(\(-x\) - \[Tau] - \[Tau]\ k\_1\)\/\@\(2  \[Tau]\))\) + 
      J\ \[ExponentialE]\^\(\(-\[Tau]\)\ k\_1\)\ 
        \(Norm(\(\(-x\) + \[Tau] - \[Tau]\ k\_1\)\/\(\ \@\(2  \[Tau]\)\))\)\n
      \t + \(1\/k\_1\) 
        \((J\ \[ExponentialE]\^\(x\ \)\ 
              \(Norm(\(x + \[Tau] + \[Tau]\ k\_1\)\/\@\(2  \[Tau]\))\) - 
            J\ \[ExponentialE]\^\(x(1 - k\_1) - \[Tau]\ k\_1\)\ 
              \(Norm(\(x + \[Tau] - \[Tau]\ k\_1\)\/\@\(2  \[Tau]\))\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["One further clean-up helps to recognize this - we set", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{
        SuperscriptBox["d", "\[Prime]",
          MultilineFunction->None], "=", 
        \(\(x + \[Tau] + \[Tau]\ k\_1\)\/\@\(2\ \[Tau]\) = 
          \(log \((S\/J)\) + 
              \((\[Sigma]\^2\/2 + r)\)\ \((T - t)\)\)\/\(\[Sigma]\ 
              \@\(T - t\)\)\)}], TraditionalForm]], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and obtain for the value", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-S\)\ \(Norm(\(-d'\)\ )\) + 
      J\ \[ExponentialE]\^\(-\(r(T - t)\)\)\ 
        \(Norm(\(-d'\)\  + \ \[Sigma] \@\( T - t\))\)\n\t + 
      \(\[Sigma]\^2\/\(2\ r\)\) 
        \((S\ \(Norm(d')\) - 
            S\ \[ExponentialE]\^\(-\(r(T - t)\)\)\ 
              \(\((J\/S)\)\^\(\(2\ r\)\/\[Sigma]\^2\)\) 
              \(Norm(d'\  - \ \(\(2\ r\)\/\[Sigma]\) \@\(T - t\))\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This is the formula for a standard Lookback Put, as given by Conze \
and Viswanathan (1991). \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Our Initial Plan of Attack", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We now have a collection of example solution methods for \
representatives of several families of options:

(a) simple log, power and related solutions;
(b) Binary option solutions;
(c) Call and Put solutions;
(d) Barrier solutions;
(e) Lookback solutions.

These will form the topics of the next five chapters respectively. Before \
proceeding to a detailed analysis of each type, we look briefly at how the \
diffusion equation representation may be generalized to handle:

(a) discrete dividends;
(b) time-dependent interest rates;
(c) coupon payments.

Inclusion of (a) is important for a wide range of instruments - the inclusion \
of (b) and (c) is particularly relevant to convertible bond modelling.\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["4.7 Dividends, Term Structures and Coupons", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In a single-factor model of a convertible bond, the convertible \
bond differential equation is taken to be the following:\
\>", "Text",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`1\/2\ S\^2\ \[PartialD]\^2 V\/\[PartialD]S\^2\ \[Sigma]\^2 - 
        \(r(t)\) V + \[PartialD]V\/\[PartialD]t + 
        \((\(r(t)\)\ S - Div(S, t))\) \[PartialD]V\/\(\[PartialD]S\ \) + 
        K(S, t) = 0\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is the natural time-dependent version of what we have presented in \
Chapter 3. If we try the change of variables used in the previous analysis of \
European options, w soon find that this fails unless ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " are constant. This is not what we want for the case of a convertible \
bond, or for any option on an asset where the dividends are time-dependent, \
as they are when they are discrete. The change of variables that does the \
job, at the expense of some slight complication with trigger levels, if they \
are present, is one which is part of a family of tricks set out by Harper \
(1994). The following discussion standardizes the problem in two stages. This \
analysis is useful both for simple single-factor convertibles, with \
deterministic time-dependent interest rates, and, with no coupons, for a \
standard option on an asset where the dividends are discrete.\n\nIn \
particular, we set, if ",
  Cell[BoxData[
      \(TraditionalForm\`E\)]],
  " is a base value of the underlying (to be chosen appropriately), and ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " is the expiry time, ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " the current time,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`x = log(S\/E)\), 
    \(TraditionalForm\`\[Tau] = 1\/2\ \[Sigma]\^2\ \((T - t)\)\)}], 
  "NumberedEquation"],

Cell[TextData[{
  "If ",
  Cell[BoxData[
      \(TraditionalForm\`q(\[Tau])\)]],
  " is the \[Tau]-dependent function representing the continuously compounded \
dividend yield ",
  Cell[BoxData[
      \(TraditionalForm\`DY(t)\)]],
  " (this may contain discrete pieces),"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    \(TraditionalForm\`\[Null]k\_1[\[Tau]] = \(\(2\ \((R\ \[Tau])\)\)\/\
\[Sigma]\^2 = \(2\ \((r\ t)\)\)\/\[Sigma]\^2\)\), 
    \(TraditionalForm\`\[Null]k\_2[\[Tau]] = \(\(2\ \((R\ \[Tau] - q\ \[Tau])\
\)\)\/\[Sigma]\^2 = \(2\ \((r\ t - DY\ t)\)\)\/\[Sigma]\^2\)\)}], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Then we obtain", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[PartialD]V\/\[PartialD]t + \ \[Null]k\_1[\[Tau]] V = \ \ 
      \[PartialD]\^2 V\/\[PartialD]x\^2\  + \ \ 
        \((\[Null]k\_2[\[Tau]]\  - 1\ )\) \[PartialD]V\/\[PartialD]x\  + \ 
        \(2\/\[Sigma]\^2\) K\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Thus far the change of variables has proceeded as for the European options \
discussed previously. The final part of the previous analysis does not work \
for the present problem unless ",
  Cell[BoxData[
      \(TraditionalForm\`k\_2[\[Tau]]\ \ is\ \(constant . \)\)]],
  " To treat the time-dependent case, let"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`z = x + F(\[Tau])\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["and set", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(x, \[Tau]) = 
      \(u(z, \[Tau])\)\ \[ExponentialE]\^\(-\(B(\[Tau])\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Making the required change of variable leads to", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u\ \((\[Null]k\_1[\[Tau]] - \[PartialD]B\/\[PartialD]\[Tau])\) + 
        \[PartialD]u\/\[PartialD]t = 
      \[PartialD]\^2 u\/\[PartialD]z\^2 + 
        \(2  K\[ExponentialE]\^\(B(\[Tau])\)\)\/\[Sigma]\^2 + 
        \((\(-\(\[PartialD]F\/\[PartialD]\[Tau]\)\) + \[Null]k\_2[\[Tau]] - 1)
            \)\ \[PartialD]V\/\[PartialD]x\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["so we can reduce the problem by making the choices", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_1[\[Tau]] = \[PartialD]B\/\[PartialD]\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_2[\[Tau]] - 1 = \[PartialD]F\/\[PartialD]\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["whence we get", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]u\/\[PartialD]t = \[PartialD]\^2 u\/\
\[PartialD]z\^2 + Q(\[Tau])\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where the diffusion equation now has a \"source term\" ",
  Cell[BoxData[
      \(TraditionalForm\`Q\)]],
  " representing the coupon payments in these coordinates:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`Q(\[Tau])\  = \(2  K\ \[ExponentialE]\^\(B(\[Tau])\)\)\
\/\[Sigma]\^2\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The quantities ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  " are like integrating factors (",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " is precisely this for the straight bond). The general plan is therefore:\n\
\n(1) Integrate to obtain ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  ";\n(2) Solve the PDE given the coupon payments (and for a CB, \
call/put/exercise conditions), using any suitable analytical or numerical \
scheme.\n\nIn practice we need some supplementary analysis to understand how \
to complete these steps, and to handle the boundary condition arising from \
considering the straight bond.\n\nNote how we have made yet another link with \
the heat equation in a traditional form. The coupon payments in a bond are \
like a source of body heat for the heat equation. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Solution of the Auxiliary Conditions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
A simplified form of the CB equation applies to the lower boundary \
condition, where we have a straight bond valuation governed by the \
equations\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[{
    FormBox[
      RowBox[{
        FractionBox[\(\[PartialD]u\), \(\[PartialD]t\),
          MultilineFunction->None], "=", \(Q \((\[Tau])\)\)}], 
      TraditionalForm], 
    FormBox[
      RowBox[{\(k\_1[\[Tau]]\), "=", 
        FractionBox[\(\[PartialD]B\), \(\[PartialD]\[Tau]\),
          MultilineFunction->None]}], TraditionalForm]}], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Integration of the latter of these two relations gives", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`B(\[Tau]) = 
      \(\[Integral]\_0\%\[Tau] k\_1[p] \[DifferentialD]p = 
        \[Integral]\_t\%T\( r(s)\) \[DifferentialD]s\)\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Then integration of the fomer, and conversion back to our original \
coordinates, give\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V\_0 = \[ExponentialE]\^\(-
            \(\[Integral]\_t\%T\( r(s)\) \[DifferentialD]s\)\)\ 
        \((Z + \[Integral]\_t\%T K[s]\ 
                \(\[ExponentialE]\^\(\[Integral]\_s\%T\( r(p)\) 
                        \[DifferentialD]p\)\) \[DifferentialD]s)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Now the coupon payments are typically discrete, so we write, \
accordingly,\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`K[
        t] = \[Sum]\+\(t\  \[LessSlantEqual] t\_\(i\ \) \[LessSlantEqual] \
T\)K\_i\ \(\[Delta](t - t\_i)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "representing the coupon payments as a general cash flow of payments ",
  Cell[BoxData[
      \(TraditionalForm\`K\_i\ at  times\ \(t\_i . \)\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V\_0 = \[ExponentialE]\^\(-\(\[Integral]\_t\%T\( \
r(s)\) \[DifferentialD]s\)\)\ \((Z + \[Sum]\+\(t\  \[LessSlantEqual] t\_\(i\ \
\) \[LessSlantEqual] T\)K\_i\ \ \[ExponentialE]\^\(\[Integral]\_t\_i\%T\( \
r(p)\) \[DifferentialD]p\))\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["This boils down to ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V\_0 = 
      Z\[ExponentialE]\^\(-\(\[Integral]\_t\%T\( r(s)\) \[DifferentialD]s\)\)\
\  + \[Sum]\+\(t\  \[LessSlantEqual] t\_\(i\ \) \[LessSlantEqual] T\)K\_i\ \ \
\[ExponentialE]\^\(-\(\[Integral]\_t\%t\_i\( r(p)\) \[DifferentialD]p\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "If the interest rate were constant at a value ",
  Cell[BoxData[
      \(TraditionalForm\`r\_0\)]],
  ", this would simplify further to "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V\_0 = 
      Z\[ExponentialE]\^\(-\(\(r\_0\)(T - t)\)\)\  + \[Sum]\+\(t\  \
\[LessSlantEqual] t\_\(i\ \) \[LessSlantEqual] T\)K\_i\ \ \
\[ExponentialE]\^\(-\(\(r\_0\)(t\_i - t)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
but we do not assume this here. In general, we require knowledge \
of, or an estimate of, the integrals of the form\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Integral]\_t\_i\%T\( r(p)\) \[DifferentialD]p\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and this must be based on yield-curve information. When we come to solve \
the full CB partial differential equation, the effect of the coupon payments \
is to define jump conditions for the value. If we integrate the PDE in time \
through a coupon payment date ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Tau]\_\(i\ \ \)\) with  payment\ \ K\_i\)]],
  ", then in a neighbourhood of the coupon payment date, the source to the \
diffusion equation is given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`Q(\[Tau]) = 
      K\_i\ \[ExponentialE]\^\(B(\[Tau]\_i)\)\ 
        \(\[Delta](\[Tau] - \[Tau]\_i)\)\)], "NumberedEquation",
  CellFrame->False,
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["So the valuation jump condition takes the form", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`u(\(\[Tau]\_i\^+\)) = 
      u(\(\[Tau]\_i\^-\))\  + \ K\_i\ \[ExponentialE]\^\(B(\[Tau]\_i)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The F function and Dividend Jump Conditions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The integration leading to the ",
  Cell[BoxData[
      \(TraditionalForm\`F\)]],
  " function takes place and generates further jump conditions whenever there \
is a discrete dividend payment. If ",
  Cell[BoxData[
      \(TraditionalForm\`B\)]],
  " has been determined from yield-curve data, and the dividend yield \
function ",
  Cell[BoxData[
      \(TraditionalForm\`q(\[Tau])\)]],
  " has been specified, the equation"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`k\_2[\[Tau]] - 1 = \[PartialD]F\/\[PartialD]\[Tau]\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[" can be solved as", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`F(\[Tau]) = 
      \(-\[Tau]\) + B(\[Tau]) - 
        \(2\ \(\[Integral]\_0\%\[Tau]\( q(s)\) 
                \[DifferentialD]s\)\)\/\[Sigma]\^2\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In practice, we would imagine ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " as being composed of discrete values of the yield isolated at dividend \
payment dates ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Tau]\_i . \)\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`F(\[Tau]) = \(-\[Tau]\) + 
        B(\[Tau]) - \[Sum]\+\(0\  \[LessSlantEqual] t\_\(i\ \) \
\[LessSlantEqual] \[Tau]\)q\_i\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In some ways, that is all there is to it, but it is useful to re-interpret \
this result in terms of jump conditions on the valuation. Just before (in \
\[Tau] time, in reality just after) a dividend payment date ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\_I, \ \(the  value  of  F  is\ \)\)]]
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`F(\[Tau]\_I - ) = \(-\[Tau]\_I\) + 
        B(\[Tau]\_I) - \[Sum]\+\(0\  \[LessSlantEqual] t\_\(i\ \) \
\[LessSlantEqual] \(\[Tau]\_I - \)\)q\_i\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["whereas just after it is ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`F(\[Tau]\_I + ) = \(-\[Tau]\_I\) + 
        B(\[Tau]\_I) - \[Sum]\+\(0\  \[LessSlantEqual] t\_\(i\ \) \
\[LessSlantEqual] \(\[Tau]\_I + \)\)\%\(\_\ \)q\_i\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["so the difference is given by ", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`F(\[Tau]\_I + ) = F(\[Tau]\_I - )\  - \ q\_I\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "On the valuation, in ",
  Cell[BoxData[
      \(TraditionalForm\`\((x, \ \[Tau])\)\)]],
  " coordinates, this leads to"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(x, \ \(\[Tau]\_I + \))\  = \ V(x - \ q\_I, \ \(\[Tau]\_I - \))\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and in ",
  Cell[BoxData[
      \(TraditionalForm\`\((S, \ t)\)\)]],
  " coordinates, bearing in mind the time reversal,"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V(S, \(t\_I - \)) = 
      V(S  \[ExponentialE]\^\(-q\_\[ImaginaryI]\), \(t\_I + \))\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
so we recover the condition that the valuation be continuous across \
the dividend payment date, with the stock price being adjusted downwards \
based on the dividend payment. This could have been taken as a starting point \
assumption on financial grounds, but it is reassuring to see it arise \
naturally from the differential equation. \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 4 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Conze, A. and Viswanathan, 1991, Path dependent options: the case of \
lookback options, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 46, p. 1893.  \n\nGeman, H. and Yor, M., 1996, Pricing and hedging \
double-barrier options: a probabilistic approach, ",
  StyleBox["Mathematical Finance, ",
    FontSlant->"Italic"],
  "6, p. 365.\n\nHarper, J., 1994, Reducing parabolic partial differential \
equations to canonical form, ",
  StyleBox["European Journal of Applied Mathematics, ",
    FontSlant->"Italic"],
  "5, p. 159.\n\nHull, J., 1996, ",
  StyleBox["Options, Futures and Other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\nJeffreys, H. and Jeffreys, B.S., 1946, ",
  StyleBox["Methods of Mathematical Physics",
    FontSlant->"Italic"],
  " (3rd edition, 1972), Cambridge University Press.\n\nKunitomo, N. and \
Ikeda, M., 1992, Pricing options with curved boundaries, ",
  StyleBox["Mathematical Finance,",
    FontSlant->"Italic"],
  " 2, p. 275.\n\nRubenstein, M. and Reiner, E., 1991, Breaking down the \
barriers, ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", September.\n\nWilmott, P., Dewynne, J. and Howison, S., 1993, ",
  StyleBox["Option Pricing - Mathematical Models and Computation",
    FontSlant->"Italic"],
  ", Oxford Financial Press.\n\n"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{736, 715},
WindowMargins->{{4, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->85,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"4. Mathematical Preliminaries"}]]}, {Cell[ 
        TextData[ {"4. Mathematical Preliminaries"}]], Inherited, Inherited}},

PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0fL5N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@S</@0000000000064/01T1T0000000@000
00000000007oo@S<^@3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 215, 8, 136, "Title"],
Cell[1949, 61, 165, 2, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2139, 67, 140, 3, 50, "Section"],
Cell[2282, 72, 1142, 19, 159, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3461, 96, 119, 1, 50, "Section"],
Cell[3583, 99, 163, 4, 30, "Text"],
Cell[3749, 105, 632, 13, 43, "NumberedEquation"],
Cell[4384, 120, 232, 5, 30, "Text"],
Cell[4619, 127, 291, 7, 39, "NumberedEquation"],
Cell[4913, 136, 210, 6, 30, "Text"],
Cell[5126, 144, 535, 11, 43, "NumberedEquation"],
Cell[5664, 157, 237, 5, 46, "Text"],
Cell[5904, 164, 163, 3, 25, "NumberedEquation"],
Cell[6070, 169, 1162, 30, 93, "Text"],
Cell[7235, 201, 527, 11, 42, "NumberedEquation"],
Cell[7765, 214, 199, 6, 29, "Text"],
Cell[7967, 222, 193, 4, 29, "NumberedEquation"],
Cell[8163, 228, 780, 17, 62, "Text"],
Cell[8946, 247, 170, 3, 36, "NumberedEquation"],
Cell[9119, 252, 81, 1, 20, "Text"],
Cell[9203, 255, 234, 5, 36, "NumberedEquation"],
Cell[9440, 262, 83, 1, 20, "Text"],
Cell[9526, 265, 218, 5, 34, "NumberedEquation"],
Cell[9747, 272, 440, 12, 29, "Text"],
Cell[10190, 286, 259, 6, 30, "NumberedEquation"],
Cell[10452, 294, 932, 21, 73, "Text"],

Cell[CellGroupData[{
Cell[11409, 319, 164, 5, 32, "Subsection"],
Cell[11576, 326, 255, 9, 20, "Text"],

Cell[CellGroupData[{
Cell[11856, 339, 174, 3, 25, "Input"],
Cell[12033, 344, 51, 1, 36, "Output"]
}, Open  ]],
Cell[12099, 348, 229, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[12353, 358, 239, 4, 50, "Input"],
Cell[12595, 364, 83, 1, 38, "Output"]
}, Open  ]],
Cell[12693, 368, 158, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[12876, 376, 212, 4, 37, "Input"],
Cell[13091, 382, 69, 1, 48, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[13209, 389, 111, 1, 32, "Subsection"],
Cell[13323, 392, 201, 4, 29, "Text"],
Cell[13527, 398, 532, 11, 32, "NumberedEquation"],
Cell[14062, 411, 445, 11, 40, "Text"],
Cell[14510, 424, 143, 2, 20, "NumberedEquation"],
Cell[14656, 428, 171, 5, 20, "Text"],
Cell[14830, 435, 386, 8, 30, "NumberedEquation"],
Cell[15219, 445, 127, 1, 20, "Text"],
Cell[15349, 448, 687, 15, 32, "NumberedEquation"],
Cell[16039, 465, 76, 1, 20, "Text"],
Cell[16118, 468, 524, 12, 32, "NumberedEquation"],
Cell[16645, 482, 401, 8, 40, "Text"],
Cell[17049, 492, 131, 2, 29, "NumberedEquation"],
Cell[17183, 496, 219, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[17427, 506, 301, 6, 50, "Input"],
Cell[17731, 514, 337, 5, 45, "Output"]
}, Open  ]],
Cell[18083, 522, 260, 9, 20, "Text"],

Cell[CellGroupData[{
Cell[18368, 535, 141, 2, 25, "Input"],
Cell[18512, 539, 277, 4, 52, "Output"]
}, Open  ]],
Cell[18804, 546, 388, 12, 29, "Text"],

Cell[CellGroupData[{
Cell[19217, 562, 136, 2, 25, "Input"],
Cell[19356, 566, 216, 4, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[19609, 575, 114, 2, 25, "Input"],
Cell[19726, 579, 83, 1, 38, "Output"]
}, Open  ]],
Cell[19824, 583, 698, 16, 73, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[20559, 604, 116, 1, 32, "Subsection"],
Cell[20678, 607, 373, 9, 29, "Text"],

Cell[CellGroupData[{
Cell[21076, 620, 118, 2, 25, "Input"],
Cell[21197, 624, 321, 5, 52, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[21555, 634, 99, 2, 25, "Input"],
Cell[21657, 638, 362, 6, 42, "Output"]
}, Open  ]],
Cell[22034, 647, 109, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[22168, 652, 143, 2, 25, "Input"],
Cell[22314, 656, 187, 3, 42, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[22538, 664, 144, 2, 25, "Input"],
Cell[22685, 668, 187, 3, 42, "Output"]
}, Open  ]],
Cell[22887, 674, 166, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[23078, 682, 138, 2, 25, "Input"],
Cell[23219, 686, 67, 1, 38, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[23323, 692, 138, 2, 25, "Input"],
Cell[23464, 696, 35, 1, 36, "Output"]
}, Open  ]],
Cell[23514, 700, 581, 19, 40, "Text"],

Cell[CellGroupData[{
Cell[24120, 723, 155, 3, 25, "Input"],
Cell[24278, 728, 185, 3, 56, "Output"]
}, Open  ]],
Cell[24478, 734, 96, 1, 20, "Text"],
Cell[24577, 737, 162, 3, 34, "NumberedEquation"],
Cell[24742, 742, 225, 5, 33, "NumberedEquation"],
Cell[24970, 749, 106, 1, 20, "Text"],
Cell[25079, 752, 146, 2, 25, "Input"],
Cell[25228, 756, 212, 4, 25, "Input"],
Cell[25443, 762, 381, 13, 29, "Text"],

Cell[CellGroupData[{
Cell[25849, 779, 168, 3, 25, "Input"],
Cell[26020, 784, 38, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[26095, 790, 172, 4, 25, "Input"],
Cell[26270, 796, 35, 1, 36, "Output"]
}, Open  ]],
Cell[26320, 800, 748, 22, 52, "Text",
  PageBreakBelow->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[27117, 828, 133, 1, 38, "Section"],
Cell[27253, 831, 168, 4, 20, "Text"],
Cell[27424, 837, 632, 13, 32, "NumberedEquation"],
Cell[28059, 852, 229, 7, 20, "Text"],
Cell[28291, 861, 143, 2, 20, "NumberedEquation"],
Cell[28437, 865, 122, 1, 20, "Text"],
Cell[28562, 868, 626, 14, 32, "NumberedEquation"],
Cell[29191, 884, 347, 10, 29, "Text"],
Cell[29541, 896, 582, 13, 32, "NumberedEquation"],
Cell[30126, 911, 146, 2, 29, "NumberedEquation"],
Cell[30275, 915, 159, 3, 29, "NumberedEquation"],
Cell[30437, 920, 366, 8, 29, "Text"],
Cell[30806, 930, 160, 3, 29, "NumberedEquation"],
Cell[30969, 935, 99, 1, 20, "Text"],
Cell[31071, 938, 530, 11, 32, "NumberedEquation"],
Cell[31604, 951, 265, 6, 29, "Text"],
Cell[31872, 959, 283, 6, 21, "NumberedEquation"],
Cell[32158, 967, 123, 1, 20, "Text"],
Cell[32284, 970, 341, 7, 32, "NumberedEquation"],
Cell[32628, 979, 504, 8, 52, "Text"],

Cell[CellGroupData[{
Cell[33157, 991, 105, 1, 32, "Subsection"],
Cell[33265, 994, 220, 5, 29, "Text"],
Cell[33488, 1001, 582, 13, 32, "NumberedEquation"],
Cell[34073, 1016, 145, 4, 20, "Text"],
Cell[34221, 1022, 818, 19, 30, "NumberedEquation"],
Cell[35042, 1043, 163, 4, 20, "Text"],
Cell[35208, 1049, 183, 4, 20, "NumberedEquation"],
Cell[35394, 1055, 435, 10, 30, "NumberedEquation"],
Cell[35832, 1067, 312, 7, 29, "Text"],
Cell[36147, 1076, 154, 3, 32, "NumberedEquation"],
Cell[36304, 1081, 162, 4, 20, "Text"],
Cell[36469, 1087, 252, 5, 32, "NumberedEquation"],
Cell[36724, 1094, 178, 5, 20, "Text"],
Cell[36905, 1101, 141, 2, 20, "NumberedEquation"],
Cell[37049, 1105, 208, 4, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[37294, 1114, 155, 4, 32, "Subsection"],
Cell[37452, 1120, 113, 1, 20, "Text"],
Cell[37568, 1123, 270, 5, 21, "NumberedEquation"],
Cell[37841, 1130, 169, 5, 20, "Text"],
Cell[38013, 1137, 341, 7, 32, "NumberedEquation"],
Cell[38357, 1146, 380, 10, 29, "Text"],
Cell[38740, 1158, 282, 6, 30, "NumberedEquation"],
Cell[39025, 1166, 279, 6, 32, "NumberedEquation"],

Cell[CellGroupData[{
Cell[39329, 1176, 166, 5, 28, "Subsubsection"],
Cell[39498, 1183, 118, 1, 20, "Text"],
Cell[39619, 1186, 188, 4, 20, "NumberedEquation"],
Cell[39810, 1192, 272, 6, 20, "Text"],
Cell[40085, 1200, 324, 7, 32, "NumberedEquation"],
Cell[40412, 1209, 191, 4, 20, "Text"],
Cell[40606, 1215, 168, 3, 20, "NumberedEquation"],
Cell[40777, 1220, 289, 7, 29, "Text"],
Cell[41069, 1229, 179, 4, 29, "NumberedEquation"],
Cell[41251, 1235, 500, 15, 29, "Text"],
Cell[41754, 1252, 146, 2, 32, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[41937, 1259, 166, 5, 28, "Subsubsection"],
Cell[42106, 1266, 245, 6, 20, "Text"],
Cell[42354, 1274, 346, 8, 34, "NumberedEquation"],
Cell[42703, 1284, 84, 1, 20, "Text"],
Cell[42790, 1287, 222, 5, 34, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[43049, 1297, 166, 5, 28, "Subsubsection"],
Cell[43218, 1304, 193, 6, 20, "Text"],
Cell[43414, 1312, 243, 5, 20, "NumberedEquation"],
Cell[43660, 1319, 286, 5, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[43995, 1330, 121, 1, 32, "Subsection"],
Cell[44119, 1333, 524, 11, 40, "Text"],
Cell[44646, 1346, 162, 3, 32, "NumberedEquation"],
Cell[44811, 1351, 85, 1, 20, "Text"],
Cell[44899, 1354, 156, 3, 20, "NumberedEquation"],
Cell[45058, 1359, 148, 4, 20, "Text"],
Cell[45209, 1365, 621, 15, 33, "NumberedEquation"],
Cell[45833, 1382, 327, 9, 29, "Text"],
Cell[46163, 1393, 458, 10, 32, "NumberedEquation"],
Cell[46624, 1405, 295, 9, 20, "Text"],

Cell[CellGroupData[{
Cell[46944, 1418, 226, 4, 37, "Input"],
Cell[47173, 1424, 177, 3, 78, "Output"]
}, Open  ]],
Cell[47365, 1430, 195, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[47585, 1440, 208, 4, 25, "Input"],
Cell[47796, 1446, 65, 1, 45, "Output"]
}, Open  ]],
Cell[47876, 1450, 159, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[48060, 1460, 222, 4, 37, "Input"],
Cell[48285, 1466, 111, 2, 52, "Output"]
}, Open  ]],
Cell[48411, 1471, 285, 9, 20, "Text"],
Cell[48699, 1482, 429, 10, 32, "NumberedEquation"],
Cell[49131, 1494, 97, 1, 20, "Text"],
Cell[49231, 1497, 307, 7, 30, "NumberedEquation"],
Cell[49541, 1506, 125, 1, 20, "Text"],
Cell[49669, 1509, 205, 4, 24, "NumberedEquation"],
Cell[49877, 1515, 300, 10, 20, "Text"],
Cell[50180, 1527, 164, 3, 24, "NumberedEquation"],
Cell[50347, 1532, 147, 4, 20, "Text"],
Cell[50497, 1538, 192, 4, 38, "NumberedEquation"],
Cell[50692, 1544, 361, 12, 29, "Text"],

Cell[CellGroupData[{
Cell[51078, 1560, 198, 4, 25, "Input"],
Cell[51279, 1566, 447, 10, 57, "Output"]
}, Open  ]],
Cell[51741, 1579, 102, 1, 20, "Text"],
Cell[51846, 1582, 227, 5, 38, "NumberedEquation"],
Cell[52076, 1589, 207, 6, 20, "Text"],
Cell[52286, 1597, 173, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[52484, 1604, 248, 6, 37, "Input"],
Cell[52735, 1612, 35739, 1684, 149, 23351, 1526, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[88489, 3299, 1239, 23, 128, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[89765, 3327, 120, 1, 32, "Subsection"],
Cell[89888, 3330, 425, 9, 40, "Text"],
Cell[90316, 3341, 129, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[90470, 3347, 111, 2, 25, "Input"],
Cell[90584, 3351, 251, 4, 56, "Print"]
}, Open  ]],
Cell[90850, 3358, 160, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[91035, 3366, 159, 3, 25, "Input"],
Cell[91197, 3371, 69, 1, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[91303, 3377, 121, 2, 25, "Input"],
Cell[91427, 3381, 98, 2, 56, "Output"]
}, Open  ]],
Cell[91540, 3386, 518, 14, 44, "Text"],

Cell[CellGroupData[{
Cell[92083, 3404, 145, 2, 25, "Input"],
Cell[92231, 3408, 98, 2, 57, "Output"]
}, Open  ]],
Cell[92344, 3413, 228, 7, 20, "Text"],
Cell[92575, 3422, 129, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[92729, 3428, 111, 2, 25, "Input"],
Cell[92843, 3432, 304, 5, 69, "Print"]
}, Open  ]],
Cell[93162, 3440, 216, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[93403, 3450, 159, 3, 25, "Input"],
Cell[93565, 3455, 53, 1, 36, "Output"]
}, Open  ]],
Cell[93633, 3459, 102, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[93760, 3464, 141, 2, 25, "Input"],
Cell[93904, 3468, 81, 1, 57, "Output"]
}, Open  ]],
Cell[94000, 3472, 201, 4, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[94250, 3482, 148, 4, 38, "Section"],
Cell[94401, 3488, 232, 7, 20, "Text"],
Cell[94636, 3497, 155, 3, 20, "NumberedEquation"],
Cell[94794, 3502, 195, 6, 20, "Text"],
Cell[94992, 3510, 132, 2, 20, "NumberedEquation"],
Cell[95127, 3514, 369, 8, 40, "Text"],
Cell[95499, 3524, 253, 5, 32, "NumberedEquation"],
Cell[95755, 3531, 475, 13, 40, "Text"],
Cell[96233, 3546, 265, 5, 32, "NumberedEquation"],
Cell[96501, 3553, 409, 12, 29, "Text"],
Cell[96913, 3567, 147, 2, 20, "NumberedEquation"],
Cell[97063, 3571, 203, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[97291, 3581, 101, 1, 32, "Subsection"],
Cell[97395, 3584, 319, 6, 40, "Text"],
Cell[97717, 3592, 302, 6, 32, "NumberedEquation"],
Cell[98022, 3600, 354, 8, 32, "NumberedEquation"],
Cell[98379, 3610, 458, 16, 29, "Text"],
Cell[98840, 3628, 405, 10, 30, "NumberedEquation"],
Cell[99248, 3640, 101, 1, 20, "Text"],
Cell[99352, 3643, 220, 5, 20, "NumberedEquation"],
Cell[99575, 3650, 186, 6, 20, "Text"],
Cell[99764, 3658, 443, 10, 32, "NumberedEquation"],
Cell[100210, 3670, 245, 9, 20, "Text"],
Cell[100458, 3681, 596, 12, 32, "NumberedEquation"],
Cell[101057, 3695, 92, 1, 20, "Text"],
Cell[101152, 3698, 259, 6, 32, "NumberedEquation"],
Cell[101414, 3706, 79, 1, 20, "Text"],
Cell[101496, 3709, 356, 8, 32, "NumberedEquation"],
Cell[101855, 3719, 549, 10, 52, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[102453, 3735, 132, 1, 38, "Section"],
Cell[102588, 3738, 331, 7, 29, "Text"],
Cell[102922, 3747, 244, 5, 44, "NumberedEquation"],
Cell[103169, 3754, 746, 17, 62, "Text"],
Cell[103918, 3773, 205, 5, 20, "NumberedEquation"],
Cell[104126, 3780, 553, 15, 40, "Text"],
Cell[104682, 3797, 262, 5, 20, "NumberedEquation"],
Cell[104947, 3804, 541, 18, 29, "Text"],
Cell[105491, 3824, 239, 5, 33, "NumberedEquation"],
Cell[105733, 3831, 772, 25, 40, "Text"],
Cell[106508, 3858, 182, 4, 24, "NumberedEquation"],
Cell[106693, 3864, 502, 16, 29, "Text"],
Cell[107198, 3882, 245, 5, 33, "NumberedEquation"],
Cell[107446, 3889, 205, 5, 20, "NumberedEquation"],
Cell[107654, 3896, 435, 11, 40, "Text"],
Cell[108092, 3909, 400, 8, 32, "NumberedEquation"],
Cell[108495, 3919, 100, 1, 20, "Text"],
Cell[108598, 3922, 335, 7, 38, "NumberedEquation"],
Cell[108936, 3931, 93, 1, 20, "Text"],
Cell[109032, 3934, 237, 5, 34, "NumberedEquation"],

Cell[CellGroupData[{
Cell[109294, 3943, 106, 1, 32, "Subsection"],
Cell[109403, 3946, 561, 9, 62, "Text"],
Cell[109967, 3957, 137, 2, 20, "NumberedEquation"],
Cell[110107, 3961, 218, 4, 32, "NumberedEquation"],
Cell[110328, 3967, 184, 4, 24, "NumberedEquation"],
Cell[110515, 3973, 147, 4, 20, "Text"],
Cell[110665, 3979, 275, 7, 33, "NumberedEquation"],
Cell[110943, 3988, 145, 4, 20, "Text"],
Cell[111091, 3994, 290, 6, 32, "NumberedEquation"],
Cell[111384, 4002, 118, 1, 20, "Text"],
Cell[111505, 4005, 277, 6, 22, "NumberedEquation"],
Cell[111785, 4013, 129, 1, 20, "Text"],
Cell[111917, 4016, 263, 6, 33, "NumberedEquation"],
Cell[112183, 4024, 477, 12, 33, "Text"],
Cell[112663, 4038, 433, 10, 40, "NumberedEquation"],
Cell[113099, 4050, 132, 1, 20, "Text"],
Cell[113234, 4053, 403, 9, 38, "NumberedEquation"],
Cell[113640, 4064, 217, 5, 20, "Text"],

Cell[CellGroupData[{
Cell[113882, 4073, 160, 4, 25, "Input"],
Cell[114045, 4079, 591, 12, 108, "Output"]
}, Open  ]],
Cell[114651, 4094, 129, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[114805, 4099, 117, 2, 25, "Input"],
Cell[114925, 4103, 124, 2, 76, "Output"]
}, Open  ]],
Cell[115064, 4108, 209, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[115298, 4118, 109, 2, 25, "Input"],
Cell[115410, 4122, 69, 1, 50, "Output"]
}, Open  ]],
Cell[115494, 4126, 898, 24, 62, "Text"],
Cell[116395, 4152, 416, 9, 34, "NumberedEquation"],
Cell[116814, 4163, 183, 4, 20, "Text"],
Cell[117000, 4169, 304, 8, 33, "NumberedEquation"],
Cell[117307, 4179, 156, 4, 20, "Text"],
Cell[117466, 4185, 215, 4, 33, "NumberedEquation"],
Cell[117684, 4191, 237, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[117946, 4201, 168, 3, 25, "Input"],
Cell[118117, 4206, 438, 10, 76, "Output"]
}, Open  ]],
Cell[118570, 4219, 214, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[118809, 4229, 116, 2, 25, "Input"],
Cell[118928, 4233, 106, 2, 57, "Output"]
}, Open  ]],
Cell[119049, 4238, 409, 7, 52, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[119495, 4250, 120, 1, 32, "Subsection"],
Cell[119618, 4253, 386, 8, 40, "Text"],
Cell[120007, 4263, 260, 6, 30, "NumberedEquation"],
Cell[120270, 4271, 1713, 36, 161, "Text"],
Cell[121986, 4309, 297, 5, 20, "NumberedEquation"],
Cell[122286, 4316, 375, 8, 40, "Text"],
Cell[122664, 4326, 336, 7, 29, "NumberedEquation"],
Cell[123003, 4335, 444, 13, 29, "Text"],

Cell[CellGroupData[{
Cell[123472, 4352, 223, 3, 37, "Input"],
Cell[123698, 4357, 86, 1, 48, "Output"]
}, Open  ]],
Cell[123799, 4361, 112, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[123936, 4366, 225, 4, 25, "Input"],
Cell[124164, 4372, 86, 1, 48, "Output"]
}, Open  ]],
Cell[124265, 4376, 126, 1, 20, "Text"],
Cell[124394, 4379, 182, 4, 20, "NumberedEquation"],
Cell[124579, 4385, 202, 5, 20, "Text"],
Cell[124784, 4392, 105, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[124914, 4398, 245, 4, 37, "Input"],
Cell[125162, 4404, 570, 11, 84, "Output"]
}, Open  ]],
Cell[125747, 4418, 106, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[125878, 4423, 137, 2, 25, "Input"],
Cell[126018, 4427, 164, 2, 54, "Output"]
}, Open  ]],
Cell[126197, 4432, 283, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[126505, 4441, 132, 2, 25, "Input"],
Cell[126640, 4445, 459, 12, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127136, 4462, 99, 2, 25, "Input"],
Cell[127238, 4466, 71, 1, 56, "Output"]
}, Open  ]],
Cell[127324, 4470, 170, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[127519, 4478, 171, 4, 25, "Input"],
Cell[127693, 4484, 165, 2, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[127895, 4491, 148, 2, 25, "Input"],
Cell[128046, 4495, 211, 3, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128294, 4503, 110, 2, 25, "Input"],
Cell[128407, 4507, 147, 2, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128591, 4514, 130, 2, 25, "Input"],
Cell[128724, 4518, 619, 18, 57, "Output"]
}, Open  ]],
Cell[129358, 4539, 268, 6, 29, "Text"],

Cell[CellGroupData[{
Cell[129651, 4549, 200, 4, 25, "Input"],
Cell[129854, 4555, 105, 2, 57, "Output"]
}, Open  ]],
Cell[129974, 4560, 213, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[130212, 4570, 322, 5, 37, "Input"],
Cell[130537, 4577, 331, 5, 57, "Output"]
}, Open  ]],
Cell[130883, 4585, 237, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[131145, 4594, 207, 4, 25, "Input"],
Cell[131355, 4600, 308, 5, 58, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[131700, 4610, 107, 2, 25, "Input"],
Cell[131810, 4614, 35, 1, 36, "Output"]
}, Open  ]],
Cell[131860, 4618, 146, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[132031, 4626, 213, 4, 25, "Input"],
Cell[132247, 4632, 35, 1, 36, "Output"]
}, Open  ]],
Cell[132297, 4636, 249, 5, 29, "Text"],
Cell[132549, 4643, 134, 2, 25, "Input"],
Cell[132686, 4647, 481, 9, 38, "NumberedEquation"],
Cell[133170, 4658, 702, 21, 40, "Text"],
Cell[133875, 4681, 347, 6, 37, "Input"],
Cell[134225, 4689, 134, 2, 25, "Input"],
Cell[134362, 4693, 511, 9, 38, "NumberedEquation"],
Cell[134876, 4704, 238, 7, 20, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[135163, 4717, 113, 1, 38, "Section"],
Cell[135279, 4720, 399, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[135703, 4731, 96, 1, 32, "Subsection"],
Cell[135802, 4734, 203, 6, 20, "Text"],
Cell[136008, 4742, 137, 2, 20, "NumberedEquation"],
Cell[136148, 4746, 109, 1, 20, "Text"],
Cell[136260, 4749, 138, 2, 20, "NumberedEquation"],
Cell[136401, 4753, 150, 4, 20, "Text"],
Cell[136554, 4759, 180, 3, 20, "NumberedEquation"],
Cell[136737, 4764, 310, 10, 29, "Text"],
Cell[137050, 4776, 241, 6, 21, "NumberedEquation"],
Cell[137294, 4784, 115, 1, 20, "Text"],
Cell[137412, 4787, 282, 6, 21, "NumberedEquation"],
Cell[137697, 4795, 456, 9, 38, "NumberedEquation"],
Cell[138156, 4806, 172, 4, 20, "Text"],
Cell[138331, 4812, 495, 10, 38, "NumberedEquation"],
Cell[138829, 4824, 202, 6, 20, "Text"],
Cell[139034, 4832, 362, 7, 32, "NumberedEquation"],
Cell[139399, 4841, 173, 6, 20, "Text"],
Cell[139575, 4849, 375, 8, 48, "NumberedEquation"],
Cell[139953, 4859, 507, 12, 40, "Text"],
Cell[140463, 4873, 171, 3, 32, "NumberedEquation"],
Cell[140637, 4878, 85, 1, 20, "Text"],
Cell[140725, 4881, 142, 2, 20, "NumberedEquation"],
Cell[140870, 4885, 311, 8, 29, "Text"],
Cell[141184, 4895, 190, 4, 20, "NumberedEquation"],
Cell[141377, 4901, 378, 9, 29, "Text"],
Cell[141758, 4912, 131, 2, 29, "NumberedEquation"],
Cell[141892, 4916, 182, 6, 20, "Text"],
Cell[142077, 4924, 183, 4, 32, "NumberedEquation"],
Cell[142263, 4930, 95, 1, 20, "Text"],
Cell[142361, 4933, 160, 3, 20, "NumberedEquation"],
Cell[142524, 4938, 88, 1, 20, "Text"],
Cell[142615, 4941, 349, 7, 40, "NumberedEquation"],
Cell[142967, 4950, 407, 13, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[143411, 4968, 100, 1, 32, "Subsection"],
Cell[143514, 4971, 345, 6, 40, "Text"],
Cell[143862, 4979, 157, 3, 20, "NumberedEquation"],
Cell[144022, 4984, 157, 3, 20, "NumberedEquation"],
Cell[144182, 4989, 184, 6, 20, "Text"],
Cell[144369, 4997, 150, 4, 25, "Input"],
Cell[144522, 5003, 149, 4, 25, "Input"],

Cell[CellGroupData[{
Cell[144696, 5011, 140, 2, 25, "Input"],
Cell[144839, 5015, 10983, 332, 118, 3311, 233, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[155859, 5352, 139, 2, 25, "Input"],
Cell[156001, 5356, 12848, 364, 118, 3483, 244, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[168864, 5723, 160, 4, 20, "Text"],
Cell[169027, 5729, 188, 4, 20, "NumberedEquation"],
Cell[169218, 5735, 185, 4, 20, "Text"],
Cell[169406, 5741, 446, 10, 40, "NumberedEquation"],
Cell[169855, 5753, 361, 8, 40, "Text"],
Cell[170219, 5763, 155, 3, 25, "Input"],

Cell[CellGroupData[{
Cell[170399, 5770, 383, 7, 50, "Input"],
Cell[170785, 5779, 280, 4, 57, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[171102, 5788, 124, 2, 25, "Input"],
Cell[171229, 5792, 240, 4, 54, "Output"]
}, Open  ]],
Cell[171484, 5799, 820, 20, 62, "Text"],

Cell[CellGroupData[{
Cell[172329, 5823, 139, 2, 37, "Input"],
Cell[172471, 5827, 61, 1, 54, "Output"]
}, Open  ]],
Cell[172547, 5831, 121, 1, 20, "Text"],
Cell[172671, 5834, 322, 7, 28, "NumberedEquation"],
Cell[172996, 5843, 129, 1, 20, "Text"],
Cell[173128, 5846, 331, 7, 28, "NumberedEquation"],
Cell[173462, 5855, 354, 8, 29, "Text"],

Cell[CellGroupData[{
Cell[173841, 5867, 115, 1, 28, "Subsubsection"],
Cell[173959, 5870, 332, 10, 29, "Text"],
Cell[174294, 5882, 274, 6, 38, "NumberedEquation"],
Cell[174571, 5890, 173, 6, 20, "Text"],
Cell[174747, 5898, 292, 6, 38, "NumberedEquation"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[175088, 5910, 93, 1, 32, "Subsection"],
Cell[175184, 5913, 246, 6, 20, "Text"],
Cell[175433, 5921, 185, 4, 20, "NumberedEquation"],
Cell[175621, 5927, 107, 1, 20, "Text"],
Cell[175731, 5930, 190, 4, 20, "NumberedEquation"],
Cell[175924, 5936, 242, 5, 29, "Text"],
Cell[176169, 5943, 164, 3, 20, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[176370, 5951, 101, 1, 32, "Subsection"],
Cell[176474, 5954, 209, 4, 29, "Text"],
Cell[176686, 5960, 208, 5, 20, "NumberedEquation"],
Cell[176897, 5967, 98, 1, 20, "Text"],
Cell[176998, 5970, 208, 5, 20, "NumberedEquation"],
Cell[177209, 5977, 105, 1, 20, "Text"],
Cell[177317, 5980, 344, 6, 28, "NumberedEquation"],
Cell[177664, 5988, 124, 1, 20, "Text"],
Cell[177791, 5991, 266, 5, 20, "NumberedEquation"],
Cell[178060, 5998, 106, 1, 20, "Text"],
Cell[178169, 6001, 287, 6, 20, "NumberedEquation"],
Cell[178459, 6009, 113, 1, 20, "Text"],
Cell[178575, 6012, 244, 5, 20, "NumberedEquation"],
Cell[178822, 6019, 361, 8, 40, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[179220, 6032, 101, 1, 32, "Subsection"],
Cell[179324, 6035, 824, 17, 73, "Text"],
Cell[180151, 6054, 143, 2, 20, "NumberedEquation"],
Cell[180297, 6058, 131, 2, 29, "NumberedEquation"],
Cell[180431, 6062, 147, 4, 20, "Text"],
Cell[180581, 6068, 949, 18, 76, "NumberedEquation"],
Cell[181533, 6088, 570, 14, 62, "Text"],
Cell[182106, 6104, 169, 3, 20, "NumberedEquation"],
Cell[182278, 6109, 168, 4, 20, "Text"],
Cell[182449, 6115, 198, 4, 32, "NumberedEquation"],
Cell[182650, 6121, 87, 1, 20, "Text"],
Cell[182740, 6124, 135, 2, 32, "NumberedEquation"],
Cell[182878, 6128, 89, 1, 20, "Text"],
Cell[182970, 6131, 295, 6, 34, "NumberedEquation"],
Cell[183268, 6139, 199, 6, 20, "Text"],
Cell[183470, 6147, 606, 12, 50, "NumberedEquation"],
Cell[184079, 6161, 243, 5, 38, "NumberedEquation"],
Cell[184325, 6168, 654, 11, 117, "Text"],

Cell[CellGroupData[{
Cell[185004, 6183, 99, 1, 28, "Subsubsection"],
Cell[185106, 6186, 503, 14, 40, "Text"],
Cell[185612, 6202, 231, 5, 32, "NumberedEquation"],
Cell[185846, 6209, 294, 7, 33, "NumberedEquation"],
Cell[186143, 6218, 145, 4, 20, "Text"],
Cell[186291, 6224, 214, 5, 20, "NumberedEquation"],
Cell[186508, 6231, 240, 9, 20, "Text"],
Cell[186751, 6242, 166, 3, 20, "NumberedEquation"],
Cell[186920, 6247, 78, 1, 20, "Text"],
Cell[187001, 6250, 235, 4, 34, "NumberedEquation"],
Cell[187239, 6256, 163, 3, 29, "NumberedEquation"],
Cell[187405, 6261, 266, 9, 20, "Text"],
Cell[187674, 6272, 364, 7, 33, "NumberedEquation"],
Cell[188041, 6281, 177, 6, 20, "Text"],
Cell[188221, 6289, 387, 8, 36, "NumberedEquation"],
Cell[188611, 6299, 213, 6, 20, "Text"],
Cell[188827, 6307, 402, 8, 36, "NumberedEquation"],
Cell[189232, 6317, 168, 5, 20, "Text"],

Cell[CellGroupData[{
Cell[189425, 6326, 172, 4, 25, "Input"],
Cell[189600, 6332, 460, 10, 80, "Output"]
}, Open  ]],
Cell[190075, 6345, 146, 4, 20, "Text"],
Cell[190224, 6351, 641, 13, 68, "NumberedEquation"],
Cell[190868, 6366, 173, 6, 20, "Text"],
Cell[191044, 6374, 324, 7, 34, "NumberedEquation"],
Cell[191371, 6383, 189, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[191585, 6391, 195, 3, 37, "Input"],
Cell[191783, 6396, 91, 1, 57, "Output"]
}, Open  ]],
Cell[191889, 6400, 77, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[191991, 6405, 193, 3, 37, "Input"],
Cell[192187, 6410, 84, 1, 48, "Output"]
}, Open  ]],
Cell[192286, 6414, 217, 5, 29, "Text"],
Cell[192506, 6421, 245, 5, 37, "Input"],
Cell[192754, 6428, 272, 5, 37, "Input"],
Cell[193029, 6435, 151, 4, 20, "Text"],
Cell[193183, 6441, 195, 4, 25, "Input"],
Cell[193381, 6447, 197, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[193603, 6455, 193, 4, 25, "Input"],
Cell[193799, 6461, 284, 5, 85, "Output"]
}, Open  ]],
Cell[194098, 6469, 189, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[194312, 6477, 120, 2, 25, "Input"],
Cell[194435, 6481, 53, 1, 36, "Output"]
}, Open  ]],
Cell[194503, 6485, 128, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[194656, 6490, 141, 2, 25, "Input"],
Cell[194800, 6494, 35, 1, 36, "Output"]
}, Open  ]],
Cell[194850, 6498, 487, 11, 40, "Text"],
Cell[195340, 6511, 427, 9, 40, "NumberedEquation"],
Cell[195770, 6522, 257, 6, 20, "Text"],
Cell[196030, 6530, 179, 4, 20, "NumberedEquation"],
Cell[196212, 6536, 77, 1, 20, "Text"],
Cell[196292, 6539, 188, 4, 29, "NumberedEquation"],
Cell[196483, 6545, 264, 9, 20, "Text"],
Cell[196750, 6556, 501, 8, 77, "NumberedEquation",
  PageBreakAbove->False],
Cell[197254, 6566, 421, 11, 29, "Text",
  PageBreakAbove->False],
Cell[197678, 6579, 237, 5, 29, "NumberedEquation",
  PageBreakAbove->False],
Cell[197918, 6586, 116, 2, 20, "Text",
  PageBreakAbove->False],
Cell[198037, 6590, 473, 7, 76, "NumberedEquation",
  PageBreakAbove->False],
Cell[198513, 6599, 524, 13, 40, "Text",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[199074, 6617, 136, 2, 28, "Subsubsection",
  PageBreakAbove->False],
Cell[199213, 6621, 655, 10, 73, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[199917, 6637, 160, 2, 32, "Subsection",
  PageBreakAbove->False],
Cell[200080, 6641, 2232, 34, 249, "Text",
  PageBreakWithin->True],
Cell[202315, 6677, 243, 5, 36, "NumberedEquation"],
Cell[202561, 6684, 829, 26, 52, "Text"],
Cell[203393, 6712, 628, 13, 32, "NumberedEquation"],
Cell[204024, 6727, 926, 20, 84, "Text"],
Cell[204953, 6749, 116, 1, 20, "Text"],
Cell[205072, 6752, 144, 2, 20, "NumberedEquation"],
Cell[205219, 6756, 184, 6, 20, "Text"],
Cell[205406, 6764, 182, 4, 20, "NumberedEquation"],
Cell[205591, 6770, 163, 6, 20, "Text"],
Cell[205757, 6778, 244, 6, 30, "NumberedEquation"],
Cell[206004, 6786, 123, 1, 20, "Text"],
Cell[206130, 6789, 126, 2, 29, "NumberedEquation"],
Cell[206259, 6793, 147, 2, 20, "NumberedEquation"],
Cell[206409, 6797, 107, 1, 20, "Text"],
Cell[206519, 6800, 628, 13, 32, "NumberedEquation"],
Cell[207150, 6815, 231, 6, 20, "Text"],
Cell[207384, 6823, 141, 2, 20, "NumberedEquation"],
Cell[207528, 6827, 171, 3, 20, "NumberedEquation"],
Cell[207702, 6832, 168, 6, 20, "Text"],
Cell[207873, 6840, 244, 6, 30, "NumberedEquation"],
Cell[208120, 6848, 178, 4, 20, "Text"],
Cell[208301, 6854, 282, 6, 21, "NumberedEquation"],
Cell[208586, 6862, 172, 6, 20, "Text"],
Cell[208761, 6870, 128, 2, 20, "NumberedEquation"],
Cell[208892, 6874, 107, 1, 20, "Text"],
Cell[209002, 6877, 142, 2, 20, "NumberedEquation"],
Cell[209147, 6881, 218, 6, 20, "Text"],
Cell[209368, 6889, 210, 4, 21, "NumberedEquation"],
Cell[209581, 6895, 203, 6, 20, "Text"],
Cell[209787, 6903, 272, 6, 30, "NumberedEquation"],
Cell[210062, 6911, 152, 4, 20, "Text"],
Cell[210217, 6917, 151, 3, 29, "NumberedEquation"],
Cell[210371, 6922, 215, 6, 20, "Text"],
Cell[210589, 6930, 128, 1, 25, "Input"],
Cell[210720, 6933, 237, 7, 20, "Text"],
Cell[210960, 6942, 630, 14, 90, "Input"],
Cell[211593, 6958, 174, 4, 20, "Text"],
Cell[211770, 6964, 296, 5, 37, "Input"],
Cell[212069, 6971, 148, 4, 20, "Text"],
Cell[212220, 6977, 223, 4, 38, "Input"],
Cell[212446, 6983, 175, 4, 20, "Text"],
Cell[212624, 6989, 129, 2, 25, "Input"],
Cell[212756, 6993, 900, 20, 125, "Input"],
Cell[213659, 7015, 250, 8, 20, "Text"],

Cell[CellGroupData[{
Cell[213934, 7027, 209, 5, 25, "Input"],
Cell[214146, 7034, 387, 8, 53, "Output"]
}, Open  ]],
Cell[214548, 7045, 207, 4, 29, "Text"],
Cell[214758, 7051, 353, 5, 52, "Input"],
Cell[215114, 7058, 552, 12, 62, "Input"],
Cell[215669, 7072, 228, 5, 29, "Text"],
Cell[215900, 7079, 144, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[216069, 7085, 137, 2, 25, "Input"],
Cell[216209, 7089, 35, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[216281, 7095, 160, 3, 25, "Input"],
Cell[216444, 7100, 35, 1, 36, "Output"]
}, Open  ]],
Cell[216494, 7104, 85, 1, 20, "Text"],
Cell[216582, 7107, 246, 5, 25, "Input"],
Cell[216831, 7114, 339, 10, 29, "Text"],

Cell[CellGroupData[{
Cell[217195, 7128, 166, 3, 25, "Input"],
Cell[217364, 7133, 617, 10, 121, "Output"]
}, Open  ]],
Cell[217996, 7146, 232, 6, 20, "Text"],
Cell[218231, 7154, 633, 12, 66, "NumberedEquation"],
Cell[218867, 7168, 126, 1, 20, "Text"],
Cell[218996, 7171, 410, 9, 38, "NumberedEquation"],
Cell[219409, 7182, 97, 1, 20, "Text"],
Cell[219509, 7185, 506, 11, 58, "NumberedEquation"],
Cell[220018, 7198, 174, 4, 20, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[220229, 7207, 105, 1, 32, "Subsection"],
Cell[220337, 7210, 791, 22, 194, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[221177, 7238, 118, 1, 38, "Section"],
Cell[221298, 7241, 223, 5, 29, "Text"],
Cell[221524, 7248, 349, 7, 32, "NumberedEquation"],
Cell[221876, 7257, 1378, 29, 128, "Text"],
Cell[223257, 7288, 146, 3, 56, "NumberedEquation"],
Cell[223406, 7293, 340, 10, 29, "Text"],
Cell[223749, 7305, 363, 6, 56, "NumberedEquation"],
Cell[224115, 7313, 87, 1, 20, "Text"],
Cell[224205, 7316, 356, 7, 32, "NumberedEquation"],
Cell[224564, 7325, 401, 8, 40, "Text"],
Cell[224968, 7335, 135, 2, 20, "NumberedEquation"],
Cell[225106, 7339, 80, 1, 20, "Text"],
Cell[225189, 7342, 201, 5, 20, "NumberedEquation"],
Cell[225393, 7349, 120, 1, 20, "Text"],
Cell[225516, 7352, 474, 9, 32, "NumberedEquation"],
Cell[225993, 7363, 123, 1, 20, "Text"],
Cell[226119, 7366, 167, 3, 30, "NumberedEquation"],
Cell[226289, 7371, 171, 3, 30, "NumberedEquation"],
Cell[226463, 7376, 86, 1, 20, "Text"],
Cell[226552, 7379, 214, 4, 32, "NumberedEquation"],
Cell[226769, 7385, 253, 6, 29, "Text"],
Cell[227025, 7393, 207, 4, 32, "NumberedEquation"],
Cell[227235, 7399, 1021, 25, 140, "Text"],

Cell[CellGroupData[{
Cell[228281, 7428, 115, 1, 32, "Subsection"],
Cell[228399, 7431, 228, 5, 29, "Text"],
Cell[228630, 7438, 454, 11, 56, "NumberedEquation"],
Cell[229087, 7451, 127, 1, 20, "Text"],
Cell[229217, 7454, 252, 6, 34, "NumberedEquation"],
Cell[229472, 7462, 167, 4, 20, "Text"],
Cell[229642, 7468, 390, 8, 34, "NumberedEquation"],
Cell[230035, 7478, 156, 4, 20, "Text"],
Cell[230194, 7484, 228, 4, 33, "NumberedEquation"],
Cell[230425, 7490, 232, 5, 20, "Text"],
Cell[230660, 7497, 353, 5, 40, "NumberedEquation"],
Cell[231016, 7504, 92, 1, 20, "Text"],
Cell[231111, 7507, 360, 6, 36, "NumberedEquation"],
Cell[231474, 7515, 225, 6, 20, "Text"],
Cell[231702, 7523, 299, 5, 33, "NumberedEquation"],
Cell[232004, 7530, 196, 4, 29, "Text"],
Cell[232203, 7536, 168, 3, 36, "NumberedEquation"],
Cell[232374, 7541, 551, 10, 52, "Text"],
Cell[232928, 7553, 244, 6, 20, "NumberedEquation"],
Cell[233175, 7561, 119, 1, 20, "Text"],
Cell[233297, 7564, 220, 5, 20, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[233554, 7574, 122, 1, 32, "Subsection"],
Cell[233679, 7577, 513, 14, 40, "Text"],
Cell[234195, 7593, 171, 3, 30, "NumberedEquation"],
Cell[234369, 7598, 90, 1, 20, "Text"],
Cell[234462, 7601, 267, 6, 34, "NumberedEquation"],
Cell[234732, 7609, 324, 9, 29, "Text"],
Cell[235059, 7620, 240, 4, 33, "NumberedEquation"],
Cell[235302, 7626, 389, 7, 29, "Text"],
Cell[235694, 7635, 262, 4, 33, "NumberedEquation"],
Cell[235959, 7641, 98, 1, 20, "Text"],
Cell[236060, 7644, 272, 4, 40, "NumberedEquation"],
Cell[236335, 7650, 103, 1, 20, "Text"],
Cell[236441, 7653, 165, 3, 20, "NumberedEquation"],
Cell[236609, 7658, 207, 6, 21, "Text"],
Cell[236819, 7666, 190, 4, 21, "NumberedEquation"],
Cell[237012, 7672, 209, 6, 21, "Text"],
Cell[237224, 7680, 210, 5, 24, "NumberedEquation"],
Cell[237437, 7687, 420, 7, 49, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[237906, 7700, 98, 1, 44, "Section"],
Cell[238007, 7703, 73, 1, 21, "Text"],
Cell[238083, 7706, 1420, 32, 253, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

