(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    193732,       5042]*)
(*NotebookOutlinePosition[    195625,       5099]*)
(*  CellTagsIndexPosition[    195581,       5095]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 5. 
Log and Power Contracts\
\>", "Title",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["Simple Analytical Models Explored", "Subsubtitle",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["5.1 Introduction", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\

It has become rather traditional to begin a discussion of option pricing by \
starting with the vanilla European Calls and Puts. Other instruments are \
generally labelled as \"exotic\". In fact, as you will have seen in Chapter \
4, there are simpler solutions to the Black-Scholes equation than those \
pertaining to the standard Calls and Puts. We shall begin our discussion by \
investigating these simpler instruments. Mathematically, it makes sense to \
investigate the simpler cases first. We can get a feeling for the behaviour \
of options without needing to introduce any \"special functions\". Rather \
more importantly, we shall not allow ourselves to be drawn into any rash \
generalizations or inferences from the vanilla European case by prematurely \
focusing on those cases. It is a good idea early on to dispel, for example, \
any notion of \[CapitalDelta] as being the probability of anything.

In this chapter, we shall investigate the simple log and power solutions of \
the Black-Scholes equation. In Chapter 6 we shall introduce the next most \
complex case, that of Binary options. That will require an appreciation of \
how to compute the cumulative normal distribution. Then, in Chapter 7, we \
shall turn to the vanilla European cases.

The simple contracts we shall consider here are certainly not devoid of \
financial interest. The reader is referred to the excellent discussion by \
Neuberger (1990). Such contracts are attracting increasing attention as a \
trading instrument (see e.g. Boussard, 1997). Notably, as we shall see, the \
hedge position for a log contract is independent of volatility.\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.2 The Log Contract", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This was found in Chapter 4 by the use of elementary separation of \
variables applied to the time-dependent Black-Scholes equation. The value of \
the contract maturing at time ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " is given by"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = Z\ \((log(S/K) + \((r - q - \[Sigma]\^2\/2)\)\ \((T - t)\))\)\ 
        \[ExponentialE]\^\(\(-r\)\ \((T - t)\)\)\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Its value at maturity (",
  Cell[BoxData[
      \(TraditionalForm\`t = T\)]],
  ") is"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V\  = \ Z\ \(log(S/K)\)\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is what is known as the \"log contract\" - it represents an \
instrument that pays the log of the asset price at maturity. In what follows \
we shall set ",
  Cell[BoxData[
      \(TraditionalForm\`Z = 1\)]],
  ". This is perhaps the simplest interesting solution to the Black-Scholes \
equation. Its form can be simplified further by working with forward prices, \
but we shall keep it in the form given. Let's define the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " version of this function, defined in terms of time to maturity, and plot \
it as a function of price and time:"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
LogContract[S_, K_, r_, q_, \[Sigma]_, t_] := 
(Log[S/K] + (r - q - \[Sigma]^2/2)*t) Exp[-r t]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[LogContract[S, 10, 0.1, 0, 0.2, \ t], \ {S, \ 5, \ 15}, 
      \ {t, \ 0, \ 2}, \ AxesLabel\  -> \ {"\<S\>", \ "\<t\>", \ "\<\>"}]; 
    \)\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.08608 .22315 -6 -8.87306 ]
[.08608 .22315 0 .12694 ]
[.20731 .17545 -5.83689 -9 ]
[.20731 .17545 .16311 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.46808 .07289 -10.673 -9 ]
[.46808 .07289 1.32698 0 ]
[.60856 .01769 -10.1726 -9 ]
[.60856 .01769 1.82736 0 ]
[.29165 .07573 -9.31117 -14 ]
[.29165 .07573 .68883 0 ]
[.70096 -0.00478 0 -6.26206 ]
[.70096 -0.00478 6 2.73794 ]
[.78313 .12104 0 -6.13858 ]
[.78313 .12104 18 2.86142 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9224 .33431 0 -5.93715 ]
[.9224 .33431 18 3.06285 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.91861 .21225 0 -9.38198 ]
[.91861 .21225 10 4.61802 ]
[.01851 .30022 -24 -2.77199 ]
[.01851 .30022 0 6.22801 ]
[.01107 .34688 -30 -2.81151 ]
[.01107 .34688 0 6.18849 ]
[.0034 .39495 -6 -2.85237 ]
[.0034 .39495 0 6.14763 ]
[-0.00451 .44451 -24 -2.89462 ]
[-0.00451 .44451 0 6.10538 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -18 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(S) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.68874 0 m
.68263 .00239 L
s
[(0)] .70096 -0.00478 -1 .39157 Mshowa
.7708 .12553 m
.76464 .12778 L
s
[(0.5)] .78313 .12104 -1 .36413 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(1)] .8565 .23339 -1 .34028 Mshowa
.9099 .3383 m
.90365 .34029 L
s
[(1.5)] .9224 .33431 -1 .31937 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(2)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70593 .0263 m
.70226 .02771 L
s
.72272 .05198 m
.71904 .05338 L
s
.73912 .07706 m
.73543 .07845 L
s
.75514 .10158 m
.75145 .10294 L
s
.78611 .14895 m
.78241 .15028 L
s
.80108 .17185 m
.79737 .17317 L
s
.81573 .19425 m
.81201 .19555 L
s
.83006 .21617 m
.82633 .21745 L
s
.8578 .2586 m
.85407 .25986 L
s
.87123 .27915 m
.8675 .28039 L
s
.88439 .29928 m
.88065 .3005 L
s
.89727 .31899 m
.89353 .3202 L
s
.92227 .35722 m
.91851 .3584 L
s
.93439 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .3951 L
s
.95792 .41176 m
.95416 .41291 L
s
gsave
.91861 .21225 -61 -13.382 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03076 .29552 m
.03689 .29316 L
s
[(-0.5)] .01851 .30022 1 -0.384 Mshowa
.02335 .34227 m
.0295 .33996 L
s
[(-0.25)] .01107 .34688 1 -0.37522 Mshowa
.01572 .39044 m
.02188 .38818 L
s
[(0)] .0034 .39495 1 -0.36614 Mshowa
.00785 .4401 m
.01403 .4379 L
s
[(0.25)] -0.00451 .44451 1 -0.35675 Mshowa
.125 Mabswid
.0293 .30476 m
.03297 .30335 L
s
.02782 .31405 m
.0315 .31265 L
s
.02634 .3234 m
.03002 .322 L
s
.02485 .3328 m
.02854 .33142 L
s
.02184 .35178 m
.02553 .35041 L
s
.02033 .36136 m
.02402 .35999 L
s
.0188 .37099 m
.02249 .36963 L
s
.01726 .38069 m
.02096 .37933 L
s
.01416 .40025 m
.01786 .3989 L
s
.0126 .41012 m
.0163 .40878 L
s
.01103 .42005 m
.01473 .41872 L
s
.00944 .43004 m
.01315 .42872 L
s
.00625 .45022 m
.00996 .4489 L
s
.00463 .4604 m
.00834 .45909 L
s
.00301 .47064 m
.00672 .46934 L
s
.00138 .48095 m
.00509 .47965 L
s
.03222 .28633 m
.03589 .28492 L
s
.03366 .2772 m
.03733 .27578 L
s
.0351 .26812 m
.03877 .2667 L
s
.03653 .2591 m
.0402 .25767 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.371 .606 .94 r
.38144 .62373 .40362 .6466 .44039 .6544 .41837 .6316 Metetra
.417 .632 .943 r
.41837 .6316 .44039 .6544 .4781 .66014 .45628 .63739 Metetra
.45 .649 .944 r
.45628 .63739 .4781 .66014 .51673 .66424 .49514 .64149 Metetra
.474 .663 .945 r
.49514 .64149 .51673 .66424 .55624 .66695 .53491 .64419 Metetra
.493 .673 .945 r
.53491 .64419 .55624 .66695 .59661 .66851 .57559 .64569 Metetra
.508 .682 .944 r
.57559 .64569 .59661 .66851 .63785 .66906 .61715 .64617 Metetra
.521 .689 .944 r
.61715 .64617 .63785 .66906 .67995 .66872 .65959 .64575 Metetra
.531 .695 .944 r
.65959 .64575 .67995 .66872 .72291 .6676 .70293 .64452 Metetra
.539 .7 .944 r
.70293 .64452 .72291 .6676 .76673 .66577 .74716 .64256 Metetra
.546 .705 .944 r
.74716 .64256 .76673 .66577 .81143 .66329 .79229 .63994 Metetra
.553 .709 .944 r
.79229 .63994 .81143 .66329 .85702 .66021 .83833 .63671 Metetra
.558 .713 .944 r
.83833 .63671 .85702 .66021 .9035 .65658 .88531 .6329 Metetra
.563 .716 .944 r
.88531 .6329 .9035 .65658 .9509 .65244 .93322 .62857 Metetra
.567 .719 .945 r
.93322 .62857 .9509 .65244 .99922 .6478 .98208 .62373 Metetra
.366 .603 .939 r
.35872 .60023 .38144 .62373 .41837 .6316 .3958 .60818 Metetra
.412 .629 .943 r
.3958 .60818 .41837 .6316 .45628 .63739 .43391 .614 Metetra
.446 .647 .944 r
.43391 .614 .45628 .63739 .49514 .64149 .473 .6181 Metetra
.471 .661 .944 r
.473 .6181 .49514 .64149 .53491 .64419 .51303 .62077 Metetra
.491 .672 .944 r
.51303 .62077 .53491 .64419 .57559 .64569 .554 .62222 Metetra
.506 .68 .944 r
.554 .62222 .57559 .64569 .61715 .64617 .59588 .62263 Metetra
.519 .688 .944 r
.59588 .62263 .61715 .64617 .65959 .64575 .63868 .6221 Metetra
.529 .694 .944 r
.63868 .6221 .65959 .64575 .70293 .64452 .68239 .62075 Metetra
.538 .699 .944 r
.68239 .62075 .70293 .64452 .74716 .64256 .72703 .61866 Metetra
.545 .704 .944 r
.72703 .61866 .74716 .64256 .79229 .63994 .7726 .61588 Metetra
.551 .708 .944 r
.7726 .61588 .79229 .63994 .83833 .63671 .81911 .61248 Metetra
.557 .712 .944 r
.81911 .61248 .83833 .63671 .88531 .6329 .86657 .60849 Metetra
.561 .715 .944 r
.86657 .60849 .88531 .6329 .93322 .62857 .91501 .60396 Metetra
.566 .719 .945 r
.91501 .60396 .93322 .62857 .98208 .62373 .96443 .5989 Metetra
.36 .6 .938 r
.33544 .57609 .35872 .60023 .3958 .60818 .37267 .58411 Metetra
.408 .626 .942 r
.37267 .58411 .3958 .60818 .43391 .614 .41097 .58996 Metetra
.442 .645 .944 r
.41097 .58996 .43391 .614 .473 .6181 .45028 .59405 Metetra
.468 .659 .944 r
.45028 .59405 .473 .6181 .51303 .62077 .49058 .59668 Metetra
.488 .67 .944 r
.49058 .59668 .51303 .62077 .554 .62222 .53183 .59807 Metetra
.504 .679 .944 r
.53183 .59807 .554 .62222 .59588 .62263 .57403 .59838 Metetra
.516 .686 .944 r
.57403 .59838 .59588 .62263 .63868 .6221 .61718 .59775 Metetra
.527 .693 .944 r
.61718 .59775 .63868 .6221 .68239 .62075 .66128 .59627 Metetra
.536 .698 .944 r
.66128 .59627 .68239 .62075 .72703 .61866 .70632 .59403 Metetra
.543 .703 .944 r
.70632 .59403 .72703 .61866 .7726 .61588 .75233 .59109 Metetra
.55 .707 .944 r
.75233 .59109 .7726 .61588 .81911 .61248 .79931 .5875 Metetra
.555 .711 .944 r
.79931 .5875 .81911 .61248 .86657 .60849 .84728 .58332 Metetra
.56 .714 .944 r
.84728 .58332 .86657 .60849 .91501 .60396 .89624 .57856 Metetra
.564 .718 .944 r
.89624 .57856 .91501 .60396 .96443 .5989 .94623 .57328 Metetra
.354 .596 .938 r
.31158 .55129 .33544 .57609 .37267 .58411 .34896 .55937 Metetra
.403 .623 .942 r
.34896 .55937 .37267 .58411 .41097 .58996 .38744 .56523 Metetra
.439 .643 .943 r
.38744 .56523 .41097 .58996 .45028 .59405 .42697 .56931 Metetra
.465 .657 .944 r
.42697 .56931 .45028 .59405 .49058 .59668 .46751 .57189 Metetra
.485 .668 .944 r
.46751 .57189 .49058 .59668 .53183 .59807 .50906 .57321 Metetra
.501 .677 .944 r
.50906 .57321 .53183 .59807 .57403 .59838 .55158 .57342 Metetra
.514 .685 .944 r
.55158 .57342 .57403 .59838 .61718 .59775 .59508 .57267 Metetra
.525 .691 .944 r
.59508 .57267 .61718 .59775 .66128 .59627 .63956 .57105 Metetra
.534 .697 .944 r
.63956 .57105 .66128 .59627 .70632 .59403 .68502 .56865 Metetra
.542 .702 .944 r
.68502 .56865 .70632 .59403 .75233 .59109 .73147 .56553 Metetra
.548 .706 .944 r
.73147 .56553 .75233 .59109 .79931 .5875 .77892 .56174 Metetra
.554 .71 .944 r
.77892 .56174 .79931 .5875 .84728 .58332 .8274 .55734 Metetra
.559 .713 .944 r
.8274 .55734 .84728 .58332 .89624 .57856 .8769 .55236 Metetra
.563 .717 .944 r
.8769 .55236 .89624 .57856 .94623 .57328 .92746 .54682 Metetra
.348 .592 .937 r
.28713 .52579 .31158 .55129 .34896 .55937 .32463 .53392 Metetra
.399 .62 .941 r
.32463 .53392 .34896 .55937 .38744 .56523 .36329 .5398 Metetra
.435 .64 .943 r
.36329 .5398 .38744 .56523 .42697 .56931 .40303 .54385 Metetra
.462 .655 .944 r
.40303 .54385 .42697 .56931 .46751 .57189 .44383 .54637 Metetra
.482 .666 .944 r
.44383 .54637 .46751 .57189 .50906 .57321 .48565 .5476 Metetra
.499 .676 .944 r
.48565 .5476 .50906 .57321 .55158 .57342 .5285 .54771 Metetra
.512 .683 .944 r
.5285 .54771 .55158 .57342 .59508 .57267 .57235 .54682 Metetra
.523 .69 .944 r
.57235 .54682 .59508 .57267 .63956 .57105 .61721 .54505 Metetra
.532 .696 .944 r
.61721 .54505 .63956 .57105 .68502 .56865 .66308 .54247 Metetra
.54 .701 .944 r
.66308 .54247 .68502 .56865 .73147 .56553 .70998 .53916 Metetra
.547 .705 .944 r
.70998 .53916 .73147 .56553 .77892 .56174 .75792 .53516 Metetra
.553 .709 .944 r
.75792 .53516 .77892 .56174 .8274 .55734 .80691 .53053 Metetra
.558 .713 .944 r
.80691 .53053 .8274 .55734 .8769 .55236 .85696 .5253 Metetra
.562 .716 .944 r
.85696 .5253 .8769 .55236 .92746 .54682 .9081 .5195 Metetra
.342 .588 .936 r
.26206 .49957 .28713 .52579 .32463 .53392 .29968 .50775 Metetra
.394 .617 .941 r
.29968 .50775 .32463 .53392 .36329 .5398 .3385 .51362 Metetra
.431 .638 .943 r
.3385 .51362 .36329 .5398 .40303 .54385 .37845 .51764 Metetra
.458 .653 .944 r
.37845 .51764 .40303 .54385 .44383 .54637 .41949 .52009 Metetra
.479 .665 .944 r
.41949 .52009 .44383 .54637 .48565 .5476 .4616 .52123 Metetra
.496 .674 .944 r
.4616 .52123 .48565 .5476 .5285 .54771 .50476 .52121 Metetra
.51 .682 .944 r
.50476 .52121 .5285 .54771 .57235 .54682 .54896 .52018 Metetra
.521 .689 .944 r
.54896 .52018 .57235 .54682 .61721 .54505 .5942 .51824 Metetra
.53 .694 .944 r
.5942 .51824 .61721 .54505 .66308 .54247 .6405 .51547 Metetra
.538 .699 .944 r
.6405 .51547 .66308 .54247 .70998 .53916 .68785 .51195 Metetra
.545 .704 .944 r
.68785 .51195 .70998 .53916 .75792 .53516 .73627 .50772 Metetra
.551 .708 .944 r
.73627 .50772 .75792 .53516 .80691 .53053 .78578 .50284 Metetra
.556 .712 .944 r
.78578 .50284 .80691 .53053 .85696 .5253 .83639 .49734 Metetra
.561 .715 .944 r
.83639 .49734 .85696 .5253 .9081 .5195 .88812 .49126 Metetra
.336 .584 .935 r
.23635 .4726 .26206 .49957 .29968 .50775 .27408 .48081 Metetra
.389 .614 .94 r
.27408 .48081 .29968 .50775 .3385 .51362 .31305 .48668 Metetra
.426 .635 .942 r
.31305 .48668 .3385 .51362 .37845 .51764 .3532 .49065 Metetra
.455 .651 .943 r
.3532 .49065 .37845 .51764 .41949 .52009 .39448 .49302 Metetra
.476 .663 .944 r
.39448 .49302 .41949 .52009 .4616 .52123 .43687 .49405 Metetra
.494 .672 .944 r
.43687 .49405 .4616 .52123 .50476 .52121 .48034 .49389 Metetra
.507 .68 .944 r
.48034 .49389 .50476 .52121 .54896 .52018 .52489 .4927 Metetra
.519 .687 .944 r
.52489 .4927 .54896 .52018 .5942 .51824 .57052 .49058 Metetra
.529 .693 .944 r
.57052 .49058 .5942 .51824 .6405 .51547 .61723 .4876 Metetra
.537 .698 .944 r
.61723 .4876 .6405 .51547 .68785 .51195 .66503 .48386 Metetra
.544 .703 .944 r
.66503 .48386 .68785 .51195 .73627 .50772 .71395 .47938 Metetra
.55 .707 .944 r
.71395 .47938 .73627 .50772 .78578 .50284 .76398 .47424 Metetra
.555 .71 .944 r
.76398 .47424 .78578 .50284 .83639 .49734 .81515 .46845 Metetra
.56 .714 .944 r
.81515 .46845 .83639 .49734 .88812 .49126 .86749 .46206 Metetra
.329 .58 .934 r
.20998 .44486 .23635 .4726 .27408 .48081 .2478 .4531 Metetra
.383 .611 .94 r
.2478 .4531 .27408 .48081 .31305 .48668 .28692 .45894 Metetra
.422 .633 .942 r
.28692 .45894 .31305 .48668 .3532 .49065 .32726 .46285 Metetra
.451 .648 .943 r
.32726 .46285 .3532 .49065 .39448 .49302 .36877 .46513 Metetra
.473 .661 .944 r
.36877 .46513 .39448 .49302 .43687 .49405 .41142 .46603 Metetra
.491 .671 .944 r
.41142 .46603 .43687 .49405 .48034 .49389 .45521 .46573 Metetra
.505 .679 .944 r
.45521 .46573 .48034 .49389 .52489 .4927 .5001 .46436 Metetra
.517 .686 .944 r
.5001 .46436 .52489 .4927 .57052 .49058 .54611 .46203 Metetra
.527 .692 .943 r
.54611 .46203 .57052 .49058 .61723 .4876 .59325 .45884 Metetra
.535 .697 .943 r
.59325 .45884 .61723 .4876 .66503 .48386 .64151 .45484 Metetra
.542 .702 .944 r
.64151 .45484 .66503 .48386 .71395 .47938 .69091 .45011 Metetra
.548 .706 .944 r
.69091 .45011 .71395 .47938 .76398 .47424 .74148 .44468 Metetra
.554 .709 .944 r
.74148 .44468 .76398 .47424 .81515 .46845 .79322 .43859 Metetra
.558 .713 .944 r
.79322 .43859 .81515 .46845 .86749 .46206 .84617 .43187 Metetra
.322 .576 .933 r
.18291 .41632 .20998 .44486 .2478 .4531 .22082 .42456 Metetra
.378 .608 .939 r
.22082 .42456 .2478 .4531 .28692 .45894 .26007 .43037 Metetra
.418 .63 .942 r
.26007 .43037 .28692 .45894 .32726 .46285 .30059 .4342 Metetra
.447 .646 .943 r
.30059 .4342 .32726 .46285 .36877 .46513 .34233 .43638 Metetra
.47 .659 .943 r
.34233 .43638 .36877 .46513 .41142 .46603 .38525 .43714 Metetra
.488 .669 .943 r
.38525 .43714 .41142 .46603 .45521 .46573 .42933 .43667 Metetra
.503 .677 .943 r
.42933 .43667 .45521 .46573 .5001 .46436 .47457 .4351 Metetra
.515 .684 .943 r
.47457 .4351 .5001 .46436 .54611 .46203 .52097 .43256 Metetra
.525 .69 .943 r
.52097 .43256 .54611 .46203 .59325 .45884 .56852 .42913 Metetra
.533 .696 .943 r
.56852 .42913 .59325 .45884 .64151 .45484 .61724 .42487 Metetra
.54 .7 .943 r
.61724 .42487 .64151 .45484 .69091 .45011 .66715 .41985 Metetra
.547 .705 .944 r
.66715 .41985 .69091 .45011 .74148 .44468 .71825 .41411 Metetra
.552 .708 .944 r
.71825 .41411 .74148 .44468 .79322 .43859 .77057 .40769 Metetra
.557 .712 .944 r
.77057 .40769 .79322 .43859 .84617 .43187 .82414 .40062 Metetra
.315 .572 .931 r
.15514 .38693 .18291 .41632 .22082 .42456 .1931 .39518 Metetra
.373 .604 .938 r
.1931 .39518 .22082 .42456 .26007 .43037 .23248 .40094 Metetra
.413 .627 .941 r
.23248 .40094 .26007 .43037 .30059 .4342 .27317 .40468 Metetra
.444 .644 .943 r
.27317 .40468 .30059 .4342 .34233 .43638 .31512 .40673 Metetra
.467 .657 .943 r
.31512 .40673 .34233 .43638 .38525 .43714 .3583 .40733 Metetra
.485 .667 .943 r
.3583 .40733 .38525 .43714 .42933 .43667 .40269 .40668 Metetra
.5 .676 .943 r
.40269 .40668 .42933 .43667 .47457 .4351 .44827 .4049 Metetra
.512 .683 .943 r
.44827 .4049 .47457 .4351 .52097 .43256 .49504 .40212 Metetra
.523 .689 .943 r
.49504 .40212 .52097 .43256 .56852 .42913 .54302 .39843 Metetra
.531 .694 .943 r
.54302 .39843 .56852 .42913 .61724 .42487 .5922 .39388 Metetra
.539 .699 .943 r
.5922 .39388 .61724 .42487 .66715 .41985 .6426 .38855 Metetra
.545 .703 .943 r
.6426 .38855 .66715 .41985 .71825 .41411 .69425 .38248 Metetra
.551 .707 .944 r
.69425 .38248 .71825 .41411 .77057 .40769 .74716 .37571 Metetra
.556 .711 .944 r
.74716 .37571 .77057 .40769 .82414 .40062 .80135 .36826 Metetra
.308 .568 .93 r
.12662 .35668 .15514 .38693 .1931 .39518 .16464 .3649 Metetra
.367 .601 .938 r
.16464 .3649 .1931 .39518 .23248 .40094 .20411 .3706 Metetra
.409 .624 .941 r
.20411 .3706 .23248 .40094 .27317 .40468 .24496 .37423 Metetra
.44 .641 .942 r
.24496 .37423 .27317 .40468 .31512 .40673 .28712 .37614 Metetra
.464 .655 .943 r
.28712 .37614 .31512 .40673 .3583 .40733 .33055 .37658 Metetra
.482 .665 .943 r
.33055 .37658 .3583 .40733 .40269 .40668 .37523 .37572 Metetra
.498 .674 .943 r
.37523 .37572 .40269 .40668 .44827 .4049 .42115 .37371 Metetra
.51 .681 .943 r
.42115 .37371 .44827 .4049 .49504 .40212 .46831 .37067 Metetra
.521 .688 .943 r
.46831 .37067 .49504 .40212 .54302 .39843 .5167 .36669 Metetra
.529 .693 .943 r
.5167 .36669 .54302 .39843 .5922 .39388 .56634 .36184 Metetra
.537 .698 .943 r
.56634 .36184 .5922 .39388 .6426 .38855 .61725 .35617 Metetra
.544 .702 .943 r
.61725 .35617 .6426 .38855 .69425 .38248 .66945 .34975 Metetra
.549 .706 .944 r
.66945 .34975 .69425 .38248 .74716 .37571 .72294 .34259 Metetra
.554 .71 .944 r
.72294 .34259 .74716 .37571 .80135 .36826 .77777 .33474 Metetra
.301 .563 .929 r
.09733 .32552 .12662 .35668 .16464 .3649 .13538 .33371 Metetra
.361 .597 .937 r
.13538 .33371 .16464 .3649 .20411 .3706 .17495 .33932 Metetra
.404 .621 .94 r
.17495 .33932 .20411 .3706 .24496 .37423 .21594 .34283 Metetra
.436 .639 .942 r
.21594 .34283 .24496 .37423 .28712 .37614 .2583 .34458 Metetra
.46 .653 .943 r
.2583 .34458 .28712 .37614 .33055 .37658 .30197 .34482 Metetra
.48 .663 .943 r
.30197 .34482 .33055 .37658 .37523 .37572 .34694 .34374 Metetra
.495 .672 .943 r
.34694 .34374 .37523 .37572 .42115 .37371 .39319 .34148 Metetra
.508 .68 .943 r
.39319 .34148 .42115 .37371 .46831 .37067 .44072 .33816 Metetra
.518 .686 .943 r
.44072 .33816 .46831 .37067 .5167 .36669 .48953 .33387 Metetra
.528 .692 .943 r
.48953 .33387 .5167 .36669 .56634 .36184 .53964 .32868 Metetra
.535 .697 .943 r
.53964 .32868 .56634 .36184 .61725 .35617 .59105 .32266 Metetra
.542 .701 .943 r
.59105 .32266 .61725 .35617 .66945 .34975 .6438 .31585 Metetra
.548 .705 .943 r
.6438 .31585 .66945 .34975 .72294 .34259 .69789 .30828 Metetra
.553 .709 .944 r
.69789 .30828 .72294 .34259 .77777 .33474 .75337 .3 Metetra
.294 .558 .927 r
.06725 .29342 .09733 .32552 .13538 .33371 .10531 .30156 Metetra
.356 .594 .936 r
.10531 .30156 .13538 .33371 .17495 .33932 .14495 .30707 Metetra
.399 .618 .94 r
.14495 .30707 .17495 .33932 .21594 .34283 .18608 .31044 Metetra
.432 .637 .942 r
.18608 .31044 .21594 .34283 .2583 .34458 .22861 .312 Metetra
.457 .65 .942 r
.22861 .312 .2583 .34458 .30197 .34482 .27252 .31203 Metetra
.477 .662 .943 r
.27252 .31203 .30197 .34482 .34694 .34374 .31776 .31069 Metetra
.492 .671 .943 r
.31776 .31069 .34694 .34374 .39319 .34148 .36434 .30816 Metetra
.505 .678 .943 r
.36434 .30816 .39319 .34148 .44072 .33816 .41224 .30453 Metetra
.516 .685 .943 r
.41224 .30453 .44072 .33816 .48953 .33387 .46147 .29991 Metetra
.526 .69 .943 r
.46147 .29991 .48953 .33387 .53964 .32868 .51203 .29436 Metetra
.533 .695 .943 r
.51203 .29436 .53964 .32868 .59105 .32266 .56396 .28795 Metetra
.54 .7 .943 r
.56396 .28795 .59105 .32266 .6438 .31585 .61726 .28072 Metetra
.546 .704 .943 r
.61726 .28072 .6438 .31585 .69789 .30828 .67196 .27272 Metetra
.551 .708 .944 r
.67196 .27272 .69789 .30828 .75337 .3 .72809 .26396 Metetra
.286 .553 .926 r
.03634 .26033 .06725 .29342 .10531 .30156 .07439 .2684 Metetra
.349 .59 .935 r
.07439 .2684 .10531 .30156 .14495 .30707 .11408 .27379 Metetra
.394 .615 .939 r
.11408 .27379 .14495 .30707 .18608 .31044 .15532 .27699 Metetra
.428 .634 .941 r
.15532 .27699 .18608 .31044 .22861 .312 .19803 .27836 Metetra
.453 .648 .942 r
.19803 .27836 .22861 .312 .27252 .31203 .24215 .27814 Metetra
.473 .66 .943 r
.24215 .27814 .27252 .31203 .31776 .31069 .28767 .27654 Metetra
.49 .669 .943 r
.28767 .27654 .31776 .31069 .36434 .30816 .33456 .2737 Metetra
.503 .677 .943 r
.33456 .2737 .36434 .30816 .41224 .30453 .38283 .26974 Metetra
.514 .683 .943 r
.38283 .26974 .41224 .30453 .46147 .29991 .43247 .26475 Metetra
.524 .689 .943 r
.43247 .26475 .46147 .29991 .51203 .29436 .48349 .25881 Metetra
.532 .694 .943 r
.48349 .25881 .51203 .29436 .56396 .28795 .53593 .25198 Metetra
.539 .699 .943 r
.53593 .25198 .56396 .28795 .61726 .28072 .58979 .24431 Metetra
.545 .703 .943 r
.58979 .24431 .61726 .28072 .67196 .27272 .6451 .23583 Metetra
.55 .707 .943 r
.6451 .23583 .67196 .27272 .72809 .26396 .70189 .22657 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -18 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(S) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  ImageSize->{184.25, 149.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003V0000^Q000`40O003h00OiWoo003VOol00>Io
o`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol00>Io
o`00iWoo0029Ool50004Ool3000BOol3000lOol008]oo`03001oogoo00Eoo`03001oogoo011oo`05
001oogooOol0000kOol008]oo`03001oogoo009oo`D0015oo`05001oogooOol0000kOol008]oo`03
001oogoo009oo`04001oogoo000BOol01@00Oomoogoo0000>goo002;Ool00`00Oomoo`03Ool00`00
Ool0000BOol01@00Oomoogoo0000>goo002;Ool00`00Oomoo`03Ool00`00Ool0000BOol01@00Oomo
ogoo0000>goo002;Ool00`00Oomoo`04Ool2000>Ool20002Ool01@00Oomoogoo0000>goo001AOol5
000cOol30007Ool00`00Oomoo`09Ool300000goo001oo`02Ool3000lOol0055oo`03001oogoo009o
o`03001oogoo04=oo`<000Aoo`80045oo`00DGoo00<007ooOol00Woo00<007ooOol0@Goo10001Goo
00<007oo0000@7oo001DOol3000GOol50002Ool5000MOol30009Ool00`00Ool00010Ool0055oo`<0
01aoo`03001oogoo00=oo`04001oogoo000KOol2000<Ool01000Ool00000?goo001AOol00`00Oomo
o`02Ool00`00Oomoo`0GOol00`00Oomoo`04Ool00`00Oomoo`0HOol300000goo001oo`0;Ool01@00
Oomoogoo0000?Woo001AOol00`00Oomoo`02Ool00`00Oomoo`0GOol00`00Oomoo`05Ool00`00Oomo
o`0EOol2000AOol01@00Oomoogoo0000?Woo001BOol5000IOol00`00Oomoo`06Ool00`00Oomoo`0A
Ool3000DOol01@00Oomoogoo0000?Goo001`Ool00`00Oomoo`06Ool00`00Oomoo`0?Ool2000GOol0
0`00Oomoo`02Ool00`00Oomoo`0jOol0071oo`03001oogoo009oo`05001oogooOol0000>Ool3000I
Ool00`00Oomoo`02Ool00`00Oomoo`0jOol006ioo`<000Eoo`<000aoo`<00003Ool007oo01Uoo`03
001oogoo00=oo`03001oogoo03Uoo`00Pgoo0P007goo00<007ooOol017oo00<007ooOol0>7oo0020
Ool3000QOol00`00Oomoo`04Ool00`00Oomoo`0hOol005Eoo`D000=oo`<001ioo`8002Aoo`03001o
ogoo00Eoo`03001oogoo03Moo`00Egoo00<007ooOol00Woo00D007ooOomoo`0001Yoo`<002Ioo`03
001oogoo00Ioo`03001oogoo03Ioo`00Egoo00<007ooOol00Woo00D007ooOomoo`0001Qoo`800003
Ool007oo02Ioo`03001oogoo00Ioo`03001oogoo03Ioo`00Egoo00<007ooOol00Woo00D007ooOomo
o`0001Eoo`<002]oo`03001oogoo00Moo`03001oogoo00Eoo`<000Ioo`8000Eoo`@001aoo`00Egoo
00<007ooOol00Woo00D007ooOomoo`00019oo`@002eoo`03001oogoo00Qoo`03001oogoo00=oo`05
001oogooOol00005Ool20009Ool00`00Oomoo`0IOol005Moo`03001oogoo009oo`05001oogooOol0
000@Ool2000aOol00`00Oomoo`08Ool00`00Oomoo`03Ool01@00Oomoogoo000047oo00<007ooOol0
6Goo001GOol00`00Oomoo`02Ool01@00Oomoogoo00003Goo0`00<goo00<007ooOol02Goo00<007oo
Ool00Woo00D007ooOomoo`00011oo`03001oogoo01Uoo`00EGoo0`001Goo0`0037oo0P0000=oo`00
Ool0<goo00<007ooOol02Goo00<007ooOol00Woo00D007ooOomoo`0000aoo`@001aoo`00JGoo0`00
>7oo00<007ooOol02Woo00D007ooOomoo`0000=oo`03001oogoo00Yoo`03001oogoo01eoo`00A7oo
0`007goo1000>goo00<007ooOol02Goo0P000Woo00D007ooOomoo`0000aoo`03001oogoo01eoo`00
@goo00D007ooOomoo`0001aoo`8003moo`03001oogoo00Yoo`04001oogooOol3000=Ool4000LOol0
04=oo`05001oogooOol0000IOol30011Ool00`00Oomoo`0;Ool00`00Oomoo`0`Ool004=oo`05001o
ogooOol0000GOol20002Ool00`00Oomoo`0oOol00`00Oomoo`0<Ool00`00Oomoo`0_Ool004Aoo`<0
01Eoo`<004Ioo`03001oogoo00aoo`80031oo`00@goo00D007ooOomoo`00019oo`800003Ool007oo
04Ioo`03001oogoo00aoo`80031oo`00@goo00D007ooOomoo`0000moo`<004]oo`03001oogoo00io
o`03001oogoo02eoo`00A7oo0`003Goo1000CGoo00<007ooOol03Woo00<007ooOol0;Goo001BOol2
001AOol00`00Oomoo`0?Ool00`00Oomoo`0/Ool002eoo`<001moo`<005=oo`03001oogoo00moo`80
01Yoo`<0011oo`00;7oo00D007ooOomoo`0001aoo`8005Ioo`03001oogoo011oo`03001oogoo01Mo
o`05001oogooOol0000?Ool002aoo`05001oogooOol0000IOol3001HOol00`00Oomoo`0AOol00`00
Oomoo`0FOol00`00Oomoo`0AOol002aoo`05001oogooOol0000GOol20002Ool00`00Oomoo`1FOol0
0`00Oomoo`0BOol00`00Oomoo`0EOol00`00Oomoo`0AOol002aoo`@001Eoo`<005eoo`03001oogoo
019oo`03001oogoo01Eoo`03001oogoo015oo`00;7oo00<007ooOol04goo0`0000=oo`00Ool0GGoo
00<007ooOol04goo00<007ooOol04Woo1P0047oo000]Ool00`00Oomoo`0@Ool2001TOol00`00Oomo
o`0COol00`00Oomoo`05Ool50009Ool00`00Oomoo`0AOol002ioo`<000aoo`@006Eoo`03001oogoo
01=oo`03001oogoo00Moo`03001oogoo00Uoo`03001oogoo015oo`00>goo0P00JGoo00<007ooOol0
4goo0P0027oo00<007ooOol07Goo000hOol4001ZOol00`00Oomoo`0EOol00`00Oomoo`05Ool00`00
Oomoo`0MOol003Ioo`8006]oo`@001Moo`03001oogoo00Eoo`03001oogoo01eoo`00<goo0`00J7oo
1@000TKN0P0067oo00<007ooOol017oo00<007ooOol07Goo000`Ool30003Ool00`00Oomoo`1MOol8
0008A]h00`00Oomoo`0FOol20005Ool00`00Oomoo`0MOol002ioo`8005moo`L000E6gP030016gTKN
00M6gP<001Qoo`04001oogooOol3000OOol002]oo`<0009oo`03001oogoo05Moo`D00096gP030016
gTKN00Q6gP030016gTKN00=6gP<00004A]h004KN000HOol00`00Oomoo`0ROol002Yoo`8005Ioo`P0
00Q6gP030016gTKN00Q6gP030016gP00008000E6gP8001Uoo`03001oogoo025oo`00:Woo0`00Bgoo
2P001DKN00<004KNA]h02DKN00<004KNA]h01TKN0`002DKN0P0067oo00<007ooOol08Goo000ZOol0
0`009SdV?@03000oOol90005@]h00`00A]i6gP08A]h00`00A]i6gP09A]h00`00A]i6gP02A]h30003
A]h00`00A]i6gP03A]h6000HOol00`00Oomoo`0POol002Uoo`<000@V?@H0031oo`T000E2_P030012
gT;N00M2gP030016gTKN00Q6gP030016gTKN00Q6gP030016gP00008000E6gPL000A6gP8001Uoo`03
001oogoo01moo`00:Goo0P0000=oo`009Sd012Hm00@002iM;Ud^GC0000E2_P030012_T:n00M2_P03
0012gT;N00M2gP030016gTKN00Q6gP030016gTKN00I6gP<000=6gPD00096gP030016gTKN00M6gP80
01Qoo`03001oogoo01moo`00:Goo00D007ooOomoo`0000DV?@03000^GBiM00<^G@03000bOS9n00Db
OP03000fWSJN00@fWP03000jWSZN00DjWP03000n_Sjn00Hn_P03000n_Sjn00Hn_P030012_T:n00M2
_P030012_T:n00M2_P030012gT;N00M2gP030016gTKN00Q6gP030016gTKN00=6gPP000Q6gP030016
gTKN00M6gP8001Moo`80021oo`00:Goo0P000goo00<002Hm9Sd00bHm00<002iM;Ud00biM00<0039n
<Wh01C9n00<003JN=Yh013JN00<003ZN>Yh01CZN00<003jn?[h01Sjn00<003jn?[h01Sjn00<004:n
@[h01d:n00<004:n@[h01d:n00<004;N@]h01d;N00<004KNA]h00dKN2P001DKN00<004KNA]h02DKN
00<004KNA]h01TKN00<004KN000067oo00<007ooOol07Goo000YOol20004Ool300039Sd00`00;Ud^
G@03;Ud00`00<WhbOP05<Wh00`00=YhfWP04=Yh00`00>YhjWP05>Yh00`00?[hn_P06?[h00`00?[hn
_P06?[h00`00@[i2_P07@[h00`00@[i2_P07@[h00`00@]i2gP03@]h90004A]h30009A]h00`00A]i6
gP09A]h00`00A]i6gP06A]h00`00A]h0000GOol00`00Oomoo`0MOol002Uoo`03001oogoo00Aoo`03
000V?@0000@000D^G@03000bOS9n00DbOP03000fWSJN00@fWP03000jWSZN00DjWP03000n_Sjn00Hn
_P03000n_Sjn00Hn_P030012_T:n00M2_P030012_T:n00=2_PT000A2gP030016gTKN00=6gP<00096
gP030016gTKN00Q6gP030016gTKN00Q6gP030016gTKN00M6gP<001Moo`03001oogoo01aoo`004Goo
0`001Woo0P0017oo100017oo0P001goo00<002Hm9Sd00RHm1P0000<^G@00<Wh01S9n00<003JN=Yh0
0cJN00<003ZN>Yh01SZN00<003jn?[h01Sjn00<003jn?[h01Sjn00<004:n@[h00d:n2@0014:n00<0
04;N@]h01d;N00<004KN00000P001TKN00<004KNA]h024KN00<004KNA]h024KN00@004KNA]i6gPD0
0003A]h0000001Moo`8001eoo`0047oo00D007ooOomoo`0000Eoo`8000Qoo`05001oogooOol00009
Ool00`009SdV?@029Sd00`00;Wd^O@02;Wd?00000cJN000jWP07>Yh00`00?[hn_P02?[hB0004@[h0
0`00@[i2_P07@[h00`00@]i2gP05@]h3000:A]h00`00A]i6gP08A]h00`00A]i6gP03A]h80006A]h0
0`00A]h0000GOol00`00Oomoo`05Ool50004Ool20005Ool40002Ool000Qoo`H0009oo`05001oogoo
Ool0000?Ool01000Oomoogoo0P002Goo00<002Hm9Sd00RHm00<002im;Wd00bim00<0039n<Wh01C9n
00<003JN=Yh00SJN3`0013jn00<003jn?[h01Sjn00<004:n@[h01d:n00<004:n@[h01d:n00<004;N
@]h00T;N0P000d;N00<004KNA]h01dKN00<004KNA]h01dKN1`001DKN00<004KNA]h01dKN0P0067oo
00<007ooOol01Woo00<007ooOol017oo0P002Goo0@000Goo000@Ool01@00Oomoogoo00003goo00D0
07ooOomoo`0000]oo`03000V?BHm008V?@03000^OBim00<^O@03000bOS9n00DbOP03000fWSJN00@f
WP03000jWSZN00DjWP03000n_Sjn00Hn_P03000n_Sjn00Hn_P030012_T:n00M2_P030012_T:n00M2
_P@000E2gP030016gTKN00Q6gP040016gTKNA]h50002A]h00`00A]i6gP08A]h00`00A]i6gP07A]h2
000GOol20007Ool00`00Oomoo`0?Ool10001Ool0011oo`05001oogooOol0000;Ool40004Ool2000;
Ool300029Sd00`00;Wd^O@03;Wd00`00<WhbOP05<Wh00`00=YhfWP04=Yh00`00>YhjWP05>Yh00`00
?[hn_P06?[h00`00?[hn_P06?[h00`00@[i2_P07@[h00`00@[i2_P04@[h30009@]h00`00A]i6gP03
A]h80008A]h00`00A]i6gP08A]h00`00A]i6gP06A]h00`00A]h0000GOol00`00Oomoo`05Ool00`00
Oomoo`0?Ool10001Ool0011oo`05001oogooOol0000;Ool00`00Oomoo`05Ool00`00Oomoo`0:Ool0
0`009UdVG@040005;Wd00`00<WhbOP05<Wh00`00=YhfWP04=Yh00`00>YhjWP05>Yh00`00?[hn_P06
?[h00`00?[hn_P06?[h00`00@[i2_P06@[h01000@[i2_T:n0`000d:n00<004;N@]h00d;N2@001DKN
00<004KNA]h024KN00<004KNA]h024KN00<004KNA]h01dKN0P0067oo00<007ooOol017oo00<007oo
Ool02goo10000Woo000@Ool01@00Oomoogoo00002goo00<007ooOol01Goo00<007ooOol02goo00<0
02IM9Ud00RIM1P0000<^O@00<Wh01S9n00<003JN=Yh013JN00<003ZN>Yh01CZN00<003jn?[h01Cjn
00<003jn?[h01cjn00<004:n@[h01T:n0`000d:n2@0014;N00<004KNA]h024KN00<004KNA]h024KN
00<004KNA]h024KN00<004KNA]h00dKN1P005goo00<007ooOol017oo00<007ooOol02goo00<007oo
Ool00goo000AOol3000<Ool40003Ool2000>Ool00`009UdVG@039Ud01000;Wd^OBim3`000SJN00<0
03ZN>Yh01CZN00<003jn?[h01Cjn00<003jn?[h00cjn4P0014:n00<004;N@]h01d;N00<004KNA]h0
24KN00<004KNA]h024KN00<004KNA]h01TKN1`001DKN00<007ooOol05Goo0P001Goo00<007ooOol0
2goo00<007ooOol00goo0008Ool30006Ool20003Ool50004Ool40004Ool00`00Oomoo`0>Ool00`00
9UdVG@039Ud00`00;Wd^O@03;Wd00`00<WhbOP04<Wh00`00=YhfWP02=YhH0004?[h00`00@[h00002
0004@[h00`00@[i2_P07@[h00`00@]i2gP07@]h00`00A]i6gP08A]h00`00A]i6gP07A]h01000A]i6
gTKN1@000TKN00<004KNA]h01dKN0P0067oo00<007ooOol00`003Goo10000Woo0007Ool01@00Oomo
ogoo00001Goo0P0017oo00@007ooOol000Qoo`04001oogooOol2000@Ool00`009UdVG@039Ud00`00
;Wd^O@03;Wd00`00<WhbOP04<Wh00`00=YhfWP04=Yh00`00>YhjWP05>Yh00`00?[hn_P06?[h00`00
?[hn_P05?[h20008@[h00`00@[i2_P07@[h00`00@]i2gP07@]h00`00A]i6gP08A]h00`00A]i6gP03
A]h70008A]h00`00A]i6gP07A]h2000GOol00`00Oomoo`0FOol000D0009oo`05001oogooOol0000<
Ool00`00Oomoo`08Ool01@00Oomoogoo00004Woo0`000bIM00<002im;Wd00bim00<0039n<Wh0139n
00<003JN=Yh013JN00<003ZN>Yh01CZN00<003jn?[h01Sjn00@003jn?[hn_P<0008n_P030012_T:n
00I2_P030012_T:n00M2_P030012gT;N00M2gP030016gTKN00=6gPX000A6gP030016gTKN00U6gP03
0016gTKN00M6gP8001Moo`03001oogoo01Eoo`001goo00D007ooOomoo`0000eoo`03001oogoo00Mo
o`05001oogooOol0000COol00`009Ud000040005;Wd00`00<WhbOP04<Wh00`00=YhfWP04=Yh00`00
>YhjWP05>Yh00`00?[hn_P06?[h30006?[h00`00@[i2_P06@[h00`00@[i2_P07@[h00`00@]i2gP03
@]h90005A]h00`00A]i6gP07A]h00`00A]i6gP09A]h00`00A]i6gP07A]h2000GOol00`00Oomoo`0D
Ool000Moo`05001oogooOol0000>Ool00`00Oomoo`02Ool40004Ool00`00Oomoo`0BOol00`009UdV
G@029Ud600000bim000bOP05<Wh00`00=YhfWP04=Yh00`00>YhjWP05>Yh00`00?[hn_P02?[h30000
0cjn000n_P07?[h00`00@[i2_P06@[h00`00@[i2_P03@[h90004@]h00`00A]i6gP08A]h00`00A]i6
gP07A]h00`00A]i6gP08A]h00`00A]i6gP07A]h3000FOol00`00Oomoo`0DOol000Moo`05001oogoo
Ool0000>Ool00`00Oomoo`02Ool00`00Oomoo`04Ool2000EOol00`009UdVG@039Ud01000;Wh^ORin
3P000SJN00<003ZN>Yh01CZN10001Cjn00<003jn?[h00Sjn4P0014;N00<004;N@]h01d;N00<004KN
A]h024KN00<004KNA]h01dKN00<004KNA]h024KN00@004KNA]i6gPD00003A]h0000001Moo`03001o
ogoo01=oo`001goo00D007ooOomoo`0000Yoo`05001oogooOol00004Ool00`00Oomoo`04Ool2000F
Ool00`009UdVG@039Ud00`00;Wh^OP03;Wh00`00=WhfOP04=Wh01000>YhjWSZN600013jn00<004:n
@[h01d:n00<004;N@]h01T;N00<004;N@]h01d;N00<004KNA]h01dKN00<004KNA]h024KN00<004KN
A]h00dKN20001dKN0P005Woo0P0057oo0008Ool3000<Ool30005Ool40003Ool00`00Oomoo`0FOol0
0`009UdVG@039Ud00`00;Wh^OP03;Wh00`00=WhfOP04=Wh00`00>YhjWP04>Yh01000>Yh000001CZN
00<003jn?[h01Cjn00<003jn?[h01Cjn00<004:n@[h01d:n00<004;N@]h01T;N00<004;N@]h01d;N
00<004KNA]h01dKN00<004KNA]h00dKN2P001DKN00<004KNA]h01dKN00<004KN00005Woo00<007oo
Ool04Woo000VOol2000HOol500000bIM000^OP04;Wh00`00=WhfOP04=Wh00`00>YhjWP02>Yh30008
>Yh00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P07@[h00`00@]i2gP06@]h00`00@]i2gP07
@]h00`00A]i6gP03A]h90005A]h00`00A]i6gP08A]h00`00A]i6gP07A]h2000GOol00`00Oomoo`0A
Ool001ioo`<000Eoo`03001oogoo01Qoo`04000ZGBYM:Ud700000bin000fOP05=Wh40003>Yh00`00
>YhjWP06>Yh00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P07@[h00`00@]i2gP06@]h00`00
@]i2gP03@]h90004A]h00`00A]i6gP08A]h00`00A]i6gP08A]h00`00A]i6gP07A]h2000FOol2000B
Ool001eoo`05001oogooOol00004Ool00`00Oomoo`0IOol00`00:UdZG@02:Ud00`00;Wh^OP02;Wh:
0006>Yh00`00>YhjWP06>Yh00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P03@[hB0004@]h0
0`00A]i6gP07A]h00`00A]i6gP08A]h00`00A]i6gP08A]h00`00A]i6gP03A]h6000FOol00`00Oomo
o`04Ool50007Ool001eoo`05001oogooOol00004Ool00`00Oomoo`0JOol00`00:UdZG@02:Ud00`00
;Wh^OP03;Wh400000cJN0000000V0004@[h00`00@]i2gP07@]h00`00@]i2gP06@]h00`00A]i6gP07
A]h00`00A]i6gP07A]h00`00A]i6gP07A]h70004A]h2000GOol00`00Oomoo`04Ool01000Oomoo`00
1goo000MOol01@00Oomoogoo00000goo0P007Goo00<002YM:Ud00RYM00<002in;Wh010001cJN00<0
03ZN>Yh013ZN00<003Zn>[h01CZn00<003jn?[h01Cjn00<003jn?[h01Cjn00<004:n@[h01d:n00<0
04;N@]h01d;N00<004;N@]h01T;N00<004KNA]h01dKN00<004KNA]h01dKN00@004KNA]i6gPD00096
gP030016gTKN00M6gP8001Ioo`8000Ioo`03001oogoo00Moo`007Goo00D007ooOomoo`0000=oo`03
001oogoo01eoo`<00004:Ud000000004;Wh00`00=YhfWP05=Yh00`00>YhjWP04>Yh00`00>[hj_P05
>[h00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P07@[h00`00@]i2gP07@]h00`00@]i2gP06
@]h00`00A]i6gP07A]h00`00A]i6gP03A]h70008A]h00`00A]i6gP07A]h2000FOol00`00Oomoo`05
Ool00`00Oomoo`06Ool001eoo`05001oogooOol00003Ool2000OOol60005;Wh00`00=YhfWP05=Yh0
0`00>YhjWP04>Yh00`00>[hj_P05>[h00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P07@[h0
0`00@]i2gP07@]h00`00@]i2gP06@]h00`00A]i6gP03A]h90005A]h00`00A]i6gP08A]h00`00A]i6
gP07A]h00`00Oomoo`0EOol00`00Oomoo`05Ool00`00Oomoo`05Ool001eoo`05001oogooOol00003
Ool00`00Oomoo`0LOol40003:Ud600000bin000fWP06=Yh00`00>YhjWP04>Yh00`00>[hj_P05>[h0
0`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P07@[h00`00@]i2gP03@]hB0004A]h00`00A]i6
gP08A]h00`00A]i6gP07A]h00`00A]i6gP07A]h2000EOol30006Ool00`00Oomoo`05Ool001ioo`<0
00Aoo`03001oogoo01Uoo`<000Aoo`03000ZGBYM008ZG@04000bOS9n<Wh?0002>Yh00`00>[hj_P05
>[h00`00?[hn_P05?[h00`00?[hn_P05?[h00`00@[i2_P03@[h90004@]h00`00A]i6gP06A]h00`00
A]i6gP07A]h00`00A]i6gP08A]h00`00A]i6gP07A]h00`00A]i6gP03A]h6000AOol300000goo0000
0002Ool01@00Oomoogoo00001goo000TOol3000GOol30008Ool00`00:UdZG@02:Ud00`00<WhbOP03
<Wh00`00=YhfWP04=Yh00`00>YhjWP02>YhP0004@[h00`00@]i2gP07@]h00`00A]i6gP06A]h00`00
A]i6gP07A]h00`00A]i6gP08A]h00`00A]i6gP03A]h90005A]h00`00Oomoo`0<Ool30005Ool01000
Oomoogoo0`0027oo0005Ool30006Ool20004Ool50003Ool40005Ool00`00Oomoo`0COol3000<Ool3
0002:Ud00`00<WhbOP03<Wh00`00=YhfWP04=Yh00`00>YhjWP04>Yh00`00?[hn_P05?[h00`00?[hn
_P06?[h00`00@[i2_P05@[h00`00@[i2_P06@[h00`00@]i2gP07@]h00`00A]i6gP06A]h00`00A]i6
gP07A]h00`00A]i6gP03A]h:0004A]h00`00A]i6gP07A]h2000:Ool30008Ool00`00Oomoo`0<Ool0
00Aoo`05001oogooOol00005Ool20005Ool01000Oomoo`001goo00@007ooOomoo`80019oo`<000mo
o`03000ZGBYM00@000DbOP03000fWSJN00@fWP03000jWSZN00@jWP03000n_Sjn00Dn_P03000n_Sjn
00Hn_P030012_T:n00E2_P030012_T:n00I2_P030012gT;N00I2gP030016gTKN00M6gP030016gTKN
00=6gPT000E6gP030016gTKN00M6gP030016gTKN00M6gP8000Eoo`@000aoo`03001oogoo00]oo`00
17oo00D007ooOomoo`0000eoo`03001oogoo00Moo`05001oogooOol0000AOol2000COol00`00:UdZ
G@02:Ud600000c9n000fWP05=Yh00`00>YhjWP04>Yh00`00?[hn_P05?[h00`00?[hn_P06?[h00`00
@[i2_P05@[h00`00@[i2_P05@[h00`00@]i2gP07@]h00`00A]i6gP03A]h90004A]h00`00A]i6gP08
A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00Oomoo`03000@Ool00`00Oomoo`0;Ool000Aoo`05
001oogooOol0000>Ool00`00Oomoo`06Ool01@00Oomoogoo00003Woo0`005Woo00<002YM:Ud00bYM
00@0039n<WhbOPL0008fWP03000jWSZN00@jWP03000n_Sjn00Dn_P03000n_Sjn00Hn_P030012_T:n
00E2_P030012_T:n00E2_P030012gT;N00=2gPT000A6gP030016gTKN00M6gP030016gTKN00Q6gP03
0016gTKN00I6gP030016gTKN00Q6gP8001=oo`03001oogoo00]oo`0017oo00D007ooOomoo`0000mo
o`04001oogooOol40004Ool00`00Oomoo`09Ool3000JOol00`00:UdZG@03:Ud00`00<WhbOP03<Wh0
1000=YhfWSJN<0001D;N00<004KNA]h01TKN00<004KNA]h01dKN00<004KNA]h01dKN00<004KNA]h0
1dKN00<004KNA]h00dKN1P004goo00<007ooOol02goo0004Ool01@00Oomoogoo00003goo00D007oo
Oomoo`0000Moo`03001oogoo00Ioo`<001ioo`03000ZGBYM00<ZG@03000bOS9n00<bOP03000fWSJN
00@fWP03000jWSZN00@jWP03000n_Sjn00@n_P03000n_Sjn00Hn_P030012_T:n00E2_P030012_T:n
00I2_P030012gT;N00M2gP030016gTKN00I6gP030016gTKN00M6gP030016gTKN00M6gP030016gTKN
00=6gPT000E6gP03001oogoo015oo`03001oogoo00]oo`0017oo00D007ooOomoo`0000]oo`05001o
ogooOol00003Ool00`00Oomoo`05Ool00`00Oomoo`03Ool3000ROol500000bYM000bOP04<Wh00`00
=YhfWP04=Yh00`00>YhjWP04>Yh00`00?[hn_P04?[h00`00?[hn_P06?[h00`00@[i2_P05@[h00`00
@[i2_P06@[h00`00@]i2gP07@]h00`00A]i6gP05A]h00`00A]i6gP08A]h00`00A]i6gP03A]h90004
A]h00`00A]i6gP08A]h00`00Oomoo`0@Ool00`00Oomoo`0;Ool000Eoo`<000eoo`<000Aoo`@000=o
o`80009oo`<002Ioo`04000ZGBYM:Ud700000c9n000fWP05=Yh00`00>YhjWP04>Yh00`00?[hn_P04
?[h00`00?[hn_P06?[h00`00@[i2_P05@[h00`00@[i2_P06@[h00`00@]i2gP07@]h00`00A]i6gP05
A]h00`00A]i6gP03A]h:0004A]h00`00A]i6gP07A]h00`00A]i6gP07A]h2000AOol00`00Oomoo`0;
Ool002=oo`@002Yoo`03000ZGBYM00<ZG@04000bOS9n<Wh70002=Yh00`00>YhjWP04>Yh00`00?[hn
_P04?[h00`00?[hn_P06?[h00`00@[i2_P05@[h00`00@[i2_P06@[h00`00@]i2gP03@]hA0005A]h0
0`00A]i6gP07A]h00`00A]i6gP07A]h00`00A]i6gP07A]h2000AOol00`00Oomoo`0:Ool002=oo`80
02eoo`03000ZGBYM00<ZG@03000bOS9n00<bOP04000fWSJN=Yh?0002?[h00`00?[hn_P02?[hJ0004
@]h00`00A]i6gP06A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00A]i6gP03
A]h6000@Ool00`00Oomoo`0:Ool002Eoo`03001oogoo02]oo`03000ZGBYM00<ZG@03000bOS9n00<b
OP03000fWSJN00<fWP03000jWSZN00DjWP04000n_Sjn?[h70004?[h00`00@[i2_P06@[h00`00@[i2
_P05@[h00`00@]i2gP07@]h00`00A]i6gP06A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00A]i6
gP03A]h90005A]h00`00Oomoo`0>Ool00`00Oomoo`0:Ool002Ioo`03001oogoo02]oo`D00003:Ud0
039n00@bOP03000fWSJN00<fWP03000jWSZN00DjWP03000n_Sjn00@n_P03000n_Sjn00Dn_P030012
_T:n00I2_P030012_T:n00E2_P030012gT;N00M2gP030016gTKN00I6gP030016gTKN00M6gP030016
gTKN00=6gPT000A6gP030016gTKN00M6gP8000moo`03001oogoo00Yoo`009goo00<007ooOol0:goo
00@002YM:UdZG@L00003<Wh003JN00@fWP03000jWSZN00DjWP03000n_Sjn00@n_P03000n_Sjn00Dn
_P030012_T:n00I2_P030012_T:n00E2_P030012gT;N00M2gP030016gTKN00I6gP030016gTKN00=6
gPT000A6gP030016gTKN00M6gP030016gTKN00M6gP8000ioo`03001oogoo00Yoo`00:7oo0P00;7oo
00<002YM:Ud00RYM00<0039n<Wh00S9n1`0000<fWP00>Yh01SZN00<003jn?[h013jn00<003jn?[h0
1Cjn00<004:n@[h01T:n00<004:n@[h01D:n00<004;N@]h00d;N4P0014KN00<004KNA]h01dKN00<0
04KNA]h01dKN00<004KNA]h01dKN0P003Goo00<007ooOol02Woo000ZOol00`00Oomoo`0ZOol00`00
:UdZG@02:Ud00`00<WhbOP03<Wh00`00=YhfWP02=Yh?0002?[h00`00?[hn_P05?[h00`00@[i2_P02
@[hA0004@]h00`00A]i6gP06A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00
A]i6gP03A]h6000<Ool00`00Oomoo`0:Ool002]oo`03001oogoo02Yoo`<0008ZG@03000bOS9n00<b
OP03000fWSJN00@fWP03000jWSZN00@jWP03000n_Sjn008n_Pl000A2_P030012gT;N00I2gP030012
gT;N00I2gP030016gTKN00I6gP030016gTKN00M6gP030016gTKN00M6gP030016gTKN00=6gPT000A6
gP8000eoo`03001oogoo00Uoo`00;7oo00<007ooOol0:Woo00<002iM00000`001C9n00<003JN=Yh0
13JN00<003ZN>Yh013ZN00<003jn?[h01Cjn00<003jn?[h01Cjn00<004:n@[h01D:n00<004;N@]h0
1T;N00<004;N@]h01T;N00<004KNA]h01TKN00<004KNA]h01dKN00<004KNA]h00dKN2@0014KN00<0
04KNA]h01dKN0P0037oo00<007ooOol02Goo000]Ool2000[Ool01000;Ud^GBiM1P0000<bOP00=Yh0
1CJN00<003ZN>Yh013ZN00<003jn?[h01Cjn00<003jn?[h01Cjn00<004:n@[h01D:n00<004;N@]h0
1T;N00<004;N@]h01D;N00<004KNA]h01dKN00<004KNA]h00dKN2@0014KN00<004KNA]h01dKN00<0
04KNA]h01dKN0P002goo00<007ooOol02Goo000_Ool00`00Oomoo`0YOol00`00;Ud^G@02;Ud01000
<WhbOS9n1`000SJN00<003ZN>Yh013ZN00<003jn?[h01Cjn00<003jn?[h01Cjn00<004:n@[h01D:n
00<004;N@]h01T;N00<004;N@]h01D;N00<004KNA]h00dKN2@0014KN00<004KNA]h01dKN00<004KN
A]h01dKN00<004KNA]h01TKN0P002goo00<007ooOol02Goo000`Ool00`00Oomoo`0YOol00`00;Ud^
G@02;Ud00`00<WhbOP03<Wh01000=YhfWSJN>00014KN00<004KNA]h01dKN00<004KNA]h01dKN00<0
04KNA]h01dKN00<004KNA]h00dKN1@002Woo00<007ooOol02Goo000aOol00`00Oomoo`0YOol30002
;Ud00`00<WhbOP03<Wh00`00=YhfWP04=Yh00`00>YhjWP04>Yh00`00?[hn_P05?[h00`00?[hn_P04
?[h00`00@[i2_P06@[h00`00@]i2gP05@]h00`00A]i6gP06A]h00`00A]i6gP06A]h00`00A]i6gP07
A]h00`00A]i6gP07A]h00`00A]i6gP03A]h90004B]h00`00Oomoo`08Ool00`00Oomoo`09Ool0039o
o`8002Uoo`03000^ORin00@000DbOP03000fWSJN00@fWP03000jWSZN00@jWP03000n_Sjn00Dn_P03
000n_Sjn00@n_P030012_T:n00I2_P030012gT;N00E2gP030016gTKN00I6gP030016gTKN00I6gP03
0016gTKN00M6gP030016gTKN00=6gPT000A6gP03001:gT[N00M:gP03001oogoo00Moo`03001oogoo
00Uoo`00=7oo00<007ooOol09goo00<002in;Wh00Rin1P0000<bOP00=Yh01CJN00<003ZN>Yh013ZN
00<003jn?[h01Cjn00<003jn?[h013jn00<004:n@[h01T:n00<004;N@]h01D;N00<004KNA]h01TKN
00<004KNA]h01TKN00<004KNA]h00dKN2@0014KN00<004KNA]h01dKN00<004[NB]h01T[N0P002Goo
00<007ooOol027oo000eOol00`00Oomoo`0WOol00`00;Wh^OP03;Wh01000<WhbOS9n3P000SZN00<0
03jn?[h01Cjn00<003jn?[h013jn00<004:n@[h01T:n00<004;N@]h01D;N00<004KNA]h00TKN4P00
14KN00<004KNA]h01dKN00<004KNA]h01dKN00<004[NB]h01T[N0P0027oo00<007ooOol027oo000f
Ool00`00Oomoo`0WOol00`00;Wh^OP03;Wh00`00<WhbOP03<Wh00`00>YhjWP04>Yh01000>[hj_SZn
9`0014KN00<004KNA]h01TKN00<004KNA]h01dKN00<004KNA]h01dKN00<004KNA]h01TKN00<004[N
B]h00d[N1P001goo00<007ooOol027oo000gOol2000XOol500000bin000bOP04<Wh00`00>YhjWP04
>Yh00`00>[hj_P03>[h00`00?[hn_P05?[h00`00@[i2_P05@[h00`00@[i2_P05@[h00`00@]i2gP05
@]h00`00A]i6gP06A]h00`00A]i6gP06A]h00`00A]i6gP07A]h00`00A]i6gP06A]h00`00A]i6gP03
A]h90005B]h00`00Oomoo`05Ool00`00Oomoo`08Ool003Uoo`03001oogoo02Moo`03000^ORin00L0
0003<Wh003ZN00DjWP03000j_SZn00<j_P03000n_Sjn00Dn_P030012_T:n00E2_P030012_T:n00E2
_P030012gT;N00E2gP030016gTKN00I6gP030016gTKN00I6gP030016gTKN00I6gP030016gTKN00=6
gPT000A6gP03001:gT[N00M:gP8000Ioo`03001oogoo00Qoo`00>Woo00<007ooOol09goo00D002in
;Wh^OP0000@fWPL0008jWP03000j_SZn00<j_P03000n_Sjn00Dn_P030012_T:n00E2_P030012_T:n
00E2_P030012gT;N00E2gP030016gTKN00I6gP030016gTKN00I6gP030016gTKN0096gPT000A6gP03
0016gTKN00M6gP03001:gT[N00M:gP8000Eoo`03001oogoo00Qoo`00>goo00<007ooOol09goo00D0
02in;Wh^OP0000HfWP04000jWSZN>Yh800000cZn000n_P06?[h00`00@[i2_P05@[h00`00@[i2_P05
@[h00`00@]i2gP05@]h00`00A]i6gP02A]hA0004A]h00`00A]i6gP07A]h00`00A]i6gP07A]h00`00
B]i:gP07B]h00`00Oomoo`03Ool00`00Oomoo`08Ool003aoo`8002Qoo`<00003;Wh003JN00DfWP03
000jWSZN00<jWP03000j_SZn008j_RH000A6gP030016gTKN00I6gP030016gTKN00I6gP030016gTKN
00M6gP030016gTKN00M6gP03001:gT[N00=:gPD000Eoo`03001oogoo00Moo`00?Woo00<007ooOol0
9Goo00<002in;Wh00`001SJN00<003ZN>Yh00cZN00<003Zn>[h013Zn00<003jn?[h01Cjn00<004:n
@[h01D:n00<004:n@[h01D:n00<004;N@]h01D;N00<004KNA]h01DKN00<004KNA]h01TKN00<004KN
A]h01TKN00<004KNA]h01dKN00<004KNA]h00dKN2@0014[n00<007ooOol00goo00<007ooOol01goo
000oOol00`00Oomoo`0UOol00`00;Wh^OP02;Wh50002=Yh00`00>YhjWP03>Yh00`00>[hj_P04>[h0
0`00?[hn_P05?[h00`00@[i2_P05@[h00`00@[i2_P05@[h00`00@]i2gP05@]h00`00A]i6gP05A]h0
0`00A]i6gP06A]h00`00A]i6gP06A]h00`00A]i6gP03A]h90004A]h00`00B_i:oP07B_h00`00Oomo
o`02Ool00`00Oomoo`07Ool0041oo`03001oogoo02Eoo`03000^ORin00<^OP03000fWSJN00P00003
>Yh003Zn00Dj_P03000n_Sjn00Dn_P030012_T:n00E2_P030012_T:n00E2_P030012gT;N00E2gP03
0016gTKN00E6gP030016gTKN0096gQ8000A6gP030016gTKN00M6gP03001:oT[n00M:oP05001oogoo
Ool00009Ool0045oo`8002Ioo`03000^ORin00<^OP03000fWSJN00<fWP04000jWSZN>Yhe0004A]h0
0`00A]i6gP07A]h00`00A]i6gP06A]h00`00A]i6gP07A]h00`00B_i:oP06B_h01@00Oomoogoo0000
2Goo0013Ool00`00Oomoo`0TOol500000bin000fWP04=Yh00`00>YhjWP03>Yh00`00?[hn_P04?[h0
0`00?[hn_P04?[h00`00@[i2_P05@[h00`00@]i2gP05@]h00`00@]i2gP06@]h00`00A]i6gP05A]h0
0`00A]i6gP05A]h00`00A]i6gP07A]h00`00A]i6gP06A]h00`00A]i6gP06A]h00`00B_i:oP03B_h5
0002Ool00`00Oomoo`07Ool004Aoo`03001oogoo02Aoo`04000^ORin;Wh700000cJN000jWP04>Yh0
0`00?[hn_P04?[h00`00?[hn_P04?[h00`00@[i2_P05@[h00`00@]i2gP05@]h00`00@]i2gP06@]h0
0`00A]i6gP05A]h00`00A]i6gP05A]h00`00A]i6gP07A]h00`00A]i6gP02A]hB0005B_h00`00Ool0
0009Ool004Eoo`03001oogoo02Aoo`03000^ORin00<^OP04000fWSJN=Yh700000cZN000n_P05?[h0
0`00?[hn_P04?[h00`00@[i2_P05@[h00`00@]i2gP05@]h00`00@]i2gP06@]h00`00A]i6gP05A]h0
0`00A]i6gP05A]h00`00A]i6gP03A]h80004A]h00`00B]i:gP07B]h00`00B_i:oP06B_h200000goo
001oo`07Ool004Ioo`8002Eoo`03000^ORin00<^OP03000fWSJN00<fWP04000jWSZN>Yh>0002?[h0
0`00@[i2_P05@[h00`00@]i2gP04@]h00`00@]i2gP03@]hI0004A]h00`00A]i6gP06A]h00`00B]i:
gP07B]h00`00B_i:oP06B_h30008Ool004Qoo`03001oogoo02=oo`D00003;Wh003JN00@fWP03000j
WSZN00<jWP03000n_Sjn00<n_P03000n_Sjn008n_QL000A2gP030016gTKN00E6gP030016gTKN00I6
gP030016gTKN00I6gP030016gTKN00I6gP03001:gT[N00M:gP03001:oT[n00=:oPD000Qoo`00BGoo
00<007ooOol08Woo00<007ooOol00Woo1`0000<fWP00>Yh013ZN00<003jn?[h00cjn00<003jn?[h0
1Cjn00<004:n@[h014:n00<004;N@]h01D;N00<004;N@]h01T;N00<004KNA]h01DKN00<004KNA]h0
1TKN00<004KNA]h01TKN00<004KNA]h01TKN00<004[NB]h00d[N2@000Woo0`0027oo001:Ool2000R
Ool00`00Oomoo`09Ool700000cZN000n_P04?[h00`00?[hn_P05?[h00`00@[i2_P04@[h00`00@]i2
gP05@]h00`00@]i2gP06@]h00`00A]i6gP05A]h00`00A]i6gP06A]h00`00A]i6gP02A]hB0007Ool4
000;Ool004aoo`03001oogoo01moo`03001oogoo011oo`L00003?[h003jn00Hn_P030012_T:n00A2
_P030012gT;N00E2gP030012gT;N00E2gP030016gTKN0096gQ4001Eoo`@000moo`00CGoo00<007oo
Ool07Woo00<007ooOol05goo9@008Woo10004goo001>Ool00`00Oomoo`0MOol00`00Oomoo`1KOol3
000GOol004moo`8001eoo`03001oogoo05Moo`@001Yoo`00DGoo00<007ooOol06Woo00<007ooOol0
Dgoo10007Woo001BOol00`00Oomoo`0IOol00`00Oomoo`1?Ool4000ROol005=oo`03001oogoo01Qo
o`03001oogoo04]oo`@002Ioo`00E7oo0P0067oo00<007ooOol0Agoo1000:Woo001FOol00`00Oomo
o`0DOol00`00Oomoo`14Ool4000^Ool005Moo`03001oogoo01=oo`03001oogoo045oo`<0039oo`00
F7oo00<007ooOol04Woo00<007ooOol0?Goo1000=Goo001IOol2000BOol00`00Oomoo`0iOol4000i
Ool005]oo`03001oogoo00moo`03001oogoo03Eoo`@003eoo`00G7oo00<007ooOol03Woo00<007oo
Ool0<Goo1000@Goo001MOol00`00Oomoo`0=Ool00`00Oomoo`0]Ool40015Ool005ioo`8000eoo`03
001oogoo02Uoo`@004Uoo`00H7oo00<007ooOol02Woo00<007ooOol09Woo0`00CGoo001QOol00`00
Oomoo`09Ool00`00Oomoo`0ROol4001@Ool0069oo`03001oogoo00Qoo`03001oogoo01ioo`@005Ao
o`00Hgoo0P0027oo00<007ooOol06Woo1000F7oo001UOol00`00Oomoo`05Ool00`00Oomoo`0FOol4
001LOol006Ioo`03001oogoo00Aoo`03001oogoo019oo`@0061oo`00Igoo00<007ooOol00goo00<0
07ooOol03Woo1000I7oo001XOol20003Ool00`00Oomoo`0;Ool3001XOol006Yoo`04001oogoo0009
Ool4001[Ool006]oo`03001oo`0000Eoo`@006moo`00K7oo0P0000=oo`0000000P00Lgoo001]Ool2
001gOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo
003VOol00>Ioo`00\
\>"],
  ImageRangeCache->{{{0, 229.313}, {185.563, 0}} -> {-0.181737, -0.0469919, 
  0.00640924, 0.00640924}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.3 The Greeks for a Log Contract", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's compute some symbolic forms for the value and Greeks of a log \
contract. It is critical that you appreciate what is happening here - ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is doing all the differentiation required to evaluate the Greeks. (If you \
run this notebook in electronic form you should note that the style of the \
output has been converted to that of a numbered equation.)"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
V = LogContract[S,K,r,q,\[Sigma],t];
\[CapitalDelta] = LogContractDelta[S_, K_, r_, q_, \[Sigma]_, t_] = 
D[V, S]; TraditionalForm[\[CapitalDelta]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\(-r\)\ t\)\/S\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalGamma] = LogContractGamma[S_, K_, r_, q_, \[Sigma]_, t_] = \

D[V, {S,2}]; TraditionalForm[\[CapitalGamma]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(-\(\[ExponentialE]\^\(\(-r\)\ t\)\/S\^2\)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Theta] = LogContractTheta[S_, K_, r_, q_, \[Sigma]_, t_] = 
-Simplify[D[V, t]]; TraditionalForm[\[Theta]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`1\/2\ \[ExponentialE]\^\(\(-r\)\ t\)\ 
      \((2\ r\ \(log(S\/K)\) - 
          \((r\ t - 1)\)\ \((\[Sigma]\^2 + 2\ q - 2\ r)\))\)\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Rho] = LogContractRho[S_, K_, r_, q_, \[Sigma]_, t_] = 
Simplify[D[V, r]]; TraditionalForm[\[Rho]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\(1\/2\)\)\ \[ExponentialE]\^\(\(-r\)\ t\)\ t\ 
      \((\(-t\)\ \[Sigma]\^2 - 2\ q\ t + 2\ r\ t + 2\ \(log(S\/K)\) - 2)
        \)\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalLambda] = LogContractVega[S_, K_, r_, q_, \[Sigma]_, t_] = \

Simplify[D[V, \[Sigma]]]; TraditionalForm[\[CapitalLambda]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\(-\[ExponentialE]\^\(\(-r\)\ t\)\)\ t\ \[Sigma]\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Check of Differential Constraints", "Subsubsection",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To make sure nothing has gone awry, we confirm the differential \
constraints. First the Black-Scholes equation, expressed in terms of \
Greeks:\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[
      \[Theta]\  + \ S \((r - q)\) \[CapitalDelta]\  + \ 
        1/2\ \[Sigma]^2\ S^2\ \[CapitalGamma]\  - \ r\ V]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0\)], "Output",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["\<\
Next we check the Vega-Gamma and Rho-Delta constraints. We do them \
simultaneously:\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[{\[CapitalLambda]\  - \ S^2\ \[Sigma]\ t\ \[CapitalGamma], 
        \[Rho]\  + \ t\ \((V\  - \ S\ \[CapitalDelta])\)}\ ]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 0}\)], "Output",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Observations about \[CapitalDelta] for a Log Contract\
\>", 
  "Subsection",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's review the form of the \[CapitalDelta] parameter. Remember \
that this is the amount of underlying needed to construct a risk-neutral \
portfolio. It is both a sensitivity and a hedge parameter. It is given \
by\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TraditionalForm[\[CapitalDelta]]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[ExponentialE]\^\(\(-r\)\ t\)\/S\)], "Output",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "The first important observation about this quantity is that it is \
unbounded above. Hypothetically, we can require an arbitrarily large \
\[CapitalDelta] if ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  " is arbitrarily small. There is no condition that "
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[LeftBracketingBar]\[CapitalDelta]\[RightBracketingBar] < 1\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "This is one good reason why one should ",
  StyleBox["never",
    FontSlant->"Italic"],
  " regard \[CapitalDelta] as a kind of probability! Second, the hedge \
parameter depends only on the asset price and the time. Indeed, if we work \
instead in terms of forward prices:"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`S\_F = S\ \ \[ExponentialE]\^\(\((r\  - q)\) t\)\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["The contract form simplifies to", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V\_F = \ 
      \(V\ \[ExponentialE]\^\(r\ t\)\  = \ \ 
        log(S\_F) - \(\[Sigma]\^2\/2\) t\)\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["and in these terms the Delta is time-independent.", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    FormBox[
      RowBox[{\(\[CapitalDelta]\_F\), "=", 
        RowBox[{
          FractionBox[\(\[PartialD]V\_F\), \(\[PartialD]S\_F\),
            MultilineFunction->None], "=", \(1\/S\_F\)}]}], 
      TraditionalForm]], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
However we view the contract, it is clear that the hedge strategy \
is stable - there is essentially no material time-dependence in this hedge \
position. 

The next observation about \[CapitalDelta] is that it is independent of the \
volatility, \[Sigma]. One of the easiest ways to mis-price a contract is to \
get the wrong volatility. Indeed, the question \"What is the right \
volatility?\" is not simply answered. However, a special property of a log \
contract is that the hedge position is independent of the volatility. So even \
if we use the \"wrong\" volatility, we shall not be mis-hedged.

Neither of these properties holds for vanilla European contracts. The hedge \
parameter \[CapitalDelta] is then strongly dependent on both time and \
volatility.\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.4 Implied Volatility for a Log Contract", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The \"implied volatility\" is that volatility which is consistent with the \
market price and the model of the price, other variables assumed to be known. \
It is instructive to ask ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to solve this - the results at first seem rather peculiar. We express the \
answer in percentage terms:"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
ImpliedVolLogContract[MktPrice_, S_, K_, r_, q_, t_] := \
100*\[Sigma] /.Solve[(Log[S/K] + (r - q - \[Sigma]^2/2)*t) Exp[-r \
t]==MktPrice, \[Sigma]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Here is a symbolic solution - there are two solutions as the equation \
involves ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\^2\)]],
  ":"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImpliedVolLogContract[P, \ S, \ K, \ r, \ q, \ t]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(-\(\(100\ \@2\ \@\(\(-E\^\(r\ t\)\)\ P - q\ t + r\ t + Log[S\/K]\)\)\
\/\@t\)\), \(100\ \@2\ \@\(\(-E\^\(r\ t\)\)\ P - q\ t + r\ t + \
Log[S\/K]\)\)\/\@t}\)], "Output"]
}, Open  ]],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has, somewhat irritatingly, put the answer in complex form! This is more \
than a  ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " foible. Here is a sample numerical value, based on ",
  Cell[BoxData[
      \(TraditionalForm\`S\  = \ \(K\  = \ 1\), \ r\  = \ 0.1, \ 
      q\  = \ 0\)]],
  " and one year to maturity.",
  StyleBox[" If the price is 2 currency units there is no real solution:",
    FontSlant->"Italic"]
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ImpliedVolLogContract[2, 1, 1, 0.1, 0, 1]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({\(0.`\[InvisibleSpace]\) - 
        205.443025491316966`\ I, \(0.`\[InvisibleSpace]\) + 
        205.443025491316966`\ I}\)], "Output"]
}, Open  ]],

Cell["\<\
Let's define a function which selects out solutions that are real \
and positive, and get some sample results:\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[
      ImpliedVolLogContract[0.5, 1, 1, 0.1, 0, 
        1], \((Im[\ #] == 0 && # > 0 &)\)]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[
      ImpliedVolLogContract[0.09048, 1, 1, 0.1, 0, 
        1], \((Im[\ #] == 0 && # > 0 &)\)]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.287587639351923041`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[
      ImpliedVolLogContract[0.0, 1, 1, 0.1, 0, 
        1], \((Im[\ #] == 0 && # > 0 &)\)]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({44.7213595499957961`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[
      ImpliedVolLogContract[\(-0.5\), 1, 1, 0.1, 0, 
        1], \((Im[\ #] == 0 && # > 0 &)\)]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({114.244077223970252`}\)], "Output"]
}, Open  ]],

Cell["\<\
We get either no solution or a rapidly varying function of the \
market price. It is a straightforward matter to work out for what market \
prices there is a real solution. The maximum allowed market price is given by\
\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(pcrit[S_, \ K_, \ r_, \ q_, 
        t_]\  := \ \ p /. \(Solve[
            q - r\  + Exp[r\ t]\ p/t\  - \ Log[S/K]/t == 0, \ 
            p]\)[\([1]\)]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["In our example, we have", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(pc\  = \ pcrit[1, 1, 0.1, 0, 1]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.090483741803595965`\)], "Output"]
}, Open  ]],

Cell["\<\
Let's plot the implied volatility as a function of the market price \
of the log contract:\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[\(Select[
          ImpliedVolLogContract[p, 1, 1, 0.1, 0, 
            1], \n\((Im[\ #] == 0 && # > 0 &)\)]\)[\([1]\)], \ {p, \ \(-3\), 
        pc}]; \)\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.948307 0.308166 0.0147151 0.00225206 [
[.02381 .00222 -6 -9 ]
[.02381 .00222 6 0 ]
[.17789 .00222 -12 -9 ]
[.17789 .00222 12 0 ]
[.33198 .00222 -6 -9 ]
[.33198 .00222 6 0 ]
[.48606 .00222 -12 -9 ]
[.48606 .00222 12 0 ]
[.64014 .00222 -6 -9 ]
[.64014 .00222 6 0 ]
[.79422 .00222 -12 -9 ]
[.79422 .00222 12 0 ]
[.93581 .12732 -12 -4.5 ]
[.93581 .12732 0 4.5 ]
[.93581 .23992 -18 -4.5 ]
[.93581 .23992 0 4.5 ]
[.93581 .35252 -18 -4.5 ]
[.93581 .35252 0 4.5 ]
[.93581 .46513 -18 -4.5 ]
[.93581 .46513 0 4.5 ]
[.93581 .57773 -18 -4.5 ]
[.93581 .57773 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.02381 .01472 m
.02381 .02097 L
s
[(-3)] .02381 .00222 0 1 Mshowa
.17789 .01472 m
.17789 .02097 L
s
[(-2.5)] .17789 .00222 0 1 Mshowa
.33198 .01472 m
.33198 .02097 L
s
[(-2)] .33198 .00222 0 1 Mshowa
.48606 .01472 m
.48606 .02097 L
s
[(-1.5)] .48606 .00222 0 1 Mshowa
.64014 .01472 m
.64014 .02097 L
s
[(-1)] .64014 .00222 0 1 Mshowa
.79422 .01472 m
.79422 .02097 L
s
[(-0.5)] .79422 .00222 0 1 Mshowa
.125 Mabswid
.05463 .01472 m
.05463 .01847 L
s
.08544 .01472 m
.08544 .01847 L
s
.11626 .01472 m
.11626 .01847 L
s
.14708 .01472 m
.14708 .01847 L
s
.20871 .01472 m
.20871 .01847 L
s
.23953 .01472 m
.23953 .01847 L
s
.27034 .01472 m
.27034 .01847 L
s
.30116 .01472 m
.30116 .01847 L
s
.36279 .01472 m
.36279 .01847 L
s
.39361 .01472 m
.39361 .01847 L
s
.42442 .01472 m
.42442 .01847 L
s
.45524 .01472 m
.45524 .01847 L
s
.51687 .01472 m
.51687 .01847 L
s
.54769 .01472 m
.54769 .01847 L
s
.57851 .01472 m
.57851 .01847 L
s
.60932 .01472 m
.60932 .01847 L
s
.67096 .01472 m
.67096 .01847 L
s
.70177 .01472 m
.70177 .01847 L
s
.73259 .01472 m
.73259 .01847 L
s
.76341 .01472 m
.76341 .01847 L
s
.82504 .01472 m
.82504 .01847 L
s
.85586 .01472 m
.85586 .01847 L
s
.88667 .01472 m
.88667 .01847 L
s
.91749 .01472 m
.91749 .01847 L
s
.97912 .01472 m
.97912 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.94831 .12732 m
.95456 .12732 L
s
[(50)] .93581 .12732 1 0 Mshowa
.94831 .23992 m
.95456 .23992 L
s
[(100)] .93581 .23992 1 0 Mshowa
.94831 .35252 m
.95456 .35252 L
s
[(150)] .93581 .35252 1 0 Mshowa
.94831 .46513 m
.95456 .46513 L
s
[(200)] .93581 .46513 1 0 Mshowa
.94831 .57773 m
.95456 .57773 L
s
[(250)] .93581 .57773 1 0 Mshowa
.125 Mabswid
.94831 .03724 m
.95206 .03724 L
s
.94831 .05976 m
.95206 .05976 L
s
.94831 .08228 m
.95206 .08228 L
s
.94831 .1048 m
.95206 .1048 L
s
.94831 .14984 m
.95206 .14984 L
s
.94831 .17236 m
.95206 .17236 L
s
.94831 .19488 m
.95206 .19488 L
s
.94831 .2174 m
.95206 .2174 L
s
.94831 .26244 m
.95206 .26244 L
s
.94831 .28496 m
.95206 .28496 L
s
.94831 .30748 m
.95206 .30748 L
s
.94831 .33 m
.95206 .33 L
s
.94831 .37504 m
.95206 .37504 L
s
.94831 .39757 m
.95206 .39757 L
s
.94831 .42009 m
.95206 .42009 L
s
.94831 .44261 m
.95206 .44261 L
s
.94831 .48765 m
.95206 .48765 L
s
.94831 .51017 m
.95206 .51017 L
s
.94831 .53269 m
.95206 .53269 L
s
.94831 .55521 m
.95206 .55521 L
s
.94831 .60025 m
.95206 .60025 L
s
.25 Mabswid
.94831 0 m
.94831 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .59126 L
.10458 .57781 L
.14415 .56488 L
.18221 .55215 L
.22272 .53826 L
.26171 .52453 L
.30316 .50952 L
.34309 .49462 L
.3815 .47983 L
.42237 .46357 L
.46172 .44733 L
.49955 .43112 L
.53984 .41313 L
.57861 .39502 L
.61984 .37476 L
.65954 .35411 L
.69774 .33298 L
.73838 .30884 L
.77751 .28355 L
.81909 .25377 L
.85916 .22105 L
.89771 .18368 L
.91726 .16113 L
.93871 .13147 L
.94807 .11586 L
.9581 .09585 L
.96296 .0841 L
.96756 .07074 L
.97169 .05516 L
.97283 .0497 L
.97403 .04275 L
.97506 .03502 L
.97619 .01484 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  ImageSize->{181.562, 112.188},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003R0000S1000`40O003h00OhWoo003ROol00>9o
o`004Goo0`0057oo1@001Goo0P0017oo100057oo1@0057oo1@0017oo0P0017oo10005Goo1@004goo
0`001Woo0P0017oo10009Woo000@Ool01@00Oomoogoo000057oo00@007ooOol000Eoo`8000Qoo`03
001oogoo019oo`04001oogoo000FOol00`00Oomoo`04Ool20008Ool00`00Oomoo`0DOol00`00Oomo
o`0BOol01@00Oomoogoo00001Goo0P0027oo00<007ooOol08goo0008Ool60006Ool00`00Oomoo`09
Ool60004Ool00`00Oomoo`0?Ool00`00Oomoo`09Ool60004Ool00`00Oomoo`0;Ool60005Ool00`00
Oomoo`0>Ool00`00Oomoo`09Ool60005Ool00`00Oomoo`0:Ool60002Ool01@00Oomoogoo00003goo
00<007ooOol08goo000DOol00`00Oomoo`0DOol00`00Oomoo`0>Ool00`00Oomoo`0DOol00`00Oomo
o`0EOol00`00Oomoo`0>Ool00`00Oomoo`0DOol00`00Oomoo`0BOol01@00Oomoogoo00003goo00<0
07ooOol08goo000BOol2000HOol00`00Oomoo`09Ool4000HOol00`00Oomoo`0DOol00`00Oomoo`0:
Ool4000GOol00`00Oomoo`0BOol01@00Oomoogoo00002goo10009Woo000DOol00`00Oomoo`0EOol0
0`00Oomoo`09Ool00`00Oomoo`0IOol00`00Oomoo`0DOol00`00Oomoo`0:Ool00`00Oomoo`0HOol0
0`00Oomoo`0BOol01@00Oomoogoo00002goo00<007ooOol09goo000@Ool01@00Oomoogoo00004goo
00D007ooOomoo`0000]oo`03001oogoo01Eoo`05001oogooOol0000FOol00`00Oomoo`0:Ool00`00
Oomoo`0HOol00`00Oomoo`0BOol01@00Oomoogoo00002goo00<007ooOol09goo000AOol3000EOol3
000<Ool4000EOol3000EOol3000<Ool4000EOol3000EOol3000<Ool4000VOol00<moo`03001oogoo
011oo`00cgoo00<007ooOol047oo003?Ool00`00Oomoo`0@Ool000Yoom4000Moo`003goo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol017oo00<007ooOol00goo00<007ooOol00goo00<007ooOol017oo00<007oo
Ool00goo00<007ooOol02Woo003?Ool00`00Oomoo`03Ool00`00Oomoo`0:Ool00<moo`03001oogoo
009oo`03001oogoo00]oo`00cgoo00<007ooOol00Woo00<007ooOol02goo003?Ool20003Ool00`00
Oomoo`0;Ool00<moo`03001oogoo009oo`03001oogoo00]oo`00cgoo00<007ooOol00Woo00<007oo
Ool02goo003?Ool00`00Oomoo`02Ool00`00Oomoo`0;Ool00<moo`03001oogoo009oo`03001oogoo
00]oo`00cgoo0P000goo00<007ooOol02goo003?Ool01@00Oomoogoo00003Woo003?Ool01@00Oomo
ogoo00003Woo003?Ool01@00Oomoogoo00003Woo003?Ool200000goo001oo`0>Ool00<moo`04001o
ogoo000?Ool00<moo`04001oogoo000?Ool00<moo`03001oo`00011oo`00cgoo00<007oo000047oo
003?Ool2000AOol00<moo`80015oo`00_goo100017oo0`001Goo00<007ooOol047oo0033Ool01000
Oomoo`000goo00<007ooOol00Woo00<007ooOol047oo0033Ool01000Oomoo`000goo00@007ooOomo
o`80019oo`00`goo00@007ooOol000=oo`05001oogoo001oo`02000AOol00;moo`@000=oo`05001o
ogooOol00002Ool00`00Ool0000BOol00;moo`03001oogoo00Aoo`07001oogooOol007oo00000Woo
00<007ooOol047oo002oOol00`00Oomoo`04Ool01000Oomoogoo0P000goo00<007ooOol047oo002o
Ool40004Ool40004Ool2000AOol00<Yoo`03001oogoo009oo`03001oogoo011oo`00bGoo00<007oo
Ool00goo00<007ooOol047oo0038Ool00`00Oomoo`04Ool00`00Oomoo`0@Ool00<Moo`03001oogoo
00Eoo`03001oogoo011oo`00aWoo00<007ooOol01Woo0P004Goo0035Ool00`00Oomoo`07Ool00`00
Oomoo`0@Ool00<Aoo`03001oogoo00Qoo`03001oogoo011oo`00`goo00<007ooOol02Goo00<007oo
Ool047oo0032Ool00`00Oomoo`0:Ool00`00Oomoo`0@Ool00<5oo`03001oogoo00]oo`80015oo`00
`7oo00<007ooOol037oo00<007ooOol047oo002oOol00`00Oomoo`0=Ool00`00Oomoo`0@Ool00;io
o`03001oogoo00ioo`03001oogoo011oo`00_Goo00<007ooOol03goo0P004Goo002kOol2000BOol0
0`00Oomoo`0@Ool00;Moo`D000=oo`<000Eoo`<000Eoo`03001oogoo011oo`00^Goo00<007ooOol0
0Woo00D007ooOomoo`0000=oo`05001oogooOol00004Ool00`00Oomoo`0@Ool00;Qoo`8000Aoo`05
001oogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol047oo002gOol00`00Ool00004Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`80015oo`00]Woo00@007ooOol000Aoo`05001o
ogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol047oo002eOol01@00Oomoogoo000017oo
00D007ooOomoo`0000=oo`05001oogooOol00004Ool00`00Oomoo`0@Ool00;=oo`8000Aoo`03001o
ogoo009oo`05001oogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol047oo002bOol00`00
Oomoo`02Ool30005Ool30005Ool30005Ool00`00Oomoo`0@Ool00;1oo`8001eoo`80015oo`00[goo
00<007ooOol07Goo00<007ooOol047oo002]Ool2000POol00`00Oomoo`0@Ool00:aoo`03001oogoo
021oo`03001oogoo011oo`00ZWoo0P008goo0P004Goo002YOol00`00Oomoo`0SOol00`00Oomoo`0@
Ool00:Moo`8002Ioo`03001oogoo011oo`00YGoo0P00:7oo00<007ooOol047oo002TOol00`00Oomo
o`0XOol00`00Oomoo`0@Ool00:9oo`8002]oo`80015oo`00XGoo00<007ooOol0:goo00<007ooOol0
47oo002OOol2000^Ool00`00Oomoo`0@Ool009eoo`80031oo`03001oogoo011oo`00W7oo00<007oo
Ool0<7oo00<007ooOol047oo002JOol2000cOol2000AOol009Uoo`03001oogoo03=oo`03001oogoo
011oo`00Ugoo0P007Woo1@000goo100017oo0`001Goo00<007ooOol047oo002EOol2000ROol00`00
Oomoo`07Ool01000Oomoo`000goo00<007ooOol00Woo00<007ooOol047oo002DOol00`00Oomoo`0R
Ool00`00Oomoo`07Ool01000Oomoo`000goo00<007ooOol00Woo0P004Goo002BOol2000UOol00`00
Oomoo`07Ool01000Oomoo`000goo00<007ooOol00Woo00<007ooOol047oo002@Ool2000WOol00`00
Oomoo`03Ool40003Ool01@00Oomoogoo000017oo00<007ooOol047oo002>Ool2000YOol00`00Oomo
o`03Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo00<007ooOol047oo002<Ool2000[Ool00`00
Oomoo`03Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo00<007ooOol047oo002:Ool2000[Ool3
0005Ool40004Ool30005Ool2000AOol008Qoo`8004Eoo`03001oogoo011oo`00QGoo0`00Agoo00<0
07ooOol047oo0023Ool2001:Ool00`00Oomoo`0@Ool0085oo`8004aoo`03001oogoo011oo`00Ogoo
0P00CWoo0P004Goo001mOol2001@Ool00`00Oomoo`0@Ool007]oo`80059oo`03001oogoo011oo`00
NGoo0P00E7oo00<007ooOol047oo001gOol2001FOol2000AOol007Eoo`8005Qoo`03001oogoo011o
o`00Lgoo0P00FWoo00<007ooOol047oo001aOol2001LOol00`00Oomoo`0@Ool006ioo`<005ioo`03
001oogoo011oo`00Jgoo0`00HGoo0P004Goo001YOol2001TOol00`00Oomoo`0@Ool006Ioo`<004eo
o`D000Aoo`<000Eoo`<000Eoo`03001oogoo011oo`00Hgoo0`00DGoo00@007ooOol000=oo`05001o
ogooOol00003Ool01@00Oomoogoo000017oo00<007ooOol047oo001QOol2001EOol00`00Oomoo`03
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`03001oogoo011oo`00Ggoo0P00F7oo00<0
07ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00004Ool2000AOol005aoo`<005]oo`05
001oogooOol00003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol047oo001JOol2
001NOol01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`03001oogoo009oo`03001oogoo011o
o`00F7oo0P00G7oo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo000017oo
00<007ooOol047oo001EOol3001OOol30005Ool30005Ool30005Ool2000AOol0059oo`<007Yoo`03
001oogoo011oo`00D7oo0P00OGoo00<007ooOol047oo001=Ool3001oOol00`00Oomoo`0@Ool004Yo
o`<0089oo`03001oogoo011oo`00B7oo0P00QGoo0P004Goo0015Ool30027Ool00`00Oomoo`0@Ool0
049oo`<008Yoo`03001oogoo011oo`00?goo0`00SGoo00<007ooOol047oo000lOol3002@Ool00`00
Oomoo`0@Ool003Uoo`<009=oo`80015oo`00=goo0P00UWoo00<007ooOol047oo000dOol3002HOol0
0`00Oomoo`0@Ool0035oo`<009]oo`03001oogoo011oo`00;Woo0`00WWoo0P004Goo000ZOol4002Q
Ool00`00Oomoo`0@Ool002Moo`<00:Eoo`03001oogoo011oo`0097oo0`00Sgoo1@0017oo100017oo
0`001Goo00<007ooOol047oo000QOol3002COol01000Oomoo`0027oo00@007ooOol000=oo`03001o
ogoo009oo`03001oogoo011oo`007goo0P00Ugoo00<007ooOol027oo00@007ooOol000=oo`03001o
ogoo009oo`80015oo`0077oo0`00VWoo00<007ooOol01goo00@007ooOol000=oo`03001oogoo009o
o`03001oogoo011oo`006Goo0`00WWoo00<007ooOol00Woo10000goo00D007ooOomoo`0000Aoo`03
001oogoo011oo`005Goo1000XGoo00<007ooOol00Woo00<007ooOol017oo00D007ooOomoo`0000Ao
o`03001oogoo011oo`004Goo1000XGoo00D007ooOomoo`0000Aoo`03001oogoo00Aoo`05001oogoo
Ool00004Ool00`00Oomoo`0@Ool000moo`800:Ioo`<000Eoo`@000Aoo`<000Eoo`80015oo`00cgoo
00<007ooOol047oo003?Ool00`00Oomoo`0@Ool00<moo`03001oogoo011oo`00\
\>"],
  ImageRangeCache->{{{0, 225.938}, {139.188, 0}} -> {-3.23922, -29.5947, 
  0.0195244, 2.67167}}]
}, Open  ]],

Cell[TextData[{
  "So the implied volatility fails to exist if the market price is greater \
than ",
  StyleBox["pcrit",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". For market prices slightly less than ",
  StyleBox["pcrit",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", the implied volatility is a highly unstable function of the market \
price. Let's make a table of values for implied volatility with the price \
varying from the critical value down to the critical value minus",
  Cell[BoxData[
      \(TraditionalForm\`\(\ 0.001\)\)]],
  ":"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{pc - 
          k/1000, \ \(Select[
            ImpliedVolLogContract[pc - k/1000, 1, 1, 0.1, 0, 
              1], \n\t\t\((Im[\ #] == 0 && # > 0 &)\)]\)[\([1]\)]}, \ {k, 1, 
        10}]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.0894837418035959686`, 
        4.70142726855504733`}, {0.0884837418035959544`, 
        6.6488222057012436`}, {0.0874837418035959579`, 
        8.14311089722711045`}, {0.0864837418035959615`, 
        9.40285453711009466`}, {0.085483741803595965`, 
        10.5127109637602456`}, {0.0844837418035959508`, 
        11.5160978707667216`}, {0.0834837418035959544`, 
        12.4388073596543335`}, {0.0824837418035959757`, 
        13.2976444114024761`}, {0.0814837418035959615`, 
        14.1042818056651313`}, {0.080483741803595965`, 
        14.8672184222580616`}}\)], "Output"]
}, Open  ]],

Cell["\<\
So even our simplest possible log contract exhibits some of the \
implied volatility pathology referred to in the introduction. However, the \
log contract is special in that it is one of the very few solutions to the \
Black-Scholes equation where there is a simple explicit formula for the \
implied volatility. \
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.5 The Power Contract", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["The value of a general power contract is given by", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`V = Z\ \((S\/K)\)\^\[Lambda]\ 
        \[ExponentialE]\^\(\((
              \((\(-\(\[Sigma]\^2\/2\)\) - q + r)\)\ \[Lambda] + 
                \(\[Lambda]\^2\ \[Sigma]\^2\)\/2 - r)\)\ \((T - t)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The expiry value of this option, when ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ T\)]],
  ", is given by"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`V = Z\ \((S\/K)\)\^\[Lambda]\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In what follows we set ",
  Cell[BoxData[
      \(TraditionalForm\`Z\  = \ 1\)]],
  ", as before. First we need the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " definition:"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
PowerContract[S_, K_, r_, q_, \[Sigma]_, t_, \[Lambda]_] := 
(S/K)^\[Lambda]*Exp[((-(\[Sigma]^2/2) - q + r)*\[Lambda] + (\[Lambda]^2*\
\[Sigma]^2)/2 - r)*t]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Recall that there are some notable special cases, given by \
\[Lambda] = 0,1. The first is just a cash payment at maturity.\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerContract[S, K, r, q, \[Sigma], t, 0]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(E\^\(\(-r\)\ t\)\)], "Output"]
}, Open  ]],

Cell["\<\
When \[Lambda] = 1 we just get the yield-adjusted asset price:\
\>",
   "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PowerContract[S, K, r, q, \[Sigma], t, 1]\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(E\^\(\(-q\)\ t\)\ S\)\/K\)], "Output"]
}, Open  ]],

Cell["This is time-independent when the yield is zero:", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      PowerContract[S, 10, 0.1, 0, 0.2, \ t, 1], \ {S, \ 5, \ 15}, \ {t, \ 
        0, \ 2}, \ AxesLabel\  -> \ {"\<S\>", \ "\<t\>", \ "\<\>"}]; \)\)], 
  "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.08608 .22315 -6 -8.87306 ]
[.08608 .22315 0 .12694 ]
[.20731 .17545 -5.83689 -9 ]
[.20731 .17545 .16311 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.46808 .07289 -10.673 -9 ]
[.46808 .07289 1.32698 0 ]
[.60856 .01769 -10.1726 -9 ]
[.60856 .01769 1.82736 0 ]
[.29165 .07573 -9.31117 -12 ]
[.29165 .07573 .68883 0 ]
[.70096 -0.00478 0 -6.26206 ]
[.70096 -0.00478 6 2.73794 ]
[.78313 .12104 0 -6.13858 ]
[.78313 .12104 18 2.86142 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9224 .33431 0 -5.93715 ]
[.9224 .33431 18 3.06285 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.91861 .21225 0 -8.04169 ]
[.91861 .21225 10 3.95831 ]
[.02411 .26511 -18 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31795 -24 -2.78699 ]
[.01568 .31795 0 6.21301 ]
[.00696 .37264 -6 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00208 .42927 -24 -2.88161 ]
[-0.00208 .42927 0 6.11839 ]
[-0.01144 .48795 -18 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 16.000 moveto
 541.000 16.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 16.000 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(S) show
69.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.68874 0 m
.68263 .00239 L
s
[(0)] .70096 -0.00478 -1 .39157 Mshowa
.7708 .12553 m
.76464 .12778 L
s
[(0.5)] .78313 .12104 -1 .36413 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(1)] .8565 .23339 -1 .34028 Mshowa
.9099 .3383 m
.90365 .34029 L
s
[(1.5)] .9224 .33431 -1 .31937 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(2)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70593 .0263 m
.70226 .02771 L
s
.72272 .05198 m
.71904 .05338 L
s
.73912 .07706 m
.73543 .07845 L
s
.75514 .10158 m
.75145 .10294 L
s
.78611 .14895 m
.78241 .15028 L
s
.80108 .17185 m
.79737 .17317 L
s
.81573 .19425 m
.81201 .19555 L
s
.83006 .21617 m
.82633 .21745 L
s
.8578 .2586 m
.85407 .25986 L
s
.87123 .27915 m
.8675 .28039 L
s
.88439 .29928 m
.88065 .3005 L
s
.89727 .31899 m
.89353 .3202 L
s
.92227 .35722 m
.91851 .3584 L
s
.93439 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .3951 L
s
.95792 .41176 m
.95416 .41291 L
s
gsave
.91861 .21225 -61 -12.0417 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 16.000 moveto
 541.000 16.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 16.000 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(t) show
69.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0.5)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31328 m
.03408 .31095 L
s
[(0.75)] .01568 .31795 1 -0.38067 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(1)] .00696 .37264 1 -0.37036 Mshowa
.01027 .42483 m
.01644 .42261 L
s
[(1.25)] -0.00208 .42927 1 -0.35964 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(1.5)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33497 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .35559 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40054 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43642 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .4717 m
.00656 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.462 .671 .954 r
.38313 .58323 .40512 .60371 .44155 .60642 .41978 .58574 Metetra
.462 .671 .954 r
.41978 .58574 .44155 .60642 .47879 .60918 .45726 .58831 Metetra
.462 .671 .954 r
.45726 .58831 .47879 .60918 .51685 .612 .49558 .59094 Metetra
.462 .671 .954 r
.49558 .59094 .51685 .612 .55577 .61489 .53477 .59363 Metetra
.462 .671 .954 r
.53477 .59363 .55577 .61489 .59557 .61785 .57486 .59637 Metetra
.462 .671 .954 r
.57486 .59637 .59557 .61785 .63629 .62087 .6159 .59919 Metetra
.462 .671 .954 r
.6159 .59919 .63629 .62087 .67796 .62396 .6579 .60207 Metetra
.462 .671 .954 r
.6579 .60207 .67796 .62396 .72061 .62713 .7009 .60501 Metetra
.462 .671 .954 r
.7009 .60501 .72061 .62713 .76428 .63037 .74494 .60803 Metetra
.462 .671 .954 r
.74494 .60803 .76428 .63037 .809 .63369 .79007 .61113 Metetra
.462 .671 .954 r
.79007 .61113 .809 .63369 .85481 .63709 .8363 .61429 Metetra
.462 .671 .954 r
.8363 .61429 .85481 .63709 .90176 .64057 .8837 .61754 Metetra
.462 .671 .954 r
.8837 .61754 .90176 .64057 .94988 .64414 .93231 .62088 Metetra
.462 .671 .954 r
.93231 .62088 .94988 .64414 .99922 .6478 .98216 .62429 Metetra
.462 .671 .954 r
.36056 .56222 .38313 .58323 .41978 .58574 .39745 .56453 Metetra
.462 .671 .954 r
.39745 .56453 .41978 .58574 .45726 .58831 .43516 .5669 Metetra
.462 .671 .954 r
.43516 .5669 .45726 .58831 .49558 .59094 .47374 .56931 Metetra
.462 .671 .954 r
.47374 .56931 .49558 .59094 .53477 .59363 .5132 .57179 Metetra
.462 .671 .954 r
.5132 .57179 .53477 .59363 .57486 .59637 .55359 .57432 Metetra
.462 .671 .954 r
.55359 .57432 .57486 .59637 .6159 .59919 .59494 .57691 Metetra
.462 .671 .954 r
.59494 .57691 .6159 .59919 .6579 .60207 .63727 .57956 Metetra
.462 .671 .954 r
.63727 .57956 .6579 .60207 .7009 .60501 .68063 .58228 Metetra
.462 .671 .954 r
.68063 .58228 .7009 .60501 .74494 .60803 .72506 .58506 Metetra
.462 .671 .954 r
.72506 .58506 .74494 .60803 .79007 .61113 .77058 .58791 Metetra
.462 .671 .954 r
.77058 .58791 .79007 .61113 .8363 .61429 .81725 .59084 Metetra
.462 .671 .954 r
.81725 .59084 .8363 .61429 .8837 .61754 .86511 .59384 Metetra
.462 .671 .954 r
.86511 .59384 .8837 .61754 .93231 .62088 .9142 .59691 Metetra
.462 .671 .954 r
.9142 .59691 .93231 .62088 .98216 .62429 .96458 .60007 Metetra
.462 .671 .954 r
.33741 .54067 .36056 .56222 .39745 .56453 .37452 .54277 Metetra
.462 .671 .954 r
.37452 .54277 .39745 .56453 .43516 .5669 .41247 .54491 Metetra
.462 .671 .954 r
.41247 .54491 .43516 .5669 .47374 .56931 .45131 .54711 Metetra
.462 .671 .954 r
.45131 .54711 .47374 .56931 .5132 .57179 .49105 .54935 Metetra
.462 .671 .954 r
.49105 .54935 .5132 .57179 .55359 .57432 .53173 .55165 Metetra
.462 .671 .954 r
.53173 .55165 .55359 .57432 .59494 .57691 .57339 .55401 Metetra
.462 .671 .954 r
.57339 .55401 .59494 .57691 .63727 .57956 .61607 .55642 Metetra
.462 .671 .954 r
.61607 .55642 .63727 .57956 .68063 .58228 .65979 .55889 Metetra
.462 .671 .954 r
.65979 .55889 .68063 .58228 .72506 .58506 .70459 .56142 Metetra
.462 .671 .954 r
.70459 .56142 .72506 .58506 .77058 .58791 .75053 .56402 Metetra
.462 .671 .954 r
.75053 .56402 .77058 .58791 .81725 .59084 .79764 .56668 Metetra
.462 .671 .954 r
.79764 .56668 .81725 .59084 .86511 .59384 .84596 .56941 Metetra
.462 .671 .954 r
.84596 .56941 .86511 .59384 .9142 .59691 .89555 .57222 Metetra
.462 .671 .954 r
.89555 .57222 .9142 .59691 .96458 .60007 .94646 .5751 Metetra
.462 .671 .954 r
.31365 .51855 .33741 .54067 .37452 .54277 .35098 .52042 Metetra
.462 .671 .954 r
.35098 .52042 .37452 .54277 .41247 .54491 .38918 .52233 Metetra
.462 .671 .954 r
.38918 .52233 .41247 .54491 .45131 .54711 .42827 .52429 Metetra
.462 .671 .954 r
.42827 .52429 .45131 .54711 .49105 .54935 .46828 .5263 Metetra
.462 .671 .954 r
.46828 .5263 .49105 .54935 .53173 .55165 .50926 .52835 Metetra
.462 .671 .954 r
.50926 .52835 .53173 .55165 .57339 .55401 .55124 .53046 Metetra
.462 .671 .954 r
.55124 .53046 .57339 .55401 .61607 .55642 .59425 .53262 Metetra
.462 .671 .954 r
.59425 .53262 .61607 .55642 .65979 .55889 .63833 .53483 Metetra
.462 .671 .954 r
.63833 .53483 .65979 .55889 .70459 .56142 .68353 .53709 Metetra
.462 .671 .954 r
.68353 .53709 .70459 .56142 .75053 .56402 .72988 .53942 Metetra
.462 .671 .954 r
.72988 .53942 .75053 .56402 .79764 .56668 .77743 .5418 Metetra
.462 .671 .954 r
.77743 .5418 .79764 .56668 .84596 .56941 .82623 .54425 Metetra
.462 .671 .954 r
.82623 .54425 .84596 .56941 .89555 .57222 .87632 .54676 Metetra
.462 .671 .954 r
.87632 .54676 .89555 .57222 .94646 .5751 .92776 .54934 Metetra
.462 .671 .954 r
.28926 .49583 .31365 .51855 .35098 .52042 .32681 .49746 Metetra
.462 .671 .954 r
.32681 .49746 .35098 .52042 .38918 .52233 .36524 .49914 Metetra
.462 .671 .954 r
.36524 .49914 .38918 .52233 .42827 .52429 .40459 .50085 Metetra
.462 .671 .954 r
.40459 .50085 .42827 .52429 .46828 .5263 .44488 .5026 Metetra
.462 .671 .954 r
.44488 .5026 .46828 .5263 .50926 .52835 .48616 .5044 Metetra
.462 .671 .954 r
.48616 .5044 .50926 .52835 .55124 .53046 .52845 .50624 Metetra
.462 .671 .954 r
.52845 .50624 .55124 .53046 .59425 .53262 .5718 .50812 Metetra
.462 .671 .954 r
.5718 .50812 .59425 .53262 .63833 .53483 .61625 .51005 Metetra
.462 .671 .954 r
.61625 .51005 .63833 .53483 .68353 .53709 .66183 .51204 Metetra
.462 .671 .954 r
.66183 .51204 .68353 .53709 .72988 .53942 .7086 .51407 Metetra
.462 .671 .954 r
.7086 .51407 .72988 .53942 .77743 .5418 .7566 .51616 Metetra
.462 .671 .954 r
.7566 .51616 .77743 .5418 .82623 .54425 .80588 .5183 Metetra
.462 .671 .954 r
.80588 .5183 .82623 .54425 .87632 .54676 .85648 .5205 Metetra
.462 .671 .954 r
.85648 .5205 .87632 .54676 .92776 .54934 .90848 .52276 Metetra
.462 .671 .954 r
.2642 .4725 .28926 .49583 .32681 .49746 .30197 .47388 Metetra
.462 .671 .954 r
.30197 .47388 .32681 .49746 .36524 .49914 .34064 .4753 Metetra
.462 .671 .954 r
.34064 .4753 .36524 .49914 .40459 .50085 .38024 .47674 Metetra
.462 .671 .954 r
.38024 .47674 .40459 .50085 .44488 .5026 .42081 .47823 Metetra
.462 .671 .954 r
.42081 .47823 .44488 .5026 .48616 .5044 .46238 .47975 Metetra
.462 .671 .954 r
.46238 .47975 .48616 .5044 .52845 .50624 .505 .4813 Metetra
.462 .671 .954 r
.505 .4813 .52845 .50624 .5718 .50812 .54869 .4829 Metetra
.462 .671 .954 r
.54869 .4829 .5718 .50812 .61625 .51005 .5935 .48454 Metetra
.462 .671 .954 r
.5935 .48454 .61625 .51005 .66183 .51204 .63948 .48622 Metetra
.462 .671 .954 r
.63948 .48622 .66183 .51204 .7086 .51407 .68668 .48795 Metetra
.462 .671 .954 r
.68668 .48795 .7086 .51407 .7566 .51616 .73513 .48972 Metetra
.462 .671 .954 r
.73513 .48972 .7566 .51616 .80588 .5183 .78489 .49154 Metetra
.462 .671 .954 r
.78489 .49154 .80588 .5183 .85648 .5205 .83602 .49341 Metetra
.462 .671 .954 r
.83602 .49341 .85648 .5205 .90848 .52276 .88857 .49533 Metetra
.462 .671 .954 r
.23846 .44853 .2642 .4725 .30197 .47388 .27644 .44964 Metetra
.462 .671 .954 r
.27644 .44964 .30197 .47388 .34064 .4753 .31535 .45078 Metetra
.462 .671 .954 r
.31535 .45078 .34064 .4753 .38024 .47674 .35521 .45195 Metetra
.462 .671 .954 r
.35521 .45195 .38024 .47674 .42081 .47823 .39605 .45315 Metetra
.462 .671 .954 r
.39605 .45315 .42081 .47823 .46238 .47975 .43792 .45438 Metetra
.462 .671 .954 r
.43792 .45438 .46238 .47975 .505 .4813 .48085 .45564 Metetra
.462 .671 .954 r
.48085 .45564 .505 .4813 .54869 .4829 .52489 .45693 Metetra
.462 .671 .954 r
.52489 .45693 .54869 .4829 .5935 .48454 .57007 .45825 Metetra
.462 .671 .954 r
.57007 .45825 .5935 .48454 .63948 .48622 .61645 .45961 Metetra
.462 .671 .954 r
.61645 .45961 .63948 .48622 .68668 .48795 .66407 .46101 Metetra
.462 .671 .954 r
.66407 .46101 .68668 .48795 .73513 .48972 .71297 .46244 Metetra
.462 .671 .954 r
.71297 .46244 .73513 .48972 .78489 .49154 .76323 .46392 Metetra
.462 .671 .954 r
.76323 .46392 .78489 .49154 .83602 .49341 .81488 .46543 Metetra
.462 .671 .954 r
.81488 .46543 .83602 .49341 .88857 .49533 .868 .46699 Metetra
.462 .671 .954 r
.212 .42389 .23846 .44853 .27644 .44964 .2502 .42472 Metetra
.462 .671 .954 r
.2502 .42472 .27644 .44964 .31535 .45078 .28934 .42557 Metetra
.462 .671 .954 r
.28934 .42557 .31535 .45078 .35521 .45195 .32945 .42645 Metetra
.462 .671 .954 r
.32945 .42645 .35521 .45195 .39605 .45315 .37056 .42734 Metetra
.462 .671 .954 r
.37056 .42734 .39605 .45315 .43792 .45438 .41273 .42826 Metetra
.462 .671 .954 r
.41273 .42826 .43792 .45438 .48085 .45564 .45598 .4292 Metetra
.462 .671 .954 r
.45598 .4292 .48085 .45564 .52489 .45693 .50036 .43016 Metetra
.462 .671 .954 r
.50036 .43016 .52489 .45693 .57007 .45825 .54592 .43116 Metetra
.462 .671 .954 r
.54592 .43116 .57007 .45825 .61645 .45961 .59269 .43217 Metetra
.462 .671 .954 r
.59269 .43217 .61645 .45961 .66407 .46101 .64074 .43322 Metetra
.462 .671 .954 r
.64074 .43322 .66407 .46101 .71297 .46244 .69011 .43429 Metetra
.462 .671 .954 r
.69011 .43429 .71297 .46244 .76323 .46392 .74086 .4354 Metetra
.462 .671 .954 r
.74086 .4354 .76323 .46392 .81488 .46543 .79305 .43653 Metetra
.462 .671 .954 r
.79305 .43653 .81488 .46543 .868 .46699 .84674 .4377 Metetra
.462 .671 .954 r
.18479 .39856 .212 .42389 .2502 .42472 .2232 .39909 Metetra
.462 .671 .954 r
.2232 .39909 .2502 .42472 .28934 .42557 .26257 .39963 Metetra
.462 .671 .954 r
.26257 .39963 .28934 .42557 .32945 .42645 .30293 .40019 Metetra
.462 .671 .954 r
.30293 .40019 .32945 .42645 .37056 .42734 .34432 .40077 Metetra
.462 .671 .954 r
.34432 .40077 .37056 .42734 .41273 .42826 .38678 .40135 Metetra
.462 .671 .954 r
.38678 .40135 .41273 .42826 .45598 .4292 .43035 .40196 Metetra
.462 .671 .954 r
.43035 .40196 .45598 .4292 .50036 .43016 .47508 .40258 Metetra
.462 .671 .954 r
.47508 .40258 .50036 .43016 .54592 .43116 .521 .40321 Metetra
.462 .671 .954 r
.521 .40321 .54592 .43116 .59269 .43217 .56818 .40386 Metetra
.462 .671 .954 r
.56818 .40386 .59269 .43217 .64074 .43322 .61667 .40454 Metetra
.462 .671 .954 r
.61667 .40454 .64074 .43322 .69011 .43429 .66651 .40523 Metetra
.462 .671 .954 r
.66651 .40523 .69011 .43429 .74086 .4354 .71776 .40593 Metetra
.462 .671 .954 r
.71776 .40593 .74086 .4354 .79305 .43653 .77049 .40666 Metetra
.462 .671 .954 r
.77049 .40666 .79305 .43653 .84674 .4377 .82477 .40742 Metetra
.462 .671 .954 r
.1568 .3725 .18479 .39856 .2232 .39909 .19542 .37271 Metetra
.462 .671 .954 r
.19542 .37271 .2232 .39909 .26257 .39963 .23502 .37293 Metetra
.462 .671 .954 r
.23502 .37293 .26257 .39963 .30293 .40019 .27563 .37316 Metetra
.462 .671 .954 r
.27563 .37316 .30293 .40019 .34432 .40077 .31729 .37339 Metetra
.462 .671 .954 r
.31729 .37339 .34432 .40077 .38678 .40135 .36004 .37363 Metetra
.462 .671 .954 r
.36004 .37363 .38678 .40135 .43035 .40196 .40393 .37387 Metetra
.462 .671 .954 r
.40393 .37387 .43035 .40196 .47508 .40258 .449 .37412 Metetra
.462 .671 .954 r
.449 .37412 .47508 .40258 .521 .40321 .4953 .37438 Metetra
.462 .671 .954 r
.4953 .37438 .521 .40321 .56818 .40386 .54288 .37464 Metetra
.462 .671 .954 r
.54288 .37464 .56818 .40386 .61667 .40454 .5918 .37491 Metetra
.462 .671 .954 r
.5918 .37491 .61667 .40454 .66651 .40523 .64212 .37519 Metetra
.462 .671 .954 r
.64212 .37519 .66651 .40523 .71776 .40593 .69388 .37548 Metetra
.462 .671 .954 r
.69388 .37548 .71776 .40593 .77049 .40666 .74716 .37578 Metetra
.462 .671 .954 r
.74716 .37578 .77049 .40666 .82477 .40742 .80203 .37608 Metetra
.462 .671 .954 r
.128 .34568 .1568 .3725 .19542 .37271 .16682 .34556 Metetra
.462 .671 .954 r
.16682 .34556 .19542 .37271 .23502 .37293 .20664 .34543 Metetra
.462 .671 .954 r
.20664 .34543 .23502 .37293 .27563 .37316 .2475 .34531 Metetra
.462 .671 .954 r
.2475 .34531 .27563 .37316 .31729 .37339 .28942 .34517 Metetra
.462 .671 .954 r
.28942 .34517 .31729 .37339 .36004 .37363 .33247 .34504 Metetra
.462 .671 .954 r
.33247 .34504 .36004 .37363 .40393 .37387 .37668 .3449 Metetra
.462 .671 .954 r
.37668 .3449 .40393 .37387 .449 .37412 .42209 .34476 Metetra
.462 .671 .954 r
.42209 .34476 .449 .37412 .4953 .37438 .46877 .34461 Metetra
.462 .671 .954 r
.46877 .34461 .4953 .37438 .54288 .37464 .51676 .34446 Metetra
.462 .671 .954 r
.51676 .34446 .54288 .37464 .5918 .37491 .56612 .34431 Metetra
.462 .671 .954 r
.56612 .34431 .5918 .37491 .64212 .37519 .6169 .34415 Metetra
.462 .671 .954 r
.6169 .34415 .64212 .37519 .69388 .37548 .66918 .34399 Metetra
.462 .671 .954 r
.66918 .34399 .69388 .37548 .74716 .37578 .72302 .34382 Metetra
.462 .671 .954 r
.72302 .34382 .74716 .37578 .80203 .37608 .77849 .34365 Metetra
.462 .671 .954 r
.09834 .31807 .128 .34568 .16682 .34556 .13737 .31759 Metetra
.462 .671 .954 r
.13737 .31759 .16682 .34556 .20664 .34543 .17741 .3171 Metetra
.462 .671 .954 r
.17741 .3171 .20664 .34543 .2475 .34531 .21851 .3166 Metetra
.462 .671 .954 r
.21851 .3166 .2475 .34531 .28942 .34517 .2607 .31608 Metetra
.462 .671 .954 r
.2607 .31608 .28942 .34517 .33247 .34504 .30403 .31555 Metetra
.462 .671 .954 r
.30403 .31555 .33247 .34504 .37668 .3449 .34855 .31501 Metetra
.462 .671 .954 r
.34855 .31501 .37668 .3449 .42209 .34476 .39431 .31445 Metetra
.462 .671 .954 r
.39431 .31445 .42209 .34476 .46877 .34461 .44136 .31387 Metetra
.462 .671 .954 r
.44136 .31387 .46877 .34461 .51676 .34446 .48976 .31328 Metetra
.462 .671 .954 r
.48976 .31328 .51676 .34446 .56612 .34431 .53956 .31267 Metetra
.462 .671 .954 r
.53956 .31267 .56612 .34431 .6169 .34415 .59083 .31205 Metetra
.462 .671 .954 r
.59083 .31205 .6169 .34415 .66918 .34399 .64363 .3114 Metetra
.462 .671 .954 r
.64363 .3114 .66918 .34399 .72302 .34382 .69803 .31073 Metetra
.462 .671 .954 r
.69803 .31073 .72302 .34382 .77849 .34365 .7541 .31005 Metetra
.462 .671 .954 r
.0678 .28963 .09834 .31807 .13737 .31759 .10702 .28878 Metetra
.462 .671 .954 r
.10702 .28878 .13737 .31759 .17741 .3171 .14728 .2879 Metetra
.462 .671 .954 r
.14728 .2879 .17741 .3171 .21851 .3166 .18861 .287 Metetra
.462 .671 .954 r
.18861 .287 .21851 .3166 .2607 .31608 .23106 .28607 Metetra
.462 .671 .954 r
.23106 .28607 .2607 .31608 .30403 .31555 .27468 .28512 Metetra
.462 .671 .954 r
.27468 .28512 .30403 .31555 .34855 .31501 .31952 .28415 Metetra
.462 .671 .954 r
.31952 .28415 .34855 .31501 .39431 .31445 .36562 .28314 Metetra
.462 .671 .954 r
.36562 .28314 .39431 .31445 .44136 .31387 .41305 .28211 Metetra
.462 .671 .954 r
.41305 .28211 .44136 .31387 .48976 .31328 .46185 .28104 Metetra
.462 .671 .954 r
.46185 .28104 .48976 .31328 .53956 .31267 .51209 .27995 Metetra
.462 .671 .954 r
.51209 .27995 .53956 .31267 .59083 .31205 .56384 .27882 Metetra
.462 .671 .954 r
.56384 .27882 .59083 .31205 .64363 .3114 .61717 .27766 Metetra
.462 .671 .954 r
.61717 .27766 .64363 .3114 .69803 .31073 .67214 .27646 Metetra
.462 .671 .954 r
.67214 .27646 .69803 .31073 .7541 .31005 .72883 .27522 Metetra
.462 .671 .954 r
.03634 .26033 .0678 .28963 .10702 .28878 .07574 .25908 Metetra
.462 .671 .954 r
.07574 .25908 .10702 .28878 .14728 .2879 .11621 .25779 Metetra
.462 .671 .954 r
.11621 .25779 .14728 .2879 .18861 .287 .15777 .25646 Metetra
.462 .671 .954 r
.15777 .25646 .18861 .287 .23106 .28607 .20048 .2551 Metetra
.462 .671 .954 r
.20048 .2551 .23106 .28607 .27468 .28512 .24438 .2537 Metetra
.462 .671 .954 r
.24438 .2537 .27468 .28512 .31952 .28415 .28953 .25227 Metetra
.462 .671 .954 r
.28953 .25227 .31952 .28415 .36562 .28314 .33597 .25079 Metetra
.462 .671 .954 r
.33597 .25079 .36562 .28314 .41305 .28211 .38377 .24926 Metetra
.462 .671 .954 r
.38377 .24926 .41305 .28211 .46185 .28104 .43298 .24769 Metetra
.462 .671 .954 r
.43298 .24769 .46185 .28104 .51209 .27995 .48367 .24608 Metetra
.462 .671 .954 r
.48367 .24608 .51209 .27995 .56384 .27882 .5359 .24441 Metetra
.462 .671 .954 r
.5359 .24441 .56384 .27882 .61717 .27766 .58975 .2427 Metetra
.462 .671 .954 r
.58975 .2427 .61717 .27766 .67214 .27646 .6453 .24093 Metetra
.462 .671 .954 r
.6453 .24093 .67214 .27646 .72883 .27522 .70262 .2391 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -16 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 20 translate 1 -1 scale
gsave
0.000 0.000 0.000 setrgbcolor
1.000 setlinewidth
gsave
newpath
 61.000 16.000 moveto
 541.000 16.000 lineto
 541.000 4.000 lineto
 61.000 4.000 lineto
 61.000 16.000 lineto
closepath
clip newpath
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
63.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(S) show
69.000 13.000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
0.000 0.000 rmoveto
1.000 setlinewidth
grestore
grestore
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: Courier
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  ImageSize->{184.25, 149.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003V0000^Q000`40O003h00OiWoo003VOol00>Io
o`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol008Mo
o`D000Aoo`<001=oo`<003eoo`00RGoo00<007ooOol01Goo00<007ooOol04Goo00D007ooOomoo`00
03aoo`00RGoo00<007ooOol00Woo1@004Woo00D007ooOomoo`0003aoo`00RGoo00<007ooOol00Woo
00@007ooOol001=oo`05001oogooOol0000lOol008Uoo`03001oogoo00=oo`03001oo`0001=oo`05
001oogooOol0000lOol008Uoo`03001oogoo00=oo`03001oo`0001=oo`05001oogooOol0000lOol0
08Uoo`03001oogoo00Aoo`8000ioo`<0009oo`05001oogooOol0000lOol008Moo`<000Moo`03001o
ogoo00Yoo`<00003Ool007oo009oo`<003eoo`00Vgoo0`0017oo0P00@Woo002IOol30006Ool00`00
Ool00011Ool004ioo`D001Uoo`D0009oo`D001ioo`<000Uoo`03001oo`00045oo`00CWoo00<007oo
Ool00Woo00<007ooOol067oo00<007ooOol00goo00@007ooOol001aoo`8000aoo`04001oo`000010
Ool004ioo`03001oogoo009oo`03001oogoo01Qoo`03001oogoo00Aoo`03001oogoo01Uoo`@000eo
o`05001oogooOol0000oOol0055oo`<001Yoo`03001oogoo00Eoo`03001oogoo01Eoo`<0015oo`05
001oogooOol0000oOol004ioo`<001eoo`03001oogoo00Ioo`03001oogoo019oo`8001Aoo`03001o
ogoo009oo`03001oogoo03aoo`00CWoo00<007ooOol00Woo00<007ooOol067oo00<007ooOol01Woo
00<007ooOol03goo0`005Woo00<007ooOol00Woo0P00?Goo001>Ool00`00Oomoo`02Ool00`00Oomo
o`0HOol00`00Oomoo`02Ool01@00Oomoogoo00003Woo0`006Goo00<007ooOol00Woo0P00?Goo001?
Ool5000HOol30005Ool3000=Ool4000JOol00`00Oomoo`04Ool00`00Oomoo`0jOol0085oo`<001mo
o`03001oogoo00=oo`03001oogoo03Yoo`00Ogoo0P008Woo00<007ooOol017oo00<007ooOol0>Goo
001lOol3000TOol00`00Oomoo`05Ool00`00Oomoo`0hOol0055oo`D000=oo`<001eoo`<002Moo`03
001oogoo00Eoo`03001oogoo03Qoo`00Dgoo00<007ooOol00Woo00D007ooOomoo`0001Yoo`@002Qo
o`03001oogoo00Ioo`03001oogoo03Moo`00Dgoo00<007ooOol00Woo00D007ooOomoo`0001Moo`<0
02aoo`03001oogoo00Moo`03001oogoo03Ioo`00Dgoo00<007ooOol00Woo00D007ooOomoo`0001Eo
o`8002moo`03001oogoo00Moo`03001oogoo00Ioo`<000Ioo`8000Eoo`@001aoo`00Dgoo00<007oo
Ool00Woo00D007ooOomoo`00019oo`<00003Ool007oo02ioo`03001oogoo00Qoo`03001oogoo00Ao
o`05001oogooOol00005Ool20009Ool00`00Oomoo`0IOol005=oo`03001oogoo009oo`05001oogoo
Ool0000?Ool3000dOol00`00Oomoo`09Ool00`00Oomoo`03Ool01@00Oomoogoo000047oo00<007oo
Ool06Goo001COol00`00Oomoo`02Ool01@00Oomoogoo00003Goo1000=Goo00<007ooOol02Goo00<0
07ooOol00goo00D007ooOomoo`00011oo`03001oogoo01Uoo`00DGoo0`001Goo0`002goo0`00>Goo
00<007ooOol02Woo00<007ooOol00Woo00D007ooOomoo`0000aoo`@001aoo`00IGoo0P00?7oo00<0
07ooOol02goo00D007ooOomoo`0000=oo`03001oogoo00Yoo`03001oogoo01eoo`00@7oo0`007goo
0`0000=oo`00Ool0>goo00<007ooOol02Woo0P000goo00D007ooOomoo`0000aoo`03001oogoo01eo
o`00?goo00D007ooOomoo`0001aoo`80045oo`03001oogoo00aoo`04001oogooOol3000=Ool4000L
Ool003moo`05001oogooOol0000IOol40012Ool00`00Oomoo`0<Ool00`00Oomoo`0aOol003moo`05
001oogooOol0000FOol30016Ool00`00Oomoo`0=Ool00`00Oomoo`0`Ool0041oo`<001Eoo`<004Qo
o`03001oogoo00ioo`03001oogoo02moo`00?goo00D007ooOomoo`00015oo`<004]oo`03001oogoo
00ioo`03001oogoo02moo`00?goo00D007ooOomoo`0000moo`8004ioo`03001oogoo00moo`03001o
ogoo02ioo`00@7oo0`003Goo0`0000=oo`00Ool0CGoo00<007ooOol047oo00<007ooOol0;Goo001=
Ool3001COol00`00Oomoo`0@Ool2000^Ool002Uoo`<001moo`@005Aoo`03001oogoo011oo`8002io
o`00:7oo00D007ooOomoo`0001]oo`<005Qoo`03001oogoo019oo`03001oogoo02]oo`00:7oo00D0
07ooOomoo`0001Uoo`8005aoo`03001oogoo015oo`03001oogoo01Qoo`<0011oo`00:7oo00D007oo
Oomoo`0001Ioo`<0009oo`03001oogoo05Uoo`03001oogoo019oo`03001oogoo01Ioo`05001oogoo
Ool0000?Ool002Qoo`@001Aoo`<0065oo`03001oogoo019oo`8001Moo`03001oogoo015oo`00:7oo
00<007ooOol04goo0P000Woo00<007ooOol0Ggoo00<007ooOol04goo00<007ooOol05Goo00<007oo
Ool04Goo000YOol00`00Oomoo`0?Ool3001VOol00`00Oomoo`0DOol00`00Oomoo`06Ool50009Ool0
0`00Oomoo`0AOol002Yoo`<000aoo`800003Ool007oo06Ioo`03001oogoo01Eoo`03001oogoo00Mo
o`03001oogoo00Moo`H0011oo`00=Woo0`00Jgoo00<007ooOol05Goo0P0027oo00<007ooOol02Goo
00<007ooOol04Goo000cOol4001]Ool00`00Oomoo`0FOol00`00Oomoo`06Ool00`00Oomoo`09Ool0
0`00Oomoo`0AOol0035oo`80075oo`03001oogoo01Ioo`03001oogoo00Ioo`03001oogoo01eoo`00
;Woo0`00Lgoo00<007ooOol05goo00<007ooOol01Goo00<007ooOol07Goo000/Ool20003Ool00`00
Oomoo`1[Ool6000IOol20006Ool00`00Oomoo`0MOol002Uoo`<005=oob0000@j_P8001Yoo`03001o
ogoo009oo`<001moo`009Woo0`000Woo00<007ooOol0;7oo9@001CZn00<003Zn>[h023Zn00<003Zn
>[h01cZn00<003Zn>[h013Zn0`0000<j_P0000006Woo00<007ooOol08goo000UOol2000IOolJ0004
>[h00`00>[hj_P07>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P08
>[h00`00>[hj_P07>[h00`00>[hj_P030005>[h2000JOol00`00Oomoo`0ROol002Eooa/000Dj_P03
000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn00Hj_P03
000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Pj_P03000j_SZn00Hj_P<000Tj_P8001Uoo`80
02=oo`003Goo0`001Woo0P001Goo100017oo0P001SZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<0
03Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<0
03Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h023Zn00<003Zn>[h00SZn0`00
0SZn00<003Zn>[h023Zn00<007ooOol067oo00<007ooOol08Goo000<Ool01@00Oomoogoo00001Goo
0P002Goo00L007ooOomoo`00Ool00006>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P05
>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P06
>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[h000020006>[h00`00>[hj
_P07>[h2000JOol00`00Oomoo`0POol000aoo`05001oogooOol0000@Ool01000Oomoo`000goo00<0
03Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<0
03Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<0
03Zn>[h01cZn00<003Zn>[h01CZn0`001CZn4@006Goo00<007ooOol087oo000<Ool01@00Oomoogoo
000047oo00<007ooOol00P000goo00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01SZn00<0
03Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h00cZn<000
1CZn00<003Zn>[h023Zn00<007ooOol067oo0P0087oo000<Ool01@00Oomoogoo000037oo10000goo
00<007ooOol00goo00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h00SZn:`0013Zn00<003Zn
>[h01cZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn00000P001CZn00<003Zn>[h023Zn00<0
03Zn>[h01cZn0P006Woo00<007ooOol07Woo000<Ool01@00Oomoogoo000037oo00<007ooOol017oo
0P001Goo4`001CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01CZn
00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h01SZn00<003Zn>[h01CZn0`002CZn00<003Zn
>[h023Zn00<003Zn>[h01cZn0P006Goo00<007ooOol07Woo000<Ool01@00Oomoogoo000037oo00<0
07ooOol017oo00<007ooOol01Woo00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<0
03Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01CZn00<003Zn>[h01cZn00<0
03Zn>[h01cZn00<003Zn>[h01SZn00@003Zn>[hj_P<000<j_P03000j_SZn00Lj_P03000j_SZn00Pj
_P03000j_SZn00Lj_P8001Uoo`03001oogoo01eoo`003Goo0`003Goo10000goo00<007ooOol01goo
00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn
00<003Zn>[h01SZn00<003Zn>[h01CZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h01SZn
0`001cZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h023Zn00<007ooOol05goo0P007Woo
000TOol00`00Oomoo`08Ool00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj
_P05>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj
_P08>[h00`00>[hj_P02>[h300000cZn000j_P08>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00
>[hj_P07>[h2000IOol00`00Oomoo`0LOol002=oo`8000]oo`03000j_SZn00@j_P03000j_SZn00Dj
_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00Dj
_P03000j_SZn00Hj_P03000j_SZn00<j_SP001Uoo`03001oogoo00Eoo`D000Aoo`8000Eoo`@0009o
o`0017oo0`001Woo0P001Woo00<007ooOol00goo100017oo00<007ooOol02gooAP0013Zn0`001cZn
00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h023Zn00<007ooOol05goo
00<007ooOol01goo00<007ooOol017oo0P002Goo0@000Goo0003Ool01@00Oomoogoo00001Goo0P00
1Woo00<007ooOol01goo00@007ooOomoo`8000aoo`03000j_SZn00Dj_P03000j_SZn00@j_P03000j
_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Lj_P03000j_SZn00Hj_P03000j
_SZn00Hj_P04000j_SZn>[h30002>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h0
0`00>[hj_P07>[h00`00>[hj_P07>[h2000IOol20007Ool00`00Oomoo`0?Ool10001Ool000=oo`05
001oogooOol0000>Ool00`00Oomoo`06Ool01@00Oomoogoo00003Woo00<003Zn>[h01CZn00<003Zn
>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01cZn00<003Zn
>[h01SZn00<003Zn>[h01SZn0`001SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01cZn
00<003Zn>[h01cZn00<003Zn>[h01cZn0P006Goo00<007ooOol01Goo00<007ooOol03goo0@000Goo
0003Ool01@00Oomoogoo00003Woo00<007ooOol01Woo00D007ooOomoo`0000moo`03000j_SZn00Dj
_P03000j_SZn00@j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Lj
_P03000j_SZn00Hj_P03000j_SZn008j_P<00003>[h003Zn00Lj_P03000j_SZn00Hj_P03000j_SZn
00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Qoo`03001oogoo00Eoo`03
001oogoo00]oo`@0009oo`000goo00D007ooOomoo`0000ioo`03001oogoo009oo`@000Aoo`03001o
ogoo00ioo`03000j_SZn00Dj_P03000j_SZn00@j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j
_SZn00Dj_P03000j_SZn00Lj_P03000j_SZn00Hj_P@000Dj_P03000j_SZn00Hj_P03000j_SZn00Hj
_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Qoo`8000Eo
o`03001oogoo00]oo`03001oogoo00=oo`000goo00D007ooOomoo`0000moo`05001oogooOol00006
Ool2000AOol00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00
>[hj_P05>[h00`00>[hj_P03>[iB000GOol20005Ool00`00Oomoo`0;Ool00`00Oomoo`03Ool000=o
o`05001oogooOol0000;Ool01@00Oomoogoo00000goo00<007ooOol017oo00<007ooOol04Goo;000
13Zn00@003Zn>[hj_P8000<j_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j
_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Qoo`04001oogooOol3
000=Ool40002Ool000Aoo`<000aoo`D000=oo`@000=oo`03001oogoo01=oo`03000j_SZn00@j_P03
000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P<0
00Hj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn
00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Qoo`03001oogoo01Ioo`008Woo0P005Goo00<0
03Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00@0
03Zn>[hj_P<00003>[h003Zn00Lj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03
000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Qoo`03001oogoo
01Eoo`008Woo0P005Woo00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h0
1CZn00<003Zn>[h01CZn0`001CZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<0
03Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn>[h01SZn0P00
67oo00<007ooOol05Goo000ROol00`00Oomoo`0FOol00`00>[hj_P04>[h00`00>[hj_P05>[h00`00
>[hj_P05>[h00`00>[hj_P05>[h01000>[hj_SZn0`0000<j_P00>[h01SZn00<003Zn>[h01SZn00<0
03Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<0
03Zn>[h01cZn00<003Zn>[h01SZn0P0067oo00<007ooOol057oo000IOol50003Ool00`00Oomoo`0H
Ooml000HOol2000DOol001]oo`03001oogoo00=oo`8001Yoo`03000j_SZn00Dj_P03000j_SZn00@j
_P03000j_SZn00Dj_P04000j_SZn>[h300000cZn000j_P06>[h00`00>[hj_P05>[h00`00>[hj_P05
>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07
>[h00`00>[hj_P07>[h00`00>[hj_P06>[h2000HOol00`00Oomoo`0COol001]oo`03001oogoo00=o
o`03001oogoo01Yoo`03000j_SZn00Dj_P03000j_SZn00@j_P03000j_SZn00Dj_P<000Dj_P03000j
_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j
_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Hj_P8001Qo
o`03001oogoo019oo`006goo00<007ooOol00goo0P0077oo00<003Zn>[h01CZn00<003Zn>[h013Zn
00<003Zn>[h00SZn10001cZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn
>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01cZn00<003Zn
>[h01cZn00<003Zn>[h01SZn0P0067oo00<007ooOol04Goo000KOol00`00Oomoo`03Ool00`00Oomo
o`0LOol00`00>[hj_P05>[h00`00>[hj_P04>[h40004>[h00`00>[hj_P05>[h00`00>[hj_P05>[h0
0`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj_P06>[h0
0`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P07>[h2000GOol2000BOol0
01]oo`03001oogoo00=oo`03001oogoo01eoocD0008j_P03000j_SZn00Hj_P03000j_SZn00Hj_P03
000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Hj_P03
000j_P0001Moo`03001oogoo00Eoo`D000Ioo`006goo00<007ooOol00goo00<007ooOol07Woo00<0
03Zn>[h01CZn0`0013Zn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h0
13Zn00<003Zn>[h00cZnA`005Woo00<007ooOol01Woo00@007ooOol000Ioo`006Goo0`0017oo0P00
8Goo00@003Zn>[hj_P<00003>[h003Zn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn
00Dj_P03000j_SZn00@j_P03000j_SZn00Hj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn
00Hj_P03000j_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Lj_P8001Moo`80
00Moo`03001oogoo00Ioo`0087oo00<007ooOol08Goo0`001CZn00<003Zn>[h013Zn00<003Zn>[h0
1CZn00<003Zn>[h01CZn00<003Zn>[h01CZn00<003Zn>[h013Zn00<003Zn>[h01SZn00<003Zn>[h0
1CZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h0
1cZn00<003Zn>[h01SZn00<003Zn00005goo00<007ooOol01Woo00<007ooOol01Goo000POol2000P
Ool200000goo000j_P06>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj
_P05>[h00`00>[hj_P04>[h00`00>[hj_P06>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj
_P06>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P06>[h00`00>[h0
000FOol00`00Oomoo`07Ool00`00Oomoo`04Ool000D000Aoo`8000Aoo`D000=oo`@000Eoo`03001o
ogoo01aoo`<000Aoob`0008j_P03000j_SZn00Hj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j
_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Lj_P03000j_SZn00Lj_P03000j_SZn00Hj_P8001Mo
o`03001oogoo00Ioo`03001oogoo00Aoo`000Woo00<007ooOol017oo0P001Goo00@007ooOol000Mo
o`03001oogoo009oo`03001oogoo01Uoo`<000Uoo`03000j_SZn00@j_P03000j_SZn00@j_P03000j
_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P04000j_SZn>[h/0003>[h00`00>[hj_P07>[h0
0`00>[hj_P07>[h00`00>[hj_P06>[h2000EOol30003Ool01@00Oomoogoo00001Woo0002Ool00`00
Oomoo`0<Ool00`00Oomoo`07Ool00`00Oomoo`02Ool00`00Oomoo`0FOol3000=Ool20005>[h00`00
>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00
>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj_P05>[h00`00>[hj_P03>[hS000A
Ool30002Ool00`00Oomoo`02Ool30007Ool0009oo`03001oogoo00eoo`03001oogoo00Ioo`05001o
ogooOol0000FOol3000BOol00`00>[hj_P03>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj
_P05>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj
_P06>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj
_P06>[h2000>Ool30005Ool00`00Oomoo`0<Ool0009oo`03001oogoo00ioo`04001oogooOol40004
Ool2000BOol3000FOol500000cZn000j_P05>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj
_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P06>[h00`00>[hj
_P05>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P06>[h20009Ool4
0008Ool00`00Oomoo`0<Ool0009oo`03001oogoo00ioo`05001oogooOol00007Ool00`00Oomoo`0?
Ool2000IOol00`00>[hj_P02>[hV0002>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P06
>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P07>[h00`00>[hj_P06
>[h20005Ool3000<Ool00`00Oomoo`0<Ool0009oo`03001oogoo00Yoo`05001oogooOol00003Ool0
0`00Oomoo`05Ool2000=Ool3000LOol00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P04>[h0
0`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P02>[hS0002>[h00`00>[hj_P06>[h00`00>[hj
_P07>[h00`00>[hj_P06>[h00`00>[hj_P06>[h200000goo00000002000@Ool00`00Oomoo`0;Ool0
00<000eoo`<000Aoo`@000Aoo`03001oogoo00Uoo`<0021oo`03000j_SZn00Dj_P03000j_SZn00@j
_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00Dj_P03000j_SZn00@j_P03000j_SZn00Hj
_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00<j_Q`000<j_P03000j_SZn00Hj_P8001Ao
o`03001oogoo00]oo`007goo00<007ooOol01Woo0`0097oo00<003Zn>[h01CZn00<003Zn>[h013Zn
00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h013Zn00<003Zn>[h01SZn
00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01cZn
00<003Zn>[h00cZn3`004goo00<007ooOol02goo000NOol20005Ool3000XOolD0002>[h00`00>[hj
_P04>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P06>[h00`00>[hj_P05>[h00`00>[hj
_P06>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P07>[h00`00>[hj_P06>[h00`00>[hj
_P06>[h2000COol00`00Oomoo`0;Ool001ioo`04001oogooOol3000]Ool00`00>[hj_P03>[h00`00
>[hj_P05>[h01000>[hj_SZn5`000SZn00<003Zn>[h01SZn00<003Zn>[h01CZn00<003Zn>[h01CZn
00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01SZn00<003Zn>[h01SZn
0P004Woo00<007ooOol02goo000NOol4000aOol00`00>[hj_P03>[h00`00>[hj_P05>[h00`00>[hj
_P04>[h00`00>[hj_P04>[h00`00>[hj_P05>[h01000>[hj_SZn6P000SZn00<003Zn>[h01SZn00<0
03Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01cZn00<003Zn>[h01SZn0P004Goo00<007ooOol0
2goo0006Ool50004Ool20005Ool40004Ool2000dOol00`00>[hj_P03>[h00`00>[hj_P05>[h00`00
>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00
>[hj_P06>[h00`00>[hj_P02>[hK0003>[h00`00>[hj_P07>[h00`00>[hj_P06>[h2000@Ool00`00
Oomoo`0;Ool000Qoo`03001oogoo00Aoo`8000Uoo`03001oogoo00=oo`03001oogoo039oo`D00003
>[h003Zn00Hj_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03
000j_SZn00@j_P03000j_SZn00Hj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Lj_P03
000j_SZn008j_QT0011oo`03001oogoo00Yoo`0027oo00<007ooOol03goo00<007ooOol017oo00<0
07ooOol0<Woo00@003Zn>[hj_QH0008j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00@j
_P03000j_SZn00Hj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Lj_P03000j_SZn00Dj
_P03000j_SZn00Lj_P03000j_SZn00Hj_P80011oo`03001oogoo00Yoo`0027oo00<007ooOol03goo
00<007ooOol01Goo00<007ooOol0<Woo00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h013Zn
00@003Zn>[hj_QP0008j_P03000j_SZn00Hj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn
00Lj_P03000j_SZn00Dj_P03000j_SZn00Lj_P03000j_SZn00Hj_P8000moo`03001oogoo00Yoo`00
27oo00<007ooOol02goo10002Goo0P00<goo00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h0
13Zn00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h00SZn6P000SZn00<0
03Zn>[h01cZn00<003Zn>[h01CZn00<003Zn>[h01cZn00<003Zn>[h01CZn00<003Zn00003Woo00<0
07ooOol02Woo0008Ool00`00Oomoo`0;Ool00`00Oomoo`0<Ool00`00Oomoo`0aOol00`00>[hj_P04
>[h00`00>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P05
>[h00`00>[hj_P06>[h00`00>[hj_P04>[h00`00>[hj_P06>[h00`00>[hj_P02>[hC0002>[h00`00
>[hj_P07>[h00`00>[hj_P05>[h2000>Ool00`00Oomoo`0:Ool000Qoo`03001oogoo00]oo`03001o
ogoo00eoo`03001oogoo035ooa<0008j_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00Dj
_P03000j_SZn00Hj_P03000j_SZn00@j_P03000j_SZn00Hj_P03000j_SZn00@j_P03000j_SZn00Lj
_P03000j_SZn00<j_Q<0008j_P8000eoo`03001oogoo00Yoo`001Woo0`003Goo10003Goo00<007oo
Ool0<Goo00<003Zn>[h01CZn00<003Zn>[h00cZn00@003Zn>[hj_Pl0008j_P03000j_SZn00Dj_P03
000j_SZn00Hj_P03000j_SZn00@j_P03000j_SZn00Hj_P03000j_SZn00@j_P03000j_SZn00Lj_P03
000j_SZn00Dj_P03000j_SZn00Lj_P03000j_SZn00<j_PD000aoo`03001oogoo00Yoo`00:7oo00<0
07ooOol0<Goo00<003Zn>[h01CZn00<003Zn>[h00cZn00<003Zn>[h013Zn00<003Zn>[h013Zn00@0
03Zn>[hj_QT0008j_P03000j_SZn00Hj_P03000j_SZn00@j_P03000j_SZn00Lj_P03000j_SZn00Dj
_P03000j_SZn00Lj_P03000j_SZn00Dj_P8000eoo`03001oogoo00Uoo`00:Goo0P00<Woo00<003Zn
>[h01CZn00<003Zn>[h00cZn00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn
>[h01CZn00<003Zn>[h013Zn00<003Zn>[h00SZn4@000SZn00<003Zn>[h01cZn00<003Zn>[h01CZn
00<003Zn>[h01cZn00<003Zn>[h01CZn0P0037oo00<007ooOol02Goo000[Ool00`00Oomoo`0`Ool=
00000cZn000j_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P04
>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P02>[hC0002>[h00`00>[hj_P07>[h00`00
>[hj_P05>[h2000;Ool00`00Oomoo`09Ool002aoo`03001oogoo035oo`03000j_SZn00@j_P04000j
_SZn>[h>0002>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h0
0`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P06>[h00`00>[hj_P03>[hC00000cZn0000000;
Ool00`00Oomoo`09Ool002eoo`03001oogoo035oo`03000j_SZn00@j_P03000j_SZn00<j_P03000j
_SZn00Dj_P03000j_SZn01P0008j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03
000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00Hj_P03000j_SZn00<j_P@000Yoo`03001oogoo
00Uoo`00;Woo0P00<Woo00<003Zn>[h013Zn00<003Zn>[h00cZn00<003Zn>[h01CZn00<003Zn>[h0
0cZn00<003Zn>[h01CZn00<003Zn>[h013Zn00<003Zn>[h00SZn4@000SZn00<003Zn>[h01CZn00<0
03Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01SZn00<003Zn>[h01CZn00<007ooOol027oo00<0
07ooOol02Goo000`Ool00`00Oomoo`0`Ool<00000cZn000j_P06>[h00`00>[hj_P03>[h00`00>[hj
_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P03>[hA0002>[h0
0`00>[hj_P07>[h00`00>[hj_P06>[h00`00>[hj_P05>[h00`00Oomoo`07Ool00`00Oomoo`09Ool0
035oo`03001oogoo031oo`03000j_SZn00<j_P03000j_SZn008j_Pl00003>[h003Zn00Hj_P03000j
_SZn00@j_P03000j_SZn00Dj_P03000j_SZn00@j_P03000j_SZn00Hj_P03000j_SZn00@j_P03000j
_SZn00<j_Q<0008j_P03000j_SZn00Dj_P8000Uoo`03001oogoo00Qoo`00<Woo00<007ooOol0<7oo
00<003Zn>[h00cZn00<003Zn>[h013Zn00<003Zn>[h013Zn00<003Zn>[h00SZn3`000SZn00<003Zn
>[h01CZn00<003Zn>[h013Zn00<003Zn>[h01SZn00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn
>[h01cZn00<003Zn>[h00cZn2@000SZn0P0027oo00<007ooOol027oo000cOol00`00Oomoo`0`Ool0
0`00>[hj_P03>[h00`00>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P04>[h00`00>[hj_P04>[h0
0`00>[hj_P02>[h@0002>[h00`00>[hj_P06>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj
_P07>[h00`00>[hj_P05>[h00`00>[hj_P03>[h50007Ool00`00Oomoo`08Ool003Aoo`80035oo`D0
0003>[h003Zn00Dj_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00Dj
_P03000j_SZn00@j_P03000j_SZn008j_Q40008j_P03000j_SZn00Dj_P03000j_SZn00Lj_P03000j
_SZn00Dj_P03000j_SZn00Hj_P03001oogoo00Eoo`03001oogoo00Qoo`00=Woo00<007ooOol0;goo
00@003Zn>[hj_Ph0008j_P03000j_SZn00@j_P03000j_SZn00@j_P03000j_SZn00Dj_P03000j_SZn
00@j_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn008j_PT0008j_P03000j_SZn00Hj_P03
000j_SZn00Hj_P03000j_SZn00Dj_P8000Ioo`03001oogoo00Qoo`00=goo00<007ooOol0;goo00<0
03Zn>[h00cZn00<003Zn>[h013Zn00@003Zn>[hj_Pl0008j_P03000j_SZn00Dj_P03000j_SZn00@j
_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn008j_Q8000<j_P03000j
_SZn00Dj_P8000Eoo`03001oogoo00Qoo`00>7oo00<007ooOol0;goo00<003Zn>[h00cZn00<003Zn
>[h013Zn00<003Zn>[h013Zn00<003Zn>[h013Zn00@003Zn>[hj_Q00008j_P03000j_SZn00Dj_P03
000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00<j_Ph0
00Aoo`03001oogoo00Qoo`00>Goo0P00<7oo1@0000<j_P00>[h01CZn00<003Zn>[h013Zn00<003Zn
>[h013Zn00<003Zn>[h013Zn00<003Zn>[h01CZn00@003Zn>[hj_Q00008j_P03000j_SZn00Hj_P03
000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00Dj_P8000Eoo`03001oogoo
00Moo`00>goo00<007ooOol0;Goo00<007ooOol00Woo3P000SZn00<003Zn>[h013Zn00<003Zn>[h0
13Zn00<003Zn>[h01CZn00<003Zn>[h013Zn00<003Zn>[h01CZn00@003Zn>[hj_Q80008j_P03000j
_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00Dj_P8000Aoo`03001oogoo00Moo`00?7oo00<007oo
Ool0;7oo00<007ooOol047oo3`000SZn00<003Zn>[h01CZn00<003Zn>[h013Zn00<003Zn>[h01CZn
00<003Zn>[h013Zn00<003Zn>[h01CZn00<003Zn>[h00cZn2@000SZn00<003Zn>[h01SZn00<003Zn
>[h01CZn0P000goo00<007ooOol01goo000mOol00`00Oomoo`0[Ool00`00Oomoo`0OOol@0002>[h0
0`00>[hj_P05>[h00`00>[hj_P04>[h00`00>[hj_P05>[h00`00>[hj_P05>[h00`00>[hj_P03>[hB
0002>[h20002Ool00`00Oomoo`07Ool003ioo`03001oogoo02Yoo`03001oogoo02mooa00008j_P03
000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Dj_P03000j_SZn00Hj_P03000j_SZn00<j_PD0
0003Ool007oo00Qoo`00?goo0P00:Woo00<007ooOol0?goo2@000SZn00<003Zn>[h01CZn00<003Zn
>[h01CZn00<003Zn>[h01SZn00<003Zn>[h01CZn0P0000=oo`00Ool027oo0011Ool00`00Oomoo`0W
Ool00`00Oomoo`18OolA0002>[h00`00>[hj_P06>[h00`00>[hj_P05>[h30009Ool0049oo`03001o
ogoo02Ioo`03001oogoo05Uoo`T000<j_P03000j_SZn00@j_P04000j_P000008Ool004=oo`03001o
ogoo02Eoo`03001oogoo069oo`h000Qoo`00A7oo0P009Goo00<007ooOol0KWoo0P0027oo0016Ool0
0`00Oomoo`0ROol00`00Oomoo`1]Ool30008Ool004Moo`03001oogoo025oo`03001oogoo06Uoo`@0
00]oo`00B7oo00<007ooOol087oo00<007ooOol0IGoo10003goo0019Ool2000POol00`00Oomoo`1R
Ool3000COol004]oo`03001oogoo01eoo`03001oogoo05ioo`@001Ioo`00C7oo00<007ooOol077oo
00<007ooOol0FWoo10006Woo001=Ool00`00Oomoo`0KOol00`00Oomoo`1FOol4000NOol004ioo`03
001oogoo01Yoo`03001oogoo059oo`@0029oo`00Cgoo0P006Woo00<007ooOol0CWoo10009Woo001A
Ool00`00Oomoo`0GOol00`00Oomoo`1;Ool3000ZOol0059oo`03001oogoo01Eoo`03001oogoo04Qo
o`@002eoo`00Dgoo00<007ooOol057oo00<007ooOol0A7oo1000<Goo001DOol2000DOol00`00Oomo
o`10Ool4000eOol005Ioo`03001oogoo015oo`03001oogoo03aoo`@003Uoo`00Egoo00<007ooOol0
47oo00<007ooOol0>7oo1000?Goo001HOol00`00Oomoo`0?Ool00`00Oomoo`0eOol30011Ool005Uo
o`03001oogoo00ioo`03001oogoo035oo`@004Aoo`00FWoo0P003Woo00<007ooOol0;Goo1000B7oo
001LOol00`00Oomoo`0;Ool00`00Oomoo`0YOol4001<Ool005eoo`03001oogoo00Yoo`03001oogoo
02Eoo`@0051oo`00GWoo00<007ooOol02Goo00<007ooOol08Goo1000E7oo001OOol20009Ool00`00
Oomoo`0NOol3001HOol0065oo`03001oogoo00Ioo`03001oogoo01Yoo`@005]oo`00HWoo00<007oo
Ool01Goo00<007ooOol05Woo1000Ggoo001SOol00`00Oomoo`04Ool00`00Oomoo`0BOol4001SOol0
06Aoo`03001oogoo00=oo`03001oogoo00ioo`@006Moo`00IGoo0P000goo00<007ooOol02Woo1000
Jgoo001WOol01000Oomoo`002Goo0`00Kgoo001XOol00`00Ool00005Ool4001bOol006Uoo`800003
Ool00000008007Ioo`00JWoo0P00NWoo003VOol00>Ioo`00iWoo003VOol00>Ioo`00iWoo003VOol0
0>Ioo`00iWoo003VOol00001\
\>"],
  ImageRangeCache->{{{0, 229.312}, {185.562, 0}} -> {-0.151408, -0.032812, 
  0.00622594, 0.00622594}}]
}, Open  ]],

Cell[TextData[{
  "Other special solutions correspond to steady-state American Puts and \
Calls. More generally, we have profiles as shown, for \[Lambda] strongly \
positive or negative. Here is a plot for ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda] > 0\)]],
  " - you should explore other values for yourself."
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[PowerContract[S, 10, 0.1, 0, 0.2, \ t, 4], \ {S, \ 5, \ 15}, 
      \ {t, \ 0, \ 2}, \ AxesLabel\  -> \ {"\<S\>", \ "\<t\>", \ "\<\>"}]; 
    \)\)], "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.08608 .22315 -6 -8.87306 ]
[.08608 .22315 0 .12694 ]
[.20731 .17545 -5.83689 -9 ]
[.20731 .17545 .16311 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.46808 .07289 -10.673 -9 ]
[.46808 .07289 1.32698 0 ]
[.60856 .01769 -10.1726 -9 ]
[.60856 .01769 1.82736 0 ]
[.29165 .07573 -9.31117 -14 ]
[.29165 .07573 .68883 0 ]
[.70096 -0.00478 0 -6.26206 ]
[.70096 -0.00478 6 2.73794 ]
[.78313 .12104 0 -6.13858 ]
[.78313 .12104 18 2.86142 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9224 .33431 0 -5.93715 ]
[.9224 .33431 18 3.06285 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.91861 .21225 0 -9.38198 ]
[.91861 .21225 10 4.61802 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01274 .33642 -6 -2.80265 ]
[.01274 .33642 0 6.19735 ]
[.00082 .41111 -12 -2.86613 ]
[.00082 .41111 0 6.13387 ]
[-0.01168 .48943 -12 -2.93303 ]
[-0.01168 .48943 0 6.06697 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -18 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(S) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.68874 0 m
.68263 .00239 L
s
[(0)] .70096 -0.00478 -1 .39157 Mshowa
.7708 .12553 m
.76464 .12778 L
s
[(0.5)] .78313 .12104 -1 .36413 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(1)] .8565 .23339 -1 .34028 Mshowa
.9099 .3383 m
.90365 .34029 L
s
[(1.5)] .9224 .33431 -1 .31937 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(2)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70593 .0263 m
.70226 .02771 L
s
.72272 .05198 m
.71904 .05338 L
s
.73912 .07706 m
.73543 .07845 L
s
.75514 .10158 m
.75145 .10294 L
s
.78611 .14895 m
.78241 .15028 L
s
.80108 .17185 m
.79737 .17317 L
s
.81573 .19425 m
.81201 .19555 L
s
.83006 .21617 m
.82633 .21745 L
s
.8578 .2586 m
.85407 .25986 L
s
.87123 .27915 m
.8675 .28039 L
s
.88439 .29928 m
.88065 .3005 L
s
.89727 .31899 m
.89353 .3202 L
s
.92227 .35722 m
.91851 .3584 L
s
.93439 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .3951 L
s
.95792 .41176 m
.95416 .41291 L
s
gsave
.91861 .21225 -61 -13.382 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(t) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02501 .33179 m
.03115 .32947 L
s
[(5)] .01274 .33642 1 -0.37719 Mshowa
.01315 .40663 m
.01932 .4044 L
s
[(10)] .00082 .41111 1 -0.36308 Mshowa
.00071 .48512 m
.00691 .48296 L
s
[(15)] -0.01168 .48943 1 -0.34822 Mshowa
.125 Mabswid
.03411 .27436 m
.03778 .27294 L
s
.03187 .28852 m
.03554 .28711 L
s
.0296 .30281 m
.03328 .30141 L
s
.02732 .31723 m
.031 .31584 L
s
.02269 .34648 m
.02637 .3451 L
s
.02034 .36131 m
.02403 .35994 L
s
.01796 .37627 m
.02166 .37491 L
s
.01557 .39138 m
.01927 .39003 L
s
.01071 .42203 m
.01442 .4207 L
s
.00825 .43758 m
.01196 .43625 L
s
.00576 .45327 m
.00947 .45196 L
s
.00325 .46912 m
.00696 .46781 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.647 .736 .914 r
.38303 .58543 .40504 .60607 .44179 .59672 .42009 .57568 Metetra
.64 .733 .916 r
.42009 .57568 .44179 .59672 .47907 .588 .45769 .5665 Metetra
.63 .728 .918 r
.45769 .5665 .47907 .588 .51693 .58011 .49586 .55808 Metetra
.618 .721 .92 r
.49586 .55808 .51693 .58011 .5554 .57331 .53464 .55066 Metetra
.603 .713 .922 r
.53464 .55066 .5554 .57331 .59455 .56785 .57411 .54447 Metetra
.585 .702 .923 r
.57411 .54447 .59455 .56785 .63446 .56407 .61432 .53981 Metetra
.563 .687 .923 r
.61432 .53981 .63446 .56407 .67522 .5623 .65537 .53701 Metetra
.538 .669 .922 r
.65537 .53701 .67522 .5623 .71697 .56294 .69738 .53643 Metetra
.509 .647 .919 r
.69738 .53643 .71697 .56294 .75985 .56644 .74048 .53849 Metetra
.477 .621 .913 r
.74048 .53849 .75985 .56644 .80404 .57332 .78485 .54364 Metetra
.442 .591 .904 r
.78485 .54364 .80404 .57332 .84977 .58416 .8307 .55244 Metetra
.405 .557 .892 r
.8307 .55244 .84977 .58416 .8973 .59963 .87827 .56549 Metetra
.368 .52 .877 r
.87827 .56549 .8973 .59963 .94699 .62054 .92788 .58351 Metetra
.331 .481 .858 r
.92788 .58351 .94699 .62054 .99922 .6478 .97991 .60734 Metetra
.649 .736 .913 r
.36046 .56427 .38303 .58543 .42009 .57568 .39784 .55412 Metetra
.642 .733 .915 r
.39784 .55412 .42009 .57568 .45769 .5665 .43575 .54449 Metetra
.633 .729 .917 r
.43575 .54449 .45769 .5665 .49586 .55808 .47425 .53554 Metetra
.622 .723 .919 r
.47425 .53554 .49586 .55808 .53464 .55066 .51336 .5275 Metetra
.608 .715 .921 r
.51336 .5275 .53464 .55066 .57411 .54447 .55314 .5206 Metetra
.591 .705 .922 r
.55314 .5206 .57411 .54447 .61432 .53981 .59366 .5151 Metetra
.571 .692 .923 r
.59366 .5151 .61432 .53981 .65537 .53701 .63502 .5113 Metetra
.548 .676 .922 r
.63502 .5113 .65537 .53701 .69738 .53643 .67731 .50955 Metetra
.522 .656 .92 r
.67731 .50955 .69738 .53643 .74048 .53849 .72066 .51022 Metetra
.492 .632 .915 r
.72066 .51022 .74048 .53849 .78485 .54364 .76524 .51374 Metetra
.459 .604 .908 r
.76524 .51374 .78485 .54364 .8307 .55244 .81124 .5206 Metetra
.424 .572 .897 r
.81124 .5206 .8307 .55244 .87827 .56549 .85889 .53137 Metetra
.389 .538 .883 r
.85889 .53137 .87827 .56549 .92788 .58351 .90849 .54669 Metetra
.353 .501 .867 r
.90849 .54669 .92788 .58351 .97991 .60734 .96037 .56732 Metetra
.65 .737 .913 r
.33731 .54258 .36046 .56427 .39784 .55412 .375 .53203 Metetra
.643 .734 .915 r
.375 .53203 .39784 .55412 .43575 .54449 .41324 .52193 Metetra
.635 .73 .917 r
.41324 .52193 .43575 .54449 .47425 .53554 .45206 .51246 Metetra
.625 .725 .918 r
.45206 .51246 .47425 .53554 .51336 .5275 .49151 .50382 Metetra
.612 .718 .92 r
.49151 .50382 .51336 .5275 .55314 .5206 .53162 .49621 Metetra
.597 .708 .922 r
.53162 .49621 .55314 .5206 .59366 .5151 .57247 .48989 Metetra
.579 .697 .922 r
.57247 .48989 .59366 .5151 .63502 .5113 .61414 .48513 Metetra
.557 .682 .922 r
.61414 .48513 .63502 .5113 .67731 .50955 .65673 .48225 Metetra
.533 .664 .92 r
.65673 .48225 .67731 .50955 .72066 .51022 .70035 .48159 Metetra
.505 .642 .916 r
.70035 .48159 .72066 .51022 .76524 .51374 .74517 .48355 Metetra
.475 .616 .91 r
.74517 .48355 .76524 .51374 .81124 .5206 .79135 .48858 Metetra
.442 .587 .901 r
.79135 .48858 .81124 .5206 .85889 .53137 .83912 .49719 Metetra
.408 .554 .889 r
.83912 .49719 .85889 .53137 .90849 .54669 .88875 .50997 Metetra
.374 .519 .874 r
.88875 .50997 .90849 .54669 .96037 .56732 .94056 .52761 Metetra
.651 .737 .913 r
.31354 .52032 .33731 .54258 .375 .53203 .35156 .50936 Metetra
.645 .735 .914 r
.35156 .50936 .375 .53203 .41324 .52193 .39013 .49881 Metetra
.637 .731 .916 r
.39013 .49881 .41324 .52193 .45206 .51246 .42929 .48882 Metetra
.628 .726 .918 r
.42929 .48882 .45206 .51246 .49151 .50382 .46907 .47957 Metetra
.616 .72 .92 r
.46907 .47957 .49151 .50382 .53162 .49621 .50953 .47127 Metetra
.602 .711 .921 r
.50953 .47127 .53162 .49621 .57247 .48989 .55072 .46415 Metetra
.585 .701 .922 r
.55072 .46415 .57247 .48989 .61414 .48513 .59272 .45846 Metetra
.566 .687 .922 r
.59272 .45846 .61414 .48513 .65673 .48225 .63562 .45449 Metetra
.543 .671 .92 r
.63562 .45449 .65673 .48225 .70035 .48159 .67954 .45256 Metetra
.518 .651 .917 r
.67954 .45256 .70035 .48159 .74517 .48355 .72461 .45303 Metetra
.49 .627 .912 r
.72461 .45303 .74517 .48355 .79135 .48858 .77101 .45631 Metetra
.459 .6 .904 r
.77101 .45631 .79135 .48858 .83912 .49719 .81894 .46288 Metetra
.427 .57 .894 r
.81894 .46288 .83912 .49719 .88875 .50997 .86865 .47327 Metetra
.394 .537 .881 r
.86865 .47327 .88875 .50997 .94056 .52761 .92045 .48809 Metetra
.652 .737 .912 r
.28914 .49749 .31354 .52032 .35156 .50936 .32749 .48611 Metetra
.646 .735 .914 r
.32749 .48611 .35156 .50936 .39013 .49881 .3664 .47509 Metetra
.639 .732 .915 r
.3664 .47509 .39013 .49881 .42929 .48882 .4059 .46458 Metetra
.631 .727 .917 r
.4059 .46458 .42929 .48882 .46907 .47957 .44603 .45474 Metetra
.62 .722 .919 r
.44603 .45474 .46907 .47957 .50953 .47127 .48684 .44576 Metetra
.607 .714 .92 r
.48684 .44576 .50953 .47127 .55072 .46415 .52838 .43785 Metetra
.592 .704 .921 r
.52838 .43785 .55072 .46415 .59272 .45846 .57072 .43125 Metetra
.574 .692 .922 r
.57072 .43125 .59272 .45846 .63562 .45449 .61395 .42622 Metetra
.553 .677 .921 r
.61395 .42622 .63562 .45449 .67954 .45256 .65818 .42306 Metetra
.529 .659 .918 r
.65818 .42306 .67954 .45256 .72461 .45303 .70354 .4221 Metetra
.503 .637 .914 r
.70354 .4221 .72461 .45303 .77101 .45631 .75019 .42373 Metetra
.475 .612 .907 r
.75019 .42373 .77101 .45631 .81894 .46288 .79831 .42836 Metetra
.444 .584 .898 r
.79831 .42836 .81894 .46288 .86865 .47327 .84815 .43648 Metetra
.413 .554 .886 r
.84815 .43648 .86865 .47327 .92045 .48809 .89998 .44865 Metetra
.653 .738 .912 r
.26408 .47404 .28914 .49749 .32749 .48611 .30276 .46224 Metetra
.648 .736 .913 r
.30276 .46224 .32749 .48611 .3664 .47509 .34201 .45075 Metetra
.641 .733 .915 r
.34201 .45075 .3664 .47509 .4059 .46458 .38187 .43971 Metetra
.633 .729 .917 r
.38187 .43971 .4059 .46458 .44603 .45474 .42236 .42928 Metetra
.623 .723 .918 r
.42236 .42928 .44603 .45474 .48684 .44576 .46352 .41962 Metetra
.611 .716 .92 r
.46352 .41962 .48684 .44576 .52838 .43785 .50542 .41094 Metetra
.597 .707 .921 r
.50542 .41094 .52838 .43785 .57072 .43125 .54812 .40345 Metetra
.581 .696 .921 r
.54812 .40345 .57072 .43125 .61395 .42622 .5917 .3974 Metetra
.562 .683 .921 r
.5917 .3974 .61395 .42622 .65818 .42306 .63626 .39305 Metetra
.54 .666 .919 r
.63626 .39305 .65818 .42306 .70354 .4221 .68192 .39073 Metetra
.516 .646 .915 r
.68192 .39073 .70354 .4221 .75019 .42373 .72885 .39076 Metetra
.489 .624 .91 r
.72885 .39076 .75019 .42373 .79831 .42836 .7772 .39354 Metetra
.461 .598 .902 r
.7772 .39354 .79831 .42836 .84815 .43648 .82721 .39951 Metetra
.431 .569 .892 r
.82721 .39951 .84815 .43648 .89998 .44865 .87912 .40918 Metetra
.653 .738 .912 r
.23833 .44997 .26408 .47404 .30276 .46224 .27735 .43773 Metetra
.649 .736 .913 r
.27735 .43773 .30276 .46224 .34201 .45075 .31695 .42576 Metetra
.643 .733 .914 r
.31695 .42576 .34201 .45075 .38187 .43971 .35716 .4142 Metetra
.635 .73 .916 r
.35716 .4142 .38187 .43971 .42236 .42928 .39802 .40317 Metetra
.626 .725 .918 r
.39802 .40317 .42236 .42928 .46352 .41962 .43955 .39284 Metetra
.615 .718 .919 r
.43955 .39284 .46352 .41962 .50542 .41094 .48182 .3834 Metetra
.603 .71 .92 r
.48182 .3834 .50542 .41094 .54812 .40345 .52489 .37503 Metetra
.587 .7 .921 r
.52489 .37503 .54812 .40345 .5917 .3974 .56883 .36798 Metetra
.57 .688 .92 r
.56883 .36798 .5917 .3974 .63626 .39305 .61374 .36249 Metetra
.55 .673 .919 r
.61374 .36249 .63626 .39305 .68192 .39073 .65973 .35883 Metetra
.527 .655 .916 r
.65973 .35883 .68192 .39073 .72885 .39076 .70696 .35734 Metetra
.503 .634 .912 r
.70696 .35734 .72885 .39076 .7772 .39354 .75557 .35835 Metetra
.476 .61 .905 r
.75557 .35835 .7772 .39354 .82721 .39951 .80579 .36228 Metetra
.448 .583 .896 r
.80579 .36228 .82721 .39951 .87912 .40918 .85784 .36959 Metetra
.654 .738 .911 r
.21187 .42523 .23833 .44997 .27735 .43773 .25123 .41255 Metetra
.65 .736 .913 r
.25123 .41255 .27735 .43773 .31695 .42576 .29118 .4001 Metetra
.644 .734 .914 r
.29118 .4001 .31695 .42576 .35716 .4142 .33176 .388 Metetra
.638 .731 .916 r
.33176 .388 .35716 .4142 .39802 .40317 .37298 .37637 Metetra
.629 .726 .917 r
.37298 .37637 .39802 .40317 .43955 .39284 .4149 .36538 Metetra
.619 .72 .918 r
.4149 .36538 .43955 .39284 .48182 .3834 .45755 .35517 Metetra
.607 .713 .92 r
.45755 .35517 .48182 .3834 .52489 .37503 .50099 .34595 Metetra
.593 .704 .92 r
.50099 .34595 .52489 .37503 .56883 .36798 .54531 .33792 Metetra
.577 .692 .92 r
.54531 .33792 .56883 .36798 .61374 .36249 .59059 .33131 Metetra
.559 .679 .919 r
.59059 .33131 .61374 .36249 .65973 .35883 .63694 .32637 Metetra
.538 .662 .917 r
.63694 .32637 .65973 .35883 .70696 .35734 .68449 .32341 Metetra
.515 .643 .913 r
.68449 .32341 .70696 .35734 .75557 .35835 .7334 .32273 Metetra
.49 .621 .907 r
.7334 .32273 .75557 .35835 .80579 .36228 .78385 .32471 Metetra
.464 .597 .9 r
.78385 .32471 .80579 .36228 .85784 .36959 .83609 .32977 Metetra
.655 .738 .911 r
.18466 .3998 .21187 .42523 .25123 .41255 .22436 .38667 Metetra
.651 .737 .912 r
.22436 .38667 .25123 .41255 .29118 .4001 .26468 .37372 Metetra
.646 .735 .914 r
.26468 .37372 .29118 .4001 .33176 .388 .30562 .36108 Metetra
.64 .732 .915 r
.30562 .36108 .33176 .388 .37298 .37637 .34723 .34885 Metetra
.632 .727 .916 r
.34723 .34885 .37298 .37637 .4149 .36538 .38953 .33719 Metetra
.623 .722 .918 r
.38953 .33719 .4149 .36538 .45755 .35517 .43257 .32623 Metetra
.612 .715 .919 r
.43257 .32623 .45755 .35517 .50099 .34595 .47641 .31616 Metetra
.599 .707 .92 r
.47641 .31616 .50099 .34595 .54531 .33792 .52112 .30717 Metetra
.584 .697 .92 r
.52112 .30717 .54531 .33792 .59059 .33131 .56678 .29946 Metetra
.567 .684 .919 r
.56678 .29946 .59059 .33131 .63694 .32637 .6135 .29328 Metetra
.548 .669 .917 r
.6135 .29328 .63694 .32637 .68449 .32341 .6614 .28889 Metetra
.526 .652 .914 r
.6614 .28889 .68449 .32341 .7334 .32273 .71063 .28659 Metetra
.503 .632 .909 r
.71063 .28659 .7334 .32273 .78385 .32471 .76137 .28671 Metetra
.479 .609 .903 r
.76137 .28671 .78385 .32471 .83609 .32977 .81383 .28962 Metetra
.656 .739 .911 r
.15666 .37366 .18466 .3998 .22436 .38667 .19673 .36005 Metetra
.652 .737 .912 r
.19673 .36005 .22436 .38667 .26468 .37372 .23741 .34661 Metetra
.647 .735 .913 r
.23741 .34661 .26468 .37372 .30562 .36108 .27873 .33341 Metetra
.641 .732 .915 r
.27873 .33341 .30562 .36108 .34723 .34885 .32072 .32058 Metetra
.634 .729 .916 r
.32072 .32058 .34723 .34885 .38953 .33719 .36341 .30824 Metetra
.626 .724 .917 r
.36341 .30824 .38953 .33719 .43257 .32623 .40685 .29653 Metetra
.616 .718 .918 r
.40685 .29653 .43257 .32623 .47641 .31616 .4511 .28562 Metetra
.604 .71 .919 r
.4511 .28562 .47641 .31616 .52112 .30717 .49621 .27568 Metetra
.59 .701 .919 r
.49621 .27568 .52112 .30717 .56678 .29946 .54228 .2669 Metetra
.574 .689 .919 r
.54228 .2669 .56678 .29946 .6135 .29328 .58939 .25951 Metetra
.557 .676 .918 r
.58939 .25951 .6135 .29328 .6614 .28889 .63766 .25374 Metetra
.537 .66 .915 r
.63766 .25374 .6614 .28889 .71063 .28659 .68724 .24986 Metetra
.516 .641 .911 r
.68724 .24986 .71063 .28659 .76137 .28671 .73829 .24819 Metetra
.492 .62 .905 r
.73829 .24819 .76137 .28671 .81383 .28962 .79102 .24905 Metetra
.656 .739 .911 r
.12786 .34676 .15666 .37366 .19673 .36005 .16828 .33268 Metetra
.653 .738 .912 r
.16828 .33268 .19673 .36005 .23741 .34661 .20933 .31871 Metetra
.648 .736 .913 r
.20933 .31871 .23741 .34661 .27873 .33341 .25103 .30495 Metetra
.643 .733 .914 r
.25103 .30495 .27873 .33341 .32072 .32058 .29341 .2915 Metetra
.637 .73 .915 r
.29341 .2915 .32072 .32058 .36341 .30824 .33651 .27849 Metetra
.629 .725 .917 r
.33651 .27849 .36341 .30824 .40685 .29653 .38037 .26603 Metetra
.619 .72 .918 r
.38037 .26603 .40685 .29653 .4511 .28562 .42503 .25428 Metetra
.608 .713 .919 r
.42503 .25428 .4511 .28562 .49621 .27568 .47056 .2434 Metetra
.596 .704 .919 r
.47056 .2434 .49621 .27568 .54228 .2669 .51704 .23357 Metetra
.581 .694 .919 r
.51704 .23357 .54228 .2669 .58939 .25951 .56456 .22499 Metetra
.565 .681 .918 r
.56456 .22499 .58939 .25951 .63766 .25374 .61323 .21787 Metetra
.547 .667 .916 r
.61323 .21787 .63766 .25374 .68724 .24986 .66319 .21247 Metetra
.527 .65 .912 r
.66319 .21247 .68724 .24986 .73829 .24819 .71459 .20907 Metetra
.505 .63 .908 r
.71459 .20907 .73829 .24819 .79102 .24905 .76762 .20796 Metetra
.657 .739 .91 r
.09821 .31908 .12786 .34676 .16828 .33268 .13899 .3045 Metetra
.654 .738 .911 r
.13899 .3045 .16828 .33268 .20933 .31871 .18041 .29 Metetra
.65 .736 .913 r
.18041 .29 .20933 .31871 .25103 .30495 .2225 .27566 Metetra
.645 .734 .914 r
.2225 .27566 .25103 .30495 .29341 .2915 .26528 .26159 Metetra
.639 .731 .915 r
.26528 .26159 .29341 .2915 .33651 .27849 .30879 .24789 Metetra
.631 .727 .916 r
.30879 .24789 .33651 .27849 .38037 .26603 .35307 .23468 Metetra
.623 .722 .917 r
.35307 .23468 .38037 .26603 .42503 .25428 .39816 .22209 Metetra
.613 .715 .918 r
.39816 .22209 .42503 .25428 .47056 .2434 .44412 .21028 Metetra
.601 .707 .919 r
.44412 .21028 .47056 .2434 .51704 .23357 .49104 .19941 Metetra
.588 .698 .919 r
.49104 .19941 .51704 .23357 .56456 .22499 .53898 .18965 Metetra
.573 .687 .918 r
.53898 .18965 .56456 .22499 .61323 .21787 .58808 .18123 Metetra
.556 .673 .916 r
.58808 .18123 .61323 .21787 .66319 .21247 .63843 .17434 Metetra
.537 .658 .913 r
.63843 .17434 .66319 .21247 .71459 .20907 .69021 .16926 Metetra
.517 .64 .909 r
.69021 .16926 .71459 .20907 .76762 .20796 .74358 .16626 Metetra
.657 .739 .91 r
.06767 .29057 .09821 .31908 .13899 .3045 .10881 .27548 Metetra
.654 .738 .911 r
.10881 .27548 .13899 .3045 .18041 .29 .15061 .26043 Metetra
.651 .737 .912 r
.15061 .26043 .18041 .29 .2225 .27566 .1931 .24551 Metetra
.646 .734 .913 r
.1931 .24551 .2225 .27566 .26528 .26159 .23629 .2308 Metetra
.641 .732 .915 r
.23629 .2308 .26528 .26159 .30879 .24789 .28022 .21641 Metetra
.634 .728 .916 r
.28022 .21641 .30879 .24789 .35307 .23468 .32492 .20243 Metetra
.626 .723 .917 r
.32492 .20243 .35307 .23468 .39816 .22209 .37045 .18901 Metetra
.617 .717 .918 r
.37045 .18901 .39816 .22209 .44412 .21028 .41686 .17626 Metetra
.606 .71 .918 r
.41686 .17626 .44412 .21028 .49104 .19941 .46422 .16436 Metetra
.594 .702 .918 r
.46422 .16436 .49104 .19941 .53898 .18965 .51261 .15345 Metetra
.58 .691 .918 r
.51261 .15345 .53898 .18965 .58808 .18123 .56215 .14373 Metetra
.564 .679 .917 r
.56215 .14373 .58808 .18123 .63843 .17434 .61293 .1354 Metetra
.547 .665 .914 r
.61293 .1354 .63843 .17434 .69021 .16926 .66512 .12869 Metetra
.528 .649 .911 r
.66512 .12869 .69021 .16926 .74358 .16626 .71887 .12386 Metetra
.658 .739 .91 r
.0362 .26121 .06767 .29057 .10881 .27548 .07771 .24558 Metetra
.655 .738 .911 r
.07771 .24558 .10881 .27548 .15061 .26043 .1199 .22997 Metetra
.652 .737 .912 r
.1199 .22997 .15061 .26043 .1931 .24551 .16278 .21445 Metetra
.647 .735 .913 r
.16278 .21445 .1931 .24551 .23629 .2308 .20638 .19909 Metetra
.642 .732 .914 r
.20638 .19909 .23629 .2308 .28022 .21641 .25074 .18398 Metetra
.636 .729 .915 r
.25074 .18398 .28022 .21641 .32492 .20243 .29588 .16924 Metetra
.629 .725 .916 r
.29588 .16924 .32492 .20243 .37045 .18901 .34186 .15497 Metetra
.62 .719 .917 r
.34186 .15497 .37045 .18901 .41686 .17626 .38873 .14129 Metetra
.61 .713 .918 r
.38873 .14129 .41686 .17626 .46422 .16436 .43655 .12836 Metetra
.599 .705 .918 r
.43655 .12836 .46422 .16436 .51261 .15345 .48541 .11631 Metetra
.586 .696 .918 r
.48541 .11631 .51261 .15345 .56215 .14373 .5354 .10531 Metetra
.572 .685 .917 r
.5354 .10531 .56215 .14373 .61293 .1354 .58664 .09556 Metetra
.556 .672 .915 r
.58664 .09556 .61293 .1354 .66512 .12869 .63926 .08727 Metetra
.538 .657 .912 r
.63926 .08727 .66512 .12869 .71887 .12386 .69342 .08065 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.09549 .2323 m
.1002 .23687 L
s
[(6)] .08608 .22315 1 .97179 Mshowa
.21633 .18498 m
.22084 .18975 L
s
[(8)] .20731 .17545 .94563 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(10)] .3345 .12542 .86223 1 Mshowa
.47614 .08324 m
.48017 .08842 L
s
[(12)] .46808 .07289 .77884 1 Mshowa
.61605 .02846 m
.6198 .03385 L
s
[(14)] .60856 .01769 .69544 1 Mshowa
.125 Mabswid
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.18558 .19702 m
.18832 .19985 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.51046 .06981 m
.51284 .07294 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.06616 .24378 m
.06901 .2465 L
s
.65216 .01432 m
.65436 .01759 L
s
gsave
.29165 .07573 -70.3112 -18 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 22 translate 1 -1 scale
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
63.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
(S) show
69.000000 15.000000 moveto
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10.000000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000000 0.000000 0.000000 setrgbcolor
0.000000 0.000000 rmoveto
1.000000 setlinewidth
%%DocumentNeededResources: font Courier
%%DocumentSuppliedResources:
%%DocumentNeededFonts: Courier
%%DocumentSuppliedFonts:
%%DocumentFonts: font Courier
grestore
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}},
  ImageSize->{184.25, 149.25},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003V0000^Q000`40O003h00OiWoo003VOol00>Io
o`00iWoo003VOol00>Ioo`00iWoo003VOol008=oo`D000Aoo`<001Aoo`<0041oo`00QGoo00<007oo
Ool01Goo00<007ooOol04Woo00D007ooOomoo`0003moo`00QGoo00<007ooOol00Woo1@004goo00D0
07ooOomoo`0003moo`00QGoo00<007ooOol00Woo00@007ooOol001Aoo`05001oogooOol0000oOol0
08Eoo`03001oogoo00=oo`03001oo`0001Aoo`05001oogooOol0000oOol008Eoo`03001oogoo00=o
o`03001oo`0001Aoo`05001oogooOol0000oOol008Eoo`03001oogoo00Aoo`8000moo`8000=oo`05
001oogooOol0000oOol008=oo`<000Moo`03001oogoo00Yoo`<00003Ool007oo00=oo`<0041oo`00
V7oo0P0017oo0P00AWoo0015Ool5001;Ool30006Ool00`00Ool00015Ool004Eoo`03001oogoo009o
o`03001oogoo04Eoo`<000Uoo`03001oo`0004Eoo`00AGoo00<007ooOol00Woo00<007ooOol0@goo
0P0037oo00<007ooOol00P00@goo0018Ool3000JOol50002Ool5000LOol3000>Ool00`00Oomoo`02
0013Ool004Eoo`<001moo`03001oogoo00=oo`04001oogoo000JOol20003Ool00`00Oomoo`0;Ool0
1@00Oomoogoo0000@goo0015Ool00`00Oomoo`02Ool00`00Oomoo`0JOol00`00Oomoo`04Ool00`00
Oomoo`0GOol3000COol00`00Oomoo`02Ool00`00Oomoo`10Ool004Eoo`03001oogoo009oo`03001o
ogoo01Yoo`03001oogoo00Eoo`03001oogoo01Aoo`@001Eoo`03001oogoo009oo`03001oogoo03mo
o`00AWoo1@0077oo00<007ooOol01Woo00<007ooOol047oo0`006Goo00<007ooOol00Woo00<007oo
Ool0?goo001WOol00`00Oomoo`06Ool00`00Oomoo`0>Ool2000LOol00`00Oomoo`03Ool00`00Oomo
o`0nOol006Moo`03001oogoo009oo`05001oogooOol0000=Ool4000MOol00`00Oomoo`04Ool00`00
Oomoo`0mOol006Eoo`<000Eoo`<000]oo`<0025oo`03001oogoo00Aoo`03001oogoo03eoo`00NGoo
0P0097oo00<007ooOol01Goo00<007ooOol0?7oo001fOol300000goo001oo`0SOol00`00Oomoo`06
Ool00`00Oomoo`0kOol007Aoo`8002Ioo`@000Qoo`8003aoo`00BGoo1@000goo0`007Goo0`008goo
1@000dJm00<007ooOol01goo00<007ooOol0>Woo001;Ool00`00Oomoo`02Ool01@00Oomoogoo0000
6Woo0P000goo00<007ooOol067oo200024Jm0P002Goo00<007ooOol0>Goo001;Ool00`00Oomoo`02
Ool01@00Oomoogoo00005goo0`0067oo20001DJm00<004JmA[d024Jm00<004Jm000027oo00<007oo
Ool0>Goo001;Ool00`00Oomoo`02Ool01@00Oomoogoo00005Goo0`005Goo1@000TZm00<004JmA[d0
2DJm00<004JmA[d01dJm00<004Jm00002Goo00<007ooOol01Woo0`001Woo0P001Goo10007Woo001;
Ool00`00Oomoo`02Ool01@00Oomoogoo00004Woo0`004goo1@0024Zm00<004JmA[d02DJm00<004Jm
A[d01TJm00@004JmA[d000Qoo`8000Ioo`05001oogooOol00005Ool20009Ool00`00Oomoo`0KOol0
04]oo`03001oogoo009oo`05001oogooOol0000?Ool3000COol30002B]d00`00B[e:_@09B[d00`00
A[e6_@08A[d00`00A[e6_@06A[d01@00A[e6_DJm000027oo00<007ooOol017oo00D007ooOomoo`00
011oo`03001oogoo01]oo`00Bgoo00<007ooOol00Woo00D007ooOomoo`0000eoo`800003Ool007oo
011oo`<000I:g@03001:_DZm00Q:_@030016_DJm00U6_@030016_DJm00E6_@050016_DJmA[d00009
Ool00`00Oomoo`03Ool01@00Oomoogoo000047oo00<007ooOol06goo0019Ool30005Ool3000;Ool3
000AOol4000:B]d00`00B[e:_@08B[d00`00A[e6_@08A[d00`00A[e6_@05A[d00`00A[e6_@02A[d0
0`00Oomoo`06Ool00`00Oomoo`03Ool01@00Oomoogoo000037oo10007Woo001MOol2000?Ool50002
C]d00`00B]e:g@09B]d00`00B[e:_@09B[d00`00A[e6_@08A[d00`00A[e6_@04A[d70008Ool00`00
Oomoo`02Ool01@00Oomoogoo000037oo00<007ooOol07goo001JOol30002Ool00`00Oomoo`07Ool5
0008C]d00`00B]e:g@09B]d00`00B[e:_@08B[d00`00A[e6_@07A[d800000d:M0012W@03@Yd00`00
Oomoo`06Ool20003Ool01@00Oomoogoo000037oo00<007ooOol07goo000fOol3000OOol2000<Ool3
0002C]d00`00C]e>g@09C]d00`00B]e:g@08B]d00`00B[e:_@09B[d01000A[e6_DJm1@000TJm00<0
04:M@Yd014:M00<004:M@Yd00d:M00<007ooOol01Woo00<007ooOol00Woo0`003Goo10007Woo000e
Ool01@00Oomoogoo00006goo0`002goo0`001TkM00<004kMC]d02DkM00<004[MB]d024[M00<004Zm
B[d014Zm200024Jm00<004:M@Yd00d:M00<004:M@Yd00d:M00<007ooOol01goo00<007ooOol0<goo
000eOol01@00Oomoogoo00006Goo0P000goo00<007ooOol017oo10002TkM00<004kMC]d024kM00<0
04[MB]d01d[M20001DZm00<004JmA[d02DJm00<004:M@Yd00T:M00<004:M@Yd014:M00<007ooOol0
1goo00<007ooOol0<Woo000eOol01@00Oomoogoo00005Woo0`002Goo0`000Tkm00<004kMC]d02DkM
00<004kMC]d02DkM00@004[MB]e:g@D0009:g@03001:_DZm00U:_@030016_DJm00Q6_@030012WD:M
0092W@030012WD:M00E2W@03001oogoo00Ioo`8003=oo`00=Woo0`0057oo0`0000=oo`00Ool01Woo
0`001Tkm00<004kMC]d02DkM00<004kMC]d01dkM1@0024[M00<004ZmB[d02DZm00<004JmA[d024Jm
00D004:M@Ye2W@0000M2W@03001oogoo00Moo`03001oogoo035oo`00=Goo00D007ooOomoo`00015o
o`8000Uoo`<000Y>o@03001>gDkM00Q>g@03001>gDkM00A>g@<0009>g@03001:gD[M00U:g@03001:
_DZm00Q:_@030016_DJm00U6_@040012WD:M0008@Yd00`00Oomoo`07Ool00`00Oomoo`0`Ool003Eo
o`05001oogooOol0000>Ool40007Ool300000e;m001>o@0;C_d00`00C]e>g@08C]d00`00C]e>g@03
0006C]d00`00B]e:g@09B]d00`00B[e:_@08B[d00`00A[e6_@08A[d01000@Ye2W@0024:M00<007oo
Ool01goo00<007ooOol0<7oo000fOol3000=Ool20008Ool30005D_d00`00C_e>o@0:C_d00`00C]e>
g@06C]d4000:C]d00`00B]e:g@08B]d00`00B[e:_@08B[d00`00A[e6_@09A[d00`00@Yd00004@Yd6
0009Ool00`00Oomoo`0_Ool004=oo`<000Ioo`@000UBo@03001>oDkm00Y>o@03001>gDkM00D0009>
g@03001>gDkM00Y>g@03001:gD[M00Q:g@03001:_DZm00Q:_@030016_DJm00A6_@/000E2W@03001o
ogoo00Qoo`03001oogoo02ioo`00@Goo0P0000=oo`00Ool00goo0`000e;m00<005;mD_d025;m00<0
04kmC_d024km100024kM00<004kMC]d02DkM00<004[MB]d02D[M00<004ZmB[d00dZm2`001DJm0P00
2T:M00<007ooOol01goo00<007ooOol06goo0`0047oo000MOol3000NOol30005Ool30007D_d00`00
D_eBo@08D_d00`00C_e>o@04C_d30002C]d00`00C]e>g@08C]d00`00C]e>g@09C]d00`00B]e:g@07
B]d70005A[d00`00A[e6_@09A[d00`00@Ye2W@08@Yd00`00Oomoo`08Ool00`00Oomoo`0IOol01@00
Oomoogoo00003goo000LOol01@00Oomoogoo00006goo0P001Goo0`002e;m00<005;mD_d025;m00<0
04kmC_d00`001TkM00<004kMC]d024kM00<004kMC]d02DkM00@004[MB]e:g@D0009:_@030016_DJm
00Q6_@030016_DJm00Q6_@8000Y2W@03001oogoo00Qoo`03001oogoo01Qoo`03001oogoo015oo`00
77oo00D007ooOomoo`0001Qoo`<0009oo`03001oo`000080009Bo@03001BoE;m00YBo@03001BoE;m
00IBo@@000Y>g@03001>gDkM00Q>g@03001>gDkM00M>g@D000Q:_@030016_DJm00Q6_@030016_DJm
00M6_@8000]2W@03001oogoo00Moo`03001oogoo01Qoo`03001oogoo015oo`0077oo00D007ooOomo
o`0001Eoo`<000Aoo`<000IBo@03001BoE;m00YBo@03001BoE;m00D000=Bo@03001>gDkM00U>g@03
001>gDkM00Q>g@04001>gDkMC]d50002B]d00`00B[e:_@09B[d00`00A[e6_@08A[d00`00A[e6_@06
A[d00`00A[d0000:@Yd00`00Oomoo`08Ool00`00Oomoo`0GOol00`00Oomoo`0AOol001aoo`@001Ao
o`80009oo`03001oo`00008000YBo@03001BoE;m00QBo@@000UBo@03001>gDkM00Q>g@03001>gDkM
00M>g@D000Q:g@03001:_DZm00Q:_@030016_DJm00Q6_@030016_DJm00I6_@030016_@0000]2W@03
001oogoo00Qoo`03001oogoo01Aoo`H0011oo`0077oo00<007ooOol04Woo0`000goo0`000U;m00<0
05;mD_d02E;m00<005;mD_d015;m0`000U;m00<005;mD_d02E;m00<004kMC]d024kM00@004kMC]e>
g@D0009>g@03001:gD[M00U:g@03001:_DZm00Q:_@030016_DJm00Q6_@030016_DJm00E6_@040016
_DJm000:@Yd00`00Oomoo`08Ool00`00Oomoo`0FOol00`00Oomoo`0AOol001eoo`03001oogoo00mo
o`<0009oo`<000IBo@03001BoE;m00UBo@03001BoE;m00<000IBo@03001BoE;m00UBo@03001>gDkM
00I>g@D000Q>g@03001:gD[M00U:g@03001:_DZm00M:_@030016_DJm00M6_AT000Yoo`03001oogoo
00Ioo`D000Yoo`03001oogoo015oo`007Woo0`002goo0`000Woo0`002U;m00<005;mD_d025;m0`00
2U;m00<005;mD_d02E;m00<004kMC]d01@000TkM00<004kMC]d02DkM00<004[MB]d024[M00<004Zm
B[d024Zm00@004JmA[e6_@D00096_@030012WD:M00E2W@040012WD:M000:@Yd00`00Oomoo`09Ool0
0`00Oomoo`07Ool00`00Oomoo`0NOol002Yoo`80009oo`<000=Bo@03001BoE;m00QBo@03001BoE;m
00ABo@<00003D_d005;m00]Bo@03001BoE;m00MBo@@000Q>g@03001>gDkM00U>g@03001:gD[M00Q:
g@03001:_DZm00=:_@P000Q6_@030012WD:M00A2W@050012WD:M@Yd0000:@Yd00`00Oomoo`08Ool0
0`00Oomoo`07Ool00`00Oomoo`0NOol002Moo`<00004Ool000000007D_d00`00D_eBo@08D_d00`00
D_eBo@030005D_d00`00D_eBo@09D_d00`00D_eBo@04D_d30002C_d00`00C]e>g@08C]d00`00C]e>
g@08C]d00`00B]e:g@07B]d70005B[d00`00A[e6_@09A[d00`00@Ye2W@03@Yd00`00@Ye2W@02@Yd0
0`00@Ye2W@07@Yd00`00Oomoo`09Ool00`00Oomoo`06Ool00`00Oomoo`0NOol002Eoo`80009oo`80
00]Bo@03001BoE;m00MBo@<000UBo@03001BoE;m00UBo@03001BoE;m00<000I>o@03001>gDkM00Q>
g@03001>gDkM00Q>g@04001:gD[MB]d50002B]d00`00B[e:_@08B[d00`00A[e6_@08A[d00`00@Ye2
W@03@Yd00`00@Ye2W@02@Yd00`00@Ye2W@08@Yd00`00Oomoo`09Ool00`00Oomoo`05Ool00`00Oomo
o`0NOol0029oo`L0009Bo@03001BoE;m00UBo@03001BoE;m00=Bo@<00003D_d005;m00YBo@03001B
oE;m00QBo@<000Y>o@03001>gDkM00Q>g@03001>gDkM00I>g@D000Q:g@03001:_DZm00Q:_@030016
_DJm00Q6_@030012WD:M0092W@030012WD:M00=2W@030012WD:M00M2W@03001oogoo00Uoo`03001o
ogoo00Eoo`03001oogoo01ioo`007goo0`0000Aoo`00000000IBo@03001BoE;m00UBo@03001Bo@00
008000EBo@03001BoE;m00UBo@03001BoE;m00ABo@<0009Bo@03001>oDkm00U>o@03001>gDkM00Q>
g@03001>gDkM00D0009>g@03001:gD[M00U:g@03001:_DZm00Q:_@030016_DJm00M6_@030012WD:M
0092W@030012WD:M00=2W@030012WD:M00Q2W@03001oogoo00Qoo`8000Ioo`03001oogoo01ioo`00
7Goo1P002U;m00<005;mD_d025;m0P002E;m00<005;mD_d02E;m00<005;mD_d00`001U;m00<004km
C_d024km00<004kMC]d01dkM100024kM00<004[MB]d02D[M00<004ZmB[d01dZm00<004JmA[d024Jm
00<004:M@Yd00P001T:M00<004:M@Yd01d:M00<007ooOol02Woo00@007ooOomoo`<0021oo`006Woo
1P000UKm00<005;mD_d02E;m00<005;mD_d015;m0`000U;m00<005;mD_d01e;m00<005;mD_d025;m
0`002U;m00<004kmC_d024km00@004kMC]e>g@D0009>g@03001>gDkM00Q>g@03001:gD[M00Q:g@03
001:_DZm00Q:_@030016_DJm00=6_A/000]oo`03001oogoo02Aoo`006Goo10001UKm00<005;mD_d0
2E;m00<005;mD_d00`001U;m00<005;mD_d01e;m00<005;mD_d015;m0`000U;m00<005;mD_d02E;m
00<004kmC_d01Tkm1@0024kM00<004kMC]d024kM00<004[MB]d024[M00<004ZmB[d00dZm2P0000I6
_@00000004Jm0003@Yd00`00@Ye2W@04@Yd00`00?W`nO007?W`00`00Oomoo`0:Ool00`00Oomoo`0S
Ool001Qoo`8000YFo@03001BoE;m00QBo@<000YBo@03001BoE;m00MBo@03001BoE;m00<000IBo@03
001BoE;m00UBo@03001>oDkm00D000=>o@03001>gDkM00Q>g@03001>gDkM00M>g@03001:gD[M00M:
g@L000E6_@030016_DJm00<000E6_@050012WD:M@Yd00006@Yd00`00?W`nO007?W`00`00Oomoo`09
Ool2000TOol0011oo`<000Ioo`03001FoEKm00UFo@03001BoE;m00ABo@<0009Bo@03001BoE;m00UB
o@03001BoE;m00EBo@@000YBo@03001BoE;m00QBo@<000U>o@03001>gDkM00Q>g@03001>gDkM00M>
g@04001:gD[MB]d50002B]d00`00A[e6_@07A[d30009A[d01@00@Ye2WD:M00001D:M00<003il?W`0
23il00<007ooOol02Goo00<007ooOol08Woo000?Ool01@00Oomoogoo00001Goo0P002eKm00<005;m
D_d00`001U;m00<005;mD_d02E;m00<005;mD_d010000e;m00<005;mD_d02E;m00<005;mD_d015;m
0`0000=Bo@00C_d02Tkm00<004kMC]d024kM00<004kMC]d00dkM1`0024[M00<004JmA[d00dJm0`00
0TJm00<004JmA[d01dJm00D004:M@Ye2W@0000I2W@03000nO3il00LnO003001oogoo00Yoo`03001o
ogoo025oo`003goo00D007ooOomoo`0000Aoo`800003Ool005Km00UFo@<000YBo@03001BoE;m00QB
o@<000QBo@03001BoE;m00UBo@03001BoE;m00<000EBo@03001>oDkm00U>o@03001>gDkM00I>g@L0
00A:g@03001:gD[M00U:g@030016_@00008000I6_@030016_DJm00M6_@050012WD:M@Yd00005@Yd0
0`00?W`nO008?W`00`00Oomoo`0:Ool00`00Oomoo`0POol000moo`05001oogooOol00004Ool01@00
Oomoogoo00001eKm0P000UKm00<005;mD_d02E;m00<005;mD_d015;m0`0000=Bo@00D_d02E;m00<0
05;mD_d025;m0`002E;m00<004kmC_d02Dkm00<004kMC]d01@000TkM00<004[MB]d01d[M00<004[M
B]d01d[M0`002DJm00<004JmA[d01dJm00<004:M@Yd00T:M00<004:M@Yd00d:M00<003il?W`01cil
00<007ooOol02Woo0P008Goo000?Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<005KmE_d0
0UKm0P001EKm00<005;mD_d02E;m00<005;mD_d00`001E;m00<005;mD_d025;m00<005;mD_d015;m
0`000U;m00<005;mD_d025;m00<004kmC_d01dkm100024kM00<004[MB]d01d[M00<004[MB]d014[M
0P000T[M00<004JmA[d024Jm00<004JmA[d01dJm00D004:M@Ye2W@0000E2W@03000nO3il00PnO003
001oogoo00Yoo`03001oogoo01moo`003goo00D007ooOomoo`0000=oo`8000Eoo`04001Fo@000008
E_d00`00D_eBo@08D_d30009D_d00`00D_eBo@08D_d00`00D_eBo@030006D_d00`00D_eBo@07D_d0
0`00C_e>o@04C_d30002C]d00`00C]e>g@08C]d00`00B]e:g@07B]d00`00B]e:g@030005B]d00`00
A[e6_@03A[dU000<Ool00`00Oomoo`0NOol000moo`05001oogooOol00004Ool00`00Oomoo`04Ool2
000:E_d00`00D_eBo@04D_d30002D_d00`00D_eBo@08D_d00`00D_eBo@06D_d4000:D_d00`00D_eB
o@07D_d00`00C_e>o@030006C]d00`00C]e>g@08C]d00`00B]e:g@06B]d30004B]d:0005A[d00`00
@Ye2W@07@Yd00`00@Ye2W@02@Yd00`00@Ye2W@02@Yd00`00?W`nO008?W`00`00Oomoo`0:Ool00`00
Oomoo`0NOol0011oo`<000Eoo`03001oogoo00Ioo`03001FoEKm00QFo@03001BoE;m00<000IBo@03
001BoE;m00QBo@03001BoE;m00D000=Bo@03001BoE;m00UBo@03001BoE;m00EBo@@000Y>g@03001>
gDkM00M>g@03001:gD[M00=:g@X000E:_@030016_DJm00Q6_@030012WD:M00M2W@030012WD:M0092
W@030012WD:M0092W@03000nO3il00PnO003001oogoo00Yoo`8001ioo`005goo0P002Goo00<005Km
E_d01eKm0`002U;m00<005;mD_d01e;m0`002E;m00<005;mD_d02E;m00<005;mD_d010000e;m00<0
04kMC]d02DkM00<004kMC]d01dkM00<004[M00001@000T[M00<004ZmB[d024Zm00<004JmA[d01dJm
00<004:M@Yd024:M00D004:M@Ye2W@0000E2W@03000nO3il00LnO003001oogoo00Yoo`8001ioo`00
5goo00<007ooOol02Goo00<005KmE_d00eKm0`000UKm00<005;mD_d02E;m00<005;mD_d00e;m0`00
0U;m00<005;mD_d025;m00<005;mD_d025;m0`0025;m00<004kMC]d024kM00<004kMC]d01TkM1000
24[M00<004ZmB[d024Zm00<004JmA[d01dJm00<004:M@Yd024:M00D004:M@Ye2W@0000A2W@03000n
O3il00PnO003001oogoo00Yoo`03001oogoo01aoo`005goo00<007ooOol02Woo00<005Km00000P00
1UKm00<005;mD_d02E;m00<005;m00000P001U;m00<005;mD_d025;m00<005;mD_d015;m0`0000=B
o@00D_d02E;m00<004kMC]d024kM00<004kMC]d01@000TkM00<004[MB]d024[M00<004ZmB[d024Zm
00<004JmA[d01dJm00<004:M@Yd01d:M00<004:M@Yd00T:M00<004:M@Yd00T:M00<003il?W`023il
00<007ooOol02Woo00<007ooOol01Goo1@0017oo0P001Goo10000Woo000GOol00`00Oomoo`0;Ool0
0`00E_eFo@08E_d00`00D_eBo@08D_d2000:D_d00`00D_eBo@08D_d00`00D_eBo@030005D_d00`00
D_eBo@08D_d00`00C]e>g@06C]d60006C]d00`00B]e:g@08B]d00`00B[e:_@07B[d00`00A[e6_@07
A[d00`00@Ye2W@08@Yd00`00@Ye2W@02@Yd01@00@Ye2WD:M00002Sil00<007ooOol02goo00<007oo
Ool01Woo00<007ooOol017oo0P002Goo0@000Goo000GOol00`00Oomoo`0<Ool00`00E_eFo@08E_d0
0`00D_eBo@04D_d30002D_d00`00D_eBo@08D_d00`00D_eBo@06D_d40009D_d00`00D_eBo@08D_d0
0`00C]e>g@08000:C]d00`00B]e:g@07B]d00`00B[e:_@08B[d00`00A[e6_@03A[dE0003@Yd00`00
?W`nO008?W`00`00Oomoo`09Ool20008Ool00`00Oomoo`0?Ool10001Ool001Moo`03001oogoo00eo
o`03001FoEKm00QFo@03001BoE;m00<000IBo@03001BoE;m00QBo@03001BoE;m00D000=Bo@03001B
oE;m00QBo@03001BoE;m00IBo@@000=>g@8000=>g@03001>gDkM00Q>g@03001:gD[M00M:g@03001:
_DZm00=:_@X000A6_@030012WD:M00Q2W@03000nOCim008nO@`000<nO003001oogoo00]oo`03001o
ogoo00Eoo`03001oogoo00moo`40005oo`005Woo00<007ooOol03goo00<005KmE_d01eKm0`002U;m
00<005;mD_d01e;m0`002E;m00<005;mD_d025;m00<005;mD_d01@000Tkm00<004kM00000P001TkM
00<004kMC]d024kM00<004[MB]d00d[M2@001DJm00<004JmA[d01dJm00<004:M@Yd024:M00<003im
?Wd00Sim00D003im?WdnO@0000HjO0D000aoo`03001oogoo00Eoo`03001oogoo00]oo`@0009oo`00
3Woo100017oo00<007ooOol047oo00<005KmE_d00eKm0`000U;m00<005;mD_d02E;m00<005;mD_d0
0e;m0`000U;m00<005;mD_d025;m00<005;mD_d01U;m10001Tkm0`002TkM00<004kMC]d01TkM1`00
14[M00<004JmA[d024Jm00<004JmA[d01dJm00<004:M@Yd01d:M00<003im?Wd00Sim00<003im?Wd0
0Sim00<003Yl>W`01cYl00<007ooOol02goo0P001Goo00<007ooOol02goo00<007ooOol00goo000B
Ool01@00Oomoogoo00004goo00<005Km00000P001U;m00<005;mD_d02E;m00<005;m00000P001U;m
00<005;mD_d025;m00<005;mD_d01@000U;m00<004kmC_d00Tkm0`000dkm00<004kMC]d024kM00<0
04kMC]d01@000T[M00<004[MB]d01d[M00<004JmA[d01dJm00<004JmA[d024Jm00<004:M@Yd01d:M
00<003im?Wd00Sim00D003im?WdnO@0000XjO003001oogoo00Uoo`03001oo`0000Eoo`03001oogoo
00]oo`03001oogoo00=oo`004Woo00D007ooOomoo`0001Aoo`03001BoE;m00QBo@03001BoE;m00QB
o@8000YBo@03001BoE;m00MBo@<000QBo@@000M>o@03001>gDkM00I>g@@000Q:g@03001:gD[M00M:
g@030016_DJm00M6_@030016_DJm00M6_@030012WD:M00Q2W@03000nOCim008nO@05000nOCim?Wd0
0009>W`00`00Oomoo`0;Ool01000Oomoogoo0`003Goo10000Woo000BOol01@00Oomoogoo00005Goo
00<005;mD_d025;m00<005;mD_d015;m0`0000=Bo@00D_d02U;m00<005;mD_d00e;m0`0000=Bo@00
D_d01U;m10002Tkm00<004kMC]d01@000TkM00<004[MB]d024[M00<004[MB]d01d[M00<004JmA[d0
1dJm00<004JmA[d01dJm00<004:M@Yd01d:M00<003im?Wd00cim00D003im?WdnO@0000TjO003001o
ogoo00]oo`03001oogoo01Ioo`003Woo100017oo00<007ooOol057oo00<005;mD_d025;m00<005;m
D_d00`001E;m00<005;mD_d02E;m00<005;m00000P001E;m00@005;mD_eBo@<000ABo@03001>oDkm
00I>o@@000Q>g@03001:gD[M00Q:g@03001:gD[M00M:g@030016_DJm00I6_@030016_DJm00=6_AD0
00<nO@03000jO3Yl00LjO003001oogoo00aoo`03001oogoo01Eoo`003Woo00<007ooOol017oo0P00
5goo00<005;mD_d01e;m0`002E;m00<005;mD_d025;m0P002E;m0`0025;m00<004kmC_d01@000Tkm
00<004kMC]d024kM00<004[MB]d024[M00<004[MB]d00d[M4P001D:M00<004:M@Yd01d:M00<003il
?W`00cil20001SYl00<007ooOol02goo00<007ooOol05Goo000>Ool00`00Oomoo`04Ool00`00Oomo
o`0GOol00`00D_eBo@03D_d30003D_d00`00D_eBo@07D_d00`00D_eBo@04D_d300000e;m001Bo@06
D_d200000e;m001Bo@07D_d40008C_d00`00C]e>g@08C]d00`00B]e:g@06B]d70004B]d00`00A[e6
_@06A[d00`00@Ye2W@08@Yd00`00@Ye2W@07@Yd00`00?W`nO003?W`01000?W`nO0000cYL1@0000<j
O000Ool03Goo00<007ooOol057oo000>Ool40003Ool00`00Oomoo`0HOol00`00D_d000020007D_d0
0`00D_eBo@07D_d00`00D_eBo@030005D_d01000D_eBoE;m0`0015;m00<005;mD_d01@000U;m00<0
04kmC_d024km00<004kMC]d024kM00<004[MB]d01@000T[M00<004[MB]d01d[M00<004JmA[d01TJm
00<004:M@Yd024:M00<004:M@Yd01T:M00<003il?W`013il00@003il?W`000LjG0<000ioo`03001o
ogoo01=oo`005Goo00<007ooOol06Goo0P002U;m00<005;mD_d01E;m10002E;m0`001e;m0`0025;m
00<004kmC_d024km00<004kMC]d01TkM100024[M00<004[MB]d01d[M00<004JmA[d01TJm00<004:M
@Yd024:M00<004:M@Yd01T:M00<003il?W`00cil00D003il?W`nO00000PjG003001oogoo00aoo`80
01Aoo`005Goo00<007ooOol06goo00<005;mD_d025;m00<005;mD_d0100015;m00<005;mD_d00e;m
0`0000=Bo@00D_d015;m0`000U;m00<005;mD_d01e;m00<004kmC_d024km00<004kMC]d01@000TkM
00<004[MB]d024[M00<004[MB]d01d[M00<004JmA[d01TJm00<004:M@Yd01d:M00<004:M@Yd01T:M
00<003il?W`013il00@003il?W`000TjG003001oogoo00aoo`03001oogoo019oo`005Goo00<007oo
Ool077oo00<005;mD_d01e;m0`002E;m00<005;m00000P001E;m00<005;m00000P001U;m00<005;m
D_d01e;m00<004kmC_d01Tkm100024kM00<004[MB]d01d[M00<004[MB]d01d[M00<004JmA[d01dJm
00<004:M@Yd01d:M00<004:M@Yd01T:M00<003il?W`013il00@003il?W`000PjG003001oogoo00eo
o`03001oogoo015oo`0057oo0P007goo00<005;mD_d00e;m0`0000=Bo@00D_d025;m0`001e;m0`00
2U;m00<005;mD_d01e;m00<004kmC_d01@000dkM00<004kMC]d01dkM00<004[MB]d01d[M00<004[M
B]d01d[M00<004JmA[d00dJm50000T:M00<003il?W`013il00@003il?W`000TjG003001oogoo00ao
o`03001oogoo015oo`0057oo00<007ooOol07goo00<005;m00000P001E;m00<005;mD_d00e;m0`00
0e;m00<005;mD_d010000e;m00<005;mD_d025;m00<005;mD_d01E;m10002DkM00<004kMC]d01TkM
00<004[MB]d01T[M4@0014Jm00<004:M@Yd01d:M00<003im?Wd00cim2`000Sil00<003YL>U`01SYL
00<007ooOol037oo0P004Woo000DOol00`00Oomoo`0POol00`00D_eBo@07D_d00`00D_d000020006
D_d30008D_d00`00D_eBo@08D_d00`00D_eBo@040003D_d00`00C]e>g@08C]d00`00C]e>g@06C]d0
0`00B]e:g@050002B]d00`00A[e6_@07A[d00`00A[e6_@07A[d00`00@Ye2W@07@Yd00`00?WdnO@05
?Wd00`00>W`jO004>W`80005>U`00`00Oomoo`0=Ool00`00Oomoo`0?Ool001Aoo`80029oo`03001B
oE;m00IBo@8000IBo@<0009Bo@03001BoE;m00MBo@03001BoE;m00IBo@@000QBo@03001>gDkM00Q>
g@03001>gDkM009>g@H000Q:g@030016_DJm00M6_@030016_DJm00I6_@030012WD:M00M2W@03000n
OCim00HnO@03000jO3Yl00DjO003000jO00000<fG0H000moo`03001oogoo00Eoo`D000Eoo`0017oo
1@000goo0`001Goo00<007ooOol08Woo00<005;mD_d00U;m0`000U;m00<005;mD_d00`001U;m00<0
05;mD_d01e;m00<005;mD_d01@000e;m00<005;mD_d01e;m00<004kMC]d01TkM1P001DkM00<004[M
B]d01d[M00<004JmA[d01dJm00<004JmA[d01TJm00<004:M@Yd01d:M00<003im?Wd01Sim00<003Yl
>W`013Yl00@003Yl>W`000LfG08000moo`8000Ioo`04001oogoo0005Ool000Ioo`03001oogoo009o
o`05001oogooOol00004Ool00`00Oomoo`0SOol40004D_d4000:D_d00`00D_eBo@05D_d40009D_d0
0`00D_eBo@07D_d00`00C]e>g@050002C]d00`00C]e>g@07C]d00`00B]e:g@07B]d00`00A[e6_@06
A[d00`00A[e6_@07A[d00`00@Ye2W@07@Yd00`00?WdnO@06?Wd00`00>W`jO004>W`00`00>W`00008
=U`00`00Oomoo`0=Ool20007Ool00`00Oomoo`05Ool000Ioo`03001oogoo009oo`05001oogooOol0
0003Ool2000SOol40002D_d40004D_d00`00D_eBo@08D_d00`00D_eBo@040003D_d00`00D_eBo@08
D_d00`00D_eBo@05D_d40008C]d00`00C]e>g@07C]d00`00B]e:g@07B]d00`00A[e6_@06A[d00`00
A[e6_@06A[d00`00@Ye2W@07@Yd00`00?WdnO@06?Wd00`00>W`jO005>W`00`00>W`00008=U`00`00
Oomoo`0=Ool00`00Oomoo`06Ool00`00Oomoo`04Ool000Ioo`03001oogoo009oo`05001oogooOol0
0003Ool00`00Oomoo`0OOol30004Ool20009D_d00`00D_eBo@07D_d30008D_d00`00D_eBo@08D_d0
0`00D_eBo@040003C_d00`00C]e>g@07C]d00`00C]e>g@07C]d00`00B]e:g@03B]dV0003?Wd00`00
>W`jO005>W`00`00>W`00008=U`00`00Oomoo`0=Ool00`00Oomoo`06Ool00`00Oomoo`03Ool000Io
o`03001oogoo009oo`05001oogooOol00003Ool00`00Oomoo`0LOol30009Ool00`00D_eBo@07D_d0
0`00D_eBo@03D_d30002D_d00`00D_eBo@07D_d00`00D_eBo@06D_d40008C_d00`00C]e>g@06C]d0
0`00C]e>g@06C]d70004B]d00`00A[e6_@06A[d00`00@[e2_@06@[d00`00@Ye2W@07@Yd00`00?Wdn
O@03?Wd:000013Yl000jO00023IL00<007ooOol037oo0`001goo00<007ooOol00goo0006Ool00`00
Oomoo`02Ool01@00Oomoogoo00000goo00<007ooOol06Goo0`003Goo00<005;mD_d01e;m00<005;m
00000P001U;m00<005;mD_d01e;m00<005;mD_d01@000U;m00<004kmC_d024km00<004kMC]d01TkM
00<004kMC]d01@000T[M00<004[MB]d01d[M00<004JmA[d01TJm00<004:m@[d01T:m00<004:M@Yd0
1d:M00<003im?Wd01Sim00<003YL>U`00cYL1`001SIL00<007ooOol027oo0`000Woo00D007ooOomo
o`0000=oo`03001oogoo00=oo`001Woo00<007ooOol00Woo00D007ooOomoo`0000=oo`03001oogoo
01Ioo`<0015oo`03001BoE;m00EBo@<000YBo@03001BoE;m00EBo@@000QBo@03001>oDkm00Q>o@03
001>gDkM009>g@H000Q:g@03001:gD[M00M:g@030016_DJm00I6_@030012_D:m00I2_@030012WD:M
00I2W@03000nOCim00LnO@03000jG3YL00DjG080008f?0@0008fG003001oogoo00Eoo`<000Eoo`03
001oogoo009oo`<000Ioo`0017oo0`001Goo0`0017oo00<007ooOol04goo0`005Goo00<005;mD_d0
10000e;m00<005;mD_d025;m00<005;mD_d010000e;m00<005;mD_d01e;m00<004kmC_d01Tkm1P00
1DkM00<004[MB]d01d[M00<004[MB]d01T[M00<004JmA[d01TJm00<004:m@[d01d:m00<004:M@Yd0
1T:M00<003im?Wd01Sim00<003YL>U`01CYL00<003YL00001CHl10000Woo100027oo00<007ooOol0
2goo000BOol00`00Oomoo`0BOol2000IOol20008D_d00`00D_eBo@07D_d30008D_d00`00D_eBo@07
D_d00`00C_e>o@050002C_d00`00C]e>g@07C]d00`00B]e:g@07B]d00`00B]e:g@06B]d00`00A[e6
_@06A[d00`00@[e2_@07@[d00`00@Ye2W@06@Yd00`00?WdnO@06?Wd00`00>U`jG005>U`20008=S`3
000<Ool00`00Oomoo`0;Ool0019oo`80011oo`<001aoo`03001BoE;m00MBo@03001BoE;m00=Bo@<0
009Bo@03001BoE;m00MBo@03001BoE;m00EBo@@000Q>o@03001>gDkM00M>g@03001:gD[M00M:g@03
001:gD[M009:gAd00092W@03000nOCim00LnO@03000jG3YL00DjG08000Pf?003001oogoo00aoo`03
001oogoo00Yoo`004Woo00<007ooOol037oo0`0087oo00<005;mD_d01e;m00<005;m00000P001U;m
00<005;mD_d01e;m00<005;mD_d010000e;m00<004kmC_d01dkm00<004kMC]d01dkM00<004[MB]d0
0d[M200014Zm00<004JmA[d01TJm00<004:M@Yd01d:M00<003jM?Yd00cjM2@000cim00<003YL>U`0
1SYL00<003Hl=S`01cHl00<007ooOol02goo00<007ooOol02Woo000BOol00`00Oomoo`09Ool3000T
Ool00`00D_eBo@05D_d3000:D_d00`00D_eBo@05D_d40008D_d00`00C_e>o@07C_d00`00C]e>g@05
C]d70004B]d00`00B[e:_@06B[d00`00A[e6_@06A[d00`00@Ye2W@07@Yd00`00?YdnW@05?Yd00`00
>W`jO003>W`80004>U`20008=S`00`00Oomoo`0;Ool00`00Oomoo`0:Ool0019oo`03001oogoo00Io
o`<002Qoo`03001BoE;m00@000=Bo@03001BoE;m00QBo@03001BoE;m00@000=Bo@03001BoE;m00MB
o@03001>oDkm00M>o@03001>gDkM00@0009>g@03001:gD[M00M:g@03001:_DZm00I:_@030016_DJm
00I6_@030012WD:M00M2W@03000nWCjM00DnW@03000jO3Yl00HjO004000fG3IL=U`60009=S`00`00
Oomoo`0:Ool00`00Oomoo`0:Ool0019oo`03001oogoo00=oo`<002aoo`8000QBo@03001BoE;m00MB
o@<000QBo@03001BoE;m00MBo@03001>oDkm00=>o@H000M>g@03001:gD[M00M:g@03001:_DZm00I:
_@030016_DJm00I6_@030012WD:M00I2W@03000nWCjM00HnW@03000jO3Yl00HjO003000fG3IL00Df
G0@000Hf?003001oogoo00Yoo`03001oogoo00Yoo`004Goo00@007ooOomoo`<0031oo`03001BoE;m
00MBo@03001BoE;m00=Bo@<0009Bo@03001BoE;m00MBo@03001BoE;m00EBo@L000A>o@03001>gDkM
00M>g@03001:gD[M00M:g@03001:_DZm00I:_@030016_DJm00I6_@030012WD:M00I2W@03000nWCjM
00DnW@03000jO3Yl00HjO003000fG3IL00HfG003000b?38l00@000<f?003001oogoo00Uoo`03001o
ogoo00Yoo`004Goo1000=7oo00<005;mD_d01e;m00<005;m00000P001U;m00<005;mD_d01e;m00<0
05;mD_d010000e;m00<004kmC_d01Tkm00<004kMC]d01dkM00<004[MB]d00d[M9@000SjM00<003Yl
>W`01SYl00<003IL=U`01SIL00<0038l<S`00c8l10002goo00<007ooOol02Woo00000goo00000003
0003Ool40004Ool2000gOol00`00D_eBo@05D_d40009D_d00`00D_eBo@05D_d40008D_d00`00C_e>
o@06C_d00`00C]e>g@05C]d70004B]d00`00A[e6_@06A[d00`00A[e6_@06A[d00`00@Ye2W@06@Yd0
0`00?WdnO@03?Wd70005>W`00`00=U`fG005=U`20007<S`2000;Ool00`00Oomoo`09Ool000=oo`03
001oogoo00Moo`03001oogoo00=oo`03001oogoo03Eoo`03001BoE;m00@000ABo@03001BoE;m00MB
o@03001BoE;m00@000=Bo@03001BoE;m00MBo@03001>oDkm00I>o@03001>gDkM00@0009>g@03001:
gD[M00M:g@030016_DJm00I6_@030016_DJm00I6_@030012WD:M00I2W@03000nOCim00DnO@04000j
G3YL>U`60008=U`20007<S`00`00Oomoo`09Ool00`00Oomoo`09Ool000=oo`03001oogoo00Moo`03
001oogoo00Aoo`03001oogoo03Eoo`8000UBo@03001BoE;m00EBo@@000QBo@03001BoE;m00MBo@03
001>oDkm009>o@H000M>g@03001:gD[M00M:g@030016_DJm00I6_@030016_DJm00I6_@030012WD:M
00I2W@03000nOCim00DnO@03000jG3YL00DjG0@000DfG08000Pb?003001oogoo00Qoo`03001oogoo
00Uoo`000goo00<007ooOol01goo00<007ooOol01Goo00<007ooOol0=Goo00<005;mD_d025;m00<0
05;mD_d010000e;m00<005;mD_d01e;m00<005;mD_d01E;m1P0014kM00<004kMC]d01dkM00<004[M
B]d01T[M00<004JmA[d01dJm00<004JmA[d01DJm00<004:M@Yd01T:M00<003im?Wd01Cim00<003YL
>U`01SYL00<003Hl=S`00`000cIL0P001c8l00<007ooOol027oo00<007ooOol02Goo0003Ool00`00
Oomoo`03Ool40009Ool2000fOol00`00D_eBo@07D_d30008D_d00`00D_eBo@07D_d00`00D_eBo@04
0002C_d00`00C]e>g@06C]d00`00C]e>g@07C]d00`00B]e:g@06B]d00`00A[e6_@03A[d:0002A[d0
0`00@Ye2W@06@Yd00`00?WdnO@05?Wd00`00>U`jG006>U`00`00=S`f?002=S`400000c8l000b?006
<S`00`00Oomoo`07Ool00`00Oomoo`09Ool000=oo`03001oogoo00=oo`03001oogoo00aoo`03001o
ogoo03Aoo`03001BoE;m00=Bo@<0009Bo@03001BoE;m00MBo@03001BoE;m00EBo@@000M>o@03001>
gDkM00I>g@03001>gDkM00M>g@03001:gD[M009:g@T000A6_@030012WD:M00=2W@l000DnO@03000j
G3YL00DjG003000f?3Hl00Df?0@000Hb?003001oogoo00Moo`03001oogoo00Uoo`000goo00<007oo
Ool00goo00<007ooOol03Goo00<007ooOol0=7oo00<005;m00000P001U;m00<005;mD_d01e;m00<0
05;mD_d010000U;m00<004kmC_d01dkm00<004kMC]d01TkM00<004kMC]d00dkM200014[M00<004Jm
A[d01dJm00<004:M@Yd01D:M00<003jM?Yd01SjM00<003Ym>Wd01P0023YL00<003Hl=S`01CHl00<0
02hK000010000c8l00<007ooOol01Woo00<007ooOol02Goo000017oo000000001Goo10003Goo00<0
07ooOol0=7oo0P002E;m00<005;mD_d01E;m10001e;m00<004kmC_d01dkm00<004kMC]d00TkM2@00
14[M00<004[MB]d01T[M00<004JmA[d01dJm00<004:M@Yd01D:M00<003jM?Yd01SjM00<003Ym>Wd0
13Ym1@001CYL00<003Hl=S`01CHl00<002hK00000bhK100027oo00<007ooOol02Goo000KOol2000f
Ool00`00D_eBo@07D_d00`00D_eBo@040002D_d00`00D_eBo@07D_d00`00C_e>o@05C_d60005C]d0
0`00B]e:g@06B]d00`00B]e:g@06B]d00`00A[e6_@06A[d00`00@Ye2W@06@Yd00`00?YdnW@06?Yd0
0`00>WdjO@04>Wd01000=U`fG3IL1P001cHl00@002hK;Q/000D^6`8000Qoo`03001oogoo00Qoo`00
7Goo00<007ooOol0=7oo00<005;mD_d01E;m10001e;m00<005;mD_d01e;m00<004kmC_d010000dkm
00<004kMC]d01TkM00<004[MB]d01T[M00<004[MB]d01T[M00<004JmA[d01TJm00<004:M@Yd01T:M
00<003jM?Yd01CjM00<003Ym>Wd01CYm00<003IL=U`01CIL10001CHl00<002hK00001RhK00<007oo
Ool01Woo00<007ooOol027oo000NOol00`00Oomoo`0dOol00`00D_eBo@040003D_d00`00D_eBo@06
D_d00`00D_eBo@05D_d40008C_d00`00C]e>g@06C]d00`00B]e:g@06B]d00`00B]e:g@02B]dL0002
?Yd00`00>WdjO@05>Wd00`00=U`fG005=U`00`00<S`b?0030003=S`00`00;Q/00006;Q/00`00Oomo
o`05Ool00`00Oomoo`08Ool001moo`03001oogoo03Aoo`8000QBo@03001BoE;m00IBo@03001BoE;m
00@000ABo@03001>oDkm00I>o@03001>gDkM00I>g@03001:gD[M009:g@P000E:_@030016_DJm00E6
_@030012WD:M00I2W@03000nOCim00<nO@H000@jO@03000fG3IL00HfG003000b?38l008b?0@0008^
6`03000^6bhK00<^6`03001oogoo00Eoo`03001oogoo00Qoo`0087oo0P00=Goo00<005;mD_d01e;m
00<005;mD_d015;m10002E;m00<004kmC_d01Tkm00<004kMC]d00TkM200014[M00<004ZmB[d01dZm
00<004JmA[d01DJm00<004:M@Yd01T:M00<003im?Wd01Cim00<003YL>U`01P001cIL00<0038l<S`0
1C8l0`0000<^6`00;Q/012hK00<007ooOol017oo00<007ooOol027oo000ROol00`00Oomoo`0cOol0
0`00D_eBo@07D_d00`00D_eBo@030004D_d00`00D_eBo@07D_d00`00C_e>o@02C_d80005C]d00`00
B]e:g@05B]d00`00B[e:_@07B[d00`00A[e6_@05A[d00`00@Ye2W@06@Yd00`00?WdnO@05?Wd00`00
>U`jG004>U`40005=U`00`00<S`b?005<S`00`00;Q/000020004;Q/00`00Oomoo`04Ool00`00Oomo
o`08Ool002=oo`03001oogoo03=oo`03001BoE;m00EBo@@000QBo@03001BoE;m00EBo@H000A>o@03
001>gDkM00M>g@03001:gD[M00E:g@03001:_DZm00M:_@030016_DJm00E6_@030012WD:M00E2W@03
000nOCim00DnO@03000jG3YL00DjG003000b?38l00<000<fG003000b?38l00Db?003000^6bhK00@0
008^6`03001oogoo00=oo`03001oogoo00Qoo`0097oo00<007ooOol0<goo00<005;mD_d010000e;m
00<005;mD_d01e;m00<005;mD_d010000U;m00<004kmC_d01Tkm00<004kMC]d01dkM00<004[MB]d0
1D[M00<004ZmB[d00dZm4P000T:M00<003im?Wd01Cim00<003YL>U`01CYL00<0038l<S`00S8l1000
1c8l00D002hK;Q/^6`00008^6`<000Eoo`03001oogoo00Qoo`009Goo0P00=7oo0P0025;m00<005;m
D_d01E;m10001e;m00<004kmC_d01Tkm00<004kMC]d01DkM3`004Woo1P001Cim00<003YL>U`013YL
00<0038l<S`01C8l0`001S8l00D002hK;Q/^6`0000@^6`03001oogoo00=oo`03001oogoo00Moo`00
9goo00<007ooOol0<goo00<005;mD_d01U;m00<005;mD_d010000U;m00<005;mD_d01e;m00<004km
C_d01Tkm00<004kMC]d010009goo1P001cYL00<0038l<S`01C8l00<002hL00000P000c8l00D002hK
;Q/^6`0000@^6`03001oogoo00=oo`03001oogoo00Moo`00:7oo00<007ooOol0<goo00<005;mD_d0
15;m10001e;m00<005;mD_d01e;m00<004kmC_d00Tkm1P00<7oo10001CYL00<0038l<S`01C8l00@0
02hL;Q`^7080008b?005000^6bhK;Q/00004;Q/00`00Oomoo`02Ool00`00Oomoo`07Ool002Uoo`03
001oogoo03=oo`03001BoE;m00<000ABo@03001BoE;m00EBo@03001BoE;m00EBo@H003Yoo`H000Lb
?003000^72hL00<^70<000@^6`05000^6bhK;Q/00004Ool00`00Oomoo`07Ool002Yoo`03001oogoo
03=oo`8000QBo@03001BoE;m00EBo@03001BoE;m00@004Eoo`@000Db?003000^72hL00D^7080008^
6`03000^6bhK008^6`05001oogooOol00009Ool002]oo`8003Aoo`03001BoE;m00MBo@03001BoE;m
009Bo@D004eoo`<000<b?003000^72hL00@^7006000Yn`00000^6`000bhK00D007ooOomoo`0000Uo
o`00;Goo00<007ooOol0<Woo00<005;mD_d01E;m1P00EGoo10001bhL00<002Wk:O/00`000RhK00D0
07ooOomoo`0000Uoo`00;Woo00<007ooOol0<Woo00<005;mD_d01000GWoo0`001BhL00<002Wk:O/0
0bWk0P0000<^6`00Ool00Woo00<007ooOol01Woo000_Ool00`00Oomoo`0bOol2001UOol30003;Q`0
0`00:O/Yn`02:O/01000:O/000000goo00<007ooOol01Woo000`Ool2000bOol00`00Oomoo`1WOol2
0002;Q`00`00:O/Yn`02:O/01000:O/Yn`000Woo00<007ooOol01Woo000bOol00`00Oomoo`0_Ool0
0`00Oomoo`1YOol30005:O/00`00:O/00002Ool00`00Oomoo`06Ool003=oo`03001oogoo02ioo`03
001oogoo06aoo`8000@Yn`05000Yn`00Ool00008Ool003Aoo`03001oogoo02eoo`03001oogoo06io
o`80008Yn`05000Yn`00Ool00008Ool003Eoo`8002eoo`03001oogoo071oo`800003:O/002Wk0080
00Qoo`00=goo00<007ooOol0:Woo00<007ooOol0LWoo00<002Wk00000P0027oo000hOol00`00Oomo
o`0YOol00`00Oomoo`1cOol200000bWk00000007Ool003Uoo`03001oogoo02Qoo`03001oogoo07Eo
o`<000Moo`00>Woo0P00:7oo00<007ooOol0MWoo0P001goo000lOol00`00Oomoo`0UOol00`00Oomo
o`1eOol30007Ool003eoo`03001oogoo02Aoo`03001oogoo075oo`@000Yoo`00?Woo00<007ooOol0
8goo00<007ooOol0KGoo10003Woo000oOol2000SOol00`00Oomoo`1ZOol3000BOol0045oo`03001o
ogoo021oo`03001oogoo06Ioo`@001Eoo`00@Woo00<007ooOol07goo00<007ooOol0HWoo10006Goo
0013Ool00`00Oomoo`0NOol00`00Oomoo`1NOol4000MOol004Aoo`8001ioo`03001oogoo05Yoo`@0
025oo`00AWoo00<007ooOol06goo00<007ooOol0EWoo10009Goo0017Ool00`00Oomoo`0JOol00`00
Oomoo`1BOol4000YOol004Qoo`03001oogoo01Uoo`03001oogoo04moo`<002eoo`00BGoo0P0067oo
00<007ooOol0C7oo1000<7oo001;Ool00`00Oomoo`0EOol00`00Oomoo`18Ool4000dOol004aoo`03
001oogoo01Aoo`03001oogoo04Aoo`@003Qoo`00CGoo00<007ooOol04goo00<007ooOol0@7oo1000
?7oo001>Ool2000COol00`00Oomoo`0lOol40010Ool0051oo`03001oogoo011oo`03001oogoo03Uo
o`<004Aoo`00DGoo00<007ooOol03goo00<007ooOol0=Goo1000Agoo001BOol00`00Oomoo`0>Ool0
0`00Oomoo`0aOol4001;Ool005=oo`03001oogoo00eoo`03001oogoo02eoo`@004moo`00E7oo0P00
3Goo00<007ooOol0:Goo1000Dgoo001FOol00`00Oomoo`0:Ool00`00Oomoo`0UOol4001GOol005Mo
o`03001oogoo00Uoo`03001oogoo029oo`<005]oo`00F7oo00<007ooOol027oo00<007ooOol07Woo
1000GWoo001IOol20008Ool00`00Oomoo`0JOol4001ROol005]oo`03001oogoo00Eoo`03001oogoo
01Ioo`@006Ioo`00G7oo00<007ooOol017oo00<007ooOol04Woo1000JWoo001MOol00`00Oomoo`03
Ool00`00Oomoo`0>Ool4001^Ool005ioo`8000=oo`03001oogoo00Yoo`@0079oo`00H7oo00@007oo
Ool000Uoo`<007Ioo`00HGoo00<007oo00001Goo1000NGoo001ROol200000goo00000002001mOol0
06=oo`80085oo`00iWoo003VOol00>Ioo`00iWoo003VOol00001\
\>"],
  ImageRangeCache->{{{0, 229.313}, {185.563, 0}} -> {-0.0807146, 0.000240802, 
  0.00579866, 0.00579866}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.6 The Greeks for a Power Contract", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Let's compute some symbolic forms for the value and Greeks of a \
power contract. Note that we are using the same symbols as for the log case. \
You should check for yourself that the identities relating the Greeks are \
satisfied.\
\>", "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
V = PowerContract[S,K,r,q,\[Sigma],t,\[Lambda]];
\[CapitalDelta] = PowerContractDelta[S_, K_, r_, q_, \[Sigma]_, t_, \
\[Lambda]_] = 
Simplify[D[V, S]]; TraditionalForm[\[CapitalDelta]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(\[ExponentialE]\^\(1\/2\ t\ 
            \((2\ r\ \((\[Lambda] - 1)\) + 
                \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))
              \)\)\ \((S\/K)\)\^\[Lambda]\ \[Lambda]\)\/S\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalGamma] = PowerContractGamma[S_, K_, r_, q_, \[Sigma]_, t_, \
\[Lambda]_] = 
Simplify[D[V, {S,2}]]; TraditionalForm[\[CapitalGamma]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(\[ExponentialE]\^\(1\/2\ t\ 
            \((2\ r\ \((\[Lambda] - 1)\) + 
                \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))
              \)\)\ \((S\/K)\)\^\[Lambda]\ \((\[Lambda] - 1)\)\ \[Lambda]\)\/S
        \^2\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Theta] = PowerContractTheta[S_, K_, r_, q_, \[Sigma]_, t_, \
\[Lambda]_] = 
-Simplify[D[V, t]]; TraditionalForm[\[Theta]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\(-\(1\/2\)\)\ 
      \[ExponentialE]\^\(1\/2\ t\ 
          \((2\ r\ \((\[Lambda] - 1)\) + 
              \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))\)\)\ 
      \((S\/K)\)\^\[Lambda]\ 
      \((2\ r\ \((\[Lambda] - 1)\) + 
          \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))\)\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[Rho] = PowerContractRho[S_, K_, r_, q_, \[Sigma]_, t_, \
\[Lambda]_] = 
Simplify[D[V, r]]; TraditionalForm[\[Rho]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[ExponentialE]\^\(1\/2\ t\ 
          \((2\ r\ \((\[Lambda] - 1)\) + 
              \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))\)\)\ 
      \((S\/K)\)\^\[Lambda]\ t\ \((\[Lambda] - 1)\)\)], "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell["\<\
\[CapitalLambda] = PowerContractVega[S_, K_, r_, q_, \[Sigma]_, t_, \
\[Lambda]_] = 
Simplify[D[V, \[Sigma]]]; TraditionalForm[\[CapitalLambda]]\
\>", "Input",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm
    \`\[ExponentialE]\^\(1\/2\ t\ 
          \((2\ r\ \((\[Lambda] - 1)\) + 
              \[Lambda]\ \((\((\[Lambda] - 1)\)\ \[Sigma]\^2 - 2\ q)\))\)\)\ 
      \((S\/K)\)\^\[Lambda]\ t\ \((\[Lambda] - 1)\)\ \[Lambda]\ \[Sigma]\)], 
  "NumberedEquation",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["5.7 Power Project", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The behaviour of the Greeks, and the hedge parameters in particular, and \
of the implied volatility, for power contracts exhibits all kinds of \
weirdness. It is left to the reader to explore the various cases. You should \
consider separately the cases:\n\n(a) ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  < \ 0\)]],
  ";\n(b) ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = \ 0\)]],
  ";\n(c) ",
  Cell[BoxData[
      \(TraditionalForm\`0 < \ \[Lambda]\  < \ 1\)]],
  ";\n(d) ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  = \ 1\)]],
  ";\n(e) ",
  Cell[BoxData[
      \(TraditionalForm\`\[Lambda]\  > \ 1\)]],
  ".\n\nIn particular, find a situation where the implied volatility has \
infinitely many values!"
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 5 Bibliography", "Section",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\nBoussard, E., 1997, ",
  StyleBox["Trading and Hedging Volatility",
    FontSlant->"Italic"],
  ", Presentation at Derivatives '97, Risk Conference, Brussels, February \
1997.\n\nNeuberger, A., 1990, The log contract and other power contracts, \
Chapter 7 in ",
  StyleBox[
  "The Handbook of Exotic Options: Instruments, Analysis and Applications",
    FontSlant->"Italic"],
  ", ed. I. Nelken, Irwin."
}], "Text",
  CellMargins->{{6.0625, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{703, 750},
WindowMargins->{{16, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingStartingPageNumber->127,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"5. Log and Power Contracts"}]]}, {Cell[ 
        TextData[ {"5. Log and Power Contracts"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1.25,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0b85N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@S=fH0000000000074/01T1T00000000000
00000000007oo@S=fD3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 217, 8, 167, "Title"],
Cell[1951, 61, 118, 1, 56, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2094, 66, 145, 3, 61, "Section"],
Cell[2242, 71, 1766, 29, 377, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4045, 105, 101, 1, 61, "Section"],
Cell[4149, 108, 338, 8, 53, "Text"],
Cell[4490, 118, 244, 4, 48, "NumberedEquation"],
Cell[4737, 124, 179, 6, 26, "Text"],
Cell[4919, 132, 146, 2, 27, "NumberedEquation"],
Cell[5068, 136, 691, 14, 91, "Text"],
Cell[5762, 152, 181, 4, 34, "Input"],

Cell[CellGroupData[{
Cell[5968, 160, 243, 4, 23, "Input"],
Cell[6214, 166, 49179, 1234, 196, 24304, 922, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[55442, 1406, 114, 1, 54, "Section"],
Cell[55559, 1409, 504, 9, 75, "Text"],
Cell[56066, 1420, 234, 5, 46, "Input"],
Cell[56303, 1427, 159, 3, 45, "NumberedEquation"],
Cell[56465, 1432, 203, 5, 34, "Input"],
Cell[56671, 1439, 171, 3, 45, "NumberedEquation"],
Cell[56845, 1444, 194, 4, 34, "Input"],
Cell[57042, 1450, 262, 6, 43, "NumberedEquation"],
Cell[57307, 1458, 187, 4, 34, "Input"],
Cell[57497, 1464, 262, 5, 43, "NumberedEquation"],
Cell[57762, 1471, 217, 5, 34, "Input"],
Cell[57982, 1478, 174, 3, 28, "NumberedEquation"],

Cell[CellGroupData[{
Cell[58181, 1485, 120, 1, 36, "Subsubsection"],
Cell[58304, 1488, 230, 5, 42, "Text"],

Cell[CellGroupData[{
Cell[58559, 1497, 220, 4, 23, "Input"],
Cell[58782, 1503, 97, 2, 25, "Output"]
}, Open  ]],
Cell[58894, 1508, 170, 4, 26, "Text"],

Cell[CellGroupData[{
Cell[59089, 1516, 224, 3, 23, "Input"],
Cell[59316, 1521, 102, 2, 25, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[59467, 1529, 148, 4, 43, "Subsection"],
Cell[59618, 1535, 303, 6, 42, "Text"],

Cell[CellGroupData[{
Cell[59946, 1545, 127, 2, 23, "Input"],
Cell[60076, 1549, 146, 2, 45, "Output"]
}, Open  ]],
Cell[60237, 1554, 356, 8, 42, "Text"],
Cell[60596, 1564, 191, 4, 27, "NumberedEquation"],
Cell[60790, 1570, 367, 8, 42, "Text"],
Cell[61160, 1580, 174, 3, 29, "NumberedEquation"],
Cell[61337, 1585, 109, 1, 26, "Text"],
Cell[61449, 1588, 226, 5, 46, "NumberedEquation"],
Cell[61678, 1595, 127, 1, 26, "Text"],
Cell[61808, 1598, 319, 7, 46, "NumberedEquation"],
Cell[62130, 1607, 851, 16, 172, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[63030, 1629, 122, 1, 54, "Section"],
Cell[63155, 1632, 438, 9, 58, "Text"],
Cell[63596, 1643, 236, 5, 34, "Input"],
Cell[63835, 1650, 240, 7, 27, "Text"],

Cell[CellGroupData[{
Cell[64100, 1661, 144, 2, 23, "Input"],
Cell[64247, 1665, 191, 3, 74, "Output"]
}, Open  ]],
Cell[64453, 1671, 576, 15, 58, "Text"],

Cell[CellGroupData[{
Cell[65054, 1690, 136, 2, 23, "Input"],
Cell[65193, 1694, 156, 3, 25, "Output"]
}, Open  ]],
Cell[65364, 1700, 196, 4, 26, "Text"],

Cell[CellGroupData[{
Cell[65585, 1708, 193, 4, 23, "Input"],
Cell[65781, 1714, 36, 1, 25, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[65854, 1720, 197, 4, 23, "Input"],
Cell[66054, 1726, 57, 1, 25, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66148, 1732, 193, 4, 23, "Input"],
Cell[66344, 1738, 56, 1, 25, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66437, 1744, 198, 4, 23, "Input"],
Cell[66638, 1750, 56, 1, 25, "Output"]
}, Open  ]],
Cell[66709, 1754, 305, 6, 42, "Text"],
Cell[67017, 1762, 247, 5, 23, "Input"],
Cell[67267, 1769, 101, 1, 26, "Text"],

Cell[CellGroupData[{
Cell[67393, 1774, 126, 2, 23, "Input"],
Cell[67522, 1778, 55, 1, 25, "Output"]
}, Open  ]],
Cell[67592, 1782, 176, 4, 26, "Text"],

Cell[CellGroupData[{
Cell[67793, 1790, 256, 5, 37, "Input"],
Cell[68052, 1797, 11700, 386, 150, 4035, 287, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[79767, 2186, 644, 17, 75, "Text"],

Cell[CellGroupData[{
Cell[80436, 2207, 290, 6, 37, "Input"],
Cell[80729, 2215, 601, 11, 74, "Output"]
}, Open  ]],
Cell[81345, 2229, 400, 7, 58, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[81782, 2241, 103, 1, 54, "Section"],
Cell[81888, 2244, 127, 1, 26, "Text"],
Cell[82018, 2247, 329, 7, 50, "NumberedEquation"],
Cell[82350, 2256, 207, 6, 26, "Text"],
Cell[82560, 2264, 151, 2, 44, "NumberedEquation"],
Cell[82714, 2268, 280, 9, 26, "Text"],
Cell[82997, 2279, 243, 5, 34, "Input"],
Cell[83243, 2286, 209, 4, 42, "Text"],

Cell[CellGroupData[{
Cell[83477, 2294, 136, 2, 23, "Input"],
Cell[83616, 2298, 50, 1, 27, "Output"]
}, Open  ]],
Cell[83681, 2302, 151, 4, 26, "Text"],

Cell[CellGroupData[{
Cell[83857, 2310, 136, 2, 23, "Input"],
Cell[83996, 2314, 60, 1, 45, "Output"]
}, Open  ]],
Cell[84071, 2318, 126, 1, 26, "Text"],

Cell[CellGroupData[{
Cell[84222, 2323, 255, 5, 37, "Input"],
Cell[84480, 2330, 49187, 1264, 196, 24791, 958, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[133682, 3597, 395, 8, 58, "Text"],

Cell[CellGroupData[{
Cell[134102, 3609, 248, 4, 37, "Input"],
Cell[134353, 3615, 54255, 1275, 196, 24077, 898, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[188657, 4896, 116, 1, 54, "Section"],
Cell[188776, 4899, 317, 6, 58, "Text"],
Cell[189096, 4907, 272, 6, 46, "Input"],
Cell[189371, 4915, 337, 7, 57, "NumberedEquation"],
Cell[189711, 4924, 227, 5, 34, "Input"],
Cell[189941, 4931, 367, 7, 57, "NumberedEquation"],
Cell[190311, 4940, 210, 5, 34, "Input"],
Cell[190524, 4947, 439, 10, 44, "NumberedEquation"],
Cell[190966, 4959, 203, 5, 34, "Input"],
Cell[191172, 4966, 328, 6, 44, "NumberedEquation"],
Cell[191503, 4974, 231, 5, 34, "Input"],
Cell[191737, 4981, 352, 7, 44, "NumberedEquation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[192126, 4993, 98, 1, 54, "Section"],
Cell[192227, 4996, 836, 22, 188, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[193100, 5023, 103, 1, 54, "Section"],
Cell[193206, 5026, 498, 12, 107, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

