(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   1235009,      33079]*)
(*NotebookOutlinePosition[   1236920,      33136]*)
(*  CellTagsIndexPosition[   1236876,      33132]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 7. 
Vanilla European Calls and Puts\
\>", "Title",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["\<\
What Can We Learn from a Mathematica Version of Black-Scholes?\
\>",
   "Subsubtitle",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["7.1 Introduction", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nOver the last few years, several treatments of the Black-Scholes model \
(Black and Scholes, 1973) within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " have been made available. In 1990 Ross Miller published an article in The \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Journal, which he elaborated in a chapter of Varian's book ",
  StyleBox["Economic and Financial Modelling with Mathematica",
    FontSlant->"Italic"],
  " (Varian, 1993). This by now standard treatment is also available as part \
of the Finance Pack, from Wolfram Research. We are all acutely aware of the \
limitations of this model and of the need to produce generalizations. Several \
are now available in analytical form, but often we need to resort to \
approximate computation. However, we would be remiss not to review the basic \
treatment, since there is much to be learnt that is applicable more \
generally. The points that we would like you to appreciate from this chapter \
are the following:\n\n(1) the general power of an analytical treatment;\n\n\
(2) the use of 2D and 3D visualization;\n\n(3) the treatment of sensitivity \
by differentiation;\n\n(4) the use of ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " for inverse calculations (here the implied volatility);\n\n(5) the \
collection of useful functions into a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Package structure.\n\nWe shall assume the results of the Black-Scholes \
analysis, in the form given by Merton (1973), with continuous dividend yield, \
as was derived in Chapter 3 in a basic application of It\[OHat]'s Lemma and \
arbitrage.\n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.2 Basic Functions", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
The following functions are defined directly here. At the end of \
this chapter we shall see how to bundle them up into a structured Package. \
First we define some functions that give the intrinsic value of Call and Put \
options at a given price for the underlying stock:
\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
CallPayoff[price_,strike_] = Max[0,price-strike];
PutPayoff[price_,strike_] = Max[0,strike-price];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\

CallPayoff[80, 60]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(20\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

CallPayoff[60, 80]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(0\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

Plot[CallPayoff[x, 60], {x, 0, 100}];
\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.00952381 0.0147151 0.0147151 [
[.21429 .00222 -6 -9 ]
[.21429 .00222 6 0 ]
[.40476 .00222 -6 -9 ]
[.40476 .00222 6 0 ]
[.59524 .00222 -6 -9 ]
[.59524 .00222 6 0 ]
[.78571 .00222 -6 -9 ]
[.78571 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .16187 -12 -4.5 ]
[.01131 .16187 0 4.5 ]
[.01131 .30902 -12 -4.5 ]
[.01131 .30902 0 4.5 ]
[.01131 .45617 -12 -4.5 ]
[.01131 .45617 0 4.5 ]
[.01131 .60332 -12 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .01472 m
.21429 .02097 L
s
[(20)] .21429 .00222 0 1 Mshowa
.40476 .01472 m
.40476 .02097 L
s
[(40)] .40476 .00222 0 1 Mshowa
.59524 .01472 m
.59524 .02097 L
s
[(60)] .59524 .00222 0 1 Mshowa
.78571 .01472 m
.78571 .02097 L
s
[(80)] .78571 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.07143 .01472 m
.07143 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.16667 .01472 m
.16667 .01847 L
s
.2619 .01472 m
.2619 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.35714 .01472 m
.35714 .01847 L
s
.45238 .01472 m
.45238 .01847 L
s
.5 .01472 m
.5 .01847 L
s
.54762 .01472 m
.54762 .01847 L
s
.64286 .01472 m
.64286 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.7381 .01472 m
.7381 .01847 L
s
.83333 .01472 m
.83333 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.92857 .01472 m
.92857 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .16187 m
.03006 .16187 L
s
[(10)] .01131 .16187 1 0 Mshowa
.02381 .30902 m
.03006 .30902 L
s
[(20)] .01131 .30902 1 0 Mshowa
.02381 .45617 m
.03006 .45617 L
s
[(30)] .01131 .45617 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(40)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .01472 L
.10458 .01472 L
.14415 .01472 L
.18221 .01472 L
.22272 .01472 L
.26171 .01472 L
.30316 .01472 L
.34309 .01472 L
.3815 .01472 L
.42237 .01472 L
.46172 .01472 L
.49955 .01472 L
.53984 .01472 L
.55838 .01472 L
.57861 .01472 L
.58392 .01472 L
.58957 .01472 L
.59232 .01472 L
.59367 .01472 L
.59491 .01472 L
.59604 .01595 L
.59723 .01779 L
.59975 .02168 L
.61984 .05272 L
.65781 .11139 L
.69823 .17385 L
.73714 .23397 L
.7785 .29787 L
.81835 .35944 L
.85668 .41866 L
.89746 .48167 L
.93672 .54234 L
.97448 .60067 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol0035o
o`D000Aoo`<001]oo`<000Aoo`<001Yoo`<000Eoo`<001]oo`<000Eoo`<001Ioo`D000=oo`<000Eo
o`<000Eoo`00<Woo00@007ooOol000=oo`05001oogooOol0000KOol00`00Oomoo`02Ool01@00Oomo
ogoo000067oo00D007ooOomoo`0000=oo`05001oogooOol0000IOol01@00Oomoogoo00000goo00D0
07ooOomoo`0001Moo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo000017oo000c
Ool00`00Oomoo`03Ool01@00Oomoogoo000067oo1@000goo00D007ooOomoo`0001Qoo`05001oogoo
Ool00003Ool01@00Oomoogoo00006Goo00D007ooOomoo`0000=oo`05001oogooOol0000GOol00`00
Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`00=7oo00<007ooOol00Woo00D0
07ooOomoo`0001Qoo`04001oogoo0004Ool01@00Oomoogoo000067oo00D007ooOomoo`0000=oo`05
001oogooOol0000IOol01@00Oomoogoo00000goo00D007ooOomoo`0001Moo`03001oogoo009oo`05
001oogooOol00003Ool01@00Oomoogoo000017oo000eOol01@00Oomoogoo00000goo00<007ooOol0
5goo00<007oo000017oo00D007ooOomoo`0001Qoo`@000Aoo`05001oogooOol0000JOol30004Ool0
1@00Oomoogoo00005goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00004Ool0
03Eoo`05001oogooOol00003Ool00`00Oomoo`0GOol00`00Ool00004Ool01@00Oomoogoo000067oo
00<007ooOol01Goo00D007ooOomoo`0001Uoo`05001oogooOol00003Ool01@00Oomoogoo00005goo
00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00004Ool0035oo`05001oogooOol0
0003Ool01@00Oomoogoo00006Woo0P0017oo00D007ooOomoo`0001Uoo`03001oogoo00Aoo`05001o
ogooOol0000IOol01@00Oomoogoo00000goo00D007ooOomoo`0001Moo`03001oogoo009oo`05001o
ogooOol00003Ool01@00Oomoogoo000017oo000bOol30005Ool3000LOol00`00Oomoo`03Ool3000K
Ool30004Ool3000KOol30005Ool3000FOol30005Ool30005Ool30005Ool00=]oo`004goo00<007oo
Ool0aGoo000COol00`00Oomoo`35Ool001=oo`03001oogoo0<Eoo`003Wooa0002Goo000COol00`00
Oomoo`0ROol00`00Oomoo`0ROol00`00Oomoo`0ROol2000TOol00`00Oomoo`0ROol00`00Oomoo`0;
Ool001=oo`03001oogoo06eoo`03001oogoo05Eoo`004goo00<007ooOol0KWoo00<007ooOol0E7oo
000COol00`00Oomoo`1_Ool00`00Oomoo`1COol001=oo`03001oogoo06moo`03001oogoo05=oo`00
4goo00<007ooOol0L7oo00<007ooOol0DWoo000COol00`00Oomoo`1aOol00`00Oomoo`1AOol001=o
o`03001oogoo075oo`03001oogoo055oo`004goo00<007ooOol0LWoo00<007ooOol0D7oo000COol0
0`00Oomoo`1cOol00`00Oomoo`1?Ool001=oo`03001oogoo07=oo`03001oogoo04moo`004goo00<0
07ooOol0M7oo00<007ooOol0CWoo000COol00`00Oomoo`1eOol00`00Oomoo`1=Ool001=oo`03001o
ogoo07Eoo`03001oogoo04eoo`004goo00<007ooOol0MWoo00<007ooOol0C7oo000COol00`00Oomo
o`1gOol00`00Oomoo`1;Ool001=oo`03001oogoo07Moo`03001oogoo04]oo`004goo00<007ooOol0
N7oo00<007ooOol0BWoo000COol00`00Oomoo`1iOol00`00Oomoo`19Ool001=oo`03001oogoo07Uo
o`03001oogoo04Uoo`004goo00<007ooOol0NWoo00<007ooOol0B7oo000COol00`00Oomoo`1jOol0
0`00Oomoo`18Ool001=oo`03001oogoo07]oo`03001oogoo04Moo`004goo00<007ooOol0O7oo00<0
07ooOol0AWoo000COol00`00Oomoo`1lOol00`00Oomoo`16Ool000=oo`D000=oo`<000Eoo`03001o
ogoo07eoo`03001oogoo04Eoo`001Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo
07eoo`03001oogoo04Eoo`001Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo07io
o`03001oogoo04Aoo`001Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`80081oo`03001oogoo
04=oo`001Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo07moo`03001oogoo04=o
o`001Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo081oo`03001oogoo049oo`00
1Goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo081oo`03001oogoo049oo`000goo
0`001Goo0`001Goo00<007ooOol0PGoo00<007ooOol0@Goo000COol00`00Oomoo`22Ool00`00Oomo
o`10Ool001=oo`03001oogoo089oo`03001oogoo041oo`004goo00<007ooOol0Pgoo00<007ooOol0
?goo000COol00`00Oomoo`24Ool00`00Oomoo`0nOol001=oo`03001oogoo08Aoo`03001oogoo03io
o`004goo00<007ooOol0QGoo00<007ooOol0?Goo000COol00`00Oomoo`26Ool00`00Oomoo`0lOol0
01=oo`03001oogoo08Ioo`03001oogoo03aoo`004goo00<007ooOol0Qgoo00<007ooOol0>goo000C
Ool00`00Oomoo`28Ool00`00Oomoo`0jOol001=oo`03001oogoo08Qoo`03001oogoo03Yoo`004goo
00<007ooOol0RGoo00<007ooOol0>Goo000COol00`00Oomoo`2:Ool00`00Oomoo`0hOol001=oo`03
001oogoo08Yoo`03001oogoo03Qoo`004goo00<007ooOol0Rgoo00<007ooOol0=goo000COol00`00
Oomoo`2<Ool00`00Oomoo`0fOol001=oo`03001oogoo08aoo`03001oogoo03Ioo`004goo00<007oo
Ool0SGoo00<007ooOol0=Goo000COol00`00Oomoo`2>Ool00`00Oomoo`0dOol001=oo`03001oogoo
08ioo`03001oogoo03Aoo`004goo00<007ooOol0Sgoo00<007ooOol0<goo0002Ool50004Ool30005
Ool00`00Oomoo`2@Ool00`00Oomoo`0bOol000=oo`04001oogoo0003Ool01@00Oomoogoo000017oo
00<007ooOol0T7oo00<007ooOol0<Woo0004Ool00`00Oomoo`03Ool01@00Oomoogoo000017oo0P00
TWoo00<007ooOol0<Goo0005Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol0TWoo
00<007ooOol0<7oo0006Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol0TWoo00<0
07ooOol0<7oo0006Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol0Tgoo00<007oo
Ool0;goo0002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`03001oogoo09Aoo`03001o
ogoo02ioo`000goo0`001Goo0`001Goo00<007ooOol0U7oo00<007ooOol0;Woo000COol00`00Oomo
o`2EOol00`00Oomoo`0]Ool001=oo`03001oogoo09Ioo`03001oogoo02aoo`004goo00<007ooOol0
UWoo00<007ooOol0;7oo000COol00`00Oomoo`2GOol00`00Oomoo`0[Ool001=oo`03001oogoo09Qo
o`03001oogoo02Yoo`004goo00<007ooOol0V7oo00<007ooOol0:Woo000COol00`00Oomoo`2IOol0
0`00Oomoo`0YOol001=oo`03001oogoo09Uoo`03001oogoo02Uoo`004goo00<007ooOol0VWoo00<0
07ooOol0:7oo000COol00`00Oomoo`2KOol00`00Oomoo`0WOol001=oo`03001oogoo09]oo`03001o
ogoo02Moo`004goo00<007ooOol0W7oo00<007ooOol09Woo000COol00`00Oomoo`2LOol00`00Oomo
o`0VOol001=oo`03001oogoo09eoo`03001oogoo02Eoo`004goo00<007ooOol0WWoo00<007ooOol0
97oo000COol00`00Oomoo`2NOol00`00Oomoo`0TOol001=oo`03001oogoo09moo`03001oogoo02=o
o`004goo00<007ooOol0Wgoo00<007ooOol08goo000COol00`00Oomoo`2POol00`00Oomoo`0ROol0
01=oo`03001oogoo0:5oo`03001oogoo025oo`004goo00<007ooOol0XGoo00<007ooOol08Goo0003
Ool30005Ool30005Ool00`00Oomoo`2ROol00`00Oomoo`0POol0009oo`05001oogooOol00003Ool0
1@00Oomoogoo000017oo00<007ooOol0Xgoo00<007ooOol07goo0006Ool01@00Oomoogoo00000goo
00<007ooOol00Woo0P00Y7oo00<007ooOol07goo0006Ool01@00Oomoogoo00000goo00<007ooOol0
0Woo00<007ooOol0Y7oo00<007ooOol07Woo0004Ool20004Ool01@00Oomoogoo000017oo00<007oo
Ool0YGoo00<007ooOol07Goo0006Ool01@00Oomoogoo00000goo00<007ooOol00Woo00<007ooOol0
YGoo00<007ooOol07Goo0002Ool01@00Oomoogoo00000goo00D007ooOomoo`0000Aoo`03001oogoo
0:Ioo`03001oogoo01aoo`000goo0`001Goo0`001Goo00<007ooOol0Ygoo00<007ooOol06goo000C
Ool00`00Oomoo`2WOol00`00Oomoo`0KOol001=oo`03001oogoo0:Qoo`03001oogoo01Yoo`004goo
00<007ooOol0ZGoo00<007ooOol06Goo000COol00`00Oomoo`2YOol00`00Oomoo`0IOol001=oo`03
001oogoo0:Yoo`03001oogoo01Qoo`004goo00<007ooOol0Zgoo00<007ooOol05goo000COol00`00
Oomoo`2[Ool00`00Oomoo`0GOol001=oo`03001oogoo0:aoo`03001oogoo01Ioo`004goo00<007oo
Ool0[Goo00<007ooOol05Goo000COol00`00Oomoo`2]Ool00`00Oomoo`0EOol001=oo`03001oogoo
0:ioo`03001oogoo01Aoo`004goo00<007ooOol0[goo00<007ooOol04goo000COol00`00Oomoo`2_
Ool00`00Oomoo`0COol001=oo`03001oogoo0;1oo`03001oogoo019oo`004goo00<007ooOol0/Goo
00<007ooOol04Goo000COol00`00Oomoo`2aOol00`00Oomoo`0AOol001=oo`03001oogoo0;9oo`03
001oogoo011oo`004goo00<007ooOol0/goo00<007ooOol03goo000COol00`00Oomoo`2cOol00`00
Oomoo`0?Ool001=oo`03001oogoo0;Aoo`03001oogoo00ioo`0017oo0`0017oo0`001Goo00<007oo
Ool0]7oo00<007ooOol03Woo0005Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol0
]Goo00<007ooOol03Goo0002Ool50003Ool01@00Oomoogoo000017oo00<007ooOol0]Woo00<007oo
Ool037oo0002Ool01000Oomoo`0017oo00D007ooOomoo`0000Aoo`800;Qoo`03001oogoo00]oo`00
0goo00<007oo000017oo00D007ooOomoo`0000Aoo`03001oogoo0<Eoo`000goo00<007oo000017oo
00D007ooOomoo`0000Aoo`03001oogoo0<Eoo`0017oo0P0017oo00D007ooOomoo`0000Aoo`03001o
ogoo0<Eoo`001Goo00<007ooOol00goo0`00cGoo003KOol00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {-10.2156, -4.75839, 
  0.671091, 0.434339}}]
}, Open  ]],

Cell["\<\

Next we define the fundamental normal distribution functions that we shall \
need, in both numerical and symbolic form:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Norm[(z_)?NumberQ] := N[0.5*Erf[z/Sqrt[2]] + 0.5];
Norm[x_] := (1 + Erf[x/Sqrt[2]])/2\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\

Using the normal distribution, we can define the Black-Scholes formula. We do \
it in two stages:
\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
done[s_, \[Sigma]_, k_, t_, r_, q_] := 
((r - q)*t + Log[s/k])/(\[Sigma]*Sqrt[t]) + (\[Sigma]*Sqrt[t])/2; 
dtwo[s_, \[Sigma]_, k_, t_, r_, q_] := 
((r - q)*t + Log[s/k])/(\[Sigma]*Sqrt[t]) - (\[Sigma]*Sqrt[t])/2; \
\>", 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
BlackScholesCall[s_, k_, v_, r_, q_, t_] := 
s*Exp[-q*t]*Norm[done[s, v, k, t, r, q]] - k*Exp[-r*t]*Norm[dtwo[s, v, k, t, \
r, q]]; 
BlackScholesPut[s_, k_, v_, r_, q_, t_] := 
k*Exp[-r*t]*Norm[-dtwo[s, v, k, t, r, q]] - s*Exp[-q*t]*Norm[-done[s, v, k, \
t, r, q]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We can now use some of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s tools to explore the properties of this function. In particular, we can \
use the graphical tools in several ways. First we can look at the payoff at \
other times:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[BlackScholesCall[x, 60., 0.29, 0.04, 0, 0.5], {x, 50, 70}, \
PlotRange -> {{50, 70}, {0, 15}}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .05572 m
.04057 .06435 L
.08481 .07474 L
.12636 .08546 L
.16632 .09666 L
.20885 .10957 L
.2498 .12297 L
.29331 .13825 L
.33524 .15399 L
.37557 .17007 L
.41848 .18817 L
.4598 .20655 L
.49953 .22508 L
.54183 .24573 L
.58254 .26646 L
.62583 .2894 L
.66752 .31233 L
.70762 .33513 L
.7503 .36016 L
.79139 .38497 L
.83505 .41206 L
.87712 .43884 L
.9176 .4652 L
.96065 .49382 L
1 .52048 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`<00<Moo`004Goo0P0000=oo`0000000P00`goo000AOol00`00Oomoo`04Ool4002o
Ool000Qoo`D000Aoo`03001oogoo00Qoo`@00;]oo`002Goo00@007ooOol000Aoo`03001oogoo00ao
o`@00;Moo`002Woo00<007ooOol017oo0P004Goo1000/goo000;Ool00`00Oomoo`03Ool00`00Oomo
o`0DOol4002_Ool000aoo`03001oogoo009oo`03001oogoo01Qoo`@00:]oo`0037oo00<007ooOol0
0Woo00<007ooOol077oo0`00Z7oo0008Ool01@00Oomoogoo000017oo0P0087oo0`00YGoo0009Ool3
0005Ool00`00Oomoo`0ROol3002ROol0015oo`03001oogoo02Eoo`<009moo`004Goo00<007ooOol0
:7oo1000Vgoo000AOol2000]Ool4002GOol0015oo`03001oogoo031oo`<009Aoo`004Goo00<007oo
Ool0<goo0`00TGoo000AOol00`00Oomoo`0fOol2002?Ool0015oo`8003Uoo`<008aoo`004Goo00<0
07ooOol0>goo0`00RGoo000:Ool30004Ool00`00Oomoo`0nOol20027Ool000]oo`03001oogoo00=o
o`03001oogoo041oo`<008Aoo`0027oo1@0017oo0P00A7oo0P00PWoo0008Ool01000Oomoo`001Goo
00<007ooOol0AGoo0P00P7oo0009Ool00`00Ool00005Ool00`00Oomoo`17Ool2001nOol000Uoo`03
001oo`0000Eoo`03001oogoo04Uoo`<007]oo`002Woo0P001Goo0P00CGoo0P00NGoo000;Ool00`00
Oomoo`03Ool00`00Oomoo`1>Ool2001gOol0015oo`03001oogoo051oo`<007Aoo`004Goo00<007oo
Ool0Dgoo0`00LGoo000AOol2001GOol2001_Ool0015oo`03001oogoo05Qoo`8006eoo`004Goo00<0
07ooOol0FWoo00<007ooOol0JWoo000AOol00`00Oomoo`1KOol2001ZOol0015oo`8005ioo`8006Qo
o`004Goo00<007ooOol0Ggoo0P00IWoo0009Ool30005Ool00`00Oomoo`1QOol3001SOol000Qoo`05
001oogooOol00004Ool00`00Oomoo`1TOol2001QOol000Qoo`05001oogooOol00004Ool2001WOol2
001OOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1XOol2001MOol000Qoo`@000Eoo`03001o
ogoo06Yoo`03001oogoo05Yoo`0027oo00<007ooOol01Woo00<007ooOol0Jgoo0P00FWoo0009Ool0
0`00Oomoo`05Ool2001^Ool2001HOol000Yoo`<000Aoo`03001oogoo06moo`8005Ioo`004Goo00<0
07ooOol0LGoo0`00Dgoo000AOol00`00Oomoo`1dOol2001AOol0015oo`8007Moo`8004moo`004Goo
00<007ooOol0N7oo00<007ooOol0C7oo000AOol00`00Oomoo`1iOol2001<Ool0015oo`03001oogoo
07]oo`8004Yoo`004Goo0P00OWoo00<007ooOol0Agoo000AOol00`00Oomoo`1nOol20017Ool000Uo
o`<000Eoo`03001oogoo081oo`8004Eoo`0027oo00D007ooOomoo`0000Aoo`03001oogoo089oo`80
04=oo`0027oo00D007ooOomoo`0000Aoo`8008Eoo`80045oo`0027oo00D007ooOomoo`0000Aoo`03
001oogoo08Ioo`8003moo`002Goo0`001Goo00<007ooOol0R7oo00<007ooOol0?7oo0008Ool01@00
Oomoogoo000017oo00<007ooOol0RGoo0P00?7oo0008Ool01@00Oomoogoo000017oo0P00S7oo0P00
>Woo0009Ool30005Ool00`00Oomoo`2=Ool00`00Oomoo`0gOol0015oo`03001oogoo08ioo`8003Mo
o`004Goo00<007ooOol0T7oo00<007ooOol0=7oo000AOol2002BOol2000dOol0015oo`03001oogoo
09=oo`80039oo`004Goo00<007ooOol0UGoo00<007ooOol0;goo000AOol00`00Oomoo`2FOol2000_
Ool0015oo`8009Uoo`03001oogoo02aoo`004Goo00<007ooOol0VGoo0P00;7oo00000goo00000003
0003Ool30005Ool00`00Oomoo`2KOol2000ZOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`2MOol00`00Oomoo`0WOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2
002OOol2000WOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2POol00`00
Oomoo`0TOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2QOol2000TOol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2SOol2000ROol000=oo`03001o
ogoo009oo`05001oogooOol00004Ool2002VOol00`00Oomoo`0OOol00004Ool000000005Ool30005
Ool00`00Oomoo`2VOol2000OOol0015oo`03001oogoo0:Qoo`03001oogoo01aoo`004Goo00<007oo
Ool0ZGoo0P0077oo000AOol2002/Ool2000JOol0015oo`03001oogoo0:eoo`03001oogoo01Moo`00
4Goo00<007ooOol0[Woo0P005goo000AOol00`00Oomoo`2`Ool00`00Oomoo`0DOol0015oo`800;9o
o`8001Aoo`0000=oo`0000000`000Woo1@0017oo00<007ooOol0/goo00<007ooOol04Goo0003Ool0
0`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0]7oo0P004Goo0003Ool00`00Oomoo`04Ool0
0`00Oomoo`04Ool00`00Oomoo`2fOol00`00Oomoo`0>Ool000=oo`03001oogoo00Eoo`03001oogoo
00=oo`800;Qoo`8000ioo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0^Goo00<0
07ooOol02goo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`2jOol2000;Ool000=o
o`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2lOol00`00Oomoo`08Ool00004Ool0
00000005Ool30005Ool2002nOol00`00Oomoo`07Ool0015oo`03001oogoo0;ioo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo00000003
0004Ool30004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo
0<Moo`000goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000
Oomoo`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=o
o`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo
00<007ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37
Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}]
}, Open  ]],

Cell[TextData[{
  "\nWe can loop this plot over ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " to create an animation of the process. Note that only a single frame is \
shown in the printed version of this text. If you are using the electronic \
form select the group of cells containing the plot and animate it using the \
\"Animate Selected Graphics\" entry in the \"Cell\" menu."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Do[Plot[BlackScholesCall[x, 60., 0.29, 0.04, 0, t], {x, 50, 70}, \
PlotRange -> {{50, 70}, {0, 15}}], {t, 0.001, 1, 0.08}]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 0 m
.04057 0 L
.08481 0 L
.12636 0 L
.16632 0 L
.20885 0 L
.2498 0 L
.26052 0 L
.26617 0 L
.27217 0 L
.27469 0 L
.27608 0 L
.2774 0 L
.27814 0 L
s
.27814 0 m
.27889 0 L
s
.2855 0 m
.2859 0 L
.28716 0 L
s
.28716 0 m
.28827 0 L
s
.28862 0 m
.28968 0 L
s
.28968 0 m
.2905 0 L
s
.29236 0 m
.29331 0 L
.29459 0 L
.29532 0 L
.296 0 L
.29738 0 L
.29815 0 L
.29886 0 L
.30006 0 L
.30136 0 L
.30273 0 L
.304 0 L
.30551 0 L
.30625 0 L
.30694 0 L
.30822 0 L
.30894 0 L
.30961 0 L
.311 0 L
.31174 0 L
.31253 0 L
.31336 0 L
.31414 0 L
.31562 0 L
.31695 0 L
.31816 0 L
.31883 0 L
.31955 0 L
.32086 0 L
.32209 0 L
.32339 0 L
.32451 0 L
.32572 0 L
.32646 0 L
.32716 0 L
.32848 0 L
.32989 0 L
.33069 0 L
.33143 0 L
.33265 0 L
.33401 0 L
.33542 0 L
.33674 0 L
.33798 0 L
.33931 0 L
.34056 0 L
.34169 0 L
.34302 0 L
.34447 0 L
.34584 0 L
Mistroke
.34709 0 L
.3484 0 L
.34961 0 L
.35095 0 L
.35238 0 L
.35388 0 L
.35531 0 L
.35658 0 L
.35798 0 L
.35917 0 L
.3603 0 L
.36154 0 L
.36287 0 L
.36419 0 L
.36558 0 L
.36677 0 L
.36807 0 L
.3693 0 L
.37041 0 L
.37161 0 L
.37291 0 L
.37412 0 L
.37523 0 L
.37742 0 L
.37873 0 L
.37998 0 L
.38232 0 L
.38362 0 L
.38483 0 L
.38625 0 L
.38758 0 L
.38874 0 L
.39 0 L
.39219 0 L
.39343 0 L
.39474 0 L
.3971 0 L
.39845 0 L
.39968 0 L
.401 0 L
.40244 0 L
.405 0 L
.40743 0 L
.40878 0 L
.41028 0 L
.41174 0 L
.41331 0 L
.41612 1e-05 L
.41876 1e-05 L
.42128 1e-05 L
Mistroke
.42397 2e-05 L
.42651 2e-05 L
.42882 3e-05 L
.43153 4e-05 L
.43448 6e-05 L
.43726 8e-05 L
.43982 .00011 L
.44247 .00014 L
.44494 .00018 L
.44766 .00024 L
.45058 .00032 L
.45364 .00042 L
.45654 .00054 L
.45934 .00068 L
.46198 .00085 L
.46728 .00129 L
.47208 .00183 L
.47725 .00259 L
.48283 .00368 L
.4929 .00645 L
.50215 .01002 L
.51241 .01515 L
.52197 .02094 L
.54369 .03666 L
.58462 .06984 L
.62813 .10568 L
.67004 .14022 L
.71036 .17345 L
.75326 .2088 L
.79456 .24283 L
.83428 .27556 L
.87657 .31041 L
.91726 .34394 L
.96054 .3796 L
1 .41212 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`P0009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo009oo`03001oogoo009oo`03
001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`06Ool0015oo`03001o
ogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Aoo`@001Ioo`03001oogoo01Eo
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Ioo`004Goo00<007ooOol0H7oo0P00IGoo
000AOol2001SOol2001SOol0015oo`03001oogoo06Aoo`03001oogoo061oo`004Goo00<007ooOol0
IGoo00<007ooOol0Ggoo000AOol00`00Oomoo`1VOol2001OOol0015oo`8006Uoo`03001oogoo05ao
o`004Goo00<007ooOol0JGoo00<007ooOol0Fgoo000AOol00`00Oomoo`1ZOol00`00Oomoo`1JOol0
015oo`03001oogoo06]oo`03001oogoo05Uoo`004Goo0P00KGoo00<007ooOol0F7oo000AOol00`00
Oomoo`1]Ool00`00Oomoo`1GOol000Qoo`D000Aoo`03001oogoo06ioo`8005Moo`002Goo00@007oo
Ool000Aoo`03001oogoo071oo`03001oogoo05Aoo`002Woo00<007ooOol017oo0P00LWoo00<007oo
Ool0Dgoo000;Ool00`00Oomoo`03Ool00`00Oomoo`1bOol00`00Oomoo`1BOol000aoo`03001oogoo
009oo`03001oogoo07=oo`03001oogoo055oo`0037oo00<007ooOol00Woo00<007ooOol0M7oo00<0
07ooOol0D7oo0008Ool01@00Oomoogoo000017oo0P00MWoo00<007ooOol0Cgoo0009Ool30005Ool0
0`00Oomoo`1fOol2001?Ool0015oo`03001oogoo07Qoo`03001oogoo04aoo`004Goo00<007ooOol0
NGoo00<007ooOol0Bgoo000AOol2001kOol2001;Ool0015oo`03001oogoo07aoo`03001oogoo04Qo
o`004Goo00<007ooOol0OGoo00<007ooOol0Agoo000AOol00`00Oomoo`1nOol20017Ool0015oo`80
085oo`03001oogoo04Aoo`004Goo00<007ooOol0PGoo00<007ooOol0@goo000:Ool30004Ool00`00
Oomoo`22Ool00`00Oomoo`12Ool000]oo`03001oogoo00=oo`03001oogoo08=oo`03001oogoo045o
o`0027oo1@0017oo0P00QGoo00<007ooOol0@7oo0008Ool01000Oomoo`001Goo00<007ooOol0QGoo
00<007ooOol0?goo0009Ool00`00Ool00005Ool00`00Oomoo`26Ool2000oOol000Uoo`03001oo`00
00Eoo`03001oogoo08Qoo`03001oogoo03aoo`002Woo0P001Goo0P00RWoo00<007ooOol0>goo000;
Ool00`00Oomoo`03Ool00`00Oomoo`2:Ool00`00Oomoo`0jOol0015oo`03001oogoo08]oo`03001o
ogoo03Uoo`004Goo00<007ooOol0S7oo00<007ooOol0>7oo000AOol2002>Ool00`00Oomoo`0gOol0
015oo`03001oogoo08ioo`8003Moo`004Goo00<007ooOol0T7oo00<007ooOol0=7oo000AOol00`00
Oomoo`2AOol00`00Oomoo`0cOol0015oo`8009=oo`8003=oo`004Goo00<007ooOol0U7oo00<007oo
Ool0<7oo0009Ool30005Ool00`00Oomoo`2EOol00`00Oomoo`0_Ool000Qoo`05001oogooOol00004
Ool00`00Oomoo`2FOol2000_Ool000Qoo`05001oogooOol00004Ool2002IOol00`00Oomoo`0/Ool0
00Qoo`05001oogooOol00004Ool00`00Oomoo`2IOol00`00Oomoo`0[Ool000Qoo`@000Eoo`03001o
ogoo09Yoo`03001oogoo02Yoo`0027oo00<007ooOol01Woo00<007ooOol0Vgoo00<007ooOol0:Goo
0009Ool00`00Oomoo`05Ool2002MOol00`00Oomoo`0XOol000Yoo`<000Aoo`03001oogoo09eoo`03
001oogoo02Moo`004Goo00<007ooOol0WWoo0P009goo000AOol00`00Oomoo`2POol00`00Oomoo`0T
Ool0015oo`800:9oo`03001oogoo02=oo`004Goo00<007ooOol0XWoo0P008goo000AOol00`00Oomo
o`2TOol00`00Oomoo`0POol0015oo`03001oogoo0:Eoo`03001oogoo01moo`004Goo0P00Ygoo0P00
7goo000AOol00`00Oomoo`2XOol00`00Oomoo`0LOol000Uoo`<000Eoo`03001oogoo0:Uoo`03001o
ogoo01]oo`0027oo00D007ooOomoo`0000Aoo`03001oogoo0:Yoo`03001oogoo01Yoo`0027oo00D0
07ooOomoo`0000Aoo`800:aoo`03001oogoo01Uoo`0027oo00D007ooOomoo`0000Aoo`03001oogoo
0:aoo`03001oogoo01Qoo`002Goo0`001Goo00<007ooOol0[Goo00<007ooOol05goo0008Ool01@00
Oomoogoo000017oo00<007ooOol0[Woo0P005goo0008Ool01@00Oomoogoo000017oo0P00/Goo00<0
07ooOol057oo0009Ool30005Ool00`00Oomoo`2aOol00`00Oomoo`0COol0015oo`03001oogoo0;9o
o`8001=oo`004Goo00<007ooOol0]7oo00<007ooOol047oo000AOol2002fOol00`00Oomoo`0?Ool0
015oo`03001oogoo0;Ioo`03001oogoo00ioo`004Goo00<007ooOol0]goo0P003Woo000AOol00`00
Oomoo`2iOol00`00Oomoo`0;Ool0015oo`800;]oo`03001oogoo00Yoo`004Goo00<007ooOol0^goo
00<007ooOol02Goo00000goo000000030003Ool30005Ool00`00Oomoo`2lOol00`00Oomoo`08Ool0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2mOol00`00Oomoo`07Ool000=o
o`03001oogoo009oo`05001oogooOol00004Ool2002oOol00`00Oomoo`06Ool000=oo`03001oogoo
009oo`05001oogooOol00004Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`37
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool20038Ool00004Ool000000005Ool30005
Ool00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol20038Ool0
015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qo
o`0000=oo`0000000`000Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`03Ool01000Oomo
o`0017oo00<007ooOol0agoo0003Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`37Ool0
00=oo`03001oogoo00Eoo`03001oogoo00=oo`800<Qoo`000goo00<007ooOol01Woo00<007ooOol0
0Woo00<007ooOol0agoo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`37Ool000=o
o`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`37Ool00004Ool000000005Ool30005
Ool20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0
015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Mo
o`004Goo0P00b7oo00000goo000000030004Ool30004Ool00`00Oomoo`37Ool000=oo`03001oogoo
00Eoo`03001oogoo00=oo`03001oogoo0<Moo`000goo00<007ooOol00Woo1@0017oo00<007ooOol0
agoo0003Ool00`00Oomoo`02Ool01000Oomoo`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00
Ool00005Ool00`00Oomoo`37Ool000=oo`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Mo
o`000goo00<007ooOol017oo0P001Goo00<007ooOol0agoo000017oo000000001goo00<007ooOol0
0goo0P00b7oo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo
000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .00099 m
.01975 .00129 L
.04057 .0017 L
.06168 .00222 L
.08481 .00294 L
.10708 .00381 L
.12734 .00478 L
.16829 .00735 L
.21021 .01105 L
.25055 .01586 L
.29346 .02259 L
.33477 .0309 L
.3745 .04076 L
.4168 .05345 L
.45751 .06788 L
.5008 .08566 L
.54249 .10512 L
.5826 .12592 L
.62527 .15014 L
.66636 .17532 L
.71002 .20385 L
.75209 .23283 L
.79257 .26189 L
.83562 .29385 L
.87708 .32545 L
.91696 .35645 L
.9594 .38997 L
1 .42241 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015ooah000Aoo`03
001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo009oo`03
001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02
Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo00<007oo
Ool00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo00Ioo`00
4Goo00<007ooOol05Goo00<007ooOol00goo20002Woo00<007ooOol05Goo00<007ooOol05Woo00<0
07ooOol05Goo00<007ooOol05Goo00<007ooOol05Goo00<007ooOol05Goo00<007ooOol01Woo000A
Ool00`00Oomoo`0SOol=002GOol0015oo`80035oo`D0099oo`004Goo00<007ooOol0=Goo1P00S7oo
000AOol00`00Oomoo`0kOol60026Ool0015oo`03001oogoo045oo`<008=oo`004Goo0P00AGoo0`00
P7oo000AOol00`00Oomoo`17Ool3001mOol0015oo`03001oogoo04Yoo`<007Yoo`004Goo00<007oo
Ool0CGoo0P00N7oo000AOol2001@Ool3001eOol0015oo`03001oogoo059oo`<0079oo`0027oo1@00
17oo00<007ooOol0EGoo0`00Kgoo0009Ool01000Oomoo`0017oo00<007ooOol0F7oo0`00K7oo000:
Ool00`00Oomoo`04Ool2001LOol3001YOol000]oo`03001oogoo00=oo`03001oogoo05ioo`8006Mo
o`0037oo00<007ooOol00Woo00<007ooOol0H7oo0P00IGoo000<Ool00`00Oomoo`02Ool00`00Oomo
o`1ROol2001SOol000Qoo`05001oogooOol00004Ool2001UOol2001QOol000Uoo`<000Eoo`03001o
ogoo06Ioo`8005moo`004Goo00<007ooOol0J7oo0P00GGoo000AOol00`00Oomoo`1ZOol00`00Oomo
o`1JOol0015oo`8006aoo`8005Yoo`004Goo00<007ooOol0KGoo0P00F7oo000AOol00`00Oomoo`1_
Ool2001FOol0015oo`03001oogoo075oo`<005=oo`004Goo0P00MGoo0P00DGoo000AOol00`00Oomo
o`1fOol2001?Ool000Yoo`<000Aoo`03001oogoo07Qoo`03001oogoo04aoo`002goo00<007ooOol0
0goo00<007ooOol0NGoo0P00C7oo0008Ool50004Ool2001lOol2001:Ool000Qoo`04001oogoo0005
Ool00`00Oomoo`1mOol00`00Oomoo`17Ool000Uoo`03001oo`0000Eoo`03001oogoo07ioo`8004Mo
o`002Goo00<007oo00001Goo00<007ooOol0P7oo00<007ooOol0A7oo000:Ool20005Ool20022Ool0
0`00Oomoo`13Ool000]oo`03001oogoo00=oo`03001oogoo089oo`8004=oo`004Goo00<007ooOol0
Q7oo00<007ooOol0@7oo000AOol00`00Oomoo`25Ool00`00Oomoo`0oOol0015oo`8008Moo`8003mo
o`004Goo00<007ooOol0R7oo00<007ooOol0?7oo000AOol00`00Oomoo`29Ool2000lOol0015oo`03
001oogoo08]oo`8003Yoo`004Goo0P00SWoo00<007ooOol0=goo000AOol00`00Oomoo`2>Ool2000g
Ool000Uoo`<000Eoo`03001oogoo091oo`03001oogoo03Aoo`0027oo00D007ooOomoo`0000Aoo`03
001oogoo095oo`03001oogoo03=oo`0027oo00D007ooOomoo`0000Aoo`8009=oo`8003=oo`0027oo
00D007ooOomoo`0000Aoo`03001oogoo09Aoo`03001oogoo031oo`0027oo10001Goo00<007ooOol0
UGoo00<007ooOol0;goo0008Ool00`00Oomoo`06Ool00`00Oomoo`2FOol2000_Ool000Uoo`03001o
ogoo00Eoo`8009Uoo`03001oogoo02aoo`002Woo0`0017oo00<007ooOol0VGoo00<007ooOol0:goo
000AOol00`00Oomoo`2JOol2000[Ool0015oo`03001oogoo09aoo`03001oogoo02Qoo`004Goo0P00
WWoo00<007ooOol09goo000AOol00`00Oomoo`2NOol2000WOol0015oo`03001oogoo0:1oo`03001o
ogoo02Aoo`004Goo00<007ooOol0XGoo00<007ooOol08goo000AOol2002SOol2000SOol0015oo`03
001oogoo0:Aoo`03001oogoo021oo`002Goo0`001Goo00<007ooOol0YGoo00<007ooOol07goo0008
Ool01@00Oomoogoo000017oo00<007ooOol0YWoo0P007goo0008Ool01@00Oomoogoo000017oo0P00
ZGoo00<007ooOol077oo0008Ool01@00Oomoogoo000017oo00<007ooOol0ZGoo00<007ooOol06goo
0009Ool30005Ool00`00Oomoo`2ZOol2000KOol000Qoo`05001oogooOol00004Ool00`00Oomoo`2/
Ool00`00Oomoo`0HOol000Qoo`05001oogooOol00004Ool2002^Ool00`00Oomoo`0GOol000Uoo`<0
00Eoo`03001oogoo0:ioo`8001Moo`004Goo00<007ooOol0/7oo00<007ooOol057oo000AOol00`00
Oomoo`2aOol00`00Oomoo`0COol0015oo`800;=oo`03001oogoo019oo`004Goo00<007ooOol0/goo
00<007ooOol04Goo000AOol00`00Oomoo`2dOol00`00Oomoo`0@Ool0015oo`03001oogoo0;Eoo`03
001oogoo00moo`004Goo0P00]goo0P003goo000AOol00`00Oomoo`2hOol00`00Oomoo`0<Ool00003
Ool0000000<000=oo`<000Eoo`03001oogoo0;Uoo`03001oogoo00]oo`000goo00<007ooOol00Woo
00D007ooOomoo`0000Aoo`03001oogoo0;Yoo`8000]oo`000goo00<007ooOol00Woo00D007ooOomo
o`0000Aoo`800;eoo`03001oogoo00Qoo`000goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03
001oogoo0;eoo`03001oogoo00Moo`000goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001o
ogoo0;ioo`03001oogoo00Ioo`000goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo
0<Moo`000goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`800<Qoo`0000Aoo`00000000Eoo`<0
00Eoo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qo
o`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00
b7oo00000goo000000030002Ool50004Ool00`00Oomoo`37Ool000=oo`03001oogoo00=oo`04001o
ogoo0004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Aoo`03001oogoo00Aoo`03001oogoo0<Mo
o`000goo00<007ooOol01Goo00<007ooOol00goo0P00b7oo0003Ool00`00Oomoo`06Ool00`00Oomo
o`02Ool00`00Oomoo`37Ool000=oo`03001oogoo00Ioo`03001oogoo009oo`03001oogoo0<Moo`00
0goo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo0<Moo`0000Aoo`00000000Eoo`<0
00Eoo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Mo
o`004Goo0P00b7oo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol20038Ool00003Ool0000000<000Aoo`<000Aoo`03001oogoo0<Moo`000goo00<007oo
Ool01Goo00<007ooOol00goo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool50004Ool00`00Oomo
o`37Ool000=oo`03001oogoo009oo`04001oogoo0005Ool20038Ool000=oo`03001oogoo00=oo`03
001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol00goo00<007oo00001Goo00<007ooOol0
agoo0003Ool00`00Oomoo`04Ool20005Ool00`00Oomoo`37Ool00004Ool000000007Ool00`00Oomo
o`03Ool20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37
Ool0015oo`03001oogoo0<Moo`00\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .00746 m
.04057 .01025 L
.08481 .01417 L
.12636 .01884 L
.16632 .02436 L
.20885 .03147 L
.2498 .03965 L
.29331 .04988 L
.33524 .06134 L
.37557 .0739 L
.41848 .08897 L
.4598 .10514 L
.49953 .12223 L
.54183 .14204 L
.58254 .16264 L
.62583 .1861 L
.66752 .21014 L
.70762 .2345 L
.7503 .26164 L
.79139 .28886 L
.83505 .31884 L
.87712 .34862 L
.9176 .37802 L
.96065 .40999 L
1 .43974 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`d000]oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Ioo`03001o
ogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Ioo`004Goo
00<007ooOol02Woo2000]Goo000AOol2000COol8002]Ool0015oo`03001oogoo01Yoo`P00:Eoo`00
4Goo00<007ooOol08Woo1P00Wgoo000AOol00`00Oomoo`0XOol4002KOol0015oo`8002eoo`@009Mo
o`004Goo00<007ooOol0<7oo1@00TWoo000AOol00`00Oomoo`0eOol4002>Ool0015oo`03001oogoo
03Uoo`@008Yoo`004Goo0P00?Woo0`00Qgoo000AOol00`00Oomoo`10Ool30024Ool000Qoo`D000Ao
o`03001oogoo04=oo`80089oo`002Goo00@007ooOol000Aoo`03001oogoo04Eoo`<007moo`002Woo
00<007ooOol017oo0P00BGoo0`00O7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`1;Ool3001iOol0
00aoo`03001oogoo009oo`03001oogoo04ioo`8007Moo`0037oo00<007ooOol00Woo00<007ooOol0
D7oo0`00M7oo0008Ool01@00Oomoogoo000017oo0P00E7oo0`00LGoo0009Ool30005Ool00`00Oomo
o`1FOol2001_Ool0015oo`03001oogoo05Qoo`<006aoo`004Goo00<007ooOol0Fgoo0P00JWoo000A
Ool2001NOol2001XOol0015oo`03001oogoo05moo`8006Ioo`004Goo00<007ooOol0HGoo0`00Hgoo
000AOol00`00Oomoo`1TOol2001QOol0015oo`8006Moo`8005moo`004Goo00<007ooOol0J7oo0P00
GGoo000:Ool30004Ool00`00Oomoo`1ZOol00`00Oomoo`1JOol000]oo`03001oogoo00=oo`03001o
ogoo06]oo`8005Yoo`0027oo1@0017oo0P00KWoo0P00F7oo0008Ool01000Oomoo`001Goo00<007oo
Ool0Kgoo0P00EWoo0009Ool00`00Ool00005Ool00`00Oomoo`1aOol3001COol000Uoo`03001oo`00
00Eoo`03001oogoo07Aoo`80055oo`002Woo0P001Goo0P00Mgoo0P00Cgoo000;Ool00`00Oomoo`03
Ool00`00Oomoo`1hOol00`00Oomoo`1<Ool0015oo`03001oogoo07Uoo`8004aoo`004Goo00<007oo
Ool0Ngoo0P00BWoo000AOol2001nOol00`00Oomoo`17Ool0015oo`03001oogoo07ioo`8004Moo`00
4Goo00<007ooOol0P7oo00<007ooOol0A7oo000AOol00`00Oomoo`21Ool20014Ool0015oo`8008Ao
o`80049oo`004Goo00<007ooOol0QGoo00<007ooOol0?goo0009Ool30005Ool00`00Oomoo`26Ool2
000oOol000Qoo`05001oogooOol00004Ool00`00Oomoo`28Ool00`00Oomoo`0lOol000Qoo`05001o
ogooOol00004Ool2002:Ool2000lOol000Qoo`05001oogooOol00004Ool00`00Oomoo`2;Ool2000j
Ool000Qoo`@000Eoo`03001oogoo08eoo`03001oogoo03Moo`0027oo00<007ooOol01Woo00<007oo
Ool0SWoo0P00=goo0009Ool00`00Oomoo`05Ool2002AOol00`00Oomoo`0dOol000Yoo`<000Aoo`03
001oogoo095oo`8003Aoo`004Goo00<007ooOol0Tgoo0P00<Woo000AOol00`00Oomoo`2EOol00`00
Oomoo`0_Ool0015oo`8009Moo`8002moo`004Goo00<007ooOol0V7oo00<007ooOol0;7oo000AOol0
0`00Oomoo`2IOol00`00Oomoo`0[Ool0015oo`03001oogoo09Yoo`8002]oo`004Goo0P00WGoo00<0
07ooOol0:7oo000AOol00`00Oomoo`2MOol00`00Oomoo`0WOol000Uoo`<000Eoo`03001oogoo09io
o`8002Moo`0027oo00D007ooOomoo`0000Aoo`03001oogoo0:1oo`03001oogoo02Aoo`0027oo00D0
07ooOomoo`0000Aoo`800:9oo`03001oogoo02=oo`0027oo00D007ooOomoo`0000Aoo`03001oogoo
0:9oo`8002=oo`002Goo0`001Goo00<007ooOol0Y7oo00<007ooOol087oo0008Ool01@00Oomoogoo
000017oo00<007ooOol0YGoo00<007ooOol07goo0008Ool01@00Oomoogoo000017oo0P00Ygoo0P00
7goo0009Ool30005Ool00`00Oomoo`2XOol00`00Oomoo`0LOol0015oo`03001oogoo0:Uoo`8001ao
o`004Goo00<007ooOol0Zgoo0P006Woo000AOol2002^Ool00`00Oomoo`0GOol0015oo`03001oogoo
0:ioo`8001Moo`004Goo00<007ooOol0/7oo00<007ooOol057oo000AOol00`00Oomoo`2aOol00`00
Oomoo`0COol0015oo`800;=oo`8001=oo`004Goo00<007ooOol0]7oo00<007ooOol047oo00000goo
000000030003Ool30005Ool00`00Oomoo`2eOol00`00Oomoo`0?Ool000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2fOol00`00Oomoo`0>Ool000=oo`03001oogoo009oo`05001o
ogooOol00004Ool2002hOol2000>Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00
Oomoo`2iOol00`00Oomoo`0;Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomo
o`2jOol2000;Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2lOol00`00
Oomoo`08Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool2002nOol00`00Oomoo`07Ool0
0004Ool000000005Ool30005Ool00`00Oomoo`2nOol00`00Oomoo`06Ool0015oo`03001oogoo0<Mo
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`0000=oo`0000000`000Woo1@0017oo00<007oo
Ool0agoo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0agoo0003Ool00`00Oomo
o`04Ool00`00Oomoo`04Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`80
0<Qoo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0agoo0003Ool00`00Oomoo`06
Ool00`00Oomoo`02Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool0
0`00Oomoo`37Ool00004Ool000000005Ool30005Ool20038Ool0015oo`03001oogoo0<Moo`004Goo
00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000A
Ool00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo000000030004Ool3
0004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0<Moo`00
0goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomoo`00
1Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03001o
ogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<007oo
Ool0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0015o
o`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .01744 m
.04057 .02209 L
.08481 .02816 L
.12636 .03492 L
.16632 .04247 L
.20885 .05169 L
.2498 .06181 L
.29331 .07394 L
.33524 .08701 L
.37557 .10089 L
.41848 .11707 L
.4598 .13402 L
.49953 .15158 L
.54183 .17159 L
.58254 .1921 L
.62583 .2152 L
.66752 .23864 L
.70762 .26223 L
.7503 .28839 L
.79139 .31453 L
.83505 .34325 L
.87712 .37178 L
.9176 .39994 L
.96065 .43059 L
1 .45917 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol50035Ool0015oo`03001oogoo009oo`P00;eoo`004Goo00<0
07ooOol02Woo2000]Goo000AOol00`00Oomoo`0BOol6002_Ool0015oo`8001Uoo`@00:]oo`004Goo
00<007ooOol077oo1P00YGoo000AOol00`00Oomoo`0ROol6002OOol0015oo`03001oogoo02Qoo`@0
09]oo`004Goo0P00;Goo1000Ugoo000AOol00`00Oomoo`0`Ool3002DOol000Qoo`D000Aoo`03001o
ogoo03=oo`<0095oo`002Goo00@007ooOol000Aoo`03001oogoo03Ioo`<008ioo`002Woo00<007oo
Ool017oo0P00>Woo1000RWoo000;Ool00`00Oomoo`03Ool00`00Oomoo`0mOol30027Ool000aoo`03
001oogoo009oo`03001oogoo041oo`<008Aoo`0037oo00<007ooOol00Woo00<007ooOol0@goo0P00
PWoo0008Ool01@00Oomoogoo000017oo0P00AWoo0`00Ogoo0009Ool30005Ool00`00Oomoo`18Ool3
001lOol0015oo`03001oogoo04]oo`<007Uoo`004Goo00<007ooOol0CWoo0P00Mgoo000AOol2001A
Ool3001dOol0015oo`03001oogoo05=oo`<0075oo`004Goo00<007ooOol0EWoo0P00Kgoo000AOol0
0`00Oomoo`1HOol2001]Ool0015oo`8005]oo`03001oogoo06Yoo`004Goo00<007ooOol0Fgoo0P00
JWoo000:Ool30004Ool00`00Oomoo`1MOol2001XOol000]oo`03001oogoo00=oo`03001oogoo05mo
o`8006Ioo`0027oo1@0017oo0P00HWoo0`00Hgoo0008Ool01000Oomoo`001Goo00<007ooOol0I7oo
0P00HGoo0009Ool00`00Ool00005Ool00`00Oomoo`1VOol2001OOol000Uoo`03001oo`0000Eoo`03
001oogoo06Qoo`8005eoo`002Woo0P001Goo0P00Jgoo00<007ooOol0FWoo000;Ool00`00Oomoo`03
Ool00`00Oomoo`1[Ool2001JOol0015oo`03001oogoo06eoo`8005Qoo`004Goo00<007ooOol0Kgoo
0P00EWoo000AOol2001bOol3001COol0015oo`03001oogoo07Aoo`80055oo`004Goo00<007ooOol0
MWoo0P00Cgoo000AOol00`00Oomoo`1hOol00`00Oomoo`1<Ool0015oo`8007Yoo`8004aoo`004Goo
00<007ooOol0Ngoo0P00BWoo0009Ool30005Ool00`00Oomoo`1mOol00`00Oomoo`17Ool000Qoo`05
001oogooOol00004Ool00`00Oomoo`1nOol20017Ool000Qoo`05001oogooOol00004Ool20021Ool2
0015Ool000Qoo`05001oogooOol00004Ool00`00Oomoo`22Ool20013Ool000Qoo`@000Eoo`03001o
ogoo08Aoo`80045oo`0027oo00<007ooOol01Woo00<007ooOol0QWoo0P00?goo0009Ool00`00Oomo
o`05Ool20029Ool00`00Oomoo`0lOol000Yoo`<000Aoo`03001oogoo08Uoo`8003aoo`004Goo00<0
07ooOol0Rgoo0P00>Woo000AOol00`00Oomoo`2=Ool00`00Oomoo`0gOol0015oo`8008moo`8003Mo
o`004Goo00<007ooOol0T7oo00<007ooOol0=7oo000AOol00`00Oomoo`2AOol2000dOol0015oo`03
001oogoo09=oo`80039oo`004Goo0P00UWoo00<007ooOol0;goo000AOol00`00Oomoo`2FOol2000_
Ool000Uoo`<000Eoo`03001oogoo09Qoo`03001oogoo02aoo`0027oo00D007ooOomoo`0000Aoo`03
001oogoo09Uoo`03001oogoo02]oo`0027oo00D007ooOomoo`0000Aoo`8009]oo`8002]oo`0027oo
00D007ooOomoo`0000Aoo`03001oogoo09aoo`03001oogoo02Qoo`002Goo0`001Goo00<007ooOol0
WGoo00<007ooOol09goo0008Ool01@00Oomoogoo000017oo00<007ooOol0WWoo0P009goo0008Ool0
1@00Oomoogoo000017oo0P00XGoo00<007ooOol097oo0009Ool30005Ool00`00Oomoo`2QOol2000T
Ool0015oo`03001oogoo0:=oo`80029oo`004Goo00<007ooOol0YGoo00<007ooOol07goo000AOol2
002WOol2000OOol0015oo`03001oogoo0:Qoo`03001oogoo01aoo`004Goo00<007ooOol0ZGoo00<0
07ooOol06goo000AOol00`00Oomoo`2ZOol2000KOol0015oo`800:eoo`03001oogoo01Qoo`004Goo
00<007ooOol0[Goo00<007ooOol05goo00000goo000000030003Ool30005Ool00`00Oomoo`2^Ool2
000GOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2`Ool00`00Oomoo`0D
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool2002bOol2000DOol000=oo`03001oogoo
009oo`05001oogooOol00004Ool00`00Oomoo`2cOol00`00Oomoo`0AOol000=oo`03001oogoo009o
o`05001oogooOol00004Ool00`00Oomoo`2dOol2000AOol000=oo`03001oogoo009oo`05001oogoo
Ool00004Ool00`00Oomoo`2fOol00`00Oomoo`0>Ool000=oo`03001oogoo009oo`05001oogooOol0
0004Ool2002hOol2000>Ool00004Ool000000005Ool30005Ool00`00Oomoo`2iOol00`00Oomoo`0;
Ool0015oo`03001oogoo0;Yoo`8000]oo`004Goo00<007ooOol0_7oo00<007ooOol027oo000AOol2
002nOol00`00Oomoo`07Ool0015oo`03001oogoo0;ioo`03001oogoo00Ioo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`0000=oo`0000000`000Woo1@0017oo00<007oo
Ool0agoo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0agoo0003Ool00`00Oomo
o`04Ool00`00Oomoo`04Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`80
0<Qoo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0agoo0003Ool00`00Oomoo`06
Ool00`00Oomoo`02Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool0
0`00Oomoo`37Ool00004Ool000000005Ool30005Ool20038Ool0015oo`03001oogoo0<Moo`004Goo
00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000A
Ool00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo000000030004Ool3
0004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0<Moo`00
0goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomoo`00
1Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03001o
ogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<007oo
Ool0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0015o
o`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .02883 m
.04057 .03499 L
.08481 .04273 L
.12636 .05104 L
.16632 .06004 L
.20885 .07075 L
.2498 .08218 L
.29331 .09559 L
.33524 .10974 L
.37557 .1245 L
.41848 .14145 L
.4598 .15895 L
.49953 .17687 L
.54183 .19709 L
.58254 .21763 L
.62583 .24058 L
.66752 .26373 L
.70762 .28692 L
.7503 .31254 L
.79139 .33805 L
.83505 .36603 L
.87712 .39378 L
.9176 .42115 L
.96065 .45092 L
1 .47869 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo1@00aGoo000A
Ool00`00Oomoo`02Ool6002oOol0015oo`8000Uoo`@00;]oo`004Goo00<007ooOol037oo1P00]Goo
000AOol00`00Oomoo`0BOol6002_Ool0015oo`03001oogoo01Qoo`@00:]oo`004Goo0P007Goo1000
Ygoo000AOol00`00Oomoo`0POol4002SOol000Qoo`D000Aoo`03001oogoo02Aoo`@009moo`002Goo
00@007ooOol000Aoo`03001oogoo02Qoo`@009]oo`002Woo00<007ooOol017oo0P00;Goo1000Ugoo
000;Ool00`00Oomoo`03Ool00`00Oomoo`0`Ool3002DOol000aoo`03001oogoo009oo`03001oogoo
03=oo`<0095oo`0037oo00<007ooOol00Woo00<007ooOol0=Woo0P00Sgoo0008Ool01@00Oomoogoo
000017oo0P00>Goo0`00S7oo0009Ool30005Ool00`00Oomoo`0kOol30029Ool0015oo`03001oogoo
03ioo`<008Ioo`004Goo00<007ooOol0@Goo0`00Pgoo000AOol20015Ool30020Ool0015oo`03001o
ogoo04Moo`8007ioo`004Goo00<007ooOol0BGoo0`00Ngoo000AOol00`00Oomoo`1<Ool2001iOol0
015oo`8004moo`8007Moo`004Goo00<007ooOol0D7oo0`00M7oo000:Ool30004Ool00`00Oomoo`1C
Ool3001aOol000]oo`03001oogoo00=oo`03001oogoo05Ioo`8006moo`0027oo1@0017oo0P00FGoo
0P00KGoo0008Ool01000Oomoo`001Goo00<007ooOol0FWoo00<007ooOol0JWoo0009Ool00`00Ool0
0005Ool00`00Oomoo`1KOol2001ZOol000Uoo`03001oo`0000Eoo`03001oogoo05eoo`<006Moo`00
2Woo0P001Goo0P00HGoo0`00I7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`1SOol3001QOol0015o
o`03001oogoo06Ioo`8005moo`004Goo00<007ooOol0J7oo0P00GGoo000AOol2001[Ool00`00Oomo
o`1JOol0015oo`03001oogoo06]oo`8005Yoo`004Goo00<007ooOol0KGoo0P00F7oo000AOol00`00
Oomoo`1_Ool2001FOol0015oo`80079oo`8005Aoo`004Goo00<007ooOol0Lgoo00<007ooOol0DGoo
0009Ool30005Ool00`00Oomoo`1dOol2001AOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1f
Ool2001?Ool000Qoo`05001oogooOol00004Ool2001iOol2001=Ool000Qoo`05001oogooOol00004
Ool00`00Oomoo`1jOol2001;Ool000Qoo`@000Eoo`03001oogoo07aoo`8004Uoo`0027oo00<007oo
Ool01Woo00<007ooOol0OWoo0P00Agoo0009Ool00`00Oomoo`05Ool20021Ool00`00Oomoo`14Ool0
00Yoo`<000Aoo`03001oogoo085oo`8004Aoo`004Goo00<007ooOol0Pgoo0P00@Woo000AOol00`00
Oomoo`25Ool00`00Oomoo`0oOol0015oo`8008Moo`8003moo`004Goo00<007ooOol0R7oo00<007oo
Ool0?7oo000AOol00`00Oomoo`29Ool2000lOol0015oo`03001oogoo08]oo`8003Yoo`004Goo0P00
SWoo00<007ooOol0=goo000AOol00`00Oomoo`2>Ool2000gOol000Uoo`<000Eoo`03001oogoo091o
o`03001oogoo03Aoo`0027oo00D007ooOomoo`0000Aoo`03001oogoo095oo`8003Aoo`0027oo00D0
07ooOomoo`0000Aoo`8009Aoo`80039oo`0027oo00D007ooOomoo`0000Aoo`03001oogoo09Eoo`03
001oogoo02moo`002Goo0`001Goo00<007ooOol0UWoo0P00;goo0008Ool01@00Oomoogoo000017oo
00<007ooOol0V7oo00<007ooOol0;7oo0008Ool01@00Oomoogoo000017oo0P00VWoo0P00;7oo0009
Ool30005Ool00`00Oomoo`2KOol2000ZOol0015oo`03001oogoo09eoo`03001oogoo02Moo`004Goo
00<007ooOol0WWoo0P009goo000AOol2002QOol00`00Oomoo`0TOol0015oo`03001oogoo0:5oo`80
02Aoo`004Goo00<007ooOol0Xgoo0P008Woo000AOol00`00Oomoo`2UOol00`00Oomoo`0OOol0015o
o`800:Moo`8001moo`004Goo00<007ooOol0Z7oo00<007ooOol077oo00000goo000000030003Ool3
0005Ool00`00Oomoo`2YOol00`00Oomoo`0KOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`2ZOol2000KOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2002]Ool0
0`00Oomoo`0HOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2]Ool00`00
Oomoo`0GOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2^Ool2000GOol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2`Ool00`00Oomoo`0DOol000=o
o`03001oogoo009oo`05001oogooOol00004Ool2002bOol2000DOol00004Ool000000005Ool30005
Ool00`00Oomoo`2cOol00`00Oomoo`0AOol0015oo`03001oogoo0;Aoo`80015oo`004Goo00<007oo
Ool0]Woo00<007ooOol03Woo000AOol2002hOol2000>Ool0015oo`03001oogoo0;Uoo`03001oogoo
00]oo`004Goo00<007ooOol0^Woo0P002goo000AOol00`00Oomoo`2lOol00`00Oomoo`08Ool0015o
o`800;ioo`03001oogoo00Moo`0000=oo`0000000`000Woo1@0017oo00<007ooOol0_Woo00<007oo
Ool01Woo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0agoo0003Ool00`00Oomo
o`04Ool00`00Oomoo`04Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`80
0<Qoo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0agoo0003Ool00`00Oomoo`06
Ool00`00Oomoo`02Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool0
0`00Oomoo`37Ool00004Ool000000005Ool30005Ool20038Ool0015oo`03001oogoo0<Moo`004Goo
00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000A
Ool00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo000000030004Ool3
0004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0<Moo`00
0goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomoo`00
1Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03001o
ogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<007oo
Ool0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0015o
o`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .04077 m
.04057 .04815 L
.08481 .05722 L
.12636 .06674 L
.16632 .07686 L
.20885 .08869 L
.2498 .10112 L
.29331 .11548 L
.33524 .13044 L
.37557 .14587 L
.41848 .16339 L
.4598 .18133 L
.49953 .19955 L
.54183 .21997 L
.58254 .24058 L
.62583 .26349 L
.66752 .2865 L
.70762 .30946 L
.7503 .33474 L
.79139 .35986 L
.83505 .38736 L
.87712 .41458 L
.9176 .4414 L
.96065 .47056 L
1 .49774 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`<00<Moo`004Goo00<007ooOol01000`goo000AOol00`00
Oomoo`04Ool6002mOol0015oo`03001oogoo00Yoo`H00;Moo`004Goo0P004Goo1000/goo000AOol0
0`00Oomoo`0DOol4002_Ool000Qoo`D000Aoo`03001oogoo01Qoo`@00:]oo`002Goo00@007ooOol0
00Aoo`03001oogoo01aoo`<00:Qoo`002Woo00<007ooOol017oo0P0087oo0`00YGoo000;Ool00`00
Oomoo`03Ool00`00Oomoo`0ROol3002ROol000aoo`03001oogoo009oo`03001oogoo02Eoo`<009mo
o`0037oo00<007ooOol00Woo00<007ooOol0:7oo1000Vgoo0008Ool01@00Oomoogoo000017oo0P00
;Goo1000Ugoo0009Ool30005Ool00`00Oomoo`0`Ool3002DOol0015oo`03001oogoo03=oo`<0095o
o`004Goo00<007ooOol0=Woo0P00Sgoo000AOol2000iOol3002<Ool0015oo`03001oogoo03]oo`<0
08Uoo`004Goo00<007ooOol0?Woo0P00Qgoo000AOol00`00Oomoo`10Ool30024Ool0015oo`8004Ao
o`80089oo`004Goo00<007ooOol0AGoo0`00Ogoo000:Ool30004Ool00`00Oomoo`18Ool3001lOol0
00]oo`03001oogoo00=oo`03001oogoo04]oo`<007Uoo`0027oo1@0017oo0P00Cgoo0P00Mgoo0008
Ool01000Oomoo`001Goo00<007ooOol0D7oo0`00M7oo0009Ool00`00Ool00005Ool00`00Oomoo`1C
Ool3001aOol000Uoo`03001oo`0000Eoo`03001oogoo05Ioo`8006moo`002Woo0P001Goo0P00FGoo
0P00KGoo000;Ool00`00Oomoo`03Ool00`00Oomoo`1JOol00`00Oomoo`1ZOol0015oo`03001oogoo
05]oo`8006Yoo`004Goo00<007ooOol0GGoo0P00J7oo000AOol2001POol2001VOol0015oo`03001o
ogoo065oo`<006=oo`004Goo00<007ooOol0I7oo0P00HGoo000AOol00`00Oomoo`1VOol2001OOol0
015oo`8006Uoo`8005eoo`004Goo00<007ooOol0JWoo00<007ooOol0FWoo0009Ool30005Ool00`00
Oomoo`1[Ool2001JOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1]Ool2001HOol000Qoo`05
001oogooOol00004Ool2001`Ool2001FOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1aOol3
001COol000Qoo`@000Eoo`03001oogoo07Aoo`80055oo`0027oo00<007ooOol01Woo00<007ooOol0
MWoo0P00Cgoo0009Ool00`00Oomoo`05Ool2001iOol00`00Oomoo`1<Ool000Yoo`<000Aoo`03001o
ogoo07Uoo`8004aoo`004Goo00<007ooOol0Ngoo0P00BWoo000AOol00`00Oomoo`1mOol00`00Oomo
o`17Ool0015oo`8007moo`8004Moo`004Goo00<007ooOol0P7oo0P00AGoo000AOol00`00Oomoo`22
Ool20013Ool0015oo`03001oogoo08Aoo`80045oo`004Goo0P00Qgoo0P00?goo000AOol00`00Oomo
o`28Ool00`00Oomoo`0lOol000Uoo`<000Eoo`03001oogoo08Uoo`8003aoo`0027oo00D007ooOomo
o`0000Aoo`03001oogoo08]oo`8003Yoo`0027oo00D007ooOomoo`0000Aoo`8008ioo`03001oogoo
03Moo`0027oo00D007ooOomoo`0000Aoo`03001oogoo08ioo`8003Moo`002Goo0`001Goo00<007oo
Ool0T7oo00<007ooOol0=7oo0008Ool01@00Oomoogoo000017oo00<007ooOol0TGoo0P00=7oo0008
Ool01@00Oomoogoo000017oo0P00U7oo0P00<Woo0009Ool30005Ool00`00Oomoo`2EOol00`00Oomo
o`0_Ool0015oo`03001oogoo09Ioo`8002moo`004Goo00<007ooOol0V7oo00<007ooOol0;7oo000A
Ool2002JOol2000/Ool0015oo`03001oogoo09]oo`8002Yoo`004Goo00<007ooOol0WGoo00<007oo
Ool09goo000AOol00`00Oomoo`2NOol2000WOol0015oo`800:5oo`03001oogoo02Aoo`004Goo00<0
07ooOol0XGoo00<007ooOol08goo00000goo000000030003Ool30005Ool00`00Oomoo`2ROol2000S
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2TOol00`00Oomoo`0POol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool2002VOol00`00Oomoo`0OOol000=oo`03001o
ogoo009oo`05001oogooOol00004Ool00`00Oomoo`2VOol2000OOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2XOol00`00Oomoo`0LOol000=oo`03001oogoo009oo`05001o
ogooOol00004Ool00`00Oomoo`2YOol2000LOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool2002/Ool2000JOol00004Ool000000005Ool30005Ool00`00Oomoo`2]Ool00`00Oomoo`0GOol0
015oo`03001oogoo0:ioo`8001Moo`004Goo00<007ooOol0/7oo0P005Goo000AOol2002cOol2000C
Ool0015oo`03001oogoo0;Aoo`03001oogoo011oo`004Goo00<007ooOol0]Goo0P0047oo000AOol0
0`00Oomoo`2gOol2000>Ool0015oo`800;Yoo`03001oogoo00]oo`0000=oo`0000000`000Woo1@00
17oo00<007ooOol0^Woo00<007ooOol02Woo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<0
07ooOol0^goo00<007ooOol02Goo0003Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`2l
Ool00`00Oomoo`08Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`800;ioo`03001oogoo00Mo
o`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0_Woo00<007ooOol01Woo0003Ool0
0`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`37Ool000=oo`03001oogoo009oo`05001oogoo
Ool00004Ool00`00Oomoo`37Ool00004Ool000000005Ool30005Ool20038Ool0015oo`03001oogoo
0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007oo
Ool0agoo000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo0000
00030004Ool30004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001o
ogoo0<Moo`000goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool0
1000Oomoo`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool0
00=oo`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P00
1Goo00<007ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001
\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .05286 m
.04057 .06126 L
.08481 .07142 L
.12636 .08193 L
.16632 .09294 L
.20885 .10567 L
.2498 .11889 L
.29331 .13402 L
.33524 .14962 L
.37557 .16558 L
.41848 .18358 L
.4598 .20188 L
.49953 .22036 L
.54183 .24097 L
.58254 .26167 L
.62583 .2846 L
.66752 .30754 L
.70762 .33036 L
.7503 .35543 L
.79139 .38029 L
.83505 .40745 L
.87712 .4343 L
.9176 .46073 L
.96065 .48944 L
1 .51619 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`D00<Eoo`004Goo0P000goo1P00_goo000AOol00`00Oomoo`08Ool4002kOol000Qo
o`D000Aoo`03001oogoo00aoo`@00;Moo`002Goo00@007ooOol000Aoo`03001oogoo011oo`@00;=o
o`002Woo00<007ooOol017oo0P005Goo1000[goo000;Ool00`00Oomoo`03Ool00`00Oomoo`0HOol4
002[Ool000aoo`03001oogoo009oo`03001oogoo01aoo`<00:Qoo`0037oo00<007ooOol00Woo00<0
07ooOol07goo0`00YGoo0008Ool01@00Oomoogoo000017oo0P008goo0`00XWoo0009Ool30005Ool0
0`00Oomoo`0UOol3002OOol0015oo`03001oogoo02Qoo`@009]oo`004Goo00<007ooOol0;7oo1000
Ugoo000AOol2000aOol3002DOol0015oo`03001oogoo03=oo`<0095oo`004Goo00<007ooOol0=Woo
0P00Sgoo000AOol00`00Oomoo`0hOol3002<Ool0015oo`8003aoo`<008Uoo`004Goo00<007ooOol0
?Woo0P00Qgoo000:Ool30004Ool00`00Oomoo`10Ool30024Ool000]oo`03001oogoo00=oo`03001o
ogoo04=oo`80089oo`0027oo1@0017oo0P00AWoo0P00P7oo0008Ool01000Oomoo`001Goo00<007oo
Ool0Agoo0P00OWoo0009Ool00`00Ool00005Ool00`00Oomoo`19Ool3001kOol000Uoo`03001oo`00
00Eoo`03001oogoo04aoo`8007Uoo`002Woo0P001Goo0P00Cgoo0P00Mgoo000;Ool00`00Oomoo`03
Ool00`00Oomoo`1@Ool3001dOol0015oo`03001oogoo05=oo`<0075oo`004Goo00<007ooOol0EWoo
0P00Kgoo000AOol2001IOol2001]Ool0015oo`03001oogoo05Yoo`03001oogoo06Yoo`004Goo00<0
07ooOol0Fgoo0P00JWoo000AOol00`00Oomoo`1MOol2001XOol0015oo`80061oo`8006Ioo`004Goo
00<007ooOol0HGoo0`00Hgoo0009Ool30005Ool00`00Oomoo`1TOol2001QOol000Qoo`05001oogoo
Ool00004Ool00`00Oomoo`1VOol2001OOol000Qoo`05001oogooOol00004Ool2001YOol2001MOol0
00Qoo`05001oogooOol00004Ool00`00Oomoo`1ZOol00`00Oomoo`1JOol000Qoo`@000Eoo`03001o
ogoo06]oo`8005Yoo`0027oo00<007ooOol01Woo00<007ooOol0KGoo0P00F7oo0009Ool00`00Oomo
o`05Ool2001`Ool2001FOol000Yoo`<000Aoo`03001oogoo075oo`<005=oo`004Goo00<007ooOol0
M7oo0P00DGoo000AOol00`00Oomoo`1fOol2001?Ool0015oo`8007Uoo`03001oogoo04aoo`004Goo
00<007ooOol0NGoo0P00C7oo000AOol00`00Oomoo`1kOol2001:Ool0015oo`03001oogoo07eoo`03
001oogoo04Moo`004Goo0P00Ogoo0P00Agoo000AOol00`00Oomoo`20Ool20015Ool000Uoo`<000Eo
o`03001oogoo089oo`8004=oo`0027oo00D007ooOomoo`0000Aoo`03001oogoo08Aoo`80045oo`00
27oo00D007ooOomoo`0000Aoo`8008Moo`8003moo`0027oo00D007ooOomoo`0000Aoo`03001oogoo
08Qoo`03001oogoo03aoo`002Goo0`001Goo00<007ooOol0RGoo0P00?7oo0008Ool01@00Oomoogoo
000017oo00<007ooOol0Rgoo0P00>Woo0008Ool01@00Oomoogoo000017oo0P00SWoo00<007ooOol0
=goo0009Ool30005Ool00`00Oomoo`2>Ool2000gOol0015oo`03001oogoo091oo`03001oogoo03Ao
o`004Goo00<007ooOol0TGoo0P00=7oo000AOol2002DOol2000bOol0015oo`03001oogoo09Eoo`03
001oogoo02moo`004Goo00<007ooOol0UWoo0P00;goo000AOol00`00Oomoo`2HOol00`00Oomoo`0/
Ool0015oo`8009Yoo`8002aoo`004Goo00<007ooOol0Vgoo0P00:Woo00000goo000000030003Ool3
0005Ool00`00Oomoo`2MOol00`00Oomoo`0WOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`2NOol2000WOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2002QOol0
0`00Oomoo`0TOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2QOol2000T
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2SOol2000ROol000=oo`03
001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2UOol00`00Oomoo`0OOol000=oo`03001o
ogoo009oo`05001oogooOol00004Ool2002WOol2000OOol00004Ool000000005Ool30005Ool00`00
Oomoo`2XOol00`00Oomoo`0LOol0015oo`03001oogoo0:Uoo`8001aoo`004Goo00<007ooOol0Zgoo
0P006Woo000AOol2002^Ool00`00Oomoo`0GOol0015oo`03001oogoo0:ioo`8001Moo`004Goo00<0
07ooOol0/7oo00<007ooOol057oo000AOol00`00Oomoo`2aOol2000DOol0015oo`800;Aoo`03001o
ogoo015oo`0000=oo`0000000`000Woo1@0017oo00<007ooOol0]7oo0P004Goo0003Ool00`00Oomo
o`03Ool01000Oomoo`0017oo00<007ooOol0]Woo00<007ooOol03Woo0003Ool00`00Oomoo`04Ool0
0`00Oomoo`04Ool00`00Oomoo`2gOol2000>Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`80
0;Yoo`03001oogoo00]oo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0^Woo0P00
2goo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`2lOol00`00Oomoo`08Ool000=o
o`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2mOol00`00Oomoo`07Ool00004Ool0
00000005Ool30005Ool2002oOol00`00Oomoo`06Ool0015oo`03001oogoo0<Moo`004Goo00<007oo
Ool0agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00
Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo000000030004Ool30004Ool0
0`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0<Moo`000goo00<0
07ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomoo`001Goo0P00
b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03001oogoo00=o
o`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<007ooOol0agoo
000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0015oo`03001o
ogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .0649 m
.04057 .07418 L
.08481 .08526 L
.12636 .09659 L
.16632 .10836 L
.20885 .12183 L
.2498 .13572 L
.29331 .15147 L
.33524 .16761 L
.37557 .18402 L
.41848 .20242 L
.4598 .22104 L
.49953 .23975 L
.54183 .26054 L
.58254 .28134 L
.62583 .30431 L
.66752 .32723 L
.70762 .34997 L
.7503 .37491 L
.79139 .39959 L
.83505 .42651 L
.87712 .45309 L
.9176 .47923 L
.96065 .5076 L
1 .53402 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol30037Ool000Qoo`D000Aoo`03001o
ogoo00@00<=oo`002Goo00@007ooOol000Aoo`03001oogoo00Aoo`@00;moo`002Woo00<007ooOol0
17oo0P002Goo1000^goo000;Ool00`00Oomoo`03Ool00`00Oomoo`0<Ool4002gOol000aoo`03001o
ogoo009oo`03001oogoo011oo`@00;=oo`0037oo00<007ooOol00Woo00<007ooOol057oo1000[goo
0008Ool01@00Oomoogoo000017oo0P006Goo1000Zgoo0009Ool30005Ool00`00Oomoo`0LOol3002X
Ool0015oo`03001oogoo01moo`<00:Eoo`004Goo00<007ooOol08Woo0`00XWoo000AOol2000VOol2
002POol0015oo`03001oogoo02Moo`<009eoo`004Goo00<007ooOol0:Woo0`00VWoo000AOol00`00
Oomoo`0]Ool3002GOol0015oo`80035oo`<009Aoo`004Goo00<007ooOol0<goo0`00TGoo000:Ool3
0004Ool00`00Oomoo`0fOol2002?Ool000]oo`03001oogoo00=oo`03001oogoo03Qoo`<008aoo`00
27oo1@0017oo0P00?7oo0`00RGoo0008Ool01000Oomoo`001Goo00<007ooOol0?Woo0P00Qgoo0009
Ool00`00Ool00005Ool00`00Oomoo`10Ool30024Ool000Uoo`03001oo`0000Eoo`03001oogoo04=o
o`80089oo`002Woo0P001Goo0P00AWoo0P00P7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`17Ool2
001nOol0015oo`03001oogoo04Uoo`<007]oo`004Goo00<007ooOol0C7oo0P00NGoo000AOol2001?
Ool2001gOol0015oo`03001oogoo051oo`<007Aoo`004Goo00<007ooOol0Dgoo0`00LGoo000AOol0
0`00Oomoo`1FOol2001_Ool0015oo`8005Uoo`8006eoo`004Goo00<007ooOol0FWoo00<007ooOol0
JWoo0009Ool30005Ool00`00Oomoo`1KOol2001ZOol000Qoo`05001oogooOol00004Ool00`00Oomo
o`1MOol2001XOol000Qoo`05001oogooOol00004Ool2001POol2001VOol000Qoo`05001oogooOol0
0004Ool00`00Oomoo`1QOol3001SOol000Qoo`@000Eoo`03001oogoo06Aoo`80065oo`0027oo00<0
07ooOol01Woo00<007ooOol0IWoo0P00Ggoo0009Ool00`00Oomoo`05Ool2001YOol2001MOol000Yo
o`<000Aoo`03001oogoo06Yoo`03001oogoo05Yoo`004Goo00<007ooOol0Jgoo0P00FWoo000AOol0
0`00Oomoo`1]Ool2001HOol0015oo`80071oo`8005Ioo`004Goo00<007ooOol0LGoo0`00Dgoo000A
Ool00`00Oomoo`1dOol2001AOol0015oo`03001oogoo07Ioo`8004moo`004Goo0P00NGoo00<007oo
Ool0C7oo000AOol00`00Oomoo`1iOol2001<Ool000Uoo`<000Eoo`03001oogoo07]oo`8004Yoo`00
27oo00D007ooOomoo`0000Aoo`03001oogoo07eoo`03001oogoo04Moo`0027oo00D007ooOomoo`00
00Aoo`8007moo`8004Moo`0027oo00D007ooOomoo`0000Aoo`03001oogoo081oo`8004Eoo`002Goo
0`001Goo00<007ooOol0PWoo0P00@goo0008Ool01@00Oomoogoo000017oo00<007ooOol0Q7oo0P00
@Goo0008Ool01@00Oomoogoo000017oo0P00Qgoo0P00?goo0009Ool30005Ool00`00Oomoo`28Ool0
0`00Oomoo`0lOol0015oo`03001oogoo08Uoo`8003aoo`004Goo00<007ooOol0Rgoo0P00>Woo000A
Ool2002>Ool00`00Oomoo`0gOol0015oo`03001oogoo08ioo`8003Moo`004Goo00<007ooOol0T7oo
00<007ooOol0=7oo000AOol00`00Oomoo`2AOol2000dOol0015oo`8009Aoo`80039oo`004Goo00<0
07ooOol0UGoo00<007ooOol0;goo00000goo000000030003Ool30005Ool00`00Oomoo`2FOol2000_
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2HOol00`00Oomoo`0/Ool0
00=oo`03001oogoo009oo`05001oogooOol00004Ool2002JOol2000/Ool000=oo`03001oogoo009o
o`05001oogooOol00004Ool00`00Oomoo`2KOol2000ZOol000=oo`03001oogoo009oo`05001oogoo
Ool00004Ool00`00Oomoo`2MOol00`00Oomoo`0WOol000=oo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`2NOol2000WOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2002Q
Ool00`00Oomoo`0TOol00004Ool000000005Ool30005Ool00`00Oomoo`2QOol2000TOol0015oo`03
001oogoo0:=oo`80029oo`004Goo00<007ooOol0YGoo00<007ooOol07goo000AOol2002WOol2000O
Ool0015oo`03001oogoo0:Qoo`03001oogoo01aoo`004Goo00<007ooOol0ZGoo0P0077oo000AOol0
0`00Oomoo`2[Ool2000JOol0015oo`800:ioo`03001oogoo01Moo`0000=oo`0000000`000Woo1@00
17oo00<007ooOol0[Woo0P005goo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0
/7oo0P005Goo0003Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`2bOol2000COol000=o
o`03001oogoo00Eoo`03001oogoo00=oo`800;Eoo`03001oogoo011oo`000goo00<007ooOol01Woo
00<007ooOol00Woo00<007ooOol0]Goo0P0047oo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool0
0`00Oomoo`2gOol2000>Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2i
Ool00`00Oomoo`0;Ool00004Ool000000005Ool30005Ool2002kOol00`00Oomoo`0:Ool0015oo`03
001oogoo0;]oo`03001oogoo00Uoo`004Goo00<007ooOol0_7oo00<007ooOol027oo000AOol00`00
Oomoo`2mOol00`00Oomoo`07Ool0015oo`800;moo`03001oogoo00Ioo`004Goo00<007ooOol0agoo
000AOol00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo00000goo000000030004
Ool30004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0<Mo
o`000goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomo
o`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03
001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<0
07ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0
015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .07682 m
.04057 .08686 L
.08481 .09873 L
.12636 .11078 L
.16632 .12318 L
.20885 .13729 L
.2498 .15174 L
.29331 .16803 L
.33524 .18464 L
.37557 .20144 L
.41848 .22019 L
.4598 .23908 L
.49953 .25801 L
.54183 .27896 L
.58254 .29988 L
.62583 .32291 L
.66752 .34583 L
.70762 .36854 L
.7503 .39338 L
.79139 .41794 L
.83505 .44469 L
.87712 .47107 L
.9176 .49699 L
.96065 .5251 L
1 .55126 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol00`00Oomoo`37Ool000Qoo`D000Ao
o`03001oogoo0<Moo`002Goo00@007ooOol000Aoo`<00<Moo`002Woo00<007ooOol017oo0P0000=o
o`0000000P00`goo000;Ool00`00Oomoo`03Ool00`00Oomoo`04Ool4002oOol000aoo`03001oogoo
009oo`03001oogoo00Qoo`@00;]oo`0037oo00<007ooOol00Woo00<007ooOol037oo0`00^7oo0008
Ool01@00Oomoogoo000017oo0P0047oo0`00]Goo0009Ool30005Ool00`00Oomoo`0BOol3002bOol0
015oo`03001oogoo01Eoo`<00:moo`004Goo00<007ooOol067oo1000Zgoo000AOol2000MOol3002X
Ool0015oo`03001oogoo01moo`<00:Eoo`004Goo00<007ooOol08Woo0`00XWoo000AOol00`00Oomo
o`0UOol2002POol0015oo`8002Qoo`<009eoo`004Goo00<007ooOol0:Woo0`00VWoo000:Ool30004
Ool00`00Oomoo`0]Ool3002GOol000]oo`03001oogoo00=oo`03001oogoo031oo`<009Aoo`0027oo
1@0017oo0P00=7oo0`00TGoo0008Ool01000Oomoo`001Goo00<007ooOol0=Woo0P00Sgoo0009Ool0
0`00Ool00005Ool00`00Oomoo`0hOol3002<Ool000Uoo`03001oo`0000Eoo`03001oogoo03]oo`<0
08Uoo`002Woo0P001Goo0P00?goo0P00Qgoo000;Ool00`00Oomoo`03Ool00`00Oomoo`10Ool30024
Ool0015oo`03001oogoo04=oo`80089oo`004Goo00<007ooOol0AGoo0P00P7oo000AOol20018Ool2
001nOol0015oo`03001oogoo04Uoo`<007]oo`004Goo00<007ooOol0C7oo0P00NGoo000AOol00`00
Oomoo`1>Ool2001gOol0015oo`80055oo`8007Eoo`004Goo00<007ooOol0DWoo0P00Lgoo0009Ool3
0005Ool00`00Oomoo`1DOol2001aOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1FOol2001_
Ool000Qoo`05001oogooOol00004Ool2001IOol3001/Ool000Qoo`05001oogooOol00004Ool00`00
Oomoo`1KOol2001ZOol000Qoo`@000Eoo`03001oogoo05eoo`8006Qoo`0027oo00<007ooOol01Woo
00<007ooOol0Ggoo0P00IWoo0009Ool00`00Oomoo`05Ool2001ROol3001SOol000Yoo`<000Aoo`03
001oogoo06Aoo`80065oo`004Goo00<007ooOol0IWoo0P00Ggoo000AOol00`00Oomoo`1XOol2001M
Ool0015oo`8006]oo`03001oogoo05Yoo`004Goo00<007ooOol0Jgoo0P00FWoo000AOol00`00Oomo
o`1]Ool2001HOol0015oo`03001oogoo06moo`8005Ioo`004Goo0P00LWoo0P00E7oo000AOol00`00
Oomoo`1cOol00`00Oomoo`1AOol000Uoo`<000Eoo`03001oogoo07Aoo`80055oo`0027oo00D007oo
Oomoo`0000Aoo`03001oogoo07Ioo`8004moo`0027oo00D007ooOomoo`0000Aoo`8007Uoo`8004eo
o`0027oo00D007ooOomoo`0000Aoo`03001oogoo07Yoo`8004]oo`002Goo0`001Goo00<007ooOol0
O7oo0P00BGoo0008Ool01@00Oomoogoo000017oo00<007ooOol0OWoo0P00Agoo0008Ool01@00Oomo
ogoo000017oo0P00PGoo00<007ooOol0A7oo0009Ool30005Ool00`00Oomoo`21Ool20014Ool0015o
o`03001oogoo08=oo`80049oo`004Goo00<007ooOol0QGoo00<007ooOol0?goo000AOol20027Ool2
000oOol0015oo`03001oogoo08Qoo`8003eoo`004Goo00<007ooOol0RWoo0P00>goo000AOol00`00
Oomoo`2<Ool2000iOol0015oo`8008moo`8003Moo`004Goo00<007ooOol0T7oo00<007ooOol0=7oo
00000goo000000030003Ool30005Ool00`00Oomoo`2AOol2000dOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2COol2000bOol000=oo`03001oogoo009oo`05001oogooOol0
0004Ool2002FOol00`00Oomoo`0_Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00
Oomoo`2FOol2000_Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2HOol0
0`00Oomoo`0/Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2IOol2000/
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool2002LOol2000ZOol00004Ool000000005
Ool30005Ool00`00Oomoo`2MOol00`00Oomoo`0WOol0015oo`03001oogoo09ioo`8002Moo`004Goo
00<007ooOol0X7oo00<007ooOol097oo000AOol2002ROol2000TOol0015oo`03001oogoo0:=oo`80
029oo`004Goo00<007ooOol0YGoo00<007ooOol07goo000AOol00`00Oomoo`2VOol2000OOol0015o
o`800:Uoo`03001oogoo01aoo`0000=oo`0000000`000Woo1@0017oo00<007ooOol0ZGoo0P0077oo
0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0Zgoo0P006Woo0003Ool00`00Oomo
o`04Ool00`00Oomoo`04Ool00`00Oomoo`2]Ool00`00Oomoo`0GOol000=oo`03001oogoo00Eoo`03
001oogoo00=oo`800:moo`8001Moo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0
/7oo00<007ooOol057oo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`2aOol2000D
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2cOol00`00Oomoo`0AOol0
0004Ool000000005Ool30005Ool2002eOol2000AOol0015oo`03001oogoo0;Ioo`03001oogoo00io
o`004Goo00<007ooOol0]goo0P003Woo000AOol00`00Oomoo`2iOol00`00Oomoo`0;Ool0015oo`80
0;]oo`8000]oo`004Goo00<007ooOol0_7oo00<007ooOol027oo000AOol00`00Oomoo`2mOol00`00
Oomoo`07Ool0015oo`03001oogoo0;ioo`03001oogoo00Ioo`004Goo0P00b7oo00000goo00000003
0004Ool30004Ool00`00Oomoo`37Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo
0<Moo`000goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000
Oomoo`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=o
o`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo
00<007ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37
Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .08858 m
.04057 .09928 L
.08481 .11185 L
.12636 .12451 L
.16632 .13748 L
.20885 .15214 L
.2498 .16708 L
.29331 .18385 L
.33524 .20086 L
.37557 .218 L
.41848 .23706 L
.4598 .25621 L
.49953 .27534 L
.54183 .29645 L
.58254 .31748 L
.62583 .34058 L
.66752 .36352 L
.70762 .38621 L
.7503 .41101 L
.79139 .43548 L
.83505 .46211 L
.87712 .48834 L
.9176 .51409 L
.96065 .542 L
1 .56795 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol00`00Oomoo`37Ool000Qoo`D000Ao
o`03001oogoo0<Moo`002Goo00@007ooOol000Aoo`03001oogoo0<Moo`002Woo00<007ooOol017oo
0P00b7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`37Ool000aoo`03001oogoo009oo`<00<Moo`00
37oo00<007ooOol00Woo00<007ooOol01000`goo0008Ool01@00Oomoogoo000017oo0P001Goo1000
_goo0009Ool30005Ool00`00Oomoo`08Ool4002kOol0015oo`03001oogoo00aoo`@00;Moo`004Goo
00<007ooOol047oo1000/goo000AOol2000EOol3002`Ool0015oo`03001oogoo01Moo`<00:eoo`00
4Goo00<007ooOol06Woo0`00ZWoo000AOol00`00Oomoo`0MOol2002XOol0015oo`80021oo`<00:Eo
o`004Goo00<007ooOol08Woo0`00XWoo000:Ool30004Ool00`00Oomoo`0UOol2002POol000]oo`03
001oogoo00=oo`03001oogoo02Moo`<009eoo`0027oo1@0017oo0P00:goo0`00VWoo0008Ool01000
Oomoo`001Goo00<007ooOol0;Goo0`00Ugoo0009Ool00`00Ool00005Ool00`00Oomoo`0`Ool3002D
Ool000Uoo`03001oo`0000Eoo`03001oogoo03=oo`<0095oo`002Woo0P001Goo0P00=goo0P00Sgoo
000;Ool00`00Oomoo`03Ool00`00Oomoo`0hOol3002<Ool0015oo`03001oogoo03]oo`<008Uoo`00
4Goo00<007ooOol0?Woo0P00Qgoo000AOol20011Ool20025Ool0015oo`03001oogoo049oo`03001o
ogoo089oo`004Goo00<007ooOol0@goo0P00PWoo000AOol00`00Oomoo`15Ool3001oOol0015oo`80
04Uoo`<007aoo`004Goo00<007ooOol0Bgoo0`00NGoo0009Ool30005Ool00`00Oomoo`1>Ool2001g
Ool000Qoo`05001oogooOol00004Ool00`00Oomoo`1@Ool2001eOol000Qoo`05001oogooOol00004
Ool2001COol2001cOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1DOol2001aOol000Qoo`@0
00Eoo`03001oogoo05Ioo`8006moo`0027oo00<007ooOol01Woo00<007ooOol0F7oo0P00KGoo0009
Ool00`00Oomoo`05Ool2001KOol00`00Oomoo`1ZOol000Yoo`<000Aoo`03001oogoo05]oo`8006Yo
o`004Goo00<007ooOol0GGoo0P00J7oo000AOol00`00Oomoo`1OOol2001VOol0015oo`80069oo`<0
06=oo`004Goo00<007ooOol0I7oo0P00HGoo000AOol00`00Oomoo`1VOol2001OOol0015oo`03001o
ogoo06Qoo`8005eoo`004Goo0P00Jgoo00<007ooOol0FWoo000AOol00`00Oomoo`1[Ool2001JOol0
00Uoo`<000Eoo`03001oogoo06eoo`8005Qoo`0027oo00D007ooOomoo`0000Aoo`03001oogoo06mo
o`8005Ioo`0027oo00D007ooOomoo`0000Aoo`80079oo`<005=oo`0027oo00D007ooOomoo`0000Ao
o`03001oogoo07Aoo`80055oo`002Goo0`001Goo00<007ooOol0MWoo0P00Cgoo0008Ool01@00Oomo
ogoo000017oo00<007ooOol0N7oo00<007ooOol0C7oo0008Ool01@00Oomoogoo000017oo0P00NWoo
0P00C7oo0009Ool30005Ool00`00Oomoo`1kOol2001:Ool0015oo`03001oogoo07eoo`03001oogoo
04Moo`004Goo00<007ooOol0OWoo0P00Agoo000AOol20021Ool20015Ool0015oo`03001oogoo089o
o`8004=oo`004Goo00<007ooOol0Q7oo0P00@Goo000AOol00`00Oomoo`26Ool2000oOol0015oo`80
08Uoo`03001oogoo03aoo`004Goo00<007ooOol0RGoo0P00?7oo00000goo000000030003Ool30005
Ool00`00Oomoo`2;Ool2000jOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomo
o`2=Ool00`00Oomoo`0gOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2002?Ool2000g
Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2@Ool00`00Oomoo`0dOol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2AOol2000dOol000=oo`03001o
ogoo009oo`05001oogooOol00004Ool00`00Oomoo`2COol2000bOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool2002FOol00`00Oomoo`0_Ool00004Ool000000005Ool30005Ool00`00Oomo
o`2FOol2000_Ool0015oo`03001oogoo09Qoo`03001oogoo02aoo`004Goo00<007ooOol0VGoo0P00
;7oo000AOol2002LOol2000ZOol0015oo`03001oogoo09eoo`03001oogoo02Moo`004Goo00<007oo
Ool0WWoo0P009goo000AOol00`00Oomoo`2POol00`00Oomoo`0TOol0015oo`800:9oo`8002Aoo`00
00=oo`0000000`000Woo1@0017oo00<007ooOol0Xgoo0P008Woo0003Ool00`00Oomoo`03Ool01000
Oomoo`0017oo00<007ooOol0YGoo00<007ooOol07goo0003Ool00`00Oomoo`04Ool00`00Oomoo`04
Ool00`00Oomoo`2VOol2000OOol000=oo`03001oogoo00Eoo`03001oogoo00=oo`800:Uoo`03001o
ogoo01aoo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0ZGoo0P0077oo0003Ool0
0`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`2[Ool2000JOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2]Ool00`00Oomoo`0GOol00004Ool000000005Ool30005Ool2
002_Ool2000GOol0015oo`03001oogoo0;1oo`8001Eoo`004Goo00<007ooOol0/Woo0P004goo000A
Ool00`00Oomoo`2dOol00`00Oomoo`0@Ool0015oo`800;Ioo`80011oo`004Goo00<007ooOol0]goo
0P003Woo000AOol00`00Oomoo`2iOol00`00Oomoo`0;Ool0015oo`03001oogoo0;Yoo`8000]oo`00
4Goo0P00_Goo00<007ooOol027oo00000goo000000030004Ool30004Ool00`00Oomoo`2mOol00`00
Oomoo`07Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0;ioo`03001oogoo00Io
o`000goo00<007ooOol00Woo1@0017oo00<007ooOol0agoo0003Ool00`00Oomoo`02Ool01000Oomo
o`001Goo0P00b7oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`37Ool000=oo`03
001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol017oo0P001Goo00<0
07ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol00`00Oomoo`37Ool0
015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .10014 m
.04057 .11143 L
.08481 .12462 L
.12636 .13784 L
.16632 .1513 L
.20885 .16646 L
.2498 .18183 L
.29331 .19902 L
.33524 .21639 L
.37557 .23384 L
.41848 .25319 L
.4598 .27257 L
.49953 .29188 L
.54183 .31315 L
.58254 .33428 L
.62583 .35746 L
.66752 .38045 L
.70762 .40314 L
.7503 .42791 L
.79139 .45232 L
.83505 .47886 L
.87712 .50498 L
.9176 .53059 L
.96065 .55834 L
1 .58414 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol00`00Oomoo`37Ool000Qoo`D000Ao
o`03001oogoo0<Moo`002Goo00@007ooOol000Aoo`03001oogoo0<Moo`002Woo00<007ooOol017oo
0P00b7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`37Ool000aoo`03001oogoo009oo`03001oogoo
0<Moo`0037oo00<007ooOol00Woo00<007ooOol0agoo0008Ool01@00Oomoogoo000017oo0`00agoo
0009Ool30005Ool00`00Oomoo`040033Ool0015oo`03001oogoo00Aoo`@00;moo`004Goo00<007oo
Ool027oo1000^goo000AOol2000=Ool3002hOol0015oo`03001oogoo00moo`<00;Eoo`004Goo00<0
07ooOol04Woo0`00/Woo000AOol00`00Oomoo`0EOol2002`Ool0015oo`8001Qoo`<00:eoo`004Goo
00<007ooOol06Woo0`00ZWoo000:Ool30004Ool00`00Oomoo`0MOol2002XOol000]oo`03001oogoo
00=oo`03001oogoo01moo`<00:Eoo`0027oo1@0017oo0P008goo0`00XWoo0008Ool01000Oomoo`00
1Goo00<007ooOol09Goo0P00X7oo0009Ool00`00Ool00005Ool00`00Oomoo`0WOol3002MOol000Uo
o`03001oo`0000Eoo`03001oogoo02Yoo`<009Yoo`002Woo0P001Goo0P00;Woo0`00Ugoo000;Ool0
0`00Oomoo`03Ool00`00Oomoo`0`Ool3002DOol0015oo`03001oogoo03=oo`<0095oo`004Goo00<0
07ooOol0=Woo0P00Sgoo000AOol2000iOol3002<Ool0015oo`03001oogoo03]oo`<008Uoo`004Goo
00<007ooOol0?Woo0P00Qgoo000AOol00`00Oomoo`10Ool20025Ool0015oo`8004=oo`03001oogoo
089oo`004Goo00<007ooOol0@goo0P00PWoo0009Ool30005Ool00`00Oomoo`15Ool3001oOol000Qo
o`05001oogooOol00004Ool00`00Oomoo`18Ool3001lOol000Qoo`05001oogooOol00004Ool2001<
Ool3001iOol000Qoo`05001oogooOol00004Ool00`00Oomoo`1>Ool2001gOol000Qoo`@000Eoo`03
001oogoo051oo`8007Eoo`0027oo00<007ooOol01Woo00<007ooOol0DWoo0P00Lgoo0009Ool00`00
Oomoo`05Ool2001EOol2001aOol000Yoo`<000Aoo`03001oogoo05Ioo`8006moo`004Goo00<007oo
Ool0F7oo0P00KGoo000AOol00`00Oomoo`1JOol00`00Oomoo`1ZOol0015oo`8005aoo`8006Yoo`00
4Goo00<007ooOol0GGoo0P00J7oo000AOol00`00Oomoo`1OOol2001VOol0015oo`03001oogoo065o
o`<006=oo`004Goo0P00IGoo0P00HGoo000AOol00`00Oomoo`1VOol2001OOol000Uoo`<000Eoo`03
001oogoo06Qoo`8005eoo`0027oo00D007ooOomoo`0000Aoo`03001oogoo06Yoo`03001oogoo05Yo
o`0027oo00D007ooOomoo`0000Aoo`8006aoo`8005Yoo`0027oo00D007ooOomoo`0000Aoo`03001o
ogoo06eoo`8005Qoo`002Goo0`001Goo00<007ooOol0Kgoo0P00EWoo0008Ool01@00Oomoogoo0000
17oo00<007ooOol0LGoo0`00Dgoo0008Ool01@00Oomoogoo000017oo0P00MGoo0P00DGoo0009Ool3
0005Ool00`00Oomoo`1fOol2001?Ool0015oo`03001oogoo07Qoo`03001oogoo04aoo`004Goo00<0
07ooOol0NGoo0P00C7oo000AOol2001lOol2001:Ool0015oo`03001oogoo07eoo`03001oogoo04Mo
o`004Goo00<007ooOol0OWoo0P00Agoo000AOol00`00Oomoo`20Ool20015Ool0015oo`8008=oo`80
04=oo`004Goo00<007ooOol0Q7oo0P00@Goo00000goo000000030003Ool30005Ool00`00Oomoo`26
Ool2000oOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`28Ool00`00Oomo
o`0lOol000=oo`03001oogoo009oo`05001oogooOol00004Ool2002:Ool2000lOol000=oo`03001o
ogoo009oo`05001oogooOol00004Ool00`00Oomoo`2;Ool2000jOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2=Ool00`00Oomoo`0gOol000=oo`03001oogoo009oo`05001o
ogooOol00004Ool00`00Oomoo`2>Ool2000gOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool2002AOol00`00Oomoo`0dOol00004Ool000000005Ool30005Ool00`00Oomoo`2AOol2000dOol0
015oo`03001oogoo09=oo`80039oo`004Goo00<007ooOol0UGoo00<007ooOol0;goo000AOol2002G
Ool2000_Ool0015oo`03001oogoo09Qoo`03001oogoo02aoo`004Goo00<007ooOol0VGoo0P00;7oo
000AOol00`00Oomoo`2KOol2000ZOol0015oo`8009ioo`03001oogoo02Moo`0000=oo`0000000`00
0Woo1@0017oo00<007ooOol0WWoo0P009goo0003Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<0
07ooOol0X7oo00<007ooOol097oo0003Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00Oomoo`2Q
Ool2000TOol000=oo`03001oogoo00Eoo`03001oogoo00=oo`800:Aoo`80029oo`000goo00<007oo
Ool01Woo00<007ooOol00Woo00<007ooOol0YGoo00<007ooOol07goo0003Ool00`00Oomoo`06Ool0
0`00Oomoo`02Ool00`00Oomoo`2VOol2000OOol000=oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`2XOol00`00Oomoo`0LOol00004Ool000000005Ool30005Ool2002ZOol2000LOol0
015oo`03001oogoo0:]oo`8001Yoo`004Goo00<007ooOol0[Goo00<007ooOol05goo000AOol00`00
Oomoo`2^Ool2000GOol0015oo`800;5oo`8001Eoo`004Goo00<007ooOol0/Woo0P004goo000AOol0
0`00Oomoo`2dOol00`00Oomoo`0@Ool0015oo`03001oogoo0;Eoo`80011oo`004Goo0P00^7oo0P00
3Woo00000goo000000030004Ool30004Ool00`00Oomoo`2iOol00`00Oomoo`0;Ool000=oo`03001o
ogoo00Eoo`03001oogoo00=oo`03001oogoo0;Yoo`8000]oo`000goo00<007ooOol00Woo1@0017oo
00<007ooOol0_7oo00<007ooOol027oo0003Ool00`00Oomoo`02Ool01000Oomoo`001Goo0P00_Woo
00<007ooOol01goo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`2nOol00`00Oomo
o`06Ool000=oo`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0<Moo`000goo00<007ooOol0
17oo0P001Goo00<007ooOol0agoo000017oo000000001goo00<007ooOol00goo0P00b7oo000AOol0
0`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37
Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .1115 m
.04057 .12333 L
.08481 .13708 L
.12636 .15079 L
.16632 .1647 L
.20885 .1803 L
.2498 .19607 L
.29331 .21363 L
.33524 .23133 L
.37557 .24906 L
.41848 .26867 L
.4598 .28827 L
.49953 .30775 L
.54183 .32916 L
.58254 .35041 L
.62583 .37367 L
.66752 .3967 L
.70762 .41942 L
.7503 .44417 L
.79139 .46855 L
.83505 .49502 L
.87712 .52106 L
.9176 .54657 L
.96065 .57419 L
1 .59985 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol00`00Oomoo`37Ool000Qoo`D000Ao
o`03001oogoo0<Moo`002Goo00@007ooOol000Aoo`03001oogoo0<Moo`002Woo00<007ooOol017oo
0P00b7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`37Ool000aoo`03001oogoo009oo`03001oogoo
0<Moo`0037oo00<007ooOol00Woo00<007ooOol0agoo0008Ool01@00Oomoogoo000017oo0P00b7oo
0009Ool30005Ool00`00Oomoo`37Ool0015oo`<00<Moo`004Goo00<007ooOol01000`goo000AOol2
0005Ool30030Ool0015oo`03001oogoo00Moo`<00;eoo`004Goo00<007ooOol02Woo0`00^Woo000A
Ool00`00Oomoo`0=Ool2002hOol0015oo`80011oo`<00;Eoo`004Goo00<007ooOol04Woo0`00/Woo
000:Ool30004Ool00`00Oomoo`0EOol3002_Ool000]oo`03001oogoo00=oo`03001oogoo01Qoo`@0
0:]oo`0027oo1@0017oo0P007Goo0`00Z7oo0008Ool01000Oomoo`001Goo00<007ooOol07goo0`00
YGoo0009Ool00`00Ool00005Ool00`00Oomoo`0ROol3002ROol000Uoo`03001oo`0000Eoo`03001o
ogoo02Eoo`800:1oo`002Woo0P001Goo0P00:7oo0`00WGoo000;Ool00`00Oomoo`03Ool00`00Oomo
o`0ZOol3002JOol0015oo`03001oogoo02eoo`8009Qoo`004Goo00<007ooOol0;goo0P00UWoo000A
Ool2000bOol3002COol0015oo`03001oogoo03Aoo`80095oo`004Goo00<007ooOol0=Woo0P00Sgoo
000AOol00`00Oomoo`0hOol3002<Ool0015oo`8003aoo`<008Uoo`004Goo00<007ooOol0?Woo0P00
Qgoo0009Ool30005Ool00`00Oomoo`10Ool20025Ool000Qoo`05001oogooOol00004Ool00`00Oomo
o`12Ool00`00Oomoo`22Ool000Qoo`05001oogooOol00004Ool20014Ool20022Ool000Qoo`05001o
ogooOol00004Ool00`00Oomoo`15Ool3001oOol000Qoo`@000Eoo`03001oogoo04Qoo`<007aoo`00
27oo00<007ooOol01Woo00<007ooOol0Bgoo0`00NGoo0009Ool00`00Oomoo`05Ool2001?Ool2001g
Ool000Yoo`<000Aoo`03001oogoo051oo`8007Eoo`004Goo00<007ooOol0DWoo0P00Lgoo000AOol0
0`00Oomoo`1DOol2001aOol0015oo`8005Moo`8006moo`004Goo00<007ooOol0F7oo0P00KGoo000A
Ool00`00Oomoo`1JOol00`00Oomoo`1ZOol0015oo`03001oogoo05]oo`8006Yoo`004Goo0P00GWoo
0P00J7oo000AOol00`00Oomoo`1OOol2001VOol000Uoo`<000Eoo`03001oogoo065oo`<006=oo`00
27oo00D007ooOomoo`0000Aoo`03001oogoo06Aoo`80065oo`0027oo00D007ooOomoo`0000Aoo`80
06Moo`8005moo`0027oo00D007ooOomoo`0000Aoo`03001oogoo06Qoo`8005eoo`002Goo0`001Goo
00<007ooOol0JWoo00<007ooOol0FWoo0008Ool01@00Oomoogoo000017oo00<007ooOol0Jgoo0P00
FWoo0008Ool01@00Oomoogoo000017oo0P00KWoo0P00F7oo0009Ool30005Ool00`00Oomoo`1_Ool2
001FOol0015oo`03001oogoo075oo`8005Aoo`004Goo00<007ooOol0Lgoo00<007ooOol0DGoo000A
Ool2001eOol2001AOol0015oo`03001oogoo07Ioo`8004moo`004Goo00<007ooOol0N7oo0P00CGoo
000AOol00`00Oomoo`1jOol2001;Ool0015oo`8007eoo`8004Uoo`004Goo00<007ooOol0OWoo0P00
Agoo00000goo000000030003Ool30005Ool00`00Oomoo`20Ool20015Ool000=oo`03001oogoo009o
o`05001oogooOol00004Ool00`00Oomoo`22Ool20013Ool000=oo`03001oogoo009oo`05001oogoo
Ool00004Ool20025Ool20011Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomo
o`26Ool2000oOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`28Ool00`00
Oomoo`0lOol000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`29Ool2000lOol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool2002<Ool2000jOol00004Ool000000005Ool3
0005Ool00`00Oomoo`2=Ool00`00Oomoo`0gOol0015oo`03001oogoo08ioo`8003Moo`004Goo00<0
07ooOol0T7oo00<007ooOol0=7oo000AOol2002BOol2000dOol0015oo`03001oogoo09=oo`80039o
o`004Goo00<007ooOol0UGoo00<007ooOol0;goo000AOol00`00Oomoo`2FOol2000_Ool0015oo`80
09Uoo`03001oogoo02aoo`0000=oo`0000000`000Woo1@0017oo00<007ooOol0VGoo0P00;7oo0003
Ool00`00Oomoo`03Ool01000Oomoo`0017oo00<007ooOol0Vgoo0P00:Woo0003Ool00`00Oomoo`04
Ool00`00Oomoo`04Ool00`00Oomoo`2MOol00`00Oomoo`0WOol000=oo`03001oogoo00Eoo`03001o
ogoo00=oo`8009moo`8002Moo`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0X7oo
00<007ooOol097oo0003Ool00`00Oomoo`06Ool00`00Oomoo`02Ool00`00Oomoo`2QOol2000TOol0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`2SOol2000ROol00004Ool00000
0005Ool30005Ool2002VOol00`00Oomoo`0OOol0015oo`03001oogoo0:Ioo`8001moo`004Goo00<0
07ooOol0Z7oo00<007ooOol077oo000AOol00`00Oomoo`2YOol2000LOol0015oo`800:aoo`8001Yo
o`004Goo00<007ooOol0[Goo00<007ooOol05goo000AOol00`00Oomoo`2^Ool2000GOol0015oo`03
001oogoo0;1oo`8001Eoo`004Goo0P00/goo0P004goo00000goo000000030004Ool30004Ool00`00
Oomoo`2dOol00`00Oomoo`0@Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0;Eo
o`80011oo`000goo00<007ooOol00Woo1@0017oo00<007ooOol0]goo0P003Woo0003Ool00`00Oomo
o`02Ool01000Oomoo`001Goo0P00^Woo00<007ooOol02goo0003Ool00`00Oomoo`03Ool00`00Ool0
0005Ool00`00Oomoo`2jOol2000;Ool000=oo`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo
0;aoo`03001oogoo00Qoo`000goo00<007ooOol017oo0P001Goo00<007ooOol0_Goo00<007ooOol0
1goo000017oo000000001goo00<007ooOol00goo0P00_goo00<007ooOol01Woo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000AOol00`00Oomoo`37Ool00001
\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-2.5 0.05 0 0.0412023 [
[.125 -0.0125 -12 -9 ]
[.125 -0.0125 12 0 ]
[.25 -0.0125 -6 -9 ]
[.25 -0.0125 6 0 ]
[.375 -0.0125 -12 -9 ]
[.375 -0.0125 12 0 ]
[.5 -0.0125 -6 -9 ]
[.5 -0.0125 6 0 ]
[.625 -0.0125 -12 -9 ]
[.625 -0.0125 12 0 ]
[.75 -0.0125 -6 -9 ]
[.75 -0.0125 6 0 ]
[.875 -0.0125 -12 -9 ]
[.875 -0.0125 12 0 ]
[1 -0.0125 -6 -9 ]
[1 -0.0125 6 0 ]
[-0.0125 .0824 -6 -4.5 ]
[-0.0125 .0824 0 4.5 ]
[-0.0125 .16481 -6 -4.5 ]
[-0.0125 .16481 0 4.5 ]
[-0.0125 .24721 -6 -4.5 ]
[-0.0125 .24721 0 4.5 ]
[-0.0125 .32962 -6 -4.5 ]
[-0.0125 .32962 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .49443 -12 -4.5 ]
[-0.0125 .49443 0 4.5 ]
[-0.0125 .57683 -12 -4.5 ]
[-0.0125 .57683 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 0 m
.125 .00625 L
s
[(52.5)] .125 -0.0125 0 1 Mshowa
.25 0 m
.25 .00625 L
s
[(55)] .25 -0.0125 0 1 Mshowa
.375 0 m
.375 .00625 L
s
[(57.5)] .375 -0.0125 0 1 Mshowa
.5 0 m
.5 .00625 L
s
[(60)] .5 -0.0125 0 1 Mshowa
.625 0 m
.625 .00625 L
s
[(62.5)] .625 -0.0125 0 1 Mshowa
.75 0 m
.75 .00625 L
s
[(65)] .75 -0.0125 0 1 Mshowa
.875 0 m
.875 .00625 L
s
[(67.5)] .875 -0.0125 0 1 Mshowa
1 0 m
1 .00625 L
s
[(70)] 1 -0.0125 0 1 Mshowa
.125 Mabswid
.025 0 m
.025 .00375 L
s
.05 0 m
.05 .00375 L
s
.075 0 m
.075 .00375 L
s
.1 0 m
.1 .00375 L
s
.15 0 m
.15 .00375 L
s
.175 0 m
.175 .00375 L
s
.2 0 m
.2 .00375 L
s
.225 0 m
.225 .00375 L
s
.275 0 m
.275 .00375 L
s
.3 0 m
.3 .00375 L
s
.325 0 m
.325 .00375 L
s
.35 0 m
.35 .00375 L
s
.4 0 m
.4 .00375 L
s
.425 0 m
.425 .00375 L
s
.45 0 m
.45 .00375 L
s
.475 0 m
.475 .00375 L
s
.525 0 m
.525 .00375 L
s
.55 0 m
.55 .00375 L
s
.575 0 m
.575 .00375 L
s
.6 0 m
.6 .00375 L
s
.65 0 m
.65 .00375 L
s
.675 0 m
.675 .00375 L
s
.7 0 m
.7 .00375 L
s
.725 0 m
.725 .00375 L
s
.775 0 m
.775 .00375 L
s
.8 0 m
.8 .00375 L
s
.825 0 m
.825 .00375 L
s
.85 0 m
.85 .00375 L
s
.9 0 m
.9 .00375 L
s
.925 0 m
.925 .00375 L
s
.95 0 m
.95 .00375 L
s
.975 0 m
.975 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .0824 m
.00625 .0824 L
s
[(2)] -0.0125 .0824 1 0 Mshowa
0 .16481 m
.00625 .16481 L
s
[(4)] -0.0125 .16481 1 0 Mshowa
0 .24721 m
.00625 .24721 L
s
[(6)] -0.0125 .24721 1 0 Mshowa
0 .32962 m
.00625 .32962 L
s
[(8)] -0.0125 .32962 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .49443 m
.00625 .49443 L
s
[(12)] -0.0125 .49443 1 0 Mshowa
0 .57683 m
.00625 .57683 L
s
[(14)] -0.0125 .57683 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .10301 m
.00375 .10301 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .18541 m
.00375 .18541 L
s
0 .20601 m
.00375 .20601 L
s
0 .22661 m
.00375 .22661 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .30902 m
.00375 .30902 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .51503 m
.00375 .51503 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .12267 m
.04057 .13499 L
.08481 .14924 L
.12636 .1634 L
.16632 .17772 L
.20885 .19372 L
.2498 .20984 L
.29331 .22776 L
.33524 .24575 L
.37557 .26374 L
.41848 .28359 L
.4598 .30339 L
.49953 .32303 L
.54183 .34459 L
.58254 .36594 L
.62583 .38929 L
.66752 .41237 L
.70762 .43512 L
.7503 .45987 L
.79139 .48423 L
.83505 .51066 L
.87712 .53663 L
.9176 .56207 L
.96065 .58959 L
1 .61515 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{175.312, 108.375},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  AnimationDisplayTime->0.160878,
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003K0000Qa000`40O003h00Ofgoo003KOol00=]o
o`006goo10000goo1@001Goo0P0017oo10001Goo100017oo10001Goo10001Goo00<007ooOol01Goo
0P0017oo10001Goo0`001Goo0`001Goo0`0017oo1@001Goo0P0017oo10001Goo0`001Goo100017oo
0`001Woo00<007ooOol01Goo0P0017oo10001Woo00<007ooOol017oo0`0017oo000OOol01@00Oomo
ogoo00000Woo00<007ooOol00goo0P0027oo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol0
1Woo00<007ooOol00Woo00<007ooOol01Goo0P0027oo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00<007ooOol00Woo00@007ooOol000Eoo`8000Qoo`05001oogoo
Ool00003Ool00`00Oomoo`06Ool01000Oomoo`000goo00<007ooOol00goo00<007ooOol01Goo0P00
27oo00<007ooOol00goo00<007ooOol00goo00D007ooOomoo`0000=oo`007goo00<007ooOol00Woo
00<007ooOol03goo00<007ooOol01Woo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol00goo
00<007ooOol03Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00<007ooOol00goo00<007ooOol03goo00D007ooOomoo`0000=oo`03001oogoo00Ioo`04001oogoo
0003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`04Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo000OOol00`00Oomoo`03Ool00`00Oomoo`0>Ool00`00Oomoo`06Ool00`00Oomoo`05
Ool00`00Oomoo`06Ool00`00Oomoo`03Ool00`00Oomoo`0>Ool01@00Oomoogoo00000goo00D007oo
Oomoo`0000=oo`05001oogooOol00003Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool01@00Oomoogoo
00000goo00<007ooOol01Woo00@007ooOol000=oo`03001oogoo00Aoo`03001oogoo00ioo`03001o
ogoo00Aoo`03001oogoo009oo`05001oogooOol00003Ool001]oo`@000Moo`03001oogoo00Uoo`@0
00Eoo`@000Aoo`@000Eoo`@000Ioo`03001oogoo00Yoo`@000Aoo`@000Aoo`05001oogooOol00003
Ool40008Ool00`00Oomoo`09Ool40004Ool40005Ool40003Ool40007Ool00`00Oomoo`0:Ool40007
Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo000KOol00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`08Ool00`00Oomoo`09Ool0
0`00Oomoo`05Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol017oo00<007ooOol02Goo00<007ooOol0
2Goo00<007ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo005oo`006goo00<007ooOol017oo
00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo
00Aoo`05001oogooOol0000;Ool00`00Oomoo`06Ool00`00Oomoo`04Ool01@00Oomoogoo000017oo
00<007ooOol017oo00D007ooOomoo`0000]oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Aoo`05001oogooOol0000;Ool00`00Oomoo`05Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`006goo100017oo0`0037oo10001Goo100017oo10001Goo10000goo1@00
2goo10001Woo0`0017oo0`001Woo0`0017oo0`0037oo10001Woo0`0017oo10001Goo0`000goo1@00
2goo100017oo1@0017oo0`0017oo003KOol00=]oo`00fgoo000AOoo20008Ool0015oo`03001oogoo
009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`02Ool00`00Oomoo`02Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol00Woo
00<007ooOol00Woo00<007ooOol00Woo00D007ooOomoo`0000Aoo`03001oogoo009oo`03001oogoo
009oo`03001oogoo009oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`02Ool00`00
Oomoo`06Ool0015oo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Io
o`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo01Eoo`03001oogoo00Io
o`004Goo00<007ooOol0agoo000AOol20038Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0
agoo000AOol00`00Oomoo`37Ool0015oo`800<Qoo`004Goo00<007ooOol0agoo000AOol00`00Oomo
o`37Ool0015oo`03001oogoo0<Moo`004Goo0P00b7oo000AOol00`00Oomoo`37Ool000Qoo`D000Ao
o`03001oogoo0<Moo`002Goo00@007ooOol000Aoo`03001oogoo0<Moo`002Woo00<007ooOol017oo
0P00b7oo000;Ool00`00Oomoo`03Ool00`00Oomoo`37Ool000aoo`03001oogoo009oo`03001oogoo
0<Moo`0037oo00<007ooOol00Woo00<007ooOol0agoo0008Ool01@00Oomoogoo000017oo0P00b7oo
0009Ool30005Ool00`00Oomoo`37Ool0015oo`03001oogoo0<Moo`004Goo00<007ooOol0agoo000A
Ool30037Ool0015oo`03001oogoo00@00<=oo`004Goo00<007ooOol017oo0`00`7oo000AOol00`00
Oomoo`07Ool3002mOol0015oo`8000]oo`<00;Yoo`004Goo00<007ooOol03Goo0P00^7oo000:Ool3
0004Ool00`00Oomoo`0?Ool3002eOol000]oo`03001oogoo00=oo`03001oogoo019oo`<00;9oo`00
27oo1@0017oo0P005Woo0P00/7oo0008Ool01000Oomoo`001Goo00<007ooOol05goo0`00[Goo0009
Ool00`00Ool00005Ool00`00Oomoo`0JOol3002ZOol000Uoo`03001oo`0000Eoo`03001oogoo01eo
o`800:Qoo`002Woo0P001Goo0P0087oo0`00YGoo000;Ool00`00Oomoo`03Ool00`00Oomoo`0ROol3
002ROol0015oo`03001oogoo02Eoo`800:1oo`004Goo00<007ooOol09goo0`00WGoo000AOol2000[
Ool3002JOol0015oo`03001oogoo02eoo`<009Moo`004Goo00<007ooOol0<7oo0`00U7oo000AOol0
0`00Oomoo`0cOol3002AOol0015oo`8003Moo`8008moo`004Goo00<007ooOol0>7oo0P00SGoo0009
Ool30005Ool00`00Oomoo`0jOol2002;Ool000Qoo`05001oogooOol00004Ool00`00Oomoo`0lOol2
0029Ool000Qoo`05001oogooOol00004Ool2000oOol20027Ool000Qoo`05001oogooOol00004Ool0
0`00Oomoo`10Ool30024Ool000Qoo`@000Eoo`03001oogoo04=oo`80089oo`0027oo00<007ooOol0
1Woo00<007ooOol0AGoo0P00P7oo0009Ool00`00Oomoo`05Ool20018Ool2001nOol000Yoo`<000Ao
o`03001oogoo04Uoo`<007]oo`004Goo00<007ooOol0C7oo0P00NGoo000AOol00`00Oomoo`1>Ool2
001gOol0015oo`80055oo`8007Eoo`004Goo00<007ooOol0DWoo0P00Lgoo000AOol00`00Oomoo`1D
Ool2001aOol0015oo`03001oogoo05Ioo`8006moo`004Goo0P00FGoo0P00KGoo000AOol00`00Oomo
o`1JOol00`00Oomoo`1ZOol000Uoo`<000Eoo`03001oogoo05]oo`8006Yoo`0027oo00D007ooOomo
o`0000Aoo`03001oogoo05eoo`8006Qoo`0027oo00D007ooOomoo`0000Aoo`80061oo`8006Ioo`00
27oo00D007ooOomoo`0000Aoo`03001oogoo065oo`<006=oo`002Goo0`001Goo00<007ooOol0I7oo
0P00HGoo0008Ool01@00Oomoogoo000017oo00<007ooOol0IWoo0P00Ggoo0008Ool01@00Oomoogoo
000017oo0P00JGoo0P00GGoo0009Ool30005Ool00`00Oomoo`1ZOol00`00Oomoo`1JOol0015oo`03
001oogoo06]oo`8005Yoo`004Goo00<007ooOol0KGoo0P00F7oo000AOol2001`Ool2001FOol0015o
o`03001oogoo075oo`8005Aoo`004Goo00<007ooOol0Lgoo00<007ooOol0DGoo000AOol00`00Oomo
o`1dOol2001AOol0015oo`8007Moo`8004moo`004Goo00<007ooOol0N7oo0P00CGoo00000goo0000
00030003Ool30005Ool00`00Oomoo`1jOol2001;Ool000=oo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`1lOol20019Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool2001o
Ool20017Ool000=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`20Ool20015Ool0
00=oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`22Ool20013Ool000=oo`03001o
ogoo009oo`05001oogooOol00004Ool00`00Oomoo`24Ool20011Ool000=oo`03001oogoo009oo`05
001oogooOol00004Ool20027Ool2000oOol00004Ool000000005Ool30005Ool00`00Oomoo`28Ool0
0`00Oomoo`0lOol0015oo`03001oogoo08Uoo`8003aoo`004Goo00<007ooOol0Rgoo0P00>Woo000A
Ool2002>Ool00`00Oomoo`0gOol0015oo`03001oogoo08ioo`8003Moo`004Goo00<007ooOol0T7oo
00<007ooOol0=7oo000AOol00`00Oomoo`2AOol2000dOol0015oo`8009Aoo`80039oo`0000=oo`00
00000`000Woo1@0017oo00<007ooOol0UGoo00<007ooOol0;goo0003Ool00`00Oomoo`03Ool01000
Oomoo`0017oo00<007ooOol0UWoo0P00;goo0003Ool00`00Oomoo`04Ool00`00Oomoo`04Ool00`00
Oomoo`2HOol00`00Oomoo`0/Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`8009Yoo`8002ao
o`000goo00<007ooOol01Woo00<007ooOol00Woo00<007ooOol0Vgoo0P00:Woo0003Ool00`00Oomo
o`06Ool00`00Oomoo`02Ool00`00Oomoo`2MOol00`00Oomoo`0WOol000=oo`03001oogoo009oo`05
001oogooOol00004Ool00`00Oomoo`2NOol2000WOol00004Ool000000005Ool30005Ool2002QOol0
0`00Oomoo`0TOol0015oo`03001oogoo0:5oo`8002Aoo`004Goo00<007ooOol0Xgoo0P008Woo000A
Ool00`00Oomoo`2UOol00`00Oomoo`0OOol0015oo`800:Moo`8001moo`004Goo00<007ooOol0Z7oo
00<007ooOol077oo000AOol00`00Oomoo`2YOol2000LOol0015oo`03001oogoo0:]oo`8001Yoo`00
4Goo0P00[Woo00<007ooOol05goo00000goo000000030004Ool30004Ool00`00Oomoo`2^Ool2000G
Ool000=oo`03001oogoo00Eoo`03001oogoo00=oo`03001oogoo0;1oo`8001Eoo`000goo00<007oo
Ool00Woo1@0017oo00<007ooOol0/Woo0P004goo0003Ool00`00Oomoo`02Ool01000Oomoo`001Goo
0P00]Goo00<007ooOol047oo0003Ool00`00Oomoo`03Ool00`00Ool00005Ool00`00Oomoo`2eOol2
000@Ool000=oo`03001oogoo00=oo`03001oo`0000Eoo`03001oogoo0;Moo`8000ioo`000goo00<0
07ooOol017oo0P001Goo00<007ooOol0^Goo00<007ooOol02goo000017oo000000001goo00<007oo
Ool00goo0P00^goo0P002goo000AOol00`00Oomoo`2lOol00`00Oomoo`08Ool0015oo`03001oogoo
0;eoo`03001oogoo00Moo`004Goo00<007ooOol0_Woo00<007ooOol01Woo000AOol00`00Oomoo`37
Ool00001\
\>"],
  ImageRangeCache->{{{0, 218.125}, {134.438, 0}} -> {48.1972, -1.80227, 
  0.129394, 0.157023}}]
}, Closed]]
}, Open  ]],

Cell[TextData[{
  "We can also exploit ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s 3D graphics capabilities very effectively. Here we take time and stock \
price as two coordinates, and construct a surface plot."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
bssurf = Plot3D[BlackScholesCall[x, 60., 0.29, 0.04,0, t], {x, 50, \
70},{t, 0.001, 1}, PlotRange -> {0, 15}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
5.55112e-17 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02494 .25992 -6 -2.73794 ]
[.02494 .25992 0 6.26206 ]
[.0131 .33416 -6 -2.80073 ]
[.0131 .33416 0 6.19927 ]
[.00067 .41207 -12 -2.86695 ]
[.00067 .41207 0 6.13305 ]
[-0.0124 .49393 -12 -2.93688 ]
[-0.0124 .49393 0 6.06312 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03716 .25514 m
.04327 .25274 L
s
[(0)] .02494 .25992 1 -0.39157 Mshowa
.02537 .32952 m
.03151 .3272 L
s
[(5)] .0131 .33416 1 -0.37762 Mshowa
.013 .40759 m
.01917 .40535 L
s
[(10)] .00067 .41207 1 -0.3629 Mshowa
0 .48963 m
.0062 .48748 L
s
[(15)] -0.0124 .49393 1 -0.34736 Mshowa
.125 Mabswid
.03485 .26973 m
.03852 .26831 L
s
.03251 .28447 m
.03618 .28305 L
s
.03016 .29934 m
.03383 .29793 L
s
.02778 .31436 m
.03145 .31296 L
s
.02295 .34483 m
.02663 .34345 L
s
.0205 .36029 m
.02419 .35892 L
s
.01802 .37591 m
.02172 .37454 L
s
.01552 .39167 m
.01922 .39032 L
s
.01045 .42367 m
.01416 .42234 L
s
.00788 .43991 m
.01159 .43859 L
s
.00528 .45632 m
.00899 .45501 L
s
.00265 .47289 m
.00637 .47159 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.583 .691 .916 r
.39594 .63248 .40382 .64085 .41702 .63956 .40917 .63112 Metetra
.58 .689 .916 r
.40917 .63112 .41702 .63956 .4303 .63836 .42249 .62986 Metetra
.577 .687 .916 r
.42249 .62986 .4303 .63836 .44367 .63724 .43589 .62868 Metetra
.574 .686 .916 r
.43589 .62868 .44367 .63724 .45712 .63621 .44937 .62759 Metetra
.571 .684 .917 r
.44937 .62759 .45712 .63621 .47067 .63527 .46294 .62658 Metetra
.568 .682 .917 r
.46294 .62658 .47067 .63527 .4843 .63441 .47661 .62567 Metetra
.564 .68 .917 r
.47661 .62567 .4843 .63441 .49802 .63365 .49037 .62484 Metetra
.561 .678 .917 r
.49037 .62484 .49802 .63365 .51184 .63297 .50422 .62411 Metetra
.558 .676 .917 r
.50422 .62411 .51184 .63297 .52576 .63238 .51817 .62346 Metetra
.555 .674 .917 r
.51817 .62346 .52576 .63238 .53978 .63188 .53222 .6229 Metetra
.552 .672 .918 r
.53222 .6229 .53978 .63188 .55389 .63147 .54637 .62244 Metetra
.549 .67 .918 r
.54637 .62244 .55389 .63147 .56811 .63114 .56062 .62206 Metetra
.546 .669 .918 r
.56062 .62206 .56811 .63114 .58244 .63091 .57498 .62177 Metetra
.542 .667 .918 r
.57498 .62177 .58244 .63091 .59687 .63077 .58945 .62158 Metetra
.539 .665 .918 r
.58945 .62158 .59687 .63077 .61142 .63071 .60403 .62147 Metetra
.536 .663 .918 r
.60403 .62147 .61142 .63071 .62608 .63075 .61872 .62146 Metetra
.533 .662 .919 r
.61872 .62146 .62608 .63075 .64085 .63087 .63353 .62153 Metetra
.53 .66 .919 r
.63353 .62153 .64085 .63087 .65574 .63108 .64846 .62169 Metetra
.527 .658 .919 r
.64846 .62169 .65574 .63108 .67075 .63138 .6635 .62194 Metetra
.524 .657 .919 r
.6635 .62194 .67075 .63138 .68588 .63177 .67867 .62228 Metetra
.521 .655 .919 r
.67867 .62228 .68588 .63177 .70114 .63224 .69397 .62271 Metetra
.519 .653 .919 r
.69397 .62271 .70114 .63224 .71653 .6328 .7094 .62323 Metetra
.516 .652 .919 r
.7094 .62323 .71653 .6328 .73204 .63345 .72495 .62384 Metetra
.513 .65 .919 r
.72495 .62384 .73204 .63345 .74769 .63419 .74064 .62453 Metetra
.51 .649 .92 r
.74064 .62453 .74769 .63419 .76347 .63501 .75646 .62531 Metetra
.508 .647 .92 r
.75646 .62531 .76347 .63501 .7794 .63591 .77243 .62617 Metetra
.505 .646 .92 r
.77243 .62617 .7794 .63591 .79546 .6369 .78853 .62712 Metetra
.502 .645 .92 r
.78853 .62712 .79546 .6369 .81166 .63798 .80478 .62816 Metetra
.5 .643 .92 r
.80478 .62816 .81166 .63798 .82802 .63913 .82118 .62928 Metetra
.497 .642 .92 r
.82118 .62928 .82802 .63913 .84452 .64038 .83773 .63048 Metetra
.495 .641 .92 r
.83773 .63048 .84452 .64038 .86117 .6417 .85443 .63177 Metetra
.492 .64 .921 r
.85443 .63177 .86117 .6417 .87798 .64311 .87129 .63314 Metetra
.49 .638 .921 r
.87129 .63314 .87798 .64311 .89495 .6446 .8883 .63459 Metetra
.488 .637 .921 r
.8883 .63459 .89495 .6446 .91208 .64617 .90548 .63613 Metetra
.486 .636 .921 r
.90548 .63613 .91208 .64617 .92937 .64782 .92282 .63774 Metetra
.483 .635 .921 r
.92282 .63774 .92937 .64782 .94683 .64955 .94033 .63944 Metetra
.481 .634 .921 r
.94033 .63944 .94683 .64955 .96446 .65136 .95802 .64122 Metetra
.479 .633 .921 r
.95802 .64122 .96446 .65136 .98226 .65325 .97587 .64308 Metetra
.477 .632 .922 r
.97587 .64308 .98226 .65325 1 .6535 .99391 .64501 Metetra
.585 .691 .915 r
.388 .62403 .39594 .63248 .40917 .63112 .40126 .62261 Metetra
.581 .69 .916 r
.40126 .62261 .40917 .63112 .42249 .62986 .4146 .62128 Metetra
.578 .688 .916 r
.4146 .62128 .42249 .62986 .43589 .62868 .42803 .62004 Metetra
.575 .686 .916 r
.42803 .62004 .43589 .62868 .44937 .62759 .44155 .61888 Metetra
.572 .684 .916 r
.44155 .61888 .44937 .62759 .46294 .62658 .45516 .61782 Metetra
.569 .682 .916 r
.45516 .61782 .46294 .62658 .47661 .62567 .46885 .61684 Metetra
.566 .68 .916 r
.46885 .61684 .47661 .62567 .49037 .62484 .48264 .61595 Metetra
.562 .678 .917 r
.48264 .61595 .49037 .62484 .50422 .62411 .49653 .61516 Metetra
.559 .676 .917 r
.49653 .61516 .50422 .62411 .51817 .62346 .51051 .61445 Metetra
.556 .674 .917 r
.51051 .61445 .51817 .62346 .53222 .6229 .52459 .61384 Metetra
.553 .672 .917 r
.52459 .61384 .53222 .6229 .54637 .62244 .53878 .61332 Metetra
.55 .67 .917 r
.53878 .61332 .54637 .62244 .56062 .62206 .55306 .61288 Metetra
.546 .669 .917 r
.55306 .61288 .56062 .62206 .57498 .62177 .56746 .61254 Metetra
.543 .667 .918 r
.56746 .61254 .57498 .62177 .58945 .62158 .58196 .61229 Metetra
.54 .665 .918 r
.58196 .61229 .58945 .62158 .60403 .62147 .59658 .61213 Metetra
.537 .663 .918 r
.59658 .61213 .60403 .62147 .61872 .62146 .6113 .61207 Metetra
.534 .661 .918 r
.6113 .61207 .61872 .62146 .63353 .62153 .62615 .61209 Metetra
.531 .66 .918 r
.62615 .61209 .63353 .62153 .64846 .62169 .64111 .6122 Metetra
.528 .658 .918 r
.64111 .6122 .64846 .62169 .6635 .62194 .6562 .61241 Metetra
.525 .656 .919 r
.6562 .61241 .6635 .62194 .67867 .62228 .6714 .6127 Metetra
.522 .655 .919 r
.6714 .6127 .67867 .62228 .69397 .62271 .68674 .61308 Metetra
.519 .653 .919 r
.68674 .61308 .69397 .62271 .7094 .62323 .7022 .61355 Metetra
.516 .652 .919 r
.7022 .61355 .7094 .62323 .72495 .62384 .7178 .61411 Metetra
.514 .65 .919 r
.7178 .61411 .72495 .62384 .74064 .62453 .73352 .61476 Metetra
.511 .649 .919 r
.73352 .61476 .74064 .62453 .75646 .62531 .74939 .6155 Metetra
.508 .647 .919 r
.74939 .6155 .75646 .62531 .77243 .62617 .7654 .61632 Metetra
.505 .646 .92 r
.7654 .61632 .77243 .62617 .78853 .62712 .78154 .61723 Metetra
.503 .644 .92 r
.78154 .61723 .78853 .62712 .80478 .62816 .79784 .61823 Metetra
.5 .643 .92 r
.79784 .61823 .80478 .62816 .82118 .62928 .81428 .61931 Metetra
.498 .642 .92 r
.81428 .61931 .82118 .62928 .83773 .63048 .83087 .62047 Metetra
.495 .64 .92 r
.83087 .62047 .83773 .63048 .85443 .63177 .84762 .62173 Metetra
.493 .639 .92 r
.84762 .62173 .85443 .63177 .87129 .63314 .86453 .62306 Metetra
.49 .638 .92 r
.86453 .62306 .87129 .63314 .8883 .63459 .88159 .62448 Metetra
.488 .637 .921 r
.88159 .62448 .8883 .63459 .90548 .63613 .89882 .62598 Metetra
.486 .636 .921 r
.89882 .62598 .90548 .63613 .92282 .63774 .91621 .62756 Metetra
.483 .635 .921 r
.91621 .62756 .92282 .63774 .94033 .63944 .93378 .62922 Metetra
.481 .634 .921 r
.93378 .62922 .94033 .63944 .95802 .64122 .95151 .63097 Metetra
.479 .633 .921 r
.95151 .63097 .95802 .64122 .97587 .64308 .96943 .63279 Metetra
.477 .632 .921 r
.96943 .63279 .97587 .64308 .99391 .64501 .98752 .6347 Metetra
.586 .692 .915 r
.37998 .6155 .388 .62403 .40126 .62261 .39328 .61401 Metetra
.583 .69 .915 r
.39328 .61401 .40126 .62261 .4146 .62128 .40665 .61262 Metetra
.579 .688 .915 r
.40665 .61262 .4146 .62128 .42803 .62004 .42012 .61131 Metetra
.576 .686 .915 r
.42012 .61131 .42803 .62004 .44155 .61888 .43367 .61009 Metetra
.573 .684 .916 r
.43367 .61009 .44155 .61888 .45516 .61782 .4473 .60896 Metetra
.57 .682 .916 r
.4473 .60896 .45516 .61782 .46885 .61684 .46103 .60792 Metetra
.567 .68 .916 r
.46103 .60792 .46885 .61684 .48264 .61595 .47485 .60698 Metetra
.563 .678 .916 r
.47485 .60698 .48264 .61595 .49653 .61516 .48877 .60612 Metetra
.56 .676 .916 r
.48877 .60612 .49653 .61516 .51051 .61445 .50279 .60535 Metetra
.557 .674 .917 r
.50279 .60535 .51051 .61445 .52459 .61384 .5169 .60468 Metetra
.554 .672 .917 r
.5169 .60468 .52459 .61384 .53878 .61332 .53112 .6041 Metetra
.55 .67 .917 r
.53112 .6041 .53878 .61332 .55306 .61288 .54544 .60361 Metetra
.547 .669 .917 r
.54544 .60361 .55306 .61288 .56746 .61254 .55987 .60321 Metetra
.544 .667 .917 r
.55987 .60321 .56746 .61254 .58196 .61229 .57441 .60291 Metetra
.541 .665 .917 r
.57441 .60291 .58196 .61229 .59658 .61213 .58906 .6027 Metetra
.538 .663 .917 r
.58906 .6027 .59658 .61213 .6113 .61207 .60382 .60258 Metetra
.535 .661 .918 r
.60382 .60258 .6113 .61207 .62615 .61209 .6187 .60255 Metetra
.532 .66 .918 r
.6187 .60255 .62615 .61209 .64111 .6122 .6337 .60261 Metetra
.529 .658 .918 r
.6337 .60261 .64111 .6122 .6562 .61241 .64882 .60277 Metetra
.526 .656 .918 r
.64882 .60277 .6562 .61241 .6714 .6127 .66407 .60301 Metetra
.523 .655 .918 r
.66407 .60301 .6714 .6127 .68674 .61308 .67944 .60335 Metetra
.52 .653 .918 r
.67944 .60335 .68674 .61308 .7022 .61355 .69494 .60377 Metetra
.517 .651 .919 r
.69494 .60377 .7022 .61355 .7178 .61411 .71057 .60429 Metetra
.514 .65 .919 r
.71057 .60429 .7178 .61411 .73352 .61476 .72634 .60489 Metetra
.511 .648 .919 r
.72634 .60489 .73352 .61476 .74939 .6155 .74225 .60558 Metetra
.508 .647 .919 r
.74225 .60558 .74939 .6155 .7654 .61632 .7583 .60636 Metetra
.506 .646 .919 r
.7583 .60636 .7654 .61632 .78154 .61723 .77449 .60723 Metetra
.503 .644 .919 r
.77449 .60723 .78154 .61723 .79784 .61823 .79083 .60819 Metetra
.5 .643 .919 r
.79083 .60819 .79784 .61823 .81428 .61931 .80732 .60923 Metetra
.498 .641 .92 r
.80732 .60923 .81428 .61931 .83087 .62047 .82395 .61036 Metetra
.495 .64 .92 r
.82395 .61036 .83087 .62047 .84762 .62173 .84075 .61157 Metetra
.493 .639 .92 r
.84075 .61157 .84762 .62173 .86453 .62306 .8577 .61287 Metetra
.49 .638 .92 r
.8577 .61287 .86453 .62306 .88159 .62448 .87482 .61425 Metetra
.488 .637 .92 r
.87482 .61425 .88159 .62448 .89882 .62598 .89209 .61571 Metetra
.486 .635 .92 r
.89209 .61571 .89882 .62598 .91621 .62756 .90954 .61726 Metetra
.484 .634 .921 r
.90954 .61726 .91621 .62756 .93378 .62922 .92716 .61889 Metetra
.481 .633 .921 r
.92716 .61889 .93378 .62922 .95151 .63097 .94495 .6206 Metetra
.479 .632 .921 r
.94495 .6206 .95151 .63097 .96943 .63279 .96292 .62239 Metetra
.477 .631 .921 r
.96292 .62239 .96943 .63279 .98752 .6347 .98107 .62427 Metetra
.587 .692 .914 r
.3719 .60689 .37998 .6155 .39328 .61401 .38523 .60534 Metetra
.584 .69 .915 r
.38523 .60534 .39328 .61401 .40665 .61262 .39864 .60387 Metetra
.581 .688 .915 r
.39864 .60387 .40665 .61262 .42012 .61131 .41213 .6025 Metetra
.577 .686 .915 r
.41213 .6025 .42012 .61131 .43367 .61009 .42571 .60121 Metetra
.574 .684 .915 r
.42571 .60121 .43367 .61009 .4473 .60896 .43938 .60002 Metetra
.571 .682 .915 r
.43938 .60002 .4473 .60896 .46103 .60792 .45314 .59892 Metetra
.568 .68 .916 r
.45314 .59892 .46103 .60792 .47485 .60698 .467 .59791 Metetra
.564 .678 .916 r
.467 .59791 .47485 .60698 .48877 .60612 .48095 .59699 Metetra
.561 .676 .916 r
.48095 .59699 .48877 .60612 .50279 .60535 .495 .59616 Metetra
.558 .674 .916 r
.495 .59616 .50279 .60535 .5169 .60468 .50915 .59543 Metetra
.555 .672 .916 r
.50915 .59543 .5169 .60468 .53112 .6041 .5234 .59479 Metetra
.551 .67 .916 r
.5234 .59479 .53112 .6041 .54544 .60361 .53775 .59424 Metetra
.548 .669 .917 r
.53775 .59424 .54544 .60361 .55987 .60321 .55221 .59379 Metetra
.545 .667 .917 r
.55221 .59379 .55987 .60321 .57441 .60291 .56679 .59343 Metetra
.542 .665 .917 r
.56679 .59343 .57441 .60291 .58906 .6027 .58147 .59316 Metetra
.539 .663 .917 r
.58147 .59316 .58906 .6027 .60382 .60258 .59627 .59299 Metetra
.536 .661 .917 r
.59627 .59299 .60382 .60258 .6187 .60255 .61119 .5929 Metetra
.532 .66 .917 r
.61119 .5929 .6187 .60255 .6337 .60261 .62622 .59292 Metetra
.529 .658 .917 r
.62622 .59292 .6337 .60261 .64882 .60277 .64138 .59302 Metetra
.526 .656 .918 r
.64138 .59302 .64882 .60277 .66407 .60301 .65666 .59321 Metetra
.523 .654 .918 r
.65666 .59321 .66407 .60301 .67944 .60335 .67207 .5935 Metetra
.52 .653 .918 r
.67207 .5935 .67944 .60335 .69494 .60377 .68761 .59388 Metetra
.517 .651 .918 r
.68761 .59388 .69494 .60377 .71057 .60429 .70329 .59435 Metetra
.515 .65 .918 r
.70329 .59435 .71057 .60429 .72634 .60489 .7191 .59491 Metetra
.512 .648 .918 r
.7191 .59491 .72634 .60489 .74225 .60558 .73505 .59556 Metetra
.509 .647 .919 r
.73505 .59556 .74225 .60558 .7583 .60636 .75114 .5963 Metetra
.506 .645 .919 r
.75114 .5963 .7583 .60636 .77449 .60723 .76737 .59712 Metetra
.503 .644 .919 r
.76737 .59712 .77449 .60723 .79083 .60819 .78375 .59804 Metetra
.501 .642 .919 r
.78375 .59804 .79083 .60819 .80732 .60923 .80028 .59904 Metetra
.498 .641 .919 r
.80028 .59904 .80732 .60923 .82395 .61036 .81697 .60013 Metetra
.496 .64 .919 r
.81697 .60013 .82395 .61036 .84075 .61157 .83381 .6013 Metetra
.493 .639 .92 r
.83381 .6013 .84075 .61157 .8577 .61287 .85081 .60256 Metetra
.491 .637 .92 r
.85081 .60256 .8577 .61287 .87482 .61425 .86797 .6039 Metetra
.488 .636 .92 r
.86797 .6039 .87482 .61425 .89209 .61571 .8853 .60533 Metetra
.486 .635 .92 r
.8853 .60533 .89209 .61571 .90954 .61726 .9028 .60685 Metetra
.484 .634 .92 r
.9028 .60685 .90954 .61726 .92716 .61889 .92047 .60844 Metetra
.481 .633 .92 r
.92047 .60844 .92716 .61889 .94495 .6206 .93832 .61012 Metetra
.479 .632 .921 r
.93832 .61012 .94495 .6206 .96292 .62239 .95634 .61188 Metetra
.477 .631 .921 r
.95634 .61188 .96292 .62239 .98107 .62427 .97455 .61372 Metetra
.588 .692 .914 r
.36375 .5982 .3719 .60689 .38523 .60534 .37711 .59658 Metetra
.585 .69 .914 r
.37711 .59658 .38523 .60534 .39864 .60387 .39055 .59505 Metetra
.582 .688 .914 r
.39055 .59505 .39864 .60387 .41213 .6025 .40408 .5936 Metetra
.579 .686 .915 r
.40408 .5936 .41213 .6025 .42571 .60121 .41769 .59225 Metetra
.575 .684 .915 r
.41769 .59225 .42571 .60121 .43938 .60002 .43139 .59099 Metetra
.572 .682 .915 r
.43139 .59099 .43938 .60002 .45314 .59892 .44519 .58982 Metetra
.569 .68 .915 r
.44519 .58982 .45314 .59892 .467 .59791 .45907 .58875 Metetra
.566 .678 .915 r
.45907 .58875 .467 .59791 .48095 .59699 .47306 .58776 Metetra
.562 .676 .915 r
.47306 .58776 .48095 .59699 .495 .59616 .48714 .58687 Metetra
.559 .674 .916 r
.48714 .58687 .495 .59616 .50915 .59543 .50132 .58608 Metetra
.556 .672 .916 r
.50132 .58608 .50915 .59543 .5234 .59479 .51561 .58538 Metetra
.552 .67 .916 r
.51561 .58538 .5234 .59479 .53775 .59424 .53 .58477 Metetra
.549 .669 .916 r
.53 .58477 .53775 .59424 .55221 .59379 .54449 .58426 Metetra
.546 .667 .916 r
.54449 .58426 .55221 .59379 .56679 .59343 .5591 .58384 Metetra
.543 .665 .916 r
.5591 .58384 .56679 .59343 .58147 .59316 .57382 .58352 Metetra
.539 .663 .917 r
.57382 .58352 .58147 .59316 .59627 .59299 .58865 .58329 Metetra
.536 .661 .917 r
.58865 .58329 .59627 .59299 .61119 .5929 .6036 .58315 Metetra
.533 .659 .917 r
.6036 .58315 .61119 .5929 .62622 .59292 .61868 .58311 Metetra
.53 .658 .917 r
.61868 .58311 .62622 .59292 .64138 .59302 .63387 .58316 Metetra
.527 .656 .917 r
.63387 .58316 .64138 .59302 .65666 .59321 .64919 .58331 Metetra
.524 .654 .917 r
.64919 .58331 .65666 .59321 .67207 .5935 .66464 .58355 Metetra
.521 .653 .918 r
.66464 .58355 .67207 .5935 .68761 .59388 .68022 .58388 Metetra
.518 .651 .918 r
.68022 .58388 .68761 .59388 .70329 .59435 .69593 .5843 Metetra
.515 .649 .918 r
.69593 .5843 .70329 .59435 .7191 .59491 .71178 .58481 Metetra
.512 .648 .918 r
.71178 .58481 .7191 .59491 .73505 .59556 .72777 .58542 Metetra
.509 .646 .918 r
.72777 .58542 .73505 .59556 .75114 .5963 .74391 .58611 Metetra
.507 .645 .918 r
.74391 .58611 .75114 .5963 .76737 .59712 .76018 .5869 Metetra
.504 .643 .918 r
.76018 .5869 .76737 .59712 .78375 .59804 .77661 .58777 Metetra
.501 .642 .919 r
.77661 .58777 .78375 .59804 .80028 .59904 .79318 .58873 Metetra
.498 .641 .919 r
.79318 .58873 .80028 .59904 .81697 .60013 .80992 .58978 Metetra
.496 .639 .919 r
.80992 .58978 .81697 .60013 .83381 .6013 .8268 .59091 Metetra
.493 .638 .919 r
.8268 .59091 .83381 .6013 .85081 .60256 .84385 .59214 Metetra
.491 .637 .919 r
.84385 .59214 .85081 .60256 .86797 .6039 .86107 .59344 Metetra
.488 .636 .92 r
.86107 .59344 .86797 .6039 .8853 .60533 .87845 .59484 Metetra
.486 .635 .92 r
.87845 .59484 .8853 .60533 .9028 .60685 .896 .59631 Metetra
.484 .634 .92 r
.896 .59631 .9028 .60685 .92047 .60844 .91372 .59787 Metetra
.481 .632 .92 r
.91372 .59787 .92047 .60844 .93832 .61012 .93162 .59952 Metetra
.479 .631 .92 r
.93162 .59952 .93832 .61012 .95634 .61188 .9497 .60124 Metetra
.477 .63 .92 r
.9497 .60124 .95634 .61188 .97455 .61372 .96796 .60305 Metetra
.59 .693 .913 r
.35553 .58943 .36375 .5982 .37711 .59658 .36892 .58774 Metetra
.586 .691 .914 r
.36892 .58774 .37711 .59658 .39055 .59505 .38239 .58613 Metetra
.583 .689 .914 r
.38239 .58613 .39055 .59505 .40408 .5936 .39595 .58462 Metetra
.58 .687 .914 r
.39595 .58462 .40408 .5936 .41769 .59225 .4096 .5832 Metetra
.577 .684 .914 r
.4096 .5832 .41769 .59225 .43139 .59099 .42333 .58187 Metetra
.573 .682 .914 r
.42333 .58187 .43139 .59099 .44519 .58982 .43716 .58063 Metetra
.57 .68 .915 r
.43716 .58063 .44519 .58982 .45907 .58875 .45108 .57949 Metetra
.567 .678 .915 r
.45108 .57949 .45907 .58875 .47306 .58776 .4651 .57844 Metetra
.563 .676 .915 r
.4651 .57844 .47306 .58776 .48714 .58687 .47921 .57749 Metetra
.56 .674 .915 r
.47921 .57749 .48714 .58687 .50132 .58608 .49343 .57663 Metetra
.557 .672 .915 r
.49343 .57663 .50132 .58608 .51561 .58538 .50775 .57587 Metetra
.553 .67 .915 r
.50775 .57587 .51561 .58538 .53 .58477 .52217 .5752 Metetra
.55 .669 .916 r
.52217 .5752 .53 .58477 .54449 .58426 .5367 .57463 Metetra
.547 .667 .916 r
.5367 .57463 .54449 .58426 .5591 .58384 .55134 .57415 Metetra
.544 .665 .916 r
.55134 .57415 .5591 .58384 .57382 .58352 .5661 .57377 Metetra
.54 .663 .916 r
.5661 .57377 .57382 .58352 .58865 .58329 .58097 .57349 Metetra
.537 .661 .916 r
.58097 .57349 .58865 .58329 .6036 .58315 .59596 .5733 Metetra
.534 .659 .916 r
.59596 .5733 .6036 .58315 .61868 .58311 .61106 .5732 Metetra
.531 .657 .917 r
.61106 .5732 .61868 .58311 .63387 .58316 .6263 .5732 Metetra
.528 .656 .917 r
.6263 .5732 .63387 .58316 .64919 .58331 .64165 .57329 Metetra
.525 .654 .917 r
.64165 .57329 .64919 .58331 .66464 .58355 .65714 .57348 Metetra
.521 .652 .917 r
.65714 .57348 .66464 .58355 .68022 .58388 .67276 .57376 Metetra
.518 .651 .917 r
.67276 .57376 .68022 .58388 .69593 .5843 .68851 .57414 Metetra
.516 .649 .917 r
.68851 .57414 .69593 .5843 .71178 .58481 .7044 .57461 Metetra
.513 .648 .918 r
.7044 .57461 .71178 .58481 .72777 .58542 .72043 .57516 Metetra
.51 .646 .918 r
.72043 .57516 .72777 .58542 .74391 .58611 .73661 .57581 Metetra
.507 .645 .918 r
.73661 .57581 .74391 .58611 .76018 .5869 .75293 .57655 Metetra
.504 .643 .918 r
.75293 .57655 .76018 .5869 .77661 .58777 .7694 .57739 Metetra
.501 .642 .918 r
.7694 .57739 .77661 .58777 .79318 .58873 .78602 .5783 Metetra
.499 .64 .918 r
.78602 .5783 .79318 .58873 .80992 .58978 .8028 .57931 Metetra
.496 .639 .919 r
.8028 .57931 .80992 .58978 .8268 .59091 .81973 .58041 Metetra
.493 .638 .919 r
.81973 .58041 .8268 .59091 .84385 .59214 .83683 .58159 Metetra
.491 .637 .919 r
.83683 .58159 .84385 .59214 .86107 .59344 .85409 .58286 Metetra
.488 .635 .919 r
.85409 .58286 .86107 .59344 .87845 .59484 .87152 .58422 Metetra
.486 .634 .919 r
.87152 .58422 .87845 .59484 .896 .59631 .88912 .58566 Metetra
.484 .633 .92 r
.88912 .58566 .896 .59631 .91372 .59787 .9069 .58718 Metetra
.481 .632 .92 r
.9069 .58718 .91372 .59787 .93162 .59952 .92486 .58879 Metetra
.479 .631 .92 r
.92486 .58879 .93162 .59952 .9497 .60124 .94299 .59048 Metetra
.477 .63 .92 r
.94299 .59048 .9497 .60124 .96796 .60305 .96131 .59226 Metetra
.591 .693 .913 r
.34724 .58058 .35553 .58943 .36892 .58774 .36066 .57881 Metetra
.588 .691 .913 r
.36066 .57881 .36892 .58774 .38239 .58613 .37417 .57713 Metetra
.584 .689 .913 r
.37417 .57713 .38239 .58613 .39595 .58462 .38776 .57555 Metetra
.581 .687 .914 r
.38776 .57555 .39595 .58462 .4096 .5832 .40144 .57406 Metetra
.578 .685 .914 r
.40144 .57406 .4096 .5832 .42333 .58187 .41521 .57266 Metetra
.575 .683 .914 r
.41521 .57266 .42333 .58187 .43716 .58063 .42907 .57135 Metetra
.571 .681 .914 r
.42907 .57135 .43716 .58063 .45108 .57949 .44302 .57014 Metetra
.568 .679 .914 r
.44302 .57014 .45108 .57949 .4651 .57844 .45707 .56902 Metetra
.564 .676 .914 r
.45707 .56902 .4651 .57844 .47921 .57749 .47122 .56801 Metetra
.561 .674 .915 r
.47122 .56801 .47921 .57749 .49343 .57663 .48547 .56708 Metetra
.558 .672 .915 r
.48547 .56708 .49343 .57663 .50775 .57587 .49982 .56625 Metetra
.554 .67 .915 r
.49982 .56625 .50775 .57587 .52217 .5752 .51428 .56552 Metetra
.551 .668 .915 r
.51428 .56552 .52217 .5752 .5367 .57463 .52884 .56489 Metetra
.548 .667 .915 r
.52884 .56489 .5367 .57463 .55134 .57415 .54352 .56436 Metetra
.544 .665 .915 r
.54352 .56436 .55134 .57415 .5661 .57377 .55831 .56392 Metetra
.541 .663 .916 r
.55831 .56392 .5661 .57377 .58097 .57349 .57321 .56357 Metetra
.538 .661 .916 r
.57321 .56357 .58097 .57349 .59596 .5733 .58824 .56333 Metetra
.535 .659 .916 r
.58824 .56333 .59596 .5733 .61106 .5732 .60338 .56318 Metetra
.531 .657 .916 r
.60338 .56318 .61106 .5732 .6263 .5732 .61865 .56312 Metetra
.528 .655 .916 r
.61865 .56312 .6263 .5732 .64165 .57329 .63405 .56317 Metetra
.525 .654 .916 r
.63405 .56317 .64165 .57329 .65714 .57348 .64957 .5633 Metetra
.522 .652 .917 r
.64957 .5633 .65714 .57348 .67276 .57376 .66523 .56354 Metetra
.519 .65 .917 r
.66523 .56354 .67276 .57376 .68851 .57414 .68102 .56386 Metetra
.516 .649 .917 r
.68102 .56386 .68851 .57414 .7044 .57461 .69695 .56428 Metetra
.513 .647 .917 r
.69695 .56428 .7044 .57461 .72043 .57516 .71302 .56479 Metetra
.51 .646 .917 r
.71302 .56479 .72043 .57516 .73661 .57581 .72924 .5654 Metetra
.507 .644 .917 r
.72924 .5654 .73661 .57581 .75293 .57655 .7456 .56609 Metetra
.504 .643 .918 r
.7456 .56609 .75293 .57655 .7694 .57739 .76212 .56688 Metetra
.502 .641 .918 r
.76212 .56688 .7694 .57739 .78602 .5783 .77878 .56776 Metetra
.499 .64 .918 r
.77878 .56776 .78602 .5783 .8028 .57931 .79561 .56873 Metetra
.496 .639 .918 r
.79561 .56873 .8028 .57931 .81973 .58041 .81259 .56978 Metetra
.494 .637 .918 r
.81259 .56978 .81973 .58041 .83683 .58159 .82974 .57093 Metetra
.491 .636 .919 r
.82974 .57093 .83683 .58159 .85409 .58286 .84705 .57216 Metetra
.489 .635 .919 r
.84705 .57216 .85409 .58286 .87152 .58422 .86453 .57348 Metetra
.486 .634 .919 r
.86453 .57348 .87152 .58422 .88912 .58566 .88218 .57488 Metetra
.484 .633 .919 r
.88218 .57488 .88912 .58566 .9069 .58718 .90001 .57637 Metetra
.481 .632 .919 r
.90001 .57637 .9069 .58718 .92486 .58879 .91802 .57794 Metetra
.479 .631 .92 r
.91802 .57794 .92486 .58879 .94299 .59048 .93622 .5796 Metetra
.477 .63 .92 r
.93622 .5796 .94299 .59048 .96131 .59226 .9546 .58134 Metetra
.592 .693 .912 r
.33887 .57164 .34724 .58058 .36066 .57881 .35233 .5698 Metetra
.589 .691 .913 r
.35233 .5698 .36066 .57881 .37417 .57713 .36587 .56805 Metetra
.586 .689 .913 r
.36587 .56805 .37417 .57713 .38776 .57555 .37949 .56639 Metetra
.582 .687 .913 r
.37949 .56639 .38776 .57555 .40144 .57406 .39321 .56482 Metetra
.579 .685 .913 r
.39321 .56482 .40144 .57406 .41521 .57266 .40701 .56335 Metetra
.576 .683 .913 r
.40701 .56335 .41521 .57266 .42907 .57135 .4209 .56197 Metetra
.572 .681 .913 r
.4209 .56197 .42907 .57135 .44302 .57014 .43489 .56069 Metetra
.569 .679 .914 r
.43489 .56069 .44302 .57014 .45707 .56902 .44897 .55951 Metetra
.566 .677 .914 r
.44897 .55951 .45707 .56902 .47122 .56801 .46315 .55842 Metetra
.562 .674 .914 r
.46315 .55842 .47122 .56801 .48547 .56708 .47744 .55743 Metetra
.559 .672 .914 r
.47744 .55743 .48547 .56708 .49982 .56625 .49182 .55654 Metetra
.555 .67 .914 r
.49182 .55654 .49982 .56625 .51428 .56552 .50631 .55575 Metetra
.552 .668 .915 r
.50631 .55575 .51428 .56552 .52884 .56489 .52091 .55505 Metetra
.549 .666 .915 r
.52091 .55505 .52884 .56489 .54352 .56436 .53562 .55445 Metetra
.545 .665 .915 r
.53562 .55445 .54352 .56436 .55831 .56392 .55045 .55395 Metetra
.542 .663 .915 r
.55045 .55395 .55831 .56392 .57321 .56357 .56539 .55355 Metetra
.539 .661 .915 r
.56539 .55355 .57321 .56357 .58824 .56333 .58045 .55325 Metetra
.535 .659 .915 r
.58045 .55325 .58824 .56333 .60338 .56318 .59563 .55304 Metetra
.532 .657 .916 r
.59563 .55304 .60338 .56318 .61865 .56312 .61094 .55293 Metetra
.529 .655 .916 r
.61094 .55293 .61865 .56312 .63405 .56317 .62637 .55292 Metetra
.526 .653 .916 r
.62637 .55292 .63405 .56317 .64957 .5633 .64193 .55301 Metetra
.523 .652 .916 r
.64193 .55301 .64957 .5633 .66523 .56354 .65763 .55319 Metetra
.52 .65 .916 r
.65763 .55319 .66523 .56354 .68102 .56386 .67346 .55346 Metetra
.516 .648 .916 r
.67346 .55346 .68102 .56386 .69695 .56428 .68943 .55384 Metetra
.513 .647 .917 r
.68943 .55384 .69695 .56428 .71302 .56479 .70555 .5543 Metetra
.511 .645 .917 r
.70555 .5543 .71302 .56479 .72924 .5654 .7218 .55486 Metetra
.508 .644 .917 r
.7218 .55486 .72924 .5654 .7456 .56609 .73821 .55551 Metetra
.505 .642 .917 r
.73821 .55551 .7456 .56609 .76212 .56688 .75477 .55626 Metetra
.502 .641 .917 r
.75477 .55626 .76212 .56688 .77878 .56776 .77148 .55709 Metetra
.499 .64 .918 r
.77148 .55709 .77878 .56776 .79561 .56873 .78835 .55802 Metetra
.496 .638 .918 r
.78835 .55802 .79561 .56873 .81259 .56978 .80538 .55903 Metetra
.494 .637 .918 r
.80538 .55903 .81259 .56978 .82974 .57093 .82257 .56014 Metetra
.491 .636 .918 r
.82257 .56014 .82974 .57093 .84705 .57216 .83993 .56133 Metetra
.489 .634 .918 r
.83993 .56133 .84705 .57216 .86453 .57348 .85747 .56261 Metetra
.486 .633 .919 r
.85747 .56261 .86453 .57348 .88218 .57488 .87517 .56398 Metetra
.484 .632 .919 r
.87517 .56398 .88218 .57488 .90001 .57637 .89306 .56543 Metetra
.481 .631 .919 r
.89306 .56543 .90001 .57637 .91802 .57794 .91112 .56697 Metetra
.479 .63 .919 r
.91112 .56697 .91802 .57794 .93622 .5796 .92937 .56859 Metetra
.477 .629 .919 r
.92937 .56859 .93622 .5796 .9546 .58134 .94781 .5703 Metetra
.594 .694 .912 r
.33044 .56262 .33887 .57164 .35233 .5698 .34393 .5607 Metetra
.59 .691 .912 r
.34393 .5607 .35233 .5698 .36587 .56805 .3575 .55887 Metetra
.587 .689 .912 r
.3575 .55887 .36587 .56805 .37949 .56639 .37116 .55714 Metetra
.584 .687 .912 r
.37116 .55714 .37949 .56639 .39321 .56482 .3849 .5555 Metetra
.58 .685 .913 r
.3849 .5555 .39321 .56482 .40701 .56335 .39874 .55395 Metetra
.577 .683 .913 r
.39874 .55395 .40701 .56335 .4209 .56197 .41267 .5525 Metetra
.574 .681 .913 r
.41267 .5525 .4209 .56197 .43489 .56069 .42669 .55115 Metetra
.57 .679 .913 r
.42669 .55115 .43489 .56069 .44897 .55951 .4408 .54989 Metetra
.567 .677 .913 r
.4408 .54989 .44897 .55951 .46315 .55842 .45502 .54874 Metetra
.563 .675 .913 r
.45502 .54874 .46315 .55842 .47744 .55743 .46933 .54768 Metetra
.56 .672 .914 r
.46933 .54768 .47744 .55743 .49182 .55654 .48375 .54672 Metetra
.556 .67 .914 r
.48375 .54672 .49182 .55654 .50631 .55575 .49828 .54586 Metetra
.553 .668 .914 r
.49828 .54586 .50631 .55575 .52091 .55505 .51291 .5451 Metetra
.55 .666 .914 r
.51291 .5451 .52091 .55505 .53562 .55445 .52766 .54444 Metetra
.546 .664 .914 r
.52766 .54444 .53562 .55445 .55045 .55395 .54252 .54388 Metetra
.543 .662 .914 r
.54252 .54388 .55045 .55395 .56539 .55355 .5575 .54342 Metetra
.54 .661 .915 r
.5575 .54342 .56539 .55355 .58045 .55325 .57259 .54305 Metetra
.536 .659 .915 r
.57259 .54305 .58045 .55325 .59563 .55304 .58781 .54279 Metetra
.533 .657 .915 r
.58781 .54279 .59563 .55304 .61094 .55293 .60315 .54263 Metetra
.53 .655 .915 r
.60315 .54263 .61094 .55293 .62637 .55292 .61862 .54256 Metetra
.526 .653 .915 r
.61862 .54256 .62637 .55292 .64193 .55301 .63422 .54259 Metetra
.523 .651 .916 r
.63422 .54259 .64193 .55301 .65763 .55319 .64996 .54272 Metetra
.52 .65 .916 r
.64996 .54272 .65763 .55319 .67346 .55346 .66583 .54295 Metetra
.517 .648 .916 r
.66583 .54295 .67346 .55346 .68943 .55384 .68184 .54327 Metetra
.514 .647 .916 r
.68184 .54327 .68943 .55384 .70555 .5543 .698 .54369 Metetra
.511 .645 .916 r
.698 .54369 .70555 .5543 .7218 .55486 .7143 .5442 Metetra
.508 .643 .917 r
.7143 .5442 .7218 .55486 .73821 .55551 .73075 .54481 Metetra
.505 .642 .917 r
.73075 .54481 .73821 .55551 .75477 .55626 .74735 .54551 Metetra
.502 .641 .917 r
.74735 .54551 .75477 .55626 .77148 .55709 .7641 .5463 Metetra
.499 .639 .917 r
.7641 .5463 .77148 .55709 .78835 .55802 .78102 .54718 Metetra
.497 .638 .917 r
.78102 .54718 .78835 .55802 .80538 .55903 .7981 .54816 Metetra
.494 .636 .918 r
.7981 .54816 .80538 .55903 .82257 .56014 .81534 .54922 Metetra
.491 .635 .918 r
.81534 .54922 .82257 .56014 .83993 .56133 .83275 .55038 Metetra
.489 .634 .918 r
.83275 .55038 .83993 .56133 .85747 .56261 .85033 .55162 Metetra
.486 .633 .918 r
.85033 .55162 .85747 .56261 .87517 .56398 .86809 .55295 Metetra
.484 .632 .918 r
.86809 .55295 .87517 .56398 .89306 .56543 .88603 .55437 Metetra
.481 .631 .919 r
.88603 .55437 .89306 .56543 .91112 .56697 .90415 .55587 Metetra
.479 .63 .919 r
.90415 .55587 .91112 .56697 .92937 .56859 .92246 .55746 Metetra
.477 .629 .919 r
.92246 .55746 .92937 .56859 .94781 .5703 .94095 .55913 Metetra
.595 .694 .911 r
.32193 .55351 .33044 .56262 .34393 .5607 .33545 .55151 Metetra
.592 .692 .912 r
.33545 .55151 .34393 .5607 .3575 .55887 .34906 .5496 Metetra
.589 .69 .912 r
.34906 .5496 .3575 .55887 .37116 .55714 .36275 .54779 Metetra
.585 .687 .912 r
.36275 .54779 .37116 .55714 .3849 .5555 .37653 .54607 Metetra
.582 .685 .912 r
.37653 .54607 .3849 .5555 .39874 .55395 .3904 .54445 Metetra
.578 .683 .912 r
.3904 .54445 .39874 .55395 .41267 .5525 .40436 .54293 Metetra
.575 .681 .912 r
.40436 .54293 .41267 .5525 .42669 .55115 .41841 .5415 Metetra
.571 .679 .913 r
.41841 .5415 .42669 .55115 .4408 .54989 .43256 .54017 Metetra
.568 .677 .913 r
.43256 .54017 .4408 .54989 .45502 .54874 .44681 .53895 Metetra
.564 .675 .913 r
.44681 .53895 .45502 .54874 .46933 .54768 .46116 .53782 Metetra
.561 .672 .913 r
.46116 .53782 .46933 .54768 .48375 .54672 .47562 .53679 Metetra
.558 .67 .913 r
.47562 .53679 .48375 .54672 .49828 .54586 .49018 .53586 Metetra
.554 .668 .913 r
.49018 .53586 .49828 .54586 .51291 .5451 .50484 .53504 Metetra
.551 .666 .914 r
.50484 .53504 .51291 .5451 .52766 .54444 .51962 .53431 Metetra
.547 .664 .914 r
.51962 .53431 .52766 .54444 .54252 .54388 .53452 .53369 Metetra
.544 .662 .914 r
.53452 .53369 .54252 .54388 .5575 .54342 .54953 .53316 Metetra
.54 .66 .914 r
.54953 .53316 .5575 .54342 .57259 .54305 .56466 .53274 Metetra
.537 .658 .914 r
.56466 .53274 .57259 .54305 .58781 .54279 .57992 .53242 Metetra
.534 .657 .914 r
.57992 .53242 .58781 .54279 .60315 .54263 .5953 .5322 Metetra
.53 .655 .915 r
.5953 .5322 .60315 .54263 .61862 .54256 .61081 .53208 Metetra
.527 .653 .915 r
.61081 .53208 .61862 .54256 .63422 .54259 .62644 .53206 Metetra
.524 .651 .915 r
.62644 .53206 .63422 .54259 .64996 .54272 .64222 .53213 Metetra
.521 .649 .915 r
.64222 .53213 .64996 .54272 .66583 .54295 .65813 .53231 Metetra
.518 .648 .915 r
.65813 .53231 .66583 .54295 .68184 .54327 .67418 .53258 Metetra
.514 .646 .916 r
.67418 .53258 .68184 .54327 .698 .54369 .69038 .53295 Metetra
.511 .645 .916 r
.69038 .53295 .698 .54369 .7143 .5442 .70672 .53341 Metetra
.508 .643 .916 r
.70672 .53341 .7143 .5442 .73075 .54481 .72321 .53397 Metetra
.505 .642 .916 r
.72321 .53397 .73075 .54481 .74735 .54551 .73986 .53463 Metetra
.502 .64 .916 r
.73986 .53463 .74735 .54551 .7641 .5463 .75666 .53538 Metetra
.5 .639 .917 r
.75666 .53538 .7641 .5463 .78102 .54718 .77362 .53622 Metetra
.497 .637 .917 r
.77362 .53622 .78102 .54718 .7981 .54816 .79074 .53715 Metetra
.494 .636 .917 r
.79074 .53715 .7981 .54816 .81534 .54922 .80803 .53818 Metetra
.491 .635 .917 r
.80803 .53818 .81534 .54922 .83275 .55038 .8255 .53929 Metetra
.489 .634 .918 r
.8255 .53929 .83275 .55038 .85033 .55162 .84313 .5405 Metetra
.486 .632 .918 r
.84313 .5405 .85033 .55162 .86809 .55295 .86094 .54179 Metetra
.484 .631 .918 r
.86094 .54179 .86809 .55295 .88603 .55437 .87893 .54317 Metetra
.481 .63 .918 r
.87893 .54317 .88603 .55437 .90415 .55587 .89711 .54464 Metetra
.479 .629 .919 r
.89711 .54464 .90415 .55587 .92246 .55746 .91547 .54619 Metetra
.477 .628 .919 r
.91547 .54619 .92246 .55746 .94095 .55913 .93403 .54783 Metetra
.597 .694 .911 r
.31334 .54431 .32193 .55351 .33545 .55151 .3269 .54223 Metetra
.593 .692 .911 r
.3269 .54223 .33545 .55151 .34906 .5496 .34054 .54025 Metetra
.59 .69 .911 r
.34054 .54025 .34906 .5496 .36275 .54779 .35427 .53835 Metetra
.587 .688 .911 r
.35427 .53835 .36275 .54779 .37653 .54607 .36808 .53656 Metetra
.583 .685 .911 r
.36808 .53656 .37653 .54607 .3904 .54445 .38198 .53486 Metetra
.58 .683 .912 r
.38198 .53486 .3904 .54445 .40436 .54293 .39598 .53326 Metetra
.576 .681 .912 r
.39598 .53326 .40436 .54293 .41841 .5415 .41006 .53175 Metetra
.573 .679 .912 r
.41006 .53175 .41841 .5415 .43256 .54017 .42425 .53035 Metetra
.569 .677 .912 r
.42425 .53035 .43256 .54017 .44681 .53895 .43853 .52905 Metetra
.566 .675 .912 r
.43853 .52905 .44681 .53895 .46116 .53782 .45292 .52785 Metetra
.562 .672 .912 r
.45292 .52785 .46116 .53782 .47562 .53679 .4674 .52675 Metetra
.559 .67 .913 r
.4674 .52675 .47562 .53679 .49018 .53586 .482 .52575 Metetra
.555 .668 .913 r
.482 .52575 .49018 .53586 .50484 .53504 .4967 .52486 Metetra
.552 .666 .913 r
.4967 .52486 .50484 .53504 .51962 .53431 .51152 .52407 Metetra
.548 .664 .913 r
.51152 .52407 .51962 .53431 .53452 .53369 .52645 .52338 Metetra
.545 .662 .913 r
.52645 .52338 .53452 .53369 .54953 .53316 .54149 .52279 Metetra
.541 .66 .913 r
.54149 .52279 .54953 .53316 .56466 .53274 .55666 .52231 Metetra
.538 .658 .914 r
.55666 .52231 .56466 .53274 .57992 .53242 .57195 .52193 Metetra
.534 .656 .914 r
.57195 .52193 .57992 .53242 .5953 .5322 .58737 .52165 Metetra
.531 .654 .914 r
.58737 .52165 .5953 .5322 .61081 .53208 .60292 .52147 Metetra
.528 .653 .914 r
.60292 .52147 .61081 .53208 .62644 .53206 .61859 .52139 Metetra
.524 .651 .914 r
.61859 .52139 .62644 .53206 .64222 .53213 .63441 .52142 Metetra
.521 .649 .915 r
.63441 .52142 .64222 .53213 .65813 .53231 .65036 .52154 Metetra
.518 .647 .915 r
.65036 .52154 .65813 .53231 .67418 .53258 .66645 .52176 Metetra
.515 .646 .915 r
.66645 .52176 .67418 .53258 .69038 .53295 .68268 .52208 Metetra
.512 .644 .915 r
.68268 .52208 .69038 .53295 .70672 .53341 .69907 .5225 Metetra
.509 .643 .916 r
.69907 .5225 .70672 .53341 .72321 .53397 .7156 .52301 Metetra
.506 .641 .916 r
.7156 .52301 .72321 .53397 .73986 .53463 .73229 .52362 Metetra
.503 .64 .916 r
.73229 .52362 .73986 .53463 .75666 .53538 .74914 .52433 Metetra
.5 .638 .916 r
.74914 .52433 .75666 .53538 .77362 .53622 .76615 .52513 Metetra
.497 .637 .916 r
.76615 .52513 .77362 .53622 .79074 .53715 .78332 .52602 Metetra
.494 .635 .917 r
.78332 .52602 .79074 .53715 .80803 .53818 .80066 .527 Metetra
.492 .634 .917 r
.80066 .527 .80803 .53818 .8255 .53929 .81817 .52808 Metetra
.489 .633 .917 r
.81817 .52808 .8255 .53929 .84313 .5405 .83585 .52925 Metetra
.486 .632 .917 r
.83585 .52925 .84313 .5405 .86094 .54179 .85372 .5305 Metetra
.484 .631 .918 r
.85372 .5305 .86094 .54179 .87893 .54317 .87176 .53185 Metetra
.481 .63 .918 r
.87176 .53185 .87893 .54317 .89711 .54464 .88999 .53328 Metetra
.479 .629 .918 r
.88999 .53328 .89711 .54464 .91547 .54619 .90841 .5348 Metetra
.476 .628 .918 r
.90841 .5348 .91547 .54619 .93403 .54783 .92703 .5364 Metetra
.598 .695 .91 r
.30468 .53503 .31334 .54431 .3269 .54223 .31827 .53287 Metetra
.595 .692 .91 r
.31827 .53287 .3269 .54223 .34054 .54025 .33195 .5308 Metetra
.591 .69 .911 r
.33195 .5308 .34054 .54025 .35427 .53835 .34571 .52882 Metetra
.588 .688 .911 r
.34571 .52882 .35427 .53835 .36808 .53656 .35956 .52694 Metetra
.585 .686 .911 r
.35956 .52694 .36808 .53656 .38198 .53486 .37349 .52516 Metetra
.581 .683 .911 r
.37349 .52516 .38198 .53486 .39598 .53326 .38752 .52348 Metetra
.578 .681 .911 r
.38752 .52348 .39598 .53326 .41006 .53175 .40164 .5219 Metetra
.574 .679 .911 r
.40164 .5219 .41006 .53175 .42425 .53035 .41586 .52042 Metetra
.57 .677 .911 r
.41586 .52042 .42425 .53035 .43853 .52905 .43018 .51904 Metetra
.567 .675 .912 r
.43018 .51904 .43853 .52905 .45292 .52785 .4446 .51777 Metetra
.563 .672 .912 r
.4446 .51777 .45292 .52785 .4674 .52675 .45912 .5166 Metetra
.56 .67 .912 r
.45912 .5166 .4674 .52675 .482 .52575 .47375 .51553 Metetra
.556 .668 .912 r
.47375 .51553 .482 .52575 .4967 .52486 .48849 .51457 Metetra
.553 .666 .912 r
.48849 .51457 .4967 .52486 .51152 .52407 .50334 .51371 Metetra
.549 .664 .912 r
.50334 .51371 .51152 .52407 .52645 .52338 .5183 .51295 Metetra
.546 .662 .913 r
.5183 .51295 .52645 .52338 .54149 .52279 .53339 .5123 Metetra
.542 .66 .913 r
.53339 .5123 .54149 .52279 .55666 .52231 .54859 .51176 Metetra
.539 .658 .913 r
.54859 .51176 .55666 .52231 .57195 .52193 .56392 .51132 Metetra
.535 .656 .913 r
.56392 .51132 .57195 .52193 .58737 .52165 .57937 .51098 Metetra
.532 .654 .913 r
.57937 .51098 .58737 .52165 .60292 .52147 .59495 .51074 Metetra
.528 .652 .914 r
.59495 .51074 .60292 .52147 .61859 .52139 .61067 .51061 Metetra
.525 .65 .914 r
.61067 .51061 .61859 .52139 .63441 .52142 .62652 .51057 Metetra
.522 .649 .914 r
.62652 .51057 .63441 .52142 .65036 .52154 .64251 .51064 Metetra
.519 .647 .914 r
.64251 .51064 .65036 .52154 .66645 .52176 .65864 .51081 Metetra
.515 .645 .915 r
.65864 .51081 .66645 .52176 .68268 .52208 .67492 .51108 Metetra
.512 .644 .915 r
.67492 .51108 .68268 .52208 .69907 .5225 .69135 .51145 Metetra
.509 .642 .915 r
.69135 .51145 .69907 .5225 .7156 .52301 .70792 .51192 Metetra
.506 .641 .915 r
.70792 .51192 .7156 .52301 .73229 .52362 .72466 .51248 Metetra
.503 .639 .915 r
.72466 .51248 .73229 .52362 .74914 .52433 .74155 .51314 Metetra
.5 .638 .916 r
.74155 .51314 .74914 .52433 .76615 .52513 .7586 .5139 Metetra
.497 .636 .916 r
.7586 .5139 .76615 .52513 .78332 .52602 .77582 .51475 Metetra
.494 .635 .916 r
.77582 .51475 .78332 .52602 .80066 .527 .79321 .51569 Metetra
.492 .634 .916 r
.79321 .51569 .80066 .527 .81817 .52808 .81077 .51673 Metetra
.489 .632 .917 r
.81077 .51673 .81817 .52808 .83585 .52925 .82851 .51786 Metetra
.486 .631 .917 r
.82851 .51786 .83585 .52925 .85372 .5305 .84642 .51908 Metetra
.484 .63 .917 r
.84642 .51908 .85372 .5305 .87176 .53185 .86452 .52038 Metetra
.481 .629 .918 r
.86452 .52038 .87176 .53185 .88999 .53328 .88281 .52178 Metetra
.479 .628 .918 r
.88281 .52178 .88999 .53328 .90841 .5348 .90128 .52326 Metetra
.476 .627 .918 r
.90128 .52326 .90841 .5348 .92703 .5364 .91996 .52483 Metetra
.6 .695 .91 r
.29595 .52566 .30468 .53503 .31827 .53287 .30957 .52341 Metetra
.596 .693 .91 r
.30957 .52341 .31827 .53287 .33195 .5308 .32328 .52125 Metetra
.593 .69 .91 r
.32328 .52125 .33195 .5308 .34571 .52882 .33708 .51919 Metetra
.589 .688 .91 r
.33708 .51919 .34571 .52882 .35956 .52694 .35096 .51723 Metetra
.586 .686 .91 r
.35096 .51723 .35956 .52694 .37349 .52516 .36493 .51537 Metetra
.582 .684 .91 r
.36493 .51537 .37349 .52516 .38752 .52348 .37899 .5136 Metetra
.579 .681 .911 r
.37899 .5136 .38752 .52348 .40164 .5219 .39315 .51194 Metetra
.575 .679 .911 r
.39315 .51194 .40164 .5219 .41586 .52042 .4074 .51038 Metetra
.572 .677 .911 r
.4074 .51038 .41586 .52042 .43018 .51904 .42175 .50893 Metetra
.568 .675 .911 r
.42175 .50893 .43018 .51904 .4446 .51777 .43621 .50758 Metetra
.565 .672 .911 r
.43621 .50758 .4446 .51777 .45912 .5166 .45076 .50633 Metetra
.561 .67 .911 r
.45076 .50633 .45912 .5166 .47375 .51553 .46543 .50519 Metetra
.557 .668 .911 r
.46543 .50519 .47375 .51553 .48849 .51457 .4802 .50416 Metetra
.554 .666 .912 r
.4802 .50416 .48849 .51457 .50334 .51371 .49508 .50323 Metetra
.55 .664 .912 r
.49508 .50323 .50334 .51371 .5183 .51295 .51008 .50241 Metetra
.547 .662 .912 r
.51008 .50241 .5183 .51295 .53339 .5123 .5252 .50169 Metetra
.543 .66 .912 r
.5252 .50169 .53339 .5123 .54859 .51176 .54044 .50108 Metetra
.539 .658 .912 r
.54044 .50108 .54859 .51176 .56392 .51132 .55581 .50057 Metetra
.536 .656 .913 r
.55581 .50057 .56392 .51132 .57937 .51098 .5713 .50017 Metetra
.533 .654 .913 r
.5713 .50017 .57937 .51098 .59495 .51074 .58692 .49988 Metetra
.529 .652 .913 r
.58692 .49988 .59495 .51074 .61067 .51061 .60267 .49969 Metetra
.526 .65 .913 r
.60267 .49969 .61067 .51061 .62652 .51057 .61856 .4996 Metetra
.522 .648 .913 r
.61856 .4996 .62652 .51057 .64251 .51064 .63459 .49962 Metetra
.519 .647 .914 r
.63459 .49962 .64251 .51064 .65864 .51081 .65076 .49973 Metetra
.516 .645 .914 r
.65076 .49973 .65864 .51081 .67492 .51108 .66708 .49995 Metetra
.513 .643 .914 r
.66708 .49995 .67492 .51108 .69135 .51145 .68355 .50027 Metetra
.509 .642 .914 r
.68355 .50027 .69135 .51145 .70792 .51192 .70017 .50069 Metetra
.506 .64 .915 r
.70017 .50069 .70792 .51192 .72466 .51248 .71695 .50121 Metetra
.503 .639 .915 r
.71695 .50121 .72466 .51248 .74155 .51314 .73388 .50182 Metetra
.5 .637 .915 r
.73388 .50182 .74155 .51314 .7586 .5139 .75098 .50254 Metetra
.497 .636 .915 r
.75098 .50254 .7586 .5139 .77582 .51475 .76825 .50334 Metetra
.494 .634 .916 r
.76825 .50334 .77582 .51475 .79321 .51569 .78568 .50425 Metetra
.492 .633 .916 r
.78568 .50425 .79321 .51569 .81077 .51673 .80329 .50524 Metetra
.489 .632 .916 r
.80329 .50524 .81077 .51673 .82851 .51786 .82108 .50633 Metetra
.486 .631 .917 r
.82108 .50633 .82851 .51786 .84642 .51908 .83905 .50751 Metetra
.484 .63 .917 r
.83905 .50751 .84642 .51908 .86452 .52038 .8572 .50878 Metetra
.481 .628 .917 r
.8572 .50878 .86452 .52038 .88281 .52178 .87555 .51014 Metetra
.478 .627 .917 r
.87555 .51014 .88281 .52178 .90128 .52326 .89408 .51159 Metetra
.476 .626 .918 r
.89408 .51159 .90128 .52326 .91996 .52483 .91281 .51313 Metetra
.601 .695 .909 r
.28713 .5162 .29595 .52566 .30957 .52341 .3008 .51386 Metetra
.598 .693 .909 r
.3008 .51386 .30957 .52341 .32328 .52125 .31454 .51162 Metetra
.594 .691 .909 r
.31454 .51162 .32328 .52125 .33708 .51919 .32837 .50947 Metetra
.591 .689 .91 r
.32837 .50947 .33708 .51919 .35096 .51723 .34229 .50742 Metetra
.587 .686 .91 r
.34229 .50742 .35096 .51723 .36493 .51537 .35629 .50547 Metetra
.584 .684 .91 r
.35629 .50547 .36493 .51537 .37899 .5136 .37039 .50362 Metetra
.58 .682 .91 r
.37039 .50362 .37899 .5136 .39315 .51194 .38458 .50188 Metetra
.577 .679 .91 r
.38458 .50188 .39315 .51194 .4074 .51038 .39887 .50024 Metetra
.573 .677 .91 r
.39887 .50024 .4074 .51038 .42175 .50893 .41325 .4987 Metetra
.569 .675 .91 r
.41325 .4987 .42175 .50893 .43621 .50758 .42774 .49727 Metetra
.566 .672 .91 r
.42774 .49727 .43621 .50758 .45076 .50633 .44233 .49595 Metetra
.562 .67 .911 r
.44233 .49595 .45076 .50633 .46543 .50519 .45703 .49474 Metetra
.558 .668 .911 r
.45703 .49474 .46543 .50519 .4802 .50416 .47184 .49363 Metetra
.555 .666 .911 r
.47184 .49363 .4802 .50416 .49508 .50323 .48676 .49263 Metetra
.551 .664 .911 r
.48676 .49263 .49508 .50323 .51008 .50241 .50179 .49173 Metetra
.548 .662 .911 r
.50179 .49173 .51008 .50241 .5252 .50169 .51695 .49095 Metetra
.544 .659 .912 r
.51695 .49095 .5252 .50169 .54044 .50108 .53222 .49027 Metetra
.54 .657 .912 r
.53222 .49027 .54044 .50108 .55581 .50057 .54762 .4897 Metetra
.537 .655 .912 r
.54762 .4897 .55581 .50057 .5713 .50017 .56315 .48924 Metetra
.533 .653 .912 r
.56315 .48924 .5713 .50017 .58692 .49988 .57881 .48888 Metetra
.53 .652 .912 r
.57881 .48888 .58692 .49988 .60267 .49969 .5946 .48864 Metetra
.526 .65 .913 r
.5946 .48864 .60267 .49969 .61856 .4996 .61053 .48849 Metetra
.523 .648 .913 r
.61053 .48849 .61856 .4996 .63459 .49962 .6266 .48845 Metetra
.52 .646 .913 r
.6266 .48845 .63459 .49962 .65076 .49973 .64281 .48852 Metetra
.516 .644 .913 r
.64281 .48852 .65076 .49973 .66708 .49995 .65917 .48868 Metetra
.513 .643 .914 r
.65917 .48868 .66708 .49995 .68355 .50027 .67568 .48895 Metetra
.51 .641 .914 r
.67568 .48895 .68355 .50027 .70017 .50069 .69234 .48932 Metetra
.507 .639 .914 r
.69234 .48932 .70017 .50069 .71695 .50121 .70916 .48979 Metetra
.503 .638 .914 r
.70916 .48979 .71695 .50121 .73388 .50182 .72614 .49037 Metetra
.5 .637 .915 r
.72614 .49037 .73388 .50182 .75098 .50254 .74329 .49103 Metetra
.497 .635 .915 r
.74329 .49103 .75098 .50254 .76825 .50334 .7606 .4918 Metetra
.494 .634 .915 r
.7606 .4918 .76825 .50334 .78568 .50425 .77809 .49266 Metetra
.492 .632 .916 r
.77809 .49266 .78568 .50425 .80329 .50524 .79575 .49362 Metetra
.489 .631 .916 r
.79575 .49362 .80329 .50524 .82108 .50633 .81358 .49467 Metetra
.486 .63 .916 r
.81358 .49467 .82108 .50633 .83905 .50751 .83161 .49581 Metetra
.483 .629 .916 r
.83161 .49581 .83905 .50751 .8572 .50878 .84981 .49704 Metetra
.481 .628 .917 r
.84981 .49704 .8572 .50878 .87555 .51014 .86821 .49837 Metetra
.478 .627 .917 r
.86821 .49837 .87555 .51014 .89408 .51159 .8868 .49978 Metetra
.476 .626 .917 r
.8868 .49978 .89408 .51159 .91281 .51313 .90559 .50128 Metetra
.603 .696 .909 r
.27824 .50665 .28713 .5162 .3008 .51386 .29194 .50422 Metetra
.6 .693 .909 r
.29194 .50422 .3008 .51386 .31454 .51162 .30572 .50188 Metetra
.596 .691 .909 r
.30572 .50188 .31454 .51162 .32837 .50947 .31958 .49964 Metetra
.593 .689 .909 r
.31958 .49964 .32837 .50947 .34229 .50742 .33354 .4975 Metetra
.589 .686 .909 r
.33354 .4975 .34229 .50742 .35629 .50547 .34758 .49547 Metetra
.585 .684 .909 r
.34758 .49547 .35629 .50547 .37039 .50362 .36171 .49353 Metetra
.582 .682 .909 r
.36171 .49353 .37039 .50362 .38458 .50188 .37593 .4917 Metetra
.578 .679 .909 r
.37593 .4917 .38458 .50188 .39887 .50024 .39025 .48998 Metetra
.575 .677 .909 r
.39025 .48998 .39887 .50024 .41325 .4987 .40468 .48836 Metetra
.571 .675 .91 r
.40468 .48836 .41325 .4987 .42774 .49727 .4192 .48685 Metetra
.567 .672 .91 r
.4192 .48685 .42774 .49727 .44233 .49595 .43382 .48545 Metetra
.563 .67 .91 r
.43382 .48545 .44233 .49595 .45703 .49474 .44856 .48416 Metetra
.56 .668 .91 r
.44856 .48416 .45703 .49474 .47184 .49363 .4634 .48297 Metetra
.556 .666 .91 r
.4634 .48297 .47184 .49363 .48676 .49263 .47835 .4819 Metetra
.552 .663 .91 r
.47835 .4819 .48676 .49263 .50179 .49173 .49342 .48093 Metetra
.549 .661 .911 r
.49342 .48093 .50179 .49173 .51695 .49095 .50861 .48008 Metetra
.545 .659 .911 r
.50861 .48008 .51695 .49095 .53222 .49027 .52393 .47933 Metetra
.541 .657 .911 r
.52393 .47933 .53222 .49027 .54762 .4897 .53936 .4787 Metetra
.538 .655 .911 r
.53936 .4787 .54762 .4897 .56315 .48924 .55493 .47817 Metetra
.534 .653 .911 r
.55493 .47817 .56315 .48924 .57881 .48888 .57062 .47776 Metetra
.53 .651 .912 r
.57062 .47776 .57881 .48888 .5946 .48864 .58645 .47745 Metetra
.527 .649 .912 r
.58645 .47745 .5946 .48864 .61053 .48849 .60242 .47724 Metetra
.523 .647 .912 r
.60242 .47724 .61053 .48849 .6266 .48845 .61853 .47715 Metetra
.52 .646 .912 r
.61853 .47715 .6266 .48845 .64281 .48852 .63478 .47716 Metetra
.517 .644 .913 r
.63478 .47716 .64281 .48852 .65917 .48868 .65118 .47727 Metetra
.513 .642 .913 r
.65118 .47727 .65917 .48868 .67568 .48895 .66773 .47749 Metetra
.51 .64 .913 r
.66773 .47749 .67568 .48895 .69234 .48932 .68444 .47781 Metetra
.507 .639 .914 r
.68444 .47781 .69234 .48932 .70916 .48979 .7013 .47824 Metetra
.504 .637 .914 r
.7013 .47824 .70916 .48979 .72614 .49037 .71833 .47876 Metetra
.501 .636 .914 r
.71833 .47876 .72614 .49037 .74329 .49103 .73552 .47939 Metetra
.498 .634 .914 r
.73552 .47939 .74329 .49103 .7606 .4918 .75288 .48011 Metetra
.495 .633 .915 r
.75288 .48011 .7606 .4918 .77809 .49266 .77041 .48093 Metetra
.492 .632 .915 r
.77041 .48093 .77809 .49266 .79575 .49362 .78812 .48185 Metetra
.489 .631 .915 r
.78812 .48185 .79575 .49362 .81358 .49467 .80601 .48286 Metetra
.486 .629 .916 r
.80601 .48286 .81358 .49467 .83161 .49581 .82408 .48396 Metetra
.483 .628 .916 r
.82408 .48396 .83161 .49581 .84981 .49704 .84235 .48516 Metetra
.481 .627 .916 r
.84235 .48516 .84981 .49704 .86821 .49837 .8608 .48645 Metetra
.478 .626 .917 r
.8608 .48645 .86821 .49837 .8868 .49978 .87945 .48783 Metetra
.475 .625 .917 r
.87945 .48783 .8868 .49978 .90559 .50128 .8983 .4893 Metetra
.605 .696 .908 r
.26927 .49701 .27824 .50665 .29194 .50422 .283 .49448 Metetra
.601 .694 .908 r
.283 .49448 .29194 .50422 .30572 .50188 .29682 .49205 Metetra
.598 .692 .908 r
.29682 .49205 .30572 .50188 .31958 .49964 .31072 .48972 Metetra
.594 .689 .908 r
.31072 .48972 .31958 .49964 .33354 .4975 .32471 .48749 Metetra
.591 .687 .908 r
.32471 .48749 .33354 .4975 .34758 .49547 .33878 .48536 Metetra
.587 .684 .908 r
.33878 .48536 .34758 .49547 .36171 .49353 .35295 .48333 Metetra
.583 .682 .908 r
.35295 .48333 .36171 .49353 .37593 .4917 .36721 .48142 Metetra
.58 .68 .909 r
.36721 .48142 .37593 .4917 .39025 .48998 .38157 .4796 Metetra
.576 .677 .909 r
.38157 .4796 .39025 .48998 .40468 .48836 .39602 .4779 Metetra
.572 .675 .909 r
.39602 .4779 .40468 .48836 .4192 .48685 .41058 .47631 Metetra
.568 .672 .909 r
.41058 .47631 .4192 .48685 .43382 .48545 .42524 .47482 Metetra
.565 .67 .909 r
.42524 .47482 .43382 .48545 .44856 .48416 .44001 .47345 Metetra
.561 .668 .909 r
.44001 .47345 .44856 .48416 .4634 .48297 .45489 .47219 Metetra
.557 .666 .91 r
.45489 .47219 .4634 .48297 .47835 .4819 .46988 .47104 Metetra
.553 .663 .91 r
.46988 .47104 .47835 .4819 .49342 .48093 .48498 .47 Metetra
.55 .661 .91 r
.48498 .47 .49342 .48093 .50861 .48008 .50021 .46908 Metetra
.546 .659 .91 r
.50021 .46908 .50861 .48008 .52393 .47933 .51555 .46826 Metetra
.542 .657 .91 r
.51555 .46826 .52393 .47933 .53936 .4787 .53103 .46756 Metetra
.538 .655 .911 r
.53103 .46756 .53936 .4787 .55493 .47817 .54663 .46697 Metetra
.535 .653 .911 r
.54663 .46697 .55493 .47817 .57062 .47776 .56236 .46649 Metetra
.531 .651 .911 r
.56236 .46649 .57062 .47776 .58645 .47745 .57823 .46612 Metetra
.528 .649 .911 r
.57823 .46612 .58645 .47745 .60242 .47724 .59423 .46585 Metetra
.524 .647 .911 r
.59423 .46585 .60242 .47724 .61853 .47715 .61038 .4657 Metetra
.521 .645 .912 r
.61038 .4657 .61853 .47715 .63478 .47716 .62667 .46566 Metetra
.517 .643 .912 r
.62667 .46566 .63478 .47716 .65118 .47727 .64311 .46572 Metetra
.514 .642 .912 r
.64311 .46572 .65118 .47727 .66773 .47749 .6597 .46589 Metetra
.51 .64 .913 r
.6597 .46589 .66773 .47749 .68444 .47781 .67645 .46616 Metetra
.507 .638 .913 r
.67645 .46616 .68444 .47781 .7013 .47824 .69336 .46654 Metetra
.504 .637 .913 r
.69336 .46654 .7013 .47824 .71833 .47876 .71043 .46702 Metetra
.501 .635 .914 r
.71043 .46702 .71833 .47876 .73552 .47939 .72767 .4676 Metetra
.498 .634 .914 r
.72767 .4676 .73552 .47939 .75288 .48011 .74508 .46828 Metetra
.495 .632 .914 r
.74508 .46828 .75288 .48011 .77041 .48093 .76266 .46906 Metetra
.492 .631 .914 r
.76266 .46906 .77041 .48093 .78812 .48185 .78042 .46993 Metetra
.489 .63 .915 r
.78042 .46993 .78812 .48185 .80601 .48286 .79836 .4709 Metetra
.486 .629 .915 r
.79836 .4709 .80601 .48286 .82408 .48396 .81649 .47197 Metetra
.483 .628 .916 r
.81649 .47197 .82408 .48396 .84235 .48516 .8348 .47313 Metetra
.48 .626 .916 r
.8348 .47313 .84235 .48516 .8608 .48645 .85331 .47439 Metetra
.478 .625 .916 r
.85331 .47439 .8608 .48645 .87945 .48783 .87202 .47573 Metetra
.475 .624 .917 r
.87202 .47573 .87945 .48783 .8983 .4893 .89093 .47716 Metetra
.606 .697 .907 r
.26022 .48727 .26927 .49701 .283 .49448 .27399 .48465 Metetra
.603 .694 .907 r
.27399 .48465 .283 .49448 .29682 .49205 .28784 .48212 Metetra
.599 .692 .907 r
.28784 .48212 .29682 .49205 .31072 .48972 .30178 .47969 Metetra
.596 .689 .907 r
.30178 .47969 .31072 .48972 .32471 .48749 .3158 .47737 Metetra
.592 .687 .908 r
.3158 .47737 .32471 .48749 .33878 .48536 .32991 .47514 Metetra
.589 .685 .908 r
.32991 .47514 .33878 .48536 .35295 .48333 .34411 .47302 Metetra
.585 .682 .908 r
.34411 .47302 .35295 .48333 .36721 .48142 .35841 .47101 Metetra
.581 .68 .908 r
.35841 .47101 .36721 .48142 .38157 .4796 .3728 .46911 Metetra
.577 .677 .908 r
.3728 .46911 .38157 .4796 .39602 .4779 .38729 .46732 Metetra
.574 .675 .908 r
.38729 .46732 .39602 .4779 .41058 .47631 .40188 .46564 Metetra
.57 .672 .908 r
.40188 .46564 .41058 .47631 .42524 .47482 .41658 .46407 Metetra
.566 .67 .908 r
.41658 .46407 .42524 .47482 .44001 .47345 .43138 .46262 Metetra
.562 .668 .909 r
.43138 .46262 .44001 .47345 .45489 .47219 .4463 .46128 Metetra
.558 .665 .909 r
.4463 .46128 .45489 .47219 .46988 .47104 .46132 .46005 Metetra
.555 .663 .909 r
.46132 .46005 .46988 .47104 .48498 .47 .47646 .45893 Metetra
.551 .661 .909 r
.47646 .45893 .48498 .47 .50021 .46908 .49172 .45793 Metetra
.547 .659 .909 r
.49172 .45793 .50021 .46908 .51555 .46826 .50711 .45705 Metetra
.543 .656 .91 r
.50711 .45705 .51555 .46826 .53103 .46756 .52261 .45628 Metetra
.539 .654 .91 r
.52261 .45628 .53103 .46756 .54663 .46697 .53825 .45562 Metetra
.536 .652 .91 r
.53825 .45562 .54663 .46697 .56236 .46649 .55402 .45507 Metetra
.532 .65 .91 r
.55402 .45507 .56236 .46649 .57823 .46612 .56993 .45464 Metetra
.528 .648 .91 r
.56993 .45464 .57823 .46612 .59423 .46585 .58597 .45432 Metetra
.525 .646 .911 r
.58597 .45432 .59423 .46585 .61038 .4657 .60216 .45411 Metetra
.521 .644 .911 r
.60216 .45411 .61038 .4657 .62667 .46566 .61849 .45401 Metetra
.518 .643 .911 r
.61849 .45401 .62667 .46566 .64311 .46572 .63497 .45402 Metetra
.514 .641 .912 r
.63497 .45402 .64311 .46572 .6597 .46589 .6516 .45413 Metetra
.511 .639 .912 r
.6516 .45413 .6597 .46589 .67645 .46616 .66839 .45436 Metetra
.507 .638 .912 r
.66839 .45436 .67645 .46616 .69336 .46654 .68535 .45469 Metetra
.504 .636 .913 r
.68535 .45469 .69336 .46654 .71043 .46702 .70246 .45512 Metetra
.501 .635 .913 r
.70246 .45512 .71043 .46702 .72767 .4676 .71974 .45565 Metetra
.498 .633 .913 r
.71974 .45565 .72767 .4676 .74508 .46828 .7372 .45629 Metetra
.495 .632 .914 r
.7372 .45629 .74508 .46828 .76266 .46906 .75483 .45703 Metetra
.491 .63 .914 r
.75483 .45703 .76266 .46906 .78042 .46993 .77264 .45787 Metetra
.489 .629 .914 r
.77264 .45787 .78042 .46993 .79836 .4709 .79063 .4588 Metetra
.486 .628 .915 r
.79063 .4588 .79836 .4709 .81649 .47197 .80881 .45983 Metetra
.483 .627 .915 r
.80881 .45983 .81649 .47197 .8348 .47313 .82718 .46095 Metetra
.48 .626 .915 r
.82718 .46095 .8348 .47313 .85331 .47439 .84575 .46217 Metetra
.477 .625 .916 r
.84575 .46217 .85331 .47439 .87202 .47573 .86451 .46348 Metetra
.475 .624 .916 r
.86451 .46348 .87202 .47573 .89093 .47716 .88348 .46488 Metetra
.608 .697 .907 r
.25109 .47744 .26022 .48727 .27399 .48465 .26489 .47472 Metetra
.605 .695 .907 r
.26489 .47472 .27399 .48465 .28784 .48212 .27878 .47209 Metetra
.601 .692 .907 r
.27878 .47209 .28784 .48212 .30178 .47969 .29276 .46956 Metetra
.598 .69 .907 r
.29276 .46956 .30178 .47969 .3158 .47737 .30682 .46714 Metetra
.594 .687 .907 r
.30682 .46714 .3158 .47737 .32991 .47514 .32096 .46482 Metetra
.59 .685 .907 r
.32096 .46482 .32991 .47514 .34411 .47302 .3352 .4626 Metetra
.587 .682 .907 r
.3352 .4626 .34411 .47302 .35841 .47101 .34953 .4605 Metetra
.583 .68 .907 r
.34953 .4605 .35841 .47101 .3728 .46911 .36396 .4585 Metetra
.579 .677 .907 r
.36396 .4585 .3728 .46911 .38729 .46732 .37848 .45662 Metetra
.575 .675 .907 r
.37848 .45662 .38729 .46732 .40188 .46564 .39311 .45485 Metetra
.571 .672 .907 r
.39311 .45485 .40188 .46564 .41658 .46407 .40784 .45319 Metetra
.567 .67 .908 r
.40784 .45319 .41658 .46407 .43138 .46262 .42268 .45165 Metetra
.564 .668 .908 r
.42268 .45165 .43138 .46262 .4463 .46128 .43763 .45023 Metetra
.56 .665 .908 r
.43763 .45023 .4463 .46128 .46132 .46005 .45269 .44892 Metetra
.556 .663 .908 r
.45269 .44892 .46132 .46005 .47646 .45893 .46787 .44773 Metetra
.552 .66 .908 r
.46787 .44773 .47646 .45893 .49172 .45793 .48316 .44665 Metetra
.548 .658 .908 r
.48316 .44665 .49172 .45793 .50711 .45705 .49858 .44569 Metetra
.544 .656 .909 r
.49858 .44569 .50711 .45705 .52261 .45628 .51413 .44485 Metetra
.54 .654 .909 r
.51413 .44485 .52261 .45628 .53825 .45562 .5298 .44413 Metetra
.536 .652 .909 r
.5298 .44413 .53825 .45562 .55402 .45507 .54561 .44351 Metetra
.533 .65 .909 r
.54561 .44351 .55402 .45507 .56993 .45464 .56155 .44302 Metetra
.529 .648 .91 r
.56155 .44302 .56993 .45464 .58597 .45432 .57763 .44264 Metetra
.525 .646 .91 r
.57763 .44264 .58597 .45432 .60216 .45411 .59385 .44237 Metetra
.522 .644 .91 r
.59385 .44237 .60216 .45411 .61849 .45401 .61023 .44221 Metetra
.518 .642 .911 r
.61023 .44221 .61849 .45401 .63497 .45402 .62675 .44216 Metetra
.514 .64 .911 r
.62675 .44216 .63497 .45402 .6516 .45413 .64342 .44223 Metetra
.511 .638 .911 r
.64342 .44223 .6516 .45413 .66839 .45436 .66026 .4424 Metetra
.508 .637 .912 r
.66026 .4424 .66839 .45436 .68535 .45469 .67725 .44268 Metetra
.504 .635 .912 r
.67725 .44268 .68535 .45469 .70246 .45512 .69441 .44307 Metetra
.501 .634 .912 r
.69441 .44307 .70246 .45512 .71974 .45565 .71174 .44356 Metetra
.498 .632 .913 r
.71174 .44356 .71974 .45565 .7372 .45629 .72924 .44415 Metetra
.494 .631 .913 r
.72924 .44415 .7372 .45629 .75483 .45703 .74692 .44485 Metetra
.491 .63 .913 r
.74692 .44485 .75483 .45703 .77264 .45787 .76478 .44565 Metetra
.488 .628 .914 r
.76478 .44565 .77264 .45787 .79063 .4588 .78282 .44654 Metetra
.485 .627 .914 r
.78282 .44654 .79063 .4588 .80881 .45983 .80105 .44754 Metetra
.483 .626 .915 r
.80105 .44754 .80881 .45983 .82718 .46095 .81948 .44863 Metetra
.48 .625 .915 r
.81948 .44863 .82718 .46095 .84575 .46217 .8381 .44981 Metetra
.477 .624 .915 r
.8381 .44981 .84575 .46217 .86451 .46348 .85692 .45109 Metetra
.474 .623 .916 r
.85692 .45109 .86451 .46348 .88348 .46488 .87595 .45245 Metetra
.61 .698 .906 r
.24187 .46752 .25109 .47744 .26489 .47472 .25572 .46469 Metetra
.607 .695 .906 r
.25572 .46469 .26489 .47472 .27878 .47209 .26964 .46196 Metetra
.603 .693 .906 r
.26964 .46196 .27878 .47209 .29276 .46956 .28365 .45933 Metetra
.599 .69 .906 r
.28365 .45933 .29276 .46956 .30682 .46714 .29775 .4568 Metetra
.596 .688 .906 r
.29775 .4568 .30682 .46714 .32096 .46482 .31193 .45438 Metetra
.592 .685 .906 r
.31193 .45438 .32096 .46482 .3352 .4626 .32621 .45206 Metetra
.588 .683 .906 r
.32621 .45206 .3352 .4626 .34953 .4605 .34057 .44986 Metetra
.585 .68 .906 r
.34057 .44986 .34953 .4605 .36396 .4585 .35504 .44777 Metetra
.581 .677 .906 r
.35504 .44777 .36396 .4585 .37848 .45662 .3696 .44579 Metetra
.577 .675 .906 r
.3696 .44579 .37848 .45662 .39311 .45485 .38426 .44393 Metetra
.573 .672 .907 r
.38426 .44393 .39311 .45485 .40784 .45319 .39903 .44218 Metetra
.569 .67 .907 r
.39903 .44218 .40784 .45319 .42268 .45165 .4139 .44056 Metetra
.565 .667 .907 r
.4139 .44056 .42268 .45165 .43763 .45023 .42888 .43904 Metetra
.561 .665 .907 r
.42888 .43904 .43763 .45023 .45269 .44892 .44398 .43765 Metetra
.557 .662 .907 r
.44398 .43765 .45269 .44892 .46787 .44773 .45919 .43638 Metetra
.553 .66 .907 r
.45919 .43638 .46787 .44773 .48316 .44665 .47452 .43523 Metetra
.549 .658 .908 r
.47452 .43523 .48316 .44665 .49858 .44569 .48998 .43419 Metetra
.545 .656 .908 r
.48998 .43419 .49858 .44569 .51413 .44485 .50556 .43328 Metetra
.541 .653 .908 r
.50556 .43328 .51413 .44485 .5298 .44413 .52127 .43248 Metetra
.537 .651 .908 r
.52127 .43248 .5298 .44413 .54561 .44351 .53711 .4318 Metetra
.533 .649 .909 r
.53711 .4318 .54561 .44351 .56155 .44302 .55309 .43124 Metetra
.53 .647 .909 r
.55309 .43124 .56155 .44302 .57763 .44264 .56921 .4308 Metetra
.526 .645 .909 r
.56921 .4308 .57763 .44264 .59385 .44237 .58547 .43047 Metetra
.522 .643 .909 r
.58547 .43047 .59385 .44237 .61023 .44221 .60188 .43025 Metetra
.518 .641 .91 r
.60188 .43025 .61023 .44221 .62675 .44216 .61845 .43015 Metetra
.515 .639 .91 r
.61845 .43015 .62675 .44216 .64342 .44223 .63516 .43016 Metetra
.511 .638 .91 r
.63516 .43016 .64342 .44223 .66026 .4424 .65204 .43029 Metetra
.508 .636 .911 r
.65204 .43029 .66026 .4424 .67725 .44268 .66908 .43052 Metetra
.504 .634 .911 r
.66908 .43052 .67725 .44268 .69441 .44307 .68628 .43086 Metetra
.501 .633 .912 r
.68628 .43086 .69441 .44307 .71174 .44356 .70365 .43131 Metetra
.498 .631 .912 r
.70365 .43131 .71174 .44356 .72924 .44415 .7212 .43186 Metetra
.494 .63 .912 r
.7212 .43186 .72924 .44415 .74692 .44485 .73893 .43251 Metetra
.491 .629 .913 r
.73893 .43251 .74692 .44485 .76478 .44565 .75684 .43327 Metetra
.488 .627 .913 r
.75684 .43327 .76478 .44565 .78282 .44654 .77493 .43413 Metetra
.485 .626 .914 r
.77493 .43413 .78282 .44654 .80105 .44754 .79322 .43509 Metetra
.482 .625 .914 r
.79322 .43509 .80105 .44754 .81948 .44863 .8117 .43614 Metetra
.479 .624 .914 r
.8117 .43614 .81948 .44863 .8381 .44981 .83037 .43729 Metetra
.477 .623 .915 r
.83037 .43729 .8381 .44981 .85692 .45109 .84925 .43854 Metetra
.474 .622 .915 r
.84925 .43854 .85692 .45109 .87595 .45245 .86834 .43987 Metetra
.612 .698 .905 r
.23257 .45751 .24187 .46752 .25572 .46469 .24646 .45457 Metetra
.609 .696 .905 r
.24646 .45457 .25572 .46469 .26964 .46196 .26042 .45173 Metetra
.605 .693 .905 r
.26042 .45173 .26964 .46196 .28365 .45933 .27447 .44899 Metetra
.601 .691 .905 r
.27447 .44899 .28365 .45933 .29775 .4568 .2886 .44636 Metetra
.598 .688 .905 r
.2886 .44636 .29775 .4568 .31193 .45438 .30282 .44383 Metetra
.594 .685 .905 r
.30282 .44383 .31193 .45438 .32621 .45206 .31713 .44141 Metetra
.59 .683 .905 r
.31713 .44141 .32621 .45206 .34057 .44986 .33154 .4391 Metetra
.586 .68 .905 r
.33154 .4391 .34057 .44986 .35504 .44777 .34604 .43691 Metetra
.582 .677 .905 r
.34604 .43691 .35504 .44777 .3696 .44579 .36063 .43484 Metetra
.578 .675 .906 r
.36063 .43484 .3696 .44579 .38426 .44393 .37533 .43288 Metetra
.574 .672 .906 r
.37533 .43288 .38426 .44393 .39903 .44218 .39013 .43104 Metetra
.57 .67 .906 r
.39013 .43104 .39903 .44218 .4139 .44056 .40504 .42932 Metetra
.566 .667 .906 r
.40504 .42932 .4139 .44056 .42888 .43904 .42006 .42772 Metetra
.562 .665 .906 r
.42006 .42772 .42888 .43904 .44398 .43765 .43519 .42624 Metetra
.558 .662 .906 r
.43519 .42624 .44398 .43765 .45919 .43638 .45044 .42489 Metetra
.554 .66 .906 r
.45044 .42489 .45919 .43638 .47452 .43523 .46581 .42365 Metetra
.55 .657 .907 r
.46581 .42365 .47452 .43523 .48998 .43419 .4813 .42254 Metetra
.546 .655 .907 r
.4813 .42254 .48998 .43419 .50556 .43328 .49691 .42155 Metetra
.542 .653 .907 r
.49691 .42155 .50556 .43328 .52127 .43248 .51266 .42068 Metetra
.538 .651 .907 r
.51266 .42068 .52127 .43248 .53711 .4318 .52854 .41993 Metetra
.534 .648 .908 r
.52854 .41993 .53711 .4318 .55309 .43124 .54455 .41931 Metetra
.53 .646 .908 r
.54455 .41931 .55309 .43124 .56921 .4308 .56071 .4188 Metetra
.526 .644 .908 r
.56071 .4188 .56921 .4308 .58547 .43047 .57701 .41841 Metetra
.523 .642 .909 r
.57701 .41841 .58547 .43047 .60188 .43025 .59346 .41814 Metetra
.519 .64 .909 r
.59346 .41814 .60188 .43025 .61845 .43015 .61007 .41798 Metetra
.515 .639 .909 r
.61007 .41798 .61845 .43015 .63516 .43016 .62682 .41794 Metetra
.511 .637 .91 r
.62682 .41794 .63516 .43016 .65204 .43029 .64374 .41801 Metetra
.508 .635 .91 r
.64374 .41801 .65204 .43029 .66908 .43052 .66082 .41819 Metetra
.504 .634 .91 r
.66082 .41819 .66908 .43052 .68628 .43086 .67807 .41849 Metetra
.501 .632 .911 r
.67807 .41849 .68628 .43086 .70365 .43131 .69549 .41889 Metetra
.498 .631 .911 r
.69549 .41889 .70365 .43131 .7212 .43186 .71308 .4194 Metetra
.494 .629 .912 r
.71308 .4194 .7212 .43186 .73893 .43251 .73086 .42002 Metetra
.491 .628 .912 r
.73086 .42002 .73893 .43251 .75684 .43327 .74882 .42074 Metetra
.488 .627 .913 r
.74882 .42074 .75684 .43327 .77493 .43413 .76696 .42156 Metetra
.485 .625 .913 r
.76696 .42156 .77493 .43413 .79322 .43509 .7853 .42248 Metetra
.482 .624 .914 r
.7853 .42248 .79322 .43509 .8117 .43614 .80383 .4235 Metetra
.479 .623 .914 r
.80383 .4235 .8117 .43614 .83037 .43729 .82257 .42462 Metetra
.476 .622 .914 r
.82257 .42462 .83037 .43729 .84925 .43854 .84151 .42583 Metetra
.473 .621 .915 r
.84151 .42583 .84925 .43854 .86834 .43987 .86065 .42714 Metetra
.614 .699 .905 r
.22319 .4474 .23257 .45751 .24646 .45457 .23711 .44435 Metetra
.611 .696 .905 r
.23711 .44435 .24646 .45457 .26042 .45173 .25111 .4414 Metetra
.607 .694 .905 r
.25111 .4414 .26042 .45173 .27447 .44899 .2652 .43854 Metetra
.603 .691 .905 r
.2652 .43854 .27447 .44899 .2886 .44636 .27937 .4358 Metetra
.6 .688 .904 r
.27937 .4358 .2886 .44636 .30282 .44383 .29363 .43316 Metetra
.596 .686 .904 r
.29363 .43316 .30282 .44383 .31713 .44141 .30798 .43064 Metetra
.592 .683 .904 r
.30798 .43064 .31713 .44141 .33154 .4391 .32242 .42822 Metetra
.588 .68 .905 r
.32242 .42822 .33154 .4391 .34604 .43691 .33695 .42593 Metetra
.584 .678 .905 r
.33695 .42593 .34604 .43691 .36063 .43484 .35159 .42375 Metetra
.58 .675 .905 r
.35159 .42375 .36063 .43484 .37533 .43288 .36632 .42169 Metetra
.576 .672 .905 r
.36632 .42169 .37533 .43288 .39013 .43104 .38116 .41975 Metetra
.572 .67 .905 r
.38116 .41975 .39013 .43104 .40504 .42932 .3961 .41794 Metetra
.568 .667 .905 r
.3961 .41794 .40504 .42932 .42006 .42772 .41116 .41625 Metetra
.564 .664 .905 r
.41116 .41625 .42006 .42772 .43519 .42624 .42632 .41468 Metetra
.56 .662 .905 r
.42632 .41468 .43519 .42624 .45044 .42489 .44161 .41324 Metetra
.555 .659 .905 r
.44161 .41324 .45044 .42489 .46581 .42365 .45701 .41192 Metetra
.551 .657 .906 r
.45701 .41192 .46581 .42365 .4813 .42254 .47253 .41073 Metetra
.547 .655 .906 r
.47253 .41073 .4813 .42254 .49691 .42155 .48819 .40966 Metetra
.543 .652 .906 r
.48819 .40966 .49691 .42155 .51266 .42068 .50397 .40872 Metetra
.539 .65 .906 r
.50397 .40872 .51266 .42068 .52854 .41993 .51989 .4079 Metetra
.535 .648 .907 r
.51989 .4079 .52854 .41993 .54455 .41931 .53594 .40721 Metetra
.531 .646 .907 r
.53594 .40721 .54455 .41931 .56071 .4188 .55213 .40663 Metetra
.527 .644 .907 r
.55213 .40663 .56071 .4188 .57701 .41841 .56847 .40618 Metetra
.523 .642 .908 r
.56847 .40618 .57701 .41841 .59346 .41814 .58496 .40585 Metetra
.519 .64 .908 r
.58496 .40585 .59346 .41814 .61007 .41798 .6016 .40564 Metetra
.515 .638 .908 r
.6016 .40564 .61007 .41798 .62682 .41794 .6184 .40555 Metetra
.512 .636 .909 r
.6184 .40555 .62682 .41794 .64374 .41801 .63536 .40557 Metetra
.508 .634 .909 r
.63536 .40557 .64374 .41801 .66082 .41819 .65248 .4057 Metetra
.504 .633 .91 r
.65248 .4057 .66082 .41819 .67807 .41849 .66978 .40595 Metetra
.501 .631 .91 r
.66978 .40595 .67807 .41849 .69549 .41889 .68724 .40631 Metetra
.497 .63 .911 r
.68724 .40631 .69549 .41889 .71308 .4194 .70488 .40678 Metetra
.494 .628 .911 r
.70488 .40678 .71308 .4194 .73086 .42002 .7227 .40736 Metetra
.491 .627 .912 r
.7227 .40736 .73086 .42002 .74882 .42074 .74071 .40804 Metetra
.488 .626 .912 r
.74071 .40804 .74882 .42074 .76696 .42156 .75891 .40882 Metetra
.484 .624 .912 r
.75891 .40882 .76696 .42156 .7853 .42248 .7773 .40971 Metetra
.481 .623 .913 r
.7773 .40971 .7853 .42248 .80383 .4235 .79589 .4107 Metetra
.478 .622 .913 r
.79589 .4107 .80383 .4235 .82257 .42462 .81468 .41178 Metetra
.475 .621 .914 r
.81468 .41178 .82257 .42462 .84151 .42583 .83367 .41297 Metetra
.473 .62 .914 r
.83367 .41297 .84151 .42583 .86065 .42714 .85288 .41424 Metetra
.616 .7 .904 r
.21372 .4372 .22319 .4474 .23711 .44435 .22768 .43403 Metetra
.613 .697 .904 r
.22768 .43403 .23711 .44435 .25111 .4414 .24172 .43096 Metetra
.609 .694 .904 r
.24172 .43096 .25111 .4414 .2652 .43854 .25585 .42799 Metetra
.605 .692 .904 r
.25585 .42799 .2652 .43854 .27937 .4358 .27006 .42513 Metetra
.602 .689 .904 r
.27006 .42513 .27937 .4358 .29363 .43316 .28435 .42238 Metetra
.598 .686 .904 r
.28435 .42238 .29363 .43316 .30798 .43064 .29874 .41974 Metetra
.594 .683 .904 r
.29874 .41974 .30798 .43064 .32242 .42822 .31322 .41722 Metetra
.59 .68 .904 r
.31322 .41722 .32242 .42822 .33695 .42593 .32779 .41481 Metetra
.586 .678 .904 r
.32779 .41481 .33695 .42593 .35159 .42375 .34246 .41253 Metetra
.582 .675 .904 r
.34246 .41253 .35159 .42375 .36632 .42169 .35723 .41036 Metetra
.578 .672 .904 r
.35723 .41036 .36632 .42169 .38116 .41975 .3721 .40832 Metetra
.574 .669 .904 r
.3721 .40832 .38116 .41975 .3961 .41794 .38708 .40641 Metetra
.569 .667 .904 r
.38708 .40641 .3961 .41794 .41116 .41625 .40217 .40462 Metetra
.565 .664 .904 r
.40217 .40462 .41116 .41625 .42632 .41468 .41737 .40297 Metetra
.561 .661 .904 r
.41737 .40297 .42632 .41468 .44161 .41324 .43269 .40143 Metetra
.557 .659 .904 r
.43269 .40143 .44161 .41324 .45701 .41192 .44813 .40003 Metetra
.553 .656 .905 r
.44813 .40003 .45701 .41192 .47253 .41073 .46369 .39876 Metetra
.548 .654 .905 r
.46369 .39876 .47253 .41073 .48819 .40966 .47938 .39761 Metetra
.544 .652 .905 r
.47938 .39761 .48819 .40966 .50397 .40872 .4952 .39659 Metetra
.54 .649 .905 r
.4952 .39659 .50397 .40872 .51989 .4079 .51115 .3957 Metetra
.536 .647 .906 r
.51115 .3957 .51989 .4079 .53594 .40721 .52724 .39494 Metetra
.532 .645 .906 r
.52724 .39494 .53594 .40721 .55213 .40663 .54347 .3943 Metetra
.528 .643 .906 r
.54347 .3943 .55213 .40663 .56847 .40618 .55985 .39379 Metetra
.524 .641 .907 r
.55985 .39379 .56847 .40618 .58496 .40585 .57638 .39339 Metetra
.52 .639 .907 r
.57638 .39339 .58496 .40585 .6016 .40564 .59306 .39313 Metetra
.516 .637 .908 r
.59306 .39313 .6016 .40564 .6184 .40555 .6099 .39298 Metetra
.512 .635 .908 r
.6099 .39298 .6184 .40555 .63536 .40557 .6269 .39295 Metetra
.508 .633 .908 r
.6269 .39295 .63536 .40557 .65248 .4057 .64407 .39304 Metetra
.504 .632 .909 r
.64407 .39304 .65248 .4057 .66978 .40595 .6614 .39324 Metetra
.501 .63 .909 r
.6614 .39324 .66978 .40595 .68724 .40631 .67891 .39356 Metetra
.497 .629 .91 r
.67891 .39356 .68724 .40631 .70488 .40678 .6966 .39399 Metetra
.494 .627 .91 r
.6966 .39399 .70488 .40678 .7227 .40736 .71447 .39453 Metetra
.49 .626 .911 r
.71447 .39453 .7227 .40736 .74071 .40804 .73252 .39517 Metetra
.487 .625 .911 r
.73252 .39517 .74071 .40804 .75891 .40882 .75077 .39592 Metetra
.484 .623 .912 r
.75077 .39592 .75891 .40882 .7773 .40971 .76921 .39678 Metetra
.481 .622 .912 r
.76921 .39678 .7773 .40971 .79589 .4107 .78786 .39773 Metetra
.478 .621 .913 r
.78786 .39773 .79589 .4107 .81468 .41178 .8067 .39879 Metetra
.475 .62 .913 r
.8067 .39879 .81468 .41178 .83367 .41297 .82576 .39994 Metetra
.472 .619 .914 r
.82576 .39994 .83367 .41297 .85288 .41424 .84503 .40119 Metetra
.618 .7 .903 r
.20416 .42691 .21372 .4372 .22768 .43403 .21816 .42362 Metetra
.615 .698 .903 r
.21816 .42362 .22768 .43403 .24172 .43096 .23225 .42042 Metetra
.611 .695 .903 r
.23225 .42042 .24172 .43096 .25585 .42799 .24641 .41733 Metetra
.608 .692 .903 r
.24641 .41733 .25585 .42799 .27006 .42513 .26066 .41435 Metetra
.604 .689 .903 r
.26066 .41435 .27006 .42513 .28435 .42238 .275 .41148 Metetra
.6 .686 .903 r
.275 .41148 .28435 .42238 .29874 .41974 .28942 .40872 Metetra
.596 .684 .903 r
.28942 .40872 .29874 .41974 .31322 .41722 .30394 .40608 Metetra
.592 .681 .903 r
.30394 .40608 .31322 .41722 .32779 .41481 .31855 .40356 Metetra
.588 .678 .903 r
.31855 .40356 .32779 .41481 .34246 .41253 .33325 .40117 Metetra
.584 .675 .903 r
.33325 .40117 .34246 .41253 .35723 .41036 .34806 .39889 Metetra
.58 .672 .903 r
.34806 .39889 .35723 .41036 .3721 .40832 .36297 .39675 Metetra
.575 .669 .903 r
.36297 .39675 .3721 .40832 .38708 .40641 .37798 .39473 Metetra
.571 .666 .903 r
.37798 .39473 .38708 .40641 .40217 .40462 .39311 .39285 Metetra
.567 .664 .903 r
.39311 .39285 .40217 .40462 .41737 .40297 .40835 .39109 Metetra
.562 .661 .903 r
.40835 .39109 .41737 .40297 .43269 .40143 .4237 .38947 Metetra
.558 .658 .903 r
.4237 .38947 .43269 .40143 .44813 .40003 .43917 .38797 Metetra
.554 .656 .903 r
.43917 .38797 .44813 .40003 .46369 .39876 .45477 .38661 Metetra
.549 .653 .904 r
.45477 .38661 .46369 .39876 .47938 .39761 .47049 .38539 Metetra
.545 .651 .904 r
.47049 .38539 .47938 .39761 .4952 .39659 .48635 .38429 Metetra
.541 .648 .904 r
.48635 .38429 .4952 .39659 .51115 .3957 .50234 .38333 Metetra
.537 .646 .905 r
.50234 .38333 .51115 .3957 .52724 .39494 .51846 .38249 Metetra
.532 .644 .905 r
.51846 .38249 .52724 .39494 .54347 .3943 .53473 .38179 Metetra
.528 .642 .905 r
.53473 .38179 .54347 .3943 .55985 .39379 .55115 .38121 Metetra
.524 .64 .906 r
.55115 .38121 .55985 .39379 .57638 .39339 .56771 .38076 Metetra
.52 .638 .906 r
.56771 .38076 .57638 .39339 .59306 .39313 .58443 .38043 Metetra
.516 .636 .907 r
.58443 .38043 .59306 .39313 .6099 .39298 .60131 .38023 Metetra
.512 .634 .907 r
.60131 .38023 .6099 .39298 .6269 .39295 .61835 .38015 Metetra
.508 .632 .907 r
.61835 .38015 .6269 .39295 .64407 .39304 .63556 .3802 Metetra
.504 .631 .908 r
.63556 .3802 .64407 .39304 .6614 .39324 .65294 .38036 Metetra
.501 .629 .908 r
.65294 .38036 .6614 .39324 .67891 .39356 .6705 .38063 Metetra
.497 .627 .909 r
.6705 .38063 .67891 .39356 .6966 .39399 .68823 .38102 Metetra
.494 .626 .91 r
.68823 .38102 .6966 .39399 .71447 .39453 .70615 .38152 Metetra
.49 .625 .91 r
.70615 .38152 .71447 .39453 .73252 .39517 .72425 .38213 Metetra
.487 .623 .911 r
.72425 .38213 .73252 .39517 .75077 .39592 .74255 .38285 Metetra
.483 .622 .911 r
.74255 .38285 .75077 .39592 .76921 .39678 .76105 .38367 Metetra
.48 .621 .912 r
.76105 .38367 .76921 .39678 .78786 .39773 .77974 .3846 Metetra
.477 .62 .912 r
.77974 .3846 .78786 .39773 .8067 .39879 .79865 .38562 Metetra
.474 .619 .913 r
.79865 .38562 .8067 .39879 .82576 .39994 .81776 .38675 Metetra
.471 .618 .914 r
.81776 .38675 .82576 .39994 .84503 .40119 .83709 .38797 Metetra
.621 .701 .903 r
.19451 .41652 .20416 .42691 .21816 .42362 .20856 .4131 Metetra
.617 .698 .902 r
.20856 .4131 .21816 .42362 .23225 .42042 .22268 .40978 Metetra
.614 .696 .902 r
.22268 .40978 .23225 .42042 .24641 .41733 .23689 .40656 Metetra
.61 .693 .902 r
.23689 .40656 .24641 .41733 .26066 .41435 .25118 .40346 Metetra
.606 .69 .902 r
.25118 .40346 .26066 .41435 .275 .41148 .26555 .40046 Metetra
.602 .687 .902 r
.26555 .40046 .275 .41148 .28942 .40872 .28001 .39758 Metetra
.598 .684 .902 r
.28001 .39758 .28942 .40872 .30394 .40608 .29457 .39482 Metetra
.594 .681 .902 r
.29457 .39482 .30394 .40608 .31855 .40356 .30922 .39218 Metetra
.59 .678 .902 r
.30922 .39218 .31855 .40356 .33325 .40117 .32396 .38966 Metetra
.586 .675 .901 r
.32396 .38966 .33325 .40117 .34806 .39889 .3388 .38728 Metetra
.581 .672 .901 r
.3388 .38728 .34806 .39889 .36297 .39675 .35375 .38502 Metetra
.577 .669 .902 r
.35375 .38502 .36297 .39675 .37798 .39473 .3688 .3829 Metetra
.573 .666 .902 r
.3688 .3829 .37798 .39473 .39311 .39285 .38396 .38091 Metetra
.568 .663 .902 r
.38396 .38091 .39311 .39285 .40835 .39109 .39924 .37905 Metetra
.564 .661 .902 r
.39924 .37905 .40835 .39109 .4237 .38947 .41463 .37733 Metetra
.56 .658 .902 r
.41463 .37733 .4237 .38947 .43917 .38797 .43014 .37575 Metetra
.555 .655 .902 r
.43014 .37575 .43917 .38797 .45477 .38661 .44577 .3743 Metetra
.551 .653 .902 r
.44577 .3743 .45477 .38661 .47049 .38539 .46153 .37299 Metetra
.546 .65 .903 r
.46153 .37299 .47049 .38539 .48635 .38429 .47742 .37181 Metetra
.542 .648 .903 r
.47742 .37181 .48635 .38429 .50234 .38333 .49344 .37077 Metetra
.537 .645 .903 r
.49344 .37077 .50234 .38333 .51846 .38249 .5096 .36986 Metetra
.533 .643 .904 r
.5096 .36986 .51846 .38249 .53473 .38179 .52591 .36909 Metetra
.529 .641 .904 r
.52591 .36909 .53473 .38179 .55115 .38121 .54236 .36845 Metetra
.525 .639 .905 r
.54236 .36845 .55115 .38121 .56771 .38076 .55897 .36794 Metetra
.52 .637 .905 r
.55897 .36794 .56771 .38076 .58443 .38043 .57573 .36756 Metetra
.516 .635 .905 r
.57573 .36756 .58443 .38043 .60131 .38023 .59264 .3673 Metetra
.512 .633 .906 r
.59264 .3673 .60131 .38023 .61835 .38015 .60973 .36717 Metetra
.508 .631 .906 r
.60973 .36717 .61835 .38015 .63556 .3802 .62698 .36717 Metetra
.504 .629 .907 r
.62698 .36717 .63556 .3802 .65294 .38036 .6444 .36729 Metetra
.501 .628 .908 r
.6444 .36729 .65294 .38036 .6705 .38063 .662 .36752 Metetra
.497 .626 .908 r
.662 .36752 .6705 .38063 .68823 .38102 .67978 .36787 Metetra
.493 .625 .909 r
.67978 .36787 .68823 .38102 .70615 .38152 .69774 .36834 Metetra
.49 .623 .909 r
.69774 .36834 .70615 .38152 .72425 .38213 .71589 .36892 Metetra
.486 .622 .91 r
.71589 .36892 .72425 .38213 .74255 .38285 .73424 .3696 Metetra
.483 .621 .911 r
.73424 .3696 .74255 .38285 .76105 .38367 .75279 .3704 Metetra
.479 .62 .911 r
.75279 .3704 .76105 .38367 .77974 .3846 .77154 .37129 Metetra
.476 .619 .912 r
.77154 .37129 .77974 .3846 .79865 .38562 .7905 .37229 Metetra
.473 .618 .913 r
.7905 .37229 .79865 .38562 .81776 .38675 .80968 .37339 Metetra
.47 .617 .913 r
.80968 .37339 .81776 .38675 .83709 .38797 .82906 .37459 Metetra
.623 .702 .902 r
.18477 .40605 .19451 .41652 .20856 .4131 .19886 .40249 Metetra
.62 .699 .902 r
.19886 .40249 .20856 .4131 .22268 .40978 .21303 .39904 Metetra
.616 .696 .901 r
.21303 .39904 .22268 .40978 .23689 .40656 .22728 .39569 Metetra
.612 .693 .901 r
.22728 .39569 .23689 .40656 .25118 .40346 .24161 .39245 Metetra
.609 .69 .901 r
.24161 .39245 .25118 .40346 .26555 .40046 .25602 .38932 Metetra
.605 .687 .901 r
.25602 .38932 .26555 .40046 .28001 .39758 .27053 .38631 Metetra
.601 .684 .901 r
.27053 .38631 .28001 .39758 .29457 .39482 .28512 .38342 Metetra
.596 .681 .9 r
.28512 .38342 .29457 .39482 .30922 .39218 .29981 .38065 Metetra
.592 .678 .9 r
.29981 .38065 .30922 .39218 .32396 .38966 .31459 .37802 Metetra
.588 .675 .9 r
.31459 .37802 .32396 .38966 .3388 .38728 .32947 .37551 Metetra
.583 .672 .9 r
.32947 .37551 .3388 .38728 .35375 .38502 .34445 .37314 Metetra
.579 .669 .9 r
.34445 .37314 .35375 .38502 .3688 .3829 .35954 .3709 Metetra
.575 .666 .9 r
.35954 .3709 .3688 .3829 .38396 .38091 .37474 .3688 Metetra
.57 .663 .9 r
.37474 .3688 .38396 .38091 .39924 .37905 .39005 .36684 Metetra
.566 .66 .901 r
.39005 .36684 .39924 .37905 .41463 .37733 .40547 .36502 Metetra
.561 .657 .901 r
.40547 .36502 .41463 .37733 .43014 .37575 .42102 .36334 Metetra
.556 .654 .901 r
.42102 .36334 .43014 .37575 .44577 .3743 .43668 .3618 Metetra
.552 .652 .901 r
.43668 .3618 .44577 .3743 .46153 .37299 .45248 .3604 Metetra
.547 .649 .901 r
.45248 .3604 .46153 .37299 .47742 .37181 .4684 .35914 Metetra
.543 .647 .902 r
.4684 .35914 .47742 .37181 .49344 .37077 .48446 .35802 Metetra
.538 .644 .902 r
.48446 .35802 .49344 .37077 .5096 .36986 .50066 .35704 Metetra
.534 .642 .902 r
.50066 .35704 .5096 .36986 .52591 .36909 .517 .3562 Metetra
.529 .64 .903 r
.517 .3562 .52591 .36909 .54236 .36845 .53349 .35549 Metetra
.525 .637 .903 r
.53349 .35549 .54236 .36845 .55897 .36794 .55014 .35492 Metetra
.521 .635 .904 r
.55014 .35492 .55897 .36794 .57573 .36756 .56693 .35448 Metetra
.517 .633 .904 r
.56693 .35448 .57573 .36756 .59264 .3673 .58389 .35418 Metetra
.512 .631 .905 r
.58389 .35418 .59264 .3673 .60973 .36717 .60101 .354 Metetra
.508 .63 .905 r
.60101 .354 .60973 .36717 .62698 .36717 .6183 .35395 Metetra
.504 .628 .906 r
.6183 .35395 .62698 .36717 .6444 .36729 .63577 .35402 Metetra
.5 .626 .907 r
.63577 .35402 .6444 .36729 .662 .36752 .65341 .35422 Metetra
.496 .625 .907 r
.65341 .35422 .662 .36752 .67978 .36787 .67124 .35454 Metetra
.493 .623 .908 r
.67124 .35454 .67978 .36787 .69774 .36834 .68925 .35497 Metetra
.489 .622 .909 r
.68925 .35497 .69774 .36834 .71589 .36892 .70745 .35552 Metetra
.485 .621 .909 r
.70745 .35552 .71589 .36892 .73424 .3696 .72585 .35618 Metetra
.482 .62 .91 r
.72585 .35618 .73424 .3696 .75279 .3704 .74445 .35694 Metetra
.479 .619 .911 r
.74445 .35694 .75279 .3704 .77154 .37129 .76326 .35782 Metetra
.475 .618 .911 r
.76326 .35782 .77154 .37129 .7905 .37229 .78227 .35879 Metetra
.472 .617 .912 r
.78227 .35879 .7905 .37229 .80968 .37339 .8015 .35987 Metetra
.469 .616 .913 r
.8015 .35987 .80968 .37339 .82906 .37459 .82095 .36105 Metetra
.626 .703 .901 r
.17494 .39548 .18477 .40605 .19886 .40249 .18908 .39179 Metetra
.622 .7 .901 r
.18908 .39179 .19886 .40249 .21303 .39904 .20329 .38819 Metetra
.619 .697 .9 r
.20329 .38819 .21303 .39904 .22728 .39569 .21758 .3847 Metetra
.615 .694 .9 r
.21758 .3847 .22728 .39569 .24161 .39245 .23195 .38132 Metetra
.611 .691 .9 r
.23195 .38132 .24161 .39245 .25602 .38932 .24641 .37806 Metetra
.607 .688 .9 r
.24641 .37806 .25602 .38932 .27053 .38631 .26095 .37491 Metetra
.603 .685 .899 r
.26095 .37491 .27053 .38631 .28512 .38342 .27558 .37188 Metetra
.599 .681 .899 r
.27558 .37188 .28512 .38342 .29981 .38065 .29031 .36898 Metetra
.594 .678 .899 r
.29031 .36898 .29981 .38065 .31459 .37802 .30513 .36622 Metetra
.59 .675 .899 r
.30513 .36622 .31459 .37802 .32947 .37551 .32005 .36358 Metetra
.586 .672 .899 r
.32005 .36358 .32947 .37551 .34445 .37314 .33507 .36109 Metetra
.581 .669 .899 r
.33507 .36109 .34445 .37314 .35954 .3709 .3502 .35873 Metetra
.577 .666 .899 r
.3502 .35873 .35954 .3709 .37474 .3688 .36543 .35652 Metetra
.572 .662 .899 r
.36543 .35652 .37474 .3688 .39005 .36684 .38077 .35445 Metetra
.567 .659 .899 r
.38077 .35445 .39005 .36684 .40547 .36502 .39623 .35252 Metetra
.563 .657 .899 r
.39623 .35252 .40547 .36502 .42102 .36334 .41181 .35074 Metetra
.558 .654 .899 r
.41181 .35074 .42102 .36334 .43668 .3618 .42752 .3491 Metetra
.553 .651 .9 r
.42752 .3491 .43668 .3618 .45248 .3604 .44335 .34761 Metetra
.549 .648 .9 r
.44335 .34761 .45248 .3604 .4684 .35914 .45931 .34627 Metetra
.544 .646 .9 r
.45931 .34627 .4684 .35914 .48446 .35802 .4754 .34507 Metetra
.539 .643 .901 r
.4754 .34507 .48446 .35802 .50066 .35704 .49164 .34401 Metetra
.535 .641 .901 r
.49164 .34401 .50066 .35704 .517 .3562 .50801 .3431 Metetra
.53 .638 .901 r
.50801 .3431 .517 .3562 .53349 .35549 .52454 .34233 Metetra
.526 .636 .902 r
.52454 .34233 .53349 .35549 .55014 .35492 .54122 .3417 Metetra
.521 .634 .903 r
.54122 .3417 .55014 .35492 .56693 .35448 .55805 .34121 Metetra
.517 .632 .903 r
.55805 .34121 .56693 .35448 .58389 .35418 .57505 .34085 Metetra
.512 .63 .904 r
.57505 .34085 .58389 .35418 .60101 .354 .59221 .34063 Metetra
.508 .628 .904 r
.59221 .34063 .60101 .354 .6183 .35395 .60954 .34053 Metetra
.504 .626 .905 r
.60954 .34053 .6183 .35395 .63577 .35402 .62705 .34057 Metetra
.5 .625 .906 r
.62705 .34057 .63577 .35402 .65341 .35422 .64474 .34073 Metetra
.496 .623 .906 r
.64474 .34073 .65341 .35422 .67124 .35454 .66261 .34101 Metetra
.492 .622 .907 r
.66261 .34101 .67124 .35454 .68925 .35497 .68067 .34141 Metetra
.488 .621 .908 r
.68067 .34141 .68925 .35497 .70745 .35552 .69892 .34193 Metetra
.485 .62 .908 r
.69892 .34193 .70745 .35552 .72585 .35618 .71737 .34257 Metetra
.481 .618 .909 r
.71737 .34257 .72585 .35618 .74445 .35694 .73602 .34331 Metetra
.478 .617 .91 r
.73602 .34331 .74445 .35694 .76326 .35782 .75488 .34416 Metetra
.474 .616 .911 r
.75488 .34416 .76326 .35782 .78227 .35879 .77395 .34512 Metetra
.471 .616 .912 r
.77395 .34512 .78227 .35879 .8015 .35987 .79324 .34618 Metetra
.468 .615 .912 r
.79324 .34618 .8015 .35987 .82095 .36105 .81275 .34733 Metetra
.628 .704 .9 r
.16502 .38483 .17494 .39548 .18908 .39179 .1792 .38099 Metetra
.625 .701 .9 r
.1792 .38099 .18908 .39179 .20329 .38819 .19345 .37725 Metetra
.621 .698 .9 r
.19345 .37725 .20329 .38819 .21758 .3847 .20779 .37361 Metetra
.618 .695 .899 r
.20779 .37361 .21758 .3847 .23195 .38132 .2222 .37008 Metetra
.614 .692 .899 r
.2222 .37008 .23195 .38132 .24641 .37806 .2367 .36667 Metetra
.61 .688 .899 r
.2367 .36667 .24641 .37806 .26095 .37491 .25129 .36338 Metetra
.606 .685 .898 r
.25129 .36338 .26095 .37491 .27558 .37188 .26596 .36021 Metetra
.601 .682 .898 r
.26596 .36021 .27558 .37188 .29031 .36898 .28073 .35717 Metetra
.597 .678 .898 r
.28073 .35717 .29031 .36898 .30513 .36622 .29559 .35426 Metetra
.593 .675 .898 r
.29559 .35426 .30513 .36622 .32005 .36358 .31055 .3515 Metetra
.588 .672 .898 r
.31055 .3515 .32005 .36358 .33507 .36109 .32561 .34887 Metetra
.583 .668 .897 r
.32561 .34887 .33507 .36109 .3502 .35873 .34077 .34639 Metetra
.579 .665 .897 r
.34077 .34639 .3502 .35873 .36543 .35652 .35604 .34405 Metetra
.574 .662 .897 r
.35604 .34405 .36543 .35652 .38077 .35445 .37142 .34187 Metetra
.569 .659 .898 r
.37142 .34187 .38077 .35445 .39623 .35252 .38692 .33983 Metetra
.564 .656 .898 r
.38692 .33983 .39623 .35252 .41181 .35074 .40253 .33794 Metetra
.559 .653 .898 r
.40253 .33794 .41181 .35074 .42752 .3491 .41827 .3362 Metetra
.555 .65 .898 r
.41827 .3362 .42752 .3491 .44335 .34761 .43413 .33462 Metetra
.55 .647 .898 r
.43413 .33462 .44335 .34761 .45931 .34627 .45013 .33319 Metetra
.545 .644 .899 r
.45013 .33319 .45931 .34627 .4754 .34507 .46626 .33191 Metetra
.54 .642 .899 r
.46626 .33191 .4754 .34507 .49164 .34401 .48253 .33078 Metetra
.535 .639 .899 r
.48253 .33078 .49164 .34401 .50801 .3431 .49894 .32979 Metetra
.531 .637 .9 r
.49894 .32979 .50801 .3431 .52454 .34233 .5155 .32896 Metetra
.526 .635 .901 r
.5155 .32896 .52454 .34233 .54122 .3417 .53222 .32827 Metetra
.521 .632 .901 r
.53222 .32827 .54122 .3417 .55805 .34121 .54909 .32772 Metetra
.517 .63 .902 r
.54909 .32772 .55805 .34121 .57505 .34085 .56613 .32731 Metetra
.512 .628 .902 r
.56613 .32731 .57505 .34085 .59221 .34063 .58333 .32704 Metetra
.508 .627 .903 r
.58333 .32704 .59221 .34063 .60954 .34053 .6007 .32691 Metetra
.504 .625 .904 r
.6007 .32691 .60954 .34053 .62705 .34057 .61825 .3269 Metetra
.499 .623 .905 r
.61825 .3269 .62705 .34057 .64474 .34073 .63598 .32703 Metetra
.495 .622 .905 r
.63598 .32703 .64474 .34073 .66261 .34101 .65389 .32729 Metetra
.491 .62 .906 r
.65389 .32729 .66261 .34101 .68067 .34141 .672 .32766 Metetra
.487 .619 .907 r
.672 .32766 .68067 .34141 .69892 .34193 .6903 .32816 Metetra
.484 .618 .908 r
.6903 .32816 .69892 .34193 .71737 .34257 .7088 .32877 Metetra
.48 .617 .909 r
.7088 .32877 .71737 .34257 .73602 .34331 .7275 .32949 Metetra
.476 .616 .909 r
.7275 .32949 .73602 .34331 .75488 .34416 .74642 .33033 Metetra
.473 .615 .91 r
.74642 .33033 .75488 .34416 .77395 .34512 .76555 .33127 Metetra
.47 .614 .911 r
.76555 .33127 .77395 .34512 .79324 .34618 .78489 .33231 Metetra
.466 .613 .912 r
.78489 .33231 .79324 .34618 .81275 .34733 .80447 .33345 Metetra
.631 .706 .9 r
.155 .3741 .16502 .38483 .1792 .38099 .16922 .37011 Metetra
.628 .703 .899 r
.16922 .37011 .1792 .38099 .19345 .37725 .18352 .36621 Metetra
.624 .699 .899 r
.18352 .36621 .19345 .37725 .20779 .37361 .1979 .36242 Metetra
.621 .696 .898 r
.1979 .36242 .20779 .37361 .2222 .37008 .21236 .35873 Metetra
.617 .693 .898 r
.21236 .35873 .2222 .37008 .2367 .36667 .22691 .35516 Metetra
.613 .689 .897 r
.22691 .35516 .2367 .36667 .25129 .36338 .24154 .35172 Metetra
.608 .686 .897 r
.24154 .35172 .25129 .36338 .26596 .36021 .25625 .34839 Metetra
.604 .682 .897 r
.25625 .34839 .26596 .36021 .28073 .35717 .27106 .3452 Metetra
.6 .679 .896 r
.27106 .3452 .28073 .35717 .29559 .35426 .28596 .34215 Metetra
.595 .675 .896 r
.28596 .34215 .29559 .35426 .31055 .3515 .30096 .33924 Metetra
.59 .672 .896 r
.30096 .33924 .31055 .3515 .32561 .34887 .31606 .33648 Metetra
.586 .668 .896 r
.31606 .33648 .32561 .34887 .34077 .34639 .33126 .33386 Metetra
.581 .665 .896 r
.33126 .33386 .34077 .34639 .35604 .34405 .34657 .3314 Metetra
.576 .661 .896 r
.34657 .3314 .35604 .34405 .37142 .34187 .36198 .32908 Metetra
.571 .658 .896 r
.36198 .32908 .37142 .34187 .38692 .33983 .37752 .32693 Metetra
.566 .655 .896 r
.37752 .32693 .38692 .33983 .40253 .33794 .39317 .32493 Metetra
.561 .652 .896 r
.39317 .32493 .40253 .33794 .41827 .3362 .40894 .32309 Metetra
.556 .649 .896 r
.40894 .32309 .41827 .3362 .43413 .33462 .42484 .32141 Metetra
.551 .646 .897 r
.42484 .32141 .43413 .33462 .45013 .33319 .44087 .31989 Metetra
.546 .643 .897 r
.44087 .31989 .45013 .33319 .46626 .33191 .45703 .31852 Metetra
.541 .64 .897 r
.45703 .31852 .46626 .33191 .48253 .33078 .47334 .31731 Metetra
.536 .638 .898 r
.47334 .31731 .48253 .33078 .49894 .32979 .48978 .31626 Metetra
.531 .635 .898 r
.48978 .31626 .49894 .32979 .5155 .32896 .50638 .31535 Metetra
.526 .633 .899 r
.50638 .31535 .5155 .32896 .53222 .32827 .52313 .3146 Metetra
.522 .631 .9 r
.52313 .3146 .53222 .32827 .54909 .32772 .54004 .314 Metetra
.517 .629 .9 r
.54004 .314 .54909 .32772 .56613 .32731 .55711 .31355 Metetra
.512 .627 .901 r
.55711 .31355 .56613 .32731 .58333 .32704 .57435 .31324 Metetra
.508 .625 .902 r
.57435 .31324 .58333 .32704 .6007 .32691 .59177 .31306 Metetra
.503 .623 .902 r
.59177 .31306 .6007 .32691 .61825 .3269 .60936 .31303 Metetra
.499 .621 .903 r
.60936 .31303 .61825 .3269 .63598 .32703 .62713 .31313 Metetra
.495 .62 .904 r
.62713 .31313 .63598 .32703 .65389 .32729 .64509 .31335 Metetra
.49 .619 .905 r
.64509 .31335 .65389 .32729 .672 .32766 .66324 .31371 Metetra
.486 .617 .906 r
.66324 .31371 .672 .32766 .6903 .32816 .68159 .31419 Metetra
.483 .616 .907 r
.68159 .31419 .6903 .32816 .7088 .32877 .70014 .31478 Metetra
.479 .615 .908 r
.70014 .31478 .7088 .32877 .7275 .32949 .71889 .31549 Metetra
.475 .614 .909 r
.71889 .31549 .7275 .32949 .74642 .33033 .73786 .31631 Metetra
.471 .613 .91 r
.73786 .31631 .74642 .33033 .76555 .33127 .75705 .31724 Metetra
.468 .613 .911 r
.75705 .31724 .76555 .33127 .78489 .33231 .77646 .31828 Metetra
.465 .612 .912 r
.77646 .31828 .78489 .33231 .80447 .33345 .79609 .31941 Metetra
.634 .707 .899 r
.14488 .3633 .155 .3741 .16922 .37011 .15915 .35915 Metetra
.631 .704 .899 r
.15915 .35915 .16922 .37011 .18352 .36621 .1735 .35509 Metetra
.627 .701 .898 r
.1735 .35509 .18352 .36621 .1979 .36242 .18793 .35113 Metetra
.624 .697 .897 r
.18793 .35113 .1979 .36242 .21236 .35873 .20243 .34727 Metetra
.62 .694 .897 r
.20243 .34727 .21236 .35873 .22691 .35516 .21702 .34354 Metetra
.616 .69 .896 r
.21702 .34354 .22691 .35516 .24154 .35172 .2317 .33992 Metetra
.611 .686 .896 r
.2317 .33992 .24154 .35172 .25625 .34839 .24646 .33644 Metetra
.607 .683 .895 r
.24646 .33644 .25625 .34839 .27106 .3452 .26131 .33309 Metetra
.603 .679 .895 r
.26131 .33309 .27106 .3452 .28596 .34215 .27625 .32988 Metetra
.598 .675 .895 r
.27625 .32988 .28596 .34215 .30096 .33924 .29129 .32681 Metetra
.593 .672 .894 r
.29129 .32681 .30096 .33924 .31606 .33648 .30643 .3239 Metetra
.588 .668 .894 r
.30643 .3239 .31606 .33648 .33126 .33386 .32167 .32114 Metetra
.583 .664 .894 r
.32167 .32114 .33126 .33386 .34657 .3314 .33701 .31854 Metetra
.578 .661 .894 r
.33701 .31854 .34657 .3314 .36198 .32908 .35247 .31609 Metetra
.573 .657 .894 r
.35247 .31609 .36198 .32908 .37752 .32693 .36803 .31381 Metetra
.568 .654 .894 r
.36803 .31381 .37752 .32693 .39317 .32493 .38372 .3117 Metetra
.563 .651 .894 r
.38372 .3117 .39317 .32493 .40894 .32309 .39953 .30975 Metetra
.558 .647 .894 r
.39953 .30975 .40894 .32309 .42484 .32141 .41546 .30796 Metetra
.552 .644 .895 r
.41546 .30796 .42484 .32141 .44087 .31989 .43152 .30634 Metetra
.547 .641 .895 r
.43152 .30634 .44087 .31989 .45703 .31852 .44772 .30489 Metetra
.542 .638 .895 r
.44772 .30489 .45703 .31852 .47334 .31731 .46406 .3036 Metetra
.537 .636 .896 r
.46406 .3036 .47334 .31731 .48978 .31626 .48054 .30247 Metetra
.532 .633 .896 r
.48054 .30247 .48978 .31626 .50638 .31535 .49718 .30151 Metetra
.527 .631 .897 r
.49718 .30151 .50638 .31535 .52313 .3146 .51396 .3007 Metetra
.522 .629 .898 r
.51396 .3007 .52313 .3146 .54004 .314 .53091 .30005 Metetra
.517 .626 .899 r
.53091 .30005 .54004 .314 .55711 .31355 .54802 .29955 Metetra
.512 .624 .899 r
.54802 .29955 .55711 .31355 .57435 .31324 .56529 .2992 Metetra
.507 .623 .9 r
.56529 .2992 .57435 .31324 .59177 .31306 .58275 .29899 Metetra
.503 .621 .901 r
.58275 .29899 .59177 .31306 .60936 .31303 .60038 .29893 Metetra
.498 .619 .902 r
.60038 .29893 .60936 .31303 .62713 .31313 .61819 .299 Metetra
.494 .618 .903 r
.61819 .299 .62713 .31313 .64509 .31335 .63619 .29921 Metetra
.489 .617 .904 r
.63619 .29921 .64509 .31335 .66324 .31371 .65439 .29955 Metetra
.485 .615 .905 r
.65439 .29955 .66324 .31371 .68159 .31419 .67279 .30001 Metetra
.481 .614 .906 r
.67279 .30001 .68159 .31419 .70014 .31478 .69139 .3006 Metetra
.477 .613 .907 r
.69139 .3006 .70014 .31478 .71889 .31549 .7102 .3013 Metetra
.473 .613 .908 r
.7102 .3013 .71889 .31549 .73786 .31631 .72922 .30212 Metetra
.47 .612 .909 r
.72922 .30212 .73786 .31631 .75705 .31724 .74846 .30304 Metetra
.466 .611 .91 r
.74846 .30304 .75705 .31724 .77646 .31828 .76793 .30407 Metetra
.463 .611 .911 r
.76793 .30407 .77646 .31828 .79609 .31941 .78762 .3052 Metetra
.637 .709 .899 r
.13466 .35244 .14488 .3633 .15915 .35915 .14898 .34811 Metetra
.634 .706 .898 r
.14898 .34811 .15915 .35915 .1735 .35509 .16338 .34388 Metetra
.631 .702 .897 r
.16338 .34388 .1735 .35509 .18793 .35113 .17785 .33974 Metetra
.627 .699 .896 r
.17785 .33974 .18793 .35113 .20243 .34727 .19241 .33571 Metetra
.623 .695 .896 r
.19241 .33571 .20243 .34727 .21702 .34354 .20705 .3318 Metetra
.619 .691 .895 r
.20705 .3318 .21702 .34354 .2317 .33992 .22177 .328 Metetra
.615 .687 .895 r
.22177 .328 .2317 .33992 .24646 .33644 .23657 .32434 Metetra
.61 .683 .894 r
.23657 .32434 .24646 .33644 .26131 .33309 .25147 .32082 Metetra
.606 .679 .893 r
.25147 .32082 .26131 .33309 .27625 .32988 .26646 .31744 Metetra
.601 .675 .893 r
.26646 .31744 .27625 .32988 .29129 .32681 .28154 .3142 Metetra
.596 .672 .893 r
.28154 .3142 .29129 .32681 .30643 .3239 .29671 .31113 Metetra
.591 .668 .892 r
.29671 .31113 .30643 .3239 .32167 .32114 .31199 .30821 Metetra
.586 .664 .892 r
.31199 .30821 .32167 .32114 .33701 .31854 .32738 .30546 Metetra
.581 .66 .892 r
.32738 .30546 .33701 .31854 .35247 .31609 .34287 .30288 Metetra
.575 .656 .892 r
.34287 .30288 .35247 .31609 .36803 .31381 .35847 .30046 Metetra
.57 .653 .892 r
.35847 .30046 .36803 .31381 .38372 .3117 .37419 .29822 Metetra
.565 .649 .892 r
.37419 .29822 .38372 .3117 .39953 .30975 .39003 .29616 Metetra
.559 .646 .892 r
.39003 .29616 .39953 .30975 .41546 .30796 .406 .29426 Metetra
.554 .643 .892 r
.406 .29426 .41546 .30796 .43152 .30634 .4221 .29255 Metetra
.548 .639 .893 r
.4221 .29255 .43152 .30634 .44772 .30489 .43833 .291 Metetra
.543 .637 .893 r
.43833 .291 .44772 .30489 .46406 .3036 .4547 .28963 Metetra
.538 .634 .894 r
.4547 .28963 .46406 .3036 .48054 .30247 .47122 .28843 Metetra
.532 .631 .894 r
.47122 .28843 .48054 .30247 .49718 .30151 .48788 .2874 Metetra
.527 .629 .895 r
.48788 .2874 .49718 .30151 .51396 .3007 .5047 .28654 Metetra
.522 .626 .896 r
.5047 .28654 .51396 .3007 .53091 .30005 .52168 .28584 Metetra
.517 .624 .897 r
.52168 .28584 .53091 .30005 .54802 .29955 .53883 .2853 Metetra
.512 .622 .898 r
.53883 .2853 .54802 .29955 .56529 .2992 .55615 .28491 Metetra
.507 .62 .899 r
.55615 .28491 .56529 .2992 .58275 .29899 .57364 .28468 Metetra
.502 .619 .9 r
.57364 .28468 .58275 .29899 .60038 .29893 .59131 .28459 Metetra
.497 .617 .901 r
.59131 .28459 .60038 .29893 .61819 .299 .60916 .28465 Metetra
.493 .616 .902 r
.60916 .28465 .61819 .299 .63619 .29921 .62721 .28485 Metetra
.488 .614 .903 r
.62721 .28485 .63619 .29921 .65439 .29955 .64545 .28518 Metetra
.484 .613 .904 r
.64545 .28518 .65439 .29955 .67279 .30001 .66389 .28564 Metetra
.479 .612 .905 r
.66389 .28564 .67279 .30001 .69139 .3006 .68254 .28622 Metetra
.475 .612 .906 r
.68254 .28622 .69139 .3006 .7102 .3013 .7014 .28692 Metetra
.471 .611 .907 r
.7014 .28692 .7102 .3013 .72922 .30212 .72048 .28774 Metetra
.468 .61 .909 r
.72048 .28774 .72922 .30212 .74846 .30304 .73978 .28867 Metetra
.464 .61 .91 r
.73978 .28867 .74846 .30304 .76793 .30407 .75931 .2897 Metetra
.461 .609 .911 r
.75931 .2897 .76793 .30407 .78762 .3052 .77906 .29083 Metetra
.641 .711 .898 r
.12433 .34152 .13466 .35244 .14898 .34811 .1387 .33701 Metetra
.638 .708 .897 r
.1387 .33701 .14898 .34811 .16338 .34388 .15315 .3326 Metetra
.634 .704 .896 r
.15315 .3326 .16338 .34388 .17785 .33974 .16768 .32827 Metetra
.631 .7 .896 r
.16768 .32827 .17785 .33974 .19241 .33571 .18229 .32405 Metetra
.627 .696 .895 r
.18229 .32405 .19241 .33571 .20705 .3318 .19698 .31994 Metetra
.623 .692 .894 r
.19698 .31994 .20705 .3318 .22177 .328 .21175 .31596 Metetra
.618 .688 .893 r
.21175 .31596 .22177 .328 .23657 .32434 .2266 .3121 Metetra
.614 .684 .892 r
.2266 .3121 .23657 .32434 .25147 .32082 .24154 .30839 Metetra
.609 .68 .892 r
.24154 .30839 .25147 .32082 .26646 .31744 .25657 .30482 Metetra
.604 .676 .891 r
.25657 .30482 .26646 .31744 .28154 .3142 .2717 .30141 Metetra
.599 .672 .891 r
.2717 .30141 .28154 .3142 .29671 .31113 .28692 .29816 Metetra
.594 .667 .89 r
.28692 .29816 .29671 .31113 .31199 .30821 .30224 .29507 Metetra
.589 .663 .89 r
.30224 .29507 .31199 .30821 .32738 .30546 .31766 .29216 Metetra
.584 .659 .889 r
.31766 .29216 .32738 .30546 .34287 .30288 .33319 .28942 Metetra
.578 .655 .889 r
.33319 .28942 .34287 .30288 .35847 .30046 .34883 .28686 Metetra
.572 .651 .889 r
.34883 .28686 .35847 .30046 .37419 .29822 .36458 .28449 Metetra
.567 .648 .889 r
.36458 .28449 .37419 .29822 .39003 .29616 .38046 .28229 Metetra
.561 .644 .889 r
.38046 .28229 .39003 .29616 .406 .29426 .39646 .28029 Metetra
.555 .641 .89 r
.39646 .28029 .406 .29426 .4221 .29255 .41259 .27846 Metetra
.55 .637 .89 r
.41259 .27846 .4221 .29255 .43833 .291 .42886 .27683 Metetra
.544 .634 .891 r
.42886 .27683 .43833 .291 .4547 .28963 .44526 .27537 Metetra
.538 .631 .891 r
.44526 .27537 .4547 .28963 .47122 .28843 .46181 .2741 Metetra
.533 .629 .892 r
.46181 .2741 .47122 .28843 .48788 .2874 .47851 .27301 Metetra
.527 .626 .893 r
.47851 .27301 .48788 .2874 .5047 .28654 .49536 .27209 Metetra
.522 .624 .894 r
.49536 .27209 .5047 .28654 .52168 .28584 .51238 .27135 Metetra
.516 .621 .895 r
.51238 .27135 .52168 .28584 .53883 .2853 .52956 .27077 Metetra
.511 .619 .896 r
.52956 .27077 .53883 .2853 .55615 .28491 .54691 .27036 Metetra
.506 .618 .897 r
.54691 .27036 .55615 .28491 .57364 .28468 .56444 .27011 Metetra
.501 .616 .898 r
.56444 .27011 .57364 .28468 .59131 .28459 .58214 .27001 Metetra
.496 .614 .899 r
.58214 .27001 .59131 .28459 .60916 .28465 .60004 .27006 Metetra
.491 .613 .9 r
.60004 .27006 .60916 .28465 .62721 .28485 .61813 .27026 Metetra
.486 .612 .902 r
.61813 .27026 .62721 .28485 .64545 .28518 .63642 .27059 Metetra
.482 .611 .903 r
.63642 .27059 .64545 .28518 .66389 .28564 .65491 .27105 Metetra
.477 .61 .904 r
.65491 .27105 .66389 .28564 .68254 .28622 .67361 .27164 Metetra
.473 .609 .906 r
.67361 .27164 .68254 .28622 .7014 .28692 .69252 .27236 Metetra
.469 .609 .907 r
.69252 .27236 .7014 .28692 .72048 .28774 .71165 .27319 Metetra
.465 .608 .908 r
.71165 .27319 .72048 .28774 .73978 .28867 .73101 .27413 Metetra
.462 .608 .91 r
.73101 .27413 .73978 .28867 .75931 .2897 .75059 .27517 Metetra
.458 .607 .911 r
.75059 .27517 .75931 .2897 .77906 .29083 .77041 .27632 Metetra
.644 .714 .898 r
.11389 .33057 .12433 .34152 .1387 .33701 .12832 .32587 Metetra
.641 .71 .897 r
.12832 .32587 .1387 .33701 .15315 .3326 .14282 .32126 Metetra
.638 .706 .896 r
.14282 .32126 .15315 .3326 .16768 .32827 .15741 .31673 Metetra
.635 .702 .895 r
.15741 .31673 .16768 .32827 .18229 .32405 .17207 .31231 Metetra
.631 .698 .894 r
.17207 .31231 .18229 .32405 .19698 .31994 .18681 .30799 Metetra
.627 .694 .893 r
.18681 .30799 .19698 .31994 .21175 .31596 .20163 .3038 Metetra
.622 .69 .892 r
.20163 .3038 .21175 .31596 .2266 .3121 .21653 .29973 Metetra
.618 .685 .891 r
.21653 .29973 .2266 .3121 .24154 .30839 .23153 .29581 Metetra
.613 .681 .89 r
.23153 .29581 .24154 .30839 .25657 .30482 .2466 .29203 Metetra
.608 .676 .889 r
.2466 .29203 .25657 .30482 .2717 .30141 .26177 .28842 Metetra
.603 .672 .888 r
.26177 .28842 .2717 .30141 .28692 .29816 .27704 .28497 Metetra
.598 .667 .888 r
.27704 .28497 .28692 .29816 .30224 .29507 .2924 .2817 Metetra
.592 .663 .887 r
.2924 .2817 .30224 .29507 .31766 .29216 .30786 .27861 Metetra
.587 .658 .887 r
.30786 .27861 .31766 .29216 .33319 .28942 .32343 .2757 Metetra
.581 .654 .887 r
.32343 .2757 .33319 .28942 .34883 .28686 .33911 .27298 Metetra
.575 .65 .886 r
.33911 .27298 .34883 .28686 .36458 .28449 .3549 .27046 Metetra
.569 .646 .886 r
.3549 .27046 .36458 .28449 .38046 .28229 .37081 .26813 Metetra
.563 .642 .886 r
.37081 .26813 .38046 .28229 .39646 .28029 .38684 .266 Metetra
.557 .638 .887 r
.38684 .266 .39646 .28029 .41259 .27846 .40301 .26407 Metetra
.551 .635 .887 r
.40301 .26407 .41259 .27846 .42886 .27683 .4193 .26234 Metetra
.545 .631 .888 r
.4193 .26234 .42886 .27683 .44526 .27537 .43574 .2608 Metetra
.539 .628 .888 r
.43574 .2608 .44526 .27537 .46181 .2741 .45232 .25945 Metetra
.533 .625 .889 r
.45232 .25945 .46181 .2741 .47851 .27301 .46905 .2583 Metetra
.527 .623 .89 r
.46905 .2583 .47851 .27301 .49536 .27209 .48593 .25734 Metetra
.522 .62 .891 r
.48593 .25734 .49536 .27209 .51238 .27135 .50298 .25656 Metetra
.516 .618 .892 r
.50298 .25656 .51238 .27135 .52956 .27077 .5202 .25596 Metetra
.51 .616 .893 r
.5202 .25596 .52956 .27077 .54691 .27036 .53758 .25553 Metetra
.505 .614 .895 r
.53758 .25553 .54691 .27036 .56444 .27011 .55515 .25527 Metetra
.499 .613 .896 r
.55515 .25527 .56444 .27011 .58214 .27001 .57289 .25517 Metetra
.494 .611 .897 r
.57289 .25517 .58214 .27001 .60004 .27006 .59083 .25522 Metetra
.489 .61 .899 r
.59083 .25522 .60004 .27006 .61813 .27026 .60896 .25543 Metetra
.484 .609 .9 r
.60896 .25543 .61813 .27026 .63642 .27059 .62729 .25578 Metetra
.479 .608 .902 r
.62729 .25578 .63642 .27059 .65491 .27105 .64583 .25626 Metetra
.475 .608 .903 r
.64583 .25626 .65491 .27105 .67361 .27164 .66458 .25687 Metetra
.47 .607 .905 r
.66458 .25687 .67361 .27164 .69252 .27236 .68354 .25761 Metetra
.466 .607 .907 r
.68354 .25761 .69252 .27236 .71165 .27319 .70273 .25846 Metetra
.462 .606 .908 r
.70273 .25846 .71165 .27319 .73101 .27413 .72214 .25943 Metetra
.458 .606 .909 r
.72214 .25943 .73101 .27413 .75059 .27517 .74179 .2605 Metetra
.455 .606 .911 r
.74179 .2605 .75059 .27517 .77041 .27632 .76167 .26166 Metetra
.648 .717 .898 r
.10333 .3196 .11389 .33057 .12832 .32587 .11782 .31471 Metetra
.645 .713 .897 r
.11782 .31471 .12832 .32587 .14282 .32126 .13238 .30989 Metetra
.642 .709 .896 r
.13238 .30989 .14282 .32126 .15741 .31673 .14702 .30515 Metetra
.639 .705 .894 r
.14702 .30515 .15741 .31673 .17207 .31231 .16174 .3005 Metetra
.635 .701 .893 r
.16174 .3005 .17207 .31231 .18681 .30799 .17654 .29596 Metetra
.631 .696 .892 r
.17654 .29596 .18681 .30799 .20163 .3038 .19142 .29153 Metetra
.627 .692 .891 r
.19142 .29153 .20163 .3038 .21653 .29973 .20637 .28723 Metetra
.622 .687 .889 r
.20637 .28723 .21653 .29973 .23153 .29581 .22142 .28308 Metetra
.618 .682 .888 r
.22142 .28308 .23153 .29581 .2466 .29203 .23655 .27907 Metetra
.613 .677 .887 r
.23655 .27907 .2466 .29203 .26177 .28842 .25176 .27523 Metetra
.607 .672 .886 r
.25176 .27523 .26177 .28842 .27704 .28497 .26708 .27156 Metetra
.602 .667 .885 r
.26708 .27156 .27704 .28497 .2924 .2817 .28248 .26808 Metetra
.596 .662 .884 r
.28248 .26808 .2924 .2817 .30786 .27861 .29799 .26478 Metetra
.59 .657 .884 r
.29799 .26478 .30786 .27861 .32343 .2757 .31359 .26169 Metetra
.584 .652 .883 r
.31359 .26169 .32343 .2757 .33911 .27298 .32931 .25879 Metetra
.578 .648 .883 r
.32931 .25879 .33911 .27298 .3549 .27046 .34514 .2561 Metetra
.572 .643 .883 r
.34514 .2561 .3549 .27046 .37081 .26813 .36108 .25363 Metetra
.565 .639 .883 r
.36108 .25363 .37081 .26813 .38684 .266 .37715 .25137 Metetra
.559 .635 .883 r
.37715 .25137 .38684 .266 .40301 .26407 .39334 .24932 Metetra
.553 .631 .883 r
.39334 .24932 .40301 .26407 .4193 .26234 .40967 .24748 Metetra
.546 .628 .884 r
.40967 .24748 .4193 .26234 .43574 .2608 .42614 .24586 Metetra
.54 .625 .885 r
.42614 .24586 .43574 .2608 .45232 .25945 .44275 .24445 Metetra
.534 .622 .886 r
.44275 .24445 .45232 .25945 .46905 .2583 .45951 .24324 Metetra
.527 .619 .887 r
.45951 .24324 .46905 .2583 .48593 .25734 .47642 .24224 Metetra
.521 .617 .888 r
.47642 .24224 .48593 .25734 .50298 .25656 .4935 .24143 Metetra
.515 .614 .889 r
.4935 .24143 .50298 .25656 .5202 .25596 .51075 .24082 Metetra
.509 .612 .891 r
.51075 .24082 .5202 .25596 .53758 .25553 .52817 .24039 Metetra
.503 .611 .892 r
.52817 .24039 .53758 .25553 .55515 .25527 .54577 .24013 Metetra
.498 .609 .894 r
.54577 .24013 .55515 .25527 .57289 .25517 .56355 .24005 Metetra
.492 .608 .896 r
.56355 .24005 .57289 .25517 .59083 .25522 .58153 .24013 Metetra
.487 .607 .897 r
.58153 .24013 .59083 .25522 .60896 .25543 .5997 .24036 Metetra
.481 .606 .899 r
.5997 .24036 .60896 .25543 .62729 .25578 .61807 .24074 Metetra
.476 .605 .901 r
.61807 .24074 .62729 .25578 .64583 .25626 .63666 .24126 Metetra
.472 .605 .903 r
.63666 .24126 .64583 .25626 .66458 .25687 .65545 .24192 Metetra
.467 .605 .905 r
.65545 .24192 .66458 .25687 .68354 .25761 .67447 .24269 Metetra
.463 .604 .906 r
.67447 .24269 .68354 .25761 .70273 .25846 .69371 .24359 Metetra
.459 .604 .908 r
.69371 .24359 .70273 .25846 .72214 .25943 .71318 .24459 Metetra
.455 .604 .91 r
.71318 .24459 .72214 .25943 .74179 .2605 .73289 .24569 Metetra
.451 .604 .911 r
.73289 .24569 .74179 .2605 .76167 .26166 .75283 .2469 Metetra
.652 .72 .898 r
.09266 .30866 .10333 .3196 .11782 .31471 .1072 .30356 Metetra
.65 .717 .897 r
.1072 .30356 .11782 .31471 .13238 .30989 .12183 .29852 Metetra
.647 .713 .896 r
.12183 .29852 .13238 .30989 .14702 .30515 .13653 .29355 Metetra
.644 .708 .894 r
.13653 .29355 .14702 .30515 .16174 .3005 .15131 .28866 Metetra
.64 .704 .893 r
.15131 .28866 .16174 .3005 .17654 .29596 .16616 .28387 Metetra
.636 .699 .891 r
.16616 .28387 .17654 .29596 .19142 .29153 .1811 .27919 Metetra
.632 .694 .889 r
.1811 .27919 .19142 .29153 .20637 .28723 .19612 .27463 Metetra
.628 .689 .888 r
.19612 .27463 .20637 .28723 .22142 .28308 .21122 .27021 Metetra
.623 .683 .886 r
.21122 .27021 .22142 .28308 .23655 .27907 .2264 .26594 Metetra
.618 .678 .885 r
.2264 .26594 .23655 .27907 .25176 .27523 .24167 .26184 Metetra
.612 .672 .883 r
.24167 .26184 .25176 .27523 .26708 .27156 .25703 .25792 Metetra
.606 .667 .882 r
.25703 .25792 .26708 .27156 .28248 .26808 .27248 .25419 Metetra
.6 .661 .881 r
.27248 .25419 .28248 .26808 .29799 .26478 .28803 .25066 Metetra
.594 .656 .88 r
.28803 .25066 .29799 .26478 .31359 .26169 .30368 .24735 Metetra
.588 .651 .879 r
.30368 .24735 .31359 .26169 .32931 .25879 .31944 .24425 Metetra
.581 .645 .879 r
.31944 .24425 .32931 .25879 .34514 .2561 .3353 .24138 Metetra
.575 .641 .878 r
.3353 .24138 .34514 .2561 .36108 .25363 .35128 .23873 Metetra
.568 .636 .878 r
.35128 .23873 .36108 .25363 .37715 .25137 .36738 .23632 Metetra
.561 .632 .879 r
.36738 .23632 .37715 .25137 .39334 .24932 .38361 .23415 Metetra
.554 .627 .879 r
.38361 .23415 .39334 .24932 .40967 .24748 .39996 .23221 Metetra
.547 .624 .88 r
.39996 .23221 .40967 .24748 .42614 .24586 .41646 .2305 Metetra
.541 .62 .88 r
.41646 .2305 .42614 .24586 .44275 .24445 .4331 .22903 Metetra
.534 .617 .881 r
.4331 .22903 .44275 .24445 .45951 .24324 .44988 .22778 Metetra
.527 .614 .883 r
.44988 .22778 .45951 .24324 .47642 .24224 .46683 .22675 Metetra
.52 .612 .884 r
.46683 .22675 .47642 .24224 .4935 .24143 .48394 .22593 Metetra
.514 .61 .886 r
.48394 .22593 .4935 .24143 .51075 .24082 .50121 .22532 Metetra
.507 .608 .888 r
.50121 .22532 .51075 .24082 .52817 .24039 .51866 .22491 Metetra
.501 .606 .89 r
.51866 .22491 .52817 .24039 .54577 .24013 .5363 .22468 Metetra
.495 .605 .892 r
.5363 .22468 .54577 .24013 .56355 .24005 .55412 .22464 Metetra
.489 .604 .894 r
.55412 .22464 .56355 .24005 .58153 .24013 .57213 .22477 Metetra
.483 .603 .896 r
.57213 .22477 .58153 .24013 .5997 .24036 .59034 .22506 Metetra
.478 .603 .898 r
.59034 .22506 .5997 .24036 .61807 .24074 .60876 .2255 Metetra
.473 .602 .9 r
.60876 .2255 .61807 .24074 .63666 .24126 .62739 .22608 Metetra
.468 .602 .902 r
.62739 .22608 .63666 .24126 .65545 .24192 .64623 .22679 Metetra
.463 .602 .904 r
.64623 .22679 .65545 .24192 .67447 .24269 .6653 .22763 Metetra
.458 .602 .906 r
.6653 .22763 .67447 .24269 .69371 .24359 .6846 .22858 Metetra
.454 .602 .908 r
.6846 .22858 .69371 .24359 .71318 .24459 .70413 .22964 Metetra
.45 .603 .91 r
.70413 .22964 .71318 .24459 .73289 .24569 .7239 .2308 Metetra
.446 .603 .912 r
.7239 .2308 .73289 .24569 .75283 .2469 .74391 .23205 Metetra
.656 .725 .9 r
.08185 .29777 .09266 .30866 .1072 .30356 .09646 .29247 Metetra
.654 .721 .898 r
.09646 .29247 .1072 .30356 .12183 .29852 .11114 .28721 Metetra
.652 .717 .896 r
.11114 .28721 .12183 .29852 .13653 .29355 .12591 .282 Metetra
.649 .713 .895 r
.12591 .282 .13653 .29355 .15131 .28866 .14075 .27685 Metetra
.646 .708 .893 r
.14075 .27685 .15131 .28866 .16616 .28387 .15567 .27179 Metetra
.642 .703 .891 r
.15567 .27179 .16616 .28387 .1811 .27919 .17068 .26682 Metetra
.638 .698 .889 r
.17068 .26682 .1811 .27919 .19612 .27463 .18576 .26196 Metetra
.634 .692 .887 r
.18576 .26196 .19612 .27463 .21122 .27021 .20092 .25724 Metetra
.629 .686 .885 r
.20092 .25724 .21122 .27021 .2264 .26594 .21616 .25267 Metetra
.624 .68 .883 r
.21616 .25267 .2264 .26594 .24167 .26184 .23149 .24827 Metetra
.618 .674 .881 r
.23149 .24827 .24167 .26184 .25703 .25792 .24691 .24404 Metetra
.612 .667 .879 r
.24691 .24404 .25703 .25792 .27248 .25419 .26241 .24002 Metetra
.606 .661 .877 r
.26241 .24002 .27248 .25419 .28803 .25066 .27801 .23622 Metetra
.599 .655 .876 r
.27801 .23622 .28803 .25066 .30368 .24735 .2937 .23264 Metetra
.592 .648 .875 r
.2937 .23264 .30368 .24735 .31944 .24425 .3095 .2293 Metetra
.585 .643 .874 r
.3095 .2293 .31944 .24425 .3353 .24138 .3254 .2262 Metetra
.578 .637 .873 r
.3254 .2262 .3353 .24138 .35128 .23873 .34142 .22337 Metetra
.571 .632 .873 r
.34142 .22337 .35128 .23873 .36738 .23632 .35755 .22079 Metetra
.563 .627 .873 r
.35755 .22079 .36738 .23632 .38361 .23415 .3738 .21847 Metetra
.556 .622 .873 r
.3738 .21847 .38361 .23415 .39996 .23221 .39019 .21642 Metetra
.548 .618 .874 r
.39019 .21642 .39996 .23221 .41646 .2305 .40671 .21463 Metetra
.541 .614 .875 r
.40671 .21463 .41646 .2305 .4331 .22903 .42337 .2131 Metetra
.534 .611 .876 r
.42337 .2131 .4331 .22903 .44988 .22778 .44019 .21182 Metetra
.526 .608 .878 r
.44019 .21182 .44988 .22778 .46683 .22675 .45716 .21079 Metetra
.519 .606 .88 r
.45716 .21079 .46683 .22675 .48394 .22593 .47429 .20999 Metetra
.512 .604 .882 r
.47429 .20999 .48394 .22593 .50121 .22532 .49159 .20941 Metetra
.505 .602 .884 r
.49159 .20941 .50121 .22532 .51866 .22491 .50907 .20905 Metetra
.498 .601 .886 r
.50907 .20905 .51866 .22491 .5363 .22468 .52674 .2089 Metetra
.491 .6 .889 r
.52674 .2089 .5363 .22468 .55412 .22464 .54459 .20893 Metetra
.485 .599 .892 r
.54459 .20893 .55412 .22464 .57213 .22477 .56264 .20915 Metetra
.479 .599 .894 r
.56264 .20915 .57213 .22477 .59034 .22506 .58089 .20953 Metetra
.473 .599 .897 r
.58089 .20953 .59034 .22506 .60876 .2255 .59935 .21006 Metetra
.467 .599 .9 r
.59935 .21006 .60876 .2255 .62739 .22608 .61803 .21074 Metetra
.462 .599 .902 r
.61803 .21074 .62739 .22608 .64623 .22679 .63692 .21154 Metetra
.457 .599 .905 r
.63692 .21154 .64623 .22679 .6653 .22763 .65604 .21247 Metetra
.453 .6 .907 r
.65604 .21247 .6653 .22763 .6846 .22858 .67539 .21351 Metetra
.448 .601 .91 r
.67539 .21351 .6846 .22858 .70413 .22964 .69498 .21464 Metetra
.444 .601 .912 r
.69498 .21464 .70413 .22964 .7239 .2308 .71481 .21587 Metetra
.44 .602 .914 r
.71481 .21587 .7239 .2308 .74391 .23205 .73489 .21718 Metetra
.66 .73 .901 r
.0709 .28698 .08185 .29777 .09646 .29247 .08557 .28149 Metetra
.658 .727 .9 r
.08557 .28149 .09646 .29247 .11114 .28721 .10032 .27602 Metetra
.657 .723 .898 r
.10032 .27602 .11114 .28721 .12591 .282 .11515 .27056 Metetra
.654 .719 .896 r
.11515 .27056 .12591 .282 .14075 .27685 .13006 .26515 Metetra
.652 .715 .894 r
.13006 .26515 .14075 .27685 .15567 .27179 .14506 .25979 Metetra
.649 .709 .892 r
.14506 .25979 .15567 .27179 .17068 .26682 .16013 .25451 Metetra
.645 .703 .889 r
.16013 .25451 .17068 .26682 .18576 .26196 .17528 .24932 Metetra
.641 .697 .886 r
.17528 .24932 .18576 .26196 .20092 .25724 .19051 .24425 Metetra
.636 .69 .883 r
.19051 .24425 .20092 .25724 .21616 .25267 .20582 .23931 Metetra
.631 .683 .881 r
.20582 .23931 .21616 .25267 .23149 .24827 .22122 .23453 Metetra
.626 .676 .878 r
.22122 .23453 .23149 .24827 .24691 .24404 .2367 .22994 Metetra
.619 .668 .875 r
.2367 .22994 .24691 .24404 .26241 .24002 .25226 .22556 Metetra
.613 .661 .873 r
.25226 .22556 .26241 .24002 .27801 .23622 .26792 .2214 Metetra
.606 .653 .871 r
.26792 .2214 .27801 .23622 .2937 .23264 .28366 .21749 Metetra
.598 .646 .869 r
.28366 .21749 .2937 .23264 .3095 .2293 .2995 .21384 Metetra
.591 .639 .867 r
.2995 .21384 .3095 .2293 .3254 .2262 .31545 .21047 Metetra
.583 .632 .866 r
.31545 .21047 .3254 .2262 .34142 .22337 .3315 .2074 Metetra
.575 .626 .865 r
.3315 .2074 .34142 .22337 .35755 .22079 .34766 .20462 Metetra
.566 .62 .865 r
.34766 .20462 .35755 .22079 .3738 .21847 .36395 .20214 Metetra
.558 .615 .865 r
.36395 .20214 .3738 .21847 .39019 .21642 .38036 .19997 Metetra
.55 .61 .866 r
.38036 .19997 .39019 .21642 .40671 .21463 .3969 .19811 Metetra
.541 .606 .867 r
.3969 .19811 .40671 .21463 .42337 .2131 .41358 .19654 Metetra
.533 .603 .869 r
.41358 .19654 .42337 .2131 .44019 .21182 .43042 .19526 Metetra
.525 .6 .871 r
.43042 .19526 .44019 .21182 .45716 .21079 .44741 .19425 Metetra
.516 .598 .874 r
.44741 .19425 .45716 .21079 .47429 .20999 .46456 .19352 Metetra
.508 .596 .877 r
.46456 .19352 .47429 .20999 .49159 .20941 .48189 .19304 Metetra
.501 .595 .88 r
.48189 .19304 .49159 .20941 .50907 .20905 .49939 .19279 Metetra
.493 .594 .883 r
.49939 .19279 .50907 .20905 .52674 .2089 .51709 .19276 Metetra
.486 .594 .886 r
.51709 .19276 .52674 .2089 .54459 .20893 .53497 .19293 Metetra
.479 .594 .89 r
.53497 .19293 .54459 .20893 .56264 .20915 .55306 .19329 Metetra
.472 .594 .893 r
.55306 .19329 .56264 .20915 .58089 .20953 .57135 .19382 Metetra
.466 .595 .897 r
.57135 .19382 .58089 .20953 .59935 .21006 .58985 .1945 Metetra
.46 .595 .9 r
.58985 .1945 .59935 .21006 .61803 .21074 .60857 .19532 Metetra
.455 .596 .903 r
.60857 .19532 .61803 .21074 .63692 .21154 .62752 .19626 Metetra
.45 .597 .906 r
.62752 .19626 .63692 .21154 .65604 .21247 .64669 .19731 Metetra
.445 .599 .909 r
.64669 .19731 .65604 .21247 .67539 .21351 .6661 .19845 Metetra
.441 .6 .912 r
.6661 .19845 .67539 .21351 .69498 .21464 .68575 .19969 Metetra
.437 .601 .914 r
.68575 .19969 .69498 .21464 .71481 .21587 .70564 .201 Metetra
.433 .602 .917 r
.70564 .201 .71481 .21587 .73489 .21718 .72579 .20238 Metetra
.663 .736 .904 r
.05979 .27634 .0709 .28698 .08557 .28149 .07452 .27069 Metetra
.662 .733 .903 r
.07452 .27069 .08557 .28149 .10032 .27602 .08934 .26502 Metetra
.661 .731 .901 r
.08934 .26502 .10032 .27602 .11515 .27056 .10424 .25936 Metetra
.66 .727 .899 r
.10424 .25936 .11515 .27056 .13006 .26515 .11922 .25369 Metetra
.658 .723 .897 r
.11922 .25369 .13006 .26515 .14506 .25979 .13428 .24804 Metetra
.656 .718 .894 r
.13428 .24804 .14506 .25979 .16013 .25451 .14943 .24243 Metetra
.653 .713 .891 r
.14943 .24243 .16013 .25451 .17528 .24932 .16466 .23687 Metetra
.65 .706 .888 r
.16466 .23687 .17528 .24932 .19051 .24425 .17998 .23139 Metetra
.646 .698 .884 r
.17998 .23139 .19051 .24425 .20582 .23931 .19537 .22601 Metetra
.641 .69 .88 r
.19537 .22601 .20582 .23931 .22122 .23453 .21084 .22077 Metetra
.636 .681 .876 r
.21084 .22077 .22122 .23453 .2367 .22994 .2264 .21569 Metetra
.629 .672 .872 r
.2264 .21569 .2367 .22994 .25226 .22556 .24204 .21082 Metetra
.622 .662 .868 r
.24204 .21082 .25226 .22556 .26792 .2214 .25776 .20619 Metetra
.615 .653 .864 r
.25776 .20619 .26792 .2214 .28366 .21749 .27356 .20182 Metetra
.607 .643 .861 r
.27356 .20182 .28366 .21749 .2995 .21384 .28946 .19774 Metetra
.598 .634 .858 r
.28946 .19774 .2995 .21384 .31545 .21047 .30545 .19399 Metetra
.589 .626 .856 r
.30545 .19399 .31545 .21047 .3315 .2074 .32154 .19059 Metetra
.579 .618 .854 r
.32154 .19059 .3315 .2074 .34766 .20462 .33774 .18754 Metetra
.57 .61 .854 r
.33774 .18754 .34766 .20462 .36395 .20214 .35405 .18487 Metetra
.56 .604 .854 r
.35405 .18487 .36395 .20214 .38036 .19997 .37048 .18257 Metetra
.55 .598 .855 r
.37048 .18257 .38036 .19997 .3969 .19811 .38704 .18064 Metetra
.54 .594 .856 r
.38704 .18064 .3969 .19811 .41358 .19654 .40374 .17907 Metetra
.531 .59 .859 r
.40374 .17907 .41358 .19654 .43042 .19526 .42059 .17786 Metetra
.521 .588 .862 r
.42059 .17786 .43042 .19526 .44741 .19425 .43759 .17698 Metetra
.512 .586 .866 r
.43759 .17698 .44741 .19425 .46456 .19352 .45476 .17641 Metetra
.502 .585 .87 r
.45476 .17641 .46456 .19352 .48189 .19304 .47211 .17612 Metetra
.494 .585 .875 r
.47211 .17612 .48189 .19304 .49939 .19279 .48963 .1761 Metetra
.485 .585 .879 r
.48963 .1761 .49939 .19279 .51709 .19276 .50735 .1763 Metetra
.477 .586 .884 r
.50735 .1763 .51709 .19276 .53497 .19293 .52526 .17672 Metetra
.469 .587 .889 r
.52526 .17672 .53497 .19293 .55306 .19329 .54338 .17732 Metetra
.462 .589 .894 r
.54338 .17732 .55306 .19329 .57135 .19382 .56171 .17808 Metetra
.455 .591 .899 r
.56171 .17808 .57135 .19382 .58985 .1945 .58026 .17898 Metetra
.449 .593 .903 r
.58026 .17898 .58985 .1945 .60857 .19532 .59902 .18 Metetra
.444 .595 .907 r
.59902 .18 .60857 .19532 .62752 .19626 .61802 .18111 Metetra
.439 .597 .911 r
.61802 .18111 .62752 .19626 .64669 .19731 .63725 .18232 Metetra
.434 .599 .914 r
.63725 .18232 .64669 .19731 .6661 .19845 .65672 .18359 Metetra
.431 .601 .917 r
.65672 .18359 .6661 .19845 .68575 .19969 .67643 .18493 Metetra
.427 .602 .919 r
.67643 .18493 .68575 .19969 .70564 .201 .69639 .18632 Metetra
.424 .604 .921 r
.69639 .18632 .70564 .201 .72579 .20238 .7166 .18776 Metetra
.664 .74 .907 r
.04854 .26577 .05979 .27634 .07452 .27069 .06332 .26002 Metetra
.664 .739 .906 r
.06332 .26002 .07452 .27069 .08934 .26502 .07819 .25425 Metetra
.664 .738 .905 r
.07819 .25425 .08934 .26502 .10424 .25936 .09315 .24844 Metetra
.663 .736 .904 r
.09315 .24844 .10424 .25936 .11922 .25369 .10819 .2426 Metetra
.663 .734 .903 r
.10819 .2426 .11922 .25369 .13428 .24804 .12333 .23674 Metetra
.662 .731 .901 r
.12333 .23674 .13428 .24804 .14943 .24243 .13855 .23085 Metetra
.661 .727 .898 r
.13855 .23085 .14943 .24243 .16466 .23687 .15386 .22496 Metetra
.66 .721 .894 r
.15386 .22496 .16466 .23687 .17998 .23139 .16926 .21906 Metetra
.657 .714 .89 r
.16926 .21906 .17998 .23139 .19537 .22601 .18474 .2132 Metetra
.654 .705 .885 r
.18474 .2132 .19537 .22601 .21084 .22077 .20031 .20739 Metetra
.65 .695 .879 r
.20031 .20739 .21084 .22077 .2264 .21569 .21597 .20167 Metetra
.645 .684 .872 r
.21597 .20167 .2264 .21569 .24204 .21082 .23171 .19609 Metetra
.638 .671 .865 r
.23171 .19609 .24204 .21082 .25776 .20619 .24752 .19071 Metetra
.63 .657 .858 r
.24752 .19071 .25776 .20619 .27356 .20182 .26342 .18559 Metetra
.621 .643 .851 r
.26342 .18559 .27356 .20182 .28946 .19774 .2794 .18079 Metetra
.61 .629 .845 r
.2794 .18079 .28946 .19774 .30545 .19399 .29545 .17637 Metetra
.599 .616 .84 r
.29545 .17637 .30545 .19399 .32154 .19059 .3116 .17239 Metetra
.587 .604 .836 r
.3116 .17239 .32154 .19059 .33774 .18754 .32784 .1689 Metetra
.575 .593 .834 r
.32784 .1689 .33774 .18754 .35405 .18487 .34417 .16592 Metetra
.562 .584 .833 r
.34417 .16592 .35405 .18487 .37048 .18257 .36062 .16347 Metetra
.549 .577 .835 r
.36062 .16347 .37048 .18257 .38704 .18064 .37718 .16155 Metetra
.537 .571 .837 r
.37718 .16155 .38704 .18064 .40374 .17907 .39388 .16013 Metetra
.524 .568 .842 r
.39388 .16013 .40374 .17907 .42059 .17786 .41073 .15918 Metetra
.512 .567 .848 r
.41073 .15918 .42059 .17786 .43759 .17698 .42773 .15866 Metetra
.501 .567 .854 r
.42773 .15866 .43759 .17698 .45476 .17641 .4449 .15852 Metetra
.489 .568 .862 r
.4449 .15852 .45476 .17641 .47211 .17612 .46225 .1587 Metetra
.479 .571 .87 r
.46225 .1587 .47211 .17612 .48963 .1761 .47978 .15915 Metetra
.469 .575 .879 r
.47978 .15915 .48963 .1761 .50735 .1763 .49752 .15982 Metetra
.46 .579 .887 r
.49752 .15982 .50735 .1763 .52526 .17672 .51546 .16067 Metetra
.452 .583 .894 r
.51546 .16067 .52526 .17672 .54338 .17732 .53361 .16166 Metetra
.444 .588 .901 r
.53361 .16166 .54338 .17732 .56171 .17808 .55198 .16276 Metetra
.438 .592 .907 r
.55198 .16276 .56171 .17808 .58026 .17898 .57057 .16393 Metetra
.433 .595 .912 r
.57057 .16393 .58026 .17898 .59902 .18 .58939 .16518 Metetra
.428 .599 .916 r
.58939 .16518 .59902 .18 .61802 .18111 .60844 .16647 Metetra
.424 .601 .92 r
.60844 .16647 .61802 .18111 .63725 .18232 .62773 .1678 Metetra
.422 .604 .922 r
.62773 .1678 .63725 .18232 .65672 .18359 .64726 .16916 Metetra
.419 .606 .925 r
.64726 .16916 .65672 .18359 .67643 .18493 .66703 .17055 Metetra
.417 .607 .926 r
.66703 .17055 .67643 .18493 .69639 .18632 .68706 .17196 Metetra
.416 .608 .928 r
.68706 .17196 .69639 .18632 .7166 .18776 .70734 .17339 Metetra
.664 .741 .907 r
.03716 .25514 .04854 .26577 .06332 .26002 .05199 .24933 Metetra
.664 .741 .907 r
.05199 .24933 .06332 .26002 .07819 .25425 .06691 .24349 Metetra
.664 .741 .907 r
.06691 .24349 .07819 .25425 .09315 .24844 .08191 .23761 Metetra
.664 .741 .907 r
.08191 .23761 .09315 .24844 .10819 .2426 .09701 .2317 Metetra
.664 .74 .907 r
.09701 .2317 .10819 .2426 .12333 .23674 .11219 .22576 Metetra
.664 .74 .907 r
.11219 .22576 .12333 .23674 .13855 .23085 .12747 .21978 Metetra
.664 .74 .906 r
.12747 .21978 .13855 .23085 .15386 .22496 .14284 .21376 Metetra
.664 .738 .906 r
.14284 .21376 .15386 .22496 .16926 .21906 .1583 .2077 Metetra
.663 .737 .904 r
.1583 .2077 .16926 .21906 .18474 .2132 .17385 .20161 Metetra
.663 .733 .902 r
.17385 .20161 .18474 .2132 .20031 .20739 .1895 .19548 Metetra
.663 .729 .898 r
.1895 .19548 .20031 .20739 .21597 .20167 .20525 .18932 Metetra
.662 .721 .893 r
.20525 .18932 .21597 .20167 .23171 .19609 .22109 .18312 Metetra
.66 .711 .885 r
.22109 .18312 .23171 .19609 .24752 .19071 .23703 .17687 Metetra
.658 .696 .874 r
.23703 .17687 .24752 .19071 .26342 .18559 .25307 .17059 Metetra
.655 .678 .86 r
.25307 .17059 .26342 .18559 .2794 .18079 .2692 .16428 Metetra
.65 .655 .841 r
.2692 .16428 .2794 .18079 .29545 .17637 .28544 .15792 Metetra
.643 .627 .819 r
.28544 .15792 .29545 .17637 .3116 .17239 .30177 .15155 Metetra
.631 .596 .796 r
.30177 .15155 .3116 .17239 .32784 .1689 .31819 .14538 Metetra
.605 .558 .775 r
.31819 .14538 .32784 .1689 .34417 .16592 .33465 .14031 Metetra
.561 .517 .764 r
.33465 .14031 .34417 .16592 .36062 .16347 .35107 .13763 Metetra
.516 .49 .766 r
.35107 .13763 .36062 .16347 .37718 .16155 .36752 .13733 Metetra
.49 .49 .784 r
.36752 .13733 .37718 .16155 .39388 .16013 .38409 .13816 Metetra
.478 .507 .81 r
.38409 .13816 .39388 .16013 .41073 .15918 .40084 .13924 Metetra
.469 .528 .836 r
.40084 .13924 .41073 .15918 .42773 .15866 .41778 .14037 Metetra
.46 .547 .858 r
.41778 .14037 .42773 .15866 .4449 .15852 .43492 .14152 Metetra
.451 .563 .878 r
.43492 .14152 .4449 .15852 .46225 .1587 .45226 .14267 Metetra
.442 .577 .893 r
.45226 .14267 .46225 .1587 .47978 .15915 .46982 .14384 Metetra
.435 .587 .904 r
.46982 .14384 .47978 .15915 .49752 .15982 .48758 .14503 Metetra
.428 .594 .913 r
.48758 .14503 .49752 .15982 .51546 .16067 .50555 .14623 Metetra
.423 .6 .919 r
.50555 .14623 .51546 .16067 .53361 .16166 .52374 .14744 Metetra
.42 .604 .923 r
.52374 .14744 .53361 .16166 .55198 .16276 .54216 .14867 Metetra
.417 .607 .926 r
.54216 .14867 .55198 .16276 .57057 .16393 .5608 .14991 Metetra
.415 .608 .928 r
.5608 .14991 .57057 .16393 .58939 .16518 .57967 .15117 Metetra
.414 .61 .929 r
.57967 .15117 .58939 .16518 .60844 .16647 .59877 .15244 Metetra
.413 .61 .93 r
.59877 .15244 .60844 .16647 .62773 .1678 .61812 .15373 Metetra
.413 .611 .931 r
.61812 .15373 .62773 .1678 .64726 .16916 .6377 .15504 Metetra
.412 .611 .931 r
.6377 .15504 .64726 .16916 .66703 .17055 .65754 .15636 Metetra
.412 .611 .931 r
.65754 .15636 .66703 .17055 .68706 .17196 .67762 .1577 Metetra
.412 .611 .931 r
.67762 .1577 .68706 .17196 .70734 .17339 .69797 .15906 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{188, 152.438},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003[0000_Q000`40O003h00Ojgoo003[Ool00>]o
o`00jgoo003[Ool00>]oo`00UGoo00<007ooOol017oo0`00C7oo002EOol00`00Oomoo`03Ool01@00
Oomoogoo0000Bgoo002FOol00`00Oomoo`02Ool01@00Oomoogoo0000Bgoo002FOol00`00Oomoo`02
Ool01@00Oomoogoo0000Bgoo002FOol00`00Oomoo`02Ool01@00Oomoogoo0000Bgoo002GOol01@00
Oomoogoo00000goo00<007ooOol0BGoo002COol01@00Oomoogoo00000goo00D007ooOomoo`0004]o
o`00Tgoo1@0017oo0`00C7oo003[Ool00>]oo`00jgoo002OOol2001:Ool009aoo`<00003Ool007oo
04Uoo`00VWoo0P0017oo0P00BGoo002GOol30006Ool00`00Ool00018Ool006ioo`<000Eoo`@001]o
o`80009oo`03001oogoo00Aoo`03001oo`0004Qoo`00KGoo00D007ooOomoo`0000Qoo`03001oogoo
01Eoo`<000]oo`04001oogoo0017Ool006eoo`05001oogooOol00008Ool00`00Oomoo`0BOol3000>
Ool00`00Oomoo`020016Ool006eoo`05001oogooOol00008Ool00`00Oomoo`0@Ool20002Ool00`00
Oomoo`0<Ool01@00Oomoogoo0000AWoo001]Ool40005Ool4000@Ool3000COol00`00Oomoo`02Ool0
0`00Oomoo`13Ool006eoo`03001oogoo00Ioo`03001oogoo00moo`8001Moo`03001oogoo009oo`03
001oogoo049oo`00KWoo00<007ooOol01Goo00<007ooOol037oo0`000Woo00<007ooOol057oo00<0
07ooOol00Woo00<007ooOol0@Woo001_Ool30004Ool40009Ool2000LOol00`00Oomoo`02Ool20013
Ool0081oo`<001ioo`03001oogoo00=oo`03001oogoo045oo`00OWoo0P000goo00<007ooOol06goo
00<007ooOol017oo00<007ooOol0@7oo001kOol3000SOol00`00Oomoo`05Ool00`00Oomoo`0oOol0
07Uoo`<002Eoo`03001oogoo00Eoo`03001oogoo03moo`00MWoo0`000goo00<007ooOol08Woo00<0
07ooOol01Woo00<007ooOol01Goo0`001Woo0P0017oo1@009Goo001:Ool30005Ool3000NOol3000[
Ool00`00Oomoo`06Ool20005Ool01@00Oomoogoo00001Goo0P001Goo00@007ooOol002Eoo`00BGoo
00D007ooOomoo`0000=oo`05001oogooOol0000KOol200000goo001oo`0[Ool00`00Oomoo`07Ool0
0`00Oomoo`03Ool01@00Oomoogoo00003Goo00<007ooOol09Goo0019Ool01@00Oomoogoo00000goo
00D007ooOomoo`0001Qoo`<0031oo`03001oogoo00Qoo`03001oogoo009oo`05001oogooOol0000>
Ool00`00Oomoo`0TOol004Uoo`05001oogooOol00003Ool01@00Oomoogoo00005Woo0P00<goo00<0
07ooOol02Goo00D007ooOomoo`0000=oo`03001oogoo00eoo`03001oogoo02=oo`00BGoo100017oo
00D007ooOomoo`0001=oo`<00003Ool007oo039oo`03001oogoo00Uoo`80009oo`05001oogooOol0
000?Ool00`00Oomoo`0SOol004Uoo`03001oogoo00Eoo`05001oogooOol0000AOol2000hOol00`00
Oomoo`08Ool00`00Ool00002Ool01@00Oomoogoo00002goo00D007ooOomoo`0002Eoo`00BWoo00<0
07ooOol017oo00D007ooOomoo`0000ioo`<003Yoo`03001oogoo00]oo`03001oogoo00<000eoo`<0
02Ioo`00Bgoo0`0017oo0`003Goo0P000Woo00<007ooOol0>7oo00<007ooOol02goo00<007ooOol0
>Goo001OOol3000oOol00`00Oomoo`0<Ool00`00Oomoo`0hOol005aoo`D0045oo`03001oogoo00ao
o`03001oogoo03Moo`00FWoo0P00AWoo00<007ooOol02goo0P00>Goo001GOol30018Ool00`00Oomo
o`0=Ool00`00Oomoo`0fOol005EooaH003Moo`03001oogoo00ioo`03001oogoo03Eoo`00:7oo1000
17oo10007Woo100000E>>000AQQ660000091n005000mnCgi?OT00002?QT01@00?QXn6ShJ00000SXk
3P00:Goo00<007ooOol03Woo00<007ooOol0=Goo000/Ool00`00Oomoo`05Ool00`00Oomoo`0IOol4
0003CSP01@00AQQ664HH00000T7h00D003gi?OTmn@00008n6@07000n6ShJ?QX003Xk00000cYL00D0
03YL>U`jG000008fG003000fOCIm00l001Yoo`03001oogoo00ioo`8003Ioo`00;7oo00<007ooOol0
1Goo00<007ooOol05Woo1`000dhh00D004HHAQQ660000091n005000mnCgi?OT00002?QT01@00?QXn
6ShJ00000SXk00D003YL>U`jG00000<jG004000fG3IL0002=Wd01@00=WdfOCIm00000cIm00@003Im
=Wd0008fO@`000ioo`03001oogoo011oo`03001oogoo03=oo`00;7oo00<007ooOol01Goo00<007oo
Ool057oo0`000U:J00D0059iDWUBN@0000=>>004001664HH0002@OP01@00?OTmnCgi00000ShI4`00
0cIm00D003Im=WdfO@0000<fO@04000fOCIm0002=Wd01@00=WdfOCIm00000cIm00<003Im=Wd03`00
4Woo00<007ooOol0<goo000XOol40004Ool4000DOol50004DYX01000DWUBN@000dhh00<004HH0000
2P0000=2FP00@U/00T9K00@0049K@U/000<nF`05000nFciK?U/00002?U`C0003=Wd01@00=WdfOCIm
00000cIm00@003Im=Wd000@fO@05000fOCIm=Wd0000COol00`00Oomoo`0bOol002Qoo`03001oogoo
00Eoo`03001oogoo01=oo`800003D[/005:J009BVP05001BVU:JDYX00002DWT60002AUX01000AUY6
FP000dIJ00D0049J@UY2FP0000=2F`040012Fd9K0003?U/01`00?U/nFciK000nG00000<jG005000j
G3YL>U`00003>U`01000>UdjG@000SIm4@0013Im00<003Im=Wd00P004Woo00<007ooOol01Woo0`00
1Woo0P001Woo0`0067oo000XOol00`00Oomoo`05Ool00`00Oomoo`0@Ool40003D[/01000DYYBVP00
0U:J100000E:NP00BUY:FP0000A6FP040016FTIJ0003AUX01000@UY2FP000d9K00@0049K@U/000<n
F`05000nFciK?U/00002?U`01@00>U`jG3YL00000cYL00D003YL>U`jG000008jG@04000fOCIm0003
=Wd01@00=WdfOCIm00000cIm00<003Im=Wd02`004Goo0P001Woo00D007ooOomoo`0000Eoo`8000Mo
o`03001oogoo01Moo`00:7oo100017oo100037oo100000=Ff`0000000e:k00<005:JDYX00`000Tij
00D004YjBWY:NP00009:FP050016FTIJAUX00002AUX01@00AUY6FTIJ00000T9J8P000cIm00D003Im
=WdfO@0000<fO@05000fOCIm=Wd00002=Wd00`00=WdfO@02=Wd01@00=Wd003Im00004goo00<007oo
Ool00goo00D007ooOomoo`0000]oo`D001Qoo`00?Woo0`0000=Fg000E]/00eKK00<005:kD[/00`00
00=>V`00CWX00dij00@004YjBWX000=:FP030016FTIJ00X00003@U/0049K0092F`040012Fd9K0003
@U/01@00?U/nFciK00000SiL00D003iL?U`nG00000<nG005000jG3YL>U`00002>U`D0004=Wd20002
=Wd00`00Oomoo`0@Ool00`00Oomoo`03Ool01@00Oomoogoo00002goo00@007ooOol001Uoo`00>goo
1@000eKL00<005KKE]/0100000=>V`00CY/00TjK00D004ijCWY>NP00009:NPL00005AW/004IkAW/0
0003AW/01@00@U]2Fd9K00000d9K00@0049K@U/000=2F`05000nFciK?U/00002?U`01@00?U`nG3iL
00000ciL00L003YL>U`jG000>U`00003>U`01@00>WdjOCYm00000cYm00D003Im=WdfO@00008fO@/0
019oo`03001oogoo009oo`05001oogooOol0000<Ool00`00Ool0000IOol003Uoo`@000=Fo003001F
g5KL00<0009B^`05001>VdjKCY/00002CY/00`00CWY>NP0400000dYk001:N`02BW/01@00AW]6NdIk
00000TIk00D004IkAW]6N`000092F`050012Fd9K@U/00002@U/00`00@U]2F`0E0002>U`01@00>U`j
G3YL00000cYm00D003Ym>WdjO@0000<fO@04000fOCIm0004=Wd01@00=WdfOCIm00004goo00D007oo
Oomoo`0000=oo`03001oogoo00Yoo`03001oo`0001Uoo`00=Woo10000eKl00<005KlE_`00`000U:k
00D005:kD[]B^`00009>V`H0009:V`05001:NdYkBW/00003BW/01000AW]6N`000TIk4@000T9l00D0
049l@Wa2O000008nO005000nO3il?W`00003?W`00`00?W`nO00D0003=Wd01000=WdfOCIm0P000cIm
00<007ooOol03goo0P000goo00D007ooOomoo`0000eoo`8001Uoo`00=7oo0`000UKl0P000UKl0`00
00=B^`00D[/00e:k00<005:kD[/00`0000=>V`00CY/00TjK00D004ZKBY]:V`00009:N`03001:NdYk
00L00003AW/004Ik0096N`040016NdIk0003@W/01000@W]2N`00149l00D0049l@Wa2O000008nO005
000nO3il?W`00003?W`01000?W`nO0000SYl00D003Yl>W`jO00000<jO005000jO3Yl>W`00002>Wd?
000BOol01000Oomoogoo0`003goo00<007ooOol05goo0009Ool40004Ool3000MOol300000eKl001F
o003E_`300000e;L001B^`02D[/00`00D[]B^`0400000djK001>V`02CY/01@00CY]>VdjK00000TZK
1P000TYk00@004IkAW/000=6N`050016NdIkAW/00002AW/01@00@W]2Nd9k00000T9k00D0049l@Wa2
O00000=2O004000nO3il0003?W`01@00?W`nO3il00000cil00@003Yl>W`000<jO005000jO3Yl>W`0
0003>W`01000>WdjO@000cYm00D003Im=WdfO@0000@fO@03001oogoo015oo`03001oogoo02aoo`00
3Goo00@007ooOol000=oo`03001oogoo01Qoo`@000=Fo006001Fo000001Bg0000e;L00<005:kD[/0
0`000Tjk00D004jKCY]>V`00009>V`03001>VdjK00@00005BY/004ZKBY/00004BW/01000AW]6N`00
0TIk00D004IkAW]6N`000096Nc8000<jO@03000fOCIm008000@fO@03001oogoo011oo`03001oogoo
02aoo`003Goo00@007ooOol000=oo`03001oogoo01Eoo`@000=Fo003001Fo5Kl00<000=Bg003001B
g5;L00<00003D[/004jk00=>^`03001>VdjK00H0009:V`05001:VdZKBY/00002BY/01@00BW]:NdYk
00000TIk2P0000=6O000@W`00T9l00@0049l@W`000=2O0050012O49l@W`00002?W`01@00?W`nO3il
00000cil00D003il?W`nO000008nO005000jO3Yl>W`00003>W`01@00>W`jO3Yl00000SYm3`004Goo
0P00;Goo000=Ool01000Oomoo`000goo00<007ooOol04goo10000UKl00<005KlE_`00`000eKL00<0
05;LD]`00`0000=B_000D[/00U:k00<004jkC[/0100000E>V`00CY]>V`0000A:V`04001:VdZK0002
BY/700001DIk0016NdIk00000dIk00D004IlAWa6O00000=2O0040012O49l0003@W`01@00@Wa2O49l
00000Sil00D003il?W`nO00000<nO005000nO3il?W`00002?W`01@00>W`jO3Yl00000cYl00D003Yl
>W`jO000008jO@09000jOCYm>Wd003Ym000jO@0000@jO@03001oogoo015oo`03001oogoo02Yoo`00
2Goo10000goo00D007ooOomoo`00019oo`<0009Fo@80009Fo0<0009Fo003001Fg5KL00@00003D]`0
05:l009B_003001B^e:k00<0009>^`05001>VdjKCY/00002CY/00`00BY]:V`0700000dZK001:V`02
BY/01@00AW]6NdIk00000TIk00D004IkAW]6N`000096O0030012O49l02/000<jO@04000jOCYm>Wd2
0002>Wd01@00>WdjOCYm00004goo00<007ooOol0:Woo0009Ool00`00Oomoo`04Ool01@00Oomoogoo
00003goo0`000eKm00<005KmE_d00`0000=Fo000E_`00UKl0`000U;L00D005;LD]aBg000009B_0<0
009>^`05001>^djkC[/00002CY/00`00CY]>V`030002BY/01000BY]:V`000dZK00D004ZKBY]:V`00
0096N`030016NdIk00X00092O0040012O49l0003@W`01@00@Wa2O49l00000T9l00D003il?W`nO000
00<nO005000nO3il?W`00002?W`01@00?W`nO3il00000cYl00<003Yl>W`0400000<jO@00Ool04Woo
00<007ooOol0:Goo0009Ool00`00Oomoo`04Ool01@00Oomoogoo00003Goo0P0000=Fo@0000000eKm
0`0000=Fo000E_`00UKl00<005Kl00000U;L00D005;LD]aBg000009Bg0<00003D[`004jk00=>^`03
001>^djk00H00003BY/004ZK00=:V`04001:VdZK0002BY/:00000dIl0016O002AW`01000AWa6O000
149l00@0049l@W`000=2O0050012O49l@W`00002@W`01@00?W`nO3il00000cil00D003il?W`nO000
008nO005000nO3il?W`00003>W`01000>W`jO0000cYl00D003Ym>WdjO@0000<jO@03000jOCYm0080
01Aoo`03001oogoo02Qoo`002Goo100017oo0`002goo100015Km00D005Km001Fo00000=Fo003001F
o5Kl008000ABg003001Bg5;L00<00003D[`005:l009B_003001>^djk00@00005C[`004jLCY`00003
BY/00`00BY]:V`0700000dZL0016W002AY`01000AYa6W0000dIl00D004IlAWa6O0000096O0050012
O49l@W`00002@W`S0003>W`01@00>W`jO3Yl00000SYm0P000cYm00@003Ym>Wd001=oo`8002Yoo`00
7Goo10000UKm00<005KmE_d00`000eKl00<005KlE_`00`000U;L00<005;LD]`0100000=Bg000D[`0
0U:l00<005:lD[`00`000Tjl00D004jlC[a>_000009>W003001:VdZK00<0009:W004001:W4ZL0003
BY`01@00AYa6W4JL00000TJL00<004IlAW`03P0000=2O000@W`00T9l00D0049l@Wa2O0000092O005
000nO3il?W`00003?W`01000?W`nO0000cil00D003il?W`nO000008nO1D001Aoo`03001oogoo02Mo
o`006Woo100000=Fo@0000000UKm0`000eKl00<005KlE_`00`000U;L00<005;LD]`00`000U;L00D0
05;LD]aBg000009B_0<00003C[`004jl00=>_003001>_4jl00H00003BY`004ZL00=:W004001:W4ZL
0002BY`00`00AYa6W00700000dJL0016O002AW`01000AWa6O0000d9l00D0049l@Wa2O00000=2O005
0012O49l@W`00002@W`01@00?W`nO3il00000cil00@003il?W`000<nO005000nO3il?W`00002?W`0
1P00>W`jO3Yl000jO0<00004>Wd003Ym0003>Wd00`00>WdjO@02000DOol00`00Oomoo`0VOol001Uo
o`80009Fo@03001FoEKm00<000=Fo003001Fo5Kl00<0009Fo003001Bg5;L00<0009Bg005001Bg5;L
D]`00002D]`300000djl001>_002C[`00`00C[a>_00400001Djl001>W4jL00000dZL00D004ZLBYa:
W000009:W0H00096W0040016W4JL0003AY`01@00AWa6O4Il00000TIl00D0049l@Wa2O0000092O003
0012O49l01`000<nO003000jO000008000<jO005000jO@00>Wd00003>Wd01000>WdjO@0057oo00<0
07ooOol01Goo0`001Woo0P001Goo0`003Woo000@Ool30006Ool20002E_d30003E_`00`00E_aFo003
0002E_`00`00E_aFo00300000e;L001Bg003D]`00`00D]aBg00300000e:l001>_002C[`00`00C[a>
_0030002C[`01@00C[a>_4jl00000TjL00<004ZLBY`01`0000E:W000BYa:W00000A6W0040016W4JL
0002AY`A0002@W`01@00@Wa2O49l00000T9l00D003il?W`nO00000<nO004000nO3il0003?W`00`00
?W`nO00I000COol20005Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`0000eoo`003goo
00D007ooOomoo`0000Eoo`800005E_d005KmE_d00003E_`300000eKl001Fo002E_`300000e;L001B
g002D]`00`00D]aBg0040002D]`01000D[aB_0000Tjl0`0000=>_000C[`00djl00<004jlC[`01P00
00=:W000BY`00dZL00@004ZLBY`0009:W0030016W4JL00L00003AY`004JL0096W0040016O4Il0003
@W`01000@Wa2O000149l00D0049l@Wa2O0000092O005000nO3il?W`00003?W`01000?W`nO0000cil
00<003il00000P000Sil00D003il?W`nO00000<jO@03000jOCYm008000<jO@03000jOCYm008001Ao
o`03001oogoo009oo`05001oogooOol0000;Ool01@00Oomoogoo00003Goo000?Ool01@00Oomoogoo
000017oo0P0000=oo`00E_d00eKm00D005Kl001Fo00000=Fo005001Fo000D_`00003D]`00`00D]aB
g0030003D]`00`00D]aBg0060002C[`01000C[a>_0000Tjl100000E>_000B[a:_00000=:W003001:
W4ZL00X00096W0040016W4JL0003AY`01@00AYa6W4JL00000TIl00D0049l@Wa2O0000092O0050012
O49l@W`00002@W`E0002?W`01@00?W`nO3il00000cil00T003Ym>WdjO@00>Wd003Ym00000cYm00@0
03Ym>Wd001Aoo`03001oogoo009oo`05001oogooOol0000;Ool01@00Oomoogoo00003Goo000?Ool0
1@00Oomoogoo000017oo00D007ooOomoo`00009Fo@<000=Fo003001Fo5Kl008000=Bo003001Bg5;L
00<000=Bg003001Bg5;L00<00003D]`004jl009>_003001>_4jl00H0009>_005001>_4jlC[`00002
B[`00`00BYa:W0030002BY`01000BYa:W0000TZL00<004JLAY`00TJL00@004JLAY`00096W1D00092
O0040012O49l0003@W`01P00?W`nO3il000nO0<00003?W`0000001/001Aoo`05001oogooOol00003
Ool00`00Oomoo`09Ool4000>Ool000moo`05001oogooOol00004Ool00`00Oomoo`02Ool20003E_`0
0`00E_aFo0030002E_`00`00D_aBo0030003D]`00`00D]aBg00300000e;L001Bg002D]`00`00C[a>
_00300000djl001>_003C[`01000C[a>_0000Tjl1P0000=:W000BY`00dZL00@004ZLBY`0009:W003
0016W4JL00L00003AY`004JL0096W0040016W4JL0003@Y`01000@Wa2O0000d9l00<0049l@W`00T9l
00@0049l@W`000=2O003000nO3il0080008nO004000nO3il0003?W`01@00?W`nO3il00000cil00D0
03im?WdnO@00008jO@8000<jO@03000jOCYm008001=oo`80009oo`05001oogooOol0000;Ool00`00
Oomoo`0?Ool000moo`05001oogooOol00004Ool20005Ool00`00E_aFo0030002E_`00`00E_aFo003
0002D]`00`00D]aBg00400000e;L001Bg002D]`00`00D]aBg00300000djl001>_002C[`00`00C[a>
_00700000dZl001:_002B[`01000BYa:W0000TZL2P000TJL00@004JLAY`000=6W0050016W4JLAY`0
0002AY`01@00@Ya2W4:L00000T9l00D0049l@Wa2O0000092O1@000<nO005000nO3il?W`00003?W`0
2@00?WdnOCim000jO@00>Wd00003>Wd01000>WdjO@004goo0P000Woo00D007ooOomoo`0000aoo`03
001oogoo00ioo`003goo00D007ooOomoo`0000Aoo`03001oogoo00Eoo`80009Fo003001Fo5Kl00<0
009Bg003001Bg5;L00<0009Bg005001Bg5;LD]`00002D]`300000djl001>_002C[`00`00C[a>_003
0002C[`01000C[a>_0000dZl00<004ZlB[`01P000TZL00@004ZLBY`000=6W0050016W4JLAY`00002
AY`00`00AYa6W00E00000d9l0012O002@W`300001D9l000nO3il00000cil00<003il?W`07@0057oo
00<007ooOol00`003Woo0`003Goo000@Ool30005Ool00`00Oomoo`06Ool00`00E_aFo0030002D_`0
0`00D]aBg0030002D]`01@00D]aBg5;L00000U;L0`0000=Bg000C[`00Tjl00<004jlC[`00`0000=>
_000C[`00djl00@004jlC[`0009:_0<00005B[`004ZLBY`00004BY`01000BYa:W0000TJL2P0000=6
W000AY`00TJL00@004JLAY`000=2W0040012W4:L0003@W`01000@Wa2O49l0`0000=2O000@W`00T9l
00@003il?W`000<nO004000nO3il0003?W`01@00?W`nO3il00000cil00D003im?WdnO@00008jO@80
00<jO@03000jOCYm008001=oo`03001oogoo025oo`005goo0P002Goo0P000U;l00<005;lD_`00`00
0U;L00D005;LD]aBg000009Bg0<0009Bg004001Bg5;L0002C[`300000djl001>_002C[`00`00C[a>
_00700000dZl001:_002B[`01000BYa:W0000TZL1`0000=6W000AY`00TJL00@004JLAY`000=6W005
0016W4JLAY`00002AY`01@00@Ya2W4:L00000T:L00@0049l@Wa2O1L000<nO005000nO3il?W`00003
?W`01@00?WdnOCim00000SYm0P000cYm00@003Ym>Wd001Aoo`80025oo`005goo00<007ooOol02Goo
00<005;lD_`00`000U;L00D005;LD]aBg000009Bg0<0009Bg003001Bg5;L00H0009>_004001>_4jl
0002C[`30002C[`01@00C[a>_4jl00000TZl00<004ZlB[`01P000TZL00@004ZLBY`000=6W0050016
W4JLAY`00002AY`00`00AYa6W00E00000d9l0012O002@W`01@00@Wa2O49l00000Sil00D003il?W`n
O000008nO1/00003>Wd007oo019oo`80025oo`005goo00<007ooOol02Woo0P000U;l00D005;LD]aB
g000009Bg0<0009Bg003001Bg5;L00<00003D]`004kL009>g003001>_4jl00H00003C[`004jl00=>
_003001>_4jl00H00003B[`004ZL009:W005001:W4ZLBY`00002BY`00`00AYa6W00:00000dJL0016
W002AY`01000AYa6W0000d:L100014:L00@0049l@W`000=2O0050012O49l@W`00002?W`01@00?W`n
O3il00000cil00@003il?W`000<nO004000nOCim0003?Wd01@00?WdnOCim00000cYm00<003Ym>Wd0
0P0057oo00<007ooOol07goo000GOol2000<Ool01@00D_aBo5;l00000U;L0`000U;L00<005;LD]`0
0`0000=Bg000D]`00U;L00<004kLC]`00`0000=>_000C[`00Tjl00@004jlC[`0009>_0@00003B[`0
04Zl009:_004001:_4Zl0002BY`:00000dJL0016W002AY`01@00AYa6W4JL00000TJL00D004JLAYa6
W0000096W0<00005@Y`004:L@Y`00002@Y`L0003?W`01000?WdnO@000cim00D003im?WdnO@0000<j
O@04000jOCYm000EOol00`00Oomoo`0NOol001Moo`03001oogoo00aoo`03001Bo5;l00<0009Bg003
001Bg5;L00<00003D]`005;L009Bg003001Bg5;L00<00003C]`004jl009>_003001>_4jl00H0009>
_005001:_4ZlB[`00002B[`600000dZl001:W002BY`01@00BYa:W4ZL00000TJL00D004JLAYa6W000
0096W1@00092W0040012W4:L0003@W`01000@Wa2O0000cil00D003il?W`nO00000<nO003000nO3il
01P001Eoo`03001oogoo01eoo`005goo00<007ooOol03Goo0P000U;L00<005;LD]`00`000U;L00@0
05;LD]`0009Bg0<0009>g004001>g4kL0002C[`300000djl001>_003C[`01000C[a>_0000TZl0`00
00E:_000B[a:_00000=:_005001:W4ZLBY`00002BY`:000014JL0016W4JL1@000T:L00D004:L@Ya2
W00000=2W0050012W4:L@Y`00002@Y`01@00@Wa2O49l00000T9l00D003il?W`nO00000<nO005000n
O3il?W`00002?W`01000?WdnO@000cim00D003im?WdnO@0000<nO@05000jOCYm>Wd00002>Wd2000C
Ool2000OOol001Moo`8000moo`03001Bg5;L00<0009Bg003001Bg5;L00H0009Bg003001>g4kL00H0
009>_004001>_4jl0002C[`700000dZl001:_002B[`01000B[a:_0000TZl1`0000E6W000AYa6W000
00=6W0040016W4JLAY`40003AY`01000@Ya2W0000T:L9P000cim00D003im?WdnO@0000<nO@05000j
OCYm>Wd00002>Wd00`00Oomoo`0COol00`00Oomoo`0LOol001Ioo`03001oogoo011oo`80009Bg003
001Bg5;L00<00003D]`005;L009Bg003001Bg5;L00<00003C]`004jl009>_003001>_4jl00H0009>
_005001:_4ZlB[`00002B[`00`00B[a:_00600000dZl001:W003BY`01000AYa6W0000TJL00<004JL
AY`04@0000=2W000@Y`00T:L00D004:L@Ya2W0000092W0050012W4:L@Y`00002@W`01@00?W`nO3il
00000cil00@003il?W`000<nO@03000nOCim01D001Eoo`03001oogoo00Eoo`<000Ioo`8000Eoo`<0
00=oo`003Woo100017oo00<007ooOol04Goo00<005;LD]`00`0000=Bg000D]`00U;L00<005;LD]`0
0`0000=>g000C]`00TkL00<004jlC[`00`0000=>_000C[`00Tjl00D004jlC[a>_000009:_0H00003
B[`004Zl009:_004001:_4Zl0002BY`<0003AY`01000AYa6W0000dJL00D004:L@Ya2W0000092W005
0012W4:L@Y`00003@Y`01000@Ya2W0000d:L00@0049l@W`000<nO005000nO3il?W`00002?W`01@00
?WdnOCim00000cim00@003im?Wd000<nO@05000nOCim?Wd00003?Wd00`00>WdjO@02000DOol00`00
Oomoo`04Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`00009oo`004Woo00D007ooOomo
o`0001Aoo`800003D]`005;L009Bg003001Bg5;L00<0009Bg004001>g4kL0002C]`300000djl001>
_002C[`00`00C[a>_00700000dZl001:_002B[`01000B[a:_0000TZl1P000TZL00<004JLAY`00`00
00E6W000AYa6W00000=6W0040016W4JL0003AY`00`00@Y`0000]0003?Wd01@00?WdnOCim00000cim
00D003Ym>WdjO@0001=oo`8000Eoo`05001oogooOol0000;Ool01@00Oomoogoo00000Woo000BOol0
1000Oomoogoo0P0057oo00@005;LD]`0009Bg0<0009Bg003001Bg5;L00H0009>g004001>_4jl0002
C[`300000djl001>_003C[`01000B[a:_0000TZl1P0000E:_000B[a:_00000=:W0<00004AY`004JL
AY`@0002@Y`01000@Ya2W0000d:L00@004:L@Y`000=2W0050012W4:L@Y`00002@Y`01@00?W`nO3il
00000cil00@003im?Wd000<nO@05000nOCim?Wd00002?WdB000DOol00`00Oomoo`02Ool01@00Oomo
ogoo00002goo00D007ooOomoo`00009oo`004Woo00D007ooOomoo`0001Ioo`D0009Bg003001Bg5;L
00<00003C]`004kL009>g003001>g4kL00H0009>_004001>_4jl0002C[`700000dZl001:_002B[`0
1000B[a:_0000TZl2P0000E6W000AYa6W00000=6W0050016W4JLAY`00002AY`01@00@Ya2W4:L0000
0T:L00D004:L@Ya2W0000092W0050012W4:L@Y`00003@Y`01000@Ya2W0000cil00D003il?W`nO000
008nO@05000nOCim?Wd00003?Wd01@00?WdnOCim00000Sim00@003im?WdnO@80008nO@05000jOCYm
>Wd0000DOol00`00Oomoo`02Ool01@00Oomoogoo000037oo0`000goo000>Ool40004Ool00`00Oomo
o`0DOol01@00D]aBg5;L00000U;L0`0000=Bg000C]`00TkL00<004kLC]`00`0000=>g000C[`00Tjl
00<004jlC[`01P000TZl00D004ZlB[a:_000009:_003001:_4Zl00T0009:_0050016W4JLAY`00002
AY`01000AYa6W0000dJL00<004JLAY`0>0000cim00D003im?WdnO@0000<jO@03001oogoo019oo`05
001oogooOol00003Ool00`00Oomoo`09Ool01@00Oomoogoo00000Woo000>Ool00`00Oomoo`04Ool2
000GOol00`00D]aBg0030002D]`01000D]aBg0000TkL0`000TkL00@004kLC]`0009>_0<00003C[`0
04jl009>_006001:_4ZlB[`004Zl1`0000I:_000000004Zl0002B[`01P00B[a:_4Zl0016W0h00005
AY`004JLAY`00003@Y`01000@Ya2W0000d:L00@004:L@Y`000=2W0040012W4:L0003@Y`01@00?Ydn
WCjM00000Sim00D003im?WdnO@0000<nO@05000nOCim?Wd00003?Wd00`00?WdnO@0=000COol00`00
Ool00002Ool01@00Oomoogoo00002goo00D007ooOomoo`00009oo`003Woo00<007ooOol017oo00<0
07ooOol05goo0P000U;L00<005;LD]`01P000TkL00<004kLC]`01P0000=>_000C[`00Tjl00<004jl
C[`01P000TZl00<004ZlB[`00`0000A:_000B[a:_0T00096W0040016W4JL0002AY`01@00AYa6W4JL
00000dJL00@004JLAY`000=2W0040012W4:L0003@Y`01000@Ya2W0000d:L00@004:L@Y`000=2W005
000nWCjM?Yd00002?Wd01@00?WdnOCim00000cim00D003im?WdnO@0000<nO@04000nOCim0002?Wd0
1@00?Wd003im00000cim00<007ooOol04goo00@007ooOomoo`<000eoo`<000=oo`003Woo10000goo
00<007ooOol067oo00<005;LD]`00`0000=Bg000C]`00TkL00<004kLC]`00`0000=>g000C]`00TkL
00@004jlC[`0009>_0<00003C[`004Zl009:_003001:_4Zl00<00003B[`0000000@00005B[`004Zl
B[`00003A[`01@00AYa6W4JL00000TJL00@004JLAY`00096W3l00004?Wd003im0003?Wd00`00Oomo
o`0COol00`00Oomoo`0FOol001Eoo`8001Yoo`80009Bg004001Bg5;L0002C]`30002C]`01000C]a>
g0000TkL1P000Tjl00@004jlC[`0009:_0L00003B[`004Zl009:_004001:_4Zl0002B[`00`00A[a6
_00=00000dJL0016W002AY`01000AYa6W0000d:L00D004:L@Ya2W0000092W0050012W4:L@Y`00002
@Y`01@00@Ya2W4:L00000d:M00@004:M@Yd000<nW@04000nOCim0003?Wd01000?WdnO@000cim00D0
03im?WdnO@00008nO@X001Eoo`03001oogoo01Eoo`005Goo00<007ooOol06Woo00<005;LD]`01P00
0TkL00<004kLC]`01P0000=>g000C[`00Tjl00<004jlC[`01P0000=:_000B[`00TZl00D004ZlB[a:
_000009:_0T00003A[`004JL0096W0050016W4JLAY`00002AY`01000AYa6W0000dJL00@004JLAY`0
0092W28000<nO@04000nOCim0003?Wd01@00?WdnOCim00000cim0P0000<nO@00?Wd00Sim00<007oo
Ool04Woo0P005goo000EOol00`00Oomoo`0KOol200000e;L001>g002C]`00`00C]a>g00300000dkL
001>g002C]`01000C]a>g0000Tjl0`0000=>_00000000TZl00@004ZlB[`0009:_0L00005B[`004Zl
B[`00003B[`01000A[a6_0000dJL00<004JL0000400000=2W000@Y`00T:L00@004:L@Y`000=2W@04
0012WD:M0003@Yd01@00@Ye2WD:M00000T:M00D003jM?YdnW@00008nWAP000<nO@03001oogoo01=o
o`03001oogoo01Aoo`005Goo00<007ooOol077oo00@005;LD]`0009>g0<0009>g004001>g4kL0002
C]`600001Tjl00000000C[`0009:_0H00003B[`004Zl009:_005001:_4ZlB[`00002B[`00`00B[a:
_0090002AY`01000AYa6W0000dJL00@004JLAY`000=6W0040012W4:L0003@Y`01000@Ya2W0000d:M
00@004:M@Yd000=2W@050012WD:M@Yd00002@Yd01@00?YdnWCjM00000SjM00D003im?WdnO@00008n
O@05000nOCim?Wd00003?Wd01P00?WdnOCim000nO@L001Aoo`03001oogoo01Aoo`0057oo00<007oo
Ool07Woo1@000TkL00<004kLC]`01P000TkL00<004jl00000P0000=>_0000000100000=:_000B[`0
0TZl00@004ZlB[`0009:_0X00003A[`004Jl0096_0040016W4JL0003AY`01000AYa6W0000TJL<`00
0cim00D003im?WdnO@0000<nO@800003?Wd003im008nO@03001oogoo01=oo`03001oogoo01=oo`00
57oo00<007ooOol07Woo00D005;LD]aBg000009>g0<00003C]`004kL009>g003001>g00000H00003
C[`004jl009>_004001:_4Zl0002B[`600000dZl001:_003B[`01000B[a:_0000TJl00<004JlA[`0
3@0000E6W000AYa6W00000=6W0040012W4:L0003@Yd01000@Ye2W@000d:M00@004:M@Yd000=2W@04
0012WD:M0003@Yd01@00?YdnWCjM00000SjM00D003jM?YdnW@00008nOA<00003?Wd007oo01Aoo`03
001oogoo019oo`0057oo0P0087oo00<005;LD]`00`000TkL00D004kLC]`004kL00@00003C]`004jl
009>_004001>_4jl0002C[`600000dZl001:_002B[`01000B[a:_0000TZl2P0000E6_000AYa6W000
00=6W0050016W4JLAY`00002AY`01000AYa6W0000dJL00@004:L@Y`000=2W@040012WD:M0003@Yd0
1000@Ye2W@000d:M00@004:M@Yd000=2W@05000nWCjM?Yd00002?Yd01@00?YdnWCjM00000cim00@0
03im?Wd000<nO@04000nOCim0002?Wd20002?Wd2000EOol00`00Oomoo`06Ool50007Ool001Aoo`03
001oogoo021oo`80009>g003001>g4kL00H0009>g004001>g4kL0002C[`60002B[`01000B[a:_000
0TZl00<004ZlB[`01P000TZl00D004ZlB[a:_0000096_0040016_4Jl0002AY`00`00AYa6W0100000
2Cim000nOCim000nO@00?Wd00003?Wd00`00Oomoo`0COol00`00Oomoo`07Ool00`00Oomoo`07Ool0
00Aoo`D000=oo`<000Eoo`03001oogoo025oo`03001>g4kL00D000=>g003001>g4kL00H0009>_004
001>_4jl0002B[`90002B[`01000B[a:_0000dZl00<004Zl0000300000=6W000AY`00TJL00D004JL
AYa6W0000096W@040012WD:M0003@Yd01000@Ye2W@000d:M00@004:M@Yd000=2W@040012WD:M0003
@Yd01000@Ye2W@000cjM00@003jM?Yd000<nW@05000nWCjM?Yd00002?Wd00`00?WdnO@0>000DOol2
0008Ool00`00Oomoo`07Ool000Ioo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`0R
Ool50002C]`00`00C]a>g00300000dkL001>g002C]`00`00C[a>_00600000dZl001:_002B[`01000
B[a:_0000TZl2@000TZl00@004JlA[`000=6_0040016_4Jl0002AY`01P00AYa6W4JL0016W2`000<n
W@05000nWCjM?Yd00002?Wd01@00?WdnOCim00000cim00@003im000000<nO@03001oogoo01=oo`03
001oogoo00Ioo`03001oogoo00Moo`001Woo00<007ooOol00Woo00D007ooOomoo`0000=oo`8002=o
o`<0009>g0H0009>g004001>g4kL0002C]`300000djl001>_002C[`01000B[a:_0000TZl1P0000=:
_000B[`00TZl00@004ZlB[`000=:_0040016_4Jl0002A[`<0002AYd01000AYe6W@000T:M00D004:M
@Ye2W@000092W@050012WD:M@Yd00002@Yd01@00@Ye2WD:M00000T:M00D004:M@Ye2W@000092W@05
000nWCjM?Yd00002?YdH0003?Wd00`00Oomoo`0COol00`00Oomoo`05Ool00`00Oomoo`07Ool000Io
o`03001oogoo009oo`05001oogooOol00003Ool00`00Oomoo`0OOol30003Ool20002C]`01000C]a>
g0000TkL1P000TkL00@004jlC[`0009>_0H00003B[`004Zl009:_004001:_4Zl0002B[`=00000dJl
0016_@02A[d01000AYe6W@000TJM00D004JMAYe6W@000096W@040012WD:M0003@Yd01000@Ye2W@00
0d:M00@004:M@Yd000=2W@040012WD:M0003@Yd01000@Ye2W@000cjM00@003jM?Yd000<nW@05000n
WCjM?Yd00002?Wd01@00?WdnOCim00000cim00<003im00001P004goo0`001Woo00<007ooOol01goo
0006Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00<007ooOol077oo0`001goo00<004kLC]`0
1P000TkL00@004kLC]`0009>g0H0009:_004001:_4Zl0002B[`00`00B[a:_00600000dZl001:_002
B[`01000A[a6_0000TJl00D004JlA[a6_0000096_D0000<nO@06000nOCim?Wd003im0P000cim00<0
07ooOol03Woo0`0000=oo`0000001Woo00<007ooOol01goo0006Ool00`00Oomoo`02Ool01@00Oomo
ogoo00000goo00<007ooOol06Goo0`002goo0P0000=>g000C]`00TkL00<004kLC]`01P000TkL00@0
04jlC[`0009:_0T0009:_004001:_4Zl0002B[`00`00B[a:_00=00001DJm0016WDJM00000TJM00D0
04JMAYe6W@0000=6W@040012WD:M0002@Yd01@00@Ye2WD:M00000T:M00D004:M@Ye2W@000092W@05
0012WD:M@Yd00002@Yd01@00@Ye2WD:M00000SjM00@003jM?Yd000<nW@05000nWCjM?Yd00002?Yd?
00000cim001oo`0;Ool30005Ool00`00Oomoo`02Ool30009Ool000Ioo`03001oogoo009oo`05001o
ogooOol00003Ool00`00Oomoo`0FOol3000?Ool01000C]a>g0000TkL0`0000=>g000C]`00TkL00<0
04kLC]`01P0000=:_000B[`00TZl00@004ZlB[`0009:_0T00005B[d004JmA[d00003A[d01000A[e6
_@000dJm00@004JMAYd00096W@030016WDJM02P0008nW@05000nWCjM?Yd00003?Yd01000?YdnW@00
0cim00@003im?Wd000<nO@03000nOCim008000Uoo`<000Qoo`03001oogoo00ioo`0017oo0`001Goo
0`0017oo00<007ooOol04goo0`004goo1@000TkL00@004kLC]`0009>g0<00003C]`004jl009>_004
001:_4Zl0002B[`600000dZl001:_@02B[d01000B[e:_@000TZm00@004JmA[d00096_A000003AYd0
04JM0096W@040012WD:M0002@Yd01@00@Ye2WD:M00000T:M00D004:M@Ye2W@000092W@050012WD:M
@Yd00002@Yd01@00@Ye2WD:M00000SjM5`000Sim0P000cim00<007ooOol00Woo100037oo00<007oo
Ool03Goo000BOol00`00Oomoo`0AOol3000GOol01000C]a>g0000TkL1P000TkL00@004kLC]`0009>
_0H0009:_004001:_4Zl0002B[`00`00B[e:_@0<00000dJm0016_@02A[d01000A[e6_@000TJM00D0
04JMAYe6W@000096W@050016WDJMAYd00002@Yd01000@Ye2W@000d:M00@004:M@Yd000=2W@040012
WD:M0003@Yd01000@Ye2W@000d:M00@003jM?Yd000<nW@04000nWCjM0002?Yd01@00?YdnWCjM0000
0cjM00<003im?Wd03@0047oo00<007ooOol03Goo000BOol2000@Ool2000KOol50002C]`01000C]a>
g0000TkL1P000TZl00@004ZlB[`0009:_0T00003B[d004Zm009:_@04001:_DZm0002A[d01000A[e6
_@000TJm?`000cjM00D003jM?YdnW@0000<nO@04000nOCim0003?Wd00`00Oomoo`0AOol00`00Oomo
o`0=Ool0019oo`03001oogoo00aoo`<001ioo`04001>g4kL0002C]`60002C]`01000C]a>g0000TZl
1P0000E:_@00B[e:_@00009:_@05001:_DZmB[d00002B[d<00001DJm0016_DJm00000dJM00@004JM
AYd000=6W@040016WDJM0002@Yd01@00@Ye2WD:M00000T:M00@004:M@Yd000=2W@040012WD:M0003
@Yd01000@Ye2W@000d:M00@003jM?Yd000<nW@04000nWCjM0002?YdA0003?Wd00`00Oomoo`0@Ool0
0`00Oomoo`0=Ool0019oo`03001oogoo00Uoo`<0029oo`D0009>g004001>g4kL0002C]`600001DZm
001:_DZm00000dZm00<004Zm00002@0000E:_@00A[e6_@0000=6_@040016_DJm0002A[d01000A[e6
_@000TJM;P000cjM00@003jM?Yd000<nW@04000nWCjM0003?Yd01@00?YdnWCjM00000Sim1@004Woo
00<007ooOol03Goo000BOol00`00Oomoo`06Ool3000VOol01000C]a>g0000TkL1P000TkM00@004jm
C[d0009:_@03001:_@0000D00003B[d004Zm009:_@04001:_DZm0003B[d00`00A[d0000?00001DJm
0016WDJM00000dJM00D004:M@Ye2W@000092W@040012WD:M0003@Yd01000@Ye2W@000d:M00@004:M
@Yd00092W@040012WD:M0003@Yd00`00?YdnW@0B00000cjM000nW@02?Yd01000?WdnO@000cim00<0
07ooOol03goo00<007ooOol03Goo000BOol00`00Oomoo`03Ool3000ZOol50002C]d01000C]e>g@00
0TkM20000TZm00@004ZmB[d0009:_@03001:_DZm00T00003A[d004Jm0096_@040016_DJm0003A[d0
1000A[e6_@000TJm00@004JMAYd00096WAd00092W@050012WD:M@Yd00002?Yd01@00?YdnWCjM0000
0cjM00@003jM?Yd0008nW@03000nWCjM00h0011oo`03001oogoo00eoo`004Goo00@007ooOomoo`<0
02ioo`04001>gDkM0002C]d600001DkM001:_DZm00000TZm00<004ZmB[d02@000TZm00@004ZmB[d0
009:_@040016_DJm0002A[dC00000dJM0012W@02@Yd01000@Ye2W@000T:M00D004:M@Ye2W@000092
W@050012WD:M@Yd00002@Yd00`00@Ye2W@0G0002?Yd01@00?YdnWCjM00000cjM00@003im?Wd000<n
O@03001oogoo00moo`03001oogoo00aoo`004Goo1000<Woo1@000TkM00@004kMC]d0009>g@03001:
_@0000D0009:_@04001:_DZm0002B[d00`00B[e:_@0<00000dJm0016_@02A[d01000A[e6_@000TJm
00D004JmA[e6_@000096W@040016WDJM0003@Yd01000@Ye2W@000T:M00D004:M@Ye2W@000092W@05
0012WD:M@Yd00002@Yd01000@Ye2W@000d:M00@004:M@Yd000<nW@04000nWCjM0003?Yd00`00?Ydn
W@0@0003?Wd00`00Oomoo`0>Ool00`00Oomoo`0<Ool0015oo`8003Eoo`04001>gDkM0002C]d80002
B[d01000B[e:_@000TZm2@0000=:_@00B[d00TZm00@004ZmB[d00096_@040016_DJm0002A[dk0003
?Yd01000?YdnW@000cjM00@003jM?Yd000<nW@03000nWCjM00D0011oo`03001oogoo00aoo`0000=o
o`0000000`000goo10001Woo00<007ooOol0<goo1@000TkM00<004kM00000TkM00<004ZmB[d01P00
0TZm00@004ZmB[d0009:_@04001:_DZm0002B[d<00000dJm0016_@02A[d01000A[e6_@000TJm00D0
04JMAYe6W@000096W@040012WD:M0002@Yd01@00@Ye2WD:M00000T:M00@004:M@Yd000=2W@040012
WD:M0003@Yd01000@Ye2W@000SjM00D003jM?YdnW@00008nWA800004?Yd003jM?Yd20002?Wd00`00
Oomoo`0=Ool00`00Oomoo`0<Ool000=oo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo03=o
o`04001>gDkM0002C]d80002B[d01000B[e:_@000TZm30000TZm00@004JmA[d00096_@040016_DJm
0002A[d00`00A[e6_@0`0003?Yd01000?YdnW@000cjM00@003jM?Yd000<nW@04000nWCjM0002?Yd:
000>Ool00`00Oomoo`0<Ool000=oo`03001oogoo00Moo`03001oogoo00Eoo`03001oogoo03=oo`D0
0005C]d004kMC]d00002B]d00`00B[e:_@0600000dZm001:_@02B[d01000B[e:_@000TZm00<004Zm
B[d03`0000E6_@00A[e6_@0000=6_@040016WDJM0003AYd01000@Ye2W@000T:M00@004:M@Yd000=2
W@040012WD:M0003@Yd00`00@Yd00003@Yd01000@Ye2W@000SjM4P0000DnW@00?YdnW@0000<nW@03
000nWCjM0080008nW@03001oogoo00aoo`03001oogoo00aoo`000goo00<007ooOol01goo00<007oo
Ool01Woo0P00=7oo00<004kM00000dkM00<004kM00001@000TZm00@004ZmB[d0009:_@03001:_DZm
00T00005B[d004JmA[d00002A[d01@00A[e6_DJm00000TJm00@004JmA[d00096_@030016_DJm01l0
00=2W@040012WD:M0002?Yd01@00?YdnWCjM00000cjM00@003jM?Yd0008nW@03000nWCjM00h0008n
W@03001oogoo00]oo`03001oogoo00aoo`000goo00<007ooOol00goo10002goo00<007ooOol0<Woo
1`000TkM00@004[MB]d0009:_@T00005B[d004ZmB[d00002B[d01P00B[e:_DZm0016_A<00005A[d0
04JMAYd00003AYd01000@Ye2W@000T:M00@004:M@Yd000=2W@040012WD:M0003@Yd00`00@Ye2W@0F
0002?Yd01@00?YdnWCjM00000SjM00D003jM?YdnW@00008nW@D000ioo`03001oogoo00]oo`000goo
00<007ooOol00goo00<007ooOol03Goo00<007ooOol0<Goo00@004kMC]d0009>g@03001>gDkM00H0
009:_@03001:_@00009:_@03001:_DZm00`00096_@040016_DJm0002A[d01000A[e6_@000TJm00D0
04JmA[e6_@000096_@040016WDJM0003AYd01000@Ye2W@000T:M00@004:M@Yd000=2W@040012WD:M
0003@Yd01000@Ye2W@000T:M00@004:M@Yd000<nW@04000nWCjM0003?Yd00`00?YdnW@0>000013jM
000nWCjM0P000SjM00<007ooOol02goo00<007ooOol02goo0003Ool00`00Oomoo`03Ool00`00Oomo
o`0>Ool00`00Oomoo`0aOol80002B]d01000B]e:g@000TZm20000TZm00@004ZmB[d0009:_@04001:
_DZm0002A[e00003?Yd01000?YdnW@000SjM00D003jM?YdnW@00008nW@03000nWCjM00X000aoo`03
001oogoo00]oo`0000Aoo`00000000Eoo`@000ioo`80039oo`04001>gDkM0002C]d900001DZm001:
_DZm00000TZm00<004ZmB[d0300000=6_@00A[d00TJm00@004JmA[d00096_@040016_DJm0002A[d0
1000A[e6_@000dJM00@004:M@Yd00092W@040012WD:M0003@Yd01000@Ye2W@000T:M00D004:M@Ye2
W@000092W@050012WD:M@Yd00002@Yd01000?YdnW@000SjM4`000cjM00@003jM?YdnW@800003?Yd0
07oo00]oo`03001oogoo00]oo`007Goo00<007ooOol0<7oo1@000TkM00H004[MB]d004[M0002B[d9
0002B[d01000B[e:_@000TZm00@004JmA[d00096_@030016_DJm03T0008nW@05000nWCjM?Yd00002
?Yd01000?YdnW@000cjM00<003jM?Yd03@002goo00<007ooOol02goo000NOol00`00Oomoo`0`Ool0
1000C]e>g@000TkM20000TZm00@004ZmB[d0009:_@03001:_DZm00l00003A[d004Jm0096_@040016
_DJm0002A[d01000A[e6_@000TJm00@004JmA[d00092W@050012WD:M@Yd00002@Yd01000@Ye2W@00
0d:M00@004:M@Yd00092W@050012WD:M@Yd00002@Yd01000@Ye2W@000SjM3`000cjM00@003jM?Yd0
00<nW@04000nWCjM?Yd200000cjM001oo`0:Ool00`00Oomoo`0;Ool001moo`03001oogoo031oo`D0
0005B]d004[MB]d00002B]d00`00B[e:_@090002B[d00`00B[d00002B[d01000A[e6_@000TJm00<0
04JmA[d0<`000T:M00@003jM?Yd000<nW@04000nWCjM0002?Yd00`00?YdnW@0?00001CjM000003jM
00002goo00<007ooOol02Woo000POol00`00Oomoo`0`Ool01@00C]e>g@00B]d02@000TZm00@004Zm
B[d0009:_@03001:_DZm00h00003A[d004Jm0096_@040016_DJm0002A[d01000A[e6_@000dJm00@0
04JmA[d00092W@040012WD:M0003@Yd01000@Ye2W@000T:M00@004:M@Yd000=2W@040012WD:M0002
@YdB0002?Yd01@00?YdnWCjM00000cjM00@003jM?Yd0008nW@H000Yoo`03001oogoo00Yoo`008Goo
0P00<Goo10000T[M00@004[MB]d0009:_@03001:_DZm00T00005B[d004ZmB[d00002B[d01000A[e6
_@000TJm00@004JmA[d00096_B`00092W@040012WD:M0003?Yd01000?YdnW@000SjM00<003jM?Yd0
40000SjM00D003jM000nW@0000Yoo`03001oogoo00Yoo`008goo00<007ooOol0;goo00<004kM0000
0T[M2@000TZm00<004Zm00000TZm00<004ZmB[d04P000TJm00@004JmA[d00096_@040016_DJm0002
A[d01000A[e6_@000d:M00@004:M@Yd00092W@040012WD:M0003@Yd00`00@Yd00003@Yd00`00@Ye2
W@0B0002?Yd01@00?YdnWCjM00000cjM00@003jM?Yd0008nW@T000Uoo`03001oogoo00Yoo`0097oo
00<007ooOol0;goo10000T[M00@004[MB]d0009:_@03001:_DZm00P0009:_@04001:_DZm0002A[d0
1000A[e6_@000TJm00@004JmA[d00096_B@000=2W@040012WD:M0002@Yd01@00@Ye2WD:M00000SjM
00@003jM?Yd0008nWA00008nW@06000nWCjM?Yd003jM0P002Goo00<007ooOol02Woo000UOol00`00
Oomoo`0^Ool01000C]e>g@000T[M2@0000E:_@00B[e:_@00009:_@03001:_DZm01800003A[d004Jm
0096_@040016_DJm0002A[d01000A[e6_@000T:m00@004:M@Yd00092W@050012WD:M@Yd00002@Yd0
1000@Ye2W@000T:M5@000SjM00D003jM?YdnW@00008nW@05000nWCjM?Yd00002?Yd<0008Ool00`00
Oomoo`0:Ool002Ioo`8002moo`D0009:g@05001:gD[M001:_@0<0002B[d00`00B[d00002A[d01000
A[e6_@000TJm00@004JmA[d00096_B800092W@040012WD:M0002@Yd01@00@Ye2WD:M00000T:M00@0
03jM?Yd0008nW@h00005?Yd003jM?Yd00003?Yd01@00?YdnW@00?Yd00`001goo00<007ooOol02Woo
000XOol00`00Oomoo`0]Ool00`00B]e:g@080002B[d01000B[e:_@000TZm00@004ZmB[d0009:_A40
0096_@040016_DJm0002A[d01000A[e6_@000TJm00D004:m@[e2_@000092W@040012WD:M0002@Yd0
1000@Ye2W@000T:M5@000SjM00D003jM?YdnW@00008nW@04000nWCjM0002?Yd=00000cjM00000008
Ool00`00Oomoo`09Ool002Uoo`03001oogoo02eoo`80009:g@03001:g@00009:g@03001:_DZm00`0
0005B[d004ZmB[d00002A[d01000A[e6_@000TJm00@004JmA[d00096_A/00092W@040012WD:M0002
@Yd01000@Ye2W@000d:M00@004:M@Yd00092W@03000nWCjM00l000<nW@04000nWCjM0003?Yd00`00
?YdnW@050007Ool00`00Oomoo`09Ool002Yoo`03001oogoo02eoo`03001:gD[M00P0009:_@06001:
_DZm001:_@000TZm00<004ZmB[d04P000TJm00@004JmA[d00096_@040016_DJm0002A[d01000@[e2
_@000T:M00@004:M@Yd00092WAP0008nW@05000nWCjM?Yd00002?Yd01P00?YdnWCjM000nW@/00009
?Yd003jM?Yd003jM000nW@0000Ioo`03001oogoo00Uoo`00:goo0P00;Woo0P0000E:g@00B]e:g@00
009:g@03001:_DZm00/0009:_@040016_DJm0002A[d01000A[e6_@000TJm00@004JmA[d00096_AL0
0092W@040012WD:M0002@Yd01@00@Ye2WD:M00000T:M00@004:M@Yd000=2W@030012W@00014000<n
W@06000nWCjM?Yd003jM2P001Goo00<007ooOol02Goo000]Ool00`00Oomoo`0]Ool:00001DZm001:
_DZm00000TZm00<004ZmB[d05@0000E6_@00A[e6_@000096_@040016_DJm0003A[d01000@[e2_@00
0T:M00<004:M00006P000cjM00@003jM?Yd000<nW@04000nWCjM0002?Yd;0003?Yd00`00?YdnW@02
0005Ool00`00Oomoo`09Ool002ioo`03001oogoo02aoo`04001:gD[M0002B]d00`00B]e:g@0;0002
B[d00`00B[d00002A[d01000A[e6_@000TJm00@004JmA[d00096_@030016_DJm01L00092W@040012
WD:M0003@Yd01000@Ye2W@000T:M00D004:M@Ye2W@000092W@030012WD:M00l0008nW@04000nWCjM
0003?Yd00`00?YdnW@090004Ool00`00Oomoo`09Ool002moo`03001oogoo02aoo`P00005B]d004Zm
B[d00002B[d00`00B[e:_@0G0002A[d01000A[e6_@000TJm00@004JmA[d00096_@040012_D:m0002
@Yd00`00@Ye2W@0H0002@Yd01000?YdnW@000cjM00@003jM?Yd0008nW@X00005?Yd003jM?Yd00002
?Yd00`00?YdnW@020004Ool00`00Oomoo`09Ool0031oo`8002eoo`05001:gD[M001:g@0<00001DZm
001:_DZm00000TZm00@004JmA[d00096_@040016_DJm0002A[d00`00A[e6_@0J0002@Yd01000@Ye2
W@000T:M00D004:M@Ye2W@000092W@040012WD:M0002@Yd?0002?Yd01000?YdnW@000cjM00<003jM
0000300017oo00<007ooOol027oo000bOol00`00Oomoo`0[Ool40002B]d01000B]e:g@000TZm00D0
04ZmB[d004Zm01D00005A[d004JmA[d00002A[d01000A[e6_@000dJm00@004:m@[d00092_@050012
WD:M0012W@0H0002?Yd01000?YdnW@000cjM00<003jM?Yd03P000cjM00H003jM?YdnW@00?Yd20004
Ool00`00Oomoo`08Ool003=oo`03001oogoo02]oo`03001:g@00009:g@/0009:_@04001:_DZm0002
A[d01P00A[e6_@00A[d00096_@030016_DJm01d00092W@040012WD:M0003@Yd01000@Ye2W@000T:M
00H004:M@Ye2W@00@Yd>0003?Yd01@00?YdnWCjM00000SjM00<003jM?Yd030000goo00<007ooOol0
27oo000dOol00`00Oomoo`0[Ool40002B]d01P00B]e:g@00B[d0009:_A@00096_@040016_DJm0002
A[d01000A[e6_@000TJm00@004JmA[d00092_@040012_D:m0002@[d00`00@Ye2W@0G0002?Yd01@00
?YdnWCjM00000SjM00<003jM?Yd030000SjM00@003jM?Yd000<nW@03000nWCjM0080009oo`03001o
ogoo00Qoo`00=Goo0P00:goo00<004[MB]d02`000TZm00H004ZmB[d004Zm0002A[d01000A[e6_@00
0TJm8`000T:M00@004:M@Yd00092W@040012WD:M0003@Yd01000@Ye2W@000T:M3P000SjM00D003jM
?YdnW@00008nW@X00005?Yd003jM?Yd00002Ool00`00Oomoo`08Ool003Moo`03001oogoo02Uoo`80
009:g@03001:g@00009:_@03001:_DZm0140029ooaD00003@Yd003jM008nW@04000nWCjM0002?Yd0
0`00?YdnW@0<0002?Yd01P00?YdnWCjM000nW@H00003Ool007oo00Uoo`00>7oo00<007ooOol0:Woo
2P00Agoo3P000SjM00D003jM?YdnW@00008nW@03000nWCjM00`0008nW@03001oo`0000Yoo`00>Goo
00<007ooOol0:Goo00<007ooOol0Fgoo3`000cjM00@003jM?Yd0008nW@D00003Ool007oo00Qoo`00
>Woo0P00:Goo00<007ooOol0JGoo2@000SjM00@003jM?YdnW@8000Uoo`00?7oo00<007ooOol09Woo
00<007ooOol0LGoo2@002Goo000mOol00`00Oomoo`0UOol00`00Oomoo`1cOol4000<Ool003ioo`03
001oogoo02Aoo`03001oogoo071oo`<0011oo`00?goo00<007ooOol08goo00<007ooOol0K7oo1000
4goo0010Ool2000SOol00`00Oomoo`1XOol4000GOol0049oo`03001oogoo021oo`03001oogoo06Ao
o`@001]oo`00@goo00<007ooOol07goo00<007ooOol0H7oo10007goo0014Ool00`00Oomoo`0NOol0
0`00Oomoo`1MOol3000SOol004Eoo`8001ioo`03001oogoo05Uoo`@002Ioo`00Agoo00<007ooOol0
6goo00<007ooOol0EGoo1000:Woo0018Ool00`00Oomoo`0JOol00`00Oomoo`1AOol4000^Ool004Uo
o`03001oogoo01Uoo`03001oogoo04eoo`@0039oo`00BWoo0P0067oo00<007ooOol0Bgoo0`00=Woo
001<Ool00`00Oomoo`0EOol00`00Oomoo`17Ool4000iOol004eoo`03001oogoo01Aoo`03001oogoo
04=oo`@003eoo`00CWoo00<007ooOol04goo00<007ooOol0?goo1000@Goo001?Ool2000COol00`00
Oomoo`0kOol40015Ool0055oo`03001oogoo011oo`03001oogoo03Qoo`<004Uoo`00DWoo00<007oo
Ool03goo00<007ooOol0=7oo1000C7oo001COol00`00Oomoo`0>Ool00`00Oomoo`0`Ool4001@Ool0
05Aoo`8000ioo`03001oogoo02aoo`@005Aoo`00EWoo00<007ooOol02goo00<007ooOol0:Goo0`00
F7oo001GOol00`00Oomoo`0:Ool00`00Oomoo`0UOol4001KOol005Qoo`03001oogoo00Uoo`03001o
ogoo025oo`@005moo`00FGoo00<007ooOol027oo00<007ooOol07Goo1000Hgoo001JOol20008Ool0
0`00Oomoo`0IOol4001WOol005aoo`03001oogoo00Eoo`03001oogoo01Ioo`<006]oo`00GGoo00<0
07ooOol017oo00<007ooOol04Woo1000KWoo001NOol00`00Oomoo`03Ool00`00Oomoo`0>Ool4001b
Ool005moo`8000=oo`03001oogoo00Yoo`@007Ioo`00HGoo00@007ooOol000Qoo`@007Yoo`00HWoo
00<007oo00001Goo0`00OWoo001SOol200000goo000000020021Ool006Aoo`8008Eoo`00jgoo003[
Ool00>]oo`00\
\>"],
  ImageRangeCache->{{{0, 234}, {189.5, 0}} -> {-0.0806895, -0.0150258, 
  0.00573941, 0.00573941}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.3 Sensitivity Analysis", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nAs we have seen with the log, power and Binary options, perhaps the most \
useful feature of the analytical treatment within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is the ability to work out sensitivity parameters by elementary \
differentiation. The minimal effort involved is readily contrasted with what \
would be needed in a numerical approach, and is a good reason for \
re-considering the use of tools such as spreadsheets, where a set of \
neighbouring values must be computed and derivatives derived numerically. We \
can literally just throw the calculation to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". In the following the use of \"",
  StyleBox["Evaluate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "\" is not strictly necessary; however, it is a good habit to get into \
using it, as it is most helpful to have it in several other more complicated \
cases, with option functions defined piecewise.\n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
BlackScholesCallDelta[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], s]];

BlackScholesPutDelta[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], s]];

BlackScholesCallGamma[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], {s, 2}]];

BlackScholesPutGamma[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], {s, 2}]];

BlackScholesCallTheta[s_,k_, v_, r_, q_, t_]= 
-Evaluate[D[BlackScholesCall[s,k, v, r, q, t], t]];

BlackScholesPutTheta[s_,k_, v_, r_, q_, t_]= 
-Evaluate[D[BlackScholesPut[s,k, v, r, q, t], t]];

BlackScholesCallRho[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], r]];

BlackScholesPutRho[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], r]];

BlackScholesCallVega[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], v]];

BlackScholesPutVega[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], v]];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  PageBreakWithin->True,
  GroupPageBreakWithin->Automatic,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.4 Exploring Delta", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallDelta[p,k,\[Sigma],r,q,t]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the Black-Scholes Delta of a Call option, its sensitivity to \
changes in the price of the underlying stock. Let's work it out, and simplify \
and plot the answer in two ways:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Simplify[BlackScholesCallDelta[p,k,\[Sigma],r,q,t]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(1\/2\ E\^\(\(-q\)\ t\)\ \((1 + 
          Erf[\(t\ \((\(-2\)\ q + 2\ r + \[Sigma]\^2)\) + 2\ Log[p\/k]\)\/\(2\
\ \@2\ \@t\ \[Sigma]\)])\)\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\

Plot[BlackScholesCallDelta[x, 60, 0.29, 0.04, 0, 0.3], 
{x, 20., 100.}];
\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0119048 0.0147151 0.588824 [
[.2619 .00222 -6 -9 ]
[.2619 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.7381 .00222 -6 -9 ]
[.7381 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .13248 -18 -4.5 ]
[.01131 .13248 0 4.5 ]
[.01131 .25024 -18 -4.5 ]
[.01131 .25024 0 4.5 ]
[.01131 .36801 -18 -4.5 ]
[.01131 .36801 0 4.5 ]
[.01131 .48577 -18 -4.5 ]
[.01131 .48577 0 4.5 ]
[.01131 .60354 -6 -4.5 ]
[.01131 .60354 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(40)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(80)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13248 m
.03006 .13248 L
s
[(0.2)] .01131 .13248 1 0 Mshowa
.02381 .25024 m
.03006 .25024 L
s
[(0.4)] .01131 .25024 1 0 Mshowa
.02381 .36801 m
.03006 .36801 L
s
[(0.6)] .01131 .36801 1 0 Mshowa
.02381 .48577 m
.03006 .48577 L
s
[(0.8)] .01131 .48577 1 0 Mshowa
.02381 .60354 m
.03006 .60354 L
s
[(1)] .01131 .60354 1 0 Mshowa
.125 Mabswid
.02381 .04416 m
.02756 .04416 L
s
.02381 .0736 m
.02756 .0736 L
s
.02381 .10304 m
.02756 .10304 L
s
.02381 .16192 m
.02756 .16192 L
s
.02381 .19136 m
.02756 .19136 L
s
.02381 .2208 m
.02756 .2208 L
s
.02381 .27969 m
.02756 .27969 L
s
.02381 .30913 m
.02756 .30913 L
s
.02381 .33857 m
.02756 .33857 L
s
.02381 .39745 m
.02756 .39745 L
s
.02381 .42689 m
.02756 .42689 L
s
.02381 .45633 m
.02756 .45633 L
s
.02381 .51522 m
.02756 .51522 L
s
.02381 .54466 m
.02756 .54466 L
s
.02381 .5741 m
.02756 .5741 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01472 L
.02721 .01472 L
.02846 .01472 L
.02961 .01472 L
.03068 .01472 L
.03279 .01472 L
.03396 .01472 L
.03506 .01472 L
.03636 .01472 L
.03756 .01472 L
.04 .01472 L
.04134 .01472 L
.04262 .01472 L
.045 .01472 L
.04633 .01472 L
.04753 .01472 L
.04988 .01472 L
.0521 .01472 L
.05469 .01472 L
.05707 .01472 L
.05968 .01472 L
.061 .01472 L
.06244 .01472 L
.06527 .01472 L
.06795 .01472 L
.07035 .01472 L
.07296 .01472 L
.0758 .01472 L
.07882 .01472 L
.08139 .01472 L
.08424 .01472 L
.08687 .01472 L
.08929 .01472 L
.09194 .01472 L
.09473 .01472 L
.09717 .01472 L
.09943 .01472 L
.10192 .01472 L
.10458 .01472 L
.10717 .01472 L
.10991 .01472 L
.11482 .01472 L
.11755 .01472 L
.12011 .01472 L
.12291 .01472 L
.12593 .01472 L
.12851 .01472 L
.13133 .01472 L
Mistroke
.13381 .01472 L
.13646 .01472 L
.14145 .01472 L
.14608 .01472 L
.15145 .01473 L
.15639 .01474 L
.16181 .01475 L
.16753 .01476 L
.17251 .01478 L
.17716 .0148 L
.18225 .01483 L
.18771 .01487 L
.19301 .01493 L
.19791 .01499 L
.20329 .01509 L
.20899 .01521 L
.21857 .01551 L
.22402 .01575 L
.22913 .01602 L
.23898 .01671 L
.24394 .01716 L
.24935 .01774 L
.25895 .01908 L
.26786 .02071 L
.27842 .02326 L
.28836 .02639 L
.30696 .03458 L
.31671 .0403 L
.32702 .04758 L
.34548 .06408 L
.36509 .08681 L
.38342 .11294 L
.4223 .18212 L
.46363 .26844 L
.50344 .35238 L
.54174 .42444 L
.58249 .486 L
.60271 .51024 L
.62172 .52936 L
.65944 .55802 L
.6796 .56906 L
.69809 .57707 L
.71699 .58352 L
.73768 .58894 L
.75727 .5928 L
.77821 .59588 L
.79953 .59815 L
.81968 .59971 L
.83967 .60083 L
.85811 .60158 L
Mistroke
.87685 .60214 L
.88763 .60238 L
.89748 .60257 L
.91691 .60286 L
.92762 .60299 L
.93779 .60308 L
.94739 .60316 L
.95761 .60322 L
.976 .60332 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True,
  ImageSize->{180.938, 111.812},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003R0000Ra000`40O003h00OhWoo003ROol00>9o
o`00@goo0`0017oo0`0097oo0`001Goo0`0097oo0`001Goo0`008Goo1@000goo0`001Goo0`000goo
0014Ool00`00Oomoo`02Ool01@00Oomoogoo00008Woo00D007ooOomoo`0000=oo`05001oogooOol0
000ROol01@00Oomoogoo00000goo00D007ooOomoo`00029oo`03001oogoo009oo`05001oogooOol0
0003Ool01@00Oomoogoo00000Woo0011Ool50003Ool01@00Oomoogoo00008Woo00D007ooOomoo`00
00=oo`05001oogooOol0000ROol01@00Oomoogoo00000goo00D007ooOomoo`00029oo`03001oogoo
009oo`05001oogooOol00003Ool01@00Oomoogoo00000Woo0011Ool01000Oomoo`0017oo00D007oo
Oomoo`00029oo`05001oogooOol00003Ool01@00Oomoogoo00008Woo00D007ooOomoo`0000=oo`05
001oogooOol0000ROol00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00009oo`00
@Woo00<007oo000017oo00D007ooOomoo`00029oo`@000Aoo`05001oogooOol0000SOol30004Ool0
1@00Oomoogoo00008Woo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool0
049oo`03001oo`0000Aoo`05001oogooOol0000ROol00`00Oomoo`05Ool01@00Oomoogoo00008Woo
00D007ooOomoo`0000=oo`05001oogooOol0000ROol00`00Oomoo`02Ool01@00Oomoogoo00000goo
00D007ooOomoo`00009oo`00@goo0P0017oo00D007ooOomoo`0002=oo`03001oogoo00Aoo`05001o
ogooOol0000ROol01@00Oomoogoo00000goo00D007ooOomoo`00029oo`03001oogoo009oo`05001o
ogooOol00003Ool01@00Oomoogoo00000Woo0014Ool00`00Oomoo`03Ool3000UOol30004Ool3000T
Ool30005Ool3000QOol30005Ool30005Ool30003Ool00>9oo`0067oo00<007ooOol0agoo000HOol0
0`00Oomoo`37Ool001Qoo`03001oogoo0<Moo`0057ooa`001goo000HOol00`00Oomoo`09Ool00`00
Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`08Ool60007Ool00`00Oomoo`08Ool00`00Oomoo`09
Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`09
Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`0:
Ool001Qoo`03001oogoo039oo`8009=oo`0067oo00<007ooOol0=7oo0P00TGoo000HOol00`00Oomo
o`0fOol2002?Ool001Qoo`03001oogoo03Qoo`8008eoo`0067oo0P00>goo00<007ooOol0RWoo000H
Ool00`00Oomoo`0jOol00`00Oomoo`2:Ool001Qoo`03001oogoo03]oo`03001oogoo08Uoo`0067oo
00<007ooOol0?7oo00<007ooOol0R7oo000HOol00`00Oomoo`0mOol00`00Oomoo`27Ool001Qoo`80
03moo`03001oogoo08Ioo`0067oo00<007ooOol0?goo00<007ooOol0QGoo000HOol00`00Oomoo`10
Ool00`00Oomoo`24Ool001Qoo`03001oogoo045oo`03001oogoo08=oo`0067oo00<007ooOol0@Goo
00<007ooOol0Pgoo000HOol00`00Oomoo`12Ool00`00Oomoo`22Ool001Qoo`8004Aoo`03001oogoo
085oo`0067oo00<007ooOol0A7oo00<007ooOol0P7oo000HOol00`00Oomoo`15Ool00`00Oomoo`1o
Ool001Qoo`03001oogoo04Eoo`03001oogoo07moo`0000Aoo`00000000Ioo`8000Aoo`D000=oo`03
001oogoo04Ioo`03001oogoo07ioo`0000D007ooOomoo`0000Eoo`8000Eoo`04001oogoo0003Ool0
0`00Oomoo`16Ool00`00Oomoo`1nOol00005001oogooOol0000=Ool00`00Oomoo`03Ool30017Ool0
0`00Oomoo`1mOol00005001oogooOol0000>Ool00`00Oomoo`02Ool00`00Oomoo`18Ool00`00Oomo
o`1lOol00005001oogooOol0000?Ool01@00Oomoogoo0000BWoo00<007ooOol0O7oo00001@00Oomo
ogoo00003goo00D007ooOomoo`0004]oo`03001oogoo07]oo`0000D007ooOomoo`0000]oo`05001o
ogooOol00003Ool00`00Oomoo`19Ool00`00Oomoo`1kOol00004Ool00000000=Ool30004Ool00`00
Oomoo`1:Ool00`00Oomoo`1jOol001Qoo`8004]oo`03001oogoo07Yoo`0067oo00<007ooOol0Bgoo
00<007ooOol0NGoo000HOol00`00Oomoo`1<Ool00`00Oomoo`1hOol001Qoo`03001oogoo04aoo`03
001oogoo07Qoo`0067oo00<007ooOol0CGoo00<007ooOol0Mgoo000HOol00`00Oomoo`1=Ool00`00
Oomoo`1gOol001Qoo`8004moo`03001oogoo07Ioo`0067oo00<007ooOol0CWoo00<007ooOol0MWoo
000HOol00`00Oomoo`1?Ool00`00Oomoo`1eOol001Qoo`03001oogoo04moo`03001oogoo07Eoo`00
67oo00<007ooOol0D7oo00<007ooOol0M7oo000HOol00`00Oomoo`1@Ool00`00Oomoo`1dOol001Qo
o`80059oo`03001oogoo07=oo`0067oo00<007ooOol0DGoo00<007ooOol0Lgoo000HOol00`00Oomo
o`1AOol00`00Oomoo`1cOol00004Ool000000006Ool20006Ool30003Ool00`00Oomoo`1BOol00`00
Oomoo`1bOol00005001oogooOol00005Ool20007Ool00`00Oomoo`02Ool00`00Oomoo`1BOol00`00
Oomoo`1bOol00005001oogooOol0000;Ool50003Ool3001COol00`00Oomoo`1aOol00005001oogoo
Ool0000;Ool01000Oomoo`0017oo00<007ooOol0Dgoo00<007ooOol0LGoo00001@00Oomoogoo0000
37oo00<007oo000017oo00<007ooOol0E7oo00<007ooOol0L7oo00001@00Oomoogoo000037oo00<0
07oo000017oo00<007ooOol0E7oo00<007ooOol0L7oo00001@00Oomoogoo00003Goo0P0017oo00<0
07ooOol0EGoo00<007ooOol0Kgoo000017oo000000003goo00<007ooOol00Woo00<007ooOol0EGoo
00<007ooOol0Kgoo000HOol2001GOol00`00Oomoo`1^Ool001Qoo`03001oogoo05Ioo`03001oogoo
06ioo`0067oo00<007ooOol0Egoo00<007ooOol0KGoo000HOol00`00Oomoo`1GOol00`00Oomoo`1]
Ool001Qoo`03001oogoo05Qoo`03001oogoo06aoo`0067oo00<007ooOol0F7oo00<007ooOol0K7oo
000HOol2001JOol00`00Oomoo`1[Ool001Qoo`03001oogoo05Uoo`03001oogoo06]oo`0067oo00<0
07ooOol0FGoo00<007ooOol0Jgoo000HOol00`00Oomoo`1JOol00`00Oomoo`1ZOol001Qoo`03001o
ogoo05Yoo`03001oogoo06Yoo`0067oo00<007ooOol0Fgoo00<007ooOol0JGoo000HOol2001LOol0
0`00Oomoo`1YOol001Qoo`03001oogoo05aoo`03001oogoo06Qoo`0067oo00<007ooOol0G7oo00<0
07ooOol0J7oo000017oo000000001Woo0P001Goo0`0017oo00<007ooOol0GGoo00<007ooOol0Igoo
00001@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`0000=oo`03001oogoo05eoo`03001oogoo
06Moo`0000D007ooOomoo`0000]oo`05001oogooOol00003Ool00`00Oomoo`1NOol00`00Oomoo`1V
Ool00005001oogooOol0000;Ool01@00Oomoogoo00000goo0`00GWoo00<007ooOol0IWoo00001@00
Oomoogoo00002goo100017oo00<007ooOol0Ggoo00<007ooOol0IGoo00001@00Oomoogoo00002goo
00<007ooOol01Goo00<007ooOol0Ggoo00<007ooOol0IGoo00001@00Oomoogoo000037oo00<007oo
Ool017oo00<007ooOol0H7oo00<007ooOol0I7oo000017oo000000003Woo0`000goo00<007ooOol0
H7oo00<007ooOol0I7oo000HOol00`00Oomoo`1QOol00`00Oomoo`1SOol001Qoo`80069oo`03001o
ogoo06=oo`0067oo00<007ooOol0HWoo00<007ooOol0HWoo000HOol00`00Oomoo`1ROol00`00Oomo
o`1ROol001Qoo`03001oogoo06=oo`03001oogoo065oo`0067oo00<007ooOol0Hgoo00<007ooOol0
HGoo000HOol00`00Oomoo`1TOol00`00Oomoo`1POol001Qoo`8006Eoo`03001oogoo061oo`0067oo
00<007ooOol0IGoo00<007ooOol0Ggoo000HOol00`00Oomoo`1VOol00`00Oomoo`1NOol001Qoo`03
001oogoo06Ioo`03001oogoo05ioo`0067oo00<007ooOol0Igoo00<007ooOol0GGoo000HOol2001Y
Ool00`00Oomoo`1LOol001Qoo`03001oogoo06Qoo`03001oogoo05aoo`0067oo00<007ooOol0JGoo
00<007ooOol0Fgoo000HOol00`00Oomoo`1ZOol00`00Oomoo`1JOol00004Ool000000006Ool20005
Ool30004Ool00`00Oomoo`1ZOol00`00Oomoo`1JOol00005001oogooOol00005Ool20004Ool01@00
Oomoogoo00000goo00<007ooOol0Jgoo00<007ooOol0FGoo00001@00Oomoogoo00002goo00D007oo
Oomoo`0000=oo`<006aoo`03001oogoo05Qoo`0000D007ooOomoo`0000]oo`05001oogooOol00003
Ool00`00Oomoo`1/Ool00`00Oomoo`1HOol00005001oogooOol0000<Ool30004Ool00`00Oomoo`1]
Ool00`00Oomoo`1GOol00005001oogooOol0000;Ool01@00Oomoogoo00000goo00<007ooOol0KWoo
00<007ooOol0EWoo00001@00Oomoogoo00002goo00D007ooOomoo`0000=oo`03001oogoo06moo`03
001oogoo05Eoo`0000Aoo`00000000eoo`<000Aoo`03001oogoo071oo`03001oogoo05Aoo`0067oo
0P00LWoo00<007ooOol0Dgoo000HOol00`00Oomoo`1bOol00`00Oomoo`1BOol001Qoo`03001oogoo
07=oo`03001oogoo055oo`0067oo00<007ooOol0M7oo0P00DGoo000HOol00`00Oomoo`1fOol00`00
Oomoo`1>Ool001Qoo`03001oogoo07Moo`03001oogoo04eoo`0067oo0P00NGoo0P00CGoo000HOol0
0`00Oomoo`1jOol00`00Oomoo`1:Ool001Qoo`03001oogoo07]oo`03001oogoo04Uoo`0067oo00<0
07ooOol0O7oo0P00BGoo000HOol00`00Oomoo`1nOol20017Ool001Qoo`03001oogoo081oo`<004Ao
o`0067oo0P00Q7oo0P00@Woo000HOol00`00Oomoo`25Ool20010Ool001Qoo`03001oogoo08Moo`<0
03eoo`0047oo1@000goo00<007ooOol0RWoo2000=Goo000BOol00`00Oomoo`03Ool00`00Oomoo`2B
Ool8000]Ool0019oo`03001oogoo00=oo`03001oogoo09Yooad0011oo`004Woo00<007ooOol00goo
0`00]goo100037oo000BOol00`00Oomoo`03Ool00`00Oomoo`37Ool0019oo`03001oogoo00=oo`03
001oogoo0<Moo`004Woo00<007ooOol0cGoo000@Ool3003?Ool00>9oo`00hWoo0000\
\>"],
  ImageRangeCache->{{{0, 225.125}, {138.75, 0}} -> {9.42048, -0.128724, 
  0.529396, 0.0107033}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[BlackScholesCallDelta[p, 60, 0.29, 0.04, 0, t],{p, 50, \
70},{t, 0.001, 1}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
6.10623e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.7672 .09664 0 -6.16225 ]
[.7672 .09664 18 2.83775 ]
[.82794 .18966 0 -6.07272 ]
[.82794 .18966 18 2.92728 ]
[.88358 .27486 0 -5.99233 ]
[.88358 .27486 18 3.00767 ]
[.93473 .3532 0 -5.91977 ]
[.93473 .3532 18 3.08023 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 6 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01568 .31795 -24 -2.78699 ]
[.01568 .31795 0 6.21301 ]
[.00696 .37264 -18 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00208 .42927 -24 -2.88161 ]
[-0.00208 .42927 0 6.11839 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.75489 .10119 m
.74873 .10346 L
s
[(0.2)] .7672 .09664 -1 .36939 Mshowa
.81555 .19399 m
.80936 .19615 L
s
[(0.4)] .82794 .18966 -1 .34949 Mshowa
.87113 .27899 m
.8649 .28106 L
s
[(0.6)] .88358 .27486 -1 .33163 Mshowa
.92222 .35714 m
.91596 .35912 L
s
[(0.8)] .93473 .3532 -1 .3155 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(1)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70561 .0258 m
.70193 .02722 L
s
.72242 .05152 m
.71874 .05292 L
s
.73884 .07664 m
.73515 .07802 L
s
.77057 .12518 m
.76687 .12652 L
s
.7859 .14863 m
.7822 .14996 L
s
.80089 .17156 m
.79718 .17287 L
s
.8299 .21593 m
.82618 .21721 L
s
.84394 .2374 m
.84021 .23867 L
s
.85767 .25842 m
.85394 .25967 L
s
.8843 .29914 m
.88056 .30036 L
s
.8972 .31887 m
.89345 .32008 L
s
.90983 .3382 m
.90608 .3394 L
s
.93435 .37571 m
.93059 .37688 L
s
.94625 .3939 m
.94249 .39506 L
s
.95791 .41174 m
.95414 .41289 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02795 .31328 m
.03408 .31095 L
s
[(0.25)] .01568 .31795 1 -0.38067 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0.5)] .00696 .37264 1 -0.37036 Mshowa
.01027 .42483 m
.01644 .42261 L
s
[(0.75)] -0.00208 .42927 1 -0.35964 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(1)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.03468 .27078 m
.03835 .26935 L
s
.03301 .2813 m
.03669 .27988 L
s
.03134 .29189 m
.03501 .29047 L
s
.02965 .30255 m
.03332 .30114 L
s
.02623 .32409 m
.02991 .3227 L
s
.02451 .33497 m
.02819 .33359 L
s
.02277 .34593 m
.02646 .34455 L
s
.02102 .35697 m
.02471 .35559 L
s
.01749 .37927 m
.02118 .37791 L
s
.0157 .39054 m
.0194 .38919 L
s
.0139 .40189 m
.0176 .40054 L
s
.01209 .41332 m
.01579 .41198 L
s
.00843 .43642 m
.01214 .4351 L
s
.00658 .4481 m
.01029 .44678 L
s
.00472 .45986 m
.00843 .45855 L
s
.00284 .4717 m
.00656 .4704 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.573 .699 .927 r
.39466 .66675 .40264 .67476 .41596 .67356 .40802 .66555 Metetra
.573 .7 .928 r
.40802 .66555 .41596 .67356 .42938 .67236 .42146 .66434 Metetra
.573 .7 .928 r
.42146 .66434 .42938 .67236 .44288 .67114 .435 .66312 Metetra
.573 .701 .929 r
.435 .66312 .44288 .67114 .45646 .66992 .44862 .6619 Metetra
.572 .702 .93 r
.44862 .6619 .45646 .66992 .47014 .66868 .46233 .66065 Metetra
.573 .703 .93 r
.46233 .66065 .47014 .66868 .48391 .66742 .47614 .65939 Metetra
.573 .704 .931 r
.47614 .65939 .48391 .66742 .49777 .66614 .49003 .65811 Metetra
.573 .704 .931 r
.49003 .65811 .49777 .66614 .51172 .66484 .50402 .65681 Metetra
.573 .705 .932 r
.50402 .65681 .51172 .66484 .52577 .6635 .51811 .65548 Metetra
.574 .707 .932 r
.51811 .65548 .52577 .6635 .53991 .66214 .53228 .65411 Metetra
.574 .708 .933 r
.53228 .65411 .53991 .66214 .55414 .66075 .54656 .65272 Metetra
.575 .709 .933 r
.54656 .65272 .55414 .66075 .56847 .65932 .56092 .65129 Metetra
.575 .71 .934 r
.56092 .65129 .56847 .65932 .58289 .65785 .57539 .64982 Metetra
.576 .711 .934 r
.57539 .64982 .58289 .65785 .59741 .65634 .58995 .64832 Metetra
.577 .712 .935 r
.58995 .64832 .59741 .65634 .61202 .65479 .60461 .64677 Metetra
.578 .714 .935 r
.60461 .64677 .61202 .65479 .62673 .65319 .61937 .64517 Metetra
.579 .715 .936 r
.61937 .64517 .62673 .65319 .64154 .65155 .63423 .64353 Metetra
.579 .716 .936 r
.63423 .64353 .64154 .65155 .65645 .64987 .64918 .64184 Metetra
.58 .718 .937 r
.64918 .64184 .65645 .64987 .67146 .64813 .66424 .6401 Metetra
.581 .719 .937 r
.66424 .6401 .67146 .64813 .68656 .64634 .67939 .63831 Metetra
.583 .72 .938 r
.67939 .63831 .68656 .64634 .70177 .64449 .69465 .63647 Metetra
.584 .722 .938 r
.69465 .63647 .70177 .64449 .71707 .6426 .71 .63457 Metetra
.585 .723 .938 r
.71 .63457 .71707 .6426 .73247 .64065 .72546 .63261 Metetra
.586 .724 .939 r
.72546 .63261 .73247 .64065 .74798 .63864 .74102 .6306 Metetra
.587 .726 .939 r
.74102 .6306 .74798 .63864 .76358 .63657 .75668 .62852 Metetra
.588 .727 .939 r
.75668 .62852 .76358 .63657 .77929 .63445 .77244 .62639 Metetra
.59 .728 .94 r
.77244 .62639 .77929 .63445 .7951 .63226 .78831 .62419 Metetra
.591 .73 .94 r
.78831 .62419 .7951 .63226 .811 .63002 .80428 .62194 Metetra
.592 .731 .94 r
.80428 .62194 .811 .63002 .82702 .62771 .82035 .61962 Metetra
.593 .732 .941 r
.82035 .61962 .82702 .62771 .84313 .62534 .83653 .61724 Metetra
.594 .733 .941 r
.83653 .61724 .84313 .62534 .85935 .62291 .85281 .61479 Metetra
.596 .735 .941 r
.85281 .61479 .85935 .62291 .87567 .62042 .8692 .61228 Metetra
.597 .736 .941 r
.8692 .61228 .87567 .62042 .8921 .61786 .88569 .60971 Metetra
.598 .737 .941 r
.88569 .60971 .8921 .61786 .90862 .61524 .90228 .60707 Metetra
.599 .738 .942 r
.90228 .60707 .90862 .61524 .92526 .61255 .91898 .60437 Metetra
.601 .739 .942 r
.91898 .60437 .92526 .61255 .942 .60981 .93579 .6016 Metetra
.602 .74 .942 r
.93579 .6016 .942 .60981 .95884 .60699 .95271 .59876 Metetra
.603 .741 .942 r
.95271 .59876 .95884 .60699 .97579 .60412 .96973 .59586 Metetra
.604 .742 .942 r
.96973 .59586 .97579 .60412 .99285 .60118 .98686 .59289 Metetra
.573 .698 .927 r
.38662 .65866 .39466 .66675 .40802 .66555 .40001 .65744 Metetra
.572 .699 .928 r
.40001 .65744 .40802 .66555 .42146 .66434 .41349 .65623 Metetra
.572 .7 .928 r
.41349 .65623 .42146 .66434 .435 .66312 .42705 .65501 Metetra
.572 .7 .929 r
.42705 .65501 .435 .66312 .44862 .6619 .44071 .65378 Metetra
.572 .701 .929 r
.44071 .65378 .44862 .6619 .46233 .66065 .45445 .65253 Metetra
.572 .702 .93 r
.45445 .65253 .46233 .66065 .47614 .65939 .46829 .65127 Metetra
.572 .703 .93 r
.46829 .65127 .47614 .65939 .49003 .65811 .48223 .64999 Metetra
.572 .704 .931 r
.48223 .64999 .49003 .65811 .50402 .65681 .49625 .64869 Metetra
.572 .705 .932 r
.49625 .64869 .50402 .65681 .51811 .65548 .51037 .64736 Metetra
.573 .706 .932 r
.51037 .64736 .51811 .65548 .53228 .65411 .52459 .646 Metetra
.573 .707 .933 r
.52459 .646 .53228 .65411 .54656 .65272 .5389 .6446 Metetra
.574 .708 .933 r
.5389 .6446 .54656 .65272 .56092 .65129 .55331 .64318 Metetra
.574 .709 .934 r
.55331 .64318 .56092 .65129 .57539 .64982 .56782 .64171 Metetra
.575 .71 .934 r
.56782 .64171 .57539 .64982 .58995 .64832 .58243 .64021 Metetra
.576 .712 .935 r
.58243 .64021 .58995 .64832 .60461 .64677 .59713 .63866 Metetra
.576 .713 .935 r
.59713 .63866 .60461 .64677 .61937 .64517 .61193 .63707 Metetra
.577 .714 .936 r
.61193 .63707 .61937 .64517 .63423 .64353 .62684 .63543 Metetra
.578 .716 .936 r
.62684 .63543 .63423 .64353 .64918 .64184 .64184 .63374 Metetra
.579 .717 .937 r
.64184 .63374 .64918 .64184 .66424 .6401 .65694 .632 Metetra
.58 .718 .937 r
.65694 .632 .66424 .6401 .67939 .63831 .67215 .6302 Metetra
.581 .72 .938 r
.67215 .6302 .67939 .63831 .69465 .63647 .68746 .62836 Metetra
.583 .721 .938 r
.68746 .62836 .69465 .63647 .71 .63457 .70287 .62645 Metetra
.584 .722 .939 r
.70287 .62645 .71 .63457 .72546 .63261 .71838 .62449 Metetra
.585 .724 .939 r
.71838 .62449 .72546 .63261 .74102 .6306 .73399 .62247 Metetra
.586 .725 .939 r
.73399 .62247 .74102 .6306 .75668 .62852 .74971 .62039 Metetra
.587 .727 .94 r
.74971 .62039 .75668 .62852 .77244 .62639 .76553 .61825 Metetra
.589 .728 .94 r
.76553 .61825 .77244 .62639 .78831 .62419 .78146 .61605 Metetra
.59 .729 .94 r
.78146 .61605 .78831 .62419 .80428 .62194 .79749 .61379 Metetra
.591 .731 .941 r
.79749 .61379 .80428 .62194 .82035 .61962 .81362 .61146 Metetra
.592 .732 .941 r
.81362 .61146 .82035 .61962 .83653 .61724 .82986 .60906 Metetra
.594 .733 .941 r
.82986 .60906 .83653 .61724 .85281 .61479 .8462 .6066 Metetra
.595 .734 .941 r
.8462 .6066 .85281 .61479 .8692 .61228 .86266 .60408 Metetra
.596 .736 .942 r
.86266 .60408 .8692 .61228 .88569 .60971 .87921 .60149 Metetra
.597 .737 .942 r
.87921 .60149 .88569 .60971 .90228 .60707 .89588 .59883 Metetra
.599 .738 .942 r
.89588 .59883 .90228 .60707 .91898 .60437 .91265 .59611 Metetra
.6 .739 .942 r
.91265 .59611 .91898 .60437 .93579 .6016 .92952 .59331 Metetra
.601 .74 .942 r
.92952 .59331 .93579 .6016 .95271 .59876 .94651 .59046 Metetra
.602 .742 .942 r
.94651 .59046 .95271 .59876 .96973 .59586 .9636 .58753 Metetra
.604 .743 .943 r
.9636 .58753 .96973 .59586 .98686 .59289 .98081 .58454 Metetra
.572 .698 .927 r
.37851 .65046 .38662 .65866 .40001 .65744 .39193 .64924 Metetra
.571 .698 .927 r
.39193 .64924 .40001 .65744 .41349 .65623 .40544 .64802 Metetra
.571 .699 .928 r
.40544 .64802 .41349 .65623 .42705 .65501 .41903 .6468 Metetra
.571 .699 .928 r
.41903 .6468 .42705 .65501 .44071 .65378 .43272 .64557 Metetra
.571 .7 .929 r
.43272 .64557 .44071 .65378 .45445 .65253 .4465 .64432 Metetra
.571 .701 .93 r
.4465 .64432 .45445 .65253 .46829 .65127 .46038 .64306 Metetra
.571 .702 .93 r
.46038 .64306 .46829 .65127 .48223 .64999 .47435 .64178 Metetra
.571 .703 .931 r
.47435 .64178 .48223 .64999 .49625 .64869 .48841 .64047 Metetra
.571 .704 .932 r
.48841 .64047 .49625 .64869 .51037 .64736 .50257 .63914 Metetra
.571 .705 .932 r
.50257 .63914 .51037 .64736 .52459 .646 .51683 .63779 Metetra
.572 .706 .933 r
.51683 .63779 .52459 .646 .5389 .6446 .53118 .6364 Metetra
.572 .707 .933 r
.53118 .6364 .5389 .6446 .55331 .64318 .54563 .63497 Metetra
.573 .708 .934 r
.54563 .63497 .55331 .64318 .56782 .64171 .56018 .63351 Metetra
.574 .71 .934 r
.56018 .63351 .56782 .64171 .58243 .64021 .57483 .63201 Metetra
.574 .711 .935 r
.57483 .63201 .58243 .64021 .59713 .63866 .58958 .63046 Metetra
.575 .712 .936 r
.58958 .63046 .59713 .63866 .61193 .63707 .60443 .62887 Metetra
.576 .714 .936 r
.60443 .62887 .61193 .63707 .62684 .63543 .61938 .62723 Metetra
.577 .715 .937 r
.61938 .62723 .62684 .63543 .64184 .63374 .63443 .62555 Metetra
.578 .716 .937 r
.63443 .62555 .64184 .63374 .65694 .632 .64958 .62381 Metetra
.579 .718 .938 r
.64958 .62381 .65694 .632 .67215 .6302 .66484 .62201 Metetra
.58 .719 .938 r
.66484 .62201 .67215 .6302 .68746 .62836 .6802 .62017 Metetra
.581 .721 .938 r
.6802 .62017 .68746 .62836 .70287 .62645 .69566 .61826 Metetra
.583 .722 .939 r
.69566 .61826 .70287 .62645 .71838 .62449 .71123 .6163 Metetra
.584 .723 .939 r
.71123 .6163 .71838 .62449 .73399 .62247 .7269 .61427 Metetra
.585 .725 .94 r
.7269 .61427 .73399 .62247 .74971 .62039 .74267 .61219 Metetra
.586 .726 .94 r
.74267 .61219 .74971 .62039 .76553 .61825 .75855 .61004 Metetra
.587 .728 .94 r
.75855 .61004 .76553 .61825 .78146 .61605 .77453 .60783 Metetra
.589 .729 .941 r
.77453 .60783 .78146 .61605 .79749 .61379 .79062 .60556 Metetra
.59 .73 .941 r
.79062 .60556 .79749 .61379 .81362 .61146 .80682 .60322 Metetra
.591 .732 .941 r
.80682 .60322 .81362 .61146 .82986 .60906 .82312 .60081 Metetra
.593 .733 .942 r
.82312 .60081 .82986 .60906 .8462 .6066 .83953 .59834 Metetra
.594 .734 .942 r
.83953 .59834 .8462 .6066 .86266 .60408 .85605 .5958 Metetra
.595 .736 .942 r
.85605 .5958 .86266 .60408 .87921 .60149 .87267 .59319 Metetra
.597 .737 .942 r
.87267 .59319 .87921 .60149 .89588 .59883 .8894 .59051 Metetra
.598 .738 .942 r
.8894 .59051 .89588 .59883 .91265 .59611 .90624 .58777 Metetra
.599 .739 .943 r
.90624 .58777 .91265 .59611 .92952 .59331 .92319 .58496 Metetra
.6 .74 .943 r
.92319 .58496 .92952 .59331 .94651 .59046 .94025 .58208 Metetra
.602 .742 .943 r
.94025 .58208 .94651 .59046 .9636 .58753 .95742 .57913 Metetra
.603 .743 .943 r
.95742 .57913 .9636 .58753 .98081 .58454 .97469 .57611 Metetra
.571 .697 .926 r
.37033 .64217 .37851 .65046 .39193 .64924 .38378 .64094 Metetra
.571 .697 .927 r
.38378 .64094 .39193 .64924 .40544 .64802 .39732 .63972 Metetra
.57 .698 .928 r
.39732 .63972 .40544 .64802 .41903 .6468 .41095 .63849 Metetra
.57 .698 .928 r
.41095 .63849 .41903 .6468 .43272 .64557 .42467 .63725 Metetra
.57 .699 .929 r
.42467 .63725 .43272 .64557 .4465 .64432 .43848 .63601 Metetra
.57 .7 .929 r
.43848 .63601 .4465 .64432 .46038 .64306 .45239 .63474 Metetra
.57 .701 .93 r
.45239 .63474 .46038 .64306 .47435 .64178 .4664 .63346 Metetra
.57 .702 .931 r
.4664 .63346 .47435 .64178 .48841 .64047 .4805 .63216 Metetra
.57 .703 .931 r
.4805 .63216 .48841 .64047 .50257 .63914 .49469 .63084 Metetra
.57 .704 .932 r
.49469 .63084 .50257 .63914 .51683 .63779 .50899 .62948 Metetra
.571 .705 .933 r
.50899 .62948 .51683 .63779 .53118 .6364 .52338 .6281 Metetra
.571 .706 .933 r
.52338 .6281 .53118 .6364 .54563 .63497 .53787 .62667 Metetra
.572 .707 .934 r
.53787 .62667 .54563 .63497 .56018 .63351 .55247 .62522 Metetra
.572 .709 .934 r
.55247 .62522 .56018 .63351 .57483 .63201 .56716 .62372 Metetra
.573 .71 .935 r
.56716 .62372 .57483 .63201 .58958 .63046 .58195 .62218 Metetra
.574 .711 .936 r
.58195 .62218 .58958 .63046 .60443 .62887 .59685 .62059 Metetra
.575 .713 .936 r
.59685 .62059 .60443 .62887 .61938 .62723 .61184 .61895 Metetra
.576 .714 .937 r
.61184 .61895 .61938 .62723 .63443 .62555 .62694 .61727 Metetra
.577 .716 .937 r
.62694 .61727 .63443 .62555 .64958 .62381 .64215 .61553 Metetra
.578 .717 .938 r
.64215 .61553 .64958 .62381 .66484 .62201 .65745 .61374 Metetra
.579 .719 .938 r
.65745 .61374 .66484 .62201 .6802 .62017 .67287 .61189 Metetra
.58 .72 .939 r
.67287 .61189 .6802 .62017 .69566 .61826 .68838 .60998 Metetra
.581 .721 .939 r
.68838 .60998 .69566 .61826 .71123 .6163 .704 .60802 Metetra
.583 .723 .94 r
.704 .60802 .71123 .6163 .7269 .61427 .71973 .60599 Metetra
.584 .724 .94 r
.71973 .60599 .7269 .61427 .74267 .61219 .73556 .6039 Metetra
.585 .726 .94 r
.73556 .6039 .74267 .61219 .75855 .61004 .7515 .60175 Metetra
.586 .727 .941 r
.7515 .60175 .75855 .61004 .77453 .60783 .76754 .59953 Metetra
.588 .729 .941 r
.76754 .59953 .77453 .60783 .79062 .60556 .78369 .59725 Metetra
.589 .73 .941 r
.78369 .59725 .79062 .60556 .80682 .60322 .79995 .5949 Metetra
.59 .731 .942 r
.79995 .5949 .80682 .60322 .82312 .60081 .81632 .59248 Metetra
.592 .733 .942 r
.81632 .59248 .82312 .60081 .83953 .59834 .83279 .59 Metetra
.593 .734 .942 r
.83279 .59 .83953 .59834 .85605 .5958 .84937 .58744 Metetra
.594 .736 .942 r
.84937 .58744 .85605 .5958 .87267 .59319 .86606 .58482 Metetra
.596 .737 .943 r
.86606 .58482 .87267 .59319 .8894 .59051 .88286 .58212 Metetra
.597 .738 .943 r
.88286 .58212 .8894 .59051 .90624 .58777 .89977 .57936 Metetra
.598 .739 .943 r
.89977 .57936 .90624 .58777 .92319 .58496 .91679 .57653 Metetra
.6 .74 .943 r
.91679 .57653 .92319 .58496 .94025 .58208 .93392 .57362 Metetra
.601 .742 .943 r
.93392 .57362 .94025 .58208 .95742 .57913 .95116 .57065 Metetra
.602 .743 .943 r
.95116 .57065 .95742 .57913 .97469 .57611 .96851 .5676 Metetra
.571 .696 .926 r
.36207 .63377 .37033 .64217 .38378 .64094 .37555 .63254 Metetra
.57 .696 .927 r
.37555 .63254 .38378 .64094 .39732 .63972 .38912 .63131 Metetra
.569 .697 .927 r
.38912 .63131 .39732 .63972 .41095 .63849 .40279 .63008 Metetra
.569 .697 .928 r
.40279 .63008 .41095 .63849 .42467 .63725 .41654 .62884 Metetra
.569 .698 .929 r
.41654 .62884 .42467 .63725 .43848 .63601 .43039 .62759 Metetra
.568 .699 .929 r
.43039 .62759 .43848 .63601 .45239 .63474 .44433 .62633 Metetra
.568 .7 .93 r
.44433 .62633 .45239 .63474 .4664 .63346 .45837 .62505 Metetra
.569 .701 .931 r
.45837 .62505 .4664 .63346 .4805 .63216 .47251 .62375 Metetra
.569 .702 .931 r
.47251 .62375 .4805 .63216 .49469 .63084 .48675 .62243 Metetra
.569 .703 .932 r
.48675 .62243 .49469 .63084 .50899 .62948 .50108 .62108 Metetra
.569 .704 .933 r
.50108 .62108 .50899 .62948 .52338 .6281 .51551 .6197 Metetra
.57 .705 .933 r
.51551 .6197 .52338 .6281 .53787 .62667 .53005 .61828 Metetra
.57 .706 .934 r
.53005 .61828 .53787 .62667 .55247 .62522 .54468 .61683 Metetra
.571 .708 .934 r
.54468 .61683 .55247 .62522 .56716 .62372 .55942 .61533 Metetra
.572 .709 .935 r
.55942 .61533 .56716 .62372 .58195 .62218 .57425 .6138 Metetra
.573 .71 .936 r
.57425 .6138 .58195 .62218 .59685 .62059 .58919 .61222 Metetra
.574 .712 .936 r
.58919 .61222 .59685 .62059 .61184 .61895 .60424 .61058 Metetra
.574 .713 .937 r
.60424 .61058 .61184 .61895 .62694 .61727 .61939 .6089 Metetra
.576 .715 .937 r
.61939 .6089 .62694 .61727 .64215 .61553 .63464 .60717 Metetra
.577 .716 .938 r
.63464 .60717 .64215 .61553 .65745 .61374 .65 .60538 Metetra
.578 .718 .938 r
.65 .60538 .65745 .61374 .67287 .61189 .66546 .60353 Metetra
.579 .719 .939 r
.66546 .60353 .67287 .61189 .68838 .60998 .68103 .60163 Metetra
.58 .721 .939 r
.68103 .60163 .68838 .60998 .704 .60802 .6967 .59966 Metetra
.581 .722 .94 r
.6967 .59966 .704 .60802 .71973 .60599 .71249 .59763 Metetra
.583 .724 .94 r
.71249 .59763 .71973 .60599 .73556 .6039 .72838 .59554 Metetra
.584 .725 .941 r
.72838 .59554 .73556 .6039 .7515 .60175 .74437 .59338 Metetra
.585 .727 .941 r
.74437 .59338 .7515 .60175 .76754 .59953 .76048 .59116 Metetra
.587 .728 .941 r
.76048 .59116 .76754 .59953 .78369 .59725 .77669 .58886 Metetra
.588 .73 .942 r
.77669 .58886 .78369 .59725 .79995 .5949 .79301 .5865 Metetra
.589 .731 .942 r
.79301 .5865 .79995 .5949 .81632 .59248 .80944 .58408 Metetra
.591 .733 .942 r
.80944 .58408 .81632 .59248 .83279 .59 .82598 .58158 Metetra
.592 .734 .943 r
.82598 .58158 .83279 .59 .84937 .58744 .84263 .57901 Metetra
.593 .735 .943 r
.84263 .57901 .84937 .58744 .86606 .58482 .85939 .57637 Metetra
.595 .737 .943 r
.85939 .57637 .86606 .58482 .88286 .58212 .87626 .57366 Metetra
.596 .738 .943 r
.87626 .57366 .88286 .58212 .89977 .57936 .89324 .57087 Metetra
.598 .739 .943 r
.89324 .57087 .89977 .57936 .91679 .57653 .91033 .56802 Metetra
.599 .741 .943 r
.91033 .56802 .91679 .57653 .93392 .57362 .92753 .56509 Metetra
.6 .742 .944 r
.92753 .56509 .93392 .57362 .95116 .57065 .94484 .56209 Metetra
.602 .743 .944 r
.94484 .56209 .95116 .57065 .96851 .5676 .96227 .55902 Metetra
.57 .695 .925 r
.35375 .62527 .36207 .63377 .37555 .63254 .36726 .62403 Metetra
.569 .695 .926 r
.36726 .62403 .37555 .63254 .38912 .63131 .38086 .62279 Metetra
.568 .695 .927 r
.38086 .62279 .38912 .63131 .40279 .63008 .39455 .62156 Metetra
.568 .696 .928 r
.39455 .62156 .40279 .63008 .41654 .62884 .40834 .62032 Metetra
.568 .697 .928 r
.40834 .62032 .41654 .62884 .43039 .62759 .42222 .61907 Metetra
.567 .698 .929 r
.42222 .61907 .43039 .62759 .44433 .62633 .4362 .61781 Metetra
.567 .698 .93 r
.4362 .61781 .44433 .62633 .45837 .62505 .45028 .61654 Metetra
.567 .699 .93 r
.45028 .61654 .45837 .62505 .47251 .62375 .46445 .61524 Metetra
.567 .701 .931 r
.46445 .61524 .47251 .62375 .48675 .62243 .47872 .61392 Metetra
.568 .702 .932 r
.47872 .61392 .48675 .62243 .50108 .62108 .4931 .61258 Metetra
.568 .703 .932 r
.4931 .61258 .50108 .62108 .51551 .6197 .50757 .6112 Metetra
.569 .704 .933 r
.50757 .6112 .51551 .6197 .53005 .61828 .52214 .60979 Metetra
.569 .705 .934 r
.52214 .60979 .53005 .61828 .54468 .61683 .53682 .60834 Metetra
.57 .707 .934 r
.53682 .60834 .54468 .61683 .55942 .61533 .5516 .60686 Metetra
.57 .708 .935 r
.5516 .60686 .55942 .61533 .57425 .6138 .56648 .60533 Metetra
.571 .71 .936 r
.56648 .60533 .57425 .6138 .58919 .61222 .58147 .60375 Metetra
.572 .711 .936 r
.58147 .60375 .58919 .61222 .60424 .61058 .59656 .60212 Metetra
.573 .713 .937 r
.59656 .60212 .60424 .61058 .61939 .6089 .61175 .60045 Metetra
.574 .714 .937 r
.61175 .60045 .61939 .6089 .63464 .60717 .62706 .59872 Metetra
.575 .716 .938 r
.62706 .59872 .63464 .60717 .65 .60538 .64247 .59693 Metetra
.576 .717 .939 r
.64247 .59693 .65 .60538 .66546 .60353 .65798 .59509 Metetra
.578 .719 .939 r
.65798 .59509 .66546 .60353 .68103 .60163 .6736 .59318 Metetra
.579 .72 .94 r
.6736 .59318 .68103 .60163 .6967 .59966 .68933 .59122 Metetra
.58 .722 .94 r
.68933 .59122 .6967 .59966 .71249 .59763 .70517 .58919 Metetra
.581 .723 .94 r
.70517 .58919 .71249 .59763 .72838 .59554 .72112 .58709 Metetra
.583 .725 .941 r
.72112 .58709 .72838 .59554 .74437 .59338 .73717 .58493 Metetra
.584 .727 .941 r
.73717 .58493 .74437 .59338 .76048 .59116 .75334 .5827 Metetra
.585 .728 .942 r
.75334 .5827 .76048 .59116 .77669 .58886 .76961 .5804 Metetra
.587 .73 .942 r
.76961 .5804 .77669 .58886 .79301 .5865 .786 .57803 Metetra
.588 .731 .942 r
.786 .57803 .79301 .5865 .80944 .58408 .80249 .57559 Metetra
.59 .733 .943 r
.80249 .57559 .80944 .58408 .82598 .58158 .8191 .57308 Metetra
.591 .734 .943 r
.8191 .57308 .82598 .58158 .84263 .57901 .83581 .5705 Metetra
.592 .735 .943 r
.83581 .5705 .84263 .57901 .85939 .57637 .85264 .56784 Metetra
.594 .737 .943 r
.85264 .56784 .85939 .57637 .87626 .57366 .86958 .56512 Metetra
.595 .738 .944 r
.86958 .56512 .87626 .57366 .89324 .57087 .88663 .56231 Metetra
.597 .739 .944 r
.88663 .56231 .89324 .57087 .91033 .56802 .90379 .55944 Metetra
.598 .741 .944 r
.90379 .55944 .91033 .56802 .92753 .56509 .92107 .55649 Metetra
.6 .742 .944 r
.92107 .55649 .92753 .56509 .94484 .56209 .93846 .55346 Metetra
.601 .743 .944 r
.93846 .55346 .94484 .56209 .96227 .55902 .95596 .55036 Metetra
.569 .693 .925 r
.34535 .61666 .35375 .62527 .36726 .62403 .35889 .61541 Metetra
.568 .694 .926 r
.35889 .61541 .36726 .62403 .38086 .62279 .37252 .61417 Metetra
.568 .694 .926 r
.37252 .61417 .38086 .62279 .39455 .62156 .38625 .61293 Metetra
.567 .695 .927 r
.38625 .61293 .39455 .62156 .40834 .62032 .40007 .61169 Metetra
.567 .696 .928 r
.40007 .61169 .40834 .62032 .42222 .61907 .41399 .61044 Metetra
.566 .696 .929 r
.41399 .61044 .42222 .61907 .4362 .61781 .428 .60919 Metetra
.566 .697 .929 r
.428 .60919 .4362 .61781 .45028 .61654 .44211 .60792 Metetra
.566 .698 .93 r
.44211 .60792 .45028 .61654 .46445 .61524 .45632 .60662 Metetra
.566 .699 .931 r
.45632 .60662 .46445 .61524 .47872 .61392 .47063 .60531 Metetra
.566 .701 .932 r
.47063 .60531 .47872 .61392 .4931 .61258 .48504 .60397 Metetra
.567 .702 .932 r
.48504 .60397 .4931 .61258 .50757 .6112 .49955 .6026 Metetra
.567 .703 .933 r
.49955 .6026 .50757 .6112 .52214 .60979 .51417 .6012 Metetra
.568 .704 .934 r
.51417 .6012 .52214 .60979 .53682 .60834 .52888 .59976 Metetra
.568 .706 .934 r
.52888 .59976 .53682 .60834 .5516 .60686 .54371 .59828 Metetra
.569 .707 .935 r
.54371 .59828 .5516 .60686 .56648 .60533 .55863 .59676 Metetra
.57 .709 .936 r
.55863 .59676 .56648 .60533 .58147 .60375 .57366 .59519 Metetra
.571 .71 .936 r
.57366 .59519 .58147 .60375 .59656 .60212 .5888 .59357 Metetra
.572 .712 .937 r
.5888 .59357 .59656 .60212 .61175 .60045 .60405 .5919 Metetra
.573 .713 .938 r
.60405 .5919 .61175 .60045 .62706 .59872 .6194 .59018 Metetra
.574 .715 .938 r
.6194 .59018 .62706 .59872 .64247 .59693 .63486 .58839 Metetra
.575 .717 .939 r
.63486 .58839 .64247 .59693 .65798 .59509 .65043 .58655 Metetra
.576 .718 .939 r
.65043 .58655 .65798 .59509 .6736 .59318 .6661 .58465 Metetra
.577 .72 .94 r
.6661 .58465 .6736 .59318 .68933 .59122 .68189 .58269 Metetra
.579 .721 .94 r
.68189 .58269 .68933 .59122 .70517 .58919 .69778 .58066 Metetra
.58 .723 .941 r
.69778 .58066 .70517 .58919 .72112 .58709 .71379 .57856 Metetra
.581 .725 .941 r
.71379 .57856 .72112 .58709 .73717 .58493 .7299 .5764 Metetra
.583 .726 .942 r
.7299 .5764 .73717 .58493 .75334 .5827 .74613 .57416 Metetra
.584 .728 .942 r
.74613 .57416 .75334 .5827 .76961 .5804 .76247 .57186 Metetra
.586 .729 .942 r
.76247 .57186 .76961 .5804 .786 .57803 .77891 .56948 Metetra
.587 .731 .943 r
.77891 .56948 .786 .57803 .80249 .57559 .79547 .56703 Metetra
.589 .732 .943 r
.79547 .56703 .80249 .57559 .8191 .57308 .81214 .56451 Metetra
.59 .734 .943 r
.81214 .56451 .8191 .57308 .83581 .5705 .82893 .56192 Metetra
.591 .735 .944 r
.82893 .56192 .83581 .5705 .85264 .56784 .84582 .55924 Metetra
.593 .737 .944 r
.84582 .55924 .85264 .56784 .86958 .56512 .86283 .5565 Metetra
.594 .738 .944 r
.86283 .5565 .86958 .56512 .88663 .56231 .87995 .55368 Metetra
.596 .739 .944 r
.87995 .55368 .88663 .56231 .90379 .55944 .89719 .55078 Metetra
.597 .741 .944 r
.89719 .55078 .90379 .55944 .92107 .55649 .91454 .54781 Metetra
.599 .742 .945 r
.91454 .54781 .92107 .55649 .93846 .55346 .932 .54476 Metetra
.6 .743 .945 r
.932 .54476 .93846 .55346 .95596 .55036 .94958 .54163 Metetra
.568 .692 .925 r
.33688 .60794 .34535 .61666 .35889 .61541 .35045 .60669 Metetra
.567 .693 .925 r
.35045 .60669 .35889 .61541 .37252 .61417 .36412 .60544 Metetra
.567 .693 .926 r
.36412 .60544 .37252 .61417 .38625 .61293 .37787 .6042 Metetra
.566 .694 .927 r
.37787 .6042 .38625 .61293 .40007 .61169 .39172 .60295 Metetra
.566 .694 .928 r
.39172 .60295 .40007 .61169 .41399 .61044 .40567 .60171 Metetra
.565 .695 .928 r
.40567 .60171 .41399 .61044 .428 .60919 .41972 .60045 Metetra
.565 .696 .929 r
.41972 .60045 .428 .60919 .44211 .60792 .43386 .59919 Metetra
.565 .697 .93 r
.43386 .59919 .44211 .60792 .45632 .60662 .44811 .5979 Metetra
.565 .698 .931 r
.44811 .5979 .45632 .60662 .47063 .60531 .46246 .59659 Metetra
.565 .699 .931 r
.46246 .59659 .47063 .60531 .48504 .60397 .47691 .59526 Metetra
.565 .701 .932 r
.47691 .59526 .48504 .60397 .49955 .6026 .49146 .5939 Metetra
.566 .702 .933 r
.49146 .5939 .49955 .6026 .51417 .6012 .50611 .59251 Metetra
.566 .703 .934 r
.50611 .59251 .51417 .6012 .52888 .59976 .52087 .59108 Metetra
.567 .705 .934 r
.52087 .59108 .52888 .59976 .54371 .59828 .53574 .58961 Metetra
.567 .706 .935 r
.53574 .58961 .54371 .59828 .55863 .59676 .55071 .58809 Metetra
.568 .708 .936 r
.55071 .58809 .55863 .59676 .57366 .59519 .56579 .58653 Metetra
.569 .709 .936 r
.56579 .58653 .57366 .59519 .5888 .59357 .58097 .58492 Metetra
.57 .711 .937 r
.58097 .58492 .5888 .59357 .60405 .5919 .59626 .58326 Metetra
.571 .712 .938 r
.59626 .58326 .60405 .5919 .6194 .59018 .61167 .58154 Metetra
.572 .714 .938 r
.61167 .58154 .6194 .59018 .63486 .58839 .62718 .57977 Metetra
.573 .716 .939 r
.62718 .57977 .63486 .58839 .65043 .58655 .6428 .57793 Metetra
.575 .717 .94 r
.6428 .57793 .65043 .58655 .6661 .58465 .65853 .57603 Metetra
.576 .719 .94 r
.65853 .57603 .6661 .58465 .68189 .58269 .67437 .57407 Metetra
.577 .721 .941 r
.67437 .57407 .68189 .58269 .69778 .58066 .69032 .57204 Metetra
.579 .722 .941 r
.69032 .57204 .69778 .58066 .71379 .57856 .70638 .56995 Metetra
.58 .724 .942 r
.70638 .56995 .71379 .57856 .7299 .5764 .72256 .56778 Metetra
.581 .726 .942 r
.72256 .56778 .7299 .5764 .74613 .57416 .73884 .56554 Metetra
.583 .727 .942 r
.73884 .56554 .74613 .57416 .76247 .57186 .75524 .56324 Metetra
.584 .729 .943 r
.75524 .56324 .76247 .57186 .77891 .56948 .77175 .56085 Metetra
.586 .731 .943 r
.77175 .56085 .77891 .56948 .79547 .56703 .78838 .5584 Metetra
.587 .732 .944 r
.78838 .5584 .79547 .56703 .81214 .56451 .80512 .55586 Metetra
.589 .734 .944 r
.80512 .55586 .81214 .56451 .82893 .56192 .82197 .55325 Metetra
.59 .735 .944 r
.82197 .55325 .82893 .56192 .84582 .55924 .83893 .55057 Metetra
.592 .737 .944 r
.83893 .55057 .84582 .55924 .86283 .5565 .85601 .54781 Metetra
.593 .738 .945 r
.85601 .54781 .86283 .5565 .87995 .55368 .87321 .54497 Metetra
.595 .74 .945 r
.87321 .54497 .87995 .55368 .89719 .55078 .89052 .54205 Metetra
.596 .741 .945 r
.89052 .54205 .89719 .55078 .91454 .54781 .90794 .53905 Metetra
.598 .742 .945 r
.90794 .53905 .91454 .54781 .932 .54476 .92548 .53598 Metetra
.599 .743 .945 r
.92548 .53598 .932 .54476 .94958 .54163 .94314 .53282 Metetra
.568 .691 .924 r
.32834 .59911 .33688 .60794 .35045 .60669 .34194 .59784 Metetra
.567 .691 .925 r
.34194 .59784 .35045 .60669 .36412 .60544 .35563 .59659 Metetra
.566 .692 .926 r
.35563 .59659 .36412 .60544 .37787 .6042 .36942 .59535 Metetra
.565 .692 .926 r
.36942 .59535 .37787 .6042 .39172 .60295 .3833 .5941 Metetra
.564 .693 .927 r
.3833 .5941 .39172 .60295 .40567 .60171 .39729 .59286 Metetra
.564 .694 .928 r
.39729 .59286 .40567 .60171 .41972 .60045 .41137 .59161 Metetra
.564 .695 .929 r
.41137 .59161 .41972 .60045 .43386 .59919 .42555 .59034 Metetra
.564 .696 .93 r
.42555 .59034 .43386 .59919 .44811 .5979 .43983 .58907 Metetra
.563 .697 .93 r
.43983 .58907 .44811 .5979 .46246 .59659 .45421 .58777 Metetra
.564 .698 .931 r
.45421 .58777 .46246 .59659 .47691 .59526 .4687 .58644 Metetra
.564 .699 .932 r
.4687 .58644 .47691 .59526 .49146 .5939 .48329 .58509 Metetra
.564 .701 .933 r
.48329 .58509 .49146 .5939 .50611 .59251 .49798 .58371 Metetra
.565 .702 .934 r
.49798 .58371 .50611 .59251 .52087 .59108 .51278 .58229 Metetra
.565 .704 .934 r
.51278 .58229 .52087 .59108 .53574 .58961 .52769 .58083 Metetra
.566 .705 .935 r
.52769 .58083 .53574 .58961 .55071 .58809 .54271 .57933 Metetra
.567 .707 .936 r
.54271 .57933 .55071 .58809 .56579 .58653 .55783 .57778 Metetra
.568 .708 .937 r
.55783 .57778 .56579 .58653 .58097 .58492 .57306 .57618 Metetra
.569 .71 .937 r
.57306 .57618 .58097 .58492 .59626 .58326 .5884 .57453 Metetra
.57 .712 .938 r
.5884 .57453 .59626 .58326 .61167 .58154 .60385 .57282 Metetra
.571 .713 .939 r
.60385 .57282 .61167 .58154 .62718 .57977 .61942 .57105 Metetra
.572 .715 .939 r
.61942 .57105 .62718 .57977 .6428 .57793 .63509 .56922 Metetra
.573 .717 .94 r
.63509 .56922 .6428 .57793 .65853 .57603 .65087 .56733 Metetra
.574 .718 .94 r
.65087 .56733 .65853 .57603 .67437 .57407 .66677 .56537 Metetra
.576 .72 .941 r
.66677 .56537 .67437 .57407 .69032 .57204 .68278 .56335 Metetra
.577 .722 .941 r
.68278 .56335 .69032 .57204 .70638 .56995 .6989 .56125 Metetra
.579 .724 .942 r
.6989 .56125 .70638 .56995 .72256 .56778 .71513 .55909 Metetra
.58 .725 .942 r
.71513 .55909 .72256 .56778 .73884 .56554 .73148 .55685 Metetra
.582 .727 .943 r
.73148 .55685 .73884 .56554 .75524 .56324 .74794 .55453 Metetra
.583 .729 .943 r
.74794 .55453 .75524 .56324 .77175 .56085 .76452 .55214 Metetra
.585 .73 .944 r
.76452 .55214 .77175 .56085 .78838 .5584 .78121 .54968 Metetra
.586 .732 .944 r
.78121 .54968 .78838 .5584 .80512 .55586 .79801 .54714 Metetra
.588 .734 .944 r
.79801 .54714 .80512 .55586 .82197 .55325 .81493 .54452 Metetra
.589 .735 .945 r
.81493 .54452 .82197 .55325 .83893 .55057 .83197 .54182 Metetra
.591 .737 .945 r
.83197 .54182 .83893 .55057 .85601 .54781 .84912 .53904 Metetra
.592 .738 .945 r
.84912 .53904 .85601 .54781 .87321 .54497 .86639 .53618 Metetra
.594 .74 .945 r
.86639 .53618 .87321 .54497 .89052 .54205 .88377 .53324 Metetra
.595 .741 .945 r
.88377 .53324 .89052 .54205 .90794 .53905 .90127 .53022 Metetra
.597 .742 .946 r
.90127 .53022 .90794 .53905 .92548 .53598 .91889 .52712 Metetra
.599 .744 .946 r
.91889 .52712 .92548 .53598 .94314 .53282 .93662 .52394 Metetra
.567 .69 .923 r
.31972 .59015 .32834 .59911 .34194 .59784 .33335 .58888 Metetra
.566 .69 .924 r
.33335 .58888 .34194 .59784 .35563 .59659 .34707 .58762 Metetra
.565 .691 .925 r
.34707 .58762 .35563 .59659 .36942 .59535 .36089 .58638 Metetra
.564 .691 .926 r
.36089 .58638 .36942 .59535 .3833 .5941 .37481 .58513 Metetra
.563 .692 .927 r
.37481 .58513 .3833 .5941 .39729 .59286 .38882 .58389 Metetra
.563 .693 .927 r
.38882 .58389 .39729 .59286 .41137 .59161 .40293 .58264 Metetra
.562 .693 .928 r
.40293 .58264 .41137 .59161 .42555 .59034 .41715 .58139 Metetra
.562 .694 .929 r
.41715 .58139 .42555 .59034 .43983 .58907 .43147 .58012 Metetra
.562 .696 .93 r
.43147 .58012 .43983 .58907 .45421 .58777 .44588 .57883 Metetra
.562 .697 .931 r
.44588 .57883 .45421 .58777 .4687 .58644 .46041 .57751 Metetra
.562 .698 .932 r
.46041 .57751 .4687 .58644 .48329 .58509 .47504 .57618 Metetra
.563 .699 .933 r
.47504 .57618 .48329 .58509 .49798 .58371 .48977 .5748 Metetra
.563 .701 .933 r
.48977 .5748 .49798 .58371 .51278 .58229 .50462 .5734 Metetra
.564 .702 .934 r
.50462 .5734 .51278 .58229 .52769 .58083 .51957 .57195 Metetra
.564 .704 .935 r
.51957 .57195 .52769 .58083 .54271 .57933 .53463 .57046 Metetra
.565 .706 .936 r
.53463 .57046 .54271 .57933 .55783 .57778 .54979 .56892 Metetra
.566 .707 .937 r
.54979 .56892 .55783 .57778 .57306 .57618 .56507 .56734 Metetra
.567 .709 .937 r
.56507 .56734 .57306 .57618 .5884 .57453 .58046 .56569 Metetra
.568 .711 .938 r
.58046 .56569 .5884 .57453 .60385 .57282 .59596 .564 Metetra
.569 .712 .939 r
.59596 .564 .60385 .57282 .61942 .57105 .61158 .56224 Metetra
.57 .714 .939 r
.61158 .56224 .61942 .57105 .63509 .56922 .6273 .56042 Metetra
.571 .716 .94 r
.6273 .56042 .63509 .56922 .65087 .56733 .64314 .55854 Metetra
.573 .718 .941 r
.64314 .55854 .65087 .56733 .66677 .56537 .65909 .55658 Metetra
.574 .72 .941 r
.65909 .55658 .66677 .56537 .68278 .56335 .67516 .55456 Metetra
.576 .721 .942 r
.67516 .55456 .68278 .56335 .6989 .56125 .69134 .55247 Metetra
.577 .723 .942 r
.69134 .55247 .6989 .56125 .71513 .55909 .70763 .55031 Metetra
.579 .725 .943 r
.70763 .55031 .71513 .55909 .73148 .55685 .72404 .54807 Metetra
.58 .727 .943 r
.72404 .54807 .73148 .55685 .74794 .55453 .74057 .54575 Metetra
.582 .729 .944 r
.74057 .54575 .74794 .55453 .76452 .55214 .75721 .54336 Metetra
.583 .73 .944 r
.75721 .54336 .76452 .55214 .78121 .54968 .77397 .54089 Metetra
.585 .732 .945 r
.77397 .54089 .78121 .54968 .79801 .54714 .79084 .53833 Metetra
.586 .734 .945 r
.79084 .53833 .79801 .54714 .81493 .54452 .80783 .5357 Metetra
.588 .735 .945 r
.80783 .5357 .81493 .54452 .83197 .54182 .82493 .53299 Metetra
.59 .737 .945 r
.82493 .53299 .83197 .54182 .84912 .53904 .84215 .5302 Metetra
.591 .738 .946 r
.84215 .5302 .84912 .53904 .86639 .53618 .8595 .52732 Metetra
.593 .74 .946 r
.8595 .52732 .86639 .53618 .88377 .53324 .87695 .52436 Metetra
.595 .741 .946 r
.87695 .52436 .88377 .53324 .90127 .53022 .89453 .52132 Metetra
.596 .743 .946 r
.89453 .52132 .90127 .53022 .91889 .52712 .91222 .51819 Metetra
.598 .744 .946 r
.91222 .51819 .91889 .52712 .93662 .52394 .93004 .51499 Metetra
.566 .689 .923 r
.31103 .58108 .31972 .59015 .33335 .58888 .32469 .5798 Metetra
.565 .689 .924 r
.32469 .5798 .33335 .58888 .34707 .58762 .33844 .57854 Metetra
.564 .689 .924 r
.33844 .57854 .34707 .58762 .36089 .58638 .35229 .57728 Metetra
.563 .69 .925 r
.35229 .57728 .36089 .58638 .37481 .58513 .36624 .57604 Metetra
.562 .69 .926 r
.36624 .57604 .37481 .58513 .38882 .58389 .38028 .5748 Metetra
.562 .691 .927 r
.38028 .5748 .38882 .58389 .40293 .58264 .39443 .57356 Metetra
.561 .692 .928 r
.39443 .57356 .40293 .58264 .41715 .58139 .40868 .57231 Metetra
.561 .693 .929 r
.40868 .57231 .41715 .58139 .43147 .58012 .42303 .57105 Metetra
.561 .694 .93 r
.42303 .57105 .43147 .58012 .44588 .57883 .43748 .56977 Metetra
.561 .695 .931 r
.43748 .56977 .44588 .57883 .46041 .57751 .45204 .56847 Metetra
.561 .697 .932 r
.45204 .56847 .46041 .57751 .47504 .57618 .46671 .56714 Metetra
.561 .698 .932 r
.46671 .56714 .47504 .57618 .48977 .5748 .48149 .56579 Metetra
.561 .7 .933 r
.48149 .56579 .48977 .5748 .50462 .5734 .49637 .56439 Metetra
.562 .701 .934 r
.49637 .56439 .50462 .5734 .51957 .57195 .51136 .56296 Metetra
.562 .703 .935 r
.51136 .56296 .51957 .57195 .53463 .57046 .52647 .56149 Metetra
.563 .704 .936 r
.52647 .56149 .53463 .57046 .54979 .56892 .54168 .55996 Metetra
.564 .706 .937 r
.54168 .55996 .54979 .56892 .56507 .56734 .55701 .55839 Metetra
.565 .708 .937 r
.55701 .55839 .56507 .56734 .58046 .56569 .57244 .55676 Metetra
.566 .71 .938 r
.57244 .55676 .58046 .56569 .59596 .564 .58799 .55508 Metetra
.567 .711 .939 r
.58799 .55508 .59596 .564 .61158 .56224 .60366 .55333 Metetra
.568 .713 .94 r
.60366 .55333 .61158 .56224 .6273 .56042 .61944 .55152 Metetra
.57 .715 .94 r
.61944 .55152 .6273 .56042 .64314 .55854 .63533 .54965 Metetra
.571 .717 .941 r
.63533 .54965 .64314 .55854 .65909 .55658 .65134 .54771 Metetra
.572 .719 .942 r
.65134 .54771 .65909 .55658 .67516 .55456 .66746 .54569 Metetra
.574 .721 .942 r
.66746 .54569 .67516 .55456 .69134 .55247 .6837 .54361 Metetra
.575 .723 .943 r
.6837 .54361 .69134 .55247 .70763 .55031 .70006 .54144 Metetra
.577 .725 .943 r
.70006 .54144 .70763 .55031 .72404 .54807 .71653 .5392 Metetra
.579 .726 .944 r
.71653 .5392 .72404 .54807 .74057 .54575 .73312 .53689 Metetra
.58 .728 .944 r
.73312 .53689 .74057 .54575 .75721 .54336 .74982 .53449 Metetra
.582 .73 .945 r
.74982 .53449 .75721 .54336 .77397 .54089 .76664 .53201 Metetra
.583 .732 .945 r
.76664 .53201 .77397 .54089 .79084 .53833 .78358 .52946 Metetra
.585 .733 .945 r
.78358 .52946 .79084 .53833 .80783 .5357 .80064 .52681 Metetra
.587 .735 .946 r
.80064 .52681 .80783 .5357 .82493 .53299 .81782 .52409 Metetra
.588 .737 .946 r
.81782 .52409 .82493 .53299 .84215 .5302 .83511 .52128 Metetra
.59 .738 .946 r
.83511 .52128 .84215 .5302 .8595 .52732 .85253 .51838 Metetra
.592 .74 .946 r
.85253 .51838 .8595 .52732 .87695 .52436 .87006 .5154 Metetra
.594 .741 .947 r
.87006 .5154 .87695 .52436 .89453 .52132 .88772 .51234 Metetra
.595 .743 .947 r
.88772 .51234 .89453 .52132 .91222 .51819 .90549 .50919 Metetra
.597 .744 .947 r
.90549 .50919 .91222 .51819 .93004 .51499 .92338 .50596 Metetra
.566 .687 .922 r
.30227 .57188 .31103 .58108 .32469 .5798 .31595 .57059 Metetra
.564 .687 .923 r
.31595 .57059 .32469 .5798 .33844 .57854 .32973 .56932 Metetra
.563 .688 .924 r
.32973 .56932 .33844 .57854 .35229 .57728 .34361 .56807 Metetra
.562 .688 .925 r
.34361 .56807 .35229 .57728 .36624 .57604 .35759 .56682 Metetra
.561 .689 .926 r
.35759 .56682 .36624 .57604 .38028 .5748 .37167 .56559 Metetra
.56 .69 .927 r
.37167 .56559 .38028 .5748 .39443 .57356 .38584 .56435 Metetra
.56 .69 .927 r
.38584 .56435 .39443 .57356 .40868 .57231 .40013 .56311 Metetra
.559 .691 .928 r
.40013 .56311 .40868 .57231 .42303 .57105 .41451 .56186 Metetra
.559 .693 .929 r
.41451 .56186 .42303 .57105 .43748 .56977 .429 .56059 Metetra
.559 .694 .93 r
.429 .56059 .43748 .56977 .45204 .56847 .4436 .55931 Metetra
.559 .695 .931 r
.4436 .55931 .45204 .56847 .46671 .56714 .4583 .55799 Metetra
.559 .697 .932 r
.4583 .55799 .46671 .56714 .48149 .56579 .47312 .55665 Metetra
.56 .698 .933 r
.47312 .55665 .48149 .56579 .49637 .56439 .48804 .55528 Metetra
.56 .7 .934 r
.48804 .55528 .49637 .56439 .51136 .56296 .50308 .55386 Metetra
.561 .701 .935 r
.50308 .55386 .51136 .56296 .52647 .56149 .51822 .5524 Metetra
.561 .703 .936 r
.51822 .5524 .52647 .56149 .54168 .55996 .53348 .5509 Metetra
.562 .705 .937 r
.53348 .5509 .54168 .55996 .55701 .55839 .54886 .54934 Metetra
.563 .707 .938 r
.54886 .54934 .55701 .55839 .57244 .55676 .56434 .54773 Metetra
.564 .709 .938 r
.56434 .54773 .57244 .55676 .58799 .55508 .57994 .54606 Metetra
.565 .711 .939 r
.57994 .54606 .58799 .55508 .60366 .55333 .59566 .54433 Metetra
.567 .712 .94 r
.59566 .54433 .60366 .55333 .61944 .55152 .61149 .54254 Metetra
.568 .714 .941 r
.61149 .54254 .61944 .55152 .63533 .54965 .62744 .54067 Metetra
.569 .716 .941 r
.62744 .54067 .63533 .54965 .65134 .54771 .6435 .53874 Metetra
.571 .718 .942 r
.6435 .53874 .65134 .54771 .66746 .54569 .65968 .53673 Metetra
.572 .72 .943 r
.65968 .53673 .66746 .54569 .6837 .54361 .67598 .53465 Metetra
.574 .722 .943 r
.67598 .53465 .6837 .54361 .70006 .54144 .6924 .5325 Metetra
.575 .724 .944 r
.6924 .5325 .70006 .54144 .71653 .5392 .70893 .53026 Metetra
.577 .726 .944 r
.70893 .53026 .71653 .5392 .73312 .53689 .72558 .52795 Metetra
.579 .728 .945 r
.72558 .52795 .73312 .53689 .74982 .53449 .74235 .52555 Metetra
.58 .73 .945 r
.74235 .52555 .74982 .53449 .76664 .53201 .75924 .52307 Metetra
.582 .732 .946 r
.75924 .52307 .76664 .53201 .78358 .52946 .77625 .5205 Metetra
.584 .733 .946 r
.77625 .5205 .78358 .52946 .80064 .52681 .79338 .51785 Metetra
.586 .735 .946 r
.79338 .51785 .80064 .52681 .81782 .52409 .81063 .51511 Metetra
.587 .737 .947 r
.81063 .51511 .81782 .52409 .83511 .52128 .828 .51229 Metetra
.589 .738 .947 r
.828 .51229 .83511 .52128 .85253 .51838 .84549 .50938 Metetra
.591 .74 .947 r
.84549 .50938 .85253 .51838 .87006 .5154 .8631 .50638 Metetra
.592 .742 .947 r
.8631 .50638 .87006 .5154 .88772 .51234 .88083 .50329 Metetra
.594 .743 .947 r
.88083 .50329 .88772 .51234 .90549 .50919 .89868 .50012 Metetra
.596 .745 .948 r
.89868 .50012 .90549 .50919 .92338 .50596 .91665 .49685 Metetra
.565 .686 .921 r
.29342 .56255 .30227 .57188 .31595 .57059 .30714 .56125 Metetra
.563 .686 .922 r
.30714 .56125 .31595 .57059 .32973 .56932 .32095 .55997 Metetra
.562 .686 .923 r
.32095 .55997 .32973 .56932 .34361 .56807 .33486 .55872 Metetra
.561 .687 .924 r
.33486 .55872 .34361 .56807 .35759 .56682 .34886 .55747 Metetra
.56 .687 .925 r
.34886 .55747 .35759 .56682 .37167 .56559 .36297 .55624 Metetra
.559 .688 .926 r
.36297 .55624 .37167 .56559 .38584 .56435 .37718 .55501 Metetra
.558 .689 .927 r
.37718 .55501 .38584 .56435 .40013 .56311 .3915 .55378 Metetra
.558 .69 .928 r
.3915 .55378 .40013 .56311 .41451 .56186 .40592 .55254 Metetra
.557 .691 .929 r
.40592 .55254 .41451 .56186 .429 .56059 .42044 .55129 Metetra
.557 .692 .93 r
.42044 .55129 .429 .56059 .4436 .55931 .43508 .55002 Metetra
.557 .694 .931 r
.43508 .55002 .4436 .55931 .4583 .55799 .44982 .54872 Metetra
.557 .695 .932 r
.44982 .54872 .4583 .55799 .47312 .55665 .46467 .5474 Metetra
.558 .697 .933 r
.46467 .5474 .47312 .55665 .48804 .55528 .47964 .54604 Metetra
.558 .698 .934 r
.47964 .54604 .48804 .55528 .50308 .55386 .49471 .54465 Metetra
.559 .7 .935 r
.49471 .54465 .50308 .55386 .51822 .5524 .5099 .54321 Metetra
.559 .702 .936 r
.5099 .54321 .51822 .5524 .53348 .5509 .52521 .54172 Metetra
.56 .704 .937 r
.52521 .54172 .53348 .5509 .54886 .54934 .54063 .54019 Metetra
.561 .706 .938 r
.54063 .54019 .54886 .54934 .56434 .54773 .55616 .5386 Metetra
.562 .707 .939 r
.55616 .5386 .56434 .54773 .57994 .54606 .57181 .53694 Metetra
.563 .709 .939 r
.57181 .53694 .57994 .54606 .59566 .54433 .58758 .53523 Metetra
.565 .711 .94 r
.58758 .53523 .59566 .54433 .61149 .54254 .60346 .53345 Metetra
.566 .714 .941 r
.60346 .53345 .61149 .54254 .62744 .54067 .61946 .5316 Metetra
.567 .716 .942 r
.61946 .5316 .62744 .54067 .6435 .53874 .63558 .52968 Metetra
.569 .718 .942 r
.63558 .52968 .6435 .53874 .65968 .53673 .65182 .52769 Metetra
.57 .72 .943 r
.65182 .52769 .65968 .53673 .67598 .53465 .66818 .52562 Metetra
.572 .722 .944 r
.66818 .52562 .67598 .53465 .6924 .5325 .68466 .52347 Metetra
.574 .724 .944 r
.68466 .52347 .6924 .5325 .70893 .53026 .70125 .52124 Metetra
.575 .726 .945 r
.70125 .52124 .70893 .53026 .72558 .52795 .71797 .51892 Metetra
.577 .728 .945 r
.71797 .51892 .72558 .52795 .74235 .52555 .73481 .51652 Metetra
.579 .73 .946 r
.73481 .51652 .74235 .52555 .75924 .52307 .75176 .51404 Metetra
.581 .732 .946 r
.75176 .51404 .75924 .52307 .77625 .5205 .76884 .51147 Metetra
.582 .733 .947 r
.76884 .51147 .77625 .5205 .79338 .51785 .78604 .50881 Metetra
.584 .735 .947 r
.78604 .50881 .79338 .51785 .81063 .51511 .80336 .50606 Metetra
.586 .737 .947 r
.80336 .50606 .81063 .51511 .828 .51229 .82081 .50322 Metetra
.588 .739 .948 r
.82081 .50322 .828 .51229 .84549 .50938 .83837 .5003 Metetra
.59 .74 .948 r
.83837 .5003 .84549 .50938 .8631 .50638 .85606 .49728 Metetra
.591 .742 .948 r
.85606 .49728 .8631 .50638 .88083 .50329 .87387 .49417 Metetra
.593 .743 .948 r
.87387 .49417 .88083 .50329 .89868 .50012 .8918 .49097 Metetra
.595 .745 .948 r
.8918 .49097 .89868 .50012 .91665 .49685 .90985 .48768 Metetra
.564 .684 .921 r
.28451 .55308 .29342 .56255 .30714 .56125 .29825 .55177 Metetra
.562 .684 .922 r
.29825 .55177 .30714 .56125 .32095 .55997 .31209 .55049 Metetra
.561 .685 .922 r
.31209 .55049 .32095 .55997 .33486 .55872 .32602 .54923 Metetra
.56 .685 .923 r
.32602 .54923 .33486 .55872 .34886 .55747 .34006 .54799 Metetra
.559 .686 .924 r
.34006 .54799 .34886 .55747 .36297 .55624 .3542 .54676 Metetra
.558 .686 .925 r
.3542 .54676 .36297 .55624 .37718 .55501 .36844 .54554 Metetra
.557 .687 .926 r
.36844 .54554 .37718 .55501 .3915 .55378 .38279 .54432 Metetra
.556 .688 .927 r
.38279 .54432 .3915 .55378 .40592 .55254 .39724 .54309 Metetra
.556 .689 .929 r
.39724 .54309 .40592 .55254 .42044 .55129 .4118 .54186 Metetra
.555 .691 .93 r
.4118 .54186 .42044 .55129 .43508 .55002 .42647 .5406 Metetra
.555 .692 .931 r
.42647 .5406 .43508 .55002 .44982 .54872 .44125 .53933 Metetra
.555 .693 .932 r
.44125 .53933 .44982 .54872 .46467 .5474 .45614 .53802 Metetra
.556 .695 .933 r
.45614 .53802 .46467 .5474 .47964 .54604 .47115 .53669 Metetra
.556 .697 .934 r
.47115 .53669 .47964 .54604 .49471 .54465 .48627 .53532 Metetra
.557 .698 .935 r
.48627 .53532 .49471 .54465 .5099 .54321 .5015 .5339 Metetra
.557 .7 .936 r
.5015 .5339 .5099 .54321 .52521 .54172 .51685 .53244 Metetra
.558 .702 .937 r
.51685 .53244 .52521 .54172 .54063 .54019 .53231 .53092 Metetra
.559 .704 .938 r
.53231 .53092 .54063 .54019 .55616 .5386 .54789 .52935 Metetra
.56 .706 .939 r
.54789 .52935 .55616 .5386 .57181 .53694 .56359 .52772 Metetra
.561 .708 .94 r
.56359 .52772 .57181 .53694 .58758 .53523 .57941 .52603 Metetra
.562 .71 .94 r
.57941 .52603 .58758 .53523 .60346 .53345 .59535 .52427 Metetra
.564 .713 .941 r
.59535 .52427 .60346 .53345 .61946 .5316 .6114 .52244 Metetra
.565 .715 .942 r
.6114 .52244 .61946 .5316 .63558 .52968 .62758 .52054 Metetra
.567 .717 .943 r
.62758 .52054 .63558 .52968 .65182 .52769 .64388 .51855 Metetra
.568 .719 .944 r
.64388 .51855 .65182 .52769 .66818 .52562 .6603 .51649 Metetra
.57 .721 .944 r
.6603 .51649 .66818 .52562 .68466 .52347 .67683 .51435 Metetra
.572 .723 .945 r
.67683 .51435 .68466 .52347 .70125 .52124 .69349 .51213 Metetra
.573 .725 .945 r
.69349 .51213 .70125 .52124 .71797 .51892 .71027 .50982 Metetra
.575 .727 .946 r
.71027 .50982 .71797 .51892 .73481 .51652 .72718 .50742 Metetra
.577 .729 .946 r
.72718 .50742 .73481 .51652 .75176 .51404 .7442 .50494 Metetra
.579 .731 .947 r
.7442 .50494 .75176 .51404 .76884 .51147 .76135 .50236 Metetra
.581 .733 .947 r
.76135 .50236 .76884 .51147 .78604 .50881 .77862 .4997 Metetra
.583 .735 .948 r
.77862 .4997 .78604 .50881 .80336 .50606 .79602 .49694 Metetra
.585 .737 .948 r
.79602 .49694 .80336 .50606 .82081 .50322 .81353 .49409 Metetra
.586 .739 .948 r
.81353 .49409 .82081 .50322 .83837 .5003 .83117 .49114 Metetra
.588 .741 .949 r
.83117 .49114 .83837 .5003 .85606 .49728 .84894 .48811 Metetra
.59 .742 .949 r
.84894 .48811 .85606 .49728 .87387 .49417 .86683 .48498 Metetra
.592 .744 .949 r
.86683 .48498 .87387 .49417 .8918 .49097 .88484 .48175 Metetra
.594 .745 .949 r
.88484 .48175 .8918 .49097 .90985 .48768 .90298 .47843 Metetra
.564 .683 .92 r
.27551 .54347 .28451 .55308 .29825 .55177 .28928 .54215 Metetra
.562 .683 .921 r
.28928 .54215 .29825 .55177 .31209 .55049 .30315 .54086 Metetra
.56 .683 .922 r
.30315 .54086 .31209 .55049 .32602 .54923 .31712 .5396 Metetra
.559 .683 .923 r
.31712 .5396 .32602 .54923 .34006 .54799 .33118 .53836 Metetra
.557 .684 .924 r
.33118 .53836 .34006 .54799 .3542 .54676 .34535 .53714 Metetra
.556 .684 .925 r
.34535 .53714 .3542 .54676 .36844 .54554 .35962 .53593 Metetra
.555 .685 .926 r
.35962 .53593 .36844 .54554 .38279 .54432 .374 .53472 Metetra
.555 .686 .927 r
.374 .53472 .38279 .54432 .39724 .54309 .38849 .53351 Metetra
.554 .687 .928 r
.38849 .53351 .39724 .54309 .4118 .54186 .40308 .53229 Metetra
.554 .689 .929 r
.40308 .53229 .4118 .54186 .42647 .5406 .41779 .53106 Metetra
.553 .69 .93 r
.41779 .53106 .42647 .5406 .44125 .53933 .4326 .5298 Metetra
.553 .692 .931 r
.4326 .5298 .44125 .53933 .45614 .53802 .44753 .52852 Metetra
.554 .693 .932 r
.44753 .52852 .45614 .53802 .47115 .53669 .46258 .52721 Metetra
.554 .695 .934 r
.46258 .52721 .47115 .53669 .48627 .53532 .47773 .52586 Metetra
.554 .697 .935 r
.47773 .52586 .48627 .53532 .5015 .5339 .49301 .52447 Metetra
.555 .699 .936 r
.49301 .52447 .5015 .5339 .51685 .53244 .5084 .52304 Metetra
.556 .701 .937 r
.5084 .52304 .51685 .53244 .53231 .53092 .52391 .52155 Metetra
.557 .703 .938 r
.52391 .52155 .53231 .53092 .54789 .52935 .53954 .52 Metetra
.558 .705 .939 r
.53954 .52 .54789 .52935 .56359 .52772 .55529 .5184 Metetra
.559 .707 .94 r
.55529 .5184 .56359 .52772 .57941 .52603 .57116 .51673 Metetra
.56 .709 .941 r
.57116 .51673 .57941 .52603 .59535 .52427 .58715 .51499 Metetra
.562 .712 .942 r
.58715 .51499 .59535 .52427 .6114 .52244 .60326 .51318 Metetra
.563 .714 .942 r
.60326 .51318 .6114 .52244 .62758 .52054 .61949 .5113 Metetra
.565 .716 .943 r
.61949 .5113 .62758 .52054 .64388 .51855 .63585 .50933 Metetra
.566 .718 .944 r
.63585 .50933 .64388 .51855 .6603 .51649 .65233 .50729 Metetra
.568 .721 .945 r
.65233 .50729 .6603 .51649 .67683 .51435 .66893 .50516 Metetra
.57 .723 .945 r
.66893 .50516 .67683 .51435 .69349 .51213 .68565 .50294 Metetra
.572 .725 .946 r
.68565 .50294 .69349 .51213 .71027 .50982 .7025 .50064 Metetra
.573 .727 .947 r
.7025 .50064 .71027 .50982 .72718 .50742 .71947 .49825 Metetra
.575 .729 .947 r
.71947 .49825 .72718 .50742 .7442 .50494 .73656 .49576 Metetra
.577 .731 .948 r
.73656 .49576 .7442 .50494 .76135 .50236 .75378 .49318 Metetra
.579 .733 .948 r
.75378 .49318 .76135 .50236 .77862 .4997 .77112 .49051 Metetra
.581 .735 .948 r
.77112 .49051 .77862 .4997 .79602 .49694 .78859 .48775 Metetra
.583 .737 .949 r
.78859 .48775 .79602 .49694 .81353 .49409 .80618 .48488 Metetra
.585 .739 .949 r
.80618 .48488 .81353 .49409 .83117 .49114 .8239 .48192 Metetra
.587 .741 .949 r
.8239 .48192 .83117 .49114 .84894 .48811 .84174 .47887 Metetra
.589 .743 .95 r
.84174 .47887 .84894 .48811 .86683 .48498 .85971 .47572 Metetra
.591 .744 .95 r
.85971 .47572 .86683 .48498 .88484 .48175 .8778 .47247 Metetra
.593 .746 .95 r
.8778 .47247 .88484 .48175 .90298 .47843 .89602 .46912 Metetra
.563 .681 .919 r
.26644 .53371 .27551 .54347 .28928 .54215 .28024 .53238 Metetra
.561 .681 .92 r
.28024 .53238 .28928 .54215 .30315 .54086 .29413 .53108 Metetra
.559 .681 .921 r
.29413 .53108 .30315 .54086 .31712 .5396 .30813 .52982 Metetra
.557 .681 .922 r
.30813 .52982 .31712 .5396 .33118 .53836 .32222 .52858 Metetra
.556 .682 .923 r
.32222 .52858 .33118 .53836 .34535 .53714 .33642 .52737 Metetra
.555 .683 .924 r
.33642 .52737 .34535 .53714 .35962 .53593 .35072 .52617 Metetra
.554 .683 .925 r
.35072 .52617 .35962 .53593 .374 .53472 .36513 .52497 Metetra
.553 .684 .926 r
.36513 .52497 .374 .53472 .38849 .53351 .37965 .52378 Metetra
.552 .685 .927 r
.37965 .52378 .38849 .53351 .40308 .53229 .39428 .52258 Metetra
.552 .687 .929 r
.39428 .52258 .40308 .53229 .41779 .53106 .40902 .52137 Metetra
.551 .688 .93 r
.40902 .52137 .41779 .53106 .4326 .5298 .42387 .52014 Metetra
.551 .69 .931 r
.42387 .52014 .4326 .5298 .44753 .52852 .43884 .51889 Metetra
.551 .691 .932 r
.43884 .51889 .44753 .52852 .46258 .52721 .45392 .5176 Metetra
.552 .693 .933 r
.45392 .5176 .46258 .52721 .47773 .52586 .46912 .51629 Metetra
.552 .695 .935 r
.46912 .51629 .47773 .52586 .49301 .52447 .48444 .51493 Metetra
.553 .697 .936 r
.48444 .51493 .49301 .52447 .5084 .52304 .49987 .51352 Metetra
.553 .699 .937 r
.49987 .51352 .5084 .52304 .52391 .52155 .51543 .51206 Metetra
.554 .701 .938 r
.51543 .51206 .52391 .52155 .53954 .52 .53111 .51054 Metetra
.555 .704 .939 r
.53111 .51054 .53954 .52 .55529 .5184 .5469 .50897 Metetra
.556 .706 .94 r
.5469 .50897 .55529 .5184 .57116 .51673 .56282 .50732 Metetra
.558 .708 .941 r
.56282 .50732 .57116 .51673 .58715 .51499 .57887 .50561 Metetra
.559 .711 .942 r
.57887 .50561 .58715 .51499 .60326 .51318 .59503 .50383 Metetra
.561 .713 .943 r
.59503 .50383 .60326 .51318 .61949 .5113 .61132 .50196 Metetra
.562 .715 .944 r
.61132 .50196 .61949 .5113 .63585 .50933 .62773 .50002 Metetra
.564 .718 .945 r
.62773 .50002 .63585 .50933 .65233 .50729 .64427 .49799 Metetra
.566 .72 .945 r
.64427 .49799 .65233 .50729 .66893 .50516 .66093 .49588 Metetra
.568 .722 .946 r
.66093 .49588 .66893 .50516 .68565 .50294 .67772 .49367 Metetra
.569 .725 .947 r
.67772 .49367 .68565 .50294 .7025 .50064 .69463 .49138 Metetra
.571 .727 .947 r
.69463 .49138 .7025 .50064 .71947 .49825 .71167 .48899 Metetra
.573 .729 .948 r
.71167 .48899 .71947 .49825 .73656 .49576 .72883 .48651 Metetra
.575 .731 .948 r
.72883 .48651 .73656 .49576 .75378 .49318 .74612 .48393 Metetra
.577 .733 .949 r
.74612 .48393 .75378 .49318 .77112 .49051 .76354 .48126 Metetra
.579 .735 .949 r
.76354 .48126 .77112 .49051 .78859 .48775 .78108 .47848 Metetra
.581 .737 .95 r
.78108 .47848 .78859 .48775 .80618 .48488 .79875 .47561 Metetra
.584 .739 .95 r
.79875 .47561 .80618 .48488 .8239 .48192 .81654 .47264 Metetra
.586 .741 .95 r
.81654 .47264 .8239 .48192 .84174 .47887 .83447 .46956 Metetra
.588 .743 .95 r
.83447 .46956 .84174 .47887 .85971 .47572 .85252 .46639 Metetra
.59 .745 .951 r
.85252 .46639 .85971 .47572 .8778 .47247 .87069 .46312 Metetra
.592 .746 .951 r
.87069 .46312 .8778 .47247 .89602 .46912 .889 .45974 Metetra
.562 .679 .918 r
.2573 .5238 .26644 .53371 .28024 .53238 .27112 .52245 Metetra
.56 .679 .919 r
.27112 .52245 .28024 .53238 .29413 .53108 .28504 .52115 Metetra
.558 .679 .92 r
.28504 .52115 .29413 .53108 .30813 .52982 .29906 .51989 Metetra
.556 .679 .921 r
.29906 .51989 .30813 .52982 .32222 .52858 .31319 .51865 Metetra
.555 .68 .922 r
.31319 .51865 .32222 .52858 .33642 .52737 .32741 .51744 Metetra
.553 .68 .923 r
.32741 .51744 .33642 .52737 .35072 .52617 .34174 .51625 Metetra
.552 .681 .924 r
.34174 .51625 .35072 .52617 .36513 .52497 .35618 .51507 Metetra
.551 .682 .926 r
.35618 .51507 .36513 .52497 .37965 .52378 .37073 .5139 Metetra
.55 .683 .927 r
.37073 .5139 .37965 .52378 .39428 .52258 .38539 .51272 Metetra
.55 .685 .928 r
.38539 .51272 .39428 .52258 .40902 .52137 .40017 .51154 Metetra
.549 .686 .929 r
.40017 .51154 .40902 .52137 .42387 .52014 .41506 .51034 Metetra
.549 .688 .931 r
.41506 .51034 .42387 .52014 .43884 .51889 .43006 .50911 Metetra
.549 .689 .932 r
.43006 .50911 .43884 .51889 .45392 .5176 .44518 .50786 Metetra
.549 .691 .933 r
.44518 .50786 .45392 .5176 .46912 .51629 .46042 .50658 Metetra
.55 .693 .934 r
.46042 .50658 .46912 .51629 .48444 .51493 .47578 .50525 Metetra
.55 .695 .936 r
.47578 .50525 .48444 .51493 .49987 .51352 .49126 .50388 Metetra
.551 .698 .937 r
.49126 .50388 .49987 .51352 .51543 .51206 .50686 .50245 Metetra
.552 .7 .938 r
.50686 .50245 .51543 .51206 .53111 .51054 .52258 .50097 Metetra
.553 .702 .939 r
.52258 .50097 .53111 .51054 .5469 .50897 .53843 .49943 Metetra
.554 .704 .94 r
.53843 .49943 .5469 .50897 .56282 .50732 .5544 .49781 Metetra
.555 .707 .941 r
.5544 .49781 .56282 .50732 .57887 .50561 .5705 .49613 Metetra
.557 .709 .942 r
.5705 .49613 .57887 .50561 .59503 .50383 .58672 .49437 Metetra
.558 .712 .943 r
.58672 .49437 .59503 .50383 .61132 .50196 .60306 .49254 Metetra
.56 .714 .944 r
.60306 .49254 .61132 .50196 .62773 .50002 .61953 .49062 Metetra
.562 .717 .945 r
.61953 .49062 .62773 .50002 .64427 .49799 .63613 .48861 Metetra
.563 .719 .946 r
.63613 .48861 .64427 .49799 .66093 .49588 .65286 .48651 Metetra
.565 .722 .947 r
.65286 .48651 .66093 .49588 .67772 .49367 .66971 .48433 Metetra
.567 .724 .947 r
.66971 .48433 .67772 .49367 .69463 .49138 .68668 .48204 Metetra
.569 .727 .948 r
.68668 .48204 .69463 .49138 .71167 .48899 .70379 .47967 Metetra
.571 .729 .949 r
.70379 .47967 .71167 .48899 .72883 .48651 .72102 .47719 Metetra
.573 .731 .949 r
.72102 .47719 .72883 .48651 .74612 .48393 .73838 .47461 Metetra
.575 .733 .95 r
.73838 .47461 .74612 .48393 .76354 .48126 .75587 .47194 Metetra
.578 .736 .95 r
.75587 .47194 .76354 .48126 .78108 .47848 .77349 .46916 Metetra
.58 .738 .951 r
.77349 .46916 .78108 .47848 .79875 .47561 .79123 .46627 Metetra
.582 .74 .951 r
.79123 .46627 .79875 .47561 .81654 .47264 .80911 .46329 Metetra
.584 .742 .951 r
.80911 .46329 .81654 .47264 .83447 .46956 .82711 .4602 Metetra
.586 .744 .951 r
.82711 .4602 .83447 .46956 .85252 .46639 .84524 .457 Metetra
.588 .745 .952 r
.84524 .457 .85252 .46639 .87069 .46312 .8635 .4537 Metetra
.591 .747 .952 r
.8635 .4537 .87069 .46312 .889 .45974 .88189 .4503 Metetra
.562 .678 .917 r
.24808 .51373 .2573 .5238 .27112 .52245 .26193 .51237 Metetra
.559 .677 .918 r
.26193 .51237 .27112 .52245 .28504 .52115 .27587 .51106 Metetra
.557 .677 .919 r
.27587 .51106 .28504 .52115 .29906 .51989 .28992 .50979 Metetra
.555 .677 .92 r
.28992 .50979 .29906 .51989 .31319 .51865 .30407 .50856 Metetra
.553 .678 .921 r
.30407 .50856 .31319 .51865 .32741 .51744 .31832 .50736 Metetra
.552 .678 .922 r
.31832 .50736 .32741 .51744 .34174 .51625 .33268 .50618 Metetra
.55 .679 .923 r
.33268 .50618 .34174 .51625 .35618 .51507 .34715 .50502 Metetra
.549 .68 .925 r
.34715 .50502 .35618 .51507 .37073 .5139 .36173 .50386 Metetra
.548 .681 .926 r
.36173 .50386 .37073 .5139 .38539 .51272 .37643 .50271 Metetra
.547 .682 .927 r
.37643 .50271 .38539 .51272 .40017 .51154 .39123 .50156 Metetra
.547 .684 .929 r
.39123 .50156 .40017 .51154 .41506 .51034 .40616 .50039 Metetra
.547 .685 .93 r
.40616 .50039 .41506 .51034 .43006 .50911 .4212 .4992 Metetra
.547 .687 .931 r
.4212 .4992 .43006 .50911 .44518 .50786 .43635 .49798 Metetra
.547 .689 .933 r
.43635 .49798 .44518 .50786 .46042 .50658 .45163 .49673 Metetra
.547 .691 .934 r
.45163 .49673 .46042 .50658 .47578 .50525 .46703 .49544 Metetra
.547 .693 .935 r
.46703 .49544 .47578 .50525 .49126 .50388 .48256 .49411 Metetra
.548 .696 .937 r
.48256 .49411 .49126 .50388 .50686 .50245 .4982 .49272 Metetra
.549 .698 .938 r
.4982 .49272 .50686 .50245 .52258 .50097 .51397 .49128 Metetra
.55 .701 .939 r
.51397 .49128 .52258 .50097 .53843 .49943 .52987 .48977 Metetra
.551 .703 .94 r
.52987 .48977 .53843 .49943 .5544 .49781 .54589 .4882 Metetra
.552 .706 .942 r
.54589 .4882 .5544 .49781 .5705 .49613 .56204 .48655 Metetra
.554 .708 .943 r
.56204 .48655 .5705 .49613 .58672 .49437 .57831 .48482 Metetra
.555 .711 .944 r
.57831 .48482 .58672 .49437 .60306 .49254 .59471 .48302 Metetra
.557 .713 .945 r
.59471 .48302 .60306 .49254 .61953 .49062 .61124 .48113 Metetra
.559 .716 .946 r
.61124 .48113 .61953 .49062 .63613 .48861 .6279 .47914 Metetra
.561 .719 .947 r
.6279 .47914 .63613 .48861 .65286 .48651 .64469 .47707 Metetra
.563 .721 .948 r
.64469 .47707 .65286 .48651 .66971 .48433 .6616 .4749 Metetra
.565 .724 .948 r
.6616 .4749 .66971 .48433 .68668 .48204 .67865 .47263 Metetra
.567 .726 .949 r
.67865 .47263 .68668 .48204 .70379 .47967 .69582 .47027 Metetra
.569 .729 .95 r
.69582 .47027 .70379 .47967 .72102 .47719 .71313 .4678 Metetra
.571 .731 .95 r
.71313 .4678 .72102 .47719 .73838 .47461 .73056 .46523 Metetra
.573 .734 .951 r
.73056 .46523 .73838 .47461 .75587 .47194 .74812 .46255 Metetra
.576 .736 .951 r
.74812 .46255 .75587 .47194 .77349 .46916 .76581 .45977 Metetra
.578 .738 .952 r
.76581 .45977 .77349 .46916 .79123 .46627 .78364 .45687 Metetra
.58 .74 .952 r
.78364 .45687 .79123 .46627 .80911 .46329 .80159 .45388 Metetra
.583 .742 .952 r
.80159 .45388 .80911 .46329 .82711 .4602 .81967 .45077 Metetra
.585 .744 .952 r
.81967 .45077 .82711 .4602 .84524 .457 .83788 .44755 Metetra
.587 .746 .953 r
.83788 .44755 .84524 .457 .8635 .4537 .85623 .44423 Metetra
.589 .748 .953 r
.85623 .44423 .8635 .4537 .88189 .4503 .8747 .4408 Metetra
.561 .676 .915 r
.23878 .5035 .24808 .51373 .26193 .51237 .25265 .50212 Metetra
.559 .675 .916 r
.25265 .50212 .26193 .51237 .27587 .51106 .26663 .50079 Metetra
.556 .675 .918 r
.26663 .50079 .27587 .51106 .28992 .50979 .2807 .49952 Metetra
.554 .675 .919 r
.2807 .49952 .28992 .50979 .30407 .50856 .29488 .49829 Metetra
.552 .675 .92 r
.29488 .49829 .30407 .50856 .31832 .50736 .30916 .4971 Metetra
.55 .676 .921 r
.30916 .4971 .31832 .50736 .33268 .50618 .32355 .49593 Metetra
.549 .677 .922 r
.32355 .49593 .33268 .50618 .34715 .50502 .33804 .49479 Metetra
.547 .677 .924 r
.33804 .49479 .34715 .50502 .36173 .50386 .35265 .49366 Metetra
.546 .679 .925 r
.35265 .49366 .36173 .50386 .37643 .50271 .36738 .49254 Metetra
.545 .68 .927 r
.36738 .49254 .37643 .50271 .39123 .50156 .38222 .49141 Metetra
.545 .681 .928 r
.38222 .49141 .39123 .50156 .40616 .50039 .39717 .49028 Metetra
.544 .683 .93 r
.39717 .49028 .40616 .50039 .4212 .4992 .41225 .48913 Metetra
.544 .685 .931 r
.41225 .48913 .4212 .4992 .43635 .49798 .42744 .48795 Metetra
.544 .687 .932 r
.42744 .48795 .43635 .49798 .45163 .49673 .44276 .48675 Metetra
.544 .689 .934 r
.44276 .48675 .45163 .49673 .46703 .49544 .4582 .4855 Metetra
.544 .691 .935 r
.4582 .4855 .46703 .49544 .48256 .49411 .47377 .48421 Metetra
.545 .694 .937 r
.47377 .48421 .48256 .49411 .4982 .49272 .48946 .48287 Metetra
.546 .696 .938 r
.48946 .48287 .4982 .49272 .51397 .49128 .50527 .48147 Metetra
.547 .699 .939 r
.50527 .48147 .51397 .49128 .52987 .48977 .52122 .48 Metetra
.548 .701 .941 r
.52122 .48 .52987 .48977 .54589 .4882 .53729 .47847 Metetra
.549 .704 .942 r
.53729 .47847 .54589 .4882 .56204 .48655 .55349 .47686 Metetra
.551 .707 .943 r
.55349 .47686 .56204 .48655 .57831 .48482 .56982 .47518 Metetra
.552 .71 .944 r
.56982 .47518 .57831 .48482 .59471 .48302 .58627 .4734 Metetra
.554 .712 .945 r
.58627 .4734 .59471 .48302 .61124 .48113 .60286 .47155 Metetra
.556 .715 .946 r
.60286 .47155 .61124 .48113 .6279 .47914 .61958 .46959 Metetra
.558 .718 .947 r
.61958 .46959 .6279 .47914 .64469 .47707 .63643 .46755 Metetra
.56 .721 .948 r
.63643 .46755 .64469 .47707 .6616 .4749 .65341 .4654 Metetra
.562 .723 .949 r
.65341 .4654 .6616 .4749 .67865 .47263 .67052 .46315 Metetra
.564 .726 .95 r
.67052 .46315 .67865 .47263 .69582 .47027 .68777 .4608 Metetra
.567 .729 .951 r
.68777 .4608 .69582 .47027 .71313 .4678 .70514 .45834 Metetra
.569 .731 .951 r
.70514 .45834 .71313 .4678 .73056 .46523 .72264 .45578 Metetra
.571 .734 .952 r
.72264 .45578 .73056 .46523 .74812 .46255 .74028 .4531 Metetra
.574 .736 .952 r
.74028 .4531 .74812 .46255 .76581 .45977 .75805 .45032 Metetra
.576 .738 .953 r
.75805 .45032 .76581 .45977 .78364 .45687 .77595 .44742 Metetra
.578 .741 .953 r
.77595 .44742 .78364 .45687 .80159 .45388 .79398 .44441 Metetra
.581 .743 .953 r
.79398 .44441 .80159 .45388 .81967 .45077 .81215 .44128 Metetra
.583 .745 .954 r
.81215 .44128 .81967 .45077 .83788 .44755 .83044 .43805 Metetra
.586 .747 .954 r
.83044 .43805 .83788 .44755 .85623 .44423 .84887 .4347 Metetra
.588 .749 .954 r
.84887 .4347 .85623 .44423 .8747 .4408 .86743 .43124 Metetra
.561 .674 .914 r
.2294 .49308 .23878 .5035 .25265 .50212 .2433 .49168 Metetra
.558 .673 .915 r
.2433 .49168 .25265 .50212 .26663 .50079 .2573 .49035 Metetra
.556 .673 .916 r
.2573 .49035 .26663 .50079 .2807 .49952 .2714 .48907 Metetra
.553 .673 .917 r
.2714 .48907 .2807 .49952 .29488 .49829 .2856 .48784 Metetra
.551 .673 .919 r
.2856 .48784 .29488 .49829 .30916 .4971 .29991 .48666 Metetra
.549 .673 .92 r
.29991 .48666 .30916 .4971 .32355 .49593 .31433 .48551 Metetra
.547 .674 .921 r
.31433 .48551 .32355 .49593 .33804 .49479 .32885 .48439 Metetra
.545 .675 .923 r
.32885 .48439 .33804 .49479 .35265 .49366 .34349 .48328 Metetra
.544 .676 .924 r
.34349 .48328 .35265 .49366 .36738 .49254 .35824 .48219 Metetra
.543 .677 .926 r
.35824 .48219 .36738 .49254 .38222 .49141 .37312 .4811 Metetra
.542 .679 .927 r
.37312 .4811 .38222 .49141 .39717 .49028 .3881 .48001 Metetra
.541 .68 .929 r
.3881 .48001 .39717 .49028 .41225 .48913 .40321 .4789 Metetra
.541 .682 .93 r
.40321 .4789 .41225 .48913 .42744 .48795 .41844 .47777 Metetra
.541 .684 .932 r
.41844 .47777 .42744 .48795 .44276 .48675 .4338 .47661 Metetra
.541 .687 .934 r
.4338 .47661 .44276 .48675 .4582 .4855 .44928 .47541 Metetra
.541 .689 .935 r
.44928 .47541 .4582 .4855 .47377 .48421 .46489 .47417 Metetra
.542 .692 .937 r
.46489 .47417 .47377 .48421 .48946 .48287 .48062 .47288 Metetra
.543 .694 .938 r
.48062 .47288 .48946 .48287 .50527 .48147 .49648 .47153 Metetra
.544 .697 .94 r
.49648 .47153 .50527 .48147 .52122 .48 .51247 .47012 Metetra
.545 .7 .941 r
.51247 .47012 .52122 .48 .53729 .47847 .52859 .46863 Metetra
.546 .702 .942 r
.52859 .46863 .53729 .47847 .55349 .47686 .54484 .46707 Metetra
.548 .705 .944 r
.54484 .46707 .55349 .47686 .56982 .47518 .56123 .46543 Metetra
.549 .708 .945 r
.56123 .46543 .56982 .47518 .58627 .4734 .57774 .4637 Metetra
.551 .711 .946 r
.57774 .4637 .58627 .4734 .60286 .47155 .59439 .46188 Metetra
.553 .714 .947 r
.59439 .46188 .60286 .47155 .61958 .46959 .61117 .45996 Metetra
.555 .717 .948 r
.61117 .45996 .61958 .46959 .63643 .46755 .62808 .45795 Metetra
.557 .72 .949 r
.62808 .45795 .63643 .46755 .65341 .4654 .64513 .45583 Metetra
.559 .723 .95 r
.64513 .45583 .65341 .4654 .67052 .46315 .66231 .4536 Metetra
.562 .726 .951 r
.66231 .4536 .67052 .46315 .68777 .4608 .67962 .45127 Metetra
.564 .729 .952 r
.67962 .45127 .68777 .4608 .70514 .45834 .69706 .44883 Metetra
.567 .731 .952 r
.69706 .44883 .70514 .45834 .72264 .45578 .71464 .44627 Metetra
.569 .734 .953 r
.71464 .44627 .72264 .45578 .74028 .4531 .73236 .4436 Metetra
.571 .736 .953 r
.73236 .4436 .74028 .4531 .75805 .45032 .7502 .44081 Metetra
.574 .739 .954 r
.7502 .44081 .75805 .45032 .77595 .44742 .76818 .43791 Metetra
.577 .741 .954 r
.76818 .43791 .77595 .44742 .79398 .44441 .78629 .43489 Metetra
.579 .743 .955 r
.78629 .43489 .79398 .44441 .81215 .44128 .80454 .43175 Metetra
.582 .746 .955 r
.80454 .43175 .81215 .44128 .83044 .43805 .82292 .42849 Metetra
.584 .748 .955 r
.82292 .42849 .83044 .43805 .84887 .4347 .84144 .42512 Metetra
.587 .75 .955 r
.84144 .42512 .84887 .4347 .86743 .43124 .86009 .42163 Metetra
.561 .672 .912 r
.21995 .48249 .2294 .49308 .2433 .49168 .23388 .48106 Metetra
.558 .671 .914 r
.23388 .48106 .2433 .49168 .2573 .49035 .2479 .47971 Metetra
.555 .671 .915 r
.2479 .47971 .2573 .49035 .2714 .48907 .26202 .47843 Metetra
.552 .67 .916 r
.26202 .47843 .2714 .48907 .2856 .48784 .27625 .4772 Metetra
.549 .67 .917 r
.27625 .4772 .2856 .48784 .29991 .48666 .29059 .47603 Metetra
.547 .671 .919 r
.29059 .47603 .29991 .48666 .31433 .48551 .30503 .47489 Metetra
.545 .671 .92 r
.30503 .47489 .31433 .48551 .32885 .48439 .31958 .47379 Metetra
.543 .672 .922 r
.31958 .47379 .32885 .48439 .34349 .48328 .33425 .47272 Metetra
.542 .673 .923 r
.33425 .47272 .34349 .48328 .35824 .48219 .34903 .47166 Metetra
.54 .674 .925 r
.34903 .47166 .35824 .48219 .37312 .4811 .36393 .47061 Metetra
.539 .676 .926 r
.36393 .47061 .37312 .4811 .3881 .48001 .37895 .46956 Metetra
.539 .678 .928 r
.37895 .46956 .3881 .48001 .40321 .4789 .39409 .4685 Metetra
.538 .68 .93 r
.39409 .4685 .40321 .4789 .41844 .47777 .40936 .46743 Metetra
.538 .682 .931 r
.40936 .46743 .41844 .47777 .4338 .47661 .42475 .46632 Metetra
.538 .684 .933 r
.42475 .46632 .4338 .47661 .44928 .47541 .44027 .46518 Metetra
.538 .686 .935 r
.44027 .46518 .44928 .47541 .46489 .47417 .45592 .464 Metetra
.538 .689 .936 r
.45592 .464 .46489 .47417 .48062 .47288 .47169 .46276 Metetra
.539 .692 .938 r
.47169 .46276 .48062 .47288 .49648 .47153 .4876 .46147 Metetra
.54 .695 .94 r
.4876 .46147 .49648 .47153 .51247 .47012 .50364 .46011 Metetra
.541 .698 .941 r
.50364 .46011 .51247 .47012 .52859 .46863 .51981 .45868 Metetra
.543 .701 .943 r
.51981 .45868 .52859 .46863 .54484 .46707 .53611 .45717 Metetra
.544 .704 .944 r
.53611 .45717 .54484 .46707 .56123 .46543 .55255 .45558 Metetra
.546 .707 .946 r
.55255 .45558 .56123 .46543 .57774 .4637 .56912 .4539 Metetra
.548 .71 .947 r
.56912 .4539 .57774 .4637 .59439 .46188 .58582 .45212 Metetra
.55 .713 .948 r
.58582 .45212 .59439 .46188 .61117 .45996 .60266 .45025 Metetra
.552 .716 .949 r
.60266 .45025 .61117 .45996 .62808 .45795 .61964 .44827 Metetra
.554 .72 .95 r
.61964 .44827 .62808 .45795 .64513 .45583 .63675 .44618 Metetra
.556 .723 .951 r
.63675 .44618 .64513 .45583 .66231 .4536 .654 .44399 Metetra
.559 .726 .952 r
.654 .44399 .66231 .4536 .67962 .45127 .67138 .44168 Metetra
.561 .729 .953 r
.67138 .44168 .67962 .45127 .69706 .44883 .6889 .43925 Metetra
.564 .731 .954 r
.6889 .43925 .69706 .44883 .71464 .44627 .70655 .43671 Metetra
.566 .734 .954 r
.70655 .43671 .71464 .44627 .73236 .4436 .72434 .43405 Metetra
.569 .737 .955 r
.72434 .43405 .73236 .4436 .7502 .44081 .74226 .43126 Metetra
.572 .739 .955 r
.74226 .43126 .7502 .44081 .76818 .43791 .76032 .42835 Metetra
.574 .742 .956 r
.76032 .42835 .76818 .43791 .78629 .43489 .77852 .42532 Metetra
.577 .744 .956 r
.77852 .42532 .78629 .43489 .80454 .43175 .79685 .42217 Metetra
.58 .747 .956 r
.79685 .42217 .80454 .43175 .82292 .42849 .81531 .4189 Metetra
.582 .749 .956 r
.81531 .4189 .82292 .42849 .84144 .42512 .83392 .4155 Metetra
.585 .751 .956 r
.83392 .4155 .84144 .42512 .86009 .42163 .85265 .41197 Metetra
.561 .67 .911 r
.21043 .4717 .21995 .48249 .23388 .48106 .22438 .47024 Metetra
.557 .669 .912 r
.22438 .47024 .23388 .48106 .2479 .47971 .23842 .46888 Metetra
.554 .668 .913 r
.23842 .46888 .2479 .47971 .26202 .47843 .25257 .46758 Metetra
.551 .668 .914 r
.25257 .46758 .26202 .47843 .27625 .4772 .26682 .46636 Metetra
.548 .667 .916 r
.26682 .46636 .27625 .4772 .29059 .47603 .28118 .46519 Metetra
.546 .668 .917 r
.28118 .46519 .29059 .47603 .30503 .47489 .29565 .46407 Metetra
.543 .668 .919 r
.29565 .46407 .30503 .47489 .31958 .47379 .31023 .463 Metetra
.541 .669 .92 r
.31023 .463 .31958 .47379 .33425 .47272 .32492 .46196 Metetra
.539 .67 .922 r
.32492 .46196 .33425 .47272 .34903 .47166 .33973 .46094 Metetra
.538 .671 .924 r
.33973 .46094 .34903 .47166 .36393 .47061 .35466 .45994 Metetra
.537 .673 .925 r
.35466 .45994 .36393 .47061 .37895 .46956 .36971 .45893 Metetra
.536 .674 .927 r
.36971 .45893 .37895 .46956 .39409 .4685 .38488 .45793 Metetra
.535 .676 .929 r
.38488 .45793 .39409 .4685 .40936 .46743 .40018 .45691 Metetra
.534 .679 .931 r
.40018 .45691 .40936 .46743 .42475 .46632 .41561 .45586 Metetra
.534 .681 .933 r
.41561 .45586 .42475 .46632 .44027 .46518 .43117 .45479 Metetra
.534 .684 .934 r
.43117 .45479 .44027 .46518 .45592 .464 .44685 .45367 Metetra
.535 .686 .936 r
.44685 .45367 .45592 .464 .47169 .46276 .46267 .4525 Metetra
.535 .689 .938 r
.46267 .4525 .47169 .46276 .4876 .46147 .47862 .45127 Metetra
.536 .692 .94 r
.47862 .45127 .4876 .46147 .50364 .46011 .49471 .44998 Metetra
.537 .696 .942 r
.49471 .44998 .50364 .46011 .51981 .45868 .51093 .44862 Metetra
.539 .699 .943 r
.51093 .44862 .51981 .45868 .53611 .45717 .52728 .44717 Metetra
.54 .702 .945 r
.52728 .44717 .53611 .45717 .55255 .45558 .54377 .44564 Metetra
.542 .705 .946 r
.54377 .44564 .55255 .45558 .56912 .4539 .5604 .44401 Metetra
.544 .709 .948 r
.5604 .44401 .56912 .4539 .58582 .45212 .57716 .44229 Metetra
.546 .712 .949 r
.57716 .44229 .58582 .45212 .60266 .45025 .59406 .44046 Metetra
.548 .716 .95 r
.59406 .44046 .60266 .45025 .61964 .44827 .6111 .43852 Metetra
.55 .719 .951 r
.6111 .43852 .61964 .44827 .63675 .44618 .62828 .43648 Metetra
.553 .722 .952 r
.62828 .43648 .63675 .44618 .654 .44399 .64559 .43431 Metetra
.556 .725 .953 r
.64559 .43431 .654 .44399 .67138 .44168 .66305 .43203 Metetra
.558 .729 .954 r
.66305 .43203 .67138 .44168 .6889 .43925 .68064 .42963 Metetra
.561 .732 .955 r
.68064 .42963 .6889 .43925 .70655 .43671 .69837 .4271 Metetra
.564 .735 .956 r
.69837 .4271 .70655 .43671 .72434 .43405 .71623 .42445 Metetra
.567 .737 .956 r
.71623 .42445 .72434 .43405 .74226 .43126 .73423 .42167 Metetra
.569 .74 .957 r
.73423 .42167 .74226 .43126 .76032 .42835 .75237 .41876 Metetra
.572 .743 .957 r
.75237 .41876 .76032 .42835 .77852 .42532 .77065 .41572 Metetra
.575 .745 .957 r
.77065 .41572 .77852 .42532 .79685 .42217 .78907 .41256 Metetra
.578 .748 .958 r
.78907 .41256 .79685 .42217 .81531 .4189 .80762 .40926 Metetra
.581 .75 .958 r
.80762 .40926 .81531 .4189 .83392 .4155 .82631 .40583 Metetra
.584 .752 .958 r
.82631 .40583 .83392 .4155 .85265 .41197 .84514 .40228 Metetra
.561 .667 .909 r
.20082 .4607 .21043 .4717 .22438 .47024 .2148 .45922 Metetra
.557 .666 .91 r
.2148 .45922 .22438 .47024 .23842 .46888 .22887 .45782 Metetra
.553 .665 .911 r
.22887 .45782 .23842 .46888 .25257 .46758 .24304 .45652 Metetra
.55 .665 .913 r
.24304 .45652 .25257 .46758 .26682 .46636 .25732 .45529 Metetra
.547 .664 .914 r
.25732 .45529 .26682 .46636 .28118 .46519 .2717 .45413 Metetra
.544 .664 .915 r
.2717 .45413 .28118 .46519 .29565 .46407 .28619 .45304 Metetra
.541 .665 .917 r
.28619 .45304 .29565 .46407 .31023 .463 .30079 .45199 Metetra
.539 .665 .919 r
.30079 .45199 .31023 .463 .32492 .46196 .31551 .45098 Metetra
.537 .666 .92 r
.31551 .45098 .32492 .46196 .33973 .46094 .33035 .45001 Metetra
.535 .668 .922 r
.33035 .45001 .33973 .46094 .35466 .45994 .34531 .44905 Metetra
.533 .669 .924 r
.34531 .44905 .35466 .45994 .36971 .45893 .36038 .44811 Metetra
.532 .671 .926 r
.36038 .44811 .36971 .45893 .38488 .45793 .37559 .44716 Metetra
.531 .673 .928 r
.37559 .44716 .38488 .45793 .40018 .45691 .39092 .44621 Metetra
.531 .675 .93 r
.39092 .44621 .40018 .45691 .41561 .45586 .40638 .44523 Metetra
.53 .678 .932 r
.40638 .44523 .41561 .45586 .43117 .45479 .42198 .44423 Metetra
.53 .681 .934 r
.42198 .44423 .43117 .45479 .44685 .45367 .4377 .44319 Metetra
.531 .684 .936 r
.4377 .44319 .44685 .45367 .46267 .4525 .45356 .44209 Metetra
.531 .687 .938 r
.45356 .44209 .46267 .4525 .47862 .45127 .46955 .44094 Metetra
.532 .69 .94 r
.46955 .44094 .47862 .45127 .49471 .44998 .48568 .43972 Metetra
.533 .693 .942 r
.48568 .43972 .49471 .44998 .51093 .44862 .50195 .43843 Metetra
.534 .697 .944 r
.50195 .43843 .51093 .44862 .52728 .44717 .51835 .43706 Metetra
.536 .7 .945 r
.51835 .43706 .52728 .44717 .54377 .44564 .5349 .43559 Metetra
.538 .704 .947 r
.5349 .43559 .54377 .44564 .5604 .44401 .55158 .43403 Metetra
.54 .707 .948 r
.55158 .43403 .5604 .44401 .57716 .44229 .5684 .43237 Metetra
.542 .711 .95 r
.5684 .43237 .57716 .44229 .59406 .44046 .58537 .4306 Metetra
.544 .715 .951 r
.58537 .4306 .59406 .44046 .6111 .43852 .60247 .42871 Metetra
.547 .718 .953 r
.60247 .42871 .6111 .43852 .62828 .43648 .61971 .42671 Metetra
.549 .722 .954 r
.61971 .42671 .62828 .43648 .64559 .43431 .6371 .42459 Metetra
.552 .725 .955 r
.6371 .42459 .64559 .43431 .66305 .43203 .65462 .42234 Metetra
.555 .729 .956 r
.65462 .42234 .66305 .43203 .68064 .42963 .67228 .41997 Metetra
.558 .732 .957 r
.67228 .41997 .68064 .42963 .69837 .4271 .69009 .41746 Metetra
.561 .735 .957 r
.69009 .41746 .69837 .4271 .71623 .42445 .70803 .41482 Metetra
.564 .738 .958 r
.70803 .41482 .71623 .42445 .73423 .42167 .72611 .41205 Metetra
.567 .741 .958 r
.72611 .41205 .73423 .42167 .75237 .41876 .74434 .40914 Metetra
.57 .744 .959 r
.74434 .40914 .75237 .41876 .77065 .41572 .7627 .40609 Metetra
.573 .746 .959 r
.7627 .40609 .77065 .41572 .78907 .41256 .7812 .40291 Metetra
.576 .749 .959 r
.7812 .40291 .78907 .41256 .80762 .40926 .79984 .3996 Metetra
.579 .751 .959 r
.79984 .3996 .80762 .40926 .82631 .40583 .81862 .39615 Metetra
.582 .754 .959 r
.81862 .39615 .82631 .40583 .84514 .40228 .83754 .39256 Metetra
.561 .665 .907 r
.19115 .44949 .20082 .4607 .2148 .45922 .20515 .44796 Metetra
.557 .663 .908 r
.20515 .44796 .2148 .45922 .22887 .45782 .21924 .44654 Metetra
.553 .662 .909 r
.21924 .44654 .22887 .45782 .24304 .45652 .23344 .44522 Metetra
.549 .662 .91 r
.23344 .44522 .24304 .45652 .25732 .45529 .24774 .44399 Metetra
.546 .661 .912 r
.24774 .44399 .25732 .45529 .2717 .45413 .26214 .44284 Metetra
.542 .661 .913 r
.26214 .44284 .2717 .45413 .28619 .45304 .27666 .44176 Metetra
.539 .661 .915 r
.27666 .44176 .28619 .45304 .30079 .45199 .29128 .44074 Metetra
.537 .662 .917 r
.29128 .44074 .30079 .45199 .31551 .45098 .30602 .43978 Metetra
.534 .663 .919 r
.30602 .43978 .31551 .45098 .33035 .45001 .32088 .43885 Metetra
.532 .664 .921 r
.32088 .43885 .33035 .45001 .34531 .44905 .33587 .43795 Metetra
.53 .665 .923 r
.33587 .43795 .34531 .44905 .36038 .44811 .35097 .43707 Metetra
.529 .667 .925 r
.35097 .43707 .36038 .44811 .37559 .44716 .36621 .43619 Metetra
.528 .669 .927 r
.36621 .43619 .37559 .44716 .39092 .44621 .38157 .43531 Metetra
.527 .672 .929 r
.38157 .43531 .39092 .44621 .40638 .44523 .39706 .43442 Metetra
.526 .674 .931 r
.39706 .43442 .40638 .44523 .42198 .44423 .41269 .4335 Metetra
.526 .677 .934 r
.41269 .4335 .42198 .44423 .4377 .44319 .42845 .43254 Metetra
.526 .68 .936 r
.42845 .43254 .4377 .44319 .45356 .44209 .44435 .43153 Metetra
.527 .684 .938 r
.44435 .43153 .45356 .44209 .46955 .44094 .46039 .43046 Metetra
.527 .687 .94 r
.46039 .43046 .46955 .44094 .48568 .43972 .47656 .42933 Metetra
.528 .691 .942 r
.47656 .42933 .48568 .43972 .50195 .43843 .49288 .42813 Metetra
.53 .694 .944 r
.49288 .42813 .50195 .43843 .51835 .43706 .50933 .42683 Metetra
.531 .698 .946 r
.50933 .42683 .51835 .43706 .5349 .43559 .52593 .42545 Metetra
.533 .702 .948 r
.52593 .42545 .5349 .43559 .55158 .43403 .54267 .42396 Metetra
.535 .706 .949 r
.54267 .42396 .55158 .43403 .5684 .43237 .55955 .42237 Metetra
.537 .71 .951 r
.55955 .42237 .5684 .43237 .58537 .4306 .57657 .42067 Metetra
.54 .714 .953 r
.57657 .42067 .58537 .4306 .60247 .42871 .59374 .41884 Metetra
.542 .718 .954 r
.59374 .41884 .60247 .42871 .61971 .42671 .61105 .4169 Metetra
.545 .721 .955 r
.61105 .4169 .61971 .42671 .6371 .42459 .6285 .41482 Metetra
.548 .725 .956 r
.6285 .41482 .6371 .42459 .65462 .42234 .6461 .41261 Metetra
.551 .729 .957 r
.6461 .41261 .65462 .42234 .67228 .41997 .66383 .41027 Metetra
.554 .732 .958 r
.66383 .41027 .67228 .41997 .69009 .41746 .68171 .40779 Metetra
.557 .736 .959 r
.68171 .40779 .69009 .41746 .70803 .41482 .69974 .40517 Metetra
.561 .739 .96 r
.69974 .40517 .70803 .41482 .72611 .41205 .7179 .4024 Metetra
.564 .742 .96 r
.7179 .4024 .72611 .41205 .74434 .40914 .7362 .3995 Metetra
.567 .745 .961 r
.7362 .3995 .74434 .40914 .7627 .40609 .75465 .39645 Metetra
.57 .748 .961 r
.75465 .39645 .7627 .40609 .7812 .40291 .77324 .39326 Metetra
.574 .751 .961 r
.77324 .39326 .7812 .40291 .79984 .3996 .79197 .38992 Metetra
.577 .753 .961 r
.79197 .38992 .79984 .3996 .81862 .39615 .81084 .38644 Metetra
.58 .755 .961 r
.81084 .38644 .81862 .39615 .83754 .39256 .82985 .38282 Metetra
.562 .662 .905 r
.1814 .43804 .19115 .44949 .20515 .44796 .19543 .43647 Metetra
.557 .661 .906 r
.19543 .43647 .20515 .44796 .21924 .44654 .20955 .43502 Metetra
.553 .659 .907 r
.20955 .43502 .21924 .44654 .23344 .44522 .22376 .43368 Metetra
.549 .658 .908 r
.22376 .43368 .23344 .44522 .24774 .44399 .23808 .43244 Metetra
.545 .657 .91 r
.23808 .43244 .24774 .44399 .26214 .44284 .25251 .43129 Metetra
.541 .657 .911 r
.25251 .43129 .26214 .44284 .27666 .44176 .26704 .43023 Metetra
.538 .657 .913 r
.26704 .43023 .27666 .44176 .29128 .44074 .28169 .42924 Metetra
.534 .658 .915 r
.28169 .42924 .29128 .44074 .30602 .43978 .29646 .42832 Metetra
.532 .658 .917 r
.29646 .42832 .30602 .43978 .32088 .43885 .31134 .42744 Metetra
.529 .659 .919 r
.31134 .42744 .32088 .43885 .33587 .43795 .32635 .4266 Metetra
.527 .661 .921 r
.32635 .4266 .33587 .43795 .35097 .43707 .34148 .42579 Metetra
.525 .663 .923 r
.34148 .42579 .35097 .43707 .36621 .43619 .35674 .425 Metetra
.524 .665 .926 r
.35674 .425 .36621 .43619 .38157 .43531 .37213 .4242 Metetra
.522 .667 .928 r
.37213 .4242 .38157 .43531 .39706 .43442 .38766 .4234 Metetra
.522 .67 .93 r
.38766 .4234 .39706 .43442 .41269 .4335 .40332 .42257 Metetra
.521 .673 .933 r
.40332 .42257 .41269 .4335 .42845 .43254 .41911 .42171 Metetra
.521 .677 .935 r
.41911 .42171 .42845 .43254 .44435 .43153 .43505 .4208 Metetra
.521 .68 .938 r
.43505 .4208 .44435 .43153 .46039 .43046 .45113 .41984 Metetra
.522 .684 .94 r
.45113 .41984 .46039 .43046 .47656 .42933 .46734 .41881 Metetra
.523 .688 .942 r
.46734 .41881 .47656 .42933 .49288 .42813 .4837 .4177 Metetra
.524 .692 .944 r
.4837 .4177 .49288 .42813 .50933 .42683 .50021 .4165 Metetra
.526 .696 .947 r
.50021 .4165 .50933 .42683 .52593 .42545 .51686 .41521 Metetra
.528 .7 .949 r
.51686 .41521 .52593 .42545 .54267 .42396 .53365 .41381 Metetra
.53 .704 .951 r
.53365 .41381 .54267 .42396 .55955 .42237 .55059 .4123 Metetra
.532 .709 .952 r
.55059 .4123 .55955 .42237 .57657 .42067 .56768 .41068 Metetra
.535 .713 .954 r
.56768 .41068 .57657 .42067 .59374 .41884 .58491 .40892 Metetra
.537 .717 .956 r
.58491 .40892 .59374 .41884 .61105 .4169 .60228 .40704 Metetra
.54 .721 .957 r
.60228 .40704 .61105 .4169 .6285 .41482 .61981 .40502 Metetra
.543 .725 .958 r
.61981 .40502 .6285 .41482 .6461 .41261 .63747 .40286 Metetra
.547 .729 .959 r
.63747 .40286 .6461 .41261 .66383 .41027 .65528 .40056 Metetra
.55 .733 .96 r
.65528 .40056 .66383 .41027 .68171 .40779 .67324 .39811 Metetra
.554 .737 .961 r
.67324 .39811 .68171 .40779 .69974 .40517 .69134 .39551 Metetra
.557 .74 .962 r
.69134 .39551 .69974 .40517 .7179 .4024 .70959 .39276 Metetra
.561 .743 .962 r
.70959 .39276 .7179 .4024 .7362 .3995 .72798 .38986 Metetra
.564 .747 .963 r
.72798 .38986 .7362 .3995 .75465 .39645 .74651 .3868 Metetra
.568 .75 .963 r
.74651 .3868 .75465 .39645 .77324 .39326 .76519 .3836 Metetra
.571 .752 .963 r
.76519 .3836 .77324 .39326 .79197 .38992 .78401 .38024 Metetra
.575 .755 .963 r
.78401 .38024 .79197 .38992 .81084 .38644 .80297 .37673 Metetra
.578 .757 .963 r
.80297 .37673 .81084 .38644 .82985 .38282 .82207 .37307 Metetra
.563 .66 .902 r
.17159 .42635 .1814 .43804 .19543 .43647 .18564 .42472 Metetra
.558 .658 .903 r
.18564 .42472 .19543 .43647 .20955 .43502 .19978 .42323 Metetra
.553 .656 .904 r
.19978 .42323 .20955 .43502 .22376 .43368 .21402 .42186 Metetra
.548 .655 .905 r
.21402 .42186 .22376 .43368 .23808 .43244 .22836 .42061 Metetra
.544 .654 .907 r
.22836 .42061 .23808 .43244 .25251 .43129 .2428 .41947 Metetra
.54 .653 .908 r
.2428 .41947 .25251 .43129 .26704 .43023 .25736 .41842 Metetra
.536 .653 .91 r
.25736 .41842 .26704 .43023 .28169 .42924 .27203 .41746 Metetra
.532 .653 .912 r
.27203 .41746 .28169 .42924 .29646 .42832 .28681 .41658 Metetra
.529 .654 .914 r
.28681 .41658 .29646 .42832 .31134 .42744 .30172 .41577 Metetra
.526 .655 .917 r
.30172 .41577 .31134 .42744 .32635 .4266 .31674 .415 Metetra
.523 .656 .919 r
.31674 .415 .32635 .4266 .34148 .42579 .3319 .41427 Metetra
.521 .658 .921 r
.3319 .41427 .34148 .42579 .35674 .425 .34719 .41356 Metetra
.519 .66 .924 r
.34719 .41356 .35674 .425 .37213 .4242 .3626 .41286 Metetra
.518 .663 .927 r
.3626 .41286 .37213 .4242 .38766 .4234 .37816 .41216 Metetra
.517 .666 .929 r
.37816 .41216 .38766 .4234 .40332 .42257 .39385 .41144 Metetra
.516 .669 .932 r
.39385 .41144 .40332 .42257 .41911 .42171 .40968 .41069 Metetra
.516 .672 .935 r
.40968 .41069 .41911 .42171 .43505 .4208 .42565 .4099 Metetra
.516 .676 .937 r
.42565 .4099 .43505 .4208 .45113 .41984 .44177 .40905 Metetra
.516 .68 .94 r
.44177 .40905 .45113 .41984 .46734 .41881 .45803 .40814 Metetra
.517 .684 .942 r
.45803 .40814 .46734 .41881 .4837 .4177 .47443 .40715 Metetra
.518 .689 .945 r
.47443 .40715 .4837 .4177 .50021 .4165 .49098 .40606 Metetra
.52 .693 .947 r
.49098 .40606 .50021 .4165 .51686 .41521 .50769 .40488 Metetra
.522 .698 .95 r
.50769 .40488 .51686 .41521 .53365 .41381 .52453 .40358 Metetra
.524 .703 .952 r
.52453 .40358 .53365 .41381 .55059 .4123 .54153 .40217 Metetra
.526 .707 .954 r
.54153 .40217 .55059 .4123 .56768 .41068 .55868 .40064 Metetra
.529 .712 .956 r
.55868 .40064 .56768 .41068 .58491 .40892 .57597 .39897 Metetra
.532 .716 .957 r
.57597 .39897 .58491 .40892 .60228 .40704 .59342 .39716 Metetra
.535 .721 .959 r
.59342 .39716 .60228 .40704 .61981 .40502 .61101 .3952 Metetra
.538 .725 .96 r
.61101 .3952 .61981 .40502 .63747 .40286 .62875 .3931 Metetra
.542 .73 .961 r
.62875 .3931 .63747 .40286 .65528 .40056 .64664 .39084 Metetra
.546 .734 .962 r
.64664 .39084 .65528 .40056 .67324 .39811 .66467 .38843 Metetra
.549 .738 .963 r
.66467 .38843 .67324 .39811 .69134 .39551 .68286 .38586 Metetra
.553 .742 .964 r
.68286 .38586 .69134 .39551 .70959 .39276 .70118 .38312 Metetra
.557 .745 .965 r
.70118 .38312 .70959 .39276 .72798 .38986 .71966 .38023 Metetra
.561 .748 .965 r
.71966 .38023 .72798 .38986 .74651 .3868 .73828 .37717 Metetra
.565 .752 .965 r
.73828 .37717 .74651 .3868 .76519 .3836 .75705 .37395 Metetra
.569 .755 .965 r
.75705 .37395 .76519 .3836 .78401 .38024 .77596 .37057 Metetra
.573 .757 .965 r
.77596 .37057 .78401 .38024 .80297 .37673 .79501 .36703 Metetra
.576 .76 .965 r
.79501 .36703 .80297 .37673 .82207 .37307 .81421 .36333 Metetra
.564 .657 .899 r
.1617 .41441 .17159 .42635 .18564 .42472 .17577 .4127 Metetra
.559 .655 .9 r
.17577 .4127 .18564 .42472 .19978 .42323 .18994 .41115 Metetra
.553 .652 .901 r
.18994 .41115 .19978 .42323 .21402 .42186 .2042 .40975 Metetra
.548 .651 .902 r
.2042 .40975 .21402 .42186 .22836 .42061 .21856 .40848 Metetra
.543 .649 .904 r
.21856 .40848 .22836 .42061 .2428 .41947 .23303 .40733 Metetra
.538 .648 .905 r
.23303 .40733 .2428 .41947 .25736 .41842 .2476 .4063 Metetra
.534 .648 .907 r
.2476 .4063 .25736 .41842 .27203 .41746 .26229 .40538 Metetra
.53 .648 .909 r
.26229 .40538 .27203 .41746 .28681 .41658 .27709 .40454 Metetra
.526 .648 .912 r
.27709 .40454 .28681 .41658 .30172 .41577 .29202 .40379 Metetra
.523 .649 .914 r
.29202 .40379 .30172 .41577 .31674 .415 .30706 .4031 Metetra
.52 .651 .917 r
.30706 .4031 .31674 .415 .3319 .41427 .32224 .40245 Metetra
.517 .652 .919 r
.32224 .40245 .3319 .41427 .34719 .41356 .33755 .40185 Metetra
.514 .655 .922 r
.33755 .40185 .34719 .41356 .3626 .41286 .35299 .40126 Metetra
.513 .657 .925 r
.35299 .40126 .3626 .41286 .37816 .41216 .36857 .40068 Metetra
.511 .66 .928 r
.36857 .40068 .37816 .41216 .39385 .41144 .38429 .40009 Metetra
.51 .664 .931 r
.38429 .40009 .39385 .41144 .40968 .41069 .40015 .39947 Metetra
.509 .668 .934 r
.40015 .39947 .40968 .41069 .42565 .4099 .41616 .39881 Metetra
.509 .672 .937 r
.41616 .39881 .42565 .4099 .44177 .40905 .43231 .3981 Metetra
.51 .676 .94 r
.43231 .3981 .44177 .40905 .45803 .40814 .44861 .39732 Metetra
.51 .681 .943 r
.44861 .39732 .45803 .40814 .47443 .40715 .46506 .39647 Metetra
.511 .685 .945 r
.46506 .39647 .47443 .40715 .49098 .40606 .48166 .39551 Metetra
.513 .69 .948 r
.48166 .39551 .49098 .40606 .50769 .40488 .49841 .39446 Metetra
.515 .695 .951 r
.49841 .39446 .50769 .40488 .52453 .40358 .51531 .39328 Metetra
.517 .7 .953 r
.51531 .39328 .52453 .40358 .54153 .40217 .53237 .39199 Metetra
.52 .706 .955 r
.53237 .39199 .54153 .40217 .55868 .40064 .54957 .39056 Metetra
.522 .711 .957 r
.54957 .39056 .55868 .40064 .57597 .39897 .56693 .38899 Metetra
.526 .716 .959 r
.56693 .38899 .57597 .39897 .59342 .39716 .58444 .38726 Metetra
.529 .721 .961 r
.58444 .38726 .59342 .39716 .61101 .3952 .60211 .38539 Metetra
.533 .726 .963 r
.60211 .38539 .61101 .3952 .62875 .3931 .61992 .38335 Metetra
.536 .73 .964 r
.61992 .38335 .62875 .3931 .64664 .39084 .63789 .38115 Metetra
.54 .735 .965 r
.63789 .38115 .64664 .39084 .66467 .38843 .656 .37878 Metetra
.545 .739 .966 r
.656 .37878 .66467 .38843 .68286 .38586 .67427 .37624 Metetra
.549 .743 .967 r
.67427 .37624 .68286 .38586 .70118 .38312 .69268 .37353 Metetra
.553 .747 .967 r
.69268 .37353 .70118 .38312 .71966 .38023 .71124 .37064 Metetra
.557 .751 .968 r
.71124 .37064 .71966 .38023 .73828 .37717 .72995 .36758 Metetra
.562 .754 .968 r
.72995 .36758 .73828 .37717 .75705 .37395 .74881 .36435 Metetra
.566 .757 .968 r
.74881 .36435 .75705 .37395 .77596 .37057 .76781 .36094 Metetra
.57 .76 .968 r
.76781 .36094 .77596 .37057 .79501 .36703 .78696 .35737 Metetra
.574 .763 .968 r
.78696 .35737 .79501 .36703 .81421 .36333 .80625 .35362 Metetra
.566 .655 .896 r
.15175 .40218 .1617 .41441 .17577 .4127 .16585 .40039 Metetra
.56 .651 .897 r
.16585 .40039 .17577 .4127 .18994 .41115 .18004 .39877 Metetra
.554 .649 .897 r
.18004 .39877 .18994 .41115 .2042 .40975 .19432 .39731 Metetra
.548 .647 .899 r
.19432 .39731 .2042 .40975 .21856 .40848 .2087 .39601 Metetra
.543 .645 .9 r
.2087 .39601 .21856 .40848 .23303 .40733 .22319 .39485 Metetra
.538 .643 .902 r
.22319 .39485 .23303 .40733 .2476 .4063 .23778 .39384 Metetra
.532 .642 .904 r
.23778 .39384 .2476 .4063 .26229 .40538 .25248 .39294 Metetra
.528 .642 .906 r
.25248 .39294 .26229 .40538 .27709 .40454 .2673 .39216 Metetra
.523 .642 .908 r
.2673 .39216 .27709 .40454 .29202 .40379 .28224 .39147 Metetra
.519 .643 .911 r
.28224 .39147 .29202 .40379 .30706 .4031 .29731 .39086 Metetra
.515 .644 .914 r
.29731 .39086 .30706 .4031 .32224 .40245 .3125 .39032 Metetra
.512 .646 .917 r
.3125 .39032 .32224 .40245 .33755 .40185 .32783 .38983 Metetra
.509 .648 .92 r
.32783 .38983 .33755 .40185 .35299 .40126 .34329 .38938 Metetra
.507 .651 .923 r
.34329 .38938 .35299 .40126 .36857 .40068 .35889 .38893 Metetra
.505 .654 .926 r
.35889 .38893 .36857 .40068 .38429 .40009 .37464 .38849 Metetra
.503 .658 .929 r
.37464 .38849 .38429 .40009 .40015 .39947 .39053 .38802 Metetra
.502 .662 .933 r
.39053 .38802 .40015 .39947 .41616 .39881 .40656 .38752 Metetra
.502 .666 .936 r
.40656 .38752 .41616 .39881 .43231 .3981 .42275 .38697 Metetra
.502 .671 .94 r
.42275 .38697 .43231 .3981 .44861 .39732 .43909 .38635 Metetra
.502 .676 .943 r
.43909 .38635 .44861 .39732 .46506 .39647 .45558 .38565 Metetra
.503 .681 .946 r
.45558 .38565 .46506 .39647 .48166 .39551 .47223 .38486 Metetra
.505 .687 .949 r
.47223 .38486 .48166 .39551 .49841 .39446 .48903 .38395 Metetra
.507 .693 .952 r
.48903 .38395 .49841 .39446 .51531 .39328 .50598 .38292 Metetra
.509 .698 .955 r
.50598 .38292 .51531 .39328 .53237 .39199 .5231 .38176 Metetra
.512 .704 .957 r
.5231 .38176 .53237 .39199 .54957 .39056 .54036 .38046 Metetra
.515 .71 .96 r
.54036 .38046 .54957 .39056 .56693 .38899 .55779 .379 Metetra
.518 .715 .962 r
.55779 .379 .56693 .38899 .58444 .38726 .57537 .37739 Metetra
.522 .721 .964 r
.57537 .37739 .58444 .38726 .60211 .38539 .5931 .3756 Metetra
.526 .726 .965 r
.5931 .3756 .60211 .38539 .61992 .38335 .61099 .37364 Metetra
.53 .731 .967 r
.61099 .37364 .61992 .38335 .63789 .38115 .62903 .37151 Metetra
.535 .736 .968 r
.62903 .37151 .63789 .38115 .656 .37878 .64723 .36919 Metetra
.539 .741 .969 r
.64723 .36919 .656 .37878 .67427 .37624 .66558 .36669 Metetra
.544 .745 .97 r
.66558 .36669 .67427 .37624 .69268 .37353 .68408 .364 Metetra
.549 .75 .97 r
.68408 .364 .69268 .37353 .71124 .37064 .70273 .36112 Metetra
.553 .754 .971 r
.70273 .36112 .71124 .37064 .72995 .36758 .72153 .35806 Metetra
.558 .757 .971 r
.72153 .35806 .72995 .36758 .74881 .36435 .74048 .35481 Metetra
.563 .76 .971 r
.74048 .35481 .74881 .36435 .76781 .36094 .75957 .35138 Metetra
.568 .763 .971 r
.75957 .35138 .76781 .36094 .78696 .35737 .77882 .34776 Metetra
.572 .766 .97 r
.77882 .34776 .78696 .35737 .80625 .35362 .79821 .34396 Metetra
.568 .652 .892 r
.14173 .38966 .15175 .40218 .16585 .40039 .15586 .38776 Metetra
.562 .648 .893 r
.15586 .38776 .16585 .40039 .18004 .39877 .17008 .38605 Metetra
.555 .645 .894 r
.17008 .38605 .18004 .39877 .19432 .39731 .18438 .38452 Metetra
.549 .642 .895 r
.18438 .38452 .19432 .39731 .2087 .39601 .19878 .38317 Metetra
.543 .64 .896 r
.19878 .38317 .2087 .39601 .22319 .39485 .21328 .382 Metetra
.537 .638 .898 r
.21328 .382 .22319 .39485 .23778 .39384 .22789 .38098 Metetra
.531 .636 .899 r
.22789 .38098 .23778 .39384 .25248 .39294 .24261 .38011 Metetra
.526 .636 .902 r
.24261 .38011 .25248 .39294 .2673 .39216 .25744 .37938 Metetra
.521 .636 .904 r
.25744 .37938 .2673 .39216 .28224 .39147 .2724 .37877 Metetra
.516 .636 .907 r
.2724 .37877 .28224 .39147 .29731 .39086 .28748 .37826 Metetra
.511 .637 .91 r
.28748 .37826 .29731 .39086 .3125 .39032 .30268 .37783 Metetra
.507 .639 .913 r
.30268 .37783 .3125 .39032 .32783 .38983 .31803 .37747 Metetra
.504 .641 .917 r
.31803 .37747 .32783 .38983 .34329 .38938 .33351 .37716 Metetra
.501 .644 .92 r
.33351 .37716 .34329 .38938 .35889 .38893 .34913 .37688 Metetra
.498 .647 .924 r
.34913 .37688 .35889 .38893 .37464 .38849 .36489 .3766 Metetra
.496 .651 .928 r
.36489 .3766 .37464 .38849 .39053 .38802 .38081 .37632 Metetra
.495 .655 .931 r
.38081 .37632 .39053 .38802 .40656 .38752 .39687 .37601 Metetra
.494 .66 .935 r
.39687 .37601 .40656 .38752 .42275 .38697 .41309 .37565 Metetra
.493 .665 .939 r
.41309 .37565 .42275 .38697 .43909 .38635 .42947 .37522 Metetra
.493 .671 .943 r
.42947 .37522 .43909 .38635 .45558 .38565 .446 .37471 Metetra
.494 .677 .946 r
.446 .37471 .45558 .38565 .47223 .38486 .46269 .3741 Metetra
.495 .683 .95 r
.46269 .3741 .47223 .38486 .48903 .38395 .47954 .37337 Metetra
.497 .689 .953 r
.47954 .37337 .48903 .38395 .50598 .38292 .49655 .37252 Metetra
.5 .696 .956 r
.49655 .37252 .50598 .38292 .5231 .38176 .51371 .37152 Metetra
.502 .702 .959 r
.51371 .37152 .5231 .38176 .54036 .38046 .53104 .37037 Metetra
.506 .708 .962 r
.53104 .37037 .54036 .38046 .55779 .379 .54853 .36905 Metetra
.509 .715 .964 r
.54853 .36905 .55779 .379 .57537 .37739 .56618 .36755 Metetra
.514 .721 .966 r
.56618 .36755 .57537 .37739 .5931 .3756 .58399 .36588 Metetra
.518 .727 .968 r
.58399 .36588 .5931 .3756 .61099 .37364 .60195 .36401 Metetra
.523 .733 .97 r
.60195 .36401 .61099 .37364 .62903 .37151 .62007 .36195 Metetra
.528 .738 .971 r
.62007 .36195 .62903 .37151 .64723 .36919 .63835 .3597 Metetra
.533 .743 .972 r
.63835 .3597 .64723 .36919 .66558 .36669 .65678 .35724 Metetra
.538 .748 .973 r
.65678 .35724 .66558 .36669 .68408 .364 .67537 .35458 Metetra
.543 .753 .974 r
.67537 .35458 .68408 .364 .70273 .36112 .69411 .35171 Metetra
.549 .757 .974 r
.69411 .35171 .70273 .36112 .72153 .35806 .713 .34864 Metetra
.554 .761 .974 r
.713 .34864 .72153 .35806 .74048 .35481 .73205 .34537 Metetra
.559 .764 .974 r
.73205 .34537 .74048 .35481 .75957 .35138 .75124 .3419 Metetra
.565 .768 .974 r
.75124 .3419 .75957 .35138 .77882 .34776 .77058 .33824 Metetra
.57 .77 .973 r
.77058 .33824 .77882 .34776 .79821 .34396 .79007 .33437 Metetra
.572 .65 .888 r
.13166 .37684 .14173 .38966 .15586 .38776 .14581 .37479 Metetra
.565 .645 .888 r
.14581 .37479 .15586 .38776 .17008 .38605 .16006 .37296 Metetra
.558 .641 .889 r
.16006 .37296 .17008 .38605 .18438 .38452 .17439 .37134 Metetra
.551 .637 .89 r
.17439 .37134 .18438 .38452 .19878 .38317 .18881 .36992 Metetra
.544 .634 .891 r
.18881 .36992 .19878 .38317 .21328 .382 .20333 .36871 Metetra
.537 .632 .893 r
.20333 .36871 .21328 .382 .22789 .38098 .21795 .36769 Metetra
.53 .63 .894 r
.21795 .36769 .22789 .38098 .24261 .38011 .23268 .36684 Metetra
.524 .629 .897 r
.23268 .36684 .24261 .38011 .25744 .37938 .24753 .36615 Metetra
.518 .628 .899 r
.24753 .36615 .25744 .37938 .2724 .37877 .26249 .36562 Metetra
.512 .628 .902 r
.26249 .36562 .2724 .37877 .28748 .37826 .27758 .36521 Metetra
.507 .629 .905 r
.27758 .36521 .28748 .37826 .30268 .37783 .2928 .36491 Metetra
.502 .63 .909 r
.2928 .36491 .30268 .37783 .31803 .37747 .30815 .3647 Metetra
.498 .632 .913 r
.30815 .3647 .31803 .37747 .33351 .37716 .32364 .36456 Metetra
.494 .635 .917 r
.32364 .36456 .33351 .37716 .34913 .37688 .33928 .36447 Metetra
.49 .639 .921 r
.33928 .36447 .34913 .37688 .36489 .3766 .35506 .3644 Metetra
.488 .643 .925 r
.35506 .3644 .36489 .3766 .38081 .37632 .371 .36433 Metetra
.485 .648 .93 r
.371 .36433 .38081 .37632 .39687 .37601 .38709 .36424 Metetra
.484 .653 .934 r
.38709 .36424 .39687 .37601 .41309 .37565 .40334 .36411 Metetra
.483 .659 .938 r
.40334 .36411 .41309 .37565 .42947 .37522 .41974 .36391 Metetra
.483 .665 .943 r
.41974 .36391 .42947 .37522 .446 .37471 .43631 .36363 Metetra
.483 .671 .947 r
.43631 .36363 .446 .37471 .46269 .3741 .45304 .36325 Metetra
.484 .678 .951 r
.45304 .36325 .46269 .3741 .47954 .37337 .46994 .36274 Metetra
.486 .685 .955 r
.46994 .36274 .47954 .37337 .49655 .37252 .487 .36209 Metetra
.488 .693 .959 r
.487 .36209 .49655 .37252 .51371 .37152 .50422 .36129 Metetra
.491 .7 .962 r
.50422 .36129 .51371 .37152 .53104 .37037 .52161 .36031 Metetra
.495 .707 .965 r
.52161 .36031 .53104 .37037 .54853 .36905 .53916 .35916 Metetra
.499 .714 .968 r
.53916 .35916 .54853 .36905 .56618 .36755 .55688 .35781 Metetra
.503 .721 .97 r
.55688 .35781 .56618 .36755 .58399 .36588 .57476 .35627 Metetra
.508 .728 .972 r
.57476 .35627 .58399 .36588 .60195 .36401 .5928 .35451 Metetra
.514 .735 .974 r
.5928 .35451 .60195 .36401 .62007 .36195 .611 .35254 Metetra
.519 .741 .975 r
.611 .35254 .62007 .36195 .63835 .3597 .62937 .35035 Metetra
.525 .747 .976 r
.62937 .35035 .63835 .3597 .65678 .35724 .64789 .34795 Metetra
.531 .752 .977 r
.64789 .34795 .65678 .35724 .67537 .35458 .66656 .34531 Metetra
.537 .757 .978 r
.66656 .34531 .67537 .35458 .69411 .35171 .6854 .34246 Metetra
.543 .761 .978 r
.6854 .34246 .69411 .35171 .713 .34864 .70438 .33938 Metetra
.55 .766 .978 r
.70438 .33938 .713 .34864 .73205 .34537 .72352 .33608 Metetra
.556 .769 .978 r
.72352 .33608 .73205 .34537 .75124 .3419 .74281 .33256 Metetra
.562 .773 .977 r
.74281 .33256 .75124 .3419 .77058 .33824 .76225 .32883 Metetra
.568 .775 .976 r
.76225 .32883 .77058 .33824 .79007 .33437 .78184 .3249 Metetra
.576 .648 .884 r
.12152 .3637 .13166 .37684 .14581 .37479 .13571 .36147 Metetra
.568 .642 .884 r
.13571 .36147 .14581 .37479 .16006 .37296 .14999 .35948 Metetra
.561 .637 .884 r
.14999 .35948 .16006 .37296 .17439 .37134 .16434 .35772 Metetra
.553 .633 .884 r
.16434 .35772 .17439 .37134 .18881 .36992 .17879 .35621 Metetra
.545 .629 .885 r
.17879 .35621 .18881 .36992 .20333 .36871 .19333 .35493 Metetra
.537 .625 .886 r
.19333 .35493 .20333 .36871 .21795 .36769 .20796 .35388 Metetra
.53 .622 .888 r
.20796 .35388 .21795 .36769 .23268 .36684 .22271 .35304 Metetra
.523 .62 .89 r
.22271 .35304 .23268 .36684 .24753 .36615 .23756 .3524 Metetra
.516 .619 .893 r
.23756 .3524 .24753 .36615 .26249 .36562 .25253 .35194 Metetra
.509 .619 .896 r
.25253 .35194 .26249 .36562 .27758 .36521 .26763 .35164 Metetra
.502 .619 .899 r
.26763 .35164 .27758 .36521 .2928 .36491 .28285 .35149 Metetra
.496 .62 .903 r
.28285 .35149 .2928 .36491 .30815 .3647 .29821 .35145 Metetra
.491 .622 .907 r
.29821 .35145 .30815 .3647 .32364 .36456 .31371 .35151 Metetra
.486 .625 .912 r
.31371 .35151 .32364 .36456 .33928 .36447 .32936 .35164 Metetra
.482 .628 .917 r
.32936 .35164 .33928 .36447 .35506 .3644 .34515 .35182 Metetra
.478 .633 .922 r
.34515 .35182 .35506 .3644 .371 .36433 .3611 .35201 Metetra
.475 .638 .927 r
.3611 .35201 .371 .36433 .38709 .36424 .37721 .35219 Metetra
.472 .644 .932 r
.37721 .35219 .38709 .36424 .40334 .36411 .39348 .35234 Metetra
.471 .65 .937 r
.39348 .35234 .40334 .36411 .41974 .36391 .40991 .35242 Metetra
.47 .657 .942 r
.40991 .35242 .41974 .36391 .43631 .36363 .42651 .35242 Metetra
.47 .665 .947 r
.42651 .35242 .43631 .36363 .45304 .36325 .44328 .35231 Metetra
.471 .673 .952 r
.44328 .35231 .45304 .36325 .46994 .36274 .46022 .35206 Metetra
.472 .681 .957 r
.46022 .35206 .46994 .36274 .487 .36209 .47733 .35167 Metetra
.475 .689 .961 r
.47733 .35167 .487 .36209 .50422 .36129 .49461 .3511 Metetra
.478 .697 .965 r
.49461 .3511 .50422 .36129 .52161 .36031 .51206 .35035 Metetra
.481 .706 .968 r
.51206 .35035 .52161 .36031 .53916 .35916 .52968 .3494 Metetra
.486 .714 .971 r
.52968 .3494 .53916 .35916 .55688 .35781 .54746 .34823 Metetra
.491 .722 .974 r
.54746 .34823 .55688 .35781 .57476 .35627 .56542 .34684 Metetra
.497 .73 .976 r
.56542 .34684 .57476 .35627 .5928 .35451 .58354 .34521 Metetra
.503 .737 .978 r
.58354 .34521 .5928 .35451 .611 .35254 .60182 .34334 Metetra
.509 .744 .98 r
.60182 .34334 .611 .35254 .62937 .35035 .62027 .34123 Metetra
.516 .751 .981 r
.62027 .34123 .62937 .35035 .64789 .34795 .63888 .33888 Metetra
.523 .757 .982 r
.63888 .33888 .64789 .34795 .66656 .34531 .65765 .33627 Metetra
.53 .762 .982 r
.65765 .33627 .66656 .34531 .6854 .34246 .67658 .33342 Metetra
.537 .767 .982 r
.67658 .33342 .6854 .34246 .70438 .33938 .69566 .33033 Metetra
.544 .771 .982 r
.69566 .33033 .70438 .33938 .72352 .33608 .7149 .32699 Metetra
.552 .775 .982 r
.7149 .32699 .72352 .33608 .74281 .33256 .73429 .32341 Metetra
.558 .779 .981 r
.73429 .32341 .74281 .33256 .76225 .32883 .75383 .3196 Metetra
.565 .781 .98 r
.75383 .3196 .76225 .32883 .78184 .3249 .77352 .31556 Metetra
.582 .646 .879 r
.11132 .35025 .12152 .3637 .13571 .36147 .12556 .34778 Metetra
.574 .64 .878 r
.12556 .34778 .13571 .36147 .14999 .35948 .13987 .34557 Metetra
.565 .634 .878 r
.13987 .34557 .14999 .35948 .16434 .35772 .15426 .34363 Metetra
.557 .628 .878 r
.15426 .34363 .16434 .35772 .17879 .35621 .16873 .34197 Metetra
.548 .623 .878 r
.16873 .34197 .17879 .35621 .19333 .35493 .18329 .34059 Metetra
.539 .618 .879 r
.18329 .34059 .19333 .35493 .20796 .35388 .19794 .33947 Metetra
.531 .614 .881 r
.19794 .33947 .20796 .35388 .22271 .35304 .21269 .33861 Metetra
.522 .611 .883 r
.21269 .33861 .22271 .35304 .23756 .3524 .22756 .338 Metetra
.514 .609 .885 r
.22756 .338 .23756 .3524 .25253 .35194 .24253 .33761 Metetra
.506 .608 .888 r
.24253 .33761 .25253 .35194 .26763 .35164 .25763 .33744 Metetra
.498 .607 .892 r
.25763 .33744 .26763 .35164 .28285 .35149 .27285 .33744 Metetra
.491 .608 .896 r
.27285 .33744 .28285 .35149 .29821 .35145 .28821 .33761 Metetra
.484 .61 .901 r
.28821 .33761 .29821 .35145 .31371 .35151 .30371 .33791 Metetra
.478 .612 .906 r
.30371 .33791 .31371 .35151 .32936 .35164 .31936 .3383 Metetra
.472 .616 .911 r
.31936 .3383 .32936 .35164 .34515 .35182 .33516 .33877 Metetra
.467 .621 .917 r
.33516 .33877 .34515 .35182 .3611 .35201 .35112 .33928 Metetra
.462 .626 .923 r
.35112 .33928 .3611 .35201 .37721 .35219 .36724 .3398 Metetra
.459 .633 .929 r
.36724 .3398 .37721 .35219 .39348 .35234 .38352 .34029 Metetra
.456 .64 .936 r
.38352 .34029 .39348 .35234 .40991 .35242 .39998 .34072 Metetra
.454 .648 .942 r
.39998 .34072 .40991 .35242 .42651 .35242 .41661 .34107 Metetra
.453 .657 .948 r
.41661 .34107 .42651 .35242 .44328 .35231 .43341 .3413 Metetra
.454 .666 .953 r
.43341 .3413 .44328 .35231 .46022 .35206 .45039 .34138 Metetra
.455 .675 .959 r
.45039 .34138 .46022 .35206 .47733 .35167 .46754 .3413 Metetra
.457 .685 .964 r
.46754 .3413 .47733 .35167 .49461 .3511 .48488 .34103 Metetra
.46 .695 .968 r
.48488 .34103 .49461 .3511 .51206 .35035 .50238 .34055 Metetra
.464 .704 .972 r
.50238 .34055 .51206 .35035 .52968 .3494 .52007 .33984 Metetra
.469 .714 .976 r
.52007 .33984 .52968 .3494 .54746 .34823 .53793 .33889 Metetra
.475 .723 .979 r
.53793 .33889 .54746 .34823 .56542 .34684 .55596 .33768 Metetra
.481 .732 .981 r
.55596 .33768 .56542 .34684 .58354 .34521 .57416 .33621 Metetra
.489 .741 .983 r
.57416 .33621 .58354 .34521 .60182 .34334 .59253 .33446 Metetra
.496 .749 .985 r
.59253 .33446 .60182 .34334 .62027 .34123 .61106 .33244 Metetra
.504 .756 .986 r
.61106 .33244 .62027 .34123 .63888 .33888 .62977 .33014 Metetra
.513 .763 .987 r
.62977 .33014 .63888 .33888 .65765 .33627 .64863 .32756 Metetra
.521 .769 .987 r
.64863 .32756 .65765 .33627 .67658 .33342 .66765 .3247 Metetra
.53 .774 .987 r
.66765 .3247 .67658 .33342 .69566 .33033 .68684 .32157 Metetra
.539 .779 .987 r
.68684 .32157 .69566 .33033 .7149 .32699 .70617 .31817 Metetra
.547 .783 .986 r
.70617 .31817 .7149 .32699 .73429 .32341 .72567 .3145 Metetra
.555 .786 .985 r
.72567 .3145 .73429 .32341 .75383 .3196 .74531 .31058 Metetra
.563 .789 .984 r
.74531 .31058 .75383 .3196 .77352 .31556 .7651 .30642 Metetra
.589 .646 .875 r
.10106 .33652 .11132 .35025 .12556 .34778 .11535 .33372 Metetra
.581 .638 .873 r
.11535 .33372 .12556 .34778 .13987 .34557 .12971 .33122 Metetra
.571 .631 .872 r
.12971 .33122 .13987 .34557 .15426 .34363 .14414 .32903 Metetra
.562 .623 .871 r
.14414 .32903 .15426 .34363 .16873 .34197 .15864 .32715 Metetra
.552 .617 .87 r
.15864 .32715 .16873 .34197 .18329 .34059 .17323 .3256 Metetra
.543 .611 .871 r
.17323 .3256 .18329 .34059 .19794 .33947 .1879 .32436 Metetra
.533 .605 .871 r
.1879 .32436 .19794 .33947 .21269 .33861 .20267 .32344 Metetra
.523 .601 .873 r
.20267 .32344 .21269 .33861 .22756 .338 .21753 .32283 Metetra
.513 .597 .875 r
.21753 .32283 .22756 .338 .24253 .33761 .23251 .3225 Metetra
.504 .595 .878 r
.23251 .3225 .24253 .33761 .25763 .33744 .2476 .32243 Metetra
.494 .593 .882 r
.2476 .32243 .25763 .33744 .27285 .33744 .26282 .32261 Metetra
.485 .593 .886 r
.26282 .32261 .27285 .33744 .28821 .33761 .27817 .32301 Metetra
.476 .594 .892 r
.27817 .32301 .28821 .33761 .30371 .33791 .29366 .32359 Metetra
.468 .597 .897 r
.29366 .32359 .30371 .33791 .31936 .3383 .3093 .32431 Metetra
.461 .6 .904 r
.3093 .32431 .31936 .3383 .33516 .33877 .3251 .32514 Metetra
.454 .605 .911 r
.3251 .32514 .33516 .33877 .35112 .33928 .34105 .32605 Metetra
.447 .611 .918 r
.34105 .32605 .35112 .33928 .36724 .3398 .35718 .32698 Metetra
.442 .618 .925 r
.35718 .32698 .36724 .3398 .38352 .34029 .37347 .32791 Metetra
.438 .626 .933 r
.37347 .32791 .38352 .34029 .39998 .34072 .38994 .32879 Metetra
.434 .636 .94 r
.38994 .32879 .39998 .34072 .41661 .34107 .40659 .32958 Metetra
.432 .646 .948 r
.40659 .32958 .41661 .34107 .43341 .3413 .42342 .33024 Metetra
.431 .656 .954 r
.42342 .33024 .43341 .3413 .45039 .34138 .44044 .33076 Metetra
.432 .668 .961 r
.44044 .33076 .45039 .34138 .46754 .3413 .45763 .33108 Metetra
.433 .679 .967 r
.45763 .33108 .46754 .3413 .48488 .34103 .47502 .33118 Metetra
.436 .691 .972 r
.47502 .33118 .48488 .34103 .50238 .34055 .49258 .33104 Metetra
.441 .703 .977 r
.49258 .33104 .50238 .34055 .52007 .33984 .51033 .33063 Metetra
.446 .714 .981 r
.51033 .33063 .52007 .33984 .53793 .33889 .52826 .32994 Metetra
.453 .725 .984 r
.52826 .32994 .53793 .33889 .55596 .33768 .54637 .32896 Metetra
.461 .736 .987 r
.54637 .32896 .55596 .33768 .57416 .33621 .56466 .32766 Metetra
.47 .746 .989 r
.56466 .32766 .57416 .33621 .59253 .33446 .58311 .32605 Metetra
.479 .755 .991 r
.58311 .32605 .59253 .33446 .61106 .33244 .60174 .32412 Metetra
.489 .763 .992 r
.60174 .32412 .61106 .33244 .62977 .33014 .62054 .32186 Metetra
.5 .771 .992 r
.62054 .32186 .62977 .33014 .64863 .32756 .6395 .31929 Metetra
.511 .778 .992 r
.6395 .31929 .64863 .32756 .66765 .3247 .65863 .3164 Metetra
.521 .784 .992 r
.65863 .3164 .66765 .3247 .68684 .32157 .67791 .3132 Metetra
.532 .788 .992 r
.67791 .3132 .68684 .32157 .70617 .31817 .69735 .30971 Metetra
.542 .792 .991 r
.69735 .30971 .70617 .31817 .72567 .3145 .71695 .30591 Metetra
.552 .796 .989 r
.71695 .30591 .72567 .3145 .74531 .31058 .73669 .30184 Metetra
.562 .798 .988 r
.73669 .30184 .74531 .31058 .7651 .30642 .75659 .2975 Metetra
.599 .649 .87 r
.09074 .32257 .10106 .33652 .11535 .33372 .10509 .31935 Metetra
.59 .639 .868 r
.10509 .31935 .11535 .33372 .12971 .33122 .11951 .31645 Metetra
.58 .629 .865 r
.11951 .31645 .12971 .33122 .14414 .32903 .13399 .31388 Metetra
.57 .62 .863 r
.13399 .31388 .14414 .32903 .15864 .32715 .14854 .31168 Metetra
.559 .611 .861 r
.14854 .31168 .15864 .32715 .17323 .3256 .16316 .30986 Metetra
.548 .603 .86 r
.16316 .30986 .17323 .3256 .1879 .32436 .17786 .30841 Metetra
.537 .596 .86 r
.17786 .30841 .1879 .32436 .20267 .32344 .19264 .30735 Metetra
.526 .589 .861 r
.19264 .30735 .20267 .32344 .21753 .32283 .20752 .30667 Metetra
.514 .584 .863 r
.20752 .30667 .21753 .32283 .23251 .3225 .22249 .30636 Metetra
.503 .58 .865 r
.22249 .30636 .23251 .3225 .2476 .32243 .23758 .30639 Metetra
.491 .577 .869 r
.23758 .30639 .2476 .32243 .26282 .32261 .25279 .30675 Metetra
.48 .575 .873 r
.25279 .30675 .26282 .32261 .27817 .32301 .26812 .3074 Metetra
.469 .575 .879 r
.26812 .3074 .27817 .32301 .29366 .32359 .28359 .30831 Metetra
.458 .577 .885 r
.28359 .30831 .29366 .32359 .3093 .32431 .29921 .30943 Metetra
.448 .58 .893 r
.29921 .30943 .3093 .32431 .3251 .32514 .31499 .31072 Metetra
.438 .584 .901 r
.31499 .31072 .3251 .32514 .34105 .32605 .33093 .31214 Metetra
.429 .591 .91 r
.33093 .31214 .34105 .32605 .35718 .32698 .34704 .31362 Metetra
.421 .599 .919 r
.34704 .31362 .35718 .32698 .37347 .32791 .36333 .31512 Metetra
.414 .608 .928 r
.36333 .31512 .37347 .32791 .38994 .32879 .3798 .31658 Metetra
.408 .619 .938 r
.3798 .31658 .38994 .32879 .40659 .32958 .39646 .31796 Metetra
.404 .631 .947 r
.39646 .31796 .40659 .32958 .42342 .33024 .41331 .3192 Metetra
.401 .644 .955 r
.41331 .3192 .42342 .33024 .44044 .33076 .43035 .32027 Metetra
.399 .658 .963 r
.43035 .32027 .44044 .33076 .45763 .33108 .44759 .32112 Metetra
.4 .672 .97 r
.44759 .32112 .45763 .33108 .47502 .33118 .46502 .32171 Metetra
.402 .686 .976 r
.46502 .32171 .47502 .33118 .49258 .33104 .48265 .322 Metetra
.407 .7 .981 r
.48265 .322 .49258 .33104 .51033 .33063 .50046 .32198 Metetra
.413 .714 .986 r
.50046 .32198 .51033 .33063 .52826 .32994 .51847 .32162 Metetra
.422 .727 .989 r
.51847 .32162 .52826 .32994 .54637 .32896 .53666 .3209 Metetra
.432 .74 .992 r
.53666 .3209 .54637 .32896 .56466 .32766 .55503 .3198 Metetra
.443 .752 .994 r
.55503 .3198 .56466 .32766 .58311 .32605 .57358 .31834 Metetra
.456 .763 .996 r
.57358 .31834 .58311 .32605 .60174 .32412 .59231 .31649 Metetra
.469 .773 .997 r
.59231 .31649 .60174 .32412 .62054 .32186 .6112 .31426 Metetra
.483 .782 .997 r
.6112 .31426 .62054 .32186 .6395 .31929 .63027 .31166 Metetra
.497 .789 .997 r
.63027 .31166 .6395 .31929 .65863 .3164 .64949 .30869 Metetra
.511 .796 .997 r
.64949 .30869 .65863 .3164 .67791 .3132 .66888 .30537 Metetra
.525 .801 .996 r
.66888 .30537 .67791 .3132 .69735 .30971 .68843 .30171 Metetra
.538 .805 .995 r
.68843 .30171 .69735 .30971 .71695 .30591 .70813 .29773 Metetra
.55 .807 .993 r
.70813 .29773 .71695 .30591 .73669 .30184 .72798 .29343 Metetra
.562 .809 .991 r
.72798 .29343 .73669 .30184 .75659 .2975 .74798 .28884 Metetra
.612 .655 .868 r
.08031 .30859 .09074 .32257 .10509 .31935 .09476 .30481 Metetra
.603 .643 .863 r
.09476 .30481 .10509 .31935 .11951 .31645 .10926 .30135 Metetra
.593 .631 .859 r
.10926 .30135 .11951 .31645 .13399 .31388 .12382 .29826 Metetra
.582 .62 .855 r
.12382 .29826 .13399 .31388 .14854 .31168 .13843 .29556 Metetra
.57 .608 .851 r
.13843 .29556 .14854 .31168 .16316 .30986 .15311 .29329 Metetra
.558 .597 .849 r
.15311 .29329 .16316 .30986 .17786 .30841 .16785 .29148 Metetra
.545 .587 .847 r
.16785 .29148 .17786 .30841 .19264 .30735 .18266 .29013 Metetra
.532 .577 .846 r
.18266 .29013 .19264 .30735 .20752 .30667 .19755 .28926 Metetra
.518 .569 .846 r
.19755 .28926 .20752 .30667 .22249 .30636 .21253 .28886 Metetra
.504 .562 .848 r
.21253 .28886 .22249 .30636 .23758 .30639 .22761 .28893 Metetra
.49 .556 .85 r
.22761 .28893 .23758 .30639 .25279 .30675 .24279 .28944 Metetra
.476 .553 .855 r
.24279 .28944 .25279 .30675 .26812 .3074 .2581 .29037 Metetra
.462 .551 .86 r
.2581 .29037 .26812 .3074 .28359 .30831 .27353 .29167 Metetra
.448 .551 .868 r
.27353 .29167 .28359 .30831 .29921 .30943 .28911 .29329 Metetra
.434 .553 .876 r
.28911 .29329 .29921 .30943 .31499 .31072 .30485 .29517 Metetra
.42 .557 .886 r
.30485 .29517 .31499 .31072 .33093 .31214 .32075 .29726 Metetra
.407 .563 .897 r
.32075 .29726 .33093 .31214 .34704 .31362 .33683 .29948 Metetra
.394 .572 .909 r
.33683 .29948 .34704 .31362 .36333 .31512 .3531 .30176 Metetra
.382 .583 .921 r
.3531 .30176 .36333 .31512 .3798 .31658 .36955 .30403 Metetra
.372 .595 .933 r
.36955 .30403 .3798 .31658 .39646 .31796 .38621 .30623 Metetra
.362 .61 .944 r
.38621 .30623 .39646 .31796 .41331 .3192 .40307 .30827 Metetra
.355 .625 .954 r
.40307 .30827 .41331 .3192 .43035 .32027 .42013 .31011 Metetra
.35 .642 .964 r
.42013 .31011 .43035 .32027 .44759 .32112 .4374 .31167 Metetra
.348 .659 .971 r
.4374 .31167 .44759 .32112 .46502 .32171 .45488 .31292 Metetra
.349 .677 .978 r
.45488 .31292 .46502 .32171 .48265 .322 .47256 .31379 Metetra
.353 .694 .982 r
.47256 .31379 .48265 .322 .50046 .32198 .49044 .31427 Metetra
.361 .712 .986 r
.49044 .31427 .50046 .32198 .51847 .32162 .50852 .31431 Metetra
.372 .728 .989 r
.50852 .31431 .51847 .32162 .53666 .3209 .5268 .3139 Metetra
.386 .744 .992 r
.5268 .3139 .53666 .3209 .55503 .3198 .54527 .31303 Metetra
.402 .759 .994 r
.54527 .31303 .55503 .3198 .57358 .31834 .56392 .31169 Metetra
.42 .773 .996 r
.56392 .31169 .57358 .31834 .59231 .31649 .58274 .30989 Metetra
.439 .785 .998 r
.58274 .30989 .59231 .31649 .6112 .31426 .60175 .30762 Metetra
.459 .796 .999 r
.60175 .30762 .6112 .31426 .63027 .31166 .62092 .30491 Metetra
.48 .804 1 r
.62092 .30491 .63027 .31166 .64949 .30869 .64025 .30177 Metetra
.499 .811 1 r
.64025 .30177 .64949 .30869 .66888 .30537 .65975 .29822 Metetra
.518 .816 .999 r
.65975 .29822 .66888 .30537 .68843 .30171 .6794 .29429 Metetra
.535 .82 .998 r
.6794 .29429 .68843 .30171 .70813 .29773 .6992 .28999 Metetra
.551 .821 .996 r
.6992 .28999 .70813 .29773 .72798 .29343 .71916 .28536 Metetra
.565 .821 .994 r
.71916 .28536 .72798 .29343 .74798 .28884 .73926 .28041 Metetra
.629 .668 .869 r
.06974 .29493 .08031 .30859 .09476 .30481 .0843 .29044 Metetra
.62 .655 .862 r
.0843 .29044 .09476 .30481 .10926 .30135 .09892 .28624 Metetra
.61 .64 .856 r
.09892 .28624 .10926 .30135 .12382 .29826 .11358 .28239 Metetra
.599 .625 .849 r
.11358 .28239 .12382 .29826 .13843 .29556 .1283 .27894 Metetra
.587 .61 .842 r
.1283 .27894 .13843 .29556 .15311 .29329 .14307 .27594 Metetra
.573 .595 .837 r
.14307 .27594 .15311 .29329 .16785 .29148 .15788 .27346 Metetra
.559 .58 .832 r
.15788 .27346 .16785 .29148 .18266 .29013 .17275 .27154 Metetra
.544 .566 .828 r
.17275 .27154 .18266 .29013 .19755 .28926 .18768 .27021 Metetra
.528 .554 .825 r
.18768 .27021 .19755 .28926 .21253 .28886 .20268 .26951 Metetra
.511 .542 .824 r
.20268 .26951 .21253 .28886 .22761 .28893 .21775 .26944 Metetra
.494 .532 .825 r
.21775 .26944 .22761 .28893 .24279 .28944 .23292 .27 Metetra
.476 .525 .828 r
.23292 .27 .24279 .28944 .2581 .29037 .24818 .27116 Metetra
.458 .519 .833 r
.24818 .27116 .2581 .29037 .27353 .29167 .26357 .27289 Metetra
.439 .516 .84 r
.26357 .27289 .27353 .29167 .28911 .29329 .27909 .27513 Metetra
.42 .515 .85 r
.27909 .27513 .28911 .29329 .30485 .29517 .29476 .27781 Metetra
.4 .518 .861 r
.29476 .27781 .30485 .29517 .32075 .29726 .31059 .28083 Metetra
.379 .523 .875 r
.31059 .28083 .32075 .29726 .33683 .29948 .3266 .28411 Metetra
.358 .532 .89 r
.3266 .28411 .33683 .29948 .3531 .30176 .34281 .28755 Metetra
.337 .543 .906 r
.34281 .28755 .3531 .30176 .36955 .30403 .35922 .29102 Metetra
.317 .558 .922 r
.35922 .29102 .36955 .30403 .38621 .30623 .37584 .29443 Metetra
.297 .575 .936 r
.37584 .29443 .38621 .30623 .40307 .30827 .39268 .29767 Metetra
.279 .593 .948 r
.39268 .29767 .40307 .30827 .42013 .31011 .40975 .30064 Metetra
.265 .613 .957 r
.40975 .30064 .42013 .31011 .4374 .31167 .42704 .30326 Metetra
.256 .634 .962 r
.42704 .30326 .4374 .31167 .45488 .31292 .44455 .30543 Metetra
.252 .655 .964 r
.44455 .30543 .45488 .31292 .47256 .31379 .46229 .30711 Metetra
.256 .676 .964 r
.46229 .30711 .47256 .31379 .49044 .31427 .48025 .30823 Metetra
.265 .697 .965 r
.48025 .30823 .49044 .31427 .50852 .31431 .49841 .30877 Metetra
.282 .718 .966 r
.49841 .30877 .50852 .31431 .5268 .3139 .51679 .30871 Metetra
.304 .739 .968 r
.51679 .30871 .5268 .3139 .54527 .31303 .53536 .30803 Metetra
.331 .76 .972 r
.53536 .30803 .54527 .31303 .56392 .31169 .55412 .30673 Metetra
.362 .779 .978 r
.55412 .30673 .56392 .31169 .58274 .30989 .57306 .30485 Metetra
.394 .796 .984 r
.57306 .30485 .58274 .30989 .60175 .30762 .59217 .30238 Metetra
.427 .811 .99 r
.59217 .30238 .60175 .30762 .62092 .30491 .61146 .29938 Metetra
.459 .823 .994 r
.61146 .29938 .62092 .30491 .64025 .30177 .6309 .29587 Metetra
.488 .831 .998 r
.6309 .29587 .64025 .30177 .65975 .29822 .65051 .29188 Metetra
.515 .836 .999 r
.65051 .29188 .65975 .29822 .6794 .29429 .67026 .28747 Metetra
.539 .838 .999 r
.67026 .28747 .6794 .29429 .6992 .28999 .69017 .28266 Metetra
.559 .837 .997 r
.69017 .28266 .6992 .28999 .71916 .28536 .71022 .2775 Metetra
.576 .834 .994 r
.71022 .2775 .71916 .28536 .73926 .28041 .73041 .27202 Metetra
.648 .693 .878 r
.05891 .28221 .06974 .29493 .0843 .29044 .07361 .27696 Metetra
.642 .679 .87 r
.07361 .27696 .0843 .29044 .09892 .28624 .08837 .27187 Metetra
.634 .663 .86 r
.08837 .27187 .09892 .28624 .11358 .28239 .10319 .26702 Metetra
.624 .645 .85 r
.10319 .26702 .11358 .28239 .1283 .27894 .11806 .26247 Metetra
.612 .625 .84 r
.11806 .26247 .1283 .27894 .14307 .27594 .13298 .25832 Metetra
.599 .605 .829 r
.13298 .25832 .14307 .27594 .15788 .27346 .14793 .25466 Metetra
.584 .584 .818 r
.14793 .25466 .15788 .27346 .17275 .27154 .16293 .2516 Metetra
.567 .563 .809 r
.16293 .2516 .17275 .27154 .18768 .27021 .17796 .24923 Metetra
.548 .543 .801 r
.17796 .24923 .18768 .27021 .20268 .26951 .19303 .24765 Metetra
.529 .525 .795 r
.19303 .24765 .20268 .26951 .21775 .26944 .20814 .24694 Metetra
.508 .507 .791 r
.20814 .24694 .21775 .26944 .23292 .27 .22331 .24713 Metetra
.486 .492 .789 r
.22331 .24713 .23292 .27 .24818 .27116 .23854 .24826 Metetra
.462 .48 .791 r
.23854 .24826 .24818 .27116 .26357 .27289 .25386 .25033 Metetra
.437 .47 .796 r
.25386 .25033 .26357 .27289 .27909 .27513 .26929 .25328 Metetra
.41 .463 .804 r
.26929 .25328 .27909 .27513 .29476 .27781 .28484 .25702 Metetra
.38 .46 .817 r
.28484 .25702 .29476 .27781 .31059 .28083 .30054 .26146 Metetra
.347 .46 .834 r
.30054 .26146 .31059 .28083 .3266 .28411 .31643 .26642 Metetra
.31 .466 .854 r
.31643 .26642 .3266 .28411 .34281 .28755 .3325 .27175 Metetra
.268 .475 .875 r
.3325 .27175 .34281 .28755 .35922 .29102 .3488 .27725 Metetra
.222 .489 .896 r
.3488 .27725 .35922 .29102 .37584 .29443 .36534 .28272 Metetra
.173 .505 .911 r
.36534 .28272 .37584 .29443 .39268 .29767 .38212 .28798 Metetra
.125 .522 .914 r
.38212 .28798 .39268 .29767 .40975 .30064 .39915 .29284 Metetra
.083 .538 .905 r
.39915 .29284 .40975 .30064 .42704 .30326 .41644 .29715 Metetra
.053 .553 .884 r
.41644 .29715 .42704 .30326 .44455 .30543 .43399 .30077 Metetra
.039 .569 .859 r
.43399 .30077 .44455 .30543 .46229 .30711 .45179 .3036 Metetra
.043 .589 .839 r
.45179 .3036 .46229 .30711 .48025 .30823 .46983 .30557 Metetra
.063 .614 .829 r
.46983 .30557 .48025 .30823 .49841 .30877 .4881 .30664 Metetra
.099 .646 .831 r
.4881 .30664 .49841 .30877 .51679 .30871 .50659 .3068 Metetra
.146 .683 .846 r
.50659 .3068 .51679 .30871 .53536 .30803 .52528 .30607 Metetra
.203 .722 .87 r
.52528 .30607 .53536 .30803 .55412 .30673 .54416 .3045 Metetra
.266 .762 .899 r
.54416 .3045 .55412 .30673 .57306 .30485 .56323 .30213 Metetra
.33 .797 .929 r
.56323 .30213 .57306 .30485 .59217 .30238 .58247 .29903 Metetra
.391 .825 .956 r
.58247 .29903 .59217 .30238 .61146 .29938 .60187 .29528 Metetra
.447 .845 .976 r
.60187 .29528 .61146 .29938 .6309 .29587 .62143 .29096 Metetra
.494 .855 .99 r
.62143 .29096 .6309 .29587 .65051 .29188 .64113 .28612 Metetra
.532 .858 .996 r
.64113 .28612 .65051 .29188 .67026 .28747 .66098 .28086 Metetra
.562 .855 .997 r
.66098 .28086 .67026 .28747 .69017 .28266 .68098 .27522 Metetra
.585 .849 .994 r
.68098 .27522 .69017 .28266 .71022 .2775 .70112 .26927 Metetra
.602 .84 .989 r
.70112 .26927 .71022 .2775 .73041 .27202 .7214 .26305 Metetra
.662 .726 .897 r
.04774 .27097 .05891 .28221 .07361 .27696 .06254 .26525 Metetra
.66 .718 .892 r
.06254 .26525 .07361 .27696 .08837 .27187 .07743 .25952 Metetra
.658 .707 .884 r
.07743 .25952 .08837 .27187 .10319 .26702 .0924 .2538 Metetra
.654 .692 .873 r
.0924 .2538 .10319 .26702 .11806 .26247 .10744 .24812 Metetra
.648 .672 .86 r
.10744 .24812 .11806 .26247 .13298 .25832 .12256 .24254 Metetra
.639 .649 .843 r
.12256 .24254 .13298 .25832 .14793 .25466 .13775 .23715 Metetra
.627 .621 .825 r
.13775 .23715 .14793 .25466 .16293 .2516 .15299 .23207 Metetra
.612 .591 .806 r
.15299 .23207 .16293 .2516 .17796 .24923 .16828 .22747 Metetra
.594 .56 .786 r
.16828 .22747 .17796 .24923 .19303 .24765 .18358 .22355 Metetra
.573 .529 .768 r
.18358 .22355 .19303 .24765 .20814 .24694 .1989 .22056 Metetra
.549 .498 .751 r
.1989 .22056 .20814 .24694 .22331 .24713 .21422 .21876 Metetra
.522 .469 .738 r
.21422 .21876 .22331 .24713 .23854 .24826 .22954 .21839 Metetra
.492 .442 .729 r
.22954 .21839 .23854 .24826 .25386 .25033 .24486 .21968 Metetra
.459 .417 .724 r
.24486 .21968 .25386 .25033 .26929 .25328 .26019 .22276 Metetra
.421 .395 .725 r
.26019 .22276 .26929 .25328 .28484 .25702 .27556 .22766 Metetra
.376 .376 .732 r
.27556 .22766 .28484 .25702 .30054 .26146 .29102 .23429 Metetra
.321 .36 .746 r
.29102 .23429 .30054 .26146 .31643 .26642 .30661 .24243 Metetra
.25 .346 .767 r
.30661 .24243 .31643 .26642 .3325 .27175 .32238 .2517 Metetra
.152 .335 .794 r
.32238 .2517 .3325 .27175 .3488 .27725 .33838 .26165 Metetra
.018 .323 .809 r
.33838 .26165 .3488 .27725 .36534 .28272 .35466 .27177 Metetra
0 .3 .775 r
.35466 .27177 .36534 .28272 .38212 .28798 .37125 .28152 Metetra
0 .261 .661 r
.37125 .28152 .38212 .28798 .39915 .29284 .38816 .29042 Metetra
.378 0 0 r
.38816 .29042 .39915 .29284 .41644 .29715 .40541 .29806 Metetra
.391 0 0 r
.40541 .29806 .41644 .29715 .43399 .30077 .42299 .30414 Metetra
.366 0 0 r
.42299 .30414 .43399 .30077 .45179 .3036 .44087 .3085 Metetra
.32 0 0 r
.44087 .3085 .45179 .3036 .46983 .30557 .45904 .31109 Metetra
.257 0 0 r
.45904 .31109 .46983 .30557 .4881 .30664 .47747 .31197 Metetra
.175 0 0 r
.47747 .31197 .4881 .30664 .50659 .3068 .49613 .31129 Metetra
.074 0 0 r
.49613 .31129 .50659 .3068 .52528 .30607 .51499 .30922 Metetra
.51499 .30922 .52528 .30607 .54416 .3045 .53403 .30599 Metetra
.187 .717 .736 r
.53403 .30599 .54416 .3045 .56323 .30213 .55324 .30181 Metetra
.321 .808 .852 r
.55324 .30181 .56323 .30213 .58247 .29903 .5726 .29688 Metetra
.434 .863 .934 r
.5726 .29688 .58247 .29903 .60187 .29528 .59211 .29136 Metetra
.515 .882 .977 r
.59211 .29136 .60187 .29528 .62143 .29096 .61177 .28541 Metetra
.568 .878 .993 r
.61177 .28541 .62143 .29096 .64113 .28612 .63156 .27912 Metetra
.6 .863 .993 r
.63156 .27912 .64113 .28612 .66098 .28086 .65149 .2726 Metetra
.619 .845 .986 r
.65149 .2726 .66098 .28086 .68098 .27522 .67156 .26589 Metetra
.63 .828 .977 r
.67156 .26589 .68098 .27522 .70112 .26927 .69177 .25905 Metetra
.637 .813 .968 r
.69177 .25905 .70112 .26927 .7214 .26305 .71213 .2521 Metetra
.664 .741 .908 r
.03634 .26033 .04774 .27097 .06254 .26525 .05119 .25453 Metetra
.663 .741 .907 r
.05119 .25453 .06254 .26525 .07743 .25952 .06613 .2487 Metetra
.663 .74 .907 r
.06613 .2487 .07743 .25952 .0924 .2538 .08116 .24283 Metetra
.663 .738 .906 r
.08116 .24283 .0924 .2538 .10744 .24812 .09628 .23692 Metetra
.663 .735 .903 r
.09628 .23692 .10744 .24812 .12256 .24254 .11148 .23098 Metetra
.662 .729 .899 r
.11148 .23098 .12256 .24254 .13775 .23715 .12679 .225 Metetra
.661 .719 .891 r
.12679 .225 .13775 .23715 .15299 .23207 .14218 .21899 Metetra
.659 .703 .879 r
.14218 .21899 .15299 .23207 .16828 .22747 .15767 .21294 Metetra
.655 .678 .86 r
.15767 .21294 .16828 .22747 .18358 .22355 .17325 .20686 Metetra
.648 .645 .834 r
.17325 .20686 .18358 .22355 .1989 .22056 .18892 .20073 Metetra
.637 .604 .8 r
.18892 .20073 .1989 .22056 .21422 .21876 .20469 .19457 Metetra
.623 .558 .761 r
.20469 .19457 .21422 .21876 .22954 .21839 .22056 .18838 Metetra
.607 .512 .72 r
.22056 .18838 .22954 .21839 .24486 .21968 .23652 .18214 Metetra
.591 .469 .681 r
.23652 .18214 .24486 .21968 .26019 .22276 .25259 .17587 Metetra
.577 .432 .648 r
.25259 .17587 .26019 .22276 .27556 .22766 .26875 .16956 Metetra
.563 .401 .619 r
.26875 .16956 .27556 .22766 .29102 .23429 .28501 .1633 Metetra
.541 .368 .596 r
.28501 .1633 .29102 .23429 .30661 .24243 .30123 .15882 Metetra
.45 .289 .57 r
.30123 .15882 .30661 .24243 .32238 .2517 .31664 .16766 Metetra
.093 .028 .484 r
.31664 .16766 .32238 .2517 .33838 .26165 .3299 .21419 Metetra
.746 .504 0 r
.3299 .21419 .33838 .26165 .35466 .27177 .34214 .28981 Metetra
.831 .453 .339 r
.34214 .28981 .35466 .27177 .37125 .28152 .35674 .34012 Metetra
.547 .22 .365 r
.35674 .34012 .37125 .28152 .38816 .29042 .37396 .35113 Metetra
.439 .124 .333 r
.37396 .35113 .38816 .29042 .40541 .29806 .39213 .34715 Metetra
.4 .072 .289 r
.39213 .34715 .40541 .29806 .42299 .30414 .41053 .34089 Metetra
.362 .008 .222 r
.41053 .34089 .42299 .30414 .44087 .3085 .42907 .33443 Metetra
.302 0 .114 r
.42907 .33443 .44087 .3085 .45904 .31109 .44773 .32793 Metetra
.196 0 0 r
.44773 .32793 .45904 .31109 .47747 .31197 .46652 .32138 Metetra
.013 0 0 r
.46652 .32138 .47747 .31197 .49613 .31129 .48544 .31479 Metetra
.244 .745 .681 r
.48544 .31479 .49613 .31129 .51499 .30922 .5045 .30815 Metetra
.463 .885 .913 r
.5045 .30815 .51499 .30922 .53403 .30599 .52368 .30146 Metetra
.573 .895 .986 r
.52368 .30146 .53403 .30599 .55324 .30181 .54301 .29473 Metetra
.617 .864 .989 r
.54301 .29473 .55324 .30181 .5726 .29688 .56246 .28794 Metetra
.634 .832 .977 r
.56246 .28794 .5726 .29688 .59211 .29136 .58206 .28111 Metetra
.641 .808 .963 r
.58206 .28111 .59211 .29136 .61177 .28541 .6018 .27424 Metetra
.643 .791 .953 r
.6018 .27424 .61177 .28541 .63156 .27912 .62167 .26731 Metetra
.645 .78 .946 r
.62167 .26731 .63156 .27912 .65149 .2726 .64169 .26033 Metetra
.645 .773 .941 r
.64169 .26033 .65149 .2726 .67156 .26589 .66185 .25331 Metetra
.646 .769 .938 r
.66185 .25331 .67156 .26589 .69177 .25905 .68216 .24623 Metetra
.646 .766 .936 r
.68216 .24623 .69177 .25905 .71213 .2521 .70262 .2391 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{150.75, 122.25},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002l0000V1000`40O003h00O_7oo002lOol00;ao
o`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo001fOol00`00
Oomoo`04Ool3000lOol007Ioo`03001oogoo00=oo`05001oogooOol0000kOol007Moo`03001oogoo
009oo`05001oogooOol0000kOol007Moo`03001oogoo009oo`05001oogooOol0000kOol007Moo`03
001oogoo009oo`05001oogooOol0000kOol007Qoo`05001oogooOol00003Ool00`00Oomoo`0iOol0
07Aoo`05001oogooOol00003Ool01@00Oomoogoo0000>goo001dOol50004Ool3000lOol00;aoo`00
_7oo001JOol30005Ool4001FOol005Uoo`05001oogooOol00008Ool00`00Oomoo`0GOol3000iOol0
05Uoo`05001oogooOol00008Ool00`00Oomoo`0DOol300000goo001oo`0iOol005Uoo`05001oogoo
Ool00008Ool00`00Oomoo`0AOol30004Ool2000iOol005Uoo`@000Eoo`@0019oo`8000Moo`03001o
o`0003Qoo`00FGoo00<007ooOol01Woo00<007ooOol047oo0`002Goo00<007oo0000>7oo001JOol0
0`00Oomoo`05Ool00`00Oomoo`0>Ool2000<Ool01000Oomoo`00=goo001KOol30004Ool4000:Ool3
000>Ool01@00Oomoogoo0000=Woo001]Ool3000AOol01000Oomoogoo0P00=Goo001[Ool3000COol0
0`00Oomoo`02Ool00`00Oomoo`0cOol0045oo`<000Eoo`<001aoo`<001Ioo`03001oogoo00=oo`03
001oogoo00Aoo`<000Ioo`8000Aoo`D001Yoo`00@7oo00D007ooOomoo`0000=oo`05001oogooOol0
000IOol3000IOol00`00Oomoo`02Ool00`00Oomoo`03Ool01@00Oomoogoo00001Goo0P001Goo00@0
07ooOol001Yoo`00@7oo00D007ooOomoo`0000=oo`05001oogooOol0000FOol3000LOol00`00Oomo
o`03Ool00`00Oomoo`02Ool01@00Oomoogoo00003Goo00<007ooOol06Woo0010Ool01@00Oomoogoo
00000goo00D007ooOomoo`0001=oo`@001ioo`03001oogoo00Aoo`05001oogooOol00003Ool00`00
Oomoo`0<Ool00`00Oomoo`0IOol0041oo`@000Aoo`05001oogooOol0000AOol2000ROol00`00Oomo
o`04Ool01@00Oomoogoo00000goo00<007ooOol03Goo00<007ooOol067oo0010Ool00`00Oomoo`05
Ool01@00Oomoogoo00003Woo10008goo00<007ooOol01Goo0P0000=oo`00Ool00Woo00<007ooOol0
3Goo00<007ooOol067oo0011Ool00`00Oomoo`04Ool01@00Oomoogoo000037oo0P009goo00<007oo
Ool01Woo00<007oo00000goo00<007ooOol02Goo00D007ooOomoo`0001Yoo`00@Woo0`0017oo0`00
2Woo1000:7oo00<007ooOol01Woo00<007ooOol00`003Goo0`006goo001COol3000/Ool00`00Oomo
o`07Ool00`00Oomoo`0]Ool002Yoo`@000Aoo`@001]oo`800003Ool007oo02aoo`03001oogoo00Mo
o`8002ioo`00;Woo00<007ooOol01Goo00<007ooOol05Goo0`00<Goo00<007ooOol027oo00<007oo
Ool0;7oo000^Ool00`00Oomoo`05Ool00`00Oomoo`0BOol30002Ool00`00Oomoo`0_Ool00`00Oomo
o`09Ool00`00Oomoo`0[Ool002ioo`03001oogoo00Eoo`03001oogoo011oo`<003Ioo`03001oogoo
00Uoo`8002aoo`00:Woo100017oo100047oo1000>7oo00<007ooOol02Woo00<007ooOol01Goo0`00
1Woo0P001Woo0`004Goo000ZOol00`00Oomoo`05Ool00`00Oomoo`0?Ool400000dEc0000000gOol0
0`00Oomoo`0;Ool00`00Oomoo`03Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol047oo000Z
Ool00`00Oomoo`05Ool00`00Oomoo`0<Ool500001TVC0015L`00>C8003Ioo`03001oogoo00]oo`80
00Aoo`05001oogooOol0000;Ool5000AOol002Yoo`@000Aoo`@000Qoo`H00008BK@004VC0015L`00
>C8003Ioo`03001oogoo00aoo`03001oogoo009oo`05001oogooOol0000;Ool01000Oomoo`004Woo
000DOol40004Ool3000MOol200001Goo000004We00000TVd00H004VC0015L`00>C82000eOol00`00
Oomoo`0=Ool01@00Oomoogoo00000goo00<007ooOol02Woo00<007oo00004Woo000HOol01000Oomo
o`000goo00<007ooOol05goo0`0000Moo`00001>5`00BOD00002BK@01P00BI<004Ec000i<P8003Io
o`03001oogoo00aoo`05001oogooOol00003Ool00`00Oomoo`0:Ool00`00Ool0000BOol001Qoo`04
001oogoo0003Ool00`00Oomoo`0EOol600001Thh001>5`00BOD00099]0060019T`00AG<003Tb0P00
=Woo00<007ooOol037oo0P000Woo00D007ooOomoo`0000eoo`80019oo`0067oo00@007ooOol000=o
o`03001oogoo019oo`H00005DWT004hhCSP00002CQL01@00BOD004Vd00000TVC00H004Ec000000P?
000eOol00`00Oomoo`0>Ool00`00Oomoo`03000?Ool00`00Oomoo`0@Ool001Aoo`@000=oo`05001o
ogooOol0000AOol5000015:k001BVP000U9i00X004hhCSP004hG0019m@00BK@00099T`060015L`00
00083`00=Goo00<007ooOol03Woo00<007ooOol09Goo000DOol00`00Oomoo`04Ool01@00Oomoogoo
00003goo1@0000AFg000D[]B^``00008BK@004VCBI<004Ec000i<P8003Eoo`03001oogoo00moo`03
001oogoo02Aoo`0057oo00<007ooOol017oo00D007ooOomoo`0000aoo`D0009Fo003001Fg00000<0
000:BQP004GhAOP0047g000me`00>KL600000dEc000i<P02000eOol00`00Oomoo`0@Ool00`00Oomo
o`0SOol001Aoo`@000Aoo`<000]oo`D0009Fo008001Fo000001>F@00CST0009:600;0015n000@OL0
03gG000i]`00=IL00002<IL01000AG<003Tb0`00=7oo00<007ooOol047oo00<007ooOol08goo000W
Ool300001Goo000005Kl00000UKl0P0000UBNP00CUU>F@00CST004XH00000TGh00X0047g000me`00
>KL003FG000aU`<00004>C8000P?000cOol2000COol00`00Oomoo`0ROol002Aoo`L00003E_`00000
00<00004DYX0059jDWX>00001cVg000eU`00<IL002Ug00<0000320l007oo02moo`D001Aoo`03001o
ogoo00Aoo`<000Ioo`8000Eoo`<000Yoo`008goo0`000UKm00@005Km0000009Bf`04001B^`00DYX4
000034YI0016>DHi00126@00@QT003gi000in@D0008aU`8000037GP00000008002aoo`H00004Dad0
0000000BOol00`00Oomoo`03Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`0000Uoo`00
8goo0P0000=Fo@00E_d00P000UKL00<005;K00000`0000A>NP00BUX0009:F@090016>DHi00126@00
@QT003gi0080008en@03000ef@0000D000034ET0000002Uoo`H00005Dah00000001CGP03000BOol0
0`00Oomoo`02Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Uoo`0077oo0`0017oo0P0000=F
o@0000000UKL00P005KL000004jK001>NP000Tij00H004YJ001:F@00AST8000013Wi000enCGi0P00
00<afP00;MX0100000=N0000Ool097oo1P000U<N0P000U=O1@004goo00D007ooOomoo`0000=oo`03
001oogoo00Uoo`05001oogooOol00009Ool001]oo`05001oogooOol00003Ool01000Ool005Kl1@00
00AB^`00CY]>V`X00006@SX0048j000n>P000ShJ1`0000<afP0000000RGK0P0000=N0000000087oo
1P000U<N0`0000I?O`00000004]o0002Cel2000BOol01@00Oomoogoo00000goo00<007ooOol02Goo
10002Woo000KOol01@00Oomoogoo00000Woo0P000Woo00L005;LD]`005:k001B^`0400002DYj001:
NTYj0016FP00AUX00002@SX03000@SX003hj000n6P00>QX003XJ000f6`D000049M/000000002JLX3
000KOol50002Dch300001Dmo00000000Agl02P0047oo0P000Woo00D007ooOomoo`0000]oo`03001o
ogoo00]oo`006goo00D007ooOomoo`00009oo`8000=oo`03001Bg00000<00007CY/004ikCW/004Yj
00000TYj400000Lj6P00=Q/0038K000b6`030003JLX01000A>]4j`005goo1@000U<n1`0000A7O`00
00000097G`H00003Bel00000015oo`03001oo`0000=oo`03001oogoo00Yoo`03001oogoo00Yoo`00
6goo00D007ooOomoo`00009oo`03001oogoo00=oo`03001B^`00009>V`04001>V`00CW/600001TIK
0016F`00@U/00092>`06000n>`00?S/003Xk2@0000<^?000JLX00VW:00<004C[A>/01P003goo2`00
0S]o1@000T=O0`0000=7G`000000100000=7?`00000047oo00<007ooOol00`003Woo0`002Goo000K
Ool01@00Oomoogoo00000Woo0P0017oo00<005:k00001P0000I:N`00BW/004Ik0002AU/01@00AU/0
049K00000T8k00h003hk000n>`00>S/003Xk000f?000=S`003IL0P000VW:00H004C[A>]4j`00>6X9
0007Ool@00000c]O0000000400001T=O00000000Acl00097?`<00003Acl007oo011oo`03001oogoo
01aoo`006goo00D007ooOomoo`00009oo`8000Eoo`03001>V`00009>V`04001:V`00BW/L00001CIL
000006W:00000TC[00@003QZ>6X0008`B@03000/1`0002800003@cl0000000@00003Acl0000000<0
015oo`03001oogoo01]oo`000goo0`001Woo0P0017oo1@000goo1@000goo00<007ooOol01Goo00@0
04jK001>V`@0000?BW/004IkAW/004Ik0016N`00@U/0049K0012F`00008nF`07000nF`00?U`003YL
00000SYL1@0000EYbP00A>]4j`00008hJP03000`B@00008/1`05000T0b@3000H000<00000c<O0000
0007000013lo000o?clo1@000TLo1@0000A7?`00001;?`80011oo`03001oogoo01]oo`000Woo00D0
07ooOomoo`0000Eoo`8000Eoo`04001oogoo0007Ool01@00Oomoogoo000027oo0`0000Q:V`00BY]:
V`00BW/004Ik6@0000@fG@00JLX00094j`03000hJP00008`B@05000/1b`7000T0`0@0002>al00`00
>al0000800000d<O00000004000014LO0017?dLo1@0000=;7`0000003goo0P001Woo0`001Woo0P00
1Goo0`000goo0002Ool01@00Oomoogoo00003Goo00<007ooOol01goo00@007ooOol000Uoo`04001>
V`00BY/700001TIk0016N`00AW/00092N`090012N`00@W`003iL000nG000008nO006000jO000>W`0
03Ym0P0000MYbP00A>]4j`00>6X00002<4T00`00;0L0000F0002?al8000014<O00137d<O1P0000=7
7`0000000`000T/O00<007ooOol03Goo00<007ooOol00goo00D007ooOomoo`0000Eoo`8000Aoo`05
001oogooOol00002Ool0009oo`05001oogooOol0000>Ool00`00Oomoo`06Ool00`00Oomoo`020009
Ool30002BY/01@00BY/004Ik00000TIk6P000dC[00H003QZ000`B@00;0L400001cJn000f_SJn000f
g`00008jg`03000jg`0000X00004?_l004<O@al800000dLO00000004000014LO00177dLO1@003goo
0P000goo00D007ooOomoo`0000]oo`05001oogooOol00002Ool0009oo`05001oogooOol0000?Ool0
1000Oomoogoo10000goo0P002Woo00@004ZK001:V`P0000;AW`0049l@W`0049l0012O000@W`00002
?W`01`00?W`003il000nO000008nO@03000jO@0000<00003>6X003190180008ng`03000no`00008n
o`P00004@al004<O@al50002Aal00`00Aal00005000014/O001;7d/O0P003goo00D007ooOomoo`00
00=oo`03001oogoo00Uoo`05001oogooOol00002Ool0009oo`05001oogooOol0000?Ool01@00Oomo
ogoo00001Woo00<007ooOol02Woo0`0000A:W000BY`00096W0060016O000AW`0049l=00000@no`00
@_m2o`P00004@al004LOAal5000014LO00177dLO1P0000=;7`00Ool03Woo00D007ooOomoo`0000=o
o`03001oogoo00Yoo`<000=oo`000Woo00D007ooOomoo`0000]oo`05001oogooOol00003Ool00`00
Oomoo`04Ool00`00Oomoo`0:Ool01000BY`004ZL2`0000A2O000@W`00092O0090012O000@W`003il
000nO@00008nO@04000nW@00?Yd4000013jM000nW@000Sjn00D003jn000n_P00008n_P03000ngP00
008ngPd00004@_l004;o@_l8000014LO00177dLO1@0000A;7`00Bam;7`D000moo`800003Ool007oo
009oo`03001oogoo00Uoo`05001oogooOol00002Ool000=oo`<000eoo`<000Aoo`@000=oo`03001o
ogoo00]oo`<0009:W00:0016W000AY`004JL0016O000@W`Z00001ckN000ngSkN000ngP000092o`P0
0004@_l004KoA_l8000014LO00177dLO1@0000=;7P0000000`003goo00<007oo00000goo00<007oo
Ool02Goo00D007ooOomoo`00009oo`002Goo0`001Woo0P001Goo100017oo00<007ooOol037oo00@0
04ZL001:W0/0000:@Y`004:L0012W000@Y`004:L0002@Yd00`00?Yd0000200002CjM000nWCjM000n
W@00?Yd00002?[h01000?[h003jn4P0000A2oP00@_i2oPP00004A_l004KoA_l800000dLN00000003
000014/N001;7T/N0P003Woo00<007ooOol00`003Goo0`000goo0008Ool01@00Oomoogoo00001Goo
0P002Goo00<007ooOol00P003goo0`0000I:W000AY`004JL0002AY`00`00AY`0000P0002?[h00`00
?[h00002?]h00`00?]h00002@]h:000014;n0012oT;n200000=6oP00A_h01P0000A;7P00Bai;7PD0
0003Bah007oo00ioo`03001oogoo01Eoo`0027oo00D007ooOomoo`00011oo`04001oogoo000@Ool0
0`00BY`0000:000024JL0012W000@Y`004:L0002@Yd300001T:M0012W@00@Yd0008nW@03000nW@00
01D00004@]h004;N0002@]h800001TKn0016oTKn0016oPD00096oP030016oP0000D00003Bah004/N
00D000moo`03001oogoo01Aoo`0027oo00D007ooOomoo`00011oo`03001oogoo0080011oo`0;001:
W4ZL0016W000AY`004JL0016W00H00002D:m0012_D:m0012_@00@[h00002@[h00`00@[h00002@[h<
00001T;N0012oT;n0016oPP00004A_h004KnA_h50002B_h00`00B_h0000700000d/N0000000>Ool2
0006Ool5000:Ool000Qoo`05001oogooOol0000<Ool40003Ool00`00Oomoo`0@Ool00`00BY`00007
00001DJL0016W000AY`0100000M2W@00@Yd004:M0012W@0H000014;N0012gP000T;N00<004;N0000
2@000TKn00<004Kn00001`0000A6oP00A_i6oPD00004B_h004/NBah50002Bah00`00Oomoo`0<Ool0
0`00Oomoo`06Ool00`00Oomoo`0:Ool000Qoo`05001oogooOol0000<Ool00`00Oomoo`04Ool00`00
Oomoo`0AOol02000BY`004JL0016W000AY`B00001T:M0012W@00@[d00092_@070012_@00@[d004:m
00000T:n3P000T;N00<004;N00002@0000A6oP00A_i6oPP00004B_h004[nB_h500000d[n001;7P06
000>Ool00`00Oomoo`05Ool00`00Oomoo`0:Ool000Qoo`05001oogooOol0000<Ool00`00Oomoo`04
Ool00`00Oomoo`0BOol:00002DJL0016W000AYd004:M0012W@0G000014:n0012_P000T;N00<004;N
00002@0000I6gP00A]i6gP00A_h8000014Kn0016oTKn200000=:oP00000030003Goo0P001Woo00<0
07ooOol02Woo0009Ool3000=Ool40003Ool00`00Oomoo`0COol00`00B[`0000@00002T:M0012W@00
@Yd004:m0012_@000T:m00<004:m00003`000T;N00@004;N0012gPX00096oP030016oP0000L00004
A_h004[nB_h5000014[n001:oT[n1`000T/N0P003Goo00<007ooOol017oo00<007ooOol02Woo0003
0006Ool20006Ool00`00Oomoo`03Ool40004Ool00`00Oomoo`0EOol500002DJL0016W000AYd004JM
0016W@0B00002T:m0012_D:m0012_@00@[h004:n30000TKN00<004KN00000TKN1`0000A6oP00A_i6
oPP00003B_h004[n00H00003B_h004[n00H00003B_h007oo00]oo`<000Eoo`03001oogoo00Yoo`00
0goo00<007ooOol00goo0P001Woo00<007ooOol01goo00@007ooOomoo`8001=oo`@00003B[`00000
00/0000:A[d004Jm0016_@00@[d004:m0002@[d?0002@[h01@00@]h004;N00000T;N2P0000=6gP00
A]h0200000A6oP00A_i6oPL00004B_h004[nB_h500000d[n001:oP060008Ool30002Ool00`00Oomo
o`03Ool00`00Oomoo`0:Ool000=oo`03001oogoo00aoo`03001oogoo00Ioo`05001oogooOol0000A
Ool30004Ool02@00A[`004Jl0016_@00A[d00002A[d?00001T:m0012_@00@[d00092_@030012_@00
00d00006A]h004KNA]h004KN1`000TKn00<004Kn00000TKn1@0000=:oP00B_h0200000A:oP00B_i:
oPL0009:oP8000Aoo`<000Eoo`04001oogooOol3000<Ool000=oo`03001oogoo00aoo`03001oogoo
00Ioo`05001oogooOol0000>Ool30008Ool:00002DJm0016_@00A[d004Jm0016_@0@000014:m0012
_@000T:n00<004KN00002`0000A6gP00A]i6gPX00003A_h004Kn00D00004B_h004[nB_h800000d[n
001:oP0600000d[n000000030008Ool00`00Oomoo`0@Ool000=oo`03001oogoo00aoo`03001oogoo
009oo`@000Aoo`03001oogoo00Uoo`<000aoo`060016_@00A[d004Jm3@0000U6_@00A[d004Jm0016
_@00@[d03P0000I6gP00A]i6gP00A]h:000014KN0016gP000TKn1`0000A:oP00B_i:oPD00004B_h0
04[nB_h70002B_h500000d[n001oo`0;Ool00`00Oomoo`0?Ool000=oo`03001oogoo00eoo`05001o
ogooOol00006Ool20008Ool3000@Ool500001dJm0016_@00A[d004Jm01000007A[d004Jm0016_@00
A[d0300000I6gP00A]i6gP00A]h:00000dKn0016oP08000014[n001:oT[n1@0000=:oP00B_h01P00
00=:oP0000001@002goo00<007ooOol03goo0003Ool00`00Oomoo`09Ool01@00Oomoogoo00000goo
00<007ooOol01Goo00<007ooOol00goo0`004goo00<004Jm00002P0000U6_@00A[d004Jm0016_@00
A[d03P0000I6_@00A]d004KN0002A]h900001TKN0016gTKN0016gPL00096oP03001:oP0000L00003
B_h004[n00D00004B_h004[nB_h70002B_h2000:Ool00`00Oomoo`0?Ool000<000aoo`D000=oo`@0
00=oo`04001oogooOol3000GOol00`00A[d00002A[d01000A[d004Jm30000TJm00H004Jm0016_@00
A[d>000014KN0016gP000TKN2@0000A6gP00A]i6gPL00004B_h004[nB_h7000014[n001:oT[n1@00
00A:oP00B_i:oPD00003B_h007oo00Uoo`03001oogoo00moo`007Woo10006goo200000M6_@00A[d0
04Jm0016_@0=000024Jm0016_DJm0016_@00A]d<00000dKN0016gP0:00000dKn001:oP0800000d[n
001:oP0800000d[n0000000600000d[n000000050009Ool00`00Oomoo`0?Ool001Ioo`D000=oo`80
01ioo`030016_@0000/00008A[d004JmA[d004Jm0016_@d00006A]d004KMA]d004KN2@0000I6gP00
A]i6gP00A]h7000014[n001:oT[n1`0000A:oP00B_i:oPD00004B_h004[nB_h500000d[n00000005
0008Ool00`00Oomoo`0?Ool001Qoo`03001oogoo00Eoo`03001oogoo01aoo`080016_@00A[d004Jm
0016_@d00005A[d004Jm0016_@0>000014KN0016gP000TKN2@0000A6gP00A]i6gPL00003B_h004[n
00P00003B_h004[n00H00003B_h004[n00L0009>oP8000Moo`03001oogoo00moo`0067oo00<007oo
Ool01Woo00<007ooOol077oo200000E6_@00A[d004Jm00d00007A[d004Jm0016g@00A]d0300000=6
gP00A]h02P0000=:gP00B_h01`000T[n00<004[n00001P000T[n1`0000A:oP00B_i:oPD00003C_h0
000000Moo`03001oogoo00ioo`0067oo00<007ooOol01goo00<007ooOol077oo00<004Jm00002@00
00M6_@00A[d004Jm0016_@0=0002A]d01000A]d004KM2@0000I6gP00A]i6gP00A]h7000014[n001:
oT[n1`0000A:oP00B_i:oPD00003B_h004[n00P00003B_h0000000<00003C_h007oo00Ioo`03001o
ogoo00ioo`0067oo00<007ooOol027oo0P007Goo00H004Jm0016_@00A[d<0002A[d01P00A[d004Jm
0016g@/00004A]d004KNA]h;000014[N001:gT[N1`0000=:oP00B_h0200000=:oP00B_h01@0000A:
oP00B_i:oPD00003C_h0000000D000Ioo`03001oogoo00ioo`0067oo00<007ooOol02Woo00<007oo
Ool06goo1@0000M6_@00A[d004Jm0016_@0=000014KM0016g@000TKM2@0000E6gP00A]h004KN00X0
0003B]h004[n00L00004B_h004[nB_h700000d[n0000000600000d[n001:oP070002C_h20005Ool0
0`00Oomoo`0>Ool001Qoo`03001oogoo00]oo`03001oogoo01]oo``00005A[d004Jm0016_@0=0000
0dKM0016g@0;000014KN0016gTKN2@0000A:oP00B_i:oPL00003B_h004[n00D00004B_h004[nB_h7
00000d[n0000000600000dkn00000004Ool00`00Oomoo`0>Ool001Ioo`<000ioo`03001oogoo01Yo
o`04001:_@00A[d=00001DJm0016_@00A]d02`0000A6g@00A]d00096g@T00003B]h004[N00X00003
B_h004[n00L00003B_h0000000H00003B_h004[n00D00004C_h004knC_h500000dkn001oo`03Ool0
0`00Oomoo`0>Ool002Qoo`8001]oo`@00007A[d004Jm0016_@00A[d02P0000I6g@00A]e6g@00A]d;
00000dKM0016gP09000014[N001:gT[N2@0000=:oP00B_h01@0000A:oP00B_i:oPL00003B_h00000
00H00003C_h0000000D000Aoo`03001oogoo00eoo`00:Woo00<007ooOol06Woo2@0000E6_@00A[d0
04Jm00/00005A]d004KM0016g@09000014[N001:gP000T[N1`0000=:gP00B_h02@000T[n1`0000=:
oP00B_h01@0000A:oP00B_i:oPD00003C_h0000000@000Aoo`03001oogoo00eoo`00:goo00<007oo
Ool06Goo00<004Zm00002`0000I6g@00A]e6g@00A]d:000014KM0016gDKM2`0000=:gP00B]h01`00
00=:oP00B_h0200000=:oP00B_h01`0000A:oP00B_i:oPD00003C_h004kn00D00003C_h004kn0080
00=oo`03001oogoo00eoo`00;7oo00<007ooOol06Goo00H004Zm001:_@00A[d=00000dKM0016g@0;
00001DKM001:g@00B]d02@0000A:gP00B]i:gPH0009:oP03001:oP0000H00003B_h004[n00P00003
B_h0000000D00003C_h0000000D00003C_h00000009oo`03001oogoo00eoo`00;Goo00<007ooOol0
6Goo1P0000E:g@00A]d004KM00X00005A]d004KM0016g@0;000014[M001:gD[M2@0000=:gP00B]h0
1`0000=:oP00B_h01`0000=:oP0000001P0000=:oP00C_h01@0000=>oP00C_h01@0000E>oP00Oomo
o`0000moo`00;Woo0P006goo2@0000E6g@00A]d004KM00X0009:g@03001:g@0000X00003B]d004[N
00T00003B_h004[n00L00004B_h004[nB_h4000014[n001:oT[n1`0000=>oP0000001@0000=>oP00
00001@0000=oo`00Ool03Woo000`Ool00`00Oomoo`0HOol00`00B]d0000;00000dKM0016g@0;0000
1D[M001:g@00B]d02@0000=:gP00B]h02@0000=:oP000000200000=:oP0000001@0000=:oP00B_h0
1@0000=>oP00C_h01@0000A>oP00001>oPD000moo`00<Goo00<007ooOol067oo00H004[M001:g@00
B]d:00001D[M001:g@00B]d02`0000=:g@00B]d02@0000A:gP00B]i:gPH00004B_h004[nB_h70000
0d[n001:oP0700000d[n0000000500000dkn0000000<000>Ool0039oo`03001oogoo01Qoo`D00003
B]d004[M00d00003B]d004[M00X0009:g@03001:g@0000P00003B]h004[n00L00003B_h004[n00L0
0003B_h0000000D00003C_h004kn00D00004C_h004knC_h60002C_h2000>Ool003=oo`8001Uoo`L0
0005B]d004[M001:g@0;00000d[M001:g@0;00000d[M001:g@0900000d[n0000000700000d[n0000
000500000d[n001:oP0700000dkn0000000>00000goo001oo`0=Ool003Eoo`03001oogoo01Qoo`/0
0003B]d004[M00X00003B]d004[M00/00003B]d004[M00L00003B_h004[n00L00003B_h004[n00L0
0003B_h0000000D00004C_h004knC_h40005Ool01@00Oomoogoo00003Woo000fOol00`00Oomoo`0G
Ool00`00B]d0000;00000d[M001:g@0:00000d[M001:g@0;00000d[M00000007000014[n001:oT[n
1P0000=:oP0000001@0000A:oP00C_i>oPL000Yoo`04001oogoo000>Ool003Moo`03001oogoo01Mo
o`06001:g@00B]d004[M2P0000=:g@00B]d02P000T[M00<004[M00001`0000A:g@00B]i:gPT00003
B_h0000000@00004B_h004[nB_h7000BOol00`00Ool0000>Ool003Qoo`8001Qoo`D00003B]d004[M
00X00003B]d004[M00/00003B]d004[M00T00003B]d004[n00L00003B_h004[n00L001Uoo`03001o
o`0000ioo`00>Woo00<007ooOol05Woo1`0000=:g@00B]d02P0000=:g@00B]d02`0000=:g@00B]d0
2@0000=:oP0000001`008Goo00<007oo00003Goo000kOol00`00Oomoo`0FOol900000d[M001:g@0:
0002B]d00`00B]d0000900000d[M00000007000[Ool2000=Ool003aoo`03001oogoo01Ioo`/00003
B]d004[M00/00003B]d004[M00P003=oo`<000eoo`00?Goo0P005goo00<004[M000030000T[M2P00
=goo100047oo000oOol00`00Oomoo`0EOol01000B]d004[M3000?Goo100057oo0010Ool00`00Oomo
o`0EOol30015Ool4000HOol0045oo`03001oogoo01Aoo`03001oogoo045oo`@001aoo`00@Woo0P00
57oo00<007ooOol0?Goo100087oo0014Ool00`00Oomoo`0AOol00`00Oomoo`0iOol4000TOol004Eo
o`03001oogoo011oo`03001oogoo03Eoo`@002Qoo`00AWoo00<007ooOol03Woo00<007ooOol0<Woo
1000;7oo0017Ool2000>Ool00`00Oomoo`0^Ool4000`Ool004Uoo`03001oogoo00]oo`03001oogoo
02Yoo`@003Aoo`00BWoo00<007ooOol02Woo00<007ooOol09goo0`00>7oo001;Ool00`00Oomoo`09
Ool00`00Oomoo`0SOol4000kOol004aoo`03001oogoo00Qoo`03001oogoo01moo`@003moo`00CGoo
0P0027oo00<007ooOol06goo1000@goo001?Ool00`00Oomoo`05Ool00`00Oomoo`0GOol40017Ool0
051oo`03001oogoo00Aoo`03001oogoo01=oo`@004]oo`00DGoo00<007ooOol00goo00<007ooOol0
3goo1000Cgoo001BOol20003Ool00`00Oomoo`0;Ool4001COol005Aoo`04001oogoo0009Ool4001G
Ool005Eoo`03001oo`0000Eoo`@005]oo`00EWoo0P0000=oo`0000000P00Ggoo001GOol2001SOol0
0;aoo`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo0000\
\>"],
  ImageRangeCache->{{{0, 187.438}, {151.812, 0}} -> {-0.20039, -0.0933552, 
  0.00826683, 0.00826683}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.5 Exploring Gamma", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallGamma[p,k,\[Sigma],r,q,t]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the Black-Scholes Gamma of a Call option, the sensitivity of delta \
to changes in the prices of the underlying stock. Let's plot it."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[BlackScholesCallGamma[x, 60, 0.29, 0.04, 0, 0.3], {x, 20., \
100.}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0119048 0.0147151 13.5474 [
[.2619 .00222 -6 -9 ]
[.2619 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.7381 .00222 -6 -9 ]
[.7381 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .15019 -24 -4.5 ]
[.01131 .15019 0 4.5 ]
[.01131 .28566 -24 -4.5 ]
[.01131 .28566 0 4.5 ]
[.01131 .42114 -24 -4.5 ]
[.01131 .42114 0 4.5 ]
[.01131 .55661 -24 -4.5 ]
[.01131 .55661 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(40)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(80)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .15019 m
.03006 .15019 L
s
[(0.01)] .01131 .15019 1 0 Mshowa
.02381 .28566 m
.03006 .28566 L
s
[(0.02)] .01131 .28566 1 0 Mshowa
.02381 .42114 m
.03006 .42114 L
s
[(0.03)] .01131 .42114 1 0 Mshowa
.02381 .55661 m
.03006 .55661 L
s
[(0.04)] .01131 .55661 1 0 Mshowa
.125 Mabswid
.02381 .04181 m
.02756 .04181 L
s
.02381 .0689 m
.02756 .0689 L
s
.02381 .096 m
.02756 .096 L
s
.02381 .12309 m
.02756 .12309 L
s
.02381 .17728 m
.02756 .17728 L
s
.02381 .20438 m
.02756 .20438 L
s
.02381 .23147 m
.02756 .23147 L
s
.02381 .25857 m
.02756 .25857 L
s
.02381 .31276 m
.02756 .31276 L
s
.02381 .33985 m
.02756 .33985 L
s
.02381 .36695 m
.02756 .36695 L
s
.02381 .39404 m
.02756 .39404 L
s
.02381 .44823 m
.02756 .44823 L
s
.02381 .47533 m
.02756 .47533 L
s
.02381 .50242 m
.02756 .50242 L
s
.02381 .52952 m
.02756 .52952 L
s
.02381 .58371 m
.02756 .58371 L
s
.02381 .6108 m
.02756 .6108 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01472 L
.02721 .01472 L
.02846 .01472 L
.03068 .01472 L
.03279 .01472 L
.03397 .01472 L
.03527 .01472 L
.03653 .01472 L
.0379 .01472 L
.04033 .01472 L
.04262 .01472 L
.04516 .01472 L
.04659 .01472 L
.04793 .01472 L
.05037 .01472 L
.05297 .01472 L
.05553 .01472 L
.05794 .01472 L
.0601 .01472 L
.06244 .01472 L
.06508 .01472 L
.06753 .01472 L
.07023 .01472 L
.07312 .01472 L
.07614 .01472 L
.07901 .01472 L
.08159 .01472 L
.08439 .01472 L
.08681 .01472 L
.08947 .01472 L
.09226 .01472 L
.09488 .01472 L
.09955 .01472 L
.10217 .01472 L
.10458 .01472 L
.10932 .01473 L
.11197 .01473 L
.11438 .01473 L
.11663 .01474 L
.11907 .01474 L
.1235 .01475 L
.12599 .01476 L
.12874 .01477 L
.13141 .01478 L
.13427 .0148 L
.1394 .01484 L
.1442 .01489 L
.14946 .01497 L
Mistroke
.15433 .01507 L
.15968 .01522 L
.16538 .01543 L
.17081 .0157 L
.17593 .01604 L
.18052 .01642 L
.18548 .01694 L
.19501 .01833 L
.20042 .0194 L
.20546 .02063 L
.21564 .02396 L
.22134 .02641 L
.22654 .02909 L
.23648 .03558 L
.24566 .04346 L
.25574 .05454 L
.26643 .06949 L
.28594 .10633 L
.30424 .15284 L
.34298 .28451 L
.38417 .44209 L
.40473 .51006 L
.41475 .53768 L
.42384 .55894 L
.43271 .57586 L
.44231 .5897 L
.44759 .59527 L
.4499 .59725 L
.45245 .5991 L
.45469 .60046 L
.45709 .60161 L
.45842 .60213 L
.45963 .60251 L
.46086 .60283 L
.46201 .60305 L
.4631 .6032 L
.46428 .6033 L
.4656 .60332 L
.4668 .60326 L
.46799 .60314 L
.46929 .60292 L
.47064 .6026 L
.4719 .60222 L
.47422 .60133 L
.47633 .60029 L
.48107 .59719 L
.48618 .5927 L
.49174 .58654 L
.50182 .57216 L
.52167 .53365 L
Mistroke
.5427 .48202 L
.58056 .37651 L
.62087 .26821 L
.65966 .18263 L
.67938 .14781 L
.7009 .11636 L
.72174 .09187 L
.74063 .07412 L
.76032 .05946 L
.7813 .04742 L
.80056 .03901 L
.82138 .03216 L
.84134 .02731 L
.86241 .02357 L
.88206 .02104 L
.9004 .01931 L
.92077 .01791 L
.93933 .017 L
.95699 .01637 L
.97619 .01587 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  AspectRatioFixed->True,
  ImageSize->{172.812, 106.75},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003H0000QA000`40O003h00Of7oo003HOol00=Qo
o`00f7oo003HOol00=Qoo`00f7oo0016Ool30004Ool3000POol30005Ool3000POol30005Ool3000L
Ool50003Ool30005Ool30003Ool004Moo`03001oogoo009oo`05001oogooOol0000NOol01@00Oomo
ogoo00000goo00D007ooOomoo`0001ioo`05001oogooOol00003Ool01@00Oomoogoo00007Goo00<0
07ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Aoo`D000=oo`05001oogoo
Ool0000NOol01@00Oomoogoo00000goo00D007ooOomoo`0001ioo`05001oogooOol00003Ool01@00
Oomoogoo00007Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Ao
o`04001oogoo0004Ool01@00Oomoogoo00007Woo00D007ooOomoo`0000=oo`05001oogooOol0000N
Ool01@00Oomoogoo00000goo00D007ooOomoo`0001eoo`03001oogoo009oo`05001oogooOol00003
Ool01@00Oomoogoo00000Woo0015Ool00`00Ool00004Ool01@00Oomoogoo00007Woo100017oo00D0
07ooOomoo`0001moo`<000Aoo`05001oogooOol0000MOol00`00Oomoo`02Ool01@00Oomoogoo0000
0goo00D007ooOomoo`00009oo`00AGoo00<007oo000017oo00D007ooOomoo`0001ioo`03001oogoo
00Eoo`05001oogooOol0000NOol01@00Oomoogoo00000goo00D007ooOomoo`0001eoo`03001oogoo
009oo`05001oogooOol00003Ool01@00Oomoogoo00000Woo0016Ool20004Ool01@00Oomoogoo0000
7goo00<007ooOol017oo00D007ooOomoo`0001ioo`05001oogooOol00003Ool01@00Oomoogoo0000
7Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Moo`03001oogoo
00=oo`<0025oo`<000Aoo`<0021oo`<000Eoo`<001aoo`<000Eoo`<000Eoo`<000=oo`00f7oo000P
Ool00`00Oomoo`2eOol0021oo`03001oogoo0;Eoo`0087oo00<007ooOol0]Goo000KOonf0007Ool0
021oo`03001oogoo01eoo`<000Qoo`03001oogoo02Qoo`03001oogoo02Moo`03001oogoo019oo`/0
00]oo`03001oogoo00Yoo`0087oo00<007ooOol087oo0`00Jgoo10008goo000POol00`00Oomoo`0S
Ool00`00Oomoo`1UOol3000WOol0021oo`03001oogoo02Aoo`03001oogoo065oo`<002Yoo`0087oo
00<007ooOol09Goo00<007ooOol0GWoo0P00;Goo000POol00`00Oomoo`0VOol00`00Oomoo`1KOol2
000_Ool0021oo`03001oogoo02Moo`03001oogoo05Qoo`80035oo`0087oo00<007ooOol09goo00<0
07ooOol0EWoo0P00<goo000POol00`00Oomoo`0XOol00`00Oomoo`1DOol00`00Oomoo`0cOol0021o
o`03001oogoo02Qoo`03001oogoo05=oo`03001oogoo03Aoo`0087oo00<007ooOol0:7oo00<007oo
Ool0DWoo00<007ooOol0=Goo000POol00`00Oomoo`0YOol00`00Oomoo`1@Ool00`00Oomoo`0fOol0
021oo`03001oogoo02Yoo`03001oogoo04ioo`03001oogoo03Moo`0087oo00<007ooOol0:Woo00<0
07ooOol0CGoo00<007ooOol0>7oo000POol00`00Oomoo`0[Ool00`00Oomoo`1<Ool00`00Oomoo`0h
Ool0021oo`03001oogoo02aoo`03001oogoo04Yoo`03001oogoo03Uoo`0087oo00<007ooOol0;7oo
00<007ooOol0BGoo00<007ooOol0>Woo000POol00`00Oomoo`0]Ool00`00Oomoo`17Ool00`00Oomo
o`0kOol0021oo`03001oogoo02eoo`03001oogoo04Moo`03001oogoo03]oo`0087oo00<007ooOol0
;Goo00<007ooOol0AWoo00<007ooOol0?7oo000POol00`00Oomoo`0^Ool00`00Oomoo`14Ool00`00
Oomoo`0mOol000<000Ioo`8000Eoo`<000Aoo`D000Aoo`03001oogoo02ioo`03001oogoo04Aoo`03
001oogoo03eoo`000goo00<007ooOol00goo0P0017oo00D007ooOomoo`0000Eoo`03001oogoo00Ao
o`03001oogoo02ioo`03001oogoo04=oo`03001oogoo03ioo`000goo00<007ooOol02Goo00D007oo
Oomoo`0000Eoo`03001oogoo00Aoo`80031oo`03001oogoo045oo`03001oogoo03moo`000goo00<0
07ooOol02Goo00D007ooOomoo`0000Eoo`03001oogoo00Aoo`03001oogoo02moo`03001oogoo045o
o`03001oogoo03moo`000goo00<007ooOol02Goo00D007ooOomoo`0000Eoo`03001oogoo00Aoo`03
001oogoo02moo`03001oogoo041oo`03001oogoo041oo`000goo00<007ooOol02Goo00D007ooOomo
o`0000Eoo`03001oogoo00Aoo`03001oogoo02moo`03001oogoo03moo`03001oogoo045oo`000goo
00<007ooOol02Goo00D007ooOomoo`0000Eoo`03001oogoo00Aoo`03001oogoo031oo`03001oogoo
03ioo`03001oogoo045oo`000`003Goo0`0017oo0`001Woo00<007ooOol0<7oo00<007ooOol0?Goo
00<007ooOol0@Woo000POol00`00Oomoo`0`Ool00`00Oomoo`0lOol00`00Oomoo`13Ool0021oo`03
001oogoo035oo`03001oogoo03]oo`03001oogoo04=oo`0087oo00<007ooOol0<Goo00<007ooOol0
>Woo00<007ooOol0A7oo000POol00`00Oomoo`0aOol00`00Oomoo`0jOol00`00Oomoo`14Ool0021o
o`03001oogoo039oo`03001oogoo03Qoo`03001oogoo04Eoo`0087oo00<007ooOol0<Woo00<007oo
Ool0>7oo00<007ooOol0AGoo000POol00`00Oomoo`0bOol00`00Oomoo`0gOol00`00Oomoo`16Ool0
021oo`03001oogoo03=oo`03001oogoo03Ioo`03001oogoo04Ioo`0087oo00<007ooOol0<goo00<0
07ooOol0=Goo00<007ooOol0Agoo000POol00`00Oomoo`0cOol00`00Oomoo`0eOol00`00Oomoo`17
Ool0021oo`03001oogoo03=oo`03001oogoo03Eoo`03001oogoo04Moo`0087oo00<007ooOol0=7oo
00<007ooOol0<goo00<007ooOol0B7oo000POol00`00Oomoo`0dOol00`00Oomoo`0cOol00`00Oomo
o`18Ool0021oo`03001oogoo03Aoo`03001oogoo039oo`03001oogoo04Uoo`0087oo00<007ooOol0
=Goo00<007ooOol0<Goo00<007ooOol0BGoo000POol00`00Oomoo`0eOol00`00Oomoo`0`Ool00`00
Oomoo`1:Ool000<000Ioo`8000Eoo`<000=oo`D000Eoo`03001oogoo03Eoo`03001oogoo031oo`03
001oogoo04Yoo`000goo00<007ooOol00goo0P0017oo00D007ooOomoo`0000=oo`04001oogoo0005
Ool00`00Oomoo`0fOol00`00Oomoo`0^Ool00`00Oomoo`1;Ool000=oo`03001oogoo00Uoo`05001o
ogooOol00004Ool00`00Oomoo`05Ool2000gOol00`00Oomoo`0^Ool00`00Oomoo`1;Ool000=oo`03
001oogoo00Uoo`05001oogooOol00005Ool00`00Oomoo`04Ool00`00Oomoo`0fOol00`00Oomoo`0^
Ool00`00Oomoo`1;Ool000=oo`03001oogoo00Uoo`05001oogooOol00006Ool00`00Oomoo`03Ool0
0`00Oomoo`0fOol00`00Oomoo`0]Ool00`00Oomoo`1<Ool000=oo`03001oogoo00Uoo`05001oogoo
Ool00006Ool00`00Oomoo`03Ool00`00Oomoo`0gOol00`00Oomoo`0/Ool00`00Oomoo`1<Ool000=o
o`03001oogoo00Uoo`05001oogooOol00002Ool01@00Oomoogoo00001Goo00<007ooOol0=goo00<0
07ooOol0;7oo00<007ooOol0C7oo0003000=Ool30004Ool30006Ool00`00Oomoo`0gOol00`00Oomo
o`0[Ool00`00Oomoo`1=Ool0021oo`03001oogoo03Qoo`03001oogoo02Yoo`03001oogoo04eoo`00
87oo00<007ooOol0>7oo00<007ooOol0:Woo00<007ooOol0CGoo000POol00`00Oomoo`0hOol00`00
Oomoo`0YOol00`00Oomoo`1>Ool0021oo`03001oogoo03Qoo`03001oogoo02Uoo`03001oogoo04io
o`0087oo00<007ooOol0>Goo00<007ooOol0:7oo00<007ooOol0CWoo000POol00`00Oomoo`0iOol0
0`00Oomoo`0WOol00`00Oomoo`1?Ool0021oo`03001oogoo03Uoo`03001oogoo02Moo`03001oogoo
04moo`0087oo00<007ooOol0>Goo00<007ooOol09goo00<007ooOol0Cgoo000POol00`00Oomoo`0j
Ool00`00Oomoo`0UOol00`00Oomoo`1@Ool0021oo`03001oogoo03Yoo`03001oogoo02Eoo`03001o
ogoo051oo`0087oo00<007ooOol0>Woo00<007ooOol09Goo00<007ooOol0D7oo000POol00`00Oomo
o`0kOol00`00Oomoo`0SOol00`00Oomoo`1AOol0021oo`03001oogoo03]oo`03001oogoo02=oo`03
001oogoo055oo`0087oo00<007ooOol0>goo00<007ooOol08Woo00<007ooOol0DWoo000POol00`00
Oomoo`0kOol00`00Oomoo`0ROol00`00Oomoo`1BOol0021oo`03001oogoo03aoo`03001oogoo025o
o`03001oogoo059oo`000`001Woo0P001Goo0`0017oo0`001Woo00<007ooOol0?7oo00<007ooOol0
87oo00<007ooOol0Dgoo0003Ool00`00Oomoo`03Ool20004Ool01@00Oomoogoo00000Woo00D007oo
Oomoo`0000Eoo`03001oogoo03aoo`03001oogoo021oo`03001oogoo05=oo`000goo00<007ooOol0
2Goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`03001oogoo03aoo`03001oogoo021oo`03001o
ogoo05=oo`000goo00<007ooOol02Goo00D007ooOomoo`0000Ioo`03001oogoo00=oo`8003ioo`03
001oogoo01ioo`03001oogoo05Aoo`000goo00<007ooOol02Goo00D007ooOomoo`0000Aoo`8000Io
o`03001oogoo03eoo`03001oogoo01ioo`03001oogoo05Aoo`000goo00<007ooOol02Goo00D007oo
Oomoo`0000Ioo`03001oogoo00=oo`03001oogoo03eoo`03001oogoo01eoo`03001oogoo05Eoo`00
0goo00<007ooOol02Goo00D007ooOomoo`00009oo`05001oogooOol00005Ool00`00Oomoo`0nOol0
0`00Oomoo`0LOol00`00Oomoo`1EOol000<000eoo`<000Aoo`<000Ioo`03001oogoo03ioo`03001o
ogoo01aoo`03001oogoo05Eoo`0087oo00<007ooOol0?Woo00<007ooOol06goo00<007ooOol0EWoo
000POol00`00Oomoo`0nOol00`00Oomoo`0KOol00`00Oomoo`1FOol0021oo`03001oogoo03moo`03
001oogoo01Yoo`03001oogoo05Ioo`0087oo00<007ooOol0?goo00<007ooOol06Goo00<007ooOol0
Egoo000POol00`00Oomoo`0oOol00`00Oomoo`0IOol00`00Oomoo`1GOol0021oo`03001oogoo03mo
o`03001oogoo01Uoo`03001oogoo05Moo`0087oo00<007ooOol0@7oo00<007ooOol05goo00<007oo
Ool0F7oo000POol00`00Oomoo`10Ool00`00Oomoo`0GOol00`00Oomoo`1HOol0021oo`03001oogoo
041oo`03001oogoo01Ioo`03001oogoo05Uoo`0087oo00<007ooOol0@7oo00<007ooOol05Woo00<0
07ooOol0FGoo000POol00`00Oomoo`11Ool00`00Oomoo`0DOol00`00Oomoo`1JOol0021oo`03001o
ogoo045oo`03001oogoo01Aoo`03001oogoo05Yoo`0087oo00<007ooOol0@Goo00<007ooOol057oo
00<007ooOol0FWoo000POol00`00Oomoo`12Ool00`00Oomoo`0BOol00`00Oomoo`1KOol0021oo`03
001oogoo049oo`03001oogoo019oo`03001oogoo05]oo`0087oo00<007ooOol0@Woo00<007ooOol0
4Goo00<007ooOol0G7oo000POol00`00Oomoo`13Ool00`00Oomoo`0@Ool00`00Oomoo`1LOol000<0
00Ioo`8000Eoo`<000Eoo`<000Eoo`03001oogoo04=oo`03001oogoo00moo`03001oogoo05eoo`00
0goo00<007ooOol00goo0P0017oo00D007ooOomoo`0000Eoo`03001oogoo00Aoo`03001oogoo04Ao
o`03001oogoo00ioo`03001oogoo05eoo`000goo00<007ooOol02Goo00D007ooOomoo`00009oo`D0
00Eoo`8004Eoo`03001oogoo00eoo`03001oogoo05ioo`000goo00<007ooOol02Goo00D007ooOomo
o`00009oo`04001oogoo0006Ool00`00Oomoo`15Ool00`00Oomoo`0<Ool00`00Oomoo`1NOol000=o
o`03001oogoo00Uoo`05001oogooOol00003Ool00`00Ool00006Ool00`00Oomoo`15Ool00`00Oomo
o`0<Ool00`00Oomoo`1NOol000=oo`03001oogoo00Uoo`05001oogooOol00003Ool00`00Ool00006
Ool00`00Oomoo`16Ool00`00Oomoo`0:Ool00`00Oomoo`1OOol000=oo`03001oogoo00Uoo`05001o
ogooOol00004Ool20006Ool00`00Oomoo`16Ool00`00Oomoo`0:Ool00`00Oomoo`1OOol000<000eo
o`<000Ioo`03001oogoo00Aoo`03001oogoo04Moo`03001oogoo00Qoo`03001oogoo061oo`0087oo
00<007ooOol0B7oo00<007ooOol01Woo00<007ooOol0HGoo000POol00`00Oomoo`19Ool00`00Oomo
o`04Ool00`00Oomoo`1ROol0021oo`03001oogoo04Uoo`<0009oo`8006Eoo`0087oo00<007ooOol0
Bgoo0`00Igoo000POol00`00Oomoo`2eOol0021oo`03001oogoo0;Eoo`00f7oo003HOol00=Qoo`00
f7oo003HOol00001\
\>"],
  ImageRangeCache->{{{0, 215}, {132.438, 0}} -> {4.95527, -0.00764439, 
  0.583097, 0.000512397}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[BlackScholesCallGamma[p, 60, 0.29, 0.04, 0, t],{p, 50, \
70},{t, 0.001, 0.2}, PlotPoints -> 40, PlotRange -> {0, 0.2}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
3.33067e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -9.92245 -9 ]
[.68156 -0.01098 2.07755 0 ]
[.78196 .11925 0 -6.14031 ]
[.78196 .11925 24 2.85969 ]
[.8558 .23232 0 -6.03228 ]
[.8558 .23232 18 2.96772 ]
[.92208 .33382 0 -5.9376 ]
[.92208 .33382 24 3.0624 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 18 3.14607 ]
[.02494 .25992 -6 -2.73794 ]
[.02494 .25992 0 6.26206 ]
[.01611 .31527 -24 -2.78472 ]
[.01611 .31527 0 6.21528 ]
[.00696 .37264 -18 -2.83339 ]
[.00696 .37264 0 6.16661 ]
[-0.00254 .43215 -24 -2.88407 ]
[-0.00254 .43215 0 6.11593 ]
[-0.0124 .49393 -18 -2.93688 ]
[-0.0124 .49393 0 6.06312 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.76963 .12375 m
.76347 .12599 L
s
[(0.05)] .78196 .11925 -1 .36451 Mshowa
.84338 .23655 m
.83717 .23866 L
s
[(0.1)] .8558 .23232 -1 .34051 Mshowa
.90958 .33782 m
.90333 .33981 L
s
[(0.15)] .92208 .33382 -1 .31947 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(0.2)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.7043 .02381 m
.70063 .02523 L
s
.72121 .04968 m
.71754 .05108 L
s
.73773 .07494 m
.73404 .07633 L
s
.75387 .09963 m
.75017 .10099 L
s
.78505 .14732 m
.78134 .14865 L
s
.80012 .17037 m
.79641 .17169 L
s
.81485 .19292 m
.81114 .19422 L
s
.82927 .21497 m
.82555 .21625 L
s
.85718 .25766 m
.85345 .25892 L
s
.8707 .27834 m
.86696 .27958 L
s
.88393 .29858 m
.88019 .2998 L
s
.89689 .3184 m
.89314 .31961 L
s
.92202 .35684 m
.91826 .35803 L
s
.93421 .37548 m
.93045 .37666 L
s
.94615 .39376 m
.94239 .39492 L
s
.95786 .41167 m
.9541 .41282 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03716 .25514 m
.04327 .25274 L
s
[(0)] .02494 .25992 1 -0.39157 Mshowa
.02837 .31059 m
.0345 .30825 L
s
[(0.05)] .01611 .31527 1 -0.38117 Mshowa
.01926 .36808 m
.02541 .3658 L
s
[(0.1)] .00696 .37264 1 -0.37036 Mshowa
.00981 .42772 m
.01599 .4255 L
s
[(0.15)] -0.00254 .43215 1 -0.3591 Mshowa
0 .48963 m
.0062 .48748 L
s
[(0.2)] -0.0124 .49393 1 -0.34736 Mshowa
.125 Mabswid
.03543 .26607 m
.0391 .26464 L
s
.03368 .27708 m
.03735 .27566 L
s
.03193 .28817 m
.0356 .28676 L
s
.03016 .29934 m
.03383 .29793 L
s
.02658 .32192 m
.03026 .32053 L
s
.02477 .33334 m
.02845 .33195 L
s
.02295 .34483 m
.02663 .34345 L
s
.02111 .35641 m
.0248 .35504 L
s
.0174 .37983 m
.02109 .37847 L
s
.01552 .39167 m
.01922 .39032 L
s
.01363 .4036 m
.01733 .40225 L
s
.01173 .41561 m
.01543 .41428 L
s
.00788 .43991 m
.01159 .43859 L
s
.00593 .4522 m
.00964 .45089 L
s
.00397 .46458 m
.00768 .46328 L
s
.00199 .47706 m
.00571 .47576 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.577 .709 .932 r
.39646 .61869 .40433 .62628 .41748 .62489 .40964 .61732 Metetra
.576 .71 .934 r
.40964 .61732 .41748 .62489 .43071 .62349 .4229 .61595 Metetra
.577 .712 .935 r
.4229 .61595 .43071 .62349 .44403 .62205 .43625 .61455 Metetra
.578 .714 .936 r
.43625 .61455 .44403 .62205 .45743 .62055 .44969 .61309 Metetra
.58 .717 .937 r
.44969 .61309 .45743 .62055 .47092 .61894 .46321 .61153 Metetra
.584 .721 .937 r
.46321 .61153 .47092 .61894 .4845 .61721 .47683 .60985 Metetra
.589 .725 .938 r
.47683 .60985 .4845 .61721 .49817 .61532 .49053 .60801 Metetra
.594 .729 .938 r
.49053 .60801 .49817 .61532 .51192 .61325 .50433 .60599 Metetra
.601 .733 .938 r
.50433 .60599 .51192 .61325 .52575 .61097 .51821 .60376 Metetra
.608 .738 .937 r
.51821 .60376 .52575 .61097 .53967 .60846 .53217 .60129 Metetra
.616 .743 .937 r
.53217 .60129 .53967 .60846 .55368 .6057 .54622 .59857 Metetra
.624 .747 .936 r
.54622 .59857 .55368 .6057 .56776 .60268 .56035 .59557 Metetra
.632 .752 .934 r
.56035 .59557 .56776 .60268 .58191 .59938 .57455 .5923 Metetra
.64 .756 .933 r
.57455 .5923 .58191 .59938 .59614 .59581 .58883 .58874 Metetra
.648 .76 .931 r
.58883 .58874 .59614 .59581 .61045 .59195 .60319 .58488 Metetra
.656 .763 .928 r
.60319 .58488 .61045 .59195 .62482 .58782 .61761 .58073 Metetra
.664 .766 .926 r
.61761 .58073 .62482 .58782 .63925 .5834 .6321 .5763 Metetra
.671 .769 .924 r
.6321 .5763 .63925 .5834 .65376 .57872 .64666 .57158 Metetra
.677 .771 .921 r
.64666 .57158 .65376 .57872 .66832 .57379 .66128 .5666 Metetra
.683 .772 .919 r
.66128 .5666 .66832 .57379 .68294 .56861 .67596 .56137 Metetra
.689 .774 .916 r
.67596 .56137 .68294 .56861 .69762 .5632 .69069 .5559 Metetra
.693 .774 .914 r
.69069 .5559 .69762 .5632 .71235 .55759 .70549 .55021 Metetra
.697 .775 .911 r
.70549 .55021 .71235 .55759 .72715 .55179 .72033 .54433 Metetra
.701 .775 .909 r
.72033 .54433 .72715 .55179 .74199 .54582 .73524 .53827 Metetra
.704 .775 .907 r
.73524 .53827 .74199 .54582 .7569 .5397 .7502 .53206 Metetra
.706 .774 .905 r
.7502 .53206 .7569 .5397 .77186 .53346 .76522 .52572 Metetra
.708 .773 .903 r
.76522 .52572 .77186 .53346 .78687 .52711 .78029 .51927 Metetra
.709 .772 .902 r
.78029 .51927 .78687 .52711 .80194 .52068 .79542 .51274 Metetra
.71 .771 .9 r
.79542 .51274 .80194 .52068 .81708 .51418 .81061 .50613 Metetra
.71 .77 .899 r
.81061 .50613 .81708 .51418 .83227 .50763 .82586 .49948 Metetra
.71 .768 .898 r
.82586 .49948 .83227 .50763 .84752 .50105 .84117 .4928 Metetra
.71 .767 .897 r
.84117 .4928 .84752 .50105 .86284 .49445 .85655 .4861 Metetra
.709 .765 .897 r
.85655 .4861 .86284 .49445 .87822 .48785 .87199 .4794 Metetra
.708 .764 .896 r
.87199 .4794 .87822 .48785 .89367 .48126 .8875 .47271 Metetra
.707 .762 .896 r
.8875 .47271 .89367 .48126 .90919 .47469 .90308 .46604 Metetra
.705 .761 .895 r
.90308 .46604 .90919 .47469 .92478 .46815 .91873 .45941 Metetra
.704 .759 .895 r
.91873 .45941 .92478 .46815 .94045 .46164 .93446 .45282 Metetra
.702 .758 .895 r
.93446 .45282 .94045 .46164 .95619 .45518 .95026 .44628 Metetra
.7 .756 .895 r
.95026 .44628 .95619 .45518 .97202 .44877 .96615 .43979 Metetra
.576 .708 .932 r
.38851 .61101 .39646 .61869 .40964 .61732 .40172 .60966 Metetra
.574 .709 .934 r
.40172 .60966 .40964 .61732 .4229 .61595 .41502 .60832 Metetra
.574 .71 .935 r
.41502 .60832 .4229 .61595 .43625 .61455 .4284 .60696 Metetra
.575 .713 .936 r
.4284 .60696 .43625 .61455 .44969 .61309 .44187 .60554 Metetra
.577 .716 .937 r
.44187 .60554 .44969 .61309 .46321 .61153 .45544 .60404 Metetra
.581 .719 .938 r
.45544 .60404 .46321 .61153 .47683 .60985 .46909 .60241 Metetra
.585 .723 .938 r
.46909 .60241 .47683 .60985 .49053 .60801 .48283 .60063 Metetra
.591 .728 .939 r
.48283 .60063 .49053 .60801 .50433 .60599 .49666 .59866 Metetra
.598 .732 .938 r
.49666 .59866 .50433 .60599 .51821 .60376 .51059 .59648 Metetra
.605 .737 .938 r
.51059 .59648 .51821 .60376 .53217 .60129 .52459 .59406 Metetra
.613 .742 .937 r
.52459 .59406 .53217 .60129 .54622 .59857 .53869 .59137 Metetra
.621 .747 .936 r
.53869 .59137 .54622 .59857 .56035 .59557 .55286 .58841 Metetra
.63 .752 .935 r
.55286 .58841 .56035 .59557 .57455 .5923 .56712 .58516 Metetra
.638 .756 .933 r
.56712 .58516 .57455 .5923 .58883 .58874 .58145 .58161 Metetra
.647 .76 .931 r
.58145 .58161 .58883 .58874 .60319 .58488 .59586 .57776 Metetra
.655 .764 .929 r
.59586 .57776 .60319 .58488 .61761 .58073 .61034 .5736 Metetra
.663 .767 .927 r
.61034 .5736 .61761 .58073 .6321 .5763 .62488 .56914 Metetra
.67 .769 .924 r
.62488 .56914 .6321 .5763 .64666 .57158 .63949 .56439 Metetra
.677 .772 .922 r
.63949 .56439 .64666 .57158 .66128 .5666 .65417 .55937 Metetra
.684 .773 .919 r
.65417 .55937 .66128 .5666 .67596 .56137 .6689 .55408 Metetra
.689 .775 .917 r
.6689 .55408 .67596 .56137 .69069 .5559 .6837 .54854 Metetra
.694 .775 .914 r
.6837 .54854 .69069 .5559 .70549 .55021 .69855 .54278 Metetra
.698 .776 .911 r
.69855 .54278 .70549 .55021 .72033 .54433 .71346 .53682 Metetra
.702 .776 .909 r
.71346 .53682 .72033 .54433 .73524 .53827 .72842 .53067 Metetra
.705 .776 .907 r
.72842 .53067 .73524 .53827 .7502 .53206 .74344 .52436 Metetra
.707 .775 .905 r
.74344 .52436 .7502 .53206 .76522 .52572 .75851 .51792 Metetra
.709 .774 .903 r
.75851 .51792 .76522 .52572 .78029 .51927 .77365 .51137 Metetra
.71 .773 .901 r
.77365 .51137 .78029 .51927 .79542 .51274 .78883 .50473 Metetra
.711 .772 .9 r
.78883 .50473 .79542 .51274 .81061 .50613 .80408 .49801 Metetra
.711 .77 .899 r
.80408 .49801 .81061 .50613 .82586 .49948 .81939 .49125 Metetra
.711 .769 .898 r
.81939 .49125 .82586 .49948 .84117 .4928 .83476 .48446 Metetra
.711 .767 .897 r
.83476 .48446 .84117 .4928 .85655 .4861 .85019 .47766 Metetra
.71 .766 .896 r
.85019 .47766 .85655 .4861 .87199 .4794 .86569 .47086 Metetra
.709 .764 .895 r
.86569 .47086 .87199 .4794 .8875 .47271 .88126 .46407 Metetra
.708 .763 .895 r
.88126 .46407 .8875 .47271 .90308 .46604 .8969 .45731 Metetra
.706 .761 .895 r
.8969 .45731 .90308 .46604 .91873 .45941 .91262 .45058 Metetra
.704 .759 .895 r
.91262 .45058 .91873 .45941 .93446 .45282 .92841 .4439 Metetra
.703 .758 .895 r
.92841 .4439 .93446 .45282 .95026 .44628 .94427 .43728 Metetra
.701 .756 .895 r
.94427 .43728 .95026 .44628 .96615 .43979 .96022 .4307 Metetra
.574 .706 .932 r
.3805 .60324 .38851 .61101 .40172 .60966 .39374 .60191 Metetra
.572 .707 .934 r
.39374 .60191 .40172 .60966 .41502 .60832 .40707 .6006 Metetra
.571 .709 .935 r
.40707 .6006 .41502 .60832 .4284 .60696 .42048 .59928 Metetra
.572 .711 .936 r
.42048 .59928 .4284 .60696 .44187 .60554 .43399 .59791 Metetra
.574 .714 .937 r
.43399 .59791 .44187 .60554 .45544 .60404 .44759 .59646 Metetra
.578 .717 .938 r
.44759 .59646 .45544 .60404 .46909 .60241 .46127 .59489 Metetra
.582 .722 .939 r
.46127 .59489 .46909 .60241 .48283 .60063 .47506 .59316 Metetra
.588 .726 .939 r
.47506 .59316 .48283 .60063 .49666 .59866 .48893 .59125 Metetra
.594 .731 .939 r
.48893 .59125 .49666 .59866 .51059 .59648 .50289 .58913 Metetra
.602 .736 .939 r
.50289 .58913 .51059 .59648 .52459 .59406 .51694 .58676 Metetra
.61 .741 .938 r
.51694 .58676 .52459 .59406 .53869 .59137 .53108 .58412 Metetra
.619 .746 .937 r
.53108 .58412 .53869 .59137 .55286 .58841 .54531 .5812 Metetra
.628 .751 .936 r
.54531 .5812 .55286 .58841 .56712 .58516 .55961 .57798 Metetra
.637 .756 .934 r
.55961 .57798 .56712 .58516 .58145 .58161 .574 .57444 Metetra
.645 .76 .932 r
.574 .57444 .58145 .58161 .59586 .57776 .58846 .57059 Metetra
.654 .764 .93 r
.58846 .57059 .59586 .57776 .61034 .5736 .60299 .56643 Metetra
.662 .767 .928 r
.60299 .56643 .61034 .5736 .62488 .56914 .61759 .56195 Metetra
.67 .77 .925 r
.61759 .56195 .62488 .56914 .63949 .56439 .63226 .55717 Metetra
.677 .772 .922 r
.63226 .55717 .63949 .56439 .65417 .55937 .64699 .55209 Metetra
.684 .774 .92 r
.64699 .55209 .65417 .55937 .6689 .55408 .66178 .54675 Metetra
.69 .776 .917 r
.66178 .54675 .6689 .55408 .6837 .54854 .67663 .54114 Metetra
.695 .776 .914 r
.67663 .54114 .6837 .54854 .69855 .54278 .69154 .5353 Metetra
.699 .777 .912 r
.69154 .5353 .69855 .54278 .71346 .53682 .70651 .52925 Metetra
.703 .777 .909 r
.70651 .52925 .71346 .53682 .72842 .53067 .72153 .52301 Metetra
.706 .776 .907 r
.72153 .52301 .72842 .53067 .74344 .52436 .73661 .5166 Metetra
.708 .776 .905 r
.73661 .5166 .74344 .52436 .75851 .51792 .75174 .51006 Metetra
.71 .775 .903 r
.75174 .51006 .75851 .51792 .77365 .51137 .76693 .5034 Metetra
.712 .774 .901 r
.76693 .5034 .77365 .51137 .78883 .50473 .78218 .49664 Metetra
.712 .772 .9 r
.78218 .49664 .78883 .50473 .80408 .49801 .79749 .48982 Metetra
.713 .771 .898 r
.79749 .48982 .80408 .49801 .81939 .49125 .81285 .48295 Metetra
.712 .769 .897 r
.81285 .48295 .81939 .49125 .83476 .48446 .82828 .47605 Metetra
.712 .768 .896 r
.82828 .47605 .83476 .48446 .85019 .47766 .84378 .46913 Metetra
.711 .766 .895 r
.84378 .46913 .85019 .47766 .86569 .47086 .85934 .46222 Metetra
.71 .764 .895 r
.85934 .46222 .86569 .47086 .88126 .46407 .87497 .45533 Metetra
.708 .763 .895 r
.87497 .45533 .88126 .46407 .8969 .45731 .89067 .44848 Metetra
.707 .761 .894 r
.89067 .44848 .8969 .45731 .91262 .45058 .90644 .44166 Metetra
.705 .759 .894 r
.90644 .44166 .91262 .45058 .92841 .4439 .92229 .43489 Metetra
.703 .758 .894 r
.92229 .43489 .92841 .4439 .94427 .43728 .93822 .42817 Metetra
.701 .756 .894 r
.93822 .42817 .94427 .43728 .96022 .4307 .95423 .42151 Metetra
.572 .705 .932 r
.37241 .59537 .3805 .60324 .39374 .60191 .38569 .59406 Metetra
.57 .706 .934 r
.38569 .59406 .39374 .60191 .40707 .6006 .39904 .59278 Metetra
.569 .707 .935 r
.39904 .59278 .40707 .6006 .42048 .59928 .41249 .5915 Metetra
.569 .709 .936 r
.41249 .5915 .42048 .59928 .43399 .59791 .42603 .59018 Metetra
.571 .712 .938 r
.42603 .59018 .43399 .59791 .44759 .59646 .43966 .58879 Metetra
.574 .716 .939 r
.43966 .58879 .44759 .59646 .46127 .59489 .45339 .58728 Metetra
.579 .72 .939 r
.45339 .58728 .46127 .59489 .47506 .59316 .46721 .58562 Metetra
.584 .725 .94 r
.46721 .58562 .47506 .59316 .48893 .59125 .48112 .58377 Metetra
.591 .73 .94 r
.48112 .58377 .48893 .59125 .50289 .58913 .49512 .5817 Metetra
.599 .735 .94 r
.49512 .5817 .50289 .58913 .51694 .58676 .50922 .57939 Metetra
.607 .74 .939 r
.50922 .57939 .51694 .58676 .53108 .58412 .5234 .57681 Metetra
.616 .746 .938 r
.5234 .57681 .53108 .58412 .54531 .5812 .53768 .57393 Metetra
.625 .751 .937 r
.53768 .57393 .54531 .5812 .55961 .57798 .55203 .57073 Metetra
.635 .756 .935 r
.55203 .57073 .55961 .57798 .574 .57444 .56647 .56722 Metetra
.644 .76 .933 r
.56647 .56722 .574 .57444 .58846 .57059 .58098 .56338 Metetra
.653 .764 .931 r
.58098 .56338 .58846 .57059 .60299 .56643 .59556 .55921 Metetra
.662 .768 .929 r
.59556 .55921 .60299 .56643 .61759 .56195 .61022 .55471 Metetra
.67 .771 .926 r
.61022 .55471 .61759 .56195 .63226 .55717 .62495 .5499 Metetra
.677 .773 .923 r
.62495 .5499 .63226 .55717 .64699 .55209 .63973 .54478 Metetra
.684 .775 .92 r
.63973 .54478 .64699 .55209 .66178 .54675 .65459 .53937 Metetra
.69 .777 .917 r
.65459 .53937 .66178 .54675 .67663 .54114 .6695 .5337 Metetra
.696 .777 .914 r
.6695 .5337 .67663 .54114 .69154 .5353 .68447 .52778 Metetra
.7 .778 .912 r
.68447 .52778 .69154 .5353 .70651 .52925 .69949 .52164 Metetra
.704 .778 .909 r
.69949 .52164 .70651 .52925 .72153 .52301 .71457 .5153 Metetra
.707 .777 .907 r
.71457 .5153 .72153 .52301 .73661 .5166 .72971 .50879 Metetra
.71 .777 .904 r
.72971 .50879 .73661 .5166 .75174 .51006 .7449 .50213 Metetra
.712 .776 .902 r
.7449 .50213 .75174 .51006 .76693 .5034 .76015 .49536 Metetra
.713 .775 .901 r
.76015 .49536 .76693 .5034 .78218 .49664 .77546 .48849 Metetra
.714 .773 .899 r
.77546 .48849 .78218 .49664 .79749 .48982 .79083 .48155 Metetra
.714 .772 .898 r
.79083 .48155 .79749 .48982 .81285 .48295 .80625 .47456 Metetra
.714 .77 .897 r
.80625 .47456 .81285 .48295 .82828 .47605 .82174 .46754 Metetra
.713 .768 .896 r
.82174 .46754 .82828 .47605 .84378 .46913 .83729 .46052 Metetra
.712 .766 .895 r
.83729 .46052 .84378 .46913 .85934 .46222 .85291 .4535 Metetra
.711 .765 .894 r
.85291 .4535 .85934 .46222 .87497 .45533 .8686 .4465 Metetra
.709 .763 .894 r
.8686 .4465 .87497 .45533 .89067 .44848 .88437 .43954 Metetra
.708 .761 .894 r
.88437 .43954 .89067 .44848 .90644 .44166 .9002 .43263 Metetra
.706 .759 .894 r
.9002 .43263 .90644 .44166 .92229 .43489 .91611 .42576 Metetra
.704 .758 .894 r
.91611 .42576 .92229 .43489 .93822 .42817 .93211 .41896 Metetra
.702 .756 .894 r
.93211 .41896 .93822 .42817 .95423 .42151 .94818 .41222 Metetra
.57 .704 .932 r
.36426 .5874 .37241 .59537 .38569 .59406 .37756 .58611 Metetra
.568 .704 .934 r
.37756 .58611 .38569 .59406 .39904 .59278 .39095 .58486 Metetra
.566 .705 .935 r
.39095 .58486 .39904 .59278 .41249 .5915 .40443 .58363 Metetra
.566 .707 .937 r
.40443 .58363 .41249 .5915 .42603 .59018 .418 .58236 Metetra
.568 .71 .938 r
.418 .58236 .42603 .59018 .43966 .58879 .43167 .58102 Metetra
.571 .714 .939 r
.43167 .58102 .43966 .58879 .45339 .58728 .44543 .57958 Metetra
.575 .718 .94 r
.44543 .57958 .45339 .58728 .46721 .58562 .45928 .57798 Metetra
.58 .723 .941 r
.45928 .57798 .46721 .58562 .48112 .58377 .47323 .57621 Metetra
.587 .728 .941 r
.47323 .57621 .48112 .58377 .49512 .5817 .48728 .57421 Metetra
.595 .734 .941 r
.48728 .57421 .49512 .5817 .50922 .57939 .50142 .57196 Metetra
.604 .739 .94 r
.50142 .57196 .50922 .57939 .5234 .57681 .51565 .56943 Metetra
.613 .745 .939 r
.51565 .56943 .5234 .57681 .53768 .57393 .52997 .5666 Metetra
.623 .75 .938 r
.52997 .5666 .53768 .57393 .55203 .57073 .54437 .56344 Metetra
.633 .756 .936 r
.54437 .56344 .55203 .57073 .56647 .56722 .55886 .55995 Metetra
.642 .76 .934 r
.55886 .55995 .56647 .56722 .58098 .56338 .57342 .55612 Metetra
.652 .765 .932 r
.57342 .55612 .58098 .56338 .59556 .55921 .58807 .55195 Metetra
.661 .769 .929 r
.58807 .55195 .59556 .55921 .61022 .55471 .60278 .54743 Metetra
.669 .772 .927 r
.60278 .54743 .61022 .55471 .62495 .5499 .61756 .54259 Metetra
.677 .774 .924 r
.61756 .54259 .62495 .5499 .63973 .54478 .63241 .53742 Metetra
.684 .776 .921 r
.63241 .53742 .63973 .54478 .65459 .53937 .64732 .53196 Metetra
.691 .778 .918 r
.64732 .53196 .65459 .53937 .6695 .5337 .66229 .52621 Metetra
.697 .779 .915 r
.66229 .52621 .6695 .5337 .68447 .52778 .67732 .52021 Metetra
.701 .779 .912 r
.67732 .52021 .68447 .52778 .69949 .52164 .6924 .51397 Metetra
.705 .779 .909 r
.6924 .51397 .69949 .52164 .71457 .5153 .70754 .50753 Metetra
.709 .778 .906 r
.70754 .50753 .71457 .5153 .72971 .50879 .72274 .50091 Metetra
.711 .778 .904 r
.72274 .50091 .72971 .50879 .7449 .50213 .738 .49414 Metetra
.713 .777 .902 r
.738 .49414 .7449 .50213 .76015 .49536 .75331 .48725 Metetra
.714 .775 .9 r
.75331 .48725 .76015 .49536 .77546 .48849 .76867 .48026 Metetra
.715 .774 .899 r
.76867 .48026 .77546 .48849 .79083 .48155 .7841 .4732 Metetra
.715 .772 .897 r
.7841 .4732 .79083 .48155 .80625 .47456 .79958 .46609 Metetra
.715 .77 .896 r
.79958 .46609 .80625 .47456 .82174 .46754 .81513 .45895 Metetra
.714 .769 .895 r
.81513 .45895 .82174 .46754 .83729 .46052 .83075 .45181 Metetra
.713 .767 .894 r
.83075 .45181 .83729 .46052 .85291 .4535 .84643 .44468 Metetra
.712 .765 .894 r
.84643 .44468 .85291 .4535 .8686 .4465 .86218 .43757 Metetra
.71 .763 .893 r
.86218 .43757 .8686 .4465 .88437 .43954 .878 .43051 Metetra
.709 .761 .893 r
.878 .43051 .88437 .43954 .9002 .43263 .8939 .42349 Metetra
.707 .759 .893 r
.8939 .42349 .9002 .43263 .91611 .42576 .90987 .41653 Metetra
.705 .758 .893 r
.90987 .41653 .91611 .42576 .93211 .41896 .92593 .40964 Metetra
.703 .756 .893 r
.92593 .40964 .93211 .41896 .94818 .41222 .94207 .40281 Metetra
.568 .702 .932 r
.35603 .57933 .36426 .5874 .37756 .58611 .36936 .57806 Metetra
.565 .703 .934 r
.36936 .57806 .37756 .58611 .39095 .58486 .38278 .57684 Metetra
.563 .704 .935 r
.38278 .57684 .39095 .58486 .40443 .58363 .39629 .57565 Metetra
.563 .706 .937 r
.39629 .57565 .40443 .58363 .418 .58236 .4099 .57443 Metetra
.564 .708 .938 r
.4099 .57443 .418 .58236 .43167 .58102 .42359 .57316 Metetra
.567 .712 .939 r
.42359 .57316 .43167 .58102 .44543 .57958 .43739 .57178 Metetra
.571 .716 .94 r
.43739 .57178 .44543 .57958 .45928 .57798 .45128 .57026 Metetra
.576 .721 .941 r
.45128 .57026 .45928 .57798 .47323 .57621 .46527 .56856 Metetra
.583 .726 .942 r
.46527 .56856 .47323 .57621 .48728 .57421 .47936 .56664 Metetra
.591 .732 .942 r
.47936 .56664 .48728 .57421 .50142 .57196 .49354 .56446 Metetra
.6 .738 .941 r
.49354 .56446 .50142 .57196 .51565 .56943 .50782 .56199 Metetra
.61 .744 .941 r
.50782 .56199 .51565 .56943 .52997 .5666 .52218 .55921 Metetra
.62 .75 .939 r
.52218 .55921 .52997 .5666 .54437 .56344 .53664 .5561 Metetra
.63 .756 .938 r
.53664 .5561 .54437 .56344 .55886 .55995 .55118 .55264 Metetra
.64 .761 .936 r
.55118 .55264 .55886 .55995 .57342 .55612 .56579 .54882 Metetra
.65 .765 .933 r
.56579 .54882 .57342 .55612 .58807 .55195 .58049 .54465 Metetra
.66 .769 .93 r
.58049 .54465 .58807 .55195 .60278 .54743 .59526 .54012 Metetra
.669 .773 .927 r
.59526 .54012 .60278 .54743 .61756 .54259 .6101 .53524 Metetra
.677 .775 .924 r
.6101 .53524 .61756 .54259 .63241 .53742 .62501 .53003 Metetra
.685 .777 .921 r
.62501 .53003 .63241 .53742 .64732 .53196 .63998 .5245 Metetra
.692 .779 .918 r
.63998 .5245 .64732 .53196 .66229 .52621 .65501 .51868 Metetra
.697 .78 .915 r
.65501 .51868 .66229 .52621 .67732 .52021 .6701 .51259 Metetra
.703 .78 .912 r
.6701 .51259 .67732 .52021 .6924 .51397 .68524 .50626 Metetra
.707 .78 .909 r
.68524 .50626 .6924 .51397 .70754 .50753 .70044 .49971 Metetra
.71 .78 .906 r
.70044 .49971 .70754 .50753 .72274 .50091 .7157 .49298 Metetra
.713 .779 .904 r
.7157 .49298 .72274 .50091 .738 .49414 .73102 .48609 Metetra
.715 .778 .902 r
.73102 .48609 .738 .49414 .75331 .48725 .74639 .47907 Metetra
.716 .776 .9 r
.74639 .47907 .75331 .48725 .76867 .48026 .76181 .47196 Metetra
.717 .775 .898 r
.76181 .47196 .76867 .48026 .7841 .4732 .7773 .46477 Metetra
.717 .773 .897 r
.7773 .46477 .7841 .4732 .79958 .46609 .79285 .45753 Metetra
.717 .771 .895 r
.79285 .45753 .79958 .46609 .81513 .45895 .80846 .45027 Metetra
.716 .769 .894 r
.80846 .45027 .81513 .45895 .83075 .45181 .82413 .44301 Metetra
.715 .767 .894 r
.82413 .44301 .83075 .45181 .84643 .44468 .83987 .43576 Metetra
.713 .765 .893 r
.83987 .43576 .84643 .44468 .86218 .43757 .85568 .42854 Metetra
.712 .763 .893 r
.85568 .42854 .86218 .43757 .878 .43051 .87157 .42137 Metetra
.71 .761 .893 r
.87157 .42137 .878 .43051 .8939 .42349 .88753 .41425 Metetra
.708 .759 .893 r
.88753 .41425 .8939 .42349 .90987 .41653 .90356 .40719 Metetra
.705 .757 .893 r
.90356 .40719 .90987 .41653 .92593 .40964 .91968 .40021 Metetra
.703 .756 .893 r
.91968 .40021 .92593 .40964 .94207 .40281 .93589 .3933 Metetra
.567 .701 .932 r
.34773 .57115 .35603 .57933 .36936 .57806 .36109 .5699 Metetra
.563 .701 .933 r
.36109 .5699 .36936 .57806 .38278 .57684 .37454 .56871 Metetra
.561 .702 .935 r
.37454 .56871 .38278 .57684 .39629 .57565 .38808 .56756 Metetra
.56 .703 .937 r
.38808 .56756 .39629 .57565 .4099 .57443 .40172 .56641 Metetra
.56 .706 .938 r
.40172 .56641 .4099 .57443 .42359 .57316 .41545 .5652 Metetra
.563 .71 .94 r
.41545 .5652 .42359 .57316 .43739 .57178 .42928 .56389 Metetra
.567 .714 .941 r
.42928 .56389 .43739 .57178 .45128 .57026 .44321 .56245 Metetra
.572 .719 .942 r
.44321 .56245 .45128 .57026 .46527 .56856 .45723 .56083 Metetra
.579 .725 .942 r
.45723 .56083 .46527 .56856 .47936 .56664 .47136 .55899 Metetra
.587 .731 .943 r
.47136 .55899 .47936 .56664 .49354 .56446 .48559 .55688 Metetra
.596 .737 .942 r
.48559 .55688 .49354 .56446 .50782 .56199 .49991 .55448 Metetra
.606 .743 .942 r
.49991 .55448 .50782 .56199 .52218 .55921 .51432 .55176 Metetra
.617 .75 .94 r
.51432 .55176 .52218 .55921 .53664 .5561 .52882 .5487 Metetra
.628 .755 .939 r
.52882 .5487 .53664 .5561 .55118 .55264 .54341 .54528 Metetra
.638 .761 .937 r
.54341 .54528 .55118 .55264 .56579 .54882 .55809 .54148 Metetra
.649 .766 .934 r
.55809 .54148 .56579 .54882 .58049 .54465 .57284 .53731 Metetra
.659 .77 .931 r
.57284 .53731 .58049 .54465 .59526 .54012 .58767 .53277 Metetra
.669 .774 .928 r
.58767 .53277 .59526 .54012 .6101 .53524 .60256 .52786 Metetra
.677 .776 .925 r
.60256 .52786 .6101 .53524 .62501 .53003 .61753 .5226 Metetra
.685 .779 .922 r
.61753 .5226 .62501 .53003 .63998 .5245 .63256 .51701 Metetra
.692 .78 .918 r
.63256 .51701 .63998 .5245 .65501 .51868 .64765 .51112 Metetra
.698 .781 .915 r
.64765 .51112 .65501 .51868 .6701 .51259 .6628 .50493 Metetra
.704 .781 .912 r
.6628 .50493 .6701 .51259 .68524 .50626 .67801 .4985 Metetra
.708 .781 .909 r
.67801 .4985 .68524 .50626 .70044 .49971 .69327 .49184 Metetra
.712 .781 .906 r
.69327 .49184 .70044 .49971 .7157 .49298 .70859 .48499 Metetra
.714 .78 .904 r
.70859 .48499 .7157 .49298 .73102 .48609 .72397 .47797 Metetra
.716 .779 .901 r
.72397 .47797 .73102 .48609 .74639 .47907 .7394 .47082 Metetra
.718 .777 .899 r
.7394 .47082 .74639 .47907 .76181 .47196 .75489 .46358 Metetra
.718 .775 .897 r
.75489 .46358 .76181 .47196 .7773 .46477 .77043 .45626 Metetra
.718 .773 .896 r
.77043 .45626 .7773 .46477 .79285 .45753 .78604 .44889 Metetra
.718 .771 .895 r
.78604 .44889 .79285 .45753 .80846 .45027 .80171 .4415 Metetra
.717 .769 .894 r
.80171 .4415 .80846 .45027 .82413 .44301 .81744 .43411 Metetra
.716 .767 .893 r
.81744 .43411 .82413 .44301 .83987 .43576 .83325 .42674 Metetra
.714 .765 .892 r
.83325 .42674 .83987 .43576 .85568 .42854 .84912 .41941 Metetra
.713 .763 .892 r
.84912 .41941 .85568 .42854 .87157 .42137 .86507 .41212 Metetra
.711 .761 .892 r
.86507 .41212 .87157 .42137 .88753 .41425 .88109 .40489 Metetra
.708 .759 .892 r
.88109 .40489 .88753 .41425 .90356 .40719 .89719 .39774 Metetra
.706 .757 .892 r
.89719 .39774 .90356 .40719 .91968 .40021 .91337 .39066 Metetra
.704 .756 .892 r
.91337 .39066 .91968 .40021 .93589 .3933 .92964 .38366 Metetra
.565 .699 .931 r
.33935 .56286 .34773 .57115 .36109 .5699 .35274 .56163 Metetra
.561 .699 .933 r
.35274 .56163 .36109 .5699 .37454 .56871 .36622 .56047 Metetra
.558 .7 .935 r
.36622 .56047 .37454 .56871 .38808 .56756 .3798 .55937 Metetra
.556 .701 .937 r
.3798 .55937 .38808 .56756 .40172 .56641 .39346 .55827 Metetra
.557 .704 .939 r
.39346 .55827 .40172 .56641 .41545 .5652 .40723 .55713 Metetra
.559 .707 .94 r
.40723 .55713 .41545 .5652 .42928 .56389 .42109 .55591 Metetra
.562 .712 .941 r
.42109 .55591 .42928 .56389 .44321 .56245 .43505 .55455 Metetra
.568 .717 .942 r
.43505 .55455 .44321 .56245 .45723 .56083 .44912 .55301 Metetra
.574 .723 .943 r
.44912 .55301 .45723 .56083 .47136 .55899 .46328 .55125 Metetra
.583 .729 .944 r
.46328 .55125 .47136 .55899 .48559 .55688 .47755 .54923 Metetra
.592 .736 .943 r
.47755 .54923 .48559 .55688 .49991 .55448 .49192 .54691 Metetra
.603 .742 .943 r
.49192 .54691 .49991 .55448 .51432 .55176 .50638 .54426 Metetra
.614 .749 .942 r
.50638 .54426 .51432 .55176 .52882 .5487 .52093 .54125 Metetra
.625 .755 .94 r
.52093 .54125 .52882 .5487 .54341 .54528 .53557 .53787 Metetra
.636 .761 .938 r
.53557 .53787 .54341 .54528 .55809 .54148 .5503 .5341 Metetra
.647 .766 .935 r
.5503 .5341 .55809 .54148 .57284 .53731 .56511 .52993 Metetra
.658 .771 .933 r
.56511 .52993 .57284 .53731 .58767 .53277 .57999 .52538 Metetra
.668 .775 .929 r
.57999 .52538 .58767 .53277 .60256 .52786 .59495 .52044 Metetra
.677 .778 .926 r
.59495 .52044 .60256 .52786 .61753 .5226 .60997 .51514 Metetra
.686 .78 .922 r
.60997 .51514 .61753 .5226 .63256 .51701 .62506 .50949 Metetra
.693 .782 .919 r
.62506 .50949 .63256 .51701 .64765 .51112 .64022 .50351 Metetra
.7 .782 .915 r
.64022 .50351 .64765 .51112 .6628 .50493 .65543 .49723 Metetra
.705 .783 .912 r
.65543 .49723 .6628 .50493 .67801 .4985 .6707 .49069 Metetra
.71 .783 .909 r
.6707 .49069 .67801 .4985 .69327 .49184 .68603 .48391 Metetra
.713 .782 .906 r
.68603 .48391 .69327 .49184 .70859 .48499 .70141 .47694 Metetra
.716 .781 .903 r
.70141 .47694 .70859 .48499 .72397 .47797 .71684 .46979 Metetra
.718 .78 .901 r
.71684 .46979 .72397 .47797 .7394 .47082 .73234 .46251 Metetra
.719 .778 .899 r
.73234 .46251 .7394 .47082 .75489 .46358 .74789 .45512 Metetra
.72 .776 .897 r
.74789 .45512 .75489 .46358 .77043 .45626 .76349 .44766 Metetra
.72 .774 .895 r
.76349 .44766 .77043 .45626 .78604 .44889 .77916 .44016 Metetra
.72 .772 .894 r
.77916 .44016 .78604 .44889 .80171 .4415 .79489 .43263 Metetra
.719 .77 .893 r
.79489 .43263 .80171 .4415 .81744 .43411 .81069 .42511 Metetra
.717 .768 .892 r
.81069 .42511 .81744 .43411 .83325 .42674 .82655 .41762 Metetra
.716 .765 .892 r
.82655 .41762 .83325 .42674 .84912 .41941 .84249 .41016 Metetra
.714 .763 .891 r
.84249 .41016 .84912 .41941 .86507 .41212 .8585 .40276 Metetra
.712 .761 .891 r
.8585 .40276 .86507 .41212 .88109 .40489 .87458 .39543 Metetra
.709 .759 .891 r
.87458 .39543 .88109 .40489 .89719 .39774 .89075 .38817 Metetra
.707 .757 .891 r
.89075 .38817 .89719 .39774 .91337 .39066 .90699 .381 Metetra
.704 .755 .892 r
.90699 .381 .91337 .39066 .92964 .38366 .92333 .37391 Metetra
.564 .698 .931 r
.3309 .55446 .33935 .56286 .35274 .56163 .34432 .55324 Metetra
.559 .697 .933 r
.34432 .55324 .35274 .56163 .36622 .56047 .35783 .55212 Metetra
.555 .698 .935 r
.35783 .55212 .36622 .56047 .3798 .55937 .37143 .55106 Metetra
.553 .699 .937 r
.37143 .55106 .3798 .55937 .39346 .55827 .38513 .55002 Metetra
.553 .701 .939 r
.38513 .55002 .39346 .55827 .40723 .55713 .39893 .54895 Metetra
.554 .705 .94 r
.39893 .54895 .40723 .55713 .42109 .55591 .41282 .54781 Metetra
.557 .709 .942 r
.41282 .54781 .42109 .55591 .43505 .55455 .42682 .54654 Metetra
.563 .714 .943 r
.42682 .54654 .43505 .55455 .44912 .55301 .44092 .5451 Metetra
.57 .721 .944 r
.44092 .5451 .44912 .55301 .46328 .55125 .45513 .54344 Metetra
.578 .727 .945 r
.45513 .54344 .46328 .55125 .47755 .54923 .46944 .54151 Metetra
.588 .734 .945 r
.46944 .54151 .47755 .54923 .49192 .54691 .48385 .53927 Metetra
.598 .741 .944 r
.48385 .53927 .49192 .54691 .50638 .54426 .49835 .5367 Metetra
.61 .748 .943 r
.49835 .5367 .50638 .54426 .52093 .54125 .51295 .53375 Metetra
.622 .755 .942 r
.51295 .53375 .52093 .54125 .53557 .53787 .52765 .53041 Metetra
.634 .761 .939 r
.52765 .53041 .53557 .53787 .5503 .5341 .54243 .52667 Metetra
.645 .767 .937 r
.54243 .52667 .5503 .5341 .56511 .52993 .55729 .52252 Metetra
.657 .772 .934 r
.55729 .52252 .56511 .52993 .57999 .52538 .57224 .51796 Metetra
.667 .776 .93 r
.57224 .51796 .57999 .52538 .59495 .52044 .58725 .513 Metetra
.677 .779 .927 r
.58725 .513 .59495 .52044 .60997 .51514 .60234 .50765 Metetra
.686 .781 .923 r
.60234 .50765 .60997 .51514 .62506 .50949 .61749 .50193 Metetra
.694 .783 .919 r
.61749 .50193 .62506 .50949 .64022 .50351 .63271 .49587 Metetra
.701 .784 .916 r
.63271 .49587 .64022 .50351 .65543 .49723 .64798 .48949 Metetra
.707 .784 .912 r
.64798 .48949 .65543 .49723 .6707 .49069 .66331 .48284 Metetra
.711 .784 .909 r
.66331 .48284 .6707 .49069 .68603 .48391 .6787 .47594 Metetra
.715 .783 .906 r
.6787 .47594 .68603 .48391 .70141 .47694 .69415 .46883 Metetra
.718 .782 .903 r
.69415 .46883 .70141 .47694 .71684 .46979 .70965 .46154 Metetra
.72 .781 .9 r
.70965 .46154 .71684 .46979 .73234 .46251 .7252 .45412 Metetra
.721 .779 .898 r
.7252 .45412 .73234 .46251 .74789 .45512 .74081 .44659 Metetra
.722 .777 .896 r
.74081 .44659 .74789 .45512 .76349 .44766 .75648 .43898 Metetra
.722 .775 .894 r
.75648 .43898 .76349 .44766 .77916 .44016 .77221 .43134 Metetra
.721 .772 .893 r
.77221 .43134 .77916 .44016 .79489 .43263 .78801 .42367 Metetra
.72 .77 .892 r
.78801 .42367 .79489 .43263 .81069 .42511 .80386 .41601 Metetra
.719 .768 .891 r
.80386 .41601 .81069 .42511 .82655 .41762 .81979 .40839 Metetra
.717 .766 .891 r
.81979 .40839 .82655 .41762 .84249 .41016 .83579 .40081 Metetra
.715 .763 .89 r
.83579 .40081 .84249 .41016 .8585 .40276 .85186 .39329 Metetra
.713 .761 .89 r
.85186 .39329 .8585 .40276 .87458 .39543 .86801 .38584 Metetra
.71 .759 .89 r
.86801 .38584 .87458 .39543 .89075 .38817 .88424 .37848 Metetra
.708 .757 .891 r
.88424 .37848 .89075 .38817 .90699 .381 .90055 .37121 Metetra
.705 .755 .891 r
.90055 .37121 .90699 .381 .92333 .37391 .91695 .36403 Metetra
.562 .696 .931 r
.32238 .54595 .3309 .55446 .34432 .55324 .33583 .54474 Metetra
.557 .695 .933 r
.33583 .54474 .34432 .55324 .35783 .55212 .34937 .54364 Metetra
.552 .696 .935 r
.34937 .54364 .35783 .55212 .37143 .55106 .363 .54263 Metetra
.55 .697 .937 r
.363 .54263 .37143 .55106 .38513 .55002 .37672 .54165 Metetra
.549 .699 .939 r
.37672 .54165 .38513 .55002 .39893 .54895 .39055 .54066 Metetra
.55 .702 .941 r
.39055 .54066 .39893 .54895 .41282 .54781 .40448 .53961 Metetra
.552 .706 .942 r
.40448 .53961 .41282 .54781 .42682 .54654 .41851 .53844 Metetra
.557 .712 .944 r
.41851 .53844 .42682 .54654 .44092 .5451 .43265 .5371 Metetra
.564 .718 .945 r
.43265 .5371 .44092 .5451 .45513 .54344 .44689 .53554 Metetra
.573 .725 .946 r
.44689 .53554 .45513 .54344 .46944 .54151 .46124 .53371 Metetra
.583 .732 .946 r
.46124 .53371 .46944 .54151 .48385 .53927 .47569 .53157 Metetra
.594 .74 .945 r
.47569 .53157 .48385 .53927 .49835 .5367 .49025 .52908 Metetra
.606 .747 .945 r
.49025 .52908 .49835 .5367 .51295 .53375 .5049 .5262 Metetra
.618 .754 .943 r
.5049 .5262 .51295 .53375 .52765 .53041 .51964 .52292 Metetra
.631 .761 .941 r
.51964 .52292 .52765 .53041 .54243 .52667 .53448 .51921 Metetra
.643 .767 .938 r
.53448 .51921 .54243 .52667 .55729 .52252 .5494 .51508 Metetra
.655 .772 .935 r
.5494 .51508 .55729 .52252 .57224 .51796 .5644 .51051 Metetra
.667 .777 .932 r
.5644 .51051 .57224 .51796 .58725 .513 .57948 .50552 Metetra
.677 .78 .928 r
.57948 .50552 .58725 .513 .60234 .50765 .59462 .50012 Metetra
.687 .783 .924 r
.59462 .50012 .60234 .50765 .61749 .50193 .60984 .49434 Metetra
.695 .785 .92 r
.60984 .49434 .61749 .50193 .63271 .49587 .62512 .48819 Metetra
.702 .786 .916 r
.62512 .48819 .63271 .49587 .64798 .48949 .64046 .48172 Metetra
.708 .786 .912 r
.64046 .48172 .64798 .48949 .66331 .48284 .65585 .47494 Metetra
.713 .786 .909 r
.65585 .47494 .66331 .48284 .6787 .47594 .6713 .46792 Metetra
.717 .785 .905 r
.6713 .46792 .6787 .47594 .69415 .46883 .68681 .46067 Metetra
.72 .783 .902 r
.68681 .46067 .69415 .46883 .70965 .46154 .70237 .45323 Metetra
.722 .782 .9 r
.70237 .45323 .70965 .46154 .7252 .45412 .71799 .44566 Metetra
.723 .78 .897 r
.71799 .44566 .7252 .45412 .74081 .44659 .73367 .43797 Metetra
.724 .778 .895 r
.73367 .43797 .74081 .44659 .75648 .43898 .7494 .43022 Metetra
.724 .775 .894 r
.7494 .43022 .75648 .43898 .77221 .43134 .76519 .42242 Metetra
.723 .773 .892 r
.76519 .42242 .77221 .43134 .78801 .42367 .78105 .41461 Metetra
.722 .771 .891 r
.78105 .41461 .78801 .42367 .80386 .41601 .79697 .40681 Metetra
.72 .768 .89 r
.79697 .40681 .80386 .41601 .81979 .40839 .81295 .39904 Metetra
.718 .766 .89 r
.81295 .39904 .81979 .40839 .83579 .40081 .82901 .39133 Metetra
.716 .763 .89 r
.82901 .39133 .83579 .40081 .85186 .39329 .84515 .38369 Metetra
.714 .761 .89 r
.84515 .38369 .85186 .39329 .86801 .38584 .86136 .37614 Metetra
.711 .759 .89 r
.86136 .37614 .86801 .38584 .88424 .37848 .87766 .36867 Metetra
.708 .757 .89 r
.87766 .36867 .88424 .37848 .90055 .37121 .89403 .3613 Metetra
.705 .755 .89 r
.89403 .3613 .90055 .37121 .91695 .36403 .9105 .35403 Metetra
.561 .695 .93 r
.31378 .53732 .32238 .54595 .33583 .54474 .32726 .53611 Metetra
.555 .694 .932 r
.32726 .53611 .33583 .54474 .34937 .54364 .34083 .53504 Metetra
.549 .693 .934 r
.34083 .53504 .34937 .54364 .363 .54263 .35448 .53408 Metetra
.546 .694 .937 r
.35448 .53408 .363 .54263 .37672 .54165 .36824 .53316 Metetra
.544 .696 .939 r
.36824 .53316 .37672 .54165 .39055 .54066 .38209 .53225 Metetra
.545 .699 .941 r
.38209 .53225 .39055 .54066 .40448 .53961 .39605 .53129 Metetra
.547 .704 .943 r
.39605 .53129 .40448 .53961 .41851 .53844 .41012 .53022 Metetra
.552 .709 .944 r
.41012 .53022 .41851 .53844 .43265 .5371 .42429 .52899 Metetra
.559 .715 .946 r
.42429 .52899 .43265 .5371 .44689 .53554 .43857 .52755 Metetra
.567 .723 .947 r
.43857 .52755 .44689 .53554 .46124 .53371 .45296 .52583 Metetra
.577 .73 .947 r
.45296 .52583 .46124 .53371 .47569 .53157 .46745 .52379 Metetra
.589 .738 .947 r
.46745 .52379 .47569 .53157 .49025 .52908 .48205 .52139 Metetra
.602 .746 .946 r
.48205 .52139 .49025 .52908 .5049 .5262 .49675 .5186 Metetra
.615 .754 .945 r
.49675 .5186 .5049 .5262 .51964 .52292 .51155 .51538 Metetra
.628 .761 .942 r
.51155 .51538 .51964 .52292 .53448 .51921 .52644 .51172 Metetra
.641 .768 .94 r
.52644 .51172 .53448 .51921 .5494 .51508 .54142 .50761 Metetra
.654 .773 .936 r
.54142 .50761 .5494 .51508 .5644 .51051 .55648 .50304 Metetra
.666 .778 .933 r
.55648 .50304 .5644 .51051 .57948 .50552 .57162 .49802 Metetra
.677 .782 .929 r
.57162 .49802 .57948 .50552 .59462 .50012 .58683 .49258 Metetra
.687 .784 .925 r
.58683 .49258 .59462 .50012 .60984 .49434 .6021 .48672 Metetra
.696 .786 .921 r
.6021 .48672 .60984 .49434 .62512 .48819 .61745 .48049 Metetra
.703 .787 .916 r
.61745 .48049 .62512 .48819 .64046 .48172 .63285 .47391 Metetra
.71 .788 .912 r
.63285 .47391 .64046 .48172 .65585 .47494 .64831 .46701 Metetra
.715 .787 .909 r
.64831 .46701 .65585 .47494 .6713 .46792 .66383 .45984 Metetra
.719 .786 .905 r
.66383 .45984 .6713 .46792 .68681 .46067 .6794 .45245 Metetra
.722 .785 .902 r
.6794 .45245 .68681 .46067 .70237 .45323 .69503 .44486 Metetra
.724 .783 .899 r
.69503 .44486 .70237 .45323 .71799 .44566 .71071 .43713 Metetra
.726 .781 .897 r
.71071 .43713 .71799 .44566 .73367 .43797 .72645 .42928 Metetra
.726 .779 .894 r
.72645 .42928 .73367 .43797 .7494 .43022 .74224 .42136 Metetra
.726 .776 .893 r
.74224 .42136 .7494 .43022 .76519 .42242 .7581 .41341 Metetra
.725 .774 .891 r
.7581 .41341 .76519 .42242 .78105 .41461 .77401 .40544 Metetra
.724 .771 .89 r
.77401 .40544 .78105 .41461 .79697 .40681 .79 .39749 Metetra
.722 .768 .889 r
.79 .39749 .79697 .40681 .81295 .39904 .80605 .38959 Metetra
.72 .766 .889 r
.80605 .38959 .81295 .39904 .82901 .39133 .82217 .38174 Metetra
.717 .763 .889 r
.82217 .38174 .82901 .39133 .84515 .38369 .83837 .37398 Metetra
.715 .761 .889 r
.83837 .37398 .84515 .38369 .86136 .37614 .85464 .3663 Metetra
.712 .758 .889 r
.85464 .3663 .86136 .37614 .87766 .36867 .871 .35873 Metetra
.709 .756 .889 r
.871 .35873 .87766 .36867 .89403 .3613 .88745 .35126 Metetra
.706 .754 .89 r
.88745 .35126 .89403 .3613 .9105 .35403 .90398 .3439 Metetra
.56 .693 .93 r
.30511 .52857 .31378 .53732 .32726 .53611 .31861 .52736 Metetra
.553 .692 .932 r
.31861 .52736 .32726 .53611 .34083 .53504 .33221 .52632 Metetra
.547 .691 .934 r
.33221 .52632 .34083 .53504 .35448 .53408 .34589 .52539 Metetra
.542 .691 .936 r
.34589 .52539 .35448 .53408 .36824 .53316 .35968 .52454 Metetra
.54 .693 .939 r
.35968 .52454 .36824 .53316 .38209 .53225 .37356 .52371 Metetra
.54 .696 .941 r
.37356 .52371 .38209 .53225 .39605 .53129 .38755 .52285 Metetra
.542 .7 .943 r
.38755 .52285 .39605 .53129 .41012 .53022 .40164 .5219 Metetra
.546 .706 .945 r
.40164 .5219 .41012 .53022 .42429 .52899 .41585 .52079 Metetra
.552 .712 .946 r
.41585 .52079 .42429 .52899 .43857 .52755 .43017 .51946 Metetra
.561 .72 .948 r
.43017 .51946 .43857 .52755 .45296 .52583 .4446 .51786 Metetra
.571 .728 .948 r
.4446 .51786 .45296 .52583 .46745 .52379 .45913 .51594 Metetra
.584 .737 .948 r
.45913 .51594 .46745 .52379 .48205 .52139 .47378 .51365 Metetra
.597 .745 .948 r
.47378 .51365 .48205 .52139 .49675 .5186 .48853 .51095 Metetra
.611 .753 .946 r
.48853 .51095 .49675 .5186 .51155 .51538 .50338 .5078 Metetra
.625 .761 .944 r
.50338 .5078 .51155 .51538 .52644 .51172 .51832 .50419 Metetra
.639 .768 .941 r
.51832 .50419 .52644 .51172 .54142 .50761 .53335 .50011 Metetra
.652 .774 .938 r
.53335 .50011 .54142 .50761 .55648 .50304 .54847 .49554 Metetra
.665 .779 .934 r
.54847 .49554 .55648 .50304 .57162 .49802 .56367 .49051 Metetra
.677 .783 .93 r
.56367 .49051 .57162 .49802 .58683 .49258 .57895 .48501 Metetra
.687 .786 .926 r
.57895 .48501 .58683 .49258 .6021 .48672 .59429 .47909 Metetra
.697 .788 .921 r
.59429 .47909 .6021 .48672 .61745 .48049 .60969 .47276 Metetra
.705 .789 .917 r
.60969 .47276 .61745 .48049 .63285 .47391 .62516 .46606 Metetra
.712 .789 .912 r
.62516 .46606 .63285 .47391 .64831 .46701 .64069 .45904 Metetra
.717 .789 .908 r
.64069 .45904 .64831 .46701 .66383 .45984 .65627 .45173 Metetra
.721 .788 .905 r
.65627 .45173 .66383 .45984 .6794 .45245 .67191 .44418 Metetra
.724 .786 .901 r
.67191 .44418 .6794 .45245 .69503 .44486 .6876 .43643 Metetra
.727 .784 .898 r
.6876 .43643 .69503 .44486 .71071 .43713 .70335 .42852 Metetra
.728 .782 .896 r
.70335 .42852 .71071 .43713 .72645 .42928 .71915 .42051 Metetra
.728 .779 .894 r
.71915 .42051 .72645 .42928 .74224 .42136 .73501 .41242 Metetra
.728 .777 .892 r
.73501 .41242 .74224 .42136 .7581 .41341 .75092 .40429 Metetra
.727 .774 .89 r
.75092 .40429 .7581 .41341 .77401 .40544 .7669 .39617 Metetra
.725 .771 .889 r
.7669 .39617 .77401 .40544 .79 .39749 .78295 .38806 Metetra
.723 .768 .888 r
.78295 .38806 .79 .39749 .80605 .38959 .79906 .38001 Metetra
.721 .766 .888 r
.79906 .38001 .80605 .38959 .82217 .38174 .81525 .37203 Metetra
.719 .763 .888 r
.81525 .37203 .82217 .38174 .83837 .37398 .83151 .36414 Metetra
.716 .761 .888 r
.83151 .36414 .83837 .37398 .85464 .3663 .84785 .35634 Metetra
.713 .758 .888 r
.84785 .35634 .85464 .3663 .871 .35873 .86428 .34866 Metetra
.71 .756 .888 r
.86428 .34866 .871 .35873 .88745 .35126 .88079 .34109 Metetra
.706 .754 .889 r
.88079 .34109 .88745 .35126 .90398 .3439 .89739 .33364 Metetra
.559 .692 .929 r
.29635 .51969 .30511 .52857 .31861 .52736 .30989 .51848 Metetra
.551 .69 .931 r
.30989 .51848 .31861 .52736 .33221 .52632 .32351 .51745 Metetra
.544 .689 .934 r
.32351 .51745 .33221 .52632 .34589 .52539 .33723 .51657 Metetra
.539 .689 .936 r
.33723 .51657 .34589 .52539 .35968 .52454 .35103 .51579 Metetra
.536 .69 .938 r
.35103 .51579 .35968 .52454 .37356 .52371 .36495 .51505 Metetra
.534 .693 .941 r
.36495 .51505 .37356 .52371 .38755 .52285 .37896 .51429 Metetra
.536 .697 .943 r
.37896 .51429 .38755 .52285 .40164 .5219 .39309 .51345 Metetra
.539 .702 .945 r
.39309 .51345 .40164 .5219 .41585 .52079 .40733 .51247 Metetra
.546 .709 .947 r
.40733 .51247 .41585 .52079 .43017 .51946 .42168 .51127 Metetra
.554 .717 .948 r
.42168 .51127 .43017 .51946 .4446 .51786 .43615 .50981 Metetra
.565 .726 .949 r
.43615 .50981 .4446 .51786 .45913 .51594 .45072 .50802 Metetra
.577 .735 .95 r
.45072 .50802 .45913 .51594 .47378 .51365 .46541 .50584 Metetra
.591 .744 .949 r
.46541 .50584 .47378 .51365 .48853 .51095 .48021 .50324 Metetra
.606 .753 .948 r
.48021 .50324 .48853 .51095 .50338 .5078 .49511 .50018 Metetra
.621 .761 .946 r
.49511 .50018 .50338 .5078 .51832 .50419 .51011 .49663 Metetra
.636 .769 .943 r
.51011 .49663 .51832 .50419 .53335 .50011 .5252 .49258 Metetra
.65 .775 .94 r
.5252 .49258 .53335 .50011 .54847 .49554 .54038 .48803 Metetra
.664 .781 .936 r
.54038 .48803 .54847 .49554 .56367 .49051 .55564 .48298 Metetra
.677 .785 .931 r
.55564 .48298 .56367 .49051 .57895 .48501 .57098 .47744 Metetra
.688 .788 .927 r
.57098 .47744 .57895 .48501 .59429 .47909 .58639 .47144 Metetra
.698 .79 .922 r
.58639 .47144 .59429 .47909 .60969 .47276 .60186 .46501 Metetra
.706 .791 .917 r
.60186 .46501 .60969 .47276 .62516 .46606 .61739 .4582 Metetra
.714 .791 .913 r
.61739 .4582 .62516 .46606 .64069 .45904 .63299 .45103 Metetra
.719 .791 .908 r
.63299 .45103 .64069 .45904 .65627 .45173 .64863 .44357 Metetra
.724 .79 .904 r
.64863 .44357 .65627 .45173 .67191 .44418 .66434 .43585 Metetra
.727 .788 .901 r
.66434 .43585 .67191 .44418 .6876 .43643 .68009 .42793 Metetra
.729 .786 .898 r
.68009 .42793 .6876 .43643 .70335 .42852 .69591 .41984 Metetra
.73 .783 .895 r
.69591 .41984 .70335 .42852 .71915 .42051 .71177 .41165 Metetra
.731 .78 .893 r
.71177 .41165 .71915 .42051 .73501 .41242 .72769 .40338 Metetra
.73 .777 .891 r
.72769 .40338 .73501 .41242 .75092 .40429 .74368 .39508 Metetra
.729 .775 .889 r
.74368 .39508 .75092 .40429 .7669 .39617 .75972 .38678 Metetra
.727 .772 .888 r
.75972 .38678 .7669 .39617 .78295 .38806 .77583 .37852 Metetra
.725 .769 .887 r
.77583 .37852 .78295 .38806 .79906 .38001 .79201 .37031 Metetra
.723 .766 .887 r
.79201 .37031 .79906 .38001 .81525 .37203 .80826 .36219 Metetra
.72 .763 .887 r
.80826 .36219 .81525 .37203 .83151 .36414 .82458 .35416 Metetra
.717 .76 .887 r
.82458 .35416 .83151 .36414 .84785 .35634 .84099 .34624 Metetra
.714 .758 .887 r
.84099 .34624 .84785 .35634 .86428 .34866 .85748 .33845 Metetra
.71 .756 .888 r
.85748 .33845 .86428 .34866 .88079 .34109 .87406 .33078 Metetra
.707 .753 .888 r
.87406 .33078 .88079 .34109 .89739 .33364 .89073 .32324 Metetra
.558 .691 .928 r
.28753 .51068 .29635 .51969 .30989 .51848 .30109 .50946 Metetra
.549 .688 .931 r
.30109 .50946 .30989 .51848 .32351 .51745 .31474 .50845 Metetra
.542 .686 .933 r
.31474 .50845 .32351 .51745 .33723 .51657 .32848 .50761 Metetra
.535 .686 .935 r
.32848 .50761 .33723 .51657 .35103 .51579 .34231 .50689 Metetra
.531 .687 .938 r
.34231 .50689 .35103 .51579 .36495 .51505 .35625 .50624 Metetra
.529 .689 .941 r
.35625 .50624 .36495 .51505 .37896 .51429 .3703 .50559 Metetra
.529 .693 .943 r
.3703 .50559 .37896 .51429 .39309 .51345 .38445 .50487 Metetra
.533 .698 .945 r
.38445 .50487 .39309 .51345 .40733 .51247 .39872 .50403 Metetra
.538 .705 .948 r
.39872 .50403 .40733 .51247 .42168 .51127 .41311 .50298 Metetra
.547 .714 .949 r
.41311 .50298 .42168 .51127 .43615 .50981 .42761 .50167 Metetra
.558 .723 .951 r
.42761 .50167 .43615 .50981 .45072 .50802 .44223 .50002 Metetra
.571 .732 .951 r
.44223 .50002 .45072 .50802 .46541 .50584 .45696 .49798 Metetra
.585 .742 .951 r
.45696 .49798 .46541 .50584 .48021 .50324 .47181 .49549 Metetra
.601 .752 .95 r
.47181 .49549 .48021 .50324 .49511 .50018 .48676 .49253 Metetra
.617 .761 .948 r
.48676 .49253 .49511 .50018 .51011 .49663 .50181 .48905 Metetra
.633 .769 .945 r
.50181 .48905 .51011 .49663 .5252 .49258 .51696 .48505 Metetra
.648 .776 .942 r
.51696 .48505 .5252 .49258 .54038 .48803 .5322 .48051 Metetra
.663 .782 .937 r
.5322 .48051 .54038 .48803 .55564 .48298 .54752 .47543 Metetra
.676 .787 .933 r
.54752 .47543 .55564 .48298 .57098 .47744 .56292 .46985 Metetra
.689 .79 .928 r
.56292 .46985 .57098 .47744 .58639 .47144 .5784 .46378 Metetra
.699 .792 .923 r
.5784 .46378 .58639 .47144 .60186 .46501 .59394 .45725 Metetra
.708 .794 .918 r
.59394 .45725 .60186 .46501 .61739 .4582 .60954 .45031 Metetra
.716 .794 .913 r
.60954 .45031 .61739 .4582 .63299 .45103 .6252 .443 Metetra
.722 .793 .908 r
.6252 .443 .63299 .45103 .64863 .44357 .64092 .43537 Metetra
.726 .791 .904 r
.64092 .43537 .64863 .44357 .66434 .43585 .65668 .42747 Metetra
.73 .789 .9 r
.65668 .42747 .66434 .43585 .68009 .42793 .67251 .41936 Metetra
.732 .787 .897 r
.67251 .41936 .68009 .42793 .69591 .41984 .68838 .41109 Metetra
.733 .784 .894 r
.68838 .41109 .69591 .41984 .71177 .41165 .70432 .4027 Metetra
.733 .781 .891 r
.70432 .4027 .71177 .41165 .72769 .40338 .7203 .39424 Metetra
.732 .778 .889 r
.7203 .39424 .72769 .40338 .74368 .39508 .73635 .38576 Metetra
.731 .775 .888 r
.73635 .38576 .74368 .39508 .75972 .38678 .75246 .37728 Metetra
.729 .772 .887 r
.75246 .37728 .75972 .38678 .77583 .37852 .76863 .36885 Metetra
.727 .769 .886 r
.76863 .36885 .77583 .37852 .79201 .37031 .78487 .36048 Metetra
.724 .766 .886 r
.78487 .36048 .79201 .37031 .80826 .36219 .80119 .35221 Metetra
.721 .763 .886 r
.80119 .35221 .80826 .36219 .82458 .35416 .81758 .34405 Metetra
.718 .76 .886 r
.81758 .34405 .82458 .35416 .84099 .34624 .83405 .33601 Metetra
.714 .757 .886 r
.83405 .33601 .84099 .34624 .85748 .33845 .85061 .3281 Metetra
.711 .755 .887 r
.85061 .3281 .85748 .33845 .87406 .33078 .86725 .32032 Metetra
.707 .753 .887 r
.86725 .32032 .87406 .33078 .89073 .32324 .88399 .31269 Metetra
.558 .689 .927 r
.27862 .50154 .28753 .51068 .30109 .50946 .29221 .50031 Metetra
.548 .686 .93 r
.29221 .50031 .30109 .50946 .31474 .50845 .30589 .49931 Metetra
.539 .684 .932 r
.30589 .49931 .31474 .50845 .32848 .50761 .31965 .4985 Metetra
.532 .683 .935 r
.31965 .4985 .32848 .50761 .34231 .50689 .33352 .49784 Metetra
.526 .683 .938 r
.33352 .49784 .34231 .50689 .35625 .50624 .34748 .49728 Metetra
.523 .685 .94 r
.34748 .49728 .35625 .50624 .3703 .50559 .36155 .49674 Metetra
.523 .689 .943 r
.36155 .49674 .3703 .50559 .38445 .50487 .37573 .49617 Metetra
.525 .694 .946 r
.37573 .49617 .38445 .50487 .39872 .50403 .39003 .49547 Metetra
.531 .701 .948 r
.39003 .49547 .39872 .50403 .41311 .50298 .40445 .49458 Metetra
.539 .71 .95 r
.40445 .49458 .41311 .50298 .42761 .50167 .41899 .49343 Metetra
.55 .719 .952 r
.41899 .49343 .42761 .50167 .44223 .50002 .43364 .49194 Metetra
.563 .73 .953 r
.43364 .49194 .44223 .50002 .45696 .49798 .44842 .49004 Metetra
.579 .74 .953 r
.44842 .49004 .45696 .49798 .47181 .49549 .46331 .48769 Metetra
.595 .751 .952 r
.46331 .48769 .47181 .49549 .48676 .49253 .47831 .48484 Metetra
.612 .761 .95 r
.47831 .48484 .48676 .49253 .50181 .48905 .49342 .48144 Metetra
.629 .77 .947 r
.49342 .48144 .50181 .48905 .51696 .48505 .50863 .47749 Metetra
.646 .777 .944 r
.50863 .47749 .51696 .48505 .5322 .48051 .52393 .47298 Metetra
.662 .784 .939 r
.52393 .47298 .5322 .48051 .54752 .47543 .53931 .46789 Metetra
.676 .789 .934 r
.53931 .46789 .54752 .47543 .56292 .46985 .55478 .46226 Metetra
.689 .793 .929 r
.55478 .46226 .56292 .46985 .5784 .46378 .57032 .45611 Metetra
.7 .795 .923 r
.57032 .45611 .5784 .46378 .59394 .45725 .58593 .44948 Metetra
.71 .796 .918 r
.58593 .44948 .59394 .45725 .60954 .45031 .6016 .4424 Metetra
.718 .796 .913 r
.6016 .4424 .60954 .45031 .6252 .443 .61733 .43494 Metetra
.724 .795 .908 r
.61733 .43494 .6252 .443 .64092 .43537 .63311 .42713 Metetra
.729 .793 .903 r
.63311 .42713 .64092 .43537 .65668 .42747 .64895 .41905 Metetra
.732 .791 .899 r
.64895 .41905 .65668 .42747 .67251 .41936 .66484 .41074 Metetra
.735 .789 .896 r
.66484 .41074 .67251 .41936 .68838 .41109 .68078 .40226 Metetra
.736 .786 .893 r
.68078 .40226 .68838 .41109 .70432 .4027 .69678 .39367 Metetra
.736 .782 .89 r
.69678 .39367 .70432 .4027 .7203 .39424 .71283 .38501 Metetra
.735 .779 .888 r
.71283 .38501 .7203 .39424 .73635 .38576 .72894 .37633 Metetra
.733 .775 .887 r
.72894 .37633 .73635 .38576 .75246 .37728 .74511 .36766 Metetra
.731 .772 .885 r
.74511 .36766 .75246 .37728 .76863 .36885 .76135 .35905 Metetra
.729 .769 .885 r
.76135 .35905 .76863 .36885 .78487 .36048 .77766 .35052 Metetra
.726 .765 .884 r
.77766 .35052 .78487 .36048 .80119 .35221 .79404 .34209 Metetra
.722 .762 .884 r
.79404 .34209 .80119 .35221 .81758 .34405 .8105 .33379 Metetra
.719 .76 .885 r
.8105 .33379 .81758 .34405 .83405 .33601 .82703 .32562 Metetra
.715 .757 .885 r
.82703 .32562 .83405 .33601 .85061 .3281 .84366 .3176 Metetra
.712 .754 .886 r
.84366 .3176 .85061 .3281 .86725 .32032 .86037 .30972 Metetra
.708 .752 .887 r
.86037 .30972 .86725 .32032 .88399 .31269 .87718 .302 Metetra
.558 .688 .927 r
.26963 .49227 .27862 .50154 .29221 .50031 .28326 .49101 Metetra
.547 .684 .929 r
.28326 .49101 .29221 .50031 .30589 .49931 .29696 .49001 Metetra
.537 .681 .932 r
.29696 .49001 .30589 .49931 .31965 .4985 .31075 .48923 Metetra
.528 .68 .934 r
.31075 .48923 .31965 .4985 .33352 .49784 .32464 .48864 Metetra
.522 .679 .937 r
.32464 .48864 .33352 .49784 .34748 .49728 .33863 .48817 Metetra
.517 .681 .94 r
.33863 .48817 .34748 .49728 .36155 .49674 .35272 .48775 Metetra
.516 .684 .943 r
.35272 .48775 .36155 .49674 .37573 .49617 .36693 .48731 Metetra
.517 .689 .946 r
.36693 .48731 .37573 .49617 .39003 .49547 .38126 .48678 Metetra
.522 .697 .949 r
.38126 .48678 .39003 .49547 .40445 .49458 .3957 .48606 Metetra
.53 .705 .951 r
.3957 .48606 .40445 .49458 .41899 .49343 .41027 .48509 Metetra
.541 .716 .953 r
.41027 .48509 .41899 .49343 .43364 .49194 .42497 .48377 Metetra
.555 .727 .954 r
.42497 .48377 .43364 .49194 .44842 .49004 .43979 .48204 Metetra
.571 .738 .955 r
.43979 .48204 .44842 .49004 .46331 .48769 .45472 .47984 Metetra
.588 .749 .954 r
.45472 .47984 .46331 .48769 .47831 .48484 .46977 .47711 Metetra
.607 .76 .952 r
.46977 .47711 .47831 .48484 .49342 .48144 .48494 .47382 Metetra
.625 .77 .95 r
.48494 .47382 .49342 .48144 .50863 .47749 .5002 .46994 Metetra
.643 .779 .946 r
.5002 .46994 .50863 .47749 .52393 .47298 .51556 .46545 Metetra
.66 .786 .941 r
.51556 .46545 .52393 .47298 .53931 .46789 .53101 .46036 Metetra
.676 .791 .936 r
.53101 .46036 .53931 .46789 .55478 .46226 .54655 .45468 Metetra
.69 .795 .93 r
.54655 .45468 .55478 .46226 .57032 .45611 .56215 .44845 Metetra
.702 .798 .924 r
.56215 .44845 .57032 .45611 .58593 .44948 .57783 .44171 Metetra
.712 .799 .918 r
.57783 .44171 .58593 .44948 .6016 .4424 .59357 .43449 Metetra
.72 .799 .913 r
.59357 .43449 .6016 .4424 .61733 .43494 .60937 .42685 Metetra
.727 .797 .908 r
.60937 .42685 .61733 .43494 .63311 .42713 .62522 .41886 Metetra
.732 .796 .903 r
.62522 .41886 .63311 .42713 .64895 .41905 .64113 .41058 Metetra
.736 .793 .898 r
.64113 .41058 .64895 .41905 .66484 .41074 .65709 .40205 Metetra
.738 .79 .895 r
.65709 .40205 .66484 .41074 .68078 .40226 .6731 .39336 Metetra
.739 .787 .891 r
.6731 .39336 .68078 .40226 .69678 .39367 .68916 .38455 Metetra
.739 .783 .889 r
.68916 .38455 .69678 .39367 .71283 .38501 .70528 .37567 Metetra
.738 .78 .887 r
.70528 .37567 .71283 .38501 .72894 .37633 .72146 .36678 Metetra
.736 .776 .885 r
.72146 .36678 .72894 .37633 .74511 .36766 .73769 .35792 Metetra
.733 .772 .884 r
.73769 .35792 .74511 .36766 .76135 .35905 .75399 .34912 Metetra
.731 .769 .883 r
.75399 .34912 .76135 .35905 .77766 .35052 .77037 .34041 Metetra
.727 .765 .883 r
.77037 .34041 .77766 .35052 .79404 .34209 .78681 .33183 Metetra
.724 .762 .883 r
.78681 .33183 .79404 .34209 .8105 .33379 .80333 .32338 Metetra
.72 .759 .883 r
.80333 .32338 .8105 .33379 .82703 .32562 .81994 .31508 Metetra
.716 .756 .884 r
.81994 .31508 .82703 .32562 .84366 .3176 .83663 .30694 Metetra
.712 .754 .885 r
.83663 .30694 .84366 .3176 .86037 .30972 .85342 .29897 Metetra
.708 .751 .886 r
.85342 .29897 .86037 .30972 .87718 .302 .87029 .29115 Metetra
.558 .687 .926 r
.26057 .48286 .26963 .49227 .28326 .49101 .27422 .48156 Metetra
.546 .682 .928 r
.27422 .48156 .28326 .49101 .29696 .49001 .28795 .48055 Metetra
.535 .679 .931 r
.28795 .48055 .29696 .49001 .31075 .48923 .30177 .4798 Metetra
.525 .676 .933 r
.30177 .4798 .31075 .48923 .32464 .48864 .31568 .47927 Metetra
.517 .675 .936 r
.31568 .47927 .32464 .48864 .33863 .48817 .32969 .47888 Metetra
.511 .676 .939 r
.32969 .47888 .33863 .48817 .35272 .48775 .34381 .47859 Metetra
.509 .679 .942 r
.34381 .47859 .35272 .48775 .36693 .48731 .35804 .4783 Metetra
.509 .684 .946 r
.35804 .4783 .36693 .48731 .38126 .48678 .37239 .47794 Metetra
.513 .691 .949 r
.37239 .47794 .38126 .48678 .3957 .48606 .38687 .47741 Metetra
.521 .7 .952 r
.38687 .47741 .3957 .48606 .41027 .48509 .40147 .47663 Metetra
.532 .711 .954 r
.40147 .47663 .41027 .48509 .42497 .48377 .4162 .47551 Metetra
.546 .723 .956 r
.4162 .47551 .42497 .48377 .43979 .48204 .43106 .47397 Metetra
.562 .735 .957 r
.43106 .47397 .43979 .48204 .45472 .47984 .44604 .47194 Metetra
.581 .748 .956 r
.44604 .47194 .45472 .47984 .46977 .47711 .46114 .46936 Metetra
.601 .76 .955 r
.46114 .46936 .46977 .47711 .48494 .47382 .47636 .46618 Metetra
.621 .771 .952 r
.47636 .46618 .48494 .47382 .5002 .46994 .49168 .46238 Metetra
.64 .78 .948 r
.49168 .46238 .5002 .46994 .51556 .46545 .5071 .45793 Metetra
.658 .788 .943 r
.5071 .45793 .51556 .46545 .53101 .46036 .52262 .45284 Metetra
.675 .794 .938 r
.52262 .45284 .53101 .46036 .54655 .45468 .53822 .44712 Metetra
.69 .798 .932 r
.53822 .44712 .54655 .45468 .56215 .44845 .55389 .44081 Metetra
.703 .801 .925 r
.55389 .44081 .56215 .44845 .57783 .44171 .56964 .43394 Metetra
.714 .802 .919 r
.56964 .43394 .57783 .44171 .59357 .43449 .58545 .42657 Metetra
.723 .801 .913 r
.58545 .42657 .59357 .43449 .60937 .42685 .60132 .41876 Metetra
.73 .8 .907 r
.60132 .41876 .60937 .42685 .62522 .41886 .61725 .41056 Metetra
.735 .798 .902 r
.61725 .41056 .62522 .41886 .64113 .41058 .63322 .40206 Metetra
.739 .795 .897 r
.63322 .40206 .64113 .41058 .65709 .40205 .64925 .39331 Metetra
.741 .792 .893 r
.64925 .39331 .65709 .40205 .6731 .39336 .66533 .38438 Metetra
.742 .788 .89 r
.66533 .38438 .6731 .39336 .68916 .38455 .68146 .37533 Metetra
.742 .784 .887 r
.68146 .37533 .68916 .38455 .70528 .37567 .69764 .36622 Metetra
.74 .78 .885 r
.69764 .36622 .70528 .37567 .72146 .36678 .71389 .35711 Metetra
.738 .776 .883 r
.71389 .35711 .72146 .36678 .73769 .35792 .73019 .34804 Metetra
.736 .772 .882 r
.73019 .34804 .73769 .35792 .75399 .34912 .74656 .33904 Metetra
.732 .768 .882 r
.74656 .33904 .75399 .34912 .77037 .34041 .76299 .33016 Metetra
.729 .765 .882 r
.76299 .33016 .77037 .34041 .78681 .33183 .7795 .32141 Metetra
.725 .761 .882 r
.7795 .32141 .78681 .33183 .80333 .32338 .79609 .31281 Metetra
.721 .758 .882 r
.79609 .31281 .80333 .32338 .81994 .31508 .81276 .30439 Metetra
.717 .755 .883 r
.81276 .30439 .81994 .31508 .83663 .30694 .82953 .29613 Metetra
.713 .753 .884 r
.82953 .29613 .83663 .30694 .85342 .29897 .84638 .28805 Metetra
.708 .751 .885 r
.84638 .28805 .85342 .29897 .87029 .29115 .86333 .28015 Metetra
.559 .686 .924 r
.25143 .47331 .26057 .48286 .27422 .48156 .26511 .47196 Metetra
.546 .681 .927 r
.26511 .47196 .27422 .48156 .28795 .48055 .27887 .47093 Metetra
.534 .676 .929 r
.27887 .47093 .28795 .48055 .30177 .4798 .29271 .4702 Metetra
.522 .673 .932 r
.29271 .4702 .30177 .4798 .31568 .47927 .30664 .46972 Metetra
.513 .671 .935 r
.30664 .46972 .31568 .47927 .32969 .47888 .32068 .46942 Metetra
.505 .672 .938 r
.32068 .46942 .32969 .47888 .34381 .47859 .33482 .46926 Metetra
.501 .674 .942 r
.33482 .46926 .34381 .47859 .35804 .4783 .34907 .46913 Metetra
.5 .679 .945 r
.34907 .46913 .35804 .4783 .37239 .47794 .36345 .46895 Metetra
.503 .686 .949 r
.36345 .46895 .37239 .47794 .38687 .47741 .37795 .46863 Metetra
.51 .695 .952 r
.37795 .46863 .38687 .47741 .40147 .47663 .39258 .46807 Metetra
.521 .706 .955 r
.39258 .46807 .40147 .47663 .4162 .47551 .40734 .46716 Metetra
.535 .719 .957 r
.40734 .46716 .4162 .47551 .43106 .47397 .42224 .46583 Metetra
.553 .732 .959 r
.42224 .46583 .43106 .47397 .44604 .47194 .43726 .464 Metetra
.572 .746 .959 r
.43726 .464 .44604 .47194 .46114 .46936 .45241 .46158 Metetra
.594 .759 .957 r
.45241 .46158 .46114 .46936 .47636 .46618 .46768 .45854 Metetra
.615 .771 .955 r
.46768 .45854 .47636 .46618 .49168 .46238 .48306 .45483 Metetra
.636 .781 .951 r
.48306 .45483 .49168 .46238 .5071 .45793 .49854 .45043 Metetra
.656 .79 .946 r
.49854 .45043 .5071 .45793 .52262 .45284 .51412 .44535 Metetra
.675 .797 .94 r
.51412 .44535 .52262 .45284 .53822 .44712 .52979 .43959 Metetra
.691 .801 .933 r
.52979 .43959 .53822 .44712 .55389 .44081 .54554 .43319 Metetra
.705 .804 .926 r
.54554 .43319 .55389 .44081 .56964 .43394 .56136 .4262 Metetra
.717 .805 .92 r
.56136 .4262 .56964 .43394 .58545 .42657 .57724 .41866 Metetra
.726 .805 .913 r
.57724 .41866 .58545 .42657 .60132 .41876 .59318 .41065 Metetra
.734 .803 .907 r
.59318 .41065 .60132 .41876 .61725 .41056 .60918 .40224 Metetra
.739 .8 .901 r
.60918 .40224 .61725 .41056 .63322 .40206 .62523 .3935 Metetra
.743 .797 .896 r
.62523 .3935 .63322 .40206 .64925 .39331 .64133 .3845 Metetra
.745 .793 .892 r
.64133 .3845 .64925 .39331 .66533 .38438 .65747 .37531 Metetra
.745 .789 .888 r
.65747 .37531 .66533 .38438 .68146 .37533 .67367 .36601 Metetra
.745 .785 .885 r
.67367 .36601 .68146 .37533 .69764 .36622 .68992 .35666 Metetra
.743 .781 .883 r
.68992 .35666 .69764 .36622 .71389 .35711 .70623 .34731 Metetra
.741 .776 .882 r
.70623 .34731 .71389 .35711 .73019 .34804 .7226 .33802 Metetra
.738 .772 .881 r
.7226 .33802 .73019 .34804 .74656 .33904 .73903 .32882 Metetra
.734 .768 .88 r
.73903 .32882 .74656 .33904 .76299 .33016 .75554 .31975 Metetra
.73 .764 .88 r
.75554 .31975 .76299 .33016 .7795 .32141 .77211 .31083 Metetra
.726 .761 .88 r
.77211 .31083 .7795 .32141 .79609 .31281 .78877 .30208 Metetra
.722 .758 .881 r
.78877 .30208 .79609 .31281 .81276 .30439 .80551 .29352 Metetra
.717 .755 .882 r
.80551 .29352 .81276 .30439 .82953 .29613 .82234 .28515 Metetra
.713 .752 .883 r
.82234 .28515 .82953 .29613 .84638 .28805 .83926 .27697 Metetra
.709 .75 .884 r
.83926 .27697 .84638 .28805 .86333 .28015 .85628 .26898 Metetra
.56 .685 .923 r
.24221 .46362 .25143 .47331 .26511 .47196 .25592 .4622 Metetra
.546 .679 .926 r
.25592 .4622 .26511 .47196 .27887 .47093 .26971 .46114 Metetra
.532 .674 .928 r
.26971 .46114 .27887 .47093 .29271 .4702 .28357 .46041 Metetra
.52 .67 .931 r
.28357 .46041 .29271 .4702 .30664 .46972 .29753 .45998 Metetra
.508 .667 .934 r
.29753 .45998 .30664 .46972 .32068 .46942 .31158 .45977 Metetra
.499 .666 .937 r
.31158 .45977 .32068 .46942 .33482 .46926 .32574 .45973 Metetra
.493 .668 .941 r
.32574 .45973 .33482 .46926 .34907 .46913 .34002 .45977 Metetra
.491 .672 .945 r
.34002 .45977 .34907 .46913 .36345 .46895 .35441 .45979 Metetra
.492 .679 .949 r
.35441 .45979 .36345 .46895 .37795 .46863 .36894 .4597 Metetra
.498 .688 .952 r
.36894 .4597 .37795 .46863 .39258 .46807 .3836 .45937 Metetra
.509 .7 .956 r
.3836 .45937 .39258 .46807 .40734 .46716 .39839 .45871 Metetra
.523 .713 .959 r
.39839 .45871 .40734 .46716 .42224 .46583 .41332 .45762 Metetra
.542 .728 .961 r
.41332 .45762 .42224 .46583 .43726 .464 .42838 .456 Metetra
.563 .743 .961 r
.42838 .456 .43726 .464 .45241 .46158 .44358 .45378 Metetra
.585 .758 .96 r
.44358 .45378 .45241 .46158 .46768 .45854 .4589 .45089 Metetra
.609 .771 .958 r
.4589 .45089 .46768 .45854 .48306 .45483 .47434 .4473 Metetra
.632 .783 .954 r
.47434 .4473 .48306 .45483 .49854 .45043 .48988 .44297 Metetra
.654 .792 .948 r
.48988 .44297 .49854 .45043 .51412 .44535 .50553 .4379 Metetra
.674 .8 .942 r
.50553 .4379 .51412 .44535 .52979 .43959 .52127 .4321 Metetra
.692 .805 .935 r
.52127 .4321 .52979 .43959 .54554 .43319 .53709 .42561 Metetra
.707 .808 .928 r
.53709 .42561 .54554 .43319 .56136 .4262 .55298 .41848 Metetra
.72 .809 .92 r
.55298 .41848 .56136 .4262 .57724 .41866 .56894 .41077 Metetra
.73 .808 .913 r
.56894 .41077 .57724 .41866 .59318 .41065 .58495 .40255 Metetra
.738 .806 .906 r
.58495 .40255 .59318 .41065 .60918 .40224 .60102 .3939 Metetra
.743 .803 .9 r
.60102 .3939 .60918 .40224 .62523 .3935 .61714 .3849 Metetra
.747 .799 .895 r
.61714 .3849 .62523 .3935 .64133 .3845 .63331 .37563 Metetra
.749 .795 .89 r
.63331 .37563 .64133 .3845 .65747 .37531 .64953 .36617 Metetra
.749 .791 .887 r
.64953 .36617 .65747 .37531 .67367 .36601 .6658 .3566 Metetra
.748 .786 .884 r
.6658 .3566 .67367 .36601 .68992 .35666 .68212 .34698 Metetra
.746 .781 .881 r
.68212 .34698 .68992 .35666 .70623 .34731 .69849 .33738 Metetra
.744 .777 .88 r
.69849 .33738 .70623 .34731 .7226 .33802 .71493 .32785 Metetra
.74 .772 .879 r
.71493 .32785 .7226 .33802 .73903 .32882 .73143 .31843 Metetra
.736 .768 .878 r
.73143 .31843 .73903 .32882 .75554 .31975 .748 .30916 Metetra
.732 .764 .878 r
.748 .30916 .75554 .31975 .77211 .31083 .76464 .30007 Metetra
.728 .76 .879 r
.76464 .30007 .77211 .31083 .78877 .30208 .78136 .29117 Metetra
.723 .757 .879 r
.78136 .29117 .78877 .30208 .80551 .29352 .79817 .28248 Metetra
.718 .754 .88 r
.79817 .28248 .80551 .29352 .82234 .28515 .81507 .27399 Metetra
.713 .751 .882 r
.81507 .27399 .82234 .28515 .83926 .27697 .83207 .26572 Metetra
.709 .748 .883 r
.83207 .26572 .83926 .27697 .85628 .26898 .84916 .25765 Metetra
.562 .685 .922 r
.2329 .45379 .24221 .46362 .25592 .4622 .24665 .45229 Metetra
.547 .678 .924 r
.24665 .45229 .25592 .4622 .26971 .46114 .26047 .45118 Metetra
.532 .672 .927 r
.26047 .45118 .26971 .46114 .28357 .46041 .27436 .45044 Metetra
.517 .667 .93 r
.27436 .45044 .28357 .46041 .29753 .45998 .28834 .45004 Metetra
.504 .663 .933 r
.28834 .45004 .29753 .45998 .31158 .45977 .30241 .44992 Metetra
.493 .661 .936 r
.30241 .44992 .31158 .45977 .32574 .45973 .31659 .45001 Metetra
.485 .662 .94 r
.31659 .45001 .32574 .45973 .34002 .45977 .33088 .45022 Metetra
.48 .665 .944 r
.33088 .45022 .34002 .45977 .35441 .45979 .34529 .45045 Metetra
.48 .671 .948 r
.34529 .45045 .35441 .45979 .36894 .4597 .35984 .4506 Metetra
.485 .681 .952 r
.35984 .4506 .36894 .4597 .3836 .45937 .37452 .45053 Metetra
.495 .693 .956 r
.37452 .45053 .3836 .45937 .39839 .45871 .38934 .45015 Metetra
.51 .707 .96 r
.38934 .45015 .39839 .45871 .41332 .45762 .4043 .44932 Metetra
.529 .723 .962 r
.4043 .44932 .41332 .45762 .42838 .456 .41941 .44796 Metetra
.551 .74 .964 r
.41941 .44796 .42838 .456 .44358 .45378 .43465 .44596 Metetra
.576 .756 .963 r
.43465 .44596 .44358 .45378 .4589 .45089 .45002 .44326 Metetra
.601 .771 .961 r
.45002 .44326 .4589 .45089 .47434 .4473 .46551 .4398 Metetra
.627 .784 .957 r
.46551 .4398 .47434 .4473 .48988 .44297 .48112 .43555 Metetra
.651 .795 .952 r
.48112 .43555 .48988 .44297 .50553 .4379 .49683 .4305 Metetra
.673 .803 .945 r
.49683 .4305 .50553 .4379 .52127 .4321 .51264 .42467 Metetra
.692 .809 .937 r
.51264 .42467 .52127 .4321 .53709 .42561 .52854 .41809 Metetra
.709 .812 .929 r
.52854 .41809 .53709 .42561 .55298 .41848 .5445 .41082 Metetra
.723 .813 .921 r
.5445 .41082 .55298 .41848 .56894 .41077 .56054 .40291 Metetra
.734 .812 .913 r
.56054 .40291 .56894 .41077 .58495 .40255 .57663 .39446 Metetra
.742 .809 .906 r
.57663 .39446 .58495 .40255 .60102 .3939 .59277 .38555 Metetra
.748 .806 .899 r
.59277 .38555 .60102 .3939 .61714 .3849 .60897 .37626 Metetra
.751 .802 .893 r
.60897 .37626 .61714 .3849 .63331 .37563 .62521 .3667 Metetra
.753 .797 .888 r
.62521 .3667 .63331 .37563 .64953 .36617 .6415 .35694 Metetra
.753 .792 .884 r
.6415 .35694 .64953 .36617 .6658 .3566 .65784 .34707 Metetra
.752 .787 .881 r
.65784 .34707 .6658 .3566 .68212 .34698 .67423 .33717 Metetra
.75 .782 .879 r
.67423 .33717 .68212 .34698 .69849 .33738 .69067 .3273 Metetra
.747 .777 .877 r
.69067 .3273 .69849 .33738 .71493 .32785 .70717 .31752 Metetra
.743 .772 .876 r
.70717 .31752 .71493 .32785 .73143 .31843 .72373 .30787 Metetra
.738 .767 .876 r
.72373 .30787 .73143 .31843 .748 .30916 .74037 .2984 Metetra
.734 .763 .876 r
.74037 .2984 .748 .30916 .76464 .30007 .75708 .28913 Metetra
.729 .759 .877 r
.75708 .28913 .76464 .30007 .78136 .29117 .77387 .28007 Metetra
.724 .756 .878 r
.77387 .28007 .78136 .29117 .79817 .28248 .79075 .27125 Metetra
.718 .752 .879 r
.79075 .27125 .79817 .28248 .81507 .27399 .80772 .26265 Metetra
.713 .75 .88 r
.80772 .26265 .81507 .27399 .83207 .26572 .82479 .25428 Metetra
.709 .747 .882 r
.82479 .25428 .83207 .26572 .84916 .25765 .84195 .24614 Metetra
.565 .684 .92 r
.22352 .44382 .2329 .45379 .24665 .45229 .2373 .44221 Metetra
.549 .677 .923 r
.2373 .44221 .24665 .45229 .26047 .45118 .25115 .44103 Metetra
.532 .67 .925 r
.25115 .44103 .26047 .45118 .27436 .45044 .26506 .44027 Metetra
.516 .663 .928 r
.26506 .44027 .27436 .45044 .28834 .45004 .27906 .43989 Metetra
.5 .658 .931 r
.27906 .43989 .28834 .45004 .30241 .44992 .29316 .43984 Metetra
.487 .655 .935 r
.29316 .43984 .30241 .44992 .31659 .45001 .30735 .44006 Metetra
.477 .655 .938 r
.30735 .44006 .31659 .45001 .33088 .45022 .32166 .44045 Metetra
.47 .657 .942 r
.32166 .44045 .33088 .45022 .34529 .45045 .33609 .44091 Metetra
.468 .663 .947 r
.33609 .44091 .34529 .45045 .35984 .4506 .35065 .44132 Metetra
.471 .672 .952 r
.35065 .44132 .35984 .4506 .37452 .45053 .36535 .44154 Metetra
.48 .685 .957 r
.36535 .44154 .37452 .45053 .38934 .45015 .3802 .44146 Metetra
.495 .7 .961 r
.3802 .44146 .38934 .45015 .4043 .44932 .39519 .44094 Metetra
.514 .717 .964 r
.39519 .44094 .4043 .44932 .41941 .44796 .41033 .43986 Metetra
.538 .736 .966 r
.41033 .43986 .41941 .44796 .43465 .44596 .42561 .43812 Metetra
.565 .754 .967 r
.42561 .43812 .43465 .44596 .45002 .44326 .44103 .43564 Metetra
.593 .771 .965 r
.44103 .43564 .45002 .44326 .46551 .4398 .45658 .43234 Metetra
.621 .786 .961 r
.45658 .43234 .46551 .4398 .48112 .43555 .47225 .42819 Metetra
.647 .798 .955 r
.47225 .42819 .48112 .43555 .49683 .4305 .48803 .42318 Metetra
.672 .807 .948 r
.48803 .42318 .49683 .4305 .51264 .42467 .50391 .41732 Metetra
.693 .813 .939 r
.50391 .41732 .51264 .42467 .52854 .41809 .51988 .41065 Metetra
.711 .816 .93 r
.51988 .41065 .52854 .41809 .5445 .41082 .53593 .40321 Metetra
.726 .817 .921 r
.53593 .40321 .5445 .41082 .56054 .40291 .55204 .3951 Metetra
.738 .816 .913 r
.55204 .3951 .56054 .40291 .57663 .39446 .56821 .3864 Metetra
.746 .813 .905 r
.56821 .3864 .57663 .39446 .59277 .38555 .58443 .37719 Metetra
.752 .809 .898 r
.58443 .37719 .59277 .38555 .60897 .37626 .6007 .3676 Metetra
.756 .804 .892 r
.6007 .3676 .60897 .37626 .62521 .3667 .61702 .35771 Metetra
.758 .799 .886 r
.61702 .35771 .62521 .3667 .6415 .35694 .63338 .34762 Metetra
.758 .793 .882 r
.63338 .34762 .6415 .35694 .65784 .34707 .64979 .33743 Metetra
.756 .788 .879 r
.64979 .33743 .65784 .34707 .67423 .33717 .66624 .32722 Metetra
.753 .782 .876 r
.66624 .32722 .67423 .33717 .69067 .3273 .68275 .31706 Metetra
.75 .777 .875 r
.68275 .31706 .69067 .3273 .70717 .31752 .69932 .30702 Metetra
.745 .771 .874 r
.69932 .30702 .70717 .31752 .72373 .30787 .71595 .29713 Metetra
.741 .766 .874 r
.71595 .29713 .72373 .30787 .74037 .2984 .73266 .28745 Metetra
.735 .762 .874 r
.73266 .28745 .74037 .2984 .75708 .28913 .74943 .27799 Metetra
.73 .758 .875 r
.74943 .27799 .75708 .28913 .77387 .28007 .76629 .26878 Metetra
.724 .754 .876 r
.76629 .26878 .77387 .28007 .79075 .27125 .78324 .25982 Metetra
.719 .751 .878 r
.78324 .25982 .79075 .27125 .80772 .26265 .80028 .25112 Metetra
.713 .748 .879 r
.80028 .25112 .80772 .26265 .82479 .25428 .81742 .24266 Metetra
.708 .746 .881 r
.81742 .24266 .82479 .25428 .84195 .24614 .83466 .23445 Metetra
.568 .685 .919 r
.21405 .4337 .22352 .44382 .2373 .44221 .22787 .43197 Metetra
.551 .676 .921 r
.22787 .43197 .2373 .44221 .25115 .44103 .24175 .4307 Metetra
.533 .668 .924 r
.24175 .4307 .25115 .44103 .26506 .44027 .25569 .42989 Metetra
.515 .66 .926 r
.25569 .42989 .26506 .44027 .27906 .43989 .26971 .42951 Metetra
.497 .654 .929 r
.26971 .42951 .27906 .43989 .29316 .43984 .28382 .42953 Metetra
.482 .65 .933 r
.28382 .42953 .29316 .43984 .30735 .44006 .29803 .42987 Metetra
.468 .648 .936 r
.29803 .42987 .30735 .44006 .32166 .44045 .31235 .43045 Metetra
.459 .649 .941 r
.31235 .43045 .32166 .44045 .33609 .44091 .3268 .43114 Metetra
.454 .654 .946 r
.3268 .43114 .33609 .44091 .35065 .44132 .34137 .43184 Metetra
.455 .662 .951 r
.34137 .43184 .35065 .44132 .36535 .44154 .35609 .43238 Metetra
.463 .675 .956 r
.35609 .43238 .36535 .44154 .3802 .44146 .37095 .43264 Metetra
.477 .691 .961 r
.37095 .43264 .3802 .44146 .39519 .44094 .38597 .43246 Metetra
.497 .71 .966 r
.38597 .43246 .39519 .44094 .41033 .43986 .40114 .43172 Metetra
.522 .73 .969 r
.40114 .43172 .41033 .43986 .42561 .43812 .41646 .43028 Metetra
.551 .751 .97 r
.41646 .43028 .42561 .43812 .44103 .43564 .43193 .42805 Metetra
.582 .77 .969 r
.43193 .42805 .44103 .43564 .45658 .43234 .44754 .42494 Metetra
.613 .787 .965 r
.44754 .42494 .45658 .43234 .47225 .42819 .46327 .42092 Metetra
.643 .801 .959 r
.46327 .42092 .47225 .42819 .48803 .42318 .47912 .41596 Metetra
.67 .811 .951 r
.47912 .41596 .48803 .42318 .50391 .41732 .49507 .41008 Metetra
.694 .818 .942 r
.49507 .41008 .50391 .41732 .51988 .41065 .51112 .4033 Metetra
.714 .822 .932 r
.51112 .4033 .51988 .41065 .53593 .40321 .52724 .3957 Metetra
.73 .822 .922 r
.52724 .3957 .53593 .40321 .55204 .3951 .54343 .38736 Metetra
.743 .821 .913 r
.54343 .38736 .55204 .3951 .56821 .3864 .55968 .37837 Metetra
.752 .817 .904 r
.55968 .37837 .56821 .3864 .58443 .37719 .57599 .36885 Metetra
.758 .813 .896 r
.57599 .36885 .58443 .37719 .6007 .3676 .59234 .3589 Metetra
.761 .807 .889 r
.59234 .3589 .6007 .3676 .61702 .35771 .60873 .34866 Metetra
.763 .801 .884 r
.60873 .34866 .61702 .35771 .63338 .34762 .62516 .33821 Metetra
.762 .795 .879 r
.62516 .33821 .63338 .34762 .64979 .33743 .64164 .32767 Metetra
.76 .788 .876 r
.64164 .32767 .64979 .33743 .66624 .32722 .65817 .31712 Metetra
.757 .782 .874 r
.65817 .31712 .66624 .32722 .68275 .31706 .67475 .30665 Metetra
.753 .776 .872 r
.67475 .30665 .68275 .31706 .69932 .30702 .69138 .29632 Metetra
.748 .771 .871 r
.69138 .29632 .69932 .30702 .71595 .29713 .70808 .28618 Metetra
.743 .765 .871 r
.70808 .28618 .71595 .29713 .73266 .28745 .72485 .27628 Metetra
.737 .761 .872 r
.72485 .27628 .73266 .28745 .74943 .27799 .7417 .26664 Metetra
.731 .756 .873 r
.7417 .26664 .74943 .27799 .76629 .26878 .75863 .25727 Metetra
.725 .753 .874 r
.75863 .25727 .76629 .26878 .78324 .25982 .77565 .24819 Metetra
.719 .749 .876 r
.77565 .24819 .78324 .25982 .80028 .25112 .79276 .23938 Metetra
.713 .747 .878 r
.79276 .23938 .80028 .25112 .81742 .24266 .80997 .23085 Metetra
.708 .744 .88 r
.80997 .23085 .81742 .24266 .83466 .23445 .82729 .22257 Metetra
.573 .685 .917 r
.2045 .42346 .21405 .4337 .22787 .43197 .21836 .42157 Metetra
.554 .676 .919 r
.21836 .42157 .22787 .43197 .24175 .4307 .23227 .42018 Metetra
.535 .667 .922 r
.23227 .42018 .24175 .4307 .25569 .42989 .24624 .41929 Metetra
.515 .658 .924 r
.24624 .41929 .25569 .42989 .26971 .42951 .26029 .4189 Metetra
.495 .65 .927 r
.26029 .4189 .26971 .42951 .28382 .42953 .27442 .41896 Metetra
.477 .644 .93 r
.27442 .41896 .28382 .42953 .29803 .42987 .28864 .41942 Metetra
.46 .64 .934 r
.28864 .41942 .29803 .42987 .31235 .43045 .30297 .42018 Metetra
.448 .64 .938 r
.30297 .42018 .31235 .43045 .3268 .43114 .31742 .42113 Metetra
.44 .643 .943 r
.31742 .42113 .3268 .43114 .34137 .43184 .33201 .42212 Metetra
.438 .651 .949 r
.33201 .42212 .34137 .43184 .35609 .43238 .34673 .42302 Metetra
.444 .664 .955 r
.34673 .42302 .35609 .43238 .37095 .43264 .36161 .42366 Metetra
.457 .681 .961 r
.36161 .42366 .37095 .43264 .38597 .43246 .37665 .42388 Metetra
.477 .701 .967 r
.37665 .42388 .38597 .43246 .40114 .43172 .39185 .42351 Metetra
.504 .724 .971 r
.39185 .42351 .40114 .43172 .41646 .43028 .40721 .42242 Metetra
.535 .747 .973 r
.40721 .42242 .41646 .43028 .43193 .42805 .42272 .42049 Metetra
.569 .769 .973 r
.42272 .42049 .43193 .42805 .44754 .42494 .43838 .41762 Metetra
.604 .789 .969 r
.43838 .41762 .44754 .42494 .46327 .42092 .45418 .41376 Metetra
.638 .805 .963 r
.45418 .41376 .46327 .42092 .47912 .41596 .4701 .40887 Metetra
.668 .817 .955 r
.4701 .40887 .47912 .41596 .49507 .41008 .48612 .40297 Metetra
.695 .824 .944 r
.48612 .40297 .49507 .41008 .51112 .4033 .50225 .39609 Metetra
.717 .828 .934 r
.50225 .39609 .51112 .4033 .52724 .3957 .51845 .3883 Metetra
.735 .828 .923 r
.51845 .3883 .52724 .3957 .54343 .38736 .53473 .3797 Metetra
.748 .826 .912 r
.53473 .3797 .54343 .38736 .55968 .37837 .55106 .3704 Metetra
.758 .822 .903 r
.55106 .3704 .55968 .37837 .57599 .36885 .56744 .36052 Metetra
.764 .816 .894 r
.56744 .36052 .57599 .36885 .59234 .3589 .58387 .35019 Metetra
.767 .81 .887 r
.58387 .35019 .59234 .3589 .60873 .34866 .60034 .33954 Metetra
.768 .803 .881 r
.60034 .33954 .60873 .34866 .62516 .33821 .61685 .32869 Metetra
.767 .796 .876 r
.61685 .32869 .62516 .33821 .64164 .32767 .63341 .31777 Metetra
.765 .789 .873 r
.63341 .31777 .64164 .32767 .65817 .31712 .65 .30685 Metetra
.761 .782 .87 r
.65 .30685 .65817 .31712 .67475 .30665 .66665 .29605 Metetra
.756 .776 .869 r
.66665 .29605 .67475 .30665 .69138 .29632 .68335 .28542 Metetra
.75 .769 .868 r
.68335 .28542 .69138 .29632 .70808 .28618 .70012 .27502 Metetra
.744 .764 .869 r
.70012 .27502 .70808 .28618 .72485 .27628 .71696 .26489 Metetra
.738 .759 .869 r
.71696 .26489 .72485 .27628 .7417 .26664 .73387 .25506 Metetra
.732 .755 .871 r
.73387 .25506 .7417 .26664 .75863 .25727 .75087 .24554 Metetra
.725 .751 .872 r
.75087 .24554 .75863 .25727 .77565 .24819 .76796 .23633 Metetra
.719 .747 .875 r
.76796 .23633 .77565 .24819 .79276 .23938 .78515 .22743 Metetra
.713 .745 .877 r
.78515 .22743 .79276 .23938 .80997 .23085 .80244 .21883 Metetra
.707 .743 .879 r
.80244 .21883 .80997 .23085 .82729 .22257 .81983 .2105 Metetra
.578 .687 .915 r
.19486 .41309 .2045 .42346 .21836 .42157 .20876 .41101 Metetra
.559 .676 .917 r
.20876 .41101 .21836 .42157 .23227 .42018 .22271 .40947 Metetra
.538 .666 .919 r
.22271 .40947 .23227 .42018 .24624 .41929 .23672 .40847 Metetra
.516 .656 .922 r
.23672 .40847 .24624 .41929 .26029 .4189 .25079 .40803 Metetra
.494 .646 .925 r
.25079 .40803 .26029 .4189 .27442 .41896 .26494 .40812 Metetra
.472 .638 .928 r
.26494 .40812 .27442 .41896 .28864 .41942 .27917 .40868 Metetra
.453 .632 .931 r
.27917 .40868 .28864 .41942 .30297 .42018 .29351 .40962 Metetra
.436 .63 .936 r
.29351 .40962 .30297 .42018 .31742 .42113 .30797 .41082 Metetra
.424 .631 .941 r
.30797 .41082 .31742 .42113 .33201 .42212 .32255 .41215 Metetra
.419 .638 .947 r
.32255 .41215 .33201 .42212 .34673 .42302 .33729 .41344 Metetra
.422 .65 .953 r
.33729 .41344 .34673 .42302 .36161 .42366 .35218 .41451 Metetra
.433 .668 .96 r
.35218 .41451 .36161 .42366 .37665 .42388 .36723 .41517 Metetra
.453 .69 .967 r
.36723 .41517 .37665 .42388 .39185 .42351 .38245 .41524 Metetra
.481 .715 .973 r
.38245 .41524 .39185 .42351 .40721 .42242 .39784 .41457 Metetra
.516 .741 .977 r
.39784 .41457 .40721 .42242 .42272 .42049 .4134 .41299 Metetra
.554 .767 .977 r
.4134 .41299 .42272 .42049 .43838 .41762 .42911 .41041 Metetra
.593 .79 .974 r
.42911 .41041 .43838 .41762 .45418 .41376 .44496 .40674 Metetra
.631 .809 .968 r
.44496 .40674 .45418 .41376 .4701 .40887 .46095 .40194 Metetra
.666 .822 .959 r
.46095 .40194 .4701 .40887 .48612 .40297 .47706 .39603 Metetra
.696 .831 .948 r
.47706 .39603 .48612 .40297 .50225 .39609 .49326 .38904 Metetra
.72 .835 .936 r
.49326 .38904 .50225 .39609 .51845 .3883 .50955 .38104 Metetra
.74 .835 .923 r
.50955 .38104 .51845 .3883 .53473 .3797 .52591 .37216 Metetra
.754 .832 .912 r
.52591 .37216 .53473 .3797 .55106 .3704 .54233 .3625 Metetra
.764 .827 .901 r
.54233 .3625 .55106 .3704 .56744 .36052 .5588 .35222 Metetra
.771 .821 .892 r
.5588 .35222 .56744 .36052 .58387 .35019 .57531 .34145 Metetra
.774 .813 .884 r
.57531 .34145 .58387 .35019 .60034 .33954 .59186 .33036 Metetra
.774 .805 .878 r
.59186 .33036 .60034 .33954 .61685 .32869 .60845 .31907 Metetra
.773 .797 .873 r
.60845 .31907 .61685 .32869 .63341 .31777 .62507 .30771 Metetra
.769 .789 .869 r
.62507 .30771 .63341 .31777 .65 .30685 .64174 .2964 Metetra
.765 .782 .867 r
.64174 .2964 .65 .30685 .66665 .29605 .65846 .28523 Metetra
.759 .775 .866 r
.65846 .28523 .66665 .29605 .68335 .28542 .67523 .27428 Metetra
.753 .768 .865 r
.67523 .27428 .68335 .28542 .70012 .27502 .69207 .26361 Metetra
.746 .762 .866 r
.69207 .26361 .70012 .27502 .71696 .26489 .70897 .25325 Metetra
.739 .757 .867 r
.70897 .25325 .71696 .26489 .73387 .25506 .72596 .24323 Metetra
.732 .752 .868 r
.72596 .24323 .73387 .25506 .75087 .24554 .74303 .23356 Metetra
.725 .749 .871 r
.74303 .23356 .75087 .24554 .76796 .23633 .76019 .22424 Metetra
.719 .745 .873 r
.76019 .22424 .76796 .23633 .78515 .22743 .77745 .21526 Metetra
.712 .743 .876 r
.77745 .21526 .78515 .22743 .80244 .21883 .79481 .20661 Metetra
.706 .741 .878 r
.79481 .20661 .80244 .21883 .81983 .2105 .81229 .19824 Metetra
.584 .688 .913 r
.18514 .40261 .19486 .41309 .20876 .41101 .19908 .40031 Metetra
.564 .677 .915 r
.19908 .40031 .20876 .41101 .22271 .40947 .21307 .39857 Metetra
.543 .666 .917 r
.21307 .39857 .22271 .40947 .23672 .40847 .22712 .39743 Metetra
.519 .654 .919 r
.22712 .39743 .23672 .40847 .25079 .40803 .24121 .39691 Metetra
.494 .642 .922 r
.24121 .39691 .25079 .40803 .26494 .40812 .25538 .39698 Metetra
.469 .632 .925 r
.25538 .39698 .26494 .40812 .27917 .40868 .26963 .39762 Metetra
.446 .624 .928 r
.26963 .39762 .27917 .40868 .29351 .40962 .28397 .39873 Metetra
.425 .619 .932 r
.28397 .39873 .29351 .40962 .30797 .41082 .29843 .4002 Metetra
.409 .618 .937 r
.29843 .4002 .30797 .41082 .32255 .41215 .31302 .40188 Metetra
.399 .623 .943 r
.31302 .40188 .32255 .41215 .33729 .41344 .32775 .40359 Metetra
.398 .634 .95 r
.32775 .40359 .33729 .41344 .35218 .41451 .34265 .40514 Metetra
.406 .652 .958 r
.34265 .40514 .35218 .41451 .36723 .41517 .35771 .40631 Metetra
.425 .675 .967 r
.35771 .40631 .36723 .41517 .38245 .41524 .37295 .4069 Metetra
.454 .703 .974 r
.37295 .4069 .38245 .41524 .39784 .41457 .38836 .40671 Metetra
.491 .734 .98 r
.38836 .40671 .39784 .41457 .4134 .41299 .40395 .40556 Metetra
.534 .764 .982 r
.40395 .40556 .4134 .41299 .42911 .41041 .41971 .40332 Metetra
.579 .791 .98 r
.41971 .40332 .42911 .41041 .44496 .40674 .43563 .39989 Metetra
.622 .813 .973 r
.43563 .39989 .44496 .40674 .46095 .40194 .45168 .39522 Metetra
.662 .829 .964 r
.45168 .39522 .46095 .40194 .47706 .39603 .46787 .38931 Metetra
.696 .839 .951 r
.46787 .38931 .47706 .39603 .49326 .38904 .48416 .3822 Metetra
.724 .843 .938 r
.48416 .3822 .49326 .38904 .50955 .38104 .50053 .37398 Metetra
.746 .843 .924 r
.50053 .37398 .50955 .38104 .52591 .37216 .51698 .36477 Metetra
.761 .839 .911 r
.51698 .36477 .52591 .37216 .54233 .3625 .53349 .35471 Metetra
.772 .833 .899 r
.53349 .35471 .54233 .3625 .5588 .35222 .55005 .34396 Metetra
.778 .825 .889 r
.55005 .34396 .5588 .35222 .57531 .34145 .56664 .33271 Metetra
.781 .816 .88 r
.56664 .33271 .57531 .34145 .59186 .33036 .58328 .3211 Metetra
.781 .807 .874 r
.58328 .3211 .59186 .33036 .60845 .31907 .59994 .30931 Metetra
.779 .798 .869 r
.59994 .30931 .60845 .31907 .62507 .30771 .61664 .29748 Metetra
.774 .789 .865 r
.61664 .29748 .62507 .30771 .64174 .2964 .63338 .28574 Metetra
.769 .781 .863 r
.63338 .28574 .64174 .2964 .65846 .28523 .65017 .27418 Metetra
.763 .773 .862 r
.65017 .27418 .65846 .28523 .67523 .27428 .66701 .26289 Metetra
.755 .766 .862 r
.66701 .26289 .67523 .27428 .69207 .26361 .68392 .25192 Metetra
.748 .76 .862 r
.68392 .25192 .69207 .26361 .70897 .25325 .70089 .24133 Metetra
.74 .754 .864 r
.70089 .24133 .70897 .25325 .72596 .24323 .71794 .23113 Metetra
.733 .75 .866 r
.71794 .23113 .72596 .24323 .74303 .23356 .73509 .22132 Metetra
.725 .746 .869 r
.73509 .22132 .74303 .23356 .76019 .22424 .75232 .21191 Metetra
.718 .743 .871 r
.75232 .21191 .76019 .22424 .77745 .21526 .76966 .20286 Metetra
.711 .74 .874 r
.76966 .20286 .77745 .21526 .79481 .20661 .7871 .19416 Metetra
.705 .738 .877 r
.7871 .19416 .79481 .20661 .81229 .19824 .80465 .18578 Metetra
.591 .691 .912 r
.17532 .39203 .18514 .40261 .19908 .40031 .18932 .38947 Metetra
.571 .679 .913 r
.18932 .38947 .19908 .40031 .21307 .39857 .20336 .3875 Metetra
.549 .667 .915 r
.20336 .3875 .21307 .39857 .22712 .39743 .21744 .38616 Metetra
.524 .653 .917 r
.21744 .38616 .22712 .39743 .24121 .39691 .23157 .3855 Metetra
.496 .64 .919 r
.23157 .3855 .24121 .39691 .25538 .39698 .24576 .38553 Metetra
.468 .627 .921 r
.24576 .38553 .25538 .39698 .26963 .39762 .26002 .38621 Metetra
.44 .616 .924 r
.26002 .38621 .26963 .39762 .28397 .39873 .27437 .38747 Metetra
.414 .607 .928 r
.27437 .38747 .28397 .39873 .29843 .4002 .28882 .3892 Metetra
.392 .604 .932 r
.28882 .3892 .29843 .4002 .31302 .40188 .30341 .39125 Metetra
.377 .606 .938 r
.30341 .39125 .31302 .40188 .32775 .40359 .31813 .39343 Metetra
.371 .615 .945 r
.31813 .39343 .32775 .40359 .34265 .40514 .33302 .39552 Metetra
.375 .632 .954 r
.33302 .39552 .34265 .40514 .35771 .40631 .34808 .39728 Metetra
.392 .657 .964 r
.34808 .39728 .35771 .40631 .37295 .4069 .36333 .39847 Metetra
.421 .688 .974 r
.36333 .39847 .37295 .4069 .38836 .40671 .37876 .39885 Metetra
.461 .723 .982 r
.37876 .39885 .38836 .40671 .40395 .40556 .39438 .39822 Metetra
.509 .758 .986 r
.39438 .39822 .40395 .40556 .41971 .40332 .41019 .3964 Metetra
.56 .791 .985 r
.41019 .3964 .41971 .40332 .43563 .39989 .42616 .39327 Metetra
.611 .818 .98 r
.42616 .39327 .43563 .39989 .45168 .39522 .44229 .38877 Metetra
.657 .837 .969 r
.44229 .38877 .45168 .39522 .46787 .38931 .45855 .38287 Metetra
.697 .848 .955 r
.45855 .38287 .46787 .38931 .48416 .3822 .47492 .37564 Metetra
.728 .853 .94 r
.47492 .37564 .48416 .3822 .50053 .37398 .49139 .36716 Metetra
.752 .852 .924 r
.49139 .36716 .50053 .37398 .51698 .36477 .50794 .35757 Metetra
.769 .847 .91 r
.50794 .35757 .51698 .36477 .53349 .35471 .52454 .34704 Metetra
.781 .839 .896 r
.52454 .34704 .53349 .35471 .55005 .34396 .54119 .33577 Metetra
.787 .83 .885 r
.54119 .33577 .55005 .34396 .56664 .33271 .55787 .32395 Metetra
.789 .819 .876 r
.55787 .32395 .56664 .33271 .58328 .3211 .57459 .31177 Metetra
.788 .809 .869 r
.57459 .31177 .58328 .3211 .59994 .30931 .59134 .29942 Metetra
.785 .798 .864 r
.59134 .29942 .59994 .30931 .61664 .29748 .60811 .28706 Metetra
.78 .789 .86 r
.60811 .28706 .61664 .29748 .63338 .28574 .62493 .27483 Metetra
.773 .779 .858 r
.62493 .27483 .63338 .28574 .65017 .27418 .64179 .26284 Metetra
.766 .771 .857 r
.64179 .26284 .65017 .27418 .66701 .26289 .6587 .25119 Metetra
.758 .764 .857 r
.6587 .25119 .66701 .26289 .68392 .25192 .67567 .23994 Metetra
.749 .757 .859 r
.67567 .23994 .68392 .25192 .70089 .24133 .69271 .22911 Metetra
.741 .751 .861 r
.69271 .22911 .70089 .24133 .71794 .23113 .70984 .21874 Metetra
.733 .747 .863 r
.70984 .21874 .71794 .23113 .73509 .22132 .72705 .20881 Metetra
.724 .743 .866 r
.72705 .20881 .73509 .22132 .75232 .21191 .74436 .19931 Metetra
.717 .74 .87 r
.74436 .19931 .75232 .21191 .76966 .20286 .76177 .19022 Metetra
.71 .738 .873 r
.76177 .19022 .76966 .20286 .7871 .19416 .7793 .1815 Metetra
.703 .736 .877 r
.7793 .1815 .7871 .19416 .80465 .18578 .79693 .17312 Metetra
.599 .694 .91 r
.16541 .38137 .17532 .39203 .18932 .38947 .17946 .37852 Metetra
.579 .682 .911 r
.17946 .37852 .18932 .38947 .20336 .3875 .19355 .37626 Metetra
.556 .668 .912 r
.19355 .37626 .20336 .3875 .21744 .38616 .20768 .37468 Metetra
.53 .653 .914 r
.20768 .37468 .21744 .38616 .23157 .3855 .22185 .37383 Metetra
.501 .638 .915 r
.22185 .37383 .23157 .3855 .24576 .38553 .23606 .37374 Metetra
.469 .622 .917 r
.23606 .37374 .24576 .38553 .26002 .38621 .25034 .37442 Metetra
.437 .608 .92 r
.25034 .37442 .26002 .38621 .27437 .38747 .26469 .3758 Metetra
.405 .596 .923 r
.26469 .3758 .27437 .38747 .28882 .3892 .27915 .37778 Metetra
.376 .588 .926 r
.27915 .37778 .28882 .3892 .30341 .39125 .29372 .38022 Metetra
.354 .587 .932 r
.29372 .38022 .30341 .39125 .31813 .39343 .30843 .3829 Metetra
.34 .593 .939 r
.30843 .3829 .31813 .39343 .33302 .39552 .3233 .3856 Metetra
.339 .608 .948 r
.3233 .3856 .33302 .39552 .34808 .39728 .33835 .38803 Metetra
.352 .633 .959 r
.33835 .38803 .34808 .39728 .36333 .39847 .35359 .38992 Metetra
.38 .667 .971 r
.35359 .38992 .36333 .39847 .37876 .39885 .36904 .39099 Metetra
.422 .707 .982 r
.36904 .39099 .37876 .39885 .39438 .39822 .38468 .39098 Metetra
.476 .75 .989 r
.38468 .39098 .39438 .39822 .41019 .3964 .40053 .38968 Metetra
.536 .789 .991 r
.40053 .38968 .41019 .3964 .42616 .39327 .41656 .38693 Metetra
.595 .822 .986 r
.41656 .38693 .42616 .39327 .44229 .38877 .43275 .38264 Metetra
.65 .846 .975 r
.43275 .38264 .44229 .38877 .45855 .38287 .44909 .37679 Metetra
.697 .859 .96 r
.44909 .37679 .45855 .38287 .47492 .37564 .46556 .36942 Metetra
.733 .864 .942 r
.46556 .36942 .47492 .37564 .49139 .36716 .48213 .36065 Metetra
.76 .862 .924 r
.48213 .36065 .49139 .36716 .50794 .35757 .49877 .35062 Metetra
.779 .856 .908 r
.49877 .35062 .50794 .35757 .52454 .34704 .51547 .33955 Metetra
.79 .846 .893 r
.51547 .33955 .52454 .34704 .54119 .33577 .53222 .32766 Metetra
.796 .835 .881 r
.53222 .32766 .54119 .33577 .55787 .32395 .54899 .31518 Metetra
.798 .822 .871 r
.54899 .31518 .55787 .32395 .57459 .31177 .5658 .30234 Metetra
.796 .81 .863 r
.5658 .30234 .57459 .31177 .59134 .29942 .58263 .28934 Metetra
.791 .798 .858 r
.58263 .28934 .59134 .29942 .60811 .28706 .59948 .27639 Metetra
.785 .787 .854 r
.59948 .27639 .60811 .28706 .62493 .27483 .61637 .26363 Metetra
.777 .777 .853 r
.61637 .26363 .62493 .27483 .64179 .26284 .6333 .25119 Metetra
.769 .768 .852 r
.6333 .25119 .64179 .26284 .6587 .25119 .65028 .23916 Metetra
.76 .76 .853 r
.65028 .23916 .6587 .25119 .67567 .23994 .66733 .2276 Metetra
.75 .753 .855 r
.66733 .2276 .67567 .23994 .69271 .22911 .68444 .21655 Metetra
.741 .748 .857 r
.68444 .21655 .69271 .22911 .70984 .21874 .70163 .20602 Metetra
.732 .743 .861 r
.70163 .20602 .70984 .21874 .72705 .20881 .71892 .19599 Metetra
.723 .739 .864 r
.71892 .19599 .72705 .20881 .74436 .19931 .73631 .18644 Metetra
.715 .737 .868 r
.73631 .18644 .74436 .19931 .76177 .19022 .7538 .17733 Metetra
.708 .735 .872 r
.7538 .17733 .76177 .19022 .7793 .1815 .7714 .16862 Metetra
.701 .734 .876 r
.7714 .16862 .7793 .1815 .79693 .17312 .78913 .16027 Metetra
.608 .698 .908 r
.1554 .37066 .16541 .38137 .17946 .37852 .16952 .36748 Metetra
.589 .686 .909 r
.16952 .36748 .17946 .37852 .19355 .37626 .18367 .36488 Metetra
.566 .671 .91 r
.18367 .36488 .19355 .37626 .20768 .37468 .19784 .36299 Metetra
.539 .655 .911 r
.19784 .36299 .20768 .37468 .22185 .37383 .21206 .36187 Metetra
.507 .637 .912 r
.21206 .36187 .22185 .37383 .23606 .37374 .2263 .36161 Metetra
.473 .619 .913 r
.2263 .36161 .23606 .37374 .25034 .37442 .2406 .36221 Metetra
.436 .601 .915 r
.2406 .36221 .25034 .37442 .26469 .3758 .25496 .36366 Metetra
.398 .584 .917 r
.25496 .36366 .26469 .3758 .27915 .37778 .26941 .36587 Metetra
.362 .572 .919 r
.26941 .36587 .27915 .37778 .29372 .38022 .28396 .36869 Metetra
.33 .565 .923 r
.28396 .36869 .29372 .38022 .30843 .3829 .29865 .37192 Metetra
.308 .567 .93 r
.29865 .37192 .30843 .3829 .3233 .3856 .31349 .37529 Metetra
.297 .579 .939 r
.31349 .37529 .3233 .3856 .33835 .38803 .32852 .3785 Metetra
.304 .603 .951 r
.32852 .3785 .33835 .38803 .35359 .38992 .34375 .38122 Metetra
.328 .639 .965 r
.34375 .38122 .35359 .38992 .36904 .39099 .35919 .38311 Metetra
.372 .685 .979 r
.35919 .38311 .36904 .39099 .38468 .39098 .37485 .38387 Metetra
.432 .736 .99 r
.37485 .38387 .38468 .39098 .40053 .38968 .39073 .38322 Metetra
.502 .785 .996 r
.39073 .38322 .40053 .38968 .41656 .38693 .40681 .38095 Metetra
.574 .826 .993 r
.40681 .38095 .41656 .38693 .43275 .38264 .42307 .37695 Metetra
.64 .856 .982 r
.42307 .37695 .43275 .38264 .44909 .37679 .4395 .37117 Metetra
.696 .872 .965 r
.4395 .37117 .44909 .37679 .46556 .36942 .45606 .36366 Metetra
.739 .878 .944 r
.45606 .36366 .46556 .36942 .48213 .36065 .47272 .35453 Metetra
.77 .875 .924 r
.47272 .35453 .48213 .36065 .49877 .35062 .48947 .344 Metetra
.79 .866 .905 r
.48947 .344 .49877 .35062 .51547 .33955 .50628 .33228 Metetra
.802 .854 .889 r
.50628 .33228 .51547 .33955 .53222 .32766 .52313 .31966 Metetra
.807 .84 .875 r
.52313 .31966 .53222 .32766 .54899 .31518 .54001 .3064 Metetra
.808 .825 .864 r
.54001 .3064 .54899 .31518 .5658 .30234 .5569 .29279 Metetra
.804 .811 .857 r
.5569 .29279 .5658 .30234 .58263 .28934 .57382 .27906 Metetra
.798 .798 .851 r
.57382 .27906 .58263 .28934 .59948 .27639 .59075 .26543 Metetra
.79 .785 .848 r
.59075 .26543 .59948 .27639 .61637 .26363 .60771 .25208 Metetra
.781 .774 .846 r
.60771 .25208 .61637 .26363 .6333 .25119 .62472 .23915 Metetra
.771 .764 .847 r
.62472 .23915 .6333 .25119 .65028 .23916 .64177 .22673 Metetra
.761 .756 .848 r
.64177 .22673 .65028 .23916 .66733 .2276 .65888 .21487 Metetra
.751 .749 .851 r
.65888 .21487 .66733 .2276 .68444 .21655 .67606 .20362 Metetra
.74 .743 .854 r
.67606 .20362 .68444 .21655 .70163 .20602 .69333 .19295 Metetra
.731 .739 .858 r
.69333 .19295 .70163 .20602 .71892 .19599 .71069 .18285 Metetra
.721 .736 .862 r
.71069 .18285 .71892 .19599 .73631 .18644 .72816 .17328 Metetra
.713 .733 .867 r
.72816 .17328 .73631 .18644 .7538 .17733 .74573 .16419 Metetra
.705 .732 .872 r
.74573 .16419 .7538 .17733 .7714 .16862 .76342 .15553 Metetra
.698 .731 .876 r
.76342 .15553 .7714 .16862 .78913 .16027 .78123 .14723 Metetra
.617 .703 .906 r
.14528 .35994 .1554 .37066 .16952 .36748 .15947 .35638 Metetra
.599 .691 .906 r
.15947 .35638 .16952 .36748 .18367 .36488 .17369 .3534 Metetra
.577 .676 .907 r
.17369 .3534 .18367 .36488 .19784 .36299 .18793 .35112 Metetra
.55 .658 .907 r
.18793 .35112 .19784 .36299 .21206 .36187 .20219 .34966 Metetra
.517 .638 .908 r
.20219 .34966 .21206 .36187 .2263 .36161 .21648 .34912 Metetra
.48 .617 .909 r
.21648 .34912 .2263 .36161 .2406 .36221 .2308 .34957 Metetra
.438 .595 .909 r
.2308 .34957 .2406 .36221 .25496 .36366 .24518 .351 Metetra
.394 .573 .91 r
.24518 .351 .25496 .36366 .26941 .36587 .25961 .35338 Metetra
.349 .555 .911 r
.25961 .35338 .26941 .36587 .28396 .36869 .27415 .35657 Metetra
.308 .541 .913 r
.27415 .35657 .28396 .36869 .29865 .37192 .2888 .36037 Metetra
.273 .536 .918 r
.2888 .36037 .29865 .37192 .31349 .37529 .30361 .36449 Metetra
.251 .543 .925 r
.30361 .36449 .31349 .37529 .32852 .3785 .3186 .3686 Metetra
.246 .564 .936 r
.3186 .3686 .32852 .3785 .34375 .38122 .3338 .3723 Metetra
.264 .6 .952 r
.3338 .3723 .34375 .38122 .35919 .38311 .34922 .3752 Metetra
.307 .652 .97 r
.34922 .3752 .35919 .38311 .37485 .38387 .36488 .3769 Metetra
.373 .713 .987 r
.36488 .3769 .37485 .38387 .39073 .38322 .38078 .37707 Metetra
.455 .775 .998 r
.38078 .37707 .39073 .38322 .40681 .38095 .3969 .37543 Metetra
.543 .828 .998 r
.3969 .37543 .40681 .38095 .42307 .37695 .41323 .37181 Metetra
.625 .867 .988 r
.41323 .37181 .42307 .37695 .4395 .37117 .42974 .36614 Metetra
.693 .888 .969 r
.42974 .36614 .4395 .37117 .45606 .36366 .4464 .35847 Metetra
.745 .894 .946 r
.4464 .35847 .45606 .36366 .47272 .35453 .46318 .34894 Metetra
.781 .889 .923 r
.46318 .34894 .47272 .35453 .48947 .344 .48004 .33778 Metetra
.803 .878 .901 r
.48004 .33778 .48947 .344 .50628 .33228 .49696 .32529 Metetra
.815 .862 .883 r
.49696 .32529 .50628 .33228 .52313 .31966 .51392 .31178 Metetra
.82 .845 .868 r
.51392 .31178 .52313 .31966 .54001 .3064 .5309 .2976 Metetra
.818 .828 .857 r
.5309 .2976 .54001 .3064 .5569 .29279 .54789 .28307 Metetra
.813 .811 .848 r
.54789 .28307 .5569 .29279 .57382 .27906 .5649 .26849 Metetra
.805 .796 .843 r
.5649 .26849 .57382 .27906 .59075 .26543 .58191 .2541 Metetra
.796 .782 .84 r
.58191 .2541 .59075 .26543 .60771 .25208 .59895 .24011 Metetra
.785 .77 .839 r
.59895 .24011 .60771 .25208 .62472 .23915 .61603 .22665 Metetra
.773 .759 .84 r
.61603 .22665 .62472 .23915 .64177 .22673 .63315 .21383 Metetra
.762 .75 .843 r
.63315 .21383 .64177 .22673 .65888 .21487 .65033 .20169 Metetra
.75 .743 .846 r
.65033 .20169 .65888 .21487 .67606 .20362 .66759 .19026 Metetra
.739 .738 .85 r
.66759 .19026 .67606 .20362 .69333 .19295 .68493 .1795 Metetra
.728 .734 .855 r
.68493 .1795 .69333 .19295 .71069 .18285 .70237 .16938 Metetra
.719 .731 .861 r
.70237 .16938 .71069 .18285 .72816 .17328 .71991 .15984 Metetra
.71 .729 .866 r
.71991 .15984 .72816 .17328 .74573 .16419 .73757 .15081 Metetra
.702 .729 .871 r
.73757 .15081 .74573 .16419 .76342 .15553 .75535 .14223 Metetra
.695 .729 .876 r
.75535 .14223 .76342 .15553 .78123 .14723 .77324 .13402 Metetra
.627 .709 .905 r
.13505 .34924 .14528 .35994 .15947 .35638 .14931 .34529 Metetra
.611 .697 .905 r
.14931 .34529 .15947 .35638 .17369 .3534 .16361 .34187 Metetra
.59 .681 .904 r
.16361 .34187 .17369 .3534 .18793 .35112 .17792 .33913 Metetra
.563 .663 .904 r
.17792 .33913 .18793 .35112 .20219 .34966 .19225 .33723 Metetra
.53 .642 .904 r
.19225 .33723 .20219 .34966 .21648 .34912 .20659 .3363 Metetra
.491 .618 .904 r
.20659 .3363 .21648 .34912 .2308 .34957 .22095 .33646 Metetra
.446 .591 .903 r
.22095 .33646 .2308 .34957 .24518 .351 .23534 .33777 Metetra
.395 .564 .902 r
.23534 .33777 .24518 .351 .25961 .35338 .24978 .34023 Metetra
.341 .539 .902 r
.24978 .34023 .25961 .35338 .27415 .35657 .26429 .34375 Metetra
.287 .517 .901 r
.26429 .34375 .27415 .35657 .2888 .36037 .27891 .34812 Metetra
.238 .503 .902 r
.27891 .34812 .2888 .36037 .30361 .36449 .29366 .35307 Metetra
.199 .5 .906 r
.29366 .35307 .30361 .36449 .3186 .3686 .30859 .3582 Metetra
.179 .514 .915 r
.30859 .3582 .3186 .3686 .3338 .3723 .32374 .36308 Metetra
.184 .548 .93 r
.32374 .36308 .3338 .3723 .34922 .3752 .33912 .36721 Metetra
.221 .603 .95 r
.33912 .36721 .34922 .3752 .36488 .3769 .35476 .37011 Metetra
.29 .675 .973 r
.35476 .37011 .36488 .3769 .38078 .37707 .37066 .37133 Metetra
.387 .754 .992 r
.37066 .37133 .38078 .37707 .3969 .37543 .38682 .3705 Metetra
.496 .826 .999 r
.38682 .3705 .3969 .37543 .41323 .37181 .40321 .3674 Metetra
.601 .878 .992 r
.40321 .3674 .41323 .37181 .42974 .36614 .41981 .3619 Metetra
.688 .906 .973 r
.41981 .3619 .42974 .36614 .4464 .35847 .43658 .35406 Metetra
.751 .914 .947 r
.43658 .35406 .4464 .35847 .46318 .34894 .45348 .34403 Metetra
.794 .907 .92 r
.45348 .34403 .46318 .34894 .48004 .33778 .47046 .3321 Metetra
.818 .891 .895 r
.47046 .3321 .48004 .33778 .49696 .32529 .48751 .31864 Metetra
.83 .871 .875 r
.48751 .31864 .49696 .32529 .51392 .31178 .50459 .30406 Metetra
.833 .85 .859 r
.50459 .30406 .51392 .31178 .5309 .2976 .52168 .28876 Metetra
.83 .83 .847 r
.52168 .28876 .5309 .2976 .54789 .28307 .53878 .27314 Metetra
.823 .81 .839 r
.53878 .27314 .54789 .28307 .5649 .26849 .55587 .25756 Metetra
.812 .792 .833 r
.55587 .25756 .5649 .26849 .58191 .2541 .57297 .24231 Metetra
.8 .777 .831 r
.57297 .24231 .58191 .2541 .59895 .24011 .59009 .2276 Metetra
.788 .763 .831 r
.59009 .2276 .59895 .24011 .61603 .22665 .60723 .21359 Metetra
.774 .752 .833 r
.60723 .21359 .61603 .22665 .63315 .21383 .62443 .20038 Metetra
.761 .743 .837 r
.62443 .20038 .63315 .21383 .65033 .20169 .64168 .18799 Metetra
.748 .737 .841 r
.64168 .18799 .65033 .20169 .66759 .19026 .65901 .17643 Metetra
.736 .731 .847 r
.65901 .17643 .66759 .19026 .68493 .1795 .67643 .16564 Metetra
.725 .728 .853 r
.67643 .16564 .68493 .1795 .70237 .16938 .69395 .15556 Metetra
.715 .726 .859 r
.69395 .15556 .70237 .16938 .71991 .15984 .71157 .14611 Metetra
.705 .725 .866 r
.71157 .14611 .71991 .15984 .73757 .15081 .72932 .1372 Metetra
.697 .725 .872 r
.72932 .1372 .73757 .15081 .75535 .14223 .74718 .12874 Metetra
.691 .726 .877 r
.74718 .12874 .75535 .14223 .77324 .13402 .76517 .12066 Metetra
.637 .715 .904 r
.1247 .3386 .13505 .34924 .14931 .34529 .13904 .33424 Metetra
.623 .704 .903 r
.13904 .33424 .14931 .34529 .16361 .34187 .15342 .33035 Metetra
.604 .689 .902 r
.15342 .33035 .16361 .34187 .17792 .33913 .16782 .32708 Metetra
.579 .671 .901 r
.16782 .32708 .17792 .33913 .19225 .33723 .18223 .32463 Metetra
.547 .648 .9 r
.18223 .32463 .19225 .33723 .20659 .3363 .19664 .32317 Metetra
.507 .621 .899 r
.19664 .32317 .20659 .3363 .22095 .33646 .21105 .32289 Metetra
.459 .591 .897 r
.21105 .32289 .22095 .33646 .23534 .33777 .22548 .32392 Metetra
.403 .558 .895 r
.22548 .32392 .23534 .33777 .24978 .34023 .23992 .32632 Metetra
.34 .525 .891 r
.23992 .32632 .24978 .34023 .26429 .34375 .25441 .33006 Metetra
.273 .493 .887 r
.25441 .33006 .26429 .34375 .27891 .34812 .26898 .33498 Metetra
.206 .466 .884 r
.26898 .33498 .27891 .34812 .29366 .35307 .28367 .3408 Metetra
.146 .45 .882 r
.28367 .3408 .29366 .35307 .30859 .3582 .29852 .34712 Metetra
.102 .452 .884 r
.29852 .34712 .30859 .3582 .32374 .36308 .31358 .3534 Metetra
.086 .477 .893 r
.31358 .3534 .32374 .36308 .33912 .36721 .3289 .35907 Metetra
.107 .53 .912 r
.3289 .35907 .33912 .36721 .35476 .37011 .34449 .36349 Metetra
.174 .611 .94 r
.34449 .36349 .35476 .37011 .37066 .37133 .36038 .36608 Metetra
.284 .711 .969 r
.36038 .36608 .37066 .37133 .38682 .3705 .37655 .36635 Metetra
.421 .81 .988 r
.37655 .36635 .38682 .3705 .40321 .3674 .393 .36395 Metetra
.56 .885 .989 r
.393 .36395 .40321 .3674 .41981 .3619 .40969 .35872 Metetra
.675 .927 .972 r
.40969 .35872 .41981 .3619 .43658 .35406 .42657 .35068 Metetra
.757 .937 .943 r
.42657 .35068 .43658 .35406 .45348 .34403 .4436 .34003 Metetra
.809 .927 .913 r
.4436 .34003 .45348 .34403 .47046 .3321 .46073 .32713 Metetra
.837 .907 .886 r
.46073 .32713 .47046 .3321 .48751 .31864 .47792 .31245 Metetra
.848 .881 .864 r
.47792 .31245 .48751 .31864 .50459 .30406 .49513 .2965 Metetra
.849 .855 .847 r
.49513 .2965 .50459 .30406 .52168 .28876 .51234 .27981 Metetra
.843 .83 .834 r
.51234 .27981 .52168 .28876 .53878 .27314 .52954 .26288 Metetra
.832 .807 .826 r
.52954 .26288 .53878 .27314 .55587 .25756 .54674 .24612 Metetra
.819 .787 .822 r
.54674 .24612 .55587 .25756 .57297 .24231 .56392 .22988 Metetra
.804 .769 .821 r
.56392 .22988 .57297 .24231 .59009 .2276 .58112 .2144 Metetra
.789 .755 .822 r
.58112 .2144 .59009 .2276 .60723 .21359 .59834 .19984 Metetra
.774 .743 .825 r
.59834 .19984 .60723 .21359 .62443 .20038 .6156 .18626 Metetra
.759 .735 .83 r
.6156 .18626 .62443 .20038 .64168 .18799 .63292 .17369 Metetra
.745 .728 .836 r
.63292 .17369 .64168 .18799 .65901 .17643 .65033 .16208 Metetra
.732 .724 .843 r
.65033 .16208 .65901 .17643 .67643 .16564 .66783 .15135 Metetra
.72 .722 .851 r
.66783 .15135 .67643 .16564 .69395 .15556 .68543 .1414 Metetra
.71 .721 .858 r
.68543 .1414 .69395 .15556 .71157 .14611 .70314 .13212 Metetra
.7 .721 .866 r
.70314 .13212 .71157 .14611 .72932 .1372 .72097 .12339 Metetra
.692 .722 .873 r
.72097 .12339 .72932 .1372 .74718 .12874 .73893 .11512 Metetra
.686 .724 .879 r
.73893 .11512 .74718 .12874 .76517 .12066 .75701 .1072 Metetra
.646 .722 .904 r
.11422 .32805 .1247 .3386 .13904 .33424 .12864 .32331 Metetra
.635 .712 .902 r
.12864 .32331 .13904 .33424 .15342 .33035 .14311 .31893 Metetra
.62 .698 .901 r
.14311 .31893 .15342 .33035 .16782 .32708 .1576 .31508 Metetra
.598 .68 .899 r
.1576 .31508 .16782 .32708 .18223 .32463 .17211 .31197 Metetra
.568 .657 .897 r
.17211 .31197 .18223 .32463 .19664 .32317 .18661 .30984 Metetra
.529 .629 .894 r
.18661 .30984 .19664 .32317 .21105 .32289 .2011 .30892 Metetra
.48 .596 .891 r
.2011 .30892 .21105 .32289 .22548 .32392 .21558 .30944 Metetra
.42 .557 .887 r
.21558 .30944 .22548 .32392 .23992 .32632 .23004 .31155 Metetra
.349 .515 .88 r
.23004 .31155 .23992 .32632 .25441 .33006 .24452 .31533 Metetra
.268 .471 .872 r
.24452 .31533 .25441 .33006 .26898 .33498 .25904 .3207 Metetra
.181 .429 .862 r
.25904 .3207 .26898 .33498 .28367 .3408 .27365 .32741 Metetra
.095 .395 .851 r
.27365 .32741 .28367 .3408 .29852 .34712 .2884 .33505 Metetra
.02 .377 .842 r
.2884 .33505 .29852 .34712 .31358 .3534 .30335 .34302 Metetra
0 .383 .838 r
.30335 .34302 .31358 .3534 .3289 .35907 .31856 .35061 Metetra
0 .425 .846 r
.31856 .35061 .3289 .35907 .34449 .36349 .33406 .35701 Metetra
.013 .507 .869 r
.33406 .35701 .34449 .36349 .36038 .36608 .3499 .36144 Metetra
.128 .627 .907 r
.3499 .36144 .36038 .36608 .37655 .36635 .36607 .36321 Metetra
.298 .763 .945 r
.36607 .36321 .37655 .36635 .393 .36395 .38256 .36182 Metetra
.486 .878 .964 r
.38256 .36182 .393 .36395 .40969 .35872 .39934 .35699 Metetra
.648 .945 .958 r
.39934 .35699 .40969 .35872 .42657 .35068 .41635 .34873 Metetra
.76 .963 .932 r
.41635 .34873 .42657 .35068 .4436 .34003 .43353 .33727 Metetra
.826 .951 .9 r
.43353 .33727 .4436 .34003 .46073 .32713 .45082 .32309 Metetra
.858 .923 .871 r
.45082 .32309 .46073 .32713 .47792 .31245 .46817 .30681 Metetra
.869 .89 .848 r
.46817 .30681 .47792 .31245 .49513 .2965 .48553 .28911 Metetra
.866 .858 .831 r
.48553 .28911 .49513 .2965 .51234 .27981 .50288 .27067 Metetra
.856 .827 .819 r
.50288 .27067 .51234 .27981 .52954 .26288 .5202 .25211 Metetra
.841 .8 .812 r
.5202 .25211 .52954 .26288 .54674 .24612 .53749 .23396 Metetra
.824 .778 .809 r
.53749 .23396 .54674 .24612 .56392 .22988 .55477 .2166 Metetra
.806 .759 .809 r
.55477 .2166 .56392 .22988 .58112 .2144 .57204 .20031 Metetra
.788 .743 .811 r
.57204 .20031 .58112 .2144 .59834 .19984 .58933 .18521 Metetra
.771 .732 .817 r
.58933 .18521 .59834 .19984 .6156 .18626 .60666 .17136 Metetra
.755 .724 .823 r
.60666 .17136 .6156 .18626 .63292 .17369 .62406 .15872 Metetra
.739 .718 .832 r
.62406 .15872 .63292 .17369 .65033 .16208 .64154 .14719 Metetra
.726 .715 .84 r
.64154 .14719 .65033 .16208 .66783 .15135 .65912 .13665 Metetra
.714 .715 .85 r
.65912 .13665 .66783 .15135 .68543 .1414 .67681 .12694 Metetra
.703 .716 .859 r
.67681 .12694 .68543 .1414 .70314 .13212 .69461 .11792 Metetra
.694 .718 .867 r
.69461 .11792 .70314 .13212 .72097 .12339 .71254 .10945 Metetra
.687 .72 .875 r
.71254 .10945 .72097 .12339 .73893 .11512 .73059 .10141 Metetra
.681 .723 .882 r
.73059 .10141 .73893 .11512 .75701 .1072 .74877 .09369 Metetra
.654 .729 .904 r
.1036 .31762 .11422 .32805 .12864 .32331 .1181 .31253 Metetra
.646 .721 .903 r
.1181 .31253 .12864 .32331 .14311 .31893 .13266 .30769 Metetra
.635 .709 .9 r
.13266 .30769 .14311 .31893 .1576 .31508 .14725 .30325 Metetra
.617 .693 .898 r
.14725 .30325 .1576 .31508 .17211 .31197 .16187 .29941 Metetra
.591 .671 .895 r
.16187 .29941 .17211 .31197 .18661 .30984 .17648 .29644 Metetra
.556 .642 .891 r
.17648 .29644 .18661 .30984 .2011 .30892 .19108 .29466 Metetra
.508 .606 .886 r
.19108 .29466 .2011 .30892 .21558 .30944 .20564 .29437 Metetra
.447 .563 .879 r
.20564 .29437 .21558 .30944 .23004 .31155 .22017 .29588 Metetra
.371 .513 .87 r
.22017 .29588 .23004 .31155 .24452 .31533 .23466 .29939 Metetra
.278 .457 .857 r
.23466 .29939 .24452 .31533 .25904 .3207 .24914 .30497 Metetra
.172 .397 .84 r
.24914 .30497 .25904 .3207 .27365 .32741 .26367 .31248 Metetra
.056 .34 .818 r
.26367 .31248 .27365 .32741 .2884 .33505 .27829 .32155 Metetra
0 .293 .791 r
.27829 .32155 .2884 .33505 .30335 .34302 .29308 .33152 Metetra
0 .269 .763 r
.29308 .33152 .30335 .34302 .31856 .35061 .30812 .34154 Metetra
0 .282 .743 r
.30812 .34154 .31856 .35061 .33406 .35701 .32348 .35058 Metetra
0 .347 .742 r
.32348 .35058 .33406 .35701 .3499 .36144 .33921 .35755 Metetra
0 .472 .77 r
.33921 .35755 .3499 .36144 .36607 .36321 .35534 .36146 Metetra
.096 .646 .824 r
.35534 .36146 .36607 .36321 .38256 .36182 .37185 .36154 Metetra
.348 .822 .881 r
.37185 .36154 .38256 .36182 .39934 .35699 .38872 .35735 Metetra
.585 .943 .907 r
.38872 .35735 .39934 .35699 .41635 .34873 .40587 .34882 Metetra
.752 .986 .9 r
.40587 .34882 .41635 .34873 .43353 .33727 .42323 .33627 Metetra
.843 .976 .875 r
.42323 .33627 .43353 .33727 .45082 .32309 .44071 .32034 Metetra
.883 .94 .849 r
.44071 .32034 .45082 .32309 .46817 .30681 .45826 .30188 Metetra
.891 .898 .826 r
.45826 .30188 .46817 .30681 .48553 .28911 .4758 .28183 Metetra
.884 .857 .81 r
.4758 .28183 .48553 .28911 .50288 .27067 .4933 .26111 Metetra
.868 .821 .799 r
.4933 .26111 .50288 .27067 .5202 .25211 .51075 .24052 Metetra
.848 .789 .794 r
.51075 .24052 .5202 .25211 .53749 .23396 .52815 .22071 Metetra
.827 .764 .792 r
.52815 .22071 .53749 .23396 .55477 .2166 .54551 .20212 Metetra
.805 .743 .795 r
.54551 .20212 .55477 .2166 .57204 .20031 .56286 .18502 Metetra
.784 .728 .8 r
.56286 .18502 .57204 .20031 .58933 .18521 .58022 .1695 Metetra
.765 .717 .807 r
.58022 .1695 .58933 .18521 .60666 .17136 .59763 .15554 Metetra
.747 .71 .817 r
.59763 .15554 .60666 .17136 .62406 .15872 .6151 .14302 Metetra
.731 .707 .828 r
.6151 .14302 .62406 .15872 .64154 .14719 .63266 .13177 Metetra
.717 .706 .839 r
.63266 .13177 .64154 .14719 .65912 .13665 .65033 .12158 Metetra
.705 .708 .85 r
.65033 .12158 .65912 .13665 .67681 .12694 .6681 .11225 Metetra
.695 .711 .861 r
.6681 .11225 .67681 .12694 .69461 .11792 .686 .1036 Metetra
.687 .715 .87 r
.686 .1036 .69461 .11792 .71254 .10945 .70403 .09546 Metetra
.68 .719 .879 r
.70403 .09546 .71254 .10945 .73059 .10141 .72218 .0877 Metetra
.675 .724 .886 r
.72218 .0877 .73059 .10141 .74877 .09369 .74045 .08019 Metetra
.66 .734 .905 r
.09285 .30729 .1036 .31762 .1181 .31253 .10742 .30194 Metetra
.655 .729 .904 r
.10742 .30194 .1181 .31253 .13266 .30769 .12206 .29671 Metetra
.648 .72 .901 r
.12206 .29671 .13266 .30769 .14725 .30325 .13676 .29171 Metetra
.636 .707 .898 r
.13676 .29171 .14725 .30325 .16187 .29941 .15149 .28714 Metetra
.616 .687 .894 r
.15149 .28714 .16187 .29941 .17648 .29644 .16624 .28324 Metetra
.587 .66 .889 r
.16624 .28324 .17648 .29644 .19108 .29466 .18097 .28037 Metetra
.545 .624 .882 r
.18097 .28037 .19108 .29466 .20564 .29437 .19566 .27892 Metetra
.487 .578 .872 r
.19566 .27892 .20564 .29437 .22017 .29588 .21029 .27936 Metetra
.41 .522 .86 r
.21029 .27936 .22017 .29588 .23466 .29939 .22484 .28209 Metetra
.311 .454 .843 r
.22484 .28209 .23466 .29939 .24914 .30497 .23932 .28742 Metetra
.188 .377 .819 r
.23932 .28742 .24914 .30497 .26367 .31248 .25377 .29542 Metetra
.042 .293 .785 r
.25377 .29542 .26367 .31248 .27829 .32155 .26824 .30588 Metetra
0 .21 .736 r
.26824 .30588 .27829 .32155 .29308 .33152 .28283 .31817 Metetra
0 .142 .673 r
.28283 .31817 .29308 .33152 .30812 .34154 .29764 .33129 Metetra
0 .109 .605 r
.29764 .33129 .30812 .34154 .32348 .35058 .31276 .34391 Metetra
.427 0 0 r
.31276 .34391 .32348 .35058 .33921 .35755 .3283 .35452 Metetra
.365 0 0 r
.3283 .35452 .33921 .35755 .35534 .36146 .34431 .36162 Metetra
.186 0 0 r
.34431 .36162 .35534 .36146 .37185 .36154 .36081 .36398 Metetra
.36081 .36398 .37185 .36154 .38872 .35735 .37775 .36082 Metetra
.447 .873 .766 r
.37775 .36082 .38872 .35735 .40587 .34882 .39507 .35197 Metetra
.713 .985 .82 r
.39507 .35197 .40587 .34882 .42323 .33627 .41265 .33784 Metetra
.856 .995 .826 r
.41265 .33784 .42323 .33627 .44071 .32034 .43038 .31936 Metetra
.909 .954 .812 r
.43038 .31936 .44071 .32034 .45826 .30188 .44816 .29778 Metetra
.916 .901 .796 r
.44816 .29778 .45826 .30188 .4758 .28183 .46592 .27447 Metetra
.902 .851 .783 r
.46592 .27447 .4758 .28183 .4933 .26111 .4836 .2507 Metetra
.878 .807 .775 r
.4836 .2507 .4933 .26111 .51075 .24052 .50119 .22755 Metetra
.852 .771 .772 r
.50119 .22755 .51075 .24052 .52815 .22071 .5187 .2058 Metetra
.825 .743 .774 r
.5187 .2058 .52815 .22071 .54551 .20212 .53615 .18593 Metetra
.799 .722 .779 r
.53615 .18593 .54551 .20212 .56286 .18502 .55357 .16815 Metetra
.775 .707 .787 r
.55357 .16815 .56286 .18502 .58022 .1695 .57101 .15246 Metetra
.754 .698 .798 r
.57101 .15246 .58022 .1695 .59763 .15554 .58849 .1387 Metetra
.735 .694 .811 r
.58849 .1387 .59763 .15554 .6151 .14302 .60604 .12661 Metetra
.719 .694 .825 r
.60604 .12661 .6151 .14302 .63266 .13177 .62368 .1159 Metetra
.705 .697 .84 r
.62368 .1159 .63266 .13177 .65033 .12158 .64144 .10629 Metetra
.694 .702 .853 r
.64144 .10629 .65033 .12158 .6681 .11225 .65931 .0975 Metetra
.685 .708 .866 r
.65931 .0975 .6681 .11225 .686 .1036 .67731 .08932 Metetra
.679 .715 .876 r
.67731 .08932 .686 .1036 .70403 .09546 .69543 .08156 Metetra
.674 .721 .885 r
.69543 .08156 .70403 .09546 .72218 .0877 .71368 .07408 Metetra
.67 .726 .891 r
.71368 .07408 .72218 .0877 .74045 .08019 .73206 .06677 Metetra
.663 .738 .906 r
.08196 .297 .09285 .30729 .10742 .30194 .09659 .29148 Metetra
.661 .736 .905 r
.09659 .29148 .10742 .30194 .12206 .29671 .11131 .28598 Metetra
.658 .731 .903 r
.11131 .28598 .12206 .29671 .13676 .29171 .12609 .28057 Metetra
.651 .722 .9 r
.12609 .28057 .13676 .29171 .15149 .28714 .14094 .27536 Metetra
.639 .707 .896 r
.14094 .27536 .15149 .28714 .16624 .28324 .15582 .27056 Metetra
.619 .684 .889 r
.15582 .27056 .16624 .28324 .18097 .28037 .17072 .26646 Metetra
.586 .651 .881 r
.17072 .26646 .18097 .28037 .19566 .27892 .18558 .26353 Metetra
.537 .605 .869 r
.18558 .26353 .19566 .27892 .21029 .27936 .20038 .26232 Metetra
.467 .546 .854 r
.20038 .26232 .21029 .27936 .22484 .28209 .21507 .26351 Metetra
.37 .471 .833 r
.21507 .26351 .22484 .28209 .23932 .28742 .22963 .26773 Metetra
.24 .379 .803 r
.22963 .26773 .23932 .28742 .25377 .29542 .24406 .27548 Metetra
.074 .27 .758 r
.24406 .27548 .25377 .29542 .26824 .30588 .25839 .28691 Metetra
0 .147 .69 r
.25839 .28691 .26824 .30588 .28283 .31817 .27273 .30164 Metetra
0 .024 .589 r
.27273 .30164 .28283 .31817 .29764 .33129 .2872 .31865 Metetra
0 0 .46 r
.2872 .31865 .29764 .33129 .31276 .34391 .30195 .33628 Metetra
.632 .096 0 r
.30195 .33628 .31276 .34391 .3283 .35452 .31715 .35234 Metetra
.609 .039 0 r
.31715 .35234 .3283 .35452 .34431 .36162 .33292 .3645 Metetra
.465 0 0 r
.33292 .3645 .34431 .36162 .36081 .36398 .34931 .37063 Metetra
.196 0 0 r
.34931 .37063 .36081 .36398 .37775 .36082 .36631 .36929 Metetra
.36631 .36929 .37775 .36082 .39507 .35197 .38382 .36002 Metetra
.603 .908 .638 r
.38382 .36002 .39507 .35197 .41265 .33784 .4017 .34341 Metetra
.85 .991 .731 r
.4017 .34341 .41265 .33784 .43038 .31936 .41977 .32094 Metetra
.936 .96 .755 r
.41977 .32094 .43038 .31936 .44816 .29778 .43788 .29459 Metetra
.941 .896 .754 r
.43788 .29459 .44816 .29778 .46592 .27447 .45591 .26649 Metetra
.916 .834 .749 r
.45591 .26649 .46592 .27447 .4836 .2507 .47381 .23852 Metetra
.882 .782 .746 r
.47381 .23852 .4836 .2507 .50119 .22755 .49156 .21212 Metetra
.847 .742 .747 r
.49156 .21212 .50119 .22755 .5187 .2058 .50918 .18823 Metetra
.814 .712 .752 r
.50918 .18823 .5187 .2058 .53615 .18593 .52671 .16725 Metetra
.784 .691 .762 r
.52671 .16725 .53615 .18593 .55357 .16815 .5442 .14923 Metetra
.758 .679 .775 r
.5442 .14923 .55357 .16815 .57101 .15246 .5617 .1339 Metetra
.736 .675 .791 r
.5617 .1339 .57101 .15246 .58849 .1387 .57925 .12087 Metetra
.717 .676 .809 r
.57925 .12087 .58849 .1387 .60604 .12661 .59689 .10967 Metetra
.702 .682 .827 r
.59689 .10967 .60604 .12661 .62368 .1159 .61462 .09987 Metetra
.691 .691 .845 r
.61462 .09987 .62368 .1159 .64144 .10629 .63247 .09106 Metetra
.682 .701 .861 r
.63247 .09106 .64144 .10629 .65931 .0975 .65045 .08295 Metetra
.675 .71 .874 r
.65045 .08295 .65931 .0975 .67731 .08932 .66854 .07528 Metetra
.671 .718 .884 r
.66854 .07528 .67731 .08932 .69543 .08156 .68676 .06788 Metetra
.668 .725 .892 r
.68676 .06788 .69543 .08156 .71368 .07408 .70511 .06063 Metetra
.666 .73 .897 r
.70511 .06063 .71368 .07408 .73206 .06677 .72358 .05346 Metetra
.664 .74 .907 r
.07094 .28669 .08196 .297 .09659 .29148 .08563 .28107 Metetra
.663 .74 .907 r
.08563 .28107 .09659 .29148 .11131 .28598 .1004 .27543 Metetra
.663 .738 .906 r
.1004 .27543 .11131 .28598 .12609 .28057 .11525 .26978 Metetra
.661 .734 .904 r
.11525 .26978 .12609 .28057 .14094 .27536 .13019 .26417 Metetra
.656 .725 .9 r
.13019 .26417 .14094 .27536 .15582 .27056 .14519 .25868 Metetra
.646 .711 .894 r
.14519 .25868 .15582 .27056 .17072 .26646 .16025 .2535 Metetra
.628 .686 .885 r
.16025 .2535 .17072 .26646 .18558 .26353 .17532 .24897 Metetra
.595 .647 .871 r
.17532 .24897 .18558 .26353 .20038 .26232 .19036 .24564 Metetra
.541 .59 .853 r
.19036 .24564 .20038 .26232 .21507 .26351 .2053 .24433 Metetra
.458 .514 .828 r
.2053 .24433 .21507 .26351 .22963 .26773 .22007 .24606 Metetra
.339 .415 .793 r
.22007 .24606 .22963 .26773 .24406 .27548 .23462 .25195 Metetra
.172 .289 .744 r
.23462 .25195 .24406 .27548 .25839 .28691 .24892 .26297 Metetra
0 .134 .666 r
.24892 .26297 .25839 .28691 .27273 .30164 .26301 .27951 Metetra
0 0 .539 r
.26301 .27951 .27273 .30164 .2872 .31865 .27703 .301 Metetra
.614 .213 0 r
.27703 .301 .2872 .31865 .30195 .33628 .2912 .32565 Metetra
.775 .299 0 r
.2912 .32565 .30195 .33628 .31715 .35234 .30579 .35039 Metetra
.777 .276 0 r
.30579 .35039 .31715 .35234 .33292 .3645 .32104 .37129 Metetra
.657 .162 .063 r
.32104 .37129 .33292 .3645 .34931 .37063 .33714 .38438 Metetra
.436 0 .037 r
.33714 .38438 .34931 .37063 .36631 .36929 .35413 .38661 Metetra
.089 0 0 r
.35413 .38661 .36631 .36929 .38382 .36002 .3719 .37677 Metetra
.3719 .37677 .38382 .36002 .4017 .34341 .39022 .35575 Metetra
.806 .934 .567 r
.39022 .35575 .4017 .34341 .41977 .32094 .4088 .32625 Metetra
.959 .946 .673 r
.4088 .32625 .41977 .32094 .43788 .29459 .42739 .29189 Metetra
.962 .874 .702 r
.42739 .29189 .43788 .29459 .45591 .26649 .44581 .2563 Metetra
.921 .799 .709 r
.44581 .2563 .45591 .26649 .47381 .23852 .46398 .22242 Metetra
.872 .738 .712 r
.46398 .22242 .47381 .23852 .49156 .21212 .4819 .19215 Metetra
.827 .695 .718 r
.4819 .19215 .49156 .21212 .50918 .18823 .49962 .16637 Metetra
.788 .666 .729 r
.49962 .16637 .50918 .18823 .52671 .16725 .51721 .14512 Metetra
.756 .65 .744 r
.51721 .14512 .52671 .16725 .5442 .14923 .53475 .12791 Metetra
.729 .645 .765 r
.53475 .12791 .5442 .14923 .5617 .1339 .55232 .11399 Metetra
.709 .65 .789 r
.55232 .11399 .5617 .1339 .57925 .12087 .56994 .10253 Metetra
.694 .662 .814 r
.56994 .10253 .57925 .12087 .59689 .10967 .58766 .09281 Metetra
.683 .677 .838 r
.58766 .09281 .59689 .10967 .61462 .09987 .60549 .08423 Metetra
.675 .692 .858 r
.60549 .08423 .61462 .09987 .63247 .09106 .62344 .07636 Metetra
.67 .707 .875 r
.62344 .07636 .63247 .09106 .65045 .08295 .64151 .0689 Metetra
.667 .718 .887 r
.64151 .0689 .65045 .08295 .66854 .07528 .6597 .06165 Metetra
.665 .726 .895 r
.6597 .06165 .66854 .07528 .68676 .06788 .67802 .05449 Metetra
.665 .732 .9 r
.67802 .05449 .68676 .06788 .70511 .06063 .69646 .04736 Metetra
.664 .736 .903 r
.69646 .04736 .70511 .06063 .72358 .05346 .71502 .04023 Metetra
.664 .741 .907 r
.0598 .27628 .07094 .28669 .08563 .28107 .07454 .2706 Metetra
.664 .741 .907 r
.07454 .2706 .08563 .28107 .1004 .27543 .08936 .26489 Metetra
.664 .74 .907 r
.08936 .26489 .1004 .27543 .11525 .26978 .10427 .25914 Metetra
.664 .74 .907 r
.10427 .25914 .11525 .26978 .13019 .26417 .11926 .25336 Metetra
.663 .737 .905 r
.11926 .25336 .13019 .26417 .14519 .25868 .13434 .24757 Metetra
.661 .732 .902 r
.13434 .24757 .14519 .25868 .16025 .2535 .14951 .24179 Metetra
.656 .72 .896 r
.14951 .24179 .16025 .2535 .17532 .24897 .16475 .23614 Metetra
.644 .697 .884 r
.16475 .23614 .17532 .24897 .19036 .24564 .18003 .23087 Metetra
.617 .655 .865 r
.18003 .23087 .19036 .24564 .2053 .24433 .19531 .22654 Metetra
.567 .59 .836 r
.19531 .22654 .2053 .24433 .22007 .24606 .21049 .22419 Metetra
.48 .497 .797 r
.21049 .22419 .22007 .24606 .23462 .25195 .22544 .22544 Metetra
.345 .372 .745 r
.22544 .22544 .23462 .25195 .24892 .26297 .24002 .23248 Metetra
.143 .209 .672 r
.24002 .23248 .24892 .26297 .26301 .27951 .25411 .24758 Metetra
0 0 .553 r
.25411 .24758 .26301 .27951 .27703 .301 .26771 .27222 Metetra
0 0 .344 r
.26771 .27222 .27703 .301 .2912 .32565 .28103 .30591 Metetra
.83 .445 0 r
.28103 .30591 .2912 .32565 .30579 .35039 .29446 .34517 Metetra
.888 .462 .159 r
.29446 .34517 .30579 .35039 .32104 .37129 .30854 .3833 Metetra
.751 .325 .238 r
.30854 .3833 .32104 .37129 .33714 .38438 .32407 .4074 Metetra
.29 0 .087 r
.32407 .4074 .33714 .38438 .35413 .38661 .34175 .40136 Metetra
.17 0 0 r
.34175 .40136 .35413 .38661 .3719 .37677 .35955 .39528 Metetra
.35955 .39528 .3719 .37677 .39022 .35575 .37785 .38102 Metetra
.725 .815 .351 r
.37785 .38102 .39022 .35575 .4088 .32625 .39738 .33674 Metetra
.979 .906 .578 r
.39738 .33674 .4088 .32625 .42739 .29189 .4168 .28696 Metetra
.967 .82 .647 r
.4168 .28696 .42739 .29189 .44581 .2563 .43579 .23887 Metetra
.898 .729 .664 r
.43579 .23887 .44581 .2563 .46398 .22242 .45428 .19711 Metetra
.831 .663 .674 r
.45428 .19711 .46398 .22242 .4819 .19215 .47233 .16357 Metetra
.777 .621 .686 r
.47233 .16357 .4819 .19215 .49962 .16637 .49008 .13803 Metetra
.736 .6 .706 r
.49008 .13803 .49962 .16637 .51721 .14512 .50768 .11909 Metetra
.706 .598 .733 r
.50768 .11909 .51721 .14512 .53475 .12791 .52525 .10499 Metetra
.687 .612 .766 r
.52525 .10499 .53475 .12791 .55232 .11399 .54287 .09408 Metetra
.675 .636 .802 r
.54287 .09408 .55232 .11399 .56994 .10253 .56057 .08509 Metetra
.669 .664 .836 r
.56057 .08509 .56994 .10253 .58766 .09281 .57838 .07716 Metetra
.666 .689 .862 r
.57838 .07716 .58766 .09281 .60549 .08423 .5963 .06976 Metetra
.664 .709 .881 r
.5963 .06976 .60549 .08423 .62344 .07636 .61435 .06259 Metetra
.664 .723 .893 r
.61435 .06259 .62344 .07636 .64151 .0689 .63251 .05549 Metetra
.664 .731 .9 r
.63251 .05549 .64151 .0689 .6597 .06165 .65079 .04841 Metetra
.664 .736 .904 r
.65079 .04841 .6597 .06165 .67802 .05449 .66919 .0413 Metetra
.664 .738 .906 r
.66919 .0413 .67802 .05449 .69646 .04736 .68772 .03416 Metetra
.664 .74 .906 r
.68772 .03416 .69646 .04736 .71502 .04023 .70636 .02696 Metetra
.664 .741 .907 r
.04854 .26577 .0598 .27628 .07454 .2706 .06332 .26002 Metetra
.664 .741 .907 r
.06332 .26002 .07454 .2706 .08936 .26489 .07819 .25425 Metetra
.664 .741 .907 r
.07819 .25425 .08936 .26489 .10427 .25914 .09315 .24843 Metetra
.664 .741 .907 r
.09315 .24843 .10427 .25914 .11926 .25336 .1082 .24259 Metetra
.664 .741 .907 r
.1082 .24259 .11926 .25336 .13434 .24757 .12333 .23671 Metetra
.664 .74 .907 r
.12333 .23671 .13434 .24757 .14951 .24179 .13856 .23079 Metetra
.664 .739 .906 r
.13856 .23079 .14951 .24179 .16475 .23614 .15387 .22484 Metetra
.663 .734 .902 r
.15387 .22484 .16475 .23614 .18003 .23087 .16928 .21886 Metetra
.66 .721 .894 r
.16928 .21886 .18003 .23087 .19531 .22654 .18478 .2129 Metetra
.651 .69 .873 r
.18478 .2129 .19531 .22654 .21049 .22419 .20034 .20712 Metetra
.625 .629 .835 r
.20034 .20712 .21049 .22419 .22544 .22544 .21592 .20214 Metetra
.563 .531 .777 r
.21592 .20214 .22544 .22544 .24002 .23248 .23136 .19966 Metetra
.45 .397 .707 r
.23136 .19966 .24002 .23248 .25411 .24758 .24635 .20345 Metetra
.258 .218 .621 r
.24635 .20345 .25411 .24758 .26771 .27222 .26044 .21982 Metetra
0 0 .485 r
.26044 .21982 .26771 .27222 .28103 .30591 .27323 .25624 Metetra
0 0 .204 r
.27323 .25624 .28103 .30591 .29446 .34517 .28472 .31673 Metetra
.939 .61 .172 r
.28472 .31673 .29446 .34517 .30854 .3833 .29574 .39505 Metetra
.379 0 0 r
.29574 .39505 .30854 .3833 .32407 .4074 .31286 .39622 Metetra
.645 .763 .934 r
.31286 .39622 .32407 .4074 .34175 .40136 .33062 .3901 Metetra
.645 .763 .934 r
.33062 .3901 .34175 .40136 .35955 .39528 .34849 .38395 Metetra
.645 .763 .934 r
.34849 .38395 .35955 .39528 .37785 .38102 .36648 .37775 Metetra
.939 .892 .756 r
.36648 .37775 .37785 .38102 .39738 .33674 .38562 .34836 Metetra
1 .829 .524 r
.38562 .34836 .39738 .33674 .4168 .28696 .40676 .26284 Metetra
.908 .694 .603 r
.40676 .26284 .4168 .28696 .43579 .23887 .42652 .19564 Metetra
.797 .588 .619 r
.42652 .19564 .43579 .23887 .45428 .19711 .44513 .15 Metetra
.719 .529 .633 r
.44513 .15 .45428 .19711 .47233 .16357 .46302 .12173 Metetra
.671 .51 .659 r
.46302 .12173 .47233 .16357 .49008 .13803 .48061 .1046 Metetra
.649 .526 .7 r
.48061 .1046 .49008 .13803 .50768 .11909 .49813 .09344 Metetra
.645 .569 .754 r
.49813 .09344 .50768 .11909 .52525 .10499 .5157 .08499 Metetra
.65 .624 .81 r
.5157 .08499 .52525 .10499 .54287 .09408 .53337 .07759 Metetra
.657 .673 .854 r
.53337 .07759 .54287 .09408 .56057 .08509 .55114 .07053 Metetra
.661 .706 .881 r
.55114 .07053 .56057 .08509 .57838 .07716 .56904 .06353 Metetra
.662 .725 .895 r
.56904 .06353 .57838 .07716 .5963 .06976 .58704 .05653 Metetra
.663 .734 .902 r
.58704 .05653 .5963 .06976 .61435 .06259 .60517 .04948 Metetra
.664 .738 .905 r
.60517 .04948 .61435 .06259 .63251 .05549 .62342 .04239 Metetra
.664 .74 .907 r
.62342 .04239 .63251 .05549 .65079 .04841 .64178 .03526 Metetra
.664 .74 .907 r
.64178 .03526 .65079 .04841 .66919 .0413 .66027 .02807 Metetra
.664 .741 .907 r
.66027 .02807 .66919 .0413 .68772 .03416 .67887 .02084 Metetra
.664 .741 .907 r
.67887 .02084 .68772 .03416 .70636 .02696 .6976 .01356 Metetra
.664 .741 .907 r
.03716 .25514 .04854 .26577 .06332 .26002 .05199 .24933 Metetra
.664 .741 .907 r
.05199 .24933 .06332 .26002 .07819 .25425 .06691 .24349 Metetra
.664 .741 .907 r
.06691 .24349 .07819 .25425 .09315 .24843 .08191 .23761 Metetra
.664 .741 .907 r
.08191 .23761 .09315 .24843 .1082 .24259 .09701 .2317 Metetra
.664 .741 .907 r
.09701 .2317 .1082 .24259 .12333 .23671 .11219 .22576 Metetra
.664 .741 .907 r
.11219 .22576 .12333 .23671 .13856 .23079 .12747 .21978 Metetra
.664 .741 .907 r
.12747 .21978 .13856 .23079 .15387 .22484 .14284 .21376 Metetra
.664 .741 .907 r
.14284 .21376 .15387 .22484 .16928 .21886 .1583 .2077 Metetra
.664 .741 .907 r
.1583 .2077 .16928 .21886 .18478 .2129 .17385 .20161 Metetra
.664 .739 .906 r
.17385 .20161 .18478 .2129 .20034 .20712 .1895 .19548 Metetra
.663 .734 .902 r
.1895 .19548 .20034 .20712 .21592 .20214 .20525 .18932 Metetra
.661 .712 .886 r
.20525 .18932 .21592 .20214 .23136 .19966 .22109 .18312 Metetra
.649 .651 .838 r
.22109 .18312 .23136 .19966 .24635 .20345 .23703 .17687 Metetra
.617 .538 .743 r
.23703 .17687 .24635 .20345 .26044 .21982 .25307 .1706 Metetra
.573 .424 .641 r
.25307 .1706 .26044 .21982 .27323 .25624 .2692 .16437 Metetra
.524 .338 .571 r
.2692 .16437 .27323 .25624 .28472 .31673 .28515 .16146 Metetra
.323 .159 .497 r
.28515 .16146 .28472 .31673 .29574 .39505 .29778 .20482 Metetra
0 0 .233 r
.29778 .20482 .29574 .39505 .31286 .39622 .30151 .3849 Metetra
.645 .763 .934 r
.30151 .3849 .31286 .39622 .33062 .3901 .31934 .3787 Metetra
.645 .763 .934 r
.31934 .3787 .33062 .3901 .34849 .38395 .33729 .37247 Metetra
.645 .763 .934 r
.33729 .37247 .34849 .38395 .36648 .37775 .35535 .36619 Metetra
.966 .715 .506 r
.35535 .36619 .36648 .37775 .38562 .34836 .38237 .17424 Metetra
.64 .403 .543 r
.38237 .17424 .38562 .34836 .40676 .26284 .40178 .11665 Metetra
.54 .33 .543 r
.40178 .11665 .40676 .26284 .42652 .19564 .41903 .10576 Metetra
.544 .356 .576 r
.41903 .10576 .42652 .19564 .44513 .15 .43621 .09888 Metetra
.573 .422 .639 r
.43621 .09888 .44513 .15 .46302 .12173 .4535 .09211 Metetra
.612 .523 .73 r
.4535 .09211 .46302 .12173 .48061 .1046 .4709 .0853 Metetra
.644 .628 .82 r
.4709 .0853 .48061 .1046 .49813 .09344 .48841 .07844 Metetra
.658 .696 .874 r
.48841 .07844 .49813 .09344 .5157 .08499 .50604 .07154 Metetra
.662 .726 .897 r
.50604 .07154 .5157 .08499 .53337 .07759 .52377 .06459 Metetra
.664 .737 .904 r
.52377 .06459 .53337 .07759 .55114 .07053 .54163 .0576 Metetra
.664 .74 .907 r
.54163 .0576 .55114 .07053 .56904 .06353 .5596 .05057 Metetra
.664 .741 .907 r
.5596 .05057 .56904 .06353 .58704 .05653 .57768 .04348 Metetra
.664 .741 .907 r
.57768 .04348 .58704 .05653 .60517 .04948 .59589 .03636 Metetra
.664 .741 .907 r
.59589 .03636 .60517 .04948 .62342 .04239 .61421 .02918 Metetra
.664 .741 .907 r
.61421 .02918 .62342 .04239 .64178 .03526 .63266 .02196 Metetra
.664 .741 .907 r
.63266 .02196 .64178 .03526 .66027 .02807 .65123 .01469 Metetra
.664 .741 .907 r
.65123 .01469 .66027 .02807 .67887 .02084 .66992 .00737 Metetra
.664 .741 .907 r
.66992 .00737 .67887 .02084 .6976 .01356 .68874 0 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(50)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(55)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(65)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(70)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{150.75, 122.25},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002l0000V1000`40O003h00O_7oo002lOol00;ao
o`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo002lOol00;ao
o`00_7oo001aOol00`00Oomoo`04Ool30011Ool0075oo`03001oogoo00=oo`05001oogooOol00010
Ool0079oo`03001oogoo009oo`05001oogooOol00010Ool0079oo`03001oogoo009oo`05001oogoo
Ool00010Ool0079oo`03001oogoo009oo`05001oogooOol00010Ool007=oo`05001oogooOol00003
Ool00`00Oomoo`0nOol006moo`05001oogooOol00003Ool01@00Oomoogoo0000@7oo001_Ool50004
Ool30011Ool00;aoo`00_7oo001GOol30005Ool4000HOol2000oOol005Ioo`05001oogooOol00008
Ool00`00Oomoo`0BOol300000eKm0000000nOol005Ioo`05001oogooOol00008Ool00`00Oomoo`0@
Ool300000eKm001Fo@02000nOol005Ioo`05001oogooOol00008Ool00`00Oomoo`0=Ool3000015Km
000005Km0`0000=Fo@00Ool0?7oo001FOol40005Ool4000>Ool300000eKm001Fo@03000015Km0000
05Km0P00?7oo001FOol00`00Oomoo`06Ool00`00Oomoo`0<Ool3000015Km000005Km0P000UKm00<0
05Km00000P0000=Fo000Ool0>goo001GOol00`00Oomoo`05Ool00`00Oomoo`09Ool400000eKm001F
o@03000015Km001FoEKm0P0000AFo000E_aFo08003]oo`00F7oo0`0017oo10001Woo0`0000AFo@00
001Fo@80009Fo003001Fo@0000800003E_`005Kl00<00003E_`0000003Yoo`00IWoo100000=Fo@00
E_d00`0000AFo000E_aFo0800004E_`005KlE_`40002E_`2000jOol003moo`<000Eoo`<001Yoo`<0
0004E_`00000E_d20002E_`00`00E_`0000200000eKl001Fo00300000eKl001Fo00300000eKl0000
000iOol003ioo`05001oogooOol00003Ool01@00Oomoogoo00005Woo100000=Fo000E_`00`0000AF
g000E_aFo0800004E]`005KlE_`2000015KL001Fo5Kl0`0000=Fo000E_`00P00>Goo000nOol01@00
Oomoogoo00000goo00D007ooOomoo`0001Aoo`<0009Ff`04001Fo0000002E[/00`00E]`000020000
0eKK001Fg00300000eKK001Fg003000015KL000005Kl0P0000=Fo0000000>7oo000nOol01@00Oomo
ogoo00000goo00D007ooOomoo`00015oo`@00004DYX005KKE]/20000159i001F^eJk0P0000AF^P00
E]]Ff`80009Ff`04001Ff`000002E]/00`00E]`00002000015Kl000005Kl0P001Woo0`001Woo0P00
1Goo0`0017oo10005Woo000nOol40004Ool01@00Oomoogoo00003goo0`0000=>5`00DYX00U:J0P00
00ABF000DWUBN@800003EYT005Jj00<00004E[X005KKE]/2000015KK001FfeKK0P0000MFg0000000
05Kl001Fo0020005Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`0000Moo`03001oogoo
01=oo`00?Woo00<007ooOol01Goo00D007ooOomoo`0000aoo`H0009>5`04001BVP000003DUP30003
EYT01@00F[YJ^P00E[X00`0000=Jf`00E]/00`0000MFf`00E]`005KK001Fo0020002E_`00`00Oomo
o`02Ool01@00Oomoogoo00002goo00D007ooOomoo`0000Moo`03001oogoo01=oo`00?goo00<007oo
Ool017oo00D007ooOomoo`0000Yoo`@00004AG8004Vd0002CQL20000158F001BF59H0P000UIh00@0
05JI0000009J^P800004F[X005[KF]/2000015[K001FfeKK0`0000EFf`00001Fn`00009Fo08000=o
o`05001oogooOol0000;Ool01@00Oomoogoo00001goo00<007ooOol04goo0010Ool30004Ool30008
Ool60002AG800`00BKA9]002000015HE001B5U8F0P0000AJM`00EWQFN080009JV@800004F[T005Zj
F[X200000e[J001Jf`0300001e[K00000000F]/005Kk00<00003E_`007oo009oo`05001oogooOol0
000;Ool01@00Oomoogoo00000goo10005Woo000YOol40004Ool4000JOol30003Ool00`00AE400002
AG801000BKA9]0000UHE00@0058FDQH000=JM`800005GYP005ZIFYT00003F[T200000ekJ001JfP02
0002F]/01000F]/000000U[K0P0000AJn`00E_aFo080009oo`05001oogooOol0000;Ool01@00Oomo
ogoo00000goo00<007ooOol05goo000]Ool00`00Oomoo`05Ool00`00Oomoo`0EOol20003Ool01P00
Ool00000AE4000=5LP030019]00000=F5@<000=JM`04001NV5jHGYP200000eji001J^@020002G]X2
000015[J001Jfe[K0P0000MJf`00000005[k001Jn`0300001E[l000007oo00000goo00<007ooOol0
2Goo00D007ooOomoo`0000=oo`03001oogoo01Moo`00;Goo00<007ooOol01Goo00<007ooOol04Woo
10001Woo0P0000=5D@00AG800TEb0P0000EN5000EQEF5@00009NMP04001JM`000002GYP20002G[T2
00001UkI001NfP00G]X0009JfP800003F]/005[K0080009Jn`800004F_/005[lF_`20002Ool3000=
Ool30004Ool4000FOol002eoo`03001oogoo00Eoo`03001oogoo011oo`8000=oo`03001oogoo00Ao
o`800003AE4004Eb0095LP800005GQ@005HEEQD00003GWH00`00HYMRU`0200001V:h001N^@00H]T0
009Nf@04001NfUkJG]X30002F]/300001U[k000005[k001Jn`<00003F_`00000039oo`00:Goo1000
17oo100047oo1@002Goo00D0056A0015D@0000=5LP05001N55hD001F5@020002GWH20002HYL01@00
H[QR^6:h00000f;I0`000UkJ0P000UkJ0P0000MNn`00000005[k001Jn`02000015[k001Jo5[l0`00
<Goo000YOol00`00Oomoo`05Ool00`00Oomoo`0?Ool2000>Ool00`00DI400002AE401000AG95LP00
0ehD0P0000=RM@0000000VJg0P0000=Vf000H[P00P000V;I0P000UkJ0P0000MNfP00G]X005kj001N
n`020002F_/200000e[k001Jn`0300000e[l0000000aOol002Uoo`03001oogoo00Eoo`03001oogoo
00aoo`H000eoo`03001AT@000095D@040015LP000003GQ@01000HWERM@000fJg00D006KHI]QVf000
009Rf@800007H]T005kJ001RnP00G]X00P000Ukj0P0000MNn`00F_/005kk001Jn`020002F_/20000
0e[l0000000`Ool002Uoo`@000Aoo`@000Uoo`@00004BK@0045B000<Ool01000DI5AT@000TEA00@0
04Eb000000=N5004001RMF9e0003I[L00`00I]QVf0020002H]T01@00H]URfF;I00000V;j0P0000MN
nP00G_X005kk001Nn`020002G_/200001e[k001Jn`00F_/005[l00<002moo`00>goo1@0000E>=`00
BK@0045B008000]oo`04001ATE6A0002AE420002IU<02`00GQ@00000HWD006[F001V]`00J]P00P00
0VKh0P0000=Vn@00H]T00P0000ERnP00H_X005kj0080009Nn`<00006G_/00000G_/005[k0P000U[k
0P0000=Jo0000000;goo000DOol40004Ool3000JOol2000015KL001BVP000Thg00@004Vd0011DP80
00]oo`04001ATE6A0002AE420002IU<20002J[D20002J]H01000J]QZf0000fKh00<006KiI_T00P00
0V;j00<006;jH_X00P000Ukj0P0000=Nn`00G_/00P000Ukk0P000U[k0P0000=Jn`00F_`00`00;Woo
000HOol01000Oomoo`000goo00<007ooOol057oo0`0000aFo000E]aFg000DYX004hg0019]000@E82
000;Ool01`00DI5ATE6A0015D@0000=VD`05001Z]FZeJ[D00002J]H20002J]P2000026Kh000006Ki
001Vn@00H_X200002F;j001RnP00G_X005kk001Nn`0300001Ukk000005kk001Jn`80009Jn`800004
F_`00000000]Ool001Qoo`04001oogoo0003Ool00`00Oomoo`0BOol300000eKl001Fo00700001dhg
0000045B000X[`0000Yoo`07001ATE6ADI4004EA00000fIC00L006ZeJ[EZ]@00J]H00002K_L20002
J_P01000I_UVn@000fKi00@006;jH_X000=RnP03001Nnekk00800007G_/00000001Nn`00G_/00P00
00MJn`00000005[k001Jo0030006Ool30006Ool20005Ool5000BOol001Qoo`04001oogoo0003Ool0
0`00Oomoo`0?Ool400000eKm001Fo002000024jJ001:64XH000iUP00CSL20000145B000X[`002Goo
0P000e6A00D004EA000006IC0080009Z]@03001^mP0000=^m`03001Zn6[h00800005I_T006LI001V
n@0200001F;j001S6P00H_X00P0000ENn`00H_/005kk0080009Nn`80009Nn`03001Jn`00009Jn`80
0004F_`000000004Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol04Woo000DOol40003Ool0
1@00Oomoogoo00003Woo1@0000=Fo@0000000`0000MBf`00CYX004XH000iUP040000145B000X[`00
2Goo0P000e6A0P0000Efd`00IU<007;e0080009^mP07001^mfkgK_L006[h00000V/I00<006LIIaT0
0P000VLJ00<006<JHaX00P0000=RnP0000000V;k0P0000MNn`00000005kk001Nn`0200001E[k0000
0000F_`00P0000AJo000000000=oo`05001oogooOol0000>Ool00`00Oomoo`0BOol001Aoo`03001o
ogoo00Aoo`05001oogooOol0000<Ool500000eKm00000002E_`00`00E_`0000800001b3C0000045B
000X[`0000Uoo`8000=AT@800006M]<006IC001bm@000Vkf00H007<g001^m`00KaP200002V/I0000
06LI001W6P00IaX006LJ0P0000YS6P00001S6`00H_/005lK001Nn`80009Nn`80000:G_/00000G_/0
05[l001Jo000F_`30003Ool01@00Oomoogoo00003Woo00<007ooOol04Woo000DOol00`00Oomoo`04
Ool01@00Oomoogoo00002Goo0`000UKm00H005Km000005Kl0002E_`200002DjK001:FP00?OT002eg
000iUP030000145B000X[`002Goo0P000e6A0P000WKC00<007;eL_D00P0000M^mP00LcMc=`00KaP0
0002KcT01000JaU[6@000VLJ0P0000QW6P00001S6P00Ha/006<K0P000UlK1P0000INn`00001Nn`00
G_/300001e[l000005[l001oo`00009oo`05001oogooOol0000>Ool00`00Oomoo`0BOol001Aoo`@0
00Aoo`<000Qoo`H00006E_d00000E_d005Kl0`0000IBg000CY]>V`00BUX200000beg000000030000
145B000X[`002Goo0P000e6A0P000WKC00<007;e00000WLf00H007<gLcL007<h0002KcT200001F/I
000006LJ00000VLJ00@006LJIaX000=S6`03001S6f<K0080009O6`08001O6`00G_/005lK001Nn`80
009Nn`800003F_`005[l00<0009oo`05001oogooOol0000>Ool00`00Oomoo`0BOol002Aoo`D00003
E_d0000000800003E_d0000000<00003D_`005;L00T000084=D000000011DP00::l000Uoo`8000=A
T@80009fd`03001bm@00009g=P04001c=`00LcP200000fli001[>@020002JcX2000026LJ000006LJ
001S6`00Ha/200001F<K001S6`00Ga/00P0000=O6`0000000UlK0P000UlK0P0000=No000F_`00P00
00IJo000000007oo0003Ool00`00Oomoo`0<Ool00`00Oomoo`0BOol002=oo`@00006E_d00000E_d0
05Km0P000UKl00<005Kl00000`0000M>V`00AU/003XJ000Y^@0400001R3C0011DP00::l000Uoo`80
00=AT@05001fdgKC001kE00200001WLf001gE`00LcP0009_F@08001[>@00JcX006/j001W>P80009W
>P03001S6f<K00800004Ha/00000Ha/300000elK00000002Ga/200000elK001O6`020002G_`20000
25[l000005[l001oo`00Ool3000=Ool3000DOol001aoo`<000Aoo`800006E_d00000E_d005Km0`00
00=Fo@00E_`00P0000EBg000D[`004jK00L000035CL0000000800004@E8002R_0009Ool20003DI40
2000M]=fd`00Ne@007Lf0002MeL3000016mI001_FP000f/j00H006Lj001W>P00IcX2000016<K001S
6f<K1@0000=S6`00Ga/00P0000EO6`00Ga/005lK00800005Ga/00000001O700200000e[l001Jo003
000XOol001]oo`05001oogooOol00003Ool01000Ool005Km1@0000=Fo@0000000P0000=Bo0000000
100000E6N`00>S/002gj00@000074=D00000@E8002R_00002Goo00<007[@00000U6A00<007KC0000
0W]D0`0000IgE`00LgT007=i0002KeX2000016/j001[>P000fLj00<006Lk00001@0000=S6`000000
0V<K0P0000=O6`0000000UlK0P0000YO6`00001O7000Ga`005/L001Jo0800004F_`00000000WOol0
01]oo`05001oogooOol00002Ool20002Ool01@00E_eFo@00E_d00P0000=Fo000E_`00P000U;L00P0
04jl001:W000AW/003Xk0P0000<MV@0000000`0000A1DP00::l000Qoo`04001jd7[@0002DI403000
M]<007]D001kMP00NiP007=i001cN@800003KeX006]J00800007JcX006/k001W>P00Ic/01@000V<K
00<006<K00000P000V<K00<005lKGa/00P0000IO6`00001O6`00Ga`20002Fa`200001e/L000005[l
001oo`0002Ioo`006goo00D007ooOomoo`00009oo`03001oogoo009oo`07001Fo@00001Fo@00E_`0
0`0000=Bo000D]`04P0000<X[`00Ool01goo00L007[@N]00056A001oD003000027]f001kV000MiT0
07=i0002KgX00`00JeX00002Je/00`00Jc][>`05000026Lk001S>`00Ha/006<K0002Ha/01@00Ha/0
0000Ga/00P0000EO6`00Ga/005lK00800005Ga`00000001K700200000e/L001Jo003000VOol001]o
o`05001oogooOol00002Ool20004Ool00`00E_d0000400000eKl0000000200002djl001:_000AW`0
03iK000f6`009MX01`0000<X[`00Ool01goo00L007[@N]00056A001oD00300000g]f00000002MiT3
00000fmj001_N`02000016]K000006/k100000=W>`0000000V<k0P0000=S6`0000000V<K0P0000AO
6`00001O6`80009O6`80009O7008001O7000Fa`005/L001K70800004Fa`00000000UOol001]oo`05
001oogooOol00002Ool00`00Oomoo`04Ool200000eKl001Fo00200000e;l001Bg00=00000aVJ0000
000300000bR_001oo`07Ool01`00N]1jd000DI4007m@00800005NmD00000001gV@0200001VnK0000
06mk001[N`D00005Ic/006LkIc/00002Ic/20002Hc/00`00Hc/00002000016<K000005lK0P000UlK
0P0000=O7000Ga`00P0000=O700000000U/L0P0000AK7000001K70<002Aoo`006goo00D007ooOomo
o`00009oo`03001oogoo00Ioo`<00003D_`005;l0080000=C]`004Zl0016W000@W`003YL000^6`00
8M/00P0000<5>@0000000`0027oo00D007[@N]00056A00<00003NmD007OH00@00006Ki/006nKKi/0
06]k10000VMK0P0000]W>`00001W>`00Hc/006<k001S>`00009S>`800006Ha/00000Ga/005lK0P00
0UlL0P0000AO7000001O70<00006Fa`00000Fa`005/L0`001Woo0`001Woo0P001Goo1@000Woo1000
0goo000LOol30003Ool20007Ool00`00E_`0000300000e;L001>g00=00000aFJ000000040008Ool0
0`00N]1jd00200001Gm@00000000MmP00P000W?K0`0000=_V`0000000P0000I[N`00001WF`00Ie/2
00000fLk00000002Hc/2000016<k000006<k0P0000ES6`00Ga/005lK00800005Ga`00000001O7002
00002UlL000005lL001K7000Fa`005/L0P0000AK7000000000Aoo`05001oogooOol00005Ool20007
Ool00`00Oomoo`06Ool10001Ool1Ool000=oo`<000Ioo`8000Eoo`<000Aoo`@000Aoo`03001oogoo
00Moo`<00003D_`0000000<00009C[`004Zl0012W000?W`003Hl00D000031EX00000008000Qoo`03
001jd7[@00<00003Oi80000000<00003Lm/0000000<00003Jg/006]l00800005Ie/00000Ie/00002
Ie`00`00Hc]S>`02000016<k000006<k0P0000ES>`00000005lK00800004Ga`00000Ga`300001UlL
000005lL001O70800005Fa`00000001K7002000015/L000000000goo00D007ooOomoo`0000ioo`03
001oogoo00Ioo`40005oo`5oo`000Woo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol00007
Ool00`00Oomoo`02000;Ool300000e;L001>g00=00000b7k000000040008Ool00`00N]1jd0070000
0f_k000000030002Ji`00`00Jg`00002Ig`00`00Ie`0000200001VML000006<k001S?080009S?004
001S?6<l0002Hc`20002Ga`01@00GaaO7000Ga`00`0000=O7000Ga`00P0000AO7000001K70<00003
Fa`005/L00<000=oo`05001oogooOol0000>Ool00`00Oomoo`06Ool10001Ool1Ool0009oo`05001o
ogooOol0000;Ool01@00Oomoogoo00001goo00<007ooOol00P002goo00<005;l00000`0000E>g000
B[`004JL00D00003<S`002XL00L000Qoo`03001jd7[@00H00005Jo/00000001W_@0200000f^L001W
W002000016Ml001WO0000VML0P0000QSG000001S?000Hc`006<l0P0000QS?000001S?000Gc`005lL
0P0000AO7000001O7080009O70800003Ga`00000009O7003001K7000009K70800004Fa`000000002
Ool01@00Oomoogoo00003Woo00<007ooOol00Woo10000goo0002Ool01@00Oomoogoo00002goo00D0
07ooOomoo`0000Moo`03001oogoo008000aoo`04001Bg000C]`7000014:L000003YL1P0000<Ig000
00000`001Goo00@007[@N]1jd0T00003Ikd006Nm00800003Ii`006Ml00800006Ig`00000Ie`006=L
0P0000=SG00000000V<l0P0000AS?000001S?080009O?0800004Ga`00000Ga`200001ElL00000000
Ga`00P0000=O7000Ga`00P0000EK7000000005/L00800006Fa`00000001oo`000goo00<007ooOol0
37oo00<007ooOol00Woo00<007ooOol017oo0002Ool01@00Oomoogoo00002goo00D007ooOomoo`00
00=oo`@000=oo`03001oogoo00aoo`@00005C]`004Zl001:_00800000bXl000V70080003Ool01000
N]1jd7[@1`0000=S_P0000000P0000=SW@0000000P0000ASO@00001SG0800005He`006=LHe`00002
Hc`2000016<l000006<l0P0000ES?000000005lL00800004Ga`00000Ga`20002Ga`200000elL001O
7002000015/L000005/L0`0000=K7000Fa`00`0000=oo`00Ool00Woo00<007ooOol037oo00<007oo
Ool00Woo00<007ooOol017oo0002Ool01@00Oomoogoo00002goo00D007ooOomoo`0000=oo`03001o
ogoo00Aoo`03001oogoo00eoo`03001Bg00000D00007A[d004:M000nO@00>Wd0400000=jd0000000
0`0000=CgP00Ekl00P0000=S_P00Gih00P0000=SO@00Hgd00P0000ASO@00001SG0<00006He`00000
Hc`006<l0P000V<l00@006<lHc`0009O?0800003Ga`00000009O7004001O7000Ga`3000015lL0000
05lL0P0000IK7000001K7000Fa`300001E/L00000000Ool00`003Goo0`0017oo10000goo0002Ool0
1@00Oomoogoo00002goo00D007ooOomoo`0000=oo`03001oogoo00Aoo`03001oogoo00ioo`04001>
g000C]`=00000bhm0000000200000g75001Q000600000g[@0000000500000e^O0000000200001UnN
000006=m001SO@800003Hed00000009SG0800008He`00000Hc`006<l001S?0800004Hc`00000Gc`2
00001ElL001O7000Ga`00P0000AO7000001O70<00004Ga`00000Ga`200000e/L001K700200001E/L
00000000Fa`00`007Woo0003Ool30003Ool30006Ool40003Ool50002Ool2000AOol300003DZm001:
_@00AYd004:M000nO@00=Wd0039M00D0009aa@H00003N]0007^H00P0009KOP800005Ggh00000Hgd0
0002Ged00`00Hed00002000016=L000006=L0P000V<l0P0000AS?000001O?0800005Gc`00000001O
7002000015lL000005lL0P0000EO7000000005lL00800003Ga`005/L00800004Fa`00000Fa`30000
0e/L0000000NOol000Qoo`05001oogooOol00005Ool20007Ool00`00Oomoo`02Ool2000BOol00`00
C]d0000D0002Dad00`00Dad00006000014MO000005=o0`0000AKOP00001OOP<00004Ged005mM0002
Ged2000016=L000006<l0`0000AS?000001O?0800003Gc`00000009O?0800004Ga`00000Ga`20000
0elL00000002Ga`200000elL001O7002000015/L000005/L0P0000=K7000Fa`010007Goo0008Ool0
1@00Oomoogoo00003Woo00<007ooOol00Woo00<007ooOol04Woo00@004kM001:_A000004Dad005<M
Dad20002Dad90002Dgl200000e]n00000002Geh200000emM001OG@020002Ged01@00Ge`00000Gc`0
0`0000AO?000001O?0800005Gc`00000001O?00200000elL00000002Ga`01000Ga`005lL0`0000AO
7000001K70800006Fa`00000Fa`005/L0P0000EK7000001oo`0001aoo`0027oo00D007ooOomoo`00
00ioo`03001oogoo009oo`03001oogoo01=oo`<00009B[d004Jm0012W@00?Yd003ZM00D0000300L0
05<M00<00003Dad005<M00H00003Acl0000000<00004Eel00000Feh300002emM000005mM001OG@00
Ged005mM00000Ull00<005llGc`00P0000AO?000001O?080009O?0800006Gc`00000Ga`005lL0P00
00AO7000001O70H00006Fa`00000Fa`005/L0P0000AK7000001K70@000Eoo`<000Ioo`8000Aoo`D0
009oo`0027oo00D007ooOomoo`0000ioo`03001oogoo009oo`03001oogoo01=oo`03001:g@000140
009C7@/00097?`L0009KGP800003Fed00000009OG@800004Ged00000Gc`200000ell001O?0020002
Gc`01000Gc`005ll0P0000EO?000Ga`005lL00800004Ga`00000Ga`2000015lL000005/L0`0000=K
7000Fa`00P0000AK7000001K70800004Fa`000000004Ool01@00Oomoogoo00001Goo0P001Goo00@0
07ooOol0009oo`0027oo00D007ooOomoo`0000ioo`03001oogoo009oo`03001oogoo01Aoo`<00003
B[d004Jm00T00003=Wd003Im01000004Bcl00000Dch5000015]N000005]M0`0000AOG@00001O?@80
0008Gc`00000Gc`005ll001O?0800004Gc`00000Gc`20002Ga`2000015lL000005lL0P0000AO7000
001O70800004Fa`005/LFa`2000015/L000005/L0P0000=K7000Fa`00P0000=oo`00Ool00Woo00D0
07ooOomoo`0000eoo`03001oogoo009oo`0027oo00D007ooOomoo`0000ioo`04001oogooOol2000H
Ool500000dJm0012W@0200000cjM000jW@0D00000d/o00000002Dch4000015]N000005]M0`0000AO
G@00001O?@800005Gcd00000001O?002000015ll000005ll0P0000EO?000000005lL00800004Ga`0
0000Ga`200001UlL000005lL001K70H00004Fa`00000Fa`4000015/L000000000goo00D007ooOomo
o`0000ioo`03001oogoo005oo`002Goo0`003Goo0`000goo00<007ooOol05Goo900000E??`000000
05<n00@00003Feh00000009K?@800004Gcd00000Gcd20002Gcd2000015ll000005ll0P0000AO?000
001O?0<00004Ga`00000Ga`20002Ga`01000Ga`005lL0P0000AK7000001K70H00003Fa`005/L0080
0004Fa`000000002Ool01@00Oomoogoo00003goo0@000Goo0Goo00030006Ool20005Ool50002Ool4
0004Ool2000COol30003Ool01000B[d004Jm3`0000TjWP00>[h003Zn000ngP00@_l01P000Tln0P00
00=C?P0000000P0000EK?@00000005/m00800004Fcd00000Gcd3000015lm000005ll0P0000AO?000
001O?0800005Gc`00000001O700200000elL001O7002000015lL000005/L0P0000AK7000001K7080
0003Fa`005/L00X00003Ool007oo009oo`03001oogoo00eoo`40005oo`5oo`000goo00<007ooOol0
0goo0P001goo00<007ooOol01Woo00D007ooOomoo`00015oo`<000Moo`<0000?A[d004Jm0012_@00
?Yd003jM000jW@00>Yh003ZN00T00003@_l0000000D00004Cch00000Dch40002Fcd2000015/m0000
05/m0P0000EO?@00000005lm00800004Gc`00000Gc`2000015ll000005lL0`0000=O7000Ga`00P00
00AO7000001O70800004Fa`00000Fa`200000e/L001K7002000015/L000005/L0P0000IK7000Ool0
07oo0003Ool00`00Oomoo`09Ool01@00Oomoogoo00000Woo0003Ool00`00Oomoo`0<Ool00`00Oomo
o`06Ool01@00Oomoogoo00003Woo0`002goo6`0000A6oP00Bai;7P@00004Dch00000Dch5000015/m
000005/m0P0000=K?@0000000U/m0P0000AO?@00001O?0800003Gc`00000009O?003001O7000009O
70800004Ga`00000Ga`2000015/L000005/L0P0000=K7000Fa`00P0000AK7000001K70800003Fa`0
05/L00<0009oo`<000eoo`<000=oo`000goo00<007ooOol037oo00<007ooOol01Woo00D007ooOomo
o`0000]oo`<000moo`03001:_@0001/00003Bah0000000800004Dch00000Dch5000015/m000005/m
0P0000EK?@00000005/m00800004Gcd00000Gc`200000ell001O?00200001ElL00000000Ga`00P00
00AO7000001K70800006Fa`00000Fa`005/L0P0000AK7000001K70800003Fa`0000000800003Fa`0
000001Qoo`000goo00<007ooOol037oo00<007ooOol00Woo100017oo00<007ooOol01Woo0`004goo
00H004Jm0016_@00A[dH00000d/N0000000200001E<N00000000Ech0100000AK?@00001K?@800005
Fcd00000001K?@0200001U/m000005ll001O?0800004Gc`00000Ga`6000015/L000005/L0P0000=K
7000Fa`00P0000AK7000001K70800006Fa`00000Fa`005/L0P0000=oo`00Ool05goo0003Ool00`00
Oomoo`0<Ool00`00Oomoo`02Ool00`00Oomoo`04Ool20005Ool3000GOol500004DJm0012_@00@[d0
03jm000n_P00?[h003jn000ngP00@]h0200000=?7P0000000P000U<N0P0000=G?P00Ecd00P0000AK
?@00001K?@80009K?@800006Fcd00000Fcd005ll0P0000AO?000001O70800004Ga`00000Fa`;0000
15/L000005/L0P0000IK7000001K7000Fa`20004Ool00`00Oomoo`0EOol000=oo`03001oogoo00ao
o`03001oogoo009oo`03001oogoo00Aoo`04001oogooOol3000KOolE00000d;N0016oP0600000dlN
0000000300000e<N00000002000015Lm000005/m0P0000EK?@00000005/m00800003Fcd000000080
0004Fc`00000Fc`2000015/L000005/L0P0000AK7000001K70800004Fa`005/LFa`200000e/L0000
00080006Ool00`00Oomoo`0EOol000<000eoo`<000Aoo`@000=oo`@001moo`030016_@0001@00003
A_h0000000@00003Cah0000000<00003Dah0000000800004Ecd00000Fcd200001E/m00000000Fcd0
0P0000=K?@00Fcd00P0000AK?000001K70800004Fa`00000Fa`200001U/L000005/L001K70P00003
Fa`0000000]oo`03001oogoo01Eoo`007Woo0P008Woo00H004Jm0016_@00A[dA00000dKn001:oP05
00000dlN0000000200000e<N00000002000015Lm000005Lm0P000U/m0P0000=K?@0000000P0000AK
?@00001K?0800004Fa`00000Fa`200001U/L000005/L001K70800004Fa`00000Fa`200000e/L0000
000=Ool00`00Oomoo`0EOol000Moo`<000Ioo`8000Aoo`D000Eoo`03001oogoo021oo`D00003A[d0
04:m00800003@[h0000000`0009:oP@00003Cah0000000800003Dah0000000800004Ecd00000Ecd3
00001U/m000005/m001K?@800004Fcd00000Fa`2000015/L000005/L0P0000=K700000000P0000AK
7000001K70800003Fa`00000009oo`03001oogoo00Yoo`03001oogoo01Eoo`001Woo00D007ooOomo
o`0000Eoo`8000Eoo`04001oogoo0006Ool00`00Oomoo`0POol;00001D:n0012gP00@]h02P0000=:
oP0000000P0000=?7P0000000P0000=C7P0000000P0000AG?@00001G?@<00003Fcd0000000800004
Fcd00000Fad2000015/L000005/L0P0000IK7000001K7000Fa`2000015/L000005/L0P0017oo00<0
07ooOol02Woo00<007ooOol05Goo0006Ool01@00Oomoogoo00003Goo00<007ooOol01goo00<007oo
Ool087oo3`0000=6gP00A]h0200000=:oP0000000P0000=?7P00Dah00P0000=C7@0000000P0000=G
7@0000000ULM0P0000=K?@0000000P0000AK7@00001K7@800004Fa`00000Fa`200000e/L00000002
000015/L000005/L0P001goo00<007ooOol02Goo00<007ooOol05Goo0006Ool01@00Oomoogoo0000
3Woo00<007ooOol01goo00<007ooOol07goo00<004KM00003`0000=6gP00A]h01P0000=>oP000000
100000AC7P00001C7@@00005Ead00000001G7@0200000e/M00000002000015/M000005/M0P0000AK
7@00001K70800003Fa`005/L00800003Fa`0000000Yoo`03001oogoo00Qoo`03001oogoo01Eoo`00
1Woo00D007ooOomoo`0000moo`03001oogoo00Moo`80021oo`800004A]d00000A]d>00000dKn0000
000400000dkn001?7P04000015<N000005LM100000EG7@00000005LM00800003Fad005/M00800004
Fad00000Fad2000015/M000005/L100000=K7000000037oo00<007ooOol02Goo00<007ooOol057oo
0006Ool01@00Oomoogoo00003goo00<007ooOol02Goo00<007ooOol07goo1P0000=6g@0000002P00
00=:oP00B_h01P0000=?7P0000000P0000AC7@00001G7@@0009G7@800003Ead0000000800004Fad0
0000Fad2000015/M000005/M0P0000=K700000000P003goo00<007ooOol027oo00<007ooOol057oo
0006Ool01@00Oomoogoo00002goo00D007ooOomoo`0000aoo`03001oogoo01moo`P00003A]h00000
00X00003B_h0000000@00003Cah0000000800004Dad00000Ead5000015LM000005LM100000AK7@00
001K7@800004Fad00000Fad2000DOol00`00Oomoo`07Ool00`00Oomoo`0DOol000Moo`<000eoo`<0
00ioo`03001oogoo01moo`T00003A]h004KN00T00003B_h0000000@00003Cah0000000800004Dad0
0000Ead5000015LM000005LM100000AK7@00001K7@800003Fad0000001Moo`03001oogoo00Ioo`03
001oogoo01Aoo`00:Goo00<007ooOol07goo2`0000=6gP00A]h01`0000=:oP000000100000=C7P00
00000P0000AC7@00001G7@D00004Ead00000Ead4000015/M000005/M0P006Goo00<007ooOol01Woo
00<007ooOol057oo000ZOol2000OOol00`00B]d0000;00000dKN0000000500000dkn000000040000
0e<N0000000200001E<M00000000Ead0100000AG7@00001G7@L001aoo`03001oogoo00Eoo`03001o
ogoo01Aoo`00;7oo00<007ooOol07Goo0P0000=:g@0000002@0000=:gP00B_h01P0000=>oP000000
100000=C7@0000000P0000AC7@00001G7@/0021oo`03001oogoo00Aoo`03001oogoo01Aoo`00;Goo
00<007ooOol07Woo0P0000=:g@0000002P0000=:oP000000100000=>oP000000100000=C7@000000
0P0000AC7@00001G7@<00003Ead0000002Eoo`03001oogoo00Aoo`03001oogoo01=oo`00;Woo00<0
07ooOol07Woo100000=:g@000000200000=:oP000000100000=>oP000000100000=C7@0000000P00
00=C7@00000010009goo00<007ooOol017oo00<007ooOol04goo000_Ool00`00Oomoo`0NOol50000
0d[M0000000700000d[n0000000400000dkn0000000400000e<M0000000200000e<M001oo`0[Ool0
0`00Oomoo`03Ool00`00Oomoo`0COol0031oo`8001moo`H00003B]d0000000H00003B_h0000000@0
0003C_d0000000@00003Dad00000008002ioo`03001oogoo009oo`03001oogoo01=oo`00<Woo00<0
07ooOol07Goo1`0000=:g@0000001@0000=:oP000000100000=>o@0000001000<goo00D007ooOomo
o`0001Eoo`00<goo00<007ooOol07Goo1`0000=:g@0000001@0000=>oP0000001000>Goo00D007oo
Oomoo`0001Eoo`00=7oo00<007ooOol07Goo1`0000=:g@0000001@00@7oo00@007ooOol001Eoo`00
=Goo00<007ooOol07Goo1`00B7oo00<007oo00005Goo000fOol2000MOol00`00Oomoo`1<Ool01000
Oomoo`0057oo000hOol00`00Oomoo`0JOol00`00Oomoo`1=Ool00`00Ool0000DOol003Uoo`03001o
ogoo01Uoo`03001oogoo04ioo`8001Aoo`00>Woo00<007ooOol067oo00<007ooOol0CGoo0`0057oo
000kOol00`00Oomoo`0GOol00`00Oomoo`19Ool4000GOol003aoo`8001Moo`03001oogoo04Ioo`<0
01]oo`00?Woo00<007ooOol057oo00<007ooOol0@Woo10007Woo000oOol00`00Oomoo`0COol00`00
Oomoo`0nOol4000ROol0041oo`03001oogoo019oo`03001oogoo03Yoo`@002Ioo`00@Goo00<007oo
Ool04Goo00<007ooOol0=Woo1000:Woo0012Ool2000AOol00`00Oomoo`0cOol3000^Ool004Aoo`03
001oogoo00eoo`03001oogoo031oo`@0035oo`00AGoo00<007ooOol037oo00<007ooOol0;7oo1000
=Goo0016Ool00`00Oomoo`0;Ool00`00Oomoo`0XOol4000iOol004Moo`03001oogoo00Yoo`03001o
ogoo02Eoo`<003eoo`00B7oo0P002Woo00<007ooOol08Goo1000@7oo001:Ool00`00Oomoo`07Ool0
0`00Oomoo`0MOol40014Ool004]oo`03001oogoo00Ioo`03001oogoo01Uoo`@004Qoo`00C7oo00<0
07ooOol01Goo00<007ooOol05Woo0`00C7oo001=Ool00`00Oomoo`04Ool00`00Oomoo`0BOol4001?
Ool004ioo`8000Aoo`03001oogoo00ioo`@005=oo`00D7oo00D007ooOomoo`0000aoo`@005Moo`00
DGoo00@007ooOol000Qoo`@005]oo`00DWoo00<007oo00001Goo0`00Ggoo001COol200000goo0000
0002001ROol005Aoo`8006Ioo`00_7oo002lOol00;aoo`00_7oo002lOol00;aoo`00_7oo002lOol0
0;aoo`00_7oo002lOol00;aoo`00_7oo0000\
\>"],
  ImageRangeCache->{{{0, 187.438}, {151.812, 0}} -> {-0.210918, -0.120776, 
  0.00868723, 0.00868723}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.6 Exploring Theta", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallTheta[p,k,sd,r,q,t]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the Theta of a Call option, the sensitivity of its value to changes \
in the time."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[BlackScholesCallTheta[x, 60, 0.29, 0.04, 0, t], {x, \
20.,100.},{t,0.01,0.5}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
5.55112e-17 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76225 .08907 0 -6.16962 ]
[.76225 .08907 18 2.83038 ]
[.82456 .18447 0 -6.07766 ]
[.82456 .18447 18 2.92234 ]
[.88151 .27169 0 -5.9953 ]
[.88151 .27169 18 3.0047 ]
[.93378 .35174 0 -5.92111 ]
[.93378 .35174 18 3.07889 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 18 3.14607 ]
[.01379 .3298 -18 -2.79704 ]
[.01379 .3298 0 6.20296 ]
[.00148 .40694 -12 -2.86257 ]
[.00148 .40694 0 6.13743 ]
[-0.01144 .48795 -6 -2.93176 ]
[-0.01144 .48795 0 6.06824 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74995 .09363 m
.7438 .09592 L
s
[(0.1)] .76225 .08907 -1 .37103 Mshowa
.81217 .18881 m
.80598 .19098 L
s
[(0.2)] .82456 .18447 -1 .35059 Mshowa
.86906 .27583 m
.86283 .2779 L
s
[(0.3)] .88151 .27169 -1 .33229 Mshowa
.92126 .35569 m
.91501 .35766 L
s
[(0.4)] .93378 .35174 -1 .3158 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(0.5)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.76286 .11338 m
.75916 .11473 L
s
.77553 .13276 m
.77182 .1341 L
s
.78796 .15178 m
.78426 .15311 L
s
.80018 .17047 m
.79647 .17178 L
s
.82395 .20683 m
.82023 .20812 L
s
.83553 .22454 m
.8318 .22582 L
s
.8469 .24193 m
.84317 .2432 L
s
.85807 .25903 m
.85434 .26028 L
s
.87985 .29234 m
.87611 .29357 L
s
.89047 .30858 m
.88673 .3098 L
s
.90091 .32455 m
.89716 .32575 L
s
.91117 .34025 m
.90742 .34144 L
s
.9312 .37088 m
.92744 .37205 L
s
.94096 .38582 m
.9372 .38699 L
s
.95058 .40053 m
.94681 .40168 L
s
.96004 .415 m
.95627 .41614 L
s
.7368 .07352 m
.73311 .0749 L
s
.72339 .05301 m
.71971 .05441 L
s
.70973 .03212 m
.70606 .03353 L
s
.6958 .01081 m
.69213 .01224 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.02606 .32516 m
.0322 .32284 L
s
[(-10)] .01379 .3298 1 -0.37844 Mshowa
.01382 .40245 m
.01998 .40021 L
s
[(-5)] .00148 .40694 1 -0.36387 Mshowa
.00095 .48363 m
.00715 .48147 L
s
[(0)] -0.01144 .48795 1 -0.3485 Mshowa
.125 Mabswid
.02366 .34032 m
.02735 .33894 L
s
.02124 .35563 m
.02492 .35425 L
s
.01879 .37108 m
.02248 .36972 L
s
.01631 .38669 m
.02001 .38533 L
s
.01129 .41837 m
.015 .41703 L
s
.00875 .43444 m
.01245 .43311 L
s
.00617 .45067 m
.00988 .44936 L
s
.00357 .46707 m
.00729 .46576 L
s
.02844 .31014 m
.03212 .30874 L
s
.0308 .29527 m
.03448 .29386 L
s
.03313 .28054 m
.03681 .27912 L
s
.03545 .26594 m
.03912 .26452 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.646 .762 .934 r
.38973 .79902 .39806 .80578 .41196 .80213 .40367 .79533 Metetra
.646 .762 .934 r
.40367 .79533 .41196 .80213 .42593 .79845 .41769 .79162 Metetra
.646 .763 .934 r
.41769 .79162 .42593 .79845 .43997 .79474 .43178 .78788 Metetra
.648 .763 .933 r
.43178 .78788 .43997 .79474 .45409 .79097 .44595 .78408 Metetra
.651 .764 .932 r
.44595 .78408 .45409 .79097 .46829 .78704 .46019 .78014 Metetra
.66 .767 .929 r
.46019 .78014 .46829 .78704 .48256 .78279 .47451 .77589 Metetra
.675 .772 .923 r
.47451 .77589 .48256 .78279 .4969 .77792 .4889 .77106 Metetra
.699 .779 .913 r
.4889 .77106 .4969 .77792 .51132 .77208 .50337 .76527 Metetra
.729 .786 .898 r
.50337 .76527 .51132 .77208 .52579 .76486 .51791 .75811 Metetra
.76 .793 .88 r
.51791 .75811 .52579 .76486 .54031 .75595 .53249 .74923 Metetra
.788 .797 .859 r
.53249 .74923 .54031 .75595 .55485 .74521 .54709 .73846 Metetra
.81 .798 .84 r
.54709 .73846 .55485 .74521 .56938 .73276 .5617 .72592 Metetra
.824 .796 .826 r
.5617 .72592 .56938 .73276 .58391 .71902 .57629 .71198 Metetra
.83 .793 .817 r
.57629 .71198 .58391 .71902 .59841 .70458 .59087 .69727 Metetra
.828 .789 .815 r
.59087 .69727 .59841 .70458 .61291 .69016 .60543 .68254 Metetra
.819 .784 .819 r
.60543 .68254 .61291 .69016 .62742 .67648 .62001 .66854 Metetra
.803 .777 .829 r
.62001 .66854 .62742 .67648 .64197 .66415 .63462 .6559 Metetra
.779 .77 .844 r
.63462 .6559 .64197 .66415 .65659 .65359 .6493 .64511 Metetra
.747 .76 .863 r
.6493 .64511 .65659 .65359 .67133 .64503 .66409 .63639 Metetra
.709 .747 .882 r
.66409 .63639 .67133 .64503 .6862 .63851 .67901 .62979 Metetra
.665 .733 .9 r
.67901 .62979 .6862 .63851 .70122 .6339 .69409 .62518 Metetra
.62 .717 .916 r
.69409 .62518 .70122 .6339 .71642 .63094 .70934 .62228 Metetra
.577 .702 .928 r
.70934 .62228 .71642 .63094 .73179 .62932 .72477 .62077 Metetra
.54 .69 .936 r
.72477 .62077 .73179 .62932 .74733 .62868 .74037 .62026 Metetra
.513 .682 .942 r
.74037 .62026 .74733 .62868 .76304 .62869 .75614 .62041 Metetra
.497 .679 .947 r
.75614 .62041 .76304 .62869 .77889 .62903 .77205 .62089 Metetra
.492 .681 .95 r
.77205 .62089 .77889 .62903 .79488 .62945 .7881 .62143 Metetra
.495 .687 .953 r
.7881 .62143 .79488 .62945 .81098 .62975 .80427 .62181 Metetra
.505 .695 .954 r
.80427 .62181 .81098 .62975 .8272 .62976 .82055 .62189 Metetra
.519 .705 .955 r
.82055 .62189 .8272 .62976 .84351 .6294 .83692 .62156 Metetra
.534 .714 .955 r
.83692 .62156 .84351 .6294 .8599 .62861 .85338 .62077 Metetra
.551 .723 .954 r
.85338 .62077 .8599 .62861 .87638 .62736 .86992 .61951 Metetra
.566 .73 .952 r
.86992 .61951 .87638 .62736 .89293 .62566 .88653 .61778 Metetra
.58 .737 .95 r
.88653 .61778 .89293 .62566 .90955 .62354 .90322 .6156 Metetra
.593 .742 .947 r
.90322 .6156 .90955 .62354 .92625 .62103 .91998 .61303 Metetra
.604 .746 .945 r
.91998 .61303 .92625 .62103 .94301 .61816 .93681 .61009 Metetra
.613 .75 .943 r
.93681 .61009 .94301 .61816 .95985 .61497 .95372 .60683 Metetra
.62 .752 .941 r
.95372 .60683 .95985 .61497 .97677 .61152 .9707 .60331 Metetra
.626 .754 .939 r
.9707 .60331 .97677 .61152 .99376 .60784 .98776 .59954 Metetra
.646 .762 .934 r
.38132 .7922 .38973 .79902 .40367 .79533 .39531 .78847 Metetra
.646 .762 .934 r
.39531 .78847 .40367 .79533 .41769 .79162 .40937 .78473 Metetra
.646 .763 .934 r
.40937 .78473 .41769 .79162 .43178 .78788 .42351 .78096 Metetra
.647 .763 .933 r
.42351 .78096 .43178 .78788 .44595 .78408 .43772 .77713 Metetra
.651 .764 .932 r
.43772 .77713 .44595 .78408 .46019 .78014 .45201 .77317 Metetra
.658 .766 .929 r
.45201 .77317 .46019 .78014 .47451 .77589 .46638 .76892 Metetra
.673 .771 .924 r
.46638 .76892 .47451 .77589 .4889 .77106 .48082 .76412 Metetra
.696 .778 .914 r
.48082 .76412 .4889 .77106 .50337 .76527 .49535 .75839 Metetra
.726 .786 .9 r
.49535 .75839 .50337 .76527 .51791 .75811 .50994 .75129 Metetra
.759 .793 .881 r
.50994 .75129 .51791 .75811 .53249 .74923 .52458 .74245 Metetra
.788 .797 .86 r
.52458 .74245 .53249 .74923 .54709 .73846 .53925 .73167 Metetra
.81 .798 .84 r
.53925 .73167 .54709 .73846 .5617 .72592 .55393 .71902 Metetra
.825 .796 .825 r
.55393 .71902 .5617 .72592 .57629 .71198 .5686 .70489 Metetra
.832 .793 .815 r
.5686 .70489 .57629 .71198 .59087 .69727 .58325 .68991 Metetra
.83 .789 .813 r
.58325 .68991 .59087 .69727 .60543 .68254 .59789 .67485 Metetra
.822 .784 .817 r
.59789 .67485 .60543 .68254 .62001 .66854 .61252 .66051 Metetra
.805 .777 .827 r
.61252 .66051 .62001 .66854 .63462 .6559 .62719 .64757 Metetra
.781 .769 .842 r
.62719 .64757 .63462 .6559 .6493 .64511 .64193 .63652 Metetra
.749 .759 .861 r
.64193 .63652 .6493 .64511 .66409 .63639 .65678 .62763 Metetra
.709 .747 .881 r
.65678 .62763 .66409 .63639 .67901 .62979 .67176 .62095 Metetra
.664 .731 .9 r
.67176 .62095 .67901 .62979 .69409 .62518 .68689 .61634 Metetra
.617 .715 .916 r
.68689 .61634 .69409 .62518 .70934 .62228 .7022 .61351 Metetra
.572 .7 .928 r
.7022 .61351 .70934 .62228 .72477 .62077 .71768 .61212 Metetra
.534 .687 .937 r
.71768 .61212 .72477 .62077 .74037 .62026 .73334 .61176 Metetra
.506 .679 .943 r
.73334 .61176 .74037 .62026 .75614 .62041 .74917 .61206 Metetra
.49 .676 .947 r
.74917 .61206 .75614 .62041 .77205 .62089 .76514 .61268 Metetra
.485 .678 .951 r
.76514 .61268 .77205 .62089 .7881 .62143 .78125 .61334 Metetra
.49 .685 .953 r
.78125 .61334 .7881 .62143 .80427 .62181 .79749 .61382 Metetra
.501 .694 .955 r
.79749 .61382 .80427 .62181 .82055 .62189 .81383 .61395 Metetra
.516 .704 .956 r
.81383 .61395 .82055 .62189 .83692 .62156 .83027 .61366 Metetra
.533 .714 .955 r
.83027 .61366 .83692 .62156 .85338 .62077 .84679 .61288 Metetra
.55 .723 .954 r
.84679 .61288 .85338 .62077 .86992 .61951 .8634 .61159 Metetra
.566 .731 .952 r
.8634 .61159 .86992 .61951 .88653 .61778 .88007 .60982 Metetra
.581 .737 .95 r
.88007 .60982 .88653 .61778 .90322 .6156 .89683 .60759 Metetra
.594 .743 .948 r
.89683 .60759 .90322 .6156 .91998 .61303 .91365 .60495 Metetra
.604 .747 .945 r
.91365 .60495 .91998 .61303 .93681 .61009 .93055 .60194 Metetra
.614 .75 .943 r
.93055 .60194 .93681 .61009 .95372 .60683 .94752 .59861 Metetra
.621 .753 .94 r
.94752 .59861 .95372 .60683 .9707 .60331 .96457 .595 Metetra
.627 .754 .938 r
.96457 .595 .9707 .60331 .98776 .59954 .98169 .59116 Metetra
.646 .762 .934 r
.37283 .78531 .38132 .7922 .39531 .78847 .38686 .78155 Metetra
.646 .762 .934 r
.38686 .78155 .39531 .78847 .40937 .78473 .40097 .77777 Metetra
.646 .762 .934 r
.40097 .77777 .40937 .78473 .42351 .78096 .41515 .77396 Metetra
.647 .763 .933 r
.41515 .77396 .42351 .78096 .43772 .77713 .42941 .7701 Metetra
.65 .764 .932 r
.42941 .7701 .43772 .77713 .45201 .77317 .44375 .76612 Metetra
.657 .766 .93 r
.44375 .76612 .45201 .77317 .46638 .76892 .45817 .76188 Metetra
.671 .771 .925 r
.45817 .76188 .46638 .76892 .48082 .76412 .47266 .75711 Metetra
.694 .777 .915 r
.47266 .75711 .48082 .76412 .49535 .75839 .48724 .75143 Metetra
.724 .786 .901 r
.48724 .75143 .49535 .75839 .50994 .75129 .50189 .7444 Metetra
.757 .793 .882 r
.50189 .7444 .50994 .75129 .52458 .74245 .5166 .73561 Metetra
.787 .797 .861 r
.5166 .73561 .52458 .74245 .53925 .73167 .53134 .72482 Metetra
.811 .798 .84 r
.53134 .72482 .53925 .73167 .55393 .71902 .54609 .71208 Metetra
.826 .797 .824 r
.54609 .71208 .55393 .71902 .5686 .70489 .56084 .69776 Metetra
.834 .794 .814 r
.56084 .69776 .5686 .70489 .58325 .68991 .57556 .6825 Metetra
.833 .789 .81 r
.57556 .6825 .58325 .68991 .59789 .67485 .59026 .6671 Metetra
.824 .784 .814 r
.59026 .6671 .59789 .67485 .61252 .66051 .60497 .6524 Metetra
.808 .778 .824 r
.60497 .6524 .61252 .66051 .62719 .64757 .6197 .63913 Metetra
.784 .769 .84 r
.6197 .63913 .62719 .64757 .64193 .63652 .6345 .62782 Metetra
.751 .759 .859 r
.6345 .62782 .64193 .63652 .65678 .62763 .6494 .61876 Metetra
.71 .746 .88 r
.6494 .61876 .65678 .62763 .67176 .62095 .66443 .61199 Metetra
.663 .73 .899 r
.66443 .61199 .67176 .62095 .68689 .61634 .67962 .60739 Metetra
.614 .713 .916 r
.67962 .60739 .68689 .61634 .7022 .61351 .69498 .60464 Metetra
.567 .697 .928 r
.69498 .60464 .7022 .61351 .71768 .61212 .71052 .60337 Metetra
.527 .683 .937 r
.71052 .60337 .71768 .61212 .73334 .61176 .72624 .60317 Metetra
.499 .675 .943 r
.72624 .60317 .73334 .61176 .74917 .61206 .74213 .60363 Metetra
.483 .672 .948 r
.74213 .60363 .74917 .61206 .76514 .61268 .75817 .6044 Metetra
.479 .675 .951 r
.75817 .6044 .76514 .61268 .78125 .61334 .77434 .60518 Metetra
.484 .683 .954 r
.77434 .60518 .78125 .61334 .79749 .61382 .79064 .60576 Metetra
.496 .692 .956 r
.79064 .60576 .79749 .61382 .81383 .61395 .80705 .60596 Metetra
.513 .703 .956 r
.80705 .60596 .81383 .61395 .83027 .61366 .82355 .6057 Metetra
.531 .714 .956 r
.82355 .6057 .83027 .61366 .84679 .61288 .84014 .60492 Metetra
.549 .723 .955 r
.84014 .60492 .84679 .61288 .8634 .61159 .8568 .60361 Metetra
.566 .731 .953 r
.8568 .60361 .8634 .61159 .88007 .60982 .87355 .60179 Metetra
.581 .738 .95 r
.87355 .60179 .88007 .60982 .89683 .60759 .89036 .5995 Metetra
.594 .743 .948 r
.89036 .5995 .89683 .60759 .91365 .60495 .90725 .59679 Metetra
.605 .748 .945 r
.90725 .59679 .91365 .60495 .93055 .60194 .92421 .5937 Metetra
.615 .751 .942 r
.92421 .5937 .93055 .60194 .94752 .59861 .94125 .59029 Metetra
.622 .753 .94 r
.94125 .59029 .94752 .59861 .96457 .595 .95837 .58661 Metetra
.628 .755 .938 r
.95837 .58661 .96457 .595 .98169 .59116 .97556 .58269 Metetra
.646 .762 .934 r
.36425 .77835 .37283 .78531 .38686 .78155 .37833 .77456 Metetra
.646 .762 .934 r
.37833 .77456 .38686 .78155 .40097 .77777 .39249 .77074 Metetra
.646 .762 .934 r
.39249 .77074 .40097 .77777 .41515 .77396 .40672 .7669 Metetra
.647 .763 .933 r
.40672 .7669 .41515 .77396 .42941 .7701 .42103 .76301 Metetra
.649 .764 .933 r
.42103 .76301 .42941 .7701 .44375 .76612 .43541 .75901 Metetra
.656 .766 .93 r
.43541 .75901 .44375 .76612 .45817 .76188 .44988 .75477 Metetra
.669 .77 .925 r
.44988 .75477 .45817 .76188 .47266 .75711 .46442 .75003 Metetra
.692 .777 .917 r
.46442 .75003 .47266 .75711 .48724 .75143 .47905 .74441 Metetra
.722 .785 .902 r
.47905 .74441 .48724 .75143 .50189 .7444 .49376 .73745 Metetra
.755 .792 .883 r
.49376 .73745 .50189 .7444 .5166 .73561 .50853 .72871 Metetra
.786 .797 .861 r
.50853 .72871 .5166 .73561 .53134 .72482 .52334 .71792 Metetra
.811 .799 .84 r
.52334 .71792 .53134 .72482 .54609 .71208 .53817 .7051 Metetra
.828 .797 .823 r
.53817 .7051 .54609 .71208 .56084 .69776 .55299 .69058 Metetra
.836 .794 .812 r
.55299 .69058 .56084 .69776 .57556 .6825 .56779 .67503 Metetra
.835 .79 .808 r
.56779 .67503 .57556 .6825 .59026 .6671 .58257 .65928 Metetra
.827 .784 .812 r
.58257 .65928 .59026 .6671 .60497 .6524 .59734 .64421 Metetra
.811 .778 .822 r
.59734 .64421 .60497 .6524 .6197 .63913 .61214 .6306 Metetra
.786 .769 .838 r
.61214 .6306 .6197 .63913 .6345 .62782 .627 .61901 Metetra
.753 .759 .858 r
.627 .61901 .6345 .62782 .6494 .61876 .64195 .60976 Metetra
.71 .745 .879 r
.64195 .60976 .6494 .61876 .66443 .61199 .65704 .60291 Metetra
.662 .729 .899 r
.65704 .60291 .66443 .61199 .67962 .60739 .67228 .59831 Metetra
.61 .711 .916 r
.67228 .59831 .67962 .60739 .69498 .60464 .6877 .59565 Metetra
.561 .693 .929 r
.6877 .59565 .69498 .60464 .71052 .60337 .7033 .59452 Metetra
.52 .679 .938 r
.7033 .59452 .71052 .60337 .72624 .60317 .71907 .59448 Metetra
.491 .671 .944 r
.71907 .59448 .72624 .60317 .74213 .60363 .73502 .59512 Metetra
.475 .669 .948 r
.73502 .59512 .74213 .60363 .75817 .6044 .75112 .59605 Metetra
.471 .672 .952 r
.75112 .59605 .75817 .6044 .77434 .60518 .76736 .59697 Metetra
.478 .68 .954 r
.76736 .59697 .77434 .60518 .79064 .60576 .78372 .59764 Metetra
.492 .691 .956 r
.78372 .59764 .79064 .60576 .80705 .60596 .80019 .59791 Metetra
.51 .702 .957 r
.80019 .59791 .80705 .60596 .82355 .6057 .81676 .59767 Metetra
.529 .714 .956 r
.81676 .59767 .82355 .6057 .84014 .60492 .83341 .59689 Metetra
.548 .723 .955 r
.83341 .59689 .84014 .60492 .8568 .60361 .85014 .59556 Metetra
.566 .732 .953 r
.85014 .59556 .8568 .60361 .87355 .60179 .86695 .59369 Metetra
.582 .739 .95 r
.86695 .59369 .87355 .60179 .89036 .5995 .88383 .59134 Metetra
.595 .744 .948 r
.88383 .59134 .89036 .5995 .90725 .59679 .90079 .58855 Metetra
.606 .748 .945 r
.90079 .58855 .90725 .59679 .92421 .5937 .91781 .58538 Metetra
.616 .751 .942 r
.91781 .58538 .92421 .5937 .94125 .59029 .93492 .58189 Metetra
.623 .753 .94 r
.93492 .58189 .94125 .59029 .95837 .58661 .9521 .57812 Metetra
.629 .755 .938 r
.9521 .57812 .95837 .58661 .97556 .58269 .96937 .57412 Metetra
.646 .762 .934 r
.35559 .77133 .36425 .77835 .37833 .77456 .36972 .7675 Metetra
.646 .762 .934 r
.36972 .7675 .37833 .77456 .39249 .77074 .38392 .76364 Metetra
.646 .762 .934 r
.38392 .76364 .39249 .77074 .40672 .7669 .3982 .75976 Metetra
.647 .763 .933 r
.3982 .75976 .40672 .7669 .42103 .76301 .41255 .75584 Metetra
.649 .763 .933 r
.41255 .75584 .42103 .76301 .43541 .75901 .42699 .75182 Metetra
.655 .765 .931 r
.42699 .75182 .43541 .75901 .44988 .75477 .4415 .74758 Metetra
.668 .769 .926 r
.4415 .74758 .44988 .75477 .46442 .75003 .4561 .74287 Metetra
.689 .776 .918 r
.4561 .74287 .46442 .75003 .47905 .74441 .47079 .73731 Metetra
.719 .784 .904 r
.47079 .73731 .47905 .74441 .49376 .73745 .48555 .73042 Metetra
.753 .792 .885 r
.48555 .73042 .49376 .73745 .50853 .72871 .50038 .72175 Metetra
.785 .797 .862 r
.50038 .72175 .50853 .72871 .52334 .71792 .51526 .71097 Metetra
.811 .799 .84 r
.51526 .71097 .52334 .71792 .53817 .7051 .53017 .69807 Metetra
.829 .798 .822 r
.53017 .69807 .53817 .7051 .55299 .69058 .54506 .68336 Metetra
.838 .795 .81 r
.54506 .68336 .55299 .69058 .56779 .67503 .55994 .66752 Metetra
.838 .79 .806 r
.55994 .66752 .56779 .67503 .58257 .65928 .57479 .6514 Metetra
.83 .784 .809 r
.57479 .6514 .58257 .65928 .59734 .64421 .58964 .63594 Metetra
.814 .778 .819 r
.58964 .63594 .59734 .64421 .61214 .6306 .6045 .62196 Metetra
.789 .769 .835 r
.6045 .62196 .61214 .6306 .627 .61901 .61942 .61007 Metetra
.754 .758 .856 r
.61942 .61007 .627 .61901 .64195 .60976 .63443 .60063 Metetra
.711 .744 .878 r
.63443 .60063 .64195 .60976 .65704 .60291 .64957 .59369 Metetra
.66 .727 .899 r
.64957 .59369 .65704 .60291 .67228 .59831 .66487 .58911 Metetra
.606 .708 .916 r
.66487 .58911 .67228 .59831 .6877 .59565 .68034 .58654 Metetra
.555 .69 .929 r
.68034 .58654 .6877 .59565 .7033 .59452 .696 .58556 Metetra
.512 .675 .938 r
.696 .58556 .7033 .59452 .71907 .59448 .71183 .58571 Metetra
.482 .666 .944 r
.71183 .58571 .71907 .59448 .73502 .59512 .72784 .58653 Metetra
.466 .665 .948 r
.72784 .58653 .73502 .59512 .75112 .59605 .744 .58763 Metetra
.464 .669 .952 r
.744 .58763 .75112 .59605 .76736 .59697 .76031 .58869 Metetra
.472 .678 .955 r
.76031 .58869 .76736 .59697 .78372 .59764 .77673 .58947 Metetra
.487 .689 .957 r
.77673 .58947 .78372 .59764 .80019 .59791 .79327 .5898 Metetra
.507 .702 .958 r
.79327 .5898 .80019 .59791 .81676 .59767 .8099 .58959 Metetra
.528 .713 .957 r
.8099 .58959 .81676 .59767 .83341 .59689 .82662 .5888 Metetra
.548 .724 .956 r
.82662 .5888 .83341 .59689 .85014 .59556 .84341 .58743 Metetra
.566 .732 .953 r
.84341 .58743 .85014 .59556 .86695 .59369 .86029 .58552 Metetra
.583 .739 .95 r
.86029 .58552 .86695 .59369 .88383 .59134 .87723 .5831 Metetra
.596 .745 .948 r
.87723 .5831 .88383 .59134 .90079 .58855 .89425 .58023 Metetra
.608 .749 .945 r
.89425 .58023 .90079 .58855 .91781 .58538 .91134 .57698 Metetra
.617 .752 .942 r
.91134 .57698 .91781 .58538 .93492 .58189 .92852 .5734 Metetra
.624 .754 .94 r
.92852 .5734 .93492 .58189 .9521 .57812 .94577 .56954 Metetra
.63 .755 .938 r
.94577 .56954 .9521 .57812 .96937 .57412 .9631 .56545 Metetra
.646 .762 .934 r
.34685 .76424 .35559 .77133 .36972 .7675 .36102 .76037 Metetra
.646 .762 .934 r
.36102 .76037 .36972 .7675 .38392 .76364 .37527 .75647 Metetra
.646 .762 .934 r
.37527 .75647 .38392 .76364 .3982 .75976 .3896 .75256 Metetra
.647 .763 .934 r
.3896 .75256 .3982 .75976 .41255 .75584 .404 .7486 Metetra
.648 .763 .933 r
.404 .7486 .41255 .75584 .42699 .75182 .41848 .74456 Metetra
.654 .765 .931 r
.41848 .74456 .42699 .75182 .4415 .74758 .43305 .74031 Metetra
.666 .769 .927 r
.43305 .74031 .4415 .74758 .4561 .74287 .4477 .73563 Metetra
.686 .775 .919 r
.4477 .73563 .4561 .74287 .47079 .73731 .46243 .73013 Metetra
.716 .784 .905 r
.46243 .73013 .47079 .73731 .48555 .73042 .47725 .72333 Metetra
.751 .792 .886 r
.47725 .72333 .48555 .73042 .50038 .72175 .49215 .71473 Metetra
.784 .798 .863 r
.49215 .71473 .50038 .72175 .51526 .71097 .5071 .70397 Metetra
.812 .8 .84 r
.5071 .70397 .51526 .71097 .53017 .69807 .52208 .691 Metetra
.83 .798 .821 r
.52208 .691 .53017 .69807 .54506 .68336 .53705 .6761 Metetra
.84 .795 .808 r
.53705 .6761 .54506 .68336 .55994 .66752 .55201 .65995 Metetra
.841 .79 .803 r
.55201 .65995 .55994 .66752 .57479 .6514 .56694 .64345 Metetra
.833 .785 .806 r
.56694 .64345 .57479 .6514 .58964 .63594 .58186 .62758 Metetra
.817 .778 .816 r
.58186 .62758 .58964 .63594 .6045 .62196 .59679 .61321 Metetra
.792 .769 .833 r
.59679 .61321 .6045 .62196 .61942 .61007 .61177 .60102 Metetra
.756 .758 .854 r
.61177 .60102 .61942 .61007 .63443 .60063 .62684 .59137 Metetra
.711 .743 .877 r
.62684 .59137 .63443 .60063 .64957 .59369 .64203 .58434 Metetra
.659 .726 .899 r
.64203 .58434 .64957 .59369 .66487 .58911 .65739 .57978 Metetra
.602 .706 .917 r
.65739 .57978 .66487 .58911 .68034 .58654 .67291 .57732 Metetra
.549 .686 .93 r
.67291 .57732 .68034 .58654 .696 .58556 .68863 .5765 Metetra
.504 .671 .939 r
.68863 .5765 .696 .58556 .71183 .58571 .70452 .57684 Metetra
.473 .662 .944 r
.70452 .57684 .71183 .58571 .72784 .58653 .72059 .57786 Metetra
.457 .66 .949 r
.72059 .57786 .72784 .58653 .744 .58763 .73681 .57914 Metetra
.456 .665 .952 r
.73681 .57914 .744 .58763 .76031 .58869 .75318 .58035 Metetra
.465 .675 .956 r
.75318 .58035 .76031 .58869 .77673 .58947 .76967 .58124 Metetra
.483 .688 .958 r
.76967 .58124 .77673 .58947 .79327 .5898 .78627 .58163 Metetra
.504 .701 .958 r
.78627 .58163 .79327 .5898 .8099 .58959 .80297 .58145 Metetra
.526 .714 .958 r
.80297 .58145 .8099 .58959 .82662 .5888 .81975 .58065 Metetra
.548 .724 .956 r
.81975 .58065 .82662 .5888 .84341 .58743 .83661 .57924 Metetra
.567 .733 .953 r
.83661 .57924 .84341 .58743 .86029 .58552 .85355 .57727 Metetra
.583 .74 .951 r
.85355 .57727 .86029 .58552 .87723 .5831 .87056 .57477 Metetra
.597 .745 .947 r
.87056 .57477 .87723 .5831 .89425 .58023 .88764 .57182 Metetra
.609 .749 .944 r
.88764 .57182 .89425 .58023 .91134 .57698 .9048 .56848 Metetra
.618 .752 .942 r
.9048 .56848 .91134 .57698 .92852 .5734 .92204 .56481 Metetra
.625 .754 .939 r
.92204 .56481 .92852 .5734 .94577 .56954 .93936 .56086 Metetra
.631 .756 .937 r
.93936 .56086 .94577 .56954 .9631 .56545 .95677 .55669 Metetra
.646 .762 .934 r
.33802 .75707 .34685 .76424 .36102 .76037 .35224 .75317 Metetra
.646 .762 .934 r
.35224 .75317 .36102 .76037 .37527 .75647 .36654 .74924 Metetra
.646 .762 .934 r
.36654 .74924 .37527 .75647 .3896 .75256 .38091 .74528 Metetra
.646 .763 .934 r
.38091 .74528 .3896 .75256 .404 .7486 .39536 .74129 Metetra
.648 .763 .933 r
.39536 .74129 .404 .7486 .41848 .74456 .40989 .73722 Metetra
.653 .765 .931 r
.40989 .73722 .41848 .74456 .43305 .74031 .4245 .73297 Metetra
.664 .768 .928 r
.4245 .73297 .43305 .74031 .4477 .73563 .4392 .72831 Metetra
.684 .775 .92 r
.4392 .72831 .4477 .73563 .46243 .73013 .45399 .72287 Metetra
.713 .783 .907 r
.45399 .72287 .46243 .73013 .47725 .72333 .46887 .71616 Metetra
.748 .792 .888 r
.46887 .71616 .47725 .72333 .49215 .71473 .48383 .70764 Metetra
.783 .798 .865 r
.48383 .70764 .49215 .71473 .5071 .70397 .49885 .69692 Metetra
.812 .8 .841 r
.49885 .69692 .5071 .70397 .52208 .691 .51391 .68388 Metetra
.831 .799 .82 r
.51391 .68388 .52208 .691 .53705 .6761 .52896 .6688 Metetra
.842 .795 .806 r
.52896 .6688 .53705 .6761 .55201 .65995 .544 .65234 Metetra
.843 .791 .801 r
.544 .65234 .55201 .65995 .56694 .64345 .55901 .63544 Metetra
.836 .785 .803 r
.55901 .63544 .56694 .64345 .58186 .62758 .57401 .61913 Metetra
.82 .778 .813 r
.57401 .61913 .58186 .62758 .59679 .61321 .589 .60436 Metetra
.794 .769 .83 r
.589 .60436 .59679 .61321 .61177 .60102 .60405 .59184 Metetra
.758 .758 .852 r
.60405 .59184 .61177 .60102 .62684 .59137 .61917 .58197 Metetra
.712 .743 .876 r
.61917 .58197 .62684 .59137 .64203 .58434 .63442 .57486 Metetra
.657 .724 .898 r
.63442 .57486 .64203 .58434 .65739 .57978 .64983 .57031 Metetra
.598 .703 .917 r
.64983 .57031 .65739 .57978 .67291 .57732 .66541 .56797 Metetra
.541 .682 .93 r
.66541 .56797 .67291 .57732 .68863 .5765 .68118 .56734 Metetra
.495 .666 .939 r
.68118 .56734 .68863 .5765 .70452 .57684 .69714 .56789 Metetra
.463 .657 .945 r
.69714 .56789 .70452 .57684 .72059 .57786 .71326 .56912 Metetra
.447 .655 .949 r
.71326 .56912 .72059 .57786 .73681 .57914 .72955 .57059 Metetra
.447 .661 .953 r
.72955 .57059 .73681 .57914 .75318 .58035 .74598 .57195 Metetra
.459 .673 .956 r
.74598 .57195 .75318 .58035 .76967 .58124 .76254 .57295 Metetra
.478 .686 .958 r
.76254 .57295 .76967 .58124 .78627 .58163 .77921 .57341 Metetra
.501 .701 .959 r
.77921 .57341 .78627 .58163 .80297 .58145 .79597 .57325 Metetra
.525 .714 .958 r
.79597 .57325 .80297 .58145 .81975 .58065 .81281 .57243 Metetra
.547 .725 .956 r
.81281 .57243 .81975 .58065 .83661 .57924 .82974 .57098 Metetra
.567 .734 .954 r
.82974 .57098 .83661 .57924 .85355 .57727 .84674 .56894 Metetra
.584 .741 .951 r
.84674 .56894 .85355 .57727 .87056 .57477 .86382 .56636 Metetra
.599 .746 .947 r
.86382 .56636 .87056 .57477 .88764 .57182 .88097 .56332 Metetra
.61 .75 .944 r
.88097 .56332 .88764 .57182 .9048 .56848 .89819 .55989 Metetra
.619 .753 .941 r
.89819 .55989 .9048 .56848 .92204 .56481 .9155 .55612 Metetra
.627 .755 .939 r
.9155 .55612 .92204 .56481 .93936 .56086 .93289 .55208 Metetra
.632 .756 .937 r
.93289 .55208 .93936 .56086 .95677 .55669 .95036 .54782 Metetra
.646 .762 .934 r
.32911 .74984 .33802 .75707 .35224 .75317 .34337 .74589 Metetra
.646 .762 .934 r
.34337 .74589 .35224 .75317 .36654 .74924 .35771 .74192 Metetra
.646 .762 .934 r
.35771 .74192 .36654 .74924 .38091 .74528 .37213 .73793 Metetra
.646 .763 .934 r
.37213 .73793 .38091 .74528 .39536 .74129 .38663 .7339 Metetra
.648 .763 .933 r
.38663 .7339 .39536 .74129 .40989 .73722 .40121 .72981 Metetra
.652 .764 .932 r
.40121 .72981 .40989 .73722 .4245 .73297 .41588 .72554 Metetra
.662 .768 .928 r
.41588 .72554 .4245 .73297 .4392 .72831 .43063 .72091 Metetra
.681 .774 .921 r
.43063 .72091 .4392 .72831 .45399 .72287 .44547 .71553 Metetra
.71 .782 .909 r
.44547 .71553 .45399 .72287 .46887 .71616 .46041 .70891 Metetra
.746 .791 .89 r
.46041 .70891 .46887 .71616 .48383 .70764 .47543 .70048 Metetra
.782 .798 .866 r
.47543 .70048 .48383 .70764 .49885 .69692 .49052 .68981 Metetra
.812 .8 .841 r
.49052 .68981 .49885 .69692 .51391 .68388 .50565 .67672 Metetra
.833 .799 .82 r
.50565 .67672 .51391 .68388 .52896 .6688 .52079 .66146 Metetra
.844 .796 .805 r
.52079 .66146 .52896 .6688 .544 .65234 .53591 .64468 Metetra
.846 .791 .798 r
.53591 .64468 .544 .65234 .55901 .63544 .551 .62737 Metetra
.839 .785 .8 r
.551 .62737 .55901 .63544 .57401 .61913 .56607 .6106 Metetra
.823 .778 .81 r
.56607 .6106 .57401 .61913 .589 .60436 .58114 .5954 Metetra
.797 .769 .827 r
.58114 .5954 .589 .60436 .60405 .59184 .59625 .58253 Metetra
.761 .757 .85 r
.59625 .58253 .60405 .59184 .61917 .58197 .61143 .57243 Metetra
.713 .742 .874 r
.61143 .57243 .61917 .58197 .63442 .57486 .62674 .56522 Metetra
.655 .722 .898 r
.62674 .56522 .63442 .57486 .64983 .57031 .6422 .56071 Metetra
.593 .7 .917 r
.6422 .56071 .64983 .57031 .66541 .56797 .65784 .55849 Metetra
.533 .678 .931 r
.65784 .55849 .66541 .56797 .68118 .56734 .67366 .55806 Metetra
.484 .66 .939 r
.67366 .55806 .68118 .56734 .69714 .56789 .68968 .55884 Metetra
.452 .651 .945 r
.68968 .55884 .69714 .56789 .71326 .56912 .70587 .5603 Metetra
.437 .65 .949 r
.70587 .5603 .71326 .56912 .72955 .57059 .72222 .56197 Metetra
.438 .657 .953 r
.72222 .56197 .72955 .57059 .74598 .57195 .73871 .5635 Metetra
.452 .67 .957 r
.73871 .5635 .74598 .57195 .76254 .57295 .75534 .5646 Metetra
.473 .685 .959 r
.75534 .5646 .76254 .57295 .77921 .57341 .77207 .56513 Metetra
.498 .7 .96 r
.77207 .56513 .77921 .57341 .79597 .57325 .78889 .56499 Metetra
.524 .714 .959 r
.78889 .56499 .79597 .57325 .81281 .57243 .8058 .56415 Metetra
.547 .726 .957 r
.8058 .56415 .81281 .57243 .82974 .57098 .82279 .56265 Metetra
.568 .735 .954 r
.82279 .56265 .82974 .57098 .84674 .56894 .83986 .56053 Metetra
.586 .742 .951 r
.83986 .56053 .84674 .56894 .86382 .56636 .857 .55787 Metetra
.6 .747 .947 r
.857 .55787 .86382 .56636 .88097 .56332 .87422 .55474 Metetra
.612 .751 .944 r
.87422 .55474 .88097 .56332 .89819 .55989 .89151 .55121 Metetra
.621 .753 .941 r
.89151 .55121 .89819 .55989 .9155 .55612 .90888 .54734 Metetra
.628 .755 .939 r
.90888 .54734 .9155 .55612 .93289 .55208 .92634 .54321 Metetra
.633 .757 .937 r
.92634 .54321 .93289 .55208 .95036 .54782 .94389 .53885 Metetra
.646 .762 .934 r
.3201 .74254 .32911 .74984 .34337 .74589 .33441 .73855 Metetra
.646 .762 .934 r
.33441 .73855 .34337 .74589 .35771 .74192 .3488 .73454 Metetra
.646 .762 .934 r
.3488 .73454 .35771 .74192 .37213 .73793 .36327 .73051 Metetra
.646 .763 .934 r
.36327 .73051 .37213 .73793 .38663 .7339 .37782 .72644 Metetra
.647 .763 .933 r
.37782 .72644 .38663 .7339 .40121 .72981 .39245 .72232 Metetra
.651 .764 .932 r
.39245 .72232 .40121 .72981 .41588 .72554 .40716 .71804 Metetra
.66 .767 .929 r
.40716 .71804 .41588 .72554 .43063 .72091 .42196 .71342 Metetra
.678 .773 .922 r
.42196 .71342 .43063 .72091 .44547 .71553 .43686 .70811 Metetra
.707 .782 .91 r
.43686 .70811 .44547 .71553 .46041 .70891 .45185 .70158 Metetra
.743 .791 .891 r
.45185 .70158 .46041 .70891 .47543 .70048 .46693 .69326 Metetra
.78 .798 .867 r
.46693 .69326 .47543 .70048 .49052 .68981 .48209 .68265 Metetra
.812 .801 .842 r
.48209 .68265 .49052 .68981 .50565 .67672 .4973 .66952 Metetra
.834 .8 .819 r
.4973 .66952 .50565 .67672 .52079 .66146 .51253 .65408 Metetra
.846 .796 .803 r
.51253 .65408 .52079 .66146 .53591 .64468 .52774 .63698 Metetra
.849 .791 .795 r
.52774 .63698 .53591 .64468 .551 .62737 .54291 .61924 Metetra
.842 .785 .797 r
.54291 .61924 .551 .62737 .56607 .6106 .55806 .60199 Metetra
.826 .778 .807 r
.55806 .60199 .56607 .6106 .58114 .5954 .57321 .58633 Metetra
.8 .769 .824 r
.57321 .58633 .58114 .5954 .59625 .58253 .58838 .57308 Metetra
.763 .757 .847 r
.58838 .57308 .59625 .58253 .61143 .57243 .60362 .56274 Metetra
.713 .741 .873 r
.60362 .56274 .61143 .57243 .62674 .56522 .61898 .55543 Metetra
.653 .72 .898 r
.61898 .55543 .62674 .56522 .6422 .56071 .6345 .55096 Metetra
.588 .696 .917 r
.6345 .55096 .6422 .56071 .65784 .55849 .65019 .54889 Metetra
.525 .673 .931 r
.65019 .54889 .65784 .55849 .67366 .55806 .66607 .54867 Metetra
.473 .655 .939 r
.66607 .54867 .67366 .55806 .68968 .55884 .68214 .5497 Metetra
.44 .645 .945 r
.68214 .5497 .68968 .55884 .70587 .5603 .69839 .5514 Metetra
.425 .645 .949 r
.69839 .5514 .70587 .5603 .72222 .56197 .71481 .55329 Metetra
.428 .653 .953 r
.71481 .55329 .72222 .56197 .73871 .5635 .73137 .55498 Metetra
.444 .667 .957 r
.73137 .55498 .73871 .5635 .75534 .5646 .74805 .55621 Metetra
.468 .684 .96 r
.74805 .55621 .75534 .5646 .77207 .56513 .76485 .5568 Metetra
.496 .7 .96 r
.76485 .5568 .77207 .56513 .78889 .56499 .78174 .55667 Metetra
.523 .714 .959 r
.78174 .55667 .78889 .56499 .8058 .56415 .79872 .5558 Metetra
.548 .726 .957 r
.79872 .5558 .8058 .56415 .82279 .56265 .81577 .55424 Metetra
.569 .736 .954 r
.81577 .55424 .82279 .56265 .83986 .56053 .8329 .55205 Metetra
.587 .743 .951 r
.8329 .55205 .83986 .56053 .857 .55787 .85011 .54929 Metetra
.602 .748 .947 r
.85011 .54929 .857 .55787 .87422 .55474 .86739 .54606 Metetra
.613 .752 .944 r
.86739 .54606 .87422 .55474 .89151 .55121 .88475 .54242 Metetra
.622 .754 .941 r
.88475 .54242 .89151 .55121 .90888 .54734 .9022 .53846 Metetra
.629 .756 .938 r
.9022 .53846 .90888 .54734 .92634 .54321 .91972 .53423 Metetra
.634 .757 .936 r
.91972 .53423 .92634 .54321 .94389 .53885 .93734 .52978 Metetra
.646 .762 .934 r
.31101 .73516 .3201 .74254 .33441 .73855 .32537 .73113 Metetra
.646 .762 .934 r
.32537 .73113 .33441 .73855 .3488 .73454 .3398 .72708 Metetra
.646 .762 .934 r
.3398 .72708 .3488 .73454 .36327 .73051 .35432 .72301 Metetra
.646 .762 .934 r
.35432 .72301 .36327 .73051 .37782 .72644 .36892 .71891 Metetra
.647 .763 .933 r
.36892 .71891 .37782 .72644 .39245 .72232 .38359 .71475 Metetra
.65 .764 .932 r
.38359 .71475 .39245 .72232 .40716 .71804 .39836 .71046 Metetra
.659 .767 .929 r
.39836 .71046 .40716 .71804 .42196 .71342 .41321 .70585 Metetra
.676 .772 .923 r
.41321 .70585 .42196 .71342 .43686 .70811 .42816 .7006 Metetra
.703 .781 .912 r
.42816 .7006 .43686 .70811 .45185 .70158 .44321 .69417 Metetra
.74 .79 .893 r
.44321 .69417 .45185 .70158 .46693 .69326 .45835 .68597 Metetra
.778 .798 .869 r
.45835 .68597 .46693 .69326 .48209 .68265 .47358 .67543 Metetra
.811 .801 .842 r
.47358 .67543 .48209 .68265 .4973 .66952 .48887 .66227 Metetra
.835 .801 .818 r
.48887 .66227 .4973 .66952 .51253 .65408 .50418 .64666 Metetra
.848 .797 .801 r
.50418 .64666 .51253 .65408 .52774 .63698 .51948 .62924 Metetra
.852 .792 .792 r
.51948 .62924 .52774 .63698 .54291 .61924 .53474 .61105 Metetra
.846 .785 .793 r
.53474 .61105 .54291 .61924 .55806 .60199 .54997 .59329 Metetra
.83 .778 .803 r
.54997 .59329 .55806 .60199 .57321 .58633 .56519 .57714 Metetra
.804 .769 .821 r
.56519 .57714 .57321 .58633 .58838 .57308 .58043 .5635 Metetra
.765 .756 .845 r
.58043 .5635 .58838 .57308 .60362 .56274 .59573 .5529 Metetra
.714 .739 .872 r
.59573 .5529 .60362 .56274 .61898 .55543 .61115 .54549 Metetra
.651 .718 .897 r
.61115 .54549 .61898 .55543 .6345 .55096 .62672 .54106 Metetra
.582 .692 .918 r
.62672 .54106 .6345 .55096 .65019 .54889 .64246 .53915 Metetra
.515 .667 .931 r
.64246 .53915 .65019 .54889 .66607 .54867 .6584 .53917 Metetra
.461 .648 .939 r
.6584 .53917 .66607 .54867 .68214 .5497 .67453 .54047 Metetra
.427 .638 .944 r
.67453 .54047 .68214 .5497 .69839 .5514 .69084 .54243 Metetra
.413 .639 .949 r
.69084 .54243 .69839 .5514 .71481 .55329 .70732 .54455 Metetra
.418 .649 .953 r
.70732 .54455 .71481 .55329 .73137 .55498 .72395 .54642 Metetra
.437 .664 .958 r
.72395 .54642 .73137 .55498 .74805 .55621 .7407 .54776 Metetra
.464 .682 .96 r
.7407 .54776 .74805 .55621 .76485 .5568 .75756 .54841 Metetra
.493 .7 .961 r
.75756 .54841 .76485 .5568 .78174 .55667 .77451 .54829 Metetra
.522 .715 .96 r
.77451 .54829 .78174 .55667 .79872 .5558 .79155 .54739 Metetra
.548 .727 .958 r
.79155 .54739 .79872 .5558 .81577 .55424 .80867 .54576 Metetra
.57 .737 .954 r
.80867 .54576 .81577 .55424 .8329 .55205 .82587 .54348 Metetra
.589 .744 .95 r
.82587 .54348 .8329 .55205 .85011 .54929 .84314 .54062 Metetra
.603 .749 .947 r
.84314 .54062 .85011 .54929 .86739 .54606 .86049 .53728 Metetra
.615 .752 .943 r
.86049 .53728 .86739 .54606 .88475 .54242 .87792 .53354 Metetra
.624 .755 .94 r
.87792 .53354 .88475 .54242 .9022 .53846 .89543 .52947 Metetra
.63 .756 .938 r
.89543 .52947 .9022 .53846 .91972 .53423 .91303 .52514 Metetra
.635 .757 .936 r
.91303 .52514 .91972 .53423 .93734 .52978 .93072 .52059 Metetra
.646 .762 .934 r
.30182 .72771 .31101 .73516 .32537 .73113 .31623 .72364 Metetra
.646 .762 .934 r
.31623 .72364 .32537 .73113 .3398 .72708 .33071 .71955 Metetra
.646 .762 .934 r
.33071 .71955 .3398 .72708 .35432 .72301 .34528 .71543 Metetra
.646 .762 .934 r
.34528 .71543 .35432 .72301 .36892 .71891 .35992 .71129 Metetra
.647 .763 .933 r
.35992 .71129 .36892 .71891 .38359 .71475 .37465 .7071 Metetra
.65 .764 .933 r
.37465 .7071 .38359 .71475 .39836 .71046 .38946 .70279 Metetra
.657 .766 .93 r
.38946 .70279 .39836 .71046 .41321 .70585 .40437 .6982 Metetra
.673 .771 .924 r
.40437 .6982 .41321 .70585 .42816 .7006 .41937 .693 Metetra
.7 .78 .913 r
.41937 .693 .42816 .7006 .44321 .69417 .43447 .68668 Metetra
.736 .79 .895 r
.43447 .68668 .44321 .69417 .45835 .68597 .44968 .6786 Metetra
.776 .798 .871 r
.44968 .6786 .45835 .68597 .47358 .67543 .46498 .66815 Metetra
.811 .802 .843 r
.46498 .66815 .47358 .67543 .48887 .66227 .48034 .65499 Metetra
.836 .801 .818 r
.48034 .65499 .48887 .66227 .50418 .64666 .49574 .63922 Metetra
.85 .798 .799 r
.49574 .63922 .50418 .64666 .51948 .62924 .51113 .62147 Metetra
.854 .792 .789 r
.51113 .62147 .51948 .62924 .53474 .61105 .52648 .6028 Metetra
.849 .786 .79 r
.52648 .6028 .53474 .61105 .54997 .59329 .5418 .5845 Metetra
.833 .778 .799 r
.5418 .5845 .54997 .59329 .56519 .57714 .55709 .56783 Metetra
.807 .769 .817 r
.55709 .56783 .56519 .57714 .58043 .5635 .5724 .55376 Metetra
.768 .756 .842 r
.5724 .55376 .58043 .5635 .59573 .5529 .58777 .54289 Metetra
.714 .738 .87 r
.58777 .54289 .59573 .5529 .61115 .54549 .60324 .53538 Metetra
.648 .715 .897 r
.60324 .53538 .61115 .54549 .62672 .54106 .61886 .531 Metetra
.575 .688 .918 r
.61886 .531 .62672 .54106 .64246 .53915 .63466 .52928 Metetra
.505 .661 .931 r
.63466 .52928 .64246 .53915 .6584 .53917 .65066 .52956 Metetra
.448 .641 .939 r
.65066 .52956 .6584 .53917 .67453 .54047 .66685 .53114 Metetra
.412 .631 .944 r
.66685 .53114 .67453 .54047 .69084 .54243 .68322 .53339 Metetra
.4 .632 .949 r
.68322 .53339 .69084 .54243 .70732 .54455 .69976 .53575 Metetra
.407 .644 .953 r
.69976 .53575 .70732 .54455 .72395 .54642 .71645 .53781 Metetra
.429 .661 .958 r
.71645 .53781 .72395 .54642 .7407 .54776 .73326 .53927 Metetra
.459 .681 .961 r
.73326 .53927 .7407 .54776 .75756 .54841 .75019 .53997 Metetra
.491 .7 .962 r
.75019 .53997 .75756 .54841 .77451 .54829 .76721 .53985 Metetra
.522 .716 .961 r
.76721 .53985 .77451 .54829 .79155 .54739 .78431 .53891 Metetra
.549 .728 .958 r
.78431 .53891 .79155 .54739 .80867 .54576 .8015 .53721 Metetra
.572 .738 .954 r
.8015 .53721 .80867 .54576 .82587 .54348 .81876 .53483 Metetra
.59 .745 .95 r
.81876 .53483 .82587 .54348 .84314 .54062 .8361 .53186 Metetra
.605 .75 .946 r
.8361 .53186 .84314 .54062 .86049 .53728 .85351 .52841 Metetra
.617 .753 .943 r
.85351 .52841 .86049 .53728 .87792 .53354 .87101 .52456 Metetra
.625 .755 .94 r
.87101 .52456 .87792 .53354 .89543 .52947 .88859 .52038 Metetra
.632 .757 .938 r
.88859 .52038 .89543 .52947 .91303 .52514 .90626 .51594 Metetra
.637 .757 .935 r
.90626 .51594 .91303 .52514 .93072 .52059 .92402 .5113 Metetra
.646 .762 .934 r
.29254 .72018 .30182 .72771 .31623 .72364 .30699 .71607 Metetra
.646 .762 .934 r
.30699 .71607 .31623 .72364 .33071 .71955 .32153 .71194 Metetra
.646 .762 .934 r
.32153 .71194 .33071 .71955 .34528 .71543 .33614 .70778 Metetra
.646 .762 .934 r
.33614 .70778 .34528 .71543 .35992 .71129 .35084 .7036 Metetra
.647 .763 .933 r
.35084 .7036 .35992 .71129 .37465 .7071 .36562 .69937 Metetra
.649 .763 .933 r
.36562 .69937 .37465 .7071 .38946 .70279 .38048 .69504 Metetra
.655 .766 .931 r
.38048 .69504 .38946 .70279 .40437 .6982 .39543 .69046 Metetra
.67 .771 .925 r
.39543 .69046 .40437 .6982 .41937 .693 .41048 .68532 Metetra
.696 .779 .915 r
.41048 .68532 .41937 .693 .43447 .68668 .42564 .6791 Metetra
.733 .789 .898 r
.42564 .6791 .43447 .68668 .44968 .6786 .44091 .67116 Metetra
.773 .798 .873 r
.44091 .67116 .44968 .6786 .46498 .66815 .45628 .66082 Metetra
.81 .802 .844 r
.45628 .66082 .46498 .66815 .48034 .65499 .47172 .64767 Metetra
.837 .802 .818 r
.47172 .64767 .48034 .65499 .49574 .63922 .48721 .63174 Metetra
.852 .798 .797 r
.48721 .63174 .49574 .63922 .51113 .62147 .50269 .61366 Metetra
.857 .793 .787 r
.50269 .61366 .51113 .62147 .52648 .6028 .51814 .5945 Metetra
.852 .786 .786 r
.51814 .5945 .52648 .6028 .5418 .5845 .53355 .57563 Metetra
.837 .778 .795 r
.53355 .57563 .5418 .5845 .55709 .56783 .54892 .5584 Metetra
.81 .768 .814 r
.54892 .5584 .55709 .56783 .5724 .55376 .5643 .54388 Metetra
.77 .755 .839 r
.5643 .54388 .5724 .55376 .58777 .54289 .57973 .53272 Metetra
.715 .737 .868 r
.57973 .53272 .58777 .54289 .60324 .53538 .59525 .52509 Metetra
.646 .712 .896 r
.59525 .52509 .60324 .53538 .61886 .531 .61093 .52078 Metetra
.568 .683 .918 r
.61093 .52078 .61886 .531 .63466 .52928 .62678 .51926 Metetra
.493 .655 .931 r
.62678 .51926 .63466 .52928 .65066 .52956 .64283 .51982 Metetra
.433 .633 .939 r
.64283 .51982 .65066 .52956 .66685 .53114 .65908 .52173 Metetra
.397 .622 .943 r
.65908 .52173 .66685 .53114 .68322 .53339 .67551 .52428 Metetra
.385 .625 .948 r
.67551 .52428 .68322 .53339 .69976 .53575 .69212 .5269 Metetra
.396 .639 .953 r
.69212 .5269 .69976 .53575 .71645 .53781 .70887 .52914 Metetra
.421 .658 .958 r
.70887 .52914 .71645 .53781 .73326 .53927 .72575 .53072 Metetra
.454 .68 .962 r
.72575 .53072 .73326 .53927 .75019 .53997 .74274 .53148 Metetra
.489 .7 .963 r
.74274 .53148 .75019 .53997 .76721 .53985 .75982 .53135 Metetra
.522 .717 .961 r
.75982 .53135 .76721 .53985 .78431 .53891 .77699 .53036 Metetra
.55 .73 .958 r
.77699 .53036 .78431 .53891 .8015 .53721 .79424 .52857 Metetra
.574 .739 .954 r
.79424 .52857 .8015 .53721 .81876 .53483 .81157 .52609 Metetra
.592 .746 .95 r
.81157 .52609 .81876 .53483 .8361 .53186 .82897 .52301 Metetra
.607 .751 .946 r
.82897 .52301 .8361 .53186 .85351 .52841 .84646 .51944 Metetra
.618 .754 .943 r
.84646 .51944 .85351 .52841 .87101 .52456 .86402 .51547 Metetra
.627 .756 .94 r
.86402 .51547 .87101 .52456 .88859 .52038 .88167 .51118 Metetra
.633 .757 .937 r
.88167 .51118 .88859 .52038 .90626 .51594 .89941 .50663 Metetra
.638 .758 .935 r
.89941 .50663 .90626 .51594 .92402 .5113 .91725 .5019 Metetra
.646 .762 .934 r
.28317 .71257 .29254 .72018 .30699 .71607 .29767 .70842 Metetra
.646 .762 .934 r
.29767 .70842 .30699 .71607 .32153 .71194 .31225 .70425 Metetra
.646 .762 .934 r
.31225 .70425 .32153 .71194 .33614 .70778 .32691 .70005 Metetra
.646 .762 .934 r
.32691 .70005 .33614 .70778 .35084 .7036 .34166 .69583 Metetra
.646 .763 .934 r
.34166 .69583 .35084 .7036 .36562 .69937 .35649 .69156 Metetra
.648 .763 .933 r
.35649 .69156 .36562 .69937 .38048 .69504 .3714 .68721 Metetra
.654 .765 .931 r
.3714 .68721 .38048 .69504 .39543 .69046 .38641 .68263 Metetra
.668 .77 .926 r
.38641 .68263 .39543 .69046 .41048 .68532 .40151 .67754 Metetra
.692 .778 .917 r
.40151 .67754 .41048 .68532 .42564 .6791 .41672 .67144 Metetra
.729 .788 .9 r
.41672 .67144 .42564 .6791 .44091 .67116 .43205 .66363 Metetra
.771 .798 .875 r
.43205 .66363 .44091 .67116 .45628 .66082 .44749 .65342 Metetra
.809 .803 .846 r
.44749 .65342 .45628 .66082 .47172 .64767 .46301 .6403 Metetra
.838 .803 .817 r
.46301 .6403 .47172 .64767 .48721 .63174 .47859 .62424 Metetra
.855 .799 .796 r
.47859 .62424 .48721 .63174 .50269 .61366 .49416 .60582 Metetra
.86 .793 .784 r
.49416 .60582 .50269 .61366 .51814 .5945 .50971 .58615 Metetra
.856 .786 .782 r
.50971 .58615 .51814 .5945 .53355 .57563 .52521 .56668 Metetra
.841 .778 .791 r
.52521 .56668 .53355 .57563 .54892 .5584 .54067 .54885 Metetra
.814 .768 .81 r
.54067 .54885 .54892 .5584 .5643 .54388 .55612 .53384 Metetra
.773 .755 .836 r
.55612 .53384 .5643 .54388 .57973 .53272 .57161 .52236 Metetra
.715 .735 .867 r
.57161 .52236 .57973 .53272 .59525 .52509 .58719 .51463 Metetra
.642 .71 .896 r
.58719 .51463 .59525 .52509 .61093 .52078 .60292 .5104 Metetra
.56 .678 .918 r
.60292 .5104 .61093 .52078 .62678 .51926 .61883 .5091 Metetra
.48 .647 .931 r
.61883 .5091 .62678 .51926 .64283 .51982 .63493 .50998 Metetra
.417 .624 .938 r
.63493 .50998 .64283 .51982 .65908 .52173 .65124 .51223 Metetra
.38 .613 .942 r
.65124 .51223 .65908 .52173 .67551 .52428 .66773 .5151 Metetra
.37 .618 .947 r
.66773 .5151 .67551 .52428 .69212 .5269 .68439 .51799 Metetra
.383 .633 .953 r
.68439 .51799 .69212 .5269 .70887 .52914 .70121 .52044 Metetra
.413 .655 .959 r
.70121 .52044 .70887 .52914 .72575 .53072 .71816 .52214 Metetra
.449 .679 .962 r
.71816 .52214 .72575 .53072 .74274 .53148 .73521 .52294 Metetra
.487 .7 .963 r
.73521 .52294 .74274 .53148 .75982 .53135 .75236 .52279 Metetra
.522 .718 .962 r
.75236 .52279 .75982 .53135 .77699 .53036 .76959 .52174 Metetra
.551 .731 .958 r
.76959 .52174 .77699 .53036 .79424 .52857 .78691 .51986 Metetra
.576 .741 .954 r
.78691 .51986 .79424 .52857 .81157 .52609 .8043 .51726 Metetra
.595 .747 .95 r
.8043 .51726 .81157 .52609 .82897 .52301 .82177 .51405 Metetra
.609 .752 .946 r
.82177 .51405 .82897 .52301 .84646 .51944 .83932 .51036 Metetra
.62 .755 .942 r
.83932 .51036 .84646 .51944 .86402 .51547 .85695 .50627 Metetra
.629 .756 .939 r
.85695 .50627 .86402 .51547 .88167 .51118 .87468 .50186 Metetra
.635 .757 .937 r
.87468 .50186 .88167 .51118 .89941 .50663 .89249 .49721 Metetra
.639 .758 .935 r
.89249 .49721 .89941 .50663 .91725 .5019 .9104 .49237 Metetra
.646 .762 .934 r
.2737 .70489 .28317 .71257 .29767 .70842 .28825 .7007 Metetra
.646 .762 .934 r
.28825 .7007 .29767 .70842 .31225 .70425 .30288 .69648 Metetra
.646 .762 .934 r
.30288 .69648 .31225 .70425 .32691 .70005 .31759 .69224 Metetra
.646 .762 .934 r
.31759 .69224 .32691 .70005 .34166 .69583 .33239 .68797 Metetra
.646 .763 .934 r
.33239 .68797 .34166 .69583 .35649 .69156 .34726 .68367 Metetra
.648 .763 .933 r
.34726 .68367 .35649 .69156 .3714 .68721 .36223 .67929 Metetra
.653 .765 .931 r
.36223 .67929 .3714 .68721 .38641 .68263 .37728 .67471 Metetra
.665 .769 .927 r
.37728 .67471 .38641 .68263 .40151 .67754 .39244 .66967 Metetra
.689 .777 .918 r
.39244 .66967 .40151 .67754 .41672 .67144 .40771 .66368 Metetra
.725 .787 .902 r
.40771 .66368 .41672 .67144 .43205 .66363 .42309 .65603 Metetra
.767 .798 .877 r
.42309 .65603 .43205 .66363 .44749 .65342 .4386 .64596 Metetra
.808 .803 .847 r
.4386 .64596 .44749 .65342 .46301 .6403 .4542 .6329 Metetra
.838 .803 .817 r
.4542 .6329 .46301 .6403 .47859 .62424 .46987 .61672 Metetra
.857 .8 .794 r
.46987 .61672 .47859 .62424 .49416 .60582 .48555 .59796 Metetra
.863 .794 .78 r
.48555 .59796 .49416 .60582 .50971 .58615 .50119 .57776 Metetra
.86 .786 .778 r
.50119 .57776 .50971 .58615 .52521 .56668 .51679 .55764 Metetra
.845 .778 .787 r
.51679 .55764 .52521 .56668 .54067 .54885 .53233 .53918 Metetra
.818 .768 .806 r
.53233 .53918 .54067 .54885 .55612 .53384 .54785 .52364 Metetra
.776 .754 .833 r
.54785 .52364 .55612 .53384 .57161 .52236 .56341 .51182 Metetra
.716 .734 .865 r
.56341 .51182 .57161 .52236 .58719 .51463 .57905 .50397 Metetra
.639 .706 .895 r
.57905 .50397 .58719 .51463 .60292 .5104 .59483 .49983 Metetra
.551 .673 .918 r
.59483 .49983 .60292 .5104 .61883 .5091 .61079 .49878 Metetra
.466 .639 .931 r
.61079 .49878 .61883 .5091 .63493 .50998 .62695 .50001 Metetra
.399 .614 .937 r
.62695 .50001 .63493 .50998 .65124 .51223 .64331 .50264 Metetra
.361 .603 .941 r
.64331 .50264 .65124 .51223 .66773 .5151 .65986 .50587 Metetra
.353 .609 .946 r
.65986 .50587 .66773 .5151 .68439 .51799 .67659 .50904 Metetra
.37 .627 .953 r
.67659 .50904 .68439 .51799 .70121 .52044 .69347 .51169 Metetra
.404 .652 .959 r
.69347 .51169 .70121 .52044 .71816 .52214 .71048 .51351 Metetra
.445 .678 .963 r
.71048 .51351 .71816 .52214 .73521 .52294 .7276 .51435 Metetra
.486 .701 .964 r
.7276 .51435 .73521 .52294 .75236 .52279 .74481 .51417 Metetra
.522 .719 .962 r
.74481 .51417 .75236 .52279 .76959 .52174 .76211 .51304 Metetra
.553 .732 .959 r
.76211 .51304 .76959 .52174 .78691 .51986 .77949 .51106 Metetra
.578 .742 .954 r
.77949 .51106 .78691 .51986 .8043 .51726 .79695 .50833 Metetra
.597 .748 .949 r
.79695 .50833 .8043 .51726 .82177 .51405 .81448 .505 Metetra
.612 .753 .945 r
.81448 .505 .82177 .51405 .83932 .51036 .8321 .50117 Metetra
.622 .755 .942 r
.8321 .50117 .83932 .51036 .85695 .50627 .8498 .49695 Metetra
.63 .757 .938 r
.8498 .49695 .85695 .50627 .87468 .50186 .8676 .49243 Metetra
.636 .758 .936 r
.8676 .49243 .87468 .50186 .89249 .49721 .88548 .48767 Metetra
.64 .758 .934 r
.88548 .48767 .89249 .49721 .9104 .49237 .90347 .48273 Metetra
.646 .762 .934 r
.26413 .69713 .2737 .70489 .28825 .7007 .27873 .69289 Metetra
.646 .762 .934 r
.27873 .69289 .28825 .7007 .30288 .69648 .29341 .68863 Metetra
.646 .762 .934 r
.29341 .68863 .30288 .69648 .31759 .69224 .30817 .68435 Metetra
.646 .762 .934 r
.30817 .68435 .31759 .69224 .33239 .68797 .32302 .68004 Metetra
.646 .763 .934 r
.32302 .68004 .33239 .68797 .34726 .68367 .33795 .67569 Metetra
.647 .763 .933 r
.33795 .67569 .34726 .68367 .36223 .67929 .35296 .67128 Metetra
.652 .764 .932 r
.35296 .67128 .36223 .67929 .37728 .67471 .36807 .6667 Metetra
.662 .768 .928 r
.36807 .6667 .37728 .67471 .39244 .66967 .38328 .66171 Metetra
.685 .776 .92 r
.38328 .66171 .39244 .66967 .40771 .66368 .3986 .65582 Metetra
.72 .786 .904 r
.3986 .65582 .40771 .66368 .42309 .65603 .41404 .64834 Metetra
.764 .797 .88 r
.41404 .64834 .42309 .65603 .4386 .64596 .42961 .63844 Metetra
.806 .804 .849 r
.42961 .63844 .4386 .64596 .4542 .6329 .44529 .62545 Metetra
.839 .804 .818 r
.44529 .62545 .4542 .6329 .46987 .61672 .46105 .60917 Metetra
.859 .8 .792 r
.46105 .60917 .46987 .61672 .48555 .59796 .47683 .59008 Metetra
.867 .794 .777 r
.47683 .59008 .48555 .59796 .50119 .57776 .49258 .56933 Metetra
.863 .787 .774 r
.49258 .56933 .50119 .57776 .51679 .55764 .50828 .54853 Metetra
.849 .778 .782 r
.50828 .54853 .51679 .55764 .53233 .53918 .52391 .52937 Metetra
.822 .767 .801 r
.52391 .52937 .53233 .53918 .54785 .52364 .53951 .51326 Metetra
.779 .753 .83 r
.53951 .51326 .54785 .52364 .56341 .51182 .55513 .50108 Metetra
.717 .732 .863 r
.55513 .50108 .56341 .51182 .57905 .50397 .57083 .49311 Metetra
.635 .703 .895 r
.57083 .49311 .57905 .50397 .59483 .49983 .58667 .48908 Metetra
.541 .666 .918 r
.58667 .48908 .59483 .49983 .61079 .49878 .60268 .48831 Metetra
.45 .629 .93 r
.60268 .48831 .61079 .49878 .62695 .50001 .61889 .48992 Metetra
.379 .602 .935 r
.61889 .48992 .62695 .50001 .64331 .50264 .6353 .49296 Metetra
.34 .592 .938 r
.6353 .49296 .64331 .50264 .65986 .50587 .65192 .49657 Metetra
.335 .6 .944 r
.65192 .49657 .65986 .50587 .67659 .50904 .66871 .50005 Metetra
.357 .621 .952 r
.66871 .50005 .67659 .50904 .69347 .51169 .68565 .50291 Metetra
.395 .649 .959 r
.68565 .50291 .69347 .51169 .71048 .51351 .70272 .50484 Metetra
.44 .677 .964 r
.70272 .50484 .71048 .51351 .7276 .51435 .7199 .50571 Metetra
.485 .702 .965 r
.7199 .50571 .7276 .51435 .74481 .51417 .73718 .5055 Metetra
.524 .72 .963 r
.73718 .5055 .74481 .51417 .76211 .51304 .75455 .50428 Metetra
.556 .734 .959 r
.75455 .50428 .76211 .51304 .77949 .51106 .77199 .50217 Metetra
.581 .744 .954 r
.77199 .50217 .77949 .51106 .79695 .50833 .78951 .49931 Metetra
.6 .75 .949 r
.78951 .49931 .79695 .50833 .81448 .505 .80711 .49583 Metetra
.614 .754 .945 r
.80711 .49583 .81448 .505 .8321 .50117 .8248 .49187 Metetra
.624 .756 .941 r
.8248 .49187 .8321 .50117 .8498 .49695 .84257 .48752 Metetra
.632 .757 .938 r
.84257 .48752 .8498 .49695 .8676 .49243 .86044 .48288 Metetra
.637 .758 .936 r
.86044 .48288 .8676 .49243 .88548 .48767 .8784 .47801 Metetra
.641 .758 .934 r
.8784 .47801 .88548 .48767 .90347 .48273 .89646 .47297 Metetra
.646 .762 .934 r
.25446 .68929 .26413 .69713 .27873 .69289 .26911 .68501 Metetra
.646 .762 .934 r
.26911 .68501 .27873 .69289 .29341 .68863 .28384 .6807 Metetra
.646 .762 .934 r
.28384 .6807 .29341 .68863 .30817 .68435 .29865 .67637 Metetra
.646 .762 .934 r
.29865 .67637 .30817 .68435 .32302 .68004 .31355 .67202 Metetra
.646 .762 .934 r
.31355 .67202 .32302 .68004 .33795 .67569 .32853 .66763 Metetra
.647 .763 .933 r
.32853 .66763 .33795 .67569 .35296 .67128 .3436 .66319 Metetra
.65 .764 .932 r
.3436 .66319 .35296 .67128 .36807 .6667 .35875 .6586 Metetra
.66 .767 .929 r
.35875 .6586 .36807 .6667 .38328 .66171 .37401 .65364 Metetra
.681 .774 .922 r
.37401 .65364 .38328 .66171 .3986 .65582 .38939 .64786 Metetra
.715 .785 .907 r
.38939 .64786 .3986 .65582 .41404 .64834 .40489 .64055 Metetra
.76 .797 .883 r
.40489 .64055 .41404 .64834 .42961 .63844 .42052 .63084 Metetra
.804 .804 .851 r
.42052 .63084 .42961 .63844 .44529 .62545 .43629 .61797 Metetra
.839 .805 .818 r
.43629 .61797 .44529 .62545 .46105 .60917 .45214 .60161 Metetra
.861 .801 .791 r
.45214 .60161 .46105 .60917 .47683 .59008 .46802 .58219 Metetra
.87 .795 .774 r
.46802 .58219 .47683 .59008 .49258 .56933 .48388 .56086 Metetra
.867 .787 .77 r
.48388 .56086 .49258 .56933 .50828 .54853 .49968 .53934 Metetra
.853 .778 .777 r
.49968 .53934 .50828 .54853 .52391 .52937 .51541 .51943 Metetra
.826 .767 .797 r
.51541 .51943 .52391 .52937 .53951 .51326 .53109 .5027 Metetra
.782 .752 .826 r
.53109 .5027 .53951 .51326 .55513 .50108 .54678 .49013 Metetra
.717 .73 .861 r
.54678 .49013 .55513 .50108 .57083 .49311 .56254 .48203 Metetra
.631 .699 .894 r
.56254 .48203 .57083 .49311 .58667 .48908 .57842 .47813 Metetra
.53 .659 .918 r
.57842 .47813 .58667 .48908 .60268 .48831 .59448 .47768 Metetra
.431 .618 .929 r
.59448 .47768 .60268 .48831 .61889 .48992 .61074 .47971 Metetra
.356 .589 .933 r
.61074 .47971 .61889 .48992 .6353 .49296 .62721 .4832 Metetra
.317 .579 .936 r
.62721 .4832 .6353 .49296 .65192 .49657 .64389 .48722 Metetra
.315 .59 .942 r
.64389 .48722 .65192 .49657 .66871 .50005 .66074 .49103 Metetra
.342 .615 .951 r
.66074 .49103 .66871 .50005 .68565 .50291 .67774 .4941 Metetra
.386 .646 .96 r
.67774 .4941 .68565 .50291 .70272 .50484 .69488 .49615 Metetra
.436 .677 .965 r
.69488 .49615 .70272 .50484 .7199 .50571 .71212 .49703 Metetra
.484 .703 .966 r
.71212 .49703 .7199 .50571 .73718 .5055 .72947 .49676 Metetra
.525 .722 .963 r
.72947 .49676 .73718 .5055 .75455 .50428 .74689 .49544 Metetra
.558 .736 .959 r
.74689 .49544 .75455 .50428 .77199 .50217 .7644 .49319 Metetra
.584 .745 .954 r
.7644 .49319 .77199 .50217 .78951 .49931 .78199 .49019 Metetra
.603 .751 .948 r
.78199 .49019 .78951 .49931 .80711 .49583 .79966 .48656 Metetra
.617 .755 .944 r
.79966 .48656 .80711 .49583 .8248 .49187 .81741 .48245 Metetra
.627 .757 .94 r
.81741 .48245 .8248 .49187 .84257 .48752 .83526 .47797 Metetra
.634 .758 .937 r
.83526 .47797 .84257 .48752 .86044 .48288 .85319 .47321 Metetra
.639 .758 .935 r
.85319 .47321 .86044 .48288 .8784 .47801 .87123 .46823 Metetra
.642 .759 .933 r
.87123 .46823 .8784 .47801 .89646 .47297 .88937 .46309 Metetra
.646 .762 .934 r
.2447 .68137 .25446 .68929 .26911 .68501 .25939 .67704 Metetra
.646 .762 .934 r
.25939 .67704 .26911 .68501 .28384 .6807 .27417 .67269 Metetra
.646 .762 .934 r
.27417 .67269 .28384 .6807 .29865 .67637 .28903 .66832 Metetra
.646 .762 .934 r
.28903 .66832 .29865 .67637 .31355 .67202 .30398 .66392 Metetra
.646 .762 .934 r
.30398 .66392 .31355 .67202 .32853 .66763 .31901 .65949 Metetra
.647 .763 .933 r
.31901 .65949 .32853 .66763 .3436 .66319 .33413 .65501 Metetra
.65 .764 .933 r
.33413 .65501 .3436 .66319 .35875 .6586 .34934 .6504 Metetra
.658 .767 .93 r
.34934 .6504 .35875 .6586 .37401 .65364 .36465 .64548 Metetra
.677 .773 .923 r
.36465 .64548 .37401 .65364 .38939 .64786 .38008 .63981 Metetra
.71 .784 .909 r
.38008 .63981 .38939 .64786 .40489 .64055 .39564 .63268 Metetra
.755 .796 .886 r
.39564 .63268 .40489 .64055 .42052 .63084 .41134 .62318 Metetra
.802 .804 .853 r
.41134 .62318 .42052 .63084 .43629 .61797 .42718 .61044 Metetra
.839 .806 .819 r
.42718 .61044 .43629 .61797 .45214 .60161 .44312 .59404 Metetra
.863 .802 .789 r
.44312 .59404 .45214 .60161 .46802 .58219 .45911 .5743 Metetra
.873 .795 .771 r
.45911 .5743 .46802 .58219 .48388 .56086 .47508 .55238 Metetra
.871 .787 .765 r
.47508 .55238 .48388 .56086 .49968 .53934 .49099 .53007 Metetra
.858 .778 .772 r
.49099 .53007 .49968 .53934 .51541 .51943 .50682 .50936 Metetra
.83 .767 .792 r
.50682 .50936 .51541 .51943 .53109 .5027 .52259 .49195 Metetra
.785 .751 .822 r
.52259 .49195 .53109 .5027 .54678 .49013 .53835 .47895 Metetra
.718 .728 .859 r
.53835 .47895 .54678 .49013 .56254 .48203 .55416 .47072 Metetra
.626 .694 .894 r
.55416 .47072 .56254 .48203 .57842 .47813 .5701 .46697 Metetra
.517 .651 .918 r
.5701 .46697 .57842 .47813 .59448 .47768 .5862 .46688 Metetra
.411 .606 .928 r
.5862 .46688 .59448 .47768 .61074 .47971 .60252 .46939 Metetra
.331 .575 .93 r
.60252 .46939 .61074 .47971 .62721 .4832 .61904 .47337 Metetra
.292 .565 .932 r
.61904 .47337 .62721 .4832 .64389 .48722 .63577 .47783 Metetra
.294 .579 .939 r
.63577 .47783 .64389 .48722 .66074 .49103 .65268 .48198 Metetra
.327 .608 .95 r
.65268 .48198 .66074 .49103 .67774 .4941 .66975 .48527 Metetra
.378 .644 .96 r
.66975 .48527 .67774 .4941 .69488 .49615 .68695 .48742 Metetra
.433 .677 .966 r
.68695 .48742 .69488 .49615 .71212 .49703 .70426 .4883 Metetra
.484 .704 .967 r
.70426 .4883 .71212 .49703 .72947 .49676 .72166 .48796 Metetra
.527 .724 .964 r
.72166 .48796 .72947 .49676 .74689 .49544 .73915 .48652 Metetra
.561 .738 .959 r
.73915 .48652 .74689 .49544 .7644 .49319 .75673 .48412 Metetra
.587 .747 .953 r
.75673 .48412 .7644 .49319 .78199 .49019 .77438 .48096 Metetra
.606 .752 .948 r
.77438 .48096 .78199 .49019 .79966 .48656 .79212 .47717 Metetra
.619 .755 .943 r
.79212 .47717 .79966 .48656 .81741 .48245 .80994 .47292 Metetra
.629 .757 .94 r
.80994 .47292 .81741 .48245 .83526 .47797 .82786 .4683 Metetra
.635 .758 .937 r
.82786 .4683 .83526 .47797 .85319 .47321 .84587 .46341 Metetra
.64 .759 .934 r
.84587 .46341 .85319 .47321 .87123 .46823 .86398 .45832 Metetra
.643 .759 .933 r
.86398 .45832 .87123 .46823 .88937 .46309 .88219 .45308 Metetra
.646 .762 .934 r
.23483 .67336 .2447 .68137 .25939 .67704 .24958 .66899 Metetra
.646 .762 .934 r
.24958 .66899 .25939 .67704 .27417 .67269 .2644 .6646 Metetra
.646 .762 .934 r
.2644 .6646 .27417 .67269 .28903 .66832 .27932 .66018 Metetra
.646 .762 .934 r
.27932 .66018 .28903 .66832 .30398 .66392 .29431 .65573 Metetra
.646 .762 .934 r
.29431 .65573 .30398 .66392 .31901 .65949 .3094 .65125 Metetra
.646 .763 .934 r
.3094 .65125 .31901 .65949 .33413 .65501 .32457 .64674 Metetra
.649 .763 .933 r
.32457 .64674 .33413 .65501 .34934 .6504 .33983 .64211 Metetra
.656 .766 .931 r
.33983 .64211 .34934 .6504 .36465 .64548 .35519 .63721 Metetra
.673 .772 .925 r
.35519 .63721 .36465 .64548 .38008 .63981 .37067 .63164 Metetra
.705 .783 .912 r
.37067 .63164 .38008 .63981 .39564 .63268 .38628 .6247 Metetra
.75 .795 .889 r
.38628 .6247 .39564 .63268 .41134 .62318 .40205 .61543 Metetra
.799 .804 .856 r
.40205 .61543 .41134 .62318 .42718 .61044 .41796 .60288 Metetra
.839 .807 .82 r
.41796 .60288 .42718 .61044 .44312 .59404 .434 .58646 Metetra
.864 .803 .788 r
.434 .58646 .44312 .59404 .45911 .5743 .4501 .56641 Metetra
.876 .796 .768 r
.4501 .56641 .45911 .5743 .47508 .55238 .46619 .54387 Metetra
.875 .787 .76 r
.46619 .54387 .47508 .55238 .49099 .53007 .48222 .52074 Metetra
.862 .778 .767 r
.48222 .52074 .49099 .53007 .50682 .50936 .49815 .49915 Metetra
.835 .766 .787 r
.49815 .49915 .50682 .50936 .52259 .49195 .51401 .481 Metetra
.789 .75 .818 r
.51401 .481 .52259 .49195 .53835 .47895 .52983 .46753 Metetra
.718 .726 .856 r
.52983 .46753 .53835 .47895 .55416 .47072 .5457 .45917 Metetra
.62 .689 .893 r
.5457 .45917 .55416 .47072 .5701 .46697 .56169 .45559 Metetra
.502 .641 .917 r
.56169 .45559 .5701 .46697 .5862 .46688 .57784 .4559 Metetra
.387 .592 .926 r
.57784 .4559 .5862 .46688 .60252 .46939 .5942 .45894 Metetra
.302 .558 .925 r
.5942 .45894 .60252 .46939 .61904 .47337 .61078 .46347 Metetra
.264 .55 .927 r
.61078 .46347 .61904 .47337 .63577 .47783 .62757 .4684 Metetra
.271 .567 .936 r
.62757 .4684 .63577 .47783 .65268 .48198 .64454 .47291 Metetra
.311 .601 .949 r
.64454 .47291 .65268 .48198 .66975 .48527 .66167 .47643 Metetra
.369 .641 .96 r
.66167 .47643 .66975 .48527 .68695 .48742 .67893 .47866 Metetra
.43 .678 .967 r
.67893 .47866 .68695 .48742 .70426 .4883 .6963 .47953 Metetra
.485 .706 .967 r
.6963 .47953 .70426 .4883 .72166 .48796 .71377 .4791 Metetra
.53 .727 .964 r
.71377 .4791 .72166 .48796 .73915 .48652 .73133 .47751 Metetra
.565 .74 .959 r
.73133 .47751 .73915 .48652 .75673 .48412 .74896 .47495 Metetra
.591 .749 .953 r
.74896 .47495 .75673 .48412 .77438 .48096 .76668 .47161 Metetra
.609 .754 .947 r
.76668 .47161 .77438 .48096 .79212 .47717 .78449 .46767 Metetra
.622 .756 .942 r
.78449 .46767 .79212 .47717 .80994 .47292 .80238 .46325 Metetra
.631 .758 .939 r
.80238 .46325 .80994 .47292 .82786 .4683 .82037 .4585 Metetra
.637 .759 .936 r
.82037 .4585 .82786 .4683 .84587 .46341 .83845 .45349 Metetra
.641 .759 .934 r
.83845 .45349 .84587 .46341 .86398 .45832 .85664 .44828 Metetra
.644 .759 .932 r
.85664 .44828 .86398 .45832 .88219 .45308 .87493 .44294 Metetra
.646 .762 .934 r
.22486 .66527 .23483 .67336 .24958 .66899 .23965 .66086 Metetra
.646 .762 .934 r
.23965 .66086 .24958 .66899 .2644 .6646 .25453 .65642 Metetra
.646 .762 .934 r
.25453 .65642 .2644 .6646 .27932 .66018 .2695 .65195 Metetra
.646 .762 .934 r
.2695 .65195 .27932 .66018 .29431 .65573 .28454 .64746 Metetra
.646 .762 .934 r
.28454 .64746 .29431 .65573 .3094 .65125 .29968 .64293 Metetra
.646 .763 .934 r
.29968 .64293 .3094 .65125 .32457 .64674 .3149 .63837 Metetra
.648 .763 .933 r
.3149 .63837 .32457 .64674 .33983 .64211 .33022 .63373 Metetra
.654 .765 .931 r
.33022 .63373 .33983 .64211 .35519 .63721 .34563 .62885 Metetra
.669 .771 .926 r
.34563 .62885 .35519 .63721 .37067 .63164 .36116 .62337 Metetra
.699 .781 .914 r
.36116 .62337 .37067 .63164 .38628 .6247 .37683 .61662 Metetra
.745 .794 .892 r
.37683 .61662 .38628 .6247 .40205 .61543 .39265 .60761 Metetra
.795 .805 .859 r
.39265 .60761 .40205 .61543 .41796 .60288 .40864 .59527 Metetra
.838 .808 .821 r
.40864 .59527 .41796 .60288 .434 .58646 .42477 .57888 Metetra
.866 .804 .787 r
.42477 .57888 .434 .58646 .4501 .56641 .44098 .55854 Metetra
.879 .797 .764 r
.44098 .55854 .4501 .56641 .46619 .54387 .4572 .53537 Metetra
.879 .788 .755 r
.4572 .53537 .46619 .54387 .48222 .52074 .47335 .51134 Metetra
.867 .778 .761 r
.47335 .51134 .48222 .52074 .49815 .49915 .48939 .4888 Metetra
.839 .766 .781 r
.48939 .4888 .49815 .49915 .51401 .481 .50534 .46984 Metetra
.793 .749 .813 r
.50534 .46984 .51401 .481 .52983 .46753 .52124 .45586 Metetra
.719 .723 .853 r
.52124 .45586 .52983 .46753 .5457 .45917 .53717 .44736 Metetra
.614 .684 .892 r
.53717 .44736 .5457 .45917 .56169 .45559 .5532 .44398 Metetra
.486 .63 .916 r
.5532 .44398 .56169 .45559 .57784 .4559 .5694 .44475 Metetra
.36 .575 .922 r
.5694 .44475 .57784 .4559 .5942 .45894 .58581 .44837 Metetra
.27 .538 .919 r
.58581 .44837 .5942 .45894 .61078 .46347 .60244 .4535 Metetra
.232 .532 .921 r
.60244 .4535 .61078 .46347 .62757 .4684 .61927 .45895 Metetra
.245 .554 .931 r
.61927 .45895 .62757 .4684 .64454 .47291 .6363 .46383 Metetra
.294 .594 .947 r
.6363 .46383 .64454 .47291 .66167 .47643 .6535 .46757 Metetra
.36 .639 .961 r
.6535 .46757 .66167 .47643 .67893 .47866 .67082 .46988 Metetra
.428 .679 .968 r
.67082 .46988 .67893 .47866 .6963 .47953 .68826 .47072 Metetra
.487 .709 .968 r
.68826 .47072 .6963 .47953 .71377 .4791 .70579 .47017 Metetra
.534 .73 .964 r
.70579 .47017 .71377 .4791 .73133 .47751 .72341 .46842 Metetra
.569 .743 .958 r
.72341 .46842 .73133 .47751 .74896 .47495 .74111 .46568 Metetra
.595 .75 .952 r
.74111 .46568 .74896 .47495 .76668 .47161 .75889 .46215 Metetra
.612 .755 .946 r
.75889 .46215 .76668 .47161 .78449 .46767 .77677 .45803 Metetra
.625 .757 .942 r
.77677 .45803 .78449 .46767 .80238 .46325 .79473 .45346 Metetra
.633 .758 .938 r
.79473 .45346 .80238 .46325 .82037 .4585 .81279 .44856 Metetra
.639 .759 .935 r
.81279 .44856 .82037 .4585 .83845 .45349 .83095 .44343 Metetra
.643 .759 .933 r
.83095 .44343 .83845 .45349 .85664 .44828 .84921 .43812 Metetra
.645 .759 .932 r
.84921 .43812 .85664 .44828 .87493 .44294 .86759 .43268 Metetra
.646 .762 .934 r
.21479 .6571 .22486 .66527 .23965 .66086 .22963 .65264 Metetra
.646 .762 .934 r
.22963 .65264 .23965 .66086 .25453 .65642 .24456 .64815 Metetra
.646 .762 .934 r
.24456 .64815 .25453 .65642 .2695 .65195 .25957 .64363 Metetra
.646 .762 .934 r
.25957 .64363 .2695 .65195 .28454 .64746 .27467 .63909 Metetra
.646 .762 .934 r
.27467 .63909 .28454 .64746 .29968 .64293 .28986 .63452 Metetra
.646 .762 .934 r
.28986 .63452 .29968 .64293 .3149 .63837 .30513 .62992 Metetra
.647 .763 .933 r
.30513 .62992 .3149 .63837 .33022 .63373 .3205 .62524 Metetra
.652 .765 .932 r
.3205 .62524 .33022 .63373 .34563 .62885 .33597 .62037 Metetra
.665 .77 .927 r
.33597 .62037 .34563 .62885 .36116 .62337 .35155 .61499 Metetra
.694 .779 .917 r
.35155 .61499 .36116 .62337 .37683 .61662 .36727 .60843 Metetra
.739 .793 .896 r
.36727 .60843 .37683 .61662 .39265 .60761 .38315 .59969 Metetra
.791 .805 .863 r
.38315 .59969 .39265 .60761 .40864 .59527 .39921 .58761 Metetra
.837 .809 .823 r
.39921 .58761 .40864 .59527 .42477 .57888 .41543 .5713 Metetra
.868 .805 .786 r
.41543 .5713 .42477 .57888 .44098 .55854 .43175 .5507 Metetra
.883 .798 .761 r
.43175 .5507 .44098 .55854 .4572 .53537 .4481 .52688 Metetra
.884 .788 .75 r
.4481 .52688 .4572 .53537 .47335 .51134 .46438 .5019 Metetra
.872 .778 .755 r
.46438 .5019 .47335 .51134 .48939 .4888 .48055 .47831 Metetra
.844 .765 .775 r
.48055 .47831 .48939 .4888 .50534 .46984 .49659 .45845 Metetra
.796 .748 .808 r
.49659 .45845 .50534 .46984 .52124 .45586 .51257 .4439 Metetra
.719 .721 .85 r
.51257 .4439 .52124 .45586 .53717 .44736 .52856 .43525 Metetra
.607 .677 .891 r
.52856 .43525 .53717 .44736 .5532 .44398 .54463 .43212 Metetra
.466 .618 .915 r
.54463 .43212 .5532 .44398 .5694 .44475 .56087 .43341 Metetra
.329 .556 .918 r
.56087 .43341 .5694 .44475 .58581 .44837 .57732 .4377 Metetra
.233 .516 .912 r
.57732 .4377 .58581 .44837 .60244 .4535 .594 .44348 Metetra
.198 .511 .913 r
.594 .44348 .60244 .4535 .61927 .45895 .61089 .44949 Metetra
.218 .539 .926 r
.61089 .44949 .61927 .45895 .6363 .46383 .62798 .45476 Metetra
.277 .586 .945 r
.62798 .45476 .6363 .46383 .6535 .46757 .64523 .45872 Metetra
.353 .637 .961 r
.64523 .45872 .6535 .46757 .67082 .46988 .66262 .46109 Metetra
.427 .68 .969 r
.66262 .46109 .67082 .46988 .68826 .47072 .68012 .46186 Metetra
.49 .712 .969 r
.68012 .46186 .68826 .47072 .70579 .47017 .69771 .46117 Metetra
.539 .733 .964 r
.69771 .46117 .70579 .47017 .72341 .46842 .7154 .45923 Metetra
.574 .745 .958 r
.7154 .45923 .72341 .46842 .74111 .46568 .73316 .45629 Metetra
.599 .752 .951 r
.73316 .45629 .74111 .46568 .75889 .46215 .75101 .45257 Metetra
.616 .756 .945 r
.75101 .45257 .75889 .46215 .77677 .45803 .76895 .44827 Metetra
.628 .758 .941 r
.76895 .44827 .77677 .45803 .79473 .45346 .78699 .44354 Metetra
.635 .759 .937 r
.78699 .44354 .79473 .45346 .81279 .44856 .80512 .4385 Metetra
.64 .759 .934 r
.80512 .4385 .81279 .44856 .83095 .44343 .82336 .43323 Metetra
.644 .759 .933 r
.82336 .43323 .83095 .44343 .84921 .43812 .8417 .42781 Metetra
.646 .759 .931 r
.8417 .42781 .84921 .43812 .86759 .43268 .86015 .42228 Metetra
.646 .762 .934 r
.2046 .64884 .21479 .6571 .22963 .65264 .2195 .64433 Metetra
.646 .762 .934 r
.2195 .64433 .22963 .65264 .24456 .64815 .23447 .63979 Metetra
.646 .762 .934 r
.23447 .63979 .24456 .64815 .25957 .64363 .24954 .63523 Metetra
.646 .762 .934 r
.24954 .63523 .25957 .64363 .27467 .63909 .26469 .63064 Metetra
.646 .762 .934 r
.26469 .63064 .27467 .63909 .28986 .63452 .27993 .62602 Metetra
.646 .762 .934 r
.27993 .62602 .28986 .63452 .30513 .62992 .29526 .62137 Metetra
.647 .763 .933 r
.29526 .62137 .30513 .62992 .3205 .62524 .31068 .61666 Metetra
.651 .764 .932 r
.31068 .61666 .3205 .62524 .33597 .62037 .3262 .6118 Metetra
.662 .768 .929 r
.3262 .6118 .33597 .62037 .35155 .61499 .34183 .60648 Metetra
.688 .778 .919 r
.34183 .60648 .35155 .61499 .36727 .60843 .3576 .60012 Metetra
.732 .791 .9 r
.3576 .60012 .36727 .60843 .38315 .59969 .37354 .59168 Metetra
.786 .804 .867 r
.37354 .59168 .38315 .59969 .39921 .58761 .38967 .5799 Metetra
.835 .81 .826 r
.38967 .5799 .39921 .58761 .41543 .5713 .40598 .56372 Metetra
.869 .807 .786 r
.40598 .56372 .41543 .5713 .43175 .5507 .42242 .5429 Metetra
.886 .799 .758 r
.42242 .5429 .43175 .5507 .4481 .52688 .4389 .51841 Metetra
.888 .789 .745 r
.4389 .51841 .4481 .52688 .46438 .5019 .45532 .49241 Metetra
.877 .778 .748 r
.45532 .49241 .46438 .5019 .48055 .47831 .47161 .46768 Metetra
.85 .765 .768 r
.47161 .46768 .48055 .47831 .49659 .45845 .48776 .44682 Metetra
.801 .747 .803 r
.48776 .44682 .49659 .45845 .51257 .4439 .50382 .43165 Metetra
.72 .718 .847 r
.50382 .43165 .51257 .4439 .52856 .43525 .51986 .42284 Metetra
.598 .67 .89 r
.51986 .42284 .52856 .43525 .54463 .43212 .53598 .41999 Metetra
.444 .603 .913 r
.53598 .41999 .54463 .43212 .56087 .43341 .55226 .42188 Metetra
.293 .533 .911 r
.55226 .42188 .56087 .43341 .57732 .4377 .56875 .42691 Metetra
.192 .49 .901 r
.56875 .42691 .57732 .4377 .594 .44348 .58548 .43342 Metetra
.159 .488 .902 r
.58548 .43342 .594 .44348 .61089 .44949 .60242 .44003 Metetra
.188 .523 .919 r
.60242 .44003 .61089 .44949 .62798 .45476 .61957 .44571 Metetra
.259 .579 .943 r
.61957 .44571 .62798 .45476 .64523 .45872 .63688 .44987 Metetra
.345 .636 .962 r
.63688 .44987 .64523 .45872 .66262 .46109 .65433 .45228 Metetra
.427 .683 .97 r
.65433 .45228 .66262 .46109 .68012 .46186 .67189 .45296 Metetra
.494 .715 .97 r
.67189 .45296 .68012 .46186 .69771 .46117 .68954 .4521 Metetra
.544 .736 .964 r
.68954 .4521 .69771 .46117 .7154 .45923 .70729 .44995 Metetra
.579 .748 .957 r
.70729 .44995 .7154 .45923 .73316 .45629 .72512 .44679 Metetra
.604 .754 .95 r
.72512 .44679 .73316 .45629 .75101 .45257 .74304 .44286 Metetra
.62 .757 .944 r
.74304 .44286 .75101 .45257 .76895 .44827 .76105 .43837 Metetra
.631 .759 .94 r
.76105 .43837 .76895 .44827 .78699 .44354 .77915 .43347 Metetra
.638 .759 .936 r
.77915 .43347 .78699 .44354 .80512 .4385 .79736 .42829 Metetra
.642 .759 .934 r
.79736 .42829 .80512 .4385 .82336 .43323 .81567 .4229 Metetra
.645 .759 .932 r
.81567 .4229 .82336 .43323 .8417 .42781 .8341 .41738 Metetra
.647 .759 .931 r
.8341 .41738 .8417 .42781 .86015 .42228 .85263 .41174 Metetra
.646 .762 .934 r
.19431 .64049 .2046 .64884 .2195 .64433 .20925 .63593 Metetra
.646 .762 .934 r
.20925 .63593 .2195 .64433 .23447 .63979 .22428 .63135 Metetra
.646 .762 .934 r
.22428 .63135 .23447 .63979 .24954 .63523 .2394 .62674 Metetra
.646 .762 .934 r
.2394 .62674 .24954 .63523 .26469 .63064 .2546 .6221 Metetra
.646 .762 .934 r
.2546 .6221 .26469 .63064 .27993 .62602 .2699 .61743 Metetra
.646 .762 .934 r
.2699 .61743 .27993 .62602 .29526 .62137 .28528 .61273 Metetra
.647 .763 .934 r
.28528 .61273 .29526 .62137 .31068 .61666 .30075 .60799 Metetra
.649 .764 .933 r
.30075 .60799 .31068 .61666 .3262 .6118 .31633 .60311 Metetra
.659 .767 .93 r
.31633 .60311 .3262 .6118 .34183 .60648 .33201 .59786 Metetra
.682 .776 .922 r
.33201 .59786 .34183 .60648 .3576 .60012 .34783 .59168 Metetra
.724 .79 .904 r
.34783 .59168 .3576 .60012 .37354 .59168 .36382 .58356 Metetra
.78 .804 .872 r
.36382 .58356 .37354 .59168 .38967 .5799 .38001 .57213 Metetra
.833 .811 .829 r
.38001 .57213 .38967 .5799 .40598 .56372 .39641 .55615 Metetra
.87 .808 .786 r
.39641 .55615 .40598 .56372 .42242 .5429 .41297 .53515 Metetra
.889 .8 .755 r
.41297 .53515 .42242 .5429 .4389 .51841 .42959 .50999 Metetra
.893 .789 .739 r
.42959 .50999 .4389 .51841 .45532 .49241 .44616 .4829 Metetra
.882 .778 .742 r
.44616 .4829 .45532 .49241 .47161 .46768 .46259 .4569 Metetra
.855 .764 .761 r
.46259 .4569 .47161 .46768 .48776 .44682 .47885 .43493 Metetra
.805 .745 .797 r
.47885 .43493 .48776 .44682 .50382 .43165 .49499 .41907 Metetra
.72 .714 .844 r
.49499 .41907 .50382 .43165 .51986 .42284 .51109 .41009 Metetra
.589 .661 .888 r
.51109 .41009 .51986 .42284 .53598 .41999 .52725 .40757 Metetra
.418 .585 .91 r
.52725 .40757 .53598 .41999 .55226 .42188 .54357 .41015 Metetra
.252 .507 .902 r
.54357 .41015 .55226 .42188 .56875 .42691 .56009 .41602 Metetra
.144 .46 .887 r
.56009 .41602 .56875 .42691 .58548 .43342 .57686 .42334 Metetra
.116 .462 .889 r
.57686 .42334 .58548 .43342 .60242 .44003 .59386 .43059 Metetra
.156 .506 .911 r
.59386 .43059 .60242 .44003 .61957 .44571 .61105 .43669 Metetra
.241 .572 .94 r
.61105 .43669 .61957 .44571 .63688 .44987 .62843 .44105 Metetra
.339 .636 .962 r
.62843 .44105 .63688 .44987 .65433 .45228 .64593 .44346 Metetra
.429 .686 .971 r
.64593 .44346 .65433 .45228 .67189 .45296 .66356 .44401 Metetra
.499 .72 .97 r
.66356 .44401 .67189 .45296 .68954 .4521 .68127 .44294 Metetra
.55 .739 .964 r
.68127 .44294 .68954 .4521 .70729 .44995 .69908 .44055 Metetra
.585 .75 .956 r
.69908 .44055 .70729 .44995 .72512 .44679 .71698 .43715 Metetra
.608 .756 .949 r
.71698 .43715 .72512 .44679 .74304 .44286 .73497 .43301 Metetra
.624 .758 .943 r
.73497 .43301 .74304 .44286 .76105 .43837 .75305 .42832 Metetra
.633 .759 .938 r
.75305 .42832 .76105 .43837 .77915 .43347 .77122 .42326 Metetra
.64 .759 .935 r
.77122 .42326 .77915 .43347 .79736 .42829 .78951 .41794 Metetra
.643 .759 .933 r
.78951 .41794 .79736 .42829 .81567 .4229 .8079 .41243 Metetra
.646 .759 .932 r
.8079 .41243 .81567 .4229 .8341 .41738 .8264 .4068 Metetra
.647 .759 .931 r
.8264 .4068 .8341 .41738 .85263 .41174 .84502 .40108 Metetra
.646 .762 .934 r
.18391 .63206 .19431 .64049 .20925 .63593 .1989 .62745 Metetra
.646 .762 .934 r
.1989 .62745 .20925 .63593 .22428 .63135 .21398 .62281 Metetra
.646 .762 .934 r
.21398 .62281 .22428 .63135 .2394 .62674 .22915 .61815 Metetra
.646 .762 .934 r
.22915 .61815 .2394 .62674 .2546 .6221 .24441 .61346 Metetra
.646 .762 .934 r
.24441 .61346 .2546 .6221 .2699 .61743 .25975 .60874 Metetra
.646 .762 .934 r
.25975 .60874 .2699 .61743 .28528 .61273 .27519 .604 Metetra
.646 .763 .934 r
.27519 .604 .28528 .61273 .30075 .60799 .29072 .59921 Metetra
.648 .763 .933 r
.29072 .59921 .30075 .60799 .31633 .60311 .30634 .59432 Metetra
.656 .766 .931 r
.30634 .59432 .31633 .60311 .33201 .59786 .32208 .58912 Metetra
.676 .774 .924 r
.32208 .58912 .33201 .59786 .34783 .59168 .33795 .58312 Metetra
.716 .787 .908 r
.33795 .58312 .34783 .59168 .36382 .58356 .35399 .57532 Metetra
.773 .803 .877 r
.35399 .57532 .36382 .58356 .38001 .57213 .37024 .5643 Metetra
.83 .812 .832 r
.37024 .5643 .38001 .57213 .39641 .55615 .38673 .5486 Metetra
.871 .81 .787 r
.38673 .5486 .39641 .55615 .41297 .53515 .4034 .52748 Metetra
.892 .801 .752 r
.4034 .52748 .41297 .53515 .42959 .50999 .42017 .50165 Metetra
.897 .79 .734 r
.42017 .50165 .42959 .50999 .44616 .4829 .43689 .47339 Metetra
.887 .777 .734 r
.43689 .47339 .44616 .4829 .46259 .4569 .45347 .44599 Metetra
.861 .763 .753 r
.45347 .44599 .46259 .4569 .47885 .43493 .46986 .42277 Metetra
.81 .744 .79 r
.46986 .42277 .47885 .43493 .49499 .41907 .48609 .40612 Metetra
.721 .71 .84 r
.48609 .40612 .49499 .41907 .51109 .41009 .50224 .39697 Metetra
.577 .652 .887 r
.50224 .39697 .51109 .41009 .52725 .40757 .51844 .39485 Metetra
.386 .564 .906 r
.51844 .39485 .52725 .40757 .54357 .41015 .53478 .39822 Metetra
.203 .475 .89 r
.53478 .39822 .54357 .41015 .56009 .41602 .55134 .40504 Metetra
.09 .425 .869 r
.55134 .40504 .56009 .41602 .57686 .42334 .56815 .41326 Metetra
.068 .433 .873 r
.56815 .41326 .57686 .42334 .59386 .43059 .58519 .4212 Metetra
.122 .488 .902 r
.58519 .4212 .59386 .43059 .61105 .43669 .60245 .42773 Metetra
.223 .565 .938 r
.60245 .42773 .61105 .43669 .62843 .44105 .61988 .43225 Metetra
.335 .637 .963 r
.61988 .43225 .62843 .44105 .64593 .44346 .63744 .43463 Metetra
.433 .691 .973 r
.63744 .43463 .64593 .44346 .66356 .44401 .65513 .43501 Metetra
.506 .724 .971 r
.65513 .43501 .66356 .44401 .68127 .44294 .67291 .43369 Metetra
.557 .743 .963 r
.67291 .43369 .68127 .44294 .69908 .44055 .69078 .43104 Metetra
.591 .753 .955 r
.69078 .43104 .69908 .44055 .71698 .43715 .70874 .42738 Metetra
.613 .757 .947 r
.70874 .42738 .71698 .43715 .73497 .43301 .7268 .42301 Metetra
.627 .759 .941 r
.7268 .42301 .73497 .43301 .75305 .42832 .74495 .41813 Metetra
.636 .76 .937 r
.74495 .41813 .75305 .42832 .77122 .42326 .7632 .4129 Metetra
.641 .76 .934 r
.7632 .4129 .77122 .42326 .78951 .41794 .78156 .40744 Metetra
.645 .759 .932 r
.78156 .40744 .78951 .41794 .8079 .41243 .80003 .40182 Metetra
.647 .759 .931 r
.80003 .40182 .8079 .41243 .8264 .4068 .81861 .39609 Metetra
.648 .759 .93 r
.81861 .39609 .8264 .4068 .84502 .40108 .83731 .39027 Metetra
.646 .762 .934 r
.1734 .62353 .18391 .63206 .1989 .62745 .18844 .61887 Metetra
.646 .762 .934 r
.18844 .61887 .1989 .62745 .21398 .62281 .20357 .61418 Metetra
.646 .762 .934 r
.20357 .61418 .21398 .62281 .22915 .61815 .21879 .60947 Metetra
.646 .762 .934 r
.21879 .60947 .22915 .61815 .24441 .61346 .2341 .60473 Metetra
.646 .762 .934 r
.2341 .60473 .24441 .61346 .25975 .60874 .2495 .59996 Metetra
.646 .762 .934 r
.2495 .59996 .25975 .60874 .27519 .604 .26499 .59516 Metetra
.646 .763 .934 r
.26499 .59516 .27519 .604 .29072 .59921 .28057 .59033 Metetra
.648 .763 .933 r
.28057 .59033 .29072 .59921 .30634 .59432 .29625 .58541 Metetra
.653 .765 .931 r
.29625 .58541 .30634 .59432 .32208 .58912 .31204 .58026 Metetra
.671 .772 .926 r
.31204 .58026 .32208 .58912 .33795 .58312 .32796 .57442 Metetra
.708 .785 .912 r
.32796 .57442 .33795 .58312 .35399 .57532 .34404 .56696 Metetra
.765 .802 .882 r
.34404 .56696 .35399 .57532 .37024 .5643 .36036 .55639 Metetra
.826 .812 .837 r
.36036 .55639 .37024 .5643 .38673 .5486 .37692 .54105 Metetra
.871 .811 .788 r
.37692 .54105 .38673 .5486 .4034 .52748 .39371 .51989 Metetra
.896 .803 .749 r
.39371 .51989 .4034 .52748 .42017 .50165 .41063 .49341 Metetra
.902 .79 .728 r
.41063 .49341 .42017 .50165 .43689 .47339 .42752 .46389 Metetra
.893 .777 .727 r
.42752 .46389 .43689 .47339 .45347 .44599 .44427 .43494 Metetra
.867 .763 .745 r
.44427 .43494 .45347 .44599 .46986 .42277 .46079 .41031 Metetra
.815 .742 .783 r
.46079 .41031 .46986 .42277 .48609 .40612 .4771 .39276 Metetra
.721 .706 .835 r
.4771 .39276 .48609 .40612 .50224 .39697 .49332 .38343 Metetra
.563 .64 .885 r
.49332 .38343 .50224 .39697 .51844 .39485 .50955 .38179 Metetra
.349 .539 .899 r
.50955 .38179 .51844 .39485 .53478 .39822 .52592 .38608 Metetra
.146 .436 .873 r
.52592 .38608 .53478 .39822 .55134 .40504 .5425 .39398 Metetra
.028 .383 .846 r
.5425 .39398 .55134 .40504 .56815 .41326 .55934 .4032 Metetra
.015 .399 .852 r
.55934 .4032 .56815 .41326 .58519 .4212 .57643 .41188 Metetra
.086 .468 .89 r
.57643 .41188 .58519 .4212 .60245 .42773 .59374 .41884 Metetra
.206 .559 .935 r
.59374 .41884 .60245 .42773 .61988 .43225 .61122 .4235 Metetra
.333 .64 .965 r
.61122 .4235 .61988 .43225 .63744 .43463 .62885 .42579 Metetra
.438 .696 .974 r
.62885 .42579 .63744 .43463 .65513 .43501 .64659 .42594 Metetra
.515 .73 .971 r
.64659 .42594 .65513 .43501 .67291 .43369 .66444 .42434 Metetra
.565 .747 .962 r
.66444 .42434 .67291 .43369 .69078 .43104 .68237 .42139 Metetra
.598 .755 .953 r
.68237 .42139 .69078 .43104 .70874 .42738 .7004 .41747 Metetra
.618 .759 .946 r
.7004 .41747 .70874 .42738 .7268 .42301 .71852 .41286 Metetra
.631 .76 .94 r
.71852 .41286 .7268 .42301 .74495 .41813 .73675 .40778 Metetra
.639 .76 .936 r
.73675 .40778 .74495 .41813 .7632 .4129 .75508 .40239 Metetra
.643 .76 .933 r
.75508 .40239 .7632 .4129 .78156 .40744 .77351 .3968 Metetra
.646 .759 .932 r
.77351 .3968 .78156 .40744 .80003 .40182 .79206 .39106 Metetra
.647 .759 .931 r
.79206 .39106 .80003 .40182 .81861 .39609 .81073 .38523 Metetra
.648 .759 .93 r
.81073 .38523 .81861 .39609 .83731 .39027 .82952 .37933 Metetra
.646 .762 .934 r
.16277 .61491 .1734 .62353 .18844 .61887 .17787 .6102 Metetra
.646 .762 .934 r
.17787 .6102 .18844 .61887 .20357 .61418 .19305 .60546 Metetra
.646 .762 .934 r
.19305 .60546 .20357 .61418 .21879 .60947 .20832 .6007 Metetra
.646 .762 .934 r
.20832 .6007 .21879 .60947 .2341 .60473 .22368 .59591 Metetra
.646 .762 .934 r
.22368 .59591 .2341 .60473 .2495 .59996 .23913 .59108 Metetra
.646 .762 .934 r
.23913 .59108 .2495 .59996 .26499 .59516 .25468 .58623 Metetra
.646 .762 .934 r
.25468 .58623 .26499 .59516 .28057 .59033 .27032 .58135 Metetra
.647 .763 .933 r
.27032 .58135 .28057 .59033 .29625 .58541 .28605 .5764 Metetra
.651 .764 .932 r
.28605 .5764 .29625 .58541 .31204 .58026 .30189 .57127 Metetra
.665 .77 .928 r
.30189 .57127 .31204 .58026 .32796 .57442 .31785 .56559 Metetra
.699 .782 .916 r
.31785 .56559 .32796 .57442 .34404 .56696 .33399 .55846 Metetra
.756 .8 .888 r
.33399 .55846 .34404 .56696 .36036 .55639 .35035 .54839 Metetra
.821 .813 .843 r
.35035 .54839 .36036 .55639 .37692 .54105 .36699 .5335 Metetra
.871 .813 .79 r
.36699 .5335 .37692 .54105 .39371 .51989 .38389 .51242 Metetra
.899 .804 .746 r
.38389 .51242 .39371 .51989 .41063 .49341 .40097 .48531 Metetra
.907 .791 .722 r
.40097 .48531 .41063 .49341 .42752 .46389 .41805 .45445 Metetra
.899 .777 .718 r
.41805 .45445 .42752 .46389 .44427 .43494 .43497 .42377 Metetra
.873 .762 .736 r
.43497 .42377 .44427 .43494 .46079 .41031 .45163 .39753 Metetra
.82 .74 .775 r
.45163 .39753 .46079 .41031 .4771 .39276 .46805 .37896 Metetra
.721 .701 .831 r
.46805 .37896 .4771 .39276 .49332 .38343 .48432 .36943 Metetra
.547 .626 .882 r
.48432 .36943 .49332 .38343 .50955 .38179 .50058 .36835 Metetra
.304 .508 .89 r
.50058 .36835 .50955 .38179 .52592 .38608 .51696 .37372 Metetra
.079 .39 .849 r
.51696 .37372 .52592 .38608 .5425 .39398 .53357 .38287 Metetra
0 .335 .815 r
.53357 .38287 .5425 .39398 .55934 .4032 .55044 .39319 Metetra
0 .361 .827 r
.55044 .39319 .55934 .4032 .57643 .41188 .56757 .40267 Metetra
.048 .447 .877 r
.56757 .40267 .57643 .41188 .59374 .41884 .58493 .41005 Metetra
.19 .554 .932 r
.58493 .41005 .59374 .41884 .61122 .4235 .60247 .4148 Metetra
.334 .645 .967 r
.60247 .4148 .61122 .4235 .62885 .42579 .62016 .41694 Metetra
.447 .703 .976 r
.62016 .41694 .62885 .42579 .64659 .42594 .63796 .41681 Metetra
.525 .735 .971 r
.63796 .41681 .64659 .42594 .66444 .42434 .65586 .41488 Metetra
.574 .751 .961 r
.65586 .41488 .66444 .42434 .68237 .42139 .67386 .41161 Metetra
.605 .757 .951 r
.67386 .41161 .68237 .42139 .7004 .41747 .69196 .40739 Metetra
.623 .76 .944 r
.69196 .40739 .7004 .41747 .71852 .41286 .71015 .40254 Metetra
.635 .76 .939 r
.71015 .40254 .71852 .41286 .73675 .40778 .72845 .39727 Metetra
.641 .76 .935 r
.72845 .39727 .73675 .40778 .75508 .40239 .74685 .39172 Metetra
.645 .76 .932 r
.74685 .39172 .75508 .40239 .77351 .3968 .76537 .386 Metetra
.647 .759 .931 r
.76537 .386 .77351 .3968 .79206 .39106 .784 .38016 Metetra
.648 .759 .93 r
.784 .38016 .79206 .39106 .81073 .38523 .80275 .37423 Metetra
.649 .759 .93 r
.80275 .37423 .81073 .38523 .82952 .37933 .82162 .36824 Metetra
.646 .762 .934 r
.15203 .60619 .16277 .61491 .17787 .6102 .16717 .60143 Metetra
.646 .762 .934 r
.16717 .60143 .17787 .6102 .19305 .60546 .18241 .59664 Metetra
.646 .762 .934 r
.18241 .59664 .19305 .60546 .20832 .6007 .19773 .59183 Metetra
.646 .762 .934 r
.19773 .59183 .20832 .6007 .22368 .59591 .21314 .58698 Metetra
.646 .762 .934 r
.21314 .58698 .22368 .59591 .23913 .59108 .22865 .58211 Metetra
.646 .762 .934 r
.22865 .58211 .23913 .59108 .25468 .58623 .24425 .57721 Metetra
.646 .762 .934 r
.24425 .57721 .25468 .58623 .27032 .58135 .25994 .57227 Metetra
.646 .763 .934 r
.25994 .57227 .27032 .58135 .28605 .5764 .27573 .56728 Metetra
.649 .764 .933 r
.27573 .56728 .28605 .5764 .30189 .57127 .29163 .56216 Metetra
.66 .768 .929 r
.29163 .56216 .30189 .57127 .31785 .56559 .30764 .5566 Metetra
.69 .779 .919 r
.30764 .5566 .31785 .56559 .33399 .55846 .32382 .54981 Metetra
.746 .798 .894 r
.32382 .54981 .33399 .55846 .35035 .54839 .34023 .54028 Metetra
.814 .813 .849 r
.34023 .54028 .35035 .54839 .36699 .5335 .35693 .52596 Metetra
.87 .815 .793 r
.35693 .52596 .36699 .5335 .38389 .51242 .37394 .50507 Metetra
.902 .806 .744 r
.37394 .50507 .38389 .51242 .40097 .48531 .39118 .47739 Metetra
.912 .792 .715 r
.39118 .47739 .40097 .48531 .41805 .45445 .40845 .44511 Metetra
.905 .778 .71 r
.40845 .44511 .41805 .45445 .43497 .42377 .42557 .41248 Metetra
.88 .761 .727 r
.42557 .41248 .43497 .42377 .45163 .39753 .4424 .38439 Metetra
.826 .738 .767 r
.4424 .38439 .45163 .39753 .46805 .37896 .45893 .36465 Metetra
.721 .695 .825 r
.45893 .36465 .46805 .37896 .48432 .36943 .47525 .3549 Metetra
.527 .61 .878 r
.47525 .3549 .48432 .36943 .50058 .36835 .49153 .35452 Metetra
.25 .47 .876 r
.49153 .35452 .50058 .36835 .51696 .37372 .50792 .36115 Metetra
0 .334 .817 r
.50792 .36115 .51696 .37372 .53357 .38287 .52454 .37173 Metetra
0 .28 .776 r
.52454 .37173 .53357 .38287 .55044 .39319 .54143 .3833 Metetra
0 .319 .796 r
.54143 .3833 .55044 .39319 .56757 .40267 .5586 .39361 Metetra
.009 .426 .862 r
.5586 .39361 .56757 .40267 .58493 .41005 .57601 .40138 Metetra
.177 .552 .931 r
.57601 .40138 .58493 .41005 .60247 .4148 .59361 .40615 Metetra
.338 .652 .969 r
.59361 .40615 .60247 .4148 .62016 .41694 .61135 .40807 Metetra
.458 .712 .977 r
.61135 .40807 .62016 .41694 .63796 .41681 .62922 .40759 Metetra
.536 .741 .97 r
.62922 .40759 .63796 .41681 .65586 .41488 .64718 .40528 Metetra
.584 .755 .959 r
.64718 .40528 .65586 .41488 .67386 .41161 .66525 .40166 Metetra
.612 .759 .949 r
.66525 .40166 .67386 .41161 .69196 .40739 .68341 .39715 Metetra
.628 .761 .942 r
.68341 .39715 .69196 .40739 .71015 .40254 .70168 .39206 Metetra
.638 .761 .937 r
.70168 .39206 .71015 .40254 .72845 .39727 .72005 .38659 Metetra
.643 .76 .934 r
.72005 .38659 .72845 .39727 .74685 .39172 .73853 .3809 Metetra
.646 .76 .932 r
.73853 .3809 .74685 .39172 .76537 .386 .75713 .37506 Metetra
.648 .759 .93 r
.75713 .37506 .76537 .386 .784 .38016 .77584 .36911 Metetra
.649 .759 .93 r
.77584 .36911 .784 .38016 .80275 .37423 .79468 .36309 Metetra
.649 .759 .929 r
.79468 .36309 .80275 .37423 .82162 .36824 .81363 .35702 Metetra
.646 .762 .934 r
.14117 .59738 .15203 .60619 .16717 .60143 .15637 .59257 Metetra
.646 .762 .934 r
.15637 .59257 .16717 .60143 .18241 .59664 .17165 .58773 Metetra
.646 .762 .934 r
.17165 .58773 .18241 .59664 .19773 .59183 .18703 .58286 Metetra
.646 .762 .934 r
.18703 .58286 .19773 .59183 .21314 .58698 .20249 .57796 Metetra
.646 .762 .934 r
.20249 .57796 .21314 .58698 .22865 .58211 .21805 .57303 Metetra
.646 .762 .934 r
.21805 .57303 .22865 .58211 .24425 .57721 .23371 .56808 Metetra
.646 .762 .934 r
.23371 .56808 .24425 .57721 .25994 .57227 .24946 .56309 Metetra
.646 .763 .934 r
.24946 .56309 .25994 .57227 .27573 .56728 .2653 .55806 Metetra
.648 .763 .933 r
.2653 .55806 .27573 .56728 .29163 .56216 .28125 .55293 Metetra
.656 .767 .931 r
.28125 .55293 .29163 .56216 .30764 .5566 .29731 .54746 Metetra
.681 .776 .923 r
.29731 .54746 .30764 .5566 .32382 .54981 .31353 .54099 Metetra
.734 .795 .901 r
.31353 .54099 .32382 .54981 .34023 .54028 .32998 .53204 Metetra
.805 .813 .857 r
.32998 .53204 .34023 .54028 .35693 .52596 .34674 .5184 Metetra
.868 .818 .797 r
.34674 .5184 .35693 .52596 .37394 .50507 .36385 .49787 Metetra
.904 .809 .743 r
.36385 .49787 .37394 .50507 .39118 .47739 .38125 .46971 Metetra
.917 .794 .709 r
.38125 .46971 .39118 .47739 .40845 .44511 .39874 .43592 Metetra
.911 .778 .7 r
.39874 .43592 .40845 .44511 .42557 .41248 .41608 .4011 Metetra
.887 .76 .716 r
.41608 .4011 .42557 .41248 .4424 .38439 .43309 .37086 Metetra
.832 .735 .757 r
.43309 .37086 .4424 .38439 .45893 .36465 .44974 .34974 Metetra
.72 .689 .819 r
.44974 .34974 .45893 .36465 .47525 .3549 .46611 .33977 Metetra
.503 .589 .874 r
.46611 .33977 .47525 .3549 .49153 .35452 .4824 .34023 Metetra
.183 .422 .856 r
.4824 .34023 .49153 .35452 .50792 .36115 .49879 .34838 Metetra
0 .267 .773 r
.49879 .34838 .50792 .36115 .52454 .37173 .51541 .36061 Metetra
0 .216 .726 r
.51541 .36061 .52454 .37173 .54143 .3833 .53232 .37356 Metetra
0 .273 .758 r
.53232 .37356 .54143 .3833 .5586 .39361 .54953 .38475 Metetra
0 .405 .845 r
.54953 .38475 .5586 .39361 .57601 .40138 .56698 .39287 Metetra
.168 .553 .93 r
.56698 .39287 .57601 .40138 .59361 .40615 .58464 .39757 Metetra
.347 .661 .972 r
.58464 .39757 .59361 .40615 .61135 .40807 .60244 .39916 Metetra
.473 .721 .978 r
.60244 .39916 .61135 .40807 .62922 .40759 .62036 .39826 Metetra
.55 .748 .969 r
.62036 .39826 .62922 .40759 .64718 .40528 .63839 .39553 Metetra
.594 .758 .957 r
.63839 .39553 .64718 .40528 .66525 .40166 .65652 .39153 Metetra
.619 .761 .947 r
.65652 .39153 .66525 .40166 .68341 .39715 .67476 .38672 Metetra
.633 .761 .94 r
.67476 .38672 .68341 .39715 .70168 .39206 .69309 .38139 Metetra
.641 .761 .935 r
.69309 .38139 .70168 .39206 .72005 .38659 .71154 .37575 Metetra
.645 .76 .933 r
.71154 .37575 .72005 .38659 .73853 .3809 .7301 .36991 Metetra
.647 .759 .931 r
.7301 .36991 .73853 .3809 .75713 .37506 .74878 .36396 Metetra
.648 .759 .93 r
.74878 .36396 .75713 .37506 .77584 .36911 .76758 .35791 Metetra
.649 .759 .929 r
.76758 .35791 .77584 .36911 .79468 .36309 .7865 .35181 Metetra
.649 .758 .929 r
.7865 .35181 .79468 .36309 .81363 .35702 .80555 .34565 Metetra
.646 .762 .934 r
.13019 .58848 .14117 .59738 .15637 .59257 .14544 .58361 Metetra
.646 .762 .934 r
.14544 .58361 .15637 .59257 .17165 .58773 .16077 .57872 Metetra
.646 .762 .934 r
.16077 .57872 .17165 .58773 .18703 .58286 .1762 .57379 Metetra
.646 .762 .934 r
.1762 .57379 .18703 .58286 .20249 .57796 .19172 .56884 Metetra
.646 .762 .934 r
.19172 .56884 .20249 .57796 .21805 .57303 .20734 .56386 Metetra
.646 .762 .934 r
.20734 .56386 .21805 .57303 .23371 .56808 .22305 .55884 Metetra
.646 .762 .934 r
.22305 .55884 .23371 .56808 .24946 .56309 .23885 .5538 Metetra
.646 .762 .934 r
.23885 .5538 .24946 .56309 .2653 .55806 .25476 .54872 Metetra
.647 .763 .933 r
.25476 .54872 .2653 .55806 .28125 .55293 .27076 .54357 Metetra
.653 .765 .932 r
.27076 .54357 .28125 .55293 .29731 .54746 .28687 .53817 Metetra
.673 .773 .926 r
.28687 .53817 .29731 .54746 .31353 .54099 .30314 .53198 Metetra
.721 .791 .907 r
.30314 .53198 .31353 .54099 .32998 .53204 .31962 .52364 Metetra
.794 .812 .866 r
.31962 .52364 .32998 .53204 .34674 .5184 .33642 .51081 Metetra
.864 .82 .803 r
.33642 .51081 .34674 .5184 .36385 .49787 .35362 .49083 Metetra
.907 .811 .742 r
.35362 .49083 .36385 .49787 .38125 .46971 .37118 .46233 Metetra
.922 .795 .703 r
.37118 .46233 .38125 .46971 .39874 .43592 .3889 .42696 Metetra
.918 .778 .691 r
.3889 .42696 .39874 .43592 .41608 .4011 .40648 .38966 Metetra
.894 .759 .705 r
.40648 .38966 .41608 .4011 .43309 .37086 .42371 .35691 Metetra
.839 .732 .746 r
.42371 .35691 .43309 .37086 .44974 .34974 .44049 .33416 Metetra
.72 .681 .812 r
.44049 .33416 .44974 .34974 .46611 .33977 .45691 .32394 Metetra
.473 .564 .867 r
.45691 .32394 .46611 .33977 .4824 .34023 .4732 .32544 Metetra
.102 .362 .827 r
.4732 .32544 .4824 .34023 .49879 .34838 .48957 .3354 Metetra
0 .188 .715 r
.48957 .3354 .49879 .34838 .51541 .36061 .50618 .34957 Metetra
0 .144 .664 r
.50618 .34957 .51541 .36061 .53232 .37356 .52311 .36406 Metetra
0 .224 .713 r
.52311 .36406 .53232 .37356 .54953 .38475 .54034 .37615 Metetra
0 .386 .827 r
.54034 .37615 .54953 .38475 .56698 .39287 .55784 .38453 Metetra
.164 .558 .931 r
.55784 .38453 .56698 .39287 .58464 .39757 .57555 .38905 Metetra
.363 .674 .976 r
.57555 .38905 .58464 .39757 .60244 .39916 .59341 .39021 Metetra
.491 .731 .979 r
.59341 .39021 .60244 .39916 .62036 .39826 .6114 .3888 Metetra
.564 .754 .967 r
.6114 .3888 .62036 .39826 .63839 .39553 .62949 .38559 Metetra
.604 .761 .954 r
.62949 .38559 .63839 .39553 .65652 .39153 .64769 .38121 Metetra
.626 .762 .944 r
.64769 .38121 .65652 .39153 .67476 .38672 .66599 .3761 Metetra
.637 .762 .938 r
.66599 .3761 .67476 .38672 .69309 .38139 .6844 .37055 Metetra
.643 .761 .934 r
.6844 .37055 .69309 .38139 .71154 .37575 .70293 .36474 Metetra
.646 .76 .932 r
.70293 .36474 .71154 .37575 .7301 .36991 .72157 .35877 Metetra
.648 .759 .93 r
.72157 .35877 .7301 .36991 .74878 .36396 .74033 .3527 Metetra
.649 .759 .93 r
.74033 .3527 .74878 .36396 .76758 .35791 .75922 .34657 Metetra
.649 .759 .929 r
.75922 .34657 .76758 .35791 .7865 .35181 .77823 .34038 Metetra
.649 .758 .929 r
.77823 .34038 .7865 .35181 .80555 .34565 .79736 .33414 Metetra
.646 .762 .934 r
.11909 .57947 .13019 .58848 .14544 .58361 .13439 .57455 Metetra
.646 .762 .934 r
.13439 .57455 .14544 .58361 .16077 .57872 .14978 .5696 Metetra
.646 .762 .934 r
.14978 .5696 .16077 .57872 .1762 .57379 .16526 .56463 Metetra
.646 .762 .934 r
.16526 .56463 .1762 .57379 .19172 .56884 .18083 .55962 Metetra
.646 .762 .934 r
.18083 .55962 .19172 .56884 .20734 .56386 .1965 .55458 Metetra
.646 .762 .934 r
.1965 .55458 .20734 .56386 .22305 .55884 .21227 .54951 Metetra
.646 .762 .934 r
.21227 .54951 .22305 .55884 .23885 .5538 .22813 .54441 Metetra
.646 .762 .934 r
.22813 .54441 .23885 .5538 .25476 .54872 .24409 .53927 Metetra
.647 .763 .934 r
.24409 .53927 .25476 .54872 .27076 .54357 .26015 .53408 Metetra
.65 .764 .933 r
.26015 .53408 .27076 .54357 .28687 .53817 .27631 .52873 Metetra
.665 .77 .928 r
.27631 .52873 .28687 .53817 .30314 .53198 .29262 .52279 Metetra
.706 .786 .914 r
.29262 .52279 .30314 .53198 .31962 .52364 .30913 .51505 Metetra
.781 .81 .876 r
.30913 .51505 .31962 .52364 .33642 .51081 .32597 .50315 Metetra
.859 .822 .811 r
.32597 .50315 .33642 .51081 .35362 .49083 .34325 .48395 Metetra
.908 .815 .743 r
.34325 .48395 .35362 .49083 .37118 .46233 .36096 .45532 Metetra
.927 .798 .697 r
.36096 .45532 .37118 .46233 .3889 .42696 .37892 .41834 Metetra
.925 .779 .68 r
.37892 .41834 .3889 .42696 .40648 .38966 .39679 .37822 Metetra
.902 .758 .693 r
.39679 .37822 .40648 .38966 .42371 .35691 .41425 .34248 Metetra
.846 .73 .735 r
.41425 .34248 .42371 .35691 .44049 .33416 .43118 .31777 Metetra
.719 .672 .804 r
.43118 .31777 .44049 .33416 .45691 .32394 .44766 .30728 Metetra
.435 .533 .858 r
.44766 .30728 .45691 .32394 .4732 .32544 .46393 .31008 Metetra
.002 .286 .784 r
.46393 .31008 .4732 .32544 .48957 .3354 .48026 .32225 Metetra
0 .096 .64 r
.48026 .32225 .48957 .3354 .50618 .34957 .49685 .3387 Metetra
0 .066 .589 r
.49685 .3387 .50618 .34957 .52311 .36406 .51377 .35489 Metetra
0 .172 .662 r
.51377 .35489 .52311 .36406 .54034 .37615 .53104 .36787 Metetra
0 .37 .81 r
.53104 .36787 .54034 .37615 .55784 .38453 .54859 .3764 Metetra
.17 .57 .934 r
.54859 .3764 .55784 .38453 .57555 .38905 .56635 .38057 Metetra
.385 .691 .98 r
.56635 .38057 .57555 .38905 .59341 .39021 .58427 .38117 Metetra
.512 .742 .978 r
.58427 .38117 .59341 .39021 .6114 .3888 .60232 .37917 Metetra
.58 .759 .963 r
.60232 .37917 .6114 .3888 .62949 .38559 .62048 .37546 Metetra
.615 .763 .95 r
.62048 .37546 .62949 .38559 .64769 .38121 .63874 .37068 Metetra
.632 .763 .941 r
.63874 .37068 .64769 .38121 .66599 .3761 .65712 .36528 Metetra
.641 .762 .936 r
.65712 .36528 .66599 .3761 .6844 .37055 .67561 .35951 Metetra
.646 .76 .933 r
.67561 .35951 .6844 .37055 .70293 .36474 .69421 .35355 Metetra
.648 .759 .931 r
.69421 .35355 .70293 .36474 .72157 .35877 .71293 .34747 Metetra
.649 .759 .93 r
.71293 .34747 .72157 .35877 .74033 .3527 .73178 .3413 Metetra
.649 .759 .929 r
.73178 .3413 .74033 .3527 .75922 .34657 .75075 .33508 Metetra
.649 .758 .929 r
.75075 .33508 .75922 .34657 .77823 .34038 .76985 .3288 Metetra
.649 .758 .929 r
.76985 .3288 .77823 .34038 .79736 .33414 .78907 .32249 Metetra
.646 .762 .934 r
.10787 .57037 .11909 .57947 .13439 .57455 .12322 .56539 Metetra
.646 .762 .934 r
.12322 .56539 .13439 .57455 .14978 .5696 .13866 .56039 Metetra
.646 .762 .934 r
.13866 .56039 .14978 .5696 .16526 .56463 .15419 .55535 Metetra
.646 .762 .934 r
.15419 .55535 .16526 .56463 .18083 .55962 .16982 .55029 Metetra
.646 .762 .934 r
.16982 .55029 .18083 .55962 .1965 .55458 .18554 .54519 Metetra
.646 .762 .934 r
.18554 .54519 .1965 .55458 .21227 .54951 .20137 .54007 Metetra
.646 .762 .934 r
.20137 .54007 .21227 .54951 .22813 .54441 .21728 .53491 Metetra
.646 .762 .934 r
.21728 .53491 .22813 .54441 .24409 .53927 .2333 .52971 Metetra
.646 .763 .934 r
.2333 .52971 .24409 .53927 .26015 .53408 .24942 .52448 Metetra
.648 .763 .933 r
.24942 .52448 .26015 .53408 .27631 .52873 .26564 .51914 Metetra
.658 .768 .93 r
.26564 .51914 .27631 .52873 .29262 .52279 .28199 .51339 Metetra
.692 .781 .92 r
.28199 .51339 .29262 .52279 .30913 .51505 .29853 .50623 Metetra
.764 .806 .887 r
.29853 .50623 .30913 .51505 .32597 .50315 .31539 .49536 Metetra
.85 .824 .822 r
.31539 .49536 .32597 .50315 .34325 .48395 .33271 .47725 Metetra
.909 .819 .746 r
.33271 .47725 .34325 .48395 .36096 .45532 .35057 .44878 Metetra
.933 .8 .691 r
.35057 .44878 .36096 .45532 .37892 .41834 .36879 .4102 Metetra
.932 .78 .669 r
.36879 .4102 .37892 .41834 .39679 .37822 .38698 .36685 Metetra
.91 .757 .679 r
.38698 .36685 .39679 .37822 .41425 .34248 .40473 .32753 Metetra
.854 .726 .721 r
.40473 .32753 .41425 .34248 .43118 .31777 .42184 .30042 Metetra
.717 .662 .794 r
.42184 .30042 .43118 .31777 .44766 .30728 .43837 .28962 Metetra
.386 .492 .844 r
.43837 .28962 .44766 .30728 .46393 .31008 .4546 .29408 Metetra
0 .191 .723 r
.4546 .29408 .46393 .31008 .48026 .32225 .47086 .30897 Metetra
0 0 .546 r
.47086 .30897 .48026 .32225 .49685 .3387 .4874 .32811 Metetra
0 0 .503 r
.4874 .32811 .49685 .3387 .51377 .35489 .50432 .34618 Metetra
0 .121 .605 r
.50432 .34618 .51377 .35489 .53104 .36787 .52161 .35999 Metetra
0 .362 .795 r
.52161 .35999 .53104 .36787 .54859 .3764 .5392 .36849 Metetra
.187 .589 .942 r
.5392 .36849 .54859 .3764 .56635 .38057 .55702 .37211 Metetra
.415 .71 .984 r
.55702 .37211 .56635 .38057 .58427 .38117 .57501 .372 Metetra
.536 .753 .976 r
.57501 .372 .58427 .38117 .60232 .37917 .59312 .36933 Metetra
.596 .764 .959 r
.59312 .36933 .60232 .37917 .62048 .37546 .61134 .36509 Metetra
.624 .765 .947 r
.61134 .36509 .62048 .37546 .63874 .37068 .62968 .35992 Metetra
.638 .763 .938 r
.62968 .35992 .63874 .37068 .65712 .36528 .64813 .35425 Metetra
.644 .761 .934 r
.64813 .35425 .65712 .36528 .67561 .35951 .6667 .3483 Metetra
.647 .76 .931 r
.6667 .3483 .67561 .35951 .69421 .35355 .68538 .3422 Metetra
.648 .759 .93 r
.68538 .3422 .69421 .35355 .71293 .34747 .70419 .336 Metetra
.649 .759 .929 r
.70419 .336 .71293 .34747 .73178 .3413 .72312 .32975 Metetra
.649 .758 .929 r
.72312 .32975 .73178 .3413 .75075 .33508 .74218 .32344 Metetra
.649 .758 .929 r
.74218 .32344 .75075 .33508 .76985 .3288 .76137 .31708 Metetra
.649 .758 .929 r
.76137 .31708 .76985 .3288 .78907 .32249 .78068 .31068 Metetra
.646 .762 .934 r
.09652 .56116 .10787 .57037 .12322 .56539 .11192 .55613 Metetra
.646 .762 .934 r
.11192 .55613 .12322 .56539 .13866 .56039 .12741 .55107 Metetra
.646 .762 .934 r
.12741 .55107 .13866 .56039 .15419 .55535 .143 .54598 Metetra
.646 .762 .934 r
.143 .54598 .15419 .55535 .16982 .55029 .15868 .54086 Metetra
.646 .762 .934 r
.15868 .54086 .16982 .55029 .18554 .54519 .17446 .5357 Metetra
.646 .762 .934 r
.17446 .5357 .18554 .54519 .20137 .54007 .19034 .53052 Metetra
.646 .762 .934 r
.19034 .53052 .20137 .54007 .21728 .53491 .20631 .5253 Metetra
.646 .762 .934 r
.20631 .5253 .21728 .53491 .2333 .52971 .22239 .52005 Metetra
.646 .762 .934 r
.22239 .52005 .2333 .52971 .24942 .52448 .23856 .51476 Metetra
.647 .763 .933 r
.23856 .51476 .24942 .52448 .26564 .51914 .25484 .50941 Metetra
.653 .766 .932 r
.25484 .50941 .26564 .51914 .28199 .51339 .27124 .50379 Metetra
.678 .776 .925 r
.27124 .50379 .28199 .51339 .29853 .50623 .2878 .49715 Metetra
.743 .801 .899 r
.2878 .49715 .29853 .50623 .31539 .49536 .30467 .48739 Metetra
.837 .825 .836 r
.30467 .48739 .31539 .49536 .33271 .47725 .32202 .47067 Metetra
.908 .823 .751 r
.32202 .47067 .33271 .47725 .35057 .44878 .34 .44279 Metetra
.938 .804 .686 r
.34 .44279 .35057 .44878 .36879 .4102 .35848 .40273 Metetra
.939 .781 .658 r
.35848 .40273 .36879 .4102 .38698 .36685 .37705 .35571 Metetra
.919 .757 .665 r
.37705 .35571 .38698 .36685 .40473 .32753 .39515 .31199 Metetra
.863 .723 .707 r
.39515 .31199 .40473 .32753 .42184 .30042 .41247 .28188 Metetra
.714 .649 .783 r
.41247 .28188 .42184 .30042 .43837 .28962 .42905 .27073 Metetra
.323 .438 .824 r
.42905 .27073 .43837 .28962 .4546 .29408 .44521 .27735 Metetra
0 .077 .637 r
.44521 .27735 .4546 .29408 .47086 .30897 .46137 .29565 Metetra
0 0 .434 r
.46137 .29565 .47086 .30897 .4874 .32811 .47783 .31798 Metetra
.588 .097 0 r
.47783 .31798 .4874 .32811 .50432 .34618 .49473 .3381 Metetra
0 .073 .545 r
.49473 .3381 .50432 .34618 .52161 .35999 .51205 .3526 Metetra
0 .364 .786 r
.51205 .3526 .52161 .35999 .5392 .36849 .52969 .36079 Metetra
.221 .619 .953 r
.52969 .36079 .5392 .36849 .55702 .37211 .54757 .36362 Metetra
.452 .731 .987 r
.54757 .36362 .55702 .37211 .57501 .372 .56562 .36264 Metetra
.562 .762 .972 r
.56562 .36264 .57501 .372 .59312 .36933 .5838 .35924 Metetra
.611 .767 .954 r
.5838 .35924 .59312 .36933 .61134 .36509 .60209 .35445 Metetra
.633 .765 .943 r
.60209 .35445 .61134 .36509 .62968 .35992 .6205 .34892 Metetra
.642 .762 .936 r
.6205 .34892 .62968 .35992 .64813 .35425 .63903 .34301 Metetra
.646 .761 .932 r
.63903 .34301 .64813 .35425 .6667 .3483 .65768 .3369 Metetra
.648 .759 .93 r
.65768 .3369 .6667 .3483 .68538 .3422 .67644 .33068 Metetra
.649 .759 .93 r
.67644 .33068 .68538 .3422 .70419 .336 .69534 .32439 Metetra
.649 .758 .929 r
.69534 .32439 .70419 .336 .72312 .32975 .71436 .31804 Metetra
.649 .758 .929 r
.71436 .31804 .72312 .32975 .74218 .32344 .7335 .31165 Metetra
.649 .758 .929 r
.7335 .31165 .74218 .32344 .76137 .31708 .75278 .30522 Metetra
.649 .758 .929 r
.75278 .30522 .76137 .31708 .78068 .31068 .77219 .29874 Metetra
.646 .762 .934 r
.08504 .55185 .09652 .56116 .11192 .55613 .1005 .54676 Metetra
.646 .762 .934 r
.1005 .54676 .11192 .55613 .12741 .55107 .11604 .54165 Metetra
.646 .762 .934 r
.11604 .54165 .12741 .55107 .143 .54598 .13168 .5365 Metetra
.646 .762 .934 r
.13168 .5365 .143 .54598 .15868 .54086 .14742 .53132 Metetra
.646 .762 .934 r
.14742 .53132 .15868 .54086 .17446 .5357 .16325 .5261 Metetra
.646 .762 .934 r
.16325 .5261 .17446 .5357 .19034 .53052 .17919 .52086 Metetra
.646 .762 .934 r
.17919 .52086 .19034 .53052 .20631 .5253 .19522 .51558 Metetra
.646 .762 .934 r
.19522 .51558 .20631 .5253 .22239 .52005 .21135 .51027 Metetra
.646 .762 .934 r
.21135 .51027 .22239 .52005 .23856 .51476 .22758 .50492 Metetra
.646 .763 .934 r
.22758 .50492 .23856 .51476 .25484 .50941 .24392 .49953 Metetra
.649 .764 .933 r
.24392 .49953 .25484 .50941 .27124 .50379 .26037 .49399 Metetra
.666 .771 .928 r
.26037 .49399 .27124 .50379 .2878 .49715 .27696 .48778 Metetra
.72 .794 .91 r
.27696 .48778 .2878 .49715 .30467 .48739 .29383 .47915 Metetra
.819 .824 .853 r
.29383 .47915 .30467 .48739 .32202 .47067 .31118 .46416 Metetra
.905 .829 .761 r
.31118 .46416 .32202 .47067 .34 .44279 .32924 .43744 Metetra
.943 .809 .682 r
.32924 .43744 .34 .44279 .35848 .40273 .34799 .3962 Metetra
.947 .783 .645 r
.34799 .3962 .35848 .40273 .37705 .35571 .36698 .34499 Metetra
.928 .757 .649 r
.36698 .34499 .37705 .35571 .39515 .31199 .38551 .29581 Metetra
.872 .72 .69 r
.38551 .29581 .39515 .31199 .41247 .28188 .40309 .26184 Metetra
.711 .635 .77 r
.40309 .26184 .41247 .28188 .42905 .27073 .41971 .2503 Metetra
.238 .366 .792 r
.41971 .2503 .42905 .27073 .44521 .27735 .43577 .25978 Metetra
0 0 .522 r
.43577 .25978 .44521 .27735 .46137 .29565 .45178 .28243 Metetra
.665 .22 0 r
.45178 .28243 .46137 .29565 .47783 .31798 .46814 .3086 Metetra
.67 .174 0 r
.46814 .3086 .47783 .31798 .49473 .3381 .485 .33086 Metetra
.506 0 0 r
.485 .33086 .49473 .3381 .51205 .3526 .50235 .34577 Metetra
0 .385 .788 r
.50235 .34577 .51205 .3526 .52969 .36079 .52005 .35326 Metetra
.274 .658 .969 r
.52005 .35326 .52969 .36079 .54757 .36362 .53799 .355 Metetra
.495 .752 .986 r
.53799 .355 .54757 .36362 .56562 .36264 .55611 .353 Metetra
.587 .769 .966 r
.55611 .353 .56562 .36264 .5838 .35924 .57435 .34883 Metetra
.624 .768 .949 r
.57435 .34883 .5838 .35924 .60209 .35445 .59272 .34353 Metetra
.639 .764 .939 r
.59272 .34353 .60209 .35445 .6205 .34892 .61121 .33768 Metetra
.646 .762 .933 r
.61121 .33768 .6205 .34892 .63903 .34301 .62981 .33157 Metetra
.648 .76 .931 r
.62981 .33157 .63903 .34301 .65768 .3369 .64854 .32532 Metetra
.649 .759 .93 r
.64854 .32532 .65768 .3369 .67644 .33068 .6674 .319 Metetra
.649 .759 .929 r
.6674 .319 .67644 .33068 .69534 .32439 .68637 .31262 Metetra
.649 .758 .929 r
.68637 .31262 .69534 .32439 .71436 .31804 .70548 .30619 Metetra
.649 .758 .929 r
.70548 .30619 .71436 .31804 .7335 .31165 .72472 .29972 Metetra
.649 .758 .929 r
.72472 .29972 .7335 .31165 .75278 .30522 .74409 .2932 Metetra
.649 .758 .929 r
.74409 .2932 .75278 .30522 .77219 .29874 .76359 .28664 Metetra
.646 .762 .934 r
.07344 .54244 .08504 .55185 .1005 .54676 .08894 .53729 Metetra
.646 .762 .934 r
.08894 .53729 .1005 .54676 .11604 .54165 .10454 .53212 Metetra
.646 .762 .934 r
.10454 .53212 .11604 .54165 .13168 .5365 .12023 .52691 Metetra
.646 .762 .934 r
.12023 .52691 .13168 .5365 .14742 .53132 .13603 .52167 Metetra
.646 .762 .934 r
.13603 .52167 .14742 .53132 .16325 .5261 .15192 .51639 Metetra
.646 .762 .934 r
.15192 .51639 .16325 .5261 .17919 .52086 .1679 .51109 Metetra
.646 .762 .934 r
.1679 .51109 .17919 .52086 .19522 .51558 .18399 .50575 Metetra
.646 .762 .934 r
.18399 .50575 .19522 .51558 .21135 .51027 .20018 .50038 Metetra
.646 .762 .934 r
.20018 .50038 .21135 .51027 .22758 .50492 .21648 .49497 Metetra
.646 .762 .934 r
.21648 .49497 .22758 .50492 .24392 .49953 .23287 .48952 Metetra
.647 .763 .933 r
.23287 .48952 .24392 .49953 .26037 .49399 .24937 .484 Metetra
.656 .767 .931 r
.24937 .484 .26037 .49399 .27696 .48778 .26601 .4781 Metetra
.696 .785 .919 r
.26601 .4781 .27696 .48778 .29383 .47915 .28287 .47052 Metetra
.792 .82 .874 r
.28287 .47052 .29383 .47915 .31118 .46416 .30017 .45757 Metetra
.897 .835 .776 r
.30017 .45757 .31118 .46416 .32924 .43744 .31826 .43281 Metetra
.947 .815 .681 r
.31826 .43281 .32924 .43744 .34799 .3962 .33726 .39096 Metetra
.955 .786 .633 r
.33726 .39096 .34799 .3962 .36698 .34499 .35675 .33506 Metetra
.938 .757 .631 r
.35675 .33506 .36698 .34499 .38551 .29581 .37582 .27894 Metetra
.882 .716 .672 r
.37582 .27894 .38551 .29581 .40309 .26184 .39375 .23985 Metetra
.706 .617 .755 r
.39375 .23985 .40309 .26184 .41971 .2503 .41041 .22785 Metetra
.124 .269 .742 r
.41041 .22785 .41971 .2503 .43577 .25978 .42628 .24126 Metetra
0 0 .382 r
.42628 .24126 .43577 .25978 .45178 .28243 .44207 .26959 Metetra
.761 .317 0 r
.44207 .26959 .45178 .28243 .46814 .3086 .45829 .30034 Metetra
.736 .24 0 r
.45829 .30034 .46814 .3086 .485 .33086 .47512 .32472 Metetra
.543 0 0 r
.47512 .32472 .485 .33086 .50235 .34577 .4925 .33954 Metetra
0 .433 .81 r
.4925 .33954 .50235 .34577 .52005 .35326 .51026 .34581 Metetra
.348 .706 .984 r
.51026 .34581 .52005 .35326 .53799 .355 .52828 .34613 Metetra
.541 .769 .982 r
.52828 .34613 .53799 .355 .55611 .353 .54647 .34298 Metetra
.61 .773 .958 r
.54647 .34298 .55611 .353 .57435 .34883 .56479 .33806 Metetra
.635 .767 .943 r
.56479 .33806 .57435 .34883 .59272 .34353 .58323 .33232 Metetra
.645 .763 .935 r
.58323 .33232 .59272 .34353 .61121 .33768 .60179 .32621 Metetra
.648 .76 .931 r
.60179 .32621 .61121 .33768 .62981 .33157 .62048 .31994 Metetra
.649 .759 .93 r
.62048 .31994 .62981 .33157 .64854 .32532 .63929 .31358 Metetra
.649 .759 .929 r
.63929 .31358 .64854 .32532 .6674 .319 .65823 .30716 Metetra
.649 .758 .929 r
.65823 .30716 .6674 .319 .68637 .31262 .6773 .30069 Metetra
.649 .758 .929 r
.6773 .30069 .68637 .31262 .70548 .30619 .6965 .29418 Metetra
.649 .758 .929 r
.6965 .29418 .70548 .30619 .72472 .29972 .71582 .28763 Metetra
.649 .758 .929 r
.71582 .28763 .72472 .29972 .74409 .2932 .73528 .28103 Metetra
.649 .758 .929 r
.73528 .28103 .74409 .2932 .76359 .28664 .75487 .27438 Metetra
.646 .762 .934 r
.0617 .53291 .07344 .54244 .08894 .53729 .07726 .52771 Metetra
.646 .762 .934 r
.07726 .52771 .08894 .53729 .10454 .53212 .09291 .52247 Metetra
.646 .762 .934 r
.09291 .52247 .10454 .53212 .12023 .52691 .10865 .51721 Metetra
.646 .762 .934 r
.10865 .51721 .12023 .52691 .13603 .52167 .1245 .51191 Metetra
.646 .762 .934 r
.1245 .51191 .13603 .52167 .15192 .51639 .14045 .50657 Metetra
.646 .762 .934 r
.14045 .50657 .15192 .51639 .1679 .51109 .15649 .5012 Metetra
.646 .762 .934 r
.15649 .5012 .1679 .51109 .18399 .50575 .17264 .4958 Metetra
.646 .762 .934 r
.17264 .4958 .18399 .50575 .20018 .50038 .18889 .49037 Metetra
.646 .762 .934 r
.18889 .49037 .20018 .50038 .21648 .49497 .20524 .4849 Metetra
.646 .762 .934 r
.20524 .4849 .21648 .49497 .23287 .48952 .2217 .47939 Metetra
.646 .763 .934 r
.2217 .47939 .23287 .48952 .24937 .484 .23826 .47384 Metetra
.65 .764 .933 r
.23826 .47384 .24937 .484 .26601 .4781 .25494 .46811 Metetra
.674 .776 .927 r
.25494 .46811 .26601 .4781 .28287 .47052 .27179 .46141 Metetra
.757 .81 .895 r
.27179 .46141 .28287 .47052 .30017 .45757 .28903 .45069 Metetra
.881 .84 .801 r
.28903 .45069 .30017 .45757 .31826 .43281 .30707 .42887 Metetra
.95 .823 .684 r
.30707 .42887 .31826 .43281 .33726 .39096 .32626 .38752 Metetra
.964 .791 .62 r
.32626 .38752 .33726 .39096 .35675 .33506 .3463 .32655 Metetra
.949 .758 .611 r
.3463 .32655 .35675 .33506 .37582 .27894 .36609 .26139 Metetra
.894 .713 .65 r
.36609 .26139 .37582 .27894 .39375 .23985 .38448 .21521 Metetra
.699 .594 .736 r
.38448 .21521 .39375 .23985 .41041 .22785 .40117 .20267 Metetra
0 .138 .662 r
.40117 .20267 .41041 .22785 .42628 .24126 .41677 .22168 Metetra
.711 .335 0 r
.41677 .22168 .42628 .24126 .44207 .26959 .43224 .25763 Metetra
.834 .399 0 r
.43224 .25763 .44207 .26959 .45829 .30034 .44826 .29384 Metetra
.784 .29 0 r
.44826 .29384 .45829 .30034 .47512 .32472 .46505 .31998 Metetra
.553 0 0 r
.46505 .31998 .47512 .32472 .4925 .33954 .48249 .33387 Metetra
.003 .519 .857 r
.48249 .33387 .4925 .33954 .51026 .34581 .50034 .33822 Metetra
.438 .753 .994 r
.50034 .33822 .51026 .34581 .52828 .34613 .51843 .33681 Metetra
.584 .78 .973 r
.51843 .33681 .52828 .34613 .54647 .34298 .5367 .3325 Metetra
.629 .772 .949 r
.5367 .3325 .54647 .34298 .56479 .33806 .5551 .32692 Metetra
.643 .765 .937 r
.5551 .32692 .56479 .33806 .58323 .33232 .57362 .32082 Metetra
.648 .761 .932 r
.57362 .32082 .58323 .33232 .60179 .32621 .59226 .31452 Metetra
.649 .759 .93 r
.59226 .31452 .60179 .32621 .62048 .31994 .61103 .30812 Metetra
.649 .759 .929 r
.61103 .30812 .62048 .31994 .63929 .31358 .62993 .30167 Metetra
.649 .758 .929 r
.62993 .30167 .63929 .31358 .65823 .30716 .64896 .29517 Metetra
.649 .758 .929 r
.64896 .29517 .65823 .30716 .6773 .30069 .66811 .28862 Metetra
.649 .758 .929 r
.66811 .28862 .6773 .30069 .6965 .29418 .6874 .28203 Metetra
.649 .758 .929 r
.6874 .28203 .6965 .29418 .71582 .28763 .70681 .27539 Metetra
.649 .758 .929 r
.70681 .27539 .71582 .28763 .73528 .28103 .72637 .2687 Metetra
.649 .758 .929 r
.72637 .2687 .73528 .28103 .75487 .27438 .74605 .26197 Metetra
.646 .762 .934 r
.04983 .52328 .0617 .53291 .07726 .52771 .06544 .51802 Metetra
.646 .762 .934 r
.06544 .51802 .07726 .52771 .09291 .52247 .08114 .51272 Metetra
.646 .762 .934 r
.08114 .51272 .09291 .52247 .10865 .51721 .09694 .50739 Metetra
.646 .762 .934 r
.09694 .50739 .10865 .51721 .1245 .51191 .11284 .50203 Metetra
.646 .762 .934 r
.11284 .50203 .1245 .51191 .14045 .50657 .12885 .49664 Metetra
.646 .762 .934 r
.12885 .49664 .14045 .50657 .15649 .5012 .14495 .49121 Metetra
.646 .762 .934 r
.14495 .49121 .15649 .5012 .17264 .4958 .16115 .48574 Metetra
.646 .762 .934 r
.16115 .48574 .17264 .4958 .18889 .49037 .17746 .48024 Metetra
.646 .762 .934 r
.17746 .48024 .18889 .49037 .20524 .4849 .19387 .47471 Metetra
.646 .762 .934 r
.19387 .47471 .20524 .4849 .2217 .47939 .21039 .46914 Metetra
.646 .762 .934 r
.21039 .46914 .2217 .47939 .23826 .47384 .22701 .46353 Metetra
.647 .763 .934 r
.22701 .46353 .23826 .47384 .25494 .46811 .24374 .45785 Metetra
.657 .768 .931 r
.24374 .45785 .25494 .46811 .27179 .46141 .26061 .45174 Metetra
.714 .795 .915 r
.26061 .45174 .27179 .46141 .28903 .45069 .27777 .44317 Metetra
.849 .841 .837 r
.27777 .44317 .28903 .45069 .30707 .42887 .29566 .42538 Metetra
.949 .835 .696 r
.29566 .42538 .30707 .42887 .32626 .38752 .31492 .38647 Metetra
.972 .797 .607 r
.31492 .38647 .32626 .38752 .3463 .32655 .33556 .32055 Metetra
.961 .761 .589 r
.33556 .32055 .3463 .32655 .36609 .26139 .35631 .24333 Metetra
.907 .711 .626 r
.35631 .24333 .36609 .26139 .38448 .21521 .37538 .1868 Metetra
.63 .534 .726 r
.37538 .1868 .38448 .21521 .40117 .20267 .39184 .17928 Metetra
0 .035 .601 r
.39184 .17928 .40117 .20267 .41677 .22168 .40724 .20102 Metetra
.823 .453 0 r
.40724 .20102 .41677 .22168 .43224 .25763 .42222 .24751 Metetra
.883 .461 .059 r
.42222 .24751 .43224 .25763 .44826 .29384 .438 .28999 Metetra
.807 .318 0 r
.438 .28999 .44826 .29384 .46505 .31998 .45479 .31689 Metetra
.515 0 0 r
.45479 .31689 .46505 .31998 .48249 .33387 .47231 .32848 Metetra
.168 .645 .928 r
.47231 .32848 .48249 .33387 .50034 .33822 .49027 .33014 Metetra
.529 .787 .992 r
.49027 .33014 .50034 .33822 .51843 .33681 .50846 .32684 Metetra
.619 .781 .959 r
.50846 .32684 .51843 .33681 .5367 .3325 .5268 .3215 Metetra
.641 .769 .94 r
.5268 .3215 .5367 .3325 .5551 .32692 .54528 .31542 Metetra
.647 .762 .933 r
.54528 .31542 .5551 .32692 .57362 .32082 .56388 .30908 Metetra
.649 .76 .93 r
.56388 .30908 .57362 .32082 .59226 .31452 .58261 .30264 Metetra
.649 .759 .929 r
.58261 .30264 .59226 .31452 .61103 .30812 .60147 .29615 Metetra
.649 .758 .929 r
.60147 .29615 .61103 .30812 .62993 .30167 .62045 .28961 Metetra
.649 .758 .929 r
.62045 .28961 .62993 .30167 .64896 .29517 .63956 .28302 Metetra
.649 .758 .929 r
.63956 .28302 .64896 .29517 .66811 .28862 .6588 .27639 Metetra
.649 .758 .929 r
.6588 .27639 .66811 .28862 .6874 .28203 .67818 .26972 Metetra
.649 .758 .929 r
.67818 .26972 .6874 .28203 .70681 .27539 .69769 .26299 Metetra
.649 .758 .929 r
.69769 .26299 .70681 .27539 .72637 .2687 .71734 .25622 Metetra
.649 .758 .929 r
.71734 .25622 .72637 .2687 .74605 .26197 .73712 .24941 Metetra
.646 .762 .934 r
.03782 .51354 .04983 .52328 .06544 .51802 .05348 .50822 Metetra
.646 .762 .934 r
.05348 .50822 .06544 .51802 .08114 .51272 .06924 .50286 Metetra
.646 .762 .934 r
.06924 .50286 .08114 .51272 .09694 .50739 .08509 .49747 Metetra
.646 .762 .934 r
.08509 .49747 .09694 .50739 .11284 .50203 .10105 .49204 Metetra
.646 .762 .934 r
.10105 .49204 .11284 .50203 .12885 .49664 .11711 .48658 Metetra
.646 .762 .934 r
.11711 .48658 .12885 .49664 .14495 .49121 .13327 .48109 Metetra
.646 .762 .934 r
.13327 .48109 .14495 .49121 .16115 .48574 .14953 .47556 Metetra
.646 .762 .934 r
.14953 .47556 .16115 .48574 .17746 .48024 .1659 .47 Metetra
.646 .762 .934 r
.1659 .47 .17746 .48024 .19387 .47471 .18237 .4644 Metetra
.646 .762 .934 r
.18237 .4644 .19387 .47471 .21039 .46914 .19895 .45876 Metetra
.646 .762 .934 r
.19895 .45876 .21039 .46914 .22701 .46353 .21563 .45309 Metetra
.646 .762 .934 r
.21563 .45309 .22701 .46353 .24374 .45785 .23242 .44738 Metetra
.648 .764 .933 r
.23242 .44738 .24374 .45785 .26061 .45174 .24933 .44153 Metetra
.674 .778 .928 r
.24933 .44153 .26061 .45174 .27777 .44317 .26643 .43464 Metetra
.791 .83 .882 r
.26643 .43464 .27777 .44317 .29566 .42538 .28408 .42166 Metetra
.94 .851 .727 r
.28408 .42166 .29566 .42538 .31492 .38647 .30316 .38838 Metetra
.981 .808 .597 r
.30316 .38838 .31492 .38647 .33556 .32055 .32437 .31913 Metetra
.973 .766 .564 r
.32437 .31913 .33556 .32055 .35631 .24333 .34644 .22543 Metetra
.902 .713 .637 r
.34644 .22543 .35631 .24333 .37538 .1868 .36544 .17398 Metetra
.661 .715 .888 r
.36544 .17398 .37538 .1868 .39184 .17928 .3819 .16768 Metetra
0 .2 .71 r
.3819 .16768 .39184 .17928 .40724 .20102 .39768 .17961 Metetra
.901 .543 .076 r
.39768 .17961 .40724 .20102 .42222 .24751 .41194 .2412 Metetra
.907 .497 .163 r
.41194 .2412 .42222 .24751 .438 .28999 .42745 .29012 Metetra
.796 .308 .045 r
.42745 .29012 .438 .28999 .45479 .31689 .4443 .31537 Metetra
.381 0 0 r
.4443 .31537 .45479 .31689 .47231 .32848 .46198 .32273 Metetra
.387 .773 .99 r
.46198 .32273 .47231 .32848 .49027 .33014 .48006 .32111 Metetra
.602 .797 .974 r
.48006 .32111 .49027 .33014 .50846 .32684 .49835 .31608 Metetra
.64 .775 .945 r
.49835 .31608 .50846 .32684 .5268 .3215 .51678 .30999 Metetra
.648 .764 .934 r
.51678 .30999 .5268 .3215 .54528 .31542 .53534 .30361 Metetra
.649 .76 .93 r
.53534 .30361 .54528 .31542 .56388 .30908 .55403 .29713 Metetra
.649 .759 .929 r
.55403 .29713 .56388 .30908 .58261 .30264 .57284 .2906 Metetra
.649 .758 .929 r
.57284 .2906 .58261 .30264 .60147 .29615 .59178 .28402 Metetra
.649 .758 .929 r
.59178 .28402 .60147 .29615 .62045 .28961 .61085 .2774 Metetra
.649 .758 .929 r
.61085 .2774 .62045 .28961 .63956 .28302 .63005 .27073 Metetra
.649 .758 .929 r
.63005 .27073 .63956 .28302 .6588 .27639 .64938 .26401 Metetra
.649 .758 .929 r
.64938 .26401 .6588 .27639 .67818 .26972 .66885 .25725 Metetra
.649 .758 .929 r
.66885 .25725 .67818 .26972 .69769 .26299 .68845 .25044 Metetra
.649 .758 .929 r
.68845 .25044 .69769 .26299 .71734 .25622 .70819 .24358 Metetra
.649 .758 .929 r
.70819 .24358 .71734 .25622 .73712 .24941 .72807 .23668 Metetra
.646 .762 .934 r
.02567 .50369 .03782 .51354 .05348 .50822 .04138 .4983 Metetra
.646 .762 .934 r
.04138 .4983 .05348 .50822 .06924 .50286 .0572 .49288 Metetra
.646 .762 .934 r
.0572 .49288 .06924 .50286 .08509 .49747 .07311 .48743 Metetra
.646 .762 .934 r
.07311 .48743 .08509 .49747 .10105 .49204 .08912 .48194 Metetra
.646 .762 .934 r
.08912 .48194 .10105 .49204 .11711 .48658 .10523 .47642 Metetra
.646 .762 .934 r
.10523 .47642 .11711 .48658 .13327 .48109 .12145 .47086 Metetra
.646 .762 .934 r
.12145 .47086 .13327 .48109 .14953 .47556 .13777 .46526 Metetra
.646 .762 .934 r
.13777 .46526 .14953 .47556 .1659 .47 .1542 .45963 Metetra
.646 .762 .934 r
.1542 .45963 .1659 .47 .18237 .4644 .17073 .45397 Metetra
.646 .762 .934 r
.17073 .45397 .18237 .4644 .19895 .45876 .18737 .44826 Metetra
.646 .762 .934 r
.18737 .44826 .19895 .45876 .21563 .45309 .20411 .44252 Metetra
.646 .762 .934 r
.20411 .44252 .21563 .45309 .23242 .44738 .22097 .43675 Metetra
.646 .763 .934 r
.22097 .43675 .23242 .44738 .24933 .44153 .23794 .43092 Metetra
.651 .766 .933 r
.23794 .43092 .24933 .44153 .26643 .43464 .25503 .42487 Metetra
.709 .799 .921 r
.25503 .42487 .26643 .43464 .28408 .42166 .27242 .41637 Metetra
.899 .867 .799 r
.27242 .41637 .28408 .42166 .30316 .38838 .29098 .393 Metetra
.987 .827 .597 r
.29098 .393 .30316 .38838 .32437 .31913 .31239 .32621 Metetra
.986 .775 .533 r
.31239 .32621 .32437 .31913 .34644 .22543 .33634 .21013 Metetra
.909 .73 .648 r
.33634 .21013 .34644 .22543 .36544 .17398 .35533 .16232 Metetra
.664 .741 .907 r
.35533 .16232 .36544 .17398 .3819 .16768 .37185 .15595 Metetra
.295 .567 .932 r
.37185 .15595 .3819 .16768 .39768 .17961 .38803 .15918 Metetra
.947 .602 .202 r
.38803 .15918 .39768 .17961 .41194 .2412 .40117 .24291 Metetra
.897 .499 .253 r
.40117 .24291 .41194 .2412 .42745 .29012 .4165 .29561 Metetra
.72 .227 .069 r
.4165 .29561 .42745 .29012 .4443 .31537 .4336 .3143 Metetra
0 .555 .732 r
.4336 .3143 .4443 .31537 .46198 .32273 .45151 .31542 Metetra
.57 .826 .993 r
.45151 .31542 .46198 .32273 .48006 .32111 .46974 .3107 Metetra
.64 .783 .95 r
.46974 .3107 .48006 .32111 .49835 .31608 .48812 .30455 Metetra
.648 .765 .934 r
.48812 .30455 .49835 .31608 .51678 .30999 .50664 .29811 Metetra
.649 .76 .93 r
.50664 .29811 .51678 .30999 .53534 .30361 .52528 .29159 Metetra
.649 .758 .929 r
.52528 .29159 .53534 .30361 .55403 .29713 .54404 .28501 Metetra
.649 .758 .929 r
.54404 .28501 .55403 .29713 .57284 .2906 .56294 .2784 Metetra
.649 .758 .929 r
.56294 .2784 .57284 .2906 .59178 .28402 .58197 .27173 Metetra
.649 .758 .929 r
.58197 .27173 .59178 .28402 .61085 .2774 .60112 .26503 Metetra
.649 .758 .929 r
.60112 .26503 .61085 .2774 .63005 .27073 .62041 .25827 Metetra
.649 .758 .929 r
.62041 .25827 .63005 .27073 .64938 .26401 .63984 .25147 Metetra
.649 .758 .929 r
.63984 .25147 .64938 .26401 .66885 .25725 .6594 .24462 Metetra
.649 .758 .929 r
.6594 .24462 .66885 .25725 .68845 .25044 .67909 .23772 Metetra
.649 .758 .929 r
.67909 .23772 .68845 .25044 .70819 .24358 .69892 .23078 Metetra
.649 .758 .929 r
.69892 .23078 .70819 .24358 .72807 .23668 .7189 .22378 Metetra
.646 .762 .934 r
.01338 .49372 .02567 .50369 .04138 .4983 .02915 .48827 Metetra
.646 .762 .934 r
.02915 .48827 .04138 .4983 .0572 .49288 .04501 .48279 Metetra
.646 .762 .934 r
.04501 .48279 .0572 .49288 .07311 .48743 .06098 .47727 Metetra
.646 .762 .934 r
.06098 .47727 .07311 .48743 .08912 .48194 .07705 .47171 Metetra
.646 .762 .934 r
.07705 .47171 .08912 .48194 .10523 .47642 .09322 .46613 Metetra
.646 .762 .934 r
.09322 .46613 .10523 .47642 .12145 .47086 .10949 .4605 Metetra
.646 .762 .934 r
.10949 .4605 .12145 .47086 .13777 .46526 .12587 .45484 Metetra
.646 .762 .934 r
.12587 .45484 .13777 .46526 .1542 .45963 .14236 .44914 Metetra
.646 .762 .934 r
.14236 .44914 .1542 .45963 .17073 .45397 .15895 .44341 Metetra
.646 .762 .934 r
.15895 .44341 .17073 .45397 .18737 .44826 .17565 .43764 Metetra
.646 .762 .934 r
.17565 .43764 .18737 .44826 .20411 .44252 .19246 .43183 Metetra
.646 .762 .934 r
.19246 .43183 .20411 .44252 .22097 .43675 .20938 .42598 Metetra
.646 .762 .934 r
.20938 .42598 .22097 .43675 .23794 .43092 .22641 .42009 Metetra
.646 .763 .934 r
.22641 .42009 .23794 .43092 .25503 .42487 .24355 .41416 Metetra
.652 .768 .934 r
.24355 .41416 .25503 .42487 .27242 .41637 .26083 .40795 Metetra
.763 .838 .909 r
.26083 .40795 .27242 .41637 .29098 .393 .2786 .39672 Metetra
.982 .869 .648 r
.2786 .39672 .29098 .393 .31239 .32621 .29908 .34821 Metetra
.997 .789 .497 r
.29908 .34821 .31239 .32621 .33634 .21013 .32532 .20714 Metetra
.969 .803 .623 r
.32532 .20714 .33634 .21013 .35533 .16232 .3451 .15053 Metetra
.664 .741 .907 r
.3451 .15053 .35533 .16232 .37185 .15595 .36168 .14409 Metetra
.264 .549 .926 r
.36168 .14409 .37185 .15595 .38803 .15918 .37786 .14782 Metetra
.959 .623 .315 r
.37786 .14782 .38803 .15918 .40117 .24291 .38943 .26155 Metetra
.811 .424 .323 r
.38943 .26155 .40117 .24291 .4165 .29561 .40513 .30526 Metetra
.47 0 0 r
.40513 .30526 .4165 .29561 .4336 .3143 .42279 .31027 Metetra
.503 .865 .986 r
.42279 .31027 .4336 .3143 .45151 .31542 .44096 .30542 Metetra
.643 .794 .955 r
.44096 .30542 .45151 .31542 .46974 .3107 .4593 .29908 Metetra
.649 .765 .933 r
.4593 .29908 .46974 .3107 .48812 .30455 .47777 .29257 Metetra
.649 .759 .929 r
.47777 .29257 .48812 .30455 .50664 .29811 .49636 .28601 Metetra
.649 .758 .929 r
.49636 .28601 .50664 .29811 .52528 .29159 .51508 .2794 Metetra
.649 .758 .929 r
.51508 .2794 .52528 .29159 .54404 .28501 .53394 .27274 Metetra
.649 .758 .929 r
.53394 .27274 .54404 .28501 .56294 .2784 .55292 .26604 Metetra
.649 .758 .929 r
.55292 .26604 .56294 .2784 .58197 .27173 .57203 .25929 Metetra
.649 .758 .929 r
.57203 .25929 .58197 .27173 .60112 .26503 .59128 .2525 Metetra
.649 .758 .929 r
.59128 .2525 .60112 .26503 .62041 .25827 .61065 .24566 Metetra
.649 .758 .929 r
.61065 .24566 .62041 .25827 .63984 .25147 .63017 .23877 Metetra
.649 .758 .929 r
.63017 .23877 .63984 .25147 .6594 .24462 .64982 .23183 Metetra
.649 .758 .929 r
.64982 .23183 .6594 .24462 .67909 .23772 .66961 .22484 Metetra
.649 .758 .929 r
.66961 .22484 .67909 .23772 .69892 .23078 .68954 .2178 Metetra
.649 .758 .929 r
.68954 .2178 .69892 .23078 .7189 .22378 .70961 .21072 Metetra
.646 .762 .934 r
.00095 .48363 .01338 .49372 .02915 .48827 .01677 .47812 Metetra
.646 .762 .934 r
.01677 .47812 .02915 .48827 .04501 .48279 .03269 .47257 Metetra
.646 .762 .934 r
.03269 .47257 .04501 .48279 .06098 .47727 .04871 .46699 Metetra
.646 .762 .934 r
.04871 .46699 .06098 .47727 .07705 .47171 .06483 .46137 Metetra
.646 .762 .934 r
.06483 .46137 .07705 .47171 .09322 .46613 .08106 .45571 Metetra
.646 .762 .934 r
.08106 .45571 .09322 .46613 .10949 .4605 .09739 .45002 Metetra
.646 .762 .934 r
.09739 .45002 .10949 .4605 .12587 .45484 .11383 .44429 Metetra
.646 .762 .934 r
.11383 .44429 .12587 .45484 .14236 .44914 .13038 .43853 Metetra
.646 .762 .934 r
.13038 .43853 .14236 .44914 .15895 .44341 .14703 .43272 Metetra
.646 .762 .934 r
.14703 .43272 .15895 .44341 .17565 .43764 .16379 .42688 Metetra
.646 .762 .934 r
.16379 .42688 .17565 .43764 .19246 .43183 .18066 .421 Metetra
.646 .762 .934 r
.18066 .421 .19246 .43183 .20938 .42598 .19765 .41508 Metetra
.646 .762 .934 r
.19765 .41508 .20938 .42598 .22641 .42009 .21474 .40913 Metetra
.646 .762 .934 r
.21474 .40913 .22641 .42009 .24355 .41416 .23195 .40313 Metetra
.646 .762 .934 r
.23195 .40313 .24355 .41416 .26083 .40795 .24927 .39709 Metetra
.647 .768 .937 r
.24927 .39709 .26083 .40795 .2786 .39672 .26672 .39098 Metetra
.764 .895 .936 r
.26672 .39098 .2786 .39672 .29908 .34821 .28466 .37973 Metetra
.995 .831 .474 r
.28466 .37973 .29908 .34821 .32532 .20714 .3097 .26733 Metetra
.92 .805 .336 r
.3097 .26733 .32532 .20714 .3451 .15053 .33476 .13861 Metetra
.664 .741 .907 r
.33476 .13861 .3451 .15053 .36168 .14409 .3514 .13209 Metetra
.936 .562 .205 r
.3514 .13209 .36168 .14409 .37786 .14782 .36447 .19483 Metetra
.841 .511 .429 r
.36447 .19483 .37786 .14782 .38943 .26155 .37628 .30242 Metetra
.496 .143 .297 r
.37628 .30242 .38943 .26155 .40513 .30526 .39384 .30625 Metetra
.377 .843 .838 r
.39384 .30625 .40513 .30526 .42279 .31027 .41201 .30007 Metetra
.647 .793 .952 r
.41201 .30007 .42279 .31027 .44096 .30542 .43031 .29356 Metetra
.649 .761 .931 r
.43031 .29356 .44096 .30542 .4593 .29908 .44873 .287 Metetra
.649 .758 .929 r
.44873 .287 .4593 .29908 .47777 .29257 .46728 .2804 Metetra
.649 .758 .929 r
.46728 .2804 .47777 .29257 .49636 .28601 .48596 .27375 Metetra
.649 .758 .929 r
.48596 .27375 .49636 .28601 .51508 .2794 .50476 .26705 Metetra
.649 .758 .929 r
.50476 .26705 .51508 .2794 .53394 .27274 .5237 .26031 Metetra
.649 .758 .929 r
.5237 .26031 .53394 .27274 .55292 .26604 .54277 .25353 Metetra
.649 .758 .929 r
.54277 .25353 .55292 .26604 .57203 .25929 .56197 .24669 Metetra
.649 .758 .929 r
.56197 .24669 .57203 .25929 .59128 .2525 .5813 .23981 Metetra
.649 .758 .929 r
.5813 .23981 .59128 .2525 .61065 .24566 .60077 .23288 Metetra
.649 .758 .929 r
.60077 .23288 .61065 .24566 .63017 .23877 .62038 .2259 Metetra
.649 .758 .929 r
.62038 .2259 .63017 .23877 .64982 .23183 .64012 .21887 Metetra
.649 .758 .929 r
.64012 .21887 .64982 .23183 .66961 .22484 .66001 .21179 Metetra
.649 .758 .929 r
.66001 .21179 .66961 .22484 .68954 .2178 .68003 .20466 Metetra
.649 .758 .929 r
.68003 .20466 .68954 .2178 .70961 .21072 .7002 .19748 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{183.312, 148.688},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003U0000^A000`40O003h00OiGoo003UOol00>Eo
o`00iGoo003UOol00>Eoo`00iGoo003UOol00>Eoo`00iGoo003UOol008=oo`D000=oo`<000Eoo`<0
04moo`00QGoo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol0001>Ool008Eoo`03
001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo0000CWoo0025Ool00`00Oomoo`02Ool0
1@00Oomoogoo00000goo00D007ooOomoo`0004ioo`00QGoo00<007ooOol00Woo00D007ooOomoo`00
00=oo`05001oogooOol0001>Ool008Eoo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomo
ogoo0000CWoo0025Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0004ioo`00
Pgoo0`001Goo0`001Goo0`00Cgoo003UOol00>Eoo`00iGoo002EOol2001>Ool0099oo`<00003Ool0
07oo04eoo`00Igoo0`001Goo0`007Woo0P0017oo0P00CGoo001VOol01@00Oomoogoo00000goo00D0
07ooOomoo`0001Yoo`<000Ioo`03001oo`0004aoo`00IWoo00D007ooOomoo`0000=oo`05001oogoo
Ool0000GOol40008Ool00`00Ool0001<Ool006Ioo`05001oogooOol00003Ool01@00Oomoogoo0000
5Goo0P0037oo00@007ooOol004]oo`00Igoo0`0017oo00D007ooOomoo`00019oo`<000ioo`03001o
ogoo008004Yoo`00IWoo00D007ooOomoo`0000=oo`05001oogooOol0000@Ool3000@Ool01@00Oomo
ogoo0000BWoo001VOol01@00Oomoogoo00000goo00D007ooOomoo`0000eoo`<001=oo`03001oogoo
009oo`03001oogoo04Moo`00Igoo0`001Goo0`002goo0`005Woo00<007ooOol00goo00<007ooOol0
AWoo001kOol2000IOol00`00Oomoo`03Ool00`00Oomoo`16Ool007Qoo`<0009oo`03001oogoo01Io
o`03001oogoo00Aoo`03001oogoo04Eoo`00MWoo0P007goo00<007ooOol017oo00<007ooOol01Goo
0`001Woo0P001Goo1@00:Woo001cOol3000QOol00`00Oomoo`04Ool00`00Oomoo`04Ool01@00Oomo
ogoo00001Goo0P001goo00<007ooOol0:Woo001aOol20002Ool00`00Oomoo`0OOol00`00Oomoo`04
Ool20005Ool01@00Oomoogoo00003Woo00<007ooOol0:Woo0016Ool30005Ool3000MOol3000VOol0
0`00Oomoo`05Ool00`00Oomoo`03Ool01@00Oomoogoo00003Woo00<007ooOol0:Woo0015Ool01@00
Oomoogoo00000goo00D007ooOomoo`0001Uoo`<002Uoo`03001oogoo00Ioo`03001oogoo009oo`05
001oogooOol0000>Ool00`00Oomoo`0ZOol004Eoo`05001oogooOol00003Ool01@00Oomoogoo0000
5goo0P0000=oo`00Ool0:Goo00<007ooOol01goo00D007ooOomoo`0000=oo`03001oogoo00aoo`03
001oogoo02Yoo`00AGoo00D007ooOomoo`0000=oo`05001oogooOol0000DOol3000^Ool00`00Oomo
o`06Ool20003Ool01@00Oomoogoo00003Woo00<007ooOol0:Woo0015Ool40004Ool01@00Oomoogoo
00004Woo0P00<Goo00<007ooOol027oo00@007ooOomoo`<000eoo`<002aoo`00AGoo00<007ooOol0
1Goo00D007ooOomoo`0000moo`@0039oo`03001oogoo00Uoo`03001oogoo03moo`00AWoo00<007oo
Ool017oo00D007ooOomoo`0000aoo`<003Ioo`03001oogoo00Uoo`80041oo`00Agoo0`0017oo0`00
2goo0P00>Goo00<007ooOol02Goo0P00@7oo001IOol4000jOol00`00Oomoo`0;Ool00`00Oomoo`0m
Ool005Moo`8003ioo`03001oogoo00]oo`03001oogoo03eoo`00E7oo0`00@7oo00<007ooOol02goo
0P00?Woo001BOol300000eKm0000000oOol00`00Oomoo`0=Ool00`00Oomoo`0kOol002Qoo`<000Ao
o`<001eoo`<00003Ool005Km00800004MRH00000000lOol00`00Oomoo`0=Ool00`00Oomoo`0kOol0
02Uoo`03001oogoo009oo`05001oogooOol0000IOol30004Ool200000eKm001f9P03MRH00`00Oomo
o`0iOol00`00Oomoo`0>Ool00`00Oomoo`0jOol002Ioo`D000=oo`05001oogooOol0000GOol20000
0goo001oo`03Ool01P00MbYg:P00E_d000=f9P8003Yoo`03001oogoo00ioo`8003]oo`009Woo00@0
07ooOol000Aoo`05001oogooOol0000DOol30008Ool00`00MbYg:P030002MRH200000gYZ001oo`0h
Ool00`00Oomoo`0?Ool00`00Oomoo`05Ool30006Ool20004Ool5000POol002Moo`03001oo`0000Ao
o`05001oogooOol0000BOol2000;Ool00`00MbX00002Oc<00`00MRIf9P0200000gYZ001oo`0hOol0
0`00Oomoo`0@Ool00`00Oomoo`03Ool01@00Oomoogoo00001Goo0P001Goo00@007ooOol0021oo`00
9goo00<007oo000017oo00D007ooOomoo`0000moo`D000]oo`03001g:P00009o<`03001f9WHV0080
0003NVX0000003Qoo`03001oogoo015oo`03001oogoo009oo`05001oogooOol0000=Ool00`00Oomo
o`0POol002Qoo`8000Aoo`05001oogooOol0000<Ool3000@Ool01000MbX007lc0P0000Ef9P00JPd0
07YZ008003Qoo`03001oogoo011oo`8000Aoo`05001oogooOol0000>Ool00`00Oomoo`0OOol002Uo
o`03001oogoo00=oo`<000]oo`80019oo`03001g:P00009o<`800004MRH006X=JPd200000gYV001o
o`0fOol00`00Oomoo`0BOol01@00Oomoogoo00000goo00<007ooOol03Goo00<007ooOol07Woo000j
Ool300000goo001oo`0AOol01@00MbX007lc00000WKd0P000VX=00@007YZ000003Qoo`03001oogoo
019oo`04001oogoo0003Ool00`00Oomoo`0=Ool00`00Oomoo`0NOol003Qoo`8001Moo`80009o<`04
001fm7Kd0003JPd01000NVX00000>7oo00<007ooOol04Goo0P000Woo00D007ooOomoo`0000]oo`05
001oogooOol0000POol003Eoo`<001Uoo`80009o<`04001fm7Kd0003JPd00`00NVX00002000fOol2
000EOol00`00Oomoo`03000=Ool3000QOol000Yoo`D000Aoo`<001eoo`80009oo`03001oogoo01Mo
o`800004Oc<007KdM_@20003JPd01@00NVX007YV0000=7oo0P0000=C7@00Ool05Goo00<007ooOol0
<goo000;Ool01000Oomoo`000goo00D007ooOomoo`0001Uoo`<001eoo`<00004Oc<007KdM_@20003
JPd01@00NVX007YV0000<Goo0`000e<M0P005Goo00<007ooOol0<goo000<Ool00`00Oomoo`03Ool0
1@00Oomoogoo00005Woo0`0087oo0P0000Mo;`00M_Afm000L]@00003JPd01000NVX007YV0P00;7oo
10000e<M00<005<MDad00`005Goo00<007ooOol0<Woo000=Ool00`00Oomoo`02Ool01@00Oomoogoo
000057oo0P000goo00<007ooOol07Goo0P0000Io;`00M_Afm000L]@20002JPd01000NVX007YV0P00
:Goo0`000U<M00<005<MDad00U<M100000=C7@00Ool05Goo00<007ooOol0<Goo000>Ool01@00Oomo
ogoo00000goo00<007ooOol03goo0`009Goo0P0000Io;`00001fm000L]@20002JPd01000NVX007YV
0`009goo00<005<M000015<M00T005<M000005<M001C7@00Dad00P005Goo00<007ooOol0<Goo000>
Ool01@00Oomoogoo00000goo00<007ooOol03Goo10009Woo00<007l_00000WlA00<007;DL]@00P00
0VX=00<007YZNVX00P0000=b8P00Ool08goo0`000e<M00<005<MDad00`0015<M0`0000=C7@00Ool0
57oo0P00<Woo000:Ool01@00Oomoogoo00000goo00D007ooOomoo`0000aoo`<002Uoo`800005Obl0
07lAOa400002L]@20002JPd01@00NVYjJP00NVH00`007goo10000e<M00<005<MDad00`000e<M00<0
05<MDad00`000e<M00<007ooOol04goo00<007ooOol0<7oo000;Ool30005Ool3000:Ool3000/Ool2
000017l_001o4GlA0`0000=fe000JPd00VX=00@007YZ001jIP<001aoo`<0009C7@03001C7E<M009C
7@<0009C7@03001C7E<M00@0009C7@03001C7E<M00<001Eoo`03001oogoo02moo`007Woo0P0000=o
o`000000;7oo0P0000Ao;`00Oa40009o4P03001fe00000=Z3@03001jJP0000D001Yoo`03001C7@00
00AC7@06001C7@00001C7@000e<M0`000e<M00<005<M00000`000e<M00<007ooOol04Woo0P00<7oo
000KOol3000aOol2000017l_001o4@000WlB00<007KD00000fX=00H007YZ001aj000MND2000GOol3
0003Dad00`00DaeC7@030004Dad300000e<M001C7@02Dad60004Dad2000DOol00`00Oomoo`05Ool3
0006Ool20005Ool3000FOol001Yoo`8003=oo`03001o;`00009o4@03001o4WlB00800004M]@006X=
JPd30002LNP4000BOol40003Dad00`00DaeC7@030003Dad00`00DaeC7@020004Dad4000015<M001C
7E<M0P0000EC7@00001C7@0001Eoo`03001oogoo00=oo`05001oogooOol00005Ool20004Ool01@00
Oomoogoo00005Goo000JOol3000aOol200001gl_001o4GlA001o4P00009k4P80009Z3@800005IJX0
077X001ei@03000>Ool30003Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad60004Dad00`00
Dad000020004Dad00`00Oomoo`0BOol20004Ool01@00Oomoogoo00003goo00<007ooOol04goo000J
Ool200000goo001oo`0_Ool2000017l_001o4@000WlB00<007/BNa800P000VX=00L006FZIJX0077X
001ei@03000=Ool00`00Dad00004Dad30002Dad00`00DaeC7@040003Dad60003Dad00`00DaeC7@03
0002Dad20002Dad3000DOol00`00Oomoo`02Ool01@00Oomoogoo00003goo00<007ooOol04goo000J
Ool01@00Oomoogoo0000;goo0P0000Ao;`00Oa40009o4P03001k4W/B0080009Z3@04001UZVFZ0002
LNP200000g7100000009Ool30003Dad01P00Dad00000Dad000=C7@<0009C7@03001C7E<M00D0009C
7@03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001oogoo01=oo`05001oogooOol00003Ool0
0`00Oomoo`0;Ool2000FOol001Uoo`03001oogoo00=oo`03001oogoo02aoo`<0009o4@08001o4WlB
001k4P00Na<0009Z3@80009UZP04001aj000MND40005Ool30003Dad00`00DaeC7@020004Dad01@00
Dad005<M000015<M100000=C7@00Dad00U<M0`000U<M00<005<MDad0100000=C7@0000000U<M0`00
5Goo00@007ooOol000=oo`03001oogoo00eoo`03001oogoo01=oo`006Goo0P001Goo00<007ooOol0
:goo00<007l_00000WlA00<007lB00000W/B00L007/C001Z3@00?8T00002IJX01P00LNP007GU001a
`@L0009C7@03001C7E<M00@0009C7@03001C7E<M00<000=C7@H000AC7@<00003Dad005<M009C7@<0
00=C7@03001C7E<M00<000=C7@03001oogoo015oo`80009oo`05001oogooOol0000;Ool01@00Oomo
ogoo00005Goo000IOol00`00Oomoo`05Ool00`00Oomoo`0YOol200001gl_001o4GlA001o4P00009k
4P07001k4`00JPd003b900000VFZ00<0077XLNP00P0000Aa`@00000000=C7@03001C7E<M00<000=C
7@03001C7E<M00<000=C7@H000=C7@03001C7E<M008000AC7@05001C7@00Dad00004Dad50004Dad2
000DOol00`00Oomoo`03000=Ool3000FOol001Uoo`03001oogoo00Ioo`03001oogoo02Qoo`800004
Obl007lA0002Oa800`00Na800002Na<30002?8T00`00IJYUZP0500000e<M001C7@03Dad30002Dad0
0`00DaeC7@040002Dad00`00Dad000040002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad0
1P00Dad00000Dad0009C7@800005Dad00000Dad0000EOol00`00Oomoo`0XOol001Uoo`8000Qoo`03
001oogoo02Moo`<0009o4@03001o4P00009k4P05001k4`00Na@00003?8T00`00IJYUZP040003Dad0
1P00Dad00000Dad000=C7@<000=C7@03001C7@0000<000=C7@03001C7E<M00<000=C7@03001C7E<M
00<000=C7@03001C7E<M008000AC7@03001C7@00008000AC7@03001oogoo015oo`8002Yoo`006Goo
00<007ooOol027oo00<007ooOol09Woo0`000WlA00<007lB00000W/B00@007/C001k50<00003?8T0
0000008000=C?P03001C7E<M008000AC7@05001C7@00Dad00003Dad400000e<M001C7@03Dad30002
Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad20002Dad2000EOol0
0`00Oomoo`0WOol001Uoo`03001oogoo00Uoo`03001oogoo02Aoo`04001oC`000002Oa401@00Oa80
07/B00000W/C00@007/D001g50H0009C?P@0009C?P03001C7E<M00<000=C7@@00003Dad005<M009C
7@06001C7@00001C7@000e<M0`000U<M00<005<MDad010000U<M00<005<MDad010000U<M00<005<M
Dad010000U<M00<007ooOol04goo00<007ooOol09Woo000HOol2000<Ool00`00Oomoo`0SOol20000
17l_001o4@000WlB00D007/B001k4`00009k5004001g57LD0003K_H70002Dch30003Dch50004Dad0
0`00DaeC7@020004Dad300000e<M001C7@02Dad30003Dad00`00DaeC7@030003Dad00`00DaeC7@03
00000e<M00000002Dad3000COol2000XOol001Qoo`03001oogoo00aoo`03001oogoo029oo`800003
Obl0000000<0009k4P03001k4`00009k5004001g57LD0002K_H01P00K_M^mfkg001N^@@0009;G`03
001C?P0000@0009C7P03001C7E<M00@0009C7@03001C7E<M008000AC7@05001C7@00Dad00004Dad3
0002Dad00`00DaeC7@040002Dad00`00DaeC7@030003Dad00`00Oomoo`0BOol00`00Oomoo`0UOol0
01Qoo`8000ioo`80029oo`@00003Oe<007lc00<0009k4`03001k5000009g5005001g5@00K_H00003
K_L01000G[UN^Eji1`000dlo00<005<NDah00`000U<N00<005<MDad010000U<M00<005<MDad00`00
0e<M00H005<M000005<M0003Dad30002Dad00`00DaeC7@0400000e<M00000002Dad3000DOol00`00
Oomoo`0TOol000Uoo`D000=oo`<000Aoo`03001oogoo00moo`03001oogoo01ioo`04001oC`000002
Oe<01000Oc<007lc0`000W/D00@007LDMa@0009g5@08001^mfkgK_L006kg001N^@@00003?U/00000
00L0009?7P03001C7U<N00<000=C7@03001C7E<M00<000=C7@03001C7E<M008000AC7@<00003Dad0
05<M009C7@<0009C7@03001C7E<M00@0009C7@03001oogoo015oo`8000Moo`<000Ioo`8000Ioo`<0
00]oo`002goo00<007ooOol00Woo00D007ooOomoo`0000=oo`03001oogoo011oo`03001oogoo01eo
o`08001oC`00001oD`00Oc<0009o<`03001k<`00009k5004001g57LD0002MaD01000K_M^m`000Vkg
0`000Uji00<003iK?U/010000T<o00<004lNCah010000U<N00<005<MDad010000U<M00<005<MDad0
10000U<M00<005<MDad00P0015<M00D005<M001C7@0000=C7@<00004Dad005<M0002Dad2000DOol0
0`00Oomoo`03Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol02Woo00060005Ool00`00Oomo
o`02Ool01@00Oomoogoo00000goo0P004Woo00<007ooOol077oo0`000WmC00D007lc001o<`00009k
<`<0009g5007001g5GLE001g5@00K_L00`0000=^m`00G[T00Uji30000d/O00<005<NDah00`000e<N
00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000e<M00D005<M001C7@0000=C
7@@0009C7@03001oogoo019oo`03001oogoo009oo`05001oogooOol0000;Ool5000;Ool000]oo`03
001oogoo009oo`05001oogooOol00002Ool00`00Oomoo`0COol00`00Oomoo`0KOol30002Oe<01`00
Oc<007lc001k<`00009k=004001g57LD0002MaD00`00MaD000020002K_L00`00J_L0000200001UkJ
0012Fd9K000E^`H0009;7`@0009?7P03001C7U<N00@0009C7P03001C7E<M00<000=C7@03001C7E<M
00<0009C7@03001C7E<M00<000=C7@03001C7@0000<000=C7@80019oo`8000Aoo`05001oogooOol0
000;Ool01000Oomoo`0037oo000;Ool00`00Oomoo`02Ool01@00Oomoogoo00000Woo00<007ooOol0
57oo00<007ooOol06Woo0`0000AoD`00Oc<0009o<`03001k<`00009k=0800003Ma@007LE00<00005
MaD006kgK_L00002J_L00`00G]YNfP02G]X01@00@U]2F`005K/0100000<^g`0000000dLO00<004lN
Cah00`000TlN00<005<NDah010000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<005<M
Dad00`000U<M00<005<M00000P0000=C7@00Ool04goo00D007ooOomoo`0000=oo`03001oogoo00Yo
o`03001oo`0000aoo`002goo00<007ooOol00Woo00D007ooOomoo`00009oo`8001Ioo`03001oogoo
01Qoo`03001oC`00009oD`05001o<`00Oc<00002Nc<00`00Nc@00002Nc@40002MaD01@00LaH006kg
00000V[g00@005kJG]YNfPL000030AP0000000T0009;7P03001?7TlN00<000=C7P03001C7E<M00<0
009C7@03001C7E<M00@0009C7@03001C7E<M00<000=C7@03001C7E<M008000AC7@03001oogoo015o
o`80009oo`05001oogooOol0000<Ool00`00Ool0000<Ool000]oo`03001oogoo009oo`05001oogoo
Ool00002Ool00`00Oomoo`0FOol00`00Oomoo`0GOol30002Oe<01`00Oc<007lc001k<`00009k=0D0
0005MaD007LEMaD00002LaH02P00J_MZm`00J_P005kJ001NfP000T9l00<001gl7O`020000SlO00<0
04/NBah010000TlN00<005<NDah010000U<N00<005<MDad00`000U<M00<005<MDad010000U<M00<0
05<MDad0100000=C7@0000000U<M0P004Woo00<007oo00000Woo00D007ooOomoo`0000eoo`8000ao
o`002Goo0`001Goo0`000goo00<007ooOol05goo00<007ooOol05Woo0`000WmC0P000Wlc00<007/c
00000`0000Ek=000NaEk5@00009g5@07001c5W<FLaH006[g00000f[h00D005kJG]YNfP000092O004
000Mo0000EX400027Wh50003Aal00`00Cai?7P030002Cah00`00DaiC7P040002Dad00`00DaeC7@03
0002Dad00`00DaeC7@040002Dad00`00DaeC7@040002Dad00`00Oomoo`0BOol00`00Oomoo`03000?
Ool00`00Oomoo`0:Ool001Moo`8001Uoo`03001oogoo01Eoo`80009oD`04001o<`00Oc<400001G/d
001k=7/d00000W/E00<007LE00000g<F00<007<F000016[h00@005kJG]YNfPL00081FPl0009?7P03
001?7TlN00<0009C7P03001C7E<M00@0009C7@03001C7E<M00<000=C7@03001C7E<M00<00003Dad0
0000009C7@<001=oo`80021oo`005Woo00<007ooOol06Woo00<007ooOol04goo0`000WmC00<007lc
00000P0000Mk<`00NcAk=000Nc@00002NaD01000MaD000000W<F00@007<FLaH000=Zn007001Zn000
G]X005kJ00000dJL00<002HL00002@000SKO00@004LOAal0009?7P@0009?7P03001C7U<N00<0009C
7P03001C7E<M00@0009C7@03001C7E<M00@0009C7@03001C7E<M00<0009C7@03001oogoo019oo`03
001oogoo01ioo`005Woo00<007ooOol06goo00<007ooOol04Goo100000AoD`000000009o<`05001k
<`00Nc@00002Nc@01P00NaEk5@00MaD000=c5P07001c5P00LaH006[h00000f[h00D005kJG]YNfP00
0096W0@000072I/000000001VP005Ud02@000T/N00<004lNCah010000U<N00<005<NDah00`000e<M
00<005<MDad00`000U<M00<005<MDad0100000=C7@0000000U<M0P0057oo00<007ooOol07Goo000F
Ool2000MOol00`00Oomoo`0?Ool00`00Hid00009000017/c001k=0000W/d00<007/E00000WLE0P00
00Ec5P00LaIc5P00009c5P07001Zn6[hJ_P006[h00000UkJ0`0000A:W000;S`^?08000030EX00000
00@000<bg`040012od;o0002Bah30003Cah00`00DaiC7P030002Dah00`00DaeC7@040002Dad00`00
DaeC7@030002Dad00`00DaeC7@040002Dad00`00Oomoo`0AOol00`00Oomoo`0MOol001Ioo`03001o
ogoo01eoo`8000eoo`T00005NgL007]F001kE@02000017/d001k=0000W/E00<007LE00000WLE00@0
07<FLaH000=c5P800003J_P006[h009Zn0800003G]X004ZL009:W003000^?2hl00800081FPd0009;
7P03001?7TlN00@0009?7P03001C7U<N00<0009C7@03001C7E<M00@0009C7@03001C7E<M00<00003
Dad00000009C7@<001=oo`03001oogoo01aoo`005Woo00<007ooOol07goo00<007ooOol01Woo2000
00ASG@00LgUcN@800006NeH007]ENeD007]E0P0000Mk=000NaEk5@00MaD00002MaD200001G<F001c
5W<F00000W<G00<006[hJ_P00V[h00<005kJG]X0300000<1NP0000000QHm0`000Sko1`000TlN00<0
04lNCah010000U<N00<005<MDad00`000U<M00<005<MDad010000U<M00<005<MDad00`000U<M00<0
07ooOol04Woo00<007ooOol06goo000FOol2000QOol00`00Oomoo`02Ool300001U<M00000000Dad0
009SG@0:001cNG=i001kM`00NeH007]E0002NeD00`00NcD00002NaD00`00MaD00002MaD01000MaIg
5P000W<F00D007<GLaMc5`0000=Zn006001Zn@00001NfP000dZL00L0039L<U`001Wl0001V`070000
0bjo00000002?_l01@00A_m6odKo00000TlN0`000TlN00<005<NDah010000U<N00<005<MDad00`00
0U<M00<005<MDad0100000=C7@0000000U<M0P004Woo0P001goo0`001Woo0P001Goo10000Woo000>
Ool40004Ool00`00Oomoo`0QOol400001E<M00000000Dad00`000V=M00P007=iLgT007]g001kEP00
0W]E00<007]E00000W/e0P0000Ak5@00MaD0009g5@03001g5WLF00800009LaH007<GLaL006lG001Z
n000009Zn@05001NfUkJG]X00002BY`500000aWl0000000200000`GL000F?@0=0002Bah00`00Cai?
7P030002Cah00`00DaiC7P040002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad00`00Oomo
o`0AOol00`00Oomoo`03Ool01@00Oomoogoo00001Goo0P002Goo0@000Goo000BOol01000Oomoo`00
97oo00<005<M00000P0000AC7@00000000=C7@03001SG@00009cN@05001kM`00NeH00002NeD01@00
NeD007/e00000W/e0P0000Ag5@00MaD0009g5P04001g5WLF0002LaL01@00KaM_5flG00000V[i00<0
06Ki00000UkJ0`000TZl00<003IM00000P000PVk1P000bZO00@003[O>]l00096o`03001;7T/N00@0
009?7P03001?7TlN00<0009C7P03001C7E<M00@0009C7@03001C7E<M00<00003Dad00000009C7@<0
01=oo`03001oogoo009oo`05001oogooOol0000@Ool10001Ool000Eoo`H000Moo`03001oogoo0080
025oo`@00004Dad005<M0003Dad00`00DaeC7@0300001W=i001kM`00NeH0009kE@06001kEG]E001k
=@000W/e00<007Lf00000WLE00@007LFMaH0009g5P800004LaL006lGKaL200000f[i001Vn@02I_T2
0002G]/00`00B[a:_00400000b8L0000000300000`WL0000000<0002Bah00`00Cai?7P040002Cah0
0`00DaiC7P030002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00Oomoo`0@Ool20004
Ool01@00Oomoogoo000047oo0@000Goo000BOol01000Oomoo`007Woo1@0000AC7@00DaeC7@8000AC
7@<0009K?@<00004NgL007]F0002NeD00`00NeD00002NcD00`00NcD00002McH2000017LE001g5P00
0WLF00@007<FLaH0009_5`04001_5flG0003I_T20003G]/3000014jl000jOCYm0`0000<Ag0000000
0`000QXm0`000c[O00<004;o@_l01P000TlN00<004lNCah010000U<N00<005<MDad00`000U<M00<0
05<MDad01@0000AC7@00DaeC7@8001=oo`05001oogooOol00003Ool00`00Oomoo`0:Ool40002Ool0
00ioo`@000=oo`03001oogoo01Uoo`D00004Dad005<MDad30002Dad01000Dad000000U<M00<005/m
00000fML1@0000IkE@00NeD007/e0002NcD00`00McH00002McH200001GLF001g5WLF00000W<F00<0
06lG00000flG0P0000UVn@00I_UVn@00G]/005kK00000djl00<003Ym00000RHm0`000PgL1@0000<Z
WP00000010000T;o00@004[nB_h0009?7P<0009?7P03001C7U<N00@0009C7P03001C7E<M00<0009C
7@03001C7E<M00@0009C7@03001oogoo00moo`<0009oo`05001oogooOol0000<Ool00`00Oomoo`03
Ool000ioo`03001oogoo00Aoo`8001Moo`L0009C7@<0009C7@03001C7E<M00<000=C7@8000=WG004
001_FVmJ0002LeT200001W]E001k=@00NcD0009g=P06001g=WLf001g=P000WLF00H007<FLaH007<F
0003KaL01000KaQ_600016Ki00<005kKG]/02`0000LIo0000000018M000NGP0@0002Bah00`00Cai?
7P030002Cah00`00DaiC7P040002Dad00`00DaeC7@0300000e<M00000002Dad3000=Ool300000goo
00000002Ool01@00Oomoogoo000037oo00<007ooOol00goo000>Ool00`00Oomoo`04Ool00`00Oomo
o`0COol60002Dad40002Dad00`00DaeC7@030003Dad00`00Dad00002Ead00`00Ie`00002KeX01P00
LeUcFG=I001cF0800006NcD007/e001g=P000WLf00<007LfMcH00P0000Ag5P00LaH0009c5P800003
KaL006lH009_6004001VnFKiI_T2000015kK001Nn`000djl00D003Ym>Wd002Xm00@000035O`00000
00L0008fg`040012od;o0002A_h70002Cah00`00Cai?7P030002Dah00`00DaeC7@040002Dad00`00
DaeC7@030002Dad00`00Oomoo`08Ool30005Ool01000Oomoogoo0`003Goo10000Woo000>Ool40002
Ool00`00Oomoo`0AOol60002Dad30003Dad00`00DaeC7@030002Dad00`00DaeC7@040002Ead30000
0fmJ001cF@02LeT01@00LeQcF000LeP00P0000Ak=@00McH0009g=P03001g=P00009g=P06001c5W<F
001c5P000W<G00T006lHKaQ_6000KaP006Ki00000VKi00D005kkG_]Nn`00009>_0L0008N7@<00003
6Qd0000000800003:Yh0000000L00096oP05001:oT[nB_h00002Cah30002Cah00`00DaiC7P030002
Dad00`00DaeC7@060002Dad30005Ool40008Ool00`00Oomoo`0GOol001Aoo`8000moo`H0009C7@<0
00=C7@03001C7E<M00<0009C7@03001C7E<M00<000=C7@<0009K?@8000=cF@04001cF7=H0002LeP0
2000MeP00000McH007Lf0002McH01P00McIg=P00LaH0009c5P03001c5g<G00800003KaP006lH009_
6006001VnFKi001Vn@000Ukk0`000Tjl00D003jM?Yd002iM00<0008J7@<0008RGP<0008f_`05000n
ocko?_l00002A_h7000014lN001?7TlN10000U<N00<005<MDad00`000U<M00<005<MDad00`000U<M
00<007ooOol00`0037oo00<007ooOol05goo000DOol00`00Oomoo`0;Ool60002Dad30003Dad00`00
DaeC7@030002Dad00`00DaeC7@030002Dad01000Dad000000ULM00<005/m00000V<l00<007=I0000
0W=H00@007=HLeP0009gF008001gE`00001g=P00McH0009g=P03001g=P00009c5P04001c5g<G0002
LaL00`00KaQ_6002KaP00`00I_T00003I_T20002G_/00`00C[a>_00700000b8m00000002000011Xm
000000000RjN2@000TKn00<004[nB_h01P000TlN00<005<NDah010000U<N00<005<MDad00`0000=C
7@0000000U<M0`0047oo00<007ooOol05Woo000DOol00`00Oomoo`08Ool300000e<M00000002Dad3
0002Dad00`00DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@030003Ead20003Hc`30000
1G=H001cF7=H00000WMH00H007MGMeL007MG0002McH00`00McH00002McH200001G<F001c5g<G0000
0W<G00L006lHKaQ_6000KaP00003I_T00`00I_X00002G_/300001DkL0012WD:M00000S9M0`0000<N
?@0000000P0000<RGP0000003P000T[n00<004lNCah01P000TlN00<005<NDah0100000AC7@00DaeC
7@@0009C7@03001oogoo00ioo`03001oogoo01Ioo`0057oo0P001Woo0`0000EC7@00000005<M00<0
009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00<000=C7@03001G7@00009G
7@03001S?00000=WF`800005LeP007MHMeP00002MeL00`00MeMgE`0200001WLf001g=P00McH0009g
=`04001c5g<G0002LaL01@00LaL006lH00000VlH0P0000=Vn@00I_X00VKj0P0000AJn`00C]a>g0L0
008V?@<0008N?@H000<f_`04000ngckO0002A_h00`00B_i:oP0400000dkn001?7P02Cah00`00Cai?
7P030002Dah00`00DaeC7@0300000e<M00000002Dad3000?Ool00`00Oomoo`0FOol001Aoo`03001o
ogoo009oo`<00005Dad00000001C7@030002Dad00`00DaeC7@030003Dad00`00DaeC7@030002Dad0
0`00DaeC7@030002Dad00`00DaeC7@040002Ead20003Ie/00`00Je][F`0200001GMH001gEgMG0000
0WMG00H007LgMcL007Lf0002McH00`00McMg=`02000017<G001c5`000g<G00@006lHKaP0009_6004
001VnVKjI_X20002F_/30002D]`01@00@Ye2W@00=Wd00`000R8m1`0000<^WP0000002P000T[n00<0
04knC_h01`0000=?7P0000001@000U<M00<005<MDad00`000U<M00<007ooOol03Goo00<007ooOol0
5Woo000DOol00`00Ool0000200001E<M00000000Dad00`000U<M00<005<MDad00`000e<M00<005<M
Dad00`000U<M00<005<MDad00`000U<M00<005<MDad00`000e<M0`000U/m00<006MK00000V]K00@0
06]JJeX000=gE`03001gE`0000=g=`80009g=P04001g=gLg0002LcL200000g<G001c5`02LaL00`00
KaP00002KaP01@00JaP006Kj00000VKj0P000U[k00<005;LD]`01`0000<ZG@0000001@0000<VOP00
00000P000SJn00@003kN?]h00096oPL0009:oP03001?7TlN00<0009?7P03001C7U<N00L00003Dad0
000000@000ioo`03001oogoo01Ioo`004goo00@007oo001C7@<00004Dad000000002Dad00`00DaeC
7@030003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030003Dad00`00Dad0
0002Dad00`00FceK?@0300000f]K001[FP02JeX200001GMG001gEgMG00000WLg00P007<g000007Lf
001g=`000g<g00<007<g00000g<G00H007<G000006lH0003JaP00`00I_YVnP020003F_/300001E;L
0012_D:m00000SIm1P000RIN0`000RjN20000T;n00D004[nB_i:oP00009:oP<0009?7P03001?7TlN
00<0009C7P03001C7E<M00<00003Dad00000009C7@8000eoo`03001oogoo01Ioo`002goo0`001Goo
100000AC7@000000009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001C7E<M00<0009C
7@03001C7E<M00<0009C7@03001C7E<M00@0009C7@<0009O?@8000=[FP03001_FVmJ00800007MeL0
07LgMcL007<g00000W<g0P0000Eg=`00LcMc=`00009c=`800007LaL007<GLaL006lH00000f/H0P00
00UVnP00IaYW6P00F_/005[k00000e;L00<004:m00000`000biM00<002YM00000P000RYn4`0000A:
oP00Cai?7P@00004Cah005<NDah40002Dah00`00Dad000030002Dad00`00Oomoo`0:Ool00`00Oomo
o`0FOol000Yoo`05001oogooOol00004Ool200000goo00000002Dad00`00DaeC7@030002Dad00`00
DaeC7@040002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030003Dad30002
Ead00`00Gcd00002Gc`00`00JeX00002KeX00`00KeU_F@0200001GLg001c=g<g00000W<g00<007<g
00000W<g00@007<gLcL0009c=`04001c5g<G0002KaP200001F/H001[6F/I00000fLJ0P000U[k1P00
0TJm00<003ZM00001P0000<ZGP0000000P000S:N00@003Jn=[h0008ngP050012oT;n@_h00002A_h6
0002Cah00`00Cai?7P030002Cah00`00DaiC7P060002Dad3000<Ool00`00Oomoo`0EOol000Yoo`05
001oogooOol00006Ool00`00Ool00002Dad30002Dad00`00DaeC7@030003Dad00`00DaeC7@030002
Dad00`00DaeC7@030002Dad00`00DaeC7@030003Dad00`00Dad00002Dad00`00EaeG7@020003Gc`3
00000fmJ001_F@02KeT00`00KeT00002LcL01000LcMc=`000W<g0P0000Ec=`00LcMc=`00009c=`03
001c5`00009_6004001_66lH0002JaT02@00JaT006LJ001W6P00F_/00002F_/01000D]aBg0000TJm
10000S9m00<002YN:Uh00`0000<ZOP0000003P0000A6oP00B_i:oPL0009?7P03001?7TlN00<0009C
7P03001C7E<M00<0009C7@03001oogoo00Yoo`03001oogoo01Eoo`002Woo00D007ooOomoo`0000Mo
o`04001oo`000003Dad00`00Dad000020003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad0
0`00DaeC7@030002Dad00`00DaeC7@040002Dad30002Ead00`00Gc`00002Hc`20003KeT00`00KeU_
F@02000017<g001c=`000W<g00@007<gLcL0009c=`06001c=g<g001c=`000VlH00L006lHKaQ_6000
JaT00003JaT00`00IaYW6P020003F_/6000014Jm000jWCZM1P000RYn1P000cJn00@003kN?]h00092
oPH0009:oP04001>oTkn0002Cah4000014lN001C7U<N1@0000AC7@00DaeC7@8000]oo`03001oogoo
01Eoo`002Woo00D007ooOomoo`0000Qoo`04001oogoo0002Dad20003Dad00`00DaeC7@030002Dad0
0`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@040002Dad30002Dad00`00EaeG7@03
00001f<l001S?6<l001_F@00009_F@03001_FFmI00800004LcL007<g0003LcL2000017<g001c=`00
0W<g0P0000M_6000KaQ_6000KaP00003JaT200002FLJ001S6V<J001Jn`00F_/00002D]`00`00A[e6
_@040003<Wd00`00;Wh0000600000c:N000000070002@_h01000A_i6oP000T[n1P000TlN00<004lN
Cah00`000U<N00<005<MDad0100000=C7@00Ool02Woo00<007ooOol05Goo000:Ool01@00Oomoogoo
00002Goo0P0000=oo`0000000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M00<005<MDad0
0`000U<M00<005<MDad00`000U<M10000U<M00<005<MDad00`000U/M0P000f<l0`0000=_F@00KeT0
0VmI00H006mI000007<g0002LcL01P00LcQc>000LcL0009c=`04001c=g<g0002KaP00`00KaQ_6002
00001F/I001[6F/I00000f<J0P000e[k1@000TJm00<003jM00002@0000<^WP0000004@000T[n00<0
04knC_h01P000TlN00<005<NDah00`0000=C7@0000000U<M0P002Woo00<007ooOol05Goo000:Ool0
1@00Oomoogoo00002goo00<007oo00000U<M0`000U<M00<005<MDad00`000U<M00<005<MDad00`00
0U<M00<005<MDad00`000U<M0`000e<M00<005<MDad00`000ULM00<005/M00000U/m00<006<l0000
0fLk00D006mIKeU_F@00009_F@03001_F@00009c=`04001c>7<hLcP20002LcL01000LcMc=g<g0P00
00A_6000KaP0009_6008001[6F/I001[6@00HaX0009S6P05001Jn`00F_/00002D]`00`00A[e6_@07
0002<Wh00`00;Wh^OP060002>[h01@00?]hngSkN00000T;n00<004KnA_h01`0000A>oP00Cai?7PL0
0003Dah0000000D00003Dad007oo00Uoo`03001oogoo01Eoo`002goo0`003Goo00@007oo0000009C
7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@<000=C7@03001C7E<M
00<0009C7@03001G7ELM008000=K?@8000=W>`05001WF`00KeT00002KeT00`00KeU_F@0200001W<g
001c>7<h001c>0800005LcL007<gLcL00002LcP00`00KaP00003KaP00`00JaT00003JaT01000HaYS
6V<J0P000U[k1P0000E:_@00?YdnW@00008fWPH0008bWP03000fWSJN00`00096oP04001:oT[n0002
C_h30002Cah00`00Cai?7P030002Cah00`00Dah000060008Ool00`00Oomoo`0EOol001aoo`<0009C
7@<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@<0009C7@03001C7E<M00@0009C7@03
001C7E<M00<0009G7@03001K?@00009O?@03001W>`00009WF`<00005KeT006mIKeT00002KcT20000
17<h001c>0000W<h0P0000=c=`00LcP00W<h0P000flH00<006lH00000f/I00/006/I001S6P00HaX0
05[k001Jn`00009Bg0/0008bOP03000bOP0000T0008ngP030012gT;N00X0009:oP03001?7TlN00<0
0004Cah004lNCah60002Dad30008Ool00`00Oomoo`0DOol001ioo`<0009C7@03001C7E<M00<0009C
7@03001C7E<M00<0009C7@<0009C7@03001C7E<M00<000=C7@03001C7E<M00<0009C7@03001G7ELM
00<0009O?@<00003Ie/006]K009[F`800003KeT006li009_>@05001_>@00LcP00003LcP00`00LcP0
0003LcP01P00KcP00000KaP0009_60800003JaT006/I009[6@03001S6V<J008000=Jn`80009Bg004
001:gD[M0002@[d00`00>Yh000090002=[h00`00>[hj_P050002@]h01000A_i6oP000T[n1`0000A>
oP00Cai?7P<0009?7P03001C7U<N00<0009C7@03001oogoo00Ioo`03001oogoo01Aoo`007goo0P00
0U<M0`000U<M1P000U<M0`000U<M00<005<MDad00`000e<M00<005<MDad00`000U<M00<005<MDad0
0`000ULM0`000Ull0P000f]K00@006]JJeX000=_>@06001_>@00001c>0000W<h00<007<hLcP00P00
00Ac>000KcP0009_>004001_66lH0002KaT01000JaU[6F/I0P0000US6P00HaYS6P00F_/005[k0000
0U;L3@000S:N00<003JN=Yh04P000T[n00<004[nB_h01P000TlN00<004lNCah00`0000=C7P000000
0U<M0P001goo00<007ooOol057oo000POol30002Dad30002Dad00`00DaeC7@030003Dad00`00Dad0
00020003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00Ead00002Ead00`00Gc`00002
Gc`00`00Je/00002JeX01`00JcX006li001_>@00009_>@800005LcP007<hLcP00002LcP00`00KcP0
0003KcP2000016lH001_6FlI0P0000E[6@00JaU[6@0000=S6P80009Jn`<00005D]`004[MB]d00002
@[d01000>YhjWP000SJN2P000SZn00@003kN?]h00092gP030016oTKn00L00004B_h004knC_h70000
14lN001C7U<N100000=C7@00Ool01Woo00<007ooOol057oo000QOol40003Dad00`00Dad000020003
Dad00`00DaeC7@020003Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@030002
Ead20003Gc`3000016]J001[>V/j0P0000=_>@00KcT00Vli00<006li00000W<h00L007<hLcQc>000
KcP00002KcP01000KcQ_>0000VlI00@006lIKaT0009[6@0;001W6P00HaX006<J001Jn`00F_/00002
D]`A0002=Yh00`00=[h0000;0002A_h00`00B_i:oP060002C_h00`00Cai?7P0600000e<N00000004
0006Ool00`00Oomoo`0DOol002=oo`03001oo`00009C7@8000=C7@03001C7E<M00<0009C7@03001C
7E<M00<0009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@<0009K7@03001O?000009S?080
009[>P04001[>V/j0003KcT00`00KcU_>@0200001W<h001c>7<h001c>0800003KcP006lh009_>003
001_6@00009_6@800004JaT006LJIaX20002HaX3000015[k000000000U;l00@004[MB]d00092_@04
000nWSjN0002>Yh00`00=Yh0000E00001D[n001:oT[n00000Tkn100000A?7P00Cai?7PH0009C7@80
00Eoo`03001oogoo01Aoo`0097oo00@007oo0000009C7@03001C7E<M00<0009C7@03001C7E<M00<0
009C7@03001C7E<M00<0009C7@03001C7E<M00<0009C7@<0009G7@03001K7E/M00800007Hc`00000
Hc`006/j00000V/j00D006/j001_>@00009_>@03001_>Fli00800007LcP007<hLcP006lh00000flh
00H006lh000006lI0002JaT200001VLJ001W6P00HaX0009S6`80009Jn`03001Bo5;l01<0008j_P04
000j_SZn0003?]h01000@]i2gP000TKn2@000Tkn00<004lNCah00`000TlN00<005<N00000`000U<M
00<007ooOol00Woo00<007ooOol057oo000UOol00`00Ool00002Dad30002Dad00`00DaeC7@030002
Dad00`00DaeC7@030002Dad60002Dad3000015<M001G7ELM100000=K?@0000000f<l0`0000=[>P00
JcX00V/j0P0000=_>@00KcT00Vli00<006li00000W<h00<006lhKcP00P0000A_>000KcP0009_>004
001[6F/I0002JaT00`00IaYW6P03000016<K001S6`000U[k0`0000ABo000B]e:g@X0008fWP03000j
_SZn00l00003A_h004[n009:oP03001:oT[n00H0009>oP03001?7TlN00D0009C7@<000Aoo`03001o
ogoo01=oo`009Woo10000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M0`000U<M00<005<M
Dad00`000U<M00<005<M00000P000ULM00<005/m00000U/m00<006<l00000V<l0P000f/j00@006/j
JcX000=_>@06001_>@00001c>0000Vlh00H006lhKcP006lh0003KcP00`00JaT00003JaT00`00IaX0
0003IaX00`00Ha]S6`03000015[k001Bo5;l0`000TkM00@004JmA[d0008n_P03000j_P0000`0008n
gPd00004B_h004knC_h7000014lN001?7TlN0`000U<N00<007ooOol00Woo00<007ooOol04goo000X
Ool20002Dad30002Dad00`00DaeC7@030002Dad30002Dad00`00DaeC7@030002Dad00`00DaeC7@02
0002Dad00`00EaeG7@030002Fcd300001f<l001S>f<k001[>P00009[>P07001[>P00KcT006li0000
0Vli0P0000=_>000KcP00Vlh0P000flh00<006li00000f/I00<006/I00000fLJ0P0000IS6`00Ha]S
6`00F_/30002E_`00`00C]e>g@0=0002>[h01000>[hj_P000Sjn0P0000E2gP00@]i2gP0000=6oP03
001:oT[n00H0009>oP03001>oTkn00H00003Cah0000000@000=oo`03001oogoo01=oo`00:Goo0`00
0U<M1P000U<M0`000U<M00<005<MDad00`000U<M00<005<MDad00`0000AC7@00DaeC7@<0009G7@<0
009K?@8000=S>`<00003JcX006/j009[>P03001_>@00009_>@04001_>Fli0003KcP04P00KcP00000
KcP006li001_>@00JaT006/I001[6@00IaX0009W6P04001S6f<KHa/20002F_`00`00E_aFo0030000
1DkM0016_DJm00000T:n00@003jn?[h0008j_QD00004B_h004[nB_h7000014kn001?7TlN100000A?
7P00DaiC7P80009oo`03001oogoo01=oo`00:Woo00@007oo0000009C7@03001C7E<M00<0009C7@03
001C7E<M00<0009C7@03001C7E<M00<00004Dad005<MDad30002Dad30002Ead00`00Fcd00002Gc`0
0`00Hc/00002Ic/20003JcX01P00JcX00000KcT000=_>@800004KcP006lh0002KcP00`00KcT00003
KcT00`00JaT00003JaT02P00IaYW6P00IaX006<K001S6`000U[l0`0000=Fo000000050000SkN00D0
04;N@]i2gP000092gPT0009:oP03001>oTkn00H00004Cah004lNCah400001E<N001oogoo00005Goo
000[Ool30002Dad30002Dad00`00DaeC7@030002Dad00`00DaeC7@030002Dad00`00Dad000020002
Dad30002Dad00`00EaeG7@020003Gc`300002FLk001W>fLk001[>P00JcX00002JcX01@00KcU_>Fli
00000Vli00<006lh00000flh0P000fli00<006li00000f/I0P0000AW6P00IaYW6P80009S6`<00004
F_`000000002E_`00`00C]d00002A]d70002>[h01P00>[hj_SZn000n_PT00005@]h004KnA_h00002
B_h7000014kn001>oTkn1P0000=?7P0000000U<N0P0000=oo`00Ool057oo000]Ool30002Dad00`00
DaeC7@030002Dad00`00DaeC7@030002Dad00`00DaeC7@020002Dad30002Dad00`00DaeC7@030002
Ead00`00Gc`00002Gc`20003Ic/200000f/j001[>P02JcX200001fli001_>Fli001_>@0000=_>006
001_>@00001_>@000Vli0P0000A[6@00JaU[6@80000:IaX006LJ001S6`00Ha/005[l0002F_`00`00
E_aFo0050002A]d01@00@[i2_P00?[h060000T[n00<004[nB_h01P0000A>oP00Cai?7PL00004Dah0
0000000EOol002ioo`80009C7@<0009C7@03001C7E<M00<0009C7@H00004Dad000000002Dad00`00
DaeC7@030002Ead00`00EaeG7@03000025ll001O?5ll001W>`00Ic/20003JcX00`00JcY[>P020000
16li001_>Fli0P0000A_>000KcT0009_>@04001_>Fli0002KcT01000JaU[6@000V/I00<006LJIaX0
0`0000AS6`000000009Jo0<00004E_`004kMC]d@0002?[h01000?]hngP000T;N00<004;N@]h02P00
00A:oP00C_i>oPH0009>oP03001?7P0000<00004Dah007oo000DOol002moo`<0009C7@H0009C7@<0
009C7@03001C7E<M0080009C7@03001C7E<M00<0009C7@03001G7ELM00<0009K7@8000=O?0800007
Ic/006LkIc/006/j00000V/j00@006/jJcX0009_>@06001_>Fli001_>@000fli0P0000=_>@00KcT0
0P0000A[6@00JaU[6@800003IaX006LJ009W6P03001S6f<K00<00004F_`005KlE_`300001DkM001:
gD[M00000T:m00@003jn?[h0008n_P03000n_Sjn00/00092gP040016oTKn0002B_h6000014kn001>
oTkn2@000U<N0`0057oo000aOol30002Dad00`00DaeC7@030002Dad00`00DaeC7@03000015<M001C
7E<M0`000U<M00<005<MDad00`000ULM0`000U/M00@005ll001O?08000=W>`<00004JcX006/jJcX2
00001fli001_>Fli001_>@0000=_>@06001_>Fli001_>@000V/i0P0000E[6@00JaY[6P0000=W6P80
0004Ha/006<K0002F_`3000015Kl001>gDkM8P0000A:oP00B_i:oPH0009>oP03001>oTkn00H0009o
o`03001oogoo019oo`00<Woo0P000U<M0`000U<M00<005<MDad00`000U<M00<005<M00000P000U<M
00<005<MDad00`000U<M0`000ULM00<005/MFad00P0000MO?000GcaO?000Ic/00002Ic/3000016/j
001[>V/j0P0000A_>@00KcU_>@800007KcT006liKcT006li00000f/i00<006/i00000V/J00H006/J
001W6P00IaX20002Ha/30002F_`50002C]d00`00B]d0000<0003?[h01000@]i2gP000T;N00<004KN
A]h02@0000=:oP00B_h00T[n00<004knC_h00`0017oo00D007ooOomoo`0001Aoo`00<goo0`000U<M
00<005<MDad00`000U<M00<005<MDad00P000U<M1P000U<M0`000U<M00<005LMEad00`000U/M00@0
05llGcaO?0<00003Ic/006Lk009W>`04001[>V/j0002JcX01000KcU_>@000Vli0P0000A_>@00KcU_
>@800003JcT006/i00800004JaX006/JJaX200001VLJ001W6P00Ha/0009S6`05001Jo000F_`00002
E_`50002B]d01000A]e6g@000T:n00D003jn?[h003jn00`00005A]h004KnA_h00002B_h:0007Ool0
1@00Oomoogoo000057oo000dOol20002Dad30002Dad6000015<M000000000U<M00<005<MDad00`00
0U<M00<005<MDad00`000ULM00L005/M001K?@00Gc`00002Hc`20003Ic/200002f/j001[>V/j001[
>P00KcT006li00000Vli00@006liKcT0009_>@03001[>@00009[>@800005JaX006/JJaX00003IaX2
0002Ha/20002F_`3000015Kl001>gDkM8@004Woo00@007ooOol001Aoo`00=Woo0P000U<M0`000U<M
00<005<MDad00P000U<M00<005<MDad00`000U<M00<005<MDad00`000U<M00<005LMEad00P000U/m
0`0000QS?000HcaS?000Ic/006Lk0`0000=[>P00JcX00V/j00<006li00000Vli0P0000M_>@00KcU_
>@00KcT00003JcT00`00JcT00002JaX01@00IaX006LJ00000VLJ0P0000IO6`00000005[l0002E_`3
000dOol00`00Ool0000DOol003Moo`<0009C7@03001C7E<M00<00004Dad005<MDad30002Dad00`00
DaeC7@030002Dad6000015LM000000000U/m0P000f<l0P0000=W>`00Ic/00VLk00H006/jJcY[>P00
JcX2000016li001_>Fli0P0000U_>@00KcT006/i001[>@00JcT00P0000A[6P00IaYW6P80009W6P80
0004Ga/005lK0002F_`3000gOol01000Oomoo`004goo000hOol20002Dad30002Dad00`00Dad00002
0002Dad00`00DaeC7@030002Dad3000015<M001C7E<M0`000ULM00<005/m00000U/m00@006<l001S
?08000=W>`07001W>`00JcX006/j00000V/j00@006liKcT0009_>@03001_>@0000=[>@03001[>@00
009[>@800007IaX006LJ001W6P00HaX00P000elK0P00>goo00<007oo00004goo000iOol30002Dad6
00000e<M000000040002Dad30002Dad00`00Dad000020002Ead00`00EaeG7@030002Fcd200001f<l
001S?6<l001W>`00009W>`800004JcX006/jJcX200001fli001_>Fli001_>@0000=[>@03001[>@00
00=[>@03001[>@00009W6P800007HaX006<KHa/005lK0000?Woo0P004goo000kOol30002Dad00`00
DaeC7@020002Dad00`00DaeC7@030002Dad00`00DaeC7@020002Dad00`00EaeG7@030002Ead30002
Gc`01000HcaS?6<l0`0000=W>`00Ic/00`0000A[>P00JcY[>P800004KcT006liKcT200002F/i001[
>@00JcT006/i001[>@0200001FLJ001W6VLJ00000f<K0P00?Woo0`004goo000lOol20002Dad30000
15<M001C7E<M0`000U<M00<005<MDad00`0000=C7@00000010000ULM0`000ULM00L005ll001O?000
Hc`00002Hc/200000fLk001W>P02IcX01000JcY[>P000V/j00@006liKcT0009[>@03001[>@0000=[
>@03001[>@00009[>P800006IaX006LJ001S6`00?7oo10005Woo000mOol30002Dad00`00Dad00002
0002Dad60002Dad20002Dad00`00DaeC7@030002Ead00`00EaeG7@020002Gc`300000f<k001S>`02
0003IcX2000016/j001[>V/j0P0000M_>@00JcU[>@00JcT00003JcT00`00JcT00002JcX01000JcY[
>P000VLJ0P00>Goo10006Woo000nOol60002Dad30002Dad00`00DaeC7@03000015<M001C7E<M0`00
0U<M00<005LMEad00`0000AK7@000000009O?0800007Hc/006<kHc/006Lj00000VLj0P0000A[>P00
JcY[>P800009JcT006/i001[>@00JcT006/i00800004JcX006/jJcX200000fLJ001oo`0fOol4000N
Ool0041oo`800004Dad000000002Dad00`00DaeC7@03000015<M001C7E<M0`000U<M00<005<MDad0
0`0000AG7@000000009K7@<0009O?004001S>f<kHc/3000016Lj001W>VLj0P0000M[>P00JcY[>P00
JcT00003JcT00`00JcT00002JcT200001F/j001[>V/j0000=7oo10008Woo0011Ool3000015<M001C
7E<M0`000U<M1@000U<M1P0000AC7@000000009G7@03001K7@00009K7@07001O?000Gc`006<k0000
0V<k0P000VLj00@006LjIcX0009[>P03001[>P0000=[>@03001[>@0000=[>@03001[>P00009[>P03
001oogoo02moo`@002Ioo`00@Woo1P000U<M0`0000AC7@00DaeC7@<0009C7@03001C7E<M0080009C
7@03001G7ELM00<0009K7@80009O?0<00006Hc/006<k001W>P000VLj0P0000A[>P00JcY[>P800009
JcT006/i001[>P00JcT006/j00800003JcX007oo02eoo`@002Yoo`00A7oo00<005<MDad00`000U<M
00<005<M00000P000U<M00<005<MDad00`0000AC7@00DaeC7@<0009G7@<00004Fad000000002Gc`2
0002Hc/300000fLj001W>P0300000f/j001[>P02000016/i001[>V/j0P0000E[>P00JcY[>P0002ao
o`<002ioo`00AGoo0P000U<M00<005<MDad00P000U<M1P000U<M1@000U<M0`0000AG7@00Fad0009K
7@07001O?000Gc`006<k00000VLk0P0000=W>P00IcX00VLj00<006/j00000V/j0P0000A[>P00JcY[
>P800003JcX007oo02Qoo`@0035oo`00AWoo1P0000AC7@000000009C7@03001C7E<M00<00004Dad0
05<MDad30002Ead00`00Ead000020002Fad20002Gc`300000fLk001W>`020003IcX00`00IcX00003
JcX00`00JcX00002JcX01000JcY[>P009Woo1000=Goo0017Ool20002Dad20002Dad00`00DaeC7@03
000015<M001C7E<M0`000U<M1@000ULM0`0000AK?@000000009O?080000>Ic/006LkIc/006Lj001W
>P00IcX006/j001[>P800005JcX006/jJcX0000SOol4000iOol004Uoo`800003Dad0000000@0009C
7@D0009C7@<00004Dad005LMEad30002Ead20002Fcd00`00Gc`00002Gc`01000Ic]W>fLk0P0000AW
>P00IcYW>P800004JcX006/jJcX200000f/j001oo`0OOol4000mOol004Yoo`80009C7@03001C7E<M
00<00004Dad005<MDad30002Dad00`00Dad000020002Ead3000015LM001K?E/m0`0000IO?000Hc`0
06Lk0002Ic/200000fLj001W>P0300001F/j001[>V/j00007Goo1000@Goo001;Ool00`00DaeC7@03
0002Dad00`00Dad000020002Dad50002Dad3000015LM001G7ELM0`000U/m0P000V<l0`0000=W>`00
Ic/00P0000=W>P00IcX00VLj00@006/jJcX001Yoo`@004Eoo`00C7oo0P000U<M1@000U<M0`000U<M
00<005<M00000P000U<M00<005LM00000P000ULM0`0000AK?0000000009S?0800005Ic/006LkIc/0
0003IcX200000f/j001oo`0FOol40019Ool004ioo`80009C7@03001C7@000080009C7@03001C7E<M
0080009C7@D0009G7@<00004Ead005/l0002Gc`01000Hc`006<l0`0000MW>`00Ic/006Lj001W>P02
000DOol4001=Ool004moo`D0009C7@H00004Dad000000002Dad00`00Dad000020002Ead00`00Ead0
00020002Gc`2000016<l001S?6<l0P000VLk0P0000=W>P00Ool04Goo1000DGoo001@Ool2000015<M
000000000U<M00<005<M00000P000U<M00<005<MDad00P000U<M1@000U/M0`0000YO?000000006<l
001S?000Ic/0009W>`03001oogoo00eoo`@005Eoo`00DGoo0P000U<M00<005<MDad00P000U<M1P00
00IC7@00000005<M0002Ead30002Fad20002Gc`20002Hc`300000fLk001oo`0;Ool4001IOol005=o
o`D00004Dad000000002Dad00`00Dad000020002Dad00`00Dad000020002Ead3000015/M00000000
0Ull0P000V<l0P002Goo1000GGoo001DOol00`00DaeC7@020002Dad50002Dad50002Dad3000015LM
001K7@000U/M00L005ll001O?000Hc`00007Ool4001QOol005Eoo`800004Dad000000002Dad00`00
Dad000020002Dad00`00Dad000020002Ead50002Fad20002Gc`20004Ool4001UOol005Moo`04001C
7E<M0002Dad20002Dad50002Dad3000015LM001G7ELM0`0000AK7@00000000=oo`<006Uoo`00F7oo
1@0000AC7@000000009C7@03001C7@0000800004Dad005LMEad3000015LM000000000Woo1000K7oo
001IOol00`00DaeC7@020002Dad50002Dad50002Ead200000goo00000002001`Ool005Yoo`800006
Dad00000001C7@000U<M0`0000AC7@00DaeC7@H007Aoo`00G7oo00<005<MDad01@000U<M1`00N7oo
001MOol2000015<M001C7E<M1P00O7oo001NOol70020Ool005moo`8008Aoo`00iGoo003UOol00>Eo
o`00iGoo003UOol00>Eoo`00iGoo003UOol00>Eoo`00\
\>"],
  ImageRangeCache->{{{0, 228.125}, {184.812, 0}} -> {-0.10018, -0.0478434, 
  0.00629509, 0.00629509}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.7 Exploring Vega", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallVega[p,k,sd,r,q,t]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the Vega of a Call option, the sensitivity of its value to changes \
in the volatility."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot3D[BlackScholesCallVega[x, 60, 0.29, 0.04, 0, t], {x, \
20.,100.},{t,0.01,0.5}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76225 .08907 0 -6.16962 ]
[.76225 .08907 18 2.83038 ]
[.82456 .18447 0 -6.07766 ]
[.82456 .18447 18 2.92234 ]
[.88151 .27169 0 -5.9953 ]
[.88151 .27169 18 3.0047 ]
[.93378 .35174 0 -5.92111 ]
[.93378 .35174 18 3.07889 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 18 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01388 .32923 -6 -2.79655 ]
[.01388 .32923 0 6.20345 ]
[.00322 .39608 -12 -2.85333 ]
[.00322 .39608 0 6.14667 ]
[-0.00791 .46584 -12 -2.91284 ]
[-0.00791 .46584 0 6.08716 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74995 .09363 m
.7438 .09592 L
s
[(0.1)] .76225 .08907 -1 .37103 Mshowa
.81217 .18881 m
.80598 .19098 L
s
[(0.2)] .82456 .18447 -1 .35059 Mshowa
.86906 .27583 m
.86283 .2779 L
s
[(0.3)] .88151 .27169 -1 .33229 Mshowa
.92126 .35569 m
.91501 .35766 L
s
[(0.4)] .93378 .35174 -1 .3158 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(0.5)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.76286 .11338 m
.75916 .11473 L
s
.77553 .13276 m
.77182 .1341 L
s
.78796 .15178 m
.78426 .15311 L
s
.80018 .17047 m
.79647 .17178 L
s
.82395 .20683 m
.82023 .20812 L
s
.83553 .22454 m
.8318 .22582 L
s
.8469 .24193 m
.84317 .2432 L
s
.85807 .25903 m
.85434 .26028 L
s
.87985 .29234 m
.87611 .29357 L
s
.89047 .30858 m
.88673 .3098 L
s
.90091 .32455 m
.89716 .32575 L
s
.91117 .34025 m
.90742 .34144 L
s
.9312 .37088 m
.92744 .37205 L
s
.94096 .38582 m
.9372 .38699 L
s
.95058 .40053 m
.94681 .40168 L
s
.96004 .415 m
.95627 .41614 L
s
.7368 .07352 m
.73311 .0749 L
s
.72339 .05301 m
.71971 .05441 L
s
.70973 .03212 m
.70606 .03353 L
s
.6958 .01081 m
.69213 .01224 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02615 .32459 m
.03229 .32226 L
s
[(5)] .01388 .32923 1 -0.37854 Mshowa
.01554 .39157 m
.0217 .38932 L
s
[(10)] .00322 .39608 1 -0.36593 Mshowa
.00446 .46148 m
.01065 .45929 L
s
[(15)] -0.00791 .46584 1 -0.3527 Mshowa
.125 Mabswid
.03433 .27297 m
.038 .27155 L
s
.03231 .28572 m
.03599 .2843 L
s
.03028 .29857 m
.03395 .29716 L
s
.02822 .31152 m
.0319 .31012 L
s
.02407 .33776 m
.02775 .33637 L
s
.02196 .35104 m
.02565 .34967 L
s
.01984 .36444 m
.02353 .36307 L
s
.0177 .37795 m
.02139 .37659 L
s
.01336 .40532 m
.01706 .40397 L
s
.01116 .41918 m
.01487 .41784 L
s
.00895 .43315 m
.01266 .43183 L
s
.00672 .44725 m
.01042 .44594 L
s
.00219 .47582 m
.0059 .47452 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.664 .741 .908 r
.39729 .59642 .40512 .60371 .41826 .59977 .41047 .59244 Metetra
.663 .741 .908 r
.41047 .59244 .41826 .59977 .43147 .59582 .42372 .58845 Metetra
.663 .741 .908 r
.42372 .58845 .43147 .59582 .44474 .59188 .43703 .58447 Metetra
.66 .74 .909 r
.43703 .58447 .44474 .59188 .45807 .58804 .4504 .58057 Metetra
.651 .737 .912 r
.4504 .58057 .45807 .58804 .47146 .58453 .46384 .57695 Metetra
.63 .729 .919 r
.46384 .57695 .47146 .58453 .48492 .58176 .47733 .57401 Metetra
.585 .71 .929 r
.47733 .57401 .48492 .58176 .49844 .58045 .49089 .57242 Metetra
.493 .667 .94 r
.49089 .57242 .49844 .58045 .51204 .58149 .50452 .57308 Metetra
.325 .574 .931 r
.50452 .57308 .51204 .58149 .52575 .58587 .51826 .57699 Metetra
.063 .404 .858 r
.51826 .57699 .52575 .58587 .53961 .59437 .53214 .58497 Metetra
0 .181 .703 r
.53214 .58497 .53961 .59437 .55369 .60733 .54621 .59744 Metetra
0 0 .534 r
.54621 .59744 .55369 .60733 .56803 .62445 .56054 .61416 Metetra
0 0 .419 r
.56054 .61416 .56803 .62445 .58267 .64473 .57516 .63418 Metetra
.606 .163 0 r
.57516 .63418 .58267 .64473 .59762 .66656 .59009 .65591 Metetra
0 0 .413 r
.59009 .65591 .59762 .66656 .61285 .68795 .60531 .67734 Metetra
0 0 .524 r
.60531 .67734 .61285 .68795 .62831 .70683 .62076 .69634 Metetra
0 .17 .701 r
.62076 .69634 .62831 .70683 .64389 .72138 .63635 .71104 Metetra
.132 .42 .863 r
.63635 .71104 .64389 .72138 .6595 .73025 .65199 .72003 Metetra
.45 .608 .914 r
.65199 .72003 .6595 .73025 .67503 .73278 .66756 .72261 Metetra
.646 .698 .884 r
.66756 .72261 .67503 .73278 .69041 .72895 .68298 .71873 Metetra
.749 .732 .837 r
.68298 .71873 .69041 .72895 .70558 .71933 .6982 .70897 Metetra
.803 .744 .798 r
.6982 .70897 .70558 .71933 .72052 .70488 .7132 .69434 Metetra
.831 .746 .771 r
.7132 .69434 .72052 .70488 .73524 .68679 .72798 .67604 Metetra
.845 .746 .754 r
.72798 .67604 .73524 .68679 .74976 .66629 .74257 .65534 Metetra
.852 .745 .745 r
.74257 .65534 .74976 .66629 .76413 .64452 .75701 .63341 Metetra
.853 .744 .743 r
.75701 .63341 .76413 .64452 .77841 .62245 .77136 .61123 Metetra
.85 .743 .745 r
.77136 .61123 .77841 .62245 .79264 .60084 .78566 .58956 Metetra
.844 .742 .751 r
.78566 .58956 .79264 .60084 .80688 .58022 .79998 .56896 Metetra
.836 .742 .76 r
.79998 .56896 .80688 .58022 .82117 .56094 .81435 .54975 Metetra
.826 .741 .771 r
.81435 .54975 .82117 .56094 .83555 .5432 .82882 .53212 Metetra
.815 .741 .783 r
.82882 .53212 .83555 .5432 .85004 .52705 .8434 .51612 Metetra
.802 .741 .796 r
.8434 .51612 .85004 .52705 .86467 .51246 .85811 .5017 Metetra
.788 .741 .809 r
.85811 .5017 .86467 .51246 .87945 .49934 .87297 .48876 Metetra
.774 .741 .822 r
.87297 .48876 .87945 .49934 .89437 .48756 .88799 .47715 Metetra
.76 .741 .835 r
.88799 .47715 .89437 .48756 .90945 .47696 .90316 .46673 Metetra
.747 .74 .846 r
.90316 .46673 .90945 .47696 .92469 .46739 .91848 .45732 Metetra
.734 .74 .856 r
.91848 .45732 .92469 .46739 .94008 .45871 .93395 .44878 Metetra
.723 .74 .865 r
.93395 .44878 .94008 .45871 .95561 .45075 .94957 .44095 Metetra
.712 .74 .873 r
.94957 .44095 .95561 .45075 .97128 .44341 .96533 .43371 Metetra
.664 .741 .908 r
.38939 .58906 .39729 .59642 .41047 .59244 .40261 .58505 Metetra
.663 .741 .908 r
.40261 .58505 .41047 .59244 .42372 .58845 .41589 .58102 Metetra
.663 .741 .908 r
.41589 .58102 .42372 .58845 .43703 .58447 .42925 .57699 Metetra
.66 .74 .909 r
.42925 .57699 .43703 .58447 .4504 .58057 .44266 .57303 Metetra
.653 .738 .912 r
.44266 .57303 .4504 .58057 .46384 .57695 .45614 .56932 Metetra
.634 .731 .917 r
.45614 .56932 .46384 .57695 .47733 .57401 .46968 .56621 Metetra
.593 .714 .928 r
.46968 .56621 .47733 .57401 .49089 .57242 .48328 .56436 Metetra
.509 .674 .939 r
.48328 .56436 .49089 .57242 .50452 .57308 .49694 .56464 Metetra
.352 .589 .934 r
.49694 .56464 .50452 .57308 .51826 .57699 .5107 .56807 Metetra
.1 .429 .872 r
.5107 .56807 .51826 .57699 .53214 .58497 .5246 .57552 Metetra
0 .207 .725 r
.5246 .57552 .53214 .58497 .54621 .59744 .53867 .58748 Metetra
0 .008 .552 r
.53867 .58748 .54621 .59744 .56054 .61416 .55299 .60377 Metetra
0 0 .431 r
.55299 .60377 .56054 .61416 .57516 .63418 .56759 .62352 Metetra
.6 .159 0 r
.56759 .62352 .57516 .63418 .59009 .65591 .58251 .64513 Metetra
0 0 .413 r
.58251 .64513 .59009 .65591 .60531 .67734 .59771 .66659 Metetra
0 0 .521 r
.59771 .66659 .60531 .67734 .62076 .69634 .61316 .68572 Metetra
0 .164 .696 r
.61316 .68572 .62076 .69634 .63635 .71104 .62876 .70057 Metetra
.126 .414 .861 r
.62876 .70057 .63635 .71104 .65199 .72003 .64441 .70968 Metetra
.448 .605 .913 r
.64441 .70968 .65199 .72003 .66756 .72261 .66001 .7123 Metetra
.646 .697 .883 r
.66001 .7123 .66756 .72261 .68298 .71873 .67548 .70838 Metetra
.75 .732 .835 r
.67548 .70838 .68298 .71873 .6982 .70897 .69076 .69849 Metetra
.803 .743 .796 r
.69076 .69849 .6982 .70897 .7132 .69434 .70582 .68366 Metetra
.831 .746 .77 r
.70582 .68366 .7132 .69434 .72798 .67604 .72066 .66515 Metetra
.846 .746 .753 r
.72066 .66515 .72798 .67604 .74257 .65534 .73531 .64425 Metetra
.852 .745 .745 r
.73531 .64425 .74257 .65534 .75701 .63341 .74982 .62215 Metetra
.852 .743 .742 r
.74982 .62215 .75701 .63341 .77136 .61123 .76425 .59987 Metetra
.849 .742 .745 r
.76425 .59987 .77136 .61123 .78566 .58956 .77863 .57816 Metetra
.843 .742 .751 r
.77863 .57816 .78566 .58956 .79998 .56896 .79303 .55758 Metetra
.835 .741 .76 r
.79303 .55758 .79998 .56896 .81435 .54975 .80748 .53845 Metetra
.825 .741 .772 r
.80748 .53845 .81435 .54975 .82882 .53212 .82203 .52094 Metetra
.813 .741 .784 r
.82203 .52094 .82882 .53212 .8434 .51612 .83669 .5051 Metetra
.8 .741 .798 r
.83669 .5051 .8434 .51612 .85811 .5017 .8515 .49086 Metetra
.786 .741 .811 r
.8515 .49086 .85811 .5017 .87297 .48876 .86645 .47811 Metetra
.771 .74 .825 r
.86645 .47811 .87297 .48876 .88799 .47715 .88155 .46669 Metetra
.757 .74 .837 r
.88155 .46669 .88799 .47715 .90316 .46673 .89681 .45644 Metetra
.744 .74 .848 r
.89681 .45644 .90316 .46673 .91848 .45732 .91222 .4472 Metetra
.731 .74 .858 r
.91222 .4472 .91848 .45732 .93395 .44878 .92777 .43879 Metetra
.72 .74 .867 r
.92777 .43879 .93395 .44878 .94957 .44095 .94348 .43109 Metetra
.709 .739 .875 r
.94348 .43109 .94957 .44095 .96533 .43371 .95932 .42395 Metetra
.664 .741 .908 r
.38141 .58163 .38939 .58906 .40261 .58505 .39467 .57758 Metetra
.663 .741 .908 r
.39467 .57758 .40261 .58505 .41589 .58102 .408 .57352 Metetra
.663 .741 .908 r
.408 .57352 .41589 .58102 .42925 .57699 .42139 .56945 Metetra
.661 .74 .909 r
.42139 .56945 .42925 .57699 .44266 .57303 .43485 .56543 Metetra
.655 .738 .911 r
.43485 .56543 .44266 .57303 .45614 .56932 .44837 .56162 Metetra
.638 .732 .916 r
.44837 .56162 .45614 .56932 .46968 .56621 .46195 .55836 Metetra
.601 .717 .926 r
.46195 .55836 .46968 .56621 .48328 .56436 .47559 .55625 Metetra
.524 .681 .938 r
.47559 .55625 .48328 .56436 .49694 .56464 .4893 .55616 Metetra
.378 .604 .937 r
.4893 .55616 .49694 .56464 .5107 .56807 .50309 .55911 Metetra
.137 .453 .884 r
.50309 .55911 .5107 .56807 .5246 .57552 .51699 .56602 Metetra
0 .235 .746 r
.51699 .56602 .5246 .57552 .53867 .58748 .53107 .57745 Metetra
0 .029 .572 r
.53107 .57745 .53867 .58748 .55299 .60377 .54538 .5933 Metetra
0 0 .444 r
.54538 .5933 .55299 .60377 .56759 .62352 .55997 .61274 Metetra
.595 .154 0 r
.55997 .61274 .56759 .62352 .58251 .64513 .57486 .63422 Metetra
0 0 .414 r
.57486 .63422 .58251 .64513 .59771 .66659 .59005 .6557 Metetra
0 0 .517 r
.59005 .6557 .59771 .66659 .61316 .68572 .60549 .67495 Metetra
0 .157 .691 r
.60549 .67495 .61316 .68572 .62876 .70057 .62109 .68996 Metetra
.119 .409 .858 r
.62109 .68996 .62876 .70057 .64441 .70968 .63677 .6992 Metetra
.445 .603 .913 r
.63677 .6992 .64441 .70968 .66001 .7123 .65241 .70187 Metetra
.646 .696 .882 r
.65241 .70187 .66001 .7123 .67548 .70838 .66792 .6979 Metetra
.751 .731 .834 r
.66792 .6979 .67548 .70838 .69076 .69849 .68325 .68787 Metetra
.804 .743 .795 r
.68325 .68787 .69076 .69849 .70582 .68366 .69837 .67285 Metetra
.832 .745 .768 r
.69837 .67285 .70582 .68366 .72066 .66515 .71327 .65412 Metetra
.846 .745 .752 r
.71327 .65412 .72066 .66515 .73531 .64425 .728 .63301 Metetra
.852 .744 .744 r
.728 .63301 .73531 .64425 .74982 .62215 .74258 .61075 Metetra
.852 .743 .742 r
.74258 .61075 .74982 .62215 .76425 .59987 .75707 .58836 Metetra
.849 .742 .745 r
.75707 .58836 .76425 .59987 .77863 .57816 .77153 .56662 Metetra
.842 .741 .752 r
.77153 .56662 .77863 .57816 .79303 .55758 .78601 .54607 Metetra
.834 .74 .761 r
.78601 .54607 .79303 .55758 .80748 .53845 .80055 .52704 Metetra
.823 .74 .773 r
.80055 .52704 .80748 .53845 .82203 .52094 .81518 .50967 Metetra
.811 .74 .786 r
.81518 .50967 .82203 .52094 .83669 .5051 .82994 .494 Metetra
.797 .74 .8 r
.82994 .494 .83669 .5051 .8515 .49086 .84482 .47995 Metetra
.783 .74 .814 r
.84482 .47995 .8515 .49086 .86645 .47811 .85986 .46739 Metetra
.768 .74 .827 r
.85986 .46739 .86645 .47811 .88155 .46669 .87505 .45616 Metetra
.754 .74 .839 r
.87505 .45616 .88155 .46669 .89681 .45644 .8904 .4461 Metetra
.74 .74 .851 r
.8904 .4461 .89681 .45644 .91222 .4472 .9059 .43702 Metetra
.728 .739 .861 r
.9059 .43702 .91222 .4472 .92777 .43879 .92154 .42876 Metetra
.717 .739 .869 r
.92154 .42876 .92777 .43879 .94348 .43109 .93733 .42118 Metetra
.707 .739 .877 r
.93733 .42118 .94348 .43109 .95932 .42395 .95326 .41414 Metetra
.664 .741 .908 r
.37336 .57414 .38141 .58163 .39467 .57758 .38666 .57005 Metetra
.663 .741 .908 r
.38666 .57005 .39467 .57758 .408 .57352 .40003 .56595 Metetra
.663 .741 .908 r
.40003 .56595 .408 .57352 .42139 .56945 .41347 .56183 Metetra
.661 .741 .909 r
.41347 .56183 .42139 .56945 .43485 .56543 .42697 .55776 Metetra
.656 .739 .911 r
.42697 .55776 .43485 .56543 .44837 .56162 .44054 .55386 Metetra
.641 .733 .915 r
.44054 .55386 .44837 .56162 .46195 .55836 .45416 .55045 Metetra
.608 .72 .924 r
.45416 .55045 .46195 .55836 .47559 .55625 .46784 .5481 Metetra
.538 .688 .936 r
.46784 .5481 .47559 .55625 .4893 .55616 .48158 .54764 Metetra
.402 .617 .938 r
.48158 .54764 .4893 .55616 .50309 .55911 .4954 .55011 Metetra
.173 .477 .895 r
.4954 .55011 .50309 .55911 .51699 .56602 .50933 .55647 Metetra
0 .263 .767 r
.50933 .55647 .51699 .56602 .53107 .57745 .52341 .56735 Metetra
0 .051 .593 r
.52341 .56735 .53107 .57745 .54538 .5933 .53771 .58273 Metetra
0 0 .458 r
.53771 .58273 .54538 .5933 .55997 .61274 .55228 .60185 Metetra
.588 .148 0 r
.55228 .60185 .55997 .61274 .57486 .63422 .56715 .62318 Metetra
0 0 .415 r
.56715 .62318 .57486 .63422 .59005 .6557 .58232 .64467 Metetra
0 0 .514 r
.58232 .64467 .59005 .6557 .60549 .67495 .59776 .66405 Metetra
0 .15 .686 r
.59776 .66405 .60549 .67495 .62109 .68996 .61337 .67921 Metetra
.113 .403 .855 r
.61337 .67921 .62109 .68996 .63677 .6992 .62906 .68859 Metetra
.443 .601 .912 r
.62906 .68859 .63677 .6992 .65241 .70187 .64473 .6913 Metetra
.646 .695 .881 r
.64473 .6913 .65241 .70187 .66792 .6979 .66029 .68728 Metetra
.752 .731 .833 r
.66029 .68728 .66792 .6979 .68325 .68787 .67568 .67712 Metetra
.805 .742 .794 r
.67568 .67712 .68325 .68787 .69837 .67285 .69085 .66189 Metetra
.833 .745 .767 r
.69085 .66189 .69837 .67285 .71327 .65412 .70582 .64293 Metetra
.846 .744 .751 r
.70582 .64293 .71327 .65412 .728 .63301 .72062 .62162 Metetra
.852 .743 .743 r
.72062 .62162 .728 .63301 .74258 .61075 .73527 .5992 Metetra
.852 .742 .741 r
.73527 .5992 .74258 .61075 .75707 .58836 .74984 .57672 Metetra
.848 .741 .745 r
.74984 .57672 .75707 .58836 .77153 .56662 .76438 .55495 Metetra
.841 .74 .752 r
.76438 .55495 .77153 .56662 .78601 .54607 .77894 .53445 Metetra
.832 .74 .762 r
.77894 .53445 .78601 .54607 .80055 .52704 .79356 .51552 Metetra
.821 .74 .774 r
.79356 .51552 .80055 .52704 .81518 .50967 .80828 .4983 Metetra
.808 .74 .788 r
.80828 .4983 .81518 .50967 .82994 .494 .82312 .48281 Metetra
.794 .74 .802 r
.82312 .48281 .82994 .494 .84482 .47995 .8381 .46896 Metetra
.78 .74 .816 r
.8381 .46896 .84482 .47995 .85986 .46739 .85323 .4566 Metetra
.765 .739 .829 r
.85323 .4566 .85986 .46739 .87505 .45616 .86851 .44558 Metetra
.751 .739 .842 r
.86851 .44558 .87505 .45616 .8904 .4461 .88394 .43569 Metetra
.737 .739 .853 r
.88394 .43569 .8904 .4461 .9059 .43702 .89953 .42678 Metetra
.725 .739 .863 r
.89953 .42678 .9059 .43702 .92154 .42876 .91526 .41867 Metetra
.713 .739 .871 r
.91526 .41867 .92154 .42876 .93733 .42118 .93113 .41121 Metetra
.704 .739 .879 r
.93113 .41121 .93733 .42118 .95326 .41414 .94714 .40427 Metetra
.664 .741 .908 r
.36524 .56658 .37336 .57414 .38666 .57005 .37858 .56245 Metetra
.663 .741 .908 r
.37858 .56245 .38666 .57005 .40003 .56595 .39199 .55831 Metetra
.663 .741 .908 r
.39199 .55831 .40003 .56595 .41347 .56183 .40547 .55415 Metetra
.662 .741 .909 r
.40547 .55415 .41347 .56183 .42697 .55776 .41902 .55003 Metetra
.657 .739 .91 r
.41902 .55003 .42697 .55776 .44054 .55386 .43262 .54605 Metetra
.645 .734 .914 r
.43262 .54605 .44054 .55386 .45416 .55045 .44629 .54249 Metetra
.614 .722 .923 r
.44629 .54249 .45416 .55045 .46784 .5481 .46002 .5399 Metetra
.551 .694 .934 r
.46002 .5399 .46784 .5481 .48158 .54764 .4738 .53908 Metetra
.426 .63 .939 r
.4738 .53908 .48158 .54764 .4954 .55011 .48765 .54107 Metetra
.209 .499 .905 r
.48765 .54107 .4954 .55011 .50933 .55647 .50159 .54688 Metetra
0 .292 .788 r
.50159 .54688 .50933 .55647 .52341 .56735 .51568 .55719 Metetra
0 .074 .615 r
.51568 .55719 .52341 .56735 .53771 .58273 .52998 .57208 Metetra
0 0 .472 r
.52998 .57208 .53771 .58273 .55228 .60185 .54453 .59085 Metetra
.581 .142 0 r
.54453 .59085 .55228 .60185 .56715 .62318 .55938 .61201 Metetra
0 0 .416 r
.55938 .61201 .56715 .62318 .58232 .64467 .57454 .6335 Metetra
0 0 .511 r
.57454 .6335 .58232 .64467 .59776 .66405 .58996 .65299 Metetra
0 .144 .681 r
.58996 .65299 .59776 .66405 .61337 .67921 .60557 .66832 Metetra
.106 .398 .852 r
.60557 .66832 .61337 .67921 .62906 .68859 .62128 .67783 Metetra
.44 .598 .911 r
.62128 .67783 .62906 .68859 .64473 .6913 .63699 .6806 Metetra
.646 .695 .881 r
.63699 .6806 .64473 .6913 .66029 .68728 .65259 .67653 Metetra
.752 .73 .832 r
.65259 .67653 .66029 .68728 .67568 .67712 .66803 .66622 Metetra
.806 .741 .792 r
.66803 .66622 .67568 .67712 .69085 .66189 .68327 .65079 Metetra
.833 .744 .765 r
.68327 .65079 .69085 .66189 .70582 .64293 .69831 .6316 Metetra
.847 .744 .75 r
.69831 .6316 .70582 .64293 .72062 .62162 .71317 .61008 Metetra
.852 .742 .742 r
.71317 .61008 .72062 .62162 .73527 .5992 .7279 .5875 Metetra
.852 .741 .741 r
.7279 .5875 .73527 .5992 .74984 .57672 .74254 .56492 Metetra
.847 .74 .745 r
.74254 .56492 .74984 .57672 .76438 .55495 .75716 .54314 Metetra
.84 .739 .753 r
.75716 .54314 .76438 .55495 .77894 .53445 .7718 .5227 Metetra
.831 .739 .763 r
.7718 .5227 .77894 .53445 .79356 .51552 .78651 .50388 Metetra
.819 .739 .776 r
.78651 .50388 .79356 .51552 .80828 .4983 .80132 .48683 Metetra
.806 .739 .79 r
.80132 .48683 .80828 .4983 .82312 .48281 .81625 .47154 Metetra
.792 .739 .804 r
.81625 .47154 .82312 .48281 .8381 .46896 .83131 .4579 Metetra
.777 .739 .818 r
.83131 .4579 .8381 .46896 .85323 .4566 .84653 .44575 Metetra
.762 .739 .832 r
.84653 .44575 .85323 .4566 .86851 .44558 .8619 .43493 Metetra
.747 .739 .844 r
.8619 .43493 .86851 .44558 .88394 .43569 .87743 .42524 Metetra
.734 .739 .855 r
.87743 .42524 .88394 .43569 .89953 .42678 .8931 .41649 Metetra
.721 .739 .865 r
.8931 .41649 .89953 .42678 .91526 .41867 .90892 .40853 Metetra
.71 .739 .874 r
.90892 .40853 .91526 .41867 .93113 .41121 .92488 .40119 Metetra
.701 .739 .88 r
.92488 .40119 .93113 .41121 .94714 .40427 .94097 .39435 Metetra
.664 .741 .908 r
.35704 .55894 .36524 .56658 .37858 .56245 .37043 .55478 Metetra
.664 .741 .908 r
.37043 .55478 .37858 .56245 .39199 .55831 .38388 .5506 Metetra
.663 .741 .908 r
.38388 .5506 .39199 .55831 .40547 .55415 .3974 .5464 Metetra
.662 .741 .908 r
.3974 .5464 .40547 .55415 .41902 .55003 .41099 .54222 Metetra
.658 .74 .91 r
.41099 .54222 .41902 .55003 .43262 .54605 .42464 .53816 Metetra
.647 .735 .913 r
.42464 .53816 .43262 .54605 .44629 .54249 .43835 .53447 Metetra
.621 .725 .921 r
.43835 .53447 .44629 .54249 .46002 .5399 .45212 .53165 Metetra
.563 .699 .932 r
.45212 .53165 .46002 .5399 .4738 .53908 .46594 .53049 Metetra
.448 .642 .94 r
.46594 .53049 .4738 .53908 .48765 .54107 .47982 .532 Metetra
.244 .521 .913 r
.47982 .532 .48765 .54107 .50159 .54688 .49379 .53724 Metetra
0 .321 .807 r
.49379 .53724 .50159 .54688 .51568 .55719 .50789 .54696 Metetra
0 .099 .637 r
.50789 .54696 .51568 .55719 .52998 .57208 .52218 .56133 Metetra
0 0 .488 r
.52218 .56133 .52998 .57208 .54453 .59085 .53672 .57973 Metetra
0 0 .412 r
.53672 .57973 .54453 .59085 .55938 .61201 .55155 .6007 Metetra
0 0 .418 r
.55155 .6007 .55938 .61201 .57454 .6335 .56668 .62218 Metetra
0 0 .508 r
.56668 .62218 .57454 .6335 .58996 .65299 .5821 .64179 Metetra
0 .137 .676 r
.5821 .64179 .58996 .65299 .60557 .66832 .59771 .65728 Metetra
.098 .391 .849 r
.59771 .65728 .60557 .66832 .62128 .67783 .61344 .66693 Metetra
.437 .596 .91 r
.61344 .66693 .62128 .67783 .63699 .6806 .62918 .66976 Metetra
.646 .694 .88 r
.62918 .66976 .63699 .6806 .65259 .67653 .64483 .66564 Metetra
.753 .73 .83 r
.64483 .66564 .65259 .67653 .66803 .66622 .66032 .65519 Metetra
.807 .741 .791 r
.66032 .65519 .66803 .66622 .68327 .65079 .67562 .63954 Metetra
.834 .743 .764 r
.67562 .63954 .68327 .65079 .69831 .6316 .69073 .62011 Metetra
.847 .743 .748 r
.69073 .62011 .69831 .6316 .71317 .61008 .70566 .59838 Metetra
.852 .742 .741 r
.70566 .59838 .71317 .61008 .7279 .5875 .72046 .57564 Metetra
.851 .74 .741 r
.72046 .57564 .7279 .5875 .74254 .56492 .73518 .55298 Metetra
.847 .739 .745 r
.73518 .55298 .74254 .56492 .75716 .54314 .74988 .5312 Metetra
.839 .739 .753 r
.74988 .5312 .75716 .54314 .7718 .5227 .76461 .51082 Metetra
.829 .738 .764 r
.76461 .51082 .7718 .5227 .78651 .50388 .7794 .49214 Metetra
.817 .738 .777 r
.7794 .49214 .78651 .50388 .80132 .48683 .7943 .47527 Metetra
.803 .738 .791 r
.7943 .47527 .80132 .48683 .81625 .47154 .80931 .46018 Metetra
.789 .738 .806 r
.80931 .46018 .81625 .47154 .83131 .4579 .82447 .44676 Metetra
.774 .738 .821 r
.82447 .44676 .83131 .4579 .84653 .44575 .83978 .43484 Metetra
.759 .738 .834 r
.83978 .43484 .84653 .44575 .8619 .43493 .85524 .42423 Metetra
.744 .738 .847 r
.85524 .42423 .8619 .43493 .87743 .42524 .87086 .41473 Metetra
.73 .738 .858 r
.87086 .41473 .87743 .42524 .8931 .41649 .88662 .40615 Metetra
.718 .738 .867 r
.88662 .40615 .8931 .41649 .90892 .40853 .90252 .39833 Metetra
.707 .738 .876 r
.90252 .39833 .90892 .40853 .92488 .40119 .91857 .39111 Metetra
.698 .739 .882 r
.91857 .39111 .92488 .40119 .94097 .39435 .93474 .38436 Metetra
.664 .741 .908 r
.34877 .55124 .35704 .55894 .37043 .55478 .36219 .54704 Metetra
.664 .741 .908 r
.36219 .54704 .37043 .55478 .38388 .5506 .37569 .54281 Metetra
.663 .741 .908 r
.37569 .54281 .38388 .5506 .3974 .5464 .38925 .53858 Metetra
.662 .741 .908 r
.38925 .53858 .3974 .5464 .41099 .54222 .40288 .53435 Metetra
.659 .74 .909 r
.40288 .53435 .41099 .54222 .42464 .53816 .41658 .53021 Metetra
.65 .736 .913 r
.41658 .53021 .42464 .53816 .43835 .53447 .43034 .5264 Metetra
.626 .727 .919 r
.43034 .5264 .43835 .53447 .45212 .53165 .44415 .52336 Metetra
.574 .704 .93 r
.44415 .52336 .45212 .53165 .46594 .53049 .45801 .52185 Metetra
.47 .653 .939 r
.45801 .52185 .46594 .53049 .47982 .532 .47193 .52289 Metetra
.279 .542 .92 r
.47193 .52289 .47982 .532 .49379 .53724 .48593 .52755 Metetra
0 .351 .826 r
.48593 .52755 .49379 .53724 .50789 .54696 .50004 .53667 Metetra
0 .125 .66 r
.50004 .53667 .50789 .54696 .52218 .56133 .51432 .5505 Metetra
0 0 .505 r
.51432 .5505 .52218 .56133 .53672 .57973 .52885 .56849 Metetra
0 0 .421 r
.52885 .56849 .53672 .57973 .55155 .6007 .54366 .58925 Metetra
0 0 .42 r
.54366 .58925 .55155 .6007 .56668 .62218 .55877 .61071 Metetra
0 0 .505 r
.55877 .61071 .56668 .62218 .5821 .64179 .57417 .63043 Metetra
0 .13 .67 r
.57417 .63043 .5821 .64179 .59771 .65728 .58979 .64609 Metetra
.091 .385 .845 r
.58979 .64609 .59771 .65728 .61344 .66693 .60553 .65589 Metetra
.434 .593 .909 r
.60553 .65589 .61344 .66693 .62918 .66976 .6213 .65878 Metetra
.647 .693 .879 r
.6213 .65878 .62918 .66976 .64483 .66564 .63699 .65461 Metetra
.754 .729 .829 r
.63699 .65461 .64483 .66564 .66032 .65519 .65254 .644 Metetra
.808 .74 .789 r
.65254 .644 .66032 .65519 .67562 .63954 .66791 .62813 Metetra
.835 .743 .763 r
.66791 .62813 .67562 .63954 .69073 .62011 .68308 .60847 Metetra
.847 .742 .747 r
.68308 .60847 .69073 .62011 .70566 .59838 .69808 .58652 Metetra
.852 .741 .74 r
.69808 .58652 .70566 .59838 .72046 .57564 .71296 .56363 Metetra
.851 .739 .74 r
.71296 .56363 .72046 .57564 .73518 .55298 .72776 .54089 Metetra
.846 .738 .745 r
.72776 .54089 .73518 .55298 .74988 .5312 .74254 .51912 Metetra
.838 .738 .754 r
.74254 .51912 .74988 .5312 .76461 .51082 .75735 .49882 Metetra
.827 .738 .765 r
.75735 .49882 .76461 .51082 .7794 .49214 .77223 .48029 Metetra
.815 .738 .779 r
.77223 .48029 .7794 .49214 .7943 .47527 .78722 .46362 Metetra
.801 .738 .794 r
.78722 .46362 .7943 .47527 .80931 .46018 .80232 .44875 Metetra
.786 .738 .809 r
.80232 .44875 .80931 .46018 .82447 .44676 .81758 .43556 Metetra
.77 .738 .823 r
.81758 .43556 .82447 .44676 .83978 .43484 .83297 .42387 Metetra
.755 .738 .837 r
.83297 .42387 .83978 .43484 .85524 .42423 .84853 .41348 Metetra
.74 .738 .849 r
.84853 .41348 .85524 .42423 .87086 .41473 .86423 .40417 Metetra
.727 .738 .86 r
.86423 .40417 .87086 .41473 .88662 .40615 .88008 .39576 Metetra
.715 .738 .87 r
.88008 .39576 .88662 .40615 .90252 .39833 .89607 .38808 Metetra
.704 .738 .878 r
.89607 .38808 .90252 .39833 .91857 .39111 .9122 .38098 Metetra
.695 .738 .884 r
.9122 .38098 .91857 .39111 .93474 .38436 .92846 .37432 Metetra
.664 .741 .908 r
.34042 .54346 .34877 .55124 .36219 .54704 .35388 .53922 Metetra
.664 .741 .908 r
.35388 .53922 .36219 .54704 .37569 .54281 .36742 .53496 Metetra
.663 .741 .908 r
.36742 .53496 .37569 .54281 .38925 .53858 .38103 .53068 Metetra
.663 .741 .908 r
.38103 .53068 .38925 .53858 .40288 .53435 .3947 .5264 Metetra
.66 .74 .909 r
.3947 .5264 .40288 .53435 .41658 .53021 .40844 .5222 Metetra
.652 .737 .912 r
.40844 .5222 .41658 .53021 .43034 .5264 .42224 .51826 Metetra
.631 .729 .918 r
.42224 .51826 .43034 .5264 .44415 .52336 .4361 .51502 Metetra
.585 .709 .928 r
.4361 .51502 .44415 .52336 .45801 .52185 .45001 .51318 Metetra
.49 .663 .939 r
.45001 .51318 .45801 .52185 .47193 .52289 .46397 .51375 Metetra
.312 .562 .926 r
.46397 .51375 .47193 .52289 .48593 .52755 .47799 .51783 Metetra
.038 .38 .844 r
.47799 .51783 .48593 .52755 .50004 .53667 .49212 .52632 Metetra
0 .153 .684 r
.49212 .52632 .50004 .53667 .51432 .5505 .5064 .53958 Metetra
0 0 .523 r
.5064 .53958 .51432 .5505 .52885 .56849 .52091 .55714 Metetra
0 0 .431 r
.52091 .55714 .52885 .56849 .54366 .58925 .5357 .57767 Metetra
0 0 .423 r
.5357 .57767 .54366 .58925 .55877 .61071 .55079 .59908 Metetra
0 0 .502 r
.55079 .59908 .55877 .61071 .57417 .63043 .56617 .61891 Metetra
0 .123 .665 r
.56617 .61891 .57417 .63043 .58979 .64609 .58179 .63475 Metetra
.083 .379 .842 r
.58179 .63475 .58979 .64609 .60553 .65589 .59755 .6447 Metetra
.431 .59 .908 r
.59755 .6447 .60553 .65589 .6213 .65878 .61335 .64765 Metetra
.647 .692 .878 r
.61335 .64765 .6213 .65878 .63699 .65461 .62909 .64343 Metetra
.755 .728 .828 r
.62909 .64343 .63699 .65461 .65254 .644 .64469 .63266 Metetra
.809 .74 .787 r
.64469 .63266 .65254 .644 .66791 .62813 .66012 .61656 Metetra
.835 .742 .761 r
.66012 .61656 .66791 .62813 .68308 .60847 .67536 .59666 Metetra
.848 .741 .746 r
.67536 .59666 .68308 .60847 .69808 .58652 .69044 .57449 Metetra
.852 .74 .74 r
.69044 .57449 .69808 .58652 .71296 .56363 .70539 .55145 Metetra
.85 .739 .74 r
.70539 .55145 .71296 .56363 .72776 .54089 .72028 .52865 Metetra
.845 .738 .745 r
.72028 .52865 .72776 .54089 .74254 .51912 .73514 .50689 Metetra
.836 .737 .755 r
.73514 .50689 .74254 .51912 .75735 .49882 .75004 .4867 Metetra
.825 .737 .767 r
.75004 .4867 .75735 .49882 .77223 .48029 .765 .46834 Metetra
.812 .737 .781 r
.765 .46834 .77223 .48029 .78722 .46362 .78008 .45187 Metetra
.798 .737 .796 r
.78008 .45187 .78722 .46362 .80232 .44875 .79528 .43724 Metetra
.783 .737 .811 r
.79528 .43724 .80232 .44875 .81758 .43556 .81062 .42429 Metetra
.767 .737 .826 r
.81062 .42429 .81758 .43556 .83297 .42387 .82611 .41284 Metetra
.751 .738 .84 r
.82611 .41284 .83297 .42387 .84853 .41348 .84175 .40267 Metetra
.737 .738 .852 r
.84175 .40267 .84853 .41348 .86423 .40417 .85755 .39356 Metetra
.723 .738 .863 r
.85755 .39356 .86423 .40417 .88008 .39576 .87348 .38532 Metetra
.711 .738 .872 r
.87348 .38532 .88008 .39576 .89607 .38808 .88956 .37778 Metetra
.701 .738 .88 r
.88956 .37778 .89607 .38808 .9122 .38098 .90578 .37079 Metetra
.693 .738 .886 r
.90578 .37079 .9122 .38098 .92846 .37432 .92212 .36422 Metetra
.664 .741 .908 r
.33199 .53562 .34042 .54346 .35388 .53922 .3455 .53133 Metetra
.664 .741 .908 r
.3455 .53133 .35388 .53922 .36742 .53496 .35908 .52703 Metetra
.663 .741 .908 r
.35908 .52703 .36742 .53496 .38103 .53068 .37272 .52271 Metetra
.663 .741 .908 r
.37272 .52271 .38103 .53068 .3947 .5264 .38644 .51838 Metetra
.661 .74 .909 r
.38644 .51838 .3947 .5264 .40844 .5222 .40023 .51411 Metetra
.654 .738 .911 r
.40023 .51411 .40844 .5222 .42224 .51826 .41408 .51007 Metetra
.636 .731 .917 r
.41408 .51007 .42224 .51826 .4361 .51502 .42798 .50663 Metetra
.595 .713 .926 r
.42798 .50663 .4361 .51502 .45001 .51318 .44194 .50447 Metetra
.508 .672 .937 r
.44194 .50447 .45001 .51318 .46397 .51375 .45593 .50458 Metetra
.344 .581 .93 r
.45593 .50458 .46397 .51375 .47799 .51783 .46999 .50807 Metetra
.081 .409 .861 r
.46999 .50807 .47799 .51783 .49212 .52632 .48413 .51592 Metetra
0 .183 .708 r
.48413 .51592 .49212 .52632 .5064 .53958 .49842 .52857 Metetra
0 0 .543 r
.49842 .52857 .5064 .53958 .52091 .55714 .51291 .54567 Metetra
0 0 .442 r
.51291 .54567 .52091 .55714 .5357 .57767 .52768 .56594 Metetra
0 0 .426 r
.52768 .56594 .5357 .57767 .55079 .59908 .54275 .58729 Metetra
0 0 .499 r
.54275 .58729 .55079 .59908 .56617 .61891 .55811 .60723 Metetra
0 .115 .659 r
.55811 .60723 .56617 .61891 .58179 .63475 .57373 .62325 Metetra
.075 .372 .838 r
.57373 .62325 .58179 .63475 .59755 .6447 .5895 .63335 Metetra
.428 .587 .907 r
.5895 .63335 .59755 .6447 .61335 .64765 .60533 .63638 Metetra
.647 .691 .877 r
.60533 .63638 .61335 .64765 .62909 .64343 .62112 .6321 Metetra
.756 .728 .826 r
.62112 .6321 .62909 .64343 .64469 .63266 .63678 .62116 Metetra
.81 .739 .786 r
.63678 .62116 .64469 .63266 .66012 .61656 .65227 .60484 Metetra
.836 .741 .759 r
.65227 .60484 .66012 .61656 .67536 .59666 .66758 .58468 Metetra
.848 .74 .745 r
.66758 .58468 .67536 .59666 .69044 .57449 .68273 .56229 Metetra
.851 .739 .739 r
.68273 .56229 .69044 .57449 .70539 .55145 .69776 .53911 Metetra
.85 .738 .74 r
.69776 .53911 .70539 .55145 .72028 .52865 .71272 .51625 Metetra
.844 .737 .746 r
.71272 .51625 .72028 .52865 .73514 .50689 .72767 .49453 Metetra
.835 .736 .755 r
.72767 .49453 .73514 .50689 .75004 .4867 .74265 .47446 Metetra
.823 .736 .768 r
.74265 .47446 .75004 .4867 .765 .46834 .75771 .45627 Metetra
.81 .736 .783 r
.75771 .45627 .765 .46834 .78008 .45187 .77288 .44003 Metetra
.795 .736 .798 r
.77288 .44003 .78008 .45187 .79528 .43724 .78817 .42565 Metetra
.779 .737 .814 r
.78817 .42565 .79528 .43724 .81062 .42429 .8036 .41296 Metetra
.763 .737 .829 r
.8036 .41296 .81062 .42429 .82611 .41284 .81919 .40176 Metetra
.747 .737 .843 r
.81919 .40176 .82611 .41284 .84175 .40267 .83492 .39181 Metetra
.733 .737 .855 r
.83492 .39181 .84175 .40267 .85755 .39356 .8508 .3829 Metetra
.719 .737 .866 r
.8508 .3829 .85755 .39356 .87348 .38532 .86683 .37483 Metetra
.708 .738 .874 r
.86683 .37483 .87348 .38532 .88956 .37778 .883 .36743 Metetra
.698 .738 .882 r
.883 .36743 .88956 .37778 .90578 .37079 .89929 .36055 Metetra
.69 .738 .888 r
.89929 .36055 .90578 .37079 .92212 .36422 .91572 .35405 Metetra
.664 .741 .908 r
.32348 .52769 .33199 .53562 .3455 .53133 .33703 .52337 Metetra
.664 .741 .908 r
.33703 .52337 .3455 .53133 .35908 .52703 .35065 .51903 Metetra
.663 .741 .908 r
.35065 .51903 .35908 .52703 .37272 .52271 .36434 .51466 Metetra
.663 .741 .908 r
.36434 .51466 .37272 .52271 .38644 .51838 .3781 .51029 Metetra
.661 .741 .909 r
.3781 .51029 .38644 .51838 .40023 .51411 .39193 .50595 Metetra
.656 .738 .911 r
.39193 .50595 .40023 .51411 .41408 .51007 .40583 .50181 Metetra
.64 .732 .915 r
.40583 .50181 .41408 .51007 .42798 .50663 .41978 .49818 Metetra
.604 .717 .925 r
.41978 .49818 .42798 .50663 .44194 .50447 .43378 .49572 Metetra
.526 .68 .936 r
.43378 .49572 .44194 .50447 .45593 .50458 .44782 .49537 Metetra
.375 .598 .934 r
.44782 .49537 .45593 .50458 .46999 .50807 .46191 .49827 Metetra
.124 .438 .876 r
.46191 .49827 .46999 .50807 .48413 .51592 .47607 .50546 Metetra
0 .213 .732 r
.47607 .50546 .48413 .51592 .49842 .52857 .49037 .51749 Metetra
0 .014 .563 r
.49037 .51749 .49842 .52857 .51291 .54567 .50485 .53409 Metetra
0 0 .453 r
.50485 .53409 .51291 .54567 .52768 .56594 .51959 .55406 Metetra
0 0 .43 r
.51959 .55406 .52768 .56594 .54275 .58729 .53464 .57535 Metetra
0 0 .497 r
.53464 .57535 .54275 .58729 .55811 .60723 .54999 .59538 Metetra
0 .108 .654 r
.54999 .59538 .55811 .60723 .57373 .62325 .5656 .61158 Metetra
.067 .365 .834 r
.5656 .61158 .57373 .62325 .5895 .63335 .58138 .62185 Metetra
.424 .584 .906 r
.58138 .62185 .5895 .63335 .60533 .63638 .59724 .62494 Metetra
.647 .69 .876 r
.59724 .62494 .60533 .63638 .62112 .6321 .61307 .62061 Metetra
.757 .727 .825 r
.61307 .62061 .62112 .6321 .63678 .62116 .62879 .60951 Metetra
.81 .738 .784 r
.62879 .60951 .63678 .62116 .65227 .60484 .64435 .59294 Metetra
.836 .74 .758 r
.64435 .59294 .65227 .60484 .66758 .58468 .65973 .57253 Metetra
.848 .739 .743 r
.65973 .57253 .66758 .58468 .68273 .56229 .67495 .54992 Metetra
.851 .738 .738 r
.67495 .54992 .68273 .56229 .69776 .53911 .69006 .52659 Metetra
.849 .737 .739 r
.69006 .52659 .69776 .53911 .71272 .51625 .70511 .50369 Metetra
.843 .736 .746 r
.70511 .50369 .71272 .51625 .72767 .49453 .72014 .48202 Metetra
.833 .735 .756 r
.72014 .48202 .72767 .49453 .74265 .47446 .73521 .46209 Metetra
.821 .735 .77 r
.73521 .46209 .74265 .47446 .75771 .45627 .75036 .4441 Metetra
.807 .735 .785 r
.75036 .4441 .75771 .45627 .77288 .44003 .76561 .42811 Metetra
.792 .736 .801 r
.76561 .42811 .77288 .44003 .78817 .42565 .781 .41399 Metetra
.776 .736 .817 r
.781 .41399 .78817 .42565 .8036 .41296 .79653 .40157 Metetra
.759 .736 .832 r
.79653 .40157 .8036 .41296 .81919 .40176 .8122 .39062 Metetra
.743 .737 .845 r
.8122 .39062 .81919 .40176 .83492 .39181 .82803 .3809 Metetra
.729 .737 .858 r
.82803 .3809 .83492 .39181 .8508 .3829 .844 .3722 Metetra
.716 .737 .868 r
.844 .3722 .8508 .3829 .86683 .37483 .86012 .3643 Metetra
.704 .737 .877 r
.86012 .3643 .86683 .37483 .883 .36743 .87637 .35703 Metetra
.695 .738 .884 r
.87637 .35703 .883 .36743 .89929 .36055 .89275 .35025 Metetra
.687 .738 .89 r
.89275 .35025 .89929 .36055 .91572 .35405 .90926 .34383 Metetra
.664 .741 .908 r
.31489 .5197 .32348 .52769 .33703 .52337 .32848 .51533 Metetra
.664 .741 .908 r
.32848 .51533 .33703 .52337 .35065 .51903 .34215 .51095 Metetra
.663 .741 .908 r
.34215 .51095 .35065 .51903 .36434 .51466 .35588 .50654 Metetra
.663 .741 .908 r
.35588 .50654 .36434 .51466 .3781 .51029 .36969 .50212 Metetra
.662 .741 .908 r
.36969 .50212 .3781 .51029 .39193 .50595 .38356 .49772 Metetra
.657 .739 .91 r
.38356 .49772 .39193 .50595 .40583 .50181 .3975 .49348 Metetra
.644 .734 .914 r
.3975 .49348 .40583 .50181 .41978 .49818 .4115 .48968 Metetra
.612 .72 .923 r
.4115 .48968 .41978 .49818 .43378 .49572 .42555 .48693 Metetra
.542 .688 .934 r
.42555 .48693 .43378 .49572 .44782 .49537 .43964 .48613 Metetra
.405 .615 .936 r
.43964 .48613 .44782 .49537 .46191 .49827 .45376 .48844 Metetra
.167 .467 .889 r
.45376 .48844 .46191 .49827 .47607 .50546 .46795 .49495 Metetra
0 .245 .756 r
.46795 .49495 .47607 .50546 .49037 .51749 .48225 .50632 Metetra
0 .037 .585 r
.48225 .50632 .49037 .51749 .50485 .53409 .49672 .52239 Metetra
0 0 .466 r
.49672 .52239 .50485 .53409 .51959 .55406 .51145 .54204 Metetra
0 0 .434 r
.51145 .54204 .51959 .55406 .53464 .57535 .52647 .56323 Metetra
0 0 .495 r
.52647 .56323 .53464 .57535 .54999 .59538 .5418 .58336 Metetra
0 .101 .648 r
.5418 .58336 .54999 .59538 .5656 .61158 .5574 .59974 Metetra
.058 .357 .829 r
.5574 .59974 .5656 .61158 .58138 .62185 .57319 .61018 Metetra
.421 .581 .905 r
.57319 .61018 .58138 .62185 .59724 .62494 .58908 .61335 Metetra
.647 .689 .875 r
.58908 .61335 .59724 .62494 .61307 .62061 .60496 .60897 Metetra
.758 .726 .823 r
.60496 .60897 .61307 .62061 .62879 .60951 .62073 .59769 Metetra
.811 .737 .782 r
.62073 .59769 .62879 .60951 .64435 .59294 .63635 .58087 Metetra
.837 .739 .756 r
.63635 .58087 .64435 .59294 .65973 .57253 .65181 .5602 Metetra
.848 .739 .742 r
.65181 .5602 .65973 .57253 .67495 .54992 .66711 .53737 Metetra
.851 .737 .737 r
.66711 .53737 .67495 .54992 .69006 .52659 .6823 .51391 Metetra
.848 .736 .739 r
.6823 .51391 .69006 .52659 .70511 .50369 .69743 .49097 Metetra
.841 .735 .746 r
.69743 .49097 .70511 .50369 .72014 .48202 .71255 .46937 Metetra
.831 .734 .757 r
.71255 .46937 .72014 .48202 .73521 .46209 .7277 .44959 Metetra
.819 .734 .771 r
.7277 .44959 .73521 .46209 .75036 .4441 .74294 .43183 Metetra
.804 .735 .787 r
.74294 .43183 .75036 .4441 .76561 .42811 .75829 .4161 Metetra
.788 .735 .803 r
.75829 .4161 .76561 .42811 .781 .41399 .77377 .40227 Metetra
.772 .735 .819 r
.77377 .40227 .781 .41399 .79653 .40157 .78939 .39012 Metetra
.755 .736 .835 r
.78939 .39012 .79653 .40157 .8122 .39062 .80516 .37943 Metetra
.739 .736 .849 r
.80516 .37943 .8122 .39062 .82803 .3809 .82108 .36995 Metetra
.725 .736 .861 r
.82108 .36995 .82803 .3809 .844 .3722 .83714 .36144 Metetra
.712 .737 .871 r
.83714 .36144 .844 .3722 .86012 .3643 .85334 .35371 Metetra
.701 .737 .879 r
.85334 .35371 .86012 .3643 .87637 .35703 .86968 .34657 Metetra
.692 .738 .886 r
.86968 .34657 .87637 .35703 .89275 .35025 .88615 .33989 Metetra
.685 .738 .891 r
.88615 .33989 .89275 .35025 .90926 .34383 .90274 .33354 Metetra
.664 .741 .908 r
.30622 .51162 .31489 .5197 .32848 .51533 .31985 .50722 Metetra
.664 .741 .908 r
.31985 .50722 .32848 .51533 .34215 .51095 .33356 .50279 Metetra
.664 .741 .908 r
.33356 .50279 .34215 .51095 .35588 .50654 .34734 .49834 Metetra
.663 .741 .908 r
.34734 .49834 .35588 .50654 .36969 .50212 .36119 .49387 Metetra
.662 .741 .908 r
.36119 .49387 .36969 .50212 .38356 .49772 .37511 .48942 Metetra
.659 .739 .91 r
.37511 .48942 .38356 .49772 .3975 .49348 .3891 .48508 Metetra
.647 .735 .913 r
.3891 .48508 .3975 .49348 .4115 .48968 .40315 .48113 Metetra
.619 .724 .921 r
.40315 .48113 .4115 .48968 .42555 .48693 .41724 .47809 Metetra
.558 .695 .932 r
.41724 .47809 .42555 .48693 .43964 .48613 .43137 .47686 Metetra
.432 .63 .937 r
.43137 .47686 .43964 .48613 .45376 .48844 .44554 .47858 Metetra
.21 .494 .901 r
.44554 .47858 .45376 .48844 .46795 .49495 .45975 .4844 Metetra
0 .279 .78 r
.45975 .4844 .46795 .49495 .48225 .50632 .47406 .49508 Metetra
0 .062 .608 r
.47406 .49508 .48225 .50632 .49672 .52239 .48853 .51058 Metetra
0 0 .481 r
.48853 .51058 .49672 .52239 .51145 .54204 .50323 .52988 Metetra
0 0 .44 r
.50323 .52988 .51145 .54204 .52647 .56323 .51823 .55095 Metetra
0 0 .493 r
.51823 .55095 .52647 .56323 .5418 .58336 .53354 .57116 Metetra
0 .094 .642 r
.53354 .57116 .5418 .58336 .5574 .59974 .54913 .58773 Metetra
.049 .35 .825 r
.54913 .58773 .5574 .59974 .57319 .61018 .56494 .59835 Metetra
.417 .577 .904 r
.56494 .59835 .57319 .61018 .58908 .61335 .58085 .6016 Metetra
.647 .688 .874 r
.58085 .6016 .58908 .61335 .60496 .60897 .59677 .59716 Metetra
.759 .726 .822 r
.59677 .59716 .60496 .60897 .62073 .59769 .6126 .5857 Metetra
.812 .737 .781 r
.6126 .5857 .62073 .59769 .63635 .58087 .62829 .56862 Metetra
.838 .739 .755 r
.62829 .56862 .63635 .58087 .65181 .5602 .64382 .54768 Metetra
.848 .738 .741 r
.64382 .54768 .65181 .5602 .66711 .53737 .6592 .52464 Metetra
.851 .736 .737 r
.6592 .52464 .66711 .53737 .6823 .51391 .67447 .50105 Metetra
.847 .735 .739 r
.67447 .50105 .6823 .51391 .69743 .49097 .68968 .47809 Metetra
.84 .734 .747 r
.68968 .47809 .69743 .49097 .71255 .46937 .70489 .45658 Metetra
.829 .733 .759 r
.70489 .45658 .71255 .46937 .7277 .44959 .72013 .43698 Metetra
.816 .734 .773 r
.72013 .43698 .7277 .44959 .74294 .43183 .73546 .41946 Metetra
.801 .734 .789 r
.73546 .41946 .74294 .43183 .75829 .4161 .7509 .40401 Metetra
.785 .734 .806 r
.7509 .40401 .75829 .4161 .77377 .40227 .76648 .39048 Metetra
.768 .735 .823 r
.76648 .39048 .77377 .40227 .78939 .39012 .78219 .37862 Metetra
.751 .735 .838 r
.78219 .37862 .78939 .39012 .80516 .37943 .79805 .3682 Metetra
.735 .736 .852 r
.79805 .3682 .80516 .37943 .82108 .36995 .81406 .35895 Metetra
.721 .736 .864 r
.81406 .35895 .82108 .36995 .83714 .36144 .83022 .35065 Metetra
.708 .737 .873 r
.83022 .35065 .83714 .36144 .85334 .35371 .84651 .34308 Metetra
.698 .737 .882 r
.84651 .34308 .85334 .35371 .86968 .34657 .86293 .33607 Metetra
.689 .738 .888 r
.86293 .33607 .86968 .34657 .88615 .33989 .87948 .32947 Metetra
.682 .738 .893 r
.87948 .32947 .88615 .33989 .90274 .33354 .89616 .32319 Metetra
.664 .741 .908 r
.29746 .50347 .30622 .51162 .31985 .50722 .31114 .49902 Metetra
.664 .741 .908 r
.31114 .49902 .31985 .50722 .33356 .50279 .32489 .49455 Metetra
.664 .741 .908 r
.32489 .49455 .33356 .50279 .34734 .49834 .33872 .49006 Metetra
.663 .741 .908 r
.33872 .49006 .34734 .49834 .36119 .49387 .35261 .48555 Metetra
.663 .741 .908 r
.35261 .48555 .36119 .49387 .37511 .48942 .36658 .48104 Metetra
.66 .74 .909 r
.36658 .48104 .37511 .48942 .3891 .48508 .38061 .47662 Metetra
.65 .736 .912 r
.38061 .47662 .3891 .48508 .40315 .48113 .39471 .47252 Metetra
.626 .726 .919 r
.39471 .47252 .40315 .48113 .41724 .47809 .40885 .46922 Metetra
.572 .702 .93 r
.40885 .46922 .41724 .47809 .43137 .47686 .42303 .46756 Metetra
.459 .644 .937 r
.42303 .46756 .43137 .47686 .44554 .47858 .43724 .46869 Metetra
.251 .52 .911 r
.43724 .46869 .44554 .47858 .45975 .4844 .45149 .47381 Metetra
0 .313 .803 r
.45149 .47381 .45975 .4844 .47406 .49508 .46581 .48376 Metetra
0 .09 .633 r
.46581 .48376 .47406 .49508 .48853 .51058 .48027 .49866 Metetra
0 0 .496 r
.48027 .49866 .48853 .51058 .50323 .52988 .49496 .51756 Metetra
0 0 .446 r
.49496 .51756 .50323 .52988 .51823 .55095 .50992 .53849 Metetra
0 0 .492 r
.50992 .53849 .51823 .55095 .53354 .57116 .52521 .55877 Metetra
0 .086 .636 r
.52521 .55877 .53354 .57116 .54913 .58773 .54079 .57553 Metetra
.04 .342 .82 r
.54079 .57553 .54913 .58773 .56494 .59835 .5566 .58634 Metetra
.413 .573 .903 r
.5566 .58634 .56494 .59835 .58085 .6016 .57255 .58968 Metetra
.647 .686 .873 r
.57255 .58968 .58085 .6016 .59677 .59716 .58851 .58519 Metetra
.76 .725 .82 r
.58851 .58519 .59677 .59716 .6126 .5857 .6044 .57354 Metetra
.813 .736 .779 r
.6044 .57354 .6126 .5857 .62829 .56862 .62016 .55619 Metetra
.838 .738 .753 r
.62016 .55619 .62829 .56862 .64382 .54768 .63576 .53498 Metetra
.848 .737 .74 r
.63576 .53498 .64382 .54768 .6592 .52464 .65122 .51172 Metetra
.85 .735 .736 r
.65122 .51172 .6592 .52464 .67447 .50105 .66657 .48801 Metetra
.846 .734 .739 r
.66657 .48801 .67447 .50105 .68968 .47809 .68187 .46505 Metetra
.838 .733 .748 r
.68187 .46505 .68968 .47809 .70489 .45658 .69716 .44365 Metetra
.827 .733 .76 r
.69716 .44365 .70489 .45658 .72013 .43698 .7125 .42425 Metetra
.813 .733 .775 r
.7125 .42425 .72013 .43698 .73546 .41946 .72792 .407 Metetra
.798 .733 .792 r
.72792 .407 .73546 .41946 .7509 .40401 .74346 .39185 Metetra
.781 .734 .809 r
.74346 .39185 .7509 .40401 .76648 .39048 .75912 .37862 Metetra
.763 .734 .826 r
.75912 .37862 .76648 .39048 .78219 .37862 .77493 .36707 Metetra
.746 .735 .841 r
.77493 .36707 .78219 .37862 .79805 .3682 .79089 .35693 Metetra
.73 .735 .855 r
.79089 .35693 .79805 .3682 .81406 .35895 .80699 .34792 Metetra
.716 .736 .867 r
.80699 .34792 .81406 .35895 .83022 .35065 .82323 .33981 Metetra
.704 .736 .876 r
.82323 .33981 .83022 .35065 .84651 .34308 .83961 .3324 Metetra
.694 .737 .884 r
.83961 .3324 .84651 .34308 .86293 .33607 .85612 .3255 Metetra
.686 .738 .89 r
.85612 .3255 .86293 .33607 .87948 .32947 .87276 .319 Metetra
.68 .738 .895 r
.87276 .319 .87948 .32947 .89616 .32319 .88951 .31276 Metetra
.664 .741 .908 r
.28862 .49524 .29746 .50347 .31114 .49902 .30235 .49075 Metetra
.664 .741 .908 r
.30235 .49075 .31114 .49902 .32489 .49455 .31614 .48624 Metetra
.664 .741 .908 r
.31614 .48624 .32489 .49455 .33872 .49006 .33001 .4817 Metetra
.663 .741 .908 r
.33001 .4817 .33872 .49006 .35261 .48555 .34395 .47714 Metetra
.663 .741 .908 r
.34395 .47714 .35261 .48555 .36658 .48104 .35796 .47258 Metetra
.661 .74 .909 r
.35796 .47258 .36658 .48104 .38061 .47662 .37204 .46808 Metetra
.653 .737 .911 r
.37204 .46808 .38061 .47662 .39471 .47252 .38618 .46384 Metetra
.632 .729 .917 r
.38618 .46384 .39471 .47252 .40885 .46922 .40038 .4603 Metetra
.585 .708 .928 r
.40038 .4603 .40885 .46922 .42303 .46756 .41461 .45824 Metetra
.483 .657 .937 r
.41461 .45824 .42303 .46756 .43724 .46869 .42887 .45878 Metetra
.292 .545 .919 r
.42887 .45878 .43724 .46869 .45149 .47381 .44315 .46318 Metetra
.004 .348 .825 r
.44315 .46318 .45149 .47381 .46581 .48376 .45749 .47238 Metetra
0 .119 .659 r
.45749 .47238 .46581 .48376 .48027 .49866 .47195 .48663 Metetra
0 0 .514 r
.47195 .48663 .48027 .49866 .49496 .51756 .48661 .50509 Metetra
0 0 .453 r
.48661 .50509 .49496 .51756 .50992 .53849 .50155 .52586 Metetra
0 0 .491 r
.50155 .52586 .50992 .53849 .52521 .55877 .51681 .5462 Metetra
0 .079 .63 r
.51681 .5462 .52521 .55877 .54079 .57553 .53238 .56315 Metetra
.031 .333 .815 r
.53238 .56315 .54079 .57553 .5566 .58634 .5482 .57415 Metetra
.408 .569 .901 r
.5482 .57415 .5566 .58634 .57255 .58968 .56417 .57759 Metetra
.647 .685 .872 r
.56417 .57759 .57255 .58968 .58851 .58519 .58018 .57304 Metetra
.761 .724 .818 r
.58018 .57304 .58851 .58519 .6044 .57354 .59613 .56119 Metetra
.814 .735 .777 r
.59613 .56119 .6044 .57354 .62016 .55619 .61195 .54357 Metetra
.839 .737 .751 r
.61195 .54357 .62016 .55619 .63576 .53498 .62763 .52207 Metetra
.849 .736 .738 r
.62763 .52207 .63576 .53498 .65122 .51172 .64317 .4986 Metetra
.85 .734 .735 r
.64317 .4986 .65122 .51172 .66657 .48801 .6586 .47478 Metetra
.845 .733 .739 r
.6586 .47478 .66657 .48801 .68187 .46505 .67399 .45184 Metetra
.837 .732 .748 r
.67399 .45184 .68187 .46505 .69716 .44365 .68937 .43057 Metetra
.825 .732 .762 r
.68937 .43057 .69716 .44365 .7125 .42425 .7048 .4114 Metetra
.81 .732 .778 r
.7048 .4114 .7125 .42425 .72792 .407 .72031 .39444 Metetra
.794 .732 .795 r
.72031 .39444 .72792 .407 .74346 .39185 .73594 .37962 Metetra
.777 .733 .812 r
.73594 .37962 .74346 .39185 .75912 .37862 .75171 .36672 Metetra
.759 .734 .829 r
.75171 .36672 .75912 .37862 .77493 .36707 .76761 .35548 Metetra
.742 .734 .845 r
.76761 .35548 .77493 .36707 .79089 .35693 .78366 .34561 Metetra
.726 .735 .858 r
.78366 .34561 .79089 .35693 .80699 .34792 .79985 .33684 Metetra
.712 .736 .87 r
.79985 .33684 .80699 .34792 .82323 .33981 .81618 .32893 Metetra
.7 .736 .879 r
.81618 .32893 .82323 .33981 .83961 .3324 .83265 .32167 Metetra
.691 .737 .886 r
.83265 .32167 .83961 .3324 .85612 .3255 .84924 .31489 Metetra
.683 .738 .892 r
.84924 .31489 .85612 .3255 .87276 .319 .86596 .30846 Metetra
.678 .738 .896 r
.86596 .30846 .87276 .319 .88951 .31276 .8828 .30227 Metetra
.664 .741 .908 r
.2797 .48693 .28862 .49524 .30235 .49075 .29347 .4824 Metetra
.664 .741 .908 r
.29347 .4824 .30235 .49075 .31614 .48624 .3073 .47784 Metetra
.664 .741 .908 r
.3073 .47784 .31614 .48624 .33001 .4817 .32121 .47326 Metetra
.663 .741 .908 r
.32121 .47326 .33001 .4817 .34395 .47714 .3352 .46866 Metetra
.663 .741 .908 r
.3352 .46866 .34395 .47714 .35796 .47258 .34926 .46404 Metetra
.661 .74 .909 r
.34926 .46404 .35796 .47258 .37204 .46808 .36338 .45947 Metetra
.655 .738 .911 r
.36338 .45947 .37204 .46808 .38618 .46384 .37758 .4551 Metetra
.638 .731 .916 r
.37758 .4551 .38618 .46384 .40038 .4603 .39183 .45133 Metetra
.596 .713 .925 r
.39183 .45133 .40038 .4603 .41461 .45824 .40611 .44888 Metetra
.506 .668 .936 r
.40611 .44888 .41461 .45824 .42887 .45878 .42041 .44885 Metetra
.332 .569 .925 r
.42041 .44885 .42887 .45878 .44315 .46318 .43473 .45252 Metetra
.054 .383 .846 r
.43473 .45252 .44315 .46318 .45749 .47238 .44909 .46094 Metetra
0 .151 .685 r
.44909 .46094 .45749 .47238 .47195 .48663 .46356 .47449 Metetra
0 0 .532 r
.46356 .47449 .47195 .48663 .48661 .50509 .4782 .49248 Metetra
0 0 .461 r
.4782 .49248 .48661 .50509 .50155 .52586 .49312 .51304 Metetra
0 0 .491 r
.49312 .51304 .50155 .52586 .51681 .5462 .50835 .53343 Metetra
0 .071 .624 r
.50835 .53343 .51681 .5462 .53238 .56315 .52391 .55057 Metetra
.021 .325 .81 r
.52391 .55057 .53238 .56315 .5482 .57415 .53973 .56178 Metetra
.404 .565 .9 r
.53973 .56178 .5482 .57415 .56417 .57759 .55572 .56532 Metetra
.646 .684 .871 r
.55572 .56532 .56417 .57759 .58018 .57304 .57177 .56071 Metetra
.762 .723 .816 r
.57177 .56071 .58018 .57304 .59613 .56119 .58778 .54866 Metetra
.815 .734 .775 r
.58778 .54866 .59613 .56119 .61195 .54357 .60367 .53075 Metetra
.839 .736 .75 r
.60367 .53075 .61195 .54357 .62763 .52207 .61943 .50897 Metetra
.849 .734 .737 r
.61943 .50897 .62763 .52207 .64317 .4986 .63505 .48528 Metetra
.849 .733 .735 r
.63505 .48528 .64317 .4986 .6586 .47478 .65057 .46136 Metetra
.844 .731 .739 r
.65057 .46136 .6586 .47478 .67399 .45184 .66604 .43846 Metetra
.835 .731 .749 r
.66604 .43846 .67399 .45184 .68937 .43057 .68151 .41735 Metetra
.822 .731 .763 r
.68151 .41735 .68937 .43057 .7048 .4114 .69703 .39843 Metetra
.807 .731 .78 r
.69703 .39843 .7048 .4114 .72031 .39444 .71264 .38179 Metetra
.79 .731 .798 r
.71264 .38179 .72031 .39444 .73594 .37962 .72837 .36732 Metetra
.772 .732 .816 r
.72837 .36732 .73594 .37962 .75171 .36672 .74422 .35476 Metetra
.754 .733 .833 r
.74422 .35476 .75171 .36672 .76761 .35548 .76022 .34384 Metetra
.737 .734 .848 r
.76022 .34384 .76761 .35548 .78366 .34561 .77636 .33426 Metetra
.721 .735 .862 r
.77636 .33426 .78366 .34561 .79985 .33684 .79265 .32573 Metetra
.708 .735 .873 r
.79265 .32573 .79985 .33684 .81618 .32893 .80907 .31801 Metetra
.697 .736 .881 r
.80907 .31801 .81618 .32893 .83265 .32167 .82562 .31089 Metetra
.688 .737 .888 r
.82562 .31089 .83265 .32167 .84924 .31489 .8423 .30422 Metetra
.681 .738 .894 r
.8423 .30422 .84924 .31489 .86596 .30846 .8591 .29785 Metetra
.676 .738 .898 r
.8591 .29785 .86596 .30846 .8828 .30227 .87602 .29171 Metetra
.664 .741 .908 r
.27069 .47854 .2797 .48693 .29347 .4824 .2845 .47397 Metetra
.664 .741 .908 r
.2845 .47397 .29347 .4824 .3073 .47784 .29838 .46936 Metetra
.664 .741 .908 r
.29838 .46936 .3073 .47784 .32121 .47326 .31234 .46474 Metetra
.664 .741 .908 r
.31234 .46474 .32121 .47326 .3352 .46866 .32636 .46009 Metetra
.663 .741 .908 r
.32636 .46009 .3352 .46866 .34926 .46404 .34047 .45542 Metetra
.662 .741 .908 r
.34047 .45542 .34926 .46404 .36338 .45947 .35464 .45078 Metetra
.657 .739 .91 r
.35464 .45078 .36338 .45947 .37758 .4551 .36889 .4463 Metetra
.643 .733 .914 r
.36889 .4463 .37758 .4551 .39183 .45133 .38319 .44231 Metetra
.607 .717 .923 r
.38319 .44231 .39183 .45133 .40611 .44888 .39752 .43949 Metetra
.528 .679 .934 r
.39752 .43949 .40611 .44888 .42041 .44885 .41188 .4389 Metetra
.369 .59 .93 r
.41188 .4389 .42041 .44885 .43473 .45252 .42624 .44183 Metetra
.105 .417 .865 r
.42624 .44183 .43473 .45252 .44909 .46094 .44063 .44944 Metetra
0 .185 .713 r
.44063 .44944 .44909 .46094 .46356 .47449 .4551 .46225 Metetra
0 0 .553 r
.4551 .46225 .46356 .47449 .4782 .49248 .46973 .47971 Metetra
0 0 .471 r
.46973 .47971 .4782 .49248 .49312 .51304 .48462 .50003 Metetra
0 0 .491 r
.48462 .50003 .49312 .51304 .50835 .53343 .49982 .52045 Metetra
0 .064 .618 r
.49982 .52045 .50835 .53343 .52391 .55057 .51536 .5378 Metetra
.01 .316 .805 r
.51536 .5378 .52391 .55057 .53973 .56178 .53118 .54922 Metetra
.399 .561 .898 r
.53118 .54922 .53973 .56178 .55572 .56532 .54719 .55287 Metetra
.646 .682 .87 r
.54719 .55287 .55572 .56532 .57177 .56071 .56329 .5482 Metetra
.764 .723 .815 r
.56329 .5482 .57177 .56071 .58778 .54866 .57936 .53594 Metetra
.816 .733 .773 r
.57936 .53594 .58778 .54866 .60367 .53075 .59533 .51773 Metetra
.84 .735 .748 r
.59533 .51773 .60367 .53075 .61943 .50897 .61116 .49565 Metetra
.849 .733 .736 r
.61116 .49565 .61943 .50897 .63505 .48528 .62686 .47175 Metetra
.849 .732 .734 r
.62686 .47175 .63505 .48528 .65057 .46136 .64247 .44776 Metetra
.843 .73 .74 r
.64247 .44776 .65057 .46136 .66604 .43846 .65802 .42491 Metetra
.833 .73 .75 r
.65802 .42491 .66604 .43846 .68151 .41735 .67359 .40399 Metetra
.82 .729 .765 r
.67359 .40399 .68151 .41735 .69703 .39843 .6892 .38536 Metetra
.804 .73 .783 r
.6892 .38536 .69703 .39843 .71264 .38179 .70491 .36907 Metetra
.786 .731 .801 r
.70491 .36907 .71264 .38179 .72837 .36732 .72073 .35496 Metetra
.767 .732 .82 r
.72073 .35496 .72837 .36732 .74422 .35476 .73668 .34276 Metetra
.749 .733 .837 r
.73668 .34276 .74422 .35476 .76022 .34384 .75277 .33217 Metetra
.732 .733 .852 r
.75277 .33217 .76022 .34384 .77636 .33426 .769 .32287 Metetra
.716 .734 .865 r
.769 .32287 .77636 .33426 .79265 .32573 .78538 .31458 Metetra
.703 .735 .876 r
.78538 .31458 .79265 .32573 .80907 .31801 .80189 .30704 Metetra
.693 .736 .884 r
.80189 .30704 .80907 .31801 .82562 .31089 .81853 .30007 Metetra
.685 .737 .891 r
.81853 .30007 .82562 .31089 .8423 .30422 .83529 .29349 Metetra
.678 .738 .896 r
.83529 .29349 .8423 .30422 .8591 .29785 .85218 .28718 Metetra
.674 .738 .899 r
.85218 .28718 .8591 .29785 .87602 .29171 .86918 .28107 Metetra
.664 .741 .908 r
.26159 .47007 .27069 .47854 .2845 .47397 .27545 .46545 Metetra
.664 .741 .908 r
.27545 .46545 .2845 .47397 .29838 .46936 .28937 .4608 Metetra
.664 .741 .908 r
.28937 .4608 .29838 .46936 .31234 .46474 .30337 .45613 Metetra
.664 .741 .908 r
.30337 .45613 .31234 .46474 .32636 .46009 .31744 .45143 Metetra
.663 .741 .908 r
.31744 .45143 .32636 .46009 .34047 .45542 .33159 .44672 Metetra
.662 .741 .908 r
.33159 .44672 .34047 .45542 .35464 .45078 .34582 .44202 Metetra
.659 .739 .91 r
.34582 .44202 .35464 .45078 .36889 .4463 .36011 .43743 Metetra
.647 .735 .913 r
.36011 .43743 .36889 .4463 .38319 .44231 .37446 .43325 Metetra
.617 .722 .921 r
.37446 .43325 .38319 .44231 .39752 .43949 .38885 .43008 Metetra
.547 .688 .932 r
.38885 .43008 .39752 .43949 .41188 .4389 .40326 .42893 Metetra
.405 .611 .933 r
.40326 .42893 .41188 .4389 .42624 .44183 .41767 .43112 Metetra
.156 .452 .882 r
.41767 .43112 .42624 .44183 .44063 .44944 .43209 .43789 Metetra
0 .222 .74 r
.43209 .43789 .44063 .44944 .4551 .46225 .44657 .44992 Metetra
0 .021 .575 r
.44657 .44992 .4551 .46225 .46973 .47971 .46119 .4668 Metetra
0 0 .482 r
.46119 .4668 .46973 .47971 .48462 .50003 .47605 .48684 Metetra
0 0 .492 r
.47605 .48684 .48462 .50003 .49982 .52045 .49122 .50727 Metetra
0 .056 .612 r
.49122 .50727 .49982 .52045 .51536 .5378 .50674 .52481 Metetra
0 .306 .799 r
.50674 .52481 .51536 .5378 .53118 .54922 .52256 .53646 Metetra
.394 .556 .897 r
.52256 .53646 .53118 .54922 .54719 .55287 .5386 .54023 Metetra
.646 .681 .868 r
.5386 .54023 .54719 .55287 .56329 .5482 .55474 .53549 Metetra
.765 .722 .813 r
.55474 .53549 .56329 .5482 .57936 .53594 .57087 .52301 Metetra
.817 .732 .771 r
.57087 .52301 .57936 .53594 .59533 .51773 .5869 .5045 Metetra
.84 .733 .746 r
.5869 .5045 .59533 .51773 .61116 .49565 .60282 .48212 Metetra
.848 .732 .735 r
.60282 .48212 .61116 .49565 .62686 .47175 .6186 .45801 Metetra
.848 .73 .734 r
.6186 .45801 .62686 .47175 .64247 .44776 .63429 .43395 Metetra
.841 .729 .74 r
.63429 .43395 .64247 .44776 .65802 .42491 .64994 .41119 Metetra
.831 .728 .752 r
.64994 .41119 .65802 .42491 .67359 .40399 .6656 .3905 Metetra
.816 .728 .768 r
.6656 .3905 .67359 .40399 .6892 .38536 .68131 .37218 Metetra
.8 .729 .786 r
.68131 .37218 .6892 .38536 .70491 .36907 .6971 .35626 Metetra
.781 .73 .805 r
.6971 .35626 .70491 .36907 .72073 .35496 .71302 .34254 Metetra
.762 .731 .824 r
.71302 .34254 .72073 .35496 .73668 .34276 .72907 .33072 Metetra
.744 .732 .841 r
.72907 .33072 .73668 .34276 .75277 .33217 .74525 .32047 Metetra
.727 .733 .856 r
.74525 .32047 .75277 .33217 .769 .32287 .76158 .31146 Metetra
.712 .734 .869 r
.76158 .31146 .769 .32287 .78538 .31458 .77804 .3034 Metetra
.699 .735 .879 r
.77804 .3034 .78538 .31458 .80189 .30704 .79464 .29604 Metetra
.689 .736 .887 r
.79464 .29604 .80189 .30704 .81853 .30007 .81137 .28919 Metetra
.682 .737 .893 r
.81137 .28919 .81853 .30007 .83529 .29349 .82821 .2827 Metetra
.676 .738 .897 r
.82821 .2827 .83529 .29349 .85218 .28718 .84518 .27644 Metetra
.672 .739 .9 r
.84518 .27644 .85218 .28718 .86918 .28107 .86226 .27035 Metetra
.664 .741 .908 r
.25241 .46152 .26159 .47007 .27545 .46545 .2663 .45685 Metetra
.664 .741 .908 r
.2663 .45685 .27545 .46545 .28937 .4608 .28027 .45216 Metetra
.664 .741 .908 r
.28027 .45216 .28937 .4608 .30337 .45613 .29432 .44744 Metetra
.664 .741 .908 r
.29432 .44744 .30337 .45613 .31744 .45143 .30844 .4427 Metetra
.663 .741 .908 r
.30844 .4427 .31744 .45143 .33159 .44672 .32263 .43793 Metetra
.663 .741 .908 r
.32263 .43793 .33159 .44672 .34582 .44202 .3369 .43317 Metetra
.66 .74 .909 r
.3369 .43317 .34582 .44202 .36011 .43743 .35124 .42849 Metetra
.651 .736 .912 r
.35124 .42849 .36011 .43743 .37446 .43325 .36565 .42413 Metetra
.625 .725 .919 r
.36565 .42413 .37446 .43325 .38885 .43008 .38009 .42063 Metetra
.565 .697 .93 r
.38009 .42063 .38885 .43008 .40326 .42893 .39455 .41895 Metetra
.439 .629 .935 r
.39455 .41895 .40326 .42893 .41767 .43112 .40901 .4204 Metetra
.207 .485 .896 r
.40901 .4204 .41767 .43112 .43209 .43789 .42347 .4263 Metetra
0 .26 .768 r
.42347 .4263 .43209 .43789 .44657 .44992 .43797 .43749 Metetra
0 .048 .6 r
.43797 .43749 .44657 .44992 .46119 .4668 .45258 .45373 Metetra
0 0 .494 r
.45258 .45373 .46119 .4668 .47605 .48684 .46742 .47345 Metetra
0 0 .494 r
.46742 .47345 .47605 .48684 .49122 .50727 .48256 .49387 Metetra
0 .049 .607 r
.48256 .49387 .49122 .50727 .50674 .52481 .49805 .51161 Metetra
0 .296 .793 r
.49805 .51161 .50674 .52481 .52256 .53646 .51387 .52349 Metetra
.388 .551 .895 r
.51387 .52349 .52256 .53646 .5386 .54023 .52992 .52739 Metetra
.646 .679 .867 r
.52992 .52739 .5386 .54023 .55474 .53549 .54611 .52259 Metetra
.766 .721 .81 r
.54611 .52259 .55474 .53549 .57087 .52301 .5623 .50988 Metetra
.819 .731 .768 r
.5623 .50988 .57087 .52301 .5869 .5045 .57841 .49104 Metetra
.841 .732 .744 r
.57841 .49104 .5869 .5045 .60282 .48212 .5944 .46836 Metetra
.848 .731 .733 r
.5944 .46836 .60282 .48212 .6186 .45801 .61027 .44405 Metetra
.847 .729 .733 r
.61027 .44405 .6186 .45801 .63429 .43395 .62605 .41995 Metetra
.84 .728 .74 r
.62605 .41995 .63429 .43395 .64994 .41119 .64179 .39731 Metetra
.828 .727 .753 r
.64179 .39731 .64994 .41119 .6656 .3905 .65754 .37687 Metetra
.813 .727 .77 r
.65754 .37687 .6656 .3905 .68131 .37218 .67334 .35891 Metetra
.796 .728 .789 r
.67334 .35891 .68131 .37218 .6971 .35626 .68924 .34339 Metetra
.777 .729 .809 r
.68924 .34339 .6971 .35626 .71302 .34254 .70525 .33009 Metetra
.757 .73 .828 r
.70525 .33009 .71302 .34254 .72907 .33072 .72139 .31866 Metetra
.738 .732 .845 r
.72139 .31866 .72907 .33072 .74525 .32047 .73767 .30875 Metetra
.721 .733 .86 r
.73767 .30875 .74525 .32047 .76158 .31146 .75409 .30002 Metetra
.707 .734 .872 r
.75409 .30002 .76158 .31146 .77804 .3034 .77064 .29218 Metetra
.695 .735 .882 r
.77064 .29218 .77804 .3034 .79464 .29604 .78733 .28499 Metetra
.686 .736 .889 r
.78733 .28499 .79464 .29604 .81137 .28919 .80414 .27826 Metetra
.679 .737 .895 r
.80414 .27826 .81137 .28919 .82821 .2827 .82107 .27184 Metetra
.674 .738 .899 r
.82107 .27184 .82821 .2827 .84518 .27644 .83811 .26563 Metetra
.67 .739 .902 r
.83811 .26563 .84518 .27644 .86226 .27035 .85527 .25955 Metetra
.664 .741 .908 r
.24313 .45288 .25241 .46152 .2663 .45685 .25707 .44817 Metetra
.664 .741 .908 r
.25707 .44817 .2663 .45685 .28027 .45216 .27108 .44343 Metetra
.664 .741 .908 r
.27108 .44343 .28027 .45216 .29432 .44744 .28517 .43866 Metetra
.664 .741 .908 r
.28517 .43866 .29432 .44744 .30844 .4427 .29934 .43387 Metetra
.663 .741 .908 r
.29934 .43387 .30844 .4427 .32263 .43793 .31358 .42906 Metetra
.663 .741 .908 r
.31358 .42906 .32263 .43793 .3369 .43317 .3279 .42424 Metetra
.661 .74 .909 r
.3279 .42424 .3369 .43317 .35124 .42849 .34229 .41947 Metetra
.654 .737 .911 r
.34229 .41947 .35124 .42849 .36565 .42413 .35674 .41495 Metetra
.633 .728 .917 r
.35674 .41495 .36565 .42413 .38009 .42063 .37124 .41115 Metetra
.581 .705 .927 r
.37124 .41115 .38009 .42063 .39455 .41895 .38577 .40896 Metetra
.47 .646 .935 r
.38577 .40896 .39455 .41895 .40901 .4204 .40028 .40967 Metetra
.258 .517 .908 r
.40028 .40967 .40901 .4204 .42347 .4263 .41478 .41468 Metetra
0 .3 .795 r
.41478 .41468 .42347 .4263 .43797 .43749 .4293 .42498 Metetra
0 .077 .626 r
.4293 .42498 .43797 .43749 .45258 .45373 .44391 .44052 Metetra
0 0 .509 r
.44391 .44052 .45258 .45373 .46742 .47345 .45872 .45986 Metetra
0 0 .496 r
.45872 .45986 .46742 .47345 .48256 .49387 .47383 .48024 Metetra
0 .042 .601 r
.47383 .48024 .48256 .49387 .49805 .51161 .48929 .49817 Metetra
0 .286 .786 r
.48929 .49817 .49805 .51161 .51387 .52349 .5051 .51031 Metetra
.382 .546 .893 r
.5051 .51031 .51387 .52349 .52992 .52739 .52118 .51434 Metetra
.646 .678 .866 r
.52118 .51434 .52992 .52739 .54611 .52259 .5374 .50949 Metetra
.767 .72 .808 r
.5374 .50949 .54611 .52259 .5623 .50988 .55366 .49652 Metetra
.82 .73 .766 r
.55366 .49652 .5623 .50988 .57841 .49104 .56984 .47735 Metetra
.841 .731 .742 r
.56984 .47735 .57841 .49104 .5944 .46836 .58592 .45436 Metetra
.848 .729 .732 r
.58592 .45436 .5944 .46836 .61027 .44405 .60188 .42986 Metetra
.846 .727 .733 r
.60188 .42986 .61027 .44405 .62605 .41995 .61774 .40574 Metetra
.838 .726 .741 r
.61774 .40574 .62605 .41995 .64179 .39731 .63357 .38325 Metetra
.825 .726 .755 r
.63357 .38325 .64179 .39731 .65754 .37687 .64941 .36311 Metetra
.809 .726 .773 r
.64941 .36311 .65754 .37687 .67334 .35891 .66531 .34554 Metetra
.791 .727 .793 r
.66531 .34554 .67334 .35891 .68924 .34339 .6813 .33046 Metetra
.771 .728 .813 r
.6813 .33046 .68924 .34339 .70525 .33009 .69741 .31759 Metetra
.751 .73 .832 r
.69741 .31759 .70525 .33009 .72139 .31866 .71364 .30656 Metetra
.733 .731 .849 r
.71364 .30656 .72139 .31866 .73767 .30875 .73002 .297 Metetra
.716 .733 .864 r
.73002 .297 .73767 .30875 .75409 .30002 .74653 .28856 Metetra
.702 .734 .876 r
.74653 .28856 .75409 .30002 .77064 .29218 .76317 .28094 Metetra
.691 .735 .885 r
.76317 .28094 .77064 .29218 .78733 .28499 .77994 .27391 Metetra
.682 .736 .892 r
.77994 .27391 .78733 .28499 .80414 .27826 .79683 .26728 Metetra
.676 .738 .897 r
.79683 .26728 .80414 .27826 .82107 .27184 .81385 .26092 Metetra
.672 .738 .9 r
.81385 .26092 .82107 .27184 .83811 .26563 .83097 .25474 Metetra
.669 .739 .903 r
.83097 .25474 .83811 .26563 .85527 .25955 .84821 .24866 Metetra
.664 .741 .908 r
.23376 .44416 .24313 .45288 .25707 .44817 .24775 .4394 Metetra
.664 .741 .908 r
.24775 .4394 .25707 .44817 .27108 .44343 .2618 .43461 Metetra
.664 .741 .908 r
.2618 .43461 .27108 .44343 .28517 .43866 .27594 .4298 Metetra
.664 .741 .908 r
.27594 .4298 .28517 .43866 .29934 .43387 .29015 .42496 Metetra
.664 .741 .908 r
.29015 .42496 .29934 .43387 .31358 .42906 .30444 .4201 Metetra
.663 .741 .908 r
.30444 .4201 .31358 .42906 .3279 .42424 .3188 .41522 Metetra
.662 .741 .909 r
.3188 .41522 .3279 .42424 .34229 .41947 .33324 .41037 Metetra
.656 .738 .91 r
.33324 .41037 .34229 .41947 .35674 .41495 .34775 .40571 Metetra
.639 .731 .915 r
.34775 .40571 .35674 .41495 .37124 .41115 .36231 .40163 Metetra
.596 .711 .924 r
.36231 .40163 .37124 .41115 .38577 .40896 .37689 .39896 Metetra
.499 .662 .934 r
.37689 .39896 .38577 .40896 .40028 .40967 .39146 .39894 Metetra
.307 .548 .918 r
.39146 .39894 .40028 .40967 .41478 .41468 .40601 .40304 Metetra
.01 .342 .822 r
.40601 .40304 .41478 .41468 .4293 .42498 .42055 .41239 Metetra
0 .11 .655 r
.42055 .41239 .4293 .42498 .44391 .44052 .43516 .42716 Metetra
0 0 .525 r
.43516 .42716 .44391 .44052 .45872 .45986 .44995 .44606 Metetra
0 0 .5 r
.44995 .44606 .45872 .45986 .47383 .48024 .46503 .46638 Metetra
0 .035 .595 r
.46503 .46638 .47383 .48024 .48929 .49817 .48046 .4845 Metetra
0 .275 .779 r
.48046 .4845 .48929 .49817 .5051 .51031 .49626 .49691 Metetra
.376 .54 .891 r
.49626 .49691 .5051 .51031 .52118 .51434 .51236 .50108 Metetra
.646 .676 .864 r
.51236 .50108 .52118 .51434 .5374 .50949 .52862 .49616 Metetra
.769 .719 .806 r
.52862 .49616 .5374 .50949 .55366 .49652 .54494 .48294 Metetra
.821 .729 .764 r
.54494 .48294 .55366 .49652 .56984 .47735 .56121 .46342 Metetra
.842 .73 .74 r
.56121 .46342 .56984 .47735 .58592 .45436 .57736 .44011 Metetra
.848 .728 .731 r
.57736 .44011 .58592 .45436 .60188 .42986 .59341 .41544 Metetra
.845 .726 .733 r
.59341 .41544 .60188 .42986 .61774 .40574 .60937 .39132 Metetra
.836 .725 .742 r
.60937 .39132 .61774 .40574 .63357 .38325 .62528 .36903 Metetra
.822 .724 .757 r
.62528 .36903 .63357 .38325 .64941 .36311 .64122 .34923 Metetra
.805 .725 .776 r
.64122 .34923 .64941 .36311 .66531 .34554 .65721 .3321 Metetra
.786 .726 .797 r
.65721 .3321 .66531 .34554 .6813 .33046 .6733 .31749 Metetra
.766 .728 .817 r
.6733 .31749 .6813 .33046 .69741 .31759 .6895 .30507 Metetra
.745 .729 .837 r
.6895 .30507 .69741 .31759 .71364 .30656 .70583 .29445 Metetra
.727 .731 .854 r
.70583 .29445 .71364 .30656 .73002 .297 .7223 .28524 Metetra
.71 .732 .868 r
.7223 .28524 .73002 .297 .74653 .28856 .7389 .27707 Metetra
.697 .734 .879 r
.7389 .27707 .74653 .28856 .76317 .28094 .75563 .26966 Metetra
.687 .735 .888 r
.75563 .26966 .76317 .28094 .77994 .27391 .77248 .26278 Metetra
.679 .737 .894 r
.77248 .26278 .77994 .27391 .79683 .26728 .78946 .25624 Metetra
.674 .738 .899 r
.78946 .25624 .79683 .26728 .81385 .26092 .80655 .24993 Metetra
.67 .739 .902 r
.80655 .24993 .81385 .26092 .83097 .25474 .82376 .24376 Metetra
.668 .739 .904 r
.82376 .24376 .83097 .25474 .84821 .24866 .84108 .23767 Metetra
.664 .741 .908 r
.2243 .43535 .23376 .44416 .24775 .4394 .23833 .43054 Metetra
.664 .741 .908 r
.23833 .43054 .24775 .4394 .2618 .43461 .25243 .4257 Metetra
.664 .741 .908 r
.25243 .4257 .2618 .43461 .27594 .4298 .26661 .42084 Metetra
.664 .741 .908 r
.26661 .42084 .27594 .4298 .29015 .42496 .28087 .41596 Metetra
.664 .741 .908 r
.28087 .41596 .29015 .42496 .30444 .4201 .2952 .41104 Metetra
.663 .741 .908 r
.2952 .41104 .30444 .4201 .3188 .41522 .30961 .40611 Metetra
.662 .741 .908 r
.30961 .40611 .3188 .41522 .33324 .41037 .3241 .40119 Metetra
.658 .739 .91 r
.3241 .40119 .33324 .41037 .34775 .40571 .33866 .3964 Metetra
.645 .733 .913 r
.33866 .3964 .34775 .40571 .36231 .40163 .35328 .39208 Metetra
.609 .717 .922 r
.35328 .39208 .36231 .40163 .37689 .39896 .36792 .38895 Metetra
.526 .675 .932 r
.36792 .38895 .37689 .39896 .39146 .39894 .38256 .38822 Metetra
.354 .576 .925 r
.38256 .38822 .39146 .39894 .40601 .40304 .39716 .39138 Metetra
.07 .384 .846 r
.39716 .39138 .40601 .40304 .42055 .41239 .41174 .39974 Metetra
0 .146 .685 r
.41174 .39974 .42055 .41239 .43516 .42716 .42635 .41366 Metetra
0 0 .544 r
.42635 .41366 .43516 .42716 .44995 .44606 .44112 .43206 Metetra
0 0 .505 r
.44112 .43206 .44995 .44606 .46503 .46638 .45616 .45227 Metetra
0 .029 .59 r
.45616 .45227 .46503 .46638 .48046 .4845 .47156 .47058 Metetra
0 .263 .772 r
.47156 .47058 .48046 .4845 .49626 .49691 .48735 .48326 Metetra
.369 .534 .888 r
.48735 .48326 .49626 .49691 .51236 .50108 .50346 .4876 Metetra
.646 .674 .863 r
.50346 .4876 .51236 .50108 .52862 .49616 .51977 .48262 Metetra
.77 .718 .803 r
.51977 .48262 .52862 .49616 .54494 .48294 .53615 .46913 Metetra
.822 .728 .761 r
.53615 .46913 .54494 .48294 .56121 .46342 .55249 .44924 Metetra
.842 .728 .738 r
.55249 .44924 .56121 .46342 .57736 .44011 .56874 .42561 Metetra
.847 .726 .73 r
.56874 .42561 .57736 .44011 .59341 .41544 .58487 .40077 Metetra
.843 .724 .733 r
.58487 .40077 .59341 .41544 .60937 .39132 .60092 .3767 Metetra
.833 .723 .743 r
.60092 .3767 .60937 .39132 .62528 .36903 .61693 .35465 Metetra
.819 .723 .76 r
.61693 .35465 .62528 .36903 .64122 .34923 .63295 .33524 Metetra
.801 .724 .779 r
.63295 .33524 .64122 .34923 .65721 .3321 .64904 .31858 Metetra
.78 .725 .801 r
.64904 .31858 .65721 .3321 .6733 .31749 .66522 .30447 Metetra
.759 .727 .822 r
.66522 .30447 .6733 .31749 .6895 .30507 .68152 .29254 Metetra
.739 .729 .842 r
.68152 .29254 .6895 .30507 .70583 .29445 .69795 .28234 Metetra
.72 .731 .859 r
.69795 .28234 .70583 .29445 .7223 .28524 .7145 .27347 Metetra
.705 .732 .872 r
.7145 .27347 .7223 .28524 .7389 .27707 .73119 .26557 Metetra
.692 .734 .883 r
.73119 .26557 .7389 .27707 .75563 .26966 .74801 .25835 Metetra
.683 .736 .891 r
.74801 .25835 .75563 .26966 .77248 .26278 .76495 .2516 Metetra
.676 .737 .896 r
.76495 .2516 .77248 .26278 .78946 .25624 .78201 .24514 Metetra
.672 .738 .9 r
.78201 .24514 .78946 .25624 .80655 .24993 .79918 .23887 Metetra
.669 .739 .903 r
.79918 .23887 .80655 .24993 .82376 .24376 .81647 .2327 Metetra
.667 .74 .905 r
.81647 .2327 .82376 .24376 .84108 .23767 .83387 .22659 Metetra
.664 .741 .908 r
.21474 .42645 .2243 .43535 .23833 .43054 .22882 .42159 Metetra
.664 .741 .908 r
.22882 .42159 .23833 .43054 .25243 .4257 .24296 .41671 Metetra
.664 .741 .908 r
.24296 .41671 .25243 .4257 .26661 .42084 .25719 .4118 Metetra
.664 .741 .908 r
.25719 .4118 .26661 .42084 .28087 .41596 .27149 .40686 Metetra
.664 .741 .908 r
.27149 .40686 .28087 .41596 .2952 .41104 .28587 .4019 Metetra
.663 .741 .908 r
.28587 .4019 .2952 .41104 .30961 .40611 .30033 .39691 Metetra
.663 .741 .908 r
.30033 .39691 .30961 .40611 .3241 .40119 .31487 .39192 Metetra
.66 .74 .909 r
.31487 .39192 .3241 .40119 .33866 .3964 .32948 .38702 Metetra
.649 .735 .912 r
.32948 .38702 .33866 .3964 .35328 .39208 .34415 .38248 Metetra
.62 .722 .919 r
.34415 .38248 .35328 .39208 .36792 .38895 .35886 .37893 Metetra
.55 .687 .93 r
.35886 .37893 .36792 .38895 .38256 .38822 .37356 .3775 Metetra
.399 .602 .929 r
.37356 .3775 .38256 .38822 .39716 .39138 .38822 .37972 Metetra
.132 .427 .868 r
.38822 .37972 .39716 .39138 .41174 .39974 .40284 .38703 Metetra
0 .186 .716 r
.40284 .38703 .41174 .39974 .42635 .41366 .41747 .40004 Metetra
0 .004 .565 r
.41747 .40004 .42635 .41366 .44112 .43206 .43222 .41786 Metetra
0 0 .511 r
.43222 .41786 .44112 .43206 .45616 .45227 .44723 .43791 Metetra
0 .022 .586 r
.44723 .43791 .45616 .45227 .47156 .47058 .4626 .4564 Metetra
0 .252 .764 r
.4626 .4564 .47156 .47058 .48735 .48326 .47837 .46937 Metetra
.362 .527 .886 r
.47837 .46937 .48735 .48326 .50346 .4876 .49449 .47389 Metetra
.646 .672 .861 r
.49449 .47389 .50346 .4876 .51977 .48262 .51084 .46884 Metetra
.772 .717 .801 r
.51084 .46884 .51977 .48262 .53615 .46913 .52729 .45506 Metetra
.823 .726 .758 r
.52729 .45506 .53615 .46913 .55249 .44924 .54371 .43479 Metetra
.842 .727 .736 r
.54371 .43479 .55249 .44924 .56874 .42561 .56004 .41084 Metetra
.846 .725 .729 r
.56004 .41084 .56874 .42561 .58487 .40077 .57626 .38586 Metetra
.841 .723 .733 r
.57626 .38586 .58487 .40077 .60092 .3767 .5924 .36186 Metetra
.83 .722 .745 r
.5924 .36186 .60092 .3767 .61693 .35465 .6085 .3401 Metetra
.815 .722 .762 r
.6085 .3401 .61693 .35465 .63295 .33524 .62462 .32114 Metetra
.796 .722 .783 r
.62462 .32114 .63295 .33524 .64904 .31858 .6408 .30501 Metetra
.774 .724 .806 r
.6408 .30501 .64904 .31858 .66522 .30447 .65708 .29143 Metetra
.753 .726 .828 r
.65708 .29143 .66522 .30447 .68152 .29254 .67347 .27999 Metetra
.732 .728 .847 r
.67347 .27999 .68152 .29254 .69795 .28234 .68999 .27022 Metetra
.714 .73 .864 r
.68999 .27022 .69795 .28234 .7145 .27347 .70664 .26169 Metetra
.699 .733 .877 r
.70664 .26169 .7145 .27347 .73119 .26557 .72342 .25405 Metetra
.688 .734 .886 r
.72342 .25405 .73119 .26557 .74801 .25835 .74032 .24701 Metetra
.679 .736 .893 r
.74032 .24701 .74801 .25835 .76495 .2516 .75734 .24037 Metetra
.674 .737 .898 r
.75734 .24037 .76495 .2516 .78201 .24514 .77448 .23397 Metetra
.67 .739 .902 r
.77448 .23397 .78201 .24514 .79918 .23887 .79174 .22772 Metetra
.667 .739 .904 r
.79174 .22772 .79918 .23887 .81647 .2327 .8091 .22155 Metetra
.666 .74 .905 r
.8091 .22155 .81647 .2327 .83387 .22659 .82658 .21541 Metetra
.664 .741 .908 r
.20509 .41746 .21474 .42645 .22882 .42159 .21921 .41256 Metetra
.664 .741 .908 r
.21921 .41256 .22882 .42159 .24296 .41671 .2334 .40762 Metetra
.664 .741 .908 r
.2334 .40762 .24296 .41671 .25719 .4118 .24767 .40267 Metetra
.664 .741 .908 r
.24767 .40267 .25719 .4118 .27149 .40686 .26202 .39768 Metetra
.664 .741 .908 r
.26202 .39768 .27149 .40686 .28587 .4019 .27645 .39266 Metetra
.663 .741 .908 r
.27645 .39266 .28587 .4019 .30033 .39691 .29096 .38762 Metetra
.663 .741 .908 r
.29096 .38762 .30033 .39691 .31487 .39192 .30554 .38257 Metetra
.661 .74 .909 r
.30554 .38257 .31487 .39192 .32948 .38702 .32021 .37757 Metetra
.653 .737 .911 r
.32021 .37757 .32948 .38702 .34415 .38248 .33494 .37283 Metetra
.63 .727 .917 r
.33494 .37283 .34415 .38248 .35886 .37893 .34971 .3689 Metetra
.572 .698 .927 r
.34971 .3689 .35886 .37893 .37356 .3775 .36447 .36681 Metetra
.441 .626 .931 r
.36447 .36681 .37356 .3775 .38822 .37972 .3792 .36808 Metetra
.195 .468 .887 r
.3792 .36808 .38822 .37972 .40284 .38703 .39387 .37429 Metetra
0 .23 .748 r
.39387 .37429 .40284 .38703 .41747 .40004 .40852 .38629 Metetra
0 .03 .589 r
.40852 .38629 .41747 .40004 .43222 .41786 .42326 .40344 Metetra
0 0 .52 r
.42326 .40344 .43222 .41786 .44723 .43791 .43823 .42328 Metetra
0 .017 .581 r
.43823 .42328 .44723 .43791 .4626 .4564 .45356 .44194 Metetra
0 .239 .756 r
.45356 .44194 .4626 .4564 .47837 .46937 .46931 .45522 Metetra
.354 .52 .883 r
.46931 .45522 .47837 .46937 .49449 .47389 .48544 .45993 Metetra
.646 .67 .859 r
.48544 .45993 .49449 .47389 .51084 .46884 .50183 .45482 Metetra
.773 .715 .798 r
.50183 .45482 .51084 .46884 .52729 .45506 .51835 .44073 Metetra
.824 .725 .756 r
.51835 .44073 .52729 .45506 .54371 .43479 .53485 .42006 Metetra
.843 .725 .734 r
.53485 .42006 .54371 .43479 .56004 .41084 .55127 .39579 Metetra
.846 .723 .728 r
.55127 .39579 .56004 .41084 .57626 .38586 .56758 .37068 Metetra
.839 .721 .733 r
.56758 .37068 .57626 .38586 .5924 .36186 .58381 .34682 Metetra
.827 .72 .746 r
.58381 .34682 .5924 .36186 .6085 .3401 .60001 .32541 Metetra
.81 .72 .765 r
.60001 .32541 .6085 .3401 .62462 .32114 .61622 .30695 Metetra
.79 .721 .788 r
.61622 .30695 .62462 .32114 .6408 .30501 .63249 .29139 Metetra
.768 .723 .811 r
.63249 .29139 .6408 .30501 .65708 .29143 .64886 .27839 Metetra
.746 .726 .833 r
.64886 .27839 .65708 .29143 .67347 .27999 .66535 .26746 Metetra
.725 .728 .853 r
.66535 .26746 .67347 .27999 .68999 .27022 .68196 .25812 Metetra
.708 .73 .868 r
.68196 .25812 .68999 .27022 .70664 .26169 .6987 .24992 Metetra
.694 .733 .881 r
.6987 .24992 .70664 .26169 .72342 .25405 .71557 .24251 Metetra
.683 .735 .89 r
.71557 .24251 .72342 .25405 .74032 .24701 .73255 .23563 Metetra
.676 .736 .896 r
.73255 .23563 .74032 .24701 .75734 .24037 .74966 .22908 Metetra
.671 .738 .9 r
.74966 .22908 .75734 .24037 .77448 .23397 .76688 .22273 Metetra
.668 .739 .903 r
.76688 .22273 .77448 .23397 .79174 .22772 .78421 .21649 Metetra
.666 .74 .905 r
.78421 .21649 .79174 .22772 .8091 .22155 .80166 .2103 Metetra
.665 .74 .906 r
.80166 .2103 .8091 .22155 .82658 .21541 .81921 .20413 Metetra
.664 .741 .908 r
.19534 .40839 .20509 .41746 .21921 .41256 .2095 .40343 Metetra
.664 .741 .908 r
.2095 .40343 .21921 .41256 .2334 .40762 .22374 .39845 Metetra
.664 .741 .908 r
.22374 .39845 .2334 .40762 .24767 .40267 .23806 .39344 Metetra
.664 .741 .908 r
.23806 .39344 .24767 .40267 .26202 .39768 .25245 .3884 Metetra
.664 .741 .908 r
.25245 .3884 .26202 .39768 .27645 .39266 .26693 .38333 Metetra
.664 .741 .908 r
.26693 .38333 .27645 .39266 .29096 .38762 .28149 .37824 Metetra
.663 .741 .908 r
.28149 .37824 .29096 .38762 .30554 .38257 .29612 .37313 Metetra
.662 .741 .908 r
.29612 .37313 .30554 .38257 .32021 .37757 .31084 .36804 Metetra
.656 .738 .91 r
.31084 .36804 .32021 .37757 .33494 .37283 .32562 .36313 Metetra
.639 .73 .915 r
.32562 .36313 .33494 .37283 .34971 .3689 .34045 .35885 Metetra
.591 .707 .924 r
.34045 .35885 .34971 .3689 .36447 .36681 .35529 .35614 Metetra
.479 .647 .932 r
.35529 .35614 .36447 .36681 .3792 .36808 .37009 .35647 Metetra
.257 .509 .903 r
.37009 .35647 .3792 .36808 .39387 .37429 .38482 .36153 Metetra
0 .277 .781 r
.38482 .36153 .39387 .37429 .40852 .38629 .3995 .37242 Metetra
0 .06 .616 r
.3995 .37242 .40852 .38629 .42326 .40344 .41423 .38881 Metetra
0 0 .53 r
.41423 .38881 .42326 .40344 .43823 .42328 .42917 .40838 Metetra
0 .012 .578 r
.42917 .40838 .43823 .42328 .45356 .44194 .44446 .42719 Metetra
0 .226 .747 r
.44446 .42719 .45356 .44194 .46931 .45522 .46018 .44079 Metetra
.345 .512 .88 r
.46018 .44079 .46931 .45522 .48544 .45993 .47632 .44572 Metetra
.645 .668 .857 r
.47632 .44572 .48544 .45993 .50183 .45482 .49275 .44053 Metetra
.775 .714 .795 r
.49275 .44053 .50183 .45482 .51835 .44073 .50933 .42612 Metetra
.825 .723 .753 r
.50933 .42612 .51835 .44073 .53485 .42006 .52592 .40503 Metetra
.843 .723 .732 r
.52592 .40503 .53485 .42006 .55127 .39579 .54243 .38043 Metetra
.844 .721 .727 r
.54243 .38043 .55127 .39579 .56758 .37068 .55883 .35524 Metetra
.837 .719 .734 r
.55883 .35524 .56758 .37068 .58381 .34682 .57515 .33156 Metetra
.823 .718 .749 r
.57515 .33156 .58381 .34682 .60001 .32541 .59144 .31058 Metetra
.805 .718 .769 r
.59144 .31058 .60001 .32541 .61622 .30695 .60774 .29268 Metetra
.784 .72 .793 r
.60774 .29268 .61622 .30695 .63249 .29139 .62411 .27775 Metetra
.761 .722 .817 r
.62411 .27775 .63249 .29139 .64886 .27839 .64058 .26535 Metetra
.738 .725 .839 r
.64058 .26535 .64886 .27839 .66535 .26746 .65716 .25495 Metetra
.718 .728 .858 r
.65716 .25495 .66535 .26746 .68196 .25812 .67386 .24602 Metetra
.701 .731 .873 r
.67386 .24602 .68196 .25812 .6987 .24992 .69069 .23814 Metetra
.688 .733 .885 r
.69069 .23814 .6987 .24992 .71557 .24251 .70764 .23096 Metetra
.679 .735 .893 r
.70764 .23096 .71557 .24251 .73255 .23563 .72471 .22421 Metetra
.673 .737 .898 r
.72471 .22421 .73255 .23563 .74966 .22908 .7419 .21774 Metetra
.669 .738 .902 r
.7419 .21774 .74966 .22908 .76688 .22273 .7592 .21141 Metetra
.667 .739 .904 r
.7592 .21141 .76688 .22273 .78421 .21649 .77661 .20517 Metetra
.665 .74 .906 r
.77661 .20517 .78421 .21649 .80166 .2103 .79413 .19895 Metetra
.665 .74 .907 r
.79413 .19895 .80166 .2103 .81921 .20413 .81176 .19273 Metetra
.664 .741 .908 r
.18549 .39922 .19534 .40839 .2095 .40343 .1997 .39421 Metetra
.664 .741 .908 r
.1997 .39421 .2095 .40343 .22374 .39845 .21398 .38918 Metetra
.664 .741 .908 r
.21398 .38918 .22374 .39845 .23806 .39344 .22835 .38411 Metetra
.664 .741 .908 r
.22835 .38411 .23806 .39344 .25245 .3884 .24279 .37903 Metetra
.664 .741 .908 r
.24279 .37903 .25245 .3884 .26693 .38333 .25731 .37391 Metetra
.664 .741 .908 r
.25731 .37391 .26693 .38333 .28149 .37824 .27192 .36876 Metetra
.663 .741 .908 r
.27192 .36876 .28149 .37824 .29612 .37313 .2866 .36359 Metetra
.663 .741 .908 r
.2866 .36359 .29612 .37313 .31084 .36804 .30137 .35842 Metetra
.659 .739 .909 r
.30137 .35842 .31084 .36804 .32562 .36313 .31621 .35337 Metetra
.646 .733 .913 r
.31621 .35337 .32562 .36313 .34045 .35885 .3311 .34879 Metetra
.608 .715 .921 r
.3311 .34879 .34045 .35885 .35529 .35614 .34601 .34549 Metetra
.515 .666 .931 r
.34601 .34549 .35529 .35614 .37009 .35647 .36089 .3449 Metetra
.318 .547 .914 r
.36089 .3449 .37009 .35647 .38482 .36153 .37568 .34877 Metetra
.007 .328 .813 r
.37568 .34877 .38482 .36153 .3995 .37242 .3904 .35847 Metetra
0 .095 .646 r
.3904 .35847 .3995 .37242 .41423 .38881 .40514 .37397 Metetra
0 0 .543 r
.40514 .37397 .41423 .38881 .42917 .40838 .42004 .39318 Metetra
0 .008 .575 r
.42004 .39318 .42917 .40838 .44446 .42719 .43529 .41212 Metetra
0 .213 .738 r
.43529 .41212 .44446 .42719 .46018 .44079 .45098 .42607 Metetra
.336 .504 .876 r
.45098 .42607 .46018 .44079 .47632 .44572 .46712 .43123 Metetra
.645 .665 .855 r
.46712 .43123 .47632 .44572 .49275 .44053 .4836 .42598 Metetra
.776 .712 .792 r
.4836 .42598 .49275 .44053 .50933 .42612 .50025 .41122 Metetra
.826 .722 .75 r
.50025 .41122 .50933 .42612 .52592 .40503 .51692 .38968 Metetra
.843 .721 .73 r
.51692 .38968 .52592 .40503 .54243 .38043 .53352 .36477 Metetra
.843 .719 .726 r
.53352 .36477 .54243 .38043 .55883 .35524 .55001 .33953 Metetra
.834 .717 .734 r
.55001 .33953 .55883 .35524 .57515 .33156 .56642 .3161 Metetra
.819 .716 .751 r
.56642 .3161 .57515 .33156 .59144 .31058 .5828 .29562 Metetra
.799 .717 .773 r
.5828 .29562 .59144 .31058 .60774 .29268 .5992 .27837 Metetra
.776 .719 .798 r
.5992 .27837 .60774 .29268 .62411 .27775 .61566 .26411 Metetra
.753 .722 .823 r
.61566 .26411 .62411 .27775 .64058 .26535 .63222 .25234 Metetra
.73 .725 .846 r
.63222 .25234 .64058 .26535 .65716 .25495 .64889 .24246 Metetra
.71 .728 .864 r
.64889 .24246 .65716 .25495 .67386 .24602 .66568 .23395 Metetra
.695 .731 .878 r
.66568 .23395 .67386 .24602 .69069 .23814 .6826 .22637 Metetra
.683 .734 .889 r
.6826 .22637 .69069 .23814 .70764 .23096 .69963 .21938 Metetra
.676 .736 .896 r
.69963 .21938 .70764 .23096 .72471 .22421 .71678 .21275 Metetra
.671 .738 .9 r
.71678 .21275 .72471 .22421 .7419 .21774 .73405 .20632 Metetra
.668 .739 .903 r
.73405 .20632 .7419 .21774 .7592 .21141 .75143 .20001 Metetra
.666 .74 .905 r
.75143 .20001 .7592 .21141 .77661 .20517 .76892 .19374 Metetra
.665 .74 .906 r
.76892 .19374 .77661 .20517 .79413 .19895 .78652 .18748 Metetra
.664 .741 .907 r
.78652 .18748 .79413 .19895 .81176 .19273 .80423 .18121 Metetra
.664 .741 .908 r
.17555 .38995 .18549 .39922 .1997 .39421 .1898 .3849 Metetra
.664 .741 .908 r
.1898 .3849 .1997 .39421 .21398 .38918 .20413 .37981 Metetra
.664 .741 .908 r
.20413 .37981 .21398 .38918 .22835 .38411 .21853 .3747 Metetra
.664 .741 .908 r
.21853 .3747 .22835 .38411 .24279 .37903 .23302 .36955 Metetra
.664 .741 .908 r
.23302 .36955 .24279 .37903 .25731 .37391 .2476 .36438 Metetra
.664 .741 .908 r
.2476 .36438 .25731 .37391 .27192 .36876 .26225 .35918 Metetra
.663 .741 .908 r
.26225 .35918 .27192 .36876 .2866 .36359 .27698 .35396 Metetra
.663 .741 .908 r
.27698 .35396 .2866 .36359 .30137 .35842 .2918 .34872 Metetra
.661 .74 .909 r
.2918 .34872 .30137 .35842 .31621 .35337 .30669 .34355 Metetra
.651 .736 .911 r
.30669 .34355 .31621 .35337 .3311 .34879 .32165 .3387 Metetra
.622 .722 .918 r
.32165 .3387 .3311 .34879 .34601 .34549 .33663 .33488 Metetra
.546 .682 .928 r
.33663 .33488 .34601 .34549 .36089 .3449 .35159 .33339 Metetra
.376 .582 .922 r
.35159 .33339 .36089 .3449 .37568 .34877 .36646 .33603 Metetra
.082 .38 .843 r
.36646 .33603 .37568 .34877 .3904 .35847 .38123 .34444 Metetra
0 .136 .679 r
.38123 .34444 .3904 .35847 .40514 .37397 .39598 .35893 Metetra
0 0 .559 r
.39598 .35893 .40514 .37397 .42004 .39318 .41085 .37769 Metetra
0 .006 .574 r
.41085 .37769 .42004 .39318 .43529 .41212 .42605 .39672 Metetra
0 .199 .728 r
.42605 .39672 .43529 .41212 .45098 .42607 .44171 .41103 Metetra
.326 .494 .872 r
.44171 .41103 .45098 .42607 .46712 .43123 .45785 .41645 Metetra
.645 .663 .853 r
.45785 .41645 .46712 .43123 .4836 .42598 .47437 .41113 Metetra
.778 .711 .789 r
.47437 .41113 .4836 .42598 .50025 .41122 .49109 .396 Metetra
.827 .72 .746 r
.49109 .396 .50025 .41122 .51692 .38968 .50785 .374 Metetra
.842 .719 .727 r
.50785 .374 .51692 .38968 .53352 .36477 .52454 .34877 Metetra
.841 .716 .726 r
.52454 .34877 .53352 .36477 .55001 .33953 .54113 .32354 Metetra
.831 .714 .736 r
.54113 .32354 .55001 .33953 .56642 .3161 .55763 .30045 Metetra
.814 .714 .754 r
.55763 .30045 .56642 .3161 .5828 .29562 .57409 .28055 Metetra
.793 .715 .778 r
.57409 .28055 .5828 .29562 .5992 .27837 .59058 .26402 Metetra
.769 .717 .805 r
.59058 .26402 .5992 .27837 .61566 .26411 .60714 .25049 Metetra
.744 .721 .83 r
.60714 .25049 .61566 .26411 .63222 .25234 .62378 .23937 Metetra
.722 .725 .852 r
.62378 .23937 .63222 .25234 .64889 .24246 .64054 .23002 Metetra
.703 .728 .87 r
.64054 .23002 .64889 .24246 .66568 .23395 .65742 .22191 Metetra
.689 .732 .883 r
.65742 .22191 .66568 .23395 .6826 .22637 .67442 .21459 Metetra
.679 .734 .892 r
.67442 .21459 .6826 .22637 .69963 .21938 .69154 .20777 Metetra
.672 .737 .898 r
.69154 .20777 .69963 .21938 .71678 .21275 .70877 .20122 Metetra
.668 .738 .902 r
.70877 .20122 .71678 .21275 .73405 .20632 .72612 .19483 Metetra
.666 .739 .905 r
.72612 .19483 .73405 .20632 .75143 .20001 .74358 .18851 Metetra
.665 .74 .906 r
.74358 .18851 .75143 .20001 .76892 .19374 .76114 .18221 Metetra
.664 .741 .907 r
.76114 .18221 .76892 .19374 .78652 .18748 .77882 .1759 Metetra
.664 .741 .907 r
.77882 .1759 .78652 .18748 .80423 .18121 .79661 .16957 Metetra
.664 .741 .908 r
.1655 .38059 .17555 .38995 .1898 .3849 .17979 .37549 Metetra
.664 .741 .908 r
.17979 .37549 .1898 .3849 .20413 .37981 .19417 .37035 Metetra
.664 .741 .908 r
.19417 .37035 .20413 .37981 .21853 .3747 .20862 .36518 Metetra
.664 .741 .908 r
.20862 .36518 .21853 .3747 .23302 .36955 .22316 .35999 Metetra
.664 .741 .908 r
.22316 .35999 .23302 .36955 .2476 .36438 .23778 .35476 Metetra
.664 .741 .908 r
.23778 .35476 .2476 .36438 .26225 .35918 .25248 .34951 Metetra
.664 .741 .908 r
.25248 .34951 .26225 .35918 .27698 .35396 .26726 .34423 Metetra
.663 .741 .908 r
.26726 .34423 .27698 .35396 .2918 .34872 .28213 .33892 Metetra
.662 .741 .908 r
.28213 .33892 .2918 .34872 .30669 .34355 .29708 .33364 Metetra
.655 .738 .91 r
.29708 .33364 .30669 .34355 .32165 .3387 .3121 .32858 Metetra
.634 .727 .915 r
.3121 .32858 .32165 .3387 .33663 .33488 .32715 .32429 Metetra
.574 .697 .925 r
.32715 .32429 .33663 .33488 .35159 .33339 .34219 .32195 Metetra
.431 .615 .927 r
.34219 .32195 .35159 .33339 .36646 .33603 .35714 .32335 Metetra
.159 .434 .869 r
.35714 .32335 .36646 .33603 .38123 .34444 .37198 .33036 Metetra
0 .183 .716 r
.37198 .33036 .38123 .34444 .39598 .35893 .38675 .3437 Metetra
0 .013 .579 r
.38675 .3437 .39598 .35893 .41085 .37769 .4016 .36188 Metetra
0 .005 .574 r
.4016 .36188 .41085 .37769 .42605 .39672 .41676 .38096 Metetra
0 .184 .718 r
.41676 .38096 .42605 .39672 .44171 .41103 .43238 .39566 Metetra
.315 .484 .867 r
.43238 .39566 .44171 .41103 .45785 .41645 .44851 .40137 Metetra
.644 .66 .85 r
.44851 .40137 .45785 .41645 .47437 .41113 .46507 .39598 Metetra
.78 .709 .785 r
.46507 .39598 .47437 .41113 .49109 .396 .48186 .38044 Metetra
.828 .718 .743 r
.48186 .38044 .49109 .396 .50785 .374 .49871 .35794 Metetra
.842 .716 .725 r
.49871 .35794 .50785 .374 .52454 .34877 .51549 .33243 Metetra
.839 .714 .725 r
.51549 .33243 .52454 .34877 .54113 .32354 .53217 .30726 Metetra
.827 .712 .737 r
.53217 .30726 .54113 .32354 .55763 .30045 .54876 .28461 Metetra
.808 .711 .758 r
.54876 .28461 .55763 .30045 .57409 .28055 .56532 .2654 Metetra
.785 .713 .784 r
.56532 .2654 .57409 .28055 .59058 .26402 .58189 .24967 Metetra
.76 .716 .812 r
.58189 .24967 .59058 .26402 .60714 .25049 .59853 .23692 Metetra
.735 .72 .838 r
.59853 .23692 .60714 .25049 .62378 .23937 .61527 .22647 Metetra
.713 .725 .859 r
.61527 .22647 .62378 .23937 .64054 .23002 .63212 .21764 Metetra
.695 .729 .876 r
.63212 .21764 .64054 .23002 .65742 .22191 .64908 .20989 Metetra
.683 .732 .888 r
.64908 .20989 .65742 .22191 .67442 .21459 .66617 .20281 Metetra
.675 .735 .896 r
.66617 .20281 .67442 .21459 .69154 .20777 .68337 .19612 Metetra
.67 .737 .901 r
.68337 .19612 .69154 .20777 .70877 .20122 .70068 .18964 Metetra
.667 .739 .904 r
.70068 .18964 .70877 .20122 .72612 .19483 .7181 .18326 Metetra
.665 .74 .906 r
.7181 .18326 .72612 .19483 .74358 .18851 .73564 .17691 Metetra
.664 .74 .907 r
.73564 .17691 .74358 .18851 .76114 .18221 .75328 .17056 Metetra
.664 .741 .907 r
.75328 .17056 .76114 .18221 .77882 .1759 .77104 .1642 Metetra
.664 .741 .908 r
.77104 .1642 .77882 .1759 .79661 .16957 .78891 .1578 Metetra
.664 .741 .908 r
.15534 .37114 .1655 .38059 .17979 .37549 .16968 .36598 Metetra
.664 .741 .908 r
.16968 .36598 .17979 .37549 .19417 .37035 .1841 .36079 Metetra
.664 .741 .908 r
.1841 .36079 .19417 .37035 .20862 .36518 .19861 .35557 Metetra
.664 .741 .908 r
.19861 .35557 .20862 .36518 .22316 .35999 .21319 .35032 Metetra
.664 .741 .908 r
.21319 .35032 .22316 .35999 .23778 .35476 .22786 .34504 Metetra
.664 .741 .908 r
.22786 .34504 .23778 .35476 .25248 .34951 .24261 .33973 Metetra
.664 .741 .908 r
.24261 .33973 .25248 .34951 .26726 .34423 .25744 .33439 Metetra
.663 .741 .908 r
.25744 .33439 .26726 .34423 .28213 .33892 .27236 .32903 Metetra
.663 .741 .908 r
.27236 .32903 .28213 .33892 .29708 .33364 .28736 .32366 Metetra
.659 .739 .909 r
.28736 .32366 .29708 .33364 .3121 .32858 .30244 .31842 Metetra
.643 .732 .913 r
.30244 .31842 .3121 .32858 .32715 .32429 .31756 .31373 Metetra
.597 .708 .922 r
.31756 .31373 .32715 .32429 .34219 .32195 .33269 .31059 Metetra
.481 .643 .928 r
.33269 .31059 .34219 .32195 .35714 .32335 .34773 .31076 Metetra
.238 .486 .891 r
.34773 .31076 .35714 .32335 .37198 .33036 .36265 .31627 Metetra
0 .237 .754 r
.36265 .31627 .37198 .33036 .38675 .3437 .37745 .32828 Metetra
0 .04 .603 r
.37745 .32828 .38675 .3437 .4016 .36188 .3923 .34574 Metetra
0 .006 .576 r
.3923 .34574 .4016 .36188 .41676 .38096 .4074 .36481 Metetra
0 .17 .708 r
.4074 .36481 .41676 .38096 .43238 .39566 .42298 .37991 Metetra
.302 .472 .862 r
.42298 .37991 .43238 .39566 .44851 .40137 .4391 .38595 Metetra
.644 .657 .848 r
.4391 .38595 .44851 .40137 .46507 .39598 .45569 .38049 Metetra
.782 .707 .781 r
.45569 .38049 .46507 .39598 .48186 .38044 .47256 .3645 Metetra
.829 .715 .739 r
.47256 .3645 .48186 .38044 .49871 .35794 .4895 .34149 Metetra
.841 .714 .723 r
.4895 .34149 .49871 .35794 .51549 .33243 .50638 .31571 Metetra
.836 .711 .725 r
.50638 .31571 .51549 .33243 .53217 .30726 .52315 .2907 Metetra
.822 .709 .74 r
.52315 .2907 .53217 .30726 .54876 .28461 .53982 .2686 Metetra
.801 .709 .763 r
.53982 .2686 .54876 .28461 .56532 .2654 .55646 .2502 Metetra
.776 .711 .791 r
.55646 .2502 .56532 .2654 .58189 .24967 .57313 .23536 Metetra
.75 .715 .82 r
.57313 .23536 .58189 .24967 .59853 .23692 .58986 .22343 Metetra
.725 .72 .846 r
.58986 .22343 .59853 .23692 .61527 .22647 .60668 .21365 Metetra
.704 .725 .867 r
.60668 .21365 .61527 .22647 .63212 .21764 .62361 .20532 Metetra
.688 .73 .882 r
.62361 .20532 .63212 .21764 .64908 .20989 .64066 .1979 Metetra
.678 .733 .892 r
.64066 .1979 .64908 .20989 .66617 .20281 .65783 .19102 Metetra
.671 .736 .899 r
.65783 .19102 .66617 .20281 .68337 .19612 .67511 .18443 Metetra
.668 .738 .903 r
.67511 .18443 .68337 .19612 .70068 .18964 .6925 .17798 Metetra
.666 .739 .905 r
.6925 .17798 .70068 .18964 .7181 .18326 .71 .17158 Metetra
.664 .74 .906 r
.71 .17158 .7181 .18326 .73564 .17691 .72761 .1652 Metetra
.664 .741 .907 r
.72761 .1652 .73564 .17691 .75328 .17056 .74533 .1588 Metetra
.664 .741 .908 r
.74533 .1588 .75328 .17056 .77104 .1642 .76317 .15237 Metetra
.664 .741 .908 r
.76317 .15237 .77104 .1642 .78891 .1578 .78111 .1459 Metetra
.664 .741 .908 r
.14508 .36159 .15534 .37114 .16968 .36598 .15947 .35637 Metetra
.664 .741 .908 r
.15947 .35637 .16968 .36598 .1841 .36079 .17394 .35113 Metetra
.664 .741 .908 r
.17394 .35113 .1841 .36079 .19861 .35557 .18849 .34585 Metetra
.664 .741 .908 r
.18849 .34585 .19861 .35557 .21319 .35032 .20312 .34055 Metetra
.664 .741 .908 r
.20312 .34055 .21319 .35032 .22786 .34504 .21783 .33521 Metetra
.664 .741 .908 r
.21783 .33521 .22786 .34504 .24261 .33973 .23263 .32985 Metetra
.664 .741 .908 r
.23263 .32985 .24261 .33973 .25744 .33439 .24752 .32445 Metetra
.664 .741 .908 r
.24752 .32445 .25744 .33439 .27236 .32903 .26248 .31903 Metetra
.663 .741 .908 r
.26248 .31903 .27236 .32903 .28736 .32366 .27754 .31359 Metetra
.661 .74 .909 r
.27754 .31359 .28736 .32366 .30244 .31842 .29267 .30821 Metetra
.651 .735 .911 r
.29267 .30821 .30244 .31842 .31756 .31373 .30787 .30318 Metetra
.617 .718 .918 r
.30787 .30318 .31756 .31373 .33269 .31059 .32308 .29933 Metetra
.525 .668 .927 r
.32308 .29933 .33269 .31059 .34773 .31076 .33822 .29828 Metetra
.317 .537 .908 r
.33822 .29828 .34773 .31076 .36265 .31627 .35323 .30221 Metetra
0 .297 .793 r
.35323 .30221 .36265 .31627 .37745 .32828 .36809 .31272 Metetra
0 .074 .632 r
.36809 .31272 .37745 .32828 .3923 .34574 .38293 .32927 Metetra
0 .011 .581 r
.38293 .32927 .3923 .34574 .4074 .36481 .39799 .34824 Metetra
0 .155 .697 r
.39799 .34824 .4074 .36481 .42298 .37991 .41351 .36376 Metetra
.289 .459 .856 r
.41351 .36376 .42298 .37991 .4391 .38595 .42962 .37017 Metetra
.643 .653 .844 r
.42962 .37017 .4391 .38595 .45569 .38049 .44625 .36464 Metetra
.784 .705 .777 r
.44625 .36464 .45569 .38049 .47256 .3645 .46319 .34817 Metetra
.83 .713 .736 r
.46319 .34817 .47256 .3645 .4895 .34149 .48023 .32461 Metetra
.84 .711 .721 r
.48023 .32461 .4895 .34149 .50638 .31571 .4972 .29861 Metetra
.833 .707 .725 r
.4972 .29861 .50638 .31571 .52315 .2907 .51406 .27386 Metetra
.816 .706 .743 r
.51406 .27386 .52315 .2907 .53982 .2686 .53082 .25246 Metetra
.793 .706 .769 r
.53082 .25246 .53982 .2686 .55646 .2502 .54754 .235 Metetra
.766 .71 .799 r
.54754 .235 .55646 .2502 .57313 .23536 .56428 .22113 Metetra
.739 .715 .829 r
.56428 .22113 .57313 .23536 .58986 .22343 .5811 .21005 Metetra
.715 .721 .854 r
.5811 .21005 .58986 .22343 .60668 .21365 .59801 .20093 Metetra
.695 .726 .874 r
.59801 .20093 .60668 .21365 .62361 .20532 .61502 .19306 Metetra
.682 .731 .888 r
.61502 .19306 .62361 .20532 .64066 .1979 .63215 .18593 Metetra
.673 .735 .896 r
.63215 .18593 .64066 .1979 .65783 .19102 .6494 .17921 Metetra
.668 .737 .902 r
.6494 .17921 .65783 .19102 .67511 .18443 .66676 .17268 Metetra
.666 .739 .905 r
.66676 .17268 .67511 .18443 .6925 .17798 .68422 .16623 Metetra
.665 .74 .906 r
.68422 .16623 .6925 .17798 .71 .17158 .7018 .15981 Metetra
.664 .741 .907 r
.7018 .15981 .71 .17158 .72761 .1652 .71949 .15337 Metetra
.664 .741 .907 r
.71949 .15337 .72761 .1652 .74533 .1588 .73729 .1469 Metetra
.664 .741 .908 r
.73729 .1469 .74533 .1588 .76317 .15237 .7552 .14041 Metetra
.664 .741 .908 r
.7552 .14041 .76317 .15237 .78111 .1459 .77323 .13387 Metetra
.664 .741 .908 r
.13472 .35194 .14508 .36159 .15947 .35637 .14915 .34667 Metetra
.664 .741 .908 r
.14915 .34667 .15947 .35637 .17394 .35113 .16366 .34137 Metetra
.664 .741 .908 r
.16366 .34137 .17394 .35113 .18849 .34585 .17826 .33604 Metetra
.664 .741 .908 r
.17826 .33604 .18849 .34585 .20312 .34055 .19294 .33068 Metetra
.664 .741 .908 r
.19294 .33068 .20312 .34055 .21783 .33521 .2077 .32529 Metetra
.664 .741 .908 r
.2077 .32529 .21783 .33521 .23263 .32985 .22255 .31986 Metetra
.664 .741 .908 r
.22255 .31986 .23263 .32985 .24752 .32445 .23748 .31441 Metetra
.664 .741 .908 r
.23748 .31441 .24752 .32445 .26248 .31903 .2525 .30893 Metetra
.663 .741 .908 r
.2525 .30893 .26248 .31903 .27754 .31359 .26761 .30342 Metetra
.662 .741 .908 r
.26761 .30342 .27754 .31359 .29267 .30821 .2828 .29792 Metetra
.656 .738 .91 r
.2828 .29792 .29267 .30821 .30787 .30318 .29806 .29263 Metetra
.633 .726 .915 r
.29806 .29263 .30787 .30318 .32308 .29933 .31336 .28817 Metetra
.564 .688 .924 r
.31336 .28817 .32308 .29933 .33822 .29828 .32861 .28596 Metetra
.392 .584 .918 r
.32861 .28596 .33822 .29828 .35323 .30221 .34372 .28823 Metetra
.079 .363 .831 r
.34372 .28823 .35323 .30221 .36809 .31272 .35865 .29704 Metetra
0 .116 .667 r
.35865 .29704 .36809 .31272 .38293 .32927 .37351 .31246 Metetra
0 .019 .589 r
.37351 .31246 .38293 .32927 .39799 .34824 .38852 .33121 Metetra
0 .14 .686 r
.38852 .33121 .39799 .34824 .41351 .36376 .40398 .34716 Metetra
.274 .445 .849 r
.40398 .34716 .41351 .36376 .42962 .37017 .42007 .354 Metetra
.643 .649 .841 r
.42007 .354 .42962 .37017 .44625 .36464 .43673 .34839 Metetra
.786 .703 .772 r
.43673 .34839 .44625 .36464 .46319 .34817 .45376 .33138 Metetra
.83 .71 .732 r
.45376 .33138 .46319 .34817 .48023 .32461 .47089 .30726 Metetra
.838 .707 .719 r
.47089 .30726 .48023 .32461 .4972 .29861 .48796 .28109 Metetra
.829 .704 .726 r
.48796 .28109 .4972 .29861 .51406 .27386 .5049 .25675 Metetra
.809 .702 .747 r
.5049 .25675 .51406 .27386 .53082 .25246 .52174 .23622 Metetra
.784 .704 .776 r
.52174 .23622 .53082 .25246 .54754 .235 .53854 .21985 Metetra
.755 .708 .808 r
.53854 .21985 .54754 .235 .56428 .22113 .55536 .20703 Metetra
.727 .715 .838 r
.55536 .20703 .56428 .22113 .5811 .21005 .57226 .19682 Metetra
.704 .721 .863 r
.57226 .19682 .5811 .21005 .59801 .20093 .58925 .18833 Metetra
.687 .728 .881 r
.58925 .18833 .59801 .20093 .61502 .19306 .60635 .18087 Metetra
.676 .733 .893 r
.60635 .18087 .61502 .19306 .63215 .18593 .62356 .17397 Metetra
.67 .736 .9 r
.62356 .17397 .63215 .18593 .6494 .17921 .64088 .16735 Metetra
.666 .738 .904 r
.64088 .16735 .6494 .17921 .66676 .17268 .65831 .16085 Metetra
.665 .74 .906 r
.65831 .16085 .66676 .17268 .68422 .16623 .67586 .15438 Metetra
.664 .74 .907 r
.67586 .15438 .68422 .16623 .7018 .15981 .69351 .14791 Metetra
.664 .741 .907 r
.69351 .14791 .7018 .15981 .71949 .15337 .71128 .14141 Metetra
.664 .741 .908 r
.71128 .14141 .71949 .15337 .73729 .1469 .72916 .13488 Metetra
.664 .741 .908 r
.72916 .13488 .73729 .1469 .7552 .14041 .74715 .12831 Metetra
.664 .741 .908 r
.74715 .12831 .7552 .14041 .77323 .13387 .76525 .1217 Metetra
.664 .741 .908 r
.12424 .34218 .13472 .35194 .14915 .34667 .13872 .33686 Metetra
.664 .741 .908 r
.13872 .33686 .14915 .34667 .16366 .34137 .15328 .3315 Metetra
.664 .741 .908 r
.15328 .3315 .16366 .34137 .17826 .33604 .16793 .32612 Metetra
.664 .741 .908 r
.16793 .32612 .17826 .33604 .19294 .33068 .18265 .3207 Metetra
.664 .741 .908 r
.18265 .3207 .19294 .33068 .2077 .32529 .19747 .31525 Metetra
.664 .741 .908 r
.19747 .31525 .2077 .32529 .22255 .31986 .21236 .30977 Metetra
.664 .741 .908 r
.21236 .30977 .22255 .31986 .23748 .31441 .22735 .30426 Metetra
.664 .741 .908 r
.22735 .30426 .23748 .31441 .2525 .30893 .24242 .29872 Metetra
.663 .741 .908 r
.24242 .29872 .2525 .30893 .26761 .30342 .25758 .29314 Metetra
.663 .741 .908 r
.25758 .29314 .26761 .30342 .2828 .29792 .27282 .28756 Metetra
.66 .739 .909 r
.27282 .28756 .2828 .29792 .29806 .29263 .28814 .28207 Metetra
.645 .732 .912 r
.28814 .28207 .29806 .29263 .31336 .28817 .30352 .2771 Metetra
.596 .705 .92 r
.30352 .2771 .31336 .28817 .32861 .28596 .31888 .27383 Metetra
.461 .625 .923 r
.31888 .27383 .32861 .28596 .34372 .28823 .33411 .27441 Metetra
.179 .433 .865 r
.33411 .27441 .34372 .28823 .35865 .29704 .34914 .2813 Metetra
0 .17 .707 r
.34914 .2813 .35865 .29704 .37351 .31246 .36403 .29531 Metetra
0 .034 .602 r
.36403 .29531 .37351 .31246 .38852 .33121 .379 .31367 Metetra
0 .126 .676 r
.379 .31367 .38852 .33121 .40398 .34716 .3944 .33006 Metetra
.257 .428 .841 r
.3944 .33006 .40398 .34716 .42007 .354 .41046 .33738 Metetra
.642 .644 .837 r
.41046 .33738 .42007 .354 .43673 .34839 .42716 .33169 Metetra
.788 .7 .767 r
.42716 .33169 .43673 .34839 .45376 .33138 .44427 .31409 Metetra
.831 .707 .727 r
.44427 .31409 .45376 .33138 .47089 .30726 .4615 .28938 Metetra
.836 .703 .717 r
.4615 .28938 .47089 .30726 .48796 .28109 .47866 .26314 Metetra
.823 .699 .728 r
.47866 .26314 .48796 .28109 .5049 .25675 .49567 .23941 Metetra
.801 .698 .752 r
.49567 .23941 .5049 .25675 .52174 .23622 .51258 .21995 Metetra
.772 .701 .784 r
.51258 .21995 .52174 .23622 .53854 .21985 .52946 .20481 Metetra
.742 .707 .819 r
.52946 .20481 .53854 .21985 .55536 .20703 .54636 .19311 Metetra
.715 .715 .849 r
.54636 .19311 .55536 .20703 .57226 .19682 .56333 .18377 Metetra
.693 .723 .873 r
.56333 .18377 .57226 .19682 .58925 .18833 .5804 .17586 Metetra
.679 .73 .888 r
.5804 .17586 .58925 .18833 .60635 .18087 .59758 .16874 Metetra
.671 .734 .898 r
.59758 .16874 .60635 .18087 .62356 .17397 .61487 .16201 Metetra
.667 .737 .903 r
.61487 .16201 .62356 .17397 .64088 .16735 .63227 .15544 Metetra
.665 .739 .905 r
.63227 .15544 .64088 .16735 .65831 .16085 .64978 .14893 Metetra
.664 .74 .907 r
.64978 .14893 .65831 .16085 .67586 .15438 .6674 .14242 Metetra
.664 .741 .907 r
.6674 .14242 .67586 .15438 .69351 .14791 .68513 .13589 Metetra
.664 .741 .908 r
.68513 .13589 .69351 .14791 .71128 .14141 .70297 .12932 Metetra
.664 .741 .908 r
.70297 .12932 .71128 .14141 .72916 .13488 .72093 .12272 Metetra
.664 .741 .908 r
.72093 .12272 .72916 .13488 .74715 .12831 .739 .11607 Metetra
.664 .741 .908 r
.739 .11607 .74715 .12831 .76525 .1217 .75719 .10938 Metetra
.664 .741 .908 r
.11366 .33233 .12424 .34218 .13872 .33686 .12818 .32695 Metetra
.664 .741 .908 r
.12818 .32695 .13872 .33686 .15328 .3315 .14279 .32154 Metetra
.664 .741 .908 r
.14279 .32154 .15328 .3315 .16793 .32612 .15748 .31609 Metetra
.664 .741 .908 r
.15748 .31609 .16793 .32612 .18265 .3207 .17226 .31062 Metetra
.664 .741 .908 r
.17226 .31062 .18265 .3207 .19747 .31525 .18712 .30511 Metetra
.664 .741 .908 r
.18712 .30511 .19747 .31525 .21236 .30977 .20207 .29957 Metetra
.664 .741 .908 r
.20207 .29957 .21236 .30977 .22735 .30426 .2171 .294 Metetra
.664 .741 .908 r
.2171 .294 .22735 .30426 .24242 .29872 .23222 .2884 Metetra
.664 .741 .908 r
.23222 .2884 .24242 .29872 .25758 .29314 .24743 .28276 Metetra
.663 .741 .908 r
.24743 .28276 .25758 .29314 .27282 .28756 .26273 .2771 Metetra
.662 .74 .908 r
.26273 .2771 .27282 .28756 .28814 .28207 .27811 .27147 Metetra
.654 .736 .91 r
.27811 .27147 .28814 .28207 .30352 .2771 .29357 .26611 Metetra
.621 .719 .916 r
.29357 .26611 .30352 .2771 .31888 .27383 .30903 .26191 Metetra
.522 .661 .923 r
.30903 .26191 .31888 .27383 .33411 .27441 .3244 .26082 Metetra
.283 .503 .892 r
.3244 .26082 .33411 .27441 .34914 .2813 .33955 .26558 Metetra
0 .236 .753 r
.33955 .26558 .34914 .2813 .36403 .29531 .35449 .27783 Metetra
0 .056 .621 r
.35449 .27783 .36403 .29531 .379 .31367 .36944 .29557 Metetra
0 .113 .666 r
.36944 .29557 .379 .31367 .3944 .33006 .38477 .31238 Metetra
.238 .409 .831 r
.38477 .31238 .3944 .33006 .41046 .33738 .40079 .32027 Metetra
.641 .639 .833 r
.40079 .32027 .41046 .33738 .42716 .33169 .41752 .31451 Metetra
.791 .697 .761 r
.41752 .31451 .42716 .33169 .44427 .31409 .43471 .29622 Metetra
.831 .703 .723 r
.43471 .29622 .44427 .31409 .4615 .28938 .45205 .27091 Metetra
.832 .698 .716 r
.45205 .27091 .4615 .28938 .47866 .26314 .46929 .24475 Metetra
.816 .694 .731 r
.46929 .24475 .47866 .26314 .49567 .23941 .48638 .22187 Metetra
.79 .694 .759 r
.48638 .22187 .49567 .23941 .51258 .21995 .50336 .20374 Metetra
.758 .699 .795 r
.50336 .20374 .51258 .21995 .52946 .20481 .5203 .18999 Metetra
.727 .707 .831 r
.5203 .18999 .52946 .20481 .54636 .19311 .53727 .17944 Metetra
.702 .717 .861 r
.53727 .17944 .54636 .19311 .56333 .18377 .55432 .17092 Metetra
.683 .725 .882 r
.55432 .17092 .56333 .18377 .5804 .17586 .57146 .1635 Metetra
.673 .732 .894 r
.57146 .1635 .5804 .17586 .59758 .16874 .58872 .15663 Metetra
.668 .736 .901 r
.58872 .15663 .59758 .16874 .61487 .16201 .60608 .15 Metetra
.665 .739 .905 r
.60608 .15 .61487 .16201 .63227 .15544 .62356 .14345 Metetra
.664 .74 .907 r
.62356 .14345 .63227 .15544 .64978 .14893 .64114 .13691 Metetra
.664 .741 .907 r
.64114 .13691 .64978 .14893 .6674 .14242 .65884 .13034 Metetra
.664 .741 .908 r
.65884 .13034 .6674 .14242 .68513 .13589 .67665 .12374 Metetra
.664 .741 .908 r
.67665 .12374 .68513 .13589 .70297 .12932 .69457 .1171 Metetra
.664 .741 .908 r
.69457 .1171 .70297 .12932 .72093 .12272 .7126 .11041 Metetra
.664 .741 .908 r
.7126 .11041 .72093 .12272 .739 .11607 .73075 .10369 Metetra
.664 .741 .908 r
.73075 .10369 .739 .11607 .75719 .10938 .74902 .09692 Metetra
.664 .741 .908 r
.10296 .32237 .11366 .33233 .12818 .32695 .11753 .31693 Metetra
.664 .741 .908 r
.11753 .31693 .12818 .32695 .14279 .32154 .13219 .31146 Metetra
.664 .741 .908 r
.13219 .31146 .14279 .32154 .15748 .31609 .14693 .30596 Metetra
.664 .741 .908 r
.14693 .30596 .15748 .31609 .17226 .31062 .16175 .30043 Metetra
.664 .741 .908 r
.16175 .30043 .17226 .31062 .18712 .30511 .17666 .29486 Metetra
.664 .741 .908 r
.17666 .29486 .18712 .30511 .20207 .29957 .19166 .28926 Metetra
.664 .741 .908 r
.19166 .28926 .20207 .29957 .2171 .294 .20674 .28363 Metetra
.664 .741 .908 r
.20674 .28363 .2171 .294 .23222 .2884 .22192 .27797 Metetra
.664 .741 .908 r
.22192 .27797 .23222 .2884 .24743 .28276 .23718 .27227 Metetra
.664 .741 .908 r
.23718 .27227 .24743 .28276 .26273 .2771 .25253 .26654 Metetra
.663 .741 .908 r
.25253 .26654 .26273 .2771 .27811 .27147 .26797 .2608 Metetra
.659 .739 .909 r
.26797 .2608 .27811 .27147 .29357 .26611 .28349 .25517 Metetra
.641 .729 .912 r
.28349 .25517 .29357 .26611 .30903 .26191 .29906 .25022 Metetra
.573 .69 .92 r
.29906 .25022 .30903 .26191 .3244 .26082 .31456 .24756 Metetra
.386 .569 .91 r
.31456 .24756 .3244 .26082 .33955 .26558 .32987 .25 Metetra
.044 .316 .802 r
.32987 .25 .33955 .26558 .35449 .27783 .3449 .26008 Metetra
0 .091 .649 r
.3449 .26008 .35449 .27783 .36944 .29557 .35985 .27686 Metetra
0 .103 .658 r
.35985 .27686 .36944 .29557 .38477 .31238 .3751 .29403 Metetra
.217 .387 .819 r
.3751 .29403 .38477 .31238 .40079 .32027 .39106 .30259 Metetra
.64 .633 .827 r
.39106 .30259 .40079 .32027 .41752 .31451 .40782 .29675 Metetra
.793 .693 .755 r
.40782 .29675 .41752 .31451 .43471 .29622 .42511 .27768 Metetra
.83 .698 .718 r
.42511 .27768 .43471 .29622 .45205 .27091 .44255 .25177 Metetra
.827 .692 .715 r
.44255 .25177 .45205 .27091 .46929 .24475 .45988 .22591 Metetra
.806 .688 .735 r
.45988 .22591 .46929 .24475 .48638 .22187 .47702 .20422 Metetra
.776 .69 .768 r
.47702 .20422 .48638 .22187 .50336 .20374 .49405 .18771 Metetra
.742 .697 .808 r
.49405 .18771 .50336 .20374 .5203 .18999 .51105 .17547 Metetra
.712 .708 .845 r
.51105 .17547 .5203 .18999 .53727 .17944 .52809 .16608 Metetra
.689 .719 .873 r
.52809 .16608 .53727 .17944 .55432 .17092 .54521 .15827 Metetra
.675 .729 .89 r
.54521 .15827 .55432 .17092 .57146 .1635 .56243 .15124 Metetra
.668 .735 .9 r
.56243 .15124 .57146 .1635 .58872 .15663 .57976 .14453 Metetra
.665 .738 .904 r
.57976 .14453 .58872 .15663 .60608 .15 .5972 .13794 Metetra
.664 .74 .906 r
.5972 .13794 .60608 .15 .62356 .14345 .61475 .13136 Metetra
.664 .741 .907 r
.61475 .13136 .62356 .14345 .64114 .13691 .63241 .12476 Metetra
.664 .741 .908 r
.63241 .12476 .64114 .13691 .65884 .13034 .65018 .11812 Metetra
.664 .741 .908 r
.65018 .11812 .65884 .13034 .67665 .12374 .66806 .11144 Metetra
.664 .741 .908 r
.66806 .11144 .67665 .12374 .69457 .1171 .68606 .10472 Metetra
.664 .741 .908 r
.68606 .10472 .69457 .1171 .7126 .11041 .70418 .09796 Metetra
.664 .741 .908 r
.70418 .09796 .7126 .11041 .73075 .10369 .72241 .09116 Metetra
.664 .741 .908 r
.72241 .09116 .73075 .10369 .74902 .09692 .74076 .08431 Metetra
.664 .741 .908 r
.09215 .3123 .10296 .32237 .11753 .31693 .10677 .30681 Metetra
.664 .741 .908 r
.10677 .30681 .11753 .31693 .13219 .31146 .12147 .30128 Metetra
.664 .741 .908 r
.12147 .30128 .13219 .31146 .14693 .30596 .13626 .29572 Metetra
.664 .741 .908 r
.13626 .29572 .14693 .30596 .16175 .30043 .15113 .29013 Metetra
.664 .741 .908 r
.15113 .29013 .16175 .30043 .17666 .29486 .16609 .2845 Metetra
.664 .741 .908 r
.16609 .2845 .17666 .29486 .19166 .28926 .18114 .27884 Metetra
.664 .741 .908 r
.18114 .27884 .19166 .28926 .20674 .28363 .19627 .27315 Metetra
.664 .741 .908 r
.19627 .27315 .20674 .28363 .22192 .27797 .2115 .26743 Metetra
.664 .741 .908 r
.2115 .26743 .22192 .27797 .23718 .27227 .22681 .26167 Metetra
.664 .741 .908 r
.22681 .26167 .23718 .27227 .25253 .26654 .24222 .25587 Metetra
.663 .741 .908 r
.24222 .25587 .25253 .26654 .26797 .2608 .25772 .25005 Metetra
.662 .74 .908 r
.25772 .25005 .26797 .2608 .28349 .25517 .2733 .24425 Metetra
.653 .735 .91 r
.2733 .24425 .28349 .25517 .29906 .25022 .28895 .23875 Metetra
.613 .712 .916 r
.28895 .23875 .29906 .25022 .31456 .24756 .3046 .2347 Metetra
.48 .628 .917 r
.3046 .2347 .31456 .24756 .32987 .25 .32008 .23473 Metetra
.174 .409 .849 r
.32008 .23473 .32987 .25 .3449 .26008 .33526 .24215 Metetra
0 .143 .688 r
.33526 .24215 .3449 .26008 .35985 .27686 .35022 .25745 Metetra
0 .098 .654 r
.35022 .25745 .35985 .27686 .3751 .29403 .3654 .27487 Metetra
.193 .362 .804 r
.3654 .27487 .3751 .29403 .39106 .30259 .38129 .28423 Metetra
.638 .625 .821 r
.38129 .28423 .39106 .30259 .40782 .29675 .39808 .27831 Metetra
.796 .689 .747 r
.39808 .27831 .40782 .29675 .42511 .27768 .41547 .25833 Metetra
.828 .692 .713 r
.41547 .25833 .42511 .27768 .44255 .25177 .43301 .23188 Metetra
.82 .685 .716 r
.43301 .23188 .44255 .25177 .45988 .22591 .4504 .20666 Metetra
.793 .681 .741 r
.4504 .20666 .45988 .22591 .47702 .20422 .46759 .18662 Metetra
.758 .685 .781 r
.46759 .18662 .47702 .20422 .49405 .18771 .48466 .17202 Metetra
.723 .696 .824 r
.48466 .17202 .49405 .18771 .51105 .17547 .50171 .16139 Metetra
.695 .711 .86 r
.50171 .16139 .51105 .17547 .52809 .16608 .51881 .15305 Metetra
.678 .724 .884 r
.51881 .15305 .52809 .16608 .54521 .15827 .536 .14581 Metetra
.669 .732 .897 r
.536 .14581 .54521 .15827 .56243 .15124 .55329 .13903 Metetra
.665 .737 .904 r
.55329 .13903 .56243 .15124 .57976 .14453 .5707 .13239 Metetra
.664 .74 .906 r
.5707 .13239 .57976 .14453 .5972 .13794 .58821 .12578 Metetra
.664 .741 .907 r
.58821 .12578 .5972 .13794 .61475 .13136 .60583 .11914 Metetra
.664 .741 .908 r
.60583 .11914 .61475 .13136 .63241 .12476 .62357 .11247 Metetra
.664 .741 .908 r
.62357 .11247 .63241 .12476 .65018 .11812 .64142 .10576 Metetra
.664 .741 .908 r
.64142 .10576 .65018 .11812 .66806 .11144 .65938 .09901 Metetra
.664 .741 .908 r
.65938 .09901 .66806 .11144 .68606 .10472 .67746 .09221 Metetra
.664 .741 .908 r
.67746 .09221 .68606 .10472 .70418 .09796 .69565 .08537 Metetra
.664 .741 .908 r
.69565 .08537 .70418 .09796 .72241 .09116 .71397 .07848 Metetra
.664 .741 .908 r
.71397 .07848 .72241 .09116 .74076 .08431 .7324 .07155 Metetra
.664 .741 .908 r
.08123 .30213 .09215 .3123 .10677 .30681 .09589 .29658 Metetra
.664 .741 .908 r
.09589 .29658 .10677 .30681 .12147 .30128 .11064 .29099 Metetra
.664 .741 .908 r
.11064 .29099 .12147 .30128 .13626 .29572 .12548 .28537 Metetra
.664 .741 .908 r
.12548 .28537 .13626 .29572 .15113 .29013 .1404 .27972 Metetra
.664 .741 .908 r
.1404 .27972 .15113 .29013 .16609 .2845 .15541 .27403 Metetra
.664 .741 .908 r
.15541 .27403 .16609 .2845 .18114 .27884 .1705 .26831 Metetra
.664 .741 .908 r
.1705 .26831 .18114 .27884 .19627 .27315 .18569 .26256 Metetra
.664 .741 .908 r
.18569 .26256 .19627 .27315 .2115 .26743 .20097 .25677 Metetra
.664 .741 .908 r
.20097 .25677 .2115 .26743 .22681 .26167 .21634 .25095 Metetra
.664 .741 .908 r
.21634 .25095 .22681 .26167 .24222 .25587 .23179 .24509 Metetra
.664 .741 .908 r
.23179 .24509 .24222 .25587 .25772 .25005 .24735 .2392 Metetra
.663 .741 .908 r
.24735 .2392 .25772 .25005 .2733 .24425 .26299 .23328 Metetra
.66 .739 .908 r
.26299 .23328 .2733 .24425 .28895 .23875 .27872 .22745 Metetra
.641 .728 .911 r
.27872 .22745 .28895 .23875 .3046 .2347 .29448 .22233 Metetra
.559 .676 .917 r
.29448 .22233 .3046 .2347 .32008 .23473 .31016 .22 Metetra
.318 .508 .886 r
.31016 .22 .32008 .23473 .33526 .24215 .32553 .22422 Metetra
0 .219 .739 r
.32553 .22422 .33526 .24215 .35022 .25745 .34058 .23731 Metetra
0 .101 .656 r
.34058 .23731 .35022 .25745 .3654 .27487 .35568 .25472 Metetra
.166 .332 .787 r
.35568 .25472 .3654 .27487 .38129 .28423 .37149 .26505 Metetra
.636 .616 .813 r
.37149 .26505 .38129 .28423 .39808 .27831 .3883 .25905 Metetra
.798 .684 .738 r
.3883 .25905 .39808 .27831 .41547 .25833 .4058 .23799 Metetra
.825 .684 .708 r
.4058 .23799 .41547 .25833 .43301 .23188 .42344 .21114 Metetra
.808 .676 .718 r
.42344 .21114 .43301 .23188 .4504 .20666 .44088 .18712 Metetra
.775 .673 .751 r
.44088 .18712 .4504 .20666 .46759 .18662 .45808 .16929 Metetra
.736 .681 .798 r
.45808 .16929 .46759 .18662 .48466 .17202 .47517 .15689 Metetra
.702 .699 .843 r
.47517 .15689 .48466 .17202 .50171 .16139 .49227 .14782 Metetra
.68 .716 .876 r
.49227 .14782 .50171 .16139 .51881 .15305 .50943 .14035 Metetra
.669 .729 .895 r
.50943 .14035 .51881 .15305 .536 .14581 .52669 .13349 Metetra
.665 .736 .903 r
.52669 .13349 .536 .14581 .55329 .13903 .54406 .12682 Metetra
.664 .739 .906 r
.54406 .12682 .55329 .13903 .5707 .13239 .56153 .12017 Metetra
.664 .741 .907 r
.56153 .12017 .5707 .13239 .58821 .12578 .57912 .1135 Metetra
.664 .741 .908 r
.57912 .1135 .58821 .12578 .60583 .11914 .59682 .1068 Metetra
.664 .741 .908 r
.59682 .1068 .60583 .11914 .62357 .11247 .61463 .10005 Metetra
.664 .741 .908 r
.61463 .10005 .62357 .11247 .64142 .10576 .63255 .09326 Metetra
.664 .741 .908 r
.63255 .09326 .64142 .10576 .65938 .09901 .6506 .08642 Metetra
.664 .741 .908 r
.6506 .08642 .65938 .09901 .67746 .09221 .66875 .07954 Metetra
.664 .741 .908 r
.66875 .07954 .67746 .09221 .69565 .08537 .68703 .07262 Metetra
.664 .741 .908 r
.68703 .07262 .69565 .08537 .71397 .07848 .70542 .06565 Metetra
.664 .741 .908 r
.70542 .06565 .71397 .07848 .7324 .07155 .72394 .05863 Metetra
.664 .741 .908 r
.07019 .29185 .08123 .30213 .09589 .29658 .0849 .28624 Metetra
.664 .741 .908 r
.0849 .28624 .09589 .29658 .11064 .29099 .09969 .28059 Metetra
.664 .741 .908 r
.09969 .28059 .11064 .29099 .12548 .28537 .11458 .27491 Metetra
.664 .741 .908 r
.11458 .27491 .12548 .28537 .1404 .27972 .12955 .26919 Metetra
.664 .741 .908 r
.12955 .26919 .1404 .27972 .15541 .27403 .14461 .26344 Metetra
.664 .741 .908 r
.14461 .26344 .15541 .27403 .1705 .26831 .15975 .25766 Metetra
.664 .741 .908 r
.15975 .25766 .1705 .26831 .18569 .26256 .17499 .25184 Metetra
.664 .741 .908 r
.17499 .25184 .18569 .26256 .20097 .25677 .19032 .24599 Metetra
.664 .741 .908 r
.19032 .24599 .20097 .25677 .21634 .25095 .20574 .24011 Metetra
.664 .741 .908 r
.20574 .24011 .21634 .25095 .23179 .24509 .22125 .23418 Metetra
.664 .741 .908 r
.22125 .23418 .23179 .24509 .24735 .2392 .23686 .22823 Metetra
.664 .741 .908 r
.23686 .22823 .24735 .2392 .26299 .23328 .25256 .22224 Metetra
.663 .741 .908 r
.25256 .22224 .26299 .23328 .27872 .22745 .26835 .21623 Metetra
.657 .737 .909 r
.26835 .21623 .27872 .22745 .29448 .22233 .28422 .21043 Metetra
.617 .711 .912 r
.28422 .21043 .29448 .22233 .31016 .22 .30008 .20608 Metetra
.46 .603 .907 r
.30008 .20608 .31016 .22 .32553 .22422 .31571 .20666 Metetra
.105 .329 .801 r
.31571 .20666 .32553 .22422 .34058 .23731 .33092 .21644 Metetra
0 .122 .669 r
.33092 .21644 .34058 .23731 .35568 .25472 .34598 .23329 Metetra
.137 .298 .765 r
.34598 .23329 .35568 .25472 .37149 .26505 .36168 .2448 Metetra
.633 .605 .803 r
.36168 .2448 .37149 .26505 .3883 .25905 .3785 .23872 Metetra
.8 .677 .728 r
.3785 .23872 .3883 .25905 .4058 .23799 .39613 .21636 Metetra
.818 .674 .704 r
.39613 .21636 .4058 .23799 .42344 .21114 .41384 .18948 Metetra
.791 .663 .722 r
.41384 .18948 .42344 .21114 .44088 .18712 .43128 .16755 Metetra
.749 .664 .767 r
.43128 .16755 .44088 .18712 .45808 .16929 .44848 .15263 Metetra
.709 .681 .821 r
.44848 .15263 .45808 .16929 .47517 .15689 .46558 .14257 Metetra
.682 .706 .865 r
.46558 .14257 .47517 .15689 .49227 .14782 .48272 .13482 Metetra
.669 .725 .891 r
.48272 .13482 .49227 .14782 .50943 .14035 .49994 .1279 Metetra
.665 .735 .902 r
.49994 .1279 .50943 .14035 .52669 .13349 .51727 .12121 Metetra
.664 .739 .906 r
.51727 .12121 .52669 .13349 .54406 .12682 .53471 .11453 Metetra
.664 .741 .907 r
.53471 .11453 .54406 .12682 .56153 .12017 .55226 .10783 Metetra
.664 .741 .908 r
.55226 .10783 .56153 .12017 .57912 .1135 .56992 .10109 Metetra
.664 .741 .908 r
.56992 .10109 .57912 .1135 .59682 .1068 .58769 .0943 Metetra
.664 .741 .908 r
.58769 .0943 .59682 .1068 .61463 .10005 .60558 .08748 Metetra
.664 .741 .908 r
.60558 .08748 .61463 .10005 .63255 .09326 .62358 .0806 Metetra
.664 .741 .908 r
.62358 .0806 .63255 .09326 .6506 .08642 .6417 .07369 Metetra
.664 .741 .908 r
.6417 .07369 .6506 .08642 .66875 .07954 .65994 .06672 Metetra
.664 .741 .908 r
.65994 .06672 .66875 .07954 .68703 .07262 .6783 .05972 Metetra
.664 .741 .908 r
.6783 .05972 .68703 .07262 .70542 .06565 .69678 .05266 Metetra
.664 .741 .908 r
.69678 .05266 .70542 .06565 .72394 .05863 .71537 .04556 Metetra
.664 .741 .908 r
.05902 .28146 .07019 .29185 .0849 .28624 .07378 .27578 Metetra
.664 .741 .908 r
.07378 .27578 .0849 .28624 .09969 .28059 .08863 .27007 Metetra
.664 .741 .908 r
.08863 .27007 .09969 .28059 .11458 .27491 .10356 .26433 Metetra
.664 .741 .908 r
.10356 .26433 .11458 .27491 .12955 .26919 .11858 .25855 Metetra
.664 .741 .908 r
.11858 .25855 .12955 .26919 .14461 .26344 .13369 .25274 Metetra
.664 .741 .908 r
.13369 .25274 .14461 .26344 .15975 .25766 .14889 .24689 Metetra
.664 .741 .908 r
.14889 .24689 .15975 .25766 .17499 .25184 .16417 .24101 Metetra
.664 .741 .908 r
.16417 .24101 .17499 .25184 .19032 .24599 .17956 .2351 Metetra
.664 .741 .908 r
.17956 .2351 .19032 .24599 .20574 .24011 .19503 .22915 Metetra
.664 .741 .908 r
.19503 .22915 .20574 .24011 .22125 .23418 .2106 .22316 Metetra
.664 .741 .908 r
.2106 .22316 .22125 .23418 .23686 .22823 .22626 .21713 Metetra
.664 .741 .908 r
.22626 .21713 .23686 .22823 .25256 .22224 .24201 .21107 Metetra
.664 .741 .908 r
.24201 .21107 .25256 .22224 .26835 .21623 .25786 .20498 Metetra
.663 .741 .908 r
.25786 .20498 .26835 .21623 .28422 .21043 .27381 .19888 Metetra
.652 .732 .908 r
.27381 .19888 .28422 .21043 .30008 .20608 .28982 .19318 Metetra
.578 .68 .91 r
.28982 .19318 .30008 .20608 .31571 .20666 .30573 .19011 Metetra
.307 .475 .862 r
.30573 .19011 .31571 .20666 .33092 .21644 .32124 .19506 Metetra
0 .179 .704 r
.32124 .19506 .33092 .21644 .34598 .23329 .33634 .21012 Metetra
.109 .259 .738 r
.33634 .21012 .34598 .23329 .36168 .2448 .35188 .22307 Metetra
.629 .589 .79 r
.35188 .22307 .36168 .2448 .3785 .23872 .36872 .2169 Metetra
.801 .667 .715 r
.36872 .2169 .3785 .23872 .39613 .21636 .38648 .19295 Metetra
.804 .658 .701 r
.38648 .19295 .39613 .21636 .41384 .18948 .40423 .16695 Metetra
.761 .645 .733 r
.40423 .16695 .41384 .18948 .43128 .16755 .42161 .14856 Metetra
.714 .656 .791 r
.42161 .14856 .43128 .16755 .44848 .15263 .43875 .13719 Metetra
.682 .689 .85 r
.43875 .13719 .44848 .15263 .46558 .14257 .45587 .12919 Metetra
.668 .719 .887 r
.45587 .12919 .46558 .14257 .48272 .13482 .47305 .12227 Metetra
.664 .734 .902 r
.47305 .12227 .48272 .13482 .49994 .1279 .49034 .11556 Metetra
.664 .739 .906 r
.49034 .11556 .49994 .1279 .51727 .12121 .50774 .10887 Metetra
.664 .741 .908 r
.50774 .10887 .51727 .12121 .53471 .11453 .52525 .10213 Metetra
.664 .741 .908 r
.52525 .10213 .53471 .11453 .55226 .10783 .54287 .09535 Metetra
.664 .741 .908 r
.54287 .09535 .55226 .10783 .56992 .10109 .56061 .08853 Metetra
.664 .741 .908 r
.56061 .08853 .56992 .10109 .58769 .0943 .57846 .08166 Metetra
.664 .741 .908 r
.57846 .08166 .58769 .0943 .60558 .08748 .59643 .07475 Metetra
.664 .741 .908 r
.59643 .07475 .60558 .08748 .62358 .0806 .61451 .0678 Metetra
.664 .741 .908 r
.61451 .0678 .62358 .0806 .6417 .07369 .63271 .0608 Metetra
.664 .741 .908 r
.63271 .0608 .6417 .07369 .65994 .06672 .65103 .05375 Metetra
.664 .741 .908 r
.65103 .05375 .65994 .06672 .6783 .05972 .66946 .04666 Metetra
.664 .741 .908 r
.66946 .04666 .6783 .05972 .69678 .05266 .68802 .03952 Metetra
.664 .741 .908 r
.68802 .03952 .69678 .05266 .71537 .04556 .70671 .03234 Metetra
.664 .741 .908 r
.04774 .27095 .05902 .28146 .07378 .27578 .06255 .26522 Metetra
.664 .741 .908 r
.06255 .26522 .07378 .27578 .08863 .27007 .07744 .25944 Metetra
.664 .741 .908 r
.07744 .25944 .08863 .27007 .10356 .26433 .09242 .25364 Metetra
.664 .741 .908 r
.09242 .25364 .10356 .26433 .11858 .25855 .10749 .2478 Metetra
.664 .741 .908 r
.10749 .2478 .11858 .25855 .13369 .25274 .12265 .24192 Metetra
.664 .741 .908 r
.12265 .24192 .13369 .25274 .14889 .24689 .1379 .23601 Metetra
.664 .741 .908 r
.1379 .23601 .14889 .24689 .16417 .24101 .15324 .23006 Metetra
.664 .741 .908 r
.15324 .23006 .16417 .24101 .17956 .2351 .16867 .22408 Metetra
.664 .741 .908 r
.16867 .22408 .17956 .2351 .19503 .22915 .1842 .21806 Metetra
.664 .741 .908 r
.1842 .21806 .19503 .22915 .2106 .22316 .19982 .21201 Metetra
.664 .741 .908 r
.19982 .21201 .2106 .22316 .22626 .21713 .21553 .20592 Metetra
.664 .741 .908 r
.21553 .20592 .22626 .21713 .24201 .21107 .23135 .19979 Metetra
.664 .741 .908 r
.23135 .19979 .24201 .21107 .25786 .20498 .24725 .19362 Metetra
.664 .741 .908 r
.24725 .19362 .25786 .20498 .27381 .19888 .26326 .18742 Metetra
.663 .74 .908 r
.26326 .18742 .27381 .19888 .28982 .19318 .27936 .18121 Metetra
.648 .727 .906 r
.27936 .18121 .28982 .19318 .30573 .19011 .29552 .17553 Metetra
.526 .629 .896 r
.29552 .17553 .30573 .19011 .32124 .19506 .31147 .17418 Metetra
.152 .315 .776 r
.31147 .17418 .32124 .19506 .33634 .21012 .32681 .18451 Metetra
.095 .224 .708 r
.32681 .18451 .33634 .21012 .35188 .22307 .34217 .19897 Metetra
.622 .565 .769 r
.34217 .19897 .35188 .22307 .36872 .2169 .35901 .19273 Metetra
.798 .652 .7 r
.35901 .19273 .36872 .2169 .38648 .19295 .37691 .16698 Metetra
.772 .629 .702 r
.37691 .16698 .38648 .19295 .40423 .16695 .39458 .14427 Metetra
.711 .622 .756 r
.39458 .14427 .40423 .16695 .42161 .14856 .41179 .13143 Metetra
.674 .663 .832 r
.41179 .13143 .42161 .14856 .43875 .13719 .42888 .12344 Metetra
.664 .712 .884 r
.42888 .12344 .43875 .13719 .45587 .12919 .44602 .1166 Metetra
.663 .734 .903 r
.44602 .1166 .45587 .12919 .47305 .12227 .46327 .1099 Metetra
.663 .74 .907 r
.46327 .1099 .47305 .12227 .49034 .11556 .48063 .10317 Metetra
.664 .741 .908 r
.48063 .10317 .49034 .11556 .50774 .10887 .4981 .0964 Metetra
.664 .741 .908 r
.4981 .0964 .50774 .10887 .52525 .10213 .51568 .08958 Metetra
.664 .741 .908 r
.51568 .08958 .52525 .10213 .54287 .09535 .53338 .08272 Metetra
.664 .741 .908 r
.53338 .08272 .54287 .09535 .56061 .08853 .55119 .07582 Metetra
.664 .741 .908 r
.55119 .07582 .56061 .08853 .57846 .08166 .56912 .06887 Metetra
.664 .741 .908 r
.56912 .06887 .57846 .08166 .59643 .07475 .58716 .06188 Metetra
.664 .741 .908 r
.58716 .06188 .59643 .07475 .61451 .0678 .60532 .05484 Metetra
.664 .741 .908 r
.60532 .05484 .61451 .0678 .63271 .0608 .6236 .04775 Metetra
.664 .741 .908 r
.6236 .04775 .63271 .0608 .65103 .05375 .642 .04062 Metetra
.664 .741 .908 r
.642 .04062 .65103 .05375 .66946 .04666 .66052 .03344 Metetra
.664 .741 .908 r
.66052 .03344 .66946 .04666 .68802 .03952 .67916 .02622 Metetra
.664 .741 .908 r
.67916 .02622 .68802 .03952 .70671 .03234 .69793 .01894 Metetra
.664 .741 .908 r
.03634 .26033 .04774 .27095 .06255 .26522 .05119 .25453 Metetra
.664 .741 .908 r
.05119 .25453 .06255 .26522 .07744 .25944 .06613 .2487 Metetra
.664 .741 .908 r
.06613 .2487 .07744 .25944 .09242 .25364 .08116 .24283 Metetra
.664 .741 .908 r
.08116 .24283 .09242 .25364 .10749 .2478 .09628 .23692 Metetra
.664 .741 .908 r
.09628 .23692 .10749 .2478 .12265 .24192 .11148 .23098 Metetra
.664 .741 .908 r
.11148 .23098 .12265 .24192 .1379 .23601 .12679 .225 Metetra
.664 .741 .908 r
.12679 .225 .1379 .23601 .15324 .23006 .14218 .21899 Metetra
.664 .741 .908 r
.14218 .21899 .15324 .23006 .16867 .22408 .15767 .21294 Metetra
.664 .741 .908 r
.15767 .21294 .16867 .22408 .1842 .21806 .17325 .20686 Metetra
.664 .741 .908 r
.17325 .20686 .1842 .21806 .19982 .21201 .18892 .20073 Metetra
.664 .741 .908 r
.18892 .20073 .19982 .21201 .21553 .20592 .20469 .19457 Metetra
.664 .741 .908 r
.20469 .19457 .21553 .20592 .23135 .19979 .22056 .18838 Metetra
.664 .741 .908 r
.22056 .18838 .23135 .19979 .24725 .19362 .23652 .18214 Metetra
.664 .741 .908 r
.23652 .18214 .24725 .19362 .26326 .18742 .25259 .17587 Metetra
.664 .741 .908 r
.25259 .17587 .26326 .18742 .27936 .18121 .26875 .16955 Metetra
.663 .741 .907 r
.26875 .16955 .27936 .18121 .29552 .17553 .28501 .1632 Metetra
.656 .727 .901 r
.28501 .1632 .29552 .17553 .31147 .17418 .30136 .15707 Metetra
.514 .585 .864 r
.30136 .15707 .31147 .17418 .32681 .18451 .31743 .15629 Metetra
.165 .247 .704 r
.31743 .15629 .32681 .18451 .34217 .19897 .33275 .16985 Metetra
.609 .522 .732 r
.33275 .16985 .34217 .19897 .35901 .19273 .34955 .16354 Metetra
.773 .614 .683 r
.34955 .16354 .35901 .19273 .37691 .16698 .3675 .13782 Metetra
.682 .565 .716 r
.3675 .13782 .37691 .16698 .39458 .14427 .38473 .12481 Metetra
.651 .625 .811 r
.38473 .12481 .39458 .14427 .41179 .13143 .40174 .11762 Metetra
.66 .711 .885 r
.40174 .11762 .41179 .13143 .42888 .12344 .41884 .11093 Metetra
.663 .737 .905 r
.41884 .11093 .42888 .12344 .44602 .1166 .43605 .10421 Metetra
.664 .741 .908 r
.43605 .10421 .44602 .1166 .46327 .1099 .45337 .09744 Metetra
.664 .741 .908 r
.45337 .09744 .46327 .1099 .48063 .10317 .4708 .09063 Metetra
.664 .741 .908 r
.4708 .09063 .48063 .10317 .4981 .0964 .48835 .08378 Metetra
.664 .741 .908 r
.48835 .08378 .4981 .0964 .51568 .08958 .506 .07688 Metetra
.664 .741 .908 r
.506 .07688 .51568 .08958 .53338 .08272 .52377 .06994 Metetra
.664 .741 .908 r
.52377 .06994 .53338 .08272 .55119 .07582 .54166 .06296 Metetra
.664 .741 .908 r
.54166 .06296 .55119 .07582 .56912 .06887 .55966 .05592 Metetra
.664 .741 .908 r
.55966 .05592 .56912 .06887 .58716 .06188 .57778 .04885 Metetra
.664 .741 .908 r
.57778 .04885 .58716 .06188 .60532 .05484 .59602 .04172 Metetra
.664 .741 .908 r
.59602 .04172 .60532 .05484 .6236 .04775 .61438 .03455 Metetra
.664 .741 .908 r
.61438 .03455 .6236 .04775 .642 .04062 .63286 .02733 Metetra
.664 .741 .908 r
.63286 .02733 .642 .04062 .66052 .03344 .65147 .02007 Metetra
.664 .741 .908 r
.65147 .02007 .66052 .03344 .67916 .02622 .6702 .01275 Metetra
.664 .741 .908 r
.6702 .01275 .67916 .02622 .69793 .01894 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{183.312, 148.688},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003U0000^A000`40O003h00OiGoo003UOol00>Eo
o`00iGoo003UOol00>Eoo`00iGoo003UOol00>Eoo`00iGoo0022Ool50003Ool30005Ool3001@Ool0
08Aoo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo0000Cgoo0024Ool00`00Oomo
o`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0004moo`00Q7oo00<007ooOol00Woo00D007oo
Oomoo`0000=oo`05001oogooOol0001?Ool008Aoo`03001oogoo009oo`05001oogooOol00003Ool0
1@00Oomoogoo0000Cgoo0024Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00
04moo`00Q7oo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol0001?Ool0089oo`<0
00Eoo`<000Eoo`<0051oo`00iGoo003UOol00>Eoo`00U7oo0P00Cgoo002AOol5001?Ool006Eoo`<0
00Eoo`<001moo`@0009Fo@8004ioo`00I7oo00D007ooOomoo`0000=oo`05001oogooOol0000KOol4
00000eKm001Fo@02E_d3001=Ool006Aoo`05001oogooOol00003Ool01@00Oomoogoo000067oo1@00
15Km00<005Km00000`00CGoo001TOol01@00Oomoogoo00000goo00D007ooOomoo`0001Ioo`D0009F
o@03001FoEKm00<00003E_d005Km008004aoo`00IGoo0`0017oo00D007ooOomoo`00019oo`D000=F
o@03001FoEKm00<000=Fo@800004E_d00000001;Ool006Aoo`05001oogooOol00003Ool01@00Oomo
ogoo00004Goo0`0000=Fo@00E_d00eKm0`0000=Fo@00E_d00eKm0`000UKm00<007ooOol0BGoo001T
Ool01@00Oomoogoo00000goo00D007ooOomoo`0000ioo`D000=Fo@06001Fo@00001Fo@000eKm00H0
05Km000005Km0003E_d2001:Ool006Eoo`<000Eoo`<000eoo`@000=Fo@03001FoEKm008000AFo@03
001Fo@00008000=Fo@@00003E_d0000004Uoo`00NWoo100000=Fo@00E_d00eKm0`000UKm00<005Km
E_d00`000eKm00<005KmE_d00P0015Km00<007ooOol0Agoo001gOol400000eKm001Fo@02E_d01P00
E_d00000E_d000=Fo@<00003E_d005Km00=Fo@@00004E_d005KmE_d30018Ool007Eoo`@000AFo@03
001FoEKm008000AFo@<00003E_d005Km009Fo@09001Fo@00001Fo@00E_d005Km00<0009Fo@8000Mo
o`<000Ioo`8000Eoo`D002]oo`00LWoo1@000UKm00<005KmE_d010000UKm00<005KmE_d00P0015Km
00<005KmE_d00P0015Km0`0000=Fo@00E_d00UKm0P001Woo00D007ooOomoo`0000Eoo`8000Moo`03
001oogoo02]oo`00L7oo100000=Fo@00E_d00UKm0`000UKm00<005KmE_d010000UKm00<005KmE_d0
10000UKm00<005KmE_d00P0000QFo@00E_eFo@00E_d005Km0P001Goo00D007ooOomoo`0000ioo`03
001oogoo02]oo`00A7oo0`001Goo0`007Woo0`0000=Fo@00E_d00eKm00D005Km001Fo@0000=Fo@<0
009Fo@04001FoEKmE_d30002E_d00`00E_eFo@040002E_d2000015Km000000000eKm0P0017oo00D0
07ooOomoo`0000ioo`03001oogoo02]oo`00@goo00D007ooOomoo`0000=oo`05001oogooOol0000J
Ool50003E_d00`00E_eFo@030003E_d01@00E_d005Km000015Km00D005Km001Fo@0000=Fo@<0009F
o@03001FoEKm00@0009Fo@03001FoEKm00<000Aoo`05001oogooOol0000>Ool00`00Oomoo`0[Ool0
04=oo`05001oogooOol00003Ool01@00Oomoogoo000067oo1@000UKm00<005KmE_d00`000eKm00<0
05KmE_d00P000eKm00<005KmE_d00`000eKm00D005Km001Fo@0000=Fo@<0009Fo@04001FoEKmE_d3
0002E_d20003Ool01@00Oomoogoo00003Woo00<007ooOol0:goo0013Ool01@00Oomoogoo00000goo
00D007ooOomoo`0001Eoo`@0009Fo@04001FoEKmE_d30002E_d00`00E_eFo@040002E_d00`00E_eF
o@030002E_d00`00E_eFo@030003E_d01@00E_d005Km00000eKm0P0000AFo@00E_d000=Fo@<0009o
o`05001oogooOol0000>Ool00`00Oomoo`0[Ool004=oo`@000Aoo`05001oogooOol0000COol30000
0eKm001Fo@03E_d01@00E_d005Km00000eKm0`000UKm00<005KmE_d010000UKm00<005KmE_d00`00
0eKm00<005KmE_d00P000eKm00<005KmE_d00`000eKm00@005Km001Fo@8000=oo`<000eoo`<002eo
o`00@goo00<007ooOol01Goo00D007ooOomoo`00011oo`D000=Fo@03001FoEKm00<000=Fo@05001F
o@00E_d00003E_d30002E_d01000E_eFoEKm0`000UKm00<005KmE_d010000UKm00<005KmE_d01000
00AFo@00E_eFo@<000=Fo@80049oo`00A7oo00<007ooOol017oo00D007ooOomoo`0000ioo`D0009F
o003001FoEKm00<0009Fo@03001FoEKm00<000=Fo@05001Fo@00E_d00004E_d01@00E_d005Km0000
0eKm0`000UKm00<005KmE_d010000UKm0P0000AFo@000000009Fo@03001FoEKm00@0045oo`00AGoo
0`0017oo0`0037oo10000eKL00<005KlE_`00`000UKl00<005KmE_d00`000eKl00<005KmE_d00P00
0eKm00<005KmE_d00`000eKm00D005Km001Fo@0000=Fo@<0009Fo@03001FoEKm00@0009Fo@03001F
oEKm00<0009Fo@80045oo`00FGoo0`0000=BN@00E]`00eKL0`0000=Fg000E_`00UKl0`000UKl00<0
05KlE_`010000UKl00<005KmE_d00`000UKm00<005KmE_d00`000eKm00D005Km001Fo@0000=Fo@<0
009Fo@04001FoEKmE_d30002E_d00`00E_eFo@030010Ool005Ioo`D000=BN@06001Fg000001F^P00
0eKL00D005Kl001Fo00000=Fo0<0009Fo003001Fo5Kl00@0009Fo003001FoEKm00<0009Fo@03001F
oEKm00<000=Fo@05001Fo@00E_d00003E_d400000eKm001Fo@02E_d30002E_d00`00Oomoo`0nOol0
05=oo`<0009oo`03001FEP0000=BN@8000AF^P03001Fg5KL008000=Fo005001Fo000E_`00003E_`3
0003E_`00`00E_aFo0030002E_`00`00E_eFo@030002E_d00`00E_eFo@030003E_d00`00E_eFo@02
0004E_d300000eKm001Fo@02E_d2000oOol0055oo`8000Aoo`09001FEUIFEUH0059i001JN00000AF
^P<0009Ff`03001Fo5Kl008000=Fo005001Fo000E_`00004E_`300000eKl001Fo002E_`30002E_`0
0`00E_eFo@030003E_d00`00E_eFo@04000015Km001FoEKm0P0015Km0`0000=Fo@000000?Woo000V
Ool30004Ool3000NOol30002Ool01@00Oomoogoo000015IF00H005YhFWQJN000E[X200000eZI001F
f`02E]/30002E]/00`00E_aFo0020003E_`01P00E_`00000E_`000=Fo005001Fo000E_`00003E_`3
0002E_d00`00E_eFo@040002E_d2000015Km000000000UKm00<005KmE_d00P0015Km00<007ooOol0
?7oo000WOol00`00Oomoo`02Ool01@00Oomoogoo00006goo0P0027oo00<0069e00000eIF00<005Yh
FWP00UYh00<005ZIFYT00UZI0`0000=J^P00E]/00UKK0`000UKL00<005KlE_`00P0015Kl00<005Kl
00000P000eKl00D005Kl001Fo00000=Fo@<0009Fo@03001FoEKm00@0009Fo@03001FoEKm00<0009F
o@03001FoEKm00<003eoo`0097oo1@000goo00D007ooOomoo`0001Qoo`<000Uoo`04001RMF9e0002
EUH00`00HYH00003FWP00`00FYUJV@02FYT00`00F[YJ^P02F[X01@00E]/005[J00000eKL0`000UKl
00<005KlE_`00`000UKl00<005KlE_`00`000eKl00D005Km001Fo00000=Fo@<0009Fo@03001FoEKm
00@0009Fo@03001FoEKm00<0009Fo@8003aoo`0097oo00@007ooOol000Aoo`05001oogooOol0000F
Ool80005Ool01`00HWERMF9e001FEP00009RUP06001JN5Yh001RU`000UZI0P0015Zj0P000e[J00D0
05KL001Jf`0000=Fo0<0009Fo003001Fo5Kl00<0009Fo003001Fo5Kl00<000=Fo005001Fo@00E_`0
0003E_d30002E_d00`00E_eFo@040002E_d00`00E_eFo@020008Ool30006Ool20004Ool5000POol0
02Eoo`03001oo`0000Aoo`05001oogooOol0000COol500001e;l0012Fd9K000DmP00009oo`8000ER
M@80009RUP03001JN000009RU`03001JV@00009N^006001J^P00001N^@000e[J0P000e[K00D005Kl
001Jn`0000=Fo0<0009Fo003001Fo5Kl00<0009Fo003001Fo5Kl00<000=Fo003001Fo@00008000=F
o@<0009Fo@04001FoEKmE_d300000eKm00000006Ool01@00Oomoogoo00001Goo0P001Goo00@007oo
Ool0021oo`009Goo00<007oo000017oo00D007ooOomoo`00011oo`D0009Fo@04001Bo5;l0002@U/0
1P005?H00000CQL000ERM@03001RUV:F009RUP03001RUf:G009RU`03001N^5jh009N^003001N^Eji
009N^@<00004G]X005[KF]/30003F_/01@00E_`005Kl00000eKl0`000UKl00<005KlE_`00`000UKl
00<005KlE_`00`0000QFo000E_`005Km001Fo00015Km0`0000AFo@00E_eFo@<000Eoo`05001oogoo
Ool0000=Ool00`00Oomoo`0POol002Ioo`8000Aoo`05001oogooOol0000>Ool2000017oo001Fo@00
0eKm1`0000<DmP00CQL00ThG00<0069eHWD00V9e00<006:FHYH00V:F00<006:GHYL00V:G00<005jh
G[P00Ujh00<005jiG[T00Uji00<005kJG]X00UkJ0P000U[K00<005[kF_/00`000eKl00D005Kl001F
o00000=Fo0<0009Fo003001Fo5Kl00<0009Fo003001Fo00000<000=Fo003001FoEKm008000AFo@80
0003E_d0000000Eoo`05001oogooOol0000>Ool00`00Oomoo`0OOol002Moo`03001oogoo00=oo`<0
00aoo`<00003Ool0000000=Fo@05001Fo@00D_`00002@Y`00`004EP000025?H01000CQM>5`000f9e
0P000f:F00<006Jf00000V:G00<006Jg00000Ujh00<006:h00000Uji0P000ekJ0P000e[K0`000e[K
00<005KlE_`00P000eKl00<005Kl00000UKl00<005KlE_`00`000UKl00<005KlE_`00`000UKl00<0
05KlE_`010000UKm00<005KmE_d00P000eKm0P0017oo00D007ooOomoo`0000moo`03001oogoo01io
o`00>Woo1P000UKm00<005KmE_d00P000e;l00P004:L@Y`0015H000DmP000ThG00@0069eHWD0009V
UP03001RUP00009V]P03001RU`00009V]`03001N^000009R^003001N^@00009R^003001NfP00009N
f@05001Jf`00G]X00004F]/30002F_/00`00E_aFo0020003E_`30002E_`00`00E_aFo0030002E_`0
0`00E_aFo0030002E_`00`00E_eFo@04000015Km001FoEKm10000goo00D007ooOomoo`0000moo`03
001oogoo01ioo`00=goo1P000UKm00<005KmE_d00`000UKm00<005;lD_`0100000@AF000000000=>
5`8000=VUP8000AV]P03001V]fJg009V]`03001R^6:h009R^003001R^6:h009R^003001NfEkI009N
f@06001NfUkJG]X005[K0P0000=Jf`00F_/00U[k0`0000=Fo000E_`00P0000=Fo000E_`00UKl0`00
0UKl00<005KlE_`00`000UKl00<005KlE_`00`000UKl00<005KmE_d00`000UKm0P000goo00D007oo
Oomoo`0000]oo`05001oogooOol0000POol003Eoo`@000=Fo@03001FoEKm00<0009Fo@03001FoEKm
00<00006B[d002Gk9O/0015H0`0000=>5`00000016JF00<006JfI[H00fJf00<006JgI[L00VJg00@0
06:hH[QR^08000=R^08000=Nf@8000=NfP8000AJf`<00004F_/005KlE_`20004E_`300000eKl001F
o002E_`300000eKl00000002E_`30002E_`00`00E_aFo0030002E_`00`00E_eFo@030003Ool3000=
Ool3000QOol0039oo`@000=Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009Bo@05001:_DZm000U
n`0300001@cf000004iH000016JF00<006JfI[H00VJf00<006Zf00000VJg00D006Jf001R^000009V
]`03001R^000009Rf003001Nf@00009Rf@05001NfP00G]X00003F]/20003F_/30002F_/00`00E_aF
o0020004E_`3000015Kl001Fo5Kl100000AFo000E_aFo0@0009Fo003001Fo5Kl00<0009Fo@8003Io
o`001goo1@0017oo0`0077oo10000UKm0P000UKm0`000UKm00<005KmE_d00`000UKm00<005;mD_d0
200000<<mP00CUP00TiH00H006JFIYH006Jf0002I[H01P00J[IZ]P00I[L000=V]P03001V]fJg009V
]`03001Rf6;H009Rf003001RfF;I009Rf@07001NfUkJG]X005[K00000UkJ00D005[k001Jn`0000=J
n`<0009Jo003001Fo5Kl008000AFo0<00004E_`005KlE_`30002E_`00`00E_aFo004000015Kl001F
oEKm0`00=Woo0008Ool01000Oomoo`000goo00D007ooOomoo`0001Qoo`<00004Ool000000003E_d3
0002E_d00`00E_eFo@030002E_d00`00E_eFo@0300002dkM000jOCYm000=F@000;H000cf00000TiH
00<006JF00000VJf00<006Jf000016Zf00<006JfI[H00VJf00<006JgI[L00VJg00@006;HH]QRf080
00=Rf@8000=NfP8000=NfP8000=Jn`05001Jn`00F_/00003F_`30002E_`00`00E_aFo002000015Kl
001Fo5Kl0P0000=Fo000E_`00UKl0`000UKl00<005KlE_`00`000UKl0P00=Goo0009Ool00`00Oomo
o`03Ool01@00Oomoogoo00005Woo1P000eKm00D005Km001Fo@0000=Fo@<0009Fo@03001FoEKm00<0
009Fo@04001>gDkM0002>Wd00`003ET000030002CUP20004I[H00`00J[IZ]P03J[H01`00I[IV]VJf
001Z]P00009V]`05001Ve`00H]P00002I]P00`00H]T00002H]T00`00G]X00002H]T01@00G]X005kJ
00000e[k0P000e[k00<005[l00000U[l00<005KlE_`00`000UKl0P0000=Fo00000000eKl00D005Kl
001Fo00000=Fo0<0009Fo003001Fo5Kl00<003Aoo`002Woo00<007ooOol00Woo00D007ooOomoo`00
01Aoo`D000=Fo@03001FoEKm008000=Fo@05001Fo@00E_d00003E_d30002E_d00`00E_eFo@070000
0`eI0000]P02000014iH001BF@0016Jf00<006ZfJ[H00VZf0P000VJf00H006ZfJ[H006Jg0003I]L0
0`00I]QVf002I]P00`00H]URf@02H]T01000H]URfF;I0P000ekJ00<005[k00000Ukk00<005[kF_/0
0P000e[l0`0000AJo000E_aFo0@0009Fo003001Fo5Kl008000=Fo005001Fo000E_`00003E_`30002
E_`00`00Oomoo`0bOol000]oo`05001oogooOol00003Ool00`00Oomoo`0?Ool50003E_d00`00E_eF
o@030002E_d00`00E_eFo@020003E_d01@00E_d005Km00000eKm0`0000EBo@00A[e6_@00008Z70@0
00033AL0059I009BF@06001V]VJf001V]`000VZf00<006ZfJ[H00P0016Zf00<006KGI]L00VKG00@0
06KHI]QVf08000=Rf@80009Rf@03001Rf@0000=NfP8000=Nn`<00003F_/005[l00800004F_/005[l
F_`30002E_`00`00E_aFo004000015Kl001Fo5Kl0`000eKl00D005Kl001Fo000009Fo0<003=oo`00
2goo00D007ooOomoo`0000=oo`03001oogoo00eoo`@000=Fo@03001FoEKm00<0009Fo@03001FoEKm
00<0009Fo@03001FoEKm008000=Fo@05001Fo@00E_d00003D_d01`00A[e6_@00:Q`0003g00800003
3AL0059I009BF@03001V]P00009V]`04001Z]VZf0002J[H00`00J[IZ]P03J[H01000I]MVefKG0P00
0VKH00H006KHI]P006;I0002I]P20003H]T300000ekJ001Nn`02000015kj001Jne[k0P0015[k0P00
00=Jo000E_`00UKl0`000UKl00<005KlE_`00`000UKl00<005KlE_`00`000UKl0`000UKl0P00<Woo
0007Ool01@00Oomoogoo00000goo00D007ooOomoo`0000aoo`@000=Fo@03001FoEKm00<0009Fo@03
001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001FoEKm008000=Fo@03001BoE;m00L000030?L0
00000080009BF@03001V]P00009V]`03001Z]P0000=Z]P03001Z]VZf009Z]P80009Ve`03001Zef[G
008000AVf005001Vf6KHI]P00004H]T00`00G]YNfP02G]X00`00G_YNnP02G_X20002G_/00`00F_]J
n`020003F_`01P00E_`00000F_`0009Fo0<0009Fo003001Fo5Kl00<0009Fo005001Fo000E_`00003
E_`2000bOol000Qoo`<000Eoo`<000Yoo`D0009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009F
o@03001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009>g@03000nW@00008E^`<0
00044CP0059IDUT20004I[L01`00J[IZ]VZf001ZeP00009Z]P03001ZeV[F008000=Ze`03001Vf6KH
009Vf080009Vf08000=Rf@8000=NfP8000=NnP8000=Nn`<0009Jn`03001Jo5[l00800003F_`005[l
00<0009Fo003001Fo5Kl00<0009Fo003001Fo5Kl008000=Fo004001Fo000E_`2000aOol001]oo`80
0004Ool000000003E_d30002E_d00`00E_eFo@030002E_d00`00E_eFo@030002E_d00`00E_eFo@03
0002E_d00`00E_eFo@030002D_d00`00C]e>g@0400000aFk0000000200001A4h0000059i000016Jg
00@006ZfJ[H0009ZeP03001Z]P0000=ZeP03001Zef[G009Ze`05001Vf6KHI]P00002I]L20002I]P0
0`00H]T00002H]T00`00G]X00002H]T00`00G_X00002G_X01@00G_/005kk00000e[k0`000U[k0`00
00=Jo000E_`00UKl0`0000AFo000E_aFo0@00004E_`005KlE_`30003E_`20007Ool30006Ool20005
Ool3000FOol001Qoo`<00003Ool0000000=Fo@03001Fo@00009Fo@03001FoEKm00<0009Fo@03001F
oEKm00<0009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001BoE;m00<00004B]d0039M
<Ud500000a4h001BN@02DWT02000I[MV]`00I[L006Zf0003J]H00`00J]IZeP02J]H02@00J]MZef[G
001Ze`00I]P00003I]L00`00I]QVf002I]P00`00H]URf@02H]T00`00H]URf@02H]T00`00G_YNnP02
G_X02000G_]Nnekk001Jn`00F_/60003F_`3000015[l001Fo5Kl0`000UKl00<005KlE_`00`000UKl
00<005KlE_`00`001Woo00D007ooOomoo`0000Eoo`8000Aoo`05001oogooOol0000EOol001Moo`D0
009Fo@03001FoEKm008000=Fo@<0009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001F
oEKm00<0009Fo@03001FoEKm00<0009Bo@05001:gD[M000bG@040000101d000000000U9i00<006Jg
00000fJg00<006[FJ]H00V[F00T006[FJ]IZeP00J]H006[G00000f[G00<006KGI]L00VKG00@006KH
I]QVf08000=Rf@8000=Rf@8000=NnP8000=Nn`L0009Jn`800003F_`0000000=Jo0<0009Fo003001F
o5Kl00<0009Fo003001Fo5Kl00<0009Fo08000Eoo`05001oogooOol0000?Ool00`00Oomoo`0COol0
01Moo`80009Fo@03001FoEKm00<0009Fo@05001Fo@00E_d00003E_d30002E_d00`00E_eFo@030002
E_d00`00E_eFo@030002E_d00`00E_eFo@030002E_d00`00D_eBo@090000101d000EF@000U9i0P00
16Jg00T006[FJ]IZeP00J]H006[F00000f[F00<006[GJ]L00V[G00T006KGI]MVe`00J]L006KH0000
0VKH00<006;I00000VKI00<006;I00000V;I00<005kj00000V;j1@0000AJn`00G_/000=Jn`<0009J
o005001Jo000F_`00003E_`3000015Kl001Fo5Kl100000AFo000E_aFo0@000Aoo`05001oogooOol0
000?Ool00`00Oomoo`0COol000moo`<000Eoo`80009Fo@<0009Fo@03001FoEKm008000=Fo@03001F
o@00009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001FoEKm00<0009Fo@03001FoEKm
00<00006C_d004:m@[d002HL1@000QEI0P0000=BNP00I[L00VJg00D006KG001ZeP00009ZeP03001Z
eP0000=ZeP09001Zef[GJ]L006[G001Ve`0000=Ze`03001Vf6KH009Vf003001VfFKI009Vf@04001R
fF;IH]T20002H_X30002G_X20003G_/00`00F_/00002F_/200000e[l00000003F_`01@00E_`005Kl
00000UKl0`000UKl00<005KlE_`00`000UKl0P0017oo00D007ooOomoo`0000eoo`8001Ioo`003Woo
00D007ooOomoo`0000Aoo`04001oo`000002E_d00`00E_eFo@030002E_d00`00E_eFo@020003E_d3
0002E_d00`00E_eFo@030002E_d00`00E_eFo@030002E_d00`00E_eFo@03000015;m001>oDkm1P00
00<0n00000000P0000<EF@00DWX00U9j00<006Jg00000VKG00<006[F00000f[F00T006[FJ]IZeP00
J]H006[G00000f[G00<006[GJ]L00V[G00@006KHI]QVf08000=Vf@800004H]T00000H_T30002H_X0
1@00G_X005kj00000ekk0P000e[k0`0000AJn`00F_aJo0<0009Fo0<0009Fo003001Fo5Kl00<0009F
o003001Fo5Kl00<000=oo`05001oogooOol0000?Ool00`00Oomoo`0COol000ioo`05001oogooOol0
0003Ool01@00Oomoogoo00000UKm0`000UKm00<005KmE_d00`000UKm00<005Km00000UKm00<005Km
E_d00`000UKm00<005KmE_d00`000UKm00<005KmE_d00`000UKm00<005;m00000P0000A>g@00>Wdj
O@<000030?P0000000@00004DWX000000004I]L00`00J]IZeP02J]H01000J]IZeP000V[F00<006[G
00000f[G00T006[GJ]MZe`00J]L006KH00000VKH00<006KI00000VKH100000=Rn@00H_X00f;j00X0
05kjG_YNnP00G_/005kk001Jn`800004G_/00000F_/30002F_`01@00E_`005[l00000eKl0`000UKl
00<005KlE_`00`0000AFo0000000009oo`05001oogooOol0000;Ool01@00Oomoogoo00005Goo000>
Ool01@00Oomoogoo00000goo0P000goo0P000UKm00<005KmE_d00`000UKm00<005KmE_d00P000eKm
0`000UKm00<005KmE_d00`000UKm00<005KmE_d00`000UKm00<005KmE_d00P000U;m00@004kMC]d0
008jO@03000E^`0000<0008IN@80009BNP03001VefKG009Ve`09001ZeV[FJ]H006[G001ZeP0000=Z
eP09001Zef[GJ]L006[G001Ze`0000=Ze`03001Vf6KH009Vf003001Vf000008000=Rn@8000=RnP80
00=NnP80009Nn`8000ANn`80009Jn`03001Jo5[l008000=Jo005001Fo000E_`00002E_`4000015Kl
001Fo5Kl0`000goo0`003Goo0`005Woo000>Ool01@00Oomoogoo00000goo00<007ooOol017oo00<0
05Km00000P000UKm00<005KmE_d00`000UKm00<005Km00000UKm00<005KmE_d00`000UKm00<005Km
E_d00`000UKm00<005KmE_d00`0000AFo@00D_eBo@d000036GT0059j00=BNP08001VefKG001Vf000
J]H000=Ze`03001ZeV[F009ZeP04001Zef[G0002J]L00`00J]L00003J]L01000I]QVf6KH0`0000IV
f000I]P006;i0002I_T00`00H_X00002H_T01`00G_X006;j001Nn`00009NnP03001Nnekk00800003
F_/005[k00<00004F_`005[lF_`30002E_`30002E_`00`00E_aFo0030002E_`2000[Ool000ioo`05
001oogooOol00003Ool00`00Oomoo`05Ool01000E_eFo@000UKm0`000UKm00<005KmE_d00P000eKm
0`000UKm00<005KmE_d00`000UKm00<005KmE_d00`0000AFo@00E_eFo@<0009Bo@07001:gD[M000b
G@002GX00`0000@0M@00000000=BNP03001Ve`00009Vf003001ZeP0000=Ze`09001ZeV[FJ]H006[G
001Ze`0000=Ze`04001Zef[GJ]L200001FKH00000000J]P00P000fKH00<006KiI_T00VKi00@006;i
H_URn@80009RnP8000=NnP<0009Nn`<00004F_/005[lF_`30002F_`00`00E_`00002F_`00`00E_aF
o003000015Kl001Fo5Kl1000:Woo000>Ool01@00Oomoogoo00000goo0P001goo00<005Km00000P00
0UKm00<005KmE_d00`000UKm00<005Km00000UKm00<005KmE_d00`000UKm00<005KmE_d00`000UKm
00<005Km00000P000U;m00<005;mD_d01`0000<9NP0000000P0000@IVP00DWYBNP<000=Vf08000=Z
e`05001Ze`00J]H00003J]L00`00J]MZe`02J]L00`00J]MZe`050003J]P01000I]QVf6KH0P000fKi
0P0000MRn@00001Rn@00H_X00002H_X01@00G_X005kj00000ekk0`0000AJn`000000009Jn`03001J
o5[l008000=Jo0<00004E_`005KlE_`30002E_`2000ZOol000moo`<000Aoo`03001oogoo00Moo`06
001FoEKmE_d005Km0`000UKm00<005KmE_d00`000UKm0`000UKm00<005KmE_d00`000UKm00<005Km
E_d00P000UKm00<005;mD_d00`000TkM00<004:m00000RXm1P0000<IVP0000000U:J00H006KHI]P0
06KH0002J]L01P00J]MZe`00J]H000=Ze`06001Zef[GJ]L006[G0`000V[G00<006[HJ]P00V[H0P00
00MVf000I]QVf000I_T00002I_T00`00H_URn@02H_T00`00H_YRnP02H_X01000G_YNnUkj0`0000IN
n`00F_/005kk0003F_/300000e[l001Jo002000015[l001Fo5Kl0`000UKl00<005KlE_`00`00:Goo
000FOol00`00Oomoo`08Ool00`00E_eFo@020002E_d00`00E_eFo@030002E_d30002E_d00`00E_eF
o@030002E_d00`00E_eFo@03000015Km001FoEKm0`000U;m00<004kMC]d0100000<Z?@000CT01000
00<IVP00DYX00e:J0P000fKH00<006[G00000f[G00P006[GJ]MZe`00J]L006[G0`0000=Ze`00J]L0
0V[G00D006[HJ]QZf000009Zf003001Vf6KH009Vf005001VnFKiI_T00003H_T20003H_X20002G_X2
0003G_/30002G_/01@00F_/005[k00000U[l0P0015[l0P000U[l00<005KlE_`00`0000AFo0000000
02Qoo`005Goo00<007ooOol02Woo0P000UKm00<005Km00000P000UKm00<005Km00000UKm00<005Km
E_d00`000UKm00<005KmE_d00`000UKm00<005Km00000P000UKm00<005;mD_d00`0000A>g@00?Ydn
W@<000030CT0000000@0009BVP8000EVf004001Zef[GJ]L20002J]L00`00J]L000020004J]L02000
J]MZe`00J]L006[H0003J]P01000I]QVf6KH0P000VKi00D006Ki001Rn@00009Rn@03001RnP00009R
nP80009RnP03001Nn`00009Nn`800003G_/00000009Jn`<0009Jo003001Jo5[l008000=Jo0<00004
E_`005KlE_`3000XOol001Eoo`03001oogoo00]oo`03001FoEKm008000=Fo@03001Fo@000080009F
o@<0009Fo@03001FoEKm00<0009Fo@03001FoEKm0080009Fo@03001FoEKm00<0009Bo@04001>gDkM
0002?Yd00`007O`0000400027KX200000e:J001Vf003I]P20002J]L00`00J]MZe`040002J]L01P00
J]MZef[G001Ze`8000=Ze`09001Zf6[HJ]P006[G001Vf000009Vn08000=Vn@03001RnF;i009Rn@03
001RnV;j008000=RnP8000=Nn`<00004G_/005[k0002F_/00`00F_aJo00300000e[l001Jo0020000
15[l001Fo5Kl0`000UKl0P009goo000EOol2000=Ool20002E_d00`00E_eFo@020003E_d20002E_d0
0`00E_eFo@030002E_d00`00E_eFo@03000015Km001FoEKm0`000UKm00<005;mD_d03P0000<M^P00
DYX00U:J00@006KHI]P0009Vf0800003J]L00000008000=Ze`80009Ze`04001Zef[G0004J]L01000
J]QZf0000f[G00D006KhI_QVn00000=Vn@8000=Rn@80000:H_X006;jH_X006;j001RnP00G_/20000
0ekj001Nn`030003F_/3000015[k001Jo5[l0P000e[l0`0000AFo000E_aFo0@002Ioo`003Goo1000
17oo00<007ooOol03Goo00<005KmE_d00`000UKm00D005Km001Fo@00009Fo@<0009Fo@03001FoEKm
00<0009Fo@03001Fo@000080009Fo@03001FoEKm00<0009Bo@04001:gD[M0002=Wd00`005K/00006
0002DYX00`00I]P00003I]P30003J]L01000J]MZe`000V[G0P000f[G00T006[GJ]MZe`00J]L006[H
00000V[G0P000VKh00D006[h001Vn@00009Vn003001Rn@00009Vn@03001RnV;j009RnP05001RnV;j
H_X00004G_X20002G_/01@00F_/005kk00000U[k0`000U[l00D005[l001Jo000009Fo0<0009Fo080
00Moo`<000Ioo`8000Ioo`<000]oo`004Goo00D007ooOomoo`00011oo`80009Fo@03001FoEKm00<0
0004E_d000000002E_d00`00E_eFo@030002E_d00`00E_eFo@020002E_d00`00E_eFo@030002D_d0
0`00D_eBo@0=0000126j001BVU:J0P000VKH0`0016[G00@006[G000000=Ze`09001Zef[GJ]L006[G
001Ze`0000=Ze`04001Zef[G0002J_L20003J_P00`00I_QVn002I_P00`00I_UVn@020003H_X20002
H_X20003G_X200000ekk001Nn`020003G_/20002F_/00`00F_aJo0020002F_`30002F_`00`00E_aF
o0030005Ool01@00Oomoogoo00001Goo0P001goo00<007ooOol02Woo000AOol01@00Oomoogoo0000
4Goo00<005KmE_d00`000UKm0P000UKm00<005KmE_d00`000UKm00<005KmE_d00`0000AFo@00E_eF
o@<0009Fo@03001BoE;m00<0009>o@050016_DJm000bG@070000126j000005:J0`0000AVf000I]P0
009Ze`04001Zef[G0004J]L01000J]MZe`000f[G00<006[GJ]L00V[G00<006[G00000f[g00<006[h
J_P00V[h00@006KhI_QVn0800007I_T006KiI_T006;j00000V;i0P000V;j00H005kj001NnP00G_/2
000015kk001Nnekk0P000e[k0`0000AJo000F_`0009Jo003001Jo5[l00<00004E_`000000004Ool0
1@00Oomoogoo00002goo1@002goo000AOol01000Oomoo`004goo0P000UKm0`000UKm00<005Km0000
0P000UKm00<005KmE_d00`000UKm00<005Km00000P000UKm00<005KmE_d00`000U;m00<004kmC_d0
200000D0]P000000026j00<00004DYX006KH0002I]P00`00J]L00003J]L02@00J]MZef[G001Ze`00
J]L00003J]L01`00J]MZef[G001Zm`0000AZm`09001Zn6[hJ_P006[h001Vn000009Vn003001VnFKi
009Vn@03001RnF;i008000=RnP80009NnP80009Nn`@00003G_/005[k00800004F_/005[lF_`20003
F_`3000015[l001Fo5Kl0`0017oo00D007ooOomoo`0000]oo`04001oogoo000<Ool000eoo`@000=o
o`8001Aoo`80009Fo@03001FoEKm0080009Fo@03001FoEKm00<0009Fo@03001FoEKm0080009Fo@03
001FoEKm00<0009Fo@03001BoE;m00<00006C]d004:m@[d002Xm2@000U:J0P0016KH00T006[GJ]MZ
e`00J]L006[G00000f[G00T006[GJ]MZe`00J_L006[G00000V[g00T006[gJ_MZm`00J_L006[h0000
0f[h00D006KhI_QVn00000=Vn@80000;H_T006;iH_T006;j001RnP00G_X00002H_X01@00G_/005kk
00000Ukk0P000e[k0`0000=Jn`00F_`00P0000AJo000F_aJo0<00004E_`007oo0003Ool01@00Oomo
ogoo000037oo00<007oo000037oo000=Ool00`00Oomoo`04Ool00`00Oomoo`0DOol00`00E_eFo@03
000015Km001FoEKm0`000UKm00<005KmE_d00`000UKm00<005Km00000P000UKm00<005KmE_d00`00
0U;m00<004kM00002`000R7K0P0000=BVP00I]P00VKH0P000V[G00H006[GJ]L006[G0003J]L01000
J]MZe`000f[g00P006[gJ_L006[g001Zm`000f[g00@006[hJ_QZn080009Vn005001Vn000I_T00002
I_T00`00H_URn@02H_T00`00H_YRnP020003H_X200000ekk000000020002G_/01@00F_/005kk0000
0U[k0P000e[l0`0000AJo000E_aFo0<000=oo`05001oogooOol0000<Ool00`00Ool0000<Ool000eo
o`03001oogoo00Aoo`03001oogoo01Eoo`80009Fo@03001Fo@000080009Fo@03001FoEKm00<0009F
o@03001FoEKm0080009Fo@03001FoEKm00<0009Fo@03001BoE;m0080009:g@@000038Q`0000000@0
00038M/005:J009BVP03001Vf000009Vf003001Ze`0000=Ze`09001Zef[GJ]L006[g001Ze`0000=Z
m`03001Zm`0000=Zm`03001Zmf[g009Zm`04001Zn6[h0002J_P20003I_P00`00I_UVn@020003H_T2
00002V;j001RnV;j001RnP00H_X005kk0`000ekk0P000Ukk0`000U[k00D005[l001Jo000009Jo0<0
009Jo080009oo`05001oogooOol0000=Ool2000<Ool000eoo`@000=oo`8001Moo`03001FoEKm0080
009Fo@03001FoEKm00<0009Fo@03001FoEKm00<00004E_d005KmE_d30002E_d00`00E_eFo@030000
0e;m0000000500000b8L000160030000102F000000000U:J0P0016KH00<006[GJ]L00V[G00@006[G
J]L000=Zm`06001Zmf[gJ_L006kg0P000f[g00H006[gJ_MZm`00J_L20003J_P01000I_QVn6Kh0P00
00MVn@00I_UVn@00H_T00002I_T00`00H_YRnP02H_X01@00H_YRnV;j00000Ukj0P0000INn`00G_/0
05kk0002G_/00`00F_]Jn`020002F_`3000015[l001Jo5[l10000Woo0`003goo00<007ooOol02Woo
000DOol00`00Oomoo`0GOol20002E_d00`00E_d000020002E_d00`00E_eFo@030002E_d00`00E_d0
00020002E_d00`00E_eFo@030002D_d50002B]d00`00>Yd0000200000`4H0000000200029M/20000
0e:K001Vf002I]P20003J]L01@00J]L006[G00000f[g00@006[gJ_L0009^m`09001Zmf[gJ_L006[g
001Zm`00009Zm`09001Zn6[hJ_P006[h001Vn000009Zn003001VnFKi009Vn@03001VnFKi008000=R
nP80009RnP80009NnP<0009Nn`8000=Nn`<00006F_/005[l001Jn`000U[l0`000U[l0P008Goo000C
Ool2000JOol00`00E_eFo@020002E_d00`00E_eFo@030002E_d00`00E_eFo@020002E_d00`00E_eF
o@030002E_d50002D_d00`00B]e:g@0400000aWL0000000400000bGK001BV`02DY/00`00I]P00002
I]P00`00J]L00003J]L01P00J_MZmf[g001Zm`8000=^m`04001Zmf[g0003J_L01000J_MZmf[g0P00
00=Zn000J_P00V[h00D006[hJ_QZn00000=Vn@800007I_T006KiI_T006;j00000V;i0P000V;j00<0
05kj00000P0000ENn`00G_/005kk00800004G_/005[kF_/20002F_/30002F_`00`00F_aJo003000P
Ool001=oo`03001oogoo01Yoo`80009Fo@03001Fo@000080009Fo@03001FoEKm00<00003E_d00000
00@0009Fo@D0009Bo@H00096_@03000bO@0000T0009BV`8000AVf003001Zef[G009Ze`04001Zmf[g
0002J_L02@00K_M^mfkg001^m`00J_L00003J_L01000J_MZm`000V[g00<006[hJ_P00V[h00P006[h
J_P006[h001Vn@000VKi00<006KiI_T00VKi00<006;iH_T00P000f;j0P000V;j0P000Ukk0P000ekk
0`0000ANn`00F_/0009Jn`03001Jo5[l00<00004F_`007oo000OOol001=oo`03001oogoo01]oo`03
001FoEKm0080009Fo@03001FoEKm00<0009Fo@80009Fo@03001FoEKm00@00003E_d005Km00<0009B
o@04001>oDkm0002A[d400000a6k0000000200029O/300000e:k001Vf002I]P20003J]L01@00J_L0
06[g00000V[g00@006kgK_L000=^m`06001Zmf[gJ_L006[g0P000f[g00H006[hJ_QZn000J_P20003
J_P00`00I_UVn@020003I_T200002V;i001RnF;i001RnP00H_X006;j0`0000UNn`00G_]Nn`00G_/0
05kk00000Ukk0P000e[k0`0000AJo000F_aJo0<001moo`004goo0P007Goo0P0000AFo@00E_eFo@<0
009Fo@<00004E_d005KmE_d4000015Km001FoEKm0P000UKm00<005;mD_d01P0000A2_@00;Ud^G@H0
008Un`07001B^e:kD[/006KH00000V;I00<006[G00000f[g00P006[gJ_L006[g001^m`000fkg00@0
06[gJ_L0009Zm`09001Zmf[gJ_L006[g001Zn000009Zn004001Zn6[hJ_P200001fKi001Vn6Kh001V
n@00009Vn@04001RnF;iH_T20002H_X20002H_X20003G_/20002G_/30002G_/01@00F_/005[k0000
0U[l0`0000AJo000Ool001ioo`000goo1@000goo0`001Goo00<007ooOol07Goo00<005Km00000P00
0UKm0`000UKm00<005Km00000`0000AFo@00E_eFo@<00004E_d005KmE_d30002D_d01@00C]e>g@00
@[d03@0000AB^`00000000=Rf@8000=Zm`800003J_L006[g009Zm`09001^mfkgK_L006kg001Zm`00
009Zm`04001Zmf[g0003J_L01000J_QZn6[h0P0000EZn000J_QZn00000AVn00A001VnFKi001Vn@00
H_T006Ki001RnP00H_X006;j001RnP02000025kk001NnP00G_/005kk0003G_/20002F_/3000015[l
001Jo5[l0P0000=oo`00Ool077oo0005Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007oo
Ool07Woo0P0000AFo@000000009Fo@03001FoEKm00<00004E_d005KmE_d30002E_d00`00E_d00002
0002E_d00`00D_eBo@05000013jm000V?BHm0`0000<04P0000000RWk0P000U:k00H006;IH]T006;I
0002J_L00`00J_MZm`020003J_L01000K_M^m`000fkg00P006[gJ_L006kg001Zm`000f[g00@006[h
J_P0009Zn005001Zn6[hJ_P00003I_P200000fKi001Vn@02I_T01@00I_UVnFKi00000V;j0P000f;j
0P000Ukj0P000ekk0`0000INn`00F_/005[k0002F_`3000015[l000000001goo0`001Woo0P001Goo
10000Woo0005Ool00`00Oomoo`02Ool01@00Oomoogoo000017oo00<007ooOol087oo00@005KmE_d0
009Fo@<0009Fo@03001Fo@000080009Fo@03001FoEKm0080009Fo@03001FoEKm00<0009Bo@03001>
g@0000<000039Sd0005J00D00003:O/005:k00=B^`8000=Rf@8000=Zm`09001Zmf[gJ_L006[g001^
m`0000=^m`03001Zm`0000=^m`06001Zmf[gJ_L006[g0P000f[h00P006[hJ_P006[h001Vn0000VKh
00@006KiI_UVn@80009Vn@800006H_X006;jH_X006;j0`0000ANnP00H_X0009Nn`8000=Nn`80009J
n`<00004F_/005[lF_`40005Ool01@00Oomoogoo00001Goo0P002Goo0@000Goo0005Ool00`00Oomo
o`02Ool01@00Oomoogoo00000goo00<007ooOol08Woo00<005Km00000`0000AFo@00E_eFo@80009F
o@03001FoEKm00<00004E_d005KmE_d30002D_d00`00D_eBo@020002B]d00`00?Yd0000200000`5J
000000050002D[/20004H]T00`00J_MZm`02J_L01000J_MZm`000f[g00H006kgK_M^m`00K_L20003
K_L01000J_MZm`000V[g00H006[hJ_QZn000J_L20003J_P02000I_QVn000J_P006Ki0002I_T20002
I_T00`00H_YRnP02H_X00`00H_YRnP020002H_X30002G_/02@00G_/005kk001Jn`00G_/00002F_/3
0002F_`30004Ool01@00Oomoogoo000047oo0@000Goo0005Ool00`00Oomoo`02Ool01@00Oomoogoo
00000goo0P008goo0`0000AFo@00E_eFo@<00004E_d005KmE_d30002E_d00`00E_d000020002E_d6
000015;m001:gD[M100000<R7@00000010000RWk0P0000=B^`00H]T00V;I0P000f[g00D006[h001Z
m`0000=Zm`04001^mfkg0002K_L02@00K_M^mfkg001^m`00J_L00002J_L200001F[h001Zmf[g0000
16[h00<006Kh00000f[h00D006KiI_UVn@00009Vn@8000=RnP800006H_X006;j001RnP000V;j00<0
05kkG_/00P000Ukk0P000Ukk0`000U[k00@005[l001Jo08000Aoo`05001oogooOol0000@Ool10001
Ool000Eoo`03001oogoo009oo`05001oogooOol00003Ool00`00Oomoo`0OOol3000017oo001FoEKm
0`000UKm00<005Km00000P000UKm00<005KmE_d00P000UKm0`000UKm00<005;m00001@000T[M00<0
03ZM00001`0000<Yn`00D[/00U:k00<006;I00000V;I00<006[g00000f[h00T006[gJ_MZm`00J_L0
06kg00000Vkg0P0000=^m`00K_L00Vkg00@006[gJ_L0009Zm`05001Zmf[gJ_L00003J_P00`00J_P0
0004J_P00`00I_UVn@0200001fKi001VnFKi001RnP00009RnP05001RnV;jH_X00003H_X300002Ukk
001Nn`00G_/005kk001Nn`000U[k0`000U[l0`000goo00D007ooOomoo`0000aoo`@0009oo`001Goo
00<007ooOol00Woo00D007ooOomoo`0000=oo`03001oogoo01aoo`<000Eoo`80009Fo@03001FoEKm
0080009Fo@H00004E_d000000002E_d00`00E_eFo@020002D_d00`00C_e>o@0600026O`300000`0B
000000020002D[/20004H]T00`00J_QZn002J_P200000f[g001Zm`02J_L01000K_M^m`000Vkg00<0
06kgK_L00Vkg00<006[g00000f[g00P006[gJ_L006[g001Zn0000V[h00H006[hJ_QZn000J_P20002
I_T01000I_UVnFKi0P000V;j0P000V;j0`0000ARnP00H_X0009Nn`80009Nn`80009Nn`<0009Jn`04
001Jo000F_/30002Ool01@00Oomoogoo000037oo00<007ooOol00goo0003Ool30005Ool30004Ool0
0`00Oomoo`0IOol30009Ool00`00E_eFo@03000015Km000000000UKm00<005KmE_d00P000UKm00<0
05KmE_d00`0000AFo@00D_eBo@<0009>o@040016g@00<Wd80002:Q`200000e:k001Rf@02H]T20003
J_P01000J_QZn0000f[g0P0000=^m`00K_L00Vkg00L006kgK_M^m`00K_L00003J_L01000J_L00000
0f[g00P006[hJ_P006[h001Zn0000V[h00@006KiI_UVn@800006I_T006Ki001RnP000VKi0P000V;j
0P000f;j0P0000QNn`00G_/005kk001Nn`000Ukk00<005[kF_/00P000U[k0`000Woo00D007ooOomo
o`0000aoo`03001oogoo00=oo`004Woo00<007ooOol05Woo0`003Goo0P000UKm0P000UKm00<005Km
E_d00`0000AFo@00E_eFo@<0009Fo@03001Fo@000080009Bo@03001>oDkm00H000034M`0000000@0
0003:Q`005:k009B^`03001Rf@00009Rf@03001Zn00000=Zn004001Zmf[g0002J_L02@00K_M^mfkg
001^m`00K_L00002K_L01000J_MZm`000V[g00<006[gJ_L00V[g00<006[h00000f[h00P006[hJ_P0
06[h001Vn@000VKi00D006KiI_UVn@0000=Vn@04001RnV;jH_X20002H_X20002G_/20003G_/01000
G_]Nnekk0`0000ENn`00F_/005[k008000=oo`<000eoo`@0009oo`004Goo00<007ooOol057oo0`00
4Woo0P000UKm00<005Km00000P000UKm00<005Km00000P000UKm00<005KmE_d00P000UKm00<005;m
D_d00`0000I>g@00@[e2_@00;Ud:0002D[/20003H]T20003J_P01000J_P000000f[g00@006kgK_L0
00=^m`08001^mfkg001^m`00J_L0009Zm`07001Zmf[gJ_L006[g000016[h00<006[h00000f[h00D0
06KiI_UVn@00009Vn@09001Vn@00I_T006Ki001RnP00009RnP80009RnP80009RnP09001Nn`00G_/0
05kk001Nn`00009Nn`80009Jn`8001Uoo`004Goo0P004goo0P005Woo00<005KmE_d00P000UKm00<0
05KmE_d00P000UKm00<005KmE_d00`0000=Fo@000000100000ABo@00C]e>g@L000063K/000000000
n0000RhL0P000U:k00<006;I00000V;I00<006[h00000V[h00T006[gJ_MZm`00J_L006kg00000fkg
00<006kg00000fkg00P006[gJ_L006kg001Zm`000V[g00L006[hJ_QZn000J_P00004J_P01@00I_UV
n@00I_T00P000fKi00<006KiI_T00P000V;j0P000V;j0P000f;j0P000Ukk0P000Ukk0`0000INn`00
000007oo000IOol0015oo`03001oogoo00moo`<001Uoo`800004E_d005KmE_d3000015Km001FoEKm
0`000UKm0P000UKm00<005KmE_d01@000TkM00D004:m@[d002YM00D000030?P000000080009B^`80
00=Rf@8000=Zn004001Zmf[g0003J_L01P00K_M^mfkg001^m`8000=^m`03001Zm`0000=^m`08001Z
mf[g001Zm`00J_P0009Zn006001Zn6[hJ_P006[h0P000VKi00@006KiI_UVn@800007I_T006Ki001R
nP00H_X00`000V;j00l006;j001RnP00G_/005kk001Nn`00G_/005kk00001Goo00<007ooOol05Woo
000AOol2000=Ool3000MOol00`00E_d000020002E_d50002E_d3000015Km001FoEKm0`0000ABo@00
D_eBo@T00085V`@0008^70<00007D[/006;IH]T006;I00000V[h00D006[h001Zm`0000=Zm`04001^
mfkg0002K_L01P00K_M^mfkg001^m`8000=^m`03001Zm`0000=Zm`08001Zn6[h001Zn000J_P0009Z
n008001VnFKi001Vn000I_T0009Vn@03001VnFKi0080009RnP<0009RnP80009RnP80009Nn`80009N
n`8000Ioo`03001oogoo01Ioo`0000=oo`0000000`000goo100017oo00<007ooOol02Goo0`008Goo
00<005KmE_d00`0000AFo@00E_eFo@<0009Fo@03001Fo@000080009Fo@03001Bo@000080009Bo@03
001:g@00008n_@03000V?@0000<0000300l00000008^7004001B^e:kD[/20003H]T20003J_P01P00
J_MZmf[g001Zm`8000=^m`04001^mfkg0002K_L01`00K_M^mfkg001^m`0000AZm`03001Zn00000=Z
n003001Zn6[h00800003I_T006Kh009Vn003001VnFKi00800006I_T006Ki001RnP000V;j00l006;j
001RnP00H_X006;j001Nn`00G_/005kk000027oo00<007ooOol05Woo0003Ool00`00Oomoo`07Ool0
1@00Oomoogoo000027oo0`009Goo0P000UKm00<005Km00000P000UKm00<005KmE_d00P000UKm00<0
05KmE_d00P000U;m00<005;mD_d04P0000=B^`00000016;I00<006[hJ_P00V[h00@006[gJ_L0009Z
m`09001^mfkgK_L006kg001^m`00009^m`800005K_L006kgK_L00003J_L00`00J_L00003J_P20000
1F[h001Zn6[h00000fKh0P0000EVn@00I_UVn@0000=Vn@04001RnV;jH_X20002H_X20002H_X20002
G_/20009Ool00`00Oomoo`0FOol000=oo`03001oogoo00Moo`03001oogoo008000Eoo`<002Uoo`03
001FoEKm0080009Fo@03001FoEKm00<00004E_d005KmE_d300000eKm0000000400001E;m001:gD[M
00000SZM00<0028M00001@000RhL0P0000=B^`00H]T00V;I0P000f[h00D006[h001Zm`00009Zm`04
001^mfkg0003K_L01000K_M^m`000Vkg00P006kgK_L006kg001Zm`000V[g0P0000EZn000J_QZn000
009Zn005001Zn000I_P00002J_P00`00I_UVn@020003I_T200000f;j001VnP03000026;j001RnP00
H_X005kk000;Ool00`00Oomoo`0FOol000=oo`03001oogoo00Moo`04001oogoo0003Ool3000]Ool2
0002E_d00`00E_d000020002E_d50002E_d20002E_d00`00D_eBo@0A00000bhL001B^`02D[/00`00
H]T00002H]T00`00J_P00003J_P02000J_MZm`00J_L006kg0003K_L00`00K_L00003K_L00`00K_L0
0003K_L01000J_MZm`000V[g00L006[hJ_QZn000J_P00003J_P00`00J_QZn00200001fKi001VnFKi
001Vn@00009Vn@03001VnVKj00<00003H_X00000009RnP8000aoo`03001oogoo01Ioo`000goo00<0
07ooOol00goo10000goo1000<Woo10000UKm0`0000AFo@00E_eFo@<00004E_d005KmE_d3000015;m
001>oDkm1P000QWl0`0000<0<`0000000P000U:k0P000f;I0P000f[h0P0000=Zm`00J_L00V[g00L0
06kgK_M^m`00K_L00003K_L01000K_L000000fkg00<006[g00000f[g00H006[hJ_P006[h0004J_P0
0`00J_P00002J_P00`00I_UVn@02I_T00`00I_UVn@0200000fKj001Vn@03000016;j001RnP003Woo
00<007ooOol05Woo0003Ool00`00Oomoo`03Ool00`00Oomoo`04Ool2000eOol00`00E_d000020002
E_d00`00E_d000020002E_d00`00E_d000020002E_d00`00D_d000020002C_d01000A]d0039m1`00
00<16000;S`00P000U:k00<006;I00000V;i00<006[h00000V[h00<006[gJ_L00V[g0P0000E^m`00
K_M^m`00009^m`04001^mfkg0003K_L00`00K_L00003J_L01000J_L000000V[h00L006[hJ_QZn000
J_P00003J_P01000I_UVnFKi0P0000EVn@00I_UVn@00009Vn@<00003H_X00000011oo`03001oogoo
01Eoo`000goo00<007ooOol00goo00<007ooOol01Woo00<007ooOol0<goo00@005KmE_d0009Fo@80
009Fo@03001FoEKm0080009Fo@03001FoEKm0080009Bo@03001>oDkm00H000035M`0000000<00004
0AP000000002D[/20004H_T01@00J_QZn6[h00000f[g00@006[gJ_L0009^m`05001^m`00K_L00002
K_L200001Fkg001^mfkg00000V[g00@006[gJ_L0009Zn005001Zn000J_P00002J_P02000J_QZn000
J_P006Ki0002I_T00`00I_UVn@0200001VKi001Vn@00H_X0019oo`03001oogoo01Eoo`0000Aoo`00
000000Eoo`@000Ioo`03001oogoo039oo`03001FoEKm00<00003E_d0000000@00003E_d0000000@0
0003E_d0000000@00006C]d004:m@[d002im20000S8l0P0000=B^`00H_T00V;i0P000V[h00D006[h
001Zm`0000=Zm`03001^m`0000=^m`04001^mfkg0002K_L02000K_M^m`00K_L006[g0002J_L20000
1F[h001Zn6[h00000V[h00@006[h000000=Zn003001VnFKi00800005I_T006KiI_T00003I_T00`00
Ool0000AOol00`00Oomoo`0EOol001Aoo`03001oogoo039oo`80009Fo@80009Fo@03001FoEKm0080
009Fo@03001FoEKm0080009Bo@03001Bo@0000T000043M`0000000l300000c8l001B^`02D[/00`00
H_T00002H_T20003J_P01P00J_MZmf[g001Zm`8000=^m`03001^m`0000=^m`03001^m`0000=^m`04
001Zmf[g0002K_L01`00J_QZn6[h001Zn000009Zn004001Zn6[hJ_P200001FKi001VnFKi00000VKi
00D006Ki001Vn@00009oo`03001oogoo00moo`03001oogoo01Eoo`005Goo0P00=7oo0P0000AFo@00
E_eFo@<00004E_d005KmE_d3000015Km001BoE;m0P000U;m00<004kMC]d00`000RiM2P0000AB^`00
0000009Rn@8000=Zn005001Zn000J_L00002J_L01`00K_M^mfkg001^m`0000=^m`04001^m`000003
K_L00`00J_L00003K_L01P00J_QZn000J_P000=Zn0800005J_P006[hJ_P00002I_T200000fKi001V
n@02I_T00`00Oomoo`02Ool00`00Oomoo`0>Ool00`00Oomoo`0EOol001Moo`03001oogoo039oo`03
001Fo@000080009Fo@03001Fo@0000800003E_d0000000@00003D_d0000000@00003B]d003jn00P0
00030CT0038l0080009B^`03001Rn@00009Rn@03001Zn00000=Zn004001Zmf[gJ_L200001Fkg001^
mfkg00000Vkg00@006kgK_L000=^m`03001^m`0000=^m`04001^m`000002J_P200001F[h001Zn6[h
00000V[h0P0000EVn@00J_UZn@0000=Vn@03001oogoo00Aoo`03001oogoo00eoo`03001oogoo01Eo
o`0067oo00<007ooOol0<Woo00<005KmE_d01@000UKm0P000UKm00<005KmE_d00P000U;m00<005;m
D_d03@0000@1>@000000009B^`8000ARn@03001Zn6[h009Zn004001Zmf[g0002J_L02000K_M^m`00
K_L006kg0002K_L2000016kg001^mfkg0P0000E^m`00K_M^m`00009Zn004001Zn6[h0003J_P00`00
J_P00002J_P01000J_UZnF[i0P0000=Vn@00Ool01goo00<007ooOol037oo00<007ooOol05Goo000I
Ool00`00Oomoo`0bOol20002E_d00`00E_d00002000015Km001FoEKm0`0000AFo@00D_eBo@<00007
D_d004[MB]d003jN00000R8m1P000S8l0P0000=B^`00H_T00V;i0P000f[h00@006[h000000=Zm`03
001^m`0000=^m`04001^mfkg0002K_L01000K_M^m`000Vkg00L006kgK_M^m`00J_P00003J_P01P00
J_QZn000J_P000=Zn0800005J_T006[iJ_T00009Ool00`00Oomoo`0=Ool00`00Oomoo`0DOol001Yo
o`8003=oo`03001FoEKm0080009Fo@03001Fo@000080009Fo@03001Fo@0000800004D_d005;mD_d=
0000104i000b?0000e:k00<006;i00000V;j00<006[h00000V[h00H006[gJ_MZm`00J_P20003K_L2
00001fkg001^mfkg001^m`0000=^m`06001^mfkg001^m`000f[h00@006[h0000009Zn0800005J_P0
06[hJ_P00002J_T00`00Oomoo`09Ool00`00Oomoo`0<Ool00`00Oomoo`0DOol001aoo`03001oogoo
035oo`800004E_d005KmE_d20002E_d50002E_d50002C_d00`00B]d0000300000ahM000000070002
D[/20003H_X20002J_P01@00J_P006[g00000V[h00D006kgK_M^m`00009^m`06001^mfkg001^m`00
0fkg00@006kg0000009^m`800005J_P006[hJ_P00002J_P01000J_QZn0000f[h00<006[i00003Goo
00<007ooOol02goo00<007ooOol057oo000MOol00`00Oomoo`0bOol4000015Km000000000UKm00<0
05Km00000P0000AFo@00D_eBo@D00096g@03000fWP0000D000040ET0038l<S`20002D[/01`00H_X0
06;j001Zn00000=Zn004001Zn6[hJ_P200000fkg001^m`02K_L00`00K_L00002K_L01000K_M^m`00
0Vkg00@006kgK_L0009^m`07001Zn6[hJ_P006[h00000f[h00H006[hJ_P006[h000>Ool00`00Oomo
o`0;Ool00`00Oomoo`0DOol001ioo`03001oogoo035oo`03001FoEKm0080009Fo@03001FoEKm0080
009Fo@03001Fo@0000800004D_d004kmC_d600026O`40000105I000000000U:k0`000f;j00<006[h
J_P00V[h00@006[hJ_P0009Zn006001^mfkgK_L006kg0P0000I^m`00K_L006kg0002K_L200001Fkg
001^mfkg00000V[h00<006[h00000f[h00@006[h0000009Zn003001oogoo00eoo`03001oogoo00Yo
o`03001oogoo01Aoo`007goo0P00<Woo0P0000AFo@00E_eFo@<00003E_d0000000<0009Fo@D0009>
o@040016g@00<Wh80002<S`20002D[/00`00H_YRnP020003J_P01000J_P000000f[h00@006kgK_L0
009^m`04001^mfkg0002K_L01000K_M^m`000Vkg00D006kgK_L006kg0080009Zn0800005J_P006[h
J_P00002J_P00`00Oomoo`0?Ool00`00Oomoo`09Ool00`00Oomoo`0DOol0025oo`03001oogoo035o
o`@0009Fo@80009Fo@03001Fo@0000800004D_d005;mD_d@00000c8l001B^`03D[/20002H_X00`00
J_P00002J_P02@00J_QZn6[h001Zn000K_L00002K_L200001fkg001^mfkg001^m`0000=^m`03001^
m`00009^m`04001Zn6[h0002J_P00`00J_QZn00200000f[h001oo`0BOol00`00Oomoo`08Ool00`00
Oomoo`0DOol0029oo`03001oogoo031oo`03001FoEKm00<00004E_d005KmE_d20002E_d00`00D_d0
0002000015;m001>gDkM1P0000DAg0000000001D00@0009B^`8000=RnP80009Zn005001Zn000J_P0
0003J_P01000K_M^m`000Vkg00H006kgK_L006kg0003K_L00`00K_L00003K_L200001f[h001Zn6[h
001Zn000009Zn003001oogoo019oo`03001oogoo00Qoo`03001oogoo01Aoo`008goo00<007ooOol0
<7oo0P0000=Fo@000000100000=Fo@0000000`000U;m1@000T[M00@004:n000^G@H000030EX0039L
0080009B^`07001RnP00H_X006[h00000f[h00<006[hJ_P00V[h00<006kg00000fkg00<006kg0000
0Vkg00@006kgK_L0009^m`04001^mfkg0002K_L01P00J_QZn000J_P0009Zn003001oogoo01Aoo`03
001oogoo00Qoo`03001oogoo01=oo`0097oo00<007ooOol0<Goo00@005KmE_d0009Fo@80009Fo@03
001Fo@0000800004D_d005;mD_d>0000105J000000000U:k0P000V;j0P000f[h0P000f[h00<006[h
00000fkg00<006kg00000Vkg00P006kg001^m`00K_L006kg0P000fkg00<006[h00000V[h00<006[h
000067oo00<007ooOol01goo00<007ooOol04goo000UOol2000aOol00`00E_eFo@03000015Km001F
oEKm0P000UKm1@0000ABo@00B]e:g@@00003:Ud0000000D0008fG0800004D[/006;j0002H_X00`00
J_P00002I_P00`00J_P00002J_P200001fkg001^mfkg001^m`0000=^m`03001^m`00009^m`07001^
mfkgK_L006kg00000V[h00<006kh00006Woo00<007ooOol01Woo00<007ooOol04goo000WOol00`00
Oomoo`0_Ool20002E_d500001UKm00000000E_d0009Bo@D000Aoo`T00004=U`005:kD[/20004H_X0
0`00I_QVn002I_P01000J_QZn0000V[h00D006kgK_L006kg008000=^m`03001^m`00009^m`800007
K_L006kgK_L006[h00000Vkh00<007ooOol067oo00<007ooOol01Woo00<007ooOol04goo000XOol0
0`00Oomoo`0`Ool2000015Km001FoEKm0P000UKm00<005Km00000P003Goo2@0000=B^`00H_X00V;j
0P000fKh0P0000=Zn000J_P00V[h00<006kg00000Vkg00L006kgK_M^m`00K_L00002K_L01000K_M^
m`000Vkg00<006kg00000Vkh00<007ooOol06Woo00<007ooOol01Goo00<007ooOol04goo000YOol0
0`00Oomoo`0`Ool00`00E_d0000200000eKm00000003000AOol3000010Ej000fG0000e:k00<006;j
00000V;j00<006Kh00000VKi00H006[hJ_QZn000J_P20002K_L2000016kg001^mfkg0P0000M^m`00
K_M^m`00K_L00002K_L200000fkh001oo`0LOol00`00Oomoo`04Ool00`00Oomoo`0COol002Yoo`80
035oo`03001FoEKm008001Moo`L0009B^`8000=RnP80009Vn@05001Vn@00J_P00002J_P01000K_M^
m`000Vkg00@006kgK_L0009^m`06001^mfkg001^m`000Vkg00@006kgK_L001moo`03001oogoo00=o
o`03001oogoo01=oo`00;7oo00<007ooOol0;goo0P006Woo10000SIL0P000U;K00L006;j001RnP00
I_T00003I_T02000J_QZn000J_P006kg0002K_L00`00K_L00003K_L00`00K_L00002K_L200001Fkg
001^mfkg00007goo00<007ooOol017oo00<007ooOol04Woo000]Ool00`00Oomoo`0^Ool00`00Oomo
o`0IOol70002D]/30002H_X20003I_T200000f[h001Zn002J_P01P00K_M^m`00K_L000=^m`03001^
m`00009^m`04001^mfkg0002K_L00`00Oomoo`0OOol00`00Oomoo`03Ool00`00Oomoo`0BOol002io
o`03001oogoo02eoo`03001oogoo01Yoo`<000032GX003IL0080009Bf`03001RnP00009RnP03001V
n@00009Vn@03001Zn6[h009Zn003001^m`00009^m`800007K_L006kgK_L006kg00000fkg00<006kg
00008goo00<007ooOol00Woo00<007ooOol04Woo000_Ool2000]Ool00`00Oomoo`0KOol3000010Uj
000000000U;K0P000f;j0P000fKi00L006[hJ_QZn000J_P00002K_L01000K_M^m`000Vkg00<006kg
00000fkg00@006kg000002=oo`03001oogoo009oo`03001oogoo019oo`00<Goo00<007ooOol0:Woo
00<007ooOol077oo10000SIL0P000U;L00D006;j001RnP00009Vn@05001Vn@00J_P00002J_P20000
1fkg001^mfkg001^m`00009^m`04001^mfkg0002K_L00`00Oomoo`0SOol01@00Oomoogoo000057oo
000bOol00`00Oomoo`0YOol00`00Oomoo`0LOol300001@Uj000fG3IL00000U;L0P000f;j0P000fKi
00@006[hJ_P0009Zn005001^mfkg001^n00200001Vkg001^m`00K_L0009^m`03001oogoo02Aoo`04
001oogoo000DOol003=oo`03001oogoo02Qoo`03001oogoo01eoo`T00004D]`006;jH_X20003I_T2
00000f[h001Zn002J_P00`00K_L00002K_P01000K_M^m`000Vkg00@006kgK_L002Qoo`03001oo`00
01Aoo`00=7oo0P00:7oo00<007ooOol07Woo10000SIM00@005;LD]aBg080009RnP03001Vn@00009V
n@07001Zn6[hJ_P006[h00000fkh0P0000M^m`00K_M^m`00K_L0000YOol01000Oomoo`004goo000f
Ool00`00Oomoo`0UOol00`00Oomoo`0OOol:0003H_X20002I_T01@00I_T006[h00000V[h00@006kh
K_P0009^n006001^mfkg001^m`00:goo00<007oo00004goo000gOol00`00Oomoo`0TOol00`00Oomo
o`0OOol01`00010000VK000fG@0000=Bg007001RnP00H_X006Ki00000fKi00D006[hJ_P006[h0080
00=^n003001^m`00009^m`03001oogoo02Yoo`8001=oo`00>7oo00<007ooOol08goo00<007ooOol0
87oo1`000U;L0P000V;j0P000fKi0P0000EZn000J_QZn00000=^n003001^n000009^m`03001oogoo
02Yoo`<001=oo`00>Goo0P008goo00<007ooOol08Goo10000SIM0P0000ABg000H_X0009RnP03001V
n@00009Vn@04001Zn6[hJ_P200001fkh001^n6kh001^m`0002Uoo`@001Ioo`00>goo00<007ooOol0
87oo00<007ooOol08Woo1P000U;L0P000f;j0P000VKi00D006Ki001Zn000009Zn006001^n6kh001^
n0009Woo10006Woo000lOol00`00Oomoo`0OOol00`00Oomoo`0ROol300000`fK000fO@020002D]`0
1`00H_X006;j001Vn@0000=Vn@03001Zn6[h00800005K_P006khK_P0000ROol4000NOol003eoo`03
001oogoo01ioo`03001oogoo02=oo`L0009Bg080009RnP8000=Vn@800005J_P006[hJ_P00002K_P0
0`00Oomoo`0NOol3000ROol003ioo`8001ioo`03001oogoo02Aoo`8000043I/003Ym>Wd2000015;L
001RnP000V;j00<006Ki00000VKi00P006[hJ_P006[h001^n00077oo10009Goo0010Ool00`00Oomo
o`0KOol00`00Oomoo`0UOol2000010fK000000000U;L0P000f;j0P000VKi0P0000=Zn000J_P00V[h
00<007ooOol05goo1000:Goo0011Ool00`00Oomoo`0JOol00`00Oomoo`0UOol500000cYm00000002
D]`01`00H_X006;j001Vn@00009Vn@05001Zn6[hJ_P0000FOol4000]Ool0049oo`03001oogoo01Uo
o`03001oogoo02Ioo`80008=^`<0009Bg080009RnP80009Vn@05001Vn@00J_P0000COol4000aOol0
04=oo`8001Uoo`03001oogoo02Moo`@0008jO@800008D]`006;j001NnP00I_T000=Vn@03001oogoo
00ioo`@003Eoo`00AGoo00<007ooOol05Goo00<007ooOol0:Goo0P0000@A^`000000009Bg08000=N
nP05001VnFKiI_T0000=Ool4000iOol004Ioo`03001oogoo01Aoo`03001oogoo02Yoo`@00003>Wd0
0000009Bg007001NnP00G_X006Ki00002goo0`00?Goo0017Ool00`00Oomoo`0COol00`00Oomoo`0[
Ool00`004K/A^`030002D]`20002G_X20008Ool40010Ool004Qoo`8001=oo`03001oogoo02aoo`<0
008jO@800004D]`005kj0006Ool40014Ool004Yoo`03001oogoo011oo`03001oogoo02moo`<0009B
g08000=oo`@004Qoo`00Bgoo00<007ooOol03goo00<007ooOol0<Woo0P0000=oo`0000000P00C7oo
001<Ool00`00Oomoo`0>Ool00`00Oomoo`0aOol4001@Ool004eoo`03001oogoo00eoo`03001oogoo
02eoo`@005Aoo`00CWoo0P003Goo00<007ooOol0:Goo1000F7oo001@Ool00`00Oomoo`0:Ool00`00
Oomoo`0VOol3001LOol0055oo`03001oogoo00Uoo`03001oogoo029oo`@005moo`00DWoo00<007oo
Ool027oo00<007ooOol07Woo1000Hgoo001COol20008Ool00`00Oomoo`0JOol4001WOol005Eoo`03
001oogoo00Eoo`03001oogoo01Ioo`@006]oo`00EWoo00<007ooOol017oo00<007ooOol04Woo1000
Kgoo001GOol00`00Oomoo`03Ool00`00Oomoo`0>Ool4001cOol005Qoo`8000=oo`03001oogoo00]o
o`<007Moo`00FWoo00@007ooOol000Uoo`@007Yoo`00Fgoo00<007oo00001Goo1000OWoo001LOol2
00000goo000000020022Ool005eoo`8008Ioo`00iGoo003UOol00>Eoo`00iGoo003UOol00>Eoo`00
iGoo0000\
\>"],
  ImageRangeCache->{{{0, 228.125}, {184.812, 0}} -> {-0.0817352, -0.03905, 
  0.00618296, 0.00618296}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.8 Exploring Rho", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallRho[p,k,sd,r,q,t]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the Rho of a Call option, the sensitivity of its value to changes \
in the (continuously compounded) risk-free interest rate."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
Plot[BlackScholesCallRho[x, 60, 0.29, 0.04, 0, 0.3], {x, \
20.,100.}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.214286 0.0119048 0.0147151 0.0331168 [
[.2619 .00222 -6 -9 ]
[.2619 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.7381 .00222 -6 -9 ]
[.7381 .00222 6 0 ]
[.97619 .00222 -9 -9 ]
[.97619 .00222 9 0 ]
[.01131 .09751 -18 -4.5 ]
[.01131 .09751 0 4.5 ]
[.01131 .1803 -6 -4.5 ]
[.01131 .1803 0 4.5 ]
[.01131 .26309 -18 -4.5 ]
[.01131 .26309 0 4.5 ]
[.01131 .34588 -12 -4.5 ]
[.01131 .34588 0 4.5 ]
[.01131 .42868 -24 -4.5 ]
[.01131 .42868 0 4.5 ]
[.01131 .51147 -12 -4.5 ]
[.01131 .51147 0 4.5 ]
[.01131 .59426 -24 -4.5 ]
[.01131 .59426 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.2619 .01472 m
.2619 .02097 L
s
[(40)] .2619 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(60)] .5 .00222 0 1 Mshowa
.7381 .01472 m
.7381 .02097 L
s
[(80)] .7381 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(100)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.08333 .01472 m
.08333 .01847 L
s
.14286 .01472 m
.14286 .01847 L
s
.20238 .01472 m
.20238 .01847 L
s
.32143 .01472 m
.32143 .01847 L
s
.38095 .01472 m
.38095 .01847 L
s
.44048 .01472 m
.44048 .01847 L
s
.55952 .01472 m
.55952 .01847 L
s
.61905 .01472 m
.61905 .01847 L
s
.67857 .01472 m
.67857 .01847 L
s
.79762 .01472 m
.79762 .01847 L
s
.85714 .01472 m
.85714 .01847 L
s
.91667 .01472 m
.91667 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .09751 m
.03006 .09751 L
s
[(2.5)] .01131 .09751 1 0 Mshowa
.02381 .1803 m
.03006 .1803 L
s
[(5)] .01131 .1803 1 0 Mshowa
.02381 .26309 m
.03006 .26309 L
s
[(7.5)] .01131 .26309 1 0 Mshowa
.02381 .34588 m
.03006 .34588 L
s
[(10)] .01131 .34588 1 0 Mshowa
.02381 .42868 m
.03006 .42868 L
s
[(12.5)] .01131 .42868 1 0 Mshowa
.02381 .51147 m
.03006 .51147 L
s
[(15)] .01131 .51147 1 0 Mshowa
.02381 .59426 m
.03006 .59426 L
s
[(17.5)] .01131 .59426 1 0 Mshowa
.125 Mabswid
.02381 .03127 m
.02756 .03127 L
s
.02381 .04783 m
.02756 .04783 L
s
.02381 .06439 m
.02756 .06439 L
s
.02381 .08095 m
.02756 .08095 L
s
.02381 .11407 m
.02756 .11407 L
s
.02381 .13062 m
.02756 .13062 L
s
.02381 .14718 m
.02756 .14718 L
s
.02381 .16374 m
.02756 .16374 L
s
.02381 .19686 m
.02756 .19686 L
s
.02381 .21342 m
.02756 .21342 L
s
.02381 .22997 m
.02756 .22997 L
s
.02381 .24653 m
.02756 .24653 L
s
.02381 .27965 m
.02756 .27965 L
s
.02381 .29621 m
.02756 .29621 L
s
.02381 .31277 m
.02756 .31277 L
s
.02381 .32932 m
.02756 .32932 L
s
.02381 .36244 m
.02756 .36244 L
s
.02381 .379 m
.02756 .379 L
s
.02381 .39556 m
.02756 .39556 L
s
.02381 .41212 m
.02756 .41212 L
s
.02381 .44523 m
.02756 .44523 L
s
.02381 .46179 m
.02756 .46179 L
s
.02381 .47835 m
.02756 .47835 L
s
.02381 .49491 m
.02756 .49491 L
s
.02381 .52803 m
.02756 .52803 L
s
.02381 .54458 m
.02756 .54458 L
s
.02381 .56114 m
.02756 .56114 L
s
.02381 .5777 m
.02756 .5777 L
s
.02381 .61082 m
.02756 .61082 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.02499 .01472 L
.02605 .01472 L
.02721 .01472 L
.02846 .01472 L
.02961 .01472 L
.03068 .01472 L
.03279 .01472 L
.03396 .01472 L
.03506 .01472 L
.03636 .01472 L
.03756 .01472 L
.04 .01472 L
.04134 .01472 L
.04262 .01472 L
.045 .01472 L
.04633 .01472 L
.04753 .01472 L
.04988 .01472 L
.0521 .01472 L
.05469 .01472 L
.05707 .01472 L
.05968 .01472 L
.061 .01472 L
.06244 .01472 L
.0638 .01472 L
.06527 .01472 L
.06795 .01472 L
.07055 .01472 L
.07296 .01472 L
.07556 .01472 L
.07843 .01472 L
.08142 .01472 L
.08424 .01472 L
.08672 .01472 L
.08934 .01472 L
.09158 .01472 L
.09403 .01472 L
.09667 .01472 L
.09949 .01472 L
.10216 .01472 L
.10458 .01472 L
.10689 .01472 L
.10943 .01472 L
.11208 .01472 L
.11456 .01472 L
.119 .01472 L
.12149 .01472 L
.12376 .01472 L
.12887 .01472 L
Mistroke
.13176 .01472 L
.13443 .01472 L
.13934 .01472 L
.14208 .01472 L
.14456 .01472 L
.14945 .01472 L
.15191 .01472 L
.15461 .01472 L
.15939 .01473 L
.16385 .01473 L
.16911 .01474 L
.17391 .01475 L
.17944 .01477 L
.18468 .01479 L
.18928 .01481 L
.19436 .01484 L
.19966 .01489 L
.20463 .01494 L
.21351 .01508 L
.21849 .01518 L
.22306 .0153 L
.22782 .01545 L
.23289 .01564 L
.24202 .01611 L
.24761 .01648 L
.25288 .0169 L
.26274 .01793 L
.27249 .01931 L
.28279 .02126 L
.29159 .02342 L
.30126 .02641 L
.32074 .03483 L
.33167 .0412 L
.34165 .0482 L
.36071 .06503 L
.38146 .08883 L
.42221 .15181 L
.46145 .22791 L
.49916 .30657 L
.53934 .38617 L
.57799 .45156 L
.6191 .50548 L
.6396 .52634 L
.65869 .54246 L
.69677 .56645 L
.7172 .57561 L
.73578 .58217 L
.75494 .58746 L
.77573 .59186 L
.79558 .59501 L
Mistroke
.81662 .59749 L
.83617 .59919 L
.85447 .60038 L
.87473 .60135 L
.89326 .60199 L
.91226 .60248 L
.93299 .60286 L
.95268 .60311 L
.96368 .60322 L
.97366 .6033 L
.97619 .60332 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{180.938, 111.812},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003R0000Ra000`40O003h00OhWoo003ROol00>9o
o`00hWoo003ROol00>9oo`00hWoo0018Ool30004Ool3000SOol30005Ool3000ROol30005Ool3000O
Ool50003Ool30005Ool30003Ool004Uoo`03001oogoo009oo`05001oogooOol0000QOol01@00Oomo
ogoo00000goo00D007ooOomoo`00021oo`05001oogooOol00003Ool01@00Oomoogoo000087oo00<0
07ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Ioo`D000=oo`05001oogoo
Ool0000QOol01@00Oomoogoo00000goo00D007ooOomoo`00021oo`05001oogooOol00003Ool01@00
Oomoogoo000087oo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Io
o`04001oogoo0004Ool01@00Oomoogoo00008Goo00D007ooOomoo`0000=oo`05001oogooOol0000P
Ool01@00Oomoogoo00000goo00D007ooOomoo`00021oo`03001oogoo009oo`05001oogooOol00003
Ool01@00Oomoogoo00000Woo0017Ool00`00Ool00004Ool01@00Oomoogoo00008Goo100017oo00D0
07ooOomoo`00025oo`<000Aoo`05001oogooOol0000POol00`00Oomoo`02Ool01@00Oomoogoo0000
0goo00D007ooOomoo`00009oo`00Agoo00<007oo000017oo00D007ooOomoo`00025oo`03001oogoo
00Eoo`05001oogooOol0000POol01@00Oomoogoo00000goo00D007ooOomoo`00021oo`03001oogoo
009oo`05001oogooOol00003Ool01@00Oomoogoo00000Woo0018Ool20004Ool01@00Oomoogoo0000
8Woo00<007ooOol017oo00D007ooOomoo`00021oo`05001oogooOol00003Ool01@00Oomoogoo0000
87oo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00002Ool004Uoo`03001oogoo
00=oo`<002Aoo`<000Aoo`<0029oo`<000Eoo`<001moo`<000Eoo`<000Eoo`<000=oo`0087oo00<0
07ooOol0_goo000POol00`00Oomoo`2oOol0021oo`03001oogoo0;moo`006goo`0001goo000POol0
0`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`08Ool60006Ool00`00Oomo
o`08Ool00`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomo
o`09Ool00`00Oomoo`08Ool00`00Oomoo`08Ool00`00Oomoo`09Ool00`00Oomoo`08Ool00`00Oomo
o`08Ool00`00Oomoo`0:Ool0021oo`03001oogoo031oo`@008]oo`0087oo0P00=Goo00<007ooOol0
R7oo000POol00`00Oomoo`0eOol30027Ool0021oo`03001oogoo03Qoo`03001oogoo08Aoo`0087oo
0P00>Woo0P00Q7oo000POol00`00Oomoo`0kOol00`00Oomoo`21Ool0021oo`03001oogoo03aoo`03
001oogoo081oo`0087oo0P00?Woo00<007ooOol0Ogoo000POol00`00Oomoo`0mOol00`00Oomoo`1o
Ool0021oo`03001oogoo03ioo`03001oogoo07ioo`0087oo00<007ooOol0?goo00<007ooOol0OGoo
0007Ool50005Ool20005Ool40004Ool20011Ool00`00Oomoo`1lOol000Qoo`04001oogoo0005Ool2
0009Ool01@00Oomoogoo0000@goo00<007ooOol0Ngoo0009Ool00`00Oomoo`0@Ool01@00Oomoogoo
0000@goo00<007ooOol0Ngoo000:Ool00`00Oomoo`0?Ool01000Oomoogoo0P00@goo00<007ooOol0
NWoo000;Ool00`00Oomoo`0:Ool40004Ool00`00Oomoo`13Ool00`00Oomoo`1iOol000]oo`03001o
ogoo00Yoo`03001oogoo00Eoo`03001oogoo04=oo`03001oogoo07Uoo`001goo00D007ooOomoo`00
00aoo`03001oogoo00Eoo`8004Eoo`03001oogoo07Qoo`0027oo0`003Goo100017oo00<007ooOol0
AGoo00<007ooOol0Mgoo000POol00`00Oomoo`15Ool00`00Oomoo`1gOol0021oo`8004Moo`03001o
ogoo07Ioo`0087oo00<007ooOol0Agoo00<007ooOol0MGoo000POol00`00Oomoo`17Ool00`00Oomo
o`1eOol0021oo`8004Uoo`03001oogoo07Aoo`0087oo00<007ooOol0BGoo00<007ooOol0Lgoo000P
Ool00`00Oomoo`19Ool00`00Oomoo`1cOol0021oo`8004]oo`03001oogoo079oo`0067oo100017oo
00<007ooOol0BWoo00<007ooOol0LWoo000LOol01@00Oomoogoo0000CGoo00<007ooOol0LGoo000L
Ool01000Oomoogoo0P00C7oo00<007ooOol0LGoo000LOol01@00Oomoogoo0000CWoo00<007ooOol0
L7oo000HOol40004Ool00`00Oomoo`1<Ool00`00Oomoo`1`Ool001Qoo`03001oogoo00Eoo`03001o
ogoo04eoo`03001oogoo06moo`0067oo00<007ooOol01Goo0P00CWoo00<007ooOol0Kgoo000HOol4
0004Ool00`00Oomoo`1=Ool00`00Oomoo`1_Ool0021oo`03001oogoo04ioo`03001oogoo06ioo`00
87oo0P00Cgoo00<007ooOol0KWoo000POol00`00Oomoo`1?Ool00`00Oomoo`1]Ool0021oo`03001o
ogoo04moo`03001oogoo06eoo`0087oo0P00DGoo00<007ooOol0K7oo000POol00`00Oomoo`1@Ool0
0`00Oomoo`1/Ool0021oo`03001oogoo055oo`03001oogoo06]oo`0087oo0P00DWoo00<007ooOol0
Jgoo0009Ool00`00Oomoo`05Ool20005Ool40004Ool00`00Oomoo`1BOol00`00Oomoo`1ZOol000Uo
o`03001oogoo00Eoo`8000Uoo`05001oogooOol0001DOol00`00Oomoo`1ZOol000Yoo`03001oogoo
00moo`04001oogooOol2001DOol00`00Oomoo`1YOol000Yoo`03001oogoo00moo`05001oogooOol0
001EOol00`00Oomoo`1YOol000Yoo`03001oogoo00]oo`@000Aoo`03001oogoo05Aoo`03001oogoo
06Qoo`002goo00<007ooOol02Woo00<007ooOol01Goo0P00EGoo00<007ooOol0J7oo0007Ool01@00
Oomoogoo000037oo00<007ooOol01Goo00<007ooOol0E7oo00<007ooOol0J7oo0007Ool5000<Ool4
0004Ool00`00Oomoo`1EOol00`00Oomoo`1WOol0021oo`03001oogoo05Eoo`03001oogoo06Moo`00
87oo0P00Egoo00<007ooOol0IWoo000POol00`00Oomoo`1FOol00`00Oomoo`1VOol0021oo`03001o
ogoo05Moo`03001oogoo06Eoo`0087oo0P00F7oo00<007ooOol0IGoo000POol00`00Oomoo`1HOol0
0`00Oomoo`1TOol0021oo`03001oogoo05Qoo`03001oogoo06Aoo`0087oo0P00FWoo00<007ooOol0
Hgoo000@Ool50003Ool30005Ool00`00Oomoo`1IOol00`00Oomoo`1SOol0019oo`03001oogoo009o
o`05001oogooOol00004Ool00`00Oomoo`1JOol00`00Oomoo`1ROol0019oo`03001oogoo009oo`05
001oogooOol00004Ool2001KOol00`00Oomoo`1ROol0019oo`03001oogoo009oo`05001oogooOol0
0004Ool00`00Oomoo`1KOol00`00Oomoo`1QOol0019oo`03001oogoo009oo`05001oogooOol00004
Ool00`00Oomoo`1LOol00`00Oomoo`1POol0019oo`03001oogoo009oo`05001oogooOol00004Ool2
001MOol00`00Oomoo`1POol0019oo`03001oogoo009oo`05001oogooOol00004Ool00`00Oomoo`1M
Ool00`00Oomoo`1OOol0011oo`<000Eoo`<000Eoo`03001oogoo05eoo`03001oogoo05moo`0087oo
0P00Ggoo00<007ooOol0GWoo000POol00`00Oomoo`1NOol00`00Oomoo`1NOol0021oo`03001oogoo
05moo`03001oogoo05eoo`0087oo0P00H7oo00<007ooOol0GGoo000POol00`00Oomoo`1POol00`00
Oomoo`1LOol0021oo`03001oogoo061oo`03001oogoo05aoo`0087oo00<007ooOol0HGoo00<007oo
Ool0Fgoo00050002Ool50005Ool20004Ool40005Ool2001SOol00`00Oomoo`1JOol0009oo`03001o
ogoo00=oo`04001oogoo0005Ool20008Ool00`00Oomoo`02Ool00`00Oomoo`1ROol00`00Oomoo`1J
Ool0009oo`03001oogoo00Aoo`03001oogoo00moo`03001oogoo009oo`03001oogoo06=oo`03001o
ogoo05Uoo`000Woo00<007ooOol01Goo00<007ooOol03Woo00<007ooOol00Woo0P00I7oo00<007oo
Ool0FGoo0002Ool00`00Oomoo`06Ool00`00Oomoo`09Ool40005Ool00`00Oomoo`1TOol00`00Oomo
o`1HOol0009oo`03001oogoo00Ioo`03001oogoo00Uoo`03001oogoo00Ioo`03001oogoo06Eoo`03
001oogoo05Moo`000Woo00<007ooOol00Woo00D007ooOomoo`0000]oo`03001oogoo00Ioo`8006Io
o`03001oogoo05Moo`000`001Goo0`0037oo10001Goo00<007ooOol0IWoo00<007ooOol0EWoo000P
Ool00`00Oomoo`1VOol00`00Oomoo`1FOol0021oo`8006Qoo`03001oogoo05Eoo`0087oo00<007oo
Ool0J7oo00<007ooOol0E7oo000POol00`00Oomoo`1YOol00`00Oomoo`1COol0021oo`8006]oo`03
001oogoo059oo`0087oo00<007ooOol0JWoo00<007ooOol0DWoo000POol00`00Oomoo`1[Ool00`00
Oomoo`1AOol0021oo`8006eoo`03001oogoo051oo`0047oo1@000goo100017oo00<007ooOol0KGoo
00<007ooOol0Cgoo000BOol00`00Oomoo`07Ool01@00Oomoogoo0000L7oo00<007ooOol0CWoo000B
Ool00`00Oomoo`07Ool01@00Oomoogoo0000LGoo00<007ooOol0CGoo000BOol00`00Oomoo`07Ool0
1000Oomoogoo0P00LGoo00<007ooOol0C7oo000BOol00`00Oomoo`03Ool40004Ool00`00Oomoo`1a
Ool00`00Oomoo`1;Ool0019oo`03001oogoo00=oo`03001oogoo00Eoo`03001oogoo079oo`8004]o
o`004Woo00<007ooOol00goo00<007ooOol01Goo0P00MGoo00<007ooOol0B7oo000@Ool30005Ool4
0004Ool00`00Oomoo`1eOol00`00Oomoo`17Ool0021oo`03001oogoo07Ioo`8004Moo`0087oo0P00
NGoo00<007ooOol0A7oo000POol00`00Oomoo`1iOol20014Ool0021oo`03001oogoo07]oo`03001o
ogoo045oo`0087oo0P00OGoo00<007ooOol0@7oo000POol00`00Oomoo`1mOol20010Ool0021oo`03
001oogoo07moo`8003ioo`0087oo0P00PWoo0`00>goo00050004Ool00`00Oomoo`05Ool20004Ool4
0005Ool00`00Oomoo`24Ool4000gOol0009oo`03001oogoo00Aoo`03001oogoo00Eoo`8000Qoo`03
001oogoo009oo`03001oogoo08Qoo`<003Aoo`000Woo00<007ooOol01Goo00<007ooOol03Woo00<0
07ooOol00Woo0P00S7oo2000;7oo0002Ool00`00Oomoo`05Ool00`00Oomoo`0>Ool00`00Oomoo`02
Ool00`00Oomoo`2COol8000TOol0009oo`03001oogoo00Eoo`03001oogoo00Yoo`@000Eoo`03001o
ogoo09]ooaP000aoo`000Woo00<007ooOol01Woo00<007ooOol02Goo00<007ooOol01Woo00<007oo
Ool0_goo0002Ool00`00Oomoo`02Ool01@00Oomoogoo00002goo00<007ooOol01Woo0P00`7oo0003
0004Ool5000;Ool40005Ool00`00Oomoo`2oOol00>9oo`00hWoo003ROol00>9oo`00hWoo0000\
\>"],
  ImageRangeCache->{{{0, 225.125}, {138.75, 0}} -> {5.72586, -2.87713, 
  0.550989, 0.198069}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
Plot3D[BlackScholesCallRho[x, 60, 0.29, 0.04, 0, t], {x, 20.,100.}, \
{t,0.01,0.5}, PlotPoints -> 40];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -12 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -11.7989 -9 ]
[.17646 .18758 .20112 0 ]
[.3345 .12542 -11.1734 -9 ]
[.3345 .12542 .8266 0 ]
[.50253 .05935 -10.5479 -9 ]
[.50253 .05935 1.45207 0 ]
[.68156 -0.01098 -14.8837 -9 ]
[.68156 -0.01098 3.11632 0 ]
[.76225 .08907 0 -6.16962 ]
[.76225 .08907 18 2.83038 ]
[.82456 .18447 0 -6.07766 ]
[.82456 .18447 18 2.92234 ]
[.88151 .27169 0 -5.9953 ]
[.88151 .27169 18 3.0047 ]
[.93378 .35174 0 -5.92111 ]
[.93378 .35174 18 3.07889 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 18 3.14607 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.0125 .33792 -12 -2.80391 ]
[.0125 .33792 0 6.19609 ]
[.00032 .41425 -12 -2.86881 ]
[.00032 .41425 0 6.13119 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.74995 .09363 m
.7438 .09592 L
s
[(0.1)] .76225 .08907 -1 .37103 Mshowa
.81217 .18881 m
.80598 .19098 L
s
[(0.2)] .82456 .18447 -1 .35059 Mshowa
.86906 .27583 m
.86283 .2779 L
s
[(0.3)] .88151 .27169 -1 .33229 Mshowa
.92126 .35569 m
.91501 .35766 L
s
[(0.4)] .93378 .35174 -1 .3158 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(0.5)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.76286 .11338 m
.75916 .11473 L
s
.77553 .13276 m
.77182 .1341 L
s
.78796 .15178 m
.78426 .15311 L
s
.80018 .17047 m
.79647 .17178 L
s
.82395 .20683 m
.82023 .20812 L
s
.83553 .22454 m
.8318 .22582 L
s
.8469 .24193 m
.84317 .2432 L
s
.85807 .25903 m
.85434 .26028 L
s
.87985 .29234 m
.87611 .29357 L
s
.89047 .30858 m
.88673 .3098 L
s
.90091 .32455 m
.89716 .32575 L
s
.91117 .34025 m
.90742 .34144 L
s
.9312 .37088 m
.92744 .37205 L
s
.94096 .38582 m
.9372 .38699 L
s
.95058 .40053 m
.94681 .40168 L
s
.96004 .415 m
.95627 .41614 L
s
.7368 .07352 m
.73311 .0749 L
s
.72339 .05301 m
.71971 .05441 L
s
.70973 .03212 m
.70606 .03353 L
s
.6958 .01081 m
.69213 .01224 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02478 .33329 m
.03091 .33097 L
s
[(10)] .0125 .33792 1 -0.37691 Mshowa
.01265 .40978 m
.01882 .40754 L
s
[(20)] .00032 .41425 1 -0.36249 Mshowa
.125 Mabswid
.03407 .27465 m
.03774 .27323 L
s
.03178 .28911 m
.03545 .28769 L
s
.02946 .3037 m
.03314 .30229 L
s
.02713 .31842 m
.03081 .31702 L
s
.0224 .3483 m
.02608 .34692 L
s
.02 .36345 m
.02369 .36208 L
s
.01757 .37874 m
.02127 .37738 L
s
.01513 .39419 m
.01882 .39283 L
s
.01016 .42552 m
.01386 .42419 L
s
.00764 .44142 m
.01135 .4401 L
s
.00509 .45748 m
.00881 .45617 L
s
.00253 .47369 m
.00624 .47239 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.664 .741 .908 r
.39729 .59642 .40512 .60371 .41826 .59977 .41047 .59244 Metetra
.664 .741 .908 r
.41047 .59244 .41826 .59977 .43147 .59581 .42372 .58845 Metetra
.663 .741 .908 r
.42372 .58845 .43147 .59581 .44474 .59183 .43703 .58443 Metetra
.663 .741 .908 r
.43703 .58443 .44474 .59183 .45807 .58785 .4504 .58041 Metetra
.662 .741 .908 r
.4504 .58041 .45807 .58785 .47147 .58389 .46385 .5764 Metetra
.66 .74 .909 r
.46385 .5764 .47147 .58389 .48494 .58001 .47735 .57246 Metetra
.654 .738 .911 r
.47735 .57246 .48494 .58001 .49847 .57633 .49093 .56871 Metetra
.643 .733 .914 r
.49093 .56871 .49847 .57633 .51207 .57305 .50457 .56532 Metetra
.624 .726 .92 r
.50457 .56532 .51207 .57305 .52574 .57042 .51829 .56254 Metetra
.594 .712 .926 r
.51829 .56254 .52574 .57042 .5395 .56875 .53208 .56068 Metetra
.549 .691 .933 r
.53208 .56068 .5395 .56875 .55336 .56835 .54598 .56006 Metetra
.487 .659 .937 r
.54598 .56006 .55336 .56835 .56734 .56951 .55999 .56096 Metetra
.41 .615 .935 r
.55999 .56096 .56734 .56951 .58146 .57239 .57413 .56357 Metetra
.322 .562 .923 r
.57413 .56357 .58146 .57239 .59575 .57703 .58844 .56794 Metetra
.235 .505 .904 r
.58844 .56794 .59575 .57703 .61023 .58335 .60294 .57399 Metetra
.161 .453 .882 r
.60294 .57399 .61023 .58335 .62491 .5911 .61763 .5815 Metetra
.108 .413 .862 r
.61763 .5815 .62491 .5911 .63981 .59996 .63254 .59012 Metetra
.083 .391 .85 r
.63254 .59012 .63981 .59996 .65492 .60951 .64766 .59946 Metetra
.085 .388 .847 r
.64766 .59946 .65492 .60951 .67024 .61934 .66299 .60908 Metetra
.111 .4 .853 r
.66299 .60908 .67024 .61934 .68575 .62902 .67851 .61857 Metetra
.156 .425 .864 r
.67851 .61857 .68575 .62902 .70144 .6382 .69421 .62755 Metetra
.213 .457 .876 r
.69421 .62755 .70144 .6382 .71729 .64658 .71006 .63573 Metetra
.275 .491 .886 r
.71006 .63573 .71729 .64658 .73327 .65396 .72606 .64289 Metetra
.336 .523 .892 r
.72606 .64289 .73327 .65396 .74936 .66018 .74216 .6489 Metetra
.393 .552 .893 r
.74216 .6489 .74936 .66018 .76556 .66519 .75837 .65369 Metetra
.443 .575 .892 r
.75837 .65369 .76556 .66519 .78183 .669 .77466 .65727 Metetra
.485 .594 .888 r
.77466 .65727 .78183 .669 .79817 .67164 .79102 .65968 Metetra
.519 .609 .882 r
.79102 .65968 .79817 .67164 .81458 .67319 .80745 .66102 Metetra
.547 .62 .876 r
.80745 .66102 .81458 .67319 .83105 .67377 .82394 .66138 Metetra
.569 .628 .871 r
.82394 .66138 .83105 .67377 .84757 .67347 .84048 .66088 Metetra
.587 .634 .865 r
.84048 .66088 .84757 .67347 .86416 .67242 .85709 .65964 Metetra
.601 .639 .86 r
.85709 .65964 .86416 .67242 .8808 .67072 .87375 .65776 Metetra
.612 .642 .856 r
.87375 .65776 .8808 .67072 .8975 .66848 .89048 .65535 Metetra
.62 .644 .853 r
.89048 .65535 .8975 .66848 .91427 .66577 .90728 .6525 Metetra
.627 .646 .85 r
.90728 .6525 .91427 .66577 .9311 .66269 .92415 .64929 Metetra
.632 .647 .847 r
.92415 .64929 .9311 .66269 .94801 .65931 .94109 .64577 Metetra
.636 .648 .845 r
.94109 .64577 .94801 .65931 .965 .65566 .95812 .64202 Metetra
.639 .649 .844 r
.95812 .64202 .965 .65566 .98207 .65182 .97522 .63807 Metetra
.641 .649 .842 r
.97522 .63807 .98207 .65182 .99922 .6478 .99241 .63395 Metetra
.664 .741 .908 r
.38939 .58906 .39729 .59642 .41047 .59244 .40261 .58505 Metetra
.664 .741 .908 r
.40261 .58505 .41047 .59244 .42372 .58845 .41589 .58101 Metetra
.663 .741 .908 r
.41589 .58101 .42372 .58845 .43703 .58443 .42925 .57696 Metetra
.663 .741 .908 r
.42925 .57696 .43703 .58443 .4504 .58041 .44266 .5729 Metetra
.662 .741 .908 r
.44266 .5729 .4504 .58041 .46385 .5764 .45615 .56885 Metetra
.66 .74 .909 r
.45615 .56885 .46385 .5764 .47735 .57246 .4697 .56485 Metetra
.655 .738 .911 r
.4697 .56485 .47735 .57246 .49093 .56871 .48332 .56102 Metetra
.645 .734 .914 r
.48332 .56102 .49093 .56871 .50457 .56532 .497 .55752 Metetra
.627 .727 .919 r
.497 .55752 .50457 .56532 .51829 .56254 .51076 .5546 Metetra
.599 .714 .925 r
.51076 .5546 .51829 .56254 .53208 .56068 .5246 .55255 Metetra
.556 .694 .932 r
.5246 .55255 .53208 .56068 .54598 .56006 .53853 .55171 Metetra
.497 .664 .936 r
.53853 .55171 .54598 .56006 .55999 .56096 .55257 .55235 Metetra
.422 .622 .935 r
.55257 .55235 .55999 .56096 .57413 .56357 .56674 .55468 Metetra
.336 .57 .925 r
.56674 .55468 .57413 .56357 .58844 .56794 .58107 .55878 Metetra
.25 .514 .908 r
.58107 .55878 .58844 .56794 .60294 .57399 .59558 .56456 Metetra
.175 .462 .886 r
.59558 .56456 .60294 .57399 .61763 .5815 .61029 .57182 Metetra
.121 .422 .866 r
.61029 .57182 .61763 .5815 .63254 .59012 .62521 .58021 Metetra
.095 .399 .854 r
.62521 .58021 .63254 .59012 .64766 .59946 .64034 .58933 Metetra
.096 .395 .851 r
.64034 .58933 .64766 .59946 .66299 .60908 .65567 .59874 Metetra
.121 .407 .856 r
.65567 .59874 .66299 .60908 .67851 .61857 .6712 .60802 Metetra
.166 .431 .867 r
.6712 .60802 .67851 .61857 .69421 .62755 .68691 .6168 Metetra
.222 .463 .878 r
.68691 .6168 .69421 .62755 .71006 .63573 .70278 .62477 Metetra
.284 .496 .887 r
.70278 .62477 .71006 .63573 .72606 .64289 .71878 .63172 Metetra
.345 .528 .892 r
.71878 .63172 .72606 .64289 .74216 .6489 .7349 .63751 Metetra
.401 .556 .893 r
.7349 .63751 .74216 .6489 .75837 .65369 .75113 .64207 Metetra
.449 .579 .891 r
.75113 .64207 .75837 .65369 .77466 .65727 .76743 .64542 Metetra
.49 .597 .887 r
.76743 .64542 .77466 .65727 .79102 .65968 .78381 .64761 Metetra
.524 .611 .881 r
.78381 .64761 .79102 .65968 .80745 .66102 .80026 .64872 Metetra
.551 .622 .875 r
.80026 .64872 .80745 .66102 .82394 .66138 .81677 .64887 Metetra
.573 .629 .87 r
.81677 .64887 .82394 .66138 .84048 .66088 .83334 .64817 Metetra
.59 .635 .864 r
.83334 .64817 .84048 .66088 .85709 .65964 .84997 .64673 Metetra
.603 .639 .859 r
.84997 .64673 .85709 .65964 .87375 .65776 .86666 .64468 Metetra
.614 .643 .855 r
.86666 .64468 .87375 .65776 .89048 .65535 .88342 .64211 Metetra
.622 .645 .852 r
.88342 .64211 .89048 .65535 .90728 .6525 .90024 .63911 Metetra
.628 .646 .849 r
.90024 .63911 .90728 .6525 .92415 .64929 .91714 .63576 Metetra
.633 .647 .847 r
.91714 .63576 .92415 .64929 .94109 .64577 .93412 .63212 Metetra
.637 .648 .845 r
.93412 .63212 .94109 .64577 .95812 .64202 .95118 .62825 Metetra
.64 .649 .843 r
.95118 .62825 .95812 .64202 .97522 .63807 .96832 .62419 Metetra
.642 .649 .842 r
.96832 .62419 .97522 .63807 .99241 .63395 .98555 .61998 Metetra
.664 .741 .908 r
.38141 .58163 .38939 .58906 .40261 .58505 .39467 .57758 Metetra
.664 .741 .908 r
.39467 .57758 .40261 .58505 .41589 .58101 .408 .57351 Metetra
.663 .741 .908 r
.408 .57351 .41589 .58101 .42925 .57696 .42139 .56942 Metetra
.663 .741 .908 r
.42139 .56942 .42925 .57696 .44266 .5729 .43485 .56532 Metetra
.663 .741 .908 r
.43485 .56532 .44266 .5729 .45615 .56885 .44838 .56122 Metetra
.661 .74 .909 r
.44838 .56122 .45615 .56885 .4697 .56485 .46198 .55718 Metetra
.656 .739 .91 r
.46198 .55718 .4697 .56485 .48332 .56102 .47564 .55327 Metetra
.647 .735 .913 r
.47564 .55327 .48332 .56102 .497 .55752 .48937 .54967 Metetra
.631 .728 .918 r
.48937 .54967 .497 .55752 .51076 .5546 .50317 .5466 Metetra
.604 .716 .924 r
.50317 .5466 .51076 .5546 .5246 .55255 .51704 .54437 Metetra
.563 .697 .931 r
.51704 .54437 .5246 .55255 .53853 .55171 .53101 .5433 Metetra
.507 .668 .936 r
.53101 .5433 .53853 .55171 .55257 .55235 .54508 .54369 Metetra
.434 .628 .935 r
.54508 .54369 .55257 .55235 .56674 .55468 .55928 .54574 Metetra
.35 .578 .927 r
.55928 .54574 .56674 .55468 .58107 .55878 .57364 .54956 Metetra
.264 .523 .911 r
.57364 .54956 .58107 .55878 .59558 .56456 .58816 .55507 Metetra
.189 .471 .89 r
.58816 .55507 .59558 .56456 .61029 .57182 .60289 .56206 Metetra
.134 .431 .87 r
.60289 .56206 .61029 .57182 .62521 .58021 .61781 .57021 Metetra
.107 .407 .858 r
.61781 .57021 .62521 .58021 .64034 .58933 .63295 .57911 Metetra
.107 .402 .855 r
.63295 .57911 .64034 .58933 .65567 .59874 .6483 .58831 Metetra
.131 .414 .86 r
.6483 .58831 .65567 .59874 .6712 .60802 .66384 .59739 Metetra
.175 .437 .869 r
.66384 .59739 .6712 .60802 .68691 .6168 .67956 .60596 Metetra
.231 .468 .88 r
.67956 .60596 .68691 .6168 .70278 .62477 .69543 .61372 Metetra
.293 .501 .888 r
.69543 .61372 .70278 .62477 .71878 .63172 .71145 .62045 Metetra
.353 .532 .893 r
.71145 .62045 .71878 .63172 .7349 .63751 .72758 .62601 Metetra
.408 .56 .894 r
.72758 .62601 .7349 .63751 .75113 .64207 .74382 .63034 Metetra
.456 .582 .891 r
.74382 .63034 .75113 .64207 .76743 .64542 .76014 .63345 Metetra
.496 .6 .886 r
.76014 .63345 .76743 .64542 .78381 .64761 .77654 .63541 Metetra
.529 .613 .88 r
.77654 .63541 .78381 .64761 .80026 .64872 .79301 .6363 Metetra
.556 .623 .874 r
.79301 .6363 .80026 .64872 .81677 .64887 .80954 .63623 Metetra
.577 .631 .869 r
.80954 .63623 .81677 .64887 .83334 .64817 .82613 .63532 Metetra
.593 .636 .863 r
.82613 .63532 .83334 .64817 .84997 .64673 .84278 .6337 Metetra
.606 .64 .858 r
.84278 .6337 .84997 .64673 .86666 .64468 .8595 .63147 Metetra
.616 .643 .854 r
.8595 .63147 .86666 .64468 .88342 .64211 .87629 .62873 Metetra
.624 .645 .851 r
.87629 .62873 .88342 .64211 .90024 .63911 .89315 .62559 Metetra
.63 .647 .848 r
.89315 .62559 .90024 .63911 .91714 .63576 .91008 .6221 Metetra
.634 .648 .846 r
.91008 .6221 .91714 .63576 .93412 .63212 .92709 .61834 Metetra
.638 .648 .844 r
.92709 .61834 .93412 .63212 .95118 .62825 .94419 .61436 Metetra
.64 .649 .843 r
.94419 .61436 .95118 .62825 .96832 .62419 .96137 .6102 Metetra
.642 .649 .842 r
.96137 .6102 .96832 .62419 .98555 .61998 .97863 .60589 Metetra
.664 .741 .908 r
.37336 .57414 .38141 .58163 .39467 .57758 .38666 .57005 Metetra
.664 .741 .908 r
.38666 .57005 .39467 .57758 .408 .57351 .40003 .56594 Metetra
.664 .741 .908 r
.40003 .56594 .408 .57351 .42139 .56942 .41347 .56182 Metetra
.663 .741 .908 r
.41347 .56182 .42139 .56942 .43485 .56532 .42697 .55767 Metetra
.663 .741 .908 r
.42697 .55767 .43485 .56532 .44838 .56122 .44054 .55353 Metetra
.661 .74 .909 r
.44054 .55353 .44838 .56122 .46198 .55718 .45418 .54943 Metetra
.657 .739 .91 r
.45418 .54943 .46198 .55718 .47564 .55327 .46789 .54545 Metetra
.649 .736 .913 r
.46789 .54545 .47564 .55327 .48937 .54967 .48166 .54175 Metetra
.634 .729 .917 r
.48166 .54175 .48937 .54967 .50317 .5466 .4955 .53854 Metetra
.609 .718 .923 r
.4955 .53854 .50317 .5466 .51704 .54437 .50942 .53613 Metetra
.57 .7 .93 r
.50942 .53613 .51704 .54437 .53101 .5433 .52342 .53484 Metetra
.516 .673 .935 r
.52342 .53484 .53101 .5433 .54508 .54369 .53753 .53496 Metetra
.446 .635 .936 r
.53753 .53496 .54508 .54369 .55928 .54574 .55176 .53674 Metetra
.364 .586 .929 r
.55176 .53674 .55928 .54574 .57364 .54956 .56613 .54027 Metetra
.279 .532 .913 r
.56613 .54027 .57364 .54956 .58816 .55507 .58068 .5455 Metetra
.203 .48 .893 r
.58068 .5455 .58816 .55507 .60289 .56206 .59542 .55223 Metetra
.148 .439 .874 r
.59542 .55223 .60289 .56206 .61781 .57021 .61036 .56013 Metetra
.119 .415 .862 r
.61036 .56013 .61781 .57021 .63295 .57911 .62551 .5688 Metetra
.118 .409 .858 r
.62551 .5688 .63295 .57911 .6483 .58831 .64086 .57779 Metetra
.142 .42 .863 r
.64086 .57779 .6483 .58831 .66384 .59739 .65641 .58666 Metetra
.185 .443 .872 r
.65641 .58666 .66384 .59739 .67956 .60596 .67214 .59502 Metetra
.241 .474 .882 r
.67214 .59502 .67956 .60596 .69543 .61372 .68803 .60257 Metetra
.301 .506 .889 r
.68803 .60257 .69543 .61372 .71145 .62045 .70406 .60907 Metetra
.361 .537 .893 r
.70406 .60907 .71145 .62045 .72758 .62601 .7202 .6144 Metetra
.415 .563 .894 r
.7202 .6144 .72758 .62601 .74382 .63034 .73646 .61849 Metetra
.463 .585 .89 r
.73646 .61849 .74382 .63034 .76014 .63345 .75279 .62137 Metetra
.502 .602 .885 r
.75279 .62137 .76014 .63345 .77654 .63541 .76921 .62309 Metetra
.534 .615 .88 r
.76921 .62309 .77654 .63541 .79301 .6363 .7857 .62375 Metetra
.56 .625 .873 r
.7857 .62375 .79301 .6363 .80954 .63623 .80225 .62346 Metetra
.58 .632 .867 r
.80225 .62346 .80954 .63623 .82613 .63532 .81887 .62235 Metetra
.596 .637 .862 r
.81887 .62235 .82613 .63532 .84278 .6337 .83555 .62054 Metetra
.609 .641 .857 r
.83555 .62054 .84278 .6337 .8595 .63147 .85229 .61813 Metetra
.618 .644 .854 r
.85229 .61813 .8595 .63147 .87629 .62873 .86911 .61523 Metetra
.625 .646 .85 r
.86911 .61523 .87629 .62873 .89315 .62559 .886 .61193 Metetra
.631 .647 .848 r
.886 .61193 .89315 .62559 .91008 .6221 .90296 .60831 Metetra
.635 .648 .845 r
.90296 .60831 .91008 .6221 .92709 .61834 .92001 .60443 Metetra
.639 .649 .844 r
.92001 .60443 .92709 .61834 .94419 .61436 .93714 .60034 Metetra
.641 .649 .842 r
.93714 .60034 .94419 .61436 .96137 .6102 .95436 .59608 Metetra
.643 .649 .841 r
.95436 .59608 .96137 .6102 .97863 .60589 .97166 .59168 Metetra
.664 .741 .908 r
.36524 .56658 .37336 .57414 .38666 .57005 .37858 .56245 Metetra
.664 .741 .908 r
.37858 .56245 .38666 .57005 .40003 .56594 .39199 .55831 Metetra
.664 .741 .908 r
.39199 .55831 .40003 .56594 .41347 .56182 .40547 .55414 Metetra
.663 .741 .908 r
.40547 .55414 .41347 .56182 .42697 .55767 .41902 .54996 Metetra
.663 .741 .908 r
.41902 .54996 .42697 .55767 .44054 .55353 .43263 .54577 Metetra
.662 .741 .909 r
.43263 .54577 .44054 .55353 .45418 .54943 .44631 .54161 Metetra
.658 .739 .91 r
.44631 .54161 .45418 .54943 .46789 .54545 .46006 .53757 Metetra
.651 .736 .912 r
.46006 .53757 .46789 .54545 .48166 .54175 .47388 .53376 Metetra
.637 .731 .916 r
.47388 .53376 .48166 .54175 .4955 .53854 .48776 .53042 Metetra
.613 .72 .922 r
.48776 .53042 .4955 .53854 .50942 .53613 .50172 .52783 Metetra
.577 .704 .928 r
.50172 .52783 .50942 .53613 .52342 .53484 .51577 .52631 Metetra
.525 .678 .934 r
.51577 .52631 .52342 .53484 .53753 .53496 .52991 .52618 Metetra
.458 .641 .936 r
.52991 .52618 .53753 .53496 .55176 .53674 .54417 .52768 Metetra
.377 .594 .93 r
.54417 .52768 .55176 .53674 .56613 .54027 .55857 .53092 Metetra
.293 .54 .916 r
.55857 .53092 .56613 .54027 .58068 .5455 .57313 .53586 Metetra
.217 .489 .897 r
.57313 .53586 .58068 .5455 .59542 .55223 .58789 .54232 Metetra
.161 .448 .878 r
.58789 .54232 .59542 .55223 .61036 .56013 .60284 .54998 Metetra
.131 .423 .866 r
.60284 .54998 .61036 .56013 .62551 .5688 .618 .55841 Metetra
.129 .417 .862 r
.618 .55841 .62551 .5688 .64086 .57779 .63336 .56718 Metetra
.152 .427 .866 r
.63336 .56718 .64086 .57779 .65641 .58666 .64892 .57584 Metetra
.195 .45 .874 r
.64892 .57584 .65641 .58666 .67214 .59502 .66466 .58398 Metetra
.25 .479 .883 r
.66466 .58398 .67214 .59502 .68803 .60257 .68056 .59131 Metetra
.31 .511 .891 r
.68056 .59131 .68803 .60257 .70406 .60907 .6966 .59758 Metetra
.369 .541 .894 r
.6966 .59758 .70406 .60907 .7202 .6144 .71276 .60268 Metetra
.423 .567 .893 r
.71276 .60268 .7202 .6144 .73646 .61849 .72903 .60653 Metetra
.469 .588 .89 r
.72903 .60653 .73646 .61849 .75279 .62137 .74539 .60917 Metetra
.508 .605 .885 r
.74539 .60917 .75279 .62137 .76921 .62309 .76182 .61065 Metetra
.539 .617 .878 r
.76182 .61065 .76921 .62309 .7857 .62375 .77833 .61108 Metetra
.564 .627 .872 r
.77833 .61108 .7857 .62375 .80225 .62346 .7949 .61057 Metetra
.584 .633 .866 r
.7949 .61057 .80225 .62346 .81887 .62235 .81154 .60925 Metetra
.599 .638 .861 r
.81154 .60925 .81887 .62235 .83555 .62054 .82825 .60724 Metetra
.611 .642 .856 r
.82825 .60724 .83555 .62054 .85229 .61813 .84502 .60466 Metetra
.62 .644 .853 r
.84502 .60466 .85229 .61813 .86911 .61523 .86187 .6016 Metetra
.627 .646 .849 r
.86187 .6016 .86911 .61523 .886 .61193 .87879 .59815 Metetra
.632 .647 .847 r
.87879 .59815 .886 .61193 .90296 .60831 .89579 .5944 Metetra
.636 .648 .845 r
.89579 .5944 .90296 .60831 .92001 .60443 .91287 .5904 Metetra
.639 .649 .843 r
.91287 .5904 .92001 .60443 .93714 .60034 .93004 .58619 Metetra
.642 .649 .842 r
.93004 .58619 .93714 .60034 .95436 .59608 .94729 .58183 Metetra
.643 .649 .841 r
.94729 .58183 .95436 .59608 .97166 .59168 .96464 .57734 Metetra
.664 .741 .908 r
.35704 .55894 .36524 .56658 .37858 .56245 .37043 .55478 Metetra
.664 .741 .908 r
.37043 .55478 .37858 .56245 .39199 .55831 .38388 .5506 Metetra
.664 .741 .908 r
.38388 .5506 .39199 .55831 .40547 .55414 .3974 .54639 Metetra
.663 .741 .908 r
.3974 .54639 .40547 .55414 .41902 .54996 .41099 .54217 Metetra
.663 .741 .908 r
.41099 .54217 .41902 .54996 .43263 .54577 .42465 .53794 Metetra
.662 .741 .908 r
.42465 .53794 .43263 .54577 .44631 .54161 .43837 .53373 Metetra
.659 .74 .91 r
.43837 .53373 .44631 .54161 .46006 .53757 .45216 .52961 Metetra
.652 .737 .912 r
.45216 .52961 .46006 .53757 .47388 .53376 .46603 .52571 Metetra
.639 .732 .915 r
.46603 .52571 .47388 .53376 .48776 .53042 .47996 .52224 Metetra
.618 .722 .921 r
.47996 .52224 .48776 .53042 .50172 .52783 .49396 .51947 Metetra
.584 .706 .927 r
.49396 .51947 .50172 .52783 .51577 .52631 .50804 .51774 Metetra
.534 .682 .933 r
.50804 .51774 .51577 .52631 .52991 .52618 .52222 .51734 Metetra
.469 .647 .936 r
.52222 .51734 .52991 .52618 .54417 .52768 .53651 .51855 Metetra
.391 .601 .931 r
.53651 .51855 .54417 .52768 .55857 .53092 .55093 .5215 Metetra
.308 .549 .918 r
.55093 .5215 .55857 .53092 .57313 .53586 .56552 .52615 Metetra
.232 .498 .9 r
.56552 .52615 .57313 .53586 .58789 .54232 .58029 .53234 Metetra
.175 .457 .882 r
.58029 .53234 .58789 .54232 .60284 .54998 .59525 .53974 Metetra
.143 .431 .87 r
.59525 .53974 .60284 .54998 .618 .55841 .61042 .54794 Metetra
.14 .424 .865 r
.61042 .54794 .618 .55841 .63336 .56718 .6258 .55648 Metetra
.163 .434 .869 r
.6258 .55648 .63336 .56718 .64892 .57584 .64137 .56492 Metetra
.205 .456 .877 r
.64137 .56492 .64892 .57584 .66466 .58398 .65712 .57285 Metetra
.26 .485 .885 r
.65712 .57285 .66466 .58398 .68056 .59131 .67303 .57995 Metetra
.319 .516 .892 r
.67303 .57995 .68056 .59131 .6966 .59758 .68908 .58599 Metetra
.378 .546 .894 r
.68908 .58599 .6966 .59758 .71276 .60268 .70526 .59084 Metetra
.43 .571 .893 r
.70526 .59084 .71276 .60268 .72903 .60653 .72154 .59445 Metetra
.476 .591 .889 r
.72154 .59445 .72903 .60653 .74539 .60917 .73792 .59685 Metetra
.514 .607 .884 r
.73792 .59685 .74539 .60917 .76182 .61065 .75437 .59808 Metetra
.544 .619 .877 r
.75437 .59808 .76182 .61065 .77833 .61108 .7709 .59828 Metetra
.569 .628 .871 r
.7709 .59828 .77833 .61108 .7949 .61057 .7875 .59754 Metetra
.587 .635 .865 r
.7875 .59754 .7949 .61057 .81154 .60925 .80416 .59602 Metetra
.602 .639 .86 r
.80416 .59602 .81154 .60925 .82825 .60724 .82089 .59381 Metetra
.613 .643 .855 r
.82089 .59381 .82825 .60724 .84502 .60466 .8377 .59105 Metetra
.622 .645 .852 r
.8377 .59105 .84502 .60466 .86187 .6016 .85457 .58783 Metetra
.629 .646 .849 r
.85457 .58783 .86187 .6016 .87879 .59815 .87152 .58424 Metetra
.634 .647 .846 r
.87152 .58424 .87879 .59815 .89579 .5944 .88856 .58036 Metetra
.637 .648 .844 r
.88856 .58036 .89579 .5944 .91287 .5904 .90567 .57623 Metetra
.64 .649 .843 r
.90567 .57623 .91287 .5904 .93004 .58619 .92288 .57192 Metetra
.642 .649 .841 r
.92288 .57192 .93004 .58619 .94729 .58183 .94017 .56746 Metetra
.644 .649 .84 r
.94017 .56746 .94729 .58183 .96464 .57734 .95756 .56288 Metetra
.664 .741 .908 r
.34877 .55124 .35704 .55894 .37043 .55478 .36219 .54704 Metetra
.664 .741 .908 r
.36219 .54704 .37043 .55478 .38388 .5506 .37569 .54281 Metetra
.664 .741 .908 r
.37569 .54281 .38388 .5506 .3974 .54639 .38925 .53857 Metetra
.663 .741 .908 r
.38925 .53857 .3974 .54639 .41099 .54217 .40288 .5343 Metetra
.663 .741 .908 r
.40288 .5343 .41099 .54217 .42465 .53794 .41658 .53003 Metetra
.662 .741 .908 r
.41658 .53003 .42465 .53794 .43837 .53373 .43035 .52577 Metetra
.659 .74 .909 r
.43035 .52577 .43837 .53373 .45216 .52961 .44419 .52159 Metetra
.654 .737 .911 r
.44419 .52159 .45216 .52961 .46603 .52571 .4581 .5176 Metetra
.642 .733 .915 r
.4581 .5176 .46603 .52571 .47996 .52224 .47207 .514 Metetra
.622 .724 .92 r
.47207 .514 .47996 .52224 .49396 .51947 .48612 .51106 Metetra
.59 .709 .926 r
.48612 .51106 .49396 .51947 .50804 .51774 .50024 .5091 Metetra
.543 .686 .932 r
.50024 .5091 .50804 .51774 .52222 .51734 .51446 .50845 Metetra
.48 .653 .935 r
.51446 .50845 .52222 .51734 .53651 .51855 .52878 .50937 Metetra
.404 .609 .932 r
.52878 .50937 .53651 .51855 .55093 .5215 .54323 .51201 Metetra
.322 .558 .921 r
.54323 .51201 .55093 .5215 .56552 .52615 .55784 .51637 Metetra
.246 .507 .904 r
.55784 .51637 .56552 .52615 .58029 .53234 .57262 .52228 Metetra
.188 .465 .886 r
.57262 .52228 .58029 .53234 .59525 .53974 .5876 .52942 Metetra
.156 .439 .873 r
.5876 .52942 .59525 .53974 .61042 .54794 .60278 .53737 Metetra
.152 .431 .869 r
.60278 .53737 .61042 .54794 .6258 .55648 .61817 .54569 Metetra
.173 .44 .872 r
.61817 .54569 .6258 .55648 .64137 .56492 .63375 .5539 Metetra
.215 .462 .879 r
.63375 .5539 .64137 .56492 .65712 .57285 .64951 .56161 Metetra
.269 .49 .887 r
.64951 .56161 .65712 .57285 .67303 .57995 .66543 .56848 Metetra
.328 .521 .893 r
.66543 .56848 .67303 .57995 .68908 .58599 .6815 .57429 Metetra
.386 .55 .895 r
.6815 .57429 .68908 .58599 .70526 .59084 .69769 .57889 Metetra
.438 .575 .893 r
.69769 .57889 .70526 .59084 .72154 .59445 .71399 .58226 Metetra
.483 .594 .889 r
.71399 .58226 .72154 .59445 .73792 .59685 .73039 .5844 Metetra
.52 .61 .883 r
.73039 .5844 .73792 .59685 .75437 .59808 .74686 .58539 Metetra
.549 .621 .876 r
.74686 .58539 .75437 .59808 .7709 .59828 .76341 .58534 Metetra
.573 .63 .87 r
.76341 .58534 .7709 .59828 .7875 .59754 .78003 .58439 Metetra
.591 .636 .864 r
.78003 .58439 .7875 .59754 .80416 .59602 .79672 .58265 Metetra
.605 .64 .859 r
.79672 .58265 .80416 .59602 .82089 .59381 .81348 .58025 Metetra
.616 .643 .854 r
.81348 .58025 .82089 .59381 .8377 .59105 .83031 .57731 Metetra
.624 .645 .851 r
.83031 .57731 .8377 .59105 .85457 .58783 .84722 .57393 Metetra
.63 .647 .848 r
.84722 .57393 .85457 .58783 .87152 .58424 .8642 .5702 Metetra
.635 .648 .845 r
.8642 .5702 .87152 .58424 .88856 .58036 .88127 .56618 Metetra
.639 .648 .844 r
.88127 .56618 .88856 .58036 .90567 .57623 .89842 .56194 Metetra
.641 .649 .842 r
.89842 .56194 .90567 .57623 .92288 .57192 .91566 .55753 Metetra
.643 .649 .841 r
.91566 .55753 .92288 .57192 .94017 .56746 .933 .55297 Metetra
.644 .649 .84 r
.933 .55297 .94017 .56746 .95756 .56288 .95042 .54829 Metetra
.664 .741 .908 r
.34042 .54346 .34877 .55124 .36219 .54704 .35388 .53922 Metetra
.664 .741 .908 r
.35388 .53922 .36219 .54704 .37569 .54281 .36742 .53496 Metetra
.664 .741 .908 r
.36742 .53496 .37569 .54281 .38925 .53857 .38103 .53067 Metetra
.663 .741 .908 r
.38103 .53067 .38925 .53857 .40288 .5343 .3947 .52637 Metetra
.663 .741 .908 r
.3947 .52637 .40288 .5343 .41658 .53003 .40845 .52205 Metetra
.662 .741 .908 r
.40845 .52205 .41658 .53003 .43035 .52577 .42226 .51774 Metetra
.66 .74 .909 r
.42226 .51774 .43035 .52577 .44419 .52159 .43614 .51349 Metetra
.655 .738 .911 r
.43614 .51349 .44419 .52159 .4581 .5176 .4501 .50942 Metetra
.644 .734 .914 r
.4501 .50942 .4581 .5176 .47207 .514 .46412 .50569 Metetra
.626 .726 .919 r
.46412 .50569 .47207 .514 .48612 .51106 .47821 .50258 Metetra
.596 .712 .925 r
.47821 .50258 .48612 .51106 .50024 .5091 .49237 .5004 Metetra
.551 .69 .931 r
.49237 .5004 .50024 .5091 .51446 .50845 .50662 .49949 Metetra
.491 .658 .935 r
.50662 .49949 .51446 .50845 .52878 .50937 .52098 .50012 Metetra
.417 .616 .933 r
.52098 .50012 .52878 .50937 .54323 .51201 .53546 .50246 Metetra
.336 .566 .923 r
.53546 .50246 .54323 .51201 .55784 .51637 .55009 .50652 Metetra
.261 .516 .907 r
.55009 .50652 .55784 .51637 .57262 .52228 .56489 .51214 Metetra
.202 .474 .89 r
.56489 .51214 .57262 .52228 .5876 .52942 .57989 .51901 Metetra
.169 .447 .877 r
.57989 .51901 .5876 .52942 .60278 .53737 .59508 .52672 Metetra
.163 .439 .872 r
.59508 .52672 .60278 .53737 .61817 .54569 .61047 .5348 Metetra
.184 .447 .874 r
.61047 .5348 .61817 .54569 .63375 .5539 .62607 .54279 Metetra
.225 .468 .881 r
.62607 .54279 .63375 .5539 .64951 .56161 .64184 .55027 Metetra
.279 .496 .888 r
.64184 .55027 .64951 .56161 .66543 .56848 .65778 .5569 Metetra
.337 .526 .893 r
.65778 .5569 .66543 .56848 .6815 .57429 .67386 .56247 Metetra
.394 .554 .895 r
.67386 .56247 .6815 .57429 .69769 .57889 .69007 .56683 Metetra
.446 .578 .893 r
.69007 .56683 .69769 .57889 .71399 .58226 .70638 .56993 Metetra
.489 .597 .888 r
.70638 .56993 .71399 .58226 .73039 .5844 .72279 .57182 Metetra
.525 .612 .882 r
.72279 .57182 .73039 .5844 .74686 .58539 .73929 .57256 Metetra
.554 .623 .875 r
.73929 .57256 .74686 .58539 .76341 .58534 .75586 .57228 Metetra
.577 .631 .869 r
.75586 .57228 .76341 .58534 .78003 .58439 .7725 .57109 Metetra
.594 .637 .863 r
.7725 .57109 .78003 .58439 .79672 .58265 .78922 .56914 Metetra
.608 .641 .858 r
.78922 .56914 .79672 .58265 .81348 .58025 .806 .56655 Metetra
.618 .644 .853 r
.806 .56655 .81348 .58025 .83031 .57731 .82286 .56344 Metetra
.626 .646 .85 r
.82286 .56344 .83031 .57731 .84722 .57393 .8398 .5599 Metetra
.632 .647 .847 r
.8398 .5599 .84722 .57393 .8642 .5702 .85682 .55602 Metetra
.636 .648 .845 r
.85682 .55602 .8642 .5702 .88127 .56618 .87392 .55188 Metetra
.639 .648 .843 r
.87392 .55188 .88127 .56618 .89842 .56194 .89111 .54752 Metetra
.642 .649 .842 r
.89111 .54752 .89842 .56194 .91566 .55753 .90839 .543 Metetra
.644 .649 .841 r
.90839 .543 .91566 .55753 .933 .55297 .92576 .53834 Metetra
.645 .649 .84 r
.92576 .53834 .933 .55297 .95042 .54829 .94323 .53358 Metetra
.664 .741 .908 r
.33199 .53562 .34042 .54346 .35388 .53922 .3455 .53133 Metetra
.664 .741 .908 r
.3455 .53133 .35388 .53922 .36742 .53496 .35908 .52703 Metetra
.664 .741 .908 r
.35908 .52703 .36742 .53496 .38103 .53067 .37272 .5227 Metetra
.663 .741 .908 r
.37272 .5227 .38103 .53067 .3947 .52637 .38644 .51836 Metetra
.663 .741 .908 r
.38644 .51836 .3947 .52637 .40845 .52205 .40023 .514 Metetra
.663 .741 .908 r
.40023 .514 .40845 .52205 .42226 .51774 .41409 .50964 Metetra
.661 .74 .909 r
.41409 .50964 .42226 .51774 .43614 .51349 .42802 .50533 Metetra
.656 .738 .91 r
.42802 .50533 .43614 .51349 .4501 .50942 .44202 .50116 Metetra
.647 .735 .913 r
.44202 .50116 .4501 .50942 .46412 .50569 .45608 .49732 Metetra
.63 .727 .918 r
.45608 .49732 .46412 .50569 .47821 .50258 .47022 .49404 Metetra
.602 .714 .924 r
.47022 .49404 .47821 .50258 .49237 .5004 .48443 .49165 Metetra
.56 .694 .93 r
.48443 .49165 .49237 .5004 .50662 .49949 .49872 .49048 Metetra
.502 .664 .934 r
.49872 .49048 .50662 .49949 .52098 .50012 .51311 .49081 Metetra
.43 .623 .933 r
.51311 .49081 .52098 .50012 .53546 .50246 .52762 .49285 Metetra
.351 .575 .924 r
.52762 .49285 .53546 .50246 .55009 .50652 .54227 .4966 Metetra
.275 .525 .91 r
.54227 .4966 .55009 .50652 .56489 .51214 .55709 .50192 Metetra
.216 .483 .893 r
.55709 .50192 .56489 .51214 .57989 .51901 .5721 .50852 Metetra
.181 .455 .881 r
.5721 .50852 .57989 .51901 .59508 .52672 .58731 .51598 Metetra
.175 .446 .875 r
.58731 .51598 .59508 .52672 .61047 .5348 .60271 .52382 Metetra
.195 .454 .877 r
.60271 .52382 .61047 .5348 .62607 .54279 .61832 .53158 Metetra
.235 .474 .883 r
.61832 .53158 .62607 .54279 .64184 .55027 .6341 .53882 Metetra
.288 .502 .89 r
.6341 .53882 .64184 .55027 .65778 .5569 .65005 .54522 Metetra
.346 .531 .894 r
.65005 .54522 .65778 .5569 .67386 .56247 .66615 .55054 Metetra
.403 .559 .895 r
.66615 .55054 .67386 .56247 .69007 .56683 .68237 .55464 Metetra
.453 .582 .892 r
.68237 .55464 .69007 .56683 .70638 .56993 .69871 .55749 Metetra
.496 .6 .887 r
.69871 .55749 .70638 .56993 .72279 .57182 .71514 .55911 Metetra
.531 .615 .881 r
.71514 .55911 .72279 .57182 .73929 .57256 .73165 .5596 Metetra
.559 .625 .874 r
.73165 .5596 .73929 .57256 .75586 .57228 .74824 .55907 Metetra
.581 .633 .868 r
.74824 .55907 .75586 .57228 .7725 .57109 .76491 .55766 Metetra
.598 .638 .862 r
.76491 .55766 .7725 .57109 .78922 .56914 .78165 .5555 Metetra
.611 .642 .857 r
.78165 .5555 .78922 .56914 .806 .56655 .79847 .55271 Metetra
.621 .644 .852 r
.79847 .55271 .806 .56655 .82286 .56344 .81536 .54943 Metetra
.628 .646 .849 r
.81536 .54943 .82286 .56344 .8398 .5599 .83233 .54573 Metetra
.633 .647 .846 r
.83233 .54573 .8398 .5599 .85682 .55602 .84938 .54171 Metetra
.637 .648 .844 r
.84938 .54171 .85682 .55602 .87392 .55188 .86652 .53744 Metetra
.64 .649 .842 r
.86652 .53744 .87392 .55188 .89111 .54752 .88374 .53297 Metetra
.643 .649 .841 r
.88374 .53297 .89111 .54752 .90839 .543 .90106 .52834 Metetra
.644 .649 .84 r
.90106 .52834 .90839 .543 .92576 .53834 .91847 .52359 Metetra
.645 .649 .839 r
.91847 .52359 .92576 .53834 .94323 .53358 .93598 .51874 Metetra
.664 .741 .908 r
.32348 .52769 .33199 .53562 .3455 .53133 .33703 .52337 Metetra
.664 .741 .908 r
.33703 .52337 .3455 .53133 .35908 .52703 .35065 .51903 Metetra
.664 .741 .908 r
.35065 .51903 .35908 .52703 .37272 .5227 .36434 .51466 Metetra
.664 .741 .908 r
.36434 .51466 .37272 .5227 .38644 .51836 .37811 .51027 Metetra
.663 .741 .908 r
.37811 .51027 .38644 .51836 .40023 .514 .39194 .50586 Metetra
.663 .741 .908 r
.39194 .50586 .40023 .514 .41409 .50964 .40584 .50146 Metetra
.661 .74 .909 r
.40584 .50146 .41409 .50964 .42802 .50533 .41982 .49709 Metetra
.657 .739 .91 r
.41982 .49709 .42802 .50533 .44202 .50116 .43386 .49284 Metetra
.649 .735 .913 r
.43386 .49284 .44202 .50116 .45608 .49732 .44797 .48888 Metetra
.633 .729 .917 r
.44797 .48888 .45608 .49732 .47022 .49404 .46215 .48544 Metetra
.607 .717 .923 r
.46215 .48544 .47022 .49404 .48443 .49165 .47641 .48284 Metetra
.568 .698 .929 r
.47641 .48284 .48443 .49165 .49872 .49048 .49074 .4814 Metetra
.513 .669 .934 r
.49074 .4814 .49872 .49048 .51311 .49081 .50516 .48145 Metetra
.443 .63 .933 r
.50516 .48145 .51311 .49081 .52762 .49285 .51971 .48316 Metetra
.365 .583 .926 r
.51971 .48316 .52762 .49285 .54227 .4966 .53438 .4866 Metetra
.29 .534 .912 r
.53438 .4866 .54227 .4966 .55709 .50192 .54923 .49163 Metetra
.23 .492 .896 r
.54923 .49163 .55709 .50192 .5721 .50852 .56425 .49795 Metetra
.194 .464 .884 r
.56425 .49795 .5721 .50852 .58731 .51598 .57947 .50515 Metetra
.187 .454 .878 r
.57947 .50515 .58731 .51598 .60271 .52382 .59489 .51275 Metetra
.206 .46 .88 r
.59489 .51275 .60271 .52382 .61832 .53158 .6105 .52026 Metetra
.246 .48 .885 r
.6105 .52026 .61832 .53158 .6341 .53882 .6263 .52727 Metetra
.298 .507 .891 r
.6263 .52727 .6341 .53882 .65005 .54522 .64226 .53343 Metetra
.355 .536 .895 r
.64226 .53343 .65005 .54522 .66615 .55054 .65838 .53849 Metetra
.411 .563 .895 r
.65838 .53849 .66615 .55054 .68237 .55464 .67462 .54233 Metetra
.461 .586 .892 r
.67462 .54233 .68237 .55464 .69871 .55749 .69097 .54491 Metetra
.503 .603 .886 r
.69097 .54491 .69871 .55749 .71514 .55911 .70742 .54627 Metetra
.537 .617 .88 r
.70742 .54627 .71514 .55911 .73165 .5596 .72395 .54651 Metetra
.564 .627 .873 r
.72395 .54651 .73165 .5596 .74824 .55907 .74057 .54573 Metetra
.585 .634 .866 r
.74057 .54573 .74824 .55907 .76491 .55766 .75726 .54409 Metetra
.601 .639 .86 r
.75726 .54409 .76491 .55766 .78165 .5555 .77403 .54171 Metetra
.614 .643 .856 r
.77403 .54171 .78165 .5555 .79847 .55271 .79087 .53874 Metetra
.623 .645 .851 r
.79087 .53874 .79847 .55271 .81536 .54943 .80779 .53527 Metetra
.63 .647 .848 r
.80779 .53527 .81536 .54943 .83233 .54573 .82479 .53142 Metetra
.635 .648 .846 r
.82479 .53142 .83233 .54573 .84938 .54171 .84188 .52727 Metetra
.638 .648 .843 r
.84188 .52727 .84938 .54171 .86652 .53744 .85905 .52287 Metetra
.641 .649 .842 r
.85905 .52287 .86652 .53744 .88374 .53297 .87631 .51829 Metetra
.643 .649 .841 r
.87631 .51829 .88374 .53297 .90106 .52834 .89367 .51356 Metetra
.645 .649 .84 r
.89367 .51356 .90106 .52834 .91847 .52359 .91113 .50871 Metetra
.646 .649 .839 r
.91113 .50871 .91847 .52359 .93598 .51874 .92868 .50377 Metetra
.664 .741 .908 r
.31489 .5197 .32348 .52769 .33703 .52337 .32848 .51533 Metetra
.664 .741 .908 r
.32848 .51533 .33703 .52337 .35065 .51903 .34215 .51095 Metetra
.664 .741 .908 r
.34215 .51095 .35065 .51903 .36434 .51466 .35588 .50654 Metetra
.664 .741 .908 r
.35588 .50654 .36434 .51466 .37811 .51027 .36969 .50211 Metetra
.663 .741 .908 r
.36969 .50211 .37811 .51027 .39194 .50586 .38357 .49766 Metetra
.663 .741 .908 r
.38357 .49766 .39194 .50586 .40584 .50146 .39752 .4932 Metetra
.662 .741 .909 r
.39752 .4932 .40584 .50146 .41982 .49709 .41154 .48877 Metetra
.658 .739 .91 r
.41154 .48877 .41982 .49709 .43386 .49284 .42563 .48444 Metetra
.651 .736 .912 r
.42563 .48444 .43386 .49284 .44797 .48888 .43979 .48037 Metetra
.637 .73 .916 r
.43979 .48037 .44797 .48888 .46215 .48544 .45401 .47677 Metetra
.612 .719 .921 r
.45401 .47677 .46215 .48544 .47641 .48284 .46831 .47396 Metetra
.575 .701 .928 r
.46831 .47396 .47641 .48284 .49074 .4814 .48268 .47227 Metetra
.523 .674 .933 r
.48268 .47227 .49074 .4814 .50516 .48145 .49715 .47202 Metetra
.456 .637 .934 r
.49715 .47202 .50516 .48145 .51971 .48316 .51172 .47342 Metetra
.379 .591 .927 r
.51172 .47342 .51971 .48316 .53438 .4866 .52643 .47653 Metetra
.305 .543 .915 r
.52643 .47653 .53438 .4866 .54923 .49163 .54129 .48125 Metetra
.244 .5 .9 r
.54129 .48125 .54923 .49163 .56425 .49795 .55633 .48729 Metetra
.207 .472 .887 r
.55633 .48729 .56425 .49795 .57947 .50515 .57156 .49422 Metetra
.199 .461 .881 r
.57156 .49422 .57947 .50515 .59489 .51275 .58699 .50157 Metetra
.217 .467 .882 r
.58699 .50157 .59489 .51275 .6105 .52026 .60262 .50885 Metetra
.256 .486 .887 r
.60262 .50885 .6105 .52026 .6263 .52727 .61843 .51561 Metetra
.308 .513 .892 r
.61843 .51561 .6263 .52727 .64226 .53343 .63441 .52152 Metetra
.365 .541 .895 r
.63441 .52152 .64226 .53343 .65838 .53849 .65054 .52632 Metetra
.42 .567 .895 r
.65054 .52632 .65838 .53849 .67462 .54233 .66679 .5299 Metetra
.468 .589 .891 r
.66679 .5299 .67462 .54233 .69097 .54491 .68316 .53221 Metetra
.509 .606 .885 r
.68316 .53221 .69097 .54491 .70742 .54627 .69963 .5333 Metetra
.543 .619 .878 r
.69963 .5333 .70742 .54627 .72395 .54651 .71619 .53327 Metetra
.569 .629 .871 r
.71619 .53327 .72395 .54651 .74057 .54573 .73283 .53225 Metetra
.589 .635 .865 r
.73283 .53225 .74057 .54573 .75726 .54409 .74954 .53037 Metetra
.605 .64 .859 r
.74954 .53037 .75726 .54409 .77403 .54171 .76634 .52779 Metetra
.616 .643 .854 r
.76634 .52779 .77403 .54171 .79087 .53874 .78321 .52462 Metetra
.625 .645 .85 r
.78321 .52462 .79087 .53874 .80779 .53527 .80016 .52098 Metetra
.631 .647 .847 r
.80016 .52098 .80779 .53527 .82479 .53142 .8172 .51698 Metetra
.636 .648 .845 r
.8172 .51698 .82479 .53142 .84188 .52727 .83432 .51268 Metetra
.64 .648 .843 r
.83432 .51268 .84188 .52727 .85905 .52287 .85153 .50816 Metetra
.642 .649 .841 r
.85153 .50816 .85905 .52287 .87631 .51829 .86883 .50347 Metetra
.644 .649 .84 r
.86883 .50347 .87631 .51829 .89367 .51356 .88623 .49864 Metetra
.645 .649 .839 r
.88623 .49864 .89367 .51356 .91113 .50871 .90372 .4937 Metetra
.646 .649 .839 r
.90372 .4937 .91113 .50871 .92868 .50377 .92131 .48867 Metetra
.664 .741 .908 r
.30622 .51162 .31489 .5197 .32848 .51533 .31985 .50722 Metetra
.664 .741 .908 r
.31985 .50722 .32848 .51533 .34215 .51095 .33356 .50279 Metetra
.664 .741 .908 r
.33356 .50279 .34215 .51095 .35588 .50654 .34734 .49834 Metetra
.664 .741 .908 r
.34734 .49834 .35588 .50654 .36969 .50211 .36119 .49386 Metetra
.663 .741 .908 r
.36119 .49386 .36969 .50211 .38357 .49766 .37511 .48937 Metetra
.663 .741 .908 r
.37511 .48937 .38357 .49766 .39752 .4932 .38911 .48486 Metetra
.662 .741 .908 r
.38911 .48486 .39752 .4932 .41154 .48877 .40317 .48038 Metetra
.659 .74 .909 r
.40317 .48038 .41154 .48877 .42563 .48444 .41731 .47598 Metetra
.653 .737 .911 r
.41731 .47598 .42563 .48444 .43979 .48037 .43152 .47179 Metetra
.64 .731 .915 r
.43152 .47179 .43979 .48037 .45401 .47677 .44579 .46804 Metetra
.618 .721 .92 r
.44579 .46804 .45401 .47677 .46831 .47396 .46014 .46503 Metetra
.583 .705 .926 r
.46014 .46503 .46831 .47396 .48268 .47227 .47455 .46308 Metetra
.533 .679 .932 r
.47455 .46308 .48268 .47227 .49715 .47202 .48905 .46252 Metetra
.468 .643 .933 r
.48905 .46252 .49715 .47202 .51172 .47342 .50366 .4636 Metetra
.394 .599 .929 r
.50366 .4636 .51172 .47342 .52643 .47653 .51839 .46639 Metetra
.319 .551 .917 r
.51839 .46639 .52643 .47653 .54129 .48125 .53328 .4708 Metetra
.258 .509 .903 r
.53328 .4708 .54129 .48125 .55633 .48729 .54834 .47654 Metetra
.22 .48 .89 r
.54834 .47654 .55633 .48729 .57156 .49422 .56359 .4832 Metetra
.211 .468 .884 r
.56359 .4832 .57156 .49422 .58699 .50157 .57903 .4903 Metetra
.228 .474 .885 r
.57903 .4903 .58699 .50157 .60262 .50885 .59467 .49733 Metetra
.266 .493 .889 r
.59467 .49733 .60262 .50885 .61843 .51561 .61049 .50385 Metetra
.318 .518 .894 r
.61049 .50385 .61843 .51561 .63441 .52152 .62649 .5095 Metetra
.374 .546 .896 r
.62649 .5095 .63441 .52152 .65054 .52632 .64263 .51404 Metetra
.428 .571 .895 r
.64263 .51404 .65054 .52632 .66679 .5299 .6589 .51734 Metetra
.476 .593 .891 r
.6589 .51734 .66679 .5299 .68316 .53221 .67529 .51938 Metetra
.516 .609 .884 r
.67529 .51938 .68316 .53221 .69963 .5333 .69178 .52019 Metetra
.548 .622 .877 r
.69178 .52019 .69963 .5333 .71619 .53327 .70836 .5199 Metetra
.574 .631 .87 r
.70836 .5199 .71619 .53327 .73283 .53225 .72502 .51862 Metetra
.593 .637 .864 r
.72502 .51862 .73283 .53225 .74954 .53037 .74177 .51652 Metetra
.608 .641 .858 r
.74177 .51652 .74954 .53037 .76634 .52779 .75859 .51372 Metetra
.619 .644 .853 r
.75859 .51372 .76634 .52779 .78321 .52462 .77549 .51036 Metetra
.627 .646 .849 r
.77549 .51036 .78321 .52462 .80016 .52098 .79247 .50655 Metetra
.633 .647 .846 r
.79247 .50655 .80016 .52098 .8172 .51698 .80954 .50239 Metetra
.637 .648 .844 r
.80954 .50239 .8172 .51698 .83432 .51268 .8267 .49796 Metetra
.641 .648 .842 r
.8267 .49796 .83432 .51268 .85153 .50816 .84394 .49332 Metetra
.643 .649 .841 r
.84394 .49332 .85153 .50816 .86883 .50347 .86128 .48852 Metetra
.644 .649 .84 r
.86128 .48852 .86883 .50347 .88623 .49864 .87872 .48359 Metetra
.646 .649 .839 r
.87872 .48359 .88623 .49864 .90372 .4937 .89626 .47856 Metetra
.646 .649 .838 r
.89626 .47856 .90372 .4937 .92131 .48867 .91389 .47345 Metetra
.664 .741 .908 r
.29746 .50347 .30622 .51162 .31985 .50722 .31114 .49902 Metetra
.664 .741 .908 r
.31114 .49902 .31985 .50722 .33356 .50279 .32489 .49455 Metetra
.664 .741 .908 r
.32489 .49455 .33356 .50279 .34734 .49834 .33872 .49006 Metetra
.664 .741 .908 r
.33872 .49006 .34734 .49834 .36119 .49386 .35261 .48554 Metetra
.663 .741 .908 r
.35261 .48554 .36119 .49386 .37511 .48937 .36658 .481 Metetra
.663 .741 .908 r
.36658 .481 .37511 .48937 .38911 .48486 .38062 .47645 Metetra
.662 .741 .908 r
.38062 .47645 .38911 .48486 .40317 .48038 .39473 .47191 Metetra
.66 .74 .909 r
.39473 .47191 .40317 .48038 .41731 .47598 .40892 .46743 Metetra
.654 .738 .911 r
.40892 .46743 .41731 .47598 .43152 .47179 .42317 .46314 Metetra
.643 .733 .914 r
.42317 .46314 .43152 .47179 .44579 .46804 .43749 .45925 Metetra
.622 .724 .919 r
.43749 .45925 .44579 .46804 .46014 .46503 .45188 .45603 Metetra
.59 .708 .925 r
.45188 .45603 .46014 .46503 .47455 .46308 .46634 .45382 Metetra
.543 .684 .931 r
.46634 .45382 .47455 .46308 .48905 .46252 .48089 .45297 Metetra
.481 .65 .933 r
.48089 .45297 .48905 .46252 .50366 .4636 .49553 .45372 Metetra
.408 .607 .929 r
.49553 .45372 .50366 .4636 .51839 .46639 .51029 .45618 Metetra
.334 .56 .919 r
.51029 .45618 .51839 .46639 .53328 .4708 .5252 .46027 Metetra
.272 .518 .905 r
.5252 .46027 .53328 .4708 .54834 .47654 .54028 .46571 Metetra
.234 .488 .893 r
.54028 .46571 .54834 .47654 .56359 .4832 .55554 .47209 Metetra
.223 .476 .887 r
.55554 .47209 .56359 .4832 .57903 .4903 .571 .47893 Metetra
.239 .481 .887 r
.571 .47893 .57903 .4903 .59467 .49733 .58665 .48571 Metetra
.277 .499 .891 r
.58665 .48571 .59467 .49733 .61049 .50385 .60249 .49197 Metetra
.328 .524 .895 r
.60249 .49197 .61049 .50385 .62649 .5095 .6185 .49736 Metetra
.383 .551 .896 r
.6185 .49736 .62649 .5095 .64263 .51404 .63466 .50163 Metetra
.437 .576 .895 r
.63466 .50163 .64263 .51404 .6589 .51734 .65095 .50465 Metetra
.484 .596 .89 r
.65095 .50465 .6589 .51734 .67529 .51938 .66736 .50641 Metetra
.523 .612 .883 r
.66736 .50641 .67529 .51938 .69178 .52019 .68387 .50695 Metetra
.554 .624 .876 r
.68387 .50695 .69178 .52019 .70836 .5199 .70047 .50638 Metetra
.578 .632 .869 r
.70047 .50638 .70836 .5199 .72502 .51862 .71716 .50485 Metetra
.597 .638 .862 r
.71716 .50485 .72502 .51862 .74177 .51652 .73393 .50251 Metetra
.611 .642 .857 r
.73393 .50251 .74177 .51652 .75859 .51372 .75078 .4995 Metetra
.621 .645 .852 r
.75078 .4995 .75859 .51372 .77549 .51036 .76771 .49595 Metetra
.629 .646 .848 r
.76771 .49595 .77549 .51036 .79247 .50655 .78472 .49197 Metetra
.635 .647 .846 r
.78472 .49197 .79247 .50655 .80954 .50239 .80182 .48766 Metetra
.639 .648 .843 r
.80182 .48766 .80954 .50239 .8267 .49796 .81902 .4831 Metetra
.641 .649 .842 r
.81902 .4831 .8267 .49796 .84394 .49332 .8363 .47834 Metetra
.643 .649 .84 r
.8363 .47834 .84394 .49332 .86128 .48852 .85368 .47343 Metetra
.645 .649 .839 r
.85368 .47343 .86128 .48852 .87872 .48359 .87116 .4684 Metetra
.646 .649 .839 r
.87116 .4684 .87872 .48359 .89626 .47856 .88873 .46329 Metetra
.647 .649 .838 r
.88873 .46329 .89626 .47856 .91389 .47345 .90641 .45809 Metetra
.664 .741 .908 r
.28862 .49524 .29746 .50347 .31114 .49902 .30235 .49075 Metetra
.664 .741 .908 r
.30235 .49075 .31114 .49902 .32489 .49455 .31614 .48624 Metetra
.664 .741 .908 r
.31614 .48624 .32489 .49455 .33872 .49006 .33001 .4817 Metetra
.664 .741 .908 r
.33001 .4817 .33872 .49006 .35261 .48554 .34395 .47714 Metetra
.663 .741 .908 r
.34395 .47714 .35261 .48554 .36658 .481 .35796 .47255 Metetra
.663 .741 .908 r
.35796 .47255 .36658 .481 .38062 .47645 .37204 .46795 Metetra
.663 .741 .908 r
.37204 .46795 .38062 .47645 .39473 .47191 .3862 .46336 Metetra
.661 .74 .909 r
.3862 .46336 .39473 .47191 .40892 .46743 .40044 .45881 Metetra
.656 .738 .91 r
.40044 .45881 .40892 .46743 .42317 .46314 .41474 .45442 Metetra
.645 .734 .913 r
.41474 .45442 .42317 .46314 .43749 .45925 .42911 .45038 Metetra
.627 .725 .918 r
.42911 .45038 .43749 .45925 .45188 .45603 .44355 .44697 Metetra
.597 .711 .924 r
.44355 .44697 .45188 .45603 .46634 .45382 .45806 .44451 Metetra
.552 .689 .93 r
.45806 .44451 .46634 .45382 .48089 .45297 .47264 .44336 Metetra
.493 .656 .933 r
.47264 .44336 .48089 .45297 .49553 .45372 .48732 .44378 Metetra
.422 .615 .93 r
.48732 .44378 .49553 .45372 .51029 .45618 .50211 .4459 Metetra
.349 .569 .921 r
.50211 .4459 .51029 .45618 .5252 .46027 .51705 .44965 Metetra
.287 .527 .908 r
.51705 .44965 .5252 .46027 .54028 .46571 .53214 .45479 Metetra
.247 .497 .896 r
.53214 .45479 .54028 .46571 .55554 .47209 .54742 .46089 Metetra
.235 .483 .89 r
.54742 .46089 .55554 .47209 .571 .47893 .5629 .46746 Metetra
.25 .488 .889 r
.5629 .46746 .571 .47893 .58665 .48571 .57856 .47398 Metetra
.288 .505 .892 r
.57856 .47398 .58665 .48571 .60249 .49197 .59442 .47999 Metetra
.338 .53 .896 r
.59442 .47999 .60249 .49197 .6185 .49736 .61044 .48511 Metetra
.393 .556 .897 r
.61044 .48511 .6185 .49736 .63466 .50163 .62662 .4891 Metetra
.445 .58 .894 r
.62662 .4891 .63466 .50163 .65095 .50465 .64293 .49184 Metetra
.491 .6 .889 r
.64293 .49184 .65095 .50465 .66736 .50641 .65935 .4933 Metetra
.529 .615 .882 r
.65935 .4933 .66736 .50641 .68387 .50695 .67589 .49355 Metetra
.56 .626 .875 r
.67589 .49355 .68387 .50695 .70047 .50638 .69251 .49272 Metetra
.583 .634 .867 r
.69251 .49272 .70047 .50638 .71716 .50485 .70922 .49093 Metetra
.601 .639 .861 r
.70922 .49093 .71716 .50485 .73393 .50251 .72602 .48836 Metetra
.614 .643 .856 r
.72602 .48836 .73393 .50251 .75078 .4995 .7429 .48513 Metetra
.624 .645 .851 r
.7429 .48513 .75078 .4995 .76771 .49595 .75986 .48139 Metetra
.631 .647 .848 r
.75986 .48139 .76771 .49595 .78472 .49197 .77691 .47725 Metetra
.636 .648 .845 r
.77691 .47725 .78472 .49197 .80182 .48766 .79404 .4728 Metetra
.64 .648 .843 r
.79404 .4728 .80182 .48766 .81902 .4831 .81127 .4681 Metetra
.642 .649 .841 r
.81127 .4681 .81902 .4831 .8363 .47834 .82859 .46323 Metetra
.644 .649 .84 r
.82859 .46323 .8363 .47834 .85368 .47343 .84601 .45821 Metetra
.645 .649 .839 r
.84601 .45821 .85368 .47343 .87116 .4684 .86353 .45309 Metetra
.646 .649 .838 r
.86353 .45309 .87116 .4684 .88873 .46329 .88115 .44788 Metetra
.647 .649 .838 r
.88115 .44788 .88873 .46329 .90641 .45809 .89887 .4426 Metetra
.664 .741 .908 r
.2797 .48693 .28862 .49524 .30235 .49075 .29347 .4824 Metetra
.664 .741 .908 r
.29347 .4824 .30235 .49075 .31614 .48624 .3073 .47784 Metetra
.664 .741 .908 r
.3073 .47784 .31614 .48624 .33001 .4817 .32121 .47326 Metetra
.664 .741 .908 r
.32121 .47326 .33001 .4817 .34395 .47714 .3352 .46865 Metetra
.664 .741 .908 r
.3352 .46865 .34395 .47714 .35796 .47255 .34926 .46402 Metetra
.663 .741 .908 r
.34926 .46402 .35796 .47255 .37204 .46795 .36339 .45938 Metetra
.663 .741 .908 r
.36339 .45938 .37204 .46795 .3862 .46336 .3776 .45472 Metetra
.661 .74 .909 r
.3776 .45472 .3862 .46336 .40044 .45881 .39188 .45011 Metetra
.657 .739 .91 r
.39188 .45011 .40044 .45881 .41474 .45442 .40623 .44563 Metetra
.648 .735 .913 r
.40623 .44563 .41474 .45442 .42911 .45038 .42065 .44145 Metetra
.631 .727 .917 r
.42065 .44145 .42911 .45038 .44355 .44697 .43513 .43784 Metetra
.603 .714 .923 r
.43513 .43784 .44355 .44697 .45806 .44451 .44969 .43514 Metetra
.561 .693 .929 r
.44969 .43514 .45806 .44451 .47264 .44336 .46432 .43368 Metetra
.505 .662 .932 r
.46432 .43368 .47264 .44336 .48732 .44378 .47903 .43377 Metetra
.436 .622 .931 r
.47903 .43377 .48732 .44378 .50211 .4459 .49386 .43554 Metetra
.364 .577 .923 r
.49386 .43554 .50211 .4459 .51705 .44965 .50882 .43896 Metetra
.301 .535 .911 r
.50882 .43896 .51705 .44965 .53214 .45479 .52394 .44378 Metetra
.26 .505 .899 r
.52394 .44378 .53214 .45479 .54742 .46089 .53923 .44959 Metetra
.247 .491 .892 r
.53923 .44959 .54742 .46089 .5629 .46746 .55472 .45589 Metetra
.262 .494 .891 r
.55472 .45589 .5629 .46746 .57856 .47398 .5704 .46214 Metetra
.298 .511 .894 r
.5704 .46214 .57856 .47398 .59442 .47999 .58627 .46788 Metetra
.348 .535 .897 r
.58627 .46788 .59442 .47999 .61044 .48511 .60231 .47273 Metetra
.402 .561 .897 r
.60231 .47273 .61044 .48511 .62662 .4891 .61851 .47644 Metetra
.454 .584 .894 r
.61851 .47644 .62662 .4891 .64293 .49184 .63484 .47888 Metetra
.499 .603 .888 r
.63484 .47888 .64293 .49184 .65935 .4933 .65128 .48005 Metetra
.536 .618 .881 r
.65128 .48005 .65935 .4933 .67589 .49355 .66784 .48002 Metetra
.565 .628 .873 r
.66784 .48002 .67589 .49355 .69251 .49272 .68449 .47891 Metetra
.588 .636 .866 r
.68449 .47891 .69251 .49272 .70922 .49093 .70123 .47686 Metetra
.605 .64 .86 r
.70123 .47686 .70922 .49093 .72602 .48836 .71805 .47405 Metetra
.617 .644 .854 r
.71805 .47405 .72602 .48836 .7429 .48513 .73496 .47062 Metetra
.626 .646 .85 r
.73496 .47062 .7429 .48513 .75986 .48139 .75195 .46669 Metetra
.633 .647 .847 r
.75195 .46669 .75986 .48139 .77691 .47725 .76903 .46238 Metetra
.637 .648 .844 r
.76903 .46238 .77691 .47725 .79404 .4728 .7862 .45778 Metetra
.641 .648 .842 r
.7862 .45778 .79404 .4728 .81127 .4681 .80347 .45296 Metetra
.643 .649 .84 r
.80347 .45296 .81127 .4681 .82859 .46323 .82083 .44797 Metetra
.645 .649 .839 r
.82083 .44797 .82859 .46323 .84601 .45821 .83829 .44286 Metetra
.646 .649 .839 r
.83829 .44286 .84601 .45821 .86353 .45309 .85584 .43764 Metetra
.647 .649 .838 r
.85584 .43764 .86353 .45309 .88115 .44788 .87351 .43234 Metetra
.647 .649 .838 r
.87351 .43234 .88115 .44788 .89887 .4426 .89127 .42698 Metetra
.664 .741 .908 r
.27069 .47854 .2797 .48693 .29347 .4824 .2845 .47397 Metetra
.664 .741 .908 r
.2845 .47397 .29347 .4824 .3073 .47784 .29838 .46936 Metetra
.664 .741 .908 r
.29838 .46936 .3073 .47784 .32121 .47326 .31234 .46474 Metetra
.664 .741 .908 r
.31234 .46474 .32121 .47326 .3352 .46865 .32636 .46009 Metetra
.664 .741 .908 r
.32636 .46009 .3352 .46865 .34926 .46402 .34047 .45541 Metetra
.663 .741 .908 r
.34047 .45541 .34926 .46402 .36339 .45938 .35465 .45071 Metetra
.663 .741 .908 r
.35465 .45071 .36339 .45938 .3776 .45472 .3689 .44601 Metetra
.662 .741 .909 r
.3689 .44601 .3776 .45472 .39188 .45011 .38323 .44133 Metetra
.658 .739 .91 r
.38323 .44133 .39188 .45011 .40623 .44563 .39763 .43676 Metetra
.65 .736 .912 r
.39763 .43676 .40623 .44563 .42065 .44145 .4121 .43245 Metetra
.635 .729 .916 r
.4121 .43245 .42065 .44145 .43513 .43784 .42663 .42866 Metetra
.61 .717 .921 r
.42663 .42866 .43513 .43784 .44969 .43514 .44124 .4257 Metetra
.57 .697 .927 r
.44124 .4257 .44969 .43514 .46432 .43368 .45591 .42395 Metetra
.516 .668 .932 r
.45591 .42395 .46432 .43368 .47903 .43377 .47067 .42369 Metetra
.45 .63 .931 r
.47067 .42369 .47903 .43377 .49386 .43554 .48553 .42511 Metetra
.379 .586 .924 r
.48553 .42511 .49386 .43554 .50882 .43896 .50051 .42818 Metetra
.316 .544 .913 r
.50051 .42818 .50882 .43896 .52394 .44378 .51566 .43268 Metetra
.274 .513 .902 r
.51566 .43268 .52394 .44378 .53923 .44959 .53097 .43819 Metetra
.26 .499 .895 r
.53097 .43819 .53923 .44959 .55472 .45589 .54648 .44421 Metetra
.273 .501 .894 r
.54648 .44421 .55472 .45589 .5704 .46214 .56217 .4502 Metetra
.309 .517 .895 r
.56217 .4502 .5704 .46214 .58627 .46788 .57806 .45567 Metetra
.358 .541 .897 r
.57806 .45567 .58627 .46788 .60231 .47273 .59411 .46023 Metetra
.412 .566 .897 r
.59411 .46023 .60231 .47273 .61851 .47644 .61033 .46365 Metetra
.463 .588 .893 r
.61033 .46365 .61851 .47644 .63484 .47888 .62668 .46579 Metetra
.507 .607 .887 r
.62668 .46579 .63484 .47888 .65128 .48005 .64315 .46666 Metetra
.543 .62 .879 r
.64315 .46666 .65128 .48005 .66784 .48002 .65972 .46633 Metetra
.571 .63 .872 r
.65972 .46633 .66784 .48002 .68449 .47891 .6764 .46494 Metetra
.592 .637 .864 r
.6764 .46494 .68449 .47891 .70123 .47686 .69316 .46264 Metetra
.608 .642 .858 r
.69316 .46264 .70123 .47686 .71805 .47405 .71001 .45959 Metetra
.62 .645 .853 r
.71001 .45959 .71805 .47405 .73496 .47062 .72695 .45595 Metetra
.628 .646 .849 r
.72695 .45595 .73496 .47062 .75195 .46669 .74398 .45184 Metetra
.635 .647 .846 r
.74398 .45184 .75195 .46669 .76903 .46238 .76109 .44737 Metetra
.639 .648 .843 r
.76109 .44737 .76903 .46238 .7862 .45778 .7783 .44262 Metetra
.642 .648 .841 r
.7783 .44262 .7862 .45778 .80347 .45296 .7956 .43768 Metetra
.644 .649 .84 r
.7956 .43768 .80347 .45296 .82083 .44797 .813 .43258 Metetra
.645 .649 .839 r
.813 .43258 .82083 .44797 .83829 .44286 .8305 .42736 Metetra
.646 .649 .838 r
.8305 .42736 .83829 .44286 .85584 .43764 .8481 .42205 Metetra
.647 .649 .838 r
.8481 .42205 .85584 .43764 .87351 .43234 .8658 .41667 Metetra
.647 .649 .837 r
.8658 .41667 .87351 .43234 .89127 .42698 .88361 .41123 Metetra
.664 .741 .908 r
.26159 .47007 .27069 .47854 .2845 .47397 .27545 .46545 Metetra
.664 .741 .908 r
.27545 .46545 .2845 .47397 .29838 .46936 .28937 .4608 Metetra
.664 .741 .908 r
.28937 .4608 .29838 .46936 .31234 .46474 .30337 .45613 Metetra
.664 .741 .908 r
.30337 .45613 .31234 .46474 .32636 .46009 .31744 .45143 Metetra
.664 .741 .908 r
.31744 .45143 .32636 .46009 .34047 .45541 .33159 .44671 Metetra
.663 .741 .908 r
.33159 .44671 .34047 .45541 .35465 .45071 .34582 .44197 Metetra
.663 .741 .908 r
.34582 .44197 .35465 .45071 .3689 .44601 .36012 .43721 Metetra
.662 .741 .908 r
.36012 .43721 .3689 .44601 .38323 .44133 .3745 .43247 Metetra
.659 .74 .909 r
.3745 .43247 .38323 .44133 .39763 .43676 .38895 .42781 Metetra
.652 .737 .911 r
.38895 .42781 .39763 .43676 .4121 .43245 .40347 .42338 Metetra
.639 .731 .915 r
.40347 .42338 .4121 .43245 .42663 .42866 .41805 .4194 Metetra
.616 .72 .92 r
.41805 .4194 .42663 .42866 .44124 .4257 .4327 .41621 Metetra
.579 .702 .926 r
.4327 .41621 .44124 .4257 .45591 .42395 .44742 .41415 Metetra
.528 .674 .931 r
.44742 .41415 .45591 .42395 .47067 .42369 .46222 .41355 Metetra
.463 .637 .931 r
.46222 .41355 .47067 .42369 .48553 .42511 .47712 .41461 Metetra
.394 .595 .925 r
.47712 .41461 .48553 .42511 .50051 .42818 .49213 .41733 Metetra
.331 .553 .915 r
.49213 .41733 .50051 .42818 .51566 .43268 .5073 .42149 Metetra
.288 .521 .904 r
.5073 .42149 .51566 .43268 .53097 .43819 .52264 .4267 Metetra
.272 .506 .897 r
.52264 .4267 .53097 .43819 .54648 .44421 .53816 .43243 Metetra
.285 .508 .895 r
.53816 .43243 .54648 .44421 .56217 .4502 .55387 .43814 Metetra
.32 .523 .897 r
.55387 .43814 .56217 .4502 .57806 .45567 .56977 .44333 Metetra
.369 .546 .898 r
.56977 .44333 .57806 .45567 .59411 .46023 .58584 .44761 Metetra
.421 .571 .897 r
.58584 .44761 .59411 .46023 .61033 .46365 .60208 .45073 Metetra
.471 .592 .893 r
.60208 .45073 .61033 .46365 .62668 .46579 .61845 .45257 Metetra
.514 .61 .886 r
.61845 .45257 .62668 .46579 .64315 .46666 .63494 .45313 Metetra
.549 .623 .878 r
.63494 .45313 .64315 .46666 .65972 .46633 .65154 .4525 Metetra
.576 .632 .87 r
.65154 .4525 .65972 .46633 .6764 .46494 .66824 .45082 Metetra
.597 .639 .863 r
.66824 .45082 .6764 .46494 .69316 .46264 .68503 .44826 Metetra
.612 .643 .857 r
.68503 .44826 .69316 .46264 .71001 .45959 .70191 .44498 Metetra
.623 .645 .852 r
.70191 .44498 .71001 .45959 .72695 .45595 .71888 .44112 Metetra
.631 .647 .848 r
.71888 .44112 .72695 .45595 .74398 .45184 .73594 .43683 Metetra
.636 .648 .845 r
.73594 .43683 .74398 .45184 .76109 .44737 .75309 .4322 Metetra
.64 .648 .842 r
.75309 .4322 .76109 .44737 .7783 .44262 .77033 .42732 Metetra
.643 .649 .841 r
.77033 .42732 .7783 .44262 .7956 .43768 .78767 .42225 Metetra
.644 .649 .839 r
.78767 .42225 .7956 .43768 .813 .43258 .80511 .41705 Metetra
.646 .649 .839 r
.80511 .41705 .813 .43258 .8305 .42736 .82265 .41173 Metetra
.646 .649 .838 r
.82265 .41173 .8305 .42736 .8481 .42205 .84029 .40633 Metetra
.647 .649 .838 r
.84029 .40633 .8481 .42205 .8658 .41667 .85804 .40087 Metetra
.647 .649 .837 r
.85804 .40087 .8658 .41667 .88361 .41123 .87589 .39535 Metetra
.664 .741 .908 r
.25241 .46152 .26159 .47007 .27545 .46545 .2663 .45685 Metetra
.664 .741 .908 r
.2663 .45685 .27545 .46545 .28937 .4608 .28027 .45216 Metetra
.664 .741 .908 r
.28027 .45216 .28937 .4608 .30337 .45613 .29432 .44744 Metetra
.664 .741 .908 r
.29432 .44744 .30337 .45613 .31744 .45143 .30844 .4427 Metetra
.664 .741 .908 r
.30844 .4427 .31744 .45143 .33159 .44671 .32263 .43793 Metetra
.663 .741 .908 r
.32263 .43793 .33159 .44671 .34582 .44197 .3369 .43313 Metetra
.663 .741 .908 r
.3369 .43313 .34582 .44197 .36012 .43721 .35125 .42832 Metetra
.662 .741 .908 r
.35125 .42832 .36012 .43721 .3745 .43247 .36567 .42352 Metetra
.66 .74 .909 r
.36567 .42352 .3745 .43247 .38895 .42781 .38017 .41878 Metetra
.654 .737 .911 r
.38017 .41878 .38895 .42781 .40347 .42338 .39474 .41423 Metetra
.643 .732 .914 r
.39474 .41423 .40347 .42338 .41805 .4194 .40938 .41008 Metetra
.621 .722 .919 r
.40938 .41008 .41805 .4194 .4327 .41621 .42409 .40665 Metetra
.588 .706 .925 r
.42409 .40665 .4327 .41621 .44742 .41415 .43885 .40429 Metetra
.539 .68 .93 r
.43885 .40429 .44742 .41415 .46222 .41355 .45369 .40335 Metetra
.477 .645 .931 r
.45369 .40335 .46222 .41355 .47712 .41461 .46863 .40404 Metetra
.408 .603 .926 r
.46863 .40404 .47712 .41461 .49213 .41733 .48368 .4064 Metetra
.346 .562 .917 r
.48368 .4064 .49213 .41733 .5073 .42149 .49887 .41022 Metetra
.302 .53 .907 r
.49887 .41022 .5073 .42149 .52264 .4267 .51422 .41511 Metetra
.285 .514 .9 r
.51422 .41511 .52264 .4267 .53816 .43243 .52976 .42055 Metetra
.297 .515 .897 r
.52976 .42055 .53816 .43243 .55387 .43814 .54549 .42597 Metetra
.331 .53 .898 r
.54549 .42597 .55387 .43814 .56977 .44333 .56141 .43088 Metetra
.379 .552 .899 r
.56141 .43088 .56977 .44333 .58584 .44761 .5775 .43486 Metetra
.431 .575 .897 r
.5775 .43486 .58584 .44761 .60208 .45073 .59376 .43767 Metetra
.48 .597 .892 r
.59376 .43767 .60208 .45073 .61845 .45257 .61015 .43919 Metetra
.522 .613 .885 r
.61015 .43919 .61845 .45257 .63494 .45313 .62666 .43944 Metetra
.556 .626 .876 r
.62666 .43944 .63494 .45313 .65154 .4525 .64329 .43851 Metetra
.582 .634 .868 r
.64329 .43851 .65154 .4525 .66824 .45082 .66001 .43655 Metetra
.601 .64 .861 r
.66001 .43655 .66824 .45082 .68503 .44826 .67683 .43372 Metetra
.615 .644 .855 r
.67683 .43372 .68503 .44826 .70191 .44498 .69374 .4302 Metetra
.625 .646 .85 r
.69374 .4302 .70191 .44498 .71888 .44112 .71074 .42615 Metetra
.633 .647 .847 r
.71074 .42615 .71888 .44112 .73594 .43683 .72783 .42167 Metetra
.638 .648 .844 r
.72783 .42167 .73594 .43683 .75309 .4322 .74502 .41689 Metetra
.641 .648 .842 r
.74502 .41689 .75309 .4322 .77033 .42732 .7623 .41187 Metetra
.644 .649 .84 r
.7623 .41187 .77033 .42732 .78767 .42225 .77968 .40669 Metetra
.645 .649 .839 r
.77968 .40669 .78767 .42225 .80511 .41705 .79715 .40137 Metetra
.646 .649 .838 r
.79715 .40137 .80511 .41705 .82265 .41173 .81473 .39596 Metetra
.647 .649 .838 r
.81473 .39596 .82265 .41173 .84029 .40633 .83242 .39048 Metetra
.647 .648 .837 r
.83242 .39048 .84029 .40633 .85804 .40087 .85021 .38493 Metetra
.647 .648 .837 r
.85021 .38493 .85804 .40087 .87589 .39535 .86811 .37933 Metetra
.664 .741 .908 r
.24313 .45288 .25241 .46152 .2663 .45685 .25707 .44817 Metetra
.664 .741 .908 r
.25707 .44817 .2663 .45685 .28027 .45216 .27108 .44343 Metetra
.664 .741 .908 r
.27108 .44343 .28027 .45216 .29432 .44744 .28517 .43866 Metetra
.664 .741 .908 r
.28517 .43866 .29432 .44744 .30844 .4427 .29934 .43387 Metetra
.664 .741 .908 r
.29934 .43387 .30844 .4427 .32263 .43793 .31358 .42905 Metetra
.664 .741 .908 r
.31358 .42905 .32263 .43793 .3369 .43313 .3279 .42421 Metetra
.663 .741 .908 r
.3279 .42421 .3369 .43313 .35125 .42832 .34229 .41935 Metetra
.663 .741 .908 r
.34229 .41935 .35125 .42832 .36567 .42352 .35677 .41449 Metetra
.661 .74 .909 r
.35677 .41449 .36567 .42352 .38017 .41878 .37131 .40967 Metetra
.656 .738 .91 r
.37131 .40967 .38017 .41878 .39474 .41423 .38594 .405 Metetra
.646 .734 .913 r
.38594 .405 .39474 .41423 .40938 .41008 .40063 .40069 Metetra
.627 .725 .917 r
.40063 .40069 .40938 .41008 .42409 .40665 .41538 .39703 Metetra
.596 .709 .923 r
.41538 .39703 .42409 .40665 .43885 .40429 .4302 .39438 Metetra
.55 .685 .928 r
.4302 .39438 .43885 .40429 .45369 .40335 .44508 .39309 Metetra
.49 .652 .931 r
.44508 .39309 .45369 .40335 .46863 .40404 .46006 .3934 Metetra
.423 .611 .927 r
.46006 .3934 .46863 .40404 .48368 .4064 .47514 .39538 Metetra
.361 .57 .919 r
.47514 .39538 .48368 .4064 .49887 .41022 .49036 .39885 Metetra
.316 .538 .909 r
.49036 .39885 .49887 .41022 .51422 .41511 .50574 .40342 Metetra
.298 .521 .902 r
.50574 .40342 .51422 .41511 .52976 .42055 .52129 .40856 Metetra
.308 .522 .899 r
.52129 .40856 .52976 .42055 .54549 .42597 .53704 .41369 Metetra
.342 .536 .899 r
.53704 .41369 .54549 .42597 .56141 .43088 .55298 .4183 Metetra
.39 .557 .899 r
.55298 .4183 .56141 .43088 .5775 .43486 .56909 .42199 Metetra
.441 .58 .897 r
.56909 .42199 .5775 .43486 .59376 .43767 .58536 .42448 Metetra
.489 .601 .891 r
.58536 .42448 .59376 .43767 .61015 .43919 .60178 .42568 Metetra
.53 .617 .883 r
.60178 .42568 .61015 .43919 .62666 .43944 .61832 .4256 Metetra
.562 .628 .875 r
.61832 .4256 .62666 .43944 .64329 .43851 .63497 .42436 Metetra
.587 .636 .867 r
.63497 .42436 .64329 .43851 .66001 .43655 .65172 .42211 Metetra
.605 .641 .86 r
.65172 .42211 .66001 .43655 .67683 .43372 .66857 .41902 Metetra
.619 .644 .854 r
.66857 .41902 .67683 .43372 .69374 .4302 .68551 .41527 Metetra
.628 .646 .849 r
.68551 .41527 .69374 .4302 .71074 .42615 .70254 .41101 Metetra
.635 .648 .846 r
.70254 .41101 .71074 .42615 .72783 .42167 .71966 .40636 Metetra
.639 .648 .843 r
.71966 .40636 .72783 .42167 .74502 .41689 .73688 .40142 Metetra
.642 .648 .841 r
.73688 .40142 .74502 .41689 .7623 .41187 .7542 .39628 Metetra
.644 .649 .84 r
.7542 .39628 .7623 .41187 .77968 .40669 .77162 .39098 Metetra
.646 .649 .839 r
.77162 .39098 .77968 .40669 .79715 .40137 .78914 .38556 Metetra
.646 .648 .838 r
.78914 .38556 .79715 .40137 .81473 .39596 .80676 .38006 Metetra
.647 .648 .837 r
.80676 .38006 .81473 .39596 .83242 .39048 .82449 .37448 Metetra
.647 .648 .837 r
.82449 .37448 .83242 .39048 .85021 .38493 .84232 .36885 Metetra
.648 .648 .837 r
.84232 .36885 .85021 .38493 .86811 .37933 .86027 .36317 Metetra
.664 .741 .908 r
.23376 .44416 .24313 .45288 .25707 .44817 .24775 .4394 Metetra
.664 .741 .908 r
.24775 .4394 .25707 .44817 .27108 .44343 .2618 .43461 Metetra
.664 .741 .908 r
.2618 .43461 .27108 .44343 .28517 .43866 .27594 .4298 Metetra
.664 .741 .908 r
.27594 .4298 .28517 .43866 .29934 .43387 .29015 .42496 Metetra
.664 .741 .908 r
.29015 .42496 .29934 .43387 .31358 .42905 .30444 .42009 Metetra
.664 .741 .908 r
.30444 .42009 .31358 .42905 .3279 .42421 .3188 .4152 Metetra
.663 .741 .908 r
.3188 .4152 .3279 .42421 .34229 .41935 .33324 .41029 Metetra
.663 .741 .908 r
.33324 .41029 .34229 .41935 .35677 .41449 .34777 .40537 Metetra
.662 .74 .909 r
.34777 .40537 .35677 .41449 .37131 .40967 .36237 .40047 Metetra
.658 .739 .91 r
.36237 .40047 .37131 .40967 .38594 .405 .37704 .3957 Metetra
.649 .735 .912 r
.37704 .3957 .38594 .405 .40063 .40069 .39178 .39123 Metetra
.632 .727 .916 r
.39178 .39123 .40063 .40069 .41538 .39703 .40659 .38735 Metetra
.603 .713 .922 r
.40659 .38735 .41538 .39703 .4302 .39438 .42146 .3844 Metetra
.561 .691 .927 r
.42146 .3844 .4302 .39438 .44508 .39309 .43639 .38276 Metetra
.504 .659 .93 r
.43639 .38276 .44508 .39309 .46006 .3934 .45141 .3827 Metetra
.438 .62 .928 r
.45141 .3827 .46006 .3934 .47514 .39538 .46652 .38429 Metetra
.376 .579 .92 r
.46652 .38429 .47514 .39538 .49036 .39885 .48177 .3874 Metetra
.33 .547 .911 r
.48177 .3874 .49036 .39885 .50574 .40342 .49717 .39163 Metetra
.311 .529 .904 r
.49717 .39163 .50574 .40342 .52129 .40856 .51275 .39646 Metetra
.32 .529 .901 r
.51275 .39646 .52129 .40856 .53704 .41369 .52852 .4013 Metetra
.353 .542 .9 r
.52852 .4013 .53704 .41369 .55298 .4183 .54447 .40561 Metetra
.4 .563 .899 r
.54447 .40561 .55298 .4183 .56909 .42199 .5606 .40897 Metetra
.451 .585 .896 r
.5606 .40897 .56909 .42199 .58536 .42448 .5769 .41114 Metetra
.498 .605 .89 r
.5769 .41114 .58536 .42448 .60178 .42568 .59333 .41201 Metetra
.537 .62 .882 r
.59333 .41201 .60178 .42568 .61832 .4256 .6099 .4116 Metetra
.569 .631 .873 r
.6099 .4116 .61832 .4256 .63497 .42436 .62658 .41005 Metetra
.592 .638 .865 r
.62658 .41005 .63497 .42436 .65172 .42211 .64335 .4075 Metetra
.61 .642 .858 r
.64335 .4075 .65172 .42211 .66857 .41902 .66023 .40415 Metetra
.622 .645 .852 r
.66023 .40415 .66857 .41902 .68551 .41527 .6772 .40017 Metetra
.631 .647 .848 r
.6772 .40017 .68551 .41527 .70254 .41101 .69427 .39571 Metetra
.637 .648 .845 r
.69427 .39571 .70254 .41101 .71966 .40636 .71143 .39089 Metetra
.641 .648 .842 r
.71143 .39089 .71966 .40636 .73688 .40142 .72868 .38581 Metetra
.643 .648 .84 r
.72868 .38581 .73688 .40142 .7542 .39628 .74604 .38053 Metetra
.645 .648 .839 r
.74604 .38053 .7542 .39628 .77162 .39098 .7635 .37512 Metetra
.646 .648 .838 r
.7635 .37512 .77162 .39098 .78914 .38556 .78106 .36961 Metetra
.647 .648 .838 r
.78106 .36961 .78914 .38556 .80676 .38006 .79872 .36401 Metetra
.647 .648 .837 r
.79872 .36401 .80676 .38006 .82449 .37448 .81649 .35835 Metetra
.647 .648 .837 r
.81649 .35835 .82449 .37448 .84232 .36885 .83437 .35264 Metetra
.648 .648 .837 r
.83437 .35264 .84232 .36885 .86027 .36317 .85236 .34688 Metetra
.664 .741 .908 r
.2243 .43535 .23376 .44416 .24775 .4394 .23833 .43054 Metetra
.664 .741 .908 r
.23833 .43054 .24775 .4394 .2618 .43461 .25243 .4257 Metetra
.664 .741 .908 r
.25243 .4257 .2618 .43461 .27594 .4298 .26661 .42084 Metetra
.664 .741 .908 r
.26661 .42084 .27594 .4298 .29015 .42496 .28087 .41596 Metetra
.664 .741 .908 r
.28087 .41596 .29015 .42496 .30444 .42009 .2952 .41104 Metetra
.664 .741 .908 r
.2952 .41104 .30444 .42009 .3188 .4152 .30961 .4061 Metetra
.663 .741 .908 r
.30961 .4061 .3188 .4152 .33324 .41029 .32411 .40114 Metetra
.663 .741 .908 r
.32411 .40114 .33324 .41029 .34777 .40537 .33868 .39616 Metetra
.662 .741 .908 r
.33868 .39616 .34777 .40537 .36237 .40047 .35332 .39119 Metetra
.659 .739 .909 r
.35332 .39119 .36237 .40047 .37704 .3957 .36805 .38632 Metetra
.651 .736 .911 r
.36805 .38632 .37704 .3957 .39178 .39123 .38284 .3817 Metetra
.637 .729 .915 r
.38284 .3817 .39178 .39123 .40659 .38735 .39771 .3776 Metetra
.611 .716 .92 r
.39771 .3776 .40659 .38735 .42146 .3844 .41263 .37436 Metetra
.571 .696 .926 r
.41263 .37436 .42146 .3844 .43639 .38276 .42761 .37237 Metetra
.517 .666 .929 r
.42761 .37237 .43639 .38276 .45141 .3827 .44267 .37192 Metetra
.453 .628 .928 r
.44267 .37192 .45141 .3827 .46652 .38429 .45782 .37312 Metetra
.391 .588 .922 r
.45782 .37312 .46652 .38429 .48177 .3874 .4731 .37585 Metetra
.344 .555 .913 r
.4731 .37585 .48177 .3874 .49717 .39163 .48853 .37974 Metetra
.324 .537 .905 r
.48853 .37974 .49717 .39163 .51275 .39646 .50413 .38425 Metetra
.333 .536 .902 r
.50413 .38425 .51275 .39646 .52852 .4013 .51991 .38878 Metetra
.365 .548 .901 r
.51991 .38878 .52852 .4013 .54447 .40561 .53589 .39278 Metetra
.411 .569 .9 r
.53589 .39278 .54447 .40561 .5606 .40897 .55204 .39583 Metetra
.461 .59 .896 r
.55204 .39583 .5606 .40897 .5769 .41114 .56836 .39766 Metetra
.507 .609 .889 r
.56836 .39766 .5769 .41114 .59333 .41201 .58482 .39819 Metetra
.545 .623 .88 r
.58482 .39819 .59333 .41201 .6099 .4116 .60141 .39745 Metetra
.575 .633 .871 r
.60141 .39745 .6099 .4116 .62658 .41005 .61811 .39557 Metetra
.597 .639 .863 r
.61811 .39557 .62658 .41005 .64335 .4075 .63492 .39273 Metetra
.614 .644 .856 r
.63492 .39273 .64335 .4075 .66023 .40415 .65183 .38912 Metetra
.625 .646 .851 r
.65183 .38912 .66023 .40415 .6772 .40017 .66883 .38491 Metetra
.633 .647 .847 r
.66883 .38491 .6772 .40017 .69427 .39571 .68593 .38025 Metetra
.638 .648 .843 r
.68593 .38025 .69427 .39571 .71143 .39089 .70313 .37526 Metetra
.642 .648 .841 r
.70313 .37526 .71143 .39089 .72868 .38581 .72042 .37004 Metetra
.644 .648 .84 r
.72042 .37004 .72868 .38581 .74604 .38053 .73781 .36464 Metetra
.646 .648 .838 r
.73781 .36464 .74604 .38053 .7635 .37512 .75531 .35912 Metetra
.646 .648 .838 r
.75531 .35912 .7635 .37512 .78106 .36961 .77291 .35351 Metetra
.647 .648 .837 r
.77291 .35351 .78106 .36961 .79872 .36401 .79062 .34783 Metetra
.647 .648 .837 r
.79062 .34783 .79872 .36401 .81649 .35835 .80843 .34208 Metetra
.648 .648 .837 r
.80843 .34208 .81649 .35835 .83437 .35264 .82636 .33629 Metetra
.648 .648 .837 r
.82636 .33629 .83437 .35264 .85236 .34688 .84439 .33045 Metetra
.664 .741 .908 r
.21474 .42645 .2243 .43535 .23833 .43054 .22882 .42159 Metetra
.664 .741 .908 r
.22882 .42159 .23833 .43054 .25243 .4257 .24296 .41671 Metetra
.664 .741 .908 r
.24296 .41671 .25243 .4257 .26661 .42084 .25719 .4118 Metetra
.664 .741 .908 r
.25719 .4118 .26661 .42084 .28087 .41596 .27149 .40686 Metetra
.664 .741 .908 r
.27149 .40686 .28087 .41596 .2952 .41104 .28587 .4019 Metetra
.664 .741 .908 r
.28587 .4019 .2952 .41104 .30961 .4061 .30033 .39691 Metetra
.663 .741 .908 r
.30033 .39691 .30961 .4061 .32411 .40114 .31487 .39189 Metetra
.663 .741 .908 r
.31487 .39189 .32411 .40114 .33868 .39616 .32949 .38686 Metetra
.662 .741 .908 r
.32949 .38686 .33868 .39616 .35332 .39119 .34419 .38182 Metetra
.66 .74 .909 r
.34419 .38182 .35332 .39119 .36805 .38632 .35897 .37686 Metetra
.654 .737 .911 r
.35897 .37686 .36805 .38632 .38284 .3817 .37382 .3721 Metetra
.641 .731 .914 r
.37382 .3721 .38284 .3817 .39771 .3776 .38873 .36779 Metetra
.618 .72 .919 r
.38873 .36779 .39771 .3776 .41263 .37436 .40371 .36427 Metetra
.581 .701 .924 r
.40371 .36427 .41263 .37436 .42761 .37237 .41874 .36192 Metetra
.53 .672 .928 r
.41874 .36192 .42761 .37237 .44267 .37192 .43385 .36108 Metetra
.468 .636 .928 r
.43385 .36108 .44267 .37192 .45782 .37312 .44904 .36187 Metetra
.406 .597 .923 r
.44904 .36187 .45782 .37312 .4731 .37585 .46435 .36422 Metetra
.359 .563 .914 r
.46435 .36422 .4731 .37585 .48853 .37974 .4798 .36775 Metetra
.337 .544 .907 r
.4798 .36775 .48853 .37974 .50413 .38425 .49543 .37193 Metetra
.345 .543 .903 r
.49543 .37193 .50413 .38425 .51991 .38878 .51124 .37615 Metetra
.376 .555 .902 r
.51124 .37615 .51991 .38878 .53589 .39278 .52723 .37983 Metetra
.422 .574 .9 r
.52723 .37983 .53589 .39278 .55204 .39583 .54341 .38255 Metetra
.471 .595 .895 r
.54341 .38255 .55204 .39583 .56836 .39766 .55975 .38403 Metetra
.516 .613 .887 r
.55975 .38403 .56836 .39766 .58482 .39819 .57623 .38421 Metetra
.553 .626 .878 r
.57623 .38421 .58482 .39819 .60141 .39745 .59285 .38312 Metetra
.581 .635 .87 r
.59285 .38312 .60141 .39745 .61811 .39557 .60958 .38092 Metetra
.603 .641 .861 r
.60958 .38092 .61811 .39557 .63492 .39273 .62642 .37778 Metetra
.618 .645 .855 r
.62642 .37778 .63492 .39273 .65183 .38912 .64336 .37391 Metetra
.628 .647 .849 r
.64336 .37391 .65183 .38912 .66883 .38491 .66039 .36948 Metetra
.635 .648 .845 r
.66039 .36948 .66883 .38491 .68593 .38025 .67752 .36463 Metetra
.64 .648 .842 r
.67752 .36463 .68593 .38025 .70313 .37526 .69476 .35948 Metetra
.643 .648 .84 r
.69476 .35948 .70313 .37526 .72042 .37004 .71209 .35412 Metetra
.645 .648 .839 r
.71209 .35412 .72042 .37004 .73781 .36464 .72952 .3486 Metetra
.646 .648 .838 r
.72952 .3486 .73781 .36464 .75531 .35912 .74706 .34298 Metetra
.647 .648 .837 r
.74706 .34298 .75531 .35912 .77291 .35351 .7647 .33727 Metetra
.647 .648 .837 r
.7647 .33727 .77291 .35351 .79062 .34783 .78245 .3315 Metetra
.648 .648 .837 r
.78245 .3315 .79062 .34783 .80843 .34208 .80031 .32567 Metetra
.648 .648 .837 r
.80031 .32567 .80843 .34208 .82636 .33629 .81828 .3198 Metetra
.648 .648 .836 r
.81828 .3198 .82636 .33629 .84439 .33045 .83636 .31389 Metetra
.664 .741 .908 r
.20509 .41746 .21474 .42645 .22882 .42159 .21921 .41256 Metetra
.664 .741 .908 r
.21921 .41256 .22882 .42159 .24296 .41671 .2334 .40762 Metetra
.664 .741 .908 r
.2334 .40762 .24296 .41671 .25719 .4118 .24767 .40267 Metetra
.664 .741 .908 r
.24767 .40267 .25719 .4118 .27149 .40686 .26202 .39768 Metetra
.664 .741 .908 r
.26202 .39768 .27149 .40686 .28587 .4019 .27645 .39266 Metetra
.664 .741 .908 r
.27645 .39266 .28587 .4019 .30033 .39691 .29096 .38762 Metetra
.664 .741 .908 r
.29096 .38762 .30033 .39691 .31487 .39189 .30555 .38255 Metetra
.663 .741 .908 r
.30555 .38255 .31487 .39189 .32949 .38686 .32021 .37746 Metetra
.663 .741 .908 r
.32021 .37746 .32949 .38686 .34419 .38182 .33496 .37236 Metetra
.661 .74 .909 r
.33496 .37236 .34419 .38182 .35897 .37686 .34979 .36731 Metetra
.656 .738 .91 r
.34979 .36731 .35897 .37686 .37382 .3721 .36469 .36242 Metetra
.645 .733 .913 r
.36469 .36242 .37382 .3721 .38873 .36779 .37966 .35791 Metetra
.624 .723 .917 r
.37966 .35791 .38873 .36779 .40371 .36427 .3947 .35412 Metetra
.591 .705 .923 r
.3947 .35412 .40371 .36427 .41874 .36192 .40979 .35142 Metetra
.542 .679 .927 r
.40979 .35142 .41874 .36192 .43385 .36108 .42494 .35017 Metetra
.483 .644 .928 r
.42494 .35017 .43385 .36108 .44904 .36187 .44017 .35055 Metetra
.422 .605 .923 r
.44017 .35055 .44904 .36187 .46435 .36422 .45552 .35249 Metetra
.374 .572 .916 r
.45552 .35249 .46435 .36422 .4798 .36775 .471 .35566 Metetra
.351 .552 .909 r
.471 .35566 .4798 .36775 .49543 .37193 .48665 .3595 Metetra
.357 .55 .905 r
.48665 .3595 .49543 .37193 .51124 .37615 .50248 .36339 Metetra
.388 .561 .902 r
.50248 .36339 .51124 .37615 .52723 .37983 .5185 .36675 Metetra
.433 .58 .9 r
.5185 .36675 .52723 .37983 .54341 .38255 .53469 .36912 Metetra
.481 .6 .894 r
.53469 .36912 .54341 .38255 .55975 .38403 .55106 .37025 Metetra
.525 .616 .886 r
.55106 .37025 .55975 .38403 .57623 .38421 .56757 .37007 Metetra
.56 .629 .877 r
.56757 .37007 .57623 .38421 .59285 .38312 .58422 .36863 Metetra
.588 .637 .868 r
.58422 .36863 .59285 .38312 .60958 .38092 .60098 .36609 Metetra
.608 .643 .86 r
.60098 .36609 .60958 .38092 .62642 .37778 .61784 .36266 Metetra
.621 .646 .853 r
.61784 .36266 .62642 .37778 .64336 .37391 .63481 .35853 Metetra
.631 .647 .848 r
.63481 .35853 .64336 .37391 .66039 .36948 .65188 .35388 Metetra
.637 .648 .844 r
.65188 .35388 .66039 .36948 .67752 .36463 .66905 .34884 Metetra
.641 .648 .842 r
.66905 .34884 .67752 .36463 .69476 .35948 .68632 .34354 Metetra
.644 .648 .84 r
.68632 .34354 .69476 .35948 .71209 .35412 .70369 .33804 Metetra
.646 .648 .838 r
.70369 .33804 .71209 .35412 .72952 .3486 .72116 .33242 Metetra
.646 .648 .838 r
.72116 .33242 .72952 .3486 .74706 .34298 .73874 .32669 Metetra
.647 .648 .837 r
.73874 .32669 .74706 .34298 .7647 .33727 .75642 .32089 Metetra
.647 .648 .837 r
.75642 .32089 .7647 .33727 .78245 .3315 .77422 .31503 Metetra
.648 .648 .837 r
.77422 .31503 .78245 .3315 .80031 .32567 .79212 .30913 Metetra
.648 .648 .836 r
.79212 .30913 .80031 .32567 .81828 .3198 .81013 .30318 Metetra
.648 .648 .836 r
.81013 .30318 .81828 .3198 .83636 .31389 .82826 .29718 Metetra
.664 .741 .908 r
.19534 .40839 .20509 .41746 .21921 .41256 .2095 .40343 Metetra
.664 .741 .908 r
.2095 .40343 .21921 .41256 .2334 .40762 .22374 .39845 Metetra
.664 .741 .908 r
.22374 .39845 .2334 .40762 .24767 .40267 .23806 .39344 Metetra
.664 .741 .908 r
.23806 .39344 .24767 .40267 .26202 .39768 .25245 .3884 Metetra
.664 .741 .908 r
.25245 .3884 .26202 .39768 .27645 .39266 .26693 .38333 Metetra
.664 .741 .908 r
.26693 .38333 .27645 .39266 .29096 .38762 .28149 .37824 Metetra
.664 .741 .908 r
.28149 .37824 .29096 .38762 .30555 .38255 .29612 .37312 Metetra
.663 .741 .908 r
.29612 .37312 .30555 .38255 .32021 .37746 .31084 .36797 Metetra
.663 .741 .908 r
.31084 .36797 .32021 .37746 .33496 .37236 .32564 .36281 Metetra
.662 .74 .908 r
.32564 .36281 .33496 .37236 .34979 .36731 .34052 .35768 Metetra
.658 .739 .91 r
.34052 .35768 .34979 .36731 .36469 .36242 .35548 .35266 Metetra
.649 .734 .912 r
.35548 .35266 .36469 .36242 .37966 .35791 .3705 .34797 Metetra
.63 .726 .916 r
.3705 .34797 .37966 .35791 .3947 .35412 .38559 .3439 Metetra
.6 .71 .921 r
.38559 .3439 .3947 .35412 .40979 .35142 .40074 .34086 Metetra
.555 .685 .926 r
.40074 .34086 .40979 .35142 .42494 .35017 .41594 .3392 Metetra
.497 .652 .928 r
.41594 .3392 .42494 .35017 .44017 .35055 .43122 .33915 Metetra
.437 .614 .924 r
.43122 .33915 .44017 .35055 .45552 .35249 .4466 .34068 Metetra
.389 .58 .917 r
.4466 .34068 .45552 .35249 .471 .35566 .46211 .34347 Metetra
.364 .56 .91 r
.46211 .34347 .471 .35566 .48665 .3595 .47779 .34696 Metetra
.37 .557 .906 r
.47779 .34696 .48665 .3595 .50248 .36339 .49364 .35052 Metetra
.4 .567 .903 r
.49364 .35052 .50248 .36339 .5185 .36675 .50968 .35353 Metetra
.444 .585 .899 r
.50968 .35353 .5185 .36675 .53469 .36912 .52591 .35555 Metetra
.491 .604 .893 r
.52591 .35555 .53469 .36912 .55106 .37025 .5423 .35631 Metetra
.534 .62 .884 r
.5423 .35631 .55106 .37025 .56757 .37007 .55884 .35576 Metetra
.568 .632 .875 r
.55884 .35576 .56757 .37007 .58422 .36863 .57551 .35396 Metetra
.594 .639 .865 r
.57551 .35396 .58422 .36863 .60098 .36609 .5923 .35109 Metetra
.612 .644 .858 r
.5923 .35109 .60098 .36609 .61784 .36266 .6092 .34736 Metetra
.625 .646 .851 r
.6092 .34736 .61784 .36266 .63481 .35853 .6262 .34297 Metetra
.634 .648 .847 r
.6262 .34297 .63481 .35853 .65188 .35388 .6433 .3381 Metetra
.639 .648 .843 r
.6433 .3381 .65188 .35388 .66905 .34884 .66051 .33289 Metetra
.643 .648 .841 r
.66051 .33289 .66905 .34884 .68632 .34354 .67781 .32744 Metetra
.645 .648 .839 r
.67781 .32744 .68632 .34354 .70369 .33804 .69522 .32182 Metetra
.646 .648 .838 r
.69522 .32182 .70369 .33804 .72116 .33242 .71273 .31608 Metetra
.647 .648 .837 r
.71273 .31608 .72116 .33242 .73874 .32669 .73035 .31026 Metetra
.647 .648 .837 r
.73035 .31026 .73874 .32669 .75642 .32089 .74808 .30437 Metetra
.648 .648 .837 r
.74808 .30437 .75642 .32089 .77422 .31503 .76592 .29843 Metetra
.648 .648 .836 r
.76592 .29843 .77422 .31503 .79212 .30913 .78386 .29244 Metetra
.648 .648 .836 r
.78386 .29244 .79212 .30913 .81013 .30318 .80192 .28641 Metetra
.648 .648 .836 r
.80192 .28641 .81013 .30318 .82826 .29718 .8201 .28034 Metetra
.664 .741 .908 r
.18549 .39922 .19534 .40839 .2095 .40343 .1997 .39421 Metetra
.664 .741 .908 r
.1997 .39421 .2095 .40343 .22374 .39845 .21398 .38918 Metetra
.664 .741 .908 r
.21398 .38918 .22374 .39845 .23806 .39344 .22835 .38411 Metetra
.664 .741 .908 r
.22835 .38411 .23806 .39344 .25245 .3884 .24279 .37903 Metetra
.664 .741 .908 r
.24279 .37903 .25245 .3884 .26693 .38333 .25731 .37391 Metetra
.664 .741 .908 r
.25731 .37391 .26693 .38333 .28149 .37824 .27192 .36876 Metetra
.664 .741 .908 r
.27192 .36876 .28149 .37824 .29612 .37312 .2866 .36359 Metetra
.663 .741 .908 r
.2866 .36359 .29612 .37312 .31084 .36797 .30137 .35838 Metetra
.663 .741 .908 r
.30137 .35838 .31084 .36797 .32564 .36281 .31622 .35316 Metetra
.662 .741 .908 r
.31622 .35316 .32564 .36281 .34052 .35768 .33115 .34795 Metetra
.659 .739 .909 r
.33115 .34795 .34052 .35768 .35548 .35266 .34616 .34282 Metetra
.652 .736 .911 r
.34616 .34282 .35548 .35266 .3705 .34797 .36125 .33796 Metetra
.636 .728 .915 r
.36125 .33796 .3705 .34797 .38559 .3439 .37639 .33363 Metetra
.609 .714 .92 r
.37639 .33363 .38559 .3439 .40074 .34086 .39159 .33024 Metetra
.567 .691 .925 r
.39159 .33024 .40074 .34086 .41594 .3392 .40685 .32817 Metetra
.512 .66 .927 r
.40685 .32817 .41594 .3392 .43122 .33915 .42218 .32768 Metetra
.453 .623 .924 r
.42218 .32768 .43122 .33915 .4466 .34068 .4376 .32878 Metetra
.404 .589 .918 r
.4376 .32878 .4466 .34068 .46211 .34347 .45314 .33117 Metetra
.378 .568 .911 r
.45314 .33117 .46211 .34347 .47779 .34696 .46884 .3343 Metetra
.382 .564 .907 r
.46884 .3343 .47779 .34696 .49364 .35052 .48472 .33751 Metetra
.412 .574 .903 r
.48472 .33751 .49364 .35052 .50968 .35353 .50079 .34018 Metetra
.455 .591 .899 r
.50079 .34018 .50968 .35353 .52591 .35555 .51704 .34183 Metetra
.502 .609 .892 r
.51704 .34183 .52591 .35555 .5423 .35631 .53346 .34221 Metetra
.543 .624 .882 r
.53346 .34221 .5423 .35631 .55884 .35576 .55003 .34127 Metetra
.576 .635 .873 r
.55003 .34127 .55884 .35576 .57551 .35396 .56673 .33911 Metetra
.6 .641 .863 r
.56673 .33911 .57551 .35396 .5923 .35109 .58355 .3359 Metetra
.617 .645 .856 r
.58355 .3359 .5923 .35109 .6092 .34736 .60048 .33188 Metetra
.629 .647 .85 r
.60048 .33188 .6092 .34736 .6262 .34297 .61752 .32724 Metetra
.636 .648 .845 r
.61752 .32724 .6262 .34297 .6433 .3381 .63466 .32216 Metetra
.641 .648 .842 r
.63466 .32216 .6433 .3381 .66051 .33289 .6519 .31677 Metetra
.644 .648 .84 r
.6519 .31677 .66051 .33289 .67781 .32744 .66924 .31118 Metetra
.646 .648 .838 r
.66924 .31118 .67781 .32744 .69522 .32182 .68669 .30544 Metetra
.647 .648 .837 r
.68669 .30544 .69522 .32182 .71273 .31608 .70424 .2996 Metetra
.647 .648 .837 r
.70424 .2996 .71273 .31608 .73035 .31026 .7219 .29368 Metetra
.647 .648 .837 r
.7219 .29368 .73035 .31026 .74808 .30437 .73967 .28771 Metetra
.648 .648 .836 r
.73967 .28771 .74808 .30437 .76592 .29843 .75755 .28168 Metetra
.648 .648 .836 r
.75755 .28168 .76592 .29843 .78386 .29244 .77554 .27561 Metetra
.648 .648 .836 r
.77554 .27561 .78386 .29244 .80192 .28641 .79365 .2695 Metetra
.648 .648 .836 r
.79365 .2695 .80192 .28641 .8201 .28034 .81187 .26335 Metetra
.664 .741 .908 r
.17555 .38995 .18549 .39922 .1997 .39421 .1898 .3849 Metetra
.664 .741 .908 r
.1898 .3849 .1997 .39421 .21398 .38918 .20413 .37981 Metetra
.664 .741 .908 r
.20413 .37981 .21398 .38918 .22835 .38411 .21853 .3747 Metetra
.664 .741 .908 r
.21853 .3747 .22835 .38411 .24279 .37903 .23302 .36955 Metetra
.664 .741 .908 r
.23302 .36955 .24279 .37903 .25731 .37391 .2476 .36438 Metetra
.664 .741 .908 r
.2476 .36438 .25731 .37391 .27192 .36876 .26225 .35918 Metetra
.664 .741 .908 r
.26225 .35918 .27192 .36876 .2866 .36359 .27698 .35396 Metetra
.664 .741 .908 r
.27698 .35396 .2866 .36359 .30137 .35838 .2918 .3487 Metetra
.663 .741 .908 r
.2918 .3487 .30137 .35838 .31622 .35316 .3067 .34342 Metetra
.663 .741 .908 r
.3067 .34342 .31622 .35316 .33115 .34795 .32169 .33813 Metetra
.66 .74 .909 r
.32169 .33813 .33115 .34795 .34616 .34282 .33675 .3329 Metetra
.655 .737 .91 r
.33675 .3329 .34616 .34282 .36125 .33796 .35189 .32787 Metetra
.641 .731 .913 r
.35189 .32787 .36125 .33796 .37639 .33363 .3671 .3233 Metetra
.617 .718 .918 r
.3671 .3233 .37639 .33363 .39159 .33024 .38236 .31957 Metetra
.579 .697 .923 r
.38236 .31957 .39159 .33024 .40685 .32817 .39767 .31708 Metetra
.527 .667 .926 r
.39767 .31708 .40685 .32817 .42218 .32768 .41305 .31614 Metetra
.469 .631 .925 r
.41305 .31614 .42218 .32768 .4376 .32878 .42851 .3168 Metetra
.419 .598 .919 r
.42851 .3168 .4376 .32878 .45314 .33117 .44409 .31878 Metetra
.392 .576 .912 r
.44409 .31878 .45314 .33117 .46884 .3343 .45982 .32153 Metetra
.395 .571 .907 r
.45982 .32153 .46884 .3343 .48472 .33751 .47572 .32439 Metetra
.424 .58 .903 r
.47572 .32439 .48472 .33751 .50079 .34018 .49182 .32669 Metetra
.467 .597 .898 r
.49182 .32669 .50079 .34018 .51704 .34183 .5081 .32796 Metetra
.512 .614 .89 r
.5081 .32796 .51704 .34183 .53346 .34221 .52454 .32795 Metetra
.552 .628 .88 r
.52454 .32795 .53346 .34221 .55003 .34127 .54114 .32661 Metetra
.583 .637 .87 r
.54114 .32661 .55003 .34127 .56673 .33911 .55787 .32407 Metetra
.606 .643 .861 r
.55787 .32407 .56673 .33911 .58355 .3359 .57473 .32052 Metetra
.622 .646 .854 r
.57473 .32052 .58355 .3359 .60048 .33188 .59169 .3162 Metetra
.632 .648 .848 r
.59169 .3162 .60048 .33188 .61752 .32724 .60876 .31132 Metetra
.638 .648 .844 r
.60876 .31132 .61752 .32724 .63466 .32216 .62594 .30604 Metetra
.642 .648 .841 r
.62594 .30604 .63466 .32216 .6519 .31677 .64321 .30049 Metetra
.645 .648 .839 r
.64321 .30049 .6519 .31677 .66924 .31118 .66059 .29476 Metetra
.646 .648 .838 r
.66059 .29476 .66924 .31118 .68669 .30544 .67808 .28891 Metetra
.647 .648 .837 r
.67808 .28891 .68669 .30544 .70424 .2996 .69568 .28297 Metetra
.647 .648 .837 r
.69568 .28297 .70424 .2996 .7219 .29368 .71338 .27696 Metetra
.648 .648 .836 r
.71338 .27696 .7219 .29368 .73967 .28771 .73119 .2709 Metetra
.648 .648 .836 r
.73119 .2709 .73967 .28771 .75755 .28168 .74912 .26479 Metetra
.648 .648 .836 r
.74912 .26479 .75755 .28168 .77554 .27561 .76715 .25864 Metetra
.648 .648 .836 r
.76715 .25864 .77554 .27561 .79365 .2695 .78531 .25245 Metetra
.648 .648 .836 r
.78531 .25245 .79365 .2695 .81187 .26335 .80357 .24622 Metetra
.664 .741 .908 r
.1655 .38059 .17555 .38995 .1898 .3849 .17979 .37549 Metetra
.664 .741 .908 r
.17979 .37549 .1898 .3849 .20413 .37981 .19417 .37035 Metetra
.664 .741 .908 r
.19417 .37035 .20413 .37981 .21853 .3747 .20862 .36518 Metetra
.664 .741 .908 r
.20862 .36518 .21853 .3747 .23302 .36955 .22316 .35999 Metetra
.664 .741 .908 r
.22316 .35999 .23302 .36955 .2476 .36438 .23778 .35476 Metetra
.664 .741 .908 r
.23778 .35476 .2476 .36438 .26225 .35918 .25248 .34951 Metetra
.664 .741 .908 r
.25248 .34951 .26225 .35918 .27698 .35396 .26726 .34422 Metetra
.664 .741 .908 r
.26726 .34422 .27698 .35396 .2918 .3487 .28213 .33891 Metetra
.663 .741 .908 r
.28213 .33891 .2918 .3487 .3067 .34342 .29708 .33357 Metetra
.663 .741 .908 r
.29708 .33357 .3067 .34342 .32169 .33813 .31212 .32822 Metetra
.661 .74 .908 r
.31212 .32822 .32169 .33813 .33675 .3329 .32724 .32289 Metetra
.657 .738 .91 r
.32724 .32289 .33675 .3329 .35189 .32787 .34244 .31771 Metetra
.646 .733 .912 r
.34244 .31771 .35189 .32787 .3671 .3233 .3577 .31291 Metetra
.625 .722 .916 r
.3577 .31291 .3671 .3233 .38236 .31957 .37302 .30885 Metetra
.59 .703 .921 r
.37302 .30885 .38236 .31957 .39767 .31708 .38839 .30594 Metetra
.541 .675 .925 r
.38839 .30594 .39767 .31708 .41305 .31614 .40382 .30454 Metetra
.485 .64 .924 r
.40382 .30454 .41305 .31614 .42851 .3168 .41933 .30473 Metetra
.435 .606 .919 r
.41933 .30473 .42851 .3168 .44409 .31878 .43495 .30628 Metetra
.407 .584 .913 r
.43495 .30628 .44409 .31878 .45982 .32153 .45071 .30865 Metetra
.408 .578 .908 r
.45071 .30865 .45982 .32153 .47572 .32439 .46664 .31113 Metetra
.436 .586 .903 r
.46664 .31113 .47572 .32439 .49182 .32669 .48277 .31306 Metetra
.479 .602 .897 r
.48277 .31306 .49182 .32669 .5081 .32796 .49907 .31393 Metetra
.523 .619 .889 r
.49907 .31393 .5081 .32796 .52454 .32795 .51555 .31351 Metetra
.561 .631 .878 r
.51555 .31351 .52454 .32795 .54114 .32661 .53218 .31176 Metetra
.591 .64 .868 r
.53218 .31176 .54114 .32661 .55787 .32407 .54895 .30883 Metetra
.612 .645 .859 r
.54895 .30883 .55787 .32407 .57473 .32052 .56583 .30495 Metetra
.626 .647 .852 r
.56583 .30495 .57473 .32052 .59169 .3162 .58283 .30034 Metetra
.635 .648 .846 r
.58283 .30034 .59169 .3162 .60876 .31132 .59994 .29522 Metetra
.64 .649 .842 r
.59994 .29522 .60876 .31132 .62594 .30604 .61715 .28975 Metetra
.644 .648 .84 r
.61715 .28975 .62594 .30604 .64321 .30049 .63446 .28404 Metetra
.646 .648 .838 r
.63446 .28404 .64321 .30049 .66059 .29476 .65188 .27819 Metetra
.647 .648 .837 r
.65188 .27819 .66059 .29476 .67808 .28891 .66941 .27222 Metetra
.647 .648 .837 r
.66941 .27222 .67808 .28891 .69568 .28297 .68704 .26619 Metetra
.648 .648 .836 r
.68704 .26619 .69568 .28297 .71338 .27696 .70479 .26009 Metetra
.648 .648 .836 r
.70479 .26009 .71338 .27696 .73119 .2709 .72265 .25395 Metetra
.648 .648 .836 r
.72265 .25395 .73119 .2709 .74912 .26479 .74062 .24776 Metetra
.648 .648 .836 r
.74062 .24776 .74912 .26479 .76715 .25864 .7587 .24153 Metetra
.648 .648 .836 r
.7587 .24153 .76715 .25864 .78531 .25245 .7769 .23526 Metetra
.648 .648 .836 r
.7769 .23526 .78531 .25245 .80357 .24622 .79521 .22895 Metetra
.664 .741 .908 r
.15534 .37114 .1655 .38059 .17979 .37549 .16968 .36598 Metetra
.664 .741 .908 r
.16968 .36598 .17979 .37549 .19417 .37035 .1841 .36079 Metetra
.664 .741 .908 r
.1841 .36079 .19417 .37035 .20862 .36518 .19861 .35557 Metetra
.664 .741 .908 r
.19861 .35557 .20862 .36518 .22316 .35999 .21319 .35032 Metetra
.664 .741 .908 r
.21319 .35032 .22316 .35999 .23778 .35476 .22786 .34504 Metetra
.664 .741 .908 r
.22786 .34504 .23778 .35476 .25248 .34951 .24261 .33973 Metetra
.664 .741 .908 r
.24261 .33973 .25248 .34951 .26726 .34422 .25744 .33439 Metetra
.664 .741 .908 r
.25744 .33439 .26726 .34422 .28213 .33891 .27236 .32902 Metetra
.663 .741 .908 r
.27236 .32902 .28213 .33891 .29708 .33357 .28736 .32362 Metetra
.663 .741 .908 r
.28736 .32362 .29708 .33357 .31212 .32822 .30245 .3182 Metetra
.662 .741 .908 r
.30245 .3182 .31212 .32822 .32724 .32289 .31762 .31279 Metetra
.659 .739 .909 r
.31762 .31279 .32724 .32289 .34244 .31771 .33288 .30748 Metetra
.65 .735 .911 r
.33288 .30748 .34244 .31771 .3577 .31291 .3482 .30246 Metetra
.632 .726 .915 r
.3482 .30246 .3577 .31291 .37302 .30885 .36359 .29808 Metetra
.601 .709 .92 r
.36359 .29808 .37302 .30885 .38839 .30594 .37902 .29476 Metetra
.556 .682 .924 r
.37902 .29476 .38839 .30594 .40382 .30454 .39451 .29287 Metetra
.501 .649 .924 r
.39451 .29287 .40382 .30454 .41933 .30473 .41006 .29258 Metetra
.451 .615 .92 r
.41006 .29258 .41933 .30473 .43495 .30628 .42572 .29368 Metetra
.421 .592 .914 r
.42572 .29368 .43495 .30628 .45071 .30865 .44151 .29564 Metetra
.422 .585 .908 r
.44151 .29564 .45071 .30865 .46664 .31113 .45748 .29774 Metetra
.449 .593 .903 r
.45748 .29774 .46664 .31113 .48277 .31306 .47363 .29927 Metetra
.491 .608 .896 r
.47363 .29927 .48277 .31306 .49907 .31393 .48997 .29974 Metetra
.534 .623 .887 r
.48997 .29974 .49907 .31393 .51555 .31351 .50648 .29888 Metetra
.571 .635 .876 r
.50648 .29888 .51555 .31351 .53218 .31176 .52314 .29672 Metetra
.598 .642 .865 r
.52314 .29672 .53218 .31176 .54895 .30883 .53994 .2934 Metetra
.617 .646 .856 r
.53994 .2934 .54895 .30883 .56583 .30495 .55686 .28917 Metetra
.63 .648 .849 r
.55686 .28917 .56583 .30495 .58283 .30034 .5739 .28428 Metetra
.638 .649 .844 r
.5739 .28428 .58283 .30034 .59994 .29522 .59104 .27893 Metetra
.642 .649 .841 r
.59104 .27893 .59994 .29522 .61715 .28975 .60829 .27328 Metetra
.645 .648 .839 r
.60829 .27328 .61715 .28975 .63446 .28404 .62564 .26743 Metetra
.646 .648 .838 r
.62564 .26743 .63446 .28404 .65188 .27819 .6431 .26145 Metetra
.647 .648 .837 r
.6431 .26145 .65188 .27819 .66941 .27222 .66067 .25539 Metetra
.647 .648 .836 r
.66067 .25539 .66941 .27222 .68704 .26619 .67834 .24926 Metetra
.648 .648 .836 r
.67834 .24926 .68704 .26619 .70479 .26009 .69613 .24308 Metetra
.648 .648 .836 r
.69613 .24308 .70479 .26009 .72265 .25395 .71403 .23685 Metetra
.648 .647 .836 r
.71403 .23685 .72265 .25395 .74062 .24776 .73205 .23058 Metetra
.648 .647 .836 r
.73205 .23058 .74062 .24776 .7587 .24153 .75017 .22427 Metetra
.648 .647 .836 r
.75017 .22427 .7587 .24153 .7769 .23526 .76842 .21792 Metetra
.648 .647 .836 r
.76842 .21792 .7769 .23526 .79521 .22895 .78678 .21153 Metetra
.664 .741 .908 r
.14508 .36159 .15534 .37114 .16968 .36598 .15947 .35637 Metetra
.664 .741 .908 r
.15947 .35637 .16968 .36598 .1841 .36079 .17394 .35113 Metetra
.664 .741 .908 r
.17394 .35113 .1841 .36079 .19861 .35557 .18849 .34585 Metetra
.664 .741 .908 r
.18849 .34585 .19861 .35557 .21319 .35032 .20312 .34055 Metetra
.664 .741 .908 r
.20312 .34055 .21319 .35032 .22786 .34504 .21783 .33521 Metetra
.664 .741 .908 r
.21783 .33521 .22786 .34504 .24261 .33973 .23263 .32985 Metetra
.664 .741 .908 r
.23263 .32985 .24261 .33973 .25744 .33439 .24752 .32445 Metetra
.664 .741 .908 r
.24752 .32445 .25744 .33439 .27236 .32902 .26248 .31903 Metetra
.664 .741 .908 r
.26248 .31903 .27236 .32902 .28736 .32362 .27754 .31357 Metetra
.663 .741 .908 r
.27754 .31357 .28736 .32362 .30245 .3182 .29268 .30809 Metetra
.663 .741 .908 r
.29268 .30809 .30245 .3182 .31762 .31279 .30791 .3026 Metetra
.66 .74 .909 r
.30791 .3026 .31762 .31279 .33288 .30748 .32322 .29716 Metetra
.654 .737 .91 r
.32322 .29716 .33288 .30748 .3482 .30246 .3386 .29195 Metetra
.639 .729 .913 r
.3386 .29195 .3482 .30246 .36359 .29808 .35405 .28726 Metetra
.612 .714 .918 r
.35405 .28726 .36359 .29808 .37902 .29476 .36955 .28352 Metetra
.57 .69 .922 r
.36955 .28352 .37902 .29476 .39451 .29287 .3851 .28115 Metetra
.517 .658 .923 r
.3851 .28115 .39451 .29287 .41006 .29258 .4007 .28035 Metetra
.467 .624 .92 r
.4007 .28035 .41006 .29258 .42572 .29368 .4164 .28099 Metetra
.436 .6 .914 r
.4164 .28099 .42572 .29368 .44151 .29564 .43223 .28252 Metetra
.435 .592 .908 r
.43223 .28252 .44151 .29564 .45748 .29774 .44823 .28422 Metetra
.462 .599 .903 r
.44823 .28422 .45748 .29774 .47363 .29927 .46442 .28534 Metetra
.503 .614 .895 r
.46442 .28534 .47363 .29927 .48997 .29974 .48079 .28537 Metetra
.545 .628 .885 r
.48079 .28537 .48997 .29974 .50648 .29888 .49733 .28407 Metetra
.58 .638 .873 r
.49733 .28407 .50648 .29888 .52314 .29672 .51403 .28147 Metetra
.606 .644 .863 r
.51403 .28147 .52314 .29672 .53994 .2934 .53086 .27775 Metetra
.623 .648 .854 r
.53086 .27775 .53994 .2934 .55686 .28917 .54782 .27319 Metetra
.634 .649 .847 r
.54782 .27319 .55686 .28917 .5739 .28428 .56489 .26803 Metetra
.64 .649 .843 r
.56489 .26803 .5739 .28428 .59104 .27893 .58207 .26246 Metetra
.644 .649 .84 r
.58207 .26246 .59104 .27893 .60829 .27328 .59935 .25664 Metetra
.646 .648 .838 r
.59935 .25664 .60829 .27328 .62564 .26743 .61675 .25066 Metetra
.647 .648 .837 r
.61675 .25066 .62564 .26743 .6431 .26145 .63425 .24457 Metetra
.647 .648 .836 r
.63425 .24457 .6431 .26145 .66067 .25539 .65185 .2384 Metetra
.648 .648 .836 r
.65185 .2384 .66067 .25539 .67834 .24926 .66957 .23219 Metetra
.648 .647 .836 r
.66957 .23219 .67834 .24926 .69613 .24308 .68741 .22592 Metetra
.648 .647 .836 r
.68741 .22592 .69613 .24308 .71403 .23685 .70535 .21961 Metetra
.648 .647 .836 r
.70535 .21961 .71403 .23685 .73205 .23058 .72341 .21326 Metetra
.648 .647 .836 r
.72341 .21326 .73205 .23058 .75017 .22427 .74158 .20687 Metetra
.648 .647 .836 r
.74158 .20687 .75017 .22427 .76842 .21792 .75987 .20044 Metetra
.648 .647 .836 r
.75987 .20044 .76842 .21792 .78678 .21153 .77828 .19396 Metetra
.664 .741 .908 r
.13472 .35194 .14508 .36159 .15947 .35637 .14915 .34667 Metetra
.664 .741 .908 r
.14915 .34667 .15947 .35637 .17394 .35113 .16366 .34137 Metetra
.664 .741 .908 r
.16366 .34137 .17394 .35113 .18849 .34585 .17826 .33604 Metetra
.664 .741 .908 r
.17826 .33604 .18849 .34585 .20312 .34055 .19294 .33068 Metetra
.664 .741 .908 r
.19294 .33068 .20312 .34055 .21783 .33521 .2077 .32529 Metetra
.664 .741 .908 r
.2077 .32529 .21783 .33521 .23263 .32985 .22255 .31986 Metetra
.664 .741 .908 r
.22255 .31986 .23263 .32985 .24752 .32445 .23748 .31441 Metetra
.664 .741 .908 r
.23748 .31441 .24752 .32445 .26248 .31903 .2525 .30893 Metetra
.664 .741 .908 r
.2525 .30893 .26248 .31903 .27754 .31357 .26761 .30341 Metetra
.663 .741 .908 r
.26761 .30341 .27754 .31357 .29268 .30809 .2828 .29786 Metetra
.663 .741 .908 r
.2828 .29786 .29268 .30809 .30791 .3026 .29809 .2923 Metetra
.662 .74 .908 r
.29809 .2923 .30791 .3026 .32322 .29716 .31345 .28676 Metetra
.657 .738 .909 r
.31345 .28676 .32322 .29716 .3386 .29195 .3289 .28137 Metetra
.645 .732 .912 r
.3289 .28137 .3386 .29195 .35405 .28726 .34441 .2764 Metetra
.622 .719 .916 r
.34441 .2764 .35405 .28726 .36955 .28352 .35998 .27225 Metetra
.584 .697 .92 r
.35998 .27225 .36955 .28352 .3851 .28115 .37559 .26938 Metetra
.534 .667 .922 r
.37559 .26938 .3851 .28115 .4007 .28035 .39125 .26805 Metetra
.483 .633 .92 r
.39125 .26805 .4007 .28035 .4164 .28099 .407 .26819 Metetra
.451 .608 .914 r
.407 .26819 .4164 .28099 .43223 .28252 .42287 .26928 Metetra
.449 .599 .908 r
.42287 .26928 .43223 .28252 .44823 .28422 .4389 .27056 Metetra
.475 .606 .902 r
.4389 .27056 .44823 .28422 .46442 .28534 .45512 .27125 Metetra
.515 .619 .893 r
.45512 .27125 .46442 .28534 .48079 .28537 .47152 .27083 Metetra
.556 .632 .882 r
.47152 .27083 .48079 .28537 .49733 .28407 .4881 .26906 Metetra
.589 .641 .87 r
.4881 .26906 .49733 .28407 .51403 .28147 .50484 .26601 Metetra
.613 .646 .86 r
.50484 .26601 .51403 .28147 .53086 .27775 .52171 .26189 Metetra
.628 .649 .851 r
.52171 .26189 .53086 .27775 .54782 .27319 .5387 .257 Metetra
.637 .649 .845 r
.5387 .257 .54782 .27319 .56489 .26803 .55581 .25157 Metetra
.642 .649 .841 r
.55581 .25157 .56489 .26803 .58207 .26246 .57303 .2458 Metetra
.645 .648 .839 r
.57303 .2458 .58207 .26246 .59935 .25664 .59035 .23983 Metetra
.647 .648 .837 r
.59035 .23983 .59935 .25664 .61675 .25066 .60778 .23372 Metetra
.647 .648 .837 r
.60778 .23372 .61675 .25066 .63425 .24457 .62532 .22753 Metetra
.648 .648 .836 r
.62532 .22753 .63425 .24457 .65185 .2384 .64297 .22127 Metetra
.648 .647 .836 r
.64297 .22127 .65185 .2384 .66957 .23219 .66073 .21497 Metetra
.648 .647 .836 r
.66073 .21497 .66957 .23219 .68741 .22592 .67861 .20862 Metetra
.648 .647 .836 r
.67861 .20862 .68741 .22592 .70535 .21961 .6966 .20223 Metetra
.648 .647 .836 r
.6966 .20223 .70535 .21961 .72341 .21326 .7147 .19579 Metetra
.648 .647 .836 r
.7147 .19579 .72341 .21326 .74158 .20687 .73292 .18932 Metetra
.648 .647 .836 r
.73292 .18932 .74158 .20687 .75987 .20044 .75125 .1828 Metetra
.648 .647 .836 r
.75125 .1828 .75987 .20044 .77828 .19396 .76971 .17625 Metetra
.664 .741 .908 r
.12424 .34218 .13472 .35194 .14915 .34667 .13872 .33686 Metetra
.664 .741 .908 r
.13872 .33686 .14915 .34667 .16366 .34137 .15328 .3315 Metetra
.664 .741 .908 r
.15328 .3315 .16366 .34137 .17826 .33604 .16793 .32612 Metetra
.664 .741 .908 r
.16793 .32612 .17826 .33604 .19294 .33068 .18265 .3207 Metetra
.664 .741 .908 r
.18265 .3207 .19294 .33068 .2077 .32529 .19747 .31525 Metetra
.664 .741 .908 r
.19747 .31525 .2077 .32529 .22255 .31986 .21236 .30977 Metetra
.664 .741 .908 r
.21236 .30977 .22255 .31986 .23748 .31441 .22735 .30426 Metetra
.664 .741 .908 r
.22735 .30426 .23748 .31441 .2525 .30893 .24242 .29872 Metetra
.664 .741 .908 r
.24242 .29872 .2525 .30893 .26761 .30341 .25758 .29314 Metetra
.664 .741 .908 r
.25758 .29314 .26761 .30341 .2828 .29786 .27282 .28754 Metetra
.663 .741 .908 r
.27282 .28754 .2828 .29786 .29809 .2923 .28816 .2819 Metetra
.663 .741 .908 r
.28816 .2819 .29809 .2923 .31345 .28676 .30358 .27627 Metetra
.659 .739 .909 r
.30358 .27627 .31345 .28676 .3289 .28137 .31908 .27073 Metetra
.651 .734 .911 r
.31908 .27073 .3289 .28137 .34441 .2764 .33466 .26549 Metetra
.631 .724 .914 r
.33466 .26549 .34441 .2764 .35998 .27225 .3503 .26095 Metetra
.597 .704 .918 r
.3503 .26095 .35998 .27225 .37559 .26938 .36597 .25756 Metetra
.55 .675 .921 r
.36597 .25756 .37559 .26938 .39125 .26805 .3817 .25569 Metetra
.5 .643 .919 r
.3817 .25569 .39125 .26805 .407 .26819 .3975 .2553 Metetra
.467 .617 .914 r
.3975 .2553 .407 .26819 .42287 .26928 .41341 .25592 Metetra
.463 .607 .908 r
.41341 .25592 .42287 .26928 .4389 .27056 .42948 .25676 Metetra
.488 .612 .901 r
.42948 .25676 .4389 .27056 .45512 .27125 .44573 .257 Metetra
.528 .625 .891 r
.44573 .257 .45512 .27125 .47152 .27083 .46218 .2561 Metetra
.567 .637 .879 r
.46218 .2561 .47152 .27083 .4881 .26906 .4788 .25384 Metetra
.598 .644 .867 r
.4788 .25384 .4881 .26906 .50484 .26601 .49557 .25032 Metetra
.62 .648 .857 r
.49557 .25032 .50484 .26601 .52171 .26189 .51248 .24581 Metetra
.633 .649 .849 r
.51248 .24581 .52171 .26189 .5387 .257 .52951 .24059 Metetra
.64 .649 .843 r
.52951 .24059 .5387 .257 .55581 .25157 .54666 .23491 Metetra
.644 .649 .84 r
.54666 .23491 .55581 .25157 .57303 .2458 .56391 .22896 Metetra
.646 .648 .838 r
.56391 .22896 .57303 .2458 .59035 .23983 .58127 .22284 Metetra
.647 .648 .837 r
.58127 .22284 .59035 .23983 .60778 .23372 .59874 .21662 Metetra
.647 .648 .836 r
.59874 .21662 .60778 .23372 .62532 .22753 .61632 .21033 Metetra
.648 .647 .836 r
.61632 .21033 .62532 .22753 .64297 .22127 .63402 .20399 Metetra
.648 .647 .836 r
.63402 .20399 .64297 .22127 .66073 .21497 .65182 .1976 Metetra
.648 .647 .836 r
.65182 .1976 .66073 .21497 .67861 .20862 .66974 .19117 Metetra
.648 .647 .836 r
.66974 .19117 .67861 .20862 .6966 .20223 .68777 .18469 Metetra
.648 .647 .836 r
.68777 .18469 .6966 .20223 .7147 .19579 .70592 .17818 Metetra
.648 .647 .836 r
.70592 .17818 .7147 .19579 .73292 .18932 .72418 .17162 Metetra
.648 .647 .836 r
.72418 .17162 .73292 .18932 .75125 .1828 .74257 .16502 Metetra
.648 .647 .836 r
.74257 .16502 .75125 .1828 .76971 .17625 .76107 .15838 Metetra
.664 .741 .908 r
.11366 .33233 .12424 .34218 .13872 .33686 .12818 .32695 Metetra
.664 .741 .908 r
.12818 .32695 .13872 .33686 .15328 .3315 .14279 .32154 Metetra
.664 .741 .908 r
.14279 .32154 .15328 .3315 .16793 .32612 .15748 .31609 Metetra
.664 .741 .908 r
.15748 .31609 .16793 .32612 .18265 .3207 .17226 .31062 Metetra
.664 .741 .908 r
.17226 .31062 .18265 .3207 .19747 .31525 .18712 .30511 Metetra
.664 .741 .908 r
.18712 .30511 .19747 .31525 .21236 .30977 .20207 .29957 Metetra
.664 .741 .908 r
.20207 .29957 .21236 .30977 .22735 .30426 .2171 .294 Metetra
.664 .741 .908 r
.2171 .294 .22735 .30426 .24242 .29872 .23222 .2884 Metetra
.664 .741 .908 r
.23222 .2884 .24242 .29872 .25758 .29314 .24743 .28276 Metetra
.664 .741 .908 r
.24743 .28276 .25758 .29314 .27282 .28754 .26273 .2771 Metetra
.663 .741 .908 r
.26273 .2771 .27282 .28754 .28816 .2819 .27812 .2714 Metetra
.663 .741 .908 r
.27812 .2714 .28816 .2819 .30358 .27627 .2936 .26568 Metetra
.661 .74 .908 r
.2936 .26568 .30358 .27627 .31908 .27073 .30916 .26001 Metetra
.655 .737 .91 r
.30916 .26001 .31908 .27073 .33466 .26549 .32481 .25454 Metetra
.64 .728 .912 r
.32481 .25454 .33466 .26549 .3503 .26095 .34051 .24961 Metetra
.611 .711 .916 r
.34051 .24961 .3503 .26095 .36597 .25756 .35626 .24572 Metetra
.567 .684 .919 r
.35626 .24572 .36597 .25756 .3817 .25569 .37205 .24327 Metetra
.518 .652 .919 r
.37205 .24327 .3817 .25569 .3975 .2553 .3879 .24232 Metetra
.483 .625 .914 r
.3879 .24232 .3975 .2553 .41341 .25592 .40386 .24244 Metetra
.478 .614 .907 r
.40386 .24244 .41341 .25592 .42948 .25676 .41997 .24281 Metetra
.502 .619 .9 r
.41997 .24281 .42948 .25676 .44573 .257 .43627 .24258 Metetra
.541 .631 .889 r
.43627 .24258 .44573 .257 .46218 .2561 .45275 .24117 Metetra
.579 .641 .876 r
.45275 .24117 .46218 .2561 .4788 .25384 .46941 .23839 Metetra
.607 .647 .864 r
.46941 .23839 .4788 .25384 .49557 .25032 .48622 .2344 Metetra
.626 .65 .854 r
.48622 .2344 .49557 .25032 .51248 .24581 .50317 .22949 Metetra
.637 .65 .846 r
.50317 .22949 .51248 .24581 .52951 .24059 .52024 .22396 Metetra
.643 .649 .841 r
.52024 .22396 .52951 .24059 .54666 .23491 .53743 .21806 Metetra
.646 .649 .839 r
.53743 .21806 .54666 .23491 .56391 .22896 .55472 .21194 Metetra
.647 .648 .837 r
.55472 .21194 .56391 .22896 .58127 .22284 .57212 .20569 Metetra
.647 .648 .836 r
.57212 .20569 .58127 .22284 .59874 .21662 .58963 .19937 Metetra
.648 .647 .836 r
.58963 .19937 .59874 .21662 .61632 .21033 .60726 .19299 Metetra
.648 .647 .836 r
.60726 .19299 .61632 .21033 .63402 .20399 .62499 .18656 Metetra
.648 .647 .836 r
.62499 .18656 .63402 .20399 .65182 .1976 .64284 .18009 Metetra
.648 .647 .836 r
.64284 .18009 .65182 .1976 .66974 .19117 .6608 .17357 Metetra
.648 .647 .836 r
.6608 .17357 .66974 .19117 .68777 .18469 .67887 .16701 Metetra
.648 .647 .836 r
.67887 .16701 .68777 .18469 .70592 .17818 .69707 .16042 Metetra
.648 .647 .836 r
.69707 .16042 .70592 .17818 .72418 .17162 .71538 .15377 Metetra
.648 .647 .836 r
.71538 .15377 .72418 .17162 .74257 .16502 .73381 .14709 Metetra
.648 .647 .836 r
.73381 .14709 .74257 .16502 .76107 .15838 .75236 .14036 Metetra
.664 .741 .908 r
.10296 .32237 .11366 .33233 .12818 .32695 .11753 .31693 Metetra
.664 .741 .908 r
.11753 .31693 .12818 .32695 .14279 .32154 .13219 .31146 Metetra
.664 .741 .908 r
.13219 .31146 .14279 .32154 .15748 .31609 .14693 .30596 Metetra
.664 .741 .908 r
.14693 .30596 .15748 .31609 .17226 .31062 .16175 .30043 Metetra
.664 .741 .908 r
.16175 .30043 .17226 .31062 .18712 .30511 .17666 .29486 Metetra
.664 .741 .908 r
.17666 .29486 .18712 .30511 .20207 .29957 .19166 .28926 Metetra
.664 .741 .908 r
.19166 .28926 .20207 .29957 .2171 .294 .20674 .28363 Metetra
.664 .741 .908 r
.20674 .28363 .2171 .294 .23222 .2884 .22192 .27797 Metetra
.664 .741 .908 r
.22192 .27797 .23222 .2884 .24743 .28276 .23718 .27227 Metetra
.664 .741 .908 r
.23718 .27227 .24743 .28276 .26273 .2771 .25253 .26654 Metetra
.664 .741 .908 r
.25253 .26654 .26273 .2771 .27812 .2714 .26797 .26078 Metetra
.663 .741 .908 r
.26797 .26078 .27812 .2714 .2936 .26568 .28351 .25499 Metetra
.662 .741 .908 r
.28351 .25499 .2936 .26568 .30916 .26001 .29913 .2492 Metetra
.659 .738 .909 r
.29913 .2492 .30916 .26001 .32481 .25454 .31484 .24353 Metetra
.647 .732 .911 r
.31484 .24353 .32481 .25454 .34051 .24961 .33061 .23826 Metetra
.623 .718 .914 r
.33061 .23826 .34051 .24961 .35626 .24572 .34644 .23386 Metetra
.584 .694 .918 r
.34644 .23386 .35626 .24572 .37205 .24327 .3623 .2308 Metetra
.536 .662 .918 r
.3623 .2308 .37205 .24327 .3879 .24232 .37821 .22926 Metetra
.5 .634 .913 r
.37821 .22926 .3879 .24232 .40386 .24244 .39422 .22885 Metetra
.493 .622 .907 r
.39422 .22885 .40386 .24244 .41997 .24281 .41037 .22872 Metetra
.517 .626 .898 r
.41037 .22872 .41997 .24281 .43627 .24258 .42671 .22798 Metetra
.555 .636 .886 r
.42671 .22798 .43627 .24258 .45275 .24117 .44324 .22602 Metetra
.591 .645 .873 r
.44324 .22602 .45275 .24117 .46941 .23839 .45995 .2227 Metetra
.616 .649 .86 r
.45995 .2227 .46941 .23839 .48622 .2344 .4768 .21823 Metetra
.632 .651 .85 r
.4768 .21823 .48622 .2344 .50317 .22949 .49379 .21293 Metetra
.641 .65 .844 r
.49379 .21293 .50317 .22949 .52024 .22396 .5109 .20711 Metetra
.645 .649 .84 r
.5109 .20711 .52024 .22396 .53743 .21806 .52812 .20101 Metetra
.647 .648 .838 r
.52812 .20101 .53743 .21806 .55472 .21194 .54546 .19474 Metetra
.647 .648 .836 r
.54546 .19474 .55472 .21194 .57212 .20569 .5629 .18838 Metetra
.648 .647 .836 r
.5629 .18838 .57212 .20569 .58963 .19937 .58045 .18196 Metetra
.648 .647 .836 r
.58045 .18196 .58963 .19937 .60726 .19299 .59811 .17549 Metetra
.648 .647 .836 r
.59811 .17549 .60726 .19299 .62499 .18656 .61589 .16898 Metetra
.648 .647 .836 r
.61589 .16898 .62499 .18656 .64284 .18009 .63378 .16242 Metetra
.648 .647 .836 r
.63378 .16242 .64284 .18009 .6608 .17357 .65179 .15583 Metetra
.648 .647 .836 r
.65179 .15583 .6608 .17357 .67887 .16701 .66991 .14919 Metetra
.648 .647 .836 r
.66991 .14919 .67887 .16701 .69707 .16042 .68814 .1425 Metetra
.648 .647 .836 r
.68814 .1425 .69707 .16042 .71538 .15377 .7065 .13578 Metetra
.648 .647 .836 r
.7065 .13578 .71538 .15377 .73381 .14709 .72498 .12901 Metetra
.648 .647 .836 r
.72498 .12901 .73381 .14709 .75236 .14036 .74357 .12219 Metetra
.664 .741 .908 r
.09215 .3123 .10296 .32237 .11753 .31693 .10677 .30681 Metetra
.664 .741 .908 r
.10677 .30681 .11753 .31693 .13219 .31146 .12147 .30128 Metetra
.664 .741 .908 r
.12147 .30128 .13219 .31146 .14693 .30596 .13626 .29572 Metetra
.664 .741 .908 r
.13626 .29572 .14693 .30596 .16175 .30043 .15113 .29013 Metetra
.664 .741 .908 r
.15113 .29013 .16175 .30043 .17666 .29486 .16609 .2845 Metetra
.664 .741 .908 r
.16609 .2845 .17666 .29486 .19166 .28926 .18114 .27884 Metetra
.664 .741 .908 r
.18114 .27884 .19166 .28926 .20674 .28363 .19627 .27315 Metetra
.664 .741 .908 r
.19627 .27315 .20674 .28363 .22192 .27797 .2115 .26743 Metetra
.664 .741 .908 r
.2115 .26743 .22192 .27797 .23718 .27227 .22681 .26167 Metetra
.664 .741 .908 r
.22681 .26167 .23718 .27227 .25253 .26654 .24222 .25587 Metetra
.664 .741 .908 r
.24222 .25587 .25253 .26654 .26797 .26078 .25772 .25005 Metetra
.663 .741 .908 r
.25772 .25005 .26797 .26078 .28351 .25499 .27331 .24419 Metetra
.663 .741 .908 r
.27331 .24419 .28351 .25499 .29913 .2492 .28899 .23831 Metetra
.661 .74 .908 r
.28899 .23831 .29913 .2492 .31484 .24353 .30476 .23248 Metetra
.654 .736 .91 r
.30476 .23248 .31484 .24353 .33061 .23826 .3206 .2269 Metetra
.635 .724 .912 r
.3206 .2269 .33061 .23826 .34644 .23386 .3365 .22199 Metetra
.6 .703 .915 r
.3365 .22199 .34644 .23386 .3623 .2308 .35244 .2183 Metetra
.555 .672 .916 r
.35244 .2183 .3623 .2308 .37821 .22926 .36842 .21612 Metetra
.517 .644 .912 r
.36842 .21612 .37821 .22926 .39422 .22885 .38449 .21513 Metetra
.509 .63 .905 r
.38449 .21513 .39422 .22885 .41037 .22872 .40069 .21448 Metetra
.532 .633 .896 r
.40069 .21448 .41037 .22872 .42671 .22798 .41707 .21319 Metetra
.569 .642 .883 r
.41707 .21319 .42671 .22798 .44324 .22602 .43365 .21064 Metetra
.602 .649 .869 r
.43365 .21064 .44324 .22602 .45995 .2227 .4504 .20675 Metetra
.625 .651 .856 r
.4504 .20675 .45995 .2227 .4768 .21823 .4673 .20178 Metetra
.637 .651 .847 r
.4673 .20178 .4768 .21823 .49379 .21293 .48434 .19611 Metetra
.644 .65 .841 r
.48434 .19611 .49379 .21293 .5109 .20711 .50149 .19004 Metetra
.646 .649 .838 r
.50149 .19004 .5109 .20711 .52812 .20101 .51875 .18376 Metetra
.647 .648 .837 r
.51875 .18376 .52812 .20101 .54546 .19474 .53612 .17737 Metetra
.648 .647 .836 r
.53612 .17737 .54546 .19474 .5629 .18838 .5536 .17091 Metetra
.648 .647 .836 r
.5536 .17091 .5629 .18838 .58045 .18196 .57119 .1644 Metetra
.648 .647 .836 r
.57119 .1644 .58045 .18196 .59811 .17549 .5889 .15785 Metetra
.648 .647 .835 r
.5889 .15785 .59811 .17549 .61589 .16898 .60672 .15125 Metetra
.648 .647 .835 r
.60672 .15125 .61589 .16898 .63378 .16242 .62465 .14461 Metetra
.648 .647 .835 r
.62465 .14461 .63378 .16242 .65179 .15583 .6427 .13793 Metetra
.648 .647 .835 r
.6427 .13793 .65179 .15583 .66991 .14919 .66086 .13121 Metetra
.648 .647 .835 r
.66086 .13121 .66991 .14919 .68814 .1425 .67915 .12444 Metetra
.648 .647 .835 r
.67915 .12444 .68814 .1425 .7065 .13578 .69755 .11762 Metetra
.648 .647 .835 r
.69755 .11762 .7065 .13578 .72498 .12901 .71607 .11077 Metetra
.648 .647 .835 r
.71607 .11077 .72498 .12901 .74357 .12219 .73472 .10386 Metetra
.664 .741 .908 r
.08123 .30213 .09215 .3123 .10677 .30681 .09589 .29658 Metetra
.664 .741 .908 r
.09589 .29658 .10677 .30681 .12147 .30128 .11064 .29099 Metetra
.664 .741 .908 r
.11064 .29099 .12147 .30128 .13626 .29572 .12548 .28537 Metetra
.664 .741 .908 r
.12548 .28537 .13626 .29572 .15113 .29013 .1404 .27972 Metetra
.664 .741 .908 r
.1404 .27972 .15113 .29013 .16609 .2845 .15541 .27403 Metetra
.664 .741 .908 r
.15541 .27403 .16609 .2845 .18114 .27884 .1705 .26831 Metetra
.664 .741 .908 r
.1705 .26831 .18114 .27884 .19627 .27315 .18569 .26256 Metetra
.664 .741 .908 r
.18569 .26256 .19627 .27315 .2115 .26743 .20097 .25677 Metetra
.664 .741 .908 r
.20097 .25677 .2115 .26743 .22681 .26167 .21634 .25095 Metetra
.664 .741 .908 r
.21634 .25095 .22681 .26167 .24222 .25587 .23179 .24509 Metetra
.664 .741 .908 r
.23179 .24509 .24222 .25587 .25772 .25005 .24735 .2392 Metetra
.664 .741 .908 r
.24735 .2392 .25772 .25005 .27331 .24419 .26299 .23327 Metetra
.663 .741 .908 r
.26299 .23327 .27331 .24419 .28899 .23831 .27873 .22731 Metetra
.663 .741 .908 r
.27873 .22731 .28899 .23831 .30476 .23248 .29456 .22135 Metetra
.658 .738 .909 r
.29456 .22135 .30476 .23248 .3206 .2269 .31047 .21552 Metetra
.646 .73 .91 r
.31047 .21552 .3206 .2269 .3365 .22199 .32645 .21015 Metetra
.617 .712 .913 r
.32645 .21015 .3365 .22199 .35244 .2183 .34248 .20581 Metetra
.575 .683 .914 r
.34248 .20581 .35244 .2183 .36842 .21612 .35853 .20292 Metetra
.536 .654 .911 r
.35853 .20292 .36842 .21612 .38449 .21513 .37466 .2013 Metetra
.526 .638 .904 r
.37466 .2013 .38449 .21513 .40069 .21448 .39091 .20008 Metetra
.548 .64 .893 r
.39091 .20008 .40069 .21448 .41707 .21319 .40735 .19818 Metetra
.584 .647 .879 r
.40735 .19818 .41707 .21319 .43365 .21064 .42398 .19499 Metetra
.614 .652 .864 r
.42398 .19499 .43365 .21064 .4504 .20675 .44078 .1905 Metetra
.633 .653 .852 r
.44078 .1905 .4504 .20675 .4673 .20178 .45773 .18506 Metetra
.642 .651 .844 r
.45773 .18506 .4673 .20178 .48434 .19611 .47481 .17905 Metetra
.646 .649 .839 r
.47481 .17905 .48434 .19611 .50149 .19004 .49199 .17276 Metetra
.647 .648 .837 r
.49199 .17276 .50149 .19004 .51875 .18376 .50929 .16634 Metetra
.648 .647 .836 r
.50929 .16634 .51875 .18376 .53612 .17737 .52671 .15984 Metetra
.648 .647 .836 r
.52671 .15984 .53612 .17737 .5536 .17091 .54423 .15329 Metetra
.648 .647 .835 r
.54423 .15329 .5536 .17091 .57119 .1644 .56186 .14669 Metetra
.648 .647 .835 r
.56186 .14669 .57119 .1644 .5889 .15785 .57961 .14006 Metetra
.648 .647 .835 r
.57961 .14006 .5889 .15785 .60672 .15125 .59747 .13337 Metetra
.648 .647 .835 r
.59747 .13337 .60672 .15125 .62465 .14461 .61544 .12665 Metetra
.648 .647 .835 r
.61544 .12665 .62465 .14461 .6427 .13793 .63354 .11988 Metetra
.648 .647 .835 r
.63354 .11988 .6427 .13793 .66086 .13121 .65175 .11307 Metetra
.648 .647 .835 r
.65175 .11307 .66086 .13121 .67915 .12444 .67008 .10622 Metetra
.648 .647 .835 r
.67008 .10622 .67915 .12444 .69755 .11762 .68852 .09932 Metetra
.648 .647 .835 r
.68852 .09932 .69755 .11762 .71607 .11077 .70709 .09237 Metetra
.648 .647 .835 r
.70709 .09237 .71607 .11077 .73472 .10386 .72578 .08538 Metetra
.664 .741 .908 r
.07019 .29185 .08123 .30213 .09589 .29658 .0849 .28624 Metetra
.664 .741 .908 r
.0849 .28624 .09589 .29658 .11064 .29099 .09969 .28059 Metetra
.664 .741 .908 r
.09969 .28059 .11064 .29099 .12548 .28537 .11458 .27491 Metetra
.664 .741 .908 r
.11458 .27491 .12548 .28537 .1404 .27972 .12955 .26919 Metetra
.664 .741 .908 r
.12955 .26919 .1404 .27972 .15541 .27403 .14461 .26344 Metetra
.664 .741 .908 r
.14461 .26344 .15541 .27403 .1705 .26831 .15975 .25766 Metetra
.664 .741 .908 r
.15975 .25766 .1705 .26831 .18569 .26256 .17499 .25184 Metetra
.664 .741 .908 r
.17499 .25184 .18569 .26256 .20097 .25677 .19032 .24599 Metetra
.664 .741 .908 r
.19032 .24599 .20097 .25677 .21634 .25095 .20574 .24011 Metetra
.664 .741 .908 r
.20574 .24011 .21634 .25095 .23179 .24509 .22125 .23418 Metetra
.664 .741 .908 r
.22125 .23418 .23179 .24509 .24735 .2392 .23686 .22823 Metetra
.664 .741 .908 r
.23686 .22823 .24735 .2392 .26299 .23327 .25256 .22223 Metetra
.664 .741 .908 r
.25256 .22223 .26299 .23327 .27873 .22731 .26835 .21621 Metetra
.663 .741 .908 r
.26835 .21621 .27873 .22731 .29456 .22135 .28424 .21015 Metetra
.662 .74 .908 r
.28424 .21015 .29456 .22135 .31047 .21552 .30022 .20412 Metetra
.654 .735 .909 r
.30022 .20412 .31047 .21552 .32645 .21015 .31628 .19835 Metetra
.633 .721 .911 r
.31628 .19835 .32645 .21015 .34248 .20581 .33239 .19335 Metetra
.595 .695 .912 r
.33239 .19335 .34248 .20581 .35853 .20292 .34854 .18968 Metetra
.556 .665 .91 r
.34854 .18968 .35853 .20292 .37466 .2013 .36473 .18737 Metetra
.543 .647 .902 r
.36473 .18737 .37466 .2013 .39091 .20008 .38105 .18551 Metetra
.565 .647 .89 r
.38105 .18551 .39091 .20008 .40735 .19818 .39754 .18294 Metetra
.6 .653 .874 r
.39754 .18294 .40735 .19818 .42398 .19499 .41423 .17905 Metetra
.626 .655 .859 r
.41423 .17905 .42398 .19499 .44078 .1905 .43109 .17393 Metetra
.64 .653 .847 r
.43109 .17393 .44078 .1905 .45773 .18506 .44808 .16803 Metetra
.645 .65 .84 r
.44808 .16803 .45773 .18506 .47481 .17905 .4652 .16174 Metetra
.647 .648 .837 r
.4652 .16174 .47481 .17905 .49199 .17276 .48243 .15528 Metetra
.648 .647 .836 r
.48243 .15528 .49199 .17276 .50929 .16634 .49977 .14874 Metetra
.648 .647 .836 r
.49977 .14874 .50929 .16634 .52671 .15984 .51722 .14215 Metetra
.648 .647 .835 r
.51722 .14215 .52671 .15984 .54423 .15329 .53478 .13551 Metetra
.648 .647 .835 r
.53478 .13551 .54423 .15329 .56186 .14669 .55245 .12884 Metetra
.648 .647 .835 r
.55245 .12884 .56186 .14669 .57961 .14006 .57024 .12211 Metetra
.648 .647 .835 r
.57024 .12211 .57961 .14006 .59747 .13337 .58815 .11535 Metetra
.648 .647 .835 r
.58815 .11535 .59747 .13337 .61544 .12665 .60616 .10854 Metetra
.648 .647 .835 r
.60616 .10854 .61544 .12665 .63354 .11988 .6243 .10168 Metetra
.648 .647 .835 r
.6243 .10168 .63354 .11988 .65175 .11307 .64256 .09478 Metetra
.648 .647 .835 r
.64256 .09478 .65175 .11307 .67008 .10622 .66093 .08784 Metetra
.648 .647 .835 r
.66093 .08784 .67008 .10622 .68852 .09932 .67942 .08085 Metetra
.648 .647 .835 r
.67942 .08085 .68852 .09932 .70709 .09237 .69804 .07382 Metetra
.648 .647 .835 r
.69804 .07382 .70709 .09237 .72578 .08538 .71678 .06673 Metetra
.664 .741 .908 r
.05902 .28146 .07019 .29185 .0849 .28624 .07378 .27578 Metetra
.664 .741 .908 r
.07378 .27578 .0849 .28624 .09969 .28059 .08863 .27007 Metetra
.664 .741 .908 r
.08863 .27007 .09969 .28059 .11458 .27491 .10356 .26433 Metetra
.664 .741 .908 r
.10356 .26433 .11458 .27491 .12955 .26919 .11858 .25855 Metetra
.664 .741 .908 r
.11858 .25855 .12955 .26919 .14461 .26344 .13369 .25274 Metetra
.664 .741 .908 r
.13369 .25274 .14461 .26344 .15975 .25766 .14889 .24689 Metetra
.664 .741 .908 r
.14889 .24689 .15975 .25766 .17499 .25184 .16417 .24101 Metetra
.664 .741 .908 r
.16417 .24101 .17499 .25184 .19032 .24599 .17956 .2351 Metetra
.664 .741 .908 r
.17956 .2351 .19032 .24599 .20574 .24011 .19503 .22915 Metetra
.664 .741 .908 r
.19503 .22915 .20574 .24011 .22125 .23418 .2106 .22316 Metetra
.664 .741 .908 r
.2106 .22316 .22125 .23418 .23686 .22823 .22626 .21713 Metetra
.664 .741 .908 r
.22626 .21713 .23686 .22823 .25256 .22223 .24201 .21107 Metetra
.664 .741 .908 r
.24201 .21107 .25256 .22223 .26835 .21621 .25786 .20498 Metetra
.664 .741 .908 r
.25786 .20498 .26835 .21621 .28424 .21015 .27381 .19884 Metetra
.663 .741 .908 r
.27381 .19884 .28424 .21015 .30022 .20412 .28986 .19269 Metetra
.66 .739 .908 r
.28986 .19269 .30022 .20412 .31628 .19835 .30599 .18661 Metetra
.648 .73 .909 r
.30599 .18661 .31628 .19835 .33239 .19335 .32219 .18097 Metetra
.617 .708 .91 r
.32219 .18097 .33239 .19335 .34854 .18968 .33843 .17646 Metetra
.578 .677 .908 r
.33843 .17646 .34854 .18968 .36473 .18737 .35471 .17333 Metetra
.563 .657 .899 r
.35471 .17333 .36473 .18737 .38105 .18551 .37109 .17075 Metetra
.584 .655 .885 r
.37109 .17075 .38105 .18551 .39754 .18294 .38765 .16742 Metetra
.616 .658 .868 r
.38765 .16742 .39754 .18294 .41423 .17905 .4044 .16274 Metetra
.637 .656 .852 r
.4044 .16274 .41423 .17905 .43109 .17393 .42132 .15699 Metetra
.645 .652 .843 r
.42132 .15699 .43109 .17393 .44808 .16803 .43836 .15071 Metetra
.647 .649 .838 r
.43836 .15071 .44808 .16803 .4652 .16174 .45552 .14421 Metetra
.648 .647 .836 r
.45552 .14421 .4652 .16174 .48243 .15528 .47278 .13762 Metetra
.648 .647 .835 r
.47278 .13762 .48243 .15528 .49977 .14874 .49016 .13099 Metetra
.648 .647 .835 r
.49016 .13099 .49977 .14874 .51722 .14215 .50765 .12431 Metetra
.648 .647 .835 r
.50765 .12431 .51722 .14215 .53478 .13551 .52525 .11759 Metetra
.648 .647 .835 r
.52525 .11759 .53478 .13551 .55245 .12884 .54297 .11083 Metetra
.648 .647 .835 r
.54297 .11083 .55245 .12884 .57024 .12211 .5608 .10402 Metetra
.648 .647 .835 r
.5608 .10402 .57024 .12211 .58815 .11535 .57875 .09717 Metetra
.648 .647 .835 r
.57875 .09717 .58815 .11535 .60616 .10854 .59681 .09027 Metetra
.648 .647 .835 r
.59681 .09027 .60616 .10854 .6243 .10168 .61499 .08333 Metetra
.648 .647 .835 r
.61499 .08333 .6243 .10168 .64256 .09478 .63329 .07634 Metetra
.648 .647 .835 r
.63329 .07634 .64256 .09478 .66093 .08784 .65171 .06931 Metetra
.648 .647 .835 r
.65171 .06931 .66093 .08784 .67942 .08085 .67025 .06223 Metetra
.648 .647 .835 r
.67025 .06223 .67942 .08085 .69804 .07382 .68891 .0551 Metetra
.648 .647 .835 r
.68891 .0551 .69804 .07382 .71678 .06673 .7077 .04793 Metetra
.664 .741 .908 r
.04774 .27095 .05902 .28146 .07378 .27578 .06255 .26522 Metetra
.664 .741 .908 r
.06255 .26522 .07378 .27578 .08863 .27007 .07744 .25944 Metetra
.664 .741 .908 r
.07744 .25944 .08863 .27007 .10356 .26433 .09242 .25364 Metetra
.664 .741 .908 r
.09242 .25364 .10356 .26433 .11858 .25855 .10749 .2478 Metetra
.664 .741 .908 r
.10749 .2478 .11858 .25855 .13369 .25274 .12265 .24192 Metetra
.664 .741 .908 r
.12265 .24192 .13369 .25274 .14889 .24689 .1379 .23601 Metetra
.664 .741 .908 r
.1379 .23601 .14889 .24689 .16417 .24101 .15324 .23006 Metetra
.664 .741 .908 r
.15324 .23006 .16417 .24101 .17956 .2351 .16867 .22408 Metetra
.664 .741 .908 r
.16867 .22408 .17956 .2351 .19503 .22915 .1842 .21806 Metetra
.664 .741 .908 r
.1842 .21806 .19503 .22915 .2106 .22316 .19982 .21201 Metetra
.664 .741 .908 r
.19982 .21201 .2106 .22316 .22626 .21713 .21553 .20592 Metetra
.664 .741 .908 r
.21553 .20592 .22626 .21713 .24201 .21107 .23135 .19979 Metetra
.664 .741 .908 r
.23135 .19979 .24201 .21107 .25786 .20498 .24725 .19362 Metetra
.664 .741 .908 r
.24725 .19362 .25786 .20498 .27381 .19884 .26326 .18742 Metetra
.664 .741 .908 r
.26326 .18742 .27381 .19884 .28986 .19269 .27936 .18118 Metetra
.663 .741 .908 r
.27936 .18118 .28986 .19269 .30599 .18661 .29557 .17491 Metetra
.659 .737 .908 r
.29557 .17491 .30599 .18661 .32219 .18097 .31186 .16878 Metetra
.64 .722 .908 r
.31186 .16878 .32219 .18097 .33843 .17646 .3282 .16333 Metetra
.604 .692 .905 r
.3282 .16333 .33843 .17646 .35471 .17333 .34458 .15922 Metetra
.585 .668 .896 r
.34458 .15922 .35471 .17333 .37109 .17075 .36104 .15579 Metetra
.606 .663 .879 r
.36104 .15579 .37109 .17075 .38765 .16742 .37768 .15152 Metetra
.633 .662 .86 r
.37768 .15152 .38765 .16742 .4044 .16274 .3945 .14596 Metetra
.645 .655 .845 r
.3945 .14596 .4044 .16274 .42132 .15699 .41147 .13966 Metetra
.648 .65 .838 r
.41147 .13966 .42132 .15699 .43836 .15071 .42856 .13311 Metetra
.648 .647 .836 r
.42856 .13311 .43836 .15071 .45552 .14421 .44576 .12648 Metetra
.648 .647 .835 r
.44576 .12648 .45552 .14421 .47278 .13762 .46306 .1198 Metetra
.648 .647 .835 r
.46306 .1198 .47278 .13762 .49016 .13099 .48048 .11308 Metetra
.648 .647 .835 r
.48048 .11308 .49016 .13099 .50765 .12431 .49801 .10632 Metetra
.648 .647 .835 r
.49801 .10632 .50765 .12431 .52525 .11759 .51565 .09952 Metetra
.648 .647 .835 r
.51565 .09952 .52525 .11759 .54297 .11083 .53341 .09267 Metetra
.648 .647 .835 r
.53341 .09267 .54297 .11083 .5608 .10402 .55128 .08577 Metetra
.648 .647 .835 r
.55128 .08577 .5608 .10402 .57875 .09717 .56927 .07883 Metetra
.648 .647 .835 r
.56927 .07883 .57875 .09717 .59681 .09027 .58738 .07185 Metetra
.648 .647 .835 r
.58738 .07185 .59681 .09027 .61499 .08333 .6056 .06482 Metetra
.648 .647 .835 r
.6056 .06482 .61499 .08333 .63329 .07634 .62394 .05774 Metetra
.648 .647 .835 r
.62394 .05774 .63329 .07634 .65171 .06931 .64241 .05061 Metetra
.648 .647 .835 r
.64241 .05061 .65171 .06931 .67025 .06223 .661 .04344 Metetra
.648 .647 .835 r
.661 .04344 .67025 .06223 .68891 .0551 .67971 .03623 Metetra
.648 .647 .835 r
.67971 .03623 .68891 .0551 .7077 .04793 .69854 .02896 Metetra
.664 .741 .908 r
.03634 .26033 .04774 .27095 .06255 .26522 .05119 .25453 Metetra
.664 .741 .908 r
.05119 .25453 .06255 .26522 .07744 .25944 .06613 .2487 Metetra
.664 .741 .908 r
.06613 .2487 .07744 .25944 .09242 .25364 .08116 .24283 Metetra
.664 .741 .908 r
.08116 .24283 .09242 .25364 .10749 .2478 .09628 .23692 Metetra
.664 .741 .908 r
.09628 .23692 .10749 .2478 .12265 .24192 .11148 .23098 Metetra
.664 .741 .908 r
.11148 .23098 .12265 .24192 .1379 .23601 .12679 .225 Metetra
.664 .741 .908 r
.12679 .225 .1379 .23601 .15324 .23006 .14218 .21899 Metetra
.664 .741 .908 r
.14218 .21899 .15324 .23006 .16867 .22408 .15767 .21294 Metetra
.664 .741 .908 r
.15767 .21294 .16867 .22408 .1842 .21806 .17325 .20686 Metetra
.664 .741 .908 r
.17325 .20686 .1842 .21806 .19982 .21201 .18892 .20073 Metetra
.664 .741 .908 r
.18892 .20073 .19982 .21201 .21553 .20592 .20469 .19457 Metetra
.664 .741 .908 r
.20469 .19457 .21553 .20592 .23135 .19979 .22056 .18838 Metetra
.664 .741 .908 r
.22056 .18838 .23135 .19979 .24725 .19362 .23652 .18214 Metetra
.664 .741 .908 r
.23652 .18214 .24725 .19362 .26326 .18742 .25259 .17587 Metetra
.664 .741 .908 r
.25259 .17587 .26326 .18742 .27936 .18118 .26875 .16955 Metetra
.664 .741 .908 r
.26875 .16955 .27936 .18118 .29557 .17491 .28501 .1632 Metetra
.663 .741 .908 r
.28501 .1632 .29557 .17491 .31186 .16878 .30138 .15681 Metetra
.66 .736 .906 r
.30138 .15681 .31186 .16878 .3282 .16333 .31783 .15053 Metetra
.636 .714 .903 r
.31783 .15053 .3282 .16333 .34458 .15922 .33434 .14511 Metetra
.613 .682 .891 r
.33434 .14511 .34458 .15922 .36104 .15579 .3509 .14054 Metetra
.633 .672 .87 r
.3509 .14054 .36104 .15579 .37768 .15152 .36764 .13503 Metetra
.648 .661 .849 r
.36764 .13503 .37768 .15152 .3945 .14596 .38454 .12861 Metetra
.648 .651 .839 r
.38454 .12861 .3945 .14596 .41147 .13966 .40156 .12199 Metetra
.648 .647 .836 r
.40156 .12199 .41147 .13966 .42856 .13311 .41869 .11531 Metetra
.648 .647 .835 r
.41869 .11531 .42856 .13311 .44576 .12648 .43592 .10859 Metetra
.648 .647 .835 r
.43592 .10859 .44576 .12648 .46306 .1198 .45326 .10183 Metetra
.648 .647 .835 r
.45326 .10183 .46306 .1198 .48048 .11308 .47072 .09503 Metetra
.648 .647 .835 r
.47072 .09503 .48048 .11308 .49801 .10632 .48829 .08818 Metetra
.648 .647 .835 r
.48829 .08818 .49801 .10632 .51565 .09952 .50597 .08129 Metetra
.648 .647 .835 r
.50597 .08129 .51565 .09952 .53341 .09267 .52377 .07435 Metetra
.648 .647 .835 r
.52377 .07435 .53341 .09267 .55128 .08577 .54168 .06737 Metetra
.648 .647 .835 r
.54168 .06737 .55128 .08577 .56927 .07883 .55971 .06034 Metetra
.648 .647 .835 r
.55971 .06034 .56927 .07883 .58738 .07185 .57786 .05327 Metetra
.648 .647 .835 r
.57786 .05327 .58738 .07185 .6056 .06482 .59613 .04614 Metetra
.648 .647 .835 r
.59613 .04614 .6056 .06482 .62394 .05774 .61452 .03898 Metetra
.648 .647 .835 r
.61452 .03898 .62394 .05774 .64241 .05061 .63303 .03176 Metetra
.648 .647 .835 r
.63303 .03176 .64241 .05061 .661 .04344 .65167 .0245 Metetra
.648 .647 .835 r
.65167 .0245 .661 .04344 .67971 .03623 .67042 .01719 Metetra
.648 .647 .835 r
.67042 .01719 .67971 .03623 .69854 .02896 .68931 .00983 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(20)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(40)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(60)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(80)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(100)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.07346 .24092 m
.0763 .24365 L
s
.11029 .2265 m
.1131 .22926 L
s
.14766 .21187 m
.15043 .21466 L
s
.22407 .18195 m
.22677 .18482 L
s
.26314 .16665 m
.2658 .16956 L
s
.3028 .15112 m
.30541 .15406 L
s
.38395 .11935 m
.38647 .12237 L
s
.42547 .10309 m
.42794 .10615 L
s
.46763 .08658 m
.47006 .08968 L
s
.55397 .05277 m
.55629 .05595 L
s
.59817 .03546 m
.60044 .03868 L
s
.64309 .01787 m
.6453 .02113 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{157.875, 128},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000350000X1000`40O003h00OaGoo0035Ool00<Eo
o`00aGoo0035Ool00<Eoo`00aGoo0035Ool00<Eoo`00aGoo001[Ool50003Ool30005Ool30017Ool0
06eoo`03001oogoo009oo`05001oogooOol00003Ool01@00Oomoogoo0000AWoo001]Ool00`00Oomo
o`02Ool01@00Oomoogoo00000goo00D007ooOomoo`0004Ioo`00KGoo00<007ooOol00Woo00D007oo
Oomoo`0000=oo`05001oogooOol00016Ool006eoo`03001oogoo009oo`05001oogooOol00003Ool0
1@00Oomoogoo0000AWoo001]Ool00`00Oomoo`02Ool01@00Oomoogoo00000goo00D007ooOomoo`00
04Ioo`00KGoo00<007ooOol00Woo00D007ooOomoo`0000=oo`05001oogooOol00016Ool006]oo`<0
00Eoo`<000Eoo`<004Moo`00aGoo0035Ool00<Eoo`00E7oo0`001Goo0`007Woo0P00AWoo001COol0
1@00Oomoogoo00000goo00D007ooOomoo`0001Yoo`D004Ioo`00Dgoo00D007ooOomoo`0000=oo`05
001oogooOol0000HOol20002Ool01000DYX00000AGoo001COol01@00Oomoogoo00000goo00D007oo
Oomoo`0001Eoo`<0009oo`80009BVP03001oo`0004Aoo`00E7oo0`0017oo00D007ooOomoo`00019o
o`<0009oo`<00004DYX005:JDYX30014Ool005=oo`05001oogooOol00003Ool01@00Oomoogoo0000
47oo0P000Woo0`0000=BVP00DYX00U:J00H005:J000007oo0013Ool005=oo`05001oogooOol00003
Ool01@00Oomoogoo00003Goo1`0000=BVP00DYX00U:J00<005:J000010000Woo00<007ooOol0@7oo
001DOol30005Ool3000<Ool200001Woo00000000DYX000=BVP03001BVP0000800008DYX005:J001B
VP00Ool0049oo`00J7oo0`000Woo00<005:J00000e:J00<005:J00000P0000=BVP00DYX00U:J0P00
00EBVP00001oo`00045oo`00IGoo10000Woo0P000e:J00<005:J00000P0000=BVP00DYX00U:J00<0
05:JDYX00`0000=BVP00Ool00P001goo0`001Woo0P001Goo1@0097oo001SOol20003Ool3000015:J
001BVU:J0`0000=BVP00DYX00U:J00<005:JDYX00`0000=BVP00DYX00U:J00<007oo00001Woo00D0
07ooOomoo`0000Eoo`8000Moo`03001oogoo02Aoo`00>7oo0`001Goo0`007Goo0`000Woo0`0000=B
VP00DYX00U:J0P0000=BVP00DYX00U:J00<005:JDYX00`0000=BVP00DYX00U:J00@005:J0000009o
o`03001oogoo00=oo`05001oogooOol0000>Ool00`00Oomoo`0TOol003Moo`05001oogooOol00003
Ool01@00Oomoogoo00006Woo0P000Woo0`0000=BVP00DYX00U:J00@005:J000000=BVP03001BVU:J
00<00003DYX005:J009BVP03001BVP0000800004DYX007ooOol20004Ool01@00Oomoogoo00003Woo
00<007ooOol097oo000gOol01@00Oomoogoo00000goo00D007ooOomoo`0001Moo`<00006Ool00000
001BVP000e:J00<005:JDYX00P000U:J00<005:JDYX00`0000=BVP00DYX00U:J00<005:J00000P00
00=BVP00DYX00U:J00@007ooOol000Aoo`05001oogooOol0000>Ool00`00Oomoo`0TOol003Moo`05
001oogooOol00003Ool01@00Oomoogoo00005Goo1P0000=BVP00DYX00U:J00<005:JDYX00`0000AB
VP00DYYBVP<00003DYX005:J009BVP03001BVP0000800004DYX005:JDYX20002DYX20002Ool01@00
Oomoogoo00000goo00<007ooOol037oo00<007ooOol097oo000gOol40004Ool01@00Oomoogoo0000
4Woo0`0000Eoo`00001BVP0000=BVP03001BVU:J00<00003DYX005:J009BVP800003DYX005:J009B
VP03001BVP0000800003DYX005:J009BVP03001BVP0000<00004DYX007ooOol20002Ool01@00Oomo
ogoo00003Woo00<007ooOol097oo000gOol00`00Oomoo`05Ool01@00Oomoogoo00003goo0`0000Io
o`00000005:J0002DYX00`00DYYBVP0300000e:J001BVP02DYX01000DYX000000e:J00<005:J0000
0P0000=BVP00DYX00U:J00<005:JDYX00`0000=BVP00DYX00U:J00@007ooOol000=oo`<000eoo`<0
02Ioo`00>7oo00<007ooOol017oo00D007ooOomoo`0000eoo`H00003DYX005:J009BVP03001BVP00
00800003DYX005:J009BVP03001BVP0000800003DYX005:J009BVP800003DYX005:J009BVP03001B
VU:J00<00003DYX00000009BVP04001BVP000003Ool00`00Oomoo`0iOol003Uoo`<000Aoo`<000]o
o`<0009oo`03001BVP0000=BVP03001BVP0000800003DYX005:J009BVP03001BVP0000800003DYX0
05:J009BVP05001BVP00DYX00002DYX00`00DYYBVP0300000e:J001BVP02DYX00`00DYX000020000
0e:J001oo`02Ool2000jOol004aoo`8000=oo`8000=BVP03001BVP0000800003DYX005:J009BVP03
001BVP0000800003DYX005:J009BVP03001BVU:J008000=BVP03001BVP0000800003DYX005:J009B
VP03001BVP0000800003DYX005:J009BVP05001oogooOol0000jOol004Uoo`<00003Ool000000080
009BVP03001BVU:J00800003DYX005:J009BVP03001BVU:J00800003DYX005:J009BVP03001BVU:J
00<00004DYX005:JDYX300000e:J001BVP02DYX00`00DYX0000200000e:J001BVP02DYX00`00DYYB
VP020003Ool00`00Oomoo`0gOol001moo`<000Aoo`<001eoo`<000=oo`80009BVP03001BVU:J00<0
0005DYX005:JDYX00002DYX300001E:J001BVU:J00000U:J0`0000ABVP00DYYBVP<00003DYX005:J
009BVP03001BVP0000800004DYX005:JDYX20002DYX300000e:J001oo`02Ool2000hOol0021oo`03
001oogoo009oo`05001oogooOol0000JOol30002Ool3000015:k001BVU:J0`0000=BVP00DYX00U:J
00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J0P0000=BVP00DYX0
0U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000`0000=BVP00DYX00U:J00<007ooOol0
0P00>7oo000MOol50003Ool01@00Oomoogoo00005goo20000U:k00<005:kD[/00P0000=BV`00DYX0
0U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00@005:J0000
00=BVP03001BVU:J00800003DYX005:J009BVP03001BVU:J00<00003DYX005:J009BVP04001BVP00
0004Ool00`00Oomoo`0eOol001eoo`04001oogoo0004Ool01@00Oomoogoo00005Goo1@0000EBg000
C[a>_000009B^`@000=BV`03001BVP0000800003DYX005:J009BVP03001BVP0000800003DYX005:J
009BVP03001BVP000080009BVP03001BVU:J00<00005DYX005:JDYX00002DYX300000e:J00000002
DYX00`00DYX0000200000e:J001oo`03Ool2000fOol001ioo`03001oo`0000Aoo`05001oogooOol0
000BOol50002E_`01@00D]aBg000C[`00`0000E>_000D[]B^`00009BV`800003DYX005:J009BVP03
001BVU:J00800003DYX005:J009BVP03001BVU:J0080009BVP03001BVU:J00<00003DYX005:J009B
VP03001BVP0000800003DYX005:J009BVP03001BVP0000800003DYX005:J009BVP04001oogooOol2
0007Ool30006Ool20004Ool5000KOol001ioo`03001oo`0000Aoo`05001oogooOol0000?Ool50002
E_d00`00E_aFo00500000dZl001>_002C[`00`00D[/0000200001E:K001BVU:J00000U:J0`0000EB
VP00DYYBVP00009BVP<00004DYX005:JDYX300000e:J001BVP02DYX00`00DYX0000200000e:J001B
VP02DYX00`00DYX0000200000e:J001BVP02DYX00`00DYYBVP020004Ool00`00Oomoo`03Ool01@00
Oomoogoo00001Goo0P001Goo00@007ooOol001]oo`007goo0P0017oo00D007ooOomoo`0000eoo`D0
0004E_d005KmE_d300001E;m001>g4kL00000TZl00<004jl00000P0000=>^`00DY/00U:K00<005:J
00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J0P0000=BVP00DYX00U:J00<0
05:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:JDYX00`0000=B
VP00Ool00goo0P0017oo00D007ooOomoo`0000eoo`03001oogoo01]oo`0087oo00<007ooOol00goo
0`002goo1@000UKm00<005Km00000P000UKm00<005;m00001`000TZL00D004jkC[/005:K00<00003
DY/005:J009BVP03001BVP0000800003DYX005:J009BVP04001BVP000003DYX00`00DYX000020000
0e:J001BVP02DYX00`00DYX00002000015:J001BVU:J0P000U:J0`0000ABVP00DYYBVP8000=oo`80
00Aoo`05001oogooOol0000>Ool00`00Oomoo`0JOol0039oo`@0009Fo@03001FoEKm0080009Fo@03
001FoEKm00800005C]d004ZmB[d00002B[`00`00BYa:W00500000e:K001BV`02DY/00`00DYX00002
00000e:J001BVP02DYX00`00DYX000020002DYX00`00DYX0000200000e:J001BVP02DYX00`00DYX0
000200000e:J001BVP02DYX00`00DYX00003000015:J001BVU:J0`0000=BVP00Ool017oo00D007oo
Oomoo`0000=oo`03001oogoo00eoo`03001oogoo01Uoo`00;goo1@0000AFo@00E_eFo@<00004E_d0
05KmE_d300001U;m001>gDkM001:_@H00005BY`004jKCY/00002DY/00`00DY]BV`0200000e:K001B
VP02DYX00`00DYX000020002DYX00`00DYYBVP0200000e:J001BVP02DYX00`00DYYBVP0200000e:J
001BVP02DYX00`00DYYBVP0300000e:J001BVP02DYX200000e:J001BVP02DYX00`00Oomoo`02Ool2
0002Ool01@00Oomoogoo00003goo00<007ooOol06Goo0004Ool50004Ool3000MOol40002E_d00`00
E_d000020002E_d00`00E_d000020002E_d00`00D_d000040002A[d01000AYa6W0000TZL00<004jK
00001@0000EBV`00DY]BV`00009BVP80009BVP03001BVU:J00<00005DYX005:JDYX00002DYX30000
1E:J001BVU:J00000U:J0`0000MBVP00DYX005:J001BVP020003DYX01000DYX0000017oo0P000Woo
00D007ooOomoo`0000]oo`05001oogooOol0000KOol000Eoo`04001oogoo0003Ool01@00Oomoogoo
00006Goo1@0000AFo@00E_eFo@80009Fo@03001FoEKm0080009Fo@03001FoEKm0080009Bo@03001>
g@000096_@P00005BY`004jKCY/00003DY/00`00DY/00002000015:J001BVU:J0`0000=BVP00DYX0
0U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J1@0000ABVP00
DYYBVP<00003DYX007oo00Eoo`03001oogoo00<000eoo`<001aoo`001Woo00<007ooOol00goo00D0
07ooOomoo`0001Ioo`D00004E_d00000E_d30002E_d00`00E_d00002000015Km001FoEKm0`0000AB
o@00D_eBo@D00005AYd004:L@Y`00002AY`01@00BYa:W000CY/01P0000ABV`00DYYBVP<00003DYX0
05:J009BVP03001BVP0000800003DYX005:J009BVP03001BVP0000800003DYX005:J009BVP03001B
VP0000800004DYX005:JDYX300000e:J001BVP02DYX00`00Oomoo`03Ool2000`Ool000Moo`03001o
ogoo009oo`05001oogooOol0000DOol40002E_d00`00E_eFo@020002E_d00`00E_eFo@020002E_d0
0`00E_d000020002E_d00`00D_d0000200001DkM001:_DZm00000TJM00D004:L@Y`004JL00D00003
CY/004jK009>V`03001BV`0000800003DY/005:J009BVP03001BVP0000800003DYX005:J009BVP03
001BVP0000800003DYX005:J009BVP03001BVP000080009BVP03001BVP0000800003DYX005:J009B
VP03001BVU:J008000Aoo`80031oo`0027oo00D007ooOomoo`0000=oo`03001oogoo00moo`H00003
E_d005Km00<0009Fo@03001Fo@0000800004E_d005KmE_d20002E_d00`00E_eFo@020002D_d00`00
C]d0000:00001DJL001:W4ZL00000TjK00<004jKCY/00P0000=BV`00DY/00U:K00<005:J00000P00
00=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P000U:J00<005:JDYX0
0P0000=BVP00DYX00U:J00<005:JDYX00`0000=BVP00Ool01Goo00<007ooOol0;Goo0008Ool01@00
Oomoogoo00000goo00<007ooOol03Goo10000UKm00<005KmE_d00P000UKm00<005KmE_d00P000UKm
00<005Km00000P000UKm00<005Km00000P0000AFo@00D_eBo@80009>g@060016_DJm0012W@000T:L
00D004JLAY`004ZL00H00005CY/005:KDY/00002DY/200000e:J001BVP02DYX00`00DYX000020000
0e:J001BVP02DYX00`00DYX000020002DYX00`00DYYBVP050002DYX00`00DYYBVP0300000e:J001B
VP02DYX00`00Oomoo`03Ool2000^Ool000Aoo`05001oogooOol00003Ool01@00Oomoogoo000037oo
1@000UKm00<005Km00000P000UKm00<005Km00000P0000AFo@00E_eFo@80009Fo@03001FoEKm0080
009Fo@03001Fo@000080009Bo@03001>g@0000`00005AY`004ZKBY/00003CY/00`00DY/000020000
1E:K001BVU:J00000U:J0P0000=BVP00DYX00U:J00<005:JDYX00P000U:J00<005:JDYX00`0000MB
VP00DYX005:J001BVP0300000e:J001BVP02DYX01000DYX000001Woo00<007ooOol0;7oo0005Ool3
0005Ool3000:Ool50002E_d00`00E_eFo@020002E_d00`00E_eFo@020002E_d00`00E_d000020000
15Km001FoEKm0`0000=Fo@0000000`000U;m1@000T[M00<004Jm00000SjM00<003im00000T:L00D0
04JLAY`004ZK00H00003DY/005:K009BV`03001BVP0000800005DYX005:JDYX00002DYX3000015:J
001BVU:J0`0000=BVP00DYX00U:J1@0000=BVP00DYX00U:J00<005:J00000P0000=BVP00Ool01Goo
00<007ooOol0;7oo000HOol50002E_d00`00E_d000020002E_d00`00E_d00002000015Km001FoEKm
0P000UKm00<005Km00000P000UKm0P000UKm00<005Km00000P000U;m00<004kM00000T[M3`0000=:
W000CY/00TjK00D005:KDY/005:K00<00003DYX005:J009BVP03001BVP0000800004DYX005:JDYX3
00000e:J001BVP02DYX00`00DYX0000200000e:J001BVP02DYX00`00DYX0000200000e:J001BVP02
DYX00`00Oomoo`04Ool00`00Oomoo`05Ool30006Ool20005Ool3000COol001Eoo`H00004E_d005Km
E_d20002E_d00`00E_eFo@020002E_d00`00E_d0000200000eKm0000000400001UKm00000000E_d0
009Fo@80009Fo@03001BoE;m00D00007B[d004:M@Yd003im00000Sim00H0049l@W`004JL0002BY`0
0`00CY]>V`050002DY/01@00DYYBVP00DYX00`0000ABVP00DYYBVP<00003DYX005:J009BVP03001B
VP0000800003DYX00000009BVP03001BVP0000800003DYX005:J009BVP04001BVP000007Ool00`00
Oomoo`03Ool01@00Oomoogoo00001Goo0P0017oo00D007ooOomoo`00019oo`0057oo10000UKm00<0
05Km00000P000UKm00<005Km00000P0000AFo@00E_eFo@80009Fo@80009Fo@03001FoEKm0080009F
o@03001Fo@0000800004E_d005KmE_d300001U;m001>gDkM001:_A<00005CY/004jKCY/00002DY/5
0002DYX00`00DYX0000200000e:J001BVP02DYX00`00DYX0000200001e:J00000000DYX005:J00<0
0003DYX005:J009BVP03001BVU:J00800003DYX007oo00Ioo`03001oogoo00=oo`05001oogooOol0
000?Ool00`00Oomoo`0@Ool001Aoo`80009Fo@04001Fo@000002E_d00`00E_eFo@020002E_d00`00
E_d00002000015Km000000000UKm00<005Km00000P0000AFo@00E_eFo@80009Fo@03001Fo@000080
009Bo@03001Bo@0000@00096_@03000nW@00008jO@03000jO@00008nO0030012O000009:W006001>
VdjKCY/004jK0`0000EBV`00DY]BV`00009BVP800003DYX005:J009BVP03001BVP0000800007DYX0
0000001BVP00DYX00`0000=BVP00DYX00U:J00<005:JDYX00`0000EBVP00DYYBVP0000Moo`03001o
ogoo009oo`05001oogooOol0000?Ool00`00Oomoo`0@Ool000eoo`<000Aoo`80009Fo@80009Fo@03
001Fo@0000800003E_d0000000@00003E_d00000009Fo@03001FoEKm0080009Fo@03001Fo@000080
0004E_d005KmE_d20002E_d00`00D_eBo@020002D_d00`00C]d0000;0002?W`01000@W`004ZL1P00
00=>V`00DY/00U:K00<005:K00000P0000EBVP00DYYBVP00009BVP800006DYX00000001BVP000U:J
0P0000IBVP00DYX005:J0002DYX3000015:J001BVU:J1@001goo00D007ooOomoo`0000=oo`03001o
ogoo00]oo`8001=oo`0037oo00D007ooOomoo`0000=oo`@00004E_d005KmE_d300000eKm00000002
E_d00`00E_eFo@020002E_d00`00E_d00002000015Km001FoEKm0P000UKm00<005Km00000P000UKm
00<005Km00000P0000ABo@00D_eBo@80009:g@030016_@00008nW@04000jO@00>Wd700000dJL001:
V`02BY/01@00CY]>V`00DY/00`0000=BV`00DYX00U:J00<005:J00001@0000ABVP00DYYBVP<00006
DYX005:JDYX005:J100000ABVP00DYYBVP<00005DYX005:JDYX00007Ool01@00Oomoogoo00000goo
00<007ooOol03Goo00<007ooOol047oo000<Ool01@00Oomoogoo00000Woo00<007ooOol00Woo1000
0UKm0P000UKm00<005Km00000P0000AFo@00E_eFo@80009Fo@03001Fo@0000800004E_d005KmE_d3
000015Km001FoEKm0P000UKm1@0000A>o@00B]e:g@T00007>Wd003il?W`0049l00000TJL00<004ZK
BY/01@0000=>V`00DY/00U:K00<005:J00001@000U:J00<005:J00000P0000=BVP00DYX00U:J00<0
05:J00000P0000ABVP00DYYBVP<00003DYX005:J009BVP04001BVP000006Ool20002Ool01@00Oomo
ogoo00002goo00D007ooOomoo`00019oo`0037oo00D007ooOomoo`00009oo`8000Aoo`03001Fo@00
00800004E_d005KmE_d20002E_d00`00E_d00002000015Km001FoEKm0`0000=Fo@00E_d00`000UKm
1@0000IFo@00000005Km0002D_d5000024[M0012_D:m000jO@00=Wd=00001DZK001>VdjK00000TjK
00<005:K00001@000U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX0
100000=BVP00DYX00U:J00<005:J00000P0000=BVP00Ool01goo00<007ooOol00`003Goo0`004goo
000<Ool01@00Oomoogoo00000Woo0P001Goo00D005KmE_d005Km00<00004E_d005KmE_d20002E_d0
0`00E_d000020002E_d500001UKm00000000E_d0009Fo@80009Fo@03001Fo@0000800006D_d004kM
C]d004[M2@0000DfG@00>W`jO0000092O0030016W00000=:V`03001>V`0000L0009BV`03001BVP00
00800003DYX005:J009BVP03001BVP0000800004DYX005:JDYX50003DYX00`00DYX0000200000e:J
001BVP02DYX00`00Oomoo`05Ool2000WOol000aoo`05001oogooOol00002Ool00`00Oomoo`05Ool0
1000E_d000000UKm00<005Km00000P000UKm00<005Km00000P0000IFo@00000005Km0002E_d20002
E_d00`00E_d00002000015Km001FoEKm0P000U;m00<005;m000010000TJm00<004:M00000SYm00<0
03IM00003P0000=>V`0000000P000U:K1@000U:J00D005:JDYX005:J00<0009BVP03001BVP000080
0007DYX005:J001BVP00DYX00`0000=BVP00DYX00U:J00@005:J000000Qoo`03001oogoo02Eoo`00
37oo00D007ooOomoo`00009oo`8000Moo`05001FoEKm001Fo@0300000eKm000000030002E_d20002
E_d00`00E_d00002000015Km001FoEKm0P000UKm00<005Km00000P0000AFo@00D_eBo@80009Bo@03
001>g@0000X0008fG@03000jG000008nO0030016O00000=:W0@00006CY/00000001BV`000U:K00<0
05:JDYX01@000U:J00<005:JDYX00P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J
00<005:JDYX00P0000=BVP00Ool027oo00<007ooOol097oo000=Ool30003Ool00`00Oomoo`07Ool4
0002E_d20002E_d00`00E_d00002000015Km001FoEKm0P000UKm00<005Km00000P0000AFo@00E_eF
o@80009Fo@03001Fo@0000800004D_d005;mD_d20002B]d00`00A[d00002?Yd01000=Wd003IM1P00
0Sil00<004Il00001`0000=>V`00DY/00U:K00<005:K00000P0000EBV`00DYYBVP00009BVP<00005
DYX005:JDYX00002DYX200000e:J001BVP02DYX00`00DYYBVP0300001E:J001BVU:J000027oo0P00
9Goo000BOol2000:Ool00`00E_d000020002E_d01000E_d000000UKm00<005Km00000P0000AFo@00
E_eFo@80009Fo@03001Fo@0000800003E_d0000000<0009Fo@D00004C_d004[MB]d;000013IM000f
G3IL1`0000E6W000BY]:V`00009>V`03001BV`0000800003DY/005:K009BV`03001BVP0000800003
DYX005:J009BVP03001BVP0000800005DYX005:JDYX00002DYX300000e:J001BVP02DYX01000DYX0
000027oo00<007ooOol08goo000BOol00`00Oomoo`0:Ool01000E_eFo@000UKm0P000UKm00<005Km
00000P0000=Fo@00E_d00`0000=Fo@000000100000=Fo@0000000UKm00<005Km00000P0000AFo@00
D_eBo@D00007B]d004:m@[d003jM00000SIm00<0039M000010000Sil00<0049l00000dJL00<004ZK
0000100000=>V`00DY/00U:K00<005:K00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00
DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00Ool027oo00<007oo
Ool01Goo0`001Woo0P001Woo0`002Goo000BOol2000<Ool00`00E_d00002000015Km001FoEKm0P00
0UKm1@0000=Fo@0000000UKm00<005KmE_d00P000UKm00@005Km0000009Fo@03001Fo@0000800004
D_d004kMC]d>00000c9M000fG00:00001DZK001>VdjK00000TjK00<005:K00000P0000=BV`00DYX0
0U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J0000
0P0000ABVP00DYYBVP8000Qoo`03001oogoo00Aoo`05001oogooOol00005Ool20007Ool00`00Oomo
o`08Ool0019oo`03001oogoo00aoo`800003E_d0000000<00006E_d00000001Fo@000UKm0P000UKm
00<005Km00000P0000AFo@00E_eFo@800004E_d005KmE_d20002D_d00`00D_d0000200001dkM001:
gD[M0012_@00008jO@<00003<Ud0000000<0008jO0060012O49l0016W0000dZK00<004jK00001000
0U:K00D005:KDY/005:J00<00003DYX005:J009BVP03001BVP0000800004DYX005:JDYX6000015:J
001BVU:J0`0000=BVP00Ool027oo00<007ooOol00goo00D007ooOomoo`0000]oo`D000Uoo`000goo
1@000goo0`0017oo00<007ooOol03Woo00D005KmE_d005Km0080009Fo@03001Fo@0000800004E_d0
05KmE_d20002E_d00`00E_d0000200000eKm0000000300000eKm000000030002D_d00`00C]d0000=
00000c9M000fG00=00001DjK001>VdjK00000U:K1@000U:J00<005:J00001@0000ABVP00DYYBVP<0
0005DYX00000DYX00002DYX300000e:J001BVP02DYX00`00Oomoo`06Ool20004Ool01@00Oomoogoo
00002goo00@007ooOol000Yoo`001Goo00<007ooOol00Woo00D007ooOomoo`0000=oo`8000moo`03
001FoEKm0080009Fo@03001Fo@0000800003E_d005Km00<00003E_d0000000<0009Fo@80009Fo@04
001Fo@000002E_d00`00D_d000040002C]d30002?Yd00`00=Wd00002<Ud00`00;Sd0000200001cIL
000jG3YL0012O0000096O004001:VdZK0002CY/00`00CY/0000200001DjK001BVe:K00000U:J0P00
00=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J
00@005:J000000Uoo`03001oogoo009oo`05001oogooOol0000<Ool00`00Ool0000:Ool000Eoo`03
001oogoo009oo`05001oogooOol00003Ool00`00Oomoo`0?Ool2000015Km001FoEKm0P000UKm1@00
00=Fo@0000000UKm00<005Km00000P0000AFo@00E_eFo@80009Fo@04001Fo@000002D_d00`00D_d0
000?00000bhl0000000700000dIl0000000600000djK001>V`02CY/00`00DY/0000200001E:K001B
VU:J00000U:J0P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P00
00=BVP00Ool02Goo0P000Woo00D007ooOomoo`0000aoo`03001oo`0000Yoo`001Goo00<007ooOol0
0Woo00D007ooOomoo`00009oo`8001=oo`@00006E_d00000001Fo@000UKm0P000UKm00@005Km0000
009Fo@03001Fo@0000800004E_d005KmE_d200000eKm001Bo@030002C_d00`00B]d0000800000biM
000^?@0300001S9L000jG3YL000nO0800005AW`004ZKBY/00002CY/01@00CY]>V`00CY/00`0000=B
V`00DY/00U:K00<005:J00000P0000EBVP00DYYBVP00009BVP800006DYX005:J001BVP000U:J0P00
00=BVP00DYX00U:J00<007ooOol01Woo00<007oo00000Woo00D007ooOomoo`0000eoo`8000Yoo`00
1Goo00<007ooOol00Woo00D007ooOomoo`00009oo`8001Aoo`04001Fo@000002E_d00`00E_d00002
000015Km001FoEKm0P0000AFo@00E_eFo@80009Fo@D00003E_d0000000<00003D_d0000000<0009:
g@030012_@00008jW@03000bO@0000@00003;S`0000000@00005?W`004IlAW`00002BY/00`00CY/0
00040002CY/01@00DY]BV`00DY/00`0000=BVP00DYX00U:J00<005:J00000P0000ABVP00DYYBVP80
0006DYX00000001BVP000U:J00<005:JDYX00P002Goo00@007ooOomoo`<000moo`03001oogoo00Qo
o`001Goo00<007ooOol00Woo00D007ooOomoo`00009oo`03001oogoo01Aoo`05001FoEKm001Fo@02
0002E_d00`00E_d0000200000eKm0000000300001UKm00000000E_d0009Fo@<00003E_d005;m0080
009Bo@03001>o@0000X00004;Ud002Xm:Sd20002<S`00`00=U`0000900001DZK001>VdjK00000TjK
1@0000=BV`00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00<005:J00000P0000ABVP00DYYB
VPH00003DYX007oo00Uoo`03001oogoo01eoo`001Goo00<007ooOol00Woo00D007ooOomoo`00009o
o`8001Ioo`@00003E_d0000000<0009Fo@80009Fo@04001Fo@000002E_d00`00E_d00002000015Km
001FoEKm0P0000ABo@00D_eBo@80009>g@05001:g@00@[d00002>Yd00`00<Ud0000400000bhl0000
00030002?W`01P00@Wa2O000AY`000=:V`03001>V`0000800006CY/005:KDY/005:K1P0000ABVP00
DYYBVPH00003DYX005:J00@00005DYX005:JDYX00002DYX20009Ool00`00Oomoo`0MOol000=oo`<0
00Eoo`<000=oo`03001oogoo01Ioo`04001Fo@000002E_d00`00E_d00002000015Km001FoEKm0P00
0UKm10000UKm1@0000=Fo@0000000`0000ABo@00C]e>g@X0008^?@03000Z?00000800003<S`003IL
00d00003CY/004jK009>V`06001BV`00001BV`000e:J00<005:J00000P0000EBVP00DYYBVP00009B
VPD000=BVP03001BVP0000800003DYX007oo00Uoo`8001eoo`004Goo00<007ooOol05goo00D005Km
E_d005Km00<00003E_d005Km00<00003E_d0000000<00006E_d00000001Fo@000UKm0P0000AFo@00
D_eBo@D00003C]d004KM00@00003>Wd0039M00D0008Z?0D00005?W`0049l@W`00002AY`00`00BY/0
0002CY/600001DjK001BVe:K00000U:J0P0000=BVP00DYX00U:J00<005:J00000P0000MBVP00DYX0
05:J001BVP0300000e:J001BVP02DYX00`00Oomoo`07Ool2000MOol0011oo`8001Yoo`@0009Fo@D0
0007E_d00000001Fo@00E_d00P000UKm00<005Km00000P0000=Fo@00E_d00`0000ABo@00D_eBo@@0
0096_@03000nW@0000H000069Q`00000000^?0000SIL300000E>V`00CY]>V`0000=>V`03001BV`00
00800006DY/005:JDYX005:J0`0000=BVP00DYX00U:J00<005:J00000P0000=BVP00DYX00U:J00@0
05:J000000Yoo`03001oogoo00Eoo`<000Ioo`8000Eoo`@0009oo`0047oo00<007ooOol06Woo00<0
05Km00000P0000=Fo@00E_d00`0000AFo@00E_eFo@80009Fo@04001Fo@000002E_d500000eKm0000
000300000e;m001>g@0900000biM000Z?@0300001RXL00000000=U`0008jG0030012O0000096O004
001:VdZK0002CY/00`00CY/0000500000e:K001BV`02DY/01P00DYX00000DYX000=BVP03001BVP00
00800003DYX005:J009BVP03001BVP0000800003DYX007oo00Yoo`03001oogoo00=oo`05001oogoo
Ool00005Ool20009Ool10001Ool0011oo`8001aoo`800003E_d005Km00<00003E_d005Km00<00003
E_d0000000<00006E_d00000001Fo@000UKm0P0000ABo@00D_eBo@@0009:g@030016_@00008nW@03
000fO@0000@000039Q`00000008^?0P00003AW`0000000H00005CY/004jKCY/00002DY/600001E:J
001BVU:J00000U:J0P000U:J00D005:JDYX005:J00<0009BVP04001BVP00000;Ool00`00Oomoo`02
Ool01@00Oomoogoo000047oo0@000Goo00050004Ool30004Ool00`00Oomoo`0MOol30002E_d50000
0eKm00000002E_d01000E_d000000UKm00<005Km00000P0000=Fo@00D_d00`0000=Bo@00C_d03P00
00@V7@00000V70<0008bG004000jG000?W`200001DIl001:VdZK00000TZK00D004jKCY/004jK00<0
0005DY/005:KDY/00002DYX6000015:J001BVU:J1@000U:J00<005:JDYX00P0000=BVP00Ool02Woo
0P000goo00D007ooOomoo`00011oo`40005oo`0000Eoo`00Oomoo`0000=oo`05001oogooOol00003
Ool00`00Oomoo`0JOol400001UKm00000000E_d0009Fo@800004E_d005KmE_d20002E_d01000E_d0
00000UKm10000U;m10000Tkm00<004[M00001@0000@bO@00:SdZ?@8000059Q`00000000Z?0060000
1Cil0016O4Il00000TZK00<004ZK000010000TjK00D005:KDY/005:K00<00005DY/005:JDYX00002
DYX300001E:J001BVU:J00000U:J0`0000EBVP00DYYBVP0000]oo`05001oogooOol00003Ool00`00
Oomoo`0:Ool40002Ool0009oo`03001oogoo00=oo`05001oogooOol00003Ool2000HOol30004Ool2
00000eKm001Fo@0300000eKm0000000300000eKm0000000300001eKm00000000E_d005Km00<00003
D_d005;m00D0009:g@040012_@00>Yd500001BHL000000009Q`00P000S8l00<003YL00002@0000E:
V`00CY]>V`00009>V`D00003DY/005:K009BV`03001BVP0000800003DYX005:J009BVP03001BVP00
00800003DYX005:J009BVP04001BVP000009Ool30002Ool01@00Oomoogoo000037oo00<007ooOol0
0goo0003Ool00`00Oomoo`02Ool01@00Oomoogoo00000Woo00<007ooOol05Goo0`002Goo100000=F
o@0000000UKm00@005Km0000009Fo@04001Fo@000002E_d01000E_d000000UKm00<005;m00000P00
00=Bo@00C]d02`0000<Z?@0000000R8L0P0000<Z70000000100000DnO000@Wa2O0000096V`05001:
VdZK001>V`0300001DjK001>VdjK00000U:K00<005:K00000P0000=BVP00DYX00U:J00<005:J0000
0P0000=BVP00DYX00U:J00<005:J00000P0000=BVP00Ool01Goo0`0000=oo`0000000Woo00D007oo
Oomoo`0000aoo`03001oogoo00=oo`0017oo00D007ooOomoo`0000=oo`03001oogoo008001=oo`<0
00aoo`03001FoEKm00800004E_d005KmE_d2000015Km001FoEKm0P0000=Fo@0000000`0000=Fo@00
00000`000U;m10000TkM00D004KM0012_@00008fO@05000^G@00000R700300001RHL00000000<S`0
008fG0`00003CY/004jK009>V`03001>V`0000@0009BV`05001BVU:J001BVP0300000e:J001BVP02
DYX00`00DYX0000200000e:J001BVP02DYX00`00Oomoo`030005Ool01000Oomoogoo0`003Goo1000
0Woo0004Ool01@00Oomoogoo00000goo00@007ooOol0015oo`<0011oo`800003E_d0000000<00003
E_d0000000<00003E_d00000009Fo@03001Fo@0000800003E_d005Km00<00003D_d005;m00d00005
9Sd00000000Qo0020002:Q`50002?W`00`00@W`00002AY`01@00BY]:V`00CY/01P0000E>V`00DY]B
V`00009BV`D00004DYX005:JDYX6000015:J001BVU:J1`0027oo00<007ooOol05goo00001@00Oomo
ogoo00000goo00D007ooOomoo`00009oo`03001oogoo00aoo`<001Eoo`05001FoEKm001Fo@020002
E_d01000E_d000000UKm00@005Km0000009Fo@@0009Fo@@0009Bo@03001>g@0000L00003;Ud00000
00@000048Q`000000002;S`00`00=U`0000;00001DjK001>VdjK00000djK00<005:K00000P0000IB
V`00DYYBVP00DYX300001U:J001BVU:J001BVP@00005DYX005:JDYX0000:Ool00`00Oomoo`0GOol0
0004Ool000000005Ool30003Ool2000:Ool3000HOol00`00E_eFo@0200000eKm0000000300000eKm
0000000300001eKm00000000E_d005Km00<00003D_d005;m00D00008B]d004JmA[d003jM000fO@D0
00037O`00000008000039Q`0000000<0008nG0060012O49l0016O0000TZK00D004jKCY/004jK00H0
0003DY/005:K009BV`06001BVP00001BVP000e:J00<005:J00000P0000=BVP00DYX00U:J00@005:J
000000Yoo`03001oogoo01Ioo`003goo00<007ooOol01Woo0`0077oo0P0000=Fo@0000000UKm00@0
05Km0000009Fo@04001Fo@000002E_d01000E_d000000UKm00@005;m0000009Bo@03001>o@0000`0
00068Q`000007O`0027l0P000Rhl00<003IL00003P0000E>V`00CY]>V`00009BV`H00005DY/005:J
DYX00002DYX20002DYX200000e:J001BVP0300000e:J001oo`09Ool00`00Oomoo`0FOol000moo`03
001oogoo00=oo`<0025oo`03001Fo@00009Fo@800003E_d0000000<00003E_d0000000<00003E_d0
000000<00003D_d0000000<00004C_d004[MB]d8000012Xm000001gl100000<V70000000100000Lj
G000@Wa2O000AW`00002BY/01000BY]:V`000TjK00<004jK00000P0000EBV`00DY]BV`00009BV`H0
0004DYX005:JDYX5000015:J001BVU:J0P002Goo00<007ooOol05Woo000>Ool20002Ool3000UOol4
00000eKm00000002E_d01000E_d000000UKm00@005Km0000009Fo@04001Fo@000002D_d00`00D_d0
0004000014[M0012_@000SZM00D0039M0000028L00<000037O`0000000800004;S`0038l<S`60000
0dIl001:V`080002CY/01@00DY]BV`00DY/00`0000EBV`00DYYBVP00009BVP<00003DYX005:J0080
0006DYX00000001BVP002Goo00<007ooOol05Woo000>Ool4000YOol01000E_d000000UKm00D005Km
000005Km00D00003E_d0000000<00003E_d0000000<00003D_d0000000/000039Sd00000008Mo080
00037O`0000000D00004>U`003il?W`400001DZK001:VdZK00000TjK00<004jKCY/01@0000=BV`00
DY/00U:K00<005:J00000P0000=BVP00DYX00U:J100000=BVP00DYX00U:J00<007ooOol01Woo00<0
07ooOol05Woo000>Ool2000/Ool00`00E_d0000400000eKm00000002E_d00`00E_d0000200000eKm
001Fo@020002E_d01000D_d000000U;m00<004kM00001`0000<^G@000000100000DIg0000000027l
00<00003<S`0000000<00096O005001:VdZK001:V`0600001TjK001>VdjK001BV`H00003DYX005:J
009BVP03001BVP0000@0009BVP03001BVU:J008000Moo`03001oogoo01Ioo`0047oo00<007ooOol0
:Woo0P0000=Fo@00E_d00P000UKm00@005Km0000009Fo@@00003E_d0000000<00003D_d0000000<0
0004C]d004KM0002@[d00`00=Wd00004000011gl000000000Qgl0P000RXL0P0000@jG000?W`nO0L0
0005BY/004jKCY/00003CY/01P00CY/00000DY/0009BV`03001BVU:J00H00004DYX005:JDYX20000
1U:J00000000DYX000Moo`03001oogoo01Ioo`004Goo00<007ooOol0:goo0`0000=Fo@0000000`00
00MFo@00000005Km001Fo@020002E_d01000E_d000000U;m00<005;m00002`000R8M0`0000DIg000
0000027l00<00006<S`00000000nO0000T9l00@004JKAY/0009:V`03001>V`0000D00006CY/005:K
DY/005:K0`0000EBV`00DYYBVP00009BVPH00004DYX005:JDYX20007Ool00`00Oomoo`0EOol0019o
o`03001oogoo02Yoo`03001Fo@0000800003E_d005Km0080009Fo@@00003E_d0000000<00003D_d0
000000<00003D_d004kM00P00005:Ud00000000Io00400001AWL00000000:Q`00P000SIL300000E>
V`00CY]>V`00009>V`D00003DY/005:K009BV`03001BVP0000800005DYX005:JDYX00002DYX30000
0e:J001oo`06Ool00`00Oomoo`0EOol001=oo`8002]oo`03001FoEKm00@00007E_d00000001Fo@00
E_d00`0000=Fo@00E_d00P000U;m00<005;m00000`000T[M00H004Jm000nW@00<Wd600000aGL0000
000200028O`2000012hl000000000Sil00<0049l00000TIl00D004ZKBY/004jK00H00005CY/005:K
DY/00002DY/00`00DY/0000200000e:J001BVP02DYX00`00DYX0000200000e:J001BVP02DYX00`00
Oomoo`04Ool00`00Oomoo`0EOol001Eoo`03001oogoo02Uoo`800003E_d005Km00<00003E_d00000
0080009Fo@@00003E_d0000000<00003D_d0000000d000037Q`0000000<000056M`00000000V7003
00000cIL0000000:00001DZK001>VdjK00000djK00<005:K000010000U:K00<005:J00001@0000=B
VP0000000U:J00@005:J000000Ioo`03001oogoo01Eoo`005Woo00<007ooOol0:Woo0`000UKm0P00
00=Fo@00E_d00`0000=Fo@00E_d00P000UKm00H005;m000005;m0002C_d:00001RHm000001WL000E
g0<0008Mo080008^?080008nG0060012O49l0016O0000TZK00D004ZKBY/004jK00H00005CY/005:K
DY/00002DY/200001E:J001BVU:J00000U:J1P0000=BVP00Ool01Goo00<007ooOol05Goo000GOol0
0`00Oomoo`0YOol00`00E_d0000200000eKm000000020002E_d400000eKm0000000300000e;m0000
000300000dkm001:g@060000139m000001gl100000<Ef`0000000P0000DV7000000003IL00l00005
CY/004jKCY/00002CY/600001U:K001BVU:J001BVP<00005DYX00000DYX00002DYX20005Ool00`00
Oomoo`0EOol001Qoo`8002Yoo`800006E_d00000E_d005Km0`0000=Fo@00E_d00P000UKm00H005Km
000005;m0002D_d400001T[M0012_D:m000jW@H000075M`00000000Ef`007O`00`0000H^?0000000
03YL0002@W`01P00AWa6O000BY/0009:V`05001>VdjK001>V`0300001DjK001BVe:K00000e:K00@0
05:J0000009BVP03001BVU:J00D00003DYX007oo00Aoo`03001oogoo01Eoo`006Woo00<007ooOol0
:Goo00<005Km0000100000=Fo@0000000P0000=Fo@0000000`0000=Fo@0000000`0000=Bo@000000
2P0000<R?@0000000`0000<A^`0000000P000RHL0P000SIL4@000TjK00D004jKCY/005:K00H00004
DY/005:JDYX70003DYX00`00Oomoo`03Ool00`00Oomoo`0DOol001]oo`03001oogoo02Uoo`03001F
o@00009Fo@800003E_d005Km00<00003E_d0000000800003E_d0000000<00003D_d004kM00P00005
;Ud00000000Io00600001AgL00000000;Q`00`0000DjG000@Wa2O0000096O006001:VdZK001:V`00
0TjK00<004jKCY/01@0000EBV`00DY]BV`00009BV`<00004DYX005:JDYX70005Ool00`00Oomoo`0D
Ool001aoo`03001oogoo02Uoo`@00003E_d000000080009Fo@800003E_d005Km0080009Bo@03001B
o@0000<0009>g@060016g@00?[d003Im1`0000<A^`000000100000<Qo00000000S8l4P0000I>V`00
CY]>V`00DY/600000e:K001BVP02DYX00`00DYX000030002DYX00`00Oomoo`03Ool00`00Oomoo`0D
Ool001eoo`8002Yoo`05001Fo@00001Fo@0500000eKm0000000200000eKm0000000300000e;m0000
000=00000ahM0000000400000aFk00000002000012XL000000000SYL00<003il00000T9l00H004Ik
AW/004ZK0002CY/01P00CY]>VdjK001>V`800005DY/005:KDY/00002DY/600000e:J001BVP020000
0e:J00000004Ool00`00Oomoo`0DOol001moo`03001oogoo02Qoo`06001Fo@00001Fo@000UKm0P00
00=Fo@00E_d00P000UKm00H005;m000005;m0002C_d:000012Xm000001GL1P0000<Ig0008O`00`00
00<b?00000001`0000=6N`0000002@0000I>V`00DY]BV`00DY/300001E:K001BVe:K00000U:J1@00
0U:J00D007ooOomoo`0001Ioo`0087oo00<007ooOol0:7oo00<005Km0000100000=Fo@0000000P00
00=Fo@0000000`0000=Bo@0000000`002Woo1@0000@=^`00000=^`D0008Z7080008jG005000nO3il
0012O00200001dIk001:VdZK001>V`00009>V`03001>VdjK00D00003DY/005:K009BV`03001BV`00
00800003DYX005:J0080009BVP80009oo`03001oogoo01Aoo`008Goo00<007ooOol0:7oo00D005Km
E_d005Km00800003E_d0000000<00003E_d00000008001=oo`H000034K/001WK00@00003<S`00000
00H000=6N`03001:V`0000L00006CY/004jKCY/005:K1P0000=BV`00DYX00U:J00<005:J00000P00
00EBVP00Oomoo`0001Ioo`008Woo0P00:Goo100000EFo@00001Fo@00009Fo@8001Uoo`03000=^`00
00D000037O`002XL00D0008nO0030012O00000@00007BY/004ZKBY/004jK00000djK00H004jK0000
05:K0002DY/00`00DY]BV`0600000e:J001BVP02DYX00`00Ool0000FOol002Aoo`03001oogoo02Qo
o`<00003E_d0000000<001aoo`<000043I/000005M/40002<S`00`00=U`0000500000dIl001:V`02
BY/00`00BY/0000700001TjK001BVe:K001BV`<00005DY/005:JDYX00002DYX50002Ool00`00Oomo
o`0COol002Eoo`03001oogoo02Moo`04001Fo@00000SOol600027O`60002?U`00`00@Wa2O0070000
14ZK001>V`000TjK00<004jKCY/01@000U:K00D005:KDY/005:J00<00005DYX005:JDYX00002Ool0
0`00Oomoo`0COol002Ioo`03001oogoo02Moo`03001oogoo02Aoo`030009V`VK00D000059Q`00000
000fG00600001DIl001:VdZK00000TZK200000E>V`00DY]BV`00009BV`D00003DY/005:J009BVP06
001BVP00001oo`005Goo000WOol2000WOol00`00Oomoo`0UOol400000aFk000Mg00300001Rhl0000
0000?U`00092O0L00007BY/004jKCY/004jK00000TjK1P0000ABV`00DY/0009BV`H00004DYX007oo
000EOol002Uoo`03001oogoo02Aoo`03001oogoo02Qoo`D0008V7080008fG0D00096O003001:V`00
00=:V`03001>V`0000@00005CY/004jKCY/00003DY/4000015:K001BVU:J0P0000ABVP00000001Eo
o`00:Woo00<007ooOol08goo00<007ooOol0:Goo00<0016k4K/0100000H^?000000003YL0002@W`:
00001DjK001>VdjK00000TjK00<004jK000010000U:K00D005:KDY/005:J00<00003DYX0000001Eo
o`00:goo00<007ooOol08Woo00<007ooOol0:Woo10000RGl0P000SHl1@000TIl00<004Ik00000TZK
00<004jKCY/0200000E>V`00DY]BV`00009BV`D00003DYX005:J00<001Eoo`00;7oo0P008Woo00<0
07ooOol0;Goo100000@^70000000008jG003000nO00000T00007BY/004jKCY/004jK00000TjK1P00
00ABV`00DY/0009BVP<00003DYX0000001Aoo`00;Woo00<007ooOol07goo00<007ooOol0;goo00<0
027l00000P0000<b?00000000`000T9l00H004IkAW/004ZK0003BY/00`00CY/0000400001DjK001B
Ve:K00000U:K1@0000IBV`00DYX005:J000DOol002moo`03001oogoo01ioo`03001oogoo031oo`80
008Z7080008jG003000nO00000`00006CY/004jKCY/004jK1P0000IBV`00DY]BV`00DY/5000DOol0
031oo`03001oogoo01eoo`03001oogoo039oo`<00003<S`0000000<00092O0060016NdIk001:V`00
0TZK00<004jKCY/01@0000E>V`00DY]BV`0000=BV`04001BV`000002Ool3000DOol0035oo`8001eo
o`03001oogoo03Eoo`80008jG003000nO00000`00005CY/004jKCY/00002CY/00`00DY/00009000G
Ool003=oo`03001oogoo01Yoo`03001oogoo03Moo`H00005@W`004IlAW`00002BY/00`00BY/00002
CY/80002Ool4000KOol003Aoo`03001oogoo01Uoo`03001oogoo03eoo``000Ioo`@001moo`00=Goo
00<007ooOol067oo00<007ooOol0C7oo0`008goo000fOol2000HOol00`00Oomoo`18Ool4000VOol0
03Qoo`03001oogoo01Eoo`03001oogoo04Aoo`@002Yoo`00>Goo00<007ooOol057oo00<007ooOol0
@7oo1000;Woo000jOol00`00Oomoo`0COol00`00Oomoo`0lOol4000bOol003]oo`80019oo`03001o
ogoo03Uoo`@003Ioo`00?Goo00<007ooOol03goo00<007ooOol0=Goo1000>Woo000nOol00`00Oomo
o`0>Ool00`00Oomoo`0aOol4000nOol003moo`03001oogoo00eoo`03001oogoo02eoo`@0049oo`00
@7oo0P003Goo00<007ooOol0:Woo0`00AWoo0012Ool00`00Oomoo`0:Ool00`00Oomoo`0VOol40019
Ool004=oo`03001oogoo00Uoo`03001oogoo029oo`@004eoo`00A7oo00<007ooOol027oo00<007oo
Ool07Woo1000DGoo0015Ool20008Ool00`00Oomoo`0JOol4001EOol004Moo`03001oogoo00Eoo`03
001oogoo01Ioo`@005Uoo`00B7oo00<007ooOol017oo00<007ooOol04Woo1000GGoo0019Ool00`00
Oomoo`03Ool00`00Oomoo`0>Ool4001QOol004Yoo`8000=oo`03001oogoo00]oo`<006Eoo`00C7oo
00@007ooOol000Uoo`@006Qoo`00CGoo00<007oo00001Goo1000K7oo001>Ool200000goo00000002
001`Ool004moo`8007Aoo`00aGoo0035Ool00<Eoo`00aGoo0035Ool00<Eoo`00aGoo0000\
\>"],
  ImageRangeCache->{{{0, 196.312}, {159, 0}} -> {-0.0880174, -0.0553531, 
  0.00735979, 0.00735979}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.9 Inverse Calculations - Implied Volatility", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "This is a classic application of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s built-in function ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", that implements various algorithms for the solution of non-linear \
equations. Its default operation is to use Newton-Raphson iteration."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?FindRoot\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("FindRoot[lhs==rhs, {x, x0}] searches for a numerical solution to the \
equation lhs==rhs, starting with x=x0."\)], "Print"]
}, Open  ]],

Cell["\<\
BlackScholesCallImpVol[s_,k_,r_,q_,t_,optionprice_] :=sd /. \
FindRoot[BlackScholesCall[s,k, sd, r, q, t]== optionprice,{sd,0.2}];

BlackScholesPutImpVol[s_,k_,r_,q_,t_,optionprice_] :=sd /. \
FindRoot[BlackScholesPut[s,k, sd, r, q, t]== optionprice,{sd,0.2}];\
\>", 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "\n",
  StyleBox["BlackScholesCallImpVol[s,k,r,q,t,optionprice]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " gives the volatility of a vanilla Call option that is implied by its \
price."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["BlackScholesCallImpVol[58.5, 60., 0.04, 0, 0.3, 3.34886]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.289999695313169105`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "When we compute the implied volatility given the price, we are solving a \
non-linear equation. The function ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is very useful generally, especially in finance, for inverting non-linear \
relations. To reinforce this point, let us look at another familiar relation \
- that between price and return in a dividend discount model. (It could just \
as easily be the relationship between value and yield for a bond.)\n\nWe are \
interested in the relationship between price and rate of return based on a \
dividend stream. We shall make up a price, some forecast dividends, and an \
associated dividend stream based on future extrapolation and a growth rate \
that gradually diminishes from 10% to 5%."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Price = 100;
Array[div,50];

div[1] = 12;
div[2] = 10;
div[3] = 15;
Do[div[k] = div[k-1]*(1 + 0.1 + (0.05-0.1)*(k-4)/10),{k, 4, 14}];
Do[div[k] = div[14]*1.05*(k - 14), {k, 15, 50}];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Now we make up a function that compares a price with a discounted \
cash flow based on the dividends:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["f[R_,price_] := price - Sum[div[k]/(1 + R)^k, {k, 1, 50}]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
Here it is (the text of the output has been reduced in size \
slightly):\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["f[r,p] ", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(p - 1254.75613708680922`\/\((1 + r)\)\^50 - 
      1219.90179994550884`\/\((1 + r)\)\^49 - 
      1185.04746280420869`\/\((1 + r)\)\^48 - 
      1150.19312566290832`\/\((1 + r)\)\^47 - 
      1115.33878852160817`\/\((1 + r)\)\^46 - 
      1080.48445138030801`\/\((1 + r)\)\^45 - 
      1045.63011423900786`\/\((1 + r)\)\^44 - 
      1010.77577709770727`\/\((1 + r)\)\^43 - 
      975.921439956407255`\/\((1 + r)\)\^42 - 
      941.067102815106792`\/\((1 + r)\)\^41 - 
      906.212765673806686`\/\((1 + r)\)\^40 - 
      871.358428532506401`\/\((1 + r)\)\^39 - 
      836.504091391206294`\/\((1 + r)\)\^38 - 
      801.649754249905832`\/\((1 + r)\)\^37 - 
      766.795417108605636`\/\((1 + r)\)\^36 - 
      731.941079967305352`\/\((1 + r)\)\^35 - 
      697.086742826005156`\/\((1 + r)\)\^34 - 
      662.232405684704872`\/\((1 + r)\)\^33 - 
      627.378068543404587`\/\((1 + r)\)\^32 - 
      592.523731402104303`\/\((1 + r)\)\^31 - 
      557.669394260804107`\/\((1 + r)\)\^30 - 
      522.815057119503912`\/\((1 + r)\)\^29 - 
      487.960719978203627`\/\((1 + r)\)\^28 - 
      453.106382836903343`\/\((1 + r)\)\^27 - 
      418.252045695603147`\/\((1 + r)\)\^26 - 
      383.397708554302809`\/\((1 + r)\)\^25 - 
      348.543371413002578`\/\((1 + r)\)\^24 - 
      313.689034271702293`\/\((1 + r)\)\^23 - 
      278.834697130402053`\/\((1 + r)\)\^22 - 
      243.980359989101813`\/\((1 + r)\)\^21 - 
      209.126022847801573`\/\((1 + r)\)\^20 - 
      174.271685706501289`\/\((1 + r)\)\^19 - 
      139.417348565201027`\/\((1 + r)\)\^18 - 
      104.563011423900786`\/\((1 + r)\)\^17 - 
      69.7086742826005156`\/\((1 + r)\)\^16 - 
      34.8543371413002578`\/\((1 + r)\)\^15 - 
      33.1946068012383355`\/\((1 + r)\)\^14 - 
      31.613911239274608`\/\((1 + r)\)\^13 - 
      29.9657926438621835`\/\((1 + r)\)\^12 - 
      28.269615701756777`\/\((1 + r)\)\^11 - 
      26.5442400955462654`\/\((1 + r)\)\^10 - 
      24.8077010238750128`\/\((1 + r)\)\^9 - 
      23.0769311850000047`\/\((1 + r)\)\^8 - 
      21.3675288750000058`\/\((1 + r)\)\^7 - 
      19.6935750000000027`\/\((1 + r)\)\^6 - 
      18.0675000000000025`\/\((1 + r)\)\^5 - 
      16.4999999999999991`\/\((1 + r)\)\^4 - 15\/\((1 + r)\)\^3 - 
      10\/\((1 + r)\)\^2 - 12\/\(1 + r\)\)], "Output",
  FontSize->8]
}, Open  ]],

Cell["So what internal rate of return is implied by the price?", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["100*R /. FindRoot[f[R, Price] == 0, {R,0}]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(24.3572153115031797`\)], "Output"]
}, Open  ]],

Cell["Just as a check:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["P - f[0.243572, P]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(100.000142625351617`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["What Is FindRoot Doing?", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Sometimes the processes going on inside ",
  StyleBox["FindRoot",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " may be a little obscure, especially when more complicated functions are \
being used. It is sometimes helpful to elaborate on the operation of the \
Newton-Raphson algorithm. The following routine allows a picture of the \
process to be drawn."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
NewtonRaphson[func_, x_, start_, iter_] :=
Module[{pts, xold = start, xnew, f, df, rangea, rangeb},
pts = {}; f = func[x]; df = D[f, x]; 
Do[AppendTo[pts, {xold, 0}];
AppendTo[pts, {xold, (f /. x -> xold)}];
xnew = xold - (f /. x -> xold)/(df /. x -> xold);
xold = xnew,{k, 1, iter}]; Print[xnew];
Plot[f, {x, 0.0001, 0.5}, PlotRange -> All,Axes -> True,
PlotStyle -> {{Thickness[0.001], Dashing[{0.02, 0.02}]}},
Epilog -> {Thickness[0.001], Line[pts]}]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
ShowCallImpliedVolatilityCalc[price_, strike_, rate_, yld_, time_, \
optprice_] := NewtonRaphson[(optprice - BlackScholesCall[price, \
strike,#,rate,yld,time])& , x, 0.1, 10]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ShowPutImpliedVolatilityCalc[price_, strike_, rate_, yld_, time_, \
optprice_] := NewtonRaphson[(optprice - BlackScholesPut[price, \
strike,#,rate,yld,time])& , x, 0.1, 10]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["BlackScholesCallImpVol[56.5, 60., 0.04, 0, 0.3, 3.34886]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.36492757965313709`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ShowCallImpliedVolatilityCalc[56.5, 60., 0.04, 0, 0.3, \
3.34886];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.364927579652485167`\)], "Print"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.023619 1.90514 0.209838 0.117497 [
[.21413 .19734 -9 -9 ]
[.21413 .19734 9 0 ]
[.40465 .19734 -9 -9 ]
[.40465 .19734 9 0 ]
[.59516 .19734 -9 -9 ]
[.59516 .19734 9 0 ]
[.78568 .19734 -9 -9 ]
[.78568 .19734 9 0 ]
[.97619 .19734 -9 -9 ]
[.97619 .19734 9 0 ]
[.01112 .09234 -12 -4.5 ]
[.01112 .09234 0 4.5 ]
[.01112 .32734 -6 -4.5 ]
[.01112 .32734 0 4.5 ]
[.01112 .44483 -6 -4.5 ]
[.01112 .44483 0 4.5 ]
[.01112 .56233 -6 -4.5 ]
[.01112 .56233 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21413 .20984 m
.21413 .21609 L
s
[(0.1)] .21413 .19734 0 1 Mshowa
.40465 .20984 m
.40465 .21609 L
s
[(0.2)] .40465 .19734 0 1 Mshowa
.59516 .20984 m
.59516 .21609 L
s
[(0.3)] .59516 .19734 0 1 Mshowa
.78568 .20984 m
.78568 .21609 L
s
[(0.4)] .78568 .19734 0 1 Mshowa
.97619 .20984 m
.97619 .21609 L
s
[(0.5)] .97619 .19734 0 1 Mshowa
.125 Mabswid
.06172 .20984 m
.06172 .21359 L
s
.09982 .20984 m
.09982 .21359 L
s
.13793 .20984 m
.13793 .21359 L
s
.17603 .20984 m
.17603 .21359 L
s
.25224 .20984 m
.25224 .21359 L
s
.29034 .20984 m
.29034 .21359 L
s
.32844 .20984 m
.32844 .21359 L
s
.36654 .20984 m
.36654 .21359 L
s
.44275 .20984 m
.44275 .21359 L
s
.48085 .20984 m
.48085 .21359 L
s
.51896 .20984 m
.51896 .21359 L
s
.55706 .20984 m
.55706 .21359 L
s
.63326 .20984 m
.63326 .21359 L
s
.67137 .20984 m
.67137 .21359 L
s
.70947 .20984 m
.70947 .21359 L
s
.74757 .20984 m
.74757 .21359 L
s
.82378 .20984 m
.82378 .21359 L
s
.86188 .20984 m
.86188 .21359 L
s
.89998 .20984 m
.89998 .21359 L
s
.93809 .20984 m
.93809 .21359 L
s
.25 Mabswid
0 .20984 m
1 .20984 L
s
.02362 .09234 m
.02987 .09234 L
s
[(-1)] .01112 .09234 1 0 Mshowa
.02362 .32734 m
.02987 .32734 L
s
[(1)] .01112 .32734 1 0 Mshowa
.02362 .44483 m
.02987 .44483 L
s
[(2)] .01112 .44483 1 0 Mshowa
.02362 .56233 m
.02987 .56233 L
s
[(3)] .01112 .56233 1 0 Mshowa
.125 Mabswid
.02362 .11584 m
.02737 .11584 L
s
.02362 .13934 m
.02737 .13934 L
s
.02362 .16284 m
.02737 .16284 L
s
.02362 .18634 m
.02737 .18634 L
s
.02362 .23334 m
.02737 .23334 L
s
.02362 .25684 m
.02737 .25684 L
s
.02362 .28034 m
.02737 .28034 L
s
.02362 .30384 m
.02737 .30384 L
s
.02362 .35083 m
.02737 .35083 L
s
.02362 .37433 m
.02737 .37433 L
s
.02362 .39783 m
.02737 .39783 L
s
.02362 .42133 m
.02737 .42133 L
s
.02362 .46833 m
.02737 .46833 L
s
.02362 .49183 m
.02737 .49183 L
s
.02362 .51533 m
.02737 .51533 L
s
.02362 .53883 m
.02737 .53883 L
s
.02362 .06884 m
.02737 .06884 L
s
.02362 .04534 m
.02737 .04534 L
s
.02362 .02184 m
.02737 .02184 L
s
.02362 .58583 m
.02737 .58583 L
s
.02362 .60933 m
.02737 .60933 L
s
.25 Mabswid
.02362 0 m
.02362 .61803 L
s
.001 w
[ .02 .02 ] 0 setdash
.02381 .60332 m
.03279 .60332 L
.03793 .60332 L
.04018 .60332 L
.04136 .60332 L
.04262 .60332 L
.04325 .60332 L
.04394 .60332 L
.04517 .60332 L
.04641 .60332 L
.04758 .60332 L
.04826 .60332 L
.04891 .60332 L
.05013 .60332 L
.05145 .60332 L
.05219 .60332 L
.05288 .60332 L
.05413 .60332 L
.05531 .60332 L
.05637 .60332 L
.05751 .60332 L
.05876 .60332 L
.06008 .60332 L
.06121 .60332 L
.06244 .60332 L
.06315 .60332 L
.0638 .60332 L
.06451 .60332 L
.06527 .60332 L
.06654 .60332 L
.06727 .60332 L
.06795 .60332 L
.06921 .60332 L
.07036 .60332 L
.07163 .60331 L
.07296 .60331 L
.07431 .60331 L
.07575 .6033 L
.07711 .60329 L
.07835 .60329 L
.0798 .60327 L
.08138 .60326 L
.08424 .60322 L
.08668 .60317 L
.08938 .60309 L
.09219 .603 L
.09482 .60288 L
.09953 .6026 L
.10458 .6022 L
.10984 .60164 L
Mistroke
.11471 .601 L
.12575 .59908 L
.13549 .59685 L
.14586 .59394 L
.18749 .57768 L
.22761 .55708 L
.26621 .53439 L
.30727 .50828 L
.34681 .48183 L
.38483 .45555 L
.42531 .42693 L
.46427 .39891 L
.50568 .36873 L
.54558 .33937 L
.58396 .31092 L
.6248 .28048 L
.66411 .25102 L
.70192 .2226 L
.74217 .19225 L
.78091 .16297 L
.8221 .13177 L
.86178 .10167 L
.89994 .07269 L
.94055 .04182 L
.97619 .01472 L
Mfstroke
[ ] 0 setdash
.21413 .20984 m
.21413 .56441 L
.88293 .20984 L
.88293 .08562 L
.71937 .20984 L
.71937 .20945 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
.71886 .20984 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  ImageSize->{196.188, 120.812},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000340000N1000`40O003h00Oa7oo0034Ool00<Ao
o`00a7oo000>Ool00`00Oomoo`2cOol000ioo`03001oogoo0;=oo`003Woo00<007ooOol0Z7oo0P00
2Goo000>Ool00`00Oomoo`2WOol00`00Oomoo`09Ool000ioo`03001oogoo0:Ioo`03001oogoo00Yo
o`003Woo00<007ooOol0/goo000>Ool00`00Oomoo`2cOol000ioo`03001oogoo0:9oo`8000moo`00
3Woo00<007ooOol0XGoo00<007ooOol03goo000>Ool00`00Oomoo`2POol00`00Oomoo`0@Ool000io
o`03001oogoo0;=oo`003Woo00<007ooOol0WGoo00<007ooOol04goo000>Ool00`00Oomoo`2LOol0
0`00Oomoo`0DOol000ioo`03001oogoo09]oo`03001oogoo01Eoo`001goo0`0017oo00<007ooOol0
/goo0008Ool00`00Oomoo`03Ool00`00Oomoo`2GOol2000JOol00003Ool0000000<0009oo`03001o
ogoo00=oo`8009Moo`<001Yoo`0027oo00<007ooOol00goo00<007ooOol0UGoo0P0000=oo`00Ool0
6Goo0008Ool00`00Oomoo`03Ool00`00Oomoo`2COol30002Ool00`00Oomoo`0HOol000Moo`8000Eo
o`03001oogoo099oo`03001oogoo00=oo`03001oogoo01Qoo`003Woo00<007ooOol0TGoo0P001Goo
00<007ooOol067oo000>Ool00`00Oomoo`2@Ool00`00Oomoo`05Ool00`00Oomoo`0HOol000ioo`03
001oogoo08ioo`8000Qoo`03001oogoo01Qoo`003Woo00<007ooOol0SGoo00<007ooOol027oo00<0
07ooOol067oo000>Ool00`00Oomoo`2;Ool2000;Ool00`00Oomoo`0HOol000ioo`03001oogoo08Yo
o`8000aoo`03001oogoo01Qoo`003Woo00<007ooOol0RGoo00<007ooOol037oo00<007ooOol067oo
000>Ool00`00Oomoo`28Ool00`00Oomoo`0=Ool00`00Oomoo`0HOol000ioo`03001oogoo01Qoo`80
00Aoo`8000=oo`<001Eoo`8000Aoo`8000=oo`@001=oo`8000Aoo`8000=oo`<001Aoo`8000Aoo`<0
00=oo`<000Yoo`03001oogoo00Moo`8000Aoo`8000=oo`<000=oo`003Woo00<007ooOol05goo00@0
07ooOol000=oo`8000Aoo`03001oogoo01=oo`04001oogoo0003Ool20004Ool00`00Oomoo`0BOol0
1000Oomoo`000goo0P001Woo00<007ooOol047oo00@007ooOol000=oo`8000Eoo`03001oogoo00Uo
o`03001oogoo00Ioo`04001oogoo0003Ool20006Ool10001Ool1Ool000ioo`03001oogoo01Moo`04
001oogoo0009Ool00`00Oomoo`0COol01000Oomoo`002Woo00<007ooOol04Goo00@007ooOol000]o
o`03001oogoo011oo`04001oogoo0002Ool20004Ool4000:Ool00`00Oomoo`06Ool01000Oomoo`00
2goo0@000Goo0Goo000>Ool00`00Oomoo`0GOol01000Oomoo`002Goo00<007ooOol04goo00@007oo
Ool000]oo`03001oogoo011oo`04001oogoo0009Ool2000COol00`00Oomoo`040005Ool00`00Ool0
000;Ool00`00Oomoo`06Ool01000Oomoo`0027oo0`000goo000>Ool00`00Oomoo`0GOol01000Oomo
o`002Goo00<007ooOol04goo00@007ooOol000Qoo`04001oogoo000BOol01000Oomoo`002goo00<0
07ooOol047oo00@007ooOol000Qoo`03001oo`0000]oo`03001oogoo00Ioo`04001oogoo0008Ool0
0`00Oomoo`03Ool000ioo`03001oogoo01Qoo`8000Uoo`8001Ioo`8000Yoo`8001Aoo`8000Uoo`<0
01Aoo`<000Uoo`8000]oo`03001oogoo00Moo`8000Uoo`@0009oo`003Woo00<007ooOol0Ogoo00<0
07ooOol05Woo00<007ooOol067oo000>Ool00`00Oomoo`1mOol2000IOol00`00Oomoo`0HOol000io
o`03001oogoo07aoo`03001oogoo01Uoo`03001oogoo01Qoo`003Woo00<007ooOol0NWoo0P0077oo
00<007ooOol067oo0009Oonf0005Ool000ioo`03001oogoo01moo`03001oogoo021oo`03001oogoo
01moo`03001oogoo015oo`03001oogoo00]oo`03001oogoo00aoo`80019oo`03001oogoo00Moo`00
3Woo00<007ooOol07goo00<007ooOol0LGoo0P007Woo000>Ool00`00Oomoo`0OOol00`00Oomoo`1C
Ool00`00Oomoo`0IOol2000POol000ioo`03001oogoo01moo`03001oogoo059oo`03001oogoo01Qo
o`80029oo`003Woo00<007ooOol07goo00<007ooOol0DGoo00<007ooOol067oo00<007ooOol08Woo
000>Ool00`00Oomoo`0OOol00`00Oomoo`1@Ool00`00Oomoo`0GOol2000UOol000ioo`03001oogoo
01moo`03001oogoo06Qoo`8002Moo`003Woo00<007ooOol07goo00<007ooOol0CGoo00<007ooOol0
5Woo0P00:Goo000>Ool00`00Oomoo`0OOol00`00Oomoo`1<Ool00`00Oomoo`0EOol2000[Ool000io
o`03001oogoo01moo`03001oogoo04]oo`03001oogoo01Aoo`8002eoo`003Woo00<007ooOol07goo
00<007ooOol0H7oo0P00;goo000>Ool00`00Oomoo`0OOol00`00Oomoo`17Ool2000EOol2000aOol0
00ioo`03001oogoo01moo`03001oogoo04Ioo`03001oogoo01=oo`8003=oo`003Woo00<007ooOol0
7goo00<007ooOol0AGoo00<007ooOol04goo00<007ooOol0<goo000>Ool00`00Oomoo`0OOol00`00
Oomoo`1IOol2000fOol000ioo`03001oogoo01moo`03001oogoo049oo`03001oogoo019oo`8003Qo
o`003Woo00<007ooOol07goo00<007ooOol0@Goo00<007ooOol04Goo0P00>Woo000>Ool00`00Oomo
o`0OOol00`00Oomoo`10Ool00`00Oomoo`0@Ool2000lOol000Moo`<000Aoo`03001oogoo01moo`03
001oogoo03moo`03001oogoo00moo`8003ioo`0027oo00<007ooOol00goo00<007ooOol07goo00<0
07ooOol0Cgoo0P00@7oo0008Ool00`00Oomoo`03Ool2000POol00`00Oomoo`0kOol2000@Ool20012
Ool000Qoo`03001oogoo00=oo`03001oogoo01moo`03001oogoo03Yoo`03001oogoo00ioo`8004Ao
o`0027oo00<007ooOol00goo00<007ooOol07goo00<007ooOol0>Goo00<007ooOol03Woo00<007oo
Ool0A7oo0007Ool20005Ool00`00Oomoo`0OOol00`00Oomoo`18Ool20017Ool000ioo`03001oogoo
01moo`03001oogoo03Eoo`8000moo`8004Uoo`003Woo00<007ooOol07goo00<007ooOol0=7oo00<0
07ooOol03Goo0P00Bgoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0cOol00`00Oomoo`0<Ool2001=
Ool000ioo`03001oogoo01moo`03001oogoo041oo`8004moo`003Woo00<007ooOol07goo00<007oo
Ool0<7oo00<007ooOol02goo0P00DGoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0_Ool00`00Oomo
o`0:Ool2001COol000ioo`03001oogoo01moo`03001oogoo02ioo`03001oogoo00Uoo`8005Eoo`00
3Woo00<007ooOol07goo00<007ooOol0>Goo00<007ooOol0EGoo000>Ool00`00Oomoo`0OOol00`00
Oomoo`0gOol2001HOol000ioo`03001oogoo01moo`03001oogoo02Yoo`8000Uoo`8005Yoo`003Woo
00<007ooOol07goo00<007ooOol0:Goo00<007ooOol01goo0P00G7oo000>Ool00`00Oomoo`0OOol0
0`00Oomoo`0XOol00`00Oomoo`06Ool2001NOol000ioo`03001oogoo01moo`03001oogoo02moo`80
061oo`003Woo00<007ooOol07goo00<007ooOol097oo0P001goo0P00HWoo000>Ool00`00Oomoo`0O
Ool00`00Oomoo`0SOol00`00Oomoo`05Ool2001TOol000Moo`@000=oo`03001oogoo01moo`03001o
ogoo029oo`03001oogoo00Aoo`8006Ioo`0027oo00<007ooOol00goo00<007ooOol07goo00<007oo
Ool0:7oo00<007ooOol0IWoo0009Ool00`00Oomoo`02Ool2000POol00`00Oomoo`0NOol20006Ool2
001YOol000Yoo`05001oogooOol0000QOol00`00Oomoo`0MOol00`00Oomoo`04Ool2001[Ool000Mo
o`04001oogoo0003Ool00`00Oomoo`0OOol00`00Oomoo`0LOol00`00Oomoo`03Ool2001]Ool000Qo
o`8000Aoo`03001oogoo01moo`03001oogoo021oo`8006moo`003Woo00<007ooOol07goo00<007oo
Ool067oo0P0017oo0P00LGoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0GOol00`00Oomoo`02Ool2
001cOol000ioo`03001oogoo01moo`03001oogoo01Ioo`04001oogooOol2001eOol000ioo`03001o
ogoo01moo`03001oogoo01Qoo`8007Moo`003Woo00<007ooOol07goo00<007ooOol04Woo0P000goo
00<007ooOol0Mgoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0AOol01000Oomoogoo0P00NWoo000>
Ool00`00Oomoo`0OOol00`00Oomoo`0@Ool00`00Oomoo`02001lOol000ioo`03001oogoo01moo`03
001oogoo015oo`8007ioo`003Woo00<007ooOol07goo00<007ooOol037oo0P0000=oo`000000P7oo
000>Ool00`00Oomoo`0OOol00`00Oomoo`0;Ool01000Ool00000PWoo000>Ool00`00Oomoo`0OOol0
0`00Oomoo`0:Ool30024Ool000ioo`03001oogoo01moo`03001oogoo00Uoo`8008Ioo`003Woo00<0
07ooOol07goo00<007ooOol01Woo0`00R7oo000>Ool00`00Oomoo`0OOol00`00Oomoo`05Ool2002:
Ool000ioo`03001oogoo01moo`03001oogoo00Aoo`8008]oo`001goo0`0017oo00<007ooOol07goo
00<007ooOol00Woo0P00SGoo000:Ool01@00Oomoogoo00008Goo00<007oo00000P00Sgoo000:Ool0
1000Oomoogoo0P0087oo0`00TGoo0008Ool20004Ool00`00Oomoo`0OOol00`00Oomoo`2AOol000Yo
o`05001oogooOol0000MOol2002FOol000Moo`<000Aoo`03001oogoo01Yoo`03001oogoo09Ioo`00
3Woo00<007ooOol06Goo00<007ooOol0Ugoo000>Ool00`00Oomoo`0EOol2002LOol000ioo`03001o
ogoo019oo`<009ioo`003Woo00<007ooOol037oo1000Xgoo000>Ool30003Ool40003Ool2002WOol0
00ioo`03001oogoo0;=oo`003Woo00<007ooOol0/goo0034Ool00<Aoo`00a7oo0034Ool00001\
\>"],
  ImageRangeCache->{{{0, 195.188}, {119.812, 0}} -> {-0.0414612, -1.98074, 
  0.00290817, 0.0471541}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
ShowPutImpliedVolatilityCalc[61, 60., 0.04, 0, 0.3, 3.34886];\
\>", 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.316236541308120422`\)], "Print",
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.023619 1.90514 0.25918 0.102763 [
[.21413 .24668 -9 -9 ]
[.21413 .24668 9 0 ]
[.40465 .24668 -9 -9 ]
[.40465 .24668 9 0 ]
[.59516 .24668 -9 -9 ]
[.59516 .24668 9 0 ]
[.78568 .24668 -9 -9 ]
[.78568 .24668 9 0 ]
[.97619 .24668 -9 -9 ]
[.97619 .24668 9 0 ]
[.01112 .05365 -12 -4.5 ]
[.01112 .05365 0 4.5 ]
[.01112 .15642 -12 -4.5 ]
[.01112 .15642 0 4.5 ]
[.01112 .36194 -6 -4.5 ]
[.01112 .36194 0 4.5 ]
[.01112 .46471 -6 -4.5 ]
[.01112 .46471 0 4.5 ]
[.01112 .56747 -6 -4.5 ]
[.01112 .56747 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21413 .25918 m
.21413 .26543 L
s
[(0.1)] .21413 .24668 0 1 Mshowa
.40465 .25918 m
.40465 .26543 L
s
[(0.2)] .40465 .24668 0 1 Mshowa
.59516 .25918 m
.59516 .26543 L
s
[(0.3)] .59516 .24668 0 1 Mshowa
.78568 .25918 m
.78568 .26543 L
s
[(0.4)] .78568 .24668 0 1 Mshowa
.97619 .25918 m
.97619 .26543 L
s
[(0.5)] .97619 .24668 0 1 Mshowa
.125 Mabswid
.06172 .25918 m
.06172 .26293 L
s
.09982 .25918 m
.09982 .26293 L
s
.13793 .25918 m
.13793 .26293 L
s
.17603 .25918 m
.17603 .26293 L
s
.25224 .25918 m
.25224 .26293 L
s
.29034 .25918 m
.29034 .26293 L
s
.32844 .25918 m
.32844 .26293 L
s
.36654 .25918 m
.36654 .26293 L
s
.44275 .25918 m
.44275 .26293 L
s
.48085 .25918 m
.48085 .26293 L
s
.51896 .25918 m
.51896 .26293 L
s
.55706 .25918 m
.55706 .26293 L
s
.63326 .25918 m
.63326 .26293 L
s
.67137 .25918 m
.67137 .26293 L
s
.70947 .25918 m
.70947 .26293 L
s
.74757 .25918 m
.74757 .26293 L
s
.82378 .25918 m
.82378 .26293 L
s
.86188 .25918 m
.86188 .26293 L
s
.89998 .25918 m
.89998 .26293 L
s
.93809 .25918 m
.93809 .26293 L
s
.25 Mabswid
0 .25918 m
1 .25918 L
s
.02362 .05365 m
.02987 .05365 L
s
[(-2)] .01112 .05365 1 0 Mshowa
.02362 .15642 m
.02987 .15642 L
s
[(-1)] .01112 .15642 1 0 Mshowa
.02362 .36194 m
.02987 .36194 L
s
[(1)] .01112 .36194 1 0 Mshowa
.02362 .46471 m
.02987 .46471 L
s
[(2)] .01112 .46471 1 0 Mshowa
.02362 .56747 m
.02987 .56747 L
s
[(3)] .01112 .56747 1 0 Mshowa
.125 Mabswid
.02362 .07421 m
.02737 .07421 L
s
.02362 .09476 m
.02737 .09476 L
s
.02362 .11531 m
.02737 .11531 L
s
.02362 .13586 m
.02737 .13586 L
s
.02362 .17697 m
.02737 .17697 L
s
.02362 .19752 m
.02737 .19752 L
s
.02362 .21808 m
.02737 .21808 L
s
.02362 .23863 m
.02737 .23863 L
s
.02362 .27973 m
.02737 .27973 L
s
.02362 .30029 m
.02737 .30029 L
s
.02362 .32084 m
.02737 .32084 L
s
.02362 .34139 m
.02737 .34139 L
s
.02362 .3825 m
.02737 .3825 L
s
.02362 .40305 m
.02737 .40305 L
s
.02362 .4236 m
.02737 .4236 L
s
.02362 .44415 m
.02737 .44415 L
s
.02362 .48526 m
.02737 .48526 L
s
.02362 .50581 m
.02737 .50581 L
s
.02362 .52636 m
.02737 .52636 L
s
.02362 .54692 m
.02737 .54692 L
s
.02362 .0331 m
.02737 .0331 L
s
.02362 .01255 m
.02737 .01255 L
s
.02362 .58802 m
.02737 .58802 L
s
.02362 .60857 m
.02737 .60857 L
s
.25 Mabswid
.02362 0 m
.02362 .61803 L
s
.001 w
[ .02 .02 ] 0 setdash
.02381 .60332 m
.02846 .60332 L
.03279 .60332 L
.03406 .60332 L
.03527 .60332 L
.03636 .60332 L
.03754 .60332 L
.03882 .60332 L
.03947 .60332 L
.04018 .60332 L
.04144 .60332 L
.04262 .60332 L
.04384 .60332 L
.04453 .60332 L
.04517 .60332 L
.04635 .60332 L
.04761 .60332 L
.04884 .60332 L
.05 .60332 L
.05104 .60332 L
.05217 .60332 L
.05341 .60331 L
.05411 .60331 L
.05477 .60331 L
.0561 .6033 L
.05684 .6033 L
.05752 .60329 L
.05873 .60328 L
.06005 .60326 L
.06129 .60323 L
.06244 .6032 L
.06486 .60312 L
.06745 .603 L
.06991 .60285 L
.07216 .60267 L
.07738 .60211 L
.08288 .60127 L
.08824 .60022 L
.09324 .59903 L
.10458 .59567 L
.12504 .58768 L
.14362 .57888 L
.18361 .55694 L
.22207 .5336 L
.26299 .50746 L
.3024 .48154 L
.34028 .45616 L
.38062 .42882 L
.41945 .40228 L
.46072 .3739 L
Mistroke
.50048 .34643 L
.53873 .31993 L
.57943 .29166 L
.61861 .26439 L
.66024 .23538 L
.70036 .20739 L
.73896 .18044 L
.78002 .15177 L
.81956 .12414 L
.85758 .09758 L
.89806 .06929 L
.93702 .04207 L
.97619 .01472 L
Mfstroke
[ ] 0 setdash
.21413 .25918 m
.21413 .53854 L
.6657 .25918 L
.6657 .23158 L
.62612 .25918 L
.62612 .25917 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
.62609 .25918 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  ImageSize->{196.188, 120.812},
  ImageMargins->{{48, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000340000N1000`40O003h00Oa7oo0034Ool00<Ao
o`00a7oo000>Ool00`00Oomoo`2cOol000ioo`03001oogoo0;=oo`003Woo00<007ooOol0ZGoo00<0
07ooOol01goo000>Ool00`00Oomoo`2XOol00`00Oomoo`08Ool000ioo`03001oogoo0:Moo`03001o
ogoo00Uoo`003Woo00<007ooOol0/goo000>Ool00`00Oomoo`2SOol2000>Ool000ioo`03001oogoo
0:9oo`03001oogoo00ioo`001goo10000goo00<007ooOol0XGoo00<007ooOol03goo0008Ool00`00
Oomoo`03Ool00`00Oomoo`2cOol00003Ool0000000<000=oo`03001oogoo009oo`8009ioo`8001Ao
o`002Woo00D007ooOomoo`0009ioo`03001oogoo01Aoo`001goo00@007ooOol000=oo`03001oogoo
09]oo`03001oogoo01Eoo`0027oo0P0017oo00<007ooOol0/goo000>Ool00`00Oomoo`2GOol2000J
Ool000ioo`03001oogoo09Ioo`03001oogoo01Yoo`003Woo00<007ooOol0UGoo00<007ooOol06goo
000>Ool00`00Oomoo`2cOol000ioo`03001oogoo095oo`80021oo`003Woo00<007ooOol0T7oo00<0
07ooOol087oo000>Ool00`00Oomoo`2?Ool00`00Oomoo`0QOol000ioo`03001oogoo0;=oo`003Woo
00<007ooOol0S7oo00<007ooOol097oo000>Ool00`00Oomoo`2;Ool00`00Oomoo`0UOol000ioo`03
001oogoo08Yoo`03001oogoo02Ioo`003Woo00<007ooOol0/goo0007Ool30004Ool00`00Oomoo`26
Ool2000[Ool000Qoo`03001oogoo00=oo`03001oogoo08Eoo`03001oogoo02]oo`0000=oo`000000
0`000Woo00<007ooOol00goo0P00QGoo00<007ooOol0;7oo0008Ool00`00Oomoo`03Ool00`00Oomo
o`2cOol000Qoo`03001oogoo00=oo`03001oogoo081oo`80035oo`001goo0P001Goo00<007ooOol0
Ogoo00<007ooOol0<Goo000>Ool00`00Oomoo`1nOol00`00Oomoo`0bOol000ioo`03001oogoo07io
o`03001oogoo039oo`003Woo00<007ooOol0/goo000>Ool00`00Oomoo`1jOol2000gOol000ioo`03
001oogoo07Uoo`03001oogoo03Moo`003Woo00<007ooOol067oo0P0017oo0P000goo0`005Goo0P00
17oo0P000goo10004goo0P0017oo0P000goo0`003Goo00<007ooOol017oo0P0017oo0P0017oo0`00
57oo0P0017oo0P000goo0`000goo000>Ool00`00Oomoo`0GOol01000Oomoo`000goo0P0017oo00<0
07ooOol04goo00@007ooOol000=oo`8000Aoo`03001oogoo019oo`04001oogoo0003Ool20006Ool0
0`00Oomoo`0@Ool01000Oomoo`000goo0P001Goo00<007ooOol04Woo00@007ooOol000=oo`8000Io
o`40005oo`5oo`003Woo00<007ooOol05goo00@007ooOol000Uoo`03001oogoo01=oo`04001oogoo
000:Ool00`00Oomoo`0AOol01000Oomoo`002goo00<007ooOol01Woo0P0027oo00@007ooOol000Qo
o`@001=oo`04001oogoo000;Ool10001Ool1Ool000ioo`03001oogoo01Moo`04001oogoo0009Ool0
0`00Oomoo`0COol01000Oomoo`002goo00<007ooOol047oo00@007ooOol000Uoo`8000Qoo`03001o
ogoo00Qoo`04001oogoo0008Ool00`00Ool0000DOol01000Oomoo`0027oo0`000goo000>Ool00`00
Oomoo`0GOol01000Oomoo`002Goo00<007ooOol04goo00@007ooOol000Qoo`04001oogoo000BOol0
1000Oomoo`002goo00<007ooOol017oo00<007ooOol02Goo00@007ooOol000Qoo`03001oo`0001Ao
o`04001oogoo0008Ool00`00Oomoo`03Ool000ioo`03001oogoo01Qoo`8000Uoo`8001Ioo`8000Yo
o`8001Aoo`8000Uoo`<000Eoo`8000eoo`8000Yoo`8001Eoo`8000Uoo`@0009oo`003Woo00<007oo
Ool0KWoo0P0000=oo`00Ool0@7oo000>Ool00`00Oomoo`1]Ool20002Ool00`00Oomoo`0oOol000io
o`03001oogoo06aoo`03001oogoo009oo`03001oogoo03moo`003Woo00<007ooOol0Jgoo00<007oo
Ool00goo00<007ooOol0?goo0009Oonf0005Ool000ioo`03001oogoo01moo`03001oogoo021oo`03
001oogoo01moo`03001oogoo008000Ioo`03001oogoo01Aoo`03001oogoo021oo`03001oogoo00Mo
o`003Woo00<007ooOol07goo00<007ooOol0A7oo00<007ooOol017oo0P00A7oo000>Ool00`00Oomo
o`0OOol00`00Oomoo`19Ool20016Ool000ioo`03001oogoo01moo`03001oogoo041oo`8000Ioo`03
001oogoo04Ioo`003Woo00<007ooOol07goo00<007ooOol0?goo00<007ooOol017oo0P00BGoo000>
Ool00`00Oomoo`0OOol00`00Oomoo`0nOol00`00Oomoo`03Ool2001;Ool000ioo`03001oogoo01mo
o`03001oogoo04=oo`03001oogoo04]oo`003Woo00<007ooOol07goo00<007ooOol0>Woo0P001Goo
0P00CWoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0iOol00`00Oomoo`03Ool2001@Ool000ioo`03
001oogoo01moo`03001oogoo03Qoo`03001oogoo00=oo`03001oogoo051oo`003Woo00<007ooOol0
7goo00<007ooOol0?7oo0P00Dgoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0dOol20004Ool2001E
Ool000ioo`03001oogoo01moo`03001oogoo03=oo`03001oogoo00=oo`03001oogoo05Eoo`003Woo
00<007ooOol07goo00<007ooOol0<Woo00<007ooOol00Woo0P00F7oo000>Ool00`00Oomoo`0OOol0
0`00Oomoo`0fOol00`00Oomoo`1HOol000Moo`<000Aoo`03001oogoo01moo`03001oogoo02ioo`80
00Aoo`8005]oo`0027oo00<007ooOol00goo00<007ooOol07goo00<007ooOol0;Goo00<007ooOol0
0Woo0P00GGoo0008Ool00`00Oomoo`03Ool2000POol00`00Oomoo`0/Ool00`00Oomoo`02Ool00`00
Oomoo`1MOol000Qoo`03001oogoo00=oo`03001oogoo01moo`03001oogoo02moo`80061oo`0027oo
00<007ooOol00goo00<007ooOol07goo00<007ooOol0:Woo00<007ooOol00P00HWoo0007Ool20005
Ool00`00Oomoo`0OOol00`00Oomoo`0YOol01000Oomoo`00I7oo000>Ool00`00Oomoo`0OOol00`00
Oomoo`0ZOol2001UOol000ioo`03001oogoo01moo`03001oogoo02Eoo`800003Ool0000006Moo`00
3Woo00<007ooOol07goo00<007ooOol097oo00@007ooOol006Uoo`003Woo00<007ooOol07goo00<0
07ooOol08goo00@007oo000006Yoo`003Woo00<007ooOol07goo00<007ooOol097oo00<007ooOol0
JWoo000>Ool00`00Oomoo`0OOol00`00Oomoo`0OOol200000goo0000001]Ool000ioo`03001oogoo
01moo`03001oogoo01ioo`04001oo`00001_Ool000ioo`03001oogoo01moo`03001oogoo01eoo`03
001oo`00075oo`003Woo00<007ooOol07goo00<007ooOol07Goo0P00LWoo000>Ool00`00Oomoo`0O
Ool00`00Oomoo`0IOol4001dOol000ioo`03001oogoo01moo`03001oogoo01Qoo`03001oo`0007Io
o`003Woo00<007ooOol07goo00<007ooOol05goo0`00Mgoo000>Ool00`00Oomoo`0OOol00`00Oomo
o`0FOol2001iOol000Moo`@000=oo`03001oogoo01moo`03001oogoo01=oo`<007]oo`0027oo00<0
07ooOol00goo00<007ooOol07goo00<007ooOol04Woo0`00O7oo0009Ool00`00Oomoo`02Ool2000P
Ool00`00Oomoo`0AOol2001nOol000Yoo`05001oogooOol0000QOol00`00Oomoo`0@Ool00`00Oomo
o`1nOol000Moo`04001oogoo0003Ool00`00Oomoo`0OOol00`00Oomoo`0>Ool20021Ool000Qoo`80
00Aoo`03001oogoo01moo`03001oogoo00aoo`8008=oo`003Woo00<007ooOol07goo00<007ooOol0
2goo0P00Q7oo000>Ool00`00Oomoo`0OOol00`00Oomoo`09Ool20026Ool000ioo`03001oogoo01mo
o`03001oogoo00Moo`8008Qoo`003Woo00<007ooOol07goo00<007ooOol01Woo0P00RGoo000>Ool0
0`00Oomoo`0OOol00`00Oomoo`04Ool2002;Ool000ioo`03001oogoo01moo`03001oogoo00=oo`03
001oogoo08]oo`003Woo00<007ooOol07goo00@007ooOomoo`8008ioo`003Woo00<007ooOol07goo
00@007oo0000091oo`003Woo00<007ooOol07goo0`00TGoo000>Ool00`00Oomoo`0OOol00`00Oomo
o`2AOol000ioo`03001oogoo01eoo`8009Aoo`003Woo00<007ooOol077oo00<007ooOol0U7oo0007
Ool30004Ool00`00Oomoo`0KOol00`00Oomoo`2EOol000Yoo`05001oogooOol0000IOol2002JOol0
00Yoo`04001oogooOol2000GOol00`00Oomoo`2JOol000Qoo`8000Aoo`03001oogoo01Eoo`03001o
ogoo09]oo`002Woo00D007ooOomoo`000;Eoo`001goo0`0017oo00<007ooOol047oo0P00XGoo000>
Ool00`00Oomoo`0>Ool2002SOol000ioo`03001oogoo0;=oo`003Woo00<007ooOol01goo1@00Ygoo
000>Ool00`00Oomoo`05002^Ool000ioo`03001oogoo0;=oo`003Woo00<007ooOol0/goo0034Ool0
0<Aoo`00a7oo0034Ool00001\
\>"],
  ImageRangeCache->{{{0, 195.188}, {119.812, 0}} -> {-0.0414612, -2.74489, 
  0.00290817, 0.053915}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["7.10 Packaging It Up", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell[TextData[{
  "The functions that we have introduced in this chapter can be made into a \
standard ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Package, which can be loaded at will. We shall make extensive use of the \
basic Black-Scholes functions throughout this text, so this will be very \
convenient for us. In particular, we shall load it whenever we wish to verify \
a numerical scheme for solving an option-pricing problem. Any numerical \
scheme has to be able to get vanilla European Calls and Puts exactly right \
for it to be credible in more complex applications, so this will be a very \
useful Package indeed.\n\nIt is not our purpose here to explain the general \
notion of a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Package or contexts. Rather, we wish to exhibit the structure of a \
Package in a form that is useful for practical applications, and which can be \
mimicked to provide other Packages containing many other useful functions. \
Indeed, we shall set up a similar Package for American options in the Chapter \
11. For a detailed explanation of Packages and contexts, you are referred to \
Chapter 13 of Shaw & Tigg (1993), and to the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Packages documentation.\n\nIt is assumed that the following Package has \
been copied as a text file into the appropriate place of your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " file structure - see the remarks at the end of this Chapter for how to do \
this. In order to test out the following functions in a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session, we advise you to quit and re-start the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " kernel to clear out the kernel of all definitions we have made. Then load \
the Package. Note that this may take a few seconds as the computation of the \
Greeks is carried out."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Quit[]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["First of all we can query what functions have been loaded:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Derivatives`BlackScholes`*\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
BlackScholesCall       BlackScholesCallVega   BlackScholesPutTheta
BlackScholesCallDelta  BlackScholesPut        BlackScholesPutVega
BlackScholesCallGamma  BlackScholesPutDelta   CallPayoff
BlackScholesCallImpVol BlackScholesPutGamma   Norm
BlackScholesCallRho    BlackScholesPutImpVol  PutPayoff
BlackScholesCallTheta  BlackScholesPutRho\
\>", "Print",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell["Next we can query the operation of individual functions:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Norm\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Norm[x] returns the cumulative normal distribution function evaluated \
at x."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesCall\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesCall[price, strike, vol, riskfree, divyield, expiry]  \
returns the Black-Scholes value of a vanilla European Call."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesPutVega\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesPutVega[price, strike, vol, riskfree, divyield, expiry]  \
returns the vega (lambda) of a vanilla European Put."\)], "Print"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?BlackScholesCallImpVol\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("BlackScholesCallImpVol[price, strike, riskfree, divyield, expiry, \
optionprice]  returns the implied volatility of a vanilla European Call."\)], 
  "Print"]
}, Open  ]],

Cell["\<\
and so on. It is always a good idea to check that the functions are \
working!\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCall[51, \ 50, \ 0.2, \ 0.03, \ 0, \ 3]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9.62758921616179463`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesCallImpVol[51, \ 50, \ 0.03, \ 0, \ 3, \ 9.62759]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    StyleBox["0.2`",
      StyleBoxAutoDelete->True,
      PrintPrecision->1]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPutGamma[51, \ 50, \ 0.2, \ 0.03, \ 0, \ 3]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0200250835153559236`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(BlackScholesPutImpVol[51, \ 50, \ 0.03, \ 0, \ 3, \ 4.32415]\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.200000048647074502`\)], "Output"]
}, Open  ]],

Cell[TextData[{
  "Now you can proceed to use the functions just as if they had been entered \
in a Notebook. Now we give the complete listing of the package. You can use \
this as a template for building your own packages. The file is called \
\"BlackScholes.m\" and it should be stored in a directory or folder called \
\"Derivatives\" in the \"ExtraPackages\" sub-directory of the \"AddOns\" \
directory of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " installation area of your system. If you have any difficulty with the \
installation of this package, I recommend you copy the text below into a text \
file, give it the name \"BlackScholes.m\", remove any line breaks within \
code, and manually place it in the right place."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Package Listing", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  PageBreakWithin->True],

Cell["\<\
(* :Title: 
\tBlackScholes Package *)

(* :Context: 
\tDerivatives`BlackScholes` *)

(* :Author: 
\tDr William T. Shaw *)

(* :Summary: 
\t*)

(* :Package Version: 
\t1.1 *)

(* :Mathematica Version: 
\t3.0, but compatible with 2.X *)

(* :History:
\tV.1.0, January 1996, by William T. Shaw *)

BeginPackage[\"Derivatives`BlackScholes`\"]

(* Usage Information *)

CallPayoff::usage = 
\"CallPayoff[price, strike] returns the payoff value of a vanilla 
European Call.\"

PutPayoff::usage = 
\"PutPayoff[price, strike] returns the payoff value of a vanilla 
European Put.\"

BlackScholesCall::usage = 
\"BlackScholesCall[price, strike, vol, riskfree, divyield, expiry] 
returns the Black-Scholes value of a vanilla European Call.\"

BlackScholesPut::usage = 
\"BlackScholesPut[price, strike, vol, riskfree, divyield, expiry] 
returns the Black-Scholes value of a vanilla European Put.\"

BlackScholesCallDelta::usage = 
\"BSCallDelta[price, strike, vol, riskfree, divyield, expiry] 
returns the delta of a vanilla European Call.\"

BlackScholesPutDelta::usage = 
\"BlackScholesPutDelta[price, strike, vol, riskfree, divyield, expiry] 
returns the delta of a vanilla European Put.\"

BlackScholesCallTheta::usage = 
\"BlackScholesCallTheta[price, strike, vol, riskfree, divyield, expiry] 
returns the theta of a vanilla European Call.\"

BlackScholesPutTheta::usage = 
\"BlackScholesPutTheta[price, strike, vol, riskfree, divyield, expiry] 
returns the theta of a vanilla European Put.\"

BlackScholesCallRho::usage = 
\"BlackScholesCallRho[price, strike, vol, riskfree, divyield, expiry] 
returns the rho of a vanilla European Call.\"

BlackScholesPutRho::usage = 
\"BlackScholesPutRho[price, strike, vol, riskfree, divyield, expiry] 
returns the rho of a vanilla European Put.\"

BlackScholesCallGamma::usage = 
\"BlackScholesCallGamma[price, strike, vol, riskfree, divyield, expiry] 
returns the gamma of a vanilla European Call.\"

BlackScholesPutGamma::usage = 
\"BlackScholesPutGamma[price, strike, vol, riskfree, divyield, expiry] 
returns the gamma of a vanilla European Call.\"

BlackScholesCallVega::usage = 
\"BlackScholesCallVega[price, strike, vol, riskfree, divyield, expiry] 
returns the vega (lambda) of a vanilla European Call.\"

BlackScholesPutVega::usage = 
\"BlackScholesPutVega[price, strike, vol, riskfree, divyield, expiry] 
returns the vega (lambda) of a vanilla European Put.\"

Norm::usage = 
\"Norm[x] returns the cumulative normal distribution function evaluated at x.\
\"

BlackScholesCallImpVol::usage = 
\"BlackScholesCallImpVol[price, strike, riskfree, divyield, expiry, \
optionprice] 
returns the implied volatility of a vanilla European Call.\"

BlackScholesPutImpVol::usage = 
\"BlackScholesPutImpVol[price, strike, riskfree, divyield, expiry, \
optionprice] 
returns the implied volatility of a vanilla European Call.\"



(* END OF USAGE INFORMATION *)

Begin[\"`Private`\"]

Norm[z_?NumberQ]:= N[0.5 + 0.5 Erf[z/Sqrt[2]]];

Norm[x_] := (1 + Erf[x / Sqrt[2]]) / 2;

dtwo[s_, v_, k_, t_, r_, q_] :=(Log[s / k] + (r-q) t) / (v Sqrt[t]) -1/2 v \
Sqrt[t];
   
done[s_, v_, k_, t_, r_, q_] :=(Log[s / k] + (r-q) t) / (v Sqrt[t]) + 1/2 v \
Sqrt[t];
   
BlackScholesCall[s_,k_, v_, r_, q_, t_] := 
s Exp[-q*t] Norm[done[s, v, k, t, r, q]] - k*Exp[-r*t] Norm[dtwo[s, v, k, t, \
r, q]];

BlackScholesPut[s_,k_, v_, r_, q_, t_]:= 
k*Exp[-r*t] Norm[-dtwo[s, v, k, t, r, q]]-s Exp[-q*t] Norm[-done[s, v, k, t, \
r, q]];

BlackScholesCallDelta[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], s]];

BlackScholesPutDelta[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], s]];

BlackScholesCallGamma[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], {s, 2}]];

BlackScholesPutGamma[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], {s, 2}]];

BlackScholesCallTheta[s_,k_, v_, r_, q_, t_]=
-Evaluate[D[BlackScholesCall[s,k, v, r, q, t], t]];

BlackScholesPutTheta[s_,k_, v_, r_, q_, t_]=
-Evaluate[D[BlackScholesPut[s,k, v, r, q, t], t]];

BlackScholesCallRho[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], r]];

BlackScholesPutRho[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], r]];

BlackScholesCallVega[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesCall[s,k, v, r, q, t], v]];

BlackScholesPutVega[s_,k_, v_, r_, q_, t_]= 
Evaluate[D[BlackScholesPut[s,k, v, r, q, t], v]];

CallPayoff[s_, k_] := Max[0, s-k];

PutPayoff[s_, k_] := Max[0, k-s];

BlackScholesCallImpVol[s_,k_,r_,q_,t_,optionprice_] := 
sd /. FindRoot[BlackScholesCall[s,k, sd, r, q, t]== optionprice,{sd,0.2}];

BlackScholesPutImpVol[s_,k_,r_,q_,t_,optionprice_] := 
sd /. FindRoot[BlackScholesPut[s,k, sd, r, q, t]== optionprice,{sd,0.2}];

End[]

EndPackage[]\
\>", "Input",
  PageWidth->Infinity,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True,
  InitializationCell->True,
  ShowSpecialCharacters->False,
  FormatType->InputForm,
  FontSize->8,
  FontWeight->"Plain"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 7 Bibliography", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Black, F. and Scholes, M., 1973, The pricing of assets on corporate \
liabilities, ",
  StyleBox["Journal of Political Economy,",
    FontSlant->"Italic"],
  " 81, pp. 637-659.\n\n",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " ",
  StyleBox["Finance Pack",
    FontSlant->"Italic"],
  ", Wolfram Research, Champaign, Ill., USA. \n\n",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  StyleBox[" 3.0",
    FontSlant->"Italic"],
  " ",
  StyleBox["Standard Add-on Packages",
    FontSlant->"Italic"],
  ", Wolfram Media-Cambridge University Press, 1996.\n\nMerton, R., 1973, \
Theory of rational option pricing, ",
  StyleBox["Bell Journal of Economics and Management Science",
    FontSlant->"Italic"],
  ", 4, p. 141.\n\nMiller, R., 1993, chapter in Varian, 1993.\n\nShaw, W.T. \
and Tigg, J., 1993,",
  StyleBox[" Applied Mathematica, Getting Starte",
    FontSlant->"Italic"],
  StyleBox["d, Getting it Done",
    FontSlant->"Italic"],
  ", Addison-Wesley, 1993.\n\nVarian, H., 1993, ",
  StyleBox["Economic and Financial Modelling with Mathematica",
    FontSlant->"Italic"],
  ", Springer-TELOS."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{"RulerBar", "EditBar"},
CellGrouping->Automatic,
WindowSize->{587, 696},
WindowMargins->{{28, Automatic}, {Automatic, 4}},
PrintingCopies->1,
PrintingStartingPageNumber->151,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"7. European Calls and Puts"}]]}, {Cell[ 
        TextData[ {"7. European Calls and Puts"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8^on3ohP==PTc0b85N`?N0080004/0B`000006G<AMP01
0000I00000400@4100800BL?00400@S0Qc0000000000064/01T1T00000000000
00000000007oo@S0Uo3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 220, 8, 171, "Title"],
Cell[1954, 61, 153, 4, 45, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2132, 69, 140, 3, 49, "Section"],
Cell[2275, 74, 1811, 36, 374, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[4123, 115, 143, 3, 48, "Section"],
Cell[4269, 120, 403, 9, 58, "Text"],
Cell[4675, 131, 206, 5, 37, "Input"],

Cell[CellGroupData[{
Cell[4906, 140, 127, 5, 37, "Input"],
Cell[5036, 147, 36, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5109, 153, 127, 5, 36, "Input"],
Cell[5239, 160, 35, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5311, 166, 147, 6, 46, "Input"],
Cell[5461, 174, 10995, 332, 116, 3299, 231, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[16471, 509, 249, 7, 40, "Text"],
Cell[16723, 518, 167, 4, 36, "Input"],
Cell[16893, 524, 228, 8, 40, "Text"],
Cell[17124, 534, 298, 7, 57, "Input"],
Cell[17425, 543, 346, 8, 80, "Input"],
Cell[17774, 553, 385, 10, 29, "Text"],

Cell[CellGroupData[{
Cell[18184, 567, 208, 5, 36, "Input"],
Cell[18395, 574, 12250, 434, 116, 3964, 326, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[30660, 1011, 507, 11, 52, "Text"],

Cell[CellGroupData[{
Cell[31192, 1026, 230, 5, 36, "Input"],

Cell[CellGroupData[{
Cell[31447, 1035, 14038, 581, 116, 5523, 469, "GraphicsData", 
"PostScript", "Graphics"],
Cell[45488, 1618, 12283, 438, 116, 4011, 329, "GraphicsData", 
"PostScript", "Graphics"],
Cell[57774, 2058, 12275, 435, 116, 3963, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[70052, 2495, 12276, 435, 116, 3964, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[82331, 2932, 12277, 435, 116, 3965, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[94611, 3369, 12289, 436, 116, 3964, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[106903, 3807, 12268, 435, 116, 3964, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[119174, 4244, 12280, 435, 116, 3964, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[131457, 4681, 12285, 435, 116, 3965, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[143745, 5118, 12278, 435, 116, 3962, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[156026, 5555, 12298, 436, 116, 3965, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[168327, 5993, 12287, 436, 116, 3962, 326, "GraphicsData", 
"PostScript", "Graphics"],
Cell[180617, 6431, 12298, 436, 116, 3965, 326, "GraphicsData", 
"PostScript", "Graphics"]
}, Closed]]
}, Open  ]],
Cell[192942, 6871, 298, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[193265, 6882, 236, 5, 36, "Input"],
Cell[193504, 6889, 163367, 3912, 160, 126852, 3454, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[356920, 10807, 173, 4, 38, "Section",
  PageBreakAbove->False],
Cell[357096, 10813, 1101, 23, 117, "Text"],
Cell[358200, 10838, 1177, 35, 332, "Input",
  PageBreakAbove->False,
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[359414, 10878, 95, 1, 38, "Section"],
Cell[359512, 10881, 435, 11, 52, "Text"],

Cell[CellGroupData[{
Cell[359972, 10896, 159, 4, 25, "Input"],
Cell[360134, 10902, 171, 3, 56, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[360342, 10910, 211, 8, 57, "Input",
  PageBreakAbove->Automatic],
Cell[360556, 10920, 11936, 396, 118, 4357, 296, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->Automatic]
}, Open  ]],

Cell[CellGroupData[{
Cell[372529, 11321, 238, 6, 36, "Input",
  PageBreakAbove->Automatic],
Cell[372770, 11329, 145450, 3702, 129, 127138, 3471, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[518269, 15037, 95, 1, 38, "Section"],
Cell[518367, 15040, 418, 11, 40, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[518810, 15055, 206, 6, 25, "Input",
  PageBreakAbove->False],
Cell[519019, 15063, 13724, 430, 113, 4500, 309, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[532780, 15498, 259, 6, 36, "Input",
  PageBreakAbove->False],
Cell[533042, 15506, 150092, 3755, 129, 127041, 3465, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[683183, 19267, 95, 1, 38, "Section"],
Cell[683281, 19270, 336, 10, 40, "Text"],

Cell[CellGroupData[{
Cell[683642, 19284, 211, 5, 36, "Input"],
Cell[683856, 19291, 163730, 3904, 156, 126552, 3439, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[847635, 23201, 94, 1, 38, "Section"],
Cell[847732, 23204, 340, 10, 40, "Text"],

Cell[CellGroupData[{
Cell[848097, 23218, 210, 5, 36, "Input"],
Cell[848310, 23225, 168547, 3976, 156, 126302, 3448, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1016906, 27207, 117, 2, 38, "Section",
  PageBreakAbove->True],
Cell[1017026, 27211, 377, 10, 40, "Text"],

Cell[CellGroupData[{
Cell[1017428, 27225, 178, 5, 25, "Input"],
Cell[1017609, 27232, 12875, 453, 118, 5076, 351, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1030521, 27690, 210, 5, 36, "Input"],
Cell[1030734, 27697, 158763, 3838, 136, 126909, 3439, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1189546, 31541, 169, 3, 38, "Section"],
Cell[1189718, 31546, 420, 11, 40, "Text"],

Cell[CellGroupData[{
Cell[1190163, 31561, 103, 2, 25, "Input"],
Cell[1190269, 31565, 144, 2, 32, "Print"]
}, Open  ]],
Cell[1190428, 31570, 371, 9, 68, "Input"],
Cell[1190802, 31581, 335, 10, 40, "Text"],

Cell[CellGroupData[{
Cell[1191162, 31595, 156, 2, 25, "Input"],
Cell[1191321, 31599, 55, 1, 36, "Output"]
}, Open  ]],
Cell[1191391, 31603, 913, 17, 96, "Text"],
Cell[1192307, 31622, 290, 11, 101, "Input"],
Cell[1192600, 31635, 182, 4, 20, "Text"],
Cell[1192785, 31641, 157, 2, 25, "Input"],
Cell[1192945, 31645, 153, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[1193123, 31653, 137, 3, 25, "Input"],
Cell[1193263, 31658, 2309, 49, 200, "Output"]
}, Open  ]],
Cell[1195587, 31710, 129, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[1195741, 31715, 142, 2, 25, "Input"],
Cell[1195886, 31719, 54, 1, 36, "Output"]
}, Open  ]],
Cell[1195955, 31723, 89, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[1196069, 31728, 118, 2, 25, "Input"],
Cell[1196190, 31732, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1196281, 31738, 102, 1, 32, "Subsection"],
Cell[1196386, 31741, 454, 10, 40, "Text"],
Cell[1196843, 31753, 587, 14, 124, "Input",
  PageBreakWithin->True],
Cell[1197433, 31769, 282, 6, 46, "Input"],
Cell[1197718, 31777, 305, 7, 46, "Input",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[1198048, 31788, 181, 3, 25, "Input",
  PageBreakAbove->False],
Cell[1198232, 31793, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1198323, 31799, 199, 6, 25, "Input",
  PageBreakAbove->False],
Cell[1198525, 31807, 54, 1, 20, "Print"],
Cell[1198582, 31810, 11782, 418, 128, 4647, 325, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]],

Cell[CellGroupData[{
Cell[1210401, 32233, 227, 7, 25, "Input",
  PageBreakAbove->False],
Cell[1210631, 32242, 84, 2, 20, "Print"],
Cell[1210718, 32246, 11208, 415, 128, 4663, 328, "GraphicsData", 
"PostScript", "Graphics",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1221987, 32668, 120, 2, 38, "Section",
  PageBreakAbove->True],
Cell[1222110, 32672, 1998, 38, 216, "Text"],
Cell[1224111, 32712, 80, 1, 25, "Input"],
Cell[1224194, 32715, 128, 2, 25, "Input"],
Cell[1224325, 32719, 131, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[1224481, 32724, 121, 2, 25, "Input"],
Cell[1224605, 32728, 420, 8, 73, "Print"]
}, Open  ]],
Cell[1225040, 32739, 129, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[1225194, 32744, 99, 2, 25, "Input"],
Cell[1225296, 32748, 112, 2, 32, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1225445, 32755, 111, 2, 25, "Input"],
Cell[1225559, 32759, 162, 2, 32, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1225758, 32766, 114, 2, 25, "Input"],
Cell[1225875, 32770, 158, 2, 32, "Print"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1226070, 32777, 117, 2, 25, "Input"],
Cell[1226190, 32781, 178, 3, 44, "Print"]
}, Open  ]],
Cell[1226383, 32787, 159, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[1226567, 32795, 141, 2, 25, "Input"],
Cell[1226711, 32799, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1226802, 32805, 154, 3, 25, "Input"],
Cell[1226959, 32810, 103, 3, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1227099, 32818, 145, 2, 20, "Input"],
Cell[1227247, 32822, 56, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[1227340, 32828, 152, 3, 20, "Input"],
Cell[1227495, 32833, 55, 1, 20, "Output"]
}, Open  ]],
Cell[1227565, 32837, 817, 14, 72, "Text"],

Cell[CellGroupData[{
Cell[1228407, 32855, 151, 3, 28, "Subsubsection",
  PageBreakAbove->Automatic,
  PageBreakWithin->True],
Cell[1228561, 32860, 5065, 172, 1402, "Input",
  PageBreakWithin->True,
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[1233675, 33038, 98, 1, 38, "Section"],
Cell[1233776, 33041, 1205, 34, 184, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

