(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    694721,      16917]*)
(*NotebookOutlinePosition[    696581,      16974]*)
(*  CellTagsIndexPosition[    696537,      16970]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 12. 
Double-Barrier, Compound, Quanto Options and Other Exotics\
\>", "Title",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["A Bundle of Exotics and Diverse Methodologies", "Subsubtitle",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["12.1 Introduction", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\

In this chapter we consider several different types of \"exotic\" option, \
including:

double-barrier options;
compound options;
Quanto options;
warrants;
chooser options;
exchange options.

These illustrate various types of problem. Double-barriers can be solved both \
by series methods and by Laplace transform methods - we present an analysis \
of the latter based on methods similar to those used for arithmetic options. \
Compound and Quanto options have in common the fact that there are two \
correlated random processes present. In the case of compound options these \
are treated through the bivariate normal distribution, whereas Quantos can be \
reduced to a single-factor approach by using the arguments developed already \
for linked asset-FX options. We also consider some other commonplace exotics, \
such as chooser and exchange options, and briefly explore the effects of \
dilution in warrant pricing.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.2 Double-Barrier Options", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Introduction", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A double-barrier option is characterized by a strike, ",
  Cell[BoxData[
      \(TraditionalForm\`K\)]],
  ", and two barriers, ",
  Cell[BoxData[
      \(TraditionalForm\`L\)]],
  ", the lower barrier, and ",
  Cell[BoxData[
      \(TraditionalForm\`U\)]],
  ", the upper barrier, with "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`L < K < U\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The option knocks out, i.e., becomes worthless, if the stock price \
hits either barrier (rebates can be added). Otherwise the payoff at maturity \
is a standard type. In this section we shall consider a double-barrier Call \
option. The naive approach to this problem is to do a Fourier series type of \
solution - indeed, the solution of the corresponding heat equation, where the \
temperature is fixed at zero at both ends, and subject to an initial \
condition, is a familiar exercise in undergraduate mathematics. Just as \
familiar are the associated phenomena of the Gibbs effect, arising from a \
discontinuity between the initial condition and the boundary condition. This \
plays havoc with the series solution, at least for smaller times to maturity, \
in the neighbourhood of the offending barrier. The derivatives of the answer \
are particularly vulnerable to corruption in this respect. Such a \
discontinuity in initial-boundary data can also cause trouble for schemes \
such as simple implicit finite-difference schemes. Monte Carlo and tree \
methods also have their own problems when it comes to barrier options. In the \
case of Monte Carlo methods, an extremely large number of samples may be \
required to model effectively the impact of the barrier, and tree models are \
easily corrupted by nodes not being placed on the barrier. Not all \
series-type methods suffer from the kinds of pathology we have described. \
Work by Kunitomo and Ikeda (1992) describes another method which they argue \
converges very rapidly.

It is highly desirable therefore to have a non-series analytical approach. \
Remarkably, one such method was published very recently while this text was \
in preparation. Geman and Yor (1996) have shown how to employ the Laplace \
transform method to double-barrier options, using methods similar to those \
deployed for arithmetic Asian options, as described in Chapter 10. They \
demonstrated good agreement with Kunitomo and Ikeda. We shall explore and \
implement this method here.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Geman-Yor Model for Double-Barrier Options", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We shall work with a double knock-out Call option whose value is given by \
",
  Cell[BoxData[
      \(TraditionalForm\`C\)]],
  ". It is convenient to write"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`C = 
      C\_BS - \[ExponentialE]\^\(\(-r\)\ t\)\ S\ \[Phi]\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "where ",
  Cell[BoxData[
      \(TraditionalForm\`C\_BS\)]],
  " is the standard Black-Scholes value and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi]\)]],
  " is a normalized measure of the diminution in the value due to the double \
barrier. As usual, the risk-free continuously compounded interest rate is ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " and the stock price at initiation of the option is ",
  Cell[BoxData[
      \(TraditionalForm\`S\)]],
  ". The knock-out term ",
  Cell[BoxData[
      \(TraditionalForm\`\[Phi]\)]],
  " can be written as a function of the time to expiry ",
  Cell[BoxData[
      \(TraditionalForm\`t\)]],
  " and the scaled variables:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`h = \(K\/S\ \ \ \ \ \ \ m = \(L\/S\ \ \ \ \ \ \ M = 
          U\/S\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["The knock-out term is Laplace-transformed to define", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Psi](\[Lambda]) = \[Integral]\_0\%\[Infinity]\( \
\[ExponentialE]\^\(\(-\[Lambda]\)\ \[Tau]\)\ \(\[Phi](\[Tau], h, m, 
              M)\)\) \[DifferentialD]\[Tau]\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Further scaling arguments can be used to write", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Psi](\[Lambda]) = \
\(\[CapitalPhi](\[Lambda]\/\[Sigma]\^2)\)\/\[Sigma]\^2\)], "NumberedEquation",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "What is remarkable about the Geman-Yor analysis is that they have found an \
exact closed-form solution for ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPhi](\[Theta])\)]],
  ". First of all, we need to define (positive) quantities ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)]],
  " by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`m = \(\[ExponentialE]\^\(-a\)\ \ \ \ \ \ \ M = \
\[ExponentialE]\^b\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We also introduce a variable ",
  Cell[BoxData[
      \(TraditionalForm\`\[Nu]\)]],
  " as follows:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Nu] = \(r - q - \[Sigma]\^2\/2\)\/\[Sigma]\^2\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Finally we need a variable ",
  Cell[BoxData[
      \(TraditionalForm\`\[Mu]\)]],
  " given in terms of the argument of the Laplace transform by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[Mu] = \@\(\[Nu]\^2 + 2\ \[Theta]\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "To build the Laplace transform, we now introduce some auxiliary functions \
",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  " given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`f(a, 
        h, \[Mu], \[Nu]) = \(h\^\(\(-\[Mu]\) + \[Nu] + 1\)\ \[ExponentialE]\^\
\(\(-\[Mu]\)\ a\)\)\/\(\[Mu]\ \((\[Mu] - \[Nu])\)\ \((\[Mu] - \[Nu] - \
1)\)\)\)], "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`g(b, 
        h, \[Mu], \[Nu]) = \(2\ \[ExponentialE]\^\(b\ \((\[Nu] + 1)\)\)\)\/\(\
\[Mu]\^2 - \((\[Nu] + 1)\)\^2\) - \(2\ h\ \[ExponentialE]\^\(b\ \[Nu]\)\)\/\(\
\[Mu]\^2 - \[Nu]\^2\) + \(h\^\(\[Mu] + \[Nu] + 1\)\ \[ExponentialE]\^\(\(-\
\[Mu]\)\ b\)\)\/\(\[Mu]\ \((\[Mu] + \[Nu])\)\ \((\[Mu] + \[Nu] + 1)\)\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Then the Laplace transform ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPhi]\)]],
  " is given by "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`\[CapitalPhi](\[Theta]) = \(\(g(b, h, \[Mu], \[Nu])\)\ \
\(sinh(\[Mu]\ a)\)\)\/\(sinh(\[Mu]\ \((a + b)\))\) + \(\(f(a, h, \[Mu], \
\[Nu])\)\ \(sinh(\[Mu]\ b)\)\)\/\(sinh(\[Mu]\ \((a + b)\))\)\)], 
  "NumberedEquation",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
At the time of writing, a closed-form inversion of this is not \
known, but one can use a variety of numerical methods to invert it. \
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Implementation of the Geman-Yor Model"
}], "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["First we just enter the basic changes of variable employed:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
h[S_, K_] := K/S; 
a[S_, L_] := Log[S/L]; 
b[S_, U_] := Log[U/S];
\[Nu][\[Sigma]_, r_, q_] := (r - q - \[Sigma]^2/2)/\[Sigma]^2;
\[Mu][\[Nu]_, \[Theta]_] := Sqrt[\[Nu]^2 + 2*\[Theta]]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "We now define the auxiliary functions ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`g\)]],
  " given by"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(f[a_, 
        h_, \[Mu]_, \[Nu]_] := \(h\^\(\(-\[Mu]\) + \[Nu] + 1\)\ \
Exp[\(-\[Mu]\)\ a]\)\/\(\[Mu]\ \((\[Mu] - \[Nu])\)\ \((\[Mu] - \[Nu] - 1)\)\)\
\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(g[b_, 
        h_, \[Mu]_, \[Nu]_] := \(2\ Exp[b\ \((\[Nu] + 1)\)]\)\/\(\[Mu]\^2 - \
\((\[Nu] + 1)\)\^2\) - \(2\ h\ Exp[b\ \[Nu]]\)\/\(\[Mu]\^2 - \[Nu]\^2\) + \(h\
\^\(\[Mu] + \[Nu] + 1\)\ Exp[\(-\[Mu]\)\ b]\)\/\(\[Mu]\ \((\[Mu] + \[Nu])\)\ \
\((\[Mu] + \[Nu] + 1)\)\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Then the Laplace transform ",
  Cell[BoxData[
      \(TraditionalForm\`\[CapitalPhi]\)]],
  " is given by "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, \[Theta]_] := 
      Module[{A = a[S, L], B = b[S, U], H = h[S, K], 
          n = \[Nu][\[Sigma], r, q], m}, 
        m = \[Mu][
            n, \[Theta]]; \ \(g[B, H, m, n]\ Sinh[m\ A] + f[A, H, m, n]\ \
Sinh[m\ B]\)\/Sinh[m\ \((A + B)\)]]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Just to check that all this is working, and to see the type of function we \
are dealing with, we plot the absolute value of the transform in complex ",
  Cell[BoxData[
      \(TraditionalForm\`\[Theta]\)]],
  "-space:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      Abs[\[CapitalPhi][20, 15, 10, 30, 0.2, 0.1, 0.5, 
          s\  + \ I\ t]], \ {s, \ \(-200\), \ 0}, \ {t, \ \(-10\), \ 10}, \ 
      PlotRange\  -> \ All, \ PlotPoints\  -> \ 80]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
1.66533e-16 1.04977 -0.0679587 1.04977 [
[.02757 .24618 -24 -8.70276 ]
[.02757 .24618 0 .29724 ]
[.17646 .18758 -23.5978 -9 ]
[.17646 .18758 .40224 0 ]
[.3345 .12542 -22.3468 -9 ]
[.3345 .12542 1.65319 0 ]
[.50253 .05935 -15.8219 -9 ]
[.50253 .05935 2.17811 0 ]
[.68156 -0.01098 -4.96123 -9 ]
[.68156 -0.01098 1.03877 0 ]
[.70096 -0.00478 0 -6.26206 ]
[.70096 -0.00478 18 2.73794 ]
[.78313 .12104 0 -6.13858 ]
[.78313 .12104 12 2.86142 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9224 .33431 0 -5.93715 ]
[.9224 .33431 6 3.06285 ]
[.98191 .42546 0 -5.85393 ]
[.98191 .42546 12 3.14607 ]
[.02411 .2651 -6 -2.74231 ]
[.02411 .2651 0 6.25769 ]
[.01101 .34724 -18 -2.81183 ]
[.01101 .34724 0 6.18817 ]
[-0.00281 .43388 -18 -2.88554 ]
[-0.00281 .43388 0 6.11446 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(-200)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(-150)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(-100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(-50)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(0)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.68874 0 m
.68263 .00239 L
s
[(-10)] .70096 -0.00478 -1 .39157 Mshowa
.7708 .12553 m
.76464 .12778 L
s
[(-5)] .78313 .12104 -1 .36413 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.9099 .3383 m
.90365 .34029 L
s
[(5)] .9224 .33431 -1 .31937 Mshowa
.96935 .42924 m
.96306 .43113 L
s
[(10)] .98191 .42546 -1 .30087 Mshowa
.125 Mabswid
.70593 .0263 m
.70226 .02771 L
s
.72272 .05198 m
.71904 .05338 L
s
.73912 .07706 m
.73543 .07845 L
s
.75514 .10158 m
.75145 .10294 L
s
.78611 .14895 m
.78241 .15028 L
s
.80108 .17185 m
.79737 .17317 L
s
.81573 .19425 m
.81201 .19555 L
s
.83006 .21617 m
.82633 .21745 L
s
.8578 .2586 m
.85407 .25986 L
s
.87123 .27915 m
.8675 .28039 L
s
.88439 .29928 m
.88065 .3005 L
s
.89727 .31899 m
.89353 .3202 L
s
.92227 .35722 m
.91851 .3584 L
s
.93439 .37576 m
.93063 .37693 L
s
.94627 .39394 m
.94251 .3951 L
s
.95792 .41176 m
.95416 .41291 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25794 L
s
[(0)] .02411 .2651 1 -0.3906 Mshowa
.02329 .34263 m
.02944 .34033 L
s
[(0.1)] .01101 .34724 1 -0.37515 Mshowa
.00954 .42945 m
.01571 .42723 L
s
[(0.2)] -0.00281 .43388 1 -0.35877 Mshowa
.125 Mabswid
.03378 .27645 m
.03745 .27502 L
s
.0312 .29273 m
.03488 .29132 L
s
.02859 .30919 m
.03227 .30779 L
s
.02596 .32582 m
.02964 .32443 L
s
.0206 .35962 m
.02429 .35825 L
s
.01788 .3768 m
.02157 .37544 L
s
.01513 .39416 m
.01883 .39281 L
s
.01235 .41171 m
.01605 .41037 L
s
.00669 .44739 m
.0104 .44607 L
s
.00382 .46553 m
.00753 .46422 L
s
.00091 .48387 m
.00463 .48257 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.653 .739 .913 r
.40126 .60028 .40512 .60387 .41159 .6021 .40774 .59851 Metetra
.655 .74 .912 r
.40774 .59851 .41159 .6021 .41808 .60029 .41424 .59669 Metetra
.657 .74 .911 r
.41424 .59669 .41808 .60029 .42459 .59844 .42076 .59484 Metetra
.658 .741 .911 r
.42076 .59484 .42459 .59844 .43111 .59657 .42729 .59296 Metetra
.659 .741 .91 r
.42729 .59296 .43111 .59657 .43766 .59468 .43385 .59107 Metetra
.659 .741 .91 r
.43385 .59107 .43766 .59468 .44421 .59279 .44041 .58916 Metetra
.659 .741 .91 r
.44041 .58916 .44421 .59279 .45079 .59089 .447 .58725 Metetra
.659 .741 .911 r
.447 .58725 .45079 .59089 .45738 .58899 .4536 .58534 Metetra
.659 .741 .911 r
.4536 .58534 .45738 .58899 .46398 .58709 .46021 .58344 Metetra
.658 .741 .911 r
.46021 .58344 .46398 .58709 .4706 .5852 .46685 .58155 Metetra
.657 .741 .912 r
.46685 .58155 .4706 .5852 .47724 .58332 .47349 .57966 Metetra
.656 .741 .912 r
.47349 .57966 .47724 .58332 .4839 .58146 .48016 .5778 Metetra
.654 .741 .913 r
.48016 .5778 .4839 .58146 .49057 .57962 .48684 .57595 Metetra
.653 .741 .914 r
.48684 .57595 .49057 .57962 .49726 .57778 .49354 .57411 Metetra
.654 .741 .914 r
.49354 .57411 .49726 .57778 .50396 .57593 .50026 .57227 Metetra
.657 .743 .913 r
.50026 .57227 .50396 .57593 .51069 .57401 .50699 .57036 Metetra
.663 .745 .911 r
.50699 .57036 .51069 .57401 .51743 .57199 .51374 .56833 Metetra
.67 .747 .908 r
.51374 .56833 .51743 .57199 .52418 .56984 .52051 .56616 Metetra
.675 .748 .906 r
.52051 .56616 .52418 .56984 .53095 .56762 .52729 .56391 Metetra
.676 .748 .905 r
.52729 .56391 .53095 .56762 .53774 .56537 .53409 .56164 Metetra
.675 .747 .905 r
.53409 .56164 .53774 .56537 .54455 .56312 .54091 .55938 Metetra
.673 .746 .905 r
.54091 .55938 .54455 .56312 .55137 .56091 .54774 .55714 Metetra
.671 .745 .906 r
.54774 .55714 .55137 .56091 .55821 .55872 .55459 .55494 Metetra
.67 .744 .906 r
.55459 .55494 .55821 .55872 .56506 .55655 .56145 .55275 Metetra
.669 .744 .907 r
.56145 .55275 .56506 .55655 .57193 .5544 .56833 .55059 Metetra
.667 .743 .907 r
.56833 .55059 .57193 .5544 .57882 .55226 .57523 .54844 Metetra
.666 .743 .907 r
.57523 .54844 .57882 .55226 .58572 .55013 .58215 .54631 Metetra
.666 .742 .908 r
.58215 .54631 .58572 .55013 .59264 .54802 .58908 .54418 Metetra
.665 .742 .908 r
.58908 .54418 .59264 .54802 .59958 .54591 .59603 .54206 Metetra
.665 .742 .908 r
.59603 .54206 .59958 .54591 .60654 .54379 .603 .53994 Metetra
.665 .743 .908 r
.603 .53994 .60654 .54379 .61351 .54167 .60999 .5378 Metetra
.667 .743 .907 r
.60999 .5378 .61351 .54167 .6205 .53951 .61699 .53564 Metetra
.668 .743 .907 r
.61699 .53564 .6205 .53951 .62751 .53732 .62401 .53344 Metetra
.669 .744 .906 r
.62401 .53344 .62751 .53732 .63453 .53512 .63104 .53122 Metetra
.669 .744 .906 r
.63104 .53122 .63453 .53512 .64157 .5329 .6381 .52899 Metetra
.669 .743 .906 r
.6381 .52899 .64157 .5329 .64863 .53067 .64517 .52676 Metetra
.658 .74 .911 r
.64517 .52676 .64863 .53067 .65572 .52866 .65226 .52473 Metetra
.655 .739 .911 r
.65226 .52473 .65572 .52866 .66282 .52669 .65938 .52275 Metetra
.653 .738 .912 r
.65938 .52275 .66282 .52669 .66994 .52475 .66651 .52081 Metetra
.65 .738 .913 r
.66651 .52081 .66994 .52475 .67709 .52284 .67367 .51891 Metetra
.648 .737 .915 r
.67367 .51891 .67709 .52284 .68425 .52097 .68085 .51704 Metetra
.646 .738 .916 r
.68085 .51704 .68425 .52097 .69144 .51911 .68805 .51519 Metetra
.649 .74 .916 r
.68805 .51519 .69144 .51911 .69864 .51721 .69527 .5133 Metetra
.654 .742 .915 r
.69527 .5133 .69864 .51721 .70586 .51523 .7025 .51131 Metetra
.658 .744 .913 r
.7025 .51131 .70586 .51523 .71309 .51315 .70974 .50922 Metetra
.662 .745 .912 r
.70974 .50922 .71309 .51315 .72034 .51102 .717 .50707 Metetra
.663 .745 .911 r
.717 .50707 .72034 .51102 .72761 .50885 .72428 .50489 Metetra
.664 .746 .911 r
.72428 .50489 .72761 .50885 .73489 .50665 .73158 .50268 Metetra
.664 .746 .911 r
.73158 .50268 .73489 .50665 .74219 .50445 .73889 .50046 Metetra
.664 .745 .911 r
.73889 .50046 .74219 .50445 .74951 .50225 .74623 .49825 Metetra
.66 .743 .912 r
.74623 .49825 .74951 .50225 .75686 .50011 .75358 .4961 Metetra
.654 .741 .914 r
.75358 .4961 .75686 .50011 .76423 .49807 .76097 .49406 Metetra
.65 .741 .916 r
.76097 .49406 .76423 .49807 .77162 .49608 .76838 .4921 Metetra
.652 .743 .917 r
.76838 .4921 .77162 .49608 .77903 .49405 .7758 .49008 Metetra
.661 .748 .915 r
.7758 .49008 .77903 .49405 .78645 .49184 .78324 .48788 Metetra
.676 .754 .91 r
.78324 .48788 .78645 .49184 .79387 .48938 .79067 .4854 Metetra
.687 .758 .905 r
.79067 .4854 .79387 .48938 .80129 .48671 .7981 .48268 Metetra
.69 .757 .903 r
.7981 .48268 .80129 .48671 .80872 .48397 .80554 .47989 Metetra
.686 .753 .903 r
.80554 .47989 .80872 .48397 .81617 .4813 .813 .47718 Metetra
.679 .749 .904 r
.813 .47718 .81617 .4813 .82365 .47875 .8205 .4746 Metetra
.674 .746 .905 r
.8205 .4746 .82365 .47875 .83116 .47629 .82802 .47212 Metetra
.67 .744 .906 r
.82802 .47212 .83116 .47629 .8387 .4739 .83557 .46971 Metetra
.668 .743 .907 r
.83557 .46971 .8387 .4739 .84626 .47155 .84314 .46735 Metetra
.666 .743 .907 r
.84314 .46735 .84626 .47155 .85384 .46922 .85074 .46501 Metetra
.665 .742 .907 r
.85074 .46501 .85384 .46922 .86144 .46691 .85835 .46268 Metetra
.665 .742 .908 r
.85835 .46268 .86144 .46691 .86906 .46459 .86599 .46036 Metetra
.664 .742 .908 r
.86599 .46036 .86906 .46459 .87671 .46228 .87365 .45804 Metetra
.664 .742 .908 r
.87365 .45804 .87671 .46228 .88437 .45997 .88133 .45571 Metetra
.664 .741 .908 r
.88133 .45571 .88437 .45997 .89205 .45766 .88903 .45339 Metetra
.664 .741 .908 r
.88903 .45339 .89205 .45766 .89976 .45534 .89675 .45106 Metetra
.664 .741 .908 r
.89675 .45106 .89976 .45534 .90749 .45302 .90449 .44872 Metetra
.664 .741 .908 r
.90449 .44872 .90749 .45302 .91523 .45069 .91225 .44639 Metetra
.664 .741 .908 r
.91225 .44639 .91523 .45069 .923 .44836 .92003 .44404 Metetra
.664 .741 .908 r
.92003 .44404 .923 .44836 .93078 .44602 .92783 .44169 Metetra
.664 .741 .908 r
.92783 .44169 .93078 .44602 .93859 .44368 .93566 .43934 Metetra
.664 .741 .908 r
.93566 .43934 .93859 .44368 .94642 .44133 .9435 .43697 Metetra
.664 .741 .908 r
.9435 .43697 .94642 .44133 .95427 .43897 .95136 .43461 Metetra
.664 .741 .908 r
.95136 .43461 .95427 .43897 .96214 .43661 .95925 .43223 Metetra
.664 .741 .908 r
.95925 .43223 .96214 .43661 .97003 .43424 .96716 .42985 Metetra
.653 .739 .913 r
.39738 .59668 .40126 .60028 .40774 .59851 .40388 .5949 Metetra
.655 .74 .912 r
.40388 .5949 .40774 .59851 .41424 .59669 .41039 .59308 Metetra
.657 .741 .911 r
.41039 .59308 .41424 .59669 .42076 .59484 .41691 .59122 Metetra
.658 .741 .911 r
.41691 .59122 .42076 .59484 .42729 .59296 .42346 .58933 Metetra
.659 .741 .91 r
.42346 .58933 .42729 .59296 .43385 .59107 .43002 .58743 Metetra
.659 .741 .91 r
.43002 .58743 .43385 .59107 .44041 .58916 .4366 .58552 Metetra
.659 .741 .91 r
.4366 .58552 .44041 .58916 .447 .58725 .44319 .5836 Metetra
.659 .741 .911 r
.44319 .5836 .447 .58725 .4536 .58534 .4498 .58168 Metetra
.658 .741 .911 r
.4498 .58168 .4536 .58534 .46021 .58344 .45643 .57977 Metetra
.658 .741 .911 r
.45643 .57977 .46021 .58344 .46685 .58155 .46307 .57787 Metetra
.657 .741 .912 r
.46307 .57787 .46685 .58155 .47349 .57966 .46973 .57599 Metetra
.655 .741 .913 r
.46973 .57599 .47349 .57966 .48016 .5778 .4764 .57412 Metetra
.654 .741 .913 r
.4764 .57412 .48016 .5778 .48684 .57595 .4831 .57227 Metetra
.653 .741 .914 r
.4831 .57227 .48684 .57595 .49354 .57411 .4898 .57044 Metetra
.653 .741 .915 r
.4898 .57044 .49354 .57411 .50026 .57227 .49653 .56859 Metetra
.656 .743 .914 r
.49653 .56859 .50026 .57227 .50699 .57036 .50328 .56668 Metetra
.663 .746 .911 r
.50328 .56668 .50699 .57036 .51374 .56833 .51004 .56465 Metetra
.671 .748 .908 r
.51004 .56465 .51374 .56833 .52051 .56616 .51682 .56246 Metetra
.676 .749 .906 r
.51682 .56246 .52051 .56616 .52729 .56391 .52361 .56019 Metetra
.676 .748 .905 r
.52361 .56019 .52729 .56391 .53409 .56164 .53042 .55789 Metetra
.675 .747 .905 r
.53042 .55789 .53409 .56164 .54091 .55938 .53725 .55561 Metetra
.673 .746 .905 r
.53725 .55561 .54091 .55938 .54774 .55714 .54409 .55336 Metetra
.672 .745 .906 r
.54409 .55336 .54774 .55714 .55459 .55494 .55095 .55114 Metetra
.67 .744 .906 r
.55095 .55114 .55459 .55494 .56145 .55275 .55783 .54894 Metetra
.669 .744 .906 r
.55783 .54894 .56145 .55275 .56833 .55059 .56472 .54677 Metetra
.668 .743 .907 r
.56472 .54677 .56833 .55059 .57523 .54844 .57163 .54461 Metetra
.666 .743 .907 r
.57163 .54461 .57523 .54844 .58215 .54631 .57856 .54246 Metetra
.666 .742 .908 r
.57856 .54246 .58215 .54631 .58908 .54418 .58551 .54033 Metetra
.665 .742 .908 r
.58551 .54033 .58908 .54418 .59603 .54206 .59247 .5382 Metetra
.665 .742 .908 r
.59247 .5382 .59603 .54206 .603 .53994 .59945 .53607 Metetra
.665 .743 .908 r
.59945 .53607 .603 .53994 .60999 .5378 .60645 .53392 Metetra
.667 .743 .907 r
.60645 .53392 .60999 .5378 .61699 .53564 .61346 .53175 Metetra
.668 .743 .907 r
.61346 .53175 .61699 .53564 .62401 .53344 .62049 .52954 Metetra
.669 .744 .906 r
.62049 .52954 .62401 .53344 .63104 .53122 .62754 .52731 Metetra
.669 .744 .906 r
.62754 .52731 .63104 .53122 .6381 .52899 .6346 .52506 Metetra
.668 .743 .906 r
.6346 .52506 .6381 .52899 .64517 .52676 .64168 .52283 Metetra
.658 .74 .91 r
.64168 .52283 .64517 .52676 .65226 .52473 .64879 .52079 Metetra
.655 .739 .912 r
.64879 .52079 .65226 .52473 .65938 .52275 .65592 .5188 Metetra
.652 .738 .912 r
.65592 .5188 .65938 .52275 .66651 .52081 .66307 .51685 Metetra
.649 .738 .914 r
.66307 .51685 .66651 .52081 .67367 .51891 .67024 .51495 Metetra
.647 .737 .915 r
.67024 .51495 .67367 .51891 .68085 .51704 .67743 .5131 Metetra
.645 .738 .916 r
.67743 .5131 .68085 .51704 .68805 .51519 .68464 .51126 Metetra
.648 .74 .916 r
.68464 .51126 .68805 .51519 .69527 .5133 .69187 .50937 Metetra
.653 .742 .915 r
.69187 .50937 .69527 .5133 .7025 .51131 .69912 .50737 Metetra
.659 .744 .913 r
.69912 .50737 .7025 .51131 .70974 .50922 .70638 .50528 Metetra
.662 .745 .912 r
.70638 .50528 .70974 .50922 .717 .50707 .71365 .50311 Metetra
.663 .746 .911 r
.71365 .50311 .717 .50707 .72428 .50489 .72094 .50091 Metetra
.664 .746 .911 r
.72094 .50091 .72428 .50489 .73158 .50268 .72825 .49869 Metetra
.665 .746 .911 r
.72825 .49869 .73158 .50268 .73889 .50046 .73558 .49645 Metetra
.664 .745 .911 r
.73558 .49645 .73889 .50046 .74623 .49825 .74292 .49423 Metetra
.66 .743 .912 r
.74292 .49423 .74623 .49825 .75358 .4961 .75029 .49207 Metetra
.653 .741 .914 r
.75029 .49207 .75358 .4961 .76097 .49406 .75769 .49004 Metetra
.648 .74 .917 r
.75769 .49004 .76097 .49406 .76838 .4921 .76512 .4881 Metetra
.65 .743 .917 r
.76512 .4881 .76838 .4921 .7758 .49008 .77256 .48611 Metetra
.66 .748 .915 r
.77256 .48611 .7758 .49008 .78324 .48788 .78001 .48392 Metetra
.676 .755 .911 r
.78001 .48392 .78324 .48788 .79067 .4854 .78745 .48141 Metetra
.689 .759 .905 r
.78745 .48141 .79067 .4854 .7981 .48268 .79489 .47864 Metetra
.692 .758 .902 r
.79489 .47864 .7981 .48268 .80554 .47989 .80235 .47579 Metetra
.687 .754 .902 r
.80235 .47579 .80554 .47989 .813 .47718 .80982 .47304 Metetra
.68 .75 .904 r
.80982 .47304 .813 .47718 .8205 .4746 .81733 .47043 Metetra
.674 .746 .905 r
.81733 .47043 .8205 .4746 .82802 .47212 .82487 .46793 Metetra
.67 .744 .906 r
.82487 .46793 .82802 .47212 .83557 .46971 .83243 .4655 Metetra
.668 .743 .907 r
.83243 .4655 .83557 .46971 .84314 .46735 .84002 .46313 Metetra
.666 .743 .907 r
.84002 .46313 .84314 .46735 .85074 .46501 .84763 .46077 Metetra
.665 .742 .907 r
.84763 .46077 .85074 .46501 .85835 .46268 .85526 .45843 Metetra
.665 .742 .908 r
.85526 .45843 .85835 .46268 .86599 .46036 .86291 .4561 Metetra
.664 .742 .908 r
.86291 .4561 .86599 .46036 .87365 .45804 .87058 .45376 Metetra
.664 .742 .908 r
.87058 .45376 .87365 .45804 .88133 .45571 .87827 .45143 Metetra
.664 .741 .908 r
.87827 .45143 .88133 .45571 .88903 .45339 .88599 .44909 Metetra
.664 .741 .908 r
.88599 .44909 .88903 .45339 .89675 .45106 .89372 .44675 Metetra
.664 .741 .908 r
.89372 .44675 .89675 .45106 .90449 .44872 .90148 .44441 Metetra
.664 .741 .908 r
.90148 .44441 .90449 .44872 .91225 .44639 .90925 .44206 Metetra
.664 .741 .908 r
.90925 .44206 .91225 .44639 .92003 .44404 .91705 .4397 Metetra
.664 .741 .908 r
.91705 .4397 .92003 .44404 .92783 .44169 .92487 .43734 Metetra
.664 .741 .908 r
.92487 .43734 .92783 .44169 .93566 .43934 .93271 .43497 Metetra
.664 .741 .908 r
.93271 .43497 .93566 .43934 .9435 .43697 .94057 .4326 Metetra
.664 .741 .908 r
.94057 .4326 .9435 .43697 .95136 .43461 .94845 .43022 Metetra
.664 .741 .908 r
.94845 .43022 .95136 .43461 .95925 .43223 .95635 .42784 Metetra
.664 .741 .908 r
.95635 .42784 .95925 .43223 .96716 .42985 .96427 .42545 Metetra
.653 .739 .913 r
.39349 .59307 .39738 .59668 .40388 .5949 .39999 .59129 Metetra
.655 .74 .912 r
.39999 .59129 .40388 .5949 .41039 .59308 .40651 .58946 Metetra
.657 .741 .911 r
.40651 .58946 .41039 .59308 .41691 .59122 .41305 .58759 Metetra
.658 .741 .911 r
.41305 .58759 .41691 .59122 .42346 .58933 .4196 .58569 Metetra
.659 .741 .911 r
.4196 .58569 .42346 .58933 .43002 .58743 .42617 .58378 Metetra
.659 .741 .91 r
.42617 .58378 .43002 .58743 .4366 .58552 .43276 .58186 Metetra
.659 .741 .91 r
.43276 .58186 .4366 .58552 .44319 .5836 .43937 .57993 Metetra
.659 .741 .911 r
.43937 .57993 .44319 .5836 .4498 .58168 .44599 .57801 Metetra
.658 .741 .911 r
.44599 .57801 .4498 .58168 .45643 .57977 .45262 .57609 Metetra
.657 .741 .911 r
.45262 .57609 .45643 .57977 .46307 .57787 .45928 .57418 Metetra
.656 .741 .912 r
.45928 .57418 .46307 .57787 .46973 .57599 .46595 .57229 Metetra
.655 .741 .913 r
.46595 .57229 .46973 .57599 .4764 .57412 .47263 .57042 Metetra
.653 .74 .914 r
.47263 .57042 .4764 .57412 .4831 .57227 .47933 .56857 Metetra
.652 .74 .915 r
.47933 .56857 .4831 .57227 .4898 .57044 .48605 .56674 Metetra
.652 .741 .915 r
.48605 .56674 .4898 .57044 .49653 .56859 .49279 .5649 Metetra
.655 .743 .914 r
.49279 .5649 .49653 .56859 .50328 .56668 .49954 .563 Metetra
.663 .746 .912 r
.49954 .563 .50328 .56668 .51004 .56465 .50632 .56096 Metetra
.671 .748 .908 r
.50632 .56096 .51004 .56465 .51682 .56246 .51311 .55875 Metetra
.676 .749 .906 r
.51311 .55875 .51682 .56246 .52361 .56019 .51991 .55645 Metetra
.677 .748 .905 r
.51991 .55645 .52361 .56019 .53042 .55789 .52674 .55413 Metetra
.676 .747 .905 r
.52674 .55413 .53042 .55789 .53725 .55561 .53357 .55183 Metetra
.674 .746 .905 r
.53357 .55183 .53725 .55561 .54409 .55336 .54043 .54956 Metetra
.672 .745 .906 r
.54043 .54956 .54409 .55336 .55095 .55114 .5473 .54732 Metetra
.67 .744 .906 r
.5473 .54732 .55095 .55114 .55783 .54894 .55419 .54511 Metetra
.669 .744 .906 r
.55419 .54511 .55783 .54894 .56472 .54677 .56109 .54293 Metetra
.668 .743 .907 r
.56109 .54293 .56472 .54677 .57163 .54461 .56801 .54075 Metetra
.667 .743 .907 r
.56801 .54075 .57163 .54461 .57856 .54246 .57495 .5386 Metetra
.666 .742 .908 r
.57495 .5386 .57856 .54246 .58551 .54033 .58191 .53645 Metetra
.665 .742 .908 r
.58191 .53645 .58551 .54033 .59247 .5382 .58889 .53432 Metetra
.665 .742 .908 r
.58889 .53432 .59247 .5382 .59945 .53607 .59588 .53218 Metetra
.665 .743 .908 r
.59588 .53218 .59945 .53607 .60645 .53392 .60289 .53003 Metetra
.667 .743 .907 r
.60289 .53003 .60645 .53392 .61346 .53175 .60991 .52784 Metetra
.668 .744 .907 r
.60991 .52784 .61346 .53175 .62049 .52954 .61695 .52562 Metetra
.669 .744 .906 r
.61695 .52562 .62049 .52954 .62754 .52731 .62401 .52337 Metetra
.669 .744 .906 r
.62401 .52337 .62754 .52731 .6346 .52506 .63109 .52112 Metetra
.668 .743 .906 r
.63109 .52112 .6346 .52506 .64168 .52283 .63819 .51889 Metetra
.659 .74 .91 r
.63819 .51889 .64168 .52283 .64879 .52079 .64531 .51682 Metetra
.655 .739 .912 r
.64531 .51682 .64879 .52079 .65592 .5188 .65245 .51483 Metetra
.652 .738 .913 r
.65245 .51483 .65592 .5188 .66307 .51685 .65961 .51288 Metetra
.649 .737 .914 r
.65961 .51288 .66307 .51685 .67024 .51495 .66679 .51098 Metetra
.646 .737 .916 r
.66679 .51098 .67024 .51495 .67743 .5131 .67399 .50914 Metetra
.644 .737 .917 r
.67399 .50914 .67743 .5131 .68464 .51126 .68122 .50731 Metetra
.647 .739 .917 r
.68122 .50731 .68464 .51126 .69187 .50937 .68846 .50543 Metetra
.653 .743 .915 r
.68846 .50543 .69187 .50937 .69912 .50737 .69572 .50343 Metetra
.659 .745 .914 r
.69572 .50343 .69912 .50737 .70638 .50528 .70299 .50132 Metetra
.662 .746 .912 r
.70299 .50132 .70638 .50528 .71365 .50311 .71028 .49913 Metetra
.664 .746 .912 r
.71028 .49913 .71365 .50311 .72094 .50091 .71758 .49692 Metetra
.664 .746 .911 r
.71758 .49692 .72094 .50091 .72825 .49869 .7249 .49468 Metetra
.665 .746 .911 r
.7249 .49468 .72825 .49869 .73558 .49645 .73224 .49243 Metetra
.664 .745 .911 r
.73224 .49243 .73558 .49645 .74292 .49423 .7396 .49019 Metetra
.66 .744 .912 r
.7396 .49019 .74292 .49423 .75029 .49207 .74699 .48802 Metetra
.652 .741 .915 r
.74699 .48802 .75029 .49207 .75769 .49004 .7544 .48601 Metetra
.646 .74 .917 r
.7544 .48601 .75769 .49004 .76512 .4881 .76184 .48409 Metetra
.648 .742 .918 r
.76184 .48409 .76512 .4881 .77256 .48611 .7693 .48213 Metetra
.659 .749 .916 r
.7693 .48213 .77256 .48611 .78001 .48392 .77677 .47995 Metetra
.677 .756 .911 r
.77677 .47995 .78001 .48392 .78745 .48141 .78422 .47742 Metetra
.691 .76 .905 r
.78422 .47742 .78745 .48141 .79489 .47864 .79168 .47459 Metetra
.694 .759 .902 r
.79168 .47459 .79489 .47864 .80235 .47579 .79914 .47168 Metetra
.689 .755 .902 r
.79914 .47168 .80235 .47579 .80982 .47304 .80663 .46888 Metetra
.681 .75 .903 r
.80663 .46888 .80982 .47304 .81733 .47043 .81415 .46623 Metetra
.675 .747 .905 r
.81415 .46623 .81733 .47043 .82487 .46793 .8217 .46371 Metetra
.671 .745 .906 r
.8217 .46371 .82487 .46793 .83243 .4655 .82927 .46127 Metetra
.668 .743 .907 r
.82927 .46127 .83243 .4655 .84002 .46313 .83687 .45888 Metetra
.666 .743 .907 r
.83687 .45888 .84002 .46313 .84763 .46077 .8445 .45652 Metetra
.665 .742 .907 r
.8445 .45652 .84763 .46077 .85526 .45843 .85214 .45416 Metetra
.665 .742 .908 r
.85214 .45416 .85526 .45843 .86291 .4561 .85981 .45182 Metetra
.664 .742 .908 r
.85981 .45182 .86291 .4561 .87058 .45376 .8675 .44947 Metetra
.664 .742 .908 r
.8675 .44947 .87058 .45376 .87827 .45143 .8752 .44713 Metetra
.664 .741 .908 r
.8752 .44713 .87827 .45143 .88599 .44909 .88293 .44478 Metetra
.664 .741 .908 r
.88293 .44478 .88599 .44909 .89372 .44675 .89068 .44243 Metetra
.664 .741 .908 r
.89068 .44243 .89372 .44675 .90148 .44441 .89845 .44007 Metetra
.664 .741 .908 r
.89845 .44007 .90148 .44441 .90925 .44206 .90624 .43771 Metetra
.664 .741 .908 r
.90624 .43771 .90925 .44206 .91705 .4397 .91406 .43534 Metetra
.664 .741 .908 r
.91406 .43534 .91705 .4397 .92487 .43734 .92189 .43297 Metetra
.664 .741 .908 r
.92189 .43297 .92487 .43734 .93271 .43497 .92974 .43059 Metetra
.664 .741 .908 r
.92974 .43059 .93271 .43497 .94057 .4326 .93762 .42821 Metetra
.664 .741 .908 r
.93762 .42821 .94057 .4326 .94845 .43022 .94551 .42582 Metetra
.664 .741 .908 r
.94551 .42582 .94845 .43022 .95635 .42784 .95343 .42342 Metetra
.664 .741 .908 r
.95343 .42342 .95635 .42784 .96427 .42545 .96137 .42101 Metetra
.653 .739 .913 r
.38958 .58945 .39349 .59307 .39999 .59129 .39609 .58765 Metetra
.655 .74 .912 r
.39609 .58765 .39999 .59129 .40651 .58946 .40262 .58582 Metetra
.657 .741 .911 r
.40262 .58582 .40651 .58946 .41305 .58759 .40917 .58394 Metetra
.658 .741 .911 r
.40917 .58394 .41305 .58759 .4196 .58569 .41573 .58203 Metetra
.659 .741 .911 r
.41573 .58203 .4196 .58569 .42617 .58378 .42231 .58011 Metetra
.659 .741 .91 r
.42231 .58011 .42617 .58378 .43276 .58186 .42891 .57818 Metetra
.659 .741 .911 r
.42891 .57818 .43276 .58186 .43937 .57993 .43553 .57625 Metetra
.659 .741 .911 r
.43553 .57625 .43937 .57993 .44599 .57801 .44216 .57432 Metetra
.658 .741 .911 r
.44216 .57432 .44599 .57801 .45262 .57609 .4488 .57239 Metetra
.657 .741 .911 r
.4488 .57239 .45262 .57609 .45928 .57418 .45547 .57048 Metetra
.656 .741 .912 r
.45547 .57048 .45928 .57418 .46595 .57229 .46215 .56858 Metetra
.654 .74 .913 r
.46215 .56858 .46595 .57229 .47263 .57042 .46884 .56671 Metetra
.653 .74 .914 r
.46884 .56671 .47263 .57042 .47933 .56857 .47555 .56486 Metetra
.651 .74 .915 r
.47555 .56486 .47933 .56857 .48605 .56674 .48228 .56303 Metetra
.651 .741 .915 r
.48228 .56303 .48605 .56674 .49279 .5649 .48903 .5612 Metetra
.655 .743 .915 r
.48903 .5612 .49279 .5649 .49954 .563 .4958 .5593 Metetra
.663 .746 .912 r
.4958 .5593 .49954 .563 .50632 .56096 .50258 .55725 Metetra
.672 .749 .908 r
.50258 .55725 .50632 .56096 .51311 .55875 .50938 .55503 Metetra
.677 .749 .905 r
.50938 .55503 .51311 .55875 .51991 .55645 .5162 .5527 Metetra
.678 .749 .904 r
.5162 .5527 .51991 .55645 .52674 .55413 .52303 .55035 Metetra
.676 .747 .904 r
.52303 .55035 .52674 .55413 .53357 .55183 .52988 .54803 Metetra
.674 .746 .905 r
.52988 .54803 .53357 .55183 .54043 .54956 .53675 .54574 Metetra
.672 .745 .905 r
.53675 .54574 .54043 .54956 .5473 .54732 .54363 .54349 Metetra
.67 .744 .906 r
.54363 .54349 .5473 .54732 .55419 .54511 .55053 .54127 Metetra
.669 .744 .906 r
.55053 .54127 .55419 .54511 .56109 .54293 .55745 .53907 Metetra
.668 .743 .907 r
.55745 .53907 .56109 .54293 .56801 .54075 .56438 .53688 Metetra
.667 .743 .907 r
.56438 .53688 .56801 .54075 .57495 .5386 .57133 .53472 Metetra
.666 .742 .908 r
.57133 .53472 .57495 .5386 .58191 .53645 .5783 .53256 Metetra
.665 .742 .908 r
.5783 .53256 .58191 .53645 .58889 .53432 .58529 .53042 Metetra
.664 .742 .908 r
.58529 .53042 .58889 .53432 .59588 .53218 .59229 .52828 Metetra
.665 .743 .908 r
.59229 .52828 .59588 .53218 .60289 .53003 .59931 .52611 Metetra
.667 .743 .907 r
.59931 .52611 .60289 .53003 .60991 .52784 .60635 .52392 Metetra
.668 .744 .907 r
.60635 .52392 .60991 .52784 .61695 .52562 .6134 .52168 Metetra
.669 .744 .906 r
.6134 .52168 .61695 .52562 .62401 .52337 .62048 .51943 Metetra
.669 .744 .906 r
.62048 .51943 .62401 .52337 .63109 .52112 .62756 .51716 Metetra
.668 .743 .907 r
.62756 .51716 .63109 .52112 .63819 .51889 .63467 .51492 Metetra
.659 .741 .91 r
.63467 .51492 .63819 .51889 .64531 .51682 .6418 .51285 Metetra
.654 .739 .912 r
.6418 .51285 .64531 .51682 .65245 .51483 .64896 .51085 Metetra
.652 .738 .913 r
.64896 .51085 .65245 .51483 .65961 .51288 .65613 .5089 Metetra
.648 .737 .914 r
.65613 .5089 .65961 .51288 .66679 .51098 .66332 .507 Metetra
.644 .737 .916 r
.66332 .507 .66679 .51098 .67399 .50914 .67054 .50516 Metetra
.643 .737 .917 r
.67054 .50516 .67399 .50914 .68122 .50731 .67778 .50335 Metetra
.646 .739 .917 r
.67778 .50335 .68122 .50731 .68846 .50543 .68504 .50147 Metetra
.653 .743 .916 r
.68504 .50147 .68846 .50543 .69572 .50343 .69231 .49947 Metetra
.659 .745 .914 r
.69231 .49947 .69572 .50343 .70299 .50132 .69959 .49734 Metetra
.662 .746 .912 r
.69959 .49734 .70299 .50132 .71028 .49913 .70689 .49514 Metetra
.664 .746 .912 r
.70689 .49514 .71028 .49913 .71758 .49692 .71421 .49291 Metetra
.665 .746 .911 r
.71421 .49291 .71758 .49692 .7249 .49468 .72154 .49066 Metetra
.665 .746 .911 r
.72154 .49066 .7249 .49468 .73224 .49243 .7289 .48839 Metetra
.665 .746 .911 r
.7289 .48839 .73224 .49243 .7396 .49019 .73627 .48613 Metetra
.66 .744 .912 r
.73627 .48613 .7396 .49019 .74699 .48802 .74367 .48396 Metetra
.65 .74 .915 r
.74367 .48396 .74699 .48802 .7544 .48601 .7511 .48195 Metetra
.644 .739 .918 r
.7511 .48195 .7544 .48601 .76184 .48409 .75855 .48006 Metetra
.645 .742 .919 r
.75855 .48006 .76184 .48409 .7693 .48213 .76603 .47814 Metetra
.657 .749 .917 r
.76603 .47814 .7693 .48213 .77677 .47995 .77351 .47598 Metetra
.677 .757 .911 r
.77351 .47598 .77677 .47995 .78422 .47742 .78098 .47343 Metetra
.693 .762 .904 r
.78098 .47343 .78422 .47742 .79168 .47459 .78844 .47054 Metetra
.697 .761 .901 r
.78844 .47054 .79168 .47459 .79914 .47168 .79592 .46756 Metetra
.691 .756 .901 r
.79592 .46756 .79914 .47168 .80663 .46888 .80341 .4647 Metetra
.682 .751 .903 r
.80341 .4647 .80663 .46888 .81415 .46623 .81095 .46202 Metetra
.675 .747 .905 r
.81095 .46202 .81415 .46623 .8217 .46371 .81851 .45948 Metetra
.671 .745 .906 r
.81851 .45948 .8217 .46371 .82927 .46127 .8261 .45702 Metetra
.668 .743 .907 r
.8261 .45702 .82927 .46127 .83687 .45888 .83372 .45462 Metetra
.666 .743 .907 r
.83372 .45462 .83687 .45888 .8445 .45652 .84135 .45224 Metetra
.665 .742 .907 r
.84135 .45224 .8445 .45652 .85214 .45416 .84901 .44988 Metetra
.665 .742 .908 r
.84901 .44988 .85214 .45416 .85981 .45182 .85669 .44752 Metetra
.664 .742 .908 r
.85669 .44752 .85981 .45182 .8675 .44947 .8644 .44516 Metetra
.664 .742 .908 r
.8644 .44516 .8675 .44947 .8752 .44713 .87212 .4428 Metetra
.664 .741 .908 r
.87212 .4428 .8752 .44713 .88293 .44478 .87986 .44044 Metetra
.664 .741 .908 r
.87986 .44044 .88293 .44478 .89068 .44243 .88763 .43808 Metetra
.664 .741 .908 r
.88763 .43808 .89068 .44243 .89845 .44007 .89541 .43571 Metetra
.664 .741 .908 r
.89541 .43571 .89845 .44007 .90624 .43771 .90322 .43334 Metetra
.664 .741 .908 r
.90322 .43334 .90624 .43771 .91406 .43534 .91105 .43096 Metetra
.664 .741 .908 r
.91105 .43096 .91406 .43534 .92189 .43297 .91889 .42858 Metetra
.664 .741 .908 r
.91889 .42858 .92189 .43297 .92974 .43059 .92676 .42619 Metetra
.664 .741 .908 r
.92676 .42619 .92974 .43059 .93762 .42821 .93465 .42379 Metetra
.664 .741 .908 r
.93465 .42379 .93762 .42821 .94551 .42582 .94256 .42139 Metetra
.664 .741 .908 r
.94256 .42139 .94551 .42582 .95343 .42342 .95049 .41898 Metetra
.664 .741 .908 r
.95049 .41898 .95343 .42342 .96137 .42101 .95845 .41656 Metetra
.653 .74 .913 r
.38565 .58581 .38958 .58945 .39609 .58765 .39217 .58401 Metetra
.655 .74 .912 r
.39217 .58401 .39609 .58765 .40262 .58582 .39871 .58216 Metetra
.657 .741 .911 r
.39871 .58216 .40262 .58582 .40917 .58394 .40527 .58027 Metetra
.659 .741 .911 r
.40527 .58027 .40917 .58394 .41573 .58203 .41184 .57836 Metetra
.659 .741 .911 r
.41184 .57836 .41573 .58203 .42231 .58011 .41843 .57643 Metetra
.659 .741 .91 r
.41843 .57643 .42231 .58011 .42891 .57818 .42504 .57449 Metetra
.659 .741 .911 r
.42504 .57449 .42891 .57818 .43553 .57625 .43167 .57255 Metetra
.659 .741 .911 r
.43167 .57255 .43553 .57625 .44216 .57432 .43831 .57061 Metetra
.658 .741 .911 r
.43831 .57061 .44216 .57432 .4488 .57239 .44496 .56868 Metetra
.657 .741 .912 r
.44496 .56868 .4488 .57239 .45547 .57048 .45164 .56676 Metetra
.656 .741 .912 r
.45164 .56676 .45547 .57048 .46215 .56858 .45833 .56486 Metetra
.654 .74 .913 r
.45833 .56486 .46215 .56858 .46884 .56671 .46503 .56298 Metetra
.652 .74 .914 r
.46503 .56298 .46884 .56671 .47555 .56486 .47176 .56113 Metetra
.65 .74 .915 r
.47176 .56113 .47555 .56486 .48228 .56303 .4785 .55931 Metetra
.65 .741 .916 r
.4785 .55931 .48228 .56303 .48903 .5612 .48525 .55749 Metetra
.654 .743 .915 r
.48525 .55749 .48903 .5612 .4958 .5593 .49203 .5556 Metetra
.663 .746 .912 r
.49203 .5556 .4958 .5593 .50258 .55725 .49883 .55354 Metetra
.673 .749 .908 r
.49883 .55354 .50258 .55725 .50938 .55503 .50564 .55129 Metetra
.678 .75 .905 r
.50564 .55129 .50938 .55503 .5162 .5527 .51247 .54893 Metetra
.679 .749 .904 r
.51247 .54893 .5162 .5527 .52303 .55035 .51931 .54656 Metetra
.677 .748 .904 r
.51931 .54656 .52303 .55035 .52988 .54803 .52617 .54421 Metetra
.675 .746 .905 r
.52617 .54421 .52988 .54803 .53675 .54574 .53305 .54191 Metetra
.672 .745 .905 r
.53305 .54191 .53675 .54574 .54363 .54349 .53995 .53964 Metetra
.671 .744 .906 r
.53995 .53964 .54363 .54349 .55053 .54127 .54686 .5374 Metetra
.669 .744 .906 r
.54686 .5374 .55053 .54127 .55745 .53907 .55378 .53519 Metetra
.668 .743 .907 r
.55378 .53519 .55745 .53907 .56438 .53688 .56073 .533 Metetra
.667 .743 .907 r
.56073 .533 .56438 .53688 .57133 .53472 .56769 .53082 Metetra
.666 .742 .908 r
.56769 .53082 .57133 .53472 .5783 .53256 .57467 .52865 Metetra
.665 .742 .908 r
.57467 .52865 .5783 .53256 .58529 .53042 .58167 .5265 Metetra
.664 .742 .908 r
.58167 .5265 .58529 .53042 .59229 .52828 .58869 .52435 Metetra
.665 .743 .908 r
.58869 .52435 .59229 .52828 .59931 .52611 .59572 .52218 Metetra
.667 .743 .907 r
.59572 .52218 .59931 .52611 .60635 .52392 .60277 .51997 Metetra
.668 .744 .907 r
.60277 .51997 .60635 .52392 .6134 .52168 .60984 .51773 Metetra
.669 .744 .906 r
.60984 .51773 .6134 .52168 .62048 .51943 .61692 .51546 Metetra
.669 .744 .906 r
.61692 .51546 .62048 .51943 .62756 .51716 .62402 .51318 Metetra
.667 .743 .907 r
.62402 .51318 .62756 .51716 .63467 .51492 .63114 .51094 Metetra
.659 .741 .91 r
.63114 .51094 .63467 .51492 .6418 .51285 .63828 .50885 Metetra
.654 .739 .912 r
.63828 .50885 .6418 .51285 .64896 .51085 .64545 .50684 Metetra
.651 .738 .913 r
.64545 .50684 .64896 .51085 .65613 .5089 .65263 .50489 Metetra
.647 .737 .915 r
.65263 .50489 .65613 .5089 .66332 .507 .65984 .503 Metetra
.643 .736 .917 r
.65984 .503 .66332 .507 .67054 .50516 .66707 .50117 Metetra
.641 .737 .918 r
.66707 .50117 .67054 .50516 .67778 .50335 .67433 .49937 Metetra
.645 .739 .918 r
.67433 .49937 .67778 .50335 .68504 .50147 .6816 .4975 Metetra
.652 .743 .916 r
.6816 .4975 .68504 .50147 .69231 .49947 .68888 .4955 Metetra
.659 .746 .914 r
.68888 .4955 .69231 .49947 .69959 .49734 .69618 .49335 Metetra
.663 .747 .912 r
.69618 .49335 .69959 .49734 .70689 .49514 .70349 .49114 Metetra
.664 .747 .912 r
.70349 .49114 .70689 .49514 .71421 .49291 .71082 .48889 Metetra
.665 .747 .911 r
.71082 .48889 .71421 .49291 .72154 .49066 .71817 .48662 Metetra
.666 .747 .911 r
.71817 .48662 .72154 .49066 .7289 .48839 .72553 .48434 Metetra
.665 .746 .911 r
.72553 .48434 .7289 .48839 .73627 .48613 .73292 .48205 Metetra
.66 .744 .912 r
.73292 .48205 .73627 .48613 .74367 .48396 .74033 .47987 Metetra
.649 .74 .916 r
.74033 .47987 .74367 .48396 .7511 .48195 .74777 .47788 Metetra
.641 .738 .919 r
.74777 .47788 .7511 .48195 .75855 .48006 .75525 .47603 Metetra
.642 .741 .921 r
.75525 .47603 .75855 .48006 .76603 .47814 .76274 .47414 Metetra
.656 .749 .918 r
.76274 .47414 .76603 .47814 .77351 .47598 .77024 .47201 Metetra
.678 .758 .912 r
.77024 .47201 .77351 .47598 .78098 .47343 .77772 .46943 Metetra
.696 .764 .904 r
.77772 .46943 .78098 .47343 .78844 .47054 .7852 .46648 Metetra
.7 .762 .9 r
.7852 .46648 .78844 .47054 .79592 .46756 .79268 .46342 Metetra
.693 .757 .901 r
.79268 .46342 .79592 .46756 .80341 .4647 .80019 .46051 Metetra
.683 .751 .903 r
.80019 .46051 .80341 .4647 .81095 .46202 .80773 .45779 Metetra
.676 .747 .905 r
.80773 .45779 .81095 .46202 .81851 .45948 .81531 .45523 Metetra
.671 .745 .906 r
.81531 .45523 .81851 .45948 .8261 .45702 .82291 .45275 Metetra
.668 .743 .907 r
.82291 .45275 .8261 .45702 .83372 .45462 .83054 .45033 Metetra
.666 .743 .907 r
.83054 .45033 .83372 .45462 .84135 .45224 .83819 .44794 Metetra
.665 .742 .907 r
.83819 .44794 .84135 .45224 .84901 .44988 .84587 .44557 Metetra
.665 .742 .908 r
.84587 .44557 .84901 .44988 .85669 .44752 .85356 .4432 Metetra
.664 .742 .908 r
.85356 .4432 .85669 .44752 .8644 .44516 .86128 .44083 Metetra
.664 .742 .908 r
.86128 .44083 .8644 .44516 .87212 .4428 .86902 .43846 Metetra
.664 .741 .908 r
.86902 .43846 .87212 .4428 .87986 .44044 .87678 .43609 Metetra
.664 .741 .908 r
.87678 .43609 .87986 .44044 .88763 .43808 .88456 .43371 Metetra
.664 .741 .908 r
.88456 .43371 .88763 .43808 .89541 .43571 .89236 .43133 Metetra
.664 .741 .908 r
.89236 .43133 .89541 .43571 .90322 .43334 .90018 .42895 Metetra
.664 .741 .908 r
.90018 .42895 .90322 .43334 .91105 .43096 .90802 .42656 Metetra
.664 .741 .908 r
.90802 .42656 .91105 .43096 .91889 .42858 .91588 .42416 Metetra
.664 .741 .908 r
.91588 .42416 .91889 .42858 .92676 .42619 .92377 .42176 Metetra
.664 .741 .908 r
.92377 .42176 .92676 .42619 .93465 .42379 .93167 .41935 Metetra
.664 .741 .908 r
.93167 .41935 .93465 .42379 .94256 .42139 .9396 .41694 Metetra
.664 .741 .908 r
.9396 .41694 .94256 .42139 .95049 .41898 .94755 .41452 Metetra
.664 .741 .908 r
.94755 .41452 .95049 .41898 .95845 .41656 .95552 .41209 Metetra
.653 .74 .913 r
.3817 .58216 .38565 .58581 .39217 .58401 .38824 .58034 Metetra
.655 .74 .912 r
.38824 .58034 .39217 .58401 .39871 .58216 .39478 .57849 Metetra
.657 .741 .911 r
.39478 .57849 .39871 .58216 .40527 .58027 .40135 .57659 Metetra
.659 .741 .911 r
.40135 .57659 .40527 .58027 .41184 .57836 .40794 .57467 Metetra
.659 .741 .911 r
.40794 .57467 .41184 .57836 .41843 .57643 .41454 .57273 Metetra
.659 .741 .91 r
.41454 .57273 .41843 .57643 .42504 .57449 .42116 .57078 Metetra
.659 .741 .911 r
.42116 .57078 .42504 .57449 .43167 .57255 .42779 .56883 Metetra
.659 .741 .911 r
.42779 .56883 .43167 .57255 .43831 .57061 .43444 .56688 Metetra
.658 .741 .911 r
.43444 .56688 .43831 .57061 .44496 .56868 .44111 .56495 Metetra
.657 .741 .912 r
.44111 .56495 .44496 .56868 .45164 .56676 .44779 .56302 Metetra
.656 .741 .912 r
.44779 .56302 .45164 .56676 .45833 .56486 .45449 .56111 Metetra
.654 .74 .913 r
.45449 .56111 .45833 .56486 .46503 .56298 .46121 .55923 Metetra
.652 .74 .914 r
.46121 .55923 .46503 .56298 .47176 .56113 .46794 .55739 Metetra
.649 .74 .916 r
.46794 .55739 .47176 .56113 .4785 .55931 .47469 .55557 Metetra
.649 .741 .916 r
.47469 .55557 .4785 .55931 .48525 .55749 .48146 .55376 Metetra
.652 .743 .916 r
.48146 .55376 .48525 .55749 .49203 .5556 .48825 .55188 Metetra
.662 .746 .913 r
.48825 .55188 .49203 .5556 .49883 .55354 .49505 .54982 Metetra
.674 .75 .908 r
.49505 .54982 .49883 .55354 .50564 .55129 .50188 .54754 Metetra
.68 .75 .905 r
.50188 .54754 .50564 .55129 .51247 .54893 .50872 .54515 Metetra
.68 .749 .904 r
.50872 .54515 .51247 .54893 .51931 .54656 .51558 .54275 Metetra
.678 .748 .904 r
.51558 .54275 .51931 .54656 .52617 .54421 .52245 .54038 Metetra
.675 .746 .905 r
.52245 .54038 .52617 .54421 .53305 .54191 .52934 .53805 Metetra
.673 .745 .905 r
.52934 .53805 .53305 .54191 .53995 .53964 .53624 .53577 Metetra
.671 .744 .906 r
.53624 .53577 .53995 .53964 .54686 .5374 .54317 .53352 Metetra
.669 .744 .906 r
.54317 .53352 .54686 .5374 .55378 .53519 .55011 .5313 Metetra
.668 .743 .907 r
.55011 .5313 .55378 .53519 .56073 .533 .55706 .52909 Metetra
.667 .743 .907 r
.55706 .52909 .56073 .533 .56769 .53082 .56404 .5269 Metetra
.666 .742 .908 r
.56404 .5269 .56769 .53082 .57467 .52865 .57103 .52473 Metetra
.665 .742 .908 r
.57103 .52473 .57467 .52865 .58167 .5265 .57804 .52257 Metetra
.664 .742 .908 r
.57804 .52257 .58167 .5265 .58869 .52435 .58507 .52041 Metetra
.665 .743 .908 r
.58507 .52041 .58869 .52435 .59572 .52218 .59211 .51823 Metetra
.667 .743 .907 r
.59211 .51823 .59572 .52218 .60277 .51997 .59917 .51601 Metetra
.669 .744 .907 r
.59917 .51601 .60277 .51997 .60984 .51773 .60625 .51376 Metetra
.669 .744 .906 r
.60625 .51376 .60984 .51773 .61692 .51546 .61335 .51147 Metetra
.669 .744 .906 r
.61335 .51147 .61692 .51546 .62402 .51318 .62046 .50919 Metetra
.667 .743 .907 r
.62046 .50919 .62402 .51318 .63114 .51094 .62759 .50694 Metetra
.659 .741 .91 r
.62759 .50694 .63114 .51094 .63828 .50885 .63475 .50484 Metetra
.654 .739 .912 r
.63475 .50484 .63828 .50885 .64545 .50684 .64193 .50282 Metetra
.651 .738 .913 r
.64193 .50282 .64545 .50684 .65263 .50489 .64912 .50087 Metetra
.647 .737 .915 r
.64912 .50087 .65263 .50489 .65984 .503 .65635 .49898 Metetra
.642 .736 .917 r
.65635 .49898 .65984 .503 .66707 .50117 .66359 .49716 Metetra
.64 .736 .919 r
.66359 .49716 .66707 .50117 .67433 .49937 .67086 .49538 Metetra
.643 .739 .919 r
.67086 .49538 .67433 .49937 .6816 .4975 .67814 .49353 Metetra
.652 .743 .917 r
.67814 .49353 .6816 .4975 .68888 .4955 .68544 .49151 Metetra
.659 .746 .914 r
.68544 .49151 .68888 .4955 .69618 .49335 .69275 .48935 Metetra
.663 .747 .912 r
.69275 .48935 .69618 .49335 .70349 .49114 .70007 .48712 Metetra
.665 .747 .912 r
.70007 .48712 .70349 .49114 .71082 .48889 .70741 .48485 Metetra
.665 .747 .911 r
.70741 .48485 .71082 .48889 .71817 .48662 .71477 .48257 Metetra
.666 .747 .911 r
.71477 .48257 .71817 .48662 .72553 .48434 .72215 .48026 Metetra
.666 .746 .91 r
.72215 .48026 .72553 .48434 .73292 .48205 .72955 .47796 Metetra
.66 .744 .912 r
.72955 .47796 .73292 .48205 .74033 .47987 .73698 .47576 Metetra
.647 .739 .916 r
.73698 .47576 .74033 .47987 .74777 .47788 .74444 .47379 Metetra
.638 .737 .92 r
.74444 .47379 .74777 .47788 .75525 .47603 .75193 .47198 Metetra
.639 .74 .922 r
.75193 .47198 .75525 .47603 .76274 .47414 .75944 .47014 Metetra
.653 .749 .92 r
.75944 .47014 .76274 .47414 .77024 .47201 .76695 .46803 Metetra
.678 .76 .912 r
.76695 .46803 .77024 .47201 .77772 .46943 .77445 .46544 Metetra
.699 .766 .904 r
.77445 .46544 .77772 .46943 .7852 .46648 .78193 .46241 Metetra
.703 .764 .899 r
.78193 .46241 .7852 .46648 .79268 .46342 .78942 .45927 Metetra
.695 .758 .9 r
.78942 .45927 .79268 .46342 .80019 .46051 .79695 .4563 Metetra
.685 .752 .902 r
.79695 .4563 .80019 .46051 .80773 .45779 .8045 .45354 Metetra
.677 .747 .904 r
.8045 .45354 .80773 .45779 .81531 .45523 .81209 .45095 Metetra
.671 .745 .906 r
.81209 .45095 .81531 .45523 .82291 .45275 .81971 .44846 Metetra
.668 .743 .907 r
.81971 .44846 .82291 .45275 .83054 .45033 .82735 .44603 Metetra
.667 .743 .907 r
.82735 .44603 .83054 .45033 .83819 .44794 .83502 .44362 Metetra
.665 .742 .907 r
.83502 .44362 .83819 .44794 .84587 .44557 .84271 .44124 Metetra
.665 .742 .908 r
.84271 .44124 .84587 .44557 .85356 .4432 .85042 .43885 Metetra
.664 .742 .908 r
.85042 .43885 .85356 .4432 .86128 .44083 .85815 .43647 Metetra
.664 .742 .908 r
.85815 .43647 .86128 .44083 .86902 .43846 .8659 .43409 Metetra
.664 .741 .908 r
.8659 .43409 .86902 .43846 .87678 .43609 .87368 .43171 Metetra
.664 .741 .908 r
.87368 .43171 .87678 .43609 .88456 .43371 .88147 .42932 Metetra
.664 .741 .908 r
.88147 .42932 .88456 .43371 .89236 .43133 .88929 .42693 Metetra
.664 .741 .908 r
.88929 .42693 .89236 .43133 .90018 .42895 .89712 .42453 Metetra
.664 .741 .908 r
.89712 .42453 .90018 .42895 .90802 .42656 .90498 .42213 Metetra
.664 .741 .908 r
.90498 .42213 .90802 .42656 .91588 .42416 .91286 .41972 Metetra
.664 .741 .908 r
.91286 .41972 .91588 .42416 .92377 .42176 .92076 .41731 Metetra
.664 .741 .908 r
.92076 .41731 .92377 .42176 .93167 .41935 .92868 .41489 Metetra
.664 .741 .908 r
.92868 .41489 .93167 .41935 .9396 .41694 .93662 .41246 Metetra
.664 .741 .908 r
.93662 .41246 .9396 .41694 .94755 .41452 .94459 .41003 Metetra
.664 .741 .908 r
.94459 .41003 .94755 .41452 .95552 .41209 .95257 .40759 Metetra
.653 .74 .913 r
.37774 .57849 .3817 .58216 .38824 .58034 .38428 .57667 Metetra
.655 .74 .912 r
.38428 .57667 .38824 .58034 .39478 .57849 .39084 .5748 Metetra
.657 .741 .911 r
.39084 .5748 .39478 .57849 .40135 .57659 .39742 .5729 Metetra
.659 .741 .911 r
.39742 .5729 .40135 .57659 .40794 .57467 .40401 .57096 Metetra
.659 .741 .911 r
.40401 .57096 .40794 .57467 .41454 .57273 .41062 .56901 Metetra
.659 .741 .911 r
.41062 .56901 .41454 .57273 .42116 .57078 .41725 .56706 Metetra
.659 .741 .911 r
.41725 .56706 .42116 .57078 .42779 .56883 .4239 .5651 Metetra
.659 .741 .911 r
.4239 .5651 .42779 .56883 .43444 .56688 .43056 .56314 Metetra
.658 .741 .911 r
.43056 .56314 .43444 .56688 .44111 .56495 .43724 .5612 Metetra
.657 .741 .912 r
.43724 .5612 .44111 .56495 .44779 .56302 .44393 .55926 Metetra
.655 .741 .912 r
.44393 .55926 .44779 .56302 .45449 .56111 .45064 .55735 Metetra
.653 .74 .913 r
.45064 .55735 .45449 .56111 .46121 .55923 .45737 .55547 Metetra
.651 .74 .915 r
.45737 .55547 .46121 .55923 .46794 .55739 .46411 .55362 Metetra
.648 .74 .916 r
.46411 .55362 .46794 .55739 .47469 .55557 .47087 .55181 Metetra
.648 .74 .917 r
.47087 .55181 .47469 .55557 .48146 .55376 .47765 .55001 Metetra
.651 .743 .916 r
.47765 .55001 .48146 .55376 .48825 .55188 .48445 .54814 Metetra
.662 .747 .913 r
.48445 .54814 .48825 .55188 .49505 .54982 .49126 .54608 Metetra
.674 .75 .908 r
.49126 .54608 .49505 .54982 .50188 .54754 .4981 .54378 Metetra
.681 .751 .904 r
.4981 .54378 .50188 .54754 .50872 .54515 .50495 .54136 Metetra
.681 .75 .903 r
.50495 .54136 .50872 .54515 .51558 .54275 .51182 .53892 Metetra
.678 .748 .904 r
.51182 .53892 .51558 .54275 .52245 .54038 .51871 .53653 Metetra
.676 .747 .904 r
.51871 .53653 .52245 .54038 .52934 .53805 .52561 .53419 Metetra
.673 .745 .905 r
.52561 .53419 .52934 .53805 .53624 .53577 .53252 .53189 Metetra
.671 .745 .906 r
.53252 .53189 .53624 .53577 .54317 .53352 .53946 .52962 Metetra
.669 .744 .906 r
.53946 .52962 .54317 .53352 .55011 .5313 .54641 .52738 Metetra
.668 .743 .907 r
.54641 .52738 .55011 .5313 .55706 .52909 .55338 .52516 Metetra
.667 .743 .907 r
.55338 .52516 .55706 .52909 .56404 .5269 .56036 .52296 Metetra
.666 .742 .908 r
.56036 .52296 .56404 .5269 .57103 .52473 .56737 .52078 Metetra
.664 .742 .908 r
.56737 .52078 .57103 .52473 .57804 .52257 .57439 .51861 Metetra
.664 .742 .908 r
.57439 .51861 .57804 .52257 .58507 .52041 .58143 .51645 Metetra
.665 .742 .908 r
.58143 .51645 .58507 .52041 .59211 .51823 .58848 .51427 Metetra
.667 .743 .907 r
.58848 .51427 .59211 .51823 .59917 .51601 .59556 .51204 Metetra
.669 .744 .907 r
.59556 .51204 .59917 .51601 .60625 .51376 .60265 .50977 Metetra
.67 .744 .906 r
.60265 .50977 .60625 .51376 .61335 .51147 .60976 .50747 Metetra
.669 .744 .906 r
.60976 .50747 .61335 .51147 .62046 .50919 .61688 .50517 Metetra
.667 .743 .907 r
.61688 .50517 .62046 .50919 .62759 .50694 .62403 .50292 Metetra
.659 .741 .91 r
.62403 .50292 .62759 .50694 .63475 .50484 .6312 .50081 Metetra
.654 .739 .912 r
.6312 .50081 .63475 .50484 .64193 .50282 .63839 .49878 Metetra
.65 .738 .913 r
.63839 .49878 .64193 .50282 .64912 .50087 .6456 .49682 Metetra
.646 .737 .915 r
.6456 .49682 .64912 .50087 .65635 .49898 .65283 .49494 Metetra
.641 .736 .918 r
.65283 .49494 .65635 .49898 .66359 .49716 .66009 .49314 Metetra
.638 .736 .919 r
.66009 .49314 .66359 .49716 .67086 .49538 .66737 .49138 Metetra
.642 .739 .919 r
.66737 .49138 .67086 .49538 .67814 .49353 .67467 .48954 Metetra
.652 .744 .917 r
.67467 .48954 .67814 .49353 .68544 .49151 .68198 .48752 Metetra
.66 .747 .914 r
.68198 .48752 .68544 .49151 .69275 .48935 .6893 .48534 Metetra
.664 .748 .913 r
.6893 .48534 .69275 .48935 .70007 .48712 .69664 .48308 Metetra
.665 .747 .912 r
.69664 .48308 .70007 .48712 .70741 .48485 .70399 .4808 Metetra
.666 .747 .911 r
.70399 .4808 .70741 .48485 .71477 .48257 .71137 .4785 Metetra
.667 .747 .911 r
.71137 .4785 .71477 .48257 .72215 .48026 .71876 .47617 Metetra
.666 .747 .91 r
.71876 .47617 .72215 .48026 .72955 .47796 .72617 .47384 Metetra
.66 .744 .912 r
.72617 .47384 .72955 .47796 .73698 .47576 .73361 .47163 Metetra
.646 .738 .917 r
.73361 .47163 .73698 .47576 .74444 .47379 .74108 .46968 Metetra
.635 .736 .921 r
.74108 .46968 .74444 .47379 .75193 .47198 .74859 .46792 Metetra
.635 .739 .923 r
.74859 .46792 .75193 .47198 .75944 .47014 .75612 .46614 Metetra
.651 .749 .921 r
.75612 .46614 .75944 .47014 .76695 .46803 .76365 .46407 Metetra
.679 .761 .913 r
.76365 .46407 .76695 .46803 .77445 .46544 .77116 .46145 Metetra
.702 .768 .903 r
.77116 .46145 .77445 .46544 .78193 .46241 .77866 .45833 Metetra
.707 .766 .898 r
.77866 .45833 .78193 .46241 .78942 .45927 .78616 .4551 Metetra
.698 .759 .899 r
.78616 .4551 .78942 .45927 .79695 .4563 .79369 .45206 Metetra
.686 .752 .902 r
.79369 .45206 .79695 .4563 .8045 .45354 .80126 .44927 Metetra
.677 .748 .904 r
.80126 .44927 .8045 .45354 .81209 .45095 .80886 .44666 Metetra
.672 .745 .906 r
.80886 .44666 .81209 .45095 .81971 .44846 .81649 .44415 Metetra
.668 .744 .906 r
.81649 .44415 .81971 .44846 .82735 .44603 .82415 .4417 Metetra
.667 .743 .907 r
.82415 .4417 .82735 .44603 .83502 .44362 .83183 .43928 Metetra
.666 .742 .907 r
.83183 .43928 .83502 .44362 .84271 .44124 .83953 .43688 Metetra
.665 .742 .908 r
.83953 .43688 .84271 .44124 .85042 .43885 .84726 .43449 Metetra
.664 .742 .908 r
.84726 .43449 .85042 .43885 .85815 .43647 .85501 .4321 Metetra
.664 .742 .908 r
.85501 .4321 .85815 .43647 .8659 .43409 .86277 .42971 Metetra
.664 .741 .908 r
.86277 .42971 .8659 .43409 .87368 .43171 .87056 .42731 Metetra
.664 .741 .908 r
.87056 .42731 .87368 .43171 .88147 .42932 .87837 .42491 Metetra
.664 .741 .908 r
.87837 .42491 .88147 .42932 .88929 .42693 .8862 .42251 Metetra
.664 .741 .908 r
.8862 .42251 .88929 .42693 .89712 .42453 .89405 .4201 Metetra
.664 .741 .908 r
.89405 .4201 .89712 .42453 .90498 .42213 .90193 .41769 Metetra
.664 .741 .908 r
.90193 .41769 .90498 .42213 .91286 .41972 .90982 .41527 Metetra
.664 .741 .908 r
.90982 .41527 .91286 .41972 .92076 .41731 .91773 .41284 Metetra
.664 .741 .908 r
.91773 .41284 .92076 .41731 .92868 .41489 .92567 .41041 Metetra
.664 .741 .908 r
.92567 .41041 .92868 .41489 .93662 .41246 .93363 .40797 Metetra
.664 .741 .908 r
.93363 .40797 .93662 .41246 .94459 .41003 .94161 .40552 Metetra
.664 .741 .908 r
.94161 .40552 .94459 .41003 .95257 .40759 .94961 .40307 Metetra
.653 .74 .913 r
.37376 .57482 .37774 .57849 .38428 .57667 .38031 .57298 Metetra
.655 .741 .912 r
.38031 .57298 .38428 .57667 .39084 .5748 .38688 .5711 Metetra
.657 .741 .912 r
.38688 .5711 .39084 .5748 .39742 .5729 .39347 .56918 Metetra
.659 .741 .911 r
.39347 .56918 .39742 .5729 .40401 .57096 .40007 .56724 Metetra
.659 .741 .911 r
.40007 .56724 .40401 .57096 .41062 .56901 .40669 .56528 Metetra
.659 .741 .911 r
.40669 .56528 .41062 .56901 .41725 .56706 .41333 .56331 Metetra
.659 .741 .911 r
.41333 .56331 .41725 .56706 .4239 .5651 .41998 .56135 Metetra
.659 .741 .911 r
.41998 .56135 .4239 .5651 .43056 .56314 .42666 .55938 Metetra
.658 .741 .911 r
.42666 .55938 .43056 .56314 .43724 .5612 .43334 .55743 Metetra
.657 .741 .912 r
.43334 .55743 .43724 .5612 .44393 .55926 .44005 .55549 Metetra
.655 .74 .913 r
.44005 .55549 .44393 .55926 .45064 .55735 .44677 .55358 Metetra
.653 .74 .914 r
.44677 .55358 .45064 .55735 .45737 .55547 .45351 .55169 Metetra
.65 .74 .915 r
.45351 .55169 .45737 .55547 .46411 .55362 .46026 .54985 Metetra
.648 .74 .916 r
.46026 .54985 .46411 .55362 .47087 .55181 .46703 .54804 Metetra
.646 .74 .918 r
.46703 .54804 .47087 .55181 .47765 .55001 .47382 .54626 Metetra
.65 .742 .917 r
.47382 .54626 .47765 .55001 .48445 .54814 .48063 .5444 Metetra
.662 .747 .913 r
.48063 .5444 .48445 .54814 .49126 .54608 .48746 .54233 Metetra
.675 .751 .908 r
.48746 .54233 .49126 .54608 .4981 .54378 .4943 .54001 Metetra
.682 .752 .904 r
.4943 .54001 .4981 .54378 .50495 .54136 .50117 .53754 Metetra
.682 .75 .903 r
.50117 .53754 .50495 .54136 .51182 .53892 .50805 .53508 Metetra
.679 .748 .903 r
.50805 .53508 .51182 .53892 .51871 .53653 .51495 .53266 Metetra
.676 .747 .904 r
.51495 .53266 .51871 .53653 .52561 .53419 .52186 .5303 Metetra
.673 .746 .905 r
.52186 .5303 .52561 .53419 .53252 .53189 .52879 .52798 Metetra
.671 .745 .906 r
.52879 .52798 .53252 .53189 .53946 .52962 .53573 .5257 Metetra
.67 .744 .906 r
.53573 .5257 .53946 .52962 .54641 .52738 .54269 .52345 Metetra
.668 .743 .907 r
.54269 .52345 .54641 .52738 .55338 .52516 .54967 .52122 Metetra
.667 .743 .907 r
.54967 .52122 .55338 .52516 .56036 .52296 .55667 .51901 Metetra
.666 .742 .908 r
.55667 .51901 .56036 .52296 .56737 .52078 .56369 .51682 Metetra
.664 .742 .908 r
.56369 .51682 .56737 .52078 .57439 .51861 .57072 .51464 Metetra
.664 .742 .908 r
.57072 .51464 .57439 .51861 .58143 .51645 .57777 .51247 Metetra
.664 .742 .908 r
.57777 .51247 .58143 .51645 .58848 .51427 .58484 .51028 Metetra
.667 .743 .907 r
.58484 .51028 .58848 .51427 .59556 .51204 .59193 .50804 Metetra
.669 .744 .907 r
.59193 .50804 .59556 .51204 .60265 .50977 .59903 .50576 Metetra
.67 .744 .906 r
.59903 .50576 .60265 .50977 .60976 .50747 .60615 .50345 Metetra
.669 .744 .906 r
.60615 .50345 .60976 .50747 .61688 .50517 .61329 .50114 Metetra
.667 .743 .907 r
.61329 .50114 .61688 .50517 .62403 .50292 .62045 .49888 Metetra
.659 .741 .91 r
.62045 .49888 .62403 .50292 .6312 .50081 .62763 .49676 Metetra
.654 .739 .912 r
.62763 .49676 .6312 .50081 .63839 .49878 .63483 .49473 Metetra
.65 .738 .914 r
.63483 .49473 .63839 .49878 .6456 .49682 .64205 .49277 Metetra
.645 .736 .916 r
.64205 .49277 .6456 .49682 .65283 .49494 .6493 .49089 Metetra
.639 .735 .918 r
.6493 .49089 .65283 .49494 .66009 .49314 .65657 .4891 Metetra
.636 .736 .92 r
.65657 .4891 .66009 .49314 .66737 .49138 .66386 .48736 Metetra
.641 .739 .92 r
.66386 .48736 .66737 .49138 .67467 .48954 .67118 .48554 Metetra
.651 .744 .917 r
.67118 .48554 .67467 .48954 .68198 .48752 .6785 .48351 Metetra
.66 .747 .914 r
.6785 .48351 .68198 .48752 .6893 .48534 .68583 .48131 Metetra
.664 .748 .913 r
.68583 .48131 .6893 .48534 .69664 .48308 .69318 .47903 Metetra
.665 .748 .912 r
.69318 .47903 .69664 .48308 .70399 .4808 .70055 .47673 Metetra
.666 .748 .911 r
.70055 .47673 .70399 .4808 .71137 .4785 .70794 .47441 Metetra
.667 .748 .911 r
.70794 .47441 .71137 .4785 .71876 .47617 .71535 .47207 Metetra
.667 .747 .91 r
.71535 .47207 .71876 .47617 .72617 .47384 .72277 .46971 Metetra
.66 .744 .912 r
.72277 .46971 .72617 .47384 .73361 .47163 .73022 .46749 Metetra
.644 .738 .917 r
.73022 .46749 .73361 .47163 .74108 .46968 .73771 .46556 Metetra
.631 .735 .922 r
.73771 .46556 .74108 .46968 .74859 .46792 .74524 .46385 Metetra
.631 .738 .925 r
.74524 .46385 .74859 .46792 .75612 .46614 .75279 .46213 Metetra
.648 .748 .923 r
.75279 .46213 .75612 .46614 .76365 .46407 .76034 .4601 Metetra
.68 .763 .914 r
.76034 .4601 .76365 .46407 .77116 .46145 .76786 .45746 Metetra
.706 .771 .903 r
.76786 .45746 .77116 .46145 .77866 .45833 .77536 .45426 Metetra
.711 .768 .897 r
.77536 .45426 .77866 .45833 .78616 .4551 .78287 .45093 Metetra
.7 .76 .898 r
.78287 .45093 .78616 .4551 .79369 .45206 .79041 .44781 Metetra
.687 .753 .901 r
.79041 .44781 .79369 .45206 .80126 .44927 .79799 .44498 Metetra
.678 .748 .904 r
.79799 .44498 .80126 .44927 .80886 .44666 .80561 .44234 Metetra
.672 .745 .905 r
.80561 .44234 .80886 .44666 .81649 .44415 .81326 .43982 Metetra
.669 .744 .906 r
.81326 .43982 .81649 .44415 .82415 .4417 .82093 .43735 Metetra
.667 .743 .907 r
.82093 .43735 .82415 .4417 .83183 .43928 .82863 .43492 Metetra
.666 .742 .907 r
.82863 .43492 .83183 .43928 .83953 .43688 .83634 .43251 Metetra
.665 .742 .908 r
.83634 .43251 .83953 .43688 .84726 .43449 .84408 .4301 Metetra
.664 .742 .908 r
.84408 .4301 .84726 .43449 .85501 .4321 .85184 .4277 Metetra
.664 .742 .908 r
.85184 .4277 .85501 .4321 .86277 .42971 .85963 .4253 Metetra
.664 .741 .908 r
.85963 .4253 .86277 .42971 .87056 .42731 .86743 .42289 Metetra
.664 .741 .908 r
.86743 .42289 .87056 .42731 .87837 .42491 .87525 .42048 Metetra
.664 .741 .908 r
.87525 .42048 .87837 .42491 .8862 .42251 .8831 .41806 Metetra
.664 .741 .908 r
.8831 .41806 .8862 .42251 .89405 .4201 .89097 .41564 Metetra
.664 .741 .908 r
.89097 .41564 .89405 .4201 .90193 .41769 .89885 .41322 Metetra
.664 .741 .908 r
.89885 .41322 .90193 .41769 .90982 .41527 .90676 .41078 Metetra
.664 .741 .908 r
.90676 .41078 .90982 .41527 .91773 .41284 .9147 .40835 Metetra
.664 .741 .908 r
.9147 .40835 .91773 .41284 .92567 .41041 .92265 .4059 Metetra
.664 .741 .908 r
.92265 .4059 .92567 .41041 .93363 .40797 .93062 .40345 Metetra
.664 .741 .908 r
.93062 .40345 .93363 .40797 .94161 .40552 .93862 .40099 Metetra
.664 .741 .908 r
.93862 .40099 .94161 .40552 .94961 .40307 .94663 .39853 Metetra
.654 .741 .914 r
.36976 .57113 .37376 .57482 .38031 .57298 .37632 .56927 Metetra
.655 .741 .913 r
.37632 .56927 .38031 .57298 .38688 .5711 .3829 .56738 Metetra
.658 .741 .912 r
.3829 .56738 .38688 .5711 .39347 .56918 .38949 .56546 Metetra
.659 .741 .911 r
.38949 .56546 .39347 .56918 .40007 .56724 .39611 .5635 Metetra
.659 .741 .911 r
.39611 .5635 .40007 .56724 .40669 .56528 .40274 .56153 Metetra
.659 .741 .911 r
.40274 .56153 .40669 .56528 .41333 .56331 .40939 .55956 Metetra
.659 .741 .911 r
.40939 .55956 .41333 .56331 .41998 .56135 .41606 .55758 Metetra
.659 .741 .911 r
.41606 .55758 .41998 .56135 .42666 .55938 .42274 .55561 Metetra
.658 .741 .911 r
.42274 .55561 .42666 .55938 .43334 .55743 .42944 .55365 Metetra
.656 .741 .912 r
.42944 .55365 .43334 .55743 .44005 .55549 .43615 .5517 Metetra
.655 .74 .913 r
.43615 .5517 .44005 .55549 .44677 .55358 .44288 .54978 Metetra
.653 .74 .914 r
.44288 .54978 .44677 .55358 .45351 .55169 .44963 .5479 Metetra
.65 .74 .915 r
.44963 .5479 .45351 .55169 .46026 .54985 .45639 .54605 Metetra
.646 .739 .917 r
.45639 .54605 .46026 .54985 .46703 .54804 .46317 .54426 Metetra
.645 .74 .918 r
.46317 .54426 .46703 .54804 .47382 .54626 .46997 .54249 Metetra
.648 .742 .918 r
.46997 .54249 .47382 .54626 .48063 .5444 .47679 .54065 Metetra
.661 .747 .914 r
.47679 .54065 .48063 .5444 .48746 .54233 .48363 .53858 Metetra
.676 .752 .908 r
.48363 .53858 .48746 .54233 .4943 .54001 .49049 .53622 Metetra
.684 .752 .903 r
.49049 .53622 .4943 .54001 .50117 .53754 .49737 .53372 Metetra
.683 .751 .902 r
.49737 .53372 .50117 .53754 .50805 .53508 .50426 .53122 Metetra
.68 .749 .903 r
.50426 .53122 .50805 .53508 .51495 .53266 .51117 .52877 Metetra
.676 .747 .904 r
.51117 .52877 .51495 .53266 .52186 .5303 .51809 .52639 Metetra
.674 .746 .905 r
.51809 .52639 .52186 .5303 .52879 .52798 .52503 .52406 Metetra
.671 .745 .906 r
.52503 .52406 .52879 .52798 .53573 .5257 .53199 .52177 Metetra
.67 .744 .906 r
.53199 .52177 .53573 .5257 .54269 .52345 .53896 .5195 Metetra
.668 .743 .907 r
.53896 .5195 .54269 .52345 .54967 .52122 .54596 .51726 Metetra
.667 .743 .907 r
.54596 .51726 .54967 .52122 .55667 .51901 .55296 .51504 Metetra
.666 .742 .908 r
.55296 .51504 .55667 .51901 .56369 .51682 .55999 .51283 Metetra
.664 .742 .908 r
.55999 .51283 .56369 .51682 .57072 .51464 .56704 .51065 Metetra
.663 .742 .908 r
.56704 .51065 .57072 .51464 .57777 .51247 .5741 .50848 Metetra
.664 .742 .908 r
.5741 .50848 .57777 .51247 .58484 .51028 .58118 .50628 Metetra
.667 .743 .908 r
.58118 .50628 .58484 .51028 .59193 .50804 .58828 .50403 Metetra
.669 .744 .907 r
.58828 .50403 .59193 .50804 .59903 .50576 .59539 .50173 Metetra
.67 .744 .906 r
.59539 .50173 .59903 .50576 .60615 .50345 .60253 .49941 Metetra
.669 .744 .906 r
.60253 .49941 .60615 .50345 .61329 .50114 .60968 .49709 Metetra
.667 .743 .907 r
.60968 .49709 .61329 .50114 .62045 .49888 .61685 .49483 Metetra
.659 .741 .91 r
.61685 .49483 .62045 .49888 .62763 .49676 .62404 .49269 Metetra
.654 .739 .912 r
.62404 .49269 .62763 .49676 .63483 .49473 .63125 .49065 Metetra
.649 .738 .914 r
.63125 .49065 .63483 .49473 .64205 .49277 .63849 .48869 Metetra
.644 .736 .916 r
.63849 .48869 .64205 .49277 .6493 .49089 .64575 .48682 Metetra
.638 .735 .919 r
.64575 .48682 .6493 .49089 .65657 .4891 .65304 .48505 Metetra
.634 .735 .921 r
.65304 .48505 .65657 .4891 .66386 .48736 .66034 .48334 Metetra
.639 .739 .921 r
.66034 .48334 .66386 .48736 .67118 .48554 .66767 .48153 Metetra
.651 .744 .918 r
.66767 .48153 .67118 .48554 .6785 .48351 .67501 .47949 Metetra
.661 .748 .914 r
.67501 .47949 .6785 .48351 .68583 .48131 .68235 .47727 Metetra
.665 .749 .913 r
.68235 .47727 .68583 .48131 .69318 .47903 .68972 .47497 Metetra
.666 .748 .912 r
.68972 .47497 .69318 .47903 .70055 .47673 .6971 .47264 Metetra
.666 .748 .911 r
.6971 .47264 .70055 .47673 .70794 .47441 .7045 .47031 Metetra
.667 .748 .911 r
.7045 .47031 .70794 .47441 .71535 .47207 .71192 .46794 Metetra
.668 .748 .91 r
.71192 .46794 .71535 .47207 .72277 .46971 .71935 .46556 Metetra
.66 .744 .912 r
.71935 .46556 .72277 .46971 .73022 .46749 .72682 .46332 Metetra
.642 .737 .918 r
.72682 .46332 .73022 .46749 .73771 .46556 .73432 .46142 Metetra
.627 .733 .924 r
.73432 .46142 .73771 .46556 .74524 .46385 .74187 .45977 Metetra
.626 .737 .927 r
.74187 .45977 .74524 .46385 .75279 .46213 .74944 .45812 Metetra
.644 .748 .925 r
.74944 .45812 .75279 .46213 .76034 .4601 .75701 .45615 Metetra
.68 .764 .915 r
.75701 .45615 .76034 .4601 .76786 .45746 .76455 .45348 Metetra
.71 .773 .902 r
.76455 .45348 .76786 .45746 .77536 .45426 .77206 .45018 Metetra
.715 .77 .896 r
.77206 .45018 .77536 .45426 .78287 .45093 .77957 .44674 Metetra
.703 .762 .897 r
.77957 .44674 .78287 .45093 .79041 .44781 .78712 .44355 Metetra
.689 .754 .901 r
.78712 .44355 .79041 .44781 .79799 .44498 .79472 .44067 Metetra
.678 .748 .904 r
.79472 .44067 .79799 .44498 .80561 .44234 .80235 .438 Metetra
.672 .745 .905 r
.80235 .438 .80561 .44234 .81326 .43982 .81001 .43546 Metetra
.669 .744 .906 r
.81001 .43546 .81326 .43982 .82093 .43735 .81769 .43298 Metetra
.667 .743 .907 r
.81769 .43298 .82093 .43735 .82863 .43492 .8254 .43054 Metetra
.666 .742 .907 r
.8254 .43054 .82863 .43492 .83634 .43251 .83314 .42811 Metetra
.665 .742 .907 r
.83314 .42811 .83634 .43251 .84408 .4301 .84089 .4257 Metetra
.664 .742 .908 r
.84089 .4257 .84408 .4301 .85184 .4277 .84867 .42328 Metetra
.664 .742 .908 r
.84867 .42328 .85184 .4277 .85963 .4253 .85646 .42087 Metetra
.664 .741 .908 r
.85646 .42087 .85963 .4253 .86743 .42289 .86428 .41845 Metetra
.664 .741 .908 r
.86428 .41845 .86743 .42289 .87525 .42048 .87212 .41602 Metetra
.664 .741 .908 r
.87212 .41602 .87525 .42048 .8831 .41806 .87998 .4136 Metetra
.664 .741 .908 r
.87998 .4136 .8831 .41806 .89097 .41564 .88787 .41116 Metetra
.664 .741 .908 r
.88787 .41116 .89097 .41564 .89885 .41322 .89577 .40873 Metetra
.664 .741 .908 r
.89577 .40873 .89885 .41322 .90676 .41078 .90369 .40628 Metetra
.664 .741 .908 r
.90369 .40628 .90676 .41078 .9147 .40835 .91164 .40383 Metetra
.664 .741 .908 r
.91164 .40383 .9147 .40835 .92265 .4059 .91961 .40137 Metetra
.664 .741 .908 r
.91961 .40137 .92265 .4059 .93062 .40345 .9276 .39891 Metetra
.664 .741 .908 r
.9276 .39891 .93062 .40345 .93862 .40099 .93561 .39644 Metetra
.664 .741 .908 r
.93561 .39644 .93862 .40099 .94663 .39853 .94364 .39397 Metetra
.654 .741 .914 r
.36574 .56742 .36976 .57113 .37632 .56927 .37231 .56555 Metetra
.656 .741 .913 r
.37231 .56555 .37632 .56927 .3829 .56738 .3789 .56365 Metetra
.658 .741 .912 r
.3789 .56365 .3829 .56738 .38949 .56546 .38551 .56171 Metetra
.659 .741 .911 r
.38551 .56171 .38949 .56546 .39611 .5635 .39213 .55974 Metetra
.659 .741 .911 r
.39213 .55974 .39611 .5635 .40274 .56153 .39877 .55776 Metetra
.659 .741 .911 r
.39877 .55776 .40274 .56153 .40939 .55956 .40543 .55578 Metetra
.659 .741 .911 r
.40543 .55578 .40939 .55956 .41606 .55758 .41211 .5538 Metetra
.658 .741 .911 r
.41211 .5538 .41606 .55758 .42274 .55561 .4188 .55182 Metetra
.658 .741 .911 r
.4188 .55182 .42274 .55561 .42944 .55365 .42551 .54985 Metetra
.656 .741 .912 r
.42551 .54985 .42944 .55365 .43615 .5517 .43223 .5479 Metetra
.654 .74 .913 r
.43223 .5479 .43615 .5517 .44288 .54978 .43897 .54597 Metetra
.652 .74 .914 r
.43897 .54597 .44288 .54978 .44963 .5479 .44573 .54408 Metetra
.649 .739 .915 r
.44573 .54408 .44963 .5479 .45639 .54605 .45251 .54224 Metetra
.645 .739 .917 r
.45251 .54224 .45639 .54605 .46317 .54426 .4593 .54046 Metetra
.643 .739 .919 r
.4593 .54046 .46317 .54426 .46997 .54249 .46611 .5387 Metetra
.647 .742 .919 r
.46611 .5387 .46997 .54249 .47679 .54065 .47294 .53688 Metetra
.661 .747 .914 r
.47294 .53688 .47679 .54065 .48363 .53858 .47979 .53481 Metetra
.678 .752 .907 r
.47979 .53481 .48363 .53858 .49049 .53622 .48666 .53242 Metetra
.685 .753 .903 r
.48666 .53242 .49049 .53622 .49737 .53372 .49355 .52988 Metetra
.684 .751 .902 r
.49355 .52988 .49737 .53372 .50426 .53122 .50045 .52734 Metetra
.681 .749 .903 r
.50045 .52734 .50426 .53122 .51117 .52877 .50737 .52487 Metetra
.677 .747 .904 r
.50737 .52487 .51117 .52877 .51809 .52639 .51431 .52247 Metetra
.674 .746 .905 r
.51431 .52247 .51809 .52639 .52503 .52406 .52126 .52012 Metetra
.672 .745 .905 r
.52126 .52012 .52503 .52406 .53199 .52177 .52823 .51781 Metetra
.67 .744 .906 r
.52823 .51781 .53199 .52177 .53896 .5195 .53521 .51553 Metetra
.668 .743 .907 r
.53521 .51553 .53896 .5195 .54596 .51726 .54222 .51328 Metetra
.667 .743 .907 r
.54222 .51328 .54596 .51726 .55296 .51504 .54924 .51105 Metetra
.666 .742 .908 r
.54924 .51105 .55296 .51504 .55999 .51283 .55628 .50883 Metetra
.664 .742 .908 r
.55628 .50883 .55999 .51283 .56704 .51065 .56334 .50664 Metetra
.663 .742 .909 r
.56334 .50664 .56704 .51065 .5741 .50848 .57041 .50446 Metetra
.664 .742 .908 r
.57041 .50446 .5741 .50848 .58118 .50628 .5775 .50226 Metetra
.667 .743 .908 r
.5775 .50226 .58118 .50628 .58828 .50403 .58461 .5 Metetra
.669 .744 .907 r
.58461 .5 .58828 .50403 .59539 .50173 .59174 .49769 Metetra
.67 .744 .906 r
.59174 .49769 .59539 .50173 .60253 .49941 .59889 .49536 Metetra
.669 .744 .906 r
.59889 .49536 .60253 .49941 .60968 .49709 .60605 .49303 Metetra
.666 .743 .908 r
.60605 .49303 .60968 .49709 .61685 .49483 .61323 .49075 Metetra
.659 .741 .91 r
.61323 .49075 .61685 .49483 .62404 .49269 .62044 .48861 Metetra
.654 .739 .912 r
.62044 .48861 .62404 .49269 .63125 .49065 .62766 .48656 Metetra
.649 .738 .914 r
.62766 .48656 .63125 .49065 .63849 .48869 .63491 .4846 Metetra
.643 .736 .916 r
.63491 .4846 .63849 .48869 .64575 .48682 .64219 .48273 Metetra
.636 .734 .919 r
.64219 .48273 .64575 .48682 .65304 .48505 .64948 .48098 Metetra
.631 .734 .922 r
.64948 .48098 .65304 .48505 .66034 .48334 .65681 .4793 Metetra
.637 .739 .922 r
.65681 .4793 .66034 .48334 .66767 .48153 .66415 .47751 Metetra
.651 .745 .918 r
.66415 .47751 .66767 .48153 .67501 .47949 .6715 .47546 Metetra
.662 .749 .915 r
.6715 .47546 .67501 .47949 .68235 .47727 .67886 .47321 Metetra
.666 .749 .913 r
.67886 .47321 .68235 .47727 .68972 .47497 .68623 .47089 Metetra
.666 .749 .912 r
.68623 .47089 .68972 .47497 .6971 .47264 .69363 .46854 Metetra
.666 .748 .911 r
.69363 .46854 .6971 .47264 .7045 .47031 .70104 .46619 Metetra
.668 .748 .911 r
.70104 .46619 .7045 .47031 .71192 .46794 .70847 .4638 Metetra
.669 .748 .91 r
.70847 .4638 .71192 .46794 .71935 .46556 .71592 .46139 Metetra
.661 .744 .912 r
.71592 .46139 .71935 .46556 .72682 .46332 .7234 .45913 Metetra
.639 .736 .918 r
.7234 .45913 .72682 .46332 .73432 .46142 .73092 .45726 Metetra
.622 .731 .925 r
.73092 .45726 .73432 .46142 .74187 .45977 .73848 .45568 Metetra
.62 .735 .929 r
.73848 .45568 .74187 .45977 .74944 .45812 .74607 .45412 Metetra
.64 .747 .927 r
.74607 .45412 .74944 .45812 .75701 .45615 .75366 .4522 Metetra
.68 .766 .916 r
.75366 .4522 .75701 .45615 .76455 .45348 .76121 .44952 Metetra
.714 .776 .901 r
.76121 .44952 .76455 .45348 .77206 .45018 .76873 .4461 Metetra
.72 .773 .894 r
.76873 .4461 .77206 .45018 .77957 .44674 .77626 .44253 Metetra
.706 .763 .896 r
.77626 .44253 .77957 .44674 .78712 .44355 .78382 .43926 Metetra
.69 .754 .9 r
.78382 .43926 .78712 .44355 .79472 .44067 .79142 .43634 Metetra
.679 .749 .903 r
.79142 .43634 .79472 .44067 .80235 .438 .79907 .43365 Metetra
.673 .745 .905 r
.79907 .43365 .80235 .438 .81001 .43546 .80674 .43108 Metetra
.669 .744 .906 r
.80674 .43108 .81001 .43546 .81769 .43298 .81444 .42859 Metetra
.667 .743 .907 r
.81444 .42859 .81769 .43298 .8254 .43054 .82217 .42614 Metetra
.666 .742 .907 r
.82217 .42614 .8254 .43054 .83314 .42811 .82991 .4237 Metetra
.665 .742 .907 r
.82991 .4237 .83314 .42811 .84089 .4257 .83768 .42127 Metetra
.664 .742 .908 r
.83768 .42127 .84089 .4257 .84867 .42328 .84547 .41884 Metetra
.664 .742 .908 r
.84547 .41884 .84867 .42328 .85646 .42087 .85329 .41641 Metetra
.664 .741 .908 r
.85329 .41641 .85646 .42087 .86428 .41845 .86112 .41398 Metetra
.664 .741 .908 r
.86112 .41398 .86428 .41845 .87212 .41602 .86898 .41155 Metetra
.664 .741 .908 r
.86898 .41155 .87212 .41602 .87998 .4136 .87685 .40911 Metetra
.664 .741 .908 r
.87685 .40911 .87998 .4136 .88787 .41116 .88475 .40666 Metetra
.664 .741 .908 r
.88475 .40666 .88787 .41116 .89577 .40873 .89267 .40421 Metetra
.664 .741 .908 r
.89267 .40421 .89577 .40873 .90369 .40628 .90061 .40176 Metetra
.664 .741 .908 r
.90061 .40176 .90369 .40628 .91164 .40383 .90857 .39929 Metetra
.664 .741 .908 r
.90857 .39929 .91164 .40383 .91961 .40137 .91656 .39682 Metetra
.664 .741 .908 r
.91656 .39682 .91961 .40137 .9276 .39891 .92456 .39435 Metetra
.664 .741 .908 r
.92456 .39435 .9276 .39891 .93561 .39644 .93259 .39187 Metetra
.664 .741 .908 r
.93259 .39187 .93561 .39644 .94364 .39397 .94064 .38938 Metetra
.654 .741 .913 r
.3617 .5637 .36574 .56742 .37231 .56555 .36828 .56182 Metetra
.656 .741 .913 r
.36828 .56182 .37231 .56555 .3789 .56365 .37488 .5599 Metetra
.658 .741 .911 r
.37488 .5599 .3789 .56365 .38551 .56171 .3815 .55795 Metetra
.659 .741 .911 r
.3815 .55795 .38551 .56171 .39213 .55974 .38813 .55597 Metetra
.659 .741 .911 r
.38813 .55597 .39213 .55974 .39877 .55776 .39479 .55398 Metetra
.659 .741 .911 r
.39479 .55398 .39877 .55776 .40543 .55578 .40146 .55199 Metetra
.659 .741 .911 r
.40146 .55199 .40543 .55578 .41211 .5538 .40814 .54999 Metetra
.658 .741 .911 r
.40814 .54999 .41211 .5538 .4188 .55182 .41484 .54801 Metetra
.657 .741 .911 r
.41484 .54801 .4188 .55182 .42551 .54985 .42156 .54603 Metetra
.656 .741 .912 r
.42156 .54603 .42551 .54985 .43223 .5479 .4283 .54407 Metetra
.654 .74 .913 r
.4283 .54407 .43223 .5479 .43897 .54597 .43505 .54214 Metetra
.652 .74 .914 r
.43505 .54214 .43897 .54597 .44573 .54408 .44182 .54025 Metetra
.648 .739 .916 r
.44182 .54025 .44573 .54408 .45251 .54224 .4486 .53842 Metetra
.644 .739 .918 r
.4486 .53842 .45251 .54224 .4593 .54046 .45541 .53664 Metetra
.641 .739 .92 r
.45541 .53664 .4593 .54046 .46611 .5387 .46222 .53491 Metetra
.645 .741 .92 r
.46222 .53491 .46611 .5387 .47294 .53688 .46906 .53311 Metetra
.66 .748 .915 r
.46906 .53311 .47294 .53688 .47979 .53481 .47592 .53104 Metetra
.679 .753 .907 r
.47592 .53104 .47979 .53481 .48666 .53242 .48281 .52862 Metetra
.687 .754 .902 r
.48281 .52862 .48666 .53242 .49355 .52988 .48971 .52602 Metetra
.686 .752 .901 r
.48971 .52602 .49355 .52988 .50045 .52734 .49663 .52345 Metetra
.682 .749 .902 r
.49663 .52345 .50045 .52734 .50737 .52487 .50356 .52095 Metetra
.678 .747 .904 r
.50356 .52095 .50737 .52487 .51431 .52247 .51051 .51853 Metetra
.674 .746 .905 r
.51051 .51853 .51431 .52247 .52126 .52012 .51747 .51616 Metetra
.672 .745 .905 r
.51747 .51616 .52126 .52012 .52823 .51781 .52445 .51384 Metetra
.67 .744 .906 r
.52445 .51384 .52823 .51781 .53521 .51553 .53145 .51155 Metetra
.668 .743 .906 r
.53145 .51155 .53521 .51553 .54222 .51328 .53846 .50928 Metetra
.667 .743 .907 r
.53846 .50928 .54222 .51328 .54924 .51105 .5455 .50704 Metetra
.665 .742 .908 r
.5455 .50704 .54924 .51105 .55628 .50883 .55255 .50481 Metetra
.664 .742 .908 r
.55255 .50481 .55628 .50883 .56334 .50664 .55962 .50261 Metetra
.663 .742 .909 r
.55962 .50261 .56334 .50664 .57041 .50446 .5667 .50043 Metetra
.664 .742 .909 r
.5667 .50043 .57041 .50446 .5775 .50226 .57381 .49822 Metetra
.667 .744 .908 r
.57381 .49822 .5775 .50226 .58461 .5 .58093 .49596 Metetra
.669 .744 .907 r
.58093 .49596 .58461 .5 .59174 .49769 .58807 .49363 Metetra
.67 .744 .906 r
.58807 .49363 .59174 .49769 .59889 .49536 .59523 .49128 Metetra
.669 .744 .906 r
.59523 .49128 .59889 .49536 .60605 .49303 .6024 .48894 Metetra
.666 .743 .908 r
.6024 .48894 .60605 .49303 .61323 .49075 .6096 .48666 Metetra
.659 .741 .91 r
.6096 .48666 .61323 .49075 .62044 .48861 .61682 .4845 Metetra
.653 .739 .912 r
.61682 .4845 .62044 .48861 .62766 .48656 .62406 .48245 Metetra
.648 .737 .914 r
.62406 .48245 .62766 .48656 .63491 .4846 .63132 .48048 Metetra
.642 .736 .917 r
.63132 .48048 .63491 .4846 .64219 .48273 .6386 .47863 Metetra
.634 .734 .92 r
.6386 .47863 .64219 .48273 .64948 .48098 .64592 .4769 Metetra
.629 .734 .923 r
.64592 .4769 .64948 .48098 .65681 .4793 .65325 .47525 Metetra
.635 .738 .923 r
.65325 .47525 .65681 .4793 .66415 .47751 .66061 .47348 Metetra
.651 .746 .919 r
.66061 .47348 .66415 .47751 .6715 .47546 .66797 .47142 Metetra
.663 .75 .915 r
.66797 .47142 .6715 .47546 .67886 .47321 .67534 .46915 Metetra
.667 .75 .913 r
.67534 .46915 .67886 .47321 .68623 .47089 .68273 .46679 Metetra
.667 .749 .912 r
.68273 .46679 .68623 .47089 .69363 .46854 .69014 .46443 Metetra
.667 .749 .911 r
.69014 .46443 .69363 .46854 .70104 .46619 .69757 .46206 Metetra
.669 .749 .91 r
.69757 .46206 .70104 .46619 .70847 .4638 .70501 .45965 Metetra
.67 .749 .909 r
.70501 .45965 .70847 .4638 .71592 .46139 .71247 .4572 Metetra
.662 .745 .912 r
.71247 .4572 .71592 .46139 .7234 .45913 .71996 .45491 Metetra
.637 .735 .919 r
.71996 .45491 .7234 .45913 .73092 .45726 .7275 .45308 Metetra
.617 .729 .927 r
.7275 .45308 .73092 .45726 .73848 .45568 .73508 .45158 Metetra
.613 .733 .931 r
.73508 .45158 .73848 .45568 .74607 .45412 .74269 .45011 Metetra
.634 .747 .929 r
.74269 .45011 .74607 .45412 .75366 .4522 .7503 .44827 Metetra
.681 .768 .917 r
.7503 .44827 .75366 .4522 .76121 .44952 .75787 .44557 Metetra
.72 .78 .9 r
.75787 .44557 .76121 .44952 .76873 .4461 .7654 .44203 Metetra
.725 .776 .893 r
.7654 .44203 .76873 .4461 .77626 .44253 .77293 .43832 Metetra
.71 .765 .895 r
.77293 .43832 .77626 .44253 .78382 .43926 .7805 .43496 Metetra
.692 .755 .9 r
.7805 .43496 .78382 .43926 .79142 .43634 .78811 .43198 Metetra
.68 .749 .903 r
.78811 .43198 .79142 .43634 .79907 .43365 .79577 .42927 Metetra
.673 .745 .905 r
.79577 .42927 .79907 .43365 .80674 .43108 .80346 .42669 Metetra
.669 .744 .906 r
.80346 .42669 .80674 .43108 .81444 .42859 .81117 .42418 Metetra
.667 .743 .907 r
.81117 .42418 .81444 .42859 .82217 .42614 .81891 .42171 Metetra
.666 .742 .907 r
.81891 .42171 .82217 .42614 .82991 .4237 .82668 .41926 Metetra
.665 .742 .907 r
.82668 .41926 .82991 .4237 .83768 .42127 .83446 .41682 Metetra
.664 .742 .908 r
.83446 .41682 .83768 .42127 .84547 .41884 .84227 .41438 Metetra
.664 .742 .908 r
.84227 .41438 .84547 .41884 .85329 .41641 .85009 .41194 Metetra
.664 .741 .908 r
.85009 .41194 .85329 .41641 .86112 .41398 .85794 .40949 Metetra
.664 .741 .908 r
.85794 .40949 .86112 .41398 .86898 .41155 .86581 .40705 Metetra
.664 .741 .908 r
.86581 .40705 .86898 .41155 .87685 .40911 .8737 .4046 Metetra
.664 .741 .908 r
.8737 .4046 .87685 .40911 .88475 .40666 .88162 .40214 Metetra
.664 .741 .908 r
.88162 .40214 .88475 .40666 .89267 .40421 .88955 .39968 Metetra
.664 .741 .908 r
.88955 .39968 .89267 .40421 .90061 .40176 .89751 .39721 Metetra
.664 .741 .908 r
.89751 .39721 .90061 .40176 .90857 .39929 .90549 .39473 Metetra
.664 .741 .908 r
.90549 .39473 .90857 .39929 .91656 .39682 .91349 .39225 Metetra
.664 .741 .908 r
.91349 .39225 .91656 .39682 .92456 .39435 .92151 .38976 Metetra
.664 .741 .908 r
.92151 .38976 .92456 .39435 .93259 .39187 .92955 .38727 Metetra
.664 .741 .908 r
.92955 .38727 .93259 .39187 .94064 .38938 .93762 .38477 Metetra
.655 .741 .913 r
.35764 .55997 .3617 .5637 .36828 .56182 .36424 .55807 Metetra
.656 .741 .912 r
.36424 .55807 .36828 .56182 .37488 .5599 .37085 .55614 Metetra
.658 .742 .911 r
.37085 .55614 .37488 .5599 .3815 .55795 .37747 .55417 Metetra
.659 .742 .911 r
.37747 .55417 .3815 .55795 .38813 .55597 .38412 .55218 Metetra
.659 .742 .911 r
.38412 .55218 .38813 .55597 .39479 .55398 .39078 .55018 Metetra
.659 .741 .911 r
.39078 .55018 .39479 .55398 .40146 .55199 .39746 .54818 Metetra
.659 .741 .911 r
.39746 .54818 .40146 .55199 .40814 .54999 .40416 .54618 Metetra
.658 .741 .911 r
.40416 .54618 .40814 .54999 .41484 .54801 .41087 .54418 Metetra
.657 .741 .911 r
.41087 .54418 .41484 .54801 .42156 .54603 .4176 .5422 Metetra
.656 .741 .912 r
.4176 .5422 .42156 .54603 .4283 .54407 .42435 .54023 Metetra
.654 .74 .913 r
.42435 .54023 .4283 .54407 .43505 .54214 .43111 .5383 Metetra
.651 .74 .914 r
.43111 .5383 .43505 .54214 .44182 .54025 .43789 .53641 Metetra
.647 .739 .916 r
.43789 .53641 .44182 .54025 .4486 .53842 .44468 .53457 Metetra
.643 .738 .918 r
.44468 .53457 .4486 .53842 .45541 .53664 .45149 .53281 Metetra
.639 .738 .921 r
.45149 .53281 .45541 .53664 .46222 .53491 .45832 .5311 Metetra
.642 .741 .921 r
.45832 .5311 .46222 .53491 .46906 .53311 .46517 .52933 Metetra
.659 .748 .916 r
.46517 .52933 .46906 .53311 .47592 .53104 .47204 .52726 Metetra
.681 .754 .907 r
.47204 .52726 .47592 .53104 .48281 .52862 .47894 .5248 Metetra
.689 .755 .902 r
.47894 .5248 .48281 .52862 .48971 .52602 .48585 .52215 Metetra
.687 .752 .901 r
.48585 .52215 .48971 .52602 .49663 .52345 .49278 .51954 Metetra
.682 .75 .902 r
.49278 .51954 .49663 .52345 .50356 .52095 .49973 .51701 Metetra
.678 .747 .903 r
.49973 .51701 .50356 .52095 .51051 .51853 .50669 .51457 Metetra
.675 .746 .904 r
.50669 .51457 .51051 .51853 .51747 .51616 .51366 .51219 Metetra
.672 .745 .905 r
.51366 .51219 .51747 .51616 .52445 .51384 .52065 .50985 Metetra
.67 .744 .906 r
.52065 .50985 .52445 .51384 .53145 .51155 .52766 .50754 Metetra
.669 .743 .906 r
.52766 .50754 .53145 .51155 .53846 .50928 .53469 .50526 Metetra
.667 .743 .907 r
.53469 .50526 .53846 .50928 .5455 .50704 .54174 .50301 Metetra
.665 .742 .908 r
.54174 .50301 .5455 .50704 .55255 .50481 .5488 .50077 Metetra
.664 .742 .908 r
.5488 .50077 .55255 .50481 .55962 .50261 .55588 .49857 Metetra
.663 .742 .909 r
.55588 .49857 .55962 .50261 .5667 .50043 .56298 .49638 Metetra
.663 .742 .909 r
.56298 .49638 .5667 .50043 .57381 .49822 .5701 .49417 Metetra
.667 .744 .908 r
.5701 .49417 .57381 .49822 .58093 .49596 .57723 .49189 Metetra
.67 .745 .907 r
.57723 .49189 .58093 .49596 .58807 .49363 .58438 .48955 Metetra
.67 .745 .906 r
.58438 .48955 .58807 .49363 .59523 .49128 .59155 .48719 Metetra
.669 .744 .906 r
.59155 .48719 .59523 .49128 .6024 .48894 .59874 .48484 Metetra
.666 .743 .908 r
.59874 .48484 .6024 .48894 .6096 .48666 .60595 .48255 Metetra
.659 .741 .91 r
.60595 .48255 .6096 .48666 .61682 .4845 .61318 .48038 Metetra
.653 .739 .913 r
.61318 .48038 .61682 .4845 .62406 .48245 .62043 .47832 Metetra
.648 .737 .915 r
.62043 .47832 .62406 .48245 .63132 .48048 .62771 .47635 Metetra
.641 .735 .917 r
.62771 .47635 .63132 .48048 .6386 .47863 .635 .47451 Metetra
.632 .733 .921 r
.635 .47451 .6386 .47863 .64592 .4769 .64233 .47281 Metetra
.626 .733 .924 r
.64233 .47281 .64592 .4769 .65325 .47525 .64968 .4712 Metetra
.633 .738 .924 r
.64968 .4712 .65325 .47525 .66061 .47348 .65705 .46945 Metetra
.651 .746 .919 r
.65705 .46945 .66061 .47348 .66797 .47142 .66442 .46738 Metetra
.664 .751 .915 r
.66442 .46738 .66797 .47142 .67534 .46915 .67181 .46507 Metetra
.668 .751 .912 r
.67181 .46507 .67534 .46915 .68273 .46679 .67921 .46268 Metetra
.667 .75 .912 r
.67921 .46268 .68273 .46679 .69014 .46443 .68663 .4603 Metetra
.667 .749 .911 r
.68663 .4603 .69014 .46443 .69757 .46206 .69407 .45791 Metetra
.669 .749 .91 r
.69407 .45791 .69757 .46206 .70501 .45965 .70153 .45547 Metetra
.672 .75 .909 r
.70153 .45547 .70501 .45965 .71247 .4572 .709 .45298 Metetra
.662 .745 .911 r
.709 .45298 .71247 .4572 .71996 .45491 .71651 .45067 Metetra
.634 .733 .92 r
.71651 .45067 .71996 .45491 .7275 .45308 .72406 .44888 Metetra
.611 .727 .928 r
.72406 .44888 .7275 .45308 .73508 .45158 .73167 .44747 Metetra
.606 .73 .933 r
.73167 .44747 .73508 .45158 .74269 .45011 .7393 .44611 Metetra
.628 .746 .932 r
.7393 .44611 .74269 .45011 .7503 .44827 .74693 .44437 Metetra
.681 .77 .919 r
.74693 .44437 .7503 .44827 .75787 .44557 .75451 .44165 Metetra
.725 .784 .899 r
.75451 .44165 .75787 .44557 .7654 .44203 .76204 .43796 Metetra
.731 .779 .891 r
.76204 .43796 .7654 .44203 .77293 .43832 .76958 .4341 Metetra
.714 .766 .894 r
.76958 .4341 .77293 .43832 .7805 .43496 .77716 .43063 Metetra
.694 .756 .899 r
.77716 .43063 .7805 .43496 .78811 .43198 .78479 .42761 Metetra
.681 .749 .903 r
.78479 .42761 .78811 .43198 .79577 .42927 .79246 .42486 Metetra
.673 .746 .905 r
.79246 .42486 .79577 .42927 .80346 .42669 .80016 .42227 Metetra
.669 .744 .906 r
.80016 .42227 .80346 .42669 .81117 .42418 .80789 .41975 Metetra
.667 .743 .907 r
.80789 .41975 .81117 .42418 .81891 .42171 .81565 .41726 Metetra
.666 .742 .907 r
.81565 .41726 .81891 .42171 .82668 .41926 .82342 .4148 Metetra
.665 .742 .907 r
.82342 .4148 .82668 .41926 .83446 .41682 .83122 .41234 Metetra
.664 .742 .908 r
.83122 .41234 .83446 .41682 .84227 .41438 .83904 .40989 Metetra
.664 .742 .908 r
.83904 .40989 .84227 .41438 .85009 .41194 .84688 .40744 Metetra
.664 .741 .908 r
.84688 .40744 .85009 .41194 .85794 .40949 .85475 .40498 Metetra
.664 .741 .908 r
.85475 .40498 .85794 .40949 .86581 .40705 .86263 .40253 Metetra
.664 .741 .908 r
.86263 .40253 .86581 .40705 .8737 .4046 .87054 .40006 Metetra
.664 .741 .908 r
.87054 .40006 .8737 .4046 .88162 .40214 .87847 .39759 Metetra
.664 .741 .908 r
.87847 .39759 .88162 .40214 .88955 .39968 .88642 .39512 Metetra
.664 .741 .908 r
.88642 .39512 .88955 .39968 .89751 .39721 .89439 .39264 Metetra
.664 .741 .908 r
.89439 .39264 .89751 .39721 .90549 .39473 .90239 .39015 Metetra
.664 .741 .908 r
.90239 .39015 .90549 .39473 .91349 .39225 .9104 .38765 Metetra
.664 .741 .908 r
.9104 .38765 .91349 .39225 .92151 .38976 .91844 .38515 Metetra
.664 .741 .908 r
.91844 .38515 .92151 .38976 .92955 .38727 .9265 .38265 Metetra
.664 .741 .908 r
.9265 .38265 .92955 .38727 .93762 .38477 .93458 .38013 Metetra
.655 .742 .913 r
.35357 .55623 .35764 .55997 .36424 .55807 .36017 .55431 Metetra
.657 .742 .912 r
.36017 .55431 .36424 .55807 .37085 .55614 .36679 .55236 Metetra
.658 .742 .911 r
.36679 .55236 .37085 .55614 .37747 .55417 .37343 .55038 Metetra
.659 .742 .911 r
.37343 .55038 .37747 .55417 .38412 .55218 .38009 .54838 Metetra
.66 .742 .911 r
.38009 .54838 .38412 .55218 .39078 .55018 .38676 .54637 Metetra
.659 .741 .911 r
.38676 .54637 .39078 .55018 .39746 .54818 .39345 .54435 Metetra
.659 .741 .911 r
.39345 .54435 .39746 .54818 .40416 .54618 .40015 .54234 Metetra
.658 .741 .911 r
.40015 .54234 .40416 .54618 .41087 .54418 .40688 .54034 Metetra
.657 .741 .912 r
.40688 .54034 .41087 .54418 .4176 .5422 .41362 .53834 Metetra
.656 .741 .912 r
.41362 .53834 .4176 .5422 .42435 .54023 .42037 .53638 Metetra
.654 .74 .913 r
.42037 .53638 .42435 .54023 .43111 .5383 .42715 .53444 Metetra
.651 .74 .915 r
.42715 .53444 .43111 .5383 .43789 .53641 .43394 .53254 Metetra
.647 .739 .916 r
.43394 .53254 .43789 .53641 .44468 .53457 .44074 .53071 Metetra
.642 .738 .919 r
.44074 .53071 .44468 .53457 .45149 .53281 .44756 .52896 Metetra
.637 .738 .921 r
.44756 .52896 .45149 .53281 .45832 .5311 .4544 .52728 Metetra
.64 .741 .922 r
.4544 .52728 .45832 .5311 .46517 .52933 .46126 .52554 Metetra
.659 .748 .916 r
.46126 .52554 .46517 .52933 .47204 .52726 .46815 .52348 Metetra
.682 .755 .907 r
.46815 .52348 .47204 .52726 .47894 .5248 .47505 .52097 Metetra
.691 .756 .901 r
.47505 .52097 .47894 .5248 .48585 .52215 .48198 .51827 Metetra
.689 .753 .9 r
.48198 .51827 .48585 .52215 .49278 .51954 .48892 .51561 Metetra
.683 .75 .902 r
.48892 .51561 .49278 .51954 .49973 .51701 .49588 .51306 Metetra
.679 .748 .903 r
.49588 .51306 .49973 .51701 .50669 .51457 .50285 .51059 Metetra
.675 .746 .904 r
.50285 .51059 .50669 .51457 .51366 .51219 .50984 .50819 Metetra
.672 .745 .905 r
.50984 .50819 .51366 .51219 .52065 .50985 .51684 .50584 Metetra
.67 .744 .906 r
.51684 .50584 .52065 .50985 .52766 .50754 .52386 .50352 Metetra
.669 .744 .906 r
.52386 .50352 .52766 .50754 .53469 .50526 .5309 .50123 Metetra
.667 .743 .907 r
.5309 .50123 .53469 .50526 .54174 .50301 .53796 .49896 Metetra
.665 .742 .908 r
.53796 .49896 .54174 .50301 .5488 .50077 .54503 .49671 Metetra
.664 .742 .908 r
.54503 .49671 .5488 .50077 .55588 .49857 .55213 .4945 Metetra
.662 .742 .909 r
.55213 .4945 .55588 .49857 .56298 .49638 .55924 .49231 Metetra
.663 .742 .909 r
.55924 .49231 .56298 .49638 .5701 .49417 .56637 .4901 Metetra
.666 .744 .908 r
.56637 .4901 .5701 .49417 .57723 .49189 .57351 .48781 Metetra
.67 .745 .907 r
.57351 .48781 .57723 .49189 .58438 .48955 .58068 .48545 Metetra
.671 .745 .906 r
.58068 .48545 .58438 .48955 .59155 .48719 .58786 .48308 Metetra
.669 .744 .906 r
.58786 .48308 .59155 .48719 .59874 .48484 .59506 .48072 Metetra
.666 .743 .908 r
.59506 .48072 .59874 .48484 .60595 .48255 .60228 .47842 Metetra
.659 .741 .91 r
.60228 .47842 .60595 .48255 .61318 .48038 .60952 .47624 Metetra
.653 .739 .913 r
.60952 .47624 .61318 .48038 .62043 .47832 .61679 .47417 Metetra
.647 .737 .915 r
.61679 .47417 .62043 .47832 .62771 .47635 .62408 .4722 Metetra
.639 .735 .918 r
.62408 .4722 .62771 .47635 .635 .47451 .63139 .47037 Metetra
.629 .733 .922 r
.63139 .47037 .635 .47451 .64233 .47281 .63873 .4687 Metetra
.622 .732 .925 r
.63873 .4687 .64233 .47281 .64968 .4712 .64609 .46713 Metetra
.631 .738 .925 r
.64609 .46713 .64968 .4712 .65705 .46945 .65347 .46541 Metetra
.652 .747 .92 r
.65347 .46541 .65705 .46945 .66442 .46738 .66086 .46332 Metetra
.665 .752 .915 r
.66086 .46332 .66442 .46738 .67181 .46507 .66826 .46097 Metetra
.669 .751 .912 r
.66826 .46097 .67181 .46507 .67921 .46268 .67568 .45856 Metetra
.668 .75 .912 r
.67568 .45856 .67921 .46268 .68663 .4603 .68311 .45615 Metetra
.668 .749 .911 r
.68311 .45615 .68663 .4603 .69407 .45791 .69056 .45374 Metetra
.67 .75 .91 r
.69056 .45374 .69407 .45791 .70153 .45547 .69803 .45128 Metetra
.673 .75 .909 r
.69803 .45128 .70153 .45547 .709 .45298 .70552 .44875 Metetra
.664 .745 .911 r
.70552 .44875 .709 .45298 .71651 .45067 .71304 .4464 Metetra
.63 .732 .921 r
.71304 .4464 .71651 .45067 .72406 .44888 .72061 .44466 Metetra
.604 .724 .93 r
.72061 .44466 .72406 .44888 .73167 .44747 .72823 .44335 Metetra
.597 .727 .936 r
.72823 .44335 .73167 .44747 .7393 .44611 .73588 .44212 Metetra
.62 .744 .935 r
.73588 .44212 .7393 .44611 .74693 .44437 .74354 .44048 Metetra
.681 .773 .92 r
.74354 .44048 .74693 .44437 .75451 .44165 .75114 .43775 Metetra
.732 .788 .897 r
.75114 .43775 .75451 .44165 .76204 .43796 .75868 .43391 Metetra
.738 .782 .888 r
.75868 .43391 .76204 .43796 .76958 .4341 .76622 .42986 Metetra
.718 .768 .893 r
.76622 .42986 .76958 .4341 .77716 .43063 .7738 .42629 Metetra
.696 .756 .899 r
.7738 .42629 .77716 .43063 .78479 .42761 .78144 .42322 Metetra
.681 .749 .903 r
.78144 .42322 .78479 .42761 .79246 .42486 .78913 .42044 Metetra
.673 .746 .905 r
.78913 .42044 .79246 .42486 .80016 .42227 .79685 .41783 Metetra
.669 .744 .906 r
.79685 .41783 .80016 .42227 .80789 .41975 .80459 .41529 Metetra
.667 .743 .907 r
.80459 .41529 .80789 .41975 .81565 .41726 .81236 .41279 Metetra
.666 .742 .907 r
.81236 .41279 .81565 .41726 .82342 .4148 .82015 .41032 Metetra
.665 .742 .907 r
.82015 .41032 .82342 .4148 .83122 .41234 .82797 .40785 Metetra
.665 .742 .908 r
.82797 .40785 .83122 .41234 .83904 .40989 .8358 .40538 Metetra
.664 .742 .908 r
.8358 .40538 .83904 .40989 .84688 .40744 .84366 .40292 Metetra
.664 .741 .908 r
.84366 .40292 .84688 .40744 .85475 .40498 .85154 .40045 Metetra
.664 .741 .908 r
.85154 .40045 .85475 .40498 .86263 .40253 .85944 .39798 Metetra
.664 .741 .908 r
.85944 .39798 .86263 .40253 .87054 .40006 .86736 .3955 Metetra
.664 .741 .908 r
.86736 .3955 .87054 .40006 .87847 .39759 .87531 .39302 Metetra
.664 .741 .908 r
.87531 .39302 .87847 .39759 .88642 .39512 .88327 .39054 Metetra
.664 .741 .908 r
.88327 .39054 .88642 .39512 .89439 .39264 .89126 .38804 Metetra
.664 .741 .908 r
.89126 .38804 .89439 .39264 .90239 .39015 .89927 .38554 Metetra
.664 .741 .908 r
.89927 .38554 .90239 .39015 .9104 .38765 .9073 .38303 Metetra
.664 .741 .908 r
.9073 .38303 .9104 .38765 .91844 .38515 .91536 .38052 Metetra
.664 .741 .908 r
.91536 .38052 .91844 .38515 .9265 .38265 .92343 .378 Metetra
.664 .741 .908 r
.92343 .378 .9265 .38265 .93458 .38013 .93153 .37548 Metetra
.656 .742 .913 r
.34947 .55247 .35357 .55623 .36017 .55431 .35609 .55053 Metetra
.657 .742 .912 r
.35609 .55053 .36017 .55431 .36679 .55236 .36272 .54857 Metetra
.659 .742 .911 r
.36272 .54857 .36679 .55236 .37343 .55038 .36937 .54657 Metetra
.659 .742 .911 r
.36937 .54657 .37343 .55038 .38009 .54838 .37603 .54456 Metetra
.66 .742 .911 r
.37603 .54456 .38009 .54838 .38676 .54637 .38272 .54253 Metetra
.659 .741 .911 r
.38272 .54253 .38676 .54637 .39345 .54435 .38942 .54051 Metetra
.659 .741 .911 r
.38942 .54051 .39345 .54435 .40015 .54234 .39613 .53849 Metetra
.658 .741 .911 r
.39613 .53849 .40015 .54234 .40688 .54034 .40287 .53647 Metetra
.657 .741 .912 r
.40287 .53647 .40688 .54034 .41362 .53834 .40962 .53448 Metetra
.656 .741 .912 r
.40962 .53448 .41362 .53834 .42037 .53638 .41638 .5325 Metetra
.653 .74 .913 r
.41638 .5325 .42037 .53638 .42715 .53444 .42317 .53056 Metetra
.65 .74 .915 r
.42317 .53056 .42715 .53444 .43394 .53254 .42997 .52866 Metetra
.646 .739 .917 r
.42997 .52866 .43394 .53254 .44074 .53071 .43678 .52683 Metetra
.64 .738 .919 r
.43678 .52683 .44074 .53071 .44756 .52896 .44361 .5251 Metetra
.635 .737 .922 r
.44361 .5251 .44756 .52896 .4544 .52728 .45046 .52344 Metetra
.637 .74 .923 r
.45046 .52344 .4544 .52728 .46126 .52554 .45733 .52174 Metetra
.658 .749 .917 r
.45733 .52174 .46126 .52554 .46815 .52348 .46423 .51969 Metetra
.684 .756 .906 r
.46423 .51969 .46815 .52348 .47505 .52097 .47115 .51713 Metetra
.694 .757 .9 r
.47115 .51713 .47505 .52097 .48198 .51827 .47809 .51436 Metetra
.69 .753 .9 r
.47809 .51436 .48198 .51827 .48892 .51561 .48504 .51166 Metetra
.684 .75 .901 r
.48504 .51166 .48892 .51561 .49588 .51306 .49201 .50908 Metetra
.679 .748 .903 r
.49201 .50908 .49588 .51306 .50285 .51059 .49899 .50659 Metetra
.675 .746 .904 r
.49899 .50659 .50285 .51059 .50984 .50819 .50599 .50418 Metetra
.673 .745 .905 r
.50599 .50418 .50984 .50819 .51684 .50584 .51301 .50181 Metetra
.671 .744 .906 r
.51301 .50181 .51684 .50584 .52386 .50352 .52004 .49948 Metetra
.669 .744 .906 r
.52004 .49948 .52386 .50352 .5309 .50123 .52709 .49717 Metetra
.667 .743 .907 r
.52709 .49717 .5309 .50123 .53796 .49896 .53416 .49489 Metetra
.665 .742 .908 r
.53416 .49489 .53796 .49896 .54503 .49671 .54125 .49263 Metetra
.663 .742 .908 r
.54125 .49263 .54503 .49671 .55213 .4945 .54835 .49041 Metetra
.662 .741 .909 r
.54835 .49041 .55213 .4945 .55924 .49231 .55548 .48822 Metetra
.662 .742 .909 r
.55548 .48822 .55924 .49231 .56637 .4901 .56262 .48601 Metetra
.666 .744 .908 r
.56262 .48601 .56637 .4901 .57351 .48781 .56978 .48371 Metetra
.67 .745 .907 r
.56978 .48371 .57351 .48781 .58068 .48545 .57696 .48134 Metetra
.671 .745 .906 r
.57696 .48134 .58068 .48545 .58786 .48308 .58415 .47895 Metetra
.669 .744 .907 r
.58415 .47895 .58786 .48308 .59506 .48072 .59136 .47658 Metetra
.666 .743 .908 r
.59136 .47658 .59506 .48072 .60228 .47842 .5986 .47427 Metetra
.659 .741 .91 r
.5986 .47427 .60228 .47842 .60952 .47624 .60585 .47208 Metetra
.653 .739 .913 r
.60585 .47208 .60952 .47624 .61679 .47417 .61313 .47001 Metetra
.647 .737 .915 r
.61313 .47001 .61679 .47417 .62408 .4722 .62043 .46804 Metetra
.638 .735 .918 r
.62043 .46804 .62408 .4722 .63139 .47037 .62775 .46621 Metetra
.627 .732 .923 r
.62775 .46621 .63139 .47037 .63873 .4687 .63511 .46457 Metetra
.618 .731 .927 r
.63511 .46457 .63873 .4687 .64609 .46713 .64249 .46306 Metetra
.628 .738 .926 r
.64249 .46306 .64609 .46713 .65347 .46541 .64988 .46137 Metetra
.652 .748 .92 r
.64988 .46137 .65347 .46541 .66086 .46332 .65729 .45926 Metetra
.667 .753 .914 r
.65729 .45926 .66086 .46332 .66826 .46097 .6647 .45687 Metetra
.67 .752 .912 r
.6647 .45687 .66826 .46097 .67568 .45856 .67212 .45442 Metetra
.669 .75 .912 r
.67212 .45442 .67568 .45856 .68311 .45615 .67957 .45198 Metetra
.668 .75 .912 r
.67957 .45198 .68311 .45615 .69056 .45374 .68704 .44956 Metetra
.67 .75 .91 r
.68704 .44956 .69056 .45374 .69803 .45128 .69452 .44708 Metetra
.675 .751 .908 r
.69452 .44708 .69803 .45128 .70552 .44875 .70202 .4445 Metetra
.665 .746 .91 r
.70202 .4445 .70552 .44875 .71304 .4464 .70955 .44211 Metetra
.627 .73 .922 r
.70955 .44211 .71304 .4464 .72061 .44466 .71714 .44043 Metetra
.596 .721 .932 r
.71714 .44043 .72061 .44466 .72823 .44335 .72478 .43923 Metetra
.587 .724 .938 r
.72478 .43923 .72823 .44335 .73588 .44212 .73246 .43813 Metetra
.611 .742 .939 r
.73246 .43813 .73588 .44212 .74354 .44048 .74013 .43663 Metetra
.681 .776 .923 r
.74013 .43663 .74354 .44048 .75114 .43775 .74775 .4339 Metetra
.739 .793 .896 r
.74775 .4339 .75114 .43775 .75868 .43391 .75529 .42987 Metetra
.746 .786 .886 r
.75529 .42987 .75868 .43391 .76622 .42986 .76284 .42562 Metetra
.722 .77 .891 r
.76284 .42562 .76622 .42986 .7738 .42629 .77043 .42193 Metetra
.697 .757 .898 r
.77043 .42193 .7738 .42629 .78144 .42322 .77809 .4188 Metetra
.682 .75 .902 r
.77809 .4188 .78144 .42322 .78913 .42044 .78578 .416 Metetra
.674 .746 .905 r
.78578 .416 .78913 .42044 .79685 .41783 .79352 .41336 Metetra
.669 .744 .906 r
.79352 .41336 .79685 .41783 .80459 .41529 .80127 .41081 Metetra
.667 .743 .907 r
.80127 .41081 .80459 .41529 .81236 .41279 .80906 .4083 Metetra
.666 .742 .907 r
.80906 .4083 .81236 .41279 .82015 .41032 .81686 .40581 Metetra
.665 .742 .907 r
.81686 .40581 .82015 .41032 .82797 .40785 .82469 .40333 Metetra
.665 .742 .908 r
.82469 .40333 .82797 .40785 .8358 .40538 .83255 .40085 Metetra
.664 .742 .908 r
.83255 .40085 .8358 .40538 .84366 .40292 .84042 .39838 Metetra
.664 .741 .908 r
.84042 .39838 .84366 .40292 .85154 .40045 .84831 .3959 Metetra
.664 .741 .908 r
.84831 .3959 .85154 .40045 .85944 .39798 .85623 .39341 Metetra
.664 .741 .908 r
.85623 .39341 .85944 .39798 .86736 .3955 .86417 .39092 Metetra
.664 .741 .908 r
.86417 .39092 .86736 .3955 .87531 .39302 .87213 .38843 Metetra
.664 .741 .908 r
.87213 .38843 .87531 .39302 .88327 .39054 .88011 .38593 Metetra
.664 .741 .908 r
.88011 .38593 .88327 .39054 .89126 .38804 .88811 .38342 Metetra
.664 .741 .908 r
.88811 .38342 .89126 .38804 .89927 .38554 .89614 .38091 Metetra
.664 .741 .908 r
.89614 .38091 .89927 .38554 .9073 .38303 .90419 .37839 Metetra
.664 .741 .908 r
.90419 .37839 .9073 .38303 .91536 .38052 .91226 .37587 Metetra
.664 .741 .908 r
.91226 .37587 .91536 .38052 .92343 .378 .92035 .37333 Metetra
.664 .741 .908 r
.92035 .37333 .92343 .378 .93153 .37548 .92846 .37079 Metetra
.657 .743 .913 r
.34536 .54871 .34947 .55247 .35609 .55053 .35198 .54674 Metetra
.658 .742 .912 r
.35198 .54674 .35609 .55053 .36272 .54857 .35863 .54476 Metetra
.659 .742 .911 r
.35863 .54476 .36272 .54857 .36937 .54657 .36529 .54274 Metetra
.66 .742 .911 r
.36529 .54274 .36937 .54657 .37603 .54456 .37196 .54072 Metetra
.66 .742 .911 r
.37196 .54072 .37603 .54456 .38272 .54253 .37866 .53868 Metetra
.66 .742 .911 r
.37866 .53868 .38272 .54253 .38942 .54051 .38537 .53665 Metetra
.659 .741 .911 r
.38537 .53665 .38942 .54051 .39613 .53849 .39209 .53461 Metetra
.658 .741 .911 r
.39209 .53461 .39613 .53849 .40287 .53647 .39884 .53259 Metetra
.657 .741 .912 r
.39884 .53259 .40287 .53647 .40962 .53448 .4056 .53059 Metetra
.655 .741 .912 r
.4056 .53059 .40962 .53448 .41638 .5325 .41238 .52861 Metetra
.653 .74 .913 r
.41238 .52861 .41638 .5325 .42317 .53056 .41917 .52666 Metetra
.65 .739 .915 r
.41917 .52666 .42317 .53056 .42997 .52866 .42598 .52476 Metetra
.645 .738 .917 r
.42598 .52476 .42997 .52866 .43678 .52683 .4328 .52294 Metetra
.639 .737 .92 r
.4328 .52294 .43678 .52683 .44361 .5251 .43965 .52122 Metetra
.632 .737 .923 r
.43965 .52122 .44361 .5251 .45046 .52344 .44651 .51959 Metetra
.633 .739 .925 r
.44651 .51959 .45046 .52344 .45733 .52174 .45338 .51794 Metetra
.656 .749 .918 r
.45338 .51794 .45733 .52174 .46423 .51969 .46029 .51589 Metetra
.687 .758 .906 r
.46029 .51589 .46423 .51969 .47115 .51713 .46722 .51328 Metetra
.696 .758 .899 r
.46722 .51328 .47115 .51713 .47809 .51436 .47418 .51045 Metetra
.692 .754 .899 r
.47418 .51045 .47809 .51436 .48504 .51166 .48114 .5077 Metetra
.685 .75 .901 r
.48114 .5077 .48504 .51166 .49201 .50908 .48812 .50509 Metetra
.68 .748 .903 r
.48812 .50509 .49201 .50908 .49899 .50659 .49512 .50258 Metetra
.676 .746 .904 r
.49512 .50258 .49899 .50659 .50599 .50418 .50213 .50014 Metetra
.673 .745 .905 r
.50213 .50014 .50599 .50418 .51301 .50181 .50916 .49776 Metetra
.671 .744 .906 r
.50916 .49776 .51301 .50181 .52004 .49948 .5162 .49541 Metetra
.669 .744 .906 r
.5162 .49541 .52004 .49948 .52709 .49717 .52327 .49309 Metetra
.667 .743 .907 r
.52327 .49309 .52709 .49717 .53416 .49489 .53035 .4908 Metetra
.665 .742 .908 r
.53035 .4908 .53416 .49489 .54125 .49263 .53745 .48854 Metetra
.663 .742 .908 r
.53745 .48854 .54125 .49263 .54835 .49041 .54456 .48631 Metetra
.661 .741 .909 r
.54456 .48631 .54835 .49041 .55548 .48822 .5517 .48411 Metetra
.662 .742 .909 r
.5517 .48411 .55548 .48822 .56262 .48601 .55885 .4819 Metetra
.666 .744 .908 r
.55885 .4819 .56262 .48601 .56978 .48371 .56602 .4796 Metetra
.67 .745 .906 r
.56602 .4796 .56978 .48371 .57696 .48134 .57321 .47721 Metetra
.671 .745 .906 r
.57321 .47721 .57696 .48134 .58415 .47895 .58042 .4748 Metetra
.669 .744 .907 r
.58042 .4748 .58415 .47895 .59136 .47658 .58765 .47242 Metetra
.666 .743 .908 r
.58765 .47242 .59136 .47658 .5986 .47427 .59489 .47011 Metetra
.659 .741 .911 r
.59489 .47011 .5986 .47427 .60585 .47208 .60216 .46791 Metetra
.653 .739 .913 r
.60216 .46791 .60585 .47208 .61313 .47001 .60945 .46582 Metetra
.646 .737 .915 r
.60945 .46582 .61313 .47001 .62043 .46804 .61676 .46385 Metetra
.637 .734 .919 r
.61676 .46385 .62043 .46804 .62775 .46621 .6241 .46204 Metetra
.624 .731 .923 r
.6241 .46204 .62775 .46621 .63511 .46457 .63147 .46043 Metetra
.614 .73 .928 r
.63147 .46043 .63511 .46457 .64249 .46306 .63886 .45898 Metetra
.625 .737 .928 r
.63886 .45898 .64249 .46306 .64988 .46137 .64628 .45733 Metetra
.653 .75 .921 r
.64628 .45733 .64988 .46137 .65729 .45926 .65369 .45519 Metetra
.669 .754 .914 r
.65369 .45519 .65729 .45926 .6647 .45687 .66111 .45275 Metetra
.671 .753 .912 r
.66111 .45275 .6647 .45687 .67212 .45442 .66855 .45026 Metetra
.669 .751 .912 r
.66855 .45026 .67212 .45442 .67957 .45198 .67601 .44781 Metetra
.668 .75 .912 r
.67601 .44781 .67957 .45198 .68704 .44956 .68349 .44536 Metetra
.671 .751 .91 r
.68349 .44536 .68704 .44956 .69452 .44708 .69099 .44286 Metetra
.677 .752 .908 r
.69099 .44286 .69452 .44708 .70202 .4445 .6985 .44023 Metetra
.667 .747 .91 r
.6985 .44023 .70202 .4445 .70955 .44211 .70604 .43779 Metetra
.622 .728 .922 r
.70604 .43779 .70955 .44211 .71714 .44043 .71365 .43617 Metetra
.587 .718 .934 r
.71365 .43617 .71714 .44043 .72478 .43923 .72132 .4351 Metetra
.575 .72 .941 r
.72132 .4351 .72478 .43923 .73246 .43813 .72902 .43416 Metetra
.599 .739 .943 r
.72902 .43416 .73246 .43813 .74013 .43663 .73672 .43282 Metetra
.68 .779 .925 r
.73672 .43282 .74013 .43663 .74775 .4339 .74435 .43009 Metetra
.748 .798 .894 r
.74435 .43009 .74775 .4339 .75529 .42987 .7519 .42585 Metetra
.754 .79 .882 r
.7519 .42585 .75529 .42987 .76284 .42562 .75944 .42138 Metetra
.727 .772 .889 r
.75944 .42138 .76284 .42562 .77043 .42193 .76704 .41756 Metetra
.7 .758 .897 r
.76704 .41756 .77043 .42193 .77809 .4188 .77471 .41436 Metetra
.683 .75 .902 r
.77471 .41436 .77809 .4188 .78578 .416 .78242 .41153 Metetra
.674 .746 .905 r
.78242 .41153 .78578 .416 .79352 .41336 .79017 .40888 Metetra
.67 .744 .906 r
.79017 .40888 .79352 .41336 .80127 .41081 .79794 .40631 Metetra
.667 .743 .907 r
.79794 .40631 .80127 .41081 .80906 .4083 .80574 .40379 Metetra
.666 .742 .907 r
.80574 .40379 .80906 .4083 .81686 .40581 .81356 .40128 Metetra
.665 .742 .907 r
.81356 .40128 .81686 .40581 .82469 .40333 .82141 .39879 Metetra
.665 .742 .908 r
.82141 .39879 .82469 .40333 .83255 .40085 .82927 .3963 Metetra
.664 .742 .908 r
.82927 .3963 .83255 .40085 .84042 .39838 .83716 .39381 Metetra
.664 .741 .908 r
.83716 .39381 .84042 .39838 .84831 .3959 .84507 .39132 Metetra
.664 .741 .908 r
.84507 .39132 .84831 .3959 .85623 .39341 .853 .38882 Metetra
.664 .741 .908 r
.853 .38882 .85623 .39341 .86417 .39092 .86096 .38632 Metetra
.664 .741 .908 r
.86096 .38632 .86417 .39092 .87213 .38843 .86893 .38381 Metetra
.664 .741 .908 r
.86893 .38381 .87213 .38843 .88011 .38593 .87693 .3813 Metetra
.664 .741 .908 r
.87693 .3813 .88011 .38593 .88811 .38342 .88495 .37878 Metetra
.664 .741 .908 r
.88495 .37878 .88811 .38342 .89614 .38091 .89299 .37626 Metetra
.664 .741 .908 r
.89299 .37626 .89614 .38091 .90419 .37839 .90105 .37373 Metetra
.664 .741 .908 r
.90105 .37373 .90419 .37839 .91226 .37587 .90914 .37119 Metetra
.664 .741 .908 r
.90914 .37119 .91226 .37587 .92035 .37333 .91725 .36864 Metetra
.664 .741 .908 r
.91725 .36864 .92035 .37333 .92846 .37079 .92538 .36609 Metetra
.658 .743 .913 r
.34123 .54493 .34536 .54871 .35198 .54674 .34786 .54294 Metetra
.658 .743 .912 r
.34786 .54294 .35198 .54674 .35863 .54476 .35451 .54093 Metetra
.659 .742 .911 r
.35451 .54093 .35863 .54476 .36529 .54274 .36118 .5389 Metetra
.66 .742 .911 r
.36118 .5389 .36529 .54274 .37196 .54072 .36787 .53686 Metetra
.66 .742 .911 r
.36787 .53686 .37196 .54072 .37866 .53868 .37458 .53481 Metetra
.66 .742 .911 r
.37458 .53481 .37866 .53868 .38537 .53665 .3813 .53277 Metetra
.659 .741 .911 r
.3813 .53277 .38537 .53665 .39209 .53461 .38803 .53073 Metetra
.658 .741 .911 r
.38803 .53073 .39209 .53461 .39884 .53259 .39479 .5287 Metetra
.657 .741 .912 r
.39479 .5287 .39884 .53259 .4056 .53059 .40156 .52668 Metetra
.655 .74 .912 r
.40156 .52668 .4056 .53059 .41238 .52861 .40835 .52469 Metetra
.653 .74 .914 r
.40835 .52469 .41238 .52861 .41917 .52666 .41515 .52274 Metetra
.649 .739 .915 r
.41515 .52274 .41917 .52666 .42598 .52476 .42197 .52084 Metetra
.644 .738 .917 r
.42197 .52084 .42598 .52476 .4328 .52294 .42881 .51903 Metetra
.637 .737 .921 r
.42881 .51903 .4328 .52294 .43965 .52122 .43566 .51732 Metetra
.629 .736 .924 r
.43566 .51732 .43965 .52122 .44651 .51959 .44253 .51573 Metetra
.629 .738 .926 r
.44253 .51573 .44651 .51959 .45338 .51794 .44942 .51413 Metetra
.655 .749 .919 r
.44942 .51413 .45338 .51794 .46029 .51589 .45633 .5121 Metetra
.689 .759 .905 r
.45633 .5121 .46029 .51589 .46722 .51328 .46328 .50942 Metetra
.699 .759 .898 r
.46328 .50942 .46722 .51328 .47418 .51045 .47025 .50652 Metetra
.694 .755 .898 r
.47025 .50652 .47418 .51045 .48114 .5077 .47722 .50372 Metetra
.686 .751 .9 r
.47722 .50372 .48114 .5077 .48812 .50509 .48422 .50108 Metetra
.68 .748 .902 r
.48422 .50108 .48812 .50509 .49512 .50258 .49123 .49855 Metetra
.676 .746 .904 r
.49123 .49855 .49512 .50258 .50213 .50014 .49825 .49609 Metetra
.673 .745 .905 r
.49825 .49609 .50213 .50014 .50916 .49776 .50529 .49369 Metetra
.671 .744 .906 r
.50529 .49369 .50916 .49776 .5162 .49541 .51235 .49133 Metetra
.669 .744 .906 r
.51235 .49133 .5162 .49541 .52327 .49309 .51942 .489 Metetra
.667 .743 .907 r
.51942 .489 .52327 .49309 .53035 .4908 .52651 .48669 Metetra
.665 .742 .908 r
.52651 .48669 .53035 .4908 .53745 .48854 .53362 .48442 Metetra
.663 .742 .908 r
.53362 .48442 .53745 .48854 .54456 .48631 .54075 .48218 Metetra
.661 .741 .909 r
.54075 .48218 .54456 .48631 .5517 .48411 .5479 .47998 Metetra
.661 .742 .91 r
.5479 .47998 .5517 .48411 .55885 .4819 .55507 .47777 Metetra
.666 .744 .908 r
.55507 .47777 .55885 .4819 .56602 .4796 .56225 .47546 Metetra
.671 .745 .906 r
.56225 .47546 .56602 .4796 .57321 .47721 .56946 .47305 Metetra
.671 .745 .906 r
.56946 .47305 .57321 .47721 .58042 .4748 .57668 .47063 Metetra
.669 .744 .907 r
.57668 .47063 .58042 .4748 .58765 .47242 .58391 .46824 Metetra
.665 .743 .908 r
.58391 .46824 .58765 .47242 .59489 .47011 .59117 .46592 Metetra
.659 .741 .911 r
.59117 .46592 .59489 .47011 .60216 .46791 .59845 .46371 Metetra
.653 .739 .913 r
.59845 .46371 .60216 .46791 .60945 .46582 .60575 .46162 Metetra
.645 .737 .916 r
.60575 .46162 .60945 .46582 .61676 .46385 .61308 .45965 Metetra
.635 .734 .919 r
.61308 .45965 .61676 .46385 .6241 .46204 .62043 .45785 Metetra
.621 .73 .924 r
.62043 .45785 .6241 .46204 .63147 .46043 .62781 .45628 Metetra
.609 .728 .93 r
.62781 .45628 .63147 .46043 .63886 .45898 .63522 .4549 Metetra
.622 .737 .929 r
.63522 .4549 .63886 .45898 .64628 .45733 .64265 .45329 Metetra
.654 .751 .921 r
.64265 .45329 .64628 .45733 .65369 .45519 .65008 .45112 Metetra
.671 .756 .914 r
.65008 .45112 .65369 .45519 .66111 .45275 .65751 .44862 Metetra
.673 .754 .912 r
.65751 .44862 .66111 .45275 .66855 .45026 .66496 .44609 Metetra
.67 .752 .912 r
.66496 .44609 .66855 .45026 .67601 .44781 .67244 .44361 Metetra
.668 .75 .912 r
.67244 .44361 .67601 .44781 .68349 .44536 .67993 .44115 Metetra
.672 .751 .91 r
.67993 .44115 .68349 .44536 .69099 .44286 .68744 .43862 Metetra
.679 .753 .907 r
.68744 .43862 .69099 .44286 .6985 .44023 .69497 .43593 Metetra
.669 .747 .909 r
.69497 .43593 .6985 .44023 .70604 .43779 .70252 .43344 Metetra
.617 .725 .923 r
.70252 .43344 .70604 .43779 .71365 .43617 .71015 .4319 Metetra
.577 .713 .936 r
.71015 .4319 .71365 .43617 .72132 .4351 .71783 .43097 Metetra
.561 .715 .944 r
.71783 .43097 .72132 .4351 .72902 .43416 .72556 .43021 Metetra
.584 .735 .947 r
.72556 .43021 .72902 .43416 .73672 .43282 .73329 .42906 Metetra
.679 .782 .928 r
.73329 .42906 .73672 .43282 .74435 .43009 .74094 .42633 Metetra
.757 .805 .891 r
.74094 .42633 .74435 .43009 .7519 .42585 .74849 .42185 Metetra
.763 .795 .879 r
.74849 .42185 .7519 .42585 .75944 .42138 .75603 .41712 Metetra
.732 .775 .887 r
.75603 .41712 .75944 .42138 .76704 .41756 .76364 .41316 Metetra
.702 .759 .896 r
.76364 .41316 .76704 .41756 .77471 .41436 .77132 .4099 Metetra
.684 .75 .902 r
.77132 .4099 .77471 .41436 .78242 .41153 .77904 .40704 Metetra
.674 .746 .905 r
.77904 .40704 .78242 .41153 .79017 .40888 .7868 .40437 Metetra
.67 .744 .906 r
.7868 .40437 .79017 .40888 .79794 .40631 .79459 .40179 Metetra
.667 .743 .907 r
.79459 .40179 .79794 .40631 .80574 .40379 .8024 .39925 Metetra
.666 .742 .907 r
.8024 .39925 .80574 .40379 .81356 .40128 .81024 .39673 Metetra
.665 .742 .907 r
.81024 .39673 .81356 .40128 .82141 .39879 .8181 .39423 Metetra
.665 .742 .908 r
.8181 .39423 .82141 .39879 .82927 .3963 .82598 .39173 Metetra
.664 .742 .908 r
.82598 .39173 .82927 .3963 .83716 .39381 .83389 .38922 Metetra
.664 .741 .908 r
.83389 .38922 .83716 .39381 .84507 .39132 .84181 .38672 Metetra
.664 .741 .908 r
.84181 .38672 .84507 .39132 .853 .38882 .84976 .38421 Metetra
.664 .741 .908 r
.84976 .38421 .853 .38882 .86096 .38632 .85773 .3817 Metetra
.664 .741 .908 r
.85773 .3817 .86096 .38632 .86893 .38381 .86572 .37918 Metetra
.664 .741 .908 r
.86572 .37918 .86893 .38381 .87693 .3813 .87373 .37665 Metetra
.664 .741 .908 r
.87373 .37665 .87693 .3813 .88495 .37878 .88177 .37412 Metetra
.664 .741 .908 r
.88177 .37412 .88495 .37878 .89299 .37626 .88983 .37158 Metetra
.664 .741 .908 r
.88983 .37158 .89299 .37626 .90105 .37373 .89791 .36904 Metetra
.664 .741 .908 r
.89791 .36904 .90105 .37373 .90914 .37119 .90601 .36648 Metetra
.664 .741 .908 r
.90601 .36648 .90914 .37119 .91725 .36864 .91413 .36393 Metetra
.664 .741 .908 r
.91413 .36393 .91725 .36864 .92538 .36609 .92228 .36136 Metetra
.659 .744 .913 r
.33708 .54113 .34123 .54493 .34786 .54294 .34372 .53912 Metetra
.659 .743 .912 r
.34372 .53912 .34786 .54294 .35451 .54093 .35038 .53709 Metetra
.66 .742 .911 r
.35038 .53709 .35451 .54093 .36118 .5389 .35706 .53504 Metetra
.66 .742 .911 r
.35706 .53504 .36118 .5389 .36787 .53686 .36376 .53299 Metetra
.66 .742 .91 r
.36376 .53299 .36787 .53686 .37458 .53481 .37048 .53093 Metetra
.66 .742 .911 r
.37048 .53093 .37458 .53481 .3813 .53277 .37721 .52887 Metetra
.659 .741 .911 r
.37721 .52887 .3813 .53277 .38803 .53073 .38396 .52682 Metetra
.658 .741 .911 r
.38396 .52682 .38803 .53073 .39479 .5287 .39072 .52478 Metetra
.657 .741 .912 r
.39072 .52478 .39479 .5287 .40156 .52668 .3975 .52276 Metetra
.655 .74 .913 r
.3975 .52276 .40156 .52668 .40835 .52469 .4043 .52076 Metetra
.652 .74 .914 r
.4043 .52076 .40835 .52469 .41515 .52274 .41112 .51881 Metetra
.649 .739 .915 r
.41112 .51881 .41515 .52274 .42197 .52084 .41795 .51691 Metetra
.643 .738 .918 r
.41795 .51691 .42197 .52084 .42881 .51903 .42479 .51509 Metetra
.635 .736 .921 r
.42479 .51509 .42881 .51903 .43566 .51732 .43165 .51341 Metetra
.626 .735 .926 r
.43165 .51341 .43566 .51732 .44253 .51573 .43853 .51186 Metetra
.624 .737 .928 r
.43853 .51186 .44253 .51573 .44942 .51413 .44543 .51032 Metetra
.653 .749 .92 r
.44543 .51032 .44942 .51413 .45633 .5121 .45236 .50831 Metetra
.692 .761 .904 r
.45236 .50831 .45633 .5121 .46328 .50942 .45932 .50556 Metetra
.703 .76 .897 r
.45932 .50556 .46328 .50942 .47025 .50652 .4663 .50257 Metetra
.696 .755 .897 r
.4663 .50257 .47025 .50652 .47722 .50372 .47329 .49973 Metetra
.687 .751 .9 r
.47329 .49973 .47722 .50372 .48422 .50108 .48029 .49705 Metetra
.681 .748 .902 r
.48029 .49705 .48422 .50108 .49123 .49855 .48731 .49449 Metetra
.677 .747 .904 r
.48731 .49449 .49123 .49855 .49825 .49609 .49435 .49202 Metetra
.673 .745 .905 r
.49435 .49202 .49825 .49609 .50529 .49369 .5014 .48961 Metetra
.671 .744 .906 r
.5014 .48961 .50529 .49369 .51235 .49133 .50847 .48723 Metetra
.669 .744 .906 r
.50847 .48723 .51235 .49133 .51942 .489 .51556 .48488 Metetra
.667 .743 .907 r
.51556 .48488 .51942 .489 .52651 .48669 .52266 .48257 Metetra
.665 .742 .907 r
.52266 .48257 .52651 .48669 .53362 .48442 .52979 .48028 Metetra
.663 .742 .908 r
.52979 .48028 .53362 .48442 .54075 .48218 .53693 .47804 Metetra
.66 .741 .91 r
.53693 .47804 .54075 .48218 .5479 .47998 .54409 .47584 Metetra
.66 .741 .91 r
.54409 .47584 .5479 .47998 .55507 .47777 .55127 .47363 Metetra
.666 .744 .908 r
.55127 .47363 .55507 .47777 .56225 .47546 .55846 .47131 Metetra
.671 .746 .906 r
.55846 .47131 .56225 .47546 .56946 .47305 .56568 .46888 Metetra
.671 .745 .906 r
.56568 .46888 .56946 .47305 .57668 .47063 .57291 .46645 Metetra
.669 .744 .907 r
.57291 .46645 .57668 .47063 .58391 .46824 .58016 .46404 Metetra
.665 .743 .908 r
.58016 .46404 .58391 .46824 .59117 .46592 .58743 .46171 Metetra
.659 .741 .911 r
.58743 .46171 .59117 .46592 .59845 .46371 .59473 .4595 Metetra
.652 .739 .913 r
.59473 .4595 .59845 .46371 .60575 .46162 .60204 .4574 Metetra
.645 .737 .916 r
.60204 .4574 .60575 .46162 .61308 .45965 .60938 .45543 Metetra
.634 .733 .92 r
.60938 .45543 .61308 .45965 .62043 .45785 .61675 .45365 Metetra
.618 .729 .926 r
.61675 .45365 .62043 .45785 .62781 .45628 .62414 .45212 Metetra
.603 .726 .931 r
.62414 .45212 .62781 .45628 .63522 .4549 .63157 .45081 Metetra
.618 .736 .931 r
.63157 .45081 .63522 .4549 .64265 .45329 .63901 .44926 Metetra
.656 .753 .921 r
.63901 .44926 .64265 .45329 .65008 .45112 .64645 .44704 Metetra
.674 .757 .914 r
.64645 .44704 .65008 .45112 .65751 .44862 .65389 .44448 Metetra
.674 .755 .911 r
.65389 .44448 .65751 .44862 .66496 .44609 .66136 .44191 Metetra
.671 .752 .912 r
.66136 .44191 .66496 .44609 .67244 .44361 .66885 .4394 Metetra
.669 .751 .912 r
.66885 .4394 .67244 .44361 .67993 .44115 .67636 .43692 Metetra
.672 .752 .91 r
.67636 .43692 .67993 .44115 .68744 .43862 .68388 .43438 Metetra
.682 .754 .906 r
.68388 .43438 .68744 .43862 .69497 .43593 .69141 .43162 Metetra
.672 .749 .908 r
.69141 .43162 .69497 .43593 .70252 .43344 .69898 .42906 Metetra
.611 .723 .925 r
.69898 .42906 .70252 .43344 .71015 .4319 .70662 .42761 Metetra
.565 .708 .938 r
.70662 .42761 .71015 .4319 .71783 .43097 .71433 .42683 Metetra
.546 .709 .947 r
.71433 .42683 .71783 .43097 .72556 .43021 .72208 .42627 Metetra
.566 .73 .951 r
.72208 .42627 .72556 .43021 .73329 .42906 .72984 .42535 Metetra
.677 .785 .932 r
.72984 .42535 .73329 .42906 .74094 .42633 .73751 .42265 Metetra
.768 .811 .888 r
.73751 .42265 .74094 .42633 .74849 .42185 .74506 .41789 Metetra
.773 .799 .874 r
.74506 .41789 .74849 .42185 .75603 .41712 .7526 .41286 Metetra
.738 .777 .885 r
.7526 .41286 .75603 .41712 .76364 .41316 .76022 .40874 Metetra
.704 .76 .896 r
.76022 .40874 .76364 .41316 .77132 .4099 .76791 .40542 Metetra
.685 .751 .902 r
.76791 .40542 .77132 .4099 .77904 .40704 .77565 .40253 Metetra
.675 .746 .905 r
.77565 .40253 .77904 .40704 .7868 .40437 .78342 .39984 Metetra
.67 .744 .906 r
.78342 .39984 .7868 .40437 .79459 .40179 .79122 .39724 Metetra
.667 .743 .907 r
.79122 .39724 .79459 .40179 .8024 .39925 .79905 .39469 Metetra
.666 .742 .907 r
.79905 .39469 .8024 .39925 .81024 .39673 .80691 .39216 Metetra
.665 .742 .907 r
.80691 .39216 .81024 .39673 .8181 .39423 .81478 .38964 Metetra
.665 .742 .908 r
.81478 .38964 .8181 .39423 .82598 .39173 .82268 .38713 Metetra
.664 .742 .908 r
.82268 .38713 .82598 .39173 .83389 .38922 .8306 .38461 Metetra
.664 .741 .908 r
.8306 .38461 .83389 .38922 .84181 .38672 .83854 .38209 Metetra
.664 .741 .908 r
.83854 .38209 .84181 .38672 .84976 .38421 .8465 .37957 Metetra
.664 .741 .908 r
.8465 .37957 .84976 .38421 .85773 .3817 .85449 .37705 Metetra
.664 .741 .908 r
.85449 .37705 .85773 .3817 .86572 .37918 .86249 .37451 Metetra
.664 .741 .908 r
.86249 .37451 .86572 .37918 .87373 .37665 .87052 .37198 Metetra
.664 .741 .908 r
.87052 .37198 .87373 .37665 .88177 .37412 .87857 .36943 Metetra
.664 .741 .908 r
.87857 .36943 .88177 .37412 .88983 .37158 .88665 .36688 Metetra
.664 .741 .908 r
.88665 .36688 .88983 .37158 .89791 .36904 .89474 .36432 Metetra
.664 .741 .908 r
.89474 .36432 .89791 .36904 .90601 .36648 .90286 .36176 Metetra
.664 .741 .908 r
.90286 .36176 .90601 .36648 .91413 .36393 .911 .35919 Metetra
.664 .741 .908 r
.911 .35919 .91413 .36393 .92228 .36136 .91917 .35661 Metetra
.66 .744 .912 r
.3329 .53733 .33708 .54113 .34372 .53912 .33956 .53528 Metetra
.66 .743 .912 r
.33956 .53528 .34372 .53912 .35038 .53709 .34623 .53323 Metetra
.66 .743 .911 r
.34623 .53323 .35038 .53709 .35706 .53504 .35293 .53116 Metetra
.66 .742 .911 r
.35293 .53116 .35706 .53504 .36376 .53299 .35963 .52909 Metetra
.66 .742 .91 r
.35963 .52909 .36376 .53299 .37048 .53093 .36636 .52702 Metetra
.66 .742 .911 r
.36636 .52702 .37048 .53093 .37721 .52887 .3731 .52496 Metetra
.659 .741 .911 r
.3731 .52496 .37721 .52887 .38396 .52682 .37986 .5229 Metetra
.658 .741 .911 r
.37986 .5229 .38396 .52682 .39072 .52478 .38664 .52085 Metetra
.657 .741 .912 r
.38664 .52085 .39072 .52478 .3975 .52276 .39343 .51882 Metetra
.655 .74 .913 r
.39343 .51882 .3975 .52276 .4043 .52076 .40024 .51682 Metetra
.652 .74 .914 r
.40024 .51682 .4043 .52076 .41112 .51881 .40706 .51485 Metetra
.648 .739 .916 r
.40706 .51485 .41112 .51881 .41795 .51691 .4139 .51295 Metetra
.642 .738 .918 r
.4139 .51295 .41795 .51691 .42479 .51509 .42076 .51115 Metetra
.634 .736 .922 r
.42076 .51115 .42479 .51509 .43165 .51341 .42763 .50948 Metetra
.622 .734 .927 r
.42763 .50948 .43165 .51341 .43853 .51186 .43452 .50797 Metetra
.619 .736 .93 r
.43452 .50797 .43853 .51186 .44543 .51032 .44142 .50651 Metetra
.651 .75 .922 r
.44142 .50651 .44543 .51032 .45236 .50831 .44836 .50452 Metetra
.696 .763 .903 r
.44836 .50452 .45236 .50831 .45932 .50556 .45533 .50169 Metetra
.706 .762 .895 r
.45533 .50169 .45932 .50556 .4663 .50257 .46233 .49861 Metetra
.698 .756 .897 r
.46233 .49861 .4663 .50257 .47329 .49973 .46933 .49571 Metetra
.688 .751 .9 r
.46933 .49571 .47329 .49973 .48029 .49705 .47635 .493 Metetra
.682 .749 .902 r
.47635 .493 .48029 .49705 .48731 .49449 .48338 .49042 Metetra
.677 .747 .903 r
.48338 .49042 .48731 .49449 .49435 .49202 .49043 .48793 Metetra
.674 .745 .905 r
.49043 .48793 .49435 .49202 .5014 .48961 .4975 .4855 Metetra
.671 .744 .905 r
.4975 .4855 .5014 .48961 .50847 .48723 .50458 .48311 Metetra
.669 .744 .906 r
.50458 .48311 .50847 .48723 .51556 .48488 .51168 .48075 Metetra
.668 .743 .907 r
.51168 .48075 .51556 .48488 .52266 .48257 .51879 .47842 Metetra
.666 .742 .907 r
.51879 .47842 .52266 .48257 .52979 .48028 .52593 .47612 Metetra
.663 .742 .909 r
.52593 .47612 .52979 .48028 .53693 .47804 .53308 .47387 Metetra
.66 .741 .91 r
.53308 .47387 .53693 .47804 .54409 .47584 .54025 .47167 Metetra
.659 .741 .91 r
.54025 .47167 .54409 .47584 .55127 .47363 .54745 .46947 Metetra
.666 .744 .909 r
.54745 .46947 .55127 .47363 .55846 .47131 .55466 .46714 Metetra
.672 .746 .906 r
.55466 .46714 .55846 .47131 .56568 .46888 .56188 .4647 Metetra
.672 .746 .906 r
.56188 .4647 .56568 .46888 .57291 .46645 .56913 .46224 Metetra
.669 .744 .907 r
.56913 .46224 .57291 .46645 .58016 .46404 .57639 .45983 Metetra
.665 .743 .908 r
.57639 .45983 .58016 .46404 .58743 .46171 .58368 .45749 Metetra
.659 .741 .911 r
.58368 .45749 .58743 .46171 .59473 .4595 .59098 .45526 Metetra
.652 .739 .913 r
.59098 .45526 .59473 .4595 .60204 .4574 .59831 .45316 Metetra
.644 .737 .916 r
.59831 .45316 .60204 .4574 .60938 .45543 .60566 .45119 Metetra
.632 .733 .92 r
.60566 .45119 .60938 .45543 .61675 .45365 .61304 .44942 Metetra
.614 .728 .927 r
.61304 .44942 .61675 .45365 .62414 .45212 .62045 .44795 Metetra
.597 .724 .933 r
.62045 .44795 .62414 .45212 .63157 .45081 .62789 .44673 Metetra
.613 .736 .933 r
.62789 .44673 .63157 .45081 .63901 .44926 .63535 .44523 Metetra
.658 .755 .922 r
.63535 .44523 .63901 .44926 .64645 .44704 .6428 .44296 Metetra
.677 .759 .913 r
.6428 .44296 .64645 .44704 .65389 .44448 .65026 .44033 Metetra
.676 .756 .911 r
.65026 .44033 .65389 .44448 .66136 .44191 .65774 .43771 Metetra
.671 .753 .912 r
.65774 .43771 .66136 .44191 .66885 .4394 .66524 .43518 Metetra
.669 .751 .912 r
.66524 .43518 .66885 .4394 .67636 .43692 .67276 .43268 Metetra
.673 .752 .91 r
.67276 .43268 .67636 .43692 .68388 .43438 .6803 .43011 Metetra
.684 .756 .905 r
.6803 .43011 .68388 .43438 .69141 .43162 .68784 .42729 Metetra
.676 .75 .907 r
.68784 .42729 .69141 .43162 .69898 .42906 .69542 .42464 Metetra
.604 .719 .926 r
.69542 .42464 .69898 .42906 .70662 .42761 .70308 .42329 Metetra
.552 .703 .94 r
.70308 .42329 .70662 .42761 .71433 .42683 .71081 .42269 Metetra
.527 .701 .95 r
.71081 .42269 .71433 .42683 .72208 .42627 .71859 .42236 Metetra
.544 .722 .956 r
.71859 .42236 .72208 .42627 .72984 .42535 .72638 .42172 Metetra
.673 .789 .936 r
.72638 .42172 .72984 .42535 .73751 .42265 .73407 .41905 Metetra
.78 .819 .884 r
.73407 .41905 .73751 .42265 .74506 .41789 .74162 .41396 Metetra
.784 .804 .869 r
.74162 .41396 .74506 .41789 .7526 .41286 .74916 .4086 Metetra
.744 .78 .882 r
.74916 .4086 .7526 .41286 .76022 .40874 .75678 .40431 Metetra
.706 .761 .895 r
.75678 .40431 .76022 .40874 .76791 .40542 .76448 .40092 Metetra
.685 .751 .901 r
.76448 .40092 .76791 .40542 .77565 .40253 .77223 .39799 Metetra
.675 .746 .904 r
.77223 .39799 .77565 .40253 .78342 .39984 .78002 .39528 Metetra
.67 .744 .906 r
.78002 .39528 .78342 .39984 .79122 .39724 .78784 .39267 Metetra
.667 .743 .907 r
.78784 .39267 .79122 .39724 .79905 .39469 .79568 .3901 Metetra
.666 .742 .907 r
.79568 .3901 .79905 .39469 .80691 .39216 .80355 .38756 Metetra
.665 .742 .907 r
.80355 .38756 .80691 .39216 .81478 .38964 .81144 .38503 Metetra
.665 .742 .908 r
.81144 .38503 .81478 .38964 .82268 .38713 .81935 .38251 Metetra
.664 .742 .908 r
.81935 .38251 .82268 .38713 .8306 .38461 .82729 .37998 Metetra
.664 .742 .908 r
.82729 .37998 .8306 .38461 .83854 .38209 .83525 .37745 Metetra
.664 .741 .908 r
.83525 .37745 .83854 .38209 .8465 .37957 .84322 .37491 Metetra
.664 .741 .908 r
.84322 .37491 .8465 .37957 .85449 .37705 .85123 .37237 Metetra
.664 .741 .908 r
.85123 .37237 .85449 .37705 .86249 .37451 .85925 .36983 Metetra
.664 .741 .908 r
.85925 .36983 .86249 .37451 .87052 .37198 .86729 .36728 Metetra
.664 .741 .908 r
.86729 .36728 .87052 .37198 .87857 .36943 .87536 .36472 Metetra
.664 .741 .908 r
.87536 .36472 .87857 .36943 .88665 .36688 .88345 .36215 Metetra
.664 .741 .908 r
.88345 .36215 .88665 .36688 .89474 .36432 .89156 .35958 Metetra
.664 .741 .908 r
.89156 .35958 .89474 .36432 .90286 .36176 .8997 .35701 Metetra
.664 .741 .908 r
.8997 .35701 .90286 .36176 .911 .35919 .90786 .35442 Metetra
.664 .741 .908 r
.90786 .35442 .911 .35919 .91917 .35661 .91604 .35183 Metetra
.661 .745 .912 r
.32871 .53352 .3329 .53733 .33956 .53528 .33538 .53144 Metetra
.661 .744 .911 r
.33538 .53144 .33956 .53528 .34623 .53323 .34207 .52936 Metetra
.661 .743 .911 r
.34207 .52936 .34623 .53323 .35293 .53116 .34877 .52727 Metetra
.661 .742 .91 r
.34877 .52727 .35293 .53116 .35963 .52909 .35549 .52519 Metetra
.66 .742 .91 r
.35549 .52519 .35963 .52909 .36636 .52702 .36222 .5231 Metetra
.66 .742 .911 r
.36222 .5231 .36636 .52702 .3731 .52496 .36897 .52102 Metetra
.659 .741 .911 r
.36897 .52102 .3731 .52496 .37986 .5229 .37574 .51895 Metetra
.658 .741 .911 r
.37574 .51895 .37986 .5229 .38664 .52085 .38253 .5169 Metetra
.657 .741 .912 r
.38253 .5169 .38664 .52085 .39343 .51882 .38933 .51486 Metetra
.655 .74 .913 r
.38933 .51486 .39343 .51882 .40024 .51682 .39615 .51285 Metetra
.652 .74 .914 r
.39615 .51285 .40024 .51682 .40706 .51485 .40299 .51088 Metetra
.648 .739 .916 r
.40299 .51088 .40706 .51485 .4139 .51295 .40984 .50898 Metetra
.641 .738 .918 r
.40984 .50898 .4139 .51295 .42076 .51115 .41671 .50718 Metetra
.632 .735 .923 r
.41671 .50718 .42076 .51115 .42763 .50948 .42359 .50553 Metetra
.618 .733 .928 r
.42359 .50553 .42763 .50948 .43452 .50797 .43048 .50407 Metetra
.612 .734 .932 r
.43048 .50407 .43452 .50797 .44142 .50651 .43739 .50269 Metetra
.648 .75 .923 r
.43739 .50269 .44142 .50651 .44836 .50452 .44434 .50074 Metetra
.7 .765 .902 r
.44434 .50074 .44836 .50452 .45533 .50169 .45133 .49781 Metetra
.71 .763 .894 r
.45133 .49781 .45533 .50169 .46233 .49861 .45834 .49464 Metetra
.7 .757 .896 r
.45834 .49464 .46233 .49861 .46933 .49571 .46536 .49168 Metetra
.69 .752 .899 r
.46536 .49168 .46933 .49571 .47635 .493 .47239 .48893 Metetra
.682 .749 .902 r
.47239 .48893 .47635 .493 .48338 .49042 .47944 .48633 Metetra
.677 .747 .903 r
.47944 .48633 .48338 .49042 .49043 .48793 .4865 .48382 Metetra
.674 .745 .905 r
.4865 .48382 .49043 .48793 .4975 .4855 .49357 .48137 Metetra
.671 .745 .905 r
.49357 .48137 .4975 .4855 .50458 .48311 .50067 .47897 Metetra
.669 .744 .906 r
.50067 .47897 .50458 .48311 .51168 .48075 .50778 .4766 Metetra
.668 .743 .907 r
.50778 .4766 .51168 .48075 .51879 .47842 .51491 .47425 Metetra
.666 .742 .907 r
.51491 .47425 .51879 .47842 .52593 .47612 .52205 .47194 Metetra
.663 .741 .909 r
.52205 .47194 .52593 .47612 .53308 .47387 .52922 .46968 Metetra
.659 .741 .91 r
.52922 .46968 .53308 .47387 .54025 .47167 .5364 .46748 Metetra
.658 .741 .911 r
.5364 .46748 .54025 .47167 .54745 .46947 .54361 .46529 Metetra
.666 .744 .909 r
.54361 .46529 .54745 .46947 .55466 .46714 .55083 .46296 Metetra
.672 .746 .906 r
.55083 .46296 .55466 .46714 .56188 .4647 .55807 .46049 Metetra
.672 .746 .906 r
.55807 .46049 .56188 .4647 .56913 .46224 .56533 .45802 Metetra
.669 .745 .907 r
.56533 .45802 .56913 .46224 .57639 .45983 .5726 .45559 Metetra
.665 .743 .908 r
.5726 .45559 .57639 .45983 .58368 .45749 .5799 .45324 Metetra
.659 .741 .911 r
.5799 .45324 .58368 .45749 .59098 .45526 .58722 .45101 Metetra
.652 .739 .913 r
.58722 .45101 .59098 .45526 .59831 .45316 .59456 .4489 Metetra
.643 .736 .917 r
.59456 .4489 .59831 .45316 .60566 .45119 .60192 .44693 Metetra
.631 .733 .921 r
.60192 .44693 .60566 .45119 .61304 .44942 .60932 .44518 Metetra
.61 .726 .928 r
.60932 .44518 .61304 .44942 .62045 .44795 .61674 .44376 Metetra
.589 .721 .935 r
.61674 .44376 .62045 .44795 .62789 .44673 .6242 .44264 Metetra
.608 .735 .935 r
.6242 .44264 .62789 .44673 .63535 .44523 .63167 .44121 Metetra
.661 .758 .922 r
.63167 .44121 .63535 .44523 .6428 .44296 .63913 .43888 Metetra
.68 .761 .912 r
.63913 .43888 .6428 .44296 .65026 .44033 .6466 .43617 Metetra
.678 .757 .911 r
.6466 .43617 .65026 .44033 .65774 .43771 .65409 .4335 Metetra
.672 .753 .911 r
.65409 .4335 .65774 .43771 .66524 .43518 .66161 .43094 Metetra
.669 .751 .912 r
.66161 .43094 .66524 .43518 .67276 .43268 .66915 .42843 Metetra
.673 .753 .911 r
.66915 .42843 .67276 .43268 .6803 .43011 .6767 .42584 Metetra
.687 .757 .904 r
.6767 .42584 .6803 .43011 .68784 .42729 .68426 .42294 Metetra
.681 .752 .905 r
.68426 .42294 .68784 .42729 .69542 .42464 .69184 .42018 Metetra
.596 .715 .927 r
.69184 .42018 .69542 .42464 .70308 .42329 .69952 .41896 Metetra
.538 .696 .942 r
.69952 .41896 .70308 .42329 .71081 .42269 .70728 .41855 Metetra
.505 .692 .952 r
.70728 .41855 .71081 .42269 .71859 .42236 .71509 .41848 Metetra
.515 .712 .961 r
.71509 .41848 .71859 .42236 .72638 .42172 .72291 .41816 Metetra
.669 .794 .941 r
.72291 .41816 .72638 .42172 .73407 .41905 .73062 .41555 Metetra
.793 .827 .879 r
.73062 .41555 .73407 .41905 .74162 .41396 .73817 .41009 Metetra
.796 .81 .863 r
.73817 .41009 .74162 .41396 .74916 .4086 .7457 .40433 Metetra
.75 .782 .879 r
.7457 .40433 .74916 .4086 .75678 .40431 .75332 .39985 Metetra
.709 .762 .894 r
.75332 .39985 .75678 .40431 .76448 .40092 .76104 .39639 Metetra
.686 .751 .901 r
.76104 .39639 .76448 .40092 .77223 .39799 .7688 .39344 Metetra
.675 .746 .904 r
.7688 .39344 .77223 .39799 .78002 .39528 .77661 .3907 Metetra
.67 .744 .906 r
.77661 .3907 .78002 .39528 .78784 .39267 .78444 .38808 Metetra
.667 .743 .907 r
.78444 .38808 .78784 .39267 .79568 .3901 .7923 .3855 Metetra
.666 .742 .907 r
.7923 .3855 .79568 .3901 .80355 .38756 .80018 .38294 Metetra
.665 .742 .907 r
.80018 .38294 .80355 .38756 .81144 .38503 .80809 .3804 Metetra
.665 .742 .908 r
.80809 .3804 .81144 .38503 .81935 .38251 .81601 .37786 Metetra
.664 .742 .908 r
.81601 .37786 .81935 .38251 .82729 .37998 .82396 .37532 Metetra
.664 .742 .908 r
.82396 .37532 .82729 .37998 .83525 .37745 .83194 .37278 Metetra
.664 .741 .908 r
.83194 .37278 .83525 .37745 .84322 .37491 .83993 .37023 Metetra
.664 .741 .908 r
.83993 .37023 .84322 .37491 .85123 .37237 .84795 .36768 Metetra
.664 .741 .908 r
.84795 .36768 .85123 .37237 .85925 .36983 .85599 .36512 Metetra
.664 .741 .908 r
.85599 .36512 .85925 .36983 .86729 .36728 .86405 .36255 Metetra
.664 .741 .908 r
.86405 .36255 .86729 .36728 .87536 .36472 .87213 .35998 Metetra
.664 .741 .908 r
.87213 .35998 .87536 .36472 .88345 .36215 .88024 .3574 Metetra
.664 .741 .908 r
.88024 .3574 .88345 .36215 .89156 .35958 .88837 .35482 Metetra
.664 .741 .908 r
.88837 .35482 .89156 .35958 .8997 .35701 .89652 .35223 Metetra
.664 .741 .908 r
.89652 .35223 .8997 .35701 .90786 .35442 .90469 .34963 Metetra
.664 .741 .908 r
.90469 .34963 .90786 .35442 .91604 .35183 .91289 .34703 Metetra
.663 .746 .912 r
.3245 .52969 .32871 .53352 .33538 .53144 .33118 .52757 Metetra
.662 .744 .911 r
.33118 .52757 .33538 .53144 .34207 .52936 .33788 .52546 Metetra
.662 .743 .911 r
.33788 .52546 .34207 .52936 .34877 .52727 .34459 .52336 Metetra
.661 .742 .91 r
.34459 .52336 .34877 .52727 .35549 .52519 .35132 .52126 Metetra
.661 .742 .91 r
.35132 .52126 .35549 .52519 .36222 .5231 .35807 .51916 Metetra
.66 .742 .911 r
.35807 .51916 .36222 .5231 .36897 .52102 .36483 .51707 Metetra
.659 .741 .911 r
.36483 .51707 .36897 .52102 .37574 .51895 .37161 .51499 Metetra
.658 .741 .911 r
.37161 .51499 .37574 .51895 .38253 .5169 .37841 .51293 Metetra
.656 .741 .912 r
.37841 .51293 .38253 .5169 .38933 .51486 .38522 .51088 Metetra
.654 .74 .913 r
.38522 .51088 .38933 .51486 .39615 .51285 .39205 .50886 Metetra
.652 .74 .914 r
.39205 .50886 .39615 .51285 .40299 .51088 .39889 .50689 Metetra
.647 .739 .916 r
.39889 .50689 .40299 .51088 .40984 .50898 .40576 .50499 Metetra
.64 .737 .919 r
.40576 .50499 .40984 .50898 .41671 .50718 .41263 .5032 Metetra
.63 .735 .923 r
.41263 .5032 .41671 .50718 .42359 .50553 .41952 .50157 Metetra
.614 .732 .929 r
.41952 .50157 .42359 .50553 .43048 .50407 .42643 .50016 Metetra
.605 .732 .935 r
.42643 .50016 .43048 .50407 .43739 .50269 .43335 .49888 Metetra
.645 .75 .925 r
.43335 .49888 .43739 .50269 .44434 .50074 .44031 .49697 Metetra
.705 .768 .901 r
.44031 .49697 .44434 .50074 .45133 .49781 .44731 .49393 Metetra
.714 .765 .892 r
.44731 .49393 .45133 .49781 .45834 .49464 .45434 .49065 Metetra
.702 .757 .895 r
.45434 .49065 .45834 .49464 .46536 .49168 .46137 .48763 Metetra
.691 .752 .899 r
.46137 .48763 .46536 .49168 .47239 .48893 .46841 .48484 Metetra
.683 .749 .901 r
.46841 .48484 .47239 .48893 .47944 .48633 .47547 .48222 Metetra
.678 .747 .903 r
.47547 .48222 .47944 .48633 .4865 .48382 .48254 .47969 Metetra
.674 .746 .904 r
.48254 .47969 .4865 .48382 .49357 .48137 .48963 .47723 Metetra
.672 .745 .905 r
.48963 .47723 .49357 .48137 .50067 .47897 .49673 .47481 Metetra
.67 .744 .906 r
.49673 .47481 .50067 .47897 .50778 .4766 .50386 .47242 Metetra
.668 .743 .907 r
.50386 .47242 .50778 .4766 .51491 .47425 .511 .47006 Metetra
.666 .742 .907 r
.511 .47006 .51491 .47425 .52205 .47194 .51816 .46774 Metetra
.662 .741 .909 r
.51816 .46774 .52205 .47194 .52922 .46968 .52534 .46548 Metetra
.658 .74 .91 r
.52534 .46548 .52922 .46968 .5364 .46748 .53253 .46328 Metetra
.657 .741 .911 r
.53253 .46328 .5364 .46748 .54361 .46529 .53975 .4611 Metetra
.665 .744 .909 r
.53975 .4611 .54361 .46529 .55083 .46296 .54698 .45876 Metetra
.673 .747 .906 r
.54698 .45876 .55083 .46296 .55807 .46049 .55424 .45627 Metetra
.672 .746 .906 r
.55424 .45627 .55807 .46049 .56533 .45802 .56151 .45377 Metetra
.669 .745 .907 r
.56151 .45377 .56533 .45802 .5726 .45559 .5688 .45134 Metetra
.665 .743 .908 r
.5688 .45134 .5726 .45559 .5799 .45324 .57611 .44898 Metetra
.659 .741 .911 r
.57611 .44898 .5799 .45324 .58722 .45101 .58344 .44673 Metetra
.652 .739 .914 r
.58344 .44673 .58722 .45101 .59456 .4489 .59079 .44462 Metetra
.643 .736 .917 r
.59079 .44462 .59456 .4489 .60192 .44693 .59817 .44266 Metetra
.629 .732 .922 r
.59817 .44266 .60192 .44693 .60932 .44518 .60557 .44093 Metetra
.606 .725 .929 r
.60557 .44093 .60932 .44518 .61674 .44376 .61301 .43956 Metetra
.58 .718 .938 r
.61301 .43956 .61674 .44376 .6242 .44264 .62048 .43857 Metetra
.602 .734 .938 r
.62048 .43857 .6242 .44264 .63167 .44121 .62797 .43721 Metetra
.664 .761 .922 r
.62797 .43721 .63167 .44121 .63913 .43888 .63545 .43481 Metetra
.685 .764 .912 r
.63545 .43481 .63913 .43888 .6466 .43617 .64293 .432 Metetra
.68 .758 .91 r
.64293 .432 .6466 .43617 .65409 .4335 .65044 .42927 Metetra
.673 .754 .911 r
.65044 .42927 .65409 .4335 .66161 .43094 .65796 .42668 Metetra
.669 .752 .912 r
.65796 .42668 .66161 .43094 .66915 .42843 .66551 .42416 Metetra
.673 .753 .911 r
.66551 .42416 .66915 .42843 .6767 .42584 .67308 .42155 Metetra
.691 .758 .903 r
.67308 .42155 .6767 .42584 .68426 .42294 .68065 .41857 Metetra
.686 .754 .903 r
.68065 .41857 .68426 .42294 .69184 .42018 .68824 .41568 Metetra
.586 .709 .928 r
.68824 .41568 .69184 .42018 .69952 .41896 .69595 .41462 Metetra
.521 .689 .944 r
.69595 .41462 .69952 .41896 .70728 .41855 .70373 .41442 Metetra
.479 .681 .954 r
.70373 .41442 .70728 .41855 .71509 .41848 .71157 .41462 Metetra
.477 .697 .965 r
.71157 .41462 .71509 .41848 .72291 .41816 .71943 .41471 Metetra
.662 .798 .948 r
.71943 .41471 .72291 .41816 .73062 .41555 .72716 .41218 Metetra
.808 .837 .872 r
.72716 .41218 .73062 .41555 .73817 .41009 .7347 .40627 Metetra
.81 .815 .855 r
.7347 .40627 .73817 .41009 .7457 .40433 .74222 .40007 Metetra
.757 .785 .876 r
.74222 .40007 .7457 .40433 .75332 .39985 .74985 .39538 Metetra
.711 .763 .893 r
.74985 .39538 .75332 .39985 .76104 .39639 .75757 .39185 Metetra
.687 .752 .901 r
.75757 .39185 .76104 .39639 .7688 .39344 .76536 .38886 Metetra
.676 .747 .904 r
.76536 .38886 .7688 .39344 .77661 .3907 .77317 .3861 Metetra
.67 .744 .906 r
.77317 .3861 .77661 .3907 .78444 .38808 .78102 .38346 Metetra
.667 .743 .907 r
.78102 .38346 .78444 .38808 .7923 .3855 .7889 .38087 Metetra
.666 .742 .907 r
.7889 .38087 .7923 .3855 .80018 .38294 .79679 .3783 Metetra
.665 .742 .907 r
.79679 .3783 .80018 .38294 .80809 .3804 .80472 .37574 Metetra
.665 .742 .908 r
.80472 .37574 .80809 .3804 .81601 .37786 .81266 .37319 Metetra
.664 .742 .908 r
.81266 .37319 .81601 .37786 .82396 .37532 .82062 .37064 Metetra
.664 .742 .908 r
.82062 .37064 .82396 .37532 .83194 .37278 .82861 .36808 Metetra
.664 .741 .908 r
.82861 .36808 .83194 .37278 .83993 .37023 .83662 .36552 Metetra
.664 .741 .908 r
.83662 .36552 .83993 .37023 .84795 .36768 .84465 .36295 Metetra
.664 .741 .908 r
.84465 .36295 .84795 .36768 .85599 .36512 .85271 .36038 Metetra
.664 .741 .908 r
.85271 .36038 .85599 .36512 .86405 .36255 .86079 .35781 Metetra
.664 .741 .908 r
.86079 .35781 .86405 .36255 .87213 .35998 .86889 .35522 Metetra
.664 .741 .908 r
.86889 .35522 .87213 .35998 .88024 .3574 .87701 .35263 Metetra
.664 .741 .908 r
.87701 .35263 .88024 .3574 .88837 .35482 .88515 .35003 Metetra
.664 .741 .908 r
.88515 .35003 .88837 .35482 .89652 .35223 .89332 .34743 Metetra
.664 .741 .908 r
.89332 .34743 .89652 .35223 .90469 .34963 .90151 .34482 Metetra
.664 .741 .908 r
.90151 .34482 .90469 .34963 .91289 .34703 .90973 .3422 Metetra
.665 .747 .911 r
.32027 .52586 .3245 .52969 .33118 .52757 .32696 .5237 Metetra
.663 .745 .911 r
.32696 .5237 .33118 .52757 .33788 .52546 .33367 .52156 Metetra
.662 .743 .91 r
.33367 .52156 .33788 .52546 .34459 .52336 .34039 .51943 Metetra
.661 .743 .91 r
.34039 .51943 .34459 .52336 .35132 .52126 .34713 .51731 Metetra
.661 .742 .91 r
.34713 .51731 .35132 .52126 .35807 .51916 .35389 .5152 Metetra
.66 .742 .911 r
.35389 .5152 .35807 .51916 .36483 .51707 .36066 .5131 Metetra
.659 .741 .911 r
.36066 .5131 .36483 .51707 .37161 .51499 .36745 .51101 Metetra
.658 .741 .911 r
.36745 .51101 .37161 .51499 .37841 .51293 .37426 .50894 Metetra
.656 .741 .912 r
.37426 .50894 .37841 .51293 .38522 .51088 .38108 .50688 Metetra
.654 .74 .913 r
.38108 .50688 .38522 .51088 .39205 .50886 .38792 .50486 Metetra
.651 .74 .914 r
.38792 .50486 .39205 .50886 .39889 .50689 .39478 .50289 Metetra
.647 .739 .916 r
.39478 .50289 .39889 .50689 .40576 .50499 .40165 .50098 Metetra
.64 .737 .919 r
.40165 .50098 .40576 .50499 .41263 .5032 .40854 .49919 Metetra
.628 .734 .924 r
.40854 .49919 .41263 .5032 .41952 .50157 .41544 .49758 Metetra
.609 .73 .931 r
.41544 .49758 .41952 .50157 .42643 .50016 .42235 .49624 Metetra
.596 .729 .937 r
.42235 .49624 .42643 .50016 .43335 .49888 .42928 .49507 Metetra
.641 .75 .928 r
.42928 .49507 .43335 .49888 .44031 .49697 .43625 .49321 Metetra
.71 .77 .899 r
.43625 .49321 .44031 .49697 .44731 .49393 .44327 .49004 Metetra
.719 .766 .89 r
.44327 .49004 .44731 .49393 .45434 .49065 .45031 .48664 Metetra
.705 .758 .894 r
.45031 .48664 .45434 .49065 .46137 .48763 .45736 .48356 Metetra
.692 .752 .898 r
.45736 .48356 .46137 .48763 .46841 .48484 .46441 .48074 Metetra
.683 .749 .901 r
.46441 .48074 .46841 .48484 .47547 .48222 .47148 .47809 Metetra
.678 .747 .903 r
.47148 .47809 .47547 .48222 .48254 .47969 .47857 .47554 Metetra
.674 .746 .904 r
.47857 .47554 .48254 .47969 .48963 .47723 .48567 .47306 Metetra
.672 .745 .905 r
.48567 .47306 .48963 .47723 .49673 .47481 .49278 .47063 Metetra
.67 .744 .906 r
.49278 .47063 .49673 .47481 .50386 .47242 .49992 .46823 Metetra
.668 .743 .907 r
.49992 .46823 .50386 .47242 .511 .47006 .50707 .46585 Metetra
.666 .742 .907 r
.50707 .46585 .511 .47006 .51816 .46774 .51424 .46352 Metetra
.662 .741 .909 r
.51424 .46352 .51816 .46774 .52534 .46548 .52143 .46125 Metetra
.658 .74 .911 r
.52143 .46125 .52534 .46548 .53253 .46328 .52864 .45905 Metetra
.656 .74 .912 r
.52864 .45905 .53253 .46328 .53975 .4611 .53587 .45689 Metetra
.665 .744 .909 r
.53587 .45689 .53975 .4611 .54698 .45876 .54312 .45454 Metetra
.674 .747 .906 r
.54312 .45454 .54698 .45876 .55424 .45627 .55039 .45202 Metetra
.673 .746 .906 r
.55039 .45202 .55424 .45627 .56151 .45377 .55767 .44951 Metetra
.669 .745 .907 r
.55767 .44951 .56151 .45377 .5688 .45134 .56497 .44706 Metetra
.665 .743 .909 r
.56497 .44706 .5688 .45134 .57611 .44898 .57229 .4447 Metetra
.659 .741 .911 r
.57229 .4447 .57611 .44898 .58344 .44673 .57964 .44244 Metetra
.651 .739 .914 r
.57964 .44244 .58344 .44673 .59079 .44462 .587 .44032 Metetra
.642 .736 .917 r
.587 .44032 .59079 .44462 .59817 .44266 .5944 .43836 Metetra
.627 .731 .922 r
.5944 .43836 .59817 .44266 .60557 .44093 .60181 .43665 Metetra
.601 .723 .931 r
.60181 .43665 .60557 .44093 .61301 .43956 .60927 .43535 Metetra
.569 .714 .94 r
.60927 .43535 .61301 .43956 .62048 .43857 .61675 .4345 Metetra
.594 .733 .94 r
.61675 .4345 .62048 .43857 .62797 .43721 .62426 .43323 Metetra
.669 .764 .922 r
.62426 .43323 .62797 .43721 .63545 .43481 .63175 .43073 Metetra
.689 .766 .91 r
.63175 .43073 .63545 .43481 .64293 .432 .63924 .42781 Metetra
.683 .76 .91 r
.63924 .42781 .64293 .432 .65044 .42927 .64676 .42503 Metetra
.674 .754 .911 r
.64676 .42503 .65044 .42927 .65796 .42668 .6543 .42241 Metetra
.669 .752 .912 r
.6543 .42241 .65796 .42668 .66551 .42416 .66186 .41987 Metetra
.673 .753 .911 r
.66186 .41987 .66551 .42416 .67308 .42155 .66945 .41725 Metetra
.694 .76 .902 r
.66945 .41725 .67308 .42155 .68065 .41857 .67703 .41419 Metetra
.693 .756 .9 r
.67703 .41419 .68065 .41857 .68824 .41568 .68463 .41113 Metetra
.573 .703 .93 r
.68463 .41113 .68824 .41568 .69595 .41462 .69235 .41026 Metetra
.502 .68 .946 r
.69235 .41026 .69595 .41462 .70373 .41442 .70016 .41028 Metetra
.449 .667 .956 r
.70016 .41028 .70373 .41442 .71157 .41462 .70803 .41081 Metetra
.428 .676 .966 r
.70803 .41081 .71157 .41462 .71943 .41471 .71594 .41137 Metetra
.652 .803 .955 r
.71594 .41137 .71943 .41471 .72716 .41218 .7237 .40896 Metetra
.825 .847 .864 r
.7237 .40896 .72716 .41218 .7347 .40627 .73122 .40253 Metetra
.824 .821 .846 r
.73122 .40253 .7347 .40627 .74222 .40007 .73873 .3958 Metetra
.765 .788 .872 r
.73873 .3958 .74222 .40007 .74985 .39538 .74636 .39089 Metetra
.714 .764 .891 r
.74636 .39089 .74985 .39538 .75757 .39185 .7541 .38727 Metetra
.688 .752 .9 r
.7541 .38727 .75757 .39185 .76536 .38886 .76189 .38425 Metetra
.676 .747 .904 r
.76189 .38425 .76536 .38886 .77317 .3861 .76972 .38148 Metetra
.67 .744 .906 r
.76972 .38148 .77317 .3861 .78102 .38346 .77759 .37882 Metetra
.668 .743 .907 r
.77759 .37882 .78102 .38346 .7889 .38087 .78548 .37622 Metetra
.666 .742 .907 r
.78548 .37622 .7889 .38087 .79679 .3783 .79339 .37363 Metetra
.665 .742 .907 r
.79339 .37363 .79679 .3783 .80472 .37574 .80133 .37106 Metetra
.665 .742 .908 r
.80133 .37106 .80472 .37574 .81266 .37319 .80928 .3685 Metetra
.664 .742 .908 r
.80928 .3685 .81266 .37319 .82062 .37064 .81727 .36593 Metetra
.664 .742 .908 r
.81727 .36593 .82062 .37064 .82861 .36808 .82527 .36336 Metetra
.664 .741 .908 r
.82527 .36336 .82861 .36808 .83662 .36552 .8333 .36079 Metetra
.664 .741 .908 r
.8333 .36079 .83662 .36552 .84465 .36295 .84134 .35821 Metetra
.664 .741 .908 r
.84134 .35821 .84465 .36295 .85271 .36038 .84942 .35562 Metetra
.664 .741 .908 r
.84942 .35562 .85271 .36038 .86079 .35781 .85751 .35303 Metetra
.664 .741 .908 r
.85751 .35303 .86079 .35781 .86889 .35522 .86563 .35044 Metetra
.664 .741 .908 r
.86563 .35044 .86889 .35522 .87701 .35263 .87376 .34783 Metetra
.664 .741 .908 r
.87376 .34783 .87701 .35263 .88515 .35003 .88193 .34522 Metetra
.664 .741 .908 r
.88193 .34522 .88515 .35003 .89332 .34743 .89011 .3426 Metetra
.664 .741 .908 r
.89011 .3426 .89332 .34743 .90151 .34482 .89832 .33998 Metetra
.664 .741 .908 r
.89832 .33998 .90151 .34482 .90973 .3422 .90655 .33735 Metetra
.667 .748 .911 r
.31601 .52202 .32027 .52586 .32696 .5237 .32272 .51981 Metetra
.665 .745 .91 r
.32272 .51981 .32696 .5237 .33367 .52156 .32944 .51763 Metetra
.663 .744 .91 r
.32944 .51763 .33367 .52156 .34039 .51943 .33617 .51549 Metetra
.662 .743 .91 r
.33617 .51549 .34039 .51943 .34713 .51731 .34293 .51335 Metetra
.661 .742 .91 r
.34293 .51335 .34713 .51731 .35389 .5152 .34969 .51123 Metetra
.66 .742 .911 r
.34969 .51123 .35389 .5152 .36066 .5131 .35648 .50912 Metetra
.659 .741 .911 r
.35648 .50912 .36066 .5131 .36745 .51101 .36328 .50701 Metetra
.658 .741 .911 r
.36328 .50701 .36745 .51101 .37426 .50894 .3701 .50493 Metetra
.656 .741 .912 r
.3701 .50493 .37426 .50894 .38108 .50688 .37693 .50287 Metetra
.654 .74 .913 r
.37693 .50287 .38108 .50688 .38792 .50486 .38378 .50084 Metetra
.651 .74 .914 r
.38378 .50084 .38792 .50486 .39478 .50289 .39065 .49886 Metetra
.646 .738 .916 r
.39065 .49886 .39478 .50289 .40165 .50098 .39753 .49695 Metetra
.639 .737 .92 r
.39753 .49695 .40165 .50098 .40854 .49919 .40443 .49517 Metetra
.626 .734 .925 r
.40443 .49517 .40854 .49919 .41544 .49758 .41134 .49359 Metetra
.604 .728 .932 r
.41134 .49359 .41544 .49758 .42235 .49624 .41826 .4923 Metetra
.585 .726 .94 r
.41826 .4923 .42235 .49624 .42928 .49507 .42519 .49127 Metetra
.636 .75 .931 r
.42519 .49127 .42928 .49507 .43625 .49321 .43217 .48948 Metetra
.717 .774 .897 r
.43217 .48948 .43625 .49321 .44327 .49004 .43921 .48615 Metetra
.724 .768 .888 r
.43921 .48615 .44327 .49004 .45031 .48664 .44626 .48262 Metetra
.707 .759 .892 r
.44626 .48262 .45031 .48664 .45736 .48356 .45332 .47947 Metetra
.693 .753 .897 r
.45332 .47947 .45736 .48356 .46441 .48074 .46039 .47661 Metetra
.684 .749 .901 r
.46039 .47661 .46441 .48074 .47148 .47809 .46748 .47394 Metetra
.678 .747 .903 r
.46748 .47394 .47148 .47809 .47857 .47554 .47457 .47137 Metetra
.675 .746 .904 r
.47457 .47137 .47857 .47554 .48567 .47306 .48169 .46888 Metetra
.672 .745 .905 r
.48169 .46888 .48567 .47306 .49278 .47063 .48882 .46643 Metetra
.67 .744 .906 r
.48882 .46643 .49278 .47063 .49992 .46823 .49596 .46401 Metetra
.668 .743 .906 r
.49596 .46401 .49992 .46823 .50707 .46585 .50313 .46163 Metetra
.666 .742 .907 r
.50313 .46163 .50707 .46585 .51424 .46352 .51031 .45928 Metetra
.662 .741 .909 r
.51031 .45928 .51424 .46352 .52143 .46125 .51751 .457 Metetra
.657 .74 .911 r
.51751 .457 .52143 .46125 .52864 .45905 .52474 .45481 Metetra
.654 .74 .912 r
.52474 .45481 .52864 .45905 .53587 .45689 .53198 .45266 Metetra
.665 .744 .91 r
.53198 .45266 .53587 .45689 .54312 .45454 .53924 .45031 Metetra
.675 .748 .906 r
.53924 .45031 .54312 .45454 .55039 .45202 .54652 .44776 Metetra
.673 .746 .906 r
.54652 .44776 .55039 .45202 .55767 .44951 .55381 .44523 Metetra
.669 .745 .907 r
.55381 .44523 .55767 .44951 .56497 .44706 .56113 .44277 Metetra
.665 .743 .909 r
.56113 .44277 .56497 .44706 .57229 .4447 .56846 .44039 Metetra
.659 .741 .911 r
.56846 .44039 .57229 .4447 .57964 .44244 .57582 .43813 Metetra
.651 .739 .914 r
.57582 .43813 .57964 .44244 .587 .44032 .5832 .436 Metetra
.641 .736 .917 r
.5832 .436 .587 .44032 .5944 .43836 .5906 .43404 Metetra
.625 .731 .923 r
.5906 .43404 .5944 .43836 .60181 .43665 .59803 .43236 Metetra
.595 .721 .932 r
.59803 .43236 .60181 .43665 .60927 .43535 .6055 .43113 Metetra
.556 .709 .943 r
.6055 .43113 .60927 .43535 .61675 .4345 .61301 .43044 Metetra
.586 .731 .944 r
.61301 .43044 .61675 .4345 .62426 .43323 .62053 .42928 Metetra
.675 .769 .922 r
.62053 .42928 .62426 .43323 .63175 .43073 .62803 .42665 Metetra
.694 .769 .909 r
.62803 .42665 .63175 .43073 .63924 .42781 .63553 .42362 Metetra
.685 .761 .909 r
.63553 .42362 .63924 .42781 .64676 .42503 .64306 .42077 Metetra
.675 .755 .911 r
.64306 .42077 .64676 .42503 .6543 .42241 .65062 .41812 Metetra
.669 .752 .913 r
.65062 .41812 .6543 .42241 .66186 .41987 .6582 .41557 Metetra
.673 .754 .911 r
.6582 .41557 .66186 .41987 .66945 .41725 .66579 .41294 Metetra
.697 .762 .901 r
.66579 .41294 .66945 .41725 .67703 .41419 .67338 .40979 Metetra
.701 .759 .897 r
.67338 .40979 .67703 .41419 .68463 .41113 .68099 .40654 Metetra
.558 .695 .931 r
.68099 .40654 .68463 .41113 .69235 .41026 .68874 .40588 Metetra
.48 .67 .947 r
.68874 .40588 .69235 .41026 .70016 .41028 .69658 .40616 Metetra
.414 .651 .956 r
.69658 .40616 .70016 .41028 .70803 .41081 .70448 .40703 Metetra
.363 .644 .963 r
.70448 .40703 .70803 .41081 .71594 .41137 .71243 .40817 Metetra
.637 .808 .965 r
.71243 .40817 .71594 .41137 .7237 .40896 .72022 .40593 Metetra
.844 .857 .853 r
.72022 .40593 .7237 .40896 .73122 .40253 .72773 .39887 Metetra
.84 .827 .835 r
.72773 .39887 .73122 .40253 .73873 .3958 .73522 .39154 Metetra
.773 .791 .868 r
.73522 .39154 .73873 .3958 .74636 .39089 .74285 .38637 Metetra
.717 .765 .89 r
.74285 .38637 .74636 .39089 .7541 .38727 .7506 .38268 Metetra
.689 .752 .9 r
.7506 .38268 .7541 .38727 .76189 .38425 .75841 .37962 Metetra
.676 .747 .904 r
.75841 .37962 .76189 .38425 .76972 .38148 .76626 .37683 Metetra
.671 .744 .906 r
.76626 .37683 .76972 .38148 .77759 .37882 .77414 .37416 Metetra
.668 .743 .907 r
.77414 .37416 .77759 .37882 .78548 .37622 .78204 .37154 Metetra
.666 .742 .907 r
.78204 .37154 .78548 .37622 .79339 .37363 .78997 .36894 Metetra
.665 .742 .907 r
.78997 .36894 .79339 .37363 .80133 .37106 .79792 .36636 Metetra
.665 .742 .908 r
.79792 .36636 .80133 .37106 .80928 .3685 .80589 .36378 Metetra
.664 .742 .908 r
.80589 .36378 .80928 .3685 .81727 .36593 .81389 .3612 Metetra
.664 .742 .908 r
.81389 .3612 .81727 .36593 .82527 .36336 .82191 .35862 Metetra
.664 .741 .908 r
.82191 .35862 .82527 .36336 .8333 .36079 .82995 .35603 Metetra
.664 .741 .908 r
.82995 .35603 .8333 .36079 .84134 .35821 .83802 .35344 Metetra
.664 .741 .908 r
.83802 .35344 .84134 .35821 .84942 .35562 .8461 .35084 Metetra
.664 .741 .908 r
.8461 .35084 .84942 .35562 .85751 .35303 .85421 .34824 Metetra
.664 .741 .908 r
.85421 .34824 .85751 .35303 .86563 .35044 .86235 .34563 Metetra
.664 .741 .908 r
.86235 .34563 .86563 .35044 .87376 .34783 .8705 .34301 Metetra
.664 .741 .908 r
.8705 .34301 .87376 .34783 .88193 .34522 .87868 .34039 Metetra
.664 .741 .908 r
.87868 .34039 .88193 .34522 .89011 .3426 .88688 .33775 Metetra
.664 .741 .908 r
.88688 .33775 .89011 .3426 .89832 .33998 .8951 .33512 Metetra
.664 .741 .908 r
.8951 .33512 .89832 .33998 .90655 .33735 .90335 .33247 Metetra
.67 .749 .91 r
.31174 .51817 .31601 .52202 .32272 .51981 .31846 .5159 Metetra
.666 .746 .91 r
.31846 .5159 .32272 .51981 .32944 .51763 .32519 .51369 Metetra
.664 .744 .91 r
.32519 .51369 .32944 .51763 .33617 .51549 .33194 .51152 Metetra
.662 .743 .91 r
.33194 .51152 .33617 .51549 .34293 .51335 .3387 .50937 Metetra
.661 .742 .91 r
.3387 .50937 .34293 .51335 .34969 .51123 .34548 .50723 Metetra
.66 .742 .91 r
.34548 .50723 .34969 .51123 .35648 .50912 .35227 .50511 Metetra
.659 .741 .911 r
.35227 .50511 .35648 .50912 .36328 .50701 .35908 .503 Metetra
.658 .741 .911 r
.35908 .503 .36328 .50701 .3701 .50493 .36591 .5009 Metetra
.656 .741 .912 r
.36591 .5009 .3701 .50493 .37693 .50287 .37276 .49883 Metetra
.654 .74 .913 r
.37276 .49883 .37693 .50287 .38378 .50084 .37962 .4968 Metetra
.651 .739 .915 r
.37962 .4968 .38378 .50084 .39065 .49886 .3865 .49481 Metetra
.646 .738 .917 r
.3865 .49481 .39065 .49886 .39753 .49695 .39339 .49291 Metetra
.637 .736 .92 r
.39339 .49291 .39753 .49695 .40443 .49517 .4003 .49113 Metetra
.623 .733 .925 r
.4003 .49113 .40443 .49517 .41134 .49359 .40721 .48957 Metetra
.599 .727 .934 r
.40721 .48957 .41134 .49359 .41826 .4923 .41414 .48836 Metetra
.572 .721 .943 r
.41414 .48836 .41826 .4923 .42519 .49127 .42108 .48747 Metetra
.629 .749 .934 r
.42108 .48747 .42519 .49127 .43217 .48948 .42807 .48577 Metetra
.725 .777 .895 r
.42807 .48577 .43217 .48948 .43921 .48615 .43512 .48226 Metetra
.73 .77 .885 r
.43512 .48226 .43921 .48615 .44626 .48262 .4422 .47858 Metetra
.71 .759 .891 r
.4422 .47858 .44626 .48262 .45332 .47947 .44927 .47536 Metetra
.694 .753 .897 r
.44927 .47536 .45332 .47947 .46039 .47661 .45636 .47247 Metetra
.685 .749 .9 r
.45636 .47247 .46039 .47661 .46748 .47394 .46345 .46977 Metetra
.679 .747 .903 r
.46345 .46977 .46748 .47394 .47457 .47137 .47056 .46718 Metetra
.675 .746 .904 r
.47056 .46718 .47457 .47137 .48169 .46888 .47768 .46467 Metetra
.672 .745 .905 r
.47768 .46467 .48169 .46888 .48882 .46643 .48483 .4622 Metetra
.67 .744 .906 r
.48483 .4622 .48882 .46643 .49596 .46401 .49199 .45977 Metetra
.668 .743 .906 r
.49199 .45977 .49596 .46401 .50313 .46163 .49916 .45738 Metetra
.666 .742 .907 r
.49916 .45738 .50313 .46163 .51031 .45928 .50636 .45502 Metetra
.662 .741 .909 r
.50636 .45502 .51031 .45928 .51751 .457 .51358 .45273 Metetra
.656 .739 .911 r
.51358 .45273 .51751 .457 .52474 .45481 .52081 .45054 Metetra
.652 .739 .913 r
.52081 .45054 .52474 .45481 .53198 .45266 .52806 .44842 Metetra
.665 .745 .91 r
.52806 .44842 .53198 .45266 .53924 .45031 .53534 .44606 Metetra
.676 .748 .906 r
.53534 .44606 .53924 .45031 .54652 .44776 .54263 .44348 Metetra
.674 .747 .906 r
.54263 .44348 .54652 .44776 .55381 .44523 .54994 .44093 Metetra
.67 .745 .907 r
.54994 .44093 .55381 .44523 .56113 .44277 .55726 .43845 Metetra
.665 .743 .909 r
.55726 .43845 .56113 .44277 .56846 .44039 .56461 .43607 Metetra
.658 .741 .911 r
.56461 .43607 .56846 .44039 .57582 .43813 .57198 .43379 Metetra
.651 .739 .914 r
.57198 .43379 .57582 .43813 .5832 .436 .57938 .43166 Metetra
.64 .736 .918 r
.57938 .43166 .5832 .436 .5906 .43404 .58679 .42971 Metetra
.623 .73 .924 r
.58679 .42971 .5906 .43404 .59803 .43236 .59424 .42805 Metetra
.589 .719 .933 r
.59424 .42805 .59803 .43236 .6055 .43113 .60172 .4269 Metetra
.54 .703 .946 r
.60172 .4269 .6055 .43113 .61301 .43044 .60924 .4264 Metetra
.575 .729 .947 r
.60924 .4264 .61301 .43044 .62053 .42928 .61678 .42537 Metetra
.682 .774 .921 r
.61678 .42537 .62053 .42928 .62803 .42665 .62429 .42258 Metetra
.7 .772 .907 r
.62429 .42258 .62803 .42665 .63553 .42362 .6318 .41941 Metetra
.688 .762 .908 r
.6318 .41941 .63553 .42362 .64306 .42077 .63935 .41649 Metetra
.676 .756 .911 r
.63935 .41649 .64306 .42077 .65062 .41812 .64692 .41381 Metetra
.669 .753 .913 r
.64692 .41381 .65062 .41812 .6582 .41557 .65451 .41126 Metetra
.673 .754 .912 r
.65451 .41126 .6582 .41557 .66579 .41294 .66212 .40862 Metetra
.701 .763 .9 r
.66212 .40862 .66579 .41294 .67338 .40979 .66973 .40539 Metetra
.712 .763 .893 r
.66973 .40539 .67338 .40979 .68099 .40654 .67733 .40188 Metetra
.539 .684 .933 r
.67733 .40188 .68099 .40654 .68874 .40588 .68511 .4015 Metetra
.456 .659 .948 r
.68511 .4015 .68874 .40588 .69658 .40616 .69297 .40204 Metetra
.372 .63 .954 r
.69297 .40204 .69658 .40616 .70448 .40703 .70091 .4033 Metetra
.277 .599 .951 r
.70091 .4033 .70448 .40703 .71243 .40817 .70892 .40514 Metetra
.615 .811 .975 r
.70892 .40514 .71243 .40817 .72022 .40593 .71674 .40314 Metetra
.864 .868 .84 r
.71674 .40314 .72022 .40593 .72773 .39887 .72422 .39533 Metetra
.856 .833 .823 r
.72422 .39533 .72773 .39887 .73522 .39154 .73169 .38729 Metetra
.781 .794 .863 r
.73169 .38729 .73522 .39154 .74285 .38637 .73933 .38184 Metetra
.72 .766 .889 r
.73933 .38184 .74285 .38637 .7506 .38268 .74708 .37806 Metetra
.69 .753 .9 r
.74708 .37806 .7506 .38268 .75841 .37962 .75491 .37497 Metetra
.677 .747 .904 r
.75491 .37497 .75841 .37962 .76626 .37683 .76277 .37216 Metetra
.671 .744 .906 r
.76277 .37216 .76626 .37683 .77414 .37416 .77067 .36947 Metetra
.668 .743 .907 r
.77067 .36947 .77414 .37416 .78204 .37154 .77858 .36684 Metetra
.666 .742 .907 r
.77858 .36684 .78204 .37154 .78997 .36894 .78653 .36423 Metetra
.665 .742 .907 r
.78653 .36423 .78997 .36894 .79792 .36636 .7945 .36163 Metetra
.665 .742 .908 r
.7945 .36163 .79792 .36636 .80589 .36378 .80249 .35904 Metetra
.664 .742 .908 r
.80249 .35904 .80589 .36378 .81389 .3612 .8105 .35645 Metetra
.664 .742 .908 r
.8105 .35645 .81389 .3612 .82191 .35862 .81853 .35385 Metetra
.664 .741 .908 r
.81853 .35385 .82191 .35862 .82995 .35603 .82659 .35125 Metetra
.664 .741 .908 r
.82659 .35125 .82995 .35603 .83802 .35344 .83467 .34865 Metetra
.664 .741 .908 r
.83467 .34865 .83802 .35344 .8461 .35084 .84278 .34603 Metetra
.664 .741 .908 r
.84278 .34603 .8461 .35084 .85421 .34824 .8509 .34342 Metetra
.664 .741 .908 r
.8509 .34342 .85421 .34824 .86235 .34563 .85905 .34079 Metetra
.664 .741 .908 r
.85905 .34079 .86235 .34563 .8705 .34301 .86722 .33816 Metetra
.664 .741 .908 r
.86722 .33816 .8705 .34301 .87868 .34039 .87542 .33552 Metetra
.664 .741 .908 r
.87542 .33552 .87868 .34039 .88688 .33775 .88363 .33288 Metetra
.664 .741 .908 r
.88363 .33288 .88688 .33775 .8951 .33512 .89188 .33023 Metetra
.664 .741 .908 r
.89188 .33023 .8951 .33512 .90335 .33247 .90014 .32757 Metetra
.673 .75 .909 r
.30744 .51432 .31174 .51817 .31846 .5159 .31418 .51198 Metetra
.668 .746 .909 r
.31418 .51198 .31846 .5159 .32519 .51369 .32092 .50974 Metetra
.664 .744 .91 r
.32092 .50974 .32519 .51369 .33194 .51152 .32768 .50754 Metetra
.662 .743 .91 r
.32768 .50754 .33194 .51152 .3387 .50937 .33445 .50537 Metetra
.661 .742 .91 r
.33445 .50537 .3387 .50937 .34548 .50723 .34124 .50322 Metetra
.66 .742 .91 r
.34124 .50322 .34548 .50723 .35227 .50511 .34805 .50109 Metetra
.659 .742 .911 r
.34805 .50109 .35227 .50511 .35908 .503 .35487 .49896 Metetra
.658 .741 .911 r
.35487 .49896 .35908 .503 .36591 .5009 .36171 .49686 Metetra
.656 .741 .912 r
.36171 .49686 .36591 .5009 .37276 .49883 .36856 .49478 Metetra
.654 .74 .913 r
.36856 .49478 .37276 .49883 .37962 .4968 .37544 .49274 Metetra
.65 .739 .915 r
.37544 .49274 .37962 .4968 .3865 .49481 .38232 .49075 Metetra
.645 .738 .917 r
.38232 .49075 .3865 .49481 .39339 .49291 .38923 .48884 Metetra
.636 .736 .92 r
.38923 .48884 .39339 .49291 .4003 .49113 .39614 .48707 Metetra
.621 .732 .926 r
.39614 .48707 .4003 .49113 .40721 .48957 .40307 .48553 Metetra
.593 .725 .936 r
.40307 .48553 .40721 .48957 .41414 .48836 .41001 .48439 Metetra
.557 .716 .947 r
.41001 .48439 .41414 .48836 .42108 .48747 .41695 .48369 Metetra
.62 .748 .938 r
.41695 .48369 .42108 .48747 .42807 .48577 .42394 .4821 Metetra
.734 .782 .891 r
.42394 .4821 .42807 .48577 .43512 .48226 .43102 .47837 Metetra
.736 .772 .882 r
.43102 .47837 .43512 .48226 .4422 .47858 .43811 .47453 Metetra
.712 .76 .89 r
.43811 .47453 .4422 .47858 .44927 .47536 .4452 .47123 Metetra
.695 .753 .896 r
.4452 .47123 .44927 .47536 .45636 .47247 .4523 .4683 Metetra
.685 .75 .9 r
.4523 .4683 .45636 .47247 .46345 .46977 .45941 .46558 Metetra
.679 .747 .902 r
.45941 .46558 .46345 .46977 .47056 .46718 .46653 .46297 Metetra
.675 .746 .904 r
.46653 .46297 .47056 .46718 .47768 .46467 .47366 .46044 Metetra
.672 .745 .905 r
.47366 .46044 .47768 .46467 .48483 .4622 .48082 .45796 Metetra
.67 .744 .906 r
.48082 .45796 .48483 .4622 .49199 .45977 .48799 .45552 Metetra
.668 .743 .906 r
.48799 .45552 .49199 .45977 .49916 .45738 .49518 .4531 Metetra
.666 .742 .907 r
.49518 .4531 .49916 .45738 .50636 .45502 .50239 .45073 Metetra
.662 .741 .909 r
.50239 .45073 .50636 .45502 .51358 .45273 .50962 .44843 Metetra
.655 .739 .911 r
.50962 .44843 .51358 .45273 .52081 .45054 .51686 .44626 Metetra
.65 .738 .914 r
.51686 .44626 .52081 .45054 .52806 .44842 .52413 .44417 Metetra
.664 .745 .91 r
.52413 .44417 .52806 .44842 .53534 .44606 .53141 .4418 Metetra
.677 .749 .905 r
.53141 .4418 .53534 .44606 .54263 .44348 .53872 .43918 Metetra
.674 .747 .905 r
.53872 .43918 .54263 .44348 .54994 .44093 .54604 .4366 Metetra
.67 .745 .907 r
.54604 .4366 .54994 .44093 .55726 .43845 .55338 .43412 Metetra
.665 .743 .909 r
.55338 .43412 .55726 .43845 .56461 .43607 .56074 .43172 Metetra
.658 .741 .911 r
.56074 .43172 .56461 .43607 .57198 .43379 .56813 .42944 Metetra
.65 .739 .914 r
.56813 .42944 .57198 .43379 .57938 .43166 .57553 .4273 Metetra
.639 .735 .918 r
.57553 .4273 .57938 .43166 .58679 .42971 .58296 .42536 Metetra
.62 .729 .924 r
.58296 .42536 .58679 .42971 .59424 .42805 .59042 .42372 Metetra
.583 .717 .935 r
.59042 .42372 .59424 .42805 .60172 .4269 .59791 .42265 Metetra
.521 .695 .948 r
.59791 .42265 .60172 .4269 .60924 .4264 .60546 .42238 Metetra
.562 .726 .951 r
.60546 .42238 .60924 .4264 .61678 .42537 .61301 .4215 Metetra
.691 .78 .919 r
.61301 .4215 .61678 .42537 .62429 .42258 .62053 .41852 Metetra
.707 .775 .905 r
.62053 .41852 .62429 .42258 .6318 .41941 .62806 .41519 Metetra
.691 .763 .907 r
.62806 .41519 .6318 .41941 .63935 .41649 .63561 .4122 Metetra
.677 .756 .911 r
.63561 .4122 .63935 .41649 .64692 .41381 .6432 .40949 Metetra
.669 .753 .913 r
.6432 .40949 .64692 .41381 .65451 .41126 .65081 .40693 Metetra
.672 .754 .912 r
.65081 .40693 .65451 .41126 .66212 .40862 .65843 .4043 Metetra
.704 .765 .9 r
.65843 .4043 .66212 .40862 .66973 .40539 .66605 .40098 Metetra
.724 .768 .887 r
.66605 .40098 .66973 .40539 .67733 .40188 .67366 .39716 Metetra
.514 .67 .934 r
.67366 .39716 .67733 .40188 .68511 .4015 .68147 .39712 Metetra
.43 .647 .949 r
.68147 .39712 .68511 .4015 .69297 .40204 .68935 .39793 Metetra
.324 .604 .949 r
.68935 .39793 .69297 .40204 .70091 .4033 .69732 .39962 Metetra
.163 .532 .922 r
.69732 .39962 .70091 .4033 .70892 .40514 .70539 .40231 Metetra
.581 .812 .987 r
.70539 .40231 .70892 .40514 .71674 .40314 .71325 .40063 Metetra
.886 .879 .822 r
.71325 .40063 .71674 .40314 .72422 .39533 .72071 .39192 Metetra
.874 .838 .808 r
.72071 .39192 .72422 .39533 .73169 .38729 .72815 .38304 Metetra
.79 .797 .858 r
.72815 .38304 .73169 .38729 .73933 .38184 .73578 .37728 Metetra
.722 .767 .888 r
.73578 .37728 .73933 .38184 .74708 .37806 .74355 .37342 Metetra
.691 .753 .899 r
.74355 .37342 .74708 .37806 .75491 .37497 .75139 .3703 Metetra
.677 .747 .904 r
.75139 .3703 .75491 .37497 .76277 .37216 .75927 .36747 Metetra
.671 .744 .906 r
.75927 .36747 .76277 .37216 .77067 .36947 .76718 .36476 Metetra
.668 .743 .907 r
.76718 .36476 .77067 .36947 .77858 .36684 .77511 .36211 Metetra
.666 .742 .907 r
.77511 .36211 .77858 .36684 .78653 .36423 .78307 .35949 Metetra
.665 .742 .907 r
.78307 .35949 .78653 .36423 .7945 .36163 .79105 .35688 Metetra
.665 .742 .908 r
.79105 .35688 .7945 .36163 .80249 .35904 .79906 .35427 Metetra
.664 .742 .908 r
.79906 .35427 .80249 .35904 .8105 .35645 .80709 .35167 Metetra
.664 .742 .908 r
.80709 .35167 .8105 .35645 .81853 .35385 .81514 .34906 Metetra
.664 .741 .908 r
.81514 .34906 .81853 .35385 .82659 .35125 .82321 .34645 Metetra
.664 .741 .908 r
.82321 .34645 .82659 .35125 .83467 .34865 .83131 .34383 Metetra
.664 .741 .908 r
.83131 .34383 .83467 .34865 .84278 .34603 .83943 .3412 Metetra
.664 .741 .908 r
.83943 .3412 .84278 .34603 .8509 .34342 .84757 .33857 Metetra
.664 .741 .908 r
.84757 .33857 .8509 .34342 .85905 .34079 .85574 .33593 Metetra
.664 .741 .908 r
.85574 .33593 .85905 .34079 .86722 .33816 .86393 .33329 Metetra
.664 .741 .908 r
.86393 .33329 .86722 .33816 .87542 .33552 .87214 .33064 Metetra
.664 .741 .908 r
.87214 .33064 .87542 .33552 .88363 .33288 .88037 .32798 Metetra
.664 .741 .908 r
.88037 .32798 .88363 .33288 .89188 .33023 .88863 .32531 Metetra
.664 .741 .908 r
.88863 .32531 .89188 .33023 .90014 .32757 .89691 .32264 Metetra
.676 .752 .908 r
.30313 .51046 .30744 .51432 .31418 .51198 .30988 .50805 Metetra
.669 .747 .909 r
.30988 .50805 .31418 .51198 .32092 .50974 .31663 .50576 Metetra
.665 .744 .909 r
.31663 .50576 .32092 .50974 .32768 .50754 .3234 .50354 Metetra
.663 .743 .91 r
.3234 .50354 .32768 .50754 .33445 .50537 .33019 .50136 Metetra
.661 .742 .91 r
.33019 .50136 .33445 .50537 .34124 .50322 .33699 .49919 Metetra
.66 .742 .91 r
.33699 .49919 .34124 .50322 .34805 .50109 .3438 .49704 Metetra
.659 .742 .911 r
.3438 .49704 .34805 .50109 .35487 .49896 .35064 .49491 Metetra
.658 .741 .911 r
.35064 .49491 .35487 .49896 .36171 .49686 .35749 .49279 Metetra
.656 .741 .912 r
.35749 .49279 .36171 .49686 .36856 .49478 .36435 .49071 Metetra
.654 .74 .913 r
.36435 .49071 .36856 .49478 .37544 .49274 .37123 .48866 Metetra
.65 .739 .915 r
.37123 .48866 .37544 .49274 .38232 .49075 .37813 .48666 Metetra
.645 .738 .917 r
.37813 .48666 .38232 .49075 .38923 .48884 .38505 .48475 Metetra
.635 .736 .921 r
.38505 .48475 .38923 .48884 .39614 .48707 .39197 .48299 Metetra
.619 .732 .927 r
.39197 .48299 .39614 .48707 .40307 .48553 .39891 .48148 Metetra
.586 .722 .937 r
.39891 .48148 .40307 .48553 .41001 .48439 .40585 .48042 Metetra
.537 .709 .95 r
.40585 .48042 .41001 .48439 .41695 .48369 .4128 .47993 Metetra
.609 .747 .943 r
.4128 .47993 .41695 .48369 .42394 .4821 .4198 .47849 Metetra
.744 .786 .887 r
.4198 .47849 .42394 .4821 .43102 .47837 .4269 .47448 Metetra
.742 .774 .879 r
.4269 .47448 .43102 .47837 .43811 .47453 .43401 .47045 Metetra
.715 .761 .889 r
.43401 .47045 .43811 .47453 .4452 .47123 .44111 .46708 Metetra
.697 .754 .896 r
.44111 .46708 .4452 .47123 .4523 .4683 .44822 .46411 Metetra
.686 .75 .9 r
.44822 .46411 .4523 .4683 .45941 .46558 .45534 .46137 Metetra
.68 .747 .902 r
.45534 .46137 .45941 .46558 .46653 .46297 .46247 .45874 Metetra
.675 .746 .904 r
.46247 .45874 .46653 .46297 .47366 .46044 .46963 .4562 Metetra
.673 .745 .905 r
.46963 .4562 .47366 .46044 .48082 .45796 .47679 .4537 Metetra
.67 .744 .906 r
.47679 .4537 .48082 .45796 .48799 .45552 .48398 .45124 Metetra
.669 .743 .906 r
.48398 .45124 .48799 .45552 .49518 .4531 .49118 .44881 Metetra
.666 .742 .907 r
.49118 .44881 .49518 .4531 .50239 .45073 .4984 .44643 Metetra
.662 .741 .909 r
.4984 .44643 .50239 .45073 .50962 .44843 .50564 .44412 Metetra
.654 .739 .912 r
.50564 .44412 .50962 .44843 .51686 .44626 .5129 .44195 Metetra
.647 .737 .915 r
.5129 .44195 .51686 .44626 .52413 .44417 .52018 .4399 Metetra
.664 .745 .911 r
.52018 .4399 .52413 .44417 .53141 .4418 .52747 .43753 Metetra
.679 .75 .905 r
.52747 .43753 .53141 .4418 .53872 .43918 .53479 .43486 Metetra
.675 .747 .905 r
.53479 .43486 .53872 .43918 .54604 .4366 .54213 .43226 Metetra
.67 .745 .907 r
.54213 .43226 .54604 .4366 .55338 .43412 .54948 .42976 Metetra
.664 .743 .909 r
.54948 .42976 .55338 .43412 .56074 .43172 .55686 .42736 Metetra
.658 .741 .911 r
.55686 .42736 .56074 .43172 .56813 .42944 .56425 .42507 Metetra
.65 .739 .914 r
.56425 .42507 .56813 .42944 .57553 .4273 .57167 .42292 Metetra
.638 .735 .919 r
.57167 .42292 .57553 .4273 .58296 .42536 .57911 .42098 Metetra
.618 .729 .925 r
.57911 .42098 .58296 .42536 .59042 .42372 .58658 .41937 Metetra
.576 .714 .937 r
.58658 .41937 .59042 .42372 .59791 .42265 .59409 .4184 Metetra
.498 .685 .951 r
.59409 .4184 .59791 .42265 .60546 .42238 .60165 .41839 Metetra
.545 .721 .955 r
.60165 .41839 .60546 .42238 .61301 .4215 .60923 .4177 Metetra
.703 .788 .917 r
.60923 .4177 .61301 .4215 .62053 .41852 .61675 .41447 Metetra
.715 .779 .903 r
.61675 .41447 .62053 .41852 .62806 .41519 .62429 .41096 Metetra
.694 .765 .906 r
.62429 .41096 .62806 .41519 .63561 .4122 .63186 .40789 Metetra
.678 .757 .911 r
.63186 .40789 .63561 .4122 .6432 .40949 .63946 .40515 Metetra
.668 .753 .914 r
.63946 .40515 .6432 .40949 .65081 .40693 .64708 .40259 Metetra
.671 .754 .913 r
.64708 .40259 .65081 .40693 .65843 .4043 .65472 .39996 Metetra
.706 .766 .899 r
.65472 .39996 .65843 .4043 .66605 .40098 .66235 .39658 Metetra
.739 .773 .88 r
.66235 .39658 .66605 .40098 .67366 .39716 .66996 .39236 Metetra
.483 .652 .934 r
.66996 .39236 .67366 .39716 .68147 .39712 .6778 .39274 Metetra
.401 .633 .948 r
.6778 .39274 .68147 .39712 .68935 .39793 .68572 .39384 Metetra
.267 .573 .939 r
.68572 .39384 .68935 .39793 .69732 .39962 .69372 .396 Metetra
.018 .436 .867 r
.69372 .396 .69732 .39962 .70539 .40231 .70186 .39971 Metetra
.527 .806 .997 r
.70186 .39971 .70539 .40231 .71325 .40063 .70976 .39847 Metetra
.909 .889 .8 r
.70976 .39847 .71325 .40063 .72071 .39192 .71718 .38867 Metetra
.892 .843 .79 r
.71718 .38867 .72071 .39192 .72815 .38304 .72459 .3788 Metetra
.8 .8 .852 r
.72459 .3788 .72815 .38304 .73578 .37728 .73222 .3727 Metetra
.725 .768 .886 r
.73222 .3727 .73578 .37728 .74355 .37342 .74 .36875 Metetra
.692 .753 .899 r
.74 .36875 .74355 .37342 .75139 .3703 .74785 .3656 Metetra
.677 .747 .904 r
.74785 .3656 .75139 .3703 .75927 .36747 .75575 .36275 Metetra
.671 .744 .906 r
.75575 .36275 .75927 .36747 .76718 .36476 .76367 .36003 Metetra
.668 .743 .907 r
.76367 .36003 .76718 .36476 .77511 .36211 .77162 .35736 Metetra
.666 .742 .907 r
.77162 .35736 .77511 .36211 .78307 .35949 .7796 .35473 Metetra
.665 .742 .907 r
.7796 .35473 .78307 .35949 .79105 .35688 .7876 .3521 Metetra
.665 .742 .908 r
.7876 .3521 .79105 .35688 .79906 .35427 .79562 .34949 Metetra
.664 .742 .908 r
.79562 .34949 .79906 .35427 .80709 .35167 .80366 .34687 Metetra
.664 .742 .908 r
.80366 .34687 .80709 .35167 .81514 .34906 .81173 .34424 Metetra
.664 .741 .908 r
.81173 .34424 .81514 .34906 .82321 .34645 .81982 .34162 Metetra
.664 .741 .908 r
.81982 .34162 .82321 .34645 .83131 .34383 .82793 .33898 Metetra
.664 .741 .908 r
.82793 .33898 .83131 .34383 .83943 .3412 .83607 .33634 Metetra
.664 .741 .908 r
.83607 .33634 .83943 .3412 .84757 .33857 .84423 .3337 Metetra
.664 .741 .908 r
.84423 .3337 .84757 .33857 .85574 .33593 .85241 .33105 Metetra
.664 .741 .908 r
.85241 .33105 .85574 .33593 .86393 .33329 .86061 .32839 Metetra
.664 .741 .908 r
.86061 .32839 .86393 .33329 .87214 .33064 .86884 .32573 Metetra
.664 .741 .908 r
.86884 .32573 .87214 .33064 .88037 .32798 .87709 .32305 Metetra
.664 .741 .908 r
.87709 .32305 .88037 .32798 .88863 .32531 .88537 .32037 Metetra
.664 .741 .908 r
.88537 .32037 .88863 .32531 .89691 .32264 .89366 .31769 Metetra
.68 .753 .907 r
.29879 .5066 .30313 .51046 .30988 .50805 .30555 .5041 Metetra
.671 .748 .908 r
.30555 .5041 .30988 .50805 .31663 .50576 .31232 .50177 Metetra
.666 .745 .909 r
.31232 .50177 .31663 .50576 .3234 .50354 .3191 .49952 Metetra
.663 .743 .91 r
.3191 .49952 .3234 .50354 .33019 .50136 .3259 .49732 Metetra
.662 .743 .91 r
.3259 .49732 .33019 .50136 .33699 .49919 .33271 .49514 Metetra
.66 .742 .91 r
.33271 .49514 .33699 .49919 .3438 .49704 .33954 .49298 Metetra
.659 .742 .911 r
.33954 .49298 .3438 .49704 .35064 .49491 .34638 .49084 Metetra
.658 .741 .911 r
.34638 .49084 .35064 .49491 .35749 .49279 .35324 .48871 Metetra
.656 .741 .912 r
.35324 .48871 .35749 .49279 .36435 .49071 .36012 .48662 Metetra
.653 .74 .913 r
.36012 .48662 .36435 .49071 .37123 .48866 .36701 .48456 Metetra
.65 .739 .915 r
.36701 .48456 .37123 .48866 .37813 .48666 .37392 .48256 Metetra
.644 .738 .917 r
.37392 .48256 .37813 .48666 .38505 .48475 .38084 .48065 Metetra
.634 .735 .921 r
.38084 .48065 .38505 .48475 .39197 .48299 .38778 .47889 Metetra
.616 .731 .928 r
.38778 .47889 .39197 .48299 .39891 .48148 .39473 .47741 Metetra
.579 .72 .939 r
.39473 .47741 .39891 .48148 .40585 .48042 .40168 .47643 Metetra
.514 .7 .954 r
.40168 .47643 .40585 .48042 .4128 .47993 .40862 .47618 Metetra
.593 .743 .948 r
.40862 .47618 .4128 .47993 .4198 .47849 .41563 .47493 Metetra
.757 .792 .881 r
.41563 .47493 .4198 .47849 .4269 .47448 .42275 .47059 Metetra
.749 .776 .875 r
.42275 .47059 .4269 .47448 .43401 .47045 .42988 .46636 Metetra
.718 .761 .887 r
.42988 .46636 .43401 .47045 .44111 .46708 .437 .46291 Metetra
.698 .754 .895 r
.437 .46291 .44111 .46708 .44822 .46411 .44412 .45991 Metetra
.687 .75 .9 r
.44412 .45991 .44822 .46411 .45534 .46137 .45126 .45713 Metetra
.68 .748 .902 r
.45126 .45713 .45534 .46137 .46247 .45874 .4584 .45449 Metetra
.676 .746 .904 r
.4584 .45449 .46247 .45874 .46963 .4562 .46557 .45193 Metetra
.673 .745 .905 r
.46557 .45193 .46963 .4562 .47679 .4537 .47275 .44942 Metetra
.671 .744 .906 r
.47275 .44942 .47679 .4537 .48398 .45124 .47994 .44694 Metetra
.669 .743 .906 r
.47994 .44694 .48398 .45124 .49118 .44881 .48716 .4445 Metetra
.666 .742 .907 r
.48716 .4445 .49118 .44881 .4984 .44643 .49439 .4421 Metetra
.662 .741 .909 r
.49439 .4421 .4984 .44643 .50564 .44412 .50164 .43978 Metetra
.653 .738 .912 r
.50164 .43978 .50564 .44412 .5129 .44195 .50891 .43762 Metetra
.644 .736 .916 r
.50891 .43762 .5129 .44195 .52018 .4399 .5162 .43562 Metetra
.664 .745 .911 r
.5162 .43562 .52018 .4399 .52747 .43753 .52351 .43324 Metetra
.68 .751 .904 r
.52351 .43324 .52747 .43753 .53479 .43486 .53084 .43053 Metetra
.676 .748 .905 r
.53084 .43053 .53479 .43486 .54213 .43226 .53819 .4279 Metetra
.67 .745 .907 r
.53819 .4279 .54213 .43226 .54948 .42976 .54556 .42538 Metetra
.664 .743 .909 r
.54556 .42538 .54948 .42976 .55686 .42736 .55295 .42297 Metetra
.658 .741 .911 r
.55295 .42297 .55686 .42736 .56425 .42507 .56036 .42067 Metetra
.649 .739 .915 r
.56036 .42067 .56425 .42507 .57167 .42292 .56779 .41853 Metetra
.637 .735 .919 r
.56779 .41853 .57167 .42292 .57911 .42098 .57524 .41659 Metetra
.616 .728 .926 r
.57524 .41659 .57911 .42098 .58658 .41937 .58273 .41501 Metetra
.568 .711 .938 r
.58273 .41501 .58658 .41937 .59409 .4184 .59025 .41414 Metetra
.469 .672 .953 r
.59025 .41414 .59409 .4184 .60165 .41839 .59783 .41443 Metetra
.523 .715 .96 r
.59783 .41443 .60165 .41839 .60923 .4177 .60542 .41397 Metetra
.718 .797 .913 r
.60542 .41397 .60923 .4177 .61675 .41447 .61296 .41042 Metetra
.723 .782 .9 r
.61296 .41042 .61675 .41447 .62429 .41096 .62051 .40672 Metetra
.697 .766 .905 r
.62051 .40672 .62429 .41096 .63186 .40789 .62809 .40357 Metetra
.678 .757 .911 r
.62809 .40357 .63186 .40789 .63946 .40515 .6357 .4008 Metetra
.668 .753 .914 r
.6357 .4008 .63946 .40515 .64708 .40259 .64334 .39823 Metetra
.669 .754 .914 r
.64334 .39823 .64708 .40259 .65472 .39996 .651 .39563 Metetra
.708 .768 .899 r
.651 .39563 .65472 .39996 .66235 .39658 .65864 .39219 Metetra
.756 .779 .871 r
.65864 .39219 .66235 .39658 .66996 .39236 .66625 .38748 Metetra
.441 .628 .932 r
.66625 .38748 .66996 .39236 .6778 .39274 .67412 .38838 Metetra
.369 .617 .947 r
.67412 .38838 .6778 .39274 .68572 .39384 .68206 .38976 Metetra
.201 .534 .924 r
.68206 .38976 .68572 .39384 .69372 .396 .6901 .39244 Metetra
0 .308 .774 r
.6901 .39244 .69372 .396 .70186 .39971 .69832 .39741 Metetra
.439 .784 .998 r
.69832 .39741 .70186 .39971 .70976 .39847 .70628 .39676 Metetra
.931 .898 .772 r
.70628 .39676 .70976 .39847 .71718 .38867 .71365 .38561 Metetra
.91 .847 .77 r
.71365 .38561 .71718 .38867 .72459 .3788 .72101 .37458 Metetra
.81 .803 .845 r
.72101 .37458 .72459 .3788 .73222 .3727 .72864 .3681 Metetra
.728 .769 .885 r
.72864 .3681 .73222 .3727 .74 .36875 .73643 .36406 Metetra
.692 .753 .898 r
.73643 .36406 .74 .36875 .74785 .3656 .7443 .36087 Metetra
.678 .747 .903 r
.7443 .36087 .74785 .3656 .75575 .36275 .75221 .358 Metetra
.671 .744 .906 r
.75221 .358 .75575 .36275 .76367 .36003 .76015 .35527 Metetra
.668 .743 .907 r
.76015 .35527 .76367 .36003 .77162 .35736 .76811 .35259 Metetra
.666 .742 .907 r
.76811 .35259 .77162 .35736 .7796 .35473 .7761 .34994 Metetra
.665 .742 .907 r
.7761 .34994 .7796 .35473 .7876 .3521 .78412 .3473 Metetra
.665 .742 .908 r
.78412 .3473 .7876 .3521 .79562 .34949 .79216 .34467 Metetra
.664 .742 .908 r
.79216 .34467 .79562 .34949 .80366 .34687 .80022 .34204 Metetra
.664 .742 .908 r
.80022 .34204 .80366 .34687 .81173 .34424 .8083 .3394 Metetra
.664 .741 .908 r
.8083 .3394 .81173 .34424 .81982 .34162 .81641 .33676 Metetra
.664 .741 .908 r
.81641 .33676 .81982 .34162 .82793 .33898 .82454 .33411 Metetra
.664 .741 .908 r
.82454 .33411 .82793 .33898 .83607 .33634 .83269 .33146 Metetra
.664 .741 .908 r
.83269 .33146 .83607 .33634 .84423 .3337 .84086 .3288 Metetra
.664 .741 .908 r
.84086 .3288 .84423 .3337 .85241 .33105 .84906 .32614 Metetra
.664 .741 .908 r
.84906 .32614 .85241 .33105 .86061 .32839 .85728 .32347 Metetra
.664 .741 .908 r
.85728 .32347 .86061 .32839 .86884 .32573 .86553 .32079 Metetra
.664 .741 .908 r
.86553 .32079 .86884 .32573 .87709 .32305 .8738 .3181 Metetra
.664 .741 .908 r
.8738 .3181 .87709 .32305 .88537 .32037 .88209 .31541 Metetra
.664 .741 .908 r
.88209 .31541 .88537 .32037 .89366 .31769 .8904 .31271 Metetra
.685 .755 .905 r
.29443 .50273 .29879 .5066 .30555 .5041 .30121 .50014 Metetra
.673 .748 .907 r
.30121 .50014 .30555 .5041 .31232 .50177 .30799 .49776 Metetra
.667 .745 .909 r
.30799 .49776 .31232 .50177 .3191 .49952 .31478 .49549 Metetra
.664 .743 .909 r
.31478 .49549 .3191 .49952 .3259 .49732 .32159 .49327 Metetra
.662 .743 .91 r
.32159 .49327 .3259 .49732 .33271 .49514 .32841 .49107 Metetra
.661 .742 .91 r
.32841 .49107 .33271 .49514 .33954 .49298 .33525 .4889 Metetra
.659 .742 .911 r
.33525 .4889 .33954 .49298 .34638 .49084 .34211 .48674 Metetra
.658 .741 .911 r
.34211 .48674 .34638 .49084 .35324 .48871 .34898 .48461 Metetra
.656 .741 .912 r
.34898 .48461 .35324 .48871 .36012 .48662 .35586 .4825 Metetra
.653 .74 .913 r
.35586 .4825 .36012 .48662 .36701 .48456 .36277 .48044 Metetra
.649 .739 .915 r
.36277 .48044 .36701 .48456 .37392 .48256 .36969 .47843 Metetra
.643 .738 .917 r
.36969 .47843 .37392 .48256 .38084 .48065 .37662 .47652 Metetra
.633 .735 .921 r
.37662 .47652 .38084 .48065 .38778 .47889 .38357 .47477 Metetra
.614 .73 .928 r
.38357 .47477 .38778 .47889 .39473 .47741 .39052 .47331 Metetra
.572 .717 .941 r
.39052 .47331 .39473 .47741 .40168 .47643 .39748 .47242 Metetra
.485 .688 .957 r
.39748 .47242 .40168 .47643 .40862 .47618 .40442 .47246 Metetra
.571 .738 .955 r
.40442 .47246 .40862 .47618 .41563 .47493 .41143 .47146 Metetra
.771 .798 .874 r
.41143 .47146 .41563 .47493 .42275 .47059 .41858 .4667 Metetra
.756 .777 .871 r
.41858 .4667 .42275 .47059 .42988 .46636 .42574 .46225 Metetra
.72 .762 .886 r
.42574 .46225 .42988 .46636 .437 .46291 .43287 .45873 Metetra
.699 .754 .895 r
.43287 .45873 .437 .46291 .44412 .45991 .44001 .45568 Metetra
.687 .75 .899 r
.44001 .45568 .44412 .45991 .45126 .45713 .44715 .45288 Metetra
.68 .748 .902 r
.44715 .45288 .45126 .45713 .4584 .45449 .45431 .45022 Metetra
.676 .746 .904 r
.45431 .45022 .4584 .45449 .46557 .45193 .46149 .44764 Metetra
.673 .745 .905 r
.46149 .44764 .46557 .45193 .47275 .44942 .46868 .44511 Metetra
.671 .744 .905 r
.46868 .44511 .47275 .44942 .47994 .44694 .47589 .44262 Metetra
.669 .743 .906 r
.47589 .44262 .47994 .44694 .48716 .4445 .48312 .44016 Metetra
.666 .742 .907 r
.48312 .44016 .48716 .4445 .49439 .4421 .49036 .43775 Metetra
.662 .741 .909 r
.49036 .43775 .49439 .4421 .50164 .43978 .49763 .43542 Metetra
.652 .738 .912 r
.49763 .43542 .50164 .43978 .50891 .43762 .50491 .43326 Metetra
.639 .735 .918 r
.50491 .43326 .50891 .43762 .5162 .43562 .51221 .43132 Metetra
.663 .746 .912 r
.51221 .43132 .5162 .43562 .52351 .43324 .51953 .42894 Metetra
.683 .752 .904 r
.51953 .42894 .52351 .43324 .53084 .43053 .52688 .42617 Metetra
.676 .748 .905 r
.52688 .42617 .53084 .43053 .53819 .4279 .53424 .42352 Metetra
.67 .745 .907 r
.53424 .42352 .53819 .4279 .54556 .42538 .54162 .42099 Metetra
.664 .743 .909 r
.54162 .42099 .54556 .42538 .55295 .42297 .54902 .41856 Metetra
.658 .741 .912 r
.54902 .41856 .55295 .42297 .56036 .42067 .55644 .41626 Metetra
.649 .739 .915 r
.55644 .41626 .56036 .42067 .56779 .41853 .56389 .41411 Metetra
.636 .735 .919 r
.56389 .41411 .56779 .41853 .57524 .41659 .57136 .41217 Metetra
.613 .727 .927 r
.57136 .41217 .57524 .41659 .58273 .41501 .57885 .41063 Metetra
.559 .707 .94 r
.57885 .41063 .58273 .41501 .59025 .41414 .58639 .40986 Metetra
.433 .655 .953 r
.58639 .40986 .59025 .41414 .59783 .41443 .59399 .41052 Metetra
.494 .706 .964 r
.59399 .41052 .59783 .41443 .60542 .41397 .6016 .41035 Metetra
.736 .807 .908 r
.6016 .41035 .60542 .41397 .61296 .41042 .60914 .40639 Metetra
.731 .786 .896 r
.60914 .40639 .61296 .41042 .62051 .40672 .6167 .40246 Metetra
.7 .768 .904 r
.6167 .40246 .62051 .40672 .62809 .40357 .6243 .39923 Metetra
.679 .758 .91 r
.6243 .39923 .62809 .40357 .6357 .4008 .63192 .39643 Metetra
.667 .753 .914 r
.63192 .39643 .6357 .4008 .64334 .39823 .63958 .39386 Metetra
.667 .754 .915 r
.63958 .39386 .64334 .39823 .651 .39563 .64725 .39129 Metetra
.709 .769 .899 r
.64725 .39129 .651 .39563 .65864 .39219 .65491 .38783 Metetra
.776 .785 .86 r
.65491 .38783 .65864 .39219 .66625 .38748 .66251 .38248 Metetra
.387 .594 .927 r
.66251 .38248 .66625 .38748 .67412 .38838 .67042 .38404 Metetra
.336 .601 .945 r
.67042 .38404 .67412 .38838 .68206 .38976 .67839 .3857 Metetra
.127 .488 .902 r
.67839 .3857 .68206 .38976 .6901 .39244 .68647 .38894 Metetra
0 .152 .639 r
.68647 .38894 .6901 .39244 .69832 .39741 .69478 .39544 Metetra
.293 .724 .969 r
.69478 .39544 .69832 .39741 .70628 .39676 .7028 .39559 Metetra
.952 .903 .738 r
.7028 .39559 .70628 .39676 .71365 .38561 .71011 .3828 Metetra
.927 .85 .746 r
.71011 .3828 .71365 .38561 .72101 .37458 .71742 .37036 Metetra
.82 .806 .838 r
.71742 .37036 .72101 .37458 .72864 .3681 .72504 .36348 Metetra
.731 .769 .883 r
.72504 .36348 .72864 .3681 .73643 .36406 .73285 .35935 Metetra
.693 .754 .898 r
.73285 .35935 .73643 .36406 .7443 .36087 .74073 .35613 Metetra
.678 .747 .903 r
.74073 .35613 .7443 .36087 .75221 .358 .74865 .35324 Metetra
.671 .744 .905 r
.74865 .35324 .75221 .358 .76015 .35527 .75661 .35048 Metetra
.668 .743 .907 r
.75661 .35048 .76015 .35527 .76811 .35259 .76459 .34779 Metetra
.666 .742 .907 r
.76459 .34779 .76811 .35259 .7761 .34994 .77259 .34513 Metetra
.665 .742 .907 r
.77259 .34513 .7761 .34994 .78412 .3473 .78062 .34248 Metetra
.665 .742 .908 r
.78062 .34248 .78412 .3473 .79216 .34467 .78868 .33983 Metetra
.664 .742 .908 r
.78868 .33983 .79216 .34467 .80022 .34204 .79675 .33718 Metetra
.664 .742 .908 r
.79675 .33718 .80022 .34204 .8083 .3394 .80485 .33453 Metetra
.664 .741 .908 r
.80485 .33453 .8083 .3394 .81641 .33676 .81298 .33188 Metetra
.664 .741 .908 r
.81298 .33188 .81641 .33676 .82454 .33411 .82112 .32922 Metetra
.664 .741 .908 r
.82112 .32922 .82454 .33411 .83269 .33146 .82929 .32655 Metetra
.664 .741 .908 r
.82929 .32655 .83269 .33146 .84086 .3288 .83748 .32388 Metetra
.664 .741 .908 r
.83748 .32388 .84086 .3288 .84906 .32614 .8457 .3212 Metetra
.664 .741 .908 r
.8457 .3212 .84906 .32614 .85728 .32347 .85393 .31852 Metetra
.664 .741 .908 r
.85393 .31852 .85728 .32347 .86553 .32079 .8622 .31582 Metetra
.664 .741 .908 r
.8622 .31582 .86553 .32079 .8738 .3181 .87048 .31312 Metetra
.664 .741 .908 r
.87048 .31312 .8738 .3181 .88209 .31541 .87879 .31041 Metetra
.664 .741 .908 r
.87879 .31041 .88209 .31541 .8904 .31271 .88712 .3077 Metetra
.69 .757 .903 r
.29004 .49887 .29443 .50273 .30121 .50014 .29684 .49616 Metetra
.675 .749 .906 r
.29684 .49616 .30121 .50014 .30799 .49776 .30364 .49373 Metetra
.668 .745 .908 r
.30364 .49373 .30799 .49776 .31478 .49549 .31045 .49143 Metetra
.664 .744 .909 r
.31045 .49143 .31478 .49549 .32159 .49327 .31726 .48919 Metetra
.662 .743 .91 r
.31726 .48919 .32159 .49327 .32841 .49107 .3241 .48698 Metetra
.661 .742 .91 r
.3241 .48698 .32841 .49107 .33525 .4889 .33094 .4848 Metetra
.659 .742 .911 r
.33094 .4848 .33525 .4889 .34211 .48674 .33781 .48263 Metetra
.658 .741 .912 r
.33781 .48263 .34211 .48674 .34898 .48461 .34469 .48049 Metetra
.656 .741 .912 r
.34469 .48049 .34898 .48461 .35586 .4825 .35159 .47837 Metetra
.653 .74 .913 r
.35159 .47837 .35586 .4825 .36277 .48044 .35851 .4763 Metetra
.649 .739 .915 r
.35851 .4763 .36277 .48044 .36969 .47843 .36544 .47429 Metetra
.643 .738 .918 r
.36544 .47429 .36969 .47843 .37662 .47652 .37238 .47238 Metetra
.632 .735 .922 r
.37238 .47238 .37662 .47652 .38357 .47477 .37934 .47063 Metetra
.612 .729 .929 r
.37934 .47063 .38357 .47477 .39052 .47331 .3863 .4692 Metetra
.564 .714 .942 r
.3863 .4692 .39052 .47331 .39748 .47242 .39326 .4684 Metetra
.45 .673 .959 r
.39326 .4684 .39748 .47242 .40442 .47246 .4002 .46876 Metetra
.539 .729 .962 r
.4002 .46876 .40442 .47246 .41143 .47146 .40721 .46811 Metetra
.789 .804 .865 r
.40721 .46811 .41143 .47146 .41858 .4667 .41439 .46282 Metetra
.764 .779 .866 r
.41439 .46282 .41858 .4667 .42574 .46225 .42157 .45813 Metetra
.723 .763 .884 r
.42157 .45813 .42574 .46225 .43287 .45873 .42872 .45452 Metetra
.7 .755 .894 r
.42872 .45452 .43287 .45873 .44001 .45568 .43587 .45143 Metetra
.688 .75 .899 r
.43587 .45143 .44001 .45568 .44715 .45288 .44303 .44861 Metetra
.681 .748 .902 r
.44303 .44861 .44715 .45288 .45431 .45022 .4502 .44593 Metetra
.676 .746 .903 r
.4502 .44593 .45431 .45022 .46149 .44764 .45739 .44333 Metetra
.673 .745 .905 r
.45739 .44333 .46149 .44764 .46868 .44511 .46459 .44079 Metetra
.671 .744 .905 r
.46459 .44079 .46868 .44511 .47589 .44262 .47181 .43828 Metetra
.669 .743 .906 r
.47181 .43828 .47589 .44262 .48312 .44016 .47905 .43581 Metetra
.666 .742 .907 r
.47905 .43581 .48312 .44016 .49036 .43775 .48631 .43338 Metetra
.662 .741 .909 r
.48631 .43338 .49036 .43775 .49763 .43542 .49359 .43104 Metetra
.651 .737 .913 r
.49359 .43104 .49763 .43542 .50491 .43326 .50088 .42889 Metetra
.635 .733 .919 r
.50088 .42889 .50491 .43326 .51221 .43132 .5082 .42702 Metetra
.663 .746 .912 r
.5082 .42702 .51221 .43132 .51953 .42894 .51553 .42464 Metetra
.686 .753 .903 r
.51553 .42464 .51953 .42894 .52688 .42617 .52289 .42179 Metetra
.677 .748 .905 r
.52289 .42179 .52688 .42617 .53424 .42352 .53027 .41911 Metetra
.67 .745 .907 r
.53027 .41911 .53424 .42352 .54162 .42099 .53766 .41657 Metetra
.664 .743 .909 r
.53766 .41657 .54162 .42099 .54902 .41856 .54507 .41413 Metetra
.657 .741 .912 r
.54507 .41413 .54902 .41856 .55644 .41626 .55251 .41182 Metetra
.649 .739 .915 r
.55251 .41182 .55644 .41626 .56389 .41411 .55997 .40967 Metetra
.635 .734 .92 r
.55997 .40967 .56389 .41411 .57136 .41217 .56745 .40774 Metetra
.61 .726 .927 r
.56745 .40774 .57136 .41217 .57885 .41063 .57496 .40622 Metetra
.55 .704 .942 r
.57496 .40622 .57885 .41063 .58639 .40986 .58251 .40557 Metetra
.387 .632 .952 r
.58251 .40557 .58639 .40986 .59399 .41052 .59013 .40666 Metetra
.455 .692 .968 r
.59013 .40666 .59399 .41052 .6016 .41035 .59776 .40685 Metetra
.758 .819 .9 r
.59776 .40685 .6016 .41035 .60914 .40639 .60531 .40236 Metetra
.741 .79 .892 r
.60531 .40236 .60914 .40639 .6167 .40246 .61288 .39819 Metetra
.703 .769 .903 r
.61288 .39819 .6167 .40246 .6243 .39923 .62049 .39487 Metetra
.68 .758 .91 r
.62049 .39487 .6243 .39923 .63192 .39643 .62813 .39204 Metetra
.666 .753 .915 r
.62813 .39204 .63192 .39643 .63958 .39386 .6358 .38947 Metetra
.664 .753 .916 r
.6358 .38947 .63958 .39386 .64725 .39129 .64349 .38695 Metetra
.707 .769 .901 r
.64349 .38695 .64725 .39129 .65491 .38783 .65117 .38351 Metetra
.799 .792 .846 r
.65117 .38351 .65491 .38783 .66251 .38248 .65875 .37737 Metetra
.315 .549 .916 r
.65875 .37737 .66251 .38248 .67042 .38404 .66671 .37973 Metetra
.302 .585 .941 r
.66671 .37973 .67042 .38404 .67839 .3857 .67469 .38167 Metetra
.044 .434 .871 r
.67469 .38167 .67839 .3857 .68647 .38894 .68282 .38551 Metetra
.523 .014 0 r
.68282 .38551 .68647 .38894 .69478 .39544 .69123 .39389 Metetra
.062 .589 .868 r
.69123 .39389 .69478 .39544 .7028 .39559 .69934 .39513 Metetra
.97 .904 .696 r
.69934 .39513 .7028 .39559 .71011 .3828 .70657 .38028 Metetra
.944 .851 .72 r
.70657 .38028 .71011 .3828 .71742 .37036 .7138 .36617 Metetra
.83 .809 .83 r
.7138 .36617 .71742 .37036 .72504 .36348 .72142 .35883 Metetra
.734 .77 .882 r
.72142 .35883 .72504 .36348 .73285 .35935 .72924 .3546 Metetra
.694 .754 .898 r
.72924 .3546 .73285 .35935 .74073 .35613 .73714 .35135 Metetra
.678 .747 .903 r
.73714 .35135 .74073 .35613 .74865 .35324 .74508 .34844 Metetra
.671 .744 .905 r
.74508 .34844 .74865 .35324 .75661 .35048 .75305 .34568 Metetra
.668 .743 .907 r
.75305 .34568 .75661 .35048 .76459 .34779 .76104 .34297 Metetra
.666 .742 .907 r
.76104 .34297 .76459 .34779 .77259 .34513 .76907 .34029 Metetra
.665 .742 .907 r
.76907 .34029 .77259 .34513 .78062 .34248 .77711 .33763 Metetra
.665 .742 .908 r
.77711 .33763 .78062 .34248 .78868 .33983 .78518 .33497 Metetra
.664 .742 .908 r
.78518 .33497 .78868 .33983 .79675 .33718 .79327 .33231 Metetra
.664 .742 .908 r
.79327 .33231 .79675 .33718 .80485 .33453 .80139 .32964 Metetra
.664 .741 .908 r
.80139 .32964 .80485 .33453 .81298 .33188 .80953 .32697 Metetra
.664 .741 .908 r
.80953 .32697 .81298 .33188 .82112 .32922 .81769 .3243 Metetra
.664 .741 .908 r
.81769 .3243 .82112 .32922 .82929 .32655 .82587 .32162 Metetra
.664 .741 .908 r
.82587 .32162 .82929 .32655 .83748 .32388 .83408 .31893 Metetra
.664 .741 .908 r
.83408 .31893 .83748 .32388 .8457 .3212 .84231 .31624 Metetra
.664 .741 .908 r
.84231 .31624 .8457 .3212 .85393 .31852 .85057 .31354 Metetra
.664 .741 .908 r
.85057 .31354 .85393 .31852 .8622 .31582 .85885 .31083 Metetra
.664 .741 .908 r
.85885 .31083 .8622 .31582 .87048 .31312 .86715 .30812 Metetra
.664 .741 .908 r
.86715 .30812 .87048 .31312 .87879 .31041 .87548 .3054 Metetra
.664 .741 .908 r
.87548 .3054 .87879 .31041 .88712 .3077 .88382 .30267 Metetra
.696 .759 .901 r
.28564 .49501 .29004 .49887 .29684 .49616 .29246 .49217 Metetra
.677 .749 .906 r
.29246 .49217 .29684 .49616 .30364 .49373 .29927 .48968 Metetra
.669 .746 .908 r
.29927 .48968 .30364 .49373 .31045 .49143 .30609 .48736 Metetra
.665 .744 .909 r
.30609 .48736 .31045 .49143 .31726 .48919 .31291 .4851 Metetra
.662 .743 .91 r
.31291 .4851 .31726 .48919 .3241 .48698 .31976 .48288 Metetra
.661 .742 .91 r
.31976 .48288 .3241 .48698 .33094 .4848 .32662 .48068 Metetra
.659 .742 .911 r
.32662 .48068 .33094 .4848 .33781 .48263 .33349 .4785 Metetra
.658 .741 .912 r
.33349 .4785 .33781 .48263 .34469 .48049 .34039 .47635 Metetra
.656 .741 .912 r
.34039 .47635 .34469 .48049 .35159 .47837 .3473 .47422 Metetra
.653 .74 .914 r
.3473 .47422 .35159 .47837 .35851 .4763 .35422 .47214 Metetra
.649 .739 .915 r
.35422 .47214 .35851 .4763 .36544 .47429 .36116 .47012 Metetra
.642 .737 .918 r
.36116 .47012 .36544 .47429 .37238 .47238 .36812 .46821 Metetra
.631 .735 .922 r
.36812 .46821 .37238 .47238 .37934 .47063 .37508 .46647 Metetra
.609 .728 .93 r
.37508 .46647 .37934 .47063 .3863 .4692 .38206 .46506 Metetra
.555 .711 .944 r
.38206 .46506 .3863 .4692 .39326 .4684 .38902 .46436 Metetra
.405 .652 .959 r
.38902 .46436 .39326 .4684 .4002 .46876 .39595 .46509 Metetra
.491 .713 .969 r
.39595 .46509 .4002 .46876 .40721 .46811 .40295 .46489 Metetra
.808 .812 .853 r
.40295 .46489 .40721 .46811 .41439 .46282 .41018 .45893 Metetra
.771 .781 .861 r
.41018 .45893 .41439 .46282 .42157 .45813 .41738 .45398 Metetra
.726 .763 .883 r
.41738 .45398 .42157 .45813 .42872 .45452 .42455 .45029 Metetra
.701 .755 .893 r
.42455 .45029 .42872 .45452 .43587 .45143 .43171 .44716 Metetra
.689 .75 .899 r
.43171 .44716 .43587 .45143 .44303 .44861 .43888 .44431 Metetra
.681 .748 .902 r
.43888 .44431 .44303 .44861 .4502 .44593 .44607 .44161 Metetra
.677 .746 .903 r
.44607 .44161 .4502 .44593 .45739 .44333 .45327 .439 Metetra
.674 .745 .904 r
.45327 .439 .45739 .44333 .46459 .44079 .46049 .43644 Metetra
.671 .744 .905 r
.46049 .43644 .46459 .44079 .47181 .43828 .46772 .43392 Metetra
.669 .743 .906 r
.46772 .43392 .47181 .43828 .47905 .43581 .47497 .43143 Metetra
.667 .743 .907 r
.47497 .43143 .47905 .43581 .48631 .43338 .48224 .42898 Metetra
.662 .741 .909 r
.48224 .42898 .48631 .43338 .49359 .43104 .48953 .42663 Metetra
.65 .737 .913 r
.48953 .42663 .49359 .43104 .50088 .42889 .49684 .42449 Metetra
.628 .731 .921 r
.49684 .42449 .50088 .42889 .5082 .42702 .50416 .42272 Metetra
.663 .747 .913 r
.50416 .42272 .5082 .42702 .51553 .42464 .51151 .42032 Metetra
.689 .755 .902 r
.51151 .42032 .51553 .42464 .52289 .42179 .51889 .41739 Metetra
.678 .749 .904 r
.51889 .41739 .52289 .42179 .53027 .41911 .52627 .41469 Metetra
.67 .745 .907 r
.52627 .41469 .53027 .41911 .53766 .41657 .53368 .41213 Metetra
.664 .743 .909 r
.53368 .41213 .53766 .41657 .54507 .41413 .54111 .40968 Metetra
.657 .741 .912 r
.54111 .40968 .54507 .41413 .55251 .41182 .54856 .40736 Metetra
.648 .738 .915 r
.54856 .40736 .55251 .41182 .55997 .40967 .55603 .40521 Metetra
.634 .734 .92 r
.55603 .40521 .55997 .40967 .56745 .40774 .56352 .40329 Metetra
.608 .725 .928 r
.56352 .40329 .56745 .40774 .57496 .40622 .57105 .4018 Metetra
.539 .699 .943 r
.57105 .4018 .57496 .40622 .58251 .40557 .57861 .40127 Metetra
.329 .601 .946 r
.57861 .40127 .58251 .40557 .59013 .40666 .58625 .40287 Metetra
.4 .67 .969 r
.58625 .40287 .59013 .40666 .59776 .40685 .5939 .40353 Metetra
.784 .831 .889 r
.5939 .40353 .59776 .40685 .60531 .40236 .60146 .39834 Metetra
.751 .793 .887 r
.60146 .39834 .60531 .40236 .61288 .39819 .60904 .3939 Metetra
.707 .77 .902 r
.60904 .3939 .61288 .39819 .62049 .39487 .61666 .39049 Metetra
.681 .759 .91 r
.61666 .39049 .62049 .39487 .62813 .39204 .62431 .38763 Metetra
.665 .753 .916 r
.62431 .38763 .62813 .39204 .6358 .38947 .632 .38508 Metetra
.66 .752 .918 r
.632 .38508 .6358 .38947 .64349 .38695 .63971 .38261 Metetra
.703 .769 .903 r
.63971 .38261 .64349 .38695 .65117 .38351 .6474 .37925 Metetra
.822 .798 .828 r
.6474 .37925 .65117 .38351 .65875 .37737 .65496 .37213 Metetra
.222 .487 .895 r
.65496 .37213 .65875 .37737 .66671 .37973 .66298 .37547 Metetra
.268 .569 .937 r
.66298 .37547 .66671 .37973 .67469 .38167 .67098 .37765 Metetra
0 .372 .83 r
.67098 .37765 .67469 .38167 .68282 .38551 .67915 .38214 Metetra
.666 .165 0 r
.67915 .38214 .68282 .38551 .69123 .39389 .68769 .39283 Metetra
.24 0 0 r
.68769 .39283 .69123 .39389 .69934 .39513 .69589 .39555 Metetra
.983 .9 .646 r
.69589 .39555 .69934 .39513 .70657 .38028 .70302 .37814 Metetra
.959 .85 .69 r
.70302 .37814 .70657 .38028 .7138 .36617 .71017 .36198 Metetra
.841 .811 .821 r
.71017 .36198 .7138 .36617 .72142 .35883 .71779 .35415 Metetra
.737 .771 .88 r
.71779 .35415 .72142 .35883 .72924 .3546 .72562 .34984 Metetra
.695 .754 .897 r
.72562 .34984 .72924 .3546 .73714 .35135 .73353 .34655 Metetra
.678 .747 .903 r
.73353 .34655 .73714 .35135 .74508 .34844 .74148 .34363 Metetra
.671 .744 .905 r
.74148 .34363 .74508 .34844 .75305 .34568 .74947 .34084 Metetra
.668 .743 .907 r
.74947 .34084 .75305 .34568 .76104 .34297 .75748 .33812 Metetra
.666 .742 .907 r
.75748 .33812 .76104 .34297 .76907 .34029 .76552 .33543 Metetra
.665 .742 .907 r
.76552 .33543 .76907 .34029 .77711 .33763 .77358 .33275 Metetra
.665 .742 .908 r
.77358 .33275 .77711 .33763 .78518 .33497 .78167 .33008 Metetra
.664 .742 .908 r
.78167 .33008 .78518 .33497 .79327 .33231 .78977 .3274 Metetra
.664 .742 .908 r
.78977 .3274 .79327 .33231 .80139 .32964 .79791 .32472 Metetra
.664 .741 .908 r
.79791 .32472 .80139 .32964 .80953 .32697 .80606 .32204 Metetra
.664 .741 .908 r
.80606 .32204 .80953 .32697 .81769 .3243 .81424 .31936 Metetra
.664 .741 .908 r
.81424 .31936 .81769 .3243 .82587 .32162 .82244 .31666 Metetra
.664 .741 .908 r
.82244 .31666 .82587 .32162 .83408 .31893 .83067 .31396 Metetra
.664 .741 .908 r
.83067 .31396 .83408 .31893 .84231 .31624 .83891 .31125 Metetra
.664 .741 .908 r
.83891 .31125 .84231 .31624 .85057 .31354 .84719 .30854 Metetra
.664 .741 .908 r
.84719 .30854 .85057 .31354 .85885 .31083 .85548 .30582 Metetra
.664 .741 .908 r
.85548 .30582 .85885 .31083 .86715 .30812 .8638 .30309 Metetra
.664 .741 .908 r
.8638 .30309 .86715 .30812 .87548 .3054 .87214 .30035 Metetra
.664 .741 .908 r
.87214 .30035 .87548 .3054 .88382 .30267 .88051 .29761 Metetra
.702 .761 .898 r
.28121 .49115 .28564 .49501 .29246 .49217 .28805 .48816 Metetra
.679 .75 .905 r
.28805 .48816 .29246 .49217 .29927 .48968 .29488 .48562 Metetra
.67 .746 .908 r
.29488 .48562 .29927 .48968 .30609 .48736 .3017 .48327 Metetra
.665 .744 .909 r
.3017 .48327 .30609 .48736 .31291 .4851 .30854 .48099 Metetra
.663 .743 .91 r
.30854 .48099 .31291 .4851 .31976 .48288 .3154 .47875 Metetra
.661 .742 .91 r
.3154 .47875 .31976 .48288 .32662 .48068 .32227 .47654 Metetra
.659 .742 .911 r
.32227 .47654 .32662 .48068 .33349 .4785 .32916 .47435 Metetra
.658 .741 .912 r
.32916 .47435 .33349 .4785 .34039 .47635 .33606 .47218 Metetra
.655 .741 .912 r
.33606 .47218 .34039 .47635 .3473 .47422 .34298 .47005 Metetra
.653 .74 .914 r
.34298 .47005 .3473 .47422 .35422 .47214 .34992 .46796 Metetra
.649 .739 .915 r
.34992 .46796 .35422 .47214 .36116 .47012 .35687 .46594 Metetra
.642 .737 .918 r
.35687 .46594 .36116 .47012 .36812 .46821 .36383 .46402 Metetra
.63 .734 .922 r
.36383 .46402 .36812 .46821 .37508 .46647 .37081 .46229 Metetra
.607 .728 .931 r
.37081 .46229 .37508 .46647 .38206 .46506 .37779 .46091 Metetra
.547 .708 .946 r
.37779 .46091 .38206 .46506 .38902 .46436 .38476 .4603 Metetra
.35 .624 .955 r
.38476 .4603 .38902 .46436 .39595 .46509 .39168 .46146 Metetra
.415 .682 .972 r
.39168 .46146 .39595 .46509 .40295 .46489 .39867 .46188 Metetra
.831 .819 .838 r
.39867 .46188 .40295 .46489 .41018 .45893 .40595 .45505 Metetra
.779 .782 .855 r
.40595 .45505 .41018 .45893 .41738 .45398 .41317 .44981 Metetra
.728 .764 .881 r
.41317 .44981 .41738 .45398 .42455 .45029 .42036 .44603 Metetra
.703 .755 .893 r
.42036 .44603 .42455 .45029 .43171 .44716 .42753 .44287 Metetra
.689 .751 .898 r
.42753 .44287 .43171 .44716 .43888 .44431 .43472 .44 Metetra
.682 .748 .901 r
.43472 .44 .43888 .44431 .44607 .44161 .44192 .43728 Metetra
.677 .746 .903 r
.44192 .43728 .44607 .44161 .45327 .439 .44913 .43465 Metetra
.674 .745 .904 r
.44913 .43465 .45327 .439 .46049 .43644 .45636 .43207 Metetra
.671 .744 .905 r
.45636 .43207 .46049 .43644 .46772 .43392 .46361 .42953 Metetra
.669 .744 .906 r
.46361 .42953 .46772 .43392 .47497 .43143 .47087 .42703 Metetra
.667 .743 .907 r
.47087 .42703 .47497 .43143 .48224 .42898 .47815 .42457 Metetra
.662 .741 .909 r
.47815 .42457 .48224 .42898 .48953 .42663 .48546 .4222 Metetra
.649 .737 .913 r
.48546 .4222 .48953 .42663 .49684 .42449 .49277 .42007 Metetra
.621 .728 .923 r
.49277 .42007 .49684 .42449 .50416 .42272 .50011 .41841 Metetra
.663 .748 .914 r
.50011 .41841 .50416 .42272 .51151 .42032 .50747 .416 Metetra
.693 .757 .901 r
.50747 .416 .51151 .42032 .51889 .41739 .51486 .41298 Metetra
.679 .749 .904 r
.51486 .41298 .51889 .41739 .52627 .41469 .52226 .41024 Metetra
.67 .745 .907 r
.52226 .41024 .52627 .41469 .53368 .41213 .52968 .40767 Metetra
.664 .743 .909 r
.52968 .40767 .53368 .41213 .54111 .40968 .53712 .40521 Metetra
.657 .741 .912 r
.53712 .40521 .54111 .40968 .54856 .40736 .54458 .40289 Metetra
.648 .738 .915 r
.54458 .40289 .54856 .40736 .55603 .40521 .55207 .40072 Metetra
.633 .734 .92 r
.55207 .40072 .55603 .40521 .56352 .40329 .55958 .39881 Metetra
.605 .724 .929 r
.55958 .39881 .56352 .40329 .57105 .4018 .56711 .39736 Metetra
.528 .695 .945 r
.56711 .39736 .57105 .4018 .57861 .40127 .57469 .39696 Metetra
.256 .56 .933 r
.57469 .39696 .57861 .40127 .58625 .40287 .58235 .39915 Metetra
.32 .632 .962 r
.58235 .39915 .58625 .40287 .5939 .40353 .59003 .40042 Metetra
.814 .845 .872 r
.59003 .40042 .5939 .40353 .60146 .39834 .59758 .39433 Metetra
.761 .796 .882 r
.59758 .39433 .60146 .39834 .60904 .3939 .60517 .38959 Metetra
.71 .771 .9 r
.60517 .38959 .60904 .3939 .61666 .39049 .61281 .3861 Metetra
.682 .759 .91 r
.61281 .3861 .61666 .39049 .62431 .38763 .62048 .38321 Metetra
.664 .753 .916 r
.62048 .38321 .62431 .38763 .632 .38508 .62818 .38066 Metetra
.656 .751 .92 r
.62818 .38066 .632 .38508 .63971 .38261 .6359 .37826 Metetra
.696 .768 .908 r
.6359 .37826 .63971 .38261 .6474 .37925 .64362 .37506 Metetra
.837 .803 .818 r
.64362 .37506 .6474 .37925 .65496 .37213 .65118 .36731 Metetra
.109 .437 .876 r
.65118 .36731 .65496 .37213 .66298 .37547 .65923 .37127 Metetra
.235 .553 .932 r
.65923 .37127 .66298 .37547 .67098 .37765 .66726 .37366 Metetra
0 .303 .78 r
.66726 .37366 .67098 .37765 .67915 .38214 .67547 .37885 Metetra
.769 .29 0 r
.67547 .37885 .67915 .38214 .68769 .39283 .68415 .39235 Metetra
.517 0 0 r
.68415 .39235 .68769 .39283 .69589 .39555 .69248 .39711 Metetra
.99 .89 .59 r
.69248 .39711 .69589 .39555 .70302 .37814 .69948 .37646 Metetra
.973 .846 .657 r
.69948 .37646 .70302 .37814 .71017 .36198 .70653 .35782 Metetra
.851 .813 .812 r
.70653 .35782 .71017 .36198 .71779 .35415 .71413 .34945 Metetra
.74 .772 .879 r
.71413 .34945 .71779 .35415 .72562 .34984 .72197 .34504 Metetra
.695 .754 .897 r
.72197 .34504 .72562 .34984 .73353 .34655 .7299 .34173 Metetra
.679 .747 .903 r
.7299 .34173 .73353 .34655 .74148 .34363 .73787 .33878 Metetra
.671 .744 .905 r
.73787 .33878 .74148 .34363 .74947 .34084 .74587 .33598 Metetra
.668 .743 .906 r
.74587 .33598 .74947 .34084 .75748 .33812 .7539 .33325 Metetra
.666 .742 .907 r
.7539 .33325 .75748 .33812 .76552 .33543 .76195 .33054 Metetra
.665 .742 .907 r
.76195 .33054 .76552 .33543 .77358 .33275 .77003 .32785 Metetra
.665 .742 .908 r
.77003 .32785 .77358 .33275 .78167 .33008 .77813 .32516 Metetra
.664 .742 .908 r
.77813 .32516 .78167 .33008 .78977 .3274 .78626 .32247 Metetra
.664 .742 .908 r
.78626 .32247 .78977 .3274 .79791 .32472 .79441 .31978 Metetra
.664 .741 .908 r
.79441 .31978 .79791 .32472 .80606 .32204 .80258 .31709 Metetra
.664 .741 .908 r
.80258 .31709 .80606 .32204 .81424 .31936 .81077 .31438 Metetra
.664 .741 .908 r
.81077 .31438 .81424 .31936 .82244 .31666 .81899 .31168 Metetra
.664 .741 .908 r
.81899 .31168 .82244 .31666 .83067 .31396 .82723 .30896 Metetra
.664 .741 .908 r
.82723 .30896 .83067 .31396 .83891 .31125 .8355 .30624 Metetra
.664 .741 .908 r
.8355 .30624 .83891 .31125 .84719 .30854 .84379 .30351 Metetra
.664 .741 .908 r
.84379 .30351 .84719 .30854 .85548 .30582 .8521 .30078 Metetra
.664 .741 .908 r
.8521 .30078 .85548 .30582 .8638 .30309 .86043 .29803 Metetra
.664 .741 .908 r
.86043 .29803 .8638 .30309 .87214 .30035 .86879 .29528 Metetra
.664 .741 .908 r
.86879 .29528 .87214 .30035 .88051 .29761 .87718 .29252 Metetra
.71 .764 .894 r
.27676 .48729 .28121 .49115 .28805 .48816 .28362 .48413 Metetra
.682 .751 .904 r
.28362 .48413 .28805 .48816 .29488 .48562 .29046 .48154 Metetra
.671 .746 .907 r
.29046 .48154 .29488 .48562 .3017 .48327 .2973 .47915 Metetra
.666 .744 .909 r
.2973 .47915 .3017 .48327 .30854 .48099 .30415 .47686 Metetra
.663 .743 .91 r
.30415 .47686 .30854 .48099 .3154 .47875 .31102 .4746 Metetra
.661 .742 .91 r
.31102 .4746 .3154 .47875 .32227 .47654 .3179 .47238 Metetra
.659 .742 .911 r
.3179 .47238 .32227 .47654 .32916 .47435 .3248 .47018 Metetra
.658 .741 .912 r
.3248 .47018 .32916 .47435 .33606 .47218 .33171 .468 Metetra
.655 .741 .912 r
.33171 .468 .33606 .47218 .34298 .47005 .33864 .46586 Metetra
.652 .74 .914 r
.33864 .46586 .34298 .47005 .34992 .46796 .34559 .46376 Metetra
.648 .739 .915 r
.34559 .46376 .34992 .46796 .35687 .46594 .35255 .46173 Metetra
.641 .737 .918 r
.35255 .46173 .35687 .46594 .36383 .46402 .35953 .45982 Metetra
.629 .734 .923 r
.35953 .45982 .36383 .46402 .37081 .46229 .36651 .45809 Metetra
.604 .727 .931 r
.36651 .45809 .37081 .46229 .37779 .46091 .3735 .45673 Metetra
.538 .704 .947 r
.3735 .45673 .37779 .46091 .38476 .4603 .38048 .45622 Metetra
.282 .588 .945 r
.38048 .45622 .38476 .4603 .39168 .46146 .38739 .45786 Metetra
.29 .622 .96 r
.38739 .45786 .39168 .46146 .39867 .46188 .39435 .45914 Metetra
.855 .826 .818 r
.39435 .45914 .39867 .46188 .40595 .45505 .40169 .45116 Metetra
.787 .783 .849 r
.40169 .45116 .40595 .45505 .41317 .44981 .40894 .44562 Metetra
.731 .764 .879 r
.40894 .44562 .41317 .44981 .42036 .44603 .41614 .44176 Metetra
.704 .755 .892 r
.41614 .44176 .42036 .44603 .42753 .44287 .42334 .43856 Metetra
.69 .751 .898 r
.42334 .43856 .42753 .44287 .43472 .44 .43053 .43566 Metetra
.682 .748 .901 r
.43053 .43566 .43472 .44 .44192 .43728 .43774 .43292 Metetra
.677 .746 .903 r
.43774 .43292 .44192 .43728 .44913 .43465 .44497 .43027 Metetra
.674 .745 .904 r
.44497 .43027 .44913 .43465 .45636 .43207 .45221 .42768 Metetra
.672 .744 .905 r
.45221 .42768 .45636 .43207 .46361 .42953 .45947 .42513 Metetra
.67 .744 .906 r
.45947 .42513 .46361 .42953 .47087 .42703 .46675 .42261 Metetra
.667 .743 .907 r
.46675 .42261 .47087 .42703 .47815 .42457 .47405 .42013 Metetra
.662 .741 .909 r
.47405 .42013 .47815 .42457 .48546 .4222 .48136 .41774 Metetra
.648 .736 .914 r
.48136 .41774 .48546 .4222 .49277 .42007 .48869 .41562 Metetra
.611 .725 .926 r
.48869 .41562 .49277 .42007 .50011 .41841 .49604 .4141 Metetra
.663 .749 .914 r
.49604 .4141 .50011 .41841 .50747 .416 .50341 .41168 Metetra
.698 .759 .899 r
.50341 .41168 .50747 .416 .51486 .41298 .51081 .40854 Metetra
.68 .749 .904 r
.51081 .40854 .51486 .41298 .52226 .41024 .51823 .40577 Metetra
.67 .745 .907 r
.51823 .40577 .52226 .41024 .52968 .40767 .52566 .40318 Metetra
.664 .743 .909 r
.52566 .40318 .52968 .40767 .53712 .40521 .53312 .40072 Metetra
.657 .741 .912 r
.53312 .40072 .53712 .40521 .54458 .40289 .54059 .39839 Metetra
.647 .738 .916 r
.54059 .39839 .54458 .40289 .55207 .40072 .54809 .39622 Metetra
.632 .733 .921 r
.54809 .39622 .55207 .40072 .55958 .39881 .55561 .39431 Metetra
.602 .723 .93 r
.55561 .39431 .55958 .39881 .56711 .39736 .56316 .39289 Metetra
.517 .69 .946 r
.56316 .39289 .56711 .39736 .57469 .39696 .57075 .39263 Metetra
.165 .504 .91 r
.57075 .39263 .57469 .39696 .58235 .39915 .57843 .39552 Metetra
.202 .569 .938 r
.57843 .39552 .58235 .39915 .59003 .40042 .58614 .39759 Metetra
.848 .858 .849 r
.58614 .39759 .59003 .40042 .59758 .39433 .59369 .39033 Metetra
.771 .799 .876 r
.59369 .39033 .59758 .39433 .60517 .38959 .60129 .38527 Metetra
.713 .772 .899 r
.60129 .38527 .60517 .38959 .61281 .3861 .60894 .38168 Metetra
.682 .76 .91 r
.60894 .38168 .61281 .3861 .62048 .38321 .61662 .37876 Metetra
.663 .752 .917 r
.61662 .37876 .62048 .38321 .62818 .38066 .62434 .37623 Metetra
.651 .75 .922 r
.62434 .37623 .62818 .38066 .6359 .37826 .63208 .37392 Metetra
.683 .765 .914 r
.63208 .37392 .6359 .37826 .64362 .37506 .63983 .37097 Metetra
.833 .807 .827 r
.63983 .37097 .64362 .37506 .65118 .36731 .64742 .36352 Metetra
.025 .439 .869 r
.64742 .36352 .65118 .36731 .65923 .37127 .65546 .36715 Metetra
.206 .541 .927 r
.65546 .36715 .65923 .37127 .66726 .37366 .66351 .3697 Metetra
0 .23 .721 r
.66351 .3697 .66726 .37366 .67547 .37885 .67176 .37561 Metetra
.837 .385 0 r
.67176 .37561 .67547 .37885 .68415 .39235 .68061 .39257 Metetra
.698 .161 0 r
.68061 .39257 .68415 .39235 .69248 .39711 .6891 .40014 Metetra
.99 .873 .53 r
.6891 .40014 .69248 .39711 .69948 .37646 .69594 .37539 Metetra
.984 .841 .621 r
.69594 .37539 .69948 .37646 .70653 .35782 .70286 .35366 Metetra
.862 .814 .801 r
.70286 .35366 .70653 .35782 .71413 .34945 .71045 .34472 Metetra
.742 .772 .877 r
.71045 .34472 .71413 .34945 .72197 .34504 .71831 .34022 Metetra
.696 .754 .897 r
.71831 .34022 .72197 .34504 .7299 .34173 .72625 .33688 Metetra
.679 .747 .903 r
.72625 .33688 .7299 .34173 .73787 .33878 .73424 .33391 Metetra
.671 .744 .905 r
.73424 .33391 .73787 .33878 .74587 .33598 .74226 .3311 Metetra
.668 .743 .906 r
.74226 .3311 .74587 .33598 .7539 .33325 .7503 .32835 Metetra
.666 .742 .907 r
.7503 .32835 .7539 .33325 .76195 .33054 .75837 .32563 Metetra
.665 .742 .907 r
.75837 .32563 .76195 .33054 .77003 .32785 .76646 .32292 Metetra
.665 .742 .908 r
.76646 .32292 .77003 .32785 .77813 .32516 .77458 .32022 Metetra
.664 .742 .908 r
.77458 .32022 .77813 .32516 .78626 .32247 .78272 .31752 Metetra
.664 .742 .908 r
.78272 .31752 .78626 .32247 .79441 .31978 .79089 .31481 Metetra
.664 .741 .908 r
.79089 .31481 .79441 .31978 .80258 .31709 .79907 .3121 Metetra
.664 .741 .908 r
.79907 .3121 .80258 .31709 .81077 .31438 .80729 .30939 Metetra
.664 .741 .908 r
.80729 .30939 .81077 .31438 .81899 .31168 .81552 .30666 Metetra
.664 .741 .908 r
.81552 .30666 .81899 .31168 .82723 .30896 .82378 .30394 Metetra
.664 .741 .908 r
.82378 .30394 .82723 .30896 .8355 .30624 .83206 .3012 Metetra
.664 .741 .908 r
.83206 .3012 .8355 .30624 .84379 .30351 .84037 .29846 Metetra
.664 .741 .908 r
.84037 .29846 .84379 .30351 .8521 .30078 .8487 .29571 Metetra
.664 .741 .908 r
.8487 .29571 .8521 .30078 .86043 .29803 .85705 .29295 Metetra
.664 .741 .908 r
.85705 .29295 .86043 .29803 .86879 .29528 .86543 .29018 Metetra
.664 .741 .908 r
.86543 .29018 .86879 .29528 .87718 .29252 .87383 .28741 Metetra
.718 .766 .891 r
.27228 .48344 .27676 .48729 .28362 .48413 .27917 .48008 Metetra
.684 .751 .902 r
.27917 .48008 .28362 .48413 .29046 .48154 .28603 .47743 Metetra
.671 .746 .907 r
.28603 .47743 .29046 .48154 .2973 .47915 .29288 .47502 Metetra
.666 .744 .909 r
.29288 .47502 .2973 .47915 .30415 .47686 .29974 .47271 Metetra
.663 .743 .909 r
.29974 .47271 .30415 .47686 .31102 .4746 .30662 .47044 Metetra
.661 .742 .91 r
.30662 .47044 .31102 .4746 .3179 .47238 .31351 .4682 Metetra
.659 .742 .911 r
.31351 .4682 .3179 .47238 .3248 .47018 .32042 .46599 Metetra
.658 .741 .912 r
.32042 .46599 .3248 .47018 .33171 .468 .32734 .4638 Metetra
.655 .741 .912 r
.32734 .4638 .33171 .468 .33864 .46586 .33429 .46165 Metetra
.652 .74 .914 r
.33429 .46165 .33864 .46586 .34559 .46376 .34124 .45954 Metetra
.648 .739 .916 r
.34124 .45954 .34559 .46376 .35255 .46173 .34822 .45751 Metetra
.641 .737 .918 r
.34822 .45751 .35255 .46173 .35953 .45982 .3552 .45559 Metetra
.628 .734 .923 r
.3552 .45559 .35953 .45982 .36651 .45809 .3622 .45386 Metetra
.602 .726 .932 r
.3622 .45386 .36651 .45809 .3735 .45673 .3692 .45253 Metetra
.528 .7 .949 r
.3692 .45253 .3735 .45673 .38048 .45622 .37618 .45211 Metetra
.199 .54 .927 r
.37618 .45211 .38048 .45622 .38739 .45786 .38307 .45429 Metetra
.08 .498 .899 r
.38307 .45429 .38739 .45786 .39435 .45914 .38999 .45678 Metetra
.882 .832 .793 r
.38999 .45678 .39435 .45914 .40169 .45116 .39741 .44726 Metetra
.795 .784 .843 r
.39741 .44726 .40169 .45116 .40894 .44562 .40469 .4414 Metetra
.733 .764 .878 r
.40469 .4414 .40894 .44562 .41614 .44176 .41191 .43746 Metetra
.705 .755 .891 r
.41191 .43746 .41614 .44176 .42334 .43856 .41912 .43423 Metetra
.69 .751 .898 r
.41912 .43423 .42334 .43856 .43053 .43566 .42633 .4313 Metetra
.682 .748 .901 r
.42633 .4313 .43053 .43566 .43774 .43292 .43355 .42854 Metetra
.677 .746 .903 r
.43355 .42854 .43774 .43292 .44497 .43027 .44079 .42588 Metetra
.674 .745 .904 r
.44079 .42588 .44497 .43027 .45221 .42768 .44804 .42327 Metetra
.672 .744 .905 r
.44804 .42327 .45221 .42768 .45947 .42513 .45532 .4207 Metetra
.67 .744 .906 r
.45532 .4207 .45947 .42513 .46675 .42261 .46261 .41816 Metetra
.668 .743 .906 r
.46261 .41816 .46675 .42261 .47405 .42013 .46992 .41566 Metetra
.662 .741 .908 r
.46992 .41566 .47405 .42013 .48136 .41774 .47724 .41326 Metetra
.647 .736 .914 r
.47724 .41326 .48136 .41774 .48869 .41562 .48459 .41115 Metetra
.599 .72 .929 r
.48459 .41115 .48869 .41562 .49604 .4141 .49194 .40981 Metetra
.664 .751 .915 r
.49194 .40981 .49604 .4141 .50341 .41168 .49933 .40735 Metetra
.703 .761 .897 r
.49933 .40735 .50341 .41168 .51081 .40854 .50675 .40408 Metetra
.681 .749 .903 r
.50675 .40408 .51081 .40854 .51823 .40577 .51418 .40128 Metetra
.67 .745 .907 r
.51418 .40128 .51823 .40577 .52566 .40318 .52162 .39868 Metetra
.664 .743 .909 r
.52162 .39868 .52566 .40318 .53312 .40072 .52909 .3962 Metetra
.656 .741 .912 r
.52909 .3962 .53312 .40072 .54059 .39839 .53658 .39386 Metetra
.647 .738 .916 r
.53658 .39386 .54059 .39839 .54809 .39622 .54409 .3917 Metetra
.631 .733 .921 r
.54409 .3917 .54809 .39622 .55561 .39431 .55162 .3898 Metetra
.599 .722 .931 r
.55162 .3898 .55561 .39431 .56316 .39289 .55919 .38841 Metetra
.505 .684 .947 r
.55919 .38841 .56316 .39289 .57075 .39263 .56679 .38828 Metetra
.053 .43 .87 r
.56679 .38828 .57075 .39263 .57843 .39552 .57449 .39199 Metetra
.027 .459 .875 r
.57449 .39199 .57843 .39552 .58614 .39759 .58223 .39515 Metetra
.883 .869 .819 r
.58223 .39515 .58614 .39759 .59369 .39033 .58977 .38632 Metetra
.782 .802 .869 r
.58977 .38632 .59369 .39033 .60129 .38527 .59738 .38092 Metetra
.717 .773 .897 r
.59738 .38092 .60129 .38527 .60894 .38168 .60505 .37724 Metetra
.683 .76 .91 r
.60505 .37724 .60894 .38168 .61662 .37876 .61275 .3743 Metetra
.661 .752 .918 r
.61275 .3743 .61662 .37876 .62434 .37623 .62048 .37179 Metetra
.645 .748 .924 r
.62048 .37179 .62434 .37623 .63208 .37392 .62824 .36957 Metetra
.663 .76 .922 r
.62824 .36957 .63208 .37392 .63983 .37097 .63602 .367 Metetra
.822 .813 .841 r
.63602 .367 .63983 .37097 .64742 .36352 .64365 .36011 Metetra
.058 .486 .89 r
.64365 .36011 .64742 .36352 .65546 .36715 .65168 .3631 Metetra
.182 .532 .923 r
.65168 .3631 .65546 .36715 .66351 .3697 .65974 .36575 Metetra
0 .156 .655 r
.65974 .36575 .66351 .3697 .67176 .37561 .66804 .37243 Metetra
.882 .456 .061 r
.66804 .37243 .67176 .37561 .68061 .39257 .67708 .3936 Metetra
.798 .319 .083 r
.67708 .3936 .68061 .39257 .6891 .40014 .68579 .40508 Metetra
.983 .851 .467 r
.68579 .40508 .6891 .40014 .69594 .37539 .69242 .37509 Metetra
.992 .833 .583 r
.69242 .37509 .69594 .37539 .70286 .35366 .69918 .34951 Metetra
.871 .815 .791 r
.69918 .34951 .70286 .35366 .71045 .34472 .70676 .33995 Metetra
.745 .773 .876 r
.70676 .33995 .71045 .34472 .71831 .34022 .71463 .33537 Metetra
.697 .754 .896 r
.71463 .33537 .71831 .34022 .72625 .33688 .72259 .332 Metetra
.679 .747 .903 r
.72259 .332 .72625 .33688 .73424 .33391 .73059 .32902 Metetra
.671 .744 .905 r
.73059 .32902 .73424 .33391 .74226 .3311 .73862 .32619 Metetra
.668 .743 .906 r
.73862 .32619 .74226 .3311 .7503 .32835 .74668 .32343 Metetra
.666 .742 .907 r
.74668 .32343 .7503 .32835 .75837 .32563 .75477 .32069 Metetra
.665 .742 .907 r
.75477 .32069 .75837 .32563 .76646 .32292 .76288 .31797 Metetra
.665 .742 .908 r
.76288 .31797 .76646 .32292 .77458 .32022 .77101 .31525 Metetra
.664 .742 .908 r
.77101 .31525 .77458 .32022 .78272 .31752 .77917 .31254 Metetra
.664 .742 .908 r
.77917 .31254 .78272 .31752 .79089 .31481 .78735 .30982 Metetra
.664 .741 .908 r
.78735 .30982 .79089 .31481 .79907 .3121 .79555 .30709 Metetra
.664 .741 .908 r
.79555 .30709 .79907 .3121 .80729 .30939 .80378 .30436 Metetra
.664 .741 .908 r
.80378 .30436 .80729 .30939 .81552 .30666 .81203 .30163 Metetra
.664 .741 .908 r
.81203 .30163 .81552 .30666 .82378 .30394 .82031 .29888 Metetra
.664 .741 .908 r
.82031 .29888 .82378 .30394 .83206 .3012 .82861 .29613 Metetra
.664 .741 .908 r
.82861 .29613 .83206 .3012 .84037 .29846 .83693 .29338 Metetra
.664 .741 .908 r
.83693 .29338 .84037 .29846 .8487 .29571 .84528 .29061 Metetra
.664 .741 .908 r
.84528 .29061 .8487 .29571 .85705 .29295 .85365 .28784 Metetra
.664 .741 .908 r
.85365 .28784 .85705 .29295 .86543 .29018 .86204 .28506 Metetra
.664 .741 .908 r
.86204 .28506 .86543 .29018 .87383 .28741 .87046 .28227 Metetra
.726 .768 .886 r
.26778 .47958 .27228 .48344 .27917 .48008 .2747 .47602 Metetra
.686 .752 .901 r
.2747 .47602 .27917 .48008 .28603 .47743 .28157 .47331 Metetra
.672 .747 .906 r
.28157 .47331 .28603 .47743 .29288 .47502 .28843 .47087 Metetra
.667 .744 .908 r
.28843 .47087 .29288 .47502 .29974 .47271 .29531 .46853 Metetra
.663 .743 .909 r
.29531 .46853 .29974 .47271 .30662 .47044 .3022 .46625 Metetra
.661 .742 .91 r
.3022 .46625 .30662 .47044 .31351 .4682 .3091 .464 Metetra
.659 .742 .911 r
.3091 .464 .31351 .4682 .32042 .46599 .31602 .46177 Metetra
.658 .741 .912 r
.31602 .46177 .32042 .46599 .32734 .4638 .32296 .45958 Metetra
.655 .741 .913 r
.32296 .45958 .32734 .4638 .33429 .46165 .32991 .45742 Metetra
.652 .74 .914 r
.32991 .45742 .33429 .46165 .34124 .45954 .33688 .4553 Metetra
.648 .739 .916 r
.33688 .4553 .34124 .45954 .34822 .45751 .34386 .45326 Metetra
.641 .737 .919 r
.34386 .45326 .34822 .45751 .3552 .45559 .35086 .45134 Metetra
.628 .733 .923 r
.35086 .45134 .3552 .45559 .3622 .45386 .35786 .44962 Metetra
.6 .725 .932 r
.35786 .44962 .3622 .45386 .3692 .45253 .36487 .44831 Metetra
.519 .697 .95 r
.36487 .44831 .3692 .45253 .37618 .45211 .37185 .44798 Metetra
.102 .48 .896 r
.37185 .44798 .37618 .45211 .38307 .45429 .37872 .45074 Metetra
0 .262 .718 r
.37872 .45074 .38307 .45429 .38999 .45678 .38558 .45497 Metetra
.909 .837 .762 r
.38558 .45497 .38999 .45678 .39741 .44726 .39311 .44335 Metetra
.802 .784 .836 r
.39311 .44335 .39741 .44726 .40469 .4414 .40042 .43716 Metetra
.736 .765 .876 r
.40042 .43716 .40469 .4414 .41191 .43746 .40766 .43314 Metetra
.706 .756 .891 r
.40766 .43314 .41191 .43746 .41912 .43423 .41488 .42987 Metetra
.691 .751 .898 r
.41488 .42987 .41912 .43423 .42633 .4313 .4221 .42692 Metetra
.683 .748 .901 r
.4221 .42692 .42633 .4313 .43355 .42854 .42934 .42414 Metetra
.678 .746 .903 r
.42934 .42414 .43355 .42854 .44079 .42588 .43659 .42146 Metetra
.674 .745 .904 r
.43659 .42146 .44079 .42588 .44804 .42327 .44386 .41884 Metetra
.672 .744 .905 r
.44386 .41884 .44804 .42327 .45532 .4207 .45114 .41625 Metetra
.67 .744 .906 r
.45114 .41625 .45532 .4207 .46261 .41816 .45844 .41369 Metetra
.668 .743 .906 r
.45844 .41369 .46261 .41816 .46992 .41566 .46577 .41118 Metetra
.663 .741 .908 r
.46577 .41118 .46992 .41566 .47724 .41326 .47311 .40876 Metetra
.646 .735 .914 r
.47311 .40876 .47724 .41326 .48459 .41115 .48046 .40665 Metetra
.584 .714 .933 r
.48046 .40665 .48459 .41115 .49194 .40981 .48783 .40552 Metetra
.666 .753 .916 r
.48783 .40552 .49194 .40981 .49933 .40735 .49523 .40303 Metetra
.709 .764 .895 r
.49523 .40303 .49933 .40735 .50675 .40408 .50266 .39959 Metetra
.681 .75 .903 r
.50266 .39959 .50675 .40408 .51418 .40128 .5101 .39676 Metetra
.67 .746 .907 r
.5101 .39676 .51418 .40128 .52162 .39868 .51756 .39415 Metetra
.663 .743 .909 r
.51756 .39415 .52162 .39868 .52909 .3962 .52504 .39167 Metetra
.656 .741 .912 r
.52504 .39167 .52909 .3962 .53658 .39386 .53254 .38932 Metetra
.646 .738 .916 r
.53254 .38932 .53658 .39386 .54409 .3917 .54007 .38715 Metetra
.63 .733 .921 r
.54007 .38715 .54409 .3917 .55162 .3898 .54762 .38526 Metetra
.596 .721 .931 r
.54762 .38526 .55162 .3898 .55919 .38841 .55519 .3839 Metetra
.492 .678 .948 r
.55519 .3839 .55919 .38841 .56679 .38828 .56281 .38391 Metetra
0 .337 .81 r
.56281 .38391 .56679 .38828 .57449 .39199 .57053 .38856 Metetra
0 .283 .74 r
.57053 .38856 .57449 .39199 .58223 .39515 .5783 .39323 Metetra
.918 .875 .779 r
.5783 .39323 .58223 .39515 .58977 .38632 .58583 .3823 Metetra
.792 .804 .862 r
.58583 .3823 .58977 .38632 .59738 .38092 .59346 .37656 Metetra
.72 .774 .896 r
.59346 .37656 .59738 .38092 .60505 .37724 .60114 .37278 Metetra
.683 .76 .91 r
.60114 .37278 .60505 .37724 .61275 .3743 .60885 .36982 Metetra
.659 .752 .918 r
.60885 .36982 .61275 .3743 .62048 .37179 .6166 .36733 Metetra
.637 .746 .927 r
.6166 .36733 .62048 .37179 .62824 .36957 .62438 .36523 Metetra
.634 .751 .932 r
.62438 .36523 .62824 .36957 .63602 .367 .63219 .36316 Metetra
.803 .819 .864 r
.63219 .36316 .63602 .367 .64365 .36011 .63988 .35713 Metetra
.137 .568 .926 r
.63988 .35713 .64365 .36011 .65168 .3631 .64788 .35915 Metetra
.165 .527 .92 r
.64788 .35915 .65168 .3631 .65974 .36575 .65596 .36184 Metetra
0 .083 .587 r
.65596 .36184 .65974 .36575 .66804 .37243 .66429 .3693 Metetra
.91 .507 .135 r
.66429 .3693 .66804 .37243 .67708 .3936 .67357 .39556 Metetra
.853 .423 .201 r
.67357 .39556 .67708 .3936 .68579 .40508 .68256 .41251 Metetra
.969 .825 .406 r
.68256 .41251 .68579 .40508 .69242 .37509 .68892 .37582 Metetra
.998 .823 .543 r
.68892 .37582 .69242 .37509 .69918 .34951 .69547 .34536 Metetra
.88 .815 .78 r
.69547 .34536 .69918 .34951 .70676 .33995 .70304 .33516 Metetra
.747 .773 .874 r
.70304 .33516 .70676 .33995 .71463 .33537 .71093 .3305 Metetra
.697 .755 .896 r
.71093 .3305 .71463 .33537 .72259 .332 .7189 .3271 Metetra
.679 .747 .903 r
.7189 .3271 .72259 .332 .73059 .32902 .72692 .3241 Metetra
.672 .745 .905 r
.72692 .3241 .73059 .32902 .73862 .32619 .73497 .32125 Metetra
.668 .743 .906 r
.73497 .32125 .73862 .32619 .74668 .32343 .74304 .31848 Metetra
.666 .742 .907 r
.74304 .31848 .74668 .32343 .75477 .32069 .75115 .31573 Metetra
.665 .742 .907 r
.75115 .31573 .75477 .32069 .76288 .31797 .75927 .31299 Metetra
.665 .742 .908 r
.75927 .31299 .76288 .31797 .77101 .31525 .76742 .31026 Metetra
.664 .742 .908 r
.76742 .31026 .77101 .31525 .77917 .31254 .7756 .30753 Metetra
.664 .742 .908 r
.7756 .30753 .77917 .31254 .78735 .30982 .78379 .3048 Metetra
.664 .741 .908 r
.78379 .3048 .78735 .30982 .79555 .30709 .79201 .30206 Metetra
.664 .741 .908 r
.79201 .30206 .79555 .30709 .80378 .30436 .80026 .29931 Metetra
.664 .741 .908 r
.80026 .29931 .80378 .30436 .81203 .30163 .80853 .29656 Metetra
.664 .741 .908 r
.80853 .29656 .81203 .30163 .82031 .29888 .81682 .2938 Metetra
.664 .741 .908 r
.81682 .2938 .82031 .29888 .82861 .29613 .82513 .29104 Metetra
.664 .741 .908 r
.82513 .29104 .82861 .29613 .83693 .29338 .83347 .28827 Metetra
.664 .741 .908 r
.83347 .28827 .83693 .29338 .84528 .29061 .84184 .28549 Metetra
.664 .741 .908 r
.84184 .28549 .84528 .29061 .85365 .28784 .85023 .2827 Metetra
.664 .741 .908 r
.85023 .2827 .85365 .28784 .86204 .28506 .85864 .2799 Metetra
.664 .741 .908 r
.85864 .2799 .86204 .28506 .87046 .28227 .86707 .2771 Metetra
.734 .77 .881 r
.26326 .47573 .26778 .47958 .2747 .47602 .27021 .47193 Metetra
.688 .752 .9 r
.27021 .47193 .2747 .47602 .28157 .47331 .27709 .46917 Metetra
.673 .747 .906 r
.27709 .46917 .28157 .47331 .28843 .47087 .28397 .4667 Metetra
.667 .744 .908 r
.28397 .4667 .28843 .47087 .29531 .46853 .29085 .46434 Metetra
.664 .743 .909 r
.29085 .46434 .29531 .46853 .3022 .46625 .29775 .46204 Metetra
.661 .742 .91 r
.29775 .46204 .3022 .46625 .3091 .464 .30467 .45978 Metetra
.659 .742 .911 r
.30467 .45978 .3091 .464 .31602 .46177 .3116 .45754 Metetra
.657 .741 .912 r
.3116 .45754 .31602 .46177 .32296 .45958 .31854 .45534 Metetra
.655 .741 .913 r
.31854 .45534 .32296 .45958 .32991 .45742 .32551 .45316 Metetra
.652 .74 .914 r
.32551 .45316 .32991 .45742 .33688 .4553 .33249 .45104 Metetra
.647 .739 .916 r
.33249 .45104 .33688 .4553 .34386 .45326 .33948 .449 Metetra
.64 .737 .919 r
.33948 .449 .34386 .45326 .35086 .45134 .34649 .44707 Metetra
.627 .733 .924 r
.34649 .44707 .35086 .45134 .35786 .44962 .3535 .44535 Metetra
.598 .724 .933 r
.3535 .44535 .35786 .44962 .36487 .44831 .36052 .44406 Metetra
.51 .693 .951 r
.36052 .44406 .36487 .44831 .37185 .44798 .36751 .44382 Metetra
0 .408 .852 r
.36751 .44382 .37185 .44798 .37872 .45074 .37436 .44721 Metetra
.593 .064 0 r
.37436 .44721 .37872 .45074 .38558 .45497 .3811 .454 Metetra
.936 .84 .724 r
.3811 .454 .38558 .45497 .39311 .44335 .38879 .43941 Metetra
.809 .784 .83 r
.38879 .43941 .39311 .44335 .40042 .43716 .39613 .43289 Metetra
.738 .765 .875 r
.39613 .43289 .40042 .43716 .40766 .43314 .40338 .4288 Metetra
.707 .756 .89 r
.40338 .4288 .40766 .43314 .41488 .42987 .41061 .42549 Metetra
.691 .751 .897 r
.41061 .42549 .41488 .42987 .4221 .42692 .41785 .42252 Metetra
.683 .748 .901 r
.41785 .42252 .4221 .42692 .42934 .42414 .4251 .41972 Metetra
.678 .747 .903 r
.4251 .41972 .42934 .42414 .43659 .42146 .43237 .41702 Metetra
.675 .745 .904 r
.43237 .41702 .43659 .42146 .44386 .41884 .43965 .41438 Metetra
.672 .745 .905 r
.43965 .41438 .44386 .41884 .45114 .41625 .44694 .41178 Metetra
.67 .744 .905 r
.44694 .41178 .45114 .41625 .45844 .41369 .45426 .4092 Metetra
.668 .743 .906 r
.45426 .4092 .45844 .41369 .46577 .41118 .4616 .40667 Metetra
.663 .741 .908 r
.4616 .40667 .46577 .41118 .47311 .40876 .46895 .40423 Metetra
.645 .735 .914 r
.46895 .40423 .47311 .40876 .48046 .40665 .47631 .40212 Metetra
.564 .706 .937 r
.47631 .40212 .48046 .40665 .48783 .40552 .48369 .40126 Metetra
.669 .756 .916 r
.48369 .40126 .48783 .40552 .49523 .40303 .4911 .3987 Metetra
.716 .767 .892 r
.4911 .3987 .49523 .40303 .50266 .39959 .49855 .39509 Metetra
.682 .75 .902 r
.49855 .39509 .50266 .39959 .5101 .39676 .50601 .39223 Metetra
.671 .746 .907 r
.50601 .39223 .5101 .39676 .51756 .39415 .51348 .3896 Metetra
.663 .743 .91 r
.51348 .3896 .51756 .39415 .52504 .39167 .52098 .38711 Metetra
.656 .741 .912 r
.52098 .38711 .52504 .39167 .53254 .38932 .52849 .38475 Metetra
.646 .738 .916 r
.52849 .38475 .53254 .38932 .54007 .38715 .53603 .38258 Metetra
.629 .732 .922 r
.53603 .38258 .54007 .38715 .54762 .38526 .54359 .38069 Metetra
.593 .72 .932 r
.54359 .38069 .54762 .38526 .55519 .3839 .55118 .37938 Metetra
.479 .672 .949 r
.55118 .37938 .55519 .3839 .56281 .38391 .55881 .37952 Metetra
0 .228 .726 r
.55881 .37952 .56281 .38391 .57053 .38856 .56654 .38522 Metetra
.478 0 0 r
.56654 .38522 .57053 .38856 .5783 .39323 .57436 .392 Metetra
.949 .876 .73 r
.57436 .392 .5783 .39323 .58583 .3823 .58187 .37826 Metetra
.801 .805 .855 r
.58187 .37826 .58583 .3823 .59346 .37656 .58951 .37216 Metetra
.723 .775 .894 r
.58951 .37216 .59346 .37656 .60114 .37278 .5972 .3683 Metetra
.684 .76 .909 r
.5972 .3683 .60114 .37278 .60885 .36982 .60493 .36532 Metetra
.657 .751 .919 r
.60493 .36532 .60885 .36982 .6166 .36733 .6127 .36285 Metetra
.629 .743 .929 r
.6127 .36285 .6166 .36733 .62438 .36523 .6205 .36088 Metetra
.592 .735 .944 r
.6205 .36088 .62438 .36523 .63219 .36316 .62834 .35947 Metetra
.76 .821 .901 r
.62834 .35947 .63219 .36316 .63988 .35713 .6361 .35479 Metetra
.277 .696 .969 r
.6361 .35479 .63988 .35713 .64788 .35915 .64406 .35528 Metetra
.156 .528 .92 r
.64406 .35528 .64788 .35915 .65596 .36184 .65215 .35794 Metetra
0 .015 .519 r
.65215 .35794 .65596 .36184 .66429 .3693 .66053 .3662 Metetra
.929 .545 .19 r
.66053 .3662 .66429 .3693 .67357 .39556 .67006 .39851 Metetra
.887 .493 .276 r
.67006 .39851 .67357 .39556 .68256 .41251 .67944 .42319 Metetra
.952 .797 .348 r
.67944 .42319 .68256 .41251 .68892 .37582 .68547 .37797 Metetra
1 .811 .501 r
.68547 .37797 .68892 .37582 .69547 .34536 .69175 .34118 Metetra
.888 .814 .769 r
.69175 .34118 .69547 .34536 .70304 .33516 .69931 .33033 Metetra
.749 .773 .873 r
.69931 .33033 .70304 .33516 .71093 .3305 .70721 .3256 Metetra
.698 .755 .896 r
.70721 .3256 .71093 .3305 .7189 .3271 .7152 .32217 Metetra
.679 .748 .903 r
.7152 .32217 .7189 .3271 .72692 .3241 .72323 .31915 Metetra
.672 .745 .905 r
.72323 .31915 .72692 .3241 .73497 .32125 .73129 .31629 Metetra
.668 .743 .906 r
.73129 .31629 .73497 .32125 .74304 .31848 .73939 .3135 Metetra
.666 .742 .907 r
.73939 .3135 .74304 .31848 .75115 .31573 .74751 .31074 Metetra
.665 .742 .907 r
.74751 .31074 .75115 .31573 .75927 .31299 .75565 .30799 Metetra
.665 .742 .908 r
.75565 .30799 .75927 .31299 .76742 .31026 .76381 .30524 Metetra
.664 .742 .908 r
.76381 .30524 .76742 .31026 .7756 .30753 .772 .3025 Metetra
.664 .742 .908 r
.772 .3025 .7756 .30753 .78379 .3048 .78022 .29975 Metetra
.664 .741 .908 r
.78022 .29975 .78379 .3048 .79201 .30206 .78846 .297 Metetra
.664 .741 .908 r
.78846 .297 .79201 .30206 .80026 .29931 .79672 .29424 Metetra
.664 .741 .908 r
.79672 .29424 .80026 .29931 .80853 .29656 .805 .29147 Metetra
.664 .741 .908 r
.805 .29147 .80853 .29656 .81682 .2938 .81331 .2887 Metetra
.664 .741 .908 r
.81331 .2887 .81682 .2938 .82513 .29104 .82164 .28592 Metetra
.664 .741 .908 r
.82164 .28592 .82513 .29104 .83347 .28827 .83 .28313 Metetra
.664 .741 .908 r
.83 .28313 .83347 .28827 .84184 .28549 .83838 .28034 Metetra
.664 .741 .908 r
.83838 .28034 .84184 .28549 .85023 .2827 .84679 .27753 Metetra
.664 .741 .908 r
.84679 .27753 .85023 .2827 .85864 .2799 .85522 .27472 Metetra
.664 .741 .908 r
.85522 .27472 .85864 .2799 .86707 .2771 .86367 .27191 Metetra
.742 .771 .876 r
.25871 .47186 .26326 .47573 .27021 .47193 .26569 .46783 Metetra
.691 .753 .899 r
.26569 .46783 .27021 .47193 .27709 .46917 .27259 .46501 Metetra
.674 .747 .905 r
.27259 .46501 .27709 .46917 .28397 .4667 .27948 .46251 Metetra
.667 .745 .908 r
.27948 .46251 .28397 .4667 .29085 .46434 .28638 .46013 Metetra
.664 .743 .909 r
.28638 .46013 .29085 .46434 .29775 .46204 .29329 .45782 Metetra
.661 .742 .91 r
.29329 .45782 .29775 .46204 .30467 .45978 .30021 .45554 Metetra
.659 .742 .911 r
.30021 .45554 .30467 .45978 .3116 .45754 .30715 .45329 Metetra
.657 .741 .912 r
.30715 .45329 .3116 .45754 .31854 .45534 .31411 .45107 Metetra
.655 .741 .913 r
.31411 .45107 .31854 .45534 .32551 .45316 .32109 .44889 Metetra
.652 .74 .914 r
.32109 .44889 .32551 .45316 .33249 .45104 .32808 .44676 Metetra
.647 .739 .916 r
.32808 .44676 .33249 .45104 .33948 .449 .33508 .44471 Metetra
.64 .737 .919 r
.33508 .44471 .33948 .449 .34649 .44707 .3421 .44277 Metetra
.626 .733 .924 r
.3421 .44277 .34649 .44707 .3535 .44535 .34912 .44106 Metetra
.596 .723 .933 r
.34912 .44106 .3535 .44535 .36052 .44406 .35615 .43979 Metetra
.502 .689 .952 r
.35615 .43979 .36052 .44406 .36751 .44382 .36314 .43963 Metetra
0 .327 .794 r
.36314 .43963 .36751 .44382 .37436 .44721 .36996 .44366 Metetra
.814 .337 0 r
.36996 .44366 .37436 .44721 .3811 .454 .37653 .45434 Metetra
.961 .84 .678 r
.37653 .45434 .3811 .454 .38879 .43941 .38445 .43544 Metetra
.815 .783 .823 r
.38445 .43544 .38879 .43941 .39613 .43289 .39182 .4286 Metetra
.74 .765 .873 r
.39182 .4286 .39613 .43289 .40338 .4288 .39908 .42443 Metetra
.708 .756 .89 r
.39908 .42443 .40338 .4288 .41061 .42549 .40633 .42109 Metetra
.692 .751 .897 r
.40633 .42109 .41061 .42549 .41785 .42252 .41358 .4181 Metetra
.683 .748 .901 r
.41358 .4181 .41785 .42252 .4251 .41972 .42085 .41528 Metetra
.678 .747 .903 r
.42085 .41528 .4251 .41972 .43237 .41702 .42812 .41256 Metetra
.675 .745 .904 r
.42812 .41256 .43237 .41702 .43965 .41438 .43542 .4099 Metetra
.673 .745 .905 r
.43542 .4099 .43965 .41438 .44694 .41178 .44273 .40728 Metetra
.671 .744 .905 r
.44273 .40728 .44694 .41178 .45426 .4092 .45006 .40469 Metetra
.669 .743 .906 r
.45006 .40469 .45426 .4092 .4616 .40667 .4574 .40214 Metetra
.663 .741 .908 r
.4574 .40214 .4616 .40667 .46895 .40423 .46477 .39968 Metetra
.645 .735 .914 r
.46477 .39968 .46895 .40423 .47631 .40212 .47215 .39756 Metetra
.538 .695 .941 r
.47215 .39756 .47631 .40212 .48369 .40126 .47953 .39704 Metetra
.674 .76 .916 r
.47953 .39704 .48369 .40126 .4911 .3987 .48696 .39436 Metetra
.724 .769 .888 r
.48696 .39436 .4911 .3987 .49855 .39509 .49442 .39056 Metetra
.683 .75 .902 r
.49442 .39056 .49855 .39509 .50601 .39223 .50189 .38767 Metetra
.671 .746 .907 r
.50189 .38767 .50601 .39223 .51348 .3896 .50938 .38503 Metetra
.663 .743 .91 r
.50938 .38503 .51348 .3896 .52098 .38711 .51689 .38253 Metetra
.655 .741 .913 r
.51689 .38253 .52098 .38711 .52849 .38475 .52442 .38016 Metetra
.645 .738 .916 r
.52442 .38016 .52849 .38475 .53603 .38258 .53197 .37799 Metetra
.628 .732 .922 r
.53197 .37799 .53603 .38258 .54359 .38069 .53954 .37611 Metetra
.59 .719 .933 r
.53954 .37611 .54359 .38069 .55118 .37938 .54714 .37482 Metetra
.466 .666 .949 r
.54714 .37482 .55118 .37938 .55881 .37952 .55478 .3751 Metetra
0 .113 .625 r
.55478 .3751 .55881 .37952 .56654 .38522 .56254 .38195 Metetra
.694 .184 0 r
.56254 .38195 .56654 .38522 .57436 .392 .5704 .39171 Metetra
.973 .87 .674 r
.5704 .39171 .57436 .392 .58187 .37826 .57789 .37419 Metetra
.809 .805 .847 r
.57789 .37419 .58187 .37826 .58951 .37216 .58554 .36774 Metetra
.726 .776 .892 r
.58554 .36774 .58951 .37216 .5972 .3683 .59325 .3638 Metetra
.684 .76 .909 r
.59325 .3638 .5972 .3683 .60493 .36532 .601 .36079 Metetra
.655 .75 .92 r
.601 .36079 .60493 .36532 .6127 .36285 .60878 .35836 Metetra
.62 .74 .932 r
.60878 .35836 .6127 .36285 .6205 .36088 .6166 .35652 Metetra
.531 .71 .954 r
.6166 .35652 .6205 .36088 .62834 .35947 .62448 .35593 Metetra
.651 .802 .956 r
.62448 .35593 .62834 .35947 .6361 .35479 .63233 .35341 Metetra
.497 .866 .984 r
.63233 .35341 .6361 .35479 .64406 .35528 .64023 .3515 Metetra
.157 .535 .923 r
.64023 .3515 .64406 .35528 .65215 .35794 .64833 .35406 Metetra
.544 .046 0 r
.64833 .35406 .65215 .35794 .66053 .3662 .65674 .36312 Metetra
.941 .572 .231 r
.65674 .36312 .66053 .3662 .67006 .39851 .66657 .40245 Metetra
.909 .542 .323 r
.66657 .40245 .67006 .39851 .67944 .42319 .67645 .43797 Metetra
.933 .77 .296 r
.67645 .43797 .67944 .42319 .68547 .37797 .68208 .38213 Metetra
.999 .798 .459 r
.68208 .38213 .68547 .37797 .69175 .34118 .688 .33696 Metetra
.895 .812 .759 r
.688 .33696 .69175 .34118 .69931 .33033 .69556 .32547 Metetra
.75 .773 .872 r
.69556 .32547 .69931 .33033 .70721 .3256 .70347 .32067 Metetra
.698 .755 .895 r
.70347 .32067 .70721 .3256 .7152 .32217 .71147 .31722 Metetra
.679 .748 .903 r
.71147 .31722 .7152 .32217 .72323 .31915 .71952 .31418 Metetra
.672 .745 .905 r
.71952 .31418 .72323 .31915 .73129 .31629 .7276 .3113 Metetra
.668 .743 .906 r
.7276 .3113 .73129 .31629 .73939 .3135 .73571 .3085 Metetra
.666 .742 .907 r
.73571 .3085 .73939 .3135 .74751 .31074 .74385 .30572 Metetra
.665 .742 .907 r
.74385 .30572 .74751 .31074 .75565 .30799 .752 .30296 Metetra
.665 .742 .908 r
.752 .30296 .75565 .30799 .76381 .30524 .76019 .3002 Metetra
.664 .742 .908 r
.76019 .3002 .76381 .30524 .772 .3025 .76839 .29744 Metetra
.664 .742 .908 r
.76839 .29744 .772 .3025 .78022 .29975 .77662 .29467 Metetra
.664 .741 .908 r
.77662 .29467 .78022 .29975 .78846 .297 .78488 .29191 Metetra
.664 .741 .908 r
.78488 .29191 .78846 .297 .79672 .29424 .79316 .28913 Metetra
.664 .741 .908 r
.79316 .28913 .79672 .29424 .805 .29147 .80146 .28635 Metetra
.664 .741 .908 r
.80146 .28635 .805 .29147 .81331 .2887 .80978 .28357 Metetra
.664 .741 .908 r
.80978 .28357 .81331 .2887 .82164 .28592 .81813 .28077 Metetra
.664 .741 .908 r
.81813 .28077 .82164 .28592 .83 .28313 .82651 .27797 Metetra
.664 .741 .908 r
.82651 .27797 .83 .28313 .83838 .28034 .83491 .27516 Metetra
.664 .741 .908 r
.83491 .27516 .83838 .28034 .84679 .27753 .84333 .27234 Metetra
.664 .741 .908 r
.84333 .27234 .84679 .27753 .85522 .27472 .85178 .26952 Metetra
.664 .741 .908 r
.85178 .26952 .85522 .27472 .86367 .27191 .86025 .26668 Metetra
.75 .772 .87 r
.25414 .46797 .25871 .47186 .26569 .46783 .26115 .4637 Metetra
.693 .753 .898 r
.26115 .4637 .26569 .46783 .27259 .46501 .26807 .46083 Metetra
.675 .747 .905 r
.26807 .46083 .27259 .46501 .27948 .46251 .27497 .4583 Metetra
.668 .745 .908 r
.27497 .4583 .27948 .46251 .28638 .46013 .28188 .4559 Metetra
.664 .743 .909 r
.28188 .4559 .28638 .46013 .29329 .45782 .2888 .45357 Metetra
.662 .742 .91 r
.2888 .45357 .29329 .45782 .30021 .45554 .29574 .45128 Metetra
.66 .742 .911 r
.29574 .45128 .30021 .45554 .30715 .45329 .30269 .44902 Metetra
.657 .741 .912 r
.30269 .44902 .30715 .45329 .31411 .45107 .30966 .44679 Metetra
.655 .741 .913 r
.30966 .44679 .31411 .45107 .32109 .44889 .31664 .4446 Metetra
.652 .74 .914 r
.31664 .4446 .32109 .44889 .32808 .44676 .32364 .44246 Metetra
.647 .738 .916 r
.32364 .44246 .32808 .44676 .33508 .44471 .33066 .4404 Metetra
.639 .736 .919 r
.33066 .4404 .33508 .44471 .3421 .44277 .33769 .43846 Metetra
.625 .732 .924 r
.33769 .43846 .3421 .44277 .34912 .44106 .34472 .43675 Metetra
.594 .723 .934 r
.34472 .43675 .34912 .44106 .35615 .43979 .35176 .43549 Metetra
.494 .686 .952 r
.35176 .43549 .35615 .43979 .36314 .43963 .35875 .43541 Metetra
0 .246 .729 r
.35875 .43541 .36314 .43963 .36996 .44366 .36555 .44005 Metetra
.907 .498 .176 r
.36555 .44005 .36996 .44366 .37653 .45434 .37182 .4569 Metetra
.982 .837 .624 r
.37182 .4569 .37653 .45434 .38445 .43544 .38008 .43141 Metetra
.819 .782 .817 r
.38008 .43141 .38445 .43544 .39182 .4286 .38748 .42427 Metetra
.742 .765 .872 r
.38748 .42427 .39182 .4286 .39908 .42443 .39477 .42004 Metetra
.708 .756 .889 r
.39477 .42004 .39908 .42443 .40633 .42109 .40203 .41667 Metetra
.692 .751 .897 r
.40203 .41667 .40633 .42109 .41358 .4181 .40929 .41365 Metetra
.684 .748 .9 r
.40929 .41365 .41358 .4181 .42085 .41528 .41657 .41081 Metetra
.678 .747 .903 r
.41657 .41081 .42085 .41528 .42812 .41256 .42386 .40808 Metetra
.675 .746 .904 r
.42386 .40808 .42812 .41256 .43542 .4099 .43116 .4054 Metetra
.673 .745 .905 r
.43116 .4054 .43542 .4099 .44273 .40728 .43849 .40277 Metetra
.671 .744 .905 r
.43849 .40277 .44273 .40728 .45006 .40469 .44583 .40016 Metetra
.669 .743 .906 r
.44583 .40016 .45006 .40469 .4574 .40214 .45319 .39758 Metetra
.664 .741 .908 r
.45319 .39758 .4574 .40214 .46477 .39968 .46057 .3951 Metetra
.645 .735 .914 r
.46057 .3951 .46477 .39968 .47215 .39756 .46796 .39297 Metetra
.505 .68 .944 r
.46796 .39297 .47215 .39756 .47953 .39704 .47534 .39284 Metetra
.681 .765 .915 r
.47534 .39284 .47953 .39704 .48696 .39436 .48279 .39001 Metetra
.731 .771 .884 r
.48279 .39001 .48696 .39436 .49442 .39056 .49027 .386 Metetra
.683 .75 .902 r
.49027 .386 .49442 .39056 .50189 .38767 .49776 .38308 Metetra
.671 .746 .907 r
.49776 .38308 .50189 .38767 .50938 .38503 .50526 .38043 Metetra
.663 .743 .91 r
.50526 .38043 .50938 .38503 .51689 .38253 .51278 .37792 Metetra
.655 .741 .913 r
.51278 .37792 .51689 .38253 .52442 .38016 .52032 .37555 Metetra
.645 .737 .917 r
.52032 .37555 .52442 .38016 .53197 .37799 .52788 .37337 Metetra
.627 .732 .923 r
.52788 .37337 .53197 .37799 .53954 .37611 .53547 .3715 Metetra
.587 .718 .933 r
.53547 .3715 .53954 .37611 .54714 .37482 .54308 .37025 Metetra
.453 .66 .95 r
.54308 .37025 .54714 .37482 .55478 .3751 .55074 .37066 Metetra
0 .004 .52 r
.55074 .37066 .55478 .3751 .56254 .38195 .55851 .3787 Metetra
.825 .359 0 r
.55851 .3787 .56254 .38195 .5704 .39171 .56643 .39263 Metetra
.989 .857 .617 r
.56643 .39263 .5704 .39171 .57789 .37419 .57389 .37008 Metetra
.816 .804 .84 r
.57389 .37008 .57789 .37419 .58554 .36774 .58156 .36329 Metetra
.728 .776 .891 r
.58156 .36329 .58554 .36774 .59325 .3638 .58928 .35927 Metetra
.684 .76 .909 r
.58928 .35927 .59325 .3638 .601 .36079 .59704 .35625 Metetra
.652 .75 .921 r
.59704 .35625 .601 .36079 .60878 .35836 .60484 .35384 Metetra
.61 .736 .935 r
.60484 .35384 .60878 .35836 .6166 .35652 .61267 .35216 Metetra
.449 .672 .959 r
.61267 .35216 .6166 .35652 .62448 .35593 .6206 .3525 Metetra
.315 .655 .971 r
.6206 .3525 .62448 .35593 .63233 .35341 .62859 .35351 Metetra
.742 .99 .883 r
.62859 .35351 .63233 .35341 .64023 .3515 .63638 .34779 Metetra
.167 .547 .928 r
.63638 .34779 .64023 .3515 .64833 .35406 .64448 .35018 Metetra
.596 .099 0 r
.64448 .35018 .64833 .35406 .65674 .36312 .65293 .36003 Metetra
.949 .592 .262 r
.65293 .36003 .65674 .36312 .66657 .40245 .66306 .40718 Metetra
.926 .577 .352 r
.66306 .40718 .66657 .40245 .67645 .43797 .67362 .45761 Metetra
.916 .747 .256 r
.67362 .45761 .67645 .43797 .68208 .38213 .67878 .38918 Metetra
.994 .783 .415 r
.67878 .38918 .68208 .38213 .688 .33696 .68423 .33265 Metetra
.9 .809 .749 r
.68423 .33265 .688 .33696 .69556 .32547 .69178 .32057 Metetra
.752 .773 .871 r
.69178 .32057 .69556 .32547 .70347 .32067 .69971 .31571 Metetra
.698 .755 .895 r
.69971 .31571 .70347 .32067 .71147 .31722 .70773 .31223 Metetra
.679 .748 .902 r
.70773 .31223 .71147 .31722 .71952 .31418 .71579 .30918 Metetra
.672 .745 .905 r
.71579 .30918 .71952 .31418 .7276 .3113 .72389 .30629 Metetra
.668 .743 .906 r
.72389 .30629 .7276 .3113 .73571 .3085 .73202 .30347 Metetra
.666 .742 .907 r
.73202 .30347 .73571 .3085 .74385 .30572 .74017 .30068 Metetra
.665 .742 .907 r
.74017 .30068 .74385 .30572 .752 .30296 .74834 .2979 Metetra
.665 .742 .908 r
.74834 .2979 .752 .30296 .76019 .3002 .75654 .29512 Metetra
.664 .742 .908 r
.75654 .29512 .76019 .3002 .76839 .29744 .76476 .29235 Metetra
.664 .742 .908 r
.76476 .29235 .76839 .29744 .77662 .29467 .77301 .28957 Metetra
.664 .741 .908 r
.77301 .28957 .77662 .29467 .78488 .29191 .78128 .28679 Metetra
.664 .741 .908 r
.78128 .28679 .78488 .29191 .79316 .28913 .78958 .284 Metetra
.664 .741 .908 r
.78958 .284 .79316 .28913 .80146 .28635 .7979 .28121 Metetra
.664 .741 .908 r
.7979 .28121 .80146 .28635 .80978 .28357 .80624 .2784 Metetra
.664 .741 .908 r
.80624 .2784 .80978 .28357 .81813 .28077 .81461 .2756 Metetra
.664 .741 .908 r
.81461 .2756 .81813 .28077 .82651 .27797 .823 .27278 Metetra
.664 .741 .908 r
.823 .27278 .82651 .27797 .83491 .27516 .83141 .26995 Metetra
.664 .741 .908 r
.83141 .26995 .83491 .27516 .84333 .27234 .83985 .26712 Metetra
.664 .741 .908 r
.83985 .26712 .84333 .27234 .85178 .26952 .84832 .26428 Metetra
.664 .741 .908 r
.84832 .26428 .85178 .26952 .86025 .26668 .85681 .26143 Metetra
.757 .771 .865 r
.24955 .46405 .25414 .46797 .26115 .4637 .25659 .45955 Metetra
.695 .753 .897 r
.25659 .45955 .26115 .4637 .26807 .46083 .26352 .45662 Metetra
.676 .747 .905 r
.26352 .45662 .26807 .46083 .27497 .4583 .27044 .45406 Metetra
.668 .745 .907 r
.27044 .45406 .27497 .4583 .28188 .4559 .27736 .45165 Metetra
.664 .743 .909 r
.27736 .45165 .28188 .4559 .2888 .45357 .28429 .4493 Metetra
.662 .742 .91 r
.28429 .4493 .2888 .45357 .29574 .45128 .29124 .447 Metetra
.66 .742 .911 r
.29124 .447 .29574 .45128 .30269 .44902 .2982 .44472 Metetra
.657 .741 .912 r
.2982 .44472 .30269 .44902 .30966 .44679 .30518 .44248 Metetra
.655 .741 .913 r
.30518 .44248 .30966 .44679 .31664 .4446 .31218 .44028 Metetra
.652 .74 .914 r
.31218 .44028 .31664 .4446 .32364 .44246 .31919 .43813 Metetra
.647 .738 .916 r
.31919 .43813 .32364 .44246 .33066 .4404 .32622 .43607 Metetra
.639 .736 .919 r
.32622 .43607 .33066 .4404 .33769 .43846 .33325 .43412 Metetra
.624 .732 .924 r
.33325 .43412 .33769 .43846 .34472 .43675 .3403 .43241 Metetra
.592 .722 .934 r
.3403 .43241 .34472 .43675 .35176 .43549 .34734 .43117 Metetra
.487 .682 .953 r
.34734 .43117 .35176 .43549 .35875 .43541 .35434 .43115 Metetra
0 .172 .665 r
.35434 .43115 .35875 .43541 .36555 .44005 .36111 .43635 Metetra
.939 .585 .314 r
.36111 .43635 .36555 .44005 .37182 .4569 .36688 .46338 Metetra
.996 .83 .563 r
.36688 .46338 .37182 .4569 .38008 .43141 .37569 .42732 Metetra
.823 .78 .812 r
.37569 .42732 .38008 .43141 .38748 .42427 .38313 .41992 Metetra
.743 .765 .87 r
.38313 .41992 .38748 .42427 .39477 .42004 .39043 .41563 Metetra
.709 .756 .889 r
.39043 .41563 .39477 .42004 .40203 .41667 .3977 .41222 Metetra
.693 .751 .896 r
.3977 .41222 .40203 .41667 .40929 .41365 .40498 .40918 Metetra
.684 .748 .9 r
.40498 .40918 .40929 .41365 .41657 .41081 .41227 .40632 Metetra
.679 .747 .902 r
.41227 .40632 .41657 .41081 .42386 .40808 .41957 .40357 Metetra
.675 .746 .904 r
.41957 .40357 .42386 .40808 .43116 .4054 .42689 .40088 Metetra
.673 .745 .905 r
.42689 .40088 .43116 .4054 .43849 .40277 .43423 .39823 Metetra
.671 .744 .905 r
.43423 .39823 .43849 .40277 .44583 .40016 .44158 .3956 Metetra
.669 .743 .906 r
.44158 .3956 .44583 .40016 .45319 .39758 .44896 .393 Metetra
.665 .741 .907 r
.44896 .393 .45319 .39758 .46057 .3951 .45635 .39049 Metetra
.645 .735 .914 r
.45635 .39049 .46057 .3951 .46796 .39297 .46375 .38835 Metetra
.465 .661 .947 r
.46375 .38835 .46796 .39297 .47534 .39284 .47114 .38864 Metetra
.69 .769 .912 r
.47114 .38864 .47534 .39284 .48279 .39001 .4786 .38562 Metetra
.738 .772 .88 r
.4786 .38562 .48279 .39001 .49027 .386 .4861 .38141 Metetra
.684 .75 .901 r
.4861 .38141 .49027 .386 .49776 .38308 .4936 .37848 Metetra
.67 .745 .907 r
.4936 .37848 .49776 .38308 .50526 .38043 .50112 .37581 Metetra
.663 .743 .91 r
.50112 .37581 .50526 .38043 .51278 .37792 .50865 .37329 Metetra
.655 .741 .913 r
.50865 .37329 .51278 .37792 .52032 .37555 .5162 .37092 Metetra
.644 .737 .917 r
.5162 .37092 .52032 .37555 .52788 .37337 .52378 .36874 Metetra
.626 .731 .923 r
.52378 .36874 .52788 .37337 .53547 .3715 .53138 .36687 Metetra
.585 .716 .934 r
.53138 .36687 .53547 .3715 .54308 .37025 .53901 .36565 Metetra
.442 .654 .95 r
.53901 .36565 .54308 .37025 .55074 .37066 .54667 .36617 Metetra
.573 .088 0 r
.54667 .36617 .55074 .37066 .55851 .3787 .55446 .37538 Metetra
.892 .47 .101 r
.55446 .37538 .55851 .3787 .56643 .39263 .56244 .39487 Metetra
.996 .838 .566 r
.56244 .39487 .56643 .39263 .57389 .37008 .56986 .3659 Metetra
.822 .803 .833 r
.56986 .3659 .57389 .37008 .58156 .36329 .57755 .35881 Metetra
.73 .776 .889 r
.57755 .35881 .58156 .36329 .58928 .35927 .58528 .35472 Metetra
.684 .76 .909 r
.58528 .35472 .58928 .35927 .59704 .35625 .59306 .35168 Metetra
.65 .749 .922 r
.59306 .35168 .59704 .35625 .60484 .35384 .60087 .34931 Metetra
.599 .732 .938 r
.60087 .34931 .60484 .35384 .61267 .35216 .60873 .34778 Metetra
.344 .616 .952 r
.60873 .34778 .61267 .35216 .6206 .3525 .6167 .34916 Metetra
0 .096 .569 r
.6167 .34916 .6206 .3525 .62859 .35351 .62488 .35581 Metetra
.884 .975 .671 r
.62488 .35581 .62859 .35351 .63638 .34779 .6325 .34413 Metetra
.185 .563 .935 r
.6325 .34413 .63638 .34779 .64448 .35018 .64061 .34632 Metetra
.639 .145 0 r
.64061 .34632 .64448 .35018 .65293 .36003 .64909 .35691 Metetra
.954 .607 .284 r
.64909 .35691 .65293 .36003 .66306 .40718 .65953 .41222 Metetra
.939 .603 .367 r
.65953 .41222 .66306 .40718 .67362 .45761 .67092 .482 Metetra
.906 .731 .232 r
.67092 .482 .67362 .45761 .67878 .38918 .67564 .40043 Metetra
.987 .768 .373 r
.67564 .40043 .67878 .38918 .68423 .33265 .68043 .32822 Metetra
.902 .804 .741 r
.68043 .32822 .68423 .33265 .69178 .32057 .68798 .31563 Metetra
.753 .773 .87 r
.68798 .31563 .69178 .32057 .69971 .31571 .69593 .31072 Metetra
.699 .755 .895 r
.69593 .31072 .69971 .31571 .70773 .31223 .70396 .30722 Metetra
.68 .747 .902 r
.70396 .30722 .70773 .31223 .71579 .30918 .71205 .30416 Metetra
.672 .745 .905 r
.71205 .30416 .71579 .30918 .72389 .30629 .72016 .30125 Metetra
.668 .743 .906 r
.72016 .30125 .72389 .30629 .73202 .30347 .7283 .29841 Metetra
.666 .742 .907 r
.7283 .29841 .73202 .30347 .74017 .30068 .73647 .29561 Metetra
.665 .742 .907 r
.73647 .29561 .74017 .30068 .74834 .2979 .74466 .29281 Metetra
.665 .742 .908 r
.74466 .29281 .74834 .2979 .75654 .29512 .75288 .29003 Metetra
.664 .742 .908 r
.75288 .29003 .75654 .29512 .76476 .29235 .76111 .28724 Metetra
.664 .742 .908 r
.76111 .28724 .76476 .29235 .77301 .28957 .76938 .28444 Metetra
.664 .741 .908 r
.76938 .28444 .77301 .28957 .78128 .28679 .77767 .28165 Metetra
.664 .741 .908 r
.77767 .28165 .78128 .28679 .78958 .284 .78598 .27884 Metetra
.664 .741 .908 r
.78598 .27884 .78958 .284 .7979 .28121 .79431 .27603 Metetra
.664 .741 .908 r
.79431 .27603 .7979 .28121 .80624 .2784 .80267 .27322 Metetra
.664 .741 .908 r
.80267 .27322 .80624 .2784 .81461 .2756 .81106 .27039 Metetra
.664 .741 .908 r
.81106 .27039 .81461 .2756 .823 .27278 .81947 .26756 Metetra
.664 .741 .908 r
.81947 .26756 .823 .27278 .83141 .26995 .8279 .26472 Metetra
.664 .741 .908 r
.8279 .26472 .83141 .26995 .83985 .26712 .83636 .26187 Metetra
.664 .741 .908 r
.83636 .26187 .83985 .26712 .84832 .26428 .84484 .25902 Metetra
.664 .741 .908 r
.84484 .25902 .84832 .26428 .85681 .26143 .85335 .25615 Metetra
.762 .77 .86 r
.24493 .46008 .24955 .46405 .25659 .45955 .25201 .45538 Metetra
.696 .753 .896 r
.25201 .45538 .25659 .45955 .26352 .45662 .25895 .4524 Metetra
.677 .747 .904 r
.25895 .4524 .26352 .45662 .27044 .45406 .26588 .44981 Metetra
.669 .745 .907 r
.26588 .44981 .27044 .45406 .27736 .45165 .27281 .44738 Metetra
.665 .743 .909 r
.27281 .44738 .27736 .45165 .28429 .4493 .27976 .44501 Metetra
.662 .742 .91 r
.27976 .44501 .28429 .4493 .29124 .447 .28672 .44269 Metetra
.66 .742 .911 r
.28672 .44269 .29124 .447 .2982 .44472 .29369 .44041 Metetra
.657 .741 .912 r
.29369 .44041 .2982 .44472 .30518 .44248 .30068 .43816 Metetra
.655 .741 .913 r
.30068 .43816 .30518 .44248 .31218 .44028 .30769 .43595 Metetra
.652 .74 .914 r
.30769 .43595 .31218 .44028 .31919 .43813 .31471 .43379 Metetra
.647 .738 .916 r
.31471 .43379 .31919 .43813 .32622 .43607 .32175 .43171 Metetra
.639 .736 .919 r
.32175 .43171 .32622 .43607 .33325 .43412 .3288 .42976 Metetra
.624 .732 .925 r
.3288 .42976 .33325 .43412 .3403 .43241 .33585 .42805 Metetra
.591 .722 .935 r
.33585 .42805 .3403 .43241 .34734 .43117 .3429 .42682 Metetra
.481 .68 .953 r
.3429 .42682 .34734 .43117 .35434 .43115 .34991 .42685 Metetra
0 .116 .612 r
.34991 .42685 .35434 .43115 .36111 .43635 .35666 .43249 Metetra
.949 .631 .393 r
.35666 .43249 .36111 .43635 .36688 .46338 .36155 .47663 Metetra
1 .818 .5 r
.36155 .47663 .36688 .46338 .37569 .42732 .37129 .42314 Metetra
.825 .777 .807 r
.37129 .42314 .37569 .42732 .38313 .41992 .37875 .41553 Metetra
.744 .764 .869 r
.37875 .41553 .38313 .41992 .39043 .41563 .38606 .41119 Metetra
.71 .756 .888 r
.38606 .41119 .39043 .41563 .3977 .41222 .39336 .40775 Metetra
.693 .751 .896 r
.39336 .40775 .3977 .41222 .40498 .40918 .40065 .40469 Metetra
.684 .749 .9 r
.40065 .40469 .40498 .40918 .41227 .40632 .40795 .40181 Metetra
.679 .747 .902 r
.40795 .40181 .41227 .40632 .41957 .40357 .41527 .39904 Metetra
.675 .746 .904 r
.41527 .39904 .41957 .40357 .42689 .40088 .4226 .39633 Metetra
.673 .745 .904 r
.4226 .39633 .42689 .40088 .43423 .39823 .42995 .39366 Metetra
.672 .744 .905 r
.42995 .39366 .43423 .39823 .44158 .3956 .43732 .39102 Metetra
.67 .743 .905 r
.43732 .39102 .44158 .3956 .44896 .393 .4447 .3884 Metetra
.665 .742 .907 r
.4447 .3884 .44896 .393 .45635 .39049 .45211 .38586 Metetra
.645 .735 .914 r
.45211 .38586 .45635 .39049 .46375 .38835 .45952 .3837 Metetra
.426 .642 .947 r
.45952 .3837 .46375 .38835 .47114 .38864 .46691 .38439 Metetra
.699 .769 .906 r
.46691 .38439 .47114 .38864 .4786 .38562 .47439 .38116 Metetra
.743 .771 .876 r
.47439 .38116 .4786 .38562 .4861 .38141 .48191 .3768 Metetra
.684 .75 .901 r
.48191 .3768 .4861 .38141 .4936 .37848 .48942 .37385 Metetra
.67 .745 .907 r
.48942 .37385 .4936 .37848 .50112 .37581 .49695 .37117 Metetra
.663 .743 .91 r
.49695 .37117 .50112 .37581 .50865 .37329 .5045 .36864 Metetra
.655 .741 .913 r
.5045 .36864 .50865 .37329 .5162 .37092 .51207 .36626 Metetra
.643 .737 .917 r
.51207 .36626 .5162 .37092 .52378 .36874 .51966 .36408 Metetra
.624 .731 .923 r
.51966 .36408 .52378 .36874 .53138 .36687 .52727 .36222 Metetra
.582 .715 .935 r
.52727 .36222 .53138 .36687 .53901 .36565 .53491 .36102 Metetra
.431 .648 .95 r
.53491 .36102 .53901 .36565 .54667 .36617 .54258 .36165 Metetra
.638 .156 0 r
.54258 .36165 .54667 .36617 .55446 .37538 .55038 .37186 Metetra
.924 .533 .189 r
.55038 .37186 .55446 .37538 .56244 .39487 .55841 .39781 Metetra
.997 .816 .536 r
.55841 .39781 .56244 .39487 .56986 .3659 .56581 .36164 Metetra
.826 .801 .828 r
.56581 .36164 .56986 .3659 .57755 .35881 .57351 .3543 Metetra
.732 .776 .888 r
.57351 .3543 .57755 .35881 .58528 .35472 .58127 .35014 Metetra
.684 .76 .909 r
.58127 .35014 .58528 .35472 .59306 .35168 .58906 .34709 Metetra
.647 .748 .923 r
.58906 .34709 .59306 .35168 .60087 .34931 .59689 .34475 Metetra
.588 .728 .94 r
.59689 .34475 .60087 .34931 .60873 .34778 .60476 .34338 Metetra
.221 .544 .928 r
.60476 .34338 .60873 .34778 .6167 .34916 .61278 .34581 Metetra
.82 .356 0 r
.61278 .34581 .6167 .34916 .62488 .35581 .62122 .36091 Metetra
.932 .9 .501 r
.62122 .36091 .62488 .35581 .6325 .34413 .62861 .3405 Metetra
.211 .582 .944 r
.62861 .3405 .6325 .34413 .64061 .34632 .63672 .34245 Metetra
.673 .182 0 r
.63672 .34245 .64061 .34632 .64909 .35691 .64523 .35374 Metetra
.957 .617 .299 r
.64523 .35374 .64909 .35691 .65953 .41222 .65593 .41674 Metetra
.949 .622 .372 r
.65593 .41674 .65953 .41222 .67092 .482 .66823 .50857 Metetra
.916 .731 .237 r
.66823 .50857 .67092 .482 .67564 .40043 .67268 .41702 Metetra
.979 .753 .339 r
.67268 .41702 .67564 .40043 .68043 .32822 .67661 .32363 Metetra
.903 .799 .735 r
.67661 .32363 .68043 .32822 .68798 .31563 .68417 .31065 Metetra
.753 .772 .869 r
.68417 .31065 .68798 .31563 .69593 .31072 .69213 .3057 Metetra
.699 .755 .895 r
.69213 .3057 .69593 .31072 .70396 .30722 .70018 .30219 Metetra
.68 .747 .902 r
.70018 .30219 .70396 .30722 .71205 .30416 .70828 .2991 Metetra
.672 .744 .905 r
.70828 .2991 .71205 .30416 .72016 .30125 .71641 .29618 Metetra
.668 .743 .906 r
.71641 .29618 .72016 .30125 .7283 .29841 .72457 .29333 Metetra
.666 .742 .907 r
.72457 .29333 .7283 .29841 .73647 .29561 .73275 .29051 Metetra
.665 .742 .907 r
.73275 .29051 .73647 .29561 .74466 .29281 .74096 .2877 Metetra
.665 .742 .908 r
.74096 .2877 .74466 .29281 .75288 .29003 .74919 .2849 Metetra
.664 .742 .908 r
.74919 .2849 .75288 .29003 .76111 .28724 .75745 .28209 Metetra
.664 .742 .908 r
.75745 .28209 .76111 .28724 .76938 .28444 .76573 .27929 Metetra
.664 .741 .908 r
.76573 .27929 .76938 .28444 .77767 .28165 .77403 .27648 Metetra
.664 .741 .908 r
.77403 .27648 .77767 .28165 .78598 .27884 .78236 .27366 Metetra
.664 .741 .908 r
.78236 .27366 .78598 .27884 .79431 .27603 .79071 .27083 Metetra
.664 .741 .908 r
.79071 .27083 .79431 .27603 .80267 .27322 .79909 .268 Metetra
.664 .741 .908 r
.79909 .268 .80267 .27322 .81106 .27039 .80749 .26516 Metetra
.664 .741 .908 r
.80749 .26516 .81106 .27039 .81947 .26756 .81592 .26231 Metetra
.664 .741 .908 r
.81592 .26231 .81947 .26756 .8279 .26472 .82437 .25946 Metetra
.664 .741 .908 r
.82437 .25946 .8279 .26472 .83636 .26187 .83285 .25659 Metetra
.664 .741 .908 r
.83285 .25659 .83636 .26187 .84484 .25902 .84135 .25372 Metetra
.664 .741 .908 r
.84135 .25372 .84484 .25902 .85335 .25615 .84987 .25084 Metetra
.766 .769 .855 r
.2403 .45604 .24493 .46008 .25201 .45538 .24741 .45118 Metetra
.698 .753 .895 r
.24741 .45118 .25201 .45538 .25895 .4524 .25436 .44815 Metetra
.678 .748 .904 r
.25436 .44815 .25895 .4524 .26588 .44981 .2613 .44554 Metetra
.669 .745 .907 r
.2613 .44554 .26588 .44981 .27281 .44738 .26825 .44308 Metetra
.665 .743 .909 r
.26825 .44308 .27281 .44738 .27976 .44501 .2752 .4407 Metetra
.662 .743 .91 r
.2752 .4407 .27976 .44501 .28672 .44269 .28218 .43837 Metetra
.66 .742 .911 r
.28218 .43837 .28672 .44269 .29369 .44041 .28916 .43607 Metetra
.657 .741 .912 r
.28916 .43607 .29369 .44041 .30068 .43816 .29616 .43381 Metetra
.655 .74 .913 r
.29616 .43381 .30068 .43816 .30769 .43595 .30318 .43159 Metetra
.651 .74 .914 r
.30318 .43159 .30769 .43595 .31471 .43379 .31022 .42942 Metetra
.646 .738 .916 r
.31022 .42942 .31471 .43379 .32175 .43171 .31726 .42734 Metetra
.638 .736 .919 r
.31726 .42734 .32175 .43171 .3288 .42976 .32432 .42538 Metetra
.623 .732 .925 r
.32432 .42538 .3288 .42976 .33585 .42805 .33139 .42367 Metetra
.589 .721 .935 r
.33139 .42367 .33585 .42805 .3429 .42682 .33845 .42244 Metetra
.477 .678 .953 r
.33845 .42244 .3429 .42682 .34991 .42685 .34546 .42251 Metetra
0 .081 .579 r
.34546 .42251 .34991 .42685 .35666 .43249 .35219 .42844 Metetra
.952 .652 .432 r
.35219 .42844 .35666 .43249 .36155 .47663 .35588 .49502 Metetra
.999 .797 .481 r
.35588 .49502 .36155 .47663 .37129 .42314 .36686 .41888 Metetra
.826 .775 .804 r
.36686 .41888 .37129 .42314 .37875 .41553 .37435 .41111 Metetra
.745 .764 .868 r
.37435 .41111 .37875 .41553 .38606 .41119 .38168 .40672 Metetra
.71 .756 .888 r
.38168 .40672 .38606 .41119 .39336 .40775 .38899 .40326 Metetra
.694 .751 .896 r
.38899 .40326 .39336 .40775 .40065 .40469 .39629 .40017 Metetra
.684 .749 .9 r
.39629 .40017 .40065 .40469 .40795 .40181 .40361 .39728 Metetra
.679 .747 .902 r
.40361 .39728 .40795 .40181 .41527 .39904 .41094 .39449 Metetra
.676 .746 .904 r
.41094 .39449 .41527 .39904 .4226 .39633 .41828 .39177 Metetra
.673 .745 .904 r
.41828 .39177 .4226 .39633 .42995 .39366 .42565 .38908 Metetra
.672 .744 .905 r
.42565 .38908 .42995 .39366 .43732 .39102 .43303 .38641 Metetra
.67 .743 .905 r
.43303 .38641 .43732 .39102 .4447 .3884 .44043 .38377 Metetra
.666 .742 .907 r
.44043 .38377 .4447 .3884 .45211 .38586 .44784 .38121 Metetra
.646 .735 .914 r
.44784 .38121 .45211 .38586 .45952 .3837 .45527 .37902 Metetra
.399 .628 .946 r
.45527 .37902 .45952 .3837 .46691 .38439 .46266 .37995 Metetra
.702 .763 .899 r
.46266 .37995 .46691 .38439 .47439 .38116 .47016 .37662 Metetra
.744 .768 .872 r
.47016 .37662 .47439 .38116 .48191 .3768 .4777 .37216 Metetra
.684 .75 .901 r
.4777 .37216 .48191 .3768 .48942 .37385 .48522 .36919 Metetra
.67 .745 .907 r
.48522 .36919 .48942 .37385 .49695 .37117 .49276 .3665 Metetra
.662 .743 .91 r
.49276 .3665 .49695 .37117 .5045 .36864 .50033 .36397 Metetra
.654 .74 .913 r
.50033 .36397 .5045 .36864 .51207 .36626 .50791 .36158 Metetra
.643 .737 .917 r
.50791 .36158 .51207 .36626 .51966 .36408 .51551 .35939 Metetra
.623 .731 .924 r
.51551 .35939 .51966 .36408 .52727 .36222 .52314 .35754 Metetra
.579 .714 .935 r
.52314 .35754 .52727 .36222 .53491 .36102 .53079 .35637 Metetra
.421 .643 .949 r
.53079 .35637 .53491 .36102 .54258 .36165 .53848 .35709 Metetra
.676 .198 0 r
.53848 .35709 .54258 .36165 .55038 .37186 .54627 .36802 Metetra
.937 .564 .229 r
.54627 .36802 .55038 .37186 .55841 .39781 .55434 .3992 Metetra
.99 .787 .536 r
.55434 .3992 .55841 .39781 .56581 .36164 .56174 .35729 Metetra
.828 .798 .823 r
.56174 .35729 .56581 .36164 .57351 .3543 .56946 .34975 Metetra
.734 .776 .887 r
.56946 .34975 .57351 .3543 .58127 .35014 .57723 .34553 Metetra
.684 .76 .909 r
.57723 .34553 .58127 .35014 .58906 .34709 .58504 .34247 Metetra
.644 .747 .924 r
.58504 .34247 .58906 .34709 .59689 .34475 .59288 .34017 Metetra
.577 .723 .942 r
.59288 .34017 .59689 .34475 .60476 .34338 .60078 .33896 Metetra
.096 .463 .889 r
.60078 .33896 .60476 .34338 .61278 .34581 .60882 .34238 Metetra
.909 .503 .132 r
.60882 .34238 .61278 .34581 .62122 .36091 .61754 .36709 Metetra
.977 .856 .466 r
.61754 .36709 .62122 .36091 .62861 .3405 .62469 .33687 Metetra
.24 .602 .952 r
.62469 .33687 .62861 .3405 .63672 .34245 .6328 .33856 Metetra
.699 .213 0 r
.6328 .33856 .63672 .34245 .64523 .35374 .64134 .35049 Metetra
.959 .623 .308 r
.64134 .35049 .64523 .35374 .65593 .41674 .65223 .41968 Metetra
.957 .635 .37 r
.65223 .41968 .65593 .41674 .66823 .50857 .66526 .53073 Metetra
.956 .749 .295 r
.66526 .53073 .66823 .50857 .67268 .41702 .6698 .43688 Metetra
.981 .742 .337 r
.6698 .43688 .67268 .41702 .67661 .32363 .67275 .31883 Metetra
.902 .794 .732 r
.67275 .31883 .67661 .32363 .68417 .31065 .68033 .30563 Metetra
.753 .772 .868 r
.68033 .30563 .68417 .31065 .69213 .3057 .68831 .30065 Metetra
.699 .754 .895 r
.68831 .30065 .69213 .3057 .70018 .30219 .69638 .29712 Metetra
.68 .747 .902 r
.69638 .29712 .70018 .30219 .70828 .2991 .70449 .29402 Metetra
.672 .744 .905 r
.70449 .29402 .70828 .2991 .71641 .29618 .71264 .29109 Metetra
.668 .743 .906 r
.71264 .29109 .71641 .29618 .72457 .29333 .72081 .28822 Metetra
.666 .742 .907 r
.72081 .28822 .72457 .29333 .73275 .29051 .72901 .28539 Metetra
.665 .742 .907 r
.72901 .28539 .73275 .29051 .74096 .2877 .73724 .28256 Metetra
.665 .742 .908 r
.73724 .28256 .74096 .2877 .74919 .2849 .74549 .27974 Metetra
.664 .742 .908 r
.74549 .27974 .74919 .2849 .75745 .28209 .75376 .27693 Metetra
.664 .742 .908 r
.75376 .27693 .75745 .28209 .76573 .27929 .76206 .2741 Metetra
.664 .741 .908 r
.76206 .2741 .76573 .27929 .77403 .27648 .77038 .27128 Metetra
.664 .741 .908 r
.77038 .27128 .77403 .27648 .78236 .27366 .77872 .26844 Metetra
.664 .741 .908 r
.77872 .26844 .78236 .27366 .79071 .27083 .78709 .2656 Metetra
.664 .741 .908 r
.78709 .2656 .79071 .27083 .79909 .268 .79549 .26276 Metetra
.664 .741 .908 r
.79549 .26276 .79909 .268 .80749 .26516 .80391 .2599 Metetra
.664 .741 .908 r
.80391 .2599 .80749 .26516 .81592 .26231 .81235 .25704 Metetra
.664 .741 .908 r
.81235 .25704 .81592 .26231 .82437 .25946 .82082 .25417 Metetra
.664 .741 .908 r
.82082 .25417 .82437 .25946 .83285 .25659 .82931 .25129 Metetra
.664 .741 .908 r
.82931 .25129 .83285 .25659 .84135 .25372 .83783 .2484 Metetra
.664 .741 .908 r
.83783 .2484 .84135 .25372 .84987 .25084 .84637 .24551 Metetra
.762 .758 .849 r
.23567 .45174 .2403 .45604 .24741 .45118 .24279 .44686 Metetra
.697 .749 .892 r
.24279 .44686 .24741 .45118 .25436 .44815 .24976 .44382 Metetra
.677 .745 .902 r
.24976 .44382 .25436 .44815 .2613 .44554 .25671 .4412 Metetra
.669 .743 .906 r
.25671 .4412 .2613 .44554 .26825 .44308 .26366 .43873 Metetra
.665 .742 .908 r
.26366 .43873 .26825 .44308 .2752 .4407 .27063 .43634 Metetra
.662 .741 .909 r
.27063 .43634 .2752 .4407 .28218 .43837 .27761 .434 Metetra
.66 .74 .91 r
.27761 .434 .28218 .43837 .28916 .43607 .28461 .43169 Metetra
.657 .74 .911 r
.28461 .43169 .28916 .43607 .29616 .43381 .29162 .42941 Metetra
.655 .739 .912 r
.29162 .42941 .29616 .43381 .30318 .43159 .29865 .42718 Metetra
.651 .738 .913 r
.29865 .42718 .30318 .43159 .31022 .42942 .3057 .425 Metetra
.646 .737 .915 r
.3057 .425 .31022 .42942 .31726 .42734 .31276 .42291 Metetra
.638 .734 .918 r
.31276 .42291 .31726 .42734 .32432 .42538 .31983 .42094 Metetra
.623 .73 .923 r
.31983 .42094 .32432 .42538 .33139 .42367 .3269 .41921 Metetra
.59 .719 .933 r
.3269 .41921 .33139 .42367 .33845 .42244 .33397 .41798 Metetra
.479 .675 .951 r
.33397 .41798 .33845 .42244 .34546 .42251 .34099 .41803 Metetra
0 .089 .591 r
.34099 .41803 .34546 .42251 .35219 .42844 .34772 .42397 Metetra
.953 .654 .43 r
.34772 .42397 .35219 .42844 .35588 .49502 .35132 .49061 Metetra
.942 .71 .557 r
.35132 .49061 .35588 .49502 .36686 .41888 .36242 .41437 Metetra
.822 .767 .8 r
.36242 .41437 .36686 .41888 .37435 .41111 .36993 .40659 Metetra
.744 .761 .866 r
.36993 .40659 .37435 .41111 .38168 .40672 .37728 .40218 Metetra
.71 .754 .886 r
.37728 .40218 .38168 .40672 .38899 .40326 .3846 .3987 Metetra
.693 .75 .895 r
.3846 .3987 .38899 .40326 .39629 .40017 .39192 .39561 Metetra
.684 .747 .899 r
.39192 .39561 .39629 .40017 .40361 .39728 .39924 .3927 Metetra
.679 .746 .902 r
.39924 .3927 .40361 .39728 .41094 .39449 .40659 .3899 Metetra
.676 .745 .903 r
.40659 .3899 .41094 .39449 .41828 .39177 .41394 .38716 Metetra
.673 .744 .904 r
.41394 .38716 .41828 .39177 .42565 .38908 .42132 .38446 Metetra
.672 .744 .905 r
.42132 .38446 .42565 .38908 .43303 .38641 .42871 .38178 Metetra
.67 .743 .905 r
.42871 .38178 .43303 .38641 .44043 .38377 .43613 .37913 Metetra
.666 .742 .907 r
.43613 .37913 .44043 .38377 .44784 .38121 .44356 .37655 Metetra
.646 .736 .915 r
.44356 .37655 .44784 .38121 .45527 .37902 .451 .37435 Metetra
.397 .629 .947 r
.451 .37435 .45527 .37902 .46266 .37995 .4584 .37528 Metetra
.701 .753 .892 r
.4584 .37528 .46266 .37995 .47016 .37662 .46591 .37193 Metetra
.743 .763 .869 r
.46591 .37193 .47016 .37662 .4777 .37216 .47346 .36746 Metetra
.684 .748 .9 r
.47346 .36746 .4777 .37216 .48522 .36919 .481 .36447 Metetra
.67 .744 .905 r
.481 .36447 .48522 .36919 .49276 .3665 .48856 .36177 Metetra
.662 .741 .908 r
.48856 .36177 .49276 .3665 .50033 .36397 .49613 .35922 Metetra
.654 .738 .911 r
.49613 .35922 .50033 .36397 .50791 .36158 .50373 .35682 Metetra
.643 .735 .915 r
.50373 .35682 .50791 .36158 .51551 .35939 .51134 .35462 Metetra
.624 .728 .921 r
.51134 .35462 .51551 .35939 .52314 .35754 .51898 .35276 Metetra
.58 .711 .932 r
.51898 .35276 .52314 .35754 .53079 .35637 .52665 .35157 Metetra
.426 .64 .946 r
.52665 .35157 .53079 .35637 .53848 .35709 .53434 .35228 Metetra
.66 .187 0 r
.53434 .35228 .53848 .35709 .54627 .36802 .54213 .36322 Metetra
.937 .564 .207 r
.54213 .36322 .54627 .36802 .55434 .3992 .55017 .39442 Metetra
.956 .737 .567 r
.55017 .39442 .55434 .3992 .56174 .35729 .55764 .35246 Metetra
.82 .78 .814 r
.55764 .35246 .56174 .35729 .56946 .34975 .56538 .34489 Metetra
.731 .763 .878 r
.56538 .34489 .56946 .34975 .57723 .34553 .57317 .34066 Metetra
.683 .749 .901 r
.57317 .34066 .57723 .34553 .58504 .34247 .58099 .33759 Metetra
.645 .736 .915 r
.58099 .33759 .58504 .34247 .59288 .34017 .58885 .33528 Metetra
.581 .71 .931 r
.58885 .33528 .59288 .34017 .60078 .33896 .59676 .33405 Metetra
.144 .47 .893 r
.59676 .33405 .60078 .33896 .60882 .34238 .60481 .33747 Metetra
.897 .494 .078 r
.60481 .33747 .60882 .34238 .61754 .36709 .61351 .36218 Metetra
.953 .755 .598 r
.61351 .36218 .61754 .36709 .62469 .33687 .62071 .33192 Metetra
.341 .589 .936 r
.62071 .33192 .62469 .33687 .6328 .33856 .62883 .33361 Metetra
.627 .164 0 r
.62883 .33361 .6328 .33856 .64134 .35049 .63737 .34553 Metetra
.961 .626 .292 r
.63737 .34553 .64134 .35049 .65223 .41968 .64819 .41479 Metetra
.968 .647 .332 r
.64819 .41479 .65223 .41968 .66526 .53073 .66111 .52598 Metetra
.976 .717 .473 r
.66111 .52598 .66526 .53073 .6698 .43688 .66577 .43199 Metetra
.977 .709 .455 r
.66577 .43199 .6698 .43688 .67275 .31883 .66886 .31379 Metetra
.898 .788 .731 r
.66886 .31379 .67275 .31883 .68033 .30563 .67647 .30057 Metetra
.753 .771 .868 r
.67647 .30057 .68033 .30563 .68831 .30065 .68447 .29558 Metetra
.699 .754 .895 r
.68447 .29558 .68831 .30065 .69638 .29712 .69256 .29203 Metetra
.68 .747 .902 r
.69256 .29203 .69638 .29712 .70449 .29402 .70069 .28892 Metetra
.672 .744 .905 r
.70069 .28892 .70449 .29402 .71264 .29109 .70885 .28597 Metetra
.668 .743 .906 r
.70885 .28597 .71264 .29109 .72081 .28822 .71704 .28309 Metetra
.666 .742 .907 r
.71704 .28309 .72081 .28822 .72901 .28539 .72526 .28023 Metetra
.665 .742 .907 r
.72526 .28023 .72901 .28539 .73724 .28256 .7335 .2774 Metetra
.665 .742 .908 r
.7335 .2774 .73724 .28256 .74549 .27974 .74176 .27456 Metetra
.664 .742 .908 r
.74176 .27456 .74549 .27974 .75376 .27693 .75005 .27173 Metetra
.664 .742 .908 r
.75005 .27173 .75376 .27693 .76206 .2741 .75836 .26889 Metetra
.664 .741 .908 r
.75836 .26889 .76206 .2741 .77038 .27128 .7667 .26605 Metetra
.664 .741 .908 r
.7667 .26605 .77038 .27128 .77872 .26844 .77507 .2632 Metetra
.664 .741 .908 r
.77507 .2632 .77872 .26844 .78709 .2656 .78345 .26035 Metetra
.664 .741 .908 r
.78345 .26035 .78709 .2656 .79549 .26276 .79187 .25748 Metetra
.664 .741 .908 r
.79187 .25748 .79549 .26276 .80391 .2599 .8003 .25461 Metetra
.664 .741 .908 r
.8003 .25461 .80391 .2599 .81235 .25704 .80876 .25173 Metetra
.664 .741 .908 r
.80876 .25173 .81235 .25704 .82082 .25417 .81725 .24885 Metetra
.664 .741 .908 r
.81725 .24885 .82082 .25417 .82931 .25129 .82576 .24595 Metetra
.664 .741 .908 r
.82576 .24595 .82931 .25129 .83783 .2484 .8343 .24305 Metetra
.664 .741 .908 r
.8343 .24305 .83783 .2484 .84637 .24551 .84286 .24014 Metetra
.755 .748 .846 r
.23103 .44718 .23567 .45174 .24279 .44686 .23815 .44243 Metetra
.695 .745 .889 r
.23815 .44243 .24279 .44686 .24976 .44382 .24513 .43942 Metetra
.676 .742 .9 r
.24513 .43942 .24976 .44382 .25671 .4412 .25209 .4368 Metetra
.669 .741 .904 r
.25209 .4368 .25671 .4412 .26366 .43873 .25906 .43432 Metetra
.665 .74 .906 r
.25906 .43432 .26366 .43873 .27063 .43634 .26604 .43193 Metetra
.662 .74 .908 r
.26604 .43193 .27063 .43634 .27761 .434 .27303 .42957 Metetra
.66 .739 .909 r
.27303 .42957 .27761 .434 .28461 .43169 .28004 .42725 Metetra
.657 .739 .91 r
.28004 .42725 .28461 .43169 .29162 .42941 .28706 .42497 Metetra
.655 .738 .911 r
.28706 .42497 .29162 .42941 .29865 .42718 .2941 .42272 Metetra
.652 .737 .912 r
.2941 .42272 .29865 .42718 .3057 .425 .30116 .42053 Metetra
.647 .735 .914 r
.30116 .42053 .3057 .425 .31276 .42291 .30823 .41842 Metetra
.639 .733 .917 r
.30823 .41842 .31276 .42291 .31983 .42094 .31531 .41644 Metetra
.624 .728 .922 r
.31531 .41644 .31983 .42094 .3269 .41921 .3224 .41469 Metetra
.592 .717 .931 r
.3224 .41469 .3269 .41921 .33397 .41798 .32948 .41342 Metetra
.486 .675 .948 r
.32948 .41342 .33397 .41798 .34099 .41803 .33651 .41343 Metetra
0 .134 .638 r
.33651 .41343 .34099 .41803 .34772 .42397 .34325 .41906 Metetra
.954 .635 .384 r
.34325 .41906 .34772 .42397 .35132 .49061 .34797 .46332 Metetra
.841 .61 .589 r
.34797 .46332 .35132 .49061 .36242 .41437 .35797 .40962 Metetra
.817 .76 .799 r
.35797 .40962 .36242 .41437 .36993 .40659 .36549 .40196 Metetra
.742 .757 .864 r
.36549 .40196 .36993 .40659 .37728 .40218 .37286 .39757 Metetra
.709 .751 .885 r
.37286 .39757 .37728 .40218 .3846 .3987 .38019 .39409 Metetra
.693 .748 .894 r
.38019 .39409 .3846 .3987 .39192 .39561 .38752 .39099 Metetra
.684 .746 .898 r
.38752 .39099 .39192 .39561 .39924 .3927 .39486 .38808 Metetra
.679 .745 .901 r
.39486 .38808 .39924 .3927 .40659 .3899 .40222 .38527 Metetra
.675 .744 .903 r
.40222 .38527 .40659 .3899 .41394 .38716 .40959 .38252 Metetra
.673 .744 .904 r
.40959 .38252 .41394 .38716 .42132 .38446 .41697 .37981 Metetra
.672 .743 .904 r
.41697 .37981 .42132 .38446 .42871 .38178 .42438 .37713 Metetra
.67 .743 .905 r
.42438 .37713 .42871 .38178 .43613 .37913 .43181 .37447 Metetra
.665 .742 .908 r
.43181 .37447 .43613 .37913 .44356 .37655 .43925 .3719 Metetra
.645 .737 .916 r
.43925 .3719 .44356 .37655 .451 .37435 .4467 .3697 Metetra
.422 .644 .949 r
.4467 .3697 .451 .37435 .4584 .37528 .45412 .37036 Metetra
.696 .742 .887 r
.45412 .37036 .4584 .37528 .46591 .37193 .46164 .36709 Metetra
.739 .757 .867 r
.46164 .36709 .46591 .37193 .47346 .36746 .46921 .36269 Metetra
.684 .746 .898 r
.46921 .36269 .47346 .36746 .481 .36447 .47676 .35969 Metetra
.67 .742 .904 r
.47676 .35969 .481 .36447 .48856 .36177 .48433 .35697 Metetra
.662 .739 .907 r
.48433 .35697 .48856 .36177 .49613 .35922 .49192 .35441 Metetra
.655 .736 .91 r
.49192 .35441 .49613 .35922 .50373 .35682 .49953 .35199 Metetra
.644 .732 .913 r
.49953 .35199 .50373 .35682 .51134 .35462 .50716 .34976 Metetra
.625 .725 .919 r
.50716 .34976 .51134 .35462 .51898 .35276 .51481 .34787 Metetra
.584 .708 .929 r
.51481 .34787 .51898 .35276 .52665 .35157 .52248 .34663 Metetra
.441 .641 .942 r
.52248 .34663 .52665 .35157 .53434 .35228 .53019 .34724 Metetra
.599 .131 0 r
.53019 .34724 .53434 .35228 .54213 .36322 .53797 .35745 Metetra
.914 .526 .12 r
.53797 .35745 .54213 .36322 .55017 .39442 .54595 .38345 Metetra
.906 .687 .594 r
.54595 .38345 .55017 .39442 .55764 .35246 .55352 .34713 Metetra
.809 .762 .809 r
.55352 .34713 .55764 .35246 .56538 .34489 .56128 .33973 Metetra
.727 .75 .871 r
.56128 .33973 .56538 .34489 .57317 .34066 .56908 .33552 Metetra
.682 .738 .893 r
.56908 .33552 .57317 .34066 .58099 .33759 .57692 .33243 Metetra
.648 .726 .905 r
.57692 .33243 .58099 .33759 .58885 .33528 .58479 .33006 Metetra
.595 .702 .918 r
.58479 .33006 .58885 .33528 .59676 .33405 .59271 .32865 Metetra
.293 .538 .915 r
.59271 .32865 .59676 .33405 .60481 .33747 .60074 .33105 Metetra
.683 .262 0 r
.60074 .33105 .60481 .33747 .61351 .36218 .60917 .34616 Metetra
.813 .616 .637 r
.60917 .34616 .61351 .36218 .62071 .33192 .61667 .32565 Metetra
.383 .552 .898 r
.61667 .32565 .62071 .33192 .62883 .33361 .62481 .32757 Metetra
0 0 .482 r
.62481 .32757 .62883 .33361 .63737 .34553 .63332 .33887 Metetra
.959 .621 .261 r
.63332 .33887 .63737 .34553 .64819 .41479 .64384 .40205 Metetra
.963 .644 .256 r
.64384 .40205 .64819 .41479 .66111 .52598 .65587 .4942 Metetra
.899 .634 .528 r
.65587 .4942 .66111 .52598 .66577 .43199 .66067 .40227 Metetra
.907 .64 .522 r
.66067 .40227 .66577 .43199 .66886 .31379 .66495 .30853 Metetra
.893 .782 .732 r
.66495 .30853 .66886 .31379 .67647 .30057 .67259 .29547 Metetra
.753 .77 .868 r
.67259 .29547 .67647 .30057 .68447 .29558 .68061 .29047 Metetra
.699 .754 .895 r
.68061 .29047 .68447 .29558 .69256 .29203 .68871 .28691 Metetra
.68 .747 .902 r
.68871 .28691 .69256 .29203 .70069 .28892 .69686 .28378 Metetra
.672 .744 .905 r
.69686 .28378 .70069 .28892 .70885 .28597 .70504 .28082 Metetra
.668 .743 .906 r
.70504 .28082 .70885 .28597 .71704 .28309 .71324 .27792 Metetra
.666 .742 .907 r
.71324 .27792 .71704 .28309 .72526 .28023 .72148 .27506 Metetra
.665 .742 .907 r
.72148 .27506 .72526 .28023 .7335 .2774 .72973 .2722 Metetra
.665 .742 .908 r
.72973 .2722 .7335 .2774 .74176 .27456 .73802 .26935 Metetra
.664 .742 .908 r
.73802 .26935 .74176 .27456 .75005 .27173 .74632 .26651 Metetra
.664 .742 .908 r
.74632 .26651 .75005 .27173 .75836 .26889 .75465 .26365 Metetra
.664 .741 .908 r
.75465 .26365 .75836 .26889 .7667 .26605 .76301 .2608 Metetra
.664 .741 .908 r
.76301 .2608 .7667 .26605 .77507 .2632 .77139 .25793 Metetra
.664 .741 .908 r
.77139 .25793 .77507 .2632 .78345 .26035 .77979 .25506 Metetra
.664 .741 .908 r
.77979 .25506 .78345 .26035 .79187 .25748 .78822 .25218 Metetra
.664 .741 .908 r
.78822 .25218 .79187 .25748 .8003 .25461 .79668 .2493 Metetra
.664 .741 .908 r
.79668 .2493 .8003 .25461 .80876 .25173 .80516 .2464 Metetra
.664 .741 .908 r
.80516 .2464 .80876 .25173 .81725 .24885 .81366 .2435 Metetra
.664 .741 .908 r
.81366 .2435 .81725 .24885 .82576 .24595 .82219 .24059 Metetra
.664 .741 .908 r
.82219 .24059 .82576 .24595 .8343 .24305 .83074 .23767 Metetra
.664 .741 .908 r
.83074 .23767 .8343 .24305 .84286 .24014 .83932 .23475 Metetra
.749 .746 .849 r
.22638 .44255 .23103 .44718 .23815 .44243 .2335 .43798 Metetra
.694 .744 .89 r
.2335 .43798 .23815 .44243 .24513 .43942 .24049 .43499 Metetra
.676 .742 .9 r
.24049 .43499 .24513 .43942 .25209 .4368 .24746 .43237 Metetra
.668 .741 .904 r
.24746 .43237 .25209 .4368 .25906 .43432 .25443 .4299 Metetra
.664 .74 .906 r
.25443 .4299 .25906 .43432 .26604 .43193 .26142 .42749 Metetra
.662 .74 .908 r
.26142 .42749 .26604 .43193 .27303 .42957 .26842 .42513 Metetra
.66 .739 .909 r
.26842 .42513 .27303 .42957 .28004 .42725 .27544 .4228 Metetra
.657 .739 .91 r
.27544 .4228 .28004 .42725 .28706 .42497 .28248 .4205 Metetra
.655 .738 .911 r
.28248 .4205 .28706 .42497 .2941 .42272 .28953 .41824 Metetra
.652 .737 .912 r
.28953 .41824 .2941 .42272 .30116 .42053 .2966 .41604 Metetra
.647 .736 .914 r
.2966 .41604 .30116 .42053 .30823 .41842 .30368 .41391 Metetra
.639 .733 .917 r
.30368 .41391 .30823 .41842 .31531 .41644 .31077 .41191 Metetra
.625 .728 .921 r
.31077 .41191 .31531 .41644 .3224 .41469 .31787 .41014 Metetra
.593 .717 .93 r
.31787 .41014 .3224 .41469 .32948 .41342 .32496 .40884 Metetra
.492 .677 .947 r
.32496 .40884 .32948 .41342 .33651 .41343 .332 .40878 Metetra
0 .192 .692 r
.332 .40878 .33651 .41343 .34325 .41906 .33877 .41396 Metetra
.946 .59 .289 r
.33877 .41396 .34325 .41906 .34797 .46332 .34438 .4411 Metetra
.824 .606 .607 r
.34438 .4411 .34797 .46332 .35797 .40962 .35351 .40478 Metetra
.812 .757 .801 r
.35351 .40478 .35797 .40962 .36549 .40196 .36104 .3973 Metetra
.741 .756 .865 r
.36104 .3973 .36549 .40196 .37286 .39757 .36841 .39293 Metetra
.708 .751 .885 r
.36841 .39293 .37286 .39757 .38019 .39409 .37576 .38946 Metetra
.692 .748 .894 r
.37576 .38946 .38019 .39409 .38752 .39099 .3831 .38635 Metetra
.684 .746 .898 r
.3831 .38635 .38752 .39099 .39486 .38808 .39045 .38343 Metetra
.678 .745 .901 r
.39045 .38343 .39486 .38808 .40222 .38527 .39782 .38062 Metetra
.675 .744 .903 r
.39782 .38062 .40222 .38527 .40959 .38252 .40521 .37786 Metetra
.673 .744 .904 r
.40521 .37786 .40959 .38252 .41697 .37981 .41261 .37514 Metetra
.671 .743 .905 r
.41261 .37514 .41697 .37981 .42438 .37713 .42003 .37245 Metetra
.669 .743 .906 r
.42003 .37245 .42438 .37713 .43181 .37447 .42747 .36979 Metetra
.665 .742 .908 r
.42747 .36979 .43181 .37447 .43925 .3719 .43492 .36722 Metetra
.645 .736 .916 r
.43492 .36722 .43925 .3719 .4467 .3697 .44239 .36501 Metetra
.463 .663 .949 r
.44239 .36501 .4467 .3697 .45412 .37036 .44982 .36526 Metetra
.687 .732 .885 r
.44982 .36526 .45412 .37036 .46164 .36709 .45735 .36216 Metetra
.733 .753 .868 r
.45735 .36216 .46164 .36709 .46921 .36269 .46493 .35789 Metetra
.684 .746 .898 r
.46493 .35789 .46921 .36269 .47676 .35969 .4725 .35488 Metetra
.67 .742 .904 r
.4725 .35488 .47676 .35969 .48433 .35697 .48008 .35215 Metetra
.663 .739 .907 r
.48008 .35215 .48433 .35697 .49192 .35441 .48768 .34957 Metetra
.655 .737 .91 r
.48768 .34957 .49192 .35441 .49953 .35199 .4953 .34713 Metetra
.644 .733 .913 r
.4953 .34713 .49953 .35199 .50716 .34976 .50295 .34488 Metetra
.626 .726 .918 r
.50295 .34488 .50716 .34976 .51481 .34787 .51061 .34295 Metetra
.587 .71 .928 r
.51061 .34295 .51481 .34787 .52248 .34663 .5183 .34167 Metetra
.451 .646 .942 r
.5183 .34167 .52248 .34663 .53019 .34724 .52601 .34214 Metetra
0 0 .471 r
.52601 .34214 .53019 .34724 .53797 .35745 .53379 .35134 Metetra
.851 .438 0 r
.53379 .35134 .53797 .35745 .54595 .38345 .5417 .37089 Metetra
.876 .667 .616 r
.5417 .37089 .54595 .38345 .55352 .34713 .54937 .34171 Metetra
.803 .759 .812 r
.54937 .34171 .55352 .34713 .56128 .33973 .55715 .33453 Metetra
.725 .749 .872 r
.55715 .33453 .56128 .33973 .56908 .33552 .56498 .33036 Metetra
.683 .738 .893 r
.56498 .33036 .56908 .33552 .57692 .33243 .57283 .32725 Metetra
.651 .727 .905 r
.57283 .32725 .57692 .33243 .58479 .33006 .58071 .32481 Metetra
.605 .707 .916 r
.58071 .32481 .58479 .33006 .59271 .32865 .58864 .32322 Metetra
.395 .596 .927 r
.58864 .32322 .59271 .32865 .60074 .33105 .59666 .32455 Metetra
0 .258 .768 r
.59666 .32455 .60074 .33105 .60917 .34616 .60485 .33118 Metetra
.752 .589 .674 r
.60485 .33118 .60917 .34616 .61667 .32565 .61261 .31938 Metetra
.363 .542 .897 r
.61261 .31938 .61667 .32565 .62481 .32757 .62076 .32152 Metetra
0 0 .524 r
.62076 .32152 .62481 .32757 .63332 .33887 .62925 .33212 Metetra
.954 .61 .238 r
.62925 .33212 .63332 .33887 .64384 .40205 .63943 .38768 Metetra
.941 .619 .178 r
.63943 .38768 .64384 .40205 .65587 .4942 .6505 .45784 Metetra
.875 .614 .54 r
.6505 .45784 .65587 .4942 .66067 .40227 .65574 .37571 Metetra
.896 .636 .538 r
.65574 .37571 .66067 .40227 .66495 .30853 .661 .30305 Metetra
.886 .777 .735 r
.661 .30305 .66495 .30853 .67259 .29547 .66869 .29033 Metetra
.752 .77 .868 r
.66869 .29033 .67259 .29547 .68061 .29047 .67673 .28533 Metetra
.699 .754 .895 r
.67673 .28533 .68061 .29047 .68871 .28691 .68485 .28176 Metetra
.68 .747 .902 r
.68485 .28176 .68871 .28691 .69686 .28378 .69301 .27862 Metetra
.672 .744 .905 r
.69301 .27862 .69686 .28378 .70504 .28082 .70121 .27564 Metetra
.668 .743 .906 r
.70121 .27564 .70504 .28082 .71324 .27792 .70943 .27273 Metetra
.666 .742 .907 r
.70943 .27273 .71324 .27792 .72148 .27506 .71768 .26985 Metetra
.665 .742 .907 r
.71768 .26985 .72148 .27506 .72973 .2722 .72595 .26698 Metetra
.665 .742 .908 r
.72595 .26698 .72973 .2722 .73802 .26935 .73425 .26412 Metetra
.664 .742 .908 r
.73425 .26412 .73802 .26935 .74632 .26651 .74258 .26125 Metetra
.664 .742 .908 r
.74258 .26125 .74632 .26651 .75465 .26365 .75092 .25839 Metetra
.664 .741 .908 r
.75092 .25839 .75465 .26365 .76301 .2608 .7593 .25551 Metetra
.664 .741 .908 r
.7593 .25551 .76301 .2608 .77139 .25793 .76769 .25263 Metetra
.664 .741 .908 r
.76769 .25263 .77139 .25793 .77979 .25506 .77612 .24975 Metetra
.664 .741 .908 r
.77612 .24975 .77979 .25506 .78822 .25218 .78456 .24685 Metetra
.664 .741 .908 r
.78456 .24685 .78822 .25218 .79668 .2493 .79303 .24395 Metetra
.664 .741 .908 r
.79303 .24395 .79668 .2493 .80516 .2464 .80153 .24104 Metetra
.664 .741 .908 r
.80153 .24104 .80516 .2464 .81366 .2435 .81005 .23812 Metetra
.664 .741 .908 r
.81005 .23812 .81366 .2435 .82219 .24059 .8186 .2352 Metetra
.664 .741 .908 r
.8186 .2352 .82219 .24059 .83074 .23767 .82717 .23226 Metetra
.664 .741 .908 r
.82717 .23226 .83074 .23767 .83932 .23475 .83577 .22932 Metetra
.742 .744 .853 r
.22171 .43786 .22638 .44255 .2335 .43798 .22882 .4335 Metetra
.692 .743 .891 r
.22882 .4335 .2335 .43798 .24049 .43499 .23582 .43053 Metetra
.675 .741 .901 r
.23582 .43053 .24049 .43499 .24746 .43237 .2428 .42792 Metetra
.668 .741 .905 r
.2428 .42792 .24746 .43237 .25443 .4299 .24978 .42545 Metetra
.664 .74 .907 r
.24978 .42545 .25443 .4299 .26142 .42749 .25678 .42303 Metetra
.662 .74 .908 r
.25678 .42303 .26142 .42749 .26842 .42513 .2638 .42066 Metetra
.66 .739 .909 r
.2638 .42066 .26842 .42513 .27544 .4228 .27083 .41832 Metetra
.657 .739 .91 r
.27083 .41832 .27544 .4228 .28248 .4205 .27787 .41601 Metetra
.655 .738 .911 r
.27787 .41601 .28248 .4205 .28953 .41824 .28493 .41374 Metetra
.652 .737 .912 r
.28493 .41374 .28953 .41824 .2966 .41604 .29201 .41152 Metetra
.647 .736 .914 r
.29201 .41152 .2966 .41604 .30368 .41391 .2991 .40938 Metetra
.639 .733 .916 r
.2991 .40938 .30368 .41391 .31077 .41191 .30621 .40736 Metetra
.626 .729 .921 r
.30621 .40736 .31077 .41191 .31787 .41014 .31332 .40557 Metetra
.595 .718 .93 r
.31332 .40557 .31787 .41014 .32496 .40884 .32042 .40424 Metetra
.499 .68 .946 r
.32042 .40424 .32496 .40884 .332 .40878 .32747 .40409 Metetra
0 .265 .754 r
.32747 .40409 .332 .40878 .33877 .41396 .33428 .4087 Metetra
.902 .492 .111 r
.33428 .4087 .33877 .41396 .34438 .4411 .34043 .42561 Metetra
.828 .627 .631 r
.34043 .42561 .34438 .4411 .35351 .40478 .34902 .39986 Metetra
.807 .755 .804 r
.34902 .39986 .35351 .40478 .36104 .3973 .35656 .3926 Metetra
.739 .755 .866 r
.35656 .3926 .36104 .3973 .36841 .39293 .36394 .38827 Metetra
.708 .751 .886 r
.36394 .38827 .36841 .39293 .37576 .38946 .3713 .3848 Metetra
.692 .748 .894 r
.3713 .3848 .37576 .38946 .3831 .38635 .37866 .38169 Metetra
.683 .746 .899 r
.37866 .38169 .3831 .38635 .39045 .38343 .38603 .37876 Metetra
.678 .745 .901 r
.38603 .37876 .39045 .38343 .39782 .38062 .39341 .37594 Metetra
.675 .744 .903 r
.39341 .37594 .39782 .38062 .40521 .37786 .4008 .37317 Metetra
.673 .744 .904 r
.4008 .37317 .40521 .37786 .41261 .37514 .40822 .37045 Metetra
.671 .743 .905 r
.40822 .37045 .41261 .37514 .42003 .37245 .41565 .36775 Metetra
.669 .743 .906 r
.41565 .36775 .42003 .37245 .42747 .36979 .4231 .36509 Metetra
.664 .742 .908 r
.4231 .36509 .42747 .36979 .43492 .36722 .43057 .36251 Metetra
.645 .736 .916 r
.43057 .36251 .43492 .36722 .44239 .36501 .43805 .3603 Metetra
.503 .681 .946 r
.43805 .3603 .44239 .36501 .44982 .36526 .4455 .36009 Metetra
.678 .728 .888 r
.4455 .36009 .44982 .36526 .45735 .36216 .45304 .35717 Metetra
.727 .749 .87 r
.45304 .35717 .45735 .36216 .46493 .35789 .46063 .35306 Metetra
.683 .745 .898 r
.46063 .35306 .46493 .35789 .4725 .35488 .46821 .35005 Metetra
.67 .742 .904 r
.46821 .35005 .4725 .35488 .48008 .35215 .47581 .3473 Metetra
.663 .739 .907 r
.47581 .3473 .48008 .35215 .48768 .34957 .48343 .3447 Metetra
.655 .737 .91 r
.48343 .3447 .48768 .34957 .4953 .34713 .49106 .34224 Metetra
.645 .733 .913 r
.49106 .34224 .4953 .34713 .50295 .34488 .49872 .33997 Metetra
.627 .726 .918 r
.49872 .33997 .50295 .34488 .51061 .34295 .50639 .33801 Metetra
.589 .711 .927 r
.50639 .33801 .51061 .34295 .5183 .34167 .51409 .33668 Metetra
.463 .652 .941 r
.51409 .33668 .5183 .34167 .52601 .34214 .52182 .33701 Metetra
0 .036 .568 r
.52182 .33701 .52601 .34214 .53379 .35134 .52959 .34504 Metetra
.699 .269 0 r
.52959 .34504 .53379 .35134 .5417 .37089 .53746 .35901 Metetra
.858 .664 .639 r
.53746 .35901 .5417 .37089 .54937 .34171 .54521 .3362 Metetra
.796 .756 .816 r
.54521 .3362 .54937 .34171 .55715 .33453 .55301 .3293 Metetra
.722 .748 .872 r
.55301 .3293 .55715 .33453 .56498 .33036 .56085 .32516 Metetra
.682 .738 .893 r
.56085 .32516 .56498 .33036 .57283 .32725 .56871 .32204 Metetra
.653 .728 .904 r
.56871 .32204 .57283 .32725 .58071 .32481 .57661 .31954 Metetra
.615 .711 .914 r
.57661 .31954 .58071 .32481 .58864 .32322 .58455 .31777 Metetra
.481 .642 .927 r
.58455 .31777 .58864 .32322 .59666 .32455 .59255 .31804 Metetra
.46 .587 .893 r
.59255 .31804 .59666 .32455 .60485 .33118 .60061 .31897 Metetra
.693 .582 .726 r
.60061 .31897 .60485 .33118 .61261 .31938 .60853 .31313 Metetra
.343 .533 .896 r
.60853 .31313 .61261 .31938 .62076 .32152 .6167 .31546 Metetra
0 .014 .569 r
.6167 .31546 .62076 .32152 .62925 .33212 .62516 .32531 Metetra
.946 .593 .204 r
.62516 .32531 .62925 .33212 .63943 .38768 .635 .37274 Metetra
.881 .562 .055 r
.635 .37274 .63943 .38768 .6505 .45784 .64525 .42352 Metetra
.864 .607 .549 r
.64525 .42352 .6505 .45784 .65574 .37571 .65105 .35444 Metetra
.896 .645 .551 r
.65105 .35444 .65574 .37571 .661 .30305 .65703 .2974 Metetra
.879 .773 .741 r
.65703 .2974 .661 .30305 .66869 .29033 .66477 .28515 Metetra
.75 .769 .868 r
.66477 .28515 .66869 .29033 .67673 .28533 .67282 .28017 Metetra
.698 .754 .895 r
.67282 .28017 .67673 .28533 .68485 .28176 .68096 .27659 Metetra
.679 .747 .902 r
.68096 .27659 .68485 .28176 .69301 .27862 .68914 .27343 Metetra
.672 .744 .905 r
.68914 .27343 .69301 .27862 .70121 .27564 .69736 .27044 Metetra
.668 .743 .906 r
.69736 .27044 .70121 .27564 .70943 .27273 .7056 .26751 Metetra
.666 .742 .907 r
.7056 .26751 .70943 .27273 .71768 .26985 .71386 .26461 Metetra
.665 .742 .907 r
.71386 .26461 .71768 .26985 .72595 .26698 .72215 .26173 Metetra
.665 .742 .908 r
.72215 .26173 .72595 .26698 .73425 .26412 .73047 .25885 Metetra
.664 .742 .908 r
.73047 .25885 .73425 .26412 .74258 .26125 .73881 .25597 Metetra
.664 .742 .908 r
.73881 .25597 .74258 .26125 .75092 .25839 .74717 .25309 Metetra
.664 .741 .908 r
.74717 .25309 .75092 .25839 .7593 .25551 .75556 .2502 Metetra
.664 .741 .908 r
.75556 .2502 .7593 .25551 .76769 .25263 .76398 .24731 Metetra
.664 .741 .908 r
.76398 .24731 .76769 .25263 .77612 .24975 .77242 .2444 Metetra
.664 .741 .908 r
.77242 .2444 .77612 .24975 .78456 .24685 .78088 .24149 Metetra
.664 .741 .908 r
.78088 .24149 .78456 .24685 .79303 .24395 .78937 .23858 Metetra
.664 .741 .908 r
.78937 .23858 .79303 .24395 .80153 .24104 .79788 .23565 Metetra
.664 .741 .908 r
.79788 .23565 .80153 .24104 .81005 .23812 .80642 .23272 Metetra
.664 .741 .908 r
.80642 .23272 .81005 .23812 .8186 .2352 .81499 .22978 Metetra
.664 .741 .908 r
.81499 .22978 .8186 .2352 .82717 .23226 .82358 .22683 Metetra
.664 .741 .908 r
.82358 .22683 .82717 .23226 .83577 .22932 .8322 .22387 Metetra
.735 .742 .857 r
.21702 .43314 .22171 .43786 .22882 .4335 .22412 .42899 Metetra
.69 .743 .892 r
.22412 .42899 .22882 .4335 .23582 .43053 .23112 .42606 Metetra
.674 .741 .901 r
.23112 .42606 .23582 .43053 .2428 .42792 .23811 .42346 Metetra
.667 .74 .905 r
.23811 .42346 .2428 .42792 .24978 .42545 .24511 .42097 Metetra
.664 .74 .907 r
.24511 .42097 .24978 .42545 .25678 .42303 .25212 .41855 Metetra
.661 .74 .908 r
.25212 .41855 .25678 .42303 .2638 .42066 .25914 .41617 Metetra
.659 .739 .909 r
.25914 .41617 .2638 .42066 .27083 .41832 .26619 .41382 Metetra
.657 .739 .91 r
.26619 .41382 .27083 .41832 .27787 .41601 .27324 .4115 Metetra
.655 .738 .911 r
.27324 .4115 .27787 .41601 .28493 .41374 .28032 .40921 Metetra
.652 .737 .912 r
.28032 .40921 .28493 .41374 .29201 .41152 .28741 .40698 Metetra
.647 .736 .914 r
.28741 .40698 .29201 .41152 .2991 .40938 .29451 .40482 Metetra
.64 .733 .916 r
.29451 .40482 .2991 .40938 .30621 .40736 .30162 .40278 Metetra
.626 .729 .921 r
.30162 .40278 .30621 .40736 .31332 .40557 .30875 .40097 Metetra
.597 .719 .929 r
.30875 .40097 .31332 .40557 .32042 .40424 .31586 .3996 Metetra
.507 .683 .945 r
.31586 .3996 .32042 .40424 .32747 .40409 .32293 .39936 Metetra
0 .344 .814 r
.32293 .39936 .32747 .40409 .33428 .4087 .32977 .40334 Metetra
.748 .283 0 r
.32977 .40334 .33428 .4087 .34043 .42561 .33626 .41404 Metetra
.83 .65 .659 r
.33626 .41404 .34043 .42561 .34902 .39986 .34451 .39486 Metetra
.801 .753 .808 r
.34451 .39486 .34902 .39986 .35656 .3926 .35205 .38787 Metetra
.737 .755 .867 r
.35205 .38787 .35656 .3926 .36394 .38827 .35945 .38358 Metetra
.707 .75 .886 r
.35945 .38358 .36394 .38827 .3713 .3848 .36683 .38012 Metetra
.691 .747 .894 r
.36683 .38012 .3713 .3848 .37866 .38169 .3742 .377 Metetra
.683 .746 .899 r
.3742 .377 .37866 .38169 .38603 .37876 .38157 .37407 Metetra
.678 .745 .901 r
.38157 .37407 .38603 .37876 .39341 .37594 .38897 .37124 Metetra
.675 .744 .903 r
.38897 .37124 .39341 .37594 .4008 .37317 .39638 .36846 Metetra
.672 .744 .904 r
.39638 .36846 .4008 .37317 .40822 .37045 .4038 .36573 Metetra
.671 .743 .905 r
.4038 .36573 .40822 .37045 .41565 .36775 .41125 .36302 Metetra
.669 .743 .906 r
.41125 .36302 .41565 .36775 .4231 .36509 .41871 .36035 Metetra
.663 .742 .908 r
.41871 .36035 .4231 .36509 .43057 .36251 .42619 .35778 Metetra
.645 .736 .916 r
.42619 .35778 .43057 .36251 .43805 .3603 .43369 .35555 Metetra
.537 .695 .941 r
.43369 .35555 .43805 .3603 .4455 .36009 .44116 .35493 Metetra
.672 .727 .891 r
.44116 .35493 .4455 .36009 .45304 .35717 .44871 .35213 Metetra
.72 .747 .874 r
.44871 .35213 .45304 .35717 .46063 .35306 .45631 .3482 Metetra
.682 .745 .898 r
.45631 .3482 .46063 .35306 .46821 .35005 .46391 .34519 Metetra
.67 .742 .904 r
.46391 .34519 .46821 .35005 .47581 .3473 .47152 .34243 Metetra
.663 .739 .907 r
.47152 .34243 .47581 .3473 .48343 .3447 .47915 .33981 Metetra
.656 .737 .909 r
.47915 .33981 .48343 .3447 .49106 .34224 .48679 .33733 Metetra
.645 .733 .913 r
.48679 .33733 .49106 .34224 .49872 .33997 .49446 .33504 Metetra
.628 .727 .918 r
.49446 .33504 .49872 .33997 .50639 .33801 .50215 .33305 Metetra
.592 .712 .927 r
.50215 .33305 .50639 .33801 .51409 .33668 .50987 .33166 Metetra
.475 .658 .941 r
.50987 .33166 .51409 .33668 .52182 .33701 .5176 .33185 Metetra
0 .146 .669 r
.5176 .33185 .52182 .33701 .52959 .34504 .52537 .33866 Metetra
0 0 .535 r
.52537 .33866 .52959 .34504 .53746 .35901 .53322 .34842 Metetra
.845 .671 .666 r
.53322 .34842 .53746 .35901 .54521 .3362 .54102 .33063 Metetra
.789 .753 .82 r
.54102 .33063 .54521 .3362 .55301 .3293 .54884 .32402 Metetra
.72 .747 .873 r
.54884 .32402 .55301 .3293 .56085 .32516 .5567 .31994 Metetra
.682 .738 .893 r
.5567 .31994 .56085 .32516 .56871 .32204 .56458 .31681 Metetra
.655 .729 .903 r
.56458 .31681 .56871 .32204 .57661 .31954 .57249 .31425 Metetra
.623 .714 .911 r
.57249 .31425 .57661 .31954 .58455 .31777 .58043 .3123 Metetra
.55 .676 .921 r
.58043 .3123 .58455 .31777 .59255 .31804 .58843 .3116 Metetra
.654 .683 .865 r
.58843 .3116 .59255 .31804 .60061 .31897 .59642 .30896 Metetra
.616 .574 .784 r
.59642 .30896 .60061 .31897 .60853 .31313 .60444 .30693 Metetra
.326 .526 .897 r
.60444 .30693 .60853 .31313 .6167 .31546 .61262 .3094 Metetra
0 .067 .617 r
.61262 .3094 .6167 .31546 .62516 .32531 .62105 .31845 Metetra
.932 .568 .158 r
.62105 .31845 .62516 .32531 .635 .37274 .63058 .35807 Metetra
.754 .449 0 r
.63058 .35807 .635 .37274 .64525 .42352 .64022 .39387 Metetra
.859 .609 .56 r
.64022 .39387 .64525 .42352 .65105 .35444 .64654 .33734 Metetra
.898 .657 .566 r
.64654 .33734 .65105 .35444 .65703 .2974 .65304 .29162 Metetra
.871 .77 .747 r
.65304 .29162 .65703 .2974 .66477 .28515 .66082 .27992 Metetra
.749 .768 .868 r
.66082 .27992 .66477 .28515 .67282 .28017 .6689 .27497 Metetra
.698 .753 .895 r
.6689 .27497 .67282 .28017 .68096 .27659 .67706 .27138 Metetra
.679 .747 .902 r
.67706 .27138 .68096 .27659 .68914 .27343 .68525 .26821 Metetra
.672 .744 .905 r
.68525 .26821 .68914 .27343 .69736 .27044 .69348 .2652 Metetra
.668 .743 .906 r
.69348 .2652 .69736 .27044 .7056 .26751 .70174 .26226 Metetra
.666 .742 .907 r
.70174 .26226 .7056 .26751 .71386 .26461 .71002 .25935 Metetra
.665 .742 .907 r
.71002 .25935 .71386 .26461 .72215 .26173 .71833 .25645 Metetra
.665 .742 .908 r
.71833 .25645 .72215 .26173 .73047 .25885 .72666 .25356 Metetra
.664 .742 .908 r
.72666 .25356 .73047 .25885 .73881 .25597 .73502 .25066 Metetra
.664 .742 .908 r
.73502 .25066 .73881 .25597 .74717 .25309 .7434 .24776 Metetra
.664 .741 .908 r
.7434 .24776 .74717 .25309 .75556 .2502 .75181 .24486 Metetra
.664 .741 .908 r
.75181 .24486 .75556 .2502 .76398 .24731 .76024 .24195 Metetra
.664 .741 .908 r
.76024 .24195 .76398 .24731 .77242 .2444 .7687 .23903 Metetra
.664 .741 .908 r
.7687 .23903 .77242 .2444 .78088 .24149 .77718 .23611 Metetra
.664 .741 .908 r
.77718 .23611 .78088 .24149 .78937 .23858 .78569 .23317 Metetra
.664 .741 .908 r
.78569 .23317 .78937 .23858 .79788 .23565 .79422 .23023 Metetra
.664 .741 .908 r
.79422 .23023 .79788 .23565 .80642 .23272 .80278 .22728 Metetra
.664 .741 .908 r
.80278 .22728 .80642 .23272 .81499 .22978 .81136 .22432 Metetra
.664 .741 .908 r
.81136 .22432 .81499 .22978 .82358 .22683 .81997 .22136 Metetra
.664 .741 .908 r
.81997 .22136 .82358 .22683 .8322 .22387 .8286 .21838 Metetra
.727 .741 .862 r
.2123 .4284 .21702 .43314 .22412 .42899 .2194 .42446 Metetra
.687 .742 .893 r
.2194 .42446 .22412 .42899 .23112 .42606 .22641 .42156 Metetra
.673 .741 .902 r
.22641 .42156 .23112 .42606 .23811 .42346 .23341 .41896 Metetra
.667 .74 .905 r
.23341 .41896 .23811 .42346 .24511 .42097 .24041 .41648 Metetra
.664 .74 .907 r
.24041 .41648 .24511 .42097 .25212 .41855 .24743 .41405 Metetra
.661 .74 .908 r
.24743 .41405 .25212 .41855 .25914 .41617 .25447 .41166 Metetra
.659 .739 .909 r
.25447 .41166 .25914 .41617 .26619 .41382 .26152 .4093 Metetra
.657 .739 .91 r
.26152 .4093 .26619 .41382 .27324 .4115 .26859 .40696 Metetra
.655 .738 .911 r
.26859 .40696 .27324 .4115 .28032 .40921 .27568 .40466 Metetra
.652 .737 .912 r
.27568 .40466 .28032 .40921 .28741 .40698 .28278 .40242 Metetra
.648 .736 .913 r
.28278 .40242 .28741 .40698 .29451 .40482 .28989 .40024 Metetra
.64 .733 .916 r
.28989 .40024 .29451 .40482 .30162 .40278 .29702 .39819 Metetra
.627 .729 .92 r
.29702 .39819 .30162 .40278 .30875 .40097 .30415 .39635 Metetra
.599 .719 .929 r
.30415 .39635 .30875 .40097 .31586 .3996 .31128 .39494 Metetra
.515 .686 .944 r
.31128 .39494 .31586 .3996 .32293 .39936 .31836 .39459 Metetra
.041 .42 .865 r
.31836 .39459 .32293 .39936 .32977 .40334 .32524 .39792 Metetra
0 .051 .582 r
.32524 .39792 .32977 .40334 .33626 .41404 .33196 .40468 Metetra
.827 .669 .688 r
.33196 .40468 .33626 .41404 .34451 .39486 .33999 .38982 Metetra
.795 .751 .813 r
.33999 .38982 .34451 .39486 .35205 .38787 .34753 .38311 Metetra
.735 .754 .868 r
.34753 .38311 .35205 .38787 .35945 .38358 .35494 .37886 Metetra
.706 .75 .886 r
.35494 .37886 .35945 .38358 .36683 .38012 .36233 .37541 Metetra
.691 .747 .895 r
.36233 .37541 .36683 .38012 .3742 .377 .36971 .37229 Metetra
.683 .746 .899 r
.36971 .37229 .3742 .377 .38157 .37407 .3771 .36935 Metetra
.678 .745 .901 r
.3771 .36935 .38157 .37407 .38897 .37124 .38451 .36651 Metetra
.675 .744 .903 r
.38451 .36651 .38897 .37124 .39638 .36846 .39193 .36373 Metetra
.672 .743 .904 r
.39193 .36373 .39638 .36846 .4038 .36573 .39937 .36099 Metetra
.67 .743 .905 r
.39937 .36099 .4038 .36573 .41125 .36302 .40683 .35827 Metetra
.668 .743 .906 r
.40683 .35827 .41125 .36302 .41871 .36035 .4143 .3556 Metetra
.663 .741 .908 r
.4143 .3556 .41871 .36035 .42619 .35778 .4218 .35302 Metetra
.645 .736 .915 r
.4218 .35302 .42619 .35778 .43369 .35555 .4293 .35077 Metetra
.563 .706 .937 r
.4293 .35077 .43369 .35555 .44116 .35493 .43679 .34979 Metetra
.668 .728 .895 r
.43679 .34979 .44116 .35493 .44871 .35213 .44436 .34708 Metetra
.713 .746 .878 r
.44436 .34708 .44871 .35213 .45631 .3482 .45197 .34332 Metetra
.682 .745 .899 r
.45197 .34332 .45631 .3482 .46391 .34519 .45958 .34031 Metetra
.67 .742 .904 r
.45958 .34031 .46391 .34519 .47152 .34243 .4672 .33754 Metetra
.663 .739 .907 r
.4672 .33754 .47152 .34243 .47915 .33981 .47484 .3349 Metetra
.656 .737 .909 r
.47484 .3349 .47915 .33981 .48679 .33733 .48251 .3324 Metetra
.646 .733 .913 r
.48251 .3324 .48679 .33733 .49446 .33504 .49019 .33009 Metetra
.63 .727 .918 r
.49019 .33009 .49446 .33504 .50215 .33305 .49789 .32806 Metetra
.595 .713 .926 r
.49789 .32806 .50215 .33305 .50987 .33166 .50562 .32661 Metetra
.487 .664 .94 r
.50562 .32661 .50987 .33166 .5176 .33185 .51336 .32664 Metetra
0 .257 .759 r
.51336 .32664 .5176 .33185 .52537 .33866 .52114 .33229 Metetra
0 .212 .737 r
.52114 .33229 .52537 .33866 .53322 .34842 .52898 .33903 Metetra
.832 .68 .695 r
.52898 .33903 .53322 .34842 .54102 .33063 .53681 .32501 Metetra
.78 .75 .825 r
.53681 .32501 .54102 .33063 .54884 .32402 .54466 .31872 Metetra
.717 .746 .875 r
.54466 .31872 .54884 .32402 .5567 .31994 .55253 .3147 Metetra
.682 .738 .893 r
.55253 .3147 .5567 .31994 .56458 .31681 .56042 .31155 Metetra
.657 .729 .903 r
.56042 .31155 .56458 .31681 .57249 .31425 .56835 .30894 Metetra
.632 .718 .909 r
.56835 .30894 .57249 .31425 .58043 .3123 .5763 .30682 Metetra
.6 .699 .912 r
.5763 .30682 .58043 .3123 .58843 .3116 .58429 .30527 Metetra
.733 .718 .837 r
.58429 .30527 .58843 .3116 .59642 .30896 .59225 .30042 Metetra
.52 .556 .832 r
.59225 .30042 .59642 .30896 .60444 .30693 .60033 .30079 Metetra
.313 .522 .898 r
.60033 .30079 .60444 .30693 .61262 .3094 .60852 .30335 Metetra
0 .124 .666 r
.60852 .30335 .61262 .3094 .62105 .31845 .61693 .31159 Metetra
.908 .532 .094 r
.61693 .31159 .62105 .31845 .63058 .35807 .62619 .34416 Metetra
0 0 .332 r
.62619 .34416 .63058 .35807 .64022 .39387 .6354 .36903 Metetra
.857 .615 .572 r
.6354 .36903 .64022 .39387 .64654 .33734 .64214 .32312 Metetra
.898 .669 .583 r
.64214 .32312 .64654 .33734 .65304 .29162 .64902 .28576 Metetra
.863 .768 .755 r
.64902 .28576 .65304 .29162 .66082 .27992 .65686 .27466 Metetra
.747 .767 .869 r
.65686 .27466 .66082 .27992 .6689 .27497 .66496 .26974 Metetra
.698 .753 .895 r
.66496 .26974 .6689 .27497 .67706 .27138 .67313 .26615 Metetra
.679 .747 .902 r
.67313 .26615 .67706 .27138 .68525 .26821 .68134 .26297 Metetra
.672 .744 .905 r
.68134 .26297 .68525 .26821 .69348 .2652 .68959 .25994 Metetra
.668 .743 .906 r
.68959 .25994 .69348 .2652 .70174 .26226 .69786 .25699 Metetra
.666 .742 .907 r
.69786 .25699 .70174 .26226 .71002 .25935 .70616 .25406 Metetra
.665 .742 .907 r
.70616 .25406 .71002 .25935 .71833 .25645 .71449 .25115 Metetra
.665 .742 .908 r
.71449 .25115 .71833 .25645 .72666 .25356 .72284 .24824 Metetra
.664 .742 .908 r
.72284 .24824 .72666 .25356 .73502 .25066 .73121 .24532 Metetra
.664 .742 .908 r
.73121 .24532 .73502 .25066 .7434 .24776 .73961 .24241 Metetra
.664 .741 .908 r
.73961 .24241 .7434 .24776 .75181 .24486 .74803 .23949 Metetra
.664 .741 .908 r
.74803 .23949 .75181 .24486 .76024 .24195 .75648 .23656 Metetra
.664 .741 .908 r
.75648 .23656 .76024 .24195 .7687 .23903 .76496 .23363 Metetra
.664 .741 .908 r
.76496 .23363 .7687 .23903 .77718 .23611 .77346 .23069 Metetra
.664 .741 .908 r
.77346 .23069 .77718 .23611 .78569 .23317 .78198 .22774 Metetra
.664 .741 .908 r
.78198 .22774 .78569 .23317 .79422 .23023 .79053 .22478 Metetra
.664 .741 .908 r
.79053 .22478 .79422 .23023 .80278 .22728 .79911 .22182 Metetra
.664 .741 .908 r
.79911 .22182 .80278 .22728 .81136 .22432 .80771 .21884 Metetra
.664 .741 .908 r
.80771 .21884 .81136 .22432 .81997 .22136 .81634 .21586 Metetra
.664 .741 .908 r
.81634 .21586 .81997 .22136 .8286 .21838 .82499 .21287 Metetra
.72 .74 .867 r
.20757 .42364 .2123 .4284 .2194 .42446 .21465 .41991 Metetra
.685 .742 .894 r
.21465 .41991 .2194 .42446 .22641 .42156 .22167 .41704 Metetra
.672 .741 .902 r
.22167 .41704 .22641 .42156 .23341 .41896 .22868 .41445 Metetra
.666 .74 .905 r
.22868 .41445 .23341 .41896 .24041 .41648 .2357 .41196 Metetra
.663 .74 .907 r
.2357 .41196 .24041 .41648 .24743 .41405 .24273 .40953 Metetra
.661 .739 .908 r
.24273 .40953 .24743 .41405 .25447 .41166 .24977 .40713 Metetra
.659 .739 .909 r
.24977 .40713 .25447 .41166 .26152 .4093 .25684 .40475 Metetra
.658 .739 .91 r
.25684 .40475 .26152 .4093 .26859 .40696 .26392 .40241 Metetra
.655 .738 .911 r
.26392 .40241 .26859 .40696 .27568 .40466 .27101 .40009 Metetra
.652 .737 .912 r
.27101 .40009 .27568 .40466 .28278 .40242 .27812 .39783 Metetra
.648 .736 .913 r
.27812 .39783 .28278 .40242 .28989 .40024 .28525 .39564 Metetra
.641 .734 .916 r
.28525 .39564 .28989 .40024 .29702 .39819 .29239 .39357 Metetra
.628 .729 .92 r
.29239 .39357 .29702 .39819 .30415 .39635 .29953 .3917 Metetra
.601 .72 .928 r
.29953 .3917 .30415 .39635 .31128 .39494 .30667 .39025 Metetra
.524 .689 .943 r
.30667 .39025 .31128 .39494 .31836 .39459 .31377 .3898 Metetra
.146 .486 .901 r
.31377 .3898 .31836 .39459 .32524 .39792 .32069 .39248 Metetra
0 .348 .824 r
.32069 .39248 .32524 .39792 .33196 .40468 .32757 .39664 Metetra
.819 .685 .716 r
.32757 .39664 .33196 .40468 .33999 .38982 .33544 .38473 Metetra
.788 .75 .818 r
.33544 .38473 .33999 .38982 .34753 .38311 .34299 .37832 Metetra
.733 .753 .869 r
.34299 .37832 .34753 .38311 .35494 .37886 .35041 .37412 Metetra
.705 .75 .887 r
.35041 .37412 .35494 .37886 .36233 .37541 .3578 .37068 Metetra
.69 .747 .895 r
.3578 .37068 .36233 .37541 .36971 .37229 .3652 .36756 Metetra
.682 .746 .899 r
.3652 .36756 .36971 .37229 .3771 .36935 .3726 .36461 Metetra
.677 .744 .901 r
.3726 .36461 .3771 .36935 .38451 .36651 .38002 .36176 Metetra
.674 .744 .903 r
.38002 .36176 .38451 .36651 .39193 .36373 .38746 .35897 Metetra
.672 .743 .904 r
.38746 .35897 .39193 .36373 .39937 .36099 .39491 .35622 Metetra
.67 .743 .905 r
.39491 .35622 .39937 .36099 .40683 .35827 .40238 .3535 Metetra
.668 .743 .906 r
.40238 .3535 .40683 .35827 .4143 .3556 .40987 .35082 Metetra
.663 .741 .909 r
.40987 .35082 .4143 .3556 .4218 .35302 .41738 .34823 Metetra
.646 .736 .915 r
.41738 .34823 .4218 .35302 .4293 .35077 .4249 .34596 Metetra
.584 .714 .932 r
.4249 .34596 .4293 .35077 .43679 .34979 .43241 .34469 Metetra
.665 .73 .898 r
.43241 .34469 .43679 .34979 .44436 .34708 .43998 .34202 Metetra
.706 .745 .881 r
.43998 .34202 .44436 .34708 .45197 .34332 .4476 .33841 Metetra
.681 .744 .899 r
.4476 .33841 .45197 .34332 .45958 .34031 .45523 .3354 Metetra
.67 .742 .904 r
.45523 .3354 .45958 .34031 .4672 .33754 .46286 .33261 Metetra
.663 .74 .907 r
.46286 .33261 .4672 .33754 .47484 .3349 .47052 .32996 Metetra
.656 .737 .909 r
.47052 .32996 .47484 .3349 .48251 .3324 .4782 .32744 Metetra
.646 .733 .912 r
.4782 .32744 .48251 .3324 .49019 .33009 .48589 .32511 Metetra
.631 .727 .917 r
.48589 .32511 .49019 .33009 .49789 .32806 .49361 .32305 Metetra
.598 .714 .926 r
.49361 .32305 .49789 .32806 .50562 .32661 .50135 .32154 Metetra
.499 .669 .939 r
.50135 .32154 .50562 .32661 .51336 .32664 .5091 .32141 Metetra
0 .358 .83 r
.5091 .32141 .51336 .32664 .52114 .33229 .51688 .32598 Metetra
.096 .38 .841 r
.51688 .32598 .52114 .33229 .52898 .33903 .52472 .33058 Metetra
.817 .69 .725 r
.52472 .33058 .52898 .33903 .53681 .32501 .53258 .31935 Metetra
.772 .748 .831 r
.53258 .31935 .53681 .32501 .54466 .31872 .54045 .31339 Metetra
.715 .745 .876 r
.54045 .31339 .54466 .31872 .55253 .3147 .54833 .30942 Metetra
.682 .738 .893 r
.54833 .30942 .55253 .3147 .56042 .31155 .55625 .30627 Metetra
.659 .73 .902 r
.55625 .30627 .56042 .31155 .56835 .30894 .56418 .3036 Metetra
.639 .721 .907 r
.56418 .3036 .56835 .30894 .5763 .30682 .57215 .30133 Metetra
.637 .714 .903 r
.57215 .30133 .5763 .30682 .58429 .30527 .58014 .29909 Metetra
.772 .735 .819 r
.58014 .29909 .58429 .30527 .59225 .30042 .58808 .29283 Metetra
.416 .527 .862 r
.58808 .29283 .59225 .30042 .60033 .30079 .59621 .29473 Metetra
.304 .522 .9 r
.59621 .29473 .60033 .30079 .60852 .30335 .6044 .2973 Metetra
0 .183 .713 r
.6044 .2973 .60852 .30335 .61693 .31159 .61279 .30473 Metetra
.87 .479 .009 r
.61279 .30473 .61693 .31159 .62619 .34416 .62183 .33121 Metetra
0 0 .526 r
.62183 .33121 .62619 .34416 .6354 .36903 .63074 .34827 Metetra
.856 .623 .586 r
.63074 .34827 .6354 .36903 .64214 .32312 .63782 .31089 Metetra
.897 .68 .601 r
.63782 .31089 .64214 .32312 .64902 .28576 .64498 .27985 Metetra
.855 .767 .764 r
.64498 .27985 .64902 .28576 .65686 .27466 .65287 .26936 Metetra
.745 .766 .87 r
.65287 .26936 .65686 .27466 .66496 .26974 .66099 .26448 Metetra
.697 .753 .895 r
.66099 .26448 .66496 .26974 .67313 .26615 .66918 .26089 Metetra
.679 .747 .902 r
.66918 .26089 .67313 .26615 .68134 .26297 .67741 .25769 Metetra
.672 .744 .905 r
.67741 .25769 .68134 .26297 .68959 .25994 .68568 .25465 Metetra
.668 .743 .906 r
.68568 .25465 .68959 .25994 .69786 .25699 .69397 .25168 Metetra
.666 .742 .907 r
.69397 .25168 .69786 .25699 .70616 .25406 .70228 .24874 Metetra
.665 .742 .907 r
.70228 .24874 .70616 .25406 .71449 .25115 .71062 .24581 Metetra
.665 .742 .908 r
.71062 .24581 .71449 .25115 .72284 .24824 .71899 .24288 Metetra
.664 .742 .908 r
.71899 .24288 .72284 .24824 .73121 .24532 .72738 .23996 Metetra
.664 .742 .908 r
.72738 .23996 .73121 .24532 .73961 .24241 .7358 .23703 Metetra
.664 .741 .908 r
.7358 .23703 .73961 .24241 .74803 .23949 .74424 .23409 Metetra
.664 .741 .908 r
.74424 .23409 .74803 .23949 .75648 .23656 .75271 .23115 Metetra
.664 .741 .908 r
.75271 .23115 .75648 .23656 .76496 .23363 .7612 .2282 Metetra
.664 .741 .908 r
.7612 .2282 .76496 .23363 .77346 .23069 .76972 .22524 Metetra
.664 .741 .908 r
.76972 .22524 .77346 .23069 .78198 .22774 .77826 .22228 Metetra
.664 .741 .908 r
.77826 .22228 .78198 .22774 .79053 .22478 .78683 .2193 Metetra
.664 .741 .908 r
.78683 .2193 .79053 .22478 .79911 .22182 .79542 .21632 Metetra
.664 .741 .908 r
.79542 .21632 .79911 .22182 .80771 .21884 .80404 .21333 Metetra
.664 .741 .908 r
.80404 .21333 .80771 .21884 .81634 .21586 .81268 .21033 Metetra
.664 .741 .908 r
.81268 .21033 .81634 .21586 .82499 .21287 .82136 .20732 Metetra
.712 .739 .872 r
.2028 .41887 .20757 .42364 .21465 .41991 .20988 .41533 Metetra
.683 .741 .895 r
.20988 .41533 .21465 .41991 .22167 .41704 .21691 .4125 Metetra
.671 .741 .902 r
.21691 .4125 .22167 .41704 .22868 .41445 .22393 .40992 Metetra
.666 .74 .905 r
.22393 .40992 .22868 .41445 .2357 .41196 .23095 .40743 Metetra
.663 .74 .907 r
.23095 .40743 .2357 .41196 .24273 .40953 .23799 .40498 Metetra
.661 .739 .908 r
.23799 .40498 .24273 .40953 .24977 .40713 .24505 .40257 Metetra
.659 .739 .909 r
.24505 .40257 .24977 .40713 .25684 .40475 .25213 .40019 Metetra
.658 .739 .91 r
.25213 .40019 .25684 .40475 .26392 .40241 .25922 .39783 Metetra
.655 .738 .911 r
.25922 .39783 .26392 .40241 .27101 .40009 .26632 .3955 Metetra
.652 .737 .912 r
.26632 .3955 .27101 .40009 .27812 .39783 .27345 .39322 Metetra
.648 .736 .913 r
.27345 .39322 .27812 .39783 .28525 .39564 .28058 .39101 Metetra
.641 .734 .916 r
.28058 .39101 .28525 .39564 .29239 .39357 .28773 .38892 Metetra
.629 .73 .92 r
.28773 .38892 .29239 .39357 .29953 .3917 .29489 .38703 Metetra
.603 .72 .927 r
.29489 .38703 .29953 .3917 .30667 .39025 .30204 .38553 Metetra
.533 .693 .941 r
.30204 .38553 .30667 .39025 .31377 .3898 .30915 .38497 Metetra
.237 .54 .925 r
.30915 .38497 .31377 .3898 .32069 .39248 .31612 .38704 Metetra
.243 .513 .908 r
.31612 .38704 .32069 .39248 .32757 .39664 .32311 .38942 Metetra
.809 .698 .742 r
.32311 .38942 .32757 .39664 .33544 .38473 .33088 .37961 Metetra
.781 .749 .824 r
.33088 .37961 .33544 .38473 .34299 .37832 .33842 .3735 Metetra
.731 .753 .87 r
.33842 .3735 .34299 .37832 .35041 .37412 .34585 .36935 Metetra
.704 .749 .887 r
.34585 .36935 .35041 .37412 .3578 .37068 .35326 .36592 Metetra
.69 .747 .895 r
.35326 .36592 .3578 .37068 .3652 .36756 .36067 .3628 Metetra
.682 .745 .899 r
.36067 .3628 .3652 .36756 .3726 .36461 .36809 .35985 Metetra
.677 .744 .902 r
.36809 .35985 .3726 .36461 .38002 .36176 .37552 .35699 Metetra
.674 .744 .903 r
.37552 .35699 .38002 .36176 .38746 .35897 .38297 .35419 Metetra
.672 .743 .904 r
.38297 .35419 .38746 .35897 .39491 .35622 .39043 .35143 Metetra
.67 .743 .905 r
.39043 .35143 .39491 .35622 .40238 .3535 .39792 .3487 Metetra
.667 .742 .906 r
.39792 .3487 .40238 .3535 .40987 .35082 .40542 .34601 Metetra
.662 .741 .909 r
.40542 .34601 .40987 .35082 .41738 .34823 .41294 .34342 Metetra
.647 .736 .915 r
.41294 .34342 .41738 .34823 .4249 .34596 .42047 .34111 Metetra
.599 .719 .928 r
.42047 .34111 .4249 .34596 .43241 .34469 .428 .3396 Metetra
.663 .731 .9 r
.428 .3396 .43241 .34469 .43998 .34202 .43559 .33695 Metetra
.701 .744 .885 r
.43559 .33695 .43998 .34202 .4476 .33841 .44322 .33347 Metetra
.68 .744 .899 r
.44322 .33347 .4476 .33841 .45523 .3354 .45085 .33047 Metetra
.67 .742 .904 r
.45085 .33047 .45523 .3354 .46286 .33261 .4585 .32767 Metetra
.664 .74 .907 r
.4585 .32767 .46286 .33261 .47052 .32996 .46617 .325 Metetra
.656 .737 .909 r
.46617 .325 .47052 .32996 .4782 .32744 .47386 .32246 Metetra
.647 .734 .912 r
.47386 .32246 .4782 .32744 .48589 .32511 .48157 .3201 Metetra
.632 .728 .917 r
.48157 .3201 .48589 .32511 .49361 .32305 .4893 .31801 Metetra
.601 .715 .925 r
.4893 .31801 .49361 .32305 .50135 .32154 .49705 .31644 Metetra
.511 .675 .938 r
.49705 .31644 .50135 .32154 .5091 .32141 .50482 .31615 Metetra
.112 .441 .878 r
.50482 .31615 .5091 .32141 .51688 .32598 .51261 .31976 Metetra
.253 .484 .886 r
.51261 .31976 .51688 .32598 .52472 .33058 .52045 .32281 Metetra
.8 .699 .753 r
.52045 .32281 .52472 .33058 .53258 .31935 .52833 .31367 Metetra
.763 .746 .837 r
.52833 .31367 .53258 .31935 .54045 .31339 .53622 .30802 Metetra
.712 .744 .877 r
.53622 .30802 .54045 .31339 .54833 .30942 .54412 .30412 Metetra
.681 .738 .893 r
.54412 .30412 .54833 .30942 .55625 .30627 .55205 .30097 Metetra
.661 .731 .902 r
.55205 .30097 .55625 .30627 .56418 .3036 .56 .29825 Metetra
.646 .724 .905 r
.56 .29825 .56418 .3036 .57215 .30133 .56798 .29583 Metetra
.663 .725 .895 r
.56798 .29583 .57215 .30133 .58014 .29909 .57598 .29305 Metetra
.794 .746 .808 r
.57598 .29305 .58014 .29909 .58808 .29283 .58391 .28588 Metetra
.319 .494 .874 r
.58391 .28588 .58808 .29283 .59621 .29473 .59207 .28875 Metetra
.302 .525 .903 r
.59207 .28875 .59621 .29473 .6044 .2973 .60027 .29127 Metetra
0 .243 .757 r
.60027 .29127 .6044 .2973 .61279 .30473 .60863 .29789 Metetra
.81 .405 0 r
.60863 .29789 .61279 .30473 .62183 .33121 .6175 .31923 Metetra
0 .139 .663 r
.6175 .31923 .62183 .33121 .63074 .34827 .6262 .33073 Metetra
.854 .633 .603 r
.6262 .33073 .63074 .34827 .63782 .31089 .63354 .30006 Metetra
.894 .69 .62 r
.63354 .30006 .63782 .31089 .64498 .27985 .64093 .2739 Metetra
.846 .766 .773 r
.64093 .2739 .64498 .27985 .65287 .26936 .64886 .26402 Metetra
.743 .765 .871 r
.64886 .26402 .65287 .26936 .66099 .26448 .657 .2592 Metetra
.696 .753 .895 r
.657 .2592 .66099 .26448 .66918 .26089 .66521 .25559 Metetra
.679 .747 .902 r
.66521 .25559 .66918 .26089 .67741 .25769 .67346 .25239 Metetra
.671 .744 .905 r
.67346 .25239 .67741 .25769 .68568 .25465 .68174 .24933 Metetra
.668 .743 .906 r
.68174 .24933 .68568 .25465 .69397 .25168 .69005 .24635 Metetra
.666 .742 .907 r
.69005 .24635 .69397 .25168 .70228 .24874 .69838 .24339 Metetra
.665 .742 .907 r
.69838 .24339 .70228 .24874 .71062 .24581 .70674 .24044 Metetra
.665 .742 .908 r
.70674 .24044 .71062 .24581 .71899 .24288 .71512 .2375 Metetra
.664 .742 .908 r
.71512 .2375 .71899 .24288 .72738 .23996 .72353 .23456 Metetra
.664 .742 .908 r
.72353 .23456 .72738 .23996 .7358 .23703 .73196 .23161 Metetra
.664 .741 .908 r
.73196 .23161 .7358 .23703 .74424 .23409 .74042 .22866 Metetra
.664 .741 .908 r
.74042 .22866 .74424 .23409 .75271 .23115 .74891 .2257 Metetra
.664 .741 .908 r
.74891 .2257 .75271 .23115 .7612 .2282 .75742 .22274 Metetra
.664 .741 .908 r
.75742 .22274 .7612 .2282 .76972 .22524 .76595 .21976 Metetra
.664 .741 .908 r
.76595 .21976 .76972 .22524 .77826 .22228 .77451 .21678 Metetra
.664 .741 .908 r
.77451 .21678 .77826 .22228 .78683 .2193 .7831 .21379 Metetra
.664 .741 .908 r
.7831 .21379 .78683 .2193 .79542 .21632 .79171 .21079 Metetra
.664 .741 .908 r
.79171 .21079 .79542 .21632 .80404 .21333 .80035 .20779 Metetra
.664 .741 .908 r
.80035 .20779 .80404 .21333 .81268 .21033 .80901 .20477 Metetra
.664 .741 .908 r
.80901 .20477 .81268 .21033 .82136 .20732 .8177 .20175 Metetra
.705 .738 .877 r
.19802 .41411 .2028 .41887 .20988 .41533 .20509 .41073 Metetra
.681 .741 .896 r
.20509 .41073 .20988 .41533 .21691 .4125 .21212 .40794 Metetra
.67 .74 .903 r
.21212 .40794 .21691 .4125 .22393 .40992 .21915 .40536 Metetra
.665 .74 .906 r
.21915 .40536 .22393 .40992 .23095 .40743 .22619 .40287 Metetra
.663 .74 .907 r
.22619 .40287 .23095 .40743 .23799 .40498 .23324 .40041 Metetra
.661 .739 .908 r
.23324 .40041 .23799 .40498 .24505 .40257 .24031 .39799 Metetra
.659 .739 .909 r
.24031 .39799 .24505 .40257 .25213 .40019 .24739 .3956 Metetra
.658 .739 .91 r
.24739 .3956 .25213 .40019 .25922 .39783 .25449 .39322 Metetra
.655 .738 .911 r
.25449 .39322 .25922 .39783 .26632 .3955 .26161 .39088 Metetra
.653 .737 .912 r
.26161 .39088 .26632 .3955 .27345 .39322 .26875 .38859 Metetra
.648 .736 .913 r
.26875 .38859 .27345 .39322 .28058 .39101 .2759 .38637 Metetra
.642 .734 .916 r
.2759 .38637 .28058 .39101 .28773 .38892 .28306 .38425 Metetra
.63 .73 .919 r
.28306 .38425 .28773 .38892 .29489 .38703 .29023 .38233 Metetra
.605 .721 .927 r
.29023 .38233 .29489 .38703 .30204 .38553 .29739 .38079 Metetra
.542 .696 .94 r
.29739 .38079 .30204 .38553 .30915 .38497 .30452 .38011 Metetra
.314 .583 .938 r
.30452 .38011 .30915 .38497 .31612 .38704 .31152 .38162 Metetra
.389 .595 .927 r
.31152 .38162 .31612 .38704 .32311 .38942 .3186 .38276 Metetra
.797 .708 .766 r
.3186 .38276 .32311 .38942 .33088 .37961 .32629 .37447 Metetra
.774 .748 .829 r
.32629 .37447 .33088 .37961 .33842 .3735 .33383 .36865 Metetra
.728 .752 .871 r
.33383 .36865 .33842 .3735 .34585 .36935 .34127 .36456 Metetra
.703 .749 .888 r
.34127 .36456 .34585 .36935 .35326 .36592 .34869 .36114 Metetra
.689 .747 .895 r
.34869 .36114 .35326 .36592 .36067 .3628 .35611 .35802 Metetra
.682 .745 .899 r
.35611 .35802 .36067 .3628 .36809 .35985 .36354 .35506 Metetra
.677 .744 .902 r
.36354 .35506 .36809 .35985 .37552 .35699 .37099 .35219 Metetra
.674 .744 .903 r
.37099 .35219 .37552 .35699 .38297 .35419 .37845 .34938 Metetra
.672 .743 .904 r
.37845 .34938 .38297 .35419 .39043 .35143 .38593 .34661 Metetra
.67 .743 .905 r
.38593 .34661 .39043 .35143 .39792 .3487 .39342 .34387 Metetra
.667 .742 .906 r
.39342 .34387 .39792 .3487 .40542 .34601 .40094 .34118 Metetra
.662 .741 .909 r
.40094 .34118 .40542 .34601 .41294 .34342 .40847 .33858 Metetra
.648 .737 .914 r
.40847 .33858 .41294 .34342 .42047 .34111 .41602 .33624 Metetra
.611 .723 .925 r
.41602 .33624 .42047 .34111 .428 .3396 .42356 .33452 Metetra
.663 .733 .902 r
.42356 .33452 .428 .3396 .43559 .33695 .43116 .33188 Metetra
.696 .743 .888 r
.43116 .33188 .43559 .33695 .44322 .33347 .43881 .32851 Metetra
.679 .744 .9 r
.43881 .32851 .44322 .33347 .45085 .33047 .44646 .32551 Metetra
.67 .742 .904 r
.44646 .32551 .45085 .33047 .4585 .32767 .45412 .3227 Metetra
.664 .74 .907 r
.45412 .3227 .4585 .32767 .46617 .325 .4618 .32001 Metetra
.657 .737 .909 r
.4618 .32001 .46617 .325 .47386 .32246 .46951 .31746 Metetra
.647 .734 .912 r
.46951 .31746 .47386 .32246 .48157 .3201 .47723 .31507 Metetra
.633 .728 .917 r
.47723 .31507 .48157 .3201 .4893 .31801 .48497 .31295 Metetra
.603 .716 .924 r
.48497 .31295 .4893 .31801 .49705 .31644 .49274 .31132 Metetra
.523 .68 .937 r
.49274 .31132 .49705 .31644 .50482 .31615 .50052 .31087 Metetra
.214 .506 .908 r
.50052 .31087 .50482 .31615 .51261 .31976 .50832 .31363 Metetra
.355 .549 .905 r
.50832 .31363 .51261 .31976 .52045 .32281 .51617 .31556 Metetra
.783 .706 .779 r
.51617 .31556 .52045 .32281 .52833 .31367 .52406 .30797 Metetra
.755 .744 .843 r
.52406 .30797 .52833 .31367 .53622 .30802 .53197 .30263 Metetra
.709 .743 .879 r
.53197 .30263 .53622 .30802 .54412 .30412 .53988 .29879 Metetra
.681 .738 .894 r
.53988 .29879 .54412 .30412 .55205 .30097 .54783 .29564 Metetra
.662 .732 .901 r
.54783 .29564 .55205 .30097 .56 .29825 .55579 .29287 Metetra
.651 .726 .904 r
.55579 .29287 .56 .29825 .56798 .29583 .56378 .29033 Metetra
.68 .732 .889 r
.56378 .29033 .56798 .29583 .57598 .29305 .57179 .28714 Metetra
.809 .754 .802 r
.57179 .28714 .57598 .29305 .58391 .28588 .57974 .27936 Metetra
.238 .463 .875 r
.57974 .27936 .58391 .28588 .59207 .28875 .58791 .28286 Metetra
.306 .533 .908 r
.58791 .28286 .59207 .28875 .60027 .29127 .59611 .28526 Metetra
0 .302 .797 r
.59611 .28526 .60027 .29127 .60863 .29789 .60446 .29109 Metetra
.717 .302 0 r
.60446 .29109 .60863 .29789 .6175 .31923 .61319 .30817 Metetra
.173 .29 .744 r
.61319 .30817 .6175 .31923 .6262 .33073 .62175 .31568 Metetra
.852 .644 .621 r
.62175 .31568 .6262 .33073 .63354 .30006 .62929 .29027 Metetra
.889 .7 .641 r
.62929 .29027 .63354 .30006 .64093 .2739 .63685 .26795 Metetra
.837 .765 .782 r
.63685 .26795 .64093 .2739 .64886 .26402 .64483 .25865 Metetra
.74 .764 .872 r
.64483 .25865 .64886 .26402 .657 .2592 .65299 .25388 Metetra
.696 .752 .895 r
.65299 .25388 .657 .2592 .66521 .25559 .66122 .25027 Metetra
.679 .747 .902 r
.66122 .25027 .66521 .25559 .67346 .25239 .66949 .24705 Metetra
.671 .744 .905 r
.66949 .24705 .67346 .25239 .68174 .24933 .67778 .24399 Metetra
.668 .743 .906 r
.67778 .24399 .68174 .24933 .69005 .24635 .68611 .24098 Metetra
.666 .742 .907 r
.68611 .24098 .69005 .24635 .69838 .24339 .69446 .23801 Metetra
.665 .742 .907 r
.69446 .23801 .69838 .24339 .70674 .24044 .70283 .23505 Metetra
.665 .742 .908 r
.70283 .23505 .70674 .24044 .71512 .2375 .71123 .23209 Metetra
.664 .742 .908 r
.71123 .23209 .71512 .2375 .72353 .23456 .71966 .22913 Metetra
.664 .741 .908 r
.71966 .22913 .72353 .23456 .73196 .23161 .72811 .22617 Metetra
.664 .741 .908 r
.72811 .22617 .73196 .23161 .74042 .22866 .73659 .2232 Metetra
.664 .741 .908 r
.73659 .2232 .74042 .22866 .74891 .2257 .74509 .22023 Metetra
.664 .741 .908 r
.74509 .22023 .74891 .2257 .75742 .22274 .75362 .21725 Metetra
.664 .741 .908 r
.75362 .21725 .75742 .22274 .76595 .21976 .76217 .21426 Metetra
.664 .741 .908 r
.76217 .21426 .76595 .21976 .77451 .21678 .77075 .21126 Metetra
.664 .741 .908 r
.77075 .21126 .77451 .21678 .7831 .21379 .77935 .20825 Metetra
.664 .741 .908 r
.77935 .20825 .7831 .21379 .79171 .21079 .78798 .20524 Metetra
.664 .741 .908 r
.78798 .20524 .79171 .21079 .80035 .20779 .79664 .20221 Metetra
.664 .741 .908 r
.79664 .20221 .80035 .20779 .80901 .20477 .80532 .19918 Metetra
.664 .741 .908 r
.80532 .19918 .80901 .20477 .8177 .20175 .81403 .19614 Metetra
.699 .738 .881 r
.1932 .40934 .19802 .41411 .20509 .41073 .20027 .40611 Metetra
.679 .74 .897 r
.20027 .40611 .20509 .41073 .21212 .40794 .20731 .40336 Metetra
.669 .74 .903 r
.20731 .40336 .21212 .40794 .21915 .40536 .21435 .40078 Metetra
.665 .74 .906 r
.21435 .40078 .21915 .40536 .22619 .40287 .2214 .39828 Metetra
.663 .74 .907 r
.2214 .39828 .22619 .40287 .23324 .40041 .22846 .39582 Metetra
.661 .739 .908 r
.22846 .39582 .23324 .40041 .24031 .39799 .23554 .39339 Metetra
.659 .739 .909 r
.23554 .39339 .24031 .39799 .24739 .3956 .24264 .39098 Metetra
.658 .739 .91 r
.24264 .39098 .24739 .3956 .25449 .39322 .24975 .3886 Metetra
.655 .738 .91 r
.24975 .3886 .25449 .39322 .26161 .39088 .25688 .38625 Metetra
.653 .737 .912 r
.25688 .38625 .26161 .39088 .26875 .38859 .26402 .38394 Metetra
.649 .736 .913 r
.26402 .38394 .26875 .38859 .2759 .38637 .27118 .38169 Metetra
.642 .734 .915 r
.27118 .38169 .2759 .38637 .28306 .38425 .27836 .37956 Metetra
.631 .73 .919 r
.27836 .37956 .28306 .38425 .29023 .38233 .28554 .37761 Metetra
.608 .722 .926 r
.28554 .37761 .29023 .38233 .29739 .38079 .29271 .37602 Metetra
.551 .7 .938 r
.29271 .37602 .29739 .38079 .30452 .38011 .29986 .37522 Metetra
.376 .615 .944 r
.29986 .37522 .30452 .38011 .31152 .38162 .3069 .37622 Metetra
.475 .639 .927 r
.3069 .37622 .31152 .38162 .3186 .38276 .31404 .37647 Metetra
.785 .715 .787 r
.31404 .37647 .3186 .38276 .32629 .37447 .32168 .36931 Metetra
.767 .748 .835 r
.32168 .36931 .32629 .37447 .33383 .36865 .32921 .36378 Metetra
.726 .751 .873 r
.32921 .36378 .33383 .36865 .34127 .36456 .33667 .35974 Metetra
.702 .749 .888 r
.33667 .35974 .34127 .36456 .34869 .36114 .3441 .35633 Metetra
.689 .747 .896 r
.3441 .35633 .34869 .36114 .35611 .35802 .35153 .35321 Metetra
.681 .745 .899 r
.35153 .35321 .35611 .35802 .36354 .35506 .35898 .35024 Metetra
.677 .744 .902 r
.35898 .35024 .36354 .35506 .37099 .35219 .36643 .34737 Metetra
.674 .744 .903 r
.36643 .34737 .37099 .35219 .37845 .34938 .37391 .34455 Metetra
.671 .743 .904 r
.37391 .34455 .37845 .34938 .38593 .34661 .3814 .34177 Metetra
.669 .743 .905 r
.3814 .34177 .38593 .34661 .39342 .34387 .38891 .33902 Metetra
.667 .742 .906 r
.38891 .33902 .39342 .34387 .40094 .34118 .39644 .33632 Metetra
.662 .741 .909 r
.39644 .33632 .40094 .34118 .40847 .33858 .40398 .33371 Metetra
.649 .737 .914 r
.40398 .33371 .40847 .33858 .41602 .33624 .41154 .33134 Metetra
.621 .727 .922 r
.41154 .33134 .41602 .33624 .42356 .33452 .41911 .32945 Metetra
.662 .734 .903 r
.41911 .32945 .42356 .33452 .43116 .33188 .42672 .3268 Metetra
.691 .743 .891 r
.42672 .3268 .43116 .33188 .43881 .32851 .43437 .32352 Metetra
.678 .743 .9 r
.43437 .32352 .43881 .32851 .44646 .32551 .44204 .32052 Metetra
.67 .742 .904 r
.44204 .32052 .44646 .32551 .45412 .3227 .44972 .3177 Metetra
.664 .74 .907 r
.44972 .3177 .45412 .3227 .4618 .32001 .45741 .315 Metetra
.657 .737 .909 r
.45741 .315 .4618 .32001 .46951 .31746 .46513 .31242 Metetra
.648 .734 .912 r
.46513 .31242 .46951 .31746 .47723 .31507 .47287 .31002 Metetra
.634 .729 .916 r
.47287 .31002 .47723 .31507 .48497 .31295 .48062 .30786 Metetra
.606 .718 .924 r
.48062 .30786 .48497 .31295 .49274 .31132 .4884 .30618 Metetra
.534 .685 .936 r
.4884 .30618 .49274 .31132 .50052 .31087 .49619 .30556 Metetra
.296 .555 .925 r
.49619 .30556 .50052 .31087 .50832 .31363 .504 .3076 Metetra
.422 .591 .913 r
.504 .3076 .50832 .31363 .51617 .31556 .51186 .30868 Metetra
.765 .711 .802 r
.51186 .30868 .51617 .31556 .52406 .30797 .51977 .30227 Metetra
.746 .743 .849 r
.51977 .30227 .52406 .30797 .53197 .30263 .52769 .29722 Metetra
.706 .742 .88 r
.52769 .29722 .53197 .30263 .53988 .29879 .53563 .29344 Metetra
.68 .737 .894 r
.53563 .29344 .53988 .29879 .54783 .29564 .54358 .29028 Metetra
.664 .732 .901 r
.54358 .29028 .54783 .29564 .55579 .29287 .55156 .28748 Metetra
.656 .728 .902 r
.55156 .28748 .55579 .29287 .56378 .29033 .55957 .28482 Metetra
.692 .737 .885 r
.55957 .28482 .56378 .29033 .57179 .28714 .56759 .28134 Metetra
.817 .76 .799 r
.56759 .28134 .57179 .28714 .57974 .27936 .57555 .27322 Metetra
.177 .443 .873 r
.57555 .27322 .57974 .27936 .58791 .28286 .58373 .27704 Metetra
.316 .544 .913 r
.58373 .27704 .58791 .28286 .59611 .28526 .59194 .27926 Metetra
.037 .357 .83 r
.59194 .27926 .59611 .28526 .60446 .29109 .60027 .28433 Metetra
0 0 .39 r
.60027 .28433 .60446 .29109 .61319 .30817 .60889 .2979 Metetra
.315 .399 .79 r
.60889 .2979 .61319 .30817 .62175 .31568 .61736 .30254 Metetra
.848 .655 .641 r
.61736 .30254 .62175 .31568 .62929 .29027 .62506 .28125 Metetra
.883 .709 .661 r
.62506 .28125 .62929 .29027 .63685 .26795 .63275 .262 Metetra
.829 .765 .792 r
.63275 .262 .63685 .26795 .64483 .25865 .64078 .25324 Metetra
.737 .763 .873 r
.64078 .25324 .64483 .25865 .65299 .25388 .64896 .24853 Metetra
.695 .752 .895 r
.64896 .24853 .65299 .25388 .66122 .25027 .65721 .24492 Metetra
.678 .747 .902 r
.65721 .24492 .66122 .25027 .66949 .24705 .66549 .24169 Metetra
.671 .744 .905 r
.66549 .24169 .66949 .24705 .67778 .24399 .6738 .23861 Metetra
.668 .743 .906 r
.6738 .23861 .67778 .24399 .68611 .24098 .68214 .23559 Metetra
.666 .742 .907 r
.68214 .23559 .68611 .24098 .69446 .23801 .69051 .2326 Metetra
.665 .742 .907 r
.69051 .2326 .69446 .23801 .70283 .23505 .6989 .22963 Metetra
.665 .742 .908 r
.6989 .22963 .70283 .23505 .71123 .23209 .70732 .22665 Metetra
.664 .742 .908 r
.70732 .22665 .71123 .23209 .71966 .22913 .71577 .22368 Metetra
.664 .741 .908 r
.71577 .22368 .71966 .22913 .72811 .22617 .72423 .2207 Metetra
.664 .741 .908 r
.72423 .2207 .72811 .22617 .73659 .2232 .73273 .21772 Metetra
.664 .741 .908 r
.73273 .21772 .73659 .2232 .74509 .22023 .74125 .21472 Metetra
.664 .741 .908 r
.74125 .21472 .74509 .22023 .75362 .21725 .74979 .21173 Metetra
.664 .741 .908 r
.74979 .21173 .75362 .21725 .76217 .21426 .75836 .20872 Metetra
.664 .741 .908 r
.75836 .20872 .76217 .21426 .77075 .21126 .76696 .2057 Metetra
.664 .741 .908 r
.76696 .2057 .77075 .21126 .77935 .20825 .77558 .20268 Metetra
.664 .741 .908 r
.77558 .20268 .77935 .20825 .78798 .20524 .78423 .19965 Metetra
.664 .741 .908 r
.78423 .19965 .78798 .20524 .79664 .20221 .79291 .19661 Metetra
.664 .741 .908 r
.79291 .19661 .79664 .20221 .80532 .19918 .80161 .19356 Metetra
.664 .741 .908 r
.80161 .19356 .80532 .19918 .81403 .19614 .81033 .1905 Metetra
.693 .738 .885 r
.18837 .40457 .1932 .40934 .20027 .40611 .19543 .40148 Metetra
.676 .74 .898 r
.19543 .40148 .20027 .40611 .20731 .40336 .20248 .39875 Metetra
.668 .74 .904 r
.20248 .39875 .20731 .40336 .21435 .40078 .20952 .39618 Metetra
.665 .74 .906 r
.20952 .39618 .21435 .40078 .2214 .39828 .21658 .39368 Metetra
.662 .74 .907 r
.21658 .39368 .2214 .39828 .22846 .39582 .22366 .39121 Metetra
.661 .739 .908 r
.22366 .39121 .22846 .39582 .23554 .39339 .23075 .38877 Metetra
.659 .739 .909 r
.23075 .38877 .23554 .39339 .24264 .39098 .23785 .38635 Metetra
.658 .739 .91 r
.23785 .38635 .24264 .39098 .24975 .3886 .24498 .38395 Metetra
.656 .738 .91 r
.24498 .38395 .24975 .3886 .25688 .38625 .25212 .38158 Metetra
.653 .737 .911 r
.25212 .38158 .25688 .38625 .26402 .38394 .25928 .37926 Metetra
.649 .736 .913 r
.25928 .37926 .26402 .38394 .27118 .38169 .26645 .377 Metetra
.643 .734 .915 r
.26645 .377 .27118 .38169 .27836 .37956 .27363 .37484 Metetra
.632 .73 .919 r
.27363 .37484 .27836 .37956 .28554 .37761 .28083 .37287 Metetra
.61 .723 .925 r
.28083 .37287 .28554 .37761 .29271 .37602 .28801 .37123 Metetra
.559 .703 .937 r
.28801 .37123 .29271 .37602 .29986 .37522 .29517 .37031 Metetra
.426 .64 .946 r
.29517 .37031 .29986 .37522 .3069 .37622 .30226 .37086 Metetra
.528 .664 .923 r
.30226 .37086 .3069 .37622 .31404 .37647 .30944 .37045 Metetra
.772 .721 .805 r
.30944 .37045 .31404 .37647 .32168 .36931 .31704 .36415 Metetra
.76 .747 .841 r
.31704 .36415 .32168 .36931 .32921 .36378 .32458 .35888 Metetra
.723 .751 .874 r
.32458 .35888 .32921 .36378 .33667 .35974 .33204 .3549 Metetra
.701 .748 .889 r
.33204 .3549 .33667 .35974 .3441 .35633 .33949 .3515 Metetra
.688 .746 .896 r
.33949 .3515 .3441 .35633 .35153 .35321 .34693 .34837 Metetra
.681 .745 .9 r
.34693 .34837 .35153 .35321 .35898 .35024 .35439 .3454 Metetra
.676 .744 .902 r
.35439 .3454 .35898 .35024 .36643 .34737 .36186 .34252 Metetra
.673 .744 .903 r
.36186 .34252 .36643 .34737 .37391 .34455 .36935 .33969 Metetra
.671 .743 .904 r
.36935 .33969 .37391 .34455 .3814 .34177 .37685 .3369 Metetra
.669 .743 .905 r
.37685 .3369 .3814 .34177 .38891 .33902 .38437 .33415 Metetra
.667 .742 .907 r
.38437 .33415 .38891 .33902 .39644 .33632 .39192 .33143 Metetra
.662 .741 .909 r
.39192 .33143 .39644 .33632 .40398 .33371 .39947 .32881 Metetra
.65 .737 .913 r
.39947 .32881 .40398 .33371 .41154 .33134 .40705 .32641 Metetra
.629 .729 .92 r
.40705 .32641 .41154 .33134 .41911 .32945 .41463 .32438 Metetra
.662 .735 .904 r
.41463 .32438 .41911 .32945 .42672 .3268 .42225 .32172 Metetra
.688 .743 .893 r
.42225 .32172 .42672 .3268 .43437 .32352 .42992 .3185 Metetra
.678 .743 .9 r
.42992 .3185 .43437 .32352 .44204 .32052 .43759 .31551 Metetra
.67 .742 .904 r
.43759 .31551 .44204 .32052 .44972 .3177 .44529 .31268 Metetra
.664 .74 .907 r
.44529 .31268 .44972 .3177 .45741 .315 .453 .30996 Metetra
.657 .737 .909 r
.453 .30996 .45741 .315 .46513 .31242 .46073 .30736 Metetra
.648 .734 .912 r
.46073 .30736 .46513 .31242 .47287 .31002 .46848 .30493 Metetra
.635 .729 .916 r
.46848 .30493 .47287 .31002 .48062 .30786 .47625 .30275 Metetra
.609 .719 .923 r
.47625 .30275 .48062 .30786 .4884 .30618 .48404 .301 Metetra
.544 .69 .934 r
.48404 .301 .4884 .30618 .49619 .30556 .49184 .30024 Metetra
.361 .592 .933 r
.49184 .30024 .49619 .30556 .504 .3076 .49967 .30164 Metetra
.47 .621 .916 r
.49967 .30164 .504 .3076 .51186 .30868 .50754 .30209 Metetra
.748 .715 .821 r
.50754 .30209 .51186 .30868 .51977 .30227 .51546 .29656 Metetra
.737 .741 .855 r
.51546 .29656 .51977 .30227 .52769 .29722 .5234 .29178 Metetra
.702 .742 .882 r
.5234 .29178 .52769 .29722 .53563 .29344 .53135 .28807 Metetra
.679 .737 .894 r
.53135 .28807 .53563 .29344 .54358 .29028 .53932 .28491 Metetra
.665 .733 .9 r
.53932 .28491 .54358 .29028 .55156 .28748 .54731 .28206 Metetra
.66 .73 .901 r
.54731 .28206 .55156 .28748 .55957 .28482 .55533 .2793 Metetra
.699 .74 .883 r
.55533 .2793 .55957 .28482 .56759 .28134 .56336 .27563 Metetra
.807 .762 .811 r
.56336 .27563 .56759 .28134 .57555 .27322 .57136 .26812 Metetra
.161 .484 .9 r
.57136 .26812 .57555 .27322 .58373 .27704 .57954 .2713 Metetra
.331 .557 .918 r
.57954 .2713 .58373 .27704 .59194 .27926 .58775 .27329 Metetra
.109 .408 .858 r
.58775 .27329 .59194 .27926 .60027 .28433 .59606 .27764 Metetra
0 0 .545 r
.59606 .27764 .60027 .28433 .60889 .2979 .6046 .28832 Metetra
.414 .477 .816 r
.6046 .28832 .60889 .2979 .61736 .30254 .61301 .29087 Metetra
.844 .666 .661 r
.61301 .29087 .61736 .30254 .62506 .28125 .62083 .27281 Metetra
.875 .717 .681 r
.62083 .27281 .62506 .28125 .63275 .262 .62864 .25605 Metetra
.82 .764 .8 r
.62864 .25605 .63275 .262 .64078 .25324 .63671 .2478 Metetra
.735 .762 .874 r
.63671 .2478 .64078 .25324 .64896 .24853 .64491 .24314 Metetra
.694 .752 .896 r
.64491 .24314 .64896 .24853 .65721 .24492 .65317 .23954 Metetra
.678 .747 .902 r
.65317 .23954 .65721 .24492 .66549 .24169 .66147 .2363 Metetra
.671 .744 .905 r
.66147 .2363 .66549 .24169 .6738 .23861 .6698 .2332 Metetra
.668 .743 .906 r
.6698 .2332 .6738 .23861 .68214 .23559 .67816 .23017 Metetra
.666 .742 .907 r
.67816 .23017 .68214 .23559 .69051 .2326 .68655 .22717 Metetra
.665 .742 .907 r
.68655 .22717 .69051 .2326 .6989 .22963 .69496 .22417 Metetra
.665 .742 .908 r
.69496 .22417 .6989 .22963 .70732 .22665 .70339 .22118 Metetra
.664 .742 .908 r
.70339 .22118 .70732 .22665 .71577 .22368 .71185 .21819 Metetra
.664 .741 .908 r
.71185 .21819 .71577 .22368 .72423 .2207 .72034 .2152 Metetra
.664 .741 .908 r
.72034 .2152 .72423 .2207 .73273 .21772 .72885 .2122 Metetra
.664 .741 .908 r
.72885 .2122 .73273 .21772 .74125 .21472 .73739 .20919 Metetra
.664 .741 .908 r
.73739 .20919 .74125 .21472 .74979 .21173 .74595 .20617 Metetra
.664 .741 .908 r
.74595 .20617 .74979 .21173 .75836 .20872 .75454 .20315 Metetra
.664 .741 .908 r
.75454 .20315 .75836 .20872 .76696 .2057 .76315 .20012 Metetra
.664 .741 .908 r
.76315 .20012 .76696 .2057 .77558 .20268 .77179 .19708 Metetra
.664 .741 .908 r
.77179 .19708 .77558 .20268 .78423 .19965 .78046 .19403 Metetra
.664 .741 .908 r
.78046 .19403 .78423 .19965 .79291 .19661 .78915 .19097 Metetra
.664 .741 .908 r
.78915 .19097 .79291 .19661 .80161 .19356 .79787 .18791 Metetra
.664 .741 .908 r
.79787 .18791 .80161 .19356 .81033 .1905 .80662 .18483 Metetra
.688 .737 .889 r
.1835 .3998 .18837 .40457 .19543 .40148 .19057 .39682 Metetra
.674 .74 .9 r
.19057 .39682 .19543 .40148 .20248 .39875 .19762 .39412 Metetra
.668 .74 .904 r
.19762 .39412 .20248 .39875 .20952 .39618 .20468 .39155 Metetra
.664 .74 .906 r
.20468 .39155 .20952 .39618 .21658 .39368 .21174 .38905 Metetra
.662 .74 .907 r
.21174 .38905 .21658 .39368 .22366 .39121 .21883 .38657 Metetra
.661 .739 .908 r
.21883 .38657 .22366 .39121 .23075 .38877 .22593 .38412 Metetra
.659 .739 .909 r
.22593 .38412 .23075 .38877 .23785 .38635 .23305 .38169 Metetra
.658 .739 .91 r
.23305 .38169 .23785 .38635 .24498 .38395 .24018 .37928 Metetra
.656 .738 .91 r
.24018 .37928 .24498 .38395 .25212 .38158 .24734 .3769 Metetra
.653 .737 .911 r
.24734 .3769 .25212 .38158 .25928 .37926 .25451 .37456 Metetra
.649 .736 .913 r
.25451 .37456 .25928 .37926 .26645 .377 .26169 .37228 Metetra
.643 .734 .915 r
.26169 .37228 .26645 .377 .27363 .37484 .26889 .3701 Metetra
.633 .731 .918 r
.26889 .3701 .27363 .37484 .28083 .37287 .27609 .3681 Metetra
.613 .723 .924 r
.27609 .3681 .28083 .37287 .28801 .37123 .28329 .36641 Metetra
.567 .706 .935 r
.28329 .36641 .28801 .37123 .29517 .37031 .29047 .36537 Metetra
.467 .659 .945 r
.29047 .36537 .29517 .37031 .30226 .37086 .29759 .36552 Metetra
.562 .681 .919 r
.29759 .36552 .30226 .37086 .30944 .37045 .30481 .36463 Metetra
.76 .725 .82 r
.30481 .36463 .30944 .37045 .31704 .36415 .31238 .35898 Metetra
.754 .747 .846 r
.31238 .35898 .31704 .36415 .32458 .35888 .31992 .35395 Metetra
.72 .75 .876 r
.31992 .35395 .32458 .35888 .33204 .3549 .32739 .35003 Metetra
.699 .748 .889 r
.32739 .35003 .33204 .3549 .33949 .3515 .33485 .34664 Metetra
.688 .746 .896 r
.33485 .34664 .33949 .3515 .34693 .34837 .34231 .34352 Metetra
.681 .745 .9 r
.34231 .34352 .34693 .34837 .35439 .3454 .34978 .34054 Metetra
.676 .744 .902 r
.34978 .34054 .35439 .3454 .36186 .34252 .35726 .33765 Metetra
.673 .744 .903 r
.35726 .33765 .36186 .34252 .36935 .33969 .36476 .33481 Metetra
.671 .743 .905 r
.36476 .33481 .36935 .33969 .37685 .3369 .37228 .33201 Metetra
.669 .743 .905 r
.37228 .33201 .37685 .3369 .38437 .33415 .37981 .32924 Metetra
.666 .742 .907 r
.37981 .32924 .38437 .33415 .39192 .33143 .38737 .32652 Metetra
.662 .741 .909 r
.38737 .32652 .39192 .33143 .39947 .32881 .39494 .32389 Metetra
.651 .737 .913 r
.39494 .32389 .39947 .32881 .40705 .32641 .40252 .32145 Metetra
.635 .731 .918 r
.40252 .32145 .40705 .32641 .41463 .32438 .41012 .3193 Metetra
.663 .736 .904 r
.41012 .3193 .41463 .32438 .42225 .32172 .41776 .31662 Metetra
.685 .742 .895 r
.41776 .31662 .42225 .32172 .42992 .3185 .42544 .31346 Metetra
.677 .743 .901 r
.42544 .31346 .42992 .3185 .43759 .31551 .43313 .31048 Metetra
.67 .742 .904 r
.43313 .31048 .43759 .31551 .44529 .31268 .44084 .30763 Metetra
.664 .74 .906 r
.44084 .30763 .44529 .31268 .453 .30996 .44856 .30489 Metetra
.657 .738 .909 r
.44856 .30489 .453 .30996 .46073 .30736 .45631 .30228 Metetra
.649 .734 .912 r
.45631 .30228 .46073 .30736 .46848 .30493 .46407 .29983 Metetra
.636 .73 .916 r
.46407 .29983 .46848 .30493 .47625 .30275 .47185 .29761 Metetra
.612 .72 .922 r
.47185 .29761 .47625 .30275 .48404 .301 .47966 .29581 Metetra
.554 .694 .933 r
.47966 .29581 .48404 .301 .49184 .30024 .48747 .29489 Metetra
.412 .62 .937 r
.48747 .29489 .49184 .30024 .49967 .30164 .49531 .29576 Metetra
.504 .642 .917 r
.49531 .29576 .49967 .30164 .50754 .30209 .50319 .29571 Metetra
.732 .717 .837 r
.50319 .29571 .50754 .30209 .51546 .29656 .51113 .29085 Metetra
.729 .74 .86 r
.51113 .29085 .51546 .29656 .5234 .29178 .51908 .28632 Metetra
.699 .741 .883 r
.51908 .28632 .5234 .29178 .53135 .28807 .52705 .28267 Metetra
.679 .737 .895 r
.52705 .28267 .53135 .28807 .53932 .28491 .53503 .27951 Metetra
.666 .733 .9 r
.53503 .27951 .53932 .28491 .54731 .28206 .54304 .27663 Metetra
.664 .732 .9 r
.54304 .27663 .54731 .28206 .55533 .2793 .55107 .27378 Metetra
.703 .742 .882 r
.55107 .27378 .55533 .2793 .56336 .27563 .55912 .27 Metetra
.787 .762 .83 r
.55912 .27 .56336 .27563 .57136 .26812 .56715 .26345 Metetra
.241 .558 .934 r
.56715 .26345 .57136 .26812 .57954 .2713 .57532 .26561 Metetra
.351 .572 .923 r
.57532 .26561 .57954 .2713 .58775 .27329 .58353 .26733 Metetra
.175 .454 .88 r
.58353 .26733 .58775 .27329 .59606 .27764 .59184 .271 Metetra
0 .136 .679 r
.59184 .271 .59606 .27764 .6046 .28832 .60032 .27932 Metetra
.485 .533 .831 r
.60032 .27932 .6046 .28832 .61301 .29087 .60868 .28033 Metetra
.838 .676 .682 r
.60868 .28033 .61301 .29087 .62083 .27281 .6166 .26485 Metetra
.867 .723 .701 r
.6166 .26485 .62083 .27281 .62864 .25605 .6245 .25012 Metetra
.811 .764 .809 r
.6245 .25012 .62864 .25605 .63671 .2478 .63261 .24233 Metetra
.732 .761 .876 r
.63261 .24233 .63671 .2478 .64491 .24314 .64084 .23773 Metetra
.694 .751 .896 r
.64084 .23773 .64491 .24314 .65317 .23954 .64912 .23413 Metetra
.678 .746 .903 r
.64912 .23413 .65317 .23954 .66147 .2363 .65743 .23088 Metetra
.671 .744 .905 r
.65743 .23088 .66147 .2363 .6698 .2332 .66578 .22777 Metetra
.668 .743 .906 r
.66578 .22777 .6698 .2332 .67816 .23017 .67416 .22472 Metetra
.666 .742 .907 r
.67416 .22472 .67816 .23017 .68655 .22717 .68256 .2217 Metetra
.665 .742 .907 r
.68256 .2217 .68655 .22717 .69496 .22417 .69098 .21869 Metetra
.665 .742 .908 r
.69098 .21869 .69496 .22417 .70339 .22118 .69944 .21568 Metetra
.664 .742 .908 r
.69944 .21568 .70339 .22118 .71185 .21819 .70792 .21268 Metetra
.664 .741 .908 r
.70792 .21268 .71185 .21819 .72034 .2152 .71642 .20967 Metetra
.664 .741 .908 r
.71642 .20967 .72034 .2152 .72885 .2122 .72495 .20665 Metetra
.664 .741 .908 r
.72495 .20665 .72885 .2122 .73739 .20919 .7335 .20362 Metetra
.664 .741 .908 r
.7335 .20362 .73739 .20919 .74595 .20617 .74209 .20059 Metetra
.664 .741 .908 r
.74209 .20059 .74595 .20617 .75454 .20315 .75069 .19755 Metetra
.664 .741 .908 r
.75069 .19755 .75454 .20315 .76315 .20012 .75933 .1945 Metetra
.664 .741 .908 r
.75933 .1945 .76315 .20012 .77179 .19708 .76798 .19145 Metetra
.664 .741 .908 r
.76798 .19145 .77179 .19708 .78046 .19403 .77667 .18838 Metetra
.664 .741 .908 r
.77667 .18838 .78046 .19403 .78915 .19097 .78538 .18531 Metetra
.664 .741 .908 r
.78538 .18531 .78915 .19097 .79787 .18791 .79412 .18222 Metetra
.664 .741 .908 r
.79412 .18222 .79787 .18791 .80662 .18483 .80288 .17913 Metetra
.683 .737 .892 r
.17861 .39503 .1835 .3998 .19057 .39682 .18568 .39214 Metetra
.672 .739 .901 r
.18568 .39214 .19057 .39682 .19762 .39412 .19274 .38947 Metetra
.667 .74 .905 r
.19274 .38947 .19762 .39412 .20468 .39155 .1998 .38691 Metetra
.664 .74 .907 r
.1998 .38691 .20468 .39155 .21174 .38905 .20688 .3844 Metetra
.662 .74 .908 r
.20688 .3844 .21174 .38905 .21883 .38657 .21398 .38191 Metetra
.66 .739 .908 r
.21398 .38191 .21883 .38657 .22593 .38412 .22109 .37945 Metetra
.659 .739 .909 r
.22109 .37945 .22593 .38412 .23305 .38169 .22822 .37701 Metetra
.658 .739 .91 r
.22822 .37701 .23305 .38169 .24018 .37928 .23537 .37458 Metetra
.656 .738 .91 r
.23537 .37458 .24018 .37928 .24734 .3769 .24253 .37219 Metetra
.653 .737 .911 r
.24253 .37219 .24734 .3769 .25451 .37456 .24971 .36983 Metetra
.649 .736 .913 r
.24971 .36983 .25451 .37456 .26169 .37228 .25691 .36754 Metetra
.644 .734 .915 r
.25691 .36754 .26169 .37228 .26889 .3701 .26411 .36534 Metetra
.634 .731 .918 r
.26411 .36534 .26889 .3701 .27609 .3681 .27133 .3633 Metetra
.615 .724 .924 r
.27133 .3633 .27609 .3681 .28329 .36641 .27855 .36157 Metetra
.575 .709 .933 r
.27855 .36157 .28329 .36641 .29047 .36537 .28574 .36041 Metetra
.499 .673 .942 r
.28574 .36041 .29047 .36537 .29759 .36552 .29289 .36021 Metetra
.586 .693 .915 r
.29289 .36021 .29759 .36552 .30481 .36463 .30015 .35895 Metetra
.749 .729 .833 r
.30015 .35895 .30481 .36463 .31238 .35898 .3077 .35381 Metetra
.747 .746 .851 r
.3077 .35381 .31238 .35898 .31992 .35395 .31523 .349 Metetra
.718 .75 .877 r
.31523 .349 .31992 .35395 .32739 .35003 .32272 .34513 Metetra
.698 .748 .89 r
.32272 .34513 .32739 .35003 .33485 .34664 .33019 .34175 Metetra
.687 .746 .896 r
.33019 .34175 .33485 .34664 .34231 .34352 .33766 .33863 Metetra
.68 .745 .9 r
.33766 .33863 .34231 .34352 .34978 .34054 .34514 .33565 Metetra
.676 .744 .902 r
.34514 .33565 .34978 .34054 .35726 .33765 .35263 .33275 Metetra
.673 .743 .904 r
.35263 .33275 .35726 .33765 .36476 .33481 .36015 .3299 Metetra
.671 .743 .905 r
.36015 .3299 .36476 .33481 .37228 .33201 .36768 .32709 Metetra
.669 .743 .906 r
.36768 .32709 .37228 .33201 .37981 .32924 .37523 .32432 Metetra
.666 .742 .907 r
.37523 .32432 .37981 .32924 .38737 .32652 .3828 .32158 Metetra
.662 .741 .909 r
.3828 .32158 .38737 .32652 .39494 .32389 .39038 .31894 Metetra
.652 .738 .912 r
.39038 .31894 .39494 .32389 .40252 .32145 .39798 .31647 Metetra
.64 .733 .916 r
.39798 .31647 .40252 .32145 .41012 .3193 .40559 .31422 Metetra
.663 .737 .905 r
.40559 .31422 .41012 .3193 .41776 .31662 .41325 .31151 Metetra
.682 .742 .897 r
.41325 .31151 .41776 .31662 .42544 .31346 .42094 .3084 Metetra
.676 .743 .901 r
.42094 .3084 .42544 .31346 .43313 .31048 .42864 .30542 Metetra
.67 .742 .904 r
.42864 .30542 .43313 .31048 .44084 .30763 .43636 .30256 Metetra
.664 .74 .906 r
.43636 .30256 .44084 .30763 .44856 .30489 .4441 .2998 Metetra
.658 .738 .909 r
.4441 .2998 .44856 .30489 .45631 .30228 .45186 .29717 Metetra
.649 .735 .912 r
.45186 .29717 .45631 .30228 .46407 .29983 .45964 .2947 Metetra
.637 .73 .916 r
.45964 .2947 .46407 .29983 .47185 .29761 .46743 .29245 Metetra
.614 .721 .922 r
.46743 .29245 .47185 .29761 .47966 .29581 .47525 .29059 Metetra
.563 .698 .932 r
.47525 .29059 .47966 .29581 .48747 .29489 .48308 .28953 Metetra
.452 .641 .938 r
.48308 .28953 .48747 .29489 .49531 .29576 .49093 .28994 Metetra
.529 .658 .917 r
.49093 .28994 .49531 .29576 .50319 .29571 .49882 .28949 Metetra
.717 .719 .85 r
.49882 .28949 .50319 .29571 .51113 .29085 .50677 .28515 Metetra
.722 .739 .865 r
.50677 .28515 .51113 .29085 .51908 .28632 .51474 .28084 Metetra
.696 .74 .885 r
.51474 .28084 .51908 .28632 .52705 .28267 .52272 .27725 Metetra
.678 .737 .895 r
.52272 .27725 .52705 .28267 .53503 .27951 .53072 .27409 Metetra
.667 .734 .9 r
.53072 .27409 .53503 .27951 .54304 .27663 .53875 .27117 Metetra
.666 .733 .9 r
.53875 .27117 .54304 .27663 .55107 .27378 .54679 .26826 Metetra
.705 .744 .882 r
.54679 .26826 .55107 .27378 .55912 .27 .55486 .26442 Metetra
.768 .76 .846 r
.55486 .26442 .55912 .27 .56715 .26345 .56291 .25865 Metetra
.341 .612 .95 r
.56291 .25865 .56715 .26345 .57532 .26561 .57108 .25996 Metetra
.373 .588 .928 r
.57108 .25996 .57532 .26561 .58353 .26733 .5793 .26138 Metetra
.236 .494 .897 r
.5793 .26138 .58353 .26733 .59184 .271 .5876 .26442 Metetra
0 .272 .778 r
.5876 .26442 .59184 .271 .60032 .27932 .59603 .27081 Metetra
.535 .574 .841 r
.59603 .27081 .60032 .27932 .60868 .28033 .60437 .27067 Metetra
.831 .686 .703 r
.60437 .27067 .60868 .28033 .6166 .26485 .61237 .25724 Metetra
.858 .729 .721 r
.61237 .25724 .6166 .26485 .6245 .25012 .62035 .24419 Metetra
.802 .763 .817 r
.62035 .24419 .6245 .25012 .63261 .24233 .62849 .23683 Metetra
.729 .76 .877 r
.62849 .23683 .63261 .24233 .64084 .23773 .63674 .23229 Metetra
.693 .751 .896 r
.63674 .23229 .64084 .23773 .64912 .23413 .64504 .22869 Metetra
.678 .746 .903 r
.64504 .22869 .64912 .23413 .65743 .23088 .65337 .22543 Metetra
.671 .744 .905 r
.65337 .22543 .65743 .23088 .66578 .22777 .66174 .2223 Metetra
.668 .743 .906 r
.66174 .2223 .66578 .22777 .67416 .22472 .67013 .21924 Metetra
.666 .742 .907 r
.67013 .21924 .67416 .22472 .68256 .2217 .67855 .2162 Metetra
.665 .742 .907 r
.67855 .2162 .68256 .2217 .69098 .21869 .68699 .21318 Metetra
.665 .742 .908 r
.68699 .21318 .69098 .21869 .69944 .21568 .69546 .21015 Metetra
.664 .742 .908 r
.69546 .21015 .69944 .21568 .70792 .21268 .70396 .20713 Metetra
.664 .741 .908 r
.70396 .20713 .70792 .21268 .71642 .20967 .71248 .2041 Metetra
.664 .741 .908 r
.71248 .2041 .71642 .20967 .72495 .20665 .72103 .20107 Metetra
.664 .741 .908 r
.72103 .20107 .72495 .20665 .7335 .20362 .7296 .19803 Metetra
.664 .741 .908 r
.7296 .19803 .7335 .20362 .74209 .20059 .7382 .19498 Metetra
.664 .741 .908 r
.7382 .19498 .74209 .20059 .75069 .19755 .74682 .19192 Metetra
.664 .741 .908 r
.74682 .19192 .75069 .19755 .75933 .1945 .75548 .18886 Metetra
.664 .741 .908 r
.75548 .18886 .75933 .1945 .76798 .19145 .76415 .18578 Metetra
.664 .741 .908 r
.76415 .18578 .76798 .19145 .77667 .18838 .77286 .1827 Metetra
.664 .741 .908 r
.77286 .1827 .77667 .18838 .78538 .18531 .78159 .17961 Metetra
.664 .741 .908 r
.78159 .17961 .78538 .18531 .79412 .18222 .79034 .1765 Metetra
.664 .741 .908 r
.79034 .1765 .79412 .18222 .80288 .17913 .79913 .17339 Metetra
.679 .737 .895 r
.1737 .39025 .17861 .39503 .18568 .39214 .18076 .38744 Metetra
.671 .739 .902 r
.18076 .38744 .18568 .39214 .19274 .38947 .18783 .38479 Metetra
.666 .739 .905 r
.18783 .38479 .19274 .38947 .1998 .38691 .19491 .38224 Metetra
.663 .74 .907 r
.19491 .38224 .1998 .38691 .20688 .3844 .202 .37972 Metetra
.662 .74 .908 r
.202 .37972 .20688 .3844 .21398 .38191 .2091 .37723 Metetra
.66 .739 .908 r
.2091 .37723 .21398 .38191 .22109 .37945 .21622 .37476 Metetra
.659 .739 .909 r
.21622 .37476 .22109 .37945 .22822 .37701 .22336 .3723 Metetra
.658 .739 .91 r
.22336 .3723 .22822 .37701 .23537 .37458 .23052 .36986 Metetra
.656 .738 .91 r
.23052 .36986 .23537 .37458 .24253 .37219 .2377 .36745 Metetra
.653 .737 .911 r
.2377 .36745 .24253 .37219 .24971 .36983 .24489 .36508 Metetra
.65 .736 .913 r
.24489 .36508 .24971 .36983 .25691 .36754 .2521 .36277 Metetra
.644 .734 .915 r
.2521 .36277 .25691 .36754 .26411 .36534 .25932 .36055 Metetra
.635 .731 .918 r
.25932 .36055 .26411 .36534 .27133 .3633 .26655 .35848 Metetra
.617 .725 .923 r
.26655 .35848 .27133 .3633 .27855 .36157 .27377 .3567 Metetra
.582 .711 .931 r
.27377 .3567 .27855 .36157 .28574 .36041 .28099 .35543 Metetra
.525 .684 .939 r
.28099 .35543 .28574 .36041 .29289 .36021 .28817 .35492 Metetra
.603 .701 .913 r
.28817 .35492 .29289 .36021 .30015 .35895 .29545 .35338 Metetra
.739 .731 .844 r
.29545 .35338 .30015 .35895 .3077 .35381 .30299 .34864 Metetra
.741 .746 .856 r
.30299 .34864 .3077 .35381 .31523 .349 .31052 .34402 Metetra
.715 .749 .879 r
.31052 .34402 .31523 .349 .32272 .34513 .31802 .34021 Metetra
.697 .748 .891 r
.31802 .34021 .32272 .34513 .33019 .34175 .3255 .33685 Metetra
.686 .746 .897 r
.3255 .33685 .33019 .34175 .33766 .33863 .33298 .33372 Metetra
.68 .745 .9 r
.33298 .33372 .33766 .33863 .34514 .33565 .34048 .33074 Metetra
.676 .744 .902 r
.34048 .33074 .34514 .33565 .35263 .33275 .34799 .32783 Metetra
.673 .743 .904 r
.34799 .32783 .35263 .33275 .36015 .3299 .35551 .32497 Metetra
.671 .743 .905 r
.35551 .32497 .36015 .3299 .36768 .32709 .36306 .32215 Metetra
.669 .743 .906 r
.36306 .32215 .36768 .32709 .37523 .32432 .37062 .31936 Metetra
.666 .742 .907 r
.37062 .31936 .37523 .32432 .3828 .32158 .3782 .31662 Metetra
.662 .741 .909 r
.3782 .31662 .3828 .32158 .39038 .31894 .3858 .31396 Metetra
.653 .738 .912 r
.3858 .31396 .39038 .31894 .39798 .31647 .39341 .31145 Metetra
.644 .734 .915 r
.39341 .31145 .39798 .31647 .40559 .31422 .40104 .30912 Metetra
.663 .737 .905 r
.40104 .30912 .40559 .31422 .41325 .31151 .40871 .30639 Metetra
.68 .742 .898 r
.40871 .30639 .41325 .31151 .42094 .3084 .41641 .30331 Metetra
.675 .743 .901 r
.41641 .30331 .42094 .3084 .42864 .30542 .42413 .30033 Metetra
.67 .742 .904 r
.42413 .30033 .42864 .30542 .43636 .30256 .43186 .29746 Metetra
.664 .74 .906 r
.43186 .29746 .43636 .30256 .4441 .2998 .43961 .29469 Metetra
.658 .738 .909 r
.43961 .29469 .4441 .2998 .45186 .29717 .44739 .29204 Metetra
.65 .735 .912 r
.44739 .29204 .45186 .29717 .45964 .2947 .45518 .28954 Metetra
.638 .73 .915 r
.45518 .28954 .45964 .2947 .46743 .29245 .46299 .28726 Metetra
.617 .722 .921 r
.46299 .28726 .46743 .29245 .47525 .29059 .47082 .28534 Metetra
.571 .702 .93 r
.47082 .28534 .47525 .29059 .48308 .28953 .47867 .28415 Metetra
.484 .657 .937 r
.47867 .28415 .48308 .28953 .49093 .28994 .48653 .28416 Metetra
.548 .67 .917 r
.48653 .28416 .49093 .28994 .49882 .28949 .49443 .2834 Metetra
.705 .72 .861 r
.49443 .2834 .49882 .28949 .50677 .28515 .5024 .27944 Metetra
.714 .738 .87 r
.5024 .27944 .50677 .28515 .51474 .28084 .51038 .27533 Metetra
.694 .74 .887 r
.51038 .27533 .51474 .28084 .52272 .27725 .51838 .2718 Metetra
.677 .737 .896 r
.51838 .2718 .52272 .27725 .53072 .27409 .52639 .26865 Metetra
.667 .734 .9 r
.52639 .26865 .53072 .27409 .53875 .27117 .53443 .2657 Metetra
.668 .734 .899 r
.53443 .2657 .53875 .27117 .54679 .26826 .54249 .26272 Metetra
.704 .745 .883 r
.54249 .26272 .54679 .26826 .55486 .26442 .55057 .25888 Metetra
.75 .758 .859 r
.55057 .25888 .55486 .26442 .56291 .25865 .55865 .25372 Metetra
.414 .646 .953 r
.55865 .25372 .56291 .25865 .57108 .25996 .56682 .25434 Metetra
.398 .605 .931 r
.56682 .25434 .57108 .25996 .5793 .26138 .57505 .25546 Metetra
.29 .529 .909 r
.57505 .25546 .5793 .26138 .5876 .26442 .58334 .2579 Metetra
.098 .381 .842 r
.58334 .2579 .5876 .26442 .59603 .27081 .59174 .2627 Metetra
.572 .605 .847 r
.59174 .2627 .59603 .27081 .60437 .27067 .60006 .26171 Metetra
.824 .695 .723 r
.60006 .26171 .60437 .27067 .61237 .25724 .60813 .24993 Metetra
.848 .735 .739 r
.60813 .24993 .61237 .25724 .62035 .24419 .61618 .23827 Metetra
.794 .763 .825 r
.61618 .23827 .62035 .24419 .62849 .23683 .62435 .23129 Metetra
.726 .759 .879 r
.62435 .23129 .62849 .23683 .63674 .23229 .63262 .22682 Metetra
.692 .751 .896 r
.63262 .22682 .63674 .23229 .64504 .22869 .64094 .22322 Metetra
.678 .746 .903 r
.64094 .22322 .64504 .22869 .65337 .22543 .64929 .21994 Metetra
.671 .744 .905 r
.64929 .21994 .65337 .22543 .66174 .2223 .65767 .2168 Metetra
.668 .743 .906 r
.65767 .2168 .66174 .2223 .67013 .21924 .66608 .21373 Metetra
.666 .742 .907 r
.66608 .21373 .67013 .21924 .67855 .2162 .67452 .21067 Metetra
.665 .742 .907 r
.67452 .21067 .67855 .2162 .68699 .21318 .68298 .20763 Metetra
.665 .742 .908 r
.68298 .20763 .68699 .21318 .69546 .21015 .69147 .20459 Metetra
.664 .742 .908 r
.69147 .20459 .69546 .21015 .70396 .20713 .69998 .20155 Metetra
.664 .741 .908 r
.69998 .20155 .70396 .20713 .71248 .2041 .70852 .19851 Metetra
.664 .741 .908 r
.70852 .19851 .71248 .2041 .72103 .20107 .71708 .19546 Metetra
.664 .741 .908 r
.71708 .19546 .72103 .20107 .7296 .19803 .72567 .1924 Metetra
.664 .741 .908 r
.72567 .1924 .7296 .19803 .7382 .19498 .73429 .18933 Metetra
.664 .741 .908 r
.73429 .18933 .7382 .19498 .74682 .19192 .74293 .18626 Metetra
.664 .741 .908 r
.74293 .18626 .74682 .19192 .75548 .18886 .7516 .18318 Metetra
.664 .741 .908 r
.7516 .18318 .75548 .18886 .76415 .18578 .7603 .18008 Metetra
.664 .741 .908 r
.7603 .18008 .76415 .18578 .77286 .1827 .76902 .17698 Metetra
.664 .741 .908 r
.76902 .17698 .77286 .1827 .78159 .17961 .77777 .17387 Metetra
.664 .741 .908 r
.77777 .17387 .78159 .17961 .79034 .1765 .78655 .17076 Metetra
.664 .741 .908 r
.78655 .17076 .79034 .1765 .79913 .17339 .79535 .16763 Metetra
.675 .737 .897 r
.16876 .38547 .1737 .39025 .18076 .38744 .17583 .38272 Metetra
.669 .739 .903 r
.17583 .38272 .18076 .38744 .18783 .38479 .1829 .3801 Metetra
.665 .739 .905 r
.1829 .3801 .18783 .38479 .19491 .38224 .18998 .37754 Metetra
.663 .739 .907 r
.18998 .37754 .19491 .38224 .202 .37972 .19708 .37502 Metetra
.661 .739 .908 r
.19708 .37502 .202 .37972 .2091 .37723 .2042 .37253 Metetra
.66 .739 .908 r
.2042 .37253 .2091 .37723 .21622 .37476 .21133 .37004 Metetra
.659 .739 .909 r
.21133 .37004 .21622 .37476 .22336 .3723 .21849 .36757 Metetra
.658 .739 .91 r
.21849 .36757 .22336 .3723 .23052 .36986 .22566 .36512 Metetra
.656 .738 .91 r
.22566 .36512 .23052 .36986 .2377 .36745 .23284 .3627 Metetra
.654 .738 .911 r
.23284 .3627 .2377 .36745 .24489 .36508 .24005 .36031 Metetra
.65 .736 .913 r
.24005 .36031 .24489 .36508 .2521 .36277 .24727 .35798 Metetra
.645 .735 .914 r
.24727 .35798 .2521 .36277 .25932 .36055 .2545 .35573 Metetra
.636 .732 .917 r
.2545 .35573 .25932 .36055 .26655 .35848 .26174 .35364 Metetra
.62 .726 .922 r
.26174 .35364 .26655 .35848 .27377 .3567 .26898 .3518 Metetra
.589 .714 .93 r
.26898 .3518 .27377 .3567 .28099 .35543 .27621 .35043 Metetra
.546 .693 .936 r
.27621 .35043 .28099 .35543 .28817 .35492 .28342 .34965 Metetra
.616 .707 .91 r
.28342 .34965 .28817 .35492 .29545 .35338 .29073 .34788 Metetra
.731 .733 .852 r
.29073 .34788 .29545 .35338 .30299 .34864 .29825 .34346 Metetra
.735 .746 .861 r
.29825 .34346 .30299 .34864 .31052 .34402 .30579 .33902 Metetra
.713 .748 .88 r
.30579 .33902 .31052 .34402 .31802 .34021 .3133 .33526 Metetra
.696 .747 .891 r
.3133 .33526 .31802 .34021 .3255 .33685 .32079 .33191 Metetra
.686 .746 .897 r
.32079 .33191 .3255 .33685 .33298 .33372 .32829 .32879 Metetra
.679 .745 .9 r
.32829 .32879 .33298 .33372 .34048 .33074 .33579 .32579 Metetra
.675 .744 .902 r
.33579 .32579 .34048 .33074 .34799 .32783 .34332 .32288 Metetra
.673 .743 .904 r
.34332 .32288 .34799 .32783 .35551 .32497 .35085 .32001 Metetra
.67 .743 .905 r
.35085 .32001 .35551 .32497 .36306 .32215 .35841 .31718 Metetra
.668 .742 .906 r
.35841 .31718 .36306 .32215 .37062 .31936 .36599 .31438 Metetra
.666 .742 .907 r
.36599 .31438 .37062 .31936 .3782 .31662 .37358 .31163 Metetra
.662 .741 .909 r
.37358 .31163 .3782 .31662 .3858 .31396 .38119 .30895 Metetra
.654 .738 .911 r
.38119 .30895 .3858 .31396 .39341 .31145 .38882 .30641 Metetra
.647 .735 .913 r
.38882 .30641 .39341 .31145 .40104 .30912 .39646 .304 Metetra
.664 .738 .905 r
.39646 .304 .40104 .30912 .40871 .30639 .40415 .30125 Metetra
.678 .742 .899 r
.40415 .30125 .40871 .30639 .41641 .30331 .41186 .2982 Metetra
.675 .743 .902 r
.41186 .2982 .41641 .30331 .42413 .30033 .41959 .29522 Metetra
.67 .742 .904 r
.41959 .29522 .42413 .30033 .43186 .29746 .42734 .29233 Metetra
.664 .74 .906 r
.42734 .29233 .43186 .29746 .43961 .29469 .43511 .28954 Metetra
.658 .738 .909 r
.43511 .28954 .43961 .29469 .44739 .29204 .44289 .28688 Metetra
.65 .735 .912 r
.44289 .28688 .44739 .29204 .45518 .28954 .4507 .28436 Metetra
.639 .731 .915 r
.4507 .28436 .45518 .28954 .46299 .28726 .45852 .28204 Metetra
.619 .723 .92 r
.45852 .28204 .46299 .28726 .47082 .28534 .46637 .28007 Metetra
.579 .705 .929 r
.46637 .28007 .47082 .28534 .47867 .28415 .47423 .27875 Metetra
.51 .67 .935 r
.47423 .27875 .47867 .28415 .48653 .28416 .4821 .27842 Metetra
.564 .679 .917 r
.4821 .27842 .48653 .28416 .49443 .2834 .49002 .2774 Metetra
.694 .721 .87 r
.49002 .2774 .49443 .2834 .5024 .27944 .498 .27375 Metetra
.708 .737 .874 r
.498 .27375 .5024 .27944 .51038 .27533 .50599 .26981 Metetra
.691 .739 .888 r
.50599 .26981 .51038 .27533 .51838 .2718 .51401 .26633 Metetra
.676 .737 .896 r
.51401 .26633 .51838 .2718 .52639 .26865 .52204 .26318 Metetra
.668 .735 .9 r
.52204 .26318 .52639 .26865 .53443 .2657 .53009 .26021 Metetra
.67 .735 .899 r
.53009 .26021 .53443 .2657 .54249 .26272 .53817 .25718 Metetra
.703 .745 .884 r
.53817 .25718 .54249 .26272 .55057 .25888 .54626 .25336 Metetra
.735 .756 .87 r
.54626 .25336 .55057 .25888 .55865 .25372 .55437 .24868 Metetra
.466 .669 .952 r
.55437 .24868 .55865 .25372 .56682 .25434 .56254 .24874 Metetra
.422 .62 .934 r
.56254 .24874 .56682 .25434 .57505 .25546 .57077 .24954 Metetra
.338 .559 .918 r
.57077 .24954 .57505 .25546 .58334 .2579 .57906 .25143 Metetra
.219 .464 .879 r
.57906 .25143 .58334 .2579 .59174 .2627 .58744 .25492 Metetra
.599 .629 .853 r
.58744 .25492 .59174 .2627 .60006 .26171 .59576 .25331 Metetra
.815 .703 .742 r
.59576 .25331 .60006 .26171 .60813 .24993 .60388 .24286 Metetra
.837 .739 .756 r
.60388 .24286 .60813 .24993 .61618 .23827 .61198 .23235 Metetra
.786 .762 .832 r
.61198 .23235 .61618 .23827 .62435 .23129 .62019 .22573 Metetra
.723 .758 .88 r
.62019 .22573 .62435 .23129 .63262 .22682 .62848 .22131 Metetra
.691 .75 .897 r
.62848 .22131 .63262 .22682 .64094 .22322 .63681 .21772 Metetra
.677 .746 .903 r
.63681 .21772 .64094 .22322 .64929 .21994 .64518 .21443 Metetra
.671 .744 .905 r
.64518 .21443 .64929 .21994 .65767 .2168 .65358 .21128 Metetra
.668 .743 .906 r
.65358 .21128 .65767 .2168 .66608 .21373 .66201 .20818 Metetra
.666 .742 .907 r
.66201 .20818 .66608 .21373 .67452 .21067 .67046 .20511 Metetra
.665 .742 .907 r
.67046 .20511 .67452 .21067 .68298 .20763 .67894 .20206 Metetra
.665 .742 .908 r
.67894 .20206 .68298 .20763 .69147 .20459 .68745 .199 Metetra
.664 .742 .908 r
.68745 .199 .69147 .20459 .69998 .20155 .69598 .19594 Metetra
.664 .741 .908 r
.69598 .19594 .69998 .20155 .70852 .19851 .70453 .19288 Metetra
.664 .741 .908 r
.70453 .19288 .70852 .19851 .71708 .19546 .71312 .18981 Metetra
.664 .741 .908 r
.71312 .18981 .71708 .19546 .72567 .1924 .72173 .18674 Metetra
.664 .741 .908 r
.72173 .18674 .72567 .1924 .73429 .18933 .73036 .18366 Metetra
.664 .741 .908 r
.73036 .18366 .73429 .18933 .74293 .18626 .73902 .18057 Metetra
.664 .741 .908 r
.73902 .18057 .74293 .18626 .7516 .18318 .74771 .17747 Metetra
.664 .741 .908 r
.74771 .17747 .7516 .18318 .7603 .18008 .75642 .17436 Metetra
.664 .741 .908 r
.75642 .17436 .7603 .18008 .76902 .17698 .76517 .17124 Metetra
.664 .741 .908 r
.76517 .17124 .76902 .17698 .77777 .17387 .77393 .16811 Metetra
.664 .741 .908 r
.77393 .16811 .77777 .17387 .78655 .17076 .78273 .16497 Metetra
.664 .741 .908 r
.78273 .16497 .78655 .17076 .79535 .16763 .79155 .16183 Metetra
.672 .737 .899 r
.16379 .38068 .16876 .38547 .17583 .38272 .17086 .37799 Metetra
.667 .739 .903 r
.17086 .37799 .17583 .38272 .1829 .3801 .17794 .37538 Metetra
.664 .739 .906 r
.17794 .37538 .1829 .3801 .18998 .37754 .18504 .37283 Metetra
.662 .739 .907 r
.18504 .37283 .18998 .37754 .19708 .37502 .19215 .3703 Metetra
.661 .739 .908 r
.19215 .3703 .19708 .37502 .2042 .37253 .19927 .3678 Metetra
.66 .739 .909 r
.19927 .3678 .2042 .37253 .21133 .37004 .20642 .3653 Metetra
.659 .739 .909 r
.20642 .3653 .21133 .37004 .21849 .36757 .21358 .36282 Metetra
.658 .739 .91 r
.21358 .36282 .21849 .36757 .22566 .36512 .22076 .36035 Metetra
.656 .738 .91 r
.22076 .36035 .22566 .36512 .23284 .3627 .22796 .35791 Metetra
.654 .738 .911 r
.22796 .35791 .23284 .3627 .24005 .36031 .23518 .35551 Metetra
.65 .737 .912 r
.23518 .35551 .24005 .36031 .24727 .35798 .24241 .35316 Metetra
.645 .735 .914 r
.24241 .35316 .24727 .35798 .2545 .35573 .24965 .35089 Metetra
.637 .732 .917 r
.24965 .35089 .2545 .35573 .26174 .35364 .25691 .34877 Metetra
.622 .726 .921 r
.25691 .34877 .26174 .35364 .26898 .3518 .26416 .34688 Metetra
.595 .716 .928 r
.26416 .34688 .26898 .3518 .27621 .35043 .27141 .34541 Metetra
.564 .7 .933 r
.27141 .34541 .27621 .35043 .28342 .34965 .27864 .34439 Metetra
.625 .712 .909 r
.27864 .34439 .28342 .34965 .29073 .34788 .28598 .34244 Metetra
.723 .734 .86 r
.28598 .34244 .29073 .34788 .29825 .34346 .29349 .33828 Metetra
.729 .745 .865 r
.29349 .33828 .29825 .34346 .30579 .33902 .30103 .334 Metetra
.71 .748 .882 r
.30103 .334 .30579 .33902 .3133 .33526 .30855 .33029 Metetra
.695 .747 .892 r
.30855 .33029 .3133 .33526 .32079 .33191 .31605 .32695 Metetra
.685 .746 .897 r
.31605 .32695 .32079 .33191 .32829 .32879 .32356 .32383 Metetra
.679 .745 .9 r
.32356 .32383 .32829 .32879 .33579 .32579 .33108 .32083 Metetra
.675 .744 .902 r
.33108 .32083 .33579 .32579 .34332 .32288 .33862 .3179 Metetra
.672 .743 .904 r
.33862 .3179 .34332 .32288 .35085 .32001 .34617 .31503 Metetra
.67 .743 .905 r
.34617 .31503 .35085 .32001 .35841 .31718 .35374 .31219 Metetra
.668 .742 .906 r
.35374 .31219 .35841 .31718 .36599 .31438 .36133 .30938 Metetra
.666 .742 .907 r
.36133 .30938 .36599 .31438 .37358 .31163 .36894 .30661 Metetra
.662 .741 .909 r
.36894 .30661 .37358 .31163 .38119 .30895 .37656 .30391 Metetra
.655 .738 .911 r
.37656 .30391 .38119 .30895 .38882 .30641 .3842 .30134 Metetra
.65 .736 .912 r
.3842 .30134 .38882 .30641 .39646 .304 .39186 .29886 Metetra
.664 .738 .905 r
.39186 .29886 .39646 .304 .40415 .30125 .39956 .2961 Metetra
.676 .742 .9 r
.39956 .2961 .40415 .30125 .41186 .2982 .40728 .29307 Metetra
.674 .743 .902 r
.40728 .29307 .41186 .2982 .41959 .29522 .41503 .29008 Metetra
.669 .742 .904 r
.41503 .29008 .41959 .29522 .42734 .29233 .42279 .28718 Metetra
.664 .74 .906 r
.42279 .28718 .42734 .29233 .43511 .28954 .43057 .28438 Metetra
.658 .738 .909 r
.43057 .28438 .43511 .28954 .44289 .28688 .43837 .28169 Metetra
.65 .735 .911 r
.43837 .28169 .44289 .28688 .4507 .28436 .44619 .27915 Metetra
.64 .731 .915 r
.44619 .27915 .4507 .28436 .45852 .28204 .45403 .2768 Metetra
.621 .724 .92 r
.45403 .2768 .45852 .28204 .46637 .28007 .46189 .27478 Metetra
.585 .708 .927 r
.46189 .27478 .46637 .28007 .47423 .27875 .46976 .27334 Metetra
.531 .68 .933 r
.46976 .27334 .47423 .27875 .4821 .27842 .47766 .27272 Metetra
.576 .687 .917 r
.47766 .27272 .4821 .27842 .49002 .2774 .48559 .27147 Metetra
.686 .722 .877 r
.48559 .27147 .49002 .2774 .498 .27375 .49358 .26805 Metetra
.701 .736 .878 r
.49358 .26805 .498 .27375 .50599 .26981 .50159 .26427 Metetra
.688 .739 .889 r
.50159 .26427 .50599 .26981 .51401 .26633 .50961 .26083 Metetra
.676 .737 .897 r
.50961 .26083 .51401 .26633 .52204 .26318 .51766 .25769 Metetra
.668 .735 .9 r
.51766 .25769 .52204 .26318 .53009 .26021 .52573 .25469 Metetra
.671 .736 .899 r
.52573 .25469 .53009 .26021 .53817 .25718 .53382 .25162 Metetra
.701 .745 .886 r
.53382 .25162 .53817 .25718 .54626 .25336 .54193 .24785 Metetra
.721 .754 .878 r
.54193 .24785 .54626 .25336 .55437 .24868 .55006 .24355 Metetra
.504 .684 .948 r
.55006 .24355 .55437 .24868 .56254 .24874 .55824 .24315 Metetra
.446 .635 .935 r
.55824 .24315 .56254 .24874 .57077 .24954 .56648 .24363 Metetra
.381 .585 .923 r
.56648 .24363 .57077 .24954 .57906 .25143 .57477 .24502 Metetra
.313 .525 .9 r
.57477 .24502 .57906 .25143 .58744 .25492 .58313 .24743 Metetra
.619 .649 .857 r
.58313 .24743 .58744 .25492 .59576 .25331 .59144 .24534 Metetra
.806 .71 .759 r
.59144 .24534 .59576 .25331 .60388 .24286 .59961 .23598 Metetra
.826 .742 .771 r
.59961 .23598 .60388 .24286 .61198 .23235 .60777 .22644 Metetra
.778 .761 .838 r
.60777 .22644 .61198 .23235 .62019 .22573 .616 .22013 Metetra
.721 .757 .881 r
.616 .22013 .62019 .22573 .62848 .22131 .62431 .21578 Metetra
.69 .75 .897 r
.62431 .21578 .62848 .22131 .63681 .21772 .63267 .21218 Metetra
.677 .746 .903 r
.63267 .21218 .63681 .21772 .64518 .21443 .64105 .20889 Metetra
.671 .744 .905 r
.64105 .20889 .64518 .21443 .65358 .21128 .64947 .20572 Metetra
.668 .743 .906 r
.64947 .20572 .65358 .21128 .66201 .20818 .65791 .20261 Metetra
.666 .742 .907 r
.65791 .20261 .66201 .20818 .67046 .20511 .66638 .19952 Metetra
.665 .742 .907 r
.66638 .19952 .67046 .20511 .67894 .20206 .67488 .19645 Metetra
.665 .742 .908 r
.67488 .19645 .67894 .20206 .68745 .199 .6834 .19338 Metetra
.664 .742 .908 r
.6834 .19338 .68745 .199 .69598 .19594 .69195 .1903 Metetra
.664 .741 .908 r
.69195 .1903 .69598 .19594 .70453 .19288 .70053 .18723 Metetra
.664 .741 .908 r
.70053 .18723 .70453 .19288 .71312 .18981 .70913 .18414 Metetra
.664 .741 .908 r
.70913 .18414 .71312 .18981 .72173 .18674 .71776 .18105 Metetra
.664 .741 .908 r
.71776 .18105 .72173 .18674 .73036 .18366 .72641 .17795 Metetra
.664 .741 .908 r
.72641 .17795 .73036 .18366 .73902 .18057 .73509 .17484 Metetra
.664 .741 .908 r
.73509 .17484 .73902 .18057 .74771 .17747 .7438 .17172 Metetra
.664 .741 .908 r
.7438 .17172 .74771 .17747 .75642 .17436 .75253 .1686 Metetra
.664 .741 .908 r
.75253 .1686 .75642 .17436 .76517 .17124 .76129 .16546 Metetra
.664 .741 .908 r
.76129 .16546 .76517 .17124 .77393 .16811 .77007 .16232 Metetra
.664 .741 .908 r
.77007 .16232 .77393 .16811 .78273 .16497 .77889 .15916 Metetra
.664 .741 .908 r
.77889 .15916 .78273 .16497 .79155 .16183 .78773 .156 Metetra
.669 .737 .901 r
.15879 .37588 .16379 .38068 .17086 .37799 .16587 .37323 Metetra
.666 .738 .904 r
.16587 .37323 .17086 .37799 .17794 .37538 .17296 .37064 Metetra
.663 .739 .906 r
.17296 .37064 .17794 .37538 .18504 .37283 .18006 .36809 Metetra
.662 .739 .907 r
.18006 .36809 .18504 .37283 .19215 .3703 .18718 .36556 Metetra
.661 .739 .908 r
.18718 .36556 .19215 .3703 .19927 .3678 .19432 .36304 Metetra
.66 .739 .909 r
.19432 .36304 .19927 .3678 .20642 .3653 .20148 .36054 Metetra
.659 .739 .909 r
.20148 .36054 .20642 .3653 .21358 .36282 .20865 .35804 Metetra
.658 .739 .91 r
.20865 .35804 .21358 .36282 .22076 .36035 .21585 .35556 Metetra
.656 .738 .91 r
.21585 .35556 .22076 .36035 .22796 .35791 .22306 .35311 Metetra
.654 .738 .911 r
.22306 .35311 .22796 .35791 .23518 .35551 .23028 .35069 Metetra
.651 .737 .912 r
.23028 .35069 .23518 .35551 .24241 .35316 .23753 .34832 Metetra
.646 .735 .914 r
.23753 .34832 .24241 .35316 .24965 .35089 .24478 .34603 Metetra
.638 .732 .917 r
.24478 .34603 .24965 .35089 .25691 .34877 .25205 .34388 Metetra
.624 .727 .921 r
.25205 .34388 .25691 .34877 .26416 .34688 .25932 .34194 Metetra
.601 .718 .927 r
.25932 .34194 .26416 .34688 .27141 .34541 .26658 .34037 Metetra
.578 .706 .93 r
.26658 .34037 .27141 .34541 .27864 .34439 .27384 .33914 Metetra
.633 .716 .907 r
.27384 .33914 .27864 .34439 .28598 .34244 .28119 .33705 Metetra
.716 .735 .866 r
.28119 .33705 .28598 .34244 .29349 .33828 .28871 .33309 Metetra
.724 .745 .869 r
.28871 .33309 .29349 .33828 .30103 .334 .29625 .32895 Metetra
.708 .748 .883 r
.29625 .32895 .30103 .334 .30855 .33029 .30378 .32529 Metetra
.693 .747 .892 r
.30378 .32529 .30855 .33029 .31605 .32695 .3113 .32197 Metetra
.684 .745 .898 r
.3113 .32197 .31605 .32695 .32356 .32383 .31882 .31884 Metetra
.679 .744 .901 r
.31882 .31884 .32356 .32383 .33108 .32083 .32635 .31584 Metetra
.675 .744 .903 r
.32635 .31584 .33108 .32083 .33862 .3179 .3339 .3129 Metetra
.672 .743 .904 r
.3339 .3129 .33862 .3179 .34617 .31503 .34146 .31002 Metetra
.67 .743 .905 r
.34146 .31002 .34617 .31503 .35374 .31219 .34905 .30716 Metetra
.668 .742 .906 r
.34905 .30716 .35374 .31219 .36133 .30938 .35665 .30434 Metetra
.666 .742 .907 r
.35665 .30434 .36133 .30938 .36894 .30661 .36427 .30156 Metetra
.662 .741 .908 r
.36427 .30156 .36894 .30661 .37656 .30391 .37191 .29885 Metetra
.656 .739 .911 r
.37191 .29885 .37656 .30391 .3842 .30134 .37956 .29625 Metetra
.652 .737 .911 r
.37956 .29625 .3842 .30134 .39186 .29886 .38724 .29371 Metetra
.664 .739 .905 r
.38724 .29371 .39186 .29886 .39956 .2961 .39495 .29092 Metetra
.675 .742 .901 r
.39495 .29092 .39956 .2961 .40728 .29307 .40269 .2879 Metetra
.674 .743 .902 r
.40269 .2879 .40728 .29307 .41503 .29008 .41044 .28491 Metetra
.669 .742 .904 r
.41044 .28491 .41503 .29008 .42279 .28718 .41822 .282 Metetra
.665 .74 .906 r
.41822 .282 .42279 .28718 .43057 .28438 .42602 .27918 Metetra
.658 .738 .909 r
.42602 .27918 .43057 .28438 .43837 .28169 .43383 .27648 Metetra
.651 .735 .911 r
.43383 .27648 .43837 .28169 .44619 .27915 .44167 .27391 Metetra
.64 .732 .915 r
.44167 .27391 .44619 .27915 .45403 .2768 .44952 .27154 Metetra
.623 .725 .919 r
.44952 .27154 .45403 .2768 .46189 .27478 .45739 .26946 Metetra
.592 .711 .926 r
.45739 .26946 .46189 .27478 .46976 .27334 .46528 .26791 Metetra
.548 .688 .931 r
.46528 .26791 .46976 .27334 .47766 .27272 .47319 .26703 Metetra
.586 .694 .916 r
.47319 .26703 .47766 .27272 .48559 .27147 .48113 .2656 Metetra
.678 .722 .883 r
.48113 .2656 .48559 .27147 .49358 .26805 .48913 .26236 Metetra
.696 .736 .882 r
.48913 .26236 .49358 .26805 .50159 .26427 .49716 .25871 Metetra
.685 .738 .891 r
.49716 .25871 .50159 .26427 .50961 .26083 .5052 .25532 Metetra
.675 .737 .897 r
.5052 .25532 .50961 .26083 .51766 .25769 .51326 .25217 Metetra
.669 .736 .9 r
.51326 .25217 .51766 .25769 .52573 .25469 .52134 .24916 Metetra
.672 .737 .899 r
.52134 .24916 .52573 .25469 .53382 .25162 .52945 .24606 Metetra
.698 .745 .887 r
.52945 .24606 .53382 .25162 .54193 .24785 .53758 .24234 Metetra
.71 .751 .885 r
.53758 .24234 .54193 .24785 .55006 .24355 .54572 .23833 Metetra
.533 .695 .943 r
.54572 .23833 .55006 .24355 .55824 .24315 .55392 .23756 Metetra
.469 .648 .936 r
.55392 .23756 .55824 .24315 .56648 .24363 .56216 .23773 Metetra
.418 .607 .926 r
.56216 .23773 .56648 .24363 .57477 .24502 .57045 .23866 Metetra
.385 .57 .91 r
.57045 .23866 .57477 .24502 .58313 .24743 .5788 .24017 Metetra
.635 .664 .861 r
.5788 .24017 .58313 .24743 .59144 .24534 .58712 .23772 Metetra
.797 .717 .776 r
.58712 .23772 .59144 .24534 .59961 .23598 .59533 .22925 Metetra
.816 .745 .786 r
.59533 .22925 .59961 .23598 .60777 .22644 .60353 .22054 Metetra
.77 .76 .844 r
.60353 .22054 .60777 .22644 .616 .22013 .6118 .21451 Metetra
.718 .756 .883 r
.6118 .21451 .616 .22013 .62431 .21578 .62012 .21021 Metetra
.69 .75 .897 r
.62012 .21021 .62431 .21578 .63267 .21218 .6285 .20662 Metetra
.677 .746 .903 r
.6285 .20662 .63267 .21218 .64105 .20889 .6369 .20331 Metetra
.671 .744 .905 r
.6369 .20331 .64105 .20889 .64947 .20572 .64534 .20013 Metetra
.668 .743 .906 r
.64534 .20013 .64947 .20572 .65791 .20261 .6538 .19701 Metetra
.666 .742 .907 r
.6538 .19701 .65791 .20261 .66638 .19952 .66229 .1939 Metetra
.665 .742 .907 r
.66229 .1939 .66638 .19952 .67488 .19645 .6708 .19081 Metetra
.665 .742 .908 r
.6708 .19081 .67488 .19645 .6834 .19338 .67934 .18772 Metetra
.664 .742 .908 r
.67934 .18772 .6834 .19338 .69195 .1903 .68791 .18463 Metetra
.664 .741 .908 r
.68791 .18463 .69195 .1903 .70053 .18723 .6965 .18154 Metetra
.664 .741 .908 r
.6965 .18154 .70053 .18723 .70913 .18414 .70512 .17843 Metetra
.664 .741 .908 r
.70512 .17843 .70913 .18414 .71776 .18105 .71376 .17533 Metetra
.664 .741 .908 r
.71376 .17533 .71776 .18105 .72641 .17795 .72243 .17221 Metetra
.664 .741 .908 r
.72243 .17221 .72641 .17795 .73509 .17484 .73113 .16908 Metetra
.664 .741 .908 r
.73113 .16908 .73509 .17484 .7438 .17172 .73986 .16595 Metetra
.664 .741 .908 r
.73986 .16595 .7438 .17172 .75253 .1686 .74861 .1628 Metetra
.664 .741 .908 r
.74861 .1628 .75253 .1686 .76129 .16546 .75739 .15965 Metetra
.664 .741 .908 r
.75739 .15965 .76129 .16546 .77007 .16232 .76619 .15649 Metetra
.664 .741 .908 r
.76619 .15649 .77007 .16232 .77889 .15916 .77503 .15331 Metetra
.664 .741 .908 r
.77503 .15331 .77889 .15916 .78773 .156 .78389 .15013 Metetra
.667 .737 .902 r
.15377 .37107 .15879 .37588 .16587 .37323 .16086 .36846 Metetra
.664 .738 .905 r
.16086 .36846 .16587 .37323 .17296 .37064 .16795 .36588 Metetra
.663 .739 .907 r
.16795 .36588 .17296 .37064 .18006 .36809 .17507 .36333 Metetra
.662 .739 .908 r
.17507 .36333 .18006 .36809 .18718 .36556 .1822 .36079 Metetra
.661 .739 .908 r
.1822 .36079 .18718 .36556 .19432 .36304 .18935 .35826 Metetra
.66 .739 .909 r
.18935 .35826 .19432 .36304 .20148 .36054 .19651 .35575 Metetra
.659 .739 .909 r
.19651 .35575 .20148 .36054 .20865 .35804 .2037 .35324 Metetra
.658 .739 .91 r
.2037 .35324 .20865 .35804 .21585 .35556 .2109 .35075 Metetra
.656 .738 .91 r
.2109 .35075 .21585 .35556 .22306 .35311 .21813 .34828 Metetra
.654 .738 .911 r
.21813 .34828 .22306 .35311 .23028 .35069 .22536 .34584 Metetra
.651 .737 .912 r
.22536 .34584 .23028 .35069 .23753 .34832 .23262 .34345 Metetra
.646 .735 .914 r
.23262 .34345 .23753 .34832 .24478 .34603 .23989 .34114 Metetra
.639 .732 .916 r
.23989 .34114 .24478 .34603 .25205 .34388 .24717 .33896 Metetra
.626 .728 .92 r
.24717 .33896 .25205 .34388 .25932 .34194 .25445 .33698 Metetra
.606 .72 .925 r
.25445 .33698 .25932 .34194 .26658 .34037 .26173 .33531 Metetra
.59 .71 .927 r
.26173 .33531 .26658 .34037 .27384 .33914 .26901 .3339 Metetra
.638 .719 .906 r
.26901 .3339 .27384 .33914 .28119 .33705 .27638 .3317 Metetra
.71 .736 .871 r
.27638 .3317 .28119 .33705 .28871 .33309 .2839 .3279 Metetra
.719 .745 .872 r
.2839 .3279 .28871 .33309 .29625 .32895 .29145 .32389 Metetra
.705 .747 .884 r
.29145 .32389 .29625 .32895 .30378 .32529 .29898 .32027 Metetra
.692 .746 .893 r
.29898 .32027 .30378 .32529 .3113 .32197 .30651 .31696 Metetra
.684 .745 .898 r
.30651 .31696 .3113 .32197 .31882 .31884 .31405 .31383 Metetra
.678 .744 .901 r
.31405 .31383 .31882 .31884 .32635 .31584 .32159 .31082 Metetra
.675 .744 .903 r
.32159 .31082 .32635 .31584 .3339 .3129 .32915 .30787 Metetra
.672 .743 .904 r
.32915 .30787 .3339 .3129 .34146 .31002 .33673 .30498 Metetra
.67 .743 .905 r
.33673 .30498 .34146 .31002 .34905 .30716 .34433 .30212 Metetra
.668 .742 .906 r
.34433 .30212 .34905 .30716 .35665 .30434 .35194 .29928 Metetra
.666 .742 .907 r
.35194 .29928 .35665 .30434 .36427 .30156 .35958 .29649 Metetra
.662 .741 .908 r
.35958 .29649 .36427 .30156 .37191 .29885 .36723 .29376 Metetra
.657 .739 .91 r
.36723 .29376 .37191 .29885 .37956 .29625 .3749 .29113 Metetra
.654 .737 .911 r
.3749 .29113 .37956 .29625 .38724 .29371 .38259 .28854 Metetra
.665 .739 .905 r
.38259 .28854 .38724 .29371 .39495 .29092 .39031 .28573 Metetra
.674 .742 .901 r
.39031 .28573 .39495 .29092 .40269 .2879 .39806 .28272 Metetra
.673 .742 .903 r
.39806 .28272 .40269 .2879 .41044 .28491 .40584 .27972 Metetra
.669 .742 .905 r
.40584 .27972 .41044 .28491 .41822 .282 .41363 .2768 Metetra
.665 .74 .906 r
.41363 .2768 .41822 .282 .42602 .27918 .42144 .27396 Metetra
.658 .738 .909 r
.42144 .27396 .42602 .27918 .43383 .27648 .42927 .27124 Metetra
.651 .736 .911 r
.42927 .27124 .43383 .27648 .44167 .27391 .43711 .26865 Metetra
.641 .732 .914 r
.43711 .26865 .44167 .27391 .44952 .27154 .44498 .26625 Metetra
.625 .726 .919 r
.44498 .26625 .44952 .27154 .45739 .26946 .45287 .26412 Metetra
.597 .713 .925 r
.45287 .26412 .45739 .26946 .46528 .26791 .46077 .26246 Metetra
.562 .694 .929 r
.46077 .26246 .46528 .26791 .47319 .26703 .46869 .26136 Metetra
.594 .699 .916 r
.46869 .26136 .47319 .26703 .48113 .2656 .47665 .25978 Metetra
.672 .723 .887 r
.47665 .25978 .48113 .2656 .48913 .26236 .48466 .25666 Metetra
.691 .735 .885 r
.48466 .25666 .48913 .26236 .49716 .25871 .4927 .25313 Metetra
.683 .738 .892 r
.4927 .25313 .49716 .25871 .5052 .25532 .50076 .24978 Metetra
.674 .737 .898 r
.50076 .24978 .5052 .25532 .51326 .25217 .50883 .24664 Metetra
.669 .736 .9 r
.50883 .24664 .51326 .25217 .52134 .24916 .51693 .2436 Metetra
.672 .737 .899 r
.51693 .2436 .52134 .24916 .52945 .24606 .52506 .24047 Metetra
.695 .744 .889 r
.52506 .24047 .52945 .24606 .53758 .24234 .5332 .23681 Metetra
.7 .749 .89 r
.5332 .23681 .53758 .24234 .54572 .23833 .54136 .23304 Metetra
.554 .703 .939 r
.54136 .23304 .54572 .23833 .55392 .23756 .54957 .23196 Metetra
.49 .659 .936 r
.54957 .23196 .55392 .23756 .56216 .23773 .55782 .23184 Metetra
.45 .626 .928 r
.55782 .23184 .56216 .23773 .57045 .23866 .56612 .23233 Metetra
.44 .603 .915 r
.56612 .23233 .57045 .23866 .5788 .24017 .57446 .23311 Metetra
.646 .676 .864 r
.57446 .23311 .5788 .24017 .58712 .23772 .58279 .23038 Metetra
.788 .722 .79 r
.58279 .23038 .58712 .23772 .59533 .22925 .59103 .22265 Metetra
.805 .747 .799 r
.59103 .22265 .59533 .22925 .60353 .22054 .59927 .21463 Metetra
.763 .76 .85 r
.59927 .21463 .60353 .22054 .6118 .21451 .60756 .20886 Metetra
.715 .756 .884 r
.60756 .20886 .6118 .21451 .62012 .21021 .61591 .20461 Metetra
.689 .749 .898 r
.61591 .20461 .62012 .21021 .6285 .20662 .6243 .20102 Metetra
.676 .746 .903 r
.6243 .20102 .6285 .20662 .6369 .20331 .63273 .19771 Metetra
.671 .744 .905 r
.63273 .19771 .6369 .20331 .64534 .20013 .64118 .19451 Metetra
.668 .743 .906 r
.64118 .19451 .64534 .20013 .6538 .19701 .64966 .19137 Metetra
.666 .742 .907 r
.64966 .19137 .6538 .19701 .66229 .1939 .65816 .18825 Metetra
.665 .742 .907 r
.65816 .18825 .66229 .1939 .6708 .19081 .66669 .18514 Metetra
.665 .742 .908 r
.66669 .18514 .6708 .19081 .67934 .18772 .67525 .18204 Metetra
.664 .742 .908 r
.67525 .18204 .67934 .18772 .68791 .18463 .68384 .17893 Metetra
.664 .741 .908 r
.68384 .17893 .68791 .18463 .6965 .18154 .69245 .17582 Metetra
.664 .741 .908 r
.69245 .17582 .6965 .18154 .70512 .17843 .70108 .1727 Metetra
.664 .741 .908 r
.70108 .1727 .70512 .17843 .71376 .17533 .70975 .16957 Metetra
.664 .741 .908 r
.70975 .16957 .71376 .17533 .72243 .17221 .71844 .16643 Metetra
.664 .741 .908 r
.71844 .16643 .72243 .17221 .73113 .16908 .72716 .16329 Metetra
.664 .741 .908 r
.72716 .16329 .73113 .16908 .73986 .16595 .7359 .16014 Metetra
.664 .741 .908 r
.7359 .16014 .73986 .16595 .74861 .1628 .74467 .15698 Metetra
.664 .741 .908 r
.74467 .15698 .74861 .1628 .75739 .15965 .75347 .1538 Metetra
.664 .741 .908 r
.75347 .1538 .75739 .15965 .76619 .15649 .76229 .15062 Metetra
.664 .741 .908 r
.76229 .15062 .76619 .15649 .77503 .15331 .77114 .14743 Metetra
.664 .741 .908 r
.77114 .14743 .77503 .15331 .78389 .15013 .78002 .14423 Metetra
.665 .737 .904 r
.14873 .36625 .15377 .37107 .16086 .36846 .15582 .36366 Metetra
.663 .738 .906 r
.15582 .36366 .16086 .36846 .16795 .36588 .16292 .3611 Metetra
.662 .739 .907 r
.16292 .3611 .16795 .36588 .17507 .36333 .17004 .35854 Metetra
.661 .739 .908 r
.17004 .35854 .17507 .36333 .1822 .36079 .17718 .356 Metetra
.661 .739 .908 r
.17718 .356 .1822 .36079 .18935 .35826 .18434 .35346 Metetra
.66 .739 .909 r
.18434 .35346 .18935 .35826 .19651 .35575 .19152 .35093 Metetra
.659 .739 .909 r
.19152 .35093 .19651 .35575 .2037 .35324 .19872 .34841 Metetra
.658 .739 .91 r
.19872 .34841 .2037 .35324 .2109 .35075 .20594 .34591 Metetra
.656 .738 .91 r
.20594 .34591 .2109 .35075 .21813 .34828 .21317 .34342 Metetra
.654 .738 .911 r
.21317 .34342 .21813 .34828 .22536 .34584 .22042 .34097 Metetra
.651 .737 .912 r
.22042 .34097 .22536 .34584 .23262 .34345 .22769 .33856 Metetra
.647 .735 .914 r
.22769 .33856 .23262 .34345 .23989 .34114 .23497 .33623 Metetra
.64 .733 .916 r
.23497 .33623 .23989 .34114 .24717 .33896 .24226 .33401 Metetra
.628 .729 .919 r
.24226 .33401 .24717 .33896 .25445 .33698 .24955 .33198 Metetra
.611 .721 .924 r
.24955 .33198 .25445 .33698 .26173 .33531 .25685 .33023 Metetra
.6 .714 .925 r
.25685 .33023 .26173 .33531 .26901 .3339 .26416 .32866 Metetra
.643 .722 .906 r
.26416 .32866 .26901 .3339 .27638 .3317 .27155 .32636 Metetra
.705 .736 .876 r
.27155 .32636 .27638 .3317 .2839 .3279 .27906 .3227 Metetra
.715 .745 .875 r
.27906 .3227 .2839 .3279 .29145 .32389 .28662 .3188 Metetra
.703 .747 .886 r
.28662 .3188 .29145 .32389 .29898 .32027 .29416 .31522 Metetra
.691 .746 .893 r
.29416 .31522 .29898 .32027 .30651 .31696 .3017 .31192 Metetra
.683 .745 .898 r
.3017 .31192 .30651 .31696 .31405 .31383 .30925 .30879 Metetra
.678 .744 .901 r
.30925 .30879 .31405 .31383 .32159 .31082 .31681 .30577 Metetra
.674 .744 .903 r
.31681 .30577 .32159 .31082 .32915 .30787 .32438 .30282 Metetra
.672 .743 .904 r
.32438 .30282 .32915 .30787 .33673 .30498 .33197 .29991 Metetra
.67 .743 .905 r
.33197 .29991 .33673 .30498 .34433 .30212 .33958 .29704 Metetra
.668 .742 .906 r
.33958 .29704 .34433 .30212 .35194 .29928 .34721 .2942 Metetra
.666 .742 .907 r
.34721 .2942 .35194 .29928 .35958 .29649 .35486 .29139 Metetra
.662 .741 .908 r
.35486 .29139 .35958 .29649 .36723 .29376 .36252 .28864 Metetra
.658 .739 .91 r
.36252 .28864 .36723 .29376 .3749 .29113 .37021 .28598 Metetra
.656 .738 .91 r
.37021 .28598 .3749 .29113 .38259 .28854 .37791 .28334 Metetra
.665 .739 .906 r
.37791 .28334 .38259 .28854 .39031 .28573 .38565 .28051 Metetra
.673 .742 .902 r
.38565 .28051 .39031 .28573 .39806 .28272 .39342 .27751 Metetra
.673 .742 .903 r
.39342 .27751 .39806 .28272 .40584 .27972 .4012 .2745 Metetra
.669 .742 .905 r
.4012 .2745 .40584 .27972 .41363 .2768 .40901 .27156 Metetra
.665 .74 .906 r
.40901 .27156 .41363 .2768 .42144 .27396 .41683 .26871 Metetra
.659 .738 .909 r
.41683 .26871 .42144 .27396 .42927 .27124 .42467 .26597 Metetra
.651 .736 .911 r
.42467 .26597 .42927 .27124 .43711 .26865 .43254 .26336 Metetra
.642 .732 .914 r
.43254 .26336 .43711 .26865 .44498 .26625 .44042 .26093 Metetra
.627 .726 .918 r
.44042 .26093 .44498 .26625 .45287 .26412 .44832 .25875 Metetra
.603 .715 .924 r
.44832 .25875 .45287 .26412 .46077 .26246 .45624 .257 Metetra
.574 .7 .927 r
.45624 .257 .46077 .26246 .46869 .26136 .46417 .2557 Metetra
.601 .703 .916 r
.46417 .2557 .46869 .26136 .47665 .25978 .47215 .25398 Metetra
.667 .724 .891 r
.47215 .25398 .47665 .25978 .48466 .25666 .48017 .25097 Metetra
.686 .735 .888 r
.48017 .25097 .48466 .25666 .4927 .25313 .48822 .24753 Metetra
.681 .737 .893 r
.48822 .24753 .4927 .25313 .50076 .24978 .49629 .24421 Metetra
.673 .737 .898 r
.49629 .24421 .50076 .24978 .50883 .24664 .50439 .24107 Metetra
.669 .736 .901 r
.50439 .24107 .50883 .24664 .51693 .2436 .5125 .23802 Metetra
.673 .737 .899 r
.5125 .23802 .51693 .2436 .52506 .24047 .52064 .23488 Metetra
.692 .744 .891 r
.52064 .23488 .52506 .24047 .5332 .23681 .5288 .23128 Metetra
.692 .747 .894 r
.5288 .23128 .5332 .23681 .54136 .23304 .53698 .22768 Metetra
.571 .709 .935 r
.53698 .22768 .54136 .23304 .54957 .23196 .5452 .22634 Metetra
.51 .669 .935 r
.5452 .22634 .54957 .23196 .55782 .23184 .55346 .22594 Metetra
.478 .642 .928 r
.55346 .22594 .55782 .23184 .56612 .23233 .56176 .22605 Metetra
.483 .629 .916 r
.56176 .22605 .56612 .23233 .57446 .23311 .57011 .2262 Metetra
.655 .686 .867 r
.57011 .2262 .57446 .23311 .58279 .23038 .57844 .22327 Metetra
.779 .726 .803 r
.57844 .22327 .58279 .23038 .59103 .22265 .58672 .21615 Metetra
.795 .749 .81 r
.58672 .21615 .59103 .22265 .59927 .21463 .59499 .20872 Metetra
.756 .759 .855 r
.59499 .20872 .59927 .21463 .60756 .20886 .60331 .20317 Metetra
.712 .755 .885 r
.60331 .20317 .60756 .20886 .61591 .20461 .61168 .19898 Metetra
.688 .749 .898 r
.61168 .19898 .61591 .20461 .6243 .20102 .62009 .1954 Metetra
.676 .746 .903 r
.62009 .1954 .6243 .20102 .63273 .19771 .62853 .19207 Metetra
.67 .744 .905 r
.62853 .19207 .63273 .19771 .64118 .19451 .637 .18886 Metetra
.668 .743 .907 r
.637 .18886 .64118 .19451 .64966 .19137 .64549 .1857 Metetra
.666 .742 .907 r
.64549 .1857 .64966 .19137 .65816 .18825 .65402 .18257 Metetra
.665 .742 .907 r
.65402 .18257 .65816 .18825 .66669 .18514 .66257 .17944 Metetra
.665 .742 .908 r
.66257 .17944 .66669 .18514 .67525 .18204 .67114 .17632 Metetra
.664 .742 .908 r
.67114 .17632 .67525 .18204 .68384 .17893 .67975 .17319 Metetra
.664 .741 .908 r
.67975 .17319 .68384 .17893 .69245 .17582 .68837 .17006 Metetra
.664 .741 .908 r
.68837 .17006 .69245 .17582 .70108 .1727 .69703 .16693 Metetra
.664 .741 .908 r
.69703 .16693 .70108 .1727 .70975 .16957 .70571 .16378 Metetra
.664 .741 .908 r
.70571 .16378 .70975 .16957 .71844 .16643 .71442 .16063 Metetra
.664 .741 .908 r
.71442 .16063 .71844 .16643 .72716 .16329 .72315 .15747 Metetra
.664 .741 .908 r
.72315 .15747 .72716 .16329 .7359 .16014 .73192 .1543 Metetra
.664 .741 .908 r
.73192 .1543 .7359 .16014 .74467 .15698 .74071 .15112 Metetra
.664 .741 .908 r
.74071 .15112 .74467 .15698 .75347 .1538 .74952 .14793 Metetra
.664 .741 .908 r
.74952 .14793 .75347 .1538 .76229 .15062 .75837 .14473 Metetra
.664 .741 .908 r
.75837 .14473 .76229 .15062 .77114 .14743 .76724 .14152 Metetra
.664 .741 .908 r
.76724 .14152 .77114 .14743 .78002 .14423 .77613 .1383 Metetra
.663 .737 .905 r
.14365 .36141 .14873 .36625 .15582 .36366 .15075 .35884 Metetra
.662 .738 .906 r
.15075 .35884 .15582 .36366 .16292 .3611 .15786 .35629 Metetra
.661 .739 .907 r
.15786 .35629 .16292 .3611 .17004 .35854 .16499 .35374 Metetra
.661 .739 .908 r
.16499 .35374 .17004 .35854 .17718 .356 .17215 .35119 Metetra
.66 .739 .908 r
.17215 .35119 .17718 .356 .18434 .35346 .17932 .34864 Metetra
.66 .739 .909 r
.17932 .34864 .18434 .35346 .19152 .35093 .18651 .3461 Metetra
.659 .739 .909 r
.18651 .3461 .19152 .35093 .19872 .34841 .19371 .34356 Metetra
.658 .739 .91 r
.19371 .34356 .19872 .34841 .20594 .34591 .20094 .34104 Metetra
.656 .738 .91 r
.20094 .34104 .20594 .34591 .21317 .34342 .20819 .33854 Metetra
.654 .738 .911 r
.20819 .33854 .21317 .34342 .22042 .34097 .21545 .33607 Metetra
.652 .737 .912 r
.21545 .33607 .22042 .34097 .22769 .33856 .22273 .33365 Metetra
.647 .735 .913 r
.22273 .33365 .22769 .33856 .23497 .33623 .23002 .33129 Metetra
.641 .733 .916 r
.23002 .33129 .23497 .33623 .24226 .33401 .23732 .32904 Metetra
.63 .729 .919 r
.23732 .32904 .24226 .33401 .24955 .33198 .24464 .32697 Metetra
.616 .723 .922 r
.24464 .32697 .24955 .33198 .25685 .33023 .25195 .32513 Metetra
.608 .717 .922 r
.25195 .32513 .25685 .33023 .26416 .32866 .25928 .32342 Metetra
.646 .724 .905 r
.25928 .32342 .26416 .32866 .27155 .32636 .26668 .32104 Metetra
.7 .737 .879 r
.26668 .32104 .27155 .32636 .27906 .3227 .2742 .31749 Metetra
.711 .744 .878 r
.2742 .31749 .27906 .3227 .28662 .3188 .28176 .31368 Metetra
.701 .746 .887 r
.28176 .31368 .28662 .3188 .29416 .31522 .28931 .31014 Metetra
.69 .746 .894 r
.28931 .31014 .29416 .31522 .3017 .31192 .29687 .30686 Metetra
.683 .745 .898 r
.29687 .30686 .3017 .31192 .30925 .30879 .30443 .30373 Metetra
.677 .744 .901 r
.30443 .30373 .30925 .30879 .31681 .30577 .312 .3007 Metetra
.674 .744 .903 r
.312 .3007 .31681 .30577 .32438 .30282 .31959 .29774 Metetra
.672 .743 .904 r
.31959 .29774 .32438 .30282 .33197 .29991 .32719 .29482 Metetra
.67 .743 .905 r
.32719 .29482 .33197 .29991 .33958 .29704 .33481 .29194 Metetra
.668 .742 .906 r
.33481 .29194 .33958 .29704 .34721 .2942 .34245 .28908 Metetra
.666 .742 .907 r
.34245 .28908 .34721 .2942 .35486 .29139 .35012 .28626 Metetra
.662 .741 .908 r
.35012 .28626 .35486 .29139 .36252 .28864 .3578 .2835 Metetra
.658 .739 .91 r
.3578 .2835 .36252 .28864 .37021 .28598 .36549 .2808 Metetra
.657 .738 .91 r
.36549 .2808 .37021 .28598 .37791 .28334 .37321 .27813 Metetra
.665 .74 .906 r
.37321 .27813 .37791 .28334 .38565 .28051 .38096 .27528 Metetra
.673 .742 .903 r
.38096 .27528 .38565 .28051 .39342 .27751 .38874 .27227 Metetra
.672 .742 .903 r
.38874 .27227 .39342 .27751 .4012 .2745 .39654 .26926 Metetra
.669 .742 .905 r
.39654 .26926 .4012 .2745 .40901 .27156 .40436 .26631 Metetra
.665 .74 .906 r
.40436 .26631 .40901 .27156 .41683 .26871 .4122 .26343 Metetra
.659 .738 .909 r
.4122 .26343 .41683 .26871 .42467 .26597 .42006 .26068 Metetra
.652 .736 .911 r
.42006 .26068 .42467 .26597 .43254 .26336 .42794 .25805 Metetra
.643 .733 .914 r
.42794 .25805 .43254 .26336 .44042 .26093 .43583 .25558 Metetra
.629 .727 .918 r
.43583 .25558 .44042 .26093 .44832 .25875 .44375 .25336 Metetra
.607 .717 .923 r
.44375 .25336 .44832 .25875 .45624 .257 .45168 .25151 Metetra
.584 .704 .926 r
.45168 .25151 .45624 .257 .46417 .2557 .45963 .25005 Metetra
.607 .707 .915 r
.45963 .25005 .46417 .2557 .47215 .25398 .46762 .2482 Metetra
.663 .724 .894 r
.46762 .2482 .47215 .25398 .48017 .25097 .47566 .24527 Metetra
.682 .734 .89 r
.47566 .24527 .48017 .25097 .48822 .24753 .48372 .24191 Metetra
.679 .737 .895 r
.48372 .24191 .48822 .24753 .49629 .24421 .49181 .23863 Metetra
.672 .737 .899 r
.49181 .23863 .49629 .24421 .50439 .24107 .49992 .23549 Metetra
.669 .736 .901 r
.49992 .23549 .50439 .24107 .5125 .23802 .50804 .23242 Metetra
.672 .738 .899 r
.50804 .23242 .5125 .23802 .52064 .23488 .5162 .22926 Metetra
.689 .743 .893 r
.5162 .22926 .52064 .23488 .5288 .23128 .52438 .22572 Metetra
.685 .745 .897 r
.52438 .22572 .5288 .23128 .53698 .22768 .53257 .22226 Metetra
.585 .713 .932 r
.53257 .22226 .53698 .22768 .5452 .22634 .5408 .2207 Metetra
.527 .678 .934 r
.5408 .2207 .5452 .22634 .55346 .22594 .54907 .22004 Metetra
.502 .655 .928 r
.54907 .22004 .55346 .22594 .56176 .22605 .55738 .2198 Metetra
.517 .648 .916 r
.55738 .2198 .56176 .22605 .57011 .2262 .56573 .21943 Metetra
.661 .695 .87 r
.56573 .21943 .57011 .2262 .57844 .22327 .57408 .21634 Metetra
.77 .73 .815 r
.57408 .21634 .57844 .22327 .58672 .21615 .58238 .20973 Metetra
.785 .75 .821 r
.58238 .20973 .58672 .21615 .59499 .20872 .59069 .20281 Metetra
.75 .758 .859 r
.59069 .20281 .59499 .20872 .60331 .20317 .59903 .19747 Metetra
.71 .754 .887 r
.59903 .19747 .60331 .20317 .61168 .19898 .60742 .19332 Metetra
.687 .749 .898 r
.60742 .19332 .61168 .19898 .62009 .1954 .61585 .18974 Metetra
.676 .745 .903 r
.61585 .18974 .62009 .1954 .62853 .19207 .62431 .1864 Metetra
.67 .744 .905 r
.62431 .1864 .62853 .19207 .637 .18886 .63279 .18318 Metetra
.667 .743 .907 r
.63279 .18318 .637 .18886 .64549 .1857 .64131 .18 Metetra
.666 .742 .907 r
.64131 .18 .64549 .1857 .65402 .18257 .64985 .17685 Metetra
.665 .742 .907 r
.64985 .17685 .65402 .18257 .66257 .17944 .65842 .17371 Metetra
.665 .742 .908 r
.65842 .17371 .66257 .17944 .67114 .17632 .66701 .17057 Metetra
.664 .742 .908 r
.66701 .17057 .67114 .17632 .67975 .17319 .67563 .16743 Metetra
.664 .741 .908 r
.67563 .16743 .67975 .17319 .68837 .17006 .68428 .16428 Metetra
.664 .741 .908 r
.68428 .16428 .68837 .17006 .69703 .16693 .69295 .16112 Metetra
.664 .741 .908 r
.69295 .16112 .69703 .16693 .70571 .16378 .70165 .15796 Metetra
.664 .741 .908 r
.70165 .15796 .70571 .16378 .71442 .16063 .71038 .15479 Metetra
.664 .741 .908 r
.71038 .15479 .71442 .16063 .72315 .15747 .71913 .15161 Metetra
.664 .741 .908 r
.71913 .15161 .72315 .15747 .73192 .1543 .72791 .14842 Metetra
.664 .741 .908 r
.72791 .14842 .73192 .1543 .74071 .15112 .73672 .14522 Metetra
.664 .741 .908 r
.73672 .14522 .74071 .15112 .74952 .14793 .74555 .14202 Metetra
.664 .741 .908 r
.74555 .14202 .74952 .14793 .75837 .14473 .75442 .1388 Metetra
.664 .741 .908 r
.75442 .1388 .75837 .14473 .76724 .14152 .76331 .13557 Metetra
.664 .741 .908 r
.76331 .13557 .76724 .14152 .77613 .1383 .77223 .13234 Metetra
.661 .737 .906 r
.13855 .35656 .14365 .36141 .15075 .35884 .14565 .35401 Metetra
.661 .738 .907 r
.14565 .35401 .15075 .35884 .15786 .35629 .15278 .35146 Metetra
.661 .739 .908 r
.15278 .35146 .15786 .35629 .16499 .35374 .15992 .3489 Metetra
.661 .739 .908 r
.15992 .3489 .16499 .35374 .17215 .35119 .16708 .34635 Metetra
.66 .739 .908 r
.16708 .34635 .17215 .35119 .17932 .34864 .17426 .34379 Metetra
.66 .739 .909 r
.17426 .34379 .17932 .34864 .18651 .3461 .18146 .34123 Metetra
.659 .739 .909 r
.18146 .34123 .18651 .3461 .19371 .34356 .18868 .33869 Metetra
.658 .739 .91 r
.18868 .33869 .19371 .34356 .20094 .34104 .19592 .33615 Metetra
.657 .739 .91 r
.19592 .33615 .20094 .34104 .20819 .33854 .20318 .33364 Metetra
.655 .738 .911 r
.20318 .33364 .20819 .33854 .21545 .33607 .21045 .33115 Metetra
.652 .737 .912 r
.21045 .33115 .21545 .33607 .22273 .33365 .21774 .32871 Metetra
.648 .736 .913 r
.21774 .32871 .22273 .33365 .23002 .33129 .22505 .32633 Metetra
.642 .733 .915 r
.22505 .32633 .23002 .33129 .23732 .32904 .23237 .32405 Metetra
.632 .73 .918 r
.23237 .32405 .23732 .32904 .24464 .32697 .23969 .32193 Metetra
.62 .724 .921 r
.23969 .32193 .24464 .32697 .25195 .32513 .24702 .32001 Metetra
.615 .72 .921 r
.24702 .32001 .25195 .32513 .25928 .32342 .25437 .31817 Metetra
.649 .726 .905 r
.25437 .31817 .25928 .32342 .26668 .32104 .26179 .31573 Metetra
.696 .737 .883 r
.26179 .31573 .26668 .32104 .2742 .31749 .26932 .31226 Metetra
.707 .744 .881 r
.26932 .31226 .2742 .31749 .28176 .31368 .27688 .30855 Metetra
.699 .746 .888 r
.27688 .30855 .28176 .31368 .28931 .31014 .28444 .30504 Metetra
.689 .746 .895 r
.28444 .30504 .28931 .31014 .29687 .30686 .29201 .30177 Metetra
.682 .745 .899 r
.29201 .30177 .29687 .30686 .30443 .30373 .29958 .29864 Metetra
.677 .744 .901 r
.29958 .29864 .30443 .30373 .312 .3007 .30716 .2956 Metetra
.674 .743 .903 r
.30716 .2956 .312 .3007 .31959 .29774 .31476 .29263 Metetra
.671 .743 .904 r
.31476 .29263 .31959 .29774 .32719 .29482 .32238 .28971 Metetra
.669 .743 .905 r
.32238 .28971 .32719 .29482 .33481 .29194 .33002 .28681 Metetra
.668 .742 .906 r
.33002 .28681 .33481 .29194 .34245 .28908 .33767 .28394 Metetra
.666 .742 .907 r
.33767 .28394 .34245 .28908 .35012 .28626 .34535 .28111 Metetra
.663 .741 .908 r
.34535 .28111 .35012 .28626 .3578 .2835 .35304 .27832 Metetra
.659 .74 .909 r
.35304 .27832 .3578 .2835 .36549 .2808 .36075 .2756 Metetra
.658 .739 .909 r
.36075 .2756 .36549 .2808 .37321 .27813 .36849 .27288 Metetra
.665 .74 .906 r
.36849 .27288 .37321 .27813 .38096 .27528 .37625 .27002 Metetra
.672 .742 .903 r
.37625 .27002 .38096 .27528 .38874 .27227 .38405 .26701 Metetra
.672 .742 .903 r
.38405 .26701 .38874 .27227 .39654 .26926 .39186 .26399 Metetra
.669 .742 .905 r
.39186 .26399 .39654 .26926 .40436 .26631 .39969 .26102 Metetra
.665 .741 .906 r
.39969 .26102 .40436 .26631 .4122 .26343 .40755 .25813 Metetra
.659 .738 .909 r
.40755 .25813 .4122 .26343 .42006 .26068 .41542 .25535 Metetra
.652 .736 .911 r
.41542 .25535 .42006 .26068 .42794 .25805 .42331 .25271 Metetra
.644 .733 .914 r
.42331 .25271 .42794 .25805 .43583 .25558 .43122 .25021 Metetra
.631 .728 .917 r
.43122 .25021 .43583 .25558 .44375 .25336 .43915 .24794 Metetra
.611 .719 .922 r
.43915 .24794 .44375 .25336 .45168 .25151 .44709 .24601 Metetra
.592 .708 .924 r
.44709 .24601 .45168 .25151 .45963 .25005 .45506 .24439 Metetra
.612 .711 .915 r
.45506 .24439 .45963 .25005 .46762 .2482 .46307 .24244 Metetra
.66 .725 .897 r
.46307 .24244 .46762 .2482 .47566 .24527 .47112 .23956 Metetra
.679 .734 .892 r
.47112 .23956 .47566 .24527 .48372 .24191 .4792 .23627 Metetra
.677 .737 .896 r
.4792 .23627 .48372 .24191 .49181 .23863 .4873 .23302 Metetra
.672 .737 .899 r
.4873 .23302 .49181 .23863 .49992 .23549 .49542 .22988 Metetra
.669 .737 .901 r
.49542 .22988 .49992 .23549 .50804 .23242 .50356 .2268 Metetra
.672 .738 .9 r
.50356 .2268 .50804 .23242 .5162 .22926 .51173 .22363 Metetra
.686 .743 .894 r
.51173 .22363 .5162 .22926 .52438 .22572 .51993 .22014 Metetra
.68 .744 .899 r
.51993 .22014 .52438 .22572 .53257 .22226 .52814 .21679 Metetra
.595 .717 .929 r
.52814 .21679 .53257 .22226 .5408 .2207 .53638 .21504 Metetra
.543 .686 .932 r
.53638 .21504 .5408 .2207 .54907 .22004 .54467 .21414 Metetra
.523 .666 .927 r
.54467 .21414 .54907 .22004 .55738 .2198 .55299 .21358 Metetra
.543 .663 .915 r
.55299 .21358 .55738 .2198 .56573 .21943 .56134 .21277 Metetra
.666 .702 .873 r
.56134 .21277 .56573 .21943 .57408 .21634 .5697 .20956 Metetra
.762 .732 .826 r
.5697 .20956 .57408 .21634 .58238 .20973 .57803 .20338 Metetra
.775 .75 .83 r
.57803 .20338 .58238 .20973 .59069 .20281 .58636 .19689 Metetra
.744 .757 .863 r
.58636 .19689 .59069 .20281 .59903 .19747 .59473 .19173 Metetra
.707 .753 .888 r
.59473 .19173 .59903 .19747 .60742 .19332 .60314 .18763 Metetra
.686 .748 .899 r
.60314 .18763 .60742 .19332 .61585 .18974 .61159 .18405 Metetra
.675 .745 .903 r
.61159 .18405 .61585 .18974 .62431 .1864 .62006 .1807 Metetra
.67 .744 .906 r
.62006 .1807 .62431 .1864 .63279 .18318 .62857 .17746 Metetra
.667 .743 .907 r
.62857 .17746 .63279 .18318 .64131 .18 .6371 .17427 Metetra
.666 .742 .907 r
.6371 .17427 .64131 .18 .64985 .17685 .64566 .1711 Metetra
.665 .742 .907 r
.64566 .1711 .64985 .17685 .65842 .17371 .65424 .16795 Metetra
.665 .742 .908 r
.65424 .16795 .65842 .17371 .66701 .17057 .66285 .16479 Metetra
.664 .742 .908 r
.66285 .16479 .66701 .17057 .67563 .16743 .67149 .16163 Metetra
.664 .741 .908 r
.67149 .16163 .67563 .16743 .68428 .16428 .68016 .15846 Metetra
.664 .741 .908 r
.68016 .15846 .68428 .16428 .69295 .16112 .68885 .15529 Metetra
.664 .741 .908 r
.68885 .15529 .69295 .16112 .70165 .15796 .69757 .15211 Metetra
.664 .741 .908 r
.69757 .15211 .70165 .15796 .71038 .15479 .70631 .14892 Metetra
.664 .741 .908 r
.70631 .14892 .71038 .15479 .71913 .15161 .71508 .14572 Metetra
.664 .741 .908 r
.71508 .14572 .71913 .15161 .72791 .14842 .72388 .14251 Metetra
.664 .741 .908 r
.72388 .14251 .72791 .14842 .73672 .14522 .73271 .1393 Metetra
.664 .741 .908 r
.73271 .1393 .73672 .14522 .74555 .14202 .74157 .13607 Metetra
.664 .741 .908 r
.74157 .13607 .74555 .14202 .75442 .1388 .75045 .13284 Metetra
.664 .741 .908 r
.75045 .13284 .75442 .1388 .76331 .13557 .75936 .12959 Metetra
.664 .741 .908 r
.75936 .12959 .76331 .13557 .77223 .13234 .76829 .12634 Metetra
.66 .737 .907 r
.13342 .35169 .13855 .35656 .14565 .35401 .14053 .34915 Metetra
.66 .738 .907 r
.14053 .34915 .14565 .35401 .15278 .35146 .14767 .34661 Metetra
.66 .739 .908 r
.14767 .34661 .15278 .35146 .15992 .3489 .15482 .34405 Metetra
.66 .739 .908 r
.15482 .34405 .15992 .3489 .16708 .34635 .16199 .34148 Metetra
.66 .739 .909 r
.16199 .34148 .16708 .34635 .17426 .34379 .16918 .33891 Metetra
.66 .739 .909 r
.16918 .33891 .17426 .34379 .18146 .34123 .1764 .33635 Metetra
.659 .739 .909 r
.1764 .33635 .18146 .34123 .18868 .33869 .18363 .33379 Metetra
.658 .739 .91 r
.18363 .33379 .18868 .33869 .19592 .33615 .19088 .33124 Metetra
.657 .739 .91 r
.19088 .33124 .19592 .33615 .20318 .33364 .19815 .32871 Metetra
.655 .738 .911 r
.19815 .32871 .20318 .33364 .21045 .33115 .20543 .3262 Metetra
.652 .737 .912 r
.20543 .3262 .21045 .33115 .21774 .32871 .21273 .32374 Metetra
.648 .736 .913 r
.21273 .32374 .21774 .32871 .22505 .32633 .22005 .32134 Metetra
.643 .734 .915 r
.22005 .32134 .22505 .32633 .23237 .32405 .22738 .31903 Metetra
.634 .73 .917 r
.22738 .31903 .23237 .32405 .23969 .32193 .23472 .31687 Metetra
.623 .726 .92 r
.23472 .31687 .23969 .32193 .24702 .32001 .24207 .31488 Metetra
.621 .722 .919 r
.24207 .31488 .24702 .32001 .25437 .31817 .24943 .31292 Metetra
.652 .728 .905 r
.24943 .31292 .25437 .31817 .26179 .31573 .25687 .31042 Metetra
.693 .738 .885 r
.25687 .31042 .26179 .31573 .26932 .31226 .2644 .30703 Metetra
.703 .744 .883 r
.2644 .30703 .26932 .31226 .27688 .30855 .27197 .30339 Metetra
.697 .746 .889 r
.27197 .30339 .27688 .30855 .28444 .30504 .27955 .29992 Metetra
.688 .745 .895 r
.27955 .29992 .28444 .30504 .29201 .30177 .28712 .29665 Metetra
.681 .745 .899 r
.28712 .29665 .29201 .30177 .29958 .29864 .29471 .29352 Metetra
.677 .744 .901 r
.29471 .29352 .29958 .29864 .30716 .2956 .3023 .29048 Metetra
.674 .743 .903 r
.3023 .29048 .30716 .2956 .31476 .29263 .30992 .2875 Metetra
.671 .743 .904 r
.30992 .2875 .31476 .29263 .32238 .28971 .31755 .28456 Metetra
.669 .743 .905 r
.31755 .28456 .32238 .28971 .33002 .28681 .3252 .28165 Metetra
.668 .742 .906 r
.3252 .28165 .33002 .28681 .33767 .28394 .33287 .27877 Metetra
.665 .742 .907 r
.33287 .27877 .33767 .28394 .34535 .28111 .34055 .27592 Metetra
.663 .741 .908 r
.34055 .27592 .34535 .28111 .35304 .27832 .34826 .27312 Metetra
.66 .74 .909 r
.34826 .27312 .35304 .27832 .36075 .2756 .35599 .27037 Metetra
.659 .739 .909 r
.35599 .27037 .36075 .2756 .36849 .27288 .36374 .26762 Metetra
.666 .74 .906 r
.36374 .26762 .36849 .27288 .37625 .27002 .37152 .26473 Metetra
.671 .742 .903 r
.37152 .26473 .37625 .27002 .38405 .26701 .37932 .26172 Metetra
.672 .742 .904 r
.37932 .26172 .38405 .26701 .39186 .26399 .38715 .25869 Metetra
.669 .742 .905 r
.38715 .25869 .39186 .26399 .39969 .26102 .395 .25571 Metetra
.665 .741 .906 r
.395 .25571 .39969 .26102 .40755 .25813 .40287 .2528 Metetra
.659 .739 .909 r
.40287 .2528 .40755 .25813 .41542 .25535 .41075 .25001 Metetra
.652 .736 .911 r
.41075 .25001 .41542 .25535 .42331 .25271 .41866 .24734 Metetra
.644 .733 .914 r
.41866 .24734 .42331 .25271 .43122 .25021 .42658 .24482 Metetra
.633 .729 .917 r
.42658 .24482 .43122 .25021 .43915 .24794 .43453 .2425 Metetra
.615 .721 .921 r
.43453 .2425 .43915 .24794 .44709 .24601 .44249 .24048 Metetra
.599 .712 .923 r
.44249 .24048 .44709 .24601 .45506 .24439 .45047 .23873 Metetra
.617 .713 .915 r
.45047 .23873 .45506 .24439 .46307 .24244 .45849 .23669 Metetra
.658 .726 .899 r
.45849 .23669 .46307 .24244 .47112 .23956 .46655 .23384 Metetra
.675 .734 .894 r
.46655 .23384 .47112 .23956 .4792 .23627 .47465 .23061 Metetra
.675 .737 .897 r
.47465 .23061 .4792 .23627 .4873 .23302 .48276 .22738 Metetra
.671 .737 .9 r
.48276 .22738 .4873 .23302 .49542 .22988 .4909 .22425 Metetra
.668 .737 .901 r
.4909 .22425 .49542 .22988 .50356 .2268 .49906 .22115 Metetra
.672 .738 .9 r
.49906 .22115 .50356 .2268 .51173 .22363 .50724 .21797 Metetra
.683 .742 .896 r
.50724 .21797 .51173 .22363 .51993 .22014 .51545 .21453 Metetra
.676 .742 .901 r
.51545 .21453 .51993 .22014 .52814 .21679 .52368 .21126 Metetra
.604 .719 .926 r
.52368 .21126 .52814 .21679 .53638 .21504 .53194 .20936 Metetra
.557 .693 .93 r
.53194 .20936 .53638 .21504 .54467 .21414 .54023 .20823 Metetra
.541 .676 .926 r
.54023 .20823 .54467 .21414 .55299 .21358 .54856 .20738 Metetra
.564 .676 .914 r
.54856 .20738 .55299 .21358 .56134 .21277 .55693 .2062 Metetra
.67 .707 .876 r
.55693 .2062 .56134 .21277 .5697 .20956 .5653 .2029 Metetra
.754 .735 .835 r
.5653 .2029 .5697 .20956 .57803 .20338 .57365 .19708 Metetra
.767 .751 .839 r
.57365 .19708 .57803 .20338 .58636 .19689 .58201 .19096 Metetra
.738 .756 .867 r
.58201 .19096 .58636 .19689 .59473 .19173 .59041 .18596 Metetra
.705 .753 .889 r
.59041 .18596 .59473 .19173 .60314 .18763 .59884 .1819 Metetra
.685 .748 .899 r
.59884 .1819 .60314 .18763 .61159 .18405 .6073 .17833 Metetra
.675 .745 .904 r
.6073 .17833 .61159 .18405 .62006 .1807 .61579 .17497 Metetra
.67 .744 .906 r
.61579 .17497 .62006 .1807 .62857 .17746 .62431 .17172 Metetra
.667 .743 .907 r
.62431 .17172 .62857 .17746 .6371 .17427 .63286 .16851 Metetra
.666 .742 .907 r
.63286 .16851 .6371 .17427 .64566 .1711 .64144 .16532 Metetra
.665 .742 .907 r
.64144 .16532 .64566 .1711 .65424 .16795 .65004 .16215 Metetra
.665 .742 .908 r
.65004 .16215 .65424 .16795 .66285 .16479 .65867 .15897 Metetra
.664 .742 .908 r
.65867 .15897 .66285 .16479 .67149 .16163 .66733 .15579 Metetra
.664 .741 .908 r
.66733 .15579 .67149 .16163 .68016 .15846 .67601 .15261 Metetra
.664 .741 .908 r
.67601 .15261 .68016 .15846 .68885 .15529 .68472 .14942 Metetra
.664 .741 .908 r
.68472 .14942 .68885 .15529 .69757 .15211 .69346 .14622 Metetra
.664 .741 .908 r
.69346 .14622 .69757 .15211 .70631 .14892 .70222 .14301 Metetra
.664 .741 .908 r
.70222 .14301 .70631 .14892 .71508 .14572 .71101 .1398 Metetra
.664 .741 .908 r
.71101 .1398 .71508 .14572 .72388 .14251 .71983 .13657 Metetra
.664 .741 .908 r
.71983 .13657 .72388 .14251 .73271 .1393 .72868 .13334 Metetra
.664 .741 .908 r
.72868 .13334 .73271 .1393 .74157 .13607 .73755 .13009 Metetra
.664 .741 .908 r
.73755 .13009 .74157 .13607 .75045 .13284 .74645 .12684 Metetra
.664 .741 .908 r
.74645 .12684 .75045 .13284 .75936 .12959 .75538 .12357 Metetra
.664 .741 .908 r
.75538 .12357 .75936 .12959 .76829 .12634 .76434 .1203 Metetra
.659 .737 .907 r
.12827 .3468 .13342 .35169 .14053 .34915 .13539 .34427 Metetra
.659 .738 .908 r
.13539 .34427 .14053 .34915 .14767 .34661 .14253 .34173 Metetra
.66 .739 .908 r
.14253 .34173 .14767 .34661 .15482 .34405 .14969 .33917 Metetra
.66 .739 .908 r
.14969 .33917 .15482 .34405 .16199 .34148 .15687 .33659 Metetra
.66 .739 .909 r
.15687 .33659 .16199 .34148 .16918 .33891 .16408 .33402 Metetra
.66 .739 .909 r
.16408 .33402 .16918 .33891 .1764 .33635 .1713 .33144 Metetra
.659 .739 .909 r
.1713 .33144 .1764 .33635 .18363 .33379 .17854 .32886 Metetra
.658 .739 .91 r
.17854 .32886 .18363 .33379 .19088 .33124 .18581 .3263 Metetra
.657 .739 .91 r
.18581 .3263 .19088 .33124 .19815 .32871 .19309 .32375 Metetra
.655 .738 .911 r
.19309 .32375 .19815 .32871 .20543 .3262 .20038 .32123 Metetra
.652 .737 .912 r
.20038 .32123 .20543 .3262 .21273 .32374 .2077 .31875 Metetra
.649 .736 .913 r
.2077 .31875 .21273 .32374 .22005 .32134 .21503 .31632 Metetra
.644 .734 .915 r
.21503 .31632 .22005 .32134 .22738 .31903 .22237 .31399 Metetra
.636 .731 .917 r
.22237 .31399 .22738 .31903 .23472 .31687 .22972 .31178 Metetra
.627 .727 .919 r
.22972 .31178 .23472 .31687 .24207 .31488 .23708 .30972 Metetra
.626 .724 .917 r
.23708 .30972 .24207 .31488 .24943 .31292 .24447 .30767 Metetra
.654 .729 .905 r
.24447 .30767 .24943 .31292 .25687 .31042 .25192 .30511 Metetra
.69 .738 .888 r
.25192 .30511 .25687 .31042 .2644 .30703 .25946 .30178 Metetra
.7 .743 .885 r
.25946 .30178 .2644 .30703 .27197 .30339 .26704 .29821 Metetra
.695 .745 .89 r
.26704 .29821 .27197 .30339 .27955 .29992 .27462 .29477 Metetra
.687 .745 .896 r
.27462 .29477 .27955 .29992 .28712 .29665 .28221 .29151 Metetra
.681 .744 .899 r
.28221 .29151 .28712 .29665 .29471 .29352 .28981 .28837 Metetra
.676 .744 .902 r
.28981 .28837 .29471 .29352 .3023 .29048 .29742 .28533 Metetra
.673 .743 .903 r
.29742 .28533 .3023 .29048 .30992 .2875 .30504 .28234 Metetra
.671 .743 .904 r
.30504 .28234 .30992 .2875 .31755 .28456 .31269 .27939 Metetra
.669 .743 .905 r
.31269 .27939 .31755 .28456 .3252 .28165 .32035 .27647 Metetra
.667 .742 .906 r
.32035 .27647 .3252 .28165 .33287 .27877 .32803 .27357 Metetra
.665 .742 .907 r
.32803 .27357 .33287 .27877 .34055 .27592 .33574 .27071 Metetra
.663 .741 .908 r
.33574 .27071 .34055 .27592 .34826 .27312 .34346 .26789 Metetra
.66 .74 .909 r
.34346 .26789 .34826 .27312 .35599 .27037 .3512 .26511 Metetra
.66 .739 .909 r
.3512 .26511 .35599 .27037 .36374 .26762 .35896 .26233 Metetra
.666 .74 .906 r
.35896 .26233 .36374 .26762 .37152 .26473 .36675 .25942 Metetra
.671 .742 .904 r
.36675 .25942 .37152 .26473 .37932 .26172 .37457 .2564 Metetra
.671 .742 .904 r
.37457 .2564 .37932 .26172 .38715 .25869 .38242 .25337 Metetra
.669 .742 .905 r
.38242 .25337 .38715 .25869 .395 .25571 .39028 .25037 Metetra
.665 .741 .906 r
.39028 .25037 .395 .25571 .40287 .2528 .39816 .24744 Metetra
.659 .739 .909 r
.39816 .24744 .40287 .2528 .41075 .25001 .40606 .24463 Metetra
.652 .736 .911 r
.40606 .24463 .41075 .25001 .41866 .24734 .41398 .24194 Metetra
.645 .734 .913 r
.41398 .24194 .41866 .24734 .42658 .24482 .42192 .23939 Metetra
.634 .729 .916 r
.42192 .23939 .42658 .24482 .43453 .2425 .42988 .23703 Metetra
.619 .722 .92 r
.42988 .23703 .43453 .2425 .44249 .24048 .43785 .23494 Metetra
.606 .715 .922 r
.43785 .23494 .44249 .24048 .45047 .23873 .44585 .23307 Metetra
.621 .716 .914 r
.44585 .23307 .45047 .23873 .45849 .23669 .45389 .23094 Metetra
.656 .726 .901 r
.45389 .23094 .45849 .23669 .46655 .23384 .46196 .22812 Metetra
.673 .734 .896 r
.46196 .22812 .46655 .23384 .47465 .23061 .47007 .22493 Metetra
.673 .737 .898 r
.47007 .22493 .47465 .23061 .48276 .22738 .4782 .22173 Metetra
.67 .737 .9 r
.4782 .22173 .48276 .22738 .4909 .22425 .48635 .21859 Metetra
.668 .737 .901 r
.48635 .21859 .4909 .22425 .49906 .22115 .49453 .21548 Metetra
.671 .738 .9 r
.49453 .21548 .49906 .22115 .50724 .21797 .50273 .2123 Metetra
.681 .742 .897 r
.50273 .2123 .50724 .21797 .51545 .21453 .51096 .2089 Metetra
.672 .741 .902 r
.51096 .2089 .51545 .21453 .52368 .21126 .5192 .20569 Metetra
.611 .721 .924 r
.5192 .20569 .52368 .21126 .53194 .20936 .52748 .20365 Metetra
.569 .698 .928 r
.52748 .20365 .53194 .20936 .54023 .20823 .53578 .20231 Metetra
.557 .684 .924 r
.53578 .20231 .54023 .20823 .54856 .20738 .54412 .2012 Metetra
.582 .685 .912 r
.54412 .2012 .54856 .20738 .55693 .2062 .55249 .19971 Metetra
.673 .712 .878 r
.55249 .19971 .55693 .2062 .5653 .2029 .56088 .19634 Metetra
.746 .737 .843 r
.56088 .19634 .5653 .2029 .57365 .19708 .56926 .19082 Metetra
.758 .751 .846 r
.56926 .19082 .57365 .19708 .58201 .19096 .57764 .18502 Metetra
.733 .755 .871 r
.57764 .18502 .58201 .19096 .59041 .18596 .58606 .18017 Metetra
.703 .752 .89 r
.58606 .18017 .59041 .18596 .59884 .1819 .59451 .17615 Metetra
.684 .748 .899 r
.59451 .17615 .59884 .1819 .6073 .17833 .60299 .17257 Metetra
.675 .745 .904 r
.60299 .17257 .6073 .17833 .61579 .17497 .6115 .16921 Metetra
.67 .744 .906 r
.6115 .16921 .61579 .17497 .62431 .17172 .62004 .16594 Metetra
.667 .743 .907 r
.62004 .16594 .62431 .17172 .63286 .16851 .62861 .16271 Metetra
.666 .742 .907 r
.62861 .16271 .63286 .16851 .64144 .16532 .6372 .15951 Metetra
.665 .742 .907 r
.6372 .15951 .64144 .16532 .65004 .16215 .64582 .15632 Metetra
.665 .742 .908 r
.64582 .15632 .65004 .16215 .65867 .15897 .65447 .15312 Metetra
.664 .742 .908 r
.65447 .15312 .65867 .15897 .66733 .15579 .66314 .14993 Metetra
.664 .741 .908 r
.66314 .14993 .66733 .15579 .67601 .15261 .67185 .14672 Metetra
.664 .741 .908 r
.67185 .14672 .67601 .15261 .68472 .14942 .68057 .14352 Metetra
.664 .741 .908 r
.68057 .14352 .68472 .14942 .69346 .14622 .68933 .1403 Metetra
.664 .741 .908 r
.68933 .1403 .69346 .14622 .70222 .14301 .69811 .13707 Metetra
.664 .741 .908 r
.69811 .13707 .70222 .14301 .71101 .1398 .70692 .13384 Metetra
.664 .741 .908 r
.70692 .13384 .71101 .1398 .71983 .13657 .71576 .1306 Metetra
.664 .741 .908 r
.71576 .1306 .71983 .13657 .72868 .13334 .72462 .12734 Metetra
.664 .741 .908 r
.72462 .12734 .72868 .13334 .73755 .13009 .73352 .12408 Metetra
.664 .741 .908 r
.73352 .12408 .73755 .13009 .74645 .12684 .74244 .12081 Metetra
.664 .741 .908 r
.74244 .12081 .74645 .12684 .75538 .12357 .75139 .11752 Metetra
.664 .741 .908 r
.75139 .11752 .75538 .12357 .76434 .1203 .76036 .11423 Metetra
.658 .737 .908 r
.12309 .3419 .12827 .3468 .13539 .34427 .13021 .33938 Metetra
.658 .738 .908 r
.13021 .33938 .13539 .34427 .14253 .34173 .13736 .33683 Metetra
.659 .739 .908 r
.13736 .33683 .14253 .34173 .14969 .33917 .14454 .33427 Metetra
.66 .739 .909 r
.14454 .33427 .14969 .33917 .15687 .33659 .15173 .33168 Metetra
.66 .739 .909 r
.15173 .33168 .15687 .33659 .16408 .33402 .15895 .32909 Metetra
.66 .739 .909 r
.15895 .32909 .16408 .33402 .1713 .33144 .16618 .3265 Metetra
.659 .739 .909 r
.16618 .3265 .1713 .33144 .17854 .32886 .17343 .32391 Metetra
.658 .739 .91 r
.17343 .32391 .17854 .32886 .18581 .3263 .18071 .32133 Metetra
.657 .739 .91 r
.18071 .32133 .18581 .3263 .19309 .32375 .188 .31877 Metetra
.655 .738 .911 r
.188 .31877 .19309 .32375 .20038 .32123 .19531 .31623 Metetra
.653 .737 .911 r
.19531 .31623 .20038 .32123 .2077 .31875 .20264 .31373 Metetra
.649 .736 .913 r
.20264 .31373 .2077 .31875 .21503 .31632 .20998 .31128 Metetra
.644 .734 .914 r
.20998 .31128 .21503 .31632 .22237 .31399 .21733 .30892 Metetra
.637 .731 .916 r
.21733 .30892 .22237 .31399 .22972 .31178 .2247 .30667 Metetra
.63 .728 .918 r
.2247 .30667 .22972 .31178 .23708 .30972 .23208 .30454 Metetra
.63 .726 .916 r
.23208 .30454 .23708 .30972 .24447 .30767 .23948 .3024 Metetra
.655 .73 .905 r
.23948 .3024 .24447 .30767 .25192 .30511 .24695 .2998 Metetra
.687 .738 .89 r
.24695 .2998 .25192 .30511 .25946 .30178 .25449 .29652 Metetra
.697 .743 .887 r
.25449 .29652 .25946 .30178 .26704 .29821 .26208 .293 Metetra
.693 .745 .891 r
.26208 .293 .26704 .29821 .27462 .29477 .26967 .28959 Metetra
.686 .745 .896 r
.26967 .28959 .27462 .29477 .28221 .29151 .27727 .28634 Metetra
.68 .744 .9 r
.27727 .28634 .28221 .29151 .28981 .28837 .28488 .2832 Metetra
.676 .744 .902 r
.28488 .2832 .28981 .28837 .29742 .28533 .29251 .28015 Metetra
.673 .743 .903 r
.29251 .28015 .29742 .28533 .30504 .28234 .30015 .27715 Metetra
.671 .743 .904 r
.30015 .27715 .30504 .28234 .31269 .27939 .3078 .27419 Metetra
.669 .742 .905 r
.3078 .27419 .31269 .27939 .32035 .27647 .31548 .27125 Metetra
.667 .742 .906 r
.31548 .27125 .32035 .27647 .32803 .27357 .32318 .26835 Metetra
.665 .742 .907 r
.32318 .26835 .32803 .27357 .33574 .27071 .33089 .26547 Metetra
.663 .741 .908 r
.33089 .26547 .33574 .27071 .34346 .26789 .33863 .26263 Metetra
.661 .74 .909 r
.33863 .26263 .34346 .26789 .3512 .26511 .34638 .25983 Metetra
.661 .74 .908 r
.34638 .25983 .3512 .26511 .35896 .26233 .35416 .25701 Metetra
.666 .741 .906 r
.35416 .25701 .35896 .26233 .36675 .25942 .36197 .25409 Metetra
.671 .742 .904 r
.36197 .25409 .36675 .25942 .37457 .2564 .3698 .25106 Metetra
.671 .742 .904 r
.3698 .25106 .37457 .2564 .38242 .25337 .37766 .24801 Metetra
.669 .742 .905 r
.37766 .24801 .38242 .25337 .39028 .25037 .38553 .245 Metetra
.665 .741 .906 r
.38553 .245 .39028 .25037 .39816 .24744 .39343 .24206 Metetra
.659 .739 .909 r
.39343 .24206 .39816 .24744 .40606 .24463 .40135 .23923 Metetra
.653 .736 .911 r
.40135 .23923 .40606 .24463 .41398 .24194 .40928 .23652 Metetra
.646 .734 .913 r
.40928 .23652 .41398 .24194 .42192 .23939 .41723 .23394 Metetra
.636 .73 .916 r
.41723 .23394 .42192 .23939 .42988 .23703 .42521 .23154 Metetra
.622 .724 .919 r
.42521 .23154 .42988 .23703 .43785 .23494 .4332 .22937 Metetra
.611 .717 .92 r
.4332 .22937 .43785 .23494 .44585 .23307 .44121 .2274 Metetra
.624 .718 .914 r
.44121 .2274 .44585 .23307 .45389 .23094 .44926 .22519 Metetra
.655 .727 .902 r
.44926 .22519 .45389 .23094 .46196 .22812 .45735 .22238 Metetra
.67 .734 .898 r
.45735 .22238 .46196 .22812 .47007 .22493 .46547 .21923 Metetra
.672 .736 .899 r
.46547 .21923 .47007 .22493 .4782 .22173 .47362 .21605 Metetra
.669 .737 .901 r
.47362 .21605 .4782 .22173 .48635 .21859 .48179 .21291 Metetra
.668 .737 .902 r
.48179 .21291 .48635 .21859 .49453 .21548 .48998 .20979 Metetra
.671 .738 .901 r
.48998 .20979 .49453 .21548 .50273 .2123 .49819 .2066 Metetra
.678 .741 .898 r
.49819 .2066 .50273 .2123 .51096 .2089 .50643 .20323 Metetra
.669 .74 .904 r
.50643 .20323 .51096 .2089 .5192 .20569 .5147 .20007 Metetra
.617 .723 .922 r
.5147 .20007 .5192 .20569 .52748 .20365 .52298 .19791 Metetra
.58 .703 .927 r
.52298 .19791 .52748 .20365 .53578 .20231 .5313 .19638 Metetra
.57 .691 .923 r
.5313 .19638 .53578 .20231 .54412 .2012 .53965 .19503 Metetra
.596 .693 .911 r
.53965 .19503 .54412 .2012 .55249 .19971 .54804 .19327 Metetra
.675 .716 .88 r
.54804 .19327 .55249 .19971 .56088 .19634 .55644 .18985 Metetra
.74 .738 .85 r
.55644 .18985 .56088 .19634 .56926 .19082 .56484 .18459 Metetra
.751 .751 .852 r
.56484 .18459 .56926 .19082 .57764 .18502 .57325 .17906 Metetra
.728 .755 .874 r
.57325 .17906 .57764 .18502 .58606 .18017 .58168 .17434 Metetra
.701 .751 .891 r
.58168 .17434 .58606 .18017 .59451 .17615 .59015 .17036 Metetra
.683 .747 .9 r
.59015 .17036 .59451 .17615 .60299 .17257 .59865 .16679 Metetra
.674 .745 .904 r
.59865 .16679 .60299 .17257 .6115 .16921 .60718 .16341 Metetra
.67 .743 .906 r
.60718 .16341 .6115 .16921 .62004 .16594 .61574 .16012 Metetra
.667 .743 .907 r
.61574 .16012 .62004 .16594 .62861 .16271 .62432 .15688 Metetra
.666 .742 .907 r
.62432 .15688 .62861 .16271 .6372 .15951 .63294 .15366 Metetra
.665 .742 .907 r
.63294 .15366 .6372 .15951 .64582 .15632 .64158 .15045 Metetra
.665 .742 .908 r
.64158 .15045 .64582 .15632 .65447 .15312 .65024 .14724 Metetra
.664 .742 .908 r
.65024 .14724 .65447 .15312 .66314 .14993 .65893 .14403 Metetra
.664 .741 .908 r
.65893 .14403 .66314 .14993 .67185 .14672 .66765 .14081 Metetra
.664 .741 .908 r
.66765 .14081 .67185 .14672 .68057 .14352 .6764 .13758 Metetra
.664 .741 .908 r
.6764 .13758 .68057 .14352 .68933 .1403 .68518 .13434 Metetra
.664 .741 .908 r
.68518 .13434 .68933 .1403 .69811 .13707 .69398 .1311 Metetra
.664 .741 .908 r
.69398 .1311 .69811 .13707 .70692 .13384 .70281 .12785 Metetra
.664 .741 .908 r
.70281 .12785 .70692 .13384 .71576 .1306 .71166 .12459 Metetra
.664 .741 .908 r
.71166 .12459 .71576 .1306 .72462 .12734 .72055 .12131 Metetra
.664 .741 .908 r
.72055 .12131 .72462 .12734 .73352 .12408 .72946 .11803 Metetra
.664 .741 .908 r
.72946 .11803 .73352 .12408 .74244 .12081 .7384 .11474 Metetra
.664 .741 .908 r
.7384 .11474 .74244 .12081 .75139 .11752 .74737 .11144 Metetra
.664 .741 .908 r
.74737 .11144 .75139 .11752 .76036 .11423 .75636 .10813 Metetra
.657 .737 .909 r
.11788 .33697 .12309 .3419 .13021 .33938 .12501 .33446 Metetra
.658 .738 .909 r
.12501 .33446 .13021 .33938 .13736 .33683 .13217 .33191 Metetra
.659 .739 .909 r
.13217 .33191 .13736 .33683 .14454 .33427 .13936 .32934 Metetra
.66 .739 .909 r
.13936 .32934 .14454 .33427 .15173 .33168 .14656 .32674 Metetra
.66 .739 .909 r
.14656 .32674 .15173 .33168 .15895 .32909 .15379 .32414 Metetra
.66 .739 .909 r
.15379 .32414 .15895 .32909 .16618 .3265 .16103 .32154 Metetra
.659 .739 .909 r
.16103 .32154 .16618 .3265 .17343 .32391 .1683 .31894 Metetra
.658 .739 .91 r
.1683 .31894 .17343 .32391 .18071 .32133 .17558 .31634 Metetra
.657 .739 .91 r
.17558 .31634 .18071 .32133 .188 .31877 .18289 .31376 Metetra
.655 .738 .911 r
.18289 .31376 .188 .31877 .19531 .31623 .19021 .31121 Metetra
.653 .737 .911 r
.19021 .31121 .19531 .31623 .20264 .31373 .19755 .30869 Metetra
.65 .736 .912 r
.19755 .30869 .20264 .31373 .20998 .31128 .2049 .30622 Metetra
.645 .735 .914 r
.2049 .30622 .20998 .31128 .21733 .30892 .21227 .30382 Metetra
.639 .732 .916 r
.21227 .30382 .21733 .30892 .2247 .30667 .21965 .30153 Metetra
.633 .729 .917 r
.21965 .30153 .2247 .30667 .23208 .30454 .22704 .29934 Metetra
.634 .727 .915 r
.22704 .29934 .23208 .30454 .23948 .3024 .23446 .29712 Metetra
.656 .731 .905 r
.23446 .29712 .23948 .3024 .24695 .2998 .24194 .29448 Metetra
.685 .738 .891 r
.24194 .29448 .24695 .2998 .25449 .29652 .2495 .29124 Metetra
.695 .743 .889 r
.2495 .29124 .25449 .29652 .26208 .293 .25709 .28777 Metetra
.691 .745 .892 r
.25709 .28777 .26208 .293 .26967 .28959 .2647 .28439 Metetra
.685 .745 .897 r
.2647 .28439 .26967 .28959 .27727 .28634 .27231 .28114 Metetra
.679 .744 .9 r
.27231 .28114 .27727 .28634 .28488 .2832 .27993 .278 Metetra
.676 .744 .902 r
.27993 .278 .28488 .2832 .29251 .28015 .28757 .27494 Metetra
.673 .743 .903 r
.28757 .27494 .29251 .28015 .30015 .27715 .29522 .27193 Metetra
.671 .743 .905 r
.29522 .27193 .30015 .27715 .3078 .27419 .30289 .26896 Metetra
.669 .742 .905 r
.30289 .26896 .3078 .27419 .31548 .27125 .31058 .26601 Metetra
.667 .742 .906 r
.31058 .26601 .31548 .27125 .32318 .26835 .31829 .26309 Metetra
.665 .742 .907 r
.31829 .26309 .32318 .26835 .33089 .26547 .32602 .2602 Metetra
.663 .741 .908 r
.32602 .2602 .33089 .26547 .33863 .26263 .33377 .25734 Metetra
.661 .74 .908 r
.33377 .25734 .33863 .26263 .34638 .25983 .34154 .25451 Metetra
.662 .74 .908 r
.34154 .25451 .34638 .25983 .35416 .25701 .34933 .25167 Metetra
.666 .741 .906 r
.34933 .25167 .35416 .25701 .36197 .25409 .35715 .24873 Metetra
.67 .742 .904 r
.35715 .24873 .36197 .25409 .3698 .25106 .365 .24569 Metetra
.671 .742 .904 r
.365 .24569 .3698 .25106 .37766 .24801 .37287 .24263 Metetra
.669 .742 .905 r
.37287 .24263 .37766 .24801 .38553 .245 .38076 .2396 Metetra
.666 .741 .906 r
.38076 .2396 .38553 .245 .39343 .24206 .38867 .23664 Metetra
.659 .739 .909 r
.38867 .23664 .39343 .24206 .40135 .23923 .3966 .23379 Metetra
.653 .737 .911 r
.3966 .23379 .40135 .23923 .40928 .23652 .40455 .23107 Metetra
.646 .734 .913 r
.40455 .23107 .40928 .23652 .41723 .23394 .41252 .22846 Metetra
.637 .73 .916 r
.41252 .22846 .41723 .23394 .42521 .23154 .42051 .22602 Metetra
.625 .725 .919 r
.42051 .22602 .42521 .23154 .4332 .22937 .42851 .22378 Metetra
.616 .719 .919 r
.42851 .22378 .4332 .22937 .44121 .2274 .43654 .22171 Metetra
.627 .72 .914 r
.43654 .22171 .44121 .2274 .44926 .22519 .44461 .21943 Metetra
.653 .728 .903 r
.44461 .21943 .44926 .22519 .45735 .22238 .45271 .21663 Metetra
.668 .734 .899 r
.45271 .21663 .45735 .22238 .46547 .21923 .46085 .21351 Metetra
.67 .736 .9 r
.46085 .21351 .46547 .21923 .47362 .21605 .46901 .21034 Metetra
.669 .737 .901 r
.46901 .21034 .47362 .21605 .48179 .21291 .47719 .2072 Metetra
.668 .737 .902 r
.47719 .2072 .48179 .21291 .48998 .20979 .4854 .20407 Metetra
.67 .738 .901 r
.4854 .20407 .48998 .20979 .49819 .2066 .49363 .20087 Metetra
.676 .741 .899 r
.49363 .20087 .49819 .2066 .50643 .20323 .50189 .19754 Metetra
.667 .74 .904 r
.50189 .19754 .50643 .20323 .5147 .20007 .51017 .19441 Metetra
.623 .725 .92 r
.51017 .19441 .5147 .20007 .52298 .19791 .51847 .19214 Metetra
.59 .707 .925 r
.51847 .19214 .52298 .19791 .5313 .19638 .5268 .19044 Metetra
.582 .697 .921 r
.5268 .19044 .5313 .19638 .53965 .19503 .53516 .18887 Metetra
.607 .7 .91 r
.53516 .18887 .53965 .19503 .54804 .19327 .54356 .18689 Metetra
.676 .72 .882 r
.54356 .18689 .54804 .19327 .55644 .18985 .55198 .18343 Metetra
.733 .739 .856 r
.55198 .18343 .55644 .18985 .56484 .18459 .5604 .17838 Metetra
.743 .751 .858 r
.5604 .17838 .56484 .18459 .57325 .17906 .56883 .17309 Metetra
.723 .754 .877 r
.56883 .17309 .57325 .17906 .58168 .17434 .57729 .16849 Metetra
.699 .751 .892 r
.57729 .16849 .58168 .17434 .59015 .17036 .58577 .16454 Metetra
.683 .747 .9 r
.58577 .16454 .59015 .17036 .59865 .16679 .59429 .16097 Metetra
.674 .745 .904 r
.59429 .16097 .59865 .16679 .60718 .16341 .60284 .15758 Metetra
.67 .743 .906 r
.60284 .15758 .60718 .16341 .61574 .16012 .61142 .15428 Metetra
.667 .743 .907 r
.61142 .15428 .61574 .16012 .62432 .15688 .62002 .15102 Metetra
.666 .742 .907 r
.62002 .15102 .62432 .15688 .63294 .15366 .62865 .14779 Metetra
.665 .742 .907 r
.62865 .14779 .63294 .15366 .64158 .15045 .63731 .14456 Metetra
.665 .742 .908 r
.63731 .14456 .64158 .15045 .65024 .14724 .64599 .14133 Metetra
.664 .742 .908 r
.64599 .14133 .65024 .14724 .65893 .14403 .6547 .13809 Metetra
.664 .741 .908 r
.6547 .13809 .65893 .14403 .66765 .14081 .66344 .13485 Metetra
.664 .741 .908 r
.66344 .13485 .66765 .14081 .6764 .13758 .6722 .13161 Metetra
.664 .741 .908 r
.6722 .13161 .6764 .13758 .68518 .13434 .681 .12836 Metetra
.664 .741 .908 r
.681 .12836 .68518 .13434 .69398 .1311 .68982 .12509 Metetra
.664 .741 .908 r
.68982 .12509 .69398 .1311 .70281 .12785 .69867 .12182 Metetra
.664 .741 .908 r
.69867 .12182 .70281 .12785 .71166 .12459 .70754 .11854 Metetra
.664 .741 .908 r
.70754 .11854 .71166 .12459 .72055 .12131 .71644 .11525 Metetra
.664 .741 .908 r
.71644 .11525 .72055 .12131 .72946 .11803 .72538 .11195 Metetra
.664 .741 .908 r
.72538 .11195 .72946 .11803 .7384 .11474 .73434 .10864 Metetra
.664 .741 .908 r
.73434 .10864 .7384 .11474 .74737 .11144 .74332 .10532 Metetra
.664 .741 .908 r
.74332 .10532 .74737 .11144 .75636 .10813 .75234 .10198 Metetra
.656 .737 .909 r
.11264 .33203 .11788 .33697 .12501 .33446 .11979 .32951 Metetra
.657 .738 .909 r
.11979 .32951 .12501 .33446 .13217 .33191 .12695 .32697 Metetra
.659 .739 .909 r
.12695 .32697 .13217 .33191 .13936 .32934 .13415 .32438 Metetra
.659 .739 .909 r
.13415 .32438 .13936 .32934 .14656 .32674 .14136 .32178 Metetra
.66 .739 .909 r
.14136 .32178 .14656 .32674 .15379 .32414 .1486 .31917 Metetra
.659 .739 .909 r
.1486 .31917 .15379 .32414 .16103 .32154 .15586 .31655 Metetra
.659 .739 .909 r
.15586 .31655 .16103 .32154 .1683 .31894 .16313 .31394 Metetra
.658 .739 .91 r
.16313 .31394 .1683 .31894 .17558 .31634 .17043 .31133 Metetra
.657 .739 .91 r
.17043 .31133 .17558 .31634 .18289 .31376 .17775 .30873 Metetra
.656 .738 .911 r
.17775 .30873 .18289 .31376 .19021 .31121 .18508 .30616 Metetra
.653 .737 .911 r
.18508 .30616 .19021 .31121 .19755 .30869 .19243 .30362 Metetra
.65 .736 .912 r
.19243 .30362 .19755 .30869 .2049 .30622 .1998 .30112 Metetra
.646 .735 .914 r
.1998 .30112 .2049 .30622 .21227 .30382 .20718 .2987 Metetra
.64 .732 .915 r
.20718 .2987 .21227 .30382 .21965 .30153 .21457 .29637 Metetra
.635 .73 .916 r
.21457 .29637 .21965 .30153 .22704 .29934 .22198 .29412 Metetra
.637 .729 .914 r
.22198 .29412 .22704 .29934 .23446 .29712 .22942 .29183 Metetra
.658 .732 .905 r
.22942 .29183 .23446 .29712 .24194 .29448 .23691 .28915 Metetra
.683 .738 .893 r
.23691 .28915 .24194 .29448 .2495 .29124 .24448 .28595 Metetra
.692 .743 .89 r
.24448 .28595 .2495 .29124 .25709 .28777 .25208 .28252 Metetra
.689 .744 .893 r
.25208 .28252 .25709 .28777 .2647 .28439 .2597 .27916 Metetra
.684 .745 .897 r
.2597 .27916 .2647 .28439 .27231 .28114 .26732 .27592 Metetra
.679 .744 .9 r
.26732 .27592 .27231 .28114 .27993 .278 .27495 .27278 Metetra
.675 .744 .902 r
.27495 .27278 .27993 .278 .28757 .27494 .2826 .26971 Metetra
.673 .743 .904 r
.2826 .26971 .28757 .27494 .29522 .27193 .29027 .26669 Metetra
.67 .743 .905 r
.29027 .26669 .29522 .27193 .30289 .26896 .29795 .2637 Metetra
.669 .742 .905 r
.29795 .2637 .30289 .26896 .31058 .26601 .30566 .26074 Metetra
.667 .742 .906 r
.30566 .26074 .31058 .26601 .31829 .26309 .31338 .25781 Metetra
.665 .742 .907 r
.31338 .25781 .31829 .26309 .32602 .2602 .32112 .2549 Metetra
.663 .741 .908 r
.32112 .2549 .32602 .2602 .33377 .25734 .32889 .25202 Metetra
.662 .74 .908 r
.32889 .25202 .33377 .25734 .34154 .25451 .33667 .24917 Metetra
.662 .74 .908 r
.33667 .24917 .34154 .25451 .34933 .25167 .34448 .2463 Metetra
.666 .741 .906 r
.34448 .2463 .34933 .25167 .35715 .24873 .35231 .24334 Metetra
.67 .742 .904 r
.35231 .24334 .35715 .24873 .365 .24569 .36017 .24029 Metetra
.671 .742 .904 r
.36017 .24029 .365 .24569 .37287 .24263 .36806 .23723 Metetra
.669 .742 .905 r
.36806 .23723 .37287 .24263 .38076 .2396 .37597 .23418 Metetra
.666 .741 .906 r
.37597 .23418 .38076 .2396 .38867 .23664 .38389 .2312 Metetra
.659 .739 .909 r
.38389 .2312 .38867 .23664 .3966 .23379 .39184 .22833 Metetra
.653 .737 .911 r
.39184 .22833 .3966 .23379 .40455 .23107 .3998 .22559 Metetra
.647 .734 .913 r
.3998 .22559 .40455 .23107 .41252 .22846 .40778 .22295 Metetra
.638 .731 .915 r
.40778 .22295 .41252 .22846 .42051 .22602 .41578 .22047 Metetra
.628 .726 .918 r
.41578 .22047 .42051 .22602 .42851 .22378 .4238 .21817 Metetra
.62 .721 .919 r
.4238 .21817 .42851 .22378 .43654 .22171 .43185 .21601 Metetra
.63 .722 .913 r
.43185 .21601 .43654 .22171 .44461 .21943 .43993 .21367 Metetra
.653 .728 .904 r
.43993 .21367 .44461 .21943 .45271 .21663 .44805 .21087 Metetra
.666 .734 .9 r
.44805 .21087 .45271 .21663 .46085 .21351 .4562 .20777 Metetra
.669 .736 .9 r
.4562 .20777 .46085 .21351 .46901 .21034 .46437 .20461 Metetra
.668 .737 .902 r
.46437 .20461 .46901 .21034 .47719 .2072 .47257 .20146 Metetra
.667 .737 .902 r
.47257 .20146 .47719 .2072 .4854 .20407 .48079 .19832 Metetra
.67 .738 .902 r
.48079 .19832 .4854 .20407 .49363 .20087 .48904 .19512 Metetra
.674 .741 .9 r
.48904 .19512 .49363 .20087 .50189 .19754 .49731 .19182 Metetra
.665 .739 .905 r
.49731 .19182 .50189 .19754 .51017 .19441 .50561 .18871 Metetra
.627 .726 .918 r
.50561 .18871 .51017 .19441 .51847 .19214 .51393 .18634 Metetra
.598 .711 .923 r
.51393 .18634 .51847 .19214 .5268 .19044 .52227 .18448 Metetra
.593 .703 .92 r
.52227 .18448 .5268 .19044 .53516 .18887 .53065 .18272 Metetra
.617 .706 .909 r
.53065 .18272 .53516 .18887 .54356 .18689 .53906 .18054 Metetra
.677 .723 .884 r
.53906 .18054 .54356 .18689 .55198 .18343 .54749 .17705 Metetra
.727 .74 .862 r
.54749 .17705 .55198 .18343 .5604 .17838 .55593 .17219 Metetra
.737 .75 .863 r
.55593 .17219 .5604 .17838 .56883 .17309 .56439 .16711 Metetra
.719 .753 .879 r
.56439 .16711 .56883 .17309 .57729 .16849 .57286 .16261 Metetra
.697 .75 .893 r
.57286 .16261 .57729 .16849 .58577 .16454 .58137 .15869 Metetra
.682 .747 .9 r
.58137 .15869 .58577 .16454 .59429 .16097 .58991 .15511 Metetra
.674 .745 .904 r
.58991 .15511 .59429 .16097 .60284 .15758 .59847 .15171 Metetra
.669 .743 .906 r
.59847 .15171 .60284 .15758 .61142 .15428 .60707 .1484 Metetra
.667 .743 .907 r
.60707 .1484 .61142 .15428 .62002 .15102 .61569 .14513 Metetra
.666 .742 .907 r
.61569 .14513 .62002 .15102 .62865 .14779 .62434 .14187 Metetra
.665 .742 .907 r
.62434 .14187 .62865 .14779 .63731 .14456 .63301 .13862 Metetra
.665 .742 .908 r
.63301 .13862 .63731 .14456 .64599 .14133 .64171 .13538 Metetra
.664 .742 .908 r
.64171 .13538 .64599 .14133 .6547 .13809 .65044 .13213 Metetra
.664 .741 .908 r
.65044 .13213 .6547 .13809 .66344 .13485 .6592 .12887 Metetra
.664 .741 .908 r
.6592 .12887 .66344 .13485 .6722 .13161 .66798 .1256 Metetra
.664 .741 .908 r
.66798 .1256 .6722 .13161 .681 .12836 .6768 .12233 Metetra
.664 .741 .908 r
.6768 .12233 .681 .12836 .68982 .12509 .68563 .11905 Metetra
.664 .741 .908 r
.68563 .11905 .68982 .12509 .69867 .12182 .6945 .11576 Metetra
.664 .741 .908 r
.6945 .11576 .69867 .12182 .70754 .11854 .7034 .11246 Metetra
.664 .741 .908 r
.7034 .11246 .70754 .11854 .71644 .11525 .71232 .10915 Metetra
.664 .741 .908 r
.71232 .10915 .71644 .11525 .72538 .11195 .72127 .10583 Metetra
.664 .741 .908 r
.72127 .10583 .72538 .11195 .73434 .10864 .73025 .1025 Metetra
.664 .741 .908 r
.73025 .1025 .73434 .10864 .74332 .10532 .73926 .09916 Metetra
.664 .741 .908 r
.73926 .09916 .74332 .10532 .75234 .10198 .74829 .09581 Metetra
.655 .737 .91 r
.10737 .32707 .11264 .33203 .11979 .32951 .11453 .32455 Metetra
.657 .738 .909 r
.11453 .32455 .11979 .32951 .12695 .32697 .12171 .322 Metetra
.658 .739 .909 r
.12171 .322 .12695 .32697 .13415 .32438 .12891 .31941 Metetra
.659 .739 .909 r
.12891 .31941 .13415 .32438 .14136 .32178 .13614 .31679 Metetra
.66 .739 .909 r
.13614 .31679 .14136 .32178 .1486 .31917 .14339 .31417 Metetra
.659 .739 .909 r
.14339 .31417 .1486 .31917 .15586 .31655 .15065 .31154 Metetra
.659 .739 .909 r
.15065 .31154 .15586 .31655 .16313 .31394 .15794 .30891 Metetra
.658 .739 .91 r
.15794 .30891 .16313 .31394 .17043 .31133 .16525 .30628 Metetra
.657 .739 .91 r
.16525 .30628 .17043 .31133 .17775 .30873 .17258 .30367 Metetra
.656 .738 .91 r
.17258 .30367 .17775 .30873 .18508 .30616 .17992 .30108 Metetra
.654 .738 .911 r
.17992 .30108 .18508 .30616 .19243 .30362 .18729 .29852 Metetra
.651 .737 .912 r
.18729 .29852 .19243 .30362 .1998 .30112 .19467 .296 Metetra
.647 .735 .913 r
.19467 .296 .1998 .30112 .20718 .2987 .20206 .29355 Metetra
.642 .733 .915 r
.20206 .29355 .20718 .2987 .21457 .29637 .20947 .29118 Metetra
.638 .731 .916 r
.20947 .29118 .21457 .29637 .22198 .29412 .2169 .28888 Metetra
.64 .73 .913 r
.2169 .28888 .22198 .29412 .22942 .29183 .22435 .28652 Metetra
.659 .733 .905 r
.22435 .28652 .22942 .29183 .23691 .28915 .23186 .28382 Metetra
.681 .739 .894 r
.23186 .28382 .23691 .28915 .24448 .28595 .23943 .28063 Metetra
.69 .743 .892 r
.23943 .28063 .24448 .28595 .25208 .28252 .24704 .27724 Metetra
.688 .744 .894 r
.24704 .27724 .25208 .28252 .2597 .27916 .25467 .2739 Metetra
.683 .744 .898 r
.25467 .2739 .2597 .27916 .26732 .27592 .2623 .27067 Metetra
.678 .744 .9 r
.2623 .27067 .26732 .27592 .27495 .27278 .26995 .26752 Metetra
.675 .743 .902 r
.26995 .26752 .27495 .27278 .2826 .26971 .27761 .26445 Metetra
.672 .743 .904 r
.27761 .26445 .2826 .26971 .29027 .26669 .28529 .26142 Metetra
.67 .743 .905 r
.28529 .26142 .29027 .26669 .29795 .2637 .29299 .25842 Metetra
.669 .742 .905 r
.29299 .25842 .29795 .2637 .30566 .26074 .30071 .25545 Metetra
.667 .742 .906 r
.30071 .25545 .30566 .26074 .31338 .25781 .30844 .2525 Metetra
.665 .742 .907 r
.30844 .2525 .31338 .25781 .32112 .2549 .3162 .24957 Metetra
.664 .741 .908 r
.3162 .24957 .32112 .2549 .32889 .25202 .32398 .24668 Metetra
.662 .74 .908 r
.32398 .24668 .32889 .25202 .33667 .24917 .33177 .2438 Metetra
.663 .74 .908 r
.33177 .2438 .33667 .24917 .34448 .2463 .3396 .24091 Metetra
.666 .741 .906 r
.3396 .24091 .34448 .2463 .35231 .24334 .34745 .23793 Metetra
.67 .742 .904 r
.34745 .23793 .35231 .24334 .36017 .24029 .35532 .23487 Metetra
.67 .742 .904 r
.35532 .23487 .36017 .24029 .36806 .23723 .36322 .23179 Metetra
.669 .742 .905 r
.36322 .23179 .36806 .23723 .37597 .23418 .37114 .22873 Metetra
.666 .741 .906 r
.37114 .22873 .37597 .23418 .38389 .2312 .37908 .22573 Metetra
.659 .739 .909 r
.37908 .22573 .38389 .2312 .39184 .22833 .38704 .22285 Metetra
.653 .737 .911 r
.38704 .22285 .39184 .22833 .3998 .22559 .39502 .22008 Metetra
.647 .735 .913 r
.39502 .22008 .3998 .22559 .40778 .22295 .40302 .21742 Metetra
.64 .732 .915 r
.40302 .21742 .40778 .22295 .41578 .22047 .41103 .2149 Metetra
.63 .727 .917 r
.41103 .2149 .41578 .22047 .4238 .21817 .41907 .21254 Metetra
.624 .723 .918 r
.41907 .21254 .4238 .21817 .43185 .21601 .42713 .2103 Metetra
.632 .723 .913 r
.42713 .2103 .43185 .21601 .43993 .21367 .43523 .20789 Metetra
.652 .729 .905 r
.43523 .20789 .43993 .21367 .44805 .21087 .44336 .20509 Metetra
.665 .734 .901 r
.44336 .20509 .44805 .21087 .4562 .20777 .45152 .20201 Metetra
.668 .736 .901 r
.45152 .20201 .4562 .20777 .46437 .20461 .45971 .19885 Metetra
.667 .737 .902 r
.45971 .19885 .46437 .20461 .47257 .20146 .46793 .1957 Metetra
.667 .737 .903 r
.46793 .1957 .47257 .20146 .48079 .19832 .47616 .19255 Metetra
.669 .738 .902 r
.47616 .19255 .48079 .19832 .48904 .19512 .48443 .18935 Metetra
.673 .74 .901 r
.48443 .18935 .48904 .19512 .49731 .19182 .49272 .18606 Metetra
.663 .738 .906 r
.49272 .18606 .49731 .19182 .50561 .18871 .50103 .18297 Metetra
.631 .727 .917 r
.50103 .18297 .50561 .18871 .51393 .18634 .50936 .18051 Metetra
.606 .714 .921 r
.50936 .18051 .51393 .18634 .52227 .18448 .51772 .1785 Metetra
.602 .707 .918 r
.51772 .1785 .52227 .18448 .53065 .18272 .52611 .17657 Metetra
.624 .71 .908 r
.52611 .17657 .53065 .18272 .53906 .18054 .53454 .17422 Metetra
.678 .725 .886 r
.53454 .17422 .53906 .18054 .54749 .17705 .54298 .17071 Metetra
.722 .741 .867 r
.54298 .17071 .54749 .17705 .55593 .17219 .55144 .16601 Metetra
.731 .75 .868 r
.55144 .16601 .55593 .17219 .56439 .16711 .55992 .1611 Metetra
.715 .752 .882 r
.55992 .1611 .56439 .16711 .57286 .16261 .56842 .15669 Metetra
.695 .75 .894 r
.56842 .15669 .57286 .16261 .58137 .15869 .57694 .1528 Metetra
.681 .747 .901 r
.57694 .1528 .58137 .15869 .58991 .15511 .5855 .14923 Metetra
.673 .745 .904 r
.5855 .14923 .58991 .15511 .59847 .15171 .59408 .14582 Metetra
.669 .743 .906 r
.59408 .14582 .59847 .15171 .60707 .1484 .60269 .14249 Metetra
.667 .743 .907 r
.60269 .14249 .60707 .1484 .61569 .14513 .61133 .1392 Metetra
.666 .742 .907 r
.61133 .1392 .61569 .14513 .62434 .14187 .62 .13593 Metetra
.665 .742 .907 r
.62 .13593 .62434 .14187 .63301 .13862 .62869 .13266 Metetra
.665 .742 .908 r
.62869 .13266 .63301 .13862 .64171 .13538 .63741 .12939 Metetra
.664 .742 .908 r
.63741 .12939 .64171 .13538 .65044 .13213 .64616 .12612 Metetra
.664 .741 .908 r
.64616 .12612 .65044 .13213 .6592 .12887 .65494 .12285 Metetra
.664 .741 .908 r
.65494 .12285 .6592 .12887 .66798 .1256 .66374 .11956 Metetra
.664 .741 .908 r
.66374 .11956 .66798 .1256 .6768 .12233 .67257 .11627 Metetra
.664 .741 .908 r
.67257 .11627 .6768 .12233 .68563 .11905 .68143 .11297 Metetra
.664 .741 .908 r
.68143 .11297 .68563 .11905 .6945 .11576 .69031 .10966 Metetra
.664 .741 .908 r
.69031 .10966 .6945 .11576 .7034 .11246 .69923 .10634 Metetra
.664 .741 .908 r
.69923 .10634 .7034 .11246 .71232 .10915 .70817 .10301 Metetra
.664 .741 .908 r
.70817 .10301 .71232 .10915 .72127 .10583 .71714 .09968 Metetra
.664 .741 .908 r
.71714 .09968 .72127 .10583 .73025 .1025 .72614 .09633 Metetra
.664 .741 .908 r
.72614 .09633 .73025 .1025 .73926 .09916 .73517 .09297 Metetra
.664 .741 .908 r
.73517 .09297 .73926 .09916 .74829 .09581 .74422 .08959 Metetra
.655 .737 .91 r
.10208 .32208 .10737 .32707 .11453 .32455 .10925 .31956 Metetra
.656 .738 .91 r
.10925 .31956 .11453 .32455 .12171 .322 .11643 .317 Metetra
.658 .739 .909 r
.11643 .317 .12171 .322 .12891 .31941 .12365 .31441 Metetra
.659 .739 .909 r
.12365 .31441 .12891 .31941 .13614 .31679 .13089 .31178 Metetra
.659 .739 .909 r
.13089 .31178 .13614 .31679 .14339 .31417 .13815 .30914 Metetra
.659 .739 .909 r
.13815 .30914 .14339 .31417 .15065 .31154 .14543 .3065 Metetra
.659 .739 .909 r
.14543 .3065 .15065 .31154 .15794 .30891 .15273 .30385 Metetra
.658 .739 .909 r
.15273 .30385 .15794 .30891 .16525 .30628 .16005 .30121 Metetra
.657 .739 .91 r
.16005 .30121 .16525 .30628 .17258 .30367 .16738 .29859 Metetra
.656 .738 .91 r
.16738 .29859 .17258 .30367 .17992 .30108 .17474 .29598 Metetra
.654 .738 .911 r
.17474 .29598 .17992 .30108 .18729 .29852 .18212 .2934 Metetra
.651 .737 .912 r
.18212 .2934 .18729 .29852 .19467 .296 .18951 .29086 Metetra
.648 .735 .913 r
.18951 .29086 .19467 .296 .20206 .29355 .19692 .28838 Metetra
.643 .733 .914 r
.19692 .28838 .20206 .29355 .20947 .29118 .20434 .28597 Metetra
.64 .731 .915 r
.20434 .28597 .20947 .29118 .2169 .28888 .21178 .28362 Metetra
.643 .731 .913 r
.21178 .28362 .2169 .28888 .22435 .28652 .21925 .2812 Metetra
.659 .734 .905 r
.21925 .2812 .22435 .28652 .23186 .28382 .22677 .27846 Metetra
.679 .739 .896 r
.22677 .27846 .23186 .28382 .23943 .28063 .23435 .2753 Metetra
.688 .742 .893 r
.23435 .2753 .23943 .28063 .24704 .27724 .24197 .27194 Metetra
.686 .744 .895 r
.24197 .27194 .24704 .27724 .25467 .2739 .24961 .26862 Metetra
.682 .744 .898 r
.24961 .26862 .25467 .2739 .2623 .27067 .25726 .26539 Metetra
.678 .744 .901 r
.25726 .26539 .2623 .27067 .26995 .26752 .26492 .26224 Metetra
.675 .743 .902 r
.26492 .26224 .26995 .26752 .27761 .26445 .27259 .25916 Metetra
.672 .743 .904 r
.27259 .25916 .27761 .26445 .28529 .26142 .28029 .25611 Metetra
.67 .743 .905 r
.28029 .25611 .28529 .26142 .29299 .25842 .288 .2531 Metetra
.669 .742 .906 r
.288 .2531 .29299 .25842 .30071 .25545 .29573 .25012 Metetra
.667 .742 .906 r
.29573 .25012 .30071 .25545 .30844 .2525 .30348 .24716 Metetra
.665 .742 .907 r
.30348 .24716 .30844 .2525 .3162 .24957 .31125 .24422 Metetra
.664 .741 .908 r
.31125 .24422 .3162 .24957 .32398 .24668 .31904 .2413 Metetra
.663 .741 .908 r
.31904 .2413 .32398 .24668 .33177 .2438 .32685 .2384 Metetra
.663 .74 .907 r
.32685 .2384 .33177 .2438 .3396 .24091 .33469 .23548 Metetra
.666 .741 .906 r
.33469 .23548 .3396 .24091 .34745 .23793 .34255 .23249 Metetra
.67 .742 .905 r
.34255 .23249 .34745 .23793 .35532 .23487 .35044 .22942 Metetra
.67 .742 .904 r
.35044 .22942 .35532 .23487 .36322 .23179 .35836 .22633 Metetra
.669 .742 .905 r
.35836 .22633 .36322 .23179 .37114 .22873 .36629 .22325 Metetra
.666 .741 .906 r
.36629 .22325 .37114 .22873 .37908 .22573 .37425 .22023 Metetra
.659 .739 .909 r
.37425 .22023 .37908 .22573 .38704 .22285 .38222 .21733 Metetra
.653 .737 .911 r
.38222 .21733 .38704 .22285 .39502 .22008 .39021 .21454 Metetra
.648 .735 .913 r
.39021 .21454 .39502 .22008 .40302 .21742 .39822 .21186 Metetra
.641 .732 .915 r
.39822 .21186 .40302 .21742 .41103 .2149 .40626 .20929 Metetra
.632 .728 .917 r
.40626 .20929 .41103 .2149 .41907 .21254 .41431 .20688 Metetra
.627 .724 .917 r
.41431 .20688 .41907 .21254 .42713 .2103 .42239 .20457 Metetra
.634 .725 .913 r
.42239 .20457 .42713 .2103 .43523 .20789 .4305 .2021 Metetra
.652 .729 .906 r
.4305 .2021 .43523 .20789 .44336 .20509 .43864 .19929 Metetra
.664 .734 .902 r
.43864 .19929 .44336 .20509 .45152 .20201 .44682 .19622 Metetra
.667 .736 .902 r
.44682 .19622 .45152 .20201 .45971 .19885 .45503 .19307 Metetra
.667 .737 .902 r
.45503 .19307 .45971 .19885 .46793 .1957 .46326 .18992 Metetra
.667 .738 .903 r
.46326 .18992 .46793 .1957 .47616 .19255 .47151 .18676 Metetra
.669 .738 .902 r
.47151 .18676 .47616 .19255 .48443 .18935 .47979 .18355 Metetra
.671 .74 .902 r
.47979 .18355 .48443 .18935 .49272 .18606 .48809 .18027 Metetra
.662 .738 .906 r
.48809 .18027 .49272 .18606 .50103 .18297 .49642 .17719 Metetra
.634 .728 .916 r
.49642 .17719 .50103 .18297 .50936 .18051 .50477 .17465 Metetra
.612 .717 .92 r
.50477 .17465 .50936 .18051 .51772 .1785 .51314 .17251 Metetra
.609 .711 .917 r
.51314 .17251 .51772 .1785 .52611 .17657 .52155 .17042 Metetra
.631 .714 .907 r
.52155 .17042 .52611 .17657 .53454 .17422 .52999 .16792 Metetra
.678 .728 .887 r
.52999 .16792 .53454 .17422 .54298 .17071 .53845 .16439 Metetra
.717 .741 .871 r
.53845 .16439 .54298 .17071 .55144 .16601 .54693 .15983 Metetra
.725 .75 .872 r
.54693 .15983 .55144 .16601 .55992 .1611 .55543 .15508 Metetra
.711 .752 .884 r
.55543 .15508 .55992 .1611 .56842 .15669 .56394 .15075 Metetra
.693 .749 .895 r
.56394 .15075 .56842 .15669 .57694 .1528 .57249 .14689 Metetra
.68 .746 .901 r
.57249 .14689 .57694 .1528 .5855 .14923 .58106 .14331 Metetra
.673 .744 .904 r
.58106 .14331 .5855 .14923 .59408 .14582 .58967 .13989 Metetra
.669 .743 .906 r
.58967 .13989 .59408 .14582 .60269 .14249 .59829 .13654 Metetra
.667 .743 .907 r
.59829 .13654 .60269 .14249 .61133 .1392 .60695 .13323 Metetra
.666 .742 .907 r
.60695 .13323 .61133 .1392 .62 .13593 .61564 .12994 Metetra
.665 .742 .907 r
.61564 .12994 .62 .13593 .62869 .13266 .62435 .12666 Metetra
.664 .742 .908 r
.62435 .12666 .62869 .13266 .63741 .12939 .63309 .12338 Metetra
.664 .742 .908 r
.63309 .12338 .63741 .12939 .64616 .12612 .64185 .12009 Metetra
.664 .741 .908 r
.64185 .12009 .64616 .12612 .65494 .12285 .65065 .11679 Metetra
.664 .741 .908 r
.65065 .11679 .65494 .12285 .66374 .11956 .65947 .11349 Metetra
.664 .741 .908 r
.65947 .11349 .66374 .11956 .67257 .11627 .66832 .11018 Metetra
.664 .741 .908 r
.66832 .11018 .67257 .11627 .68143 .11297 .6772 .10686 Metetra
.664 .741 .908 r
.6772 .10686 .68143 .11297 .69031 .10966 .6861 .10353 Metetra
.664 .741 .908 r
.6861 .10353 .69031 .10966 .69923 .10634 .69503 .10019 Metetra
.664 .741 .908 r
.69503 .10019 .69923 .10634 .70817 .10301 .704 .09684 Metetra
.664 .741 .908 r
.704 .09684 .70817 .10301 .71714 .09968 .71299 .09349 Metetra
.664 .741 .908 r
.71299 .09349 .71714 .09968 .72614 .09633 .722 .09012 Metetra
.664 .741 .908 r
.722 .09012 .72614 .09633 .73517 .09297 .73105 .08674 Metetra
.664 .741 .908 r
.73105 .08674 .73517 .09297 .74422 .08959 .74013 .08335 Metetra
.654 .737 .91 r
.09676 .31708 .10208 .32208 .10925 .31956 .10393 .31455 Metetra
.656 .738 .91 r
.10393 .31455 .10925 .31956 .11643 .317 .11113 .31199 Metetra
.658 .739 .909 r
.11113 .31199 .11643 .317 .12365 .31441 .11836 .30938 Metetra
.659 .739 .909 r
.11836 .30938 .12365 .31441 .13089 .31178 .12561 .30674 Metetra
.659 .739 .909 r
.12561 .30674 .13089 .31178 .13815 .30914 .13288 .30409 Metetra
.659 .739 .909 r
.13288 .30409 .13815 .30914 .14543 .3065 .14017 .30143 Metetra
.659 .739 .909 r
.14017 .30143 .14543 .3065 .15273 .30385 .14748 .29877 Metetra
.658 .739 .909 r
.14748 .29877 .15273 .30385 .16005 .30121 .15481 .29612 Metetra
.657 .739 .91 r
.15481 .29612 .16005 .30121 .16738 .29859 .16216 .29347 Metetra
.656 .738 .91 r
.16216 .29347 .16738 .29859 .17474 .29598 .16953 .29085 Metetra
.654 .738 .911 r
.16953 .29085 .17474 .29598 .18212 .2934 .17692 .28825 Metetra
.652 .737 .912 r
.17692 .28825 .18212 .2934 .18951 .29086 .18433 .28569 Metetra
.648 .736 .913 r
.18433 .28569 .18951 .29086 .19692 .28838 .19175 .28318 Metetra
.644 .734 .914 r
.19175 .28318 .19692 .28838 .20434 .28597 .19918 .28073 Metetra
.642 .732 .914 r
.19918 .28073 .20434 .28597 .21178 .28362 .20664 .27833 Metetra
.645 .732 .912 r
.20664 .27833 .21178 .28362 .21925 .2812 .21412 .27586 Metetra
.66 .734 .905 r
.21412 .27586 .21925 .2812 .22677 .27846 .22166 .27309 Metetra
.678 .739 .897 r
.22166 .27309 .22677 .27846 .23435 .2753 .22925 .26995 Metetra
.686 .742 .894 r
.22925 .26995 .23435 .2753 .24197 .27194 .23688 .26662 Metetra
.685 .744 .896 r
.23688 .26662 .24197 .27194 .24961 .26862 .24453 .26331 Metetra
.681 .744 .899 r
.24453 .26331 .24961 .26862 .25726 .26539 .25219 .26008 Metetra
.677 .744 .901 r
.25219 .26008 .25726 .26539 .26492 .26224 .25986 .25693 Metetra
.674 .743 .903 r
.25986 .25693 .26492 .26224 .27259 .25916 .26755 .25384 Metetra
.672 .743 .904 r
.26755 .25384 .27259 .25916 .28029 .25611 .27525 .25078 Metetra
.67 .743 .905 r
.27525 .25078 .28029 .25611 .288 .2531 .28298 .24776 Metetra
.668 .742 .906 r
.28298 .24776 .288 .2531 .29573 .25012 .29072 .24476 Metetra
.667 .742 .906 r
.29072 .24476 .29573 .25012 .30348 .24716 .29849 .24179 Metetra
.665 .742 .907 r
.29849 .24179 .30348 .24716 .31125 .24422 .30627 .23883 Metetra
.664 .741 .907 r
.30627 .23883 .31125 .24422 .31904 .2413 .31408 .2359 Metetra
.663 .741 .908 r
.31408 .2359 .31904 .2413 .32685 .2384 .3219 .23298 Metetra
.664 .741 .907 r
.3219 .23298 .32685 .2384 .33469 .23548 .32975 .23003 Metetra
.667 .741 .906 r
.32975 .23003 .33469 .23548 .34255 .23249 .33763 .22702 Metetra
.669 .742 .905 r
.33763 .22702 .34255 .23249 .35044 .22942 .34554 .22394 Metetra
.67 .742 .905 r
.34554 .22394 .35044 .22942 .35836 .22633 .35347 .22083 Metetra
.669 .742 .905 r
.35347 .22083 .35836 .22633 .36629 .22325 .36142 .21774 Metetra
.666 .741 .906 r
.36142 .21774 .36629 .22325 .37425 .22023 .36939 .2147 Metetra
.659 .739 .909 r
.36939 .2147 .37425 .22023 .38222 .21733 .37737 .21178 Metetra
.653 .737 .911 r
.37737 .21178 .38222 .21733 .39021 .21454 .38538 .20898 Metetra
.648 .735 .913 r
.38538 .20898 .39021 .21454 .39822 .21186 .39341 .20627 Metetra
.642 .732 .914 r
.39341 .20627 .39822 .21186 .40626 .20929 .40145 .20367 Metetra
.634 .729 .916 r
.40145 .20367 .40626 .20929 .41431 .20688 .40952 .2012 Metetra
.63 .726 .916 r
.40952 .2012 .41431 .20688 .42239 .20457 .41761 .19882 Metetra
.636 .726 .913 r
.41761 .19882 .42239 .20457 .4305 .2021 .42574 .1963 Metetra
.652 .73 .907 r
.42574 .1963 .4305 .2021 .43864 .19929 .4339 .19347 Metetra
.663 .734 .903 r
.4339 .19347 .43864 .19929 .44682 .19622 .44209 .19041 Metetra
.666 .736 .902 r
.44209 .19041 .44682 .19622 .45503 .19307 .45031 .18726 Metetra
.666 .737 .903 r
.45031 .18726 .45503 .19307 .46326 .18992 .45856 .1841 Metetra
.667 .738 .903 r
.45856 .1841 .46326 .18992 .47151 .18676 .46683 .18093 Metetra
.668 .739 .903 r
.46683 .18093 .47151 .18676 .47979 .18355 .47512 .17772 Metetra
.67 .74 .902 r
.47512 .17772 .47979 .18355 .48809 .18027 .48344 .17446 Metetra
.662 .738 .906 r
.48344 .17446 .48809 .18027 .49642 .17719 .49179 .17137 Metetra
.637 .729 .915 r
.49179 .17137 .49642 .17719 .50477 .17465 .50015 .16876 Metetra
.618 .72 .918 r
.50015 .16876 .50477 .17465 .51314 .17251 .50854 .1665 Metetra
.616 .714 .916 r
.50854 .1665 .51314 .17251 .52155 .17042 .51696 .16426 Metetra
.637 .718 .906 r
.51696 .16426 .52155 .17042 .52999 .16792 .52542 .16164 Metetra
.678 .729 .889 r
.52542 .16164 .52999 .16792 .53845 .16439 .5339 .15809 Metetra
.713 .742 .874 r
.5339 .15809 .53845 .16439 .54693 .15983 .54239 .15365 Metetra
.72 .749 .875 r
.54239 .15365 .54693 .15983 .55543 .15508 .55091 .14903 Metetra
.708 .751 .886 r
.55091 .14903 .55543 .15508 .56394 .15075 .55945 .14478 Metetra
.691 .749 .896 r
.55945 .14478 .56394 .15075 .57249 .14689 .56801 .14093 Metetra
.68 .746 .901 r
.56801 .14093 .57249 .14689 .58106 .14331 .5766 .13736 Metetra
.673 .744 .904 r
.5766 .13736 .58106 .14331 .58967 .13989 .58522 .13392 Metetra
.669 .743 .906 r
.58522 .13392 .58967 .13989 .59829 .13654 .59387 .13056 Metetra
.667 .742 .907 r
.59387 .13056 .59829 .13654 .60695 .13323 .60255 .12724 Metetra
.666 .742 .907 r
.60255 .12724 .60695 .13323 .61564 .12994 .61125 .12393 Metetra
.665 .742 .907 r
.61125 .12393 .61564 .12994 .62435 .12666 .61998 .12063 Metetra
.664 .742 .908 r
.61998 .12063 .62435 .12666 .63309 .12338 .62874 .11732 Metetra
.664 .742 .908 r
.62874 .11732 .63309 .12338 .64185 .12009 .63752 .11402 Metetra
.664 .741 .908 r
.63752 .11402 .64185 .12009 .65065 .11679 .64633 .1107 Metetra
.664 .741 .908 r
.64633 .1107 .65065 .11679 .65947 .11349 .65517 .10738 Metetra
.664 .741 .908 r
.65517 .10738 .65947 .11349 .66832 .11018 .66404 .10405 Metetra
.664 .741 .908 r
.66404 .10405 .66832 .11018 .6772 .10686 .67294 .10071 Metetra
.664 .741 .908 r
.67294 .10071 .6772 .10686 .6861 .10353 .68186 .09736 Metetra
.664 .741 .908 r
.68186 .09736 .6861 .10353 .69503 .10019 .69082 .09401 Metetra
.664 .741 .908 r
.69082 .09401 .69503 .10019 .704 .09684 .6998 .09064 Metetra
.664 .741 .908 r
.6998 .09064 .704 .09684 .71299 .09349 .70881 .08726 Metetra
.664 .741 .908 r
.70881 .08726 .71299 .09349 .722 .09012 .71785 .08387 Metetra
.664 .741 .908 r
.71785 .08387 .722 .09012 .73105 .08674 .72691 .08047 Metetra
.664 .741 .908 r
.72691 .08047 .73105 .08674 .74013 .08335 .73601 .07706 Metetra
.654 .737 .911 r
.09141 .31205 .09676 .31708 .10393 .31455 .0986 .30952 Metetra
.656 .738 .91 r
.0986 .30952 .10393 .31455 .11113 .31199 .1058 .30694 Metetra
.658 .739 .91 r
.1058 .30694 .11113 .31199 .11836 .30938 .11304 .30433 Metetra
.659 .739 .909 r
.11304 .30433 .11836 .30938 .12561 .30674 .1203 .30168 Metetra
.659 .739 .909 r
.1203 .30168 .12561 .30674 .13288 .30409 .12758 .29901 Metetra
.659 .739 .909 r
.12758 .29901 .13288 .30409 .14017 .30143 .13488 .29634 Metetra
.659 .739 .909 r
.13488 .29634 .14017 .30143 .14748 .29877 .14221 .29367 Metetra
.658 .739 .909 r
.14221 .29367 .14748 .29877 .15481 .29612 .14955 .291 Metetra
.658 .739 .91 r
.14955 .291 .15481 .29612 .16216 .29347 .15691 .28834 Metetra
.656 .738 .91 r
.15691 .28834 .16216 .29347 .16953 .29085 .1643 .28569 Metetra
.655 .738 .911 r
.1643 .28569 .16953 .29085 .17692 .28825 .1717 .28307 Metetra
.652 .737 .912 r
.1717 .28307 .17692 .28825 .18433 .28569 .17911 .28049 Metetra
.649 .736 .913 r
.17911 .28049 .18433 .28569 .19175 .28318 .18655 .27795 Metetra
.646 .734 .914 r
.18655 .27795 .19175 .28318 .19918 .28073 .194 .27547 Metetra
.643 .733 .914 r
.194 .27547 .19918 .28073 .20664 .27833 .20147 .27302 Metetra
.647 .732 .911 r
.20147 .27302 .20664 .27833 .21412 .27586 .20897 .2705 Metetra
.661 .735 .905 r
.20897 .2705 .21412 .27586 .22166 .27309 .21652 .26771 Metetra
.677 .739 .898 r
.21652 .26771 .22166 .27309 .22925 .26995 .22412 .26458 Metetra
.684 .742 .895 r
.22412 .26458 .22925 .26995 .23688 .26662 .23176 .26127 Metetra
.684 .744 .896 r
.23176 .26127 .23688 .26662 .24453 .26331 .23942 .25797 Metetra
.68 .744 .899 r
.23942 .25797 .24453 .26331 .25219 .26008 .24709 .25475 Metetra
.677 .744 .901 r
.24709 .25475 .25219 .26008 .25986 .25693 .25478 .25159 Metetra
.674 .743 .903 r
.25478 .25159 .25986 .25693 .26755 .25384 .26248 .24849 Metetra
.672 .743 .904 r
.26248 .24849 .26755 .25384 .27525 .25078 .2702 .24543 Metetra
.67 .743 .905 r
.2702 .24543 .27525 .25078 .28298 .24776 .27793 .24239 Metetra
.668 .742 .906 r
.27793 .24239 .28298 .24776 .29072 .24476 .28569 .23938 Metetra
.667 .742 .906 r
.28569 .23938 .29072 .24476 .29849 .24179 .29347 .23639 Metetra
.665 .742 .907 r
.29347 .23639 .29849 .24179 .30627 .23883 .30127 .23342 Metetra
.664 .741 .907 r
.30127 .23342 .30627 .23883 .31408 .2359 .30909 .23046 Metetra
.663 .741 .908 r
.30909 .23046 .31408 .2359 .3219 .23298 .31693 .22752 Metetra
.664 .741 .907 r
.31693 .22752 .3219 .23298 .32975 .23003 .32479 .22455 Metetra
.667 .741 .906 r
.32479 .22455 .32975 .23003 .33763 .22702 .33269 .22152 Metetra
.669 .742 .905 r
.33269 .22152 .33763 .22702 .34554 .22394 .3406 .21843 Metetra
.67 .742 .905 r
.3406 .21843 .34554 .22394 .35347 .22083 .34855 .21531 Metetra
.669 .742 .905 r
.34855 .21531 .35347 .22083 .36142 .21774 .35651 .2122 Metetra
.666 .741 .906 r
.35651 .2122 .36142 .21774 .36939 .2147 .3645 .20914 Metetra
.659 .739 .909 r
.3645 .20914 .36939 .2147 .37737 .21178 .3725 .20621 Metetra
.654 .737 .911 r
.3725 .20621 .37737 .21178 .38538 .20898 .38052 .20338 Metetra
.649 .735 .912 r
.38052 .20338 .38538 .20898 .39341 .20627 .38856 .20064 Metetra
.643 .733 .914 r
.38856 .20064 .39341 .20627 .40145 .20367 .39662 .19801 Metetra
.636 .73 .916 r
.39662 .19801 .40145 .20367 .40952 .2012 .40471 .19549 Metetra
.632 .727 .916 r
.40471 .19549 .40952 .2012 .41761 .19882 .41282 .19304 Metetra
.638 .727 .913 r
.41282 .19304 .41761 .19882 .42574 .1963 .42096 .19048 Metetra
.652 .731 .907 r
.42096 .19048 .42574 .1963 .4339 .19347 .42913 .18763 Metetra
.662 .734 .904 r
.42913 .18763 .4339 .19347 .44209 .19041 .43734 .18457 Metetra
.665 .736 .903 r
.43734 .18457 .44209 .19041 .45031 .18726 .44558 .18143 Metetra
.666 .737 .903 r
.44558 .18143 .45031 .18726 .45856 .1841 .45384 .17826 Metetra
.666 .738 .903 r
.45384 .17826 .45856 .1841 .46683 .18093 .46212 .17508 Metetra
.668 .739 .903 r
.46212 .17508 .46683 .18093 .47512 .17772 .47043 .17186 Metetra
.669 .739 .903 r
.47043 .17186 .47512 .17772 .48344 .17446 .47877 .16861 Metetra
.661 .738 .907 r
.47877 .16861 .48344 .17446 .49179 .17137 .48713 .16551 Metetra
.64 .73 .914 r
.48713 .16551 .49179 .17137 .50015 .16876 .49551 .16284 Metetra
.623 .722 .917 r
.49551 .16284 .50015 .16876 .50854 .1665 .50391 .16046 Metetra
.622 .717 .914 r
.50391 .16046 .50854 .1665 .51696 .16426 .51235 .1581 Metetra
.641 .721 .906 r
.51235 .1581 .51696 .16426 .52542 .16164 .52082 .15537 Metetra
.678 .731 .89 r
.52082 .15537 .52542 .16164 .5339 .15809 .52931 .15181 Metetra
.709 .742 .878 r
.52931 .15181 .5339 .15809 .54239 .15365 .53783 .14747 Metetra
.716 .749 .878 r
.53783 .14747 .54239 .15365 .55091 .14903 .54637 .14297 Metetra
.704 .75 .888 r
.54637 .14297 .55091 .14903 .55945 .14478 .55492 .13878 Metetra
.69 .748 .896 r
.55492 .13878 .55945 .14478 .56801 .14093 .56351 .13495 Metetra
.679 .746 .902 r
.56351 .13495 .56801 .14093 .5766 .13736 .57212 .13137 Metetra
.672 .744 .904 r
.57212 .13137 .5766 .13736 .58522 .13392 .58076 .12792 Metetra
.669 .743 .906 r
.58076 .12792 .58522 .13392 .59387 .13056 .58942 .12455 Metetra
.667 .742 .907 r
.58942 .12455 .59387 .13056 .60255 .12724 .59811 .1212 Metetra
.666 .742 .907 r
.59811 .1212 .60255 .12724 .61125 .12393 .60684 .11788 Metetra
.665 .742 .907 r
.60684 .11788 .61125 .12393 .61998 .12063 .61558 .11456 Metetra
.664 .742 .908 r
.61558 .11456 .61998 .12063 .62874 .11732 .62436 .11123 Metetra
.664 .742 .908 r
.62436 .11123 .62874 .11732 .63752 .11402 .63316 .10791 Metetra
.664 .741 .908 r
.63316 .10791 .63752 .11402 .64633 .1107 .642 .10458 Metetra
.664 .741 .908 r
.642 .10458 .64633 .1107 .65517 .10738 .65085 .10124 Metetra
.664 .741 .908 r
.65085 .10124 .65517 .10738 .66404 .10405 .65974 .09789 Metetra
.664 .741 .908 r
.65974 .09789 .66404 .10405 .67294 .10071 .66866 .09453 Metetra
.664 .741 .908 r
.66866 .09453 .67294 .10071 .68186 .09736 .6776 .09116 Metetra
.664 .741 .908 r
.6776 .09116 .68186 .09736 .69082 .09401 .68657 .08778 Metetra
.664 .741 .908 r
.68657 .08778 .69082 .09401 .6998 .09064 .69557 .08439 Metetra
.664 .741 .908 r
.69557 .08439 .6998 .09064 .70881 .08726 .7046 .081 Metetra
.664 .741 .908 r
.7046 .081 .70881 .08726 .71785 .08387 .71366 .07759 Metetra
.664 .741 .908 r
.71366 .07759 .71785 .08387 .72691 .08047 .72275 .07417 Metetra
.664 .741 .908 r
.72275 .07417 .72691 .08047 .73601 .07706 .73187 .07074 Metetra
.654 .737 .911 r
.08603 .307 .09141 .31205 .0986 .30952 .09323 .30446 Metetra
.655 .738 .91 r
.09323 .30446 .0986 .30952 .1058 .30694 .10045 .30188 Metetra
.658 .739 .91 r
.10045 .30188 .1058 .30694 .11304 .30433 .10769 .29925 Metetra
.659 .739 .909 r
.10769 .29925 .11304 .30433 .1203 .30168 .11496 .29659 Metetra
.659 .739 .909 r
.11496 .29659 .1203 .30168 .12758 .29901 .12226 .29391 Metetra
.659 .739 .909 r
.12226 .29391 .12758 .29901 .13488 .29634 .12957 .29122 Metetra
.659 .739 .909 r
.12957 .29122 .13488 .29634 .14221 .29367 .13691 .28853 Metetra
.659 .739 .909 r
.13691 .28853 .14221 .29367 .14955 .291 .14426 .28585 Metetra
.658 .739 .91 r
.14426 .28585 .14955 .291 .15691 .28834 .15164 .28317 Metetra
.656 .739 .91 r
.15164 .28317 .15691 .28834 .1643 .28569 .15903 .28051 Metetra
.655 .738 .911 r
.15903 .28051 .1643 .28569 .1717 .28307 .16644 .27787 Metetra
.653 .737 .911 r
.16644 .27787 .1717 .28307 .17911 .28049 .17387 .27526 Metetra
.65 .736 .912 r
.17387 .27526 .17911 .28049 .18655 .27795 .18132 .2727 Metetra
.647 .735 .913 r
.18132 .2727 .18655 .27795 .194 .27547 .18878 .27018 Metetra
.645 .733 .913 r
.18878 .27018 .194 .27547 .20147 .27302 .19627 .26769 Metetra
.649 .733 .911 r
.19627 .26769 .20147 .27302 .20897 .2705 .20378 .26512 Metetra
.661 .735 .905 r
.20378 .26512 .20897 .2705 .21652 .26771 .21135 .26231 Metetra
.676 .739 .898 r
.21135 .26231 .21652 .26771 .22412 .26458 .21896 .25918 Metetra
.683 .742 .896 r
.21896 .25918 .22412 .26458 .23176 .26127 .22661 .2559 Metetra
.683 .743 .897 r
.22661 .2559 .23176 .26127 .23942 .25797 .23428 .25261 Metetra
.68 .744 .899 r
.23428 .25261 .23942 .25797 .24709 .25475 .24196 .24939 Metetra
.676 .743 .901 r
.24196 .24939 .24709 .25475 .25478 .25159 .24966 .24622 Metetra
.674 .743 .903 r
.24966 .24622 .25478 .25159 .26248 .24849 .25738 .24311 Metetra
.671 .743 .904 r
.25738 .24311 .26248 .24849 .2702 .24543 .26511 .24004 Metetra
.67 .743 .905 r
.26511 .24004 .2702 .24543 .27793 .24239 .27286 .23699 Metetra
.668 .742 .906 r
.27286 .23699 .27793 .24239 .28569 .23938 .28063 .23397 Metetra
.667 .742 .906 r
.28063 .23397 .28569 .23938 .29347 .23639 .28842 .23096 Metetra
.665 .742 .907 r
.28842 .23096 .29347 .23639 .30127 .23342 .29624 .22797 Metetra
.664 .741 .907 r
.29624 .22797 .30127 .23342 .30909 .23046 .30407 .225 Metetra
.663 .741 .908 r
.30407 .225 .30909 .23046 .31693 .22752 .31192 .22203 Metetra
.664 .741 .907 r
.31192 .22203 .31693 .22752 .32479 .22455 .3198 .21905 Metetra
.667 .741 .906 r
.3198 .21905 .32479 .22455 .33269 .22152 .32771 .216 Metetra
.669 .742 .905 r
.32771 .216 .33269 .22152 .3406 .21843 .33564 .21289 Metetra
.67 .742 .905 r
.33564 .21289 .3406 .21843 .34855 .21531 .3436 .20976 Metetra
.669 .742 .905 r
.3436 .20976 .34855 .21531 .35651 .2122 .35158 .20663 Metetra
.666 .742 .906 r
.35158 .20663 .35651 .2122 .3645 .20914 .35958 .20355 Metetra
.659 .739 .909 r
.35958 .20355 .3645 .20914 .3725 .20621 .3676 .2006 Metetra
.654 .737 .911 r
.3676 .2006 .3725 .20621 .38052 .20338 .37563 .19776 Metetra
.649 .736 .912 r
.37563 .19776 .38052 .20338 .38856 .20064 .38369 .19499 Metetra
.644 .733 .914 r
.38369 .19499 .38856 .20064 .39662 .19801 .39177 .19232 Metetra
.638 .73 .915 r
.39177 .19232 .39662 .19801 .40471 .19549 .39987 .18976 Metetra
.634 .728 .915 r
.39987 .18976 .40471 .19549 .41282 .19304 .40799 .18725 Metetra
.64 .728 .912 r
.40799 .18725 .41282 .19304 .42096 .19048 .41615 .18464 Metetra
.652 .731 .907 r
.41615 .18464 .42096 .19048 .42913 .18763 .42434 .18177 Metetra
.661 .734 .904 r
.42434 .18177 .42913 .18763 .43734 .18457 .43256 .17871 Metetra
.665 .736 .903 r
.43256 .17871 .43734 .18457 .44558 .18143 .44081 .17557 Metetra
.665 .737 .903 r
.44081 .17557 .44558 .18143 .45384 .17826 .44909 .17239 Metetra
.666 .738 .904 r
.44909 .17239 .45384 .17826 .46212 .17508 .45739 .1692 Metetra
.667 .739 .903 r
.45739 .1692 .46212 .17508 .47043 .17186 .46571 .16597 Metetra
.668 .739 .904 r
.46571 .16597 .47043 .17186 .47877 .16861 .47407 .16272 Metetra
.66 .737 .907 r
.47407 .16272 .47877 .16861 .48713 .16551 .48244 .15962 Metetra
.642 .731 .913 r
.48244 .15962 .48713 .16551 .49551 .16284 .49084 .15688 Metetra
.628 .724 .916 r
.49084 .15688 .49551 .16284 .50391 .16046 .49926 .15441 Metetra
.628 .72 .913 r
.49926 .15441 .50391 .16046 .51235 .1581 .50771 .15193 Metetra
.645 .723 .905 r
.50771 .15193 .51235 .1581 .52082 .15537 .5162 .1491 Metetra
.678 .732 .891 r
.5162 .1491 .52082 .15537 .52931 .15181 .52471 .14553 Metetra
.705 .742 .881 r
.52471 .14553 .52931 .15181 .53783 .14747 .53324 .14128 Metetra
.711 .748 .881 r
.53324 .14128 .53783 .14747 .54637 .14297 .5418 .13688 Metetra
.702 .75 .889 r
.5418 .13688 .54637 .14297 .55492 .13878 .55038 .13274 Metetra
.688 .748 .897 r
.55038 .13274 .55492 .13878 .56351 .13495 .55898 .12893 Metetra
.678 .746 .902 r
.55898 .12893 .56351 .13495 .57212 .13137 .56761 .12535 Metetra
.672 .744 .905 r
.56761 .12535 .57212 .13137 .58076 .12792 .57626 .12189 Metetra
.669 .743 .906 r
.57626 .12189 .58076 .12792 .58942 .12455 .58495 .1185 Metetra
.667 .742 .907 r
.58495 .1185 .58942 .12455 .59811 .1212 .59366 .11514 Metetra
.666 .742 .907 r
.59366 .11514 .59811 .1212 .60684 .11788 .6024 .11179 Metetra
.665 .742 .907 r
.6024 .11179 .60684 .11788 .61558 .11456 .61116 .10845 Metetra
.664 .742 .908 r
.61116 .10845 .61558 .11456 .62436 .11123 .61996 .10511 Metetra
.664 .742 .908 r
.61996 .10511 .62436 .11123 .63316 .10791 .62878 .10177 Metetra
.664 .741 .908 r
.62878 .10177 .63316 .10791 .642 .10458 .63763 .09841 Metetra
.664 .741 .908 r
.63763 .09841 .642 .10458 .65085 .10124 .64651 .09505 Metetra
.664 .741 .908 r
.64651 .09505 .65085 .10124 .65974 .09789 .65542 .09169 Metetra
.664 .741 .908 r
.65542 .09169 .65974 .09789 .66866 .09453 .66435 .08831 Metetra
.664 .741 .908 r
.66435 .08831 .66866 .09453 .6776 .09116 .67331 .08492 Metetra
.664 .741 .908 r
.67331 .08492 .6776 .09116 .68657 .08778 .68231 .08152 Metetra
.664 .741 .908 r
.68231 .08152 .68657 .08778 .69557 .08439 .69133 .07811 Metetra
.664 .741 .908 r
.69133 .07811 .69557 .08439 .7046 .081 .70038 .0747 Metetra
.664 .741 .908 r
.70038 .0747 .7046 .081 .71366 .07759 .70945 .07127 Metetra
.664 .741 .908 r
.70945 .07127 .71366 .07759 .72275 .07417 .71856 .06783 Metetra
.664 .741 .908 r
.71856 .06783 .72275 .07417 .73187 .07074 .7277 .06438 Metetra
.653 .737 .911 r
.08063 .30193 .08603 .307 .09323 .30446 .08783 .29938 Metetra
.655 .738 .911 r
.08783 .29938 .09323 .30446 .10045 .30188 .09506 .29679 Metetra
.657 .739 .91 r
.09506 .29679 .10045 .30188 .10769 .29925 .10232 .29414 Metetra
.659 .739 .909 r
.10232 .29414 .10769 .29925 .11496 .29659 .1096 .29147 Metetra
.659 .739 .909 r
.1096 .29147 .11496 .29659 .12226 .29391 .1169 .28878 Metetra
.659 .739 .909 r
.1169 .28878 .12226 .29391 .12957 .29122 .12423 .28608 Metetra
.659 .739 .909 r
.12423 .28608 .12957 .29122 .13691 .28853 .13158 .28338 Metetra
.659 .739 .909 r
.13158 .28338 .13691 .28853 .14426 .28585 .13894 .28067 Metetra
.658 .739 .91 r
.13894 .28067 .14426 .28585 .15164 .28317 .14633 .27798 Metetra
.657 .739 .91 r
.14633 .27798 .15164 .28317 .15903 .28051 .15374 .2753 Metetra
.655 .738 .911 r
.15374 .2753 .15903 .28051 .16644 .27787 .16116 .27264 Metetra
.653 .737 .911 r
.16116 .27264 .16644 .27787 .17387 .27526 .16861 .27001 Metetra
.65 .736 .912 r
.16861 .27001 .17387 .27526 .18132 .2727 .17607 .26742 Metetra
.648 .735 .913 r
.17607 .26742 .18132 .2727 .18878 .27018 .18354 .26487 Metetra
.646 .734 .913 r
.18354 .26487 .18878 .27018 .19627 .26769 .19104 .26234 Metetra
.65 .734 .91 r
.19104 .26234 .19627 .26769 .20378 .26512 .19857 .25972 Metetra
.661 .736 .905 r
.19857 .25972 .20378 .26512 .21135 .26231 .20615 .25689 Metetra
.675 .739 .899 r
.20615 .25689 .21135 .26231 .21896 .25918 .21377 .25377 Metetra
.681 .742 .897 r
.21377 .25377 .21896 .25918 .22661 .2559 .22143 .2505 Metetra
.681 .743 .898 r
.22143 .2505 .22661 .2559 .23428 .25261 .22911 .24722 Metetra
.679 .744 .9 r
.22911 .24722 .23428 .25261 .24196 .24939 .23681 .244 Metetra
.676 .743 .902 r
.23681 .244 .24196 .24939 .24966 .24622 .24452 .24083 Metetra
.673 .743 .903 r
.24452 .24083 .24966 .24622 .25738 .24311 .25225 .23771 Metetra
.671 .743 .904 r
.25225 .23771 .25738 .24311 .26511 .24004 .25999 .23462 Metetra
.669 .742 .905 r
.25999 .23462 .26511 .24004 .27286 .23699 .26776 .23156 Metetra
.668 .742 .906 r
.26776 .23156 .27286 .23699 .28063 .23397 .27554 .22852 Metetra
.667 .742 .906 r
.27554 .22852 .28063 .23397 .28842 .23096 .28335 .2255 Metetra
.666 .742 .907 r
.28335 .2255 .28842 .23096 .29624 .22797 .29118 .2225 Metetra
.664 .741 .907 r
.29118 .2225 .29624 .22797 .30407 .225 .29902 .21951 Metetra
.664 .741 .908 r
.29902 .21951 .30407 .225 .31192 .22203 .30689 .21652 Metetra
.664 .741 .907 r
.30689 .21652 .31192 .22203 .3198 .21905 .31479 .21351 Metetra
.667 .741 .906 r
.31479 .21351 .3198 .21905 .32771 .216 .32271 .21044 Metetra
.669 .742 .905 r
.32271 .21044 .32771 .216 .33564 .21289 .33066 .20732 Metetra
.67 .742 .905 r
.33066 .20732 .33564 .21289 .3436 .20976 .33863 .20418 Metetra
.669 .742 .905 r
.33863 .20418 .3436 .20976 .35158 .20663 .34662 .20103 Metetra
.667 .742 .906 r
.34662 .20103 .35158 .20663 .35958 .20355 .35464 .19793 Metetra
.659 .739 .909 r
.35464 .19793 .35958 .20355 .3676 .2006 .36267 .19497 Metetra
.654 .737 .911 r
.36267 .19497 .3676 .2006 .37563 .19776 .37072 .19211 Metetra
.65 .736 .912 r
.37072 .19211 .37563 .19776 .38369 .19499 .37879 .18931 Metetra
.645 .734 .914 r
.37879 .18931 .38369 .19499 .39177 .19232 .38688 .18661 Metetra
.639 .731 .915 r
.38688 .18661 .39177 .19232 .39987 .18976 .395 .184 Metetra
.637 .729 .915 r
.395 .184 .39987 .18976 .40799 .18725 .40314 .18143 Metetra
.641 .729 .912 r
.40314 .18143 .40799 .18725 .41615 .18464 .41131 .17877 Metetra
.652 .732 .908 r
.41131 .17877 .41615 .18464 .42434 .18177 .41952 .17589 Metetra
.66 .735 .905 r
.41952 .17589 .42434 .18177 .43256 .17871 .42775 .17283 Metetra
.664 .736 .904 r
.42775 .17283 .43256 .17871 .44081 .17557 .43602 .16968 Metetra
.665 .737 .904 r
.43602 .16968 .44081 .17557 .44909 .17239 .44431 .1665 Metetra
.666 .738 .904 r
.44431 .1665 .44909 .17239 .45739 .1692 .45263 .16329 Metetra
.667 .739 .904 r
.45263 .16329 .45739 .1692 .46571 .16597 .46097 .16006 Metetra
.667 .739 .904 r
.46097 .16006 .46571 .16597 .47407 .16272 .46934 .15681 Metetra
.66 .737 .907 r
.46934 .15681 .47407 .16272 .48244 .15962 .47773 .1537 Metetra
.644 .732 .912 r
.47773 .1537 .48244 .15962 .49084 .15688 .48614 .1509 Metetra
.632 .725 .915 r
.48614 .1509 .49084 .15688 .49926 .15441 .49458 .14833 Metetra
.632 .722 .912 r
.49458 .14833 .49926 .15441 .50771 .15193 .50305 .14575 Metetra
.648 .725 .905 r
.50305 .14575 .50771 .15193 .5162 .1491 .51155 .14283 Metetra
.678 .734 .893 r
.51155 .14283 .5162 .1491 .52471 .14553 .52008 .13925 Metetra
.702 .743 .883 r
.52008 .13925 .52471 .14553 .53324 .14128 .52863 .13508 Metetra
.707 .748 .884 r
.52863 .13508 .53324 .14128 .5418 .13688 .5372 .13076 Metetra
.699 .749 .891 r
.5372 .13076 .5418 .13688 .55038 .13274 .5458 .12667 Metetra
.687 .748 .898 r
.5458 .12667 .55038 .13274 .55898 .12893 .55442 .12288 Metetra
.678 .746 .902 r
.55442 .12288 .55898 .12893 .56761 .12535 .56307 .11929 Metetra
.672 .744 .905 r
.56307 .11929 .56761 .12535 .57626 .12189 .57174 .11582 Metetra
.669 .743 .906 r
.57174 .11582 .57626 .12189 .58495 .1185 .58045 .11241 Metetra
.667 .742 .907 r
.58045 .11241 .58495 .1185 .59366 .11514 .58918 .10903 Metetra
.666 .742 .907 r
.58918 .10903 .59366 .11514 .6024 .11179 .59793 .10567 Metetra
.665 .742 .907 r
.59793 .10567 .6024 .11179 .61116 .10845 .60672 .10231 Metetra
.664 .742 .908 r
.60672 .10231 .61116 .10845 .61996 .10511 .61553 .09895 Metetra
.664 .742 .908 r
.61553 .09895 .61996 .10511 .62878 .10177 .62437 .09559 Metetra
.664 .741 .908 r
.62437 .09559 .62878 .10177 .63763 .09841 .63324 .09222 Metetra
.664 .741 .908 r
.63324 .09222 .63763 .09841 .64651 .09505 .64214 .08884 Metetra
.664 .741 .908 r
.64214 .08884 .64651 .09505 .65542 .09169 .65107 .08545 Metetra
.664 .741 .908 r
.65107 .08545 .65542 .09169 .66435 .08831 .66002 .08205 Metetra
.664 .741 .908 r
.66002 .08205 .66435 .08831 .67331 .08492 .669 .07864 Metetra
.664 .741 .908 r
.669 .07864 .67331 .08492 .68231 .08152 .67801 .07523 Metetra
.664 .741 .908 r
.67801 .07523 .68231 .08152 .69133 .07811 .68705 .0718 Metetra
.664 .741 .908 r
.68705 .0718 .69133 .07811 .70038 .0747 .69612 .06836 Metetra
.664 .741 .908 r
.69612 .06836 .70038 .0747 .70945 .07127 .70522 .06491 Metetra
.664 .741 .908 r
.70522 .06491 .70945 .07127 .71856 .06783 .71435 .06145 Metetra
.664 .741 .908 r
.71435 .06145 .71856 .06783 .7277 .06438 .72351 .05798 Metetra
.653 .737 .911 r
.07519 .29684 .08063 .30193 .08783 .29938 .08241 .29427 Metetra
.655 .738 .911 r
.08241 .29427 .08783 .29938 .09506 .29679 .08965 .29167 Metetra
.657 .739 .91 r
.08965 .29167 .09506 .29679 .10232 .29414 .09691 .28901 Metetra
.659 .739 .909 r
.09691 .28901 .10232 .29414 .1096 .29147 .10421 .28633 Metetra
.659 .739 .909 r
.10421 .28633 .1096 .29147 .1169 .28878 .11152 .28362 Metetra
.659 .739 .909 r
.11152 .28362 .1169 .28878 .12423 .28608 .11886 .28091 Metetra
.659 .739 .909 r
.11886 .28091 .12423 .28608 .13158 .28338 .12622 .27819 Metetra
.659 .739 .909 r
.12622 .27819 .13158 .28338 .13894 .28067 .1336 .27547 Metetra
.658 .739 .91 r
.1336 .27547 .13894 .28067 .14633 .27798 .141 .27276 Metetra
.657 .739 .91 r
.141 .27276 .14633 .27798 .15374 .2753 .14842 .27006 Metetra
.655 .738 .911 r
.14842 .27006 .15374 .2753 .16116 .27264 .15585 .26738 Metetra
.653 .737 .911 r
.15585 .26738 .16116 .27264 .16861 .27001 .16331 .26473 Metetra
.651 .736 .912 r
.16331 .26473 .16861 .27001 .17607 .26742 .17078 .26211 Metetra
.649 .735 .912 r
.17078 .26211 .17607 .26742 .18354 .26487 .17827 .25953 Metetra
.648 .734 .912 r
.17827 .25953 .18354 .26487 .19104 .26234 .18579 .25696 Metetra
.651 .734 .91 r
.18579 .25696 .19104 .26234 .19857 .25972 .19333 .2543 Metetra
.662 .736 .905 r
.19333 .2543 .19857 .25972 .20615 .25689 .20092 .25144 Metetra
.674 .739 .9 r
.20092 .25144 .20615 .25689 .21377 .25377 .20856 .24833 Metetra
.68 .742 .898 r
.20856 .24833 .21377 .25377 .22143 .2505 .21623 .24507 Metetra
.68 .743 .898 r
.21623 .24507 .22143 .2505 .22911 .24722 .22392 .2418 Metetra
.678 .743 .9 r
.22392 .2418 .22911 .24722 .23681 .244 .23163 .23858 Metetra
.675 .743 .902 r
.23163 .23858 .23681 .244 .24452 .24083 .23935 .2354 Metetra
.673 .743 .903 r
.23935 .2354 .24452 .24083 .25225 .23771 .24709 .23227 Metetra
.671 .743 .904 r
.24709 .23227 .25225 .23771 .25999 .23462 .25485 .22918 Metetra
.669 .742 .905 r
.25485 .22918 .25999 .23462 .26776 .23156 .26263 .2261 Metetra
.668 .742 .906 r
.26263 .2261 .26776 .23156 .27554 .22852 .27043 .22305 Metetra
.667 .742 .906 r
.27043 .22305 .27554 .22852 .28335 .2255 .27825 .22001 Metetra
.666 .742 .907 r
.27825 .22001 .28335 .2255 .29118 .2225 .28609 .21699 Metetra
.664 .741 .907 r
.28609 .21699 .29118 .2225 .29902 .21951 .29395 .21398 Metetra
.664 .741 .907 r
.29395 .21398 .29902 .21951 .30689 .21652 .30184 .21097 Metetra
.665 .741 .907 r
.30184 .21097 .30689 .21652 .31479 .21351 .30974 .20794 Metetra
.667 .741 .906 r
.30974 .20794 .31479 .21351 .32271 .21044 .31768 .20486 Metetra
.669 .742 .905 r
.31768 .20486 .32271 .21044 .33066 .20732 .32564 .20173 Metetra
.67 .742 .905 r
.32564 .20173 .33066 .20732 .33863 .20418 .33363 .19857 Metetra
.669 .743 .905 r
.33363 .19857 .33863 .20418 .34662 .20103 .34164 .1954 Metetra
.667 .742 .906 r
.34164 .1954 .34662 .20103 .35464 .19793 .34967 .19228 Metetra
.659 .739 .909 r
.34967 .19228 .35464 .19793 .36267 .19497 .35771 .1893 Metetra
.654 .737 .911 r
.35771 .1893 .36267 .19497 .37072 .19211 .36578 .18642 Metetra
.65 .736 .912 r
.36578 .18642 .37072 .19211 .37879 .18931 .37387 .18361 Metetra
.646 .734 .913 r
.37387 .18361 .37879 .18931 .38688 .18661 .38197 .18087 Metetra
.641 .732 .914 r
.38197 .18087 .38688 .18661 .395 .184 .3901 .17821 Metetra
.638 .73 .914 r
.3901 .17821 .395 .184 .40314 .18143 .39826 .1756 Metetra
.643 .73 .912 r
.39826 .1756 .40314 .18143 .41131 .17877 .40645 .17289 Metetra
.652 .732 .908 r
.40645 .17289 .41131 .17877 .41952 .17589 .41467 .16999 Metetra
.66 .735 .905 r
.41467 .16999 .41952 .17589 .42775 .17283 .42292 .16692 Metetra
.664 .736 .904 r
.42292 .16692 .42775 .17283 .43602 .16968 .4312 .16376 Metetra
.665 .737 .904 r
.4312 .16376 .43602 .16968 .44431 .1665 .43951 .16057 Metetra
.665 .738 .904 r
.43951 .16057 .44431 .1665 .45263 .16329 .44784 .15736 Metetra
.666 .739 .904 r
.44784 .15736 .45263 .16329 .46097 .16006 .4562 .15411 Metetra
.666 .739 .904 r
.4562 .15411 .46097 .16006 .46934 .15681 .46458 .15086 Metetra
.66 .737 .907 r
.46458 .15086 .46934 .15681 .47773 .1537 .47299 .14774 Metetra
.646 .732 .912 r
.47299 .14774 .47773 .1537 .48614 .1509 .48142 .14489 Metetra
.635 .727 .914 r
.48142 .14489 .48614 .1509 .49458 .14833 .48988 .14223 Metetra
.636 .724 .912 r
.48988 .14223 .49458 .14833 .50305 .14575 .49836 .13955 Metetra
.651 .727 .905 r
.49836 .13955 .50305 .14575 .51155 .14283 .50688 .13655 Metetra
.677 .735 .894 r
.50688 .13655 .51155 .14283 .52008 .13925 .51542 .13297 Metetra
.699 .743 .885 r
.51542 .13297 .52008 .13925 .52863 .13508 .52399 .12886 Metetra
.704 .748 .886 r
.52399 .12886 .52863 .13508 .5372 .13076 .53259 .12462 Metetra
.696 .749 .892 r
.53259 .12462 .5372 .13076 .5458 .12667 .5412 .12058 Metetra
.685 .747 .898 r
.5412 .12058 .5458 .12667 .55442 .12288 .54984 .11679 Metetra
.677 .745 .902 r
.54984 .11679 .55442 .12288 .56307 .11929 .5585 .1132 Metetra
.672 .744 .905 r
.5585 .1132 .56307 .11929 .57174 .11582 .5672 .10972 Metetra
.668 .743 .906 r
.5672 .10972 .57174 .11582 .58045 .11241 .57592 .10629 Metetra
.667 .742 .907 r
.57592 .10629 .58045 .11241 .58918 .10903 .58467 .1029 Metetra
.666 .742 .907 r
.58467 .1029 .58918 .10903 .59793 .10567 .59344 .09951 Metetra
.665 .742 .907 r
.59344 .09951 .59793 .10567 .60672 .10231 .60225 .09614 Metetra
.664 .742 .908 r
.60225 .09614 .60672 .10231 .61553 .09895 .61108 .09276 Metetra
.664 .742 .908 r
.61108 .09276 .61553 .09895 .62437 .09559 .61994 .08937 Metetra
.664 .741 .908 r
.61994 .08937 .62437 .09559 .63324 .09222 .62883 .08598 Metetra
.664 .741 .908 r
.62883 .08598 .63324 .09222 .64214 .08884 .63774 .08258 Metetra
.664 .741 .908 r
.63774 .08258 .64214 .08884 .65107 .08545 .64669 .07917 Metetra
.664 .741 .908 r
.64669 .07917 .65107 .08545 .66002 .08205 .65566 .07576 Metetra
.664 .741 .908 r
.65566 .07576 .66002 .08205 .669 .07864 .66466 .07233 Metetra
.664 .741 .908 r
.66466 .07233 .669 .07864 .67801 .07523 .6737 .06889 Metetra
.664 .741 .908 r
.6737 .06889 .67801 .07523 .68705 .0718 .68276 .06544 Metetra
.664 .741 .908 r
.68276 .06544 .68705 .0718 .69612 .06836 .69185 .06198 Metetra
.664 .741 .908 r
.69185 .06198 .69612 .06836 .70522 .06491 .70096 .05851 Metetra
.664 .741 .908 r
.70096 .05851 .70522 .06491 .71435 .06145 .71011 .05503 Metetra
.664 .741 .908 r
.71011 .05503 .71435 .06145 .72351 .05798 .71929 .05154 Metetra
.653 .738 .912 r
.06973 .29172 .07519 .29684 .08241 .29427 .07695 .28915 Metetra
.655 .738 .911 r
.07695 .28915 .08241 .29427 .08965 .29167 .0842 .28653 Metetra
.657 .739 .91 r
.0842 .28653 .08965 .29167 .09691 .28901 .09148 .28386 Metetra
.659 .739 .909 r
.09148 .28386 .09691 .28901 .10421 .28633 .09879 .28116 Metetra
.659 .739 .909 r
.09879 .28116 .10421 .28633 .11152 .28362 .10611 .27844 Metetra
.659 .74 .909 r
.10611 .27844 .11152 .28362 .11886 .28091 .11346 .27571 Metetra
.659 .739 .909 r
.11346 .27571 .11886 .28091 .12622 .27819 .12083 .27297 Metetra
.659 .739 .909 r
.12083 .27297 .12622 .27819 .1336 .27547 .12822 .27024 Metetra
.658 .739 .91 r
.12822 .27024 .1336 .27547 .141 .27276 .13563 .26751 Metetra
.657 .739 .91 r
.13563 .26751 .141 .27276 .14842 .27006 .14307 .26479 Metetra
.656 .738 .91 r
.14307 .26479 .14842 .27006 .15585 .26738 .15052 .26209 Metetra
.654 .737 .911 r
.15052 .26209 .15585 .26738 .16331 .26473 .15798 .25942 Metetra
.652 .737 .912 r
.15798 .25942 .16331 .26473 .17078 .26211 .16547 .25677 Metetra
.65 .736 .912 r
.16547 .25677 .17078 .26211 .17827 .25953 .17298 .25416 Metetra
.649 .735 .912 r
.17298 .25416 .17827 .25953 .18579 .25696 .1805 .25155 Metetra
.653 .735 .91 r
.1805 .25155 .18579 .25696 .19333 .2543 .18806 .24886 Metetra
.662 .737 .905 r
.18806 .24886 .19333 .2543 .20092 .25144 .19566 .24598 Metetra
.673 .739 .9 r
.19566 .24598 .20092 .25144 .20856 .24833 .20331 .24287 Metetra
.679 .742 .898 r
.20331 .24287 .20856 .24833 .21623 .24507 .21099 .23962 Metetra
.679 .743 .899 r
.21099 .23962 .21623 .24507 .22392 .2418 .2187 .23636 Metetra
.677 .743 .901 r
.2187 .23636 .22392 .2418 .23163 .23858 .22642 .23313 Metetra
.675 .743 .902 r
.22642 .23313 .23163 .23858 .23935 .2354 .23416 .22995 Metetra
.673 .743 .903 r
.23416 .22995 .23935 .2354 .24709 .23227 .24191 .22681 Metetra
.671 .743 .904 r
.24191 .22681 .24709 .23227 .25485 .22918 .24968 .2237 Metetra
.669 .742 .905 r
.24968 .2237 .25485 .22918 .26263 .2261 .25747 .22061 Metetra
.668 .742 .906 r
.25747 .22061 .26263 .2261 .27043 .22305 .26529 .21755 Metetra
.667 .742 .906 r
.26529 .21755 .27043 .22305 .27825 .22001 .27312 .21449 Metetra
.666 .742 .907 r
.27312 .21449 .27825 .22001 .28609 .21699 .28097 .21145 Metetra
.665 .741 .907 r
.28097 .21145 .28609 .21699 .29395 .21398 .28885 .20843 Metetra
.664 .741 .907 r
.28885 .20843 .29395 .21398 .30184 .21097 .29675 .2054 Metetra
.665 .741 .907 r
.29675 .2054 .30184 .21097 .30974 .20794 .30467 .20235 Metetra
.667 .742 .906 r
.30467 .20235 .30974 .20794 .31768 .20486 .31262 .19925 Metetra
.669 .742 .905 r
.31262 .19925 .31768 .20486 .32564 .20173 .3206 .1961 Metetra
.669 .742 .905 r
.3206 .1961 .32564 .20173 .33363 .19857 .3286 .19293 Metetra
.669 .743 .905 r
.3286 .19293 .33363 .19857 .34164 .1954 .33662 .18974 Metetra
.667 .742 .906 r
.33662 .18974 .34164 .1954 .34967 .19228 .34467 .1866 Metetra
.659 .739 .909 r
.34467 .1866 .34967 .19228 .35771 .1893 .35273 .18361 Metetra
.654 .738 .911 r
.35273 .18361 .35771 .1893 .36578 .18642 .36081 .18071 Metetra
.651 .736 .912 r
.36081 .18071 .36578 .18642 .37387 .18361 .36891 .17787 Metetra
.647 .734 .913 r
.36891 .17787 .37387 .18361 .38197 .18087 .37703 .17509 Metetra
.642 .732 .914 r
.37703 .17509 .38197 .18087 .3901 .17821 .38518 .1724 Metetra
.64 .73 .914 r
.38518 .1724 .3901 .17821 .39826 .1756 .39335 .16973 Metetra
.644 .731 .912 r
.39335 .16973 .39826 .1756 .40645 .17289 .40156 .16698 Metetra
.652 .733 .908 r
.40156 .16698 .40645 .17289 .41467 .16999 .40979 .16406 Metetra
.659 .735 .906 r
.40979 .16406 .41467 .16999 .42292 .16692 .41806 .16098 Metetra
.663 .737 .905 r
.41806 .16098 .42292 .16692 .4312 .16376 .42636 .15782 Metetra
.664 .737 .904 r
.42636 .15782 .4312 .16376 .43951 .16057 .43468 .15462 Metetra
.665 .738 .904 r
.43468 .15462 .43951 .16057 .44784 .15736 .44303 .15139 Metetra
.666 .739 .904 r
.44303 .15139 .44784 .15736 .4562 .15411 .4514 .14814 Metetra
.666 .739 .905 r
.4514 .14814 .4562 .15411 .46458 .15086 .4598 .14488 Metetra
.66 .737 .907 r
.4598 .14488 .46458 .15086 .47299 .14774 .46823 .14174 Metetra
.648 .733 .911 r
.46823 .14174 .47299 .14774 .48142 .14489 .47667 .13884 Metetra
.639 .728 .913 r
.47667 .13884 .48142 .14489 .48988 .14223 .48514 .1361 Metetra
.64 .726 .911 r
.48514 .1361 .48988 .14223 .49836 .13955 .49364 .13333 Metetra
.654 .729 .904 r
.49364 .13333 .49836 .13955 .50688 .13655 .50218 .13027 Metetra
.677 .736 .895 r
.50218 .13027 .50688 .13655 .51542 .13297 .51074 .12668 Metetra
.696 .743 .887 r
.51074 .12668 .51542 .13297 .52399 .12886 .51933 .12262 Metetra
.701 .747 .888 r
.51933 .12262 .52399 .12886 .53259 .12462 .52794 .11845 Metetra
.694 .748 .893 r
.52794 .11845 .53259 .12462 .5412 .12058 .53657 .11445 Metetra
.684 .747 .899 r
.53657 .11445 .5412 .12058 .54984 .11679 .54523 .11067 Metetra
.676 .745 .903 r
.54523 .11067 .54984 .11679 .5585 .1132 .55391 .10707 Metetra
.671 .744 .905 r
.55391 .10707 .5585 .1132 .5672 .10972 .56263 .10358 Metetra
.668 .743 .906 r
.56263 .10358 .5672 .10972 .57592 .10629 .57137 .10014 Metetra
.667 .742 .907 r
.57137 .10014 .57592 .10629 .58467 .1029 .58013 .09672 Metetra
.665 .742 .907 r
.58013 .09672 .58467 .1029 .59344 .09951 .58893 .09332 Metetra
.665 .742 .907 r
.58893 .09332 .59344 .09951 .60225 .09614 .59775 .08992 Metetra
.664 .742 .908 r
.59775 .08992 .60225 .09614 .61108 .09276 .6066 .08653 Metetra
.664 .742 .908 r
.6066 .08653 .61108 .09276 .61994 .08937 .61548 .08312 Metetra
.664 .741 .908 r
.61548 .08312 .61994 .08937 .62883 .08598 .62439 .07971 Metetra
.664 .741 .908 r
.62439 .07971 .62883 .08598 .63774 .08258 .63332 .07629 Metetra
.664 .741 .908 r
.63332 .07629 .63774 .08258 .64669 .07917 .64229 .07286 Metetra
.664 .741 .908 r
.64229 .07286 .64669 .07917 .65566 .07576 .65128 .06943 Metetra
.664 .741 .908 r
.65128 .06943 .65566 .07576 .66466 .07233 .6603 .06598 Metetra
.664 .741 .908 r
.6603 .06598 .66466 .07233 .6737 .06889 .66935 .06252 Metetra
.664 .741 .908 r
.66935 .06252 .6737 .06889 .68276 .06544 .67843 .05905 Metetra
.664 .741 .908 r
.67843 .05905 .68276 .06544 .69185 .06198 .68754 .05557 Metetra
.664 .741 .908 r
.68754 .05557 .69185 .06198 .70096 .05851 .69668 .05208 Metetra
.664 .741 .908 r
.69668 .05208 .70096 .05851 .71011 .05503 .70585 .04858 Metetra
.664 .741 .908 r
.70585 .04858 .71011 .05503 .71929 .05154 .71505 .04507 Metetra
.653 .738 .912 r
.06423 .28657 .06973 .29172 .07695 .28915 .07147 .28399 Metetra
.655 .738 .911 r
.07147 .28399 .07695 .28915 .0842 .28653 .07873 .28136 Metetra
.657 .739 .91 r
.07873 .28136 .0842 .28653 .09148 .28386 .08602 .27868 Metetra
.659 .739 .909 r
.08602 .27868 .09148 .28386 .09879 .28116 .09334 .27596 Metetra
.659 .739 .909 r
.09334 .27596 .09879 .28116 .10611 .27844 .10067 .27323 Metetra
.659 .74 .909 r
.10067 .27323 .10611 .27844 .11346 .27571 .10803 .27048 Metetra
.659 .739 .909 r
.10803 .27048 .11346 .27571 .12083 .27297 .11542 .26773 Metetra
.659 .739 .909 r
.11542 .26773 .12083 .27297 .12822 .27024 .12282 .26498 Metetra
.658 .739 .91 r
.12282 .26498 .12822 .27024 .13563 .26751 .13024 .26223 Metetra
.657 .739 .91 r
.13024 .26223 .13563 .26751 .14307 .26479 .13769 .2595 Metetra
.656 .738 .91 r
.13769 .2595 .14307 .26479 .15052 .26209 .14515 .25678 Metetra
.654 .738 .911 r
.14515 .25678 .15052 .26209 .15798 .25942 .15263 .25408 Metetra
.652 .737 .911 r
.15263 .25408 .15798 .25942 .16547 .25677 .16013 .25141 Metetra
.65 .736 .912 r
.16013 .25141 .16547 .25677 .17298 .25416 .16765 .24877 Metetra
.65 .735 .911 r
.16765 .24877 .17298 .25416 .1805 .25155 .17519 .24612 Metetra
.654 .735 .909 r
.17519 .24612 .1805 .25155 .18806 .24886 .18276 .2434 Metetra
.662 .737 .905 r
.18276 .2434 .18806 .24886 .19566 .24598 .19038 .24049 Metetra
.672 .74 .901 r
.19038 .24049 .19566 .24598 .20331 .24287 .19804 .23738 Metetra
.678 .742 .899 r
.19804 .23738 .20331 .24287 .21099 .23962 .20573 .23414 Metetra
.679 .743 .899 r
.20573 .23414 .21099 .23962 .2187 .23636 .21345 .23088 Metetra
.677 .743 .901 r
.21345 .23088 .2187 .23636 .22642 .23313 .22118 .22765 Metetra
.674 .743 .902 r
.22118 .22765 .22642 .23313 .23416 .22995 .22893 .22446 Metetra
.672 .743 .904 r
.22893 .22446 .23416 .22995 .24191 .22681 .2367 .22131 Metetra
.67 .743 .904 r
.2367 .22131 .24191 .22681 .24968 .2237 .24448 .21819 Metetra
.669 .742 .905 r
.24448 .21819 .24968 .2237 .25747 .22061 .25229 .21509 Metetra
.668 .742 .906 r
.25229 .21509 .25747 .22061 .26529 .21755 .26011 .21201 Metetra
.667 .742 .906 r
.26011 .21201 .26529 .21755 .27312 .21449 .26796 .20894 Metetra
.666 .742 .907 r
.26796 .20894 .27312 .21449 .28097 .21145 .27583 .20589 Metetra
.665 .741 .907 r
.27583 .20589 .28097 .21145 .28885 .20843 .28372 .20284 Metetra
.664 .741 .907 r
.28372 .20284 .28885 .20843 .29675 .2054 .29163 .19979 Metetra
.665 .741 .907 r
.29163 .19979 .29675 .2054 .30467 .20235 .29957 .19672 Metetra
.667 .742 .906 r
.29957 .19672 .30467 .20235 .31262 .19925 .30754 .19361 Metetra
.668 .742 .905 r
.30754 .19361 .31262 .19925 .3206 .1961 .31553 .19044 Metetra
.669 .742 .905 r
.31553 .19044 .3206 .1961 .3286 .19293 .32354 .18726 Metetra
.669 .743 .905 r
.32354 .18726 .3286 .19293 .33662 .18974 .33158 .18406 Metetra
.667 .742 .906 r
.33158 .18406 .33662 .18974 .34467 .1866 .33964 .18089 Metetra
.659 .739 .909 r
.33964 .18089 .34467 .1866 .35273 .18361 .34772 .17788 Metetra
.654 .738 .911 r
.34772 .17788 .35273 .18361 .36081 .18071 .35581 .17496 Metetra
.651 .736 .912 r
.35581 .17496 .36081 .18071 .36891 .17787 .36393 .1721 Metetra
.647 .735 .913 r
.36393 .1721 .36891 .17787 .37703 .17509 .37207 .16929 Metetra
.643 .733 .914 r
.37207 .16929 .37703 .17509 .38518 .1724 .38023 .16655 Metetra
.642 .731 .914 r
.38023 .16655 .38518 .1724 .39335 .16973 .38842 .16384 Metetra
.645 .731 .912 r
.38842 .16384 .39335 .16973 .40156 .16698 .39664 .16105 Metetra
.652 .733 .908 r
.39664 .16105 .40156 .16698 .40979 .16406 .40489 .15811 Metetra
.659 .735 .906 r
.40489 .15811 .40979 .16406 .41806 .16098 .41317 .15502 Metetra
.663 .737 .905 r
.41317 .15502 .41806 .16098 .42636 .15782 .42148 .15184 Metetra
.664 .738 .905 r
.42148 .15184 .42636 .15782 .43468 .15462 .42982 .14863 Metetra
.665 .738 .905 r
.42982 .14863 .43468 .15462 .44303 .15139 .43819 .1454 Metetra
.665 .739 .905 r
.43819 .1454 .44303 .15139 .4514 .14814 .44658 .14213 Metetra
.665 .739 .905 r
.44658 .14213 .4514 .14814 .4598 .14488 .45499 .13887 Metetra
.66 .737 .907 r
.45499 .13887 .4598 .14488 .46823 .14174 .46343 .13571 Metetra
.649 .733 .911 r
.46343 .13571 .46823 .14174 .47667 .13884 .4719 .13276 Metetra
.641 .729 .912 r
.4719 .13276 .47667 .13884 .48514 .1361 .48038 .12995 Metetra
.643 .728 .91 r
.48038 .12995 .48514 .1361 .49364 .13333 .4889 .1271 Metetra
.656 .73 .904 r
.4889 .1271 .49364 .13333 .50218 .13027 .49745 .12398 Metetra
.676 .736 .896 r
.49745 .12398 .50218 .13027 .51074 .12668 .50603 .12038 Metetra
.693 .743 .889 r
.50603 .12038 .51074 .12668 .51933 .12262 .51464 .11637 Metetra
.698 .747 .889 r
.51464 .11637 .51933 .12262 .52794 .11845 .52327 .11225 Metetra
.692 .748 .894 r
.52327 .11225 .52794 .11845 .53657 .11445 .53192 .10828 Metetra
.683 .747 .899 r
.53192 .10828 .53657 .11445 .54523 .11067 .5406 .10452 Metetra
.676 .745 .903 r
.5406 .10452 .54523 .11067 .55391 .10707 .5493 .10091 Metetra
.671 .744 .905 r
.5493 .10091 .55391 .10707 .56263 .10358 .55803 .0974 Metetra
.668 .743 .906 r
.55803 .0974 .56263 .10358 .57137 .10014 .56679 .09394 Metetra
.666 .742 .907 r
.56679 .09394 .57137 .10014 .58013 .09672 .57557 .09051 Metetra
.665 .742 .907 r
.57557 .09051 .58013 .09672 .58893 .09332 .58438 .08709 Metetra
.665 .742 .907 r
.58438 .08709 .58893 .09332 .59775 .08992 .59322 .08368 Metetra
.664 .742 .908 r
.59322 .08368 .59775 .08992 .6066 .08653 .60209 .08026 Metetra
.664 .742 .908 r
.60209 .08026 .6066 .08653 .61548 .08312 .61099 .07683 Metetra
.664 .741 .908 r
.61099 .07683 .61548 .08312 .62439 .07971 .61992 .0734 Metetra
.664 .741 .908 r
.61992 .0734 .62439 .07971 .63332 .07629 .62887 .06996 Metetra
.664 .741 .908 r
.62887 .06996 .63332 .07629 .64229 .07286 .63786 .06652 Metetra
.664 .741 .908 r
.63786 .06652 .64229 .07286 .65128 .06943 .64687 .06306 Metetra
.664 .741 .908 r
.64687 .06306 .65128 .06943 .6603 .06598 .65591 .05959 Metetra
.664 .741 .908 r
.65591 .05959 .6603 .06598 .66935 .06252 .66498 .05611 Metetra
.664 .741 .908 r
.66498 .05611 .66935 .06252 .67843 .05905 .67408 .05262 Metetra
.664 .741 .908 r
.67408 .05262 .67843 .05905 .68754 .05557 .68321 .04912 Metetra
.664 .741 .908 r
.68321 .04912 .68754 .05557 .69668 .05208 .69237 .04561 Metetra
.664 .741 .908 r
.69237 .04561 .69668 .05208 .70585 .04858 .70156 .04209 Metetra
.664 .741 .908 r
.70156 .04209 .70585 .04858 .71505 .04507 .71078 .03855 Metetra
.653 .738 .912 r
.05871 .28141 .06423 .28657 .07147 .28399 .06595 .27881 Metetra
.655 .738 .911 r
.06595 .27881 .07147 .28399 .07873 .28136 .07323 .27616 Metetra
.657 .739 .91 r
.07323 .27616 .07873 .28136 .08602 .27868 .08053 .27347 Metetra
.658 .739 .91 r
.08053 .27347 .08602 .27868 .09334 .27596 .08786 .27074 Metetra
.659 .739 .909 r
.08786 .27074 .09334 .27596 .10067 .27323 .09521 .26799 Metetra
.659 .74 .909 r
.09521 .26799 .10067 .27323 .10803 .27048 .10258 .26523 Metetra
.659 .74 .909 r
.10258 .26523 .10803 .27048 .11542 .26773 .10997 .26246 Metetra
.659 .739 .909 r
.10997 .26246 .11542 .26773 .12282 .26498 .11739 .25969 Metetra
.658 .739 .91 r
.11739 .25969 .12282 .26498 .13024 .26223 .12482 .25693 Metetra
.657 .739 .91 r
.12482 .25693 .13024 .26223 .13769 .2595 .13228 .25418 Metetra
.656 .738 .91 r
.13228 .25418 .13769 .2595 .14515 .25678 .13975 .25144 Metetra
.655 .738 .911 r
.13975 .25144 .14515 .25678 .15263 .25408 .14725 .24872 Metetra
.653 .737 .911 r
.14725 .24872 .15263 .25408 .16013 .25141 .15476 .24602 Metetra
.651 .736 .912 r
.15476 .24602 .16013 .25141 .16765 .24877 .16229 .24335 Metetra
.651 .735 .911 r
.16229 .24335 .16765 .24877 .17519 .24612 .16985 .24067 Metetra
.655 .736 .909 r
.16985 .24067 .17519 .24612 .18276 .2434 .17744 .23791 Metetra
.663 .737 .905 r
.17744 .23791 .18276 .2434 .19038 .24049 .18507 .23499 Metetra
.672 .74 .901 r
.18507 .23499 .19038 .24049 .19804 .23738 .19274 .23187 Metetra
.677 .742 .9 r
.19274 .23187 .19804 .23738 .20573 .23414 .20044 .22864 Metetra
.678 .743 .9 r
.20044 .22864 .20573 .23414 .21345 .23088 .20817 .22538 Metetra
.676 .743 .901 r
.20817 .22538 .21345 .23088 .22118 .22765 .21591 .22215 Metetra
.674 .743 .903 r
.21591 .22215 .22118 .22765 .22893 .22446 .22368 .21895 Metetra
.672 .743 .904 r
.22368 .21895 .22893 .22446 .2367 .22131 .23146 .21579 Metetra
.67 .743 .905 r
.23146 .21579 .2367 .22131 .24448 .21819 .23925 .21266 Metetra
.669 .742 .905 r
.23925 .21266 .24448 .21819 .25229 .21509 .24707 .20954 Metetra
.668 .742 .906 r
.24707 .20954 .25229 .21509 .26011 .21201 .25491 .20645 Metetra
.667 .742 .906 r
.25491 .20645 .26011 .21201 .26796 .20894 .26278 .20336 Metetra
.666 .742 .907 r
.26278 .20336 .26796 .20894 .27583 .20589 .27066 .20029 Metetra
.665 .741 .907 r
.27066 .20029 .27583 .20589 .28372 .20284 .27856 .19723 Metetra
.664 .741 .907 r
.27856 .19723 .28372 .20284 .29163 .19979 .28649 .19416 Metetra
.665 .741 .907 r
.28649 .19416 .29163 .19979 .29957 .19672 .29444 .19107 Metetra
.667 .742 .906 r
.29444 .19107 .29957 .19672 .30754 .19361 .30242 .18793 Metetra
.668 .742 .906 r
.30242 .18793 .30754 .19361 .31553 .19044 .31043 .18476 Metetra
.669 .742 .905 r
.31043 .18476 .31553 .19044 .32354 .18726 .31846 .18155 Metetra
.669 .743 .905 r
.31846 .18155 .32354 .18726 .33158 .18406 .32651 .17834 Metetra
.668 .742 .906 r
.32651 .17834 .33158 .18406 .33964 .18089 .33459 .17515 Metetra
.659 .739 .909 r
.33459 .17515 .33964 .18089 .34772 .17788 .34268 .17213 Metetra
.654 .738 .911 r
.34268 .17213 .34772 .17788 .35581 .17496 .35079 .16919 Metetra
.652 .737 .912 r
.35079 .16919 .35581 .17496 .36393 .1721 .35892 .16629 Metetra
.648 .735 .913 r
.35892 .16629 .36393 .1721 .37207 .16929 .36707 .16346 Metetra
.645 .733 .913 r
.36707 .16346 .37207 .16929 .38023 .16655 .37525 .16068 Metetra
.643 .732 .913 r
.37525 .16068 .38023 .16655 .38842 .16384 .38346 .15792 Metetra
.646 .732 .911 r
.38346 .15792 .38842 .16384 .39664 .16105 .39169 .1551 Metetra
.653 .733 .909 r
.39169 .1551 .39664 .16105 .40489 .15811 .39996 .15213 Metetra
.659 .735 .906 r
.39996 .15213 .40489 .15811 .41317 .15502 .40826 .14903 Metetra
.662 .737 .905 r
.40826 .14903 .41317 .15502 .42148 .15184 .41658 .14584 Metetra
.664 .738 .905 r
.41658 .14584 .42148 .15184 .42982 .14863 .42494 .14262 Metetra
.664 .738 .905 r
.42494 .14262 .42982 .14863 .43819 .1454 .43332 .13937 Metetra
.665 .739 .905 r
.43332 .13937 .43819 .1454 .44658 .14213 .44172 .1361 Metetra
.665 .739 .905 r
.44172 .1361 .44658 .14213 .45499 .13887 .45016 .13283 Metetra
.66 .737 .907 r
.45016 .13283 .45499 .13887 .46343 .13571 .45861 .12965 Metetra
.651 .734 .91 r
.45861 .12965 .46343 .13571 .4719 .13276 .46709 .12665 Metetra
.644 .73 .912 r
.46709 .12665 .4719 .13276 .48038 .12995 .4756 .12377 Metetra
.646 .729 .91 r
.4756 .12377 .48038 .12995 .4889 .1271 .48413 .12085 Metetra
.657 .731 .904 r
.48413 .12085 .4889 .1271 .49745 .12398 .4927 .11767 Metetra
.676 .737 .896 r
.4927 .11767 .49745 .12398 .50603 .12038 .5013 .11407 Metetra
.691 .743 .891 r
.5013 .11407 .50603 .12038 .51464 .11637 .50992 .1101 Metetra
.695 .747 .891 r
.50992 .1101 .51464 .11637 .52327 .11225 .51857 .10603 Metetra
.69 .747 .895 r
.51857 .10603 .52327 .11225 .53192 .10828 .52724 .10209 Metetra
.682 .746 .9 r
.52724 .10209 .53192 .10828 .5406 .10452 .53593 .09833 Metetra
.675 .745 .903 r
.53593 .09833 .5406 .10452 .5493 .10091 .54465 .09471 Metetra
.671 .744 .905 r
.54465 .09471 .5493 .10091 .55803 .0974 .5534 .09119 Metetra
.668 .743 .906 r
.5534 .09119 .55803 .0974 .56679 .09394 .56218 .08772 Metetra
.666 .742 .907 r
.56218 .08772 .56679 .09394 .57557 .09051 .57098 .08427 Metetra
.665 .742 .907 r
.57098 .08427 .57557 .09051 .58438 .08709 .57981 .08083 Metetra
.665 .742 .907 r
.57981 .08083 .58438 .08709 .59322 .08368 .58867 .07739 Metetra
.664 .742 .908 r
.58867 .07739 .59322 .08368 .60209 .08026 .59756 .07395 Metetra
.664 .742 .908 r
.59756 .07395 .60209 .08026 .61099 .07683 .60648 .07051 Metetra
.664 .741 .908 r
.60648 .07051 .61099 .07683 .61992 .0734 .61543 .06706 Metetra
.664 .741 .908 r
.61543 .06706 .61992 .0734 .62887 .06996 .6244 .0636 Metetra
.664 .741 .908 r
.6244 .0636 .62887 .06996 .63786 .06652 .6334 .06013 Metetra
.664 .741 .908 r
.6334 .06013 .63786 .06652 .64687 .06306 .64244 .05665 Metetra
.664 .741 .908 r
.64244 .05665 .64687 .06306 .65591 .05959 .6515 .05316 Metetra
.664 .741 .908 r
.6515 .05316 .65591 .05959 .66498 .05611 .66059 .04966 Metetra
.664 .741 .908 r
.66059 .04966 .66498 .05611 .67408 .05262 .66971 .04615 Metetra
.664 .741 .908 r
.66971 .04615 .67408 .05262 .68321 .04912 .67886 .04263 Metetra
.664 .741 .908 r
.67886 .04263 .68321 .04912 .69237 .04561 .68804 .0391 Metetra
.664 .741 .908 r
.68804 .0391 .69237 .04561 .70156 .04209 .69725 .03555 Metetra
.664 .741 .908 r
.69725 .03555 .70156 .04209 .71078 .03855 .70649 .032 Metetra
.653 .738 .912 r
.05315 .27622 .05871 .28141 .06595 .27881 .06041 .27361 Metetra
.655 .739 .911 r
.06041 .27361 .06595 .27881 .07323 .27616 .0677 .27094 Metetra
.657 .739 .91 r
.0677 .27094 .07323 .27616 .08053 .27347 .07501 .26823 Metetra
.658 .739 .91 r
.07501 .26823 .08053 .27347 .08786 .27074 .08235 .26549 Metetra
.659 .74 .909 r
.08235 .26549 .08786 .27074 .09521 .26799 .08971 .26272 Metetra
.659 .74 .909 r
.08971 .26272 .09521 .26799 .10258 .26523 .09709 .25995 Metetra
.659 .74 .909 r
.09709 .25995 .10258 .26523 .10997 .26246 .1045 .25716 Metetra
.659 .739 .909 r
.1045 .25716 .10997 .26246 .11739 .25969 .11193 .25438 Metetra
.658 .739 .91 r
.11193 .25438 .11739 .25969 .12482 .25693 .11937 .2516 Metetra
.657 .739 .91 r
.11937 .2516 .12482 .25693 .13228 .25418 .12684 .24882 Metetra
.656 .738 .91 r
.12684 .24882 .13228 .25418 .13975 .25144 .13433 .24606 Metetra
.655 .738 .911 r
.13433 .24606 .13975 .25144 .14725 .24872 .14184 .24332 Metetra
.653 .737 .911 r
.14184 .24332 .14725 .24872 .15476 .24602 .14936 .2406 Metetra
.652 .736 .911 r
.14936 .2406 .15476 .24602 .16229 .24335 .15691 .2379 Metetra
.652 .736 .911 r
.15691 .2379 .16229 .24335 .16985 .24067 .16448 .23518 Metetra
.655 .736 .909 r
.16448 .23518 .16985 .24067 .17744 .23791 .17208 .2324 Metetra
.663 .738 .905 r
.17208 .2324 .17744 .23791 .18507 .23499 .17972 .22945 Metetra
.671 .74 .902 r
.17972 .22945 .18507 .23499 .19274 .23187 .1874 .22633 Metetra
.676 .742 .9 r
.1874 .22633 .19274 .23187 .20044 .22864 .19512 .2231 Metetra
.677 .743 .9 r
.19512 .2231 .20044 .22864 .20817 .22538 .20286 .21985 Metetra
.676 .743 .901 r
.20286 .21985 .20817 .22538 .21591 .22215 .21062 .21661 Metetra
.674 .743 .903 r
.21062 .21661 .21591 .22215 .22368 .21895 .21839 .21341 Metetra
.672 .743 .904 r
.21839 .21341 .22368 .21895 .23146 .21579 .22619 .21024 Metetra
.67 .743 .905 r
.22619 .21024 .23146 .21579 .23925 .21266 .234 .20709 Metetra
.669 .742 .905 r
.234 .20709 .23925 .21266 .24707 .20954 .24183 .20396 Metetra
.668 .742 .906 r
.24183 .20396 .24707 .20954 .25491 .20645 .24968 .20085 Metetra
.667 .742 .906 r
.24968 .20085 .25491 .20645 .26278 .20336 .25756 .19775 Metetra
.666 .742 .907 r
.25756 .19775 .26278 .20336 .27066 .20029 .26546 .19466 Metetra
.665 .741 .907 r
.26546 .19466 .27066 .20029 .27856 .19723 .27338 .19158 Metetra
.665 .741 .907 r
.27338 .19158 .27856 .19723 .28649 .19416 .28132 .18849 Metetra
.665 .741 .907 r
.28132 .18849 .28649 .19416 .29444 .19107 .28929 .18538 Metetra
.667 .742 .906 r
.28929 .18538 .29444 .19107 .30242 .18793 .29728 .18223 Metetra
.668 .742 .906 r
.29728 .18223 .30242 .18793 .31043 .18476 .3053 .17904 Metetra
.669 .742 .905 r
.3053 .17904 .31043 .18476 .31846 .18155 .31334 .17582 Metetra
.669 .743 .905 r
.31334 .17582 .31846 .18155 .32651 .17834 .32141 .17259 Metetra
.668 .742 .906 r
.32141 .17259 .32651 .17834 .33459 .17515 .32951 .16937 Metetra
.659 .739 .909 r
.32951 .16937 .33459 .17515 .34268 .17213 .33761 .16634 Metetra
.655 .738 .911 r
.33761 .16634 .34268 .17213 .35079 .16919 .34573 .16338 Metetra
.652 .737 .912 r
.34573 .16338 .35079 .16919 .35892 .16629 .35388 .16046 Metetra
.649 .735 .913 r
.35388 .16046 .35892 .16629 .36707 .16346 .36205 .15759 Metetra
.646 .734 .913 r
.36205 .15759 .36707 .16346 .37525 .16068 .37025 .15478 Metetra
.644 .732 .913 r
.37025 .15478 .37525 .16068 .38346 .15792 .37847 .15198 Metetra
.647 .732 .911 r
.37847 .15198 .38346 .15792 .39169 .1551 .38672 .14912 Metetra
.653 .734 .909 r
.38672 .14912 .39169 .1551 .39996 .15213 .395 .14612 Metetra
.659 .736 .907 r
.395 .14612 .39996 .15213 .40826 .14903 .40331 .14301 Metetra
.662 .737 .905 r
.40331 .14301 .40826 .14903 .41658 .14584 .41165 .13981 Metetra
.663 .738 .905 r
.41165 .13981 .41658 .14584 .42494 .14262 .42002 .13657 Metetra
.664 .738 .905 r
.42002 .13657 .42494 .14262 .43332 .13937 .42842 .13331 Metetra
.665 .739 .905 r
.42842 .13331 .43332 .13937 .44172 .1361 .43684 .13003 Metetra
.664 .739 .905 r
.43684 .13003 .44172 .1361 .45016 .13283 .44529 .12675 Metetra
.66 .737 .907 r
.44529 .12675 .45016 .13283 .45861 .12965 .45377 .12355 Metetra
.652 .734 .91 r
.45377 .12355 .45861 .12965 .46709 .12665 .46226 .12051 Metetra
.646 .731 .911 r
.46226 .12051 .46709 .12665 .4756 .12377 .47078 .11757 Metetra
.648 .73 .909 r
.47078 .11757 .4756 .12377 .48413 .12085 .47934 .11458 Metetra
.659 .733 .904 r
.47934 .11458 .48413 .12085 .4927 .11767 .48792 .11135 Metetra
.676 .738 .897 r
.48792 .11135 .4927 .11767 .5013 .11407 .49653 .10774 Metetra
.689 .743 .892 r
.49653 .10774 .5013 .11407 .50992 .1101 .50517 .1038 Metetra
.693 .746 .892 r
.50517 .1038 .50992 .1101 .51857 .10603 .51384 .09977 Metetra
.688 .747 .896 r
.51384 .09977 .51857 .10603 .52724 .10209 .52253 .09586 Metetra
.681 .746 .9 r
.52253 .09586 .52724 .10209 .53593 .09833 .53124 .0921 Metetra
.675 .745 .903 r
.53124 .0921 .53593 .09833 .54465 .09471 .53998 .08848 Metetra
.671 .744 .905 r
.53998 .08848 .54465 .09471 .5534 .09119 .54875 .08494 Metetra
.668 .743 .906 r
.54875 .08494 .5534 .09119 .56218 .08772 .55754 .08145 Metetra
.666 .742 .907 r
.55754 .08145 .56218 .08772 .57098 .08427 .56637 .07798 Metetra
.665 .742 .907 r
.56637 .07798 .57098 .08427 .57981 .08083 .57522 .07452 Metetra
.665 .742 .907 r
.57522 .07452 .57981 .08083 .58867 .07739 .5841 .07107 Metetra
.664 .742 .908 r
.5841 .07107 .58867 .07739 .59756 .07395 .593 .06761 Metetra
.664 .742 .908 r
.593 .06761 .59756 .07395 .60648 .07051 .60194 .06415 Metetra
.664 .741 .908 r
.60194 .06415 .60648 .07051 .61543 .06706 .6109 .06067 Metetra
.664 .741 .908 r
.6109 .06067 .61543 .06706 .6244 .0636 .6199 .05719 Metetra
.664 .741 .908 r
.6199 .05719 .6244 .0636 .6334 .06013 .62892 .05371 Metetra
.664 .741 .908 r
.62892 .05371 .6334 .06013 .64244 .05665 .63797 .05021 Metetra
.664 .741 .908 r
.63797 .05021 .64244 .05665 .6515 .05316 .64705 .0467 Metetra
.664 .741 .908 r
.64705 .0467 .6515 .05316 .66059 .04966 .65617 .04318 Metetra
.664 .741 .908 r
.65617 .04318 .66059 .04966 .66971 .04615 .66531 .03964 Metetra
.664 .741 .908 r
.66531 .03964 .66971 .04615 .67886 .04263 .67448 .0361 Metetra
.664 .741 .908 r
.67448 .0361 .67886 .04263 .68804 .0391 .68368 .03255 Metetra
.664 .741 .908 r
.68368 .03255 .68804 .0391 .69725 .03555 .69291 .02898 Metetra
.664 .741 .908 r
.69291 .02898 .69725 .03555 .70649 .032 .70217 .02541 Metetra
.653 .738 .912 r
.04757 .271 .05315 .27622 .06041 .27361 .05484 .26837 Metetra
.655 .739 .911 r
.05484 .26837 .06041 .27361 .0677 .27094 .06213 .26569 Metetra
.657 .739 .91 r
.06213 .26569 .0677 .27094 .07501 .26823 .06946 .26297 Metetra
.658 .739 .91 r
.06946 .26297 .07501 .26823 .08235 .26549 .07681 .26021 Metetra
.659 .74 .909 r
.07681 .26021 .08235 .26549 .08971 .26272 .08418 .25743 Metetra
.659 .74 .909 r
.08418 .25743 .08971 .26272 .09709 .25995 .09158 .25464 Metetra
.659 .74 .909 r
.09158 .25464 .09709 .25995 .1045 .25716 .09899 .25184 Metetra
.659 .739 .909 r
.09899 .25184 .1045 .25716 .11193 .25438 .10643 .24904 Metetra
.658 .739 .91 r
.10643 .24904 .11193 .25438 .11937 .2516 .11389 .24624 Metetra
.658 .739 .91 r
.11389 .24624 .11937 .2516 .12684 .24882 .12137 .24345 Metetra
.657 .739 .91 r
.12137 .24345 .12684 .24882 .13433 .24606 .12887 .24067 Metetra
.655 .738 .911 r
.12887 .24067 .13433 .24606 .14184 .24332 .13639 .2379 Metetra
.654 .737 .911 r
.13639 .2379 .14184 .24332 .14936 .2406 .14393 .23516 Metetra
.653 .737 .911 r
.14393 .23516 .14936 .2406 .15691 .2379 .15149 .23242 Metetra
.653 .736 .911 r
.15149 .23242 .15691 .2379 .16448 .23518 .15908 .22968 Metetra
.656 .736 .909 r
.15908 .22968 .16448 .23518 .17208 .2324 .16669 .22686 Metetra
.663 .738 .906 r
.16669 .22686 .17208 .2324 .17972 .22945 .17435 .2239 Metetra
.671 .74 .902 r
.17435 .2239 .17972 .22945 .1874 .22633 .18204 .22077 Metetra
.675 .742 .901 r
.18204 .22077 .1874 .22633 .19512 .2231 .18977 .21754 Metetra
.676 .742 .901 r
.18977 .21754 .19512 .2231 .20286 .21985 .19752 .21429 Metetra
.675 .743 .902 r
.19752 .21429 .20286 .21985 .21062 .21661 .20529 .21105 Metetra
.673 .743 .903 r
.20529 .21105 .21062 .21661 .21839 .21341 .21308 .20783 Metetra
.671 .743 .904 r
.21308 .20783 .21839 .21341 .22619 .21024 .22089 .20465 Metetra
.67 .742 .905 r
.22089 .20465 .22619 .21024 .234 .20709 .22871 .20149 Metetra
.669 .742 .905 r
.22871 .20149 .234 .20709 .24183 .20396 .23656 .19835 Metetra
.667 .742 .906 r
.23656 .19835 .24183 .20396 .24968 .20085 .24443 .19522 Metetra
.666 .742 .906 r
.24443 .19522 .24968 .20085 .25756 .19775 .25232 .19211 Metetra
.666 .742 .907 r
.25232 .19211 .25756 .19775 .26546 .19466 .26023 .189 Metetra
.665 .741 .907 r
.26023 .189 .26546 .19466 .27338 .19158 .26816 .1859 Metetra
.665 .741 .907 r
.26816 .1859 .27338 .19158 .28132 .18849 .27612 .18279 Metetra
.665 .741 .907 r
.27612 .18279 .28132 .18849 .28929 .18538 .2841 .17966 Metetra
.667 .742 .906 r
.2841 .17966 .28929 .18538 .29728 .18223 .29211 .1765 Metetra
.668 .742 .906 r
.29211 .1765 .29728 .18223 .3053 .17904 .30014 .17329 Metetra
.669 .742 .905 r
.30014 .17329 .3053 .17904 .31334 .17582 .3082 .17006 Metetra
.669 .743 .905 r
.3082 .17006 .31334 .17582 .32141 .17259 .31629 .16681 Metetra
.668 .742 .906 r
.31629 .16681 .32141 .17259 .32951 .16937 .32439 .16357 Metetra
.658 .739 .909 r
.32439 .16357 .32951 .16937 .33761 .16634 .33251 .16052 Metetra
.655 .738 .911 r
.33251 .16052 .33761 .16634 .34573 .16338 .34065 .15754 Metetra
.652 .737 .912 r
.34065 .15754 .34573 .16338 .35388 .16046 .34881 .1546 Metetra
.65 .736 .912 r
.34881 .1546 .35388 .16046 .36205 .15759 .357 .1517 Metetra
.647 .734 .913 r
.357 .1517 .36205 .15759 .37025 .15478 .36521 .14885 Metetra
.646 .733 .913 r
.36521 .14885 .37025 .15478 .37847 .15198 .37345 .146 Metetra
.648 .733 .911 r
.37345 .146 .37847 .15198 .38672 .14912 .38171 .14311 Metetra
.653 .734 .909 r
.38171 .14311 .38672 .14912 .395 .14612 .39001 .14009 Metetra
.659 .736 .907 r
.39001 .14009 .395 .14612 .40331 .14301 .39834 .13696 Metetra
.662 .737 .906 r
.39834 .13696 .40331 .14301 .41165 .13981 .4067 .13375 Metetra
.663 .738 .905 r
.4067 .13375 .41165 .13981 .42002 .13657 .41508 .1305 Metetra
.664 .738 .905 r
.41508 .1305 .42002 .13657 .42842 .13331 .4235 .12722 Metetra
.665 .739 .905 r
.4235 .12722 .42842 .13331 .43684 .13003 .43194 .12393 Metetra
.664 .739 .906 r
.43194 .12393 .43684 .13003 .44529 .12675 .4404 .12063 Metetra
.66 .737 .907 r
.4404 .12063 .44529 .12675 .45377 .12355 .44889 .11742 Metetra
.653 .735 .91 r
.44889 .11742 .45377 .12355 .46226 .12051 .4574 .11433 Metetra
.648 .732 .91 r
.4574 .11433 .46226 .12051 .47078 .11757 .46594 .11133 Metetra
.65 .731 .909 r
.46594 .11133 .47078 .11757 .47934 .11458 .47451 .10828 Metetra
.66 .734 .904 r
.47451 .10828 .47934 .11458 .48792 .11135 .48311 .10501 Metetra
.675 .738 .898 r
.48311 .10501 .48792 .11135 .49653 .10774 .49174 .10139 Metetra
.687 .743 .894 r
.49174 .10139 .49653 .10774 .50517 .1038 .5004 .09748 Metetra
.691 .746 .894 r
.5004 .09748 .50517 .1038 .51384 .09977 .50909 .09349 Metetra
.686 .747 .897 r
.50909 .09349 .51384 .09977 .52253 .09586 .51779 .08959 Metetra
.68 .746 .901 r
.51779 .08959 .52253 .09586 .53124 .0921 .52653 .08584 Metetra
.674 .744 .904 r
.52653 .08584 .53124 .0921 .53998 .08848 .53528 .08221 Metetra
.67 .743 .905 r
.53528 .08221 .53998 .08848 .54875 .08494 .54407 .07866 Metetra
.668 .743 .906 r
.54407 .07866 .54875 .08494 .55754 .08145 .55288 .07515 Metetra
.666 .742 .907 r
.55288 .07515 .55754 .08145 .56637 .07798 .56172 .07166 Metetra
.665 .742 .907 r
.56172 .07166 .56637 .07798 .57522 .07452 .57059 .06818 Metetra
.665 .742 .907 r
.57059 .06818 .57522 .07452 .5841 .07107 .57949 .06471 Metetra
.664 .742 .908 r
.57949 .06471 .5841 .07107 .593 .06761 .58842 .06123 Metetra
.664 .742 .908 r
.58842 .06123 .593 .06761 .60194 .06415 .59737 .05775 Metetra
.664 .741 .908 r
.59737 .05775 .60194 .06415 .6109 .06067 .60636 .05425 Metetra
.664 .741 .908 r
.60636 .05425 .6109 .06067 .6199 .05719 .61537 .05075 Metetra
.664 .741 .908 r
.61537 .05075 .6199 .05719 .62892 .05371 .62441 .04724 Metetra
.664 .741 .908 r
.62441 .04724 .62892 .05371 .63797 .05021 .63349 .04372 Metetra
.664 .741 .908 r
.63349 .04372 .63797 .05021 .64705 .0467 .64259 .04019 Metetra
.664 .741 .908 r
.64259 .04019 .64705 .0467 .65617 .04318 .65172 .03665 Metetra
.664 .741 .908 r
.65172 .03665 .65617 .04318 .66531 .03964 .66088 .0331 Metetra
.664 .741 .908 r
.66088 .0331 .66531 .03964 .67448 .0361 .67007 .02953 Metetra
.664 .741 .908 r
.67007 .02953 .67448 .0361 .68368 .03255 .67929 .02596 Metetra
.664 .741 .908 r
.67929 .02596 .68368 .03255 .69291 .02898 .68854 .02237 Metetra
.664 .741 .908 r
.68854 .02237 .69291 .02898 .70217 .02541 .69782 .01877 Metetra
.653 .738 .912 r
.04196 .26577 .04757 .271 .05484 .26837 .04924 .26312 Metetra
.655 .739 .911 r
.04924 .26312 .05484 .26837 .06213 .26569 .05654 .26042 Metetra
.657 .739 .91 r
.05654 .26042 .06213 .26569 .06946 .26297 .06388 .25768 Metetra
.658 .739 .91 r
.06388 .25768 .06946 .26297 .07681 .26021 .07124 .2549 Metetra
.659 .74 .909 r
.07124 .2549 .07681 .26021 .08418 .25743 .07862 .25211 Metetra
.659 .74 .909 r
.07862 .25211 .08418 .25743 .09158 .25464 .08603 .2493 Metetra
.659 .74 .909 r
.08603 .2493 .09158 .25464 .09899 .25184 .09346 .24648 Metetra
.659 .739 .909 r
.09346 .24648 .09899 .25184 .10643 .24904 .10091 .24366 Metetra
.659 .739 .909 r
.10091 .24366 .10643 .24904 .11389 .24624 .10838 .24085 Metetra
.658 .739 .91 r
.10838 .24085 .11389 .24624 .12137 .24345 .11588 .23804 Metetra
.657 .739 .91 r
.11588 .23804 .12137 .24345 .12887 .24067 .12339 .23524 Metetra
.656 .738 .91 r
.12339 .23524 .12887 .24067 .13639 .2379 .13092 .23245 Metetra
.654 .737 .911 r
.13092 .23245 .13639 .2379 .14393 .23516 .13848 .22968 Metetra
.653 .737 .911 r
.13848 .22968 .14393 .23516 .15149 .23242 .14605 .22692 Metetra
.654 .736 .91 r
.14605 .22692 .15149 .23242 .15908 .22968 .15365 .22414 Metetra
.657 .737 .909 r
.15365 .22414 .15908 .22968 .16669 .22686 .16128 .2213 Metetra
.663 .738 .906 r
.16128 .2213 .16669 .22686 .17435 .2239 .16895 .21831 Metetra
.67 .74 .903 r
.16895 .21831 .17435 .2239 .18204 .22077 .17665 .21518 Metetra
.674 .741 .901 r
.17665 .21518 .18204 .22077 .18977 .21754 .18439 .21195 Metetra
.675 .742 .901 r
.18439 .21195 .18977 .21754 .19752 .21429 .19215 .2087 Metetra
.674 .743 .902 r
.19215 .2087 .19752 .21429 .20529 .21105 .19994 .20545 Metetra
.673 .743 .903 r
.19994 .20545 .20529 .21105 .21308 .20783 .20774 .20223 Metetra
.671 .743 .904 r
.20774 .20223 .21308 .20783 .22089 .20465 .21556 .19904 Metetra
.67 .742 .905 r
.21556 .19904 .22089 .20465 .22871 .20149 .2234 .19586 Metetra
.668 .742 .905 r
.2234 .19586 .22871 .20149 .23656 .19835 .23126 .19271 Metetra
.667 .742 .906 r
.23126 .19271 .23656 .19835 .24443 .19522 .23914 .18957 Metetra
.666 .742 .906 r
.23914 .18957 .24443 .19522 .25232 .19211 .24704 .18643 Metetra
.666 .742 .907 r
.24704 .18643 .25232 .19211 .26023 .189 .25497 .18331 Metetra
.665 .741 .907 r
.25497 .18331 .26023 .189 .26816 .1859 .26292 .18019 Metetra
.665 .741 .907 r
.26292 .18019 .26816 .1859 .27612 .18279 .27089 .17706 Metetra
.665 .741 .907 r
.27089 .17706 .27612 .18279 .2841 .17966 .27888 .17391 Metetra
.667 .742 .906 r
.27888 .17391 .2841 .17966 .29211 .1765 .28691 .17073 Metetra
.668 .742 .906 r
.28691 .17073 .29211 .1765 .30014 .17329 .29496 .16751 Metetra
.669 .742 .906 r
.29496 .16751 .30014 .17329 .3082 .17006 .30303 .16426 Metetra
.669 .743 .905 r
.30303 .16426 .3082 .17006 .31629 .16681 .31113 .161 Metetra
.669 .743 .906 r
.31113 .161 .31629 .16681 .32439 .16357 .31925 .15773 Metetra
.658 .739 .91 r
.31925 .15773 .32439 .16357 .33251 .16052 .32739 .15468 Metetra
.655 .738 .911 r
.32739 .15468 .33251 .16052 .34065 .15754 .33554 .15167 Metetra
.653 .737 .911 r
.33554 .15167 .34065 .15754 .34881 .1546 .34372 .1487 Metetra
.65 .736 .912 r
.34372 .1487 .34881 .1546 .357 .1517 .35192 .14577 Metetra
.648 .735 .913 r
.35192 .14577 .357 .1517 .36521 .14885 .36015 .14288 Metetra
.647 .734 .912 r
.36015 .14288 .36521 .14885 .37345 .146 .3684 .14 Metetra
.649 .734 .911 r
.3684 .14 .37345 .146 .38171 .14311 .37668 .13707 Metetra
.654 .735 .909 r
.37668 .13707 .38171 .14311 .39001 .14009 .38499 .13403 Metetra
.658 .736 .907 r
.38499 .13403 .39001 .14009 .39834 .13696 .39334 .13088 Metetra
.661 .737 .906 r
.39334 .13088 .39834 .13696 .4067 .13375 .40171 .12766 Metetra
.663 .738 .906 r
.40171 .12766 .4067 .13375 .41508 .1305 .41011 .12439 Metetra
.664 .738 .905 r
.41011 .12439 .41508 .1305 .4235 .12722 .41854 .1211 Metetra
.664 .739 .905 r
.41854 .1211 .4235 .12722 .43194 .12393 .427 .11779 Metetra
.664 .739 .906 r
.427 .11779 .43194 .12393 .4404 .12063 .43548 .11449 Metetra
.66 .738 .907 r
.43548 .11449 .4404 .12063 .44889 .11742 .44399 .11125 Metetra
.654 .735 .909 r
.44399 .11125 .44889 .11742 .4574 .11433 .45252 .10813 Metetra
.65 .733 .91 r
.45252 .10813 .4574 .11433 .46594 .11133 .46107 .10507 Metetra
.652 .732 .908 r
.46107 .10507 .46594 .11133 .47451 .10828 .46966 .10196 Metetra
.661 .734 .904 r
.46966 .10196 .47451 .10828 .48311 .10501 .47828 .09865 Metetra
.675 .739 .899 r
.47828 .09865 .48311 .10501 .49174 .10139 .48692 .09503 Metetra
.686 .743 .895 r
.48692 .09503 .49174 .10139 .5004 .09748 .4956 .09114 Metetra
.689 .746 .895 r
.4956 .09114 .5004 .09748 .50909 .09349 .5043 .08718 Metetra
.685 .746 .898 r
.5043 .08718 .50909 .09349 .51779 .08959 .51303 .08329 Metetra
.679 .745 .901 r
.51303 .08329 .51779 .08959 .52653 .08584 .52178 .07954 Metetra
.674 .744 .904 r
.52178 .07954 .52653 .08584 .53528 .08221 .53056 .0759 Metetra
.67 .743 .905 r
.53056 .0759 .53528 .08221 .54407 .07866 .53936 .07234 Metetra
.668 .743 .906 r
.53936 .07234 .54407 .07866 .55288 .07515 .54819 .06881 Metetra
.666 .742 .907 r
.54819 .06881 .55288 .07515 .56172 .07166 .55705 .0653 Metetra
.665 .742 .907 r
.55705 .0653 .56172 .07166 .57059 .06818 .56594 .06181 Metetra
.665 .742 .907 r
.56594 .06181 .57059 .06818 .57949 .06471 .57486 .05831 Metetra
.664 .742 .908 r
.57486 .05831 .57949 .06471 .58842 .06123 .58381 .05481 Metetra
.664 .742 .908 r
.58381 .05481 .58842 .06123 .59737 .05775 .59278 .05131 Metetra
.664 .741 .908 r
.59278 .05131 .59737 .05775 .60636 .05425 .60178 .04779 Metetra
.664 .741 .908 r
.60178 .04779 .60636 .05425 .61537 .05075 .61082 .04427 Metetra
.664 .741 .908 r
.61082 .04427 .61537 .05075 .62441 .04724 .61988 .04074 Metetra
.664 .741 .908 r
.61988 .04074 .62441 .04724 .63349 .04372 .62897 .0372 Metetra
.664 .741 .908 r
.62897 .0372 .63349 .04372 .64259 .04019 .63809 .03365 Metetra
.664 .741 .908 r
.63809 .03365 .64259 .04019 .65172 .03665 .64724 .03009 Metetra
.664 .741 .908 r
.64724 .03009 .65172 .03665 .66088 .0331 .65642 .02651 Metetra
.664 .741 .908 r
.65642 .02651 .66088 .0331 .67007 .02953 .66563 .02293 Metetra
.664 .741 .908 r
.66563 .02293 .67007 .02953 .67929 .02596 .67487 .01933 Metetra
.664 .741 .908 r
.67487 .01933 .67929 .02596 .68854 .02237 .68415 .01572 Metetra
.664 .741 .908 r
.68415 .01572 .68854 .02237 .69782 .01877 .69345 .0121 Metetra
.653 .739 .912 r
.03631 .2605 .04196 .26577 .04924 .26312 .0436 .25783 Metetra
.655 .739 .911 r
.0436 .25783 .04924 .26312 .05654 .26042 .05092 .25512 Metetra
.657 .739 .91 r
.05092 .25512 .05654 .26042 .06388 .25768 .05827 .25236 Metetra
.659 .739 .91 r
.05827 .25236 .06388 .25768 .07124 .2549 .06564 .24957 Metetra
.659 .74 .909 r
.06564 .24957 .07124 .2549 .07862 .25211 .07304 .24676 Metetra
.659 .74 .909 r
.07304 .24676 .07862 .25211 .08603 .2493 .08046 .24393 Metetra
.659 .74 .909 r
.08046 .24393 .08603 .2493 .09346 .24648 .0879 .2411 Metetra
.659 .74 .909 r
.0879 .2411 .09346 .24648 .10091 .24366 .09536 .23826 Metetra
.659 .739 .909 r
.09536 .23826 .10091 .24366 .10838 .24085 .10285 .23543 Metetra
.658 .739 .91 r
.10285 .23543 .10838 .24085 .11588 .23804 .11035 .2326 Metetra
.657 .739 .91 r
.11035 .2326 .11588 .23804 .12339 .23524 .11788 .22978 Metetra
.656 .738 .91 r
.11788 .22978 .12339 .23524 .13092 .23245 .12542 .22697 Metetra
.655 .738 .911 r
.12542 .22697 .13092 .23245 .13848 .22968 .13299 .22418 Metetra
.654 .737 .911 r
.13299 .22418 .13848 .22968 .14605 .22692 .14058 .22139 Metetra
.654 .737 .91 r
.14058 .22139 .14605 .22692 .15365 .22414 .14819 .21858 Metetra
.657 .737 .908 r
.14819 .21858 .15365 .22414 .16128 .2213 .15583 .21571 Metetra
.663 .738 .906 r
.15583 .21571 .16128 .2213 .16895 .21831 .16351 .21271 Metetra
.67 .74 .903 r
.16351 .21271 .16895 .21831 .17665 .21518 .17123 .20957 Metetra
.674 .741 .902 r
.17123 .20957 .17665 .21518 .18439 .21195 .17898 .20634 Metetra
.675 .742 .902 r
.17898 .20634 .18439 .21195 .19215 .2087 .18676 .20308 Metetra
.674 .743 .902 r
.18676 .20308 .19215 .2087 .19994 .20545 .19455 .19982 Metetra
.672 .743 .903 r
.19455 .19982 .19994 .20545 .20774 .20223 .20237 .1966 Metetra
.671 .743 .904 r
.20237 .1966 .20774 .20223 .21556 .19904 .2102 .19339 Metetra
.67 .742 .905 r
.2102 .19339 .21556 .19904 .2234 .19586 .21805 .19021 Metetra
.668 .742 .906 r
.21805 .19021 .2234 .19586 .23126 .19271 .22593 .18704 Metetra
.667 .742 .906 r
.22593 .18704 .23126 .19271 .23914 .18957 .23382 .18388 Metetra
.666 .742 .906 r
.23382 .18388 .23914 .18957 .24704 .18643 .24174 .18073 Metetra
.666 .742 .907 r
.24174 .18073 .24704 .18643 .25497 .18331 .24968 .17759 Metetra
.665 .741 .907 r
.24968 .17759 .25497 .18331 .26292 .18019 .25764 .17445 Metetra
.665 .741 .907 r
.25764 .17445 .26292 .18019 .27089 .17706 .26563 .1713 Metetra
.665 .741 .907 r
.26563 .1713 .27089 .17706 .27888 .17391 .27364 .16813 Metetra
.667 .742 .906 r
.27364 .16813 .27888 .17391 .28691 .17073 .28168 .16493 Metetra
.668 .742 .906 r
.28168 .16493 .28691 .17073 .29496 .16751 .28974 .1617 Metetra
.669 .742 .906 r
.28974 .1617 .29496 .16751 .30303 .16426 .29783 .15843 Metetra
.669 .743 .905 r
.29783 .15843 .30303 .16426 .31113 .161 .30594 .15515 Metetra
.669 .743 .906 r
.30594 .15515 .31113 .161 .31925 .15773 .31408 .15186 Metetra
.658 .739 .91 r
.31408 .15186 .31925 .15773 .32739 .15468 .32223 .1488 Metetra
.655 .738 .911 r
.32223 .1488 .32739 .15468 .33554 .15167 .3304 .14577 Metetra
.653 .737 .911 r
.3304 .14577 .33554 .15167 .34372 .1487 .33859 .14277 Metetra
.651 .736 .912 r
.33859 .14277 .34372 .1487 .35192 .14577 .34681 .13981 Metetra
.649 .735 .912 r
.34681 .13981 .35192 .14577 .36015 .14288 .35505 .13689 Metetra
.648 .734 .912 r
.35505 .13689 .36015 .14288 .3684 .14 .36332 .13397 Metetra
.65 .734 .911 r
.36332 .13397 .3684 .14 .37668 .13707 .37162 .131 Metetra
.654 .735 .909 r
.37162 .131 .37668 .13707 .38499 .13403 .37995 .12794 Metetra
.658 .736 .907 r
.37995 .12794 .38499 .13403 .39334 .13088 .38831 .12477 Metetra
.661 .737 .906 r
.38831 .12477 .39334 .13088 .40171 .12766 .3967 .12154 Metetra
.663 .738 .906 r
.3967 .12154 .40171 .12766 .41011 .12439 .40512 .11826 Metetra
.664 .738 .906 r
.40512 .11826 .41011 .12439 .41854 .1211 .41356 .11495 Metetra
.664 .739 .906 r
.41356 .11495 .41854 .1211 .427 .11779 .42203 .11163 Metetra
.663 .739 .906 r
.42203 .11163 .427 .11779 .43548 .11449 .43053 .10831 Metetra
.66 .738 .907 r
.43053 .10831 .43548 .11449 .44399 .11125 .43905 .10505 Metetra
.655 .736 .909 r
.43905 .10505 .44399 .11125 .45252 .10813 .4476 .10189 Metetra
.651 .734 .91 r
.4476 .10189 .45252 .10813 .46107 .10507 .45618 .09878 Metetra
.654 .733 .908 r
.45618 .09878 .46107 .10507 .46966 .10196 .46478 .09562 Metetra
.662 .735 .904 r
.46478 .09562 .46966 .10196 .47828 .09865 .47341 .09227 Metetra
.674 .739 .899 r
.47341 .09227 .47828 .09865 .48692 .09503 .48208 .08863 Metetra
.684 .743 .896 r
.48208 .08863 .48692 .09503 .4956 .09114 .49077 .08477 Metetra
.687 .745 .896 r
.49077 .08477 .4956 .09114 .5043 .08718 .49949 .08083 Metetra
.684 .746 .898 r
.49949 .08083 .5043 .08718 .51303 .08329 .50824 .07696 Metetra
.678 .745 .902 r
.50824 .07696 .51303 .08329 .52178 .07954 .51701 .07321 Metetra
.673 .744 .904 r
.51701 .07321 .52178 .07954 .53056 .0759 .5258 .06956 Metetra
.67 .743 .905 r
.5258 .06956 .53056 .0759 .53936 .07234 .53463 .06598 Metetra
.668 .743 .906 r
.53463 .06598 .53936 .07234 .54819 .06881 .54348 .06243 Metetra
.666 .742 .907 r
.54348 .06243 .54819 .06881 .55705 .0653 .55236 .05891 Metetra
.665 .742 .907 r
.55236 .05891 .55705 .0653 .56594 .06181 .56126 .05539 Metetra
.665 .742 .907 r
.56126 .05539 .56594 .06181 .57486 .05831 .5702 .05188 Metetra
.664 .742 .908 r
.5702 .05188 .57486 .05831 .58381 .05481 .57916 .04836 Metetra
.664 .742 .908 r
.57916 .04836 .58381 .05481 .59278 .05131 .58816 .04483 Metetra
.664 .741 .908 r
.58816 .04483 .59278 .05131 .60178 .04779 .59718 .0413 Metetra
.664 .741 .908 r
.59718 .0413 .60178 .04779 .61082 .04427 .60623 .03775 Metetra
.664 .741 .908 r
.60623 .03775 .61082 .04427 .61988 .04074 .61532 .0342 Metetra
.664 .741 .908 r
.61532 .0342 .61988 .04074 .62897 .0372 .62443 .03064 Metetra
.664 .741 .908 r
.62443 .03064 .62897 .0372 .63809 .03365 .63357 .02707 Metetra
.664 .741 .908 r
.63357 .02707 .63809 .03365 .64724 .03009 .64274 .02348 Metetra
.664 .741 .908 r
.64274 .02348 .64724 .03009 .65642 .02651 .65194 .01989 Metetra
.664 .741 .908 r
.65194 .01989 .65642 .02651 .66563 .02293 .66117 .01628 Metetra
.664 .741 .908 r
.66117 .01628 .66563 .02293 .67487 .01933 .67043 .01266 Metetra
.664 .741 .908 r
.67043 .01266 .67487 .01933 .68415 .01572 .67973 .00903 Metetra
.664 .741 .908 r
.67973 .00903 .68415 .01572 .69345 .0121 .68905 .00539 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.03716 .25514 m
.04196 .25962 L
s
[(-200)] .02757 .24618 1 .93395 Mshowa
.18558 .19702 m
.19014 .20174 L
s
[(-150)] .17646 .18758 .96648 1 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(-100)] .3345 .12542 .86223 1 Mshowa
.51046 .06981 m
.51442 .07504 L
s
[(-50)] .50253 .05935 .75799 1 Mshowa
.68874 0 m
.69233 .00549 L
s
[(0)] .68156 -0.01098 .65374 1 Mshowa
.125 Mabswid
.06616 .24378 m
.06901 .2465 L
s
.09549 .2323 m
.09832 .23504 L
s
.12517 .22067 m
.12797 .22345 L
s
.1552 .20892 m
.15797 .21172 L
s
.21633 .18498 m
.21903 .18784 L
s
.24744 .1728 m
.25012 .17569 L
s
.27893 .16047 m
.28157 .16339 L
s
.3108 .14799 m
.31341 .15094 L
s
.37572 .12257 m
.37826 .12558 L
s
.40878 .10962 m
.41128 .11266 L
s
.44225 .09652 m
.44471 .09959 L
s
.47614 .08324 m
.47856 .08635 L
s
.54521 .0562 m
.54755 .05937 L
s
.58041 .04242 m
.5827 .04562 L
s
.61605 .02846 m
.6183 .03169 L
s
.65216 .01432 m
.65436 .01759 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{293.812, 238.125},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004U0000kQ000`40O003h00Oogoo9Woo003oOolV
Ool00?moobIoo`00ogoo9Woo003oOolVOol00?moobIoo`00ogoo9Woo0035Ool2001NOol00<Aoo`04
001oogoo001MOol00<Aoo`04001oogoo001MOol00<Aoo`04001oogoo001MOol00<Aoo`04001oogoo
000<Ool30004Ool20018Ool00<Eoo`8000ioo`03001oogoo009oo`04001oogoo0017Ool00<ioo`D0
009oo`03001oogoo009oo`04001oogoo0017Ool00=Eoo`03001oogoo009oo`04001oogoo0017Ool0
0=Eoo`03001oogoo009oo`04001oogoo0017Ool00=Aoo`8000Eoo`8004Qoo`00b7oo0`00FWoo0036
Ool6001IOol00<Aoo`800004Ool00000E_d2001IOol00<1oo`L00003E_d005Km00<005Qoo`00_Woo
1P0000AFo@00E_eFo@D005Qoo`00_7oo0P0000Moo`00000005Km001Fo@0300000eKm00000003001G
Ool00;Uoo`<00005Ool00000E_d00002E_d20002E_d01000E_d000000UKm0P00EWoo002gOol20000
1goo00000000E_d005Km00<00004E_d005KmE_d200001EKm00000000E_d00P00EGoo002?Ool30004
Ool2000LOol60002E_d01000E_d000000UKm1@0000=Fo@0000000P000UKm0P00EGoo002BOol01000
Oomoo`000Woo00<007ooOol05goo0P0000Ioo`00000005Km0002E_d200001eKm00000000E_d005Km
00<00003E_d005Km00<00003E_d0000005Aoo`00RGoo1@0017oo00@007ooOol0009oo`03001oogoo
01Aoo`<00003Ool00000009Fo@03001Fo@0000800005E_d00000E_d00002E_d2000015Km001FoEKm
10000UKm0P00E7oo002?Ool30003Ool01000Oomoo`0057oo0P000Woo0P0000=Fo@0000000`0000MF
o@00000005Km001Fo@0300000eKm001Fo@0300000eKm0000000300000eKm0000001COol008moo`03
001oogoo00=oo`04001oogoo000AOol300001Goo00000000E_d00P0000AFo@00E_eFo@80009Fo@04
001Fo@000002E_d01000E_d000000UKm00@005Km0000009Fo@03001Fo@000080059oo`00Sgoo1000
0goo0P003goo0`000Woo0P0000MFo@00000005Km001Fo@03000015Km001FoEKm0P0000AFo@00E_eF
o@800004E_d005KmE_d2000015Km001FoEKm0P0000=Fo@000000DWoo002UOol700000eKm00000002
E_d01000E_d000000UKm00<005Km00000P0000=Fo@00E_d00`0000=Fo@00E_d00`0000=Fo@000000
0`0000AFo@00001Fo@80055oo`00XWoo1@0000AFo@00E_eFo@800004E_d005KmE_d2000015Km001F
oEKm0P0000AFo@00E_eFo@800004E_d005KmE_d200001EKm00000000E_d01@0000=Fo@00E_d00`00
D7oo002POol500000eKm001Fo@0300000eKm001Fo@0300000eKm001Fo@0300000eKm001Fo@030000
0eKm0000000300001EKm000005Km00000UKm0P0000AFo@00E_eFo@800003E_d00000051oo`00WGoo
1@000UKl00@005Km0000009Fo@04001Fo@000002E_d01000E_d000000UKm00D005Km000005Km00<0
0003E_d005Km00<00003E_d005Km00<00003E_d005Km00<00003E_d005Km00<004moo`00VWoo0`00
00Eoo`00001Fo000009Fo0800004E_d005KmE_d2000015Km001FoEKm0P0000=Fo@0000000`0000=F
o@0000000UKm00@005Km0000009Fo@04001Fo@000002E_d01000E_d000000UKm00@005Km0000009F
o@8004ioo`00V7oo1P0000=Fo000E_`00`0000=Fo000E_d00`0000=Fo@00E_d00`0000EFo@00001F
o@00009Fo@800004E_d005KmE_d2000015Km001FoEKm0P0000AFo@00E_eFo@@0009Fo@800003E_d0
000000<004ioo`00UWoo0P0000Aoo`00001Fo0800005E_`00000E_`00002E_`200000eKm00000003
00001eKm00000000E_d005Km00<00003E_d005Km00<00003E_d005Km00<00003E_d005Km00<00003
E_d0000000<00005E_d00000E_d00002E_d3001<Ool009=oo`<00005Ool00000E_`00002E_`20000
0eKl001Fo00300000eKl00000002E_d01000E_d000000UKm00@005Km0000009Fo@06001Fo@00001F
o@000UKm0P0000AFo@00E_eFo@800003E_d0000000<00007E_d00000001Fo@00E_d00`0000=Fo@00
Ool0C7oo002@Ool300001goo00000000E_`005Kl00<00003E_`005Kl0080009Fo0800004E_`005Km
E_d2000015Km001FoEKm0P0000AFo@00E_eFo@800003E_d005Km00<00003E_d0000000<00005E_d0
0000E_d00002E_d2000015Km001FoEKm0P0000AFo@00E_eFo@8004aoo`00SWoo0P000Woo0P000UKl
00H005Kl000005Kl0002E_`200001eKl00000000E_`005Kl00<00003E_d005Km00<00003E_d005Km
00<00003E_d005Km0080009Fo@800004E_d005KmE_d200000eKm001Fo@0500001eKm00000000E_d0
05Km00<00003E_d0000004]oo`00Rgoo0`000Woo0P0000AFo000E_aFo0800003F_`005Kl00<00003
E_`00000009Fo004001Fo0000002E_`01P00E_d00000E_d0009Fo@800003E_d0000000<00007E_d0
0000001Fo@00E_d00`0000=Fo@00E_d00P000UKm10000UKm00D005Km000005Km00D004]oo`00R7oo
0`000Woo0`0000=Fo000E_`00`0000=Fo000F_`00P000UKl0P0000AFo000E_aFo0800004E_`005Kl
E_`200000eKm0000000300000eKm00000002E_d01P00E_d00000E_d0009Fo@800004E_d005KmE_d2
000015Km001FoEKm10000UKm0P0000=Fo@0000000UKm00@005Km000004Yoo`00G7oo0`0017oo0P00
17oo0P006goo100000=oo`0000000U;m00H005Kl000005Kl0002E_`200001e[l00000000F_`005Kl
00<00003E_`005Kl00<00005E_`00000E_d00002E_d2000015Km001FoEKm0P0000=Fo@00E_d00`00
00=Fo@00E_d00`0000=Fo@00E_d00`0000=Fo@0000000`0000EFo@00001Fo@00009Fo@800003E_d0
000004Uoo`00GGoo00<007ooOol00Woo00@007ooOol0009oo`04001oogoo000HOol20003Ool20000
15;m001BoE;m0P0000=Fo000E_`00`0000=Fo00000000U[l00@005[l0000009Fo005001Fo000001F
o00300000eKl001Fo@0300000eKm001Fo@0300000eKm001Fo@020002E_d01@00E_d00000E_d01@00
00MFo@00000005Km001Fo@0300000eKm001Fo@0300000eKm001Fo@030018Ool005Ioo`D0009oo`03
001oogoo009oo`04001oogoo0002Ool01000Oomoo`005Goo0`000Woo0`0000=Bo@00D_d00`0000=B
o@00E_`00P000UKl0P0000AJo000F_aJo0800004F_`005KlE_`200000eKl00000002E_`01P00E_`0
0000E_d0009Fo@800004E_d005KmE_d200000eKm0000000300000eKm00000002E_d01P00E_d00000
E_d0009Fo@800004E_d005KmE_d2000015Km001FoEKm0P0000=Fo@0000003Goo0`00>7oo001MOol0
0`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol001=oo`800003Ool000000080009Fo@@0009B
o@@00007E_`00000001Fo000F_`00`0000=Jo000F_`00`0000EFo000001Fo000009Fo0800003E_`0
05Km00<00003E_d0000000<00005E_d00000E_d00002E_d2000015Km001FoEKm0P0000=Fo@00E_d0
1@0000MFo@00000005Km001Fo@0300000eKm00000003000?Ool00`00Oomoo`0eOol005eoo`03001o
ogoo009oo`04001oogoo0002Ool01000Oomoo`0047oo0`0000Ioo`00001Fo000E_`300000eKm001B
o@0300000e;m001Bo@020002E_`01000E_`000000U[l00D005[l000005[l00<00003E_`005Kl00<0
0003E_`005Kl0080009Fo@800004E_d005KmE_d200000eKm001Fo@0300000eKm001Fo@0300000eKm
001Fo@020002E_d400000eKm0000000300001eKm00000000E_d005Km008000Ioo`D000Aoo`03001o
ogoo03Eoo`00G7oo0P001Goo0P0017oo0P003Woo0`0000Moo`00000005Kl001Fo0020002E_`01000
E_d000000U;m00H005;m000005;m0002E_`2000015Kl001Jo5[l0P0000=Jo00000000U[l00@005Kl
0000009Fo0@00007E_`00000001Fo@00E_d00`0000=Fo@00E_d00P000UKm00H005Km000005Km0002
E_d200000eKm0000000300000eKm00000002E_d01P00E_d00000E_d0009Fo@800003E_d0000000]o
o`<003Qoo`00Mgoo0P000Woo0P000UKl00@005Kl0000009Fo006001Fo000001Fo@000U;m0P0000=B
o@00D_d00`0000=Fo000E_`00`0000EJo000001Jo000009Jo0800007E_`00000001Fo000E_`00P00
0UKl00H005Km000005Km0002E_d2000015Km001FoEKm0P0000=Fo@0000000`0000EFo@00001Fo@00
009Fo@800004E_d005KmE_d200000eKm001Fo@0300000eKm001Fo@03000:Ool00`00Oomoo`0hOol0
07Aoo`@00005Ool00000E_`00002E_`2000015Kl001Fo5Kl0P0000=Fo00000000`0000=Bo@000000
0P000U;m00@005Kl0000009Fo0800003F_`005[l00<0009Jo003001Fo000009Fo0800004E_`005Kl
E_`200000eKm001Fo@0300000eKm0000000300001EKm000005Km00000UKm0P0000=Fo@00E_d01@00
00MFo@00000005Km001Fo@02000015Km001FoEKm0P0000AFo@00000000Uoo`@003Moo`00LWoo0P00
0goo0P0000=Fo@00E_`00`0000=Fo00000000`0000EFo000001Fo000009Fo@800003D_d005;m00D0
0007D_`00000001Fo000F_`00P000U[l00D005[l000005Kl00<00003E_`005Kl00<00003E_`005Km
0080009Fo@800003E_d005Km00<00003E_d005Km00<00003E_d005Km00800003E_d005Km00<00004
E_d005KmE_d200000eKm00000003000015Km000005Km0P00A7oo001_Ool800000e;m000000030000
1EKl000005Kl00000UKl0P0000=Fo000E_`00`0000=Fo@00D_d00P000U;m00@005;m0000009Bo006
001Fo000001Jo0000U[l0P0000=Jo00000000U[l00D005Kl000005Kl00D00007E_d00000001Fo@00
E_d00P000UKm00H005Km000005Km0002E_d200000eKm001Fo@0300000eKm0000000300001EKm0000
05Km00000UKm0P0000=Fo@00E_d00`00@goo001]Ool200000goo00000002D_d01P00D_d00000D_d0
009Fo@800003E_`005Kl00<00003E_`005Kl0080009Fo006001Fo@00001Bo@000U;m0P0000ABo@00
D_aBo0800003E_`005[l00<00005F_`00000F_`00002F_`200000eKl00000002E_`01P00E_`00000
E_d0009Fo@800004E_d005KmE_d200000eKm001Fo@0300000eKm000000020002E_d200000eKm001F
o@0300000eKm001Fo@0300000eKm001Fo@0200000eKm00000013Ool006Yoo`<00005Ool00000D_d0
0002D_d200000e;m001Bo@0300000eKm001Fo0020002E_`01P00E_`00000E_`0009Fo0800003E_d0
000000<00003D_d005;m00<00003D_`005Kl0080009Jo0800003F_`005[l00<00005F_`00000E_`0
0002E_`200000eKl001Fo@0300000eKm001Fo@0300000eKm0000000200001eKm00000000E_d005Km
00D00005E_d00000E_d00002E_d2000015Km001FoEKm0P0000=Fo@0000000`00@Woo001WOol30002
Ool200000e;m001Bo@0300000e;m001Bo@020002D_d01P00E_d00000E_`0009Fo0800003E_`00000
00<00005E_`00000E_d00002D_d40002D_d01@00D_`00000E_`00`0000=Jo000F_`00P000U[l0P00
00=Jo000E_`00`0000=Fo000E_`00P000UKm100000MFo@00000005Km001Fo@020002E_d01`00E_d0
0000E_d005Km00<00003E_d005Km00<00003E_d0000000<00006E_d00000E_d005Km0`00@Goo001U
Ool700000e;m0000000300000e;m0000000200000e;m0000000300000eKm0000000300001EKl0000
05Kl00000UKl0P0000=Fo000E_d00`0000=Bo@00D_d01@0000=Bo00000000UKl00@005[l0000009J
o0800003Fa`005[l0080009Fo0@00007E_`00000001Fo@00E_d00P0000AFo@00E_eFo@800004E_d0
05KmE_d200000eKm001Fo@0300000eKm000000020002E_d200000eKm001Fo@0300000eKm001Fo@02
000015Km00000000@7oo001ROol50002E_d01P00D_d00000D_d0009Bo@800003D_d005;m0080009B
o@04001Bo@000002E_`01`00E_`00000E_`005Kl00<00003E_`005Kl0080009Fo@06001Bo@00001B
o@000U;m0P0000ABo000E_aFo0800003F_`005[l0080009K70800007F_`00000001Fo000E_`00P00
00AFo000E_eFo@800003E_d005Km00<00003E_d005Km00<00003E_d0000000800007E_d00000001F
o@00E_d00`0000=Fo@00E_d00P0000AFo@00E_eFo@800003E_d005Km00<0041oo`00H7oo0P0000Eo
o`00001Fo000009Fo@800003D_d005;m00<00003D_d005;m0080009Bo@06001Bo@00001Bo@000UKl
0P0000=Fo000E_`00P000UKl00@005Kl0000009Fo007001Fo@00001Bo@00D_d00`0000=Bo@00D_`0
0`0000=Fo000F_`00P0000ANn`000000009K7003001Jo000009Fo0800003E_`005Kl00<00003E_d0
05Km0080009Fo@@00007E_d00000001Fo@00E_d00P0000AFo@00001Fo@800004E_d005KmE_d20000
0eKm001Fo@0300000eKm00000002000015Km00000000?goo001MOol300001Woo000005Kl001Fo0<0
0003E_d0000000<00003D_d0000000800004D_d005;mD_d200000e;m001Bo@0300000eKl001Fo002
0002E_`01@00E_`00000E_`01@0000=Fo@0000000P000U;m100000ABo000E_aFo080009Jn`05001N
nekk001K700200001e[l00000000E_`005Kl0080009Fo0@00007E_d00000001Fo@00E_d00P0000AF
o@00E_eFo@800004E_d005KmE_d200000eKm001Fo@0300000eKm0000000200001eKm00000000E_d0
05Km0080009Fo@03001oogoo03eoo`00<Goo0`000goo0`0017oo0P006goo0`0000Aoo`00001Fo0D0
0005E_`00000E_d00002D_d200000e;m001Bo@0500000e;m000000020002D_d400000eKl00000003
00000eKl00000002E_`01`00E_`00000E_d005;m00<00003C_d004kM00D00003E_`005[k0080009N
n`800003Fa`00000009Jo005001Fo000001Fo00300000eKl001Fo@02000015Km001FoEKm0P0000=F
o@00E_d00`0000=Fo@00E_d0200000MFo@00000005Km001Fo@02000015Km001FoEKm0P0000AFo@00
E_eFo@8003ioo`00<Woo00<007ooOol01Goo00@007ooOol0009oo`03001oogoo01Eoo`<00005Ool0
0000001Fo0020002E_`01`00E_`00000E_`005Km00<00003D_d005;m0080009Bo@07001Bo@00001B
o@00D_d00`0000=Bo@00E_`00P000UKl00H005Kl000005Kl0002E_`200000eKl001Fo@0300000e;m
001>o@02000014kM001Bg5;L100000AJn`000000009Nn`03001O7000009Jo0800005E_`00000E_`0
0002E_`200000eKm001Fo@0300000eKm000000020002E_d400001eKm00000000E_d005Km00800004
E_d005KmE_d200000eKm001Fo@0300000eKm001Fo@0300000eKm0000000mOol002]oo`D0009oo`03
001oogoo00Eoo`04001oogoo0002Ool00`00Oomoo`0COol700001EKl000005Kl00000UKl0P0000=F
o000E_`00`0000=Fo@00D_d00P0000ABo@00D_eBo@800003D_d005;m0080009Bo@06001Bo@00001F
o0000UKl0P0000=Fo000E_`00`0000=Fo000E_`00P000UKm100000=>o@00C]d00`0000=Bg@00E_`0
0P000Ukk00D005kk000005lL00<00006Fa`00000E_`005Kl0`0000=Fo00000000P000UKm0P0000=F
o@00E_d00`0000=Fo@00E_d00P0000AFo@00001Fo@800003E_d005Km00<00003E_d0000000800003
E_d0000000<00006E_d00000001Fo@00?Goo000bOol00`00Oomoo`02Ool30003Ool01000Oomoo`00
4Woo1@000UKm00L005Km000005Kl001Fo00300000eKl001Fo0020002E_`400000e;m000000030000
0e;m000000020002D_d01`00D_d00000D_d005Kl00<00003E_`005Kl0080009Fo0@00007E_`00000
001Fo000D_d010000TkM00H005;M000005Kl0002G_/2000015kk000005lL0`0000=K7000E_`00P00
0UKl0P0000=Fo000E_d00`0000=Fo@00E_d00P0000AFo@00E_eFo@800003E_d005Km00<00003E_d0
000000800007E_d00000001Fo@00E_d00P0000AFo@00E_eFo@800004E_d005KmE_d2000lOol0039o
o`03001oogoo009oo`03001oogoo00=oo`04001oogoo000?Ool300001Goo000005Km00000UKm0P00
00=Fo@00E_`00`0000=Fo00000000P0000MFo000000005Kl001Fo@020002D_d01`00D_d00000D_d0
05;m00D00003D_d0000000<00003E_`0000000800007E_`00000001Fo000E_`00P000UKl00L005Kl
000005;m001>g@0500000dkM001Fo0030002G_/01000Ga]O6`000UlK00D005/L000005Kl00<00003
E_`005Kl00800004E_d005KmE_d200000eKm001Fo@0300000eKm0000000200001eKm00000000E_d0
05Km00800004E_d005KmE_d200000eKm001Fo@0300000eKm001Fo@0300000eKm0000000kOol0035o
o`8000Aoo`@000=oo`8000ioo`80009oo`800003E_d005Km00<00003E_d000000080009Fo0800003
E_`005Kl0080009Fo006001Fo000001Fo@000U;m0P0000=Bo@00D_d00`0000=Bo@00D_d00P0000AB
o@00D_eBo@800003E_`005Kl0080009Fo006001Fo000001Fo0000UKl0P0000=Fo00000000P000TkM
00<004kM00000`000UKl00D005kk000005lK0080009O6`800005Fa`00000E_`00002E_`200000eKl
001Fo@0300000eKm0000000200001eKm00000000E_d005Km00<00003E_d005Km00800003E_d005Km
00<00003E_d005Km0080009Fo@@00006E_d00000001Fo@00>goo001;Ool30002Ool200000eKl0000
000300001UKm000005Km001Fo@<00003E_`005Kl0080009Fo005001Fo000001Fo00500000e;m0000
00020002D_d400000e;m001Bo@0300000e;m001Fo002000015Kl001Fo5Kl0P0000=Fo000E_`00`00
00IFo000001Fo000D_d5000014[M001>g@000UKl0P0000=Nn`0000000V<K0P000UlK00@005/L001F
o0<00003E_`000000080009Fo@800003E_d005Km0080009Fo@05001Fo@00001Fo@02000015Km0000
05Km10000UKm100000MFo@00000005Km001Fo@02000015Km001FoEKm0P00>Woo0019Ool700000eKl
00000002E_`01`00E_d00000E_d005Km00<00003E_d005Kl00800003E_`0000000<00003E_`00000
009Fo007001Fo@00001Bo@00D_d00`0000=Bo@00D_d010000U;m100000=Fo00000000`0000=Fo000
00000P000UKl0P0000=Fo000E_`00P000U;m00<004kM00001P0000YFo000001Nn`00Ha/006<K001O
6`800003Fa`00000009Fo0800003E_`005Kl00<00003E_d005Km00800004E_d005KmE_d200000eKm
0000000300001UKm000005Km001Fo@<00003E_d005Km00800004E_d005KmE_d200000eKm001Fo@03
00000eKm0000000iOol004Ioo`D0009Fo006001Fo000001Fo0000UKl0P0000=Fo@00E_d00`0000=F
o@0000000P0000=Fo00000000UKl00H005Kl000005Kl0002E_`200000eKm001Bo@0300000e;m001B
o@0200000e;m001Bo@0300000e;m001Bo@020002E_`01`00E_`00000E_`005Kl00<00003E_`005Kl
00800004E_`005;mD_d2000014[M001:g@000TkM0P0000=Fg000G_/00P000V<K0P0000AO6`00001K
70<00003E_`005Kl00800004E_`005KmE_d200000eKm0000000300001UKm000005Km001Fo@<00003
E_d005Km00800004E_d005KmE_d200000eKm001Fo@0300000eKm0000000200000eKm00000003000i
Ool004Aoo`800003Ool00000009Fo007001Fo000001Fo000E_`00`0000=Fo00000000P000UKm0P00
00=Fo@00E_d00P000UKl00L005Kl000005Kl001Fo00300000eKl000000020002D_d400000e;m001B
o@0300000e;m001Bo@02000015;m001Fo5Kl0P0000=Fo000E_`00P000UKl00D005Kl000005Kl00D0
0003C_d0000000D00003C]d005KL0080009NnP80009S6P04001O6elK0002Fa`01@00E_`00000E_`0
1@0000IFo000001Fo@00E_d300000eKm001Fo@040002E_d200000eKm001Fo@0300000eKm00000002
00001eKm00000000E_d005Km00800004E_d005KmE_d2000hOol0045oo`L00003E_`005Kl00800003
E_`0000000<00006E_`00000E_`005Km0`0000=Fo@00E_d00P000UKm00D005Kl000005Kl00D00006
E_`00000E_`005Km0`0000=Bo@00D_d010000U;m100000=Bo@00D_d00`0000=Fo000E_`00P0000=F
o00000000`0000EFo000001Fo000009Fo0800005C_d004[M001:g@0500001eKL001NnP00H_X006<J
00800003Ga/00000009K70800006F_`00000E_`005Kl0`0000=Fo000E_d00P0000AFo@00E_eFo@80
0003E_d0000000<00003E_d0000000800007E_d00000001Fo@00E_d00P0000AFo@00E_eFo@800003
E_d005Km00<00003E_d0000003Moo`00?goo0P000Woo0`0000=Fo00000000P0000=Fo00000000UKl
00L005Kl000005Kl001Fo00300000eKm001Fo@0200000eKm0000000300000eKl00000002E_`01`00
E_`00000E_`005Kl00<00003E_d005;m00800003D_d005;m00<00003D_d005;m00@0009Bo0@00003
E_`00000009Fo007001Fo000001Fo000E_`00`0000=Fo000C_d010000TJm00@004Zm001Fg080009R
nP80009RnP03001O6`0000800006Fa`00000F_`005Kl0P000UKl00D005Kl000005Km00D00006E_d0
0000E_d005Km0`0000=Fo@00E_d00P0000AFo@00E_eFo@800003E_d005Km00<00003E_d000000080
0003E_d0000000<003Moo`00?7oo0`000Woo0P000UKl0P0000=Fo000E_`00P000UKl00L005Kl0000
05Kl001Fo00300000eKl0000000200000eKm00000002E_d01P00E_d00000E_`0009Fo0800003E_`0
05Kl0080009Fo0@00003D_d0000000<00003D_d005;m00800003D_d005;m00<00003D_`005Kl0080
0004E_`005KlE_`200000eKl001Fo0020002E_`400000dkm001:g@0700002UKL001RnP00H_X006;j
001VnP000V<K0P0000=K7000F_`00P0000=Fo00000000`0000EFo000001Fo@00009Fo@800003E_d0
05Km00800004E_d005KmE_d200000eKm0000000300000eKm0000000200001eKm00000000E_d005Km
00800003E_d005Km00<003Ioo`00>Goo200000MFo000000005Kl001Fo0020002E_`01@00E_`00000
E_`01@0000IFo000001Fo000E_d20002E_d01`00E_d00000E_d005Kl00<00003E_`0000000800007
E_`00000001Fo000E_d00P0000ABo@00D_eBo@@00003D_d0000000<00003D_d005;l00800003E_`0
05Kl00<00003E_`0000000800007E_`00000001Fo000E_`010000TKM00@004Jm001:_@@0009RnP80
0008I_X00000Ha/006<K001K70800005F_`00000E_`00002E_`200000eKl001Fo@0300000eKm001F
o@0800001UKm000005Km001Fo@<00003E_d005Km00800004E_d005KmE_d200000eKm001Fo@020000
15Km00000000=Goo000gOol20002Ool200000eKl001Fo0020002E_`01@00E_`00000E_`01@0000=F
o00000000UKl00L005Kl000005Kl001Fo00300000eKm001Fo@0200000eKm0000000300001UKl0000
05Kl001Fo080009Fo007001Fo000001Fo@00D_d00`0000=Bo@00D_d00P0000ABo@00C_e>o@@00003
D_`000000080009Fo0800003E_`005Kl0080009Fo007001Fo000001Fo000C_d01`0000=6_@00E]/0
0`000V;j00@006KiI_T0009S6`800006Fa`00000F_`005Kl0`0000=Fo000E_`00P0000=Fo@000000
0`0000EFo@00001Fo@00009Fo@800003E_d005Km00800004E_d005KmE_d200000eKm001Fo@030000
0eKm00000002000015Km000005Km0`00=7oo000dOol30002Ool200000eKm001Fo00300000eKl001F
o00200000eKl00000002E_`01P00E_`00000E_`0009Fo0800003E_`005Kl0080009Fo0@00003E_d0
0000009Fo@07001Fo@00001Fo000E_`00P000UKl00D005Kl000005Kl00D00003D_d0000000800003
D_d0000000<00003C_d005;m00800003D_`005Kl00<00003E_`005Kl00800004E_`005KlE_`20000
0eKl000000020002C]d00`00A[d000050002E]/01@00H_X00000I_T00P0000=S6`00Ha/00P0000=O
7000F_`00P000UKl100000EFo000001Fo@00009Fo@800003E_d0000000<00003E_d005Km00800003
E_d0000000<00003E_d0000000800007E_d00000001Fo@00E_d00P0000=Fo@00E_d00`0027oo0P00
:Woo0009Ool40003Ool20004Ool2000JOol700000e;m0000000300000eKl0000000200000eKl0000
0002E_`01`00E_`00000E_`005Kl00<00003E_`0000000800007E_`00000001Fo000E_d00P0000AF
o@00E_eFo@800003E_d005Kl00800003E_`0000000<00005E_`00000E_`00002E_d200000e;m001B
o@02000015;m001BoE;m100000=Bo@00D_`00P000UKl00L005Kl000005Kl001Fo00300001UKl0000
05Kl001Fo0D00006@[d004:M0012W@000UKK0P000V;i00T006KiI_T006<K001S6P00Ga`00P0000MJ
o000000005Kl001Fo00200000eKl001Fo@0300000eKm0000000200000eKm0000000300001UKm0000
05Km001Fo@<00003E_d005Km00800004E_d005KmE_d200000eKm001Fo@0200000eKm00000007Ool0
1000Oomoo`00:Goo000:Ool00`00Oomoo`02Ool01000Oomoo`000Woo00@007ooOol001Ioo`<00006
Ool00000D_d005;m0P000U;m00L005Km000005Kl001Fo0020002E_`01@00E_`00000E_`01@0000IF
o000001Fo000E_`300000eKl001Fo00200000eKm001Fo@0300000eKm0000000200000eKl00000002
E_`01`00E_`00000E_`005Kl00<00003E_d005;m00800003D_d005;m00<00003C_d004km00@0009B
o005001Fo000001Fo0040002E_`200000eKl001Fo0020002E_`;00000e:k001Rn@02000016Ki001Z
n@000f<J00H005lL000005/L0002E_`200000eKl0000000200001eKm00000000E_d005Km00800004
E_d005KmE_d200000eKm001Fo@02000015Km001FoEKm0P0000=Fo@0000000`0000=Fo@0000000P00
00AFo@00001Fo@8000Ioo`04001oogoo000YOol000=oo`D000=oo`05001oogooOol00002Ool01000
Oomoo`000Woo00<007ooOol04Woo0P000Woo0P0000=Bo@00D_d00`0000=Bo@00D_d00P0000=Bo@00
E_`00`0000=Fo000E_`00P0000=Fo00000000UKl00L005Kl000005Kl001Fo00300000eKl00000002
00000eKl0000000300001UKm000005Km001Fo@80009Fo007001Fo000001Fo000E_`00`0000=Fo000
00000P0000=Bo@0000000`0000=Bo@0000000P0000=>g@00C]d01@0000IFo000001Fo000E_`30000
0eKl001Fo002000035[l00000000F_`004[M0012_@00?Yd0008nW@04001B^`000002I_T00`00J_UZ
n@0200001f<J001W6P00Ga`005/L00<00006E_`00000E_`005Kl0P000UKm00<005Km00000`0000EF
o@00000005Km00@00003E_d0000000<00006E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Fo@00
E_d00`001Goo00@007ooOol002Uoo`0037oo00@007ooOol0009oo`04001oogoo0002Ool00`00Oomo
o`0?Ool700000eKm0000000300000e;m0000000200000e;m0000000300000eKl0000000200000eKl
00000002E_`01@00E_`00000E_`01@0000IFo000001Fo000E_`20002E_`01`00E_d00000E_d005Km
0080009Fo@05001Fo000001Fo0040002E_`200000eKl001Fo@02000015;m001BoE;m0P0000=Bo@00
C_d010000TkM00L005;l000005Kl001Fo00300000eKl001Fo00200000eKl00000002F_`00`00F_`0
000:00000e:k001Vn@0200000f[i001Zn002000016LJ000005lL0P0000MK7000000005Kl001Fo002
000015Kl001FoEKm0P0000=Fo@0000000P0000MFo@00000005Km001Fo@0200000eKm001Fo@030000
0eKm001Fo@0200000eKm001Fo@0300000eKm0000000200000eKm00000005Ool01000Oomoo`00:Goo
0009Ool01000Oomoo`000Woo00@007ooOol0009oo`04001oogoo000?Ool200001Woo000005Km001F
o@80009Fo@07001Bo@00001Bo@00D_d00P000U;m00L005;m000005Kl001Fo00300000eKl001Fo002
00000eKl00000002E_`01`00E_`00000E_`005Kl0080009Fo005001Fo000001Fo@0400000eKm0000
000300001UKl000005Kl001Fo0<00003E_`005Kl00800003E_d005;m00<00003D_d005;m00800003
C_d004kM00D00003E_`005Kl0080009Fo0@00005E_`00000E_`00002F_`200000e[l000000080000
0e:k00000002I]P00`00J_P00002J_P01@00IaYW6P00Ga`00P000U/L00D005Kl000005Kl00D00006
E_d00000E_d005Km0P0000AFo@00E_eFo@800003E_d005Km00800003E_d0000000<00003E_d00000
00800007E_d00000001Fo@00E_d00P0000AFo@00000000Aoo`8002Yoo`002Woo0P0017oo0P0017oo
0P003Goo0`0000Ioo`00001Fo@00E_d300000eKm001Fo@0200000e;m001Bo@0300000e;m001Bo@02
00000e;m001Fo0020002E_`400001EKl000005Kl00000UKl0P0000=Fo000E_`00P0000=Fo0000000
0`0000IFo000001Fo@00E_d300000eKm001Fo@0200000eKl001Fo00300000eKl0000000200000eKl
0000000300000e;m0000000200000e;m0000000300000dkM001>g@0400001eKl00000000E_`005Kl
00800003E_`005Kl00<00009F_`00000Fa`004KN000nW@00008fO@09000bG@00D[/006KH001Vf000
00=Zn007001W6P00IaX005lL00000U/L0P0000IFo000001Fo000E_`300000eKm001Fo@0200000eKm
0000000300000eKm0000000200001EKm000005Km00000UKm0P0000=Fo@00E_d00P0000AFo@00E_eF
o@800003E_d005Km00<0031oo`008Woo100000Aoo`00001Fo@D00003E_d0000000800003E_d00000
00<00003D_d0000000800003D_d0000000800007E_`00000001Fo000E_`00P0000=Fo000E_`00`00
00=Fo00000000P0000=Fo00000000UKl00L005Kl000005Kl001Fo@0300000eKm0000000200000eKm
0000000300001UKl000005Kl001Fo080009Fo007001Fo@00001Bo@00D_d00P0000ABo@00C_e>o@@0
0003C]d005Kl0080009Fo005001Fo000001Fo0040002E_`200000e[l001K700:00000e:k00000002
I]P200001F[h001^m`00IaX00P0000EO7000000005/L00800003E_`005Kl00800003E_`0000000H0
0003E_d005Km00<00003E_d005Km00800003E_d005Km00<00003E_d005Km00800003E_d0000000<0
0003E_d0000000800003E_d00000031oo`0087oo0P0000=oo`0000000P0000=Fo@0000000UKm00L0
05Km000005Km001Fo@0300000e;m001Bo@0200000e;m001Bo@0200000e;m001Bo@020002E_`01`00
E_`00000E_`005Kl00<00006E_`00000E_`005Kl0`0000=Fo000E_`00P0000=Fo00000000`0000IF
o@00001Fo@00E_d2000015Km001Fo5Kl0P0000=Fo000E_`00P0000=Fo00000000`0000=Fo@000000
0P0000=Bo@0000000`0000=>o@00B]d0100000AFo000E_aFo0800006E_`00000E_`005Kl0`0000=F
o000F_`00P0000=O700000001P0000HZ7000CY/006KH0002J]L00`00K_M^m`020002IaT00`00Ga/0
000200000e/L00000002000015Kl000005Kl1@0000=Fo@00E_d00P0000AFo@00E_eFo@800003E_d0
000000800003E_d0000000<00006E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Fo@000000;goo
000MOol300001Woo000005Km001Fo@<00003E_d005Km00800003E_d005Km00<00003E_d000000080
0003D_d0000000800003D_d0000000<00003D_d005Kl00800003E_`0000000<00006E_`00000E_`0
05Kl0`0000=Fo00000000P0000=Fo00000000UKl00L005Kl000005Km001Fo@0200000eKm001Fo@03
00000eKl0000000200000eKl00000002E_`01`00E_`00000E_d005;m0080009Bo@03001Bo@0000<0
0003B]d004kM00D00006E_`00000E_`005Kl0P000UKl00D005Kl000005[l0080009O6`<00003=Wd0
02Xm00800003CY/0000000=Ze`09001^m`00K_L006LI001[6@00009O6`800003Fa`005Kl00P00003
E_d0000000800003E_d0000000<00006E_d00000E_d005Km0P0000AFo@00E_eFo@800003E_d005Km
00800003E_d0000000<00003E_d0000000<002ioo`0077oo1@0000=Fo@0000000`0000=Fo@000000
0P0000=Fo@0000000`0000IFo@00001Fo@00D_d200000e;m001Bo@020002D_d00`00D_d000030000
0eKl00000002E_`01`00E_`00000E_`005Kl00<00006E_`00000E_`005Kl0P0000AFo000E_aFo080
0003E_`0000000800003E_d0000000<00006E_d00000E_`005Kl0P000UKl00L005Kl000005Kl001F
o@02000015;m001BoE;m0P0000=Bo@00C]d0100000A>g@00E_aFo0800003F_`005Kl00800003E_`0
000000<00006E_`00000F_`005lK0P000SJN1P0000]>V`00J]MZe`00J]L006kg001^mP00009[6@06
001O6`00Ga/005/L1`000UKl0P0000=Fo@0000000P0000=Fo@00E_d00`0000=Fo@00E_d00P0000=F
o@0000000`0000=Fo@0000000P0000EFo@00001Fo@00009Fo@800003E_d005Km008002ioo`0077oo
0`0000=Bo@0000000UKm00L005Km000005Km001Fo@0300000eKm001Fo@0200000eKm001Fo@020000
0e;m001Bo@0300000e;m001Bo@0200000e;m001Bo@0300000eKl001Fo00200000eKl000000030000
1UKl000005Kl001Fo0800003E_`0000000<00006E_`00000E_`005Km0P000UKm00L005Km000005Km
001Fo002000015Kl001Fo5Kl0P0000=Fo00000000P0000=Fo@0000000`0000=Bo@0000000P0000=>
g@00B]d01@0000=Fo000F_`00P0000=Fo00000000UKl00L005Kl000005Kl001Jo00200000elK0000
00070002CY/20002J]L02@00K_L006kf001[6@00JaT00002Ga/500000eKl001Fo00300000eKl001F
o@04000015Km000005Km1@0000IFo@00001Fo@00E_d300000eKm001Fo@0200000eKm001Fo@030000
0eKm001Fo@0200000eKm0000000]Ool001Eoo`8000Eoo`03001Bo@0000800003D_d005Km00800003
E_d005Km00<00003E_d0000000800003E_d0000000800003E_d0000000<00003D_d0000000800003
D_d0000000<00003D_d0000000800005E_`00000E_`00002E_`200000eKl0000000200000eKl0000
0002E_`01`00E_`00000E_`005Kl00<00003E_d005Km00800003E_d0000000800003E_`0000000<0
0006E_`00000E_`005Kl0`0000=Fo@00D_d00P0000=Bo@00D_d010000TKM00@004kM001Fo0800005
F_`00000F_`00002E_`200000eKl001Fo00200000e[l00000002Ha/300000biM000Mo0040002J]L0
2000J[H006kf001^e@00JaT400001UlK000005/L001Fo0800003E_`0000000<00003E_d000000080
0005E_d00000E_d00002E_d200000eKm001Fo@02000015Km001FoEKm0P0000=Fo@0000000P0000=F
o@0000000`0000AFo@00001Fo@8002aoo`0057oo00@007ooOol000Aoo`<00003D_d0000000800003
E_d0000000<00006E_d00000E_d005Km0P0000=Fo@00E_d00`0000=Fo@00D_d00P0000=Bo@00D_d0
0`0000=Bo@00D_d00P0000=Bo@00E_`00P0000=Fo000E_`00`0000IFo000001Fo000E_`300000eKl
001Fo00200000eKl000000020002E_`400001UKm000005Km001Fo@<00003E_`005Kl00800003E_`0
05Kl0080009Fo0@00003D_d0000000800003C_d004kM00L00006E_`00000F_`005[l0`0000=Fo000
00000P0000MFo000001Jo000Ha/00002IaX60002BW/02000J]L006Zf001^mP00K]D300001f/I001O
6`00Ga/005/L00800005E_`00000E_`00002E_`200000eKm0000000200000eKm001Fo@0300000eKm
0000000200000eKm0000000300001UKm000005Km001Fo@800003E_d005Km00<00003E_d005Km0080
02aoo`0057oo00@007ooOol000=oo`08001oogooOol005;m001Bo@<00003E_d005Km00800003E_d0
05Km00<00003E_d005Km00800003E_d0000000800003D_d0000000<00003D_d0000000800003D_d0
05;m00800003E_`005Kl00<00006E_`00000E_`005Kl0`0000=Fo00000000P0000IFo000001Fo000
E_`300000eKl001Fo@0200000eKm001Fo@0300000eKm0000000200000eKl0000000200001eKl0000
0000E_`005Km00800003D_d005;m00D00003B]d004KM00@00003E_`005[l0080009Jo0800003E_`0
05Kl00800003E_`005[l00<00004IaX000005O`200000aGL0000000300000fZf001Z]@0300001G;E
001[6@00KaP00P0000AO6`00001K70800003E_`005Kl00<00003E_`0000000L00007E_d00000001F
o@00E_d00P0000=Fo@00E_d00`0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000`0000=F
o@000000:goo000DOol01000Oomoo`000goo0P000goo100000=Bo@0000000P0000=Fo@0000000`00
00=Fo@0000000P0000IFo@00001Fo@00E_d20002D_d01`00D_d00000D_d005;m00@00003D_d00000
00<00006E_`00000F_`005Kl0`0000IFo000001Fo000E_`200000eKl001Fo00300000eKl00000002
00000eKm000000020002E_d200000eKm001Fo00200000eKl001Fo00300000eKl001Fo00200000eKm
00000002000015;m001>oDkm100000E2_@00B[d005Kl00800007F_/00000001Jo000E_`00P0000=F
o00000000P000U[l0P000VLI0P0000@Z?@0000000096FPD00007K]D00000L]@006lH00000VlH00D0
05lK000005/L00@00007E_`00000001Fo000E_d00P0000=Fo@00E_d00P0000AFo@00E_eFo@800003
E_d0000000800003E_d0000000<00006E_d00000E_d005Km0P0000AFo@00E_eFo@800003E_d00000
02Yoo`0057oo00@007ooOol000=oo`03001oogoo00=oo`07001Bo@00001Bo@00E_d00`0000=Fo@00
E_d00P0000=Fo@00E_d00P0000=Fo@00E_d00`0000=Fo@00D_d00P0000=Bo@00D_d00P0000=Bo@00
D_d00`0000=Bo@00E_`00P0000=Fo000F_`00`0000IJo000001Fo000E_`200000eKl000000030000
1UKl000005Kl001Fo0800003E_d005Km00<00003E_d005Km00800003E_`005Kl0080009Fo0@00006
E_`00000E_d005;m1@0000=>o@0000001P0000=Jn`0000000U[k00D005[l000005Kl00800003E_`0
05Kl00<00005F_`006LI001[6@0900000fZe001^]@0300000g;D00000002KaP00`00KaP00002Ga/0
1000Fa`005Kl0P0000AFo000E_aFo0800003E_d0000000800003E_d0000000<00006E_d00000E_d0
05Km0P0000=Fo@00E_d00`0000=Fo@00E_d00P0000=Fo@0000002@00:Goo000EOol20004Ool00`00
Oomoo`04Ool00`00D_d0000400000eKm0000000200000eKm0000000200000eKm0000000300000eKm
0000000200000e;m0000000200000e;m0000000300000e;m0000000200000eKl0000000300001U[l
000005[l001Fo0800003E_`00000009Fo007001Fo000001Fo000E_`00P0000=Fo00000000`0000=F
o@0000000P0000=Fo@0000000P0000MFo000000005Kl001Fo00200000eKl001Fo@020002D_d01`00
D_d00000C_d004[M00H0009Jn`05001Jn`00001Jo00200000eKl001Fo0020002E_`01@00F_`00000
JaT00P0000<Mf`0000000P000Shj00P006Ze001^]@00L]D007;D0P0000I_6000KaQ_6000Ga/20000
15/L000005Kl1@0000AFo000001Fo@@00003E_d005Km00<00003E_d005Km00800003E_d000000080
0003E_d0000000<00006E_d00000E_d005Km0P0000AFo@00E_eFo@8002Uoo`006goo00<007ooOol0
1Goo00D005;mD_d005;m00800003E_d005Km00@00007E_d00000001Fo@00E_d00P0000=Fo@00D_d0
0P0000=Bo@00D_d00`0000=Bo@00D_d00P0000=Bo@00D_d00P0000AFo000E_aFo0800003F_`00000
00800007E_`00000001Fo000E_`00P0000=Fo00000000P0000=Fo00000000UKl00L005Km000005Km
001Fo@0200000e;m001Fo00300000eKl0000000200000eKl0000000200000eKm0000000300000e;m
0000000200001T[M000n_@00A[d0009Jn`800006G_/00000F_`005Kl0P0000=Fo00000000`0000AJ
o000001W6@<000049Sd000000002?SX200001Vje000007;D001bd`<00005KaP006lG001O6`020000
1U/L000005Kl001Fo0<00003E_`005Km00L00003E_d0000000<00006E_d00000E_d005Km0P0000AF
o@00E_eFo@800003E_d005Km00800003E_d0000000H002Qoo`006goo0P001goo00L005;m000005;m
001Fo@0300000eKm0000000200000eKm0000000200000eKm0000000300000e;m0000000200000e;m
0000000200000e;m0000000200000e;m001Fo00300001UKl000005[l001Jo0<00003E_`000000080
0006E_`00000E_`005Kl0`0000=Fo000E_`00P0000=Fo@0000000P0000=Fo@0000000`0000IFo000
001Fo000E_`200000eKl001Fo00300000eKm001Bo@0200000e;m001>o@090002F_/01000G_/005[l
0P0000=Fo00000000UKl00<005;l00001P0000<Yn`0000001@0000E^]@00KY@007;D00800004L[<0
07:b0002KaL20002Ha/01000Fa`005Kl0P0000=Fo00000000`0000IFo@00001Fo@00E_d2000015Km
001FoEKm0P0000=Fo@00E_d00P0000=Fo@000000200000IFo@00001Fo@00E_d200000eKm0000000X
Ool001Yoo`8000Uoo`03001Bo@0000@00006E_d00000E_d005Km0P0000=Fo@00E_d00`0000=Fo@00
E_d00P0000=Bo@00D_d00P0000=Bo@00D_d00`0000=Bo@00D_d0100000IFo000001Fo000F_`30000
1U[l000005Kl001Fo0800003E_`005Kl00<00003E_`0000000800006E_`00000E_d005Km0`0000=F
o@00D_d00P0000=Fo000E_`00P0000=Fo00000000`0000=Fo00000000P0000=Bo@0000000P0000=>
o@00B]d00`000T:m00D005[k000005kk00800005F_`00000E_`00002E_`4000015Kl000006Ki1`00
00LenP00K[D006jD001be00200002G:c001b/P00KaL007<G001S6`02000015/L000005Kl0P0000AF
o000E_aFo0800003E_d0000000800003E_d0000000<00003E_d0000000800006E_d00000E_d005Km
0P0000AFo@00E_eFo@800003E_d005Km00800003E_d0000000<002Moo`006Woo00<007ooOol02Goo
0P0000=Bo@00D_d00P0000=Fo@00E_d00P0000=Fo@0000000`0000=Fo@0000000P0000=Fo@000000
0P0000=Bo@0000000`0000=Bo@0000000P0000=Bo@00D_d00P0000=Bo000E_`00P000U[l0P0000=J
o000E_`00P0000=Fo00000000`0000IFo000001Fo000E_`200000eKl001Fo@020002E_d400000e;m
0000000200001EKl000005Kl00000UKl0P0000=Fo000E_d00P0000=Bo@00D_d010000TKM1@0000AN
fP00001Nn`800003F_`0000000P00007E_`00000H_X003HK00000RHL1@0000Q^U000001bd`00L[<0
07:b0P0000Ec5`00LaL006<K00800004Fa`00000E_`500000eKl0000000500000eKm001Fo@030000
0eKm001Fo@0200000eKm001Fo@0200000eKm0000000300000eKm0000000200001UKm000005Km001F
o@<002Ioo`006Woo00<007ooOol02goo0`0000=Bo@0000000P0000=Fo@0000000UKm00L005Km0000
05Km001Fo@0200000eKm001Bo@0300000e;m001Bo@0200000e;m001Bo@0400000e;m000000020000
1eKl00000000F_`005[l00800003F_`00000009Fo005001Fo000001Fo00400000eKl000000020000
1eKm00000000E_d005;m00800003D_d005Kl00800003E_`005Kl00<00003E_`0000000800003E_d0
000000800003D_d0000000T0009NfP03001Nnekk00@00006E_`00000E_`005;l0P0000=Bg000H_X0
1`0000/YV@00KY@006jC001bd`00L[<007:b00800003LaL007<G00800004Ha/00000Fa`200000eKl
001Fo00300000eKl001Fo@04000015Km000005Km1@0000=Fo@0000000P0000IFo@00001Fo@00E_d3
00000eKm001Fo@0200000eKm001Fo@0200000eKm0000000VOol001Yoo`03001oogoo00]oo`03001B
o@0000<00007E_d00000001Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000`0000=Bo@000000
0P0000=Bo@0000000P0000=Bo@00D_d0100000=Bo00000000U[l00D005[l000005[l00800004F_`0
05KlE_`200001UKl000005Kl001Fo0800003E_`005Kl00<00003E_d0000000800003D_d000000080
0003E_`0000000<00006E_`00000E_`005Kl0P0000=Fo@00D_d00P000U;m00<004km0000100000<n
O@00G]X01@0000EJo000000005Kl00@00003D_d005;L00800006G]X0048j@SX002XL0`0000M^U000
KY<007;C001b/`0200001W:b000007<G001c=`80009S6`04001K7000E_`200000eKl000000030000
0eKm0000000200001UKm000005Km001Fo@<00003E_d005Km00800003E_d005Km00800003E_d00000
00<00003E_d0000000800004E_d00000E_d2000UOol001Yoo`8000eoo`800003D_d0000000800003
E_d0000000800006E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Bo@00D_d00P0000=Bo@00D_d0
1@0000=Bo@0000000P0000=Bo000F_`00P0000AJo000001Jo0D00006E_`00000E_`005Kl0P0000=F
o00000000`0000IFo000001Fo@00E_d200000e;m001Bo@020002E_`01`00E_`00000E_`005Kl0080
0003E_`0000000800003D_d0000000<00005C]d004Jm000V?@0600000f;j00000002F_`200000eKl
001Fo00400000dkM001NfP0800000fjD001^T`0200001G:c001b/P00L[800`0000Ec=`00LcL006<K
00800004Fa`00000E_`2000015Kl001Fo5Kl0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@00
00000`0000=Fo@0000000P0000IFo@00001Fo@00E_d300000eKm001Fo@0200000eKm001Fo@03000T
Ool001Yoo`03001oogoo00ioo`04001Bo@00D_d200000eKm001Fo@0300000eKm001Fo@0200000eKm
0000000200000eKm0000000200000e;m0000000300000e;m0000000200000e;m001Bo@04000015[l
000005[l0P000U[l00D005[l000005Kl00@00003E_`00000009Fo007001Fo000001Fo000E_d00P00
00=Fo@0000000P000U;m00D005Kl000005Kl00@00006E_`00000E_d005Km0`0000=Bo@00D_d02@00
00ARf@00I]P0009RnP800003F_`005Kl00800003E_`005;M00@00003D[/004iI009>F@@00009KY<0
06jC001b/`00L[8007:b00@00005LcL007<g001S6`0200001U/L000005Kl001Fo0<00003E_`00000
00D00003E_d005Km00800004E_d005KmE_d200000eKm001Fo@0200000eKm0000000200000eKm0000
000300000eKm0000000200000eKm0000000TOol001Uoo`03001oogoo011oo`<00003D_d0000000<0
0003E_d0000000800006E_d00000E_d005Km0P0000=Fo@00D_d00`0000=Bo@00D_d00P0000=Bo@00
D_d0100000=>o@00D_`00P0000=Jo00000000U[l00D005[l000005[l00800003E_`005Kl00<00003
E_`005Kl00800003E_`0000000800006E_d00000E_d005;m1@0000IFo000001Fo000E_`200000eKl
0000000300000eKm0000000200000e;m001>g@0500002Rhl001Vf6KH001RnP00H_X005[l0P0000=F
o00000000P0000A>g@000000009B^`05001>F@00000Z?00200001fjC001^T`00L[<007:b00800007
LY400000001c=`00LcL00P0000AS6`00001K70@00007E_`00000001Fo000E_d00P0000=Fo@00E_d0
0P0000=Fo@000000200000IFo@00001Fo@00E_d200000eKm001Fo@0300000eKm001Fo@0200000eKm
0000000SOol001Uoo`03001oogoo011oo`03001Bo@0000800003D_d005Km00800003E_d005Km0080
0003E_d0000000<00003E_d0000000800003D_d0000000800003D_d0000000800003D_d004km00@0
0003D_`00000009Jo005001Jo000001Jo00200000e[l001Fo00300000eKl0000000200001UKl0000
05Kl001Fo0800003E_d0000000<00003D_d0000000800003E_`005Kl00800005E_`00000E_`00002
E_d200000eKm001Bo@0400000dkM0000000400000fKH001Vf002000016;j000005[k0P0000=Fo000
D_`00P0000=>g@00?Yd010000UYG0`0000=^T`00KY<00`0000Ab/P00001bT@@00003LcL007<g0080
009S6`04001K7000E_`200000eKl0000000300000eKm0000000200001UKm000005Km001Fo@800004
E_d005KmE_d200000eKm0000000200000eKm0000000200000eKm00000003000015Km000005Km0P00
1Woo0`006Goo000IOol2000BOol200000e;m0000000200000eKm0000000200001eKm00000000E_d0
05Km00800003E_d005Km00800003D_d005;m00800003D_d005;m00D00003C_d0000000800003D_`0
05[l00800004F_/00000F_`500001UKl000005Kl001Fo0800003E_`0000000800003E_`00000009F
o@07001Fo@00001Bo@00D_d0100000IJo000001Fo000E_`300000eKm0000000700000dkM000n_P02
00001b7l001Vf6KH001Ze`00009RnP03001Nnekk00<00003D_`0000000H0009JE`<00007KY<006jC
001^LP00L[800P0000=bT@0000000`0000Ec=`00LcL006<K00800004Fa`00000E_`2000015Kl001F
o5Kl0P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000001P0000=Fo@00E_d00P0000=Fo@00
E_d00P0000=Fo@00E_d020002Goo00<007ooOol05Woo000IOol00`00Oomoo`0COol01000D_d005Km
100000EFo@00000005Km00@00003E_d0000000800003E_d0000000800003D_d0000000<00003D_d0
000000800003C_d004kM00@00003F_`00000009Nn`05001Jo5[l001Jo00200000eKl000000020000
1UKl000005Kl001Fo0<00003E_`005Km00800003E_d0000000800003D_d005Kl00800003F_`00000
00<00003E_`0000000800006D_d00000D_d005;m200000IVf000J]MZe`00H_X2000015kk000005Kl
0P0000=Bo000C]d01`0000ARE@000000009^T`04001^LP00L[8300000g:A0000000300001G<g001c
=`00Ga/00P0000AK7000001Fo0d00003E_d005Km00800003E_d005Km00800004E_d005KmE_d20000
0eKm0000000200000eKm0000000200001UKm000005Km001Fo@<000Qoo`03001oogoo01Ioo`006Goo
00<007ooOol057oo100000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm001Fo@0400001e;m
00000000D_d005;m00800003D_d005;m00@00003C]d005;l00800007F_/005kk001Nn`00F_`00P00
00IJo000001Fo000E_`200000eKl0000000300000eKl0000000200001UKm000005Km001Bo@@00003
E_`00000009Jo003001Fo00000H00003D_d0000000800003D_d004kM00H0009Ze`04001Z]P00IaX2
000015kk000005Kl10000TkM00@002Hm0000009RE@<0009^T`05001^LP00001b/P0200000g:A0000
00040002LcL2000015lK000005/L0P0000=Fo000E_`00P0000AFo000E_eFo@800003E_d000000080
0003E_d0000000800003E_d0000000H00003E_d005Km00800003E_d005Km00800003E_d005Km0080
0004E_d000000004Ool3000IOol001Uoo`03001oogoo01Eoo`07001Bo@00001Fo@00E_d00P0000=F
o@0000000P0000=Fo@0000000`0000=Fo@0000000P0000=Bo@0000000P0000=Bo@0000000P0000=B
o@00C_d0100000=Bo000F_/00`0000ANn`00001Jo0800003F_`005Kl00800003E_`00000009Fo007
001Fo000001Fo000E_`00P0000=Fo@0000000P0000=Bo@0000000P000UKl100000IFo000001Fo000
E_d200000e;m001Bo@04000014[M000n_Sjn0P0000AZe`00J[IZ]P80009W6P07001Nn`00001Fo000
D_`01`0000=RE@00JW@00P0000A^T`00KW9^LP800004L[800000LY4600000g<g001c>00200001UlK
000005/L001Fo0800003E_`0000000<00006E_d00000E_d005Km0P0000=Fo@00E_d00P0000=Fo@00
E_d00P0000=Fo@0000000`0000=Fo@0000000P0000=Fo@0000000P0000AFo@00001Fo@8000Aoo`03
001oogoo01Uoo`006Goo0P005goo00<005Km00000`0000=Fo@0000000P0000MFo@00000005Km001F
o@0200000eKm001Fo@0200000e;m001Bo@0200000e;m001Bo@0400000dkm00000005000015[k0000
05kk0P0000=Jo00000000P0000MFo000000005Kl001Fo00200000eKl0000000200001UKl000005Kl
001Fo@800003D_d004km00D00004F_`00000E_`400000eKm0000000200000e;m000000080002J[H0
2P00K[D006LJ001W6@00G_/005Kl0P0000=Bo@000000100000=RE@00JW@0100000E^LP00KW8007:b
00800003LY40000000D00003LcP007=H0080009O70<00006E_`00000E_`005Kl0`0000=Fo@000000
0P0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d300000eKm001Fo@0200000eKm
001Fo@0200000eKm000000030003Ool4000HOol001Qoo`03001oogoo01Qoo`04001Fo@00E_d30000
0eKm0000000200000eKm0000000200000eKm0000000200000e;m0000000200000e;m000000030000
0e;m0000000200001DkM001:g@00D]d00P000Ukk00L005kkG_/005[l001Jo00300000eKl00000002
00001UKl000005Kl001Fo0800003E_`0000000800003E_d0000000<00003C_d005Kl00800006F_`0
0000E_`005Kl0P0000=Fo@00D_d00P0000ABo@00D_eBo@H0009Z]P800003K[@006LI00800004G_/0
0000E_`20002D_d00`00B_h0000400001g/C00000000KW8006ib00800003LY4007:A00P00005LeP0
07=H001O7002000015/L000005Kl100000MFo@00000005Km001Fo@0200000eKm001Fo@0200000eKm
001Fo@0200000eKm0000000200000eKm0000000300000eKm0000000200001UKm000005Km001Fo@<0
01ioo`001Goo0P0017oo0P000goo0`001Goo00<007ooOol06Goo100000IFo@00001Fo@00E_d20000
0eKm001Fo@0200000eKm001Fo@0400001e;m00000000D_d005;m00800003D_d004km00H00003C]d0
05kk00800004G_/00000F_`300001U[l000005[l001Fo0800003E_`0000000800006E_`00000E_`0
05Kl1@0000=Bo@0000000P0000IFo000001Jo000E_`200000eKl0000000200000e;m000000030000
0e;m001:g@04000016Zf000006jd0P000VLI00D005lL000005LM00D00006B_h00000JW@007/C0`00
00=^LP00KW800P0000=bT@00LY402@0000EcF000KeT005lL00800006Fa`00000E_`005Kl0P0000=F
o@0000000`0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm001Fo@08
00000eKm0000000200000eKm0000000NOol000Aoo`04001oogoo0003Ool20004Ool00`00Oomoo`04
Ool00`00Oomoo`0JOol01@00E_d00000E_d0100000=Fo@0000000P0000=Fo@0000000`0000=Fo@00
00000P0000=Bo@0000000P0000=Bo@0000000P0000=>o@00C]d01P0000ANn`00001Nn`<00004F_`0
0000F_`400001UKl000005Kl001Fo0800003E_`0000000<00003E_`0000000800003D_d000000080
0003E_`005[l00800006E_`00000E_`005Km0`0000=Bo@00D_d0100000=:g@0000000P0000=^]@00
K[@00P0000EW6@00JaX005lL0080009G7@07001Bo@00001>oP00JW@00`0000D04000KW8006ib0080
0003LY4007:A00D00003Obd0000000800005KeT006mI001O700200000e/L0000000200001UKl0000
05Km001Fo@<00003E_d005Km00800003E_d005Km00800003E_d0000000800003E_d0000000800006
E_d00000E_d005Km0`0000=Fo@00E_d00P0000=Fo@0000007Goo0004Ool01000Oomoo`002Goo00<0
07ooOol017oo00<007ooOol06goo00L005Km000005Km001Fo@0300000eKm001Fo@0200000eKm001F
o@0200000eKm001Bo@0200000e;m001Bo@0200000e;m001Bo@0400000d[M0016g@0400001Ekk0000
0000G_/00P0000INo000001Jo000F_`200000eKl0000000200000eKl0000000300001UKl000005Kl
001Fo@800003D_d004[M00@00004G_/00000E_`500000eKm0000000200000e;m0000000200000d[M
000fWP0200000fjd001b/`0200001F/J001[>P00Ga`01@0000EBo@00000006Yd00D00003KW8006ib
00800003LY4007:A00L00009Oc000000001_F@00KeT005lL00800003Fa`005Kl00800003E_`00000
00800003E_d0000000<00003E_d0000000800006E_d00000E_d005Km0P0000=Fo@00E_d00P0000=F
o@0000000P0000=Fo@0000000`0000=Fo@0000000`0077oo0004Ool01000Oomoo`002Goo00<007oo
Ool017oo0P007Goo0P0000=Fo00000000P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@000000
0P0000=Bo@0000000P0000=Bo@0000000P0000=Bo@0000001P0000=>g@0000000ekj00L006;kH_/0
05lK001Jo00200000e[l0000000200000eKl0000000600000eKl0000000200000eKm000000020000
0dKN001Fo00200001Ekk000005Kl00000UKl0P0000=Fo@00D_d00P0000=Bo@00C_d01P0000A^]000
001b/`800004JcX006/j0002Ga`01@00Ead00000D_d00`0000=k4`0000000P0000I^LP00001^L@00
LY4200000gJ@0000000500002gmA00000000KeT006/j001O7000Fa`00P0000IFo000001Fo000E_d2
00000eKm0000000800000eKm0000000200000eKm0000000200001UKm000005Km001Fo@800003E_d0
05Km00<00003E_d005Km008001aoo`0017oo00@007ooOol000Uoo`03001oogoo00Aoo`03001oogoo
01ioo`04001Fo000E_`200000eKm001Fo@0200000eKm001Fo@0200000eKm0000000200000e;m0000
000300000e;m0000000200000e;m001>o@0600000d[M001NnP02000016;k000005lK0P0000EJo000
000005[l00@00006E_`00000E_`005Kl0P0000=Fo000E_`00P0000=Fo@000000100000=Fo0000000
0UlK100000=Fo00000000P0000=Bo@0000000P0000=>o@0000000`0000A^]000001b/`<00003JcX0
0000009O70800004Ead00000D_d200000g/C0000000300000fib001^L@0200000g:A001fT0090000
17mC000006/j0P0000AO7000001K70800003E_`0000000800003E_d0000000D00003E_d005Km00<0
0003E_d005Km00800003E_d005Km00800003E_d0000000800003E_d0000000800003E_d0000000H0
01]oo`001Goo0P002Goo0P001Goo00<007ooOol087oo0`0000=Fo00000000P0000=Fo@0000000P00
00MFo@00000005Km001Fo@0200000e;m001Bo@0200000e;m001Bo@0400000dkm000000060002H_X0
1@00H_X00000Ga/00`0000=Jo00000000P0000AFo000001Fo0@00003E_`0000000800006E_`00000
E_d005;m100000MFo000Ga/005lK001Fo00200000eKl0000000200000e;m0000000200000dkm001:
g@02000016jd000007:c100000E[>P00000005lL00800004Ead00000Ead700000fib001^L@020000
0g:A001fT00:00000gme00000002JcX01@00Ga`00000Fa`00P0000=Fo000E_`00P0000=Fo@00E_d0
1`0000=Fo@0000000`0000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm001Fo@0200000eKm
0000000200000eKm0000000KOol001Moo`03001oogoo021oo`03001Fo00000<00005E_d00000001F
o@0400000eKm0000000200000eKm0000000200000e;m0000000200000e;m0000000200000dkm001:
g@0300001TJm001RnP00H_T0009RnPD00003F_`005[l00800006E_`00000E_`005Kl0P0000=Fo000
E_`00P0000=Fo00000000P0000EBo@00000005Kl00<00004Ga/00000E_`200000eKm001Fo@020000
0e;m001Bo@04000014[M000007:c0P0000=jlP0000000P000V/j0P0000AO7000001G7@@00003D_d0
000000800003KW8006ia00800003LY4007J@00/00007NgH006/j001[>P00Ga`00P0000=K70000000
0P0000=Fo00000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d800000eKm0000000200000eKm
0000000200001UKm000005Km001Fo@800003E_d0000001Yoo`005goo00<007ooOol08Goo0P0000=F
o00000000P0000IFo000001Fo@00E_d200000eKm001Fo@0200000eKm001Bo@0200000e;m001Bo@02
00000e;m001Bo@0400000dKM000fWP03000016;i000006;j100000=O6`00F_`00P0000AJo000001F
o0@00003E_`0000000800003E_`0000000800005E_d00000001Bo@03000015[l000005lK0P0000=F
o00000000P0000=Fo@0000000P0000=Bo@0000000P0000=:g@00L[<0100000EoD@00001[>P00009W
>`07001O7000001G7@00Ead00P0000MBo@00000006ib001^L@0200000g:A001fT00800002gmC0000
0000NgL006/j001W>`00Ga`00P0000=K7000E_`00P0000=Fo000E_d00P0000=Fo@0000000P0000=F
o@0000000P0000IFo@00001Fo@00E_d300000eKm001Fo@0200000eKm001Fo@0200000eKm00000002
00000eKm00000003000IOol001Moo`8002Aoo`04001Fo000E_`200000eKl0000000200000eKm0000
000200000eKm0000000200000e;m0000000200000e;m0000000300000e;m0000000;000016;j0000
05lK0P0000AJo000001Jo0800003E_`005Kl00800003E_`005Kl00<00003E_`0000000800003E_d0
000000@00004Fa`00000Ga/200000eKl001Fo@0200000eKm001Bo@0200000e;m001Bo@0400000g:c
00000003000027mc000006Lk001W>`00Ga`4000015LM000005;m0P0000I^LP00001^L@00LY420000
0gk>00000006000017me000007]g0P0000MW>`00Ic/005lL001K700200000eKl00000002000015Km
000005Km100000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@000000200000IFo@00001Fo@00E_d2
00000eKm001Fo@03000HOol001Moo`03001oogoo02Aoo`<00006E_`00000E_`005Km0P0000=Fo@00
E_d0100000EFo@00000005;m00@00003D_d005;m00800003D_d004km00H00006I]T006KHI]P006;j
0P0000AO6`00001Jo0800003F_`005Kl00800003E_`0000000<00006E_`00000E_`005Kl0P0000=F
o@000000100000=K700000000UlL0`0000=Fo@0000000P0000=Bo@0000000P0000EBo@00000007:c
00D0000;Og<007me001W>`00Ic/005/L001G7@0200000eLM0000000200001Vib000006ia001bT@d0
0003NgL007Mg00800004Ic/00000Ga`2000015/L000005Kl100000=Fo@0000001`0000IFo@00001F
o@00E_d200000eKm001Fo@0200000eKm0000000800000eKm0000000200000eKm0000000HOol001Mo
o`03001oogoo02Aoo`05001Fo000001Fo00500000eKm0000000200000eKm0000000200001UKm0000
05;m001Bo@@00003D_d0000000800003C_d0000000@0009Vf003001Ve`00009VnP05001O6`00001J
o00400001EKl00000000E_`0100000=Fo00000000P0000IFo000001Fo@00D_d400000e/L001O7002
000015Kl000005Km100000=Bo@0000000P0000ABo@00001b/`H0009oM@800003Ic/006Lk00<00004
Ead00000Ead400001Fia001^L@00LY402P0000EoTP00000007Mg00800003LeT006Lk00800004Ga`0
0000Fa`200000eKl001Fo00200000eKm001Fo@0200000eKm0000000200000eKm0000000200000eKm
0000000200001UKm000005Km001Fo@800003E_d005Km00/001Moo`005Woo00<007ooOol09Woo0P00
00IFo000001Fo000E_`200000eKm0000000200000eKm0000000200000eKm0000000200000e;m0000
000200000e;m0000000400001TKN000N?@00I]P0009Ve`06001VnP00IaX005lK0P000U/L00D005[l
000005Kl00800003E_`005Kl00800003E_`0000000800003E_`0000000800003D_d0000000@00004
Ga`00000E_`200000eKm001Fo@0200000e;m001Bo@02000015;m000007[b0`0000EoL`00OgD007^G
00800004Ic/00000Fa`200002ULM000005LM001G7@00KW4006ia3P0000UoUP00000007=I001_F@00
Hc/00P0000AK7000001K70@00003E_`0000000800006E_d00000E_d005Km0P0000=Fo@00E_d00P00
00=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm0000
000GOol001Ioo`8002Uoo`03001Fo00000<00006E_`00000E_`005Km0P0000=Fo@00E_d00P0000=F
o@00D_d00P0000=Bo@00D_d0100000=Bo@0000001P000VKG00L006Zf001W6P00IaT00002Ga/01@00
Fa`00000F_`00P0000=Fo00000000P0000IFo000001Fo000E_`200000eKl001Fo@0200001E;m000j
_P00Fad00P0000AO7000001Fo0@00003E_d0000000800003D_d0000000T00006NiL00000LgX006<l
0P0000AK7000001G7@D00003KW4006ia00l00005NiL00000001_F@0900000eLL001Fo00200000eKl
0000000200000eKm0000000200000eKm0000000200001UKm000005Km001Fo@800003E_d005Km0080
0003E_d0000000800003E_d0000000800003E_d0000000<001Ioo`005Woo00<007ooOol0:Goo00@0
05Kl001Fo0800003E_`0000000800003E_d0000000800003E_d0000000800003D_d0000000800003
D_d0000000<00003D_d0000000D00003I]L006Zf00800005IaT006LI001O6`02000015/L000005[l
0P0000=Fo000E_`00P0000=Fo00000000P0000=Fo00000000P0000=Fo@000000100000AG7@00001O
70800003E_`005Km00800003E_d0000000<00003D_d0000000@00003@]h001HM00800005LgX006mk
001S?002000015/L000005LM100000=^L@00KW403P0000AkU`00001gV@<00003JeX006<k00800003
Fa`0000000H00003E_`005Km00800003E_d0000000800003E_d0000000800003E_d0000000800003
E_d0000000800006E_d00000E_d005Km0P0000=Fo@00E_d00P0000=Fo@0000000`005Goo000FOol0
0`00Oomoo`0ZOol300000eKl0000000200001eKl00000000E_d005Km00800003E_d0000000800003
D_d0000000800003D_d0000000800003D_d004km00D00003J[H006Zf008000=W6@05001O6`00001K
700200000e[l0000000200001UKl000005Kl001Fo0800003E_`0000000800003E_d0000000@00004
Ead00000Fa`400001eKm00000000E_d005;m00800003D_d0000000T00005Kg/00000001S?0020000
35/L000005LM001G7@00KW4006ia001^L@h00008MiT00000001[FP00Je/0009S?004001K7000Ea`2
00000eKl0000000;00000eKm001Fo@0200000eKm001Fo@0200000eKm0000000200000eKm00000002
00000eKm0000000200001UKm000005Km001Fo@8001Eoo`005Woo00<007ooOol0:goo00D005[l0000
05Kl00@00003E_`0000000800003E_d0000000800006E_d00000D_d005;m0P0000=Bo@00D_d01000
00=>o@00@[h00P0000AZ]P00001ZU@800005IaT006LI001O6`0200001U/L000005/L001Fo0800003
E_`0000000800006E_`00000E_`005Kl0P0000=Fo@00D_d0100000=G7@0000000P0000EFo0000000
05Km00@00003E_d0000000800003D_d0000000P00004Ie`00000Gc`200000e/L0000000300001Fia
001^L@00KW404@0000=_N`00Je/00P0000AS?000001K70800006Ea`00000E_`005Kl0P0000=Fo@00
0000400000=Fo@00E_d00P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@00
000057oo000FOol00`00Oomoo`0/Ool01`00F_`00000E_`005Kl00800003E_`0000000800003E_d0
000000800003E_d0000000800003D_d0000000800003D_d0000000L0000;J[H006ZE001ZU000IaT0
06/I001O6`0200000e/L0000000200001UKl000005Kl001Fo0800003E_`0000000800003E_`00000
00<00003D_d0000000800003Ead0000000800004E_`00000E_d400000eKm0000000200000e;m0000
0008000016=L000005ll0P0000=K7000Ead00P0000E^L@00KW4006ia01000005Kg/006]k001WF`02
000015ll000005/L0P0000=G700000000P0000IFo000001Fo@00E_d4000015Km000005Km100000=F
o@0000002`0000=Fo@0000000P0000=Fo@00E_d01@004goo000EOol00`00Ool0000^Ool00`00F_`0
000300000eKl0000000200001UKm000005Km001Fo@800003E_d0000000800003D_d0000000800003
D_d005;m00H00003J[H00000009ZU0800004JaT006/I0002Ga/01000Fa`005/L0P0000=Fo0000000
0`0000IFo000001Fo000E_`200000eKl0000000200000e;m0000000200000eLM001K700200001UKl
000005Km001Fo@800003E_d005;m00800003D_d004km00P00004Gcd00000Gc`400000eLM00000002
KW400`00KW40000@00001F]k001WF`00Ic/00P0000AO7000001K70800003E_`0000000800003E_`0
000000800003E_d0000000800006E_d00000E_d005Km0P0000=Fo@0000000P0000=Fo@0000000P00
00=Fo@0000000P0000=Fo@0000001@0000=Fo@0000004goo000EOol2000`Ool00`00F_`000030000
0eKl0000000200000eKm0000000200000eKm0000000200000e;m0000000200000e;m000000020000
0dkm0000000400001FZE001ZU000JW<00P0000A[6@00001O6`800005Fa`00000001K700200000eKl
0000000200000eKl0000000200001UKl000005Kl001Fo@800003D_d004km00@00004Fa`00000E_`4
00000eKm0000000200000e;m0000000200000dkm00000006000015/m000005/L100000IG7@00KW5^
L@00KW4700000d[M0000000800001FMK00000000Ic/00P0000AO7000001K70800003E_`005Kl0080
0003E_`0000000800003E_d0000000800003E_d0000000800006E_d00000E_d005Km0P0000=Fo@00
E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000000`001Woo0`0017oo0P000goo000E
Ool00`00Oomoo`0`Ool4000015Kl000005Kl100000=Fo@0000000P0000=Fo@0000000P0000=Bo@00
00000P0000=Bo@0000000`0000U:g@00000006ZE001ZU000JW<00`000V/I0P0000AO6`00001K7080
0006Fa`00000E_`005Kl0P0000=Fo00000000P0000=Fo00000000P0000=Fo@0000000P0000=>o@00
00000P0000IK7000001Fo000E_d200000eKm0000000200000e;m0000000200000dkm001:gP060000
1U/m000005/L001K7@<0009^L@03001^L@0000L00003B]d0000000T00004He`00000Hc/2000015lL
000005/L100000IFo000001Fo000E_d200000eKm0000000200000eKm0000000200000eKm00000002
00000eKm0000000200001UKm000005Km001Fo@800003E_d005Km00800003E_d0000000<000Ioo`03
001oogoo009oo`04001oogoo0002Ool001Eoo`03001oogoo02eoo`<00003Ool005[k00800006E_`0
0000E_`005Kl0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Bo@0000000P0000=Bo@00D_d01000
00=:o@0000000VZD0P0000EZLP00001[6@00009[6@05001O6`00001K700200000e/L000000020000
1UKl000005Kl001Fo0800003E_`0000000800003E_d0000000800003C_d0000000800003Fa`00000
00800006E_d00000E_d005Km0P0000=Bo@00D_d0100000=:g@000000100000=K7@0000000P0000EK
7@00001^L@00009^L@H00006Ea`00000C]d004KN1P0000AG?@00001S?0800004Hc`00000Ga`20000
0e/L001Fo00200000eKl0000000200000eKm0000000500000eKm001Fo@0200000eKm001Fo@020000
0eKm0000000200000eKm0000000200000eKm0000000200001UKm000005Km001Fo@8000Ioo`03001o
ogoo009oo`04001oogoo0002Ool001Eoo`03001oogoo02Yoo`<000Eoo`05001Nn`00001Fo0040000
0eKl0000000200001UKm000005Km001Fo@800003D_d005;m00@00003D_d0000000800007B_d006ZD
001ZL`00JW80100000E[6@00JaT005lK00800006Fa`00000Fa`005Kl0P0000=Fo00000000P0000IF
o000001Fo000E_d200000eKm001Bo@0200000e;m001Fo@0200000eLL0000000200000eKm00000002
00000eKm0000000200000e;m0000000200000d[M0012gP0400000eLM001K7@02000015LM001^L@00
0Via1`0000=G700000000P0000=6gP00@]h0100000AG?@00001S?0800004Hc`00000Ga`400001UKl
000005Kl001Fo0800003E_d0000000800003E_d0000000800003E_d0000000800006E_d00000E_d0
05Km0P0000=Fo@00E_d00P0000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000001Goo00<007oo
Ool00Woo00@007ooOol0009oo`005Goo0P00:7oo0`002Goo00L005kk000005[l001Fo00200000eKl
0000000200000eKm0000000200000eKm0000000200000e;m0000000200000e;m0000000300000fZD
001ZL`0200002W[A000007m?001[6@00JcX005lK0P0000=K700000000P0000AFo000001Fo0@00003
E_`0000000800003E_d0000000800003D_d0000000800006E_d00000Ea`005Kl0P0000=Fo@000000
0P0000=Fo@0000000P0000=Bo@000000100000=6gP00B_h0100000UK7@00000006ia001^L@00LY00
1`0000=Fo@000000100000=6gP0000000P0000AG?@00001O?0800006Gc`00000Fa`005/L0P0000=F
o00000000P0000IFo000001Fo@00E_d200000eKm0000000200000eKm0000000200000eKm00000002
00000eKm0000000500000eKm0000000:0004Ool00`00Oomoo`02Ool01000Oomoo`000Woo0002Ool2
0004Ool20003Ool40003Ool00`00Oomoo`0UOol3000=Ool00`00G_]Nn`0300000eKl000000020000
15Kl000005Km100000=Fo@0000000P0000=Bo@0000000P0000=Bo@00C_d00P0000=ZU000JW<00`00
00=o;`00Odl00P0000E[>P00JcX005lK00800003Fa`005/L00800006E_`00000E_`005Kl0P0000=F
o00000000P0000=Fo@0000000P0000=Bo@0000000P0000=Fo@0000000P0000IFo000001Fo@00E_d2
00000eKm0000000200000e;m0000000600000d[n00000002000025/M000006ia001^L@00LY050000
15LL000005Km1P0000=6gP00B_h00P0000AG7@00001O?0800003Gc`005/L00800004Fa`00000E_`<
00000eKm0000000200000eKm0000000;00000eKm0000000200001UKm000005Km001Fo@800003E_d0
000000=oo`8000Eoo`8000=oo`0000Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo00=oo`03001o
ogoo029oo`<0011oo`06001Nn`00G_/005[l0P0000=Fo00000000P0000IFo000001Fo@00E_d20000
0eKm001Bo@0300000e;m0000000200000dkm00000002JW<3000017l_000007mB0P0000E[>P00IcX0
05lL00@00004Fa`00000E_`400001UKl000005Kl001Fo0800003E_d0000000800003D_d000000080
0003E_d0000000800003E_`0000000800006E_d00000E_d005;m0P0000=Bo@00C_d01P0000=>oP00
00000P0000MG7@00000006ia001bT006000015LL000005Km200000A>oP00001G7@800003Gc`005ll
00800003Fa`0000000<00005E_`00000001Fo00400000eKm0000000200000eKm0000000200001UKm
000005Km001Fo@800003E_d0000000800003E_d0000000800003E_d0000000800003E_d000000080
00moo`0000Eoo`00Oomoo`0000Yoo`03001oogoo009oo`03001oogoo01moo`<001Aoo`800004G_/0
0000F_`200000eKl0000000200000eKm0000000200000eKm0000000200000e;m0000000200000e;m
0000000300000fYc001ZLP03000017m?000007mC0P0000MW>P00IcX005lL001K700200001U/L0000
05Kl001Fo0800003E_`0000000800006E_`00000E_d005Km0P0000=Bo@00D_d00P0000=Fo@00E_`0
0P0000=Fo00000000P0000=Fo@0000000P0000=Bo@0000000P0000=Bo@000000100000=>oP000000
0P0000IG7@00001^L@00LY0700001ULM000005Km001>o@H00006C_h00000Ead005ll0P0000=O7000
00000P0000=K700000000P0000IFo000001Fo000E_d200000eKm0000000200000eKm000000020000
0eKm0000000200001UKm000005Km001Fo@800003E_d005Km00800003E_d00000008000=oo`03001o
ogoo00eoo`0000Eoo`00Oomoo`0000]oo`05001oogooOol0000NOol3000IOol01@00G_/00000F_`0
0P0000=Fo000E_`00P0000=Fo@0000000P0000=Fo@0000000P0000=Bo@0000000P0000=Bo@000000
0P0000=ZL`00JW800`0000EoC`00000007]E00800004IcX00000Ga`2000015/L000005/L100000AF
o000001Fo0@00003E_`0000000800003E_d0000000800003D_d0000000800004E_`00000E_`40000
0eKm0000000200000e;m0000000200000e;m0000000400000dkm0000000400000fia001bT0050000
15LM000005LM100000=Bo@000000100000=>o@0000000P0000=O?00000000P0000AK7000001K70@0
0003E_`0000000800004E_d00000E_d400000eKm0000000200000eKm0000000200000eKm00000002
00000eKm0000000200000eKm00000007Ool00`00Oomoo`0=Ool00005Ool007ooOol00008Ool01000
Oomoo`000goo0P006Woo0`007Goo00D006;k000005[l00@00006E_d00000E_d005Km0P0000=Fo@00
00000P0000=Bo@0000000P0000MBo@00000006Yc001ZLP05000017mC000007MG0P0000EW>P000000
05lL00800004Fa`00000F_`400001EKl00000000E_`00P0000=Fo00000000P0000=Fo@0000000P00
00=Bo@0000000P0000IFo000001Fo000E_d200000eKm0000000200000e;m0000000200000e;m001>
o@0400000dkm001G7@0300000fia001bT006000015LM000005LM4P0000AO7000001K70800003Ea`0
05Kl00800003E_`0000000800003E_d0000000D00003E_d005Km00800003E_d005Km00800003E_d0
000000800003E_d00000008000Yoo`03001oogoo00aoo`000Woo0P002Woo0P0017oo00<007ooOol0
5Woo0`008Goo00<006<K00000P0000EK7000000005Km00@00006E_d00000E_d005Km0P0000=Bo@00
D_d00P0000IBo@00001ZL`00JW8500001GmC00000000MeP00`0000AW>`00001O70800006Fa`00000
F_`005Kl0P0000=Fo00000000P0000EFo000E_`005Km00800003E_d0000000800003D_d000000080
0003E_`0000000800006E_d00000E_d005Km0P0000=Bo@00D_d0100000=>o@000000100000IC7@00
001^L@00LY07000015Km000005Km0`0000=Bo@00C_d0100000=Bo@0000000P0000AK7000001K7080
0003Fa`0000000800004E_`00000E_`400000eKm0000000700000eKm0000000200001UKm000005Km
001Fo@8000ioo`03001oogoo00aoo`0057oo00<007ooOol057oo0P009Goo0P0000IS6`00001K7000
E_`200000eKm0000000200001EKm00000000E_d0100000=Bo@0000001@0000=jd@0000001@0000Mg
F000LeT006mJ001S>`02000015lL000005/L100000AFo000001Fo0@00005E_`00000001Fo@020000
0eKm001Bo@0200000e;m001Fo@0200000eKl0000000200000eKm0000000200000eKm000000020000
0e;m0000000200000dkm0000000400001E<M001^L@00LY000`0000=Fo@0000001P0000=Fo@000000
0P0000=>o@0000000P0000=Bo@0000000P0000AK7000001K70800003Fa`0000000800006E_`00000
E_`005Km0P0000=Fo@0000000P0000=Fo@000000300047oo00<007ooOol037oo000COol00`00Oomo
o`0BOol3000YOol00`00Ha]S6`03000015Kl000005Km100000=Fo@0000000P0000EFo@00000005;m
00@00004JW800000Obl300000dKN000VGP0300001FmJ001[FP00Ha/00P0000AO7000001K70@00006
E_`00000E_`005Kl0P0000=Fo00000000P0000EFo@00000005Km00@00006E_d00000E_`005Kl0P00
00=Fo@0000000P0000=Fo@0000000P0000=Bo@0000000P0000=>o@00C_d01@0000=^L@00LY001000
00EFo@00E_d005Km00@00003E_d0000000800003C_d0000000800003D_d0000000800004Fa`00000
Fa`200000e/L001G700200000eKl0000000200001UKm000005Km001Fo@800005E_d00000001Fo@04
000EOol00`00Oomoo`0<Ool001=oo`03001oogoo00moo`<002aoo`04001S6`00Ha/300001UKl0000
05Km001Fo@800003E_d0000000800003E_d0000000800004D_d00000JW8200000gl_000000080000
1F]J001W>`00Ha/00P0000IO7000001K7000E_`200000eKl00000002000015Kl000005Kl100000EF
o@00000005Km00@00003E_d0000000800006E_`00000E_d005Km0P0000=Fo@0000000P0000=Bo@00
0000100000=>o@0000000P0000=^L@00LY002@0000=Fo@0000000P0000=Fo@0000000P0000=Bo@00
00000P0000=Bo@00Ead00P0000AK7000001K70@00004Ea`00000E_`=00000eKm0000000IOol00`00
Oomoo`0<Ool001=oo`8000eoo`<0031oo`05001S6`00001K700200000eKl0000000200001UKm0000
05Km001Fo@800003E_d005;m00800003D_d006Yb00@00003D_d0000000L00005Ic/00000001S6`02
00000e/L00000002000015Kl000005Kl100000IFo000001Fo000E_`200001EKm00000000E_d01000
00=Fo@0000000P0000=Fo00000000P0000IFo@00001Fo@00D_d200000e;m001Bo@0400001dkm0000
0000KW4007:@00L00006E_d00000E_d005Km0P0000=Fo@0000000P0000=Bo@00D_d0100000AG7@00
001K70800003Fa`0000000800004Ea`00000E_`300000eKm0000000200000eKm00000002000KOol0
0`00Oomoo`0<Ool001=oo`03001oogoo00Uoo`<003Aoo`05001O6`00001K700200000eKl00000002
00000eKm0000000200000eKm0000000200001E;m00000000JW801@0000=Bo@000000200000AS>`00
001O6`800003Fa`0000000800004E_`00000E_`400000eKl0000000200001UKm000005Km001Fo@<0
0004E_d00000E_d400000eKl0000000200000eKm0000000200000e;m0000000200000e;m00000002
00001Tkm000006ia001bT0<00003E_d0000000800003E_d0000000800006E_d00000E_d005;m1000
00=Bo@0000000P0000AG7@00001K70800003Fa`005/L00H00006E_`00000E_d005Km0P000Woo00<0
07ooOol06Woo00<007ooOol037oo000COol00`00Oomoo`06Ool3000hOol2000015lL000005/L0P00
00EFo000000005Km00@00003E_d0000000800004D_d00000JW8600000e;m00000005000014[n0000
06<l0P0000IO7000001K7000Fa`200001UKl000005Kl001Fo0800003E_`0000000800003E_d00000
00800005E_d00000001Fo@0200000eKm001Fo00200000eKl0000000200000eKm0000000200000e;m
0000000200000e;m0000000200001E;m001^L@00LY00100000=Fo@0000000P0000=Fo@0000000P00
00=Fo@0000000P0000=Bo@0000000P0000=Bo@0000000P0000AG7@00001K70H00003E_`000000080
0003E_`00000008000Eoo`03001oogoo01Yoo`03001oogoo00]oo`004goo00<007ooOol00goo0`00
?Goo00D005lL000005LL00800003E_d005Km00800005E_d00000001Fo@0400000e;m001ZLP070000
0e;m00000005000014[n000005ll0P0000=O700000000P0000AK7000001Fo0@00004E_`00000E_`4
00000eKm0000000200001EKm00000000E_d0100000AFo000001Fo0@00003E_d0000000800003D_d0
000000800003D_d0000000@0009bT0800006E_d00000E_d005Km0P0000=Fo@0000000P0000=Fo@00
00000P0000=Bo@0000000P0000=Bo@0000000P0000=G7@000000100000AG7000001Fo0@000Yoo`03
001oogoo01Uoo`03001oogoo00]oo`004Woo0P000Woo0`00@Goo00D005lL000005LL00@00006E_d0
0000E_d005Km0P0000MFo@00000005;m001jd@0200000e;m0000000200000e;m001>o@0600000dkn
001O?00200000elL00000002000015/L000005Kl100000IFo000001Fo000E_`200000eKm00000002
00001EKm00000000E_d0100000IFo000001Fo000E_d200000eKm0000000200000e;m000000020000
0e;m0000000300000g:@001nc`0200000eKm00000002000015Km000005Km100000=Fo@0000000P00
00=Bo@0000000P0000=Bo@000000100000=K700000000P0000=G700000003Woo00<007ooOol06Goo
00<007ooOol02goo000BOol40015Ool01@00Fa`00000Ea`0100000=Fo@0000000P0000IFo@00001F
o@00E_d600000e;m001Bo@0400000dkm00000004000014kn000005lL0`0000AK7000001K70800003
E_`0000000800003E_`0000000800006E_`00000E_d005Km0P0000=Fo@0000000P0000EFo@000000
05Km00@00006E_`00000E_d005Km0P0000=Bo@00D_d00P0000=Bo@00D_d00P0000=bT00000000`00
00=Fo@0000000P0000IFo@00001Fo@00E_d200000eKm0000000200000e;m0000000600001ULM0000
05LL001K7080015oo`03001oogoo01Qoo`03001oogoo00]oo`004Woo0P00B7oo00D005/L000005Kl
00@00003E_d0000000800003E_d0000000800005E_d00000001Bo@0400000e;m0000000200000dkm
00000004000015;m000005/L100000=K7000Fa`00P0000=Fo000E_`00P0000=Fo00000000P0000=F
o00000000P0000IFo@00001Fo@00E_d200001EKm00000000E_d0100000=Fo00000000P0000=Fo@00
00000P0000=Bo@0000000P0000=Bo@00LY001P0000=Fo@0000000P0000=Fo@0000000P0000AFo@00
001Fo@P00003D_d0000000800003Ead00000008001Aoo`03001oogoo01Moo`03001oogoo00]oo`00
57oo00<007ooOol0AWoo00D005/L000005Kl00@00003E_d0000000800003E_d0000000L00005D_d0
0000001Bo@0400000dkm00000004000015;m000005/L100000AK7000001K70<00004E_`00000E_`4
00000eKl0000000200000eKm0000000200001EKm00000000E_d00P0000=Fo@0000000P0000=Fo000
00000P0000=Fo@0000000P0000=Bo@0000000`0000=bT00000001@0000=Fo@0000000P0000=Fo@00
00000P0000YFo@00001Fo@00E_d005;m001Bo@800003D_d005Km008001Qoo`03001oogoo01Moo`03
001oogoo00]oo`005Goo00<007ooOol0AWoo0P0000EK7000000005Km00800003E_d0000000800003
E_d0000000L00003D_d0000000800005D_d00000001Bo@0600001U;m000005/L001K70800004Fa`0
0000Fa`300001UKl000005Kl001Fo0800003E_`0000000800003E_d0000000800005E_d00000001F
o@0200000eKm0000000200000eKl0000000200000eKm0000000200000e;m0000000200001G:@0000
0000D_d0100000=Fo@0000000P0000=Fo@0000000P0000=Fo@0000001P0000=Bo@0000000P006goo
00<007ooOol05Woo00<007ooOol02goo000FOol00`00Oomoo`17Ool00`00Fa`00003000015Km0000
05Km0`0000=Fo@0000000P0000=Fo@0000000P0000=Bo@0000000P0000MBo@00000005;m001>o@04
00000e;m00000002000015/L000005/L0P0000EG7000000005Kl00@00004E_`00000E_`400000eKm
0000000200001EKm00000000E_d00P0000=Fo@00E_`00P0000=Fo00000000P0000=Fo@0000000P00
00EBo@00000007:@00@00003D_d0000000800003E_d0000000800003E_d0000000800005E_d00000
001Fo@04000POol00`00Oomoo`0EOol00`00Oomoo`0;Ool001Moo`8004Qoo`03001G700000<00008
E_d00000E_d005Km001Fo@@00003E_d0000000800003D_d0000000800003D_d0000000<00003C_d0
000000800003D_d0000000800004Fa`00000Fa`200001ELL00000000E_`0100000IFo000001Fo000
E_d200000eKm0000000200001EKm00000000E_d0100000AFo000001Fo0@00003E_d0000000800004
D_d00000LY0500000e;m001Bo@0200000eKm001Fo@0200000eKm00000006000UOol00`00Oomoo`0E
Ool00`00Oomoo`0:Ool001Uoo`03001oogoo04Ioo`03001G7@0000<00003E_d0000000800005E_d0
0000001Fo@0200000eKm0000000200000e;m0000000200000e;m0000000300000dkm000000020000
0e;m00000002000015/L000005/L0P0000EFo000E_`005Kl00@00003E_`0000000800004E_d00000
E_d400001EKm00000000E_d0100000IFo000001Fo000E_d200000eKm0000000800000e;m00000002
00000e;m00000002000015Km000005Km0`00:Goo00<007ooOol05Goo00<007ooOol02Woo000JOol0
0`00Oomoo`16Ool00`00Ead0000300000eKm0000000200000eKm0000000200001EKm001Fo@00D_d0
0P0000=Bo@00D_d00P0000=Bo@00D_d00`0000=>o@0000000P0000=Bo@0000000P0000AK7000001K
70@00003E_`005Kl00@00003E_`0000000800006E_d00000E_d005Km0P0000EFo@00000005Km00@0
0003E_`0000000800008E_d00000E_d005;m001Bo@H00003D_d0000000800003D_d0000000800003
E_d0000002eoo`03001oogoo01Aoo`03001oogoo00Yoo`006goo00<007ooOol0AWoo00<005Km0000
0`0000=Fo@0000000P0000=Fo@0000000P0000EFo@00000005;m00@00003D_d0000000800003D_d0
000000800005D_d00000001Bo@04000015/L000005/L100000AFo000001Fo0<00003E_`000000080
0003E_d0000000800004E_d00000E_d300000eKm0000000200000eKl0000000200000eKm0000000;
00000e;m0000000200000e;m00000002000`Ool00`00Oomoo`0COol00`00Oomoo`0:Ool001aoo`03
001oogoo04Ioo`800005E_d00000001Fo@0400000eKm0000000200001EKm00000000E_d0100000=B
o@0000000P0000=Bo@0000000P0000EBo@00000005;m00@00004Fa`00000Fa`4000015Kl000005Kl
0`0000=Fo00000000P0000=Fo@0000000P0000EFo@00000005Km00800003E_d0000000800003E_`0
000000800005E_d00000001Bo@0900000e;m00000002000eOol00`00Oomoo`0BOol00`00Oomoo`0:
Ool001eoo`8004Qoo`03001Fo@0000800003E_d0000000800003E_d0000000800003E_d000000080
0005E_d00000001Bo@0400000e;m0000000200001E;m00000000E_d0100000AK7000001G70@00005
E_`00000001Fo00200000eKl0000000200000eKm0000000200001EKm00000000E_d00P0000=Fo@00
00000P0000=Fo00000000P0000EFo000000005;m00@00003D_d00000008003Uoo`03001oogoo019o
o`03001oogoo00Yoo`007goo00<007ooOol0AWoo00<005Km00000`0000EFo@00000005Km00@00003
E_d0000000800005E_d00000001Bo@0400000e;m0000000200001E;m00000000E_d0100000AK7000
001G70@00005E_`00000001Fo00200000eKl0000000200000eKm0000000200001EKm00000000E_d0
0P0000=Fo@00E_d00P0000=Fo00000000P0000EFo000000005;m00@003ioo`03001oogoo015oo`03
001oogoo00Yoo`0087oo00<007ooOol0AWoo00<005Km00000`0000EFo@00000005Km00@00003E_d0
000000800005E_d00000001Bo@0400000e;m0000000200001E;m00000000E_d0100000AG7000001G
70@00005E_`00000001Fo00200000eKl0000000200000eKm0000000200001EKm00000000E_d01000
00AFo@00001Fo0P0009oo`03001oogoo03ioo`03001oogoo015oo`03001oogoo00Uoo`008Goo00<0
07ooOol0AWoo00@005Km001Fo@800005E_d00000001Fo@0400000eKm0000000200001EKm00000000
D_d0100000=Bo@0000000P0000EBo@00000005Km00@00004Ea`00000Ea`400001EKl00000000E_`0
0P0000=Fo00000000P0000=Fo@0000000P0000EFo@00000005Km00@00004E_d00000E_`30017Ool0
0`00Oomoo`0AOol00`00Oomoo`09Ool0029oo`8004Moo`<00004E_d00000E_d400001EKm00000000
E_d0100000=Fo@0000000P0000EBo@00000005;m00@00003D_d0000000800005E_d00000001G7002
00000eKl0000000200001EKl00000000E_`00P0000=Fo000E_d00P0000=Fo@0000000P0000EFo@00
000005Km00@00003E_d0000004]oo`03001oogoo011oo`03001oogoo00Uoo`0097oo00<007ooOol0
AGoo0`0000IFo@00001Fo@00E_d200001EKm00000000E_d0100000=Fo@0000000P0000EBo@000000
05;m00@00003D_d0000000800005E_d00000001G700200000eKl001Fo00200001EKl00000000E_`0
100000AFo@00001Fo@P00003E_d00000008004ioo`03001oogoo00moo`03001oogoo00Uoo`009Goo
00<007ooOol0AGoo0`0000=Fo@0000000P0000AFo@00001Fo@<00003E_d0000000800003E_d00000
00800005D_d00000001Bo@0400000e;m0000000200001EKm00000000Ea`0100000AFo000001Fo0<0
0003E_`0000000800004E_d00000E_d300000eKm00000002001COol00`00Oomoo`0>Ool00`00Oomo
o`09Ool002Ioo`03001oogoo04Eoo`<00003E_d0000000800004E_d00000E_d300000eKm00000002
00000eKm0000000200001E;m00000000D_d0100000=Bo@0000000P0000EFo@00000005Kl00@00004
E_`00000E_`300000eKl00000002000015Km000005Km0`00Egoo00<007ooOol03Woo00<007ooOol0
2Goo000WOol20017Ool200000eKm00000002000025Km000005Km001Fo@00E_d400000eKm00000002
00001E;m00000000D_d0100000=Bo@0000000P0000EFo@00000005Kl00@00004E_`00000E_`30000
0eKl0000000200000eKm0000001KOol00`00Oomoo`0=Ool00`00Oomoo`09Ool002Uoo`03001oogoo
04Eoo`800003E_d0000000800003E_d0000000800004E_d00000E_d300000eKm0000000200001E;m
00000000D_d0100000=Bo@0000000P0000EFo@00000005Kl00@00004E_`00000E_`300000eKl0000
0002001NOol00`00Oomoo`0<Ool00`00Oomoo`09Ool002Yoo`03001oogoo04Eoo`<00005D_d00000
001Fo@04000015Km000005Km0`0000=Fo@0000000P0000EBo@00000005;m00@00003D_d000000080
0005E_d00000001Fo004000015Kl000005Kl0`00Hgoo00<007ooOol037oo00<007ooOol027oo000[
Ool00`00Oomoo`15Ool300001E;m00000000E_d0100000AFo@00001Fo@<00003E_d0000000800005
D_d00000001Bo@0400000e;m0000000200001EKm00000000E_`0100000=Fo0000000IWoo00<007oo
Ool037oo00<007ooOol027oo000/Ool20016Ool300001E;m00000000E_d0100000AFo@00001Fo@<0
0003E_d0000000800005D_d00000001Bo@0400000e;m0000000200001EKm00000000E_`01000JGoo
00<007ooOol02goo00<007ooOol027oo000^Ool00`00Oomoo`14Ool300001E;m00000000E_d01000
00AFo@00001Fo@<00003E_d0000000800005D_d00000001Bo@0400000e;m00000002000015Km0000
0000KWoo00<007ooOol02Woo00<007ooOol027oo000_Ool00`00Oomoo`14Ool300001E;m00000000
E_d0100000AFo@00001Fo@<00003E_d0000000800005D_d00000001Bo@0400000e;m00000002001a
Ool00`00Oomoo`0:Ool00`00Oomoo`08Ool0031oo`03001oogoo04Aoo`<00005D_d00000001Fo@07
00001EKm00000000E_d0100000EFo@00000005;m00@007Ioo`03001oogoo00Uoo`03001oogoo00Qo
o`00<Goo00<007ooOol0A7oo0`0000EBo@00000005Km00800003E_d0000000800005E_d00000001F
o@04000015Km00000000Ngoo00<007ooOol027oo00<007ooOol027oo000bOol20015Ool:00000eKm
0000000200001EKm00000000E_d01000Ogoo00<007ooOol027oo00<007ooOol01goo000dOol00`00
Oomoo`13Ool00`00D_d0000300001EKm00000000E_d0100000AFo@00000008=oo`03001oogoo00Qo
o`03001oogoo00Moo`00=Goo00<007ooOol0@goo00<005;m00000`0000EFo@00000005Km00@008Mo
o`03001oogoo00Moo`03001oogoo00Moo`00=Woo00<007ooOol0@goo00<005;m00000`0000AFo@00
000008aoo`03001oogoo00Ioo`03001oogoo00Moo`00=goo0P00A7oo00<005;m00000`00Sgoo00<0
07ooOol01Woo00<007ooOol01goo000iOol00`00Oomoo`12Ool00`00Oomoo`2BOol00`00Oomoo`05
Ool00`00Oomoo`07Ool003Yoo`03001oogoo045oo`03001oogoo09=oo`03001oogoo00Aoo`03001o
ogoo00Moo`00>goo00<007ooOol0@7oo00<007ooOol0U7oo00<007ooOol00goo00<007ooOol01goo
000lOol20010Ool00`00Oomoo`2DOol00`00Oomoo`04Ool00`00Oomoo`06Ool003ioo`03001oogoo
03eoo`03001oogoo09Eoo`03001oogoo00=oo`03001oogoo00Ioo`00?goo00<007ooOol0?7oo00<0
07ooOol0UWoo00<007ooOol00Woo00<007ooOol01Woo0010Ool00`00Oomoo`0kOol00`00Oomoo`2G
Ool01@00Oomoogoo000027oo0011Ool00`00Oomoo`0jOol00`00Oomoo`2GOol01@00Oomoogoo0000
27oo0012Ool2000jOol00`00Oomoo`2HOol01000Oomoo`0027oo0014Ool00`00Oomoo`0gOol00`00
Oomoo`2IOol00`00Ool00008Ool004Eoo`03001oogoo03Ioo`03001oogoo09Uoo`04001oogoo0007
Ool004Ioo`03001oogoo03Eoo`03001oogoo09Yoo`03001oo`0000Moo`00Agoo0P00=Goo00<007oo
Ool0Vgoo0P001goo0019Ool00`00Oomoo`0bOol00`00Oomoo`2JOol30007Ool004Yoo`03001oogoo
031oo`03001oogoo09Moo`@000Yoo`00Bgoo00<007ooOol0;goo00<007ooOol0Tgoo10003Woo001<
Ool2000_Ool00`00Oomoo`2@Ool3000BOol004ioo`03001oogoo02aoo`03001oogoo08aoo`@001Eo
o`00Cgoo00<007ooOol0:goo00<007ooOol0R7oo10006Goo001@Ool00`00Oomoo`0ZOol00`00Oomo
o`24Ool4000MOol0055oo`03001oogoo02Uoo`03001oogoo081oo`@0025oo`00DWoo0P00:Goo00<0
07ooOol0O7oo10009Goo001DOol00`00Oomoo`0VOol00`00Oomoo`1iOol3000YOol005Eoo`03001o
ogoo02Eoo`03001oogoo07Eoo`@002aoo`00EWoo00<007ooOol097oo00<007ooOol0LGoo1000<7oo
001GOol2000TOol00`00Oomoo`1]Ool4000dOol005Uoo`03001oogoo025oo`03001oogoo06Uoo`@0
03Qoo`00FWoo00<007ooOol087oo00<007ooOol0IGoo1000?7oo001KOol00`00Oomoo`0OOol00`00
Oomoo`1ROol30010Ool005aoo`8001moo`03001oogoo05ioo`@004=oo`00GWoo00<007ooOol077oo
00<007ooOol0FWoo1000Agoo001OOol00`00Oomoo`0KOol00`00Oomoo`1FOol4001;Ool0061oo`03
001oogoo01Yoo`03001oogoo059oo`@004moo`00HGoo0P006Woo00<007ooOol0CWoo1000Dgoo001S
Ool00`00Oomoo`0GOol00`00Oomoo`1;Ool3001GOol006Aoo`03001oogoo01Ioo`03001oogoo04Mo
o`@005Yoo`00IGoo00<007ooOol05Goo00<007ooOol0@goo1000GWoo001VOol00`00Oomoo`0DOol0
0`00Oomoo`0oOol4001ROol006Moo`8001Aoo`03001oogoo03]oo`@006Ioo`00JGoo00<007ooOol0
4Goo00<007ooOol0=goo1000JWoo001ZOol00`00Oomoo`0@Ool00`00Oomoo`0dOol3001^Ool006]o
o`03001oogoo00moo`03001oogoo031oo`@0075oo`00K7oo0P003Woo00<007ooOol0;Goo1000MGoo
001^Ool00`00Oomoo`0;Ool00`00Oomoo`0YOol4001iOol006moo`03001oogoo00Yoo`03001oogoo
02Eoo`@007eoo`00L7oo00<007ooOol02Goo00<007ooOol08Goo1000PGoo001aOol20009Ool00`00
Oomoo`0NOol30025Ool007=oo`03001oogoo00Ioo`03001oogoo01Yoo`@008Qoo`00M7oo00<007oo
Ool01Goo00<007ooOol05Woo1000S7oo001eOol00`00Oomoo`04Ool00`00Oomoo`0BOol4002@Ool0
07Ioo`03001oogoo00=oo`03001oogoo00ioo`@009Aoo`00Mgoo0P000goo00<007ooOol02Woo1000
V7oo001iOol01000Oomoo`002Goo0`00W7oo001jOol00`00Ool00005Ool4002OOol007]oo`800003
Ool0000000800:=oo`00O7oo0P00Ygoo003oOolVOol00?moobIoo`00ogoo9Woo003oOolVOol00001
\
\>"],
  ImageRangeCache->{{{0, 292.812}, {237.125, 0}} -> {-0.066928, 0.00660802, 
  0.00356925, 0.00356925}}]
}, Open  ]],

Cell[TextData[{
  "We now need to invert this transform, evaluated at ",
  Cell[BoxData[
      \(TraditionalForm\`\(\[Sigma]\^2\) T\)]],
  ", and subtract its discounted and normalized value from the Black-Scholes \
value. First we load the standard Package:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Now we define the Laplace inversion by integration along a vertical line \
in the complex plane (the Bromwich integral), with real part given by the \
parameter \"contour\". The integration works, but ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " may warn us about the oscillatory nature of the integration. If you get \
irritating messages, you might like to turn it off, using"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Off[NIntegrate::slwcon]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "However, if you choose parameters that result in the integration yielding \
strange answers, you should not turn off such messages, but possibly explore \
the options to ",
  StyleBox["NIntegrate",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", as discussed in Section 2.6. Now we define the valuation by the use of \
the Bromwich integral. In the following function we compute the Black-Scholes \
value, the correction for a unit stock price, and the full value."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(DoubleBarrierCall[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, contour_] := \n
      Module[{bs, \ dim}, \n\t
        bs\  = \ BlackScholesCall[S, K, \[Sigma], r, q, t]; \n\t
        dim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\[IndentingNewLine]\t\[CapitalPhi][S, K, L, 
                    U, \[Sigma], r, q, contour\  + \ I\ p]*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n\t{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; \tbs - S*dim]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that I have not explored the various approximate schemes for \
transform inversion - here we are concerned with getting an accurate answer. \
You may wish to explore the stability under changes in the contour parameter. \
The range {{",
  Cell[BoxData[
      \(TraditionalForm\`contour - 1500  I\)]],
  "}, {",
  Cell[BoxData[
      \(TraditionalForm\`contour + 1500  I\)]],
  "}} has been found to be useful for several practical cases. The truncation \
at ",
  Cell[BoxData[
      \(TraditionalForm\`1500\)]],
  " may need to be modified for other situations."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The following numerical examples were considered by Geman and Yor \
(1996) - we see that our model agrees with their numerical simulations:\
\>", 
  "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCall[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0410889171638021721`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCall[2, \ 2, \ 1.5, \ 3.0, \ 0.5, \ 0.05, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0178568367282250779`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCall[2, \ 1.75, \ 1.0, \ 3.0, \ 0.5, \ 0.05, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0761723171390595332`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Hedging the Double-Barrier Call", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "It is now just a matter of differentiating the analytical result to obtain \
Delta and the other Greeks. Geman and Yor worked out a formula for the \
Laplace transform of Delta, then inverted this. We can just as easily get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to do the differentiations for us. Let's first take a look at Delta. We \
need to bear in mind that we have put a factor involving the current stock \
price outside the integration. So the differentiation will involve the \
original diminution function, and its derivative. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(Delta\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, z_]\  = \ 
      Evaluate[D[\[CapitalPhi][S, K, L, U, \[Sigma], r, q, z], \ S]]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(DoubleBarrierCallDelta[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, 
        contour_] := \n
      Module[{bsd, \ dim, \ deldim}, \n\t\tbsd\  = \ 
          BlackScholesCallDelta[S, K, \[Sigma], r, q, t]; \n\t\tdeldim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\n\t\t\t\t\((\[CapitalPhi][S, K, L, U, \[Sigma], r, 
                        q, contour\  + \ I\ p] + 
                      S*Delta\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p])\)*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n\t\t\t\t{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; bsd\  - deldim]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCallDelta[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0118013318207395023`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Other Greeks", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "For all the other Greeks except Theta, we just differentiate the Laplace \
transform, and add some corrections. The following set of functions serve as \
the necessary building blocks - the function ",
  Cell[BoxData[
      FormBox[
        StyleBox["Aux\[CapitalPhi]",
          FontFamily->"Courier",
          FontWeight->"Bold"], TraditionalForm]]],
  "  plays a role in the computation of both Theta and Vega, and Theta, the \
time-derivative, is computed using the appropriate Laplace transform \
identity. In what follows we define each Greek and perform a sample \
evaluation. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Gamma\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, z_]\  = \ 
      Evaluate[D[\[CapitalPhi][S, K, L, U, \[Sigma], r, q, z], \ {S, 2}]]; \n
    Rho\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, z_]\  = \ 
      Evaluate[D[\[CapitalPhi][S, K, L, U, \[Sigma], r, q, z], r]]; \n
    Vega\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, z_]\  = \ 
      Evaluate[D[\[CapitalPhi][S, K, L, U, \[Sigma], r, q, z], \[Sigma]]]; \n
    Aux\[CapitalPhi][S_, K_, L_, U_, \[Sigma]_, r_, q_, z_]\  = \ 
      z*\[CapitalPhi][S, K, L, U, \[Sigma], r, q, z]; \)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(DoubleBarrierCallGamma[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, 
        contour_] := \n
      Module[{bsd, \ dim, \ deldim, \ gamdim}, \n\t\tbsg\  = \ 
          BlackScholesCallGamma[S, K, \[Sigma], r, q, t]; \n\t\tgamdim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\n\t\t\t\((2*
                        Delta\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p] + 
                      S*Gamma\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p])\)*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n\t\t\t\t{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; bsg\  - \ gamdim]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCallGamma[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-0.454403857270716837`\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(DoubleBarrierCallRho[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, 
        contour_] := \n
      Module[{bsr, \ dim, \ rhodim}, \n\t\tbsr\  = \ 
          BlackScholesCallRho[S, K, \[Sigma], r, q, t]; \n\t\trhodim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\n\t\t\t\((t*\[CapitalPhi][S, K, L, U, \[Sigma], r, 
                          q, contour\  + \ I\ p] - 
                      Rho\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                        contour\  + \ I\ p])\)*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; \ \ bsr\  + \ S*rhodim]\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCallRho[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0791006299131088397`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(DoubleBarrierCallVega[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, 
        contour_] := \n
      Module[{bsr, \ dim, \ rhodim}, \n\t\tbsv\  = \ 
          BlackScholesCallVega[S, K, \[Sigma], r, q, t]; \n\t\tvegdim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\n\t\t\t\t\t\t\((Vega\[CapitalPhi][S, K, L, 
                        U, \[Sigma], r, q, contour\  + \ I\ p] + 
                      2*\[Sigma]*t*
                        Aux\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p])\)*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n\t{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; \ bsv\  - S*vegdim]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCallVega[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(-0.38284046250363545`\)\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(DoubleBarrierCallTheta[S_, K_, L_, U_, \[Sigma]_, r_, q_, t_, 
        contour_] := \n
      Module[{bsr, \ dim, \ rhodim}, \n\t\tbst\  = \ 
          BlackScholesCallTheta[S, K, \[Sigma], r, q, t]; \n\t\tthetdim\  = \ 
          Re[\ Exp[\(-r\)*t]/\((2\ Pi)\)*
              NIntegrate[\n\t\t\t\t\t\t\((\[Sigma]^2*
                        Aux\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p] - 
                      r*\[CapitalPhi][S, K, L, U, \[Sigma], r, q, 
                          contour\  + \ I\ p])\)*
                  Exp[\((contour + I\ p)\)*\[Sigma]^2*
                      t], \n\t{p, \ \(-1500\), \ 1500}, \ 
                MaxRecursion\  -> \ 10]]; \ bst\  + S*thetdim]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(DoubleBarrierCallTheta[2, \ 2, \ 1.5, \ 2.5, \ 0.2, \ 0.02, \ 0, \ 1, \ 
      10]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.0367020336521083612`\)], "Output"]
}, Open  ]],

Cell["\<\
All of these results can be confirmed by the use of an approximate \
differencing technique, but they illustrate the point that the use of \
symbolic differentiation is not just confined to where there is a simple \
\"nice formula\".\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.3 Compound Options", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In this section we present a brief summary of the valuation of compound \
options as described by Hull (1996) together with some sample valuations. \
This model makes use of the standard ",
  StyleBox["BlackScholes",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " package for determining critical prices at the first exercise date, and \
also uses the Package (new in version 3.0) for the multivariate normal \
distribution. This avoids the use of any approximations, but at the time of \
writing I can only recommend its use for two dimensions. We have not \
considered the computation of Greeks here due to some issues it raises \
differentiating the new Package functions - they can of course be estimated \
using central difference methods."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Basic Package Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
First we load some routines to model the multivariate normal \
distribution and vanilla options.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Statistics`MultinormalDistribution`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["We want the two-variable case of the multi-normal object:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?MultinormalDistribution\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("MultinormalDistribution[mu, sigma] represents the multivariate normal \
(Gaussian) distribution with mean vector mu and covariance matrix sigma. For \
a p-variate random vector to be distributed MultinormalDistribution[mu, \
sigma], mu must be a p-variate vector, and sigma must be a p x p symmetric  \
positive definite matrix."\)], "Print"]
}, Open  ]],

Cell["\<\
The cumulative distribution function (CDF) or M-function, as \
described by Hull (1996), is defined as follows:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(mu\  = \ {0, 0}; \nsigma[rho_]\  := \ {{1, \ rho}, \ {rho, \ 1}}\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Mfunc[a_, \ b_, \ rho_]\  := \ 
      Re[CDF[MultinormalDistribution[mu, \ sigma[rho]], \ {a, \ b}]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Here is a plot for zero correlation:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot3D[
      Mfunc[a, \ b, \ 0], \ {a, \ \(-3\), \ 3}, \ {b, \ \(-3\), \ 3}]; \)\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .81114 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% SurfaceGraphics
%%IncludeResource: font Courier
%%IncludeFont: Courier
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-1.11022e-16 1.04977 -0.0679587 1.04977 [
[.12586 .2075 -12 -8.99448 ]
[.12586 .2075 0 .00552 ]
[.3345 .12542 -5.5867 -9 ]
[.3345 .12542 .4133 0 ]
[.56094 .0364 -5.16972 -9 ]
[.56094 .0364 .83028 0 ]
[.75679 .08071 0 -6.17777 ]
[.75679 .08071 12 2.82223 ]
[.8565 .23339 0 -6.03127 ]
[.8565 .23339 6 2.96873 ]
[.9429 .3657 0 -5.9083 ]
[.9429 .3657 6 3.0917 ]
[.02411 .26511 -6 -2.74232 ]
[.02411 .26511 0 6.25768 ]
[.01566 .3181 -24 -2.78712 ]
[.01566 .3181 0 6.21288 ]
[.00691 .37294 -18 -2.83365 ]
[.00691 .37294 0 6.16635 ]
[-0.00215 .42974 -24 -2.88201 ]
[-0.00215 .42974 0 6.11799 ]
[-0.01154 .4886 -6 -2.93231 ]
[-0.01154 .4886 0 6.06769 ]
[ 0 0 0 0 ]
[ 1 .81114 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(-2)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(2)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.7445 .0853 m
.73835 .08759 L
s
[(-2)] .75679 .08071 -1 .37284 Mshowa
.84407 .23761 m
.83786 .23973 L
s
[(0)] .8565 .23339 -1 .34028 Mshowa
.93037 .36962 m
.92411 .37158 L
s
[(2)] .9429 .3657 -1 .31296 Mshowa
.125 Mabswid
.7708 .12553 m
.7671 .12688 L
s
.79613 .16428 m
.79242 .1656 L
s
.82054 .20161 m
.81682 .20291 L
s
.86679 .27235 m
.86305 .2736 L
s
.88871 .30589 m
.88497 .30711 L
s
.9099 .3383 m
.90615 .3395 L
s
.71716 .04348 m
.71349 .04489 L
s
.95018 .39992 m
.94642 .40107 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
.03634 .26033 m
.04245 .25795 L
s
[(0)] .02411 .26511 1 -0.3906 Mshowa
.02792 .31343 m
.03405 .31109 L
s
[(0.25)] .01566 .3181 1 -0.38064 Mshowa
.01921 .36838 m
.02537 .3661 L
s
[(0.5)] .00691 .37294 1 -0.3703 Mshowa
.01019 .4253 m
.01637 .42308 L
s
[(0.75)] -0.00215 .42974 1 -0.35955 Mshowa
.00085 .48428 m
.00704 .48212 L
s
[(1)] -0.01154 .4886 1 -0.34838 Mshowa
.125 Mabswid
.03468 .27081 m
.03835 .26938 L
s
.03301 .28135 m
.03668 .27993 L
s
.03132 .29197 m
.035 .29056 L
s
.02963 .30266 m
.03331 .30126 L
s
.02621 .32427 m
.02989 .32287 L
s
.02448 .33518 m
.02816 .33379 L
s
.02273 .34617 m
.02642 .34479 L
s
.02098 .35724 m
.02467 .35586 L
s
.01744 .3796 m
.02113 .37824 L
s
.01564 .39091 m
.01934 .38955 L
s
.01384 .40229 m
.01754 .40094 L
s
.01202 .41375 m
.01573 .41241 L
s
.00835 .43692 m
.01206 .4356 L
s
.0065 .44864 m
.01021 .44732 L
s
.00463 .46043 m
.00834 .45912 L
s
.00274 .47231 m
.00646 .47101 L
s
.25 Mabswid
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
.40529 .59895 m
.03716 .25514 L
s
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.40529 .59895 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.39787 .81114 L
s
.39787 .81114 m
.40529 .59895 L
s
0 0 m
1 0 L
1 .81114 L
0 .81114 L
closepath
clip
newpath
.5 Mabswid
.656 .74 .911 r
.38311 .58349 .40511 .60397 .44186 .59365 .42018 .57289 Metetra
.641 .735 .917 r
.42018 .57289 .44186 .59365 .47912 .5846 .45775 .56355 Metetra
.605 .724 .929 r
.45775 .56355 .47912 .5846 .51693 .57854 .49587 .55721 Metetra
.525 .694 .945 r
.49587 .55721 .51693 .57854 .55544 .57802 .53466 .55643 Metetra
.379 .625 .949 r
.53466 .55643 .55544 .57802 .59491 .58551 .57439 .56367 Metetra
.183 .513 .914 r
.57439 .56367 .59491 .58551 .63567 .6017 .61538 .57963 Metetra
.042 .42 .865 r
.61538 .57963 .63567 .6017 .67797 .6241 .65789 .60182 Metetra
.045 .421 .866 r
.65789 .60182 .67797 .6241 .72176 .64729 .70192 .6248 Metetra
.183 .511 .913 r
.70192 .6248 .72176 .64729 .7667 .66533 .74716 .64261 Metetra
.365 .619 .949 r
.74716 .64261 .7667 .66533 .81237 .67464 .7932 .65166 Metetra
.505 .691 .952 r
.7932 .65166 .81237 .67464 .85845 .67517 .83971 .65187 Metetra
.585 .728 .942 r
.83971 .65187 .85845 .67517 .90487 .66918 .88662 .64555 Metetra
.624 .745 .934 r
.88662 .64555 .90487 .66918 .95175 .65941 .93403 .63542 Metetra
.639 .752 .93 r
.93403 .63542 .95175 .65941 .99922 .6478 .98205 .62344 Metetra
.656 .74 .911 r
.36055 .56248 .38311 .58349 .42018 .57289 .39793 .55159 Metetra
.642 .735 .917 r
.39793 .55159 .42018 .57289 .45775 .56355 .43582 .54194 Metetra
.606 .724 .929 r
.43582 .54194 .45775 .56355 .49587 .55721 .47425 .53527 Metetra
.528 .694 .945 r
.47425 .53527 .49587 .55721 .53466 .55643 .51333 .53412 Metetra
.385 .627 .949 r
.51333 .53412 .53466 .55643 .57439 .56367 .55331 .54095 Metetra
.197 .518 .916 r
.55331 .54095 .57439 .56367 .61538 .57963 .59452 .55646 Metetra
.064 .43 .872 r
.59452 .55646 .61538 .57963 .65789 .60182 .63723 .57815 Metetra
.074 .432 .873 r
.63723 .57815 .65789 .60182 .70192 .6248 .68147 .60062 Metetra
.211 .517 .915 r
.68147 .60062 .70192 .6248 .74716 .64261 .72699 .61794 Metetra
.385 .617 .943 r
.72699 .61794 .74716 .64261 .7932 .65166 .77336 .62653 Metetra
.516 .683 .941 r
.77336 .62653 .7932 .65166 .83971 .65187 .82029 .62632 Metetra
.591 .716 .931 r
.82029 .62632 .83971 .65187 .88662 .64555 .86768 .61959 Metetra
.627 .731 .922 r
.86768 .61959 .88662 .64555 .93403 .63542 .91559 .60905 Metetra
.642 .738 .918 r
.91559 .60905 .93403 .63542 .98205 .62344 .96414 .59666 Metetra
.657 .74 .911 r
.3374 .54092 .36055 .56248 .39793 .55159 .3751 .52971 Metetra
.642 .735 .916 r
.3751 .52971 .39793 .55159 .43582 .54194 .41332 .5197 Metetra
.608 .724 .928 r
.41332 .5197 .43582 .54194 .47425 .53527 .45206 .5126 Metetra
.533 .695 .943 r
.45206 .5126 .47425 .53527 .51333 .53412 .49144 .51089 Metetra
.4 .631 .948 r
.49144 .51089 .51333 .53412 .55331 .54095 .53166 .51698 Metetra
.228 .531 .921 r
.53166 .51698 .55331 .54095 .59452 .55646 .57307 .53153 Metetra
.114 .451 .884 r
.57307 .53153 .59452 .55646 .63723 .57815 .61595 .55211 Metetra
.134 .455 .885 r
.61595 .55211 .63723 .57815 .68147 .60062 .66037 .57344 Metetra
.266 .528 .914 r
.66037 .57344 .68147 .60062 .72699 .61794 .70609 .5897 Metetra
.422 .61 .927 r
.70609 .5897 .72699 .61794 .77336 .62653 .75275 .59741 Metetra
.536 .663 .919 r
.75275 .59741 .77336 .62653 .82029 .62632 .80004 .5965 Metetra
.6 .691 .906 r
.80004 .5965 .82029 .62632 .86768 .61959 .84783 .58921 Metetra
.631 .704 .898 r
.84783 .58921 .86768 .61959 .91559 .60905 .8962 .57819 Metetra
.644 .709 .895 r
.8962 .57819 .91559 .60905 .96414 .59666 .94525 .56535 Metetra
.657 .74 .911 r
.31364 .51879 .3374 .54092 .3751 .52971 .35167 .50722 Metetra
.644 .736 .916 r
.35167 .50722 .3751 .52971 .41332 .5197 .39021 .49678 Metetra
.611 .725 .926 r
.39021 .49678 .41332 .5197 .45206 .5126 .42928 .48907 Metetra
.544 .697 .94 r
.42928 .48907 .45206 .5126 .49144 .51089 .46896 .48646 Metetra
.427 .638 .944 r
.46896 .48646 .49144 .51089 .53166 .51698 .50944 .4912 Metetra
.284 .552 .925 r
.50944 .4912 .53166 .51698 .57307 .53153 .55103 .50392 Metetra
.198 .485 .897 r
.55103 .50392 .57307 .53153 .61595 .55211 .59404 .5223 Metetra
.229 .486 .893 r
.59404 .5223 .61595 .55211 .66037 .57344 .63855 .54133 Metetra
.346 .539 .9 r
.63855 .54133 .66037 .57344 .70609 .5897 .6844 .55553 Metetra
.471 .595 .895 r
.6844 .55553 .70609 .5897 .75275 .59741 .73123 .56162 Metetra
.559 .632 .88 r
.73123 .56162 .75275 .59741 .80004 .5965 .77876 .55956 Metetra
.609 .652 .868 r
.77876 .55956 .80004 .5965 .84783 .58921 .82687 .55146 Metetra
.633 .662 .86 r
.82687 .55146 .84783 .58921 .8962 .57819 .8756 .53983 Metetra
.643 .666 .857 r
.8756 .53983 .8962 .57819 .94525 .56535 .92503 .52646 Metetra
.658 .74 .91 r
.28924 .49605 .31364 .51879 .35167 .50722 .32761 .48408 Metetra
.646 .736 .915 r
.32761 .48408 .35167 .50722 .39021 .49678 .36649 .4731 Metetra
.618 .725 .923 r
.36649 .4731 .39021 .49678 .42928 .48907 .4059 .46456 Metetra
.561 .7 .934 r
.4059 .46456 .42928 .48907 .46896 .48646 .44589 .46055 Metetra
.467 .65 .938 r
.44589 .46055 .46896 .48646 .50944 .4912 .48664 .46311 Metetra
.361 .579 .924 r
.48664 .46311 .50944 .4912 .55103 .50392 .52842 .47278 Metetra
.306 .524 .901 r
.52842 .47278 .55103 .50392 .59404 .5223 .57151 .48743 Metetra
.34 .517 .885 r
.57151 .48743 .59404 .5223 .63855 .54133 .61604 .50257 Metetra
.429 .544 .871 r
.61604 .50257 .63855 .54133 .6844 .55553 .66189 .51332 Metetra
.517 .575 .853 r
.66189 .51332 .6844 .55553 .73123 .56162 .70875 .51679 Metetra
.578 .597 .835 r
.70875 .51679 .73123 .56162 .77876 .55956 .75638 .51296 Metetra
.613 .609 .823 r
.75638 .51296 .77876 .55956 .82687 .55146 .80465 .50372 Metetra
.631 .616 .817 r
.80465 .50372 .82687 .55146 .8756 .53983 .85359 .4913 Metetra
.638 .618 .814 r
.85359 .4913 .8756 .53983 .92503 .52646 .90326 .4773 Metetra
.659 .74 .91 r
.26419 .47268 .28924 .49605 .32761 .48408 .30289 .46027 Metetra
.649 .736 .913 r
.30289 .46027 .32761 .48408 .36649 .4731 .34213 .44865 Metetra
.626 .727 .919 r
.34213 .44865 .36649 .4731 .4059 .46456 .3819 .43901 Metetra
.582 .705 .927 r
.3819 .43901 .4059 .46456 .44589 .46055 .42224 .43311 Metetra
.514 .664 .929 r
.42224 .43311 .44589 .46055 .48664 .46311 .46328 .43261 Metetra
.443 .607 .916 r
.46328 .43261 .48664 .46311 .52842 .47278 .50526 .43793 Metetra
.412 .56 .892 r
.50526 .43793 .52842 .47278 .57151 .48743 .54843 .44728 Metetra
.438 .542 .865 r
.54843 .44728 .57151 .48743 .61604 .50257 .59293 .45692 Metetra
.496 .547 .838 r
.59293 .45692 .61604 .50257 .66189 .51332 .63867 .46283 Metetra
.552 .559 .815 r
.63867 .46283 .66189 .51332 .70875 .51679 .68544 .46267 Metetra
.591 .569 .797 r
.68544 .46267 .70875 .51679 .75638 .51296 .73302 .45645 Metetra
.615 .576 .787 r
.73302 .45645 .75638 .51296 .80465 .50372 .7813 .44574 Metetra
.627 .579 .781 r
.7813 .44574 .80465 .50372 .85359 .4913 .83029 .43237 Metetra
.632 .581 .779 r
.83029 .43237 .85359 .4913 .90326 .4773 .88005 .41765 Metetra
.66 .74 .909 r
.23845 .44867 .26419 .47268 .30289 .46027 .2775 .43578 Metetra
.653 .737 .911 r
.2775 .43578 .30289 .46027 .34213 .44865 .3171 .42343 Metetra
.636 .729 .915 r
.3171 .42343 .34213 .44865 .3819 .43901 .35725 .41252 Metetra
.605 .711 .919 r
.35725 .41252 .3819 .43901 .42224 .43311 .39797 .40437 Metetra
.56 .678 .918 r
.39797 .40437 .42224 .43311 .46328 .43261 .43936 .40022 Metetra
.515 .633 .905 r
.43936 .40022 .46328 .43261 .50526 .43793 .48159 .40036 Metetra
.496 .591 .879 r
.48159 .40036 .50526 .43793 .54843 .44728 .52489 .40341 Metetra
.511 .565 .847 r
.52489 .40341 .54843 .44728 .59293 .45692 .56936 .40654 Metetra
.544 .555 .816 r
.56936 .40654 .59293 .45692 .63867 .46283 .61497 .40676 Metetra
.577 .554 .792 r
.61497 .40676 .63867 .46283 .68544 .46267 .66158 .40237 Metetra
.602 .557 .776 r
.66158 .40237 .68544 .46267 .73302 .45645 .70902 .3934 Metetra
.617 .559 .767 r
.70902 .3934 .73302 .45645 .7813 .44574 .75722 .38102 Metetra
.624 .56 .762 r
.75722 .38102 .7813 .44574 .83029 .43237 .80617 .3666 Metetra
.628 .561 .761 r
.80617 .3666 .83029 .43237 .88005 .41765 .85591 .35111 Metetra
.661 .74 .909 r
.21199 .42398 .23845 .44867 .2775 .43578 .25139 .4106 Metetra
.656 .738 .91 r
.25139 .4106 .2775 .43578 .3171 .42343 .29137 .3975 Metetra
.646 .732 .912 r
.29137 .3975 .3171 .42343 .35725 .41252 .33192 .3853 Metetra
.626 .718 .913 r
.33192 .3853 .35725 .41252 .39797 .40437 .37306 .37487 Metetra
.598 .694 .909 r
.37306 .37487 .39797 .40437 .43936 .40022 .41484 .36703 Metetra
.571 .658 .896 r
.41484 .36703 .43936 .40022 .48159 .40036 .4574 .36196 Metetra
.558 .621 .871 r
.4574 .36196 .48159 .40036 .52489 .40341 .5009 .35871 Metetra
.563 .592 .84 r
.5009 .35871 .52489 .40341 .56936 .40654 .54543 .35538 Metetra
.58 .574 .811 r
.54543 .35538 .56936 .40654 .61497 .40676 .59098 .35 Metetra
.598 .566 .789 r
.59098 .35 .61497 .40676 .66158 .40237 .63747 .34145 Metetra
.612 .562 .774 r
.63747 .34145 .66158 .40237 .70902 .3934 .68481 .32977 Metetra
.621 .562 .766 r
.68481 .32977 .70902 .3934 .75722 .38102 .73293 .31575 Metetra
.626 .562 .762 r
.73293 .31575 .75722 .38102 .80617 .3666 .78184 .30028 Metetra
.628 .562 .76 r
.78184 .30028 .80617 .3666 .85591 .35111 .83157 .284 Metetra
.662 .741 .908 r
.18478 .39861 .21199 .42398 .25139 .4106 .22454 .38473 Metetra
.659 .739 .909 r
.22454 .38473 .25139 .4106 .29137 .3975 .26491 .37093 Metetra
.653 .735 .909 r
.26491 .37093 .29137 .3975 .33192 .3853 .30587 .35756 Metetra
.642 .726 .909 r
.30587 .35756 .33192 .3853 .37306 .37487 .34744 .34513 Metetra
.626 .708 .905 r
.34744 .34513 .37306 .37487 .41484 .36703 .38966 .33412 Metetra
.611 .682 .893 r
.38966 .33412 .41484 .36703 .4574 .36196 .43262 .32461 Metetra
.602 .652 .872 r
.43262 .32461 .4574 .36196 .5009 .35871 .47641 .31605 Metetra
.602 .625 .847 r
.47641 .31605 .5009 .35871 .54543 .35538 .52112 .30731 Metetra
.609 .605 .822 r
.52112 .30731 .54543 .35538 .59098 .35 .56675 .29724 Metetra
.617 .593 .803 r
.56675 .29724 .59098 .35 .63747 .34145 .61327 .28519 Metetra
.625 .586 .79 r
.61327 .28519 .63747 .34145 .68481 .32977 .66063 .27121 Metetra
.63 .583 .783 r
.66063 .27121 .68481 .32977 .73293 .31575 .70879 .25575 Metetra
.633 .582 .78 r
.70879 .25575 .73293 .31575 .78184 .30028 .75777 .23932 Metetra
.634 .582 .778 r
.75777 .23932 .78184 .30028 .83157 .284 .80758 .22229 Metetra
.663 .741 .908 r
.1568 .37252 .18478 .39861 .22454 .38473 .19692 .35817 Metetra
.661 .74 .908 r
.19692 .35817 .22454 .38473 .26491 .37093 .23767 .34374 Metetra
.658 .737 .908 r
.23767 .34374 .26491 .37093 .30587 .35756 .27904 .3294 Metetra
.653 .732 .907 r
.27904 .3294 .30587 .35756 .34744 .34513 .32105 .31543 Metetra
.645 .721 .904 r
.32105 .31543 .34744 .34513 .38966 .33412 .36373 .30205 Metetra
.636 .704 .895 r
.36373 .30205 .38966 .33412 .43262 .32461 .40713 .2893 Metetra
.631 .683 .881 r
.40713 .2893 .43262 .32461 .47641 .31605 .45131 .2769 Metetra
.629 .662 .863 r
.45131 .2769 .47641 .31605 .52112 .30731 .49632 .26426 Metetra
.632 .645 .845 r
.49632 .26426 .52112 .30731 .56675 .29724 .54219 .2508 Metetra
.635 .632 .83 r
.54219 .2508 .56675 .29724 .61327 .28519 .5889 .2362 Metetra
.639 .625 .82 r
.5889 .2362 .61327 .28519 .66063 .27121 .63644 .22047 Metetra
.641 .621 .815 r
.63644 .22047 .66063 .27121 .70879 .25575 .68481 .20384 Metetra
.643 .62 .812 r
.68481 .20384 .70879 .25575 .75777 .23932 .73401 .18658 Metetra
.643 .619 .811 r
.73401 .18658 .75777 .23932 .80758 .22229 .78407 .16885 Metetra
.663 .741 .908 r
.12799 .34569 .1568 .37252 .19692 .35817 .16848 .33087 Metetra
.663 .741 .908 r
.16848 .33087 .19692 .35817 .23767 .34374 .20961 .31588 Metetra
.661 .739 .908 r
.20961 .31588 .23767 .34374 .27904 .3294 .2514 .30078 Metetra
.659 .736 .907 r
.2514 .30078 .27904 .3294 .32105 .31543 .29385 .2857 Metetra
.655 .73 .905 r
.29385 .2857 .32105 .31543 .36373 .30205 .33698 .27074 Metetra
.651 .721 .899 r
.33698 .27074 .36373 .30205 .40713 .2893 .38084 .25589 Metetra
.648 .709 .891 r
.38084 .25589 .40713 .2893 .45131 .2769 .42546 .24104 Metetra
.647 .695 .881 r
.42546 .24104 .45131 .2769 .49632 .26426 .47087 .22592 Metetra
.648 .683 .87 r
.47087 .22592 .49632 .26426 .54219 .2508 .5171 .21028 Metetra
.649 .674 .861 r
.5171 .21028 .54219 .2508 .5889 .2362 .56416 .19397 Metetra
.651 .669 .854 r
.56416 .19397 .5889 .2362 .63644 .22047 .61205 .17699 Metetra
.652 .666 .85 r
.61205 .17699 .63644 .22047 .68481 .20384 .66077 .15945 Metetra
.653 .664 .849 r
.66077 .15945 .68481 .20384 .73401 .18658 .71036 .14145 Metetra
.653 .664 .848 r
.71036 .14145 .73401 .18658 .78407 .16885 .76082 .12305 Metetra
.663 .741 .908 r
.09834 .31807 .12799 .34569 .16848 .33087 .1392 .30279 Metetra
.663 .741 .908 r
.1392 .30279 .16848 .33087 .20961 .31588 .18071 .28728 Metetra
.663 .74 .908 r
.18071 .28728 .20961 .31588 .2514 .30078 .2229 .27157 Metetra
.662 .739 .907 r
.2229 .27157 .2514 .30078 .29385 .2857 .26578 .2557 Metetra
.66 .736 .906 r
.26578 .2557 .29385 .2857 .33698 .27074 .30937 .23969 Metetra
.659 .732 .903 r
.30937 .23969 .33698 .27074 .38084 .25589 .35369 .22356 Metetra
.658 .726 .899 r
.35369 .22356 .38084 .25589 .42546 .24104 .39878 .20724 Metetra
.657 .719 .894 r
.39878 .20724 .42546 .24104 .47087 .22592 .44465 .19064 Metetra
.657 .713 .889 r
.44465 .19064 .47087 .22592 .5171 .21028 .49132 .17365 Metetra
.658 .708 .884 r
.49132 .17365 .5171 .21028 .56416 .19397 .53883 .15622 Metetra
.658 .704 .881 r
.53883 .15622 .56416 .19397 .61205 .17699 .58717 .13834 Metetra
.659 .703 .879 r
.58717 .13834 .61205 .17699 .66077 .15945 .63638 .12004 Metetra
.659 .702 .878 r
.63638 .12004 .66077 .15945 .71036 .14145 .68646 .10135 Metetra
.659 .701 .878 r
.68646 .10135 .71036 .14145 .76082 .12305 .73744 .08231 Metetra
.664 .741 .908 r
.0678 .28964 .09834 .31807 .1392 .30279 .10903 .27388 Metetra
.663 .741 .908 r
.10903 .27388 .1392 .30279 .18071 .28728 .15092 .25787 Metetra
.663 .741 .908 r
.15092 .25787 .18071 .28728 .2229 .27157 .19352 .24161 Metetra
.663 .74 .908 r
.19352 .24161 .2229 .27157 .26578 .2557 .23683 .22511 Metetra
.663 .739 .907 r
.23683 .22511 .26578 .2557 .30937 .23969 .28086 .20837 Metetra
.662 .738 .906 r
.28086 .20837 .30937 .23969 .35369 .22356 .32565 .1914 Metetra
.662 .735 .904 r
.32565 .1914 .35369 .22356 .39878 .20724 .37121 .17415 Metetra
.661 .732 .902 r
.37121 .17415 .39878 .20724 .44465 .19064 .41757 .15661 Metetra
.661 .73 .9 r
.41757 .15661 .44465 .19064 .49132 .17365 .46474 .13873 Metetra
.661 .728 .898 r
.46474 .13873 .49132 .17365 .53883 .15622 .51275 .12048 Metetra
.662 .726 .897 r
.51275 .12048 .53883 .15622 .58717 .13834 .56162 .10187 Metetra
.662 .725 .896 r
.56162 .10187 .58717 .13834 .63638 .12004 .61137 .08288 Metetra
.662 .725 .896 r
.61137 .08288 .63638 .12004 .68646 .10135 .66202 .06354 Metetra
.662 .725 .896 r
.66202 .06354 .68646 .10135 .73744 .08231 .7136 .04383 Metetra
.664 .741 .908 r
.03634 .26033 .0678 .28964 .10903 .27388 .07793 .24409 Metetra
.664 .741 .908 r
.07793 .24409 .10903 .27388 .15092 .25787 .12022 .22757 Metetra
.663 .741 .908 r
.12022 .22757 .15092 .25787 .19352 .24161 .16322 .21079 Metetra
.663 .741 .908 r
.16322 .21079 .19352 .24161 .23683 .22511 .20695 .19372 Metetra
.663 .741 .908 r
.20695 .19372 .23683 .22511 .28086 .20837 .25143 .17637 Metetra
.663 .74 .907 r
.25143 .17637 .28086 .20837 .32565 .1914 .29669 .15874 Metetra
.663 .739 .907 r
.29669 .15874 .32565 .1914 .37121 .17415 .34273 .1408 Metetra
.663 .738 .906 r
.34273 .1408 .37121 .17415 .41757 .15661 .38959 .12254 Metetra
.663 .737 .905 r
.38959 .12254 .41757 .15661 .46474 .13873 .43728 .10396 Metetra
.663 .737 .905 r
.43728 .10396 .46474 .13873 .51275 .12048 .48583 .08502 Metetra
.663 .736 .904 r
.48583 .08502 .51275 .12048 .56162 .10187 .53526 .06573 Metetra
.663 .736 .904 r
.53526 .06573 .56162 .10187 .61137 .08288 .58559 .04609 Metetra
.663 .736 .904 r
.58559 .04609 .61137 .08288 .66202 .06354 .63685 .02607 Metetra
.663 .736 .904 r
.63685 .02607 .66202 .06354 .7136 .04383 .68907 .00568 Metetra
0 g
.25 Mabswid
.68874 0 m
.96935 .42924 L
s
.96935 .42924 m
1 .6535 L
s
1 .6535 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.03716 .25514 m
0 .48963 L
s
0 .48963 m
.70298 .24544 L
s
.70298 .24544 m
.68874 0 L
s
.68874 0 m
.03716 .25514 L
s
.03716 .25514 m
.68874 0 L
s
.13514 .21677 m
.13978 .22141 L
s
[(-2)] .12586 .2075 1 .99877 Mshowa
.34306 .13535 m
.34735 .14032 L
s
[(0)] .3345 .12542 .86223 1 Mshowa
.56863 .04703 m
.57247 .05235 L
s
[(2)] .56094 .0364 .72324 1 Mshowa
.125 Mabswid
.18558 .19702 m
.18832 .19985 L
s
.23703 .17687 m
.23971 .17975 L
s
.28951 .15632 m
.29214 .15925 L
s
.39772 .11395 m
.40023 .11699 L
s
.4535 .09211 m
.45595 .0952 L
s
.51046 .06981 m
.51284 .07294 L
s
.08568 .23614 m
.08851 .23887 L
s
.62804 .02377 m
.63027 .02701 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  ImageSize->{215, 174.188},
  ImageMargins->{{18, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003G0000[Q000`40O003h00Oegoo003GOol00=Mo
o`00egoo003GOol00=Moo`00egoo003GOol00=Moo`00egoo003GOol007ioo`@001Moo`8003aoo`00
Ogoo00<007ooOol057oo0`0000=oo`000000>goo0020Ool00`00Oomoo`0AOol200000goo00000003
000kOol0085oo`03001oogoo00eoo`<00003Ool0000000=Fo003001Fo00003Yoo`00OWoo00@007oo
Ool000aoo`<00004Ool000000005E_`00`00E_`0000jOol007moo`8000]oo`800004Ool000000008
E_`01000E_aFo000>Goo0029Ool300001Woo00000000E_`000QFo005001Fo5KlE_`0000hOol008Mo
o`80009oo`8000EFo003001Fo5Kl00EFo005001Fo5KlE_`0000hOol008Aoo`<00004Ool000000007
E_`00`00E_aFo005E_`01000E_aFo5Kl0`00=Woo0022Ool300000goo0000000;E_`00`00E_aFo005
E_`300000eKl0000000fOol007moo`<00006Ool00000001Fo00035Kl00<005KlE_`00UKl0`0015Kl
00<007ooOol0=7oo001mOol20002Ool20005E_`00`00E_aFo009E_`00`00E_`000020002E_`00`00
E_aFo003E_`00`00Oomoo`0cOol007Yoo`<00004Ool000000008E_`00`00E_aFo008E_`20005E_`0
0`00E_aFo003E_`2000;Ool4000UOol007Moo`<0009oo`8000aFo003001Fo5Kl00EFo0800003E_`0
05Kl00AFo003001Fo5Kl00AFo003001oogoo00Yoo`03001oogoo02Eoo`00MGoo0P000Woo0`0000=F
o000E_`02eKl00<005KlE_`00UKl0`0015Kl00<005KlE_`00UKl00<005KlE_`015Kl0P0017oo1@00
0goo00<007ooOol097oo001bOol3000017oo000000001EKl00<005KlE_`02UKl00@005Kl000000QF
o005001Fo5KlE_`00007E_`2000<Ool00`00Oomoo`0SOol0071oo`800004Ool000000009E_`00`00
E_aFo008E_`3000;E_`01000E_aFo0001UKl100027oo00@007ooOol002Eoo`00E7oo0P005goo0`00
00Eoo`00001Fo00000]Fo003001Fo5Kl00AFo0<0009Fo003001Fo5Kl00UFo004001Fo5Kl0003E_`3
0002E]`20009Ool2000VOol005=oo`04001oogoo000DOol2000017oo0000000015Kl00<005KlE_`0
2EKl00<005KlE_`00`001UKl00<005KlE_`02EKl00<005Kl00000`001UKL0P00<7oo001COol01000
Oomoo`004Goo0`0000=oo`00000025Kl00<005KlE_`025Kl0`002UKl00<005KlE_`025Kl0`002EKL
0P00<7oo001COol01000Oomoo`003Woo0`000Woo0P002eKl00<005KlE_`01EKl0P0000=Fo000E_`0
2eKl00<005KlE_`01EKl0P0000=Fg00000002UKL0P00;goo001COol01000Oomoo`0037oo0P000Woo
0`0000=Fo000E_`02UKl00<005KlE_`00UKl0`0015Kl00<005KlE_`02EKl00<005KlE_`00UKl0`00
15KL00<005KLE]`02EKL0P00;Woo001DOol2000:Ool70005E_`00`00E_aFo009E_`01000E_`00000
25Kl00<005KlE_`02EKl00@005Kl000000MFg08000YFg08002ioo`00GWoo0P0000Aoo`00000000UF
o003001Fo5Kl00QFo08000]Fo003001Fo5Kl00MFo0<000UFg08000YFg0<002eoo`00Fgoo0`0000Eo
o`00001Fo00000]Fo003001Fo5Kl00AFo0<00003E_`005Kl00YFo003001Fo5Kl00AFo0<0009Fg003
001Fg5KL00QFg08000MFg0800004D[/00000000/Ool005Uoo`800004Ool000000004E_`00`00E_aF
o009E_`00`00E_aFo0030005E_`00`00E_aFo009E_`00`00E_aFo0030006E]`00`00E]aFg007E]`2
0004E]`30004D[/2000/Ool005Ioo`H000QFo003001Fo5Kl00QFo0<000UFo003001Fo5Kl00QFo0<0
00UFg003001Fg5KL00MFg004001Fg000E]`20007D[/00`00Ool0000[Ool005Aoo`80009oo`<000YF
o003001Fo5Kl00EFo0800003E_`005Kl00YFo003001Fo5Kl00EFo0800003E]`005KL00YFg003001F
g5KL00IFg0@000YB^`03001oo`0002Yoo`00DGoo0`0000Ioo`00000005Km000;E_`00`00E_aFo002
E_`30004E_`00`00E_aFo008E_`00`00E_aFo002E_`30004E]`00`00E]aFg009E]`00`00E]aFg003
E]`30002D[/00`00D[]B^`09D[/2000ZOol004ioo`<00004Ool000000005E_d00`00E_aFo009E_`0
1000E_`0000025Kl00<005KlE_`025Kl00@005Kl000000QFg003001Fg5KL00QFg003001Fg5KL00<0
009B^`05001B^e:kD[/0000:D[/00`00Ool0000YOol004aoo`800004Ool000000009E_d00`00E_aF
o008E_`2000;E_`00`00E_aFo007E_`2000:E]`00`00E]aFg008E]`30005D[/01@00D[]B^e:k0000
2e:k00<007oo0000:7oo0019Ool60002E_d00`00E_eFo@08E_d00`00E_aFo004E_`300000eKl001F
o00:E_`00`00E_aFo004E_`300000eKL001Fg00:E]`00`00E]aFg005E]`200000e:k001B^`05D[/0
0`00D[]B^`02D[/00`00D[]B^`08D[/01000Ool000009goo000[Ool4000HOol50006E_d00`00E_eF
o@08E_d00`00E_aFo0030005E_`00`00E_aFo009E_`00`00E_aFo0030005E]`00`00E]aFg008E]`0
0`00E]aFg002E]`30004D[/00`00D[]B^`03D[/00`00D[]B^`02D[/00`00D[]B^`07D[/300000goo
001oo`0VOol002aoo`03001oogoo01Eoo`D000YFo@03001FoEKm00MFo@<000UFo003001Fo5Kl00QF
o0<000UFg003001Fg5KL00QFg004001Fg0000007D[/00`00D[]B^`03D[/00`00D[]B^`03D[/00`00
D[]B^`03D[/30002DWT01000Oomoo`009Woo000UOol50003Ool00`00Oomoo`0BOol200001Goo0000
05Km00002eKm00<005KmE_d00eKm0`000UKl00<005KlE_`025Kl00<005KlE_`015Kl0`0000=Fg000
E]`02UKL00<005KLE]`01eKL0P002U:k00<005:kD[/00U:k00<005:kD[/00e:k00<005:kD[/00`00
1U9i00<007oo00009Woo000^Ool00`00Oomoo`0>Ool60004E_d00`00E_eFo@09E_d00`00E_d00002
0006E_`00`00E_aFo008E_`00`00E_aFo0030005E]`00`00E]aFg008E]`00`00E]aFg004E]`30000
0e:k001B^`0:D[/01@00D[]B^e:k00001E:k0`002E9i00@007ooOol002Eoo`00:goo00@007ooOol0
00eoo`<00003Ool0000000QFo@03001FoEKm00QFo@8000YFo003001Fo5Kl00MFo0<000UFg003001F
g5KL00QFg003001Fg5KL00<000EB^`03001B^e:k00QB^`05001B^e:kD[/00003D[/200000e9i001B
N@09DWT01@00Oomoogoo000097oo000/Ool2000<Ool20002Ool2000;E_d00`00E_eFo@05E_d20000
0eKl001Fo00:E_`00`00E_aFo003E_`300000eKl001Fg00:E]`00`00E]aFg007E]`30009D[/00`00
D[]B^`08D[/00`00D[]B^`040004DWT00`00DWUBN@08DWT01000Oomoo`0097oo000gOol3000017oo
000000000UKm00<005KmE_d025Km00<005KmE_d00UKm0`0015Kl00<005KlE_`02EKl00<005Kl0000
0P001EKl00<005KLE]`025KL00<005KLE]`015KL0`0000=B^`00D[/02E:k00<005:kD[/025:k00@0
05:k000000MBN@03001BNE9i00QBN@05001oogooOol0000SOol003Eoo`800004Ool000000006E_d0
0`00E_eFo@08E_d01000E_d0000025Kl00<005KlE_`025Kl0P002EKl00<005KLE]`025KL00<005KL
E]`00`001E:k00<005:kD[/025:k00<005:kD[/01e:k0P0000=BN@00DWT01e9i00<0059iDWT0259i
00<007ooOol00P008Woo000bOol300000goo0000000:E_d00`00E_eFo@07E_d2000;E_`00`00E_aF
o004E_`300000eKl001Fo009E_`00`00E]aFg007E]`30009D[/00`00D[]B^`07D[/00`00D[]B^`04
D[/300001E9j001BNE9i00001e9i00<0059iDWT0259i00D007ooOomoo`00029oo`00<7oo1P000UKm
00<005KmE_d025Km00<005KmE_d00eKm0`000UKm00<005KlE_`02EKl00<005KlE_`00`001EKl00<0
05KlE_`01eKl00<005KLE]`015KL0`0000=B^`00D[/02E:k00<005:kD[/025:k00<005:kD[/00`00
1E9j00<0059i0000259i00<0059iDWT01e9i00<007ooOol00Woo00<007ooOol017oo0P006Goo000]
Ool60006E_d00`00E_eFo@08E_d00`00E_d000020006E_d00`00E_aFo008E_`30009E_`00`00E_aF
o007E_`00`00E]aFg0030005D[/00`00D[]B^`08D[/00`00D[]B^`07D[/30008DWX00`00DWT00008
DWT00`00DWUBN@07DWT20004Ool00`00Oomoo`02Ool01000Oomoo`0067oo000[Ool200000goo0000
000:E_d00`00E_eFo@07E_d2000:E_d00`00E_aFo005E_`200000eKl001Fo00:E_`00`00E_aFo006
E_`30009D[/00`00D[]B^`07D[/00`00D[]B^`04D[/300000e9j001BNP08DWX00`00DWT00009DWT0
0`00DWUBN@03DWT300000e9H001oo`03Ool00`00Oomoo`02Ool01000Oomoo`0067oo000XOol60002
E_d00`00E_eFo@08E_d00`00E_eFo@03E_d300000eKm001Fo@0:E_d00`00E_aFo002E_`20004E_`0
0`00E_aFo008E_`00`00E_aFo003E_`300000e;L001B^`09D[/00`00D[]B^`08D[/00`00D[]B^`03
0005DWX00`00DWYBNP07DWX20009DWT00`00DWUBN@030005DUP00`00Oomoo`02Ool01@00Oomoogoo
00000Woo00<007ooOol05Woo000UOol60006E_d00`00E_eFo@08E_d00`00E_d000020005E_d00`00
E_eFo@09E_d01000E_`000001eKl00<005KlE_`025Kl00<005Kl00000P001E;L00<005:kD[/025:k
00<005:kD[/01e:k0`00259j00<0059jDWX01e9j0P002E9i0`00259H00<007ooOol00goo00@007oo
Ool0009oo`03001oogoo01Ioo`008goo0P0000=oo`0000002UKm00<005KmE_d01eKm0P002EKm00<0
05KmE_d025Km0P002UKl00<005KlE_`01eKl0P002E;L00<005:kD[/01e:k00<005:kD[/015:k0`00
00=BVP00DWX02E9j00<0059jDWX01e9j00<0059iDWT0159i0`0000=BF000DUP0259H00<007ooOol0
0Woo0P000goo0P006Goo000POol60002E_d00`00E_eFo@08E_d00`00E_eFo@03E_d300000eKm001F
o@09E_d00`00E_eFo@04E_d30002E_`00`00E_aFo007E_`00`00E_aFo004E_`300000e;L001Bg009
D]`00`00D[]B^`07D[/00`00D[]B^`030005DYX00`00DWYBNP07DWX00`00DWYBNP06DWX20004DWT2
0005DUP00`00DUQBF007DUP00`00Oomoo`03Ool00`00Oomoo`0KOol001moo`@000IFo@03001FoEKm
00QFo@03001Fo@00008000EFo@03001FoEKm00QFo@03001FoEKm00<000IFo003001Fo5Kl00MFo003
001Fo5Kl00<000EBg003001Bg5;L00MBg003001B^e:k00IB^`@000QBVP03001BNU9j00MBNP03001B
NU9j00=BNP<00003DWX0059i00<000MBF003001BF59H00MBF003001oogoo00Aoo`03001oogoo01Yo
o`007goo00<005KmE_d025Km00<005KmE_d01eKm0P002EKm00<005KmE_d01eKm0`002UKl00<005Kl
E_`01EKl100025;L00<005;LD]`025;L00<005:kD[/01@000U:K00<005:JDYX025:J00<0059jDWX0
1e9j00<0059j00000P000e9j0`002U9H00<0059HDUP0259H00<007ooOol00goo00<007ooOol06Woo
000HOol20005Ool2000:E_d00`00E_eFo@03E_d300000eKm001Fo@09E_d00`00E_eFo@03E_d30003
E_d00`00E_aFo008E_`00`00E_aFo0040002D_`00`00D]aBg008D]`00`00D]aBg007D]`30008DY/0
0`00DYYBVP07DYX00`00DWYBNP06DWX300000e9j000000030002DUT2000:DUP00`00DUQBF007DUP0
0`00Oomoo`04Ool00`00Oomoo`0IOol001Moo`04001oogoo0003Ool200000goo001Fo@09E_d00`00
E_d000020005E_d00`00E_eFo@08E_d00`00E_d000020007E_d00`00E_aFo007E_`30007D_`00`00
D]aBg008D]`00`00D]aBg003D]`300000e:k001BV`08DY/00`00DYYBVP07DYX00`00DWYBNP03DWX3
0000159j00000000259I0P002E9H00<0059HDUP0259H00<007ooOol017oo00<007ooOol067oo000G
Ool01000Oomoo`000goo00D007ooOomoo`0000UFo@8000UFo@03001FoEKm00MFo@<000YFo@03001F
o5Kl00=Fo0<00003D_`005;l00QBo003001Bg5;L00MBg003001Bg5;L00<000EB^`03001BVe:K00MB
V`03001BVU:J00MBVP03001BNP0000800006DWX00000001>F@002E9I0P002E9H00<0059HDUP0259H
00<007ooOol017oo00<007ooOol067oo000GOol01000Oomoo`000goo0P000goo00<005KmE_d00eKm
0`000UKm00<005KmE_d01eKm00<005KmE_d00eKm0`000eKm00<005KmE_d025Km00<005Kl00000P00
1E;l00<005;lD_`01e;l00<005;LD]`01E;L10002E:k00<005:KDY/01U:K00<005:JDYX01U:J0`00
00ABNP00000000E>F@03001BFE9I00MBF@8000UBF003001BF59H00MBF003001oogoo00Eoo`03001o
ogoo01Moo`005goo00@007ooOol000=oo`03001oogoo00=oo`03001Fo@00008000IFo@03001FoEKm
00MFo@03001Fo@00008000MFo@03001FoEKm00MFo@8000UBo003001Bo5;l00MBo003001Bg5;L00@0
009B_003001B^e:k00QB^`03001BVe:K00MBV`03001BVU:J009BVPL000Q>F@03001BFE9I00MBF@03
001BF00000QBF003001BF59H00IBF0<000Moo`03001oogoo01Ioo`0067oo0P0017oo0P001Goo0P00
2EKm00<005KmE_d01EKm10002UKm00<005KmE_d00eKm0`0000=Fo@00D_`02E;l00<005;lD_`01E;l
10001e:l00<005:kD[/025:k00<005:KDY/01U:K00<005:J00001@000TiI00<004iICUT01diI00<0
059IDUT0259I00<0059H00001e9H00<0059HDUP00e9H0`000U8h00<007ooOol01Goo00<007ooOol0
5Woo000NOol00`00Oomoo`06Ool00`00E_eFo@07E_d00`00E_eFo@040004E_d00`00E_eFo@08E_d0
0`00E_d000020005E_d00`00D_aBo008D_`00`00D_aBo0040002D]`00`00D[aB_007D[`00`00D[]B
^`07D[/00`00DY]BV`05DY/40008CUT00`00CUU>F@07CUT00`00DUUBF@07DUT00`00DUP00008DUP0
0`00DUP000020005DSP00`00Oomoo`06Ool00`00Oomoo`0EOol001ioo`03001oogoo00Moo`03001F
oEKm00IFo@<000UFo@03001FoEKm00MFo@8000UFo@03001Bo5;l00MBo0<000MBg003001B_5:l00MB
_003001B^e:k00MB^`03001BVe:K00D00003CWX004iI00Q>F@03001>FDiI00M>F@03001BFE9I00MB
F@04001BF59H0006DUP30009DSP00`00Oomoo`06Ool00`00Oomoo`0DOol001ioo`03001oogoo00Qo
o`03001FoEKm009Fo@<0009Fo@03001FoEKm00QFo@03001FoEKm00=Fo@<00003E_d005Km00UFo@03
001Bo5;l00=Bo0<00003D_`005;L00QBg003001B_5:l00IB_003001B^e:k00IB^`H000E>NP03001>
FDiI00M>F@03001>FDiI00M>F@03001BFE9I00MBF@04001BF59H0002DUP300000e8h001B>009DSP0
0`00Oomoo`06Ool00`00Oomoo`0DOol001eoo`8000]oo`04001Fo@000006E_d00`00E_eFo@08E_d0
0`00E_d000020005E_d00`00E_eFo@08E_d00`00D_`000020005D_`00`00D]aBg007D]`00`00D[aB
_006D[`00`00D[]B^`080008CWX00`00CUU>F@07CUT00`00CUU>F@07CUT00`00DUUBF@07DUT00`00
DUQBF0030005DSP00`00DSQB>007DSP00`00Oomoo`07Ool00`00Oomoo`0COol000Eoo`8000Aoo`80
00=oo`@0009oo`<000Aoo`03001oogoo00]oo`03001FoEKm00MFo@03001FoEKm00MFo@8000UFo@03
001FoEKm00MFo@8000UBo003001Bg5;L00IBg003001B_5:l00EB_0@0009>N`<000=>N`03001>NTij
00I>NP03001>FDiI00M>F@03001>FDiI00M>F@03001BFE9I00IBF@<00003DSP0058h00EB>003001B
>58h00QB>003001oogoo00Moo`03001oogoo019oo`0017oo00@007ooOol000=oo`8000Aoo`03001o
ogoo00Eoo`04001oogooOol2000<Ool00`00E_eFo@08E_d00`00E_eFo@03E_d30002E_d00`00E_eF
o@07E_d00`00E_eFo@03E_d30002D_d00`00D_aBo007D_`00`00D]aBg006D]`00`00D[aB_0040002
CY/40006CW/00`00CWY>NP06CWX00`00CUU>F@07CUT00`00CUU>F@07CUT00`00DUUBF@03DUT30000
0dhh001B>002DSP00`00DSQB>003DSP00`00DSQB>008DSP00`00Oomoo`07Ool00`00Oomoo`0BOol0
00Aoo`04001oogoo000:Ool00`00Oomoo`04Ool01@00Oomoogoo00003Woo00<005KmE_d025Km00<0
05Km00000P001UKm00<005KmE_d01eKm00<005Km00000P001U;m00<005;lD_`01e;l00<005;LD]`0
0U;L1P0014jK10002Dik00<004ijCWX01Tij00<004iICUT01TiI00<004iICUT01diI00<0059IDUT0
0`001Dhh00D0058hDSQB>00000EB>003001B>58h00MB>003001oogoo00Qoo`03001oogoo015oo`00
17oo00@007ooOol000]oo`03001oogoo00<000Aoo`03001oogoo00eoo`03001FoEKm00MFo@8000YF
o@03001FoEKm00EFo@<000YBo@03001Bo5;l00EBo0H000A>_003001>VdjK00<000A>V`03001>Ndik
00I>N`03001>NTij00I>NP03001>FDiI00M>F@03001>FDiI00E>F@@000Q>>005001B>58hDSP00005
DSP00`00DSQB>008DSP00`00Oomoo`08Ool00`00Oomoo`0@Ool000Aoo`04001oogoo0008Ool01000
Oomoo`000Woo00<007ooOol017oo00<007ooOol03Woo00<005KmE_d00eKm0`000UKm00<005KmE_d0
25Km00<005KmE_d010000eKm00<005;mD_d025;m00<005;lD_`010000U;L00<004jlC[`01Djl0`00
1djK00<004ikCW/01Tik00<004ijCWX01dij00<004iICUT01TiI00<004iICUT01@000Thh00<004hh
CSP01dhh00<0058hDSP00U8h00<0058hDSP00e8h00<0058hDSP01e8h00<007ooOol027oo00<007oo
Ool047oo0005Ool2000:Ool20003Ool40002Ool2000AOol00`00E_d000020006E_d00`00E_eFo@07
E_d30008E_d00`00D_eBo@07D_d30007D]`00`00C[a>_002C[`20002C[`00`00CY]>V`06CY/00`00
CW]>N`06CW/00`00CWY>NP06CWX00`00CUU>F@03CUT60008CSP00`00CSQ>>007CSP00`00DSQB>002
DSP00`00DSQB>002DSP00`00DSQB>007DSP00`00Oomoo`09Ool00`00Oomoo`0?Ool001aoo`03001o
ogoo015oo`8000UFo@03001FoEKm00=Fo@<0009Fo@03001FoEKm00MFo@03001BoE;m00=Bo@<00003
D_d005;L00QBg0@000E>_003001>VdjK00E>V`03001>Ndik00I>N`03001>NTij00=>NPT000A>F@03
001>>4hh00M>>003001>>4hh00M>>003001B>58h00=B>005001B>58hDSP0000:DSP00`00Oomoo`09
Ool00`00Oomoo`03Ool40007Ool001aoo`03001oogoo01=oo`03001FoEKm00MFo@03001Fo@000080
00IFo@03001FoEKm00MFo@03001Bo@00008000EBo@03001Bg5;L00ABg0@000M>_003001>VdjK00I>
V`03001>Ndik009>N`T000A:F@03001>FDiI00M>F@03001>>4hh00I>>003001>>4hh00M>>003001B
>58h00=B>003001B>58h009B>003001B>58h00IB>08000]oo`03001oogoo00Aoo`03001oogoo00Mo
o`0077oo0P005Goo00<005KmE_d01UKm0P002UKm00<005KmE_d01EKm0`002E;m00<005;LD]`00`00
0e;L00<004jlC[`01Tjl00<004jKCY/00TjK200014YJ00<004YIBUT01dYI00<004iICUT01TiI00<0
04hhCSP01dhh00<004hhCSP01dhh00<0058hDSP00e8h00D0058hDSQB>00000EB>0<00003DSP007oo
00]oo`03001oogoo00Aoo`03001oogoo00Ioo`0077oo0P005Woo00<005KmE_d00UKm0`000UKm00<0
05KmE_d025Km00<005KmE_d010000e;m00<005;mD_d01U;m0`001e;L00<004jlC[`00Tjl200014Ik
00<004YJBUX01TYJ00<004YIBUT01TYI00<004iICUT01TiI00<004hhCSP01dhh00<004hhCSP01dhh
00<0058hDSP0158h00@0058hDSP0009B>0<000EB>003001oogoo00Yoo`03001oogoo00Aoo`03001o
ogoo00Eoo`0077oo00<007ooOol05Woo00@005Km000000IFo@03001FoEKm00MFo@<000QBo@03001B
oE;m009Bo@<0009Bo@03001Bg5;L009Bg0P000A:_0030016NdIk00I6N`03001:FTYJ00E:FP03001:
FDYI00M:F@03001>FDiI00I>F@03001>>4hh00I>>003001>>4hh00M>>003001B>58h00EB>0D000QB
>003001oogoo00Yoo`05001oogooOol00002Ool00`00Oomoo`05Ool000Uoo`8000Aoo`8000=oo`<0
00Aoo`03001oogoo01Qoo`03001FoEKm00MFo@03001FoEKm00=Fo@<0009Fo@03001BoE;m00MBo@@0
00ABo@H000A>g@03001:_4Zl00I:_0030016NdIk00E6N`03001:FTYJ00I:FP03001:FDYI00I:F@03
001>FDiI00I>F@03001>>4hh00M>>003001>>4hh00M>>003001B>58h00D0009>>003001B>58h00MB
>003001oogoo00Uoo`03001oo`0000=oo`8000Qoo`0027oo00@007ooOol000=oo`8000Ioo`04001o
ogooOol2000JOol00`00E_eFo@07E_d00`00E_d000020006E_d00`00D_eBo@04D_d40002D_d40002
D]d00`00C]e>g@06C]d00`00B[a:_005B[`00`00AW]6N`06AW/00`00BUY:FP05BUX00`00BUU:F@07
BUT00`00CUU>F@05CUT00`00CSQ>>007CSP00`00CSQ>>006CSP40005CSP01000CSQ>>0002E8h00<0
07ooOol02goo00<007ooOol02Woo0008Ool01000Oomoo`002goo00D007ooOomoo`0001aoo`03001F
oEKm00IFo@<000UFo@03001BoE;m00<0009Bo@@000MBg@03001>gDkM00I>g@03001:_4Zl00E:_003
0016NdIk00E6N`03001:FTYJ00I:FP03001:FDYI00I:F@03001>FDiI00I>F@03001>>4hh00I>>003
001>>4hh00=>>0<00003CSP004hh00M>>003001>>00000UB>003001oogoo00]oo`03001oogoo00Yo
o`0027oo00@007ooOol000Qoo`<000Aoo`8001aoo`03001FoEKm009Fo@<000=Fo@03001FoEKm00IF
o@<00003D_d00000008000=Bo@03001BgE;M00IBg@03001>gDkM00E>g@03001:_4Zl00I:_0030016
NdIk00A6N`03001:FTYJ00I:FP03001:FDYI00I:F@03001>FDiI00I>F@03001>>4hh00M>>003001>
>000008000E>>003001>>4hh00I>>08000YB>003001oogoo00]oo`03001oogoo00Uoo`0027oo00@0
07ooOol000Qoo`03001oogoo00Aoo`03001oogoo01aoo`04001Fo@000007E_d00`00E_eFo@03E_d2
000015Km0000000025;m00<005;MD]d01E;M00<004kMC]d01TkM00<004ZlB[`01DZl00<004IkAW/0
1DIk00<004YJBUX01TYJ00<004YIBUT01TYI00<004iICUT01DiI00<004hhCSP00dhh1P0024hh00<0
04hhCSP01Thh00<004hh00002E8h00<007ooOol037oo00<007ooOol027oo0009Ool20009Ool40003
Ool00`00Oomoo`0MOol20009E_d00`00E_d000050002D_d00`00D_eBo@07D_d00`00D]eBg@05D]d0
0`00C]e>g@02C]dA0002AW/00`00BUY:FP06BUX00`00BUU:F@06BUT00`00CUU>F@02CUT90004CSP0
0`00CSQ>>007CSP00`00CSQ>>007CSP2000:DSP00`00Oomoo`0:Ool2000:Ool001]oo`03001oogoo
01moo`03001FoEKm00EFo@<00003E_d0000000IBo@03001BoE;m00MBo@03001BgE;M009Bg@L000A6
_@030012W4:L00E2W003000nG3iL008nG1T000A:>@03001>>4hh00M>>003001>>4hh00M>>003001>
>4hh00M>>003001B>58h00QB>003001oogoo00aoo`03001oogoo00Moo`006Woo0P008Woo00@005Km
E_eFo@<00004E_d000000009D_d00`00D_eBo@05D_d60004C]d00`00A[e6_@05A[d00`00@Ya2W005
@Y`00`00?U`nG004?U`00`00@U]2F`06@U/00`00ASY6>P06ASX00`00BSU:>@06BST00`00CSQ>>006
CSP00`00CSQ>>007CSP00`00CSQ>>008CSP00`00DSQB>007DSP00`00Oomoo`0=Ool00`00Oomoo`06
Ool001Yoo`03001oogoo029oo`L000=Fo@03001BoE;m00MBo@03001BoE;m00@0009Bo@03001>gDkM
00I>g@030016_DJm00E6_@030012W4:L00E2W003000nG3iL00@nG0030012Fd9K00I2F`030016>THj
00I6>P03001:>DXi00E:>@03001>>4hh00I>>003001>>4hh00Q>>003001>>4hh00M>>003001B>58h
00IB>0<000ioo`03001oogoo00Ioo`006Woo0P0087oo0`0000=oo`00000025Km00<005;mD_d01U;m
0`001e;m00<004kMC]d01TkM00<004JmA[d01DJm00<004:L@Y`01D:L00<003iL?U`013iL00<0049K
@U/01D9K00<004HjASX01THj00<004XiBST01DXi00<004hhCSP01dhh00<004hhCSP01dhh00<004hh
CSP01dhh0P00158h0`000U9H00<007ooOol03Goo00<007ooOol01Goo0002Ool20004Ool20004Ool0
0`00Oomoo`02Ool30004Ool00`00Oomoo`0LOol30005Ool00`00E_eFo@07E_d00`00D_eBo@02D_d3
0002D_d00`00D_eBo@06D_d00`00C]e>g@06C]d00`00A[e6_@05A[d00`00@Ya2W004@Y`00`00?U`n
G004?U`00`00@U]2F`05@U/00`00ASY6>P06ASX00`00BSU:>@06BST00`00CSQ>>006CSP00`00CSQ>
>007CSP00`00CSQ>>008CSP50006DUP00`00Oomoo`0;Ool30006Ool00005Ool007ooOol00003Ool2
0004Ool00`00Oomoo`05Ool01@00Oomoogoo00006goo0`002Goo00<005KmE_d01eKm00@005;m0000
00IBo@03001BoE;m00IBo@03001>gDkM009>gA400092W003000nG3iL00@nG0030012Fd9K00E2F`03
0016>THj00I6>P03001:>DXi00E:>@03001>>4hh00M>>003001>>4hh00M>>003001>>4hh00I>>0@0
00QBF003001oogoo00Qoo`<0009oo`03001oogoo00Aoo`0000Eoo`00Oomoo`0000Yoo`03001oogoo
00Aoo`03001oogoo008001Qoo`<000eoo`03001FoEKm00EFo@<000UBo@03001BoE;m00ABo@H000A:
g@030012_D:m00E2_@03000jOCYm008jO@H000<nG0030012Fd9K00E2F`030016>THj00I6>P03001:
>DXi00I:>@03001>>4hh00I>>003001>>4hh00M>>003001>>4hh00=>>0<00004DUP0059H0008DUP0
0`00Oomoo`05Ool30005Ool00`00Oomoo`04Ool00005Ool007ooOol0000:Ool01000Oomoogoo0`00
0goo00<007ooOol05Goo0P004Goo00<005KmE_d010000eKm00<005;mD_d01e;m00<005;mD_d00`00
0e;m00<004[MB]d01D[M00<004:m@[d01D:m00<003Ym>Wd013Ym00<003Hl=S`01@001d9K00<004Hj
ASX01DHj00<004XiBST01TXi00<004hhCSP01dhh00<004hhCSP01Thh00<004hhCSP00`001E9H00<0
059H0000259H00@007ooOomoo`<000Qoo`03001oogoo00Aoo`0000Eoo`00Oomoo`0000]oo`04001o
ogoo0005Ool2000COol3000DOol20008E_d00`00D_eBo@05D_d40007D_d00`00B]e:g@05B]d00`00
@[e2_@05@[d00`00>WdjO@04>Wd00`00=S`f?003=S`40004@U/00`00ASY6>P06ASX00`00BSU:>@06
BST00`00CSQ>>006CSP00`00CSQ>>005CSP40008DUP01000DUQBF0001e9H10002goo00<007ooOol0
17oo0002Ool20009Ool40002Ool40002Ool00`00Oomoo`0?Ool3000HOol00`00E_eFo@07E_d00`00
D_eBo@040002D_d00`00D_eBo@07D_d00`00B]e:g@05B]d00`00@[e2_@04@[d00`00>WdjO@04>Wd0
0`00=S`f?004=S`00`00>S/j>`050008ASX00`00BSU:>@06BST00`00CSQ>>006CSP00`00CSQ>>005
0002CUT00`00DUQBF008DUP01000DUQBF0001U9H00<007ooOol03Goo00<007ooOol00goo000IOol0
0`00Oomoo`0<Ool3000LOol00`00E_eFo@06E_d30007D_d00`00D_eBo@07D_d00`00B]e:g@02B]d8
0002@[d00`00>WdjO@04>Wd00`00=S`f?004=S`00`00>S/j>`03>S/70002ASX00`00BSU:>@06BST0
0`00CSQ>>003CSP60008CUT00`00DUQBF007DUP01000DUQBF0001e9H00<007ooOol037oo00<007oo
Ool00goo000IOol00`00Oomoo`09Ool3000POol00`00E_eFo@02E_d30002E_d00`00D_eBo@06D_d0
0`00D_eBo@05D_d60004A]d01000>YhjWSZN1P0013Ym00<003Hl=S`00cHl00<003Xk>S/013Xk00<0
03hj?SX00chj4@0014YI00<004iICUT01diI00<0059HDUP0259H00@0059HDUP000IBF003001oogoo
00aoo`03001oogoo00=oo`006Goo00<007ooOol01Woo0`0097oo00@005Km000000IFo@03001BoE;m
00IBo@03001BoE;m00@0009>o@030016gDKM00I6g@03000jWSZN00<jWP03000^GBiM00D000Hf?003
000j>cXk00@j>`03000n>Shj00Dn>P030016>THj00I6>P03001:FDYI00M:F@03001>FDiI00I>F@03
001BF59H00QBF005001BF59HDUP00005DUP00`00Oomoo`0<Ool00`00Oomoo`03Ool001Qoo`8000Eo
o`<002Qoo`8000QFo@03001BoE;m00ABo@@000M>o@030016gDKM00I6g@03000jWSZN00<jWP03000^
GBiM00<^G@@000@f?003000j>cXk00@j>`03000n>Shj00Dn>P030016>THj00I6>P03001:FDYI00I:
F@03001>FDiI00M>F@03001BF59H00MBF005001BF59HDUP00006DUP00`00Oomoo`0;Ool00`00Oomo
o`03Ool001Qoo`80009oo`<002eoo`03001FoEKm00IFo@03001BoE;m00<000ABo@03001>oDkm00E>
o@030016gDKM00I6g@03000jWSZN00<jWP03000^GBiM00<^G@03000V72HL00D000Hj>`03000n>Shj
00Dn>P030016>THj00I6>P03001:FDYI00I:F@03001>FDiI00M>F@03001BF59H00QBF005001BF59H
DUP00003DUP3000=Ool00`00Oomoo`03Ool0015oo`<000Aoo`@0035oo`03001FoEKm00AFo@@000QB
o@03001>oDkm00E>o@030016gDKM0096g@T00003>Yh002iM00@^G@03000V72HL00<V70@000@j>`03
000n>Shj00Dn>P030016>THj00E6>P03001:FDYI00M:F@03001>FDiI00I>F@03001BF59H00QBF003
001BF59H00D0009BNP03001oogoo00]oo`03001oogoo00=oo`004Woo00<007ooOol00goo0P00=7oo
00<005KmE_d00`0015Km00<005;mD_d01U;m00<004kmC_d00Tkm1`0014KN00<003JN=Yh00SJN1000
12iM00<002HL9Q`00bHL00<002XL:Q`00`0000<j>`00?SX01Shj00<004HjASX01THj00<004YIBUT0
1TYI00<004iICUT01diI00<0059HDUP01U9H10000e9j00<0059jDWX00U9j00<007ooOol02goo00<0
07ooOol00Woo000BOol00`00Oomoo`05Ool00`00Oomoo`0bOol20008E_d00`00D_eBo@04D_d60005
C_d00`00A]i6gP04A]h00`00=YhfWP04=Yh400000biM000V70059Q`00`00:Q`Z7002:Q`30006?SX0
0`00ASY6>P06ASX00`00BUU:F@07BUT00`00CUU>F@06CUT00`00DUQBF003DUP30002DWX00`00DWYB
NP02DWX00`00DWYBNP02DWX00`00Oomoo`0;Ool00`00Oomoo`02Ool0019oo`03001oogoo00Ioo`03
001oogoo039oo`03001FoEKm00MFo@03001BoE;m00<0009Bo@03001>oDkm00M>o@030016gTKN00A6
gP03000fWSJN00@fWP03000V?BHm00<000HV7003000Z72XL00@Z70<000@n>P030016>THj00I6>P03
001:FDYI00I:F@03001>FDiI00I>F@03001BF59H00<000IBNP03001BNU9j009BNP03001BNU9j009B
NP03001oogoo00Yoo`03001oogoo009oo`004Woo00<007ooOol01goo00<007ooOol0<Woo00<005Km
E_d01EKm10001U;m00<004kmC_d01dkm00<004KNA]h014KN00<003JN=Yh013JN00<002Hm9Sd00RHm
0`0012HL00<002XL:Q`00bXL00<003Hk=S/01@0024Hj00<004YIBUT01TYI00<004iICUT00diI1P00
2E9j00<0059jDWX00e9j00D0059jDWYBNP0000aoo`03001oogoo009oo`004Goo0P002Woo0P00<goo
00<005KmE_d010000eKm00<005;mD_d01E;m00<004kmC_d01Dkm3P000SJN00<002Hm9Sd00RHm00<0
01Wl00000P0000<V7000:Q`01BXL00<003Hk=S/00cHk1`000THj00<004YIBUT00dYI2@0014ij00<0
059jDWX02E9j00<0059jDWX00e9j00@0059jDWX000aoo`03001oogoo009oo`007goo00<007ooOol0
<Goo0P0025Km00<005;mD_d01E;m00<004kmC_d010000Tkm00<004KNA]h01DKN00@003:N<YhbWPD0
00<V?@04000Io1Wl6O`30005:Q`00`00=S/f>`04=S/00`00@U]2F`03@U/80004BWX00`00CWY>NP07
CWX00`00DWYBNP08DWX00`00DWYBNP03DWX01@00DWYBNU9j00002goo00<007ooOol00Woo000POol0
0`00Oomoo`0aOol00`00E_eFo@07E_d00`00D_eBo@02D_d50007C_d00`00A]i6gP05A]h00`00<Yhb
WP03<Yh00`007Qd0000200000bHm000Io0046O`20004:Q`00`00=S/f>`04=S/00`00@U]2F`05@U/0
0`00BWY:NP07BWX00`00CWY>NP06CWX00`00DWYBNP08DWX00`00DWYBNP04DWX01000DWYBNP002goo
00<007ooOol00Woo000QOol00`00Oomoo`0aOol00`00E_eFo@06E_d50005D_d00`00C_e>o@05C_d0
0`00A]i6gP05A]h00`00<YhbWP03<Yh010007QdN7AhM0`0011Wl00@001gl0000008Z7003000f>cHk
00Df>`030012Fd9K00E2F`03001:NTYj00I:NP03001>NTij00M>NP03001BNU9j00QBNP03001BNU9j
00ABNP@000]oo`03001oogoo005oo`008Woo00<007ooOol0<Goo00<005KmE_d00UKm0`0000=Fo@00
D_d025;m00<004kmC_d01Dkm00<004KNA]h00TKN1P000c:N00<001hM7Qd00ahM0P000aWl00<001gl
7O`00P0000<Z7000=S/01CHk00<0049K@U/01D9K00<004YjBWX01dYj00<004ijCWX01dij00<0059j
DWX01e9j00<0059jDWX01P0000=B^`00Ool02Woo00<007ooOol00Goo000SOol2000bOol01000E_d0
00001EKm00<005;mD_d01e;m00<004kmC_d00Tkm1`000d;N00<003:N<Yh01@0011hM00@000gL0000
008Io004000Mo1gl7O`20007=S/00`00@U]2F`05@U/00`00BWY:NP07BWX00`00CWY>NP06CWX00`00
DWYBNP07DWX30005D[/01000D[]B^`002Woo00<007ooOol00Goo000UOol00`00Oomoo`0`Ool00`00
E_eFo@06E_d00`00D_eBo@05D_d60004C_d00`00@]i2gP04@]h00`00<YhbWP03<Yh300037Qd00`00
3M`=g00300057O`30004=S/00`00@U]2F`06@U/00`00BWY:NP06BWX00`00CWY>NP07CWX00`00DWYB
NP03DWX300000e:k001B^`06D[/00`00D[/0000:Ool00`00Oomoo`01Ool002Ioo`03001oogoo031o
o`03001FoEKm00IFo@03001BoE;m00@000=Bo@03001>oDkm00E>o@030012gT;N00A2gP03000bWS:N
00@bWP@000037Qd000gL008=g08000@Mo003000^?2hl00H000M2F`03001:NTYj00M:NP03001>NTij
00I>NP03001BNU9j00<000EB^`03001B^e:k00EB^`03001B^`0000Uoo`03001oogoo005oo`009goo
00<007ooOol0<7oo00<005KmE_d015Km100025;m00<004kmC_d01Dkm00<004;N@]h014;N00<003:N
<Yh013:N00<001XM6Qd00`0010gL0P000agl00<002hl;S`012hl10001D9K00<004YjBWX01TYj00<0
04ijCWX00dij1P0025:k00<005:kD[/01U:k00<005:k000027oo00<007ooOol00Goo000XOol2000a
Ool00`00E_eFo@030004D_d00`00D_eBo@06D_d00`00C_e>o@02C_d?0002<Yh00`006QdJ7@026Qd2
00023M`010004M`000000Qgl00<002hl;S`012hl00<003il?W`01P001DYj2@0014jK00<005:kD[/0
25:k00<005:kD[/01E:k00<005:k000027oo00<007ooOol00Goo000ZOol00`00Oomoo`0_Ool20008
D_d00`00D_eBo@04D_d60004C_d00`00@]i2gP04@]h01000<WhbOS9n1@000aXM0`0000@=g0004M`A
g0<000L^?003000nO3il00@nO0H000A6W003001>VdjK00M>V`03001B^e:k00MB^`03001B^e:k00IB
^`<000Qoo`40005oo`5oo`00:goo00<007ooOol0<7oo00<005;mD_d01U;m00<005;mD_d00`000e;m
00<004kmC_d01Dkm00<004;N@]h014;N00<0039n<Wh0139n100000<J7@002K/00`000a7L0P001Rhl
00<003il?W`01Cil00<004JLAY`01dJL00<004jKCY/01TjK00<005:kD[/025:k00<005:kD[/00e:k
1@0027oo0@000Goo0Goo000/Ool00`00Oomoo`0`Ool00`00D_eBo@04D_d40007D_d00`00C_e>o@05
C_d00`00@]i2gP04@]h00`00<WhbOP04<Wh00`005QdF7@0300022K/200044M`30004;S`00`00?W`n
O005?W`00`00AYa6W006AY`00`00CY]>V`07CY/00`00D[]B^`07D[/00`00D[]B^`030004D]`20007
Ool10001Ool1Ool002eoo`80035oo`03001BoE;m00<000=Bo@03001BoE;m00IBo@03001>oDkm009>
o@l0008bOP03000F7AHM008F7@800089^`80008Ag003000R7B8M00<0008^?003000nO3il00DnO003
0016W4JL00I6W003001>VdjK00M>V`03001B^e:k00EB^`@000MBg003001Bg00000Ioo`40005oo`5o
o`00;goo00<007ooOol0;goo0P001e;m00<005;mD_d015;m1P003goo1@000QHM00D000Fk000000Vk
00<000034M`0028M00<R7@<000LnO0030016W4JL00M6W003001>VdjK00I>V`03001B^e:k00D0009B
g003001Bg5;L00MBg08000Ioo`40005oo`5oo`00<7oo00<007ooOol0;goo00<005;mD_d01U;m00<0
05;mD_d00`006Woo10000PFk0`0000@9^`00000000HR7@H0008nO0030016W4JL00I6W003001>VdjK
00=>V`H000QBg003001Bg5;L00MBg08000Eoo`40005oo`5oo`00<Goo00<007ooOol0;goo00<005;m
D_d015;m100087oo0P000`Fk0P000PVk0P001B8M00<003Im=Wd00cIm4@0014kL00<005;LD]`01e;L
00<005;LD]`01e;L0`0017oo0@000Goo0Goo000bOol00`00Oomoo`0_Ool00`00D_eBo@03000VOol2
00031K/200022K/300038Qd00`00=WdfO@05=Wd00`00A[e6_@06A[d00`00C]a>g007C]`00`00D]aB
g007D]`00`00D]aBg003D]`300000e;m00000005Ool10001Ool003=oo`80031oo`8002]oo`800006
1K/000Fk0009^`000QXM0`0000<R7@00=Wd01SIm00<004JmA[d01TJm00<004kLC]`01TkL00<005;L
D]`01e;L00<005;LD]`00`001E;m0P0017oo0@000Goo000eOol00`00Oomoo`0]Ool00`00Oomoo`0/
Ool3000010Fk0000000011XM0`001cIm00<004JmA[d01DJm00<004kLC]`01dkL00<005;LD]`01E;L
10002E;m0P000goo0@000Goo000fOol00`00Oomoo`0/Ool00`00Oomoo`0^Ool200021K/200066Qd3
0004=Wd00`00A[e6_@06A[d00`00C]a>g006C]`00`00D]aBg0050002D_d00`00D_eBo@08D_d20003
Ool10001Ool003Moo`03001oogoo02]oo`03001oogoo031oo`800085^`03000J7AXM00<J7@03000b
OS9n00H000M6_@03001>g4kL00=>g0H000QBo@03001BoE;m00QBo@80009oo`40005oo`00>7oo0P00
:goo00<007ooOol0<Woo0P0000@5^`00000000<J7@03000bOS9n00@bOPh000E:g@03001BoE;m00MB
o@03001BoE;m00ABo@<00005Dad007ooOol00001Ool003Yoo`03001oogoo02Qoo`03001oogoo03Ao
o`<000065Qd00000000J7@001S9n00<004:n@[h01D:n00<004[MB]d024[M00<005;mD_d01e;m00<0
05;mD_d00`001E<M00<007oo00000Goo000kOol00`00Oomoo`0WOol00`00Oomoo`0fOol200035Qd3
0006<Wh00`00@[i2_P05@[h00`00B]e:g@08B]d00`00D_eBo@03D_d60008Dad20001Ool1000003ao
o`03001oogoo02Ioo`03001oogoo03Moo`8000DF7@<000@bOP030012_T:n00E2_P03001:gD[M00I:
g@L000A>o@03001C7E<M00QC7@40005oo`400000?Goo0P009Woo00<007ooOol0>Goo0P000aHM00<0
02in;Wh01@0024:n00<004[MB]d01@000T[n00<004kmC_d01dkm00<005<MDad01U<M1000000oOol0
0`00Oomoo`0SOol00`00Oomoo`0kOol300000aHM000^OP04;Wh<0008B_h00`00C_e>o@07C_d00`00
DaeC7@0500000goo000000010000041oo`03001oogoo025oo`03001oogoo03moo`<000H^OP030012
gT;N00I2gP03001:oT[n00M:oP03001>oDkm00E>o@@0009oo`@000=oo`00@Goo00<007ooOol087oo
00<007ooOol0@Woo0`0012in00<004;N@]h01T;N00<004[nB_h01T[n00<004kmC_d01@000Woo1000
1goo0012Ool00`00Oomoo`0OOol00`00Oomoo`15Ool60008@]h00`00B_i:oP02B_h60004Ool3000;
Ool004=oo`8001moo`03001oogoo04Yoo`h000Ioo`@000ioo`00AGoo00<007ooOol077oo00<007oo
Ool0FWoo10004Woo0016Ool00`00Oomoo`0KOol00`00Oomoo`1FOol4000FOol004Moo`03001oogoo
01Yoo`03001oogoo059oo`@001Yoo`00B7oo0P006Woo00<007ooOol0CWoo10007Woo001:Ool00`00
Oomoo`0GOol00`00Oomoo`1;Ool3000ROol004]oo`03001oogoo01Ioo`03001oogoo04Moo`@002Eo
o`00C7oo00<007ooOol05Goo00<007ooOol0@goo1000:Goo001=Ool00`00Oomoo`0DOol00`00Oomo
o`0oOol4000]Ool004ioo`8001Aoo`03001oogoo03]oo`@0035oo`00D7oo00<007ooOol04Goo00<0
07ooOol0=goo1000=Goo001AOol00`00Oomoo`0@Ool00`00Oomoo`0dOol3000iOol0059oo`03001o
ogoo00moo`03001oogoo031oo`@003aoo`00Dgoo0P003goo00<007ooOol0;7oo1000@7oo001EOol0
0`00Oomoo`0<Ool00`00Oomoo`0XOol40014Ool005Ioo`03001oogoo00]oo`03001oogoo02Aoo`@0
04Qoo`00Egoo00<007ooOol02Woo00<007ooOol087oo1000C7oo001HOol20009Ool00`00Oomoo`0N
Ool3001@Ool005Yoo`03001oogoo00Ioo`03001oogoo01Yoo`@005=oo`00Fgoo00<007ooOol01Goo
00<007ooOol05Woo1000Egoo001LOol00`00Oomoo`04Ool00`00Oomoo`0BOol4001KOol005eoo`03
001oogoo00=oo`03001oogoo00ioo`@005moo`00GWoo0P000goo00<007ooOol02Woo1000Hgoo001P
Ool01000Oomoo`002Goo0`00Igoo001QOol00`00Ool00005Ool4001ZOol0069oo`800003Ool00000
008006ioo`00Hgoo0P00LWoo003GOol00=Moo`00egoo003GOol00=Moo`00egoo003GOol00=Moo`00
\
\>"],
  ImageRangeCache->{{{0, 214}, {173.188, 0}} -> {-0.12264, 0.0107188, 
  0.00502446, 0.00502446}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Critical Price Computations", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "First a point of notation: The first exercise date is ",
  Cell[BoxData[
      \(TraditionalForm\`T\_1\)]],
  " and the second exercise date is ",
  Cell[BoxData[
      \(TraditionalForm\`T\_2\)]],
  ". The second phase of option has time to expiry ",
  Cell[BoxData[
      \(TraditionalForm\`T\_2 - T\_1\)]],
  " and a strike ",
  Cell[BoxData[
      \(TraditionalForm\`X\_2\)]],
  ".  First we get the critical prices where the value of the underlying \
option equals the strike on the first date."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CriticalCallPrice[Xone_, \ Xtwo_, \ \[Sigma]_, r_, q_, \ \ Tone_, \ 
        Ttwo_]\  := \n\t
      P\  /. FindRoot\ [
          Xone == BlackScholesCall[P, \ Xtwo, \[Sigma], \ r, \ q, \ 
              Ttwo - Tone], \ {P, \ Xtwo}]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalCallPrice[3, \ 50, \ 0.2, \ 0.05, 0, \ 1, \ 2]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(46.0058679464002207`\)], "Output"]
}, Open  ]],

Cell[BoxData[
    \(\(CriticalPutPrice[Xone_, \ Xtwo_, \ \[Sigma]_, r_, q_, \ \ Tone_, \ 
        Ttwo_]\  := \n\t
      P\  /. FindRoot\ [
          Xone == BlackScholesPut[P, \ Xtwo, \[Sigma], \ r, \ q, \ 
              Ttwo - Tone], \ {P, \ Xtwo}]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(CriticalPutPrice[3, \ 50, \ 0.2, \ 0.05, 0, \ 1, \ 2]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(49.429812973919498`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Auxiliary Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
To define the compound valuations we need the following \
functions:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(aone[S_, \ Sc_, \ \[Sigma]_, \ r_, \ q_, \ 
        Tone_]\  := \n\t\((Log[S/Sc]\  + \ \((r\  - \ q\  + \ \[Sigma]^2/2)\)*
              Tone)\)/\((\[Sigma]*Sqrt[Tone])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(atwo[S_, \ Sc_, \ \[Sigma]_, \ r_, \ q_, \ 
        Tone_]\  := \n\t\((Log[S/Sc]\  + \ \((r\  - \ q\  - \ \[Sigma]^2/2)\)*
              Tone)\)/\((\[Sigma]*Sqrt[Tone])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(bone[S_, \ Xtwo_, \ \[Sigma]_, \ r_, \ q_, \ 
        Ttwo_]\  := \n\t\((Log[
              S/Xtwo]\  + \ \((r\  - \ q\  + \ \[Sigma]^2/2)\)*
              Ttwo)\)/\((\[Sigma]*Sqrt[Ttwo])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(btwo[S_, \ Xtwo_, \ \[Sigma]_, \ r_, \ q_, \ 
        Ttwo_]\  := \n\t\((Log[
              S/Xtwo]\  + \ \((r\  - \ q\  - \ \[Sigma]^2/2)\)*
              Ttwo)\)/\((\[Sigma]*Sqrt[Ttwo])\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Call on a Call", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We are now able to write down the pricing formula for the Call on a \
Call.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CallCall[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := \n
      Module[{Sc, \ ao, \ at, \ bo, \ bt, \ rho\  = \ Sqrt[To/Tt]}, \n\t
        Sc\  = \ CriticalCallPrice[Xo, \ Xt, \ \[Sigma], r, q, \ \ To, \ 
            Tt]; \ \n\tao\  = aone[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        at\  = atwo[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        bo\  = \ bone[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        bt\  = \ btwo[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        S*Exp[\(-q\)*Tt]*Mfunc[ao, \ bo, \ rho]\  - \ 
          Xt*Exp[\(-r\)*Tt]*Mfunc[at, bt, rho]\  - \ 
          Xo*Exp[\(-r\)*To]*Norm[at]\n]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
If you prefer to work with interest rates and yields in annual \
terms rather than the continuously compounded ones employed above we make a \
simple change of variables:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CallCallAnn[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := 
      CallCall[S, \ Xo, Xt, \ \[Sigma], \ Log[1 + r]\ , \ 
        Log[1 + q], \ \ To, \ Tt]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sample Values", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let us consider the following sample parameters:\n\nindex: 100;\nstrike on \
underlying: 100;\nvolatility: 20%\nrisk-free: 10% ",
  StyleBox["annual",
    FontSlant->"Italic"],
  ";\ndividend-yield: 5% ",
  StyleBox["annual",
    FontSlant->"Italic"],
  ";\ntotal time to final expiry (Hull's ",
  Cell[BoxData[
      \(TraditionalForm\`T\_2\)]],
  "): 1 year.\n\nThe strikes on the intermediate date are given by the \
following list:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(klist\  = \ {5, \ 7.5, \ 10, \ 12.5, \ 15}; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "The times to the intermediate date, ",
  Cell[BoxData[
      \(TraditionalForm\`T\_1\)]],
  ", are given by the following list, giving fractions of a year:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(tlist\  = \ {0.25, \ 0.375, \ 0.50, \ 0.625, \ 0.75}; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
We make a list of values as a matrix, with each row corresponding \
to a fixed strike, each column corresponding to one time.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      Table[CallCallAnn[100, klist[\([i]\)], \ 100, \ 0.2, \ 0.10, \ 0.05, \ 
          tlist[\([j]\)], \ \ 1.0], \ {i, \ 1, \ 5}, \ {j, \ 1, \ 5}]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"5.30918011355948493`", "5.66387757795904889`", 
            "6.01016486533471194`", "6.35066222649677669`", 
            "6.69355854969013108`"},
          {"3.66329580788647835`", "4.2000391603108973`", 
            "4.68152370286294239`", "5.13252240932328263`", 
            "5.57094662938833451`"},
          {"2.44415888103996969`", "3.07077311037373235`", 
            "3.62276941487057468`", "4.13383387295717596`", 
            "4.62502037068630933`"},
          {"1.58335954084706287`", "2.2150935856072067`", 
            "2.78302620688823676`", "3.31375190360433657`", 
            "3.82488715875792983`"},
          {"0.9985756893925398`", "1.5770068463237834`", 
            "2.12127338323648073`", "2.64174279716901105`", 
            "3.14853603203862419`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]],

Cell[TextData[{
  "These results compare well with those published by Rubenstein in his ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  " article (1991) and elsewhere."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put on a Call", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(PutCall[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := \n
      Module[{Sc, \ ao, \ at, \ bo, \ bt, \ rho\  = \ Sqrt[To/Tt]}, \n\t
        Sc\  = \ CriticalCallPrice[Xo, \ Xt, \ \[Sigma], r, q, \ \ To, \ 
            Tt]; \ \n\tao\  = aone[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        at\  = atwo[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        bo\  = \ bone[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        bt\  = \ btwo[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t\(-S\)*
            Exp[\(-q\)*Tt]*Mfunc[\(-ao\), \ bo, \ \(-rho\)]\  + \ 
          Xt*Exp[\(-r\)*Tt]*Mfunc[\(-at\), bt, \(-rho\)]\  + \ 
          Xo*Exp[\(-r\)*To]*Norm[\(-at\)]\n]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(PutCallAnn[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := 
      PutCall[S, \ Xo, Xt, \ \[Sigma], \ Log[1 + r]\ , \ 
        Log[1 + q], \ \ To, \ Tt]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sample Values", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["For the sample parameters as considered previously, we have", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      Table[PutCallAnn[100, klist[\([i]\)], \ 100, \ 0.2, \ 0.10, \ 0.05, \ 
          tlist[\([j]\)], \ \ 1.0], \ {i, \ 1, \ 5}, \ {j, \ 1, \ 5}]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"0.414162624948381363`", "0.71103894753689234`", 
            "1.00018987889355104`", "1.28422754597338695`", 
            "1.57133292423398973`"},
          {"1.20941354387024091`", "1.65942518454139475`", 
            "2.05520518137552521`", "2.42151434873430515`", 
            "2.7762520213009636`"},
          {"2.43141184109297103`", "2.94238378847371784`", 
            "3.38010736576967296`", "3.77825244256306191`", 
            "4.15785687523774516`"},
          {"4.01174772541338597`", "4.49892885703534695`", 
            "4.9240206557450854`", "5.31359711253053212`", 
            "5.68525477824217517`"},
          {"5.86809916293706379`", "6.27306686211460551`", 
            "6.64592428721616634`", "6.99701463301458126`", 
            "7.3364347642593124`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Put on a Put", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(PutPut[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := \n
      Module[{Sc, \ ao, \ at, \ bo, \ bt, \ rho\  = \ Sqrt[To/Tt]}, \n\t
        Sc\  = \ CriticalPutPrice[Xo, \ Xt, \ \[Sigma], r, q, \ \ To, \ 
            Tt]; \ \n\tao\  = aone[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        at\  = atwo[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        bo\  = \ bone[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        bt\  = \ btwo[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        S*Exp[\(-q\)*Tt]*Mfunc[ao, \ \(-bo\), \ \(-rho\)]\  - \ 
          Xt*Exp[\(-r\)*Tt]*Mfunc[at, \(-bt\), \(-rho\)]\ \  + \ 
          Xo*Exp[\(-r\)*To]*Norm[at]\n]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(PutPutAnn[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := 
      PutPut[S, \ Xo, Xt, \ \[Sigma], \ Log[1 + r]\ , \ Log[1 + q], \ \ To, \ 
        Tt]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sample Values", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["For the sample parameters as considered previously, we have", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      Table[PutPutAnn[100, klist[\([i]\)], \ 100, \ 0.2, \ 0.10, \ 0.05, \ 
          tlist[\([j]\)], \ \ 1.0], \ {i, \ 1, \ 5}, \ {j, \ 1, \ 5}]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.12554604259212265`", "1.4267561004107141`", 
            "1.68969077650039434`", "1.93406520568393958`", 
            "2.17390046346059806`"},
          {"2.69240881438319945`", "2.97935513086537095`", 
            "3.23818440888077052`", "3.48303899154490537`", 
            "3.72561433483341364`"},
          {"4.68041535323650847`", "4.8642206338200733`", 
            "5.05548176487147848`", "5.25099185829084369`", 
            "5.4540507332579553`"},
          {"6.90661750987603184`", "6.96541958100483071`", 
            "7.06211052673864081`", "7.18217654746809586`", 
            "7.32127671576944472`"},
          {"9.2543788697019842`", "9.20242823453268776`", 
            "9.20159673781987841`", "9.23689110496009568`", 
            "9.30038728926418656`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Call on a Put", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CallPut[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ Tt_]\  := \n
      Module[{Sc, \ ao, \ at, \ bo, \ bt, \ rho\  = \ Sqrt[To/Tt]}, \n\t
        Sc\  = \ CriticalPutPrice[Xo, \ Xt, \ \[Sigma], r, q, \ \ To, \ 
            Tt]; \ \n\tao\  = aone[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        at\  = atwo[S, \ Sc, \ \[Sigma], \ r, \ q, \ To]; \n\t
        bo\  = \ bone[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t
        bt\  = \ btwo[S, \ Xt, \ \[Sigma], \ r, \ q, \ Tt]; \n\t\(-S\)*
            Exp[\(-q\)*Tt]*Mfunc[\(-ao\), \ \(-bo\), \ rho]\  + \ 
          Xt*Exp[\(-r\)*Tt]*Mfunc[\(-at\), \(-bt\), rho]\ \  - \ 
          Xo*Exp[\(-r\)*To]*Norm[\(-at\)]\n]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(CallPutAnn[S_, \ Xo_, Xt_, \ \[Sigma]_, r_, q_, \ \ To_, \ 
        Tt_]\  := \ 
      CallPut[S, \ Xo, Xt, \ \[Sigma], \ Log[1 + r]\ , \ 
        Log[1 + q], \ \ To, \ Tt]\ \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Sample Values", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["For the sample parameters as considered previously, we have", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TableForm[
      Table[CallPutAnn[100, klist[\([i]\)], \ 100, \ 0.2, \ 0.10, \ 0.05, \ 
          tlist[\([j]\)], \ \ 1.0], \ {i, \ 1, \ 5}, \ {j, \ 1, \ 5}]]\)], 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[GridBox[{
          {"1.69155920177351237`", "2.05059034547003582`", 
            "2.37066145328557276`", "2.67149555482269107`", 
            "2.96712184789988686`"},
          {"0.817286804573468117`", "1.19096476280752261`", 
            "1.53549860508161328`", "1.8650427146731733`", 
            "2.1913046077075835`"},
          {"0.364159021410211503`", "0.663605628312189832`", 
            "0.969139470063208285`", "1.27756910411397339`", 
            "1.5922098894099852`"},
          {"0.149225285425821923`", "0.352579923981757747`", 
            "0.592111684524859249`", "0.853326847025606127`", 
            "1.13190480396142012`"},
          {"0.0558510744141101866`", "0.177364644800429012`", 
            "0.347941576246603379`", "0.552614914556645864`", 
            "0.783484229148077737`"}
          },
        RowSpacings->1,
        ColumnSpacings->3,
        RowAlignments->Baseline,
        ColumnAlignments->{Left}],
      (TableForm[ #]&)]], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.4 Quanto Options", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "In Chapter 3 we analysed the various options that can be built out of an \
asset and an FX rate. We identified a certain class of options, known as \
\"Quantos\", that could be reduced to a standard form by various parameter \
substitutions. One can also approach Quantos through the methods given by \
Reiner (1992). Reiner's model is based on the ",
  StyleBox["foreign-market",
    FontSlant->"Italic"],
  " valuation model. The Quanto option may be valued in much the same way as \
was done with the geometric Asian, by making a suitable substitution for the \
parameters. "
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->1,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Quanto Effective Yield", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We define a function ",
  StyleBox["QuantoQ",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " (the ",
  Cell[BoxData[
      \(TraditionalForm\`x\_3\)]],
  " as defined by Reiner), which is the effective yield parameter."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(QuantoQ[q_, \ rd_, \ rf_, \ sdstock_, \ sdforex_, \ corr_]\  := \ 
      q\  + \ rd\  - rf\  + \ corr*sdstock*sdforex\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Note that we are using continuously compounded variables for ",
  StyleBox["rf",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["q",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["r",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", whereas Reiner uses annual measures."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->1,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Quanto Call Exact Symbolic Model", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["The Black-Scholes formula for the Quanto valuation is", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[BoxData[
    \(QuantoCall[pricef_, \ strikef_, \ \n\tsdstock_, \ sdforex_, \ rf_, \ 
        rd_, \ q_, \ corr_, \ t_, \ \ fixed_, \ spot_]\  := \ \n
      fixed*BlackScholesCall[pricef, \ strikef, \ sdstock, \ rd, \ 
          QuantoQ[q, \ rd, \ rf, \ sdstock, \ sdforex, \ corr], t]\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Now we compute the sensitivities by symbolic differentiation. Note \
that the Delta is given w.r.t. domestic currency, so that it does indeed give \
the required amount of the equity in a hedge portfolio. The hedge portfolio \
should also contain some domestic and foreign cash. Note also that we define \
a pair of Rhos and a pair of Vegas.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
QuantoCallDelta[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, \
q_, corr_, t_, fixed_, spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd,q, corr, t, fixed, \
spot], pricef]/spot; \
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallGamma[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, \
q_, 
corr_, t_, fixed_, spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], {pricef,2}]/spot^2;\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallTheta[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, \
q_, 
corr_, t_, fixed_, spot_] = 
D[-QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], t];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallVega[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_,spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], sdstock];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallVegaFX[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, \
q_, 
corr_, t_, fixed_,spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], sdforex];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallRhoF[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_, spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], rf];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoCallRhoD[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_, spot_] = 
D[QuantoCall[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], rd];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Quanto Put Exact Symbolic Model", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["First we introduce the value function:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(QuantoPut[pricef_, \ strikef_, sdstock_, \nsdforex_, \ rf_, \ rd_, \ 
        q_, \ corr_, \ t_, \ fixed_, \ spot_]\  := \ \n
      fixed*BlackScholesPut[pricef, \ strikef, \ sdstock, \ rd, \ 
          QuantoQ[q, \ rd, \ rf, \ sdstock, \ sdforex, \ corr], t]\)], "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["Now we compute the sensitivities by differentiation:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
QuantoPutDelta[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_, spot_] = 
D[QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], pricef]/spot;\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoPutGamma[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_, spot_] = 
D[QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], {pricef,2}]/spot^2;\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoPutTheta[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, \

corr_, t_, fixed_, spot_] = 
D[-QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], t];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoPutVega[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, 
corr_, t_, fixed_,spot_] = 
D[QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], sdstock];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell[BoxData[
    \(\(QuantoPutVegaFX[pricef_, \ strikef_, \ sdstock_, \ sdforex_, \ rf_, \ 
        rd_, \ q_, \ \ncorr_, \ t_, \ fixed_, spot_]\  = \ \n
      D[QuantoPut[pricef, \ strikef, \ sdstock, \ sdforex, \ rf, \ rd, \ 
          q, \ \ncorr, \ t, \ fixed, \ spot], \ sdforex]; \)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
QuantoPutRhoF[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, 
corr_, t_, fixed_, spot_] = 
D[QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], rf];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True],

Cell["\<\
QuantoPutRhoD[pricef_, strikef_, sdstock_, sdforex_, rf_, rd_, q_, 
corr_, t_, fixed_, spot_] = 
D[QuantoPut[pricef, strikef, sdstock, sdforex, rf, rd, q, 
corr, t, fixed, spot], rd];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False,
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.5 Warrants", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell["The Lauterbach-Schultz Analysis", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  AspectRatioFixed->True],

Cell[TextData[{
  "In their 1990 paper, Lauterbach and Schultz (LS) looked at two types of \
models for valuing warrants. The Black-Scholes model may be modified to value \
European warrants issued by a company on its own stock, by allowing for the \
effects of dilution. This dilution adjustment was discussed in the original \
Black-Scholes paper and so will be referred to as the Black-Scholes \
warrant-pricing model. This clearly ignores the following: early exercise; \
extension; volatility and interest-rate changes over the warrant life; \
variation of volatility with stock price. \n\nCox, in a 1975 Stanford working \
paper, introduced constant elasticity of variance (CEV) models where the \
volatility is inversely proportional to a fractional power of the stock \
price. The special case where the volatility varied as the inverse square \
root of the stock price is called the SRCEV model. This family of models is \
discussed in detail in Chapter 28.\n\nLS also investigated other \
modifications to BS that allow for early exercise, and stochastic interest \
rates and volatilities. They concluded that these modifications did not \
generate significant improvements in the pricing model. They concluded, \
however, that the CEV model outperforms the dilution-adjusted Black-Scholes \
model given a good estimate of the elasticity, but that it is also true that \
the SRCEV model also outperforms the dilution-adjusted BS model. These \
comparisons were based on a data analysis study that explored how well the \
models explained market price data - the BS model was found to be markedly \
inferior at explaining prices across the market. Nevertheless, the \
dilution-adjusted Black-Scholes model has found widespread popularity, and it \
is the one developed here. \n\nThe reasons for focusing on this model are \
pedagogical, since the main focus of this book is on explaining how to do \
things in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". The CEV models can be coded up using the functions built in to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". However, the dilution adjustment to the Black-Scholes model requires the \
solution of non-linear equations (somewhat analogously to the computation of \
implied volatility), so it is perhaps more instructive to review this here. \
As noted, the CEV family is discussed in Chapter 28, and can also be made the \
basis of a dilution-adjusted CEV calculation of value."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->Automatic,
  PageBreakWithin->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Dilution Equation for Warrants", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
Let us suppose that the unknown warrant price is \"warprice\", and \
that the number of outstanding warrants is \"warrants\", the number of shares \
per warrant is \"shperwar\" and the number of outstanding shares is \
\"shares\".\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
WarrantEqn[p_,k_,sd_,r_,q_,t_,warprice_, shares_, warrants_, \
shperwar_] :=
(shares*shperwar/(shares+shperwar*warrants))*
BlackScholesCall[p*Exp[-q*t]+warrants*warprice/shares,k,sd,r,0,t]\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
WarrantValue[p_,k_,sd_,r_,q_,t_, shares_, warrants_, shperwar_] :=
warprice /. FindRoot[
warprice == WarrantEqn[p,k,sd,r,q,t,warprice,shares, warrants, shperwar],
{warprice, 10}]
\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["WarrantValue[100,90,0.1,0.04,0.04,1, 100, 10, 1]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(10.1663980249335095`\)], "Output"]
}, Open  ]],

Cell["\<\
As remarked in the introduction to this section, this calculation \
is meant to be illustrative, as there are other real-world complications in \
warrant pricing that are mostly beyond the scope of this text, but see \
Chapter 28 for a discussion of CEV models. One irritating point of convention \
is that, depending on the market, warrant values may be reported in raw \
currency terms or as a percentage of their denomination! Also, the formulae \
we have given relate to purely domestic warrants - see Section 3.6 for a \
discussion of FX management. \
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell["Warrant Hedge Parameters", "Subsubsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["\<\
These are slightly tricky as the dilution effect rears its head in \
several places simultaneously. Here is an illustrative computation of Delta \
with the dilution effect incorporated - along the way the warrant value is \
reported.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell["Clear[wdone];", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
wdone[price_, strike_, sd_, r_, t_] :=
(Log[price/strike] + (r + sd^2/2)*t)/(sd*Sqrt[t])\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
di[shares_, warrants_, shperwar_] := 
(shares*shperwar)/(shares+warrants*shperwar)\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
WarrantDelta[price_, strike_, vol_, rate_, q_, time_,shares_, 
warrants_, shperwar_] :=
Module[{w, delbs, dil},
w = WarrantValue[price,strike,vol,rate,q, time,shares, warrants, 
shperwar];
Print[w];
dil = di[shares,warrants,shperwar];
delbs = Norm[wdone[price*Exp[-q*time]+w*warrants/shares,strike,vol,rate,time]]\
;
dil*delbs*Exp[-q*time]/(1 - dil*delbs*warrants/shares)]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["\<\
WarrantDelta[780,815,0.238120,4*Log[1+1.351875/400],0.03,3.89589,
29135984,9550,513.619632]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(47542.128774447363`\)], "Print"],

Cell[BoxData[
    \(220.2015567386991`\)], "Output"]
}, Open  ]],

Cell["\<\
It may be useful to express Delta on a \"per share\" basis - here \
it is divided by the shares per warrant (some people also prefer to multiply \
by 100 to get Delta on a percentage basis):\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic],

Cell[CellGroupData[{

Cell["%/513.619632", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->Automatic,
  InitializationCell->True,
  AspectRatioFixed->True],

Cell[BoxData[
    \(0.42872496107917275`\)], "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.6 Chooser Options", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
The option known as a \"chooser option\", also called by Hull \
(1996) an \"as you like it\" option, contains the feature that the holder can \
decide after a specified period of time whether the option is a Call or a \
Put. The basic European chooser option can be regarded as a package \
consisting of a simple Call and a simple Put with different weights and times \
to expiry.  As usual, we need the functions used in Black-Scholes, and load \
the Package to treat this first.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Derivatives`BlackScholes`\>"]\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Additional Functions for Use in Choosers", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "We now use the characterization of a chooser as a package consisting of a \
Call and a Put. Note that ",
  StyleBox["t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is time until expiry, ",
  StyleBox["tchoose",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is time until choice."
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
EuroChooser[p_, k_, sd_, r_, q_, t_, tchoose_] := 
BlackScholesCall[p,k,sd,r,q,t] +
Exp[-q*(t - tchoose)]*BlackScholesPut[p,k*Exp[-(r-q)*(t - \
tchoose)],sd,r,q,tchoose]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
EuroChooserDelta[p_, k_, sd_, r_, q_, t_, tchoose_] = 
D[EuroChooser[p, k, sd, r, q, t, tchoose], p];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
EuroChooserGamma[p_, k_, sd_, r_, q_, t_, tchoose_] = 
D[EuroChooser[p, k, sd, r, q, t, tchoose], {p,2}];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
In the following, note that we want the rate of change with respect \
to real time - we therefore decrement BOTH the time to expiry and the time \
till choice.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
EuroChooserTheta[p_, k_, sd_, r_, q_, t_, tchoose_] = 
D[-EuroChooser[p, k, sd, r, q, t, tchoose], t] -
D[EuroChooser[p, k, sd, r, q, t, tchoose], tchoose];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
EuroChooserVega[p_, k_, sd_, r_, q_, t_, tchoose_] = 
D[EuroChooser[p, k, sd, r, q, t, tchoose], sd];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
EuroChooserRho[p_, k_, sd_, r_, q_, t_, tchoose_] = 
D[EuroChooser[p, k, sd, r, q, t, tchoose], r];\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["12.7 Exchange Options", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
These options are in one sense rather like Quantos, in that \
although there are two correlated random processes, the combination can be \
reduced to a one-dimensional system. The matter was sorted out by Margrabe \
(1978).\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Additional Functions for Use in Exchange Options", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Our notation is as used by Hull (1996), Section 18.1. First we \
check that our standard Package has been loaded, by confirming that the CDF \
for the normal distribution is available:\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Norm\)\)], "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Norm[x] returns the cumulative normal distribution function evaluated \
at x."\)], "Print"]
}, Open  ]],

Cell["We just need the following two functions:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell["\<\
done[priceone_, pricetwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, t_] \
:=
(Log[pricetwo/priceone] + 
(qone - qtwo + (sdone^2+sdtwo^2 - 2 corr sdone sdtwo)/2)*t)/
Sqrt[(sdone^2 + sdtwo^2 - 2 corr sdone sdtwo)*t]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["\<\
dtwo[priceone_, pricetwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, t_] \
:=
(Log[pricetwo/priceone] + 
(qone - qtwo - (sdone^2+sdtwo^2 - 2 corr sdone sdtwo)/2)*t)/
Sqrt[(sdone^2 + sdtwo^2 - 2 corr sdone sdtwo)*t]\
\>", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exchange Value and Sensitivity Functions", "Subsection",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "\nThe relevant variables are ",
  StyleBox["priceone",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and ",
  StyleBox["pricetwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", the values of the two underlying assets, and their volatilities ",
  StyleBox["sdone",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["sdtwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and continuous yields ",
  StyleBox["qone",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", ",
  StyleBox["qtwo",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ". The correlation between the assets is ",
  StyleBox["corr",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " and the time to expiry is ",
  StyleBox["t",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ".\n\nFirst we introduce the value function:"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
Exchange[priceone_, pricetwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_]:=
pricetwo*Exp[-qtwo*t]*
Norm[done[priceone, pricetwo, sdone, sdtwo, qone, qtwo, corr, t]] - \
priceone*Exp[-qone*t]*
Norm[dtwo[priceone, pricetwo, sdone, sdtwo, qone, qtwo, corr, t]]\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  InitializationCell->True,
  AspectRatioFixed->True],

Cell["We check that it evaluates correctly:", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Exchange[100, 100, 0.25, 0.25, 0, 0, 0, 366/365]", "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \(14.0506290704468312`\)], "Output"]
}, Open  ]],

Cell["\<\
Now we compute the sensitivities by differentiation. The formulae \
are independent of the risk-free rate so Rho is always zero.\
\>", "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
ExchangeDeltaOne[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], pone];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeDeltaTwo[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], ptwo];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeGammaOne[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], {pone, 2}];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeGammaTwo[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], {ptwo, 2}];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeVegaOne[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], sdone];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeVegaTwo[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], sdtwo];\
\>", 
  "Input",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
ExchangeTheta[pone_, ptwo_, sdone_, sdtwo_, qone_, qtwo_, corr_, \
t_] =
-D[Exchange[pone, ptwo, sdone, sdtwo, qone, qtwo, corr, t], t];\
\>", "Input",\

  CellMargins->{{5, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 12 Bibliography", "Section",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Geman, H. and Yor, M., 1996, Pricing and hedging double-barrier options: a \
probabilistic approach, ",
  StyleBox["Mathematical Finance",
    FontSlant->"Italic"],
  ", 6, p. 365.\n\nHull, J.C., 1996, ",
  StyleBox["Options, Futures, and Other Derivatives",
    FontSlant->"Italic"],
  ", 3rd edition, Prentice-Hall.\n\nKunitomo, N. and Ikeda, M., 1992, Pricing \
options with curved boundaries, ",
  StyleBox["Mathematical Finance,",
    FontSlant->"Italic"],
  " 2, p. 275.\n\nLauterbach, B. and Schultz, P., 1990, Pricing warrants: an \
empirical study of the Black-Scholes model and its alternatives, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 45, p. 1181.\n\nMargrabe, W., 1978,  The value of an option to exchange \
one asset for another, ",
  StyleBox["Journal of Finance",
    FontSlant->"Italic"],
  ", 33, p. 177.\n\nReiner, E., 1992, Quanto Mechanics, in ",
  StyleBox["From Black-Scholes to Black Holes",
    FontSlant->"Italic"],
  " (based on ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", Mar. 92), RISK-FINEX Publications.\n\nRubenstein, M., 1991, Double \
trouble, in ",
  StyleBox["RISK Magazine",
    FontSlant->"Italic"],
  ", Dec. 1991-Jan.1992.\n"
}], "Text",
  CellMargins->{{5, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{640, 665},
WindowMargins->{{15, Automatic}, {Automatic, 0}},
PrintingCopies->1,
PrintingStartingPageNumber->237,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"12. Exotics"}]]}, {Cell[ 
        TextData[ {"12. Exotics"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True,
"FirstPageFace"->Right},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
PrivateFontOptions->{"FontType"->"Outline"},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0b85N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@S<[40000000000074/01T1T00000000000
00000000007oo@S<[73oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 248, 8, 171, "Title"],
Cell[1982, 61, 125, 1, 45, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2132, 66, 141, 3, 49, "Section"],
Cell[2276, 71, 1051, 25, 269, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3364, 101, 103, 1, 49, "Section"],

Cell[CellGroupData[{
Cell[3492, 106, 91, 1, 43, "Subsection"],
Cell[3586, 109, 376, 12, 44, "Text"],
Cell[3965, 123, 127, 2, 21, "NumberedEquation"],
Cell[4095, 127, 2110, 31, 208, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[6242, 163, 125, 1, 32, "Subsection"],
Cell[6370, 166, 246, 7, 20, "Text"],
Cell[6619, 175, 180, 4, 20, "NumberedEquation"],
Cell[6802, 181, 775, 22, 52, "Text"],
Cell[7580, 205, 189, 3, 29, "NumberedEquation"],
Cell[7772, 210, 124, 1, 20, "Text"],
Cell[7899, 213, 282, 4, 33, "NumberedEquation"],
Cell[8184, 219, 119, 1, 20, "Text"],
Cell[8306, 222, 196, 4, 33, "NumberedEquation"],
Cell[8505, 228, 445, 13, 29, "Text"],
Cell[8953, 243, 187, 3, 20, "NumberedEquation"],
Cell[9143, 248, 188, 6, 20, "Text"],
Cell[9334, 256, 168, 3, 34, "NumberedEquation"],
Cell[9505, 261, 233, 6, 20, "Text"],
Cell[9741, 269, 157, 3, 21, "NumberedEquation"],
Cell[9901, 274, 287, 10, 20, "Text"],
Cell[10191, 286, 281, 5, 33, "NumberedEquation"],
Cell[10475, 293, 432, 7, 36, "NumberedEquation"],
Cell[10910, 302, 195, 6, 20, "Text"],
Cell[11108, 310, 312, 5, 32, "NumberedEquation"],
Cell[11423, 317, 217, 5, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[11677, 327, 185, 5, 32, "Subsection"],
Cell[11865, 334, 132, 1, 20, "Text"],
Cell[12000, 337, 265, 7, 68, "Input"],
Cell[12268, 346, 249, 9, 20, "Text"],
Cell[12520, 357, 242, 5, 40, "Input"],
Cell[12765, 364, 358, 6, 42, "Input"],
Cell[13126, 372, 195, 6, 20, "Text"],
Cell[13324, 380, 379, 7, 69, "Input"],
Cell[13706, 389, 307, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[14038, 400, 288, 5, 37, "Input"],
Cell[14329, 407, 573377, 13753, 245, 503729, 12888, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[587721, 14163, 328, 7, 29, "Text"],
Cell[588052, 14172, 128, 2, 25, "Input"],
Cell[588183, 14176, 480, 9, 52, "Text"],
Cell[588666, 14187, 113, 2, 25, "Input"],
Cell[588782, 14191, 562, 11, 52, "Text"],
Cell[589347, 14204, 619, 11, 88, "Input"],
Cell[589969, 14217, 655, 16, 52, "Text"],

Cell[CellGroupData[{
Cell[590649, 14237, 90, 1, 28, "Subsubsection"],
Cell[590742, 14240, 223, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[590990, 14249, 167, 3, 25, "Input"],
Cell[591160, 14254, 56, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[591253, 14260, 167, 3, 25, "Input"],
Cell[591423, 14265, 56, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[591516, 14271, 170, 3, 25, "Input"],
Cell[591689, 14276, 56, 1, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[591806, 14284, 110, 1, 32, "Subsection"],
Cell[591919, 14287, 646, 11, 62, "Text"],
Cell[592568, 14300, 232, 4, 37, "Input"],
Cell[592803, 14306, 778, 13, 100, "Input"],

Cell[CellGroupData[{
Cell[593606, 14323, 172, 3, 25, "Input"],
Cell[593781, 14328, 56, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[593874, 14334, 98, 1, 28, "Subsubsection"],
Cell[593975, 14337, 674, 14, 62, "Text"],
Cell[594652, 14353, 638, 9, 101, "Input"],
Cell[595293, 14364, 822, 14, 100, "Input"],

Cell[CellGroupData[{
Cell[596140, 14382, 172, 3, 25, "Input"],
Cell[596315, 14387, 60, 1, 36, "Output"]
}, Open  ]],
Cell[596390, 14391, 772, 14, 100, "Input"],

Cell[CellGroupData[{
Cell[597187, 14409, 170, 3, 25, "Input"],
Cell[597360, 14414, 56, 1, 36, "Output"]
}, Open  ]],
Cell[597431, 14418, 816, 14, 100, "Input"],

Cell[CellGroupData[{
Cell[598272, 14436, 171, 3, 25, "Input"],
Cell[598446, 14441, 59, 1, 36, "Output"]
}, Open  ]],
Cell[598520, 14445, 818, 14, 100, "Input"],

Cell[CellGroupData[{
Cell[599363, 14463, 172, 3, 25, "Input"],
Cell[599538, 14468, 56, 1, 36, "Output"]
}, Open  ]],
Cell[599609, 14472, 314, 6, 40, "Text"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[599984, 14485, 97, 1, 38, "Section"],
Cell[600084, 14488, 842, 15, 84, "Text"],

Cell[CellGroupData[{
Cell[600951, 14507, 102, 1, 32, "Subsection"],
Cell[601056, 14510, 177, 4, 20, "Text"],
Cell[601236, 14516, 166, 3, 25, "Input",
  InitializationCell->True],
Cell[601405, 14521, 128, 2, 25, "Input"],
Cell[601536, 14525, 130, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[601691, 14530, 118, 2, 25, "Input"],
Cell[601812, 14534, 363, 5, 81, "Print"]
}, Open  ]],
Cell[602190, 14542, 192, 4, 29, "Text"],
Cell[602385, 14548, 157, 3, 37, "Input"],
Cell[602545, 14553, 193, 4, 37, "Input"],
Cell[602741, 14559, 109, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[602875, 14564, 176, 4, 25, "Input"],
Cell[603054, 14570, 48638, 1080, 181, 20147, 723, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[651741, 15656, 106, 1, 32, "Subsection"],
Cell[651850, 15659, 588, 16, 40, "Text"],
Cell[652441, 15677, 326, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[652792, 15687, 144, 2, 25, "Input"],
Cell[652939, 15691, 54, 1, 36, "Output"]
}, Open  ]],
Cell[653008, 15695, 324, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[653357, 15705, 143, 2, 25, "Input"],
Cell[653503, 15709, 53, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[653605, 15716, 98, 1, 32, "Subsection"],
Cell[653706, 15719, 149, 4, 20, "Text"],
Cell[653858, 15725, 260, 4, 37, "Input"],
Cell[654121, 15731, 260, 4, 37, "Input"],
Cell[654384, 15737, 279, 5, 37, "Input"],
Cell[654666, 15744, 279, 5, 37, "Input"]
}, Open  ]],

Cell[CellGroupData[{
Cell[654982, 15754, 93, 1, 32, "Subsection"],
Cell[655078, 15757, 156, 4, 20, "Text"],
Cell[655237, 15763, 726, 11, 137, "Input"],
Cell[655966, 15776, 251, 5, 29, "Text"],
Cell[656220, 15783, 259, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[656504, 15791, 95, 1, 28, "Subsubsection"],
Cell[656602, 15794, 524, 14, 117, "Text"],
Cell[657129, 15810, 138, 2, 25, "Input"],
Cell[657270, 15814, 244, 6, 20, "Text"],
Cell[657517, 15822, 148, 2, 25, "Input"],
Cell[657668, 15826, 206, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[657899, 15834, 252, 5, 37, "Input"],
Cell[658154, 15841, 987, 22, 88, "Output"]
}, Open  ]],
Cell[659156, 15866, 248, 6, 29, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[659453, 15878, 92, 1, 32, "Subsection"],
Cell[659548, 15881, 759, 11, 137, "Input"],
Cell[660310, 15894, 257, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[660592, 15902, 95, 1, 28, "Subsubsection"],
Cell[660690, 15905, 132, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[660847, 15910, 251, 5, 37, "Input"],
Cell[661101, 15917, 989, 22, 88, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[662151, 15946, 91, 1, 32, "Subsection"],
Cell[662245, 15949, 745, 11, 137, "Input"],
Cell[662993, 15962, 255, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[663273, 15970, 95, 1, 28, "Subsubsection"],
Cell[663371, 15973, 132, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[663528, 15978, 250, 5, 37, "Input"],
Cell[663781, 15985, 987, 22, 88, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[664829, 16014, 92, 1, 32, "Subsection"],
Cell[664924, 16017, 760, 11, 137, "Input"],
Cell[665687, 16030, 268, 5, 37, "Input"],

Cell[CellGroupData[{
Cell[665980, 16039, 95, 1, 28, "Subsubsection"],
Cell[666078, 16042, 132, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[666235, 16047, 251, 5, 37, "Input"],
Cell[666489, 16054, 1002, 22, 88, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[667564, 16084, 143, 3, 38, "Section"],
Cell[667710, 16089, 761, 16, 62, "Text"],

Cell[CellGroupData[{
Cell[668496, 16109, 149, 3, 32, "Subsection"],
Cell[668648, 16114, 364, 12, 20, "Text"],
Cell[669015, 16128, 207, 3, 37, "Input"],
Cell[669225, 16133, 510, 19, 29, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[669772, 16157, 159, 3, 32, "Subsection"],
Cell[669934, 16162, 174, 3, 20, "Text"],
Cell[670111, 16167, 361, 6, 64, "Input"],
Cell[670475, 16175, 422, 7, 52, "Text"],
Cell[670900, 16184, 307, 7, 57, "Input",
  InitializationCell->True],
Cell[671210, 16193, 313, 8, 57, "Input",
  InitializationCell->True],
Cell[671526, 16203, 298, 8, 57, "Input",
  InitializationCell->True],
Cell[671827, 16213, 301, 8, 57, "Input",
  InitializationCell->True],
Cell[672131, 16223, 303, 8, 57, "Input",
  InitializationCell->True],
Cell[672437, 16233, 297, 8, 57, "Input",
  InitializationCell->True],
Cell[672737, 16243, 297, 8, 57, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[673071, 16256, 110, 1, 32, "Subsection"],
Cell[673184, 16259, 111, 1, 20, "Text"],
Cell[673298, 16262, 351, 6, 64, "Input"],
Cell[673652, 16270, 125, 1, 20, "Text"],
Cell[673780, 16273, 305, 8, 57, "Input",
  InitializationCell->True],
Cell[674088, 16283, 311, 8, 48, "Input",
  InitializationCell->True],
Cell[674402, 16293, 296, 8, 48, "Input",
  InitializationCell->True],
Cell[674701, 16303, 297, 7, 48, "Input",
  InitializationCell->True],
Cell[675001, 16312, 359, 5, 52, "Input"],
Cell[675363, 16319, 293, 7, 48, "Input",
  InitializationCell->True],
Cell[675659, 16328, 318, 8, 48, "Input",
  PageBreakAbove->False,
  InitializationCell->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[676026, 16342, 118, 2, 42, "Section",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[676169, 16348, 187, 4, 33, "Subsection",
  PageBreakAbove->Automatic],
Cell[676359, 16354, 2640, 42, 241, "Text",
  PageBreakAbove->Automatic,
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[679036, 16401, 142, 2, 33, "Subsection",
  PageBreakAbove->Automatic],
Cell[679181, 16405, 340, 7, 33, "Text",
  PageBreakAbove->Automatic],
Cell[679524, 16414, 356, 10, 37, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[679883, 16426, 344, 10, 57, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],

Cell[CellGroupData[{
Cell[680252, 16440, 205, 4, 25, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[680460, 16446, 54, 1, 36, "Output"]
}, Open  ]],
Cell[680529, 16450, 665, 11, 73, "Text",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[681219, 16465, 135, 2, 28, "Subsubsection",
  PageBreakAbove->Automatic],
Cell[681357, 16469, 343, 7, 40, "Text",
  PageBreakAbove->Automatic],
Cell[681703, 16478, 170, 4, 25, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[681876, 16484, 253, 7, 28, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[682132, 16493, 247, 7, 28, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[682382, 16502, 537, 15, 97, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],

Cell[CellGroupData[{
Cell[682944, 16521, 256, 7, 28, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[683203, 16530, 52, 1, 20, "Print"],
Cell[683258, 16533, 52, 1, 20, "Output"]
}, Open  ]],
Cell[683325, 16537, 300, 6, 20, "Text",
  PageBreakAbove->Automatic],

Cell[CellGroupData[{
Cell[683650, 16547, 169, 4, 20, "Input",
  PageBreakAbove->Automatic,
  InitializationCell->True],
Cell[683822, 16553, 54, 1, 20, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[683949, 16562, 144, 3, 42, "Section"],
Cell[684096, 16567, 561, 9, 46, "Text"],
Cell[684660, 16578, 128, 2, 20, "Input"],

Cell[CellGroupData[{
Cell[684813, 16584, 167, 3, 33, "Subsection"],
Cell[684983, 16589, 438, 14, 20, "Text"],
Cell[685424, 16605, 277, 7, 37, "Input"],
Cell[685704, 16614, 209, 5, 28, "Input"],
Cell[685916, 16621, 213, 5, 28, "Input"],
Cell[686132, 16628, 288, 7, 20, "Text"],
Cell[686423, 16637, 264, 6, 37, "Input"],
Cell[686690, 16645, 209, 5, 28, "Input"],
Cell[686902, 16652, 207, 5, 28, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[687158, 16663, 145, 3, 42, "Section"],
Cell[687306, 16668, 304, 6, 33, "Text"],

Cell[CellGroupData[{
Cell[687635, 16678, 175, 3, 32, "Subsection"],
Cell[687813, 16683, 313, 7, 29, "Text"],

Cell[CellGroupData[{
Cell[688151, 16694, 99, 2, 25, "Input"],
Cell[688253, 16698, 112, 2, 32, "Print"]
}, Open  ]],
Cell[688380, 16703, 114, 1, 20, "Text"],
Cell[688497, 16706, 345, 9, 57, "Input",
  InitializationCell->True],
Cell[688845, 16717, 345, 9, 57, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[689227, 16731, 167, 3, 32, "Subsection"],
Cell[689397, 16736, 997, 37, 73, "Text"],
Cell[690397, 16775, 396, 11, 80, "Input",
  InitializationCell->True],
Cell[690796, 16788, 158, 3, 20, "Text"],

Cell[CellGroupData[{
Cell[690979, 16795, 148, 2, 25, "Input"],
Cell[691130, 16799, 54, 1, 36, "Output"]
}, Open  ]],
Cell[691199, 16803, 257, 6, 29, "Text"],
Cell[691459, 16811, 252, 7, 46, "Input"],
Cell[691714, 16820, 252, 7, 46, "Input"],
Cell[691969, 16829, 257, 7, 46, "Input"],
Cell[692229, 16838, 257, 7, 46, "Input"],
Cell[692489, 16847, 252, 7, 46, "Input"],
Cell[692744, 16856, 252, 7, 46, "Input"],
Cell[692999, 16865, 246, 7, 36, "Input"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[693294, 16878, 99, 1, 38, "Section"],
Cell[693396, 16881, 1297, 32, 216, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

