(***********************************************************************

                    Mathematica-Compatible Notebook

This notebook can be used on any computer system with Mathematica 3.0,
MathReader 3.0, or any compatible application. The data for the notebook 
starts with the line of stars above.

To get the notebook into a Mathematica-compatible application, do one of 
the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing the 
word CacheID, otherwise Mathematica-compatible applications may try to 
use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
***********************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    270139,      10370]*)
(*NotebookOutlinePosition[    271940,      10425]*)
(*  CellTagsIndexPosition[    271896,      10421]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["\<\
Chapter 26. 
Building Yield Curves from 
Market Data\
\>", "Title",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  TextAlignment->Left,
  TextJustification->0,
  AspectRatioFixed->True],

Cell["Useful Tools, and a Case Study Using LIBOR and Swap Data", "Subsubtitle",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["26.1 Introduction", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Our purpose in this chapter is to discuss some simple approaches to the \
construction of the yield curve from market data. The first question is - \
What market data are available? The idealized case consists of an ample \
family of zero-coupon Treasury (more carefully, zero-default-risk) bonds, \
with diverse maturities, to which some form of interpolation can be applied.  \
These rarely exist outside the major markets and are usually incomplete. The \
available data usually consist of some cocktail of: \n\n(a) zero-coupon \
bonds;\n(b) short term borrowing data such as LIBOR;\n(c) FRAs;\n(d) futures;\
\n(e) swaps;\n(f) coupon bonds and par bonds in particular.\n\nThe process of \
reconciling such diverse sources of information, with all the real-world \
complications of calendar management and local market conventions for day \
counts, is rather beyond the scope of this text. Most of the tools you need \
are built into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " or its Packages, and what we shall do here is to point you to some \
relevant functions that you may find useful, and then implement the simplest \
possible yield-curve construction technique where we fuse LIBOR data and swap \
data in a combination of boot-strapping, interpolation and extrapolation. \
Although limited in scope, this is a valuable prototype for more complicated \
schemes."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.2 Functions You May Find Useful", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Let's quickly review some very useful functions built in to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Fit\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Fit[data, funs, vars] finds a least-squares fit to a list of data as a \
linear combination of the functions funs of variables vars. The data can have \
the form {{x1, y1, ... , f1}, {x2, y2, ... , f2}, ... }, where the number of \
coordinates x, y, ...  is equal to the number of variables in the list vars. \
The data can also be of the form {f1, f2, ... }, with a single coordinate \
assumed to take values 1, 2, ... . The argument funs can be any list of \
functions that depend only on the objects vars."\)], "Print"]
}, Open  ]],

Cell[TextData[{
  "For example, let's make up some data and use this function. Note that you \
can supply a set of basis functions as a list, and ",
  StyleBox["Fit",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " determines the best ",
  StyleBox["linear",
    FontSlant->"Italic"],
  " combination of these basis functions."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(data\  = \ {{1, \ 2}, \ {2, \ 3}, \ {3, \ 4.5}, \ {4, \ 4.8}, \ {5, \ 
          6}, \ {7, \ 8}}; \nf[x_]\  = \ Fit[data, \ {1, \ x}, \ x]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(1.11285714285714254`\[InvisibleSpace]\) + 
      0.982857142857143273`\ x\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[f[x], \ {x, \ 0, \ 7}, \ PlotRange\  -> \ {0, \ 9}, \ 
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .07642 m
.06244 .09559 L
.10458 .11649 L
.14415 .13612 L
.18221 .155 L
.22272 .17509 L
.26171 .19444 L
.30316 .215 L
.34309 .23481 L
.3815 .25386 L
.42237 .27413 L
.46172 .29366 L
.49955 .31242 L
.53984 .33241 L
.57861 .35164 L
.61984 .37209 L
.65954 .39179 L
.69774 .41074 L
.73838 .4309 L
.77751 .45031 L
.81909 .47094 L
.85916 .49082 L
.89771 .50994 L
.93871 .53028 L
.97619 .54887 L
s
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`80
08Aoo`003Woo00@007oo0000089oo`003Woo00D007ooOomoo`00085oo`003Woo00<007ooOol00Woo
0P00Ogoo000>Ool00`00Oomoo`04Ool2001mOol000ioo`03001oogoo00Ioo`03001oogoo07Yoo`00
1goo10000goo00<007ooOol01goo0P00NWoo0008Ool00`00Oomoo`03Ool00`00Oomoo`09Ool3001g
Ool000Uoo`03001oogoo009oo`8000eoo`@007=oo`002Woo00D007ooOomoo`0000moo`<007=oo`00
1goo00@007ooOol000=oo`03001oogoo00eoo`@0079oo`0027oo0P0017oo00<007ooOol04Goo0`00
Kgoo000>Ool00`00Oomoo`0DOol3001/Ool000ioo`03001oogoo01Moo`8006Yoo`003Woo00<007oo
Ool06Goo0P00J7oo000>Ool00`00Oomoo`0KOol00`00Oomoo`1UOol000ioo`03001oogoo01aoo`80
06Eoo`003Woo00<007ooOol07Woo1000HGoo000>Ool00`00Oomoo`0OOol4001POol000ioo`03001o
ogoo01moo`<00003Ool0000005ioo`003Woo00<007ooOol09Goo00<007ooOol0Fgoo000>Ool00`00
Oomoo`0VOol2001KOol000ioo`03001oogoo02Qoo`<005Qoo`003Woo00<007ooOol0:goo0P00EWoo
0008Ool30003Ool00`00Oomoo`0]Ool2001DOol000Uoo`03001oogoo009oo`03001oogoo02moo`03
001oogoo055oo`001goo10000goo0P00<Goo0P00DGoo0007Ool00`00Ool00004Ool00`00Oomoo`0b
Ool2001?Ool000Moo`03001oo`0000Aoo`03001oogoo03Aoo`03001oogoo04aoo`0027oo0P0017oo
00<007ooOol0=Goo0P00C7oo000>Ool00`00Oomoo`0`Ool30004Ool30019Ool000ioo`03001oogoo
031oo`<000Moo`8004Moo`003Woo00<007ooOol0<7oo0`002Goo0P00AGoo000>Ool00`00Oomoo`0n
Ool20002Ool3000nOol000ioo`03001oogoo041oo`D003ioo`003Woo00<007ooOol0@Woo0`00?Woo
000>Ool00`00Oomoo`15Ool2000lOol000ioo`03001oogoo04Moo`8003Yoo`003Woo00<007ooOol0
BGoo00<007ooOol0=goo000>Ool00`00Oomoo`1:Ool2000gOol000ioo`03001oogoo04aoo`<003Ao
o`003Woo00<007ooOol0Cgoo0P00<Woo0008Ool20004Ool00`00Oomoo`1AOol2000`Ool000Moo`04
001oogoo0003Ool00`00Oomoo`1COol3000]Ool000Moo`04001oogoo0003Ool2001DOol3000]Ool0
00Moo`<000Aoo`03001oogoo05=oo`H002Yoo`001goo00<007ooOol017oo00<007ooOol0FGoo0P00
:7oo0008Ool30003Ool00`00Oomoo`1KOol2000VOol000ioo`03001oogoo05eoo`03001oogoo02=o
o`003Woo00<007ooOol0GWoo0`008Woo000>Ool00`00Oomoo`1QOol3000OOol000ioo`03001oogoo
06Aoo`8001eoo`003Woo00<007ooOol0IWoo0P006goo000>Ool00`00Oomoo`1XOol00`00Oomoo`0H
Ool000ioo`03001oogoo06Uoo`8001Qoo`003Woo00<007ooOol0Jgoo0P005Woo000>Ool00`00Oomo
o`1]Ool00`00Oomoo`0COol000ioo`03001oogoo06ioo`8001=oo`003Woo00<007ooOol0L7oo0`00
47oo000>Ool00`00Oomoo`1cOol2000>Ool000Qoo`8000Aoo`03001oogoo07Eoo`8000aoo`001goo
00@007ooOol000=oo`03001oogoo07Moo`<000Uoo`001goo00@007ooOol000=oo`8007Qoo`<000Uo
o`0027oo0P0017oo00<007ooOol0Mgoo0`002Goo0007Ool01000Oomoo`000goo00<007ooOol0Pgoo
0008Ool20004Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>
Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo0000\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell["We can also interpolate with a built-in function:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Interpolation\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("Interpolation[data] constructs an InterpolatingFunction object which \
represents an approximate function that interpolates the data. The data can \
have the forms {{x1, f1}, {x2, f2}, ... } or {f1, f2, ... }, where in the \
second case, the xi are taken to have values 1, 2, ... ."\)], "Print"]
}, Open  ]],

Cell["Let's do it:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g\  = \ Interpolation[data]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True],

Cell[BoxData[
    TagBox[\(InterpolatingFunction[{{1.`, 7.`}}, "<>"]\),
      False,
      Editable->False]], "Output",
  PageBreakWithin->True,
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[g[x], \ {x, \ 1, \ 7}, \ PlotRange\  -> \ {0, \ 9}, \n\t
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.15986 .13734 m
.16385 .13777 L
.16756 .13829 L
.17598 .13986 L
.1842 .1419 L
.19298 .1446 L
.21189 .15211 L
.2291 .16074 L
.26332 .18204 L
.29625 .20627 L
.33128 .23419 L
.36501 .26136 L
.39744 .28605 L
.41547 .29859 L
.42402 .30413 L
.42819 .30673 L
.43 .30783 L
.43096 .30841 L
.43197 .30901 L
.43302 .30931 L
.434 .30957 L
.43591 .31008 L
.44011 .31113 L
.449 .31315 L
.4652 .31613 L
.49775 .32036 L
.5324 .32411 L
.54837 .32622 L
.5568 .32755 L
.56141 .32835 L
.56575 .32917 L
.5668 .32937 L
.56791 .32959 L
.56849 .32982 L
.56913 .33011 L
.57024 .33061 L
.5752 .33289 L
.58395 .33709 L
.6012 .34605 L
.63597 .36637 L
.66943 .38811 L
.705 .41267 L
.73927 .43688 L
.77224 .45986 L
.80732 .48313 L
.84109 .50355 L
.87357 .52058 L
.89163 .52868 L
.90814 .53509 L
.92541 .54066 L
Mistroke
.94142 .54471 L
.94998 .5464 L
.95773 .54763 L
.96634 .54865 L
.97106 .54906 L
.9755 .54933 L
.97619 .54936 L
Mfstroke
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03
001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=o
o`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000Moo`@000=oo`03001oogoo08=oo`00
27oo00<007ooOol00goo00<007ooOol0Pgoo0009Ool00`00Oomoo`02Ool2000>Ool4001bOol000Yo
o`05001oogooOol0000?Ool300000goo00000002001^Ool000Moo`04001oogoo0003Ool00`00Oomo
o`0=Ool30005Ool2001/Ool000Qoo`8000Aoo`03001oogoo01Moo`8006Yoo`003Woo00<007ooOol0
6Goo0P00J7oo000>Ool00`00Oomoo`0KOol00`00Oomoo`1UOol000ioo`03001oogoo01aoo`8006Eo
o`003Woo00<007ooOol07Woo00<007ooOol0HWoo000>Ool00`00Oomoo`0OOol00`00Oomoo`1QOol0
00ioo`03001oogoo01moo`<0065oo`003Woo00<007ooOol07goo1000H7oo000>Ool00`00Oomoo`0O
Ool300000goo001oo`1NOol000ioo`03001oogoo02Aoo`03001oogoo05aoo`003Woo00<007ooOol0
9Goo0P00G7oo000>Ool00`00Oomoo`0WOol00`00Oomoo`1IOol000ioo`03001oogoo02Qoo`03001o
ogoo05Qoo`0027oo0`000goo00<007ooOol0:Goo00<007ooOol0Egoo0009Ool00`00Oomoo`02Ool0
0`00Oomoo`0ZOol00`00Oomoo`1FOol000Moo`@000=oo`8002aoo`03001oogoo05Eoo`001goo00<0
07oo000017oo00<007ooOol0;7oo00<007ooOol0E7oo0007Ool00`00Ool00004Ool00`00Oomoo`0]
Ool2001DOol000Qoo`8000Aoo`03001oogoo02moo`80059oo`003Woo00<007ooOol0<7oo0`00D7oo
000>Ool00`00Oomoo`0`Ool9001:Ool000ioo`03001oogoo031oo`<000Ioo`P0049oo`003Woo00<0
07ooOol0@Goo1000?Woo000>Ool00`00Oomoo`12Ool6000kOol000ioo`03001oogoo049oo`<000=o
o`8003Uoo`003Woo00<007ooOol0BWoo00<007ooOol0=Woo000>Ool00`00Oomoo`1;Ool00`00Oomo
o`0eOol000ioo`03001oogoo04aoo`8003Eoo`003Woo00<007ooOol0CWoo00<007ooOol0<Woo000>
Ool00`00Oomoo`1?Ool00`00Oomoo`0aOol000ioo`03001oogoo051oo`80035oo`0027oo0P0017oo
00<007ooOol0DWoo0P00;goo0007Ool01000Oomoo`000goo00<007ooOol0Dgoo0`00;Goo0007Ool0
1000Oomoo`000goo0P00E7oo1000;7oo0007Ool30004Ool00`00Oomoo`1COol300000goo001oo`0Z
Ool000Moo`03001oogoo00Aoo`03001oogoo05Qoo`03001oogoo02Qoo`0027oo0`000goo00<007oo
Ool0FGoo0P00:7oo000>Ool00`00Oomoo`1KOol2000VOol000ioo`03001oogoo05eoo`03001oogoo
02=oo`003Woo00<007ooOol0GWoo0P008goo000>Ool00`00Oomoo`1POol00`00Oomoo`0POol000io
o`03001oogoo065oo`03001oogoo01moo`003Woo00<007ooOol0HWoo0P007goo000>Ool00`00Oomo
o`1TOol2000MOol000ioo`03001oogoo06Ioo`03001oogoo01Yoo`003Woo00<007ooOol0Igoo0P00
6Woo000>Ool00`00Oomoo`1YOol2000HOol000ioo`03001oogoo06]oo`8001Ioo`003Woo00<007oo
Ool0KGoo0P0057oo0008Ool20004Ool00`00Oomoo`1_Ool5000?Ool000Moo`04001oogoo0003Ool0
0`00Oomoo`1dOol60009Ool000Moo`04001oogoo0003Ool2001hOol30009Ool000Qoo`8000Aoo`03
001oogoo07Moo`<000Uoo`001goo00@007ooOol000=oo`03001oogoo08=oo`0027oo0P0017oo00<0
07ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo
000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell[TextData[{
  StyleBox["Interpolation",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " has an interesting option:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[Interpolation]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({InterpolationOrder \[Rule] 3}\)], "Output"]
}, Open  ]],

Cell["\<\
Let's look at the effect of varying it. Linear interpolation is \
order 1:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g\  = \ Interpolation[data, \ InterpolationOrder\  -> 1]; \n
    Plot[g[x], \ {x, \ 1, \ 7}, \ PlotRange\  -> \ {0, \ 9}, \ \n\t
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.15986 .13734 m
.19298 .15406 L
.2291 .17228 L
.26301 .1894 L
.28009 .19802 L
.28755 .20179 L
.29147 .20377 L
.29362 .20485 L
.29563 .20587 L
.29666 .20657 L
.29777 .20741 L
.30012 .20919 L
.30426 .21233 L
.31241 .2185 L
.33036 .23208 L
.36569 .25884 L
.39973 .2846 L
.4169 .2976 L
.42437 .30326 L
.42831 .30624 L
.43046 .30787 L
.43151 .30866 L
.43246 .30909 L
.4335 .30925 L
.43462 .30942 L
.43699 .30978 L
.44115 .31041 L
.44933 .31164 L
.4673 .31437 L
.50146 .31954 L
.53431 .32451 L
.55093 .32703 L
.55975 .32837 L
.56439 .32907 L
.56672 .32942 L
.56736 .32952 L
.56804 .32962 L
.56868 .33001 L
.56927 .33037 L
.57832 .33585 L
.58663 .34088 L
.60292 .35075 L
.63719 .37151 L
.67016 .39148 L
.70524 .41261 L
.73901 .42965 L
.77149 .44604 L
.80606 .4635 L
.83934 .48029 L
.87472 .49815 L
Mistroke
.9088 .51535 L
.94158 .5319 L
.97619 .54936 L
Mfstroke
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03
001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=o
o`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000Moo`@000=oo`03001oogoo08=oo`00
27oo00<007ooOol00goo00<007ooOol0Pgoo0009Ool00`00Oomoo`02Ool2000>Ool3001cOol000Yo
o`05001oogooOol0000?Ool6001`Ool000Moo`04001oogoo0003Ool00`00Oomoo`0=Ool30003Ool2
001^Ool000Qoo`8000Aoo`03001oogoo01Eoo`03001oogoo06]oo`003Woo00<007ooOol05Woo00<0
07ooOol0JWoo000>Ool00`00Oomoo`0GOol2001ZOol000ioo`03001oogoo01Uoo`<006Moo`003Woo
00<007ooOol077oo0P00IGoo000>Ool00`00Oomoo`0NOol00`00Oomoo`1ROol000ioo`03001oogoo
01moo`<0065oo`003Woo00<007ooOol07goo0`00HGoo000>Ool00`00Oomoo`0OOol4001POol000io
o`03001oogoo02=oo`03001oogoo05eoo`003Woo00<007ooOol097oo0P00GGoo000>Ool00`00Oomo
o`0VOol2001KOol000ioo`03001oogoo02Qoo`03001oogoo05Qoo`0027oo0`000goo00<007ooOol0
:Goo0P00F7oo0009Ool00`00Oomoo`02Ool00`00Oomoo`0[Ool00`00Oomoo`1EOol000Moo`@000=o
o`8002eoo`03001oogoo05Aoo`001goo00<007oo000017oo00<007ooOol0;Goo0P00E7oo0007Ool0
0`00Ool00004Ool00`00Oomoo`0_Ool00`00Oomoo`1AOol000Qoo`8000Aoo`03001oogoo031oo`03
001oogoo051oo`003Woo00<007ooOol0<7oo0`00D7oo000>Ool00`00Oomoo`0`Ool:0019Ool000io
o`03001oogoo031oo`<000Moo`@004Eoo`003Woo00<007ooOol0?Woo1`00?Woo000>Ool00`00Oomo
o`12Ool4000mOol000ioo`03001oogoo049oo`<00003Ool0000003]oo`003Woo00<007ooOol0B7oo
0P00>Goo000>Ool00`00Oomoo`1:Ool00`00Oomoo`0fOol000ioo`03001oogoo04]oo`03001oogoo
03Eoo`003Woo00<007ooOol0C7oo0P00=Goo000>Ool00`00Oomoo`1>Ool2000cOol000ioo`03001o
ogoo051oo`80035oo`0027oo0P0017oo00<007ooOol0DWoo0P00;goo0007Ool01000Oomoo`000goo
00<007ooOol0Dgoo0`00;Goo0007Ool01000Oomoo`000goo0P00E7oo1000;7oo0007Ool30004Ool0
0`00Oomoo`1COol300000goo0000000ZOol000Moo`03001oogoo00Aoo`03001oogoo05Uoo`8002Qo
o`0027oo0`000goo00<007ooOol0Fgoo0`009Goo000>Ool00`00Oomoo`1NOol2000SOol000ioo`03
001oogoo061oo`03001oogoo021oo`003Woo00<007ooOol0HGoo00<007ooOol07goo000>Ool00`00
Oomoo`1ROol2000OOol000ioo`03001oogoo06Aoo`8001eoo`003Woo00<007ooOol0IWoo0P006goo
000>Ool00`00Oomoo`1XOol3000HOol000ioo`03001oogoo06]oo`8001Ioo`003Woo00<007ooOol0
KGoo0P0057oo000>Ool00`00Oomoo`1_Ool2000BOol000ioo`03001oogoo075oo`<000moo`003Woo
00<007ooOol0M7oo0P003Goo0008Ool20004Ool00`00Oomoo`1fOol00`00Oomoo`0:Ool000Moo`04
001oogoo0003Ool00`00Oomoo`1gOol30009Ool000Moo`04001oogoo0003Ool2001hOol30009Ool0
00Qoo`8000Aoo`03001oogoo07Moo`<000Uoo`001goo00@007ooOol000=oo`03001oogoo08=oo`00
27oo0P0017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo
00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell["Piecewise constant is order zero:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g\  = \ Interpolation[data, \ InterpolationOrder\  -> 0]; \n
    Plot[g[x], \ {x, \ 1, \ 7}, \ PlotRange\  -> \ {0, \ 9}, \ \n\t
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.15986 .20601 m
.19298 .20601 L
.2291 .20601 L
.26301 .20601 L
.28009 .20601 L
.28755 .20601 L
.29147 .20601 L
.29362 .20601 L
.29467 .20601 L
.29563 .20601 L
.29666 .30902 L
.29777 .30902 L
.30012 .30902 L
.30426 .30902 L
.31241 .30902 L
.33036 .30902 L
.36439 .30902 L
.39713 .30902 L
.4154 .30902 L
.42403 .30902 L
.42781 .30902 L
.42996 .30902 L
.43101 .30902 L
.43197 .30902 L
.43297 .32962 L
.43391 .32962 L
.436 .32962 L
.44026 .32962 L
.44924 .32962 L
.46551 .32962 L
.49836 .32962 L
.53332 .32962 L
.55068 .32962 L
.55918 .32962 L
.5629 .32962 L
.56487 .32962 L
.56595 .32962 L
.56697 .32962 L
.56796 .32962 L
.56903 .41202 L
.57001 .41202 L
.5709 .41202 L
.57456 .41202 L
.58255 .41202 L
.59933 .41202 L
.63311 .41202 L
.66558 .41202 L
.68365 .41202 L
.69222 .41202 L
.69638 .41202 L
Mistroke
.70016 .41202 L
.70114 .41202 L
.7022 .41202 L
.7032 .41202 L
.70411 .54936 L
.70509 .54936 L
.70616 .54936 L
.70833 .54936 L
.71723 .54936 L
.73344 .54936 L
.76813 .54936 L
.80152 .54936 L
.83702 .54936 L
.87122 .54936 L
.90412 .54936 L
.93912 .54936 L
.97282 .54936 L
.97619 .54936 L
Mfstroke
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03
001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=o
o`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000Moo`@000=oo`03001oogoo08=oo`00
27oo00<007ooOol00goo00<007ooOol0Pgoo0009Ool00`00Oomoo`02Ool2000>Ool3001cOol000Yo
o`05001oogooOol0000?Ool3001cOol000Moo`04001oogoo0003Ool00`00Oomoo`0=Ool3001cOol0
00Qoo`8000Aoo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000io
o`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo
00iooa@0065oo`003Woo00<007ooOol07goo0`00HGoo000>Ool00`00Oomoo`0OOol3001QOol000io
o`03001oogoo021oo`03001oogoo061oo`003Woo00<007ooOol087oo00<007ooOol0H7oo000>Ool0
0`00Oomoo`0POol00`00Oomoo`1POol000ioo`03001oogoo021oo`03001oogoo061oo`0027oo0`00
0goo00<007ooOol087oo00<007ooOol0H7oo0009Ool00`00Oomoo`02Ool00`00Oomoo`0POol00`00
Oomoo`1POol000Moo`@000=oo`80025oo`03001oogoo061oo`001goo00<007oo000017oo00<007oo
Ool087oo00<007ooOol0H7oo0007Ool00`00Ool00004Ool00`00Oomoo`0POol00`00Oomoo`1POol0
00Qoo`8000Aoo`03001oogoo021oo`03001oogoo061oo`003Woo00<007ooOol087oo00<007ooOol0
3Goo0`00D7oo000>Ool00`00Oomoo`0POolC001@Ool000ioo`03001oogoo031oo`<0051oo`003Woo
00<007ooOol0<Woo4`00?Woo000>Ool00`00Oomoo`12Ool3000nOol000ioo`03001oogoo049oo`<0
03ioo`003Woo00<007ooOol0@goo00<007ooOol0?Goo000>Ool00`00Oomoo`13Ool00`00Oomoo`0m
Ool000ioo`03001oogoo04=oo`03001oogoo03eoo`003Woo00<007ooOol0@goo00<007ooOol0?Goo
000>Ool00`00Oomoo`13Ool00`00Oomoo`0mOol000ioo`03001oogoo04=oo`03001oogoo03eoo`00
27oo0P0017oo00<007ooOol0@goo00<007ooOol0?Goo0007Ool01000Oomoo`000goo00<007ooOol0
@goo00<007ooOol03Goo0`00;Goo0007Ool01000Oomoo`000goo0P00A7oo4`00;Goo0007Ool30004
Ool00`00Oomoo`1COol3000]Ool000Moo`03001oogoo00Aoo`03001oogoo05Eoo`03001oogoo02]o
o`0027oo0`000goo00<007ooOol0EGoo00<007ooOol0:goo000>Ool00`00Oomoo`1EOol00`00Oomo
o`0[Ool000ioo`03001oogoo05Eoo`03001oogoo02]oo`003Woo00<007ooOol0EGoo00<007ooOol0
:goo000>Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool000ioo`03001oogoo05Eoo`03001oogoo02]o
o`003Woo00<007ooOol0EGoo00<007ooOol0:goo000>Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool0
00ioo`03001oogoo05Eoo`03001oogoo02]oo`003Woo00<007ooOol0EGoo00<007ooOol0:goo000>
Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool000ioo`03001oogoo05Eoo`03001oogoo02]oo`003Woo
00<007ooOol0EGoo00<007ooOol0:goo0008Ool20004Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool0
00Moo`04001oogoo0003Ool00`00Oomoo`1EOol00`00Oomoo`0OOol30009Ool000Moo`04001oogoo
0003Ool2001FOolU0009Ool000Qoo`8000Aoo`03001oogoo07Moo`<000Uoo`001goo00@007ooOol0
00=oo`03001oogoo08=oo`0027oo0P0017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000io
o`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo
08=oo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell[TextData[{
  "If you want it the other way round (and this may be the default in \
different versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "), it is easily done: "
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(datafix\  = \ 
      Table[{\(Last[data]\)[\([1]\)]\  - \ data[\([i, 1]\)], \ 
          data[\([i, \ 2]\)]}, \ {i, \ Length[data]}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g\  = \ Interpolation[datafix, \ InterpolationOrder\  -> 0]; \n
    Plot[g[\(Last[data]\)[\([1]\)] - x], \ {x, \ 1, \ 7}, \ 
      PlotRange\  -> \ {0, \ 9}, \ \n\t
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.15986 .13734 m
.19298 .13734 L
.2291 .13734 L
.26301 .13734 L
.28009 .13734 L
.28755 .13734 L
.29147 .13734 L
.29362 .13734 L
.29467 .13734 L
.29563 .13734 L
.29666 .20601 L
.29777 .20601 L
.30012 .20601 L
.30426 .20601 L
.31241 .20601 L
.33036 .20601 L
.36439 .20601 L
.39713 .20601 L
.4154 .20601 L
.42403 .20601 L
.42781 .20601 L
.42996 .20601 L
.43101 .20601 L
.43197 .20601 L
.43297 .30902 L
.43391 .30902 L
.436 .30902 L
.44026 .30902 L
.44924 .30902 L
.46551 .30902 L
.49836 .30902 L
.53332 .30902 L
.55068 .30902 L
.55918 .30902 L
.5629 .30902 L
.56487 .30902 L
.56595 .30902 L
.56697 .30902 L
.56796 .30902 L
.56903 .32962 L
.57001 .32962 L
.5709 .32962 L
.57456 .32962 L
.58255 .32962 L
.59933 .32962 L
.63311 .32962 L
.66558 .32962 L
.68365 .32962 L
.69222 .32962 L
.69638 .32962 L
Mistroke
.70016 .32962 L
.70114 .32962 L
.7022 .32962 L
.7032 .32962 L
.70411 .41202 L
.70509 .41202 L
.70616 .41202 L
.70833 .41202 L
.71723 .41202 L
.73344 .41202 L
.76813 .41202 L
.80152 .41202 L
.83702 .41202 L
.87122 .41202 L
.90412 .41202 L
.93912 .41202 L
.97282 .41202 L
.97619 .41202 L
Mfstroke
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03
001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=o
o`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000Moo`@000=oo`03001oogoo08=oo`00
27oo00<007ooOol00goo00<007ooOol0Pgoo0009Ool00`00Oomoo`02Ool2000>OolD001ROol000Yo
o`05001oogooOol0000?Ool3000@Ool00`00Oomoo`1POol000Moo`04001oogoo0003Ool00`00Oomo
o`0=Ool3000@Ool00`00Oomoo`1POol000Qoo`8000Aoo`03001oogoo021oo`03001oogoo061oo`00
3Woo00<007ooOol087oo00<007ooOol0H7oo000>Ool00`00Oomoo`0POol00`00Oomoo`1POol000io
o`03001oogoo021oo`03001oogoo061oo`003Woo00<007ooOol087oo00<007ooOol0H7oo000>Ool0
0`00Oomoo`0POol00`00Oomoo`1POol000ioo`03001oogoo01mooa@0051oo`003Woo00<007ooOol0
7goo0`0047oo00<007ooOol0CWoo000>Ool00`00Oomoo`0OOol3000@Ool00`00Oomoo`1>Ool000io
o`03001oogoo039oo`03001oogoo04ioo`003Woo00<007ooOol0<Woo00<007ooOol0CWoo000>Ool0
0`00Oomoo`0bOol00`00Oomoo`1>Ool000ioo`03001oogoo039oo`03001oogoo04ioo`0027oo0`00
0goo00<007ooOol0<Woo00<007ooOol0CWoo0009Ool00`00Oomoo`02Ool00`00Oomoo`0bOol00`00
Oomoo`1>Ool000Moo`@000=oo`8003=oo`03001oogoo04ioo`001goo00<007oo000017oo00<007oo
Ool0<Woo00<007ooOol0CWoo0007Ool00`00Ool00004Ool00`00Oomoo`0bOol00`00Oomoo`1>Ool0
00Qoo`8000Aoo`03001oogoo039oo`03001oogoo04ioo`003Woo00<007ooOol0<7oo0`00D7oo000>
Ool00`00Oomoo`0`OolD000oOol000ioo`03001oogoo031oo`<0011oo`03001oogoo03eoo`003Woo
00<007ooOol0@Woo5000;Goo000>Ool00`00Oomoo`12Ool3000@Ool00`00Oomoo`0[Ool000ioo`03
001oogoo049oo`<0011oo`03001oogoo02]oo`003Woo00<007ooOol0EGoo00<007ooOol0:goo000>
Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool000ioo`03001oogoo05Eoo`03001oogoo02]oo`003Woo
00<007ooOol0EGoo00<007ooOol0:goo000>Ool00`00Oomoo`1EOol00`00Oomoo`0[Ool000ioo`03
001oogoo05Eoo`03001oogoo02]oo`0027oo0P0017oo00<007ooOol0EGoo00<007ooOol0:goo0007
Ool01000Oomoo`000goo00<007ooOol0Dgoo0`00;Goo0007Ool01000Oomoo`000goo0P00E7oo9P00
2Woo0007Ool30004Ool00`00Oomoo`1COol3000]Ool000Moo`03001oogoo00Aoo`03001oogoo08=o
o`0027oo0`000goo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`00
3Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007oo
Ool0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>
Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo0008Ool20004Ool0
0`00Oomoo`23Ool000Moo`04001oogoo0003Ool00`00Oomoo`1gOol30009Ool000Moo`04001oogoo
0003Ool2001hOol30009Ool000Qoo`8000Aoo`03001oogoo07Moo`<000Uoo`001goo00@007ooOol0
00=oo`03001oogoo08=oo`0027oo0P0017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000io
o`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo
08=oo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell["Quadratic interpolation, not surprisingly, is order 2:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g\  = \ Interpolation[data, \ InterpolationOrder\  -> 2]; \n
    Plot[g[x], \ {x, \ 1, \ 7}, \ PlotRange\  -> \ {0, \ 9}, \ \n\t
      Epilog\  -> \ {PointSize[0.03], \ Map[Point, \ data]}]; \)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.136054 0 0.0686704 [
[.15986 -0.0125 -3 -9 ]
[.15986 -0.0125 3 0 ]
[.29592 -0.0125 -3 -9 ]
[.29592 -0.0125 3 0 ]
[.43197 -0.0125 -3 -9 ]
[.43197 -0.0125 3 0 ]
[.56803 -0.0125 -3 -9 ]
[.56803 -0.0125 3 0 ]
[.70408 -0.0125 -3 -9 ]
[.70408 -0.0125 3 0 ]
[.84014 -0.0125 -3 -9 ]
[.84014 -0.0125 3 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.01131 .13734 -6 -4.5 ]
[.01131 .13734 0 4.5 ]
[.01131 .27468 -6 -4.5 ]
[.01131 .27468 0 4.5 ]
[.01131 .41202 -6 -4.5 ]
[.01131 .41202 0 4.5 ]
[.01131 .54936 -6 -4.5 ]
[.01131 .54936 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 0 m
.15986 .00625 L
s
[(1)] .15986 -0.0125 0 1 Mshowa
.29592 0 m
.29592 .00625 L
s
[(2)] .29592 -0.0125 0 1 Mshowa
.43197 0 m
.43197 .00625 L
s
[(3)] .43197 -0.0125 0 1 Mshowa
.56803 0 m
.56803 .00625 L
s
[(4)] .56803 -0.0125 0 1 Mshowa
.70408 0 m
.70408 .00625 L
s
[(5)] .70408 -0.0125 0 1 Mshowa
.84014 0 m
.84014 .00625 L
s
[(6)] .84014 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(7)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.05102 0 m
.05102 .00375 L
s
.07823 0 m
.07823 .00375 L
s
.10544 0 m
.10544 .00375 L
s
.13265 0 m
.13265 .00375 L
s
.18707 0 m
.18707 .00375 L
s
.21429 0 m
.21429 .00375 L
s
.2415 0 m
.2415 .00375 L
s
.26871 0 m
.26871 .00375 L
s
.32313 0 m
.32313 .00375 L
s
.35034 0 m
.35034 .00375 L
s
.37755 0 m
.37755 .00375 L
s
.40476 0 m
.40476 .00375 L
s
.45918 0 m
.45918 .00375 L
s
.48639 0 m
.48639 .00375 L
s
.51361 0 m
.51361 .00375 L
s
.54082 0 m
.54082 .00375 L
s
.59524 0 m
.59524 .00375 L
s
.62245 0 m
.62245 .00375 L
s
.64966 0 m
.64966 .00375 L
s
.67687 0 m
.67687 .00375 L
s
.73129 0 m
.73129 .00375 L
s
.7585 0 m
.7585 .00375 L
s
.78571 0 m
.78571 .00375 L
s
.81293 0 m
.81293 .00375 L
s
.86735 0 m
.86735 .00375 L
s
.89456 0 m
.89456 .00375 L
s
.92177 0 m
.92177 .00375 L
s
.94898 0 m
.94898 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02381 .13734 m
.03006 .13734 L
s
[(2)] .01131 .13734 1 0 Mshowa
.02381 .27468 m
.03006 .27468 L
s
[(4)] .01131 .27468 1 0 Mshowa
.02381 .41202 m
.03006 .41202 L
s
[(6)] .01131 .41202 1 0 Mshowa
.02381 .54936 m
.03006 .54936 L
s
[(8)] .01131 .54936 1 0 Mshowa
.125 Mabswid
.02381 .03434 m
.02756 .03434 L
s
.02381 .06867 m
.02756 .06867 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .17168 m
.02756 .17168 L
s
.02381 .20601 m
.02756 .20601 L
s
.02381 .24035 m
.02756 .24035 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .34335 m
.02756 .34335 L
s
.02381 .37769 m
.02756 .37769 L
s
.02381 .44636 m
.02756 .44636 L
s
.02381 .48069 m
.02756 .48069 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .5837 m
.02756 .5837 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.15986 .13734 m
.19298 .15089 L
.2291 .16799 L
.26301 .18626 L
.28009 .19626 L
.28755 .20079 L
.29147 .20322 L
.29362 .20457 L
.29467 .20523 L
.29563 .20583 L
.29666 .2068 L
.29777 .20796 L
.30012 .21042 L
.30426 .2147 L
.31241 .22289 L
.33036 .23987 L
.36439 .26815 L
.39713 .29049 L
.4154 .30088 L
.42403 .30527 L
.42781 .30709 L
.42996 .30809 L
.43101 .30858 L
.43197 .30901 L
.43297 .30894 L
.43391 .30888 L
.436 .30874 L
.44026 .3085 L
.44456 .30833 L
.44699 .30826 L
.44807 .30823 L
.44924 .30821 L
.45025 .30819 L
.45117 .30818 L
.45206 .30817 L
.45299 .30816 L
.45401 .30816 L
.45496 .30816 L
.45601 .30816 L
.45713 .30817 L
.45821 .30818 L
.45939 .3082 L
.46152 .30824 L
.46359 .30829 L
.46551 .30836 L
.46951 .30853 L
.47382 .30877 L
.48159 .30937 L
.49083 .31034 L
.49928 .31148 L
Mistroke
.51559 .31436 L
.53336 .3185 L
.56825 .3297 L
.60183 .34432 L
.63752 .36398 L
.6719 .38695 L
.70499 .41254 L
.74018 .43235 L
.77407 .45085 L
.80667 .4681 L
.84136 .48589 L
.87476 .50245 L
.91025 .51944 L
.94445 .53523 L
.97619 .54936 L
Mfstroke
.03 w
.15986 .13734 Mdot
.29592 .20601 Mdot
.43197 .30902 Mdot
.56803 .32962 Mdot
.70408 .41202 Mdot
.97619 .54936 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol001eo
o`<000moo`@000ioo`<000moo`<000ioo`<000moo`80011oo`03001oogoo00Uoo`007Woo00<007oo
Ool03goo00<007ooOol04Goo00<007ooOol03Goo00<007ooOol047oo00<007ooOol02goo00@007oo
Ool000moo`03001oogoo00Uoo`007Woo00<007ooOol047oo00<007ooOol047oo00<007ooOol02goo
10004Goo00<007ooOol02goo00@007ooOol0011oo`03001oogoo00Qoo`007Woo00<007ooOol04Goo
00<007ooOol03Goo0P003Woo00<007oo00003goo0`003Woo0`004Goo00<007ooOol027oo000NOol0
0`00Oomoo`0>Ool01000Oomoo`004Goo00<007ooOol02goo00<007oo00003goo00<007ooOol03Woo
00<007ooOol04Woo00<007ooOol01goo000MOol2000AOol2000?Ool3000?Ool2000?Ool4000>Ool3
000>Ool40009Ool009Aoo`00U7oo002DOol000]ooh8000Moo`003Woo00<007ooOol0Pgoo000>Ool0
0`00Oomoo`23Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23
Ool000ioo`03001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03
001oogoo08=oo`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=o
o`003Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000Moo`@000=oo`03001oogoo08=oo`00
27oo00<007ooOol00goo00<007ooOol0Pgoo0009Ool00`00Oomoo`02Ool2000>Ool3001cOol000Yo
o`05001oogooOol0000?Ool6001`Ool000Moo`04001oogoo0003Ool00`00Oomoo`0=Ool30003Ool2
001^Ool000Qoo`8000Aoo`03001oogoo01Eoo`8006aoo`003Woo00<007ooOol05goo0P00JWoo000>
Ool00`00Oomoo`0IOol2001XOol000ioo`03001oogoo01]oo`03001oogoo06Eoo`003Woo00<007oo
Ool077oo0P00IGoo000>Ool00`00Oomoo`0NOol00`00Oomoo`1ROol000ioo`03001oogoo01moo`<0
065oo`003Woo00<007ooOol07goo0`00HGoo000>Ool00`00Oomoo`0OOol4001POol000ioo`03001o
ogoo029oo`03001oogoo05ioo`003Woo00<007ooOol08goo00<007ooOol0GGoo000>Ool00`00Oomo
o`0TOol2001MOol000ioo`03001oogoo02Ioo`8005]oo`0027oo0`000goo00<007ooOol0:7oo00<0
07ooOol0F7oo0009Ool00`00Oomoo`02Ool00`00Oomoo`0YOol2001HOol000Moo`@000=oo`8002ao
o`03001oogoo05Eoo`001goo00<007oo000017oo00<007ooOol0;7oo00<007ooOol0E7oo0007Ool0
0`00Ool00004Ool00`00Oomoo`0]Ool2001DOol000Qoo`8000Aoo`03001oogoo02moo`80059oo`00
3Woo00<007ooOol0<7oo2000Bgoo000>Ool00`00Oomoo`0`Ool30005Ool70014Ool000ioo`03001o
ogoo031oo`<000aoo`<0045oo`003Woo00<007ooOol0@Woo0`00?Woo000>Ool00`00Oomoo`12Ool6
000kOol000ioo`03001oogoo049oo`<000=oo`8003Uoo`003Woo00<007ooOol0BWoo00<007ooOol0
=Woo000>Ool00`00Oomoo`1;Ool00`00Oomoo`0eOol000ioo`03001oogoo04aoo`8003Eoo`003Woo
00<007ooOol0CWoo0P00<goo000>Ool00`00Oomoo`1@Ool00`00Oomoo`0`Ool000ioo`03001oogoo
055oo`80031oo`0027oo0P0017oo00<007ooOol0Dgoo00<007ooOol0;Goo0007Ool01000Oomoo`00
0goo00<007ooOol0Dgoo0`00;Goo0007Ool01000Oomoo`000goo0P00E7oo1000;7oo0007Ool30004
Ool00`00Oomoo`1COol300000goo0000000ZOol000Moo`03001oogoo00Aoo`03001oogoo05Uoo`80
02Qoo`0027oo0`000goo00<007ooOol0Fgoo0P009Woo000>Ool00`00Oomoo`1MOol00`00Oomoo`0S
Ool000ioo`03001oogoo05ioo`8002=oo`003Woo00<007ooOol0H7oo0P008Goo000>Ool00`00Oomo
o`1ROol2000OOol000ioo`03001oogoo06Aoo`<001aoo`003Woo00<007ooOol0Igoo0P006Woo000>
Ool00`00Oomoo`1YOol00`00Oomoo`0GOol000ioo`03001oogoo06Yoo`03001oogoo01Ioo`003Woo
00<007ooOol0Jgoo0P005Woo000>Ool00`00Oomoo`1]Ool2000DOol000ioo`03001oogoo06moo`80
019oo`003Woo00<007ooOol0LGoo0`003goo0008Ool20004Ool00`00Oomoo`1dOol2000=Ool000Mo
o`04001oogoo0003Ool00`00Oomoo`1fOol40009Ool000Moo`04001oogoo0003Ool2001hOol30009
Ool000Qoo`8000Aoo`03001oogoo07Moo`<000Uoo`001goo00@007ooOol000=oo`03001oogoo08=o
o`0027oo0P0017oo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`00
3Woo00<007ooOol0Pgoo000>Ool00`00Oomoo`23Ool000ioo`03001oogoo08=oo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.814103, -1.19527, 
  0.0568209, 0.112577}}]
}, Open  ]],

Cell[TextData[{
  "Other types of fitting are also possible. We saw how to implement a robust \
",
  Cell[BoxData[
      \(TraditionalForm\`L\^1\)]],
  " fit in Chapter 17. Frequently, the fitting process may involve a model \
non-linear in the unknown parameters. There is a Package to do this:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Statistics`NonlinearFit`\>"]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?NonlinearFit\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \("NonlinearFit[data, model, vars, params, (opts)] searches for a \
least-squares  fit to a list of data according to the model containing the \
variables vars and the parameters params.  Parameters may be expressed as a \
list of symbols  or a list of lists, where each parameter is listed with \
starting value(s) and bounds in one of several different ways: {symbol, \
start} or {symbol, min, max} or {symbol, {start1, start2}} or {symbol, \
{start1, start2}, min, max}.  The data can have the form {{x1, y1, ..., f1}, \
{x2, y2, ..., f2}, ...},  where the number of coordinates x, y, ... is equal \
to the number of variables in the list vars.  The data can also be of the \
form {f1, f2, ...}, with a single coordinate assumed to take values 1, 2, \
....  Method specifies the LevenbergMarquardt or FindMinimum search methods.  \
AccuracyGoal and PrecisionGoal specify the number of digits of absolute and \
relative error allowed in the residual sum of squares."\)], "Print"]
}, Open  ]],

Cell[BoxData[
    \(\(exptdata\  = \ 
      Table[{x, \ 
          1/\((\((x - 10)\)^2\  + \ 0.1)\)\  + \ 
            1/\((\((x - 13)\)^2\  + \ 0.5)\)\  + \ 0.3\ *Random[]}, \ {x, \ 
          0, \ 20, \ 0.1}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[NonlinearFit]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({AccuracyGoal \[Rule] Automatic, Gradient \[Rule] Automatic, 
      MaxIterations \[Rule] 30, Method \[Rule] LevenbergMarquardt, 
      PrecisionGoal \[Rule] Automatic, ShowProgress \[Rule] False, 
      Tolerance \[Rule] Automatic, Weights \[Rule] Equal, 
      WorkingPrecision \[Rule] 16}\)], "Output"]
}, Open  ]],

Cell["Clear[a,b,c,d,e,f,g]", "Input"],

Cell[CellGroupData[{

Cell[BoxData[
    \(NonlinearFit[exptdata, 
      a\  + \ b/\((\((x - c)\)^2\  + \ d)\)\  + \ 
        e/\((\((x - f)\)^2\  + \ g)\), \ 
      x, \ {a, b, {c, 11}, d, \ e, \ {f, 14}, \ g}, \ 
      MaxIterations\  -> \ 40]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({a \[Rule] 0.150672795848386265`, b \[Rule] 1.00533240560062142`, 
      c \[Rule] 10.0003787609715999`, d \[Rule] 0.100411783877250582`, 
      e \[Rule] 1.00294010528055199`, f \[Rule] 12.9976858875142408`, 
      g \[Rule] 0.493341125657965129`}\)], "Output",
  CellMargins->{{42, Inherited}, {Inherited, Inherited}}]
}, Open  ]],

Cell[TextData[{
  "Note that some early releases of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3.0 contained a bug in this package - if it does not work essentially as \
shown you can download a working version from ",
  StyleBox["MathSource,",
    FontSlant->"Italic"],
  " at http://www.wolfram.com on the World Wide Web. This of course is a good \
place to visit anyway! For a more extensive discussion of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "'s data modelling functions, see Shaw and Tigg (1993), for collections of \
data import and analysis routines, as well as techniques for time series \
analysis, robust regression, and two and three-dimensional graphics. You \
should also look at the calendar Package, which can be loaded with"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(Needs["\<Miscellaneous`Calendar`\>"]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "and contains several useful functions - ",
  StyleBox["DaysBetween",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " is particularly useful as a simple converter from dates to time."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?Miscellaneous`Calendar`*\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False],

Cell["\<\
Calendar                  EasterSundayGreekOrthodox Monday
CalendarChange            Friday                    Saturday
DayOfWeek                 Gregorian                 Sunday
DaysBetween               Islamic                   Thursday
DaysPlus                  JewishNewYear             Tuesday
EasterSunday              Julian                    Wednesday\
\>", "Print",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.3 Building the Yield Curve from LIBOR and Swap Data ", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
In this section we will take raw market data for LIBOR and swap \
data and extract various associated measures of the yield curve. In each step \
we have taken the simplest approach possible, in order to ensure that the \
overall logic is correct, in order to illustrate the general principles \
without getting involved in detail. No account of day count conventions is \
taken, for example.

It is perfectly possible to replace parts of this calculation by more \
sophisticated versions. For example, a more sophisticated interpolation of \
swap rates could be used. We could also use a combination of LIBOR, futures \
and swaps to model short-, medium- and longer-term information in the term \
structure. An excellent and explicit discussion of curve construction is \
given by Miron and Swanell (1991). 

Note also that choices have to be made about at least two sets of issues:

(1) are the swap data based on swaps paid annually or semi-annually?
(2) are interest-rate data based on continuous compounding or an annual \
representation?\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Plan of Attack", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  PageBreakAbove->False,
  GroupPageBreakWithin->True,
  AspectRatioFixed->True],

Cell["\<\
We develop a six-stage analysis program:

(1) interpolate raw swap data to annual or semi-annual values;
(2) build discount factors for short term using LIBOR data;
(3) create dummy swap values for cross-over domain based on LIBOR data;
(4) bootstrap with real and dummy swap data to get medium- and long-term \
discount factors;
(5) create single unified discount factor structure;
(6) build yields and forward rates for interest rates.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakWithin->True,
  GroupPageBreakWithin->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.4 Swap Data Interpolation", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  PageBreakAbove->True],

Cell["\<\
The swap data are typically only available for certain lifetimes, \
so we interpolate to generate intermediate values, depending on whether the \
swap is annual or semi-annual. In our working example we invent data for 2, \
3, 4, 5, 7, and 10 years.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(rawswapdata\  = {{2, \ 10.4}, \ {3, \ 10.25}, \ {4, \ 10.15}, \ {5, \ 
          10.15}, \ {7, \ 9.8}, \ {10, \ 9.7}}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->3]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->4]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->4]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->4]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(rawplot\  = \ 
      ListPlot[rawswapdata, \ PlotStyle\  -> \ PointSize[0.03], \ 
        PlotRange\  -> \ {{0, \ 11}, \ {9, \ 11}}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -2.78115 0.309017 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .07725 -24 -4.5 ]
[-0.0125 .07725 0 4.5 ]
[-0.0125 .15451 -18 -4.5 ]
[-0.0125 .15451 0 4.5 ]
[-0.0125 .23176 -24 -4.5 ]
[-0.0125 .23176 0 4.5 ]
[-0.0125 .30902 -12 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .38627 -30 -4.5 ]
[-0.0125 .38627 0 4.5 ]
[-0.0125 .46353 -24 -4.5 ]
[-0.0125 .46353 0 4.5 ]
[-0.0125 .54078 -30 -4.5 ]
[-0.0125 .54078 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07725 m
.00625 .07725 L
s
[(9.25)] -0.0125 .07725 1 0 Mshowa
0 .15451 m
.00625 .15451 L
s
[(9.5)] -0.0125 .15451 1 0 Mshowa
0 .23176 m
.00625 .23176 L
s
[(9.75)] -0.0125 .23176 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(10)] -0.0125 .30902 1 0 Mshowa
0 .38627 m
.00625 .38627 L
s
[(10.25)] -0.0125 .38627 1 0 Mshowa
0 .46353 m
.00625 .46353 L
s
[(10.5)] -0.0125 .46353 1 0 Mshowa
0 .54078 m
.00625 .54078 L
s
[(10.75)] -0.0125 .54078 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.18182 .43262 Mdot
.27273 .38627 Mdot
.36364 .35537 Mdot
.45455 .35537 Mdot
.63636 .24721 Mdot
.90909 .21631 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000bOol4000BOol3000BOol2000COol2000?Ool30004Ool20008Ool003=oo`03001oogoo
01=oo`03001oogoo011oo`04001oogoo000AOol01000Oomoo`003goo00<007ooOol00Woo00@007oo
Ool000Moo`00=7oo00<007ooOol047oo10004Goo00@007ooOol0015oo`04001oogoo000?Ool00`00
Oomoo`02Ool01000Oomoo`001goo000eOol00`00Oomoo`0?Ool00`00Ool0000BOol3000COol2000@
Ool00`00Oomoo`02Ool01000Oomoo`001goo000bOol01000Oomoo`004Goo00<007oo00004Woo00<0
07ooOol04Woo00@007ooOol000moo`03001oogoo009oo`04001oogoo0007Ool003=oo`8001=oo`80
01=oo`<0019oo`8000moo`8000Eoo`8000Qoo`00U7oo002DOol001moogD000007goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007oo
Ool0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`001goo0`0017oo0P000goo1000
0Woo0`000goo00<007ooOol0LWoo000:Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007oo
Ool007Aoo`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0007Ool01000Oomoo`00
2goo00<007ooOol00`000goo00<007ooOol0LWoo0007Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo079oo`0027oo0P002Woo0P000goo10000Woo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000=Ool30004
Ool20003Ool30003Ool00`00Oomoo`1bOol0011oo`04001oogooOol20006Ool01000Oomoo`00M7oo
000>Ool3000;Ool00`00Oomoo`02001cOol000eoo`04001oogoo0008Ool30003Ool00`00Oomoo`1b
Ool000eoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`1bOol000ioo`8000Uoo`@0009o
o`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`001goo0`0017oo0P0017oo00<007ooOol00Woo0`000goo00<007ooOol0IGoo0`002Woo000:
Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007ooOol006Moo`<000Yoo`0027oo0`002Woo
00<007ooOol017oo00<007ooOol00P00IWoo0`002Woo0007Ool01000Oomoo`002Woo00@007ooOomo
o`<000=oo`03001oogoo04Eoo`<002Yoo`001goo00@007ooOol000]oo`04001oogoo0005Ool00`00
Oomoo`15Ool3000ZOol000Qoo`8000Uoo`@0009oo`@0009oo`03001oogoo04Eoo`<002Yoo`007goo
00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`004goo0`0017oo0P00
0goo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0LWoo000D
Ool00`00Oomoo`02Ool01000Oomoo`000Woo0P00Lgoo000DOol00`00Oomoo`02Ool01000Oomoo`00
0Woo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0LWoo000C
Ool20005Ool20003Ool00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol09Woo
0`001goo0`00?goo000OOol00`00Oomoo`0VOol30007Ool3000oOol00004Ool000000004Ool20004
Ool20003Ool40002Ool30003Ool00`00Oomoo`0VOol30007Ool3000oOol0009oo`03001oogoo009o
o`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool01000Oomoo`007Goo0`00E7oo0002Ool00`00
Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00P0077oo0`00E7oo0002Ool0
0`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol00`000goo00<007ooOol06goo0`00E7oo0002
Ool00`00Oomoo`02Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo
079oo`0000=oo`0000001Goo0P002Woo0P000goo10000Woo00<007ooOol0LWoo000OOol00`00Oomo
o`1bOol001moo`03001oogoo015oo`<005ioo`007goo00<007ooOol04Goo0`00GWoo0007Ool30004
Ool20004Ool20003Ool30003Ool00`00Oomoo`0AOol3001NOol000Qoo`03001oogoo009oo`04001o
ogoo0003Ool20006Ool01000Oomoo`00M7oo0008Ool00`00Oomoo`02Ool01000Oomoo`002goo00<0
07ooOol00P00Lgoo0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo0`000goo00<007ooOol0LWoo
0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol0LWoo0007Ool2
0005Ool20009Ool40002Ool00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0
LWoo000OOol00`00Oomoo`1bOol00004Ool000000004Ool20004Ool20004Ool00`00Oomoo`02Ool3
0003Ool00`00Oomoo`1bOol0009oo`03001oogoo009oo`04001oogoo0003Ool20004Ool00`00Oomo
o`05Ool01000Oomoo`00M7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo
00<007ooOol00P00Lgoo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00@007ooOomoo`<000=o
o`03001oogoo079oo`000Woo00<007ooOol00Woo00@007ooOol000]oo`04001oogoo0005Ool00`00
Oomoo`1bOol00003Ool0000000Eoo`8000Uoo`@0009oo`@0009oo`03001oogoo079oo`007goo00<0
07ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`004goo0`000goo0`000goo
00<007ooOol0LWoo000DOol00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`1bOol001Aoo`03
001oogoo00=oo`03001oogoo009oo`8007=oo`0057oo00<007ooOol00goo00<007ooOol0Mgoo000D
Ool00`00Oomoo`03Ool00`00Oomoo`1gOol001=oo`8000Aoo`8007Uoo`00U7oo002DOol009Aoo`00
\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.98575, 8.65221, 
  0.0949397, 0.0279301}}]
}, Open  ]],

Cell[TextData[{
  "Now we build a function that linearly interpolates the data. In ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " this can all be done with one line, with the ",
  StyleBox["InterpolationOrder",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  "  option set to unity characterizing the fact that we are using a linear \
scheme."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(swapfunc\  = \ 
      Interpolation[rawswapdata, \ InterpolationOrder\  -> \ 1]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[\(InterpolatingFunction[{{2.`, 10.`}}, "<>"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Filling in Data for Annual Swap", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In this case the data are filled in for all the missing annual \
values:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnnSwapOriginalRange\  = \ 
      Table[{i, \ N[swapfunc[i], \ 5]}, \ {i, \ 2, \ 10, \ 1}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8", ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9", ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
So we can plot the interpolated data along with the original \
points, to illustrate the nature of the interpolation:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(annplot\  = \ 
      ListPlot[AnnSwapOriginalRange, \ PlotStyle\  -> \ PointSize[0.01], \ 
        PlotRange\  -> \ {{0, \ 11}, \ {9, \ 11}}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -2.78115 0.309017 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .07725 -24 -4.5 ]
[-0.0125 .07725 0 4.5 ]
[-0.0125 .15451 -18 -4.5 ]
[-0.0125 .15451 0 4.5 ]
[-0.0125 .23176 -24 -4.5 ]
[-0.0125 .23176 0 4.5 ]
[-0.0125 .30902 -12 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .38627 -30 -4.5 ]
[-0.0125 .38627 0 4.5 ]
[-0.0125 .46353 -24 -4.5 ]
[-0.0125 .46353 0 4.5 ]
[-0.0125 .54078 -30 -4.5 ]
[-0.0125 .54078 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07725 m
.00625 .07725 L
s
[(9.25)] -0.0125 .07725 1 0 Mshowa
0 .15451 m
.00625 .15451 L
s
[(9.5)] -0.0125 .15451 1 0 Mshowa
0 .23176 m
.00625 .23176 L
s
[(9.75)] -0.0125 .23176 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(10)] -0.0125 .30902 1 0 Mshowa
0 .38627 m
.00625 .38627 L
s
[(10.25)] -0.0125 .38627 1 0 Mshowa
0 .46353 m
.00625 .46353 L
s
[(10.5)] -0.0125 .46353 1 0 Mshowa
0 .54078 m
.00625 .54078 L
s
[(10.75)] -0.0125 .54078 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.01 w
.18182 .43262 Mdot
.27273 .38627 Mdot
.36364 .35537 Mdot
.45455 .35537 Mdot
.54545 .30129 Mdot
.63636 .24721 Mdot
.72727 .23691 Mdot
.81818 .22661 Mdot
.90909 .21631 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000bOol4000BOol3000BOol2000COol2000?Ool30004Ool20008Ool003=oo`03001oogoo
01=oo`03001oogoo011oo`04001oogoo000AOol01000Oomoo`003goo00<007ooOol00Woo00@007oo
Ool000Moo`00=7oo00<007ooOol047oo10004Goo00@007ooOol0015oo`04001oogoo000?Ool00`00
Oomoo`02Ool01000Oomoo`001goo000eOol00`00Oomoo`0?Ool00`00Ool0000BOol3000COol2000@
Ool00`00Oomoo`02Ool01000Oomoo`001goo000bOol01000Oomoo`004Goo00<007oo00004Woo00<0
07ooOol04Woo00@007ooOol000moo`03001oogoo009oo`04001oogoo0007Ool003=oo`8001=oo`80
01=oo`<0019oo`8000moo`8000Eoo`8000Qoo`00U7oo002DOol001moogD000007goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007oo
Ool0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`001goo0`0017oo0P000goo1000
0Woo0`000goo00<007ooOol0LWoo000:Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007oo
Ool007Aoo`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0007Ool01000Oomoo`00
2goo00<007ooOol00`000goo00<007ooOol0LWoo0007Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo079oo`0027oo0P002Woo0P000goo10000Woo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000=Ool30004
Ool20003Ool30003Ool00`00Oomoo`1bOol0011oo`04001oogooOol20006Ool01000Oomoo`00M7oo
000>Ool3000;Ool00`00Oomoo`02001cOol000eoo`04001oogoo0008Ool30003Ool00`00Oomoo`1b
Ool000eoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`1bOol000ioo`8000Uoo`@0009o
o`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`001goo0`0017oo0P0017oo00<007ooOol00Woo0`000goo00<007ooOol0LWoo000:Ool01000
Oomoogoo0P0017oo00<007ooOol01Goo00@007ooOol006Qoo`03001oogoo00Uoo`0027oo0`002Woo
00<007ooOol017oo00<007ooOol00P00GGoo00<007ooOol04goo0007Ool01000Oomoo`002Woo00@0
07ooOomoo`<000=oo`03001oogoo055oo`03001oogoo01ioo`001goo00@007ooOol000]oo`04001o
ogoo0005Ool00`00Oomoo`17Ool00`00Oomoo`0XOol000Qoo`8000Uoo`@0009oo`@0009oo`03001o
ogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`00
4goo0`0017oo0P000goo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<0
07ooOol0?7oo00<007ooOol0<goo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo0P00Lgoo000D
Ool00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000
Oomoo`000Woo00<007ooOol0LWoo000COol20005Ool20003Ool00`00Oomoo`1bOol001moo`03001o
ogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`0WOol00`00Oomoo`08Ool00`00Oomo
o`0mOol00004Ool000000004Ool20004Ool20003Ool40002Ool30003Ool00`00Oomoo`1bOol0009o
o`03001oogoo009oo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool01000Oomoo`00M7oo0002
Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00P007Goo00<007oo
Ool0Dgoo0002Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol00`000goo00<007ooOol0
LWoo0002Ool00`00Oomoo`02Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00=oo`03
001oogoo079oo`0000=oo`0000001Goo0P002Woo0P000goo10000Woo00<007ooOol0LWoo000OOol0
0`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol04Woo00<007ooOol0GGoo0007
Ool30004Ool20004Ool20003Ool30003Ool00`00Oomoo`1bOol000Qoo`03001oogoo009oo`04001o
ogoo0003Ool20006Ool01000Oomoo`00M7oo0008Ool00`00Oomoo`02Ool01000Oomoo`002goo00<0
07ooOol00P00Lgoo0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo0`000goo00<007ooOol0LWoo
0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol0LWoo0007Ool2
0005Ool20009Ool40002Ool00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0
LWoo000OOol00`00Oomoo`1bOol00004Ool000000004Ool20004Ool20004Ool00`00Oomoo`02Ool3
0003Ool00`00Oomoo`1bOol0009oo`03001oogoo009oo`04001oogoo0003Ool20004Ool00`00Oomo
o`05Ool01000Oomoo`00M7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo
00<007ooOol00P00Lgoo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00@007ooOomoo`<000=o
o`03001oogoo079oo`000Woo00<007ooOol00Woo00@007ooOol000]oo`04001oogoo0005Ool00`00
Oomoo`1bOol00003Ool0000000Eoo`8000Uoo`@0009oo`@0009oo`03001oogoo079oo`007goo00<0
07ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`004goo0`000goo0`000goo
00<007ooOol0LWoo000DOol00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`1bOol001Aoo`03
001oogoo00=oo`03001oogoo009oo`8007=oo`0057oo00<007ooOol00goo00<007ooOol0Mgoo000D
Ool00`00Oomoo`03Ool00`00Oomoo`1gOol001=oo`8000Aoo`8007Uoo`00U7oo002DOol009Aoo`00
\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.98575, 8.65221, 
  0.0949397, 0.0279301}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[rawplot, \ annplot]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -2.78115 0.309017 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .07725 -24 -4.5 ]
[-0.0125 .07725 0 4.5 ]
[-0.0125 .15451 -18 -4.5 ]
[-0.0125 .15451 0 4.5 ]
[-0.0125 .23176 -24 -4.5 ]
[-0.0125 .23176 0 4.5 ]
[-0.0125 .30902 -12 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .38627 -30 -4.5 ]
[-0.0125 .38627 0 4.5 ]
[-0.0125 .46353 -24 -4.5 ]
[-0.0125 .46353 0 4.5 ]
[-0.0125 .54078 -30 -4.5 ]
[-0.0125 .54078 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07725 m
.00625 .07725 L
s
[(9.25)] -0.0125 .07725 1 0 Mshowa
0 .15451 m
.00625 .15451 L
s
[(9.5)] -0.0125 .15451 1 0 Mshowa
0 .23176 m
.00625 .23176 L
s
[(9.75)] -0.0125 .23176 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(10)] -0.0125 .30902 1 0 Mshowa
0 .38627 m
.00625 .38627 L
s
[(10.25)] -0.0125 .38627 1 0 Mshowa
0 .46353 m
.00625 .46353 L
s
[(10.5)] -0.0125 .46353 1 0 Mshowa
0 .54078 m
.00625 .54078 L
s
[(10.75)] -0.0125 .54078 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.18182 .43262 Mdot
.27273 .38627 Mdot
.36364 .35537 Mdot
.45455 .35537 Mdot
.63636 .24721 Mdot
.90909 .21631 Mdot
.01 w
.18182 .43262 Mdot
.27273 .38627 Mdot
.36364 .35537 Mdot
.45455 .35537 Mdot
.54545 .30129 Mdot
.63636 .24721 Mdot
.72727 .23691 Mdot
.81818 .22661 Mdot
.90909 .21631 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000bOol4000BOol3000BOol2000COol2000?Ool30004Ool20008Ool003=oo`03001oogoo
01=oo`03001oogoo011oo`04001oogoo000AOol01000Oomoo`003goo00<007ooOol00Woo00@007oo
Ool000Moo`00=7oo00<007ooOol047oo10004Goo00@007ooOol0015oo`04001oogoo000?Ool00`00
Oomoo`02Ool01000Oomoo`001goo000eOol00`00Oomoo`0?Ool00`00Ool0000BOol3000COol2000@
Ool00`00Oomoo`02Ool01000Oomoo`001goo000bOol01000Oomoo`004Goo00<007oo00004Woo00<0
07ooOol04Woo00@007ooOol000moo`03001oogoo009oo`04001oogoo0007Ool003=oo`8001=oo`80
01=oo`<0019oo`8000moo`8000Eoo`8000Qoo`00U7oo002DOol001moogD000007goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007oo
Ool0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`001goo0`0017oo0P000goo1000
0Woo0`000goo00<007ooOol0LWoo000:Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007oo
Ool007Aoo`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0007Ool01000Oomoo`00
2goo00<007ooOol00`000goo00<007ooOol0LWoo0007Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo079oo`0027oo0P002Woo0P000goo10000Woo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000=Ool30004
Ool20003Ool30003Ool00`00Oomoo`1bOol0011oo`04001oogooOol20006Ool01000Oomoo`00M7oo
000>Ool3000;Ool00`00Oomoo`02001cOol000eoo`04001oogoo0008Ool30003Ool00`00Oomoo`1b
Ool000eoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`1bOol000ioo`8000Uoo`@0009o
o`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`001goo0`0017oo0P0017oo00<007ooOol00Woo0`000goo00<007ooOol0IGoo0`002Woo000:
Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007ooOol006Moo`<000Yoo`0027oo0`002Woo
00<007ooOol017oo00<007ooOol00P00GGoo00<007ooOol01Woo0`002Woo0007Ool01000Oomoo`00
2Woo00@007ooOomoo`<000=oo`03001oogoo04Eoo`<000Uoo`03001oogoo01ioo`001goo00@007oo
Ool000]oo`04001oogoo0005Ool00`00Oomoo`15Ool3000ZOol000Qoo`8000Uoo`@0009oo`@0009o
o`03001oogoo04Eoo`<002Yoo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03
001oogoo079oo`004goo0`0017oo0P000goo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000
Oomoo`000Woo00<007ooOol0?7oo00<007ooOol0<goo000DOol00`00Oomoo`02Ool01000Oomoo`00
0Woo0P00Lgoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0LWoo000DOol00`00
Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0LWoo000COol20005Ool20003Ool00`00Oomoo`1b
Ool001moo`03001oogoo079oo`007goo00<007ooOol09Woo0`001goo0`00?goo000OOol00`00Oomo
o`0VOol30007Ool3000oOol00004Ool000000004Ool20004Ool20003Ool40002Ool30003Ool00`00
Oomoo`0VOol30007Ool3000oOol0009oo`03001oogoo009oo`04001oogoo0003Ool20004Ool00`00
Oomoo`05Ool01000Oomoo`007Goo0`00E7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<0
07ooOol017oo00<007ooOol00P0077oo0`00E7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002goo
00<007ooOol00`000goo00<007ooOol06goo0`00E7oo0002Ool00`00Oomoo`02Ool01000Oomoo`00
27oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo079oo`0000=oo`0000001Goo0P002Woo
0P000goo10000Woo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo015oo`<0
05ioo`007goo00<007ooOol04Goo0`00GWoo0007Ool30004Ool20004Ool20003Ool30003Ool00`00
Oomoo`0AOol3001NOol000Qoo`03001oogoo009oo`04001oogoo0003Ool20006Ool01000Oomoo`00
M7oo0008Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol00P00Lgoo0008Ool00`00Oomo
o`02Ool01000Oomoo`0027oo0`000goo00<007ooOol0LWoo0008Ool00`00Oomoo`02Ool01000Oomo
o`0027oo00<007ooOol00goo00<007ooOol0LWoo0007Ool20005Ool20009Ool40002Ool00`00Oomo
o`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol00004
Ool000000004Ool20004Ool20004Ool00`00Oomoo`02Ool30003Ool00`00Oomoo`1bOol0009oo`03
001oogoo009oo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool01000Oomoo`00M7oo0002Ool0
0`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0002Ool00`00
Oomoo`02Ool01000Oomoo`002Woo00@007ooOomoo`<000=oo`03001oogoo079oo`000Woo00<007oo
Ool00Woo00@007ooOol000]oo`04001oogoo0005Ool00`00Oomoo`1bOol00003Ool0000000Eoo`80
00Uoo`@0009oo`@0009oo`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1b
Ool001moo`03001oogoo079oo`004goo0`000goo0`000goo00<007ooOol0LWoo000DOol00`00Oomo
o`03Ool00`00Oomoo`02Ool00`00Oomoo`1bOol001Aoo`03001oogoo00=oo`03001oogoo009oo`80
07=oo`0057oo00<007ooOol00goo00<007ooOol0Mgoo000DOol00`00Oomoo`03Ool00`00Oomoo`1g
Ool001=oo`8000Aoo`8007Uoo`00U7oo002DOol009Aoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.98575, 8.65221, 
  0.0949397, 0.0279301}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Filling in Data for Semi-annual Swap", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In this case the data are filled in for value every six months. \
(Note that we are now re-interpreting the same test data set as being based \
on a swap with semi-annual payments.) We plot them as before:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(SemiAnnSwapOriginalRange\  = \ 
      Table[{i, \ N[swapfunc[i], \ 5]}, \ {i, \ 2, \ 10, \ 0.5}]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(semiannplot\  = \ 
      ListPlot[SemiAnnSwapOriginalRange, \ 
        PlotStyle\  -> \ PointSize[0.01], \ 
        PlotRange\  -> \ {{0, \ 11}, \ {9, \ 11}}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -2.78115 0.309017 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .07725 -24 -4.5 ]
[-0.0125 .07725 0 4.5 ]
[-0.0125 .15451 -18 -4.5 ]
[-0.0125 .15451 0 4.5 ]
[-0.0125 .23176 -24 -4.5 ]
[-0.0125 .23176 0 4.5 ]
[-0.0125 .30902 -12 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .38627 -30 -4.5 ]
[-0.0125 .38627 0 4.5 ]
[-0.0125 .46353 -24 -4.5 ]
[-0.0125 .46353 0 4.5 ]
[-0.0125 .54078 -30 -4.5 ]
[-0.0125 .54078 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07725 m
.00625 .07725 L
s
[(9.25)] -0.0125 .07725 1 0 Mshowa
0 .15451 m
.00625 .15451 L
s
[(9.5)] -0.0125 .15451 1 0 Mshowa
0 .23176 m
.00625 .23176 L
s
[(9.75)] -0.0125 .23176 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(10)] -0.0125 .30902 1 0 Mshowa
0 .38627 m
.00625 .38627 L
s
[(10.25)] -0.0125 .38627 1 0 Mshowa
0 .46353 m
.00625 .46353 L
s
[(10.5)] -0.0125 .46353 1 0 Mshowa
0 .54078 m
.00625 .54078 L
s
[(10.75)] -0.0125 .54078 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.01 w
.18182 .43262 Mdot
.22727 .40945 Mdot
.27273 .38627 Mdot
.31818 .37082 Mdot
.36364 .35537 Mdot
.40909 .35537 Mdot
.45455 .35537 Mdot
.5 .32833 Mdot
.54545 .30129 Mdot
.59091 .27425 Mdot
.63636 .24721 Mdot
.68182 .24206 Mdot
.72727 .23691 Mdot
.77273 .23176 Mdot
.81818 .22661 Mdot
.86364 .22146 Mdot
.90909 .21631 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000bOol4000BOol3000BOol2000COol2000?Ool30004Ool20008Ool003=oo`03001oogoo
01=oo`03001oogoo011oo`04001oogoo000AOol01000Oomoo`003goo00<007ooOol00Woo00@007oo
Ool000Moo`00=7oo00<007ooOol047oo10004Goo00@007ooOol0015oo`04001oogoo000?Ool00`00
Oomoo`02Ool01000Oomoo`001goo000eOol00`00Oomoo`0?Ool00`00Ool0000BOol3000COol2000@
Ool00`00Oomoo`02Ool01000Oomoo`001goo000bOol01000Oomoo`004Goo00<007oo00004Woo00<0
07ooOol04Woo00@007ooOol000moo`03001oogoo009oo`04001oogoo0007Ool003=oo`8001=oo`80
01=oo`<0019oo`8000moo`8000Eoo`8000Qoo`00U7oo002DOol001moogD000007goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007oo
Ool0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`001goo0`0017oo0P000goo1000
0Woo0`000goo00<007ooOol0LWoo000:Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007oo
Ool007Aoo`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0007Ool01000Oomoo`00
2goo00<007ooOol00`000goo00<007ooOol0LWoo0007Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo079oo`0027oo0P002Woo0P000goo10000Woo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000=Ool30004
Ool20003Ool30003Ool00`00Oomoo`1bOol0011oo`04001oogooOol20006Ool01000Oomoo`00M7oo
000>Ool3000;Ool00`00Oomoo`02001cOol000eoo`04001oogoo0008Ool30003Ool00`00Oomoo`1b
Ool000eoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`1bOol000ioo`8000Uoo`@0009o
o`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`001goo0`0017oo0P0017oo00<007ooOol00Woo0`000goo00<007ooOol0LWoo000:Ool01000
Oomoogoo0P0017oo00<007ooOol01Goo00@007ooOol006=oo`03001oogoo009oo`03001oogoo00Uo
o`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Egoo00<007ooOol00goo00<007ooOol0
4goo0007Ool01000Oomoo`002Woo00@007ooOomoo`<000=oo`03001oogoo04aoo`03001oogoo009o
o`03001oogoo01ioo`001goo00@007ooOol000]oo`04001oogoo0005Ool00`00Oomoo`17Ool00`00
Oomoo`0XOol000Qoo`8000Uoo`@0009oo`@0009oo`03001oogoo079oo`007goo00<007ooOol0LWoo
000OOol00`00Oomoo`11Ool00`00Oomoo`0^Ool001moo`03001oogoo079oo`004goo0`0017oo0P00
0goo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<007ooOol0?7oo00<0
07ooOol0<goo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo0P00Lgoo000DOol00`00Oomoo`02
Ool01000Oomoo`000Woo00<007ooOol0LWoo000DOol00`00Oomoo`02Ool01000Oomoo`000Woo00<0
07ooOol0=goo00<007ooOol0>7oo000COol20005Ool20003Ool00`00Oomoo`1bOol001moo`03001o
ogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`0WOol00`00Oomoo`02Ool00`00Oomo
o`03Ool00`00Oomoo`0mOol00004Ool000000004Ool20004Ool20003Ool40002Ool30003Ool00`00
Oomoo`0ROol00`00Oomoo`1=Ool0009oo`03001oogoo009oo`04001oogoo0003Ool20004Ool00`00
Oomoo`05Ool01000Oomoo`00M7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol0
17oo00<007ooOol00P007Goo00<007ooOol0Dgoo0002Ool00`00Oomoo`02Ool01000Oomoo`002goo
00<007ooOol00`000goo00<007ooOol0LWoo0002Ool00`00Oomoo`02Ool01000Oomoo`0027oo00@0
07ooOol0009oo`03001oogoo00=oo`03001oogoo079oo`0000=oo`0000001Goo0P002Woo0P000goo
10000Woo00<007ooOol05goo00<007ooOol0F7oo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`007goo00<007ooOol04Woo00<007ooOol0GGoo0007Ool30004Ool20004Ool20003Ool30003
Ool00`00Oomoo`1bOol000Qoo`03001oogoo009oo`04001oogoo0003Ool20006Ool01000Oomoo`00
M7oo0008Ool00`00Oomoo`02Ool01000Oomoo`002goo00<007ooOol00P00Lgoo0008Ool00`00Oomo
o`02Ool01000Oomoo`0027oo0`000goo00<007ooOol0LWoo0008Ool00`00Oomoo`02Ool01000Oomo
o`0027oo00<007ooOol00goo00<007ooOol0LWoo0007Ool20005Ool20009Ool40002Ool00`00Oomo
o`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol00004
Ool000000004Ool20004Ool20004Ool00`00Oomoo`02Ool30003Ool00`00Oomoo`1bOol0009oo`03
001oogoo009oo`04001oogoo0003Ool20004Ool00`00Oomoo`05Ool01000Oomoo`00M7oo0002Ool0
0`00Oomoo`02Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0002Ool00`00
Oomoo`02Ool01000Oomoo`002Woo00@007ooOomoo`<000=oo`03001oogoo079oo`000Woo00<007oo
Ool00Woo00@007ooOol000]oo`04001oogoo0005Ool00`00Oomoo`1bOol00003Ool0000000Eoo`80
00Uoo`@0009oo`@0009oo`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1b
Ool001moo`03001oogoo079oo`004goo0`000goo0`000goo00<007ooOol0LWoo000DOol00`00Oomo
o`03Ool00`00Oomoo`02Ool00`00Oomoo`1bOol001Aoo`03001oogoo00=oo`03001oogoo009oo`80
07=oo`0057oo00<007ooOol00goo00<007ooOol0Mgoo000DOol00`00Oomoo`03Ool00`00Oomoo`1g
Ool001=oo`8000Aoo`8007Uoo`00U7oo002DOol009Aoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.98575, 8.65221, 
  0.0949397, 0.0279301}}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[rawplot, semiannplot]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -2.78115 0.309017 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .07725 -24 -4.5 ]
[-0.0125 .07725 0 4.5 ]
[-0.0125 .15451 -18 -4.5 ]
[-0.0125 .15451 0 4.5 ]
[-0.0125 .23176 -24 -4.5 ]
[-0.0125 .23176 0 4.5 ]
[-0.0125 .30902 -12 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .38627 -30 -4.5 ]
[-0.0125 .38627 0 4.5 ]
[-0.0125 .46353 -24 -4.5 ]
[-0.0125 .46353 0 4.5 ]
[-0.0125 .54078 -30 -4.5 ]
[-0.0125 .54078 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .07725 m
.00625 .07725 L
s
[(9.25)] -0.0125 .07725 1 0 Mshowa
0 .15451 m
.00625 .15451 L
s
[(9.5)] -0.0125 .15451 1 0 Mshowa
0 .23176 m
.00625 .23176 L
s
[(9.75)] -0.0125 .23176 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(10)] -0.0125 .30902 1 0 Mshowa
0 .38627 m
.00625 .38627 L
s
[(10.25)] -0.0125 .38627 1 0 Mshowa
0 .46353 m
.00625 .46353 L
s
[(10.5)] -0.0125 .46353 1 0 Mshowa
0 .54078 m
.00625 .54078 L
s
[(10.75)] -0.0125 .54078 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .01545 m
.00375 .01545 L
s
0 .0309 m
.00375 .0309 L
s
0 .04635 m
.00375 .04635 L
s
0 .0618 m
.00375 .0618 L
s
0 .09271 m
.00375 .09271 L
s
0 .10816 m
.00375 .10816 L
s
0 .12361 m
.00375 .12361 L
s
0 .13906 m
.00375 .13906 L
s
0 .16996 m
.00375 .16996 L
s
0 .18541 m
.00375 .18541 L
s
0 .20086 m
.00375 .20086 L
s
0 .21631 m
.00375 .21631 L
s
0 .24721 m
.00375 .24721 L
s
0 .26266 m
.00375 .26266 L
s
0 .27812 m
.00375 .27812 L
s
0 .29357 m
.00375 .29357 L
s
0 .32447 m
.00375 .32447 L
s
0 .33992 m
.00375 .33992 L
s
0 .35537 m
.00375 .35537 L
s
0 .37082 m
.00375 .37082 L
s
0 .40172 m
.00375 .40172 L
s
0 .41717 m
.00375 .41717 L
s
0 .43262 m
.00375 .43262 L
s
0 .44807 m
.00375 .44807 L
s
0 .47898 m
.00375 .47898 L
s
0 .49443 m
.00375 .49443 L
s
0 .50988 m
.00375 .50988 L
s
0 .52533 m
.00375 .52533 L
s
0 .55623 m
.00375 .55623 L
s
0 .57168 m
.00375 .57168 L
s
0 .58713 m
.00375 .58713 L
s
0 .60258 m
.00375 .60258 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.18182 .43262 Mdot
.27273 .38627 Mdot
.36364 .35537 Mdot
.45455 .35537 Mdot
.63636 .24721 Mdot
.90909 .21631 Mdot
.01 w
.18182 .43262 Mdot
.22727 .40945 Mdot
.27273 .38627 Mdot
.31818 .37082 Mdot
.36364 .35537 Mdot
.40909 .35537 Mdot
.45455 .35537 Mdot
.5 .32833 Mdot
.54545 .30129 Mdot
.59091 .27425 Mdot
.63636 .24721 Mdot
.68182 .24206 Mdot
.72727 .23691 Mdot
.77273 .23176 Mdot
.81818 .22661 Mdot
.86364 .22146 Mdot
.90909 .21631 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000bOol4000BOol3000BOol2000COol2000?Ool30004Ool20008Ool003=oo`03001oogoo
01=oo`03001oogoo011oo`04001oogoo000AOol01000Oomoo`003goo00<007ooOol00Woo00@007oo
Ool000Moo`00=7oo00<007ooOol047oo10004Goo00@007ooOol0015oo`04001oogoo000?Ool00`00
Oomoo`02Ool01000Oomoo`001goo000eOol00`00Oomoo`0?Ool00`00Ool0000BOol3000COol2000@
Ool00`00Oomoo`02Ool01000Oomoo`001goo000bOol01000Oomoo`004Goo00<007oo00004Woo00<0
07ooOol04Woo00@007ooOol000moo`03001oogoo009oo`04001oogoo0007Ool003=oo`8001=oo`80
01=oo`<0019oo`8000moo`8000Eoo`8000Qoo`00U7oo002DOol001moogD000007goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol0
0Woo00<007ooOol00Woo00<007ooOol00goo00<007ooOol00Woo00<007ooOol00Woo00<007ooOol0
0goo00<007ooOol00goo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007oo
Ool0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`001goo0`0017oo0P000goo1000
0Woo0`000goo00<007ooOol0LWoo000:Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007oo
Ool007Aoo`0027oo0`002Woo00<007ooOol017oo00<007ooOol00P00Lgoo0007Ool01000Oomoo`00
2goo00<007ooOol00`000goo00<007ooOol0LWoo0007Ool01000Oomoo`0027oo00@007ooOol0009o
o`03001oogoo00=oo`03001oogoo079oo`0027oo0P002Woo0P000goo10000Woo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<007ooOol0LWoo000=Ool30004
Ool20003Ool30003Ool00`00Oomoo`1bOol0011oo`04001oogooOol20006Ool01000Oomoo`00M7oo
000>Ool3000;Ool00`00Oomoo`02001cOol000eoo`04001oogoo0008Ool30003Ool00`00Oomoo`1b
Ool000eoo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`1bOol000ioo`8000Uoo`@0009o
o`03001oogoo079oo`007goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo
079oo`001goo0`0017oo0P0017oo00<007ooOol00Woo0`000goo00<007ooOol0IGoo0`002Woo000:
Ool01000Oomoogoo0P0017oo00<007ooOol01Goo00@007ooOol006=oo`04001oogooOol3000:Ool0
00Qoo`<000Yoo`03001oogoo00Aoo`03001oogoo008005Moo`03001oogoo00=oo`03001oogoo00Io
o`<000Yoo`001goo00@007ooOol000Yoo`04001oogooOol30003Ool00`00Oomoo`15Ool30004Ool0
0`00Oomoo`02Ool00`00Oomoo`0NOol000Moo`04001oogoo000;Ool01000Oomoo`001Goo00<007oo
Ool0AGoo0`00:Woo0008Ool20009Ool40002Ool40002Ool00`00Oomoo`15Ool3000ZOol001moo`03
001oogoo079oo`007goo00<007ooOol0@Goo00<007ooOol0;Woo000OOol00`00Oomoo`1bOol001=o
o`<000Aoo`8000=oo`03001oogoo079oo`0057oo00<007ooOol00Woo00@007ooOol0009oo`03001o
ogoo03aoo`03001oogoo03=oo`0057oo00<007ooOol00Woo00@007ooOol0009oo`8007=oo`0057oo
00<007ooOol00Woo00@007ooOol0009oo`03001oogoo079oo`0057oo00<007ooOol00Woo00@007oo
Ool0009oo`03001oogoo03Moo`03001oogoo03Qoo`004goo0P001Goo0P000goo00<007ooOol0LWoo
000OOol00`00Oomoo`1bOol001moo`03001oogoo02Ioo`<000Moo`<003moo`007goo00<007ooOol0
9Woo0`000goo00@007ooOomoo`<003moo`0000Aoo`00000000Aoo`8000Aoo`8000=oo`@0009oo`<0
00=oo`03001oogoo029oo`04001oogooOol30007Ool3000oOol0009oo`03001oogoo009oo`04001o
ogoo0003Ool20004Ool00`00Oomoo`05Ool01000Oomoo`007Goo0`00E7oo0002Ool00`00Oomoo`02
Ool01000Oomoo`002Woo00<007ooOol017oo00<007ooOol00P0077oo0`00E7oo0002Ool00`00Oomo
o`02Ool01000Oomoo`002goo00<007ooOol00`000goo00<007ooOol06goo0`00E7oo0002Ool00`00
Oomoo`02Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00=oo`03001oogoo079oo`00
00=oo`0000001Goo0P002Woo0P000goo10000Woo00<007ooOol05goo00<007ooOol0F7oo000OOol0
0`00Oomoo`1bOol001moo`03001oogoo015oo`<005ioo`007goo00<007ooOol04Goo0`00GWoo0007
Ool30004Ool20004Ool20003Ool30003Ool00`00Oomoo`0AOol3001NOol000Qoo`03001oogoo009o
o`04001oogoo0003Ool20006Ool01000Oomoo`00M7oo0008Ool00`00Oomoo`02Ool01000Oomoo`00
2goo00<007ooOol00P00Lgoo0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo0`000goo00<007oo
Ool0LWoo0008Ool00`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol0LWoo
0007Ool20005Ool20009Ool40002Ool00`00Oomoo`1bOol001moo`03001oogoo079oo`007goo00<0
07ooOol0LWoo000OOol00`00Oomoo`1bOol00004Ool000000004Ool20004Ool20004Ool00`00Oomo
o`02Ool30003Ool00`00Oomoo`1bOol0009oo`03001oogoo009oo`04001oogoo0003Ool20004Ool0
0`00Oomoo`05Ool01000Oomoo`00M7oo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00<007oo
Ool017oo00<007ooOol00P00Lgoo0002Ool00`00Oomoo`02Ool01000Oomoo`002Woo00@007ooOomo
o`<000=oo`03001oogoo079oo`000Woo00<007ooOol00Woo00@007ooOol000]oo`04001oogoo0005
Ool00`00Oomoo`1bOol00003Ool0000000Eoo`8000Uoo`@0009oo`@0009oo`03001oogoo079oo`00
7goo00<007ooOol0LWoo000OOol00`00Oomoo`1bOol001moo`03001oogoo079oo`004goo0`000goo
0`000goo00<007ooOol0LWoo000DOol00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`1bOol0
01Aoo`03001oogoo00=oo`03001oogoo009oo`8007=oo`0057oo00<007ooOol00goo00<007ooOol0
Mgoo000DOol00`00Oomoo`03Ool00`00Oomoo`1gOol001=oo`8000Aoo`8007Uoo`00U7oo002DOol0
09Aoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.98575, 8.65221, 
  0.0949397, 0.0279301}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.5 Short-Term LIBOR Discount Factors", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
At this stage we have real or interpolated swap data for annual or \
semi-annual values beginning at two years. The issue here is that swap rates \
are not available for shorter time scales, so one uses other instruments. In \
a more sophisticated scheme, LIBOR and futures data might be used for short \
and medium term rates, with futures being used to cover the join between \
LIBOR and swap data. In the following exercise we are interested in joining \
just the LIBOR and swap data. 

We assume that we have short-term interest-rate data for the following \
time-intervals. These calenderization conventions are taken for simplicity - \
you will probably want to be more accurate:

overnight = 1/365 Y 
1 week = 1/52 Y
1 month = 1/12 Y
3 month = 1/4 Y\t
6 month = 1/2 Y
12month = 1 Y\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(LiborTimes\  = \ {1/365, \ 1/52, \ 1/12, \ 1/4, \ 1/2, 1} // N\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.00273972602739726012`, 0.0192307692307692317`, 
      0.0833333333333333214`, 0.25`, 0.5`, 1.`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(LiborValues\  = \ {9, \ 8.875, \ 8.3125, \ 8.375, \ 8.4375, \ 
        8.4375}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{"9", ",", 
        StyleBox["8.875`",
          StyleBoxAutoDelete->True,
          PrintPrecision->4], ",", 
        StyleBox["8.3125`",
          StyleBoxAutoDelete->True,
          PrintPrecision->5], ",", 
        StyleBox["8.375`",
          StyleBoxAutoDelete->True,
          PrintPrecision->4], ",", 
        StyleBox["8.4375`",
          StyleBoxAutoDelete->True,
          PrintPrecision->5], ",", 
        StyleBox["8.4375`",
          StyleBoxAutoDelete->True,
          PrintPrecision->5]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Libor\  = \ Transpose[{LiborTimes, \ LiborValues}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\({0.00273972602739726012`, 9}\), ",", 
        RowBox[{"{", 
          RowBox[{"0.0192307692307692317`", ",", 
            StyleBox["8.875`",
              StyleBoxAutoDelete->True,
              PrintPrecision->4]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"0.0833333333333333214`", ",", 
            StyleBox["8.3125`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"0.25`", ",", 
            StyleBox["8.375`",
              StyleBoxAutoDelete->True,
              PrintPrecision->4]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"0.5`", ",", 
            StyleBox["8.4375`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"1.`", ",", 
            StyleBox["8.4375`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[Libor, \ PlotStyle\  -> \ PointSize[0.03], \n\t
      PlotRange\  -> \ {8, 10}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0211931 0.954997 -2.47214 0.309017 [
[.21219 -0.0125 -9 -9 ]
[.21219 -0.0125 9 0 ]
[.40319 -0.0125 -9 -9 ]
[.40319 -0.0125 9 0 ]
[.59419 -0.0125 -9 -9 ]
[.59419 -0.0125 9 0 ]
[.78519 -0.0125 -9 -9 ]
[.78519 -0.0125 9 0 ]
[.97619 -0.0125 -3 -9 ]
[.97619 -0.0125 3 0 ]
[.00869 .07725 -24 -4.5 ]
[.00869 .07725 0 4.5 ]
[.00869 .15451 -18 -4.5 ]
[.00869 .15451 0 4.5 ]
[.00869 .23176 -24 -4.5 ]
[.00869 .23176 0 4.5 ]
[.00869 .30902 -6 -4.5 ]
[.00869 .30902 0 4.5 ]
[.00869 .38627 -24 -4.5 ]
[.00869 .38627 0 4.5 ]
[.00869 .46353 -18 -4.5 ]
[.00869 .46353 0 4.5 ]
[.00869 .54078 -24 -4.5 ]
[.00869 .54078 0 4.5 ]
[.00869 .61803 -12 -4.5 ]
[.00869 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21219 0 m
.21219 .00625 L
s
[(0.2)] .21219 -0.0125 0 1 Mshowa
.40319 0 m
.40319 .00625 L
s
[(0.4)] .40319 -0.0125 0 1 Mshowa
.59419 0 m
.59419 .00625 L
s
[(0.6)] .59419 -0.0125 0 1 Mshowa
.78519 0 m
.78519 .00625 L
s
[(0.8)] .78519 -0.0125 0 1 Mshowa
.97619 0 m
.97619 .00625 L
s
[(1)] .97619 -0.0125 0 1 Mshowa
.125 Mabswid
.06894 0 m
.06894 .00375 L
s
.11669 0 m
.11669 .00375 L
s
.16444 0 m
.16444 .00375 L
s
.25994 0 m
.25994 .00375 L
s
.30769 0 m
.30769 .00375 L
s
.35544 0 m
.35544 .00375 L
s
.45094 0 m
.45094 .00375 L
s
.49869 0 m
.49869 .00375 L
s
.54644 0 m
.54644 .00375 L
s
.64194 0 m
.64194 .00375 L
s
.68969 0 m
.68969 .00375 L
s
.73744 0 m
.73744 .00375 L
s
.83294 0 m
.83294 .00375 L
s
.88069 0 m
.88069 .00375 L
s
.92844 0 m
.92844 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
.02119 .07725 m
.02744 .07725 L
s
[(8.25)] .00869 .07725 1 0 Mshowa
.02119 .15451 m
.02744 .15451 L
s
[(8.5)] .00869 .15451 1 0 Mshowa
.02119 .23176 m
.02744 .23176 L
s
[(8.75)] .00869 .23176 1 0 Mshowa
.02119 .30902 m
.02744 .30902 L
s
[(9)] .00869 .30902 1 0 Mshowa
.02119 .38627 m
.02744 .38627 L
s
[(9.25)] .00869 .38627 1 0 Mshowa
.02119 .46353 m
.02744 .46353 L
s
[(9.5)] .00869 .46353 1 0 Mshowa
.02119 .54078 m
.02744 .54078 L
s
[(9.75)] .00869 .54078 1 0 Mshowa
.02119 .61803 m
.02744 .61803 L
s
[(10)] .00869 .61803 1 0 Mshowa
.125 Mabswid
.02119 .01545 m
.02494 .01545 L
s
.02119 .0309 m
.02494 .0309 L
s
.02119 .04635 m
.02494 .04635 L
s
.02119 .0618 m
.02494 .0618 L
s
.02119 .09271 m
.02494 .09271 L
s
.02119 .10816 m
.02494 .10816 L
s
.02119 .12361 m
.02494 .12361 L
s
.02119 .13906 m
.02494 .13906 L
s
.02119 .16996 m
.02494 .16996 L
s
.02119 .18541 m
.02494 .18541 L
s
.02119 .20086 m
.02494 .20086 L
s
.02119 .21631 m
.02494 .21631 L
s
.02119 .24721 m
.02494 .24721 L
s
.02119 .26266 m
.02494 .26266 L
s
.02119 .27812 m
.02494 .27812 L
s
.02119 .29357 m
.02494 .29357 L
s
.02119 .32447 m
.02494 .32447 L
s
.02119 .33992 m
.02494 .33992 L
s
.02119 .35537 m
.02494 .35537 L
s
.02119 .37082 m
.02494 .37082 L
s
.02119 .40172 m
.02494 .40172 L
s
.02119 .41717 m
.02494 .41717 L
s
.02119 .43262 m
.02494 .43262 L
s
.02119 .44807 m
.02494 .44807 L
s
.02119 .47898 m
.02494 .47898 L
s
.02119 .49443 m
.02494 .49443 L
s
.02119 .50988 m
.02494 .50988 L
s
.02119 .52533 m
.02494 .52533 L
s
.02119 .55623 m
.02494 .55623 L
s
.02119 .57168 m
.02494 .57168 L
s
.02119 .58713 m
.02494 .58713 L
s
.02119 .60258 m
.02494 .60258 L
s
.25 Mabswid
.02119 0 m
.02119 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .30902 Mdot
.03956 .27039 Mdot
.10078 .09657 Mdot
.25994 .11588 Mdot
.49869 .13519 Mdot
.97619 .13519 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol002Yo
o`8000Aoo`8000=oo`@000Uoo`8000Aoo`8000Aoo`<000Qoo`8000Aoo`8000Aoo`8000Uoo`8000Ao
o`8000Aoo`8000moo`<000Aoo`00:Goo00@007ooOol000=oo`8000Aoo`03001oogoo00Qoo`04001o
ogoo0003Ool20005Ool00`00Oomoo`06Ool01000Oomoo`000goo0P000goo00@007ooOol000Moo`04
001oogoo0003Ool20003Ool01000Oomoo`003goo00<007ooOol00goo000YOol01000Oomoo`002Woo
00<007ooOol01goo00@007ooOol000Qoo`@000Moo`04001oogoo0008Ool01000Oomoo`001goo00@0
07ooOol000Qoo`04001oogoo000?Ool00`00Oomoo`03Ool002Uoo`04001oogoo000;Ool00`00Oomo
o`06Ool01000Oomoo`0027oo00<007oo000027oo00@007ooOol000Qoo`<000Qoo`04001oogoo0009
Ool2000@Ool00`00Oomoo`03Ool002Uoo`04001oogoo0008Ool01000Oomoo`0027oo00@007ooOol0
00Qoo`03001oo`0000Qoo`04001oogoo0008Ool00`00Oomoo`08Ool01000Oomoo`0027oo00@007oo
Ool000moo`03001oogoo00=oo`00:Woo0P002Woo0P002Woo0P002Woo0P002Goo0P002Woo0`0027oo
0P002Woo0P003goo0P001Goo002DOol009Aoo`005gooO0000Goo000JOol00`00Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00Oomoo`03Ool00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool00`00
Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`03Ool00`00Oomoo`02Ool001Yo
o`8007Qoo`006Woo00<007ooOol0Mgoo000JOol2001hOol001Yoo`03001oogoo07Moo`006Woo0P00
N7oo000JOol00`00Oomoo`1gOol000=oo`8000Aoo`8000=oo`@0009oo`<000=oo`8007Qoo`000Woo
00@007ooOol000=oo`8000Aoo`03001oogoo00Eoo`04001oogoo001iOol0009oo`04001oogoo000:
Ool00`00Oomoo`04Ool00`00Oomoo`02001hOol000=oo`8000aoo`03001oogoo00<000=oo`03001o
ogoo07Moo`000Woo00@007ooOol000Qoo`04001oogoo0002Ool00`00Oomoo`03Ool20006Ool3001_
Ool000=oo`8000Yoo`8000=oo`@0009oo`03001oogoo00Eoo`<006moo`006Woo0P001Woo0`004Goo
0`00Fgoo000JOol00`00Oomoo`0IOol3001KOol001Yoo`8001Yoo`<001Yoo`<003Moo`<000Aoo`00
2Goo0P0017oo0P000goo0`000goo0P00=goo0`00=goo0`0017oo0008Ool01000Oomoo`000goo0P00
1Woo00@007ooOol003Qoo`<003Moo`<000Aoo`0027oo00@007ooOol000]oo`03001oogoo008007Qo
o`002Goo0P002Goo0`000goo00<007ooOol0Mgoo0008Ool01000Oomoo`0027oo00<007ooOol00goo
0P00N7oo0009Ool20009Ool40002Ool00`00Oomoo`1gOol001Yoo`8007Qoo`006Woo00<007ooOol0
Mgoo000JOol2001hOol001Yoo`03001oogoo07Moo`000goo0P0017oo0P0017oo00<007ooOol00Woo
0`000goo0P00N7oo0002Ool01000Oomoo`000goo0P0017oo00<007ooOol01Goo00@007ooOol007Uo
o`000Woo00@007ooOol000Yoo`03001oogoo00Aoo`03001oogoo008007Qoo`000goo0P002goo00@0
07ooOomoo`<000=oo`03001oogoo07Moo`000Woo00@007ooOol000]oo`04001oogoo0005Ool2001h
Ool000=oo`8000Uoo`@0009oo`@0009oo`03001oogoo07Moo`006Woo1000MWoo000JOol4001fOol0
01Yoo`@007Ioo`0057oo0`000goo0P00N7oo000GOol01000Oomoo`00NGoo000EOol3000017oo0000
0000N7oo000DOol01@00Oomoo`00Ool00`00N7oo000DOol01@00Oomoo`00Ool00`00N7oo000EOol2
0003Ool00`00Oomoo`1gOol001Yoo`8007Qoo`006Woo00<007ooOol0Mgoo000JOol2001hOol001Yo
o`03001oogoo07Moo`000Woo0`0017oo0P000goo10000Woo0`000goo0P00N7oo0005Ool01000Oomo
ogoo0P0017oo00<007ooOol01Goo00@007ooOol007Uoo`000goo0`002Woo00<007ooOol017oo00<0
07ooOol00P00N7oo0002Ool01000Oomoo`002goo00<007ooOol00`000goo00<007ooOol0Mgoo0002
Ool01000Oomoo`0027oo00@007ooOol0009oo`03001oogoo00=oo`8007Qoo`000goo0P002Woo0P00
0goo10000Woo0P00N7oo000JOol00`00Oomoo`1gOol001Yoo`8007Qoo`006Woo00<007ooOol0Mgoo
0008Ool30004Ool20003Ool30003Ool2001hOol000]oo`04001oogooOol20006Ool01000Oomoo`00
NGoo0009Ool3000;Ool00`00Oomoo`02001hOol000Qoo`04001oogoo0008Ool30003Ool00`00Oomo
o`1gOol000Qoo`04001oogoo0008Ool00`00Oomoo`03Ool2001hOol000Uoo`8000Uoo`@0009oo`03
001oogoo07Moo`006Woo0P00N7oo000JOol00`00Oomoo`1gOol001Yoo`8007Qoo`006Woo00<007oo
Ool0Mgoo0002Ool30004Ool20004Ool00`00Oomoo`02Ool30003Ool2001hOol000Eoo`04001oogoo
Ool20004Ool00`00Oomoo`05Ool01000Oomoo`00NGoo0003Ool3000:Ool00`00Oomoo`04Ool00`00
Oomoo`02001hOol0009oo`04001oogoo000:Ool01000Oomoogoo0`000goo0P00N7oo0002Ool01000
Oomoo`002goo00@007ooOol000Eoo`03001oogoo07Moo`000goo0P002Goo10000Woo10000Woo0P00
N7oo000JOol00`00Oomoo`1gOol001Yoo`8007Qoo`006Woo00<007ooOol0Mgoo000>Ool30004Ool2
0003Ool2001hOol000moo`03001oogoo009oo`04001oogoo0002Ool00`00Oomoo`1gOol000moo`03
001oogoo009oo`04001oogoo0002Ool2001hOol000moo`03001oogoo009oo`04001oogoo001lOol0
00moo`03001oogoo009oo`04001oogoo001lOol000ioo`8000Eoo`8007eoo`00U7oo0000\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.227573, 7.72125, 
  0.00856772, 0.026478}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Calculation of Short-Term Discount Factors", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The discount factors associated with the short-term LIBOR data are \
calculated using the formulae

(a) if input data are continuously compounded,\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`{t, 
        r} \[Rule] {t, \[ExponentialE]\^\(1\/100\ \((\(-r\)\ t)\)\)}\)], 
  "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["(b) if input data are annually compounded,", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(TraditionalForm\`{t, r} \[Rule] {t, 1\/\((1 + r\/100)\)\^t}\)], 
  "NumberedEquation",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation of this mapping on pairs of values is"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(LiborDisc[libordata_List, \ compounding_String]\  := \n\t
      Which[\n\t\tcompounding == 
          "\<cc\>", \n\t\tMap[{#[\([1]\)], \ 
              Exp[\(-#[\([1]\)]\)*#[\([2]\)]/100]} &, \ 
          libordata], \n\t\t\tcompounding == 
          "\<an\>", \n\t\tMap[{#[\([1]\)], \ 
              1/\((1 + #[\([1]\)]/100)\)^#[\([2]\)]} &, \ 
          libordata], \n\t\tTrue, \ "\<This compounding not supported\>"]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(cclibordiscs\  = \ LiborDisc[Libor, \ "\<cc\>"]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.00273972602739726012`, 
        0.999753455054735518`}, {0.0192307692307692317`, 
        0.998294724867484184`}, {0.0833333333333333214`, 
        0.993096853605472595`}, {0.25`, 0.979280167667322132`}, {0.5`, 
        0.958690009342706694`}, {1.`, 0.919086534013518985`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(anlibordiscs\  = \ LiborDisc[Libor, \ "\<an\>"]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    \({{0.00273972602739726012`, 
        0.999753458431585428`}, {0.0192307692307692317`, 
        0.998294888675374991`}, {0.0833333333333333214`, 
        0.993099718378453744`}, {0.25`, 0.979305754963671759`}, {0.5`, 
        0.958790790695936934`}, {1.`, 0.919471788668628065`}}\)], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[anlibordiscs, \ PlotStyle\  -> \ PointSize[0.03]]; \)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0211931 0.954997 -6.72661 7.33173 [
[.21219 .59263 -9 -9 ]
[.21219 .59263 9 0 ]
[.40319 .59263 -9 -9 ]
[.40319 .59263 9 0 ]
[.59419 .59263 -9 -9 ]
[.59419 .59263 9 0 ]
[.78519 .59263 -9 -9 ]
[.78519 .59263 9 0 ]
[.97619 .59263 -3 -9 ]
[.97619 .59263 3 0 ]
[.00869 .01859 -24 -4.5 ]
[.00869 .01859 0 4.5 ]
[.00869 .16522 -24 -4.5 ]
[.00869 .16522 0 4.5 ]
[.00869 .31186 -24 -4.5 ]
[.00869 .31186 0 4.5 ]
[.00869 .45849 -24 -4.5 ]
[.00869 .45849 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21219 .60513 m
.21219 .61138 L
s
[(0.2)] .21219 .59263 0 1 Mshowa
.40319 .60513 m
.40319 .61138 L
s
[(0.4)] .40319 .59263 0 1 Mshowa
.59419 .60513 m
.59419 .61138 L
s
[(0.6)] .59419 .59263 0 1 Mshowa
.78519 .60513 m
.78519 .61138 L
s
[(0.8)] .78519 .59263 0 1 Mshowa
.97619 .60513 m
.97619 .61138 L
s
[(1)] .97619 .59263 0 1 Mshowa
.125 Mabswid
.06894 .60513 m
.06894 .60888 L
s
.11669 .60513 m
.11669 .60888 L
s
.16444 .60513 m
.16444 .60888 L
s
.25994 .60513 m
.25994 .60888 L
s
.30769 .60513 m
.30769 .60888 L
s
.35544 .60513 m
.35544 .60888 L
s
.45094 .60513 m
.45094 .60888 L
s
.49869 .60513 m
.49869 .60888 L
s
.54644 .60513 m
.54644 .60888 L
s
.64194 .60513 m
.64194 .60888 L
s
.68969 .60513 m
.68969 .60888 L
s
.73744 .60513 m
.73744 .60888 L
s
.83294 .60513 m
.83294 .60888 L
s
.88069 .60513 m
.88069 .60888 L
s
.92844 .60513 m
.92844 .60888 L
s
.25 Mabswid
0 .60513 m
1 .60513 L
s
.02119 .01859 m
.02744 .01859 L
s
[(0.92)] .00869 .01859 1 0 Mshowa
.02119 .16522 m
.02744 .16522 L
s
[(0.94)] .00869 .16522 1 0 Mshowa
.02119 .31186 m
.02744 .31186 L
s
[(0.96)] .00869 .31186 1 0 Mshowa
.02119 .45849 m
.02744 .45849 L
s
[(0.98)] .00869 .45849 1 0 Mshowa
.125 Mabswid
.02119 .05525 m
.02494 .05525 L
s
.02119 .09191 m
.02494 .09191 L
s
.02119 .12856 m
.02494 .12856 L
s
.02119 .20188 m
.02494 .20188 L
s
.02119 .23854 m
.02494 .23854 L
s
.02119 .2752 m
.02494 .2752 L
s
.02119 .34852 m
.02494 .34852 L
s
.02119 .38517 m
.02494 .38517 L
s
.02119 .42183 m
.02494 .42183 L
s
.02119 .49515 m
.02494 .49515 L
s
.02119 .53181 m
.02494 .53181 L
s
.02119 .56847 m
.02494 .56847 L
s
.25 Mabswid
.02119 0 m
.02119 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .60332 Mdot
.03956 .59263 Mdot
.10078 .55454 Mdot
.25994 .4534 Mdot
.49869 .30299 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo002DOol009Aoo`00U7oo002DOol0009oo`8000Aoo`8000=oo`<000=oo`@0009oo`03001o
ogoo07Qoo`0000Eoo`00Oomoo`0000=oo`8000Ioo`05001oogooOol00004Ool00`00Oomoo`1cOol3
0002Ool00005Ool007ooOol00009Ool30004Ool01000Oomoogoo0P00M7oo0`000Woo00001Goo001o
ogoo000027oo00@007ooOol000Eoo`04001oogoo001eOol30002Ool00005Ool007ooOol00008Ool0
1000Oomoo`000Woo00@007ooOol0009oo`03001oogoo07Qoo`000Woo0P002Woo0P0017oo0P000goo
00<007ooOol0N7oo000IOol00`00Oomoo`1hOol001Uoo`8007Uoo`006Goo00<007ooOol0N7oo000I
Ool00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo0P00NGoo000IOol00`00Oomoo`1hOol0
01Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol00`00Oomoo`1hOol001Uoo`8007Uo
o`006Goo00<007ooOol0N7oo0002Ool20004Ool20003Ool30004Ool30002Ool00`00Oomoo`1hOol0
0005Ool007ooOol00003Ool20006Ool00`00Oomoo`02Ool01@00Oomoogoo0000NWoo00001Goo001o
ogoo00002Goo0`000Woo10000Woo0P00NGoo00001Goo001oogoo000027oo00@007ooOol0009oo`03
001oo`0000=oo`03001oogoo07Qoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Ool0
0003Ool00`00Oomoo`1hOol0009oo`8000Yoo`8000Aoo`8000=oo`03001oogoo07Qoo`006Goo00<0
07ooOol0N7oo000IOol2001iOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol0
0`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo0P00NGoo000IOol00`00Oomoo`1hOol001Uo
o`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol2001iOol001Uoo`03001oogoo07Qoo`00
6Goo00<007ooOol0N7oo0002Ool20004Ool20003Ool30004Ool20003Ool00`00Oomoo`0gOol3000n
Ool00005Ool007ooOol00003Ool20006Ool01000Oomoo`000Woo00@007ooOol003Uoo`<003ioo`00
00Eoo`00Oomoo`0000Uoo`<0009oo`04001oogoo0002Ool2000hOol3000nOol00005Ool007ooOol0
0008Ool01000Oomoo`000Woo0`000goo00<007ooOol0N7oo00001Goo001oogoo000027oo00@007oo
Ool0009oo`03001oogoo00=oo`03001oogoo07Qoo`000Woo0P002Woo0P0017oo0`000Woo00<007oo
Ool0N7oo000IOol2001iOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol00`00
Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo0P00NGoo000IOol00`00Oomoo`1hOol001Uoo`03
001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol2001iOol001Uoo`03001oogoo07Qoo`006Goo
00<007ooOol0N7oo0002Ool20004Ool20003Ool30004Ool20003Ool00`00Oomoo`1hOol00005Ool0
07ooOol00003Ool20006Ool01000Oomoo`000Woo00@007ooOol001]oo`<005aoo`0000Eoo`00Oomo
o`0000Uoo`<0009oo`04001oogoo0002Ool2000JOol3001LOol00005Ool007ooOol00008Ool01000
Oomoo`000goo0P000goo00<007ooOol06Goo0`00G7oo00001Goo001oogoo000027oo00@007ooOol0
009oo`04001oogoo0002Ool00`00Oomoo`1hOol0009oo`8000Yoo`8000Aoo`8000=oo`03001oogoo
07Qoo`006Goo0P00NGoo000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007oo
Ool0N7oo000IOol00`00Oomoo`1hOol001Uoo`8007Uoo`006Goo00<007ooOol03Woo0P0017oo0P00
0goo10002Goo0P0017oo0P0017oo0`0027oo0P0017oo0P0017oo0P002Woo0P0017oo0P0017oo0P00
3goo0`000goo000IOol00`00Oomoo`06Ool30004Ool01000Oomoo`000goo0P0017oo00<007ooOol0
27oo00@007ooOol000=oo`8000Eoo`03001oogoo00Ioo`04001oogoo0003Ool20003Ool01000Oomo
o`0027oo00@007ooOol000=oo`8000=oo`04001oogoo000?Ool00`00Oomoo`02Ool001Uoo`03001o
ogoo00Ioo`<000Aoo`04001oogoo000:Ool00`00Oomoo`07Ool01000Oomoo`0027oo10001goo00@0
07ooOol000Qoo`04001oogoo0008Ool01000Oomoo`0027oo00@007ooOol000moo`03001oogoo009o
o`006Goo00<007ooOol01Woo0`0017oo00@007ooOol000]oo`03001oogoo00Ioo`04001oogoo0008
Ool00`00Ool00008Ool01000Oomoo`0027oo0`002Goo00@007ooOol000Uoo`80011oo`03001oogoo
009oo`006Goo0P003Woo00@007ooOol000Qoo`04001oogoo0008Ool01000Oomoo`0027oo00<007oo
000027oo00@007ooOol000Qoo`03001oogoo00Uoo`04001oogoo0008Ool01000Oomoo`003goo00<0
07ooOol00Woo000IOol00`00Oomoo`0>Ool2000:Ool2000:Ool2000:Ool20009Ool2000:Ool30009
Ool2000:Ool2000?Ool20004Ool001Uoo`@007Moo`0067oo1@00Mgoo000FOomn000001Qoo`<000Ao
o`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=o
o`03001oogoo00=oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00=oo`03001oogoo00=o
o`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=o
o`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo005o
o`006Goo00<007ooOol0N7oo002DOol009Aoo`00U7oo002DOol009Aoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-0.215192, 0.908747, 
  0.00842059, 0.00109683}}]
}, Open  ]],

Cell["\<\
From here on the LIBOR data are taken to be in an annual \
representation.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
26.6 Calculation of Dummy Swaps for Short Term 
and Crossover\
\>", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Annually Paid Swaps", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "In this case we have just one dummy swap value, based on the discount \
factor for one year. The swap rate for one year is just given in terms of a \
single discount factor, ",
  Cell[BoxData[
      \(TraditionalForm\`P\)]],
  ", being the discount factor appropriate for one year."
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(P\  = \ \(Last[anlibordiscs]\)[\([2]\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.919471788668628065`\)], "Output"]
}, Open  ]],

Cell["\<\
The dummy (made-up) swap rate for a one-year swap is then just\
\>",
   "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(X\  = \ 100*\((1\  - \ P)\)/P\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(8.7580948457347123`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Semi-annually Paid Swaps", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
In this case we have to find three dummy swap values based on the \
discount factors for 0.5 year and 1 year (to get the first two dummy values) \
with interpolation being used to get a third dummy value for 1.5 years. Let's \
extract the discount factors appropriate to six months (Pone) and one year \
(Ptwo):\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Ponepair\  = \ anlibordiscs[\([5]\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.5`, 0.958790790695936934`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Pone\  = \ Ponepair[\([2]\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.958790790695936934`\)], "Output"]
}, Open  ]],

Cell["\<\
The dummy (made-up) swap rate for a six-month semi-annual swap is \
then just\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Xone\  = \ 100*\((1\  - \ Pone)\)/\((0.5*Pone)\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(8.59607950012773258`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Ptwopair\  = \ Last[anlibordiscs]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({1.`, 0.919471788668628065`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Ptwo\  = \ Ptwopair[\([2]\)]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(0.919471788668628065`\)], "Output"]
}, Open  ]],

Cell["\<\
The dummy (made-up) swap rate for a 12-month semi-annual swap is \
then just\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Xtwo = \ 100*\((1\  - \ Ptwo)\)/\((0.5*\((Pone + Ptwo)\))\)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(8.57475543793407091`\)], "Output"]
}, Open  ]],

Cell["\<\
The dummy swap rate for 1.5 years is given by interpolating the \
one-year dummy value with the real two-year value (taking the data in this \
case to be for semi-annual payments):\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Xthree\  = \ 
      0.5*\((Xtwo\  + \ \(First[SemiAnnSwapOriginalRange]\)[\([2]\)])\)\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9.48737771896703563`\)], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Creating Extended Swap Series", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Recall that we had the following two interpolated swap series, for \
semi-annual and annual swap data:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnnSwapOriginalRange\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8", ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9", ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(SemiAnnSwapOriginalRange\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2.5`", ",", 
            StyleBox["10.3249999999999997`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3.`", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3.5`", ",", 
            StyleBox["10.2000000000000001`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4.`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4.5`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5.`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5.5`", ",", 
            StyleBox["10.0625000000000008`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6.`", ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6.5`", ",", 
            StyleBox["9.88750000000000106`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7.`", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7.5`", ",", 
            StyleBox["9.78333333333333321`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8.`", ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8.5`", ",", 
            StyleBox["9.75`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9.`", ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9.5`", ",", 
            StyleBox["9.71666666666666678`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10.`", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Now we join the dummy swap values to the real swap values. First \
the annual case:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(AnnSwap\  = Join[\ {{1.0, \ X}}, AnnSwapOriginalRange]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "8.7580948457347123`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6", ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8", ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9", ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["Then the semi-annual version:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SemiAnnSwap\  = \ 
      Join[{{0.5, \ Xone}, \ {1.0, \ Xtwo}, \ {1.5, \ 
            Xthree}}, \n\t\tSemiAnnSwapOriginalRange]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", "8.59607950012773258`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "8.57475543793407091`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "9.48737771896703563`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"2.5`", ",", 
            StyleBox["10.3249999999999997`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3.`", ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"3.5`", ",", 
            StyleBox["10.2000000000000001`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4.`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"4.5`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5.`", ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"5.5`", ",", 
            StyleBox["10.0625000000000008`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6.`", ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"6.5`", ",", 
            StyleBox["9.88750000000000106`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7.`", ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"7.5`", ",", 
            StyleBox["9.78333333333333321`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8.`", ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8.5`", ",", 
            StyleBox["9.75`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9.`", ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"9.5`", ",", 
            StyleBox["9.71666666666666678`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"10.`", ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["We can plot the joined-up series: ", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[SemiAnnSwap, \ PlotStyle\  -> \ PointSize[0.03], \n\t
      PlotRange\  -> \ {{0, 11}, \ {8, \ 11}}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.0909091 -1.64809 0.206011 [
[.18182 -0.0125 -3 -9 ]
[.18182 -0.0125 3 0 ]
[.36364 -0.0125 -3 -9 ]
[.36364 -0.0125 3 0 ]
[.54545 -0.0125 -3 -9 ]
[.54545 -0.0125 3 0 ]
[.72727 -0.0125 -3 -9 ]
[.72727 -0.0125 3 0 ]
[.90909 -0.0125 -6 -9 ]
[.90909 -0.0125 6 0 ]
[-0.0125 .10301 -18 -4.5 ]
[-0.0125 .10301 0 4.5 ]
[-0.0125 .20601 -6 -4.5 ]
[-0.0125 .20601 0 4.5 ]
[-0.0125 .30902 -18 -4.5 ]
[-0.0125 .30902 0 4.5 ]
[-0.0125 .41202 -12 -4.5 ]
[-0.0125 .41202 0 4.5 ]
[-0.0125 .51503 -24 -4.5 ]
[-0.0125 .51503 0 4.5 ]
[-0.0125 .61803 -12 -4.5 ]
[-0.0125 .61803 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18182 0 m
.18182 .00625 L
s
[(2)] .18182 -0.0125 0 1 Mshowa
.36364 0 m
.36364 .00625 L
s
[(4)] .36364 -0.0125 0 1 Mshowa
.54545 0 m
.54545 .00625 L
s
[(6)] .54545 -0.0125 0 1 Mshowa
.72727 0 m
.72727 .00625 L
s
[(8)] .72727 -0.0125 0 1 Mshowa
.90909 0 m
.90909 .00625 L
s
[(10)] .90909 -0.0125 0 1 Mshowa
.125 Mabswid
.04545 0 m
.04545 .00375 L
s
.09091 0 m
.09091 .00375 L
s
.13636 0 m
.13636 .00375 L
s
.22727 0 m
.22727 .00375 L
s
.27273 0 m
.27273 .00375 L
s
.31818 0 m
.31818 .00375 L
s
.40909 0 m
.40909 .00375 L
s
.45455 0 m
.45455 .00375 L
s
.5 0 m
.5 .00375 L
s
.59091 0 m
.59091 .00375 L
s
.63636 0 m
.63636 .00375 L
s
.68182 0 m
.68182 .00375 L
s
.77273 0 m
.77273 .00375 L
s
.81818 0 m
.81818 .00375 L
s
.86364 0 m
.86364 .00375 L
s
.95455 0 m
.95455 .00375 L
s
.25 Mabswid
0 0 m
1 0 L
s
0 .10301 m
.00625 .10301 L
s
[(8.5)] -0.0125 .10301 1 0 Mshowa
0 .20601 m
.00625 .20601 L
s
[(9)] -0.0125 .20601 1 0 Mshowa
0 .30902 m
.00625 .30902 L
s
[(9.5)] -0.0125 .30902 1 0 Mshowa
0 .41202 m
.00625 .41202 L
s
[(10)] -0.0125 .41202 1 0 Mshowa
0 .51503 m
.00625 .51503 L
s
[(10.5)] -0.0125 .51503 1 0 Mshowa
0 .61803 m
.00625 .61803 L
s
[(11)] -0.0125 .61803 1 0 Mshowa
.125 Mabswid
0 .0206 m
.00375 .0206 L
s
0 .0412 m
.00375 .0412 L
s
0 .0618 m
.00375 .0618 L
s
0 .0824 m
.00375 .0824 L
s
0 .12361 m
.00375 .12361 L
s
0 .14421 m
.00375 .14421 L
s
0 .16481 m
.00375 .16481 L
s
0 .18541 m
.00375 .18541 L
s
0 .22661 m
.00375 .22661 L
s
0 .24721 m
.00375 .24721 L
s
0 .26781 m
.00375 .26781 L
s
0 .28842 m
.00375 .28842 L
s
0 .32962 m
.00375 .32962 L
s
0 .35022 m
.00375 .35022 L
s
0 .37082 m
.00375 .37082 L
s
0 .39142 m
.00375 .39142 L
s
0 .43262 m
.00375 .43262 L
s
0 .45322 m
.00375 .45322 L
s
0 .47383 m
.00375 .47383 L
s
0 .49443 m
.00375 .49443 L
s
0 .53563 m
.00375 .53563 L
s
0 .55623 m
.00375 .55623 L
s
0 .57683 m
.00375 .57683 L
s
0 .59743 m
.00375 .59743 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.04545 .1228 Mdot
.09091 .11841 Mdot
.13636 .30642 Mdot
.18182 .49443 Mdot
.22727 .47898 Mdot
.27273 .46353 Mdot
.31818 .45322 Mdot
.36364 .44292 Mdot
.40909 .44292 Mdot
.45455 .44292 Mdot
.5 .4249 Mdot
.54545 .40687 Mdot
.59091 .38885 Mdot
.63636 .37082 Mdot
.68182 .36739 Mdot
.72727 .36395 Mdot
.77273 .36052 Mdot
.81818 .35709 Mdot
.86364 .35365 Mdot
.90909 .35022 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True,
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol002eo
o`@001=oo`<001=oo`8001Eoo`80011oo`<000Aoo`8000Qoo`00;Woo00<007ooOol057oo00<007oo
Ool04Goo00@007ooOol001=oo`04001oogoo000@Ool00`00Oomoo`02Ool01000Oomoo`001goo000_
Ool00`00Oomoo`0AOol4000BOol01000Oomoo`004goo00@007ooOol0011oo`03001oogoo009oo`04
001oogoo0007Ool0031oo`03001oogoo011oo`03001oo`0001=oo`<001Eoo`80015oo`03001oogoo
009oo`04001oogoo0007Ool002eoo`04001oogoo000BOol00`00Ool0000COol00`00Oomoo`0DOol0
1000Oomoo`0047oo00<007ooOol00Woo00@007ooOol000Moo`00;Woo0P0057oo0P0057oo0`0057oo
0P0047oo0P001Goo0P0027oo002DOol009Aoo`00U7oo000IOomk000001Uoo`03001oogoo07Qoo`00
6Goo00<007ooOol0N7oo000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007oo
Ool0N7oo000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000I
Ool00`00Oomoo`1hOol000Qoo`8000Aoo`8000=oo`<000=oo`03001oogoo07Qoo`001goo00@007oo
Ool000=oo`8000Ioo`04001oogoo001jOol000Moo`04001oogoo000;Ool00`00Oomoo`02001iOol0
00Qoo`8000Uoo`<000=oo`03001oogoo00Moo`<006ioo`001goo00@007ooOol000Qoo`03001oogoo
00=oo`04001oogooOol30003Ool3001^Ool000Qoo`8000Uoo`@0009oo`04001oogooOol30003Ool3
001^Ool001Uoo`04001oogooOol3001dOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo
000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol00`00
Oomoo`1hOol001=oo`<000=oo`03001oogoo07Qoo`005Woo00@007ooOol007Yoo`0057oo0`000Woo
0P00NGoo000COol01000Oomoo`000Woo00<007ooOol0N7oo000COol01000Oomoo`000Woo00<007oo
Ool0N7oo000DOol20003Ool00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0
N7oo000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol0
0`00Oomoo`1hOol000Moo`<000Aoo`8000=oo`<000=oo`03001oogoo07Qoo`002Woo00@007ooOomo
o`8000Ioo`04001oogoo000>Ool3001YOol000Qoo`<000]oo`03001oogoo008000eoo`<006Uoo`00
1goo00@007ooOol000Qoo`<000=oo`03001oogoo00aoo`<006Uoo`001goo00@007ooOol000Qoo`03
001oogoo00=oo`03001oogoo07Qoo`0027oo0P002Goo10000Woo00<007ooOol0N7oo000IOol00`00
Oomoo`1hOol001Uoo`03001oogoo05moo`<000=oo`<0009oo`<000]oo`006Goo00<007ooOol0E7oo
0`000goo0`000Woo0`000goo0`000Woo0`002goo000IOol00`00Oomoo`19Ool30003Ool30002Ool3
0003Ool30002Ool30003Ool30002Ool3000;Ool001Uoo`03001oogoo04Uoo`<000=oo`<0009oo`<0
00=oo`<001]oo`006Goo00<007ooOol0A7oo0`000Woo0`000goo0`009Woo000IOol00`00Oomoo`14
Ool3000aOol000eoo`<000Aoo`8000=oo`03001oogoo03ioo`<000=oo`<0035oo`003Woo00<007oo
Ool00Woo00@007ooOol0009oo`03001oogoo03ioo`<003Moo`003Woo00<007ooOol00Woo00@007oo
Ool0009oo`8003moo`<003Moo`003Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo03Uo
o`<003aoo`003Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo03Uoo`<003aoo`003Goo
0P001Goo0P000goo00<007ooOol0:7oo0`000goo0`000Woo0`000goo0`00?7oo000IOol00`00Oomo
o`0ROol30003Ool30003Ool30002Ool30012Ool001Uoo`03001oogoo01eoo`<0009oo`<000=oo`<0
00=oo`<0009oo`<0049oo`006Goo00<007ooOol07Goo0`000Woo0`00Dgoo000IOol00`00Oomoo`0G
Ool30003Ool3001HOol001Uoo`03001oogoo01Moo`<005ioo`006Goo00<007ooOol04Woo0`000Woo
0`00GWoo000017oo0000000017oo0P0017oo0P000goo0`000goo00<007ooOol04Woo0`00Hgoo0002
Ool00`00Oomoo`02Ool01000Oomoo`000goo0P001Woo00@007ooOol001Aoo`<006=oo`000Woo00<0
07ooOol00Woo00@007ooOol000]oo`03001oogoo008007Uoo`000Woo00<007ooOol00Woo00@007oo
Ool000Qoo`<000=oo`03001oogoo07Qoo`000Woo00<007ooOol00Woo00@007ooOol000Qoo`03001o
ogoo00=oo`03001oogoo07Qoo`0000=oo`0000001Goo0P002Goo10000Woo00<007ooOol0N7oo000I
Ool00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol00`00Oomo
o`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol00`00Oomoo`1hOol000eo
o`<000=oo`<000=oo`03001oogoo07Qoo`003Woo00<007ooOol00goo00<007ooOol00Woo00<007oo
Ool0N7oo000>Ool00`00Oomoo`03Ool00`00Oomoo`02Ool2001iOol000ioo`03001oogoo00=oo`03
001oogoo07eoo`003Woo00<007ooOol00goo00<007ooOol0OGoo000=Ool20004Ool2001oOol009Ao
o`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-2.30524, 7.57522, 
  0.0903202, 0.0398567}}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.7 Bootstrapping the Swap-Discount Relations", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
For completeness we give the swap rates as a function of discount \
factors - this is a simple ratio with a summation in the denominator:\
\>", 
  "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
SwapTable[discfacs_List, frequencies_List] :=
\tTable[100*(1 -discfacs[[k]])/Apply[Plus,
\t\t\t\tTake[discfacs*frequencies, k]],
\t\t{k, 1, Length[discfacs]}]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
SimpleSwap[discfacs_List, freq_] :=
\tModule[{dummy = Table[1,{ Length[discfacs]}]},
\tSwapTable[discfacs, dummy/freq]]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[TextData[{
  "Note that that the ",
  StyleBox["frequencies",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " variable is defined reciprocally - it is 0.5 for a semi-annual swap."
}], "Text"],

Cell[CellGroupData[{

Cell["Swap Rates to Discount Rates", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "The following is a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation of the bootstrapping algorithm to extract discount factors \
from swap rates:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Discounts[swapdata_List, frequency_] :=
\tModule[{len = Length[swapdata], running = 0, newdisc, disclist = {}, y, \
k=1},
    
\t\tDo[(newdisc =
           (1 - swapdata[[k]]*running*frequency/100)/(
              1+swapdata[[k]]*frequency/100);
\t\t\t\trunning+=newdisc;
\t\t\t\tk+=1;
\t\t\t\tAppendTo[disclist, newdisc]), {k, 1, len}];
\t\tdisclist]\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["Applied to Annual Swap Data", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Let's just extract the swap rates for 1 to 10 years:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({anntimedata, \ annswapdata}\  = \ Transpose[AnnSwap]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "2", ",", "3", ",", "4", ",", "5", 
            ",", "6", ",", "7", ",", "8", ",", "9", ",", "10"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{"8.7580948457347123`", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Now we calculate the discount factor every year for 1 to 10 \
years:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(andisc\  = \ Discounts[annswapdata, \ 1]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.919471788668628065`, 0.819180193821071256`, 0.74538609686603694`, 
      0.678956091643570402`, 0.616392275663704491`, 0.566498024046705683`, 
      0.522862770373813923`, 0.477818724070004829`, 0.43706037004105962`, 
      0.400171600260824344`}\)], "Output"]
}, Open  ]],

Cell["or, in pairs, timestamped:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(tandisc\  = \ 
      Transpose[{anntimedata, \ Discounts[annswapdata, \ 1]}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.919471788668628065`"}], "}"}], 
        ",", \({2, 0.819180193821071256`}\), 
        ",", \({3, 0.74538609686603694`}\), 
        ",", \({4, 0.678956091643570402`}\), 
        ",", \({5, 0.616392275663704491`}\), 
        ",", \({6, 0.566498024046705683`}\), 
        ",", \({7, 0.522862770373813923`}\), 
        ",", \({8, 0.477818724070004829`}\), 
        ",", \({9, 0.43706037004105962`}\), 
        ",", \({10, 0.400171600260824344`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[tandisc, \ PlotStyle\  -> \ PointSize[0.03]]; \)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
-0.0820106 0.10582 -0.438862 1.13346 [
[.34127 .00202 -3 -9 ]
[.34127 .00202 3 0 ]
[.55291 .00202 -3 -9 ]
[.55291 .00202 3 0 ]
[.76455 .00202 -3 -9 ]
[.76455 .00202 3 0 ]
[.97619 .00202 -6 -9 ]
[.97619 .00202 6 0 ]
[.11713 .12787 -18 -4.5 ]
[.11713 .12787 0 4.5 ]
[.11713 .24121 -18 -4.5 ]
[.11713 .24121 0 4.5 ]
[.11713 .35456 -18 -4.5 ]
[.11713 .35456 0 4.5 ]
[.11713 .4679 -18 -4.5 ]
[.11713 .4679 0 4.5 ]
[.11713 .58125 -18 -4.5 ]
[.11713 .58125 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.34127 .01452 m
.34127 .02077 L
s
[(4)] .34127 .00202 0 1 Mshowa
.55291 .01452 m
.55291 .02077 L
s
[(6)] .55291 .00202 0 1 Mshowa
.76455 .01452 m
.76455 .02077 L
s
[(8)] .76455 .00202 0 1 Mshowa
.97619 .01452 m
.97619 .02077 L
s
[(10)] .97619 .00202 0 1 Mshowa
.125 Mabswid
.18254 .01452 m
.18254 .01827 L
s
.23545 .01452 m
.23545 .01827 L
s
.28836 .01452 m
.28836 .01827 L
s
.39418 .01452 m
.39418 .01827 L
s
.44709 .01452 m
.44709 .01827 L
s
.5 .01452 m
.5 .01827 L
s
.60582 .01452 m
.60582 .01827 L
s
.65873 .01452 m
.65873 .01827 L
s
.71164 .01452 m
.71164 .01827 L
s
.81746 .01452 m
.81746 .01827 L
s
.87037 .01452 m
.87037 .01827 L
s
.92328 .01452 m
.92328 .01827 L
s
.07672 .01452 m
.07672 .01827 L
s
.02381 .01452 m
.02381 .01827 L
s
.25 Mabswid
0 .01452 m
1 .01452 L
s
.12963 .12787 m
.13588 .12787 L
s
[(0.5)] .11713 .12787 1 0 Mshowa
.12963 .24121 m
.13588 .24121 L
s
[(0.6)] .11713 .24121 1 0 Mshowa
.12963 .35456 m
.13588 .35456 L
s
[(0.7)] .11713 .35456 1 0 Mshowa
.12963 .4679 m
.13588 .4679 L
s
[(0.8)] .11713 .4679 1 0 Mshowa
.12963 .58125 m
.13588 .58125 L
s
[(0.9)] .11713 .58125 1 0 Mshowa
.125 Mabswid
.12963 .03719 m
.13338 .03719 L
s
.12963 .05986 m
.13338 .05986 L
s
.12963 .08253 m
.13338 .08253 L
s
.12963 .1052 m
.13338 .1052 L
s
.12963 .15054 m
.13338 .15054 L
s
.12963 .1732 m
.13338 .1732 L
s
.12963 .19587 m
.13338 .19587 L
s
.12963 .21854 m
.13338 .21854 L
s
.12963 .26388 m
.13338 .26388 L
s
.12963 .28655 m
.13338 .28655 L
s
.12963 .30922 m
.13338 .30922 L
s
.12963 .33189 m
.13338 .33189 L
s
.12963 .37723 m
.13338 .37723 L
s
.12963 .3999 m
.13338 .3999 L
s
.12963 .42256 m
.13338 .42256 L
s
.12963 .44523 m
.13338 .44523 L
s
.12963 .49057 m
.13338 .49057 L
s
.12963 .51324 m
.13338 .51324 L
s
.12963 .53591 m
.13338 .53591 L
s
.12963 .55858 m
.13338 .55858 L
s
.12963 .60392 m
.13338 .60392 L
s
.25 Mabswid
.12963 0 m
.12963 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.02381 .60332 Mdot
.12963 .48964 Mdot
.23545 .406 Mdot
.34127 .3307 Mdot
.44709 .25979 Mdot
.55291 .20324 Mdot
.65873 .15378 Mdot
.76455 .10272 Mdot
.87037 .05653 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol003=o
o`<001Uoo`8001Yoo`8001Ioo`<000Aoo`8000Qoo`00=7oo00<007ooOol05goo00@007ooOol001Qo
o`04001oogoo000FOol00`00Oomoo`02Ool01000Oomoo`001goo000bOol4000HOol01000Oomoo`00
67oo00@007ooOol001Ioo`03001oogoo009oo`04001oogoo0007Ool0039oo`03001oo`0001Uoo`<0
01Yoo`8001Moo`03001oogoo009oo`04001oogoo0007Ool0039oo`03001oo`0001Uoo`03001oogoo
01Uoo`04001oogoo000FOol00`00Oomoo`02Ool01000Oomoo`001goo000cOol2000JOol3000IOol2
000FOol20005Ool20008Ool009Aoo`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1/Ool3000:
Ool000MoohD000Qoo`0067oo00<007ooOol0K7oo0`002Woo000HOol00`00Oomoo`1iOol001Qoo`03
001oogoo07Uoo`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1NOol3000HOol001Qoo`03001o
ogoo05ioo`<001Qoo`0067oo00<007ooOol0GWoo0`0067oo000HOol00`00Oomoo`1iOol001Qoo`03
001oogoo07Uoo`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1@Ool3000VOol001Qoo`03001o
ogoo051oo`<002Ioo`001Woo0P0017oo0P000goo0`0017oo00<007ooOol0D7oo0`009Woo0005Ool0
1000Oomoo`000goo0P001Woo00D007ooOomoo`0007]oo`001Goo00@007ooOol000]oo`04001oogoo
Ool2001jOol000Eoo`04001oogoo0008Ool30004Ool00`00Oomoo`1iOol000Eoo`04001oogoo0008
Ool00`00Oomoo`04Ool00`00Oomoo`12Ool3000dOol000Ioo`8000Uoo`@000=oo`03001oogoo049o
o`<003Aoo`0067oo00<007ooOol0@Woo0`00=7oo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo
07Uoo`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo03Aoo`<0049o
o`0067oo00<007ooOol0=7oo0`00@Woo000HOol00`00Oomoo`0dOol30012Ool001Qoo`03001oogoo
07Uoo`001Woo0P0017oo0P0017oo0P0017oo00<007ooOol0NGoo0005Ool01000Oomoo`000goo0P00
0goo00@007ooOol000=oo`03001oogoo07Uoo`001Goo00@007ooOol000Qoo`04001oogoo0003Ool2
001jOol000Eoo`04001oogoo0008Ool30004Ool00`00Oomoo`1iOol000Eoo`04001oogoo0008Ool0
0`00Oomoo`04Ool00`00Oomoo`0VOol3001@Ool000Ioo`8000Yoo`<000=oo`03001oogoo02Ioo`<0
051oo`0067oo00<007ooOol09Woo0`00D7oo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo07Uo
o`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo07Uoo`0067oo00<0
07ooOol0NGoo000HOol00`00Oomoo`0HOol3001NOol001Qoo`03001oogoo01Qoo`<005ioo`001Woo
0P0017oo0P0017oo00<007ooOol00goo00<007ooOol067oo0`00GWoo0005Ool01000Oomoo`000goo
0P0017oo00<007ooOol00goo00<007ooOol0NGoo0005Ool01000Oomoo`002Woo00<007ooOol00Woo
0P00NWoo0005Ool01000Oomoo`002Woo00<007ooOol00Woo00<007ooOol0NGoo0005Ool01000Oomo
o`002goo00D007ooOomoo`0007]oo`001Woo0P002Goo10000goo00<007ooOol0NGoo000HOol00`00
Oomoo`1iOol001Qoo`03001oogoo07Uoo`0067oo00<007ooOol02Woo0`00K7oo000HOol00`00Oomo
o`0:Ool3001/Ool001Qoo`03001oogoo00Yoo`<006aoo`0067oo00<007ooOol0NGoo000HOol00`00
Oomoo`1iOol001Qoo`03001oogoo07Uoo`0067oo00<007ooOol0NGoo0006Ool20004Ool20004Ool2
0004Ool00`00Oomoo`1iOol000Eoo`04001oogoo0003Ool20003Ool01000Oomoo`000goo00<007oo
Ool0NGoo0005Ool01000Oomoo`0027oo00@007ooOol000=oo`8007Yoo`001Goo00@007ooOol000Uo
o`8000Aoo`03001oogoo07Uoo`001Goo00@007ooOol000Qoo`04001oogoo0002Ool3001jOol000Io
o`8000Yoo`8000=oo`<007Yoo`005goo0`00NWoo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo
07Uoo`0067oo00<007ooOol0NGoo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo07Uoo`0067oo
00<007ooOol0NGoo000HOol00`00Oomoo`1iOol001Qoo`03001oogoo07Uoo`001Woo0P0017oo0P00
0goo0`0017oo00<007ooOol0NGoo0005Ool01000Oomoo`000goo0P001Woo00D007ooOomoo`0007]o
o`001Goo00@007ooOol000Uoo`<000=oo`8007Yoo`001Goo00@007ooOol000Qoo`04001oogoo0003
Ool00`00Oomoo`1iOol000Eoo`04001oogoo0008Ool01000Oomoo`000goo00<007ooOol0NGoo0006
Ool50007Ool20004Ool00`00Oomoo`1iOol000Qoo`<000eoo`03001oogoo07Uoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {0.255639, 0.328997, 
  0.071373, 0.00666339}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Applied to Semi-annual Swap Data", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Let's just extract the swap rates for 0.5 to 10 years:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({satimedata, \ saswapdata}\  = \ Transpose[SemiAnnSwap]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", 
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", 
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "2", ",", "2.5`", ",", "3.`", ",", 
            "3.5`", ",", "4.`", ",", "4.5`", ",", "5.`", ",", "5.5`", ",", 
            "6.`", ",", "6.5`", ",", "7.`", ",", "7.5`", ",", "8.`", ",", 
            "8.5`", ",", "9.`", ",", "9.5`", ",", "10.`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
          "8.59607950012773258`", ",", "8.57475543793407091`", ",", 
            "9.48737771896703563`", ",", 
            StyleBox["10.4000000000000003`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.3249999999999997`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.2499999999999991`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.2000000000000001`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.1500000000000012`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["10.0625000000000008`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.97500000000000142`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.88750000000000106`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.80000000000000071`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.78333333333333321`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.76666666666666749`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.75`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.7333333333333325`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.71666666666666678`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5], ",", 
            StyleBox["9.69999999999999928`",
              StyleBoxAutoDelete->True,
              PrintPrecision->5]}], "}"}]}], "}"}]], "Output",
  GroupPageBreakWithin->True]
}, Open  ]],

Cell["\<\
Now we calculate the discount factors every six months from 0.5 \
year to 10 years:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(semiandisc\  = \ Discounts[saswapdata, \ 0.5]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0.958790790695936934`, 0.919471788668628065`, 0.869647686833746824`, 
      0.814742078096661259`, 0.7760162346138328`, 0.739731971777335317`, 
      0.705044312002860973`, 0.672367521502291687`, 0.639892954082599807`, 
      0.608986870409326463`, 0.583024264921749946`, 0.558780941425362076`, 
      0.536145586606575363`, 0.515014769227637358`, 0.491783180146382203`, 
      0.469711393827077383`, 0.44874023665234537`, 0.428813630992501337`, 
      0.409878429478065431`, 0.391884258335618085`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(tsadisc\  = \ 
      Transpose[{satimedata, \ Discounts[saswapdata, \ 0.5]}]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->True],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", "0.958790790695936934`"}], "}"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.919471788668628065`"}], "}"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.869647686833746824`"}], "}"}], 
        ",", \({2, 0.814742078096661259`}\), 
        ",", \({2.5`, 0.7760162346138328`}\), 
        ",", \({3.`, 0.739731971777335317`}\), 
        ",", \({3.5`, 0.705044312002860973`}\), 
        ",", \({4.`, 0.672367521502291687`}\), 
        ",", \({4.5`, 0.639892954082599807`}\), 
        ",", \({5.`, 0.608986870409326463`}\), 
        ",", \({5.5`, 0.583024264921749946`}\), 
        ",", \({6.`, 0.558780941425362076`}\), 
        ",", \({6.5`, 0.536145586606575363`}\), 
        ",", \({7.`, 0.515014769227637358`}\), 
        ",", \({7.5`, 0.491783180146382203`}\), 
        ",", \({8.`, 0.469711393827077383`}\), 
        ",", \({8.5`, 0.44874023665234537`}\), 
        ",", \({9.`, 0.428813630992501337`}\), 
        ",", \({9.5`, 0.409878429478065431`}\), 
        ",", \({10.`, 0.391884258335618085`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[tsadisc, \ PlotStyle\  -> \ PointSize[0.03]]; \)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 -0.392168 1.03827 [
[.21429 .01064 -3 -9 ]
[.21429 .01064 3 0 ]
[.40476 .01064 -3 -9 ]
[.40476 .01064 3 0 ]
[.59524 .01064 -3 -9 ]
[.59524 .01064 3 0 ]
[.78571 .01064 -3 -9 ]
[.78571 .01064 3 0 ]
[.97619 .01064 -6 -9 ]
[.97619 .01064 6 0 ]
[.01131 .12697 -18 -4.5 ]
[.01131 .12697 0 4.5 ]
[.01131 .2308 -18 -4.5 ]
[.01131 .2308 0 4.5 ]
[.01131 .33462 -18 -4.5 ]
[.01131 .33462 0 4.5 ]
[.01131 .43845 -18 -4.5 ]
[.01131 .43845 0 4.5 ]
[.01131 .54228 -18 -4.5 ]
[.01131 .54228 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .02314 m
.21429 .02939 L
s
[(2)] .21429 .01064 0 1 Mshowa
.40476 .02314 m
.40476 .02939 L
s
[(4)] .40476 .01064 0 1 Mshowa
.59524 .02314 m
.59524 .02939 L
s
[(6)] .59524 .01064 0 1 Mshowa
.78571 .02314 m
.78571 .02939 L
s
[(8)] .78571 .01064 0 1 Mshowa
.97619 .02314 m
.97619 .02939 L
s
[(10)] .97619 .01064 0 1 Mshowa
.125 Mabswid
.07143 .02314 m
.07143 .02689 L
s
.11905 .02314 m
.11905 .02689 L
s
.16667 .02314 m
.16667 .02689 L
s
.2619 .02314 m
.2619 .02689 L
s
.30952 .02314 m
.30952 .02689 L
s
.35714 .02314 m
.35714 .02689 L
s
.45238 .02314 m
.45238 .02689 L
s
.5 .02314 m
.5 .02689 L
s
.54762 .02314 m
.54762 .02689 L
s
.64286 .02314 m
.64286 .02689 L
s
.69048 .02314 m
.69048 .02689 L
s
.7381 .02314 m
.7381 .02689 L
s
.83333 .02314 m
.83333 .02689 L
s
.88095 .02314 m
.88095 .02689 L
s
.92857 .02314 m
.92857 .02689 L
s
.25 Mabswid
0 .02314 m
1 .02314 L
s
.02381 .12697 m
.03006 .12697 L
s
[(0.5)] .01131 .12697 1 0 Mshowa
.02381 .2308 m
.03006 .2308 L
s
[(0.6)] .01131 .2308 1 0 Mshowa
.02381 .33462 m
.03006 .33462 L
s
[(0.7)] .01131 .33462 1 0 Mshowa
.02381 .43845 m
.03006 .43845 L
s
[(0.8)] .01131 .43845 1 0 Mshowa
.02381 .54228 m
.03006 .54228 L
s
[(0.9)] .01131 .54228 1 0 Mshowa
.125 Mabswid
.02381 .04391 m
.02756 .04391 L
s
.02381 .06467 m
.02756 .06467 L
s
.02381 .08544 m
.02756 .08544 L
s
.02381 .1062 m
.02756 .1062 L
s
.02381 .14773 m
.02756 .14773 L
s
.02381 .1685 m
.02756 .1685 L
s
.02381 .18927 m
.02756 .18927 L
s
.02381 .21003 m
.02756 .21003 L
s
.02381 .25156 m
.02756 .25156 L
s
.02381 .27233 m
.02756 .27233 L
s
.02381 .29309 m
.02756 .29309 L
s
.02381 .31386 m
.02756 .31386 L
s
.02381 .35539 m
.02756 .35539 L
s
.02381 .37615 m
.02756 .37615 L
s
.02381 .39692 m
.02756 .39692 L
s
.02381 .41769 m
.02756 .41769 L
s
.02381 .45922 m
.02756 .45922 L
s
.02381 .47998 m
.02756 .47998 L
s
.02381 .50075 m
.02756 .50075 L
s
.02381 .52151 m
.02756 .52151 L
s
.02381 .00238 m
.02756 .00238 L
s
.02381 .56304 m
.02756 .56304 L
s
.02381 .58381 m
.02756 .58381 L
s
.02381 .60457 m
.02756 .60457 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.03 w
.07143 .60332 Mdot
.11905 .5625 Mdot
.16667 .51076 Mdot
.21429 .45376 Mdot
.2619 .41355 Mdot
.30952 .37588 Mdot
.35714 .33986 Mdot
.40476 .30593 Mdot
.45238 .27222 Mdot
.5 .24013 Mdot
.54762 .21317 Mdot
.59524 .188 Mdot
.64286 .1645 Mdot
.69048 .14256 Mdot
.7381 .11844 Mdot
.78571 .09552 Mdot
.83333 .07375 Mdot
.88095 .05306 Mdot
.92857 .0334 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo000YOol4000FOol3000EOol2000FOol2000COol30004Ool20003Ool002Yoo`03001oogoo
01Moo`03001oogoo01=oo`04001oogoo000DOol01000Oomoo`004goo00<007ooOol00Woo00@007oo
Ool0009oo`00:goo00<007ooOol057oo100057oo00@007ooOol001Aoo`04001oogoo000COol00`00
Oomoo`02Ool01000Oomoo`000Woo000/Ool00`00Oomoo`0COol00`00Ool0000EOol3000FOol2000D
Ool00`00Oomoo`02Ool01000Oomoo`000Woo000YOol01000Oomoo`005Goo00<007oo00005Goo00<0
07ooOol05Goo00@007ooOol001=oo`03001oogoo009oo`04001oogoo0002Ool002Yoo`8001Moo`80
01Ioo`<001Eoo`8001=oo`8000Eoo`8000=oo`004goo0P00Ogoo000COol00`00Oomoo`1eOol30006
Ool001=oo`03001oogoo07Eoo`<000Ioo`0047ooP@000goo000COol00`00Oomoo`1_Ool3000<Ool0
01=oo`80071oo`<000aoo`004goo00<007ooOol0JGoo0`004Woo000COol00`00Oomoo`1YOol3000B
Ool001=oo`8006Aoo`<000=oo`<0019oo`004goo00<007ooOol0Hgoo0`0067oo000COol2001TOol3
000HOol001=oo`03001oogoo05eoo`<001ioo`004goo00<007ooOol0GGoo0`007Woo000COol2001N
Ool3000NOol0009oo`8000Aoo`8000=oo`<000=oo`03001oogoo05Moo`<002Aoo`0000Eoo`00Oomo
o`0000=oo`8000Ioo`04001oogoo001IOol3000TOol00005Ool007ooOol0000;Ool00`00Oomoo`02
001HOol3000TOol00005Ool007ooOol00008Ool30003Ool00`00Oomoo`1AOol3000ZOol00005Ool0
07ooOol00008Ool00`00Oomoo`03Ool2001BOol3000ZOol0009oo`8000Uoo`@0009oo`03001oogoo
055oo`<002Yoo`004goo00<007ooOol0Bgoo0`00<7oo000COol2001<Ool3000`Ool001=oo`03001o
ogoo04]oo`<0031oo`004goo00<007ooOol0AGoo0`00=Woo000COol20016Ool3000fOol001=oo`03
001oogoo04Eoo`<003Ioo`004goo0P00@7oo0`00?7oo0002Ool20004Ool20004Ool20003Ool00`00
Oomoo`0oOol3000lOol00005Ool007ooOol00003Ool20003Ool01000Oomoo`000Woo00<007ooOol0
?goo0`00?7oo00001Goo001oogoo000027oo00@007ooOol0009oo`8007moo`0000Eoo`00Oomoo`00
00Qoo`<000=oo`03001oogoo03Uoo`<0049oo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`03
001oogoo03Uoo`<0049oo`000Woo0P002Woo0`000Woo0P00>Woo0`00@Woo000COol00`00Oomoo`1n
Ool001=oo`8003=oo`<004Uoo`004goo00<007ooOol0<Woo0`00BGoo000COol00`00Oomoo`0bOol3
0019Ool001=oo`8007moo`004goo00<007ooOol0;7oo0`00Cgoo000COol00`00Oomoo`0/Ool3001?
Ool0009oo`8000Aoo`8000Aoo`03001oogoo009oo`8002eoo`<004moo`0000Eoo`00Oomoo`0000=o
o`8000Aoo`03001oogoo009oo`03001oogoo07ioo`0000Eoo`00Oomoo`0000Yoo`04001oogooOol2
000WOol3001EOol00005Ool007ooOol0000:Ool01@00Oomoogoo0000:7oo0`00EGoo00001Goo001o
ogoo00002goo00@007ooOol002Qoo`<005Eoo`000Woo0P002Goo10000Woo0P00Ogoo000COol00`00
Oomoo`1nOol001=oo`03001oogoo021oo`<005]oo`004goo0P008Goo0`00Fgoo000COol00`00Oomo
o`0POol3001KOol001=oo`8007moo`004goo00<007ooOol0OWoo000COol00`00Oomoo`0JOol3001Q
Ool001=oo`8001]oo`<0065oo`000Woo0P0017oo0P0017oo0P000goo00<007ooOol06Woo0`00HGoo
00001Goo001oogoo00000goo0P000goo00@007ooOol0009oo`03001oogoo07ioo`0000Eoo`00Oomo
o`0000Qoo`04001oogoo0002Ool2001oOol00005Ool007ooOol00009Ool20003Ool00`00Oomoo`0D
Ool3001WOol00005Ool007ooOol00008Ool01000Oomoo`000Woo0P005Goo0`00Igoo0002Ool2000:
Ool20003Ool00`00Oomoo`0DOol3001WOol001=oo`03001oogoo07ioo`004goo0P00Ogoo000COol0
0`00Oomoo`1nOol001=oo`03001oogoo07ioo`004goo0P003goo0`00KGoo000COol00`00Oomoo`0>
Ool3001]Ool001=oo`8000moo`<006eoo`000Woo0P0017oo0P000goo0`000goo00<007ooOol0OWoo
00001Goo001oogoo00000goo0P001Woo00@007ooOol0081oo`0000Eoo`00Oomoo`0000Uoo`<0009o
o`8007moo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`1nOol00005Ool007oo
Ool00008Ool01000Oomoo`000Woo00<007ooOol027oo0`00Lgoo0002Ool2000:Ool20003Ool20009
Ool3001cOol001=oo`03001oogoo00Qoo`<007=oo`004goo0P00Ogoo000COol00`00Oomoo`1nOol0
01=oo`03001oogoo009oo`<007Uoo`004goo0P000goo0`00NGoo000COol00`00Oomoo`02Ool3001i
Ool001=oo`03001oogoo07ioo`00U7oo0000\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-1.61017, 0.305294, 
  0.0821599, 0.00753634}}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
26.8 Joining the LIBOR Discount Factors to the Swap Discount \
Factors\
\>", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
Having used the swap data to create discount factors out to ten \
years, we now join it up with the short-term discount factors based on LIBOR \
data.\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["Joining Up with Annual Swap Data", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Here are the long-term discount factors:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(tandisc\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.919471788668628065`"}], "}"}], 
        ",", \({2, 0.819180193821071256`}\), 
        ",", \({3, 0.74538609686603694`}\), 
        ",", \({4, 0.678956091643570402`}\), 
        ",", \({5, 0.616392275663704491`}\), 
        ",", \({6, 0.566498024046705683`}\), 
        ",", \({7, 0.522862770373813923`}\), 
        ",", \({8, 0.477818724070004829`}\), 
        ",", \({9, 0.43706037004105962`}\), 
        ",", \({10, 0.400171600260824344`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell["Here are the short-term ones:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(anlibordiscs\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.00273972602739726012`, 
        0.999753458431585428`}, {0.0192307692307692317`, 
        0.998294888675374991`}, {0.0833333333333333214`, 
        0.993099718378453744`}, {0.25`, 0.979305754963671759`}, {0.5`, 
        0.958790790695936934`}, {1.`, 0.919471788668628065`}}\)], "Output"]
}, Open  ]],

Cell["\<\
They are consistent where they join up, as they should be, so we \
just join them up with the duplicate point deleted:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(andiscs\  = \ Join[anlibordiscs, \ Rest[tandisc]]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.00273972602739726012`, 
        0.999753458431585428`}, {0.0192307692307692317`, 
        0.998294888675374991`}, {0.0833333333333333214`, 
        0.993099718378453744`}, {0.25`, 0.979305754963671759`}, {0.5`, 
        0.958790790695936934`}, {1.`, 0.919471788668628065`}, {2, 
        0.819180193821071256`}, {3, 0.74538609686603694`}, {4, 
        0.678956091643570402`}, {5, 0.616392275663704491`}, {6, 
        0.566498024046705683`}, {7, 0.522862770373813923`}, {8, 
        0.477818724070004829`}, {9, 0.43706037004105962`}, {10, 
        0.400171600260824344`}}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[andiscs, \ PlotStyle\  -> \ PointSize[0.02]]; \)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 -0.37813 0.98169 [
[.21429 .59106 -3 -9 ]
[.21429 .59106 3 0 ]
[.40476 .59106 -3 -9 ]
[.40476 .59106 3 0 ]
[.59524 .59106 -3 -9 ]
[.59524 .59106 3 0 ]
[.78571 .59106 -3 -9 ]
[.78571 .59106 3 0 ]
[.97619 .59106 -6 -9 ]
[.97619 .59106 6 0 ]
[.01131 .01455 -18 -4.5 ]
[.01131 .01455 0 4.5 ]
[.01131 .11272 -18 -4.5 ]
[.01131 .11272 0 4.5 ]
[.01131 .21088 -18 -4.5 ]
[.01131 .21088 0 4.5 ]
[.01131 .30905 -18 -4.5 ]
[.01131 .30905 0 4.5 ]
[.01131 .40722 -18 -4.5 ]
[.01131 .40722 0 4.5 ]
[.01131 .50539 -18 -4.5 ]
[.01131 .50539 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60356 m
.21429 .60981 L
s
[(2)] .21429 .59106 0 1 Mshowa
.40476 .60356 m
.40476 .60981 L
s
[(4)] .40476 .59106 0 1 Mshowa
.59524 .60356 m
.59524 .60981 L
s
[(6)] .59524 .59106 0 1 Mshowa
.78571 .60356 m
.78571 .60981 L
s
[(8)] .78571 .59106 0 1 Mshowa
.97619 .60356 m
.97619 .60981 L
s
[(10)] .97619 .59106 0 1 Mshowa
.125 Mabswid
.07143 .60356 m
.07143 .60731 L
s
.11905 .60356 m
.11905 .60731 L
s
.16667 .60356 m
.16667 .60731 L
s
.2619 .60356 m
.2619 .60731 L
s
.30952 .60356 m
.30952 .60731 L
s
.35714 .60356 m
.35714 .60731 L
s
.45238 .60356 m
.45238 .60731 L
s
.5 .60356 m
.5 .60731 L
s
.54762 .60356 m
.54762 .60731 L
s
.64286 .60356 m
.64286 .60731 L
s
.69048 .60356 m
.69048 .60731 L
s
.7381 .60356 m
.7381 .60731 L
s
.83333 .60356 m
.83333 .60731 L
s
.88095 .60356 m
.88095 .60731 L
s
.92857 .60356 m
.92857 .60731 L
s
.25 Mabswid
0 .60356 m
1 .60356 L
s
.02381 .01455 m
.03006 .01455 L
s
[(0.4)] .01131 .01455 1 0 Mshowa
.02381 .11272 m
.03006 .11272 L
s
[(0.5)] .01131 .11272 1 0 Mshowa
.02381 .21088 m
.03006 .21088 L
s
[(0.6)] .01131 .21088 1 0 Mshowa
.02381 .30905 m
.03006 .30905 L
s
[(0.7)] .01131 .30905 1 0 Mshowa
.02381 .40722 m
.03006 .40722 L
s
[(0.8)] .01131 .40722 1 0 Mshowa
.02381 .50539 m
.03006 .50539 L
s
[(0.9)] .01131 .50539 1 0 Mshowa
.125 Mabswid
.02381 .03418 m
.02756 .03418 L
s
.02381 .05381 m
.02756 .05381 L
s
.02381 .07345 m
.02756 .07345 L
s
.02381 .09308 m
.02756 .09308 L
s
.02381 .13235 m
.02756 .13235 L
s
.02381 .15198 m
.02756 .15198 L
s
.02381 .17162 m
.02756 .17162 L
s
.02381 .19125 m
.02756 .19125 L
s
.02381 .23052 m
.02756 .23052 L
s
.02381 .25015 m
.02756 .25015 L
s
.02381 .26979 m
.02756 .26979 L
s
.02381 .28942 m
.02756 .28942 L
s
.02381 .32869 m
.02756 .32869 L
s
.02381 .34832 m
.02756 .34832 L
s
.02381 .36796 m
.02756 .36796 L
s
.02381 .38759 m
.02756 .38759 L
s
.02381 .42686 m
.02756 .42686 L
s
.02381 .44649 m
.02756 .44649 L
s
.02381 .46612 m
.02756 .46612 L
s
.02381 .48576 m
.02756 .48576 L
s
.02381 .52503 m
.02756 .52503 L
s
.02381 .54466 m
.02756 .54466 L
s
.02381 .56429 m
.02756 .56429 L
s
.02381 .58393 m
.02756 .58393 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.02407 .60332 Mdot
.02564 .60189 Mdot
.03175 .59679 Mdot
.04762 .58325 Mdot
.07143 .56311 Mdot
.11905 .52451 Mdot
.21429 .42605 Mdot
.30952 .35361 Mdot
.40476 .2884 Mdot
.5 .22698 Mdot
.59524 .178 Mdot
.69048 .13516 Mdot
.78571 .09094 Mdot
.88095 .05093 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo002DOol009Aoo`00U7oo0002Ool20004Ool20004Ool30002Ool00`00Oomoo`1nOol00005
Ool007ooOol00003Ool20005Ool01@00Oomoogoo0000P7oo00001Goo001oogoo000027oo10000Woo
0P00Mgoo0P001Woo00001Goo001oogoo000027oo00<007oo00000goo00<007ooOol0MWoo0P001Woo
00001Goo001oogoo000027oo00<007oo00000goo0P00Ogoo0002Ool2000:Ool20003Ool00`00Oomo
o`1nOol001=oo`03001oogoo06Yoo`80019oo`004goo0P00Jgoo0P004Woo000COol00`00Oomoo`1n
Ool001=oo`8007moo`004goo00<007ooOol0OWoo000COol00`00Oomoo`1NOol2000NOol0009oo`80
00Aoo`8000=oo`<000=oo`8005moo`8001ioo`0000Eoo`00Oomoo`0000=oo`8000Ioo`04001oogoo
0020Ool00005Ool007ooOol0000;Ool00`00Oomoo`02001oOol00005Ool007ooOol00008Ool30003
Ool00`00Oomoo`1nOol00005Ool007ooOol00008Ool00`00Oomoo`03Ool00`00Oomoo`1nOol0009o
o`8000Uoo`@0009oo`8005=oo`8002Yoo`004goo00<007ooOol0DWoo0P00:Woo000COol2001oOol0
01=oo`03001oogoo07ioo`004goo00<007ooOol0OWoo000COol2001oOol001=oo`03001oogoo04Eo
o`8003Moo`004goo0P00AWoo0P00=goo0002Ool20004Ool20004Ool20003Ool00`00Oomoo`1nOol0
0005Ool007ooOol00003Ool20003Ool01000Oomoo`000Woo00<007ooOol0OWoo00001Goo001oogoo
000027oo00@007ooOol0009oo`8007moo`0000Eoo`00Oomoo`0000Qoo`<000=oo`03001oogoo07io
o`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`03001oogoo03Uoo`8004=oo`000Woo0P002Woo
0`000Woo0P00>Woo0P00@goo000COol00`00Oomoo`1nOol001=oo`8007moo`004goo00<007ooOol0
OWoo000COol00`00Oomoo`1nOol001=oo`8007moo`004goo00<007ooOol0OWoo000COol2000^Ool2
001?Ool0009oo`8000Aoo`8000Aoo`03001oogoo009oo`03001oogoo02eoo`8004moo`0000Eoo`00
Oomoo`0000=oo`8000Aoo`03001oogoo009oo`03001oogoo07ioo`0000Eoo`00Oomoo`0000Yoo`04
001oogooOol2001oOol00005Ool007ooOol0000:Ool01@00Oomoogoo0000P7oo00001Goo001oogoo
00002goo00<007ooOol00P00Ogoo0002Ool20009Ool40002Ool00`00Oomoo`1nOol001=oo`03001o
ogoo07ioo`004goo0P008Woo0P00Fgoo000COol00`00Oomoo`0QOol2001KOol001=oo`8007moo`00
4goo00<007ooOol0OWoo000COol00`00Oomoo`1nOol0009oo`8000Aoo`8000Aoo`8000=oo`8007mo
o`0000Eoo`00Oomoo`0000=oo`8000=oo`04001oogoo0002Ool00`00Oomoo`1nOol00005Ool007oo
Ool00008Ool01000Oomoo`000Woo0P00Ogoo00001Goo001oogoo00002Goo0P000goo00<007ooOol0
OWoo00001Goo001oogoo000027oo00@007ooOol0009oo`03001oogoo01Eoo`8006Moo`000Woo0P00
2Woo0P000goo0P005Woo0P00Igoo000COol00`00Oomoo`1nOol001=oo`8007moo`004goo00<007oo
Ool0OWoo000COol00`00Oomoo`1nOol001=oo`8007moo`004goo00<007ooOol0OWoo000COol2001o
Ool0009oo`8000Aoo`8000=oo`<000=oo`03001oogoo07ioo`0000Eoo`00Oomoo`0000=oo`8000Io
o`04001oogoo0020Ool00005Ool007ooOol00009Ool30002Ool2001oOol00005Ool007ooOol00008
Ool01000Oomoo`000Woo00<007ooOol0OWoo00001Goo001oogoo000027oo00@007ooOol0009oo`80
00Yoo`8007=oo`000Woo0P002Woo0P000goo00<007ooOol02Goo0P00Lgoo000COol00`00Oomoo`0C
Ool4000FOol3000EOol2000FOol2000COol30004Ool20003Ool001=oo`8001Eoo`03001oogoo01Mo
o`03001oogoo01=oo`04001oogoo000DOol01000Oomoo`004goo00<007ooOol00Woo00@007ooOol0
009oo`004goo00<007ooOol05Goo00<007ooOol057oo100057oo00@007ooOol001Aoo`04001oogoo
000COol00`00Oomoo`02Ool01000Oomoo`000Woo000COol20003Ool2000BOol00`00Oomoo`0COol0
0`00Ool0000EOol3000FOol2000DOol00`00Oomoo`02Ool01000Oomoo`000Woo000COol00`00Oomo
o`02Ool2000?Ool01000Oomoo`005Goo00<007oo00005Goo00<007ooOol05Goo00@007ooOol001=o
o`03001oogoo009oo`04001oogoo0002Ool001=oo`04001oo`00000COol2000GOol2000FOol3000E
Ool2000COol20005Ool20003Ool001=oo`@007eoo`004goo0P00Ogoo000@Oon10003Ool0019oo`<0
00Aoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00Aoo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00=oo`03001oogoo00Aoo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo00=oo`03001oogoo
00Aoo`004goo00<007ooOol0OWoo002DOol009Aoo`00U7oo002DOol00001\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-1.61017, 0.328515, 
  0.0821599, 0.00797069}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Joining Up with Semi-annual Swap Data", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["Here are the long-term discount factors:", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(tsadisc\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", "0.958790790695936934`"}], "}"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.919471788668628065`"}], "}"}], 
        ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "0.869647686833746824`"}], "}"}], 
        ",", \({2, 0.814742078096661259`}\), 
        ",", \({2.5`, 0.7760162346138328`}\), 
        ",", \({3.`, 0.739731971777335317`}\), 
        ",", \({3.5`, 0.705044312002860973`}\), 
        ",", \({4.`, 0.672367521502291687`}\), 
        ",", \({4.5`, 0.639892954082599807`}\), 
        ",", \({5.`, 0.608986870409326463`}\), 
        ",", \({5.5`, 0.583024264921749946`}\), 
        ",", \({6.`, 0.558780941425362076`}\), 
        ",", \({6.5`, 0.536145586606575363`}\), 
        ",", \({7.`, 0.515014769227637358`}\), 
        ",", \({7.5`, 0.491783180146382203`}\), 
        ",", \({8.`, 0.469711393827077383`}\), 
        ",", \({8.5`, 0.44874023665234537`}\), 
        ",", \({9.`, 0.428813630992501337`}\), 
        ",", \({9.5`, 0.409878429478065431`}\), 
        ",", \({10.`, 0.391884258335618085`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell["Here are the shor- term ones (same as before)", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(anlibordiscs\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({{0.00273972602739726012`, 
        0.999753458431585428`}, {0.0192307692307692317`, 
        0.998294888675374991`}, {0.0833333333333333214`, 
        0.993099718378453744`}, {0.25`, 0.979305754963671759`}, {0.5`, 
        0.958790790695936934`}, {1.`, 0.919471788668628065`}}\)], "Output"]
}, Open  ]],

Cell["\<\
They are consistent where they join up, and overlap, as they should \
be, so we just join them up with the duplicate points deleted:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(\(sadiscs\  = \ Join[Take[anlibordiscs, \ 4], \ tsadisc]; \)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  GroupPageBreakWithin->False],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(sadplot\  = \ 
      ListPlot[sadiscs, \ PlotStyle\  -> \ PointSize[0.02]]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0952381 -0.364749 0.968307 [
[.21429 .59106 -3 -9 ]
[.21429 .59106 3 0 ]
[.40476 .59106 -3 -9 ]
[.40476 .59106 3 0 ]
[.59524 .59106 -3 -9 ]
[.59524 .59106 3 0 ]
[.78571 .59106 -3 -9 ]
[.78571 .59106 3 0 ]
[.97619 .59106 -6 -9 ]
[.97619 .59106 6 0 ]
[.01131 .02257 -18 -4.5 ]
[.01131 .02257 0 4.5 ]
[.01131 .1194 -18 -4.5 ]
[.01131 .1194 0 4.5 ]
[.01131 .21623 -18 -4.5 ]
[.01131 .21623 0 4.5 ]
[.01131 .31307 -18 -4.5 ]
[.01131 .31307 0 4.5 ]
[.01131 .4099 -18 -4.5 ]
[.01131 .4099 0 4.5 ]
[.01131 .50673 -18 -4.5 ]
[.01131 .50673 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.21429 .60356 m
.21429 .60981 L
s
[(2)] .21429 .59106 0 1 Mshowa
.40476 .60356 m
.40476 .60981 L
s
[(4)] .40476 .59106 0 1 Mshowa
.59524 .60356 m
.59524 .60981 L
s
[(6)] .59524 .59106 0 1 Mshowa
.78571 .60356 m
.78571 .60981 L
s
[(8)] .78571 .59106 0 1 Mshowa
.97619 .60356 m
.97619 .60981 L
s
[(10)] .97619 .59106 0 1 Mshowa
.125 Mabswid
.07143 .60356 m
.07143 .60731 L
s
.11905 .60356 m
.11905 .60731 L
s
.16667 .60356 m
.16667 .60731 L
s
.2619 .60356 m
.2619 .60731 L
s
.30952 .60356 m
.30952 .60731 L
s
.35714 .60356 m
.35714 .60731 L
s
.45238 .60356 m
.45238 .60731 L
s
.5 .60356 m
.5 .60731 L
s
.54762 .60356 m
.54762 .60731 L
s
.64286 .60356 m
.64286 .60731 L
s
.69048 .60356 m
.69048 .60731 L
s
.7381 .60356 m
.7381 .60731 L
s
.83333 .60356 m
.83333 .60731 L
s
.88095 .60356 m
.88095 .60731 L
s
.92857 .60356 m
.92857 .60731 L
s
.25 Mabswid
0 .60356 m
1 .60356 L
s
.02381 .02257 m
.03006 .02257 L
s
[(0.4)] .01131 .02257 1 0 Mshowa
.02381 .1194 m
.03006 .1194 L
s
[(0.5)] .01131 .1194 1 0 Mshowa
.02381 .21623 m
.03006 .21623 L
s
[(0.6)] .01131 .21623 1 0 Mshowa
.02381 .31307 m
.03006 .31307 L
s
[(0.7)] .01131 .31307 1 0 Mshowa
.02381 .4099 m
.03006 .4099 L
s
[(0.8)] .01131 .4099 1 0 Mshowa
.02381 .50673 m
.03006 .50673 L
s
[(0.9)] .01131 .50673 1 0 Mshowa
.125 Mabswid
.02381 .04194 m
.02756 .04194 L
s
.02381 .06131 m
.02756 .06131 L
s
.02381 .08067 m
.02756 .08067 L
s
.02381 .10004 m
.02756 .10004 L
s
.02381 .13877 m
.02756 .13877 L
s
.02381 .15814 m
.02756 .15814 L
s
.02381 .1775 m
.02756 .1775 L
s
.02381 .19687 m
.02756 .19687 L
s
.02381 .2356 m
.02756 .2356 L
s
.02381 .25497 m
.02756 .25497 L
s
.02381 .27433 m
.02756 .27433 L
s
.02381 .2937 m
.02756 .2937 L
s
.02381 .33243 m
.02756 .33243 L
s
.02381 .3518 m
.02756 .3518 L
s
.02381 .37116 m
.02756 .37116 L
s
.02381 .39053 m
.02756 .39053 L
s
.02381 .42926 m
.02756 .42926 L
s
.02381 .44863 m
.02756 .44863 L
s
.02381 .46799 m
.02756 .46799 L
s
.02381 .48736 m
.02756 .48736 L
s
.02381 .52609 m
.02756 .52609 L
s
.02381 .54546 m
.02756 .54546 L
s
.02381 .56483 m
.02756 .56483 L
s
.02381 .58419 m
.02756 .58419 L
s
.02381 .00321 m
.02756 .00321 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.02407 .60332 Mdot
.02564 .60191 Mdot
.03175 .59688 Mdot
.04762 .58352 Mdot
.07143 .56365 Mdot
.11905 .52558 Mdot
.16667 .47734 Mdot
.21429 .42417 Mdot
.2619 .38667 Mdot
.30952 .35154 Mdot
.35714 .31795 Mdot
.40476 .28631 Mdot
.45238 .25486 Mdot
.5 .22494 Mdot
.54762 .1998 Mdot
.59524 .17632 Mdot
.64286 .1544 Mdot
.69048 .13394 Mdot
.7381 .11145 Mdot
.78571 .09008 Mdot
.83333 .06977 Mdot
.88095 .05047 Mdot
.92857 .03214 Mdot
.97619 .01472 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo002DOol009Aoo`00U7oo0002Ool20004Ool20004Ool30002Ool2001oOol00005Ool007oo
Ool00003Ool20005Ool01@00Oomoogoo0000N7oo0P001Woo00001Goo001oogoo000027oo10000Woo
0P00Mgoo0P001Woo00001Goo001oogoo000027oo00<007oo00000goo00<007ooOol0L7oo0P0037oo
00001Goo001oogoo000027oo00<007oo00000goo00<007ooOol0L7oo0P0037oo0002Ool2000:Ool2
0003Ool2001oOol001=oo`03001oogoo06Yoo`80019oo`004goo0P00Jgoo0P004Woo000COol00`00
Oomoo`1TOol2000HOol001=oo`03001oogoo06Aoo`8001Qoo`004goo0P00Ogoo000COol00`00Oomo
o`1NOol2000NOol001=oo`8005moo`8001ioo`000Woo0P0017oo0P000goo0`000goo00<007ooOol0
OWoo00001Goo001oogoo00000goo0P001Woo00@007ooOol005Yoo`8002Aoo`0000Eoo`00Oomoo`00
00]oo`03001oogoo008005Uoo`8002Aoo`0000Eoo`00Oomoo`0000Qoo`<000=oo`03001oogoo059o
o`8002Yoo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`8005=oo`8002Yoo`000Woo0P002Goo
10000Woo00<007ooOol0OWoo000COol00`00Oomoo`1;Ool2000aOol001=oo`8004aoo`80035oo`00
4goo00<007ooOol0OWoo000COol20016Ool2000gOol001=oo`03001oogoo04Eoo`8003Moo`004goo
00<007ooOol0OWoo0002Ool20004Ool20004Ool20003Ool20010Ool2000mOol00005Ool007ooOol0
0003Ool20003Ool01000Oomoo`000Woo00<007ooOol0?goo0P00?Goo00001Goo001oogoo000027oo
00@007ooOol0009oo`8007moo`0000Eoo`00Oomoo`0000Qoo`<000=oo`03001oogoo03Uoo`8004=o
o`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`03001oogoo03Uoo`8004=oo`000Woo0P002Woo
0`000Woo0P00Ogoo000COol00`00Oomoo`1nOol001=oo`8003Aoo`8004Uoo`004goo00<007ooOol0
<goo0P00BGoo000COol00`00Oomoo`1nOol001=oo`8007moo`004goo00<007ooOol0;Goo0P00Cgoo
000COol2000^Ool2001?Ool0009oo`8000Aoo`8000Aoo`03001oogoo009oo`03001oogoo07ioo`00
00Eoo`00Oomoo`0000=oo`8000Aoo`03001oogoo009oo`03001oogoo07ioo`0000Eoo`00Oomoo`00
00Yoo`04001oogooOol2000XOol2001EOol00005Ool007ooOol0000:Ool01@00Oomoogoo0000:Goo
0P00EGoo00001Goo001oogoo00002goo00<007ooOol00P00Ogoo0002Ool20009Ool40002Ool00`00
Oomoo`1nOol001=oo`80029oo`8005]oo`004goo00<007ooOol08Goo0P00Fgoo000COol00`00Oomo
o`1nOol001=oo`8007moo`004goo00<007ooOol0OWoo000COol2000LOol2001QOol0009oo`8000Ao
o`8000Aoo`8000=oo`03001oogoo01]oo`80065oo`0000Eoo`00Oomoo`0000=oo`8000=oo`04001o
ogoo0002Ool00`00Oomoo`1nOol00005Ool007ooOol00008Ool01000Oomoo`000Woo0P00Ogoo0000
1Goo001oogoo00002Goo0P000goo00<007ooOol05Goo0P00Igoo00001Goo001oogoo000027oo00@0
07ooOol0009oo`8001Ioo`8006Moo`000Woo0P002Woo0P000goo00<007ooOol0OWoo000COol00`00
Oomoo`1nOol001=oo`8007moo`004goo00<007ooOol0OWoo000COol2001oOol001=oo`03001oogoo
00moo`8006eoo`004goo00<007ooOol03goo0P00KGoo0002Ool20004Ool20003Ool30003Ool2001o
Ool00005Ool007ooOol00003Ool20006Ool01000Oomoo`00P7oo00001Goo001oogoo00002Goo0`00
0Woo0P00Ogoo00001Goo001oogoo000027oo00@007ooOol0009oo`03001oogoo07ioo`0000Eoo`00
Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`09Ool2001cOol0009oo`8000Yoo`8000=oo`80
00Yoo`8007=oo`004goo00<007ooOol04goo10005Woo0`005Goo0P005Woo0P004goo0`0017oo0P00
0goo000COol2000EOol00`00Oomoo`0GOol00`00Oomoo`0COol01000Oomoo`0057oo00@007ooOol0
01=oo`03001oogoo009oo`04001oogoo0002Ool001=oo`03001oogoo01Eoo`03001oogoo01Aoo`@0
01Aoo`04001oogoo000DOol01000Oomoo`004goo00<007ooOol00Woo00@007ooOol0009oo`004goo
00<007ooOol00Woo0P004Woo00<007ooOol04goo00<007oo00005Goo0`005Woo0P0057oo00<007oo
Ool00Woo00@007ooOol0009oo`004goo0P000goo0P003goo00@007ooOol001Eoo`03001oo`0001Eo
o`03001oogoo01Eoo`04001oogoo000COol00`00Oomoo`02Ool01000Oomoo`000Woo000COol00`00
Oomoo`0DOol2000GOol2000FOol3000EOol2000COol20005Ool20003Ool001=oo`@007eoo`004goo
00@007oo000007eoo`004Woo0`00Ogoo000@Oon10003Ool001=oo`03001oogoo07ioo`004goo00<0
07ooOol0OWoo002DOol009Aoo`00U7oo002DOol00001\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-1.61017, 0.323378, 
  0.0821599, 0.00808086}}]
}, Open  ]],

Cell[TextData[{
  "At this stage we have a representation of the yield curve as a time series \
from very small times out to 10 years, obtained by fusing LIBOR and swap \
data. From now on the working data set will be the time-stamped discount \
factors based on fusing LIBOR data with semi-annual payment swap data, as \
embodied in the data set, ",
  StyleBox["sadiscs",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", consisting of a list of pairs (time, discount_factor(time))"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["26.9 The Yield Curve and Forwards Calculation", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Here we give a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementation of the mappings for continuously compounded and annual \
variables:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell[TextData[{
  "Work with ",
  Cell[BoxData[
      \(TraditionalForm\`t\  = \ 0\)]],
  " (now!) so only maturities enter relationship. "
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True],

Cell["\<\
YieldCC[{maturity_, discount_} ] :=
\t{maturity, -100*Log[discount]/maturity}\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell["\<\
YieldAnn[{maturity_, discount_} ] :=
\t{maturity, 100*((1/discount)^(1/maturity) - 1)}\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[CellGroupData[{

Cell["YieldAnn[sadiscs[[3]]]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    \({0.0833333333333333214`, 8.66400187783415987`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["yieldsa = Map[YieldAnn, sadiscs]", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\({0.00273972602739726012`, 9.41729347487319046`}\), 
        ",", \({0.0192307692307692317`, 9.27980959130800897`}\), 
        ",", \({0.0833333333333333214`, 8.66400187783415987`}\), 
        ",", \({0.25`, 8.72433831873964393`}\), ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", "8.78081095705902647`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "8.75809484573471053`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "9.75840044472098355`"}], "}"}], 
        ",", \({2, 10.7872868599133253`}\), 
        ",", \({2.5`, 10.6755470257477336`}\), 
        ",", \({3.`, 10.5711611015386397`}\), 
        ",", \({3.5`, 10.5011350569776729`}\), 
        ",", \({4.`, 10.4328596405617979`}\), 
        ",", \({4.5`, 10.430047866371872`}\), 
        ",", \({5.`, 10.4277984985660543`}\), 
        ",", \({5.5`, 10.30683683075444`}\), 
        ",", \({6.`, 10.1859961972440515`}\), 
        ",", \({6.5`, 10.0648915358619639`}\), 
        ",", \({7.`, 9.94326158349538857`}\), 
        ",", \({7.5`, 9.92509354315240699`}\), 
        ",", \({8.`, 9.90592686353217644`}\), 
        ",", \({8.5`, 9.88584779660905965`}\), 
        ",", \({9.`, 9.86492066267110701`}\), 
        ",", \({9.5`, 9.8431936125184638`}\), 
        ",", \({10.`, 9.8207026691642163`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
  "The data in the set ",
  StyleBox["yieldsa",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  " extends to ten years. We extrapolate to times larger than ten years by \
taking the yield to be constant thereafter. The simplest way of sorting this \
is to add a data point at, say, 30 years, and duplicate the 10-year value \
into it:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(extrayield\  = \ {30, \ \(Last[yieldsa]\)[\([2]\)]}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({30, 9.8207026691642163`}\)], "Output"]
}, Open  ]],

Cell["\<\
Similarly, in order to treat very short times, we take the yield at \
time zero to be the same as that at the ON rate:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(firstyield\  = \ {0, \ \(First[yieldsa]\)[\([2]\)]}\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({0, 9.41729347487319046`}\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(yields\  = \ Prepend[Append[yieldsa, \ extrayield], \ firstyield]\)], 
  "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{\({0, 9.41729347487319046`}\), 
        ",", \({0.00273972602739726012`, 9.41729347487319046`}\), 
        ",", \({0.0192307692307692317`, 9.27980959130800897`}\), 
        ",", \({0.0833333333333333214`, 8.66400187783415987`}\), 
        ",", \({0.25`, 8.72433831873964393`}\), ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["0.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->1], ",", "8.78081095705902647`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "8.75809484573471053`"}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            StyleBox["1.5`",
              StyleBoxAutoDelete->True,
              PrintPrecision->2], ",", "9.75840044472098355`"}], "}"}], 
        ",", \({2, 10.7872868599133253`}\), 
        ",", \({2.5`, 10.6755470257477336`}\), 
        ",", \({3.`, 10.5711611015386397`}\), 
        ",", \({3.5`, 10.5011350569776729`}\), 
        ",", \({4.`, 10.4328596405617979`}\), 
        ",", \({4.5`, 10.430047866371872`}\), 
        ",", \({5.`, 10.4277984985660543`}\), 
        ",", \({5.5`, 10.30683683075444`}\), 
        ",", \({6.`, 10.1859961972440515`}\), 
        ",", \({6.5`, 10.0648915358619639`}\), 
        ",", \({7.`, 9.94326158349538857`}\), 
        ",", \({7.5`, 9.92509354315240699`}\), 
        ",", \({8.`, 9.90592686353217644`}\), 
        ",", \({8.5`, 9.88584779660905965`}\), 
        ",", \({9.`, 9.86492066267110701`}\), 
        ",", \({9.5`, 9.8431936125184638`}\), 
        ",", \({10.`, 9.8207026691642163`}\), 
        ",", \({30, 9.8207026691642163`}\)}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
So, finally, we get the yield curve based on the fused, \
interpolated and extrapolated information:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell["\<\
ycurve = ListPlot[yields, PlotRange -> {{0, 20}, All},
\tPlotJoined -> True];\
\>", "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  AspectRatioFixed->True],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0 0.05 -2.38707 0.277214 [
[.125 .09536 -9 -9 ]
[.125 .09536 9 0 ]
[.25 .09536 -3 -9 ]
[.25 .09536 3 0 ]
[.375 .09536 -9 -9 ]
[.375 .09536 9 0 ]
[.5 .09536 -6 -9 ]
[.5 .09536 6 0 ]
[.625 .09536 -12 -9 ]
[.625 .09536 12 0 ]
[.75 .09536 -6 -9 ]
[.75 .09536 6 0 ]
[.875 .09536 -12 -9 ]
[.875 .09536 12 0 ]
[1 .09536 -6 -9 ]
[1 .09536 6 0 ]
[-0.0125 .24647 -18 -4.5 ]
[-0.0125 .24647 0 4.5 ]
[-0.0125 .38507 -12 -4.5 ]
[-0.0125 .38507 0 4.5 ]
[-0.0125 .52368 -24 -4.5 ]
[-0.0125 .52368 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.125 .10786 m
.125 .11411 L
s
[(2.5)] .125 .09536 0 1 Mshowa
.25 .10786 m
.25 .11411 L
s
[(5)] .25 .09536 0 1 Mshowa
.375 .10786 m
.375 .11411 L
s
[(7.5)] .375 .09536 0 1 Mshowa
.5 .10786 m
.5 .11411 L
s
[(10)] .5 .09536 0 1 Mshowa
.625 .10786 m
.625 .11411 L
s
[(12.5)] .625 .09536 0 1 Mshowa
.75 .10786 m
.75 .11411 L
s
[(15)] .75 .09536 0 1 Mshowa
.875 .10786 m
.875 .11411 L
s
[(17.5)] .875 .09536 0 1 Mshowa
1 .10786 m
1 .11411 L
s
[(20)] 1 .09536 0 1 Mshowa
.125 Mabswid
.025 .10786 m
.025 .11161 L
s
.05 .10786 m
.05 .11161 L
s
.075 .10786 m
.075 .11161 L
s
.1 .10786 m
.1 .11161 L
s
.15 .10786 m
.15 .11161 L
s
.175 .10786 m
.175 .11161 L
s
.2 .10786 m
.2 .11161 L
s
.225 .10786 m
.225 .11161 L
s
.275 .10786 m
.275 .11161 L
s
.3 .10786 m
.3 .11161 L
s
.325 .10786 m
.325 .11161 L
s
.35 .10786 m
.35 .11161 L
s
.4 .10786 m
.4 .11161 L
s
.425 .10786 m
.425 .11161 L
s
.45 .10786 m
.45 .11161 L
s
.475 .10786 m
.475 .11161 L
s
.525 .10786 m
.525 .11161 L
s
.55 .10786 m
.55 .11161 L
s
.575 .10786 m
.575 .11161 L
s
.6 .10786 m
.6 .11161 L
s
.65 .10786 m
.65 .11161 L
s
.675 .10786 m
.675 .11161 L
s
.7 .10786 m
.7 .11161 L
s
.725 .10786 m
.725 .11161 L
s
.775 .10786 m
.775 .11161 L
s
.8 .10786 m
.8 .11161 L
s
.825 .10786 m
.825 .11161 L
s
.85 .10786 m
.85 .11161 L
s
.9 .10786 m
.9 .11161 L
s
.925 .10786 m
.925 .11161 L
s
.95 .10786 m
.95 .11161 L
s
.975 .10786 m
.975 .11161 L
s
.25 Mabswid
0 .10786 m
1 .10786 L
s
0 .24647 m
.00625 .24647 L
s
[(9.5)] -0.0125 .24647 1 0 Mshowa
0 .38507 m
.00625 .38507 L
s
[(10)] -0.0125 .38507 1 0 Mshowa
0 .52368 m
.00625 .52368 L
s
[(10.5)] -0.0125 .52368 1 0 Mshowa
.125 Mabswid
0 .13558 m
.00375 .13558 L
s
0 .1633 m
.00375 .1633 L
s
0 .19102 m
.00375 .19102 L
s
0 .21874 m
.00375 .21874 L
s
0 .27419 m
.00375 .27419 L
s
0 .30191 m
.00375 .30191 L
s
0 .32963 m
.00375 .32963 L
s
0 .35735 m
.00375 .35735 L
s
0 .41279 m
.00375 .41279 L
s
0 .44051 m
.00375 .44051 L
s
0 .46824 m
.00375 .46824 L
s
0 .49596 m
.00375 .49596 L
s
0 .08014 m
.00375 .08014 L
s
0 .05242 m
.00375 .05242 L
s
0 .02469 m
.00375 .02469 L
s
0 .5514 m
.00375 .5514 L
s
0 .57912 m
.00375 .57912 L
s
0 .60684 m
.00375 .60684 L
s
.25 Mabswid
0 0 m
0 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
0 .22354 m
.00014 .22354 L
.00096 .18543 L
.00417 .01472 L
.0125 .03144 L
.025 .0471 L
.05 .0408 L
.075 .3181 L
.1 .60332 L
.125 .57234 L
.15 .54341 L
.175 .52399 L
.2 .50507 L
.225 .50429 L
.25 .50366 L
.275 .47013 L
.3 .43663 L
.325 .40306 L
.35 .36934 L
.375 .36431 L
.4 .35899 L
.425 .35343 L
.45 .34763 L
.475 .3416 L
.5 .33537 L
s
.5 .33537 m
1 .33537 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  Evaluatable->False,
  AspectRatioFixed->True,
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo002DOol009Aoo`00U7oo002DOol009Aoo`006Goo00<007ooOol0N7oo000IOol00`00Oomo
o`1hOol001Uoo`03001oogoo07Qoo`006Goo00<007ooOol0N7oo000IOol20004Ool40003Ool20003
Ool30006Ool30006Ool00`00Oomoo`03Ool20003Ool30003Ool30004Ool40003Ool40003Ool20003
Ool60003Ool50004Ool00`00Oomoo`03Ool20003Ool70003Ool20003Ool001Uoo`03001oo`00009o
o`<000Eoo`8000Ioo`03001oogoo00Ioo`03001oogoo00=oo`03001oogoo00=oo`8000Ioo`05001o
ogooOol00004Ool01000Oomoo`001Goo00<007ooOol00goo0P001Woo0P001goo00<007ooOol00goo
00<007ooOol00goo0P001Woo0P0017oo00@007ooOol0009oo`006Goo00<007ooOol00P0000Aoo`00
Ool000aoo`03001oogoo00Ioo`03001oogoo00Aoo`03001oogoo00Yoo`05001oogooOol00004Ool0
1000Oomoo`001Woo00<007ooOol02Woo0P001goo00<007ooOol017oo00<007ooOol02Woo00<007oo
00000goo00@007ooOol0009oo`006Goo00<007ooOol00goo00@007ooOol000Qoo`<000Ioo`<000Mo
o`03001oogoo00Moo`<000Aoo`03001oogoo009oo`04001oogoo0007Ool00`00Oomoo`06Ool30000
0goo001oo`03Ool40006Ool00`00Oomoo`07Ool30003Ool01000Oomoo`000Woo0@000Goo0Goo000I
Ool00`00Oomoo`03Ool01000Oomoo`0027oo00<007ooOol01Woo00<007ooOol027oo00<007ooOol0
1Woo00<007ooOol017oo00<007ooOol00Woo00@007ooOol000Aoo`04001oogoo0008Ool01@00Oomo
ogoo000017oo00@007ooOol000Moo`03001oogoo00Ioo`04001oogoo0002Ool01000Oomoo`000Woo
0@000Goo0Goo000IOol00`00Oomoo`03Ool30009Ool40005Ool40004Ool40008Ool40002Ool20005
Ool30005Ool20009Ool50004Ool40004Ool40008Ool60004Ool20003Ool001Uoo`03001oogoo00=o
o`03001oogoo079oo`006Goo00<007ooOol017oo00<007ooOol0LGoo000IOol00`00Oomoo`04Ool0
0`00Oomoo`1aOol001UoogD000Ioo`006Goo00<007ooOol017oo00<007ooOol017oo00<007ooOol0
37oo00<007ooOol02goo00<007ooOol037oo00<007ooOol02goo00<007ooOol037oo00<007ooOol0
2goo00<007ooOol037oo00<007ooOol017oo000IOol00`00Oomoo`04Ool00`00Oomoo`1aOol001Uo
o`03001oogoo00Aoo`03001oogoo075oo`006Goo00<007ooOol017oo00<007ooOol0LGoo000IOol0
0`00Oomoo`04Ool00`00Oomoo`1aOol001Uoo`03001oogoo00Aoo`03001oogoo075oo`006Goo00<0
07ooOol017oo00<007ooOol0LGoo000IOol00`00Oomoo`04Ool00`00Oomoo`1aOol001Uoo`03001o
ogoo00Eoo`03001oogoo071oo`006Goo00<007ooOol01Goo00<007ooOol0L7oo000IOol00`00Oomo
o`05Ool00`00Oomoo`1`Ool001Uoo`03001oogoo00Eoo`03001oogoo071oo`006Goo00<007ooOol0
1Goo00<007ooOol0L7oo0007Ool30004Ool20003Ool30003Ool00`00Oomoo`05Ool00`00Oomoo`1`
Ool000Yoo`04001oogooOol20006Ool01000Oomoo`001goo00<007ooOol0L7oo0008Ool3000;Ool0
0`00Oomoo`020006Ool00`00Oomoo`1`Ool000Moo`04001oogoo0008Ool30003Ool00`00Oomoo`05
Ool00`00Oomoo`1`Ool000Moo`04001oogoo0008Ool00`00Oomoo`03Ool00`00Oomoo`05Ool00`00
Oomoo`1`Ool000Qoo`8000Uoo`@0009oo`03001oogoo00Eoo`03001oogoo071oo`006Goo00<007oo
Ool01Woo00<007ooOol0Kgoo000IOol00`00Oomoo`06Ool00`00Oomoo`1_Ool001Uoo`03001oogoo
00Ioo`03001oogoo06moo`006Goo00<007ooOol01Woo00<007ooOol0Kgoo000IOol00`00Oomoo`06
Ool00`00Oomoo`1_Ool001Uoo`03001oogoo00Ioo`03001oogoo06moo`006Goo00<007ooOol01Woo
00<007ooOol0;Goo?0001Woo000IOol00`00Oomoo`06Ool00`00Oomoo`0ZOol30012Ool001Uoo`03
001oogoo00Ioo`03001oogoo02Aoo`H004Eoo`006Goo00<007ooOol01Woo00<007ooOol08Goo0`00
Bgoo000=Ool30004Ool20003Ool00`00Oomoo`07Ool00`00Oomoo`0LOol4001>Ool000ioo`03001o
ogoo009oo`04001oogoo0002Ool00`00Oomoo`07Ool00`00Oomoo`0LOol00`00Oomoo`1?Ool000io
o`03001oogoo009oo`04001oogoo0002Ool20008Ool00`00Oomoo`0KOol00`00Oomoo`1@Ool000io
o`03001oogoo009oo`04001oogoo0002Ool00`00Oomoo`07Ool00`00Oomoo`0JOol00`00Oomoo`1A
Ool000ioo`03001oogoo009oo`04001oogoo0002Ool00`00Oomoo`07Ool00`00Oomoo`0IOol00`00
Oomoo`1BOol000eoo`8000Eoo`8000=oo`03001oogoo00Moo`03001oogoo01Uoo`03001oogoo059o
o`006Goo00<007ooOol01goo00<007ooOol067oo00<007ooOol0Dgoo000IOol00`00Oomoo`07Ool0
0`00Oomoo`0GOol00`00Oomoo`1DOol001Uoo`03001oogoo00Moo`03001oogoo01Ioo`03001oogoo
05Eoo`006Goo00<007ooOol01goo00<007ooOol05Woo00<007ooOol0EGoo000IOol00`00Oomoo`07
Ool00`00Oomoo`0EOol00`00Oomoo`1FOol001Uoo`03001oogoo00Qoo`03001oogoo01=oo`03001o
ogoo05Moo`006Goo00<007ooOol027oo00<007ooOol04Woo00<007ooOol0F7oo000IOol00`00Oomo
o`08Ool00`00Oomoo`0BOol00`00Oomoo`1HOol001Uoo`03001oogoo00Qoo`03001oogoo015oo`03
001oogoo05Uoo`006Goo00<007ooOol027oo00<007ooOol047oo00<007ooOol0FWoo000017oo0000
000017oo0P0017oo0P000goo0`000goo00<007ooOol027oo00<007ooOol027oo2000GGoo0002Ool0
0`00Oomoo`02Ool01000Oomoo`000goo0P001Woo00@007ooOol000Yoo`03001oogoo00Moo`03001o
ogoo06=oo`000Woo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo008000Uoo`03001oogoo
00Eoo`8006Ioo`000Woo00<007ooOol00Woo00@007ooOol000Qoo`<000=oo`03001oogoo00Qoo`03
001oogoo00Aoo`03001oogoo06Ioo`000Woo00<007ooOol00Woo00@007ooOol000Qoo`03001oogoo
00=oo`03001oogoo00Qoo`03001oogoo00=oo`03001oogoo06Moo`0000=oo`0000001Goo0P002Goo
10000Woo00<007ooOol027oo00<007ooOol00goo00<007ooOol0Igoo000IOol00`00Oomoo`09Ool0
1@00Oomoogoo0000JWoo000IOol00`00Oomoo`09Ool01000Oomoo`00Jgoo000IOol00`00Oomoo`09
Ool00`00Ool0001/Ool001Uoo`03001oogoo00Uoo`03001oo`0006aoo`006Goo00<007ooOol02Goo
0P00KGoo000IOol00`00Oomoo`09Ool00`00Oomoo`1/Ool001Uoo`03001oogoo07Qoo`006Goo00<0
07ooOol0N7oo002DOol009Aoo`00U7oo002DOol009Aoo`00U7oo002DOol009Aoo`00U7oo0000\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-4.39292, 8.31584, 
  0.172618, 0.0311344}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Yield Curve Interpolation", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
For our purposes it may be useful to define an interpolated \
function for the yield:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(yieldfunc\  = \ 
      Interpolation[yields, \ InterpolationOrder\  -> \ 1]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    TagBox[\(InterpolatingFunction[{{0, 30.`}}, "<>"]\),
      False,
      Editable->False]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(yieldfunc[1.2]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(9.15821708532921974`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[yieldfunc[T], \ {T, \ 0, \ 15}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0634921 -2.39499 0.278048 [
[.15079 .09494 -3 -9 ]
[.15079 .09494 3 0 ]
[.27778 .09494 -3 -9 ]
[.27778 .09494 3 0 ]
[.40476 .09494 -3 -9 ]
[.40476 .09494 3 0 ]
[.53175 .09494 -3 -9 ]
[.53175 .09494 3 0 ]
[.65873 .09494 -6 -9 ]
[.65873 .09494 6 0 ]
[.78571 .09494 -6 -9 ]
[.78571 .09494 6 0 ]
[.9127 .09494 -6 -9 ]
[.9127 .09494 6 0 ]
[.01131 .24646 -18 -4.5 ]
[.01131 .24646 0 4.5 ]
[.01131 .38548 -12 -4.5 ]
[.01131 .38548 0 4.5 ]
[.01131 .52451 -24 -4.5 ]
[.01131 .52451 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .10744 m
.15079 .11369 L
s
[(2)] .15079 .09494 0 1 Mshowa
.27778 .10744 m
.27778 .11369 L
s
[(4)] .27778 .09494 0 1 Mshowa
.40476 .10744 m
.40476 .11369 L
s
[(6)] .40476 .09494 0 1 Mshowa
.53175 .10744 m
.53175 .11369 L
s
[(8)] .53175 .09494 0 1 Mshowa
.65873 .10744 m
.65873 .11369 L
s
[(10)] .65873 .09494 0 1 Mshowa
.78571 .10744 m
.78571 .11369 L
s
[(12)] .78571 .09494 0 1 Mshowa
.9127 .10744 m
.9127 .11369 L
s
[(14)] .9127 .09494 0 1 Mshowa
.125 Mabswid
.05556 .10744 m
.05556 .11119 L
s
.0873 .10744 m
.0873 .11119 L
s
.11905 .10744 m
.11905 .11119 L
s
.18254 .10744 m
.18254 .11119 L
s
.21429 .10744 m
.21429 .11119 L
s
.24603 .10744 m
.24603 .11119 L
s
.30952 .10744 m
.30952 .11119 L
s
.34127 .10744 m
.34127 .11119 L
s
.37302 .10744 m
.37302 .11119 L
s
.43651 .10744 m
.43651 .11119 L
s
.46825 .10744 m
.46825 .11119 L
s
.5 .10744 m
.5 .11119 L
s
.56349 .10744 m
.56349 .11119 L
s
.59524 .10744 m
.59524 .11119 L
s
.62698 .10744 m
.62698 .11119 L
s
.69048 .10744 m
.69048 .11119 L
s
.72222 .10744 m
.72222 .11119 L
s
.75397 .10744 m
.75397 .11119 L
s
.81746 .10744 m
.81746 .11119 L
s
.84921 .10744 m
.84921 .11119 L
s
.88095 .10744 m
.88095 .11119 L
s
.94444 .10744 m
.94444 .11119 L
s
.97619 .10744 m
.97619 .11119 L
s
.25 Mabswid
0 .10744 m
1 .10744 L
s
.02381 .24646 m
.03006 .24646 L
s
[(9.5)] .01131 .24646 1 0 Mshowa
.02381 .38548 m
.03006 .38548 L
s
[(10)] .01131 .38548 1 0 Mshowa
.02381 .52451 m
.03006 .52451 L
s
[(10.5)] .01131 .52451 1 0 Mshowa
.125 Mabswid
.02381 .13524 m
.02756 .13524 L
s
.02381 .16305 m
.02756 .16305 L
s
.02381 .19085 m
.02756 .19085 L
s
.02381 .21866 m
.02756 .21866 L
s
.02381 .27426 m
.02756 .27426 L
s
.02381 .30207 m
.02756 .30207 L
s
.02381 .32987 m
.02756 .32987 L
s
.02381 .35768 m
.02756 .35768 L
s
.02381 .41329 m
.02756 .41329 L
s
.02381 .44109 m
.02756 .44109 L
s
.02381 .4689 m
.02756 .4689 L
s
.02381 .4967 m
.02756 .4967 L
s
.02381 .07963 m
.02756 .07963 L
s
.02381 .05183 m
.02756 .05183 L
s
.02381 .02402 m
.02756 .02402 L
s
.02381 .55231 m
.02756 .55231 L
s
.02381 .58012 m
.02756 .58012 L
s
.02381 .60792 m
.02756 .60792 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .22346 m
.02605 .14215 L
.02729 .09021 L
.02846 .04111 L
.02954 .01472 L
.03072 .01659 L
.03279 .01986 L
.03746 .02727 L
.04262 .03369 L
.04762 .03864 L
.05018 .04117 L
.05293 .04389 L
.05424 .04519 L
.05549 .04643 L
.05662 .04628 L
.05784 .04604 L
.06244 .04512 L
.07302 .04302 L
.07883 .04186 L
.08167 .04129 L
.08429 .04077 L
.08558 .04052 L
.08678 .04028 L
.08803 .04657 L
.08869 .05238 L
.08941 .05861 L
.09493 .10704 L
.10458 .19155 L
.12439 .36646 L
.13484 .46062 L
.14057 .51222 L
.14594 .56066 L
.14825 .58144 L
.1495 .59277 L
.15068 .60332 L
.15195 .60326 L
.15262 .6026 L
.15334 .6019 L
.15578 .59951 L
.16664 .58888 L
.18579 .57035 L
.22728 .53632 L
.24809 .52359 L
.25722 .51813 L
.26727 .51213 L
.27192 .50934 L
.27448 .50781 L
.27575 .50705 L
.27692 .50635 L
.27801 .50583 L
Mistroke
.27918 .50581 L
.28164 .50574 L
.28595 .50564 L
.30573 .50515 L
.31604 .50493 L
.32718 .50471 L
.3323 .50461 L
.3351 .50455 L
.3377 .5045 L
.33993 .50446 L
.34118 .50443 L
.34231 .50333 L
.34737 .49797 L
.38726 .45572 L
.42565 .41505 L
.44507 .39441 L
.45538 .38342 L
.46079 .37766 L
.46374 .37452 L
.46648 .3716 L
.46774 .37025 L
.46893 .3696 L
.46959 .3695 L
.4703 .36938 L
.47154 .36918 L
.47707 .3683 L
.48677 .36676 L
.5058 .36368 L
.54677 .35669 L
.58622 .34958 L
.62416 .34242 L
.63501 .3403 L
.64528 .33828 L
.65451 .33646 L
.65704 .33596 L
.65832 .33571 L
.65904 .33563 L
.65973 .33563 L
.66456 .33563 L
.70343 .33563 L
.74209 .33563 L
.7832 .33563 L
.82279 .33563 L
.86086 .33563 L
.90139 .33563 L
.94041 .33563 L
.97619 .33563 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`00U7oo002DOol009Aoo`00U7oo000IOol00`00Oomoo`1hOol001Uoo`03001oogoo07Qoo`006Goo
0P00NGoo000IOol3001hOol001Uoo`@007Moo`006Goo0P0000=oo`00000010001Goo10003Goo0`00
3Goo0P003Goo0P002Woo0`0017oo0P001goo0`000goo10001Woo0`0017oo0`001goo000IOol20006
Ool00`00Oomoo`04Ool00`00Oomoo`0>Ool00`00Oomoo`0;Ool01000Oomoo`002goo00@007ooOol0
00Yoo`03001oogoo009oo`04001oogoo0007Ool00`00Oomoo`03Ool00`00Oomoo`07Ool00`00Oomo
o`04Ool00`00Oomoo`06Ool001Uoo`8000Ioo`03001oogoo00Eoo`03001oogoo00]oo`@000aoo`04
001oogoo000;Ool01000Oomoo`002Woo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo00Ao
o`03001oogoo00Ioo`03001oogoo009oo`@000Moo`006Goo0P001Woo00<007ooOol01Woo00<007oo
Ool02Woo00<007oo00003Goo0`003Goo0P002goo00<007ooOol00Woo00@007ooOol000Moo`03001o
ogoo00Eoo`03001oogoo00Eoo`03001oogoo009oo`03001oo`0000Qoo`006Goo00<007ooOol01Goo
00<007ooOol00goo00@007ooOol000aoo`03001oo`0000eoo`03001oogoo00aoo`04001oogoo000:
Ool00`00Oomoo`02Ool01000Oomoo`001goo00<007ooOol00Woo00@007ooOol000Moo`03001oogoo
009oo`03001oo`0000Qoo`006Goo0P001Woo00<007ooOol017oo0P003Woo0P003Woo0`0037oo0P00
2Woo0P001Goo0P001goo0P001Goo0P001goo0P001Goo0P0027oo000IOol00`00Oomoo`06Ool00`00
Oomoo`1_Ool001Uoo`03001oogoo00Ioo`03001oogoo06moo`005WooOP00000IOol01@00Oomoogoo
00000goo0P000Woo00D007ooOomoo`0000=oo`05001oogooOol00003Ool01@00Oomoogoo00000goo
00D007ooOomoo`0000=oo`05001oogooOol00003Ool01000Oomoo`000goo00D007ooOomoo`0000=o
o`05001oogooOol00003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol0
0003Ool01@00Oomoogoo00000goo00D007ooOomoo`0000=oo`05001oogooOol00003Ool001Uoo`03
001oogoo00Ioo`03001oogoo06moo`006Goo00<007ooOol01Woo00<007ooOol0Kgoo000IOol20007
Ool00`00Oomoo`1_Ool001Uoo`03001oogoo00Ioo`03001oogoo06moo`006Goo00<007ooOol01goo
00<007ooOol0KWoo000IOol20008Ool00`00Oomoo`1^Ool001Uoo`03001oogoo00Moo`03001oogoo
06ioo`006Goo00<007ooOol01goo00<007ooOol0KWoo000IOol20008Ool00`00Oomoo`1^Ool001Uo
o`03001oogoo00Moo`03001oogoo06ioo`006Goo00<007ooOol01goo00<007ooOol0KWoo000IOol0
0`00Oomoo`07Ool00`00Oomoo`1^Ool001Uoo`8000Qoo`03001oogoo06ioo`001goo0`0017oo0P00
0goo0`000goo00<007ooOol027oo00<007ooOol0KGoo000:Ool01000Oomoogoo0P001Woo00@007oo
Ool000Yoo`03001oogoo06eoo`0027oo0`002goo00<007ooOol00P002Goo00<007ooOol0KGoo0007
Ool01000Oomoo`0027oo0`000goo00<007ooOol027oo00<007ooOol0KGoo0007Ool01000Oomoo`00
27oo00<007ooOol00goo00<007ooOol027oo00<007ooOol0KGoo0008Ool20009Ool40002Ool00`00
Oomoo`08Ool00`00Oomoo`1]Ool001Uoo`8000Uoo`03001oogoo06eoo`006Goo00<007ooOol02Goo
00<007ooOol0K7oo000IOol00`00Oomoo`09Ool00`00Oomoo`1/Ool001Uoo`8000Yoo`03001oogoo
06aoo`006Goo00<007ooOol02Goo00<007ooOol0K7oo000IOol00`00Oomoo`09Ool00`00Oomoo`1/
Ool001Uoo`03001oogoo00Uoo`03001oogoo06aoo`006Goo0P002Woo00<007ooOol0K7oo000IOol0
0`00Oomoo`0:Ool00`00Oomoo`0iOol_0003Ool001Uoo`03001oogoo00Yoo`03001oogoo03Aoo`D0
039oo`006Goo0P002goo00<007ooOol0;goo1@00=goo000IOol00`00Oomoo`0:Ool00`00Oomoo`0Z
Ool5000lOol000eoo`<000Aoo`8000=oo`03001oogoo00Yoo`03001oogoo02Moo`<0045oo`003Woo
00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00Yoo`03001oogoo02Ioo`8004=oo`003Woo
00<007ooOol00Woo00@007ooOol0009oo`8000]oo`03001oogoo02Ioo`03001oogoo049oo`003Woo
00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00Yoo`03001oogoo02Aoo`8004Eoo`003Woo
00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00Yoo`03001oogoo02=oo`03001oogoo04Eo
o`003Goo0P001Goo0P000goo0P002goo00<007ooOol08Woo00<007ooOol0AWoo000IOol00`00Oomo
o`0;Ool00`00Oomoo`0QOol00`00Oomoo`16Ool001Uoo`03001oogoo00]oo`03001oogoo021oo`03
001oogoo04Moo`006Goo00<007ooOol02goo00<007ooOol07goo00<007ooOol0B7oo000IOol2000<
Ool00`00Oomoo`0NOol00`00Oomoo`19Ool001Uoo`03001oogoo00]oo`03001oogoo01eoo`03001o
ogoo04Yoo`006Goo00<007ooOol02goo00<007ooOol077oo00<007ooOol0Bgoo000IOol2000<Ool0
0`00Oomoo`0KOol00`00Oomoo`1<Ool001Uoo`03001oogoo00]oo`03001oogoo01Yoo`03001oogoo
04eoo`006Goo00<007ooOol02goo00<007ooOol06Goo00<007ooOol0CWoo000IOol00`00Oomoo`0<
Ool00`00Oomoo`0GOol00`00Oomoo`1?Ool001Uoo`8000eoo`03001oogoo01Ioo`03001oogoo051o
o`0000Aoo`00000000Aoo`8000Aoo`8000=oo`<000=oo`03001oogoo00aoo`03001oogoo00eoo`X0
059oo`000Woo00<007ooOol00Woo00@007ooOol000=oo`8000Ioo`04001oogoo000>Ool00`00Oomo
o`0;Ool2001LOol0009oo`03001oogoo009oo`04001oogoo000;Ool00`00Oomoo`02000=Ool00`00
Oomoo`09Ool2001NOol0009oo`03001oogoo009oo`04001oogoo0008Ool30003Ool00`00Oomoo`0<
Ool00`00Oomoo`08Ool00`00Oomoo`1NOol0009oo`03001oogoo009oo`04001oogoo0008Ool00`00
Oomoo`03Ool00`00Oomoo`0<Ool00`00Oomoo`06Ool2001QOol00003Ool0000000Eoo`8000Uoo`@0
009oo`03001oogoo00aoo`03001oogoo00Eoo`03001oogoo065oo`006Goo0P003Goo00<007ooOol0
17oo00<007ooOol0HWoo000IOol00`00Oomoo`0<Ool00`00Oomoo`03Ool00`00Oomoo`1SOol001Uo
o`03001oogoo00aoo`03001oogoo009oo`03001oogoo06Aoo`006Goo0P003Woo00@007ooOol006Mo
o`006Goo00<007ooOol03Goo00<007oo0000J7oo000IOol00`00Oomoo`0=Ool2001YOol001Uoo`80
00ioo`03001oogoo06Qoo`006Goo00<007ooOol0N7oo000IOol00`00Oomoo`1hOol009Aoo`00U7oo
002DOol009Aoo`00U7oo002DOol00001\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-3.22769, 8.41922, 
  0.126281, 0.0288362}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Discount Factors to Forward LIBOR Rates Calculation", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Given the discount factor structure we have defined a function ",
  StyleBox["yieldfunc[T]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", based on linear interpolation of the known yield values. This allows us \
to define an interpolated discount function"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(DiscFunc[T_]\  := \ 1/\((1\  + \ yieldfunc[T]/100)\)^T\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[DiscFunc[T], \ {T, \ 0, \ 30}]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.031746 0.0147151 0.588604 [
[.18254 .00222 -3 -9 ]
[.18254 .00222 3 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.65873 .00222 -6 -9 ]
[.65873 .00222 6 0 ]
[.81746 .00222 -6 -9 ]
[.81746 .00222 6 0 ]
[.97619 .00222 -6 -9 ]
[.97619 .00222 6 0 ]
[.01131 .13244 -18 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -18 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(10)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(15)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(20)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(25)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(30)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(0.2)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.4)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(0.6)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(0.8)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(1)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .54358 L
.10458 .46953 L
.14415 .41838 L
.18221 .37354 L
.22272 .33643 L
.26171 .30434 L
.30316 .27175 L
.34309 .24415 L
.3815 .21956 L
.42237 .19629 L
.46172 .17638 L
.49955 .1593 L
.53984 .1431 L
.57861 .12922 L
.61984 .1161 L
.65954 .10489 L
.69774 .09528 L
.73838 .08617 L
.77751 .07838 L
.81909 .07103 L
.85916 .06475 L
.89771 .05937 L
.93871 .05428 L
.97619 .05014 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol002Eo
o`<000moo`<000Aoo`8000]oo`<000=oo`<000]oo`@000=oo`8000aoo`@0009oo`<000]oo`<000Ao
o`8000=oo`00:7oo00<007ooOol03Goo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo00Eo
o`03001oogoo00Uoo`03001oogoo009oo`04001oogoo000<Ool00`00Oomoo`05Ool00`00Oomoo`0;
Ool01000Oomoo`000Woo0@000Goo0Goo000XOol00`00Oomoo`0=Ool00`00Oomoo`02Ool01000Oomo
o`002goo00<007ooOol01Goo00<007ooOol02Woo00D007ooOomoo`00009oo`03001oogoo00]oo`03
001oogoo00Aoo`03001oogoo00]oo`04001oogoo0002Ool10001Ool1Ool002Eoo`<0011oo`03001o
ogoo009oo`04001oogoo000;Ool00`00Oomoo`02Ool3000>Ool01000Oomoo`000Woo00<007ooOol0
37oo00<007ooOol00`0037oo0P000goo00@007ooOol0009oo`009Goo00<007ooOol047oo00<007oo
Ool00Woo00@007ooOol000]oo`03001oogoo009oo`03001oogoo00]oo`04001oogoo0002Ool01000
Oomoo`002goo00@007ooOol0009oo`03001oogoo00ioo`04001oogoo0002Ool10001Ool1Ool002Eo
o`@000ioo`8000Eoo`8000]oo`8000Aoo`@000]oo`8000Aoo`8000eoo`8000=oo`@000Yoo`<000Ao
o`8000=oo`00U7oo000COol00`00Oomoo`1nOol001=oo`03001oogoo07ioo`0047ooP@000goo000C
Ool00`00Oomoo`1nOol001=oo`03001oogoo07ioo`004goo0P00Ogoo000COol00`00Oomoo`1eOol3
0006Ool001=oo`03001oogoo06]oo`X000Uoo`004goo00<007ooOol0IWoo1@004goo000COol2001Q
Ool6000HOol001=oo`03001oogoo05]oo`D001ioo`004goo00<007ooOol0EWoo1@008goo000COol0
0`00Oomoo`1AOol5000XOol001=oo`8004ioo`@002eoo`004goo00<007ooOol0BWoo0`00<Goo0002
Ool20004Ool20003Ool40002Ool00`00Oomoo`17Ool3000dOol00005Ool007ooOol00003Ool20004
Ool00`00Oomoo`02Ool00`00Oomoo`13Ool4000gOol00005Ool007ooOol0000:Ool01000Oomoogoo
0P00@Goo0`00>goo00001Goo001oogoo00002goo00@007ooOol0041oo`8003ioo`0000Eoo`00Oomo
o`0000Qoo`04001oogoo0002Ool00`00Oomoo`0kOol30010Ool0009oo`8000Yoo`8000=oo`8003Yo
o`8004=oo`004goo00<007ooOol0=Woo0`00AGoo000COol00`00Oomoo`0dOol20018Ool001=oo`03
001oogoo039oo`8004Yoo`004goo0P00<Woo00<007ooOol0BWoo000COol00`00Oomoo`0_Ool2001=
Ool001=oo`03001oogoo02eoo`8004moo`004goo00<007ooOol0;7oo00<007ooOol0Cgoo000COol2
000[Ool2001BOol001=oo`03001oogoo02Qoo`8005Aoo`000Woo0P0017oo0P0017oo0`000Woo00<0
07ooOol09goo00<007ooOol0E7oo00001Goo001oogoo00000goo0P001Goo00D007ooOomoo`0002Mo
o`8005Moo`0000Eoo`00Oomoo`0000Qoo`@0009oo`8002Eoo`03001oogoo05Moo`0000Eoo`00Oomo
o`0000Qoo`03001oo`0000=oo`03001oogoo02=oo`03001oogoo05Qoo`0000Eoo`00Oomoo`0000Qo
o`03001oo`0000=oo`03001oogoo029oo`03001oogoo05Uoo`000Woo0P002Woo0P000goo00<007oo
Ool087oo0P00G7oo000COol2000POol00`00Oomoo`1LOol001=oo`03001oogoo01ioo`03001oogoo
05eoo`004goo00<007ooOol07Goo00<007ooOol0GWoo000COol2000LOol2001QOol001=oo`03001o
ogoo01Yoo`03001oogoo065oo`004goo00<007ooOol06Goo00<007ooOol0HWoo000COol00`00Oomo
o`0HOol00`00Oomoo`1SOol001=oo`8001Moo`8006Ioo`004goo00<007ooOol05Goo00<007ooOol0
IWoo0002Ool20004Ool20004Ool20003Ool00`00Oomoo`0DOol00`00Oomoo`1WOol00005Ool007oo
Ool00003Ool20003Ool01000Oomoo`000Woo00<007ooOol04goo00<007ooOol0J7oo00001Goo001o
ogoo000027oo00@007ooOol0009oo`8001=oo`03001oogoo06Uoo`0000Eoo`00Oomoo`0000Qoo`<0
00=oo`03001oogoo015oo`03001oogoo06Yoo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`03
001oogoo011oo`03001oogoo06]oo`000Woo0P002Woo0`000Woo00<007ooOol03goo00<007ooOol0
K7oo000COol2000?Ool00`00Oomoo`1]Ool001=oo`03001oogoo00eoo`03001oogoo06ioo`004goo
00<007ooOol037oo00<007ooOol0Kgoo000COol2000=Ool00`00Oomoo`1_Ool001=oo`03001oogoo
00]oo`03001oogoo071oo`004goo00<007ooOol02Woo00<007ooOol0LGoo000COol00`00Oomoo`09
Ool00`00Oomoo`1bOol001=oo`8000Yoo`03001oogoo079oo`004goo00<007ooOol027oo00<007oo
Ool0Lgoo0002Ool20004Ool20004Ool20003Ool00`00Oomoo`07Ool00`00Oomoo`1dOol00005Ool0
07ooOol00003Ool20003Ool01000Oomoo`000Woo00<007ooOol01goo00<007ooOol0M7oo00001Goo
001oogoo000027oo00@007ooOol0009oo`8000Moo`03001oogoo07Eoo`0000Eoo`00Oomoo`0000Uo
o`8000=oo`03001oogoo00Ioo`03001oogoo07Eoo`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002
Ool00`00Oomoo`05Ool00`00Oomoo`1fOol0009oo`8000Yoo`8000=oo`03001oogoo00Aoo`03001o
ogoo07Moo`004goo0P001Goo00<007ooOol0Mgoo000COol00`00Oomoo`03Ool00`00Oomoo`1hOol0
01=oo`03001oogoo00=oo`03001oogoo07Qoo`004goo0P000goo00<007ooOol0NGoo000COol00`00
Oomoo`02Ool00`00Oomoo`1iOol001=oo`05001oogooOol0001lOol001=oo`04001oogoo001mOol0
01=oo`800003Ool007oo07aoo`004goo00<007oo0000OWoo000=Ool30003Ool00`00Ool0001nOol0
00ioo`03001oogoo009oo`8007moo`003Woo00<007ooOol00Woo0P00Ogoo000>Ool00`00Oomoo`02
Ool00`00Oomoo`1nOol000ioo`03001oogoo009oo`03001oogoo07ioo`003Goo0P00QGoo002DOol0
0001\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-4.83052, -0.142482, 
  0.24648, 0.0132937}}]
}, Open  ]],

Cell["\<\
Note that this gives us a suitable interpolation of the discrete \
discount factor points already calculated, with a suitable exponential decay \
for times greater than 10 years, with the decay factor being that already \
computed at 10 years. We can see this by overlaying the plot of the \
interpolated discount function with the plot given previously:\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[%, \ sadplot]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.031746 0.0147151 0.588604 [
[.18254 .00222 -3 -9 ]
[.18254 .00222 3 0 ]
[.34127 .00222 -6 -9 ]
[.34127 .00222 6 0 ]
[.5 .00222 -6 -9 ]
[.5 .00222 6 0 ]
[.65873 .00222 -6 -9 ]
[.65873 .00222 6 0 ]
[.81746 .00222 -6 -9 ]
[.81746 .00222 6 0 ]
[.97619 .00222 -6 -9 ]
[.97619 .00222 6 0 ]
[.01131 .13244 -18 -4.5 ]
[.01131 .13244 0 4.5 ]
[.01131 .25016 -18 -4.5 ]
[.01131 .25016 0 4.5 ]
[.01131 .36788 -18 -4.5 ]
[.01131 .36788 0 4.5 ]
[.01131 .4856 -18 -4.5 ]
[.01131 .4856 0 4.5 ]
[.01131 .60332 -6 -4.5 ]
[.01131 .60332 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.18254 .01472 m
.18254 .02097 L
s
[(5)] .18254 .00222 0 1 Mshowa
.34127 .01472 m
.34127 .02097 L
s
[(10)] .34127 .00222 0 1 Mshowa
.5 .01472 m
.5 .02097 L
s
[(15)] .5 .00222 0 1 Mshowa
.65873 .01472 m
.65873 .02097 L
s
[(20)] .65873 .00222 0 1 Mshowa
.81746 .01472 m
.81746 .02097 L
s
[(25)] .81746 .00222 0 1 Mshowa
.97619 .01472 m
.97619 .02097 L
s
[(30)] .97619 .00222 0 1 Mshowa
.125 Mabswid
.05556 .01472 m
.05556 .01847 L
s
.0873 .01472 m
.0873 .01847 L
s
.11905 .01472 m
.11905 .01847 L
s
.15079 .01472 m
.15079 .01847 L
s
.21429 .01472 m
.21429 .01847 L
s
.24603 .01472 m
.24603 .01847 L
s
.27778 .01472 m
.27778 .01847 L
s
.30952 .01472 m
.30952 .01847 L
s
.37302 .01472 m
.37302 .01847 L
s
.40476 .01472 m
.40476 .01847 L
s
.43651 .01472 m
.43651 .01847 L
s
.46825 .01472 m
.46825 .01847 L
s
.53175 .01472 m
.53175 .01847 L
s
.56349 .01472 m
.56349 .01847 L
s
.59524 .01472 m
.59524 .01847 L
s
.62698 .01472 m
.62698 .01847 L
s
.69048 .01472 m
.69048 .01847 L
s
.72222 .01472 m
.72222 .01847 L
s
.75397 .01472 m
.75397 .01847 L
s
.78571 .01472 m
.78571 .01847 L
s
.84921 .01472 m
.84921 .01847 L
s
.88095 .01472 m
.88095 .01847 L
s
.9127 .01472 m
.9127 .01847 L
s
.94444 .01472 m
.94444 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .13244 m
.03006 .13244 L
s
[(0.2)] .01131 .13244 1 0 Mshowa
.02381 .25016 m
.03006 .25016 L
s
[(0.4)] .01131 .25016 1 0 Mshowa
.02381 .36788 m
.03006 .36788 L
s
[(0.6)] .01131 .36788 1 0 Mshowa
.02381 .4856 m
.03006 .4856 L
s
[(0.8)] .01131 .4856 1 0 Mshowa
.02381 .60332 m
.03006 .60332 L
s
[(1)] .01131 .60332 1 0 Mshowa
.125 Mabswid
.02381 .04415 m
.02756 .04415 L
s
.02381 .07358 m
.02756 .07358 L
s
.02381 .10301 m
.02756 .10301 L
s
.02381 .16187 m
.02756 .16187 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .30902 m
.02756 .30902 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .45617 m
.02756 .45617 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .60332 m
.06244 .54358 L
.10458 .46953 L
.14415 .41838 L
.18221 .37354 L
.22272 .33643 L
.26171 .30434 L
.30316 .27175 L
.34309 .24415 L
.3815 .21956 L
.42237 .19629 L
.46172 .17638 L
.49955 .1593 L
.53984 .1431 L
.57861 .12922 L
.61984 .1161 L
.65954 .10489 L
.69774 .09528 L
.73838 .08617 L
.77751 .07838 L
.81909 .07103 L
.85916 .06475 L
.89771 .05937 L
.93871 .05428 L
.97619 .05014 L
s
.02 w
.0239 .60317 Mdot
.02442 .60232 Mdot
.02646 .59926 Mdot
.03175 .59114 Mdot
.03968 .57906 Mdot
.05556 .55592 Mdot
.07143 .52659 Mdot
.0873 .49428 Mdot
.10317 .47148 Mdot
.11905 .45012 Mdot
.13492 .42971 Mdot
.15079 .41047 Mdot
.16667 .39136 Mdot
.18254 .37317 Mdot
.19841 .35789 Mdot
.21429 .34362 Mdot
.23016 .33029 Mdot
.24603 .31785 Mdot
.2619 .30418 Mdot
.27778 .29119 Mdot
.29365 .27885 Mdot
.30952 .26712 Mdot
.3254 .25597 Mdot
.34127 .24538 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol002Eo
o`<000moo`<000Aoo`8000]oo`<000=oo`<000]oo`@000=oo`8000aoo`@0009oo`<000]oo`<000Ao
o`8000=oo`00:7oo00<007ooOol03Goo00<007ooOol00Woo00@007ooOol000]oo`03001oogoo00Eo
o`03001oogoo00Uoo`03001oogoo009oo`04001oogoo000<Ool00`00Oomoo`05Ool00`00Oomoo`0;
Ool01000Oomoo`000Woo0@000Goo0Goo000XOol00`00Oomoo`0=Ool00`00Oomoo`02Ool01000Oomo
o`002goo00<007ooOol01Goo00<007ooOol02Woo00D007ooOomoo`00009oo`03001oogoo00]oo`03
001oogoo00Aoo`03001oogoo00]oo`04001oogoo0002Ool10001Ool1Ool002Eoo`<0011oo`03001o
ogoo009oo`04001oogoo000;Ool00`00Oomoo`02Ool3000>Ool01000Oomoo`000Woo00<007ooOol0
37oo00<007ooOol00`0037oo0P000goo00@007ooOol0009oo`009Goo00<007ooOol047oo00<007oo
Ool00Woo00@007ooOol000]oo`03001oogoo009oo`03001oogoo00]oo`04001oogoo0002Ool01000
Oomoo`002goo00@007ooOol0009oo`03001oogoo00ioo`04001oogoo0002Ool10001Ool1Ool002Eo
o`@000ioo`8000Eoo`8000]oo`8000Aoo`@000]oo`8000Aoo`8000eoo`8000=oo`@000Yoo`<000Ao
o`8000=oo`00U7oo000COol00`00Oomoo`1nOol001=oo`03001oogoo07ioo`0047ooP@000goo000C
Ool00`00Oomoo`1nOol001=oo`03001oogoo07ioo`004goo0P00Ogoo000COol00`00Oomoo`1eOol3
0006Ool001=oo`03001oogoo06]oo`X000Uoo`004goo00<007ooOol0IWoo1@004goo000COol2001Q
Ool6000HOol001=oo`03001oogoo05]oo`D001ioo`004goo00<007ooOol0EWoo1@008goo000COol0
0`00Oomoo`1AOol5000XOol001=oo`8004ioo`@002eoo`004goo00<007ooOol0BWoo0`00<Goo0002
Ool20004Ool20003Ool40002Ool00`00Oomoo`17Ool3000dOol00005Ool007ooOol00003Ool20004
Ool00`00Oomoo`02Ool00`00Oomoo`13Ool4000gOol00005Ool007ooOol0000:Ool01000Oomoogoo
0P00@Goo0`00>goo00001Goo001oogoo00002goo00@007ooOol0041oo`8003ioo`0000Eoo`00Oomo
o`0000Qoo`04001oogoo0002Ool00`00Oomoo`0kOol30010Ool0009oo`8000Yoo`8000=oo`8003Yo
o`8004=oo`004goo00<007ooOol0=Woo0`00AGoo000COol00`00Oomoo`0dOol20018Ool001=oo`03
001oogoo039oo`8004Yoo`004goo0P00<Woo00<007ooOol0BWoo000COol00`00Oomoo`0_Ool2001=
Ool001=oo`03001oogoo02eoo`8004moo`004goo00<007ooOol0;7oo00<007ooOol0Cgoo000COol2
000[Ool2001BOol001=oo`03001oogoo02Qoo`8005Aoo`000Woo0P0017oo0P0017oo0`000Woo00<0
07ooOol09goo00<007ooOol0E7oo00001Goo001oogoo00000goo0P001Goo00D007ooOomoo`0002Mo
o`8005Moo`0000Eoo`00Oomoo`0000Qoo`@0009oo`8002Aoo`@005Moo`0000Eoo`00Oomoo`0000Qo
o`03001oo`0000=oo`03001oogoo02=oo`8005Uoo`0000Eoo`00Oomoo`0000Qoo`03001oo`0000=o
o`03001oogoo025oo`8005]oo`000Woo0P002Woo0P000goo00<007ooOol07goo1000Fgoo000COol2
000POol2001MOol001=oo`03001oogoo01eoo`8005moo`004goo00<007ooOol06goo1000Ggoo000C
Ool2000LOol2001QOol001=oo`03001oogoo01Uoo`8006=oo`004goo00<007ooOol06Goo0P00Hgoo
000COol00`00Oomoo`0GOol2001UOol001=oo`8001Ioo`@006Eoo`004goo00<007ooOol05Goo0P00
Igoo0002Ool20004Ool20004Ool20003Ool00`00Oomoo`0COol2001YOol00005Ool007ooOol00003
Ool20003Ool01000Oomoo`000Woo00<007ooOol04goo0P00JGoo00001Goo001oogoo000027oo00@0
07ooOol0009oo`80019oo`8006]oo`0000Eoo`00Oomoo`0000Qoo`<000=oo`03001oogoo015oo`80
06]oo`0000Eoo`00Oomoo`0000Qoo`03001oogoo00=oo`03001oogoo011oo`03001oogoo06]oo`00
0Woo0P002Woo0`000Woo00<007ooOol03goo0P00KGoo000COol2000?Ool3001]Ool001=oo`03001o
ogoo00eoo`8006moo`004goo00<007ooOol037oo0`00Kgoo000COol2000<Ool2001aOol001=oo`03
001oogoo00]oo`80075oo`004goo00<007ooOol02Woo00<007ooOol0LGoo000COol00`00Oomoo`09
Ool2001cOol001=oo`8000Yoo`8007=oo`004goo00<007ooOol027oo00<007ooOol0Lgoo0002Ool2
0004Ool20004Ool20003Ool00`00Oomoo`06Ool2001fOol00005Ool007ooOol00003Ool20003Ool0
1000Oomoo`000Woo00<007ooOol01Woo0P00MWoo00001Goo001oogoo000027oo00@007ooOol0009o
o`8000Moo`03001oogoo07Eoo`0000Eoo`00Oomoo`0000Uoo`8000=oo`03001oogoo00Aoo`<007Mo
o`0000Eoo`00Oomoo`0000Qoo`04001oogoo0002Ool00`00Oomoo`04Ool2001hOol0009oo`8000Yo
o`8000=oo`03001oogoo00Aoo`03001oogoo07Moo`004goo0P001Goo00<007ooOol0Mgoo000COol0
0`00Oomoo`02Ool2001jOol001=oo`03001oogoo009oo`8007Yoo`004goo0P000goo00<007ooOol0
NGoo000COol00`00Oomoo`02Ool00`00Oomoo`1iOol001=oo`03001oogoo008007aoo`004goo00<0
07ooOol00P00O7oo000COol200000goo001oo`1lOol001=oo`<007ioo`003Goo0`000goo0`00OWoo
000>Ool01000Oomoogoo0`00Ogoo000>Ool01000Oomoogoo0`00Ogoo000>Ool01000Oomoogoo0P00
P7oo000>Ool00`00Oomoo`02Ool00`00Oomoo`1nOol000eoo`8008Eoo`00U7oo0000\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-4.83052, -0.142482, 
  0.24648, 0.0132937}}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Interest-Rate Forward", "Subsubsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "Given the function ",
  StyleBox["DiscFunc[T]",
    FontFamily->"Courier",
    FontWeight->"Bold"],
  ", we can define the continuously compounded discrete forward rate \
function:"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(CCDFwd[T_, 
        dT_] := \(-\(\(100\ \((Log[DiscFunc[dT + T]] - 
                  Log[DiscFunc[T]])\)\)\/dT\)\)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
The corresponding annually compounded discrete forward rate \
function is\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(ACDFwd[T_, dT_] := 
      100\ \((\((DiscFunc[T]\/DiscFunc[dT + T])\)\^\(1/dT\) - 1)\)\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ACDFwd[1, \ 0.25]\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \(11.2819657084497038`\)], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[ACDFwd[T, \ 0.25], \ {T, \ 0, \ 15}, \ 
      PlotRange\  -> \ All]; \)\)], "Input",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
/Courier findfont 10  scalefont  setfont
% Scaling calculations
0.0238095 0.0634921 -0.786925 0.0963166 [
[.15079 .06743 -3 -9 ]
[.15079 .06743 3 0 ]
[.27778 .06743 -3 -9 ]
[.27778 .06743 3 0 ]
[.40476 .06743 -3 -9 ]
[.40476 .06743 3 0 ]
[.53175 .06743 -3 -9 ]
[.53175 .06743 3 0 ]
[.65873 .06743 -6 -9 ]
[.65873 .06743 6 0 ]
[.78571 .06743 -6 -9 ]
[.78571 .06743 6 0 ]
[.9127 .06743 -6 -9 ]
[.9127 .06743 6 0 ]
[.01131 .17624 -12 -4.5 ]
[.01131 .17624 0 4.5 ]
[.01131 .27256 -12 -4.5 ]
[.01131 .27256 0 4.5 ]
[.01131 .36887 -12 -4.5 ]
[.01131 .36887 0 4.5 ]
[.01131 .46519 -12 -4.5 ]
[.01131 .46519 0 4.5 ]
[.01131 .56151 -12 -4.5 ]
[.01131 .56151 0 4.5 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15079 .07993 m
.15079 .08618 L
s
[(2)] .15079 .06743 0 1 Mshowa
.27778 .07993 m
.27778 .08618 L
s
[(4)] .27778 .06743 0 1 Mshowa
.40476 .07993 m
.40476 .08618 L
s
[(6)] .40476 .06743 0 1 Mshowa
.53175 .07993 m
.53175 .08618 L
s
[(8)] .53175 .06743 0 1 Mshowa
.65873 .07993 m
.65873 .08618 L
s
[(10)] .65873 .06743 0 1 Mshowa
.78571 .07993 m
.78571 .08618 L
s
[(12)] .78571 .06743 0 1 Mshowa
.9127 .07993 m
.9127 .08618 L
s
[(14)] .9127 .06743 0 1 Mshowa
.125 Mabswid
.05556 .07993 m
.05556 .08368 L
s
.0873 .07993 m
.0873 .08368 L
s
.11905 .07993 m
.11905 .08368 L
s
.18254 .07993 m
.18254 .08368 L
s
.21429 .07993 m
.21429 .08368 L
s
.24603 .07993 m
.24603 .08368 L
s
.30952 .07993 m
.30952 .08368 L
s
.34127 .07993 m
.34127 .08368 L
s
.37302 .07993 m
.37302 .08368 L
s
.43651 .07993 m
.43651 .08368 L
s
.46825 .07993 m
.46825 .08368 L
s
.5 .07993 m
.5 .08368 L
s
.56349 .07993 m
.56349 .08368 L
s
.59524 .07993 m
.59524 .08368 L
s
.62698 .07993 m
.62698 .08368 L
s
.69048 .07993 m
.69048 .08368 L
s
.72222 .07993 m
.72222 .08368 L
s
.75397 .07993 m
.75397 .08368 L
s
.81746 .07993 m
.81746 .08368 L
s
.84921 .07993 m
.84921 .08368 L
s
.88095 .07993 m
.88095 .08368 L
s
.94444 .07993 m
.94444 .08368 L
s
.97619 .07993 m
.97619 .08368 L
s
.25 Mabswid
0 .07993 m
1 .07993 L
s
.02381 .17624 m
.03006 .17624 L
s
[(10)] .01131 .17624 1 0 Mshowa
.02381 .27256 m
.03006 .27256 L
s
[(11)] .01131 .27256 1 0 Mshowa
.02381 .36887 m
.03006 .36887 L
s
[(12)] .01131 .36887 1 0 Mshowa
.02381 .46519 m
.03006 .46519 L
s
[(13)] .01131 .46519 1 0 Mshowa
.02381 .56151 m
.03006 .56151 L
s
[(14)] .01131 .56151 1 0 Mshowa
.125 Mabswid
.02381 .09919 m
.02756 .09919 L
s
.02381 .11845 m
.02756 .11845 L
s
.02381 .13772 m
.02756 .13772 L
s
.02381 .15698 m
.02756 .15698 L
s
.02381 .19551 m
.02756 .19551 L
s
.02381 .21477 m
.02756 .21477 L
s
.02381 .23403 m
.02756 .23403 L
s
.02381 .2533 m
.02756 .2533 L
s
.02381 .29182 m
.02756 .29182 L
s
.02381 .31109 m
.02756 .31109 L
s
.02381 .33035 m
.02756 .33035 L
s
.02381 .34961 m
.02756 .34961 L
s
.02381 .38814 m
.02756 .38814 L
s
.02381 .4074 m
.02756 .4074 L
s
.02381 .42666 m
.02756 .42666 L
s
.02381 .44593 m
.02756 .44593 L
s
.02381 .48445 m
.02756 .48445 L
s
.02381 .50372 m
.02756 .50372 L
s
.02381 .52298 m
.02756 .52298 L
s
.02381 .54224 m
.02756 .54224 L
s
.02381 .06066 m
.02756 .06066 L
s
.02381 .0414 m
.02756 .0414 L
s
.02381 .02214 m
.02756 .02214 L
s
.02381 .00287 m
.02756 .00287 L
s
.02381 .58077 m
.02756 .58077 L
s
.02381 .60003 m
.02756 .60003 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
.5 Mabswid
.02381 .05337 m
.02499 .04981 L
.02605 .04881 L
.02729 .05032 L
.02846 .05439 L
.02954 .05806 L
.03053 .05878 L
.03279 .06033 L
.03756 .06318 L
.03872 .06378 L
.03997 .06421 L
.04068 .06409 L
.04133 .06396 L
.04262 .06363 L
.04514 .06273 L
.04748 .0616 L
.05278 .058 L
.05394 .05702 L
.05519 .05588 L
.05637 .05542 L
.05746 .05527 L
.06007 .05491 L
.06244 .05458 L
.06488 .05425 L
.0675 .05389 L
.06876 .05371 L
.06996 .05355 L
.07104 .0534 L
.07221 .06309 L
.0735 .07958 L
.07491 .09855 L
.07778 .13948 L
.08293 .22149 L
.08555 .2674 L
.08624 .27998 L
.08698 .29365 L
.08767 .302 L
.08831 .30588 L
.08975 .31473 L
.09133 .32442 L
.0941 .34142 L
.10458 .40656 L
.12376 .53464 L
.12902 .56799 L
.1319 .58624 L
.1333 .59516 L
.13459 .60332 L
.13576 .58663 L
.13688 .56119 L
.13934 .50334 L
Mistroke
.14456 .37274 L
.14694 .30984 L
.14819 .27594 L
.14951 .23957 L
.15067 .20715 L
.15194 .20296 L
.1531 .20218 L
.15417 .20145 L
.15954 .19782 L
.16441 .19453 L
.17329 .19005 L
.17791 .18807 L
.18044 .18701 L
.18167 .1865 L
.18281 .18597 L
.18402 .1852 L
.18533 .18438 L
.18811 .18262 L
.19301 .17953 L
.19581 .17776 L
.19717 .1769 L
.19841 .17612 L
.19972 .17693 L
.20112 .17782 L
.20409 .1798 L
.20906 .18339 L
.21168 .1854 L
.21311 .18654 L
.21381 .18711 L
.21445 .18743 L
.21562 .18693 L
.21687 .1864 L
.21954 .18527 L
.22422 .18328 L
.24328 .17623 L
.2485 .17427 L
.25406 .17198 L
.2588 .17002 L
.2601 .16948 L
.26078 .1692 L
.2615 .1689 L
.26213 .16935 L
.26282 .17126 L
.26403 .17465 L
.26886 .18849 L
.2741 .2042 L
.27651 .21164 L
.27772 .21545 L
.27841 .21562 L
.27905 .21561 L
Mistroke
.28362 .21553 L
.28891 .21544 L
.29043 .21542 L
.29187 .21539 L
.29317 .21537 L
.29391 .21536 L
.29459 .21537 L
.30456 .21552 L
.30703 .21556 L
.30838 .21559 L
.30964 .2156 L
.3103 .21559 L
.31101 .21558 L
.31251 .21556 L
.31514 .21553 L
.32026 .21546 L
.32277 .21542 L
.32388 .21541 L
.3251 .21539 L
.32627 .20909 L
.32736 .20113 L
.32976 .18348 L
.33484 .14547 L
.33706 .12848 L
.33944 .11012 L
.34059 .10112 L
.34168 .09549 L
.34265 .09477 L
.34371 .094 L
.34841 .09057 L
.35355 .08682 L
.36391 .07932 L
.38242 .0659 L
.40192 .05144 L
.42336 .03563 L
.43318 .02804 L
.43859 .02394 L
.44373 .02018 L
.4485 .01669 L
.4498 .01573 L
.45052 .01521 L
.45119 .01472 L
.4525 .01479 L
.45369 .02423 L
.46278 .09874 L
.46504 .11784 L
.46628 .12845 L
.46746 .13854 L
.46812 .14429 L
.46884 .14537 L
Mistroke
.47011 .14523 L
.47258 .14496 L
.4776 .1444 L
.48034 .1441 L
.48291 .14382 L
.48512 .14349 L
.48749 .14301 L
.49253 .14199 L
.5014 .14029 L
.54262 .134 L
.58231 .12739 L
.62049 .12043 L
.63062 .11848 L
.63558 .11781 L
.64016 .11718 L
.64148 .117 L
.64272 .11683 L
.64402 .11982 L
.64544 .12351 L
.65115 .13856 L
.65366 .14526 L
.65633 .15245 L
.65743 .15543 L
.65861 .15864 L
.65991 .15897 L
.66113 .15897 L
.6624 .15897 L
.66304 .15897 L
.66373 .15897 L
.66497 .15897 L
.66613 .15897 L
.66684 .15897 L
.66751 .15897 L
.66876 .15897 L
.66946 .15897 L
.67011 .15897 L
.67082 .15897 L
.67157 .15897 L
.67274 .15897 L
.674 .15897 L
.67519 .15897 L
.67628 .15897 L
.68147 .15897 L
.70002 .15897 L
.70923 .15897 L
.71904 .15897 L
.72043 .15897 L
.72122 .15897 L
.72194 .15897 L
.72258 .15897 L
Mistroke
.72328 .15897 L
.724 .15897 L
.72468 .15897 L
.72597 .15897 L
.72715 .15897 L
.7278 .15897 L
.72851 .15897 L
.72979 .15897 L
.73099 .15897 L
.73231 .15897 L
.73498 .15897 L
.73617 .15897 L
.73745 .15897 L
.73866 .15897 L
.73977 .15897 L
.74105 .15897 L
.74226 .15897 L
.74335 .15897 L
.74452 .15897 L
.7458 .15897 L
.74651 .15897 L
.74716 .15897 L
.74832 .15897 L
.74959 .15897 L
.75077 .15897 L
.7519 .15897 L
.7529 .15897 L
.754 .15897 L
.75872 .15897 L
.77943 .15897 L
.82017 .15897 L
.85939 .15897 L
.8971 .15897 L
.93726 .15897 L
.9759 .15897 L
.97619 .15897 L
Mfstroke
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ImageSize->{148.312, 91.5625},
  ImageMargins->{{34, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002D0000Fa000`40O003h00OU7oo002DOol009Ao
o`003Goo0P00QGoo000=Ool00`00Oomoo`24Ool000eoo`03001oogoo00eoo`@000ioo`<000ioo`80
00Eoo`8000Uoo`8000]oo`<000Aoo`8000Qoo`<000=oo`@000Qoo`<000Aoo`<000Qoo`003Goo0P00
3goo00<007ooOol03goo00<007ooOol037oo00@007ooOol000=oo`<000Qoo`04001oogoo000;Ool0
0`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol00goo00<007ooOol02Goo00<007ooOol017oo
00<007ooOol01goo000=Ool00`00Oomoo`0?Ool00`00Oomoo`0<Ool4000=Ool01000Oomoo`000Woo
00@007ooOol000Qoo`04001oogoo000;Ool00`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol0
17oo00<007ooOol027oo00<007ooOol00Woo100027oo000=Ool00`00Oomoo`0@Ool00`00Oomoo`0;
Ool00`00Ool0000>Ool300000goo00000003Ool00`00Oomoo`07Ool2000<Ool00`00Oomoo`02Ool0
1000Oomoo`0027oo00<007ooOol01Goo00<007ooOol01goo00<007ooOol00Woo00<007oo00002Goo
000=Ool2000>Ool01000Oomoo`003Goo00<007oo00003Woo00@007ooOol000Eoo`03001oogoo00Io
o`04001oogoo000;Ool00`00Oomoo`02Ool01000Oomoo`0027oo00<007ooOol00Woo00@007ooOol0
00Uoo`03001oogoo009oo`03001oo`0000Uoo`003Goo0P000goo0`002Goo0P003goo0P003goo0`00
1Goo00<007ooOol01goo0P002goo0P001Goo0P0027oo0P001Goo0P002Goo0P001Goo0P002Goo000=
Ool200001Woo00000000Ool002Yoo`03001oogoo00Ioo`03001oogoo04Uoo`003Goo00@007oo0000
00=oo`03001oogoo02Moo`03001oogoo00Qoo`03001oogoo04Qoo`003Goo00<007ooOol017oo00<0
07ooOol09Woo00<007ooOol02Goo00<007ooOol0B7oo000:Oon:000000eoo`03001oogoo00Aoo`03
001oogoo00Qoo`03001oogoo00ioo`03001oogoo00Moo`03001oogoo00Aoo`03001oogoo00Eoo`03
001oogoo00Moo`03001oogoo00ioo`03001oogoo00ioo`03001oogoo00moo`03001oogoo00Yoo`00
3Goo00<007ooOol017oo00<007ooOol08Woo00<007ooOol03Goo00<007ooOol0B7oo000=Ool20005
Ool00`00Oomoo`0ROol00`00Oomoo`0=Ool00`00Oomoo`18Ool000eoo`03001oogoo00Aoo`03001o
ogoo025oo`03001oogoo00moo`03001oogoo04Moo`003Goo0P001Goo00<007ooOol08Goo00<007oo
Ool03goo00<007ooOol047oo1P00<Goo000=Ool00`00Oomoo`05Ool00`00Oomoo`0POol00`00Oomo
o`0?Ool00`00Oomoo`0:Ool60005Ool00`00Oomoo`0_Ool000eoo`03001oogoo00Eoo`03001oogoo
021oo`03001oogoo00moo`03001oogoo00Eoo`D000]oo`03001oogoo02moo`003Goo0P001Woo00<0
07ooOol087oo00<007ooOol03goo00<007ooOol01@004Goo00<007ooOol0;Woo000=Ool00`00Oomo
o`05Ool00`00Oomoo`0POol00`00Oomoo`0?Ool3000FOol00`00Oomoo`0^Ool000eoo`8000Ioo`03
001oogoo021oo`03001oogoo02Uoo`03001oogoo02Uoo`03001oogoo005oo`0000Aoo`00000000Ao
o`8000=oo`03001oogoo00Eoo`03001oogoo021oo`03001oogoo02Uoobd000=oo`000Woo00<007oo
Ool00Woo00@007ooOol0009oo`03001oogoo00Eoo`03001oogoo01Eoo`8000Qoo`03001oogoo05Yo
o`000Woo00<007ooOol00Woo00@007ooOol0009oo`8000Ioo`03001oogoo00aoo`8000Aoo`<00003
Ool007oo00Moo`03001oogoo05Yoo`000Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo
00Eoo`03001oogoo00]oo`800003Ool00000008000Aoo`03001oogoo00Ioo`03001oogoo05Yoo`00
0Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00Eoo`03001oogoo00Uoo`<000=oo`03
001oogoo00Eoo`03001oogoo00Eoo`03001oogoo05Yoo`0000=oo`0000001Goo0P000goo0P001Woo
00<007ooOol01goo0P003Woo00<007ooOol01Goo00<007ooOol0FWoo000=Ool00`00Oomoo`05Ool0
0`00Oomoo`07Ool00`00Oomoo`0=Ool00`00Oomoo`05Ool00`00Oomoo`1JOol000eoo`8000Ioo`03
001oogoo00Moo`03001oogoo00ioo`P005aoo`003Goo00<007ooOol01Goo00<007ooOol01goo00<0
07ooOol0LWoo000=Ool00`00Oomoo`05Ool00`00Oomoo`06Ool00`00Oomoo`1cOol000eoo`8000Io
o`03001oogoo00Ioo`03001oogoo07=oo`003Goo00<007ooOol01Goo00<007ooOol01Woo00<007oo
Ool0Lgoo000=Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`1cOol00004Ool000000003
Ool30003Ool20007Ool00`00Oomoo`05Ool00`00Oomoo`1cOol0009oo`03001oogoo00=oo`03001o
ogoo009oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo07=oo`000Woo00<007ooOol00goo
00<007ooOol00Woo0P001goo00<007ooOol01Goo00<007ooOol0Lgoo0002Ool00`00Oomoo`03Ool0
0`00Oomoo`02Ool00`00Oomoo`06Ool00`00Oomoo`05Ool00`00Oomoo`1cOol0009oo`03001oogoo
00=oo`03001oogoo009oo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo07=oo`0000=oo`00
000017oo0P0017oo0P001goo00<007ooOol01Goo00<007ooOol0Lgoo000=Ool00`00Oomoo`06Ool0
0`00Oomoo`05Ool00`00Oomoo`1cOol000eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
07=oo`003Goo0P001goo00<007ooOol01Goo00<007ooOol0Lgoo000=Ool00`00Oomoo`06Ool00`00
Oomoo`05Ool00`00Oomoo`1cOol000eoo`8000Moo`03001oogoo00Eoo`03001oogoo07=oo`003Goo
00<007ooOol01goo00<007ooOol017oo00<007ooOol0Lgoo000=Ool00`00Oomoo`07Ool00`00Oomo
o`04Ool00`00Oomoo`1cOol00004Ool000000003Ool40002Ool20008Ool00`00Oomoo`04Ool00`00
Oomoo`1cOol0009oo`03001oogoo00=oo`03001oogoo009oo`03001oogoo00Moo`03001oogoo00Ao
o`03001oogoo07=oo`000Woo00<007ooOol017oo00@007ooOomoo`8000Qoo`03001oogoo00Aoo`03
001oogoo07=oo`000Woo00<007ooOol01Goo00@007ooOol000Uoo`03001oogoo00Aoo`03001oogoo
07=oo`000Woo00<007ooOol00Woo00@007ooOol0009oo`03001oogoo00Qoo`03001oogoo00=oo`03
001oogoo07=oo`0000=oo`0000001Goo0P000goo0P002Goo00<007ooOol00goo00<007ooOol0Lgoo
000=Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`1cOol000eoo`03001oogoo00Qoo`03
001oogoo00=oo`03001oogoo07=oo`003Goo0P002Goo00<007ooOol00goo00<007ooOol0Lgoo000=
Ool00`00Oomoo`08Ool00`00Oomoo`03Ool00`00Oomoo`1cOol000eoo`8000Uoo`03001oogoo00=o
o`03001oogoo07=oo`003Goo00<007ooOol027oo00<007ooOol00goo00<007ooOol0Lgoo000=Ool0
0`00Oomoo`09Ool00`00Oomoo`02Ool00`00Oomoo`1cOol000eoo`8000Yoo`05001oogooOol0001f
Ool00004Ool000000003Ool30003Ool00`00Oomoo`09Ool01@00Oomoogoo0000MWoo0002Ool00`00
Oomoo`05Ool01000Oomoo`002goo00D007ooOomoo`0007Ioo`000Woo00<007ooOol01Goo00<007oo
Ool00P002Woo00D007ooOomoo`0007Ioo`000Woo00<007ooOol00goo0P000goo00<007ooOol02Woo
00@007ooOol007Ioo`000Woo00<007ooOol01Goo00<007ooOol00P002goo00@007ooOol007Ioo`00
00=oo`00000017oo0`000goo00<007ooOol02Woo00@007ooOol007Ioo`003Goo00<007ooOol02Woo
00@007ooOol007Ioo`003Goo0P002goo00@007ooOol007Ioo`003Goo00<007ooOol02Woo00@007oo
Ool007Ioo`003Goo0P0037oo00<007oo0000MWoo000=Ool00`00Oomoo`0;Ool00`00Ool0001fOol0
00eoo`03001oogoo00]oo`03001oo`0007Ioo`003Goo0P0037oo00<007oo0000MWoo000017oo0000
000017oo0`000Woo00<007ooOol02goo00<007oo0000MWoo0002Ool00`00Oomoo`04Ool01@00Oomo
ogoo00003Woo0P00MWoo0002Ool00`00Oomoo`02Ool40002Ool2000=Ool2001fOol0009oo`03001o
ogoo009oo`03001oo`0000=oo`03001oogoo00aoo`8007Ioo`000Woo00<007ooOol00Woo00<007oo
00000goo0P003Goo0P00MWoo00000goo00000005Ool20003Ool00`00Oomoo`0<Ool2001fOol000eo
o`03001oogoo00aoo`8007Ioo`003Goo0P003Goo00<007ooOol0MGoo000=Ool00`00Oomoo`24Ool0
00eoo`03001oogoo08Aoo`003Goo00<007ooOol0Q7oo002DOol009Aoo`00\
\>"],
  ImageRangeCache->{{{0, 147.312}, {90.5625, 0}} -> {-1.57793, 7.94338, 
  0.115082, 0.0758622}}]
}, Open  ]],

Cell[TextData[{
  "Note: for times ",
  Cell[BoxData[
      \(TraditionalForm\`T\)]],
  " greater than 10 years, the forward rate is now a constant independent of \
",
  Cell[BoxData[
      \(TraditionalForm\`T\  > \ 10\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`d\ T\)]],
  ":"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[CellGroupData[{

Cell[BoxData[
    \({ACDFwd[12, \ 0.1], \ ACDFwd[14, \ 0.1], \ ACDFwd[14, \ 2]}\)], "Input",\

  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[BoxData[
    \({9.82070266916408307`, 9.82070266916383971`, 9.8207026691642163`}\)], 
  "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Remarks", "Subsection",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell["\<\
This is the simplest possible modelling process, but is complete \
within its limitations. More complex schemes can be built using many \
instruments and the fitting techniques we have described. Different types of \
interpolation can be used and the effects of this on the details of the yield \
curve and forward rates are well worth investigating. A good place to start, \
including more real-world effects, is the text by Miron and Swannell (1991). \
\
\>", "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Chapter 26 Bibliography", "Section",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}],

Cell[TextData[{
  "\nMiron, P. and Swannell, P., 1991, ",
  StyleBox["Pricing and Hedging Swaps",
    FontSlant->"Italic"],
  ", Euromoney.\n\nRebonato, R., 1996, ",
  StyleBox["Interest-Rate Option Models",
    FontSlant->"Italic"],
  ", Wiley.\n\nShaw, W.T. and Tigg, J., 1993, ",
  StyleBox["Applied Mathematica",
    FontSlant->"Italic"],
  ", Addison-Wesley.\n\n"
}], "Text",
  CellMargins->{{6, Inherited}, {Inherited, Inherited}}]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"Macintosh 3.0",
ScreenRectangle->{{0, 1152}, {0, 850}},
AutoGeneratedPackage->None,
WindowToolbars->{},
CellGrouping->Automatic,
WindowSize->{636, 701},
WindowMargins->{{20, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->482,
PrintingPageRange->{1, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"26. Yield Curves"}]]}, {Cell[ 
        TextData[ {"26. Yield Curves"}]], Inherited, Inherited}},
PageFooters->{{Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}, {Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page"]}]], Inherited}},
PrintingOptions->{"PrintingMargins"->{{0, 0}, {57, 57}},
"PaperSize"->{439, 716},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
PrivateNotebookOptions->{"ColorPalette"->{RGBColor, 128}},
ShowCellLabel->False,
ShowCellTags->False,
RenderingOptions->{"ObjectDithering"->True,
"RasterDithering"->False},
Magnification->1,
StyleDefinitions -> "Default.nb",
MacintoshSystemPageSetup->"\<\
00<0004/0B`000003;T8`?n3ohP==PTh0b85N`?P0080004/0B`000006G<AP001
0000I00000400@4100800BL?00400@S<]<0000000000064/01T1T00000000000
00000000007oo@S<]a3oo@001HL:kP00\>"
]


(***********************************************************************
Cached data follows.  If you edit this Notebook file directly, not using
Mathematica, you must remove the line containing CacheID at the top of 
the file.  The cache data will then be recreated when you save this file 
from within Mathematica.
***********************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1731, 51, 229, 9, 174, "Title"],
Cell[1963, 62, 136, 1, 47, "Subsubtitle"],

Cell[CellGroupData[{
Cell[2124, 67, 141, 3, 50, "Section"],
Cell[2268, 72, 1523, 25, 319, "Text"]
}, Open  ]],

Cell[CellGroupData[{
Cell[3828, 102, 110, 1, 50, "Section"],
Cell[3941, 105, 208, 6, 31, "Text"],

Cell[CellGroupData[{
Cell[4174, 115, 98, 2, 26, "Input"],
Cell[4275, 119, 543, 7, 120, "Print"]
}, Open  ]],
Cell[4833, 129, 404, 11, 29, "Text"],

Cell[CellGroupData[{
Cell[5262, 144, 231, 4, 37, "Input"],
Cell[5496, 150, 109, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[5642, 157, 216, 3, 37, "Input"],
Cell[5861, 162, 7332, 320, 98, 3635, 270, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[13208, 485, 122, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[13355, 490, 108, 2, 25, "Input"],
Cell[13466, 494, 316, 4, 69, "Print"]
}, Open  ]],
Cell[13797, 501, 85, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[13907, 506, 172, 4, 25, "Input",
  PageBreakWithin->True],
Cell[14082, 512, 174, 5, 36, "Output",
  PageBreakWithin->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[14293, 522, 218, 3, 37, "Input"],
Cell[14514, 527, 7906, 354, 98, 4148, 303, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[22435, 884, 196, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[22656, 894, 112, 2, 25, "Input"],
Cell[22771, 898, 64, 1, 36, "Output"]
}, Open  ]],
Cell[22850, 902, 155, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[23030, 910, 281, 4, 50, "Input"],
Cell[23314, 916, 7787, 349, 98, 4094, 299, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[31116, 1268, 106, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[31247, 1273, 311, 5, 50, "Input"],
Cell[31561, 1280, 8166, 367, 98, 4337, 314, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[39742, 1650, 260, 7, 29, "Text"],
Cell[40005, 1659, 228, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[40258, 1667, 317, 5, 50, "Input"],
Cell[40578, 1674, 8055, 365, 98, 4337, 314, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[48648, 2042, 127, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[48800, 2047, 281, 4, 50, "Input"],
Cell[49084, 2053, 7982, 361, 98, 4285, 311, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[57081, 2417, 365, 8, 40, "Text"],
Cell[57449, 2427, 127, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[57601, 2433, 107, 2, 25, "Input"],
Cell[57711, 2437, 999, 13, 206, "Print"]
}, Open  ]],
Cell[58725, 2453, 284, 6, 50, "Input"],

Cell[CellGroupData[{
Cell[59034, 2463, 111, 2, 25, "Input"],
Cell[59148, 2467, 325, 5, 72, "Output"]
}, Open  ]],
Cell[59488, 2475, 37, 0, 25, "Input"],

Cell[CellGroupData[{
Cell[59550, 2479, 292, 6, 37, "Input"],
Cell[59845, 2487, 340, 5, 48, "Output"]
}, Open  ]],
Cell[60200, 2495, 845, 17, 73, "Text"],
Cell[61048, 2514, 126, 2, 25, "Input"],
Cell[61177, 2518, 303, 8, 29, "Text",
  PageBreakAbove->False],

Cell[CellGroupData[{
Cell[61505, 2530, 144, 3, 25, "Input",
  PageBreakAbove->False],
Cell[61652, 2535, 468, 9, 73, "Print",
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[62169, 2550, 179, 3, 38, "Section"],
Cell[62351, 2555, 1124, 21, 161, "Text"],

Cell[CellGroupData[{
Cell[63500, 2580, 196, 5, 32, "Subsection",
  PageBreakAbove->False],
Cell[63699, 2587, 573, 13, 96, "Text",
  PageBreakWithin->True]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[64321, 2606, 128, 2, 38, "Section",
  PageBreakAbove->True],
Cell[64452, 2610, 330, 6, 40, "Text"],

Cell[CellGroupData[{
Cell[64807, 2620, 207, 3, 37, "Input"],
Cell[65017, 2625, 1171, 32, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[66225, 2662, 228, 4, 37, "Input"],
Cell[66456, 2668, 8444, 341, 98, 3601, 276, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[74915, 3012, 430, 11, 40, "Text"],

Cell[CellGroupData[{
Cell[75370, 3027, 169, 3, 25, "Input"],
Cell[75542, 3032, 120, 3, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[75699, 3040, 110, 1, 32, "Subsection"],
Cell[75812, 3043, 153, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[75990, 3051, 180, 3, 25, "Input"],
Cell[76173, 3056, 1725, 47, 48, "Output"]
}, Open  ]],
Cell[77913, 3106, 198, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[78136, 3114, 267, 5, 37, "Input"],
Cell[78406, 3121, 8531, 345, 98, 3658, 279, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[86974, 3471, 120, 2, 25, "Input"],
Cell[87097, 3475, 8669, 351, 98, 3778, 286, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[95815, 3832, 115, 1, 32, "Subsection"],
Cell[95933, 3835, 286, 5, 29, "Text"],
Cell[96222, 3842, 195, 4, 37, "Input"],

Cell[CellGroupData[{
Cell[96442, 3850, 284, 6, 37, "Input"],
Cell[96729, 3858, 8808, 354, 98, 3806, 287, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[105574, 4217, 122, 2, 25, "Input"],
Cell[105699, 4221, 8939, 361, 98, 3926, 294, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[114699, 4589, 114, 1, 38, "Section"],
Cell[114816, 4592, 869, 20, 172, "Text"],

Cell[CellGroupData[{
Cell[115710, 4616, 155, 3, 25, "Input"],
Cell[115868, 4621, 132, 2, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[116037, 4628, 168, 3, 25, "Input"],
Cell[116208, 4633, 568, 17, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[116813, 4655, 140, 2, 25, "Input"],
Cell[116956, 4659, 1008, 27, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[118001, 4691, 184, 3, 37, "Input"],
Cell[118188, 4696, 8286, 333, 98, 3775, 273, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[126511, 5034, 121, 1, 32, "Subsection"],
Cell[126635, 5037, 227, 6, 40, "Text"],
Cell[126865, 5045, 194, 4, 24, "NumberedEquation"],
Cell[127062, 5051, 115, 1, 20, "Text"],
Cell[127180, 5054, 163, 3, 36, "NumberedEquation"],
Cell[127346, 5059, 202, 6, 20, "Text"],
Cell[127551, 5067, 512, 10, 100, "Input"],

Cell[CellGroupData[{
Cell[128088, 5081, 137, 2, 25, "Input"],
Cell[128228, 5085, 311, 5, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[128576, 5095, 167, 3, 25, "Input"],
Cell[128746, 5100, 341, 6, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[129124, 5111, 157, 3, 25, "Input"],
Cell[129284, 5116, 6989, 246, 98, 2730, 189, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[136288, 5365, 155, 4, 20, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[136492, 5375, 145, 4, 53, "Section"],

Cell[CellGroupData[{
Cell[136662, 5383, 98, 1, 32, "Subsection"],
Cell[136763, 5386, 370, 8, 40, "Text"],

Cell[CellGroupData[{
Cell[137158, 5398, 129, 2, 25, "Input"],
Cell[137290, 5402, 55, 1, 36, "Output"]
}, Open  ]],
Cell[137360, 5406, 146, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[137531, 5414, 119, 2, 25, "Input"],
Cell[137653, 5418, 53, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[137755, 5425, 103, 1, 32, "Subsection"],
Cell[137861, 5428, 392, 7, 40, "Text"],

Cell[CellGroupData[{
Cell[138278, 5439, 126, 2, 25, "Input"],
Cell[138407, 5443, 63, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[138507, 5449, 118, 2, 25, "Input"],
Cell[138628, 5453, 55, 1, 36, "Output"]
}, Open  ]],
Cell[138698, 5457, 158, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[138881, 5465, 138, 2, 25, "Input"],
Cell[139022, 5469, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[139113, 5475, 123, 2, 25, "Input"],
Cell[139239, 5479, 62, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[139338, 5485, 118, 2, 25, "Input"],
Cell[139459, 5489, 55, 1, 36, "Output"]
}, Open  ]],
Cell[139529, 5493, 157, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[139711, 5501, 151, 3, 25, "Input"],
Cell[139865, 5506, 54, 1, 36, "Output"]
}, Open  ]],
Cell[139934, 5510, 261, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[140220, 5519, 178, 4, 25, "Input"],
Cell[140401, 5525, 54, 1, 36, "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[140504, 5532, 108, 1, 32, "Subsection"],
Cell[140615, 5535, 183, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[140823, 5543, 110, 2, 25, "Input"],
Cell[140936, 5547, 1725, 47, 48, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[142698, 5599, 114, 2, 25, "Input"],
Cell[142815, 5603, 3230, 87, 72, "Output"]
}, Open  ]],
Cell[146060, 5693, 164, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[146249, 5701, 144, 2, 25, "Input"],
Cell[146396, 5705, 1911, 52, 48, "Output"]
}, Open  ]],
Cell[148322, 5760, 102, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[148449, 5765, 217, 4, 37, "Input"],
Cell[148669, 5771, 3793, 102, 85, "Output"]
}, Open  ]],
Cell[152477, 5876, 107, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[152609, 5881, 235, 4, 37, "Input"],
Cell[152847, 5887, 7455, 310, 98, 3388, 254, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[160363, 6204, 170, 3, 38, "Section"],
Cell[160536, 6209, 269, 7, 29, "Text"],
Cell[160808, 6218, 266, 7, 57, "Input"],
Cell[161077, 6227, 227, 6, 46, "Input"],
Cell[161307, 6235, 204, 6, 20, "Text"],

Cell[CellGroupData[{
Cell[161536, 6245, 155, 3, 32, "Subsection"],
Cell[161694, 6250, 259, 7, 29, "Text"],
Cell[161956, 6259, 459, 14, 134, "Input"],

Cell[CellGroupData[{
Cell[162440, 6277, 109, 1, 28, "Subsubsection"],
Cell[162552, 6280, 125, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[162702, 6285, 143, 2, 25, "Input"],
Cell[162848, 6289, 1524, 37, 48, "Output"]
}, Open  ]],
Cell[164387, 6329, 149, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[164561, 6337, 130, 2, 25, "Input"],
Cell[164694, 6341, 283, 4, 48, "Output"]
}, Open  ]],
Cell[164992, 6348, 99, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[165116, 6353, 166, 3, 25, "Input"],
Cell[165285, 6358, 661, 16, 60, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[165983, 6379, 151, 3, 25, "Input"],
Cell[166137, 6384, 6753, 267, 98, 3056, 217, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[172939, 6657, 114, 1, 28, "Subsubsection"],
Cell[173056, 6660, 127, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[173208, 6665, 175, 3, 25, "Input"],
Cell[173386, 6670, 2986, 72, 85, "Output"]
}, Open  ]],
Cell[176387, 6745, 164, 4, 20, "Text"],

Cell[CellGroupData[{
Cell[176576, 6753, 135, 2, 25, "Input"],
Cell[176714, 6757, 533, 7, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[177284, 6769, 196, 4, 25, "Input"],
Cell[177483, 6775, 1464, 36, 85, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[178984, 6816, 151, 3, 25, "Input"],
Cell[179138, 6821, 7310, 297, 98, 3482, 245, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[186521, 7126, 154, 4, 53, "Section"],
Cell[186678, 7132, 231, 5, 29, "Text"],

Cell[CellGroupData[{
Cell[186934, 7141, 114, 1, 28, "Subsubsection"],
Cell[187051, 7144, 113, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[187189, 7149, 97, 2, 25, "Input"],
Cell[187289, 7153, 661, 16, 60, "Output"]
}, Open  ]],
Cell[187965, 7172, 102, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[188092, 7177, 102, 2, 25, "Input"],
Cell[188197, 7181, 311, 5, 48, "Output"]
}, Open  ]],
Cell[188523, 7189, 199, 4, 29, "Text"],

Cell[CellGroupData[{
Cell[188747, 7197, 139, 2, 25, "Input"],
Cell[188889, 7201, 607, 10, 72, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[189533, 7216, 151, 3, 25, "Input"],
Cell[189687, 7221, 7529, 300, 98, 3515, 246, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[197265, 7527, 119, 1, 28, "Subsubsection"],
Cell[197387, 7530, 113, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[197525, 7535, 97, 2, 25, "Input"],
Cell[197625, 7539, 1464, 36, 85, "Output"]
}, Open  ]],
Cell[199104, 7578, 118, 1, 20, "Text"],

Cell[CellGroupData[{
Cell[199247, 7583, 102, 2, 25, "Input"],
Cell[199352, 7587, 311, 5, 48, "Output"]
}, Open  ]],
Cell[199678, 7595, 213, 4, 29, "Text"],
Cell[199894, 7601, 183, 4, 25, "Input"],

Cell[CellGroupData[{
Cell[200102, 7609, 170, 3, 25, "Input"],
Cell[200275, 7614, 7539, 310, 98, 3703, 258, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[207829, 7927, 559, 11, 52, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[208437, 7944, 169, 3, 38, "Section"],
Cell[208609, 7949, 293, 9, 29, "Text"],
Cell[208905, 7960, 256, 8, 20, "Text"],
Cell[209164, 7970, 185, 5, 32, "Input"],
Cell[209352, 7977, 194, 5, 36, "Input"],

Cell[CellGroupData[{
Cell[209571, 7986, 122, 2, 25, "Input"],
Cell[209696, 7990, 80, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[209813, 7996, 132, 2, 20, "Input"],
Cell[209948, 8000, 1663, 37, 90, "Output"]
}, Open  ]],
Cell[211626, 8040, 426, 10, 50, "Text"],

Cell[CellGroupData[{
Cell[212077, 8054, 141, 2, 20, "Input"],
Cell[212221, 8058, 59, 1, 20, "Output"]
}, Open  ]],
Cell[212295, 8062, 199, 4, 21, "Text"],

Cell[CellGroupData[{
Cell[212519, 8070, 141, 2, 20, "Input"],
Cell[212663, 8074, 59, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[212759, 8080, 158, 3, 20, "Input"],
Cell[212920, 8085, 1753, 39, 90, "Output"]
}, Open  ]],
Cell[214688, 8127, 181, 4, 21, "Text"],

Cell[CellGroupData[{
Cell[214894, 8135, 185, 5, 32, "Input"],
Cell[215082, 8142, 8862, 364, 100, 3572, 293, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[223981, 8511, 107, 1, 30, "Subsubsection"],
Cell[224091, 8514, 166, 4, 21, "Text"],

Cell[CellGroupData[{
Cell[224282, 8522, 165, 3, 20, "Input"],
Cell[224450, 8527, 118, 3, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[224605, 8535, 104, 2, 20, "Input"],
Cell[224712, 8539, 54, 1, 20, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[224803, 8545, 132, 2, 20, "Input"],
Cell[224938, 8549, 9929, 399, 98, 4731, 330, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[234916, 8954, 130, 1, 32, "Subsection"],
Cell[235049, 8957, 356, 8, 29, "Text"],
Cell[235408, 8967, 144, 2, 25, "Input"],

Cell[CellGroupData[{
Cell[235577, 8973, 131, 2, 25, "Input"],
Cell[235711, 8977, 8220, 319, 98, 3616, 257, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[243946, 9299, 435, 7, 52, "Text"],

Cell[CellGroupData[{
Cell[244406, 9310, 114, 2, 25, "Input"],
Cell[244523, 9314, 8418, 340, 98, 4074, 282, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],

Cell[CellGroupData[{
Cell[252978, 9659, 107, 1, 28, "Subsubsection"],
Cell[253088, 9662, 269, 8, 29, "Text"],
Cell[253360, 9672, 205, 4, 37, "Input"],
Cell[253568, 9678, 154, 4, 20, "Text"],
Cell[253725, 9684, 178, 4, 44, "Input"],

Cell[CellGroupData[{
Cell[253928, 9692, 107, 2, 25, "Input"],
Cell[254038, 9696, 54, 1, 36, "Output"]
}, Open  ]],

Cell[CellGroupData[{
Cell[254129, 9702, 168, 3, 25, "Input"],
Cell[254300, 9707, 13880, 592, 98, 7439, 508, "GraphicsData", 
"PostScript", "Graphics"]
}, Open  ]],
Cell[268195, 10302, 361, 13, 20, "Text"],

Cell[CellGroupData[{
Cell[268581, 10319, 151, 3, 25, "Input"],
Cell[268735, 10324, 102, 2, 36, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[268898, 10333, 86, 1, 32, "Subsection"],
Cell[268987, 10336, 536, 9, 62, "Text"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[269572, 10351, 99, 1, 38, "Section"],
Cell[269674, 10354, 437, 12, 96, "Text"]
}, Open  ]]
}, Open  ]]
}
]
*)




(***********************************************************************
End of Mathematica Notebook file.
***********************************************************************)

